#!/bin/sh
# ********************************************************************
# (C) Copyright IBM Corp.  2005-2006
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
# ********************************************************************


err_schema() {
    echo -n "Schema name $TBSCHEM is undefined Provide the right schema name"
    exit 1
}

error_connect() {
		echo -n "One of the parameters specified to connect to the database is incorrect"
		exit 1
}



callmigration6127() {
	echo -n " Starting migration from RPM v6.1.2.6 to RPM v6.1.2.7....."
	echo -n " -----------------------------------------------------------------------------------------------------------------"
	db2 connect to $DB user $USR using $PWRD
	RETVAL=$?
	if [ $RETVAL != 0 ]; then
		 error_connect
	fi
		db2 set CURRENT SCHEMA $TBSCHEM
		RETVAL=$?
		if [ $RETVAL != 0 ]; then
		    err_schema
		fi
	db2 -tvf migration6127.sql > migration6127.out
	db2 connect reset
	echo -n " -----------------------------------------------------------------------------------------------------------------"
	
}	


checkdb() {
		echo -n " Running Statistics......"
		echo -n " -----------------------------------------------------------------------------------------------------------------"
		$CURDIR/ReorgStats61.sh $DB $USR $PWRD $TBSCHEM > Reorgstats61.out
		echo -n " -----------------------------------------------------------------------------------------------------------------"
		
		
		if [ $ALIAS == "Y" ]; then
			db2 connect to $DB user $DBUSER using $DBPASS 
		else
			db2 connect to $DB user $USR using $PWRD
		fi
		RETVAL=$?
		if [ $RETVAL != 0 ]; then
		 	error_connect
		fi
		db2 SET CURRENT SCHEMA $TBSCHEM
		RETVAL=$?
		if [ $RETVAL != 0 ]; then
		    err_schema
		fi
		
		db2 "call sp_get_version"
		
		db2 connect reset
		echo -n " -----------------------------------------------------------------------------------------------------------------"
		exit 0
}		




echo -n " *************************************************************************************************************"
echo -n " 				WELCOME TO RATIONAL PORTFOLIO MANAGER V6.1.2.7 MIGRATION"
echo -n " *************************************************************************************************************"

if [ "$CURDIR" == "" ]; then
		CURDIR=$PWD
fi

echo -n "DID YOU COPY pmoffice.a library to INSTHOME/sqllib/function/ directory as stated in migration guide? (type y for yes and n for no)"
read response
if [ $response == "n" ]; then
	echo -n "You can't continue the migration until you copy the library file. Please refer to the migration guide."
	exit 1
fi

echo -n Please enter the database name to be migrated...
read DB
if [ "$DB" == "" ]; then
	echo The database name cannot be blank, provide valid database name....
	read DB
fi


echo -n Please enter the username of the instance owner...
read USR
if [ "$USR" == "" ]; then
 	echo -n The username of the instance owner cannot be blank, provide valid username....
	read USR
fi

echo -n  Please enter the password of the instance owner...
read PWRD
if [ "$PWRD" == "" ]; then
	echo -n  The password of the instance owner cannot be blank, provide a valid password...
	read PWRD
fi	

echo -n  Please enter the username of the connected user if any or leave it blank...
read DBUSER
if [ "$DBUSER" == "" ]; then
	 	ALIAS="N"
else 
		ALIAS="Y" 
fi

if [ $ALIAS == "Y" ]; then
	echo -n  Please enter the password for the connected user...
	read DBPASS
	if [ "$DBPASS" == "" ]; then
		echo -n  The password for the connected user cannot be blank, provide valid password....
		read DBPASS
	fi
fi

TBSCHEM=$USR



echo -n " Restarting DB2 instance...."
echo -n " -----------------------------------------------------------------------------------------------------------------"
db2 force applications all
db2stop force
db2start
echo -n " -----------------------------------------------------------------------------------------------------------------"

db2 connect to $DB user $USR using $PWRD
RETVAL=$?
if [ $RETVAL != 0 ]; then
	error_connect
fi

db2 SET CURRENT SCHEMA $TBSCHEM
RETVAL=$?
if [ $RETVAL != 0 ]; then
  err_schema
fi


export VERSION=`db2 -x "SELECT CAST(ELEMENT_NAME AS CHAR(7)) FROM TMT_SETTINGS WHERE ELEMENT_ID='CLIENT_VERSION_NUMBER'"` 


if [ $VERSION != "6.1.2.6" ]; then
	 echo -n  Your database is not at the right level 6.1.2.6. You need to upgrade your database to RPM v6.1.2.6 level first, then upgrade to RPM v6.1.2.7 level.
	 exit 1
fi

echo -n " Dropping triggers...."
echo -n " -----------------------------------------------------------------------------------------------------------------"

 		$CURDIR/drop_triggers61.sh $DB $USR $PWRD $TBSCHEM> drop_triggers61.out
echo -n " -----------------------------------------------------------------------------------------------------------------"

callmigration6127

echo -n " Creating triggers......"
echo -n " -----------------------------------------------------------------------------------------------------------------"
db2 connect to $DB user $USR using $PWRD
RETVAL=$?
if [ $RETVAL != 0 ]; then
	error_connect
fi
db2 set CURRENT SCHEMA $TBSCHEM
RETVAL=$?
if [ $RETVAL != 0 ]; then
		 err_schema
fi
db2 -tvf triggers61.sql > triggers61.out
db2 connect reset 
echo -n " -----------------------------------------------------------------------------------------------------------------"



cd ..
cd csp

echo -n " Creating Store Procedures....."
echo -n " ----------------------------------------------------------------------------------------------------------------"
if [ $ALIAS == "Y" ]; then
	db2 connect to $DB user $DBUSER using $DBPASS 
else
	db2 connect to $DB user $USR using $PWRD
fi
if [ $RETVAL != 0 ]; then
	error_connect
fi	
db2 -tvf createsp.sql > createsp.out

db2 connect reset
echo -n " -----------------------------------------------------------------------------------------------------------------"


echo -n " Binding the code....."
echo -n " -----------------------------------------------------------------------------------------------------------------"
db2 force applications all
db2stop force
db2start

if [ $ALIAS == "Y" ]; then
	./bindall.sh $DB $DBUSER $DBPASS $TBSCHEM > bindall.out 
else
	./bindall.sh $DB $USR $PWRD $TBSCHEM > bindall.out
fi


echo -n " -----------------------------------------------------------------------------------------------------------------"


cd ..
cd migration

checkdb 





