
CREATE TRIGGER depend_d AFTER DELETE ON dependencies REFERENCING OLD AS o FOR EACH ROW MODE DB2SQL 
	WHEN ((select published from tmt_wbs where element_id = o.succ_project_id) = '1') 
BEGIN ATOMIC 
	insert into alerts (element_id, project_id, ALERT_ID, parent_table, rec_user, value_list, flag) values
	(o.succ_id, o.succ_project_id, 'SCDEPENDENCYDETAILSCHANGED', 'TMT_WBS', o.rec_user,
	(select element_name from tmt_wbs where element_id = o.pred_id) ||'#!'||
	(select element_name from tmt_wbs where element_id = o.pred_project_id)|| '#!' ||
	(select element_name from tmt_wbs where element_id = o.succ_id) ||'#!'||
	(select element_name from tmt_wbs where element_id = o.succ_project_id)||'#!deleted', 'I'); END;

CREATE TRIGGER depend_i AFTER INSERT ON dependencies REFERENCING NEW AS n FOR EACH ROW MODE DB2SQL 
	WHEN ((select published from tmt_wbs where element_id = n.succ_project_id) = '1') 
BEGIN ATOMIC 
	insert into alerts
	(element_id, project_id, ALERT_ID, parent_table, rec_user, value_list, flag) values
	(n.succ_id, n.succ_project_id, case when n.pred_project_id = n.succ_project_id then 'SCNEWDEPENDENCY' else 
	'SCNEWCROSSPROJECTDEPENDENCY' end, 'TMT_WBS', n.rec_user, case when n.pred_project_id = n.succ_project_id then 
	(select element_name from tmt_wbs where element_id = n.pred_id) ||'#!' ||
	(select element_name from tmt_wbs where element_id = n.succ_id) ||'#!' else
	(select element_name from tmt_wbs where element_id = n.succ_project_id) ||'#!' end ||
	(select element_name from tmt_wbs where element_id = n.pred_project_id), 'I'); END;

CREATE TRIGGER depend_u AFTER UPDATE OF DEP_TYPE, LEAD_OR_LAG_TYPE, AMOUNT, LAG_DISPLAY_UNIT
ON dependencies REFERENCING OLD AS o NEW AS n FOR EACH ROW MODE DB2SQL 
	WHEN ((select published from tmt_wbs where element_id = n.succ_project_id) = '1') 
BEGIN ATOMIC 
	insert into alerts (element_id, project_id, ALERT_ID, parent_table, rec_user, value_list, flag) values
	(n.succ_id, n.succ_project_id, 'SCDEPENDENCYDETAILSCHANGED', 'TMT_WBS', n.rec_user,
	(select element_name from tmt_wbs where element_id = n.pred_id) ||'#!'||
	(select element_name from tmt_wbs where element_id = n.pred_project_id)|| '#!' ||
	(select element_name from tmt_wbs where element_id = n.succ_id)||'#!'||
	(select element_name from tmt_wbs where element_id = n.succ_project_id) || '#!changed', 'I'); END;

CREATE TRIGGER mailgroup_d AFTER DELETE ON tmt_contacts REFERENCING OLD AS o FOR EACH ROW MODE DB2SQL 
	WHEN (o.parent_id is not null and o.type_id = 13) 
BEGIN ATOMIC
	insert into alerts (element_id, RESOURCE_ID, ALERT_ID, parent_table, rec_user, value_list, flag)
	values (o.element_id, o.element_id, 'SCMAILGROUPMEMBERREMOVE', 'RESOURCES', o.rec_user,
	(select element_name from tmt_contacts where element_id = o.parent_id), 'I'); END;

CREATE TRIGGER mailgroup_i AFTER INSERT ON tmt_contacts REFERENCING NEW AS n FOR EACH ROW MODE DB2SQL 
	WHEN (n.parent_id is not null and n.element_id <> n.rec_user and n.type_id = 13) 
BEGIN ATOMIC
	insert into alerts (element_id, RESOURCE_ID, ALERT_ID, parent_table, rec_user, value_list, flag)
	values (n.element_id, n.element_id, 'SCMAILGROUPMEMBERADD', 'RESOURCES', n.rec_user,
	(select element_name from tmt_contacts where element_id = n.parent_id), 'I'); END;

CREATE TRIGGER resource_i AFTER INSERT ON history_resources REFERENCING
	  NEW AS n FOR EACH ROW MODE DB2SQL BEGIN ATOMIC insert into alerts (element_id, RESOURCE_ID,
	  ALERT_ID, parent_table, rec_user, value_list, flag) (select p.manager_id, n.resource_id,
	  'SCRESOURCERECORDREAPPROVAL', 'RESOURCES', n.rec_user, n.full_name, 'I'
	  from tmt_pools c, tmt_pools p where c.element_id = n.resource_id
	  and p.element_id = c.parent_id and p.manager_id <> n.rec_user); END;

CREATE TRIGGER res_assign_d
AFTER DELETE ON resource_assignments
REFERENCING OLD AS o FOR EACH ROW MODE DB2SQL
WHEN (EXISTS(select element_id from tmt_wbs where element_id =o.element_id) and 
	(select published from tmt_wbs where element_id =o.project_id) = '1')
BEGIN ATOMIC
	insert into alerts (element_id, project_id, ALERT_ID, parent_table, rec_user, value_list, flag, resource_id) values
	(o.element_id, o.project_id, 'SCRESOURCEASSIGNMENTCHANGEREMOVE', 'TMT_WBS', o.rec_user,
	(select w.element_name from tmt_wbs w where w.element_id = o.element_id )
	||'#!'|| (select element_name from tmt_wbs where element_id =o.project_id), 'I', o.resource_id); END;

CREATE TRIGGER res_assign_i
AFTER INSERT ON resource_assignments
REFERENCING NEW AS n FOR EACH ROW MODE DB2SQL
WHEN (EXISTS(select element_id from tmt_wbs where element_id = n.element_id) and 
	(select published from tmt_wbs where element_id =n.project_id) = '1')
BEGIN ATOMIC
	insert into alerts (element_id, project_id, ALERT_ID, parent_table, rec_user, value_list, flag, resource_id) values
	(n.element_id, n.project_id, 'SCRESOURCEASSIGNMENTCHANGEADD', 'TMT_WBS', n.rec_user,
	(select w.element_name from tmt_wbs w where w.element_id = n.element_id )
	||'#!'|| (select element_name from tmt_wbs where element_id =n.project_id), 'I', n.resource_id); END;

CREATE TRIGGER tskassign_u_sched
AFTER UPDATE OF sched_start ON task_assignments
REFERENCING OLD AS o NEW AS n FOR EACH ROW MODE DB2SQL
WHEN (o.sched_start is null and n.sched_start is not null and 
		(select published from tmt_wbs where element_id =n.project_id) = '1')
BEGIN ATOMIC
	insert into alerts (element_id, project_id, ALERT_ID, parent_table, rec_user, value_list, flag, resource_id) values
	(n.task_id, n.project_id, 'SCSCHEDULEASSIGNMENTADD', 'TMT_WBS', n.rec_user,
	(select full_name from resources where resource_id = n.resource_id) ||'#!'||
	(select element_name from tmt_wbs where element_id = n.task_id) ||'#!'||
	(select element_name from tmt_wbs where element_id = n.project_id) , 'I', n.resource_id); END;

CREATE TRIGGER tskassign_d_sched
AFTER DELETE ON task_assignments 
REFERENCING OLD AS o FOR EACH ROW MODE DB2SQL
WHEN (o.sched_start is not null and 
		EXISTS(select element_id from tmt_wbs where element_id =o.task_id) and 
		(select published from tmt_wbs where element_id =o.project_id) = '1') 
BEGIN ATOMIC
	insert into alerts (element_id, project_id, ALERT_ID, parent_table, rec_user, value_list, flag, resource_id) values
	(o.task_id, o.project_id, 'SCSCHEDULEASSIGNMENTREMOVE', 'TMT_WBS', o.rec_user,
	(select full_name from resources where resource_id = o.resource_id) ||'#!'||
	(select element_name from tmt_wbs where element_id = o.task_id) ||'#!'||
	(select element_name from tmt_wbs where element_id = o.project_id), 'I', o.resource_id); END;

--CREATE TRIGGER wbs_u_complete 
--AFTER UPDATE OF actual_finish ON tmt_wbs 
--REFERENCING OLD AS o NEW AS n FOR EACH ROW MODE DB2SQL
--WHEN (o.actual_finish is null and n.actual_finish is not null
--		and (select published from tmt_wbs where element_id = n.project_id) = '1') 
--BEGIN ATOMIC 
--	insert into alerts (element_id, ALERT_ID, parent_table, rec_user, value_list, flag) values
--	(o.element_id, 'SCTASKCOMPLETION', 'TMT_WBS', n.rec_user, n.element_name||'#!'||
--	(select element_name from tmt_wbs where element_id = n.project_id), 'I'); END;

CREATE TRIGGER wbs_u_cost 
AFTER UPDATE OF CURRENT_BUDGET ON tmt_wbs 
REFERENCING OLD AS o NEW AS n FOR EACH ROW MODE DB2SQL 
WHEN (n.BASELINE1_BUDGET <> 0
	  AND (n.CURRENT_BUDGET-n.BASELINE1_BUDGET)<> (o.CURRENT_BUDGET-o.BASELINE1_BUDGET)
	  AND (select published from tmt_wbs where element_id = n.project_id) = '1') 
BEGIN ATOMIC insert
	into alerts (element_id, project_id, ALERT_ID, parent_table, rec_user, value_list,
	flag, VARIANCE) values (o.element_id, n.project_id, 'SCCOSTVARIANCE', 'TMT_WBS', n.rec_user,
	n.element_name || '#!'||(select w.element_name from tmt_wbs w where w.element_id =n.project_id)||'#!'
	|| RTRIM(CHAR(INT(o.CURRENT_BUDGET))) || LTRIM(SUBSTR(CHAR(o.CURRENT_BUDGET), LENGTH(CHAR(o.CURRENT_BUDGET))-3, 3))|| '#!'
	|| RTRIM(CHAR(INT(o.BASELINE1_BUDGET))) || LTRIM(SUBSTR(CHAR(o.BASELINE1_BUDGET), LENGTH(CHAR(o.BASELINE1_BUDGET))-3, 3)) || '#!'
	|| RTRIM(CHAR(INT(n.CURRENT_BUDGET))) || LTRIM(SUBSTR(CHAR(n.CURRENT_BUDGET), LENGTH(CHAR(n.CURRENT_BUDGET))-3, 3))|| '#!'
	|| RTRIM(CHAR(INT(n.BASELINE1_BUDGET))) || LTRIM(SUBSTR(CHAR(n.BASELINE1_BUDGET), LENGTH(CHAR(n.BASELINE1_BUDGET))-3, 3)),
	'I', INT((n.CURRENT_BUDGET-n.BASELINE1_BUDGET)/n.BASELINE1_BUDGET*100)); END;

CREATE TRIGGER wbs_u_published AFTER UPDATE OF published ON tmt_wbs REFERENCING
	   OLD AS o  NEW AS n FOR EACH ROW MODE DB2SQL WHEN (o.published <> n.published
	  and n.published = '1') BEGIN ATOMIC insert into alerts (element_id, project_id, ALERT_id,
	  parent_table, rec_user, value_list, flag) values (o.element_id, o.project_id, 'SCNEWPROJECTPUBLISHED',
	  'TMT_WBS', n.rec_user, n.element_name, 'I'); END;

CREATE TRIGGER wbs_u_schedule 
AFTER UPDATE OF BASELINE1_VARIANCE ON tmt_wbs 
REFERENCING OLD AS o NEW AS n FOR EACH ROW MODE DB2SQL 
WHEN (n.BASELINE1_VARIANCE <> o.BASELINE1_VARIANCE
	AND n.BASELINE1_DURATION > 0
	AND (select published from tmt_wbs where element_id = n.project_id) = '1') 
BEGIN ATOMIC 
	insert into alerts (element_id, project_id, ALERT_ID, parent_table, rec_user, value_list, flag, VARIANCE) values 
	(o.element_id, o.project_id, 'SCSCHEDULEVARIANCE', 'TMT_WBS', n.rec_user, n.element_name || '#!'||
	(select w.element_name from tmt_wbs w where w.element_id = n.project_id )||'#!'
	|| CASE WHEN N.SCHED_FINISH IS NOT NULL THEN CHAR(DATE(n.SCHED_FINISH)) ELSE CHAR(DATE(n.PLANNED_FINISH)) END || '#!'
	|| CHAR(DATE(n.BASELINE1_FINISH)) || '#!'
	|| RTRIM(CHAR(n.BASELINE1_VARIANCE/n.MINUTE_PER_DAY)) || ' Day(s), '
	|| RTRIM(CHAR((n.BASELINE1_VARIANCE - n.MINUTE_PER_DAY * (n.BASELINE1_VARIANCE/n.MINUTE_PER_DAY))/60)) || ' Hour(s), '
	|| RTRIM(CHAR((n.BASELINE1_VARIANCE - n.MINUTE_PER_DAY * (n.BASELINE1_VARIANCE/n.MINUTE_PER_DAY))
	- 60 * ((n.BASELINE1_VARIANCE - n.MINUTE_PER_DAY * (n.BASELINE1_VARIANCE/n.MINUTE_PER_DAY))/60))) || ' Minute(s)' || '#!'
	|| CHAR(INT(n.BASELINE1_VARIANCE/DEC(n.BASELINE1_DURATION)*100)),
	'I', INT(n.BASELINE1_VARIANCE/DEC(n.BASELINE1_DURATION)*100)); END;

CREATE TRIGGER wbs_u_template AFTER UPDATE OF published ON tmt_wbs REFERENCING
	   OLD AS o NEW AS n FOR EACH ROW MODE DB2SQL WHEN ((o.t_rank is null and
	  n.t_rank is not null) or o.t_rank <> n.t_rank) BEGIN ATOMIC insert into
	  alerts (element_id, project_id, ALERT_ID, parent_table, rec_user, value_list, flag)
	  values (n.element_id, n.project_id, 'SCNEWTEMPLATE', 'TMT_WBS', n.rec_user, n.element_name||'#!'||
	  (select element_name from tmt_wbs w where w.element_id = n.project_id),
	  'I'); END;

------------------------------------------------------------------------------------------
-- Trigger wbs_i_scope_qual
------------------------------------------------------------------------------------------
-- CREATE TRIGGER wbs_i_scope_qual
--	AFTER INSERT ON wbs_rtf
--	REFERENCING NEW AS n
--	FOR EACH ROW
--	MODE DB2SQL
--	WHEN (n.bit_flag in (1, 2) and ((select case when c.type_id = 1 then c.published else p.published end
--		from tmt_wbs c, tmt_wbs p where c.element_id = n.element_id
--		and c.project_id = p.element_id) = '1'))
--	BEGIN ATOMIC
--   		insert into alerts (element_id, project_id, ALERT_ID, parent_table, rec_user, value_list, flag)
--     		select n.element_id, w.project_id, case when n.bit_flag = 1 then 'SCSCOPEVARIANCE' else 'SCQUALITYVARIANCE' end,
--     		'TMT_WBS', w.rec_user, w.element_name||'#!'
--     		||(select t1.element_name from tmt_wbs t1 where t1.element_id = w.project_id), 'I'
--     		from tmt_wbs w where element_id = n.element_id; end;
------------------------------------------------------------------------------------------
-- Trigger wbs_u_scope_qual
------------------------------------------------------------------------------------------
-- CREATE TRIGGER wbs_u_scope_qual
--	AFTER UPDATE OF clob_data ON wbs_rtf
--	REFERENCING  OLD AS o  NEW AS n
--	FOR EACH ROW
--	MODE DB2SQL
--	WHEN (n.bit_flag in (1, 2) and ((o.clob_data is null and n.clob_data is not null)
--		or varchar(o.clob_data) <> varchar(n.clob_data))
--		and ((select case when c.type_id = 1 then c.published else p.published end as published
--		from tmt_wbs c, tmt_wbs p where c.element_id = n.element_id
--		and c.project_id = p.element_id) = '1'))
--	BEGIN ATOMIC
--     		insert into alerts (element_id, project_id, ALERT_ID, parent_table, rec_user, value_list, flag)
--     		select o.element_id, w.project_id, case when n.bit_flag = 1 then 'SCSCOPEVARIANCE' else 'SCQUALITYVARIANCE' end,
--     		'TMT_WBS', w.rec_user, w.element_name||'#!'||
--     		(select t1.element_name from tmt_wbs t1 where t1.element_id = w.project_id), 'I'
--     		from tmt_wbs w where element_id = n.element_id; end;
------------------------------------------------------------------------------------------
-- FINANCIAL SYSTEM TRIGGERS
------------------------------------------------------------------------------------------

------------------------------------------------------------------------------------------
-- Trigger FS_TA_SCHED_U
------------------------------------------------------------------------------------------
CREATE TRIGGER FS_TA_SCHED_U AFTER  UPDATE OF SCHED_START, SCHED_FINISH,
SCHED_WORK ON TASK_ASSIGNMENTS  REFERENCING  OLD AS o  NEW AS n  FOR EACH
ROW  MODE DB2SQL

WHEN ( (n.SCHED_WORK>0) AND ((o.SCHED_START<>n.SCHED_START) OR (o.SCHED_FINISH<>n.SCHED_FINISH) OR (o.SCHED_WORK<>n.SCHED_WORK )))

BEGIN ATOMIC

     UPDATE TMT_WBS SET FS_STATE = CASE WHEN FS_STATE IN (0,4,5) THEN 1 WHEN  FS_STATE=6 THEN 7 WHEN FS_STATE=2 THEN 3 ELSE FS_STATE END

	WHERE TYPE_ID IN (3,59, 236, 237, 238, 239, 240, 241, 260) AND ELEMENT_ID=n.TASK_ID AND FS_LABOR_CODE IS NOT NULL AND FS_LABOR_CODE <> 'PMOFFICE' AND FS_STATE NOT IN (3, 8);--

END;


------------------------------------------------------------------------------------------
-- Trigger FS_TC_ASSIGN_I
------------------------------------------------------------------------------------------
CREATE TRIGGER FS_TC_ASSIGN_I AFTER  INSERT  ON TMT_TIMECODES  REFERENCING
 NEW AS n  FOR EACH ROW  MODE DB2SQL

WHEN ( n.TYPE_ID IN (3,59, 236, 237, 238, 239, 240, 241, 260) )

BEGIN ATOMIC

    UPDATE TMT_WBS SET FS_STATE = CASE WHEN FS_STATE IN (0,4,5) THEN 1 WHEN  FS_STATE=6 THEN 7 WHEN FS_STATE=2 THEN 3 ELSE FS_STATE END

    WHERE ELEMENT_ID=n.ELEMENT_ID AND FS_LABOR_CODE IS NOT NULL AND FS_LABOR_CODE <> 'PMOFFICE' AND FS_STATE NOT IN (3, 8);--

END;

------------------------------------------------------------------------------------------
-- Trigger FS_TC_UNASSIGN_D
------------------------------------------------------------------------------------------
CREATE TRIGGER FS_TC_UNASSIGN_D AFTER  DELETE  ON TMT_TIMECODES  REFERENCING
 OLD AS o  FOR EACH ROW  MODE DB2SQL

WHEN ( o.TYPE_ID IN (3,59, 236, 237, 238, 239, 240, 241, 260) )

BEGIN ATOMIC

    UPDATE TMT_WBS SET FS_STATE = CASE WHEN FS_STATE IN (0,4,5) THEN 1 WHEN  FS_STATE=6 THEN 7 WHEN FS_STATE=2 THEN 3 ELSE FS_STATE END

    WHERE ELEMENT_ID=o.ELEMENT_ID AND FS_LABOR_CODE IS NOT NULL AND FS_LABOR_CODE <> 'PMOFFICE' AND FS_STATE NOT IN (3, 8);--

END;


------------------------------------------------------------------------------------------
-- Trigger FS_WBS_DIRTY_U
------------------------------------------------------------------------------------------
CREATE TRIGGER FS_WBS_DIRTY_U NO CASCADE BEFORE  UPDATE OF ELEMENT_NAME,
FD ON TMT_WBS  REFERENCING  OLD AS o  NEW AS n  FOR EACH ROW  MODE DB2SQL


WHEN ( o.TYPE_ID IN (3,59, 236, 237, 238, 239, 240, 241, 260) AND o.FS_LABOR_CODE IS NOT NULL AND o.FS_LABOR_CODE <> 'PMOFFICE' AND o.FS_STATE NOT IN (3, 8) AND ((o.FD <> n.FD AND n.FD=39) OR (n.ELEMENT_NAME<>o.ELEMENT_NAME)) )

BEGIN ATOMIC

     SET n.FS_STATE = CASE WHEN o.FS_STATE IN (0,4,5) THEN 1 WHEN  o.FS_STATE=6 THEN 7 WHEN o.FS_STATE=2 THEN 3 ELSE o.FS_STATE END;--

END;

------------------------------------------------------------------------------------------
-- Trigger For PDA
------------------------------------------------------------------------------------------
CREATE TRIGGER TMT_COSTS_D AFTER DELETE ON TMT_COSTS REFERENCING OLD AS o FOR EACH ROW MODE DB2SQL
BEGIN ATOMIC UPDATE PDA_LOOKUP SET REC_STATUS = 'D' WHERE ELEMENT_ID = o.ELEMENT_ID; END;

------------------------------------------------------------------------------------------
-- No charge code is created while inserting a new record into TMT_WBS
------------------------------------------------------------------------------------------
--CREATE TRIGGER WBS_LABOR
--AFTER INSERT
--ON TMT_WBS
--REFERENCING NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( (n.TRANSFER_FLAG IS NULL) OR ((n.TRANSFER_FLAG <> 'C') AND (n.TRANSFER_FLAG <> 'R') AND (n.TRANSFER_FLAG <> 'D') AND (n.TRANSFER_FLAG <> 'S')) )
--BEGIN ATOMIC
--	INSERT INTO BUDGETS(BUDGET_ID, PROJECT_ID, ELEMENT_ID, TYPE_ID, CREATED_BY, CHARGECODE_ID, AUTO_FLAG, REC_USER) VALUES(GET_UNIQUE_ID(), n.PROJECT_ID, n.ELEMENT_ID, n.TYPE_ID, n.REC_USER, 'EXPENSE_LABOR', 1, n.REC_USER); INSERT INTO BUDGETS(BUDGET_ID, PROJECT_ID, ELEMENT_ID, TYPE_ID, CREATED_BY, CHARGECODE_ID, AUTO_FLAG, REC_USER) VALUES(GET_UNIQUE_ID(), n.PROJECT_ID, n.ELEMENT_ID, n.TYPE_ID, n.REC_USER, 'CAPITAL_LABOR', 1, n.REC_USER); INSERT INTO BUDGETS(BUDGET_ID, PROJECT_ID, ELEMENT_ID, TYPE_ID, CREATED_BY, CHARGECODE_ID, AUTO_FLAG, REC_USER) VALUES(GET_UNIQUE_ID(), n.PROJECT_ID, n.ELEMENT_ID, n.TYPE_ID, n.REC_USER, 'BILLABLE_BENEFIT_LABOR', 1, n.REC_USER); INSERT INTO BUDGETS(BUDGET_ID, PROJECT_ID, ELEMENT_ID, TYPE_ID, CREATED_BY, CHARGECODE_ID, AUTO_FLAG, REC_USER) VALUES(GET_UNIQUE_ID(), n.PROJECT_ID, n.ELEMENT_ID, n.TYPE_ID, n.REC_USER, 'NON_BILLABLE_BENEFIT_LABOR', 1, n.REC_USER); END;

------------------------------------------------------------------------------------------
-- There is no budget charge code delete
------------------------------------------------------------------------------------------
--CREATE TRIGGER BUDGET_DELETE
--AFTER DELETE
--ON BUDGETS
--REFERENCING OLD AS o
--FOR EACH ROW MODE DB2SQL
--BEGIN ATOMIC
--     UPDATE TMT_WBS SET ROLLUP_BITS = BIT_OR(ROLLUP_BITS, 2097152) WHERE ELEMENT_ID = o.ELEMENT_ID; UPDATE TMT_CRI SET ROLLUP_BITS = BIT_OR(ROLLUP_BITS, 2097152) WHERE ELEMENT_ID = o.ELEMENT_ID; END;

CREATE TRIGGER BUDGET_UPDATE NO CASCADE
BEFORE UPDATE OF
	AUTO_FLAG,
	ESTIMATED_BUDGET,
	ESTIMATED_ACTUAL,
	ESTIMATED_ETC,
	ESTIMATED_PAID_TODATE,
	ETC,
	PAID_TODATE,
	PROB_PERCENT
ON BUDGETS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	o.AUTO_FLAG <> n.AUTO_FLAG OR
	o.ESTIMATED_BUDGET <> n.ESTIMATED_BUDGET OR
	o.ESTIMATED_ACTUAL <> n.ESTIMATED_ACTUAL OR
	o.ESTIMATED_ETC <> n.ESTIMATED_ETC OR
	o.ESTIMATED_PAID_TODATE <> n.ESTIMATED_PAID_TODATE OR
	o.ETC <> n.ETC OR
	o.PAID_TODATE <> n.PAID_TODATE OR
	o.PROB_PERCENT <> n.PROB_PERCENT
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 2097152) < 1048576) THEN n.ROLLUP_BITS + 1048576 ELSE n.ROLLUP_BITS END; END;

CREATE TRIGGER BUDGET_BUDGET NO CASCADE
BEFORE UPDATE OF
	BUDGET
ON BUDGETS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	o.BUDGET <> n.BUDGET
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 2097152) < 1048576) THEN n.ROLLUP_BITS + 1048576 ELSE n.ROLLUP_BITS END,
     	n.TIMEPHASED_BITS = CASE WHEN (MOD(n.TIMEPHASED_BITS, 4) < 2) THEN n.TIMEPHASED_BITS + 2 ELSE n.TIMEPHASED_BITS END; END;

CREATE TRIGGER BUDGET_ACTUAL NO CASCADE
BEFORE UPDATE OF
	ACTUAL
ON BUDGETS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	o.ACTUAL <> n.ACTUAL
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 2097152) < 1048576) THEN n.ROLLUP_BITS + 1048576 ELSE n.ROLLUP_BITS END,
     	n.TIMEPHASED_BITS = CASE WHEN (MOD(n.TIMEPHASED_BITS, 8) < 4) THEN n.TIMEPHASED_BITS + 4 ELSE n.TIMEPHASED_BITS END,
     	n.COST_CENTER_BITS = CASE WHEN (MOD(n.COST_CENTER_BITS, 8) < 4) THEN n.COST_CENTER_BITS + 4 ELSE n.COST_CENTER_BITS END; END;

--CREATE TRIGGER BUDGET_EDITED
--AFTER UPDATE OF
--	ACTUAL_EDITED
--ON BUDGETS
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN (o.ACTUAL_EDITED <> n.ACTUAL_EDITED)
--BEGIN ATOMIC
--     DELETE FROM TIMEPHASED_DELTA WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_ID = n.ELEMENT_ID AND CHARGECODE_ID = n.CHARGECODE_ID; END;

CREATE TRIGGER PBB_PROCESS_FLAG NO CASCADE
BEFORE UPDATE OF
	PROCESS_FLAG
ON PROJECT_BUDGET_BREAKDOWN
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	(o.PROCESS_FLAG = 'T' AND n.PROCESS_FLAG = 'D') OR
	(o.PROCESS_FLAG = 'D' AND n.PROCESS_FLAG = 'T')
)
BEGIN ATOMIC
     SET n.PROCESS_FLAG = 'I'; END;

CREATE TRIGGER WBS_BUDGET_UPDATE
AFTER UPDATE OF
	AUTO_FLAG,
	ESTIMATED_BUDGET,
	ESTIMATED_ACTUAL,
	ESTIMATED_ETC,
	ESTIMATED_PAID_TODATE,
	BUDGET,
	ACTUAL,
	ETC,
	PAID_TODATE,
	PROB_PERCENT
ON BUDGETS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	o.AUTO_FLAG <> n.AUTO_FLAG OR
	o.ESTIMATED_BUDGET <> n.ESTIMATED_BUDGET OR
	o.ESTIMATED_ACTUAL <> n.ESTIMATED_ACTUAL OR
	o.ESTIMATED_ETC <> n.ESTIMATED_ETC OR
	o.ESTIMATED_PAID_TODATE <> n.ESTIMATED_PAID_TODATE OR
	o.BUDGET <> n.BUDGET OR
	o.ACTUAL <> n.ACTUAL OR
	o.ETC <> n.ETC OR
	o.PAID_TODATE <> n.PAID_TODATE OR
	o.PROB_PERCENT <> n.PROB_PERCENT
	)
)
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END,
     	TIMEPHASED_BITS = CASE WHEN (MOD(TIMEPHASED_BITS, 4) < 2) THEN TIMEPHASED_BITS + 2 ELSE TIMEPHASED_BITS END
     WHERE PROJECT_ID = n.PROJECT_ID
     	AND ELEMENT_ID = n.ELEMENT_ID; UPDATE TMT_CRI
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END,
     	TIMEPHASED_BITS = CASE WHEN (MOD(TIMEPHASED_BITS, 4) < 2) THEN TIMEPHASED_BITS + 2 ELSE TIMEPHASED_BITS END
     WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_ID = n.ELEMENT_ID AND LEVEL_ID > 1; END;

CREATE TRIGGER WBS_ACTUAL_BIT NO CASCADE
BEFORE UPDATE OF FD, MILESTONE_PERCENT, ACTUAL_START, ACTUAL_FINISH
ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	(o.FD <> n.FD) OR
	(o.MILESTONE_PERCENT <> n.MILESTONE_PERCENT) OR
	COALESCE(o.ACTUAL_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.ACTUAL_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.ACTUAL_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.ACTUAL_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 2048) < 1024) THEN n.ROLLUP_BITS + 1024 ELSE n.ROLLUP_BITS END; END;

CREATE TRIGGER TA_ACTUAL_BIT NO CASCADE
BEFORE UPDATE OF ACTUAL_START, ACTUAL_FINISH, WORK_TODATE, MILESTONE_PERCENT
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.ACTUAL_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.ACTUAL_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.ACTUAL_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.ACTUAL_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) OR
	(o.WORK_TODATE <> n.WORK_TODATE ) OR
	(o.MILESTONE_PERCENT <> n.MILESTONE_PERCENT )
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 2048) < 1024) THEN n.ROLLUP_BITS + 1024 ELSE n.ROLLUP_BITS END; END;

CREATE TRIGGER TA_AD_WBS
AFTER UPDATE OF ACTUAL_START, ACTUAL_FINISH, WORK_TODATE, MILESTONE_PERCENT
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.ACTUAL_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.ACTUAL_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.ACTUAL_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.ACTUAL_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) OR
	(o.WORK_TODATE <> n.WORK_TODATE ) OR
	(o.MILESTONE_PERCENT <> n.MILESTONE_PERCENT )
	)
)
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END 
     WHERE ELEMENT_ID = n.TASK_ID; END;


CREATE TRIGGER TA_AD_CRI
AFTER UPDATE OF ACTUAL_START, ACTUAL_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.TABLE_TYPE = 3) AND (n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.ACTUAL_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.ACTUAL_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.ACTUAL_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.ACTUAL_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     UPDATE TMT_CRI
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
     WHERE ELEMENT_ID = n.TASK_ID; END;


CREATE TRIGGER WBS_SCHED_BIT NO CASCADE
BEFORE UPDATE OF SCHED_START, SCHED_FINISH, SCHED_DURATION
ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.SCHED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.SCHED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.SCHED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.SCHED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) OR
	(o.SCHED_DURATION <> n.SCHED_DURATION)
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 4096) < 2048) THEN n.ROLLUP_BITS + 2048 ELSE n.ROLLUP_BITS END; END;

CREATE TRIGGER TA_SCHED_BIT NO CASCADE
BEFORE UPDATE OF SCHED_START, SCHED_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.SCHED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.SCHED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.SCHED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.SCHED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 4096) < 2048) THEN n.ROLLUP_BITS + 2048 ELSE n.ROLLUP_BITS END, n.COST_CENTER_BITS = 1; END;

CREATE TRIGGER TA_SD_WBS
AFTER UPDATE OF SCHED_START, SCHED_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.TABLE_TYPE = 1) AND (n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.SCHED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.SCHED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.SCHED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.SCHED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END 
     WHERE ELEMENT_ID = n.TASK_ID; END;


CREATE TRIGGER TA_SD_CRI
AFTER UPDATE OF SCHED_START, SCHED_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.TABLE_TYPE = 3) AND (n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.SCHED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.SCHED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.SCHED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.SCHED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     UPDATE TMT_CRI
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
     WHERE ELEMENT_ID = n.TASK_ID; END;


CREATE TRIGGER WBS_PLANNED_BIT NO CASCADE
BEFORE UPDATE OF PLANNED_START, PLANNED_FINISH, PLANNED_DURATION
ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.PLANNED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.PLANNED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.PLANNED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.PLANNED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) OR
	o.planned_duration <> n.planned_duration
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 8192) < 4096) THEN n.ROLLUP_BITS + 4096 ELSE n.ROLLUP_BITS END; END;

CREATE TRIGGER TA_PLANNED_BIT NO CASCADE
BEFORE UPDATE OF PLANNED_START, PLANNED_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.PLANNED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.PLANNED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.PLANNED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.PLANNED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 8192) < 4096) THEN n.ROLLUP_BITS + 4096 ELSE n.ROLLUP_BITS END; END;

CREATE TRIGGER TA_ND_WBS
AFTER UPDATE OF PLANNED_START, PLANNED_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.TABLE_TYPE = 1) AND (n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.PLANNED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.PLANNED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.PLANNED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.PLANNED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END 
     WHERE ELEMENT_ID = n.TASK_ID; END;

CREATE TRIGGER TA_ND_CRI
AFTER UPDATE OF PLANNED_START, PLANNED_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.TABLE_TYPE = 3) AND (n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.PLANNED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.PLANNED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.PLANNED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.PLANNED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     UPDATE TMT_CRI
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
     WHERE ELEMENT_ID = n.TASK_ID; END;

CREATE TRIGGER WBS_FORECAST_BIT NO CASCADE
BEFORE UPDATE OF FORECAST_START, FORECAST_FINISH, FORECAST_DURATION
ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.FORECAST_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.FORECAST_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.FORECAST_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.FORECAST_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) OR 
	(o.FORECAST_DURATION <> n.FORECAST_DURATION)
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 16384) < 8192) THEN n.ROLLUP_BITS + 8192 ELSE n.ROLLUP_BITS END; END;

CREATE TRIGGER TA_FORECAST_BIT NO CASCADE
BEFORE UPDATE OF FORECAST_START, FORECAST_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.FORECAST_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.FORECAST_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.FORECAST_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.FORECAST_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 16384) < 8192) THEN n.ROLLUP_BITS + 8192 ELSE n.ROLLUP_BITS END; END;

CREATE TRIGGER TA_FD_WBS
AFTER UPDATE OF FORECAST_START, FORECAST_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.TABLE_TYPE = 1) AND (n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.FORECAST_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.FORECAST_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.FORECAST_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.FORECAST_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END 
     WHERE ELEMENT_ID = n.TASK_ID; END;

CREATE TRIGGER TA_FD_CRI
AFTER UPDATE OF FORECAST_START, FORECAST_FINISH
ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.TABLE_TYPE = 3) AND (n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.FORECAST_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.FORECAST_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.FORECAST_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.FORECAST_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     UPDATE TMT_CRI
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
     WHERE ELEMENT_ID = n.TASK_ID; END;

CREATE TRIGGER TA_EFFORTS_BEFORE NO CASCADE
BEFORE UPDATE OF
	PLANNED_WORK,
	SCHED_WORK,
	ETC,
	WORK_TODATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	o.PLANNED_WORK <> n.PLANNED_WORK OR
	o.SCHED_WORK <> n.SCHED_WORK OR
	o.ETC <> n.ETC OR
	o.WORK_TODATE <> n.WORK_TODATE
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 8) < 4) THEN n.ROLLUP_BITS + 4 ELSE n.ROLLUP_BITS END; END;

CREATE TRIGGER TA_EFFORTS_WBS
AFTER UPDATE OF
	PLANNED_WORK,
	SCHED_WORK,
	ETC,
	WORK_TODATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.TABLE_TYPE = 1) AND (n.ROLLUP_BITS <> -1) AND
	(
	o.PLANNED_WORK <> n.PLANNED_WORK OR
	o.SCHED_WORK <> n.SCHED_WORK OR
	o.ETC <> n.ETC OR
	o.WORK_TODATE <> n.WORK_TODATE
	)
)
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) AND (MOD(ROLLUP_BITS, 8) < 4) THEN ROLLUP_BITS + 5
	     	WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1
	     	WHEN (MOD(ROLLUP_BITS, 8) < 4) THEN ROLLUP_BITS + 4 ELSE ROLLUP_BITS END 
     WHERE ELEMENT_ID = n.TASK_ID; UPDATE BUDGETS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
     WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_ID = n.TASK_ID
     	AND CAPITAL_EXPENSE IN (11, 14, 21, 24, 31, 34); END;


CREATE TRIGGER TA_EFFORTS_CRI
AFTER UPDATE OF
	PLANNED_WORK,
	SCHED_WORK,
	ETC,
	WORK_TODATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.TABLE_TYPE = 3) AND (n.ROLLUP_BITS <> -1) AND
	(
	o.PLANNED_WORK <> n.PLANNED_WORK OR
	o.SCHED_WORK <> n.SCHED_WORK OR
	o.ETC <> n.ETC OR
	o.WORK_TODATE <> n.WORK_TODATE
	)
)
BEGIN ATOMIC
     UPDATE TMT_CRI
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN ROLLUP_BITS + 1 ELSE ROLLUP_BITS END
     WHERE ELEMENT_ID = n.TASK_ID; UPDATE BUDGETS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
     WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_ID = n.TASK_ID
     	AND CAPITAL_EXPENSE IN (11, 14, 21, 24, 31, 34); END;

CREATE TRIGGER TA_RATES_BEFORE NO CASCADE
BEFORE UPDATE OF
	BILLABLE,
	CAPITAL_EXPENSE,
	REGULAR_COST,
	REGULAR_RATE,
	SPECIAL_COST,
	SPECIAL_RATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.ROLLUP_BITS <> -1) AND
	(
	o.BILLABLE <> n.BILLABLE OR
	o.CAPITAL_EXPENSE <> n.CAPITAL_EXPENSE OR
	o.REGULAR_COST <> n.REGULAR_COST OR
	o.REGULAR_RATE <> n.REGULAR_RATE OR
	o.SPECIAL_COST <> n.SPECIAL_COST OR
	o.SPECIAL_RATE <> n.SPECIAL_RATE
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 8) < 4) THEN n.ROLLUP_BITS + 4 ELSE n.ROLLUP_BITS END; END;


CREATE TRIGGER TA_RATES_WBS
AFTER UPDATE OF
	BILLABLE,
	CAPITAL_EXPENSE,
	REGULAR_COST,
	REGULAR_RATE,
	SPECIAL_COST,
	SPECIAL_RATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.TABLE_TYPE = 1) AND (n.ROLLUP_BITS <> -1) AND
	(
	o.BILLABLE <> n.BILLABLE OR
	o.CAPITAL_EXPENSE <> n.CAPITAL_EXPENSE OR
	o.REGULAR_COST <> n.REGULAR_COST OR
	o.REGULAR_RATE <> n.REGULAR_RATE OR
	o.SPECIAL_COST <> n.SPECIAL_COST OR
	o.SPECIAL_RATE <> n.SPECIAL_RATE
	)
)
BEGIN ATOMIC
	UPDATE TMT_WBS
	SET ROLLUP_BITS = ROLLUP_BITS + CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN 1048576 ELSE 0 END 
	+ CASE WHEN (MOD(ROLLUP_BITS, 8) < 4) THEN 4 ELSE 0 END 
	+ CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN 1 ELSE 0 END 
	WHERE PROJECT_ID = n.PROJECT_ID
	AND ELEMENT_ID = n.TASK_ID; UPDATE BUDGETS
	SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
	WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_ID = n.TASK_ID AND CAPITAL_EXPENSE IN (11, 14, 21, 24, 31, 34); END;


CREATE TRIGGER TA_RATES_CRI
AFTER UPDATE OF
	BILLABLE,
	CAPITAL_EXPENSE,
	REGULAR_COST,
	REGULAR_RATE,
	SPECIAL_COST,
	SPECIAL_RATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN (
	(n.TABLE_TYPE = 3) AND (n.ROLLUP_BITS <> -1) AND
	(
	o.BILLABLE <> n.BILLABLE OR
	o.CAPITAL_EXPENSE <> n.CAPITAL_EXPENSE OR
	o.REGULAR_COST <> n.REGULAR_COST OR
	o.REGULAR_RATE <> n.REGULAR_RATE OR
	o.SPECIAL_COST <> n.SPECIAL_COST OR
	o.SPECIAL_RATE <> n.SPECIAL_RATE
	)
)
BEGIN ATOMIC
	UPDATE TMT_CRI
	SET ROLLUP_BITS = ROLLUP_BITS + CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN 1048576 ELSE 0 END 
	+ CASE WHEN (MOD(ROLLUP_BITS, 8) < 4) THEN 4 ELSE 0 END 
	+ CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN 1 ELSE 0 END 
	WHERE PROJECT_ID = n.PROJECT_ID
	AND ELEMENT_ID = n.TASK_ID; UPDATE BUDGETS
	SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
	WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_ID = n.TASK_ID AND CAPITAL_EXPENSE IN (11, 14, 21, 24, 31, 34); END;


CREATE TRIGGER TA_LABOR
AFTER INSERT
ON TASK_ASSIGNMENTS
REFERENCING NEW AS n
FOR EACH ROW MODE DB2SQL
BEGIN ATOMIC
	UPDATE BUDGETS
	SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN
ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
	WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_ID = n.TASK_ID
     	AND CAPITAL_EXPENSE IN (11, 14, 21, 24, 31, 34); UPDATE TMT_WBS
	SET ROLLUP_BITS = ROLLUP_BITS + CASE WHEN (MOD(ROLLUP_BITS, 2097152) <
1048576) THEN 1048576 ELSE 0 END
	+ CASE WHEN (MOD(ROLLUP_BITS, 8192) < 4096) THEN 4096 ELSE 0 END
	+ CASE WHEN (MOD(ROLLUP_BITS, 8) < 4) THEN 4 ELSE 0 END
	+ CASE WHEN (MOD(ROLLUP_BITS, 4) < 2) THEN 2 ELSE 0 END
	+ CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN 1 ELSE 0 END 
	WHERE ELEMENT_ID = n.TASK_ID; UPDATE TMT_CRI
	SET ROLLUP_BITS = ROLLUP_BITS + CASE WHEN (MOD(ROLLUP_BITS, 2097152) <
1048576) THEN 1048576 ELSE 0 END
	+ CASE WHEN (MOD(ROLLUP_BITS, 8192) < 4096) THEN 4096 ELSE 0 END
	+ CASE WHEN (MOD(ROLLUP_BITS, 8) < 4) THEN 4 ELSE 0 END
	+ CASE WHEN (MOD(ROLLUP_BITS, 4) < 2) THEN 2 ELSE 0 END
	+ CASE WHEN (MOD(ROLLUP_BITS, 2) < 1) THEN 1 ELSE 0 END
	WHERE ELEMENT_ID = n.TASK_ID; END;

CREATE TRIGGER COST_DELETE
AFTER DELETE
ON TMT_COSTS
REFERENCING OLD AS o
FOR EACH ROW MODE DB2SQL
BEGIN ATOMIC
	UPDATE BUDGETS
	SET ROLLUP_BITS = ROLLUP_BITS + CASE WHEN (MOD(ROLLUP_BITS, 4194304) < 2097152) THEN 2097152 ELSE 0 END 
	+ CASE WHEN (MOD(ROLLUP_BITS, 8388608) < 4194304) THEN 4194304 ELSE 0 END 
	WHERE PROJECT_ID = o.PROJECT_ID
	AND BUDGET_ID = o.BUDGET_ID; UPDATE TMT_WBS
	SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 4194304) < 2097152) THEN ROLLUP_BITS + 2097152 ELSE ROLLUP_BITS END 
	WHERE ELEMENT_ID = (SELECT ELEMENT_ID FROM BUDGETS
	WHERE BUDGET_ID = o.BUDGET_ID AND PROJECT_ID = o.PROJECT_ID); UPDATE TMT_CRI
	SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 4194304) < 2097152) THEN ROLLUP_BITS + 2097152 ELSE ROLLUP_BITS END
	WHERE ELEMENT_ID = (SELECT ELEMENT_ID FROM BUDGETS
	WHERE BUDGET_ID = o.BUDGET_ID AND PROJECT_ID = o.PROJECT_ID) AND LEVEL_ID > 1; END;

CREATE TRIGGER COST_UPDATE
AFTER UPDATE OF
	BILLABLE, ACTUAL_AMOUNT, TAX1_AMOUNT, TAX2_AMOUNT, EXCHANGE_RATE
ON TMT_COSTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	o.BILLABLE <> n.BILLABLE OR
	o.ACTUAL_AMOUNT <> n.ACTUAL_AMOUNT OR
	o.TAX1_AMOUNT <> n.TAX1_AMOUNT OR
	o.TAX2_AMOUNT <> n.TAX2_AMOUNT OR
	o.EXCHANGE_RATE <> n.EXCHANGE_RATE
)
BEGIN ATOMIC
	UPDATE BUDGETS
	SET ROLLUP_BITS = ROLLUP_BITS + CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN 1048576 ELSE 0 END 
	+ CASE WHEN (MOD(ROLLUP_BITS, 8388608) < 4194304) THEN 4194304 ELSE 0 END 
	WHERE BUDGET_ID = n.BUDGET_ID
	AND PROJECT_ID = n.PROJECT_ID; UPDATE TMT_WBS
	SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
	WHERE ELEMENT_ID = (SELECT ELEMENT_ID FROM BUDGETS
	WHERE BUDGET_ID = n.BUDGET_ID AND PROJECT_ID = n.PROJECT_ID); UPDATE TMT_CRI
	SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 2097152) < 1048576) THEN ROLLUP_BITS + 1048576 ELSE ROLLUP_BITS END
	WHERE ELEMENT_ID = (SELECT ELEMENT_ID FROM BUDGETS WHERE BUDGET_ID = n.BUDGET_ID) AND LEVEL_ID > 1; END;

CREATE TRIGGER WBS_ASSIGNED_INS
AFTER  INSERT  ON TMT_CRI
REFERENCING  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN ( (n.WBS_ID IS NOT NULL) AND (n.WBS_ID <> n.ELEMENT_ID) )
BEGIN ATOMIC
     UPDATE TMT_CRI SET WBS_ASSIGNED = WBS_ASSIGNED + 1 
     WHERE TYPE_ID IN (4,5,6,35,213,215,226,227,244,257,64) AND RANK >SUBSTR(n.RANK,1,12) AND RANK < n.RANK AND SUBSTR(n.RANK, 1, CASE WHEN LENGTH(RANK) > LENGTH(n.RANK) THEN 1 ELSE LENGTH(RANK) END) = RANK; END;

CREATE TRIGGER WBS_ASSIGNED_DEL
AFTER DELETE
ON TMT_CRI
REFERENCING OLD AS o
FOR EACH ROW MODE DB2SQL
WHEN ( (o.WBS_ID IS NOT NULL) AND (o.WBS_ID <> o.ELEMENT_ID) )
BEGIN ATOMIC
     UPDATE TMT_CRI SET WBS_ASSIGNED = WBS_ASSIGNED - 1
     WHERE TYPE_ID IN (4,5,6,35,213,215,226,227,244,257,64) AND RANK >SUBSTR(o.RANK,1,12) AND RANK < o.RANK AND SUBSTR(o.RANK, 1, CASE WHEN LENGTH(RANK) > LENGTH(o.RANK) THEN 1 ELSE LENGTH(RANK) END) = RANK; END;

CREATE TRIGGER WBS_ASSIGNED_MOVE
AFTER  UPDATE OF PARENT_ID ON TMT_CRI
REFERENCING  OLD AS o  NEW AS n
FOR EACH ROW  MODE DB2SQL
WHEN ( (o.PARENT_ID <> n.PARENT_ID) AND (n.WBS_ASSIGNED > 0) )
BEGIN ATOMIC
     UPDATE TMT_CRI 
     SET WBS_ASSIGNED = WBS_ASSIGNED + n.WBS_ASSIGNED 
     WHERE TYPE_ID IN (4,5,6,35,213,215,226,227,244,257,64) 
     	AND (LEVEL_ID < n.LEVEL_ID) AND RANK >SUBSTR(n.RANK,1,12) 
     	AND RANK < n.RANK 
     	AND SUBSTR(n.RANK, 1, CASE WHEN LENGTH(RANK) > LENGTH(n.RANK) THEN 1 ELSE LENGTH(RANK) END) = RANK; UPDATE TMT_CRI 
     SET WBS_ASSIGNED = WBS_ASSIGNED - o.WBS_ASSIGNED 
     WHERE TYPE_ID IN (4,5,6,35,213,215,226,227,244,257,64) 
     	AND RANK >SUBSTR(o.RANK,1,12) 
     	AND RANK < o.RANK 
     	AND SUBSTR(o.RANK, 1, CASE WHEN LENGTH(RANK) > LENGTH(o.RANK) THEN 1 ELSE LENGTH(RANK) END) = RANK; END;


CREATE TRIGGER CRI_ACTUAL_BIT NO CASCADE
BEFORE UPDATE OF FD, MILESTONE_PERCENT, ACTUAL_START, ACTUAL_FINISH
ON TMT_CRI
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	(n.WBS_ID IS NULL) AND
	(n.ROLLUP_BITS <> -1) AND
	(
	(o.FD <> n.FD) OR
	(o.MILESTONE_PERCENT <> n.MILESTONE_PERCENT) OR
	COALESCE(o.ACTUAL_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.ACTUAL_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.ACTUAL_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.ACTUAL_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 2048) < 1024) THEN n.ROLLUP_BITS + 1024 ELSE n.ROLLUP_BITS END; END;

CREATE TRIGGER CRI_SCHED_BIT NO CASCADE
BEFORE UPDATE OF SCHED_START, SCHED_FINISH
ON TMT_CRI
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	(n.WBS_ID IS NULL) AND
	(n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.SCHED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.SCHED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.SCHED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.SCHED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 4096) < 2048) THEN n.ROLLUP_BITS + 2048 ELSE n.ROLLUP_BITS END; END;

CREATE TRIGGER CRI_FORECAST_BIT NO CASCADE
BEFORE UPDATE OF FORECAST_START, FORECAST_FINISH
ON TMT_CRI
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	(n.WBS_ID IS NULL) AND
	(n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.FORECAST_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.FORECAST_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.FORECAST_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.FORECAST_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 16384) < 8192) THEN n.ROLLUP_BITS + 8192 ELSE n.ROLLUP_BITS END  ; END;

CREATE TRIGGER CRI_PLANNED_BIT NO CASCADE
BEFORE UPDATE OF PLANNED_START, PLANNED_FINISH
ON TMT_CRI
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN (
	(n.WBS_ID IS NULL) AND
	(n.ROLLUP_BITS <> -1) AND
	(
	COALESCE(o.PLANNED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) <> COALESCE(n.PLANNED_START, TIMESTAMP('1900-01-01 00:00:00.000000')) OR
	COALESCE(o.PLANNED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000')) <> COALESCE(n.PLANNED_FINISH, TIMESTAMP('2100-01-01 00:00:00.000000'))
	)
)
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 8192) < 4096) THEN n.ROLLUP_BITS + 4096 ELSE n.ROLLUP_BITS END; END;

------------------------------------------------------------------------------------------
-- No charge code is created while inserting a new record into TMT_CRI
------------------------------------------------------------------------------------------
--CREATE TRIGGER CRI_LABOR
--AFTER INSERT
--ON TMT_CRI
--REFERENCING NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ((n.TYPE_ID <> 14) AND ((n.WBS_ID IS NULL) OR (n.WBS_ID <> n.ELEMENT_ID)) AND (SUBSTR(n.PARENT_RANK, 1, 4) <> 'PRJ#') AND NOT
--	 EXISTS(SELECT BUDGET_ID FROM BUDGETS WHERE ELEMENT_ID = n.ELEMENT_ID AND CHARGECODE_ID LIKE '%LABOR'))
--BEGIN ATOMIC
--	INSERT INTO BUDGETS(BUDGET_ID, PROJECT_ID, ELEMENT_ID, TYPE_ID, CREATED_BY, CHARGECODE_ID, AUTO_FLAG, REC_USER) VALUES(GET_UNIQUE_ID(), n.PROJECT_ID, n.ELEMENT_ID, n.TYPE_ID, n.REC_USER, 'EXPENSE_LABOR', 1, n.REC_USER); INSERT INTO BUDGETS(BUDGET_ID, PROJECT_ID, ELEMENT_ID, TYPE_ID, CREATED_BY, CHARGECODE_ID, AUTO_FLAG, REC_USER) VALUES(GET_UNIQUE_ID(), n.PROJECT_ID, n.ELEMENT_ID, n.TYPE_ID, n.REC_USER, 'CAPITAL_LABOR', 1, n.REC_USER); INSERT INTO BUDGETS(BUDGET_ID, PROJECT_ID, ELEMENT_ID, TYPE_ID, CREATED_BY, CHARGECODE_ID, AUTO_FLAG, REC_USER) VALUES(GET_UNIQUE_ID(), n.PROJECT_ID, n.ELEMENT_ID, n.TYPE_ID, n.REC_USER, 'BILLABLE_BENEFIT_LABOR', 1, n.REC_USER); INSERT INTO BUDGETS(BUDGET_ID, PROJECT_ID, ELEMENT_ID, TYPE_ID, CREATED_BY, CHARGECODE_ID, AUTO_FLAG, REC_USER) VALUES(GET_UNIQUE_ID(), n.PROJECT_ID, n.ELEMENT_ID, n.TYPE_ID, n.REC_USER, 'NON_BILLABLE_BENEFIT_LABOR', 1, n.REC_USER); END;

CREATE TRIGGER CRI_ASSIGN_NAMES
AFTER UPDATE OF ASSIGN_NAMES ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ( (n.TRANSFER_FLAG = 'C') OR (n.TRANSFER_FLAG = 'W') )
BEGIN ATOMIC
	UPDATE TMT_CRI SET ASSIGN_NAMES = n.ASSIGN_NAMES WHERE PROJECT_ID = n.PROJECT_ID AND WBS_ID = n.ELEMENT_ID AND LEVEL_ID > 1; END;

--CREATE TRIGGER WBS_STAGE_INSERT
--AFTER INSERT ON TMT_WBS
--REFERENCING NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = n.STAGE_ID) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 1, n.TYPE_ID, n.ELEMENT_ID, n.STAGE_ID, '0') ; END;

CREATE TRIGGER WBS_STAGE_INSERT
AFTER INSERT ON TMT_WBS
REFERENCING NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ( EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID 
  AND ELEMENT_TYPE = CASE WHEN n.TYPE_ID IN (154, 1) THEN 1
       WHEN n.TYPE_ID IN (2,36,37,39,58,60,225,230,231,259) THEN 2
       WHEN n.TYPE_ID IN (3,59,236,237,238,239,240,241,260) THEN 3 END
  AND STAGE_ID = n.STAGE_ID) )
BEGIN ATOMIC
	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 1, n.TYPE_ID, n.ELEMENT_ID, n.STAGE_ID, '0', n.REC_USER, NULL) ; END;

--CREATE TRIGGER WBS_STAGE_AFTER
--AFTER UPDATE OF STAGE_ID ON TMT_WBS
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = n.STAGE_ID) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 1, n.TYPE_ID, n.ELEMENT_ID, n.STAGE_ID, '0') ; END;

CREATE TRIGGER WBS_STAGE_AFTER
AFTER UPDATE OF STAGE_ID ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ( o.STAGE_ID <> n.STAGE_ID AND EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID 
  AND ELEMENT_TYPE = CASE WHEN n.TYPE_ID IN (154, 1) THEN 1
       WHEN n.TYPE_ID IN (2,36,37,39,58,60,225,230,231,259) THEN 2
       WHEN n.TYPE_ID IN (3,59,236,237,238,239,240,241,260) THEN 3 END
  AND STAGE_ID = n.STAGE_ID) )
BEGIN ATOMIC
	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 1, n.TYPE_ID, n.ELEMENT_ID, n.STAGE_ID, '0', n.REC_USER, n.REVISION_HISTORY) ; END;

--CREATE TRIGGER CRI_STAGE_INSERT
--AFTER INSERT ON TMT_CRI
--REFERENCING NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = n.STAGE_ID) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 3, n.TYPE_ID, n.ELEMENT_ID, n.STAGE_ID, '0') ; END;

CREATE TRIGGER CRI_STAGE_INSERT
AFTER INSERT ON TMT_CRI
REFERENCING NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ( EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID 
AND ELEMENT_TYPE = CASE WHEN n.TYPE_ID IN (4, 65) THEN 4
	 WHEN n.TYPE_ID IN (257, 258) THEN 257
	 WHEN n.TYPE_ID IN (5,66) THEN 5
	 WHEN n.TYPE_ID IN (6,67) THEN 6
	 WHEN n.TYPE_ID IN (213, 235) THEN 213
	 WHEN n.TYPE_ID IN (226, 233) THEN 226
	 WHEN n.TYPE_ID IN (227, 234) THEN 227 END
AND STAGE_ID = n.STAGE_ID) )
BEGIN ATOMIC
	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 3, n.TYPE_ID, n.ELEMENT_ID, n.STAGE_ID, '0', n.REC_USER, NULL) ; END;

--CREATE TRIGGER CRI_STAGE_AFTER
--AFTER UPDATE OF STAGE_ID ON TMT_CRI
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = n.STAGE_ID) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 3, n.TYPE_ID, n.ELEMENT_ID, n.STAGE_ID, '0') ; END;

CREATE TRIGGER CRI_STAGE_AFTER
AFTER UPDATE OF STAGE_ID ON TMT_CRI
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ( o.STAGE_ID <> n.STAGE_ID AND EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID 
AND ELEMENT_TYPE = CASE WHEN n.TYPE_ID IN (4, 65) THEN 4
	 WHEN n.TYPE_ID IN (257, 258) THEN 257
	 WHEN n.TYPE_ID IN (5,66) THEN 5
	 WHEN n.TYPE_ID IN (6,67) THEN 6
	 WHEN n.TYPE_ID IN (213, 235) THEN 213
	 WHEN n.TYPE_ID IN (226, 233) THEN 226
	 WHEN n.TYPE_ID IN (227, 234) THEN 227 END
AND STAGE_ID = n.STAGE_ID) )
BEGIN ATOMIC
	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 3, n.TYPE_ID, n.ELEMENT_ID, n.STAGE_ID, '0', n.REC_USER, n.REVISION_HISTORY) ; END;


-- WBS State **********************************************
--CREATE TRIGGER WBS_CLOSED
--AFTER UPDATE OF MILESTONE_PERCENT, FD
--ON TMT_WBS
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( (o.FD <>  n.FD OR o.MILESTONE_PERCENT <> n.MILESTONE_PERCENT) AND n.MILESTONE_PERCENT = 10000 AND n.FD = 39 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 4949) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 1, n.TYPE_ID, n.ELEMENT_ID, 4949, '0') ; END;
--
--CREATE TRIGGER WBS_TOBE_VALIDATED
--AFTER UPDATE OF MILESTONE_PERCENT, FD
--ON TMT_WBS
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.FD <> n.FD AND n.MILESTONE_PERCENT = 10000 AND n.FD = 42 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 4948) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 1, n.TYPE_ID, n.ELEMENT_ID, 4948, '0') ; END;

-- Change Request State **********************************************
--CREATE TRIGGER CR_CLOSED
--AFTER UPDATE OF CLOSED_BY
--ON TMT_CRI
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.CLOSED_BY IS NULL AND n.CLOSED_BY IS NOT NULL AND n.TYPE_ID = 4 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 4910) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 3, n.TYPE_ID, n.ELEMENT_ID, 4910, '0') ; END;
--
--CREATE TRIGGER CR_TOBE_VALIDATED
--AFTER UPDATE OF MILESTONE_PERCENT, FD
--ON TMT_CRI
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.FD <> n.FD AND n.MILESTONE_PERCENT = 10000 AND n.FD = 42 AND n.TYPE_ID = 4 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 4909) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 2, n.TYPE_ID, n.ELEMENT_ID, 4909, '0') ; END;

-- Issue State **************************************************************
--CREATE TRIGGER IS_CLOSED
--AFTER UPDATE OF CLOSED_BY
--ON TMT_CRI
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.CLOSED_BY IS NULL AND n.CLOSED_BY IS NOT NULL AND n.TYPE_ID = 6 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 4939) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 2, n.TYPE_ID, n.ELEMENT_ID, 4939, '0') ; END;
--
--CREATE TRIGGER IS_TOBE_VALIDATED
--AFTER UPDATE OF MILESTONE_PERCENT, FD
--ON TMT_CRI
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.FD <> n.FD AND n.MILESTONE_PERCENT = 10000 AND n.FD = 42 AND n.TYPE_ID = 6 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 4938) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 2, n.TYPE_ID, n.ELEMENT_ID, 4938, '0') ; END;

-- Risk State *****************************************************************
--CREATE TRIGGER RS_CLOSED
--AFTER UPDATE OF CLOSED_BY
--ON TMT_CRI
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.CLOSED_BY IS NULL AND n.CLOSED_BY IS NOT NULL AND n.TYPE_ID = 5 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 4920) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 2, n.TYPE_ID, n.ELEMENT_ID, 4920, '0') ; END;
--
--CREATE TRIGGER RS_TOBE_VALIDATED
--AFTER UPDATE OF MILESTONE_PERCENT, FD
--ON TMT_CRI
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.FD <> n.FD AND n.MILESTONE_PERCENT = 10000 AND n.FD = 42 AND n.TYPE_ID = 5 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 4919) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 2, n.TYPE_ID, n.ELEMENT_ID, 4919, '0') ; END;

CREATE TRIGGER TA_DB_LABOR
AFTER DELETE ON TASK_ASSIGNMENTS
REFERENCING  OLD AS o
FOR EACH ROW  MODE DB2SQL
WHEN ( o.PROJECT_ID <> o.TASK_ID )
BEGIN ATOMIC
     UPDATE BUDGETS SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 8) < 4) THEN ROLLUP_BITS + 4 ELSE ROLLUP_BITS END
     WHERE PROJECT_ID = o.PROJECT_ID AND ELEMENT_ID = o.TASK_ID AND CAPITAL_EXPENSE IN (11, 14, 21, 24, 31, 34); UPDATE TMT_WBS SET EV_FLAG = '1'
     WHERE PROJECT_ID = o.PROJECT_ID AND ELEMENT_ID = o.TASK_ID; END;

--******************************************* leveling
CREATE TRIGGER TA_U_LEVEL_CHANGE
AFTER UPDATE OF RESOURCE_ID, RESOURCE_TYPE, BIT_FLAG, PROPOSED_WORK, UNRESOLVED_START, UNRESOLVED_FINISH, WORK_TODATE
ON TASK_ASSIGNMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ((o.RESOURCE_ID <> n.RESOURCE_ID) OR (o.RESOURCE_TYPE <> n.RESOURCE_TYPE) OR (o.BIT_FLAG <> n.BIT_FLAG) OR (o.PROPOSED_WORK <> n.PROPOSED_WORK)
	OR (o.UNRESOLVED_START <> n.UNRESOLVED_START) OR (o.UNRESOLVED_FINISH <> n.UNRESOLVED_FINISH) OR (o.WORK_TODATE <> n.WORK_TODATE))
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) AND LEVEL_STATE NOT IN (0, 4) 
     	AND ((SELECT LEVEL_LOCK FROM PROJECT_DETAILS WHERE PROJECT_ID = n.PROJECT_ID) = 0) 
     	THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END, 
     LEVEL_STATE = CASE WHEN ((SELECT LEVEL_LOCK FROM PROJECT_DETAILS WHERE PROJECT_ID = n.PROJECT_ID) > 0)
     	THEN LEVEL_STATE ELSE 0 END  
     WHERE ELEMENT_ID = n.TASK_ID; END;

--CREATE TRIGGER WBS_LEVEL_CHANGE NO CASCADE
--BEFORE UPDATE OF RANK, TYPE_ID, PRIORITY, CONSTRAINT_DT, CONSTRAINT_TYPE
--ON TMT_WBS
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( EXISTS (SELECT LEVEL_START FROM TMT_WBS WHERE ELEMENT_ID = n.PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0 ) AND
--	((o.RANK <> n.RANK) OR (o.TYPE_ID <> n.TYPE_ID) OR (o.PRIORITY <> n.PRIORITY) OR
--	(o.CONSTRAINT_DT <> n.CONSTRAINT_DT) OR (o.CONSTRAINT_TYPE <> n.CONSTRAINT_TYPE))
--)
--BEGIN ATOMIC
--     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 262144) < 131072) THEN n.ROLLUP_BITS + 131072 ELSE n.ROLLUP_BITS END END;
--
--CREATE TRIGGER TA_D_LEVEL_CHANGE
--AFTER DELETE ON TASK_ASSIGNMENTS
--REFERENCING OLD AS o FOR EACH ROW MODE DB2SQL
--WHEN ( EXISTS (SELECT LEVEL_START FROM TMT_WBS WHERE ELEMENT_ID = o.PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0 ) )
--BEGIN ATOMIC
--     UPDATE TMT_WBS
--     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END 
--     WHERE ELEMENT_ID = o.TASK_ID; END;
--
CREATE TRIGGER DEP_D_DEPAND_FLAG
AFTER DELETE ON DEPENDENCIES
REFERENCING OLD AS o FOR EACH ROW MODE DB2SQL
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 524288) < 262144) THEN ROLLUP_BITS + 262144 ELSE ROLLUP_BITS END 
     WHERE ELEMENT_ID = o.SUCC_ID; END;

CREATE TRIGGER DEP_I_DEPAND_FLAG
AFTER INSERT ON DEPENDENCIES
REFERENCING NEW AS n FOR EACH ROW MODE DB2SQL
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 1048576) < 524288) THEN ROLLUP_BITS + 524288 ELSE ROLLUP_BITS END 
     WHERE ELEMENT_ID = n.PRED_ID; END;

CREATE TRIGGER DEP_D_LEVEL_CHANGE
AFTER DELETE ON DEPENDENCIES
REFERENCING OLD AS o FOR EACH ROW MODE DB2SQL
WHEN ( EXISTS (SELECT LEVEL_START FROM TMT_WBS WHERE ELEMENT_ID = o.PRED_PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0 ) )
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END 
     WHERE ELEMENT_ID = o.PRED_ID; END;

CREATE TRIGGER DEP_D_LEVEL_SUCC
AFTER DELETE ON DEPENDENCIES
REFERENCING OLD AS o FOR EACH ROW MODE DB2SQL
WHEN ( EXISTS (SELECT LEVEL_START FROM TMT_WBS WHERE ELEMENT_ID = o.SUCC_PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0  ) )
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END 
     WHERE ELEMENT_ID = o.SUCC_ID; END;

CREATE TRIGGER DEP_I_LEVEL_CHANGE
AFTER INSERT ON DEPENDENCIES
REFERENCING NEW AS n FOR EACH ROW MODE DB2SQL
WHEN ( EXISTS (SELECT LEVEL_START FROM TMT_WBS WHERE ELEMENT_ID = n.PRED_PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0  ) )
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END 
     WHERE ELEMENT_ID = n.PRED_ID; END;

CREATE TRIGGER DEP_I_LEVEL_SUCC
AFTER INSERT ON DEPENDENCIES
REFERENCING NEW AS n FOR EACH ROW MODE DB2SQL
WHEN ( EXISTS (SELECT LEVEL_START FROM TMT_WBS WHERE ELEMENT_ID = n.SUCC_PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0  ) )
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END 
     WHERE ELEMENT_ID = n.SUCC_ID; END;

CREATE TRIGGER DEP_U_LEVEL_CHANGE
AFTER UPDATE ON DEPENDENCIES
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ( EXISTS (SELECT LEVEL_START FROM TMT_WBS WHERE ELEMENT_ID = n.PRED_PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0  ) )
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END 
     WHERE ELEMENT_ID = n.PRED_ID; END;

CREATE TRIGGER DEP_U_LEVEL_SUCC
AFTER UPDATE ON DEPENDENCIES
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ( EXISTS (SELECT LEVEL_START FROM TMT_WBS WHERE ELEMENT_ID = n.SUCC_PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0  ) )
BEGIN ATOMIC
     UPDATE TMT_WBS
     SET ROLLUP_BITS = CASE WHEN (MOD(ROLLUP_BITS, 262144) < 131072) THEN ROLLUP_BITS + 131072 ELSE ROLLUP_BITS END 
     WHERE ELEMENT_ID = n.SUCC_ID; END;

--CREATE TRIGGER WBS_U_LEVEL_CHANGE NO CASCADE
--BEFORE UPDATE OF SD, FD, START_DT, FINISH_DT, DURATION
--ON TMT_WBS
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( EXISTS (SELECT LEVEL_START FROM TMT_WBS WHERE ELEMENT_ID = n.PROJECT_ID AND LEVEL_START IS NOT NULL AND LEVEL_STATE <> 0  ) AND (n.TYPE_ID NOT IN (3,59) ) AND
--	( (o.SD <> n.SD) OR (o.FD <> n.FD) OR (o.START_DT <> n.START_DT) OR (o.FINISH_DT <> n.FINISH_DT) OR (o.DURATION <> n.DURATION) )
--)
--BEGIN ATOMIC
--     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 262144) < 131072) THEN n.ROLLUP_BITS + 131072 ELSE n.ROLLUP_BITS END END;

--******************************************* END leveling

--************************************** Begin of New triggers Definitions: November 10, 2002  **************************
--***********************************************************************************************************************
-- Begin of REQUIREMENTS Triggers
--CREATE TRIGGER REQ_CLOSED
--AFTER UPDATE OF CLOSED_BY
--ON TMT_CRI
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.CLOSED_BY IS NULL AND n.CLOSED_BY IS NOT NULL AND n.TYPE_ID = 213 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 4910) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 3, n.TYPE_ID, n.ELEMENT_ID, 4910, '0') ; END;
--
--CREATE TRIGGER REQ_TOBE_VALIDATED
--AFTER UPDATE OF MILESTONE_PERCENT, FD
--ON TMT_CRI
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.FD <> n.FD AND n.MILESTONE_PERCENT = 10000 AND n.FD = 42 AND n.TYPE_ID = 213 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 4909) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 3, n.TYPE_ID, n.ELEMENT_ID, 4909, '0') ; END;

-- Begin of DEFECTS Triggers
--CREATE TRIGGER DF_CLOSED
--AFTER UPDATE OF CLOSED_BY
--ON TMT_CRI
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.CLOSED_BY IS NULL AND n.CLOSED_BY IS NOT NULL AND n.TYPE_ID = 226 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 5386) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 3, n.TYPE_ID, n.ELEMENT_ID, 5386, '0') ; END;
--
--CREATE TRIGGER DF_TOBE_VALIDATED
--AFTER UPDATE OF MILESTONE_PERCENT, FD
--ON TMT_CRI
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.FD <> n.FD AND n.MILESTONE_PERCENT = 10000 AND n.FD = 42 AND n.TYPE_ID = 226 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 5385) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 3, n.TYPE_ID, n.ELEMENT_ID, 5385, '0') ; END;

-- Begin of SERVICE_REQUESTS Triggers
--CREATE TRIGGER SR_CLOSED
--AFTER UPDATE OF CLOSED_BY
--ON TMT_CRI
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.CLOSED_BY IS NULL AND n.CLOSED_BY IS NOT NULL AND n.TYPE_ID = 227 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 5417) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 3, n.TYPE_ID, n.ELEMENT_ID, 5417, '0') ; END;
--
--CREATE TRIGGER SR_TOBE_VALIDATED
--AFTER UPDATE OF MILESTONE_PERCENT, FD
--ON TMT_CRI
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.FD <> n.FD AND n.MILESTONE_PERCENT = 10000 AND n.FD = 42 AND n.TYPE_ID = 227 AND
--EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = 5416) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 3, n.TYPE_ID, n.ELEMENT_ID, 5416, '0') ; END;

--CREATE TRIGGER DOC_STAGE_AFTER
--AFTER UPDATE OF STAGE_ID ON TMT_DOCUMENTS
--REFERENCING OLD AS o NEW AS n
--FOR EACH ROW MODE DB2SQL
--WHEN ( EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = n.STAGE_ID) )
--BEGIN ATOMIC
--	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 4, n.TYPE_ID, n.ELEMENT_ID, n.STAGE_ID, '0') ; END;

CREATE TRIGGER DOC_STAGE_AFTER
AFTER UPDATE OF STAGE_ID ON TMT_DOCUMENTS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ( o.STAGE_ID <> n.STAGE_ID AND n.LINK_DOC <> 'Y' AND EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID 
AND ELEMENT_TYPE = CASE WHEN n.TYPE_ID IN (14,61,62,63,140) THEN 14 
WHEN n.TYPE_ID = 242 THEN 242
END
AND STAGE_ID = n.STAGE_ID) )
BEGIN ATOMIC
	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 4, n.TYPE_ID, n.ELEMENT_ID, n.STAGE_ID, '0', n.REC_USER, n.REVISION_HISTORY) ; END;


-- When Delete Project in WBS, than delete from PROJECT_STATE_DEF & PROJECT_STATE_TRG **********************
--CREATE TRIGGER WBS_DEL_PROJECT
--AFTER DELETE ON TMT_WBS
--REFERENCING OLD AS o
--FOR EACH ROW MODE DB2SQL
--WHEN ( o.TYPE_ID = 1 AND EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_TRG WHERE PROJECT_ID = o.PROJECT_ID 
--AND TABLE_TYPE = 1 AND ELEMENT_ID = o.ELEMENT_ID) )
--BEGIN ATOMIC
--	DELETE FROM PROJECT_STATE_DEF WHERE PROJECT_ID = o.PROJECT_ID; DELETE FROM PROJECT_STATE_TRG WHERE PROJECT_ID = o.PROJECT_ID; END;

-- DEPEND_FLAG in TMT_WBS ----------------------------------- ***************************************
-- Has been modified: DEP_D_LEVEL_CHANGE & DEP_I_LEVEL_CHANGE ***************************************

CREATE TRIGGER WBS_TO100_FLAG NO CASCADE
BEFORE UPDATE OF MILESTONE_PERCENT
ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ( (n.ROLLUP_BITS <> -1) AND (o.MILESTONE_PERCENT <> n.MILESTONE_PERCENT) AND (n.MILESTONE_PERCENT = 10000)
	AND EXISTS(SELECT PRED_ID FROM DEPENDENCIES WHERE PRED_ID = n.ELEMENT_ID) )
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 1048576) < 524288) THEN n.ROLLUP_BITS + 524288 ELSE n.ROLLUP_BITS END ; END;

CREATE TRIGGER WBS_FROM100_FLAG NO CASCADE
BEFORE UPDATE OF MILESTONE_PERCENT
ON TMT_WBS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ( (n.ROLLUP_BITS <> -1) AND (o.MILESTONE_PERCENT <> n.MILESTONE_PERCENT) AND (o.MILESTONE_PERCENT = 10000)
	AND EXISTS(SELECT PRED_ID FROM DEPENDENCIES WHERE PRED_ID = n.ELEMENT_ID) )
BEGIN ATOMIC
     SET n.ROLLUP_BITS = CASE WHEN (MOD(n.ROLLUP_BITS, 1048576) < 524288) THEN n.ROLLUP_BITS + 524288 ELSE n.ROLLUP_BITS END; END;

-- END *** DEPEND_FLAG in TMT_WBS ***************************************************************************

-- SCORECARDS State in TMT_ASSIGNED_SCORECARDS  *************************************************************
-- **********************************************************************************************************
CREATE TRIGGER SC_STAGE_INSERT
AFTER INSERT ON TMT_ASSIGNED_SCORECARDS
REFERENCING NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ( EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = n.STAGE_ID) )
BEGIN ATOMIC
	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 10, n.TYPE_ID, n.ELEMENT_ID, n.STAGE_ID, '0', n.REC_USER, NULL) ; END;

CREATE TRIGGER SC_STAGE_AFTER
AFTER UPDATE OF STAGE_ID ON TMT_ASSIGNED_SCORECARDS
REFERENCING OLD AS o NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ( EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID = n.PROJECT_ID AND ELEMENT_TYPE = n.TYPE_ID AND STAGE_ID = n.STAGE_ID) )
BEGIN ATOMIC
	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 10, n.TYPE_ID, n.ELEMENT_ID, n.STAGE_ID, '0', n.REC_USER, n.REVISION_HISTORY) ; END;

-- END *** SCORECARDS in TMT_ASSIGNED_SCORECARDS  ***************************************************************

-- Triggers for DOC_STAGE_INSERT ---------------------------------------------------------------------------------------
CREATE TRIGGER DOC_STAGE_INSERT
AFTER INSERT ON TMT_DOCUMENTS
REFERENCING NEW AS n
FOR EACH ROW MODE DB2SQL
WHEN ( EXISTS (SELECT STAGE_ID FROM PROJECT_STATE_DEF WHERE PROJECT_ID =
n.PROJECT_ID
AND ELEMENT_TYPE = CASE WHEN n.TYPE_ID IN (14,61,62,63,140) THEN 14
WHEN n.TYPE_ID = 242 THEN 242
END
AND STAGE_ID = n.STAGE_ID AND n.LINK_DOC <> 'Y') )
BEGIN ATOMIC
	INSERT INTO PROJECT_STATE_TRG VALUES(n.PROJECT_ID, 4, n.TYPE_ID, n.ELEMENT_ID, n.STAGE_ID, '0', n.REC_USER, NULL) ; END;

