/********************************************************************
# (C) Copyright IBM Corp.  2005-2006
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#********************************************************************/
/*
#Stored Procedure  :SP_SEARCH_2
#Short Description :Search and Assign a resource or profile to a element.
#Description       :Search or Search and Assign a resource or profile with effort to a WBS or CRI element.
				   :If search/assign a participant (without effort) all effort related parameters are not required.
				   :Search criteria is a sum of input parameters, witch tells how to filter out the result set.
				   :This function gets called from RPM client, Import and Level/Calculate.
RULE1: cannot assign more then once a resource to a element.
RULE2: cannot assign a resource to a project if the resource is already assigned to a task whitin the project.
RULE3: cannot assign to a task from project reserve more effort then remaining effort at the project.
RULE4: if bitflag = 9 then assignment without effort, otherwise assignment with effort.
RULE5: if bitflag between 1 and 8 the assignment is contoured: contour > 0 and work > 0
RULE6: if search by name then partially ignore the search criteria: org_id, geo_id, pool_id, work_loc, and_roles, and_skills, shw_roles, shw_skills
RULE7: if effort = 0 and bitflag < 9 then both start  finish dates are required
RULE8: if assign units <> 100%, then bitflag < 9 and effort > 0 and both start  finish dates are required
RULE9: cannot assign a resource if is not active, effective date is in the future or discontinue date is in the past
#Table Name        :
#Parameters        :
 $$bitflag  = 0 - ASP ---------------------------------------------------------------------------------------------
            = 1 - Daily         -- contour3
            = 2 - Weekly        -- contour3{, contur1}, {amount falg}
            = 3 - Monthly(C)    -- contour3{, contur2}
            = 4 - Monthly(B)    -- contour3{, contur2}
            = 5 - Monthly(A)    -- contour3{, contur2 | contour2  countour1}
            = 6 - Yearly(C)     -- contour2, contur4
            = 7 - Yearly(B)     -- contour2, contur4, contur1(only one day)
            = 8 - Yearly(A)     -- contour2, contur4, contur1(many days)
            = 9 - Participant
 $$contour (contour structure) ------------------------------------------------------------------------------------
  contur1:  DAY:    bits 0-6    0 = Su, 1 = Mo, 2 = Tu, 3 = We, 4 = Th, 5 = Fr, 6 = Sa,
  contur2:  WEEK:   bits 7-11   A) 7 = week1, 8 = week2, 8 = week3, 9 = week4
                                B) 1-First/2-Second/3-Third/4-Fourth/5-Last
                                C) The Day 1 to 31
  contour3:     EVRY:   bits 12-16  EVRY 1 to 31 day / week / month
  contour4:     MONTH:  bits 17-28  17 = Jan, 18 = Feb, ... 28 = Dec
  2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
  8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
                                                S F T W T M S
                                      w1 to w4  A(7 days from the 1st)
                                      5 4 3 2 1 B(Last, Fourth, Third, Second, First)
                                      31 .... 1 C(The Day 6)
                            31 ........ 1 (Day)
                            31 ........ 1 (week)
                            12 ........ 1 (month)
  D N O S A J J M A M F J (Month)
  Exemple:
  2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1 
  9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 
                                      5-4-3-2-1               B) 1-First/2-Second/3-Third/4-Fourth/5-Last
  X D N O S A J J M A M F J --EVERY-- -THE-DAY- S F T W T M S C) The Day 1 to 31
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 = 4096      1 (every day) 	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 = 8192      1 (every 2nd day) 
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 = 12288     1 (every 3rd day) 
  1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 = 536875014 2 (Monday, Tuesday, every week)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 = 4102      2 (Monday, Tuesday, every week)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 1 1 0 = 8198      2 (Monday, Monday, every 2nd week)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 = 4864      3 (6th day of every month)
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 1 0 0 0 0 0 1 0 = 8322      4 (1st Monday of every 2nd month)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 = 4224      5 (week1 of every month)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 = 4480      5 (week1, week2 of every month)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 1 1 0 0 0 0 0 0 0 = 8576      5 (week1, week2 of every 2nd month)
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 1 0 0 0 0 0 1 1 0 = 8454      5 (Monday, Tuesday, week2 every 2nd month)	
  1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 1 0 0 0 0 0 1 1 0 = 536879366 5 (Monday, Tuesday, week2 every 2nd month)	
  0 1 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0 0 0 0 = 301990656 6 (6th day of month 9, 12)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 1 0 = 301990146 7 (1st Monday of month 1, 2)	
  0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 1 0 0 0 0 0 1 0 0 = 4356      7 (2nd Tuesday, every month)	
  0 1 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 0 0 0 1 1 0 = 301990534 8 (Monday, Tuesday; week1, week3 of month 9, 12)	
  1 1 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 0 0 0 1 1 0 = 838861446 8 (Monday, Tuesday; week1, week3 of month 9, 12)	
 $$consider_planned_sched ------------------------------------------------------------------------------------------
  CONSIDER                                              inner current project   outer current project   all projects
  scheduled                                                         1                   2                   3
  scheduled  planned                                               4                   5                   6
  NONE(consider all available time)                                 7
  scheduled outer current project				                                        13                  14
  scheduled  planned outer current project				                                15                  16
  scheduled outer current project  planned all projects            20
  scheduled outer current project  planned inner current project   21
  scheduled all projects    planned outer current project          22
  planned inner current project                                     24
  search from project assignments		= 10  pool_id   = resource_id
  search create new profile				= 12  and_roles = the first role_id is used to create a new profile
  duplicate profile						= 12  pool_id   = profile_id
  Note: bits 30 - 31 (in addition to above values): Consider All Scop Element Exceptions = 1 consider, 0 don't consider
  $$eunit, applies to effort
		= 0 MINUTE (default), 
		= 1 h, 
		= 2 d, 
		= 3 w
  $$unit, applies to work
		= 0 h/d,
		= 3 h/w, 
		= 4 h/m, 
		= 1 d/w, 
		= 2 d/m, 
		= 5 w/m
		= 100 then work = percent units
#Returns           :Result Set of resource(s), Best Start date, Best Finish date and daly contour (5*480,2*0,3*400,...) 
				   :For search the number of rows depends on search criteria and is limited by max_rows parameter.
				   :For search/assign is always only one row, the resource whith has been assigned
#Errors            :The errors are returned only in search/assign
				   : TASK_ASSIGNMENT_ERROR (200062) - many reasons the assignment failed.
				   : CHILD_TASK_ASSIGNMENT_ALREADY_EXISTS (20083) - violation of RULE2
				   : CANNOT_ASSIGN_FROM_PROJECT_AFFECTED (20085) - violation of RULE3
				   : RESOURCE_NOT_ACTIVE (200019) - violation of RULE9
				   : F_DATE_EARLIER_THAN_S_DATE - finish date is earlier then start date
				   : 200220 - violation of RULE7
*/
Prompt == FUNCTION SP_SEARCH_2 ==
CREATE OR REPLACE FUNCTION SP_SEARCH_2 wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
2ee
2 :e:
1FUNCTION:
1SP_SEARCH_2:
1PRJ_ID:
1VARCHAR2:
1START_DATE001:
1FINISH_DATE001:
1EFFORT001:
1PLS_INTEGER:
1WORK001:
1BITFLAG001:
1EUNIT:
1UNIT:
1CONTOUR001:
1OCCURRENCE:
1CONSIDER_PLANNED_SCHED:
1FNAME:
1COST001:
1ORG_ID001:
1GEO_ID001:
1POOL_ID001:
1WORK_LOCATION001:
1AND_ROLES:
1AND_SKILLS:
1SHW_ROLES:
1SHW_SKILLS:
1MAX_ROWS:
1P_UPDATE:
1WORK_TYPE:
1P_TYPE:
1LAST_ID:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_WBS_ID:
132:
1H_PROJECT_ID:
1H_TASK_ID:
1H_RS_ID:
1H_UNIQUEID:
1H_REC_USER:
1H_FNAME:
1255:
1H_COST:
110:
1H_AND_ROLES:
18000:
1H_AND_SKILLS:
1H_SHW_ROLES:
1H_SHW_SKILLS:
1H_AND_ROLE:
1H_AND_SKILL:
1H_SHW_ROLE:
1H_SHW_SKILL:
1H_SOURCE_IDS:
1H_TARGET_IDS:
1H_RES_ID:
1H_LAST_ID:
1H_RANK:
1200:
1H_DAYOFYEAR:
1DATE:
1H_START_DATE:
126:
1H_START_DT:
1H_FINISH_DT:
1H_TASK_START_DT:
1H_TASK_FINISH_DT:
1H_RES_TYPE:
11:
1H_ASSIGN_ID:
1H_ROLE_ID:
1H_SKILL_ID:
1H_FULL_NAME:
1H_REC_STATUS:
1H_BITFLAG:
10:
1H_EXISTS:
1H_TYPE:
1H_COUNT:
1H_UPDATE:
1H_WORK_TYPE:
1H_WORK_LOCATION:
1H_CRI:
1H_MAX_ROWS:
1H_WORK:
1H_OCCURRENCE:
1H_EFFORT:
1H_EUNIT:
1H_UNIT:
1H_CONSIDER:
1H_CONTOUR:
1H_EVERY:
1H_PER_DAY:
1H_PER_WEEK:
1H_DURATION:
1H_BIT_MINUTES:
1H_NUMBER:
1H_PRIORITY:
1H_DAYS:
1H_RES_COST:
1NUMBER:
10.00:
1H_RATE_ECHANGE:
1H_TOTAL:
1H_POOL_TYPE:
1H_CURRENCY_ID:
1H_JULIAN_DAY:
1H_J_EFECTIVE_DATE:
1H_J_DISCONTI_DATE:
1H_WEEK:
1H_ACTIVE:
1H_TASK_TYPE:
1H_SCHED_WORK:
1H_PARTICIPANT:
1STMT:
17999:
1STWHERE:
12999:
1H_TEMP_STMT:
1H_TABLE_TYPE:
1H_TEMP_PARTICIPANT:
1H_TEMP_SECURITY_ID:
1CHAR:
1H_TEMP2_SECURITY_ID:
1H_PROFILE_ID:
1H_RESOURCE_ID:
1H_CRI_SECURITY_ID:
1H_CHAR:
1H_TEMP_VAR:
1H_TEMP_BITFLAG:
1H_TEMP_SC_STATE:
1I:
1II:
1I_ROW:
1I_RETURN:
1ISF:
1TOKEN:
1I_NOSKILL:
1NUMITEMS:
1BCREATEPROFILE:
1LEN:
1K:
1NLEN:
1NLEN001:
1NLEN002:
1C12:
1CURSOR:
1C42:
1PROJECT_ASSIGNMENT_ID:
1TASK_ASSIGNMENTS:
1PROJECT_ID:
1TASK_ID:
1RESOURCE_ID:
1SELECT PROJECT_ASSIGNMENT_ID:n         FROM TASK_ASSIGNMENTS:n         WHERE +
1PROJECT_ID = h_project_id:n         AND TASK_ID = h_project_id:n         AND +
1RESOURCE_ID = h_res_id:
1CU42:
1RS_ID:
1RES_ID:
1RTRIM:
1ROLE_ID:
1COALESCE:
1ROLE_NAME:
1SKILL_ID:
1SKILL_NAME:
1SHOW_ROLE:
1SHOW_SKILL:
1PMO_TMP_SEARCH:
1SELECT:n         RS_ID:n         , RES_ID:n         , RTRIM(ROLE_ID):n       +
1  , COALESCE(ROLE_NAME , ''):n         , RTRIM(SKILL_ID):n         , COALESCE+
1(SKILL_NAME , ''):n         , COALESCE(SHOW_ROLE , ''):n         , COALESCE(S+
1HOW_SKILL , ''):n         FROM PMO_TMP_SEARCH:n         ORDER BY RES_ID:
1C775C0:
1C775E1:
1C775E0:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_SEARCH_2(:
1':
1, :
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
1196:
1+:
1FPUT:
1               ENTER:: SP_SEARCH_2(197) :: cplan.sql(338) :: :
1PMO_CNST:
1!=:
1SC_SRCH_PARTICIPANT:
1IS NULL:
18:
1239:
1      RETURN_INVALID:: SP_SEARCH_2(240) :: cplan.sql(381) :: :
1 :
1STATE:: :
1ELEMENT_ID_REQUIRED:
1242:
1                EXIT:: SP_SEARCH_2(243) :: cplan.sql(384) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1SC_SRCH_FROM_PROJECT:
1=:
1SC_SRCH_PROFILE:
1251:
1      RETURN_INVALID:: SP_SEARCH_2(252) :: cplan.sql(393) :: :
1INVALID_PARAMS:
1254:
1                EXIT:: SP_SEARCH_2(255) :: cplan.sql(396) :: :
1ELSIF:
1259:
1      RETURN_INVALID:: SP_SEARCH_2(260) :: cplan.sql(401) :: :
1262:
1                EXIT:: SP_SEARCH_2(263) :: cplan.sql(404) :: :
15:
1282:
1      RETURN_INVALID:: SP_SEARCH_2(283) :: cplan.sql(424) :: :
1285:
1                EXIT:: SP_SEARCH_2(286) :: cplan.sql(427) :: :
1265:
1IS NOT NULL:
1PMO_ORASTD:
1DAYS:
1DUAL:
1SELECT PMO_ORASTD.DAYS(finish_date001) - PMO_ORASTD.DAYS(start_date001):n    +
1     INTO h_total:n         FROM DUAL:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
1311:
1           EXCEPTION:: SP_SEARCH_2(312) :: cplan.sql(453) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SC_SQL_SUCCESS:
1<:
1318:
1      RETURN_INVALID:: SP_SEARCH_2(319) :: cplan.sql(460) :: :
1F_DATE_EARLIER_THAN_S_DATE:
1321:
1                EXIT:: SP_SEARCH_2(322) :: cplan.sql(463) :: :
1SELECT SYSTIMESTAMP INTO h_start_date FROM DUAL:
1339:
1           EXCEPTION:: SP_SEARCH_2(340) :: cplan.sql(481) :: :
1PMO_FUNC:
1SHIFTRIGHT:
1BITAND:
1SC_C0000000:
130:
1SC_1FFFFFFF:
1>:
1LEVEL_LOCK:
1PROJECT_DETAILS:
1SELECT LEVEL_LOCK INTO h_temp_var:n         FROM PROJECT_DETAILS:n         WH+
1ERE PROJECT_ID = h_project_id:
1373:
1           EXCEPTION:: SP_SEARCH_2(374) :: cplan.sql(515) :: :
1REC_STATUS:
1TMT_WBS:
1ELEMENT_ID:
1SELECT REC_STATUS INTO h_rec_status:n            FROM TMT_WBS:n            WH+
1ERE ELEMENT_ID = h_project_id:
1391:
1           EXCEPTION:: SP_SEARCH_2(392) :: cplan.sql(533) :: :
1STRCMP:
1L:
1398:
1        RETURN_ERROR:: SP_SEARCH_2(399) :: cplan.sql(540) :: :
1LEV_LOCKED_BY_COPY:
1 ID:: :
1401:
1                EXIT:: SP_SEARCH_2(402) :: cplan.sql(543) :: :
1PMO_RETURN_ERROR_SP:
1GETTABLE:
1GETASSNTYPE:
1412:
1      RETURN_INVALID:: SP_SEARCH_2(413) :: cplan.sql(554) :: :
1415:
1                EXIT:: SP_SEARCH_2(416) :: cplan.sql(557) :: :
1SC_TABLE_WBS:
1428:
1      RETURN_INVALID:: SP_SEARCH_2(429) :: cplan.sql(570) :: :
1431:
1                EXIT:: SP_SEARCH_2(432) :: cplan.sql(573) :: :
1ASSIGNRESOURCE:
1446:
1        RETURN_ERROR:: SP_SEARCH_2(447) :: cplan.sql(588) :: :
1449:
1                EXIT:: SP_SEARCH_2(450) :: cplan.sql(591) :: :
1455:
1        RETURN_ERROR:: SP_SEARCH_2(456) :: cplan.sql(597) :: :
1458:
1                EXIT:: SP_SEARCH_2(459) :: cplan.sql(600) :: :
1ISPROJECTTYPE:
1MINUTE_PER_DAY:
1MINUTE_PER_WEEK:
1PRIORITY:
1SELECT:n               PROJECT_ID:n               , COALESCE(MINUTE_PER_DAY, +
1480):n               , COALESCE(MINUTE_PER_WEEK, 2400):n               , PRIO+
1RITY:n               INTO:n               h_project_id:n               , h_pe+
1r_day:n               , h_per_week:n               , h_priority:n            +
1   FROM TMT_WBS:n               WHERE ELEMENT_ID = h_task_id:
1486:
1           EXCEPTION:: SP_SEARCH_2(487) :: cplan.sql(628) :: :
1B:
1TASK_TYPE:
1UNRESOLVED_START:
1UNRESOLVED_FINISH:
1A:
1PLANNED_EFFORT:
1SCHED_EFFORT:
1SELECT:n               B.TASK_TYPE:n               , B.UNRESOLVED_START:n    +
1           , B.UNRESOLVED_FINISH:n               , B.PROJECT_ID:n            +
1   , COALESCE(A.MINUTE_PER_DAY, 480):n               , COALESCE(A.MINUTE_PER_+
1WEEK, 2400):n               , A.PRIORITY:n               , B.PLANNED_EFFORT ++
1 B.SCHED_EFFORT:n               INTO:n               h_task_type:n           +
1    , h_task_start_DT:n               , h_task_finish_DT:n               , h_+
1project_id:n               , h_per_day:n               , h_per_week:n        +
1       , h_priority:n               , h_sched_work:n               FROM TMT_W+
1BS A, TMT_WBS B:n               WHERE B.ELEMENT_ID = h_task_id:n             +
1  AND A.ELEMENT_ID = B.PROJECT_ID:
1521:
1           EXCEPTION:: SP_SEARCH_2(522) :: cplan.sql(663) :: :
1529:
1        RETURN_ERROR:: SP_SEARCH_2(530) :: cplan.sql(671) :: :
1RECORD_NOT_FOUND:
1532:
1                EXIT:: SP_SEARCH_2(533) :: cplan.sql(674) :: :
1SC_TABLE_CRI:
1TYPE_ID:
1WBS_ID:
1TMT_CRI:
1SELECT:n            PROJECT_ID:n            , TYPE_ID:n            , WBS_ID:n+
1            INTO:n            h_project_id:n            , h_type:n           +
1 , h_wbs_id:n            FROM TMT_CRI:n            WHERE ELEMENT_ID = h_task_+
1id:
1556:
1           EXCEPTION:: SP_SEARCH_2(557) :: cplan.sql(698) :: :
1563:
1        RETURN_ERROR:: SP_SEARCH_2(564) :: cplan.sql(705) :: :
1566:
1                EXIT:: SP_SEARCH_2(567) :: cplan.sql(708) :: :
1START_DT:
1TO_TIMESTAMP:
1SELECT:n            CASE:n            WHEN START_DT > h_start_DT:n           +
1 THEN START_DT:n            ELSE TO_TIMESTAMP(h_start_DT):n            END:n +
1           , COALESCE(MINUTE_PER_DAY, 480):n            , COALESCE(MINUTE_PER+
1_WEEK, 2400):n            , PRIORITY:n            INTO:n            h_start_D+
1T:n            , h_per_day:n            , h_per_week:n            , h_priorit+
1y:n            FROM TMT_WBS:n            WHERE PROJECT_ID = h_project_id:n   +
1         AND ELEMENT_ID = h_project_id:
1599:
1           EXCEPTION:: SP_SEARCH_2(600) :: cplan.sql(741) :: :
1606:
1        RETURN_ERROR:: SP_SEARCH_2(607) :: cplan.sql(748) :: :
1609:
1                EXIT:: SP_SEARCH_2(610) :: cplan.sql(751) :: :
1FC_MIRROR_TYPE:
1624:
1        RETURN_ERROR:: SP_SEARCH_2(625) :: cplan.sql(766) :: :
1627:
1                EXIT:: SP_SEARCH_2(628) :: cplan.sql(769) :: :
1NICK_NAME:
1JULIAN_DAY:
1ACTIVE:
1EFFECTIVE_DATE:
1DISCONTINUE_DATE:
1RESOURCES:
1SELECT:n            NICK_NAME:n            , PMO_ORASTD.JULIAN_DAY(h_start_DT+
1):n            , ACTIVE:n            , COALESCE(PMO_ORASTD.JULIAN_DAY(EFFECTI+
1VE_DATE) ,0):n            , COALESCE(PMO_ORASTD.JULIAN_DAY(DISCONTINUE_DATE) +
1,0):n            INTO:n            h_full_name:n            , h_julian_day:n +
1           , h_active:n            , h_j_efective_date:n            , h_j_dis+
1conti_date:n            FROM RESOURCES:n            WHERE RESOURCE_ID = h_res+
1_id:
1658:
1           EXCEPTION:: SP_SEARCH_2(659) :: cplan.sql(800) :: :
1SC_SQL_NO_DATA:
1665:
1        RETURN_ERROR:: SP_SEARCH_2(666) :: cplan.sql(807) :: :
1668:
1                EXIT:: SP_SEARCH_2(669) :: cplan.sql(810) :: :
1676:
1        RETURN_ERROR:: SP_SEARCH_2(677) :: cplan.sql(818) :: :
1RESOURCE_NOT_ACTIVE:
1679:
1                EXIT:: SP_SEARCH_2(680) :: cplan.sql(821) :: :
1698:
1        RETURN_ERROR:: SP_SEARCH_2(699) :: cplan.sql(840) :: :
1CANNOT_ASSIGN_COMPETENCY:
1701:
1                EXIT:: SP_SEARCH_2(702) :: cplan.sql(843) :: :
1SELECT:n            PRIORITY:n            , COALESCE(MINUTE_PER_DAY, 480):n  +
1          , COALESCE(MINUTE_PER_WEEK, 2400):n            INTO:n            h_+
1priority:n            , h_per_day:n            , h_per_week:n            FROM+
1 TMT_WBS:n            WHERE ELEMENT_ID = h_project_id:
1728:
1           EXCEPTION:: SP_SEARCH_2(729) :: cplan.sql(870) :: :
1TMT_POOLS:
1SELECT:n               480:n               , 2400:n               INTO:n     +
1          h_per_day:n               , h_per_week:n               FROM TMT_POO+
1LS WHERE ELEMENT_ID = h_project_id:
1752:
1           EXCEPTION:: SP_SEARCH_2(753) :: cplan.sql(894) :: :
1759:
1        RETURN_ERROR:: SP_SEARCH_2(760) :: cplan.sql(901) :: :
1762:
1                EXIT:: SP_SEARCH_2(763) :: cplan.sql(904) :: :
1100:
11899-12-31 00::00::00:
1:
1FC_CALC_DURATION:
1799:
1      RETURN_INVALID:: SP_SEARCH_2(800) :: cplan.sql(941) :: :
1802:
1                EXIT:: SP_SEARCH_2(803) :: cplan.sql(944) :: :
1807:
1        RETURN_ERROR:: SP_SEARCH_2(808) :: cplan.sql(949) :: :
1810:
1                EXIT:: SP_SEARCH_2(811) :: cplan.sql(952) :: :
12:
13:
14:
1SYS_GUID:
1PROFILES:
1PROFILE_ID:
1SELECT REC_STATUS INTO h_rec_status:n            FROM PROFILES:n            W+
1HERE PROFILE_ID = h_res_id:
1867:
1           EXCEPTION:: SP_SEARCH_2(868) :: cplan.sql(1009) :: :
1RES_ASSIGNMENT_ID:
1RESOURCE_ASSIGNMENTS:
1RESOURCE_TYPE:
1PARTICIPANT:
1SC_TEAMMEMBER:
1SELECT:n                  RES_ASSIGNMENT_ID:n                  INTO:n        +
1          h_uniqueID:n                  FROM RESOURCE_ASSIGNMENTS:n          +
1        WHERE ELEMENT_ID = h_task_id:n                  AND RESOURCE_ID = h_r+
1es_id:n                  AND RESOURCE_TYPE = '1':n                  AND PARTI+
1CIPANT = PMO_CNST.SC_TEAMMEMBER:
1892:
1           EXCEPTION:: SP_SEARCH_2(893) :: cplan.sql(1034) :: :
1MAX:
1PROF_NUMBER:
1SELECT:n            COALESCE(MAX(PROF_NUMBER),0)+1:n            INTO:n       +
1     h_number:n            FROM PROFILES:n            WHERE PROJECT_ID=h_proj+
1ect_id:n            AND ROLE_ID=h_role_id:
1923:
1           EXCEPTION:: SP_SEARCH_2(924) :: cplan.sql(1065) :: :
1STRINGCONCAT:
1TCP:
1ELEMENT_NAME:
1TC:
1TMT_COMPETENCIES:
1PARENT_ID:
1SELECT:n            PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORAST+
1D.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(TCP.ELEM+
1ENT_NAME):n            , ', '), RTRIM(TC.ELEMENT_NAME)), '('), RTRIM(TO_CHAR(+
1h_number))), ')'):n            INTO h_full_name:n            FROM TMT_COMPETE+
1NCIES TC, TMT_COMPETENCIES TCP:n            WHERE TC.ELEMENT_ID = h_role_id:n+
1            AND TCP.ELEMENT_ID = TC.PARENT_ID:
1944:
1           EXCEPTION:: SP_SEARCH_2(945) :: cplan.sql(1086) :: :
1951:
1        RETURN_ERROR:: SP_SEARCH_2(952) :: cplan.sql(1093) :: :
1954:
1                EXIT:: SP_SEARCH_2(955) :: cplan.sql(1096) :: :
1U:
1D:
1PROFILE_NAME:
1POOL_ID:
1GEO_ID:
1ORG_ID:
1INSERT INTO PROFILES:n            (:n            PROFILE_ID:n            , PR+
1OJECT_ID:n            , ROLE_ID:n            , PROF_NUMBER:n            , PRO+
1FILE_NAME:n            , REC_STATUS:n            , POOL_ID:n            , GEO+
1_ID:n            , ORG_ID:n            ):n            VALUES:n            (:n+
1            h_res_id:n            , h_project_id:n            , h_role_id:n  +
1          , h_number:n            , h_full_name:n            , h_char:n      +
1      , pool_id001:n            , geo_id001:n            , org_id001:n       +
1     ):
1FOUND:
11002:
1           EXCEPTION:: SP_SEARCH_2(1003) :: cplan.sql(1144) :: :
1UPDATE PROFILES SET REC_STATUS = 'U',:n            POOL_ID = pool_id001,:n   +
1         GEO_ID  = geo_id001,:n            ORG_ID  = org_id001:n            W+
1HERE PROFILE_ID = h_res_id:
11027:
1           EXCEPTION:: SP_SEARCH_2(1028) :: cplan.sql(1169) :: :
1RESOURCE_ASSIGN_SKILLS:
1DELETE FROM RESOURCE_ASSIGN_SKILLS:n         WHERE RES_ASSIGNMENT_ID IN (:n  +
1       SELECT PROFILE_ID:n         FROM PROFILES:n         WHERE PROJECT_ID =+
1 h_project_id:n         AND REC_STATUS='D':n         AND PROFILE_ID <> h_res_+
1id):
11055:
1           EXCEPTION:: SP_SEARCH_2(1056) :: cplan.sql(1197) :: :
1RESOURCE_ASSIGN_ROLES:
1DELETE FROM RESOURCE_ASSIGN_ROLES:n         WHERE RES_ASSIGNMENT_ID IN (:n   +
1      SELECT PROFILE_ID:n         FROM PROFILES:n         WHERE PROJECT_ID = +
1h_project_id:n         AND REC_STATUS='D':n         AND PROFILE_ID <> h_res_i+
1d):
11080:
1           EXCEPTION:: SP_SEARCH_2(1081) :: cplan.sql(1222) :: :
1DELETE FROM PROFILES:n         WHERE PROJECT_ID = h_project_id:n         AND +
1REC_STATUS='D':n         AND PROFILE_ID <> h_res_id:
11102:
1           EXCEPTION:: SP_SEARCH_2(1103) :: cplan.sql(1244) :: :
1FC_DECLARE_PMO_TMP_SEARCH:
1SC_MAX_ROWS:
1SELECT ':
1' AS RESOURCE_ID :
1FROM TMT_COMPETENCIES r :
1WHERE r.TYPE_ID=73 :
1      AND r.ELEMENT_ID = ':
1SELECT ra.RESOURCE_ID :
1FROM RESOURCE_ASSIGNMENTS ra :
1JOIN TASK_ASSIGNMENTS ta ON ra.ELEMENT_ID=':
1' :
1AND ra.RESOURCE_ID=':
1AND ra.RES_ASSIGNMENT_ID=ta.RES_ASSIGNMENT_ID :
1SELECT r.RESOURCE_ID :
1FROM RESOURCES r :
1WHERE r.CALENDAR_ID IS NOT NuLL :
1        AND r.RESOURCE_ID = ':
1EXECUTE:
1IMMEDIATE:
1DELETE FROM PMO_TMP_CPLN_0007:
1DELETE FROM PMO_TMP_CPLN_007a:
1PMO_TMP_CPLN_007A:
1RANK:
1PUBLIC_FLAG:
1P:
1JOIN:
1RA:
1ELEMENT_TYPE:
1INSERT INTO PMO_TMP_CPLN_007a:n               SELECT ELEMENT_ID, RANK:n      +
1         FROM TMT_POOLS:n               WHERE PUBLIC_FLAG = '1':n            +
1   UNION ALL:n               ---- All pools to which the resource is assigned+
1:n               SELECT P.ELEMENT_ID, P.RANK:n               FROM TMT_POOLS P+
1:n               JOIN RESOURCE_ASSIGNMENTS RA ON:n               P.ELEMENT_ID+
1 = RA.ELEMENT_ID:n               AND RA.ELEMENT_TYPE = 11:n               AND+
1 RA.RESOURCE_ID = h_rec_user:
11229:
1           EXCEPTION:: SP_SEARCH_2(1230) :: cplan.sql(1371) :: :
1PMO_TMP_CPLN_0007:
1M:
1LENGTH:
1INSERT INTO PMO_TMP_CPLN_0007:n               SELECT M.RANK:n               F+
1ROM PMO_TMP_CPLN_007a M:n               WHERE NOT EXISTS (:n               SE+
1LECT A.ELEMENT_ID:n               FROM PMO_TMP_CPLN_007a A:n               WH+
1ERE A.RANK < M.RANK:n               AND SUBSTR(M.RANK, 1, LENGTH(A.RANK)) = A+
1.RANK):
11257:
1           EXCEPTION:: SP_SEARCH_2(1258) :: cplan.sql(1399) :: :
132767:
165535:
1-:
165536:
1WHILE:
1LOOP:
1ASCII:
137:
195:
1EXIT:
11284:
1      RETURN_INVALID:: SP_SEARCH_2(1285) :: cplan.sql(1426) :: :
11287:
1                EXIT:: SP_SEARCH_2(1288) :: cplan.sql(1429) :: :
1REPLACE:
1SELECT REPLACE(h_fname,'''',''''''):n            INTO h_fname:n            FR+
1OM DUAL:
11304:
1           EXCEPTION:: SP_SEARCH_2(1305) :: cplan.sql(1446) :: :
1SELECT p.PROFILE_ID AS RESOURCE_ID FROM PROFILES p JOIN TMT_COMPETENCIES r ON+
1 p.ROLE_ID = r.ELEMENT_ID:
1 WHERE p.PROFILE_ID=':
1SELECT r.RESOURCE_ID FROM RESOURCES r JOIN RESOURCE_ASSIGNMENTS ra ON BITAND(+
1ra.PARTICIPANT,4096)>0 AND ra.ELEMENT_ID=':
1' AND ra.RESOURCE_ID=r.RESOURCE_ID:
1 WHERE 0=0 :
1SELECT DISTINCT r.RESOURCE_ID FROM RESOURCES r:
1 AND r.RESOURCE_ID > ':
1 AND r.CALENDAR_ID IS NOT NuLL :
1AND UPPER(r.FULL_NAME) like UPPER(':
1%'):
1SELECT r.RESOURCE_ID FROM RESOURCES r:
1SELECT DISTINCT r.RESOURCE_ID :
1FROM TMT_POOLS TP, PMO_TMP_CPLN_0007 MP, RESOURCES r:
1AND TP.ELEMENT_ID=R.RESOURCE_ID :
1  AND (MP.RANK >= SUBSTR(TP.RANK, 1, 8) :
1        AND MP.RANK <= TP.RANK :
1        AND length(MP.RANK) <= length(TP.RANK) :
1        AND substr(TP.RANK, 1, length(MP.RANK)) = MP.RANK) :
1  AND r.CALENDAR_ID IS NOT NuLL :
1  AND UPPER(r.FULL_NAME) LIKE UPPER(':
1SELECT RANK INTO h_rank:n               FROM TMT_POOLS:n               WHERE +
1ELEMENT_ID = pool_id001:
11389:
1           EXCEPTION:: SP_SEARCH_2(1390) :: cplan.sql(1531) :: :
1,TMT_POOLS p:
1 AND r.RESOURCE_ID = p.ELEMENT_ID :
1AND p.RANK LIKE ':
1%' :
11403:
1        RETURN_ERROR:: SP_SEARCH_2(1404) :: cplan.sql(1545) :: :
11406:
1                EXIT:: SP_SEARCH_2(1407) :: cplan.sql(1548) :: :
1TMT_ORGANIZATIONS:
1SELECT RANK INTO h_rank:n               FROM TMT_ORGANIZATIONS:n             +
1  WHERE ELEMENT_ID = org_id001:
11425:
1           EXCEPTION:: SP_SEARCH_2(1426) :: cplan.sql(1567) :: :
1,TMT_ORGANIZATIONS o:
1 AND r.RESOURCE_ID=o.ELEMENT_ID :
1AND o.RANK LIKE ':
11436:
1        RETURN_ERROR:: SP_SEARCH_2(1437) :: cplan.sql(1578) :: :
11439:
1                EXIT:: SP_SEARCH_2(1440) :: cplan.sql(1581) :: :
1TMT_GEOGRAPHIES:
1SELECT RANK INTO h_rank:n               FROM TMT_GEOGRAPHIES:n               +
1WHERE ELEMENT_ID = geo_id001:
11458:
1           EXCEPTION:: SP_SEARCH_2(1459) :: cplan.sql(1600) :: :
1,TMT_GEOGRAPHIES g:
1 AND r.RESOURCE_ID=g.ELEMENT_ID :
1AND g.RANK LIKE ':
11469:
1        RETURN_ERROR:: SP_SEARCH_2(1470) :: cplan.sql(1611) :: :
11472:
1                EXIT:: SP_SEARCH_2(1473) :: cplan.sql(1614) :: :
1 AND r.WORK_LOCATION=:
1PMO_STPU:
1STRNPCPY:
1SELECT RANK INTO h_rank:n               FROM TMT_COMPETENCIES:n              +
1 WHERE ELEMENT_ID = h_role_id:
11504:
1           EXCEPTION:: SP_SEARCH_2(1505) :: cplan.sql(1646) :: :
11511:
1        RETURN_ERROR:: SP_SEARCH_2(1512) :: cplan.sql(1653) :: :
11514:
1                EXIT:: SP_SEARCH_2(1515) :: cplan.sql(1656) :: :
1 AND EXISTS :
1(SELECT t1.ELEMENT_ID :
1  FROM TMT_COMPETENCIES t1:
1  WHERE t1.RANK LIKE ':
1%':
1AND r.ELEMENT_ID=t1.ELEMENT_ID) :
1AND r.RESOURCE_ID=t1.RESOURCE_ID) :
11559:
1           EXCEPTION:: SP_SEARCH_2(1560) :: cplan.sql(1701) :: :
11566:
1        RETURN_ERROR:: SP_SEARCH_2(1567) :: cplan.sql(1708) :: :
11569:
1                EXIT:: SP_SEARCH_2(1570) :: cplan.sql(1711) :: :
1     AND r.RESOURCE_ID=t1.RESOURCE_ID) :
1 ORDER BY RESOURCE_ID  :
1OPEN:
1CLOSE:
11633:
1           EXCEPTION:: SP_SEARCH_2(1634) :: cplan.sql(1775) :: :
1CONTINUE0000:
11653:
1           EXCEPTION:: SP_SEARCH_2(1654) :: cplan.sql(1795) :: :
1COUNT:
1TSK_ASSIGNMENT_ID:
1SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_temp_var :n                  FROM TASK+
1_ASSIGNMENTS:n                  WHERE PROJECT_ID = h_project_id:n            +
1      AND RESOURCE_ID = h_res_id:n                  AND RESOURCE_TYPE = '0':
11684:
1           EXCEPTION:: SP_SEARCH_2(1685) :: cplan.sql(1826) :: :
11700:
1           EXCEPTION:: SP_SEARCH_2(1701) :: cplan.sql(1842) :: :
11706:
1        RETURN_ERROR:: SP_SEARCH_2(1707) :: cplan.sql(1848) :: :
1CHILD_TASK_ASSIGNMENT_ALREADY_:
11709:
1                EXIT:: SP_SEARCH_2(1710) :: cplan.sql(1851) :: :
1FC_SEARCH_2:
11735:
1           EXCEPTION:: SP_SEARCH_2(1736) :: cplan.sql(1877) :: :
1DB_OFFSET:
1TASK_ASSIGNMENT_ERROR:
11745:
1        RETURN_ERROR:: SP_SEARCH_2(1746) :: cplan.sql(1887) :: :
11748:
1                EXIT:: SP_SEARCH_2(1749) :: cplan.sql(1890) :: :
1UPDATE PROFILES SET REC_STATUS='U' WHERE PROFILE_ID = h_res_id AND REC_STATUS+
1='D':
11766:
1           EXCEPTION:: SP_SEARCH_2(1767) :: cplan.sql(1908) :: :
11808:
1           EXCEPTION:: SP_SEARCH_2(1809) :: cplan.sql(1950) :: :
1ASSIGN_STRING_TOLONG:
11817:
1        RETURN_ERROR:: SP_SEARCH_2(1818) :: cplan.sql(1959) :: :
11820:
1                EXIT:: SP_SEARCH_2(1821) :: cplan.sql(1962) :: :
1NOTNULL:
1UPDATE PMO_TMP_SEARCH SET:n                  ROLE_ID = h_source_ids:n        +
1          , ROLE_NAME = h_and_roles:n                  , SKILL_ID = h_target_+
1ids:n                  , SKILL_NAME = h_and_skills:n                  , SHOW_+
1ROLE = h_shw_roles:n                  , SHOW_SKILL = h_shw_skills:n          +
1        WHERE RS_ID = h_last_id:
11888:
1           EXCEPTION:: SP_SEARCH_2(1889) :: cplan.sql(2030) :: :
1UPDATE PMO_TMP_SEARCH SET:n               ROLE_ID = NuLL:n               , RO+
1LE_NAME = NuLL:n               , SKILL_ID = NuLL:n               , SKILL_NAME+
1 = NuLL:n               , SHOW_ROLE = NuLL:n               , SHOW_SKILL = NuL+
1L:n               WHERE RS_ID = h_last_id:
11931:
1           EXCEPTION:: SP_SEARCH_2(1932) :: cplan.sql(2073) :: :
1,:
11992:
1           EXCEPTION:: SP_SEARCH_2(1993) :: cplan.sql(2134) :: :
1UPDATE PMO_TMP_SEARCH SET:n            ROLE_ID = h_source_ids:n            , +
1ROLE_NAME = h_and_roles:n            , SKILL_ID = h_target_ids:n            ,+
1 SKILL_NAME = h_and_skills:n            , SHOW_ROLE = h_shw_roles:n          +
1  , SHOW_SKILL = h_shw_skills:n            WHERE RS_ID = h_last_id:
12020:
1           EXCEPTION:: SP_SEARCH_2(2021) :: cplan.sql(2162) :: :
1SUM:
1AVAILABLE_MIN:
1CALENDAR_DAY_OF_YEAR:
1DAY_YEAR:
1TO_DATE:
1SELECT COALESCE(SUM(AVAILABLE_MIN),0) INTO h_effort:n            FROM CALENDA+
1R_DAY_OF_YEAR:n            WHERE ELEMENT_ID=h_project_id:n            AND DAY+
1_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_+
1DT, 1, 10)):
12042:
1           EXCEPTION:: SP_SEARCH_2(2043) :: cplan.sql(2184) :: :
1CURRENCY_ID:
1TODECIMAL:
1TD:
1TW:
1TMT_DATAFIELDS:
1SELECT CURRENCY_ID, PMO_ORASTD.TODECIMAL(td.ELEMENT_NAME,20,4) INTO h_currenc+
1y_id, h_rate_echange:n      FROM TMT_WBS tw JOIN TMT_DATAFIELDS td on tw.CURR+
1ENCY_ID = td.ELEMENT_ID:n      WHERE tw.ELEMENT_ID = h_project_id:
12064:
1           EXCEPTION:: SP_SEARCH_2(2065) :: cplan.sql(2206) :: :
1DELETE FROM PMO_TMP_CPLN_0005:
1PMO_TMP_CPLN_0005:
1REGULAR_COST:
1REGULAR_SALE:
1SPECIAL_COST:
1SPECIAL_SALE:
1INSERT INTO PMO_TMP_CPLN_0005:n         SELECT RES_ID:n         , REGULAR_COS+
1T:n         , REGULAR_SALE:n         , SPECIAL_COST:n         , SPECIAL_SALE+
1:n         FROM PMO_TMP_SEARCH:n         WHERE RS_ID = h_rs_id:n         AND +
1(REGULAR_COST + REGULAR_SALE) > 0:
12102:
1           EXCEPTION:: SP_SEARCH_2(2103) :: cplan.sql(2244) :: :
1INSERT INTO PMO_TMP_CPLN_0005:n            SELECT RES_ID:n            , MAX(R+
1EGULAR_COST) AS REGULAR_COST:n            , MAX(REGULAR_SALE) AS REGULAR_SALE+
1:n            , MAX(SPECIAL_COST) AS SPECIAL_COST:n            , MAX(SPECIAL_+
1SALE) AS SPECIAL_SALE:n            FROM PMO_TMP_SEARCH:n            WHERE RS_+
1ID <> h_rs_id:n            GROUP BY RES_ID:
12131:
1           EXCEPTION:: SP_SEARCH_2(2132) :: cplan.sql(2273) :: :
1INSERT INTO PMO_TMP_CPLN_0005:n               SELECT RES_ID, 0.00, 0.00, 0.00+
1, 0.00:n               FROM PMO_TMP_SEARCH:n               WHERE RS_ID = h_rs+
1_id:n               GROUP BY RES_ID:
12155:
1           EXCEPTION:: SP_SEARCH_2(2156) :: cplan.sql(2297) :: :
1RS:
1WORK_LOCATION:
1O1:
1ORGANIZATION:
1G1:
1GEOGRAPHY:
1ROUND:
1PA:
1RES_SALE_RATE:
1TEMP:
1REGULAR_RATE:
1SPECIAL_RATE:
1START_DATE:
1FINISH_DATE:
1ASSIGNABLE:
1PLANNED:
1SCHED:
1PROPOSED:
1WORK_EFFORT:
1UNIT_EFFORT:
1AVAILABLE:
1VERIFIED:
1STATUS2_ID:
1FULL_NAME:
1PP:
1POOL_NAME:
1E_MAIL_ADDRESS:
1UNIQUE_ID:
1CONTOUR:
1ASSIGN_STRING:
1ASSIGNED_FLAG:
1FS_LABOR_CODE:
1RES_MODIFIED_DATE:
1LEFT:
1OUTER:
1PROJECT_ASSIGNMENTS:
1R:
1RATES:
1RATE_ID:
1SELECT h_res_id AS ELEMENT_ID, 41 AS TYPE_ID, p.PROFILE_NAME AS ELEMENT_NAME,+
1 rs.ROLE_NAME, rs.SKILL_NAME, rs.SHOW_ROLE, rs.SHOW_SKILL, h_work_location AS+
1 WORK_LOCATION,:n      o1.ELEMENT_NAME AS ORGANIZATION,g1.ELEMENT_NAME AS GEO+
1GRAPHY,:n      ROUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.REGULAR_COST WHEN rs+
1.RES_SALE_RATE=0 THEN temp.REGULAR_COST * h_rate_echange ELSE a.REGULAR_COST +
1* h_rate_echange END,2) AS REGULAR_COST,:n      ROUND(CASE WHEN pa.REGULAR_RA+
1TE>0 THEN pa.REGULAR_RATE WHEN rs.RES_SALE_RATE=0 THEN temp.REGULAR_SALE * h_+
1rate_echange ELSE a.REGULAR_SALE * h_rate_echange END,2) AS REGULAR_SALE,:n  +
1    ROUND(CASE WHEN pa.SPECIAL_COST>0 THEN pa.SPECIAL_COST WHEN rs.RES_SALE_R+
1ATE=0 THEN temp.SPECIAL_COST * h_rate_echange ELSE a.SPECIAL_COST * h_rate_ec+
1hange END,2) AS SPECIAL_COST,:n      ROUND(CASE WHEN pa.SPECIAL_RATE>0 THEN p+
1a.SPECIAL_RATE WHEN rs.RES_SALE_RATE=0 THEN temp.SPECIAL_SALE * h_rate_echang+
1e ELSE a.SPECIAL_SALE * h_rate_echange END,2) AS SPECIAL_SALE,:n      rs.STAR+
1T_DATE,rs.FINISH_DATE,rs.ASSIGNABLE,rs.PLANNED,rs.SCHED, CASE WHEN rs.PROPOSE+
1D > 0 THEN rs.PROPOSED ELSE h_effort END AS WORK_EFFORT, h_eunit AS UNIT_EFFO+
1RT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS AVAILABLE,:n      '' AS VERIFIE+
1D, 0 AS STATUS2_ID, '' AS FULL_NAME, pp.ELEMENT_NAME AS POOL_NAME, '' AS E_MA+
1IL_ADDRESS, '' AS UNIQUE_ID, h_per_day AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIG+
1N_STRING, h_and_roles AS ROLE_ID, h_and_skills AS SKILL_ID, 0 AS ASSIGNED_FLA+
1G,:n      'PMOFFICE' AS FS_LABOR_CODE, TO_TIMESTAMP('1899-12-31 00::00::00') +
1AS RES_MODIFIED_DATE:n      FROM PMO_TMP_CPLN_0005 temp JOIN PMO_TMP_SEARCH r+
1s ON rs.RES_ID = temp.RES_ID:n      LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON+
1 pa.PROJECT_ID=h_project_id AND pa.RESOURCE_ID=rs.RES_ID:n      JOIN PROFILES+
1 p ON p.PROFILE_ID = h_res_id:n      JOIN TMT_COMPETENCIES r ON p.ROLE_ID=r.E+
1LEMENT_ID:n      LEFT OUTER JOIN RATES a ON r.RATE_ID=a.RATE_ID:n      LEFT O+
1UTER JOIN TMT_ORGANIZATIONS o1 ON p.ORG_ID = o1.ELEMENT_ID:n      LEFT OUTER +
1JOIN TMT_GEOGRAPHIES g1 ON p.GEO_ID = g1.ELEMENT_ID:n      LEFT OUTER JOIN TM+
1T_POOLS pp ON pp.ELEMENT_ID = p.POOL_ID:n      WHERE NOT (rs.ROLE_NAME IS NuL+
1L AND rs.SKILL_NAME IS NuLL AND rs.SHOW_ROLE IS NuLL AND rs.SHOW_SKILL IS NuL+
1L):
12181:
1                EXIT:: SP_SEARCH_2(2182) :: cplan.sql(2323) :: :
1O2:
1G2:
1DEFAULT_FLAG:
1RV:
1RM:
1PLANNED_WORK:
1SCHED_WORK:
1REC_DATETIME:
1MANAGER_ID:
1VERIFIED_1:
1SELECT rs.RES_ID AS ELEMENT_ID, 13 AS TYPE_ID, r.FULL_NAME AS ELEMENT_NAME, r+
1s.ROLE_NAME, rs.SKILL_NAME, rs.SHOW_ROLE, rs.SHOW_SKILL, r.WORK_LOCATION,:n  +
1       o2.ELEMENT_NAME AS ORGANIZATION,g2.ELEMENT_NAME AS GEOGRAPHY,:n       +
1  ROUND(CASE WHEN rs.DEFAULT_FLAG='0' THEN rs.REGULAR_COST * CASE WHEN (r.CUR+
1RENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE +
1(SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELE+
1MENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FR+
1OM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END WHEN rs.DEFAULT_FLAG=+
1'1' THEN rs.REGULAR_COST ELSE rs.REGULAR_COST * h_rate_echange END,2) AS REGU+
1LAR_COST,:n         ROUND(CASE WHEN rs.DEFAULT_FLAG='0' THEN rs.REGULAR_SALE +
1* CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id+
1 = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DA+
1TAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELE+
1MENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END WHE+
1N rs.DEFAULT_FLAG='1' THEN rs.REGULAR_SALE ELSE rs.REGULAR_SALE * h_rate_echa+
1nge END,2) AS REGULAR_SALE,:n         ROUND(CASE WHEN rs.DEFAULT_FLAG='0' THE+
1N rs.SPECIAL_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_+
1ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME+
1,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_OR+
1ASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CU+
1RRENCY_ID) END WHEN rs.DEFAULT_FLAG='1' THEN rs.SPECIAL_COST ELSE rs.SPECIAL_+
1COST * h_rate_echange END,2) AS SPECIAL_COST,:n         ROUND(CASE WHEN rs.DE+
1FAULT_FLAG='0' THEN rs.SPECIAL_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_i+
1d) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODE+
1CIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id+
1) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE+
1 ELEMENT_ID = r.CURRENCY_ID) END WHEN rs.DEFAULT_FLAG='1' THEN rs.SPECIAL_SAL+
1E ELSE rs.SPECIAL_SALE * h_rate_echange END,2) AS SPECIAL_SALE,:n         rs.+
1START_DATE, rs.FINISH_DATE,rs.ASSIGNABLE,rs.PLANNED,rs.SCHED, CASE WHEN rs.PR+
1OPOSED > 0 THEN rs.PROPOSED ELSE h_effort END AS WORK_EFFORT, h_eunit AS UNIT+
1_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS AVAILABLE,:n         rv.FU+
1LL_NAME AS VERIFIED, r.STATUS2_ID, pp.ELEMENT_NAME AS POOL_NAME, RM.FULL_NAME+
1, RM.E_MAIL_ADDRESS, '' AS UNIQUE_ID, h_per_day AS MINUTE_PER_DAY, rs.CONTOUR+
1 AS ASSIGN_STRING, rs.ROLE_ID, rs.SKILL_ID, COALESCE(pa.PLANNED_WORK + pa.SCH+
1ED_WORK,0) AS ASSIGNED_FLAG,:n         r.FS_LABOR_CODE, r.REC_DATETIME AS RES+
1_MODIFIED_DATE:n         FROM PMO_TMP_SEARCH rs JOIN TMT_POOLS p ON rs.RES_ID+
1=p.ELEMENT_ID:n         JOIN TMT_POOLS pp ON pp.ELEMENT_ID=p.PARENT_ID:n     +
1    LEFT OUTER JOIN RESOURCES RM ON pp.MANAGER_ID=RM.RESOURCE_ID:n         JO+
1IN RESOURCES r ON rs.RES_ID=r.RESOURCE_ID LEFT OUTER JOIN RESOURCES rv ON rv.+
1RESOURCE_ID=r.VERIFIED_1:n         LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON +
1pa.PROJECT_ID=h_project_id AND pa.RESOURCE_ID=rs.RES_ID:n         LEFT OUTER +
1JOIN TMT_ORGANIZATIONS o1 ON r.RESOURCE_ID=o1.ELEMENT_ID:n         LEFT OUTER+
1 JOIN TMT_ORGANIZATIONS o2 ON o1.PARENT_ID=o2.ELEMENT_ID:n         LEFT OUTER+
1 JOIN TMT_GEOGRAPHIES g1 ON r.RESOURCE_ID=g1.ELEMENT_ID:n         LEFT OUTER +
1JOIN TMT_GEOGRAPHIES g2 ON g1.PARENT_ID=g2.ELEMENT_ID:n         ORDER BY ELEM+
1ENT_ID:
12206:
1                EXIT:: SP_SEARCH_2(2207) :: cplan.sql(2348) :: :
1DELETE FROM PMO_TMP_CPLN_0006:
1PMO_TMP_CPLN_0006:
1INSERT INTO PMO_TMP_CPLN_0006:n            SELECT RES_ID, REGULAR_COST, REGUL+
1AR_SALE, SPECIAL_COST, SPECIAL_SALE, 1:n            FROM PMO_TMP_SEARCH WHERE+
1 DEFAULT_FLAG = '1':
12230:
1           EXCEPTION:: SP_SEARCH_2(2231) :: cplan.sql(2372) :: :
1INSERT INTO PMO_TMP_CPLN_0006:n            SELECT  RES_ID,                   +
1          MAX(REGULAR_COST) AS REGULAR_COST,:n            MAX(REGULAR_SALE) A+
1S REGULAR_SALE,  MAX(SPECIAL_COST) AS SPECIAL_COST,:n            MAX(SPECIAL_+
1SALE) AS SPECIAL_SALE , 0:n            FROM PMO_TMP_SEARCH:n            WHERE+
1 RES_ID NOT IN (SELECT RES_ID:n            FROM PMO_TMP_SEARCH WHERE DEFAULT_+
1FLAG = '1'):n            GROUP BY RES_ID:
12256:
1           EXCEPTION:: SP_SEARCH_2(2257) :: cplan.sql(2398) :: :
1DEF:
1PRF:
1RR:
1SELECT rs.RES_ID AS ELEMENT_ID, 13 AS TYPE_ID, r.FULL_NAME AS ELEMENT_NAME, r+
1s.ROLE_NAME, rs.SKILL_NAME, rs.SHOW_ROLE, rs.SHOW_SKILL, r.WORK_LOCATION,:n  +
1       o2.ELEMENT_NAME AS ORGANIZATION,g2.ELEMENT_NAME AS GEOGRAPHY,:n       +
1  :n         ROUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.REGULAR_COST:n        +
1 WHEN a.REGULAR_COST>0 THEN a.REGULAR_COST * CASE WHEN (r.CURRENCY_ID = h_cur+
1rency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORAS+
1TD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_curr+
1ency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELD+
1S WHERE ELEMENT_ID = r.CURRENCY_ID) END:n         WHEN def.REGULAR_COST>0 THE+
1N def.REGULAR_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY+
1_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAM+
1E,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_O+
1RASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.C+
1URRENCY_ID) END:n         else prf.REGULAR_COST * h_rate_echange:n         EN+
1D,2) AS REGULAR_COST,:n         :n         ROUND(CASE WHEN pa.REGULAR_COST> 0+
1 THEN pa.REGULAR_RATE:n         WHEN a.REGULAR_COST > 0 THEN a.REGULAR_SALE *+
1 CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id +
1= 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DAT+
1AFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEM+
1ENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END:n   +
1      WHEN def.REGULAR_COST>0 THEN def.REGULAR_SALE * CASE WHEN (r.CURRENCY_I+
1D = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT+
1 PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID+
1 = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_+
1DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END:n         else prf.REGULAR_S+
1ALE * h_rate_echange:n         END,2) AS REGULAR_SALE,:n         :n         R+
1OUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.SPECIAL_COST:n         WHEN a.REGULA+
1R_COST>0 THEN a.SPECIAL_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (+
1r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(E+
1LEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SE+
1LECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMEN+
1T_ID = r.CURRENCY_ID) END:n         WHEN def.REGULAR_COST>0 THEN def.SPECIAL_+
1COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_curren+
1cy_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM T+
1MT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMA+
1L(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) EN+
1D:n         else prf.SPECIAL_COST * h_rate_echange:n         END,2) AS SPECIA+
1L_COST,:n         :n         ROUND(CASE WHEN pa.REGULAR_RATE>0 THEN pa.SPECIA+
1L_RATE:n         WHEN a.REGULAR_SALE>0 THEN a.SPECIAL_SALE * CASE WHEN (r.CUR+
1RENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE +
1(SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELE+
1MENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FR+
1OM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END:n         WHEN def.RE+
1GULAR_COST>0 THEN def.SPECIAL_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_id+
1) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODEC+
1IMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id)+
1 / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE +
1ELEMENT_ID = r.CURRENCY_ID) END:n         else prf.SPECIAL_SALE * h_rate_echa+
1nge:n         END,2) AS SPECIAL_SALE,:n         :n         rs.START_DATE,rs.F+
1INISH_DATE,rs.ASSIGNABLE,rs.PLANNED,rs.SCHED, CASE WHEN rs.PROPOSED > 0 THEN +
1rs.PROPOSED ELSE h_effort END AS WORK_EFFORT, h_eunit AS UNIT_EFFORT,(rs.ASSI+
1GNABLE - rs.PLANNED - rs.SCHED) AS AVAILABLE,:n         rv.FULL_NAME AS VERIF+
1IED, r.STATUS2_ID, pp.ELEMENT_NAME AS POOL_NAME, RM.FULL_NAME, RM.E_MAIL_ADDR+
1ESS, '' AS UNIQUE_ID, h_per_day AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRIN+
1G,:n         rs.ROLE_ID, rs.SKILL_ID, COALESCE(pa.PLANNED_WORK + pa.SCHED_WOR+
1K,0) AS ASSIGNED_FLAG,:n         r.FS_LABOR_CODE, r.REC_DATETIME AS RES_MODIF+
1IED_DATE:n         FROM PMO_TMP_SEARCH rs:n         LEFT JOIN PMO_TMP_CPLN_00+
106 def ON rs.RES_ID=def.RES_ID AND def.DEFAULT_FLAG = 1:n         LEFT JOIN P+
1MO_TMP_CPLN_0006 prf ON rs.RES_ID=prf.RES_ID AND prf.DEFAULT_FLAG = 0:n      +
1   JOIN TMT_POOLS p ON rs.RES_ID=p.ELEMENT_ID:n         JOIN TMT_POOLS pp ON +
1pp.ELEMENT_ID=p.PARENT_ID:n         LEFT OUTER JOIN RESOURCES RM ON pp.MANAGE+
1R_ID=RM.RESOURCE_ID:n         JOIN RESOURCES r ON rs.RES_ID=r.RESOURCE_ID LEF+
1T OUTER JOIN RESOURCES rv ON rv.RESOURCE_ID=r.VERIFIED_1:n         LEFT OUTER+
1 JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=h_project_id AND pa.RESOURCE_ID+
1=rs.RES_ID:n         LEFT OUTER JOIN RESOURCES rr ON r.VERIFIED_1=rr.RESOURCE+
1_ID:n         LEFT OUTER JOIN TMT_ORGANIZATIONS o1 ON r.RESOURCE_ID=o1.ELEMEN+
1T_ID LEFT OUTER JOIN TMT_ORGANIZATIONS o2 ON o1.PARENT_ID=o2.ELEMENT_ID:n    +
1     LEFT OUTER JOIN TMT_GEOGRAPHIES g1 ON r.RESOURCE_ID=g1.ELEMENT_ID LEFT O+
1UTER JOIN TMT_GEOGRAPHIES g2 ON g1.PARENT_ID=g2.ELEMENT_ID:n         LEFT OUT+
1ER JOIN RATES a ON r.RATE_ID=a.RATE_ID:n         WHERE NOT (rs.ROLE_NAME IS N+
1uLL AND rs.SKILL_NAME IS NuLL AND rs.SHOW_ROLE IS NuLL AND rs.SHOW_SKILL IS N+
1uLL):n         ORDER BY ELEMENT_ID:
12307:
1                EXIT:: SP_SEARCH_2(2308) :: cplan.sql(2449) :: :
1RETURN_ERROR_EXCEPTION:
12317:
1           EXCEPTION:: SP_SEARCH_2(2318) :: cplan.sql(2459) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1ERROR:
1ID:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
12329:
1           EXCEPTION:: SP_SEARCH_2(2330) :: cplan.sql(2471) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
366b
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 81 b0 a3 a0 1c
81 b0 a3 a0 1c 81 b0 a3
a0 1c 81 b0 a3 a0 1c 81
b0 a3 a0 1c 81 b0 a3 a0
1c 81 b0 a3 :2 a0 6b 1c 81
b0 a0 f4 b4 bf c8 :8 a0 12a
bd b7 11 a4 b1 a0 f4 b4
bf c8 :10 a0 12a bd b7 11 a4
b1 a3 :2 a0 6b 1c 81 b0 a3
:2 a0 6b 1c 81 b0 a3 :2 a0 6b
1c 81 b0 a3 :2 a0 6b 1c 81
b0 :2 a0 6b :2 a0 6b 6e a5 b
a 10 :2 a0 6b 6e :2 a0 6b 6e
a5 b 6e 7e a0 b4 2e 6e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e :3 a0 a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 d :2 a0 d :2 a0 d :2 a0
d :2 a0 d :2 a0 d :2 a0 d :2 a0
d :2 a0 d :2 a0 d :2 a0 d :2 a0
d :2 a0 d :2 a0 d :2 a0 d :2 a0
d :2 a0 d :2 a0 d :2 a0 d :2 a0
d :2 a0 d :2 a0 d a0 51 d
a0 51 d :2 a0 51 a5 b a0
7e a0 6b b4 2e 5a a0 7e
b4 2e :2 5a a 10 :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b a0 a5 57 b7 19 3c a0
7e b4 2e :2 5a a0 7e b4 2e
:2 5a a 10 a0 7e b4 2e :2 5a
a 10 a0 7e b4 2e :2 5a a
10 :2 a0 51 a5 b a0 7e a0
6b b4 2e 5a a 10 a0 7e
b4 2e :2 5a :2 a0 51 a5 b a0
7e a0 6b b4 2e 5a a 10
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b a0 a5 57 a0
b7 a0 7e b4 2e :2 5a a0 7e
b4 2e :2 5a a 10 :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b a0 a5 57 b7 :2 19 3c b7
19 3c :2 a0 51 a5 b 7e 51
b4 2e :2 a0 51 a5 b 7e 51
b4 2e a 10 :2 a0 51 a5 b
7e 51 b4 2e a 10 :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b a0 a5 57 b7 19 3c
a0 7e b4 2e :2 5a a0 51 d
b7 :2 a0 d b7 :2 19 3c :2 a0 51
a5 b a0 7e a0 6b b4 2e
5a a0 7e b4 2e :2 5a a 10
a0 7e b4 2e :2 5a a 10 :8 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :2 a0 51 a5 b 7e 51
b4 2e 5a a 10 :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b a0 a5 57 b7 19 3c a0
51 d b7 19 3c a0 7e b4
2e :2 5a a0 7e b4 2e :2 5a a
10 :3 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 19 3c :2 a0 d
:3 a0 6b :3 a0 51 a5 b :2 a0 6b
a5 b 5a 51 a5 b d :4 a0
51 a5 b :2 a0 6b a5 b d
:3 a0 6b :3 a0 51 a5 b :2 a0 6b
a5 b 5a 51 a5 b d :4 a0
51 a5 b :2 a0 6b a5 b d
:3 a0 51 a5 b a0 7e a0 6b
b4 2e 51 a6 122 51 122 124
d :2 a0 51 a5 b 7e 51 b4
2e a0 51 d :5 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 51
a5 b 7e 51 b4 2e :5 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 6b a0 6e a5 b 7e 51
b4 2e :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 19 3c b7 19 3c :4 a0
51 a5 b a5 b d :2 a0 51
a5 b 7e 51 b4 2e :4 a0 51
a5 b a5 b d :2 a0 51 a5
b 7e 51 b4 2e :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b a0 a5 57 b7 19 3c :3 a0
6b d b7 19 3c :2 a0 d :2 a0
51 a5 b 7e 51 b4 2e :2 a0
d b7 :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b a0 a5
57 b7 :2 19 3c :8 a0 a5 b d
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 51 a5 b :2 a0 a5 57
b7 19 3c :2 a0 51 a5 b a0
7e a0 6b b4 2e :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c :2 a0
51 a5 b a0 7e a0 6b b4
2e :3 a0 51 a5 b a5 b 7e
51 b4 2e :d a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :27 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c :2 a0 6b a0 7e a0
6b b4 2e 5a :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 19 3c a0 b7
:2 a0 51 a5 b a0 7e a0 6b
b4 2e :9 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c :13 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 19 3c a0 7e b4 2e
5a :3 a0 6b d :3 a0 a5 b d
b7 19 3c b7 19 :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 :2 19 3c :2 a0
51 a5 b a0 7e a0 6b b4
2e :15 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 a0 b7 :2 a0 6b a0
7e a0 6b b4 2e 5a :2 a0 :2 51
a5 b 7e 6e b4 2e :2 a0 51
a5 b 7e 51 b4 2e :2 a0 51
a5 b a0 7e a0 51 a5 b
b4 2e a 10 5a 52 10 :2 a0
51 a5 b a0 7e a0 51 a5
b b4 2e 52 10 :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c b7
:2 19 3c b7 19 3c b7 19 3c
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 51 a5 b a0 7e a0 6b
b4 2e a0 7e b4 2e 5a a
10 :2 a0 51 a5 b 7e 51 b4
2e a 10 :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b :2 a0
a5 57 b7 19 3c :2 a0 51 a5
b a0 7e a0 6b b4 2e :b a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :5 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c b7
19 3c b7 19 3c b7 19 3c
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 51 a5 b 7e 51 b4 2e
a 10 a0 51 d b7 19 3c
:2 a0 51 a5 b a0 7e a0 6b
b4 2e a0 7e b4 2e 5a a0
6e d a0 6e d b7 :2 a0 d
:2 a0 d b7 :2 19 3c a0 51 d
b7 a0 7e 51 b4 2e a0 6e
d :6 a0 51 a5 b :2 a0 51 a5
b :2 a0 51 a5 b :2 a0 51 a5
b :2 a0 51 a5 b :2 a0 51 a5
b :2 a0 51 a5 b :2 a0 51 a5
b :3 a0 a5 b d :2 a0 51 a5
b a0 7e a0 6b b4 2e :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b a0 a5 57 a0 b7
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 51 a5 b :2 a0 a5 57
b7 :2 19 3c b7 :2 a0 d b7 :2 19
3c :2 a0 51 a5 b 7e 51 b4
2e 5a :2 a0 51 a5 b 7e 51
b4 2e 5a a 10 :2 a0 51 a5
b 7e 51 b4 2e 5a :2 a0 51
a5 b 7e 51 b4 2e 5a 52
10 a0 51 d a0 b7 :2 a0 51
a5 b 7e 51 b4 2e a0 51
d a0 b7 19 :2 a0 51 a5 b
7e 51 b4 2e 5a :2 a0 51 a5
b 7e 51 b4 2e 5a 52 10
a0 51 d a0 b7 19 :2 a0 51
a5 b 7e 51 b4 2e a0 51
d b7 :2 19 3c b7 19 3c b7
:2 19 3c :2 a0 51 a5 b a0 7e
a0 6b b4 2e a0 51 d :2 a0
d :2 a0 d :3 a0 :2 51 a5 b d
a0 4d d :2 a0 51 a5 b 7e
51 b4 2e :2 a0 b4 2e d a0
51 d b7 :5 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b a0
7e a0 6b b4 2e 5a :2 a0 51
a5 b 7e 51 b4 2e :b a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 6b a0 7e a0 6b b4 2e
5a a0 51 d b7 19 3c b7
19 3c b7 a0 51 d b7 :2 19
3c b7 :2 19 3c a0 7e 51 b4
2e :9 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :1f a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b a0
7e a0 6b b4 2e 5a :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b :2 a0 a5 57 b7 19 3c
:2 a0 51 a5 b 7e 51 b4 2e
a0 6e d b7 a0 6e d b7
:2 19 3c :13 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f a0 b7 :2 a0 :2 51 a5 b
7e 6e b4 2e :a a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c :9 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :9 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :6 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
19 3c :2 a0 b4 2e d :2 a0 51
a5 b 7e 51 b4 2e :2 a0 51
a5 b a0 7e a0 6b b4 2e
a 10 :2 a0 51 a5 b a0 7e
a0 6b b4 2e a 10 a0 7e
b4 2e 5a :2 a0 d b7 19 3c
b7 19 3c :2 a0 51 a5 b 7e
51 b4 2e :3 a0 6b d b7 19
3c a0 7e b4 2e 5a a0 7e
b4 2e 5a a 10 a0 7e b4
2e 5a a 10 a0 7e b4 2e
:2 5a a 10 a0 51 d b7 19
3c a0 6e d a0 6e d :2 a0
51 a5 b 7e 51 b4 2e :2 a0
51 a5 b a0 7e a0 6b b4
2e a0 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e d
a0 b7 :2 a0 51 a5 b 7e 51
b4 2e a0 6e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e d b7 19 a0
6e 7e 6e b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e d b7 :2 19
3c b7 :2 a0 51 a5 b a0 7e
a0 6b b4 2e :2 a0 51 a5 b
a0 7e a0 6b b4 2e a 10
:2 a0 51 a5 b a0 7e a0 6b
b4 2e a 10 :2 a0 6e 11e 11d
:2 a0 6b 51 d a0 7e b4 2e
5a :2 a0 6e 11e 11d :2 a0 6b 51
d :17 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :15 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 :2 a0 d b7 :2 19 3c b7
19 3c a0 7e b4 2e 5a :4 a0
a5 b 51 a5 b 7e 51 b4
2e :4 a0 a5 b 51 a5 b 51
a5 b 7e 51 b4 2e a6 122
:4 a0 a5 b 51 a5 b 51 a5
b 122 124 d a0 51 d :3 a0
7e a0 b4 2e 82 :5 a0 7e 51
b4 2e 51 a5 b a5 b 51
a5 b 7e 51 b4 2e :5 a0 7e
51 b4 2e 51 a5 b a5 b
51 a5 b 7e 51 b4 2e a
10 a0 2b b7 19 3c :2 a0 7e
51 b4 2e d b7 a0 47 :2 a0
7e b4 2e :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b a0
a5 57 b7 19 3c :4 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
19 3c :2 a0 51 a5 b a0 7e
a0 6b b4 2e a0 6e d a0
6e d :2 a0 7e a0 b4 2e d
:2 a0 7e 6e b4 2e d a0 b7
:2 a0 51 a5 b a0 7e a0 6b
b4 2e a0 6e d :2 a0 7e a0
b4 2e d :2 a0 7e 6e b4 2e
d a0 6e d b7 19 :2 a0 51
a5 b a0 7e a0 6b b4 2e
a0 6e d a0 6e d a0 7e
b4 2e 5a :2 a0 7e 6e b4 2e
d :2 a0 7e a0 b4 2e d :2 a0
7e 6e b4 2e d b7 19 3c
:2 a0 7e 6e b4 2e d a0 7e
b4 2e 5a :2 a0 7e 6e b4 2e
d :2 a0 7e a0 b4 2e d :2 a0
7e 6e b4 2e d b7 19 3c
b7 a0 7e b4 2e 5a :2 a0 7e
6e b4 2e d a0 6e d b7
:2 a0 7e 6e b4 2e 7e 6e b4
2e d a0 6e d a0 7e b4
2e 5a :2 a0 7e 6e b4 2e d
:2 a0 7e a0 b4 2e d :2 a0 7e
6e b4 2e d b7 19 3c :2 a0
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
d b7 :2 19 3c a0 7e b4 2e
5a :2 a0 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e d b7
19 3c b7 :2 19 3c b7 :2 19 3c
:2 a0 51 a5 b a0 7e a0 6b
b4 2e a0 7e b4 2e 5a :5 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :4 a0 a5 b 51 a5 b
d :2 a0 7e 6e b4 2e d :2 a0
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
d b7 :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 :2 19 3c b7 19 3c a0
7e b4 2e 5a :5 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a :2 a0
7e 6e b4 2e d :2 a0 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e d b7
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 b7
:2 19 3c b7 19 3c a0 7e b4
2e 5a :5 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a :2 a0 7e 6e
b4 2e d :2 a0 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e d b7 :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b :2 a0 a5 57 b7 :2 19 3c
b7 19 3c :2 a0 51 a5 b 7e
51 b4 2e :2 a0 51 a5 b 7e
51 b4 2e a 10 :2 a0 7e 6e
b4 2e 7e :3 a0 51 a5 b a5
b b4 2e 7e 6e b4 2e d
b7 19 3c b7 19 3c a0 7e
b4 2e 5a :4 a0 a5 b 51 a5
b d a0 51 d :3 a0 7e a0
b4 2e 82 :3 a0 6b :2 51 a0 7e
51 b4 2e a0 a5 b d :5 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 19 3c :2 a0 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e d :2 a0 51
a5 b a0 7e a0 6b b4 2e
:2 a0 7e 6e b4 2e d b7 :2 a0
7e 6e b4 2e d b7 :2 19 3c
:2 a0 51 a5 b a0 7e a0 6b
b4 2e a0 2b b7 19 3c :2 a0
7e 51 b4 2e d b7 a0 47
b7 19 3c :2 a0 51 a5 b a0
7e a0 6b b4 2e :4 a0 a5 b
51 a5 b d a0 51 d :3 a0
7e a0 b4 2e 82 :3 a0 6b :2 51
a0 7e 51 b4 2e a0 a5 b
d :5 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 19 3c :2 a0 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e d :2 a0 7e 51 b4
2e d b7 a0 47 b7 19 3c
:2 a0 7e a0 b4 2e d b7 :2 19
3c :2 a0 7e 6e b4 2e d :3 a0
11c 11d :3 a0 e9 d3 :2 a0 f :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c :2 a0 6b a0 7e a0
6b b4 2e 5a a0 2b b7 19
3c :2 a0 51 a5 b 7e 51 b4
2e :2 a0 51 a5 b a0 7e a0
6b b4 2e a 10 :2 a0 51 a5
b a0 7e a0 6b b4 2e a
10 :2 a0 51 a5 b a0 7e a0
6b b4 2e a0 7e b4 2e 5a
52 10 5a a 10 :2 a0 e9 dd
b3 :2 a0 e9 d3 :2 a0 f :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c :2 a0 6b a0 7e a0 6b
b4 2e 5a :2 a0 e9 c1 :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f a0 3b
b7 :2 a0 e9 c1 :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c b7
19 3c :2 a0 d :2 a0 d :2 a0 51
a5 b 7e 51 b4 2e :2 a0 :2 51
a5 b 7e 6e b4 2e :2 a0 b4
2e d a0 51 d b7 :3 a0 51
a5 b a5 b 7e 51 b4 2e
:9 a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 51 a5 b 7e 51
b4 2e :2 a0 e9 c1 :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c b7
19 3c b7 :2 19 3c :20 a0 51 a5
b :3 51 a5 b d a0 7e 51
b4 2e :2 a0 e9 c1 :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 7e a0
6b b4 2e :3 a0 6b d b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :3 a0 a5 57 a0 b7 :2 a0
:2 51 a5 b 7e 6e b4 2e 5a
:2 a0 :2 51 a5 b 7e 6e b4 2e
5a a 10 :5 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 :2 19 3c a0 2b
b7 :2 a0 51 a5 b a0 7e a0
6b b4 2e :2 a0 d :2 a0 d b7
19 3c :1e a0 :4 51 a5 b d a0
7e 51 b4 2e :2 a0 7e 51 b4
2e d b7 19 3c :2 a0 7e a0
51 a5 b b4 2e a0 2b b7
19 3c b7 :2 19 3c 93 4f b0
46 b7 a0 47 :2 a0 e9 c1 :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
7e a0 6b b4 2e 5a a0 7e
51 b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a a 10 5a
52 10 :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :3 a0 a5 57 b7 19
3c a0 7e 51 b4 2e a0 6e
d a0 6e d a0 6e d :2 a0
e9 dd b3 a0 51 d :a a0 e9
d3 5 :2 a0 f :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
:2 a0 6b a0 7e a0 6b b4 2e
5a a0 2b b7 19 3c a0 7e
b4 2e 5a :2 a0 6b :2 a0 a5 b
7e 51 b4 2e a 10 :3 a0 a5
b 51 a5 b 7e 51 b4 2e
a 10 :2 a0 d b7 19 3c :2 a0
6b :2 a0 a5 b 7e 51 b4 2e
a0 7e 51 b4 2e :f a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 19 3c
a0 6e d a0 6e d a0 6e
d a0 6e d a0 6e d a0
6e d :2 a0 d a0 51 d b7
19 3c a0 7e 51 b4 2e a0
7e b4 2e 5a a 10 :9 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 19
3c a0 7e b4 2e 5a a0 7e
b4 2e 5a :3 a0 7e 6e b4 2e
:2 51 a5 b d b7 19 3c :3 a0
7e a0 b4 2e :2 51 a5 b d
b7 19 3c a0 7e b4 2e 5a
:3 a0 7e a0 b4 2e :2 51 a5 b
d b7 19 3c a0 7e b4 2e
5a a0 7e b4 2e 5a :3 a0 7e
6e b4 2e :2 51 a5 b d b7
19 3c :3 a0 7e a0 b4 2e :2 51
a5 b d b7 19 3c a0 7e
b4 2e 5a :3 a0 7e a0 b4 2e
:2 51 a5 b d b7 19 3c a0
7e b4 2e 5a a0 7e b4 2e
5a :3 a0 7e 6e b4 2e :2 51 a5
b d b7 19 3c :3 a0 7e a0
b4 2e :2 51 a5 b d b7 19
3c a0 7e b4 2e 5a a0 7e
b4 2e 5a :3 a0 7e 6e b4 2e
:2 51 a5 b d b7 19 3c :3 a0
7e a0 b4 2e :2 51 a5 b d
b7 19 3c :2 a0 7e 51 b4 2e
d :2 a0 d b7 a0 47 :2 a0 e9
c1 :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f a0 7e 51 b4 2e :f a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 19
3c b7 19 3c a0 7e b4 2e
5a :2 a0 51 a5 b 7e 51 b4
2e :e a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 a0 51 d 4f
b7 :2 19 3c b7 19 3c :13 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 51 a5 b a0 7e a0 6b
b4 2e :2 a0 6e 11e 11d :2 a0 6b
51 d :b a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b a0 7e a0 6b
b4 2e 5a :12 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a :6 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 19 3c b7
19 3c :cc a0 12a e9 dd :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 65 b7 a0 7e 51 b4
2e :12d a0 12a e9 dd :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 65 b7 :2 a0 6e 11e 11d :2 a0
6b 51 d :8 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :14 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :1b7 a0 12a e9 dd :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 65 b7 :2 19 3c b7
:2 19 3c b7 :2 a0 6b :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 :2 a0 6b :2 6e
7e :2 a0 6b b4 2e 7e 6e b4
2e 7e :2 a0 6b b4 2e a5 57
b7 19 3c :9 a0 12a e9 dd :2 a0
65 b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c :7 a0 12a e9 dd :2 a0 65 b7
a4 b1 11 4f b7 a6 9 a4
b1 11 68 4f 1d 17 b5 
366b
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 6f 6b 59 77 84
80 56 8c 95 91 7f 9d aa
a6 7c b2 bb b7 a5 c3 d0
cc a2 d8 e1 dd cb e9 f6
f2 c8 fe 107 103 f1 10f 11c
118 ee 124 12d 129 117 135 142
13e 114 14a 153 14f 13d 15b 168
164 13a 170 179 175 163 181 18e
18a 160 196 19f 19b 189 1a7 1b4
1b0 186 1bc 1c5 1c1 1af 1cd 1da
1d6 1ac 1e2 1eb 1e7 1d5 1f3 200
1fc 1d2 208 211 20d 1fb 219 1f8
21e 222 226 22a 22d 231 24e 239
23d 240 241 249 238 26b 259 235
25d 25e 266 258 288 276 255 27a
27b 283 275 2a5 293 272 297 298
2a0 292 2c2 2b0 28f 2b4 2b5 2bd
2af 2df 2cd 2ac 2d1 2d2 2da 2cc
2fc 2ea 2c9 2ee 2ef 2f7 2e9 319
307 2e6 30b 30c 314 306 336 324
303 328 329 331 323 353 341 320
345 346 34e 340 370 35e 33d 362
363 36b 35d 38d 37b 35a 37f 380
388 37a 3aa 398 377 39c 39d 3a5
397 3c7 3b5 394 3b9 3ba 3c2 3b4
3e4 3d2 3b1 3d6 3d7 3df 3d1 401
3ef 3ce 3f3 3f4 3fc 3ee 41e 40c
3eb 410 411 419 40b 43b 429 408
42d 42e 436 428 458 446 425 44a
44b 453 445 475 463 442 467 468
470 462 492 480 45f 484 485 48d
47f 4ae 49d 4a1 4a9 47c 4ca 4b5
4b9 4bc 4bd 4c5 49c 4e7 4d5 499
4d9 4da 4e2 4d4 504 4f2 4d1 4f6
4f7 4ff 4f1 521 50f 4ee 513 514
51c 50e 53e 52c 50b 530 531 539
52b 55b 549 528 54d 54e 556 548
578 566 545 56a 56b 573 565 595
583 562 587 588 590 582 5b2 5a0
57f 5a4 5a5 5ad 59f 5cf 5bd 59c
5c1 5c2 5ca 5bc 5ec 5da 5b9 5de
5df 5e7 5d9 608 5f7 5fb 5d6 603
5f6 624 613 617 5f3 61f 612 640
62f 633 60f 63b 62e 65c 64b 64f
62b 657 64a 678 667 66b 647 673
666 694 683 687 663 68f 682 6b0
69f 6a3 67f 6ab 69e 6cc 6bb 6bf
69b 6c7 6ba 6e8 6d7 6db 6b7 6e3
6d6 704 6f3 6f7 6d3 6ff 6f2 720
70f 713 6ef 71b 70e 73c 72b 72f
70b 737 72a 758 747 74b 727 753
746 774 763 767 743 76f 762 790
77f 783 75f 78b 77e 7ac 79b 79f
77b 7a7 79a 7c8 7b7 7bb 797 7c3
7b6 7e4 7d3 7d7 7b3 7df 7d2 800
7ef 7f3 7cf 7fb 7ee 81c 80b 80f
7eb 817 80a 838 827 82b 807 833
826 854 843 847 823 84f 842 870
85f 863 83f 86b 85e 88c 87b 87f
85b 887 87a 8a8 897 89b 877 8a3
896 8c4 8b3 8b7 893 8bf 8b2 8e0
8cf 8d3 8af 8db 8ce 8fc 8eb 8ef
8cb 8f7 8ea 918 907 90b 8e7 913
906 934 923 927 903 92f 922 950
93f 943 91f 94b 93e 96c 95b 95f
93b 967 95a 988 977 97b 957 983
976 9a5 993 973 997 998 9a0 992
9c1 9b0 9b4 98f 9bc 9af 9dd 9cc
9d0 9ac 9d8 9cb 9f9 9e8 9ec 9c8
9f4 9e7 a16 a04 9e4 a08 a09 a11
a03 a33 a21 a00 a25 a26 a2e a20
a50 a3e a1d a42 a43 a4b a3d a6c
a5b a5f a3a a67 a5a a88 a77 a7b
a57 a83 a76 aa5 a93 a73 a97 a98
aa0 a92 ac2 ab0 a8f ab4 ab5 abd
aaf adf acd aac ad1 ad2 ada acc
afc aea ac9 aee aef af7 ae9 b19
b07 ae6 b0b b0c b14 b06 b36 b24
b03 b28 b29 b31 b23 b52 b41 b45
b20 b4d b40 b6e b5d b61 b3d b69
b5c b8a b79 b7d b59 b85 b78 ba6
b95 b99 b75 ba1 b94 bc2 bb1 bb5
b91 bbd bb0 bde bcd bd1 bad bd9
bcc bfa be9 bed bc9 bf5 be8 c16
c05 c09 be5 c11 c04 c33 c21 c01
c25 c26 c2e c20 c4f c3e c42 c1d
c4a c3d c6b c5a c5e c66 c3a c83
c72 c76 c7e c59 c9f c8e c92 c9a
c56 cb7 ca6 caa cb2 c8d cd3 cc2
cc6 cce c8a ceb cda cde ce6 cc1
d07 cf6 cfa d02 cbe d26 d0e d12
d16 d19 d21 cf5 d2d d31 cf2 d42
d45 d49 d4d d51 d55 d59 d5d d61
d65 d69 d75 d7a d7c d88 d8c d8e
d92 da3 da4 da7 dab daf db3 db7
dbb dbf dc3 dc7 dcb dcf dd3 dd7
ddb ddf de3 de7 deb df7 dfc dfe
e0a e0e e2c e14 e18 e1c e1f e27
e13 e4c e37 e3b e10 e3f e47 e36
e6c e57 e5b e33 e5f e67 e56 e8c
e77 e7b e53 e7f e87 e76 e93 e97
e73 e9b e9f ea3 ea6 eab eac 1
eae eb3 eb7 ebb ebe ec3 ec7 ecb
ece ed3 ed4 ed6 edb ede ee2 ee3
ee8 eed ef0 ef5 ef6 efb efe f02
f03 f08 f0b f10 f11 f16 f19 f1e
f1f f24 f27 f2c f2d f32 f35 f39
f3a f3f f42 f47 f48 f4d f50 f55
f56 f5b f5e f63 f64 f69 f6c f70
f71 f76 f79 f7e f7f f84 f87 f8c
f8d f92 f95 f99 f9d fa1 fa2 fa4
fa9 faa fac fad fb2 fb5 fba fbb
fc0 fc3 fc7 fcb fcf fd0 fd2 fd7
fd8 fda fdb fe0 fe3 fe8 fe9 fee
ff1 ff5 ff9 ffd ffe 1000 1005 1006
1008 1009 100e 1011 1016 1017 101c 101f
1023 1027 102b 102c 102e 1033 1034 1036
1037 103c 103f 1044 1045 104a 104d 1051
1055 1059 105a 105c 1061 1062 1064 1065
106a 106d 1072 1073 1078 107b 107f 1083
1087 1088 108a 108f 1090 1092 1093 1098
109b 10a0 10a1 10a6 10a9 10ad 10b1 10b5
10b6 10b8 10bd 10be 10c0 10c1 10c6 10c9
10ce 10cf 10d4 10d7 10db 10df 10e3 10e4
10e6 10eb 10ec 10ee 10ef 10f4 10f7 10fc
10fd 1102 1105 110a 110b 1110 1113 1117
1118 111d 1120 1125 1126 112b 112e 1133
1134 1139 113c 1141 1142 1147 114a 114e
114f 1154 1157 115c 115d 1162 1165 116a
116b 1170 1173 1178 1179 117e 1181 1185
1186 118b 118e 1193 1194 1199 119c 11a1
11a2 11a7 11aa 11af 11b0 11b5 11b8 11bc
11bd 11c2 11c5 11ca 11cb 11d0 11d3 11d8
11d9 11de 11e1 11e6 11e7 11ec 11ef 11f3
11f4 11f9 11fc 1201 1202 1207 120a 120f
1210 1215 1218 121c 1220 1224 1225 1227
122c 122d 122f 1230 1235 1238 123d 123e
1243 1246 124b 124c 1251 1254 1258 1259
125e 1261 1266 1267 126c 126f 1274 1275
127a 127d 1282 1283 1288 128b 128f 1290
1295 1298 129d 129e 12a3 12a6 12ab 12ac
12b1 12b4 12b9 12ba 12bf 12c2 12c6 12c7
12cc 12cf 12d4 12d5 12da 12dd 12e2 12e3
12e8 12eb 12f0 12f1 12f6 12f9 12fd 12fe
1303 1306 130b 130c 1311 1314 1319 131a
131f 1322 1326 132a 132e 132f 1331 1336
1337 1339 133a 133f 1342 1347 1348 134d
1350 1354 1358 135c 135d 135f 1364 1365
1367 1368 136d 1370 1375 1376 137b 137e
1382 1386 138a 138b 138d 1392 1393 1395
1396 139b 139e 13a3 13a4 13a9 13ac 13b0
13b4 13b8 13b9 13bb 13c0 13c1 13c3 13c4
13c9 13cc 13d1 13d2 13d7 13da 13df 13e0
13e5 13e8 13ec 13ed 13f2 13f5 13fa 13fb
1400 1403 1408 1409 140e 1411 1416 1417
141c 141f 1423 1424 1429 142c 1431 1432
1437 143a 143f 1440 1445 1446 144b 144d
1451 1454 1458 145c 145f 1463 1467 146a
146d 1472 1475 1478 147b 147c 1481 1482
1 1484 1489 148d 1491 1494 1499 149e
14a1 14a5 14a6 14ab 14ac 14b1 14b3 14b7
14ba 14be 14c2 14c6 14ca 14ce 14d2 14d6
14da 14de 14e2 14e6 14ea 14ee 14f2 14f6
14fa 14fe 1502 1506 150a 150e 1512 1516
151a 151e 1522 1526 152a 152e 1532 1536
153a 153e 1542 1546 154a 154e 1552 1556
155a 155e 1562 1566 156a 156e 1572 1576
157a 157e 1582 1586 158a 158e 1592 1596
159a 159e 15a2 15a6 15aa 15ae 15b2 15b6
15ba 15be 15c2 15c6 15c9 15cd 15d1 15d4
15d8 15dc 15e0 15e3 15e4 15e6 15ea 15ed
15f1 15f4 15f5 15fa 15fd 1601 1604 1605
160a 160d 1 1610 1615 1619 161d 1620
1624 1628 162b 162e 1633 1636 1639 163c
163d 1642 1643 1 1645 164a 164e 1652
1655 165a 165f 1662 1666 1667 166c 166f
1674 1675 167a 167d 1682 1683 1688 168b
168f 1693 1697 169b 169e 169f 16a1 16a6
16a7 16a9 16aa 16af 16b0 16b5 16b7 16bb
16be 16c2 16c6 16c9 16cd 16d1 16d4 16d7
16dc 16df 16e2 16e5 16e6 16eb 16ec 1
16ee 16f3 16f7 16fb 16fe 1703 1708 170b
170f 1710 1715 1716 171b 171d 1721 1724
1728 172c 172f 1733 1737 173a 173e 173f
1744 1746 174a 174d 1751 1754 1755 175a
175d 1760 1764 1767 1768 176d 1770 1
1773 1778 177c 177f 1780 1785 1788 1
178b 1790 1794 1797 1798 179d 17a0 1
17a3 17a8 17ac 17b0 17b3 17b4 17b6 17ba
17bd 17c1 17c4 17c5 17ca 1 17cd 17d2
17d6 17d9 17da 17df 17e2 17e5 17e9 17ed
17f0 17f1 17f3 17f7 17fa 17fe 1801 1802
1807 1 180a 180f 1813 1817 181a 181e
1822 1825 1828 182d 1830 1833 1836 1837
183c 183d 1 183f 1844 1848 184c 184f
1854 1859 185c 1860 1861 1866 1869 186e
186f 1874 1877 187c 187d 1882 1885 1889
188d 1891 1895 1898 1899 189b 18a0 18a1
18a3 18a4 18a9 18aa 18af 18b1 18b5 18b8
18bc 18c0 18c3 18c7 18cb 18ce 18d1 18d6
18d9 18dc 18df 18e0 18e5 18e6 1 18e8
18ed 18f1 18f5 18f8 18fd 1902 1905 1909
190a 190f 1910 1915 1917 191b 191e 1922
1926 1929 192d 1931 1934 1938 1939 193e
1942 1944 1948 194b 194c 1951 1954 1957
195b 195e 195f 1964 1967 1 196a 196f
1973 1977 197a 197e 1982 1985 1988 198d
1990 1993 1996 1997 199c 199d 1 199f
19a4 19a8 19ac 19af 19b4 19b9 19bc 19c0
19c1 19c6 19c9 19ce 19cf 19d4 19d7 19dc
19dd 19e2 19e5 19e9 19ed 19f1 19f5 19f8
19f9 19fb 1a00 1a01 1a03 1a04 1a09 1a0a
1a0f 1a11 1a15 1a18 1a1c 1a20 1a23 1a27
1a2b 1a2e 1a31 1a36 1a39 1a3c 1a3f 1a40
1a45 1a46 1 1a48 1a4d 1a51 1a55 1a58
1a5d 1a62 1a65 1a69 1a6a 1a6f 1a70 1a75
1a77 1a7b 1a7e 1a82 1a86 1a89 1a8d 1a91
1a94 1a98 1a99 1a9e 1aa0 1aa4 1aa8 1aab
1aad 1ab1 1ab4 1ab8 1abc 1abf 1ac0 1ac2
1ac5 1ac8 1ac9 1ace 1ad2 1ad6 1ad9 1ada
1adc 1adf 1ae2 1ae3 1 1ae8 1aed 1af1
1af5 1af8 1af9 1afb 1afe 1b01 1b02 1
1b07 1b0c 1b10 1b14 1b17 1b1b 1b1f 1b22
1b25 1b2a 1b2d 1b30 1b33 1b34 1b39 1b3a
1 1b3c 1b41 1b45 1b49 1b4c 1b51 1b56
1b59 1b5d 1b5e 1b63 1b66 1b6b 1b6c 1b71
1b74 1b79 1b7a 1b7f 1b82 1b86 1b8a 1b8e
1b92 1b95 1b96 1b98 1b9d 1b9e 1ba0 1ba1
1ba6 1ba7 1bac 1bae 1bb2 1bb5 1bb9 1bbd
1bc0 1bc4 1bc8 1bcb 1bce 1bd3 1bd6 1bd9
1bdc 1bdd 1be2 1be3 1 1be5 1bea 1bee
1bf2 1bf5 1bfa 1bff 1c02 1c06 1c07 1c0c
1c0d 1c12 1c14 1c18 1c1b 1c1f 1c23 1c26
1c2a 1c2e 1c31 1c35 1c36 1c3b 1c3d 1c41
1c44 1c48 1c4b 1c4c 1c51 1c54 1c57 1c5b
1c5e 1c62 1c64 1c68 1c6c 1c70 1c72 1c76
1c7a 1c7d 1c81 1c85 1c88 1c89 1c8b 1c8f
1c92 1c96 1c99 1c9a 1c9f 1ca2 1ca6 1ca9
1caa 1caf 1cb2 1 1cb5 1cba 1cbe 1cc1
1cc2 1cc7 1cca 1 1ccd 1cd2 1cd6 1cda
1cde 1ce2 1ce6 1cea 1cee 1cf2 1cfe 1d02
1d06 1d09 1d0d 1d11 1d13 1d17 1d1b 1d1f
1d22 1d26 1d2a 1d2c 1d2d 1d32 1 1d36
1d3a 1d3e 1d41 1d45 1d49 1d4d 1d51 1d54
1d58 1d5c 1d5f 1d62 1d67 1d6a 1d6d 1d70
1d71 1d76 1d77 1 1d79 1d7e 1d82 1d86
1d89 1d8e 1d93 1d96 1d9a 1d9b 1da0 1da3
1da8 1da9 1dae 1db1 1db6 1db7 1dbc 1dbf
1dc3 1dc4 1dc9 1dcc 1dd1 1dd2 1dd7 1dda
1dde 1de2 1de5 1de8 1de9 1deb 1dec 1df1
1df2 1df7 1df9 1dfd 1e00 1e02 1e06 1e08
1e14 1e16 1e18 1e19 1e1e 1e22 1e24 1e30
1e32 1e36 1e3a 1e3d 1e41 1e44 1e48 1e4b
1e4c 1e51 1e54 1e58 1e5c 1e5f 1e60 1e62
1e65 1e68 1e69 1e6e 1 1e71 1e76 1e7a
1e7e 1e81 1e85 1e89 1e8c 1e8f 1e94 1e97
1e9a 1e9d 1e9e 1ea3 1ea4 1 1ea6 1eab
1eaf 1eb3 1eb6 1ebb 1ec0 1ec3 1ec7 1ec8
1ecd 1ed0 1ed5 1ed6 1edb 1ede 1ee3 1ee4
1ee9 1eec 1ef0 1ef4 1ef8 1efc 1eff 1f00
1f02 1f07 1f08 1f0a 1f0b 1f10 1f11 1f16
1f18 1f1c 1f1f 1f23 1f27 1f2a 1f2e 1f32
1f35 1f38 1f3d 1f40 1f43 1f46 1f47 1f4c
1f4d 1 1f4f 1f54 1f58 1f5c 1f5f 1f64
1f69 1f6c 1f70 1f71 1f76 1f77 1f7c 1f7e
1f82 1f85 1f89 1f8d 1f90 1f94 1f98 1f9b
1f9f 1fa0 1fa5 1fa7 1fab 1fae 1fb2 1fb5
1fb9 1fbb 1fbf 1fc2 1fc6 1fc9 1fca 1fcf
1fd2 1fd5 1fd9 1fdc 1fdd 1fe2 1fe5 1
1fe8 1fed 1ff1 1ff5 1ff9 2005 2009 200d
2010 2014 2018 201a 201e 2022 2026 2029
202d 2031 2033 2034 2039 1 203d 2041
2045 2048 204c 2050 2054 2058 205b 205f
2063 2066 2069 206e 2071 2074 2077 2078
207d 207e 1 2080 2085 2089 208d 2090
2095 209a 209d 20a1 20a2 20a7 20aa 20af
20b0 20b5 20b8 20bd 20be 20c3 20c6 20ca
20cb 20d0 20d3 20d8 20d9 20de 20e1 20e5
20e9 20ec 20ef 20f0 20f2 20f3 20f8 20f9
20fe 2100 2104 2107 2109 210d 210f 211b
211d 211f 2120 2125 2129 212b 2137 2139
213b 213f 2142 2146 214a 214e 2152 2156
215a 215d 2161 2165 2169 216c 216d 216f
2173 2177 217a 217b 217d 2180 2183 2184
2186 218a 218e 2192 2196 219a 219d 219e
21a0 21a4 21a8 21ab 21ac 21ae 21b2 21b6
21ba 21be 21c1 21c5 21c9 21cd 21d0 21d1
21d3 21d7 21db 21de 21df 21e1 21e4 21e7
21e8 21ea 21ee 21f2 21f6 21fa 21fe 2201
2202 2204 2208 220c 220f 2210 2212 2216
221a 221e 2222 2225 2226 2228 222c 222f
2233 2236 2237 223c 223f 2240 2242 2245
2247 224b 224f 2253 2257 225a 225b 225d
2260 2263 2264 2269 226d 2270 2274 2278
227c 2280 2284 2288 2294 2298 229c 229f
22a3 22a7 22a9 22ad 22b1 22b5 22b8 22bc
22c0 22c2 22c3 22c8 1 22cc 22d0 22d4
22d7 22db 22df 22e3 22e7 22ea 22ee 22f2
22f5 22f8 22fd 2300 2303 2306 2307 230c
230d 1 230f 2314 2318 231c 231f 2324
2329 232c 2330 2331 2336 2339 233e 233f
2344 2347 234c 234d 2352 2355 2359 235a
235f 2362 2367 2368 236d 2370 2374 2378
237b 237e 237f 2381 2382 2387 2388 238d
238f 2393 2396 2398 239c 239e 23aa 23ac
23ae 23af 23b4 23b8 23ba 23c6 23c8 23cc
23d0 23d3 23d4 23d6 23d9 23dc 23dd 23e2
23e6 23ea 23ee 23f2 23f6 2402 2406 240a
240d 2411 2415 2417 241b 241f 2423 2426
242a 242e 2430 2431 2436 1 243a 243e
2442 2445 2449 244d 2451 2455 2458 245c
2460 2463 2466 246b 246e 2471 2474 2475
247a 247b 1 247d 2482 2486 248a 248d
2492 2497 249a 249e 249f 24a4 24a7 24ac
24ad 24b2 24b5 24ba 24bb 24c0 24c3 24c7
24c8 24cd 24d0 24d5 24d6 24db 24de 24e2
24e6 24e9 24ec 24ed 24ef 24f0 24f5 24f6
24fb 24fd 2501 2504 2506 250a 250c 2518
251a 251c 251d 2522 2526 2528 2534 2536
253a 253e 2541 2545 254a 254b 254d 2550
2553 2554 2559 255d 2561 2564 2568 256c
256f 2572 2577 257a 257d 2580 2581 2586
2587 1 2589 258e 2592 2596 2599 259e
25a3 25a6 25aa 25ab 25b0 25b3 25b8 25b9
25be 25c1 25c6 25c7 25cc 25cf 25d3 25d7
25db 25df 25e2 25e3 25e5 25ea 25eb 25ed
25ee 25f3 25f6 25fb 25fc 2601 2604 2609
260a 260f 2612 2616 2617 261c 261f 2624
2625 262a 262b 2630 2632 2636 2639 263d
2641 2644 2648 264c 264f 2652 2657 265a
265d 2660 2661 2666 2667 1 2669 266e
2672 2676 2679 267e 2683 2686 268a 268b
2690 2691 2696 2698 269c 269f 26a3 26a7
26aa 26ae 26b2 26b5 26b9 26bd 26be 26c3
26c5 26c9 26cc 26ce 26d2 26d5 26d9 26dd
26e1 26e5 26e8 26e9 26eb 26ec 26ee 26f2
26f6 26fa 26fd 26fe 2700 2703 2706 2707
270c 2710 2714 2718 271c 271f 2720 2722
2723 2725 2729 272d 2731 2734 2735 2737
273a 273d 273e 2743 2747 274b 274e 2752
2756 2759 275c 2761 2764 2767 276a 276b
2770 2771 1 2773 2778 277c 2780 2783
2788 278d 2790 2794 2795 279a 279d 27a2
27a3 27a8 27ab 27b0 27b1 27b6 27b9 27bd
27c1 27c5 27c9 27cc 27cd 27cf 27d4 27d5
27d7 27d8 27dd 27de 27e3 27e5 27e9 27ec
27f0 27f4 27f7 27fb 27ff 2802 2805 280a
280d 2810 2813 2814 2819 281a 1 281c
2821 2825 2829 282c 2831 2836 2839 283d
283e 2843 2844 2849 284b 284f 2852 2856
285a 285d 2861 2865 2868 286c 286d 2872
2874 2878 287b 287f 2883 2887 288a 288e
2890 2894 2897 289b 289f 28a3 28a7 28ab
28ae 28af 28b1 28b4 28b7 28b8 28bd 28c1
28c5 28c9 28cb 28cf 28d3 28d6 28da 28de
28e1 28e4 28e9 28ec 28ef 28f2 28f3 28f8
28f9 1 28fb 2900 2904 2908 290b 2910
2915 2918 291c 291d 2922 2925 292a 292b
2930 2933 2938 2939 293e 2941 2945 2949
294d 2951 2954 2955 2957 295c 295d 295f
2960 2965 2966 296b 296d 2971 2974 2978
297c 297f 2983 2987 298a 298d 2992 2995
2998 299b 299c 29a1 29a2 1 29a4 29a9
29ad 29b1 29b4 29b9 29be 29c1 29c5 29c6
29cb 29cc 29d1 29d3 29d7 29da 29de 29e2
29e5 29e9 29ed 29f0 29f4 29f5 29fa 29fc
2a00 2a04 2a07 2a0b 2a0f 2a13 2a17 2a1b
2a1f 2a23 2a27 2a28 2a2a 2a2e 2a32 2a36
2a39 2a3a 2a3c 2a3f 2a42 2a43 2a48 2a4c
2a50 2a53 2a57 2a5b 2a5e 2a61 2a66 2a69
2a6c 2a6f 2a70 2a75 2a76 1 2a78 2a7d
2a81 2a85 2a88 2a8d 2a92 2a95 2a99 2a9a
2a9f 2aa2 2aa7 2aa8 2aad 2ab0 2ab5 2ab6
2abb 2abe 2ac2 2ac6 2aca 2acb 2acd 2ad2
2ad3 2ad5 2ad6 2adb 2ade 2ae3 2ae4 2ae9
2aec 2af1 2af2 2af7 2afa 2afe 2aff 2b04
2b07 2b0c 2b0d 2b12 2b13 2b18 2b1a 2b1e
2b21 2b25 2b29 2b2c 2b30 2b34 2b37 2b3a
2b3f 2b42 2b45 2b48 2b49 2b4e 2b4f 1
2b51 2b56 2b5a 2b5e 2b61 2b66 2b6b 2b6e
2b72 2b73 2b78 2b79 2b7e 2b80 2b84 2b87
2b8b 2b8f 2b92 2b96 2b9a 2b9d 2b9e 2ba0
2ba4 2ba8 2ba9 2bae 2bb0 2bb4 2bb7 2bbb
2bbf 2bc2 2bc3 2bc5 2bc9 2bcc 2bd0 2bd3
2bd4 2bd9 2bdd 2be1 2be4 2be8 2bec 2bef
2bf2 2bf7 2bfa 2bfd 2c00 2c01 2c06 2c07
1 2c09 2c0e 2c12 2c16 2c19 2c1e 2c23
2c26 2c2a 2c2b 2c30 2c33 2c38 2c39 2c3e
2c41 2c46 2c47 2c4c 2c4f 2c53 2c57 2c5b
2c5f 2c62 2c63 2c65 2c6a 2c6b 2c6d 2c6e
2c73 2c76 2c7b 2c7c 2c81 2c84 2c89 2c8a
2c8f 2c92 2c96 2c97 2c9c 2c9f 2ca4 2ca5
2caa 2cab 2cb0 2cb2 2cb6 2cb9 2cbd 2cc1
2cc4 2cc8 2ccc 2ccf 2cd2 2cd7 2cda 2cdd
2ce0 2ce1 2ce6 2ce7 1 2ce9 2cee 2cf2
2cf6 2cf9 2cfe 2d03 2d06 2d0a 2d0b 2d10
2d11 2d16 2d18 2d1c 2d1f 2d23 2d27 2d2a
2d2e 2d32 2d35 2d39 2d3d 2d3e 2d43 2d45
2d49 2d4c 2d50 2d54 2d57 2d58 2d5a 2d5e
2d61 2d65 2d68 2d69 2d6e 2d72 2d76 2d7a
2d7d 2d7e 2d80 2d81 2d83 2d86 2d89 2d8a
2d8f 2d93 2d97 2d9b 2d9f 2da3 2da7 2dab
2daf 2db3 2db7 2dbb 2dbf 2dc3 2dcf 2dd3
2dd7 2dda 2dde 2de2 2de4 2de8 2dec 2df0
2df3 2df7 2dfb 2dfd 2dfe 2e03 1 2e07
2e0b 2e0f 2e12 2e16 2e1a 2e1e 2e22 2e25
2e29 2e2d 2e30 2e33 2e38 2e3b 2e3e 2e41
2e42 2e47 2e48 1 2e4a 2e4f 2e53 2e57
2e5a 2e5f 2e64 2e67 2e6b 2e6c 2e71 2e74
2e79 2e7a 2e7f 2e82 2e87 2e88 2e8d 2e90
2e94 2e95 2e9a 2e9d 2ea2 2ea3 2ea8 2eab
2eaf 2eb3 2eb6 2eb9 2eba 2ebc 2ebd 2ec2
2ec3 2ec8 2eca 2ece 2ed1 2ed3 2ed7 2ed9
2ee5 2ee7 2ee9 2eea 2eef 2ef3 2ef5 2f01
2f03 2f05 2f09 2f0d 2f11 2f15 2f19 2f1d
2f21 2f25 2f29 2f2d 2f31 2f35 2f39 2f3d
2f41 2f45 2f49 2f4d 2f51 2f55 2f59 2f5d
2f61 2f65 2f69 2f6d 2f71 2f75 2f79 2f7d
2f81 2f85 2f89 2f8d 2f91 2f95 2f99 2f9d
2fa1 2fad 2fb1 2fb5 2fb8 2fbc 2fc0 2fc2
2fc6 2fca 2fce 2fd1 2fd5 2fd9 2fdb 2fdc
2fe1 1 2fe5 2fe9 2fed 2ff0 2ff4 2ff8
2ffc 3000 3003 3007 300b 300e 3011 3016
3019 301c 301f 3020 3025 3026 1 3028
302d 3031 3035 3038 303d 3042 3045 3049
304a 304f 3052 3057 3058 305d 3060 3065
3066 306b 306e 3072 3073 3078 307b 3080
3081 3086 3089 308d 3091 3094 3097 3098
309a 309b 30a0 30a1 30a6 30a8 30ac 30af
30b1 30b5 30b7 30c3 30c5 30c7 30c8 30cd
30d1 30d3 30df 30e1 30e3 30e7 30eb 30ee
30f2 30f6 30f9 30fd 3100 3104 3107 3108
310d 3110 3114 3118 311b 311f 3123 3126
3129 312e 3131 3134 3137 3138 313d 313e
1 3140 3145 3149 314d 3150 3155 315a
315d 3161 3162 3167 316a 316f 3170 3175
3178 317d 317e 3183 3186 318a 318e 3192
3196 3199 319a 319c 31a1 31a2 31a4 31a5
31aa 31ad 31b2 31b3 31b8 31bb 31c0 31c1
31c6 31c9 31cd 31ce 31d3 31d6 31db 31dc
31e1 31e2 31e7 31e9 31ed 31f0 31f4 31f8
31fb 31ff 3203 3206 3209 320e 3211 3214
3217 3218 321d 321e 1 3220 3225 3229
322d 3230 3235 323a 323d 3241 3242 3247
3248 324d 324f 3253 3256 325a 325e 3261
3265 3269 326c 3270 3274 3275 327a 327c
3280 3283 3287 3289 328d 3291 3294 3295
3297 329b 329e 32a2 32a5 32a6 32ab 32af
32b3 32b7 32bb 32bf 32c3 32c7 32cb 32cf
32db 32df 32e3 32e6 32ea 32ee 32f0 32f4
32f8 32fc 32ff 3303 3307 3309 330a 330f
1 3313 3317 331b 331e 3322 3326 332a
332e 3331 3335 3339 333c 333f 3344 3347
334a 334d 334e 3353 3354 1 3356 335b
335f 3363 3366 336b 3370 3373 3377 3378
337d 3380 3385 3386 338b 338e 3393 3394
3399 339c 33a0 33a1 33a6 33a9 33ae 33af
33b4 33b7 33bb 33bf 33c2 33c5 33c6 33c8
33c9 33ce 33cf 33d4 33d6 33da 33dd 33df
33e3 33e5 33f1 33f3 33f5 33f6 33fb 33ff
3401 340d 340f 3413 3417 341a 341e 3421
3425 3428 3429 342e 3431 3435 3439 343c
3440 3444 3447 344a 344f 3452 3455 3458
3459 345e 345f 1 3461 3466 346a 346e
3471 3476 347b 347e 3482 3483 3488 348b
3490 3491 3496 3499 349e 349f 34a4 34a7
34ab 34af 34b3 34b7 34ba 34bb 34bd 34c2
34c3 34c5 34c6 34cb 34ce 34d3 34d4 34d9
34dc 34e1 34e2 34e7 34ea 34ee 34ef 34f4
34f7 34fc 34fd 3502 3503 3508 350a 350e
3511 3515 3519 351c 3520 3524 3527 352a
352f 3532 3535 3538 3539 353e 353f 1
3541 3546 354a 354e 3551 3556 355b 355e
3562 3563 3568 3569 356e 3570 3574 3577
357b 357f 3582 3586 358a 358d 3591 3595
3596 359b 359d 35a1 35a4 35a8 35ac 35b0
35b4 35b8 35bc 35c0 35c4 35c8 35cc 35d0
35d4 35d8 35dc 35e0 35e4 35e8 35ec 35f0
35fc 3600 3604 3607 360b 360f 3611 3615
3619 361d 3620 3624 3628 362a 362b 3630
1 3634 3638 363c 363f 3643 3647 364b
364f 3652 3656 365a 365d 3660 3665 3668
366b 366e 366f 3674 3675 1 3677 367c
3680 3684 3687 368c 3691 3694 3698 3699
369e 36a1 36a6 36a7 36ac 36af 36b4 36b5
36ba 36bd 36c1 36c2 36c7 36ca 36cf 36d0
36d5 36d8 36dc 36e0 36e3 36e6 36e7 36e9
36ea 36ef 36f0 36f5 36f7 36fb 36fe 3700
3704 3706 3712 3714 3716 3717 371c 3720
3722 372e 3730 3734 3738 373b 373f 3742
3746 3749 374a 374f 3752 3756 375a 375d
3761 3765 3768 376b 3770 3773 3776 3779
377a 377f 3780 1 3782 3787 378b 378f
3792 3797 379c 379f 37a3 37a4 37a9 37ac
37b1 37b2 37b7 37ba 37bf 37c0 37c5 37c8
37cc 37d0 37d4 37d8 37db 37dc 37de 37e3
37e4 37e6 37e7 37ec 37ef 37f4 37f5 37fa
37fd 3802 3803 3808 380b 380f 3810 3815
3818 381d 381e 3823 3824 3829 382b 382f
3832 3836 383a 383d 3841 3845 3848 384b
3850 3853 3856 3859 385a 385f 3860 1
3862 3867 386b 386f 3872 3877 387c 387f
3883 3884 3889 388a 388f 3891 3895 3898
389c 38a0 38a3 38a7 38ab 38ae 38b2 38b6
38b7 38bc 38be 38c2 38c5 38c9 38cc 38cd
38d2 38d5 38d9 38dd 38e1 38e4 38e8 38ec
38f0 38f4 38f5 38f7 38fb 38fd 3901 3904
3906 390a 390e 3912 3915 3919 391d 3920
3923 3928 392b 392e 3931 3932 3937 3938
1 393a 393f 3943 3947 394a 394f 3954
3957 395b 395c 3961 3964 3969 396a 396f
3972 3977 3978 397d 3980 3984 3988 398c
3990 3993 3994 3996 399b 399c 399e 399f
39a4 39a7 39ac 39ad 39b2 39b5 39ba 39bb
39c0 39c3 39c7 39c8 39cd 39d0 39d5 39d6
39db 39dc 39e1 39e3 39e7 39ea 39ee 39f2
39f5 39f9 39fd 3a00 3a03 3a08 3a0b 3a0e
3a11 3a12 3a17 3a18 1 3a1a 3a1f 3a23
3a27 3a2a 3a2f 3a34 3a37 3a3b 3a3c 3a41
3a42 3a47 3a49 3a4d 3a50 3a54 3a58 3a5b
3a5f 3a63 3a66 3a6a 3a6e 3a6f 3a74 3a76
3a7a 3a7e 3a81 3a85 3a89 3a8c 3a8d 3a8f
3a93 3a96 3a9a 3a9d 3a9e 3aa3 3aa7 3aab
3aaf 3ab3 3ab7 3abb 3abf 3ac3 3ac7 3acb
3acf 3ad3 3ad7 3adb 3adf 3ae3 3ae7 3aeb
3aef 3af3 3af7 3b03 3b07 3b0b 3b0e 3b12
3b16 3b18 3b1c 3b20 3b24 3b27 3b2b 3b2f
3b31 3b32 3b37 1 3b3b 3b3f 3b43 3b46
3b4a 3b4e 3b52 3b56 3b59 3b5d 3b61 3b64
3b67 3b6c 3b6f 3b72 3b75 3b76 3b7b 3b7c
1 3b7e 3b83 3b87 3b8b 3b8e 3b93 3b98
3b9b 3b9f 3ba0 3ba5 3ba8 3bad 3bae 3bb3
3bb6 3bbb 3bbc 3bc1 3bc4 3bc8 3bc9 3bce
3bd1 3bd6 3bd7 3bdc 3bdf 3be3 3be7 3bea
3bed 3bee 3bf0 3bf1 3bf6 3bf7 3bfc 3bfe
3c02 3c05 3c07 3c0b 3c0d 3c19 3c1b 3c1d
3c1e 3c23 3c27 3c29 3c35 3c37 3c3b 3c3f
3c42 3c46 3c49 3c4d 3c50 3c51 3c56 3c59
3c5d 3c61 3c64 3c68 3c6c 3c6f 3c72 3c77
3c7a 3c7d 3c80 3c81 3c86 3c87 1 3c89
3c8e 3c92 3c96 3c99 3c9e 3ca3 3ca6 3caa
3cab 3cb0 3cb3 3cb8 3cb9 3cbe 3cc1 3cc6
3cc7 3ccc 3ccf 3cd3 3cd7 3cdb 3cdf 3ce2
3ce3 3ce5 3cea 3ceb 3ced 3cee 3cf3 3cf6
3cfb 3cfc 3d01 3d04 3d09 3d0a 3d0f 3d12
3d16 3d17 3d1c 3d1f 3d24 3d25 3d2a 3d2b
3d30 3d32 3d36 3d39 3d3d 3d41 3d44 3d48
3d4c 3d4f 3d52 3d57 3d5a 3d5d 3d60 3d61
3d66 3d67 1 3d69 3d6e 3d72 3d76 3d79
3d7e 3d83 3d86 3d8a 3d8b 3d90 3d91 3d96
3d98 3d9c 3d9f 3da3 3da7 3daa 3dae 3db2
3db5 3db9 3dbd 3dbe 3dc3 3dc7 3dc9 3dcd
3dd1 3dd4 3dd8 3ddb 3ddf 3de2 3de3 3de8
3deb 3def 3df3 3df6 3df9 3dfa 3dfc 3dff
3e04 3e05 3e0a 3e0e 3e12 3e15 3e16 3e18
3e1b 3e1e 3e1f 3e24 3e28 3e2c 3e2f 3e30
3e32 3e36 3e39 3e3d 3e40 3e41 3e43 3e44
1 3e49 3e4e 1 3e51 3e56 3e5a 3e5e
3e61 3e62 3e64 3e68 3e6b 3e6f 3e72 3e73
3e75 3e76 1 3e7b 3e80 3e84 3e88 3e8b
3e8f 3e93 3e96 3e99 3e9e 3ea1 3ea4 3ea7
3ea8 3ead 3eae 1 3eb0 3eb5 3eb9 3ebd
3ec0 3ec5 3eca 3ecd 3ed1 3ed2 3ed7 3eda
3edf 3ee0 3ee5 3ee8 3eed 3eee 3ef3 3ef6
3efa 3efe 3f02 3f06 3f09 3f0a 3f0c 3f11
3f12 3f14 3f15 3f1a 3f1d 3f22 3f23 3f28
3f2b 3f30 3f31 3f36 3f39 3f3d 3f3e 3f43
3f46 3f4b 3f4c 3f51 3f52 3f57 3f59 3f5d
3f60 3f64 3f68 3f6b 3f6f 3f73 3f76 3f79
3f7e 3f81 3f84 3f87 3f88 3f8d 3f8e 1
3f90 3f95 3f99 3f9d 3fa0 3fa5 3faa 3fad
3fb1 3fb2 3fb7 3fb8 3fbd 3fbf 3fc3 3fc6
3fca 3fce 3fd1 3fd5 3fd9 3fdc 3fe0 3fe4
3fe5 3fea 3fec 3ff0 3ff3 3ff5 3ff9 3ffd
4000 4002 4006 4009 400b 400f 4012 4016
401a 401d 401e 4020 4023 4026 4027 402c
4030 4034 4037 4038 403a 403e 4041 4045
4048 4049 404e 4052 4055 4056 405b 1
405e 4063 4067 406b 406e 406f 4071 4074
4077 4078 1 407d 4082 4086 408a 408d
4091 4095 4098 409b 40a0 40a3 40a6 40a9
40aa 40af 40b0 1 40b2 40b7 40bb 40bf
40c2 40c7 40cc 40cf 40d3 40d4 40d9 40dc
40e1 40e2 40e7 40ea 40ef 40f0 40f5 40f8
40fc 4100 4104 4108 410b 410c 410e 4113
4114 4116 4117 411c 411f 4124 4125 412a
412d 4132 4133 4138 413b 413f 4140 4145
4148 414d 414e 4153 4154 4159 415b 415f
4162 4166 416a 416d 4171 4175 4178 417b
4180 4183 4186 4189 418a 418f 4190 1
4192 4197 419b 419f 41a2 41a7 41ac 41af
41b3 41b4 41b9 41ba 41bf 41c1 41c5 41c8
41cc 41d0 41d3 41d7 41db 41de 41e2 41e6
41e7 41ec 41ee 41f2 41f5 41f9 41fd 4200
4201 4203 4207 420a 420e 4211 4212 4217
421b 421f 4223 4227 422b 422f 4233 4237
423b 423f 4243 424f 4253 4257 425a 425e
4262 4264 4268 426c 4270 4273 4277 427b
427d 427e 4283 1 4287 428b 428f 4292
4296 429a 429e 42a2 42a5 42a9 42ad 42b0
42b3 42b8 42bb 42be 42c1 42c2 42c7 42c8
1 42ca 42cf 42d3 42d7 42da 42df 42e4
42e7 42eb 42ec 42f1 42f4 42f9 42fa 42ff
4302 4307 4308 430d 4310 4314 4315 431a
431d 4322 4323 4328 432b 432f 4333 4336
4339 433a 433c 433d 4342 4343 4348 434a
434e 4351 4353 4357 4359 4365 4367 4369
436a 436f 4373 4375 4381 4383 4387 438b
438e 4392 4395 4399 439c 439d 43a2 43a5
43a9 43ad 43b1 43b5 43b9 43c5 43c9 43cd
43d0 43d4 43d8 43da 43de 43e2 43e6 43e9
43ed 43f1 43f3 43f4 43f9 1 43fd 4401
4405 4408 440c 4410 4414 4418 441b 441f
4423 4426 4429 442e 4431 4434 4437 4438
443d 443e 1 4440 4445 4449 444d 4450
4455 445a 445d 4461 4462 4467 446a 446f
4470 4475 4478 447d 447e 4483 4486 448a
448b 4490 4493 4498 4499 449e 44a1 44a5
44a9 44ac 44af 44b0 44b2 44b3 44b8 44b9
44be 44c0 44c4 44c7 44c9 44cd 44cf 44db
44dd 44df 44e0 44e5 44e9 44eb 44f7 44f9
44fd 4501 4504 4508 450b 450f 4512 4513
4518 451b 451f 4523 4526 452a 452e 4531
4534 4539 453c 453f 4542 4543 4548 4549
1 454b 4550 4554 4558 455b 4560 4565
4568 456c 456d 4572 4575 457a 457b 4580
4583 4588 4589 458e 4591 4595 4599 459d
45a1 45a4 45a5 45a7 45ac 45ad 45af 45b0
45b5 45b8 45bd 45be 45c3 45c6 45cb 45cc
45d1 45d4 45d8 45d9 45de 45e1 45e6 45e7
45ec 45ed 45f2 45f4 45f8 45fb 45ff 4603
4606 460a 460e 4611 4614 4619 461c 461f
4622 4623 4628 4629 1 462b 4630 4634
4638 463b 4640 4645 4648 464c 464d 4652
4653 4658 465a 465e 4661 4665 4669 466c
4670 4674 4677 467b 467f 4680 4685 4687
468b 468e 4690 4694 4697 4699 469d 46a0
46a2 46a6 46a9 46ad 46b1 46b4 46b5 46b7
46ba 46bd 46be 46c3 46c7 46cb 46ce 46cf
46d1 46d4 46d7 46d8 1 46dd 46e2 46e6
46e9 46ed 46ef 46f3 46f6 46fa 46fe 4701
4702 4704 4708 470b 470f 4712 4713 4718
471c 471f 4720 4725 4728 472c 4731 4735
4739 473e 4742 4744 4748 474c 4750 4754
4758 475c 475e 4762 4766 4769 476d 4770
4774 4776 477a 477d 4780 4781 4786 478a
478f 4793 4797 479b 479f 47a3 47a7 47ab
47ae 47af 47b1 47b5 47b9 47bc 47bd 47bf
47c3 47c7 47ca 47cb 47cd 47d1 47d5 47d8
47d9 47db 47df 47e3 47e6 47e7 47e9 47ed
47f1 47f4 47f5 47f7 47fb 47ff 4802 4803
4805 4809 480d 4810 4811 4813 4817 481b
481f 4820 4822 4826 482a 482e 4831 4832
4834 4838 483b 483f 4842 4843 4848 484c
4850 4853 4857 485b 485e 4861 4866 4869
486c 486f 4870 4875 4876 1 4878 487d
4881 4885 4888 488d 4892 4895 4899 489a
489f 48a2 48a7 48a8 48ad 48b0 48b5 48b6
48bb 48be 48c2 48c6 48ca 48ce 48d1 48d2
48d4 48d9 48da 48dc 48dd 48e2 48e3 48e8
48ea 48ee 48f1 48f5 48f9 48fc 4900 4904
4907 490a 490f 4912 4915 4918 4919 491e
491f 1 4921 4926 492a 492e 4931 4936
493b 493e 4942 4943 4948 4949 494e 4950
4954 4957 495b 495f 4962 4966 496a 496d
4971 4972 4977 497b 497d 4981 4985 4988
4989 498b 498e 4991 4992 4997 499b 499f
49a2 49a6 49aa 49ad 49b0 49b5 49b8 49bb
49be 49bf 49c4 49c5 1 49c7 49cc 49d0
49d4 49d7 49dc 49e1 49e4 49e8 49e9 49ee
49f1 49f6 49f7 49fc 49ff 4a04 4a05 4a0a
4a0d 4a11 4a15 4a19 4a1a 4a1c 4a21 4a22
4a24 4a25 4a2a 4a2d 4a32 4a33 4a38 4a3b
4a40 4a41 4a46 4a49 4a4d 4a4e 4a53 4a56
4a5b 4a5c 4a61 4a62 4a67 4a69 4a6d 4a70
4a74 4a78 4a7b 4a7f 4a83 4a86 4a89 4a8e
4a91 4a94 4a97 4a98 4a9d 4a9e 1 4aa0
4aa5 4aa9 4aad 4ab0 4ab5 4aba 4abd 4ac1
4ac2 4ac7 4ac8 4acd 4acf 4ad3 4ad6 4ada
4ade 4ae1 4ae5 4ae9 4aec 4aed 4aef 4af3
4af7 4af8 4afd 4aff 4b03 4b07 4b0a 4b0c
4b10 4b14 4b18 4b1a 4b1e 4b22 4b25 4b29
4b2d 4b30 4b31 4b33 4b36 4b39 4b3a 4b3f
4b42 4b46 4b4a 4b4d 4b4e 4b50 4b53 4b56
4b57 4b5c 1 4b5f 4b64 4b68 4b6c 4b6f
4b70 4b72 4b75 4b78 4b79 4b7e 4b81 4b85
4b89 4b8c 4b8d 4b8f 4b92 4b95 4b96 4b9b
1 4b9e 4ba3 4ba7 4baa 4bae 4bb2 4bb4
4bb8 4bbc 4bbf 4bc0 4bc2 4bc5 4bc8 4bc9
4bce 4bd2 4bd5 4bd9 4bdd 4bdf 4be3 4be7
4beb 4bee 4bef 4bf1 4bf4 4bf7 4bf8 4bfd
4c00 4c04 4c08 4c0b 4c0c 4c0e 4c11 4c14
4c15 4c1a 1 4c1d 4c22 4c26 4c29 4c2d
4c31 4c33 4c37 4c3b 4c3f 4c42 4c43 4c45
4c48 4c4b 4c4c 4c51 4c55 4c58 4c5c 4c5e
4c62 4c66 4c69 4c6b 4c6f 4c72 4c74 4c78
4c7c 4c7f 4c83 4c87 4c8a 4c8b 4c8d 4c91
4c94 4c98 4c9b 4c9c 4ca1 4ca5 4ca8 4cac
4cb0 4cb4 4cb8 4cbc 4cc0 4cc4 4cc8 4ccc
4cd0 4cd3 4cd6 4cd7 4cd9 4cdd 4ce1 4ce2
4ce6 4cea 4cee 4cf1 4cf2 4cf4 4cf7 4cfa
4cfb 4d00 4d04 4d08 4d09 4d0e 4d12 4d16
4d19 4d1d 4d1f 4d23 4d27 4d2b 4d2f 4d33
4d3f 4d43 4d47 4d4a 4d4e 4d52 4d54 4d58
4d5c 4d60 4d63 4d67 4d6b 4d6d 4d6e 4d73
1 4d77 4d7b 4d7f 4d82 4d86 4d8a 4d8e
4d92 4d95 4d99 4d9d 4da0 4da3 4da8 4dab
4dae 4db1 4db2 4db7 4db8 1 4dba 4dbf
4dc3 4dc7 4dca 4dcf 4dd4 4dd7 4ddb 4ddc
4de1 4de4 4de9 4dea 4def 4df2 4df7 4df8
4dfd 4e00 4e04 4e05 4e0a 4e0d 4e12 4e13
4e18 4e1b 4e1f 4e23 4e26 4e29 4e2a 4e2c
4e2d 4e32 4e33 4e38 4e3a 4e3e 4e41 4e43
4e47 4e49 4e55 4e57 4e59 4e5a 4e5f 4e63
4e65 4e71 4e73 4e77 4e7b 4e7e 4e82 4e85
4e89 4e8c 4e8d 4e92 4e95 4e99 4e9d 4ea0
4ea1 4ea3 4ea6 4ea9 4eaa 4eaf 4eb3 4eb7
4ebb 4ebf 4ec3 4ec7 4ecb 4ecf 4ed3 4ed7
4edb 4ee7 4eeb 4eef 4ef2 4ef6 4efa 4efc
4f00 4f04 4f08 4f0b 4f0f 4f13 4f15 4f16
4f1b 1 4f1f 4f23 4f27 4f2a 4f2e 4f32
4f36 4f3a 4f3d 4f41 4f45 4f48 4f4b 4f50
4f53 4f56 4f59 4f5a 4f5f 4f60 1 4f62
4f67 4f6b 4f6f 4f72 4f77 4f7c 4f7f 4f83
4f84 4f89 4f8c 4f91 4f92 4f97 4f9a 4f9f
4fa0 4fa5 4fa8 4fac 4fad 4fb2 4fb5 4fba
4fbb 4fc0 4fc3 4fc7 4fcb 4fce 4fd1 4fd2
4fd4 4fd5 4fda 4fdb 4fe0 4fe2 4fe6 4fe9
4feb 4fef 4ff1 4ffd 4fff 5001 5002 5007
500b 500d 5019 501b 501f 5023 5026 502a
502d 5031 5034 5035 503a 503d 5041 5044
5048 504a 504e 5051 5053 5057 505a 505c
5060 5063 5067 5069 506d 5071 5074 5076
507a 507e 5081 5085 5088 508b 508c 5091
5095 5099 509d 50a1 50a5 50a9 50ad 50b1
50b5 50c1 50c5 50c9 50cc 50d0 50d4 50d6
50da 50de 50e2 50e5 50e9 50ed 50ef 50f0
50f5 1 50f9 50fd 5101 5104 5108 510c
5110 5114 5117 511b 511f 5122 5125 512a
512d 5130 5133 5134 5139 513a 1 513c
5141 5145 5149 514c 5151 5156 5159 515d
515e 5163 5166 516b 516c 5171 5174 5179
517a 517f 5182 5186 5187 518c 518f 5194
5195 519a 519d 51a1 51a5 51a8 51ab 51ac
51ae 51af 51b4 51b5 51ba 51bc 51c0 51c3
51c5 51c9 51cb 51d7 51d9 51db 51dc 51e1
51e5 51e7 51f3 51f5 51f9 51fd 5201 5205
5209 520d 5211 5215 5219 521d 5221 5225
5229 522d 5231 5235 5239 523d 5241 5245
5249 524d 5251 5255 5259 525d 5261 5265
5269 526d 5271 527d 5281 5285 5288 528c
5290 5292 5296 529a 529e 52a1 52a5 52a9
52ab 52ac 52b1 1 52b5 52b9 52bd 52c0
52c4 52c8 52cc 52d0 52d3 52d7 52db 52de
52e1 52e6 52e9 52ec 52ef 52f0 52f5 52f6
1 52f8 52fd 5301 5305 5308 530d 5312
5315 5319 531a 531f 5322 5327 5328 532d
5330 5335 5336 533b 533e 5342 5343 5348
534b 5350 5351 5356 5359 535d 5361 5364
5367 5368 536a 536b 5370 5371 5376 5378
537c 537f 5381 5385 5387 5393 5395 5397
5398 539d 53a1 53a3 53af 53b1 53b5 53b9
53bc 53c0 53c3 53c7 53ca 53cb 53d0 53d3
53d7 53db 53de 53e2 53e6 53e9 53ec 53f1
53f4 53f7 53fa 53fb 5400 5401 1 5403
5408 540c 5410 5413 5418 541d 5420 5424
5425 542a 542d 5432 5433 5438 543b 5440
5441 5446 5449 544d 5451 5455 5459 545c
545d 545f 5464 5465 5467 5468 546d 5470
5475 5476 547b 547e 5483 5484 5489 548c
5490 5491 5496 5499 549e 549f 54a4 54a5
54aa 54ac 54b0 54b3 54b7 54bb 54be 54c2
54c6 54c9 54cc 54d1 54d4 54d7 54da 54db
54e0 54e1 1 54e3 54e8 54ec 54f0 54f3
54f8 54fd 5500 5504 5505 550a 550b 5510
5512 5516 5519 551d 5521 5524 5528 552c
552f 5533 5537 5538 553d 553f 5543 5546
554a 554e 5551 5552 5554 5557 555a 555b
5560 5564 5569 556d 556f 5573 5578 557c
557e 5582 5586 5589 558d 5591 5595 5599
559d 55a1 55a5 55a9 55ad 55b1 55b5 55b9
55bd 55c1 55c5 55c9 55cd 55d1 55d5 55e1
55e5 55ea 55ee 55f1 55f2 55f7 55fb 55ff
5602 5605 5609 560b 560f 5613 5616 5619
561d 561f 5623 5627 562a 562c 5630 5634
5638 563b 563f 5643 5645 5646 564b 1
564f 5653 5657 565a 565e 5662 5666 566a
566d 5671 5675 5678 567b 5680 5683 5686
5689 568a 568f 5690 1 5692 5697 569b
569f 56a2 56a7 56ac 56af 56b3 56b4 56b9
56bc 56c1 56c2 56c7 56ca 56cf 56d0 56d5
56d8 56dc 56dd 56e2 56e5 56ea 56eb 56f0
56f3 56f7 56fb 56fe 5701 5702 5704 5705
570a 570b 5710 5712 5716 5719 571b 571f
5721 572d 572f 5731 5732 5737 573b 573d
5749 574b 574f 5751 5755 5759 575c 575f
5760 5762 5765 576a 576b 5770 5774 5778
577c 5780 5784 5788 578c 5790 5794 5798
57a4 57a8 57ad 57b1 57b4 57b5 57ba 57be
57c2 57c5 57c8 57cc 57ce 57d2 57d6 57d9
57dc 57e0 57e2 57e6 57ea 57ed 57ef 57f3
57f7 57fb 57fe 5802 5806 5808 5809 580e
1 5812 5816 581a 581d 5821 5825 5829
582d 5830 5834 5838 583b 583e 5843 5846
5849 584c 584d 5852 5853 1 5855 585a
585e 5862 5865 586a 586f 5872 5876 5877
587c 587f 5884 5885 588a 588d 5892 5893
5898 589b 589f 58a0 58a5 58a8 58ad 58ae
58b3 58b6 58ba 58be 58c1 58c4 58c5 58c7
58c8 58cd 58ce 58d3 58d5 58d9 58dc 58de
58e2 58e4 58f0 58f2 58f4 58f5 58fa 58fe
5900 590c 590e 5910 5914 5918 591b 591f
5923 5927 592b 592f 5933 5937 593b 593f
594b 594f 5954 5958 595b 595c 5961 5965
5969 596c 596f 5973 5975 5979 597d 5980
5983 5987 5989 598d 5991 5994 5996 599a
599e 59a2 59a5 59a9 59ad 59af 59b0 59b5
1 59b9 59bd 59c1 59c4 59c8 59cc 59d0
59d4 59d7 59db 59df 59e2 59e5 59ea 59ed
59f0 59f3 59f4 59f9 59fa 1 59fc 5a01
5a05 5a09 5a0c 5a11 5a16 5a19 5a1d 5a1e
5a23 5a26 5a2b 5a2c 5a31 5a34 5a39 5a3a
5a3f 5a42 5a46 5a47 5a4c 5a4f 5a54 5a55
5a5a 5a5d 5a61 5a65 5a68 5a6b 5a6c 5a6e
5a6f 5a74 5a75 5a7a 5a7c 5a80 5a83 5a85
5a89 5a8b 5a97 5a99 5a9b 5a9c 5aa1 5aa5
5aa7 5ab3 5ab5 5ab9 5abd 5ac1 5ac5 5ac9
5acd 5ad1 5ad5 5ad9 5ae5 5ae9 5aee 5af2
5af5 5af6 5afb 5aff 5b03 5b06 5b09 5b0d
5b0f 5b13 5b17 5b1a 5b1d 5b21 5b23 5b27
5b2b 5b2e 5b30 5b34 5b38 5b3c 5b3f 5b43
5b47 5b49 5b4a 5b4f 1 5b53 5b57 5b5b
5b5e 5b62 5b66 5b6a 5b6e 5b71 5b75 5b79
5b7c 5b7f 5b84 5b87 5b8a 5b8d 5b8e 5b93
5b94 1 5b96 5b9b 5b9f 5ba3 5ba6 5bab
5bb0 5bb3 5bb7 5bb8 5bbd 5bc0 5bc5 5bc6
5bcb 5bce 5bd3 5bd4 5bd9 5bdc 5be0 5be1
5be6 5be9 5bee 5bef 5bf4 5bf7 5bfb 5bff
5c02 5c05 5c06 5c08 5c09 5c0e 5c0f 5c14
5c16 5c1a 5c1d 5c1f 5c23 5c25 5c31 5c33
5c35 5c36 5c3b 5c3f 5c41 5c4d 5c4f 5c53
5c57 5c5b 5c5f 5c63 5c67 5c73 5c77 5c7c
5c80 5c83 5c84 5c89 5c8d 5c91 5c94 5c97
5c9b 5c9d 5ca1 5ca5 5ca8 5cab 5caf 5cb1
5cb5 5cb9 5cbc 5cbe 5cc2 5cc6 5cca 5ccd
5cd1 5cd5 5cd7 5cd8 5cdd 1 5ce1 5ce5
5ce9 5cec 5cf0 5cf4 5cf8 5cfc 5cff 5d03
5d07 5d0a 5d0d 5d12 5d15 5d18 5d1b 5d1c
5d21 5d22 1 5d24 5d29 5d2d 5d31 5d34
5d39 5d3e 5d41 5d45 5d46 5d4b 5d4e 5d53
5d54 5d59 5d5c 5d61 5d62 5d67 5d6a 5d6e
5d6f 5d74 5d77 5d7c 5d7d 5d82 5d85 5d89
5d8d 5d90 5d93 5d94 5d96 5d97 5d9c 5d9d
5da2 5da4 5da8 5dab 5dad 5db1 5db3 5dbf
5dc1 5dc3 5dc4 5dc9 5dcd 5dcf 5ddb 5ddd
5ddf 5de3 5de6 5dea 5dee 5def 5df4 5df8
5dfc 5e00 5e03 5e04 5e06 5e09 5e0c 5e0d
5e12 5e16 5e1a 5e1d 5e1e 5e20 5e24 5e27
5e2b 5e2e 5e2f 1 5e34 5e39 5e3d 5e41
5e44 5e45 5e47 5e4b 5e4e 5e52 5e55 5e56
1 5e5b 5e60 5e64 5e67 5e68 5e6d 5e70
5e74 5e78 5e7c 5e7e 5e82 5e85 5e87 5e8b
5e8e 5e92 5e96 5e99 5e9a 5e9c 5e9f 5ea2
5ea3 5ea8 5eac 5eb0 5eb4 5eb7 5ebb 5ebd
5ec1 5ec4 5ec8 5ecb 5ecc 5ed1 5ed4 5ed8
5edb 5edc 5ee1 1 5ee4 5ee9 5eed 5ef0
5ef1 5ef6 1 5ef9 5efe 5f02 5f05 5f06
5f0b 5f0e 1 5f11 5f16 5f1a 5f1d 5f21
5f23 5f27 5f2a 5f2e 5f33 5f37 5f3b 5f40
5f44 5f48 5f4c 5f4f 5f50 5f52 5f55 5f58
5f59 5f5e 5f62 5f66 5f69 5f6a 5f6c 5f70
5f73 5f77 5f7a 5f7b 5f80 5f84 5f89 5f8c
5f90 5f91 5f96 5f99 5f9e 5f9f 5fa4 5fa7
5fac 5fad 5fb2 5fb5 5fba 5fbb 5fc0 5fc3
5fc8 5fc9 5fce 5fd1 5fd5 5fd6 5fdb 5fde
5fe3 5fe4 5fe9 5fed 5ff1 5ff3 5ff7 5ffb
5ffe 5fff 6001 6004 6007 6008 600d 6011
6016 6019 601e 601f 6024 6027 602c 602d
6032 6035 6039 603a 603f 6042 6047 6048
604d 6050 6055 6056 605b 605e 6062 6063
6068 606b 6070 6071 6076 6079 607e 607f
6084 6088 608a 608e 6092 6097 609a 609f
60a0 60a5 60a8 60ad 60ae 60b3 60b6 60bb
60bc 60c1 60c4 60c8 60c9 60ce 60d1 60d6
60d7 60dc 60e0 60e2 60e6 60ea 60ed 60ef
60f3 60f7 60fa 60fb 60fd 6101 6104 6108
610b 610c 6111 6115 6119 611c 611d 611f
6123 6126 612a 612d 612e 1 6133 6138
613c 6140 6143 6144 6146 614a 614d 6151
6154 6155 1 615a 615f 6163 6167 616c
6171 6175 6179 617d 6180 6183 6187 618b
618e 618f 6194 6197 619b 619f 61a4 61a9
61ad 61b1 61b5 61b8 61bb 61bf 61c3 61c7
61cb 61cf 61d3 61d7 61db 61df 61e3 61e7
61eb 61ef 61f3 61f7 61fb 61ff 6203 6207
620b 620f 6213 6217 621b 6227 622b 6230
6234 6237 6238 623d 6241 6245 6248 624b
624f 6251 6255 6259 625c 625f 6263 6265
6269 626d 6270 6272 6276 627a 627e 6281
6285 6289 628b 628c 6291 1 6295 6299
629d 62a0 62a4 62a8 62ac 62b0 62b3 62b7
62bb 62be 62c1 62c6 62c9 62cc 62cf 62d0
62d5 62d6 1 62d8 62dd 62e1 62e5 62e8
62ed 62f2 62f5 62f9 62fa 62ff 6302 6307
6308 630d 6310 6315 6316 631b 631e 6322
6323 6328 632b 6330 6331 6336 6339 633d
6341 6344 6347 6348 634a 634b 6350 6351
6356 6358 635c 635f 6361 6365 6367 6373
6375 6377 6378 637d 6381 6383 638f 6391
6395 6399 639d 63a1 63a5 63a9 63ad 63b1
63b5 63b9 63bd 63c1 63c5 63c9 63cd 63d1
63d5 63d9 63dd 63e1 63e5 63f1 63f5 63fa
63fe 6401 6402 6407 640b 640f 6412 6415
6419 641b 641f 6423 6426 6429 642d 642f
6433 6437 643a 643c 6440 6444 6448 644b
644f 6453 6455 6456 645b 1 645f 6463
6467 646a 646e 6472 6476 647a 647d 6481
6485 6488 648b 6490 6493 6496 6499 649a
649f 64a0 1 64a2 64a7 64ab 64af 64b2
64b7 64bc 64bf 64c3 64c4 64c9 64cc 64d1
64d2 64d7 64da 64df 64e0 64e5 64e8 64ec
64ed 64f2 64f5 64fa 64fb 6500 6503 6507
650b 650e 6511 6512 6514 6515 651a 651b
6520 6522 6526 6529 652b 652f 6531 653d
653f 6541 6542 6547 654b 654d 6559 655b
655d 6561 6565 6569 656b 656f 6573 6576
6578 657c 657f 6583 6586 6587 658c 658f
6593 6597 659b 659f 65a0 65a2 65a5 65a6
65a8 65ab 65ae 65af 65b4 65b8 65bc 65c0
65c4 65c5 65c7 65ca 65cb 65cd 65d0 65d1
65d3 65d6 65d9 65da 65df 65e0 65e2 65e6
65ea 65ee 65f2 65f3 65f5 65f8 65f9 65fb
65fe 65ff 6601 6603 6607 660b 660f 6612
6616 661a 661e 6622 6625 6629 662a 662f
6631 6635 6639 663d 6641 6645 6648 664b
664c 6651 6654 6655 6657 6658 665a 665d
665e 6660 6663 6666 6667 666c 6670 6674
6678 667c 6680 6683 6686 6687 668c 668f
6690 6692 6693 6695 6698 6699 669b 669e
66a1 66a2 1 66a7 66ac 66b0 66b6 66b8
66bc 66bf 66c3 66c7 66ca 66cd 66ce 66d3
66d7 66d9 66dd 66e4 66e8 66ec 66ef 66f0
66f5 66f9 66fd 6700 6704 6708 670b 670e
6713 6716 6719 671c 671d 6722 6723 1
6725 672a 672e 6732 6735 673a 673f 6742
6746 6747 674c 674f 6754 6755 675a 675d
6762 6763 6768 676b 676f 6773 6777 677b
677e 677f 6781 6786 6787 6789 678a 678f
6790 6795 6797 679b 679e 67a2 67a6 67a9
67ad 67b1 67b4 67b7 67bc 67bf 67c2 67c5
67c6 67cb 67cc 1 67ce 67d3 67d7 67db
67de 67e3 67e8 67eb 67ef 67f0 67f5 67f6
67fb 67fd 6801 6804 6808 680c 680f 6813
6817 681a 681e 681f 6824 6826 682a 682d
6831 6835 6839 683d 6849 684d 6851 6854
6858 685c 685e 6862 6866 686a 686d 6871
6875 6877 6878 687d 1 6881 6885 6889
688c 6890 6894 6898 689c 689f 68a3 68a7
68aa 68ad 68b2 68b5 68b8 68bb 68bc 68c1
68c2 1 68c4 68c9 68cd 68d1 68d4 68d9
68de 68e1 68e5 68e6 68eb 68ee 68f3 68f4
68f9 68fc 6901 6902 6907 690a 690e 690f
6914 6917 691c 691d 6922 6925 6929 692d
6930 6933 6934 6936 6937 693c 693d 6942
6944 6948 694b 694d 6951 6953 695f 6961
6963 6964 6969 696d 696f 697b 697d 697f
6983 6986 698a 698e 6991 6992 6994 6998
699b 699f 69a2 69a3 69a8 69ac 69b1 69b5
69b9 69be 69c2 69c6 69ca 69cd 69d1 69d2
69d7 69db 69df 69e3 69e6 69eb 69ec 69f1
69f5 69f9 69fb 69ff 6a03 6a06 6a07 6a09
6a0d 6a10 6a14 6a17 6a18 6a1d 6a21 6a26
6a2a 6a2e 6a32 6a35 6a39 6a3a 6a3f 6a43
6a47 6a4b 6a4e 6a53 6a54 6a59 6a5d 6a61
6a66 6a6a 6a6c 6a70 6a74 6a78 6a7b 6a7c
6a7e 6a82 6a85 6a89 6a8c 6a8d 6a92 6a96
6a9b 6a9f 6aa3 6aa8 6aac 6ab0 6ab3 6ab4
6ab9 6abc 6ac0 6ac4 6ac7 6acc 6acd 6ad2
6ad6 6ada 6ade 6ae1 6ae5 6ae6 6aeb 6aef
6af3 6af7 6afa 6aff 6b00 6b05 6b09 6b0b
6b0f 6b12 6b16 6b1a 6b1d 6b22 6b23 6b28
6b2c 6b30 6b33 6b34 6b39 6b3c 6b40 6b44
6b47 6b4c 6b4d 6b52 6b56 6b5a 6b5e 6b61
6b65 6b66 6b6b 6b6f 6b73 6b77 6b7a 6b7f
6b80 6b85 6b89 6b8b 6b8f 6b92 6b94 6b98
6b9b 6b9c 6ba1 6ba4 6ba8 6bac 6baf 6bb4
6bb5 6bba 6bbe 6bc2 6bc7 6bcb 6bcd 6bd1
6bd5 6bd8 6bdd 6bde 6be3 6be6 6beb 6bec
6bf1 6bf5 6bf9 6bfe 6c02 6c06 6c09 6c0a
6c0f 6c12 6c16 6c1a 6c1d 6c22 6c23 6c28
6c2c 6c30 6c34 6c37 6c3b 6c3c 6c41 6c45
6c49 6c4d 6c50 6c55 6c56 6c5b 6c5f 6c61
6c65 6c68 6c6c 6c70 6c73 6c78 6c79 6c7e
6c81 6c86 6c87 6c8c 6c8f 6c94 6c95 6c9a
6c9d 6ca2 6ca3 6ca8 6cab 6cb0 6cb1 6cb6
6cb9 6cbe 6cbf 6cc4 6cc8 6cca 6cce 6cd2
6cd5 6cd9 6cdc 6cdd 6ce2 6ce5 6ce9 6ced
6cf0 6cf5 6cf6 6cfb 6cfe 6d02 6d03 6d08
6d0b 6d10 6d11 6d16 6d1a 6d1c 6d20 6d23
6d25 6d29 6d2d 6d30 6d32 6d36 6d3a 6d3d
6d41 6d45 6d48 6d49 6d4b 6d4f 6d52 6d56
6d59 6d5a 6d5f 6d63 6d66 6d67 6d6c 6d6f
6d73 6d77 6d7b 6d7f 6d83 6d8f 6d93 6d97
6d9a 6d9e 6da2 6da4 6da8 6dac 6db0 6db3
6db7 6dbb 6dbd 6dbe 6dc3 1 6dc7 6dcb
6dcf 6dd2 6dd6 6dda 6dde 6de2 6de5 6de9
6ded 6df0 6df3 6df8 6dfb 6dfe 6e01 6e02
6e07 6e08 1 6e0a 6e0f 6e13 6e17 6e1a
6e1f 6e24 6e27 6e2b 6e2c 6e31 6e34 6e39
6e3a 6e3f 6e42 6e47 6e48 6e4d 6e50 6e54
6e55 6e5a 6e5d 6e62 6e63 6e68 6e6b 6e6f
6e73 6e76 6e79 6e7a 6e7c 6e7d 6e82 6e83
6e88 6e8a 6e8e 6e91 6e93 6e97 6e99 6ea5
6ea7 6ea9 6eaa 6eaf 6eb3 6eb5 6ec1 6ec3
6ec7 6ecb 6ece 6ed2 6ed5 6ed9 6edc 6edd
6ee2 6ee5 6ee9 6eed 6ef1 6ef5 6ef6 6ef8
6efb 6efc 6efe 6f02 6f06 6f0a 6f0d 6f12
6f13 6f18 6f1c 6f20 6f24 6f27 6f2c 6f2d
6f32 6f35 6f3a 6f3b 6f40 6f43 6f47 6f48
6f4d 6f50 6f55 6f56 6f5b 6f5f 6f61 6f65
6f69 6f6c 6f70 6f74 6f77 6f7a 6f7f 6f82
6f85 6f88 6f89 6f8e 6f8f 1 6f91 6f96
6f9a 6f9e 6fa1 6fa6 6fab 6fae 6fb2 6fb3
6fb8 6fbb 6fc0 6fc1 6fc6 6fc9 6fce 6fcf
6fd4 6fd7 6fdb 6fdf 6fe3 6fe7 6fea 6feb
6fed 6ff2 6ff3 6ff5 6ff6 6ffb 6ffe 7003
7004 7009 700c 7011 7012 7017 701a 701e
701f 7024 7027 702c 702d 7032 7033 7038
703a 703e 7041 7045 7049 704c 7050 7054
7057 705a 705f 7062 7065 7068 7069 706e
706f 1 7071 7076 707a 707e 7081 7086
708b 708e 7092 7093 7098 7099 709e 70a0
70a4 70a7 70ab 70af 70b2 70b6 70ba 70bd
70c1 70c5 70c6 70cb 70cd 70d1 70d5 70d8
70da 70de 70e1 70e5 70e8 70e9 70ee 70f1
70f5 70f9 70fd 7101 7105 7111 7115 7119
711c 7120 7124 7126 712a 712e 7132 7135
7139 713d 713f 7140 7145 1 7149 714d
7151 7154 7158 715c 7160 7164 7167 716b
716f 7172 7175 717a 717d 7180 7183 7184
7189 718a 1 718c 7191 7195 7199 719c
71a1 71a6 71a9 71ad 71ae 71b3 71b6 71bb
71bc 71c1 71c4 71c9 71ca 71cf 71d2 71d6
71d7 71dc 71df 71e4 71e5 71ea 71ed 71f1
71f5 71f8 71fb 71fc 71fe 71ff 7204 7205
720a 720c 7210 7213 7215 7219 721b 7227
7229 722b 722c 7231 7235 7237 7243 7245
7249 724d 7250 7254 7257 725b 725e 725f
7264 7267 726b 726f 7272 7277 7278 727d
7281 7285 7289 728c 7291 7292 7297 729a
729f 72a0 72a5 72a8 72ac 72ad 72b2 72b5
72ba 72bb 72c0 72c4 72c6 72ca 72ce 72d1
72d5 72d9 72dc 72df 72e4 72e7 72ea 72ed
72ee 72f3 72f4 1 72f6 72fb 72ff 7303
7306 730b 7310 7313 7317 7318 731d 7320
7325 7326 732b 732e 7333 7334 7339 733c
7340 7344 7348 734c 734f 7350 7352 7357
7358 735a 735b 7360 7363 7368 7369 736e
7371 7376 7377 737c 737f 7383 7384 7389
738c 7391 7392 7397 7398 739d 739f 73a3
73a6 73aa 73ae 73b1 73b5 73b9 73bc 73bf
73c4 73c7 73ca 73cd 73ce 73d3 73d4 1
73d6 73db 73df 73e3 73e6 73eb 73f0 73f3
73f7 73f8 73fd 73fe 7403 7405 7409 740c
7410 7414 7417 741b 741f 7422 7426 742a
742b 7430 7432 7436 743a 743d 743f 7443
7446 744a 744d 744e 7453 7456 745a 745e
7462 7466 746a 7476 747a 747e 7481 7485
7489 748b 748f 7493 7497 749a 749e 74a2
74a4 74a5 74aa 1 74ae 74b2 74b6 74b9
74bd 74c1 74c5 74c9 74cc 74d0 74d4 74d7
74da 74df 74e2 74e5 74e8 74e9 74ee 74ef
1 74f1 74f6 74fa 74fe 7501 7506 750b
750e 7512 7513 7518 751b 7520 7521 7526
7529 752e 752f 7534 7537 753b 753c 7541
7544 7549 754a 754f 7552 7556 755a 755d
7560 7561 7563 7564 7569 756a 756f 7571
7575 7578 757a 757e 7580 758c 758e 7590
7591 7596 759a 759c 75a8 75aa 75ae 75b2
75b5 75b9 75bc 75c0 75c3 75c4 75c9 75cc
75d0 75d4 75d7 75dc 75dd 75e2 75e6 75ea
75ee 75f1 75f6 75f7 75fc 75ff 7604 7605
760a 760d 7611 7612 7617 761a 761f 7620
7625 7629 762b 762f 7633 7636 763a 763e
7641 7644 7649 764c 764f 7652 7653 7658
7659 1 765b 7660 7664 7668 766b 7670
7675 7678 767c 767d 7682 7685 768a 768b
7690 7693 7698 7699 769e 76a1 76a5 76a9
76ad 76b1 76b4 76b5 76b7 76bc 76bd 76bf
76c0 76c5 76c8 76cd 76ce 76d3 76d6 76db
76dc 76e1 76e4 76e8 76e9 76ee 76f1 76f6
76f7 76fc 76fd 7702 7704 7708 770b 770f
7713 7716 771a 771e 7721 7724 7729 772c
772f 7732 7733 7738 7739 1 773b 7740
7744 7748 774b 7750 7755 7758 775c 775d
7762 7763 7768 776a 776e 7771 7775 7779
777c 7780 7784 7787 778b 778f 7790 7795
7797 779b 779f 77a2 77a4 77a8 77ab 77af
77b3 77b6 77b7 77b9 77bc 77bf 77c0 77c5
77c9 77cd 77d0 77d1 77d3 77d6 77d9 77da
1 77df 77e4 77e8 77ec 77ef 77f4 77f5
77fa 77fd 7801 7805 7809 780c 780d 780f
7810 7812 7813 7818 781b 7820 7821 7826
782a 782c 7830 7833 7835 7839 783c 7840
7843 7844 7849 784c 7850 7854 7858 785c
785d 785f 7862 7863 7865 7869 786d 7870
7874 7878 787c 7880 7883 7887 7888 788d
788f 7893 7897 789b 789e 78a1 78a4 78a8
78ab 78ae 78af 78b4 78b8 78b9 78bb 78bf
78c3 78c7 78cb 78cf 78d3 78df 78e3 78e7
78ea 78ee 78f2 78f4 78f8 78fc 7900 7903
7907 790b 790d 790e 7913 1 7917 791b
791f 7922 7926 792a 792e 7932 7935 7939
793d 7940 7943 7948 794b 794e 7951 7952
7957 7958 1 795a 795f 7963 7967 796a
796f 7974 7977 797b 797c 7981 7984 7989
798a 798f 7992 7997 7998 799d 79a0 79a4
79a5 79aa 79ad 79b2 79b3 79b8 79bb 79bf
79c3 79c6 79c9 79ca 79cc 79cd 79d2 79d3
79d8 79da 79de 79e1 79e3 79e7 79e9 79f5
79f7 79f9 79fa 79ff 7a03 7a05 7a11 7a13
7a17 7a1b 7a1e 7a22 7a25 7a29 7a2c 7a2d
7a32 7a35 7a39 7a3d 7a40 7a44 7a48 7a4b
7a4e 7a53 7a56 7a59 7a5c 7a5d 7a62 7a63
1 7a65 7a6a 7a6e 7a72 7a75 7a7a 7a7f
7a82 7a86 7a87 7a8c 7a8f 7a94 7a95 7a9a
7a9d 7aa2 7aa3 7aa8 7aab 7aaf 7ab3 7ab7
7abb 7abe 7abf 7ac1 7ac6 7ac7 7ac9 7aca
7acf 7ad2 7ad7 7ad8 7add 7ae0 7ae5 7ae6
7aeb 7aee 7af2 7af3 7af8 7afb 7b00 7b01
7b06 7b07 7b0c 7b0e 7b12 7b15 7b19 7b1d
7b20 7b24 7b28 7b2b 7b2e 7b33 7b36 7b39
7b3c 7b3d 7b42 7b43 1 7b45 7b4a 7b4e
7b52 7b55 7b5a 7b5f 7b62 7b66 7b67 7b6c
7b6d 7b72 7b74 7b78 7b7b 7b7f 7b83 7b86
7b8a 7b8e 7b91 7b95 7b99 7b9a 7b9f 7ba1
7ba5 7ba8 7bac 7bb0 7bb3 7bb8 7bb9 7bbe
7bc1 7bc6 7bc7 7bcc 7bcf 7bd4 7bd5 7bda
7bdd 7be2 7be3 7be8 7beb 7bef 7bf0 7bf5
7bf8 7bfd 7bfe 7c03 7c07 7c0b 7c0f 7c12
7c13 7c15 7c19 7c1c 7c20 7c23 7c24 7c29
7c2d 7c31 7c34 7c39 7c3a 7c3f 7c43 7c45
7c49 7c4d 7c50 7c55 7c56 7c5b 7c5f 7c61
7c65 7c69 7c6c 7c70 7c74 7c77 7c78 7c7a
7c7e 7c81 7c85 7c88 7c89 7c8e 7c92 7c98
7c9a 7c9e 7ca1 7ca5 7ca9 7cac 7caf 7cb0
7cb5 7cb9 7cbb 7cbf 7cc6 7cc8 7ccc 7ccf
7cd3 7cd7 7cda 7cdb 7cdd 7ce1 7ce4 7ce8
7ceb 7cec 7cf1 7cf5 7cf9 7cfd 7d01 7d02
7d04 7d07 7d08 7d0a 7d0e 7d12 7d15 7d19
7d1d 7d21 7d25 7d28 7d2c 7d2d 7d32 7d34
7d38 7d3c 7d40 7d43 7d46 7d49 7d4d 7d50
7d53 7d54 7d59 7d5d 7d5e 7d60 7d64 7d68
7d6c 7d70 7d74 7d78 7d84 7d88 7d8c 7d8f
7d93 7d97 7d99 7d9d 7da1 7da5 7da8 7dac
7db0 7db2 7db3 7db8 1 7dbc 7dc0 7dc4
7dc7 7dcb 7dcf 7dd3 7dd7 7dda 7dde 7de2
7de5 7de8 7ded 7df0 7df3 7df6 7df7 7dfc
7dfd 1 7dff 7e04 7e08 7e0c 7e0f 7e14
7e19 7e1c 7e20 7e21 7e26 7e29 7e2e 7e2f
7e34 7e37 7e3c 7e3d 7e42 7e45 7e49 7e4a
7e4f 7e52 7e57 7e58 7e5d 7e60 7e64 7e68
7e6b 7e6e 7e6f 7e71 7e72 7e77 7e78 7e7d
7e7f 7e83 7e86 7e88 7e8c 7e8e 7e9a 7e9c
7e9e 7e9f 7ea4 7ea8 7eaa 7eb6 7eb8 7ebc
7ec0 7ec3 7ec7 7eca 7ece 7ed1 7ed2 7ed7
7eda 7ede 7ee2 7ee5 7ee9 7eed 7ef0 7ef3
7ef8 7efb 7efe 7f01 7f02 7f07 7f08 1
7f0a 7f0f 7f13 7f17 7f1a 7f1f 7f24 7f27
7f2b 7f2c 7f31 7f34 7f39 7f3a 7f3f 7f42
7f47 7f48 7f4d 7f50 7f54 7f58 7f5c 7f60
7f63 7f64 7f66 7f6b 7f6c 7f6e 7f6f 7f74
7f77 7f7c 7f7d 7f82 7f85 7f8a 7f8b 7f90
7f93 7f97 7f98 7f9d 7fa0 7fa5 7fa6 7fab
7fac 7fb1 7fb3 7fb7 7fba 7fbe 7fc2 7fc5
7fc9 7fcd 7fd0 7fd3 7fd8 7fdb 7fde 7fe1
7fe2 7fe7 7fe8 1 7fea 7fef 7ff3 7ff7
7ffa 7fff 8004 8007 800b 800c 8011 8012
8017 8019 801d 8020 8024 8028 802b 802f
8033 8036 803a 803e 803f 8044 8046 804a
804d 8051 8055 8058 805d 805e 8063 8066
806b 806c 8071 8074 8079 807a 807f 8082
8087 8088 808d 8090 8094 8095 809a 809d
80a2 80a3 80a8 80ab 80b0 80b1 80b6 80ba
80be 80c2 80c5 80c8 80c9 80ce 80d2 80d4
80d8 80df 80e1 80e5 80e8 80ec 80f0 80f3
80f7 80f8 80fd 8101 8103 8107 810b 810e
8112 8116 8119 811e 811f 8124 8128 812c
8130 8134 8137 813b 813f 8143 8147 814c
8151 8155 8159 815e 8162 8166 8169 816c
8170 8172 8176 817a 817d 8180 8184 8186
818a 818e 8191 8195 8199 819c 81a0 81a3
81a7 81aa 81ab 81b0 81b3 81b7 81bd 81bf
81c3 81c6 81ca 81ce 81d1 81d2 81d4 81d7
81da 81db 81e0 81e4 81e8 81eb 81ec 81ee
81f2 81f5 81f9 81fc 81fd 1 8202 8207
820b 820f 8212 8213 8215 8219 821c 8220
8223 8224 1 8229 822e 8232 8236 8239
823a 823c 8240 8243 8247 824a 824b 8250
8254 8257 8258 825d 1 8260 8265 1
8268 826d 8271 8275 827a 827e 827f 8283
8287 828c 8291 8295 8299 829e 82a2 82a6
82a9 82ac 82b0 82b2 82b6 82ba 82bd 82c0
82c4 82c6 82ca 82ce 82d1 82d5 82d9 82dc
82e0 82e3 82e7 82ea 82eb 82f0 82f3 82f7
82fb 8300 8302 8306 830a 830d 8311 8315
8317 831b 831f 8323 8326 832a 832e 8330
8331 8336 1 833a 833e 8342 8345 8349
834d 8351 8355 8358 835c 8360 8363 8366
836b 836e 8371 8374 8375 837a 837b 1
837d 8382 8386 838a 838d 8392 8397 839a
839e 839f 83a4 83a7 83ac 83ad 83b2 83b5
83ba 83bb 83c0 83c3 83c7 83c8 83cd 83d0
83d5 83d6 83db 83de 83e2 83e6 83e9 83ec
83ed 83ef 83f0 83f5 83f6 83fb 83fd 8401
8404 8406 840a 840c 8418 841a 841c 841d
8422 8426 8428 8434 8436 843a 8440 8442
8446 844a 844f 8451 8455 8459 845c 8460
8464 8466 846a 846e 8472 8475 8479 847d
847f 8480 8485 1 8489 848d 8491 8494
8498 849c 84a0 84a4 84a7 84ab 84af 84b2
84b5 84ba 84bd 84c0 84c3 84c4 84c9 84ca
1 84cc 84d1 84d5 84d9 84dc 84e1 84e6
84e9 84ed 84ee 84f3 84f6 84fb 84fc 8501
8504 8509 850a 850f 8512 8516 8517 851c
851f 8524 8525 852a 852d 8531 8535 8538
853b 853c 853e 853f 8544 8545 854a 854c
8550 8553 8555 8559 855b 8567 8569 856b
856c 8571 8575 8577 8583 8585 8587 858b
858f 8592 8594 8598 859b 859f 85a3 85a7
85ab 85af 85b3 85b7 85bb 85be 85bf 85c1
85c4 85c7 85c8 85cd 85d1 85d5 85d8 85db
85dc 85de 85e1 85e6 85e7 85ec 85f0 85f4
85f5 85fa 85fe 8602 8605 8609 860b 860f
8613 8617 861a 861b 861d 861e 8620 8623
8626 8627 862c 8630 8634 8638 863c 8640
8644 8648 864c 8650 865c 8660 8664 8667
866b 866f 8671 8675 8679 867d 8680 8684
8688 868a 868b 8690 1 8694 8698 869c
869f 86a3 86a7 86ab 86af 86b2 86b6 86ba
86bd 86c0 86c5 86c8 86cb 86ce 86cf 86d4
86d5 1 86d7 86dc 86e0 86e4 86e7 86ec
86f1 86f4 86f8 86f9 86fe 8701 8706 8707
870c 870f 8714 8715 871a 871d 8721 8722
8727 872a 872f 8730 8735 8738 873c 8740
8743 8746 8747 8749 874a 874f 8750 8755
8757 875b 875e 8760 8764 8766 8772 8774
8776 8777 877c 8780 8782 878e 8790 8794
8798 879b 879c 879e 87a1 87a4 87a5 87aa
87ae 87b2 87b7 87b9 87bd 87c1 87c4 87c8
87cc 87ce 87d2 87d6 87da 87dd 87e1 87e5
87e7 87e8 87ed 1 87f1 87f5 87f9 87fc
8800 8804 8808 880c 880f 8813 8817 881a
881d 8822 8825 8828 882b 882c 8831 8832
1 8834 8839 883d 8841 8844 8849 884e
8851 8855 8856 885b 885e 8863 8864 8869
886c 8871 8872 8877 887a 887e 887f 8884
8887 888c 888d 8892 8895 8899 889d 88a0
88a3 88a4 88a6 88a7 88ac 88ad 88b2 88b4
88b8 88bb 88bd 88c1 88c3 88cf 88d1 88d3
88d4 88d9 88dd 88df 88eb 88ed 88f1 88f5
88f8 88fc 8900 8903 8906 890b 890e 8911
8914 8915 891a 891b 1 891d 8922 8926
892a 892d 8932 8937 893a 893e 893f 8944
8947 894c 894d 8952 8955 895a 895b 8960
8963 8967 896b 896f 8973 8976 8977 8979
897e 897f 8981 8982 8987 898a 898f 8990
8995 8998 899d 899e 89a3 89a6 89aa 89ab
89b0 89b3 89b8 89b9 89be 89bf 89c4 89c6
89ca 89cd 89d1 89d5 89d8 89dc 89e0 89e3
89e6 89eb 89ee 89f1 89f4 89f5 89fa 89fb
1 89fd 8a02 8a06 8a0a 8a0d 8a12 8a17
8a1a 8a1e 8a1f 8a24 8a25 8a2a 8a2c 8a30
8a33 8a37 8a3b 8a3e 8a42 8a46 8a49 8a4d
8a51 8a52 8a57 8a59 8a5d 8a60 8a62 8a66
8a69 8a6b 8a6f 8a73 8a76 8a7a 8a7e 8a82
8a86 8a8a 8a8e 8a92 8a96 8a9a 8a9e 8aa2
8aa6 8aaa 8aae 8ab2 8ab6 8aba 8abe 8ac2
8ac6 8aca 8ace 8ad2 8ad6 8ada 8ade 8ae2
8ae6 8aea 8aee 8af2 8af6 8af9 8afa 8afc
8aff 8b02 8b05 8b06 8b08 8b0c 8b10 8b13
8b16 8b17 8b1c 8b20 8b24 8b29 8b2b 8b2f
8b33 8b36 8b3a 8b3e 8b40 8b44 8b48 8b4c
8b4f 8b53 8b57 8b59 8b5a 8b5f 1 8b63
8b67 8b6b 8b6e 8b72 8b76 8b7a 8b7e 8b81
8b85 8b89 8b8c 8b8f 8b94 8b97 8b9a 8b9d
8b9e 8ba3 8ba4 1 8ba6 8bab 8baf 8bb3
8bb6 8bbb 8bc0 8bc3 8bc7 8bc8 8bcd 8bd0
8bd5 8bd6 8bdb 8bde 8be3 8be4 8be9 8bec
8bf0 8bf1 8bf6 8bf9 8bfe 8bff 8c04 8c07
8c0b 8c0f 8c12 8c15 8c16 8c18 8c19 8c1e
8c1f 8c24 8c26 8c2a 8c2d 8c2f 8c33 8c35
8c41 8c43 8c45 8c46 8c4b 8c4f 8c51 8c5d
8c5f 8c63 8c67 8c6a 8c6e 8c71 8c72 8c77
8c7b 8c7f 8c83 8c86 8c8a 8c8c 8c90 8c93
8c97 8c9b 8c9e 8ca2 8ca6 8ca9 8cac 8cb1
8cb4 8cb7 8cba 8cbb 8cc0 8cc1 1 8cc3
8cc8 8ccc 8cd0 8cd3 8cd8 8cdd 8ce0 8ce4
8ce5 8cea 8ced 8cf2 8cf3 8cf8 8cfb 8d00
8d01 8d06 8d09 8d0d 8d11 8d15 8d16 8d18
8d1d 8d1e 8d20 8d21 8d26 8d29 8d2e 8d2f
8d34 8d37 8d3c 8d3d 8d42 8d45 8d49 8d4a
8d4f 8d52 8d57 8d58 8d5d 8d5e 8d63 8d65
8d69 8d6c 8d70 8d74 8d77 8d7b 8d7f 8d82
8d85 8d8a 8d8d 8d90 8d93 8d94 8d99 8d9a
1 8d9c 8da1 8da5 8da9 8dac 8db1 8db6
8db9 8dbd 8dbe 8dc3 8dc4 8dc9 8dcb 8dcf
8dd2 8dd6 8dda 8ddd 8de1 8de5 8de9 8dea
8def 8df3 8df5 8df9 8dfd 8e00 8e03 8e04
8e06 8e09 8e0e 8e0f 8e14 8e17 8e1b 8e1f
8e22 8e25 8e26 8e28 8e2b 8e30 8e31 8e36
1 8e39 8e3e 8e42 8e46 8e4a 8e4e 8e52
8e5e 8e62 8e67 8e6b 8e6e 8e6f 8e74 8e78
8e7c 8e7f 8e82 8e86 8e88 8e8c 8e90 8e93
8e96 8e9a 8e9c 8ea0 8ea4 8ea7 8ea9 8ead
8eb1 8eb5 8eb8 8ebc 8ec0 8ec2 8ec3 8ec8
1 8ecc 8ed0 8ed4 8ed7 8edb 8edf 8ee3
8ee7 8eea 8eee 8ef2 8ef5 8ef8 8efd 8f00
8f03 8f06 8f07 8f0c 8f0d 1 8f0f 8f14
8f18 8f1c 8f1f 8f24 8f29 8f2c 8f30 8f31
8f36 8f39 8f3e 8f3f 8f44 8f47 8f4c 8f4d
8f52 8f55 8f59 8f5a 8f5f 8f62 8f67 8f68
8f6d 8f70 8f74 8f78 8f7b 8f7e 8f7f 8f81
8f82 8f87 8f88 8f8d 8f8f 8f93 8f96 8f98
8f9c 8f9e 8faa 8fac 8fae 8faf 8fb4 8fb8
8fba 8fc6 8fc8 8fca 8fce 8fd2 8fd5 8fd9
8fdf 8fe1 8fe5 8fe9 8fec 8fed 8fef 8ff3
8ff6 8ffa 8ffd 8ffe 9003 9007 900b 900f
9013 9017 901b 901d 9021 9024 9028 902c
9030 9034 9038 903c 9040 9044 9048 904c
9050 9054 9058 905c 9060 9064 9068 906c
9070 9074 9078 907c 9080 9084 9088 908c
9090 9094 9098 909c 909f 90a2 90a5 90a8
90a9 90ab 90af 90b3 90b6 90b9 90ba 90bf
90c3 90c7 90ca 90cd 90ce 90d3 90d7 90d9
90dd 90e0 90e4 90e8 90eb 90ef 90f2 90f3
90f5 90f6 90fb 90ff 9105 9107 910b 910e
9110 9114 9118 911b 9123 9125 9126 9129
912b 912f 9136 913a 913e 9143 9145 9149
914d 9150 9154 9158 915a 915e 9162 9166
9169 916d 9171 9173 9174 9179 1 917d
9181 9185 9188 918c 9190 9194 9198 919b
919f 91a3 91a6 91a9 91ae 91b1 91b4 91b7
91b8 91bd 91be 1 91c0 91c5 91c9 91cd
91d0 91d5 91da 91dd 91e1 91e2 91e7 91ea
91ef 91f0 91f5 91f8 91fd 91fe 9203 9206
920a 920b 9210 9213 9218 9219 921e 9221
9225 9229 922c 922f 9230 9232 9233 9238
9239 923e 9240 9244 9247 9249 924d 924f
925b 925d 925f 9260 9265 9269 926b 9277
9279 927d 9281 9284 9288 928b 928c 9291
9294 9298 929b 929e 929f 92a4 92a7 92ab
92af 92b2 92b3 92b5 92b8 92bb 92bc 92c1
1 92c4 92c9 1 92cc 92d1 92d5 92d9
92dc 92e0 92e4 92e7 92ea 92ef 92f2 92f5
92f8 92f9 92fe 92ff 1 9301 9306 930a
930e 9311 9316 931b 931e 9322 9323 9328
932b 9330 9331 9336 9339 933e 933f 9344
9347 934b 934f 9353 9354 9356 935b 935c
935e 935f 9364 9367 936c 936d 9372 9375
937a 937b 9380 9383 9387 9388 938d 9390
9395 9396 939b 939c 93a1 93a3 93a7 93aa
93ae 93b2 93b5 93b9 93bd 93c0 93c3 93c8
93cb 93ce 93d1 93d2 93d7 93d8 1 93da
93df 93e3 93e7 93ea 93ef 93f4 93f7 93fb
93fc 9401 9402 9407 9409 940d 9410 9414
9418 941b 941f 9423 9427 9428 942d 942f
9433 9436 943a 943d 9440 9441 9446 944a
944f 9453 9457 945c 9460 9464 9469 946d
9471 9475 947a 947e 947f 9483 9486 948a
948e 9492 9496 949a 949e 94a2 94a6 94aa
94ae 94b2 94b7 94bc 94c0 94c4 94c8 94cd
94d1 94d5 94d8 94db 94df 94e1 94e5 94e9
94ec 94ef 94f3 94f5 94f9 94fd 9500 9504
9508 950b 950f 9512 9516 9519 951a 951f
9522 9526 952c 952e 9532 9535 9539 953c
953d 9542 9545 9549 954d 9550 9554 9558
9559 955b 955e 9561 9562 1 9567 956c
9570 9574 9578 9579 957b 957e 957f 9581
9584 9587 9588 1 958d 9592 9596 959a
959e 95a0 95a4 95a7 95ab 95af 95b2 95b6
95ba 95bb 95bd 95c0 95c3 95c4 95c9 95cd
95d0 95d3 95d4 95d9 95dd 95e1 95e5 95e9
95ed 95f1 95f5 95f9 95fd 9601 9605 9609
960d 9611 9615 9621 9625 962a 962e 9631
9632 9637 963b 963f 9642 9645 9649 964b
964f 9653 9656 9659 965d 965f 9663 9667
966a 966c 9670 9674 9678 967b 967f 9683
9685 9686 968b 1 968f 9693 9697 969a
969e 96a2 96a6 96aa 96ad 96b1 96b5 96b8
96bb 96c0 96c3 96c6 96c9 96ca 96cf 96d0
1 96d2 96d7 96db 96df 96e2 96e7 96ec
96ef 96f3 96f4 96f9 96fc 9701 9702 9707
970a 970f 9710 9715 9718 971c 971d 9722
9725 972a 972b 9730 9733 9737 973b 973e
9741 9742 9744 9745 974a 974b 9750 9752
9756 9759 975b 975f 9761 976d 976f 9771
9772 9777 977b 977d 9789 978b 978d 9791
9794 9798 979d 97a1 97a5 97aa 97ae 97b2
97b7 97bb 97bf 97c4 97c8 97cc 97d1 97d5
97d9 97de 97e2 97e6 97ea 97ee 97f2 97f5
97f9 97fb 97ff 9802 9806 9809 980c 980d
9812 9816 9819 981a 981f 1 9822 9827
982b 982f 9833 9837 983b 983f 9843 9847
984b 9857 985b 9860 9864 9867 9868 986d
9871 9875 9878 987b 987f 9881 9885 9889
988c 988f 9893 9895 9899 989d 98a0 98a2
98a6 98aa 98ae 98b1 98b5 98b9 98bb 98bc
98c1 1 98c5 98c9 98cd 98d0 98d4 98d8
98dc 98e0 98e3 98e7 98eb 98ee 98f1 98f6
98f9 98fc 98ff 9900 9905 9906 1 9908
990d 9911 9915 9918 991d 9922 9925 9929
992a 992f 9932 9937 9938 993d 9940 9945
9946 994b 994e 9952 9953 9958 995b 9960
9961 9966 9969 996d 9971 9974 9977 9978
997a 997b 9980 9981 9986 9988 998c 998f
9991 9995 9997 99a3 99a5 99a7 99a8 99ad
99b1 99b3 99bf 99c1 99c3 99c7 99ca 99ce
99d1 99d2 99d7 99da 99de 99e1 99e2 99e7
99ea 99ee 99f2 99f6 99f9 99fe 99ff 9a04
9a07 9a0a 9a0b 9a0d 9a11 9a13 9a17 9a1a
9a1e 9a22 9a26 9a29 9a2d 9a2e 9a33 9a36
9a39 9a3a 9a3c 9a40 9a42 9a46 9a49 9a4d
9a50 9a51 9a56 9a59 9a5d 9a61 9a65 9a68
9a6c 9a6d 9a72 9a75 9a78 9a79 9a7b 9a7f
9a81 9a85 9a88 9a8c 9a8f 9a90 9a95 9a98
9a9c 9a9f 9aa0 9aa5 9aa8 9aac 9ab0 9ab4
9ab7 9abc 9abd 9ac2 9ac5 9ac8 9ac9 9acb
9acf 9ad1 9ad5 9ad8 9adc 9ae0 9ae4 9ae7
9aeb 9aec 9af1 9af4 9af7 9af8 9afa 9afe
9b00 9b04 9b07 9b0b 9b0e 9b0f 9b14 9b17
9b1b 9b1f 9b23 9b26 9b2a 9b2b 9b30 9b33
9b36 9b37 9b39 9b3d 9b3f 9b43 9b46 9b4a
9b4d 9b4e 9b53 9b56 9b5a 9b5d 9b5e 9b63
9b66 9b6a 9b6e 9b72 9b75 9b7a 9b7b 9b80
9b83 9b86 9b87 9b89 9b8d 9b8f 9b93 9b96
9b9a 9b9e 9ba2 9ba5 9ba9 9baa 9baf 9bb2
9bb5 9bb6 9bb8 9bbc 9bbe 9bc2 9bc5 9bc9
9bcc 9bcd 9bd2 9bd5 9bd9 9bdc 9bdd 9be2
9be5 9be9 9bed 9bf1 9bf4 9bf9 9bfa 9bff
9c02 9c05 9c06 9c08 9c0c 9c0e 9c12 9c15
9c19 9c1d 9c21 9c24 9c28 9c29 9c2e 9c31
9c34 9c35 9c37 9c3b 9c3d 9c41 9c44 9c48
9c4c 9c4f 9c52 9c53 9c58 9c5c 9c60 9c64
9c68 9c6a 9c6e 9c75 9c79 9c7d 9c82 9c84
9c88 9c8c 9c8f 9c93 9c97 9c99 9c9d 9ca1
9ca5 9ca8 9cac 9cb0 9cb2 9cb3 9cb8 1
9cbc 9cc0 9cc4 9cc7 9ccb 9ccf 9cd3 9cd7
9cda 9cde 9ce2 9ce5 9ce8 9ced 9cf0 9cf3
9cf6 9cf7 9cfc 9cfd 1 9cff 9d04 9d08
9d0c 9d0f 9d14 9d19 9d1c 9d20 9d21 9d26
9d29 9d2e 9d2f 9d34 9d37 9d3c 9d3d 9d42
9d45 9d49 9d4a 9d4f 9d52 9d57 9d58 9d5d
9d60 9d64 9d68 9d6b 9d6e 9d6f 9d71 9d72
9d77 9d78 9d7d 9d7f 9d83 9d86 9d88 9d8c
9d8e 9d9a 9d9c 9d9e 9d9f 9da4 9da8 9daa
9db6 9db8 9dbc 9dbf 9dc2 9dc3 9dc8 9dcc
9dd0 9dd4 9dd8 9ddc 9de0 9de4 9de8 9dec
9df0 9df4 9df8 9dfc 9e00 9e04 9e10 9e14
9e19 9e1d 9e20 9e21 9e26 9e2a 9e2e 9e31
9e34 9e38 9e3a 9e3e 9e42 9e45 9e48 9e4c
9e4e 9e52 9e56 9e59 9e5b 9e5f 9e63 9e67
9e6a 9e6e 9e72 9e74 9e75 9e7a 1 9e7e
9e82 9e86 9e89 9e8d 9e91 9e95 9e99 9e9c
9ea0 9ea4 9ea7 9eaa 9eaf 9eb2 9eb5 9eb8
9eb9 9ebe 9ebf 1 9ec1 9ec6 9eca 9ece
9ed1 9ed6 9edb 9ede 9ee2 9ee3 9ee8 9eeb
9ef0 9ef1 9ef6 9ef9 9efe 9eff 9f04 9f07
9f0b 9f0c 9f11 9f14 9f19 9f1a 9f1f 9f22
9f26 9f2a 9f2d 9f30 9f31 9f33 9f34 9f39
9f3a 9f3f 9f41 9f45 9f48 9f4a 9f4e 9f50
9f5c 9f5e 9f60 9f61 9f66 9f6a 9f6c 9f78
9f7a 9f7c 9f80 9f83 9f85 9f89 9f8c 9f90
9f93 9f94 9f99 9f9c 9fa0 9fa4 9fa7 9fa8
9faa 9fad 9fb0 9fb1 9fb6 9fba 9fbe 9fc2
9fc6 9fca 9fce 9fd2 9fd6 9fda 9fde 9fe2
9fe6 9fea 9fee 9ffa 9ffe a002 a005 a009
a00d a00f a013 a017 a01b a01e a022 a026
a028 a029 a02e 1 a032 a036 a03a a03d
a041 a045 a049 a04d a050 a054 a058 a05b
a05e a063 a066 a069 a06c a06d a072 a073
1 a075 a07a a07e a082 a085 a08a a08f
a092 a096 a097 a09c a09f a0a4 a0a5 a0aa
a0ad a0b2 a0b3 a0b8 a0bb a0bf a0c0 a0c5
a0c8 a0cd a0ce a0d3 a0d6 a0da a0de a0e1
a0e4 a0e5 a0e7 a0e8 a0ed a0ee a0f3 a0f5
a0f9 a0fc a0fe a102 a104 a110 a112 a114
a115 a11a a11e a120 a12c a12e a130 a134
a137 a13b a13d a13f a143 a147 a14a a14c
a150 a153 a157 a15b a15f a163 a167 a16b
a16f a173 a177 a17b a17f a183 a187 a18b
a18f a193 a197 a19b a19f a1ab a1af a1b3
a1b6 a1ba a1be a1c0 a1c4 a1c8 a1cc a1cf
a1d3 a1d7 a1d9 a1da a1df 1 a1e3 a1e7
a1eb a1ee a1f2 a1f6 a1fa a1fe a201 a205
a209 a20c a20f a214 a217 a21a a21d a21e
a223 a224 1 a226 a22b a22f a233 a236
a23b a240 a243 a247 a248 a24d a250 a255
a256 a25b a25e a263 a264 a269 a26c a270
a271 a276 a279 a27e a27f a284 a287 a28b
a28f a292 a295 a296 a298 a299 a29e a29f
a2a4 a2a6 a2aa a2ad a2af a2b3 a2b5 a2c1
a2c3 a2c5 a2c6 a2cb a2cf a2d1 a2dd a2df
a2e3 a2e7 a2ea a2eb a2ed a2f1 a2f4 a2f8
a2fb a2fc a301 a305 a309 a30e a313 a317
a31b a31f a322 a325 a329 a32d a331 a335
a339 a33d a341 a345 a349 a34d a351 a355
a361 a365 a36a a36e a371 a372 a377 a37b
a37f a382 a385 a389 a38b a38f a393 a396
a399 a39d a39f a3a3 a3a7 a3aa a3ac a3b0
a3b4 a3b8 a3bb a3bf a3c3 a3c5 a3c6 a3cb
1 a3cf a3d3 a3d7 a3da a3de a3e2 a3e6
a3ea a3ed a3f1 a3f5 a3f8 a3fb a400 a403
a406 a409 a40a a40f a410 1 a412 a417
a41b a41f a422 a427 a42c a42f a433 a434
a439 a43c a441 a442 a447 a44a a44f a450
a455 a458 a45c a45d a462 a465 a46a a46b
a470 a473 a477 a47b a47e a481 a482 a484
a485 a48a a48b a490 a492 a496 a499 a49b
a49f a4a1 a4ad a4af a4b1 a4b2 a4b7 a4bb
a4bd a4c9 a4cb a4cf a4d3 a4d6 a4da a4dd
a4e1 a4e4 a4e5 a4ea a4ed a4f1 a4f5 a4f9
a4fd a501 a505 a509 a50d a511 a515 a519
a51d a521 a525 a529 a52d a531 a535 a541
a545 a54a a54e a551 a552 a557 a55b a55f
a562 a565 a569 a56b a56f a573 a576 a579
a57d a57f a583 a587 a58a a58c a590 a594
a598 a59b a59f a5a3 a5a5 a5a6 a5ab 1
a5af a5b3 a5b7 a5ba a5be a5c2 a5c6 a5ca
a5cd a5d1 a5d5 a5d8 a5db a5e0 a5e3 a5e6
a5e9 a5ea a5ef a5f0 1 a5f2 a5f7 a5fb
a5ff a602 a607 a60c a60f a613 a614 a619
a61c a621 a622 a627 a62a a62f a630 a635
a638 a63c a63d a642 a645 a64a a64b a650
a653 a657 a65b a65e a661 a662 a664 a665
a66a a66b a670 a672 a676 a679 a67b a67f
a681 a68d a68f a691 a692 a697 a69b a69d
a6a9 a6ab a6af a6b3 a6b6 a6ba a6bd a6c1
a6c4 a6c5 a6ca a6cd a6d1 a6d5 a6d9 a6dd
a6e1 a6e5 a6f1 a6f5 a6fa a6fe a701 a702
a707 a70b a70f a712 a715 a719 a71b a71f
a723 a726 a729 a72d a72f a733 a737 a73a
a73c a740 a744 a748 a74b a74f a753 a755
a756 a75b 1 a75f a763 a767 a76a a76e
a772 a776 a77a a77d a781 a785 a788 a78b
a790 a793 a796 a799 a79a a79f a7a0 1
a7a2 a7a7 a7ab a7af a7b2 a7b7 a7bc a7bf
a7c3 a7c4 a7c9 a7cc a7d1 a7d2 a7d7 a7da
a7df a7e0 a7e5 a7e8 a7ec a7ed a7f2 a7f5
a7fa a7fb a800 a803 a807 a80b a80e a811
a812 a814 a815 a81a a81b a820 a822 a826
a829 a82b a82f a831 a83d a83f a841 a842
a847 a84b a84d a859 a85b a85d a861 a864
a866 a86a a86d a871 a875 a879 a87d a881
a885 a889 a88d a891 a895 a899 a89d a8a1
a8a5 a8a9 a8ad a8b1 a8b5 a8b9 a8bd a8c1
a8c5 a8c9 a8cd a8d1 a8d5 a8d9 a8dd a8e1
a8e5 a8e9 a8ed a8f1 a8f5 a8f9 a8fd a901
a905 a909 a90d a911 a915 a919 a91d a921
a925 a929 a92d a931 a935 a939 a93d a941
a945 a949 a94d a951 a955 a959 a95d a961
a965 a969 a96d a971 a975 a979 a97d a981
a985 a989 a98d a991 a995 a999 a99d a9a1
a9a5 a9a9 a9ad a9b1 a9b5 a9b9 a9bd a9c1
a9c5 a9c9 a9cd a9d1 a9d5 a9d9 a9dd a9e1
a9e5 a9e9 a9ed a9f1 a9f5 a9f9 a9fd aa01
aa05 aa09 aa0d aa11 aa15 aa19 aa1d aa21
aa25 aa29 aa2d aa31 aa35 aa39 aa3d aa41
aa45 aa49 aa4d aa51 aa55 aa59 aa5d aa61
aa65 aa69 aa6d aa71 aa75 aa79 aa7d aa81
aa85 aa89 aa8d aa91 aa95 aa99 aa9d aaa1
aaa5 aaa9 aaad aab1 aab5 aab9 aabd aac1
aac5 aac9 aacd aad1 aad5 aad9 aadd aae1
aae5 aae9 aaed aaf1 aaf5 aaf9 aafd ab01
ab05 ab09 ab0d ab11 ab15 ab19 ab1d ab21
ab25 ab29 ab2d ab31 ab35 ab39 ab3d ab41
ab45 ab49 ab4d ab51 ab55 ab59 ab5d ab61
ab65 ab69 ab6d ab71 ab75 ab79 ab7d ab81
ab85 ab89 ab8d ab91 ab95 ab99 ab9d aba9
abae abb2 abb6 abba abbd abc1 abc5 abc8
abcb abd0 abd3 abd6 abd9 abda abdf abe0
1 abe2 abe7 abeb abef abf2 abf7 abfc
abff ac03 ac04 ac09 ac0a ac0f ac11 ac15
ac18 ac1c ac20 ac24 ac26 ac2a ac2d ac30
ac31 ac36 ac3a ac3e ac42 ac46 ac4a ac4e
ac52 ac56 ac5a ac5e ac62 ac66 ac6a ac6e
ac72 ac76 ac7a ac7e ac82 ac86 ac8a ac8e
ac92 ac96 ac9a ac9e aca2 aca6 acaa acae
acb2 acb6 acba acbe acc2 acc6 acca acce
acd2 acd6 acda acde ace2 ace6 acea acee
acf2 acf6 acfa acfe ad02 ad06 ad0a ad0e
ad12 ad16 ad1a ad1e ad22 ad26 ad2a ad2e
ad32 ad36 ad3a ad3e ad42 ad46 ad4a ad4e
ad52 ad56 ad5a ad5e ad62 ad66 ad6a ad6e
ad72 ad76 ad7a ad7e ad82 ad86 ad8a ad8e
ad92 ad96 ad9a ad9e ada2 ada6 adaa adae
adb2 adb6 adba adbe adc2 adc6 adca adce
add2 add6 adda adde ade2 ade6 adea adee
adf2 adf6 adfa adfe ae02 ae06 ae0a ae0e
ae12 ae16 ae1a ae1e ae22 ae26 ae2a ae2e
ae32 ae36 ae3a ae3e ae42 ae46 ae4a ae4e
ae52 ae56 ae5a ae5e ae62 ae66 ae6a ae6e
ae72 ae76 ae7a ae7e ae82 ae86 ae8a ae8e
ae92 ae96 ae9a ae9e aea2 aea6 aeaa aeae
aeb2 aeb6 aeba aebe aec2 aec6 aeca aece
aed2 aed6 aeda aede aee2 aee6 aeea aeee
aef2 aef6 aefa aefe af02 af06 af0a af0e
af12 af16 af1a af1e af22 af26 af2a af2e
af32 af36 af3a af3e af42 af46 af4a af4e
af52 af56 af5a af5e af62 af66 af6a af6e
af72 af76 af7a af7e af82 af86 af8a af8e
af92 af96 af9a af9e afa2 afa6 afaa afae
afb2 afb6 afba afbe afc2 afc6 afca afce
afd2 afd6 afda afde afe2 afe6 afea afee
aff2 aff6 affa affe b002 b006 b00a b00e
b012 b016 b01a b01e b022 b026 b02a b02e
b032 b036 b03a b03e b042 b046 b04a b04e
b052 b056 b05a b05e b062 b066 b06a b06e
b072 b076 b07a b07e b082 b086 b08a b08e
b092 b096 b09a b09e b0a2 b0a6 b0aa b0ae
b0b2 b0b6 b0ba b0be b0c2 b0c6 b0ca b0ce
b0d2 b0d6 b0da b0de b0e2 b0e6 b0ea b0f6
b0fb b0ff b103 b107 b10a b10e b112 b115
b118 b11d b120 b123 b126 b127 b12c b12d
1 b12f b134 b138 b13c b13f b144 b149
b14c b150 b151 b156 b157 b15c b15e b162
b165 b169 b16d b171 b173 b177 b17b b180
b185 b189 b18d b191 b194 b197 b19b b19f
b1a3 b1a7 b1ab b1af b1b3 b1b7 b1bb b1c7
b1cb b1d0 b1d4 b1d7 b1d8 b1dd b1e1 b1e5
b1e8 b1eb b1ef b1f1 b1f5 b1f9 b1fc b1ff
b203 b205 b209 b20d b210 b212 b216 b21a
b21e b221 b225 b229 b22b b22c b231 1
b235 b239 b23d b240 b244 b248 b24c b250
b253 b257 b25b b25e b261 b266 b269 b26c
b26f b270 b275 b276 1 b278 b27d b281
b285 b288 b28d b292 b295 b299 b29a b29f
b2a2 b2a7 b2a8 b2ad b2b0 b2b5 b2b6 b2bb
b2be b2c2 b2c3 b2c8 b2cb b2d0 b2d1 b2d6
b2d9 b2dd b2e1 b2e4 b2e7 b2e8 b2ea b2eb
b2f0 b2f1 b2f6 b2f8 b2fc b2ff b301 b305
b307 b313 b315 b317 b318 b31d b321 b323
b32f b331 b335 b339 b33d b341 b345 b349
b34d b351 b355 b359 b35d b361 b365 b369
b36d b371 b375 b379 b37d b381 b38d b391
b396 b39a b39d b39e b3a3 b3a7 b3ab b3ae
b3b1 b3b5 b3b7 b3bb b3bf b3c2 b3c5 b3c9
b3cb b3cf b3d3 b3d6 b3d8 b3dc b3e0 b3e4
b3e7 b3eb b3ef b3f1 b3f2 b3f7 1 b3fb
b3ff b403 b406 b40a b40e b412 b416 b419
b41d b421 b424 b427 b42c b42f b432 b435
b436 b43b b43c 1 b43e b443 b447 b44b
b44e b453 b458 b45b b45f b460 b465 b468
b46d b46e b473 b476 b47b b47c b481 b484
b488 b489 b48e b491 b496 b497 b49c b49f
b4a3 b4a7 b4aa b4ad b4ae b4b0 b4b1 b4b6
b4b7 b4bc b4be b4c2 b4c5 b4c7 b4cb b4cd
b4d9 b4db b4dd b4de b4e3 b4e7 b4e9 b4f5
b4f7 b4fb b4ff b503 b507 b50b b50f b513
b517 b51b b51f b523 b527 b52b b52f b533
b537 b53b b53f b543 b547 b54b b54f b553
b557 b55b b55f b563 b567 b56b b56f b573
b577 b57b b57f b583 b587 b58b b58f b593
b597 b59b b59f b5a3 b5a7 b5ab b5af b5b3
b5b7 b5bb b5bf b5c3 b5c7 b5cb b5cf b5d3
b5d7 b5db b5df b5e3 b5e7 b5eb b5ef b5f3
b5f7 b5fb b5ff b603 b607 b60b b60f b613
b617 b61b b61f b623 b627 b62b b62f b633
b637 b63b b63f b643 b647 b64b b64f b653
b657 b65b b65f b663 b667 b66b b66f b673
b677 b67b b67f b683 b687 b68b b68f b693
b697 b69b b69f b6a3 b6a7 b6ab b6af b6b3
b6b7 b6bb b6bf b6c3 b6c7 b6cb b6cf b6d3
b6d7 b6db b6df b6e3 b6e7 b6eb b6ef b6f3
b6f7 b6fb b6ff b703 b707 b70b b70f b713
b717 b71b b71f b723 b727 b72b b72f b733
b737 b73b b73f b743 b747 b74b b74f b753
b757 b75b b75f b763 b767 b76b b76f b773
b777 b77b b77f b783 b787 b78b b78f b793
b797 b79b b79f b7a3 b7a7 b7ab b7af b7b3
b7b7 b7bb b7bf b7c3 b7c7 b7cb b7cf b7d3
b7d7 b7db b7df b7e3 b7e7 b7eb b7ef b7f3
b7f7 b7fb b7ff b803 b807 b80b b80f b813
b817 b81b b81f b823 b827 b82b b82f b833
b837 b83b b83f b843 b847 b84b b84f b853
b857 b85b b85f b863 b867 b86b b86f b873
b877 b87b b87f b883 b887 b88b b88f b893
b897 b89b b89f b8a3 b8a7 b8ab b8af b8b3
b8b7 b8bb b8bf b8c3 b8c7 b8cb b8cf b8d3
b8d7 b8db b8df b8e3 b8e7 b8eb b8ef b8f3
b8f7 b8fb b8ff b903 b907 b90b b90f b913
b917 b91b b91f b923 b927 b92b b92f b933
b937 b93b b93f b943 b947 b94b b94f b953
b957 b95b b95f b963 b967 b96b b96f b973
b977 b97b b97f b983 b987 b98b b98f b993
b997 b99b b99f b9a3 b9a7 b9ab b9af b9b3
b9b7 b9bb b9bf b9c3 b9c7 b9cb b9cf b9d3
b9d7 b9db b9df b9e3 b9e7 b9eb b9ef b9f3
b9f7 b9fb b9ff ba03 ba07 ba0b ba0f ba13
ba17 ba1b ba1f ba23 ba27 ba2b ba2f ba33
ba37 ba3b ba3f ba43 ba47 ba4b ba4f ba53
ba57 ba5b ba5f ba63 ba67 ba6b ba6f ba73
ba77 ba7b ba7f ba83 ba87 ba8b ba8f ba93
ba97 ba9b ba9f baa3 baa7 baab baaf bab3
bab7 babb babf bac3 bac7 bacb bacf bad3
bad7 badb badf bae3 bae7 baeb baef baf3
baf7 bafb baff bb03 bb07 bb0b bb0f bb13
bb17 bb1b bb1f bb23 bb27 bb2b bb2f bb33
bb37 bb3b bb3f bb43 bb47 bb4b bb4f bb53
bb57 bb5b bb5f bb63 bb67 bb6b bb6f bb73
bb77 bb7b bb7f bb83 bb87 bb8b bb8f bb93
bb97 bb9b bb9f bba3 bba7 bbab bbaf bbb3
bbb7 bbbb bbbf bbc3 bbc7 bbcb bbcf bbd3
bbdf bbe4 bbe8 bbec bbf0 bbf3 bbf7 bbfb
bbfe bc01 bc06 bc09 bc0c bc0f bc10 bc15
bc16 1 bc18 bc1d bc21 bc25 bc28 bc2d
bc32 bc35 bc39 bc3a bc3f bc40 bc45 bc47
bc4b bc4e bc52 bc56 bc5a bc5c bc60 bc64
bc67 bc69 bc6d bc71 bc74 bc76 bc7a bc7e
bc81 bc85 bc89 bc8c bc90 bc94 bc97 bc9a
bc9f bca2 bca5 bca8 bca9 bcae bcaf 1
bcb1 bcb6 bcba bcbe bcc1 bcc6 bccb bcce
bcd2 bcd3 bcd8 bcd9 bcde bce2 bce6 bce9
bcee bcf3 bcf6 bcfa bcfe bd01 bd02 bd07
bd0a bd0f bd10 bd15 bd18 bd1c bd20 bd23
bd24 bd29 bd2a bd2f bd31 bd35 bd38 bd3c
bd40 bd44 bd48 bd4c bd50 bd54 bd58 bd5c
bd68 bd6d bd71 bd75 bd79 bd7d bd7f bd80
bd85 1 bd89 bd8d bd91 bd94 bd98 bd9c
bda0 bda4 bda7 bdab bdaf bdb2 bdb5 bdba
bdbd bdc0 bdc3 bdc4 bdc9 bdca 1 bdcc
bdd1 bdd5 bdd9 bddc bde1 bde6 bde9 bded
bdee bdf3 bdf4 bdf9 bdfd be01 be04 be09
be0e be11 be15 be16 be1b be1e be23 be24
be29 be2c be30 be34 be37 be3a be3b be3d
be3e be43 be44 be49 be4b be4f be52 be56
be5a be5e be62 be66 be6a be6e be7a be7f
be83 be87 be8b be8f be91 be95 be97 bea3
bea5 bea7 bea8 bead beb1 beb3 bebf bec3
bec5 bec6 becf 
366b
2
0 1 a 6 d :3 6 14 :3 6
15 :3 6 10 :3 6 e :3 6 11 :3 6
c :3 6 b :3 6 11 :3 6 11 :3 6
1d :3 6 c :3 6 e :3 6 10 :3 6
10 :3 6 11 :3 6 17 :3 6 10 :3 6
11 :3 6 10 :3 6 11 :3 6 f :3 6
f :3 6 10 :3 6 d :3 6 e :3 6
12 :2 6 1 3 a 13 a :2 1
4 d 16 15 :2 d :2 4 11 1a
19 :2 11 :2 4 e 17 16 :2 e :2 4
c 15 14 :2 c :2 4 f 18 17
:2 f :2 4 f 18 17 :2 f :2 4 c
15 14 :2 c :2 4 b 14 13 :2 b
:2 4 10 19 18 :2 10 :2 4 11 1a
19 :2 11 :2 4 10 19 18 :2 10 :2 4
11 1a 19 :2 11 :2 4 f 18 17
:2 f :2 4 10 19 18 :2 10 :2 4 f
18 17 :2 f :2 4 10 19 18 :2 10
:2 4 11 1a 19 :2 11 :2 4 11 1a
19 :2 11 :2 4 d 16 15 :2 d :2 4
e 17 16 :2 e :2 4 b 14 13
:2 b :2 4 :3 10 :2 4 11 1a 19 :2 11
:2 4 f 18 17 :2 f :2 4 10 19
18 :2 10 :2 4 14 1d 1c :2 14 :2 4
15 1e 1d :2 15 :2 4 f 18 17
:2 f :2 4 10 19 18 :2 10 :2 4 e
17 16 :2 e :2 4 f 18 17 :2 f
:2 4 10 19 18 :2 10 :2 4 11 1a
19 :2 11 :2 4 :2 e 1d e :2 4 :2 d
1c d :2 4 :2 b 1a b :2 4 :2 c
1b c :2 4 :2 d 1c d :2 4 :2 10
1f 10 :2 4 :2 14 23 14 :2 4 :2 a
19 a :2 4 :2 f 1e f :2 4 :2 b
1a b :2 4 :2 11 20 11 :2 4 :2 d
1c d :2 4 :2 c 1b c :2 4 :2 b
1a b :2 4 :2 f 1e f :2 4 :2 e
1d e :2 4 :2 c 1b c :2 4 :2 e
1d e :2 4 :2 f 1e f :2 4 :2 f
1e f :2 4 :2 12 21 12 :2 4 :2 d
1c d :2 4 :2 f 1e f :2 4 :2 b
1a b :2 4 :2 f 19 f :2 4 :2 13
1d 13 :2 4 :2 c 1b c :2 4 :2 10
1f 10 :2 4 :2 12 21 12 :2 4 :2 11
20 11 :2 4 :2 16 25 16 :2 4 :2 16
25 16 :2 4 :2 b 1a b :2 4 d
16 15 :2 d :2 4 :2 10 1f 10 :2 4
:2 11 20 11 :2 4 :2 12 21 12 :2 4
9 12 11 :2 9 :2 4 c 15 14
:2 c :2 4 10 19 18 :2 10 :2 4 :2 11
20 11 :2 4 :2 17 26 17 :2 4 17
1c 1b :2 17 :2 4 18 1d 1c :2 18
:2 4 11 1a 19 :2 11 :2 4 12 1b
1a :2 12 :2 4 16 1f 1e :2 16 :2 4
b 14 13 :2 b :2 4 :2 f 1e f
:2 4 :2 13 22 13 :2 4 :2 14 23 14
:2 4 :2 6 1e 6 :2 4 :2 7 1f 7
:2 4 :2 a 22 a :2 4 :2 d 25 d
:2 4 :2 8 20 8 :2 4 a 13 12
:2 a :2 4 :2 e 26 e :2 4 :3 d :2 4
:3 13 :2 4 :3 8 :2 4 :3 6 :2 4 :3 9 :2 4
:3 c :2 4 :3 c 4 7 b 14 :3 b
:2 7 e 0 :2 7 11 f 10 1d
e 18 e 1c a :6 7 e 0
:2 7 a :2 c 12 c 15 c 12
c 15 c 15 c 15 f 13
fffa :6 7 e 17 :3 e :2 7 e 17
:3 e :2 7 e 17 :3 e 7 4 e
17 :3 e 4 7 :2 11 23 :2 2d 3a
:2 23 :3 7 :2 d 19 25 :2 2f 43 :2 25
53 59 5c :2 53 6a 7f 8 :2 6a
d 10 :2 6a 17 1a :2 6a b e
:2 6a 13 16 :2 6a 1b 1e :2 6a 2c
2f :2 6a b e :2 6a 13 16 :2 6a
1b 1e :2 6a 2d 30 :2 6a b e
:2 6a 13 16 1f 27 :2 1f 33 :2 16
:2 6a b e :2 6a 13 16 1f 27
:2 1f 31 :2 16 :2 6a b e :2 6a 13
16 1f 27 :2 1f 34 :2 16 :2 6a b
e :2 6a 13 16 1f 27 :2 1f 2f
:2 16 :2 6a b e :2 6a 13 16 1f
27 :2 1f 2e :2 16 :2 6a b e :2 6a
13 16 1f 27 :2 1f 34 :2 16 :2 6a
b e :2 6a 13 16 1f 27 :2 1f
34 :2 16 :2 6a b e :2 6a 13 16
1f 27 :2 1f 40 :2 16 :2 6a b e
:2 6a 13 16 :2 6a 1b 1e :2 6a 24
27 :2 6a b e :2 6a 13 16 :2 6a
1b 1e :2 6a 26 29 :2 6a b e
:2 6a 13 16 :2 6a 1b 1e :2 6a 28
2b :2 6a b e :2 6a 13 16 :2 6a
1b 1e :2 6a 28 2b :2 6a b e
:2 6a 13 16 :2 6a 1b 1e :2 6a 29
2c :2 6a b e :2 6a 13 16 1f
27 :2 1f 3a :2 16 :2 6a b e :2 6a
13 16 :2 6a 1b 1e :2 6a 28 2b
:2 6a b e :2 6a 13 16 :2 6a 1b
1e :2 6a 29 2c :2 6a b e :2 6a
13 16 :2 6a 1b 1e :2 6a 28 2b
:2 6a b e :2 6a 13 16 :2 6a 1b
1e :2 6a 29 2c :2 6a b e :2 6a
13 16 1f 27 :2 1f 32 :2 16 :2 6a
b e :2 6a 13 16 1f 27 :2 1f
32 :2 16 :2 6a b e :2 6a 13 16
1f 27 :2 1f 33 :2 16 :2 6a b e
:2 6a 13 16 1f 27 :2 1f 30 :2 16
:2 6a b e :2 6a 13 16 :2 6a 1b
1e :2 6a 26 29 :2 6a b e :2 6a
13 16 :2 6a 1b 1e :2 6a 2a 2d
:2 6a 32 35 :2 6a :2 7 49 :2 4 7
:2 11 23 :2 2d 39 3c 4b 4e 4f
:2 4b :2 23 :3 7 :2 d 12 1e 5b 5e
:2 1e :2 7 52 :3 4 14 :2 4 14 :2 4
13 :2 4 f :2 4 e :2 4 13 :2 4
14 :2 4 13 :2 4 14 :2 4 12 :2 4
10 :2 4 e :2 4 14 :2 4 12 :2 4
f :2 4 12 :2 4 e :2 4 e :2 4
11 :2 4 11 :2 4 10 :2 4 13 :2 4
13 :2 4 14 4 8 11 1c :2 8
22 1f :2 2b :2 1f 7 :4 46 45 44
:2 7 a :2 14 26 :2 30 3c 3f 4e
51 52 :2 4e :2 26 :3 a :2 10 15 21
5e 61 :2 21 6e 71 :2 21 75 78
:2 21 82 85 8e 96 :2 9f :2 8e b5
:2 85 :2 21 :2 a 55 :2 7 a :2 14 26
:2 30 3c 3f 4e 51 52 :2 4e :2 26
:3 a :2 10 15 21 5e 61 :2 21 :2 a
55 :3 7 :2 12 28 :2 31 46 :2 7 57
:2 4 :4 9 8 7 :4 1f 1e 1d :2 7
:4 39 38 37 :2 7 :4 53 52 51 :2 7
f 18 24 :2 f 2a 27 :2 33 :2 27
e :2 7 :4 c b a 25 2e 3a
:2 25 3f 3d :2 48 :2 3d 24 :2 a d
:2 17 29 :2 33 3f 42 51 54 55
:2 51 :2 29 :3 d :2 13 18 24 61 64
:2 24 71 74 :2 24 78 7b :2 24 85
88 91 99 :2 a2 :2 91 b3 :2 88 :2 24
:2 d 58 :2 a d :2 17 29 :2 33 3f
42 51 54 55 :2 51 :2 29 :3 d :2 13
18 24 61 64 :2 24 :2 d 58 :3 a
:2 15 2b :2 34 44 :2 a 7 59 :4 f
e d :4 29 28 27 :3 d :2 17 29
:2 33 3f 42 51 54 55 :2 51 :2 29
:3 d :2 13 18 24 61 64 :2 24 71
74 :2 24 78 7b :2 24 85 88 91
99 :2 a2 :2 91 b3 :2 88 :2 24 :2 d 58
:2 a d :2 17 29 :2 33 3f 42 51
54 55 :2 51 :2 29 :3 d :2 13 18 24
61 64 :2 24 :2 d 58 :3 a :2 15 2b
:2 34 44 :2 a 3e 59 :2 7 49 :2 4
7 10 1b :2 7 1e 20 :2 1e 26
2f 37 :2 26 3a 3c :2 3a :2 7 42
4b 53 :2 42 56 58 :2 56 :2 7 a
:2 14 26 :2 30 3c 3f 4e 51 52
:2 4e :2 26 :3 a :2 10 15 21 5e 61
:2 21 6e 71 :2 21 75 78 :2 21 82
85 8e 96 :2 9f :2 8e b0 :2 85 :2 21
:2 a 55 :2 7 a :2 14 26 :2 30 3c
3f 4e 51 52 :2 4e :2 26 :3 a :2 10
15 21 5e 61 :2 21 :2 a 55 :3 7
:2 12 28 :2 31 41 :2 7 5a :2 4 :4 9
8 :2 7 1a 7 24 7 1a 7
:4 4 8 11 1c :2 8 22 1f :2 2b
:2 1f 7 :4 46 45 44 :2 7 :4 14 13
12 :2 7 11 1c 21 33 3e 43
:2 f :2 a :2 13 1f a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
55 58 59 :2 55 :2 2c :3 10 :2 16 1b
27 64 67 :2 27 74 77 :2 27 7b
7e :2 27 8a 8d :2 27 95 98 :2 27
a4 a7 ae b7 ba :2 a7 :2 27 :2 10
5c :2 d :2 a :4 16 :2 a 7 :3 31 b
:2 14 1f 1d :2 28 :2 1d a 3d 46
4f :2 3d 52 54 :2 52 3c :2 a d
:2 17 29 :2 33 3f 42 51 54 55
:2 51 :2 29 :3 d :2 13 18 24 61 64
:2 24 71 74 :2 24 78 7b :2 24 85
88 91 99 :2 a2 :2 91 bf :2 88 :2 24
:2 d 58 :2 a d :2 17 29 :2 33 3f
42 51 54 55 :2 51 :2 29 :3 d :2 13
18 24 61 64 :2 24 :2 d 58 :3 a
:2 15 2b :2 34 50 :2 a 57 :3 7 e
7 31 :2 4 :4 9 8 7 :4 27 26
25 :2 7 11 23 35 :2 a :2 13 1f
a 7 f d :2 16 22 d 1d
:2 a :2 f d :2 16 22 d 10 :2 1a
2c :2 36 42 46 55 58 59 :2 55
:2 2c :3 10 :2 16 1b 27 64 67 :2 27
74 77 :2 27 7b 7e :2 27 8a 8d
:2 27 95 98 :2 27 a4 a7 ae b7
ba :2 a7 :2 27 :2 10 5c :2 d :2 a :4 16
:2 a 7 :4 40 :3 4 12 :2 4 13 :2 1c
28 2f 38 40 :2 2f 44 :2 4d :2 28
27 5c :2 13 :2 4 e 15 1e 26
:2 15 2a :2 33 :2 e :2 4 d :2 16 22
29 32 3e :2 29 42 :2 4b :2 22 21
5a :2 d :2 4 12 19 22 2e :2 19
32 :2 3b :2 12 :2 4 1c 25 31 :2 1c
36 34 :2 3f :2 34 54 :2 17 5b :2 12
4 7 10 18 :2 7 1b 1d :2 1b
7 15 7 11 21 f 10 1d
:2 a :2 13 1f a 7 f d :2 16
22 d 1d :2 a :2 f d :2 16 22
d 10 :2 1a 2c :2 36 42 46 55
58 59 :2 55 :2 2c :3 10 :2 16 1b 27
64 67 :2 27 74 77 :2 27 7b 7e
:2 27 8a 8d :2 27 95 98 :2 27 a4
a7 ae b7 ba :2 a7 :2 27 :2 10 5c
:2 d :2 a :4 16 :2 a 7 :3 1f a 13
1f :2 a 22 24 :2 22 14 24 12
13 20 :2 d :2 16 22 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 58 5b 5c :2 58 :2 2f :3 13 :2 19
1e 2a 67 6a :2 2a 77 7a :2 2a
7e 81 :2 2a 8d 90 :2 2a 98 9b
:2 2a a7 aa b1 ba bd :2 aa :2 2a
:2 13 5f :2 10 :2 d :4 19 :2 d a :3 26
d :2 18 1f 2d :2 d 32 34 :2 32
10 :2 1a 2c :2 36 42 46 55 58
59 :2 55 :2 2c :3 10 :2 16 1b 27 64
67 :2 27 74 77 :2 27 7b 7e :2 27
88 8b 94 9c :2 a5 :2 94 ba :2 8b
:2 27 c2 c5 :2 27 cd d0 :2 27 d5
d8 :2 27 e5 e8 :2 27 :2 10 5c :2 d
10 :2 1a 2c :2 36 42 45 54 57
58 :2 54 :2 2c :3 10 :2 16 1b 27 64
67 :2 27 :2 10 5b :3 d :2 18 2c :2 35
49 57 :2 d 36 :2 a 26 :3 7 17
20 29 31 :2 20 :2 17 7 a 13
21 :2 a 24 26 :2 24 a 14 20
29 31 :2 20 :2 14 a d 16 1e
:2 d 21 23 :2 21 10 :2 1a 2c :2 36
42 45 54 57 58 :2 54 :2 2c :3 10
:2 16 1b 27 64 67 :2 27 74 77
:2 27 7b 7e :2 27 88 8b 94 9c
:2 a5 :2 94 b6 :2 8b :2 27 :2 10 5b :2 d
10 :2 1a 2c :2 36 42 45 54 57
58 :2 54 :2 2c :3 10 :2 16 1b 27 64
67 :2 27 :2 10 5b :3 d :2 18 2e :2 37
47 :2 d 25 :3 a 1a :2 23 a 28
:3 7 14 7 a 13 20 :2 a 23
26 :2 23 a 16 a 28 d :2 17
29 :2 33 3f 42 51 54 55 :2 51
:2 29 :3 d :2 13 18 24 61 64 :2 24
71 74 :2 24 78 7b :2 24 85 88
91 99 :2 a2 :2 91 b3 :2 88 :2 24 :2 d
58 :2 a d :2 17 29 :2 33 3f 42
51 54 55 :2 51 :2 29 :3 d :2 13 18
24 61 64 :2 24 :2 d 58 :3 a :2 15
2b :2 34 44 :2 a :5 7 1a 29 37
3f 4a 54 25 :2 1a 7 a 13
24 :2 a 27 2a :2 27 d :2 17 29
:2 33 3f 43 52 55 56 :2 52 :2 29
:3 d :2 13 18 24 61 64 :2 24 71
74 :2 24 78 7b :2 24 85 88 91
99 :2 91 ab :2 88 :2 24 b3 b6 :2 24
be c1 :2 24 c6 c9 :2 24 d5 d8
:2 24 :2 d 59 :2 a d :2 17 29 :2 33
3f 42 51 54 55 :2 51 :2 29 :3 d
:2 13 18 24 61 64 :2 24 :2 d 58
:3 a :2 15 29 32 43 :2 29 47 54
:2 a 2c :2 7 a 13 1e :2 a 23
21 :2 2c :2 21 d :2 17 29 :2 33 3f
43 52 55 56 :2 52 :2 29 :3 d :2 13
18 24 61 64 :2 24 71 74 :2 24
78 7b :2 24 85 88 91 99 :2 a2
:2 91 ad :2 88 :2 24 b5 b8 :2 24 c0
c3 :2 24 c8 cb :2 24 d7 da :2 24
:2 d 59 :2 a d :2 17 29 :2 33 3f
42 51 54 55 :2 51 :2 29 :3 d :2 13
18 24 61 64 :2 24 :2 d 58 :3 a
:2 15 29 :2 32 3c 49 :2 a 40 :2 7
a 13 21 :2 a 26 24 :2 2f :2 24
d 1b 24 2c :2 1b :2 d 30 33
:2 30 10 12 1b 12 1b 12 10
:3 12 15 16 23 fffa 10 :2 19 25
10 d 15 13 :2 1c 28 13 23
:2 10 :2 15 13 :2 1c 28 13 16 :2 20
32 :2 3c 48 4c 5b 5e 5f :2 5b
:2 32 :3 16 :2 1c 21 2d 6a 6d :2 2d
7a 7d :2 2d 81 84 :2 2d 90 93
:2 2d 9b 9e :2 2d aa ad b4 bd
c0 :2 ad :2 2d :2 16 62 :2 13 :2 10 :4 1c
:2 10 d :4 35 10 :2 12 14 12 14
12 14 12 1b 1d 12 1b 1d
12 14 12 14 25 27 10 :7 12
15 1d 20 28 16 18 25 14
16 23 25 fffa 10 :2 19 25 10
d 15 13 :2 1c 28 13 23 :2 10
:2 15 13 :2 1c 28 13 16 :2 20 32
:2 3c 48 4c 5b 5e 5f :2 5b :2 32
:3 16 :2 1c 21 2d 6a 6d :2 2d 7a
7d :2 2d 81 84 :2 2d 90 93 :2 2d
9b 9e :2 2d aa ad b4 bd c0
:2 ad :2 2d :2 16 62 :2 13 :2 10 :4 1c :2 10
d :7 a e :2 17 23 20 :2 2c :2 20
d 10 :2 1a 2c :2 36 42 46 55
58 59 :2 55 :2 2c :3 10 :2 16 1b 27
64 67 :2 27 74 77 :2 27 7b 7e
:2 27 88 8b 94 9c :2 a5 :2 94 b8
:2 8b :2 27 c0 c3 :2 27 cb ce :2 27
d3 d6 :2 27 e0 e3 :2 27 :2 10 5c
:2 d 10 :2 1a 2c :2 36 42 45 54
57 58 :2 54 :2 2c :3 10 :2 16 1b 27
64 67 :2 27 :2 10 5b :3 d :2 18 2c
:2 35 47 52 :2 d 3c :2 a 7 3c
d 16 24 :2 d 29 27 :2 32 :2 27
d :2 f d :2 f 12 13 20 fffa
d :2 16 22 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 58
5b 5c :2 58 :2 2f :3 13 :2 19 1e 2a
67 6a :2 2a 77 7a :2 2a 7e 81
:2 2a 8d 90 :2 2a 98 9b :2 2a a7
aa b1 ba bd :2 aa :2 2a :2 13 5f
:2 10 :2 d :4 19 :2 d a :3 3f e :2 17
23 20 :2 2c :2 20 d 10 :2 1a 2c
:2 36 42 46 55 58 59 :2 55 :2 2c
:3 10 :2 16 1b 27 64 67 :2 27 74
77 :2 27 7b 7e :2 27 88 8b 94
9c :2 a5 :2 94 b8 :2 8b :2 27 c0 c3
:2 27 cb ce :2 27 d3 d6 :2 27 e0
e3 :2 27 :2 10 5c :2 d 10 :2 1a 2c
:2 36 42 45 54 57 58 :2 54 :2 2c
:3 10 :2 16 1b 27 64 67 :2 27 :2 10
5b :3 d :2 18 2c :2 35 47 52 :2 d
3c :2 a 12 1d :2 12 1f f 18
f 18 f d :3 f 12 13 20
11 1e fffa d :2 16 22 d a
12 10 :2 19 25 10 20 :2 d :2 12
10 :2 19 25 10 13 :2 1d 2f :2 39
45 49 58 5b 5c :2 58 :2 2f :3 13
:2 19 1e 2a 67 6a :2 2a 77 7a
:2 2a 7e 81 :2 2a 8d 90 :2 2a 98
9b :2 2a a7 aa b1 ba bd :2 aa
:2 2a :2 13 5f :2 10 :2 d :4 19 :2 d a
:3 3f e :2 17 23 20 :2 2c :2 20 d
10 :2 1a 2c :2 36 42 46 55 58
59 :2 55 :2 2c :3 10 :2 16 1b 27 64
67 :2 27 74 77 :2 27 7b 7e :2 27
88 8b 94 9c :2 a5 :2 94 b8 :2 8b
:2 27 c0 c3 :2 27 cb ce :2 27 d3
d6 :2 27 e3 e6 :2 27 :2 10 5c :2 d
10 :2 1a 2c :2 36 42 45 54 57
58 :2 54 :2 2c :3 10 :2 16 1b 27 64
67 :2 27 :2 10 5b :3 d :2 18 2c :2 35
47 55 :2 d 3c :2 a :4 e :2 d 1d
:2 26 :2 d 17 26 :2 17 d 24 :2 a
3f 3c d :2 17 29 :2 33 3f 43
52 55 56 :2 52 :2 29 :3 d :2 13 18
24 61 64 :2 24 71 74 :2 24 78
7b :2 24 85 88 91 99 :2 a2 :2 91
b3 :2 88 :2 24 bb be :2 24 c6 c9
:2 24 ce d1 :2 24 de e1 :2 24 :2 d
59 :2 a d :2 17 29 :2 33 3f 42
51 54 55 :2 51 :2 29 :3 d :2 13 18
24 61 64 :2 24 :2 d 58 :3 a :2 15
29 :2 32 42 50 :2 a :4 7 a 13
1f :2 a 25 22 :2 2e :2 22 d f
1a 25 :2 f 18 23 2e f 18
23 2e d :4 f 12 13 21 fffa
d :2 16 22 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 58
5b 5c :2 58 :2 2f :3 13 :2 19 1e 2a
67 6a :2 2a 77 7a :2 2a 7e 81
:2 2a 8d 90 :2 2a 98 9b :2 2a a7
aa b1 ba bd :2 aa :2 2a :2 13 5f
:2 10 :2 d :4 19 :2 d a :3 3e e :2 17
22 20 :2 2b :2 20 d 10 :2 1a 2c
:2 36 42 46 55 58 59 :2 55 :2 2c
:3 10 :2 16 1b 27 64 67 :2 27 74
77 :2 27 7b 7e :2 27 88 8b 94
9c :2 a5 :2 94 b8 :2 8b :2 27 c0 c3
:2 27 cb ce :2 27 d3 d6 :2 27 df
e2 :2 27 :2 10 5c :2 d 10 :2 1a 2c
:2 36 42 45 54 57 58 :2 54 :2 2c
:3 10 :2 16 1b 27 64 67 :2 27 :2 10
5b :3 d :2 18 2c :2 35 47 51 :2 d
a 3b 11 :2 1a 25 23 :2 2e :2 23
:2 10 17 21 24 :2 10 27 29 :2 27
31 3a 4d :2 31 50 52 :2 50 58
13 21 :2 58 26 24 2f 42 :2 26
:2 24 :2 31 30 :2 10 49 13 21 :2 49
26 24 2f 42 :2 26 :2 24 :2 10 13
:2 1d 2f :2 39 45 49 58 5b 5c
:2 58 :2 2f :3 13 :2 19 1e 2a 67 6a
:2 2a 77 7a :2 2a 7e 81 :2 2a 8b
8e 97 9f :2 a8 :2 97 be :2 8e :2 2a
c6 c9 :2 2a d1 d4 :2 2a d9 dc
:2 2a e5 e8 :2 2a :2 13 5f :2 10 13
:2 1d 2f :2 39 45 48 57 5a 5b
:2 57 :2 2f :3 13 :2 19 1e 2a 67 6a
:2 2a :2 13 5e :3 10 :2 1b 2f :2 38 4d
57 :2 10 45 :2 d 3e 3b :2 a 3e
:2 7 1f :2 4 7 10 18 :2 7 1b
1d :2 1b a 13 1f :2 a 24 22
:2 2d :2 22 :4 42 41 :2 a 59 d 1a
:2 59 1d 1f :2 1d :2 a d :2 17 29
:2 33 3f 43 52 55 56 :2 52 :2 29
:3 d :2 13 18 24 61 64 :2 24 71
74 :2 24 78 7b :2 24 85 88 91
99 :2 a2 :2 91 bd :2 88 :2 24 c5 c8
:2 24 d0 d3 :2 24 d8 db :2 24 e8
eb :2 24 :2 d 59 :2 a d :2 17 29
:2 33 3f 42 51 54 55 :2 51 :2 29
:3 d :2 13 18 24 61 64 :2 24 :2 d
58 :3 a :2 15 29 :2 32 4c 12 :2 a
21 :2 7 a 13 1e :2 a 24 21
:2 2d :2 21 d f 18 f 18 d
:2 f 12 13 20 fffa d :2 16 22
d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 58 5b 5c :2 58
:2 2f :3 13 :2 19 1e 2a 67 6a :2 2a
77 7a :2 2a 7e 81 :2 2a 8d 90
:2 2a 98 9b :2 2a a7 aa b1 ba
bd :2 aa :2 2a :2 13 5f :2 10 :2 d :4 19
:2 d a :3 41 e :2 17 22 20 :2 2b
:2 20 d 10 12 15 25 32 fffa
10 :2 19 25 10 d 15 13 :2 1c
28 13 23 :2 10 :2 15 13 :2 1c 28
13 16 :2 20 32 :2 3c 48 4c 5b
5e 5f :2 5b :2 32 :3 16 :2 1c 21 2d
6a 6d :2 2d 7a 7d :2 2d 81 84
:2 2d 90 93 :2 2d 9b 9e :2 2d aa
ad b4 bd c0 :2 ad :2 2d :2 16 62
:2 13 :2 10 :4 1c :2 10 d :3 3b 11 :2 1a
25 23 :2 2e :2 23 10 13 :2 1d 2f
:2 39 45 49 58 5b 5c :2 58 :2 2f
:3 13 :2 19 1e 2a 67 6a :2 2a 77
7a :2 2a 7e 81 :2 2a 8b 8e 97
9f :2 a8 :2 97 b9 :2 8e :2 2a c1 c4
:2 2a cc cf :2 2a d4 d7 :2 2a e4
e7 :2 2a :2 13 5f :2 10 13 :2 1d 2f
:2 39 45 48 57 5a 5b :2 57 :2 2f
:3 13 :2 19 1e 2a 67 6a :2 2a :2 13
5e :3 10 :2 1b 2f :2 38 48 56 :2 10
3e :2 d 3b :2 a 41 :2 7 1f :2 4
7 10 1c :2 7 1f 21 :2 1f 27
30 38 :2 27 3b 3e :2 3b :3 7 11
7 42 :2 4 7 10 1b :2 7 20
1e :2 29 :2 1e :4 b :2 a 18 :2 a 19
a 22 a 18 :2 a 19 a :5 7
13 7 3d a e 10 :2 e a
19 :2 a 18 29 35 42 4b 54
:2 42 58 61 69 :2 58 22 2b 37
:2 22 3b 44 4f :2 3b 53 20 2b
:2 53 2f 38 46 :2 2f 4a 53 5e
:2 4a 22 2b 37 :2 22 3b 43 4d
:2 18 a d 16 22 :2 d 27 25
:2 30 :2 25 10 :2 1a 2c :2 36 42 45
54 57 58 :2 54 :2 2c :3 10 :2 16 1b
27 64 67 :2 27 74 77 :2 27 7b
7e :2 27 88 8b 94 9c :2 a5 :2 94
b6 :2 8b :2 27 :2 10 5b :2 d 10 :2 1a
2c :2 36 42 45 54 57 58 :2 54
:2 2c :3 10 :2 16 1b 27 64 67 :2 27
:2 10 5b :3 d :2 18 2e :2 37 47 :2 d
a 3f 10 19 25 :2 10 28 2b
:2 28 10 :2 1a 2c :2 36 42 46 55
58 59 :2 55 :2 2c :3 10 :2 16 1b 27
64 67 :2 27 74 77 :2 27 7b 7e
:2 27 88 8b 94 9c :2 94 a9 :2 8b
:2 27 b1 b4 :2 27 bc bf :2 27 c4
c7 :2 27 d4 d7 :2 27 :2 10 5c :2 d
10 :2 1a 2c :2 36 42 45 54 57
58 :2 54 :2 2c :3 10 :2 16 1b 27 64
67 :2 27 :2 10 5b :3 d :2 18 2c 35
41 :2 2c 45 53 :2 d 2d 3f :2 a
12 a 19 a :4 7 b 14 1c
:2 b 1f 21 :2 1f a 29 32 3a
:2 29 3d 40 :2 3d 28 :2 a e 17
1f :2 e 22 24 :2 22 d 2b 34
3c :2 2b 3f 41 :2 3f 2a :3 d 18
d a 44 10 19 21 :2 10 24
26 :2 24 d 18 d a 28 44
11 1a 22 :2 11 25 27 :2 25 10
2e 37 3f :2 2e 42 44 :2 42 2d
:2 10 d 18 d a 47 44 10
19 21 :2 10 24 26 :2 24 d 18
d 28 44 :2 a 45 :2 7 :4 4 7
10 1c :2 7 21 1f :2 2a :2 1f 7
19 :2 7 16 :2 7 17 :2 7 14 1b
28 2b :2 14 :2 7 17 7 a 13
20 :2 a 23 25 :2 23 a :3 16 :2 a
1c a 27 14 24 12 13 20
:2 d :2 16 22 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 58
5b 5c :2 58 :2 2f :3 13 :2 19 1e 2a
68 6b :2 2a 78 7b :2 2a 7f 82
:2 2a 8e 91 :2 2a 99 9c :2 2a a8
ab b2 bb be :2 ab :2 2a :2 13 5f
:2 10 :2 d :4 19 :2 d a :3 7 e :2 17
22 20 :2 2b :2 20 d 10 19 21
:2 10 24 26 :2 24 :2 13 18 19 26
17 25 :2 17 25 2e fffa 13 :2 1c
28 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 5e 61 62
:2 5e :2 35 :3 19 :2 1f 24 30 6e 71
:2 30 7e 81 :2 30 85 88 :2 30 94
97 :2 30 9f a2 :2 30 ae b1 b8
c1 c4 :2 b1 :2 30 :2 19 65 :2 16 :2 13
:4 1f :2 13 10 :3 28 14 :2 1d 28 26
:2 31 :2 26 :2 13 25 13 41 :2 10 28
:2 d 3b d 1f d :4 a :4 7 a
19 1c :2 19 d 16 1a d 12
13 1e 11 19 fffa d :2 16 22
d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 58 5b 5c :2 58
:2 2f :3 13 :2 19 1e 2a 68 6b :2 2a
78 7b :2 2a 7f 82 :2 2a 8e 91
:2 2a 99 9c :2 2a a8 ab b2 bb
be :2 ab :2 2a :2 13 5f :2 10 :2 d :4 19
:2 d a :3 1e d 18 25 30 3d
48 55 60 6d 78 85 8b 8f
16 1c 1f 35 3b 43 :2 12 23
27 38 13 16 23 11 15 22
25 fffa d :2 16 22 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 58 5b 5c :2 58 :2 2f :3 13 :2 19
1e 2a 68 6b :2 2a 78 7b :2 2a
7f 82 :2 2a 8e 91 :2 2a 99 9c
:2 2a a8 ab b2 bb be :2 ab :2 2a
:2 13 5f :2 10 :2 d :4 19 :2 d a :3 1e
e :2 17 22 20 :2 2b :2 20 d 10
:2 1a 2c :2 36 42 46 55 58 59
:2 55 :2 2c :3 10 :2 16 1b 27 65 68
:2 27 75 78 :2 27 7c 7f :2 27 89
8c 95 9d :2 a6 :2 95 b9 :2 8c :2 27
c1 c4 :2 27 cc cf :2 27 d4 d7
:2 27 e1 e4 :2 27 :2 10 5c :2 d 10
:2 1a 2c :2 36 42 45 54 57 58
:2 54 :2 2c :3 10 :2 16 1b 27 65 68
:2 27 :2 10 5b :3 d :2 18 2c :2 35 47
52 :2 d 3b :2 a d 16 1e :2 d
21 23 :2 21 d 17 d 25 d
17 d :4 a 19 d :8 f d :8 f
d 14 10 1c :3 1a 10 :2 19 25
10 21 10 :2 19 25 10 :4 d a
12 10 :2 19 25 10 20 :2 d :2 12
10 :2 19 25 10 13 :2 1d 2f :2 39
45 49 58 5c 5d :2 58 :2 2f :3 13
:2 19 1e 2a 69 6c :2 2a 79 7c
:2 2a 80 83 :2 2a 8f 92 :2 2a 9a
9d :2 2a a9 ac b3 bc bf :2 ac
:2 2a :2 13 60 :2 10 :2 d :4 19 :2 d a
:3 1e 7 1e d 14 22 25 :2 d
28 2a :2 28 14 21 d 17 d
17 d 17 13 20 d 14 10
1c :3 1a 10 :2 19 25 10 21 10
:2 19 25 10 :4 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 58
5c 5d :2 58 :2 2f :3 13 :2 19 1e 2a
69 6c :2 2a 79 7c :2 2a 80 83
:2 2a 8f 92 :2 2a 9a 9d :2 2a a9
ac b3 bc bf :2 ac :2 2a :2 13 60
:2 10 :2 d :4 19 :2 d a :4 2e 1e :2 7
16 10 11 f 10 1d :2 e 1c
a 11 d 19 :3 17 d :2 16 22
d 1e d :2 16 22 d :4 a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 55 59 5a :2 55 :2 2c :3 10
:2 16 1b 27 66 69 :2 27 76 79
:2 27 7d 80 :2 27 8c 8f :2 27 97
9a :2 27 a6 a9 b0 b9 bc :2 a9
:2 27 :2 10 5d :2 d :2 a :4 16 :2 a 7
:3 3a 16 10 11 f 10 1d :2 e
1c a 11 d 19 :3 17 d :2 16
22 d 1e d :2 16 22 d :4 a
7 f d :2 16 22 d 1d :2 a
:2 f d :2 16 22 d 10 :2 1a 2c
:2 36 42 46 55 59 5a :2 55 :2 2c
:3 10 :2 16 1b 27 66 69 :2 27 76
79 :2 27 7d 80 :2 27 8c 8f :2 27
97 9a :2 27 a6 a9 b0 b9 bc
:2 a9 :2 27 :2 10 5d :2 d :2 a :4 16 :2 a
7 :3 3a 16 10 1d :2 e 1c a
11 d 19 :3 17 d :2 16 22 d
1e d :2 16 22 d :4 a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 55 59 5a :2 55 :2 2c :3 10 :2 16
1b 27 66 69 :2 27 76 79 :2 27
7d 80 :2 27 8c 8f :2 27 97 9a
:2 27 a6 a9 b0 b9 bc :2 a9 :2 27
:2 10 5d :2 d :2 a :4 16 :2 a 7 :4 3a
:3 4 :3 17 4 7 10 18 :2 7 1b
1d :2 1b 23 2c 38 :2 23 3e 3b
:2 47 :2 3b :2 7 5b d 19 :2 5b 1f
1c :2 28 :2 1c :2 7 :4 b :2 a 17 a
20 :2 7 3d :2 4 7 10 1c :2 7
1f 21 :2 1f 7 15 :2 1e 7 23
:2 4 :4 8 7 :4 20 1f :2 7 :4 39 38
:2 7 :4 10 f e :3 7 14 7 25
:3 4 c :2 4 f 4 7 10 18
:2 7 1b 1d :2 1b a 13 1f :2 a
24 22 :2 2d :2 22 a 12 1e 21
:2 12 2a 2d :2 12 11 14 :2 12 11
14 :2 12 11 14 :2 12 32 35 :2 12
3f 42 :2 12 a 7 3d d 16
22 :2 d 25 27 :2 25 a 12 11
14 :2 12 11 14 :2 12 43 46 :2 12
53 56 :2 12 11 14 :2 12 2c 2f
:2 12 38 3b :2 12 11 14 :2 12 a
2a 3d a 12 11 14 :2 12 11
14 :2 12 11 14 :2 12 35 38 :2 12
41 44 :2 12 a :4 7 1f a 13
1f :2 a 25 22 :2 2e :2 22 42 d
19 :2 42 1f 1c :2 28 :2 1c :2 a 41
d 19 :2 41 1f 1c :2 28 :2 1c :3 a
12 1c :3 a :2 13 1f a :4 e :2 d
15 1f :3 d :2 16 22 d 1c 17
23 15 16 17 19 25 27 15
1f 10 15 2a 10 12 1f 22
14 17 14 17 25 10 17 13
1f :3 1d 13 :2 1c 28 13 24 13
:2 1c 28 13 :4 10 d 15 13 :2 1c
28 13 23 :2 10 :2 15 13 :2 1c 28
13 16 :2 20 32 :2 3c 48 4c 5b
5f 60 :2 5b :2 32 :3 16 :2 1c 21 2d
6c 6f :2 2d 7c 7f :2 2d 83 86
:2 2d 92 95 :2 2d 9d a0 :2 2d ac
af b6 bf c2 :2 af :2 2d :2 16 63
:2 13 :2 10 :4 1c :2 10 d :3 22 1c 17
19 15 27 17 19 15 27 16
18 1f 21 14 1b 1d 26 2d
2f 38 3a 10 17 13 1f :3 1d
13 :2 1c 28 13 24 13 :2 1c 28
13 :4 10 d 15 13 :2 1c 28 13
23 :2 10 :2 15 13 :2 1c 28 13 16
:2 20 32 :2 3c 48 4c 5b 5f 60
:2 5b :2 32 :3 16 :2 1c 21 2d 6c 6f
:2 2d 7c 7f :2 2d 83 86 :2 2d 92
95 :2 2d 9d a0 :2 2d ac af b6
bf c2 :2 af :2 2d :2 16 63 :2 13 :2 10
:4 1c :2 10 d :4 22 d 19 d :4 a
3c :2 7 :4 b :2 a 12 1b 22 :2 1b
2a :2 12 2d 2f :2 2d 3a 41 4a
51 :2 4a 59 :2 41 5d :2 3a 64 66
:2 3a :2 16 71 78 81 88 :2 81 90
:2 78 94 :2 71 :2 11 :2 a f :2 a 10
14 12 18 :2 12 a 10 19 1f
26 2f 31 33 :2 2f 36 :2 1f :2 19
3b :2 10 3e 41 :2 3e 17 20 26
2d 36 38 3a :2 36 3d :2 26 :2 20
42 :2 17 45 48 :2 45 :4 10 4b :3 d
12 14 16 :2 12 d 18 e a
d 11 :3 f 10 :2 1a 2c :2 36 42
45 54 58 59 :2 54 :2 2c :3 10 :2 16
1b 27 66 69 :2 27 76 79 :2 27
7d 80 :2 27 8a 8d 96 9e :2 a7
:2 96 b8 :2 8d :2 27 :2 10 5c :2 d 10
:2 1a 2c :2 36 42 45 54 58 59
:2 54 :2 2c :3 10 :2 16 1b 27 66 69
:2 27 :2 10 5c :3 d :2 18 2e :2 37 47
:2 d 15 :2 a 14 1c :2 12 :2 d :2 16
22 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 58 5c 5d
:2 58 :2 2f :3 13 :2 19 1e 2a 69 6c
:2 2a 79 7c :2 2a 80 83 :2 2a 8f
92 :2 2a 9a 9d :2 2a a9 ac b3
bc bf :2 ac :2 2a :2 13 60 :2 10 :2 d
:4 19 :2 d a :4 1e :2 7 a 13 1f
:2 a 24 22 :2 2d :2 22 a 10 :2 a
15 :2 a 15 1d 20 :2 15 :2 a 15
1d 20 :2 15 a 7 3d d 16
22 :2 d 27 25 :2 30 :2 25 a 10
:2 a 12 17 1a :2 12 :2 a 12 17
1a :2 12 :2 a 15 a 45 3d d
16 22 :2 d 27 25 :2 30 :2 25 d
15 :2 d 18 d :4 11 :2 10 1b 23
26 :2 1b :2 10 1b 23 26 :2 1b :2 10
1b 23 26 :2 1b 10 26 :3 d 18
20 23 :2 18 d :4 11 :2 10 1b 23
26 :2 1b :2 10 1b 23 26 :2 1b :2 10
1b 23 26 :2 1b 10 24 :2 d 44
:4 11 :2 10 18 1d 20 :2 18 :2 10 1b
10 29 10 18 1d 20 :2 18 17
1a :2 18 :2 10 1b 10 :4 14 :2 13 1e
26 29 :2 1e :2 13 1e 26 29 :2 1e
:2 13 1e 26 29 :2 1e 13 29 :3 10
1b 23 26 :2 1b 17 1a :2 1b 17
1a :2 1b 17 1a :2 1b 17 1a :2 1b
17 1a :2 1b 10 :4 d :4 11 :2 10 1b
23 26 :2 1b 4f 52 :2 1b 5a 5d
:2 1b 10 24 :2 d :4 a :4 7 a 13
1f :2 a 25 22 :2 2e :2 22 :4 e d
17 21 15 16 23 :2 10 :2 19 25
10 d 15 13 :2 1c 28 13 23
:2 10 :2 15 13 :2 1c 28 13 16 :2 20
32 :2 3c 48 4c 5b 5f 60 :2 5b
:2 32 :3 16 :2 1c 21 2d 6c 6f :2 2d
7c 7f :2 2d 83 86 :2 2d 92 95
:2 2d 9d a0 :2 2d ac af b6 bf
c2 :2 af :2 2d :2 16 63 :2 13 :2 10 :4 1c
:2 10 d :3 26 11 :2 1a 25 23 :2 2e
:2 23 :2 10 18 21 28 :2 21 31 :2 18
:2 10 18 1d 20 :2 18 :2 10 1b 23
26 :2 1b 17 1a :2 1b 2f 32 :2 1b
39 3c :2 1b 10 3e 13 :2 1d 2f
:2 39 45 49 58 5c 5d :2 58 :2 2f
:3 13 :2 19 1e 2a 69 6c :2 2a 79
7c :2 2a 80 83 :2 2a 8d 90 99
a1 :2 aa :2 99 bb :2 90 :2 2a c3 c6
:2 2a ce d1 :2 2a d6 d9 :2 2a e4
e7 :2 2a :2 13 60 :2 10 13 :2 1d 2f
:2 39 45 48 57 5b 5c :2 57 :2 2f
:3 13 :2 19 1e 2a 69 6c :2 2a :2 13
5f :3 10 :2 1b 2f :2 38 48 54 :2 10
:4 d 26 :2 a :4 e d 17 21 15
16 23 :2 10 :2 19 25 10 d 15
13 :2 1c 28 13 23 :2 10 :2 15 13
:2 1c 28 13 16 :2 20 32 :2 3c 48
4c 5b 5f 60 :2 5b :2 32 :3 16 :2 1c
21 2d 6c 6f :2 2d 7c 7f :2 2d
83 86 :2 2d 92 95 :2 2d 9d a0
:2 2d ac af b6 bf c2 :2 af :2 2d
:2 16 63 :2 13 :2 10 :4 1c :2 10 d :3 25
11 :2 1a 25 23 :2 2e :2 23 :2 10 18
1d 20 :2 18 :2 10 1b 23 26 :2 1b
17 1a :2 1b 2f 32 :2 1b 39 3c
:2 1b 10 3e 13 :2 1d 2f :2 39 45
49 58 5c 5d :2 58 :2 2f :3 13 :2 19
1e 2a 69 6c :2 2a 79 7c :2 2a
80 83 :2 2a 8d 90 99 a1 :2 aa
:2 99 bb :2 90 :2 2a c3 c6 :2 2a ce
d1 :2 2a d6 d9 :2 2a e3 e6 :2 2a
:2 13 60 :2 10 13 :2 1d 2f :2 39 45
48 57 5b 5c :2 57 :2 2f :3 13 :2 19
1e 2a 69 6c :2 2a :2 13 5f :3 10
:2 1b 2f :2 38 48 53 :2 10 :4 d 25
:2 a :4 e d 17 21 15 16 23
:2 10 :2 19 25 10 d 15 13 :2 1c
28 13 23 :2 10 :2 15 13 :2 1c 28
13 16 :2 20 32 :2 3c 48 4c 5b
5f 60 :2 5b :2 32 :3 16 :2 1c 21 2d
6c 6f :2 2d 7c 7f :2 2d 83 86
:2 2d 92 95 :2 2d 9d a0 :2 2d ac
af b6 bf c2 :2 af :2 2d :2 16 63
:2 13 :2 10 :4 1c :2 10 d :3 25 11 :2 1a
25 23 :2 2e :2 23 :2 10 18 1d 20
:2 18 :2 10 1b 23 26 :2 1b 17 1a
:2 1b 2f 32 :2 1b 39 3c :2 1b 10
3e 13 :2 1d 2f :2 39 45 49 58
5c 5d :2 58 :2 2f :3 13 :2 19 1e 2a
69 6c :2 2a 79 7c :2 2a 80 83
:2 2a 8d 90 99 a1 :2 aa :2 99 bb
:2 90 :2 2a c3 c6 :2 2a ce d1 :2 2a
d6 d9 :2 2a e3 e6 :2 2a :2 13 60
:2 10 13 :2 1d 2f :2 39 45 48 57
5b 5c :2 57 :2 2f :3 13 :2 19 1e 2a
69 6c :2 2a :2 13 5f :3 10 :2 1b 2f
:2 38 48 53 :2 10 :4 d 25 :2 a d
16 27 :2 d 2a 2c :2 2a 32 3b
4c :2 32 4f 52 :2 4f :3 d 18 20
23 :2 18 3b 3e 46 4f 61 :2 46
:2 3e :2 18 65 68 :2 18 d 56 :2 a
3e :2 7 :4 b :2 a 15 1e 25 :2 1e
33 :2 15 :2 a f :2 a 10 14 12
1c :2 12 a d 1a :2 23 2c 30
34 35 36 :2 34 39 :2 1a d 17
21 15 16 23 :2 10 :2 19 25 10
d 15 13 :2 1c 28 13 23 :2 10
:2 15 13 :2 1c 28 13 16 :2 20 32
:2 3c 48 4c 5b 5f 60 :2 5b :2 32
:3 16 :2 1c 21 2d 6c 6f :2 2d 7c
7f :2 2d 83 86 :2 2d 92 95 :2 2d
9d a0 :2 2d ac af b6 bf c2
:2 af :2 2d :2 16 63 :2 13 :2 10 :4 1c :2 10
d :3 1c 11 :2 1a 26 23 :2 2f :2 23
10 13 :2 1d 2f :2 39 45 49 58
5c 5d :2 58 :2 2f :3 13 :2 19 1e 2a
69 6c :2 2a 79 7c :2 2a 80 83
:2 2a 8d 90 99 a1 :2 aa :2 99 bb
:2 90 :2 2a c3 c6 :2 2a ce d1 :2 2a
d6 d9 :2 2a e3 e6 :2 2a :2 13 60
:2 10 13 :2 1d 2f :2 39 45 48 57
5b 5c :2 57 :2 2f :3 13 :2 19 1e 2a
69 6c :2 2a :2 13 5f :3 10 :2 1b 2f
:2 38 48 53 :2 10 3f :3 d 18 20
23 :2 18 14 17 :2 18 14 17 :2 18
14 17 :2 18 31 34 :2 18 3b 3e
:2 18 d 10 19 25 :2 10 2a 28
:2 33 :2 28 10 1b 23 26 :2 1b 10
43 10 1b 23 26 :2 1b 10 :4 d
10 19 25 :2 10 2a 28 :2 33 :2 28
:2 10 43 :2 d a f 11 13 :2 f
a 1c e a 22 :2 7 a 13
1f :2 a 25 22 :2 2e :2 22 a 15
1e 25 :2 1e 34 :2 15 :2 a f :2 a
10 14 12 1c :2 12 a d 1a
:2 23 2c 30 34 35 36 :2 34 39
:2 1a d 17 21 15 16 23 :2 10
:2 19 25 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 5b 5f
60 :2 5b :2 32 :3 16 :2 1c 21 2d 6c
6f :2 2d 7c 7f :2 2d 83 86 :2 2d
92 95 :2 2d 9d a0 :2 2d ac af
b6 bf c2 :2 af :2 2d :2 16 63 :2 13
:2 10 :4 1c :2 10 d :3 1c 11 :2 1a 26
23 :2 2f :2 23 10 13 :2 1d 2f :2 39
45 49 58 5c 5d :2 58 :2 2f :3 13
:2 19 1e 2a 69 6c :2 2a 79 7c
:2 2a 80 83 :2 2a 8d 90 99 a1
:2 aa :2 99 bb :2 90 :2 2a c3 c6 :2 2a
ce d1 :2 2a d6 d9 :2 2a e3 e6
:2 2a :2 13 60 :2 10 13 :2 1d 2f :2 39
45 48 57 5b 5c :2 57 :2 2f :3 13
:2 19 1e 2a 69 6c :2 2a :2 13 5f
:3 10 :2 1b 2f :2 38 48 53 :2 10 3f
:3 d 18 20 23 :2 18 14 17 :2 18
14 17 :2 18 14 17 :2 18 31 34
:2 18 3b 3e :2 18 14 17 :2 18 d
a f 11 13 :2 f a 1c e
a 3e :3 7 f 14 17 :2 f 7
:5 4 c 11 14 :2 c :2 4 9 :2 11
:2 4 d 16 :2 7 a e :2 a :2 13
1f a 14 a :2 13 1f a :4 7
b :2 14 20 1d :2 29 :2 1d :3 a 39
:2 7 a 13 1b :2 a 1e 20 :2 1e
26 2f 3b :2 26 41 3e :2 4a :2 3e
:2 a 62 d 19 :2 62 1f 1c :2 28
:2 1c :2 a 42 d 19 :2 42 1f 1c
:2 28 :2 1c :4 3c 3b :2 42 41 :3 a f
:3 a 10 19 :2 a d 11 :2 d :2 16
22 d 17 d :2 16 22 d :4 a
e :2 17 22 20 :2 2b :2 20 d 10
16 :3 10 :2 19 25 10 d 15 13
:2 1c 28 13 23 :2 10 :2 15 13 :2 1c
28 13 16 :2 20 32 :2 3c 48 4c
5b 5f 60 :2 5b :2 32 :3 16 :2 1c 21
2d 6c 6f :2 2d 7c 7f :2 2d 83
86 :2 2d 92 95 :2 2d 9d a0 :2 2d
ac af b6 bf c2 :2 af :2 2d :2 16
63 :2 13 :2 10 :4 1c :2 10 d :3 3b 12
d 3b 10 16 :3 10 :2 19 25 10
d 15 13 :2 1c 28 13 23 :2 10
:2 15 13 :2 1c 28 13 16 :2 20 32
:2 3c 48 4c 5b 5f 60 :2 5b :2 32
:3 16 :2 1c 21 2d 6c 6f :2 2d 7c
7f :2 2d 83 86 :2 2d 92 95 :2 2d
9d a0 :2 2d ac af b6 bf c2
:2 af :2 2d :2 16 63 :2 13 :2 10 :4 1c :2 10
d :7 a 51 :3 7 16 :2 7 17 7
a 13 1b :2 a 1e 20 :2 1e d
14 20 23 :2 d 26 28 :2 26 d
:3 1c :2 d 19 d 2c 10 1e 27
2f :2 1e :2 10 33 36 :2 33 1a 20
38 18 19 26 17 25 17 :2 13
:2 1c 28 13 10 18 16 :2 1f 2b
16 26 :2 13 :2 18 16 :2 1f 2b 16
19 :2 23 35 :2 3f 4b 4f 5e 62
63 :2 5e :2 35 :3 19 :2 1f 24 30 6f
72 :2 30 7f 82 :2 30 86 89 :2 30
95 98 :2 30 a0 a3 :2 30 af b2
b9 c2 c5 :2 b2 :2 30 :2 19 66 :2 16
:2 13 :4 1f :2 13 10 :3 38 13 1c 28
:2 13 2b 2d :2 2b 16 1c :3 16 :2 1f
2b 16 13 1b 19 :2 22 2e 19
29 :2 16 :2 1b 19 :2 22 2e 19 1c
:2 26 38 :2 42 4e 52 61 65 66
:2 61 :2 38 :3 1c :2 22 27 33 72 75
:2 33 82 85 :2 33 89 8c :2 33 98
9b :2 33 a3 a6 :2 33 b2 b5 bc
c5 c8 :2 b5 :2 33 :2 1c 69 :2 19 :2 16
:4 22 :2 16 13 :3 2f 16 :2 20 32 :2 3c
48 4c 5b 5f 60 :2 5b :2 32 :3 16
:2 1c 21 2d 6c 6f :2 2d 7c 7f
:2 2d 83 86 :2 2d 90 93 9c a4
:2 ad :2 9c ce :2 93 :2 2d d6 d9 :2 2d
e1 e4 :2 2d e9 ec :2 2d f5 f8
:2 2d :2 16 63 :2 13 16 :2 20 32 :2 3c
48 4b 5a 5e 5f :2 5a :2 32 :3 16
:2 1c 21 2d 6c 6f :2 2d :2 16 62
:3 13 :2 1e 32 :2 3b 5b 1b :2 13 2f
:2 10 38 :2 d :5 a 16 22 2f 3d
48 52 5e 1b 28 36 43 51
1b 28 34 3e 46 4f 57 1b
27 33 40 4b 56 1b 25 30
3c 48 51 5c :2 48 60 1b 21
:2 16 a d 16 19 :2 16 10 16
:3 10 :2 19 25 10 d 15 13 :2 1c
28 13 23 :2 10 :2 15 13 :2 1c 28
13 16 :2 20 32 :2 3c 48 4c 5b
5f 60 :2 5b :2 32 :3 16 :2 1c 21 2d
6c 6f :2 2d 7c 7f :2 2d 83 86
:2 2d 92 95 :2 2d 9d a0 :2 2d ac
af b6 bf c2 :2 af :2 2d :2 16 63
:2 13 :2 10 :4 1c :2 10 d :3 1b 10 1b
19 :2 24 :2 19 10 1c :2 25 10 2e
:2 d 10 :2 1a 2c :2 36 42 46 55
59 5a :2 55 :2 2c :3 10 :2 16 1b 27
66 69 :2 27 76 79 :2 27 7d 80
:2 27 8a 8d 96 9e :2 96 a9 :2 8d
:2 27 b1 b4 :2 27 bc bf :2 27 c4
c7 :2 27 d3 d6 :2 27 :2 10 5d :2 d
10 :2 1a 2c :2 36 42 45 54 58
59 :2 54 :2 2c :3 10 :2 16 1b 27 66
69 :2 27 :2 10 5c :3 d :2 18 2c 36
43 :2 d a 1b 11 18 24 27
:2 11 2a 2c :2 2a 10 36 3d 4b
4e :2 36 51 53 :2 51 35 :2 10 17
24 39 46 53 10 17 13 1f
:3 1d 13 :2 1c 28 13 24 13 :2 1c
28 13 :4 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 5b 5f
60 :2 5b :2 32 :3 16 :2 1c 21 2d 6c
6f :2 2d 7c 7f :2 2d 83 86 :2 2d
92 95 :2 2d 9d a0 :2 2d ac af
b6 bf c2 :2 af :2 2d :2 16 63 :2 13
:2 10 :4 1c :2 10 d :4 58 1b :4 a 22
d 16 21 :2 d 27 24 :2 30 :2 24
d 1b :2 d 1c d 44 :3 a 16
22 2f 3d 48 52 5e 1b 28
36 43 51 1b 28 34 3e 46
4f 57 1b 27 33 40 4b 56
1b 25 30 3c 48 4b 4e 54
:2 16 a d 16 18 :2 16 d 16
1c 1e :2 16 d 1a :2 a d 15
13 1e 2a :2 15 :2 13 :2 d 2d :2 a
:4 7 9 7 :3 4 8 1 7 d
:3 7 :2 10 1c 7 4 c a :2 13
1f a 1a :2 7 :2 c a :2 13 1f
a d :2 17 29 :2 33 3f 43 52
56 57 :2 52 :2 29 :3 d :2 13 18 24
63 66 :2 24 73 76 :2 24 7a 7d
:2 24 89 8c :2 24 94 97 :2 24 a3
a6 ad b6 b9 :2 a6 :2 24 :2 d 5a
:2 a :2 7 :4 13 :2 7 4 :3 1 8 13
11 :2 1c :2 11 7 37 40 43 :2 40
36 4b a 12 :2 4b 15 17 :2 15
4a :2 36 35 :2 7 a :2 14 26 :2 30
3c 40 4f 53 54 :2 4f :2 26 :3 a
:2 10 15 21 60 63 :2 21 70 73
:2 21 77 7a :2 21 84 87 90 98
:2 90 a3 :2 87 :2 21 ab ae :2 21 b6
b9 :2 21 be c1 :2 21 cd d0 :2 21
:2 a 57 :2 7 a :2 14 26 :2 30 3c
3f 4e 52 53 :2 4e :2 26 :3 a :2 10
15 21 60 63 :2 21 :2 a 56 :3 7
:2 12 26 30 3d :2 7 1b :2 4 7
11 14 :2 11 7 12 :2 7 15 :2 7
14 :2 7 c :4 7 c :2 7 10 a
:7 c :3 a d 12 :2 d :2 16 22 d
18 d :2 16 22 d :4 a e :2 17
23 20 :2 2c :2 20 :3 d 3c :2 a :4 e
d 29 :2 34 3b 46 :2 29 50 52
:2 50 :2 d 14 1d 24 :2 1d 2e :2 14
31 33 :2 31 :3 d 18 d 35 :2 a
d :2 18 1f 2b :2 d 35 38 :2 35
10 12 15 :2 12 1a 13 1d 15
21 15 20 15 22 15 21 15
22 19 21 13 1a 16 22 :3 20
16 :2 1f 2b 16 27 16 :2 1f 2b
16 :4 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 5e 62 63
:2 5e :2 35 :3 19 :2 1f 24 30 6f 72
:2 30 7f 82 :2 30 86 89 :2 30 95
98 :2 30 a0 a3 :2 30 af b2 b9
c2 c5 :2 b2 :2 30 :2 19 66 :2 16 :2 13
:4 1f :2 13 10 :4 17 :3 d 1c :2 d 1d
:2 d 1c :2 d 1d :2 d 1d :2 d 1d
:2 d 1b :2 d 12 d 3a :2 a d
f 12 :2 f :4 19 18 :2 d 17 10
:5 12 16 1e 10 17 13 1f :3 1d
13 :2 1c 28 13 24 13 :2 1c 28
13 :4 10 d 15 13 :2 1c 28 13
23 :2 10 :2 15 13 :2 1c 28 13 16
:2 20 32 :2 3c 48 4c 5b 5f 60
:2 5b :2 32 :3 16 :2 1c 21 2d 6c 6f
:2 2d 7c 7f :2 2d 83 86 :2 2d 92
95 :2 2d 9d a0 :2 2d ac af b6
bf c2 :2 af :2 2d :2 16 63 :2 13 :2 10
:4 1c :2 10 d :4 30 :2 a :4 e d :4 11
:2 10 1f 26 32 35 :2 26 3a 3d
:2 1f 10 2a :3 d 1c 23 2f 32
:2 23 3e 41 :2 1c d 26 :2 a :4 e
:2 d 1d 24 31 34 :2 24 3f 42
:2 1d d 25 :2 a :4 e d :4 11 :2 10
20 27 34 37 :2 27 3c 3f :2 20
10 2b :3 d 1d 24 31 34 :2 24
41 44 :2 1d d 27 :2 a :4 e :2 d
1d 24 31 34 :2 24 40 43 :2 1d
d 26 :2 a :4 e d :4 11 :2 10 1f
26 32 35 :2 26 3a 3d :2 1f 10
2a :3 d 1c 23 2f 32 :2 23 3e
41 :2 1c d 26 :2 a :4 e d :4 11
:2 10 20 27 34 37 :2 27 3c 3f
:2 20 10 2b :3 d 1d 24 31 34
:2 24 41 44 :2 1d d 27 :3 a f
11 13 :2 f :2 a 17 a 7 b
16 a 10 :3 a :2 13 1f a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 55 59 5a :2 55 :2 2c :3 10
:2 16 1b 27 66 69 :2 27 76 79
:2 27 7d 80 :2 27 8c 8f :2 27 97
9a :2 27 a6 a9 b0 b9 bc :2 a9
:2 27 :2 10 5d :2 d :2 a :4 16 :2 a 7
:3 16 a c f :2 c 14 d 17
f 1b f 1a f 1c f 1b
f 1c 13 1b d 14 10 1c
:3 1a 10 :2 19 25 10 21 10 :2 19
25 10 :4 d a 12 10 :2 19 25
10 20 :2 d :2 12 10 :2 19 25 10
13 :2 1d 2f :2 39 45 49 58 5c
5d :2 58 :2 2f :3 13 :2 19 1e 2a 69
6c :2 2a 79 7c :2 2a 80 83 :2 2a
8f 92 :2 2a 9a 9d :2 2a a9 ac
b3 bc bf :2 ac :2 2a :2 13 60 :2 10
:2 d :4 19 :2 d a :4 11 :2 7 16 :2 4
:4 8 7 a 13 1b :2 a 1e 20
:2 1e 14 1d 21 38 12 13 1e
11 22 2a 31 49 51 58 :2 d
:2 16 22 d a 12 10 :2 19 25
10 20 :2 d :2 12 10 :2 19 25 10
13 :2 1d 2f :2 39 45 49 58 5c
5d :2 58 :2 2f :3 13 :2 19 1e 2a 69
6c :2 2a 79 7c :2 2a 80 83 :2 2a
8f 92 :2 2a 9a 9d :2 2a a9 ac
b3 bc bf :2 ac :2 2a :2 13 60 :2 10
:2 d :4 19 :2 d a :4 22 a 18 :2 a
:4 7 24 :2 4 e 1b 26 30 33
4b 5a c 14 17 1c 2b 31
34 42 45 d 10 1d :2 7 :2 10
1c 7 4 c a :2 13 1f a
1a :2 7 :2 c a :2 13 1f a d
:2 17 29 :2 33 3f 43 52 56 57
:2 52 :2 29 :3 d :2 13 18 24 63 66
:2 24 73 76 :2 24 7a 7d :2 24 89
8c :2 24 94 97 :2 24 a3 a6 ad
b6 b9 :2 a6 :2 24 :2 d 5a :2 a :2 7
:4 13 :2 7 4 :3 1 7 10 1c :2 7
21 1f :2 2a :2 1f 7 f 19 :3 7
:2 10 1c 7 16 11 :4 c f 10
18 f 1e a 11 d 19 :3 17
d :2 16 22 d 1e d :2 16 22
d :4 a 7 f d :2 16 22 d
1d :2 a :2 f d :2 16 22 d 10
:2 1a 2c :2 36 42 46 55 59 5a
:2 55 :2 2c :3 10 :2 16 1b 27 66 69
:2 27 76 79 :2 27 7d 80 :2 27 8c
8f :2 27 97 9a :2 27 a6 a9 b0
b9 bc :2 a9 :2 27 :2 10 5d :2 d :2 a
:4 16 :2 a 7 :3 3a b :2 14 1f 1d
:2 28 :2 1d a 19 14 f 13 24
f 13 24 f 13 24 f 13
24 12 13 1c 16 d 14 10
1c :3 1a 10 :2 19 25 10 21 10
:2 19 25 10 :4 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 58
5c 5d :2 58 :2 2f :3 13 :2 19 1e 2a
69 6c :2 2a 79 7c :2 2a 80 83
:2 2a 8f 92 :2 2a 9a 9d :2 2a a9
ac b3 bc bf :2 ac :2 2a :2 13 60
:2 10 :2 d :4 19 :2 d a :3 38 e :2 17
22 20 :2 2b :2 20 d 1c 17 15
16 1e 19 10 17 13 1f :3 1d
13 :2 1c 28 13 24 13 :2 1c 28
13 :4 10 d 15 13 :2 1c 28 13
23 :2 10 :2 15 13 :2 1c 28 13 16
:2 20 32 :2 3c 48 4c 5b 5f 60
:2 5b :2 32 :3 16 :2 1c 21 2d 6c 6f
:2 2d 7c 7f :2 2d 83 86 :2 2d 92
95 :2 2d 9d a0 :2 2d ac af b6
bf c2 :2 af :2 2d :2 16 63 :2 13 :2 10
:4 1c :2 10 d :4 3b :2 a 38 :3 7 c
1e 2a 3c 45 47 57 65 68
73 76 82 85 90 93 9f b2
7 a 1a 27 2a 3a 7 17
1a 2e 31 43 46 5b 60 6f
83 85 94 ad 7 17 1a 2e
31 43 46 5b 60 6f 83 85
94 ad 7 17 1a 2e 31 43
46 5b 60 6f 83 85 94 ad
7 17 1a 2e 31 43 46 5b
60 6f 83 85 94 ad 7 a
15 18 24 27 32 35 3d 40
51 54 66 69 77 87 94 9f
ac af bc bf c9 cc d6 d
1c 2e 39 3c 4c 5d 73 7e
8b 9b 9e a9 b8 c7 d0 e0
ef 15 24 4b c 1e 23 28
37 3d 40 49 4e 7 c 12
17 2b 31 34 3f 50 53 5f
62 7 c 15 1a 1c 29 7
c 1d 22 24 2c 2e 7 c
12 17 1d 22 24 2c 2e 7
c 12 17 29 2f 31 3a 3d
7 c 12 17 27 2d 2f 38
3b 7 c 12 17 21 27 2a
37 39 12 15 2b 2e 45 48
5e 61 17 7 17 a :2 14 26
:2 30 3c 3f 4e 52 53 :2 4e :2 26
:3 a :2 10 15 21 60 63 :2 21 :2 a
56 :3 7 e 7 3a a 14 16
:2 14 a f 21 24 2e 40 49
4b 58 66 69 74 77 83 86
91 94 a0 a2 a d 1d 2a
2d 3d a 1a 1d 33 36 50
52 60 73 75 83 aa b5 bf
d7 ec f9 112 11d 127 13f 154
161 163 179 17c 192 195 1a7 1aa
1b9 1d2 a 1a 1d 33 36 50
52 60 73 75 83 aa b5 bf
d7 ec f9 112 11d 127 13f 154
161 163 179 17c 192 195 1a7 1aa
1b9 1d2 a 1a 1d 33 36 50
52 60 73 75 83 aa b5 bf
d7 ec f9 112 11d 127 13f 154
161 163 179 17c 192 195 1a7 1aa
1b9 1d2 a 1a 1d 33 36 50
52 60 73 75 83 aa b5 bf
d7 ec f9 112 11d 127 13f 154
161 163 179 17c 192 195 1a7 1aa
1b9 1d2 a d 19 1c 28 2b
36 39 41 44 55 58 6a 6d
7b 8b 98 a3 b0 b3 c0 c3
cd d0 da a d 1a 24 26
32 35 45 50 53 5e 61 77
82 8f 9f a2 ad bc bf c8
cb d5 de e1 f0 f3 104 a
c 1b 1d 2d f 1e 21 26
30 35 38 3f 41 a f 19
1f 22 2d 2f a f 15 1a
24 2a 2d 38 3b a f 19
1e 21 28 2a 36 3b 41 46
50 56 59 65 67 a f 15
1a 2e 34 37 42 53 56 62
65 a f 15 1a 2c 32 34
40 43 a f 15 1a 2c 32
35 3f 42 a f 15 1a 2a
30 32 3e 41 a f 15 1a
2a 30 33 3d 40 13 1a a
1a d :2 17 29 :2 33 3f 42 51
55 56 :2 51 :2 29 :3 d :2 13 18 24
63 66 :2 24 :2 d 59 :3 a 11 a
18 a 12 1c :3 a :2 13 1f a
19 14 1c 2a 38 46 12 27
d 14 10 1c :3 1a 10 :2 19 25
10 21 10 :2 19 25 10 :4 d a
12 10 :2 19 25 10 20 :2 d :2 12
10 :2 19 25 10 13 :2 1d 2f :2 39
45 49 58 5c 5d :2 58 :2 2f :3 13
:2 19 1e 2a 69 6c :2 2a 79 7c
:2 2a 80 83 :2 2a 8f 92 :2 2a 9a
9d :2 2a a9 ac b3 bc bf :2 ac
:2 2a :2 13 60 :2 10 :2 d :4 19 :2 d a
:3 7 19 15 39 3d 4e d 11
22 31 35 46 d 11 22 12
13 29 12 27 16 d 14 10
1c :3 1a 10 :2 19 25 10 21 10
:2 19 25 10 :4 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 58
5c 5d :2 58 :2 2f :3 13 :2 19 1e 2a
69 6c :2 2a 79 7c :2 2a 80 83
:2 2a 8f 92 :2 2a 9a 9d :2 2a a9
ac b3 bc bf :2 ac :2 2a :2 13 60
:2 10 :2 d :4 19 :2 d a :3 7 a f
21 24 2e 40 49 4b 58 66
69 74 77 83 86 91 94 a0
a2 a d 1d 2a 2d 3d a
1a 1d 31 34 f 11 25 27
41 43 51 64 66 74 9b a6
b0 c8 dd ea 103 10e 118 130
145 152 154 f 13 27 2b 45
47 55 68 6a 78 9f aa b4
cc e1 ee 107 112 11c 134 149
156 158 f 13 22 14 a 1a
1d 32 35 f 11 27 29 43
45 53 66 68 76 9d a8 b2
ca df ec 105 110 11a 132 147
154 156 f 13 27 2b 45 47
55 68 6a 78 9f aa b4 cc
e1 ee 107 112 11c 134 149 156
158 f 13 22 14 a 1a 1d
31 34 f 11 25 27 41 43
51 64 66 74 9b a6 b0 c8
dd ea 103 10e 118 130 145 152
154 f 13 27 2b 45 47 55
68 6a 78 9f aa b4 cc e1
ee 107 112 11c 134 149 156 158
f 13 22 14 a 1a 1d 31
34 f 11 25 27 41 43 51
64 66 74 9b a6 b0 c8 dd
ea 103 10e 118 130 145 152 154
f 13 27 2b 45 47 55 68
6a 78 9f aa b4 cc e1 ee
107 112 11c 134 149 156 158 f
13 22 14 a d 18 1b 27
2a 35 38 40 43 54 57 69
6c 7a 8a 97 a2 af b2 bf
c2 cc cf d9 a d 1a 24
26 32 35 45 50 53 5e 61
77 82 8f 9f a2 ad a d
16 19 23 2c 2f 3e 41 52
a c 1b 1d 2d f 1e a
f 14 26 2d 30 37 3b 46
4a a f 14 26 2d 30 37
3b 46 4a a f 19 1e 21
28 2a a f 19 1f 22 2d
2f a f 15 1a 24 2a 2d
38 3b a f 19 1e 21 28
2a 36 3b 41 46 50 56 59
65 67 a f 15 1a 2e 34
37 42 53 56 62 65 a f
15 1a 24 2a 2c 37 3a a
f 15 1a 2c 32 34 40 43
4e 53 59 5e 70 76 79 83
86 a f 15 1a 2a 30 32
3e 41 4c 51 57 5c 6c 72
75 7f 82 a f 15 1a 20
25 27 2f 31 15 18 2e 31
48 4b 61 64 13 1a a 1a
d :2 17 29 :2 33 3f 42 51 55
56 :2 51 :2 29 :3 d :2 13 18 24 63
66 :2 24 :2 d 59 :3 a 11 a :4 7
:4 4 1 9 12 9 a :2 14 26
:2 30 3c 40 4f 53 54 :2 4f :2 26
:3 a :2 10 15 21 60 63 :2 21 :3 a
:2 10 15 39 5c 5f :2 68 :2 39 76
79 :2 39 81 84 :2 8d :2 39 :2 a 57
:3 7 c 15 1e 2c a 13 1e
f e 7 e 7 e 7 29
:2 4 :2 9 7 :2 10 1c :2 7 :2 11 23
:2 2d 39 3d 4c 50 51 :2 4c :2 23
:3 7 :2 d 12 1e 5d 60 :2 1e :3 7
:2 d 12 36 42 45 :2 36 4d 50
:2 36 5c 5f 66 6f 72 :2 5f :2 36
:2 7 54 :2 4 7 c 1b a 13
1e f e 7 e 7 e 7
:2 4 :4 10 :2 4 :8 1 
366b
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 :4 7 :4 8
:4 9 :4 a :4 b :4 c
:4 d :4 e :4 f :4 10
:4 11 :4 12 :4 13 :4 14
:4 15 :4 16 :4 17 :4 18
:4 19 :4 1a :4 1b :4 1c
:4 1d 2 :4 1e :2 1
:7 20 :7 21 :7 22 :7 23
:7 24 :7 25 :7 26 :7 27
:7 28 :7 29 :7 2a :7 2b
:7 2c :7 2d :7 2e :7 2f
:7 30 :7 31 :7 32 :7 33
:7 34 :5 35 :7 36 :7 37
:7 38 :7 39 :7 3a :7 3b
:7 3c :7 3d :7 3e :7 3f
:7 40 :6 41 :6 42 :6 43
:6 44 :6 45 :6 46 :6 47
:6 48 :6 49 :6 4a :6 4b
:6 4c :6 4d :6 4e :6 4f
:6 50 :6 51 :6 52 :6 53
:6 54 :6 55 :6 56 :6 57
:6 58 :6 59 :6 5a :6 5b
:6 5c :6 5d :6 5e :6 5f
:6 60 :6 61 :7 62 :6 63
:6 64 :6 65 :7 66 :7 67
:7 68 :6 69 :6 6a :7 6b
:7 6c :7 6d :7 6e :7 6f
:7 70 :6 71 :6 72 :6 73
:6 74 :6 75 :6 76 :6 77
:6 78 :7 79 :6 7a :5 7b
:5 7c :5 7d :5 7e :5 7f
:5 80 :5 81 :7 84 :2 86
0 :2 86 87 88
:2 89 :2 8a :2 8b 87
:5 86 :2 8d 0 :2 8d
8f 90 :2 91 :2 92
:2 93 :2 94 :2 95 :2 96
97 98 8f :5 8d
:7 9b :7 9e :7 a1 :7 a3
:b a5 :11 a6 a7 :2 a6
:2 a7 :2 a6 :2 a7 :2 a6
:2 a8 :2 a6 :2 a8 :2 a6
:2 a8 :2 a6 :2 a8 :2 a6
:2 a9 :2 a6 :2 a9 :2 a6
:2 a9 :2 a6 :2 a9 :2 a6
:2 aa :2 a6 :9 aa :2 a6
:2 ab :2 a6 :9 ab :2 a6
:2 ac :2 a6 :9 ac :2 a6
:2 ad :2 a6 :9 ad :2 a6
:2 ae :2 a6 :9 ae :2 a6
:2 af :2 a6 :9 af :2 a6
:2 b0 :2 a6 :9 b0 :2 a6
:2 b1 :2 a6 :9 b1 :2 a6
:2 b2 :2 a6 :2 b2 :2 a6
:2 b2 :2 a6 :2 b2 :2 a6
:2 b3 :2 a6 :2 b3 :2 a6
:2 b3 :2 a6 :2 b3 :2 a6
:2 b4 :2 a6 :2 b4 :2 a6
:2 b4 :2 a6 :2 b4 :2 a6
:2 b5 :2 a6 :2 b5 :2 a6
:2 b5 :2 a6 :2 b5 :2 a6
:2 b6 :2 a6 :2 b6 :2 a6
:2 b6 :2 a6 :2 b6 :2 a6
:2 b7 :2 a6 :9 b7 :2 a6
:2 b8 :2 a6 :2 b8 :2 a6
:2 b8 :2 a6 :2 b8 :2 a6
:2 b9 :2 a6 :2 b9 :2 a6
:2 b9 :2 a6 :2 b9 :2 a6
:2 ba :2 a6 :2 ba :2 a6
:2 ba :2 a6 :2 ba :2 a6
:2 bb :2 a6 :2 bb :2 a6
:2 bb :2 a6 :2 bb :2 a6
:2 bc :2 a6 :9 bc :2 a6
:2 bd :2 a6 :9 bd :2 a6
:2 be :2 a6 :9 be :2 a6
:2 bf :2 a6 :9 bf :2 a6
:2 c0 :2 a6 :2 c0 :2 a6
:2 c0 :2 a6 :2 c0 :2 a6
:2 c1 :2 a6 :2 c1 :2 a6
:2 c1 :2 a6 :2 c1 :2 a6
:2 c1 :4 a6 :3 a5 :11 c4
:b c5 :3 c4 :3 cd :3 ce
:3 cf :3 d0 :3 d1 :3 d2
:3 d3 :3 d4 :3 d5 :3 d6
:3 d8 :3 da :3 dc :3 de
:3 df :3 e0 :3 e1 :3 e2
:3 e3 :3 e4 :3 e5 :3 e6
:3 e8 :3 e9 :14 ee :11 ef
:20 f0 :3 ef :11 f2 :b f3
:3 f2 :9 f5 :3 ee :1e f8
:c f9 :2 f8 :14 fa :11 fb
:20 fc :3 fb :11 fe :b ff
:3 fe :9 101 102 fa
:e 102 :11 103 :20 104 :3 103
:11 106 :b 107 :3 106 :9 109
102 :3 fa f9 :2 f8
:1f 119 :11 11a :20 11b :3 11a
:11 11d :b 11e :3 11d :9 120
:3 119 :6 124 :3 125 124
:3 127 126 :3 124 :14 12a
:6 12b :2 12a :6 12d 12e
12f 12d :5 130 12c
132 :5 133 :3 132 :2 134
:5 136 :11 137 :24 138 :3 137
:2 135 :6 134 131 :3 12b
:16 13d :11 13e :20 13f :3 13e
:11 141 :b 142 :3 141 :9 144
:3 13d :3 146 12b :2 12a
:e 149 :4 14b :5 14c 14a
14e :5 14f :3 14e :2 150
:5 152 :11 153 :24 154 :3 153
:2 151 :6 150 14d :6 149
:3 15a :14 15e :d 160 :14 162
:d 164 :13 166 :9 168 :3 169
:2 16b 16c :2 16d 16b
:5 16e 16a 170 :5 171
:3 170 :2 172 :5 174 :11 175
:24 176 :3 175 :2 173 :6 172
16f :3 168 :9 17b :2 17d
17e :2 17f 17d :5 180
17c 182 :5 183 :3 182
:2 184 :5 186 :11 187 :24 188
:3 187 :2 185 :6 184 181
:3 17b :b 18d :11 18e :30 18f
:3 18e :11 191 :b 192 :3 191
:a 194 :3 18d :3 17b :a 198
:9 199 :a 19a :9 19b :11 19c
:20 19d :3 19c :11 19f :b 1a0
:3 19f :9 1a2 :3 19b :5 1a4
:3 199 :3 1a6 :9 1a8 :3 1a9
1a8 :11 1ac :20 1ad :3 1ac
:11 1af :b 1b0 :3 1af :9 1b2
1aa :3 1a8 :7 1ba 1bb
:3 1ba :9 1bd :11 1be :2e 1bf
:3 1be :11 1c1 :b 1c2 :3 1c1
:c 1c4 :3 1bd :b 1c6 :11 1c7
:30 1c8 :3 1c7 :11 1ca :b 1cb
:3 1ca :a 1cd :3 1c6 :b 1d0
:c 1d1 1d4 :2 1d5 :2 1d6
1d7 1d9 1da 1db
1dc 1dd :2 1de 1d4
:5 1df 1d2 1e1 :5 1e2
:3 1e1 :2 1e3 :5 1e5 :11 1e6
:24 1e7 :3 1e6 :2 1e4 :6 1e3
1e0 :4 1d1 :2 1ee :2 1ef
:2 1f0 :2 1f1 :3 1f2 :3 1f3
:2 1f4 :4 1f5 1f7 1f8
1f9 1fa 1fb 1fc
1fd 1fe :4 1ff :3 200
:4 201 1ee :5 202 1ec
204 :5 205 :3 204 :2 206
:5 208 :11 209 :24 20a :3 209
:2 207 :6 206 203 :4 1eb
:3 1d1 :a 210 :11 211 :30 212
:3 211 :11 214 :b 215 :3 214
:a 217 :3 210 219 1d0
:b 219 21c 21d 21e
220 221 222 223
:2 224 21c :5 225 21a
227 :5 228 :3 227 :2 229
:5 22b :11 22c :24 22d :3 22c
:2 22a :6 229 226 :3 219
:a 232 :11 233 :30 234 :3 233
:11 236 :b 237 :3 236 :a 239
:3 232 :2 241 242 :2 243
:2 245 :2 246 247 249
24a 24b 24c 24d
:2 24e :2 24f 240 :5 250
23e 252 :5 253 :3 252
:2 254 :5 256 :11 257 :24 258
:3 257 :2 255 :6 254 251
:3 219 :a 25d :11 25e :30 25f
:3 25e :11 261 :b 262 :3 261
:a 264 :3 25d :5 267 :5 26b
:6 26c :3 267 219 1d0
:11 270 :30 271 :3 270 :11 273
:b 274 :3 273 :a 276 26e
:3 1d0 :b 27a 27e :3 27f
280 :4 281 :4 282 284
285 286 287 288
289 :2 28a 27e :5 28b
27c 28d :5 28e :3 28d
:2 28f :5 291 :11 292 :24 293
:3 292 :2 290 :6 28f 28c
:3 27a :a 298 :11 299 :30 29a
:3 299 :11 29c :b 29d :3 29c
:a 29f 2a0 298 :a 2a0
:14 2a1 :2 2a2 :2 2a1 :8 2a2
:5 2a1 2a2 :2 2a3 :2 2a2
:8 2a3 :2 2a1 :11 2a4 :30 2a5
:3 2a4 :11 2a7 :b 2a8 :3 2a7
:a 2aa 2a3 :2 2a1 2a0
:3 298 :3 27a :3 168 :9 2b2
:13 2b8 :2 2b9 :2 2b8 :4 2b9
:2 2b8 :11 2ba :30 2bb :3 2ba
:11 2bd :b 2be :3 2bd :7 2c0
2c1 :2 2c0 2b9 :2 2b8
:b 2c5 2c8 :2 2c9 :2 2ca
2cc 2cd 2ce 2cf
:2 2d0 2c8 :5 2d1 2c6
2d3 :5 2d4 :3 2d3 :2 2d5
:5 2d7 :11 2d8 :24 2d9 :3 2d8
:2 2d6 :6 2d5 2d2 :3 2c5
:a 2e0 2e6 2e7 :3 2e8
2e3 :5 2e9 2e1 2eb
:5 2ec :3 2eb :2 2ed :5 2ef
:11 2f0 :24 2f1 :3 2f0 :2 2ee
:6 2ed 2ea :3 2e0 :a 2f6
:11 2f7 :30 2f8 :3 2f7 :11 2fa
:b 2fb :3 2fa :a 2fd :3 2f6
:3 2e0 :3 2c5 :3 2b2 :14 304
:3 305 :3 304 :b 308 :5 30a
:3 30b :3 30c 30a :3 30e
:3 30f 30d :3 30a :3 312
308 :5 315 :3 317 :e 319
:b 31a :2 31b :2 31a :a 31b
:8 31c :3 319 :b 31e :11 31f
:20 320 :3 31f :11 322 :b 323
:3 322 :9 325 326 31e
:9 326 :11 327 :2e 328 :3 327
:11 32a :b 32b :3 32a :c 32d
326 :3 31e 315 :3 330
32f :3 315 :16 332 :16 333
:3 334 335 333 :9 335
:3 336 337 335 333
:16 337 :3 338 339 337
333 :9 339 :3 33a 339
:3 333 :3 332 313 :3 308
:b 344 :3 345 :3 347 :3 348
:8 34b :3 34c :9 353 :5 354
:3 355 353 :2 359 35a
:2 35b 359 :5 35c 358
35e :5 35f :3 35e :2 360
:5 362 :11 363 :24 364 :3 363
:2 361 :6 360 35d :3 356
:a 369 :9 36a 36d 36f
370 :2 371 :2 372 373
:3 374 36d :5 375 36b
377 :5 378 :3 377 :2 379
:5 37b :11 37c :24 37d :3 37c
:2 37a :6 379 376 :3 36a
:a 382 :3 383 :3 382 :3 36a
369 :3 387 386 :3 369
356 :3 353 :5 38b :3 38e
390 391 :2 392 :2 393
38e :5 394 38c 396
:5 397 :3 396 :2 398 :5 39a
:11 39b :24 39c :3 39b :2 399
:6 398 395 :3 38b :d 3a3
:6 3a4 3a5 :4 3a6 :3 3a7
:4 3a8 3a3 :5 3a9 3a1
3ab :5 3ac :3 3ab :2 3ad
:5 3af :11 3b0 :24 3b1 :3 3b0
:2 3ae :6 3ad 3aa :3 38b
:a 3b6 :11 3b7 :30 3b8 :3 3b7
:11 3ba :b 3bb :3 3ba :a 3bd
:3 3b6 :9 3c0 :3 3c1 3c0
:3 3c3 3c2 :3 3c0 3c7
3c9 3ca 3cb 3cc
3cd 3ce 3cf 3d0
3d1 3d5 3d6 3d7
3d8 3d9 3da 3db
3dc 3dd 3c7 :6 3df
:5 3e0 3df :5 3e2 3e1
:3 3df 3c6 3e5 :5 3e6
:3 3e5 :2 3e7 :5 3e9 :11 3ea
:24 3eb :3 3ea :2 3e8 :6 3e7
3e4 :3 38b 3ef 38b
:a 3ef :2 3f3 :2 3f4 :2 3f5
:2 3f6 :2 3f7 3f3 :6 3f8
:5 3f9 3f8 :5 3fb 3fa
:3 3f8 3f2 3fe :5 3ff
:3 3fe :2 400 :5 402 :11 403
:24 404 :3 403 :2 401 :6 400
3fd :4 3ef :3 38b 40d
40e 40f 410 :2 411
412 :2 413 40d :6 414
:5 415 414 :5 417 416
:3 414 40c 41a :5 41b
:3 41a :2 41c :5 41e :11 41f
:24 420 :3 41f :2 41d :6 41c
419 :3 344 426 427
428 429 :2 42a 42b
:2 42c 426 :6 42d :5 42e
42d :5 430 42f :3 42d
425 433 :5 434 :3 433
:2 435 :5 437 :11 438 :24 439
:3 438 :2 436 :6 435 432
:3 344 43f :2 440 441
:2 442 43f :6 443 :5 444
443 :5 446 445 :3 443
43e 449 :5 44a :3 449
:2 44b :5 44d :11 44e :24 44f
:3 44e :2 44c :6 44b 448
:6 344 :5 471 :17 473 :2 474
:2 473 :6 474 :2 473 :5 475
:3 476 :3 475 474 :2 473
:9 47a :5 47b :3 47a :13 47e
:6 47f :2 47e :3 480 47f
:2 47e :3 484 :3 485 :9 486
:b 48d :a 48e :2 48f :2 48e
:2 490 :2 48e :2 491 :2 48e
:2 491 :2 48e :2 491 :3 48e
492 48d :9 492 :2 493
:2 494 :2 493 :2 495 :2 493
:2 495 :2 493 :2 495 :2 493
:2 496 :2 493 :2 496 :2 493
:2 496 :2 493 :2 497 :3 493
492 48d :2 499 :2 49a
:2 499 :2 49b :2 499 :2 49c
:2 499 :2 49c :2 499 :2 49c
:3 499 498 :3 48d 486
:c 4a6 :2 4a7 :2 4a6 :6 4a7
:2 4a6 4a7 :2 4a8 :2 4a7
:6 4a8 :2 4a6 :5 4ac :5 4ad
:5 4af :5 4b2 :5 4b3 4b6
:2 4b7 4b8 4b9 :4 4bc
:2 4bd :3 4be :4 4bf :2 4c0
:3 4c1 4b6 :6 4c2 :5 4c3
4c2 :5 4c5 4c4 :3 4c2
4b5 4c8 :5 4c9 :3 4c8
:2 4ca :5 4cc :11 4cd :24 4ce
:3 4cd :2 4cb :6 4ca 4c7
:3 4af 4d6 :2 4d7 :2 4d8
:2 4da :2 4db :4 4dc :8 4dd
4d6 :6 4de :5 4df 4de
:5 4e1 4e0 :3 4de 4d5
4e4 :5 4e5 :3 4e4 :2 4e6
:5 4e8 :11 4e9 :24 4ea :3 4e9
:2 4e7 :6 4e6 4e3 :4 4af
:3 4f0 4ee :3 4af 4a8
:2 4a6 :5 4f4 4f6 :1c 4f7
:2 4f6 :c 4f7 :3 4f6 :3 4f8
:8 4fb :15 4fc :15 4fd :2 4fc
:2 4fe 4fd :2 4fc :7 500
4fb 501 4fb :5 503
:11 504 :20 505 :3 504 :11 507
:b 508 :3 507 :9 50a :3 503
:2 50e 50f 510 50e
:5 511 50d 513 :5 514
:3 513 :2 515 :5 517 :11 518
:24 519 :3 518 :2 516 :6 515
512 :6 4f4 :b 51f 521
522 521 :3 523 :7 524
:7 525 526 51f :b 526
528 529 528 :7 52a
:7 52b :3 52c 526 51f
:b 52e :3 531 :3 532 :5 534
:7 535 :7 536 :7 537 :3 534
:7 53a :5 53c :7 53d :7 53e
:7 53f :3 53c 52e :5 542
:7 543 :3 544 542 :6 547
:2 548 :3 547 :3 54a :5 54c
:7 54d :7 54e :7 54f :3 54c
:6 552 :2 553 :2 552 :2 554
:2 552 :2 555 :2 552 :2 556
:2 552 :2 557 :3 552 545
:3 542 :5 55a :f 55b :3 55a
541 :3 52e 52d :3 51f
:b 560 :5 561 :2 563 564
:2 565 563 :5 566 562
568 :5 569 :3 568 :2 56a
:5 56c :11 56d :24 56e :3 56d
:2 56b :6 56a 567 :3 561
:a 573 :a 574 :7 577 :6 578
:2 579 :2 578 :2 579 :2 578
:2 579 :3 578 573 :11 57b
:30 57c :3 57b :11 57e :b 57f
:3 57e :a 581 57a :3 573
:3 561 :5 585 :2 587 588
:2 589 587 :5 58a 586
58c :5 58d :3 58c :2 58e
:5 590 :11 591 :24 592 :3 591
:2 58f :6 58e 58b :3 585
:a 597 :7 598 :6 599 :2 59a
:2 599 :2 59a :2 599 :2 59a
:3 599 597 :11 59c :30 59d
:3 59c :11 59f :b 5a0 :3 59f
:a 5a2 59b :3 597 :3 585
:5 5a6 :2 5a8 5a9 :2 5aa
5a8 :5 5ab 5a7 5ad
:5 5ae :3 5ad :2 5af :5 5b1
:11 5b2 :24 5b3 :3 5b2 :2 5b0
:6 5af 5ac :3 5a6 :a 5b8
:7 5b9 :6 5ba :2 5bb :2 5ba
:2 5bb :2 5ba :2 5bb :3 5ba
5b8 :11 5bd :30 5be :3 5bd
:11 5c0 :b 5c1 :3 5c0 :a 5c3
5bc :3 5b8 :3 5a6 :14 5c7
:16 5c8 :3 5c7 :3 560 :5 5cd
:a 5ce :3 5d0 :8 5d1 :f 5d3
:2 5d6 5d7 :2 5d8 5d6
:5 5d9 5d5 5db :5 5dc
:3 5db :2 5dd :5 5df :11 5e0
:24 5e1 :3 5e0 :2 5de :6 5dd
5da :3 5d1 :a 5e6 :11 5e7
:30 5e8 :3 5e7 :11 5ea :b 5eb
:3 5ea :a 5ed :3 5e6 :6 5f0
:2 5f1 :2 5f0 :2 5f2 :2 5f0
:2 5f3 :2 5f0 :2 5f3 :2 5f0
:2 5f3 :3 5f0 :b 5f5 :7 5f6
5f5 :7 5f8 5f7 :3 5f5
:b 5fc :2 5fd :3 5fc :7 5ff
5d1 600 5d1 :3 5cd
:b 604 :a 605 :3 607 :8 608
:f 60a :2 60d 60e :2 60f
60d :5 610 60c 612
:5 613 :3 612 :2 614 :5 616
:11 617 :24 618 :3 617 :2 615
:6 614 611 :3 608 :a 61d
:11 61e :30 61f :3 61e :11 621
:b 622 :3 621 :a 624 :3 61d
:6 627 :2 628 :2 627 :2 629
:2 627 :2 62a :2 627 :2 62a
:2 627 :2 62a :2 627 :2 62b
:3 627 :7 62c 608 62d
608 :3 604 :7 630 49e
:3 486 :7 633 :5 637 63a
:4 63b :3 63c :5 63d 63c
:5 63f 63e :3 63c :a 643
:2 644 :3 643 :17 649 :2 64a
:2 649 :6 64a :2 649 64a
:2 64b :2 64a :b 64b :3 64a
:2 649 :5 64e :4 650 :3 651
:5 652 651 :5 654 653
:3 651 :a 657 :4 659 :5 65a
658 65c :5 65d :3 65c
:2 65e :5 660 :11 661 :24 662
:3 661 :2 65f :6 65e 65b
:3 657 :2 66a 657 :4 66d
:5 66e 66c 670 :5 671
:3 670 :2 672 :5 674 :11 675
:24 676 :3 675 :2 673 :6 672
66f :4 66b :3 657 64b
:2 649 :3 67d :3 67e :9 67f
:a 680 :5 682 :3 683 680
:c 685 :3 688 689 :2 68a
:2 68b 68c 688 :5 68d
687 68f :5 690 :3 68f
:2 691 :5 693 :11 694 :24 695
:3 694 :2 692 :6 691 68e
:3 685 :9 69a :4 69c :5 69d
69b 69f :5 6a0 :3 69f
:2 6a1 :5 6a3 :11 6a4 :24 6a5
:3 6a4 :2 6a2 :6 6a1 69e
:3 69a :11 6aa :30 6ab :3 6aa
:11 6ad :b 6ae :3 6ad :7 6b0
6b1 :2 6b0 :3 69a :3 685
684 :3 680 :8 6b6 :5 6b7
:7 6b8 :6 6b9 :a 6ba :2 6bb
:3 6b6 :5 6bd :4 6bf :5 6c0
6be 6c2 :5 6c3 :3 6c2
:2 6c4 :5 6c6 :11 6c7 :24 6c8
:3 6c7 :2 6c5 :6 6c4 6c1
:3 6bd :7 6cd :5 6ce :3 6cd
:11 6d1 :2e 6d2 :3 6d1 :11 6d4
:b 6d5 :3 6d4 :8 6d7 6d8
6bd :18 6d8 :6 6da :6 6db
:5 6dc 6db :5 6de 6dd
:3 6db 6d9 6e1 :5 6e2
:3 6e1 :2 6e3 :5 6e5 :11 6e6
:24 6e7 :3 6e6 :2 6e4 :6 6e3
6e0 :4 6d8 :3 6bd :2 6ed
67f :b 6ef :3 6f0 :3 6f1
:3 6ef :8 6f4 :5 6f5 :7 6f6
:6 6f7 :8 6f8 :3 6f4 :5 6fa
:7 6fb :3 6fa :9 6fe :2 6ff
:3 6fe 6ee :3 67f 702
703 :3 63a 704 a4
:4 708 :5 709 707 70b
:5 70c :3 70b :2 70d :5 70f
:11 710 :24 711 :3 710 :2 70e
:6 70d 70a :3 a4 :f 717
:2 718 :2 717 :4 718 :6 717
:11 719 :2e 71a :3 719 :11 71c
:b 71d :3 71c :8 71f 718
:2 717 :5 722 :3 723 :3 724
:3 725 :5 727 :3 729 72b
72c 72d 72e 72f
730 731 732 733
734 :3 72c :3 735 :5 736
735 :5 738 737 :3 735
:a 73b :2 73c :3 73b :12 741
:c 742 :2 741 :3 744 742
:2 741 :b 748 :5 74a 74d
:2 74e :2 74f :2 750 :2 751
:2 752 :2 753 :2 754 74d
:6 755 :5 756 755 :5 758
757 :3 755 74c 75b
:5 75c :3 75b :2 75d :5 75f
:11 760 :24 761 :3 760 :2 75e
:6 75d 75a :6 74a :3 767
:3 768 :3 769 :3 76a :3 76b
:3 76d :3 771 :3 772 :3 748
:c 775 778 779 77a
77b 77c 77d 77e
:2 77f 778 :6 780 :5 781
780 :5 783 782 :3 780
777 786 :5 787 :3 786
:2 788 :5 78a :11 78b :24 78c
:3 78b :2 789 :6 788 785
:6 775 :5 792 :5 793 :c 794
:3 793 :c 797 :3 792 :5 79a
:c 79b :3 79a :5 79e :5 79f
:c 7a0 :3 79f :c 7a3 :3 79e
:5 7a6 :c 7a7 :3 7a6 :5 7aa
:5 7ab :c 7ac :3 7ab :c 7af
:3 7aa :5 7b2 :5 7b3 :c 7b4
:3 7b3 :c 7b7 :3 7b2 :7 7bb
:3 7bd 72b 7be 722
:4 7c0 :5 7c1 7bf 7c3
:5 7c4 :3 7c3 :2 7c5 :5 7c7
:11 7c8 :24 7c9 :3 7c8 :2 7c6
:6 7c5 7c2 :3 722 :5 7ce
7d1 :2 7d2 :2 7d3 :2 7d4
:2 7d5 :2 7d6 :2 7d7 :2 7d8
7d1 :6 7d9 :5 7da 7d9
:5 7dc 7db :3 7d9 7d0
7df :5 7e0 :3 7df :2 7e1
:5 7e3 :11 7e4 :24 7e5 :3 7e4
:2 7e2 :6 7e1 7de :6 7ce
:3 722 :5 7ec :9 7ed :4 7ef
7f0 :2 7f1 :7 7f2 7ef
:5 7f3 7ee 7f5 :5 7f6
:3 7f5 :2 7f7 :5 7f9 :11 7fa
:24 7fb :3 7fa :2 7f8 :6 7f7
7f4 :4 7ed :3 800 801
7ff :3 7ed :3 7ec :7 806
:9 807 :3 808 806 :5 809
805 80b :5 80c :3 80b
:2 80d :5 80f :11 810 :24 811
:3 810 :2 80e :6 80d 80a
:3 a4 :b 819 :5 81c :5 81d
822 823 824 825
826 827 828 :2 829
:2 82a 822 :6 82b :5 82c
82b :5 82e 82d :3 82b
821 831 :5 832 :3 831
:2 833 :5 835 :11 836 :24 837
:3 836 :2 834 :6 833 830
:3 819 :a 83d 83f 840
:3 841 :3 842 :3 843 :3 844
845 :2 846 847 83f
:6 848 :5 849 848 :5 84b
84a :3 848 83e 84e
:5 84f :3 84e :2 850 :5 852
:11 853 :24 854 :3 853 :2 851
:6 850 84d :3 83d :a 859
85b 85c 85d :2 85e
85f 85b :6 860 :5 861
860 :5 863 862 :3 860
85a 866 :5 867 :3 866
:2 868 :5 86a :11 86b :24 86c
:3 86b :2 869 :6 868 865
:6 859 :3 83d :12 873 :6 874
:e 875 :e 876 :e 877 :e 878
:19 879 :12 87a :3 87b :9 87c
:c 87d :6 87e :7 87f :9 880
:9 881 :9 882 :9 883 :8 884
:3 873 :11 885 :b 886 :3 885
:3 888 819 :5 88a :13 88b
:6 88c :20 88d :20 88e :20 88f
:20 890 :19 891 :1c 892 :5 893
:9 894 :7 895 :9 896 :10 897
:c 898 :9 899 :9 89a :9 89b
:9 89c 89d :3 88b :11 89e
:b 89f :3 89e :3 8a1 88a
:5 8a4 :5 8a5 8a8 :5 8a9
:2 8aa 8a8 :6 8ab :5 8ac
8ab :5 8ae 8ad :3 8ab
8a7 8b1 :5 8b2 :3 8b1
:2 8b3 :5 8b5 :11 8b6 :24 8b7
:3 8b6 :2 8b4 :6 8b3 8b0
:3 8a2 8bd :4 8be :6 8bf
:3 8c0 8c1 :2 8c2 :2 8c3
8c4 8bd :6 8c5 :5 8c6
8c5 :5 8c8 8c7 :3 8c5
8bc 8cb :5 8cc :3 8cb
:2 8cd :5 8cf :11 8d0 :24 8d1
:3 8d0 :2 8ce :6 8cd 8ca
:3 8a2 :13 8d6 :6 8d7 :5 8d9
:17 8da :17 8db :3 8dc 8dd
:5 8df :17 8e0 :17 8e1 :3 8e2
8e3 :5 8e5 :17 8e6 :17 8e7
:3 8e8 8e9 :5 8eb :17 8ec
:17 8ed :3 8ee 8ef :19 8f1
:12 8f2 :a 8f3 :5 8f4 :2 8f5
:a 8f6 :a 8f7 :7 8f8 :7 8f9
:9 8fa :10 8fb :c 8fc :9 8fd
:12 8fe :12 8ff :9 900 :8 901
902 :3 8d6 :11 903 :b 904
:3 903 :3 906 8a2 :3 88a
889 :3 819 a4 :3 90c
:11 90d :b 90e :17 90f :3 90d
:2 911 :3 912 :3 913 914
912 911 912 :3 915
:3 90c :2 916 :5 918 :11 919
:b 91a :18 91b :3 919 :2 91d
91e :3 91f 920 91e
91d 91e :3 921 :2 917
:6 916 90b :7 1 
bed1
4
:3 0 1 :3 0 2
:a 0 3666 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 4
:3 0 5 :7 0 9
8 :3 0 4 :3 0
6 :7 0 d c
:3 0 d 7c 0
b 8 :3 0 7
:7 0 11 10 :3 0
8 :3 0 9 :7 0
15 14 :3 0 11
a2 0 f 8
:3 0 a :7 0 19
18 :3 0 8 :3 0
b :7 0 1d 1c
:3 0 15 c8 0
13 8 :3 0 c
:7 0 21 20 :3 0
8 :3 0 d :7 0
25 24 :3 0 19
ee 0 17 8
:3 0 e :7 0 29
28 :3 0 8 :3 0
f :7 0 2d 2c
:3 0 1d 114 0
1b 4 :3 0 10
:7 0 31 30 :3 0
4 :3 0 11 :7 0
35 34 :3 0 21
13a 0 1f 4
:3 0 12 :7 0 39
38 :3 0 4 :3 0
13 :7 0 3d 3c
:3 0 25 160 0
23 4 :3 0 14
:7 0 41 40 :3 0
8 :3 0 15 :7 0
45 44 :3 0 29
186 0 27 4
:3 0 16 :7 0 49
48 :3 0 4 :3 0
17 :7 0 4d 4c
:3 0 2d 1ac 0
2b 4 :3 0 18
:7 0 51 50 :3 0
4 :3 0 19 :7 0
55 54 :3 0 31
1d2 0 2f 8
:3 0 1a :7 0 59
58 :3 0 8 :3 0
1b :7 0 5d 5c
:3 0 35 1f8 0
33 8 :3 0 1c
:7 0 61 60 :3 0
8 :3 0 1d :7 0
65 64 :3 0 39
:2 0 37 4 :3 0
1e :7 0 69 68
:3 0 4 :3 0 1f
:7 0 6d 6c :3 0
20 :3 0 21 :3 0
22 :2 0 4 71
72 0 6f 73
0 3666 2 74
:2 0 24 :2 0 57
4 :3 0 24 :2 0
55 77 79 :6 0
7c 7a 0 3664
0 23 :6 0 24
:2 0 5b 4 :3 0
59 7e 80 :6 0
83 81 0 3664
0 25 :6 0 24
:2 0 5f 4 :3 0
5d 85 87 :6 0
8a 88 0 3664
0 26 :6 0 24
:2 0 63 4 :3 0
61 8c 8e :6 0
91 8f 0 3664
0 27 :6 0 24
:2 0 67 4 :3 0
65 93 95 :6 0
98 96 0 3664
0 28 :6 0 2b
:2 0 6b 4 :3 0
69 9a 9c :6 0
9f 9d 0 3664
0 29 :6 0 2d
:2 0 6f 4 :3 0
6d a1 a3 :6 0
a6 a4 0 3664
0 2a :6 0 2f
:2 0 73 4 :3 0
71 a8 aa :6 0
ad ab 0 3664
0 2c :6 0 2f
:2 0 77 4 :3 0
75 af b1 :6 0
b4 b2 0 3664
0 2e :6 0 2f
:2 0 7b 4 :3 0
79 b6 b8 :6 0
bb b9 0 3664
0 30 :6 0 2f
:2 0 7f 4 :3 0
7d bd bf :6 0
c2 c0 0 3664
0 31 :6 0 2f
:2 0 83 4 :3 0
81 c4 c6 :6 0
c9 c7 0 3664
0 32 :6 0 2f
:2 0 87 4 :3 0
85 cb cd :6 0
d0 ce 0 3664
0 33 :6 0 2f
:2 0 8b 4 :3 0
89 d2 d4 :6 0
d7 d5 0 3664
0 34 :6 0 2f
:2 0 8f 4 :3 0
8d d9 db :6 0
de dc 0 3664
0 35 :6 0 2f
:2 0 93 4 :3 0
91 e0 e2 :6 0
e5 e3 0 3664
0 36 :6 0 2f
:2 0 97 4 :3 0
95 e7 e9 :6 0
ec ea 0 3664
0 37 :6 0 24
:2 0 9b 4 :3 0
99 ee f0 :6 0
f3 f1 0 3664
0 38 :6 0 24
:2 0 9f 4 :3 0
9d f5 f7 :6 0
fa f8 0 3664
0 39 :6 0 3c
:2 0 a3 4 :3 0
a1 fc fe :6 0
101 ff 0 3664
0 3a :6 0 a9
499 0 a7 4
:3 0 a5 103 105
:6 0 108 106 0
3664 0 3b :6 0
40 :2 0 ad 3e
:3 0 10a :7 0 10d
10b 0 3664 0
3d :6 0 4 :3 0
40 :2 0 ab 10f
111 :6 0 114 112
0 3664 0 3f
:6 0 40 :2 0 b1
4 :3 0 af 116
118 :6 0 11b 119
0 3664 0 41
:6 0 40 :2 0 b5
4 :3 0 b3 11d
11f :6 0 122 120
0 3664 0 42
:6 0 40 :2 0 b9
4 :3 0 b7 124
126 :6 0 129 127
0 3664 0 43
:6 0 46 :2 0 bd
4 :3 0 bb 12b
12d :6 0 130 12e
0 3664 0 44
:6 0 24 :2 0 c1
4 :3 0 bf 132
134 :6 0 137 135
0 3664 0 45
:6 0 24 :2 0 c5
4 :3 0 c3 139
13b :6 0 13e 13c
0 3664 0 47
:6 0 24 :2 0 c9
4 :3 0 c7 140
142 :6 0 145 143
0 3664 0 48
:6 0 2b :2 0 cd
4 :3 0 cb 147
149 :6 0 14c 14a
0 3664 0 49
:6 0 46 :2 0 d1
4 :3 0 cf 14e
150 :6 0 153 151
0 3664 0 4a
:6 0 4d :2 0 d5
4 :3 0 d3 155
157 :6 0 15a 158
0 3664 0 4b
:6 0 4d :2 0 d7
8 :3 0 15c :7 0
160 15d 15e 3664
0 4c :6 0 4d
:2 0 d9 8 :3 0
162 :7 0 166 163
164 3664 0 4e
:6 0 4d :2 0 db
8 :3 0 168 :7 0
16c 169 16a 3664
0 4f :6 0 4d
:2 0 dd 8 :3 0
16e :7 0 172 16f
170 3664 0 50
:6 0 4d :2 0 df
8 :3 0 174 :7 0
178 175 176 3664
0 51 :6 0 4d
:2 0 e1 8 :3 0
17a :7 0 17e 17b
17c 3664 0 52
:6 0 4d :2 0 e3
8 :3 0 180 :7 0
184 181 182 3664
0 53 :6 0 4d
:2 0 e5 8 :3 0
186 :7 0 18a 187
188 3664 0 54
:6 0 4d :2 0 e7
8 :3 0 18c :7 0
190 18d 18e 3664
0 55 :6 0 4d
:2 0 e9 8 :3 0
192 :7 0 196 193
194 3664 0 56
:6 0 4d :2 0 eb
8 :3 0 198 :7 0
19c 199 19a 3664
0 57 :6 0 4d
:2 0 ed 8 :3 0
19e :7 0 1a2 19f
1a0 3664 0 58
:6 0 4d :2 0 ef
8 :3 0 1a4 :7 0
1a8 1a5 1a6 3664
0 59 :6 0 4d
:2 0 f1 8 :3 0
1aa :7 0 1ae 1ab
1ac 3664 0 5a
:6 0 4d :2 0 f3
8 :3 0 1b0 :7 0
1b4 1b1 1b2 3664
0 5b :6 0 4d
:2 0 f5 8 :3 0
1b6 :7 0 1ba 1b7
1b8 3664 0 5c
:6 0 4d :2 0 f7
8 :3 0 1bc :7 0
1c0 1bd 1be 3664
0 5d :6 0 4d
:2 0 f9 8 :3 0
1c2 :7 0 1c6 1c3
1c4 3664 0 5e
:6 0 4d :2 0 fb
8 :3 0 1c8 :7 0
1cc 1c9 1ca 3664
0 5f :6 0 4d
:2 0 fd 8 :3 0
1ce :7 0 1d2 1cf
1d0 3664 0 60
:6 0 4d :2 0 ff
8 :3 0 1d4 :7 0
1d8 1d5 1d6 3664
0 61 :6 0 4d
:2 0 101 8 :3 0
1da :7 0 1de 1db
1dc 3664 0 62
:6 0 4d :2 0 103
8 :3 0 1e0 :7 0
1e4 1e1 1e2 3664
0 63 :6 0 67
:2 0 105 8 :3 0
1e6 :7 0 1ea 1e7
1e8 3664 0 64
:6 0 67 :2 0 107
66 :3 0 1ec :7 0
1f0 1ed 1ee 3664
0 65 :6 0 4d
:2 0 109 66 :3 0
1f2 :7 0 1f6 1f3
1f4 3664 0 68
:6 0 4d :2 0 10b
8 :3 0 1f8 :7 0
1fc 1f9 1fa 3664
0 69 :6 0 4d
:2 0 10d 8 :3 0
1fe :7 0 202 1ff
200 3664 0 6a
:6 0 4d :2 0 10f
8 :3 0 204 :7 0
208 205 206 3664
0 6b :6 0 4d
:2 0 111 8 :3 0
20a :7 0 20e 20b
20c 3664 0 6c
:6 0 4d :2 0 113
8 :3 0 210 :7 0
214 211 212 3664
0 6d :6 0 4d
:2 0 115 8 :3 0
216 :7 0 21a 217
218 3664 0 6e
:6 0 46 :2 0 117
8 :3 0 21c :7 0
220 21d 21e 3664
0 6f :6 0 4d
:2 0 11b 4 :3 0
119 222 224 :6 0
227 225 0 3664
0 70 :6 0 4d
:2 0 11d 8 :3 0
229 :7 0 22d 22a
22b 3664 0 71
:6 0 4d :2 0 11f
8 :3 0 22f :7 0
233 230 231 3664
0 72 :6 0 75
:2 0 121 8 :3 0
235 :7 0 239 236
237 3664 0 73
:6 0 77 :2 0 125
4 :3 0 123 23b
23d :6 0 240 23e
0 3664 0 74
:6 0 75 :2 0 129
4 :3 0 127 242
244 :6 0 247 245
0 3664 0 76
:6 0 4d :2 0 12d
4 :3 0 12b 249
24b :6 0 24e 24c
0 3664 0 78
:6 0 4d :2 0 12f
8 :3 0 250 :7 0
254 251 252 3664
0 79 :6 0 24
:2 0 131 8 :3 0
256 :7 0 25a 257
258 3664 0 7a
:6 0 24 :2 0 135
7c :3 0 133 25c
25e :6 0 261 25f
0 3664 0 7b
:6 0 24 :2 0 139
7c :3 0 137 263
265 :6 0 268 266
0 3664 0 7d
:6 0 24 :2 0 13d
4 :3 0 13b 26a
26c :6 0 26f 26d
0 3664 0 7e
:6 0 24 :2 0 141
4 :3 0 13f 271
273 :6 0 276 274
0 3664 0 7f
:6 0 46 :2 0 145
4 :3 0 143 278
27a :6 0 27d 27b
0 3664 0 80
:6 0 4d :2 0 149
4 :3 0 147 27f
281 :6 0 284 282
0 3664 0 81
:6 0 4d :2 0 14b
8 :3 0 286 :7 0
28a 287 288 3664
0 82 :6 0 4d
:2 0 14d 8 :3 0
28c :7 0 290 28d
28e 3664 0 83
:6 0 46 :2 0 14f
8 :3 0 292 :7 0
296 293 294 3664
0 84 :6 0 46
:2 0 151 8 :3 0
298 0 2 :5 0
29c 299 29a 3664
0 85 :6 0 4d
:2 0 153 8 :3 0
29e 0 2 :5 0
2a2 29f 2a0 3664
0 86 :6 0 4d
:2 0 155 8 :3 0
2a4 0 2 :5 0
2a8 2a5 2a6 3664
0 87 :6 0 4d
:2 0 157 8 :3 0
2aa 0 2 :5 0
2ae 2ab 2ac 3664
0 88 :6 0 24
:2 0 159 8 :3 0
2b0 0 2 :5 0
2b4 2b1 2b2 3664
0 89 :6 0 4d
:2 0 15d 4 :3 0
15b 2b6 2b8 :6 0
2bb 2b9 0 3664
0 8a :6 0 161
c56 0 15f 8
:3 0 2bd 0 2
:5 0 2c1 2be 2bf
3664 0 8b :6 0
165 c8a 0 163
8 :3 0 2c3 :7 0
2c6 2c4 0 3664
0 8c :6 0 8
:3 0 2c8 :7 0 2cb
2c9 0 3664 0
8d :6 0 169 cbe
0 167 8 :3 0
2cd :7 0 2d0 2ce
0 3664 0 8e
:6 0 8 :3 0 2d2
:7 0 2d5 2d3 0
3664 0 8f :6 0
16d cf2 0 16b
8 :3 0 2d7 :7 0
2da 2d8 0 3664
0 90 :6 0 8
:3 0 2dc :7 0 2df
2dd 0 3664 0
91 :9 0 16f 8
:3 0 2e1 :7 0 2e4
2e2 0 3664 0
92 :6 0 21 :3 0
22 :2 0 4 2e6
2e7 0 2e8 :7 0
2eb 2e9 0 3664
0 93 :6 0 94
:3 0 95 :a 0 2
2fa :4 0 2ed 2f0
0 2ee :3 0 96
:3 0 97 :3 0 98
:3 0 25 :3 0 99
:3 0 25 :3 0 9a
:3 0 39 :4 0 9b
1 :8 0 2fb 2ed
2f0 2fc 0 3664
171 2fc 2fe 2fb
2fd :6 0 2fa :7 0
2fc 94 :3 0 9c
:a 0 3 315 :5 0
300 303 0 301
:3 0 9d :3 0 9e
:3 0 9f :3 0 a0
:3 0 a1 :3 0 a2
:3 0 9f :3 0 a3
:3 0 a1 :3 0 a4
:3 0 a1 :3 0 a5
:3 0 a1 :3 0 a6
:3 0 a7 :3 0 9e
:4 0 a8 1 :8 0
316 300 303 317
0 3664 173 317
319 316 318 :6 0
315 :7 0 317 322
323 0 175 21
:3 0 22 :2 0 4
31b 31c 0 31d
:7 0 320 31e 0
3664 0 a9 :6 0
329 32a 0 177
21 :3 0 22 :2 0
4 324 :7 0 327
325 0 3664 0
aa :6 0 330 331
0 179 21 :3 0
22 :2 0 4 32b
:7 0 32e 32c 0
3664 0 ab :6 0
336 337 0 17b
21 :3 0 22 :2 0
4 332 :7 0 335
333 0 3664 0
ac :6 0 ad :3 0
ae :3 0 ad :3 0
af :3 0 339 33a
0 2 :4 0 17d
33b 33d 338 33f
33e :2 0 b0 :3 0
b1 :3 0 341 342
0 b2 :4 0 ad
:3 0 b3 :3 0 345
346 0 2 :4 0
17f 347 349 b4
:4 0 b5 :2 0 b6
:3 0 181 34c 34e
:3 0 b7 :4 0 b5
:2 0 b8 :4 0 184
351 353 :3 0 b5
:2 0 3 :3 0 187
355 357 :3 0 b5
:2 0 b8 :4 0 18a
359 35b :3 0 b5
:2 0 b9 :4 0 18d
35d 35f :3 0 b5
:2 0 b8 :4 0 190
361 363 :3 0 b5
:2 0 5 :3 0 193
365 367 :3 0 b5
:2 0 b8 :4 0 196
369 36b :3 0 b5
:2 0 b9 :4 0 199
36d 36f :3 0 b5
:2 0 b8 :4 0 19c
371 373 :3 0 b5
:2 0 6 :3 0 19f
375 377 :3 0 b5
:2 0 b8 :4 0 1a2
379 37b :3 0 b5
:2 0 b9 :4 0 1a5
37d 37f :3 0 b5
:2 0 a1 :3 0 ba
:3 0 7 :3 0 1a8
383 385 bb :4 0
1aa 382 388 1ad
381 38a :3 0 b5
:2 0 b9 :4 0 1b0
38c 38e :3 0 b5
:2 0 a1 :3 0 ba
:3 0 9 :3 0 1b3
392 394 bb :4 0
1b5 391 397 1b8
390 399 :3 0 b5
:2 0 b9 :4 0 1bb
39b 39d :3 0 b5
:2 0 a1 :3 0 ba
:3 0 a :3 0 1be
3a1 3a3 bb :4 0
1c0 3a0 3a6 1c3
39f 3a8 :3 0 b5
:2 0 b9 :4 0 1c6
3aa 3ac :3 0 b5
:2 0 a1 :3 0 ba
:3 0 b :3 0 1c9
3b0 3b2 bb :4 0
1cb 3af 3b5 1ce
3ae 3b7 :3 0 b5
:2 0 b9 :4 0 1d1
3b9 3bb :3 0 b5
:2 0 a1 :3 0 ba
:3 0 c :3 0 1d4
3bf 3c1 bb :4 0
1d6 3be 3c4 1d9
3bd 3c6 :3 0 b5
:2 0 b9 :4 0 1dc
3c8 3ca :3 0 b5
:2 0 a1 :3 0 ba
:3 0 d :3 0 1df
3ce 3d0 bb :4 0
1e1 3cd 3d3 1e4
3cc 3d5 :3 0 b5
:2 0 b9 :4 0 1e7
3d7 3d9 :3 0 b5
:2 0 a1 :3 0 ba
:3 0 e :3 0 1ea
3dd 3df bb :4 0
1ec 3dc 3e2 1ef
3db 3e4 :3 0 b5
:2 0 b9 :4 0 1f2
3e6 3e8 :3 0 b5
:2 0 a1 :3 0 ba
:3 0 f :3 0 1f5
3ec 3ee bb :4 0
1f7 3eb 3f1 1fa
3ea 3f3 :3 0 b5
:2 0 b9 :4 0 1fd
3f5 3f7 :3 0 b5
:2 0 b8 :4 0 200
3f9 3fb :3 0 b5
:2 0 10 :3 0 203
3fd 3ff :3 0 b5
:2 0 b8 :4 0 206
401 403 :3 0 b5
:2 0 b9 :4 0 209
405 407 :3 0 b5
:2 0 b8 :4 0 20c
409 40b :3 0 b5
:2 0 11 :3 0 20f
40d 40f :3 0 b5
:2 0 b8 :4 0 212
411 413 :3 0 b5
:2 0 b9 :4 0 215
415 417 :3 0 b5
:2 0 b8 :4 0 218
419 41b :3 0 b5
:2 0 12 :3 0 21b
41d 41f :3 0 b5
:2 0 b8 :4 0 21e
421 423 :3 0 b5
:2 0 b9 :4 0 221
425 427 :3 0 b5
:2 0 b8 :4 0 224
429 42b :3 0 b5
:2 0 13 :3 0 227
42d 42f :3 0 b5
:2 0 b8 :4 0 22a
431 433 :3 0 b5
:2 0 b9 :4 0 22d
435 437 :3 0 b5
:2 0 b8 :4 0 230
439 43b :3 0 b5
:2 0 14 :3 0 233
43d 43f :3 0 b5
:2 0 b8 :4 0 236
441 443 :3 0 b5
:2 0 b9 :4 0 239
445 447 :3 0 b5
:2 0 a1 :3 0 ba
:3 0 15 :3 0 23c
44b 44d bb :4 0
23e 44a 450 241
449 452 :3 0 b5
:2 0 b9 :4 0 244
454 456 :3 0 b5
:2 0 b8 :4 0 247
458 45a :3 0 b5
:2 0 16 :3 0 24a
45c 45e :3 0 b5
:2 0 b8 :4 0 24d
460 462 :3 0 b5
:2 0 b9 :4 0 250
464 466 :3 0 b5
:2 0 b8 :4 0 253
468 46a :3 0 b5
:2 0 17 :3 0 256
46c 46e :3 0 b5
:2 0 b8 :4 0 259
470 472 :3 0 b5
:2 0 b9 :4 0 25c
474 476 :3 0 b5
:2 0 b8 :4 0 25f
478 47a :3 0 b5
:2 0 18 :3 0 262
47c 47e :3 0 b5
:2 0 b8 :4 0 265
480 482 :3 0 b5
:2 0 b9 :4 0 268
484 486 :3 0 b5
:2 0 b8 :4 0 26b
488 48a :3 0 b5
:2 0 19 :3 0 26e
48c 48e :3 0 b5
:2 0 b8 :4 0 271
490 492 :3 0 b5
:2 0 b9 :4 0 274
494 496 :3 0 b5
:2 0 a1 :3 0 ba
:3 0 1a :3 0 277
49a 49c bb :4 0
279 499 49f 27c
498 4a1 :3 0 b5
:2 0 b9 :4 0 27f
4a3 4a5 :3 0 b5
:2 0 a1 :3 0 ba
:3 0 1b :3 0 282
4a9 4ab bb :4 0
284 4a8 4ae 287
4a7 4b0 :3 0 b5
:2 0 b9 :4 0 28a
4b2 4b4 :3 0 b5
:2 0 a1 :3 0 ba
:3 0 1c :3 0 28d
4b8 4ba bb :4 0
28f 4b7 4bd 292
4b6 4bf :3 0 b5
:2 0 b9 :4 0 295
4c1 4c3 :3 0 b5
:2 0 a1 :3 0 ba
:3 0 1d :3 0 298
4c7 4c9 bb :4 0
29a 4c6 4cc 29d
4c5 4ce :3 0 b5
:2 0 b9 :4 0 2a0
4d0 4d2 :3 0 b5
:2 0 b8 :4 0 2a3
4d4 4d6 :3 0 b5
:2 0 1e :3 0 2a6
4d8 4da :3 0 b5
:2 0 b8 :4 0 2a9
4dc 4de :3 0 b5
:2 0 b9 :4 0 2ac
4e0 4e2 :3 0 b5
:2 0 b8 :4 0 2af
4e4 4e6 :3 0 b5
:2 0 1f :3 0 2b2
4e8 4ea :3 0 b5
:2 0 b8 :4 0 2b5
4ec 4ee :3 0 b5
:2 0 bc :4 0 2b8
4f0 4f2 :3 0 2bb
343 4f4 :2 0 4f6
2c0 4f7 340 4f6
0 4f8 2c2 0
35c4 ad :3 0 ae
:3 0 4f9 4fa 0
ad :3 0 bd :3 0
4fc 4fd 0 46
:2 0 2 :4 0 be
:2 0 bf :2 0 46
:2 0 2c4 502 504
:3 0 2c7 4fe 506
4fb 508 507 :2 0
b0 :3 0 c0 :3 0
50a 50b 0 b2
:4 0 c1 :4 0 b5
:2 0 b6 :3 0 2cb
50f 511 :3 0 2ce
50c 513 :2 0 515
2d1 516 509 515
0 517 2d3 0
35c4 25 :3 0 3
:3 0 518 519 0
35c4 3f :3 0 5
:3 0 51b 51c 0
35c4 42 :3 0 6
:3 0 51e 51f 0
35c4 2a :3 0 10
:3 0 521 522 0
35c4 2c :3 0 11
:3 0 524 525 0
35c4 2e :3 0 16
:3 0 527 528 0
35c4 30 :3 0 17
:3 0 52a 52b 0
35c4 31 :3 0 18
:3 0 52d 52e 0
35c4 32 :3 0 19
:3 0 530 531 0
35c4 29 :3 0 1f
:3 0 533 534 0
35c4 58 :3 0 7
:3 0 536 537 0
35c4 56 :3 0 9
:3 0 539 53a 0
35c4 57 :3 0 e
:3 0 53c 53d 0
35c4 55 :3 0 1a
:3 0 53f 540 0
35c4 59 :3 0 b
:3 0 542 543 0
35c4 5b :3 0 f
:3 0 545 546 0
35c4 5a :3 0 c
:3 0 548 549 0
35c4 4f :3 0 1d
:3 0 54b 54c 0
35c4 4c :3 0 a
:3 0 54e 54f 0
35c4 5c :3 0 d
:3 0 551 552 0
35c4 51 :3 0 1b
:3 0 554 555 0
35c4 52 :3 0 1c
:3 0 557 558 0
35c4 71 :3 0 4d
:2 0 55a 55b 0
35c4 72 :3 0 4d
:2 0 55d 55e 0
35c4 a1 :3 0 4c
:3 0 4d :2 0 2d5
560 563 c2 :3 0
c3 :2 0 c4 :3 0
565 567 0 2da
566 569 :3 0 56a
:2 0 3 :3 0 c5
:2 0 2dd 56d 56e
:3 0 56f :2 0 570
:2 0 56b 572 571
:2 0 ad :3 0 ae
:3 0 574 575 0
ad :3 0 bd :3 0
577 578 0 c6
:2 0 2 :4 0 c7
:2 0 bf :2 0 46
:2 0 2df 57d 57f
:3 0 2e2 579 581
576 583 582 :2 0
b0 :3 0 c0 :3 0
585 586 0 b2
:4 0 c8 :4 0 b5
:2 0 b6 :3 0 2e6
58a 58c :3 0 b5
:2 0 c9 :4 0 2e9
58e 590 :3 0 b5
:2 0 ca :4 0 2ec
592 594 :3 0 b5
:2 0 a1 :3 0 ba
:3 0 c2 :3 0 cb
:3 0 599 59a 0
2ef 598 59c bb
:4 0 2f1 597 59f
2f4 596 5a1 :3 0
2f7 587 5a3 :2 0
5a5 2fa 5a6 584
5a5 0 5a7 2fc
0 5d0 ad :3 0
ae :3 0 5a8 5a9
0 ad :3 0 bd
:3 0 5ab 5ac 0
46 :2 0 2 :4 0
cc :2 0 bf :2 0
46 :2 0 2fe 5b1
5b3 :3 0 301 5ad
5b5 5aa 5b7 5b6
:2 0 b0 :3 0 c0
:3 0 5b9 5ba 0
b2 :4 0 cd :4 0
b5 :2 0 b6 :3 0
305 5be 5c0 :3 0
308 5bb 5c2 :2 0
5c4 30b 5c5 5b8
5c4 0 5c6 30d
0 5d0 ce :3 0
cf :3 0 5c7 5c8
0 c2 :3 0 cb
:3 0 5ca 5cb 0
d0 :3 0 30f 5c9
5ce :2 0 5d0 312
5d1 573 5d0 0
5d2 316 0 35c4
10 :3 0 c5 :2 0
318 5d4 5d5 :3 0
5d6 :2 0 5d7 :2 0
12 :3 0 c5 :2 0
31a 5da 5db :3 0
5dc :2 0 5dd :2 0
5d8 5df 5de :2 0
13 :3 0 c5 :2 0
31c 5e2 5e3 :3 0
5e4 :2 0 5e5 :2 0
5e0 5e7 5e6 :2 0
14 :3 0 c5 :2 0
31e 5ea 5eb :3 0
5ec :2 0 5ed :2 0
5e8 5ef 5ee :2 0
a1 :3 0 5b :3 0
4d :2 0 320 5f1
5f4 c2 :3 0 c3
:2 0 d1 :3 0 5f6
5f8 0 325 5f7
5fa :3 0 5fb :2 0
5f0 5fd 5fc :2 0
16 :3 0 c5 :2 0
328 600 601 :3 0
602 :2 0 603 :2 0
a1 :3 0 5b :3 0
4d :2 0 32a 605
608 c2 :3 0 d2
:2 0 d3 :3 0 60a
60c 0 32f 60b
60e :3 0 60f :2 0
604 611 610 :2 0
ad :3 0 ae :3 0
613 614 0 ad
:3 0 bd :3 0 616
617 0 c6 :2 0
2 :4 0 d4 :2 0
bf :2 0 46 :2 0
332 61c 61e :3 0
335 618 620 615
622 621 :2 0 b0
:3 0 c0 :3 0 624
625 0 b2 :4 0
d5 :4 0 b5 :2 0
b6 :3 0 339 629
62b :3 0 b5 :2 0
c9 :4 0 33c 62d
62f :3 0 b5 :2 0
ca :4 0 33f 631
633 :3 0 b5 :2 0
a1 :3 0 ba :3 0
c2 :3 0 d6 :3 0
638 639 0 342
637 63b bb :4 0
344 636 63e 347
635 640 :3 0 34a
626 642 :2 0 644
34d 645 623 644
0 646 34f 0
670 ad :3 0 ae
:3 0 647 648 0
ad :3 0 bd :3 0
64a 64b 0 46
:2 0 2 :4 0 d7
:2 0 bf :2 0 46
:2 0 351 650 652
:3 0 354 64c 654
649 656 655 :2 0
b0 :3 0 c0 :3 0
658 659 0 b2
:4 0 d8 :4 0 b5
:2 0 b6 :3 0 358
65d 65f :3 0 35b
65a 661 :2 0 663
35e 664 657 663
0 665 360 0
670 ce :3 0 cf
:3 0 666 667 0
c2 :3 0 d6 :3 0
669 66a 0 d0
:3 0 362 668 66d
:2 0 670 d9 :3 0
365 6dd 16 :3 0
c5 :2 0 369 672
673 :3 0 674 :2 0
675 :2 0 17 :3 0
c5 :2 0 36b 678
679 :3 0 67a :2 0
67b :2 0 676 67d
67c :2 0 ad :3 0
ae :3 0 67f 680
0 ad :3 0 bd
:3 0 682 683 0
c6 :2 0 2 :4 0
da :2 0 bf :2 0
46 :2 0 36d 688
68a :3 0 370 684
68c 681 68e 68d
:2 0 b0 :3 0 c0
:3 0 690 691 0
b2 :4 0 db :4 0
b5 :2 0 b6 :3 0
374 695 697 :3 0
b5 :2 0 c9 :4 0
377 699 69b :3 0
b5 :2 0 ca :4 0
37a 69d 69f :3 0
b5 :2 0 a1 :3 0
ba :3 0 c2 :3 0
d6 :3 0 6a4 6a5
0 37d 6a3 6a7
bb :4 0 37f 6a2
6aa 382 6a1 6ac
:3 0 385 692 6ae
:2 0 6b0 388 6b1
68f 6b0 0 6b2
38a 0 6db ad
:3 0 ae :3 0 6b3
6b4 0 ad :3 0
bd :3 0 6b6 6b7
0 46 :2 0 2
:4 0 dc :2 0 bf
:2 0 46 :2 0 38c
6bc 6be :3 0 38f
6b8 6c0 6b5 6c2
6c1 :2 0 b0 :3 0
c0 :3 0 6c4 6c5
0 b2 :4 0 dd
:4 0 b5 :2 0 b6
:3 0 393 6c9 6cb
:3 0 396 6c6 6cd
:2 0 6cf 399 6d0
6c3 6cf 0 6d1
39b 0 6db ce
:3 0 cf :3 0 6d2
6d3 0 c2 :3 0
d6 :3 0 6d5 6d6
0 d0 :3 0 39d
6d4 6d9 :2 0 6db
3a0 6dc 67e 6db
0 6de 612 670
0 6de 3a4 0
6df 3a7 6e0 5fe
6df 0 6e1 3a9
0 35c4 a1 :3 0
4c :3 0 4d :2 0
3ab 6e2 6e5 d2
:2 0 de :2 0 3b0
6e7 6e9 :3 0 a1
:3 0 5a :3 0 4d
:2 0 3b3 6eb 6ee
d2 :2 0 de :2 0
3b8 6f0 6f2 :3 0
6ea 6f4 6f3 :2 0
a1 :3 0 56 :3 0
4d :2 0 3bb 6f6
6f9 d2 :2 0 de
:2 0 3c0 6fb 6fd
:3 0 6f5 6ff 6fe
:2 0 ad :3 0 ae
:3 0 701 702 0
ad :3 0 bd :3 0
704 705 0 c6
:2 0 2 :4 0 df
:2 0 bf :2 0 46
:2 0 3c3 70a 70c
:3 0 3c6 706 70e
703 710 70f :2 0
b0 :3 0 c0 :3 0
712 713 0 b2
:4 0 e0 :4 0 b5
:2 0 b6 :3 0 3ca
717 719 :3 0 b5
:2 0 c9 :4 0 3cd
71b 71d :3 0 b5
:2 0 ca :4 0 3d0
71f 721 :3 0 b5
:2 0 a1 :3 0 ba
:3 0 c2 :3 0 d6
:3 0 726 727 0
3d3 725 729 bb
:4 0 3d5 724 72c
3d8 723 72e :3 0
3db 714 730 :2 0
732 3de 733 711
732 0 734 3e0
0 75d ad :3 0
ae :3 0 735 736
0 ad :3 0 bd
:3 0 738 739 0
46 :2 0 2 :4 0
e1 :2 0 bf :2 0
46 :2 0 3e2 73e
740 :3 0 3e5 73a
742 737 744 743
:2 0 b0 :3 0 c0
:3 0 746 747 0
b2 :4 0 e2 :4 0
b5 :2 0 b6 :3 0
3e9 74b 74d :3 0
3ec 748 74f :2 0
751 3ef 752 745
751 0 753 3f1
0 75d ce :3 0
cf :3 0 754 755
0 c2 :3 0 d6
:3 0 757 758 0
d0 :3 0 3f3 756
75b :2 0 75d 3f6
75e 700 75d 0
75f 3fa 0 35c4
15 :3 0 c5 :2 0
3fc 761 762 :3 0
763 :2 0 764 :2 0
53 :3 0 e3 :2 0
766 767 0 769
3fe 76e 53 :3 0
15 :3 0 76a 76b
0 76d 400 76f
765 769 0 770
0 76d 0 770
402 0 35c4 a1
:3 0 4c :3 0 4d
:2 0 405 771 774
c2 :3 0 c3 :2 0
c4 :3 0 776 778
0 40a 777 77a
:3 0 77b :2 0 5
:3 0 e4 :2 0 40d
77e 77f :3 0 780
:2 0 781 :2 0 77c
783 782 :2 0 6
:3 0 e4 :2 0 40f
786 787 :3 0 788
:2 0 789 :2 0 784
78b 78a :2 0 e5
:3 0 e6 :3 0 6
:3 0 e5 :3 0 e6
:3 0 5 :3 0 69
:3 0 e7 :4 0 e8
1 :8 0 79b 21
:3 0 e9 :3 0 796
797 0 ea :3 0
798 799 0 79b
411 7ee eb :3 0
21 :3 0 e9 :3 0
79d 79e 0 ea
:3 0 79f 7a0 0
7a2 414 7a4 416
7a3 7a2 :2 0 7ec
ec :3 0 21 :3 0
e9 :3 0 7a7 7a8
0 ea :3 0 7a9
7aa 0 7e4 ad
:3 0 ae :3 0 7ac
7ad 0 ad :3 0
bd :3 0 7af 7b0
0 ed :2 0 2
:4 0 ee :2 0 bf
:2 0 46 :2 0 418
7b5 7b7 :3 0 41b
7b1 7b9 7ae 7bb
7ba :2 0 b0 :3 0
c0 :3 0 7bd 7be
0 b2 :4 0 ef
:4 0 b5 :2 0 b6
:3 0 41f 7c2 7c4
:3 0 b5 :2 0 c9
:4 0 422 7c6 7c8
:3 0 b5 :2 0 f0
:4 0 425 7ca 7cc
:3 0 b5 :2 0 ea
:3 0 428 7ce 7d0
:3 0 b5 :2 0 f1
:4 0 42b 7d2 7d4
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 42e
7d7 7db 432 7d6
7dd :3 0 435 7bf
7df :2 0 7e1 438
7e2 7bc 7e1 0
7e3 43a 0 7e4
43c 7e7 :3 0 7e7
0 7e7 7e6 7e4
7e5 :6 0 7e9 4
:3 0 43f 7eb 441
7ea 7e9 :2 0 7ec
443 :2 0 7ee 0
7ee 7ed 79b 7ec
:6 0 868 1 :3 0
21 :3 0 e9 :3 0
7f0 7f1 0 21
:3 0 d2 :2 0 f5
:3 0 7f3 7f5 0
448 7f4 7f7 :3 0
7f8 :2 0 a1 :3 0
69 :3 0 4d :2 0
44b 7fa 7fd f6
:2 0 4d :2 0 450
7ff 801 :3 0 802
:2 0 7f9 804 803
:2 0 ad :3 0 ae
:3 0 806 807 0
ad :3 0 bd :3 0
809 80a 0 c6
:2 0 2 :4 0 f7
:2 0 bf :2 0 46
:2 0 453 80f 811
:3 0 456 80b 813
808 815 814 :2 0
b0 :3 0 c0 :3 0
817 818 0 b2
:4 0 f8 :4 0 b5
:2 0 b6 :3 0 45a
81c 81e :3 0 b5
:2 0 c9 :4 0 45d
820 822 :3 0 b5
:2 0 ca :4 0 460
824 826 :3 0 b5
:2 0 a1 :3 0 ba
:3 0 c2 :3 0 f9
:3 0 82b 82c 0
463 82a 82e bb
:4 0 465 829 831
468 828 833 :3 0
46b 819 835 :2 0
837 46e 838 816
837 0 839 470
0 862 ad :3 0
ae :3 0 83a 83b
0 ad :3 0 bd
:3 0 83d 83e 0
46 :2 0 2 :4 0
fa :2 0 bf :2 0
46 :2 0 472 843
845 :3 0 475 83f
847 83c 849 848
:2 0 b0 :3 0 c0
:3 0 84b 84c 0
b2 :4 0 fb :4 0
b5 :2 0 b6 :3 0
479 850 852 :3 0
47c 84d 854 :2 0
856 47f 857 84a
856 0 858 481
0 862 ce :3 0
cf :3 0 859 85a
0 c2 :3 0 f9
:3 0 85c 85d 0
d0 :3 0 483 85b
860 :2 0 862 486
863 805 862 0
864 48a 0 868
89 :3 0 46 :2 0
865 866 0 868
48c 869 78c 868
0 86a 490 0
35c4 5 :3 0 c5
:2 0 492 86c 86d
:3 0 86e :2 0 86f
:2 0 6 :3 0 c5
:2 0 494 872 873
:3 0 874 :2 0 875
:2 0 870 877 876
:2 0 b6 :3 0 3f
:3 0 e7 :4 0 fc
1 :8 0 882 21
:3 0 e9 :3 0 87d
87e 0 ea :3 0
87f 880 0 882
496 8d5 eb :3 0
21 :3 0 e9 :3 0
884 885 0 ea
:3 0 886 887 0
889 499 88b 49b
88a 889 :2 0 8d3
ec :3 0 21 :3 0
e9 :3 0 88e 88f
0 ea :3 0 890
891 0 8cb ad
:3 0 ae :3 0 893
894 0 ad :3 0
bd :3 0 896 897
0 ed :2 0 2
:4 0 fd :2 0 bf
:2 0 46 :2 0 49d
89c 89e :3 0 4a0
898 8a0 895 8a2
8a1 :2 0 b0 :3 0
c0 :3 0 8a4 8a5
0 b2 :4 0 fe
:4 0 b5 :2 0 b6
:3 0 4a4 8a9 8ab
:3 0 b5 :2 0 c9
:4 0 4a7 8ad 8af
:3 0 b5 :2 0 f0
:4 0 4aa 8b1 8b3
:3 0 b5 :2 0 ea
:3 0 4ad 8b5 8b7
:3 0 b5 :2 0 f1
:4 0 4b0 8b9 8bb
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 4b3
8be 8c2 4b7 8bd
8c4 :3 0 4ba 8a6
8c6 :2 0 8c8 4bd
8c9 8a3 8c8 0
8ca 4bf 0 8cb
4c1 8ce :3 0 8ce
0 8ce 8cd 8cb
8cc :6 0 8d0 6
:3 0 4c4 8d2 4c6
8d1 8d0 :2 0 8d3
4c8 :2 0 8d5 0
8d5 8d4 882 8d3
:6 0 8d7 1 :3 0
4cb 8d8 878 8d7
0 8d9 4cd 0
35c4 41 :3 0 3f
:3 0 8da 8db 0
35c4 6a :3 0 ff
:3 0 100 :3 0 8de
8df 0 101 :3 0
a1 :3 0 4f :3 0
4d :2 0 4cf 8e2
8e5 c2 :3 0 102
:3 0 8e7 8e8 0
4d2 8e1 8ea 8eb
:2 0 103 :2 0 4d5
8e0 8ee 8dd 8ef
0 35c4 4f :3 0
101 :3 0 a1 :3 0
4f :3 0 4d :2 0
4d8 8f3 8f6 c2
:3 0 104 :3 0 8f8
8f9 0 4db 8f2
8fb 8f1 8fc 0
35c4 54 :3 0 ff
:3 0 100 :3 0 8ff
900 0 101 :3 0
a1 :3 0 5b :3 0
4d :2 0 4de 903
906 c2 :3 0 102
:3 0 908 909 0
4e1 902 90b 90c
:2 0 103 :2 0 4e4
901 90f 8fe 910
0 35c4 5b :3 0
101 :3 0 a1 :3 0
5b :3 0 4d :2 0
4e7 914 917 c2
:3 0 104 :3 0 919
91a 0 4ea 913
91c 912 91d 0
35c4 45 :3 0 a1
:3 0 5b :3 0 4d
:2 0 4ed 920 923
c2 :3 0 d2 :2 0
d3 :3 0 925 927
0 4f2 926 929
:3 0 46 :2 0 4f5
92c 92b 4d :3 0
92e 0 4f7 :2 0
91f 930 0 35c4
a1 :3 0 4f :3 0
4d :2 0 4fa 932
935 105 :2 0 4d
:2 0 4ff 937 939
:3 0 82 :3 0 4d
:2 0 93b 93c 0
1203 106 :3 0 82
:3 0 107 :3 0 98
:3 0 25 :4 0 108
1 :8 0 949 21
:3 0 e9 :3 0 944
945 0 ea :3 0
946 947 0 949
502 99c eb :3 0
21 :3 0 e9 :3 0
94b 94c 0 ea
:3 0 94d 94e 0
950 505 952 507
951 950 :2 0 99a
ec :3 0 21 :3 0
e9 :3 0 955 956
0 ea :3 0 957
958 0 992 ad
:3 0 ae :3 0 95a
95b 0 ad :3 0
bd :3 0 95d 95e
0 ed :2 0 2
:4 0 109 :2 0 bf
:2 0 46 :2 0 509
963 965 :3 0 50c
95f 967 95c 969
968 :2 0 b0 :3 0
c0 :3 0 96b 96c
0 b2 :4 0 10a
:4 0 b5 :2 0 b6
:3 0 510 970 972
:3 0 b5 :2 0 c9
:4 0 513 974 976
:3 0 b5 :2 0 f0
:4 0 516 978 97a
:3 0 b5 :2 0 ea
:3 0 519 97c 97e
:3 0 b5 :2 0 f1
:4 0 51c 980 982
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 51f
985 989 523 984
98b :3 0 526 96d
98d :2 0 98f 529
990 96a 98f 0
991 52b 0 992
52d 995 :3 0 995
0 995 994 992
993 :6 0 997 8
:3 0 530 999 532
998 997 :2 0 99a
534 :2 0 99c 0
99c 99b 949 99a
:6 0 1203 1 :3 0
a1 :3 0 82 :3 0
4d :2 0 537 99e
9a1 105 :2 0 4d
:2 0 53c 9a3 9a5
:3 0 10b :3 0 4b
:3 0 10c :3 0 10d
:3 0 25 :4 0 10e
1 :8 0 9b2 21
:3 0 e9 :3 0 9ad
9ae 0 ea :3 0
9af 9b0 0 9b2
53f a05 eb :3 0
21 :3 0 e9 :3 0
9b4 9b5 0 ea
:3 0 9b6 9b7 0
9b9 542 9bb 544
9ba 9b9 :2 0 a03
ec :3 0 21 :3 0
e9 :3 0 9be 9bf
0 ea :3 0 9c0
9c1 0 9fb ad
:3 0 ae :3 0 9c3
9c4 0 ad :3 0
bd :3 0 9c6 9c7
0 ed :2 0 2
:4 0 10f :2 0 bf
:2 0 46 :2 0 546
9cc 9ce :3 0 549
9c8 9d0 9c5 9d2
9d1 :2 0 b0 :3 0
c0 :3 0 9d4 9d5
0 b2 :4 0 110
:4 0 b5 :2 0 b6
:3 0 54d 9d9 9db
:3 0 b5 :2 0 c9
:4 0 550 9dd 9df
:3 0 b5 :2 0 f0
:4 0 553 9e1 9e3
:3 0 b5 :2 0 ea
:3 0 556 9e5 9e7
:3 0 b5 :2 0 f1
:4 0 559 9e9 9eb
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 55c
9ee 9f2 560 9ed
9f4 :3 0 563 9d6
9f6 :2 0 9f8 566
9f9 9d3 9f8 0
9fa 568 0 9fb
56a 9fe :3 0 9fe
0 9fe 9fd 9fb
9fc :6 0 a00 a
:3 0 56d a02 56f
a01 a00 :2 0 a03
571 :2 0 a05 0
a05 a04 9b2 a03
:6 0 a82 1 :3 0
e5 :3 0 111 :3 0
a07 a08 0 4b
:3 0 112 :4 0 574
a09 a0c d2 :2 0
4d :2 0 579 a0e
a10 :3 0 ad :3 0
ae :3 0 a12 a13
0 ad :3 0 bd
:3 0 a15 a16 0
24 :2 0 2 :4 0
113 :2 0 bf :2 0
46 :2 0 57c a1b
a1d :3 0 57f a17
a1f a14 a21 a20
:2 0 b0 :3 0 c0
:3 0 a23 a24 0
b2 :4 0 114 :4 0
b5 :2 0 b6 :3 0
583 a28 a2a :3 0
b5 :2 0 c9 :4 0
586 a2c a2e :3 0
b5 :2 0 ca :4 0
589 a30 a32 :3 0
b5 :2 0 a1 :3 0
ba :3 0 c2 :3 0
115 :3 0 a37 a38
0 58c a36 a3a
bb :4 0 58e a35
a3d 591 a34 a3f
:3 0 b5 :2 0 116
:4 0 594 a41 a43
:3 0 b5 :2 0 b8
:4 0 597 a45 a47
:3 0 b5 :2 0 25
:3 0 59a a49 a4b
:3 0 b5 :2 0 b8
:4 0 59d a4d a4f
:3 0 5a0 a25 a51
:2 0 a53 5a3 a54
a22 a53 0 a55
5a5 0 a7f ad
:3 0 ae :3 0 a56
a57 0 ad :3 0
bd :3 0 a59 a5a
0 46 :2 0 2
:4 0 117 :2 0 bf
:2 0 46 :2 0 5a7
a5f a61 :3 0 5aa
a5b a63 a58 a65
a64 :2 0 b0 :3 0
c0 :3 0 a67 a68
0 b2 :4 0 118
:4 0 b5 :2 0 b6
:3 0 5ae a6c a6e
:3 0 5b1 a69 a70
:2 0 a72 5b4 a73
a66 a72 0 a74
5b6 0 a7f ce
:3 0 119 :3 0 a75
a76 0 c2 :3 0
115 :3 0 a78 a79
0 25 :3 0 d0
:3 0 5b8 a77 a7d
:2 0 a7f 5bc a80
a11 a7f 0 a81
5c0 0 a82 5c2
a83 9a6 a82 0
a84 5c5 0 1203
79 :3 0 11a :3 0
a1 :3 0 4f :3 0
4d :2 0 5c7 a87
a8a 5ca a86 a8c
a85 a8d 0 1203
a1 :3 0 79 :3 0
4d :2 0 5cc a8f
a92 d2 :2 0 4d
:2 0 5d1 a94 a96
:3 0 4f :3 0 11b
:3 0 a1 :3 0 4f
:3 0 4d :2 0 5d4
a9a a9d 5d7 a99
a9f a98 aa0 0
b0f a1 :3 0 4f
:3 0 4d :2 0 5d9
aa2 aa5 d2 :2 0
4d :2 0 5de aa7
aa9 :3 0 ad :3 0
ae :3 0 aab aac
0 ad :3 0 bd
:3 0 aae aaf 0
c6 :2 0 2 :4 0
11c :2 0 bf :2 0
46 :2 0 5e1 ab4
ab6 :3 0 5e4 ab0
ab8 aad aba ab9
:2 0 b0 :3 0 c0
:3 0 abc abd 0
b2 :4 0 11d :4 0
b5 :2 0 b6 :3 0
5e8 ac1 ac3 :3 0
b5 :2 0 c9 :4 0
5eb ac5 ac7 :3 0
b5 :2 0 ca :4 0
5ee ac9 acb :3 0
b5 :2 0 a1 :3 0
ba :3 0 c2 :3 0
d6 :3 0 ad0 ad1
0 5f1 acf ad3
bb :4 0 5f3 ace
ad6 5f6 acd ad8
:3 0 5f9 abe ada
:2 0 adc 5fc add
abb adc 0 ade
5fe 0 b07 ad
:3 0 ae :3 0 adf
ae0 0 ad :3 0
bd :3 0 ae2 ae3
0 46 :2 0 2
:4 0 11e :2 0 bf
:2 0 46 :2 0 600
ae8 aea :3 0 603
ae4 aec ae1 aee
aed :2 0 b0 :3 0
c0 :3 0 af0 af1
0 b2 :4 0 11f
:4 0 b5 :2 0 b6
:3 0 607 af5 af7
:3 0 60a af2 af9
:2 0 afb 60d afc
aef afb 0 afd
60f 0 b07 ce
:3 0 cf :3 0 afe
aff 0 c2 :3 0
d6 :3 0 b01 b02
0 d0 :3 0 611
b00 b05 :2 0 b07
614 b08 aaa b07
0 b09 618 0
b0f 79 :3 0 c2
:3 0 120 :3 0 b0b
b0c 0 b0a b0d
0 b0f 61a b10
a97 b0f 0 b11
61e 0 1203 26
:3 0 25 :3 0 b12
b13 0 1203 a1
:3 0 6a :3 0 4d
:2 0 620 b15 b18
c3 :2 0 4d :2 0
625 b1a b1c :3 0
39 :3 0 14 :3 0
b1e b1f 0 b21
628 b7f ad :3 0
ae :3 0 b22 b23
0 ad :3 0 bd
:3 0 b25 b26 0
c6 :2 0 2 :4 0
121 :2 0 bf :2 0
46 :2 0 62a b2b
b2d :3 0 62d b27
b2f b24 b31 b30
:2 0 b0 :3 0 c0
:3 0 b33 b34 0
b2 :4 0 122 :4 0
b5 :2 0 b6 :3 0
631 b38 b3a :3 0
b5 :2 0 c9 :4 0
634 b3c b3e :3 0
b5 :2 0 ca :4 0
637 b40 b42 :3 0
b5 :2 0 a1 :3 0
ba :3 0 c2 :3 0
d6 :3 0 b47 b48
0 63a b46 b4a
bb :4 0 63c b45
b4d 63f b44 b4f
:3 0 642 b35 b51
:2 0 b53 645 b54
b32 b53 0 b55
647 0 b7e ad
:3 0 ae :3 0 b56
b57 0 ad :3 0
bd :3 0 b59 b5a
0 46 :2 0 2
:4 0 123 :2 0 bf
:2 0 46 :2 0 649
b5f b61 :3 0 64c
b5b b63 b58 b65
b64 :2 0 b0 :3 0
c0 :3 0 b67 b68
0 b2 :4 0 124
:4 0 b5 :2 0 b6
:3 0 650 b6c b6e
:3 0 653 b69 b70
:2 0 b72 656 b73
b66 b72 0 b74
658 0 b7e ce
:3 0 cf :3 0 b75
b76 0 c2 :3 0
d6 :3 0 b78 b79
0 d0 :3 0 65a
b77 b7c :2 0 b7e
65d b80 b1d b21
0 b81 0 b7e
0 b81 661 0
1203 84 :3 0 125
:3 0 79 :3 0 4f
:3 0 26 :3 0 39
:3 0 29 :3 0 47
:3 0 664 b83 b8a
b82 b8b 0 1203
a1 :3 0 84 :3 0
4d :2 0 66b b8d
b90 c3 :2 0 4d
:2 0 670 b92 b94
:3 0 ad :3 0 ae
:3 0 b96 b97 0
ad :3 0 bd :3 0
b99 b9a 0 24
:2 0 2 :4 0 126
:2 0 bf :2 0 46
:2 0 673 b9f ba1
:3 0 676 b9b ba3
b98 ba5 ba4 :2 0
b0 :3 0 c0 :3 0
ba7 ba8 0 b2
:4 0 127 :4 0 b5
:2 0 b6 :3 0 67a
bac bae :3 0 b5
:2 0 c9 :4 0 67d
bb0 bb2 :3 0 b5
:2 0 ca :4 0 680
bb4 bb6 :3 0 b5
:2 0 a1 :3 0 ba
:3 0 84 :3 0 683
bba bbc bb :4 0
685 bb9 bbf 688
bb8 bc1 :3 0 b5
:2 0 116 :4 0 68b
bc3 bc5 :3 0 b5
:2 0 b8 :4 0 68e
bc7 bc9 :3 0 b5
:2 0 47 :3 0 691
bcb bcd :3 0 b5
:2 0 b8 :4 0 694
bcf bd1 :3 0 697
ba9 bd3 :2 0 bd5
69a bd6 ba6 bd5
0 bd7 69c 0
c03 ad :3 0 ae
:3 0 bd8 bd9 0
ad :3 0 bd :3 0
bdb bdc 0 46
:2 0 2 :4 0 128
:2 0 bf :2 0 46
:2 0 69e be1 be3
:3 0 6a1 bdd be5
bda be7 be6 :2 0
b0 :3 0 c0 :3 0
be9 bea 0 b2
:4 0 129 :4 0 b5
:2 0 b6 :3 0 6a5
bee bf0 :3 0 6a8
beb bf2 :2 0 bf4
6ab bf5 be8 bf4
0 bf6 6ad 0
c03 ce :3 0 119
:3 0 bf7 bf8 0
a1 :3 0 84 :3 0
4d :2 0 6af bfa
bfd 47 :3 0 d0
:3 0 6b2 bf9 c01
:2 0 c03 6b6 c04
b95 c03 0 c05
6ba 0 1203 a1
:3 0 4c :3 0 4d
:2 0 6bc c06 c09
c2 :3 0 d2 :2 0
c4 :3 0 c0b c0d
0 6c1 c0c c0f
:3 0 ad :3 0 ae
:3 0 c11 c12 0
ad :3 0 bd :3 0
c14 c15 0 24
:2 0 2 :4 0 12a
:2 0 bf :2 0 46
:2 0 6c4 c1a c1c
:3 0 6c7 c16 c1e
c13 c20 c1f :2 0
b0 :3 0 c0 :3 0
c22 c23 0 b2
:4 0 12b :4 0 b5
:2 0 b6 :3 0 6cb
c27 c29 :3 0 b5
:2 0 c9 :4 0 6ce
c2b c2d :3 0 b5
:2 0 ca :4 0 6d1
c2f c31 :3 0 b5
:2 0 a1 :3 0 ba
:3 0 21 :3 0 e9
:3 0 c36 c37 0
6d4 c35 c39 bb
:4 0 6d6 c34 c3c
6d9 c33 c3e :3 0
b5 :2 0 116 :4 0
6dc c40 c42 :3 0
b5 :2 0 b8 :4 0
6df c44 c46 :3 0
b5 :2 0 47 :3 0
6e2 c48 c4a :3 0
b5 :2 0 b8 :4 0
6e5 c4c c4e :3 0
6e8 c24 c50 :2 0
c52 6eb c53 c21
c52 0 c54 6ed
0 c7e ad :3 0
ae :3 0 c55 c56
0 ad :3 0 bd
:3 0 c58 c59 0
46 :2 0 2 :4 0
12c :2 0 bf :2 0
46 :2 0 6ef c5e
c60 :3 0 6f2 c5a
c62 c57 c64 c63
:2 0 b0 :3 0 c0
:3 0 c66 c67 0
b2 :4 0 12d :4 0
b5 :2 0 b6 :3 0
6f6 c6b c6d :3 0
6f9 c68 c6f :2 0
c71 6fc c72 c65
c71 0 c73 6fe
0 c7e ce :3 0
119 :3 0 c74 c75
0 21 :3 0 e9
:3 0 c77 c78 0
47 :3 0 d0 :3 0
700 c76 c7c :2 0
c7e 704 c7f c10
c7e 0 c80 708
0 1203 a1 :3 0
79 :3 0 4d :2 0
70a c81 c84 c2
:3 0 d2 :2 0 120
:3 0 c86 c88 0
70f c87 c8a :3 0
12e :3 0 a1 :3 0
4f :3 0 4d :2 0
712 c8d c90 715
c8c c92 c3 :2 0
4d :2 0 719 c94
c96 :3 0 98 :3 0
a1 :3 0 12f :3 0
a1 :3 0 130 :3 0
131 :3 0 25 :3 0
5e :3 0 5f :3 0
63 :3 0 10c :3 0
10d :3 0 26 :4 0
132 1 :8 0 cab
21 :3 0 e9 :3 0
ca6 ca7 0 ea
:3 0 ca8 ca9 0
cab 71c cfe eb
:3 0 21 :3 0 e9
:3 0 cad cae 0
ea :3 0 caf cb0
0 cb2 71f cb4
721 cb3 cb2 :2 0
cfc ec :3 0 21
:3 0 e9 :3 0 cb7
cb8 0 ea :3 0
cb9 cba 0 cf4
ad :3 0 ae :3 0
cbc cbd 0 ad
:3 0 bd :3 0 cbf
cc0 0 ed :2 0
2 :4 0 133 :2 0
bf :2 0 46 :2 0
723 cc5 cc7 :3 0
726 cc1 cc9 cbe
ccb cca :2 0 b0
:3 0 c0 :3 0 ccd
cce 0 b2 :4 0
134 :4 0 b5 :2 0
b6 :3 0 72a cd2
cd4 :3 0 b5 :2 0
c9 :4 0 72d cd6
cd8 :3 0 b5 :2 0
f0 :4 0 730 cda
cdc :3 0 b5 :2 0
ea :3 0 733 cde
ce0 :3 0 b5 :2 0
f1 :4 0 736 ce2
ce4 :3 0 b5 :2 0
f2 :3 0 f3 :3 0
46 :2 0 f4 :2 0
739 ce7 ceb 73d
ce6 ced :3 0 740
ccf cef :2 0 cf1
743 cf2 ccc cf1
0 cf3 745 0
cf4 747 cf7 :3 0
cf7 0 cf7 cf6
cf4 cf5 :6 0 cf9
c :3 0 74a cfb
74c cfa cf9 :2 0
cfc 74e :2 0 cfe
0 cfe cfd cab
cfc :6 0 d00 1
:3 0 751 d84 135
:3 0 136 :3 0 135
:3 0 137 :3 0 135
:3 0 138 :3 0 135
:3 0 98 :3 0 a1
:3 0 139 :3 0 12f
:3 0 a1 :3 0 139
:3 0 130 :3 0 139
:3 0 131 :3 0 135
:3 0 13a :3 0 135
:3 0 13b :3 0 71
:3 0 43 :3 0 44
:3 0 25 :3 0 5e
:3 0 5f :3 0 63
:3 0 72 :3 0 10c
:3 0 139 :3 0 10c
:3 0 135 :3 0 135
:3 0 10d :3 0 26
:3 0 139 :3 0 10d
:3 0 135 :3 0 98
:4 0 13c 1 :8 0
d2e 21 :3 0 e9
:3 0 d29 d2a 0
ea :3 0 d2b d2c
0 d2e 753 d81
eb :3 0 21 :3 0
e9 :3 0 d30 d31
0 ea :3 0 d32
d33 0 d35 756
d37 758 d36 d35
:2 0 d7f ec :3 0
21 :3 0 e9 :3 0
d3a d3b 0 ea
:3 0 d3c d3d 0
d77 ad :3 0 ae
:3 0 d3f d40 0
ad :3 0 bd :3 0
d42 d43 0 ed
:2 0 2 :4 0 13d
:2 0 bf :2 0 46
:2 0 75a d48 d4a
:3 0 75d d44 d4c
d41 d4e d4d :2 0
b0 :3 0 c0 :3 0
d50 d51 0 b2
:4 0 13e :4 0 b5
:2 0 b6 :3 0 761
d55 d57 :3 0 b5
:2 0 c9 :4 0 764
d59 d5b :3 0 b5
:2 0 f0 :4 0 767
d5d d5f :3 0 b5
:2 0 ea :3 0 76a
d61 d63 :3 0 b5
:2 0 f1 :4 0 76d
d65 d67 :3 0 b5
:2 0 f2 :3 0 f3
:3 0 46 :2 0 f4
:2 0 770 d6a d6e
774 d69 d70 :3 0
777 d52 d72 :2 0
d74 77a d75 d4f
d74 0 d76 77c
0 d77 77e d7a
:3 0 d7a 0 d7a
d79 d77 d78 :6 0
d7c e :3 0 781
d7e 783 d7d d7c
:2 0 d7f 785 :2 0
d81 0 d81 d80
d2e d7f :6 0 d83
1 :3 0 788 d85
c97 d00 0 d86
0 d83 0 d86
78a 0 e02 21
:3 0 e9 :3 0 d87
d88 0 21 :3 0
c3 :2 0 f5 :3 0
d8a d8c 0 78f
d8b d8e :3 0 d8f
:2 0 ad :3 0 ae
:3 0 d91 d92 0
ad :3 0 bd :3 0
d94 d95 0 24
:2 0 2 :4 0 13f
:2 0 bf :2 0 46
:2 0 792 d9a d9c
:3 0 795 d96 d9e
d93 da0 d9f :2 0
b0 :3 0 c0 :3 0
da2 da3 0 b2
:4 0 140 :4 0 b5
:2 0 b6 :3 0 799
da7 da9 :3 0 b5
:2 0 c9 :4 0 79c
dab dad :3 0 b5
:2 0 ca :4 0 79f
daf db1 :3 0 b5
:2 0 a1 :3 0 ba
:3 0 c2 :3 0 141
:3 0 db6 db7 0
7a2 db5 db9 bb
:4 0 7a4 db4 dbc
7a7 db3 dbe :3 0
b5 :2 0 116 :4 0
7aa dc0 dc2 :3 0
b5 :2 0 b8 :4 0
7ad dc4 dc6 :3 0
b5 :2 0 26 :3 0
7b0 dc8 dca :3 0
b5 :2 0 b8 :4 0
7b3 dcc dce :3 0
7b6 da4 dd0 :2 0
dd2 7b9 dd3 da1
dd2 0 dd4 7bb
0 dfe ad :3 0
ae :3 0 dd5 dd6
0 ad :3 0 bd
:3 0 dd8 dd9 0
46 :2 0 2 :4 0
142 :2 0 bf :2 0
46 :2 0 7bd dde
de0 :3 0 7c0 dda
de2 dd7 de4 de3
:2 0 b0 :3 0 c0
:3 0 de6 de7 0
b2 :4 0 143 :4 0
b5 :2 0 b6 :3 0
7c4 deb ded :3 0
7c7 de8 def :2 0
df1 7ca df2 de5
df1 0 df3 7cc
0 dfe ce :3 0
119 :3 0 df4 df5
0 c2 :3 0 141
:3 0 df7 df8 0
26 :3 0 d0 :3 0
7ce df6 dfc :2 0
dfe 7d2 dff d90
dfe 0 e00 7d6
0 e02 d9 :3 0
7d8 1057 a1 :3 0
79 :3 0 4d :2 0
7db e03 e06 c2
:3 0 d2 :2 0 144
:3 0 e08 e0a 0
7e0 e09 e0c :3 0
98 :3 0 145 :3 0
146 :3 0 25 :3 0
4f :3 0 23 :3 0
147 :3 0 10d :3 0
26 :4 0 148 1
:8 0 e1d 21 :3 0
e9 :3 0 e18 e19
0 ea :3 0 e1a
e1b 0 e1d 7e3
e70 eb :3 0 21
:3 0 e9 :3 0 e1f
e20 0 ea :3 0
e21 e22 0 e24
7e6 e26 7e8 e25
e24 :2 0 e6e ec
:3 0 21 :3 0 e9
:3 0 e29 e2a 0
ea :3 0 e2b e2c
0 e66 ad :3 0
ae :3 0 e2e e2f
0 ad :3 0 bd
:3 0 e31 e32 0
ed :2 0 2 :4 0
149 :2 0 bf :2 0
46 :2 0 7ea e37
e39 :3 0 7ed e33
e3b e30 e3d e3c
:2 0 b0 :3 0 c0
:3 0 e3f e40 0
b2 :4 0 14a :4 0
b5 :2 0 b6 :3 0
7f1 e44 e46 :3 0
b5 :2 0 c9 :4 0
7f4 e48 e4a :3 0
b5 :2 0 f0 :4 0
7f7 e4c e4e :3 0
b5 :2 0 ea :3 0
7fa e50 e52 :3 0
b5 :2 0 f1 :4 0
7fd e54 e56 :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 800 e59
e5d 804 e58 e5f
:3 0 807 e41 e61
:2 0 e63 80a e64
e3e e63 0 e65
80c 0 e66 80e
e69 :3 0 e69 0
e69 e68 e66 e67
:6 0 e6b 10 :3 0
811 e6d 813 e6c
e6b :2 0 e6e 815
:2 0 e70 0 e70
e6f e1d e6e :6 0
fe7 1 :3 0 21
:3 0 e9 :3 0 e72
e73 0 21 :3 0
c3 :2 0 f5 :3 0
e75 e77 0 81a
e76 e79 :3 0 e7a
:2 0 ad :3 0 ae
:3 0 e7c e7d 0
ad :3 0 bd :3 0
e7f e80 0 24
:2 0 2 :4 0 14b
:2 0 bf :2 0 46
:2 0 81d e85 e87
:3 0 820 e81 e89
e7e e8b e8a :2 0
b0 :3 0 c0 :3 0
e8d e8e 0 b2
:4 0 14c :4 0 b5
:2 0 b6 :3 0 824
e92 e94 :3 0 b5
:2 0 c9 :4 0 827
e96 e98 :3 0 b5
:2 0 ca :4 0 82a
e9a e9c :3 0 b5
:2 0 a1 :3 0 ba
:3 0 c2 :3 0 141
:3 0 ea1 ea2 0
82d ea0 ea4 bb
:4 0 82f e9f ea7
832 e9e ea9 :3 0
b5 :2 0 116 :4 0
835 eab ead :3 0
b5 :2 0 b8 :4 0
838 eaf eb1 :3 0
b5 :2 0 26 :3 0
83b eb3 eb5 :3 0
b5 :2 0 b8 :4 0
83e eb7 eb9 :3 0
841 e8f ebb :2 0
ebd 844 ebe e8c
ebd 0 ebf 846
0 ee9 ad :3 0
ae :3 0 ec0 ec1
0 ad :3 0 bd
:3 0 ec3 ec4 0
46 :2 0 2 :4 0
14d :2 0 bf :2 0
46 :2 0 848 ec9
ecb :3 0 84b ec5
ecd ec2 ecf ece
:2 0 b0 :3 0 c0
:3 0 ed1 ed2 0
b2 :4 0 14e :4 0
b5 :2 0 b6 :3 0
84f ed6 ed8 :3 0
852 ed3 eda :2 0
edc 855 edd ed0
edc 0 ede 857
0 ee9 ce :3 0
119 :3 0 edf ee0
0 c2 :3 0 141
:3 0 ee2 ee3 0
26 :3 0 d0 :3 0
859 ee1 ee7 :2 0
ee9 85d eea e7b
ee9 0 eeb 861
0 fe7 14f :3 0
41 :3 0 14f :3 0
150 :3 0 41 :3 0
a1 :3 0 12f :3 0
a1 :3 0 130 :3 0
131 :3 0 41 :3 0
5e :3 0 5f :3 0
63 :3 0 10c :3 0
98 :3 0 25 :3 0
10d :3 0 25 :4 0
151 1 :8 0 f05
21 :3 0 e9 :3 0
f00 f01 0 ea
:3 0 f02 f03 0
f05 863 f58 eb
:3 0 21 :3 0 e9
:3 0 f07 f08 0
ea :3 0 f09 f0a
0 f0c 866 f0e
868 f0d f0c :2 0
f56 ec :3 0 21
:3 0 e9 :3 0 f11
f12 0 ea :3 0
f13 f14 0 f4e
ad :3 0 ae :3 0
f16 f17 0 ad
:3 0 bd :3 0 f19
f1a 0 ed :2 0
2 :4 0 152 :2 0
bf :2 0 46 :2 0
86a f1f f21 :3 0
86d f1b f23 f18
f25 f24 :2 0 b0
:3 0 c0 :3 0 f27
f28 0 b2 :4 0
153 :4 0 b5 :2 0
b6 :3 0 871 f2c
f2e :3 0 b5 :2 0
c9 :4 0 874 f30
f32 :3 0 b5 :2 0
f0 :4 0 877 f34
f36 :3 0 b5 :2 0
ea :3 0 87a f38
f3a :3 0 b5 :2 0
f1 :4 0 87d f3c
f3e :3 0 b5 :2 0
f2 :3 0 f3 :3 0
46 :2 0 f4 :2 0
880 f41 f45 884
f40 f47 :3 0 887
f29 f49 :2 0 f4b
88a f4c f26 f4b
0 f4d 88c 0
f4e 88e f51 :3 0
f51 0 f51 f50
f4e f4f :6 0 f53
12 :3 0 891 f55
893 f54 f53 :2 0
f56 895 :2 0 f58
0 f58 f57 f05
f56 :6 0 fe7 1
:3 0 21 :3 0 e9
:3 0 f5a f5b 0
21 :3 0 c3 :2 0
f5 :3 0 f5d f5f
0 89a f5e f61
:3 0 f62 :2 0 ad
:3 0 ae :3 0 f64
f65 0 ad :3 0
bd :3 0 f67 f68
0 24 :2 0 2
:4 0 154 :2 0 bf
:2 0 46 :2 0 89d
f6d f6f :3 0 8a0
f69 f71 f66 f73
f72 :2 0 b0 :3 0
c0 :3 0 f75 f76
0 b2 :4 0 155
:4 0 b5 :2 0 b6
:3 0 8a4 f7a f7c
:3 0 b5 :2 0 c9
:4 0 8a7 f7e f80
:3 0 b5 :2 0 ca
:4 0 8aa f82 f84
:3 0 b5 :2 0 a1
:3 0 ba :3 0 c2
:3 0 141 :3 0 f89
f8a 0 8ad f88
f8c bb :4 0 8af
f87 f8f 8b2 f86
f91 :3 0 b5 :2 0
116 :4 0 8b5 f93
f95 :3 0 b5 :2 0
b8 :4 0 8b8 f97
f99 :3 0 b5 :2 0
25 :3 0 8bb f9b
f9d :3 0 b5 :2 0
b8 :4 0 8be f9f
fa1 :3 0 8c1 f77
fa3 :2 0 fa5 8c4
fa6 f74 fa5 0
fa7 8c6 0 fd1
ad :3 0 ae :3 0
fa8 fa9 0 ad
:3 0 bd :3 0 fab
fac 0 46 :2 0
2 :4 0 156 :2 0
bf :2 0 46 :2 0
8c8 fb1 fb3 :3 0
8cb fad fb5 faa
fb7 fb6 :2 0 b0
:3 0 c0 :3 0 fb9
fba 0 b2 :4 0
157 :4 0 b5 :2 0
b6 :3 0 8cf fbe
fc0 :3 0 8d2 fbb
fc2 :2 0 fc4 8d5
fc5 fb8 fc4 0
fc6 8d7 0 fd1
ce :3 0 119 :3 0
fc7 fc8 0 c2
:3 0 141 :3 0 fca
fcb 0 25 :3 0
d0 :3 0 8d9 fc9
fcf :2 0 fd1 8dd
fd2 f63 fd1 0
fd3 8e1 0 fe7
23 :3 0 e4 :2 0
8e3 fd5 fd6 :3 0
fd7 :2 0 79 :3 0
c2 :3 0 120 :3 0
fda fdb 0 fd9
fdc 0 fe4 4f
:3 0 158 :3 0 4f
:3 0 8e5 fdf fe1
fde fe2 0 fe4
8e7 fe5 fd8 fe4
0 fe6 8ea 0
fe7 8ec fe8 e0d
fe7 0 1059 ad
:3 0 ae :3 0 fe9
fea 0 ad :3 0
bd :3 0 fec fed
0 24 :2 0 2
:4 0 159 :2 0 bf
:2 0 46 :2 0 8f2
ff2 ff4 :3 0 8f5
fee ff6 feb ff8
ff7 :2 0 b0 :3 0
c0 :3 0 ffa ffb
0 b2 :4 0 15a
:4 0 b5 :2 0 b6
:3 0 8f9 fff 1001
:3 0 b5 :2 0 c9
:4 0 8fc 1003 1005
:3 0 b5 :2 0 ca
:4 0 8ff 1007 1009
:3 0 b5 :2 0 a1
:3 0 ba :3 0 c2
:3 0 d6 :3 0 100e
100f 0 902 100d
1011 bb :4 0 904
100c 1014 907 100b
1016 :3 0 b5 :2 0
116 :4 0 90a 1018
101a :3 0 b5 :2 0
b8 :4 0 90d 101c
101e :3 0 b5 :2 0
25 :3 0 910 1020
1022 :3 0 b5 :2 0
b8 :4 0 913 1024
1026 :3 0 916 ffc
1028 :2 0 102a 919
102b ff9 102a 0
102c 91b 0 1056
ad :3 0 ae :3 0
102d 102e 0 ad
:3 0 bd :3 0 1030
1031 0 46 :2 0
2 :4 0 15b :2 0
bf :2 0 46 :2 0
91d 1036 1038 :3 0
920 1032 103a 102f
103c 103b :2 0 b0
:3 0 c0 :3 0 103e
103f 0 b2 :4 0
15c :4 0 b5 :2 0
b6 :3 0 924 1043
1045 :3 0 927 1040
1047 :2 0 1049 92a
104a 103d 1049 0
104b 92c 0 1056
ce :3 0 119 :3 0
104c 104d 0 c2
:3 0 d6 :3 0 104f
1050 0 25 :3 0
d0 :3 0 92e 104e
1054 :2 0 1056 932
1058 c8b e02 0
1059 0 1056 0
1059 936 0 1203
a1 :3 0 5b :3 0
4d :2 0 93a 105a
105d c2 :3 0 c3
:2 0 d3 :3 0 105f
1061 0 93f 1060
1063 :3 0 15d :3 0
e5 :3 0 15e :3 0
41 :3 0 15f :3 0
a1 :3 0 e5 :3 0
15e :3 0 160 :3 0
a1 :3 0 e5 :3 0
15e :3 0 161 :3 0
4a :3 0 6c :3 0
70 :3 0 6d :3 0
6e :3 0 162 :3 0
9a :3 0 39 :4 0
163 1 :8 0 1080
21 :3 0 e9 :3 0
107b 107c 0 ea
:3 0 107d 107e 0
1080 942 10d3 eb
:3 0 21 :3 0 e9
:3 0 1082 1083 0
ea :3 0 1084 1085
0 1087 945 1089
947 1088 1087 :2 0
10d1 ec :3 0 21
:3 0 e9 :3 0 108c
108d 0 ea :3 0
108e 108f 0 10c9
ad :3 0 ae :3 0
1091 1092 0 ad
:3 0 bd :3 0 1094
1095 0 ed :2 0
2 :4 0 164 :2 0
bf :2 0 46 :2 0
949 109a 109c :3 0
94c 1096 109e 1093
10a0 109f :2 0 b0
:3 0 c0 :3 0 10a2
10a3 0 b2 :4 0
165 :4 0 b5 :2 0
b6 :3 0 950 10a7
10a9 :3 0 b5 :2 0
c9 :4 0 953 10ab
10ad :3 0 b5 :2 0
f0 :4 0 956 10af
10b1 :3 0 b5 :2 0
ea :3 0 959 10b3
10b5 :3 0 b5 :2 0
f1 :4 0 95c 10b7
10b9 :3 0 b5 :2 0
f2 :3 0 f3 :3 0
46 :2 0 f4 :2 0
95f 10bc 10c0 963
10bb 10c2 :3 0 966
10a4 10c4 :2 0 10c6
969 10c7 10a1 10c6
0 10c8 96b 0
10c9 96d 10cc :3 0
10cc 0 10cc 10cb
10c9 10ca :6 0 10ce
14 :3 0 970 10d0
972 10cf 10ce :2 0
10d1 974 :2 0 10d3
0 10d3 10d2 1080
10d1 :6 0 1200 1
:3 0 21 :3 0 e9
:3 0 10d5 10d6 0
21 :3 0 d2 :2 0
166 :3 0 10d8 10da
0 979 10d9 10dc
:3 0 10dd :2 0 ad
:3 0 ae :3 0 10df
10e0 0 ad :3 0
bd :3 0 10e2 10e3
0 24 :2 0 2
:4 0 167 :2 0 bf
:2 0 46 :2 0 97c
10e8 10ea :3 0 97f
10e4 10ec 10e1 10ee
10ed :2 0 b0 :3 0
c0 :3 0 10f0 10f1
0 b2 :4 0 168
:4 0 b5 :2 0 b6
:3 0 983 10f5 10f7
:3 0 b5 :2 0 c9
:4 0 986 10f9 10fb
:3 0 b5 :2 0 ca
:4 0 989 10fd 10ff
:3 0 b5 :2 0 a1
:3 0 ba :3 0 c2
:3 0 141 :3 0 1104
1105 0 98c 1103
1107 bb :4 0 98e
1102 110a 991 1101
110c :3 0 b5 :2 0
116 :4 0 994 110e
1110 :3 0 b5 :2 0
b8 :4 0 997 1112
1114 :3 0 b5 :2 0
39 :3 0 99a 1116
1118 :3 0 b5 :2 0
b8 :4 0 99d 111a
111c :3 0 9a0 10f2
111e :2 0 1120 9a3
1121 10ef 1120 0
1122 9a5 0 114d
ad :3 0 ae :3 0
1123 1124 0 ad
:3 0 bd :3 0 1126
1127 0 46 :2 0
2 :4 0 169 :2 0
bf :2 0 46 :2 0
9a7 112c 112e :3 0
9aa 1128 1130 1125
1132 1131 :2 0 b0
:3 0 c0 :3 0 1134
1135 0 b2 :4 0
16a :4 0 b5 :2 0
b6 :3 0 9ae 1139
113b :3 0 9b1 1136
113d :2 0 113f 9b4
1140 1133 113f 0
1141 9b6 0 114d
ce :3 0 119 :3 0
1142 1143 0 c2
:3 0 141 :3 0 1145
1146 0 39 :3 0
d0 :3 0 9b8 1144
114a :2 0 114d d9
:3 0 9bc 11fe 21
:3 0 e9 :3 0 114e
114f 0 21 :3 0
d2 :2 0 f5 :3 0
1151 1153 0 9c2
1152 1155 :3 0 1156
:2 0 f2 :3 0 70
:3 0 46 :2 0 46
:2 0 9c5 1158 115c
d2 :2 0 4d :4 0
9cb 115e 1160 :3 0
a1 :3 0 6e :3 0
4d :2 0 9ce 1162
1165 105 :2 0 4d
:2 0 9d3 1167 1169
:3 0 a1 :3 0 6c
:3 0 4d :2 0 9d6
116b 116e a1 :3 0
105 :2 0 6e :3 0
4d :2 0 9d9 1170
1174 9de 1171 1176
:3 0 116a 1178 1177
:2 0 1179 :2 0 1161
117b 117a :2 0 a1
:3 0 6c :3 0 4d
:2 0 9e1 117d 1180
a1 :3 0 f6 :2 0
6d :3 0 4d :2 0
9e4 1182 1186 9e9
1183 1188 :3 0 117c
118a 1189 :2 0 ad
:3 0 ae :3 0 118c
118d 0 ad :3 0
bd :3 0 118f 1190
0 24 :2 0 2
:4 0 16b :2 0 bf
:2 0 46 :2 0 9ec
1195 1197 :3 0 9ef
1191 1199 118e 119b
119a :2 0 b0 :3 0
c0 :3 0 119d 119e
0 b2 :4 0 16c
:4 0 b5 :2 0 b6
:3 0 9f3 11a2 11a4
:3 0 b5 :2 0 c9
:4 0 9f6 11a6 11a8
:3 0 b5 :2 0 ca
:4 0 9f9 11aa 11ac
:3 0 b5 :2 0 a1
:3 0 ba :3 0 c2
:3 0 16d :3 0 11b1
11b2 0 9fc 11b0
11b4 bb :4 0 9fe
11af 11b7 a01 11ae
11b9 :3 0 b5 :2 0
116 :4 0 a04 11bb
11bd :3 0 b5 :2 0
b8 :4 0 a07 11bf
11c1 :3 0 b5 :2 0
39 :3 0 a0a 11c3
11c5 :3 0 b5 :2 0
b8 :4 0 a0d 11c7
11c9 :3 0 a10 119f
11cb :2 0 11cd a13
11ce 119c 11cd 0
11cf a15 0 11f9
ad :3 0 ae :3 0
11d0 11d1 0 ad
:3 0 bd :3 0 11d3
11d4 0 46 :2 0
2 :4 0 16e :2 0
bf :2 0 46 :2 0
a17 11d9 11db :3 0
a1a 11d5 11dd 11d2
11df 11de :2 0 b0
:3 0 c0 :3 0 11e1
11e2 0 b2 :4 0
16f :4 0 b5 :2 0
b6 :3 0 a1e 11e6
11e8 :3 0 a21 11e3
11ea :2 0 11ec a24
11ed 11e0 11ec 0
11ee a26 0 11f9
ce :3 0 119 :3 0
11ef 11f0 0 c2
:3 0 16d :3 0 11f2
11f3 0 39 :3 0
d0 :3 0 a28 11f1
11f7 :2 0 11f9 a2c
11fa 118b 11f9 0
11fb a30 0 11fc
a32 11fd 1157 11fc
0 11ff 10de 114d
0 11ff a34 0
1200 a37 1201 1064
1200 0 1202 a3a
0 1203 a3c 1204
93a 1203 0 1205
a49 0 35c4 a1
:3 0 4f :3 0 4d
:2 0 a4b 1206 1209
d2 :2 0 4d :2 0
a50 120b 120d :3 0
a1 :3 0 5b :3 0
4d :2 0 a53 120f
1212 c2 :3 0 d2
:2 0 d3 :3 0 1214
1216 0 a58 1215
1218 :3 0 16 :3 0
c5 :2 0 a5b 121b
121c :3 0 121d :2 0
1219 121f 121e :2 0
a1 :3 0 6a :3 0
4d :2 0 a5d 1221
1224 d2 :2 0 4d
:2 0 a62 1226 1228
:3 0 1220 122a 1229
:2 0 ad :3 0 ae
:3 0 122c 122d 0
ad :3 0 bd :3 0
122f 1230 0 24
:2 0 2 :4 0 170
:2 0 bf :2 0 46
:2 0 a65 1235 1237
:3 0 a68 1231 1239
122e 123b 123a :2 0
b0 :3 0 c0 :3 0
123d 123e 0 b2
:4 0 171 :4 0 b5
:2 0 b6 :3 0 a6c
1242 1244 :3 0 b5
:2 0 c9 :4 0 a6f
1246 1248 :3 0 b5
:2 0 ca :4 0 a72
124a 124c :3 0 b5
:2 0 a1 :3 0 ba
:3 0 c2 :3 0 172
:3 0 1251 1252 0
a75 1250 1254 bb
:4 0 a77 124f 1257
a7a 124e 1259 :3 0
b5 :2 0 116 :4 0
a7d 125b 125d :3 0
b5 :2 0 b8 :4 0
a80 125f 1261 :3 0
b5 :2 0 25 :3 0
a83 1263 1265 :3 0
b5 :2 0 b8 :4 0
a86 1267 1269 :3 0
a89 123f 126b :2 0
126d a8c 126e 123c
126d 0 126f a8e
0 1299 ad :3 0
ae :3 0 1270 1271
0 ad :3 0 bd
:3 0 1273 1274 0
46 :2 0 2 :4 0
173 :2 0 bf :2 0
46 :2 0 a90 1279
127b :3 0 a93 1275
127d 1272 127f 127e
:2 0 b0 :3 0 c0
:3 0 1281 1282 0
b2 :4 0 174 :4 0
b5 :2 0 b6 :3 0
a97 1286 1288 :3 0
a9a 1283 128a :2 0
128c a9d 128d 1280
128c 0 128e a9f
0 1299 ce :3 0
119 :3 0 128f 1290
0 c2 :3 0 172
:3 0 1292 1293 0
25 :3 0 d0 :3 0
aa1 1291 1297 :2 0
1299 aa5 129a 122b
1299 0 129b aa9
0 13f7 a1 :3 0
4c :3 0 4d :2 0
aab 129c 129f c2
:3 0 c3 :2 0 c4
:3 0 12a1 12a3 0
ab0 12a2 12a5 :3 0
131 :3 0 a1 :3 0
12f :3 0 a1 :3 0
130 :3 0 63 :3 0
5e :3 0 5f :3 0
10c :3 0 10d :3 0
25 :4 0 175 1
:8 0 12b8 21 :3 0
e9 :3 0 12b3 12b4
0 ea :3 0 12b5
12b6 0 12b8 ab3
130b eb :3 0 21
:3 0 e9 :3 0 12ba
12bb 0 ea :3 0
12bc 12bd 0 12bf
ab6 12c1 ab8 12c0
12bf :2 0 1309 ec
:3 0 21 :3 0 e9
:3 0 12c4 12c5 0
ea :3 0 12c6 12c7
0 1301 ad :3 0
ae :3 0 12c9 12ca
0 ad :3 0 bd
:3 0 12cc 12cd 0
ed :2 0 2 :4 0
176 :2 0 bf :2 0
46 :2 0 aba 12d2
12d4 :3 0 abd 12ce
12d6 12cb 12d8 12d7
:2 0 b0 :3 0 c0
:3 0 12da 12db 0
b2 :4 0 177 :4 0
b5 :2 0 b6 :3 0
ac1 12df 12e1 :3 0
b5 :2 0 c9 :4 0
ac4 12e3 12e5 :3 0
b5 :2 0 f0 :4 0
ac7 12e7 12e9 :3 0
b5 :2 0 ea :3 0
aca 12eb 12ed :3 0
b5 :2 0 f1 :4 0
acd 12ef 12f1 :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 ad0 12f4
12f8 ad4 12f3 12fa
:3 0 ad7 12dc 12fc
:2 0 12fe ada 12ff
12d9 12fe 0 1300
adc 0 1301 ade
1304 :3 0 1304 0
1304 1303 1301 1302
:6 0 1306 16 :3 0
ae1 1308 ae3 1307
1306 :2 0 1309 ae5
:2 0 130b 0 130b
130a 12b8 1309 :6 0
13f4 1 :3 0 21
:3 0 e9 :3 0 130d
130e 0 21 :3 0
d2 :2 0 166 :3 0
1310 1312 0 aea
1311 1314 :3 0 1315
:2 0 5e :3 0 5f
:3 0 178 :3 0 10d
:3 0 25 :4 0 179
1 :8 0 1322 21
:3 0 e9 :3 0 131d
131e 0 ea :3 0
131f 1320 0 1322
aed 1375 eb :3 0
21 :3 0 e9 :3 0
1324 1325 0 ea
:3 0 1326 1327 0
1329 af0 132b af2
132a 1329 :2 0 1373
ec :3 0 21 :3 0
e9 :3 0 132e 132f
0 ea :3 0 1330
1331 0 136b ad
:3 0 ae :3 0 1333
1334 0 ad :3 0
bd :3 0 1336 1337
0 ed :2 0 2
:4 0 17a :2 0 bf
:2 0 46 :2 0 af4
133c 133e :3 0 af7
1338 1340 1335 1342
1341 :2 0 b0 :3 0
c0 :3 0 1344 1345
0 b2 :4 0 17b
:4 0 b5 :2 0 b6
:3 0 afb 1349 134b
:3 0 b5 :2 0 c9
:4 0 afe 134d 134f
:3 0 b5 :2 0 f0
:4 0 b01 1351 1353
:3 0 b5 :2 0 ea
:3 0 b04 1355 1357
:3 0 b5 :2 0 f1
:4 0 b07 1359 135b
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 b0a
135e 1362 b0e 135d
1364 :3 0 b11 1346
1366 :2 0 1368 b14
1369 1343 1368 0
136a b16 0 136b
b18 136e :3 0 136e
0 136e 136d 136b
136c :6 0 1370 18
:3 0 b1b 1372 b1d
1371 1370 :2 0 1373
b1f :2 0 1375 0
1375 1374 1322 1373
:6 0 13f1 1 :3 0
21 :3 0 e9 :3 0
1377 1378 0 21
:3 0 d2 :2 0 166
:3 0 137a 137c 0
b24 137b 137e :3 0
137f :2 0 ad :3 0
ae :3 0 1381 1382
0 ad :3 0 bd
:3 0 1384 1385 0
24 :2 0 2 :4 0
17c :2 0 bf :2 0
46 :2 0 b27 138a
138c :3 0 b2a 1386
138e 1383 1390 138f
:2 0 b0 :3 0 c0
:3 0 1392 1393 0
b2 :4 0 17d :4 0
b5 :2 0 b6 :3 0
b2e 1397 1399 :3 0
b5 :2 0 c9 :4 0
b31 139b 139d :3 0
b5 :2 0 ca :4 0
b34 139f 13a1 :3 0
b5 :2 0 a1 :3 0
ba :3 0 c2 :3 0
d6 :3 0 13a6 13a7
0 b37 13a5 13a9
bb :4 0 b39 13a4
13ac b3c 13a3 13ae
:3 0 b5 :2 0 116
:4 0 b3f 13b0 13b2
:3 0 b5 :2 0 b8
:4 0 b42 13b4 13b6
:3 0 b5 :2 0 25
:3 0 b45 13b8 13ba
:3 0 b5 :2 0 b8
:4 0 b48 13bc 13be
:3 0 b4b 1394 13c0
:2 0 13c2 b4e 13c3
1391 13c2 0 13c4
b50 0 13ee ad
:3 0 ae :3 0 13c5
13c6 0 ad :3 0
bd :3 0 13c8 13c9
0 46 :2 0 2
:4 0 17e :2 0 bf
:2 0 46 :2 0 b52
13ce 13d0 :3 0 b55
13ca 13d2 13c7 13d4
13d3 :2 0 b0 :3 0
c0 :3 0 13d6 13d7
0 b2 :4 0 17f
:4 0 b5 :2 0 b6
:3 0 b59 13db 13dd
:3 0 b5c 13d8 13df
:2 0 13e1 b5f 13e2
13d5 13e1 0 13e3
b61 0 13ee ce
:3 0 119 :3 0 13e4
13e5 0 c2 :3 0
d6 :3 0 13e7 13e8
0 25 :3 0 d0
:3 0 b63 13e6 13ec
:2 0 13ee b67 13ef
1380 13ee 0 13f0
b6b 0 13f1 b6d
13f2 1316 13f1 0
13f3 b70 0 13f4
b72 13f5 12a6 13f4
0 13f6 b75 0
13f7 b77 13f8 120e
13f7 0 13f9 b7a
0 35c4 a1 :3 0
d :3 0 4d :2 0
b7c 13fa 13fd d2
:2 0 4d :2 0 b81
13ff 1401 :3 0 a1
:3 0 5a :3 0 4d
:2 0 b84 1403 1406
c3 :2 0 180 :2 0
b89 1408 140a :3 0
1402 140c 140b :2 0
56 :3 0 4d :2 0
140e 140f 0 1411
b8c 1412 140d 1411
0 1413 b8e 0
35c4 a1 :3 0 4c
:3 0 4d :2 0 b90
1414 1417 c2 :3 0
d2 :2 0 c4 :3 0
1419 141b 0 b95
141a 141d :3 0 5
:3 0 c5 :2 0 b98
1420 1421 :3 0 1422
:2 0 41 :3 0 181
:4 0 1424 1425 0
142a 42 :3 0 181
:4 0 1427 1428 0
142a b9a 1432 41
:3 0 5 :3 0 142b
142c 0 1431 42
:3 0 5 :3 0 142e
142f 0 1431 b9d
1433 1423 142a 0
1434 0 1431 0
1434 ba0 0 1438
58 :3 0 4d :2 0
1435 1436 0 1438
ba3 15ce 89 :3 0
d2 :2 0 4d :2 0
ba8 143a 143c :3 0
42 :3 0 182 :4 0
143e 143f 0 1556
82 :3 0 183 :3 0
41 :3 0 42 :3 0
a1 :3 0 59 :3 0
4d :2 0 bab 1445
1448 a1 :3 0 5a
:3 0 4d :2 0 bae
144a 144d a1 :3 0
5b :3 0 4d :2 0
bb1 144f 1452 a1
:3 0 4c :3 0 4d
:2 0 bb4 1454 1457
a1 :3 0 5c :3 0
4d :2 0 bb7 1459
145c a1 :3 0 57
:3 0 4d :2 0 bba
145e 1461 a1 :3 0
5e :3 0 4d :2 0
bbd 1463 1466 a1
:3 0 5f :3 0 4d
:2 0 bc0 1468 146b
56 :3 0 58 :3 0
60 :3 0 bc3 1442
1470 1441 1471 0
1556 a1 :3 0 82
:3 0 4d :2 0 bd1
1473 1476 c2 :3 0
d2 :2 0 d6 :3 0
1478 147a 0 bd6
1479 147c :3 0 ad
:3 0 ae :3 0 147e
147f 0 ad :3 0
bd :3 0 1481 1482
0 c6 :2 0 2
:4 0 184 :2 0 bf
:2 0 46 :2 0 bd9
1487 1489 :3 0 bdc
1483 148b 1480 148d
148c :2 0 b0 :3 0
c0 :3 0 148f 1490
0 b2 :4 0 185
:4 0 b5 :2 0 b6
:3 0 be0 1494 1496
:3 0 b5 :2 0 c9
:4 0 be3 1498 149a
:3 0 b5 :2 0 ca
:4 0 be6 149c 149e
:3 0 b5 :2 0 a1
:3 0 ba :3 0 c2
:3 0 d6 :3 0 14a3
14a4 0 be9 14a2
14a6 bb :4 0 beb
14a1 14a9 bee 14a0
14ab :3 0 bf1 1491
14ad :2 0 14af bf4
14b0 148e 14af 0
14b1 bf6 0 14db
ad :3 0 ae :3 0
14b2 14b3 0 ad
:3 0 bd :3 0 14b5
14b6 0 46 :2 0
2 :4 0 186 :2 0
bf :2 0 46 :2 0
bf8 14bb 14bd :3 0
bfb 14b7 14bf 14b4
14c1 14c0 :2 0 b0
:3 0 c0 :3 0 14c3
14c4 0 b2 :4 0
187 :4 0 b5 :2 0
b6 :3 0 bff 14c8
14ca :3 0 c02 14c5
14cc :2 0 14ce c05
14cf 14c2 14ce 0
14d0 c07 0 14db
ce :3 0 cf :3 0
14d1 14d2 0 c2
:3 0 d6 :3 0 14d4
14d5 0 d0 :3 0
c09 14d3 14d8 :2 0
14db d9 :3 0 c0c
1554 a1 :3 0 82
:3 0 4d :2 0 c10
14dc 14df c3 :2 0
4d :2 0 c15 14e1
14e3 :3 0 ad :3 0
ae :3 0 14e5 14e6
0 ad :3 0 bd
:3 0 14e8 14e9 0
24 :2 0 2 :4 0
188 :2 0 bf :2 0
46 :2 0 c18 14ee
14f0 :3 0 c1b 14ea
14f2 14e7 14f4 14f3
:2 0 b0 :3 0 c0
:3 0 14f6 14f7 0
b2 :4 0 189 :4 0
b5 :2 0 b6 :3 0
c1f 14fb 14fd :3 0
b5 :2 0 c9 :4 0
c22 14ff 1501 :3 0
b5 :2 0 ca :4 0
c25 1503 1505 :3 0
b5 :2 0 a1 :3 0
ba :3 0 82 :3 0
c28 1509 150b bb
:4 0 c2a 1508 150e
c2d 1507 1510 :3 0
b5 :2 0 116 :4 0
c30 1512 1514 :3 0
b5 :2 0 b8 :4 0
c33 1516 1518 :3 0
b5 :2 0 25 :3 0
c36 151a 151c :3 0
b5 :2 0 b8 :4 0
c39 151e 1520 :3 0
c3c 14f8 1522 :2 0
1524 c3f 1525 14f5
1524 0 1526 c41
0 1552 ad :3 0
ae :3 0 1527 1528
0 ad :3 0 bd
:3 0 152a 152b 0
46 :2 0 2 :4 0
18a :2 0 bf :2 0
46 :2 0 c43 1530
1532 :3 0 c46 152c
1534 1529 1536 1535
:2 0 b0 :3 0 c0
:3 0 1538 1539 0
b2 :4 0 18b :4 0
b5 :2 0 b6 :3 0
c4a 153d 153f :3 0
c4d 153a 1541 :2 0
1543 c50 1544 1537
1543 0 1545 c52
0 1552 ce :3 0
119 :3 0 1546 1547
0 a1 :3 0 82
:3 0 4d :2 0 c54
1549 154c 25 :3 0
d0 :3 0 c57 1548
1550 :2 0 1552 c5b
1553 14e4 1552 0
1555 147d 14db 0
1555 c5f 0 1556
c62 155b 42 :3 0
6 :3 0 1557 1558
0 155a c66 155c
143d 1556 0 155d
0 155a 0 155d
c68 0 15cd a1
:3 0 56 :3 0 4d
:2 0 c6b 155e 1561
105 :2 0 4d :2 0
c70 1563 1565 :3 0
1566 :2 0 a1 :3 0
5a :3 0 4d :2 0
c73 1568 156b c3
:2 0 180 :2 0 c78
156d 156f :3 0 1570
:2 0 1567 1572 1571
:2 0 a1 :3 0 5a
:3 0 4d :2 0 c7b
1574 1577 d2 :2 0
46 :2 0 c80 1579
157b :3 0 157c :2 0
a1 :3 0 5a :3 0
4d :2 0 c83 157e
1581 d2 :2 0 18c
:2 0 c88 1583 1585
:3 0 1586 :2 0 157d
1588 1587 :2 0 59
:3 0 18c :2 0 158a
158b 0 158e d9
:3 0 c8b 15c8 a1
:3 0 5a :3 0 4d
:2 0 c8d 158f 1592
d2 :2 0 4d :2 0
c92 1594 1596 :3 0
59 :3 0 46 :2 0
1598 1599 0 159c
d9 :3 0 c95 159d
1597 159c 0 15c9
a1 :3 0 5a :3 0
4d :2 0 c97 159e
15a1 d2 :2 0 18d
:2 0 c9c 15a3 15a5
:3 0 15a6 :2 0 a1
:3 0 5a :3 0 4d
:2 0 c9f 15a8 15ab
d2 :2 0 18e :2 0
ca4 15ad 15af :3 0
15b0 :2 0 15a7 15b2
15b1 :2 0 59 :3 0
46 :2 0 15b4 15b5
0 15b8 d9 :3 0
ca7 15b9 15b3 15b8
0 15c9 a1 :3 0
5a :3 0 4d :2 0
ca9 15ba 15bd d2
:2 0 de :2 0 cae
15bf 15c1 :3 0 59
:3 0 18d :2 0 15c3
15c4 0 15c6 cb1
15c7 15c2 15c6 0
15c9 1589 158e 0
15c9 cb3 0 15ca
cb8 15cb 1573 15ca
0 15cc cba 0
15cd cbc 15cf 141e
1438 0 15d0 0
15cd 0 15d0 cbf
0 35c4 a1 :3 0
5b :3 0 4d :2 0
cc2 15d1 15d4 c2
:3 0 d2 :2 0 d3
:3 0 15d6 15d8 0
cc7 15d7 15da :3 0
8d :3 0 4d :2 0
15dc 15dd 0 1ac7
2e :3 0 16 :3 0
15df 15e0 0 1ac7
30 :3 0 17 :3 0
15e2 15e3 0 1ac7
48 :3 0 f2 :3 0
2e :3 0 46 :2 0
24 :2 0 cca 15e6
15ea 15e5 15eb 0
1ac7 4b :4 0 15ed
15ee 0 1ac7 a1
:3 0 6a :3 0 4d
:2 0 cce 15f0 15f3
d2 :2 0 4d :2 0
cd3 15f5 15f7 :3 0
39 :3 0 18f :4 0
15fa 15fb :3 0 15f9
15fc 0 1601 8d
:3 0 46 :2 0 15fe
15ff 0 1601 cd6
16f7 10b :3 0 4b
:3 0 190 :3 0 191
:3 0 39 :4 0 192
1 :8 0 160d 21
:3 0 e9 :3 0 1608
1609 0 ea :3 0
160a 160b 0 160d
cd9 1660 eb :3 0
21 :3 0 e9 :3 0
160f 1610 0 ea
:3 0 1611 1612 0
1614 cdc 1616 cde
1615 1614 :2 0 165e
ec :3 0 21 :3 0
e9 :3 0 1619 161a
0 ea :3 0 161b
161c 0 1656 ad
:3 0 ae :3 0 161e
161f 0 ad :3 0
bd :3 0 1621 1622
0 ed :2 0 2
:4 0 193 :2 0 bf
:2 0 46 :2 0 ce0
1627 1629 :3 0 ce3
1623 162b 1620 162d
162c :2 0 b0 :3 0
c0 :3 0 162f 1630
0 b2 :4 0 194
:4 0 b5 :2 0 b6
:3 0 ce7 1634 1636
:3 0 b5 :2 0 c9
:4 0 cea 1638 163a
:3 0 b5 :2 0 f0
:4 0 ced 163c 163e
:3 0 b5 :2 0 ea
:3 0 cf0 1640 1642
:3 0 b5 :2 0 f1
:4 0 cf3 1644 1646
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 cf6
1649 164d cfa 1648
164f :3 0 cfd 1631
1651 :2 0 1653 d00
1654 162e 1653 0
1655 d02 0 1656
d04 1659 :3 0 1659
0 1659 1658 1656
1657 :6 0 165b 1a
:3 0 d07 165d d09
165c 165b :2 0 165e
d0b :2 0 1660 0
1660 165f 160d 165e
:6 0 16f6 1 :3 0
21 :3 0 e9 :3 0
1662 1663 0 21
:3 0 d2 :2 0 f5
:3 0 1665 1667 0
d10 1666 1669 :3 0
166a :2 0 a1 :3 0
4f :3 0 4d :2 0
d13 166c 166f 105
:2 0 4d :2 0 d18
1671 1673 :3 0 195
:3 0 28 :3 0 196
:3 0 10d :3 0 26
:3 0 9a :3 0 39
:3 0 197 :3 0 198
:3 0 c2 :3 0 199
:4 0 19a 1 :8 0
1686 21 :3 0 e9
:3 0 1681 1682 0
ea :3 0 1683 1684
0 1686 d1b 16d9
eb :3 0 21 :3 0
e9 :3 0 1688 1689
0 ea :3 0 168a
168b 0 168d d1e
168f d20 168e 168d
:2 0 16d7 ec :3 0
21 :3 0 e9 :3 0
1692 1693 0 ea
:3 0 1694 1695 0
16cf ad :3 0 ae
:3 0 1697 1698 0
ad :3 0 bd :3 0
169a 169b 0 ed
:2 0 2 :4 0 19b
:2 0 bf :2 0 46
:2 0 d22 16a0 16a2
:3 0 d25 169c 16a4
1699 16a6 16a5 :2 0
b0 :3 0 c0 :3 0
16a8 16a9 0 b2
:4 0 19c :4 0 b5
:2 0 b6 :3 0 d29
16ad 16af :3 0 b5
:2 0 c9 :4 0 d2c
16b1 16b3 :3 0 b5
:2 0 f0 :4 0 d2f
16b5 16b7 :3 0 b5
:2 0 ea :3 0 d32
16b9 16bb :3 0 b5
:2 0 f1 :4 0 d35
16bd 16bf :3 0 b5
:2 0 f2 :3 0 f3
:3 0 46 :2 0 f4
:2 0 d38 16c2 16c6
d3c 16c1 16c8 :3 0
d3f 16aa 16ca :2 0
16cc d42 16cd 16a7
16cc 0 16ce d44
0 16cf d46 16d2
:3 0 16d2 0 16d2
16d1 16cf 16d0 :6 0
16d4 1c :3 0 d49
16d6 d4b 16d5 16d4
:2 0 16d7 d4d :2 0
16d9 0 16d9 16d8
1686 16d7 :6 0 16eb
1 :3 0 21 :3 0
e9 :3 0 16db 16dc
0 21 :3 0 d2
:2 0 f5 :3 0 16de
16e0 0 d52 16df
16e2 :3 0 16e3 :2 0
8d :3 0 46 :2 0
16e5 16e6 0 16e8
d55 16e9 16e4 16e8
0 16ea d57 0
16eb d59 16ec 1674
16eb 0 16ed d5c
0 16ee d5e 16f3
8d :3 0 46 :2 0
16ef 16f0 0 16f2
d60 16f4 166b 16ee
0 16f5 0 16f2
0 16f5 d62 0
16f6 d65 16f8 15f8
1601 0 16f9 0
16f6 0 16f9 d68
0 1ac7 8d :3 0
c3 :2 0 4d :2 0
d6d 16fb 16fd :3 0
a1 :3 0 19d :3 0
19e :3 0 62 :3 0
190 :3 0 98 :3 0
25 :3 0 a0 :3 0
48 :4 0 19f 1
:8 0 170e 21 :3 0
e9 :3 0 1709 170a
0 ea :3 0 170b
170c 0 170e d70
1761 eb :3 0 21
:3 0 e9 :3 0 1710
1711 0 ea :3 0
1712 1713 0 1715
d73 1717 d75 1716
1715 :2 0 175f ec
:3 0 21 :3 0 e9
:3 0 171a 171b 0
ea :3 0 171c 171d
0 1757 ad :3 0
ae :3 0 171f 1720
0 ad :3 0 bd
:3 0 1722 1723 0
ed :2 0 2 :4 0
1a0 :2 0 bf :2 0
46 :2 0 d77 1728
172a :3 0 d7a 1724
172c 1721 172e 172d
:2 0 b0 :3 0 c0
:3 0 1730 1731 0
b2 :4 0 1a1 :4 0
b5 :2 0 b6 :3 0
d7e 1735 1737 :3 0
b5 :2 0 c9 :4 0
d81 1739 173b :3 0
b5 :2 0 f0 :4 0
d84 173d 173f :3 0
b5 :2 0 ea :3 0
d87 1741 1743 :3 0
b5 :2 0 f1 :4 0
d8a 1745 1747 :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 d8d 174a
174e d91 1749 1750
:3 0 d94 1732 1752
:2 0 1754 d97 1755
172f 1754 0 1756
d99 0 1757 d9b
175a :3 0 175a 0
175a 1759 1757 1758
:6 0 175c 1e :3 0
d9e 175e da0 175d
175c :2 0 175f da2
:2 0 1761 0 1761
1760 170e 175f :6 0
18ea 1 :3 0 e5
:3 0 1a2 :3 0 e5
:3 0 1a2 :3 0 e5
:3 0 1a2 :3 0 e5
:3 0 1a2 :3 0 e5
:3 0 1a2 :3 0 9f
:3 0 1a3 :3 0 1a4
:3 0 9f :3 0 1a5
:3 0 1a4 :3 0 9f
:3 0 ba :3 0 62
:3 0 4a :3 0 1a6
:3 0 1a5 :3 0 1a6
:3 0 1a3 :3 0 1a5
:3 0 10d :3 0 48
:3 0 1a3 :3 0 10d
:3 0 1a5 :3 0 1a7
:4 0 1a8 1 :8 0
1788 21 :3 0 e9
:3 0 1783 1784 0
ea :3 0 1785 1786
0 1788 da5 17db
eb :3 0 21 :3 0
e9 :3 0 178a 178b
0 ea :3 0 178c
178d 0 178f da8
1791 daa 1790 178f
:2 0 17d9 ec :3 0
21 :3 0 e9 :3 0
1794 1795 0 ea
:3 0 1796 1797 0
17d1 ad :3 0 ae
:3 0 1799 179a 0
ad :3 0 bd :3 0
179c 179d 0 ed
:2 0 2 :4 0 1a9
:2 0 bf :2 0 46
:2 0 dac 17a2 17a4
:3 0 daf 179e 17a6
179b 17a8 17a7 :2 0
b0 :3 0 c0 :3 0
17aa 17ab 0 b2
:4 0 1aa :4 0 b5
:2 0 b6 :3 0 db3
17af 17b1 :3 0 b5
:2 0 c9 :4 0 db6
17b3 17b5 :3 0 b5
:2 0 f0 :4 0 db9
17b7 17b9 :3 0 b5
:2 0 ea :3 0 dbc
17bb 17bd :3 0 b5
:2 0 f1 :4 0 dbf
17bf 17c1 :3 0 b5
:2 0 f2 :3 0 f3
:3 0 46 :2 0 f4
:2 0 dc2 17c4 17c8
dc6 17c3 17ca :3 0
dc9 17ac 17cc :2 0
17ce dcc 17cf 17a9
17ce 0 17d0 dce
0 17d1 dd0 17d4
:3 0 17d4 0 17d4
17d3 17d1 17d2 :6 0
17d6 20 :3 0 dd3
17d8 dd5 17d7 17d6
:2 0 17d9 dd7 :2 0
17db 0 17db 17da
1788 17d9 :6 0 18ea
1 :3 0 21 :3 0
e9 :3 0 17dd 17de
0 21 :3 0 d2
:2 0 166 :3 0 17e0
17e2 0 ddc 17e1
17e4 :3 0 17e5 :2 0
ad :3 0 ae :3 0
17e7 17e8 0 ad
:3 0 bd :3 0 17ea
17eb 0 24 :2 0
2 :4 0 1ab :2 0
bf :2 0 46 :2 0
ddf 17f0 17f2 :3 0
de2 17ec 17f4 17e9
17f6 17f5 :2 0 b0
:3 0 c0 :3 0 17f8
17f9 0 b2 :4 0
1ac :4 0 b5 :2 0
b6 :3 0 de6 17fd
17ff :3 0 b5 :2 0
c9 :4 0 de9 1801
1803 :3 0 b5 :2 0
ca :4 0 dec 1805
1807 :3 0 b5 :2 0
a1 :3 0 ba :3 0
c2 :3 0 141 :3 0
180c 180d 0 def
180b 180f bb :4 0
df1 180a 1812 df4
1809 1814 :3 0 b5
:2 0 116 :4 0 df7
1816 1818 :3 0 b5
:2 0 b8 :4 0 dfa
181a 181c :3 0 b5
:2 0 48 :3 0 dfd
181e 1820 :3 0 b5
:2 0 b8 :4 0 e00
1822 1824 :3 0 e03
17fa 1826 :2 0 1828
e06 1829 17f7 1828
0 182a e08 0
1854 ad :3 0 ae
:3 0 182b 182c 0
ad :3 0 bd :3 0
182e 182f 0 46
:2 0 2 :4 0 1ad
:2 0 bf :2 0 46
:2 0 e0a 1834 1836
:3 0 e0d 1830 1838
182d 183a 1839 :2 0
b0 :3 0 c0 :3 0
183c 183d 0 b2
:4 0 1ae :4 0 b5
:2 0 b6 :3 0 e11
1841 1843 :3 0 e14
183e 1845 :2 0 1847
e17 1848 183b 1847
0 1849 e19 0
1854 ce :3 0 119
:3 0 184a 184b 0
c2 :3 0 141 :3 0
184d 184e 0 48
:3 0 d0 :3 0 e1b
184c 1852 :2 0 1854
e1f 1855 17e6 1854
0 1856 e23 0
18ea a1 :3 0 4f
:3 0 4d :2 0 e25
1857 185a 105 :2 0
4d :2 0 e2a 185c
185e :3 0 81 :3 0
1af :4 0 1860 1861
0 1863 e2d 1868
81 :3 0 1b0 :4 0
1864 1865 0 1867
e2f 1869 185f 1863
0 186a 0 1867
0 186a e31 0
18ea 190 :3 0 191
:3 0 98 :3 0 a0
:3 0 19e :3 0 1b1
:3 0 10b :3 0 1b2
:3 0 1b3 :3 0 1b4
:3 0 39 :3 0 25
:3 0 48 :3 0 62
:3 0 4a :3 0 81
:3 0 14 :3 0 13
:3 0 12 :4 0 1b5
1 :8 0 1894 1b6
:4 0 187f :3 0 d0
:3 0 d2 :2 0 e36
1882 1883 :3 0 21
:3 0 e9 :3 0 1885
1886 0 4d :2 0
1887 1888 0 188a
e39 1891 21 :3 0
e9 :3 0 188b 188c
0 180 :2 0 188d
188e 0 1890 e3b
1892 1884 188a 0
1893 0 1890 0
1893 e3d 0 1894
e40 18e7 eb :3 0
21 :3 0 e9 :3 0
1896 1897 0 ea
:3 0 1898 1899 0
189b e43 189d e45
189c 189b :2 0 18e5
ec :3 0 21 :3 0
e9 :3 0 18a0 18a1
0 ea :3 0 18a2
18a3 0 18dd ad
:3 0 ae :3 0 18a5
18a6 0 ad :3 0
bd :3 0 18a8 18a9
0 ed :2 0 2
:4 0 1b7 :2 0 bf
:2 0 46 :2 0 e47
18ae 18b0 :3 0 e4a
18aa 18b2 18a7 18b4
18b3 :2 0 b0 :3 0
c0 :3 0 18b6 18b7
0 b2 :4 0 1b8
:4 0 b5 :2 0 b6
:3 0 e4e 18bb 18bd
:3 0 b5 :2 0 c9
:4 0 e51 18bf 18c1
:3 0 b5 :2 0 f0
:4 0 e54 18c3 18c5
:3 0 b5 :2 0 ea
:3 0 e57 18c7 18c9
:3 0 b5 :2 0 f1
:4 0 e5a 18cb 18cd
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 e5d
18d0 18d4 e61 18cf
18d6 :3 0 e64 18b8
18d8 :2 0 18da e67
18db 18b5 18da 0
18dc e69 0 18dd
e6b 18e0 :3 0 18e0
0 18e0 18df 18dd
18de :6 0 18e2 22
:3 0 e6e 18e4 e70
18e3 18e2 :2 0 18e5
e72 :2 0 18e7 0
18e7 18e6 1894 18e5
:6 0 18ea 1 :3 0
d9 :3 0 e75 196c
f2 :3 0 4b :3 0
46 :2 0 46 :2 0
e7b 18eb 18ef d2
:2 0 1b0 :4 0 e81
18f1 18f3 :3 0 190
:3 0 10b :3 0 1b2
:3 0 14 :3 0 1b3
:3 0 13 :3 0 1b4
:3 0 12 :3 0 191
:3 0 39 :4 0 1b9
1 :8 0 1915 1b6
:4 0 1900 :3 0 d0
:3 0 d2 :2 0 e86
1903 1904 :3 0 21
:3 0 e9 :3 0 1906
1907 0 4d :2 0
1908 1909 0 190b
e89 1912 21 :3 0
e9 :3 0 190c 190d
0 180 :2 0 190e
190f 0 1911 e8b
1913 1905 190b 0
1914 0 1911 0
1914 e8d 0 1915
e90 1968 eb :3 0
21 :3 0 e9 :3 0
1917 1918 0 ea
:3 0 1919 191a 0
191c e93 191e e95
191d 191c :2 0 1966
ec :3 0 21 :3 0
e9 :3 0 1921 1922
0 ea :3 0 1923
1924 0 195e ad
:3 0 ae :3 0 1926
1927 0 ad :3 0
bd :3 0 1929 192a
0 ed :2 0 2
:4 0 1ba :2 0 bf
:2 0 46 :2 0 e97
192f 1931 :3 0 e9a
192b 1933 1928 1935
1934 :2 0 b0 :3 0
c0 :3 0 1937 1938
0 b2 :4 0 1bb
:4 0 b5 :2 0 b6
:3 0 e9e 193c 193e
:3 0 b5 :2 0 c9
:4 0 ea1 1940 1942
:3 0 b5 :2 0 f0
:4 0 ea4 1944 1946
:3 0 b5 :2 0 ea
:3 0 ea7 1948 194a
:3 0 b5 :2 0 f1
:4 0 eaa 194c 194e
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 ead
1951 1955 eb1 1950
1957 :3 0 eb4 1939
1959 :2 0 195b eb7
195c 1936 195b 0
195d eb9 0 195e
ebb 1961 :3 0 1961
0 1961 1960 195e
195f :6 0 1963 24
:3 0 ebe 1965 ec0
1964 1963 :2 0 1966
ec2 :2 0 1968 0
1968 1967 1915 1966
:6 0 196a 1 :3 0
ec5 196b 18f4 196a
0 196d 16fe 18ea
0 196d ec7 0
1ac7 1bc :3 0 195
:3 0 191 :3 0 190
:3 0 98 :3 0 25
:3 0 10b :3 0 191
:3 0 39 :4 0 1bd
1 :8 0 198d 1b6
:4 0 1978 :3 0 d0
:3 0 d2 :2 0 ecc
197b 197c :3 0 21
:3 0 e9 :3 0 197e
197f 0 4d :2 0
1980 1981 0 1983
ecf 198a 21 :3 0
e9 :3 0 1984 1985
0 180 :2 0 1986
1987 0 1989 ed1
198b 197d 1983 0
198c 0 1989 0
198c ed3 0 198d
ed6 19e0 eb :3 0
21 :3 0 e9 :3 0
198f 1990 0 ea
:3 0 1991 1992 0
1994 ed9 1996 edb
1995 1994 :2 0 19de
ec :3 0 21 :3 0
e9 :3 0 1999 199a
0 ea :3 0 199b
199c 0 19d6 ad
:3 0 ae :3 0 199e
199f 0 ad :3 0
bd :3 0 19a1 19a2
0 ed :2 0 2
:4 0 1be :2 0 bf
:2 0 46 :2 0 edd
19a7 19a9 :3 0 ee0
19a3 19ab 19a0 19ad
19ac :2 0 b0 :3 0
c0 :3 0 19af 19b0
0 b2 :4 0 1bf
:4 0 b5 :2 0 b6
:3 0 ee4 19b4 19b6
:3 0 b5 :2 0 c9
:4 0 ee7 19b8 19ba
:3 0 b5 :2 0 f0
:4 0 eea 19bc 19be
:3 0 b5 :2 0 ea
:3 0 eed 19c0 19c2
:3 0 b5 :2 0 f1
:4 0 ef0 19c4 19c6
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 ef3
19c9 19cd ef7 19c8
19cf :3 0 efa 19b1
19d1 :2 0 19d3 efd
19d4 19ae 19d3 0
19d5 eff 0 19d6
f01 19d9 :3 0 19d9
0 19d9 19d8 19d6
19d7 :6 0 19db 26
:3 0 f04 19dd f06
19dc 19db :2 0 19de
f08 :2 0 19e0 0
19e0 19df 198d 19de
:6 0 1ac7 1 :3 0
1c0 :3 0 195 :3 0
191 :3 0 190 :3 0
98 :3 0 25 :3 0
10b :3 0 191 :3 0
39 :4 0 1c1 1
:8 0 1a01 1b6 :4 0
19ec :3 0 d0 :3 0
d2 :2 0 f0d 19ef
19f0 :3 0 21 :3 0
e9 :3 0 19f2 19f3
0 4d :2 0 19f4
19f5 0 19f7 f10
19fe 21 :3 0 e9
:3 0 19f8 19f9 0
180 :2 0 19fa 19fb
0 19fd f12 19ff
19f1 19f7 0 1a00
0 19fd 0 1a00
f14 0 1a01 f17
1a54 eb :3 0 21
:3 0 e9 :3 0 1a03
1a04 0 ea :3 0
1a05 1a06 0 1a08
f1a 1a0a f1c 1a09
1a08 :2 0 1a52 ec
:3 0 21 :3 0 e9
:3 0 1a0d 1a0e 0
ea :3 0 1a0f 1a10
0 1a4a ad :3 0
ae :3 0 1a12 1a13
0 ad :3 0 bd
:3 0 1a15 1a16 0
ed :2 0 2 :4 0
1c2 :2 0 bf :2 0
46 :2 0 f1e 1a1b
1a1d :3 0 f21 1a17
1a1f 1a14 1a21 1a20
:2 0 b0 :3 0 c0
:3 0 1a23 1a24 0
b2 :4 0 1c3 :4 0
b5 :2 0 b6 :3 0
f25 1a28 1a2a :3 0
b5 :2 0 c9 :4 0
f28 1a2c 1a2e :3 0
b5 :2 0 f0 :4 0
f2b 1a30 1a32 :3 0
b5 :2 0 ea :3 0
f2e 1a34 1a36 :3 0
b5 :2 0 f1 :4 0
f31 1a38 1a3a :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 f34 1a3d
1a41 f38 1a3c 1a43
:3 0 f3b 1a25 1a45
:2 0 1a47 f3e 1a48
1a22 1a47 0 1a49
f40 0 1a4a f42
1a4d :3 0 1a4d 0
1a4d 1a4c 1a4a 1a4b
:6 0 1a4f 28 :3 0
f45 1a51 f47 1a50
1a4f :2 0 1a52 f49
:2 0 1a54 0 1a54
1a53 1a01 1a52 :6 0
1ac7 1 :3 0 190
:3 0 98 :3 0 25
:3 0 10b :3 0 191
:3 0 39 :4 0 1c4
1 :8 0 1a72 1b6
:4 0 1a5d :3 0 d0
:3 0 d2 :2 0 f4e
1a60 1a61 :3 0 21
:3 0 e9 :3 0 1a63
1a64 0 4d :2 0
1a65 1a66 0 1a68
f51 1a6f 21 :3 0
e9 :3 0 1a69 1a6a
0 180 :2 0 1a6b
1a6c 0 1a6e f53
1a70 1a62 1a68 0
1a71 0 1a6e 0
1a71 f55 0 1a72
f58 1ac5 eb :3 0
21 :3 0 e9 :3 0
1a74 1a75 0 ea
:3 0 1a76 1a77 0
1a79 f5b 1a7b f5d
1a7a 1a79 :2 0 1ac3
ec :3 0 21 :3 0
e9 :3 0 1a7e 1a7f
0 ea :3 0 1a80
1a81 0 1abb ad
:3 0 ae :3 0 1a83
1a84 0 ad :3 0
bd :3 0 1a86 1a87
0 ed :2 0 2
:4 0 1c5 :2 0 bf
:2 0 46 :2 0 f5f
1a8c 1a8e :3 0 f62
1a88 1a90 1a85 1a92
1a91 :2 0 b0 :3 0
c0 :3 0 1a94 1a95
0 b2 :4 0 1c6
:4 0 b5 :2 0 b6
:3 0 f66 1a99 1a9b
:3 0 b5 :2 0 c9
:4 0 f69 1a9d 1a9f
:3 0 b5 :2 0 f0
:4 0 f6c 1aa1 1aa3
:3 0 b5 :2 0 ea
:3 0 f6f 1aa5 1aa7
:3 0 b5 :2 0 f1
:4 0 f72 1aa9 1aab
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 f75
1aae 1ab2 f79 1aad
1ab4 :3 0 f7c 1a96
1ab6 :2 0 1ab8 f7f
1ab9 1a93 1ab8 0
1aba f81 0 1abb
f83 1abe :3 0 1abe
0 1abe 1abd 1abb
1abc :6 0 1ac0 2a
:3 0 f86 1ac2 f88
1ac1 1ac0 :2 0 1ac3
f8a :2 0 1ac5 0
1ac5 1ac4 1a72 1ac3
:6 0 1ac7 1 :3 0
f8d 1ac8 15db 1ac7
0 1ac9 f98 0
35c4 84 :3 0 1c7
:4 0 1acb 1acc :3 0
1aca 1acd 0 35c4
a1 :3 0 4f :3 0
4d :2 0 f9a 1acf
1ad2 d2 :2 0 4d
:2 0 f9f 1ad4 1ad6
:3 0 a1 :3 0 5b
:3 0 4d :2 0 fa2
1ad8 1adb c2 :3 0
c3 :2 0 d3 :3 0
1add 1adf 0 fa7
1ade 1ae1 :3 0 1ad7
1ae3 1ae2 :2 0 a1
:3 0 5b :3 0 4d
:2 0 faa 1ae5 1ae8
c2 :3 0 c3 :2 0
d1 :3 0 1aea 1aec
0 faf 1aeb 1aee
:3 0 1ae4 1af0 1aef
:2 0 1e :3 0 e4
:2 0 fb2 1af3 1af4
:3 0 1af5 :2 0 3a
:3 0 1e :3 0 1af7
1af8 0 1afa fb4
1afb 1af6 1afa 0
1afc fb6 0 1afd
fb8 1afe 1af1 1afd
0 1aff fba 0
35c4 a1 :3 0 55
:3 0 4d :2 0 fbc
1b00 1b03 d2 :2 0
4d :2 0 fc1 1b05
1b07 :3 0 55 :3 0
c2 :3 0 1c8 :3 0
1b0a 1b0b 0 1b09
1b0c 0 1b0e fc4
1b0f 1b08 1b0e 0
1b10 fc6 0 35c4
16 :3 0 c5 :2 0
fc8 1b12 1b13 :3 0
1b14 :2 0 17 :3 0
c5 :2 0 fca 1b17
1b18 :3 0 1b19 :2 0
1b15 1b1b 1b1a :2 0
18 :3 0 c5 :2 0
fcc 1b1e 1b1f :3 0
1b20 :2 0 1b1c 1b22
1b21 :2 0 19 :3 0
c5 :2 0 fce 1b25
1b26 :3 0 1b27 :2 0
1b28 :2 0 1b23 1b2a
1b29 :2 0 8b :3 0
46 :2 0 1b2c 1b2d
0 1b2f fd0 1b30
1b2b 1b2f 0 1b31
fd2 0 35c4 74
:3 0 182 :4 0 1b32
1b33 0 35c4 76
:3 0 182 :4 0 1b35
1b36 0 35c4 a1
:3 0 4f :3 0 4d
:2 0 fd4 1b38 1b3b
105 :2 0 4d :2 0
fd9 1b3d 1b3f :3 0
a1 :3 0 5b :3 0
4d :2 0 fdc 1b41
1b44 c2 :3 0 d2
:2 0 d3 :3 0 1b46
1b48 0 fe1 1b47
1b4a :3 0 74 :3 0
1c9 :4 0 b5 :2 0
39 :3 0 fe4 1b4e
1b50 :3 0 b5 :2 0
1ca :4 0 fe7 1b52
1b54 :3 0 b5 :2 0
1cb :4 0 fea 1b56
1b58 :3 0 b5 :2 0
1cc :4 0 fed 1b5a
1b5c :3 0 b5 :2 0
1cd :4 0 ff0 1b5e
1b60 :3 0 b5 :2 0
48 :3 0 ff3 1b62
1b64 :3 0 b5 :2 0
b8 :4 0 ff6 1b66
1b68 :3 0 1b4c 1b69
0 1b6c d9 :3 0
ff9 1bb3 a1 :3 0
5b :3 0 4d :2 0
ffb 1b6d 1b70 d2
:2 0 2d :2 0 1000
1b72 1b74 :3 0 74
:3 0 1ce :4 0 b5
:2 0 1cf :4 0 1003
1b78 1b7a :3 0 b5
:2 0 1d0 :4 0 1006
1b7c 1b7e :3 0 b5
:2 0 25 :3 0 1009
1b80 1b82 :3 0 b5
:2 0 1d1 :4 0 100c
1b84 1b86 :3 0 b5
:2 0 1d2 :4 0 100f
1b88 1b8a :3 0 b5
:2 0 39 :3 0 1012
1b8c 1b8e :3 0 b5
:2 0 1d1 :4 0 1015
1b90 1b92 :3 0 b5
:2 0 1d3 :4 0 1018
1b94 1b96 :3 0 1b76
1b97 0 1b99 101b
1b9a 1b75 1b99 0
1bb5 74 :3 0 1d4
:4 0 b5 :2 0 1d5
:4 0 101d 1b9d 1b9f
:3 0 b5 :2 0 1d6
:4 0 1020 1ba1 1ba3
:3 0 b5 :2 0 1d7
:4 0 1023 1ba5 1ba7
:3 0 b5 :2 0 39
:3 0 1026 1ba9 1bab
:3 0 b5 :2 0 1d1
:4 0 1029 1bad 1baf
:3 0 1b9b 1bb0 0
1bb2 102c 1bb4 1b4b
1b6c 0 1bb5 0
1bb2 0 1bb5 102e
0 1bb6 1032 2524
a1 :3 0 5b :3 0
4d :2 0 1034 1bb7
1bba c2 :3 0 c3
:2 0 d3 :3 0 1bbc
1bbe 0 1039 1bbd
1bc0 :3 0 a1 :3 0
5b :3 0 4d :2 0
103c 1bc2 1bc5 c2
:3 0 c3 :2 0 d1
:3 0 1bc7 1bc9 0
1041 1bc8 1bcb :3 0
1bc1 1bcd 1bcc :2 0
a1 :3 0 a :3 0
4d :2 0 1044 1bcf
1bd2 c2 :3 0 c3
:2 0 c4 :3 0 1bd4
1bd6 0 1049 1bd5
1bd8 :3 0 1bce 1bda
1bd9 :2 0 1d8 :3 0
1d9 :3 0 1da :4 0
1bde :4 0 1bdf :2 0
1cff 21 :3 0 e9
:3 0 1be1 1be2 0
4d :2 0 1be3 1be4
0 1cff 14 :3 0
c5 :2 0 104c 1be7
1be8 :3 0 1be9 :2 0
1d8 :3 0 1d9 :3 0
1db :4 0 1bed :4 0
1bee :2 0 1cf7 21
:3 0 e9 :3 0 1bf0
1bf1 0 4d :2 0
1bf2 1bf3 0 1cf7
1dc :3 0 10d :3 0
1dd :3 0 178 :3 0
1de :3 0 1df :3 0
10d :3 0 1df :3 0
1dd :3 0 178 :3 0
1df :3 0 1e0 :3 0
196 :3 0 1e1 :3 0
1df :3 0 10d :3 0
1e1 :3 0 10d :3 0
1e1 :3 0 1e2 :3 0
1e1 :3 0 9a :3 0
29 :4 0 1e3 1
:8 0 1c22 1b6 :4 0
1c0d :3 0 d0 :3 0
d2 :2 0 1050 1c10
1c11 :3 0 21 :3 0
e9 :3 0 1c13 1c14
0 4d :2 0 1c15
1c16 0 1c18 1053
1c1f 21 :3 0 e9
:3 0 1c19 1c1a 0
180 :2 0 1c1b 1c1c
0 1c1e 1055 1c20
1c12 1c18 0 1c21
0 1c1e 0 1c21
1057 0 1c22 105a
1c75 eb :3 0 21
:3 0 e9 :3 0 1c24
1c25 0 ea :3 0
1c26 1c27 0 1c29
105d 1c2b 105f 1c2a
1c29 :2 0 1c73 ec
:3 0 21 :3 0 e9
:3 0 1c2e 1c2f 0
ea :3 0 1c30 1c31
0 1c6b ad :3 0
ae :3 0 1c33 1c34
0 ad :3 0 bd
:3 0 1c36 1c37 0
ed :2 0 2 :4 0
1e4 :2 0 bf :2 0
46 :2 0 1061 1c3c
1c3e :3 0 1064 1c38
1c40 1c35 1c42 1c41
:2 0 b0 :3 0 c0
:3 0 1c44 1c45 0
b2 :4 0 1e5 :4 0
b5 :2 0 b6 :3 0
1068 1c49 1c4b :3 0
b5 :2 0 c9 :4 0
106b 1c4d 1c4f :3 0
b5 :2 0 f0 :4 0
106e 1c51 1c53 :3 0
b5 :2 0 ea :3 0
1071 1c55 1c57 :3 0
b5 :2 0 f1 :4 0
1074 1c59 1c5b :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 1077 1c5e
1c62 107b 1c5d 1c64
:3 0 107e 1c46 1c66
:2 0 1c68 1081 1c69
1c43 1c68 0 1c6a
1083 0 1c6b 1085
1c6e :3 0 1c6e 0
1c6e 1c6d 1c6b 1c6c
:6 0 1c70 2c :3 0
1088 1c72 108a 1c71
1c70 :2 0 1c73 108c
:2 0 1c75 0 1c75
1c74 1c22 1c73 :6 0
1cf7 1 :3 0 1e6
:3 0 1e7 :3 0 1dd
:3 0 1dc :3 0 1e7
:3 0 139 :3 0 10d
:3 0 1dc :3 0 139
:3 0 139 :3 0 1dd
:3 0 1e7 :3 0 1dd
:3 0 f2 :3 0 1e7
:3 0 1dd :3 0 1e8
:3 0 139 :3 0 1dd
:3 0 139 :3 0 1dd
:4 0 1e9 1 :8 0
1ca2 1b6 :4 0 1c8d
:3 0 d0 :3 0 d2
:2 0 1091 1c90 1c91
:3 0 21 :3 0 e9
:3 0 1c93 1c94 0
4d :2 0 1c95 1c96
0 1c98 1094 1c9f
21 :3 0 e9 :3 0
1c99 1c9a 0 180
:2 0 1c9b 1c9c 0
1c9e 1096 1ca0 1c92
1c98 0 1ca1 0
1c9e 0 1ca1 1098
0 1ca2 109b 1cf5
eb :3 0 21 :3 0
e9 :3 0 1ca4 1ca5
0 ea :3 0 1ca6
1ca7 0 1ca9 109e
1cab 10a0 1caa 1ca9
:2 0 1cf3 ec :3 0
21 :3 0 e9 :3 0
1cae 1caf 0 ea
:3 0 1cb0 1cb1 0
1ceb ad :3 0 ae
:3 0 1cb3 1cb4 0
ad :3 0 bd :3 0
1cb6 1cb7 0 ed
:2 0 2 :4 0 1ea
:2 0 bf :2 0 46
:2 0 10a2 1cbc 1cbe
:3 0 10a5 1cb8 1cc0
1cb5 1cc2 1cc1 :2 0
b0 :3 0 c0 :3 0
1cc4 1cc5 0 b2
:4 0 1eb :4 0 b5
:2 0 b6 :3 0 10a9
1cc9 1ccb :3 0 b5
:2 0 c9 :4 0 10ac
1ccd 1ccf :3 0 b5
:2 0 f0 :4 0 10af
1cd1 1cd3 :3 0 b5
:2 0 ea :3 0 10b2
1cd5 1cd7 :3 0 b5
:2 0 f1 :4 0 10b5
1cd9 1cdb :3 0 b5
:2 0 f2 :3 0 f3
:3 0 46 :2 0 f4
:2 0 10b8 1cde 1ce2
10bc 1cdd 1ce4 :3 0
10bf 1cc6 1ce6 :2 0
1ce8 10c2 1ce9 1cc3
1ce8 0 1cea 10c4
0 1ceb 10c6 1cee
:3 0 1cee 0 1cee
1ced 1ceb 1cec :6 0
1cf0 2e :3 0 10c9
1cf2 10cb 1cf1 1cf0
:2 0 1cf3 10cd :2 0
1cf5 0 1cf5 1cf4
1ca2 1cf3 :6 0 1cf7
1 :3 0 10d0 1cfc
39 :3 0 14 :3 0
1cf8 1cf9 0 1cfb
10d5 1cfd 1bea 1cf7
0 1cfe 0 1cfb
0 1cfe 10d7 0
1cff 10da 1d00 1bdb
1cff 0 1d01 10de
0 2523 10 :3 0
e4 :2 0 10e0 1d03
1d04 :3 0 1d05 :2 0
8e :3 0 a1 :3 0
1e8 :3 0 10 :3 0
10e2 1d09 1d0b 4d
:2 0 10e4 1d08 1d0e
105 :2 0 1ec :2 0
10e9 1d10 1d12 :3 0
101 :3 0 a1 :3 0
1e8 :3 0 10 :3 0
10ec 1d16 1d18 4d
:2 0 10ee 1d15 1d1b
1ed :2 0 10f1 1d14
1d1e 1ee :2 0 1ef
:2 0 10f4 1d20 1d22
:3 0 10f7 1d24 1d23
101 :3 0 a1 :3 0
1e8 :3 0 10 :3 0
10f9 1d28 1d2a 4d
:2 0 10fb 1d27 1d2d
1ed :2 0 10fe 1d26
1d30 0 1d31 0
1101 :2 0 1d07 1d33
0 1e3e 8f :3 0
4d :2 0 1d35 1d36
0 1e3e 1f0 :3 0
8f :3 0 8e :3 0
f6 :2 0 1f1 :3 0
1106 1d3b 1d3d :3 0
1d3e 1d7a a1 :3 0
1f2 :3 0 f2 :3 0
2a :3 0 8f :3 0
bf :2 0 46 :2 0
1109 1d45 1d47 :3 0
46 :2 0 110c 1d42
1d4a 1110 1d41 1d4c
4d :2 0 1112 1d40
1d4f c3 :2 0 1f3
:2 0 1117 1d51 1d53
:3 0 a1 :3 0 1f2
:3 0 f2 :3 0 2a
:3 0 8f :3 0 bf
:2 0 46 :2 0 111a
1d5a 1d5c :3 0 46
:2 0 111d 1d57 1d5f
1121 1d56 1d61 4d
:2 0 1123 1d55 1d64
c3 :2 0 1f4 :2 0
1128 1d66 1d68 :3 0
1d54 1d6a 1d69 :2 0
1f5 :8 0 1d6e 112b
1d6f 1d6b 1d6e 0
1d70 112d 0 1d78
8f :3 0 8f :3 0
bf :2 0 46 :2 0
112f 1d73 1d75 :3 0
1d71 1d76 0 1d78
1132 1d7a 1f1 :3 0
1d3f 1d78 :4 0 1e3e
8f :3 0 8e :3 0
d2 :2 0 1137 1d7d
1d7e :3 0 ad :3 0
ae :3 0 1d80 1d81
0 ad :3 0 bd
:3 0 1d83 1d84 0
c6 :2 0 2 :4 0
1f6 :2 0 bf :2 0
46 :2 0 113a 1d89
1d8b :3 0 113d 1d85
1d8d 1d82 1d8f 1d8e
:2 0 b0 :3 0 c0
:3 0 1d91 1d92 0
b2 :4 0 1f7 :4 0
b5 :2 0 b6 :3 0
1141 1d96 1d98 :3 0
b5 :2 0 c9 :4 0
1144 1d9a 1d9c :3 0
b5 :2 0 ca :4 0
1147 1d9e 1da0 :3 0
b5 :2 0 a1 :3 0
ba :3 0 c2 :3 0
d6 :3 0 1da5 1da6
0 114a 1da4 1da8
bb :4 0 114c 1da3
1dab 114f 1da2 1dad
:3 0 1152 1d93 1daf
:2 0 1db1 1155 1db2
1d90 1db1 0 1db3
1157 0 1ddc ad
:3 0 ae :3 0 1db4
1db5 0 ad :3 0
bd :3 0 1db7 1db8
0 46 :2 0 2
:4 0 1f8 :2 0 bf
:2 0 46 :2 0 1159
1dbd 1dbf :3 0 115c
1db9 1dc1 1db6 1dc3
1dc2 :2 0 b0 :3 0
c0 :3 0 1dc5 1dc6
0 b2 :4 0 1f9
:4 0 b5 :2 0 b6
:3 0 1160 1dca 1dcc
:3 0 1163 1dc7 1dce
:2 0 1dd0 1166 1dd1
1dc4 1dd0 0 1dd2
1168 0 1ddc ce
:3 0 cf :3 0 1dd3
1dd4 0 c2 :3 0
d6 :3 0 1dd6 1dd7
0 d0 :3 0 116a
1dd5 1dda :2 0 1ddc
116d 1ddd 1d7f 1ddc
0 1dde 1171 0
1e3e 1fa :3 0 2a
:3 0 2a :3 0 e7
:4 0 1fb 1 :8 0
1de9 21 :3 0 e9
:3 0 1de4 1de5 0
ea :3 0 1de6 1de7
0 1de9 1173 1e3c
eb :3 0 21 :3 0
e9 :3 0 1deb 1dec
0 ea :3 0 1ded
1dee 0 1df0 1176
1df2 1178 1df1 1df0
:2 0 1e3a ec :3 0
21 :3 0 e9 :3 0
1df5 1df6 0 ea
:3 0 1df7 1df8 0
1e32 ad :3 0 ae
:3 0 1dfa 1dfb 0
ad :3 0 bd :3 0
1dfd 1dfe 0 ed
:2 0 2 :4 0 1fc
:2 0 bf :2 0 46
:2 0 117a 1e03 1e05
:3 0 117d 1dff 1e07
1dfc 1e09 1e08 :2 0
b0 :3 0 c0 :3 0
1e0b 1e0c 0 b2
:4 0 1fd :4 0 b5
:2 0 b6 :3 0 1181
1e10 1e12 :3 0 b5
:2 0 c9 :4 0 1184
1e14 1e16 :3 0 b5
:2 0 f0 :4 0 1187
1e18 1e1a :3 0 b5
:2 0 ea :3 0 118a
1e1c 1e1e :3 0 b5
:2 0 f1 :4 0 118d
1e20 1e22 :3 0 b5
:2 0 f2 :3 0 f3
:3 0 46 :2 0 f4
:2 0 1190 1e25 1e29
1194 1e24 1e2b :3 0
1197 1e0d 1e2d :2 0
1e2f 119a 1e30 1e0a
1e2f 0 1e31 119c
0 1e32 119e 1e35
:3 0 1e35 0 1e35
1e34 1e32 1e33 :6 0
1e37 31 :3 0 11a1
1e39 11a3 1e38 1e37
:2 0 1e3a 11a5 :2 0
1e3c 0 1e3c 1e3b
1de9 1e3a :6 0 1e3e
1 :3 0 11a8 1e3f
1d06 1e3e 0 1e40
11ae 0 2523 a1
:3 0 5b :3 0 4d
:2 0 11b0 1e41 1e44
c2 :3 0 d2 :2 0
d3 :3 0 1e46 1e48
0 11b5 1e47 1e4a
:3 0 74 :3 0 1fe
:4 0 1e4c 1e4d 0
1e61 76 :3 0 1ff
:4 0 1e4f 1e50 0
1e61 76 :3 0 76
:3 0 b5 :2 0 39
:3 0 11b8 1e54 1e56
:3 0 1e52 1e57 0
1e61 76 :3 0 76
:3 0 b5 :2 0 b8
:4 0 11bb 1e5b 1e5d
:3 0 1e59 1e5e 0
1e61 d9 :3 0 11be
1f4c a1 :3 0 5b
:3 0 4d :2 0 11c3
1e62 1e65 c2 :3 0
d2 :2 0 d1 :3 0
1e67 1e69 0 11c8
1e68 1e6b :3 0 74
:3 0 200 :4 0 1e6d
1e6e 0 1e81 74
:3 0 74 :3 0 b5
:2 0 25 :3 0 11cb
1e72 1e74 :3 0 1e70
1e75 0 1e81 74
:3 0 74 :3 0 b5
:2 0 201 :4 0 11ce
1e79 1e7b :3 0 1e77
1e7c 0 1e81 76
:3 0 202 :4 0 1e7e
1e7f 0 1e81 11d1
1e82 1e6c 1e81 0
1f4e a1 :3 0 a
:3 0 4d :2 0 11d6
1e83 1e86 c2 :3 0
d2 :2 0 c4 :3 0
1e88 1e8a 0 11db
1e89 1e8c :3 0 74
:3 0 203 :4 0 1e8e
1e8f 0 1ed5 76
:3 0 202 :4 0 1e91
1e92 0 1ed5 1e
:3 0 e4 :2 0 11de
1e95 1e96 :3 0 1e97
:2 0 76 :3 0 76
:3 0 b5 :2 0 204
:4 0 11e0 1e9b 1e9d
:3 0 1e99 1e9e 0
1eae 76 :3 0 76
:3 0 b5 :2 0 3a
:3 0 11e3 1ea2 1ea4
:3 0 1ea0 1ea5 0
1eae 76 :3 0 76
:3 0 b5 :2 0 1d1
:4 0 11e6 1ea9 1eab
:3 0 1ea7 1eac 0
1eae 11e9 1eaf 1e98
1eae 0 1eb0 11ed
0 1ed5 76 :3 0
76 :3 0 b5 :2 0
205 :4 0 11ef 1eb3
1eb5 :3 0 1eb1 1eb6
0 1ed5 10 :3 0
e4 :2 0 11f2 1eb9
1eba :3 0 1ebb :2 0
76 :3 0 76 :3 0
b5 :2 0 206 :4 0
11f4 1ebf 1ec1 :3 0
1ebd 1ec2 0 1ed2
76 :3 0 76 :3 0
b5 :2 0 2a :3 0
11f7 1ec6 1ec8 :3 0
1ec4 1ec9 0 1ed2
76 :3 0 76 :3 0
b5 :2 0 207 :4 0
11fa 1ecd 1ecf :3 0
1ecb 1ed0 0 1ed2
11fd 1ed3 1ebc 1ed2
0 1ed4 1201 0
1ed5 1203 1f48 14
:3 0 e4 :2 0 1209
1ed7 1ed8 :3 0 1ed9
:2 0 74 :3 0 74
:3 0 b5 :2 0 208
:4 0 120b 1edd 1edf
:3 0 1edb 1ee0 0
1ee5 76 :3 0 202
:4 0 1ee2 1ee3 0
1ee5 120e 1f2d 74
:3 0 74 :3 0 b5
:2 0 209 :4 0 1211
1ee8 1eea :3 0 b5
:2 0 20a :4 0 1214
1eec 1eee :3 0 1ee6
1eef 0 1f2c 76
:3 0 202 :4 0 1ef1
1ef2 0 1f2c 1e
:3 0 e4 :2 0 1217
1ef5 1ef6 :3 0 1ef7
:2 0 76 :3 0 76
:3 0 b5 :2 0 204
:4 0 1219 1efb 1efd
:3 0 1ef9 1efe 0
1f0e 76 :3 0 76
:3 0 b5 :2 0 3a
:3 0 121c 1f02 1f04
:3 0 1f00 1f05 0
1f0e 76 :3 0 76
:3 0 b5 :2 0 1d1
:4 0 121f 1f09 1f0b
:3 0 1f07 1f0c 0
1f0e 1222 1f0f 1ef8
1f0e 0 1f10 1226
0 1f2c 76 :3 0
76 :3 0 b5 :2 0
20b :4 0 1228 1f13
1f15 :3 0 b5 :2 0
20c :4 0 122b 1f17
1f19 :3 0 b5 :2 0
20d :4 0 122e 1f1b
1f1d :3 0 b5 :2 0
20e :4 0 1231 1f1f
1f21 :3 0 b5 :2 0
20f :4 0 1234 1f23
1f25 :3 0 b5 :2 0
210 :4 0 1237 1f27
1f29 :3 0 1f11 1f2a
0 1f2c 123a 1f2e
1eda 1ee5 0 1f2f
0 1f2c 0 1f2f
123f 0 1f47 10
:3 0 e4 :2 0 1242
1f31 1f32 :3 0 1f33
:2 0 76 :3 0 76
:3 0 b5 :2 0 211
:4 0 1244 1f37 1f39
:3 0 b5 :2 0 2a
:3 0 1247 1f3b 1f3d
:3 0 b5 :2 0 207
:4 0 124a 1f3f 1f41
:3 0 1f35 1f42 0
1f44 124d 1f45 1f34
1f44 0 1f46 124f
0 1f47 1251 1f49
1e8d 1ed5 0 1f4a
0 1f47 0 1f4a
1254 0 1f4b 1257
1f4d 1e4b 1e61 0
1f4e 0 1f4b 0
1f4e 1259 0 2523
a1 :3 0 5b :3 0
4d :2 0 125d 1f4f
1f52 c2 :3 0 c3
:2 0 d3 :3 0 1f54
1f56 0 1262 1f55
1f58 :3 0 14 :3 0
e4 :2 0 1265 1f5b
1f5c :3 0 1f5d :2 0
1dd :3 0 3b :3 0
178 :3 0 10d :3 0
14 :4 0 212 1
:8 0 1f6a 21 :3 0
e9 :3 0 1f65 1f66
0 ea :3 0 1f67
1f68 0 1f6a 1267
1fbd eb :3 0 21
:3 0 e9 :3 0 1f6c
1f6d 0 ea :3 0
1f6e 1f6f 0 1f71
126a 1f73 126c 1f72
1f71 :2 0 1fbb ec
:3 0 21 :3 0 e9
:3 0 1f76 1f77 0
ea :3 0 1f78 1f79
0 1fb3 ad :3 0
ae :3 0 1f7b 1f7c
0 ad :3 0 bd
:3 0 1f7e 1f7f 0
ed :2 0 2 :4 0
213 :2 0 bf :2 0
46 :2 0 126e 1f84
1f86 :3 0 1271 1f80
1f88 1f7d 1f8a 1f89
:2 0 b0 :3 0 c0
:3 0 1f8c 1f8d 0
b2 :4 0 214 :4 0
b5 :2 0 b6 :3 0
1275 1f91 1f93 :3 0
b5 :2 0 c9 :4 0
1278 1f95 1f97 :3 0
b5 :2 0 f0 :4 0
127b 1f99 1f9b :3 0
b5 :2 0 ea :3 0
127e 1f9d 1f9f :3 0
b5 :2 0 f1 :4 0
1281 1fa1 1fa3 :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 1284 1fa6
1faa 1288 1fa5 1fac
:3 0 128b 1f8e 1fae
:2 0 1fb0 128e 1fb1
1f8b 1fb0 0 1fb2
1290 0 1fb3 1292
1fb6 :3 0 1fb6 0
1fb6 1fb5 1fb3 1fb4
:6 0 1fb8 33 :3 0
1295 1fba 1297 1fb9
1fb8 :2 0 1fbb 1299
:2 0 1fbd 0 1fbd
1fbc 1f6a 1fbb :6 0
205f 1 :3 0 21
:3 0 e9 :3 0 1fbf
1fc0 0 21 :3 0
d2 :2 0 f5 :3 0
1fc2 1fc4 0 129e
1fc3 1fc6 :3 0 1fc7
:2 0 90 :3 0 a1
:3 0 1e8 :3 0 3b
:3 0 12a1 1fcb 1fcd
4d :2 0 12a3 1fca
1fd0 1fc9 1fd1 0
1fed 74 :3 0 74
:3 0 b5 :2 0 215
:4 0 12a6 1fd5 1fd7
:3 0 1fd3 1fd8 0
1fed 76 :3 0 76
:3 0 b5 :2 0 216
:4 0 12a9 1fdc 1fde
:3 0 b5 :2 0 217
:4 0 12ac 1fe0 1fe2
:3 0 b5 :2 0 3b
:3 0 12af 1fe4 1fe6
:3 0 b5 :2 0 218
:4 0 12b2 1fe8 1fea
:3 0 1fda 1feb 0
1fed 12b5 205c ad
:3 0 ae :3 0 1fee
1fef 0 ad :3 0
bd :3 0 1ff1 1ff2
0 24 :2 0 2
:4 0 219 :2 0 bf
:2 0 46 :2 0 12b9
1ff7 1ff9 :3 0 12bc
1ff3 1ffb 1ff0 1ffd
1ffc :2 0 b0 :3 0
c0 :3 0 1fff 2000
0 b2 :4 0 21a
:4 0 b5 :2 0 b6
:3 0 12c0 2004 2006
:3 0 b5 :2 0 c9
:4 0 12c3 2008 200a
:3 0 b5 :2 0 ca
:4 0 12c6 200c 200e
:3 0 b5 :2 0 a1
:3 0 ba :3 0 c2
:3 0 d6 :3 0 2013
2014 0 12c9 2012
2016 bb :4 0 12cb
2011 2019 12ce 2010
201b :3 0 b5 :2 0
116 :4 0 12d1 201d
201f :3 0 b5 :2 0
b8 :4 0 12d4 2021
2023 :3 0 b5 :2 0
14 :3 0 12d7 2025
2027 :3 0 b5 :2 0
b8 :4 0 12da 2029
202b :3 0 12dd 2001
202d :2 0 202f 12e0
2030 1ffe 202f 0
2031 12e2 0 205b
ad :3 0 ae :3 0
2032 2033 0 ad
:3 0 bd :3 0 2035
2036 0 46 :2 0
2 :4 0 21b :2 0
bf :2 0 46 :2 0
12e4 203b 203d :3 0
12e7 2037 203f 2034
2041 2040 :2 0 b0
:3 0 c0 :3 0 2043
2044 0 b2 :4 0
21c :4 0 b5 :2 0
b6 :3 0 12eb 2048
204a :3 0 12ee 2045
204c :2 0 204e 12f1
204f 2042 204e 0
2050 12f3 0 205b
ce :3 0 119 :3 0
2051 2052 0 c2
:3 0 d6 :3 0 2054
2055 0 14 :3 0
d0 :3 0 12f5 2053
2059 :2 0 205b 12f9
205d 1fc8 1fed 0
205e 0 205b 0
205e 12fd 0 205f
1300 2060 1f5e 205f
0 2061 1303 0
228b 12 :3 0 e4
:2 0 1305 2063 2064
:3 0 2065 :2 0 1dd
:3 0 3b :3 0 21d
:3 0 10d :3 0 12
:4 0 21e 1 :8 0
2072 21 :3 0 e9
:3 0 206d 206e 0
ea :3 0 206f 2070
0 2072 1307 20c5
eb :3 0 21 :3 0
e9 :3 0 2074 2075
0 ea :3 0 2076
2077 0 2079 130a
207b 130c 207a 2079
:2 0 20c3 ec :3 0
21 :3 0 e9 :3 0
207e 207f 0 ea
:3 0 2080 2081 0
20bb ad :3 0 ae
:3 0 2083 2084 0
ad :3 0 bd :3 0
2086 2087 0 ed
:2 0 2 :4 0 21f
:2 0 bf :2 0 46
:2 0 130e 208c 208e
:3 0 1311 2088 2090
2085 2092 2091 :2 0
b0 :3 0 c0 :3 0
2094 2095 0 b2
:4 0 220 :4 0 b5
:2 0 b6 :3 0 1315
2099 209b :3 0 b5
:2 0 c9 :4 0 1318
209d 209f :3 0 b5
:2 0 f0 :4 0 131b
20a1 20a3 :3 0 b5
:2 0 ea :3 0 131e
20a5 20a7 :3 0 b5
:2 0 f1 :4 0 1321
20a9 20ab :3 0 b5
:2 0 f2 :3 0 f3
:3 0 46 :2 0 f4
:2 0 1324 20ae 20b2
1328 20ad 20b4 :3 0
132b 2096 20b6 :2 0
20b8 132e 20b9 2093
20b8 0 20ba 1330
0 20bb 1332 20be
:3 0 20be 0 20be
20bd 20bb 20bc :6 0
20c0 35 :3 0 1335
20c2 1337 20c1 20c0
:2 0 20c3 1339 :2 0
20c5 0 20c5 20c4
2072 20c3 :6 0 215d
1 :3 0 21 :3 0
e9 :3 0 20c7 20c8
0 21 :3 0 d2
:2 0 f5 :3 0 20ca
20cc 0 133e 20cb
20ce :3 0 20cf :2 0
74 :3 0 74 :3 0
b5 :2 0 221 :4 0
1341 20d3 20d5 :3 0
20d1 20d6 0 20eb
76 :3 0 76 :3 0
b5 :2 0 222 :4 0
1344 20da 20dc :3 0
b5 :2 0 223 :4 0
1347 20de 20e0 :3 0
b5 :2 0 3b :3 0
134a 20e2 20e4 :3 0
b5 :2 0 218 :4 0
134d 20e6 20e8 :3 0
20d8 20e9 0 20eb
1350 215a ad :3 0
ae :3 0 20ec 20ed
0 ad :3 0 bd
:3 0 20ef 20f0 0
24 :2 0 2 :4 0
224 :2 0 bf :2 0
46 :2 0 1353 20f5
20f7 :3 0 1356 20f1
20f9 20ee 20fb 20fa
:2 0 b0 :3 0 c0
:3 0 20fd 20fe 0
b2 :4 0 225 :4 0
b5 :2 0 b6 :3 0
135a 2102 2104 :3 0
b5 :2 0 c9 :4 0
135d 2106 2108 :3 0
b5 :2 0 ca :4 0
1360 210a 210c :3 0
b5 :2 0 a1 :3 0
ba :3 0 c2 :3 0
d6 :3 0 2111 2112
0 1363 2110 2114
bb :4 0 1365 210f
2117 1368 210e 2119
:3 0 b5 :2 0 116
:4 0 136b 211b 211d
:3 0 b5 :2 0 b8
:4 0 136e 211f 2121
:3 0 b5 :2 0 12
:3 0 1371 2123 2125
:3 0 b5 :2 0 b8
:4 0 1374 2127 2129
:3 0 1377 20ff 212b
:2 0 212d 137a 212e
20fc 212d 0 212f
137c 0 2159 ad
:3 0 ae :3 0 2130
2131 0 ad :3 0
bd :3 0 2133 2134
0 46 :2 0 2
:4 0 226 :2 0 bf
:2 0 46 :2 0 137e
2139 213b :3 0 1381
2135 213d 2132 213f
213e :2 0 b0 :3 0
c0 :3 0 2141 2142
0 b2 :4 0 227
:4 0 b5 :2 0 b6
:3 0 1385 2146 2148
:3 0 1388 2143 214a
:2 0 214c 138b 214d
2140 214c 0 214e
138d 0 2159 ce
:3 0 119 :3 0 214f
2150 0 c2 :3 0
d6 :3 0 2152 2153
0 12 :3 0 d0
:3 0 138f 2151 2157
:2 0 2159 1393 215b
20d0 20eb 0 215c
0 2159 0 215c
1397 0 215d 139a
215e 2066 215d 0
215f 139d 0 228b
13 :3 0 e4 :2 0
139f 2161 2162 :3 0
2163 :2 0 1dd :3 0
3b :3 0 228 :3 0
10d :3 0 13 :4 0
229 1 :8 0 2170
21 :3 0 e9 :3 0
216b 216c 0 ea
:3 0 216d 216e 0
2170 13a1 21c3 eb
:3 0 21 :3 0 e9
:3 0 2172 2173 0
ea :3 0 2174 2175
0 2177 13a4 2179
13a6 2178 2177 :2 0
21c1 ec :3 0 21
:3 0 e9 :3 0 217c
217d 0 ea :3 0
217e 217f 0 21b9
ad :3 0 ae :3 0
2181 2182 0 ad
:3 0 bd :3 0 2184
2185 0 ed :2 0
2 :4 0 22a :2 0
bf :2 0 46 :2 0
13a8 218a 218c :3 0
13ab 2186 218e 2183
2190 218f :2 0 b0
:3 0 c0 :3 0 2192
2193 0 b2 :4 0
22b :4 0 b5 :2 0
b6 :3 0 13af 2197
2199 :3 0 b5 :2 0
c9 :4 0 13b2 219b
219d :3 0 b5 :2 0
f0 :4 0 13b5 219f
21a1 :3 0 b5 :2 0
ea :3 0 13b8 21a3
21a5 :3 0 b5 :2 0
f1 :4 0 13bb 21a7
21a9 :3 0 b5 :2 0
f2 :3 0 f3 :3 0
46 :2 0 f4 :2 0
13be 21ac 21b0 13c2
21ab 21b2 :3 0 13c5
2194 21b4 :2 0 21b6
13c8 21b7 2191 21b6
0 21b8 13ca 0
21b9 13cc 21bc :3 0
21bc 0 21bc 21bb
21b9 21ba :6 0 21be
37 :3 0 13cf 21c0
13d1 21bf 21be :2 0
21c1 13d3 :2 0 21c3
0 21c3 21c2 2170
21c1 :6 0 225b 1
:3 0 21 :3 0 e9
:3 0 21c5 21c6 0
21 :3 0 d2 :2 0
f5 :3 0 21c8 21ca
0 13d8 21c9 21cc
:3 0 21cd :2 0 74
:3 0 74 :3 0 b5
:2 0 22c :4 0 13db
21d1 21d3 :3 0 21cf
21d4 0 21e9 76
:3 0 76 :3 0 b5
:2 0 22d :4 0 13de
21d8 21da :3 0 b5
:2 0 22e :4 0 13e1
21dc 21de :3 0 b5
:2 0 3b :3 0 13e4
21e0 21e2 :3 0 b5
:2 0 218 :4 0 13e7
21e4 21e6 :3 0 21d6
21e7 0 21e9 13ea
2258 ad :3 0 ae
:3 0 21ea 21eb 0
ad :3 0 bd :3 0
21ed 21ee 0 24
:2 0 2 :4 0 22f
:2 0 bf :2 0 46
:2 0 13ed 21f3 21f5
:3 0 13f0 21ef 21f7
21ec 21f9 21f8 :2 0
b0 :3 0 c0 :3 0
21fb 21fc 0 b2
:4 0 230 :4 0 b5
:2 0 b6 :3 0 13f4
2200 2202 :3 0 b5
:2 0 c9 :4 0 13f7
2204 2206 :3 0 b5
:2 0 ca :4 0 13fa
2208 220a :3 0 b5
:2 0 a1 :3 0 ba
:3 0 c2 :3 0 d6
:3 0 220f 2210 0
13fd 220e 2212 bb
:4 0 13ff 220d 2215
1402 220c 2217 :3 0
b5 :2 0 116 :4 0
1405 2219 221b :3 0
b5 :2 0 b8 :4 0
1408 221d 221f :3 0
b5 :2 0 13 :3 0
140b 2221 2223 :3 0
b5 :2 0 b8 :4 0
140e 2225 2227 :3 0
1411 21fd 2229 :2 0
222b 1414 222c 21fa
222b 0 222d 1416
0 2257 ad :3 0
ae :3 0 222e 222f
0 ad :3 0 bd
:3 0 2231 2232 0
46 :2 0 2 :4 0
231 :2 0 bf :2 0
46 :2 0 1418 2237
2239 :3 0 141b 2233
223b 2230 223d 223c
:2 0 b0 :3 0 c0
:3 0 223f 2240 0
b2 :4 0 232 :4 0
b5 :2 0 b6 :3 0
141f 2244 2246 :3 0
1422 2241 2248 :2 0
224a 1425 224b 223e
224a 0 224c 1427
0 2257 ce :3 0
119 :3 0 224d 224e
0 c2 :3 0 d6
:3 0 2250 2251 0
13 :3 0 d0 :3 0
1429 224f 2255 :2 0
2257 142d 2259 21ce
21e9 0 225a 0
2257 0 225a 1431
0 225b 1434 225c
2164 225b 0 225d
1437 0 228b a1
:3 0 53 :3 0 4d
:2 0 1439 225e 2261
105 :2 0 4d :2 0
143e 2263 2265 :3 0
a1 :3 0 53 :3 0
4d :2 0 1441 2267
226a c3 :2 0 e3
:2 0 1446 226c 226e
:3 0 2266 2270 226f
:2 0 76 :3 0 76
:3 0 b5 :2 0 233
:4 0 1449 2274 2276
:3 0 b5 :2 0 ba
:3 0 a1 :3 0 15
:3 0 4d :2 0 144c
227a 227d 144f 2279
227f 1451 2278 2281
:3 0 b5 :2 0 c9
:4 0 1454 2283 2285
:3 0 2272 2286 0
2288 1457 2289 2271
2288 0 228a 1459
0 228b 145b 228c
1f59 228b 0 228d
1460 0 2523 16
:3 0 e4 :2 0 1462
228f 2290 :3 0 2291
:2 0 91 :3 0 a1
:3 0 1e8 :3 0 2e
:3 0 1464 2295 2297
4d :2 0 1466 2294
229a 2293 229b 0
23e4 85 :3 0 4d
:2 0 229d 229e 0
23e4 1f0 :3 0 85
:3 0 91 :3 0 f6
:2 0 1f1 :3 0 146b
22a3 22a5 :3 0 22a6
23e3 48 :3 0 234
:3 0 235 :3 0 22a9
22aa 0 24 :2 0
24 :2 0 85 :3 0
bf :2 0 46 :2 0
146e 22af 22b1 :3 0
2e :3 0 1471 22ab
22b4 22a8 22b5 0
23e1 1dd :3 0 3b
:3 0 1a6 :3 0 10d
:3 0 48 :4 0 236
1 :8 0 22c2 21
:3 0 e9 :3 0 22bd
22be 0 ea :3 0
22bf 22c0 0 22c2
1476 2315 eb :3 0
21 :3 0 e9 :3 0
22c4 22c5 0 ea
:3 0 22c6 22c7 0
22c9 1479 22cb 147b
22ca 22c9 :2 0 2313
ec :3 0 21 :3 0
e9 :3 0 22ce 22cf
0 ea :3 0 22d0
22d1 0 230b ad
:3 0 ae :3 0 22d3
22d4 0 ad :3 0
bd :3 0 22d6 22d7
0 ed :2 0 2
:4 0 237 :2 0 bf
:2 0 46 :2 0 147d
22dc 22de :3 0 1480
22d8 22e0 22d5 22e2
22e1 :2 0 b0 :3 0
c0 :3 0 22e4 22e5
0 b2 :4 0 238
:4 0 b5 :2 0 b6
:3 0 1484 22e9 22eb
:3 0 b5 :2 0 c9
:4 0 1487 22ed 22ef
:3 0 b5 :2 0 f0
:4 0 148a 22f1 22f3
:3 0 b5 :2 0 ea
:3 0 148d 22f5 22f7
:3 0 b5 :2 0 f1
:4 0 1490 22f9 22fb
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 1493
22fe 2302 1497 22fd
2304 :3 0 149a 22e6
2306 :2 0 2308 149d
2309 22e3 2308 0
230a 149f 0 230b
14a1 230e :3 0 230e
0 230e 230d 230b
230c :6 0 2310 3a
:3 0 14a4 2312 14a6
2311 2310 :2 0 2313
14a8 :2 0 2315 0
2315 2314 22c2 2313
:6 0 23e1 39 :3 0
21 :3 0 e9 :3 0
2317 2318 0 21
:3 0 c3 :2 0 f5
:3 0 231a 231c 0
14ad 231b 231e :3 0
231f :2 0 ad :3 0
ae :3 0 2321 2322
0 ad :3 0 bd
:3 0 2324 2325 0
24 :2 0 2 :4 0
239 :2 0 bf :2 0
46 :2 0 14b0 232a
232c :3 0 14b3 2326
232e 2323 2330 232f
:2 0 b0 :3 0 c0
:3 0 2332 2333 0
b2 :4 0 23a :4 0
b5 :2 0 b6 :3 0
14b7 2337 2339 :3 0
b5 :2 0 c9 :4 0
14ba 233b 233d :3 0
b5 :2 0 ca :4 0
14bd 233f 2341 :3 0
b5 :2 0 a1 :3 0
ba :3 0 c2 :3 0
d6 :3 0 2346 2347
0 14c0 2345 2349
bb :4 0 14c2 2344
234c 14c5 2343 234e
:3 0 b5 :2 0 116
:4 0 14c8 2350 2352
:3 0 b5 :2 0 b8
:4 0 14cb 2354 2356
:3 0 b5 :2 0 48
:3 0 14ce 2358 235a
:3 0 b5 :2 0 b8
:4 0 14d1 235c 235e
:3 0 14d4 2334 2360
:2 0 2362 14d7 2363
2331 2362 0 2364
14d9 0 238e ad
:3 0 ae :3 0 2365
2366 0 ad :3 0
bd :3 0 2368 2369
0 46 :2 0 2
:4 0 23b :2 0 bf
:2 0 46 :2 0 14db
236e 2370 :3 0 14de
236a 2372 2367 2374
2373 :2 0 b0 :3 0
c0 :3 0 2376 2377
0 b2 :4 0 23c
:4 0 b5 :2 0 b6
:3 0 14e2 237b 237d
:3 0 14e5 2378 237f
:2 0 2381 14e8 2382
2375 2381 0 2383
14ea 0 238e ce
:3 0 119 :3 0 2384
2385 0 c2 :3 0
d6 :3 0 2387 2388
0 48 :3 0 d0
:3 0 14ec 2386 238c
:2 0 238e 14f0 238f
2320 238e 0 2390
14f4 0 23e1 76
:3 0 76 :3 0 b5
:2 0 23d :4 0 14f6
2393 2395 :3 0 b5
:2 0 23e :4 0 14f9
2397 2399 :3 0 b5
:2 0 23f :4 0 14fc
239b 239d :3 0 b5
:2 0 240 :4 0 14ff
239f 23a1 :3 0 b5
:2 0 3b :3 0 1502
23a3 23a5 :3 0 b5
:2 0 241 :4 0 1505
23a7 23a9 :3 0 2391
23aa 0 23e1 a1
:3 0 5b :3 0 4d
:2 0 1508 23ac 23af
c2 :3 0 d2 :2 0
d3 :3 0 23b1 23b3
0 150d 23b2 23b5
:3 0 76 :3 0 76
:3 0 b5 :2 0 242
:4 0 1510 23b9 23bb
:3 0 23b7 23bc 0
23be 1513 23c7 76
:3 0 76 :3 0 b5
:2 0 243 :4 0 1515
23c1 23c3 :3 0 23bf
23c4 0 23c6 1518
23c8 23b6 23be 0
23c9 0 23c6 0
23c9 151a 0 23e1
a1 :3 0 5b :3 0
4d :2 0 151d 23ca
23cd c2 :3 0 d2
:2 0 d3 :3 0 23cf
23d1 0 1522 23d0
23d3 :3 0 1f5 :8 0
23d7 1525 23d8 23d4
23d7 0 23d9 1527
0 23e1 85 :3 0
85 :3 0 bf :2 0
24 :2 0 1529 23dc
23de :3 0 23da 23df
0 23e1 152c 23e3
1f1 :3 0 22a7 23e1
:4 0 23e4 1534 23e5
2292 23e4 0 23e6
1538 0 2523 a1
:3 0 5b :3 0 4d
:2 0 153a 23e7 23ea
c2 :3 0 c3 :2 0
d3 :3 0 23ec 23ee
0 153f 23ed 23f0
:3 0 92 :3 0 a1
:3 0 1e8 :3 0 30
:3 0 1542 23f4 23f6
4d :2 0 1544 23f3
23f9 23f2 23fa 0
2519 85 :3 0 4d
:2 0 23fc 23fd 0
2519 1f0 :3 0 85
:3 0 92 :3 0 f6
:2 0 1f1 :3 0 1549
2402 2404 :3 0 2405
2518 48 :3 0 234
:3 0 235 :3 0 2408
2409 0 24 :2 0
24 :2 0 85 :3 0
bf :2 0 46 :2 0
154c 240e 2410 :3 0
30 :3 0 154f 240a
2413 2407 2414 0
2516 1dd :3 0 3b
:3 0 1a6 :3 0 10d
:3 0 48 :4 0 236
1 :8 0 2421 21
:3 0 e9 :3 0 241c
241d 0 ea :3 0
241e 241f 0 2421
1554 2474 eb :3 0
21 :3 0 e9 :3 0
2423 2424 0 ea
:3 0 2425 2426 0
2428 1557 242a 1559
2429 2428 :2 0 2472
ec :3 0 21 :3 0
e9 :3 0 242d 242e
0 ea :3 0 242f
2430 0 246a ad
:3 0 ae :3 0 2432
2433 0 ad :3 0
bd :3 0 2435 2436
0 ed :2 0 2
:4 0 244 :2 0 bf
:2 0 46 :2 0 155b
243b 243d :3 0 155e
2437 243f 2434 2441
2440 :2 0 b0 :3 0
c0 :3 0 2443 2444
0 b2 :4 0 245
:4 0 b5 :2 0 b6
:3 0 1562 2448 244a
:3 0 b5 :2 0 c9
:4 0 1565 244c 244e
:3 0 b5 :2 0 f0
:4 0 1568 2450 2452
:3 0 b5 :2 0 ea
:3 0 156b 2454 2456
:3 0 b5 :2 0 f1
:4 0 156e 2458 245a
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 1571
245d 2461 1575 245c
2463 :3 0 1578 2445
2465 :2 0 2467 157b
2468 2442 2467 0
2469 157d 0 246a
157f 246d :3 0 246d
0 246d 246c 246a
246b :6 0 246f 3d
:3 0 1582 2471 1584
2470 246f :2 0 2472
1586 :2 0 2474 0
2474 2473 2421 2472
:6 0 2516 3c :3 0
21 :3 0 e9 :3 0
2476 2477 0 21
:3 0 c3 :2 0 f5
:3 0 2479 247b 0
158b 247a 247d :3 0
247e :2 0 ad :3 0
ae :3 0 2480 2481
0 ad :3 0 bd
:3 0 2483 2484 0
24 :2 0 2 :4 0
246 :2 0 bf :2 0
46 :2 0 158e 2489
248b :3 0 1591 2485
248d 2482 248f 248e
:2 0 b0 :3 0 c0
:3 0 2491 2492 0
b2 :4 0 247 :4 0
b5 :2 0 b6 :3 0
1595 2496 2498 :3 0
b5 :2 0 c9 :4 0
1598 249a 249c :3 0
b5 :2 0 ca :4 0
159b 249e 24a0 :3 0
b5 :2 0 a1 :3 0
ba :3 0 c2 :3 0
d6 :3 0 24a5 24a6
0 159e 24a4 24a8
bb :4 0 15a0 24a3
24ab 15a3 24a2 24ad
:3 0 b5 :2 0 116
:4 0 15a6 24af 24b1
:3 0 b5 :2 0 b8
:4 0 15a9 24b3 24b5
:3 0 b5 :2 0 48
:3 0 15ac 24b7 24b9
:3 0 b5 :2 0 b8
:4 0 15af 24bb 24bd
:3 0 15b2 2493 24bf
:2 0 24c1 15b5 24c2
2490 24c1 0 24c3
15b7 0 24ed ad
:3 0 ae :3 0 24c4
24c5 0 ad :3 0
bd :3 0 24c7 24c8
0 46 :2 0 2
:4 0 248 :2 0 bf
:2 0 46 :2 0 15b9
24cd 24cf :3 0 15bc
24c9 24d1 24c6 24d3
24d2 :2 0 b0 :3 0
c0 :3 0 24d5 24d6
0 b2 :4 0 249
:4 0 b5 :2 0 b6
:3 0 15c0 24da 24dc
:3 0 15c3 24d7 24de
:2 0 24e0 15c6 24e1
24d4 24e0 0 24e2
15c8 0 24ed ce
:3 0 119 :3 0 24e3
24e4 0 c2 :3 0
d6 :3 0 24e6 24e7
0 48 :3 0 d0
:3 0 15ca 24e5 24eb
:2 0 24ed 15ce 24ee
247f 24ed 0 24ef
15d2 0 2516 76
:3 0 76 :3 0 b5
:2 0 23d :4 0 15d4
24f2 24f4 :3 0 b5
:2 0 23e :4 0 15d7
24f6 24f8 :3 0 b5
:2 0 23f :4 0 15da
24fa 24fc :3 0 b5
:2 0 240 :4 0 15dd
24fe 2500 :3 0 b5
:2 0 3b :3 0 15e0
2502 2504 :3 0 b5
:2 0 241 :4 0 15e3
2506 2508 :3 0 b5
:2 0 24a :4 0 15e6
250a 250c :3 0 24f0
250d 0 2516 85
:3 0 85 :3 0 bf
:2 0 24 :2 0 15e9
2511 2513 :3 0 250f
2514 0 2516 15ec
2518 1f1 :3 0 2406
2516 :4 0 2519 15f2
251a 23f1 2519 0
251b 15f6 0 2523
74 :3 0 74 :3 0
b5 :2 0 76 :3 0
15f8 251e 2520 :3 0
251c 2521 0 2523
15fb 2525 1b40 1bb6
0 2526 0 2523
0 2526 1603 0
35c4 74 :3 0 74
:3 0 b5 :2 0 24b
:4 0 1606 2529 252b
:3 0 2527 252c 0
35c4 24c :3 0 93
:3 0 74 :3 0 252f
2530 0 2531 :2 0
35c4 1f1 :3 0 93
:3 0 39 :4 0 2537
:2 0 29df 2534 2535
:3 0 93 :3 0 1b6
:3 0 2538 2539 :3 0
21 :3 0 e9 :3 0
253b 253c 0 4d
:2 0 253d 253e 0
2540 1609 2547 21
:3 0 e9 :3 0 2541
2542 0 180 :2 0
2543 2544 0 2546
160b 2548 253a 2540
0 2549 0 2546
0 2549 160d 0
29df 21 :3 0 e9
:3 0 254a 254b 0
21 :3 0 c3 :2 0
f5 :3 0 254d 254f
0 1612 254e 2551
:3 0 2552 :2 0 1f5
:8 0 2556 1615 2557
2553 2556 0 2558
1617 0 29df a1
:3 0 4f :3 0 4d
:2 0 1619 2559 255c
d2 :2 0 4d :2 0
161e 255e 2560 :3 0
a1 :3 0 5b :3 0
4d :2 0 1621 2562
2565 c2 :3 0 c3
:2 0 c4 :3 0 2567
2569 0 1626 2568
256b :3 0 2561 256d
256c :2 0 a1 :3 0
5b :3 0 4d :2 0
1629 256f 2572 c2
:3 0 c3 :2 0 d1
:3 0 2574 2576 0
162e 2575 2578 :3 0
256e 257a 2579 :2 0
a1 :3 0 5b :3 0
4d :2 0 1631 257c
257f c2 :3 0 c3
:2 0 d3 :3 0 2581
2583 0 1636 2582
2585 :3 0 14 :3 0
c5 :2 0 1639 2588
2589 :3 0 258a :2 0
2586 258c 258b :2 0
258d :2 0 257b 258f
258e :2 0 24c :3 0
95 :4 0 2594 :2 0
2679 2592 2595 :3 0
95 :3 0 28 :4 0
2599 :2 0 2679 2596
2597 :3 0 95 :3 0
1b6 :3 0 259a 259b
:3 0 21 :3 0 e9
:3 0 259d 259e 0
4d :2 0 259f 25a0
0 25a2 163b 25a9
21 :3 0 e9 :3 0
25a3 25a4 0 180
:2 0 25a5 25a6 0
25a8 163d 25aa 259c
25a2 0 25ab 0
25a8 0 25ab 163f
0 2679 21 :3 0
e9 :3 0 25ac 25ad
0 21 :3 0 d2
:2 0 f5 :3 0 25af
25b1 0 1644 25b0
25b3 :3 0 25b4 :2 0
24d :3 0 95 :4 0
25b9 :2 0 25bf 25b7
0 21 :3 0 e9
:3 0 25ba 25bb 0
ea :3 0 25bc 25bd
0 25bf 1647 2612
eb :3 0 21 :3 0
e9 :3 0 25c1 25c2
0 ea :3 0 25c3
25c4 0 25c6 164a
25c8 164c 25c7 25c6
:2 0 2610 ec :3 0
21 :3 0 e9 :3 0
25cb 25cc 0 ea
:3 0 25cd 25ce 0
2608 ad :3 0 ae
:3 0 25d0 25d1 0
ad :3 0 bd :3 0
25d3 25d4 0 ed
:2 0 2 :4 0 24e
:2 0 bf :2 0 46
:2 0 164e 25d9 25db
:3 0 1651 25d5 25dd
25d2 25df 25de :2 0
b0 :3 0 c0 :3 0
25e1 25e2 0 b2
:4 0 24f :4 0 b5
:2 0 b6 :3 0 1655
25e6 25e8 :3 0 b5
:2 0 c9 :4 0 1658
25ea 25ec :3 0 b5
:2 0 f0 :4 0 165b
25ee 25f0 :3 0 b5
:2 0 ea :3 0 165e
25f2 25f4 :3 0 b5
:2 0 f1 :4 0 1661
25f6 25f8 :3 0 b5
:2 0 f2 :3 0 f3
:3 0 46 :2 0 f4
:2 0 1664 25fb 25ff
1668 25fa 2601 :3 0
166b 25e3 2603 :2 0
2605 166e 2606 25e0
2605 0 2607 1670
0 2608 1672 260b
:3 0 260b 0 260b
260a 2608 2609 :6 0
260d 40 :3 0 1675
260f 1677 260e 260d
:2 0 2610 1679 :2 0
2612 0 2612 2611
25bf 2610 :6 0 2616
3f :3 0 250 :3 0
2614 :4 0 2616 167c
2676 24d :3 0 95
:4 0 261a :2 0 2620
2618 0 21 :3 0
e9 :3 0 261b 261c
0 ea :3 0 261d
261e 0 2620 167f
2673 eb :3 0 21
:3 0 e9 :3 0 2622
2623 0 ea :3 0
2624 2625 0 2627
1682 2629 1684 2628
2627 :2 0 2671 ec
:3 0 21 :3 0 e9
:3 0 262c 262d 0
ea :3 0 262e 262f
0 2669 ad :3 0
ae :3 0 2631 2632
0 ad :3 0 bd
:3 0 2634 2635 0
ed :2 0 2 :4 0
251 :2 0 bf :2 0
46 :2 0 1686 263a
263c :3 0 1689 2636
263e 2633 2640 263f
:2 0 b0 :3 0 c0
:3 0 2642 2643 0
b2 :4 0 252 :4 0
b5 :2 0 b6 :3 0
168d 2647 2649 :3 0
b5 :2 0 c9 :4 0
1690 264b 264d :3 0
b5 :2 0 f0 :4 0
1693 264f 2651 :3 0
b5 :2 0 ea :3 0
1696 2653 2655 :3 0
b5 :2 0 f1 :4 0
1699 2657 2659 :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 169c 265c
2660 16a0 265b 2662
:3 0 16a3 2644 2664
:2 0 2666 16a6 2667
2641 2666 0 2668
16a8 0 2669 16aa
266c :3 0 266c 0
266c 266b 2669 266a
:6 0 266e 42 :3 0
16ad 2670 16af 266f
266e :2 0 2671 16b1
:2 0 2673 0 2673
2672 2620 2671 :6 0
2675 3f :3 0 16b4
2677 25b5 2616 0
2678 0 2675 0
2678 16b6 0 2679
16b9 267a 2590 2679
0 267b 16be 0
29df 2e :3 0 16
:3 0 267c 267d 0
29df 30 :3 0 17
:3 0 267f 2680 0
29df a1 :3 0 4f
:3 0 4d :2 0 16c0
2682 2685 105 :2 0
4d :2 0 16c5 2687
2689 :3 0 f2 :3 0
45 :3 0 46 :2 0
46 :2 0 16c8 268b
268f d2 :2 0 46
:4 0 16ce 2691 2693
:3 0 47 :3 0 18f
:4 0 2696 2697 :3 0
2695 2698 0 269d
4e :3 0 4d :2 0
269a 269b 0 269d
16d1 27e9 12e :3 0
a1 :3 0 4f :3 0
4d :2 0 16d4 269f
26a2 16d7 269e 26a4
c3 :2 0 4d :2 0
16db 26a6 26a8 :3 0
253 :3 0 254 :3 0
82 :3 0 97 :3 0
98 :3 0 25 :3 0
9a :3 0 39 :3 0
197 :4 0 255 1
:8 0 26b9 21 :3 0
e9 :3 0 26b4 26b5
0 ea :3 0 26b6
26b7 0 26b9 16de
270c eb :3 0 21
:3 0 e9 :3 0 26bb
26bc 0 ea :3 0
26bd 26be 0 26c0
16e1 26c2 16e3 26c1
26c0 :2 0 270a ec
:3 0 21 :3 0 e9
:3 0 26c5 26c6 0
ea :3 0 26c7 26c8
0 2702 ad :3 0
ae :3 0 26ca 26cb
0 ad :3 0 bd
:3 0 26cd 26ce 0
ed :2 0 2 :4 0
256 :2 0 bf :2 0
46 :2 0 16e5 26d3
26d5 :3 0 16e8 26cf
26d7 26cc 26d9 26d8
:2 0 b0 :3 0 c0
:3 0 26db 26dc 0
b2 :4 0 257 :4 0
b5 :2 0 b6 :3 0
16ec 26e0 26e2 :3 0
b5 :2 0 c9 :4 0
16ef 26e4 26e6 :3 0
b5 :2 0 f0 :4 0
16f2 26e8 26ea :3 0
b5 :2 0 ea :3 0
16f5 26ec 26ee :3 0
b5 :2 0 f1 :4 0
16f8 26f0 26f2 :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 16fb 26f5
26f9 16ff 26f4 26fb
:3 0 1702 26dd 26fd
:2 0 26ff 1705 2700
26da 26ff 0 2701
1707 0 2702 1709
2705 :3 0 2705 0
2705 2704 2702 2703
:6 0 2707 44 :3 0
170c 2709 170e 2708
2707 :2 0 270a 1710
:2 0 270c 0 270c
270b 26b9 270a :6 0
27e5 3f :3 0 a1
:3 0 82 :3 0 4d
:2 0 1713 270e 2711
105 :2 0 4d :2 0
1718 2713 2715 :3 0
24d :3 0 93 :4 0
271a :2 0 2720 2718
0 21 :3 0 e9
:3 0 271b 271c 0
ea :3 0 271d 271e
0 2720 171b 2773
eb :3 0 21 :3 0
e9 :3 0 2722 2723
0 ea :3 0 2724
2725 0 2727 171e
2729 1720 2728 2727
:2 0 2771 ec :3 0
21 :3 0 e9 :3 0
272c 272d 0 ea
:3 0 272e 272f 0
2769 ad :3 0 ae
:3 0 2731 2732 0
ad :3 0 bd :3 0
2734 2735 0 ed
:2 0 2 :4 0 258
:2 0 bf :2 0 46
:2 0 1722 273a 273c
:3 0 1725 2736 273e
2733 2740 273f :2 0
b0 :3 0 c0 :3 0
2742 2743 0 b2
:4 0 259 :4 0 b5
:2 0 b6 :3 0 1729
2747 2749 :3 0 b5
:2 0 c9 :4 0 172c
274b 274d :3 0 b5
:2 0 f0 :4 0 172f
274f 2751 :3 0 b5
:2 0 ea :3 0 1732
2753 2755 :3 0 b5
:2 0 f1 :4 0 1735
2757 2759 :3 0 b5
:2 0 f2 :3 0 f3
:3 0 46 :2 0 f4
:2 0 1738 275c 2760
173c 275b 2762 :3 0
173f 2744 2764 :2 0
2766 1742 2767 2741
2766 0 2768 1744
0 2769 1746 276c
:3 0 276c 0 276c
276b 2769 276a :6 0
276e 46 :3 0 1749
2770 174b 276f 276e
:2 0 2771 174d :2 0
2773 0 2773 2772
2720 2771 :6 0 27e2
3f :3 0 ad :3 0
ae :3 0 2775 2776
0 ad :3 0 bd
:3 0 2778 2779 0
24 :2 0 2 :4 0
25a :2 0 bf :2 0
46 :2 0 1750 277e
2780 :3 0 1753 277a
2782 2777 2784 2783
:2 0 b0 :3 0 c0
:3 0 2786 2787 0
b2 :4 0 25b :4 0
b5 :2 0 b6 :3 0
1757 278b 278d :3 0
b5 :2 0 c9 :4 0
175a 278f 2791 :3 0
b5 :2 0 ca :4 0
175d 2793 2795 :3 0
b5 :2 0 a1 :3 0
ba :3 0 c2 :3 0
25c :3 0 279a 279b
0 1760 2799 279d
bb :4 0 1762 2798
27a0 1765 2797 27a2
:3 0 b5 :2 0 116
:4 0 1768 27a4 27a6
:3 0 b5 :2 0 b8
:4 0 176b 27a8 27aa
:3 0 b5 :2 0 39
:3 0 176e 27ac 27ae
:3 0 b5 :2 0 b8
:4 0 1771 27b0 27b2
:3 0 1774 2788 27b4
:2 0 27b6 1777 27b7
2785 27b6 0 27b8
1779 0 27e2 ad
:3 0 ae :3 0 27b9
27ba 0 ad :3 0
bd :3 0 27bc 27bd
0 46 :2 0 2
:4 0 25d :2 0 bf
:2 0 46 :2 0 177b
27c2 27c4 :3 0 177e
27be 27c6 27bb 27c8
27c7 :2 0 b0 :3 0
c0 :3 0 27ca 27cb
0 b2 :4 0 25e
:4 0 b5 :2 0 b6
:3 0 1782 27cf 27d1
:3 0 1785 27cc 27d3
:2 0 27d5 1788 27d6
27c9 27d5 0 27d7
178a 0 27e2 ce
:3 0 119 :3 0 27d8
27d9 0 c2 :3 0
25c :3 0 27db 27dc
0 39 :3 0 d0
:3 0 178c 27da 27e0
:2 0 27e2 1790 27e3
2716 27e2 0 27e4
1795 0 27e5 1797
27e6 26a9 27e5 0
27e7 179a 0 27e8
179c 27ea 2694 269d
0 27eb 0 27e8
0 27eb 179e 0
2980 88 :3 0 25f
:3 0 47 :3 0 25
:3 0 26 :3 0 39
:3 0 65 :3 0 4f
:3 0 2e :3 0 30
:3 0 31 :3 0 32
:3 0 41 :3 0 42
:3 0 60 :3 0 58
:3 0 56 :3 0 59
:3 0 5a :3 0 54
:3 0 5b :3 0 45
:3 0 52 :3 0 4c
:3 0 5c :3 0 57
:3 0 51 :3 0 5e
:3 0 63 :3 0 29
:3 0 a1 :3 0 5e
:3 0 4d :2 0 17a1
280a 280d 4d :2 0
67 :2 0 67 :2 0
17a4 27ed 2812 27ec
2813 0 2980 88
:3 0 c3 :2 0 4d
:2 0 17c7 2816 2818
:3 0 24d :3 0 93
:4 0 281d :2 0 2823
281b 0 21 :3 0
e9 :3 0 281e 281f
0 ea :3 0 2820
2821 0 2823 17ca
2876 eb :3 0 21
:3 0 e9 :3 0 2825
2826 0 ea :3 0
2827 2828 0 282a
17cd 282c 17cf 282b
282a :2 0 2874 ec
:3 0 21 :3 0 e9
:3 0 282f 2830 0
ea :3 0 2831 2832
0 286c ad :3 0
ae :3 0 2834 2835
0 ad :3 0 bd
:3 0 2837 2838 0
ed :2 0 2 :4 0
260 :2 0 bf :2 0
46 :2 0 17d1 283d
283f :3 0 17d4 2839
2841 2836 2843 2842
:2 0 b0 :3 0 c0
:3 0 2845 2846 0
b2 :4 0 261 :4 0
b5 :2 0 b6 :3 0
17d8 284a 284c :3 0
b5 :2 0 c9 :4 0
17db 284e 2850 :3 0
b5 :2 0 f0 :4 0
17de 2852 2854 :3 0
b5 :2 0 ea :3 0
17e1 2856 2858 :3 0
b5 :2 0 f1 :4 0
17e4 285a 285c :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 17e7 285f
2863 17eb 285e 2865
:3 0 17ee 2847 2867
:2 0 2869 17f1 286a
2844 2869 0 286b
17f3 0 286c 17f5
286f :3 0 286f 0
286f 286e 286c 286d
:6 0 2871 48 :3 0
17f8 2873 17fa 2872
2871 :2 0 2874 17fc
:2 0 2876 0 2876
2875 2823 2874 :6 0
28f1 3f :3 0 88
:3 0 c2 :3 0 f6
:2 0 262 :3 0 2879
287b 0 1801 287a
287d :3 0 88 :3 0
c2 :3 0 263 :3 0
2880 2881 0 287f
2882 0 2884 1804
2885 287e 2884 0
2886 1806 0 28f1
ad :3 0 ae :3 0
2887 2888 0 ad
:3 0 bd :3 0 288a
288b 0 24 :2 0
2 :4 0 264 :2 0
bf :2 0 46 :2 0
1808 2890 2892 :3 0
180b 288c 2894 2889
2896 2895 :2 0 b0
:3 0 c0 :3 0 2898
2899 0 b2 :4 0
265 :4 0 b5 :2 0
b6 :3 0 180f 289d
289f :3 0 b5 :2 0
c9 :4 0 1812 28a1
28a3 :3 0 b5 :2 0
ca :4 0 1815 28a5
28a7 :3 0 b5 :2 0
a1 :3 0 ba :3 0
88 :3 0 1818 28ab
28ad bb :4 0 181a
28aa 28b0 181d 28a9
28b2 :3 0 b5 :2 0
116 :4 0 1820 28b4
28b6 :3 0 b5 :2 0
b8 :4 0 1823 28b8
28ba :3 0 b5 :2 0
47 :3 0 1826 28bc
28be :3 0 b5 :2 0
b8 :4 0 1829 28c0
28c2 :3 0 182c 289a
28c4 :2 0 28c6 182f
28c7 2897 28c6 0
28c8 1831 0 28f1
ad :3 0 ae :3 0
28c9 28ca 0 ad
:3 0 bd :3 0 28cc
28cd 0 46 :2 0
2 :4 0 266 :2 0
bf :2 0 46 :2 0
1833 28d2 28d4 :3 0
1836 28ce 28d6 28cb
28d8 28d7 :2 0 b0
:3 0 c0 :3 0 28da
28db 0 b2 :4 0
267 :4 0 b5 :2 0
b6 :3 0 183a 28df
28e1 :3 0 183d 28dc
28e3 :2 0 28e5 1840
28e6 28d9 28e5 0
28e7 1842 0 28f1
ce :3 0 119 :3 0
28e8 28e9 0 88
:3 0 47 :3 0 d0
:3 0 1844 28ea 28ee
:2 0 28f1 d9 :3 0
1848 297c f2 :3 0
45 :3 0 46 :2 0
46 :2 0 184e 28f2
28f6 d2 :2 0 46
:4 0 1854 28f8 28fa
:3 0 28fb :2 0 f2
:3 0 4b :3 0 46
:2 0 46 :2 0 1857
28fd 2901 d2 :2 0
1b0 :4 0 185d 2903
2905 :3 0 2906 :2 0
28fc 2908 2907 :2 0
190 :3 0 10b :3 0
191 :3 0 39 :3 0
10b :4 0 268 1
:8 0 2925 1b6 :4 0
2910 :3 0 d0 :3 0
d2 :2 0 1862 2913
2914 :3 0 21 :3 0
e9 :3 0 2916 2917
0 4d :2 0 2918
2919 0 291b 1865
2922 21 :3 0 e9
:3 0 291c 291d 0
180 :2 0 291e 291f
0 2921 1867 2923
2915 291b 0 2924
0 2921 0 2924
1869 0 2925 186c
2978 eb :3 0 21
:3 0 e9 :3 0 2927
2928 0 ea :3 0
2929 292a 0 292c
186f 292e 1871 292d
292c :2 0 2976 ec
:3 0 21 :3 0 e9
:3 0 2931 2932 0
ea :3 0 2933 2934
0 296e ad :3 0
ae :3 0 2936 2937
0 ad :3 0 bd
:3 0 2939 293a 0
ed :2 0 2 :4 0
269 :2 0 bf :2 0
46 :2 0 1873 293f
2941 :3 0 1876 293b
2943 2938 2945 2944
:2 0 b0 :3 0 c0
:3 0 2947 2948 0
b2 :4 0 26a :4 0
b5 :2 0 b6 :3 0
187a 294c 294e :3 0
b5 :2 0 c9 :4 0
187d 2950 2952 :3 0
b5 :2 0 f0 :4 0
1880 2954 2956 :3 0
b5 :2 0 ea :3 0
1883 2958 295a :3 0
b5 :2 0 f1 :4 0
1886 295c 295e :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 1889 2961
2965 188d 2960 2967
:3 0 1890 2949 2969
:2 0 296b 1893 296c
2946 296b 0 296d
1895 0 296e 1897
2971 :3 0 2971 0
2971 2970 296e 296f
:6 0 2973 4a :3 0
189a 2975 189c 2974
2973 :2 0 2976 189e
:2 0 2978 0 2978
2977 2925 2976 :6 0
297a 3f :3 0 18a1
297b 2909 297a 0
297d 2819 28f1 0
297d 18a3 0 2980
1f5 :8 0 2980 18a6
29d8 a1 :3 0 4c
:3 0 4d :2 0 18ab
2981 2984 c2 :3 0
c3 :2 0 c4 :3 0
2986 2988 0 18b0
2987 298a :3 0 41
:3 0 5 :3 0 298c
298d 0 2992 42
:3 0 6 :3 0 298f
2990 0 2992 18b3
2993 298b 2992 0
2994 18b6 0 29d7
88 :3 0 25f :3 0
47 :3 0 25 :3 0
26 :3 0 39 :3 0
65 :3 0 4f :3 0
2e :3 0 30 :3 0
31 :3 0 32 :3 0
41 :3 0 42 :3 0
60 :3 0 58 :3 0
56 :3 0 59 :3 0
5a :3 0 54 :3 0
5b :3 0 45 :3 0
52 :3 0 4c :3 0
5c :3 0 57 :3 0
51 :3 0 5e :3 0
63 :3 0 29 :3 0
4d :2 0 4d :2 0
67 :2 0 67 :2 0
18b8 2996 29b7 2995
29b8 0 29d7 88
:3 0 d2 :2 0 4d
:2 0 18db 29bb 29bd
:3 0 87 :3 0 87
:3 0 bf :2 0 46
:2 0 18de 29c1 29c3
:3 0 29bf 29c4 0
29c6 18e1 29c7 29be
29c6 0 29c8 18e3
0 29d7 87 :3 0
a1 :3 0 d2 :2 0
55 :3 0 4d :2 0
18e5 29ca 29ce 18ea
29cb 29d0 :3 0 1f5
:8 0 29d4 18ed 29d5
29d1 29d4 0 29d6
18ef 0 29d7 18f1
29d9 268a 2980 0
29da 0 29d7 0
29da 18f6 0 29df
250 :5 0 29dd 4c
0 29de 18f9 29dd
29dc 29df 18fb 29e1
1f1 :4 0 29df :4 0
35c4 24d :3 0 93
:4 0 29e5 :2 0 29eb
29e3 0 21 :3 0
e9 :3 0 29e6 29e7
0 ea :3 0 29e8
29e9 0 29eb 1904
2a3e eb :3 0 21
:3 0 e9 :3 0 29ed
29ee 0 ea :3 0
29ef 29f0 0 29f2
1907 29f4 1909 29f3
29f2 :2 0 2a3c ec
:3 0 21 :3 0 e9
:3 0 29f7 29f8 0
ea :3 0 29f9 29fa
0 2a34 ad :3 0
ae :3 0 29fc 29fd
0 ad :3 0 bd
:3 0 29ff 2a00 0
ed :2 0 2 :4 0
26b :2 0 bf :2 0
46 :2 0 190b 2a05
2a07 :3 0 190e 2a01
2a09 29fe 2a0b 2a0a
:2 0 b0 :3 0 c0
:3 0 2a0d 2a0e 0
b2 :4 0 26c :4 0
b5 :2 0 b6 :3 0
1912 2a12 2a14 :3 0
b5 :2 0 c9 :4 0
1915 2a16 2a18 :3 0
b5 :2 0 f0 :4 0
1918 2a1a 2a1c :3 0
b5 :2 0 ea :3 0
191b 2a1e 2a20 :3 0
b5 :2 0 f1 :4 0
191e 2a22 2a24 :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 1921 2a27
2a2b 1925 2a26 2a2d
:3 0 1928 2a0f 2a2f
:2 0 2a31 192b 2a32
2a0c 2a31 0 2a33
192d 0 2a34 192f
2a37 :3 0 2a37 0
2a37 2a36 2a34 2a35
:6 0 2a39 4c :3 0
1932 2a3b 1934 2a3a
2a39 :2 0 2a3c 1936
:2 0 2a3e 0 2a3e
2a3d 29eb 2a3c :6 0
35c4 1 :3 0 88
:3 0 c2 :3 0 d2
:2 0 26d :3 0 2a41
2a43 0 193b 2a42
2a45 :3 0 2a46 :2 0
88 :3 0 c3 :2 0
4d :2 0 1940 2a49
2a4b :3 0 2a4c :2 0
a1 :3 0 4f :3 0
4d :2 0 1943 2a4e
2a51 105 :2 0 4d
:2 0 1948 2a53 2a55
:3 0 2a56 :2 0 2a4d
2a58 2a57 :2 0 2a59
:2 0 2a47 2a5b 2a5a
:2 0 ad :3 0 ae
:3 0 2a5d 2a5e 0
ad :3 0 bd :3 0
2a60 2a61 0 24
:2 0 2 :4 0 26e
:2 0 bf :2 0 46
:2 0 194b 2a66 2a68
:3 0 194e 2a62 2a6a
2a5f 2a6c 2a6b :2 0
b0 :3 0 c0 :3 0
2a6e 2a6f 0 b2
:4 0 26f :4 0 b5
:2 0 b6 :3 0 1952
2a73 2a75 :3 0 b5
:2 0 c9 :4 0 1955
2a77 2a79 :3 0 b5
:2 0 ca :4 0 1958
2a7b 2a7d :3 0 b5
:2 0 a1 :3 0 ba
:3 0 88 :3 0 195b
2a81 2a83 bb :4 0
195d 2a80 2a86 1960
2a7f 2a88 :3 0 b5
:2 0 116 :4 0 1963
2a8a 2a8c :3 0 b5
:2 0 b8 :4 0 1966
2a8e 2a90 :3 0 b5
:2 0 47 :3 0 1969
2a92 2a94 :3 0 b5
:2 0 b8 :4 0 196c
2a96 2a98 :3 0 196f
2a70 2a9a :2 0 2a9c
1972 2a9d 2a6d 2a9c
0 2a9e 1974 0
2ac6 ad :3 0 ae
:3 0 2a9f 2aa0 0
ad :3 0 bd :3 0
2aa2 2aa3 0 46
:2 0 2 :4 0 270
:2 0 bf :2 0 46
:2 0 1976 2aa8 2aaa
:3 0 1979 2aa4 2aac
2aa1 2aae 2aad :2 0
b0 :3 0 c0 :3 0
2ab0 2ab1 0 b2
:4 0 271 :4 0 b5
:2 0 b6 :3 0 197d
2ab5 2ab7 :3 0 1980
2ab2 2ab9 :2 0 2abb
1983 2abc 2aaf 2abb
0 2abd 1985 0
2ac6 ce :3 0 119
:3 0 2abe 2abf 0
88 :3 0 47 :3 0
d0 :3 0 1987 2ac0
2ac4 :2 0 2ac6 198b
2ac7 2a5c 2ac6 0
2ac8 198f 0 35c4
8b :3 0 c3 :2 0
46 :2 0 1993 2aca
2acc :3 0 27 :3 0
272 :4 0 2ace 2acf
0 2e13 28 :3 0
182 :4 0 2ad1 2ad2
0 2e13 3a :3 0
182 :4 0 2ad4 2ad5
0 2e13 24c :3 0
9c :4 0 2ada :2 0
2e13 2ad8 2adb :3 0
85 :3 0 4d :2 0
2adc 2add 0 2e13
1f1 :3 0 9c :3 0
26 :3 0 39 :3 0
48 :3 0 33 :3 0
49 :3 0 34 :3 0
35 :3 0 36 :4 0
2aea :2 0 2d30 2ae0
2aeb :3 0 1996 :3 0
9c :3 0 1b6 :3 0
2aec 2aed :3 0 21
:3 0 e9 :3 0 2aef
2af0 0 4d :2 0
2af1 2af2 0 2af4
199f 2afb 21 :3 0
e9 :3 0 2af5 2af6
0 180 :2 0 2af7
2af8 0 2afa 19a1
2afc 2aee 2af4 0
2afd 0 2afa 0
2afd 19a3 0 2d30
21 :3 0 e9 :3 0
2afe 2aff 0 21
:3 0 c3 :2 0 f5
:3 0 2b01 2b03 0
19a8 2b02 2b05 :3 0
2b06 :2 0 1f5 :8 0
2b0a 19ab 2b0b 2b07
2b0a 0 2b0c 19ad
0 2d30 48 :3 0
e4 :2 0 19af 2b0e
2b0f :3 0 2b10 :2 0
e5 :3 0 111 :3 0
2b12 2b13 0 48
:3 0 39 :3 0 19b1
2b14 2b17 d2 :2 0
4d :2 0 19b6 2b19
2b1b :3 0 2b11 2b1d
2b1c :2 0 a1 :3 0
1e8 :3 0 27 :3 0
19b9 2b20 2b22 4d
:2 0 19bb 2b1f 2b25
d2 :2 0 4d :2 0
19c0 2b27 2b29 :3 0
2b1e 2b2b 2b2a :2 0
27 :3 0 26 :3 0
2b2d 2b2e 0 2b30
19c3 2b31 2b2c 2b30
0 2b32 19c5 0
2d30 e5 :3 0 111
:3 0 2b33 2b34 0
28 :3 0 39 :3 0
19c7 2b35 2b38 c3
:2 0 4d :2 0 19cc
2b3a 2b3c :3 0 85
:3 0 c3 :2 0 4d
:2 0 19d1 2b3f 2b41
:3 0 a7 :3 0 a0
:3 0 37 :3 0 a2
:3 0 2e :3 0 a3
:3 0 38 :3 0 a4
:3 0 30 :3 0 a5
:3 0 31 :3 0 a6
:3 0 32 :3 0 9d
:3 0 3a :4 0 273
1 :8 0 2b68 1b6
:4 0 2b53 :3 0 d0
:3 0 d2 :2 0 19d6
2b56 2b57 :3 0 21
:3 0 e9 :3 0 2b59
2b5a 0 4d :2 0
2b5b 2b5c 0 2b5e
19d9 2b65 21 :3 0
e9 :3 0 2b5f 2b60
0 180 :2 0 2b61
2b62 0 2b64 19db
2b66 2b58 2b5e 0
2b67 0 2b64 0
2b67 19dd 0 2b68
19e0 2bbb eb :3 0
21 :3 0 e9 :3 0
2b6a 2b6b 0 ea
:3 0 2b6c 2b6d 0
2b6f 19e3 2b71 19e5
2b70 2b6f :2 0 2bb9
ec :3 0 21 :3 0
e9 :3 0 2b74 2b75
0 ea :3 0 2b76
2b77 0 2bb1 ad
:3 0 ae :3 0 2b79
2b7a 0 ad :3 0
bd :3 0 2b7c 2b7d
0 ed :2 0 2
:4 0 274 :2 0 bf
:2 0 46 :2 0 19e7
2b82 2b84 :3 0 19ea
2b7e 2b86 2b7b 2b88
2b87 :2 0 b0 :3 0
c0 :3 0 2b8a 2b8b
0 b2 :4 0 275
:4 0 b5 :2 0 b6
:3 0 19ee 2b8f 2b91
:3 0 b5 :2 0 c9
:4 0 19f1 2b93 2b95
:3 0 b5 :2 0 f0
:4 0 19f4 2b97 2b99
:3 0 b5 :2 0 ea
:3 0 19f7 2b9b 2b9d
:3 0 b5 :2 0 f1
:4 0 19fa 2b9f 2ba1
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 19fd
2ba4 2ba8 1a01 2ba3
2baa :3 0 1a04 2b8c
2bac :2 0 2bae 1a07
2baf 2b89 2bae 0
2bb0 1a09 0 2bb1
1a0b 2bb4 :3 0 2bb4
0 2bb4 2bb3 2bb1
2bb2 :6 0 2bb6 4f
:3 0 1a0e 2bb8 1a10
2bb7 2bb6 :2 0 2bb9
1a12 :2 0 2bbb 0
2bbb 2bba 2b68 2bb9
:6 0 2bbd 4e :3 0
1a15 2bbe 2b42 2bbd
0 2bbf 1a17 0
2bd8 2e :3 0 182
:4 0 2bc0 2bc1 0
2bd8 30 :3 0 182
:4 0 2bc3 2bc4 0
2bd8 31 :3 0 182
:4 0 2bc6 2bc7 0
2bd8 32 :3 0 182
:4 0 2bc9 2bca 0
2bd8 37 :3 0 182
:4 0 2bcc 2bcd 0
2bd8 38 :3 0 182
:4 0 2bcf 2bd0 0
2bd8 28 :3 0 39
:3 0 2bd2 2bd3 0
2bd8 85 :3 0 4d
:2 0 2bd5 2bd6 0
2bd8 1a19 2bd9 2b3d
2bd8 0 2bda 1a23
0 2d30 85 :3 0
c3 :2 0 4d :2 0
1a27 2bdc 2bde :3 0
3a :3 0 e4 :2 0
1a2a 2be1 2be2 :3 0
2be3 :2 0 2bdf 2be5
2be4 :2 0 a7 :3 0
a0 :3 0 a2 :3 0
a3 :3 0 a4 :3 0
a5 :3 0 a6 :3 0
9d :3 0 3a :4 0
276 1 :8 0 2c06
1b6 :4 0 2bf1 :3 0
d0 :3 0 d2 :2 0
1a2e 2bf4 2bf5 :3 0
21 :3 0 e9 :3 0
2bf7 2bf8 0 4d
:2 0 2bf9 2bfa 0
2bfc 1a31 2c03 21
:3 0 e9 :3 0 2bfd
2bfe 0 180 :2 0
2bff 2c00 0 2c02
1a33 2c04 2bf6 2bfc
0 2c05 0 2c02
0 2c05 1a35 0
2c06 1a38 2c59 eb
:3 0 21 :3 0 e9
:3 0 2c08 2c09 0
ea :3 0 2c0a 2c0b
0 2c0d 1a3b 2c0f
1a3d 2c0e 2c0d :2 0
2c57 ec :3 0 21
:3 0 e9 :3 0 2c12
2c13 0 ea :3 0
2c14 2c15 0 2c4f
ad :3 0 ae :3 0
2c17 2c18 0 ad
:3 0 bd :3 0 2c1a
2c1b 0 ed :2 0
2 :4 0 277 :2 0
bf :2 0 46 :2 0
1a3f 2c20 2c22 :3 0
1a42 2c1c 2c24 2c19
2c26 2c25 :2 0 b0
:3 0 c0 :3 0 2c28
2c29 0 b2 :4 0
278 :4 0 b5 :2 0
b6 :3 0 1a46 2c2d
2c2f :3 0 b5 :2 0
c9 :4 0 1a49 2c31
2c33 :3 0 b5 :2 0
f0 :4 0 1a4c 2c35
2c37 :3 0 b5 :2 0
ea :3 0 1a4f 2c39
2c3b :3 0 b5 :2 0
f1 :4 0 1a52 2c3d
2c3f :3 0 b5 :2 0
f2 :3 0 f3 :3 0
46 :2 0 f4 :2 0
1a55 2c42 2c46 1a59
2c41 2c48 :3 0 1a5c
2c2a 2c4a :2 0 2c4c
1a5f 2c4d 2c27 2c4c
0 2c4e 1a61 0
2c4f 1a63 2c52 :3 0
2c52 0 2c52 2c51
2c4f 2c50 :6 0 2c54
51 :3 0 1a66 2c56
1a68 2c55 2c54 :2 0
2c57 1a6a :2 0 2c59
0 2c59 2c58 2c06
2c57 :6 0 2c5b 4e
:3 0 1a6d 2c5c 2be6
2c5b 0 2c5d 1a6f
0 2d30 33 :3 0
e4 :2 0 1a71 2c5f
2c60 :3 0 2c61 :2 0
2e :3 0 e4 :2 0
1a73 2c64 2c65 :3 0
2c66 :2 0 2e :3 0
f2 :3 0 2e :3 0
b5 :2 0 279 :4 0
1a75 2c6b 2c6d :3 0
46 :2 0 2f :2 0
1a78 2c69 2c71 2c68
2c72 0 2c74 1a7c
2c75 2c67 2c74 0
2c76 1a7e 0 2c83
2e :3 0 f2 :3 0
2e :3 0 b5 :2 0
33 :3 0 1a80 2c7a
2c7c :3 0 46 :2 0
2f :2 0 1a83 2c78
2c80 2c77 2c81 0
2c83 1a87 2c84 2c62
2c83 0 2c85 1a8a
0 2d30 48 :3 0
e4 :2 0 1a8c 2c87
2c88 :3 0 2c89 :2 0
37 :3 0 f2 :3 0
37 :3 0 b5 :2 0
48 :3 0 1a8e 2c8e
2c90 :3 0 46 :2 0
2f :2 0 1a91 2c8c
2c94 2c8b 2c95 0
2c97 1a95 2c98 2c8a
2c97 0 2c99 1a97
0 2d30 34 :3 0
e4 :2 0 1a99 2c9b
2c9c :3 0 2c9d :2 0
30 :3 0 e4 :2 0
1a9b 2ca0 2ca1 :3 0
2ca2 :2 0 30 :3 0
f2 :3 0 30 :3 0
b5 :2 0 279 :4 0
1a9d 2ca7 2ca9 :3 0
46 :2 0 2f :2 0
1aa0 2ca5 2cad 2ca4
2cae 0 2cb0 1aa4
2cb1 2ca3 2cb0 0
2cb2 1aa6 0 2cbf
30 :3 0 f2 :3 0
30 :3 0 b5 :2 0
34 :3 0 1aa8 2cb6
2cb8 :3 0 46 :2 0
2f :2 0 1aab 2cb4
2cbc 2cb3 2cbd 0
2cbf 1aaf 2cc0 2c9e
2cbf 0 2cc1 1ab2
0 2d30 49 :3 0
e4 :2 0 1ab4 2cc3
2cc4 :3 0 2cc5 :2 0
38 :3 0 f2 :3 0
38 :3 0 b5 :2 0
49 :3 0 1ab6 2cca
2ccc :3 0 46 :2 0
2f :2 0 1ab9 2cc8
2cd0 2cc7 2cd1 0
2cd3 1abd 2cd4 2cc6
2cd3 0 2cd5 1abf
0 2d30 35 :3 0
e4 :2 0 1ac1 2cd7
2cd8 :3 0 2cd9 :2 0
31 :3 0 e4 :2 0
1ac3 2cdc 2cdd :3 0
2cde :2 0 31 :3 0
f2 :3 0 31 :3 0
b5 :2 0 279 :4 0
1ac5 2ce3 2ce5 :3 0
46 :2 0 2f :2 0
1ac8 2ce1 2ce9 2ce0
2cea 0 2cec 1acc
2ced 2cdf 2cec 0
2cee 1ace 0 2cfb
31 :3 0 f2 :3 0
31 :3 0 b5 :2 0
35 :3 0 1ad0 2cf2
2cf4 :3 0 46 :2 0
2f :2 0 1ad3 2cf0
2cf8 2cef 2cf9 0
2cfb 1ad7 2cfc 2cda
2cfb 0 2cfd 1ada
0 2d30 36 :3 0
e4 :2 0 1adc 2cff
2d00 :3 0 2d01 :2 0
32 :3 0 e4 :2 0
1ade 2d04 2d05 :3 0
2d06 :2 0 32 :3 0
f2 :3 0 32 :3 0
b5 :2 0 279 :4 0
1ae0 2d0b 2d0d :3 0
46 :2 0 2f :2 0
1ae3 2d09 2d11 2d08
2d12 0 2d14 1ae7
2d15 2d07 2d14 0
2d16 1ae9 0 2d23
32 :3 0 f2 :3 0
32 :3 0 b5 :2 0
36 :3 0 1aeb 2d1a
2d1c :3 0 46 :2 0
2f :2 0 1aee 2d18
2d20 2d17 2d21 0
2d23 1af2 2d24 2d02
2d23 0 2d25 1af5
0 2d30 85 :3 0
85 :3 0 bf :2 0
46 :2 0 1af7 2d28
2d2a :3 0 2d26 2d2b
0 2d30 3a :3 0
26 :3 0 2d2d 2d2e
0 2d30 1afa 2d32
1f1 :4 0 2d30 :4 0
2e13 24d :3 0 9c
:4 0 2d36 :2 0 2d3c
2d34 0 21 :3 0
e9 :3 0 2d37 2d38
0 ea :3 0 2d39
2d3a 0 2d3c 1b09
2d8f eb :3 0 21
:3 0 e9 :3 0 2d3e
2d3f 0 ea :3 0
2d40 2d41 0 2d43
1b0c 2d45 1b0e 2d44
2d43 :2 0 2d8d ec
:3 0 21 :3 0 e9
:3 0 2d48 2d49 0
ea :3 0 2d4a 2d4b
0 2d85 ad :3 0
ae :3 0 2d4d 2d4e
0 ad :3 0 bd
:3 0 2d50 2d51 0
ed :2 0 2 :4 0
27a :2 0 bf :2 0
46 :2 0 1b10 2d56
2d58 :3 0 1b13 2d52
2d5a 2d4f 2d5c 2d5b
:2 0 b0 :3 0 c0
:3 0 2d5e 2d5f 0
b2 :4 0 27b :4 0
b5 :2 0 b6 :3 0
1b17 2d63 2d65 :3 0
b5 :2 0 c9 :4 0
1b1a 2d67 2d69 :3 0
b5 :2 0 f0 :4 0
1b1d 2d6b 2d6d :3 0
b5 :2 0 ea :3 0
1b20 2d6f 2d71 :3 0
b5 :2 0 f1 :4 0
1b23 2d73 2d75 :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 1b26 2d78
2d7c 1b2a 2d77 2d7e
:3 0 1b2d 2d60 2d80
:2 0 2d82 1b30 2d83
2d5d 2d82 0 2d84
1b32 0 2d85 1b34
2d88 :3 0 2d88 0
2d88 2d87 2d85 2d86
:6 0 2d8a 53 :3 0
1b37 2d8c 1b39 2d8b
2d8a :2 0 2d8d 1b3b
:2 0 2d8f 0 2d8f
2d8e 2d3c 2d8d :6 0
2e13 1 :3 0 85
:3 0 c3 :2 0 4d
:2 0 1b40 2d92 2d94
:3 0 a7 :3 0 a0
:3 0 37 :3 0 a2
:3 0 2e :3 0 a3
:3 0 38 :3 0 a4
:3 0 30 :3 0 a5
:3 0 31 :3 0 a6
:3 0 32 :3 0 9d
:3 0 3a :4 0 27c
1 :8 0 2dbb 1b6
:4 0 2da6 :3 0 d0
:3 0 d2 :2 0 1b45
2da9 2daa :3 0 21
:3 0 e9 :3 0 2dac
2dad 0 4d :2 0
2dae 2daf 0 2db1
1b48 2db8 21 :3 0
e9 :3 0 2db2 2db3
0 180 :2 0 2db4
2db5 0 2db7 1b4a
2db9 2dab 2db1 0
2dba 0 2db7 0
2dba 1b4c 0 2dbb
1b4f 2e0e eb :3 0
21 :3 0 e9 :3 0
2dbd 2dbe 0 ea
:3 0 2dbf 2dc0 0
2dc2 1b52 2dc4 1b54
2dc3 2dc2 :2 0 2e0c
ec :3 0 21 :3 0
e9 :3 0 2dc7 2dc8
0 ea :3 0 2dc9
2dca 0 2e04 ad
:3 0 ae :3 0 2dcc
2dcd 0 ad :3 0
bd :3 0 2dcf 2dd0
0 ed :2 0 2
:4 0 27d :2 0 bf
:2 0 46 :2 0 1b56
2dd5 2dd7 :3 0 1b59
2dd1 2dd9 2dce 2ddb
2dda :2 0 b0 :3 0
c0 :3 0 2ddd 2dde
0 b2 :4 0 27e
:4 0 b5 :2 0 b6
:3 0 1b5d 2de2 2de4
:3 0 b5 :2 0 c9
:4 0 1b60 2de6 2de8
:3 0 b5 :2 0 f0
:4 0 1b63 2dea 2dec
:3 0 b5 :2 0 ea
:3 0 1b66 2dee 2df0
:3 0 b5 :2 0 f1
:4 0 1b69 2df2 2df4
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 1b6c
2df7 2dfb 1b70 2df6
2dfd :3 0 1b73 2ddf
2dff :2 0 2e01 1b76
2e02 2ddc 2e01 0
2e03 1b78 0 2e04
1b7a 2e07 :3 0 2e07
0 2e07 2e06 2e04
2e05 :6 0 2e09 55
:3 0 1b7d 2e0b 1b7f
2e0a 2e09 :2 0 2e0c
1b81 :2 0 2e0e 0
2e0e 2e0d 2dbb 2e0c
:6 0 2e10 1 :3 0
1b84 2e11 2d95 2e10
0 2e12 1b86 0
2e13 1b88 2e14 2acd
2e13 0 2e15 1b91
0 35c4 6 :3 0
e4 :2 0 1b93 2e17
2e18 :3 0 2e19 :2 0
a1 :3 0 56 :3 0
4d :2 0 1b95 2e1b
2e1e d2 :2 0 4d
:2 0 1b9a 2e20 2e22
:3 0 a1 :3 0 27f
:3 0 280 :3 0 58
:3 0 281 :3 0 10d
:3 0 25 :3 0 282
:3 0 283 :3 0 f2
:3 0 41 :3 0 283
:3 0 f2 :3 0 42
:4 0 284 1 :8 0
2e38 21 :3 0 e9
:3 0 2e33 2e34 0
ea :3 0 2e35 2e36
0 2e38 1b9d 2e8b
eb :3 0 21 :3 0
e9 :3 0 2e3a 2e3b
0 ea :3 0 2e3c
2e3d 0 2e3f 1ba0
2e41 1ba2 2e40 2e3f
:2 0 2e89 ec :3 0
21 :3 0 e9 :3 0
2e44 2e45 0 ea
:3 0 2e46 2e47 0
2e81 ad :3 0 ae
:3 0 2e49 2e4a 0
ad :3 0 bd :3 0
2e4c 2e4d 0 ed
:2 0 2 :4 0 285
:2 0 bf :2 0 46
:2 0 1ba4 2e52 2e54
:3 0 1ba7 2e4e 2e56
2e4b 2e58 2e57 :2 0
b0 :3 0 c0 :3 0
2e5a 2e5b 0 b2
:4 0 286 :4 0 b5
:2 0 b6 :3 0 1bab
2e5f 2e61 :3 0 b5
:2 0 c9 :4 0 1bae
2e63 2e65 :3 0 b5
:2 0 f0 :4 0 1bb1
2e67 2e69 :3 0 b5
:2 0 ea :3 0 1bb4
2e6b 2e6d :3 0 b5
:2 0 f1 :4 0 1bb7
2e6f 2e71 :3 0 b5
:2 0 f2 :3 0 f3
:3 0 46 :2 0 f4
:2 0 1bba 2e74 2e78
1bbe 2e73 2e7a :3 0
1bc1 2e5c 2e7c :2 0
2e7e 1bc4 2e7f 2e59
2e7e 0 2e80 1bc6
0 2e81 1bc8 2e84
:3 0 2e84 0 2e84
2e83 2e81 2e82 :6 0
2e86 57 :3 0 1bcb
2e88 1bcd 2e87 2e86
:2 0 2e89 1bcf :2 0
2e8b 0 2e8b 2e8a
2e38 2e89 :6 0 2e8d
1 :3 0 1bd2 2e93
60 :3 0 4d :2 0
2e8e 2e8f 0 2e92
0 2e92 1bd4 2e94
2e23 2e8d 0 2e95
0 2e92 0 2e95
1bd7 0 2e96 1bda
2e97 2e1a 2e96 0
2e98 1bdc 0 35c4
287 :3 0 e5 :3 0
288 :3 0 289 :3 0
1a4 :3 0 6b :3 0
68 :3 0 10c :3 0
28a :3 0 1e0 :3 0
28b :3 0 289 :3 0
28a :3 0 287 :3 0
289 :3 0 10d :3 0
28a :3 0 10d :3 0
25 :4 0 28c 1
:8 0 2eb2 21 :3 0
e9 :3 0 2ead 2eae
0 ea :3 0 2eaf
2eb0 0 2eb2 1bde
2f05 eb :3 0 21
:3 0 e9 :3 0 2eb4
2eb5 0 ea :3 0
2eb6 2eb7 0 2eb9
1be1 2ebb 1be3 2eba
2eb9 :2 0 2f03 ec
:3 0 21 :3 0 e9
:3 0 2ebe 2ebf 0
ea :3 0 2ec0 2ec1
0 2efb ad :3 0
ae :3 0 2ec3 2ec4
0 ad :3 0 bd
:3 0 2ec6 2ec7 0
ed :2 0 2 :4 0
28d :2 0 bf :2 0
46 :2 0 1be5 2ecc
2ece :3 0 1be8 2ec8
2ed0 2ec5 2ed2 2ed1
:2 0 b0 :3 0 c0
:3 0 2ed4 2ed5 0
b2 :4 0 28e :4 0
b5 :2 0 b6 :3 0
1bec 2ed9 2edb :3 0
b5 :2 0 c9 :4 0
1bef 2edd 2edf :3 0
b5 :2 0 f0 :4 0
1bf2 2ee1 2ee3 :3 0
b5 :2 0 ea :3 0
1bf5 2ee5 2ee7 :3 0
b5 :2 0 f1 :4 0
1bf8 2ee9 2eeb :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 1bfb 2eee
2ef2 1bff 2eed 2ef4
:3 0 1c02 2ed6 2ef6
:2 0 2ef8 1c05 2ef9
2ed3 2ef8 0 2efa
1c07 0 2efb 1c09
2efe :3 0 2efe 0
2efe 2efd 2efb 2efc
:6 0 2f00 59 :3 0
1c0c 2f02 1c0e 2f01
2f00 :2 0 2f03 1c10
:2 0 2f05 0 2f05
2f04 2eb2 2f03 :6 0
35c4 1 :3 0 a1
:3 0 5b :3 0 4d
:2 0 1c13 2f07 2f0a
c2 :3 0 d2 :2 0
d3 :3 0 2f0c 2f0e
0 1c18 2f0d 2f10
:3 0 1d8 :3 0 1d9
:3 0 28f :4 0 2f14
:4 0 2f15 :2 0 318b
21 :3 0 e9 :3 0
2f17 2f18 0 4d
:2 0 2f19 2f1a 0
318b 290 :3 0 9e
:3 0 291 :3 0 292
:3 0 293 :3 0 294
:3 0 a7 :3 0 9d
:3 0 27 :3 0 291
:3 0 292 :4 0 295
1 :8 0 2f3d 1b6
:4 0 2f28 :3 0 d0
:3 0 d2 :2 0 1c1d
2f2b 2f2c :3 0 21
:3 0 e9 :3 0 2f2e
2f2f 0 4d :2 0
2f30 2f31 0 2f33
1c20 2f3a 21 :3 0
e9 :3 0 2f34 2f35
0 180 :2 0 2f36
2f37 0 2f39 1c22
2f3b 2f2d 2f33 0
2f3c 0 2f39 0
2f3c 1c24 0 2f3d
1c27 2f90 eb :3 0
21 :3 0 e9 :3 0
2f3f 2f40 0 ea
:3 0 2f41 2f42 0
2f44 1c2a 2f46 1c2c
2f45 2f44 :2 0 2f8e
ec :3 0 21 :3 0
e9 :3 0 2f49 2f4a
0 ea :3 0 2f4b
2f4c 0 2f86 ad
:3 0 ae :3 0 2f4e
2f4f 0 ad :3 0
bd :3 0 2f51 2f52
0 ed :2 0 2
:4 0 296 :2 0 bf
:2 0 46 :2 0 1c2e
2f57 2f59 :3 0 1c31
2f53 2f5b 2f50 2f5d
2f5c :2 0 b0 :3 0
c0 :3 0 2f5f 2f60
0 b2 :4 0 297
:4 0 b5 :2 0 b6
:3 0 1c35 2f64 2f66
:3 0 b5 :2 0 c9
:4 0 1c38 2f68 2f6a
:3 0 b5 :2 0 f0
:4 0 1c3b 2f6c 2f6e
:3 0 b5 :2 0 ea
:3 0 1c3e 2f70 2f72
:3 0 b5 :2 0 f1
:4 0 1c41 2f74 2f76
:3 0 b5 :2 0 f2
:3 0 f3 :3 0 46
:2 0 f4 :2 0 1c44
2f79 2f7d 1c48 2f78
2f7f :3 0 1c4b 2f61
2f81 :2 0 2f83 1c4e
2f84 2f5e 2f83 0
2f85 1c50 0 2f86
1c52 2f89 :3 0 2f89
0 2f89 2f88 2f86
2f87 :6 0 2f8b 5b
:3 0 1c55 2f8d 1c57
2f8c 2f8b :2 0 2f8e
1c59 :2 0 2f90 0
2f90 2f8f 2f3d 2f8e
:6 0 318b 1 :3 0
21 :3 0 e9 :3 0
2f92 2f93 0 21
:3 0 d2 :2 0 166
:3 0 2f95 2f97 0
1c5e 2f96 2f99 :3 0
2f9a :2 0 290 :3 0
9e :3 0 19d :3 0
291 :3 0 291 :3 0
19d :3 0 292 :3 0
292 :3 0 19d :3 0
293 :3 0 293 :3 0
19d :3 0 294 :3 0
294 :3 0 a7 :3 0
9d :3 0 27 :3 0
9e :4 0 298 1
:8 0 2fc4 1b6 :4 0
2faf :3 0 d0 :3 0
d2 :2 0 1c63 2fb2
2fb3 :3 0 21 :3 0
e9 :3 0 2fb5 2fb6
0 4d :2 0 2fb7
2fb8 0 2fba 1c66
2fc1 21 :3 0 e9
:3 0 2fbb 2fbc 0
180 :2 0 2fbd 2fbe
0 2fc0 1c68 2fc2
2fb4 2fba 0 2fc3
0 2fc0 0 2fc3
1c6a 0 2fc4 1c6d
3017 eb :3 0 21
:3 0 e9 :3 0 2fc6
2fc7 0 ea :3 0
2fc8 2fc9 0 2fcb
1c70 2fcd 1c72 2fcc
2fcb :2 0 3015 ec
:3 0 21 :3 0 e9
:3 0 2fd0 2fd1 0
ea :3 0 2fd2 2fd3
0 300d ad :3 0
ae :3 0 2fd5 2fd6
0 ad :3 0 bd
:3 0 2fd8 2fd9 0
ed :2 0 2 :4 0
299 :2 0 bf :2 0
46 :2 0 1c74 2fde
2fe0 :3 0 1c77 2fda
2fe2 2fd7 2fe4 2fe3
:2 0 b0 :3 0 c0
:3 0 2fe6 2fe7 0
b2 :4 0 29a :4 0
b5 :2 0 b6 :3 0
1c7b 2feb 2fed :3 0
b5 :2 0 c9 :4 0
1c7e 2fef 2ff1 :3 0
b5 :2 0 f0 :4 0
1c81 2ff3 2ff5 :3 0
b5 :2 0 ea :3 0
1c84 2ff7 2ff9 :3 0
b5 :2 0 f1 :4 0
1c87 2ffb 2ffd :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 1c8a 3000
3004 1c8e 2fff 3006
:3 0 1c91 2fe8 3008
:2 0 300a 1c94 300b
2fe5 300a 0 300c
1c96 0 300d 1c98
3010 :3 0 3010 0
3010 300f 300d 300e
:6 0 3012 5d :3 0
1c9b 3014 1c9d 3013
3012 :2 0 3015 1c9f
:2 0 3017 0 3017
3016 2fc4 3015 :6 0
3097 1 :3 0 21
:3 0 e9 :3 0 3019
301a 0 21 :3 0
d2 :2 0 166 :3 0
301c 301e 0 1ca4
301d 3020 :3 0 3021
:2 0 290 :3 0 9e
:3 0 a7 :3 0 9d
:3 0 27 :3 0 9e
:4 0 29b 1 :8 0
303f 1b6 :4 0 302a
:3 0 d0 :3 0 d2
:2 0 1ca9 302d 302e
:3 0 21 :3 0 e9
:3 0 3030 3031 0
4d :2 0 3032 3033
0 3035 1cac 303c
21 :3 0 e9 :3 0
3036 3037 0 180
:2 0 3038 3039 0
303b 1cae 303d 302f
3035 0 303e 0
303b 0 303e 1cb0
0 303f 1cb3 3092
eb :3 0 21 :3 0
e9 :3 0 3041 3042
0 ea :3 0 3043
3044 0 3046 1cb6
3048 1cb8 3047 3046
:2 0 3090 ec :3 0
21 :3 0 e9 :3 0
304b 304c 0 ea
:3 0 304d 304e 0
3088 ad :3 0 ae
:3 0 3050 3051 0
ad :3 0 bd :3 0
3053 3054 0 ed
:2 0 2 :4 0 29c
:2 0 bf :2 0 46
:2 0 1cba 3059 305b
:3 0 1cbd 3055 305d
3052 305f 305e :2 0
b0 :3 0 c0 :3 0
3061 3062 0 b2
:4 0 29d :4 0 b5
:2 0 b6 :3 0 1cc1
3066 3068 :3 0 b5
:2 0 c9 :4 0 1cc4
306a 306c :3 0 b5
:2 0 f0 :4 0 1cc7
306e 3070 :3 0 b5
:2 0 ea :3 0 1cca
3072 3074 :3 0 b5
:2 0 f1 :4 0 1ccd
3076 3078 :3 0 b5
:2 0 f2 :3 0 f3
:3 0 46 :2 0 f4
:2 0 1cd0 307b 307f
1cd4 307a 3081 :3 0
1cd7 3063 3083 :2 0
3085 1cda 3086 3060
3085 0 3087 1cdc
0 3088 1cde 308b
:3 0 308b 0 308b
308a 3088 3089 :6 0
308d 5f :3 0 1ce1
308f 1ce3 308e 308d
:2 0 3090 1ce5 :2 0
3092 0 3092 3091
303f 3090 :6 0 3094
1 :3 0 1ce8 3095
3022 3094 0 3096
1cea 0 3097 1cec
3098 2f9b 3097 0
3099 1cef 0 318b
24c :3 0 a9 :3 0
39 :3 0 10d :3 0
145 :3 0 1df :3 0
1b1 :3 0 1a4 :3 0
29e :3 0 a2 :3 0
29e :3 0 a4 :3 0
29e :3 0 a5 :3 0
29e :3 0 a6 :3 0
53 :3 0 29f :3 0
2a0 :3 0 1a4 :3 0
2a1 :3 0 2a2 :3 0
1a4 :3 0 2a3 :3 0
2a4 :3 0 2a5 :3 0
291 :3 0 2a5 :3 0
291 :3 0 29e :3 0
2a6 :3 0 2a7 :3 0
291 :3 0 68 :3 0
139 :3 0 291 :3 0
68 :3 0 291 :3 0
2a4 :3 0 2a5 :3 0
2a8 :3 0 2a5 :3 0
2a8 :3 0 29e :3 0
2a6 :3 0 2a7 :3 0
292 :3 0 68 :3 0
139 :3 0 292 :3 0
68 :3 0 292 :3 0
2a4 :3 0 2a5 :3 0
293 :3 0 2a5 :3 0
293 :3 0 29e :3 0
2a6 :3 0 2a7 :3 0
293 :3 0 68 :3 0
139 :3 0 293 :3 0
68 :3 0 293 :3 0
2a4 :3 0 2a5 :3 0
2a9 :3 0 2a5 :3 0
2a9 :3 0 29e :3 0
2a6 :3 0 2a7 :3 0
294 :3 0 68 :3 0
139 :3 0 294 :3 0
68 :3 0 294 :3 0
29e :3 0 2aa :3 0
29e :3 0 2ab :3 0
29e :3 0 2ac :3 0
29e :3 0 2ad :3 0
29e :3 0 2ae :3 0
29e :3 0 2af :3 0
29e :3 0 2af :3 0
58 :3 0 2b0 :3 0
59 :3 0 2b1 :3 0
29e :3 0 2ac :3 0
29e :3 0 2ad :3 0
29e :3 0 2ae :3 0
2b2 :3 0 2b3 :3 0
2b4 :3 0 2b5 :3 0
2b6 :3 0 1a4 :3 0
2b7 :3 0 2b8 :3 0
2b9 :3 0 5e :3 0
12f :3 0 29e :3 0
2ba :3 0 2bb :3 0
2e :3 0 a0 :3 0
30 :3 0 a3 :3 0
2bc :3 0 2bd :3 0
150 :3 0 2be :3 0
290 :3 0 2a7 :3 0
1e0 :3 0 a7 :3 0
29e :3 0 29e :3 0
9e :3 0 2a7 :3 0
9e :3 0 2bf :3 0
2c0 :3 0 1e0 :3 0
2c1 :3 0 2a5 :3 0
2a5 :3 0 98 :3 0
25 :3 0 2a5 :3 0
9a :3 0 29e :3 0
9e :3 0 1e0 :3 0
190 :3 0 1df :3 0
1df :3 0 191 :3 0
39 :3 0 1e0 :3 0
1a6 :3 0 2c2 :3 0
1df :3 0 a0 :3 0
2c2 :3 0 10d :3 0
2bf :3 0 2c0 :3 0
1e0 :3 0 2c3 :3 0
139 :3 0 2c2 :3 0
2c4 :3 0 139 :3 0
2c4 :3 0 2bf :3 0
2c0 :3 0 1e0 :3 0
21d :3 0 2a0 :3 0
1df :3 0 1b4 :3 0
2a0 :3 0 10d :3 0
2bf :3 0 2c0 :3 0
1e0 :3 0 228 :3 0
2a2 :3 0 1df :3 0
1b3 :3 0 2a2 :3 0
10d :3 0 2bf :3 0
2c0 :3 0 1e0 :3 0
178 :3 0 2b6 :3 0
2b6 :3 0 10d :3 0
1df :3 0 1b2 :3 0
29e :3 0 a2 :3 0
29e :3 0 a4 :3 0
29e :3 0 a5 :3 0
29e :3 0 a6 :4 0
2c5 1 :a 0 3168
:2 0 318b 309b 3166
:2 0 ad :3 0 ae
:3 0 3169 316a 0
ad :3 0 bd :3 0
316c 316d 0 46
:2 0 2 :4 0 2c6
:2 0 bf :2 0 46
:2 0 1cf1 3172 3174
:3 0 1cf4 316e 3176
316b 3178 3177 :2 0
b0 :3 0 c0 :3 0
317a 317b 0 b2
:4 0 2c7 :4 0 b5
:2 0 b6 :3 0 1cf8
317f 3181 :3 0 1cfb
317c 3183 :2 0 3185
1cfe 3186 3179 3185
0 3187 1d00 0
318b 20 :3 0 a9
:3 0 3189 :2 0 318b
1d02 35c1 8b :3 0
d2 :2 0 46 :2 0
1d0c 318d 318f :3 0
24c :3 0 aa :3 0
29e :3 0 9e :3 0
10d :3 0 145 :3 0
2c2 :3 0 2b5 :3 0
1a4 :3 0 29e :3 0
a2 :3 0 29e :3 0
a4 :3 0 29e :3 0
a5 :3 0 29e :3 0
a6 :3 0 2c2 :3 0
29f :3 0 2c8 :3 0
1a4 :3 0 2a1 :3 0
2c9 :3 0 1a4 :3 0
2a3 :3 0 2a4 :3 0
29e :3 0 2ca :3 0
29e :3 0 291 :3 0
2c2 :3 0 287 :3 0
6b :3 0 2c2 :3 0
287 :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 2c2 :3 0
287 :3 0 29e :3 0
2ca :3 0 29e :3 0
291 :3 0 29e :3 0
291 :3 0 68 :3 0
291 :3 0 2a4 :3 0
29e :3 0 2ca :3 0
29e :3 0 292 :3 0
2c2 :3 0 287 :3 0
6b :3 0 2c2 :3 0
287 :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 2c2 :3 0
287 :3 0 29e :3 0
2ca :3 0 29e :3 0
292 :3 0 29e :3 0
292 :3 0 68 :3 0
292 :3 0 2a4 :3 0
29e :3 0 2ca :3 0
29e :3 0 293 :3 0
2c2 :3 0 287 :3 0
6b :3 0 2c2 :3 0
287 :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 2c2 :3 0
287 :3 0 29e :3 0
2ca :3 0 29e :3 0
293 :3 0 29e :3 0
293 :3 0 68 :3 0
293 :3 0 2a4 :3 0
29e :3 0 2ca :3 0
29e :3 0 294 :3 0
2c2 :3 0 287 :3 0
6b :3 0 2c2 :3 0
287 :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 2c2 :3 0
287 :3 0 29e :3 0
2ca :3 0 29e :3 0
294 :3 0 29e :3 0
294 :3 0 68 :3 0
294 :3 0 29e :3 0
2aa :3 0 29e :3 0
2ab :3 0 29e :3 0
2ac :3 0 29e :3 0
2ad :3 0 29e :3 0
2ae :3 0 29e :3 0
2af :3 0 29e :3 0
2af :3 0 58 :3 0
2b0 :3 0 59 :3 0
2b1 :3 0 29e :3 0
2ac :3 0 29e :3 0
2ad :3 0 29e :3 0
2ae :3 0 2b2 :3 0
2cb :3 0 2b5 :3 0
2b3 :3 0 2c2 :3 0
2b4 :3 0 2b6 :3 0
1a4 :3 0 2b7 :3 0
2cc :3 0 2b5 :3 0
2cc :3 0 2b8 :3 0
2b9 :3 0 5e :3 0
12f :3 0 29e :3 0
2ba :3 0 2bb :3 0
29e :3 0 a0 :3 0
29e :3 0 a3 :3 0
a1 :3 0 2a5 :3 0
2cd :3 0 2a5 :3 0
2ce :3 0 2bc :3 0
2c2 :3 0 2bd :3 0
2c2 :3 0 2cf :3 0
2be :3 0 a7 :3 0
29e :3 0 1e0 :3 0
178 :3 0 1df :3 0
29e :3 0 9e :3 0
1df :3 0 10d :3 0
1e0 :3 0 178 :3 0
2b6 :3 0 2b6 :3 0
10d :3 0 1df :3 0
1a7 :3 0 2bf :3 0
2c0 :3 0 1e0 :3 0
162 :3 0 2cc :3 0
2b6 :3 0 2d0 :3 0
2cc :3 0 9a :3 0
1e0 :3 0 162 :3 0
2c2 :3 0 29e :3 0
9e :3 0 2c2 :3 0
9a :3 0 2bf :3 0
2c0 :3 0 1e0 :3 0
162 :3 0 2cb :3 0
2cb :3 0 9a :3 0
2c2 :3 0 2d1 :3 0
2bf :3 0 2c0 :3 0
1e0 :3 0 2c1 :3 0
2a5 :3 0 2a5 :3 0
98 :3 0 25 :3 0
2a5 :3 0 9a :3 0
29e :3 0 9e :3 0
2bf :3 0 2c0 :3 0
1e0 :3 0 21d :3 0
2a0 :3 0 2c2 :3 0
9a :3 0 2a0 :3 0
10d :3 0 2bf :3 0
2c0 :3 0 1e0 :3 0
21d :3 0 2c8 :3 0
2a0 :3 0 1a7 :3 0
2c8 :3 0 10d :3 0
2bf :3 0 2c0 :3 0
1e0 :3 0 228 :3 0
2a2 :3 0 2c2 :3 0
9a :3 0 2a2 :3 0
10d :3 0 2bf :3 0
2c0 :3 0 1e0 :3 0
228 :3 0 2c9 :3 0
2a2 :3 0 1a7 :3 0
2c9 :3 0 10d :3 0
10d :4 0 2d2 1
:a 0 32c0 :2 0 32e3
3192 32be :2 0 ad
:3 0 ae :3 0 32c1
32c2 0 ad :3 0
bd :3 0 32c4 32c5
0 46 :2 0 2
:4 0 2d3 :2 0 bf
:2 0 46 :2 0 1d0f
32ca 32cc :3 0 1d12
32c6 32ce 32c3 32d0
32cf :2 0 b0 :3 0
c0 :3 0 32d2 32d3
0 b2 :4 0 2d4
:4 0 b5 :2 0 b6
:3 0 1d16 32d7 32d9
:3 0 1d19 32d4 32db
:2 0 32dd 1d1c 32de
32d1 32dd 0 32df
1d1e 0 32e3 20
:3 0 aa :3 0 32e1
:2 0 32e3 1d20 35bd
1d8 :3 0 1d9 :3 0
2d5 :4 0 32e6 :4 0
32e7 :2 0 35bc 21
:3 0 e9 :3 0 32e9
32ea 0 4d :2 0
32eb 32ec 0 35bc
2d6 :3 0 9e :3 0
291 :3 0 292 :3 0
293 :3 0 294 :3 0
a7 :3 0 2ca :4 0
2d7 1 :8 0 330c
1b6 :4 0 32f7 :3 0
d0 :3 0 d2 :2 0
1d26 32fa 32fb :3 0
21 :3 0 e9 :3 0
32fd 32fe 0 4d
:2 0 32ff 3300 0
3302 1d29 3309 21
:3 0 e9 :3 0 3303
3304 0 180 :2 0
3305 3306 0 3308
1d2b 330a 32fc 3302
0 330b 0 3308
0 330b 1d2d 0
330c 1d30 335f eb
:3 0 21 :3 0 e9
:3 0 330e 330f 0
ea :3 0 3310 3311
0 3313 1d33 3315
1d35 3314 3313 :2 0
335d ec :3 0 21
:3 0 e9 :3 0 3318
3319 0 ea :3 0
331a 331b 0 3355
ad :3 0 ae :3 0
331d 331e 0 ad
:3 0 bd :3 0 3320
3321 0 ed :2 0
2 :4 0 2d8 :2 0
bf :2 0 46 :2 0
1d37 3326 3328 :3 0
1d3a 3322 332a 331f
332c 332b :2 0 b0
:3 0 c0 :3 0 332e
332f 0 b2 :4 0
2d9 :4 0 b5 :2 0
b6 :3 0 1d3e 3333
3335 :3 0 b5 :2 0
c9 :4 0 1d41 3337
3339 :3 0 b5 :2 0
f0 :4 0 1d44 333b
333d :3 0 b5 :2 0
ea :3 0 1d47 333f
3341 :3 0 b5 :2 0
f1 :4 0 1d4a 3343
3345 :3 0 b5 :2 0
f2 :3 0 f3 :3 0
46 :2 0 f4 :2 0
1d4d 3348 334c 1d51
3347 334e :3 0 1d54
3330 3350 :2 0 3352
1d57 3353 332d 3352
0 3354 1d59 0
3355 1d5b 3358 :3 0
3358 0 3358 3357
3355 3356 :6 0 335a
61 :3 0 1d5e 335c
1d60 335b 335a :2 0
335d 1d62 :2 0 335f
0 335f 335e 330c
335d :6 0 35bc 1
:3 0 2d6 :3 0 9e
:3 0 19d :3 0 291
:3 0 291 :3 0 19d
:3 0 292 :3 0 292
:3 0 19d :3 0 293
:3 0 293 :3 0 19d
:3 0 294 :3 0 294
:3 0 a7 :3 0 9e
:3 0 9e :3 0 a7
:3 0 2ca :3 0 9e
:4 0 2da 1 :8 0
338b 1b6 :4 0 3376
:3 0 d0 :3 0 d2
:2 0 1d67 3379 337a
:3 0 21 :3 0 e9
:3 0 337c 337d 0
4d :2 0 337e 337f
0 3381 1d6a 3388
21 :3 0 e9 :3 0
3382 3383 0 180
:2 0 3384 3385 0
3387 1d6c 3389 337b
3381 0 338a 0
3387 0 338a 1d6e
0 338b 1d71 33de
eb :3 0 21 :3 0
e9 :3 0 338d 338e
0 ea :3 0 338f
3390 0 3392 1d74
3394 1d76 3393 3392
:2 0 33dc ec :3 0
21 :3 0 e9 :3 0
3397 3398 0 ea
:3 0 3399 339a 0
33d4 ad :3 0 ae
:3 0 339c 339d 0
ad :3 0 bd :3 0
339f 33a0 0 ed
:2 0 2 :4 0 2db
:2 0 bf :2 0 46
:2 0 1d78 33a5 33a7
:3 0 1d7b 33a1 33a9
339e 33ab 33aa :2 0
b0 :3 0 c0 :3 0
33ad 33ae 0 b2
:4 0 2dc :4 0 b5
:2 0 b6 :3 0 1d7f
33b2 33b4 :3 0 b5
:2 0 c9 :4 0 1d82
33b6 33b8 :3 0 b5
:2 0 f0 :4 0 1d85
33ba 33bc :3 0 b5
:2 0 ea :3 0 1d88
33be 33c0 :3 0 b5
:2 0 f1 :4 0 1d8b
33c2 33c4 :3 0 b5
:2 0 f2 :3 0 f3
:3 0 46 :2 0 f4
:2 0 1d8e 33c7 33cb
1d92 33c6 33cd :3 0
1d95 33af 33cf :2 0
33d1 1d98 33d2 33ac
33d1 0 33d3 1d9a
0 33d4 1d9c 33d7
:3 0 33d7 0 33d7
33d6 33d4 33d5 :6 0
33d9 63 :3 0 1d9f
33db 1da1 33da 33d9
:2 0 33dc 1da3 :2 0
33de 0 33de 33dd
338b 33dc :6 0 35bc
1 :3 0 24c :3 0
ab :3 0 29e :3 0
9e :3 0 10d :3 0
145 :3 0 2c2 :3 0
2b5 :3 0 1a4 :3 0
29e :3 0 a2 :3 0
29e :3 0 a4 :3 0
29e :3 0 a5 :3 0
29e :3 0 a6 :3 0
2c2 :3 0 29f :3 0
2c8 :3 0 1a4 :3 0
2a1 :3 0 2c9 :3 0
1a4 :3 0 2a3 :3 0
2a4 :3 0 2a5 :3 0
291 :3 0 2a5 :3 0
291 :3 0 139 :3 0
291 :3 0 139 :3 0
291 :3 0 2c2 :3 0
287 :3 0 6b :3 0
2c2 :3 0 287 :3 0
6b :3 0 e5 :3 0
288 :3 0 1a4 :3 0
28b :3 0 10d :3 0
6b :3 0 e5 :3 0
288 :3 0 1a4 :3 0
28b :3 0 10d :3 0
2c2 :3 0 287 :3 0
2dd :3 0 291 :3 0
2dd :3 0 291 :3 0
2c2 :3 0 287 :3 0
6b :3 0 2c2 :3 0
287 :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 2c2 :3 0
287 :3 0 2de :3 0
291 :3 0 68 :3 0
291 :3 0 2a4 :3 0
2a5 :3 0 291 :3 0
2a5 :3 0 2a8 :3 0
139 :3 0 291 :3 0
139 :3 0 292 :3 0
2c2 :3 0 287 :3 0
6b :3 0 2c2 :3 0
287 :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 2c2 :3 0
287 :3 0 2dd :3 0
291 :3 0 2dd :3 0
292 :3 0 2c2 :3 0
287 :3 0 6b :3 0
2c2 :3 0 287 :3 0
6b :3 0 e5 :3 0
288 :3 0 1a4 :3 0
28b :3 0 10d :3 0
6b :3 0 e5 :3 0
288 :3 0 1a4 :3 0
28b :3 0 10d :3 0
2c2 :3 0 287 :3 0
2de :3 0 292 :3 0
68 :3 0 292 :3 0
2a4 :3 0 2a5 :3 0
291 :3 0 2a5 :3 0
293 :3 0 139 :3 0
291 :3 0 139 :3 0
293 :3 0 2c2 :3 0
287 :3 0 6b :3 0
2c2 :3 0 287 :3 0
6b :3 0 e5 :3 0
288 :3 0 1a4 :3 0
28b :3 0 10d :3 0
6b :3 0 e5 :3 0
288 :3 0 1a4 :3 0
28b :3 0 10d :3 0
2c2 :3 0 287 :3 0
2dd :3 0 291 :3 0
2dd :3 0 293 :3 0
2c2 :3 0 287 :3 0
6b :3 0 2c2 :3 0
287 :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 2c2 :3 0
287 :3 0 2de :3 0
293 :3 0 68 :3 0
293 :3 0 2a4 :3 0
2a5 :3 0 2a8 :3 0
2a5 :3 0 2a9 :3 0
139 :3 0 292 :3 0
139 :3 0 294 :3 0
2c2 :3 0 287 :3 0
6b :3 0 2c2 :3 0
287 :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 6b :3 0
e5 :3 0 288 :3 0
1a4 :3 0 28b :3 0
10d :3 0 2c2 :3 0
287 :3 0 2dd :3 0
291 :3 0 2dd :3 0
294 :3 0 2c2 :3 0
287 :3 0 6b :3 0
2c2 :3 0 287 :3 0
6b :3 0 e5 :3 0
288 :3 0 1a4 :3 0
28b :3 0 10d :3 0
6b :3 0 e5 :3 0
288 :3 0 1a4 :3 0
28b :3 0 10d :3 0
2c2 :3 0 287 :3 0
2de :3 0 294 :3 0
68 :3 0 294 :3 0
29e :3 0 2aa :3 0
29e :3 0 2ab :3 0
29e :3 0 2ac :3 0
29e :3 0 2ad :3 0
29e :3 0 2ae :3 0
29e :3 0 2af :3 0
29e :3 0 2af :3 0
58 :3 0 2b0 :3 0
59 :3 0 2b1 :3 0
29e :3 0 2ac :3 0
29e :3 0 2ad :3 0
29e :3 0 2ae :3 0
2b2 :3 0 2cb :3 0
2b5 :3 0 2b3 :3 0
2c2 :3 0 2b4 :3 0
2b6 :3 0 1a4 :3 0
2b7 :3 0 2cc :3 0
2b5 :3 0 2cc :3 0
2b8 :3 0 2b9 :3 0
5e :3 0 12f :3 0
29e :3 0 2ba :3 0
2bb :3 0 29e :3 0
a0 :3 0 29e :3 0
a3 :3 0 a1 :3 0
2a5 :3 0 2cd :3 0
2a5 :3 0 2ce :3 0
2bc :3 0 2c2 :3 0
2bd :3 0 2c2 :3 0
2cf :3 0 2be :3 0
a7 :3 0 29e :3 0
2bf :3 0 1e0 :3 0
2d6 :3 0 2dd :3 0
29e :3 0 9e :3 0
2dd :3 0 9e :3 0
2dd :3 0 2ca :3 0
2bf :3 0 1e0 :3 0
2d6 :3 0 2de :3 0
29e :3 0 9e :3 0
2de :3 0 9e :3 0
2de :3 0 2ca :3 0
1e0 :3 0 178 :3 0
1df :3 0 29e :3 0
9e :3 0 1df :3 0
10d :3 0 1e0 :3 0
178 :3 0 2b6 :3 0
2b6 :3 0 10d :3 0
1df :3 0 1a7 :3 0
2bf :3 0 2c0 :3 0
1e0 :3 0 162 :3 0
2cc :3 0 2b6 :3 0
2d0 :3 0 2cc :3 0
9a :3 0 1e0 :3 0
162 :3 0 2c2 :3 0
29e :3 0 9e :3 0
2c2 :3 0 9a :3 0
2bf :3 0 2c0 :3 0
1e0 :3 0 162 :3 0
2cb :3 0 2cb :3 0
9a :3 0 2c2 :3 0
2d1 :3 0 2bf :3 0
2c0 :3 0 1e0 :3 0
2c1 :3 0 2a5 :3 0
2a5 :3 0 98 :3 0
25 :3 0 2a5 :3 0
9a :3 0 29e :3 0
9e :3 0 2bf :3 0
2c0 :3 0 1e0 :3 0
162 :3 0 2df :3 0
2c2 :3 0 2d1 :3 0
2df :3 0 9a :3 0
2bf :3 0 2c0 :3 0
1e0 :3 0 21d :3 0
2a0 :3 0 2c2 :3 0
9a :3 0 2a0 :3 0
10d :3 0 2bf :3 0
2c0 :3 0 1e0 :3 0
21d :3 0 2c8 :3 0
2a0 :3 0 1a7 :3 0
2c8 :3 0 10d :3 0
2bf :3 0 2c0 :3 0
1e0 :3 0 228 :3 0
2a2 :3 0 2c2 :3 0
9a :3 0 2a2 :3 0
10d :3 0 2bf :3 0
2c0 :3 0 1e0 :3 0
228 :3 0 2c9 :3 0
2a2 :3 0 1a7 :3 0
2c9 :3 0 10d :3 0
2bf :3 0 2c0 :3 0
1e0 :3 0 2c3 :3 0
139 :3 0 2c2 :3 0
2c4 :3 0 139 :3 0
2c4 :3 0 29e :3 0
a2 :3 0 29e :3 0
a4 :3 0 29e :3 0
a5 :3 0 29e :3 0
a6 :3 0 10d :4 0
2e0 1 :a 0 3599
:2 0 35bc 33e1 3597
:2 0 ad :3 0 ae
:3 0 359a 359b 0
ad :3 0 bd :3 0
359d 359e 0 46
:2 0 2 :4 0 2e1
:2 0 bf :2 0 46
:2 0 1da6 35a3 35a5
:3 0 1da9 359f 35a7
359c 35a9 35a8 :2 0
b0 :3 0 c0 :3 0
35ab 35ac 0 b2
:4 0 2e2 :4 0 b5
:2 0 b6 :3 0 1dad
35b0 35b2 :3 0 1db0
35ad 35b4 :2 0 35b6
1db3 35b7 35aa 35b6
0 35b8 1db5 0
35bc 20 :3 0 ab
:3 0 35ba :2 0 35bc
1db7 35be 3190 32e3
0 35bf 0 35bc
0 35bf 1dbf 0
35c0 1dc2 35c2 2f11
318b 0 35c3 0
35c0 0 35c3 1dc4
0 35c4 1dc7 3665
21 :3 0 2e3 :2 0
4 35c5 35c6 0
ad :3 0 ae :3 0
35c8 35c9 0 ad
:3 0 bd :3 0 35cb
35cc 0 ed :2 0
2 :4 0 2e4 :2 0
bf :2 0 46 :2 0
1e03 35d1 35d3 :3 0
1e06 35cd 35d5 35ca
35d7 35d6 :2 0 b0
:3 0 c0 :3 0 35d9
35da 0 b2 :4 0
2e5 :4 0 b5 :2 0
b6 :3 0 1e0a 35de
35e0 :3 0 1e0d 35db
35e2 :2 0 35fb b0
:3 0 c0 :3 0 35e4
35e5 0 b2 :4 0
2e6 :4 0 b5 :2 0
21 :3 0 2e7 :3 0
35ea 35eb 0 1e10
35e9 35ed :3 0 b5
:2 0 116 :4 0 1e13
35ef 35f1 :3 0 b5
:2 0 21 :3 0 2e8
:3 0 35f4 35f5 0
1e16 35f3 35f7 :3 0
1e19 35e6 35f9 :2 0
35fb 1e1c 35fc 35d8
35fb 0 35fd 1e1f
0 360d 24c :3 0
ac :3 0 21 :3 0
2e7 :3 0 2e9 :3 0
21 :3 0 2e8 :3 0
2ea :3 0 e7 :4 0
2eb 1 :a 0 3609
:2 0 360d 35ff 3607
:2 0 20 :3 0 ac
:3 0 360b :2 0 360d
1e21 360f 1e25 360e
360d :2 0 3663 ec
:3 0 21 :3 0 e9
:3 0 3612 3613 0
ea :3 0 3614 3615
0 365b ad :3 0
ae :3 0 3617 3618
0 ad :3 0 bd
:3 0 361a 361b 0
ed :2 0 2 :4 0
2ec :2 0 bf :2 0
46 :2 0 1e27 3620
3622 :3 0 1e2a 361c
3624 3619 3626 3625
:2 0 b0 :3 0 c0
:3 0 3628 3629 0
b2 :4 0 2ed :4 0
b5 :2 0 b6 :3 0
1e2e 362d 362f :3 0
1e31 362a 3631 :2 0
364b b0 :3 0 c0
:3 0 3633 3634 0
b2 :4 0 f0 :4 0
b5 :2 0 ea :3 0
1e34 3638 363a :3 0
b5 :2 0 f1 :4 0
1e37 363c 363e :3 0
b5 :2 0 f2 :3 0
f3 :3 0 46 :2 0
f4 :2 0 1e3a 3641
3645 1e3e 3640 3647
:3 0 1e41 3635 3649
:2 0 364b 1e44 364c
3627 364b 0 364d
1e47 0 365b 24c
:3 0 ac :3 0 2e9
:3 0 21 :3 0 2e8
:3 0 2ea :3 0 e7
:4 0 2ee 1 :a 0
3657 :2 0 365b 364f
3655 :2 0 20 :3 0
ac :3 0 3659 :2 0
365b 1e49 365e :3 0
365e 0 365e 365d
365b 365c :6 0 3660
1 :3 0 1e4e 3662
1e50 3661 3660 :2 0
3663 1e52 :2 0 3665
1e55 3665 3664 35c4
3663 :6 0 3666 :2 0
2 74 3665 3669
:3 0 3668 3666 366a
:8 0 
1ebf
4
:3 0 1 3 1
7 1 b 1
f 1 13 1
17 1 1b 1
1f 1 23 1
27 1 2b 1
2f 1 33 1
37 1 3b 1
3f 1 43 1
47 1 4b 1
4f 1 53 1
57 1 5b 1
5f 1 63 1
67 1 6b 1b
6 a e 12
16 1a 1e 22
26 2a 2e 32
36 3a 3e 42
46 4a 4e 52
56 5a 5e 62
66 6a 6e 1
78 1 76 1
7f 1 7d 1
86 1 84 1
8d 1 8b 1
94 1 92 1
9b 1 99 1
a2 1 a0 1
a9 1 a7 1
b0 1 ae 1
b7 1 b5 1
be 1 bc 1
c5 1 c3 1
cc 1 ca 1
d3 1 d1 1
da 1 d8 1
e1 1 df 1
e8 1 e6 1
ef 1 ed 1
f6 1 f4 1
fd 1 fb 1
104 1 102 1
109 1 110 1
10e 1 117 1
115 1 11e 1
11c 1 125 1
123 1 12c 1
12a 1 133 1
131 1 13a 1
138 1 141 1
13f 1 148 1
146 1 14f 1
14d 1 156 1
154 1 15b 1
161 1 167 1
16d 1 173 1
179 1 17f 1
185 1 18b 1
191 1 197 1
19d 1 1a3 1
1a9 1 1af 1
1b5 1 1bb 1
1c1 1 1c7 1
1cd 1 1d3 1
1d9 1 1df 1
1e5 1 1eb 1
1f1 1 1f7 1
1fd 1 203 1
209 1 20f 1
215 1 21b 1
223 1 221 1
228 1 22e 1
234 1 23c 1
23a 1 243 1
241 1 24a 1
248 1 24f 1
255 1 25d 1
25b 1 264 1
262 1 26b 1
269 1 272 1
270 1 279 1
277 1 280 1
27e 1 285 1
28b 1 291 1
297 1 29d 1
2a3 1 2a9 1
2af 1 2b7 1
2b5 1 2bc 1
2c2 1 2c7 1
2cc 1 2d1 1
2d6 1 2db 1
2e0 1 2e5 1
2f9 1 314 1
31a 1 321 1
328 1 32f 1
33c 1 348 2
34b 34d 2 350
352 2 354 356
2 358 35a 2
35c 35e 2 360
362 2 364 366
2 368 36a 2
36c 36e 2 370
372 2 374 376
2 378 37a 2
37c 37e 1 384
2 386 387 2
380 389 2 38b
38d 1 393 2
395 396 2 38f
398 2 39a 39c
1 3a2 2 3a4
3a5 2 39e 3a7
2 3a9 3ab 1
3b1 2 3b3 3b4
2 3ad 3b6 2
3b8 3ba 1 3c0
2 3c2 3c3 2
3bc 3c5 2 3c7
3c9 1 3cf 2
3d1 3d2 2 3cb
3d4 2 3d6 3d8
1 3de 2 3e0
3e1 2 3da 3e3
2 3e5 3e7 1
3ed 2 3ef 3f0
2 3e9 3f2 2
3f4 3f6 2 3f8
3fa 2 3fc 3fe
2 400 402 2
404 406 2 408
40a 2 40c 40e
2 410 412 2
414 416 2 418
41a 2 41c 41e
2 420 422 2
424 426 2 428
42a 2 42c 42e
2 430 432 2
434 436 2 438
43a 2 43c 43e
2 440 442 2
444 446 1 44c
2 44e 44f 2
448 451 2 453
455 2 457 459
2 45b 45d 2
45f 461 2 463
465 2 467 469
2 46b 46d 2
46f 471 2 473
475 2 477 479
2 47b 47d 2
47f 481 2 483
485 2 487 489
2 48b 48d 2
48f 491 2 493
495 1 49b 2
49d 49e 2 497
4a0 2 4a2 4a4
1 4aa 2 4ac
4ad 2 4a6 4af
2 4b1 4b3 1
4b9 2 4bb 4bc
2 4b5 4be 2
4c0 4c2 1 4c8
2 4ca 4cb 2
4c4 4cd 2 4cf
4d1 2 4d3 4d5
2 4d7 4d9 2
4db 4dd 2 4df
4e1 2 4e3 4e5
2 4e7 4e9 2
4eb 4ed 2 4ef
4f1 4 344 34a
34f 4f3 1 4f5
1 4f7 2 501
503 3 4ff 500
505 2 50e 510
2 50d 512 1
514 1 516 2
561 562 1 568
2 564 568 1
56c 2 57c 57e
3 57a 57b 580
2 589 58b 2
58d 58f 2 591
593 1 59b 2
59d 59e 2 595
5a0 2 588 5a2
1 5a4 1 5a6
2 5b0 5b2 3
5ae 5af 5b4 2
5bd 5bf 2 5bc
5c1 1 5c3 1
5c5 2 5cc 5cd
3 5a7 5c6 5cf
1 5d1 1 5d3
1 5d9 1 5e1
1 5e9 2 5f2
5f3 1 5f9 2
5f5 5f9 1 5ff
2 606 607 1
60d 2 609 60d
2 61b 61d 3
619 61a 61f 2
628 62a 2 62c
62e 2 630 632
1 63a 2 63c
63d 2 634 63f
2 627 641 1
643 1 645 2
64f 651 3 64d
64e 653 2 65c
65e 2 65b 660
1 662 1 664
2 66b 66c 3
646 665 66e 1
671 1 677 2
687 689 3 685
686 68b 2 694
696 2 698 69a
2 69c 69e 1
6a6 2 6a8 6a9
2 6a0 6ab 2
693 6ad 1 6af
1 6b1 2 6bb
6bd 3 6b9 6ba
6bf 2 6c8 6ca
2 6c7 6cc 1
6ce 1 6d0 2
6d7 6d8 3 6b2
6d1 6da 2 6dd
6dc 1 6de 1
6e0 2 6e3 6e4
1 6e8 2 6e6
6e8 2 6ec 6ed
1 6f1 2 6ef
6f1 2 6f7 6f8
1 6fc 2 6fa
6fc 2 709 70b
3 707 708 70d
2 716 718 2
71a 71c 2 71e
720 1 728 2
72a 72b 2 722
72d 2 715 72f
1 731 1 733
2 73d 73f 3
73b 73c 741 2
74a 74c 2 749
74e 1 750 1
752 2 759 75a
3 734 753 75c
1 75e 1 760
1 768 1 76c
2 76e 76f 2
772 773 1 779
2 775 779 1
77d 1 785 2
795 79a 1 7a1
1 79c 2 7b4
7b6 3 7b2 7b3
7b8 2 7c1 7c3
2 7c5 7c7 2
7c9 7cb 2 7cd
7cf 2 7d1 7d3
3 7d8 7d9 7da
2 7d5 7dc 2
7c0 7de 1 7e0
1 7e2 2 7ab
7e3 1 7e7 1
7a6 2 7a4 7eb
1 7f6 2 7f2
7f6 2 7fb 7fc
1 800 2 7fe
800 2 80e 810
3 80c 80d 812
2 81b 81d 2
81f 821 2 823
825 1 82d 2
82f 830 2 827
832 2 81a 834
1 836 1 838
2 842 844 3
840 841 846 2
84f 851 2 84e
853 1 855 1
857 2 85e 85f
3 839 858 861
1 863 3 7ee
864 867 1 869
1 86b 1 871
2 87c 881 1
888 1 883 2
89b 89d 3 899
89a 89f 2 8a8
8aa 2 8ac 8ae
2 8b0 8b2 2
8b4 8b6 2 8b8
8ba 3 8bf 8c0
8c1 2 8bc 8c3
2 8a7 8c5 1
8c7 1 8c9 2
892 8ca 1 8ce
1 88d 2 88b
8d2 1 8d5 1
8d8 2 8e3 8e4
2 8e6 8e9 2
8ec 8ed 2 8f4
8f5 2 8f7 8fa
2 904 905 2
907 90a 2 90d
90e 2 915 916
2 918 91b 2
921 922 1 928
2 924 928 1
92a 2 92d 92f
2 933 934 1
938 2 936 938
2 943 948 1
94f 1 94a 2
962 964 3 960
961 966 2 96f
971 2 973 975
2 977 979 2
97b 97d 2 97f
981 3 986 987
988 2 983 98a
2 96e 98c 1
98e 1 990 2
959 991 1 995
1 954 2 952
999 2 99f 9a0
1 9a4 2 9a2
9a4 2 9ac 9b1
1 9b8 1 9b3
2 9cb 9cd 3
9c9 9ca 9cf 2
9d8 9da 2 9dc
9de 2 9e0 9e2
2 9e4 9e6 2
9e8 9ea 3 9ef
9f0 9f1 2 9ec
9f3 2 9d7 9f5
1 9f7 1 9f9
2 9c2 9fa 1
9fe 1 9bd 2
9bb a02 2 a0a
a0b 1 a0f 2
a0d a0f 2 a1a
a1c 3 a18 a19
a1e 2 a27 a29
2 a2b a2d 2
a2f a31 1 a39
2 a3b a3c 2
a33 a3e 2 a40
a42 2 a44 a46
2 a48 a4a 2
a4c a4e 2 a26
a50 1 a52 1
a54 2 a5e a60
3 a5c a5d a62
2 a6b a6d 2
a6a a6f 1 a71
1 a73 3 a7a
a7b a7c 3 a55
a74 a7e 1 a80
2 a05 a81 1
a83 2 a88 a89
1 a8b 2 a90
a91 1 a95 2
a93 a95 2 a9b
a9c 1 a9e 2
aa3 aa4 1 aa8
2 aa6 aa8 2
ab3 ab5 3 ab1
ab2 ab7 2 ac0
ac2 2 ac4 ac6
2 ac8 aca 1
ad2 2 ad4 ad5
2 acc ad7 2
abf ad9 1 adb
1 add 2 ae7
ae9 3 ae5 ae6
aeb 2 af4 af6
2 af3 af8 1
afa 1 afc 2
b03 b04 3 ade
afd b06 1 b08
3 aa1 b09 b0e
1 b10 2 b16
b17 1 b1b 2
b19 b1b 1 b20
2 b2a b2c 3
b28 b29 b2e 2
b37 b39 2 b3b
b3d 2 b3f b41
1 b49 2 b4b
b4c 2 b43 b4e
2 b36 b50 1
b52 1 b54 2
b5e b60 3 b5c
b5d b62 2 b6b
b6d 2 b6a b6f
1 b71 1 b73
2 b7a b7b 3
b55 b74 b7d 2
b7f b80 6 b84
b85 b86 b87 b88
b89 2 b8e b8f
1 b93 2 b91
b93 2 b9e ba0
3 b9c b9d ba2
2 bab bad 2
baf bb1 2 bb3
bb5 1 bbb 2
bbd bbe 2 bb7
bc0 2 bc2 bc4
2 bc6 bc8 2
bca bcc 2 bce
bd0 2 baa bd2
1 bd4 1 bd6
2 be0 be2 3
bde bdf be4 2
bed bef 2 bec
bf1 1 bf3 1
bf5 2 bfb bfc
3 bfe bff c00
3 bd7 bf6 c02
1 c04 2 c07
c08 1 c0e 2
c0a c0e 2 c19
c1b 3 c17 c18
c1d 2 c26 c28
2 c2a c2c 2
c2e c30 1 c38
2 c3a c3b 2
c32 c3d 2 c3f
c41 2 c43 c45
2 c47 c49 2
c4b c4d 2 c25
c4f 1 c51 1
c53 2 c5d c5f
3 c5b c5c c61
2 c6a c6c 2
c69 c6e 1 c70
1 c72 3 c79
c7a c7b 3 c54
c73 c7d 1 c7f
2 c82 c83 1
c89 2 c85 c89
2 c8e c8f 1
c91 1 c95 2
c93 c95 2 ca5
caa 1 cb1 1
cac 2 cc4 cc6
3 cc2 cc3 cc8
2 cd1 cd3 2
cd5 cd7 2 cd9
cdb 2 cdd cdf
2 ce1 ce3 3
ce8 ce9 cea 2
ce5 cec 2 cd0
cee 1 cf0 1
cf2 2 cbb cf3
1 cf7 1 cb6
2 cb4 cfb 1
cfe 2 d28 d2d
1 d34 1 d2f
2 d47 d49 3
d45 d46 d4b 2
d54 d56 2 d58
d5a 2 d5c d5e
2 d60 d62 2
d64 d66 3 d6b
d6c d6d 2 d68
d6f 2 d53 d71
1 d73 1 d75
2 d3e d76 1
d7a 1 d39 2
d37 d7e 1 d81
2 d84 d85 1
d8d 2 d89 d8d
2 d99 d9b 3
d97 d98 d9d 2
da6 da8 2 daa
dac 2 dae db0
1 db8 2 dba
dbb 2 db2 dbd
2 dbf dc1 2
dc3 dc5 2 dc7
dc9 2 dcb dcd
2 da5 dcf 1
dd1 1 dd3 2
ddd ddf 3 ddb
ddc de1 2 dea
dec 2 de9 dee
1 df0 1 df2
3 df9 dfa dfb
3 dd4 df3 dfd
1 dff 2 d86
e00 2 e04 e05
1 e0b 2 e07
e0b 2 e17 e1c
1 e23 1 e1e
2 e36 e38 3
e34 e35 e3a 2
e43 e45 2 e47
e49 2 e4b e4d
2 e4f e51 2
e53 e55 3 e5a
e5b e5c 2 e57
e5e 2 e42 e60
1 e62 1 e64
2 e2d e65 1
e69 1 e28 2
e26 e6d 1 e78
2 e74 e78 2
e84 e86 3 e82
e83 e88 2 e91
e93 2 e95 e97
2 e99 e9b 1
ea3 2 ea5 ea6
2 e9d ea8 2
eaa eac 2 eae
eb0 2 eb2 eb4
2 eb6 eb8 2
e90 eba 1 ebc
1 ebe 2 ec8
eca 3 ec6 ec7
ecc 2 ed5 ed7
2 ed4 ed9 1
edb 1 edd 3
ee4 ee5 ee6 3
ebf ede ee8 1
eea 2 eff f04
1 f0b 1 f06
2 f1e f20 3
f1c f1d f22 2
f2b f2d 2 f2f
f31 2 f33 f35
2 f37 f39 2
f3b f3d 3 f42
f43 f44 2 f3f
f46 2 f2a f48
1 f4a 1 f4c
2 f15 f4d 1
f51 1 f10 2
f0e f55 1 f60
2 f5c f60 2
f6c f6e 3 f6a
f6b f70 2 f79
f7b 2 f7d f7f
2 f81 f83 1
f8b 2 f8d f8e
2 f85 f90 2
f92 f94 2 f96
f98 2 f9a f9c
2 f9e fa0 2
f78 fa2 1 fa4
1 fa6 2 fb0
fb2 3 fae faf
fb4 2 fbd fbf
2 fbc fc1 1
fc3 1 fc5 3
fcc fcd fce 3
fa7 fc6 fd0 1
fd2 1 fd4 1
fe0 2 fdd fe3
1 fe5 5 e70
eeb f58 fd3 fe6
2 ff1 ff3 3
fef ff0 ff5 2
ffe 1000 2 1002
1004 2 1006 1008
1 1010 2 1012
1013 2 100a 1015
2 1017 1019 2
101b 101d 2 101f
1021 2 1023 1025
2 ffd 1027 1
1029 1 102b 2
1035 1037 3 1033
1034 1039 2 1042
1044 2 1041 1046
1 1048 1 104a
3 1051 1052 1053
3 102c 104b 1055
3 1057 fe8 1058
2 105b 105c 1
1062 2 105e 1062
2 107a 107f 1
1086 1 1081 2
1099 109b 3 1097
1098 109d 2 10a6
10a8 2 10aa 10ac
2 10ae 10b0 2
10b2 10b4 2 10b6
10b8 3 10bd 10be
10bf 2 10ba 10c1
2 10a5 10c3 1
10c5 1 10c7 2
1090 10c8 1 10cc
1 108b 2 1089
10d0 1 10db 2
10d7 10db 2 10e7
10e9 3 10e5 10e6
10eb 2 10f4 10f6
2 10f8 10fa 2
10fc 10fe 1 1106
2 1108 1109 2
1100 110b 2 110d
110f 2 1111 1113
2 1115 1117 2
1119 111b 2 10f3
111d 1 111f 1
1121 2 112b 112d
3 1129 112a 112f
2 1138 113a 2
1137 113c 1 113e
1 1140 3 1147
1148 1149 3 1122
1141 114b 1 1154
2 1150 1154 3
1159 115a 115b 1
115f 2 115d 115f
2 1163 1164 1
1168 2 1166 1168
2 116c 116d 2
1172 1173 1 1175
2 116f 1175 2
117e 117f 2 1184
1185 1 1187 2
1181 1187 2 1194
1196 3 1192 1193
1198 2 11a1 11a3
2 11a5 11a7 2
11a9 11ab 1 11b3
2 11b5 11b6 2
11ad 11b8 2 11ba
11bc 2 11be 11c0
2 11c2 11c4 2
11c6 11c8 2 11a0
11ca 1 11cc 1
11ce 2 11d8 11da
3 11d6 11d7 11dc
2 11e5 11e7 2
11e4 11e9 1 11eb
1 11ed 3 11f4
11f5 11f6 3 11cf
11ee 11f8 1 11fa
1 11fb 2 11fe
11fd 2 10d3 11ff
1 1201 c 93d
99c a84 a8e b11
b14 b81 b8c c05
c80 1059 1202 1
1204 2 1207 1208
1 120c 2 120a
120c 2 1210 1211
1 1217 2 1213
1217 1 121a 2
1222 1223 1 1227
2 1225 1227 2
1234 1236 3 1232
1233 1238 2 1241
1243 2 1245 1247
2 1249 124b 1
1253 2 1255 1256
2 124d 1258 2
125a 125c 2 125e
1260 2 1262 1264
2 1266 1268 2
1240 126a 1 126c
1 126e 2 1278
127a 3 1276 1277
127c 2 1285 1287
2 1284 1289 1
128b 1 128d 3
1294 1295 1296 3
126f 128e 1298 1
129a 2 129d 129e
1 12a4 2 12a0
12a4 2 12b2 12b7
1 12be 1 12b9
2 12d1 12d3 3
12cf 12d0 12d5 2
12de 12e0 2 12e2
12e4 2 12e6 12e8
2 12ea 12ec 2
12ee 12f0 3 12f5
12f6 12f7 2 12f2
12f9 2 12dd 12fb
1 12fd 1 12ff
2 12c8 1300 1
1304 1 12c3 2
12c1 1308 1 1313
2 130f 1313 2
131c 1321 1 1328
1 1323 2 133b
133d 3 1339 133a
133f 2 1348 134a
2 134c 134e 2
1350 1352 2 1354
1356 2 1358 135a
3 135f 1360 1361
2 135c 1363 2
1347 1365 1 1367
1 1369 2 1332
136a 1 136e 1
132d 2 132b 1372
1 137d 2 1379
137d 2 1389 138b
3 1387 1388 138d
2 1396 1398 2
139a 139c 2 139e
13a0 1 13a8 2
13aa 13ab 2 13a2
13ad 2 13af 13b1
2 13b3 13b5 2
13b7 13b9 2 13bb
13bd 2 1395 13bf
1 13c1 1 13c3
2 13cd 13cf 3
13cb 13cc 13d1 2
13da 13dc 2 13d9
13de 1 13e0 1
13e2 3 13e9 13ea
13eb 3 13c4 13e3
13ed 1 13ef 2
1375 13f0 1 13f2
2 130b 13f3 1
13f5 2 129b 13f6
1 13f8 2 13fb
13fc 1 1400 2
13fe 1400 2 1404
1405 1 1409 2
1407 1409 1 1410
1 1412 2 1415
1416 1 141c 2
1418 141c 1 141f
2 1426 1429 2
142d 1430 2 1432
1433 2 1434 1437
1 143b 2 1439
143b 2 1446 1447
2 144b 144c 2
1450 1451 2 1455
1456 2 145a 145b
2 145f 1460 2
1464 1465 2 1469
146a d 1443 1444
1449 144e 1453 1458
145d 1462 1467 146c
146d 146e 146f 2
1474 1475 1 147b
2 1477 147b 2
1486 1488 3 1484
1485 148a 2 1493
1495 2 1497 1499
2 149b 149d 1
14a5 2 14a7 14a8
2 149f 14aa 2
1492 14ac 1 14ae
1 14b0 2 14ba
14bc 3 14b8 14b9
14be 2 14c7 14c9
2 14c6 14cb 1
14cd 1 14cf 2
14d6 14d7 3 14b1
14d0 14d9 2 14dd
14de 1 14e2 2
14e0 14e2 2 14ed
14ef 3 14eb 14ec
14f1 2 14fa 14fc
2 14fe 1500 2
1502 1504 1 150a
2 150c 150d 2
1506 150f 2 1511
1513 2 1515 1517
2 1519 151b 2
151d 151f 2 14f9
1521 1 1523 1
1525 2 152f 1531
3 152d 152e 1533
2 153c 153e 2
153b 1540 1 1542
1 1544 2 154a
154b 3 154d 154e
154f 3 1526 1545
1551 2 1554 1553
3 1440 1472 1555
1 1559 2 155b
155c 2 155f 1560
1 1564 2 1562
1564 2 1569 156a
1 156e 2 156c
156e 2 1575 1576
1 157a 2 1578
157a 2 157f 1580
1 1584 2 1582
1584 1 158c 2
1590 1591 1 1595
2 1593 1595 1
159a 2 159f 15a0
1 15a4 2 15a2
15a4 2 15a9 15aa
1 15ae 2 15ac
15ae 1 15b6 2
15bb 15bc 1 15c0
2 15be 15c0 1
15c5 4 15c8 159d
15b9 15c7 1 15c9
1 15cb 2 155d
15cc 2 15ce 15cf
2 15d2 15d3 1
15d9 2 15d5 15d9
3 15e7 15e8 15e9
2 15f1 15f2 1
15f6 2 15f4 15f6
2 15fd 1600 2
1607 160c 1 1613
1 160e 2 1626
1628 3 1624 1625
162a 2 1633 1635
2 1637 1639 2
163b 163d 2 163f
1641 2 1643 1645
3 164a 164b 164c
2 1647 164e 2
1632 1650 1 1652
1 1654 2 161d
1655 1 1659 1
1618 2 1616 165d
1 1668 2 1664
1668 2 166d 166e
1 1672 2 1670
1672 2 1680 1685
1 168c 1 1687
2 169f 16a1 3
169d 169e 16a3 2
16ac 16ae 2 16b0
16b2 2 16b4 16b6
2 16b8 16ba 2
16bc 16be 3 16c3
16c4 16c5 2 16c0
16c7 2 16ab 16c9
1 16cb 1 16cd
2 1696 16ce 1
16d2 1 1691 2
168f 16d6 1 16e1
2 16dd 16e1 1
16e7 1 16e9 2
16d9 16ea 1 16ec
1 16ed 1 16f1
2 16f3 16f4 2
1660 16f5 2 16f7
16f8 1 16fc 2
16fa 16fc 2 1708
170d 1 1714 1
170f 2 1727 1729
3 1725 1726 172b
2 1734 1736 2
1738 173a 2 173c
173e 2 1740 1742
2 1744 1746 3
174b 174c 174d 2
1748 174f 2 1733
1751 1 1753 1
1755 2 171e 1756
1 175a 1 1719
2 1717 175e 2
1782 1787 1 178e
1 1789 2 17a1
17a3 3 179f 17a0
17a5 2 17ae 17b0
2 17b2 17b4 2
17b6 17b8 2 17ba
17bc 2 17be 17c0
3 17c5 17c6 17c7
2 17c2 17c9 2
17ad 17cb 1 17cd
1 17cf 2 1798
17d0 1 17d4 1
1793 2 1791 17d8
1 17e3 2 17df
17e3 2 17ef 17f1
3 17ed 17ee 17f3
2 17fc 17fe 2
1800 1802 2 1804
1806 1 180e 2
1810 1811 2 1808
1813 2 1815 1817
2 1819 181b 2
181d 181f 2 1821
1823 2 17fb 1825
1 1827 1 1829
2 1833 1835 3
1831 1832 1837 2
1840 1842 2 183f
1844 1 1846 1
1848 3 184f 1850
1851 3 182a 1849
1853 1 1855 2
1858 1859 1 185d
2 185b 185d 1
1862 1 1866 2
1868 1869 1 1881
2 1880 1881 1
1889 1 188f 2
1891 1892 2 187e
1893 1 189a 1
1895 2 18ad 18af
3 18ab 18ac 18b1
2 18ba 18bc 2
18be 18c0 2 18c2
18c4 2 18c6 18c8
2 18ca 18cc 3
18d1 18d2 18d3 2
18ce 18d5 2 18b9
18d7 1 18d9 1
18db 2 18a4 18dc
1 18e0 1 189f
2 189d 18e4 5
1761 17db 1856 186a
18e7 3 18ec 18ed
18ee 1 18f2 2
18f0 18f2 1 1902
2 1901 1902 1
190a 1 1910 2
1912 1913 2 18ff
1914 1 191b 1
1916 2 192e 1930
3 192c 192d 1932
2 193b 193d 2
193f 1941 2 1943
1945 2 1947 1949
2 194b 194d 3
1952 1953 1954 2
194f 1956 2 193a
1958 1 195a 1
195c 2 1925 195d
1 1961 1 1920
2 191e 1965 1
1968 2 196c 196b
1 197a 2 1979
197a 1 1982 1
1988 2 198a 198b
2 1977 198c 1
1993 1 198e 2
19a6 19a8 3 19a4
19a5 19aa 2 19b3
19b5 2 19b7 19b9
2 19bb 19bd 2
19bf 19c1 2 19c3
19c5 3 19ca 19cb
19cc 2 19c7 19ce
2 19b2 19d0 1
19d2 1 19d4 2
199d 19d5 1 19d9
1 1998 2 1996
19dd 1 19ee 2
19ed 19ee 1 19f6
1 19fc 2 19fe
19ff 2 19eb 1a00
1 1a07 1 1a02
2 1a1a 1a1c 3
1a18 1a19 1a1e 2
1a27 1a29 2 1a2b
1a2d 2 1a2f 1a31
2 1a33 1a35 2
1a37 1a39 3 1a3e
1a3f 1a40 2 1a3b
1a42 2 1a26 1a44
1 1a46 1 1a48
2 1a11 1a49 1
1a4d 1 1a0c 2
1a0a 1a51 1 1a5f
2 1a5e 1a5f 1
1a67 1 1a6d 2
1a6f 1a70 2 1a5c
1a71 1 1a78 1
1a73 2 1a8b 1a8d
3 1a89 1a8a 1a8f
2 1a98 1a9a 2
1a9c 1a9e 2 1aa0
1aa2 2 1aa4 1aa6
2 1aa8 1aaa 3
1aaf 1ab0 1ab1 2
1aac 1ab3 2 1a97
1ab5 1 1ab7 1
1ab9 2 1a82 1aba
1 1abe 1 1a7d
2 1a7b 1ac2 a
15de 15e1 15e4 15ec
15ef 16f9 196d 19e0
1a54 1ac5 1 1ac8
2 1ad0 1ad1 1
1ad5 2 1ad3 1ad5
2 1ad9 1ada 1
1ae0 2 1adc 1ae0
2 1ae6 1ae7 1
1aed 2 1ae9 1aed
1 1af2 1 1af9
1 1afb 1 1afc
1 1afe 2 1b01
1b02 1 1b06 2
1b04 1b06 1 1b0d
1 1b0f 1 1b11
1 1b16 1 1b1d
1 1b24 1 1b2e
1 1b30 2 1b39
1b3a 1 1b3e 2
1b3c 1b3e 2 1b42
1b43 1 1b49 2
1b45 1b49 2 1b4d
1b4f 2 1b51 1b53
2 1b55 1b57 2
1b59 1b5b 2 1b5d
1b5f 2 1b61 1b63
2 1b65 1b67 1
1b6a 2 1b6e 1b6f
1 1b73 2 1b71
1b73 2 1b77 1b79
2 1b7b 1b7d 2
1b7f 1b81 2 1b83
1b85 2 1b87 1b89
2 1b8b 1b8d 2
1b8f 1b91 2 1b93
1b95 1 1b98 2
1b9c 1b9e 2 1ba0
1ba2 2 1ba4 1ba6
2 1ba8 1baa 2
1bac 1bae 1 1bb1
3 1bb3 1b9a 1bb4
1 1bb5 2 1bb8
1bb9 1 1bbf 2
1bbb 1bbf 2 1bc3
1bc4 1 1bca 2
1bc6 1bca 2 1bd0
1bd1 1 1bd7 2
1bd3 1bd7 1 1be6
1 1c0f 2 1c0e
1c0f 1 1c17 1
1c1d 2 1c1f 1c20
2 1c0c 1c21 1
1c28 1 1c23 2
1c3b 1c3d 3 1c39
1c3a 1c3f 2 1c48
1c4a 2 1c4c 1c4e
2 1c50 1c52 2
1c54 1c56 2 1c58
1c5a 3 1c5f 1c60
1c61 2 1c5c 1c63
2 1c47 1c65 1
1c67 1 1c69 2
1c32 1c6a 1 1c6e
1 1c2d 2 1c2b
1c72 1 1c8f 2
1c8e 1c8f 1 1c97
1 1c9d 2 1c9f
1ca0 2 1c8c 1ca1
1 1ca8 1 1ca3
2 1cbb 1cbd 3
1cb9 1cba 1cbf 2
1cc8 1cca 2 1ccc
1cce 2 1cd0 1cd2
2 1cd4 1cd6 2
1cd8 1cda 3 1cdf
1ce0 1ce1 2 1cdc
1ce3 2 1cc7 1ce5
1 1ce7 1 1ce9
2 1cb2 1cea 1
1cee 1 1cad 2
1cab 1cf2 4 1bef
1bf4 1c75 1cf5 1
1cfa 2 1cfc 1cfd
3 1be0 1be5 1cfe
1 1d00 1 1d02
1 1d0a 2 1d0c
1d0d 1 1d11 2
1d0f 1d11 1 1d17
2 1d19 1d1a 2
1d1c 1d1d 2 1d1f
1d21 1 1d13 1
1d29 2 1d2b 1d2c
2 1d2e 1d2f 2
1d25 1d32 1 1d3a
2 1d39 1d3a 2
1d44 1d46 3 1d43
1d48 1d49 1 1d4b
2 1d4d 1d4e 1
1d52 2 1d50 1d52
2 1d59 1d5b 3
1d58 1d5d 1d5e 1
1d60 2 1d62 1d63
1 1d67 2 1d65
1d67 1 1d6d 1
1d6f 2 1d72 1d74
2 1d70 1d77 1
1d7c 2 1d7b 1d7c
2 1d88 1d8a 3
1d86 1d87 1d8c 2
1d95 1d97 2 1d99
1d9b 2 1d9d 1d9f
1 1da7 2 1da9
1daa 2 1da1 1dac
2 1d94 1dae 1
1db0 1 1db2 2
1dbc 1dbe 3 1dba
1dbb 1dc0 2 1dc9
1dcb 2 1dc8 1dcd
1 1dcf 1 1dd1
2 1dd8 1dd9 3
1db3 1dd2 1ddb 1
1ddd 2 1de3 1de8
1 1def 1 1dea
2 1e02 1e04 3
1e00 1e01 1e06 2
1e0f 1e11 2 1e13
1e15 2 1e17 1e19
2 1e1b 1e1d 2
1e1f 1e21 3 1e26
1e27 1e28 2 1e23
1e2a 2 1e0e 1e2c
1 1e2e 1 1e30
2 1df9 1e31 1
1e35 1 1df4 2
1df2 1e39 5 1d34
1d37 1d7a 1dde 1e3c
1 1e3f 2 1e42
1e43 1 1e49 2
1e45 1e49 2 1e53
1e55 2 1e5a 1e5c
4 1e4e 1e51 1e58
1e5f 2 1e63 1e64
1 1e6a 2 1e66
1e6a 2 1e71 1e73
2 1e78 1e7a 4
1e6f 1e76 1e7d 1e80
2 1e84 1e85 1
1e8b 2 1e87 1e8b
1 1e94 2 1e9a
1e9c 2 1ea1 1ea3
2 1ea8 1eaa 3
1e9f 1ea6 1ead 1
1eaf 2 1eb2 1eb4
1 1eb8 2 1ebe
1ec0 2 1ec5 1ec7
2 1ecc 1ece 3
1ec3 1eca 1ed1 1
1ed3 5 1e90 1e93
1eb0 1eb7 1ed4 1
1ed6 2 1edc 1ede
2 1ee1 1ee4 2
1ee7 1ee9 2 1eeb
1eed 1 1ef4 2
1efa 1efc 2 1f01
1f03 2 1f08 1f0a
3 1eff 1f06 1f0d
1 1f0f 2 1f12
1f14 2 1f16 1f18
2 1f1a 1f1c 2
1f1e 1f20 2 1f22
1f24 2 1f26 1f28
4 1ef0 1ef3 1f10
1f2b 2 1f2d 1f2e
1 1f30 2 1f36
1f38 2 1f3a 1f3c
2 1f3e 1f40 1
1f43 1 1f45 2
1f2f 1f46 2 1f48
1f49 1 1f4a 3
1f4c 1e82 1f4d 2
1f50 1f51 1 1f57
2 1f53 1f57 1
1f5a 2 1f64 1f69
1 1f70 1 1f6b
2 1f83 1f85 3
1f81 1f82 1f87 2
1f90 1f92 2 1f94
1f96 2 1f98 1f9a
2 1f9c 1f9e 2
1fa0 1fa2 3 1fa7
1fa8 1fa9 2 1fa4
1fab 2 1f8f 1fad
1 1faf 1 1fb1
2 1f7a 1fb2 1
1fb6 1 1f75 2
1f73 1fba 1 1fc5
2 1fc1 1fc5 1
1fcc 2 1fce 1fcf
2 1fd4 1fd6 2
1fdb 1fdd 2 1fdf
1fe1 2 1fe3 1fe5
2 1fe7 1fe9 3
1fd2 1fd9 1fec 2
1ff6 1ff8 3 1ff4
1ff5 1ffa 2 2003
2005 2 2007 2009
2 200b 200d 1
2015 2 2017 2018
2 200f 201a 2
201c 201e 2 2020
2022 2 2024 2026
2 2028 202a 2
2002 202c 1 202e
1 2030 2 203a
203c 3 2038 2039
203e 2 2047 2049
2 2046 204b 1
204d 1 204f 3
2056 2057 2058 3
2031 2050 205a 2
205c 205d 2 1fbd
205e 1 2060 1
2062 2 206c 2071
1 2078 1 2073
2 208b 208d 3
2089 208a 208f 2
2098 209a 2 209c
209e 2 20a0 20a2
2 20a4 20a6 2
20a8 20aa 3 20af
20b0 20b1 2 20ac
20b3 2 2097 20b5
1 20b7 1 20b9
2 2082 20ba 1
20be 1 207d 2
207b 20c2 1 20cd
2 20c9 20cd 2
20d2 20d4 2 20d9
20db 2 20dd 20df
2 20e1 20e3 2
20e5 20e7 2 20d7
20ea 2 20f4 20f6
3 20f2 20f3 20f8
2 2101 2103 2
2105 2107 2 2109
210b 1 2113 2
2115 2116 2 210d
2118 2 211a 211c
2 211e 2120 2
2122 2124 2 2126
2128 2 2100 212a
1 212c 1 212e
2 2138 213a 3
2136 2137 213c 2
2145 2147 2 2144
2149 1 214b 1
214d 3 2154 2155
2156 3 212f 214e
2158 2 215a 215b
2 20c5 215c 1
215e 1 2160 2
216a 216f 1 2176
1 2171 2 2189
218b 3 2187 2188
218d 2 2196 2198
2 219a 219c 2
219e 21a0 2 21a2
21a4 2 21a6 21a8
3 21ad 21ae 21af
2 21aa 21b1 2
2195 21b3 1 21b5
1 21b7 2 2180
21b8 1 21bc 1
217b 2 2179 21c0
1 21cb 2 21c7
21cb 2 21d0 21d2
2 21d7 21d9 2
21db 21dd 2 21df
21e1 2 21e3 21e5
2 21d5 21e8 2
21f2 21f4 3 21f0
21f1 21f6 2 21ff
2201 2 2203 2205
2 2207 2209 1
2211 2 2213 2214
2 220b 2216 2
2218 221a 2 221c
221e 2 2220 2222
2 2224 2226 2
21fe 2228 1 222a
1 222c 2 2236
2238 3 2234 2235
223a 2 2243 2245
2 2242 2247 1
2249 1 224b 3
2252 2253 2254 3
222d 224c 2256 2
2258 2259 2 21c3
225a 1 225c 2
225f 2260 1 2264
2 2262 2264 2
2268 2269 1 226d
2 226b 226d 2
2273 2275 2 227b
227c 1 227e 2
2277 2280 2 2282
2284 1 2287 1
2289 4 2061 215f
225d 228a 1 228c
1 228e 1 2296
2 2298 2299 1
22a2 2 22a1 22a2
2 22ae 22b0 4
22ac 22ad 22b2 22b3
2 22bc 22c1 1
22c8 1 22c3 2
22db 22dd 3 22d9
22da 22df 2 22e8
22ea 2 22ec 22ee
2 22f0 22f2 2
22f4 22f6 2 22f8
22fa 3 22ff 2300
2301 2 22fc 2303
2 22e7 2305 1
2307 1 2309 2
22d2 230a 1 230e
1 22cd 2 22cb
2312 1 231d 2
2319 231d 2 2329
232b 3 2327 2328
232d 2 2336 2338
2 233a 233c 2
233e 2340 1 2348
2 234a 234b 2
2342 234d 2 234f
2351 2 2353 2355
2 2357 2359 2
235b 235d 2 2335
235f 1 2361 1
2363 2 236d 236f
3 236b 236c 2371
2 237a 237c 2
2379 237e 1 2380
1 2382 3 2389
238a 238b 3 2364
2383 238d 1 238f
2 2392 2394 2
2396 2398 2 239a
239c 2 239e 23a0
2 23a2 23a4 2
23a6 23a8 2 23ad
23ae 1 23b4 2
23b0 23b4 2 23b8
23ba 1 23bd 2
23c0 23c2 1 23c5
2 23c7 23c8 2
23cb 23cc 1 23d2
2 23ce 23d2 1
23d6 1 23d8 2
23db 23dd 7 22b6
2315 2390 23ab 23c9
23d9 23e0 3 229c
229f 23e3 1 23e5
2 23e8 23e9 1
23ef 2 23eb 23ef
1 23f5 2 23f7
23f8 1 2401 2
2400 2401 2 240d
240f 4 240b 240c
2411 2412 2 241b
2420 1 2427 1
2422 2 243a 243c
3 2438 2439 243e
2 2447 2449 2
244b 244d 2 244f
2451 2 2453 2455
2 2457 2459 3
245e 245f 2460 2
245b 2462 2 2446
2464 1 2466 1
2468 2 2431 2469
1 246d 1 242c
2 242a 2471 1
247c 2 2478 247c
2 2488 248a 3
2486 2487 248c 2
2495 2497 2 2499
249b 2 249d 249f
1 24a7 2 24a9
24aa 2 24a1 24ac
2 24ae 24b0 2
24b2 24b4 2 24b6
24b8 2 24ba 24bc
2 2494 24be 1
24c0 1 24c2 2
24cc 24ce 3 24ca
24cb 24d0 2 24d9
24db 2 24d8 24dd
1 24df 1 24e1
3 24e8 24e9 24ea
3 24c3 24e2 24ec
1 24ee 2 24f1
24f3 2 24f5 24f7
2 24f9 24fb 2
24fd 24ff 2 2501
2503 2 2505 2507
2 2509 250b 2
2510 2512 5 2415
2474 24ef 250e 2515
3 23fb 23fe 2518
1 251a 2 251d
251f 7 1d01 1e40
1f4e 228d 23e6 251b
2522 2 2524 2525
2 2528 252a 1
253f 1 2545 2
2547 2548 1 2550
2 254c 2550 1
2555 1 2557 2
255a 255b 1 255f
2 255d 255f 2
2563 2564 1 256a
2 2566 256a 2
2570 2571 1 2577
2 2573 2577 2
257d 257e 1 2584
2 2580 2584 1
2587 1 25a1 1
25a7 2 25a9 25aa
1 25b2 2 25ae
25b2 2 25b8 25be
1 25c5 1 25c0
2 25d8 25da 3
25d6 25d7 25dc 2
25e5 25e7 2 25e9
25eb 2 25ed 25ef
2 25f1 25f3 2
25f5 25f7 3 25fc
25fd 25fe 2 25f9
2600 2 25e4 2602
1 2604 1 2606
2 25cf 2607 1
260b 1 25ca 2
25c8 260f 2 2612
2615 2 2619 261f
1 2626 1 2621
2 2639 263b 3
2637 2638 263d 2
2646 2648 2 264a
264c 2 264e 2650
2 2652 2654 2
2656 2658 3 265d
265e 265f 2 265a
2661 2 2645 2663
1 2665 1 2667
2 2630 2668 1
266c 1 262b 2
2629 2670 1 2673
2 2676 2677 4
2593 2598 25ab 2678
1 267a 2 2683
2684 1 2688 2
2686 2688 3 268c
268d 268e 1 2692
2 2690 2692 2
2699 269c 2 26a0
26a1 1 26a3 1
26a7 2 26a5 26a7
2 26b3 26b8 1
26bf 1 26ba 2
26d2 26d4 3 26d0
26d1 26d6 2 26df
26e1 2 26e3 26e5
2 26e7 26e9 2
26eb 26ed 2 26ef
26f1 3 26f6 26f7
26f8 2 26f3 26fa
2 26de 26fc 1
26fe 1 2700 2
26c9 2701 1 2705
1 26c4 2 26c2
2709 2 270f 2710
1 2714 2 2712
2714 2 2719 271f
1 2726 1 2721
2 2739 273b 3
2737 2738 273d 2
2746 2748 2 274a
274c 2 274e 2750
2 2752 2754 2
2756 2758 3 275d
275e 275f 2 275a
2761 2 2745 2763
1 2765 1 2767
2 2730 2768 1
276c 1 272b 2
2729 2770 2 277d
277f 3 277b 277c
2781 2 278a 278c
2 278e 2790 2
2792 2794 1 279c
2 279e 279f 2
2796 27a1 2 27a3
27a5 2 27a7 27a9
2 27ab 27ad 2
27af 27b1 2 2789
27b3 1 27b5 1
27b7 2 27c1 27c3
3 27bf 27c0 27c5
2 27ce 27d0 2
27cd 27d2 1 27d4
1 27d6 3 27dd
27de 27df 4 2773
27b8 27d7 27e1 1
27e3 2 270c 27e4
1 27e6 1 27e7
2 27e9 27ea 2
280b 280c 20 27ee
27ef 27f0 27f1 27f2
27f3 27f4 27f5 27f6
27f7 27f8 27f9 27fa
27fb 27fc 27fd 27fe
27ff 2800 2801 2802
2803 2804 2805 2806
2807 2808 2809 280e
280f 2810 2811 1
2817 2 2815 2817
2 281c 2822 1
2829 1 2824 2
283c 283e 3 283a
283b 2840 2 2849
284b 2 284d 284f
2 2851 2853 2
2855 2857 2 2859
285b 3 2860 2861
2862 2 285d 2864
2 2848 2866 1
2868 1 286a 2
2833 286b 1 286f
1 282e 2 282c
2873 1 287c 2
2878 287c 1 2883
1 2885 2 288f
2891 3 288d 288e
2893 2 289c 289e
2 28a0 28a2 2
28a4 28a6 1 28ac
2 28ae 28af 2
28a8 28b1 2 28b3
28b5 2 28b7 28b9
2 28bb 28bd 2
28bf 28c1 2 289b
28c3 1 28c5 1
28c7 2 28d1 28d3
3 28cf 28d0 28d5
2 28de 28e0 2
28dd 28e2 1 28e4
1 28e6 3 28eb
28ec 28ed 5 2876
2886 28c8 28e7 28ef
3 28f3 28f4 28f5
1 28f9 2 28f7
28f9 3 28fe 28ff
2900 1 2904 2
2902 2904 1 2912
2 2911 2912 1
291a 1 2920 2
2922 2923 2 290f
2924 1 292b 1
2926 2 293e 2940
3 293c 293d 2942
2 294b 294d 2
294f 2951 2 2953
2955 2 2957 2959
2 295b 295d 3
2962 2963 2964 2
295f 2966 2 294a
2968 1 296a 1
296c 2 2935 296d
1 2971 1 2930
2 292e 2975 1
2978 2 297c 297b
4 27eb 2814 297d
297f 2 2982 2983
1 2989 2 2985
2989 2 298e 2991
1 2993 20 2997
2998 2999 299a 299b
299c 299d 299e 299f
29a0 29a1 29a2 29a3
29a4 29a5 29a6 29a7
29a8 29a9 29aa 29ab
29ac 29ad 29ae 29af
29b0 29b1 29b2 29b3
29b4 29b5 29b6 1
29bc 2 29ba 29bc
2 29c0 29c2 1
29c5 1 29c7 2
29cc 29cd 1 29cf
2 29c9 29cf 1
29d3 1 29d5 4
2994 29b9 29c8 29d6
2 29d8 29d9 1
29db 8 2536 2549
2558 267b 267e 2681
29da 29de 2 29e4
29ea 1 29f1 1
29ec 2 2a04 2a06
3 2a02 2a03 2a08
2 2a11 2a13 2
2a15 2a17 2 2a19
2a1b 2 2a1d 2a1f
2 2a21 2a23 3
2a28 2a29 2a2a 2
2a25 2a2c 2 2a10
2a2e 1 2a30 1
2a32 2 29fb 2a33
1 2a37 1 29f6
2 29f4 2a3b 1
2a44 2 2a40 2a44
1 2a4a 2 2a48
2a4a 2 2a4f 2a50
1 2a54 2 2a52
2a54 2 2a65 2a67
3 2a63 2a64 2a69
2 2a72 2a74 2
2a76 2a78 2 2a7a
2a7c 1 2a82 2
2a84 2a85 2 2a7e
2a87 2 2a89 2a8b
2 2a8d 2a8f 2
2a91 2a93 2 2a95
2a97 2 2a71 2a99
1 2a9b 1 2a9d
2 2aa7 2aa9 3
2aa5 2aa6 2aab 2
2ab4 2ab6 2 2ab3
2ab8 1 2aba 1
2abc 3 2ac1 2ac2
2ac3 3 2a9e 2abd
2ac5 1 2ac7 1
2acb 2 2ac9 2acb
8 2ae1 2ae2 2ae3
2ae4 2ae5 2ae6 2ae7
2ae8 1 2af3 1
2af9 2 2afb 2afc
1 2b04 2 2b00
2b04 1 2b09 1
2b0b 1 2b0d 2
2b15 2b16 1 2b1a
2 2b18 2b1a 1
2b21 2 2b23 2b24
1 2b28 2 2b26
2b28 1 2b2f 1
2b31 2 2b36 2b37
1 2b3b 2 2b39
2b3b 1 2b40 2
2b3e 2b40 1 2b55
2 2b54 2b55 1
2b5d 1 2b63 2
2b65 2b66 2 2b52
2b67 1 2b6e 1
2b69 2 2b81 2b83
3 2b7f 2b80 2b85
2 2b8e 2b90 2
2b92 2b94 2 2b96
2b98 2 2b9a 2b9c
2 2b9e 2ba0 3
2ba5 2ba6 2ba7 2
2ba2 2ba9 2 2b8d
2bab 1 2bad 1
2baf 2 2b78 2bb0
1 2bb4 1 2b73
2 2b71 2bb8 1
2bbb 1 2bbe 9
2bbf 2bc2 2bc5 2bc8
2bcb 2bce 2bd1 2bd4
2bd7 1 2bd9 1
2bdd 2 2bdb 2bdd
1 2be0 1 2bf3
2 2bf2 2bf3 1
2bfb 1 2c01 2
2c03 2c04 2 2bf0
2c05 1 2c0c 1
2c07 2 2c1f 2c21
3 2c1d 2c1e 2c23
2 2c2c 2c2e 2
2c30 2c32 2 2c34
2c36 2 2c38 2c3a
2 2c3c 2c3e 3
2c43 2c44 2c45 2
2c40 2c47 2 2c2b
2c49 1 2c4b 1
2c4d 2 2c16 2c4e
1 2c52 1 2c11
2 2c0f 2c56 1
2c59 1 2c5c 1
2c5e 1 2c63 2
2c6a 2c6c 3 2c6e
2c6f 2c70 1 2c73
1 2c75 2 2c79
2c7b 3 2c7d 2c7e
2c7f 2 2c76 2c82
1 2c84 1 2c86
2 2c8d 2c8f 3
2c91 2c92 2c93 1
2c96 1 2c98 1
2c9a 1 2c9f 2
2ca6 2ca8 3 2caa
2cab 2cac 1 2caf
1 2cb1 2 2cb5
2cb7 3 2cb9 2cba
2cbb 2 2cb2 2cbe
1 2cc0 1 2cc2
2 2cc9 2ccb 3
2ccd 2cce 2ccf 1
2cd2 1 2cd4 1
2cd6 1 2cdb 2
2ce2 2ce4 3 2ce6
2ce7 2ce8 1 2ceb
1 2ced 2 2cf1
2cf3 3 2cf5 2cf6
2cf7 2 2cee 2cfa
1 2cfc 1 2cfe
1 2d03 2 2d0a
2d0c 3 2d0e 2d0f
2d10 1 2d13 1
2d15 2 2d19 2d1b
3 2d1d 2d1e 2d1f
2 2d16 2d22 1
2d24 2 2d27 2d29
e 2ae9 2afd 2b0c
2b32 2bda 2c5d 2c85
2c99 2cc1 2cd5 2cfd
2d25 2d2c 2d2f 2
2d35 2d3b 1 2d42
1 2d3d 2 2d55
2d57 3 2d53 2d54
2d59 2 2d62 2d64
2 2d66 2d68 2
2d6a 2d6c 2 2d6e
2d70 2 2d72 2d74
3 2d79 2d7a 2d7b
2 2d76 2d7d 2
2d61 2d7f 1 2d81
1 2d83 2 2d4c
2d84 1 2d88 1
2d47 2 2d45 2d8c
1 2d93 2 2d91
2d93 1 2da8 2
2da7 2da8 1 2db0
1 2db6 2 2db8
2db9 2 2da5 2dba
1 2dc1 1 2dbc
2 2dd4 2dd6 3
2dd2 2dd3 2dd8 2
2de1 2de3 2 2de5
2de7 2 2de9 2deb
2 2ded 2def 2
2df1 2df3 3 2df8
2df9 2dfa 2 2df5
2dfc 2 2de0 2dfe
1 2e00 1 2e02
2 2dcb 2e03 1
2e07 1 2dc6 2
2dc4 2e0b 1 2e0e
1 2e11 8 2ad0
2ad3 2ad6 2ad9 2ade
2d32 2d8f 2e12 1
2e14 1 2e16 2
2e1c 2e1d 1 2e21
2 2e1f 2e21 2
2e32 2e37 1 2e3e
1 2e39 2 2e51
2e53 3 2e4f 2e50
2e55 2 2e5e 2e60
2 2e62 2e64 2
2e66 2e68 2 2e6a
2e6c 2 2e6e 2e70
3 2e75 2e76 2e77
2 2e72 2e79 2
2e5d 2e7b 1 2e7d
1 2e7f 2 2e48
2e80 1 2e84 1
2e43 2 2e41 2e88
1 2e8b 2 2e90
2e91 2 2e93 2e94
1 2e95 1 2e97
2 2eac 2eb1 1
2eb8 1 2eb3 2
2ecb 2ecd 3 2ec9
2eca 2ecf 2 2ed8
2eda 2 2edc 2ede
2 2ee0 2ee2 2
2ee4 2ee6 2 2ee8
2eea 3 2eef 2ef0
2ef1 2 2eec 2ef3
2 2ed7 2ef5 1
2ef7 1 2ef9 2
2ec2 2efa 1 2efe
1 2ebd 2 2ebb
2f02 2 2f08 2f09
1 2f0f 2 2f0b
2f0f 1 2f2a 2
2f29 2f2a 1 2f32
1 2f38 2 2f3a
2f3b 2 2f27 2f3c
1 2f43 1 2f3e
2 2f56 2f58 3
2f54 2f55 2f5a 2
2f63 2f65 2 2f67
2f69 2 2f6b 2f6d
2 2f6f 2f71 2
2f73 2f75 3 2f7a
2f7b 2f7c 2 2f77
2f7e 2 2f62 2f80
1 2f82 1 2f84
2 2f4d 2f85 1
2f89 1 2f48 2
2f46 2f8d 1 2f98
2 2f94 2f98 1
2fb1 2 2fb0 2fb1
1 2fb9 1 2fbf
2 2fc1 2fc2 2
2fae 2fc3 1 2fca
1 2fc5 2 2fdd
2fdf 3 2fdb 2fdc
2fe1 2 2fea 2fec
2 2fee 2ff0 2
2ff2 2ff4 2 2ff6
2ff8 2 2ffa 2ffc
3 3001 3002 3003
2 2ffe 3005 2
2fe9 3007 1 3009
1 300b 2 2fd4
300c 1 3010 1
2fcf 2 2fcd 3014
1 301f 2 301b
301f 1 302c 2
302b 302c 1 3034
1 303a 2 303c
303d 2 3029 303e
1 3045 1 3040
2 3058 305a 3
3056 3057 305c 2
3065 3067 2 3069
306b 2 306d 306f
2 3071 3073 2
3075 3077 3 307c
307d 307e 2 3079
3080 2 3064 3082
1 3084 1 3086
2 304f 3087 1
308b 1 304a 2
3048 308f 1 3092
1 3095 2 3017
3096 1 3098 2
3171 3173 3 316f
3170 3175 2 317e
3180 2 317d 3182
1 3184 1 3186
7 2f16 2f1b 2f90
3099 3167 3187 318a
1 318e 2 318c
318e 2 32c9 32cb
3 32c7 32c8 32cd
2 32d6 32d8 2
32d5 32da 1 32dc
1 32de 3 32bf
32df 32e2 1 32f9
2 32f8 32f9 1
3301 1 3307 2
3309 330a 2 32f6
330b 1 3312 1
330d 2 3325 3327
3 3323 3324 3329
2 3332 3334 2
3336 3338 2 333a
333c 2 333e 3340
2 3342 3344 3
3349 334a 334b 2
3346 334d 2 3331
334f 1 3351 1
3353 2 331c 3354
1 3358 1 3317
2 3315 335c 1
3378 2 3377 3378
1 3380 1 3386
2 3388 3389 2
3375 338a 1 3391
1 338c 2 33a4
33a6 3 33a2 33a3
33a8 2 33b1 33b3
2 33b5 33b7 2
33b9 33bb 2 33bd
33bf 2 33c1 33c3
3 33c8 33c9 33ca
2 33c5 33cc 2
33b0 33ce 1 33d0
1 33d2 2 339b
33d3 1 33d7 1
3396 2 3394 33db
2 35a2 35a4 3
35a0 35a1 35a6 2
35af 35b1 2 35ae
35b3 1 35b5 1
35b7 7 32e8 32ed
335f 33de 3598 35b8
35bb 2 35bd 35be
1 35bf 2 35c1
35c2 3b 4f8 517
51a 51d 520 523
526 529 52c 52f
532 535 538 53b
53e 541 544 547
54a 54d 550 553
556 559 55c 55f
5d2 6e1 75f 770
86a 8d9 8dc 8f0
8fd 911 91e 931
1205 13f9 1413 15d0
1ac9 1ace 1aff 1b10
1b31 1b34 1b37 2526
252d 2532 29e1 2a3e
2ac8 2e15 2e98 2f05
35c3 2 35d0 35d2
3 35ce 35cf 35d4
2 35dd 35df 2
35dc 35e1 2 35e8
35ec 2 35ee 35f0
2 35f2 35f6 2
35e7 35f8 2 35e3
35fa 1 35fc 3
35fd 3608 360c 1
35c7 2 361f 3621
3 361d 361e 3623
2 362c 362e 2
362b 3630 2 3637
3639 2 363b 363d
3 3642 3643 3644
2 363f 3646 2
3636 3648 2 3632
364a 1 364c 4
3616 364d 3656 365a
1 365e 1 3611
2 360f 3662 69
7b 82 89 90
97 9e a5 ac
b3 ba c1 c8
cf d6 dd e4
eb f2 f9 100
107 10c 113 11a
121 128 12f 136
13d 144 14b 152
159 15f 165 16b
171 177 17d 183
189 18f 195 19b
1a1 1a7 1ad 1b3
1b9 1bf 1c5 1cb
1d1 1d7 1dd 1e3
1e9 1ef 1f5 1fb
201 207 20d 213
219 21f 226 22c
232 238 23f 246
24d 253 259 260
267 26e 275 27c
283 289 28f 295
29b 2a1 2a7 2ad
2b3 2ba 2c0 2c5
2ca 2cf 2d4 2d9
2de 2e3 2ea 2fa
315 31f 326 32d
334 
1
4
0 
3669
0
1
a0
65
86
0 1 1 1 4 1 6 1
8 1 a 1 c 1 e 1
10 1 12 1 14 1 16 1
18 1 1a 1 1c 1 1e 1
20 1 22 1 24 1 26 1
28 1 2a 1 2c 1 2e 1
1 31 1 33 1 35 1 37
1 39 3a 1 3c 3d 1 3f
40 3f 42 3f 44 3f 46 3f
48 3f 4a 1 4c 1 4e 4f
4e 51 1 53 1 55 1 57
1 59 1 5b 1 5d 1 5f
1 61 1 63 1 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 400 ee 1004 0
0 0 0 0 0 0 0 0

2a3 1 0
6b 1 0
2b 1 0
e6 1 0
255 1 0
2a9 1 0
21b 1 0
bc 1 0
5f 1 0
270 1 0
4b 1 0
285 1 0
d8 1 0
2e5 1 0
2bc 1 0
1af 1 0
215 1 0
185 1 0
8b 1 0
29d 1 0
27 1 0
2cc 1 0
2ed 1 2
25b 1 0
1f 1 0
53 1 0
1fd 1 0
a7 1 0
1b 1 0
1f1 1 0
2f 1 0
2af 1 0
131 1 0
300 1 3
13f 1 0
fb 1 0
29db 3f 4c
16d 1 0
d1 1 0
138 1 0
173 1 0
63 1 0
43 1 0
23a 1 0
67 1 0
234 1 0
109 1 0
2c2 1 0
291 1 0
4f 1 0
2d6 1 0
1d3 1 0
13 1 0
1a9 1 0
ed 1 0
b5 1 0
19d 1 0
123 1 0
115 1 0
154 1 0
3f 1 0
248 1 0
1d9 1 0
221 1 0
10e 1 0
ae 1 0
3b 1 0
37 1 0
33 1 0
f 1 0
197 1 0
31a 1 0
92 1 0
3 1 0
c3 1 0
27e 1 0
15b 1 0
5b 1 0
328 1 0
146 1 0
321 1 0
22e 1 0
209 1 0
12a 1 0
11c 1 0
28b 1 0
18b 1 0
1f7 1 0
2 0 1
1c7 1 0
1bb 1 0
23 1 0
84 1 0
102 1 0
297 1 0
20f 1 0
7 1 0
1eb 1 0
1a3 1 0
2d1 1 0
191 1 0
a0 1 0
241 1 0
1df 1 0
179 1 0
277 1 0
269 1 0
203 1 0
161 1 0
24f 1 0
228 1 0
167 1 0
57 1 0
f4 1 0
7d 1 0
1c1 1 0
ca 1 0
47 1 0
17f 1 0
df 1 0
2db 1 0
1e5 1 0
17 1 0
2e0 1 0
14d 1 0
2b5 1 0
1cd 1 0
99 1 0
2c7 1 0
76 1 0
1b5 1 0
b 1 0
32f 1 0
262 1 0
0

/
SHOW ERRORS FUNCTION SP_SEARCH_2;
ALTER FUNCTION SP_SEARCH_2 COMPILE DEBUG;
/*#################################################################################################*/
/*#################################################################################################*/
/*
#Stored Procedure  :SP_SEARCH_1
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_SEARCH_1 ==
CREATE OR REPLACE FUNCTION SP_SEARCH_1 wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
29b
2 :e:
1FUNCTION:
1SP_SEARCH_1:
1PRJ_ID:
1VARCHAR2:
1START_DATE001:
1FINISH_DATE001:
1EFFORT001:
1PLS_INTEGER:
1WORK001:
1BITFLAG001:
1EUNIT:
1UNIT:
1CONTOUR001:
1OCCURRENCE:
1CONSIDER_PLANNED_SCHED:
1WORK_TYPE:
1TYPE001:
1AND_ROLES:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_PROJECT_ID:
132:
1H_PRJ_ASSIGN_ID:
1H_TASK_ID:
1H_UNIQUEID:
1H_REC_USER:
1H_RES_ID:
1H_SECURITY_ID:
1H_START_DATE:
126:
1H_START_DT:
1H_FINISH_DT:
1H_DAYOFYEAR:
1DATE:
1H_FINISH_D:
1H_RES_TYPE:
11:
1H_ASSIGN_ID:
1H_AND_ROLES:
18000:
1H_SHW_ROLE:
1H_SHW_SKILL:
1H_FULL_NAME:
1255:
1H_COUNT:
10:
1H_BITFLAG:
1H_TYPE:
1H_WORK_TYPE:
1H_CRI:
1H_WORK:
1H_OCCURRENCE:
1H_EFFORT:
1H_EUNIT:
1H_UNIT:
1H_CONSIDER:
1H_CONTOUR:
1H_EVERY:
1H_PER_DAY:
1H_PER_WEEK:
1H_UPDATE:
1H_DAYS:
1H_DURATION:
1H_PARTICIPANT:
1H_PRIORITY:
1H_TOTAL:
1H_TOTPR:
1H_FROM_PRJ:
1H_SCHED_WORK:
1H_JULIAN_DAY:
1H_J_EFECTIVE_DATE:
1H_J_DISCONTI_DATE:
1H_WEEK:
1H_ACTIVE:
1H_REC_STATUS:
1H_RES_COST:
1NUMBER:
10.00:
1STMT:
17999:
1H_TEMP_PARTICIPANT:
1H_TEMP_SECURITY_ID:
1CHAR:
1H_TEMP2_SECURITY_ID:
1H_CRI_SECURITY_ID:
1H_TEMP_BITFLAG:
1H_TEMP_SC_STATE:
1H_TABLE_TYPE:
1H_TASK_START_DT:
1H_TASK_FINISH_DT:
1H_TASK_TYPE:
1H_TEMP_VAR:
1ID:
1I:
1II:
1I_RETURN:
1ISF:
1NUMITEMS:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_SEARCH_1(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
1109:
1+:
1FPUT:
1               ENTER:: SP_SEARCH_1(110) :: cplan.sql(2611) :: :
1:
1GETTABLE:
1IS NULL:
18:
1150:
1      RETURN_INVALID:: SP_SEARCH_1(151) :: cplan.sql(2652) :: :
1 :
1STATE:: :
1PMO_CNST:
1ELEMENT_ID_REQUIRED:
1153:
1                EXIT:: SP_SEARCH_1(154) :: cplan.sql(2655) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1160:
1      RETURN_INVALID:: SP_SEARCH_1(161) :: cplan.sql(2662) :: :
1163:
1                EXIT:: SP_SEARCH_1(164) :: cplan.sql(2665) :: :
1=:
15:
1180:
1      RETURN_INVALID:: SP_SEARCH_1(181) :: cplan.sql(2682) :: :
1INVALID_PARAMS:
1183:
1                EXIT:: SP_SEARCH_1(184) :: cplan.sql(2685) :: :
1IS NOT NULL:
1PMO_ORASTD:
1DAYS:
1DUAL:
1SELECT PMO_ORASTD.DAYS(finish_date001) - PMO_ORASTD.DAYS(start_date001):n    +
1     INTO h_total:n         FROM DUAL:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
1201:
1           EXCEPTION:: SP_SEARCH_1(202) :: cplan.sql(2703) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SC_SQL_SUCCESS:
1<:
1208:
1      RETURN_INVALID:: SP_SEARCH_1(209) :: cplan.sql(2710) :: :
1F_DATE_EARLIER_THAN_S_DATE:
1211:
1                EXIT:: SP_SEARCH_1(212) :: cplan.sql(2713) :: :
1ELSIF:
1SELECT SYSTIMESTAMP INTO h_start_date:n         FROM DUAL:
1229:
1           EXCEPTION:: SP_SEARCH_1(230) :: cplan.sql(2731) :: :
1>:
1PMO_STPU:
1STRNPCPY:
133:
1ISPROJECTTYPE:
1!=:
1PROJECT_ID:
1MINUTE_PER_DAY:
1MINUTE_PER_WEEK:
1PRIORITY:
1TMT_WBS:
1ELEMENT_ID:
1SELECT PROJECT_ID, COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2+
1400), PRIORITY:n            INTO h_project_id, h_per_day, h_per_week, h_prior+
1ity:n            FROM TMT_WBS WHERE ELEMENT_ID = h_task_id:
1262:
1           EXCEPTION:: SP_SEARCH_1(263) :: cplan.sql(2764) :: :
1B:
1TYPE_ID:
1TASK_TYPE:
1UNRESOLVED_START:
1UNRESOLVED_FINISH:
1A:
1SC_TASK:
1SC_TASK_TYPE1:
1SC_TASK_TYPE2:
1SC_TASK_TYPE3:
1SC_MILESTONE:
1SC_PERSONAL_TASK:
1SC_ACTIVE_ADM_TASK:
1SC_INACTIVE_ADM_TASK:
1SC_WBS_CHANGE_REQUEST_TASK:
1SC_WBS_ACTION_ITEM_TASK:
1SC_WBS_RISK_TASK:
1SC_WBS_ISSUE_TASK:
1SC_WBS_REQUIREMENT_TASK:
1SC_WBS_DEFECT_TASK:
1SC_WBS_SERVICE_REQUEST_TASK:
1SELECT B.TYPE_ID, B.TASK_TYPE, B.UNRESOLVED_START, B.UNRESOLVED_FINISH, B.PRO+
1JECT_ID,:n            COALESCE(A.MINUTE_PER_DAY, 480), COALESCE(A.MINUTE_PER_+
1WEEK, 2400), A.PRIORITY:n            INTO h_type, h_task_type, h_task_start_D+
1T, h_task_finish_DT, h_project_id,:n            h_per_day, h_per_week, h_prio+
1rity FROM TMT_WBS A, TMT_WBS B:n            WHERE B.ELEMENT_ID = h_task_id AN+
1D B.TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE+
12,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_+
1CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_+
1REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_C+
1NST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT+
1_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) AND A.ELEMENT_ID = B.PROJECT_ID:
1281:
1           EXCEPTION:: SP_SEARCH_1(282) :: cplan.sql(2783) :: :
1SC_SQL_NO_DATA:
1WBS_ID:
1TMT_CRI:
1SELECT PROJECT_ID, WBS_ID INTO h_project_id, id:n            FROM TMT_CRI WHE+
1RE ELEMENT_ID = h_task_id:
1299:
1           EXCEPTION:: SP_SEARCH_1(300) :: cplan.sql(2801) :: :
1SELECT COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORI+
1TY:n               INTO h_per_day, h_per_week, h_priority:n               FRO+
1M TMT_WBS WHERE ELEMENT_ID = h_project_id:
1320:
1           EXCEPTION:: SP_SEARCH_1(321) :: cplan.sql(2822) :: :
1326:
1        RETURN_ERROR:: SP_SEARCH_1(327) :: cplan.sql(2828) :: :
1RECORD_NOT_FOUND:
1 ID:: :
1329:
1                EXIT:: SP_SEARCH_1(330) :: cplan.sql(2831) :: :
1PMO_RETURN_ERROR_SP:
1LEVEL_LOCK:
1PROJECT_DETAILS:
1SELECT LEVEL_LOCK INTO h_temp_var:n         FROM PROJECT_DETAILS:n         WH+
1ERE PROJECT_ID = h_project_id:
1350:
1           EXCEPTION:: SP_SEARCH_1(351) :: cplan.sql(2852) :: :
1REC_STATUS:
1SELECT REC_STATUS INTO h_rec_status:n            FROM TMT_WBS:n            WH+
1ERE ELEMENT_ID = h_project_id:
1368:
1           EXCEPTION:: SP_SEARCH_1(369) :: cplan.sql(2870) :: :
1STRCMP:
1L:
1375:
1        RETURN_ERROR:: SP_SEARCH_1(376) :: cplan.sql(2877) :: :
1LEV_LOCKED_BY_COPY:
1378:
1                EXIT:: SP_SEARCH_1(379) :: cplan.sql(2880) :: :
1SELECT COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORI+
1TY:n         INTO h_per_day, h_per_week, h_priority:n         FROM TMT_WBS WH+
1ERE ELEMENT_ID = h_project_id:
1396:
1           EXCEPTION:: SP_SEARCH_1(397) :: cplan.sql(2898) :: :
1TMT_POOLS:
1SELECT 480, 2400 INTO h_per_day, h_per_week FROM TMT_POOLS WHERE ELEMENT_ID =+
1 h_project_id:
1411:
1           EXCEPTION:: SP_SEARCH_1(412) :: cplan.sql(2913) :: :
1417:
1        RETURN_ERROR:: SP_SEARCH_1(418) :: cplan.sql(2919) :: :
1420:
1                EXIT:: SP_SEARCH_1(421) :: cplan.sql(2922) :: :
1FC_CALC_DURATION:
1451:
1      RETURN_INVALID:: SP_SEARCH_1(452) :: cplan.sql(2953) :: :
1454:
1                EXIT:: SP_SEARCH_1(455) :: cplan.sql(2956) :: :
1459:
1        RETURN_ERROR:: SP_SEARCH_1(460) :: cplan.sql(2961) :: :
1462:
1                EXIT:: SP_SEARCH_1(463) :: cplan.sql(2964) :: :
1100:
12:
13:
14:
1*:
1FC_DECLARE_PMO_TMP_SEARCH:
1PMO_FUNC:
1SHIFTRIGHT:
1BITAND:
1SC_C0000000:
130:
1SC_1FFFFFFF:
1ASSIGNRESOURCE:
1        RETURN_ERROR:: SP_SEARCH_1(501) :: cplan.sql(3002) :: :
1503:
1                EXIT:: SP_SEARCH_1(504) :: cplan.sql(3005) :: :
1MIN:
1BAK_START_DT:
1TASK_ASSIGNMENTS:
1TASK_ID:
1RESOURCE_ID:
1RESOURCE_TYPE:
1SELECT MIN(BAK_START_DT), PMO_ORASTD.DAYS(h_start_date) - PMO_ORASTD.DAYS(MIN+
1(BAK_START_DT)):n         INTO h_start_DT, h_total :n         FROM TASK_ASSIG+
1NMENTS:n         WHERE PROJECT_ID = h_project_id :n         AND TASK_ID = h_p+
1roject_id :n         AND RESOURCE_ID = h_res_id :n         AND RESOURCE_TYPE +
1= h_res_type:
1526:
1           EXCEPTION:: SP_SEARCH_1(527) :: cplan.sql(3028) :: :
1PROJECT_ASSIGNMENT_ID:
1SCHED_START:
1TO_DATE:
1SCHED_FINISH:
1SCHED_WORK:
1BIT_FLAG:
1SELECT PROJECT_ASSIGNMENT_ID, SCHED_START, CASE WHEN TO_DATE(SubStr(h_start_d+
1ate, 1, 10)) >= TO_DATE(SubStr(SCHED_START, 1, 10)) AND:n               TO_DA+
1TE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(SCHED_FINISH, 1, 10)) AND SC+
1HED_WORK >= h_effort THEN 0 ELSE 1 END,SCHED_WORK:n               INTO h_prj_+
1assign_id, h_start_DT, h_total, h_sched_work:n               FROM TASK_ASSIGN+
1MENTS:n               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND+
1 RESOURCE_TYPE='0' AND BIT_FLAG = '2':
1556:
1           EXCEPTION:: SP_SEARCH_1(557) :: cplan.sql(3058) :: :
1PLANNED_START:
1PLANNED_FINISH:
1PLANNED_WORK:
1SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start+
1_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND:n                +
1     TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1,+
1 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END:n                     IN+
1TO h_prj_assign_id, h_start_DT, h_total :n                     FROM TASK_ASSI+
1GNMENTS:n                     WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_re+
1s_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0':
1578:
1           EXCEPTION:: SP_SEARCH_1(579) :: cplan.sql(3080) :: :
1SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start+
1_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND:n                +
1  TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10+
1)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END:n                  INTO h_p+
1rj_assign_id, h_start_DT, h_total :n                  FROM TASK_ASSIGNMENTS:n+
1                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RES+
1OURCE_TYPE='0' AND BIT_FLAG = '0':
1600:
1           EXCEPTION:: SP_SEARCH_1(601) :: cplan.sql(3102) :: :
1SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start+
1_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND:n               T+
1O_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) +
1AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END:n               INTO h_prj_ass+
1ign_id, h_start_DT, h_total :n               FROM TASK_ASSIGNMENTS:n         +
1      WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='+
11' AND BIT_FLAG = '0':
1620:
1           EXCEPTION:: SP_SEARCH_1(621) :: cplan.sql(3122) :: :
1SELECT PROJECT_ASSIGNMENT_ID, SCHED_START,CASE WHEN TO_DATE(SubStr(h_start_da+
1te, 1, 10)) BETWEEN TO_DATE(SubStr(SCHED_START, 1, 10)) AND TO_DATE(SubStr(SC+
1HED_FINISH, 1, 10)) AND h_effort <= SCHED_WORK THEN 0 ELSE 1 END, SCHED_WORK+
1:n               INTO h_prj_assign_id, h_start_DT,h_total, h_sched_work :n   +
1            FROM TASK_ASSIGNMENTS:n               WHERE TASK_ID=h_project_id +
1AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='2':
1640:
1           EXCEPTION:: SP_SEARCH_1(641) :: cplan.sql(3142) :: :
1SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start+
1_date, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLAN+
1NED_WORK THEN 0 ELSE 1 END:n                     INTO h_prj_assign_id, h_star+
1t_DT,h_total :n                     FROM TASK_ASSIGNMENTS:n                  +
1   WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' +
1AND BIT_FLAG = '0':
1661:
1           EXCEPTION:: SP_SEARCH_1(662) :: cplan.sql(3163) :: :
1SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START,CASE WHEN TO_DATE(SubStr(h_start_+
1date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubSt+
1r(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END:n   +
1               INTO h_prj_assign_id, h_start_DT,h_total :n                  F+
1ROM TASK_ASSIGNMENTS:n                  WHERE TASK_ID=h_project_id AND RESOUR+
1CE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0':
1683:
1           EXCEPTION:: SP_SEARCH_1(684) :: cplan.sql(3185) :: :
1SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START,CASE WHEN TO_DATE(SubStr(h_start_+
1date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubSt+
1r(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END:n   +
1            INTO h_prj_assign_id, h_start_DT,h_total :n               FROM TA+
1SK_ASSIGNMENTS:n               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_r+
1es_id AND RESOURCE_TYPE='1' AND BIT_FLAG = '0':
1702:
1           EXCEPTION:: SP_SEARCH_1(703) :: cplan.sql(3204) :: :
1712:
1        RETURN_ERROR:: SP_SEARCH_1(713) :: cplan.sql(3214) :: :
1TASK_ASSIGNMENT_ERROR:
1715:
1                EXIT:: SP_SEARCH_1(716) :: cplan.sql(3217) :: :
1SUM:
1SCHED_MIN:
1TASK_DAY_OF_YEAR:
1TSK_ASSIGNMENT_ID:
1DAY_YEAR:
1SELECT SUM(SCHED_MIN) INTO h_sched_work :n                  FROM TASK_DAY_OF_+
1YEAR:n                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID+
1 = h_prj_assign_id :n                  AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_+
1start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10)):
1734:
1           EXCEPTION:: SP_SEARCH_1(735) :: cplan.sql(3236) :: :
1SELECT SUM(SCHED_MIN) INTO h_sched_work :n                  FROM TASK_DAY_OF_+
1YEAR:n                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID+
1 = h_prj_assign_id :n                  AND DAY_YEAR >= TO_DATE(SubStr(h_start+
1_date, 1, 10)):
1753:
1           EXCEPTION:: SP_SEARCH_1(754) :: cplan.sql(3255) :: :
1761:
1        RETURN_ERROR:: SP_SEARCH_1(762) :: cplan.sql(3263) :: :
1764:
1                EXIT:: SP_SEARCH_1(765) :: cplan.sql(3266) :: :
1ISNOTPROJECTTYPE:
1772:
1        RETURN_ERROR:: SP_SEARCH_1(773) :: cplan.sql(3274) :: :
1775:
1                EXIT:: SP_SEARCH_1(776) :: cplan.sql(3277) :: :
1NICK_NAME:
1JULIAN_DAY:
1ACTIVE:
1EFFECTIVE_DATE:
1DISCONTINUE_DATE:
1RESOURCES:
1SELECT NICK_NAME, PMO_ORASTD.JULIAN_DAY(h_start_DT), ACTIVE, COALESCE(PMO_ORA+
1STD.JULIAN_DAY(EFFECTIVE_DATE),0),COALESCE(PMO_ORASTD.JULIAN_DAY(DISCONTINUE_+
1DATE),0):n            INTO h_full_name, h_julian_day, h_active, h_j_efective_+
1date, h_j_disconti_date:n            FROM RESOURCES :n            WHERE RESOU+
1RCE_ID = h_res_id:
1796:
1           EXCEPTION:: SP_SEARCH_1(797) :: cplan.sql(3298) :: :
1803:
1        RETURN_ERROR:: SP_SEARCH_1(804) :: cplan.sql(3305) :: :
1806:
1                EXIT:: SP_SEARCH_1(807) :: cplan.sql(3308) :: :
1814:
1        RETURN_ERROR:: SP_SEARCH_1(815) :: cplan.sql(3316) :: :
1RESOURCE_NOT_ACTIVE:
1817:
1                EXIT:: SP_SEARCH_1(818) :: cplan.sql(3319) :: :
1RES_ASSIGNMENT_ID:
1PARTICIPANT:
1SECURITY_ID:
1RESOURCE_ASSIGNMENTS:
1SELECT RES_ASSIGNMENT_ID, PARTICIPANT, SECURITY_ID INTO h_assign_id, h_partic+
1ipant, h_security_id:n            FROM RESOURCE_ASSIGNMENTS:n            WHER+
1E ELEMENT_ID = h_task_id AND RESOURCE_TYPE = '0' AND RESOURCE_ID = h_res_id:
1834:
1           EXCEPTION:: SP_SEARCH_1(835) :: cplan.sql(3336) :: :
1PROFILE_NAME:
1PROFILES:
1PROFILE_ID:
1SELECT PROFILE_NAME INTO h_full_name FROM PROFILES WHERE PROFILE_ID = h_res_i+
1d:
1851:
1           EXCEPTION:: SP_SEARCH_1(852) :: cplan.sql(3353) :: :
1858:
1        RETURN_ERROR:: SP_SEARCH_1(859) :: cplan.sql(3360) :: :
1861:
1                EXIT:: SP_SEARCH_1(862) :: cplan.sql(3363) :: :
1BAK_FINISH_DT:
1BAK_WORK:
1SELECT BAK_START_DT, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN T+
1O_DATE(SubStr(BAK_START_DT, 1, 10)) AND TO_DATE(SubStr(BAK_FINISH_DT, 1, 10))+
1 AND BAK_WORK >= h_effort THEN 0 ELSE 1 END:n            INTO h_start_DT, h_t+
1otal :n            FROM TASK_ASSIGNMENTS:n            WHERE TASK_ID=h_project+
1_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1':
1879:
1           EXCEPTION:: SP_SEARCH_1(880) :: cplan.sql(3381) :: :
1886:
1        RETURN_ERROR:: SP_SEARCH_1(887) :: cplan.sql(3388) :: :
1889:
1                EXIT:: SP_SEARCH_1(890) :: cplan.sql(3391) :: :
1SELECT RES_ASSIGNMENT_ID, PARTICIPANT, SECURITY_ID INTO h_assign_id, h_partic+
1ipant, h_security_id:n            FROM RESOURCE_ASSIGNMENTS:n            WHER+
1E ELEMENT_ID = h_task_id AND RESOURCE_TYPE = '1' AND RESOURCE_ID = h_res_id:
1906:
1           EXCEPTION:: SP_SEARCH_1(907) :: cplan.sql(3408) :: :
1DB2TIMESTAMP:
1TIME:
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)), PMO_ORASTD+
1.TIME(h_start_date)) INTO h_start_DT:n            FROM DUAL:
1926:
1           EXCEPTION:: SP_SEARCH_1(927) :: cplan.sql(3428) :: :
1SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) > TO_DATE(SubStr(h_star+
1t_DT, 1, 10)) THEN h_start_date:n            ELSE h_start_DT END:n           +
1 INTO h_start_DT:n            FROM DUAL:
1944:
1           EXCEPTION:: SP_SEARCH_1(945) :: cplan.sql(3446) :: :
110:
1FC_SEARCH_2:
1PMO_TMP_SEARCH:
1PLANNED_SCHED_FLAG:
1RES_ID:
1UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id:
1FOUND:
1977:
1           EXCEPTION:: SP_SEARCH_1(978) :: cplan.sql(3479) :: :
1UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='1' WHERE RES_ID=h_res_id:
1996:
1           EXCEPTION:: SP_SEARCH_1(997) :: cplan.sql(3498) :: :
1SC_SRCH_PROFILE:
11034:
1           EXCEPTION:: SP_SEARCH_1(1035) :: cplan.sql(3536) :: :
1LENGTH:
1TRUNC:
1/:
1WHILE:
1LOOP:
1COUNT:
1PROPOSED_START:
1SELECT COUNT(*), MIN(PROPOSED_START) INTO h_total, h_start_DT :n             +
1  FROM TASK_ASSIGNMENTS:n               WHERE PROJECT_ID = h_project_id AND T+
1ASK_ID = h_project_id:n               AND RESOURCE_ID = h_res_id AND RESOURCE+
1_TYPE = h_res_type:
11076:
1           EXCEPTION:: SP_SEARCH_1(1077) :: cplan.sql(3578) :: :
1',':
1SELECT PROJECT_ASSIGNMENT_ID, SCHED_START, CASE WHEN TO_DATE(SubStr(h_start_d+
1ate, 1, 10)) >= TO_DATE(SubStr(SCHED_START, 1, 10)) AND:n                    +
1    TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(SCHED_FINISH, 1, 10+
1)) AND SCHED_WORK >= h_effort THEN 0 ELSE 1 END,SCHED_WORK:n                 +
1       INTO h_prj_assign_id, h_start_DT, h_total, h_sched_work:n             +
1           FROM TASK_ASSIGNMENTS:n                        WHERE TASK_ID=h_pro+
1ject_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '2':
11125:
1           EXCEPTION:: SP_SEARCH_1(1126) :: cplan.sql(3627) :: :
1SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start+
1_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND:n                +
1              TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_F+
1INISH, 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END:n              +
1                INTO h_prj_assign_id, h_start_DT, h_total :n                 +
1             FROM TASK_ASSIGNMENTS:n                              WHERE TASK_+
1ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG =+
1 '0':
11147:
1           EXCEPTION:: SP_SEARCH_1(1148) :: cplan.sql(3649) :: :
1SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start+
1_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND:n                +
1           TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINI+
1SH, 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END:n                 +
1          INTO h_prj_assign_id, h_start_DT, h_total :n                       +
1    FROM TASK_ASSIGNMENTS:n                           WHERE TASK_ID=h_project+
1_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0':
11169:
1           EXCEPTION:: SP_SEARCH_1(1170) :: cplan.sql(3671) :: :
1SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start+
1_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND:n                +
1        TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH,+
1 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END:n                    +
1    INTO h_prj_assign_id, h_start_DT, h_total FROM TASK_ASSIGNMENTS:n        +
1                WHERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id:n   +
1                     AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1' AND BIT_F+
1LAG = '0':
11189:
1           EXCEPTION:: SP_SEARCH_1(1190) :: cplan.sql(3691) :: :
1SELECT PROJECT_ASSIGNMENT_ID, SCHED_START,CASE WHEN TO_DATE(SubStr(h_start_da+
1te, 1, 10)) BETWEEN TO_DATE(SubStr(SCHED_START, 1, 10)) AND TO_DATE(SubStr(SC+
1HED_FINISH, 1, 10)) AND h_effort <= SCHED_WORK THEN 0 ELSE 1 END, SCHED_WORK+
1:n                        INTO h_prj_assign_id, h_start_DT,h_total, h_sched_w+
1ork FROM TASK_ASSIGNMENTS:n                        WHERE PROJECT_ID = h_proje+
1ct_id AND TASK_ID=h_project_id:n                        AND RESOURCE_ID=h_res+
1_id AND RESOURCE_TYPE='0' AND BIT_FLAG='2':
11209:
1           EXCEPTION:: SP_SEARCH_1(1210) :: cplan.sql(3711) :: :
1SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start+
1_date, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLAN+
1NED_WORK THEN 0 ELSE 1 END:n                              INTO h_prj_assign_i+
1d, h_start_DT,h_total FROM TASK_ASSIGNMENTS:n                              WH+
1ERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id:n                     +
1         AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0':
11229:
1           EXCEPTION:: SP_SEARCH_1(1230) :: cplan.sql(3731) :: :
1SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start+
1_date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubS+
1tr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END:n  +
1                         INTO h_prj_assign_id, h_start_DT,h_total FROM TASK_A+
1SSIGNMENTS:n                           WHERE PROJECT_ID = h_project_id AND TA+
1SK_ID=h_project_id:n                           AND RESOURCE_ID=h_res_id AND R+
1ESOURCE_TYPE='0' AND BIT_FLAG = '0':
11251:
1           EXCEPTION:: SP_SEARCH_1(1252) :: cplan.sql(3753) :: :
1SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START,CASE WHEN TO_DATE(SubStr(h_start_+
1date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubSt+
1r(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END:n   +
1                     INTO h_prj_assign_id, h_start_DT,h_total FROM TASK_ASSIG+
1NMENTS:n                        WHERE PROJECT_ID = h_project_id AND TASK_ID=h+
1_project_id:n                        AND RESOURCE_ID=h_res_id AND RESOURCE_TY+
1PE='1' AND BIT_FLAG = '0':
11270:
1           EXCEPTION:: SP_SEARCH_1(1271) :: cplan.sql(3772) :: :
1SELECT SUM(SCHED_MIN) INTO h_sched_work :n                              FROM +
1TASK_DAY_OF_YEAR:n                              WHERE PROJECT_ID = h_project_+
1id AND TSK_ASSIGNMENT_ID = h_prj_assign_id :n                              AN+
1D DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_+
1finish_DT, 1, 10)):
11297:
1           EXCEPTION:: SP_SEARCH_1(1298) :: cplan.sql(3799) :: :
1SELECT SUM(SCHED_MIN) INTO h_sched_work :n                              FROM +
1TASK_DAY_OF_YEAR:n                              WHERE PROJECT_ID = h_project_+
1id AND TSK_ASSIGNMENT_ID = h_prj_assign_id :n                              AN+
1D DAY_YEAR >= TO_DATE(SubStr(h_start_date, 1, 10)):
11316:
1           EXCEPTION:: SP_SEARCH_1(1317) :: cplan.sql(3818) :: :
1CONTINUE0000:
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)), PMO_ORASTD+
1.TIME(h_start_date)):n                  INTO h_start_DT:n                  FR+
1OM DUAL:
11348:
1           EXCEPTION:: SP_SEARCH_1(1349) :: cplan.sql(3850) :: :
1SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) > TO_DATE(SubStr(h_star+
1t_DT, 1, 10)) THEN h_start_date ELSE h_start_DT END:n                  INTO h+
1_start_DT:n                  FROM DUAL:
11365:
1           EXCEPTION:: SP_SEARCH_1(1366) :: cplan.sql(3867) :: :
11398:
1           EXCEPTION:: SP_SEARCH_1(1399) :: cplan.sql(3900) :: :
11417:
1           EXCEPTION:: SP_SEARCH_1(1418) :: cplan.sql(3919) :: :
11453:
1           EXCEPTION:: SP_SEARCH_1(1454) :: cplan.sql(3955) :: :
1DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id:
11483:
1           EXCEPTION:: SP_SEARCH_1(1484) :: cplan.sql(3985) :: :
1UPDATE RESOURCE_ASSIGNMENTS SET SECURITY_ID = h_security_id, PARTICIPANT = h_+
1participant:n            WHERE RES_ASSIGNMENT_ID = h_assign_id:
11503:
1           EXCEPTION:: SP_SEARCH_1(1504) :: cplan.sql(4005) :: :
11511:
1        RETURN_ERROR:: SP_SEARCH_1(1512) :: cplan.sql(4013) :: :
11514:
1                EXIT:: SP_SEARCH_1(1515) :: cplan.sql(4016) :: :
1CANNOT_ASSIGN_TO_DEFAULT_CRI:
11519:
1        RETURN_ERROR:: SP_SEARCH_1(1520) :: cplan.sql(4021) :: :
11522:
1                EXIT:: SP_SEARCH_1(1523) :: cplan.sql(4024) :: :
1CANNOT_CHANGE_OBJECT_TYPE:
11528:
1        RETURN_ERROR:: SP_SEARCH_1(1529) :: cplan.sql(4030) :: :
11531:
1                EXIT:: SP_SEARCH_1(1532) :: cplan.sql(4033) :: :
11537:
1        RETURN_ERROR:: SP_SEARCH_1(1538) :: cplan.sql(4039) :: :
11540:
1                EXIT:: SP_SEARCH_1(1541) :: cplan.sql(4042) :: :
1EXECUTE:
1IMMEDIATE:
1DELETE FROM PMO_TMP_CPLN_0003:
1INSERT INTO PMO_TMP_CPLN_0003 SELECT RESOURCE_ID, RESOURCE_TYPE, :
1 MIN(PLANNED_START) AS PLANNED_START,  MAX(PLANNED_FINISH) AS PLANNED_FINISH,+
1 :
1 SUM(PLANNED_WORK) AS PLANNED_EFFORT,  MIN(SCHED_START) AS START_DATE,  :
1 MAX(SCHED_FINISH) AS FINISH_DATE,     SUM(SCHED_WORK) AS WORK,  :
1 SUM(ETC) AS REMAINING :
1 FROM TASK_ASSIGNMENTS WHERE PROJECT_ID=':
1' AND :
1      (RESOURCE_ID IN (':
1') OR RESOURCE_ID IN (':
1')) GROUP BY RESOURCE_ID, RESOURCE_TYPE :
11574:
1           EXCEPTION:: SP_SEARCH_1(1575) :: cplan.sql(4076) :: :
1SQL :: :
1DELETE FROM PMO_TMP_CPLN_0002:
1INSERT INTO PMO_TMP_CPLN_0002 SELECT RESOURCE_ID AS PROJECT_ASSIGNMENT_ID,MIN+
1(PLANNED_START) AS PRJ_PLANNED_START, :
1 MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH,  SUM(PLANNED_WORK) AS PLANNED_REM+
1AINING, :
1 MIN(SCHED_START) AS PRJ_SCHED_START,        MAX(SCHED_FINISH) AS PRJ_SCHED_F+
1INISH, :
1 SUM(SCHED_WORK) AS SCHED_REMAINING  :
1 FROM TASK_ASSIGNMENTS :
1 WHERE TASK_ID=':
1 (RESOURCE_ID IN (':
1')) GROUP BY RESOURCE_ID :
11608:
1           EXCEPTION:: SP_SEARCH_1(1609) :: cplan.sql(4110) :: :
1FC_U_CREATETMPTABLE:
1INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME,RESOURCE_TYPE,TYPE_ID+
1,E_MAIL_ADDRESS,PLANNED_START,:
1PLANNED_FINISH,PLANNED_EFFORT,START_DATE,FINISH_DATE,WORK,REMAINING,BEST_STAR+
1T,BEST_FINISH,:
1ASSIGNABLE,PLANNED,SCHED,WORK_EFFORT,PERIOD_AVAILABILITY,PROJECT_ASSIGNMENT_I+
1D,PRJ_PLANNED_START,:
1PRJ_PLANNED_FINISH,PLANNED_REMAINING,PRJ_SCHED_START,PRJ_SCHED_FINISH,SCHED_R+
1EMAINING,REGULAR_COST,:
1REGULAR_RATE,SPECIAL_COST,SPECIAL_RATE) :
1SELECT DISTINCT r.RESOURCE_ID,r.FULL_NAME AS ELEMENT_NAME,'0',:
1CASE WHEN ta.TSK_ASSIGNMENT_ID IS NuLL THEN 185 ELSE 13 END AS TYPE_ID,:
1r.E_MAIL_ADDRESS, a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.STAR+
1T_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,:
1rs.START_DATE, rs.FINISH_DATE, rs.ASSIGNABLE, rs.PLANNED, rs.SCHED, rs.PROPOS+
1ED, (rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) ,:
1tb.PROJECT_ASSIGNMENT_ID,tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLAN+
1NED_REMAINING,:
1tb.PRJ_SCHED_START, tb.PRJ_SCHED_FINISH, tb.SCHED_REMAINING, pa.REGULAR_COST,+
1 pa.REGULAR_RATE, pa.SPECIAL_COST, pa.SPECIAL_RATE :
1FROM RESOURCES r LEFT OUTER JOIN PMO_TMP_CPLN_0003 a ON a.RESOURCE_TYPE='0' A+
1ND a.RESOURCE_ID=r.RESOURCE_ID :
1LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=':
1' AND r.RESOURCE_ID = pa.RESOURCE_ID :
1LEFT OUTER JOIN TASK_ASSIGNMENTS ta ON ta.PROJECT_ID=':
1' AND r.RESOURCE_ID = ta.RESOURCE_ID :
1LEFT OUTER JOIN PMO_TMP_CPLN_0002 tb ON r.RESOURCE_ID = tb.PROJECT_ASSIGNMENT+
1_ID :
1LEFT OUTER JOIN PMO_TMP_SEARCH rs ON r.RESOURCE_ID=rs.RES_ID WHERE r.RESOURCE+
1_ID IN (':
1') :
1UNION ALL :
1SELECT DISTINCT p.PROFILE_ID , p.PROFILE_NAME AS ELEMENT_NAME, '1', 41 AS TYP+
1E_ID,:
1'', a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FINI+
1SH_DATE, a.WORK, a.REMAINING,:
1rs.START_DATE , rs.FINISH_DATE, rs.ASSIGNABLE ,rs.PLANNED,rs.SCHED, rs.PROPOS+
1ED,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) ,:
1FROM PROFILES p LEFT OUTER JOIN PMO_TMP_CPLN_0003 a ON a.RESOURCE_TYPE='1' AN+
1D p.PROFILE_ID=a.RESOURCE_ID :
1LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID= ':
1' AND p.PROFILE_ID = pa.RESOURCE_ID :
1JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=p.ROLE_ID :
1JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID LEFT OUTER JOIN PMO_TMP_S+
1EARCH rs ON a.RESOURCE_ID=rs.RES_ID :
1LEFT OUTER JOIN PMO_TMP_CPLN_0002 tb ON p.PROFILE_ID = tb.PROJECT_ASSIGNMENT_+
1ID :
1WHERE p.PROFILE_ID IN (':
1ORDER BY TYPE_ID,ELEMENT_NAME:
11665:
1           EXCEPTION:: SP_SEARCH_1(1666) :: cplan.sql(4167) :: :
1SELECT T.ELEMENT_ID, '' AS RES_ASSIGNMENT_ID, T.ELEMENT_NAME AS ELEMENT_NAME,+
1 T.TYPE_ID AS TYPE_ID, :
1 T.E_MAIL_ADDRESS, T.PLANNED_START, T.PLANNED_FINISH, T.PLANNED_EFFORT, T.STA+
1RT_DATE, T.FINISH_DATE, T.WORK, T.REMAINING, :
1 T.BEST_START, T.BEST_FINISH, T.ASSIGNABLE AS CALENDAR,T.PLANNED AS PLANNED_W+
1ORK,:
1 T.SCHED AS SCHED_WORK, T.WORK_EFFORT, :
1 AS UNIT_EFFORT,T.PERIOD_AVAILABILITY, :
1 AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, '' AS OBJECTIVES,'' AS RANK,+
1T.PROJECT_ASSIGNMENT_ID,T.PRJ_PLANNED_START, T.PRJ_PLANNED_FINISH, T.PLANNED_+
1REMAINING, :
1T.PRJ_SCHED_START, T.PRJ_SCHED_FINISH, T.SCHED_REMAINING, T.REGULAR_COST, T.R+
1EGULAR_RATE, T.SPECIAL_COST, T.SPECIAL_RATE :
1 FROM PMO_TMP_CPLN01_TEMP T:
1 LEFT OUTER JOIN PMO_TMP_SEARCH rs ON T.ELEMENT_ID=rs.RES_ID WHERE T.RESOURCE+
1_TYPE IN ('0') :
1 UNION ALL :
1 SELECT T.ELEMENT_ID, '' AS RES_ASSIGNMENT_ID, T.ELEMENT_NAME, T.TYPE_ID, :
1 '' AS E_MAIL_ADDRESS, T.PLANNED_START, T.PLANNED_FINISH, T.PLANNED_EFFORT, T+
1.START_DATE, T.FINISH_DATE, T.WORK, T.REMAINING, :
1 T.BEST_START, T.BEST_FINISH, T.ASSIGNABLE AS CALENDAR, :
1 T.PLANNED AS PLANNED_WORK,T.SCHED AS SCHED_WORK, T.WORK_EFFORT, :
1 AS UNIT_EFFORT, :
1 T.PERIOD_AVAILABILITY, :
1 AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING,'' AS OBJECTIVES,'' AS RANK, :
1 T.PROJECT_ASSIGNMENT_ID,T.PRJ_PLANNED_START, T.PRJ_PLANNED_FINISH, T.PLANNED+
1_REMAINING, :
1  T.PRJ_SCHED_START, T.PRJ_SCHED_FINISH, T.SCHED_REMAINING, T.REGULAR_COST, T+
1.REGULAR_RATE, T.SPECIAL_COST, T.SPECIAL_RATE :
1 FROM PMO_TMP_CPLN01_TEMP T LEFT OUTER JOIN PMO_TMP_SEARCH rs ON T.ELEMENT_ID+
1=rs.RES_ID :
1 WHERE T.RESOURCE_TYPE = '1' ORDER BY TYPE_ID,ELEMENT_NAME:
1DELETE FROM PMO_TMP_CPLN_0001:
1INSERT INTO PMO_TMP_CPLN_0001 :
1SELECT  RESOURCE_ID, MIN(PLANNED_START) AS PLANNED_START, :
1 MAX(PLANNED_FINISH) AS PLANNED_FINISH,   SUM(PLANNED_WORK) AS PLANNED_EFFORT+
1, :
1 MIN(SCHED_START) AS START_DATE,          MAX(SCHED_FINISH) AS FINISH_DATE, :
1 SUM(SCHED_WORK) AS WORK,                 SUM(ETC) AS REMAINING :
1 RESOURCE_ID IN (':
1') AND PROJECT_ID<>TASK_ID GROUP BY RESOURCE_ID:
11721:
1           EXCEPTION:: SP_SEARCH_1(1722) :: cplan.sql(4223) :: :
1INSERT INTO PMO_TMP_CPLN_0002 :
1SELECT RESOURCE_ID AS PROJECT_ASSIGNMENT_ID,      MIN(PLANNED_START) AS PRJ_P+
1LANNED_START, :
1 MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH, SUM(PLANNED_WORK) AS PLANNED_REMA+
1INING, :
1 MIN(SCHED_START) AS PRJ_SCHED_START,       MAX(SCHED_FINISH) AS PRJ_SCHED_FI+
1NISH, :
1 SUM(SCHED_WORK) AS SCHED_REMAINING :
1 FROM TASK_ASSIGNMENTS WHERE TASK_ID=':
1' AND RESOURCE_ID IN (':
1') GROUP BY RESOURCE_ID:
11753:
1           EXCEPTION:: SP_SEARCH_1(1754) :: cplan.sql(4255) :: :
1PLANNED_FINISH,PLANNED_EFFORT,START_DATE,FINISH_DATE,WORK,REMAINING,PROJECT_A+
1SSIGNMENT_ID,PRJ_PLANNED_START,:
1REGULAR_RATE,SPECIAL_COST,SPECIAL_RATE):
1SELECT DISTINCT r.RESOURCE_ID, r.FULL_NAME,'0',CASE WHEN ta.TSK_ASSIGNMENT_ID+
1 IS NuLL THEN 185 ELSE 13 END ,:
1tb.PROJECT_ASSIGNMENT_ID, tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLA+
1NNED_REMAINING,:
1FROM RESOURCES r LEFT OUTER JOIN PMO_TMP_CPLN_0001 a ON a.RESOURCE_ID=r.RESOU+
1RCE_ID :
1WHERE r.RESOURCE_ID IN (':
1'):
11791:
1           EXCEPTION:: SP_SEARCH_1(1792) :: cplan.sql(4293) :: :
1SELECT T.ELEMENT_ID, '' AS RES_ASSIGNMENT_ID, T.ELEMENT_NAME AS ELEMENT_NAME,+
1 T.TYPE_ID, :
1T.E_MAIL_ADDRESS, T.PLANNED_START, T.PLANNED_FINISH, T.PLANNED_EFFORT, T.STAR+
1T_DATE, T.FINISH_DATE, T.WORK, T.REMAINING, :
1rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS +
1CALENDAR,rs.PLANNED AS PLANNED_WORK, :
1rs.SCHED AS SCHED_WORK, rs.PROPOSED AS WORK_EFFORT, :
1 AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILIT+
1Y, :
1 AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, '' AS OBJECTIVES,'' AS RANK,+
1T.PROJECT_ASSIGNMENT_ID, T.PRJ_PLANNED_START, T.PRJ_PLANNED_FINISH, T.PLANNED+
1_REMAINING, :
1FROM PMO_TMP_CPLN01_TEMP T :
1LEFT OUTER JOIN PMO_TMP_SEARCH rs ON T.ELEMENT_ID=rs.RES_ID :
1ORDER BY ELEMENT_NAME:
1SELECT RESOURCE_ID, MIN(PLANNED_START) AS PLANNED_START, :
1       MAX(PLANNED_FINISH) AS PLANNED_FINISH,    SUM(PLANNED_WORK) AS PLANNED+
1_EFFORT, :
1       MIN(SCHED_START) AS START_DATE,           MAX(SCHED_FINISH) AS FINISH_+
1DATE, :
1       SUM(SCHED_WORK) AS WORK,                  SUM(ETC) AS REMAINING :
1 WHERE PROJECT_ID=':
1'        AND :
1       RESOURCE_ID IN (':
1')   AND :
1       PROJECT_ID<>TASK_ID GROUP BY RESOURCE_ID :
11838:
1           EXCEPTION:: SP_SEARCH_1(1839) :: cplan.sql(4340) :: :
1SELECT    RESOURCE_ID AS PROJECT_ASSIGNMENT_ID,          MIN(PLANNED_START) A+
1S PRJ_PLANNED_START, :
1          MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH,     SUM(PLANNED_WORK) AS+
1 PLANNED_REMAINING, :
1          MIN(SCHED_START) AS PRJ_SCHED_START,           MAX(SCHED_FINISH) AS+
1 PRJ_SCHED_FINISH, :
1          SUM(SCHED_WORK) AS SCHED_REMAINING  :
1') GROUP BY RESOURCE_ID :
11870:
1           EXCEPTION:: SP_SEARCH_1(1871) :: cplan.sql(4372) :: :
1SELECT DISTINCT p.PROFILE_ID , p.PROFILE_NAME ,'1', 41 ,:
1'' , a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FIN+
1ISH_DATE, a.WORK, a.REMAINING,:
1FROM PROFILES p LEFT OUTER JOIN PMO_TMP_CPLN_0001 a ON p.PROFILE_ID=a.RESOURC+
1E_ID JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=p.ROLE_ID :
1JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID :
11908:
1           EXCEPTION:: SP_SEARCH_1(1909) :: cplan.sql(4410) :: :
1SELECT T.ELEMENT_ID, '' AS RES_ASSIGNMENT_ID, T.ELEMENT_NAME, T.TYPE_ID, :
1rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS +
1CALENDAR, :
1rs.PLANNED AS PLANNED_WORK,rs.SCHED AS SCHED_WORK, rs.PROPOSED AS WORK_EFFORT+
1, :
1(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, :
1 AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, '' AS OBJECTIVES,'' AS RANK,+
1  :
1T.PROJECT_ASSIGNMENT_ID,T.PRJ_PLANNED_START, T.PRJ_PLANNED_FINISH, T.PLANNED_+
1REMAINING, :
1FROM PMO_TMP_CPLN01_TEMP T LEFT OUTER JOIN PMO_TMP_SEARCH rs ON T.ELEMENT_ID=+
1rs.RES_ID :
11930:
1                EXIT:: SP_SEARCH_1(1931) :: cplan.sql(4432) :: :
11933:
1                EXIT:: SP_SEARCH_1(1934) :: cplan.sql(4435) :: :
1PMO_SELECT_READ:
1RETURN_ERROR_EXCEPTION:
11941:
1           EXCEPTION:: SP_SEARCH_1(1942) :: cplan.sql(4443) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
11953:
1           EXCEPTION:: SP_SEARCH_1(1954) :: cplan.sql(4455) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
31f0
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 1c 81 b0 a3
a0 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 1c 51
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 1c 51 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 81 b0 a3 :2 a0 6b 1c
81 b0 :2 a0 6b :2 a0 6b 6e a5
b a 10 :2 a0 6b 6e :2 a0 6b
6e a5 b 6e 7e a0 b4 2e
6e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e :3 a0 a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 d :2 a0 d :2 a0
d :2 a0 d :2 a0 d :2 a0 d :2 a0
d :2 a0 d :2 a0 d :2 a0 d :2 a0
d :2 a0 d :2 a0 d :2 a0 d :2 a0
d a0 6e d a0 4d d :4 a0
51 a5 b a5 b d a0 51
d a0 51 d a0 51 d a0
7e b4 2e 5a :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
a0 a5 57 b7 19 3c a0 7e
b4 2e 5a :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b a0
a5 57 b7 19 3c :2 a0 51 a5
b 7e 51 b4 2e :2 a0 51 a5
b 7e 51 b4 2e a 10 :2 a0
51 a5 b 7e 51 b4 2e a
10 :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b a0 a5 57
b7 19 3c a0 7e b4 2e :2 5a
a0 7e b4 2e :2 5a a 10 :8 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :2 a0 51 a5 b 7e 51
b4 2e 5a a 10 :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b a0 a5 57 b7 19 3c a0
51 d a0 b7 a0 7e b4 2e
:2 5a a0 7e b4 2e :2 5a a 10
:3 a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 :2 19 3c :2 a0 d a0
7e b4 2e 5a a0 6e d b7
19 3c :3 a0 :2 51 a5 b d :3 a0
:2 51 a5 b d :2 a0 d :2 a0 51
a5 b 7e 51 b4 2e :4 a0 :2 51
a5 b :2 51 a5 b d :3 a0 6b
:3 51 a0 a5 b d :3 a0 51 a5
b a5 b 7e 51 b4 2e :d a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 :45 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c :2 a0
6b a0 7e a0 6b b4 2e 5a
:7 a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b a0 7e a0 6b
b4 2e 5a a0 7e b4 2e 5a
a0 51 d b7 19 3c :b a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 :2 19 3c b7 a0 51 d b7
:2 19 3c a0 51 d :5 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
51 a5 b 7e 51 b4 2e :5 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 6e a5 b 7e
51 b4 2e :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b :2 a0
a5 57 b7 19 3c b7 19 3c
b7 :b a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a :5 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 19
3c b7 19 3c b7 :2 19 3c a0
7e 51 b4 2e :2 a0 d :2 a0 d
:6 a0 51 a5 b :2 a0 51 a5 b
:2 a0 51 a5 b :2 a0 51 a5 b
:2 a0 51 a5 b :2 a0 51 a5 b
:2 a0 51 a5 b :2 a0 51 a5 b
:3 a0 a5 b d :2 a0 51 a5 b
a0 7e a0 6b b4 2e :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b a0 a5 57 a0 b7 :2 a0
51 a5 b 7e 51 b4 2e :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 51 a5 b :2 a0 a5 57 b7
:2 19 3c :2 a0 d a0 7e b4 2e
5a a0 6e d b7 19 3c a0
b7 :2 a0 51 a5 b 7e 51 b4
2e 5a :2 a0 51 a5 b 7e 51
b4 2e 5a a 10 :2 a0 51 a5
b 7e 51 b4 2e 5a :2 a0 51
a5 b 7e 51 b4 2e 5a 52
10 a0 51 d a0 b7 :2 a0 51
a5 b 7e 51 b4 2e a0 51
d a0 b7 19 :2 a0 51 a5 b
7e 51 b4 2e 5a :2 a0 51 a5
b 7e 51 b4 2e 5a 52 10
a0 51 d a0 b7 19 :2 a0 51
a5 b 7e 51 b4 2e :3 a0 51
a5 b 7e :2 a0 51 a5 b b4
2e d a0 51 d b7 :2 19 3c
b7 :2 19 3c :2 a0 b4 2e d a0
6e d a0 6e d :3 a0 6b :3 a0
51 a5 b :2 a0 6b a5 b 5a
51 a5 b d :4 a0 51 a5 b
:2 a0 6b a5 b d :2 a0 51 a5
b 7e 51 b4 2e :8 a0 a5 b
d :2 a0 51 a5 b 7e 51 b4
2e :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 51 a5 b :2 a0 a5
57 b7 19 3c a0 4d d :14 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a a0 7e b4 2e :2 5a a
10 a0 51 d :2 a0 51 a5 b
7e 51 b4 2e :2 a0 d b7 19
3c b7 19 3c a0 51 d a0
51 d a0 7e 51 b4 2e :2 a0
:2 51 a5 b 7e 6e b4 2e :1c a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a a0 51 d :2 a0 51 a5
b 7e 51 b4 2e 5a :2 a0 51
a5 b 7e 51 b4 2e 5a 52
10 :1a a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 a0 51 d b7
:2 19 3c a0 b7 :2 a0 6b a0 7e
a0 6b b4 2e 5a :2 a0 51 a5
b 7e 51 b4 2e 5a 52 10
:1a a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 :2 19 3c b7 :1a a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c b7 :2 a0 :2 51 a5 b
7e 6e b4 2e :19 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a a0
51 d :2 a0 51 a5 b 7e 51
b4 2e 5a :2 a0 51 a5 b 7e
51 b4 2e 5a 52 10 :14 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 a0 51 d 4f b7 :2 19 3c
a0 b7 :2 a0 6b a0 7e a0 6b
b4 2e 5a :17 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :2 19 3c
b7 :17 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 :2 19 3c b7 :2 19
3c :2 a0 6b a0 7e a0 6b b4
2e 5a :2 a0 51 a5 b 7e 51
b4 2e :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 a0 b7 a0 7e b4 2e 5a
a0 7e 51 b4 2e a0 51 d
:f a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 a0 51 d :c a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c :2 a0 51 a5 b a0
7e a0 51 a5 b b4 2e :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 19
3c b7 :2 19 3c a0 51 d a0
b7 :2 a0 51 a5 b 7e 51 b4
2e 5a :3 a0 51 a5 b a5 b
5a 7e 51 b4 2e a 10 :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 :2 19
3c :2 a0 :2 51 a5 b 7e 6e b4
2e :3 a0 7e a0 b4 2e :2 51 a5
b d :15 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c :2 a0
:2 51 a5 b 7e 6e b4 2e 5a
:2 a0 51 a5 b 7e 51 b4 2e
5a :2 a0 51 a5 b a0 7e a0
51 a5 b b4 2e 5a a 10
5a 52 10 :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b :2 a0
a5 57 b7 19 3c :c a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:3 a0 7e a0 b4 2e :2 51 a5 b
d :5 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 19 3c :14 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 6b a0 7e a0 6b b4 2e
5a :2 a0 51 a5 b 7e 51 b4
2e 5a a 10 :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 19 3c :c a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c :2 a0 d :2 a0 51 a5
b 7e 51 b4 2e :a a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :a a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f a0 51 d :2 a0 d :2 a0 51
a5 b 7e 51 b4 2e a0 51
d b7 19 3c :1e a0 :4 51 a5 b
d a0 7e 51 b4 2e :2 a0 51
a5 b 7e 51 b4 2e :4 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :4 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:2 19 3c b7 19 3c b7 :2 a0 d
:2 a0 d :2 a0 :2 51 a5 b 7e 6e
b4 2e :3 a0 6b d b7 :4 a0 51
a5 b :2 a0 6b a5 b d b7
:2 19 3c :1e a0 :4 51 a5 b d a0
7e 51 b4 2e :4 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 19 3c b7
:2 19 3c b7 :3 a0 a5 b 51 a5
b 7e 51 b4 2e a0 51 d
b7 :5 a0 a5 b 51 a5 b 7e
51 b4 2e 5a 7e 51 b4 2e
a5 b d b7 :2 19 3c a0 7e
51 b4 2e a0 51 d :3 a0 51
a5 b a0 7e a0 b4 2e 82
a0 51 d :3 a0 6b :2 51 :2 a0 51
a5 b 7e 51 b4 2e 7e 51
b4 2e a0 a5 b d :3 a0 6b
:2 51 :2 a0 51 a5 b 7e 51 b4
2e 7e 51 b4 2e 7e 51 b4
2e a0 a5 b d :2 a0 d a0
51 d a0 51 d :2 a0 d :e a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 51 a5 b 7e 51 b4
2e 5a a0 7e b4 2e :2 5a a
10 a0 51 d :2 a0 6b :2 a0 a5
b 7e 51 b4 2e :2 a0 d b7
19 3c b7 19 3c :2 a0 :2 51 a5
b 7e 6e b4 2e :3 a0 6b d
:3 a0 a5 b 51 a5 b 7e 51
b4 2e :2 a0 d b7 :3 a0 7e 6e
b4 2e :2 51 a5 b d :3 a0 7e
a0 b4 2e :2 51 a5 b d b7
:2 19 3c b7 :3 a0 a5 b 51 a5
b 7e 51 b4 2e :2 a0 d b7
:3 a0 7e 6e b4 2e :2 51 a5 b
d :3 a0 7e a0 b4 2e :2 51 a5
b d b7 :2 19 3c b7 :2 19 3c
a0 51 d a0 51 d :2 a0 51
a5 b 7e 51 b4 2e a0 7e
51 b4 2e :2 a0 :2 51 a5 b 7e
6e b4 2e :1c a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b a0
7e a0 6b b4 2e 5a a0 51
d :2 a0 51 a5 b 7e 51 b4
2e 5a :2 a0 51 a5 b 7e 51
b4 2e 5a 52 10 :1a a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
a0 51 d b7 :2 19 3c a0 b7
:2 a0 6b a0 7e a0 6b b4 2e
5a :2 a0 51 a5 b 7e 51 b4
2e 5a 52 10 :1a a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :2 19
3c b7 :1c a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c b7
:2 a0 :2 51 a5 b 7e 6e b4 2e
:1b a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b a0 7e a0 6b
b4 2e 5a :2 a0 51 a5 b 7e
51 b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a 52 10 :16 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 a0 51 d 4f b7 :2 19
3c a0 b7 :2 a0 6b a0 7e a0
6b b4 2e 5a :19 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :2 19
3c b7 :19 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c b7
:2 19 3c :2 a0 6b a0 7e a0 6b
b4 2e 5a :2 a0 51 a5 b 7e
51 b4 2e a0 51 d b7 a0
7e b4 2e 5a a0 7e 51 b4
2e a0 51 d :f a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 a0
51 d :c a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c :2 a0
51 a5 b a0 7e a0 51 a5
b b4 2e :3 a0 51 a5 b 7e
51 b4 2e d a0 3b b7 19
3c b7 19 3c a0 51 d b7
:2 19 3c b7 :3 a0 51 a5 b 7e
51 b4 2e d a0 3b b7 :2 19
3c :2 a0 d a0 51 d :a a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:a a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f a0 51 d :2 a0 d :2 a0
51 a5 b 7e 51 b4 2e a0
51 d b7 19 3c :1e a0 :4 51 a5
b d a0 7e 51 b4 2e :2 a0
51 a5 b 7e 51 b4 2e :4 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:4 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c b7 19 3c a0 b7
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 d :2 a0 d :2 a0 :2 51 a5 b
7e 6e b4 2e :3 a0 6b d b7
:3 a0 51 a5 b d b7 :2 19 3c
:1e a0 :4 51 a5 b d a0 7e 51
b4 2e :4 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 19 3c b7 :2 19 3c
:3 a0 51 a5 b 7e 51 b4 2e
d 93 4f b0 46 b7 a0 47
b7 19 3c b7 :2 19 3c a0 7e
51 b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a a 10 :2 a0
51 a5 b 7e 51 b4 2e :3 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:7 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c a0 7e 51 b4 2e
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 a0
b7 :2 a0 7e a0 6b b4 2e :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 a0 b7
19 :2 a0 7e a0 6b b4 2e :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 19
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 b7
:2 19 3c b7 19 3c :3 a0 a5 b
51 a5 b 7e 51 b4 2e 5a
:3 a0 a5 b 51 a5 b 7e 51
b4 2e 5a a 10 :2 a0 6e 11e
11d :2 a0 6b 51 d a0 6e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e d
:3 a0 11e 11d a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6e 11e 11d :2 a0 6b
51 d a0 6e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e d :3 a0 11e 11d
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
6b a0 b4 2e d a0 6e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e d
:3 a0 11e 11d a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f a0 6e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :3 a0 51 a5 b a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :3 a0 51 a5 b a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :3 a0
51 a5 b a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:3 a0 51 a5 b a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e d a0 b7 :3 a0
a5 b 51 a5 b 7e 51 b4
2e :2 a0 6e 11e 11d :2 a0 6b 51
d a0 6e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e d :3 a0 11e 11d a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6e
11e 11d :2 a0 6b 51 d a0 6e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e d :3 a0 11e 11d
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
6b a0 b4 2e d a0 6e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e d
:3 a0 11e 11d a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f a0 6e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :3 a0 51 a5 b a5 b b4
2e 7e 6e b4 2e 7e :3 a0 51
a5 b a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e 6e
b4 2e d b7 19 :2 a0 6e 11e
11d :2 a0 6b 51 d a0 6e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e d
:3 a0 11e 11d a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6e 11e 11d :2 a0 6b
51 d a0 6e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
d :3 a0 11e 11d a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 b4 2e
d a0 6e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e d :3 a0 11e 11d a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f a0 6e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :3 a0 51 a5
b a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :3 a0 51
a5 b a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e 6e
b4 2e d b7 :2 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :3 a0 6b a0 a5
b 65 b7 :2 a0 6b :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 :2 a0 6b :2 6e
7e :2 a0 6b b4 2e 7e 6e b4
2e 7e :2 a0 6b b4 2e a5 57
b7 19 3c :9 a0 12a e9 dd :2 a0
65 b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c :7 a0 12a e9 dd :2 a0 65 b7
a4 b1 11 4f b7 a6 9 a4
b1 11 68 4f 1d 17 b5 
31f0
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 6f 6b 59 77 84
80 56 8c 95 91 7f 9d aa
a6 7c b2 bb b7 a5 c3 d0
cc a2 d8 e1 dd cb e9 f6
f2 c8 fe 107 103 f1 10f 11c
118 ee 124 12d 129 117 135 114
13a 13e 142 146 149 14d 16a 155
159 15c 15d 165 154 187 175 151
179 17a 182 174 1a4 192 171 196
197 19f 191 1c1 1af 18e 1b3 1b4
1bc 1ae 1de 1cc 1ab 1d0 1d1 1d9
1cb 1fb 1e9 1c8 1ed 1ee 1f6 1e8
218 206 1e5 20a 20b 213 205 235
223 202 227 228 230 222 252 240
21f 244 245 24d 23f 26f 25d 23c
261 262 26a 25c 28b 27a 27e 286
259 2a3 292 296 29e 279 2c0 2ae
276 2b2 2b3 2bb 2ad 2dd 2cb 2aa
2cf 2d0 2d8 2ca 2fa 2e8 2c7 2ec
2ed 2f5 2e7 317 305 2e4 309 30a
312 304 334 322 301 326 327 32f
321 351 33f 31e 343 344 34c 33e
36d 35c 360 33b 368 35b 389 378
37c 358 384 377 3a5 394 398 374
3a0 393 3c1 3b0 3b4 390 3bc 3af
3dd 3cc 3d0 3ac 3d8 3cb 3f9 3e8
3ec 3c8 3f4 3e7 415 404 408 3e4
410 403 431 420 424 400 42c 41f
44d 43c 440 41c 448 43b 469 458
45c 438 464 457 485 474 478 454
480 473 4a1 490 494 470 49c 48f
4bd 4ac 4b0 48c 4b8 4ab 4d9 4c8
4cc 4a8 4d4 4c7 4f5 4e4 4e8 4c4
4f0 4e3 511 500 504 4e0 50c 4ff
52d 51c 520 4fc 528 51b 549 538
53c 518 544 537 565 554 558 534
560 553 581 570 574 550 57c 56f
59d 58c 590 56c 598 58b 5b9 5a8
5ac 588 5b4 5a7 5d5 5c4 5c8 5a4
5d0 5c3 5f1 5e0 5e4 5c0 5ec 5df
60d 5fc 600 5dc 608 5fb 629 618
61c 5f8 624 617 645 634 638 614
640 633 661 650 654 630 65c 64f
67e 66c 64c 670 671 679 66b 69b
689 668 68d 68e 696 688 6b7 6a6
6aa 685 6b2 6a5 6d4 6c2 6a2 6c6
6c7 6cf 6c1 6f0 6df 6e3 6be 6eb
6de 70d 6fb 6db 6ff 700 708 6fa
72a 718 6f7 71c 71d 725 717 747
735 714 739 73a 742 734 763 752
756 731 75e 751 77f 76e 772 74e
77a 76d 79b 78a 78e 76a 796 789
7b8 7a6 786 7aa 7ab 7b3 7a5 7d5
7c3 7a2 7c7 7c8 7d0 7c2 7f1 7e0
7e4 7bf 7ec 7df 80d 7fc 800 7dc
808 7fb 82a 818 7f8 81c 81d 825
817 846 835 839 814 841 834 862
851 855 831 85d 850 87e 86d 871
84d 879 86c 89a 889 88d 869 895
888 8b6 8a5 8a9 8b1 885 8d5 8bd
8c1 8c5 8c8 8d0 8a4 8dc 8e0 8a1
8e4 8e8 8ec 8ef 8f4 8f5 1 8f7
8fc 900 904 907 90c 910 914 917
91c 91d 91f 924 927 92b 92c 931
936 939 93e 93f 944 947 94b 94c
951 954 959 95a 95f 962 967 968
96d 970 975 976 97b 97e 982 983
988 98b 990 991 996 999 99e 99f
9a4 9a7 9ac 9ad 9b2 9b5 9b9 9ba
9bf 9c2 9c7 9c8 9cd 9d0 9d5 9d6
9db 9de 9e2 9e6 9ea 9eb 9ed 9f2
9f3 9f5 9f6 9fb 9fe a03 a04 a09
a0c a10 a14 a18 a19 a1b a20 a21
a23 a24 a29 a2c a31 a32 a37 a3a
a3e a42 a46 a47 a49 a4e a4f a51
a52 a57 a5a a5f a60 a65 a68 a6c
a70 a74 a75 a77 a7c a7d a7f a80
a85 a88 a8d a8e a93 a96 a9a a9e
aa2 aa3 aa5 aaa aab aad aae ab3
ab6 abb abc ac1 ac4 ac8 acc ad0
ad1 ad3 ad8 ad9 adb adc ae1 ae4
ae9 aea aef af2 af6 afa afe aff
b01 b06 b07 b09 b0a b0f b12 b17
b18 b1d b20 b24 b28 b2c b2d b2f
b34 b35 b37 b38 b3d b40 b45 b46
b4b b4e b52 b56 b5a b5b b5d b62
b63 b65 b66 b6b b6e b73 b74 b79
b7c b80 b84 b88 b89 b8b b90 b91
b93 b94 b99 b9c ba1 ba2 ba7 baa
baf bb0 bb5 bb8 bbc bbd bc2 bc5
bca bcb bd0 bd3 bd8 bd9 bde be1
be6 be7 bec bef bf3 bf4 bf9 bfc
c01 c02 c07 c0a c0f c10 c15 c16
c1b c1d c21 c24 c28 c2c c2f c33
c37 c3a c3d c42 c45 c48 c4b c4c
c51 c52 1 c54 c59 c5d c61 c64
c69 c6e c71 c75 c76 c7b c7c c81
c83 c87 c8a c8e c92 c96 c9a c9e
ca2 ca6 caa cae cb2 cb6 cba cbe
cc2 cc6 cca cce cd2 cd6 cda cde
ce2 ce6 cea cee cf2 cf6 cfa cfe
d02 d06 d0a d0e d12 d16 d1a d1e
d22 d26 d2a d2e d32 d36 d3a d3e
d42 d47 d4b d4f d50 d54 d58 d5c
d60 d64 d67 d68 d6a d6b d6d d71
d75 d78 d7c d80 d83 d87 d8b d8e
d92 d96 d99 d9a d9f da2 da6 daa
dad db1 db5 db8 dbb dc0 dc3 dc6
dc9 dca dcf dd0 1 dd2 dd7 ddb
ddf de2 de7 dec def df3 df4 df9
dfc e01 e02 e07 e0a e0f e10 e15
e18 e1c e20 e24 e28 e2b e2c e2e
e33 e34 e36 e37 e3c e3d e42 e44
e48 e4b e4f e53 e56 e5a e5e e61
e64 e69 e6c e6f e72 e73 e78 e79
1 e7b e80 e84 e88 e8b e90 e95
e98 e9c e9d ea2 ea3 ea8 eaa eae
eb1 eb5 eb9 ebc ec0 ec4 ec7 ecb
ecc ed1 ed3 ed7 eda ede ee1 ee2
ee7 eea eee ef2 ef5 ef9 efd f00
f03 f08 f0b f0e f11 f12 f17 f18
1 f1a f1f f23 f27 f2a f2f f34
f37 f3b f3c f41 f44 f49 f4a f4f
f52 f57 f58 f5d f60 f64 f68 f6c
f70 f73 f74 f76 f7b f7c f7e f7f
f84 f85 f8a f8c f90 f93 f97 f9b
f9e fa2 fa6 fa9 fac fb1 fb4 fb7
fba fbb fc0 fc1 1 fc3 fc8 fcc
fd0 fd3 fd8 fdd fe0 fe4 fe5 fea
feb ff0 ff2 ff6 ff9 ffd 1001 1004
1008 100c 100f 1013 1014 1019 101b 101f
1022 1026 102a 102d 102e 1030 1033 1036
1037 103c 1040 1044 1047 1048 104a 104d
1050 1051 1 1056 105b 105f 1063 1066
1067 1069 106c 106f 1070 1 1075 107a
107e 1082 1085 1089 108d 1090 1093 1098
109b 109e 10a1 10a2 10a7 10a8 1 10aa
10af 10b3 10b7 10ba 10bf 10c4 10c7 10cb
10cc 10d1 10d4 10d9 10da 10df 10e2 10e7
10e8 10ed 10f0 10f4 10f8 10fc 1100 1103
1104 1106 110b 110c 110e 110f 1114 1115
111a 111c 1120 1123 1127 112b 112e 1132
1136 1139 113c 1141 1144 1147 114a 114b
1150 1151 1 1153 1158 115c 1160 1163
1168 116d 1170 1174 1175 117a 117b 1180
1182 1186 1189 118d 1191 1194 1198 119c
119f 11a3 11a4 11a9 11ab 11af 11b2 11b6
11b9 11ba 11bf 11c2 11c5 11c9 11cc 11cd
11d2 11d5 1 11d8 11dd 11e1 11e5 11e9
11ed 11f1 11f5 11f9 11fd 1209 120d 1211
1214 1218 121c 121e 1222 1226 122a 122d
1231 1235 1237 1238 123d 1 1241 1245
1249 124c 1250 1254 1258 125c 125f 1263
1267 126a 126d 1272 1275 1278 127b 127c
1281 1282 1 1284 1289 128d 1291 1294
1299 129e 12a1 12a5 12a6 12ab 12ae 12b3
12b4 12b9 12bc 12c1 12c2 12c7 12ca 12ce
12cf 12d4 12d7 12dc 12dd 12e2 12e5 12e9
12ed 12f0 12f3 12f4 12f6 12f7 12fc 12fd
1302 1304 1308 130b 130d 1311 1313 131f
1321 1323 1324 1329 132d 132f 133b 133d
1341 1345 1348 134c 134f 1353 1356 1357
135c 135f 1363 1367 136a 136b 136d 1370
1373 1374 1379 1 137c 1381 1385 1389
138c 1390 1394 1397 139a 139f 13a2 13a5
13a8 13a9 13ae 13af 1 13b1 13b6 13ba
13be 13c1 13c6 13cb 13ce 13d2 13d3 13d8
13db 13e0 13e1 13e6 13e9 13ee 13ef 13f4
13f7 13fb 13ff 1403 1407 140a 140b 140d
1412 1413 1415 1416 141b 141c 1421 1423
1427 142a 142e 1432 1435 1439 143d 1440
1443 1448 144b 144e 1451 1452 1457 1458
1 145a 145f 1463 1467 146a 146f 1474
1477 147b 147c 1481 1482 1487 1489 148d
1490 1494 1498 149b 149f 14a3 14a6 14aa
14ab 14b0 14b2 14b6 14b9 14bd 14c0 14c4
14c8 14ca 14ce 14d1 14d2 14d7 14da 14dd
14e1 14e4 14e5 14ea 14ed 1 14f0 14f5
14f9 14fd 1501 150d 1511 1515 1518 151c
1520 1522 1526 152a 152e 1531 1535 1539
153b 153c 1541 1 1545 1549 154d 1550
1554 1558 155c 1560 1563 1567 156b 156e
1571 1576 1579 157c 157f 1580 1585 1586
1 1588 158d 1591 1595 1598 159d 15a2
15a5 15a9 15aa 15af 15b2 15b7 15b8 15bd
15c0 15c5 15c6 15cb 15ce 15d2 15d3 15d8
15db 15e0 15e1 15e6 15e9 15ed 15f1 15f4
15f7 15f8 15fa 15fb 1600 1601 1606 1608
160c 160f 1611 1615 1617 1623 1625 1627
1628 162d 1631 1633 163f 1641 1643 1647
164b 164e 1652 1656 165a 165e 1661 1662
1667 166a 166e 1673 1677 1679 167d 1680
1684 1688 168c 168f 1692 1693 1695 1699
169d 16a1 16a5 16a8 16ab 16ac 16ae 16b2
16b6 16ba 16be 16c2 16c6 16c9 16ca 16cc
16cf 16d2 16d3 16d8 16dc 16e0 16e4 16e8
16eb 16ee 16ef 16f1 16f4 16f7 16f8 16fa
16fe 1702 1706 170a 170d 1710 1713 1716
171a 171b 171d 1721 1725 1729 172d 1730
1731 1733 1734 1736 1739 173c 173d 1742
1746 174a 174e 1752 1756 175a 175e 1762
1766 176a 176e 1772 1776 1782 1786 178a
178d 1791 1795 1797 179b 179f 17a3 17a6
17aa 17ae 17b0 17b1 17b6 1 17ba 17be
17c2 17c5 17c9 17cd 17d1 17d5 17d8 17dc
17e0 17e3 17e6 17eb 17ee 17f1 17f4 17f5
17fa 17fb 1 17fd 1802 1806 180a 180d
1812 1817 181a 181e 181f 1824 1827 182c
182d 1832 1835 183a 183b 1840 1843 1847
1848 184d 1850 1855 1856 185b 185e 1862
1866 1869 186c 186d 186f 1870 1875 1876
187b 187d 1881 1884 1886 188a 188c 1898
189a 189c 189d 18a2 18a6 18a8 18b4 18b6
18b8 18bc 18c0 18c4 18c8 18cc 18d0 18d4
18d8 18dc 18e0 18e4 18e8 18ec 18f0 18f4
18f8 18fc 1900 1904 1908 190c 1910 1914
1918 191c 1920 1924 1928 192c 1930 1934
1938 193c 1940 1944 1948 194c 1950 1954
1958 195c 1960 1964 1968 196c 1970 1974
1978 197c 1980 1984 1988 198c 1990 1994
1998 199c 19a0 19a4 19a8 19ac 19b0 19b4
19b8 19bc 19c0 19c4 19c8 19cc 19d8 19dc
19e0 19e3 19e7 19eb 19ed 19f1 19f5 19f9
19fc 1a00 1a04 1a06 1a07 1a0c 1 1a10
1a14 1a18 1a1b 1a1f 1a23 1a27 1a2b 1a2e
1a32 1a36 1a39 1a3c 1a41 1a44 1a47 1a4a
1a4b 1a50 1a51 1 1a53 1a58 1a5c 1a60
1a63 1a68 1a6d 1a70 1a74 1a75 1a7a 1a7d
1a82 1a83 1a88 1a8b 1a90 1a91 1a96 1a99
1a9d 1a9e 1aa3 1aa6 1aab 1aac 1ab1 1ab4
1ab8 1abc 1abf 1ac2 1ac3 1ac5 1ac6 1acb
1acc 1ad1 1ad3 1ad7 1ada 1adc 1ae0 1ae2
1aee 1af0 1af2 1af3 1af8 1afc 1afe 1b0a
1b0c 1b0e 1b12 1b16 1b19 1b1d 1b21 1b24
1b28 1b2b 1b2f 1b32 1b33 1b38 1b3b 1b3f
1b43 1b47 1b4b 1b4f 1b53 1b57 1b63 1b67
1b6b 1b6e 1b72 1b76 1b78 1b7c 1b80 1b84
1b87 1b8b 1b8f 1b91 1b92 1b97 1 1b9b
1b9f 1ba3 1ba6 1baa 1bae 1bb2 1bb6 1bb9
1bbd 1bc1 1bc4 1bc7 1bcc 1bcf 1bd2 1bd5
1bd6 1bdb 1bdc 1 1bde 1be3 1be7 1beb
1bee 1bf3 1bf8 1bfb 1bff 1c00 1c05 1c08
1c0d 1c0e 1c13 1c16 1c1b 1c1c 1c21 1c24
1c28 1c29 1c2e 1c31 1c36 1c37 1c3c 1c3f
1c43 1c47 1c4a 1c4d 1c4e 1c50 1c51 1c56
1c57 1c5c 1c5e 1c62 1c65 1c67 1c6b 1c6d
1c79 1c7b 1c7d 1c7e 1c83 1c87 1c89 1c95
1c97 1c9b 1c9f 1ca2 1ca6 1ca9 1cad 1cb0
1cb1 1cb6 1cb9 1cbd 1cc0 1cc1 1cc6 1cc9
1ccd 1cd0 1cd4 1cd6 1cda 1cdd 1ce1 1ce5
1ce9 1ced 1cf1 1cf5 1cf9 1cfd 1d01 1d05
1d09 1d15 1d19 1d1d 1d20 1d24 1d28 1d2a
1d2e 1d32 1d36 1d39 1d3d 1d41 1d43 1d44
1d49 1 1d4d 1d51 1d55 1d58 1d5c 1d60
1d64 1d68 1d6b 1d6f 1d73 1d76 1d79 1d7e
1d81 1d84 1d87 1d88 1d8d 1d8e 1 1d90
1d95 1d99 1d9d 1da0 1da5 1daa 1dad 1db1
1db2 1db7 1dba 1dbf 1dc0 1dc5 1dc8 1dcd
1dce 1dd3 1dd6 1dda 1ddb 1de0 1de3 1de8
1de9 1dee 1df1 1df5 1df9 1dfc 1dff 1e00
1e02 1e03 1e08 1e09 1e0e 1e10 1e14 1e17
1e19 1e1d 1e1f 1e2b 1e2d 1e2f 1e30 1e35
1e39 1e3b 1e47 1e49 1e4b 1e4f 1e53 1e56
1e5a 1e5e 1e61 1e64 1e69 1e6c 1e6f 1e72
1e73 1e78 1e79 1 1e7b 1e80 1e84 1e88
1e8b 1e90 1e95 1e98 1e9c 1e9d 1ea2 1ea5
1eaa 1eab 1eb0 1eb3 1eb8 1eb9 1ebe 1ec1
1ec5 1ec9 1ecd 1ed1 1ed4 1ed5 1ed7 1edc
1edd 1edf 1ee0 1ee5 1ee8 1eed 1eee 1ef3
1ef6 1efb 1efc 1f01 1f04 1f08 1f09 1f0e
1f11 1f16 1f17 1f1c 1f1d 1f22 1f24 1f28
1f2b 1f2f 1f33 1f36 1f3a 1f3e 1f41 1f44
1f49 1f4c 1f4f 1f52 1f53 1f58 1f59 1
1f5b 1f60 1f64 1f68 1f6b 1f70 1f75 1f78
1f7c 1f7d 1f82 1f83 1f88 1f8a 1f8e 1f91
1f95 1f99 1f9c 1fa0 1fa4 1fa7 1fab 1faf
1fb0 1fb5 1fb7 1fbb 1fbf 1fc2 1fc4 1fc8
1fcb 1fcf 1fd1 1fd5 1fd9 1fdc 1fe0 1fe3
1fe7 1feb 1fef 1ff3 1ff7 1ffb 2007 200b
200f 2012 2016 201a 201c 2020 2024 2028
202b 202f 2033 2035 2036 203b 1 203f
2043 2047 204a 204e 2052 2056 205a 205d
2061 2065 2068 206b 2070 2073 2076 2079
207a 207f 2080 1 2082 2087 208b 208f
2092 2097 209c 209f 20a3 20a4 20a9 20ac
20b1 20b2 20b7 20ba 20bf 20c0 20c5 20c8
20cc 20cd 20d2 20d5 20da 20db 20e0 20e3
20e7 20eb 20ee 20f1 20f2 20f4 20f5 20fa
20fb 2100 2102 2106 2109 210b 210f 2111
211d 211f 2121 2122 2127 212b 212d 2139
213b 213f 2143 2146 2147 2149 214c 214f
2150 2155 2159 215d 2161 2165 2169 2175
2179 217d 2180 2184 2188 218a 218e 2192
2196 2199 219d 21a1 21a3 21a4 21a9 1
21ad 21b1 21b5 21b8 21bc 21c0 21c4 21c8
21cb 21cf 21d3 21d6 21d9 21de 21e1 21e4
21e7 21e8 21ed 21ee 1 21f0 21f5 21f9
21fd 2200 2205 220a 220d 2211 2212 2217
221a 221f 2220 2225 2228 222d 222e 2233
2236 223a 223b 2240 2243 2248 2249 224e
2251 2255 2259 225c 225f 2260 2262 2263
2268 2269 226e 2270 2274 2277 2279 227d
227f 228b 228d 228f 2290 2295 2299 229b
22a7 22a9 22ad 22b1 22b4 22b8 22bd 22be
22c0 22c3 22c6 22c7 22cc 22d0 22d4 22d7
22db 22df 22e2 22e5 22ea 22ed 22f0 22f3
22f4 22f9 22fa 1 22fc 2301 2305 2309
230c 2311 2316 2319 231d 231e 2323 2326
232b 232c 2331 2334 2339 233a 233f 2342
2346 234a 234e 2352 2355 2356 2358 235d
235e 2360 2361 2366 2369 236e 236f 2374
2377 237c 237d 2382 2385 2389 238a 238f
2392 2397 2398 239d 239e 23a3 23a5 23a9
23ac 23b0 23b4 23b7 23bb 23bf 23c2 23c5
23ca 23cd 23d0 23d3 23d4 23d9 23da 1
23dc 23e1 23e5 23e9 23ec 23f1 23f6 23f9
23fd 23fe 2403 2404 2409 240b 240f 2412
2416 241a 241d 2421 2425 2428 242c 2430
2431 2436 2438 243c 243f 2441 2445 2448
244a 244e 2452 2456 245a 245e 2462 2466
246a 246e 2472 2476 2482 2486 248a 248d
2491 2495 2497 249b 249f 24a3 24a6 24aa
24ae 24b0 24b1 24b6 1 24ba 24be 24c2
24c5 24c9 24cd 24d1 24d5 24d8 24dc 24e0
24e3 24e6 24eb 24ee 24f1 24f4 24f5 24fa
24fb 1 24fd 2502 2506 250a 250d 2512
2517 251a 251e 251f 2524 2527 252c 252d
2532 2535 253a 253b 2540 2543 2547 2548
254d 2550 2555 2556 255b 255e 2562 2566
2569 256c 256d 256f 2570 2575 2576 257b
257d 2581 2584 2586 258a 258c 2598 259a
259c 259d 25a2 25a6 25a8 25b4 25b6 25ba
25be 25c1 25c5 25c8 25cc 25cf 25d0 25d5
25d8 25dc 25e0 25e4 25e8 25ec 25f8 25fc
2600 2603 2607 260b 260d 2611 2615 2619
261c 2620 2624 2626 2627 262c 1 2630
2634 2638 263b 263f 2643 2647 264b 264e
2652 2656 2659 265c 2661 2664 2667 266a
266b 2670 2671 1 2673 2678 267c 2680
2683 2688 268d 2690 2694 2695 269a 269d
26a2 26a3 26a8 26ab 26b0 26b1 26b6 26b9
26bd 26be 26c3 26c6 26cb 26cc 26d1 26d4
26d8 26dc 26df 26e2 26e3 26e5 26e6 26eb
26ec 26f1 26f3 26f7 26fa 26fc 2700 2702
270e 2710 2712 2713 2718 271c 271e 272a
272c 2730 2734 2737 273b 273e 2742 2745
2746 274b 274e 2752 2756 2759 275d 2761
2764 2767 276c 276f 2772 2775 2776 277b
277c 1 277e 2783 2787 278b 278e 2793
2798 279b 279f 27a0 27a5 27a8 27ad 27ae
27b3 27b6 27bb 27bc 27c1 27c4 27c8 27cc
27d0 27d4 27d7 27d8 27da 27df 27e0 27e2
27e3 27e8 27eb 27f0 27f1 27f6 27f9 27fe
27ff 2804 2807 280b 280c 2811 2814 2819
281a 281f 2820 2825 2827 282b 282e 2832
2836 2839 283d 2841 2844 2847 284c 284f
2852 2855 2856 285b 285c 1 285e 2863
2867 286b 286e 2873 2878 287b 287f 2880
2885 2886 288b 288d 2891 2894 2898 289c
289f 28a3 28a7 28aa 28ae 28b2 28b3 28b8
28ba 28be 28c1 28c3 28c7 28ca 28cc 28d0
28d4 28d7 28db 28de 28e1 28e2 28e7 28eb
28ef 28f3 28f7 28fb 28ff 2903 2907 290b
290f 2913 2917 291a 291b 291d 2921 2925
2928 2929 292b 292f 2933 2936 2937 2939
293d 2941 2944 2945 2947 294b 294f 2952
2953 2955 2959 295d 2960 2961 2963 2967
296b 296e 296f 2971 2975 2979 297c 297d
297f 2983 2987 298b 298c 298e 2992 2996
299a 299d 299e 29a0 29a4 29a7 29ab 29ae
29af 29b4 29b8 29bc 29bf 29c3 29c7 29ca
29cd 29d2 29d5 29d8 29db 29dc 29e1 29e2
1 29e4 29e9 29ed 29f1 29f4 29f9 29fe
2a01 2a05 2a06 2a0b 2a0e 2a13 2a14 2a19
2a1c 2a21 2a22 2a27 2a2a 2a2e 2a32 2a36
2a3a 2a3d 2a3e 2a40 2a45 2a46 2a48 2a49
2a4e 2a4f 2a54 2a56 2a5a 2a5d 2a61 2a65
2a68 2a6c 2a70 2a73 2a76 2a7b 2a7e 2a81
2a84 2a85 2a8a 2a8b 1 2a8d 2a92 2a96
2a9a 2a9d 2aa2 2aa7 2aaa 2aae 2aaf 2ab4
2ab5 2aba 2abc 2ac0 2ac3 2ac7 2acb 2ace
2ad2 2ad6 2ad9 2add 2ade 2ae3 2ae7 2ae9
2aed 2af1 2af4 2af5 2af7 2afa 2afd 2afe
2b03 2b07 2b0b 2b0e 2b12 2b16 2b19 2b1c
2b21 2b24 2b27 2b2a 2b2b 2b30 2b31 1
2b33 2b38 2b3c 2b40 2b43 2b48 2b4d 2b50
2b54 2b55 2b5a 2b5d 2b62 2b63 2b68 2b6b
2b70 2b71 2b76 2b79 2b7d 2b81 2b85 2b86
2b88 2b8d 2b8e 2b90 2b91 2b96 2b99 2b9e
2b9f 2ba4 2ba7 2bac 2bad 2bb2 2bb5 2bb9
2bba 2bbf 2bc2 2bc7 2bc8 2bcd 2bce 2bd3
2bd5 2bd9 2bdc 2be0 2be4 2be7 2beb 2bef
2bf2 2bf5 2bfa 2bfd 2c00 2c03 2c04 2c09
2c0a 1 2c0c 2c11 2c15 2c19 2c1c 2c21
2c26 2c29 2c2d 2c2e 2c33 2c34 2c39 2c3b
2c3f 2c42 2c46 2c4a 2c4d 2c51 2c55 2c58
2c59 2c5b 2c5f 2c63 2c64 2c69 2c6b 2c6f
2c73 2c76 2c7a 2c7e 2c82 2c86 2c89 2c8a
2c8f 2c92 2c96 2c9b 2c9f 2ca1 2ca5 2ca8
2cac 2cae 2cb2 2cb6 2cb9 2cba 2cbc 2cbf
2cc2 2cc3 2cc8 2ccb 2ccf 2cd3 2cd6 2cd7
2cd9 2cdc 2cdf 2ce0 2ce5 1 2ce8 2ced
2cf1 2cf5 2cf8 2cf9 2cfb 2cfe 2d01 2d02
2d07 2d0a 2d0e 2d12 2d15 2d16 2d18 2d1b
2d1e 2d1f 2d24 1 2d27 2d2c 2d30 2d33
2d37 2d3b 2d3d 2d41 2d45 2d48 2d49 2d4b
2d4e 2d51 2d52 2d57 2d5b 2d5e 2d62 2d66
2d68 2d6c 2d70 2d74 2d77 2d78 2d7a 2d7d
2d80 2d81 2d86 2d89 2d8d 2d91 2d94 2d95
2d97 2d9a 2d9d 2d9e 2da3 1 2da6 2dab
2daf 2db2 2db6 2dba 2dbc 2dc0 2dc4 2dc8
2dcb 2dcc 2dce 2dd1 2dd4 2dd5 2dda 2dde
2de2 2de6 2de9 2dea 2dec 2def 2df3 2df7
2dfa 2dfb 2dfd 2dfe 2e03 2e07 2e0b 2e0e
2e12 2e14 2e18 2e1c 2e1f 2e21 2e25 2e29
2e2c 2e30 2e34 2e35 2e3a 2e3e 2e42 2e47
2e4b 2e4f 2e54 2e58 2e5c 2e60 2e64 2e67
2e6b 2e6f 2e73 2e76 2e77 2e79 2e7d 2e81
2e84 2e85 2e87 2e8a 2e8d 2e8e 2e90 2e94
2e98 2e9c 2ea0 2ea4 2ea7 2ea8 2eaa 2eae
2eb2 2eb5 2eb6 2eb8 2ebc 2ec0 2ec4 2ec7
2ec8 2eca 2ecd 2ed0 2ed1 2ed6 2eda 2ede
2ee2 2ee6 2eea 2eee 2ef2 2ef6 2ef7 2ef9
2efd 2f01 2f05 2f08 2f09 2f0b 2f0e 2f11
2f12 2f17 2f1b 2f1f 2f22 2f26 2f2a 2f2d
2f30 2f35 2f38 2f3b 2f3e 2f3f 2f44 2f45
1 2f47 2f4c 2f50 2f54 2f57 2f5c 2f61
2f64 2f68 2f69 2f6e 2f71 2f76 2f77 2f7c
2f7f 2f84 2f85 2f8a 2f8d 2f91 2f95 2f99
2f9a 2f9c 2fa1 2fa2 2fa4 2fa5 2faa 2fad
2fb2 2fb3 2fb8 2fbb 2fc0 2fc1 2fc6 2fc9
2fcd 2fce 2fd3 2fd6 2fdb 2fdc 2fe1 2fe2
2fe7 2fe9 2fed 2ff0 2ff4 2ff8 2ffb 2fff
3003 3006 3009 300e 3011 3014 3017 3018
301d 301e 1 3020 3025 3029 302d 3030
3035 303a 303d 3041 3042 3047 3048 304d
304f 3053 3056 305a 305e 3061 3065 3069
306c 306d 306f 3073 3077 3078 307d 307f
3083 3086 308a 308b 308f 3093 3097 309b
309f 30a3 30a7 30ab 30af 30b3 30b7 30bb
30bf 30c3 30c7 30cb 30cf 30d3 30d7 30db
30df 30eb 30ef 30f3 30f6 30fa 30fe 3100
3104 3108 310c 310f 3113 3117 3119 311a
311f 1 3123 3127 312b 312e 3132 3136
313a 313e 3141 3145 3149 314c 314f 3154
3157 315a 315d 315e 3163 3164 1 3166
316b 316f 3173 3176 317b 3180 3183 3187
3188 318d 3190 3195 3196 319b 319e 31a3
31a4 31a9 31ac 31b0 31b1 31b6 31b9 31be
31bf 31c4 31c7 31cb 31cf 31d2 31d5 31d6
31d8 31d9 31de 31df 31e4 31e6 31ea 31ed
31ef 31f3 31f5 3201 3203 3205 3206 320b
320f 3211 321d 321f 3223 3227 322a 322e
3231 3235 3238 3239 323e 3241 3245 3248
3249 324e 3251 1 3254 3259 325d 3260
3264 3268 326c 326f 3270 3272 3275 3278
3279 327e 3282 3286 328a 328c 3290 3293
3295 3299 329c 32a0 32a3 32a7 32ab 32ae
32b2 32b6 32b9 32bc 32bd 32c2 32c6 32ca
32cd 32d0 32d1 32d3 32d6 32db 32dc 32e1
32e5 32e9 32ed 32f1 32f5 32f9 32fd 3301
3305 3309 330d 3311 3315 3319 331d 3321
3325 3329 332d 3331 3335 3339 333d 3341
3345 3349 334d 3351 335d 3361 3365 3368
336c 3370 3372 3376 337a 337e 3381 3385
3389 338b 338c 3391 1 3395 3399 339d
33a0 33a4 33a8 33ac 33b0 33b3 33b7 33bb
33be 33c1 33c6 33c9 33cc 33cf 33d0 33d5
33d6 1 33d8 33dd 33e1 33e5 33e8 33ed
33f2 33f5 33f9 33fa 33ff 3402 3407 3408
340d 3410 3415 3416 341b 341e 3422 3423
3428 342b 3430 3431 3436 3439 343d 3441
3444 3447 3448 344a 344b 3450 3451 3456
3458 345c 345f 3461 3465 3467 3473 3475
3477 3478 347d 3481 3483 348f 3491 3495
3499 349c 34a0 34a3 34a7 34aa 34ab 34b0
34b3 34b7 34ba 34be 34c2 34c6 34c9 34ca
34cc 34cf 34d2 34d3 34d8 34db 34df 34e3
34e6 34e7 34e9 34ec 34ef 34f0 34f5 1
34f8 34fd 3501 3505 3509 350d 3511 3515
3519 351d 3521 3525 3529 352d 3531 3535
3539 353d 3541 3545 3549 354d 3551 3555
3559 355d 3561 3565 3571 3575 3579 357c
3580 3584 3586 358a 358e 3592 3595 3599
359d 359f 35a0 35a5 1 35a9 35ad 35b1
35b4 35b8 35bc 35c0 35c4 35c7 35cb 35cf
35d2 35d5 35da 35dd 35e0 35e3 35e4 35e9
35ea 1 35ec 35f1 35f5 35f9 35fc 3601
3606 3609 360d 360e 3613 3616 361b 361c
3621 3624 3629 362a 362f 3632 3636 3637
363c 363f 3644 3645 364a 364d 3651 3655
3658 365b 365c 365e 365f 3664 3665 366a
366c 3670 3673 3675 3679 367b 3687 3689
368b 368c 3691 3695 3697 36a3 36a5 36a7
36ab 36ae 36b2 36b4 36b8 36bc 36bf 36c3
36c5 36c9 36cd 36d0 36d4 36d7 36db 36de
36df 36e4 36e7 36eb 36ef 36f2 36f3 36f5
36f8 36fb 36fc 3701 1 3704 3709 370d
3711 3715 3719 371d 3721 3725 3729 372d
3731 3735 3739 373d 3741 3745 3749 374d
3751 3755 3759 375d 3761 3765 3769 376d
3771 377d 3781 3785 3788 378c 3790 3792
3796 379a 379e 37a1 37a5 37a9 37ab 37ac
37b1 1 37b5 37b9 37bd 37c0 37c4 37c8
37cc 37d0 37d3 37d7 37db 37de 37e1 37e6
37e9 37ec 37ef 37f0 37f5 37f6 1 37f8
37fd 3801 3805 3808 380d 3812 3815 3819
381a 381f 3822 3827 3828 382d 3830 3835
3836 383b 383e 3842 3843 3848 384b 3850
3851 3856 3859 385d 3861 3864 3867 3868
386a 386b 3870 3871 3876 3878 387c 387f
3881 3885 3887 3893 3895 3897 3898 389d
38a1 38a3 38af 38b1 38b3 38b7 38bb 38be
38c0 38c4 38c8 38cc 38d0 38d4 38d8 38dc
38e0 38e4 38e8 38ec 38f0 38f4 38f8 38fc
3900 3904 3908 390c 3910 3914 3918 391c
3920 3924 3928 3934 3938 393c 393f 3943
3947 3949 394d 3951 3955 3958 395c 3960
3962 3963 3968 1 396c 3970 3974 3977
397b 397f 3983 3987 398a 398e 3992 3995
3998 399d 39a0 39a3 39a6 39a7 39ac 39ad
1 39af 39b4 39b8 39bc 39bf 39c4 39c9
39cc 39d0 39d1 39d6 39d9 39de 39df 39e4
39e7 39ec 39ed 39f2 39f5 39f9 39fa 39ff
3a02 3a07 3a08 3a0d 3a10 3a14 3a18 3a1b
3a1e 3a1f 3a21 3a22 3a27 3a28 3a2d 3a2f
3a33 3a36 3a38 3a3c 3a3e 3a4a 3a4c 3a4e
3a4f 3a54 3a58 3a5a 3a66 3a68 3a6a 3a6e
3a72 3a75 3a77 3a7b 3a7f 3a82 3a85 3a86
3a88 3a8b 3a90 3a91 3a96 3a9a 3a9e 3aa2
3aa6 3aaa 3aae 3ab2 3ab6 3aba 3abe 3ac2
3ac6 3aca 3ace 3ad2 3ad6 3ada 3ade 3ae2
3ae6 3aea 3aee 3af2 3af6 3afa 3b06 3b0a
3b0e 3b11 3b15 3b19 3b1b 3b1f 3b23 3b27
3b2a 3b2e 3b32 3b34 3b35 3b3a 1 3b3e
3b42 3b46 3b49 3b4d 3b51 3b55 3b59 3b5c
3b60 3b64 3b67 3b6a 3b6f 3b72 3b75 3b78
3b79 3b7e 3b7f 1 3b81 3b86 3b8a 3b8e
3b91 3b96 3b9b 3b9e 3ba2 3ba3 3ba8 3bab
3bb0 3bb1 3bb6 3bb9 3bbe 3bbf 3bc4 3bc7
3bcb 3bcc 3bd1 3bd4 3bd9 3bda 3bdf 3be2
3be6 3bea 3bed 3bf0 3bf1 3bf3 3bf4 3bf9
3bfa 3bff 3c01 3c05 3c08 3c0a 3c0e 3c10
3c1c 3c1e 3c20 3c21 3c26 3c2a 3c2c 3c38
3c3a 3c3e 3c42 3c45 3c49 3c4c 3c50 3c53
3c54 3c59 3c5c 3c60 3c63 3c67 3c6b 3c6f
3c72 3c73 3c75 3c78 3c7b 3c7c 3c81 3c84
3c88 3c8c 3c8f 3c90 3c92 3c95 3c98 3c99
3c9e 1 3ca1 3ca6 3caa 3cae 3cb2 3cb6
3cba 3cbe 3cc2 3cc6 3cca 3cce 3cd2 3cd6
3cda 3cde 3ce2 3ce6 3cea 3cee 3cf2 3cf6
3d02 3d06 3d0a 3d0d 3d11 3d15 3d17 3d1b
3d1f 3d23 3d26 3d2a 3d2e 3d30 3d31 3d36
1 3d3a 3d3e 3d42 3d45 3d49 3d4d 3d51
3d55 3d58 3d5c 3d60 3d63 3d66 3d6b 3d6e
3d71 3d74 3d75 3d7a 3d7b 1 3d7d 3d82
3d86 3d8a 3d8d 3d92 3d97 3d9a 3d9e 3d9f
3da4 3da7 3dac 3dad 3db2 3db5 3dba 3dbb
3dc0 3dc3 3dc7 3dc8 3dcd 3dd0 3dd5 3dd6
3ddb 3dde 3de2 3de6 3de9 3dec 3ded 3def
3df0 3df5 3df6 3dfb 3dfd 3e01 3e04 3e06
3e0a 3e0c 3e18 3e1a 3e1c 3e1d 3e22 3e26
3e28 3e34 3e36 3e38 3e3c 3e3f 3e43 3e45
3e47 3e4b 3e4f 3e52 3e56 3e58 3e5c 3e60
3e63 3e67 3e6a 3e6e 3e71 3e72 3e77 3e7a
3e7e 3e82 3e86 3e8a 3e8e 3e92 3e96 3e9a
3e9e 3ea2 3ea6 3eaa 3eae 3eb2 3eb6 3eba
3ebe 3ec2 3ec6 3eca 3ece 3ed2 3ed6 3ee2
3ee6 3eea 3eed 3ef1 3ef5 3ef7 3efb 3eff
3f03 3f06 3f0a 3f0e 3f10 3f11 3f16 1
3f1a 3f1e 3f22 3f25 3f29 3f2d 3f31 3f35
3f38 3f3c 3f40 3f43 3f46 3f4b 3f4e 3f51
3f54 3f55 3f5a 3f5b 1 3f5d 3f62 3f66
3f6a 3f6d 3f72 3f77 3f7a 3f7e 3f7f 3f84
3f87 3f8c 3f8d 3f92 3f95 3f9a 3f9b 3fa0
3fa3 3fa7 3fa8 3fad 3fb0 3fb5 3fb6 3fbb
3fbe 3fc2 3fc6 3fc9 3fcc 3fcd 3fcf 3fd0
3fd5 3fd6 3fdb 3fdd 3fe1 3fe4 3fe6 3fea
3fec 3ff8 3ffa 3ffc 3ffd 4002 4006 4008
4014 4016 4018 401c 4020 4023 4025 4029
402d 4031 4035 4039 403d 4041 4045 4049
404d 4051 4055 4059 405d 4061 4065 4069
406d 4071 4075 4079 407d 4081 408d 4091
4095 4098 409c 40a0 40a2 40a6 40aa 40ae
40b1 40b5 40b9 40bb 40bc 40c1 1 40c5
40c9 40cd 40d0 40d4 40d8 40dc 40e0 40e3
40e7 40eb 40ee 40f1 40f6 40f9 40fc 40ff
4100 4105 4106 1 4108 410d 4111 4115
4118 411d 4122 4125 4129 412a 412f 4132
4137 4138 413d 4140 4145 4146 414b 414e
4152 4153 4158 415b 4160 4161 4166 4169
416d 4171 4174 4177 4178 417a 417b 4180
4181 4186 4188 418c 418f 4191 4195 4197
41a3 41a5 41a7 41a8 41ad 41b1 41b3 41bf
41c1 41c3 41c7 41cb 41ce 41d0 41d4 41d8
41db 41df 41e3 41e6 41ea 41ed 41f1 41f4
41f5 41fa 41fd 4201 4205 4208 4209 420b
420e 4211 4212 4217 421b 421f 4222 4226
422a 422d 4230 4235 4238 423b 423e 423f
4244 4245 1 4247 424c 4250 4254 4257
425c 4261 4264 4268 4269 426e 4271 4276
4277 427c 427f 4284 4285 428a 428d 4291
4295 4299 429d 42a0 42a1 42a3 42a8 42a9
42ab 42ac 42b1 42b4 42b9 42ba 42bf 42c2
42c7 42c8 42cd 42d0 42d4 42d5 42da 42dd
42e2 42e3 42e8 42e9 42ee 42f0 42f4 42f7
42fb 42ff 4302 4306 430a 430d 4310 4315
4318 431b 431e 431f 4324 4325 1 4327
432c 4330 4334 4337 433c 4341 4344 4348
4349 434e 434f 4354 4356 435a 435d 4361
4365 4368 436c 4370 4373 4377 437b 437c
4381 4385 4387 438b 438e 438f 4394 4397
439b 439e 43a1 43a2 43a7 43ab 43ae 43b2
43b6 43ba 43be 43c2 43c6 43ca 43ce 43d2
43d6 43da 43de 43e2 43e6 43ea 43ee 43fa
43fe 4402 4405 4409 440d 440f 4413 4417
441b 441e 4422 4426 4428 4429 442e 1
4432 4436 443a 443d 4441 4445 4449 444d
4450 4454 4458 445b 445e 4463 4466 4469
446c 446d 4472 4473 1 4475 447a 447e
4482 4485 448a 448f 4492 4496 4497 449c
449f 44a4 44a5 44aa 44ad 44b2 44b3 44b8
44bb 44bf 44c0 44c5 44c8 44cd 44ce 44d3
44d6 44da 44de 44e1 44e4 44e5 44e7 44e8
44ed 44ee 44f3 44f5 44f9 44fc 44fe 4502
4504 4510 4512 4514 4515 451a 451e 4520
452c 452e 4530 4534 4537 453b 453f 4543
4547 454b 454f 4553 4557 455b 455f 4563
4567 456b 4577 457b 457f 4582 4586 458a
458c 4590 4594 4598 459b 459f 45a3 45a5
45a6 45ab 1 45af 45b3 45b7 45ba 45be
45c2 45c6 45ca 45cd 45d1 45d5 45d8 45db
45e0 45e3 45e6 45e9 45ea 45ef 45f0 1
45f2 45f7 45fb 45ff 4602 4607 460c 460f
4613 4614 4619 461c 4621 4622 4627 462a
462f 4630 4635 4638 463c 463d 4642 4645
464a 464b 4650 4653 4657 465b 465e 4661
4662 4664 4665 466a 466b 4670 4672 4676
4679 467b 467f 4681 468d 468f 4691 4692
4697 469b 469d 46a9 46ab 46ad 46b1 46b5
46b8 46bc 46c0 46c3 46c4 46c6 46ca 46cd
46d1 46d4 46d5 46d7 46d8 46dd 46e1 46e5
46e8 46ec 46f0 46f3 46f6 46fb 46fe 4701
4704 4705 470a 470b 1 470d 4712 4716
471a 471d 4722 4727 472a 472e 472f 4734
4737 473c 473d 4742 4745 474a 474b 4750
4753 4757 475b 475f 4763 4766 4767 4769
476e 476f 4771 4772 4777 477a 477f 4780
4785 4788 478d 478e 4793 4796 479a 479b
47a0 47a3 47a8 47a9 47ae 47af 47b4 47b6
47ba 47bd 47c1 47c5 47c8 47cc 47d0 47d3
47d6 47db 47de 47e1 47e4 47e5 47ea 47eb
1 47ed 47f2 47f6 47fa 47fd 4802 4807
480a 480e 480f 4814 4815 481a 481c 4820
4823 4827 482b 482e 4832 4836 4839 483d
4841 4842 4847 4849 484d 4850 4852 4856
485a 485d 4861 4864 4868 486c 486e 4872
4876 4879 487a 487c 487f 4882 4883 4888
488b 488f 4893 4897 489a 489b 489d 489e
48a0 48a3 48a6 48a9 48aa 1 48af 48b4
48b8 48bc 48bf 48c3 48c7 48ca 48cd 48d2
48d5 48d8 48db 48dc 48e1 48e2 1 48e4
48e9 48ed 48f1 48f4 48f9 48fe 4901 4905
4906 490b 490e 4913 4914 4919 491c 4921
4922 4927 492a 492e 4932 4936 493a 493d
493e 4940 4945 4946 4948 4949 494e 4951
4956 4957 495c 495f 4964 4965 496a 496d
4971 4972 4977 497a 497f 4980 4985 4986
498b 498d 4991 4994 4998 499c 499f 49a3
49a7 49aa 49ad 49b2 49b5 49b8 49bb 49bc
49c1 49c2 1 49c4 49c9 49cd 49d1 49d4
49d9 49de 49e1 49e5 49e6 49eb 49ec 49f1
49f3 49f7 49fa 49fe 4a02 4a05 4a09 4a0d
4a10 4a14 4a18 4a19 4a1e 4a20 4a24 4a28
4a2b 4a2f 4a33 4a36 4a39 4a3a 4a3c 4a3f
4a44 4a45 4a4a 4a4e 4a52 4a56 4a59 4a5d
4a5e 4a63 4a66 4a69 4a6a 4a6c 4a70 4a74
4a78 4a7c 4a80 4a84 4a88 4a8c 4a90 4a94
4a98 4a9c 4aa0 4aa4 4aa8 4aac 4ab0 4ab4
4ab8 4abc 4ac0 4ac4 4ad0 4ad4 4ad8 4adb
4adf 4ae3 4ae5 4ae9 4aed 4af1 4af4 4af8
4afc 4afe 4aff 4b04 1 4b08 4b0c 4b10
4b13 4b17 4b1b 4b1f 4b23 4b26 4b2a 4b2e
4b31 4b34 4b39 4b3c 4b3f 4b42 4b43 4b48
4b49 1 4b4b 4b50 4b54 4b58 4b5b 4b60
4b65 4b68 4b6c 4b6d 4b72 4b75 4b7a 4b7b
4b80 4b83 4b88 4b89 4b8e 4b91 4b95 4b96
4b9b 4b9e 4ba3 4ba4 4ba9 4bac 4bb0 4bb4
4bb7 4bba 4bbb 4bbd 4bbe 4bc3 4bc4 4bc9
4bcb 4bcf 4bd2 4bd4 4bd8 4bda 4be6 4be8
4bea 4beb 4bf0 4bf4 4bf6 4c02 4c04 4c08
4c0c 4c0f 4c13 4c16 4c1a 4c1d 4c1e 4c23
4c26 4c2a 4c2e 4c31 4c35 4c39 4c3c 4c3f
4c44 4c47 4c4a 4c4d 4c4e 4c53 4c54 1
4c56 4c5b 4c5f 4c63 4c66 4c6b 4c70 4c73
4c77 4c78 4c7d 4c80 4c85 4c86 4c8b 4c8e
4c93 4c94 4c99 4c9c 4ca0 4ca4 4ca8 4cac
4caf 4cb0 4cb2 4cb7 4cb8 4cba 4cbb 4cc0
4cc3 4cc8 4cc9 4cce 4cd1 4cd6 4cd7 4cdc
4cdf 4ce3 4ce4 4ce9 4cec 4cf1 4cf2 4cf7
4cf8 4cfd 4cff 4d03 4d06 4d0a 4d0e 4d11
4d15 4d19 4d1c 4d1f 4d24 4d27 4d2a 4d2d
4d2e 4d33 4d34 1 4d36 4d3b 4d3f 4d43
4d46 4d4b 4d50 4d53 4d57 4d58 4d5d 4d5e
4d63 4d65 4d69 4d6c 4d70 4d74 4d77 4d7b
4d7f 4d82 4d86 4d8a 4d8b 4d90 4d92 4d96
4d99 4d9d 4da1 4da4 4da7 4da8 4daa 4dad
4db2 4db3 4db8 4dbb 4dbf 4dc3 4dc6 4dc7
4dc9 4dcc 4dcf 4dd0 4dd5 4dd8 4ddc 4de0
4de3 4de4 4de6 4dea 4ded 4df1 4df4 4df5
4df7 4df8 4dfd 1 4e00 4e05 1 4e08
4e0d 4e11 4e15 4e18 4e1c 4e20 4e23 4e26
4e2b 4e2e 4e31 4e34 4e35 4e3a 4e3b 1
4e3d 4e42 4e46 4e4a 4e4d 4e52 4e57 4e5a
4e5e 4e5f 4e64 4e67 4e6c 4e6d 4e72 4e75
4e7a 4e7b 4e80 4e83 4e87 4e8b 4e8f 4e93
4e96 4e97 4e99 4e9e 4e9f 4ea1 4ea2 4ea7
4eaa 4eaf 4eb0 4eb5 4eb8 4ebd 4ebe 4ec3
4ec6 4eca 4ecb 4ed0 4ed3 4ed8 4ed9 4ede
4edf 4ee4 4ee6 4eea 4eed 4ef1 4ef5 4ef8
4efc 4f00 4f03 4f06 4f0b 4f0e 4f11 4f14
4f15 4f1a 4f1b 1 4f1d 4f22 4f26 4f2a
4f2d 4f32 4f37 4f3a 4f3e 4f3f 4f44 4f45
4f4a 4f4c 4f50 4f53 4f57 4f5b 4f5e 4f62
4f66 4f69 4f6d 4f71 4f72 4f77 4f79 4f7d
4f80 4f84 4f88 4f8c 4f90 4f94 4f98 4f9c
4fa0 4fa4 4fa8 4fac 4fb0 4fbc 4fc0 4fc4
4fc7 4fcb 4fcf 4fd1 4fd5 4fd9 4fdd 4fe0
4fe4 4fe8 4fea 4feb 4ff0 1 4ff4 4ff8
4ffc 4fff 5003 5007 500b 500f 5012 5016
501a 501d 5020 5025 5028 502b 502e 502f
5034 5035 1 5037 503c 5040 5044 5047
504c 5051 5054 5058 5059 505e 5061 5066
5067 506c 506f 5074 5075 507a 507d 5081
5082 5087 508a 508f 5090 5095 5098 509c
50a0 50a3 50a6 50a7 50a9 50aa 50af 50b0
50b5 50b7 50bb 50be 50c0 50c4 50c6 50d2
50d4 50d6 50d7 50dc 50e0 50e2 50ee 50f0
50f2 50f6 50fa 50fe 5101 5105 5106 510b
510e 5111 5112 5114 5118 511c 5120 5124
5128 512c 5138 513c 5140 5143 5147 514b
514d 5151 5155 5159 515c 5160 5164 5166
5167 516c 1 5170 5174 5178 517b 517f
5183 5187 518b 518e 5192 5196 5199 519c
51a1 51a4 51a7 51aa 51ab 51b0 51b1 1
51b3 51b8 51bc 51c0 51c3 51c8 51cd 51d0
51d4 51d5 51da 51dd 51e2 51e3 51e8 51eb
51f0 51f1 51f6 51f9 51fd 51fe 5203 5206
520b 520c 5211 5214 5218 521c 521f 5222
5223 5225 5226 522b 522c 5231 5233 5237
523a 523c 5240 5242 524e 5250 5252 5253
5258 525c 525e 526a 526c 5270 5274 5277
527b 527e 5282 5285 5286 528b 528e 5292
5296 5299 529d 52a1 52a4 52a7 52ac 52af
52b2 52b5 52b6 52bb 52bc 1 52be 52c3
52c7 52cb 52ce 52d3 52d8 52db 52df 52e0
52e5 52e8 52ed 52ee 52f3 52f6 52fb 52fc
5301 5304 5308 530c 5310 5314 5317 5318
531a 531f 5320 5322 5323 5328 532b 5330
5331 5336 5339 533e 533f 5344 5347 534b
534c 5351 5354 5359 535a 535f 5360 5365
5367 536b 536e 5372 5376 5379 537d 5381
5384 5387 538c 538f 5392 5395 5396 539b
539c 1 539e 53a3 53a7 53ab 53ae 53b3
53b8 53bb 53bf 53c0 53c5 53c6 53cb 53cd
53d1 53d4 53d8 53dc 53df 53e3 53e7 53ea
53ee 53f2 53f3 53f8 53fa 53fe 5401 5405
5409 540d 5411 5415 5419 541d 5421 5425
5429 542d 5431 5435 5439 543d 5441 5445
5449 544d 5451 545d 5461 5465 5468 546c
5470 5472 5476 547a 547e 5481 5485 5489
548b 548c 5491 1 5495 5499 549d 54a0
54a4 54a8 54ac 54b0 54b3 54b7 54bb 54be
54c1 54c6 54c9 54cc 54cf 54d0 54d5 54d6
1 54d8 54dd 54e1 54e5 54e8 54ed 54f2
54f5 54f9 54fa 54ff 5502 5507 5508 550d
5510 5515 5516 551b 551e 5522 5523 5528
552b 5530 5531 5536 5539 553d 5541 5544
5547 5548 554a 554b 5550 5551 5556 5558
555c 555f 5561 5565 5567 5573 5575 5577
5578 557d 5581 5583 558f 5591 5595 5599
559c 55a0 55a3 55a7 55aa 55ab 55b0 55b3
55b7 55bb 55be 55bf 55c1 55c4 55c7 55c8
55cd 1 55d0 55d5 55d9 55dd 55e0 55e4
55e8 55eb 55ee 55f3 55f6 55f9 55fc 55fd
5602 5603 1 5605 560a 560e 5612 5615
561a 561f 5622 5626 5627 562c 562f 5634
5635 563a 563d 5642 5643 5648 564b 564f
5653 5657 565b 565e 565f 5661 5666 5667
5669 566a 566f 5672 5677 5678 567d 5680
5685 5686 568b 568e 5692 5693 5698 569b
56a0 56a1 56a6 56a7 56ac 56ae 56b2 56b5
56b9 56bd 56c0 56c4 56c8 56cb 56ce 56d3
56d6 56d9 56dc 56dd 56e2 56e3 1 56e5
56ea 56ee 56f2 56f5 56fa 56ff 5702 5706
5707 570c 570d 5712 5714 5718 571b 571f
5723 5726 572a 572e 5731 5735 5739 573a
573f 5741 5745 5748 574c 5750 5754 5758
575c 5760 5764 5768 576c 5770 5774 5778
5784 5788 578c 578f 5793 5797 5799 579d
57a1 57a5 57a8 57ac 57b0 57b2 57b3 57b8
1 57bc 57c0 57c4 57c7 57cb 57cf 57d3
57d7 57da 57de 57e2 57e5 57e8 57ed 57f0
57f3 57f6 57f7 57fc 57fd 1 57ff 5804
5808 580c 580f 5814 5819 581c 5820 5821
5826 5829 582e 582f 5834 5837 583c 583d
5842 5845 5849 584a 584f 5852 5857 5858
585d 5860 5864 5868 586b 586e 586f 5871
5872 5877 5878 587d 587f 5883 5886 5888
588c 588e 589a 589c 589e 589f 58a4 58a8
58aa 58b6 58b8 58ba 58be 58c2 58c5 58c9
58cd 58d1 58d5 58d9 58dc 58dd 58df 58e2
58e5 58e6 58eb 58ef 58f3 58f7 58fb 58ff
5903 5907 590b 590f 5913 591f 5923 5927
592a 592e 5932 5934 5938 593c 5940 5943
5947 594b 594d 594e 5953 1 5957 595b
595f 5962 5966 596a 596e 5972 5975 5979
597d 5980 5983 5988 598b 598e 5991 5992
5997 5998 1 599a 599f 59a3 59a7 59aa
59af 59b4 59b7 59bb 59bc 59c1 59c4 59c9
59ca 59cf 59d2 59d7 59d8 59dd 59e0 59e4
59e5 59ea 59ed 59f2 59f3 59f8 59fb 59ff
5a03 5a06 5a09 5a0a 5a0c 5a0d 5a12 5a13
5a18 5a1a 5a1e 5a21 5a23 5a27 5a29 5a35
5a37 5a39 5a3a 5a3f 5a43 5a45 5a51 5a53
5a57 5a5b 5a5f 5a63 5a67 5a6b 5a6f 5a73
5a77 5a7b 5a87 5a8b 5a8f 5a92 5a96 5a9a
5a9c 5aa0 5aa4 5aa8 5aab 5aaf 5ab3 5ab5
5ab6 5abb 1 5abf 5ac3 5ac7 5aca 5ace
5ad2 5ad6 5ada 5add 5ae1 5ae5 5ae8 5aeb
5af0 5af3 5af6 5af9 5afa 5aff 5b00 1
5b02 5b07 5b0b 5b0f 5b12 5b17 5b1c 5b1f
5b23 5b24 5b29 5b2c 5b31 5b32 5b37 5b3a
5b3f 5b40 5b45 5b48 5b4c 5b4d 5b52 5b55
5b5a 5b5b 5b60 5b63 5b67 5b6b 5b6e 5b71
5b72 5b74 5b75 5b7a 5b7b 5b80 5b82 5b86
5b89 5b8b 5b8f 5b91 5b9d 5b9f 5ba1 5ba2
5ba7 5bab 5bad 5bb9 5bbb 5bbf 5bc2 5bc6
5bca 5bce 5bd2 5bd6 5bda 5bdd 5bde 5be0
5be3 5be6 5be7 5bec 5bf0 5bf3 5bf7 5bf9
5bfd 5c00 5c04 5c08 5c0c 5c10 5c14 5c18
5c1c 5c20 5c24 5c28 5c2c 5c30 5c34 5c38
5c3c 5c40 5c44 5c48 5c4c 5c50 5c54 5c58
5c5c 5c60 5c64 5c68 5c6c 5c70 5c74 5c78
5c7b 5c7e 5c81 5c84 5c85 5c87 5c8b 5c8f
5c92 5c95 5c96 5c9b 5c9f 5ca3 5ca6 5ca7
5ca9 5cac 5caf 5cb0 5cb5 5cb9 5cbd 5cc1
5cc5 5cd1 5cd5 5cda 5cde 5ce1 5ce2 5ce7
5ceb 5cef 5cf2 5cf5 5cf9 5cfb 5cff 5d03
5d06 5d09 5d0d 5d0f 5d13 5d17 5d1a 5d1c
5d20 5d24 5d28 5d2b 5d2f 5d33 5d35 5d36
5d3b 1 5d3f 5d43 5d47 5d4a 5d4e 5d52
5d56 5d5a 5d5d 5d61 5d65 5d68 5d6b 5d70
5d73 5d76 5d79 5d7a 5d7f 5d80 1 5d82
5d87 5d8b 5d8f 5d92 5d97 5d9c 5d9f 5da3
5da4 5da9 5dac 5db1 5db2 5db7 5dba 5dbf
5dc0 5dc5 5dc8 5dcc 5dcd 5dd2 5dd5 5dda
5ddb 5de0 5de3 5de7 5deb 5dee 5df1 5df2
5df4 5df5 5dfa 5dfb 5e00 5e02 5e06 5e09
5e0b 5e0f 5e11 5e1d 5e1f 5e21 5e22 5e27
5e2b 5e2d 5e39 5e3b 5e3d 5e41 5e45 5e49
5e4d 5e59 5e5d 5e62 5e66 5e69 5e6a 5e6f
5e73 5e77 5e7a 5e7d 5e81 5e83 5e87 5e8b
5e8e 5e91 5e95 5e97 5e9b 5e9f 5ea2 5ea4
5ea8 5eac 5eb0 5eb3 5eb7 5ebb 5ebd 5ebe
5ec3 1 5ec7 5ecb 5ecf 5ed2 5ed6 5eda
5ede 5ee2 5ee5 5ee9 5eed 5ef0 5ef3 5ef8
5efb 5efe 5f01 5f02 5f07 5f08 1 5f0a
5f0f 5f13 5f17 5f1a 5f1f 5f24 5f27 5f2b
5f2c 5f31 5f34 5f39 5f3a 5f3f 5f42 5f47
5f48 5f4d 5f50 5f54 5f55 5f5a 5f5d 5f62
5f63 5f68 5f6b 5f6f 5f73 5f76 5f79 5f7a
5f7c 5f7d 5f82 5f83 5f88 5f8a 5f8e 5f91
5f93 5f97 5f99 5fa5 5fa7 5fa9 5faa 5faf
5fb3 5fb5 5fc1 5fc3 5fc5 5fc9 5fcd 5fd0
5fd2 5fd6 5fd9 5fdb 5fdf 5fe3 5fe7 5feb
5fef 5ff3 5ff7 5ffb 5ffe 6001 6002 6004
6007 600c 600d 6012 6016 601a 601e 6021
6025 6027 602b 602f 6033 6037 603a 603b
603d 6041 6045 6048 6049 604b 604f 6051
6055 6059 605c 6060 6064 6068 606c 6070
6074 6078 607c 6080 6084 6088 608c 6090
6094 6098 609c 60a0 60a4 60a8 60ac 60b0
60b4 60b8 60bc 60c0 60c4 60c8 60cc 60d0
60d4 60d7 60da 60dd 60e0 60e1 60e3 60e7
60eb 60ee 60f1 60f2 60f7 60fb 60ff 6103
6107 6113 6117 611c 6120 6123 6124 6129
612d 6131 6134 6137 613b 613d 6141 6145
6148 614b 614f 6151 6155 6159 615c 615e
6162 6166 616a 616d 6171 6175 6177 6178
617d 1 6181 6185 6189 618c 6190 6194
6198 619c 619f 61a3 61a7 61aa 61ad 61b2
61b5 61b8 61bb 61bc 61c1 61c2 1 61c4
61c9 61cd 61d1 61d4 61d9 61de 61e1 61e5
61e6 61eb 61ee 61f3 61f4 61f9 61fc 6201
6202 6207 620a 620e 620f 6214 6217 621c
621d 6222 6225 6229 622d 6230 6233 6234
6236 6237 623c 623d 6242 6244 6248 624b
624d 6251 6253 625f 6261 6263 6264 6269
626d 626f 627b 627d 627f 6283 6286 6288
628c 6290 6293 6295 6299 629d 62a1 62a2
62a4 62a7 62a8 62aa 62ad 62b0 62b1 62b6
62ba 62bd 62c1 62c3 62c7 62cb 62cf 62d3
62d7 62d8 62da 62dd 62de 62e0 62e3 62e6
62e7 62ec 62ef 62f2 62f5 62f6 62fb 62fc
62fe 6302 6304 6308 630c 630f 6313 6316
6319 631a 631f 6323 6326 632a 632e 6332
6336 6339 633a 633c 6340 6343 6347 6348
634d 634f 6353 6356 635a 635e 6362 6366
6369 636c 636f 6373 6377 637a 637b 637d
6380 6383 6384 6389 638c 638f 6390 6395
6399 639a 639c 63a0 63a4 63a8 63ac 63af
63b2 63b5 63b9 63bd 63c0 63c1 63c3 63c6
63c9 63ca 63cf 63d2 63d5 63d6 63db 63de
63e1 63e2 63e7 63eb 63ec 63ee 63f2 63f6
63fa 63fe 6402 6405 6409 640d 6410 6414
6418 641c 6420 6424 6428 642c 6430 6434
6438 643c 6440 6444 6448 644c 6450 6454
6458 6464 6468 646c 646f 6473 6477 6479
647d 6481 6485 6488 648c 6490 6492 6493
6498 1 649c 64a0 64a4 64a7 64ab 64af
64b3 64b7 64ba 64be 64c2 64c5 64c8 64cd
64d0 64d3 64d6 64d7 64dc 64dd 1 64df
64e4 64e8 64ec 64ef 64f4 64f9 64fc 6500
6501 6506 6509 650e 650f 6514 6517 651c
651d 6522 6525 6529 652a 652f 6532 6537
6538 653d 6540 6544 6548 654b 654e 654f
6551 6552 6557 6558 655d 655f 6563 6566
6568 656c 656e 657a 657c 657e 657f 6584
6588 658a 6596 6598 659c 65a0 65a3 65a4
65a6 65a9 65ac 65ad 65b2 65b5 65b9 65bc
65bd 65c2 65c5 1 65c8 65cd 65d1 65d4
65d8 65dc 65e0 65e3 65e7 65eb 65ec 65ee
65f1 65f4 65f5 65fa 65fe 6602 6606 6608
660c 660f 6611 6615 6618 661c 6620 6623
6626 6627 6629 662c 6631 6632 6637 663b
663f 6643 6646 664a 664e 6652 6656 6657
6659 665c 665d 665f 6662 6665 6666 666b
666f 6673 6677 6679 667d 6681 6685 6688
668d 668e 6693 6696 6699 669a 669c 66a0
66a4 66a8 66ac 66af 66b3 66b4 66b9 66bc
66bf 66c0 66c2 66c6 66c8 66cc 66d0 66d3
66d5 66d9 66dd 66e1 66e2 66e4 66e7 66e8
66ea 66ed 66f0 66f1 66f6 66fa 66fe 6702
6704 6708 670c 6710 6713 6718 6719 671e
6721 6724 6725 6727 672b 672f 6733 6737
673a 673e 673f 6744 6747 674a 674b 674d
6751 6753 6757 675b 675e 6760 6764 6768
676b 676f 6772 6776 677a 677d 6781 6785
6789 678c 678d 678f 6792 6795 6796 679b
679f 67a2 67a5 67a6 67ab 67af 67b3 67b6
67b9 67ba 67bc 67bf 67c4 67c5 67ca 67ce
67d2 67d6 67da 67de 67e2 67e6 67ea 67ee
67f2 67f6 67fa 67fe 6802 6806 680a 680e
6812 6816 681a 681e 6822 6826 682a 682e
6832 6836 683a 6846 684a 684e 6851 6855
6859 685b 685f 6863 6867 686a 686e 6872
6874 6875 687a 1 687e 6882 6886 6889
688d 6891 6895 6899 689c 68a0 68a4 68a7
68aa 68af 68b2 68b5 68b8 68b9 68be 68bf
1 68c1 68c6 68ca 68ce 68d1 68d6 68db
68de 68e2 68e3 68e8 68eb 68f0 68f1 68f6
68f9 68fe 68ff 6904 6907 690b 690c 6911
6914 6919 691a 691f 6922 6926 692a 692d
6930 6931 6933 6934 6939 693a 693f 6941
6945 6948 694a 694e 6950 695c 695e 6960
6961 6966 696a 696c 6978 697a 697e 6982
6985 6989 698c 6990 6993 6994 6999 699c
69a0 69a3 69a7 69ab 69af 69b2 69b3 69b5
69b8 69bb 69bc 69c1 69c4 69c8 69cc 69cf
69d0 69d2 69d5 69d8 69d9 69de 1 69e1
69e6 69ea 69ee 69f2 69f6 69fa 69fe 6a02
6a06 6a0a 6a0e 6a12 6a16 6a1a 6a1e 6a22
6a26 6a2a 6a2e 6a32 6a36 6a3a 6a3e 6a42
6a46 6a4a 6a4e 6a5a 6a5e 6a62 6a65 6a69
6a6d 6a6f 6a73 6a77 6a7b 6a7e 6a82 6a86
6a88 6a89 6a8e 1 6a92 6a96 6a9a 6a9d
6aa1 6aa5 6aa9 6aad 6ab0 6ab4 6ab8 6abb
6abe 6ac3 6ac6 6ac9 6acc 6acd 6ad2 6ad3
1 6ad5 6ada 6ade 6ae2 6ae5 6aea 6aef
6af2 6af6 6af7 6afc 6aff 6b04 6b05 6b0a
6b0d 6b12 6b13 6b18 6b1b 6b1f 6b20 6b25
6b28 6b2d 6b2e 6b33 6b36 6b3a 6b3e 6b41
6b44 6b45 6b47 6b48 6b4d 6b4e 6b53 6b55
6b59 6b5c 6b5e 6b62 6b64 6b70 6b72 6b74
6b75 6b7a 6b7e 6b80 6b8c 6b8e 6b90 6b94
6b97 6b9b 6b9d 6ba1 6ba5 6ba8 6bac 6bae
6bb2 6bb6 6bb9 6bbd 6bc0 6bc4 6bc7 6bc8
6bcd 6bd0 6bd4 6bd8 6bdb 6bdc 6bde 6be1
6be4 6be5 6bea 1 6bed 6bf2 6bf6 6bfa
6bfe 6c02 6c06 6c0a 6c0e 6c12 6c16 6c1a
6c1e 6c22 6c26 6c2a 6c2e 6c32 6c36 6c3a
6c3e 6c42 6c46 6c4a 6c4e 6c52 6c56 6c5a
6c66 6c6a 6c6e 6c71 6c75 6c79 6c7b 6c7f
6c83 6c87 6c8a 6c8e 6c92 6c94 6c95 6c9a
1 6c9e 6ca2 6ca6 6ca9 6cad 6cb1 6cb5
6cb9 6cbc 6cc0 6cc4 6cc7 6cca 6ccf 6cd2
6cd5 6cd8 6cd9 6cde 6cdf 1 6ce1 6ce6
6cea 6cee 6cf1 6cf6 6cfb 6cfe 6d02 6d03
6d08 6d0b 6d10 6d11 6d16 6d19 6d1e 6d1f
6d24 6d27 6d2b 6d2c 6d31 6d34 6d39 6d3a
6d3f 6d42 6d46 6d4a 6d4d 6d50 6d51 6d53
6d54 6d59 6d5a 6d5f 6d61 6d65 6d68 6d6a
6d6e 6d70 6d7c 6d7e 6d80 6d81 6d86 6d8a
6d8c 6d98 6d9a 6d9c 6da0 6da4 6da7 6da9
6dad 6db1 6db5 6db9 6dbd 6dc1 6dc5 6dc9
6dcd 6dd1 6dd5 6dd9 6ddd 6de1 6de5 6de9
6ded 6df1 6df5 6df9 6dfd 6e01 6e05 6e09
6e0d 6e11 6e15 6e19 6e25 6e29 6e2d 6e30
6e34 6e38 6e3a 6e3e 6e42 6e46 6e49 6e4d
6e51 6e53 6e54 6e59 1 6e5d 6e61 6e65
6e68 6e6c 6e70 6e74 6e78 6e7b 6e7f 6e83
6e86 6e89 6e8e 6e91 6e94 6e97 6e98 6e9d
6e9e 1 6ea0 6ea5 6ea9 6ead 6eb0 6eb5
6eba 6ebd 6ec1 6ec2 6ec7 6eca 6ecf 6ed0
6ed5 6ed8 6edd 6ede 6ee3 6ee6 6eea 6eeb
6ef0 6ef3 6ef8 6ef9 6efe 6f01 6f05 6f09
6f0c 6f0f 6f10 6f12 6f13 6f18 6f19 6f1e
6f20 6f24 6f27 6f29 6f2d 6f2f 6f3b 6f3d
6f3f 6f40 6f45 6f49 6f4b 6f57 6f59 6f5b
6f5f 6f63 6f66 6f68 6f6c 6f70 6f73 6f76
6f77 6f79 6f7c 6f81 6f82 6f87 6f8b 6f8f
6f93 6f97 6f9b 6f9f 6fa3 6fa7 6fab 6faf
6fb3 6fb7 6fbb 6fbf 6fc3 6fc7 6fcb 6fcf
6fd3 6fd7 6fdb 6fdf 6fe3 6fe7 6feb 6fef
6ff3 6fff 7003 7007 700a 700e 7012 7014
7018 701c 7020 7023 7027 702b 702d 702e
7033 1 7037 703b 703f 7042 7046 704a
704e 7052 7055 7059 705d 7060 7063 7068
706b 706e 7071 7072 7077 7078 1 707a
707f 7083 7087 708a 708f 7094 7097 709b
709c 70a1 70a4 70a9 70aa 70af 70b2 70b7
70b8 70bd 70c0 70c4 70c5 70ca 70cd 70d2
70d3 70d8 70db 70df 70e3 70e6 70e9 70ea
70ec 70ed 70f2 70f3 70f8 70fa 70fe 7101
7103 7107 7109 7115 7117 7119 711a 711f
7123 7125 7131 7133 7137 713b 713e 7142
7145 7149 714c 714d 7152 7155 7159 715d
7160 7161 7163 7166 7169 716a 716f 7172
7176 717a 717d 717e 7180 7183 7186 7187
718c 1 718f 7194 7198 719c 71a0 71a4
71a8 71ac 71b0 71b4 71b8 71bc 71c0 71c4
71c8 71cc 71d0 71d4 71d8 71dc 71e0 71e4
71e8 71ec 71f8 71fc 7200 7203 7207 720b
720d 7211 7215 7219 721c 7220 7224 7226
7227 722c 1 7230 7234 7238 723b 723f
7243 7247 724b 724e 7252 7256 7259 725c
7261 7264 7267 726a 726b 7270 7271 1
7273 7278 727c 7280 7283 7288 728d 7290
7294 7295 729a 729d 72a2 72a3 72a8 72ab
72b0 72b1 72b6 72b9 72bd 72be 72c3 72c6
72cb 72cc 72d1 72d4 72d8 72dc 72df 72e2
72e3 72e5 72e6 72eb 72ec 72f1 72f3 72f7
72fa 72fc 7300 7302 730e 7310 7312 7313
7318 731c 731e 732a 732c 732e 7332 7335
7339 733b 733d 7341 7345 7348 734c 734e
7352 7356 7359 735d 7360 7364 7367 7368
736d 7370 7374 7378 737c 7380 7384 7388
738c 7390 7394 7398 739c 73a0 73a4 73a8
73ac 73b0 73b4 73b8 73bc 73c0 73c4 73c8
73cc 73d0 73d4 73e0 73e4 73e8 73eb 73ef
73f3 73f5 73f9 73fd 7401 7404 7408 740c
740e 740f 7414 1 7418 741c 7420 7423
7427 742b 742f 7433 7436 743a 743e 7441
7444 7449 744c 744f 7452 7453 7458 7459
1 745b 7460 7464 7468 746b 7470 7475
7478 747c 747d 7482 7485 748a 748b 7490
7493 7498 7499 749e 74a1 74a5 74a6 74ab
74ae 74b3 74b4 74b9 74bc 74c0 74c4 74c7
74ca 74cb 74cd 74ce 74d3 74d4 74d9 74db
74df 74e2 74e4 74e8 74ea 74f6 74f8 74fa
74fb 7500 7504 7506 7512 7514 7516 751a
751e 7521 7523 7527 752b 752f 7533 7537
753b 753f 7543 7547 754b 754f 7553 7557
755b 755f 7563 7567 756b 756f 7573 7577
757b 757f 7583 7587 7593 7597 759b 759e
75a2 75a6 75a8 75ac 75b0 75b4 75b7 75bb
75bf 75c1 75c2 75c7 1 75cb 75cf 75d3
75d6 75da 75de 75e2 75e6 75e9 75ed 75f1
75f4 75f7 75fc 75ff 7602 7605 7606 760b
760c 1 760e 7613 7617 761b 761e 7623
7628 762b 762f 7630 7635 7638 763d 763e
7643 7646 764b 764c 7651 7654 7658 7659
765e 7661 7666 7667 766c 766f 7673 7677
767a 767d 767e 7680 7681 7686 7687 768c
768e 7692 7695 7697 769b 769d 76a9 76ab
76ad 76ae 76b3 76b7 76b9 76c5 76c7 76c9
76cd 76d1 76d4 76d6 76da 76de 76e1 76e5
76e9 76ec 76f0 76f3 76f7 76fa 76fb 7700
7703 7707 770b 770e 770f 7711 7714 7717
7718 771d 7721 7724 7728 772a 772e 7731
7732 7737 773a 773e 7741 7744 7745 774a
774e 7751 7755 7759 775d 7761 7765 7769
776d 7771 7775 7779 777d 7781 7785 7789
778d 7791 779d 77a1 77a5 77a8 77ac 77b0
77b2 77b6 77ba 77be 77c1 77c5 77c9 77cb
77cc 77d1 1 77d5 77d9 77dd 77e0 77e4
77e8 77ec 77f0 77f3 77f7 77fb 77fe 7801
7806 7809 780c 780f 7810 7815 7816 1
7818 781d 7821 7825 7828 782d 7832 7835
7839 783a 783f 7842 7847 7848 784d 7850
7855 7856 785b 785e 7862 7863 7868 786b
7870 7871 7876 7879 787d 7881 7884 7887
7888 788a 788b 7890 7891 7896 7898 789c
789f 78a1 78a5 78a7 78b3 78b5 78b7 78b8
78bd 78c1 78c3 78cf 78d1 78d3 78d7 78da
78de 78e2 78e6 78ea 78ee 78f2 78f6 78fa
78fe 7902 7906 790a 790e 791a 791e 7922
7925 7929 792d 792f 7933 7937 793b 793e
7942 7946 7948 7949 794e 1 7952 7956
795a 795d 7961 7965 7969 796d 7970 7974
7978 797b 797e 7983 7986 7989 798c 798d
7992 7993 1 7995 799a 799e 79a2 79a5
79aa 79af 79b2 79b6 79b7 79bc 79bf 79c4
79c5 79ca 79cd 79d2 79d3 79d8 79db 79df
79e0 79e5 79e8 79ed 79ee 79f3 79f6 79fa
79fe 7a01 7a04 7a05 7a07 7a08 7a0d 7a0e
7a13 7a15 7a19 7a1c 7a1e 7a22 7a24 7a30
7a32 7a34 7a35 7a3a 7a3e 7a40 7a4c 7a4e
7a50 7a54 7a58 7a5b 7a5f 7a63 7a66 7a67
7a69 7a6d 7a70 7a74 7a77 7a78 7a7a 7a7b
7a80 7a84 7a88 7a8c 7a8f 7a90 7a92 7a95
7a98 7a99 7a9e 7aa2 7aa6 7aac 7aae 7ab2
7ab5 7ab7 7abb 7abe 7ac2 7ac5 7ac9 7acb
7acf 7ad3 7ad6 7ad8 7adc 7ae0 7ae4 7ae7
7ae8 7aea 7aed 7af0 7af1 7af6 7afa 7afe
7b04 7b06 7b0a 7b0e 7b11 7b15 7b19 7b1d
7b21 7b24 7b28 7b2c 7b30 7b34 7b38 7b3c
7b40 7b44 7b48 7b4c 7b50 7b5c 7b60 7b64
7b67 7b6b 7b6f 7b71 7b75 7b79 7b7d 7b80
7b84 7b88 7b8a 7b8b 7b90 1 7b94 7b98
7b9c 7b9f 7ba3 7ba7 7bab 7baf 7bb2 7bb6
7bba 7bbd 7bc0 7bc5 7bc8 7bcb 7bce 7bcf
7bd4 7bd5 1 7bd7 7bdc 7be0 7be4 7be7
7bec 7bf1 7bf4 7bf8 7bf9 7bfe 7c01 7c06
7c07 7c0c 7c0f 7c14 7c15 7c1a 7c1d 7c21
7c22 7c27 7c2a 7c2f 7c30 7c35 7c38 7c3c
7c40 7c43 7c46 7c47 7c49 7c4a 7c4f 7c50
7c55 7c57 7c5b 7c5e 7c60 7c64 7c66 7c72
7c74 7c76 7c77 7c7c 7c80 7c82 7c8e 7c90
7c94 7c98 7c9c 7ca0 7ca4 7ca8 7cac 7cb0
7cb4 7cb8 7cc4 7cc8 7ccc 7ccf 7cd3 7cd7
7cd9 7cdd 7ce1 7ce5 7ce8 7cec 7cf0 7cf2
7cf3 7cf8 1 7cfc 7d00 7d04 7d07 7d0b
7d0f 7d13 7d17 7d1a 7d1e 7d22 7d25 7d28
7d2d 7d30 7d33 7d36 7d37 7d3c 7d3d 1
7d3f 7d44 7d48 7d4c 7d4f 7d54 7d59 7d5c
7d60 7d61 7d66 7d69 7d6e 7d6f 7d74 7d77
7d7c 7d7d 7d82 7d85 7d89 7d8a 7d8f 7d92
7d97 7d98 7d9d 7da0 7da4 7da8 7dab 7dae
7daf 7db1 7db2 7db7 7db8 7dbd 7dbf 7dc3
7dc6 7dc8 7dcc 7dce 7dda 7ddc 7dde 7ddf
7de4 7de8 7dea 7df6 7df8 7dfc 7dff 7e03
7e07 7e0b 7e0f 7e13 7e17 7e1a 7e1b 7e1d
7e20 7e23 7e24 7e29 7e2d 7e30 7e34 7e36
7e3a 7e3d 7e41 7e45 7e49 7e4d 7e51 7e55
7e59 7e5d 7e61 7e65 7e69 7e6d 7e71 7e75
7e79 7e7d 7e81 7e85 7e89 7e8d 7e91 7e95
7e99 7e9d 7ea1 7ea5 7ea9 7ead 7eb1 7eb5
7eb8 7ebb 7ebe 7ec1 7ec2 7ec4 7ec8 7ecc
7ecf 7ed2 7ed3 7ed8 7edc 7ee0 7ee3 7ee4
7ee6 7ee9 7eec 7eed 7ef2 7ef6 7efa 7efe
7f02 7f0e 7f12 7f17 7f1b 7f1e 7f1f 7f24
7f28 7f2c 7f2f 7f32 7f36 7f38 7f3c 7f40
7f43 7f46 7f4a 7f4c 7f50 7f54 7f57 7f59
7f5d 7f61 7f65 7f68 7f6c 7f70 7f72 7f73
7f78 1 7f7c 7f80 7f84 7f87 7f8b 7f8f
7f93 7f97 7f9a 7f9e 7fa2 7fa5 7fa8 7fad
7fb0 7fb3 7fb6 7fb7 7fbc 7fbd 1 7fbf
7fc4 7fc8 7fcc 7fcf 7fd4 7fd9 7fdc 7fe0
7fe1 7fe6 7fe9 7fee 7fef 7ff4 7ff7 7ffc
7ffd 8002 8005 8009 800a 800f 8012 8017
8018 801d 8020 8024 8028 802b 802e 802f
8031 8032 8037 8038 803d 803f 8043 8046
8048 804c 804e 805a 805c 805e 805f 8064
8068 806a 8076 8078 807a 807e 8082 8086
808a 8096 809a 809f 80a3 80a6 80a7 80ac
80b0 80b4 80b7 80ba 80be 80c0 80c4 80c8
80cb 80ce 80d2 80d4 80d8 80dc 80df 80e1
80e5 80e9 80ed 80f0 80f4 80f8 80fa 80fb
8100 1 8104 8108 810c 810f 8113 8117
811b 811f 8122 8126 812a 812d 8130 8135
8138 813b 813e 813f 8144 8145 1 8147
814c 8150 8154 8157 815c 8161 8164 8168
8169 816e 8171 8176 8177 817c 817f 8184
8185 818a 818d 8191 8192 8197 819a 819f
81a0 81a5 81a8 81ac 81b0 81b3 81b6 81b7
81b9 81ba 81bf 81c0 81c5 81c7 81cb 81ce
81d0 81d4 81d6 81e2 81e4 81e6 81e7 81ec
81f0 81f2 81fe 8200 8202 8206 820a 820d
820f 8213 8216 821a 821c 8220 8224 8227
8228 822a 822d 8230 8231 8236 823a 823e
8242 8246 824a 824e 8252 8256 8259 825c
825d 825f 8262 8267 8268 826d 8271 8275
8279 827c 8280 8282 8286 828a 828e 8291
8292 8294 8298 829a 829e 82a2 82a5 82a9
82ad 82b1 82b5 82b9 82bd 82c1 82c5 82c9
82cd 82d1 82d5 82d9 82dd 82e1 82e5 82e9
82ed 82f1 82f5 82f9 82fd 8301 8305 8309
830d 8311 8315 8319 831d 8320 8323 8326
8329 832a 832c 8330 8334 8337 833a 833b
8340 8344 8348 834c 8350 835c 8360 8365
8369 836c 836d 8372 8376 837a 837d 8380
8384 8386 838a 838e 8391 8394 8398 839a
839e 83a2 83a5 83a7 83ab 83af 83b3 83b6
83ba 83be 83c0 83c1 83c6 1 83ca 83ce
83d2 83d5 83d9 83dd 83e1 83e5 83e8 83ec
83f0 83f3 83f6 83fb 83fe 8401 8404 8405
840a 840b 1 840d 8412 8416 841a 841d
8422 8427 842a 842e 842f 8434 8437 843c
843d 8442 8445 844a 844b 8450 8453 8457
8458 845d 8460 8465 8466 846b 846e 8472
8476 8479 847c 847d 847f 8480 8485 8486
848b 848d 8491 8494 8496 849a 849c 84a8
84aa 84ac 84ad 84b2 84b6 84b8 84c4 84c6
84c8 84cc 84cf 84d1 84d5 84d9 84dc 84e0
84e4 84e8 84eb 84ec 84ee 84f1 84f4 84f5
84fa 84fe 8506 8508 8509 850c 850e 8512
8519 851b 851f 8522 8524 8528 852c 852f
8533 8536 8539 853a 853f 8542 8546 854a
854d 854e 8550 8553 8556 8557 855c 1
855f 8564 8568 856c 856f 8570 8572 8575
8578 8579 857e 8582 8586 858a 8596 859a
859f 85a3 85a6 85a7 85ac 85b0 85b4 85b7
85ba 85be 85c0 85c4 85c8 85cb 85ce 85d2
85d4 85d8 85dc 85df 85e1 85e5 85e9 85ed
85f0 85f4 85f8 85fa 85fb 8600 1 8604
8608 860c 860f 8613 8617 861b 861f 8622
8626 862a 862d 8630 8635 8638 863b 863e
863f 8644 8645 1 8647 864c 8650 8654
8657 865c 8661 8664 8668 8669 866e 8671
8676 8677 867c 867f 8684 8685 868a 868d
8691 8692 8697 869a 869f 86a0 86a5 86a8
86ac 86b0 86b3 86b6 86b7 86b9 86ba 86bf
86c0 86c5 86c7 86cb 86ce 86d0 86d4 86d6
86e2 86e4 86e6 86e7 86ec 86f0 86f2 86fe
8700 8702 8706 870a 870e 8712 8716 871a
871e 872a 872e 8733 8737 873a 873b 8740
8744 8748 874b 874e 8752 8754 8758 875c
875f 8762 8766 8768 876c 8770 8773 8775
8779 877d 8781 8784 8788 878c 878e 878f
8794 1 8798 879c 87a0 87a3 87a7 87ab
87af 87b3 87b6 87ba 87be 87c1 87c4 87c9
87cc 87cf 87d2 87d3 87d8 87d9 1 87db
87e0 87e4 87e8 87eb 87f0 87f5 87f8 87fc
87fd 8802 8805 880a 880b 8810 8813 8818
8819 881e 8821 8825 8826 882b 882e 8833
8834 8839 883c 8840 8844 8847 884a 884b
884d 884e 8853 8854 8859 885b 885f 8862
8864 8868 886a 8876 8878 887a 887b 8880
8884 8886 8892 8894 8896 889a 889e 88a1
88a5 88a8 88ab 88ac 88b1 88b5 88b9 88bc
88c0 88c4 88c7 88ca 88cf 88d2 88d5 88d8
88d9 88de 88df 1 88e1 88e6 88ea 88ee
88f1 88f6 88fb 88fe 8902 8903 8908 890b
8910 8911 8916 8919 891e 891f 8924 8927
892b 892f 8933 8937 893a 893b 893d 8942
8943 8945 8946 894b 894e 8953 8954 8959
895c 8961 8962 8967 896a 896e 896f 8974
8977 897c 897d 8982 8983 8988 898a 898e
8991 8995 8999 899c 89a0 89a4 89a7 89aa
89af 89b2 89b5 89b8 89b9 89be 89bf 1
89c1 89c6 89ca 89ce 89d1 89d6 89db 89de
89e2 89e3 89e8 89e9 89ee 89f0 89f4 89f7
89fb 89ff 8a02 8a06 8a0a 8a0d 8a11 8a15
8a16 8a1b 8a1f 8a21 8a25 8a29 8a2c 8a30
8a33 8a34 8a39 8a3d 8a41 8a44 8a48 8a4c
8a4f 8a52 8a57 8a5a 8a5d 8a60 8a61 8a66
8a67 1 8a69 8a6e 8a72 8a76 8a79 8a7e
8a83 8a86 8a8a 8a8b 8a90 8a93 8a98 8a99
8a9e 8aa1 8aa6 8aa7 8aac 8aaf 8ab3 8ab7
8abb 8abf 8ac2 8ac3 8ac5 8aca 8acb 8acd
8ace 8ad3 8ad6 8adb 8adc 8ae1 8ae4 8ae9
8aea 8aef 8af2 8af6 8af7 8afc 8aff 8b04
8b05 8b0a 8b0b 8b10 8b12 8b16 8b19 8b1d
8b21 8b24 8b28 8b2c 8b2f 8b32 8b37 8b3a
8b3d 8b40 8b41 8b46 8b47 1 8b49 8b4e
8b52 8b56 8b59 8b5e 8b63 8b66 8b6a 8b6b
8b70 8b71 8b76 8b78 8b7c 8b7f 8b83 8b87
8b8a 8b8e 8b92 8b95 8b99 8b9d 8b9e 8ba3
8ba7 8ba9 8bad 8bb1 8bb5 8bb8 8bbc 8bbf
8bc0 8bc5 8bc9 8bcd 8bd0 8bd4 8bd8 8bdb
8bde 8be3 8be6 8be9 8bec 8bed 8bf2 8bf3
1 8bf5 8bfa 8bfe 8c02 8c05 8c0a 8c0f
8c12 8c16 8c17 8c1c 8c1f 8c24 8c25 8c2a
8c2d 8c32 8c33 8c38 8c3b 8c3f 8c43 8c47
8c4b 8c4e 8c4f 8c51 8c56 8c57 8c59 8c5a
8c5f 8c62 8c67 8c68 8c6d 8c70 8c75 8c76
8c7b 8c7e 8c82 8c83 8c88 8c8b 8c90 8c91
8c96 8c97 8c9c 8c9e 8ca2 8ca5 8ca9 8cad
8cb0 8cb4 8cb8 8cbb 8cbe 8cc3 8cc6 8cc9
8ccc 8ccd 8cd2 8cd3 1 8cd5 8cda 8cde
8ce2 8ce5 8cea 8cef 8cf2 8cf6 8cf7 8cfc
8cfd 8d02 8d04 8d08 8d0b 8d0f 8d13 8d16
8d1a 8d1e 8d21 8d25 8d29 8d2a 8d2f 8d31
8d35 8d39 8d3d 8d40 8d44 8d48 8d4b 8d4e
8d53 8d56 8d59 8d5c 8d5d 8d62 8d63 1
8d65 8d6a 8d6e 8d72 8d75 8d7a 8d7f 8d82
8d86 8d87 8d8c 8d8f 8d94 8d95 8d9a 8d9d
8da2 8da3 8da8 8dab 8daf 8db3 8db7 8dbb
8dbe 8dbf 8dc1 8dc6 8dc7 8dc9 8dca 8dcf
8dd2 8dd7 8dd8 8ddd 8de0 8de5 8de6 8deb
8dee 8df2 8df3 8df8 8dfb 8e00 8e01 8e06
8e07 8e0c 8e0e 8e12 8e15 8e19 8e1d 8e20
8e24 8e28 8e2b 8e2e 8e33 8e36 8e39 8e3c
8e3d 8e42 8e43 1 8e45 8e4a 8e4e 8e52
8e55 8e5a 8e5f 8e62 8e66 8e67 8e6c 8e6d
8e72 8e74 8e78 8e7b 8e7f 8e83 8e86 8e8a
8e8e 8e91 8e95 8e99 8e9a 8e9f 8ea1 8ea5
8ea9 8eac 8eae 8eb2 8eb5 8eb9 8ebd 8ec1
8ec2 8ec4 8ec7 8ec8 8eca 8ecd 8ed0 8ed1
8ed6 8ed9 8edd 8ee1 8ee5 8ee6 8ee8 8eeb
8eec 8eee 8ef1 8ef4 8ef5 8efa 1 8efd
8f02 8f06 8f0a 8f0f 8f14 8f18 8f1c 8f20
8f23 8f26 8f2a 8f2e 8f33 8f36 8f3b 8f3c
8f41 8f44 8f49 8f4a 8f4f 8f52 8f57 8f58
8f5d 8f60 8f65 8f66 8f6b 8f6e 8f73 8f74
8f79 8f7c 8f80 8f81 8f86 8f89 8f8e 8f8f
8f94 8f97 8f9c 8f9d 8fa2 8fa5 8fa9 8faa
8faf 8fb2 8fb7 8fb8 8fbd 8fc0 8fc4 8fc5
8fca 8fcd 8fd2 8fd3 8fd8 8fdc 8fe0 8fe4
8fe8 8fed 8ff1 8ff5 8ffa 8ffe 9001 9002
9007 900b 900f 9012 9015 9019 901b 901f
9023 9026 9029 902d 902f 9033 9037 903a
903c 9040 9044 9048 904b 904f 9053 9055
9056 905b 1 905f 9063 9067 906a 906e
9072 9076 907a 907d 9081 9085 9088 908b
9090 9093 9096 9099 909a 909f 90a0 1
90a2 90a7 90ab 90af 90b2 90b7 90bc 90bf
90c3 90c4 90c9 90cc 90d1 90d2 90d7 90da
90df 90e0 90e5 90e8 90ec 90ed 90f2 90f5
90fa 90fb 9100 9103 9107 910b 910e 9111
9112 9114 9115 911a 911b 9120 9124 9128
912b 9130 9135 9138 913c 913d 9142 9143
9148 914a 914e 9151 9153 9157 9159 9165
9167 9169 916a 916f 9173 9175 9181 9183
9187 918b 9190 9195 9199 919d 91a1 91a4
91a7 91ab 91af 91b4 91b7 91bc 91bd 91c2
91c5 91ca 91cb 91d0 91d3 91d8 91d9 91de
91e1 91e6 91e7 91ec 91ef 91f4 91f5 91fa
91fd 9201 9202 9207 920a 920f 9210 9215
9218 921d 921e 9223 9226 922a 922b 9230
9233 9238 9239 923e 9241 9245 9246 924b
924e 9253 9254 9259 925d 9261 9265 9269
926e 9272 9276 927b 927f 9282 9283 9288
928c 9290 9293 9296 929a 929c 92a0 92a4
92a7 92aa 92ae 92b0 92b4 92b8 92bb 92bd
92c1 92c5 92c9 92cc 92d0 92d4 92d6 92d7
92dc 1 92e0 92e4 92e8 92eb 92ef 92f3
92f7 92fb 92fe 9302 9306 9309 930c 9311
9314 9317 931a 931b 9320 9321 1 9323
9328 932c 9330 9333 9338 933d 9340 9344
9345 934a 934d 9352 9353 9358 935b 9360
9361 9366 9369 936d 936e 9373 9376 937b
937c 9381 9384 9388 938c 938f 9392 9393
9395 9396 939b 939c 93a1 93a5 93a9 93ac
93b1 93b6 93b9 93bd 93be 93c3 93c4 93c9
93cb 93cf 93d2 93d4 93d8 93da 93e6 93e8
93ea 93eb 93f0 93f4 93f6 9402 9404 9408
940c 940f 9413 9414 9419 941d 9421 9426
9429 942e 942f 9434 9437 943c 943d 9442
9445 944a 944b 9450 9453 9458 9459 945e
9461 9466 9467 946c 946f 9474 9475 947a
947d 9482 9483 9488 948b 9490 9491 9496
9499 949e 949f 94a4 94a7 94ac 94ad 94b2
94b5 94ba 94bb 94c0 94c3 94c8 94c9 94ce
94d1 94d5 94d6 94db 94de 94e3 94e4 94e9
94ec 94f1 94f2 94f7 94fa 94fe 94ff 9504
9507 950c 950d 9512 9515 951a 951b 9520
9523 9528 9529 952e 9531 9535 9536 953b
953e 9543 9544 9549 954c 9551 9552 9557
955a 955f 9560 9565 9568 956d 956e 9573
9576 957b 957c 9581 9584 9589 958a 958f
9592 9597 9598 959d 95a0 95a5 95a6 95ab
95ae 95b3 95b4 95b9 95bc 95c0 95c1 95c6
95c9 95ce 95cf 95d4 95d7 95dc 95dd 95e2
95e5 95ea 95eb 95f0 95f3 95f8 95f9 95fe
9601 9606 9607 960c 960f 9613 9614 9619
961c 9621 9622 9627 962a 962f 9630 9635
9639 963d 9641 9645 964a 964e 9652 9657
965b 965e 965f 9664 9668 966c 966f 9672
9676 9678 967c 9680 9683 9686 968a 968c
9690 9694 9697 9699 969d 96a1 96a5 96a8
96ac 96b0 96b2 96b3 96b8 1 96bc 96c0
96c4 96c7 96cb 96cf 96d3 96d7 96da 96de
96e2 96e5 96e8 96ed 96f0 96f3 96f6 96f7
96fc 96fd 1 96ff 9704 9708 970c 970f
9714 9719 971c 9720 9721 9726 9729 972e
972f 9734 9737 973c 973d 9742 9745 9749
974a 974f 9752 9757 9758 975d 9760 9764
9768 976b 976e 976f 9771 9772 9777 9778
977d 9781 9785 9788 978d 9792 9795 9799
979a 979f 97a0 97a5 97a7 97ab 97ae 97b0
97b4 97b6 97c2 97c4 97c6 97c7 97cc 97d0
97d2 97de 97e0 97e4 97e9 97ec 97f1 97f2
97f7 97fa 97ff 9800 9805 9808 980d 980e
9813 9816 981a 981e 9822 9825 9826 9828
9829 982b 982c 9831 9834 9839 983a 983f
9842 9847 9848 984d 9850 9854 9858 985c
985f 9860 9862 9863 9865 9866 986b 986e
9873 9874 9879 987c 9881 9882 9887 988a
988f 9890 9895 9898 989d 989e 98a3 98a6
98ab 98ac 98b1 98b4 98b9 98ba 98bf 98c2
98c7 98c8 98cd 98d0 98d5 98d6 98db 98de
98e3 98e4 98e9 98ec 98f0 98f4 98f8 98fb
98fc 98fe 98ff 9901 9902 9907 990a 990f
9910 9915 9918 991d 991e 9923 9926 992a
992e 9932 9935 9936 9938 9939 993b 993c
9941 9944 9949 994a 994f 9952 9957 9958
995d 9960 9965 9966 996b 996e 9973 9974
9979 997c 9981 9982 9987 998b 998f 9991
9995 9999 999d 999e 99a0 99a3 99a4 99a6
99a9 99ac 99ad 99b2 99b6 99ba 99bf 99c4
99c8 99cc 99d0 99d3 99d6 99da 99de 99e3
99e6 99eb 99ec 99f1 99f4 99f9 99fa 99ff
9a02 9a07 9a08 9a0d 9a10 9a15 9a16 9a1b
9a1e 9a23 9a24 9a29 9a2c 9a30 9a31 9a36
9a39 9a3e 9a3f 9a44 9a47 9a4c 9a4d 9a52
9a55 9a59 9a5a 9a5f 9a62 9a67 9a68 9a6d
9a71 9a75 9a79 9a7d 9a82 9a86 9a8a 9a8f
9a93 9a96 9a97 9a9c 9aa0 9aa4 9aa7 9aaa
9aae 9ab0 9ab4 9ab8 9abb 9abe 9ac2 9ac4
9ac8 9acc 9acf 9ad1 9ad5 9ad9 9add 9ae0
9ae4 9ae8 9aea 9aeb 9af0 1 9af4 9af8
9afc 9aff 9b03 9b07 9b0b 9b0f 9b12 9b16
9b1a 9b1d 9b20 9b25 9b28 9b2b 9b2e 9b2f
9b34 9b35 1 9b37 9b3c 9b40 9b44 9b47
9b4c 9b51 9b54 9b58 9b59 9b5e 9b61 9b66
9b67 9b6c 9b6f 9b74 9b75 9b7a 9b7d 9b81
9b82 9b87 9b8a 9b8f 9b90 9b95 9b98 9b9c
9ba0 9ba3 9ba6 9ba7 9ba9 9baa 9baf 9bb0
9bb5 9bb9 9bbd 9bc0 9bc5 9bca 9bcd 9bd1
9bd2 9bd7 9bd8 9bdd 9bdf 9be3 9be6 9be8
9bec 9bee 9bfa 9bfc 9bfe 9bff 9c04 9c08
9c0a 9c16 9c18 9c1c 9c20 9c25 9c2a 9c2e
9c32 9c36 9c39 9c3c 9c40 9c44 9c49 9c4c
9c51 9c52 9c57 9c5a 9c5f 9c60 9c65 9c68
9c6d 9c6e 9c73 9c76 9c7b 9c7c 9c81 9c84
9c89 9c8a 9c8f 9c92 9c96 9c97 9c9c 9c9f
9ca4 9ca5 9caa 9cad 9cb1 9cb2 9cb7 9cba
9cbf 9cc0 9cc5 9cc9 9ccd 9cd1 9cd5 9cda
9cde 9ce2 9ce7 9ceb 9cee 9cef 9cf4 9cf8
9cfc 9cff 9d02 9d06 9d08 9d0c 9d10 9d13
9d16 9d1a 9d1c 9d20 9d24 9d27 9d29 9d2d
9d31 9d35 9d38 9d3c 9d40 9d42 9d43 9d48
1 9d4c 9d50 9d54 9d57 9d5b 9d5f 9d63
9d67 9d6a 9d6e 9d72 9d75 9d78 9d7d 9d80
9d83 9d86 9d87 9d8c 9d8d 1 9d8f 9d94
9d98 9d9c 9d9f 9da4 9da9 9dac 9db0 9db1
9db6 9db9 9dbe 9dbf 9dc4 9dc7 9dcc 9dcd
9dd2 9dd5 9dd9 9dda 9ddf 9de2 9de7 9de8
9ded 9df0 9df4 9df8 9dfb 9dfe 9dff 9e01
9e02 9e07 9e08 9e0d 9e11 9e15 9e18 9e1d
9e22 9e25 9e29 9e2a 9e2f 9e30 9e35 9e37
9e3b 9e3e 9e40 9e44 9e46 9e52 9e54 9e56
9e57 9e5c 9e60 9e62 9e6e 9e70 9e74 9e78
9e7b 9e7f 9e80 9e85 9e89 9e8d 9e92 9e95
9e9a 9e9b 9ea0 9ea3 9ea8 9ea9 9eae 9eb1
9eb6 9eb7 9ebc 9ebf 9ec4 9ec5 9eca 9ecd
9ed2 9ed3 9ed8 9edb 9ee0 9ee1 9ee6 9ee9
9eee 9eef 9ef4 9ef7 9efc 9efd 9f02 9f05
9f0a 9f0b 9f10 9f13 9f17 9f18 9f1d 9f20
9f25 9f26 9f2b 9f2e 9f33 9f34 9f39 9f3c
9f40 9f41 9f46 9f49 9f4e 9f4f 9f54 9f57
9f5c 9f5d 9f62 9f65 9f6a 9f6b 9f70 9f73
9f77 9f78 9f7d 9f80 9f85 9f86 9f8b 9f8f
9f93 9f97 9f9b 9fa0 9fa4 9fa8 9fad 9fb1
9fb4 9fb5 9fba 9fbe 9fc2 9fc5 9fc8 9fcc
9fce 9fd2 9fd6 9fd9 9fdc 9fe0 9fe2 9fe6
9fea 9fed 9fef 9ff3 9ff7 9ffb 9ffe a002
a006 a008 a009 a00e 1 a012 a016 a01a
a01d a021 a025 a029 a02d a030 a034 a038
a03b a03e a043 a046 a049 a04c a04d a052
a053 1 a055 a05a a05e a062 a065 a06a
a06f a072 a076 a077 a07c a07f a084 a085
a08a a08d a092 a093 a098 a09b a09f a0a0
a0a5 a0a8 a0ad a0ae a0b3 a0b6 a0ba a0be
a0c1 a0c4 a0c5 a0c7 a0c8 a0cd a0ce a0d3
a0d7 a0db a0de a0e3 a0e8 a0eb a0ef a0f0
a0f5 a0f6 a0fb a0fd a101 a104 a106 a10a
a10c a118 a11a a11c a11d a122 a126 a128
a134 a136 a13a a13f a142 a147 a148 a14d
a150 a155 a156 a15b a15e a163 a164 a169
a16c a170 a174 a178 a17b a17c a17e a17f
a181 a182 a187 a18a a18f a190 a195 a198
a19c a1a0 a1a4 a1a7 a1a8 a1aa a1ab a1ad
a1ae a1b3 a1b6 a1bb a1bc a1c1 a1c4 a1c9
a1ca a1cf a1d2 a1d7 a1d8 a1dd a1e0 a1e5
a1e6 a1eb a1ee a1f3 a1f4 a1f9 a1fd a1ff
a203 a207 a20b a210 a215 a219 a21d a221
a224 a227 a22b a22f a234 a237 a23c a23d
a242 a245 a24a a24b a250 a253 a258 a259
a25e a261 a266 a267 a26c a26f a274 a275
a27a a27d a282 a283 a288 a28b a28f a290
a295 a298 a29d a29e a2a3 a2a6 a2ab a2ac
a2b1 a2b4 a2b8 a2b9 a2be a2c1 a2c6 a2c7
a2cc a2cf a2d4 a2d5 a2da a2de a2e2 a2e6
a2ea a2ef a2f3 a2f7 a2fc a300 a303 a304
a309 a30d a311 a314 a317 a31b a31d a321
a325 a328 a32b a32f a331 a335 a339 a33c
a33e a342 a346 a34a a34d a351 a355 a357
a358 a35d 1 a361 a365 a369 a36c a370
a374 a378 a37c a37f a383 a387 a38a a38d
a392 a395 a398 a39b a39c a3a1 a3a2 1
a3a4 a3a9 a3ad a3b1 a3b4 a3b9 a3be a3c1
a3c5 a3c6 a3cb a3ce a3d3 a3d4 a3d9 a3dc
a3e1 a3e2 a3e7 a3ea a3ee a3ef a3f4 a3f7
a3fc a3fd a402 a405 a409 a40d a410 a413
a414 a416 a417 a41c a41d a422 a426 a42a
a42d a432 a437 a43a a43e a43f a444 a445
a44a a44c a450 a453 a455 a459 a45b a467
a469 a46b a46c a471 a475 a477 a483 a485
a489 a48d a492 a497 a49b a49f a4a3 a4a6
a4a9 a4ad a4b1 a4b6 a4b9 a4be a4bf a4c4
a4c7 a4cc a4cd a4d2 a4d5 a4da a4db a4e0
a4e3 a4e8 a4e9 a4ee a4f1 a4f6 a4f7 a4fc
a4ff a503 a504 a509 a50c a511 a512 a517
a51a a51e a51f a524 a527 a52c a52d a532
a536 a53a a53e a542 a547 a54b a54f a554
a558 a55b a55c a561 a565 a569 a56c a56f
a573 a575 a579 a57d a580 a583 a587 a589
a58d a591 a594 a596 a59a a59e a5a2 a5a5
a5a9 a5ad a5af a5b0 a5b5 1 a5b9 a5bd
a5c1 a5c4 a5c8 a5cc a5d0 a5d4 a5d7 a5db
a5df a5e2 a5e5 a5ea a5ed a5f0 a5f3 a5f4
a5f9 a5fa 1 a5fc a601 a605 a609 a60c
a611 a616 a619 a61d a61e a623 a626 a62b
a62c a631 a634 a639 a63a a63f a642 a646
a647 a64c a64f a654 a655 a65a a65d a661
a665 a668 a66b a66c a66e a66f a674 a675
a67a a67e a682 a685 a68a a68f a692 a696
a697 a69c a69d a6a2 a6a4 a6a8 a6ab a6ad
a6b1 a6b3 a6bf a6c1 a6c3 a6c4 a6c9 a6cd
a6cf a6db a6dd a6e1 a6e5 a6e8 a6ec a6ed
a6f2 a6f6 a6fa a6ff a702 a707 a708 a70d
a710 a715 a716 a71b a71e a723 a724 a729
a72c a731 a732 a737 a73a a73f a740 a745
a748 a74d a74e a753 a756 a75b a75c a761
a764 a769 a76a a76f a772 a777 a778 a77d
a780 a784 a785 a78a a78d a792 a793 a798
a79b a7a0 a7a1 a7a6 a7a9 a7ae a7af a7b4
a7b7 a7bc a7bd a7c2 a7c5 a7c9 a7ca a7cf
a7d2 a7d7 a7d8 a7dd a7e1 a7e5 a7e9 a7ed
a7f2 a7f6 a7fa a7ff a803 a806 a807 a80c
a810 a814 a817 a81a a81e a820 a824 a828
a82b a82e a832 a834 a838 a83c a83f a841
a845 a849 a84d a850 a854 a858 a85a a85b
a860 1 a864 a868 a86c a86f a873 a877
a87b a87f a882 a886 a88a a88d a890 a895
a898 a89b a89e a89f a8a4 a8a5 1 a8a7
a8ac a8b0 a8b4 a8b7 a8bc a8c1 a8c4 a8c8
a8c9 a8ce a8d1 a8d6 a8d7 a8dc a8df a8e4
a8e5 a8ea a8ed a8f1 a8f2 a8f7 a8fa a8ff
a900 a905 a908 a90c a910 a913 a916 a917
a919 a91a a91f a920 a925 a929 a92d a930
a935 a93a a93d a941 a942 a947 a948 a94d
a94f a953 a956 a958 a95c a95e a96a a96c
a96e a96f a974 a978 a97a a986 a988 a98c
a991 a994 a999 a99a a99f a9a2 a9a7 a9a8
a9ad a9b0 a9b5 a9b6 a9bb a9be a9c2 a9c6
a9ca a9cd a9ce a9d0 a9d1 a9d3 a9d4 a9d9
a9dc a9e1 a9e2 a9e7 a9ea a9ef a9f0 a9f5
a9f8 a9fc aa00 aa04 aa07 aa08 aa0a aa0b
aa0d aa0e aa13 aa16 aa1b aa1c aa21 aa24
aa29 aa2a aa2f aa32 aa37 aa38 aa3d aa40
aa45 aa46 aa4b aa4e aa53 aa54 aa59 aa5d
aa5f aa63 aa67 aa6a aa6e aa72 aa75 aa79
aa7d aa80 aa83 aa88 aa8b aa8e aa91 aa92
aa97 aa98 1 aa9a aa9f aaa3 aaa7 aaaa
aaaf aab4 aab7 aabb aabc aac1 aac2 aac7
aac9 aacd aad0 aad4 aad8 aadb aadf aae3
aae6 aae9 aaee aaf1 aaf4 aaf7 aaf8 aafd
aafe 1 ab00 ab05 ab09 ab0d ab10 ab15
ab1a ab1d ab21 ab22 ab27 ab28 ab2d ab2f
ab33 ab36 ab3a ab3e ab42 ab45 ab49 ab4a
ab4c ab50 ab52 ab56 ab5a ab5d ab61 ab65
ab68 ab6c ab70 ab73 ab76 ab7b ab7e ab81
ab84 ab85 ab8a ab8b 1 ab8d ab92 ab96
ab9a ab9d aba2 aba7 abaa abae abaf abb4
abb5 abba abbe abc2 abc5 abca abcf abd2
abd6 abda abdd abde abe3 abe6 abeb abec
abf1 abf4 abf8 abfc abff ac00 ac05 ac06
ac0b ac0d ac11 ac14 ac18 ac1c ac20 ac24
ac28 ac2c ac30 ac34 ac38 ac44 ac49 ac4d
ac51 ac55 ac59 ac5b ac5c ac61 1 ac65
ac69 ac6d ac70 ac74 ac78 ac7c ac80 ac83
ac87 ac8b ac8e ac91 ac96 ac99 ac9c ac9f
aca0 aca5 aca6 1 aca8 acad acb1 acb5
acb8 acbd acc2 acc5 acc9 acca accf acd0
acd5 acd9 acdd ace0 ace5 acea aced acf1
acf2 acf7 acfa acff ad00 ad05 ad08 ad0c
ad10 ad13 ad16 ad17 ad19 ad1a ad1f ad20
ad25 ad27 ad2b ad2e ad32 ad36 ad3a ad3e
ad42 ad46 ad4a ad56 ad5b ad5f ad63 ad67
ad6b ad6d ad71 ad73 ad7f ad81 ad83 ad84
ad89 ad8d ad8f ad9b ad9f ada1 ada2 adab 
31f0
2
0 1 a 6 d :3 6 14 :3 6
15 :3 6 10 :3 6 e :3 6 11 :3 6
c :3 6 b :3 6 11 :3 6 11 :3 6
1d :3 6 10 :3 6 e :3 6 10 :3 6
12 :2 6 1 3 a 13 a :2 1
4 11 1a 19 :2 11 :2 4 14 1d
1c :2 14 :2 4 e 17 16 :2 e :2 4
f 18 17 :2 f :2 4 f 18 17
:2 f :2 4 d 16 15 :2 d :2 4 12
1b 1a :2 12 :2 4 11 1a 19 :2 11
:2 4 f 18 17 :2 f :2 4 10 19
18 :2 10 :2 4 :3 10 :2 4 :3 f :2 4 f
18 17 :2 f :2 4 10 19 18 :2 10
:2 4 10 19 18 :2 10 :2 4 f 18
17 :2 f :2 4 10 19 18 :2 10 :2 4
10 19 18 :2 10 :2 4 :2 c 1b c
:2 4 :2 e 1d e :2 4 :2 b 1a b
:2 4 :2 10 1f 10 :2 4 :2 a 19 a
:2 4 :2 b 1a b :2 4 :2 11 20 11
:2 4 :2 d 1c d :2 4 :2 c 1b c
:2 4 :2 b 1a b :2 4 :2 f 1e f
:2 4 :2 e 1d e :2 4 :2 c 1b c
:2 4 :2 e 1d e :2 4 :2 f 1e f
:2 4 :2 d 1c d :2 4 :2 b 1a b
:2 4 :2 f 1e f :2 4 :2 12 21 12
:2 4 :2 f 1e f :2 4 :2 c 1b c
:2 4 :2 c 1b c :2 4 :2 f 1e f
:2 4 :2 11 20 11 :2 4 :2 11 20 11
:2 4 :2 16 25 16 :2 4 :2 16 25 16
:2 4 :2 b 1a b :2 4 d 16 15
:2 d :2 4 11 1a 19 :2 11 :2 4 :2 f
19 f :2 4 9 12 11 :2 9 :2 4
:2 17 26 17 :2 4 17 1c 1b :2 17
:2 4 18 1d 1c :2 18 :2 4 16 1f
1e :2 16 :2 4 :2 13 22 13 :2 4 :2 14
23 14 :2 4 :2 11 20 11 :2 4 14
1d 1c :2 14 :2 4 15 1e 1d :2 15
:2 4 :2 10 1f 10 :2 4 :2 f 1e f
:2 4 7 10 f :2 7 :2 4 :2 6 1e
6 :2 4 :2 7 1f 7 :2 4 :2 d 25
d :2 4 :2 8 20 8 :2 4 :3 d :2 4
e 17 :3 e 4 7 :2 11 23 :2 2d
3a :2 23 :3 7 :2 d 19 25 :2 2f 43
:2 25 53 59 5c :2 53 6a 7f 8
:2 6a d 10 :2 6a 17 1a :2 6a b
e :2 6a 13 16 :2 6a 1b 1e :2 6a
2c 2f :2 6a b e :2 6a 13 16
:2 6a 1b 1e :2 6a 2d 30 :2 6a b
e :2 6a 13 16 1f 27 :2 1f 33
:2 16 :2 6a b e :2 6a 13 16 1f
27 :2 1f 31 :2 16 :2 6a b e :2 6a
13 16 1f 27 :2 1f 34 :2 16 :2 6a
b e :2 6a 13 16 1f 27 :2 1f
2f :2 16 :2 6a b e :2 6a 13 16
1f 27 :2 1f 2e :2 16 :2 6a b e
:2 6a 13 16 1f 27 :2 1f 34 :2 16
:2 6a b e :2 6a 13 16 1f 27
:2 1f 34 :2 16 :2 6a b e :2 6a 13
16 1f 27 :2 1f 40 :2 16 :2 6a b
e :2 6a 13 16 1f 27 :2 1f 33
:2 16 :2 6a b e :2 6a 13 16 1f
27 :2 1f 31 :2 16 :2 6a b e :2 6a
13 16 :2 6a 1b 1e :2 6a 28 2b
:2 6a b e :2 6a 13 16 :2 6a 1b
1e :2 6a 2a 2d :2 6a 32 35 :2 6a
:2 7 49 :2 4 7 :2 11 23 :2 2d 39
3c 4b 4e 4f :2 4b :2 23 :3 7 :2 d
12 1e 5c 5f :2 1e :2 7 52 :3 4
14 :2 4 14 :2 4 13 :2 4 12 :2 4
10 :2 4 e :2 4 14 :2 4 f :2 4
12 :2 4 e :2 4 e :2 4 11 :2 4
11 :2 4 13 :2 4 11 :2 4 13 :2 4
17 :2 4 14 1d 26 2e :2 1d :2 14
:2 4 13 :2 4 12 :2 4 10 4 :4 8
7 a :2 14 26 :2 30 3c 3f 4e
51 52 :2 4e :2 26 :3 a :2 10 15 21
5f 62 :2 21 6f 72 :2 21 76 79
:2 21 83 86 8f 97 :2 a0 :2 8f b6
:2 86 :2 21 :2 a 55 :2 7 a :2 14 26
:2 30 3c 3f 4e 51 52 :2 4e :2 26
:3 a :2 10 15 21 5f 62 :2 21 :2 a
55 :3 7 :2 12 28 :2 31 46 :2 7 18
:2 4 :4 8 7 a :2 14 26 :2 30 3c
3f 4e 51 52 :2 4e :2 26 :3 a :2 10
15 21 5f 62 :2 21 6f 72 :2 21
76 79 :2 21 83 86 8f 97 :2 a0
:2 8f b6 :2 86 :2 21 :2 a 55 :2 7 a
:2 14 26 :2 30 3c 3f 4e 51 52
:2 4e :2 26 :3 a :2 10 15 21 5f 62
:2 21 :2 a 55 :3 7 :2 12 28 :2 31 46
:2 7 1b :2 4 7 10 1b :2 7 1e
20 :2 1e 26 2f 37 :2 26 3a 3c
:2 3a :2 7 42 4b 53 :2 42 56 58
:2 56 :2 7 a :2 14 26 :2 30 3c 3f
4e 51 52 :2 4e :2 26 :3 a :2 10 15
21 5f 62 :2 21 6f 72 :2 21 76
79 :2 21 83 86 8f 97 :2 a0 :2 8f
b1 :2 86 :2 21 :2 a 55 :2 7 a :2 14
26 :2 30 3c 3f 4e 51 52 :2 4e
:2 26 :3 a :2 10 15 21 5f 62 :2 21
:2 a 55 :3 7 :2 12 28 :2 31 41 :2 7
5a :2 4 :4 9 8 7 :4 2b 2a 29
:2 7 11 1c 21 33 3e 43 :2 f
:2 a :2 13 1f a 7 f d :2 16
22 d 1d :2 a :2 f d :2 16 22
d 10 :2 1a 2c :2 36 42 46 55
58 59 :2 55 :2 2c :3 10 :2 16 1b 27
65 68 :2 27 75 78 :2 27 7c 7f
:2 27 8b 8e :2 27 96 99 :2 27 a5
a8 af b8 bb :2 a8 :2 27 :2 10 5c
:2 d :2 a :4 16 :2 a 7 :3 48 b :2 14
1f 1d :2 28 :2 1d a 3d 46 4f
:2 3d 52 54 :2 52 3c :2 a d :2 17
29 :2 33 3f 42 51 54 55 :2 51
:2 29 :3 d :2 13 18 24 62 65 :2 24
72 75 :2 24 79 7c :2 24 86 89
92 9a :2 a3 :2 92 c0 :2 89 :2 24 :2 d
58 :2 a d :2 17 29 :2 33 3f 42
51 54 55 :2 51 :2 29 :3 d :2 13 18
24 62 65 :2 24 :2 d 58 :3 a :2 15
2b :2 34 50 :2 a 57 :3 7 e 7
4 48 :4 c b a :4 2a 29 28
:2 a 11 23 f :2 a :2 13 1f a
7 f d :2 16 22 d 1d :2 a
:2 f d :2 16 22 d 10 :2 1a 2c
:2 36 42 46 55 58 59 :2 55 :2 2c
:3 10 :2 16 1b 27 65 68 :2 27 75
78 :2 27 7c 7f :2 27 8b 8e :2 27
96 99 :2 27 a5 a8 af b8 bb
:2 a8 :2 27 :2 10 5c :2 d :2 a :4 16 :2 a
7 :4 43 48 :3 4 12 4 :4 8 :2 7
16 7 20 :3 4 10 17 21 24
:2 10 :2 4 12 19 25 28 :2 12 :2 4
13 4 7 10 18 :2 7 1b 1d
:2 1b 7 13 1a 21 2e 31 :2 1a
36 39 :2 13 :2 7 15 :2 1e 27 2a
2d 31 :2 15 7 a 18 21 29
:2 18 :2 a 2d 30 :2 2d 14 20 29
3f 48 60 12 20 2b 37 12
20 2d :2 d :2 16 22 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 58 5b 5c :2 58 :2 2f :3 13 :2 19
1e 2a 68 6b :2 2a 78 7b :2 2a
7f 82 :2 2a 8e 91 :2 2a 99 9c
:2 2a a8 ab b2 bb be :2 ab :2 2a
:2 13 5f :2 10 :2 d :4 19 :2 d a :4 32
14 16 1f 21 2c 2e 40 42
55 57 d 16 18 2e 37 39
51 53 12 1a 27 38 4a d
18 24 34 3c 3f 47 13 15
22 30 32 3e 47 4f 58 66
6f 7d 86 94 9d aa b3 c4
cd e0 e9 fe 107 122 12b 143
14c 15d 166 178 181 199 1a2 1b5
1be 1df 1e1 1ee 1f0 :2 d :2 16 22
d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 58 5b 5c :2 58
:2 2f :3 13 :2 19 1e 2a 68 6b :2 2a
78 7b :2 2a 7f 82 :2 2a 8e 91
:2 2a 99 9c :2 2a a8 ab b2 bb
be :2 ab :2 2a :2 13 5f :2 10 :2 d :4 19
:2 d a :7 7 b :2 14 1f 1d :2 28
:2 1d a 14 20 2c 3a 12 20
2d :2 d :2 16 22 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
58 5b 5c :2 58 :2 2f :3 13 :2 19 1e
2a 68 6b :2 2a 78 7b :2 2a 7f
82 :2 2a 8e 91 :2 2a 99 9c :2 2a
a8 ab b2 bb be :2 ab :2 2a :2 13
5f :2 10 :2 d :4 19 :2 d a :3 38 e
:2 17 22 20 :2 2b :2 20 d :4 11 :2 10
20 10 21 :2 d 17 20 36 3f
57 15 20 2c 15 23 30 :2 10
:2 19 25 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 5b 5e
5f :2 5b :2 32 :3 16 :2 1c 21 2d 6b
6e :2 2d 7b 7e :2 2d 82 85 :2 2d
91 94 :2 2d 9c 9f :2 2d ab ae
b5 be c1 :2 ae :2 2d :2 16 62 :2 13
:2 10 :4 1c :2 10 d :4 3b 10 :2 1a 2c
:2 36 42 46 55 58 59 :2 55 :2 2c
:3 10 :2 16 1b 27 65 68 :2 27 75
78 :2 27 7c 7f :2 27 89 8c 95
9d :2 a6 :2 95 b9 :2 8c :2 27 c1 c4
:2 27 cc cf :2 27 d4 d7 :2 27 e1
e4 :2 27 :2 10 5c :2 d 10 :2 1a 2c
:2 36 42 45 54 57 58 :2 54 :2 2c
:3 10 :2 16 1b 27 65 68 :2 27 :2 10
5b :3 d :2 18 2c :2 35 47 52 :2 d
:4 a 38 a 1a a :5 7 15 7
11 21 f 10 1d :2 a :2 13 1f
a 7 f d :2 16 22 d 1d
:2 a :2 f d :2 16 22 d 10 :2 1a
2c :2 36 42 46 55 58 59 :2 55
:2 2c :3 10 :2 16 1b 27 65 68 :2 27
75 78 :2 27 7c 7f :2 27 8b 8e
:2 27 96 99 :2 27 a5 a8 af b8
bb :2 a8 :2 27 :2 10 5c :2 d :2 a :4 16
:2 a 7 :3 1f a 13 1f :2 a 22
24 :2 22 14 24 12 13 20 :2 d
:2 16 22 d a 12 10 :2 19 25
10 20 :2 d :2 12 10 :2 19 25 10
13 :2 1d 2f :2 39 45 49 58 5b
5c :2 58 :2 2f :3 13 :2 19 1e 2a 68
6b :2 2a 78 7b :2 2a 7f 82 :2 2a
8e 91 :2 2a 99 9c :2 2a a8 ab
b2 bb be :2 ab :2 2a :2 13 5f :2 10
:2 d :4 19 :2 d a :3 26 d :2 18 1f
2d :2 d 32 34 :2 32 10 :2 1a 2c
:2 36 42 46 55 58 59 :2 55 :2 2c
:3 10 :2 16 1b 27 65 68 :2 27 75
78 :2 27 7c 7f :2 27 89 8c 95
9d :2 a6 :2 95 bb :2 8c :2 27 c3 c6
:2 27 ce d1 :2 27 d6 d9 :2 27 e6
e9 :2 27 :2 10 5c :2 d 10 :2 1a 2c
:2 36 42 45 54 57 58 :2 54 :2 2c
:3 10 :2 16 1b 27 65 68 :2 27 :2 10
5b :3 d :2 18 2c :2 35 49 57 :2 d
36 :2 a 26 :2 7 1f 11 1a 30
39 51 f 1a 26 f 1d 2a
:2 a :2 13 1f a 7 f d :2 16
22 d 1d :2 a :2 f d :2 16 22
d 10 :2 1a 2c :2 36 42 46 55
58 59 :2 55 :2 2c :3 10 :2 16 1b 27
65 68 :2 27 75 78 :2 27 7c 7f
:2 27 8b 8e :2 27 96 99 :2 27 a5
a8 af b8 bb :2 a8 :2 27 :2 10 5c
:2 d :2 a :4 16 :2 a 7 :3 4 b :2 14
1f 1d :2 28 :2 1d a 23 2e 3e
4e 5b :2 d :2 16 22 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 58 5b 5c :2 58 :2 2f :3 13 :2 19
1e 2a 68 6b :2 2a 78 7b :2 2a
7f 82 :2 2a 8e 91 :2 2a 99 9c
:2 2a a8 ab b2 bb be :2 ab :2 2a
:2 13 5f :2 10 :2 d :4 19 :2 d a :3 38
e :2 17 22 20 :2 2b :2 20 d 10
:2 1a 2c :2 36 42 46 55 58 59
:2 55 :2 2c :3 10 :2 16 1b 27 65 68
:2 27 75 78 :2 27 7c 7f :2 27 89
8c 95 9d :2 a6 :2 95 b7 :2 8c :2 27
bf c2 :2 27 ca cd :2 27 d2 d5
:2 27 e2 e5 :2 27 :2 10 5c :2 d 10
:2 1a 2c :2 36 42 45 54 57 58
:2 54 :2 2c :3 10 :2 16 1b 27 65 68
:2 27 :2 10 5b :3 d :2 18 2c :2 35 45
53 :2 d 3b :2 a 38 :2 7 :4 4 7
b d :2 b 7 15 :2 7 16 :2 7
15 26 32 3f 48 51 :2 3f 55
5e 66 :2 55 22 2b 37 :2 22 3b
44 4f :2 3b 53 20 2b :2 53 2f
38 46 :2 2f 4a 53 5e :2 4a 22
2b 37 :2 22 3b 43 4d :2 15 7
a 13 1f :2 a 24 22 :2 2d :2 22
d :2 17 29 :2 33 3f 42 51 54
55 :2 51 :2 29 :3 d :2 13 18 24 62
65 :2 24 72 75 :2 24 79 7c :2 24
86 89 92 9a :2 a3 :2 92 b4 :2 89
:2 24 :2 d 58 :2 a d :2 17 29 :2 33
3f 42 51 54 55 :2 51 :2 29 :3 d
:2 13 18 24 62 65 :2 24 :2 d 58
:3 a :2 15 2b :2 34 44 :2 a 7 3c
d 16 22 :2 d 25 28 :2 25 d
:2 17 29 :2 33 3f 43 52 55 56
:2 52 :2 29 :3 d :2 13 18 24 62 65
:2 24 72 75 :2 24 79 7c :2 24 86
89 92 9a :2 92 a7 :2 89 :2 24 af
b2 :2 24 ba bd :2 24 c2 c5 :2 24
d2 d5 :2 24 :2 d 59 :2 a d :2 17
29 :2 33 3f 42 51 54 55 :2 51
:2 29 :3 d :2 13 18 24 62 65 :2 24
:2 d 58 :3 a :2 15 29 32 3e :2 29
42 50 :2 a 2a 3c :3 7 16 7
:4 b :2 a 19 a 20 :2 7 4 f
b 14 1c :2 b 1f 21 :2 1f a
29 32 3a :2 29 3d 40 :2 3d 28
:2 a b 14 1c :2 b 1f 21 :2 1f
a 28 31 39 :2 28 3c 3e :2 3c
27 :3 a 15 a 7 41 d 16
1e :2 d 21 23 :2 21 a 15 a
7 25 41 e 17 1f :2 e 22
24 :2 22 d 2b 34 3c :2 2b 3f
41 :2 3f 2a :2 d a 15 a 7
44 41 d 16 1e :2 d 21 23
:2 21 a 14 1d 29 :2 14 2c 2e
37 3f :2 2e :2 14 :2 a 15 a 25
41 :2 7 45 f :3 4 :3 17 :2 4 12
:2 4 13 :2 4 d :2 16 22 29 32
3e :2 29 42 :2 4b :2 22 21 5a :2 d
:2 4 12 19 22 2e :2 19 32 :2 3b
:2 12 4 7 10 18 :2 7 1b 1d
:2 1b 7 1a 29 37 3f 4a 54
25 :2 1a 7 a 13 24 :2 a 27
2a :2 27 d :2 17 29 :2 33 3f 43
52 55 56 :2 52 :2 29 :3 d :2 13 18
24 62 65 :2 24 72 75 :2 24 79
7c :2 24 86 89 92 9a :2 92 ac
:2 89 :2 24 b4 b7 :2 24 bf c2 :2 24
c7 ca :2 24 d6 d9 :2 24 :2 d 59
:2 a d :2 17 29 :2 33 3f 42 51
54 55 :2 51 :2 29 :3 d :2 13 18 24
62 65 :2 24 :2 d 58 :3 a :2 15 29
32 43 :2 29 47 54 :2 a 2c :3 7
15 7 11 15 24 2f 34 44
4f 54 58 f 1b f 10 1d
e 18 e 1c e 1e :2 a :2 13
1f a 7 f d :2 16 22 d
1d :2 a :2 f d :2 16 22 d 10
:2 1a 2c :2 36 42 46 55 58 59
:2 55 :2 2c :3 10 :2 16 1b 27 65 68
:2 27 75 78 :2 27 7c 7f :2 27 8b
8e :2 27 96 99 :2 27 a5 a8 af
b8 bb :2 a8 :2 27 :2 10 5c :2 d :2 a
:4 16 :2 a 7 :3 1f b :2 14 1f 1d
:2 28 :2 1d a :4 3e 3d 3c :3 a 18
a d 16 1f :2 d 22 24 :2 22
d 1d d 26 :2 a 57 :3 7 16
:2 7 12 7 a e 10 :2 e d
14 20 23 :2 d 26 28 :2 26 17
2e 45 4d 54 6d 75 7c 10
18 1f 37 3f 46 60 6e 89
15 26 32 3b 15 16 1e 2f
3b 48 5e :2 10 :2 19 25 10 d
15 13 :2 1c 28 13 23 :2 10 :2 15
13 :2 1c 28 13 16 :2 20 32 :2 3c
48 4c 5b 5e 5f :2 5b :2 32 :3 16
:2 1c 21 2d 6b 6e :2 2d 7b 7e
:2 2d 82 85 :2 2d 91 94 :2 2d 9c
9f :2 2d ab ae b5 be c1 :2 ae
:2 2d :2 16 62 :2 13 :2 10 :4 1c :2 10 d
:3 2c 11 :2 1a 25 23 :2 2e :2 23 :2 10
1e 10 14 1d 2b :2 14 2e 30
:2 2e 13 37 40 49 :2 37 4c 4e
:2 4c 36 :2 13 1d 34 4d 55 5c
75 7d 84 16 1e 25 3d 45
4c 68 78 1b 2c 38 1b 1c
24 35 41 4e 64 :2 16 :2 1f 2b
16 13 1b 19 :2 22 2e 19 29
:2 16 :2 1b 19 :2 22 2e 19 1c :2 26
38 :2 42 4e 52 61 64 65 :2 61
:2 38 :3 1c :2 22 27 33 71 74 :2 33
81 84 :2 33 88 8b :2 33 97 9a
:2 33 a2 a5 :2 33 b1 b4 bb c4
c7 :2 b4 :2 33 :2 1c 68 :2 19 :2 16 :4 22
:2 16 13 :4 51 13 22 13 :4 10 d
3e 14 :2 1d 28 26 :2 31 :2 26 13
45 4e 57 :2 45 5a 5c :2 5a 44
:2 13 1a 31 4a 52 59 72 7a
81 13 1b 22 3a 42 49 65
75 18 29 35 18 19 21 32
3e 4b 61 :2 13 :2 1c 28 13 10
18 16 :2 1f 2b 16 26 :2 13 :2 18
16 :2 1f 2b 16 19 :2 23 35 :2 3f
4b 4f 5e 61 62 :2 5e :2 35 :3 19
:2 1f 24 30 6e 71 :2 30 7e 81
:2 30 85 88 :2 30 94 97 :2 30 9f
a2 :2 30 ae b1 b8 c1 c4 :2 b1
:2 30 :2 19 65 :2 16 :2 13 :4 1f :2 13 10
:4 5f 3e :2 d 2c 17 2e 47 4f
56 6f 77 7e 10 18 1f 37
3f 46 62 72 15 26 32 15
16 1e 2f 3b 48 5e :2 10 :2 19
25 10 d 15 13 :2 1c 28 13
23 :2 10 :2 15 13 :2 1c 28 13 16
:2 20 32 :2 3c 48 4c 5b 5e 5f
:2 5b :2 32 :3 16 :2 1c 21 2d 6b 6e
:2 2d 7b 7e :2 2d 82 85 :2 2d 91
94 :2 2d 9c 9f :2 2d ab ae b5
be c1 :2 ae :2 2d :2 16 62 :2 13 :2 10
:4 1c :2 10 d :7 a 12 d 14 20
23 :2 d 26 28 :2 26 17 2e 44
4c 53 71 79 80 99 a1 a8
c2 ce ec 15 26 31 3a 15
16 1e 2f 3b 48 5e :2 10 :2 19
25 10 d 15 13 :2 1c 28 13
23 :2 10 :2 15 13 :2 1c 28 13 16
:2 20 32 :2 3c 48 4c 5b 5e 5f
:2 5b :2 32 :3 16 :2 1c 21 2d 6b 6e
:2 2d 7b 7e :2 2d 82 85 :2 2d 91
94 :2 2d 9c 9f :2 2d ab ae b5
be c1 :2 ae :2 2d :2 16 62 :2 13 :2 10
:4 1c :2 10 d :3 2c 11 :2 1a 25 23
:2 2e :2 23 :2 10 1e 10 14 1d 2b
:2 14 2e 30 :2 2e 13 37 40 49
:2 37 4c 4e :2 4c 36 :2 13 1d 34
4d 55 5c 75 7d 84 a0 ac
1b 2c 37 1b 1c 24 35 41
4e 64 :2 16 :2 1f 2b 16 13 1b
19 :2 22 2e 19 29 :2 16 :2 1b 19
:2 22 2e 19 1c :2 26 38 :2 42 4e
52 61 64 65 :2 61 :2 38 :3 1c :2 22
27 33 71 74 :2 33 81 84 :2 33
88 8b :2 33 97 9a :2 33 a2 a5
:2 33 b1 b4 bb c4 c7 :2 b4 :2 33
:2 1c 68 :2 19 :2 16 :4 22 :2 16 13 :4 51
13 22 :2 13 :4 10 d 3e 14 :2 1d
28 26 :2 31 :2 26 13 1a 31 49
51 58 76 7e 85 a0 a8 af
cb d7 18 29 34 18 19 21
32 3e 4b 61 :2 13 :2 1c 28 13
10 18 16 :2 1f 2b 16 26 :2 13
:2 18 16 :2 1f 2b 16 19 :2 23 35
:2 3f 4b 4f 5e 61 62 :2 5e :2 35
:3 19 :2 1f 24 30 6e 71 :2 30 7e
81 :2 30 85 88 :2 30 94 97 :2 30
9f a2 :2 30 ae b1 b8 c1 c4
:2 b1 :2 30 :2 19 65 :2 16 :2 13 :4 1f :2 13
10 :4 41 3e :2 d 2c 17 2e 46
4e 55 73 7b 82 9d a5 ac
c8 d4 15 26 31 15 16 1e
2f 3b 48 5e :2 10 :2 19 25 10
d 15 13 :2 1c 28 13 23 :2 10
:2 15 13 :2 1c 28 13 16 :2 20 32
:2 3c 48 4c 5b 5e 5f :2 5b :2 32
:3 16 :2 1c 21 2d 6b 6e :2 2d 7b
7e :2 2d 82 85 :2 2d 91 94 :2 2d
9c 9f :2 2d ab ae b5 be c1
:2 ae :2 2d :2 16 62 :2 13 :2 10 :4 1c :2 10
d :7 a :4 7 b :2 14 1f 1d :2 28
:2 1d a d 16 1f :2 d 22 24
:2 22 10 :2 1a 2c :2 36 42 46 55
58 59 :2 55 :2 2c :3 10 :2 16 1b 27
65 68 :2 27 75 78 :2 27 7c 7f
:2 27 89 8c 95 9d :2 a6 :2 95 be
:2 8c :2 27 c6 c9 :2 27 d1 d4 :2 27
d9 dc :2 27 e5 e8 :2 27 :2 10 5c
:2 d 10 :2 1a 2c :2 36 42 45 54
57 58 :2 54 :2 2c :3 10 :2 16 1b 27
65 68 :2 27 :2 10 5b :3 d :2 18 2c
:2 35 4c 56 :2 d a 26 :4 11 :2 10
14 16 :2 14 10 20 10 1a 1e
2e 18 19 26 37 4b 17 28
30 37 51 59 60 :2 13 :2 1c 28
13 10 18 16 :2 1f 2b 16 26
:2 13 :2 18 16 :2 1f 2b 16 19 :2 23
35 :2 3f 4b 4f 5e 61 62 :2 5e
:2 35 :3 19 :2 1f 24 30 6e 71 :2 30
7e 81 :2 30 85 88 :2 30 94 97
:2 30 9f a2 :2 30 ae b1 b8 c1
c4 :2 b1 :2 30 :2 19 65 :2 16 :2 13 :4 1f
:2 13 10 :4 18 10 20 10 1a 1e
2e 18 19 26 37 4b 17 23
2b 32 :2 13 :2 1c 28 13 10 18
16 :2 1f 2b 16 26 :2 13 :2 18 16
:2 1f 2b 16 19 :2 23 35 :2 3f 4b
4f 5e 61 62 :2 5e :2 35 :3 19 :2 1f
24 30 6e 71 :2 30 7e 81 :2 30
85 88 :2 30 94 97 :2 30 9f a2
:2 30 ae b1 b8 c1 c4 :2 b1 :2 30
:2 19 65 :2 16 :2 13 :4 1f :2 13 10 :7 d
10 19 27 :2 10 2c 2a 35 3f
:2 2c :2 2a 13 :2 1d 2f :2 39 45 49
58 5b 5c :2 58 :2 2f :3 13 :2 19 1e
2a 68 6b :2 2a 78 7b :2 2a 7f
82 :2 2a 8c 8f 98 a0 :2 a9 :2 98
c1 :2 8f :2 2a c9 cc :2 2a d4 d7
:2 2a dc df :2 2a e8 eb :2 2a :2 13
5f :2 10 13 :2 1d 2f :2 39 45 48
57 5a 5b :2 57 :2 2f :3 13 :2 19 1e
2a 68 6b :2 2a :2 13 5e :3 10 :2 1b
2f :2 38 4f 59 :2 10 42 :2 d 2e
26 :3 a 18 a 7 38 e 17
23 :2 e 26 28 :2 26 d 30 41
4a 52 :2 41 :2 30 2f 57 5a :2 57
:3 d :2 17 29 :2 33 3f 43 52 55
56 :2 52 :2 29 :3 d :2 13 18 24 62
65 :2 24 72 75 :2 24 79 7c :2 24
86 89 92 9a :2 a3 :2 92 bb :2 89
:2 24 c3 c6 :2 24 ce d1 :2 24 d6
d9 :2 24 e2 e5 :2 24 :2 d 59 :2 a
d :2 17 29 :2 33 3f 42 51 54
55 :2 51 :2 29 :3 d :2 13 18 24 62
65 :2 24 :2 d 58 :3 a :2 15 29 :2 32
49 53 :2 a 5c 38 :2 7 a 11
1d 20 :2 a 23 25 :2 23 a 18
1f 2a 2d :2 1f 37 3a :2 18 a
14 1f 2a 35 42 4a 53 5e
69 7c 85 90 9b 12 1f 2d
37 4a 12 13 21 :2 d :2 16 22
d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 58 5b 5c :2 58
:2 2f :3 13 :2 19 1e 2a 68 6b :2 2a
78 7b :2 2a 7f 82 :2 2a 8e 91
:2 2a 99 9c :2 2a a8 ab b2 bb
be :2 ab :2 2a :2 13 5f :2 10 :2 d :4 19
:2 d a :3 29 e :2 17 22 20 :2 2b
:2 20 d 10 :2 1a 2c :2 36 42 46
55 58 59 :2 55 :2 2c :3 10 :2 16 1b
27 65 68 :2 27 75 78 :2 27 7c
7f :2 27 89 8c 95 9d :2 a6 :2 95
b9 :2 8c :2 27 c1 c4 :2 27 cc cf
:2 27 d4 d7 :2 27 e0 e3 :2 27 :2 10
5c :2 d 10 :2 1a 2c :2 36 42 45
54 57 58 :2 54 :2 2c :3 10 :2 16 1b
27 65 68 :2 27 :2 10 5b :3 d :2 18
2c :2 35 47 51 :2 d 3b :2 a e
15 1f 22 :2 e 25 27 :2 25 d
31 3a 4d :2 31 50 52 :2 50 30
5a 10 1e :2 5a 23 21 2c 3f
:2 23 :2 21 59 :2 30 2f :2 d 10 :2 1a
2c :2 36 42 46 55 58 59 :2 55
:2 2c :3 10 :2 16 1b 27 65 68 :2 27
75 78 :2 27 7c 7f :2 27 89 8c
95 9d :2 a6 :2 95 bc :2 8c :2 27 c4
c7 :2 27 cf d2 :2 27 d7 da :2 27
e3 e6 :2 27 :2 10 5c :2 d 10 :2 1a
2c :2 36 42 45 54 57 58 :2 54
:2 2c :3 10 :2 16 1b 27 65 68 :2 27
:2 10 5b :3 d :2 18 2c :2 35 4a 54
:2 d 44 :2 a 14 27 34 45 52
61 12 13 20 2e 46 54 :2 d
:2 16 22 d a 12 10 :2 19 25
10 20 :2 d :2 12 10 :2 19 25 10
13 :2 1d 2f :2 39 45 49 58 5b
5c :2 58 :2 2f :3 13 :2 19 1e 2a 68
6b :2 2a 78 7b :2 2a 7f 82 :2 2a
8e 91 :2 2a 99 9c :2 2a a8 ab
b2 bb be :2 ab :2 2a :2 13 5f :2 10
:2 d :4 19 :2 d a :4 29 a 19 20
2c 2f :2 20 39 3c :2 19 a 14
26 37 46 53 :2 d :2 16 22 d
a 12 10 :2 19 25 10 20 :2 d
:2 12 10 :2 19 25 10 13 :2 1d 2f
:2 39 45 49 58 5b 5c :2 58 :2 2f
:3 13 :2 19 1e 2a 68 6b :2 2a 78
7b :2 2a 7f 82 :2 2a 8e 91 :2 2a
99 9c :2 2a a8 ab b2 bb be
:2 ab :2 2a :2 13 5f :2 10 :2 d :4 19 :2 d
a :3 7 e :2 17 22 20 :2 2b :2 20
d 10 :2 1a 2c :2 36 42 46 55
58 59 :2 55 :2 2c :3 10 :2 16 1b 27
65 68 :2 27 75 78 :2 27 7c 7f
:2 27 89 8c 95 9d :2 a6 :2 95 b9
:2 8c :2 27 c1 c4 :2 27 cc cf :2 27
d4 d7 :2 27 e0 e3 :2 27 :2 10 5c
:2 d 10 :2 1a 2c :2 36 42 45 54
57 58 :2 54 :2 2c :3 10 :2 16 1b 27
65 68 :2 27 :2 10 5b :3 d :2 18 2c
:2 35 47 51 :2 d 3b :2 a 14 2c
34 3b 59 61 68 82 8a 91
ac b8 12 1e 12 13 1b 2c
38 45 :2 d :2 16 22 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 58 5b 5c :2 58 :2 2f :3 13 :2 19
1e 2a 68 6b :2 2a 78 7b :2 2a
7f 82 :2 2a 8e 91 :2 2a 99 9c
:2 2a a8 ab b2 bb be :2 ab :2 2a
:2 13 5f :2 10 :2 d :4 19 :2 d a :3 7
e :2 17 22 20 :2 2b :2 20 d 40
49 52 :2 40 55 57 :2 55 3f :2 d
10 :2 1a 2c :2 36 42 46 55 58
59 :2 55 :2 2c :3 10 :2 16 1b 27 65
68 :2 27 75 78 :2 27 7c 7f :2 27
89 8c 95 9d :2 a6 :2 95 be :2 8c
:2 27 c6 c9 :2 27 d1 d4 :2 27 d9
dc :2 27 e5 e8 :2 27 :2 10 5c :2 d
10 :2 1a 2c :2 36 42 45 54 57
58 :2 54 :2 2c :3 10 :2 16 1b 27 65
68 :2 27 :2 10 5b :3 d :2 18 2c :2 35
4c 56 :2 d 5a :2 a 14 27 34
45 52 61 12 13 20 2e 46
54 :2 d :2 16 22 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
58 5b 5c :2 58 :2 2f :3 13 :2 19 1e
2a 68 6b :2 2a 78 7b :2 2a 7f
82 :2 2a 8e 91 :2 2a 99 9c :2 2a
a8 ab b2 bb be :2 ab :2 2a :2 13
5f :2 10 :2 d :4 19 :2 d a :8 7 15
7 a 13 1f :2 a 22 24 :2 22
14 1f 2c 34 3b 50 5b 60
74 12 :2 d :2 16 22 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 58 5b 5c :2 58 :2 2f :3 13 :2 19
1e 2a 68 6b :2 2a 78 7b :2 2a
7f 82 :2 2a 8e 91 :2 2a 99 9c
:2 2a a8 ab b2 bb be :2 ab :2 2a
:2 13 5f :2 10 :2 d :4 19 :2 d a :3 26
1e 26 2d 45 4d 54 6d :3 12
:2 d :2 16 22 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 58
5b 5c :2 58 :2 2f :3 13 :2 19 1e 2a
68 6b :2 2a 78 7b :2 2a 7f 82
:2 2a 8e 91 :2 2a 99 9c :2 2a a8
ab b2 bb be :2 ab :2 2a :2 13 5f
:2 10 :2 d :4 19 :2 d a :3 26 a 15
:2 a 19 a d 16 23 :2 d 26
28 :2 26 d 19 d 2a :3 a 16
22 2f 3d 48 52 5e 1b 28
35 42 4f 1b 28 34 3e 46
4f 57 5e 1b 27 34 3f 4a
58 1b 26 32 3e 41 44 4a
:2 16 a d 16 18 :2 16 10 19
26 :2 10 29 2b :2 29 1a 2d 4a
51 13 1a 16 22 :3 20 16 :2 1f
2b 16 27 16 :2 1f 2b 16 :4 13
10 18 16 :2 1f 2b 16 26 :2 13
:2 18 16 :2 1f 2b 16 19 :2 23 35
:2 3f 4b 4f 5e 61 62 :2 5e :2 35
:3 19 :2 1f 24 30 6e 71 :2 30 7e
81 :2 30 85 88 :2 30 94 97 :2 30
9f a2 :2 30 ae b1 b8 c1 c4
:2 b1 :2 30 :2 19 65 :2 16 :2 13 :4 1f :2 13
10 :4 2d 1a 2d 4a 51 13 1a
16 22 :3 20 16 :2 1f 2b 16 27
16 :2 1f 2b 16 :4 13 10 18 16
:2 1f 2b 16 26 :2 13 :2 18 16 :2 1f
2b 16 19 :2 23 35 :2 3f 4b 4f
5e 61 62 :2 5e :2 35 :3 19 :2 1f 24
30 6e 71 :2 30 7e 81 :2 30 85
88 :2 30 94 97 :2 30 9f a2 :2 30
ae b1 b8 c1 c4 :2 b1 :2 30 :2 19
65 :2 16 :2 13 :4 1f :2 13 10 :7 d 1a
:2 a 26 a 18 :2 a 19 a d
14 20 23 :2 d 26 28 :2 26 d
18 :2 21 d 2c d 18 1f 28
34 :2 1f 38 :2 41 :2 18 d :5 a 16
22 2f 3d 48 52 5e 1b 28
35 42 4f 1b 28 34 3e 46
4f 57 5e 1b 27 34 3f 4a
58 1b 26 32 3e 41 44 4a
:2 16 a d 16 18 :2 16 17 2a
47 4e 10 17 13 1f :3 1d 13
:2 1c 28 13 24 13 :2 1c 28 13
:4 10 d 15 13 :2 1c 28 13 23
:2 10 :2 15 13 :2 1c 28 13 16 :2 20
32 :2 3c 48 4c 5b 5f 60 :2 5b
:2 32 :3 16 :2 1c 21 2d 6c 6f :2 2d
7c 7f :2 2d 83 86 :2 2d 92 95
:2 2d 9d a0 :2 2d ac af b6 bf
c2 :2 af :2 2d :2 16 63 :2 13 :2 10 :4 1c
:2 10 d :4 1a :2 a :4 7 1f a 13
1a :2 13 28 :2 a 2b 2d :2 2b a
16 a 30 a 16 1d 26 2d
:2 26 3b :2 1d 3e 40 :2 1d 1c 43
45 :2 1c :2 16 a :4 7 a 13 15
:2 13 a 15 :2 a 10 19 22 :2 10
27 25 30 :2 25 a d 1b :2 d
19 :2 22 2b 2f 33 3c 45 :2 33
48 4a :2 33 4c 4d :2 33 50 :2 19
:2 d 1b :2 24 2d 30 33 3c 45
:2 33 48 4a :2 33 4d 4f :2 33 51
52 :2 33 55 :2 1b :2 d 1d :2 d 1b
:2 d 18 :2 d 1b d 17 21 25
3a 43 15 16 23 34 3e 14
22 2f 3f :2 10 :2 19 25 10 d
15 13 :2 1c 28 13 23 :2 10 :2 15
13 :2 1c 28 13 16 :2 20 32 :2 3c
48 4c 5b 5f 60 :2 5b :2 32 :3 16
:2 1c 21 2d 6c 6f :2 2d 7c 7f
:2 2d 83 86 :2 2d 92 95 :2 2d 9d
a0 :2 2d ac af b6 bf c2 :2 af
:2 2d :2 16 63 :2 13 :2 10 :4 1c :2 10 d
:3 30 11 1a 23 :2 11 26 28 :2 26
10 :4 31 30 2f :3 10 1e 10 13
:2 1e 25 33 :2 13 3f 41 :2 3f 13
23 13 43 :2 10 4a :2 d 10 17
23 26 :2 10 29 2b :2 29 10 1e
:2 27 10 13 1c 23 :2 1c 31 :2 13
34 36 :2 34 13 22 13 38 13
22 29 35 38 :2 29 41 44 :2 22
:2 13 22 29 35 38 :2 29 42 45
:2 22 13 :4 10 2f 13 1c 23 :2 1c
30 :2 13 33 35 :2 33 13 21 13
37 13 21 28 33 36 :2 28 3f
42 :2 21 :2 13 21 28 33 36 :2 28
40 43 :2 21 13 :4 10 :5 d 1c :2 d
18 d 10 19 25 :2 10 28 2a
:2 28 13 17 19 :2 17 16 1d 29
2c :2 16 2f 31 :2 2f 20 37 4e
56 5d 76 7e 85 19 21 28
40 48 4f 69 77 92 1e 2f
3b 44 1e 1f 27 38 44 51
67 :2 19 :2 22 2e 19 16 1e 1c
:2 25 31 1c 2c :2 19 :2 1e 1c :2 25
31 1c 1f :2 29 3b :2 45 51 55
64 68 69 :2 64 :2 3b :3 1f :2 25 2a
36 75 78 :2 36 85 88 :2 36 8c
8f :2 36 9b 9e :2 36 a6 a9 :2 36
b5 b8 bf c8 cb :2 b8 :2 36 :2 1f
6c :2 1c :2 19 :4 25 :2 19 16 :3 35 1a
:2 23 2e 2c :2 37 :2 2c :2 19 27 19
1d 26 34 :2 1d 37 39 :2 37 1c
40 49 52 :2 40 55 57 :2 55 3f
:2 1c 26 3d 56 5e 65 7e 86
8d 1f 27 2e 46 4e 55 71
81 24 35 41 24 25 2d 3e
4a 57 6d :2 1f :2 28 34 1f 1c
24 22 :2 2b 37 22 32 :2 1f :2 24
22 :2 2b 37 22 25 :2 2f 41 :2 4b
57 5b 6a 6e 6f :2 6a :2 41 :3 25
:2 2b 30 3c 7b 7e :2 3c 8b 8e
:2 3c 92 95 :2 3c a1 a4 :2 3c ac
af :2 3c bb be c5 ce d1 :2 be
:2 3c :2 25 72 :2 22 :2 1f :4 2b :2 1f 1c
:4 5a 1c 2b 1c :4 19 16 47 1d
:2 26 31 2f :2 3a :2 2f 1c 4e 57
60 :2 4e 63 65 :2 63 4d :2 1c 23
3a 53 5b 62 7b 83 8a 1c
24 2b 43 4b 52 6e 7e 21
32 3e 21 22 2a 3b 47 54
6a :2 1c :2 25 31 1c 19 21 1f
:2 28 34 1f 2f :2 1c :2 21 1f :2 28
34 1f 22 :2 2c 3e :2 48 54 58
67 6b 6c :2 67 :2 3e :3 22 :2 28 2d
39 78 7b :2 39 88 8b :2 39 8f
92 :2 39 9e a1 :2 39 a9 ac :2 39
b8 bb c2 cb ce :2 bb :2 39 :2 22
6f :2 1f :2 1c :4 28 :2 1c 19 :4 68 47
:2 16 35 20 37 50 58 5f 78
80 87 19 21 28 40 48 4f
6b 7b 1e 2f 3b 48 1f 2c
3d 45 1d 29 36 4c :2 19 :2 22
2e 19 16 1e 1c :2 25 31 1c
2c :2 19 :2 1e 1c :2 25 31 1c 1f
:2 29 3b :2 45 51 55 64 68 69
:2 64 :2 3b :3 1f :2 25 2a 36 75 78
:2 36 85 88 :2 36 8c 8f :2 36 9b
9e :2 36 a6 a9 :2 36 b5 b8 bf
c8 cb :2 b8 :2 36 :2 1f 6c :2 1c :2 19
:4 25 :2 19 16 :7 13 1b 16 1d 29
2c :2 16 2f 31 :2 2f 20 37 4d
55 5c 7a 82 89 a2 aa b1
cb d7 f5 1e 2f 3a 43 55
1f 2c 3d 45 1d 29 36 4c
:2 19 :2 22 2e 19 16 1e 1c :2 25
31 1c 2c :2 19 :2 1e 1c :2 25 31
1c 1f :2 29 3b :2 45 51 55 64
68 69 :2 64 :2 3b :3 1f :2 25 2a 36
75 78 :2 36 85 88 :2 36 8c 8f
:2 36 9b 9e :2 36 a6 a9 :2 36 b5
b8 bf c8 cb :2 b8 :2 36 :2 1f 6c
:2 1c :2 19 :4 25 :2 19 16 :3 35 1a :2 23
2e 2c :2 37 :2 2c 19 1d 26 34
:2 1d 37 39 :2 37 1c 40 49 52
:2 40 55 57 :2 55 3f :2 1c 26 3d
56 5e 65 7e 86 8d a9 b5
24 35 40 4d 25 32 43 4b
23 2f 3c 52 :2 1f :2 28 34 1f
1c 24 22 :2 2b 37 22 32 :2 1f
:2 24 22 :2 2b 37 22 25 :2 2f 41
:2 4b 57 5b 6a 6e 6f :2 6a :2 41
:3 25 :2 2b 30 3c 7b 7e :2 3c 8b
8e :2 3c 92 95 :2 3c a1 a4 :2 3c
ac af :2 3c bb be c5 ce d1
:2 be :2 3c :2 25 72 :2 22 :2 1f :4 2b :2 1f
1c :4 5a 1c 2b :2 1c :4 19 16 47
1d :2 26 31 2f :2 3a :2 2f 1c 23
3a 53 5b 62 80 88 8f aa
b2 b9 d5 e1 21 32 3d 4a
22 2f 40 48 20 2c 39 4f
:2 1c :2 25 31 1c 19 21 1f :2 28
34 1f 2f :2 1c :2 21 1f :2 28 34
1f 22 :2 2c 3e :2 48 54 58 67
6b 6c :2 67 :2 3e :3 22 :2 28 2d 39
78 7b :2 39 88 8b :2 39 8f 92
:2 39 9e a1 :2 39 a9 ac :2 39 b8
bb c2 cb ce :2 bb :2 39 :2 22 6f
:2 1f :2 1c :4 28 :2 1c 19 :4 4a 47 :2 16
35 20 37 4f 57 5e 7c 84
8b a6 ae b5 d1 dd 1e 2f
3a 47 1f 2c 3d 45 1d 29
36 4c :2 19 :2 22 2e 19 16 1e
1c :2 25 31 1c 2c :2 19 :2 1e 1c
:2 25 31 1c 1f :2 29 3b :2 45 51
55 64 68 69 :2 64 :2 3b :3 1f :2 25
2a 36 75 78 :2 36 85 88 :2 36
8c 8f :2 36 9b 9e :2 36 a6 a9
:2 36 b5 b8 bf c8 cb :2 b8 :2 36
:2 1f 6c :2 1c :2 19 :4 25 :2 19 16 :7 13
:4 10 14 :2 1d 28 26 :2 31 :2 26 13
16 1f 28 :2 16 2b 2d :2 2b 16
24 16 2f :4 1a 19 1c 20 22
:2 20 1c 2c 1c 26 2a 3a 24
25 32 43 57 23 34 3c 43
5d 65 6c :2 1f :2 28 34 1f 1c
24 22 :2 2b 37 22 32 :2 1f :2 24
22 :2 2b 37 22 25 :2 2f 41 :2 4b
57 5b 6a 6e 6f :2 6a :2 41 :3 25
:2 2b 30 3c 7b 7e :2 3c 8b 8e
:2 3c 92 95 :2 3c a1 a4 :2 3c ac
af :2 3c bb be c5 ce d1 :2 be
:2 3c :2 25 72 :2 22 :2 1f :4 2b :2 1f 1c
:4 24 1c 2c 1c 26 2a 3a 24
25 32 43 57 23 2f 37 3e
:2 1f :2 28 34 1f 1c 24 22 :2 2b
37 22 32 :2 1f :2 24 22 :2 2b 37
22 25 :2 2f 41 :2 4b 57 5b 6a
6e 6f :2 6a :2 41 :3 25 :2 2b 30 3c
7b 7e :2 3c 8b 8e :2 3c 92 95
:2 3c a1 a4 :2 3c ac af :2 3c bb
be c5 ce d1 :2 be :2 3c :2 25 72
:2 22 :2 1f :4 2b :2 1f 1c :7 19 1c 25
33 :2 1c 38 36 41 4b :2 38 :2 36
1c 27 30 39 :2 27 3c 3e :2 27
1c 21 1c 4e :2 19 37 :3 16 24
16 :4 13 41 13 1e 27 30 :2 1e
33 35 :2 1e 13 18 13 :5 10 1e
:2 10 1c 10 1a 25 32 3a 41
56 61 66 :2 18 :2 13 :2 1c 28 13
10 18 16 :2 1f 2b 16 26 :2 13
:2 18 16 :2 1f 2b 16 19 :2 23 35
:2 3f 4b 4f 5e 62 63 :2 5e :2 35
:3 19 :2 1f 24 30 6f 72 :2 30 7f
82 :2 30 86 89 :2 30 95 98 :2 30
a0 a3 :2 30 af b2 b9 c2 c5
:2 b2 :2 30 :2 19 66 :2 16 :2 13 :4 1f :2 13
10 :3 2c 24 2c 33 4b 53 5a
73 85 :2 18 :2 13 :2 1c 28 13 10
18 16 :2 1f 2b 16 26 :2 13 :2 18
16 :2 1f 2b 16 19 :2 23 35 :2 3f
4b 4f 5e 62 63 :2 5e :2 35 :3 19
:2 1f 24 30 6f 72 :2 30 7f 82
:2 30 86 89 :2 30 95 98 :2 30 a0
a3 :2 30 af b2 b9 c2 c5 :2 b2
:2 30 :2 19 66 :2 16 :2 13 :4 1f :2 13 10
:3 2c 10 1b :2 10 1f 10 13 1c
29 :2 13 2c 2e :2 2c 13 1f 13
30 :3 10 1c 28 35 43 4e 58
64 1b 28 35 42 4f 1b 28
34 3e 46 4f 57 5e 1b 27
34 3f 4a 58 1b 26 32 3e
41 44 4a :2 1c 10 13 1c 1e
:2 1c 16 1f 2c :2 16 2f 31 :2 2f
20 33 50 57 19 20 1c 28
:3 26 1c :2 25 31 1c 2d 1c :2 25
31 1c :4 19 16 1e 1c :2 25 31
1c 2c :2 19 :2 1e 1c :2 25 31 1c
1f :2 29 3b :2 45 51 55 64 68
69 :2 64 :2 3b :3 1f :2 25 2a 36 75
78 :2 36 85 88 :2 36 8c 8f :2 36
9b 9e :2 36 a6 a9 :2 36 b5 b8
bf c8 cb :2 b8 :2 36 :2 1f 6c :2 1c
:2 19 :4 25 :2 19 16 :4 33 20 33 50
57 19 20 1c 28 :3 26 1c :2 25
31 1c 2d 1c :2 25 31 1c :4 19
16 1e 1c :2 25 31 1c 2c :2 19
:2 1e 1c :2 25 31 1c 1f :2 29 3b
:2 45 51 55 64 68 69 :2 64 :2 3b
:3 1f :2 25 2a 36 75 78 :2 36 85
88 :2 36 8c 8f :2 36 9b 9e :2 36
a6 a9 :2 36 b5 b8 bf c8 cb
:2 b8 :2 36 :2 1f 6c :2 1c :2 19 :4 25 :2 19
16 :7 13 20 :2 10 d 2c 13 1c
28 :2 13 2b 2d :2 2b 10 1e :2 10
1f 10 13 1a 26 29 :2 13 2c
2e :2 2c 13 1e :2 27 13 32 13
1e 27 33 :2 1e 13 :5 10 1c 28
35 43 4e 58 64 1b 28 35
42 4f 1b 28 34 3e 46 4f
57 5e 1b 27 34 3f 4a 58
1b 26 32 3e 41 44 4a :2 1c
10 13 1c 1e :2 1c 1d 30 4d
54 16 1d 19 25 :3 23 19 :2 22
2e 19 2a 19 :2 22 2e 19 :4 16
13 1b 19 :2 22 2e 19 29 :2 16
:2 1b 19 :2 22 2e 19 1c :2 26 38
:2 42 4e 52 61 65 66 :2 61 :2 38
:3 1c :2 22 27 33 72 75 :2 33 82
85 :2 33 89 8c :2 33 98 9b :2 33
a3 a6 :2 33 b2 b5 bc c5 c8
:2 b5 :2 33 :2 1c 69 :2 19 :2 16 :4 22 :2 16
13 :4 20 :2 10 2f 2c :3 d 18 21
2a :2 18 2d 2f :2 18 d f d
:3 30 e a 17 :2 7 :4 4 8 11
14 :2 11 7 1c 25 2d :2 1c 30
32 :2 30 1b :2 7 a 13 22 :2 a
25 27 :2 25 19 34 48 d 14
10 1c :3 1a 10 :2 19 25 10 21
10 :2 19 25 10 :4 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
58 5c 5d :2 58 :2 2f :3 13 :2 19 1e
2a 69 6c :2 2a 79 7c :2 2a 80
83 :2 2a 8f 92 :2 2a 9a 9d :2 2a
a9 ac b3 bc bf :2 ac :2 2a :2 13
60 :2 10 :2 d :4 19 :2 d a :4 29 14
2d 3b 4a 58 13 27 d 14
10 1c :3 1a 10 :2 19 25 10 21
10 :2 19 25 10 :4 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
58 5c 5d :2 58 :2 2f :3 13 :2 19 1e
2a 69 6c :2 2a 79 7c :2 2a 80
83 :2 2a 8f 92 :2 2a 9a 9d :2 2a
a9 ac b3 bc bf :2 ac :2 2a :2 13
60 :2 10 :2 d :4 19 :2 d a :7 7 a
13 15 :2 13 d :2 17 29 :2 33 3f
43 52 56 57 :2 52 :2 29 :3 d :2 13
18 24 63 66 :2 24 73 76 :2 24
7a 7d :2 24 87 8a 93 9b :2 a4
:2 93 ba :2 8a :2 24 c2 c5 :2 24 cd
d0 :2 24 d5 d8 :2 24 e1 e4 :2 24
:2 d 5a :2 a d :2 17 29 :2 33 3f
42 51 55 56 :2 51 :2 29 :3 d :2 13
18 24 63 66 :2 24 :2 d 59 :3 a
:2 15 29 :2 32 47 51 :2 a 7 19
d 18 16 :2 21 :2 16 d :2 17 29
:2 33 3f 43 52 56 57 :2 52 :2 29
:3 d :2 13 18 24 63 66 :2 24 73
76 :2 24 7a 7d :2 24 87 8a 93
9b :2 a4 :2 93 c3 :2 8a :2 24 cb ce
:2 24 d6 d9 :2 24 de e1 :2 24 ea
ed :2 24 :2 d 5a :2 a d :2 17 29
:2 33 3f 42 51 55 56 :2 51 :2 29
:3 d :2 13 18 24 63 66 :2 24 :2 d
59 :3 a :2 15 29 :2 32 50 12 :2 a
7 3e 19 d 18 16 :2 21 :2 16
d :2 17 29 :2 33 3f 43 52 56
57 :2 52 :2 29 :3 d :2 13 18 24 63
66 :2 24 73 76 :2 24 7a 7d :2 24
87 8a 93 9b :2 a4 :2 93 c0 :2 8a
:2 24 c8 cb :2 24 d3 d6 :2 24 db
de :2 24 e7 ea :2 24 :2 d 5a :2 a
d :2 17 29 :2 33 3f 42 51 55
56 :2 51 :2 29 :3 d :2 13 18 24 63
66 :2 24 :2 d 59 :3 a :2 15 29 :2 32
4d 12 :2 a 3b 19 d :2 17 29
:2 33 3f 43 52 56 57 :2 52 :2 29
:3 d :2 13 18 24 63 66 :2 24 73
76 :2 24 7a 7d :2 24 87 8a 93
9b :2 a4 :2 93 bc :2 8a :2 24 c4 c7
:2 24 cf d2 :2 24 d7 da :2 24 e6
e9 :2 24 :2 d 5a :2 a d :2 17 29
:2 33 3f 42 51 55 56 :2 51 :2 29
:3 d :2 13 18 24 63 66 :2 24 :2 d
59 :3 a :2 15 29 :2 32 49 56 :2 a
:4 7 35 :2 4 8 11 18 :2 11 25
:2 8 28 2a :2 28 7 32 3b 42
:2 3b 50 :2 32 53 55 :2 53 31 :3 7
f 19 :3 7 :2 10 1c :2 7 f e
11 :2 f e 11 :2 f e 11 :2 f
e 11 :2 f e 11 :2 f 3e 41
:2 f 4e 51 :2 f e 11 :2 f 2d
30 :2 f 3b 3e :2 f 5a 5d :2 f
13 16 :2 f 7 a 12 1c :2 a
11 d 19 :3 17 d :2 16 22 d
1e d :2 16 22 d :4 a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 55 59 5a :2 55 :2 2c :3 10 :2 16
1b 27 66 69 :2 27 76 79 :2 27
7d 80 :2 27 8c 8f :2 27 97 9a
:2 27 a6 a9 b0 b9 bc :2 a9 :2 27
:3 10 :2 16 1b 3f 48 4b :2 3f :2 10
5d :2 d :2 a :4 16 :2 a 7 :3 58 7
f 19 :3 7 :2 10 1c :2 7 d e
11 :2 d e 11 :2 d e 11 :2 d
e 11 :2 d e 11 :2 d 25 28
:2 d 35 38 :2 d e 11 :2 d 28
2b :2 d 36 39 :2 d 55 58 :2 d
13 16 :2 d 7 a 12 1c :2 a
11 d 19 :3 17 d :2 16 22 d
1e d :2 16 22 d :4 a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 55 59 5a :2 55 :2 2c :3 10 :2 16
1b 27 66 69 :2 27 76 79 :2 27
7d 80 :2 27 8c 8f :2 27 97 9a
:2 27 a6 a9 b0 b9 bc :2 a9 :2 27
:3 10 :2 16 1b 3f 48 4b :2 3f :2 10
5d :2 d :2 a :4 16 :2 a 7 :3 58 7
:2 10 :3 1c :2 7 d e 11 :2 d e
11 :2 d e 11 :2 d e 11 :2 d
e 11 :2 d e 11 :2 d e 11
:2 d e 11 :2 d e 11 :2 d e
11 :2 d e 11 :2 d e 11 :2 d
4e 51 :2 d 13 16 :2 d e 11
:2 d 4b 4e :2 d 13 16 :2 d e
11 :2 d e 11 :2 d 13 16 :2 d
21 24 :2 d e 11 :2 d e 11
:2 d e 11 :2 d e 11 :2 d e
11 :2 d e 11 :2 d e 11 :2 d
e 11 :2 d 4f 52 :2 d 13 16
:2 d e 11 :2 d e 11 :2 d e
11 :2 d e 11 :2 d 2d 30 :2 d
3c 3f :2 d e 11 :2 d 7 a
12 1c :2 a 11 d 19 :3 17 d
:2 16 22 d 1e d :2 16 22 d
:4 a 7 f d :2 16 22 d 1d
:2 a :2 f d :2 16 22 d 10 :2 1a
2c :2 36 42 46 55 59 5a :2 55
:2 2c :3 10 :2 16 1b 27 66 69 :2 27
76 79 :2 27 7d 80 :2 27 8c 8f
:2 27 97 9a :2 27 a6 a9 b0 b9
bc :2 a9 :2 27 :3 10 :2 16 1b 3f 48
4b :2 3f :2 10 5d :2 d :2 a :4 16 :2 a
7 :3 58 7 d e 11 :2 d e
11 :2 d e 11 :2 d 3b 3e 46
4f 58 :2 46 :2 3e :2 d 13 16 :2 d
e 11 :2 d 15 18 20 29 34
:2 20 :2 18 :2 d 13 16 :2 d e 11
:2 d e 11 :2 d e 11 :2 d e
11 :2 d e 11 :2 d e 11 :2 d
e 11 :2 d e 11 :2 d 55 58
60 13 1c :2 60 :2 58 :2 d 20 23
:2 d e 11 :2 d 2c 2f 37 40
4b :2 37 :2 2f :2 d 13 16 :2 d e
11 :2 d e 11 :2 d e 11 :2 d
e 11 :2 d 7 4 58 a 13
1a :2 13 28 :2 a 2b 2d :2 2b 7
f 19 :3 7 :2 10 1c :2 7 f e
11 :2 f e 11 :2 f e 11 :2 f
e 11 :2 f e 11 :2 f 3e 41
:2 f 4e 51 :2 f e 11 :2 f 27
2a :2 f 35 38 :2 f 7 a 12
1c :2 a 11 d 19 :3 17 d :2 16
22 d 1e d :2 16 22 d :4 a
7 f d :2 16 22 d 1d :2 a
:2 f d :2 16 22 d 10 :2 1a 2c
:2 36 42 46 55 59 5a :2 55 :2 2c
:3 10 :2 16 1b 27 66 69 :2 27 76
79 :2 27 7d 80 :2 27 8c 8f :2 27
97 9a :2 27 a6 a9 b0 b9 bc
:2 a9 :2 27 :3 10 :2 16 1b 3f 48 4b
:2 3f :2 10 5d :2 d :2 a :4 16 :2 a 7
:3 2f 7 f 19 :3 7 :2 10 1c :2 7
f e 11 :2 f e 11 :2 f e
11 :2 f e 11 :2 f e 11 :2 f
3b 3e :2 f 4b 4e :2 f 13 16
:2 f 21 24 :2 f 7 a 12 1c
:2 a 11 d 19 :3 17 d :2 16 22
d 1e d :2 16 22 d :4 a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 55 59 5a :2 55 :2 2c :3 10
:2 16 1b 27 66 69 :2 27 76 79
:2 27 7d 80 :2 27 8c 8f :2 27 97
9a :2 27 a6 a9 b0 b9 bc :2 a9
:2 27 :3 10 :2 16 1b 3f 48 4b :2 3f
:2 10 5d :2 d :2 a :4 16 :2 a 7 :3 2f
7 :2 10 :3 1c :2 7 d e 11 :2 d
e 11 :2 d e 11 :2 d e 11
:2 d e 11 :2 d e 11 :2 d e
11 :2 d e 11 :2 d e 11 :2 d
4e 51 :2 d 13 16 :2 d e 11
:2 d 4b 4e :2 d 13 16 :2 d e
11 :2 d e 11 :2 d 2e 31 :2 d
3c 3f :2 d 7 a 12 1c :2 a
11 d 19 :3 17 d :2 16 22 d
1e d :2 16 22 d :4 a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 55 59 5a :2 55 :2 2c :3 10 :2 16
1b 27 66 69 :2 27 76 79 :2 27
7d 80 :2 27 8c 8f :2 27 97 9a
:2 27 a6 a9 b0 b9 bc :2 a9 :2 27
:3 10 :2 16 1b 3f 48 4b :2 3f :2 10
5d :2 d :2 a :4 16 :2 a 7 :3 2f 7
d e 11 :2 d e 11 :2 d e
11 :2 d 48 4b 53 5c 65 :2 53
:2 4b :2 d 13 16 :2 d e 11 19
22 2d :2 19 :2 11 :2 d 13 16 :2 d
e 11 :2 d e 11 :2 d e 11
:2 d e 11 :2 d 7 2f 58 7
f 19 :3 7 :2 10 1c :2 7 f e
11 :2 f e 11 :2 f e 11 :2 f
e 11 :2 f e 11 :2 f e 11
:2 f 28 2b :2 f 38 3b :2 f e
11 :2 f 2d 30 :2 f 3c 3f :2 f
e 11 :2 f 7 a 12 1c :2 a
11 d 19 :3 17 d :2 16 22 d
1e d :2 16 22 d :4 a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 55 59 5a :2 55 :2 2c :3 10 :2 16
1b 27 66 69 :2 27 76 79 :2 27
7d 80 :2 27 8c 8f :2 27 97 9a
:2 27 a6 a9 b0 b9 bc :2 a9 :2 27
:3 10 :2 16 1b 3f 48 4b :2 3f :2 10
5d :2 d :2 a :4 16 :2 a 7 :3 4 7
f 19 :3 7 :2 10 1c :2 7 f e
11 :2 f e 11 :2 f e 11 :2 f
e 11 :2 f e 11 :2 f 3b 3e
:2 f 4b 4e :2 f 13 16 :2 f 22
25 :2 f 7 a 12 1c :2 a 11
d 19 :3 17 d :2 16 22 d 1e
d :2 16 22 d :4 a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
55 59 5a :2 55 :2 2c :3 10 :2 16 1b
27 66 69 :2 27 76 79 :2 27 7d
80 :2 27 8c 8f :2 27 97 9a :2 27
a6 a9 b0 b9 bc :2 a9 :2 27 :3 10
:2 16 1b 3f 48 4b :2 3f :2 10 5d
:2 d :2 a :4 16 :2 a 7 :3 4 7 :2 10
:3 1c :2 7 d e 11 :2 d e 11
:2 d e 11 :2 d e 11 :2 d e
11 :2 d e 11 :2 d e 11 :2 d
e 11 :2 d e 11 :2 d 4e 51
:2 d 13 16 :2 d e 11 :2 d e
11 :2 d e 11 :2 d 2d 30 :2 d
3c 3f :2 d 7 a 12 1c :2 a
11 d 19 :3 17 d :2 16 22 d
1e d :2 16 22 d :4 a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 55 59 5a :2 55 :2 2c :3 10 :2 16
1b 27 66 69 :2 27 76 79 :2 27
7d 80 :2 27 8c 8f :2 27 97 9a
:2 27 a6 a9 b0 b9 bc :2 a9 :2 27
:3 10 :2 16 1b 3f 48 4b :2 3f :2 10
5d :2 d :2 a :4 16 :2 a 7 :3 4 7
f e 11 :2 f e 11 :2 f e
11 :2 f 13 16 1e 27 30 :2 1e
:2 16 :2 f 34 37 :2 f e 11 :2 f
54 57 5f 13 1e :2 5f :2 57 :2 f
13 16 :2 f e 11 :2 f e 11
:2 f e 11 :2 f e 11 :2 f 7
:4 4 7 :2 11 23 :2 2d 39 3c 4b
4f 50 :2 4b :2 23 :3 7 :2 d 12 1e
5d 60 :2 1e :2 7 53 :2 4 7 :2 11
23 :2 2d 39 3c 4b 4f 50 :2 4b
:2 23 :3 7 :2 d 12 1e 5d 60 :2 1e
:2 7 53 :3 4 b :2 16 26 :2 b 4
1 9 12 9 a :2 14 26 :2 30
3c 40 4f 53 54 :2 4f :2 26 :3 a
:2 10 15 21 60 63 :2 21 :3 a :2 10
15 39 5c 5f :2 68 :2 39 76 79
:2 39 81 84 :2 8d :2 39 :2 a 57 :3 7
c 15 1e 2c a 13 1e f
e 7 e 7 e 7 29 :2 4
:2 9 7 :2 10 1c :2 7 :2 11 23 :2 2d
39 3d 4c 50 51 :2 4c :2 23 :3 7
:2 d 12 1e 5d 60 :2 1e :3 7 :2 d
12 36 42 45 :2 36 4d 50 :2 36
5c 5f 66 6f 72 :2 5f :2 36 :2 7
54 :2 4 7 c 1b a 13 1e
f e 7 e 7 e 7 :2 4
:4 10 :2 4 :8 1 
31f0
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 :4 7 :4 8
:4 9 :4 a :4 b :4 c
:4 d :4 e :4 f :4 10
:4 11 2 :4 12 :2 1
:7 14 :7 15 :7 16 :7 17
:7 18 :7 19 :7 1a :7 1b
:7 1c :7 1d :5 1e :5 1f
:7 20 :7 21 :7 22 :7 23
:7 24 :7 25 :6 26 :6 27
:6 28 :6 29 :6 2a :6 2b
:6 2c :6 2d :6 2e :6 2f
:6 30 :6 31 :6 32 :6 33
:6 34 :6 35 :6 36 :6 37
:6 38 :6 39 :6 3a :6 3b
:6 3c :6 3d :6 3e :6 3f
:6 40 :6 41 :7 42 :7 43
:6 44 :7 45 :6 46 :7 47
:7 48 :7 49 :6 4a :6 4b
:6 4c :7 4d :7 4e :6 4f
:6 50 :7 51 :6 52 :6 53
:6 54 :6 55 :5 56 :7 58
:b 5a :11 5b 5c :2 5b
:2 5c :2 5b :2 5c :2 5b
:2 5d :2 5b :2 5d :2 5b
:2 5d :2 5b :2 5d :2 5b
:2 5e :2 5b :2 5e :2 5b
:2 5e :2 5b :2 5e :2 5b
:2 5f :2 5b :9 5f :2 5b
:2 60 :2 5b :9 60 :2 5b
:2 61 :2 5b :9 61 :2 5b
:2 62 :2 5b :9 62 :2 5b
:2 63 :2 5b :9 63 :2 5b
:2 64 :2 5b :9 64 :2 5b
:2 65 :2 5b :9 65 :2 5b
:2 66 :2 5b :9 66 :2 5b
:2 67 :2 5b :9 67 :2 5b
:2 68 :2 5b :9 68 :2 5b
:2 69 :2 5b :2 69 :2 5b
:2 69 :2 5b :2 69 :2 5b
:2 6a :2 5b :2 6a :2 5b
:2 6a :2 5b :2 6a :2 5b
:2 6a :4 5b :3 5a :11 6d
:b 6e :3 6d :3 76 :3 77
:3 78 :3 79 :3 7b :3 7d
:3 7f :3 81 :3 82 :3 83
:3 84 :3 85 :3 86 :3 87
:3 89 :3 8a :3 8b :a 8c
:3 8d :3 92 :3 93 :5 95
:11 96 :20 97 :3 96 :11 99
:b 9a :3 99 :9 9c :3 95
:5 9f :11 a0 :20 a1 :3 a0
:11 a3 :b a4 :3 a3 :9 a6
:3 9f :1f b3 :11 b4 :20 b5
:3 b4 :11 b7 :b b8 :3 b7
:9 ba :3 b3 :e bd :6 bf
c0 c1 bf :5 c2
be c4 :5 c5 :3 c4
:2 c6 :5 c8 :11 c9 :24 ca
:3 c9 :2 c7 :6 c6 c3
:3 bd :16 cf :11 d0 :20 d1
:3 d0 :11 d3 :b d4 :3 d3
:9 d6 :3 cf :3 d9 da
bd :e da :2 dc dd
dc :5 de db e0
:5 e1 :3 e0 :2 e2 :5 e4
:11 e5 :24 e6 :3 e5 :2 e3
:6 e2 df :4 da :3 bd
:3 ec :5 ed :3 ee :3 ed
:8 f1 :8 f2 :3 f3 :9 f5
:d f7 :b f8 :c fa :6 fc
:4 fd :3 fe fc :5 ff
fb 101 :5 102 :3 101
:2 103 :5 105 :11 106 :24 107
:3 106 :2 104 :6 103 100
:4 fa :a 10d :8 10e :5 10f
:7 110 :27 111 10d :5 112
10c 114 :5 115 :3 114
:2 116 :5 118 :11 119 :24 11a
:3 119 :2 117 :6 116 113
:4 10b :3 fa :a 120 :4 122
:3 123 122 :5 124 121
126 :5 127 :3 126 :2 128
:5 12a :11 12b :24 12c :3 12b
:2 129 :6 128 125 :3 120
:a 131 :5 132 :3 133 :3 132
:5 136 :3 137 :3 138 136
:5 139 135 13b :5 13c
:3 13b :2 13d :5 13f :11 140
:24 141 :3 140 :2 13e :6 13d
13a :4 131 :11 146 :30 147
:3 146 :11 149 :b 14a :3 149
:a 14c 145 :3 131 120
:3 14f 14e :3 120 :3 152
:2 154 155 :2 156 154
:5 157 153 159 :5 15a
:3 159 :2 15b :5 15d :11 15e
:24 15f :3 15e :2 15c :6 15b
158 :3 f5 :9 164 :2 166
167 :2 168 166 :5 169
165 16b :5 16c :3 16b
:2 16d :5 16f :11 170 :24 171
:3 170 :2 16e :6 16d 16a
:3 164 :b 176 :11 177 :30 178
:3 177 :11 17a :b 17b :3 17a
:a 17d :3 176 :3 164 f5
:5 182 :3 183 :3 184 182
:5 185 181 187 :5 188
:3 187 :2 189 :5 18b :11 18c
:24 18d :3 18c :2 18a :6 189
186 :3 180 :a 191 :6 193
:5 194 192 196 :5 197
:3 196 :2 198 :5 19a :11 19b
:24 19c :3 19b :2 199 :6 198
195 :3 191 :a 1a0 :11 1a1
:30 1a2 :3 1a1 :11 1a4 :b 1a5
:3 1a4 :a 1a7 :3 1a0 :3 191
180 :3 f5 :5 1ba :3 1bb
:3 1bc :e 1bd :b 1be :2 1bf
:2 1be :a 1bf :8 1c0 :3 1bd
:b 1c2 :11 1c3 :20 1c4 :3 1c3
:11 1c6 :b 1c7 :3 1c6 :9 1c9
1ca 1c2 :9 1ca :11 1cb
:2e 1cc :3 1cb :11 1ce :b 1cf
:3 1ce :c 1d1 1ca :3 1c2
:3 1d4 :5 1d5 :3 1d6 :3 1d5
1d8 1ba :16 1d8 :16 1d9
:3 1da 1db 1d9 :9 1db
:3 1dc 1dd 1db 1d9
:16 1dd :3 1de 1df 1dd
1d9 :9 1df :f 1e0 :3 1e1
1df :3 1d9 1d8 :3 1ba
:5 1e6 :3 1e8 :3 1e9 :14 1eb
:d 1ed :9 1ef :7 1f0 1f1
:3 1f0 :9 1f3 :11 1f4 :2e 1f5
:3 1f4 :11 1f7 :b 1f8 :3 1f7
:c 1fa :3 1f3 :3 1fd :9 200
:2 201 202 :2 203 :2 204
:2 205 :2 206 200 :5 207
1ff 209 :5 20a :3 209
:2 20b :5 20d :11 20e :24 20f
:3 20e :2 20c :6 20b 208
:3 1ef :12 214 :3 215 :9 216
:3 217 :3 216 :3 214 :3 21b
:3 21c :5 21d :a 21e :8 220
:9 221 :4 222 223 :6 224
220 :5 225 21f 227
:5 228 :3 227 :2 229 :5 22b
:11 22c :24 22d :3 22c :2 22a
:6 229 226 :3 21e :a 232
:3 233 :16 234 :8 236 :8 237
:3 238 239 :6 23a 236
:5 23b 235 23d :5 23e
:3 23d :2 23f :5 241 :11 242
:24 243 :3 242 :2 240 :6 23f
23c :4 234 :3 248 247
:3 234 24a 232 :16 24a
:8 24c :8 24d :3 24e 24f
:6 250 24c :5 251 24b
253 :5 254 :3 253 :2 255
:5 257 :11 258 :24 259 :3 258
:2 256 :6 255 252 :4 24a
:3 232 21e :8 260 :8 261
:3 262 263 :6 264 260
:5 265 25f 267 :5 268
:3 267 :2 269 :5 26b :11 26c
:24 26d :3 26c :2 26a :6 269
266 :4 25e :3 21e 21d
:a 273 :e 275 :4 276 277
:6 278 275 :5 279 274
27b :5 27c :3 27b :2 27d
:5 27f :11 280 :24 281 :3 280
:2 27e :6 27d 27a :3 273
:a 286 :3 287 :16 288 :a 28a
:3 28b 28c :6 28d 28a
:5 28e 289 290 :5 291
:3 290 :2 292 :5 294 :11 295
:24 296 :3 295 :2 293 :6 292
28f :4 288 :3 29b 29c
29a :3 288 29e 286
:a 29e :d 2a0 :3 2a1 2a2
:6 2a3 2a0 :5 2a4 29f
2a6 :5 2a7 :3 2a6 :2 2a8
:5 2aa :11 2ab :24 2ac :3 2ab
:2 2a9 :6 2a8 2a5 :4 29e
:3 286 273 :d 2b3 :3 2b4
2b5 :6 2b6 2b3 :5 2b7
2b2 2b9 :5 2ba :3 2b9
:2 2bb :5 2bd :11 2be :24 2bf
:3 2be :2 2bc :6 2bb 2b8
:4 2b1 :3 273 272 :3 21d
:a 2c6 :9 2c7 :11 2c8 :30 2c9
:3 2c8 :11 2cb :b 2cc :3 2cb
:a 2ce 2cf 2c7 :5 2cf
:5 2d0 :3 2d1 :3 2d3 2d4
:4 2d5 :7 2d6 2d3 :5 2d7
2d2 2d9 :5 2da :3 2d9
:2 2db :5 2dd :11 2de :24 2df
:3 2de :2 2dc :6 2db 2d8
:4 2d0 :3 2e4 :3 2e6 2e7
:4 2e8 :4 2e9 2e6 :5 2ea
2e5 2ec :5 2ed :3 2ec
:2 2ee :5 2f0 :11 2f1 :24 2f2
:3 2f1 :2 2ef :6 2ee 2eb
:4 2e3 :3 2d0 :d 2f8 :11 2f9
:30 2fa :3 2f9 :11 2fc :b 2fd
:3 2fc :a 2ff :3 2f8 2cf
:3 2c7 :3 302 303 2c6
:19 303 :11 304 :30 305 :3 304
:11 307 :b 308 :3 307 :a 30a
303 :3 2c6 :a 30d :c 30e
:d 311 :5 312 313 :2 314
311 :5 315 310 317
:5 318 :3 317 :2 319 :5 31b
:11 31c :24 31d :3 31c :2 31a
:6 319 316 :3 30d :a 322
:11 323 :30 324 :3 323 :11 326
:b 327 :3 326 :a 329 :3 322
:16 32c :2 32d :2 32c :8 32d
:6 32c :11 32e :30 32f :3 32e
:11 331 :b 332 :3 331 :a 334
32d :2 32c :6 338 339
:5 33a 338 :5 33b 337
33d :5 33e :3 33d :2 33f
:5 341 :11 342 :24 343 :3 342
:2 340 :6 33f 33c :4 30d
:c 348 :6 34b :5 34c 34a
34e :5 34f :3 34e :2 350
:5 352 :11 353 :24 354 :3 353
:2 351 :6 350 34d :3 347
:a 359 :11 35a :30 35b :3 35a
:11 35d :b 35e :3 35d :a 360
:3 359 :c 364 :2 365 366
:5 367 364 :5 368 363
36a :5 36b :3 36a :2 36c
:5 36e :11 36f :24 370 :3 36f
:2 36d :6 36c 369 :3 347
:16 375 :11 376 :30 377 :3 376
:11 379 :b 37a :3 379 :a 37c
:3 375 :6 380 381 :5 382
380 :5 383 37f 385
:5 386 :3 385 :2 387 :5 389
:11 38a :24 38b :3 38a :2 388
:6 387 384 :4 347 :3 30d
:3 391 :9 393 :9 395 396
395 :5 397 394 399
:5 39a :3 399 :2 39b :5 39d
:11 39e :24 39f :3 39e :2 39c
:6 39b 398 :3 393 :7 3a5
3a6 3a7 3a8 3a5
:5 3a9 3a4 3ab :5 3ac
:3 3ab :2 3ad :5 3af :11 3b0
:24 3b1 :3 3b0 :2 3ae :6 3ad
3aa :3 393 :3 3b6 :3 3b7
:9 3b8 :3 3b9 :3 3b8 :8 3bc
:5 3bd :8 3be :6 3bf :7 3c0
:3 3bc :5 3c2 :9 3c3 :5 3c5
:6 3c6 :5 3c7 3c6 :5 3c9
3c8 :3 3c6 3c4 3cc
:5 3cd :3 3cc :2 3ce :5 3d0
:11 3d1 :24 3d2 :3 3d1 :2 3cf
:6 3ce 3cb :4 3c3 :5 3d8
:6 3d9 :5 3da 3d9 :5 3dc
3db :3 3d9 3d7 3df
:5 3e0 :3 3df :2 3e1 :5 3e3
:11 3e4 :24 3e5 :3 3e4 :2 3e2
:6 3e1 3de :4 3d6 :3 3c3
:3 3c2 393 :3 3ed :3 3ee
:a 3f0 :5 3f1 3f0 :d 3f3
3f2 :3 3f0 :8 3f6 :5 3f7
:8 3f8 :6 3f9 :7 3fa :3 3f6
:5 3fc :5 3fe :6 3ff :5 400
3ff :5 402 401 :3 3ff
3fd 405 :5 406 :3 405
:2 407 :5 409 :11 40a :24 40b
:3 40a :2 408 :6 407 404
:6 3fc 3eb :3 393 1ef
:c 414 :3 415 414 :16 417
416 :3 414 :5 419 :3 41a
:c 41b :3 41c :17 41f :1b 421
:3 423 :3 424 :3 425 :3 426
:5 429 42a :4 42b :4 42c
429 :5 42d 428 42f
:5 430 :3 42f :2 431 :5 433
:11 434 :24 435 :3 434 :2 432
:6 431 42e :3 41b :12 43a
:3 43b :b 43c :3 43d :3 43c
:3 43a :a 441 :5 442 :c 443
:3 444 443 :c 446 :c 447
445 :3 443 441 :c 44a
:3 44b 44a :c 44d :c 44e
44c :3 44a 449 :3 441
:3 452 :3 453 :9 455 :5 456
:a 457 :8 459 :9 45a :4 45b
45c :6 45d 459 :5 45e
458 460 :5 461 :3 460
:2 462 :5 464 :11 465 :24 466
:3 465 :2 463 :6 462 45f
:3 457 :a 46b :3 46c :16 46d
:8 46f :8 470 :3 471 472
:6 473 46f :5 474 46e
476 :5 477 :3 476 :2 478
:5 47a :11 47b :24 47c :3 47b
:2 479 :6 478 475 :4 46d
:3 481 480 :3 46d 483
46b :16 483 :8 485 :8 486
:3 487 488 :6 489 485
:5 48a 484 48c :5 48d
:3 48c :2 48e :5 490 :11 491
:24 492 :3 491 :2 48f :6 48e
48b :4 483 :3 46b 457
:8 499 :8 49a :4 49b :4 49c
:4 49d 499 :5 49e 498
4a0 :5 4a1 :3 4a0 :2 4a2
:5 4a4 :11 4a5 :24 4a6 :3 4a5
:2 4a3 :6 4a2 49f :4 497
:3 457 456 :a 4ac :e 4ae
:5 4af :4 4b0 :4 4b1 4ae
:5 4b2 4ad 4b4 :5 4b5
:3 4b4 :2 4b6 :5 4b8 :11 4b9
:24 4ba :3 4b9 :2 4b7 :6 4b6
4b3 :3 4ac :a 4bf :16 4c0
:a 4c2 :4 4c3 :4 4c4 :4 4c5
4c2 :5 4c6 4c1 4c8
:5 4c9 :3 4c8 :2 4ca :5 4cc
:11 4cd :24 4ce :3 4cd :2 4cb
:6 4ca 4c7 :4 4c0 :3 4d3
4d4 4d2 :3 4c0 4d6
4bf :a 4d6 :d 4d8 :4 4d9
:4 4da :4 4db 4d8 :5 4dc
4d7 4de :5 4df :3 4de
:2 4e0 :5 4e2 :11 4e3 :24 4e4
:3 4e3 :2 4e1 :6 4e0 4dd
:4 4d6 :3 4bf 4ac :d 4eb
:4 4ec :4 4ed :4 4ee 4eb
:5 4ef 4ea 4f1 :5 4f2
:3 4f1 :2 4f3 :5 4f5 :11 4f6
:24 4f7 :3 4f6 :2 4f4 :6 4f3
4f0 :4 4e9 :3 4ac 4ab
:3 456 :a 4fe :9 4ff :3 500
4ff :5 502 :5 503 :3 504
:3 506 507 :4 508 :7 509
506 :5 50a 505 50c
:5 50d :3 50c :2 50e :5 510
:11 511 :24 512 :3 511 :2 50f
:6 50e 50b :4 503 :3 517
:3 519 51a :4 51b :4 51c
519 :5 51d 518 51f
:5 520 :3 51f :2 521 :5 523
:11 524 :24 525 :3 524 :2 522
:6 521 51e :4 516 :3 503
:d 52a :b 52b :2 52c :3 52a
:3 502 :3 52f 501 :3 4ff
4fe :b 532 :2 533 531
:3 4fe :3 536 :3 537 :8 53a
53b 53c 53a :5 53d
539 53f :5 540 :3 53f
:2 541 :5 543 :11 544 :24 545
:3 544 :2 542 :6 541 53e
:3 455 :8 54b 54c 54d
54b :5 54e 54a 550
:5 551 :3 550 :2 552 :5 554
:11 555 :24 556 :3 555 :2 553
:6 552 54f :3 455 :3 55b
:3 55c :9 55d :3 55e :3 55d
:8 561 :5 562 :8 563 :6 564
:7 565 :3 561 :5 567 :9 568
:5 56a :6 56b :5 56c 56b
:5 56e 56d :3 56b 569
571 :5 572 :3 571 :2 573
:5 575 :11 576 :24 577 :3 576
:2 574 :6 573 570 :4 568
:5 57d :6 57e :5 57f 57e
:5 581 580 :3 57e 57c
584 :5 585 :3 584 :2 586
:5 588 :11 589 :24 58a :3 589
:2 587 :6 586 583 :4 57b
:3 568 :3 567 590 455
:9 590 :3 591 :3 592 :a 594
:5 595 594 :7 597 596
:3 594 :8 59a :5 59b :8 59c
:6 59d :7 59e :3 59a :5 59f
:5 5a1 :6 5a2 :5 5a3 5a2
:5 5a5 5a4 :3 5a2 5a0
5a8 :5 5a9 :3 5a8 :2 5aa
:5 5ac :11 5ad :24 5ae :3 5ad
:2 5ab :6 5aa 5a7 :6 59f
590 :3 455 :b 5b4 5b5
5b6 :3 41b 5b7 41b
:3 419 411 :3 1ef :12 5bc
:9 5bd :4 5bf :6 5c0 :5 5c1
5c0 :5 5c3 5c2 :3 5c0
5be 5c6 :5 5c7 :3 5c6
:2 5c8 :5 5ca :11 5cb :24 5cc
:3 5cb :2 5c9 :6 5c8 5c5
:4 5bd :5 5d2 :2 5d3 5d2
:6 5d4 :5 5d5 5d4 :5 5d7
5d6 :3 5d4 5d1 5da
:5 5db :3 5da :2 5dc :5 5de
:11 5df :24 5e0 :3 5df :2 5dd
:6 5dc 5d9 :4 5d0 :3 5bd
:5 5e6 :11 5e7 :30 5e8 :3 5e7
:11 5ea :b 5eb :3 5ea :a 5ed
5ee 5e6 :7 5ee :11 5ef
:30 5f0 :3 5ef :11 5f2 :b 5f3
:3 5f2 :7 5f5 5f6 :2 5f5
5f7 5ee 5e6 :7 5f7
:11 5f8 :30 5f9 :3 5f8 :11 5fb
:b 5fc :3 5fb :7 5fe 5ff
:2 5fe 5f7 5e6 :11 601
:30 602 :3 601 :11 604 :b 605
:3 604 :a 607 600 :3 5e6
:3 5bc :1c 60b :5 60d :5 60e
:2 610 :2 611 :2 610 :2 612
:2 610 :2 613 :2 610 :2 614
:2 610 :2 615 :2 610 :2 615
:2 610 :2 615 :2 610 :2 616
:2 610 :2 616 :2 610 :2 616
:2 610 :2 616 :2 610 :2 617
:3 610 :5 61a :6 61b :5 61c
61b :5 61e 61d :3 61b
619 621 :5 622 :3 621
:2 623 :5 625 :11 626 :24 627
:b 628 :3 626 :2 624 :6 623
620 :3 60b :5 62e :5 62f
631 632 :2 633 :2 632
:2 634 :2 632 :2 635 :2 632
:2 636 :2 632 :2 637 :2 632
:2 637 :2 632 :2 637 :2 632
:2 638 :2 632 :2 638 :2 632
:2 638 :2 632 :2 638 :2 632
:2 639 :2 632 631 :5 63c
:6 63d :5 63e 63d :5 640
63f :3 63d 63b 643
:5 644 :3 643 :2 645 :5 647
:11 648 :24 649 :b 64a :3 648
:2 646 :6 645 642 :3 60b
:7 64f 651 652 :2 653
:2 652 :2 654 :2 652 :2 655
:2 652 :2 656 :2 652 :2 657
:2 652 :2 658 :2 652 :2 659
:2 652 :2 65a :2 652 :2 65b
:2 652 :2 65c :2 652 :2 65d
:2 652 :2 65e :2 652 :2 65e
:2 652 :2 65f :2 652 :2 660
:2 652 :2 660 :2 652 :2 661
:2 652 :2 662 :2 652 :2 663
:2 652 :2 664 :2 652 :2 664
:2 652 :2 665 :2 652 :2 666
:2 652 :2 667 :2 652 :2 668
:2 652 :2 669 :2 652 :2 66a
:2 652 :2 66b :2 652 :2 66c
:2 652 :2 66c :2 652 :2 66d
:2 652 :2 66e :2 652 :2 66f
:2 652 :2 670 :2 652 :2 671
:2 652 :2 671 :2 652 :2 671
:2 652 :2 672 :2 652 651
:5 675 :6 676 :5 677 676
:5 679 678 :3 676 674
67c :5 67d :3 67c :2 67e
:5 680 :11 681 :24 682 :b 683
:3 681 :2 67f :6 67e 67b
:3 60b 688 689 :2 68a
:2 689 :2 68b :2 689 :2 68c
:2 689 :9 68c :2 689 :2 68d
:2 689 :2 68e :2 689 :9 68e
:2 689 :2 68f :2 689 :2 690
:2 689 :2 691 :2 689 :2 692
:2 689 :2 693 :2 689 :2 694
:2 689 :2 695 :2 689 :2 696
:2 689 :2 697 :2 689 :3 697
:2 698 :4 697 :2 689 :2 698
:2 689 :2 699 :2 689 :9 699
:2 689 :2 69a :2 689 :2 69b
:2 689 :2 69c :2 689 :2 69d
:2 689 :2 69e :2 689 688
69f 60b :c 69f :5 6a1
:5 6a2 :2 6a4 :2 6a5 :2 6a4
:2 6a6 :2 6a4 :2 6a7 :2 6a4
:2 6a8 :2 6a4 :2 6a9 :2 6a4
:2 6a9 :2 6a4 :2 6a9 :2 6a4
:2 6aa :2 6a4 :2 6aa :2 6a4
:2 6aa :3 6a4 :5 6ad :6 6ae
:5 6af 6ae :5 6b1 6b0
:3 6ae 6ac 6b4 :5 6b5
:3 6b4 :2 6b6 :5 6b8 :11 6b9
:24 6ba :b 6bb :3 6b9 :2 6b7
:6 6b6 6b3 :3 69f :5 6c1
:5 6c2 :2 6c4 :2 6c5 :2 6c4
:2 6c6 :2 6c4 :2 6c7 :2 6c4
:2 6c8 :2 6c4 :2 6c9 :2 6c4
:2 6c9 :2 6c4 :2 6c9 :2 6c4
:2 6ca :2 6c4 :2 6ca :3 6c4
:5 6cd :6 6ce :5 6cf 6ce
:5 6d1 6d0 :3 6ce 6cc
6d4 :5 6d5 :3 6d4 :2 6d6
:5 6d8 :11 6d9 :24 6da :b 6db
:3 6d9 :2 6d7 :6 6d6 6d3
:3 69f :7 6e0 6e1 6e2
:2 6e3 :2 6e2 :2 6e4 :2 6e2
:2 6e5 :2 6e2 :2 6e6 :2 6e2
:2 6e7 :2 6e2 :2 6e8 :2 6e2
:2 6e9 :2 6e2 :2 6ea :2 6e2
:2 6eb :2 6e2 :2 6eb :2 6e2
:2 6ec :2 6e2 :2 6ed :2 6e2
:2 6ed :2 6e2 :2 6ee :2 6e2
:2 6ef :2 6e2 :2 6f0 :2 6e2
:2 6f0 :2 6e2 :2 6f0 :2 6e2
6e1 :5 6f3 :6 6f4 :5 6f5
6f4 :5 6f7 6f6 :3 6f4
6f2 6fa :5 6fb :3 6fa
:2 6fc :5 6fe :11 6ff :24 700
:b 701 :3 6ff :2 6fd :6 6fc
6f9 :3 69f 706 707
:2 708 :2 707 :2 709 :2 707
:2 70a :2 707 :9 70a :2 707
:2 70b :2 707 :9 70c :2 707
:2 70d :2 707 :2 70e :2 707
:2 70f :2 707 :2 710 :2 707
:2 711 :2 707 706 69f
60b :5 714 :5 715 :2 717
:2 718 :2 717 :2 719 :2 717
:2 71a :2 717 :2 71b :2 717
:2 71c :2 717 :2 71d :2 717
:2 71d :2 717 :2 71d :2 717
:2 71e :2 717 :2 71e :2 717
:2 71e :2 717 :2 71f :3 717
:5 722 :6 723 :5 724 723
:5 726 725 :3 723 721
729 :5 72a :3 729 :2 72b
:5 72d :11 72e :24 72f :b 730
:3 72e :2 72c :6 72b 728
:3 712 :5 736 :5 737 :2 739
:2 73a :2 739 :2 73b :2 739
:2 73c :2 739 :2 73d :2 739
:2 73e :2 739 :2 73e :2 739
:2 73e :2 739 :2 73f :2 739
:2 73f :3 739 :5 742 :6 743
:5 744 743 :5 746 745
:3 743 741 749 :5 74a
:3 749 :2 74b :5 74d :11 74e
:24 74f :b 750 :3 74e :2 74c
:6 74b 748 :3 712 :7 755
757 758 :2 759 :2 758
:2 75a :2 758 :2 75b :2 758
:2 75c :2 758 :2 75d :2 758
:2 75e :2 758 :2 75f :2 758
:2 760 :2 758 :2 761 :2 758
:2 761 :2 758 :2 762 :2 758
:2 763 :2 758 :2 764 :2 758
:2 765 :2 758 :2 765 :2 758
:2 765 :2 758 757 :5 768
:6 769 :5 76a 769 :5 76c
76b :3 769 767 76f
:5 770 :3 76f :2 771 :5 773
:11 774 :24 775 :b 776 :3 774
:2 772 :6 771 76e :3 712
:2 77b :2 77c :2 77b :2 77d
:2 77b :2 77e :2 77b :9 77f
:2 77b :2 77f :2 77b :2 780
:2 77b :3 780 :2 781 :4 780
:2 77b :2 782 :2 77b :2 783
:2 77b :2 784 :2 77b :2 785
:2 77b :2 786 :3 77b 712
:3 60b :11 78a :b 78b :3 78a
:11 78d :b 78e :3 78d :8 790
59 :3 794 :11 795 :b 796
:17 797 :3 795 :2 799 :3 79a
:3 79b 79c 79a 799
79a :3 79d :3 794 :2 79e
:5 7a0 :11 7a1 :b 7a2 :18 7a3
:3 7a1 :2 7a5 7a6 :3 7a7
7a8 7a6 7a5 7a6
:3 7a9 :2 79f :6 79e 793
:7 1 
adad
4
:3 0 1 :3 0 2
:a 0 31eb 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 4
:3 0 5 :7 0 9
8 :3 0 4 :3 0
6 :7 0 d c
:3 0 d 7c 0
b 8 :3 0 7
:7 0 11 10 :3 0
8 :3 0 9 :7 0
15 14 :3 0 11
a2 0 f 8
:3 0 a :7 0 19
18 :3 0 8 :3 0
b :7 0 1d 1c
:3 0 15 c8 0
13 8 :3 0 c
:7 0 21 20 :3 0
8 :3 0 d :7 0
25 24 :3 0 19
ee 0 17 8
:3 0 e :7 0 29
28 :3 0 8 :3 0
f :7 0 2d 2c
:3 0 1d 114 0
1b 8 :3 0 10
:7 0 31 30 :3 0
8 :3 0 11 :7 0
35 34 :3 0 21
:2 0 1f 4 :3 0
12 :7 0 39 38
:3 0 4 :3 0 13
:7 0 3d 3c :3 0
14 :3 0 15 :3 0
16 :2 0 4 41
42 0 3f 43
0 31eb 2 44
:2 0 18 :2 0 33
4 :3 0 18 :2 0
31 47 49 :6 0
4c 4a 0 31e9
0 17 :6 0 18
:2 0 37 4 :3 0
35 4e 50 :6 0
53 51 0 31e9
0 19 :6 0 18
:2 0 3b 4 :3 0
39 55 57 :6 0
5a 58 0 31e9
0 1a :6 0 18
:2 0 3f 4 :3 0
3d 5c 5e :6 0
61 5f 0 31e9
0 1b :6 0 18
:2 0 43 4 :3 0
41 63 65 :6 0
68 66 0 31e9
0 1c :6 0 18
:2 0 47 4 :3 0
45 6a 6c :6 0
6f 6d 0 31e9
0 1d :6 0 20
:2 0 4b 4 :3 0
49 71 73 :6 0
76 74 0 31e9
0 1e :6 0 20
:2 0 4f 4 :3 0
4d 78 7a :6 0
7d 7b 0 31e9
0 1f :6 0 20
:2 0 53 4 :3 0
51 7f 81 :6 0
84 82 0 31e9
0 21 :6 0 59
276 0 57 4
:3 0 55 86 88
:6 0 8b 89 0
31e9 0 22 :6 0
27 :2 0 5b 24
:3 0 8d :7 0 90
8e 0 31e9 0
23 :6 0 24 :3 0
92 :7 0 95 93
0 31e9 0 25
:6 0 18 :2 0 5f
4 :3 0 5d 97
99 :6 0 9c 9a
0 31e9 0 26
:6 0 2a :2 0 63
4 :3 0 61 9e
a0 :6 0 a3 a1
0 31e9 0 28
:6 0 2a :2 0 67
4 :3 0 65 a5
a7 :6 0 aa a8
0 31e9 0 29
:6 0 2a :2 0 6b
4 :3 0 69 ac
ae :6 0 b1 af
0 31e9 0 2b
:6 0 2e :2 0 6f
4 :3 0 6d b3
b5 :6 0 b8 b6
0 31e9 0 2c
:6 0 30 :2 0 73
4 :3 0 71 ba
bc :6 0 bf bd
0 31e9 0 2d
:6 0 30 :2 0 75
8 :3 0 c1 :7 0
c5 c2 c3 31e9
0 2f :6 0 30
:2 0 77 8 :3 0
c7 :7 0 cb c8
c9 31e9 0 31
:6 0 30 :2 0 79
8 :3 0 cd :7 0
d1 ce cf 31e9
0 32 :6 0 30
:2 0 7b 8 :3 0
d3 :7 0 d7 d4
d5 31e9 0 33
:6 0 30 :2 0 7d
8 :3 0 d9 :7 0
dd da db 31e9
0 34 :6 0 30
:2 0 7f 8 :3 0
df :7 0 e3 e0
e1 31e9 0 35
:6 0 30 :2 0 81
8 :3 0 e5 :7 0
e9 e6 e7 31e9
0 36 :6 0 30
:2 0 83 8 :3 0
eb :7 0 ef ec
ed 31e9 0 37
:6 0 30 :2 0 85
8 :3 0 f1 :7 0
f5 f2 f3 31e9
0 38 :6 0 30
:2 0 87 8 :3 0
f7 :7 0 fb f8
f9 31e9 0 39
:6 0 30 :2 0 89
8 :3 0 fd :7 0
101 fe ff 31e9
0 3a :6 0 30
:2 0 8b 8 :3 0
103 :7 0 107 104
105 31e9 0 3b
:6 0 30 :2 0 8d
8 :3 0 109 :7 0
10d 10a 10b 31e9
0 3c :6 0 30
:2 0 8f 8 :3 0
10f :7 0 113 110
111 31e9 0 3d
:6 0 30 :2 0 91
8 :3 0 115 :7 0
119 116 117 31e9
0 3e :6 0 30
:2 0 93 8 :3 0
11b :7 0 11f 11c
11d 31e9 0 3f
:6 0 30 :2 0 95
8 :3 0 121 :7 0
125 122 123 31e9
0 40 :6 0 30
:2 0 97 8 :3 0
127 :7 0 12b 128
129 31e9 0 41
:6 0 30 :2 0 99
8 :3 0 12d :7 0
131 12e 12f 31e9
0 42 :6 0 30
:2 0 9b 8 :3 0
133 :7 0 137 134
135 31e9 0 43
:6 0 30 :2 0 9d
8 :3 0 139 :7 0
13d 13a 13b 31e9
0 44 :6 0 30
:2 0 9f 8 :3 0
13f :7 0 143 140
141 31e9 0 45
:6 0 30 :2 0 a1
8 :3 0 145 :7 0
149 146 147 31e9
0 46 :6 0 30
:2 0 a3 8 :3 0
14b :7 0 14f 14c
14d 31e9 0 47
:6 0 30 :2 0 a5
8 :3 0 151 :7 0
155 152 153 31e9
0 48 :6 0 30
:2 0 a7 8 :3 0
157 :7 0 15b 158
159 31e9 0 49
:6 0 30 :2 0 a9
8 :3 0 15d :7 0
161 15e 15f 31e9
0 4a :6 0 27
:2 0 ab 8 :3 0
163 :7 0 167 164
165 31e9 0 4b
:6 0 27 :2 0 af
4 :3 0 ad 169
16b :6 0 16e 16c
0 31e9 0 4c
:6 0 50 :2 0 b3
4 :3 0 b1 170
172 :6 0 175 173
0 31e9 0 4d
:6 0 52 :2 0 b5
4f :3 0 177 :7 0
17b 178 179 31e9
0 4e :6 0 30
:2 0 b9 4 :3 0
b7 17d 17f :6 0
182 180 0 31e9
0 51 :6 0 18
:2 0 bb 8 :3 0
184 :7 0 188 185
186 31e9 0 53
:6 0 18 :2 0 bf
55 :3 0 bd 18a
18c :6 0 18f 18d
0 31e9 0 54
:6 0 18 :2 0 c3
55 :3 0 c1 191
193 :6 0 196 194
0 31e9 0 56
:6 0 30 :2 0 c7
4 :3 0 c5 198
19a :6 0 19d 19b
0 31e9 0 57
:6 0 30 :2 0 c9
8 :3 0 19f :7 0
1a3 1a0 1a1 31e9
0 58 :6 0 30
:2 0 cb 8 :3 0
1a5 :7 0 1a9 1a6
1a7 31e9 0 59
:6 0 20 :2 0 cd
8 :3 0 1ab :7 0
1af 1ac 1ad 31e9
0 5a :6 0 20
:2 0 d1 4 :3 0
cf 1b1 1b3 :6 0
1b6 1b4 0 31e9
0 5b :6 0 30
:2 0 d5 4 :3 0
d3 1b8 1ba :6 0
1bd 1bb 0 31e9
0 5c :6 0 30
:2 0 d7 8 :3 0
1bf :7 0 1c3 1c0
1c1 31e9 0 5d
:6 0 18 :2 0 d9
8 :3 0 1c5 :7 0
1c9 1c6 1c7 31e9
0 5e :6 0 27
:2 0 dd 4 :3 0
db 1cb 1cd :6 0
1d0 1ce 0 31e9
0 5f :6 0 27
:2 0 df 8 :3 0
1d2 0 2 :5 0
1d6 1d3 1d4 31e9
0 60 :6 0 30
:2 0 e1 8 :3 0
1d8 0 2 :5 0
1dc 1d9 1da 31e9
0 61 :6 0 30
:2 0 e3 8 :3 0
1de 0 2 :5 0
1e2 1df 1e0 31e9
0 62 :6 0 e7
8a1 0 e5 8
:3 0 1e4 0 2
:5 0 1e8 1e5 1e6
31e9 0 63 :6 0
1f5 1f6 0 e9
8 :3 0 1ea :7 0
1ed 1eb 0 31e9
0 64 :6 0 15
:3 0 16 :2 0 4
1ef 1f0 0 1f1
:7 0 1f4 1f2 0
31e9 0 65 :6 0
66 :3 0 67 :3 0
66 :3 0 68 :3 0
1f8 1f9 0 2
:4 0 eb 1fa 1fc
1f7 1fe 1fd :2 0
69 :3 0 6a :3 0
200 201 0 6b
:4 0 66 :3 0 6c
:3 0 204 205 0
2 :4 0 ed 206
208 6d :4 0 6e
:2 0 6f :3 0 ef
20b 20d :3 0 70
:4 0 6e :2 0 71
:4 0 f2 210 212
:3 0 6e :2 0 3
:3 0 f5 214 216
:3 0 6e :2 0 71
:4 0 f8 218 21a
:3 0 6e :2 0 72
:4 0 fb 21c 21e
:3 0 6e :2 0 71
:4 0 fe 220 222
:3 0 6e :2 0 5
:3 0 101 224 226
:3 0 6e :2 0 71
:4 0 104 228 22a
:3 0 6e :2 0 72
:4 0 107 22c 22e
:3 0 6e :2 0 71
:4 0 10a 230 232
:3 0 6e :2 0 6
:3 0 10d 234 236
:3 0 6e :2 0 71
:4 0 110 238 23a
:3 0 6e :2 0 72
:4 0 113 23c 23e
:3 0 6e :2 0 73
:3 0 74 :3 0 7
:3 0 116 242 244
75 :4 0 118 241
247 11b 240 249
:3 0 6e :2 0 72
:4 0 11e 24b 24d
:3 0 6e :2 0 73
:3 0 74 :3 0 9
:3 0 121 251 253
75 :4 0 123 250
256 126 24f 258
:3 0 6e :2 0 72
:4 0 129 25a 25c
:3 0 6e :2 0 73
:3 0 74 :3 0 a
:3 0 12c 260 262
75 :4 0 12e 25f
265 131 25e 267
:3 0 6e :2 0 72
:4 0 134 269 26b
:3 0 6e :2 0 73
:3 0 74 :3 0 b
:3 0 137 26f 271
75 :4 0 139 26e
274 13c 26d 276
:3 0 6e :2 0 72
:4 0 13f 278 27a
:3 0 6e :2 0 73
:3 0 74 :3 0 c
:3 0 142 27e 280
75 :4 0 144 27d
283 147 27c 285
:3 0 6e :2 0 72
:4 0 14a 287 289
:3 0 6e :2 0 73
:3 0 74 :3 0 d
:3 0 14d 28d 28f
75 :4 0 14f 28c
292 152 28b 294
:3 0 6e :2 0 72
:4 0 155 296 298
:3 0 6e :2 0 73
:3 0 74 :3 0 e
:3 0 158 29c 29e
75 :4 0 15a 29b
2a1 15d 29a 2a3
:3 0 6e :2 0 72
:4 0 160 2a5 2a7
:3 0 6e :2 0 73
:3 0 74 :3 0 f
:3 0 163 2ab 2ad
75 :4 0 165 2aa
2b0 168 2a9 2b2
:3 0 6e :2 0 72
:4 0 16b 2b4 2b6
:3 0 6e :2 0 73
:3 0 74 :3 0 10
:3 0 16e 2ba 2bc
75 :4 0 170 2b9
2bf 173 2b8 2c1
:3 0 6e :2 0 72
:4 0 176 2c3 2c5
:3 0 6e :2 0 73
:3 0 74 :3 0 11
:3 0 179 2c9 2cb
75 :4 0 17b 2c8
2ce 17e 2c7 2d0
:3 0 6e :2 0 72
:4 0 181 2d2 2d4
:3 0 6e :2 0 71
:4 0 184 2d6 2d8
:3 0 6e :2 0 12
:3 0 187 2da 2dc
:3 0 6e :2 0 71
:4 0 18a 2de 2e0
:3 0 6e :2 0 72
:4 0 18d 2e2 2e4
:3 0 6e :2 0 71
:4 0 190 2e6 2e8
:3 0 6e :2 0 13
:3 0 193 2ea 2ec
:3 0 6e :2 0 71
:4 0 196 2ee 2f0
:3 0 6e :2 0 76
:4 0 199 2f2 2f4
:3 0 19c 202 2f6
:2 0 2f8 1a1 2f9
1ff 2f8 0 2fa
1a3 0 3149 66
:3 0 67 :3 0 2fb
2fc 0 66 :3 0
77 :3 0 2fe 2ff
0 27 :2 0 2
:4 0 78 :2 0 79
:2 0 27 :2 0 1a5
304 306 :3 0 1a8
300 308 2fd 30a
309 :2 0 69 :3 0
7a :3 0 30c 30d
0 6b :4 0 7b
:4 0 6e :2 0 6f
:3 0 1ac 311 313
:3 0 1af 30e 315
:2 0 317 1b2 318
30b 317 0 319
1b4 0 3149 17
:3 0 3 :3 0 31a
31b 0 3149 1f
:3 0 5 :3 0 31d
31e 0 3149 22
:3 0 6 :3 0 320
321 0 3149 1c
:3 0 13 :3 0 323
324 0 3149 37
:3 0 7 :3 0 326
327 0 3149 35
:3 0 9 :3 0 329
32a 0 3149 36
:3 0 e :3 0 32c
32d 0 3149 38
:3 0 b :3 0 32f
330 0 3149 3a
:3 0 f :3 0 332
333 0 3149 39
:3 0 c :3 0 335
336 0 3149 32
:3 0 11 :3 0 338
339 0 3149 31
:3 0 a :3 0 33b
33c 0 3149 3b
:3 0 d :3 0 33e
33f 0 3149 33
:3 0 10 :3 0 341
342 0 3149 1a
:3 0 17 :3 0 344
345 0 3149 23
:3 0 7c :4 0 347
348 0 3149 19
:4 0 34a 34b 0
3149 5a :3 0 7d
:3 0 73 :3 0 32
:3 0 30 :2 0 1b6
34f 352 1b9 34e
354 34d 355 0
3149 5d :3 0 30
:2 0 357 358 0
3149 4e :3 0 50
:2 0 35a 35b 0
3149 3f :3 0 30
:2 0 35d 35e 0
3149 3 :3 0 7e
:2 0 1bb 361 362
:3 0 363 :2 0 66
:3 0 67 :3 0 365
366 0 66 :3 0
77 :3 0 368 369
0 7f :2 0 2
:4 0 80 :2 0 79
:2 0 27 :2 0 1bd
36e 370 :3 0 1c0
36a 372 367 374
373 :2 0 69 :3 0
7a :3 0 376 377
0 6b :4 0 81
:4 0 6e :2 0 6f
:3 0 1c4 37b 37d
:3 0 6e :2 0 82
:4 0 1c7 37f 381
:3 0 6e :2 0 83
:4 0 1ca 383 385
:3 0 6e :2 0 73
:3 0 74 :3 0 84
:3 0 85 :3 0 38a
38b 0 1cd 389
38d 75 :4 0 1cf
388 390 1d2 387
392 :3 0 1d5 378
394 :2 0 396 1d8
397 375 396 0
398 1da 0 3c1
66 :3 0 67 :3 0
399 39a 0 66
:3 0 77 :3 0 39c
39d 0 27 :2 0
2 :4 0 86 :2 0
79 :2 0 27 :2 0
1dc 3a2 3a4 :3 0
1df 39e 3a6 39b
3a8 3a7 :2 0 69
:3 0 7a :3 0 3aa
3ab 0 6b :4 0
87 :4 0 6e :2 0
6f :3 0 1e3 3af
3b1 :3 0 1e6 3ac
3b3 :2 0 3b5 1e9
3b6 3a9 3b5 0
3b7 1eb 0 3c1
88 :3 0 89 :3 0
3b8 3b9 0 84
:3 0 85 :3 0 3bb
3bc 0 8a :3 0
1ed 3ba 3bf :2 0
3c1 1f0 3c2 364
3c1 0 3c3 1f4
0 3149 12 :3 0
7e :2 0 1f6 3c5
3c6 :3 0 3c7 :2 0
66 :3 0 67 :3 0
3c9 3ca 0 66
:3 0 77 :3 0 3cc
3cd 0 7f :2 0
2 :4 0 8b :2 0
79 :2 0 27 :2 0
1f8 3d2 3d4 :3 0
1fb 3ce 3d6 3cb
3d8 3d7 :2 0 69
:3 0 7a :3 0 3da
3db 0 6b :4 0
8c :4 0 6e :2 0
6f :3 0 1ff 3df
3e1 :3 0 6e :2 0
82 :4 0 202 3e3
3e5 :3 0 6e :2 0
83 :4 0 205 3e7
3e9 :3 0 6e :2 0
73 :3 0 74 :3 0
84 :3 0 85 :3 0
3ee 3ef 0 208
3ed 3f1 75 :4 0
20a 3ec 3f4 20d
3eb 3f6 :3 0 210
3dc 3f8 :2 0 3fa
213 3fb 3d9 3fa
0 3fc 215 0
425 66 :3 0 67
:3 0 3fd 3fe 0
66 :3 0 77 :3 0
400 401 0 27
:2 0 2 :4 0 8d
:2 0 79 :2 0 27
:2 0 217 406 408
:3 0 21a 402 40a
3ff 40c 40b :2 0
69 :3 0 7a :3 0
40e 40f 0 6b
:4 0 8e :4 0 6e
:2 0 6f :3 0 21e
413 415 :3 0 221
410 417 :2 0 419
224 41a 40d 419
0 41b 226 0
425 88 :3 0 89
:3 0 41c 41d 0
84 :3 0 85 :3 0
41f 420 0 8a
:3 0 228 41e 423
:2 0 425 22b 426
3c8 425 0 427
22f 0 3149 73
:3 0 31 :3 0 30
:2 0 231 428 42b
8f :2 0 90 :2 0
236 42d 42f :3 0
73 :3 0 39 :3 0
30 :2 0 239 431
434 8f :2 0 90
:2 0 23e 436 438
:3 0 430 43a 439
:2 0 73 :3 0 35
:3 0 30 :2 0 241
43c 43f 8f :2 0
90 :2 0 246 441
443 :3 0 43b 445
444 :2 0 66 :3 0
67 :3 0 447 448
0 66 :3 0 77
:3 0 44a 44b 0
7f :2 0 2 :4 0
91 :2 0 79 :2 0
27 :2 0 249 450
452 :3 0 24c 44c
454 449 456 455
:2 0 69 :3 0 7a
:3 0 458 459 0
6b :4 0 92 :4 0
6e :2 0 6f :3 0
250 45d 45f :3 0
6e :2 0 82 :4 0
253 461 463 :3 0
6e :2 0 83 :4 0
256 465 467 :3 0
6e :2 0 73 :3 0
74 :3 0 84 :3 0
93 :3 0 46c 46d
0 259 46b 46f
75 :4 0 25b 46a
472 25e 469 474
:3 0 261 45a 476
:2 0 478 264 479
457 478 0 47a
266 0 4a3 66
:3 0 67 :3 0 47b
47c 0 66 :3 0
77 :3 0 47e 47f
0 27 :2 0 2
:4 0 94 :2 0 79
:2 0 27 :2 0 268
484 486 :3 0 26b
480 488 47d 48a
489 :2 0 69 :3 0
7a :3 0 48c 48d
0 6b :4 0 95
:4 0 6e :2 0 6f
:3 0 26f 491 493
:3 0 272 48e 495
:2 0 497 275 498
48b 497 0 499
277 0 4a3 88
:3 0 89 :3 0 49a
49b 0 84 :3 0
93 :3 0 49d 49e
0 8a :3 0 279
49c 4a1 :2 0 4a3
27c 4a4 446 4a3
0 4a5 280 0
3149 5 :3 0 96
:2 0 282 4a7 4a8
:3 0 4a9 :2 0 4aa
:2 0 6 :3 0 96
:2 0 284 4ad 4ae
:3 0 4af :2 0 4b0
:2 0 4ab 4b2 4b1
:2 0 97 :3 0 98
:3 0 6 :3 0 97
:3 0 98 :3 0 5
:3 0 44 :3 0 99
:4 0 9a 1 :8 0
4c2 15 :3 0 9b
:3 0 4bd 4be 0
9c :3 0 4bf 4c0
0 4c2 286 515
9d :3 0 15 :3 0
9b :3 0 4c4 4c5
0 9c :3 0 4c6
4c7 0 4c9 289
4cb 28b 4ca 4c9
:2 0 513 9e :3 0
15 :3 0 9b :3 0
4ce 4cf 0 9c
:3 0 4d0 4d1 0
50b 66 :3 0 67
:3 0 4d3 4d4 0
66 :3 0 77 :3 0
4d6 4d7 0 9f
:2 0 2 :4 0 a0
:2 0 79 :2 0 27
:2 0 28d 4dc 4de
:3 0 290 4d8 4e0
4d5 4e2 4e1 :2 0
69 :3 0 7a :3 0
4e4 4e5 0 6b
:4 0 a1 :4 0 6e
:2 0 6f :3 0 294
4e9 4eb :3 0 6e
:2 0 82 :4 0 297
4ed 4ef :3 0 6e
:2 0 a2 :4 0 29a
4f1 4f3 :3 0 6e
:2 0 9c :3 0 29d
4f5 4f7 :3 0 6e
:2 0 a3 :4 0 2a0
4f9 4fb :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 2a3 4fe 502
2a7 4fd 504 :3 0
2aa 4e6 506 :2 0
508 2ad 509 4e3
508 0 50a 2af
0 50b 2b1 50e
:3 0 50e 0 50e
50d 50b 50c :6 0
510 2 :3 0 2b4
512 2b6 511 510
:2 0 513 2b8 :2 0
515 0 515 514
4c2 513 :6 0 590
1 :3 0 15 :3 0
9b :3 0 517 518
0 15 :3 0 8f
:2 0 a7 :3 0 51a
51c 0 2bd 51b
51e :3 0 51f :2 0
73 :3 0 44 :3 0
30 :2 0 2c0 521
524 a8 :2 0 30
:2 0 2c5 526 528
:3 0 529 :2 0 520
52b 52a :2 0 66
:3 0 67 :3 0 52d
52e 0 66 :3 0
77 :3 0 530 531
0 7f :2 0 2
:4 0 a9 :2 0 79
:2 0 27 :2 0 2c8
536 538 :3 0 2cb
532 53a 52f 53c
53b :2 0 69 :3 0
7a :3 0 53e 53f
0 6b :4 0 aa
:4 0 6e :2 0 6f
:3 0 2cf 543 545
:3 0 6e :2 0 82
:4 0 2d2 547 549
:3 0 6e :2 0 83
:4 0 2d5 54b 54d
:3 0 6e :2 0 73
:3 0 74 :3 0 84
:3 0 ab :3 0 552
553 0 2d8 551
555 75 :4 0 2da
550 558 2dd 54f
55a :3 0 2e0 540
55c :2 0 55e 2e3
55f 53d 55e 0
560 2e5 0 589
66 :3 0 67 :3 0
561 562 0 66
:3 0 77 :3 0 564
565 0 27 :2 0
2 :4 0 ac :2 0
79 :2 0 27 :2 0
2e7 56a 56c :3 0
2ea 566 56e 563
570 56f :2 0 69
:3 0 7a :3 0 572
573 0 6b :4 0
ad :4 0 6e :2 0
6f :3 0 2ee 577
579 :3 0 2f1 574
57b :2 0 57d 2f4
57e 571 57d 0
57f 2f6 0 589
88 :3 0 89 :3 0
580 581 0 84
:3 0 ab :3 0 583
584 0 8a :3 0
2f8 582 587 :2 0
589 2fb 58a 52c
589 0 58b 2ff
0 590 63 :3 0
27 :2 0 58c 58d
0 590 ae :3 0
301 5ff 5 :3 0
7e :2 0 305 592
593 :3 0 594 :2 0
595 :2 0 6 :3 0
7e :2 0 307 598
599 :3 0 59a :2 0
59b :2 0 596 59d
59c :2 0 6f :3 0
1f :3 0 99 :4 0
af 1 :8 0 5a8
15 :3 0 9b :3 0
5a3 5a4 0 9c
:3 0 5a5 5a6 0
5a8 309 5fb 9d
:3 0 15 :3 0 9b
:3 0 5aa 5ab 0
9c :3 0 5ac 5ad
0 5af 30c 5b1
30e 5b0 5af :2 0
5f9 9e :3 0 15
:3 0 9b :3 0 5b4
5b5 0 9c :3 0
5b6 5b7 0 5f1
66 :3 0 67 :3 0
5b9 5ba 0 66
:3 0 77 :3 0 5bc
5bd 0 9f :2 0
2 :4 0 b0 :2 0
79 :2 0 27 :2 0
310 5c2 5c4 :3 0
313 5be 5c6 5bb
5c8 5c7 :2 0 69
:3 0 7a :3 0 5ca
5cb 0 6b :4 0
b1 :4 0 6e :2 0
6f :3 0 317 5cf
5d1 :3 0 6e :2 0
82 :4 0 31a 5d3
5d5 :3 0 6e :2 0
a2 :4 0 31d 5d7
5d9 :3 0 6e :2 0
9c :3 0 320 5db
5dd :3 0 6e :2 0
a3 :4 0 323 5df
5e1 :3 0 6e :2 0
a4 :3 0 a5 :3 0
27 :2 0 a6 :2 0
326 5e4 5e8 32a
5e3 5ea :3 0 32d
5cc 5ec :2 0 5ee
330 5ef 5c9 5ee
0 5f0 332 0
5f1 334 5f4 :3 0
5f4 0 5f4 5f3
5f1 5f2 :6 0 5f6
4 :3 0 337 5f8
339 5f7 5f6 :2 0
5f9 33b :2 0 5fb
0 5fb 5fa 5a8
5f9 :6 0 5fd 1
:3 0 33e 5fe 59e
5fd 0 600 4b3
590 0 600 340
0 3149 21 :3 0
1f :3 0 601 602
0 3149 6 :3 0
7e :2 0 343 605
606 :3 0 607 :2 0
22 :3 0 7c :4 0
609 60a 0 60c
345 60d 608 60c
0 60e 347 0
3149 1d :3 0 a4
:3 0 1d :3 0 27
:2 0 18 :2 0 349
610 614 60f 615
0 3149 26 :3 0
a4 :3 0 26 :3 0
27 :2 0 27 :2 0
34d 618 61c 617
61d 0 3149 29
:3 0 12 :3 0 61f
620 0 3149 73
:3 0 32 :3 0 30
:2 0 351 622 625
b2 :2 0 30 :2 0
356 627 629 :3 0
1d :3 0 a4 :3 0
a4 :3 0 29 :3 0
27 :2 0 18 :2 0
359 62d 631 27
:2 0 18 :2 0 35d
62c 635 62b 636
0 a07 26 :3 0
b3 :3 0 b4 :3 0
639 63a 0 27
:2 0 27 :2 0 b5
:2 0 29 :3 0 361
63b 640 638 641
0 a07 b6 :3 0
73 :3 0 32 :3 0
30 :2 0 366 644
647 369 643 649
b7 :2 0 30 :2 0
36d 64b 64d :3 0
b8 :3 0 73 :3 0
b9 :3 0 73 :3 0
ba :3 0 bb :3 0
17 :3 0 3d :3 0
3e :3 0 43 :3 0
bc :3 0 bd :3 0
1a :4 0 be 1
:8 0 662 15 :3 0
9b :3 0 65d 65e
0 9c :3 0 65f
660 0 662 370
6b5 9d :3 0 15
:3 0 9b :3 0 664
665 0 9c :3 0
666 667 0 669
373 66b 375 66a
669 :2 0 6b3 9e
:3 0 15 :3 0 9b
:3 0 66e 66f 0
9c :3 0 670 671
0 6ab 66 :3 0
67 :3 0 673 674
0 66 :3 0 77
:3 0 676 677 0
9f :2 0 2 :4 0
bf :2 0 79 :2 0
27 :2 0 377 67c
67e :3 0 37a 678
680 675 682 681
:2 0 69 :3 0 7a
:3 0 684 685 0
6b :4 0 c0 :4 0
6e :2 0 6f :3 0
37e 689 68b :3 0
6e :2 0 82 :4 0
381 68d 68f :3 0
6e :2 0 a2 :4 0
384 691 693 :3 0
6e :2 0 9c :3 0
387 695 697 :3 0
6e :2 0 a3 :4 0
38a 699 69b :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 38d 69e
6a2 391 69d 6a4
:3 0 394 686 6a6
:2 0 6a8 397 6a9
683 6a8 0 6aa
399 0 6ab 39b
6ae :3 0 6ae 0
6ae 6ad 6ab 6ac
:6 0 6b0 6 :3 0
39e 6b2 3a0 6b1
6b0 :2 0 6b3 3a2
:2 0 6b5 0 6b5
6b4 662 6b3 :6 0
6b7 1 :3 0 3a5
759 c1 :3 0 c2
:3 0 c1 :3 0 c3
:3 0 c1 :3 0 c4
:3 0 c1 :3 0 c5
:3 0 c1 :3 0 b8
:3 0 73 :3 0 c6
:3 0 b9 :3 0 73
:3 0 c6 :3 0 ba
:3 0 c6 :3 0 bb
:3 0 32 :3 0 5d
:3 0 5b :3 0 5c
:3 0 17 :3 0 3d
:3 0 3e :3 0 43
:3 0 bc :3 0 c6
:3 0 bc :3 0 c1
:3 0 c1 :3 0 bd
:3 0 1a :3 0 c1
:3 0 c2 :3 0 84
:3 0 c7 :3 0 84
:3 0 c8 :3 0 84
:3 0 c9 :3 0 84
:3 0 ca :3 0 84
:3 0 cb :3 0 84
:3 0 cc :3 0 84
:3 0 cd :3 0 84
:3 0 ce :3 0 84
:3 0 cf :3 0 84
:3 0 d0 :3 0 84
:3 0 d1 :3 0 84
:3 0 d2 :3 0 84
:3 0 d3 :3 0 84
:3 0 d4 :3 0 84
:3 0 d5 :3 0 c6
:3 0 bd :3 0 c1
:3 0 b8 :4 0 d6
1 :8 0 703 15
:3 0 9b :3 0 6fe
6ff 0 9c :3 0
700 701 0 703
3a7 756 9d :3 0
15 :3 0 9b :3 0
705 706 0 9c
:3 0 707 708 0
70a 3aa 70c 3ac
70b 70a :2 0 754
9e :3 0 15 :3 0
9b :3 0 70f 710
0 9c :3 0 711
712 0 74c 66
:3 0 67 :3 0 714
715 0 66 :3 0
77 :3 0 717 718
0 9f :2 0 2
:4 0 d7 :2 0 79
:2 0 27 :2 0 3ae
71d 71f :3 0 3b1
719 721 716 723
722 :2 0 69 :3 0
7a :3 0 725 726
0 6b :4 0 d8
:4 0 6e :2 0 6f
:3 0 3b5 72a 72c
:3 0 6e :2 0 82
:4 0 3b8 72e 730
:3 0 6e :2 0 a2
:4 0 3bb 732 734
:3 0 6e :2 0 9c
:3 0 3be 736 738
:3 0 6e :2 0 a3
:4 0 3c1 73a 73c
:3 0 6e :2 0 a4
:3 0 a5 :3 0 27
:2 0 a6 :2 0 3c4
73f 743 3c8 73e
745 :3 0 3cb 727
747 :2 0 749 3ce
74a 724 749 0
74b 3d0 0 74c
3d2 74f :3 0 74f
0 74f 74e 74c
74d :6 0 751 8
:3 0 3d5 753 3d7
752 751 :2 0 754
3d9 :2 0 756 0
756 755 703 754
:6 0 758 1 :3 0
3dc 75a 64e 6b7
0 75b 0 758
0 75b 3de 0
a07 15 :3 0 9b
:3 0 75c 75d 0
15 :3 0 8f :2 0
d9 :3 0 75f 761
0 3e3 760 763
:3 0 764 :2 0 b8
:3 0 da :3 0 17
:3 0 5f :3 0 db
:3 0 bd :3 0 1a
:4 0 dc 1 :8 0
773 15 :3 0 9b
:3 0 76e 76f 0
9c :3 0 770 771
0 773 3e6 7c6
9d :3 0 15 :3 0
9b :3 0 775 776
0 9c :3 0 777
778 0 77a 3e9
77c 3eb 77b 77a
:2 0 7c4 9e :3 0
15 :3 0 9b :3 0
77f 780 0 9c
:3 0 781 782 0
7bc 66 :3 0 67
:3 0 784 785 0
66 :3 0 77 :3 0
787 788 0 9f
:2 0 2 :4 0 dd
:2 0 79 :2 0 27
:2 0 3ed 78d 78f
:3 0 3f0 789 791
786 793 792 :2 0
69 :3 0 7a :3 0
795 796 0 6b
:4 0 de :4 0 6e
:2 0 6f :3 0 3f4
79a 79c :3 0 6e
:2 0 82 :4 0 3f7
79e 7a0 :3 0 6e
:2 0 a2 :4 0 3fa
7a2 7a4 :3 0 6e
:2 0 9c :3 0 3fd
7a6 7a8 :3 0 6e
:2 0 a3 :4 0 400
7aa 7ac :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 403 7af 7b3
407 7ae 7b5 :3 0
40a 797 7b7 :2 0
7b9 40d 7ba 794
7b9 0 7bb 40f
0 7bc 411 7bf
:3 0 7bf 0 7bf
7be 7bc 7bd :6 0
7c1 a :3 0 414
7c3 416 7c2 7c1
:2 0 7c4 418 :2 0
7c6 0 7c6 7c5
773 7c4 :6 0 8b5
1 :3 0 15 :3 0
9b :3 0 7c8 7c9
0 15 :3 0 8f
:2 0 a7 :3 0 7cb
7cd 0 41d 7cc
7cf :3 0 7d0 :2 0
5f :3 0 96 :2 0
420 7d3 7d4 :3 0
7d5 :2 0 5a :3 0
27 :2 0 7d7 7d8
0 7da 422 7db
7d6 7da 0 7dc
424 0 843 73
:3 0 b9 :3 0 73
:3 0 ba :3 0 bb
:3 0 3d :3 0 3e
:3 0 43 :3 0 bc
:3 0 bd :3 0 17
:4 0 df 1 :8 0
7ee 15 :3 0 9b
:3 0 7e9 7ea 0
9c :3 0 7eb 7ec
0 7ee 426 841
9d :3 0 15 :3 0
9b :3 0 7f0 7f1
0 9c :3 0 7f2
7f3 0 7f5 429
7f7 42b 7f6 7f5
:2 0 83f 9e :3 0
15 :3 0 9b :3 0
7fa 7fb 0 9c
:3 0 7fc 7fd 0
837 66 :3 0 67
:3 0 7ff 800 0
66 :3 0 77 :3 0
802 803 0 9f
:2 0 2 :4 0 e0
:2 0 79 :2 0 27
:2 0 42d 808 80a
:3 0 430 804 80c
801 80e 80d :2 0
69 :3 0 7a :3 0
810 811 0 6b
:4 0 e1 :4 0 6e
:2 0 6f :3 0 434
815 817 :3 0 6e
:2 0 82 :4 0 437
819 81b :3 0 6e
:2 0 a2 :4 0 43a
81d 81f :3 0 6e
:2 0 9c :3 0 43d
821 823 :3 0 6e
:2 0 a3 :4 0 440
825 827 :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 443 82a 82e
447 829 830 :3 0
44a 812 832 :2 0
834 44d 835 80f
834 0 836 44f
0 837 451 83a
:3 0 83a 0 83a
839 837 838 :6 0
83c c :3 0 454
83e 456 83d 83c
:2 0 83f 458 :2 0
841 0 841 840
7ee 83f :6 0 843
1 :3 0 45b 8b2
66 :3 0 67 :3 0
844 845 0 66
:3 0 77 :3 0 847
848 0 18 :2 0
2 :4 0 e2 :2 0
79 :2 0 27 :2 0
45e 84d 84f :3 0
461 849 851 846
853 852 :2 0 69
:3 0 7a :3 0 855
856 0 6b :4 0
e3 :4 0 6e :2 0
6f :3 0 465 85a
85c :3 0 6e :2 0
82 :4 0 468 85e
860 :3 0 6e :2 0
83 :4 0 46b 862
864 :3 0 6e :2 0
73 :3 0 74 :3 0
84 :3 0 e4 :3 0
869 86a 0 46e
868 86c 75 :4 0
470 867 86f 473
866 871 :3 0 6e
:2 0 e5 :4 0 476
873 875 :3 0 6e
:2 0 71 :4 0 479
877 879 :3 0 6e
:2 0 1a :3 0 47c
87b 87d :3 0 6e
:2 0 71 :4 0 47f
87f 881 :3 0 482
857 883 :2 0 885
485 886 854 885
0 887 487 0
8b1 66 :3 0 67
:3 0 888 889 0
66 :3 0 77 :3 0
88b 88c 0 27
:2 0 2 :4 0 e6
:2 0 79 :2 0 27
:2 0 489 891 893
:3 0 48c 88d 895
88a 897 896 :2 0
69 :3 0 7a :3 0
899 89a 0 6b
:4 0 e7 :4 0 6e
:2 0 6f :3 0 490
89e 8a0 :3 0 493
89b 8a2 :2 0 8a4
496 8a5 898 8a4
0 8a6 498 0
8b1 88 :3 0 e8
:3 0 8a7 8a8 0
84 :3 0 e4 :3 0
8aa 8ab 0 1a
:3 0 8a :3 0 49a
8a9 8af :2 0 8b1
49e 8b3 7d1 843
0 8b4 0 8b1
0 8b4 4a2 0
8b5 4a5 8ba 5a
:3 0 27 :2 0 8b6
8b7 0 8b9 4a8
8bb 765 8b5 0
8bc 0 8b9 0
8bc 4aa 0 a07
5e :3 0 30 :2 0
8bd 8be 0 a07
e9 :3 0 5e :3 0
ea :3 0 b8 :3 0
17 :4 0 eb 1
:8 0 8cb 15 :3 0
9b :3 0 8c6 8c7
0 9c :3 0 8c8
8c9 0 8cb 4ad
91e 9d :3 0 15
:3 0 9b :3 0 8cd
8ce 0 9c :3 0
8cf 8d0 0 8d2
4b0 8d4 4b2 8d3
8d2 :2 0 91c 9e
:3 0 15 :3 0 9b
:3 0 8d7 8d8 0
9c :3 0 8d9 8da
0 914 66 :3 0
67 :3 0 8dc 8dd
0 66 :3 0 77
:3 0 8df 8e0 0
9f :2 0 2 :4 0
ec :2 0 79 :2 0
27 :2 0 4b4 8e5
8e7 :3 0 4b7 8e1
8e9 8de 8eb 8ea
:2 0 69 :3 0 7a
:3 0 8ed 8ee 0
6b :4 0 ed :4 0
6e :2 0 6f :3 0
4bb 8f2 8f4 :3 0
6e :2 0 82 :4 0
4be 8f6 8f8 :3 0
6e :2 0 a2 :4 0
4c1 8fa 8fc :3 0
6e :2 0 9c :3 0
4c4 8fe 900 :3 0
6e :2 0 a3 :4 0
4c7 902 904 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 4ca 907
90b 4ce 906 90d
:3 0 4d1 8ef 90f
:2 0 911 4d4 912
8ec 911 0 913
4d6 0 914 4d8
917 :3 0 917 0
917 916 914 915
:6 0 919 e :3 0
4db 91b 4dd 91a
919 :2 0 91c 4df
:2 0 91e 0 91e
91d 8cb 91c :6 0
a07 1 :3 0 73
:3 0 5e :3 0 30
:2 0 4e2 920 923
b2 :2 0 30 :2 0
4e7 925 927 :3 0
ee :3 0 4d :3 0
bc :3 0 bd :3 0
17 :4 0 ef 1
:8 0 934 15 :3 0
9b :3 0 92f 930
0 9c :3 0 931
932 0 934 4ea
987 9d :3 0 15
:3 0 9b :3 0 936
937 0 9c :3 0
938 939 0 93b
4ed 93d 4ef 93c
93b :2 0 985 9e
:3 0 15 :3 0 9b
:3 0 940 941 0
9c :3 0 942 943
0 97d 66 :3 0
67 :3 0 945 946
0 66 :3 0 77
:3 0 948 949 0
9f :2 0 2 :4 0
f0 :2 0 79 :2 0
27 :2 0 4f1 94e
950 :3 0 4f4 94a
952 947 954 953
:2 0 69 :3 0 7a
:3 0 956 957 0
6b :4 0 f1 :4 0
6e :2 0 6f :3 0
4f8 95b 95d :3 0
6e :2 0 82 :4 0
4fb 95f 961 :3 0
6e :2 0 a2 :4 0
4fe 963 965 :3 0
6e :2 0 9c :3 0
501 967 969 :3 0
6e :2 0 a3 :4 0
504 96b 96d :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 507 970
974 50b 96f 976
:3 0 50e 958 978
:2 0 97a 511 97b
955 97a 0 97c
513 0 97d 515
980 :3 0 980 0
980 97f 97d 97e
:6 0 982 10 :3 0
518 984 51a 983
982 :2 0 985 51c
:2 0 987 0 987
986 934 985 :6 0
a04 1 :3 0 97
:3 0 f2 :3 0 989
98a 0 4d :3 0
f3 :4 0 51f 98b
98e 8f :2 0 30
:2 0 524 990 992
:3 0 66 :3 0 67
:3 0 994 995 0
66 :3 0 77 :3 0
997 998 0 18
:2 0 2 :4 0 f4
:2 0 79 :2 0 27
:2 0 527 99d 99f
:3 0 52a 999 9a1
996 9a3 9a2 :2 0
69 :3 0 7a :3 0
9a5 9a6 0 6b
:4 0 f5 :4 0 6e
:2 0 6f :3 0 52e
9aa 9ac :3 0 6e
:2 0 82 :4 0 531
9ae 9b0 :3 0 6e
:2 0 83 :4 0 534
9b2 9b4 :3 0 6e
:2 0 73 :3 0 74
:3 0 84 :3 0 f6
:3 0 9b9 9ba 0
537 9b8 9bc 75
:4 0 539 9b7 9bf
53c 9b6 9c1 :3 0
6e :2 0 e5 :4 0
53f 9c3 9c5 :3 0
6e :2 0 71 :4 0
542 9c7 9c9 :3 0
6e :2 0 17 :3 0
545 9cb 9cd :3 0
6e :2 0 71 :4 0
548 9cf 9d1 :3 0
54b 9a7 9d3 :2 0
9d5 54e 9d6 9a4
9d5 0 9d7 550
0 a01 66 :3 0
67 :3 0 9d8 9d9
0 66 :3 0 77
:3 0 9db 9dc 0
27 :2 0 2 :4 0
f7 :2 0 79 :2 0
27 :2 0 552 9e1
9e3 :3 0 555 9dd
9e5 9da 9e7 9e6
:2 0 69 :3 0 7a
:3 0 9e9 9ea 0
6b :4 0 f8 :4 0
6e :2 0 6f :3 0
559 9ee 9f0 :3 0
55c 9eb 9f2 :2 0
9f4 55f 9f5 9e8
9f4 0 9f6 561
0 a01 88 :3 0
e8 :3 0 9f7 9f8
0 84 :3 0 f6
:3 0 9fa 9fb 0
17 :3 0 8a :3 0
563 9f9 9ff :2 0
a01 567 a02 993
a01 0 a03 56b
0 a04 56d a05
928 a04 0 a06
570 0 a07 572
b56 73 :3 0 b9
:3 0 73 :3 0 ba
:3 0 bb :3 0 3d
:3 0 3e :3 0 43
:3 0 bc :3 0 bd
:3 0 17 :4 0 f9
1 :8 0 a19 15
:3 0 9b :3 0 a14
a15 0 9c :3 0
a16 a17 0 a19
57a a6c 9d :3 0
15 :3 0 9b :3 0
a1b a1c 0 9c
:3 0 a1d a1e 0
a20 57d a22 57f
a21 a20 :2 0 a6a
9e :3 0 15 :3 0
9b :3 0 a25 a26
0 9c :3 0 a27
a28 0 a62 66
:3 0 67 :3 0 a2a
a2b 0 66 :3 0
77 :3 0 a2d a2e
0 9f :2 0 2
:4 0 fa :2 0 79
:2 0 27 :2 0 581
a33 a35 :3 0 584
a2f a37 a2c a39
a38 :2 0 69 :3 0
7a :3 0 a3b a3c
0 6b :4 0 fb
:4 0 6e :2 0 6f
:3 0 588 a40 a42
:3 0 6e :2 0 82
:4 0 58b a44 a46
:3 0 6e :2 0 a2
:4 0 58e a48 a4a
:3 0 6e :2 0 9c
:3 0 591 a4c a4e
:3 0 6e :2 0 a3
:4 0 594 a50 a52
:3 0 6e :2 0 a4
:3 0 a5 :3 0 27
:2 0 a6 :2 0 597
a55 a59 59b a54
a5b :3 0 59e a3d
a5d :2 0 a5f 5a1
a60 a3a a5f 0
a61 5a3 0 a62
5a5 a65 :3 0 a65
0 a65 a64 a62
a63 :6 0 a67 12
:3 0 5a8 a69 5aa
a68 a67 :2 0 a6a
5ac :2 0 a6c 0
a6c a6b a19 a6a
:6 0 b55 1 :3 0
15 :3 0 9b :3 0
a6e a6f 0 15
:3 0 8f :2 0 d9
:3 0 a71 a73 0
5b1 a72 a75 :3 0
a76 :2 0 3d :3 0
3e :3 0 fc :3 0
bd :3 0 17 :4 0
fd 1 :8 0 a83
15 :3 0 9b :3 0
a7e a7f 0 9c
:3 0 a80 a81 0
a83 5b4 ad6 9d
:3 0 15 :3 0 9b
:3 0 a85 a86 0
9c :3 0 a87 a88
0 a8a 5b7 a8c
5b9 a8b a8a :2 0
ad4 9e :3 0 15
:3 0 9b :3 0 a8f
a90 0 9c :3 0
a91 a92 0 acc
66 :3 0 67 :3 0
a94 a95 0 66
:3 0 77 :3 0 a97
a98 0 9f :2 0
2 :4 0 fe :2 0
79 :2 0 27 :2 0
5bb a9d a9f :3 0
5be a99 aa1 a96
aa3 aa2 :2 0 69
:3 0 7a :3 0 aa5
aa6 0 6b :4 0
ff :4 0 6e :2 0
6f :3 0 5c2 aaa
aac :3 0 6e :2 0
82 :4 0 5c5 aae
ab0 :3 0 6e :2 0
a2 :4 0 5c8 ab2
ab4 :3 0 6e :2 0
9c :3 0 5cb ab6
ab8 :3 0 6e :2 0
a3 :4 0 5ce aba
abc :3 0 6e :2 0
a4 :3 0 a5 :3 0
27 :2 0 a6 :2 0
5d1 abf ac3 5d5
abe ac5 :3 0 5d8
aa7 ac7 :2 0 ac9
5db aca aa4 ac9
0 acb 5dd 0
acc 5df acf :3 0
acf 0 acf ace
acc acd :6 0 ad1
14 :3 0 5e2 ad3
5e4 ad2 ad1 :2 0
ad4 5e6 :2 0 ad6
0 ad6 ad5 a83
ad4 :6 0 b52 1
:3 0 15 :3 0 9b
:3 0 ad8 ad9 0
15 :3 0 8f :2 0
d9 :3 0 adb add
0 5eb adc adf
:3 0 ae0 :2 0 66
:3 0 67 :3 0 ae2
ae3 0 66 :3 0
77 :3 0 ae5 ae6
0 18 :2 0 2
:4 0 100 :2 0 79
:2 0 27 :2 0 5ee
aeb aed :3 0 5f1
ae7 aef ae4 af1
af0 :2 0 69 :3 0
7a :3 0 af3 af4
0 6b :4 0 101
:4 0 6e :2 0 6f
:3 0 5f5 af8 afa
:3 0 6e :2 0 82
:4 0 5f8 afc afe
:3 0 6e :2 0 83
:4 0 5fb b00 b02
:3 0 6e :2 0 73
:3 0 74 :3 0 84
:3 0 93 :3 0 b07
b08 0 5fe b06
b0a 75 :4 0 600
b05 b0d 603 b04
b0f :3 0 6e :2 0
e5 :4 0 606 b11
b13 :3 0 6e :2 0
71 :4 0 609 b15
b17 :3 0 6e :2 0
17 :3 0 60c b19
b1b :3 0 6e :2 0
71 :4 0 60f b1d
b1f :3 0 612 af5
b21 :2 0 b23 615
b24 af2 b23 0
b25 617 0 b4f
66 :3 0 67 :3 0
b26 b27 0 66
:3 0 77 :3 0 b29
b2a 0 27 :2 0
2 :4 0 102 :2 0
79 :2 0 27 :2 0
619 b2f b31 :3 0
61c b2b b33 b28
b35 b34 :2 0 69
:3 0 7a :3 0 b37
b38 0 6b :4 0
103 :4 0 6e :2 0
6f :3 0 620 b3c
b3e :3 0 623 b39
b40 :2 0 b42 626
b43 b36 b42 0
b44 628 0 b4f
88 :3 0 e8 :3 0
b45 b46 0 84
:3 0 93 :3 0 b48
b49 0 17 :3 0
8a :3 0 62a b47
b4d :2 0 b4f 62e
b50 ae1 b4f 0
b51 632 0 b52
634 b53 a77 b52
0 b54 637 0
b55 639 b57 62a
a07 0 b58 0
b55 0 b58 63c
0 3149 63 :3 0
8f :2 0 30 :2 0
641 b5a b5c :3 0
21 :3 0 5 :3 0
b5e b5f 0 c88
22 :3 0 6 :3 0
b61 b62 0 c88
5e :3 0 104 :3 0
21 :3 0 22 :3 0
73 :3 0 38 :3 0
30 :2 0 644 b68
b6b 73 :3 0 39
:3 0 30 :2 0 647
b6d b70 73 :3 0
3a :3 0 30 :2 0
64a b72 b75 73
:3 0 31 :3 0 30
:2 0 64d b77 b7a
73 :3 0 3b :3 0
30 :2 0 650 b7c
b7f 73 :3 0 36
:3 0 30 :2 0 653
b81 b84 73 :3 0
3d :3 0 30 :2 0
656 b86 b89 73
:3 0 3e :3 0 30
:2 0 659 b8b b8e
35 :3 0 37 :3 0
41 :3 0 65c b65
b93 b64 b94 0
c88 73 :3 0 5e
:3 0 30 :2 0 66a
b96 b99 84 :3 0
8f :2 0 93 :3 0
b9b b9d 0 66f
b9c b9f :3 0 66
:3 0 67 :3 0 ba1
ba2 0 66 :3 0
77 :3 0 ba4 ba5
0 7f :2 0 2
:4 0 105 :2 0 79
:2 0 27 :2 0 672
baa bac :3 0 675
ba6 bae ba3 bb0
baf :2 0 69 :3 0
7a :3 0 bb2 bb3
0 6b :4 0 106
:4 0 6e :2 0 6f
:3 0 679 bb7 bb9
:3 0 6e :2 0 82
:4 0 67c bbb bbd
:3 0 6e :2 0 83
:4 0 67f bbf bc1
:3 0 6e :2 0 73
:3 0 74 :3 0 84
:3 0 93 :3 0 bc6
bc7 0 682 bc5
bc9 75 :4 0 684
bc4 bcc 687 bc3
bce :3 0 68a bb4
bd0 :2 0 bd2 68d
bd3 bb1 bd2 0
bd4 68f 0 bfe
66 :3 0 67 :3 0
bd5 bd6 0 66
:3 0 77 :3 0 bd8
bd9 0 27 :2 0
2 :4 0 107 :2 0
79 :2 0 27 :2 0
691 bde be0 :3 0
694 bda be2 bd7
be4 be3 :2 0 69
:3 0 7a :3 0 be6
be7 0 6b :4 0
108 :4 0 6e :2 0
6f :3 0 698 beb
bed :3 0 69b be8
bef :2 0 bf1 69e
bf2 be5 bf1 0
bf3 6a0 0 bfe
88 :3 0 89 :3 0
bf4 bf5 0 84
:3 0 93 :3 0 bf7
bf8 0 8a :3 0
6a2 bf6 bfb :2 0
bfe ae :3 0 6a5
c77 73 :3 0 5e
:3 0 30 :2 0 6a9
bff c02 b7 :2 0
30 :2 0 6ae c04
c06 :3 0 66 :3 0
67 :3 0 c08 c09
0 66 :3 0 77
:3 0 c0b c0c 0
18 :2 0 2 :4 0
109 :2 0 79 :2 0
27 :2 0 6b1 c11
c13 :3 0 6b4 c0d
c15 c0a c17 c16
:2 0 69 :3 0 7a
:3 0 c19 c1a 0
6b :4 0 10a :4 0
6e :2 0 6f :3 0
6b8 c1e c20 :3 0
6e :2 0 82 :4 0
6bb c22 c24 :3 0
6e :2 0 83 :4 0
6be c26 c28 :3 0
6e :2 0 73 :3 0
74 :3 0 5e :3 0
6c1 c2c c2e 75
:4 0 6c3 c2b c31
6c6 c2a c33 :3 0
6e :2 0 e5 :4 0
6c9 c35 c37 :3 0
6e :2 0 71 :4 0
6cc c39 c3b :3 0
6e :2 0 17 :3 0
6cf c3d c3f :3 0
6e :2 0 71 :4 0
6d2 c41 c43 :3 0
6d5 c1b c45 :2 0
c47 6d8 c48 c18
c47 0 c49 6da
0 c75 66 :3 0
67 :3 0 c4a c4b
0 66 :3 0 77
:3 0 c4d c4e 0
27 :2 0 2 :4 0
10b :2 0 79 :2 0
27 :2 0 6dc c53
c55 :3 0 6df c4f
c57 c4c c59 c58
:2 0 69 :3 0 7a
:3 0 c5b c5c 0
6b :4 0 10c :4 0
6e :2 0 6f :3 0
6e3 c60 c62 :3 0
6e6 c5d c64 :2 0
c66 6e9 c67 c5a
c66 0 c68 6eb
0 c75 88 :3 0
e8 :3 0 c69 c6a
0 73 :3 0 5e
:3 0 30 :2 0 6ed
c6c c6f 17 :3 0
8a :3 0 6f0 c6b
c73 :2 0 c75 6f4
c76 c07 c75 0
c78 ba0 bfe 0
c78 6f8 0 c88
22 :3 0 22 :3 0
c79 c7a 0 c88
22 :3 0 7e :2 0
6fb c7d c7e :3 0
c7f :2 0 22 :3 0
7c :4 0 c81 c82
0 c84 6fd c85
c80 c84 0 c86
6ff 0 c88 ae
:3 0 701 d06 73
:3 0 35 :3 0 30
:2 0 708 c89 c8c
b2 :2 0 30 :2 0
70d c8e c90 :3 0
c91 :2 0 73 :3 0
39 :3 0 30 :2 0
710 c93 c96 b7
:2 0 10d :2 0 715
c98 c9a :3 0 c9b
:2 0 c92 c9d c9c
:2 0 73 :3 0 39
:3 0 30 :2 0 718
c9f ca2 8f :2 0
27 :2 0 71d ca4
ca6 :3 0 ca7 :2 0
73 :3 0 39 :3 0
30 :2 0 720 ca9
cac 8f :2 0 10e
:2 0 725 cae cb0
:3 0 cb1 :2 0 ca8
cb3 cb2 :2 0 38
:3 0 10e :2 0 cb5
cb6 0 cb9 ae
:3 0 728 d02 73
:3 0 39 :3 0 30
:2 0 72a cba cbd
8f :2 0 30 :2 0
72f cbf cc1 :3 0
38 :3 0 27 :2 0
cc3 cc4 0 cc7
ae :3 0 732 cc8
cc2 cc7 0 d03
73 :3 0 39 :3 0
30 :2 0 734 cc9
ccc 8f :2 0 10f
:2 0 739 cce cd0
:3 0 cd1 :2 0 73
:3 0 39 :3 0 30
:2 0 73c cd3 cd6
8f :2 0 110 :2 0
741 cd8 cda :3 0
cdb :2 0 cd2 cdd
cdc :2 0 38 :3 0
27 :2 0 cdf ce0
0 ce3 ae :3 0
744 ce4 cde ce3
0 d03 73 :3 0
39 :3 0 30 :2 0
746 ce5 ce8 8f
:2 0 90 :2 0 74b
cea cec :3 0 35
:3 0 73 :3 0 3e
:3 0 30 :2 0 74e
cef cf2 111 :2 0
73 :3 0 35 :3 0
30 :2 0 751 cf5
cf8 754 cf4 cfa
:3 0 cee cfb 0
d00 38 :3 0 10f
:2 0 cfd cfe 0
d00 757 d01 ced
d00 0 d03 cb4
cb9 0 d03 75a
0 d04 75f d05
c9e d04 0 d07
b5d c88 0 d07
761 0 3149 59
:3 0 112 :4 0 d09
d0a :3 0 d08 d0b
0 3149 2b :3 0
7c :4 0 d0d d0e
0 3149 2c :3 0
7c :4 0 d10 d11
0 3149 34 :3 0
113 :3 0 114 :3 0
d14 d15 0 115
:3 0 73 :3 0 3a
:3 0 30 :2 0 764
d18 d1b 84 :3 0
116 :3 0 d1d d1e
0 767 d17 d20
d21 :2 0 117 :2 0
76a d16 d24 d13
d25 0 3149 3a
:3 0 115 :3 0 73
:3 0 3a :3 0 30
:2 0 76d d29 d2c
84 :3 0 118 :3 0
d2e d2f 0 770
d28 d31 d27 d32
0 3149 73 :3 0
32 :3 0 30 :2 0
773 d34 d37 b2
:2 0 30 :2 0 778
d39 d3b :3 0 59
:3 0 119 :3 0 5a
:3 0 32 :3 0 1a
:3 0 1d :3 0 1c
:3 0 28 :3 0 77b
d3e d45 d3d d46
0 1c32 73 :3 0
59 :3 0 30 :2 0
782 d48 d4b b7
:2 0 30 :2 0 787
d4d d4f :3 0 66
:3 0 67 :3 0 d51
d52 0 66 :3 0
77 :3 0 d54 d55
0 18 :2 0 2
:4 0 a6 :2 0 79
:2 0 27 :2 0 78a
d5a d5c :3 0 78d
d56 d5e d53 d60
d5f :2 0 69 :3 0
7a :3 0 d62 d63
0 6b :4 0 11a
:4 0 6e :2 0 6f
:3 0 791 d67 d69
:3 0 6e :2 0 82
:4 0 794 d6b d6d
:3 0 6e :2 0 83
:4 0 797 d6f d71
:3 0 6e :2 0 73
:3 0 74 :3 0 59
:3 0 79a d75 d77
75 :4 0 79c d74
d7a 79f d73 d7c
:3 0 6e :2 0 e5
:4 0 7a2 d7e d80
:3 0 6e :2 0 71
:4 0 7a5 d82 d84
:3 0 6e :2 0 28
:3 0 7a8 d86 d88
:3 0 6e :2 0 71
:4 0 7ab d8a d8c
:3 0 7ae d64 d8e
:2 0 d90 7b1 d91
d61 d90 0 d92
7b3 0 dbe 66
:3 0 67 :3 0 d93
d94 0 66 :3 0
77 :3 0 d96 d97
0 27 :2 0 2
:4 0 11b :2 0 79
:2 0 27 :2 0 7b5
d9c d9e :3 0 7b8
d98 da0 d95 da2
da1 :2 0 69 :3 0
7a :3 0 da4 da5
0 6b :4 0 11c
:4 0 6e :2 0 6f
:3 0 7bc da9 dab
:3 0 7bf da6 dad
:2 0 daf 7c2 db0
da3 daf 0 db1
7c4 0 dbe 88
:3 0 e8 :3 0 db2
db3 0 73 :3 0
59 :3 0 30 :2 0
7c6 db5 db8 28
:3 0 8a :3 0 7c9
db4 dbc :2 0 dbe
7cd dbf d50 dbe
0 dc0 7d1 0
1c32 21 :4 0 dc1
dc2 0 1c32 11d
:3 0 11e :3 0 97
:3 0 98 :3 0 1f
:3 0 97 :3 0 98
:3 0 11d :3 0 11e
:3 0 21 :3 0 44
:3 0 11f :3 0 b8
:3 0 17 :3 0 120
:3 0 17 :3 0 121
:3 0 1d :3 0 122
:3 0 26 :4 0 123
1 :8 0 dde 15
:3 0 9b :3 0 dd9
dda 0 9c :3 0
ddb ddc 0 dde
7d3 e31 9d :3 0
15 :3 0 9b :3 0
de0 de1 0 9c
:3 0 de2 de3 0
de5 7d6 de7 7d8
de6 de5 :2 0 e2f
9e :3 0 15 :3 0
9b :3 0 dea deb
0 9c :3 0 dec
ded 0 e27 66
:3 0 67 :3 0 def
df0 0 66 :3 0
77 :3 0 df2 df3
0 9f :2 0 2
:4 0 124 :2 0 79
:2 0 27 :2 0 7da
df8 dfa :3 0 7dd
df4 dfc df1 dfe
dfd :2 0 69 :3 0
7a :3 0 e00 e01
0 6b :4 0 125
:4 0 6e :2 0 6f
:3 0 7e1 e05 e07
:3 0 6e :2 0 82
:4 0 7e4 e09 e0b
:3 0 6e :2 0 a2
:4 0 7e7 e0d e0f
:3 0 6e :2 0 9c
:3 0 7ea e11 e13
:3 0 6e :2 0 a3
:4 0 7ed e15 e17
:3 0 6e :2 0 a4
:3 0 a5 :3 0 27
:2 0 a6 :2 0 7f0
e1a e1e 7f4 e19
e20 :3 0 7f7 e02
e22 :2 0 e24 7fa
e25 dff e24 0
e26 7fc 0 e27
7fe e2a :3 0 e2a
0 e2a e29 e27
e28 :6 0 e2c 16
:3 0 801 e2e 803
e2d e2c :2 0 e2f
805 :2 0 e31 0
e31 e30 dde e2f
:6 0 1c32 1 :3 0
15 :3 0 9b :3 0
e33 e34 0 15
:3 0 8f :2 0 a7
:3 0 e36 e38 0
80a e37 e3a :3 0
e3b :2 0 21 :3 0
96 :2 0 80d e3e
e3f :3 0 e40 :2 0
e41 :2 0 e3c e43
e42 :2 0 46 :3 0
27 :2 0 e45 e46
0 e57 73 :3 0
44 :3 0 30 :2 0
80f e48 e4b a8
:2 0 30 :2 0 814
e4d e4f :3 0 1f
:3 0 21 :3 0 e51
e52 0 e54 817
e55 e50 e54 0
e56 819 0 e57
81b e58 e44 e57
0 e59 81e 0
1c32 33 :3 0 30
:2 0 e5a e5b 0
1c32 44 :3 0 30
:2 0 e5d e5e 0
1c32 63 :3 0 8f
:2 0 27 :2 0 822
e61 e63 :3 0 a4
:3 0 26 :3 0 27
:2 0 27 :2 0 825
e65 e69 8f :2 0
30 :4 0 82b e6b
e6d :3 0 126 :3 0
127 :3 0 128 :3 0
a4 :3 0 1f :3 0
128 :3 0 a4 :3 0
127 :3 0 128 :3 0
a4 :3 0 22 :3 0
128 :3 0 a4 :3 0
129 :3 0 12a :3 0
37 :3 0 12a :3 0
19 :3 0 21 :3 0
44 :3 0 47 :3 0
11f :3 0 120 :3 0
17 :3 0 121 :3 0
1d :3 0 122 :3 0
12b :4 0 12c 1
:8 0 e91 15 :3 0
9b :3 0 e8c e8d
0 9c :3 0 e8e
e8f 0 e91 82e
ee4 9d :3 0 15
:3 0 9b :3 0 e93
e94 0 9c :3 0
e95 e96 0 e98
831 e9a 833 e99
e98 :2 0 ee2 9e
:3 0 15 :3 0 9b
:3 0 e9d e9e 0
9c :3 0 e9f ea0
0 eda 66 :3 0
67 :3 0 ea2 ea3
0 66 :3 0 77
:3 0 ea5 ea6 0
9f :2 0 2 :4 0
12d :2 0 79 :2 0
27 :2 0 835 eab
ead :3 0 838 ea7
eaf ea4 eb1 eb0
:2 0 69 :3 0 7a
:3 0 eb3 eb4 0
6b :4 0 12e :4 0
6e :2 0 6f :3 0
83c eb8 eba :3 0
6e :2 0 82 :4 0
83f ebc ebe :3 0
6e :2 0 a2 :4 0
842 ec0 ec2 :3 0
6e :2 0 9c :3 0
845 ec4 ec6 :3 0
6e :2 0 a3 :4 0
848 ec8 eca :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 84b ecd
ed1 84f ecc ed3
:3 0 852 eb5 ed5
:2 0 ed7 855 ed8
eb2 ed7 0 ed9
857 0 eda 859
edd :3 0 edd 0
edd edc eda edb
:6 0 edf 18 :3 0
85c ee1 85e ee0
edf :2 0 ee2 860
:2 0 ee4 0 ee4
ee3 e91 ee2 :6 0
1017 1 :3 0 15
:3 0 9b :3 0 ee6
ee7 0 15 :3 0
8f :2 0 a7 :3 0
ee9 eeb 0 865
eea eed :3 0 eee
:2 0 46 :3 0 27
:2 0 ef0 ef1 0
f87 73 :3 0 47
:3 0 30 :2 0 868
ef3 ef6 8f :2 0
30 :2 0 86d ef8
efa :3 0 efb :2 0
73 :3 0 44 :3 0
30 :2 0 870 efd
f00 8f :2 0 27
:2 0 875 f02 f04
:3 0 f05 :2 0 efc
f07 f06 :2 0 126
:3 0 12f :3 0 128
:3 0 a4 :3 0 1f
:3 0 128 :3 0 a4
:3 0 12f :3 0 128
:3 0 a4 :3 0 22
:3 0 128 :3 0 a4
:3 0 130 :3 0 131
:3 0 37 :3 0 19
:3 0 21 :3 0 44
:3 0 11f :3 0 120
:3 0 17 :3 0 121
:3 0 1d :3 0 122
:3 0 12b :4 0 132
1 :8 0 f29 15
:3 0 9b :3 0 f24
f25 0 9c :3 0
f26 f27 0 f29
878 f7c 9d :3 0
15 :3 0 9b :3 0
f2b f2c 0 9c
:3 0 f2d f2e 0
f30 87b f32 87d
f31 f30 :2 0 f7a
9e :3 0 15 :3 0
9b :3 0 f35 f36
0 9c :3 0 f37
f38 0 f72 66
:3 0 67 :3 0 f3a
f3b 0 66 :3 0
77 :3 0 f3d f3e
0 9f :2 0 2
:4 0 133 :2 0 79
:2 0 27 :2 0 87f
f43 f45 :3 0 882
f3f f47 f3c f49
f48 :2 0 69 :3 0
7a :3 0 f4b f4c
0 6b :4 0 134
:4 0 6e :2 0 6f
:3 0 886 f50 f52
:3 0 6e :2 0 82
:4 0 889 f54 f56
:3 0 6e :2 0 a2
:4 0 88c f58 f5a
:3 0 6e :2 0 9c
:3 0 88f f5c f5e
:3 0 6e :2 0 a3
:4 0 892 f60 f62
:3 0 6e :2 0 a4
:3 0 a5 :3 0 27
:2 0 a6 :2 0 895
f65 f69 899 f64
f6b :3 0 89c f4d
f6d :2 0 f6f 89f
f70 f4a f6f 0
f71 8a1 0 f72
8a3 f75 :3 0 f75
0 f75 f74 f72
f73 :6 0 f77 1a
:3 0 8a6 f79 8a8
f78 f77 :2 0 f7a
8aa :2 0 f7c 0
f7c f7b f29 f7a
:6 0 f7e 1 :3 0
8ad f83 33 :3 0
10e :2 0 f7f f80
0 f82 8af f84
f08 f7e 0 f85
0 f82 0 f85
8b1 0 f87 ae
:3 0 8b4 1015 15
:3 0 9b :3 0 f88
f89 0 15 :3 0
8f :2 0 d9 :3 0
f8b f8d 0 8b9
f8c f8f :3 0 f90
:2 0 73 :3 0 44
:3 0 30 :2 0 8bc
f92 f95 8f :2 0
27 :2 0 8c1 f97
f99 :3 0 f9a :2 0
f91 f9c f9b :2 0
126 :3 0 12f :3 0
128 :3 0 a4 :3 0
1f :3 0 128 :3 0
a4 :3 0 12f :3 0
128 :3 0 a4 :3 0
22 :3 0 128 :3 0
a4 :3 0 130 :3 0
131 :3 0 37 :3 0
19 :3 0 21 :3 0
44 :3 0 11f :3 0
120 :3 0 17 :3 0
121 :3 0 1d :3 0
122 :3 0 12b :4 0
135 1 :8 0 fbe
15 :3 0 9b :3 0
fb9 fba 0 9c
:3 0 fbb fbc 0
fbe 8c4 1011 9d
:3 0 15 :3 0 9b
:3 0 fc0 fc1 0
9c :3 0 fc2 fc3
0 fc5 8c7 fc7
8c9 fc6 fc5 :2 0
100f 9e :3 0 15
:3 0 9b :3 0 fca
fcb 0 9c :3 0
fcc fcd 0 1007
66 :3 0 67 :3 0
fcf fd0 0 66
:3 0 77 :3 0 fd2
fd3 0 9f :2 0
2 :4 0 136 :2 0
79 :2 0 27 :2 0
8cb fd8 fda :3 0
8ce fd4 fdc fd1
fde fdd :2 0 69
:3 0 7a :3 0 fe0
fe1 0 6b :4 0
137 :4 0 6e :2 0
6f :3 0 8d2 fe5
fe7 :3 0 6e :2 0
82 :4 0 8d5 fe9
feb :3 0 6e :2 0
a2 :4 0 8d8 fed
fef :3 0 6e :2 0
9c :3 0 8db ff1
ff3 :3 0 6e :2 0
a3 :4 0 8de ff5
ff7 :3 0 6e :2 0
a4 :3 0 a5 :3 0
27 :2 0 a6 :2 0
8e1 ffa ffe 8e5
ff9 1000 :3 0 8e8
fe2 1002 :2 0 1004
8eb 1005 fdf 1004
0 1006 8ed 0
1007 8ef 100a :3 0
100a 0 100a 1009
1007 1008 :6 0 100c
1c :3 0 8f2 100e
8f4 100d 100c :2 0
100f 8f6 :2 0 1011
0 1011 1010 fbe
100f :6 0 1013 1
:3 0 8f9 1014 f9d
1013 0 1016 eef
f87 0 1016 8fb
0 1017 8fe 108e
126 :3 0 12f :3 0
128 :3 0 a4 :3 0
1f :3 0 128 :3 0
a4 :3 0 12f :3 0
128 :3 0 a4 :3 0
22 :3 0 128 :3 0
a4 :3 0 130 :3 0
131 :3 0 37 :3 0
19 :3 0 21 :3 0
44 :3 0 11f :3 0
120 :3 0 17 :3 0
121 :3 0 1d :3 0
122 :3 0 12b :4 0
138 1 :8 0 1038
15 :3 0 9b :3 0
1033 1034 0 9c
:3 0 1035 1036 0
1038 901 108b 9d
:3 0 15 :3 0 9b
:3 0 103a 103b 0
9c :3 0 103c 103d
0 103f 904 1041
906 1040 103f :2 0
1089 9e :3 0 15
:3 0 9b :3 0 1044
1045 0 9c :3 0
1046 1047 0 1081
66 :3 0 67 :3 0
1049 104a 0 66
:3 0 77 :3 0 104c
104d 0 9f :2 0
2 :4 0 139 :2 0
79 :2 0 27 :2 0
908 1052 1054 :3 0
90b 104e 1056 104b
1058 1057 :2 0 69
:3 0 7a :3 0 105a
105b 0 6b :4 0
13a :4 0 6e :2 0
6f :3 0 90f 105f
1061 :3 0 6e :2 0
82 :4 0 912 1063
1065 :3 0 6e :2 0
a2 :4 0 915 1067
1069 :3 0 6e :2 0
9c :3 0 918 106b
106d :3 0 6e :2 0
a3 :4 0 91b 106f
1071 :3 0 6e :2 0
a4 :3 0 a5 :3 0
27 :2 0 a6 :2 0
91e 1074 1078 922
1073 107a :3 0 925
105c 107c :2 0 107e
928 107f 1059 107e
0 1080 92a 0
1081 92c 1084 :3 0
1084 0 1084 1083
1081 1082 :6 0 1086
1e :3 0 92f 1088
931 1087 1086 :2 0
1089 933 :2 0 108b
0 108b 108a 1038
1089 :6 0 108d 1
:3 0 936 108f e6e
1017 0 1090 0
108d 0 1090 938
0 1091 93b 12a5
a4 :3 0 26 :3 0
27 :2 0 27 :2 0
93d 1092 1096 8f
:2 0 30 :4 0 943
1098 109a :3 0 126
:3 0 127 :3 0 128
:3 0 a4 :3 0 1f
:3 0 128 :3 0 a4
:3 0 127 :3 0 128
:3 0 a4 :3 0 129
:3 0 37 :3 0 12a
:3 0 12a :3 0 19
:3 0 21 :3 0 44
:3 0 47 :3 0 11f
:3 0 120 :3 0 17
:3 0 121 :3 0 1d
:3 0 122 :3 0 12b
:4 0 13b 1 :8 0
10bb 15 :3 0 9b
:3 0 10b6 10b7 0
9c :3 0 10b8 10b9
0 10bb 946 110e
9d :3 0 15 :3 0
9b :3 0 10bd 10be
0 9c :3 0 10bf
10c0 0 10c2 949
10c4 94b 10c3 10c2
:2 0 110c 9e :3 0
15 :3 0 9b :3 0
10c7 10c8 0 9c
:3 0 10c9 10ca 0
1104 66 :3 0 67
:3 0 10cc 10cd 0
66 :3 0 77 :3 0
10cf 10d0 0 9f
:2 0 2 :4 0 13c
:2 0 79 :2 0 27
:2 0 94d 10d5 10d7
:3 0 950 10d1 10d9
10ce 10db 10da :2 0
69 :3 0 7a :3 0
10dd 10de 0 6b
:4 0 13d :4 0 6e
:2 0 6f :3 0 954
10e2 10e4 :3 0 6e
:2 0 82 :4 0 957
10e6 10e8 :3 0 6e
:2 0 a2 :4 0 95a
10ea 10ec :3 0 6e
:2 0 9c :3 0 95d
10ee 10f0 :3 0 6e
:2 0 a3 :4 0 960
10f2 10f4 :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 963 10f7 10fb
967 10f6 10fd :3 0
96a 10df 10ff :2 0
1101 96d 1102 10dc
1101 0 1103 96f
0 1104 971 1107
:3 0 1107 0 1107
1106 1104 1105 :6 0
1109 20 :3 0 974
110b 976 110a 1109
:2 0 110c 978 :2 0
110e 0 110e 110d
10bb 110c :6 0 122d
1 :3 0 15 :3 0
9b :3 0 1110 1111
0 15 :3 0 8f
:2 0 a7 :3 0 1113
1115 0 97d 1114
1117 :3 0 1118 :2 0
46 :3 0 27 :2 0
111a 111b 0 11ac
73 :3 0 47 :3 0
30 :2 0 980 111d
1120 8f :2 0 30
:2 0 985 1122 1124
:3 0 1125 :2 0 73
:3 0 44 :3 0 30
:2 0 988 1127 112a
8f :2 0 27 :2 0
98d 112c 112e :3 0
112f :2 0 1126 1131
1130 :2 0 126 :3 0
12f :3 0 128 :3 0
a4 :3 0 1f :3 0
128 :3 0 a4 :3 0
130 :3 0 37 :3 0
131 :3 0 19 :3 0
21 :3 0 44 :3 0
11f :3 0 120 :3 0
17 :3 0 121 :3 0
1d :3 0 122 :3 0
12b :4 0 13e 1
:8 0 114d 15 :3 0
9b :3 0 1148 1149
0 9c :3 0 114a
114b 0 114d 990
11a0 9d :3 0 15
:3 0 9b :3 0 114f
1150 0 9c :3 0
1151 1152 0 1154
993 1156 995 1155
1154 :2 0 119e 9e
:3 0 15 :3 0 9b
:3 0 1159 115a 0
9c :3 0 115b 115c
0 1196 66 :3 0
67 :3 0 115e 115f
0 66 :3 0 77
:3 0 1161 1162 0
9f :2 0 2 :4 0
13f :2 0 79 :2 0
27 :2 0 997 1167
1169 :3 0 99a 1163
116b 1160 116d 116c
:2 0 69 :3 0 7a
:3 0 116f 1170 0
6b :4 0 140 :4 0
6e :2 0 6f :3 0
99e 1174 1176 :3 0
6e :2 0 82 :4 0
9a1 1178 117a :3 0
6e :2 0 a2 :4 0
9a4 117c 117e :3 0
6e :2 0 9c :3 0
9a7 1180 1182 :3 0
6e :2 0 a3 :4 0
9aa 1184 1186 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 9ad 1189
118d 9b1 1188 118f
:3 0 9b4 1171 1191
:2 0 1193 9b7 1194
116e 1193 0 1195
9b9 0 1196 9bb
1199 :3 0 1199 0
1199 1198 1196 1197
:6 0 119b 22 :3 0
9be 119d 9c0 119c
119b :2 0 119e 9c2
:2 0 11a0 0 11a0
119f 114d 119e :6 0
11a2 1 :3 0 9c5
11a8 33 :3 0 10e
:2 0 11a3 11a4 0
11a7 0 11a7 9c7
11a9 1132 11a2 0
11aa 0 11a7 0
11aa 9ca 0 11ac
ae :3 0 9cd 122b
15 :3 0 9b :3 0
11ad 11ae 0 15
:3 0 8f :2 0 d9
:3 0 11b0 11b2 0
9d2 11b1 11b4 :3 0
11b5 :2 0 126 :3 0
12f :3 0 128 :3 0
a4 :3 0 1f :3 0
128 :3 0 a4 :3 0
12f :3 0 128 :3 0
a4 :3 0 130 :3 0
37 :3 0 131 :3 0
19 :3 0 21 :3 0
44 :3 0 11f :3 0
120 :3 0 17 :3 0
121 :3 0 1d :3 0
122 :3 0 12b :4 0
141 1 :8 0 11d4
15 :3 0 9b :3 0
11cf 11d0 0 9c
:3 0 11d1 11d2 0
11d4 9d5 1227 9d
:3 0 15 :3 0 9b
:3 0 11d6 11d7 0
9c :3 0 11d8 11d9
0 11db 9d8 11dd
9da 11dc 11db :2 0
1225 9e :3 0 15
:3 0 9b :3 0 11e0
11e1 0 9c :3 0
11e2 11e3 0 121d
66 :3 0 67 :3 0
11e5 11e6 0 66
:3 0 77 :3 0 11e8
11e9 0 9f :2 0
2 :4 0 142 :2 0
79 :2 0 27 :2 0
9dc 11ee 11f0 :3 0
9df 11ea 11f2 11e7
11f4 11f3 :2 0 69
:3 0 7a :3 0 11f6
11f7 0 6b :4 0
143 :4 0 6e :2 0
6f :3 0 9e3 11fb
11fd :3 0 6e :2 0
82 :4 0 9e6 11ff
1201 :3 0 6e :2 0
a2 :4 0 9e9 1203
1205 :3 0 6e :2 0
9c :3 0 9ec 1207
1209 :3 0 6e :2 0
a3 :4 0 9ef 120b
120d :3 0 6e :2 0
a4 :3 0 a5 :3 0
27 :2 0 a6 :2 0
9f2 1210 1214 9f6
120f 1216 :3 0 9f9
11f8 1218 :2 0 121a
9fc 121b 11f5 121a
0 121c 9fe 0
121d a00 1220 :3 0
1220 0 1220 121f
121d 121e :6 0 1222
24 :3 0 a03 1224
a05 1223 1222 :2 0
1225 a07 :2 0 1227
0 1227 1226 11d4
1225 :6 0 1229 1
:3 0 a0a 122a 11b6
1229 0 122c 1119
11ac 0 122c a0c
0 122d a0f 12a1
126 :3 0 12f :3 0
128 :3 0 a4 :3 0
1f :3 0 128 :3 0
a4 :3 0 12f :3 0
128 :3 0 a4 :3 0
130 :3 0 37 :3 0
131 :3 0 19 :3 0
21 :3 0 44 :3 0
11f :3 0 120 :3 0
17 :3 0 121 :3 0
1d :3 0 122 :3 0
12b :4 0 144 1
:8 0 124b 15 :3 0
9b :3 0 1246 1247
0 9c :3 0 1248
1249 0 124b a12
129e 9d :3 0 15
:3 0 9b :3 0 124d
124e 0 9c :3 0
124f 1250 0 1252
a15 1254 a17 1253
1252 :2 0 129c 9e
:3 0 15 :3 0 9b
:3 0 1257 1258 0
9c :3 0 1259 125a
0 1294 66 :3 0
67 :3 0 125c 125d
0 66 :3 0 77
:3 0 125f 1260 0
9f :2 0 2 :4 0
145 :2 0 79 :2 0
27 :2 0 a19 1265
1267 :3 0 a1c 1261
1269 125e 126b 126a
:2 0 69 :3 0 7a
:3 0 126d 126e 0
6b :4 0 146 :4 0
6e :2 0 6f :3 0
a20 1272 1274 :3 0
6e :2 0 82 :4 0
a23 1276 1278 :3 0
6e :2 0 a2 :4 0
a26 127a 127c :3 0
6e :2 0 9c :3 0
a29 127e 1280 :3 0
6e :2 0 a3 :4 0
a2c 1282 1284 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 a2f 1287
128b a33 1286 128d
:3 0 a36 126f 128f
:2 0 1291 a39 1292
126c 1291 0 1293
a3b 0 1294 a3d
1297 :3 0 1297 0
1297 1296 1294 1295
:6 0 1299 26 :3 0
a40 129b a42 129a
1299 :2 0 129c a44
:2 0 129e 0 129e
129d 124b 129c :6 0
12a0 1 :3 0 a47
12a2 109b 122d 0
12a3 0 12a0 0
12a3 a49 0 12a4
a4c 12a6 e64 1091
0 12a7 0 12a4
0 12a7 a4e 0
1c32 15 :3 0 9b
:3 0 12a8 12a9 0
15 :3 0 8f :2 0
a7 :3 0 12ab 12ad
0 a53 12ac 12af
:3 0 12b0 :2 0 73
:3 0 44 :3 0 30
:2 0 a56 12b2 12b5
8f :2 0 27 :2 0
a5b 12b7 12b9 :3 0
66 :3 0 67 :3 0
12bb 12bc 0 66
:3 0 77 :3 0 12be
12bf 0 18 :2 0
2 :4 0 147 :2 0
79 :2 0 27 :2 0
a5e 12c4 12c6 :3 0
a61 12c0 12c8 12bd
12ca 12c9 :2 0 69
:3 0 7a :3 0 12cc
12cd 0 6b :4 0
148 :4 0 6e :2 0
6f :3 0 a65 12d1
12d3 :3 0 6e :2 0
82 :4 0 a68 12d5
12d7 :3 0 6e :2 0
83 :4 0 a6b 12d9
12db :3 0 6e :2 0
73 :3 0 74 :3 0
84 :3 0 149 :3 0
12e0 12e1 0 a6e
12df 12e3 75 :4 0
a70 12de 12e6 a73
12dd 12e8 :3 0 6e
:2 0 e5 :4 0 a76
12ea 12ec :3 0 6e
:2 0 71 :4 0 a79
12ee 12f0 :3 0 6e
:2 0 1d :3 0 a7c
12f2 12f4 :3 0 6e
:2 0 71 :4 0 a7f
12f6 12f8 :3 0 a82
12ce 12fa :2 0 12fc
a85 12fd 12cb 12fc
0 12fe a87 0
1329 66 :3 0 67
:3 0 12ff 1300 0
66 :3 0 77 :3 0
1302 1303 0 27
:2 0 2 :4 0 14a
:2 0 79 :2 0 27
:2 0 a89 1308 130a
:3 0 a8c 1304 130c
1301 130e 130d :2 0
69 :3 0 7a :3 0
1310 1311 0 6b
:4 0 14b :4 0 6e
:2 0 6f :3 0 a90
1315 1317 :3 0 a93
1312 1319 :2 0 131b
a96 131c 130f 131b
0 131d a98 0
1329 88 :3 0 e8
:3 0 131e 131f 0
84 :3 0 149 :3 0
1321 1322 0 1d
:3 0 8a :3 0 a9a
1320 1326 :2 0 1329
ae :3 0 a9e 148f
19 :3 0 96 :2 0
aa2 132b 132c :3 0
132d :2 0 63 :3 0
8f :2 0 27 :2 0
aa6 1330 1332 :3 0
47 :3 0 30 :2 0
1334 1335 0 13a1
14c :3 0 14d :3 0
47 :3 0 14e :3 0
b8 :3 0 17 :3 0
14f :3 0 19 :3 0
150 :3 0 128 :3 0
a4 :3 0 1f :3 0
128 :3 0 a4 :3 0
22 :4 0 151 1
:8 0 134c 15 :3 0
9b :3 0 1347 1348
0 9c :3 0 1349
134a 0 134c aa9
139f 9d :3 0 15
:3 0 9b :3 0 134e
134f 0 9c :3 0
1350 1351 0 1353
aac 1355 aae 1354
1353 :2 0 139d 9e
:3 0 15 :3 0 9b
:3 0 1358 1359 0
9c :3 0 135a 135b
0 1395 66 :3 0
67 :3 0 135d 135e
0 66 :3 0 77
:3 0 1360 1361 0
9f :2 0 2 :4 0
152 :2 0 79 :2 0
27 :2 0 ab0 1366
1368 :3 0 ab3 1362
136a 135f 136c 136b
:2 0 69 :3 0 7a
:3 0 136e 136f 0
6b :4 0 153 :4 0
6e :2 0 6f :3 0
ab7 1373 1375 :3 0
6e :2 0 82 :4 0
aba 1377 1379 :3 0
6e :2 0 a2 :4 0
abd 137b 137d :3 0
6e :2 0 9c :3 0
ac0 137f 1381 :3 0
6e :2 0 a3 :4 0
ac3 1383 1385 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 ac6 1388
138c aca 1387 138e
:3 0 acd 1370 1390
:2 0 1392 ad0 1393
136d 1392 0 1394
ad2 0 1395 ad4
1398 :3 0 1398 0
1398 1397 1395 1396
:6 0 139a 28 :3 0
ad7 139c ad9 139b
139a :2 0 139d adb
:2 0 139f 0 139f
139e 134c 139d :6 0
13a1 1 :3 0 ade
140d 47 :3 0 30
:2 0 13a2 13a3 0
140c 14c :3 0 14d
:3 0 47 :3 0 14e
:3 0 b8 :3 0 17
:3 0 14f :3 0 19
:3 0 150 :3 0 128
:3 0 a4 :3 0 1f
:4 0 154 1 :8 0
13b7 15 :3 0 9b
:3 0 13b2 13b3 0
9c :3 0 13b4 13b5
0 13b7 ae1 140a
9d :3 0 15 :3 0
9b :3 0 13b9 13ba
0 9c :3 0 13bb
13bc 0 13be ae4
13c0 ae6 13bf 13be
:2 0 1408 9e :3 0
15 :3 0 9b :3 0
13c3 13c4 0 9c
:3 0 13c5 13c6 0
1400 66 :3 0 67
:3 0 13c8 13c9 0
66 :3 0 77 :3 0
13cb 13cc 0 9f
:2 0 2 :4 0 155
:2 0 79 :2 0 27
:2 0 ae8 13d1 13d3
:3 0 aeb 13cd 13d5
13ca 13d7 13d6 :2 0
69 :3 0 7a :3 0
13d9 13da 0 6b
:4 0 156 :4 0 6e
:2 0 6f :3 0 aef
13de 13e0 :3 0 6e
:2 0 82 :4 0 af2
13e2 13e4 :3 0 6e
:2 0 a2 :4 0 af5
13e6 13e8 :3 0 6e
:2 0 9c :3 0 af8
13ea 13ec :3 0 6e
:2 0 a3 :4 0 afb
13ee 13f0 :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 afe 13f3 13f7
b02 13f2 13f9 :3 0
b05 13db 13fb :2 0
13fd b08 13fe 13d8
13fd 0 13ff b0a
0 1400 b0c 1403
:3 0 1403 0 1403
1402 1400 1401 :6 0
1405 2a :3 0 b0f
1407 b11 1406 1405
:2 0 1408 b13 :2 0
140a 0 140a 1409
13b7 1408 :6 0 140c
1 :3 0 b16 140e
1333 13a1 0 140f
0 140c 0 140f
b19 0 148d 73
:3 0 47 :3 0 30
:2 0 b1c 1410 1413
73 :3 0 a8 :2 0
37 :3 0 30 :2 0
b1f 1415 1419 b24
1416 141b :3 0 66
:3 0 67 :3 0 141d
141e 0 66 :3 0
77 :3 0 1420 1421
0 18 :2 0 2
:4 0 157 :2 0 79
:2 0 27 :2 0 b27
1426 1428 :3 0 b2a
1422 142a 141f 142c
142b :2 0 69 :3 0
7a :3 0 142e 142f
0 6b :4 0 158
:4 0 6e :2 0 6f
:3 0 b2e 1433 1435
:3 0 6e :2 0 82
:4 0 b31 1437 1439
:3 0 6e :2 0 83
:4 0 b34 143b 143d
:3 0 6e :2 0 73
:3 0 74 :3 0 84
:3 0 149 :3 0 1442
1443 0 b37 1441
1445 75 :4 0 b39
1440 1448 b3c 143f
144a :3 0 6e :2 0
e5 :4 0 b3f 144c
144e :3 0 6e :2 0
71 :4 0 b42 1450
1452 :3 0 6e :2 0
1d :3 0 b45 1454
1456 :3 0 6e :2 0
71 :4 0 b48 1458
145a :3 0 b4b 1430
145c :2 0 145e b4e
145f 142d 145e 0
1460 b50 0 148a
66 :3 0 67 :3 0
1461 1462 0 66
:3 0 77 :3 0 1464
1465 0 27 :2 0
2 :4 0 159 :2 0
79 :2 0 27 :2 0
b52 146a 146c :3 0
b55 1466 146e 1463
1470 146f :2 0 69
:3 0 7a :3 0 1472
1473 0 6b :4 0
15a :4 0 6e :2 0
6f :3 0 b59 1477
1479 :3 0 b5c 1474
147b :2 0 147d b5f
147e 1471 147d 0
147f b61 0 148a
88 :3 0 e8 :3 0
1480 1481 0 84
:3 0 149 :3 0 1483
1484 0 1d :3 0
8a :3 0 b63 1482
1488 :2 0 148a b67
148b 141c 148a 0
148c b6b 0 148d
b6d 148e 132e 148d
0 1490 12ba 1329
0 1490 b70 0
1495 46 :3 0 27
:2 0 1491 1492 0
1495 ae :3 0 b73
151e 73 :3 0 46
:3 0 30 :2 0 b76
1496 1499 8f :2 0
27 :2 0 b7b 149b
149d :3 0 149e :2 0
15b :3 0 73 :3 0
32 :3 0 30 :2 0
b7e 14a1 14a4 b81
14a0 14a6 14a7 :2 0
b7 :2 0 30 :2 0
b85 14a9 14ab :3 0
149f 14ad 14ac :2 0
66 :3 0 67 :3 0
14af 14b0 0 66
:3 0 77 :3 0 14b2
14b3 0 18 :2 0
2 :4 0 15c :2 0
79 :2 0 27 :2 0
b88 14b8 14ba :3 0
b8b 14b4 14bc 14b1
14be 14bd :2 0 69
:3 0 7a :3 0 14c0
14c1 0 6b :4 0
15d :4 0 6e :2 0
6f :3 0 b8f 14c5
14c7 :3 0 6e :2 0
82 :4 0 b92 14c9
14cb :3 0 6e :2 0
83 :4 0 b95 14cd
14cf :3 0 6e :2 0
73 :3 0 74 :3 0
84 :3 0 149 :3 0
14d4 14d5 0 b98
14d3 14d7 75 :4 0
b9a 14d2 14da b9d
14d1 14dc :3 0 6e
:2 0 e5 :4 0 ba0
14de 14e0 :3 0 6e
:2 0 71 :4 0 ba3
14e2 14e4 :3 0 6e
:2 0 1d :3 0 ba6
14e6 14e8 :3 0 6e
:2 0 71 :4 0 ba9
14ea 14ec :3 0 bac
14c2 14ee :2 0 14f0
baf 14f1 14bf 14f0
0 14f2 bb1 0
151c 66 :3 0 67
:3 0 14f3 14f4 0
66 :3 0 77 :3 0
14f6 14f7 0 27
:2 0 2 :4 0 15e
:2 0 79 :2 0 27
:2 0 bb3 14fc 14fe
:3 0 bb6 14f8 1500
14f5 1502 1501 :2 0
69 :3 0 7a :3 0
1504 1505 0 6b
:4 0 15f :4 0 6e
:2 0 6f :3 0 bba
1509 150b :3 0 bbd
1506 150d :2 0 150f
bc0 1510 1503 150f
0 1511 bc2 0
151c 88 :3 0 e8
:3 0 1512 1513 0
84 :3 0 149 :3 0
1515 1516 0 1d
:3 0 8a :3 0 bc4
1514 151a :2 0 151c
bc8 151d 14ae 151c
0 151f 12b1 1495
0 151f bcc 0
1c32 a4 :3 0 26
:3 0 27 :2 0 27
:2 0 bcf 1520 1524
8f :2 0 30 :4 0
bd5 1526 1528 :3 0
2b :3 0 a4 :3 0
2b :3 0 6e :2 0
1d :3 0 bd8 152d
152f :3 0 27 :2 0
2a :2 0 bdb 152b
1533 152a 1534 0
171f 160 :3 0 97
:3 0 161 :3 0 21
:3 0 162 :3 0 73
:3 0 97 :3 0 161
:3 0 163 :3 0 73
:3 0 97 :3 0 161
:3 0 164 :3 0 2d
:3 0 48 :3 0 4c
:3 0 49 :3 0 4a
:3 0 165 :3 0 121
:3 0 1d :4 0 166
1 :8 0 1551 15
:3 0 9b :3 0 154c
154d 0 9c :3 0
154e 154f 0 1551
bdf 15a4 9d :3 0
15 :3 0 9b :3 0
1553 1554 0 9c
:3 0 1555 1556 0
1558 be2 155a be4
1559 1558 :2 0 15a2
9e :3 0 15 :3 0
9b :3 0 155d 155e
0 9c :3 0 155f
1560 0 159a 66
:3 0 67 :3 0 1562
1563 0 66 :3 0
77 :3 0 1565 1566
0 9f :2 0 2
:4 0 167 :2 0 79
:2 0 27 :2 0 be6
156b 156d :3 0 be9
1567 156f 1564 1571
1570 :2 0 69 :3 0
7a :3 0 1573 1574
0 6b :4 0 168
:4 0 6e :2 0 6f
:3 0 bed 1578 157a
:3 0 6e :2 0 82
:4 0 bf0 157c 157e
:3 0 6e :2 0 a2
:4 0 bf3 1580 1582
:3 0 6e :2 0 9c
:3 0 bf6 1584 1586
:3 0 6e :2 0 a3
:4 0 bf9 1588 158a
:3 0 6e :2 0 a4
:3 0 a5 :3 0 27
:2 0 a6 :2 0 bfc
158d 1591 c00 158c
1593 :3 0 c03 1575
1595 :2 0 1597 c06
1598 1572 1597 0
1599 c08 0 159a
c0a 159d :3 0 159d
0 159d 159c 159a
159b :6 0 159f 2c
:3 0 c0d 15a1 c0f
15a0 159f :2 0 15a2
c11 :2 0 15a4 0
15a4 15a3 1551 15a2
:6 0 171f 1 :3 0
15 :3 0 9b :3 0
15a6 15a7 0 15
:3 0 8f :2 0 d9
:3 0 15a9 15ab 0
c16 15aa 15ad :3 0
15ae :2 0 66 :3 0
67 :3 0 15b0 15b1
0 66 :3 0 77
:3 0 15b3 15b4 0
18 :2 0 2 :4 0
169 :2 0 79 :2 0
27 :2 0 c19 15b9
15bb :3 0 c1c 15b5
15bd 15b2 15bf 15be
:2 0 69 :3 0 7a
:3 0 15c1 15c2 0
6b :4 0 16a :4 0
6e :2 0 6f :3 0
c20 15c6 15c8 :3 0
6e :2 0 82 :4 0
c23 15ca 15cc :3 0
6e :2 0 83 :4 0
c26 15ce 15d0 :3 0
6e :2 0 73 :3 0
74 :3 0 84 :3 0
e4 :3 0 15d5 15d6
0 c29 15d4 15d8
75 :4 0 c2b 15d3
15db c2e 15d2 15dd
:3 0 6e :2 0 e5
:4 0 c31 15df 15e1
:3 0 6e :2 0 71
:4 0 c34 15e3 15e5
:3 0 6e :2 0 1d
:3 0 c37 15e7 15e9
:3 0 6e :2 0 71
:4 0 c3a 15eb 15ed
:3 0 c3d 15c3 15ef
:2 0 15f1 c40 15f2
15c0 15f1 0 15f3
c42 0 161d 66
:3 0 67 :3 0 15f4
15f5 0 66 :3 0
77 :3 0 15f7 15f8
0 27 :2 0 2
:4 0 16b :2 0 79
:2 0 27 :2 0 c44
15fd 15ff :3 0 c47
15f9 1601 15f6 1603
1602 :2 0 69 :3 0
7a :3 0 1605 1606
0 6b :4 0 16c
:4 0 6e :2 0 6f
:3 0 c4b 160a 160c
:3 0 c4e 1607 160e
:2 0 1610 c51 1611
1604 1610 0 1612
c53 0 161d 88
:3 0 e8 :3 0 1613
1614 0 84 :3 0
e4 :3 0 1616 1617
0 1d :3 0 8a
:3 0 c55 1615 161b
:2 0 161d c59 161e
15af 161d 0 161f
c5d 0 171f a4
:3 0 4c :3 0 27
:2 0 27 :2 0 c5f
1620 1624 8f :2 0
30 :4 0 c65 1626
1628 :3 0 1629 :2 0
73 :3 0 4a :3 0
30 :2 0 c68 162b
162e b2 :2 0 30
:2 0 c6d 1630 1632
:3 0 1633 :2 0 73
:3 0 48 :3 0 30
:2 0 c70 1635 1638
73 :3 0 b2 :2 0
4a :3 0 30 :2 0
c73 163a 163e c78
163b 1640 :3 0 1641
:2 0 1634 1643 1642
:2 0 1644 :2 0 162a
1646 1645 :2 0 66
:3 0 67 :3 0 1648
1649 0 66 :3 0
77 :3 0 164b 164c
0 18 :2 0 2
:4 0 16d :2 0 79
:2 0 27 :2 0 c7b
1651 1653 :3 0 c7e
164d 1655 164a 1657
1656 :2 0 69 :3 0
7a :3 0 1659 165a
0 6b :4 0 16e
:4 0 6e :2 0 6f
:3 0 c82 165e 1660
:3 0 6e :2 0 82
:4 0 c85 1662 1664
:3 0 6e :2 0 83
:4 0 c88 1666 1668
:3 0 6e :2 0 73
:3 0 74 :3 0 84
:3 0 16f :3 0 166d
166e 0 c8b 166c
1670 75 :4 0 c8d
166b 1673 c90 166a
1675 :3 0 6e :2 0
e5 :4 0 c93 1677
1679 :3 0 6e :2 0
71 :4 0 c96 167b
167d :3 0 6e :2 0
1d :3 0 c99 167f
1681 :3 0 6e :2 0
71 :4 0 c9c 1683
1685 :3 0 c9f 165b
1687 :2 0 1689 ca2
168a 1658 1689 0
168b ca4 0 16b5
66 :3 0 67 :3 0
168c 168d 0 66
:3 0 77 :3 0 168f
1690 0 27 :2 0
2 :4 0 170 :2 0
79 :2 0 27 :2 0
ca6 1695 1697 :3 0
ca9 1691 1699 168e
169b 169a :2 0 69
:3 0 7a :3 0 169d
169e 0 6b :4 0
171 :4 0 6e :2 0
6f :3 0 cad 16a2
16a4 :3 0 cb0 169f
16a6 :2 0 16a8 cb3
16a9 169c 16a8 0
16aa cb5 0 16b5
88 :3 0 e8 :3 0
16ab 16ac 0 84
:3 0 16f :3 0 16ae
16af 0 1d :3 0
8a :3 0 cb7 16ad
16b3 :2 0 16b5 cbb
16b6 1647 16b5 0
16b7 cbf 0 171f
172 :3 0 173 :3 0
174 :3 0 28 :3 0
42 :3 0 1e :3 0
175 :3 0 bd :3 0
1a :3 0 122 :3 0
121 :3 0 1d :4 0
176 1 :8 0 16ca
15 :3 0 9b :3 0
16c5 16c6 0 9c
:3 0 16c7 16c8 0
16ca cc1 171d 9d
:3 0 15 :3 0 9b
:3 0 16cc 16cd 0
9c :3 0 16ce 16cf
0 16d1 cc4 16d3
cc6 16d2 16d1 :2 0
171b 9e :3 0 15
:3 0 9b :3 0 16d6
16d7 0 9c :3 0
16d8 16d9 0 1713
66 :3 0 67 :3 0
16db 16dc 0 66
:3 0 77 :3 0 16de
16df 0 9f :2 0
2 :4 0 177 :2 0
79 :2 0 27 :2 0
cc8 16e4 16e6 :3 0
ccb 16e0 16e8 16dd
16ea 16e9 :2 0 69
:3 0 7a :3 0 16ec
16ed 0 6b :4 0
178 :4 0 6e :2 0
6f :3 0 ccf 16f1
16f3 :3 0 6e :2 0
82 :4 0 cd2 16f5
16f7 :3 0 6e :2 0
a2 :4 0 cd5 16f9
16fb :3 0 6e :2 0
9c :3 0 cd8 16fd
16ff :3 0 6e :2 0
a3 :4 0 cdb 1701
1703 :3 0 6e :2 0
a4 :3 0 a5 :3 0
27 :2 0 a6 :2 0
cde 1706 170a ce2
1705 170c :3 0 ce5
16ee 170e :2 0 1710
ce8 1711 16eb 1710
0 1712 cea 0
1713 cec 1716 :3 0
1716 0 1716 1715
1713 1714 :6 0 1718
2e :3 0 cef 171a
cf1 1719 1718 :2 0
171b cf3 :2 0 171d
0 171d 171c 16ca
171b :6 0 171f 1
:3 0 cf6 1963 2c
:3 0 a4 :3 0 2c
:3 0 6e :2 0 1d
:3 0 cfc 1723 1725
:3 0 27 :2 0 2a
:2 0 cff 1721 1729
1720 172a 0 1962
179 :3 0 2d :3 0
17a :3 0 17b :3 0
1d :4 0 17c 1
:8 0 1737 15 :3 0
9b :3 0 1732 1733
0 9c :3 0 1734
1735 0 1737 d03
178a 9d :3 0 15
:3 0 9b :3 0 1739
173a 0 9c :3 0
173b 173c 0 173e
d06 1740 d08 173f
173e :2 0 1788 9e
:3 0 15 :3 0 9b
:3 0 1743 1744 0
9c :3 0 1745 1746
0 1780 66 :3 0
67 :3 0 1748 1749
0 66 :3 0 77
:3 0 174b 174c 0
9f :2 0 2 :4 0
17d :2 0 79 :2 0
27 :2 0 d0a 1751
1753 :3 0 d0d 174d
1755 174a 1757 1756
:2 0 69 :3 0 7a
:3 0 1759 175a 0
6b :4 0 17e :4 0
6e :2 0 6f :3 0
d11 175e 1760 :3 0
6e :2 0 82 :4 0
d14 1762 1764 :3 0
6e :2 0 a2 :4 0
d17 1766 1768 :3 0
6e :2 0 9c :3 0
d1a 176a 176c :3 0
6e :2 0 a3 :4 0
d1d 176e 1770 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 d20 1773
1777 d24 1772 1779
:3 0 d27 175b 177b
:2 0 177d d2a 177e
1758 177d 0 177f
d2c 0 1780 d2e
1783 :3 0 1783 0
1783 1782 1780 1781
:6 0 1785 30 :3 0
d31 1787 d33 1786
1785 :2 0 1788 d35
:2 0 178a 0 178a
1789 1737 1788 :6 0
1962 1 :3 0 15
:3 0 9b :3 0 178c
178d 0 15 :3 0
8f :2 0 d9 :3 0
178f 1791 0 d3a
1790 1793 :3 0 1794
:2 0 66 :3 0 67
:3 0 1796 1797 0
66 :3 0 77 :3 0
1799 179a 0 18
:2 0 2 :4 0 17f
:2 0 79 :2 0 27
:2 0 d3d 179f 17a1
:3 0 d40 179b 17a3
1798 17a5 17a4 :2 0
69 :3 0 7a :3 0
17a7 17a8 0 6b
:4 0 180 :4 0 6e
:2 0 6f :3 0 d44
17ac 17ae :3 0 6e
:2 0 82 :4 0 d47
17b0 17b2 :3 0 6e
:2 0 83 :4 0 d4a
17b4 17b6 :3 0 6e
:2 0 73 :3 0 74
:3 0 84 :3 0 e4
:3 0 17bb 17bc 0
d4d 17ba 17be 75
:4 0 d4f 17b9 17c1
d52 17b8 17c3 :3 0
6e :2 0 e5 :4 0
d55 17c5 17c7 :3 0
6e :2 0 71 :4 0
d58 17c9 17cb :3 0
6e :2 0 1d :3 0
d5b 17cd 17cf :3 0
6e :2 0 71 :4 0
d5e 17d1 17d3 :3 0
d61 17a9 17d5 :2 0
17d7 d64 17d8 17a6
17d7 0 17d9 d66
0 1803 66 :3 0
67 :3 0 17da 17db
0 66 :3 0 77
:3 0 17dd 17de 0
27 :2 0 2 :4 0
181 :2 0 79 :2 0
27 :2 0 d68 17e3
17e5 :3 0 d6b 17df
17e7 17dc 17e9 17e8
:2 0 69 :3 0 7a
:3 0 17eb 17ec 0
6b :4 0 182 :4 0
6e :2 0 6f :3 0
d6f 17f0 17f2 :3 0
d72 17ed 17f4 :2 0
17f6 d75 17f7 17ea
17f6 0 17f8 d77
0 1803 88 :3 0
e8 :3 0 17f9 17fa
0 84 :3 0 e4
:3 0 17fc 17fd 0
1d :3 0 8a :3 0
d79 17fb 1801 :2 0
1803 d7d 1804 1795
1803 0 1805 d81
0 1962 11e :3 0
128 :3 0 a4 :3 0
1f :3 0 128 :3 0
a4 :3 0 11e :3 0
128 :3 0 a4 :3 0
183 :3 0 184 :3 0
37 :3 0 21 :3 0
44 :3 0 11f :3 0
120 :3 0 17 :3 0
121 :3 0 1d :3 0
122 :4 0 185 1
:8 0 1820 15 :3 0
9b :3 0 181b 181c
0 9c :3 0 181d
181e 0 1820 d83
1873 9d :3 0 15
:3 0 9b :3 0 1822
1823 0 9c :3 0
1824 1825 0 1827
d86 1829 d88 1828
1827 :2 0 1871 9e
:3 0 15 :3 0 9b
:3 0 182c 182d 0
9c :3 0 182e 182f
0 1869 66 :3 0
67 :3 0 1831 1832
0 66 :3 0 77
:3 0 1834 1835 0
9f :2 0 2 :4 0
186 :2 0 79 :2 0
27 :2 0 d8a 183a
183c :3 0 d8d 1836
183e 1833 1840 183f
:2 0 69 :3 0 7a
:3 0 1842 1843 0
6b :4 0 187 :4 0
6e :2 0 6f :3 0
d91 1847 1849 :3 0
6e :2 0 82 :4 0
d94 184b 184d :3 0
6e :2 0 a2 :4 0
d97 184f 1851 :3 0
6e :2 0 9c :3 0
d9a 1853 1855 :3 0
6e :2 0 a3 :4 0
d9d 1857 1859 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 da0 185c
1860 da4 185b 1862
:3 0 da7 1844 1864
:2 0 1866 daa 1867
1841 1866 0 1868
dac 0 1869 dae
186c :3 0 186c 0
186c 186b 1869 186a
:6 0 186e 32 :3 0
db1 1870 db3 186f
186e :2 0 1871 db5
:2 0 1873 0 1873
1872 1820 1871 :6 0
1962 1 :3 0 15
:3 0 9b :3 0 1875
1876 0 15 :3 0
8f :2 0 a7 :3 0
1878 187a 0 dba
1879 187c :3 0 187d
:2 0 73 :3 0 44
:3 0 30 :2 0 dbd
187f 1882 8f :2 0
27 :2 0 dc2 1884
1886 :3 0 1887 :2 0
187e 1889 1888 :2 0
66 :3 0 67 :3 0
188b 188c 0 66
:3 0 77 :3 0 188e
188f 0 18 :2 0
2 :4 0 188 :2 0
79 :2 0 27 :2 0
dc5 1894 1896 :3 0
dc8 1890 1898 188d
189a 1899 :2 0 69
:3 0 7a :3 0 189c
189d 0 6b :4 0
189 :4 0 6e :2 0
6f :3 0 dcc 18a1
18a3 :3 0 6e :2 0
82 :4 0 dcf 18a5
18a7 :3 0 6e :2 0
83 :4 0 dd2 18a9
18ab :3 0 6e :2 0
73 :3 0 74 :3 0
84 :3 0 149 :3 0
18b0 18b1 0 dd5
18af 18b3 75 :4 0
dd7 18ae 18b6 dda
18ad 18b8 :3 0 6e
:2 0 e5 :4 0 ddd
18ba 18bc :3 0 6e
:2 0 71 :4 0 de0
18be 18c0 :3 0 6e
:2 0 1d :3 0 de3
18c2 18c4 :3 0 6e
:2 0 71 :4 0 de6
18c6 18c8 :3 0 de9
189e 18ca :2 0 18cc
dec 18cd 189b 18cc
0 18ce dee 0
18f8 66 :3 0 67
:3 0 18cf 18d0 0
66 :3 0 77 :3 0
18d2 18d3 0 27
:2 0 2 :4 0 18a
:2 0 79 :2 0 27
:2 0 df0 18d8 18da
:3 0 df3 18d4 18dc
18d1 18de 18dd :2 0
69 :3 0 7a :3 0
18e0 18e1 0 6b
:4 0 18b :4 0 6e
:2 0 6f :3 0 df7
18e5 18e7 :3 0 dfa
18e2 18e9 :2 0 18eb
dfd 18ec 18df 18eb
0 18ed dff 0
18f8 88 :3 0 e8
:3 0 18ee 18ef 0
84 :3 0 149 :3 0
18f1 18f2 0 1d
:3 0 8a :3 0 e01
18f0 18f6 :2 0 18f8
e05 18f9 188a 18f8
0 18fa e09 0
1962 172 :3 0 173
:3 0 174 :3 0 28
:3 0 42 :3 0 1e
:3 0 175 :3 0 bd
:3 0 1a :3 0 122
:3 0 121 :3 0 1d
:4 0 18c 1 :8 0
190d 15 :3 0 9b
:3 0 1908 1909 0
9c :3 0 190a 190b
0 190d e0b 1960
9d :3 0 15 :3 0
9b :3 0 190f 1910
0 9c :3 0 1911
1912 0 1914 e0e
1916 e10 1915 1914
:2 0 195e 9e :3 0
15 :3 0 9b :3 0
1919 191a 0 9c
:3 0 191b 191c 0
1956 66 :3 0 67
:3 0 191e 191f 0
66 :3 0 77 :3 0
1921 1922 0 9f
:2 0 2 :4 0 18d
:2 0 79 :2 0 27
:2 0 e12 1927 1929
:3 0 e15 1923 192b
1920 192d 192c :2 0
69 :3 0 7a :3 0
192f 1930 0 6b
:4 0 18e :4 0 6e
:2 0 6f :3 0 e19
1934 1936 :3 0 6e
:2 0 82 :4 0 e1c
1938 193a :3 0 6e
:2 0 a2 :4 0 e1f
193c 193e :3 0 6e
:2 0 9c :3 0 e22
1940 1942 :3 0 6e
:2 0 a3 :4 0 e25
1944 1946 :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 e28 1949 194d
e2c 1948 194f :3 0
e2f 1931 1951 :2 0
1953 e32 1954 192e
1953 0 1955 e34
0 1956 e36 1959
:3 0 1959 0 1959
1958 1956 1957 :6 0
195b 34 :3 0 e39
195d e3b 195c 195b
:2 0 195e e3d :2 0
1960 0 1960 195f
190d 195e :6 0 1962
1 :3 0 e40 1964
1529 171f 0 1965
0 1962 0 1965
e47 0 1c32 21
:3 0 21 :3 0 1966
1967 0 1c32 73
:3 0 46 :3 0 30
:2 0 e4a 1969 196c
8f :2 0 27 :2 0
e4f 196e 1970 :3 0
97 :3 0 18f :3 0
128 :3 0 a4 :3 0
21 :3 0 97 :3 0
190 :3 0 1f :3 0
21 :3 0 99 :4 0
191 1 :8 0 1982
15 :3 0 9b :3 0
197d 197e 0 9c
:3 0 197f 1980 0
1982 e52 19d5 9d
:3 0 15 :3 0 9b
:3 0 1984 1985 0
9c :3 0 1986 1987
0 1989 e55 198b
e57 198a 1989 :2 0
19d3 9e :3 0 15
:3 0 9b :3 0 198e
198f 0 9c :3 0
1990 1991 0 19cb
66 :3 0 67 :3 0
1993 1994 0 66
:3 0 77 :3 0 1996
1997 0 9f :2 0
2 :4 0 192 :2 0
79 :2 0 27 :2 0
e59 199c 199e :3 0
e5c 1998 19a0 1995
19a2 19a1 :2 0 69
:3 0 7a :3 0 19a4
19a5 0 6b :4 0
193 :4 0 6e :2 0
6f :3 0 e60 19a9
19ab :3 0 6e :2 0
82 :4 0 e63 19ad
19af :3 0 6e :2 0
a2 :4 0 e66 19b1
19b3 :3 0 6e :2 0
9c :3 0 e69 19b5
19b7 :3 0 6e :2 0
a3 :4 0 e6c 19b9
19bb :3 0 6e :2 0
a4 :3 0 a5 :3 0
27 :2 0 a6 :2 0
e6f 19be 19c2 e73
19bd 19c4 :3 0 e76
19a6 19c6 :2 0 19c8
e79 19c9 19a3 19c8
0 19ca e7b 0
19cb e7d 19ce :3 0
19ce 0 19ce 19cd
19cb 19cc :6 0 19d0
36 :3 0 e80 19d2
e82 19d1 19d0 :2 0
19d3 e84 :2 0 19d5
0 19d5 19d4 1982
19d3 :6 0 1b6a 1
:3 0 128 :3 0 a4
:3 0 1f :3 0 128
:3 0 a4 :3 0 21
:3 0 1f :3 0 21
:3 0 21 :3 0 99
:4 0 194 1 :8 0
19e7 15 :3 0 9b
:3 0 19e2 19e3 0
9c :3 0 19e4 19e5
0 19e7 e87 1a3a
9d :3 0 15 :3 0
9b :3 0 19e9 19ea
0 9c :3 0 19eb
19ec 0 19ee e8a
19f0 e8c 19ef 19ee
:2 0 1a38 9e :3 0
15 :3 0 9b :3 0
19f3 19f4 0 9c
:3 0 19f5 19f6 0
1a30 66 :3 0 67
:3 0 19f8 19f9 0
66 :3 0 77 :3 0
19fb 19fc 0 9f
:2 0 2 :4 0 195
:2 0 79 :2 0 27
:2 0 e8e 1a01 1a03
:3 0 e91 19fd 1a05
19fa 1a07 1a06 :2 0
69 :3 0 7a :3 0
1a09 1a0a 0 6b
:4 0 196 :4 0 6e
:2 0 6f :3 0 e95
1a0e 1a10 :3 0 6e
:2 0 82 :4 0 e98
1a12 1a14 :3 0 6e
:2 0 a2 :4 0 e9b
1a16 1a18 :3 0 6e
:2 0 9c :3 0 e9e
1a1a 1a1c :3 0 6e
:2 0 a3 :4 0 ea1
1a1e 1a20 :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 ea4 1a23 1a27
ea8 1a22 1a29 :3 0
eab 1a0b 1a2b :2 0
1a2d eae 1a2e 1a08
1a2d 0 1a2f eb0
0 1a30 eb2 1a33
:3 0 1a33 0 1a33
1a32 1a30 1a31 :6 0
1a35 38 :3 0 eb5
1a37 eb7 1a36 1a35
:2 0 1a38 eb9 :2 0
1a3a 0 1a3a 1a39
19e7 1a38 :6 0 1b6a
1 :3 0 45 :3 0
197 :2 0 1a3c 1a3d
0 1b6a 22 :3 0
6 :3 0 1a3f 1a40
0 1b6a 73 :3 0
33 :3 0 30 :2 0
ebc 1a42 1a45 8f
:2 0 30 :2 0 ec1
1a47 1a49 :3 0 3f
:3 0 10f :2 0 1a4b
1a4c 0 1a4e ec4
1a4f 1a4a 1a4e 0
1a50 ec6 0 1b6a
62 :3 0 198 :3 0
28 :3 0 17 :3 0
1a :3 0 1d :3 0
4e :3 0 32 :3 0
23 :3 0 23 :3 0
23 :3 0 23 :3 0
21 :3 0 22 :3 0
41 :3 0 37 :3 0
35 :3 0 38 :3 0
39 :3 0 34 :3 0
45 :3 0 26 :3 0
33 :3 0 31 :3 0
3b :3 0 36 :3 0
3f :3 0 3d :3 0
43 :3 0 1c :3 0
30 :2 0 30 :2 0
50 :2 0 50 :2 0
ec8 1a52 1a73 1a51
1a74 0 1b6a 62
:3 0 8f :2 0 30
:2 0 eeb 1a77 1a79
:3 0 73 :3 0 33
:3 0 30 :2 0 eee
1a7b 1a7e 8f :2 0
30 :2 0 ef3 1a80
1a82 :3 0 199 :3 0
19a :3 0 19b :3 0
1d :4 0 19c 1
:8 0 1a9e 19d :4 0
1a89 :3 0 8a :3 0
8f :2 0 ef8 1a8c
1a8d :3 0 15 :3 0
9b :3 0 1a8f 1a90
0 30 :2 0 1a91
1a92 0 1a94 efb
1a9b 15 :3 0 9b
:3 0 1a95 1a96 0
10d :2 0 1a97 1a98
0 1a9a efd 1a9c
1a8e 1a94 0 1a9d
0 1a9a 0 1a9d
eff 0 1a9e f02
1af1 9d :3 0 15
:3 0 9b :3 0 1aa0
1aa1 0 9c :3 0
1aa2 1aa3 0 1aa5
f05 1aa7 f07 1aa6
1aa5 :2 0 1aef 9e
:3 0 15 :3 0 9b
:3 0 1aaa 1aab 0
9c :3 0 1aac 1aad
0 1ae7 66 :3 0
67 :3 0 1aaf 1ab0
0 66 :3 0 77
:3 0 1ab2 1ab3 0
9f :2 0 2 :4 0
19e :2 0 79 :2 0
27 :2 0 f09 1ab8
1aba :3 0 f0c 1ab4
1abc 1ab1 1abe 1abd
:2 0 69 :3 0 7a
:3 0 1ac0 1ac1 0
6b :4 0 19f :4 0
6e :2 0 6f :3 0
f10 1ac5 1ac7 :3 0
6e :2 0 82 :4 0
f13 1ac9 1acb :3 0
6e :2 0 a2 :4 0
f16 1acd 1acf :3 0
6e :2 0 9c :3 0
f19 1ad1 1ad3 :3 0
6e :2 0 a3 :4 0
f1c 1ad5 1ad7 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 f1f 1ada
1ade f23 1ad9 1ae0
:3 0 f26 1ac2 1ae2
:2 0 1ae4 f29 1ae5
1abf 1ae4 0 1ae6
f2b 0 1ae7 f2d
1aea :3 0 1aea 0
1aea 1ae9 1ae7 1ae8
:6 0 1aec 3a :3 0
f30 1aee f32 1aed
1aec :2 0 1aef f34
:2 0 1af1 0 1af1
1af0 1a9e 1aef :6 0
1af3 1 :3 0 f37
1b64 199 :3 0 19a
:3 0 19b :3 0 1d
:4 0 1a0 1 :8 0
1b0e 19d :4 0 1af9
:3 0 8a :3 0 8f
:2 0 f3b 1afc 1afd
:3 0 15 :3 0 9b
:3 0 1aff 1b00 0
30 :2 0 1b01 1b02
0 1b04 f3e 1b0b
15 :3 0 9b :3 0
1b05 1b06 0 10d
:2 0 1b07 1b08 0
1b0a f40 1b0c 1afe
1b04 0 1b0d 0
1b0a 0 1b0d f42
0 1b0e f45 1b61
9d :3 0 15 :3 0
9b :3 0 1b10 1b11
0 9c :3 0 1b12
1b13 0 1b15 f48
1b17 f4a 1b16 1b15
:2 0 1b5f 9e :3 0
15 :3 0 9b :3 0
1b1a 1b1b 0 9c
:3 0 1b1c 1b1d 0
1b57 66 :3 0 67
:3 0 1b1f 1b20 0
66 :3 0 77 :3 0
1b22 1b23 0 9f
:2 0 2 :4 0 1a1
:2 0 79 :2 0 27
:2 0 f4c 1b28 1b2a
:3 0 f4f 1b24 1b2c
1b21 1b2e 1b2d :2 0
69 :3 0 7a :3 0
1b30 1b31 0 6b
:4 0 1a2 :4 0 6e
:2 0 6f :3 0 f53
1b35 1b37 :3 0 6e
:2 0 82 :4 0 f56
1b39 1b3b :3 0 6e
:2 0 a2 :4 0 f59
1b3d 1b3f :3 0 6e
:2 0 9c :3 0 f5c
1b41 1b43 :3 0 6e
:2 0 a3 :4 0 f5f
1b45 1b47 :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 f62 1b4a 1b4e
f66 1b49 1b50 :3 0
f69 1b32 1b52 :2 0
1b54 f6c 1b55 1b2f
1b54 0 1b56 f6e
0 1b57 f70 1b5a
:3 0 1b5a 0 1b5a
1b59 1b57 1b58 :6 0
1b5c 3c :3 0 f73
1b5e f75 1b5d 1b5c
:2 0 1b5f f77 :2 0
1b61 0 1b61 1b60
1b0e 1b5f :6 0 1b63
1 :3 0 f7a 1b65
1a83 1af3 0 1b66
0 1b63 0 1b66
f7c 0 1b67 f7f
1b68 1a7a 1b67 0
1b69 f81 0 1b6a
f83 1c2f 21 :3 0
1f :3 0 1b6b 1b6c
0 1c2e 22 :3 0
6 :3 0 1b6e 1b6f
0 1c2e a4 :3 0
26 :3 0 27 :2 0
27 :2 0 f8b 1b71
1b75 8f :2 0 27
:4 0 f91 1b77 1b79
:3 0 45 :3 0 84
:3 0 1a3 :3 0 1b7c
1b7d 0 1b7b 1b7e
0 1b80 f94 1b8f
45 :3 0 115 :3 0
73 :3 0 3a :3 0
30 :2 0 f96 1b83
1b86 84 :3 0 118
:3 0 1b88 1b89 0
f99 1b82 1b8b 1b81
1b8c 0 1b8e f9c
1b90 1b7a 1b80 0
1b91 0 1b8e 0
1b91 f9e 0 1c2e
62 :3 0 198 :3 0
28 :3 0 17 :3 0
1a :3 0 1d :3 0
4e :3 0 32 :3 0
23 :3 0 23 :3 0
23 :3 0 23 :3 0
21 :3 0 22 :3 0
41 :3 0 37 :3 0
35 :3 0 38 :3 0
39 :3 0 34 :3 0
45 :3 0 26 :3 0
33 :3 0 31 :3 0
3b :3 0 36 :3 0
3f :3 0 3d :3 0
43 :3 0 1c :3 0
30 :2 0 30 :2 0
50 :2 0 50 :2 0
fa1 1b93 1bb4 1b92
1bb5 0 1c2e 62
:3 0 8f :2 0 30
:2 0 fc4 1bb8 1bba
:3 0 199 :3 0 19a
:3 0 19b :3 0 1d
:4 0 19c 1 :8 0
1bd6 19d :4 0 1bc1
:3 0 8a :3 0 8f
:2 0 fc9 1bc4 1bc5
:3 0 15 :3 0 9b
:3 0 1bc7 1bc8 0
30 :2 0 1bc9 1bca
0 1bcc fcc 1bd3
15 :3 0 9b :3 0
1bcd 1bce 0 10d
:2 0 1bcf 1bd0 0
1bd2 fce 1bd4 1bc6
1bcc 0 1bd5 0
1bd2 0 1bd5 fd0
0 1bd6 fd3 1c29
9d :3 0 15 :3 0
9b :3 0 1bd8 1bd9
0 9c :3 0 1bda
1bdb 0 1bdd fd6
1bdf fd8 1bde 1bdd
:2 0 1c27 9e :3 0
15 :3 0 9b :3 0
1be2 1be3 0 9c
:3 0 1be4 1be5 0
1c1f 66 :3 0 67
:3 0 1be7 1be8 0
66 :3 0 77 :3 0
1bea 1beb 0 9f
:2 0 2 :4 0 1a4
:2 0 79 :2 0 27
:2 0 fda 1bf0 1bf2
:3 0 fdd 1bec 1bf4
1be9 1bf6 1bf5 :2 0
69 :3 0 7a :3 0
1bf8 1bf9 0 6b
:4 0 1a5 :4 0 6e
:2 0 6f :3 0 fe1
1bfd 1bff :3 0 6e
:2 0 82 :4 0 fe4
1c01 1c03 :3 0 6e
:2 0 a2 :4 0 fe7
1c05 1c07 :3 0 6e
:2 0 9c :3 0 fea
1c09 1c0b :3 0 6e
:2 0 a3 :4 0 fed
1c0d 1c0f :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 ff0 1c12 1c16
ff4 1c11 1c18 :3 0
ff7 1bfa 1c1a :2 0
1c1c ffa 1c1d 1bf7
1c1c 0 1c1e ffc
0 1c1f ffe 1c22
:3 0 1c22 0 1c22
1c21 1c1f 1c20 :6 0
1c24 3e :3 0 1001
1c26 1003 1c25 1c24
:2 0 1c27 1005 :2 0
1c29 0 1c29 1c28
1bd6 1c27 :6 0 1c2b
1 :3 0 1008 1c2c
1bbb 1c2b 0 1c2d
100a 0 1c2e 100c
1c30 1971 1b6a 0
1c31 0 1c2e 0
1c31 1012 0 1c32
1015 2624 73 :3 0
1a6 :3 0 29 :3 0
1022 1c34 1c36 30
:2 0 1024 1c33 1c39
a8 :2 0 18 :2 0
1029 1c3b 1c3d :3 0
64 :3 0 30 :2 0
1c3f 1c40 0 1c42
102c 1c5a 64 :3 0
1a7 :3 0 73 :3 0
1a6 :3 0 29 :3 0
102e 1c46 1c48 30
:2 0 1030 1c45 1c4b
79 :2 0 27 :2 0
1033 1c4d 1c4f :3 0
1c50 :2 0 1a8 :2 0
b5 :2 0 1036 1c52
1c54 :3 0 1039 1c44
1c56 1c43 1c57 0
1c59 103b 1c5b 1c3e
1c42 0 1c5c 0
1c59 0 1c5c 103d
0 2623 64 :3 0
b2 :2 0 30 :2 0
1042 1c5e 1c60 :3 0
2f :3 0 30 :2 0
1c62 1c63 0 2620
1a9 :3 0 73 :3 0
2f :3 0 30 :2 0
1045 1c66 1c69 64
:3 0 a8 :2 0 1aa
:3 0 104a 1c6c 1c6e
:3 0 1c6f 261f 46
:3 0 30 :2 0 1c71
1c72 0 261d 1d
:3 0 b3 :3 0 b4
:3 0 1c75 1c76 0
18 :2 0 18 :2 0
73 :3 0 2f :3 0
30 :2 0 104d 1c7a
1c7d 111 :2 0 b5
:2 0 1050 1c7f 1c81
:3 0 79 :2 0 27
:2 0 1053 1c83 1c85
:3 0 29 :3 0 1056
1c77 1c88 1c74 1c89
0 261d 26 :3 0
b3 :3 0 b4 :3 0
1c8c 1c8d 0 27
:2 0 27 :2 0 73
:3 0 2f :3 0 30
:2 0 105b 1c91 1c94
111 :2 0 b5 :2 0
105e 1c96 1c98 :3 0
79 :2 0 18 :2 0
1061 1c9a 1c9c :3 0
79 :2 0 27 :2 0
1064 1c9e 1ca0 :3 0
29 :3 0 1067 1c8e
1ca3 1c8b 1ca4 0
261d 1f :3 0 5
:3 0 1ca6 1ca7 0
261d 46 :3 0 30
:2 0 1ca9 1caa 0
261d 44 :3 0 30
:2 0 1cac 1cad 0
261d 21 :3 0 1f
:3 0 1caf 1cb0 0
261d 1ab :3 0 11d
:3 0 1ac :3 0 44
:3 0 21 :3 0 11f
:3 0 b8 :3 0 17
:3 0 120 :3 0 17
:3 0 121 :3 0 1d
:3 0 122 :3 0 26
:4 0 1ad 1 :8 0
1cc6 15 :3 0 9b
:3 0 1cc1 1cc2 0
9c :3 0 1cc3 1cc4
0 1cc6 106c 1d19
9d :3 0 15 :3 0
9b :3 0 1cc8 1cc9
0 9c :3 0 1cca
1ccb 0 1ccd 106f
1ccf 1071 1cce 1ccd
:2 0 1d17 9e :3 0
15 :3 0 9b :3 0
1cd2 1cd3 0 9c
:3 0 1cd4 1cd5 0
1d0f 66 :3 0 67
:3 0 1cd7 1cd8 0
66 :3 0 77 :3 0
1cda 1cdb 0 9f
:2 0 2 :4 0 1ae
:2 0 79 :2 0 27
:2 0 1073 1ce0 1ce2
:3 0 1076 1cdc 1ce4
1cd9 1ce6 1ce5 :2 0
69 :3 0 7a :3 0
1ce8 1ce9 0 6b
:4 0 1af :4 0 6e
:2 0 6f :3 0 107a
1ced 1cef :3 0 6e
:2 0 82 :4 0 107d
1cf1 1cf3 :3 0 6e
:2 0 a2 :4 0 1080
1cf5 1cf7 :3 0 6e
:2 0 9c :3 0 1083
1cf9 1cfb :3 0 6e
:2 0 a3 :4 0 1086
1cfd 1cff :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 1089 1d02 1d06
108d 1d01 1d08 :3 0
1090 1cea 1d0a :2 0
1d0c 1093 1d0d 1ce7
1d0c 0 1d0e 1095
0 1d0f 1097 1d12
:3 0 1d12 0 1d12
1d11 1d0f 1d10 :6 0
1d14 41 :3 0 109a
1d16 109c 1d15 1d14
:2 0 1d17 109e :2 0
1d19 0 1d19 1d18
1cc6 1d17 :6 0 261d
40 :3 0 73 :3 0
44 :3 0 30 :2 0
10a1 1d1b 1d1e b2
:2 0 30 :2 0 10a6
1d20 1d22 :3 0 1d23
:2 0 21 :3 0 96
:2 0 10a9 1d26 1d27
:3 0 1d28 :2 0 1d29
:2 0 1d24 1d2b 1d2a
:2 0 46 :3 0 27
:2 0 1d2d 1d2e 0
1d41 97 :3 0 f2
:3 0 1d30 1d31 0
1f :3 0 21 :3 0
10ab 1d32 1d35 a8
:2 0 30 :2 0 10b0
1d37 1d39 :3 0 1f
:3 0 21 :3 0 1d3b
1d3c 0 1d3e 10b3
1d3f 1d3a 1d3e 0
1d40 10b5 0 1d41
10b7 1d42 1d2c 1d41
0 1d43 10ba 0
261d a4 :3 0 26
:3 0 27 :2 0 27
:2 0 10bc 1d44 1d48
8f :2 0 27 :4 0
10c2 1d4a 1d4c :3 0
3a :3 0 84 :3 0
1a3 :3 0 1d4f 1d50
0 1d4e 1d51 0
1d7f 73 :3 0 1a6
:3 0 2c :3 0 10c5
1d54 1d56 30 :2 0
10c7 1d53 1d59 8f
:2 0 30 :2 0 10cc
1d5b 1d5d :3 0 2c
:3 0 1d :3 0 1d5f
1d60 0 1d62 10cf
1d7c 2c :3 0 a4
:3 0 2c :3 0 6e
:2 0 1b0 :4 0 10d1
1d66 1d68 :3 0 27
:2 0 2a :2 0 10d4
1d64 1d6c 1d63 1d6d
0 1d7b 2c :3 0
a4 :3 0 2c :3 0
6e :2 0 1d :3 0
10d8 1d72 1d74 :3 0
27 :2 0 2a :2 0
10db 1d70 1d78 1d6f
1d79 0 1d7b 10df
1d7d 1d5e 1d62 0
1d7e 0 1d7b 0
1d7e 10e2 0 1d7f
10e5 1dad 73 :3 0
1a6 :3 0 2b :3 0
10e8 1d81 1d83 30
:2 0 10ea 1d80 1d86
8f :2 0 30 :2 0
10ef 1d88 1d8a :3 0
2b :3 0 1d :3 0
1d8c 1d8d 0 1d8f
10f2 1da9 2b :3 0
a4 :3 0 2b :3 0
6e :2 0 1b0 :4 0
10f4 1d93 1d95 :3 0
27 :2 0 2a :2 0
10f7 1d91 1d99 1d90
1d9a 0 1da8 2b
:3 0 a4 :3 0 2b
:3 0 6e :2 0 1d
:3 0 10fb 1d9f 1da1
:3 0 27 :2 0 2a
:2 0 10fe 1d9d 1da5
1d9c 1da6 0 1da8
1102 1daa 1d8b 1d8f
0 1dab 0 1da8
0 1dab 1105 0
1dac 1108 1dae 1d4d
1d7f 0 1daf 0
1dac 0 1daf 110a
0 261d 33 :3 0
30 :2 0 1db0 1db1
0 261d 44 :3 0
30 :2 0 1db3 1db4
0 261d 73 :3 0
46 :3 0 30 :2 0
110d 1db6 1db9 8f
:2 0 27 :2 0 1112
1dbb 1dbd :3 0 63
:3 0 8f :2 0 27
:2 0 1117 1dc0 1dc2
:3 0 a4 :3 0 26
:3 0 27 :2 0 27
:2 0 111a 1dc4 1dc8
8f :2 0 30 :4 0
1120 1dca 1dcc :3 0
126 :3 0 127 :3 0
128 :3 0 a4 :3 0
1f :3 0 128 :3 0
a4 :3 0 127 :3 0
128 :3 0 a4 :3 0
22 :3 0 128 :3 0
a4 :3 0 129 :3 0
12a :3 0 37 :3 0
12a :3 0 19 :3 0
21 :3 0 44 :3 0
47 :3 0 11f :3 0
120 :3 0 17 :3 0
121 :3 0 1d :3 0
122 :3 0 12b :4 0
1b1 1 :8 0 1df0
15 :3 0 9b :3 0
1deb 1dec 0 9c
:3 0 1ded 1dee 0
1df0 1123 1e43 9d
:3 0 15 :3 0 9b
:3 0 1df2 1df3 0
9c :3 0 1df4 1df5
0 1df7 1126 1df9
1128 1df8 1df7 :2 0
1e41 9e :3 0 15
:3 0 9b :3 0 1dfc
1dfd 0 9c :3 0
1dfe 1dff 0 1e39
66 :3 0 67 :3 0
1e01 1e02 0 66
:3 0 77 :3 0 1e04
1e05 0 9f :2 0
2 :4 0 1b2 :2 0
79 :2 0 27 :2 0
112a 1e0a 1e0c :3 0
112d 1e06 1e0e 1e03
1e10 1e0f :2 0 69
:3 0 7a :3 0 1e12
1e13 0 6b :4 0
1b3 :4 0 6e :2 0
6f :3 0 1131 1e17
1e19 :3 0 6e :2 0
82 :4 0 1134 1e1b
1e1d :3 0 6e :2 0
a2 :4 0 1137 1e1f
1e21 :3 0 6e :2 0
9c :3 0 113a 1e23
1e25 :3 0 6e :2 0
a3 :4 0 113d 1e27
1e29 :3 0 6e :2 0
a4 :3 0 a5 :3 0
27 :2 0 a6 :2 0
1140 1e2c 1e30 1144
1e2b 1e32 :3 0 1147
1e14 1e34 :2 0 1e36
114a 1e37 1e11 1e36
0 1e38 114c 0
1e39 114e 1e3c :3 0
1e3c 0 1e3c 1e3b
1e39 1e3a :6 0 1e3e
43 :3 0 1151 1e40
1153 1e3f 1e3e :2 0
1e41 1155 :2 0 1e43
0 1e43 1e42 1df0
1e41 :6 0 1f76 40
:3 0 15 :3 0 9b
:3 0 1e45 1e46 0
15 :3 0 8f :2 0
a7 :3 0 1e48 1e4a
0 115a 1e49 1e4c
:3 0 1e4d :2 0 46
:3 0 27 :2 0 1e4f
1e50 0 1ee6 73
:3 0 47 :3 0 30
:2 0 115d 1e52 1e55
8f :2 0 30 :2 0
1162 1e57 1e59 :3 0
1e5a :2 0 73 :3 0
44 :3 0 30 :2 0
1165 1e5c 1e5f 8f
:2 0 27 :2 0 116a
1e61 1e63 :3 0 1e64
:2 0 1e5b 1e66 1e65
:2 0 126 :3 0 12f
:3 0 128 :3 0 a4
:3 0 1f :3 0 128
:3 0 a4 :3 0 12f
:3 0 128 :3 0 a4
:3 0 22 :3 0 128
:3 0 a4 :3 0 130
:3 0 131 :3 0 37
:3 0 19 :3 0 21
:3 0 44 :3 0 11f
:3 0 120 :3 0 17
:3 0 121 :3 0 1d
:3 0 122 :3 0 12b
:4 0 1b4 1 :8 0
1e88 15 :3 0 9b
:3 0 1e83 1e84 0
9c :3 0 1e85 1e86
0 1e88 116d 1edb
9d :3 0 15 :3 0
9b :3 0 1e8a 1e8b
0 9c :3 0 1e8c
1e8d 0 1e8f 1170
1e91 1172 1e90 1e8f
:2 0 1ed9 9e :3 0
15 :3 0 9b :3 0
1e94 1e95 0 9c
:3 0 1e96 1e97 0
1ed1 66 :3 0 67
:3 0 1e99 1e9a 0
66 :3 0 77 :3 0
1e9c 1e9d 0 9f
:2 0 2 :4 0 1b5
:2 0 79 :2 0 27
:2 0 1174 1ea2 1ea4
:3 0 1177 1e9e 1ea6
1e9b 1ea8 1ea7 :2 0
69 :3 0 7a :3 0
1eaa 1eab 0 6b
:4 0 1b6 :4 0 6e
:2 0 6f :3 0 117b
1eaf 1eb1 :3 0 6e
:2 0 82 :4 0 117e
1eb3 1eb5 :3 0 6e
:2 0 a2 :4 0 1181
1eb7 1eb9 :3 0 6e
:2 0 9c :3 0 1184
1ebb 1ebd :3 0 6e
:2 0 a3 :4 0 1187
1ebf 1ec1 :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 118a 1ec4 1ec8
118e 1ec3 1eca :3 0
1191 1eac 1ecc :2 0
1ece 1194 1ecf 1ea9
1ece 0 1ed0 1196
0 1ed1 1198 1ed4
:3 0 1ed4 0 1ed4
1ed3 1ed1 1ed2 :6 0
1ed6 45 :3 0 119b
1ed8 119d 1ed7 1ed6
:2 0 1ed9 119f :2 0
1edb 0 1edb 1eda
1e88 1ed9 :6 0 1edd
40 :3 0 11a2 1ee2
33 :3 0 10e :2 0
1ede 1edf 0 1ee1
11a4 1ee3 1e67 1edd
0 1ee4 0 1ee1
0 1ee4 11a6 0
1ee6 ae :3 0 11a9
1f74 15 :3 0 9b
:3 0 1ee7 1ee8 0
15 :3 0 8f :2 0
d9 :3 0 1eea 1eec
0 11ae 1eeb 1eee
:3 0 1eef :2 0 73
:3 0 44 :3 0 30
:2 0 11b1 1ef1 1ef4
8f :2 0 27 :2 0
11b6 1ef6 1ef8 :3 0
1ef9 :2 0 1ef0 1efb
1efa :2 0 126 :3 0
12f :3 0 128 :3 0
a4 :3 0 1f :3 0
128 :3 0 a4 :3 0
12f :3 0 128 :3 0
a4 :3 0 22 :3 0
128 :3 0 a4 :3 0
130 :3 0 131 :3 0
37 :3 0 19 :3 0
21 :3 0 44 :3 0
11f :3 0 120 :3 0
17 :3 0 121 :3 0
1d :3 0 122 :3 0
12b :4 0 1b7 1
:8 0 1f1d 15 :3 0
9b :3 0 1f18 1f19
0 9c :3 0 1f1a
1f1b 0 1f1d 11b9
1f70 9d :3 0 15
:3 0 9b :3 0 1f1f
1f20 0 9c :3 0
1f21 1f22 0 1f24
11bc 1f26 11be 1f25
1f24 :2 0 1f6e 9e
:3 0 15 :3 0 9b
:3 0 1f29 1f2a 0
9c :3 0 1f2b 1f2c
0 1f66 66 :3 0
67 :3 0 1f2e 1f2f
0 66 :3 0 77
:3 0 1f31 1f32 0
9f :2 0 2 :4 0
1b8 :2 0 79 :2 0
27 :2 0 11c0 1f37
1f39 :3 0 11c3 1f33
1f3b 1f30 1f3d 1f3c
:2 0 69 :3 0 7a
:3 0 1f3f 1f40 0
6b :4 0 1b9 :4 0
6e :2 0 6f :3 0
11c7 1f44 1f46 :3 0
6e :2 0 82 :4 0
11ca 1f48 1f4a :3 0
6e :2 0 a2 :4 0
11cd 1f4c 1f4e :3 0
6e :2 0 9c :3 0
11d0 1f50 1f52 :3 0
6e :2 0 a3 :4 0
11d3 1f54 1f56 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 11d6 1f59
1f5d 11da 1f58 1f5f
:3 0 11dd 1f41 1f61
:2 0 1f63 11e0 1f64
1f3e 1f63 0 1f65
11e2 0 1f66 11e4
1f69 :3 0 1f69 0
1f69 1f68 1f66 1f67
:6 0 1f6b 47 :3 0
11e7 1f6d 11e9 1f6c
1f6b :2 0 1f6e 11eb
:2 0 1f70 0 1f70
1f6f 1f1d 1f6e :6 0
1f72 40 :3 0 11ee
1f73 1efc 1f72 0
1f75 1e4e 1ee6 0
1f75 11f0 0 1f76
11f3 1fef 126 :3 0
12f :3 0 128 :3 0
a4 :3 0 1f :3 0
128 :3 0 a4 :3 0
12f :3 0 128 :3 0
a4 :3 0 22 :3 0
128 :3 0 a4 :3 0
130 :3 0 131 :3 0
37 :3 0 19 :3 0
21 :3 0 44 :3 0
11f :3 0 b8 :3 0
17 :3 0 120 :3 0
17 :3 0 121 :3 0
1d :3 0 122 :3 0
12b :4 0 1ba 1
:8 0 1f99 15 :3 0
9b :3 0 1f94 1f95
0 9c :3 0 1f96
1f97 0 1f99 11f6
1fec 9d :3 0 15
:3 0 9b :3 0 1f9b
1f9c 0 9c :3 0
1f9d 1f9e 0 1fa0
11f9 1fa2 11fb 1fa1
1fa0 :2 0 1fea 9e
:3 0 15 :3 0 9b
:3 0 1fa5 1fa6 0
9c :3 0 1fa7 1fa8
0 1fe2 66 :3 0
67 :3 0 1faa 1fab
0 66 :3 0 77
:3 0 1fad 1fae 0
9f :2 0 2 :4 0
1bb :2 0 79 :2 0
27 :2 0 11fd 1fb3
1fb5 :3 0 1200 1faf
1fb7 1fac 1fb9 1fb8
:2 0 69 :3 0 7a
:3 0 1fbb 1fbc 0
6b :4 0 1bc :4 0
6e :2 0 6f :3 0
1204 1fc0 1fc2 :3 0
6e :2 0 82 :4 0
1207 1fc4 1fc6 :3 0
6e :2 0 a2 :4 0
120a 1fc8 1fca :3 0
6e :2 0 9c :3 0
120d 1fcc 1fce :3 0
6e :2 0 a3 :4 0
1210 1fd0 1fd2 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 1213 1fd5
1fd9 1217 1fd4 1fdb
:3 0 121a 1fbd 1fdd
:2 0 1fdf 121d 1fe0
1fba 1fdf 0 1fe1
121f 0 1fe2 1221
1fe5 :3 0 1fe5 0
1fe5 1fe4 1fe2 1fe3
:6 0 1fe7 49 :3 0
1224 1fe9 1226 1fe8
1fe7 :2 0 1fea 1228
:2 0 1fec 0 1fec
1feb 1f99 1fea :6 0
1fee 40 :3 0 122b
1ff0 1dcd 1f76 0
1ff1 0 1fee 0
1ff1 122d 0 1ff2
1230 220b a4 :3 0
26 :3 0 27 :2 0
27 :2 0 1232 1ff3
1ff7 8f :2 0 30
:4 0 1238 1ff9 1ffb
:3 0 126 :3 0 127
:3 0 128 :3 0 a4
:3 0 1f :3 0 128
:3 0 a4 :3 0 127
:3 0 128 :3 0 a4
:3 0 129 :3 0 37
:3 0 12a :3 0 12a
:3 0 19 :3 0 21
:3 0 44 :3 0 47
:3 0 11f :3 0 b8
:3 0 17 :3 0 120
:3 0 17 :3 0 121
:3 0 1d :3 0 122
:3 0 12b :4 0 1bd
1 :8 0 201e 15
:3 0 9b :3 0 2019
201a 0 9c :3 0
201b 201c 0 201e
123b 2071 9d :3 0
15 :3 0 9b :3 0
2020 2021 0 9c
:3 0 2022 2023 0
2025 123e 2027 1240
2026 2025 :2 0 206f
9e :3 0 15 :3 0
9b :3 0 202a 202b
0 9c :3 0 202c
202d 0 2067 66
:3 0 67 :3 0 202f
2030 0 66 :3 0
77 :3 0 2032 2033
0 9f :2 0 2
:4 0 1be :2 0 79
:2 0 27 :2 0 1242
2038 203a :3 0 1245
2034 203c 2031 203e
203d :2 0 69 :3 0
7a :3 0 2040 2041
0 6b :4 0 1bf
:4 0 6e :2 0 6f
:3 0 1249 2045 2047
:3 0 6e :2 0 82
:4 0 124c 2049 204b
:3 0 6e :2 0 a2
:4 0 124f 204d 204f
:3 0 6e :2 0 9c
:3 0 1252 2051 2053
:3 0 6e :2 0 a3
:4 0 1255 2055 2057
:3 0 6e :2 0 a4
:3 0 a5 :3 0 27
:2 0 a6 :2 0 1258
205a 205e 125c 2059
2060 :3 0 125f 2042
2062 :2 0 2064 1262
2065 203f 2064 0
2066 1264 0 2067
1266 206a :3 0 206a
0 206a 2069 2067
2068 :6 0 206c 4b
:3 0 1269 206e 126b
206d 206c :2 0 206f
126d :2 0 2071 0
2071 2070 201e 206f
:6 0 2191 40 :3 0
15 :3 0 9b :3 0
2073 2074 0 15
:3 0 8f :2 0 a7
:3 0 2076 2078 0
1272 2077 207a :3 0
207b :2 0 73 :3 0
47 :3 0 30 :2 0
1275 207d 2080 8f
:2 0 30 :2 0 127a
2082 2084 :3 0 2085
:2 0 73 :3 0 44
:3 0 30 :2 0 127d
2087 208a 8f :2 0
27 :2 0 1282 208c
208e :3 0 208f :2 0
2086 2091 2090 :2 0
126 :3 0 12f :3 0
128 :3 0 a4 :3 0
1f :3 0 128 :3 0
a4 :3 0 130 :3 0
37 :3 0 131 :3 0
19 :3 0 21 :3 0
44 :3 0 11f :3 0
b8 :3 0 17 :3 0
120 :3 0 17 :3 0
121 :3 0 1d :3 0
122 :3 0 12b :4 0
1c0 1 :8 0 20af
15 :3 0 9b :3 0
20aa 20ab 0 9c
:3 0 20ac 20ad 0
20af 1285 2102 9d
:3 0 15 :3 0 9b
:3 0 20b1 20b2 0
9c :3 0 20b3 20b4
0 20b6 1288 20b8
128a 20b7 20b6 :2 0
2100 9e :3 0 15
:3 0 9b :3 0 20bb
20bc 0 9c :3 0
20bd 20be 0 20f8
66 :3 0 67 :3 0
20c0 20c1 0 66
:3 0 77 :3 0 20c3
20c4 0 9f :2 0
2 :4 0 1c1 :2 0
79 :2 0 27 :2 0
128c 20c9 20cb :3 0
128f 20c5 20cd 20c2
20cf 20ce :2 0 69
:3 0 7a :3 0 20d1
20d2 0 6b :4 0
1c2 :4 0 6e :2 0
6f :3 0 1293 20d6
20d8 :3 0 6e :2 0
82 :4 0 1296 20da
20dc :3 0 6e :2 0
a2 :4 0 1299 20de
20e0 :3 0 6e :2 0
9c :3 0 129c 20e2
20e4 :3 0 6e :2 0
a3 :4 0 129f 20e6
20e8 :3 0 6e :2 0
a4 :3 0 a5 :3 0
27 :2 0 a6 :2 0
12a2 20eb 20ef 12a6
20ea 20f1 :3 0 12a9
20d3 20f3 :2 0 20f5
12ac 20f6 20d0 20f5
0 20f7 12ae 0
20f8 12b0 20fb :3 0
20fb 0 20fb 20fa
20f8 20f9 :6 0 20fd
4d :3 0 12b3 20ff
12b5 20fe 20fd :2 0
2100 12b7 :2 0 2102
0 2102 2101 20af
2100 :6 0 2104 40
:3 0 12ba 210a 33
:3 0 10e :2 0 2105
2106 0 2109 0
2109 12bc 210b 2092
2104 0 210c 0
2109 0 210c 12bf
0 210e ae :3 0
12c2 218f 15 :3 0
9b :3 0 210f 2110
0 15 :3 0 8f
:2 0 d9 :3 0 2112
2114 0 12c6 2113
2116 :3 0 2117 :2 0
126 :3 0 12f :3 0
128 :3 0 a4 :3 0
1f :3 0 128 :3 0
a4 :3 0 12f :3 0
128 :3 0 a4 :3 0
130 :3 0 37 :3 0
131 :3 0 19 :3 0
21 :3 0 44 :3 0
11f :3 0 b8 :3 0
17 :3 0 120 :3 0
17 :3 0 121 :3 0
1d :3 0 122 :3 0
12b :4 0 1c3 1
:8 0 2138 15 :3 0
9b :3 0 2133 2134
0 9c :3 0 2135
2136 0 2138 12c9
218b 9d :3 0 15
:3 0 9b :3 0 213a
213b 0 9c :3 0
213c 213d 0 213f
12cc 2141 12ce 2140
213f :2 0 2189 9e
:3 0 15 :3 0 9b
:3 0 2144 2145 0
9c :3 0 2146 2147
0 2181 66 :3 0
67 :3 0 2149 214a
0 66 :3 0 77
:3 0 214c 214d 0
9f :2 0 2 :4 0
1c4 :2 0 79 :2 0
27 :2 0 12d0 2152
2154 :3 0 12d3 214e
2156 214b 2158 2157
:2 0 69 :3 0 7a
:3 0 215a 215b 0
6b :4 0 1c5 :4 0
6e :2 0 6f :3 0
12d7 215f 2161 :3 0
6e :2 0 82 :4 0
12da 2163 2165 :3 0
6e :2 0 a2 :4 0
12dd 2167 2169 :3 0
6e :2 0 9c :3 0
12e0 216b 216d :3 0
6e :2 0 a3 :4 0
12e3 216f 2171 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 12e6 2174
2178 12ea 2173 217a
:3 0 12ed 215c 217c
:2 0 217e 12f0 217f
2159 217e 0 2180
12f2 0 2181 12f4
2184 :3 0 2184 0
2184 2183 2181 2182
:6 0 2186 4f :3 0
12f7 2188 12f9 2187
2186 :2 0 2189 12fb
:2 0 218b 0 218b
218a 2138 2189 :6 0
218d 40 :3 0 12fe
218e 2118 218d 0
2190 207c 210e 0
2190 1300 0 2191
1303 2207 126 :3 0
12f :3 0 128 :3 0
a4 :3 0 1f :3 0
128 :3 0 a4 :3 0
12f :3 0 128 :3 0
a4 :3 0 130 :3 0
37 :3 0 131 :3 0
19 :3 0 21 :3 0
44 :3 0 11f :3 0
b8 :3 0 17 :3 0
120 :3 0 17 :3 0
121 :3 0 1d :3 0
122 :3 0 12b :4 0
1c6 1 :8 0 21b1
15 :3 0 9b :3 0
21ac 21ad 0 9c
:3 0 21ae 21af 0
21b1 1306 2204 9d
:3 0 15 :3 0 9b
:3 0 21b3 21b4 0
9c :3 0 21b5 21b6
0 21b8 1309 21ba
130b 21b9 21b8 :2 0
2202 9e :3 0 15
:3 0 9b :3 0 21bd
21be 0 9c :3 0
21bf 21c0 0 21fa
66 :3 0 67 :3 0
21c2 21c3 0 66
:3 0 77 :3 0 21c5
21c6 0 9f :2 0
2 :4 0 1c7 :2 0
79 :2 0 27 :2 0
130d 21cb 21cd :3 0
1310 21c7 21cf 21c4
21d1 21d0 :2 0 69
:3 0 7a :3 0 21d3
21d4 0 6b :4 0
1c8 :4 0 6e :2 0
6f :3 0 1314 21d8
21da :3 0 6e :2 0
82 :4 0 1317 21dc
21de :3 0 6e :2 0
a2 :4 0 131a 21e0
21e2 :3 0 6e :2 0
9c :3 0 131d 21e4
21e6 :3 0 6e :2 0
a3 :4 0 1320 21e8
21ea :3 0 6e :2 0
a4 :3 0 a5 :3 0
27 :2 0 a6 :2 0
1323 21ed 21f1 1327
21ec 21f3 :3 0 132a
21d5 21f5 :2 0 21f7
132d 21f8 21d2 21f7
0 21f9 132f 0
21fa 1331 21fd :3 0
21fd 0 21fd 21fc
21fa 21fb :6 0 21ff
51 :3 0 1334 2201
1336 2200 21ff :2 0
2202 1338 :2 0 2204
0 2204 2203 21b1
2202 :6 0 2206 40
:3 0 133b 2208 1ffc
2191 0 2209 0
2206 0 2209 133d
0 220a 1340 220c
1dc3 1ff2 0 220d
0 220a 0 220d
1342 0 2543 15
:3 0 9b :3 0 220e
220f 0 15 :3 0
8f :2 0 a7 :3 0
2211 2213 0 1347
2212 2215 :3 0 2216
:2 0 73 :3 0 44
:3 0 30 :2 0 134a
2218 221b 8f :2 0
27 :2 0 134f 221d
221f :3 0 46 :3 0
10e :2 0 2221 2222
0 2224 1352 232f
19 :3 0 96 :2 0
1354 2226 2227 :3 0
2228 :2 0 63 :3 0
8f :2 0 27 :2 0
1358 222b 222d :3 0
47 :3 0 30 :2 0
222f 2230 0 229c
14c :3 0 14d :3 0
47 :3 0 14e :3 0
b8 :3 0 17 :3 0
14f :3 0 19 :3 0
150 :3 0 128 :3 0
a4 :3 0 1f :3 0
128 :3 0 a4 :3 0
22 :4 0 1c9 1
:8 0 2247 15 :3 0
9b :3 0 2242 2243
0 9c :3 0 2244
2245 0 2247 135b
229a 9d :3 0 15
:3 0 9b :3 0 2249
224a 0 9c :3 0
224b 224c 0 224e
135e 2250 1360 224f
224e :2 0 2298 9e
:3 0 15 :3 0 9b
:3 0 2253 2254 0
9c :3 0 2255 2256
0 2290 66 :3 0
67 :3 0 2258 2259
0 66 :3 0 77
:3 0 225b 225c 0
9f :2 0 2 :4 0
1ca :2 0 79 :2 0
27 :2 0 1362 2261
2263 :3 0 1365 225d
2265 225a 2267 2266
:2 0 69 :3 0 7a
:3 0 2269 226a 0
6b :4 0 1cb :4 0
6e :2 0 6f :3 0
1369 226e 2270 :3 0
6e :2 0 82 :4 0
136c 2272 2274 :3 0
6e :2 0 a2 :4 0
136f 2276 2278 :3 0
6e :2 0 9c :3 0
1372 227a 227c :3 0
6e :2 0 a3 :4 0
1375 227e 2280 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 1378 2283
2287 137c 2282 2289
:3 0 137f 226b 228b
:2 0 228d 1382 228e
2268 228d 0 228f
1384 0 2290 1386
2293 :3 0 2293 0
2293 2292 2290 2291
:6 0 2295 53 :3 0
1389 2297 138b 2296
2295 :2 0 2298 138d
:2 0 229a 0 229a
2299 2247 2298 :6 0
229c 40 :3 0 1390
2308 47 :3 0 30
:2 0 229d 229e 0
2307 14c :3 0 14d
:3 0 47 :3 0 14e
:3 0 b8 :3 0 17
:3 0 14f :3 0 19
:3 0 150 :3 0 128
:3 0 a4 :3 0 1f
:4 0 1cc 1 :8 0
22b2 15 :3 0 9b
:3 0 22ad 22ae 0
9c :3 0 22af 22b0
0 22b2 1393 2305
9d :3 0 15 :3 0
9b :3 0 22b4 22b5
0 9c :3 0 22b6
22b7 0 22b9 1396
22bb 1398 22ba 22b9
:2 0 2303 9e :3 0
15 :3 0 9b :3 0
22be 22bf 0 9c
:3 0 22c0 22c1 0
22fb 66 :3 0 67
:3 0 22c3 22c4 0
66 :3 0 77 :3 0
22c6 22c7 0 9f
:2 0 2 :4 0 1cd
:2 0 79 :2 0 27
:2 0 139a 22cc 22ce
:3 0 139d 22c8 22d0
22c5 22d2 22d1 :2 0
69 :3 0 7a :3 0
22d4 22d5 0 6b
:4 0 1ce :4 0 6e
:2 0 6f :3 0 13a1
22d9 22db :3 0 6e
:2 0 82 :4 0 13a4
22dd 22df :3 0 6e
:2 0 a2 :4 0 13a7
22e1 22e3 :3 0 6e
:2 0 9c :3 0 13aa
22e5 22e7 :3 0 6e
:2 0 a3 :4 0 13ad
22e9 22eb :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 13b0 22ee 22f2
13b4 22ed 22f4 :3 0
13b7 22d6 22f6 :2 0
22f8 13ba 22f9 22d3
22f8 0 22fa 13bc
0 22fb 13be 22fe
:3 0 22fe 0 22fe
22fd 22fb 22fc :6 0
2300 55 :3 0 13c1
2302 13c3 2301 2300
:2 0 2303 13c5 :2 0
2305 0 2305 2304
22b2 2303 :6 0 2307
40 :3 0 13c8 2309
222e 229c 0 230a
0 2307 0 230a
13cb 0 2328 73
:3 0 47 :3 0 30
:2 0 13ce 230b 230e
73 :3 0 a8 :2 0
37 :3 0 30 :2 0
13d1 2310 2314 13d6
2311 2316 :3 0 2f
:3 0 73 :3 0 2f
:3 0 30 :2 0 13d9
2319 231c 79 :2 0
27 :2 0 13dc 231e
2320 :3 0 2318 2321
0 2325 1cf :3 0
2323 :4 0 2325 13df
2326 2317 2325 0
2327 13e2 0 2328
13e4 2329 2229 2328
0 232a 13e7 0
232e 46 :3 0 27
:2 0 232b 232c 0
232e 13e9 2330 2220
2224 0 2331 0
232e 0 2331 13ec
0 2332 13ef 2341
2f :3 0 73 :3 0
2f :3 0 30 :2 0
13f1 2334 2337 79
:2 0 27 :2 0 13f4
2339 233b :3 0 2333
233c 0 2340 1cf
:3 0 233e :4 0 2340
13f7 2342 2217 2332
0 2343 0 2340
0 2343 13fa 0
2543 21 :3 0 21
:3 0 2344 2345 0
2543 3f :3 0 30
:2 0 2347 2348 0
2543 97 :3 0 18f
:3 0 128 :3 0 a4
:3 0 21 :3 0 97
:3 0 190 :3 0 1f
:3 0 21 :3 0 99
:4 0 1d0 1 :8 0
235a 15 :3 0 9b
:3 0 2355 2356 0
9c :3 0 2357 2358
0 235a 13fd 23ad
9d :3 0 15 :3 0
9b :3 0 235c 235d
0 9c :3 0 235e
235f 0 2361 1400
2363 1402 2362 2361
:2 0 23ab 9e :3 0
15 :3 0 9b :3 0
2366 2367 0 9c
:3 0 2368 2369 0
23a3 66 :3 0 67
:3 0 236b 236c 0
66 :3 0 77 :3 0
236e 236f 0 9f
:2 0 2 :4 0 1d1
:2 0 79 :2 0 27
:2 0 1404 2374 2376
:3 0 1407 2370 2378
236d 237a 2379 :2 0
69 :3 0 7a :3 0
237c 237d 0 6b
:4 0 1d2 :4 0 6e
:2 0 6f :3 0 140b
2381 2383 :3 0 6e
:2 0 82 :4 0 140e
2385 2387 :3 0 6e
:2 0 a2 :4 0 1411
2389 238b :3 0 6e
:2 0 9c :3 0 1414
238d 238f :3 0 6e
:2 0 a3 :4 0 1417
2391 2393 :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 141a 2396 239a
141e 2395 239c :3 0
1421 237e 239e :2 0
23a0 1424 23a1 237b
23a0 0 23a2 1426
0 23a3 1428 23a6
:3 0 23a6 0 23a6
23a5 23a3 23a4 :6 0
23a8 57 :3 0 142b
23aa 142d 23a9 23a8
:2 0 23ab 142f :2 0
23ad 0 23ad 23ac
235a 23ab :6 0 2543
40 :3 0 128 :3 0
a4 :3 0 1f :3 0
128 :3 0 a4 :3 0
21 :3 0 1f :3 0
21 :3 0 21 :3 0
99 :4 0 1d3 1
:8 0 23bf 15 :3 0
9b :3 0 23ba 23bb
0 9c :3 0 23bc
23bd 0 23bf 1432
2412 9d :3 0 15
:3 0 9b :3 0 23c1
23c2 0 9c :3 0
23c3 23c4 0 23c6
1435 23c8 1437 23c7
23c6 :2 0 2410 9e
:3 0 15 :3 0 9b
:3 0 23cb 23cc 0
9c :3 0 23cd 23ce
0 2408 66 :3 0
67 :3 0 23d0 23d1
0 66 :3 0 77
:3 0 23d3 23d4 0
9f :2 0 2 :4 0
1d4 :2 0 79 :2 0
27 :2 0 1439 23d9
23db :3 0 143c 23d5
23dd 23d2 23df 23de
:2 0 69 :3 0 7a
:3 0 23e1 23e2 0
6b :4 0 1d5 :4 0
6e :2 0 6f :3 0
1440 23e6 23e8 :3 0
6e :2 0 82 :4 0
1443 23ea 23ec :3 0
6e :2 0 a2 :4 0
1446 23ee 23f0 :3 0
6e :2 0 9c :3 0
1449 23f2 23f4 :3 0
6e :2 0 a3 :4 0
144c 23f6 23f8 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 144f 23fb
23ff 1453 23fa 2401
:3 0 1456 23e3 2403
:2 0 2405 1459 2406
23e0 2405 0 2407
145b 0 2408 145d
240b :3 0 240b 0
240b 240a 2408 2409
:6 0 240d 59 :3 0
1460 240f 1462 240e
240d :2 0 2410 1464
:2 0 2412 0 2412
2411 23bf 2410 :6 0
2543 40 :3 0 45
:3 0 197 :2 0 2414
2415 0 2543 22
:3 0 6 :3 0 2417
2418 0 2543 73
:3 0 33 :3 0 30
:2 0 1467 241a 241d
8f :2 0 30 :2 0
146c 241f 2421 :3 0
3f :3 0 10f :2 0
2423 2424 0 2426
146f 2427 2422 2426
0 2428 1471 0
2543 62 :3 0 198
:3 0 28 :3 0 17
:3 0 1a :3 0 1d
:3 0 4e :3 0 32
:3 0 23 :3 0 23
:3 0 23 :3 0 23
:3 0 21 :3 0 22
:3 0 41 :3 0 37
:3 0 35 :3 0 38
:3 0 39 :3 0 34
:3 0 45 :3 0 26
:3 0 33 :3 0 31
:3 0 3b :3 0 36
:3 0 3f :3 0 3d
:3 0 43 :3 0 1c
:3 0 30 :2 0 30
:2 0 50 :2 0 50
:2 0 1473 242a 244b
2429 244c 0 2543
62 :3 0 8f :2 0
30 :2 0 1496 244f
2451 :3 0 73 :3 0
33 :3 0 30 :2 0
1499 2453 2456 8f
:2 0 30 :2 0 149e
2458 245a :3 0 199
:3 0 19a :3 0 19b
:3 0 1d :4 0 19c
1 :8 0 2476 19d
:4 0 2461 :3 0 8a
:3 0 8f :2 0 14a3
2464 2465 :3 0 15
:3 0 9b :3 0 2467
2468 0 30 :2 0
2469 246a 0 246c
14a6 2473 15 :3 0
9b :3 0 246d 246e
0 10d :2 0 246f
2470 0 2472 14a8
2474 2466 246c 0
2475 0 2472 0
2475 14aa 0 2476
14ad 24c9 9d :3 0
15 :3 0 9b :3 0
2478 2479 0 9c
:3 0 247a 247b 0
247d 14b0 247f 14b2
247e 247d :2 0 24c7
9e :3 0 15 :3 0
9b :3 0 2482 2483
0 9c :3 0 2484
2485 0 24bf 66
:3 0 67 :3 0 2487
2488 0 66 :3 0
77 :3 0 248a 248b
0 9f :2 0 2
:4 0 1d6 :2 0 79
:2 0 27 :2 0 14b4
2490 2492 :3 0 14b7
248c 2494 2489 2496
2495 :2 0 69 :3 0
7a :3 0 2498 2499
0 6b :4 0 1d7
:4 0 6e :2 0 6f
:3 0 14bb 249d 249f
:3 0 6e :2 0 82
:4 0 14be 24a1 24a3
:3 0 6e :2 0 a2
:4 0 14c1 24a5 24a7
:3 0 6e :2 0 9c
:3 0 14c4 24a9 24ab
:3 0 6e :2 0 a3
:4 0 14c7 24ad 24af
:3 0 6e :2 0 a4
:3 0 a5 :3 0 27
:2 0 a6 :2 0 14ca
24b2 24b6 14ce 24b1
24b8 :3 0 14d1 249a
24ba :2 0 24bc 14d4
24bd 2497 24bc 0
24be 14d6 0 24bf
14d8 24c2 :3 0 24c2
0 24c2 24c1 24bf
24c0 :6 0 24c4 5b
:3 0 14db 24c6 14dd
24c5 24c4 :2 0 24c7
14df :2 0 24c9 0
24c9 24c8 2476 24c7
:6 0 24cb 40 :3 0
14e2 253c 199 :3 0
19a :3 0 19b :3 0
1d :4 0 1a0 1
:8 0 24e6 19d :4 0
24d1 :3 0 8a :3 0
8f :2 0 14e6 24d4
24d5 :3 0 15 :3 0
9b :3 0 24d7 24d8
0 30 :2 0 24d9
24da 0 24dc 14e9
24e3 15 :3 0 9b
:3 0 24dd 24de 0
10d :2 0 24df 24e0
0 24e2 14eb 24e4
24d6 24dc 0 24e5
0 24e2 0 24e5
14ed 0 24e6 14f0
2539 9d :3 0 15
:3 0 9b :3 0 24e8
24e9 0 9c :3 0
24ea 24eb 0 24ed
14f3 24ef 14f5 24ee
24ed :2 0 2537 9e
:3 0 15 :3 0 9b
:3 0 24f2 24f3 0
9c :3 0 24f4 24f5
0 252f 66 :3 0
67 :3 0 24f7 24f8
0 66 :3 0 77
:3 0 24fa 24fb 0
9f :2 0 2 :4 0
1d8 :2 0 79 :2 0
27 :2 0 14f7 2500
2502 :3 0 14fa 24fc
2504 24f9 2506 2505
:2 0 69 :3 0 7a
:3 0 2508 2509 0
6b :4 0 1d9 :4 0
6e :2 0 6f :3 0
14fe 250d 250f :3 0
6e :2 0 82 :4 0
1501 2511 2513 :3 0
6e :2 0 a2 :4 0
1504 2515 2517 :3 0
6e :2 0 9c :3 0
1507 2519 251b :3 0
6e :2 0 a3 :4 0
150a 251d 251f :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 150d 2522
2526 1511 2521 2528
:3 0 1514 250a 252a
:2 0 252c 1517 252d
2507 252c 0 252e
1519 0 252f 151b
2532 :3 0 2532 0
2532 2531 252f 2530
:6 0 2534 5d :3 0
151e 2536 1520 2535
2534 :2 0 2537 1522
:2 0 2539 0 2539
2538 24e6 2537 :6 0
253b 40 :3 0 1525
253d 245b 24cb 0
253e 0 253b 0
253e 1527 0 253f
152a 2540 2452 253f
0 2541 152c 0
2543 ae :3 0 152e
260c 73 :3 0 46
:3 0 30 :2 0 153a
2544 2547 8f :2 0
30 :2 0 153f 2549
254b :3 0 21 :3 0
1f :3 0 254d 254e
0 260a 22 :3 0
6 :3 0 2550 2551
0 260a a4 :3 0
26 :3 0 27 :2 0
27 :2 0 1542 2553
2557 8f :2 0 27
:4 0 1548 2559 255b
:3 0 45 :3 0 84
:3 0 1a3 :3 0 255e
255f 0 255d 2560
0 2562 154b 256b
45 :3 0 73 :3 0
3a :3 0 30 :2 0
154d 2564 2567 2563
2568 0 256a 1550
256c 255c 2562 0
256d 0 256a 0
256d 1552 0 260a
62 :3 0 198 :3 0
28 :3 0 17 :3 0
1a :3 0 1d :3 0
4e :3 0 32 :3 0
23 :3 0 23 :3 0
23 :3 0 23 :3 0
21 :3 0 22 :3 0
41 :3 0 37 :3 0
35 :3 0 38 :3 0
39 :3 0 34 :3 0
45 :3 0 26 :3 0
33 :3 0 31 :3 0
3b :3 0 36 :3 0
3f :3 0 3d :3 0
43 :3 0 1c :3 0
30 :2 0 30 :2 0
50 :2 0 50 :2 0
1555 256f 2590 256e
2591 0 260a 62
:3 0 8f :2 0 30
:2 0 1578 2594 2596
:3 0 199 :3 0 19a
:3 0 19b :3 0 1d
:4 0 19c 1 :8 0
25b2 19d :4 0 259d
:3 0 8a :3 0 8f
:2 0 157d 25a0 25a1
:3 0 15 :3 0 9b
:3 0 25a3 25a4 0
30 :2 0 25a5 25a6
0 25a8 1580 25af
15 :3 0 9b :3 0
25a9 25aa 0 10d
:2 0 25ab 25ac 0
25ae 1582 25b0 25a2
25a8 0 25b1 0
25ae 0 25b1 1584
0 25b2 1587 2605
9d :3 0 15 :3 0
9b :3 0 25b4 25b5
0 9c :3 0 25b6
25b7 0 25b9 158a
25bb 158c 25ba 25b9
:2 0 2603 9e :3 0
15 :3 0 9b :3 0
25be 25bf 0 9c
:3 0 25c0 25c1 0
25fb 66 :3 0 67
:3 0 25c3 25c4 0
66 :3 0 77 :3 0
25c6 25c7 0 9f
:2 0 2 :4 0 1da
:2 0 79 :2 0 27
:2 0 158e 25cc 25ce
:3 0 1591 25c8 25d0
25c5 25d2 25d1 :2 0
69 :3 0 7a :3 0
25d4 25d5 0 6b
:4 0 1db :4 0 6e
:2 0 6f :3 0 1595
25d9 25db :3 0 6e
:2 0 82 :4 0 1598
25dd 25df :3 0 6e
:2 0 a2 :4 0 159b
25e1 25e3 :3 0 6e
:2 0 9c :3 0 159e
25e5 25e7 :3 0 6e
:2 0 a3 :4 0 15a1
25e9 25eb :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 15a4 25ee 25f2
15a8 25ed 25f4 :3 0
15ab 25d6 25f6 :2 0
25f8 15ae 25f9 25d3
25f8 0 25fa 15b0
0 25fb 15b2 25fe
:3 0 25fe 0 25fe
25fd 25fb 25fc :6 0
2600 5f :3 0 15b5
2602 15b7 2601 2600
:2 0 2603 15b9 :2 0
2605 0 2605 2604
25b2 2603 :6 0 2607
40 :3 0 15bc 2608
2597 2607 0 2609
15be 0 260a 15c0
260b 254c 260a 0
260d 1dbe 2543 0
260d 15c6 0 261d
2f :3 0 73 :3 0
2f :3 0 30 :2 0
15c9 260f 2612 79
:2 0 27 :2 0 15cc
2614 2616 :3 0 260e
2617 0 261d 1cf
:5 0 261b 61 0
261c 15cf 261b 261a
261d 15d1 261f 1aa
:3 0 1c70 261d :4 0
2620 15e1 2621 1c61
2620 0 2622 15e4
0 2623 15e6 2625
d3c 1c32 0 2626
0 2623 0 2626
15e9 0 3149 62
:3 0 b7 :2 0 30
:2 0 15ee 2628 262a
:3 0 262b :2 0 73
:3 0 32 :3 0 30
:2 0 15f1 262d 2630
b2 :2 0 30 :2 0
15f6 2632 2634 :3 0
2635 :2 0 262c 2637
2636 :2 0 73 :3 0
42 :3 0 30 :2 0
15f9 2639 263c 8f
:2 0 30 :2 0 15fe
263e 2640 :3 0 175
:3 0 172 :3 0 28
:4 0 1dc 1 :8 0
265b 19d :4 0 2646
:3 0 8a :3 0 8f
:2 0 1603 2649 264a
:3 0 15 :3 0 9b
:3 0 264c 264d 0
30 :2 0 264e 264f
0 2651 1606 2658
15 :3 0 9b :3 0
2652 2653 0 10d
:2 0 2654 2655 0
2657 1608 2659 264b
2651 0 265a 0
2657 0 265a 160a
0 265b 160d 26ae
9d :3 0 15 :3 0
9b :3 0 265d 265e
0 9c :3 0 265f
2660 0 2662 1610
2664 1612 2663 2662
:2 0 26ac 9e :3 0
15 :3 0 9b :3 0
2667 2668 0 9c
:3 0 2669 266a 0
26a4 66 :3 0 67
:3 0 266c 266d 0
66 :3 0 77 :3 0
266f 2670 0 9f
:2 0 2 :4 0 1dd
:2 0 79 :2 0 27
:2 0 1614 2675 2677
:3 0 1617 2671 2679
266e 267b 267a :2 0
69 :3 0 7a :3 0
267d 267e 0 6b
:4 0 1de :4 0 6e
:2 0 6f :3 0 161b
2682 2684 :3 0 6e
:2 0 82 :4 0 161e
2686 2688 :3 0 6e
:2 0 a2 :4 0 1621
268a 268c :3 0 6e
:2 0 9c :3 0 1624
268e 2690 :3 0 6e
:2 0 a3 :4 0 1627
2692 2694 :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 162a 2697 269b
162e 2696 269d :3 0
1631 267f 269f :2 0
26a1 1634 26a2 267c
26a1 0 26a3 1636
0 26a4 1638 26a7
:3 0 26a7 0 26a7
26a6 26a4 26a5 :6 0
26a9 61 :3 0 163b
26ab 163d 26aa 26a9
:2 0 26ac 163f :2 0
26ae 0 26ae 26ad
265b 26ac :6 0 26b0
1 :3 0 1642 2724
175 :3 0 174 :3 0
1e :3 0 173 :3 0
42 :3 0 172 :3 0
28 :4 0 1df 1
:8 0 26ce 19d :4 0
26b9 :3 0 8a :3 0
8f :2 0 1646 26bc
26bd :3 0 15 :3 0
9b :3 0 26bf 26c0
0 30 :2 0 26c1
26c2 0 26c4 1649
26cb 15 :3 0 9b
:3 0 26c5 26c6 0
10d :2 0 26c7 26c8
0 26ca 164b 26cc
26be 26c4 0 26cd
0 26ca 0 26cd
164d 0 26ce 1650
2721 9d :3 0 15
:3 0 9b :3 0 26d0
26d1 0 9c :3 0
26d2 26d3 0 26d5
1653 26d7 1655 26d6
26d5 :2 0 271f 9e
:3 0 15 :3 0 9b
:3 0 26da 26db 0
9c :3 0 26dc 26dd
0 2717 66 :3 0
67 :3 0 26df 26e0
0 66 :3 0 77
:3 0 26e2 26e3 0
9f :2 0 2 :4 0
1e0 :2 0 79 :2 0
27 :2 0 1657 26e8
26ea :3 0 165a 26e4
26ec 26e1 26ee 26ed
:2 0 69 :3 0 7a
:3 0 26f0 26f1 0
6b :4 0 1e1 :4 0
6e :2 0 6f :3 0
165e 26f5 26f7 :3 0
6e :2 0 82 :4 0
1661 26f9 26fb :3 0
6e :2 0 a2 :4 0
1664 26fd 26ff :3 0
6e :2 0 9c :3 0
1667 2701 2703 :3 0
6e :2 0 a3 :4 0
166a 2705 2707 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 166d 270a
270e 1671 2709 2710
:3 0 1674 26f2 2712
:2 0 2714 1677 2715
26ef 2714 0 2716
1679 0 2717 167b
271a :3 0 271a 0
271a 2719 2717 2718
:6 0 271c 63 :3 0
167e 271e 1680 271d
271c :2 0 271f 1682
:2 0 2721 0 2721
2720 26ce 271f :6 0
2723 1 :3 0 1685
2725 2641 26b0 0
2726 0 2723 0
2726 1687 0 28f9
62 :3 0 8f :2 0
10d :2 0 168c 2728
272a :3 0 66 :3 0
67 :3 0 272c 272d
0 66 :3 0 77
:3 0 272f 2730 0
18 :2 0 2 :4 0
1e2 :2 0 79 :2 0
27 :2 0 168f 2735
2737 :3 0 1692 2731
2739 272e 273b 273a
:2 0 69 :3 0 7a
:3 0 273d 273e 0
6b :4 0 1e3 :4 0
6e :2 0 6f :3 0
1696 2742 2744 :3 0
6e :2 0 82 :4 0
1699 2746 2748 :3 0
6e :2 0 83 :4 0
169c 274a 274c :3 0
6e :2 0 73 :3 0
74 :3 0 84 :3 0
16f :3 0 2751 2752
0 169f 2750 2754
75 :4 0 16a1 274f
2757 16a4 274e 2759
:3 0 6e :2 0 e5
:4 0 16a7 275b 275d
:3 0 6e :2 0 71
:4 0 16aa 275f 2761
:3 0 6e :2 0 1d
:3 0 16ad 2763 2765
:3 0 6e :2 0 71
:4 0 16b0 2767 2769
:3 0 16b3 273f 276b
:2 0 276d 16b6 276e
273c 276d 0 276f
16b8 0 279a 66
:3 0 67 :3 0 2770
2771 0 66 :3 0
77 :3 0 2773 2774
0 27 :2 0 2
:4 0 1e4 :2 0 79
:2 0 27 :2 0 16ba
2779 277b :3 0 16bd
2775 277d 2772 277f
277e :2 0 69 :3 0
7a :3 0 2781 2782
0 6b :4 0 1e5
:4 0 6e :2 0 6f
:3 0 16c1 2786 2788
:3 0 16c4 2783 278a
:2 0 278c 16c7 278d
2780 278c 0 278e
16c9 0 279a 88
:3 0 e8 :3 0 278f
2790 0 84 :3 0
16f :3 0 2792 2793
0 1d :3 0 8a
:3 0 16cb 2791 2797
:2 0 279a ae :3 0
16cf 28f6 62 :3 0
84 :3 0 8f :2 0
1e6 :3 0 279c 279e
0 16d5 279d 27a0
:3 0 66 :3 0 67
:3 0 27a2 27a3 0
66 :3 0 77 :3 0
27a5 27a6 0 18
:2 0 2 :4 0 1e7
:2 0 79 :2 0 27
:2 0 16d8 27ab 27ad
:3 0 16db 27a7 27af
27a4 27b1 27b0 :2 0
69 :3 0 7a :3 0
27b3 27b4 0 6b
:4 0 1e8 :4 0 6e
:2 0 6f :3 0 16df
27b8 27ba :3 0 6e
:2 0 82 :4 0 16e2
27bc 27be :3 0 6e
:2 0 83 :4 0 16e5
27c0 27c2 :3 0 6e
:2 0 73 :3 0 74
:3 0 84 :3 0 1e6
:3 0 27c7 27c8 0
16e8 27c6 27ca 75
:4 0 16ea 27c5 27cd
16ed 27c4 27cf :3 0
6e :2 0 e5 :4 0
16f0 27d1 27d3 :3 0
6e :2 0 71 :4 0
16f3 27d5 27d7 :3 0
6e :2 0 1d :3 0
16f6 27d9 27db :3 0
6e :2 0 71 :4 0
16f9 27dd 27df :3 0
16fc 27b5 27e1 :2 0
27e3 16ff 27e4 27b2
27e3 0 27e5 1701
0 2810 66 :3 0
67 :3 0 27e6 27e7
0 66 :3 0 77
:3 0 27e9 27ea 0
27 :2 0 2 :4 0
1e9 :2 0 79 :2 0
27 :2 0 1703 27ef
27f1 :3 0 1706 27eb
27f3 27e8 27f5 27f4
:2 0 69 :3 0 7a
:3 0 27f7 27f8 0
6b :4 0 1ea :4 0
6e :2 0 6f :3 0
170a 27fc 27fe :3 0
170d 27f9 2800 :2 0
2802 1710 2803 27f6
2802 0 2804 1712
0 2810 88 :3 0
e8 :3 0 2805 2806
0 84 :3 0 1e6
:3 0 2808 2809 0
1d :3 0 8a :3 0
1714 2807 280d :2 0
2810 ae :3 0 1718
2811 27a1 2810 0
28f8 62 :3 0 84
:3 0 8f :2 0 1eb
:3 0 2813 2815 0
171e 2814 2817 :3 0
66 :3 0 67 :3 0
2819 281a 0 66
:3 0 77 :3 0 281c
281d 0 18 :2 0
2 :4 0 1ec :2 0
79 :2 0 27 :2 0
1721 2822 2824 :3 0
1724 281e 2826 281b
2828 2827 :2 0 69
:3 0 7a :3 0 282a
282b 0 6b :4 0
1ed :4 0 6e :2 0
6f :3 0 1728 282f
2831 :3 0 6e :2 0
82 :4 0 172b 2833
2835 :3 0 6e :2 0
83 :4 0 172e 2837
2839 :3 0 6e :2 0
73 :3 0 74 :3 0
84 :3 0 1eb :3 0
283e 283f 0 1731
283d 2841 75 :4 0
1733 283c 2844 1736
283b 2846 :3 0 6e
:2 0 e5 :4 0 1739
2848 284a :3 0 6e
:2 0 71 :4 0 173c
284c 284e :3 0 6e
:2 0 1d :3 0 173f
2850 2852 :3 0 6e
:2 0 71 :4 0 1742
2854 2856 :3 0 1745
282c 2858 :2 0 285a
1748 285b 2829 285a
0 285c 174a 0
2886 66 :3 0 67
:3 0 285d 285e 0
66 :3 0 77 :3 0
2860 2861 0 27
:2 0 2 :4 0 1ee
:2 0 79 :2 0 27
:2 0 174c 2866 2868
:3 0 174f 2862 286a
285f 286c 286b :2 0
69 :3 0 7a :3 0
286e 286f 0 6b
:4 0 1ef :4 0 6e
:2 0 6f :3 0 1753
2873 2875 :3 0 1756
2870 2877 :2 0 2879
1759 287a 286d 2879
0 287b 175b 0
2886 88 :3 0 e8
:3 0 287c 287d 0
84 :3 0 1eb :3 0
287f 2880 0 1d
:3 0 8a :3 0 175d
287e 2884 :2 0 2886
1761 2887 2818 2886
0 28f8 66 :3 0
67 :3 0 2888 2889
0 66 :3 0 77
:3 0 288b 288c 0
18 :2 0 2 :4 0
1f0 :2 0 79 :2 0
27 :2 0 1765 2891
2893 :3 0 1768 288d
2895 288a 2897 2896
:2 0 69 :3 0 7a
:3 0 2899 289a 0
6b :4 0 1f1 :4 0
6e :2 0 6f :3 0
176c 289e 28a0 :3 0
6e :2 0 82 :4 0
176f 28a2 28a4 :3 0
6e :2 0 83 :4 0
1772 28a6 28a8 :3 0
6e :2 0 73 :3 0
74 :3 0 84 :3 0
149 :3 0 28ad 28ae
0 1775 28ac 28b0
75 :4 0 1777 28ab
28b3 177a 28aa 28b5
:3 0 6e :2 0 e5
:4 0 177d 28b7 28b9
:3 0 6e :2 0 71
:4 0 1780 28bb 28bd
:3 0 6e :2 0 28
:3 0 1783 28bf 28c1
:3 0 6e :2 0 71
:4 0 1786 28c3 28c5
:3 0 1789 289b 28c7
:2 0 28c9 178c 28ca
2898 28c9 0 28cb
178e 0 28f5 66
:3 0 67 :3 0 28cc
28cd 0 66 :3 0
77 :3 0 28cf 28d0
0 27 :2 0 2
:4 0 1f2 :2 0 79
:2 0 27 :2 0 1790
28d5 28d7 :3 0 1793
28d1 28d9 28ce 28db
28da :2 0 69 :3 0
7a :3 0 28dd 28de
0 6b :4 0 1f3
:4 0 6e :2 0 6f
:3 0 1797 28e2 28e4
:3 0 179a 28df 28e6
:2 0 28e8 179d 28e9
28dc 28e8 0 28ea
179f 0 28f5 88
:3 0 e8 :3 0 28eb
28ec 0 84 :3 0
149 :3 0 28ee 28ef
0 28 :3 0 8a
:3 0 17a1 28ed 28f3
:2 0 28f5 17a5 28f7
272b 279a 0 28f8
0 28f5 0 28f8
17a9 0 28f9 17ae
28fa 2638 28f9 0
28fb 17b1 0 3149
73 :3 0 1a6 :3 0
2b :3 0 17b3 28fd
28ff 30 :2 0 17b5
28fc 2902 b2 :2 0
30 :2 0 17ba 2904
2906 :3 0 2907 :2 0
73 :3 0 1a6 :3 0
2c :3 0 17bd 290a
290c 30 :2 0 17bf
2909 290f b2 :2 0
30 :2 0 17c4 2911
2913 :3 0 2914 :2 0
2908 2916 2915 :2 0
1f4 :3 0 1f5 :3 0
1f6 :4 0 291a :4 0
291b :2 0 2c26 15
:3 0 9b :3 0 291d
291e 0 30 :2 0
291f 2920 0 2c26
51 :3 0 1f7 :4 0
6e :2 0 1f8 :4 0
17c7 2924 2926 :3 0
6e :2 0 1f9 :4 0
17ca 2928 292a :3 0
6e :2 0 1fa :4 0
17cd 292c 292e :3 0
6e :2 0 1fb :4 0
17d0 2930 2932 :3 0
6e :2 0 1fc :4 0
17d3 2934 2936 :3 0
6e :2 0 17 :3 0
17d6 2938 293a :3 0
6e :2 0 1fd :4 0
17d9 293c 293e :3 0
6e :2 0 1fe :4 0
17dc 2940 2942 :3 0
6e :2 0 2b :3 0
17df 2944 2946 :3 0
6e :2 0 1ff :4 0
17e2 2948 294a :3 0
6e :2 0 2c :3 0
17e5 294c 294e :3 0
6e :2 0 200 :4 0
17e8 2950 2952 :3 0
2922 2953 0 2c26
1f4 :3 0 1f5 :3 0
51 :3 0 2957 :4 0
2958 :2 0 296f 19d
:4 0 295a :3 0 8a
:3 0 8f :2 0 17ed
295d 295e :3 0 15
:3 0 9b :3 0 2960
2961 0 30 :2 0
2962 2963 0 2965
17f0 296c 15 :3 0
9b :3 0 2966 2967
0 10d :2 0 2968
2969 0 296b 17f2
296d 295f 2965 0
296e 0 296b 0
296e 17f4 0 296f
17f7 29cd 9d :3 0
15 :3 0 9b :3 0
2971 2972 0 9c
:3 0 2973 2974 0
2976 17fa 2978 17fc
2977 2976 :2 0 29cb
9e :3 0 15 :3 0
9b :3 0 297b 297c
0 9c :3 0 297d
297e 0 29c3 66
:3 0 67 :3 0 2980
2981 0 66 :3 0
77 :3 0 2983 2984
0 9f :2 0 2
:4 0 201 :2 0 79
:2 0 27 :2 0 17fe
2989 298b :3 0 1801
2985 298d 2982 298f
298e :2 0 69 :3 0
7a :3 0 2991 2992
0 6b :4 0 202
:4 0 6e :2 0 6f
:3 0 1805 2996 2998
:3 0 6e :2 0 82
:4 0 1808 299a 299c
:3 0 6e :2 0 a2
:4 0 180b 299e 29a0
:3 0 6e :2 0 9c
:3 0 180e 29a2 29a4
:3 0 6e :2 0 a3
:4 0 1811 29a6 29a8
:3 0 6e :2 0 a4
:3 0 a5 :3 0 27
:2 0 a6 :2 0 1814
29ab 29af 1818 29aa
29b1 :3 0 181b 2993
29b3 :2 0 29c0 69
:3 0 7a :3 0 29b5
29b6 0 6b :4 0
203 :4 0 6e :2 0
51 :3 0 181e 29ba
29bc :3 0 1821 29b7
29be :2 0 29c0 1824
29c1 2990 29c0 0
29c2 1827 0 29c3
1829 29c6 :3 0 29c6
0 29c6 29c5 29c3
29c4 :6 0 29c8 65
:3 0 182c 29ca 182e
29c9 29c8 :2 0 29cb
1830 :2 0 29cd 0
29cd 29cc 296f 29cb
:6 0 2c26 1 :3 0
1f4 :3 0 1f5 :3 0
204 :4 0 29d1 :4 0
29d2 :2 0 2c26 15
:3 0 9b :3 0 29d4
29d5 0 30 :2 0
29d6 29d7 0 2c26
51 :3 0 205 :4 0
6e :2 0 206 :4 0
1833 29db 29dd :3 0
6e :2 0 207 :4 0
1836 29df 29e1 :3 0
6e :2 0 208 :4 0
1839 29e3 29e5 :3 0
6e :2 0 209 :4 0
183c 29e7 29e9 :3 0
6e :2 0 20a :4 0
183f 29eb 29ed :3 0
6e :2 0 17 :3 0
1842 29ef 29f1 :3 0
6e :2 0 1fd :4 0
1845 29f3 29f5 :3 0
6e :2 0 20b :4 0
1848 29f7 29f9 :3 0
6e :2 0 2b :3 0
184b 29fb 29fd :3 0
6e :2 0 1ff :4 0
184e 29ff 2a01 :3 0
6e :2 0 2c :3 0
1851 2a03 2a05 :3 0
6e :2 0 20c :4 0
1854 2a07 2a09 :3 0
29d9 2a0a 0 2c26
1f4 :3 0 1f5 :3 0
51 :3 0 2a0e :4 0
2a0f :2 0 2a26 19d
:4 0 2a11 :3 0 8a
:3 0 8f :2 0 1859
2a14 2a15 :3 0 15
:3 0 9b :3 0 2a17
2a18 0 30 :2 0
2a19 2a1a 0 2a1c
185c 2a23 15 :3 0
9b :3 0 2a1d 2a1e
0 10d :2 0 2a1f
2a20 0 2a22 185e
2a24 2a16 2a1c 0
2a25 0 2a22 0
2a25 1860 0 2a26
1863 2a84 9d :3 0
15 :3 0 9b :3 0
2a28 2a29 0 9c
:3 0 2a2a 2a2b 0
2a2d 1866 2a2f 1868
2a2e 2a2d :2 0 2a82
9e :3 0 15 :3 0
9b :3 0 2a32 2a33
0 9c :3 0 2a34
2a35 0 2a7a 66
:3 0 67 :3 0 2a37
2a38 0 66 :3 0
77 :3 0 2a3a 2a3b
0 9f :2 0 2
:4 0 20d :2 0 79
:2 0 27 :2 0 186a
2a40 2a42 :3 0 186d
2a3c 2a44 2a39 2a46
2a45 :2 0 69 :3 0
7a :3 0 2a48 2a49
0 6b :4 0 20e
:4 0 6e :2 0 6f
:3 0 1871 2a4d 2a4f
:3 0 6e :2 0 82
:4 0 1874 2a51 2a53
:3 0 6e :2 0 a2
:4 0 1877 2a55 2a57
:3 0 6e :2 0 9c
:3 0 187a 2a59 2a5b
:3 0 6e :2 0 a3
:4 0 187d 2a5d 2a5f
:3 0 6e :2 0 a4
:3 0 a5 :3 0 27
:2 0 a6 :2 0 1880
2a62 2a66 1884 2a61
2a68 :3 0 1887 2a4a
2a6a :2 0 2a77 69
:3 0 7a :3 0 2a6c
2a6d 0 6b :4 0
203 :4 0 6e :2 0
51 :3 0 188a 2a71
2a73 :3 0 188d 2a6e
2a75 :2 0 2a77 1890
2a78 2a47 2a77 0
2a79 1893 0 2a7a
1895 2a7d :3 0 2a7d
0 2a7d 2a7c 2a7a
2a7b :6 0 2a7f 67
:3 0 1898 2a81 189a
2a80 2a7f :2 0 2a82
189c :2 0 2a84 0
2a84 2a83 2a26 2a82
:6 0 2c26 1 :3 0
15 :3 0 9b :3 0
2a86 2a87 0 20f
:4 0 2a89 2a8a :3 0
2a88 2a8b 0 2c26
51 :3 0 210 :4 0
6e :2 0 211 :4 0
189f 2a8f 2a91 :3 0
6e :2 0 212 :4 0
18a2 2a93 2a95 :3 0
6e :2 0 213 :4 0
18a5 2a97 2a99 :3 0
6e :2 0 214 :4 0
18a8 2a9b 2a9d :3 0
6e :2 0 215 :4 0
18ab 2a9f 2aa1 :3 0
6e :2 0 216 :4 0
18ae 2aa3 2aa5 :3 0
6e :2 0 217 :4 0
18b1 2aa7 2aa9 :3 0
6e :2 0 218 :4 0
18b4 2aab 2aad :3 0
6e :2 0 219 :4 0
18b7 2aaf 2ab1 :3 0
6e :2 0 21a :4 0
18ba 2ab3 2ab5 :3 0
6e :2 0 21b :4 0
18bd 2ab7 2ab9 :3 0
6e :2 0 21c :4 0
18c0 2abb 2abd :3 0
6e :2 0 17 :3 0
18c3 2abf 2ac1 :3 0
6e :2 0 21d :4 0
18c6 2ac3 2ac5 :3 0
6e :2 0 21e :4 0
18c9 2ac7 2ac9 :3 0
6e :2 0 17 :3 0
18cc 2acb 2acd :3 0
6e :2 0 21f :4 0
18cf 2acf 2ad1 :3 0
6e :2 0 220 :4 0
18d2 2ad3 2ad5 :3 0
6e :2 0 221 :4 0
18d5 2ad7 2ad9 :3 0
6e :2 0 2b :3 0
18d8 2adb 2add :3 0
6e :2 0 222 :4 0
18db 2adf 2ae1 :3 0
6e :2 0 223 :4 0
18de 2ae3 2ae5 :3 0
6e :2 0 224 :4 0
18e1 2ae7 2ae9 :3 0
6e :2 0 225 :4 0
18e4 2aeb 2aed :3 0
6e :2 0 226 :4 0
18e7 2aef 2af1 :3 0
6e :2 0 219 :4 0
18ea 2af3 2af5 :3 0
6e :2 0 21a :4 0
18ed 2af7 2af9 :3 0
6e :2 0 227 :4 0
18f0 2afb 2afd :3 0
6e :2 0 228 :4 0
18f3 2aff 2b01 :3 0
6e :2 0 17 :3 0
18f6 2b03 2b05 :3 0
6e :2 0 229 :4 0
18f9 2b07 2b09 :3 0
6e :2 0 22a :4 0
18fc 2b0b 2b0d :3 0
6e :2 0 22b :4 0
18ff 2b0f 2b11 :3 0
6e :2 0 22c :4 0
1902 2b13 2b15 :3 0
6e :2 0 22d :4 0
1905 2b17 2b19 :3 0
6e :2 0 2c :3 0
1908 2b1b 2b1d :3 0
6e :2 0 222 :4 0
190b 2b1f 2b21 :3 0
6e :2 0 22e :4 0
190e 2b23 2b25 :3 0
2a8d 2b26 0 2c26
1f4 :3 0 1f5 :3 0
51 :3 0 2b2a :4 0
2b2b :2 0 2b42 19d
:4 0 2b2d :3 0 8a
:3 0 8f :2 0 1913
2b30 2b31 :3 0 15
:3 0 9b :3 0 2b33
2b34 0 30 :2 0
2b35 2b36 0 2b38
1916 2b3f 15 :3 0
9b :3 0 2b39 2b3a
0 10d :2 0 2b3b
2b3c 0 2b3e 1918
2b40 2b32 2b38 0
2b41 0 2b3e 0
2b41 191a 0 2b42
191d 2ba0 9d :3 0
15 :3 0 9b :3 0
2b44 2b45 0 9c
:3 0 2b46 2b47 0
2b49 1920 2b4b 1922
2b4a 2b49 :2 0 2b9e
9e :3 0 15 :3 0
9b :3 0 2b4e 2b4f
0 9c :3 0 2b50
2b51 0 2b96 66
:3 0 67 :3 0 2b53
2b54 0 66 :3 0
77 :3 0 2b56 2b57
0 9f :2 0 2
:4 0 22f :2 0 79
:2 0 27 :2 0 1924
2b5c 2b5e :3 0 1927
2b58 2b60 2b55 2b62
2b61 :2 0 69 :3 0
7a :3 0 2b64 2b65
0 6b :4 0 230
:4 0 6e :2 0 6f
:3 0 192b 2b69 2b6b
:3 0 6e :2 0 82
:4 0 192e 2b6d 2b6f
:3 0 6e :2 0 a2
:4 0 1931 2b71 2b73
:3 0 6e :2 0 9c
:3 0 1934 2b75 2b77
:3 0 6e :2 0 a3
:4 0 1937 2b79 2b7b
:3 0 6e :2 0 a4
:3 0 a5 :3 0 27
:2 0 a6 :2 0 193a
2b7e 2b82 193e 2b7d
2b84 :3 0 1941 2b66
2b86 :2 0 2b93 69
:3 0 7a :3 0 2b88
2b89 0 6b :4 0
203 :4 0 6e :2 0
51 :3 0 1944 2b8d
2b8f :3 0 1947 2b8a
2b91 :2 0 2b93 194a
2b94 2b63 2b93 0
2b95 194d 0 2b96
194f 2b99 :3 0 2b99
0 2b99 2b98 2b96
2b97 :6 0 2b9b 69
:3 0 1952 2b9d 1954
2b9c 2b9b :2 0 2b9e
1956 :2 0 2ba0 0
2ba0 2b9f 2b42 2b9e
:6 0 2c26 1 :3 0
51 :3 0 231 :4 0
6e :2 0 232 :4 0
1959 2ba4 2ba6 :3 0
6e :2 0 233 :4 0
195c 2ba8 2baa :3 0
6e :2 0 234 :4 0
195f 2bac 2bae :3 0
6e :2 0 74 :3 0
73 :3 0 38 :3 0
30 :2 0 1962 2bb2
2bb5 1965 2bb1 2bb7
1967 2bb0 2bb9 :3 0
6e :2 0 235 :4 0
196a 2bbb 2bbd :3 0
6e :2 0 82 :4 0
196d 2bbf 2bc1 :3 0
6e :2 0 74 :3 0
73 :3 0 3d :3 0
30 :2 0 1970 2bc5
2bc8 1973 2bc4 2bca
1975 2bc3 2bcc :3 0
6e :2 0 236 :4 0
1978 2bce 2bd0 :3 0
6e :2 0 237 :4 0
197b 2bd2 2bd4 :3 0
6e :2 0 238 :4 0
197e 2bd6 2bd8 :3 0
6e :2 0 239 :4 0
1981 2bda 2bdc :3 0
6e :2 0 23a :4 0
1984 2bde 2be0 :3 0
6e :2 0 23b :4 0
1987 2be2 2be4 :3 0
6e :2 0 23c :4 0
198a 2be6 2be8 :3 0
6e :2 0 23d :4 0
198d 2bea 2bec :3 0
6e :2 0 23e :4 0
1990 2bee 2bf0 :3 0
6e :2 0 74 :3 0
73 :3 0 38 :3 0
30 :2 0 1993 2bf4
2bf7 1996 2bf3 2bf9
1998 2bf2 2bfb :3 0
6e :2 0 23f :4 0
199b 2bfd 2bff :3 0
6e :2 0 240 :4 0
199e 2c01 2c03 :3 0
6e :2 0 74 :3 0
73 :3 0 3d :3 0
30 :2 0 19a1 2c07
2c0a 19a4 2c06 2c0c
19a6 2c05 2c0e :3 0
6e :2 0 241 :4 0
19a9 2c10 2c12 :3 0
6e :2 0 242 :4 0
19ac 2c14 2c16 :3 0
6e :2 0 243 :4 0
19af 2c18 2c1a :3 0
6e :2 0 244 :4 0
19b2 2c1c 2c1e :3 0
6e :2 0 245 :4 0
19b5 2c20 2c22 :3 0
2ba2 2c23 0 2c26
ae :3 0 19b8 3100
73 :3 0 1a6 :3 0
2c :3 0 19c5 2c28
2c2a 30 :2 0 19c7
2c27 2c2d 8f :2 0
30 :2 0 19cc 2c2f
2c31 :3 0 1f4 :3 0
1f5 :3 0 246 :4 0
2c35 :4 0 2c36 :2 0
2e96 15 :3 0 9b
:3 0 2c38 2c39 0
30 :2 0 2c3a 2c3b
0 2e96 51 :3 0
247 :4 0 6e :2 0
248 :4 0 19cf 2c3f
2c41 :3 0 6e :2 0
249 :4 0 19d2 2c43
2c45 :3 0 6e :2 0
24a :4 0 19d5 2c47
2c49 :3 0 6e :2 0
24b :4 0 19d8 2c4b
2c4d :3 0 6e :2 0
1fc :4 0 19db 2c4f
2c51 :3 0 6e :2 0
17 :3 0 19de 2c53
2c55 :3 0 6e :2 0
1fd :4 0 19e1 2c57
2c59 :3 0 6e :2 0
24c :4 0 19e4 2c5b
2c5d :3 0 6e :2 0
2b :3 0 19e7 2c5f
2c61 :3 0 6e :2 0
24d :4 0 19ea 2c63
2c65 :3 0 2c3d 2c66
0 2e96 1f4 :3 0
1f5 :3 0 51 :3 0
2c6a :4 0 2c6b :2 0
2c82 19d :4 0 2c6d
:3 0 8a :3 0 8f
:2 0 19ef 2c70 2c71
:3 0 15 :3 0 9b
:3 0 2c73 2c74 0
30 :2 0 2c75 2c76
0 2c78 19f2 2c7f
15 :3 0 9b :3 0
2c79 2c7a 0 10d
:2 0 2c7b 2c7c 0
2c7e 19f4 2c80 2c72
2c78 0 2c81 0
2c7e 0 2c81 19f6
0 2c82 19f9 2ce0
9d :3 0 15 :3 0
9b :3 0 2c84 2c85
0 9c :3 0 2c86
2c87 0 2c89 19fc
2c8b 19fe 2c8a 2c89
:2 0 2cde 9e :3 0
15 :3 0 9b :3 0
2c8e 2c8f 0 9c
:3 0 2c90 2c91 0
2cd6 66 :3 0 67
:3 0 2c93 2c94 0
66 :3 0 77 :3 0
2c96 2c97 0 9f
:2 0 2 :4 0 24e
:2 0 79 :2 0 27
:2 0 1a00 2c9c 2c9e
:3 0 1a03 2c98 2ca0
2c95 2ca2 2ca1 :2 0
69 :3 0 7a :3 0
2ca4 2ca5 0 6b
:4 0 24f :4 0 6e
:2 0 6f :3 0 1a07
2ca9 2cab :3 0 6e
:2 0 82 :4 0 1a0a
2cad 2caf :3 0 6e
:2 0 a2 :4 0 1a0d
2cb1 2cb3 :3 0 6e
:2 0 9c :3 0 1a10
2cb5 2cb7 :3 0 6e
:2 0 a3 :4 0 1a13
2cb9 2cbb :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 1a16 2cbe 2cc2
1a1a 2cbd 2cc4 :3 0
1a1d 2ca6 2cc6 :2 0
2cd3 69 :3 0 7a
:3 0 2cc8 2cc9 0
6b :4 0 203 :4 0
6e :2 0 51 :3 0
1a20 2ccd 2ccf :3 0
1a23 2cca 2cd1 :2 0
2cd3 1a26 2cd4 2ca3
2cd3 0 2cd5 1a29
0 2cd6 1a2b 2cd9
:3 0 2cd9 0 2cd9
2cd8 2cd6 2cd7 :6 0
2cdb 6b :3 0 1a2e
2cdd 1a30 2cdc 2cdb
:2 0 2cde 1a32 :2 0
2ce0 0 2ce0 2cdf
2c82 2cde :6 0 2e96
1 :3 0 1f4 :3 0
1f5 :3 0 204 :4 0
2ce4 :4 0 2ce5 :2 0
2e96 15 :3 0 9b
:3 0 2ce7 2ce8 0
30 :2 0 2ce9 2cea
0 2e96 51 :3 0
250 :4 0 6e :2 0
251 :4 0 1a35 2cee
2cf0 :3 0 6e :2 0
252 :4 0 1a38 2cf2
2cf4 :3 0 6e :2 0
253 :4 0 1a3b 2cf6
2cf8 :3 0 6e :2 0
254 :4 0 1a3e 2cfa
2cfc :3 0 6e :2 0
255 :4 0 1a41 2cfe
2d00 :3 0 6e :2 0
17 :3 0 1a44 2d02
2d04 :3 0 6e :2 0
256 :4 0 1a47 2d06
2d08 :3 0 6e :2 0
2b :3 0 1a4a 2d0a
2d0c :3 0 6e :2 0
257 :4 0 1a4d 2d0e
2d10 :3 0 2cec 2d11
0 2e96 1f4 :3 0
1f5 :3 0 51 :3 0
2d15 :4 0 2d16 :2 0
2d2d 19d :4 0 2d18
:3 0 8a :3 0 8f
:2 0 1a52 2d1b 2d1c
:3 0 15 :3 0 9b
:3 0 2d1e 2d1f 0
30 :2 0 2d20 2d21
0 2d23 1a55 2d2a
15 :3 0 9b :3 0
2d24 2d25 0 10d
:2 0 2d26 2d27 0
2d29 1a57 2d2b 2d1d
2d23 0 2d2c 0
2d29 0 2d2c 1a59
0 2d2d 1a5c 2d8b
9d :3 0 15 :3 0
9b :3 0 2d2f 2d30
0 9c :3 0 2d31
2d32 0 2d34 1a5f
2d36 1a61 2d35 2d34
:2 0 2d89 9e :3 0
15 :3 0 9b :3 0
2d39 2d3a 0 9c
:3 0 2d3b 2d3c 0
2d81 66 :3 0 67
:3 0 2d3e 2d3f 0
66 :3 0 77 :3 0
2d41 2d42 0 9f
:2 0 2 :4 0 258
:2 0 79 :2 0 27
:2 0 1a63 2d47 2d49
:3 0 1a66 2d43 2d4b
2d40 2d4d 2d4c :2 0
69 :3 0 7a :3 0
2d4f 2d50 0 6b
:4 0 259 :4 0 6e
:2 0 6f :3 0 1a6a
2d54 2d56 :3 0 6e
:2 0 82 :4 0 1a6d
2d58 2d5a :3 0 6e
:2 0 a2 :4 0 1a70
2d5c 2d5e :3 0 6e
:2 0 9c :3 0 1a73
2d60 2d62 :3 0 6e
:2 0 a3 :4 0 1a76
2d64 2d66 :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 1a79 2d69 2d6d
1a7d 2d68 2d6f :3 0
1a80 2d51 2d71 :2 0
2d7e 69 :3 0 7a
:3 0 2d73 2d74 0
6b :4 0 203 :4 0
6e :2 0 51 :3 0
1a83 2d78 2d7a :3 0
1a86 2d75 2d7c :2 0
2d7e 1a89 2d7f 2d4e
2d7e 0 2d80 1a8c
0 2d81 1a8e 2d84
:3 0 2d84 0 2d84
2d83 2d81 2d82 :6 0
2d86 6d :3 0 1a91
2d88 1a93 2d87 2d86
:2 0 2d89 1a95 :2 0
2d8b 0 2d8b 2d8a
2d2d 2d89 :6 0 2e96
1 :3 0 15 :3 0
9b :3 0 2d8d 2d8e
0 20f :4 0 2d90
2d91 :3 0 2d8f 2d92
0 2e96 51 :3 0
210 :4 0 6e :2 0
25a :4 0 1a98 2d96
2d98 :3 0 6e :2 0
213 :4 0 1a9b 2d9a
2d9c :3 0 6e :2 0
25b :4 0 1a9e 2d9e
2da0 :3 0 6e :2 0
25c :4 0 1aa1 2da2
2da4 :3 0 6e :2 0
217 :4 0 1aa4 2da6
2da8 :3 0 6e :2 0
25d :4 0 1aa7 2daa
2dac :3 0 6e :2 0
21a :4 0 1aaa 2dae
2db0 :3 0 6e :2 0
25e :4 0 1aad 2db2
2db4 :3 0 6e :2 0
21c :4 0 1ab0 2db6
2db8 :3 0 6e :2 0
17 :3 0 1ab3 2dba
2dbc :3 0 6e :2 0
21d :4 0 1ab6 2dbe
2dc0 :3 0 6e :2 0
21e :4 0 1ab9 2dc2
2dc4 :3 0 6e :2 0
17 :3 0 1abc 2dc6
2dc8 :3 0 6e :2 0
21f :4 0 1abf 2dca
2dcc :3 0 6e :2 0
220 :4 0 1ac2 2dce
2dd0 :3 0 6e :2 0
25f :4 0 1ac5 2dd2
2dd4 :3 0 6e :2 0
2b :3 0 1ac8 2dd6
2dd8 :3 0 6e :2 0
260 :4 0 1acb 2dda
2ddc :3 0 2d94 2ddd
0 2e96 1f4 :3 0
1f5 :3 0 51 :3 0
2de1 :4 0 2de2 :2 0
2df9 19d :4 0 2de4
:3 0 8a :3 0 8f
:2 0 1ad0 2de7 2de8
:3 0 15 :3 0 9b
:3 0 2dea 2deb 0
30 :2 0 2dec 2ded
0 2def 1ad3 2df6
15 :3 0 9b :3 0
2df0 2df1 0 10d
:2 0 2df2 2df3 0
2df5 1ad5 2df7 2de9
2def 0 2df8 0
2df5 0 2df8 1ad7
0 2df9 1ada 2e57
9d :3 0 15 :3 0
9b :3 0 2dfb 2dfc
0 9c :3 0 2dfd
2dfe 0 2e00 1add
2e02 1adf 2e01 2e00
:2 0 2e55 9e :3 0
15 :3 0 9b :3 0
2e05 2e06 0 9c
:3 0 2e07 2e08 0
2e4d 66 :3 0 67
:3 0 2e0a 2e0b 0
66 :3 0 77 :3 0
2e0d 2e0e 0 9f
:2 0 2 :4 0 261
:2 0 79 :2 0 27
:2 0 1ae1 2e13 2e15
:3 0 1ae4 2e0f 2e17
2e0c 2e19 2e18 :2 0
69 :3 0 7a :3 0
2e1b 2e1c 0 6b
:4 0 262 :4 0 6e
:2 0 6f :3 0 1ae8
2e20 2e22 :3 0 6e
:2 0 82 :4 0 1aeb
2e24 2e26 :3 0 6e
:2 0 a2 :4 0 1aee
2e28 2e2a :3 0 6e
:2 0 9c :3 0 1af1
2e2c 2e2e :3 0 6e
:2 0 a3 :4 0 1af4
2e30 2e32 :3 0 6e
:2 0 a4 :3 0 a5
:3 0 27 :2 0 a6
:2 0 1af7 2e35 2e39
1afb 2e34 2e3b :3 0
1afe 2e1d 2e3d :2 0
2e4a 69 :3 0 7a
:3 0 2e3f 2e40 0
6b :4 0 203 :4 0
6e :2 0 51 :3 0
1b01 2e44 2e46 :3 0
1b04 2e41 2e48 :2 0
2e4a 1b07 2e4b 2e1a
2e4a 0 2e4c 1b0a
0 2e4d 1b0c 2e50
:3 0 2e50 0 2e50
2e4f 2e4d 2e4e :6 0
2e52 6f :3 0 1b0f
2e54 1b11 2e53 2e52
:2 0 2e55 1b13 :2 0
2e57 0 2e57 2e56
2df9 2e55 :6 0 2e96
1 :3 0 51 :3 0
263 :4 0 6e :2 0
264 :4 0 1b16 2e5b
2e5d :3 0 6e :2 0
265 :4 0 1b19 2e5f
2e61 :3 0 6e :2 0
266 :4 0 1b1c 2e63
2e65 :3 0 6e :2 0
74 :3 0 73 :3 0
38 :3 0 30 :2 0
1b1f 2e69 2e6c 1b22
2e68 2e6e 1b24 2e67
2e70 :3 0 6e :2 0
267 :4 0 1b27 2e72
2e74 :3 0 6e :2 0
74 :3 0 73 :3 0
3d :3 0 30 :2 0
1b2a 2e78 2e7b 1b2d
2e77 2e7d 1b2f 2e76
2e7f :3 0 6e :2 0
268 :4 0 1b32 2e81
2e83 :3 0 6e :2 0
237 :4 0 1b35 2e85
2e87 :3 0 6e :2 0
269 :4 0 1b38 2e89
2e8b :3 0 6e :2 0
26a :4 0 1b3b 2e8d
2e8f :3 0 6e :2 0
26b :4 0 1b3e 2e91
2e93 :3 0 2e59 2e94
0 2e96 1b41 2e97
2c32 2e96 0 3102
1f4 :3 0 1f5 :3 0
246 :4 0 2e9a :4 0
2e9b :2 0 30ff 15
:3 0 9b :3 0 2e9d
2e9e 0 30 :2 0
2e9f 2ea0 0 30ff
51 :3 0 247 :4 0
6e :2 0 26c :4 0
1b4e 2ea4 2ea6 :3 0
6e :2 0 26d :4 0
1b51 2ea8 2eaa :3 0
6e :2 0 26e :4 0
1b54 2eac 2eae :3 0
6e :2 0 26f :4 0
1b57 2eb0 2eb2 :3 0
6e :2 0 209 :4 0
1b5a 2eb4 2eb6 :3 0
6e :2 0 270 :4 0
1b5d 2eb8 2eba :3 0
6e :2 0 17 :3 0
1b60 2ebc 2ebe :3 0
6e :2 0 271 :4 0
1b63 2ec0 2ec2 :3 0
6e :2 0 272 :4 0
1b66 2ec4 2ec6 :3 0
6e :2 0 2c :3 0
1b69 2ec8 2eca :3 0
6e :2 0 273 :4 0
1b6c 2ecc 2ece :3 0
6e :2 0 274 :4 0
1b6f 2ed0 2ed2 :3 0
2ea2 2ed3 0 30ff
1f4 :3 0 1f5 :3 0
51 :3 0 2ed7 :4 0
2ed8 :2 0 2eef 19d
:4 0 2eda :3 0 8a
:3 0 8f :2 0 1b74
2edd 2ede :3 0 15
:3 0 9b :3 0 2ee0
2ee1 0 30 :2 0
2ee2 2ee3 0 2ee5
1b77 2eec 15 :3 0
9b :3 0 2ee6 2ee7
0 10d :2 0 2ee8
2ee9 0 2eeb 1b79
2eed 2edf 2ee5 0
2eee 0 2eeb 0
2eee 1b7b 0 2eef
1b7e 2f4d 9d :3 0
15 :3 0 9b :3 0
2ef1 2ef2 0 9c
:3 0 2ef3 2ef4 0
2ef6 1b81 2ef8 1b83
2ef7 2ef6 :2 0 2f4b
9e :3 0 15 :3 0
9b :3 0 2efb 2efc
0 9c :3 0 2efd
2efe 0 2f43 66
:3 0 67 :3 0 2f00
2f01 0 66 :3 0
77 :3 0 2f03 2f04
0 9f :2 0 2
:4 0 275 :2 0 79
:2 0 27 :2 0 1b85
2f09 2f0b :3 0 1b88
2f05 2f0d 2f02 2f0f
2f0e :2 0 69 :3 0
7a :3 0 2f11 2f12
0 6b :4 0 276
:4 0 6e :2 0 6f
:3 0 1b8c 2f16 2f18
:3 0 6e :2 0 82
:4 0 1b8f 2f1a 2f1c
:3 0 6e :2 0 a2
:4 0 1b92 2f1e 2f20
:3 0 6e :2 0 9c
:3 0 1b95 2f22 2f24
:3 0 6e :2 0 a3
:4 0 1b98 2f26 2f28
:3 0 6e :2 0 a4
:3 0 a5 :3 0 27
:2 0 a6 :2 0 1b9b
2f2b 2f2f 1b9f 2f2a
2f31 :3 0 1ba2 2f13
2f33 :2 0 2f40 69
:3 0 7a :3 0 2f35
2f36 0 6b :4 0
203 :4 0 6e :2 0
51 :3 0 1ba5 2f3a
2f3c :3 0 1ba8 2f37
2f3e :2 0 2f40 1bab
2f41 2f10 2f40 0
2f42 1bae 0 2f43
1bb0 2f46 :3 0 2f46
0 2f46 2f45 2f43
2f44 :6 0 2f48 71
:3 0 1bb3 2f4a 1bb5
2f49 2f48 :2 0 2f4b
1bb7 :2 0 2f4d 0
2f4d 2f4c 2eef 2f4b
:6 0 30ff 1 :3 0
1f4 :3 0 1f5 :3 0
204 :4 0 2f51 :4 0
2f52 :2 0 30ff 15
:3 0 9b :3 0 2f54
2f55 0 30 :2 0
2f56 2f57 0 30ff
51 :3 0 250 :4 0
6e :2 0 277 :4 0
1bba 2f5b 2f5d :3 0
6e :2 0 278 :4 0
1bbd 2f5f 2f61 :3 0
6e :2 0 279 :4 0
1bc0 2f63 2f65 :3 0
6e :2 0 27a :4 0
1bc3 2f67 2f69 :3 0
6e :2 0 255 :4 0
1bc6 2f6b 2f6d :3 0
6e :2 0 17 :3 0
1bc9 2f6f 2f71 :3 0
6e :2 0 256 :4 0
1bcc 2f73 2f75 :3 0
6e :2 0 2c :3 0
1bcf 2f77 2f79 :3 0
6e :2 0 27b :4 0
1bd2 2f7b 2f7d :3 0
2f59 2f7e 0 30ff
1f4 :3 0 1f5 :3 0
51 :3 0 2f82 :4 0
2f83 :2 0 2f9a 19d
:4 0 2f85 :3 0 8a
:3 0 8f :2 0 1bd7
2f88 2f89 :3 0 15
:3 0 9b :3 0 2f8b
2f8c 0 30 :2 0
2f8d 2f8e 0 2f90
1bda 2f97 15 :3 0
9b :3 0 2f91 2f92
0 10d :2 0 2f93
2f94 0 2f96 1bdc
2f98 2f8a 2f90 0
2f99 0 2f96 0
2f99 1bde 0 2f9a
1be1 2ff8 9d :3 0
15 :3 0 9b :3 0
2f9c 2f9d 0 9c
:3 0 2f9e 2f9f 0
2fa1 1be4 2fa3 1be6
2fa2 2fa1 :2 0 2ff6
9e :3 0 15 :3 0
9b :3 0 2fa6 2fa7
0 9c :3 0 2fa8
2fa9 0 2fee 66
:3 0 67 :3 0 2fab
2fac 0 66 :3 0
77 :3 0 2fae 2faf
0 9f :2 0 2
:4 0 27c :2 0 79
:2 0 27 :2 0 1be8
2fb4 2fb6 :3 0 1beb
2fb0 2fb8 2fad 2fba
2fb9 :2 0 69 :3 0
7a :3 0 2fbc 2fbd
0 6b :4 0 27d
:4 0 6e :2 0 6f
:3 0 1bef 2fc1 2fc3
:3 0 6e :2 0 82
:4 0 1bf2 2fc5 2fc7
:3 0 6e :2 0 a2
:4 0 1bf5 2fc9 2fcb
:3 0 6e :2 0 9c
:3 0 1bf8 2fcd 2fcf
:3 0 6e :2 0 a3
:4 0 1bfb 2fd1 2fd3
:3 0 6e :2 0 a4
:3 0 a5 :3 0 27
:2 0 a6 :2 0 1bfe
2fd6 2fda 1c02 2fd5
2fdc :3 0 1c05 2fbe
2fde :2 0 2feb 69
:3 0 7a :3 0 2fe0
2fe1 0 6b :4 0
203 :4 0 6e :2 0
51 :3 0 1c08 2fe5
2fe7 :3 0 1c0b 2fe2
2fe9 :2 0 2feb 1c0e
2fec 2fbb 2feb 0
2fed 1c11 0 2fee
1c13 2ff1 :3 0 2ff1
0 2ff1 2ff0 2fee
2fef :6 0 2ff3 73
:3 0 1c16 2ff5 1c18
2ff4 2ff3 :2 0 2ff6
1c1a :2 0 2ff8 0
2ff8 2ff7 2f9a 2ff6
:6 0 30ff 1 :3 0
15 :3 0 9b :3 0
2ffa 2ffb 0 20f
:4 0 2ffd 2ffe :3 0
2ffc 2fff 0 30ff
51 :3 0 210 :4 0
6e :2 0 25a :4 0
1c1d 3003 3005 :3 0
6e :2 0 213 :4 0
1c20 3007 3009 :3 0
6e :2 0 214 :4 0
1c23 300b 300d :3 0
6e :2 0 27e :4 0
1c26 300f 3011 :3 0
6e :2 0 27f :4 0
1c29 3013 3015 :3 0
6e :2 0 219 :4 0
1c2c 3017 3019 :3 0
6e :2 0 21a :4 0
1c2f 301b 301d :3 0
6e :2 0 280 :4 0
1c32 301f 3021 :3 0
6e :2 0 21c :4 0
1c35 3023 3025 :3 0
6e :2 0 17 :3 0
1c38 3027 3029 :3 0
6e :2 0 229 :4 0
1c3b 302b 302d :3 0
6e :2 0 281 :4 0
1c3e 302f 3031 :3 0
6e :2 0 22c :4 0
1c41 3033 3035 :3 0
6e :2 0 22d :4 0
1c44 3037 3039 :3 0
6e :2 0 2c :3 0
1c47 303b 303d :3 0
6e :2 0 260 :4 0
1c4a 303f 3041 :3 0
3001 3042 0 30ff
1f4 :3 0 1f5 :3 0
51 :3 0 3046 :4 0
3047 :2 0 305e 19d
:4 0 3049 :3 0 8a
:3 0 8f :2 0 1c4f
304c 304d :3 0 15
:3 0 9b :3 0 304f
3050 0 30 :2 0
3051 3052 0 3054
1c52 305b 15 :3 0
9b :3 0 3055 3056
0 10d :2 0 3057
3058 0 305a 1c54
305c 304e 3054 0
305d 0 305a 0
305d 1c56 0 305e
1c59 30bc 9d :3 0
15 :3 0 9b :3 0
3060 3061 0 9c
:3 0 3062 3063 0
3065 1c5c 3067 1c5e
3066 3065 :2 0 30ba
9e :3 0 15 :3 0
9b :3 0 306a 306b
0 9c :3 0 306c
306d 0 30b2 66
:3 0 67 :3 0 306f
3070 0 66 :3 0
77 :3 0 3072 3073
0 9f :2 0 2
:4 0 282 :2 0 79
:2 0 27 :2 0 1c60
3078 307a :3 0 1c63
3074 307c 3071 307e
307d :2 0 69 :3 0
7a :3 0 3080 3081
0 6b :4 0 283
:4 0 6e :2 0 6f
:3 0 1c67 3085 3087
:3 0 6e :2 0 82
:4 0 1c6a 3089 308b
:3 0 6e :2 0 a2
:4 0 1c6d 308d 308f
:3 0 6e :2 0 9c
:3 0 1c70 3091 3093
:3 0 6e :2 0 a3
:4 0 1c73 3095 3097
:3 0 6e :2 0 a4
:3 0 a5 :3 0 27
:2 0 a6 :2 0 1c76
309a 309e 1c7a 3099
30a0 :3 0 1c7d 3082
30a2 :2 0 30af 69
:3 0 7a :3 0 30a4
30a5 0 6b :4 0
203 :4 0 6e :2 0
51 :3 0 1c80 30a9
30ab :3 0 1c83 30a6
30ad :2 0 30af 1c86
30b0 307f 30af 0
30b1 1c89 0 30b2
1c8b 30b5 :3 0 30b5
0 30b5 30b4 30b2
30b3 :6 0 30b7 75
:3 0 1c8e 30b9 1c90
30b8 30b7 :2 0 30ba
1c92 :2 0 30bc 0
30bc 30bb 305e 30ba
:6 0 30ff 1 :3 0
51 :3 0 284 :4 0
6e :2 0 264 :4 0
1c95 30c0 30c2 :3 0
6e :2 0 285 :4 0
1c98 30c4 30c6 :3 0
6e :2 0 286 :4 0
1c9b 30c8 30ca :3 0
6e :2 0 74 :3 0
73 :3 0 38 :3 0
30 :2 0 1c9e 30ce
30d1 1ca1 30cd 30d3
1ca3 30cc 30d5 :3 0
6e :2 0 23f :4 0
1ca6 30d7 30d9 :3 0
6e :2 0 287 :4 0
1ca9 30db 30dd :3 0
6e :2 0 74 :3 0
73 :3 0 3d :3 0
30 :2 0 1cac 30e1
30e4 1caf 30e0 30e6
1cb1 30df 30e8 :3 0
6e :2 0 288 :4 0
1cb4 30ea 30ec :3 0
6e :2 0 289 :4 0
1cb7 30ee 30f0 :3 0
6e :2 0 237 :4 0
1cba 30f2 30f4 :3 0
6e :2 0 28a :4 0
1cbd 30f6 30f8 :3 0
6e :2 0 26b :4 0
1cc0 30fa 30fc :3 0
30be 30fd 0 30ff
1cc3 3101 2917 2c26
0 3102 0 30ff
0 3102 1cd0 0
3149 66 :3 0 67
:3 0 3103 3104 0
66 :3 0 77 :3 0
3106 3107 0 27
:2 0 2 :4 0 28b
:2 0 79 :2 0 27
:2 0 1cd4 310c 310e
:3 0 1cd7 3108 3110
3105 3112 3111 :2 0
69 :3 0 7a :3 0
3114 3115 0 6b
:4 0 28c :4 0 6e
:2 0 6f :3 0 1cdb
3119 311b :3 0 1cde
3116 311d :2 0 311f
1ce1 3120 3113 311f
0 3121 1ce3 0
3149 66 :3 0 67
:3 0 3122 3123 0
66 :3 0 77 :3 0
3125 3126 0 27
:2 0 2 :4 0 28d
:2 0 79 :2 0 27
:2 0 1ce5 312b 312d
:3 0 1ce8 3127 312f
3124 3131 3130 :2 0
69 :3 0 7a :3 0
3133 3134 0 6b
:4 0 28e :4 0 6e
:2 0 6f :3 0 1cec
3138 313a :3 0 1cef
3135 313c :2 0 313e
1cf2 313f 3132 313e
0 3140 1cf4 0
3149 14 :3 0 88
:3 0 28f :3 0 3142
3143 0 51 :3 0
1cf6 3144 3146 3147
:2 0 3149 1cf8 31ea
15 :3 0 290 :2 0
4 314a 314b 0
66 :3 0 67 :3 0
314d 314e 0 66
:3 0 77 :3 0 3150
3151 0 9f :2 0
2 :4 0 291 :2 0
79 :2 0 27 :2 0
1d26 3156 3158 :3 0
1d29 3152 315a 314f
315c 315b :2 0 69
:3 0 7a :3 0 315e
315f 0 6b :4 0
292 :4 0 6e :2 0
6f :3 0 1d2d 3163
3165 :3 0 1d30 3160
3167 :2 0 3180 69
:3 0 7a :3 0 3169
316a 0 6b :4 0
293 :4 0 6e :2 0
15 :3 0 294 :3 0
316f 3170 0 1d33
316e 3172 :3 0 6e
:2 0 e5 :4 0 1d36
3174 3176 :3 0 6e
:2 0 15 :3 0 295
:3 0 3179 317a 0
1d39 3178 317c :3 0
1d3c 316b 317e :2 0
3180 1d3f 3181 315d
3180 0 3182 1d42
0 3192 296 :3 0
65 :3 0 15 :3 0
294 :3 0 297 :3 0
15 :3 0 295 :3 0
5f :3 0 99 :4 0
298 1 :a 0 318e
:2 0 3192 3184 318c
:2 0 14 :3 0 65
:3 0 3190 :2 0 3192
1d44 3194 1d48 3193
3192 :2 0 31e8 9e
:3 0 15 :3 0 9b
:3 0 3197 3198 0
9c :3 0 3199 319a
0 31e0 66 :3 0
67 :3 0 319c 319d
0 66 :3 0 77
:3 0 319f 31a0 0
9f :2 0 2 :4 0
299 :2 0 79 :2 0
27 :2 0 1d4a 31a5
31a7 :3 0 1d4d 31a1
31a9 319e 31ab 31aa
:2 0 69 :3 0 7a
:3 0 31ad 31ae 0
6b :4 0 29a :4 0
6e :2 0 6f :3 0
1d51 31b2 31b4 :3 0
1d54 31af 31b6 :2 0
31d0 69 :3 0 7a
:3 0 31b8 31b9 0
6b :4 0 a2 :4 0
6e :2 0 9c :3 0
1d57 31bd 31bf :3 0
6e :2 0 a3 :4 0
1d5a 31c1 31c3 :3 0
6e :2 0 a4 :3 0
a5 :3 0 27 :2 0
a6 :2 0 1d5d 31c6
31ca 1d61 31c5 31cc
:3 0 1d64 31ba 31ce
:2 0 31d0 1d67 31d1
31ac 31d0 0 31d2
1d6a 0 31e0 296
:3 0 65 :3 0 297
:3 0 15 :3 0 295
:3 0 5f :3 0 99
:4 0 29b 1 :a 0
31dc :2 0 31e0 31d4
31da :2 0 14 :3 0
65 :3 0 31de :2 0
31e0 1d6c 31e3 :3 0
31e3 0 31e3 31e2
31e0 31e1 :6 0 31e5
1 :3 0 1d71 31e7
1d73 31e6 31e5 :2 0
31e8 1d75 :2 0 31ea
1d78 31ea 31e9 3149
31e8 :6 0 31eb :2 0
2 44 31ea 31ee
:3 0 31ed 31eb 31ef
:8 0 
1dbd
4
:3 0 1 3 1
7 1 b 1
f 1 13 1
17 1 1b 1
1f 1 23 1
27 1 2b 1
2f 1 33 1
37 1 3b f
6 a e 12
16 1a 1e 22
26 2a 2e 32
36 3a 3e 1
48 1 46 1
4f 1 4d 1
56 1 54 1
5d 1 5b 1
64 1 62 1
6b 1 69 1
72 1 70 1
79 1 77 1
80 1 7e 1
87 1 85 1
8c 1 91 1
98 1 96 1
9f 1 9d 1
a6 1 a4 1
ad 1 ab 1
b4 1 b2 1
bb 1 b9 1
c0 1 c6 1
cc 1 d2 1
d8 1 de 1
e4 1 ea 1
f0 1 f6 1
fc 1 102 1
108 1 10e 1
114 1 11a 1
120 1 126 1
12c 1 132 1
138 1 13e 1
144 1 14a 1
150 1 156 1
15c 1 162 1
16a 1 168 1
171 1 16f 1
176 1 17e 1
17c 1 183 1
18b 1 189 1
192 1 190 1
199 1 197 1
19e 1 1a4 1
1aa 1 1b2 1
1b0 1 1b9 1
1b7 1 1be 1
1c4 1 1cc 1
1ca 1 1d1 1
1d7 1 1dd 1
1e3 1 1e9 1
1ee 1 1fb 1
207 2 20a 20c
2 20f 211 2
213 215 2 217
219 2 21b 21d
2 21f 221 2
223 225 2 227
229 2 22b 22d
2 22f 231 2
233 235 2 237
239 2 23b 23d
1 243 2 245
246 2 23f 248
2 24a 24c 1
252 2 254 255
2 24e 257 2
259 25b 1 261
2 263 264 2
25d 266 2 268
26a 1 270 2
272 273 2 26c
275 2 277 279
1 27f 2 281
282 2 27b 284
2 286 288 1
28e 2 290 291
2 28a 293 2
295 297 1 29d
2 29f 2a0 2
299 2a2 2 2a4
2a6 1 2ac 2
2ae 2af 2 2a8
2b1 2 2b3 2b5
1 2bb 2 2bd
2be 2 2b7 2c0
2 2c2 2c4 1
2ca 2 2cc 2cd
2 2c6 2cf 2
2d1 2d3 2 2d5
2d7 2 2d9 2db
2 2dd 2df 2
2e1 2e3 2 2e5
2e7 2 2e9 2eb
2 2ed 2ef 2
2f1 2f3 4 203
209 20e 2f5 1
2f7 1 2f9 2
303 305 3 301
302 307 2 310
312 2 30f 314
1 316 1 318
2 350 351 1
353 1 360 2
36d 36f 3 36b
36c 371 2 37a
37c 2 37e 380
2 382 384 1
38c 2 38e 38f
2 386 391 2
379 393 1 395
1 397 2 3a1
3a3 3 39f 3a0
3a5 2 3ae 3b0
2 3ad 3b2 1
3b4 1 3b6 2
3bd 3be 3 398
3b7 3c0 1 3c2
1 3c4 2 3d1
3d3 3 3cf 3d0
3d5 2 3de 3e0
2 3e2 3e4 2
3e6 3e8 1 3f0
2 3f2 3f3 2
3ea 3f5 2 3dd
3f7 1 3f9 1
3fb 2 405 407
3 403 404 409
2 412 414 2
411 416 1 418
1 41a 2 421
422 3 3fc 41b
424 1 426 2
429 42a 1 42e
2 42c 42e 2
432 433 1 437
2 435 437 2
43d 43e 1 442
2 440 442 2
44f 451 3 44d
44e 453 2 45c
45e 2 460 462
2 464 466 1
46e 2 470 471
2 468 473 2
45b 475 1 477
1 479 2 483
485 3 481 482
487 2 490 492
2 48f 494 1
496 1 498 2
49f 4a0 3 47a
499 4a2 1 4a4
1 4a6 1 4ac
2 4bc 4c1 1
4c8 1 4c3 2
4db 4dd 3 4d9
4da 4df 2 4e8
4ea 2 4ec 4ee
2 4f0 4f2 2
4f4 4f6 2 4f8
4fa 3 4ff 500
501 2 4fc 503
2 4e7 505 1
507 1 509 2
4d2 50a 1 50e
1 4cd 2 4cb
512 1 51d 2
519 51d 2 522
523 1 527 2
525 527 2 535
537 3 533 534
539 2 542 544
2 546 548 2
54a 54c 1 554
2 556 557 2
54e 559 2 541
55b 1 55d 1
55f 2 569 56b
3 567 568 56d
2 576 578 2
575 57a 1 57c
1 57e 2 585
586 3 560 57f
588 1 58a 3
515 58b 58e 1
591 1 597 2
5a2 5a7 1 5ae
1 5a9 2 5c1
5c3 3 5bf 5c0
5c5 2 5ce 5d0
2 5d2 5d4 2
5d6 5d8 2 5da
5dc 2 5de 5e0
3 5e5 5e6 5e7
2 5e2 5e9 2
5cd 5eb 1 5ed
1 5ef 2 5b8
5f0 1 5f4 1
5b3 2 5b1 5f8
1 5fb 2 5ff
5fe 1 604 1
60b 1 60d 3
611 612 613 3
619 61a 61b 2
623 624 1 628
2 626 628 3
62e 62f 630 3
632 633 634 4
63c 63d 63e 63f
2 645 646 1
648 1 64c 2
64a 64c 2 65c
661 1 668 1
663 2 67b 67d
3 679 67a 67f
2 688 68a 2
68c 68e 2 690
692 2 694 696
2 698 69a 3
69f 6a0 6a1 2
69c 6a3 2 687
6a5 1 6a7 1
6a9 2 672 6aa
1 6ae 1 66d
2 66b 6b2 1
6b5 2 6fd 702
1 709 1 704
2 71c 71e 3
71a 71b 720 2
729 72b 2 72d
72f 2 731 733
2 735 737 2
739 73b 3 740
741 742 2 73d
744 2 728 746
1 748 1 74a
2 713 74b 1
74f 1 70e 2
70c 753 1 756
2 759 75a 1
762 2 75e 762
2 76d 772 1
779 1 774 2
78c 78e 3 78a
78b 790 2 799
79b 2 79d 79f
2 7a1 7a3 2
7a5 7a7 2 7a9
7ab 3 7b0 7b1
7b2 2 7ad 7b4
2 798 7b6 1
7b8 1 7ba 2
783 7bb 1 7bf
1 77e 2 77c
7c3 1 7ce 2
7ca 7ce 1 7d2
1 7d9 1 7db
2 7e8 7ed 1
7f4 1 7ef 2
807 809 3 805
806 80b 2 814
816 2 818 81a
2 81c 81e 2
820 822 2 824
826 3 82b 82c
82d 2 828 82f
2 813 831 1
833 1 835 2
7fe 836 1 83a
1 7f9 2 7f7
83e 2 7dc 841
2 84c 84e 3
84a 84b 850 2
859 85b 2 85d
85f 2 861 863
1 86b 2 86d
86e 2 865 870
2 872 874 2
876 878 2 87a
87c 2 87e 880
2 858 882 1
884 1 886 2
890 892 3 88e
88f 894 2 89d
89f 2 89c 8a1
1 8a3 1 8a5
3 8ac 8ad 8ae
3 887 8a6 8b0
2 8b2 8b3 2
7c6 8b4 1 8b8
2 8ba 8bb 2
8c5 8ca 1 8d1
1 8cc 2 8e4
8e6 3 8e2 8e3
8e8 2 8f1 8f3
2 8f5 8f7 2
8f9 8fb 2 8fd
8ff 2 901 903
3 908 909 90a
2 905 90c 2
8f0 90e 1 910
1 912 2 8db
913 1 917 1
8d6 2 8d4 91b
2 921 922 1
926 2 924 926
2 92e 933 1
93a 1 935 2
94d 94f 3 94b
94c 951 2 95a
95c 2 95e 960
2 962 964 2
966 968 2 96a
96c 3 971 972
973 2 96e 975
2 959 977 1
979 1 97b 2
944 97c 1 980
1 93f 2 93d
984 2 98c 98d
1 991 2 98f
991 2 99c 99e
3 99a 99b 9a0
2 9a9 9ab 2
9ad 9af 2 9b1
9b3 1 9bb 2
9bd 9be 2 9b5
9c0 2 9c2 9c4
2 9c6 9c8 2
9ca 9cc 2 9ce
9d0 2 9a8 9d2
1 9d4 1 9d6
2 9e0 9e2 3
9de 9df 9e4 2
9ed 9ef 2 9ec
9f1 1 9f3 1
9f5 3 9fc 9fd
9fe 3 9d7 9f6
a00 1 a02 2
987 a03 1 a05
7 637 642 75b
8bc 8bf 91e a06
2 a13 a18 1
a1f 1 a1a 2
a32 a34 3 a30
a31 a36 2 a3f
a41 2 a43 a45
2 a47 a49 2
a4b a4d 2 a4f
a51 3 a56 a57
a58 2 a53 a5a
2 a3e a5c 1
a5e 1 a60 2
a29 a61 1 a65
1 a24 2 a22
a69 1 a74 2
a70 a74 2 a7d
a82 1 a89 1
a84 2 a9c a9e
3 a9a a9b aa0
2 aa9 aab 2
aad aaf 2 ab1
ab3 2 ab5 ab7
2 ab9 abb 3
ac0 ac1 ac2 2
abd ac4 2 aa8
ac6 1 ac8 1
aca 2 a93 acb
1 acf 1 a8e
2 a8c ad3 1
ade 2 ada ade
2 aea aec 3
ae8 ae9 aee 2
af7 af9 2 afb
afd 2 aff b01
1 b09 2 b0b
b0c 2 b03 b0e
2 b10 b12 2
b14 b16 2 b18
b1a 2 b1c b1e
2 af6 b20 1
b22 1 b24 2
b2e b30 3 b2c
b2d b32 2 b3b
b3d 2 b3a b3f
1 b41 1 b43
3 b4a b4b b4c
3 b25 b44 b4e
1 b50 2 ad6
b51 1 b53 2
a6c b54 2 b56
b57 1 b5b 2
b59 b5b 2 b69
b6a 2 b6e b6f
2 b73 b74 2
b78 b79 2 b7d
b7e 2 b82 b83
2 b87 b88 2
b8c b8d d b66
b67 b6c b71 b76
b7b b80 b85 b8a
b8f b90 b91 b92
2 b97 b98 1
b9e 2 b9a b9e
2 ba9 bab 3
ba7 ba8 bad 2
bb6 bb8 2 bba
bbc 2 bbe bc0
1 bc8 2 bca
bcb 2 bc2 bcd
2 bb5 bcf 1
bd1 1 bd3 2
bdd bdf 3 bdb
bdc be1 2 bea
bec 2 be9 bee
1 bf0 1 bf2
2 bf9 bfa 3
bd4 bf3 bfc 2
c00 c01 1 c05
2 c03 c05 2
c10 c12 3 c0e
c0f c14 2 c1d
c1f 2 c21 c23
2 c25 c27 1
c2d 2 c2f c30
2 c29 c32 2
c34 c36 2 c38
c3a 2 c3c c3e
2 c40 c42 2
c1c c44 1 c46
1 c48 2 c52
c54 3 c50 c51
c56 2 c5f c61
2 c5e c63 1
c65 1 c67 2
c6d c6e 3 c70
c71 c72 3 c49
c68 c74 2 c77
c76 1 c7c 1
c83 1 c85 6
b60 b63 b95 c78
c7b c86 2 c8a
c8b 1 c8f 2
c8d c8f 2 c94
c95 1 c99 2
c97 c99 2 ca0
ca1 1 ca5 2
ca3 ca5 2 caa
cab 1 caf 2
cad caf 1 cb7
2 cbb cbc 1
cc0 2 cbe cc0
1 cc5 2 cca
ccb 1 ccf 2
ccd ccf 2 cd4
cd5 1 cd9 2
cd7 cd9 1 ce1
2 ce6 ce7 1
ceb 2 ce9 ceb
2 cf0 cf1 2
cf6 cf7 2 cf3
cf9 2 cfc cff
4 d02 cc8 ce4
d01 1 d03 2
d06 d05 2 d19
d1a 2 d1c d1f
2 d22 d23 2
d2a d2b 2 d2d
d30 2 d35 d36
1 d3a 2 d38
d3a 6 d3f d40
d41 d42 d43 d44
2 d49 d4a 1
d4e 2 d4c d4e
2 d59 d5b 3
d57 d58 d5d 2
d66 d68 2 d6a
d6c 2 d6e d70
1 d76 2 d78
d79 2 d72 d7b
2 d7d d7f 2
d81 d83 2 d85
d87 2 d89 d8b
2 d65 d8d 1
d8f 1 d91 2
d9b d9d 3 d99
d9a d9f 2 da8
daa 2 da7 dac
1 dae 1 db0
2 db6 db7 3
db9 dba dbb 3
d92 db1 dbd 1
dbf 2 dd8 ddd
1 de4 1 ddf
2 df7 df9 3
df5 df6 dfb 2
e04 e06 2 e08
e0a 2 e0c e0e
2 e10 e12 2
e14 e16 3 e1b
e1c e1d 2 e18
e1f 2 e03 e21
1 e23 1 e25
2 dee e26 1
e2a 1 de9 2
de7 e2e 1 e39
2 e35 e39 1
e3d 2 e49 e4a
1 e4e 2 e4c
e4e 1 e53 1
e55 2 e47 e56
1 e58 1 e62
2 e60 e62 3
e66 e67 e68 1
e6c 2 e6a e6c
2 e8b e90 1
e97 1 e92 2
eaa eac 3 ea8
ea9 eae 2 eb7
eb9 2 ebb ebd
2 ebf ec1 2
ec3 ec5 2 ec7
ec9 3 ece ecf
ed0 2 ecb ed2
2 eb6 ed4 1
ed6 1 ed8 2
ea1 ed9 1 edd
1 e9c 2 e9a
ee1 1 eec 2
ee8 eec 2 ef4
ef5 1 ef9 2
ef7 ef9 2 efe
eff 1 f03 2
f01 f03 2 f23
f28 1 f2f 1
f2a 2 f42 f44
3 f40 f41 f46
2 f4f f51 2
f53 f55 2 f57
f59 2 f5b f5d
2 f5f f61 3
f66 f67 f68 2
f63 f6a 2 f4e
f6c 1 f6e 1
f70 2 f39 f71
1 f75 1 f34
2 f32 f79 1
f7c 1 f81 2
f83 f84 2 ef2
f85 1 f8e 2
f8a f8e 2 f93
f94 1 f98 2
f96 f98 2 fb8
fbd 1 fc4 1
fbf 2 fd7 fd9
3 fd5 fd6 fdb
2 fe4 fe6 2
fe8 fea 2 fec
fee 2 ff0 ff2
2 ff4 ff6 3
ffb ffc ffd 2
ff8 fff 2 fe3
1001 1 1003 1
1005 2 fce 1006
1 100a 1 fc9
2 fc7 100e 1
1011 2 1015 1014
2 ee4 1016 2
1032 1037 1 103e
1 1039 2 1051
1053 3 104f 1050
1055 2 105e 1060
2 1062 1064 2
1066 1068 2 106a
106c 2 106e 1070
3 1075 1076 1077
2 1072 1079 2
105d 107b 1 107d
1 107f 2 1048
1080 1 1084 1
1043 2 1041 1088
1 108b 2 108e
108f 1 1090 3
1093 1094 1095 1
1099 2 1097 1099
2 10b5 10ba 1
10c1 1 10bc 2
10d4 10d6 3 10d2
10d3 10d8 2 10e1
10e3 2 10e5 10e7
2 10e9 10eb 2
10ed 10ef 2 10f1
10f3 3 10f8 10f9
10fa 2 10f5 10fc
2 10e0 10fe 1
1100 1 1102 2
10cb 1103 1 1107
1 10c6 2 10c4
110b 1 1116 2
1112 1116 2 111e
111f 1 1123 2
1121 1123 2 1128
1129 1 112d 2
112b 112d 2 1147
114c 1 1153 1
114e 2 1166 1168
3 1164 1165 116a
2 1173 1175 2
1177 1179 2 117b
117d 2 117f 1181
2 1183 1185 3
118a 118b 118c 2
1187 118e 2 1172
1190 1 1192 1
1194 2 115d 1195
1 1199 1 1158
2 1156 119d 1
11a0 2 11a5 11a6
2 11a8 11a9 2
111c 11aa 1 11b3
2 11af 11b3 2
11ce 11d3 1 11da
1 11d5 2 11ed
11ef 3 11eb 11ec
11f1 2 11fa 11fc
2 11fe 1200 2
1202 1204 2 1206
1208 2 120a 120c
3 1211 1212 1213
2 120e 1215 2
11f9 1217 1 1219
1 121b 2 11e4
121c 1 1220 1
11df 2 11dd 1224
1 1227 2 122b
122a 2 110e 122c
2 1245 124a 1
1251 1 124c 2
1264 1266 3 1262
1263 1268 2 1271
1273 2 1275 1277
2 1279 127b 2
127d 127f 2 1281
1283 3 1288 1289
128a 2 1285 128c
2 1270 128e 1
1290 1 1292 2
125b 1293 1 1297
1 1256 2 1254
129b 1 129e 2
12a1 12a2 1 12a3
2 12a5 12a6 1
12ae 2 12aa 12ae
2 12b3 12b4 1
12b8 2 12b6 12b8
2 12c3 12c5 3
12c1 12c2 12c7 2
12d0 12d2 2 12d4
12d6 2 12d8 12da
1 12e2 2 12e4
12e5 2 12dc 12e7
2 12e9 12eb 2
12ed 12ef 2 12f1
12f3 2 12f5 12f7
2 12cf 12f9 1
12fb 1 12fd 2
1307 1309 3 1305
1306 130b 2 1314
1316 2 1313 1318
1 131a 1 131c
3 1323 1324 1325
3 12fe 131d 1327
1 132a 1 1331
2 132f 1331 2
1346 134b 1 1352
1 134d 2 1365
1367 3 1363 1364
1369 2 1372 1374
2 1376 1378 2
137a 137c 2 137e
1380 2 1382 1384
3 1389 138a 138b
2 1386 138d 2
1371 138f 1 1391
1 1393 2 135c
1394 1 1398 1
1357 2 1355 139c
2 1336 139f 2
13b1 13b6 1 13bd
1 13b8 2 13d0
13d2 3 13ce 13cf
13d4 2 13dd 13df
2 13e1 13e3 2
13e5 13e7 2 13e9
13eb 2 13ed 13ef
3 13f4 13f5 13f6
2 13f1 13f8 2
13dc 13fa 1 13fc
1 13fe 2 13c7
13ff 1 1403 1
13c2 2 13c0 1407
2 13a4 140a 2
140d 140e 2 1411
1412 2 1417 1418
1 141a 2 1414
141a 2 1425 1427
3 1423 1424 1429
2 1432 1434 2
1436 1438 2 143a
143c 1 1444 2
1446 1447 2 143e
1449 2 144b 144d
2 144f 1451 2
1453 1455 2 1457
1459 2 1431 145b
1 145d 1 145f
2 1469 146b 3
1467 1468 146d 2
1476 1478 2 1475
147a 1 147c 1
147e 3 1485 1486
1487 3 1460 147f
1489 1 148b 2
140f 148c 2 148f
148e 2 1490 1493
2 1497 1498 1
149c 2 149a 149c
2 14a2 14a3 1
14a5 1 14aa 2
14a8 14aa 2 14b7
14b9 3 14b5 14b6
14bb 2 14c4 14c6
2 14c8 14ca 2
14cc 14ce 1 14d6
2 14d8 14d9 2
14d0 14db 2 14dd
14df 2 14e1 14e3
2 14e5 14e7 2
14e9 14eb 2 14c3
14ed 1 14ef 1
14f1 2 14fb 14fd
3 14f9 14fa 14ff
2 1508 150a 2
1507 150c 1 150e
1 1510 3 1517
1518 1519 3 14f2
1511 151b 2 151e
151d 3 1521 1522
1523 1 1527 2
1525 1527 2 152c
152e 3 1530 1531
1532 2 154b 1550
1 1557 1 1552
2 156a 156c 3
1568 1569 156e 2
1577 1579 2 157b
157d 2 157f 1581
2 1583 1585 2
1587 1589 3 158e
158f 1590 2 158b
1592 2 1576 1594
1 1596 1 1598
2 1561 1599 1
159d 1 155c 2
155a 15a1 1 15ac
2 15a8 15ac 2
15b8 15ba 3 15b6
15b7 15bc 2 15c5
15c7 2 15c9 15cb
2 15cd 15cf 1
15d7 2 15d9 15da
2 15d1 15dc 2
15de 15e0 2 15e2
15e4 2 15e6 15e8
2 15ea 15ec 2
15c4 15ee 1 15f0
1 15f2 2 15fc
15fe 3 15fa 15fb
1600 2 1609 160b
2 1608 160d 1
160f 1 1611 3
1618 1619 161a 3
15f3 1612 161c 1
161e 3 1621 1622
1623 1 1627 2
1625 1627 2 162c
162d 1 1631 2
162f 1631 2 1636
1637 2 163c 163d
1 163f 2 1639
163f 2 1650 1652
3 164e 164f 1654
2 165d 165f 2
1661 1663 2 1665
1667 1 166f 2
1671 1672 2 1669
1674 2 1676 1678
2 167a 167c 2
167e 1680 2 1682
1684 2 165c 1686
1 1688 1 168a
2 1694 1696 3
1692 1693 1698 2
16a1 16a3 2 16a0
16a5 1 16a7 1
16a9 3 16b0 16b1
16b2 3 168b 16aa
16b4 1 16b6 2
16c4 16c9 1 16d0
1 16cb 2 16e3
16e5 3 16e1 16e2
16e7 2 16f0 16f2
2 16f4 16f6 2
16f8 16fa 2 16fc
16fe 2 1700 1702
3 1707 1708 1709
2 1704 170b 2
16ef 170d 1 170f
1 1711 2 16da
1712 1 1716 1
16d5 2 16d3 171a
5 1535 15a4 161f
16b7 171d 2 1722
1724 3 1726 1727
1728 2 1731 1736
1 173d 1 1738
2 1750 1752 3
174e 174f 1754 2
175d 175f 2 1761
1763 2 1765 1767
2 1769 176b 2
176d 176f 3 1774
1775 1776 2 1771
1778 2 175c 177a
1 177c 1 177e
2 1747 177f 1
1783 1 1742 2
1740 1787 1 1792
2 178e 1792 2
179e 17a0 3 179c
179d 17a2 2 17ab
17ad 2 17af 17b1
2 17b3 17b5 1
17bd 2 17bf 17c0
2 17b7 17c2 2
17c4 17c6 2 17c8
17ca 2 17cc 17ce
2 17d0 17d2 2
17aa 17d4 1 17d6
1 17d8 2 17e2
17e4 3 17e0 17e1
17e6 2 17ef 17f1
2 17ee 17f3 1
17f5 1 17f7 3
17fe 17ff 1800 3
17d9 17f8 1802 1
1804 2 181a 181f
1 1826 1 1821
2 1839 183b 3
1837 1838 183d 2
1846 1848 2 184a
184c 2 184e 1850
2 1852 1854 2
1856 1858 3 185d
185e 185f 2 185a
1861 2 1845 1863
1 1865 1 1867
2 1830 1868 1
186c 1 182b 2
1829 1870 1 187b
2 1877 187b 2
1880 1881 1 1885
2 1883 1885 2
1893 1895 3 1891
1892 1897 2 18a0
18a2 2 18a4 18a6
2 18a8 18aa 1
18b2 2 18b4 18b5
2 18ac 18b7 2
18b9 18bb 2 18bd
18bf 2 18c1 18c3
2 18c5 18c7 2
189f 18c9 1 18cb
1 18cd 2 18d7
18d9 3 18d5 18d6
18db 2 18e4 18e6
2 18e3 18e8 1
18ea 1 18ec 3
18f3 18f4 18f5 3
18ce 18ed 18f7 1
18f9 2 1907 190c
1 1913 1 190e
2 1926 1928 3
1924 1925 192a 2
1933 1935 2 1937
1939 2 193b 193d
2 193f 1941 2
1943 1945 3 194a
194b 194c 2 1947
194e 2 1932 1950
1 1952 1 1954
2 191d 1955 1
1959 1 1918 2
1916 195d 6 172b
178a 1805 1873 18fa
1960 2 1963 1964
2 196a 196b 1
196f 2 196d 196f
2 197c 1981 1
1988 1 1983 2
199b 199d 3 1999
199a 199f 2 19a8
19aa 2 19ac 19ae
2 19b0 19b2 2
19b4 19b6 2 19b8
19ba 3 19bf 19c0
19c1 2 19bc 19c3
2 19a7 19c5 1
19c7 1 19c9 2
1992 19ca 1 19ce
1 198d 2 198b
19d2 2 19e1 19e6
1 19ed 1 19e8
2 1a00 1a02 3
19fe 19ff 1a04 2
1a0d 1a0f 2 1a11
1a13 2 1a15 1a17
2 1a19 1a1b 2
1a1d 1a1f 3 1a24
1a25 1a26 2 1a21
1a28 2 1a0c 1a2a
1 1a2c 1 1a2e
2 19f7 1a2f 1
1a33 1 19f2 2
19f0 1a37 2 1a43
1a44 1 1a48 2
1a46 1a48 1 1a4d
1 1a4f 20 1a53
1a54 1a55 1a56 1a57
1a58 1a59 1a5a 1a5b
1a5c 1a5d 1a5e 1a5f
1a60 1a61 1a62 1a63
1a64 1a65 1a66 1a67
1a68 1a69 1a6a 1a6b
1a6c 1a6d 1a6e 1a6f
1a70 1a71 1a72 1
1a78 2 1a76 1a78
2 1a7c 1a7d 1
1a81 2 1a7f 1a81
1 1a8b 2 1a8a
1a8b 1 1a93 1
1a99 2 1a9b 1a9c
2 1a88 1a9d 1
1aa4 1 1a9f 2
1ab7 1ab9 3 1ab5
1ab6 1abb 2 1ac4
1ac6 2 1ac8 1aca
2 1acc 1ace 2
1ad0 1ad2 2 1ad4
1ad6 3 1adb 1adc
1add 2 1ad8 1adf
2 1ac3 1ae1 1
1ae3 1 1ae5 2
1aae 1ae6 1 1aea
1 1aa9 2 1aa7
1aee 1 1af1 1
1afb 2 1afa 1afb
1 1b03 1 1b09
2 1b0b 1b0c 2
1af8 1b0d 1 1b14
1 1b0f 2 1b27
1b29 3 1b25 1b26
1b2b 2 1b34 1b36
2 1b38 1b3a 2
1b3c 1b3e 2 1b40
1b42 2 1b44 1b46
3 1b4b 1b4c 1b4d
2 1b48 1b4f 2
1b33 1b51 1 1b53
1 1b55 2 1b1e
1b56 1 1b5a 1
1b19 2 1b17 1b5e
1 1b61 2 1b64
1b65 1 1b66 1
1b68 7 19d5 1a3a
1a3e 1a41 1a50 1a75
1b69 3 1b72 1b73
1b74 1 1b78 2
1b76 1b78 1 1b7f
2 1b84 1b85 2
1b87 1b8a 1 1b8d
2 1b8f 1b90 20
1b94 1b95 1b96 1b97
1b98 1b99 1b9a 1b9b
1b9c 1b9d 1b9e 1b9f
1ba0 1ba1 1ba2 1ba3
1ba4 1ba5 1ba6 1ba7
1ba8 1ba9 1baa 1bab
1bac 1bad 1bae 1baf
1bb0 1bb1 1bb2 1bb3
1 1bb9 2 1bb7
1bb9 1 1bc3 2
1bc2 1bc3 1 1bcb
1 1bd1 2 1bd3
1bd4 2 1bc0 1bd5
1 1bdc 1 1bd7
2 1bef 1bf1 3
1bed 1bee 1bf3 2
1bfc 1bfe 2 1c00
1c02 2 1c04 1c06
2 1c08 1c0a 2
1c0c 1c0e 3 1c13
1c14 1c15 2 1c10
1c17 2 1bfb 1c19
1 1c1b 1 1c1d
2 1be6 1c1e 1
1c22 1 1be1 2
1bdf 1c26 1 1c29
1 1c2c 5 1b6d
1b70 1b91 1bb6 1c2d
2 1c2f 1c30 c
d47 dc0 dc3 e31
e59 e5c e5f 12a7
151f 1965 1968 1c31
1 1c35 2 1c37
1c38 1 1c3c 2
1c3a 1c3c 1 1c41
1 1c47 2 1c49
1c4a 2 1c4c 1c4e
2 1c51 1c53 1
1c55 1 1c58 2
1c5a 1c5b 1 1c5f
2 1c5d 1c5f 2
1c67 1c68 1 1c6b
2 1c6a 1c6b 2
1c7b 1c7c 2 1c7e
1c80 2 1c82 1c84
4 1c78 1c79 1c86
1c87 2 1c92 1c93
2 1c95 1c97 2
1c99 1c9b 2 1c9d
1c9f 4 1c8f 1c90
1ca1 1ca2 2 1cc0
1cc5 1 1ccc 1
1cc7 2 1cdf 1ce1
3 1cdd 1cde 1ce3
2 1cec 1cee 2
1cf0 1cf2 2 1cf4
1cf6 2 1cf8 1cfa
2 1cfc 1cfe 3
1d03 1d04 1d05 2
1d00 1d07 2 1ceb
1d09 1 1d0b 1
1d0d 2 1cd6 1d0e
1 1d12 1 1cd1
2 1ccf 1d16 2
1d1c 1d1d 1 1d21
2 1d1f 1d21 1
1d25 2 1d33 1d34
1 1d38 2 1d36
1d38 1 1d3d 1
1d3f 2 1d2f 1d40
1 1d42 3 1d45
1d46 1d47 1 1d4b
2 1d49 1d4b 1
1d55 2 1d57 1d58
1 1d5c 2 1d5a
1d5c 1 1d61 2
1d65 1d67 3 1d69
1d6a 1d6b 2 1d71
1d73 3 1d75 1d76
1d77 2 1d6e 1d7a
2 1d7c 1d7d 2
1d52 1d7e 1 1d82
2 1d84 1d85 1
1d89 2 1d87 1d89
1 1d8e 2 1d92
1d94 3 1d96 1d97
1d98 2 1d9e 1da0
3 1da2 1da3 1da4
2 1d9b 1da7 2
1da9 1daa 1 1dab
2 1dad 1dae 2
1db7 1db8 1 1dbc
2 1dba 1dbc 1
1dc1 2 1dbf 1dc1
3 1dc5 1dc6 1dc7
1 1dcb 2 1dc9
1dcb 2 1dea 1def
1 1df6 1 1df1
2 1e09 1e0b 3
1e07 1e08 1e0d 2
1e16 1e18 2 1e1a
1e1c 2 1e1e 1e20
2 1e22 1e24 2
1e26 1e28 3 1e2d
1e2e 1e2f 2 1e2a
1e31 2 1e15 1e33
1 1e35 1 1e37
2 1e00 1e38 1
1e3c 1 1dfb 2
1df9 1e40 1 1e4b
2 1e47 1e4b 2
1e53 1e54 1 1e58
2 1e56 1e58 2
1e5d 1e5e 1 1e62
2 1e60 1e62 2
1e82 1e87 1 1e8e
1 1e89 2 1ea1
1ea3 3 1e9f 1ea0
1ea5 2 1eae 1eb0
2 1eb2 1eb4 2
1eb6 1eb8 2 1eba
1ebc 2 1ebe 1ec0
3 1ec5 1ec6 1ec7
2 1ec2 1ec9 2
1ead 1ecb 1 1ecd
1 1ecf 2 1e98
1ed0 1 1ed4 1
1e93 2 1e91 1ed8
1 1edb 1 1ee0
2 1ee2 1ee3 2
1e51 1ee4 1 1eed
2 1ee9 1eed 2
1ef2 1ef3 1 1ef7
2 1ef5 1ef7 2
1f17 1f1c 1 1f23
1 1f1e 2 1f36
1f38 3 1f34 1f35
1f3a 2 1f43 1f45
2 1f47 1f49 2
1f4b 1f4d 2 1f4f
1f51 2 1f53 1f55
3 1f5a 1f5b 1f5c
2 1f57 1f5e 2
1f42 1f60 1 1f62
1 1f64 2 1f2d
1f65 1 1f69 1
1f28 2 1f26 1f6d
1 1f70 2 1f74
1f73 2 1e43 1f75
2 1f93 1f98 1
1f9f 1 1f9a 2
1fb2 1fb4 3 1fb0
1fb1 1fb6 2 1fbf
1fc1 2 1fc3 1fc5
2 1fc7 1fc9 2
1fcb 1fcd 2 1fcf
1fd1 3 1fd6 1fd7
1fd8 2 1fd3 1fda
2 1fbe 1fdc 1
1fde 1 1fe0 2
1fa9 1fe1 1 1fe5
1 1fa4 2 1fa2
1fe9 1 1fec 2
1fef 1ff0 1 1ff1
3 1ff4 1ff5 1ff6
1 1ffa 2 1ff8
1ffa 2 2018 201d
1 2024 1 201f
2 2037 2039 3
2035 2036 203b 2
2044 2046 2 2048
204a 2 204c 204e
2 2050 2052 2
2054 2056 3 205b
205c 205d 2 2058
205f 2 2043 2061
1 2063 1 2065
2 202e 2066 1
206a 1 2029 2
2027 206e 1 2079
2 2075 2079 2
207e 207f 1 2083
2 2081 2083 2
2088 2089 1 208d
2 208b 208d 2
20a9 20ae 1 20b5
1 20b0 2 20c8
20ca 3 20c6 20c7
20cc 2 20d5 20d7
2 20d9 20db 2
20dd 20df 2 20e1
20e3 2 20e5 20e7
3 20ec 20ed 20ee
2 20e9 20f0 2
20d4 20f2 1 20f4
1 20f6 2 20bf
20f7 1 20fb 1
20ba 2 20b8 20ff
1 2102 2 2107
2108 2 210a 210b
1 210c 1 2115
2 2111 2115 2
2132 2137 1 213e
1 2139 2 2151
2153 3 214f 2150
2155 2 215e 2160
2 2162 2164 2
2166 2168 2 216a
216c 2 216e 2170
3 2175 2176 2177
2 2172 2179 2
215d 217b 1 217d
1 217f 2 2148
2180 1 2184 1
2143 2 2141 2188
1 218b 2 218f
218e 2 2071 2190
2 21ab 21b0 1
21b7 1 21b2 2
21ca 21cc 3 21c8
21c9 21ce 2 21d7
21d9 2 21db 21dd
2 21df 21e1 2
21e3 21e5 2 21e7
21e9 3 21ee 21ef
21f0 2 21eb 21f2
2 21d6 21f4 1
21f6 1 21f8 2
21c1 21f9 1 21fd
1 21bc 2 21ba
2201 1 2204 2
2207 2208 1 2209
2 220b 220c 1
2214 2 2210 2214
2 2219 221a 1
221e 2 221c 221e
1 2223 1 2225
1 222c 2 222a
222c 2 2241 2246
1 224d 1 2248
2 2260 2262 3
225e 225f 2264 2
226d 226f 2 2271
2273 2 2275 2277
2 2279 227b 2
227d 227f 3 2284
2285 2286 2 2281
2288 2 226c 228a
1 228c 1 228e
2 2257 228f 1
2293 1 2252 2
2250 2297 2 2231
229a 2 22ac 22b1
1 22b8 1 22b3
2 22cb 22cd 3
22c9 22ca 22cf 2
22d8 22da 2 22dc
22de 2 22e0 22e2
2 22e4 22e6 2
22e8 22ea 3 22ef
22f0 22f1 2 22ec
22f3 2 22d7 22f5
1 22f7 1 22f9
2 22c2 22fa 1
22fe 1 22bd 2
22bb 2302 2 229f
2305 2 2308 2309
2 230c 230d 2
2312 2313 1 2315
2 230f 2315 2
231a 231b 2 231d
231f 2 2322 2324
1 2326 2 230a
2327 1 2329 2
232a 232d 2 232f
2330 1 2331 2
2335 2336 2 2338
233a 2 233d 233f
2 2341 2342 2
2354 2359 1 2360
1 235b 2 2373
2375 3 2371 2372
2377 2 2380 2382
2 2384 2386 2
2388 238a 2 238c
238e 2 2390 2392
3 2397 2398 2399
2 2394 239b 2
237f 239d 1 239f
1 23a1 2 236a
23a2 1 23a6 1
2365 2 2363 23aa
2 23b9 23be 1
23c5 1 23c0 2
23d8 23da 3 23d6
23d7 23dc 2 23e5
23e7 2 23e9 23eb
2 23ed 23ef 2
23f1 23f3 2 23f5
23f7 3 23fc 23fd
23fe 2 23f9 2400
2 23e4 2402 1
2404 1 2406 2
23cf 2407 1 240b
1 23ca 2 23c8
240f 2 241b 241c
1 2420 2 241e
2420 1 2425 1
2427 20 242b 242c
242d 242e 242f 2430
2431 2432 2433 2434
2435 2436 2437 2438
2439 243a 243b 243c
243d 243e 243f 2440
2441 2442 2443 2444
2445 2446 2447 2448
2449 244a 1 2450
2 244e 2450 2
2454 2455 1 2459
2 2457 2459 1
2463 2 2462 2463
1 246b 1 2471
2 2473 2474 2
2460 2475 1 247c
1 2477 2 248f
2491 3 248d 248e
2493 2 249c 249e
2 24a0 24a2 2
24a4 24a6 2 24a8
24aa 2 24ac 24ae
3 24b3 24b4 24b5
2 24b0 24b7 2
249b 24b9 1 24bb
1 24bd 2 2486
24be 1 24c2 1
2481 2 247f 24c6
1 24c9 1 24d3
2 24d2 24d3 1
24db 1 24e1 2
24e3 24e4 2 24d0
24e5 1 24ec 1
24e7 2 24ff 2501
3 24fd 24fe 2503
2 250c 250e 2
2510 2512 2 2514
2516 2 2518 251a
2 251c 251e 3
2523 2524 2525 2
2520 2527 2 250b
2529 1 252b 1
252d 2 24f6 252e
1 2532 1 24f1
2 24ef 2536 1
2539 2 253c 253d
1 253e 1 2540
b 220d 2343 2346
2349 23ad 2412 2416
2419 2428 244d 2541
2 2545 2546 1
254a 2 2548 254a
3 2554 2555 2556
1 255a 2 2558
255a 1 2561 2
2565 2566 1 2569
2 256b 256c 20
2570 2571 2572 2573
2574 2575 2576 2577
2578 2579 257a 257b
257c 257d 257e 257f
2580 2581 2582 2583
2584 2585 2586 2587
2588 2589 258a 258b
258c 258d 258e 258f
1 2595 2 2593
2595 1 259f 2
259e 259f 1 25a7
1 25ad 2 25af
25b0 2 259c 25b1
1 25b8 1 25b3
2 25cb 25cd 3
25c9 25ca 25cf 2
25d8 25da 2 25dc
25de 2 25e0 25e2
2 25e4 25e6 2
25e8 25ea 3 25ef
25f0 25f1 2 25ec
25f3 2 25d7 25f5
1 25f7 1 25f9
2 25c2 25fa 1
25fe 1 25bd 2
25bb 2602 1 2605
1 2608 5 254f
2552 256d 2592 2609
2 260c 260b 2
2610 2611 2 2613
2615 1 2619 f
1c73 1c8a 1ca5 1ca8
1cab 1cae 1cb1 1d19
1d43 1daf 1db2 1db5
260d 2618 261c 2
1c64 261f 1 2621
2 1c5c 2622 2
2624 2625 1 2629
2 2627 2629 2
262e 262f 1 2633
2 2631 2633 2
263a 263b 1 263f
2 263d 263f 1
2648 2 2647 2648
1 2650 1 2656
2 2658 2659 2
2645 265a 1 2661
1 265c 2 2674
2676 3 2672 2673
2678 2 2681 2683
2 2685 2687 2
2689 268b 2 268d
268f 2 2691 2693
3 2698 2699 269a
2 2695 269c 2
2680 269e 1 26a0
1 26a2 2 266b
26a3 1 26a7 1
2666 2 2664 26ab
1 26ae 1 26bb
2 26ba 26bb 1
26c3 1 26c9 2
26cb 26cc 2 26b8
26cd 1 26d4 1
26cf 2 26e7 26e9
3 26e5 26e6 26eb
2 26f4 26f6 2
26f8 26fa 2 26fc
26fe 2 2700 2702
2 2704 2706 3
270b 270c 270d 2
2708 270f 2 26f3
2711 1 2713 1
2715 2 26de 2716
1 271a 1 26d9
2 26d7 271e 1
2721 2 2724 2725
1 2729 2 2727
2729 2 2734 2736
3 2732 2733 2738
2 2741 2743 2
2745 2747 2 2749
274b 1 2753 2
2755 2756 2 274d
2758 2 275a 275c
2 275e 2760 2
2762 2764 2 2766
2768 2 2740 276a
1 276c 1 276e
2 2778 277a 3
2776 2777 277c 2
2785 2787 2 2784
2789 1 278b 1
278d 3 2794 2795
2796 3 276f 278e
2798 1 279f 2
279b 279f 2 27aa
27ac 3 27a8 27a9
27ae 2 27b7 27b9
2 27bb 27bd 2
27bf 27c1 1 27c9
2 27cb 27cc 2
27c3 27ce 2 27d0
27d2 2 27d4 27d6
2 27d8 27da 2
27dc 27de 2 27b6
27e0 1 27e2 1
27e4 2 27ee 27f0
3 27ec 27ed 27f2
2 27fb 27fd 2
27fa 27ff 1 2801
1 2803 3 280a
280b 280c 3 27e5
2804 280e 1 2816
2 2812 2816 2
2821 2823 3 281f
2820 2825 2 282e
2830 2 2832 2834
2 2836 2838 1
2840 2 2842 2843
2 283a 2845 2
2847 2849 2 284b
284d 2 284f 2851
2 2853 2855 2
282d 2857 1 2859
1 285b 2 2865
2867 3 2863 2864
2869 2 2872 2874
2 2871 2876 1
2878 1 287a 3
2881 2882 2883 3
285c 287b 2885 2
2890 2892 3 288e
288f 2894 2 289d
289f 2 28a1 28a3
2 28a5 28a7 1
28af 2 28b1 28b2
2 28a9 28b4 2
28b6 28b8 2 28ba
28bc 2 28be 28c0
2 28c2 28c4 2
289c 28c6 1 28c8
1 28ca 2 28d4
28d6 3 28d2 28d3
28d8 2 28e1 28e3
2 28e0 28e5 1
28e7 1 28e9 3
28f0 28f1 28f2 3
28cb 28ea 28f4 4
28f6 2811 2887 28f7
2 2726 28f8 1
28fa 1 28fe 2
2900 2901 1 2905
2 2903 2905 1
290b 2 290d 290e
1 2912 2 2910
2912 2 2923 2925
2 2927 2929 2
292b 292d 2 292f
2931 2 2933 2935
2 2937 2939 2
293b 293d 2 293f
2941 2 2943 2945
2 2947 2949 2
294b 294d 2 294f
2951 1 295c 2
295b 295c 1 2964
1 296a 2 296c
296d 2 2959 296e
1 2975 1 2970
2 2988 298a 3
2986 2987 298c 2
2995 2997 2 2999
299b 2 299d 299f
2 29a1 29a3 2
29a5 29a7 3 29ac
29ad 29ae 2 29a9
29b0 2 2994 29b2
2 29b9 29bb 2
29b8 29bd 2 29b4
29bf 1 29c1 2
297f 29c2 1 29c6
1 297a 2 2978
29ca 2 29da 29dc
2 29de 29e0 2
29e2 29e4 2 29e6
29e8 2 29ea 29ec
2 29ee 29f0 2
29f2 29f4 2 29f6
29f8 2 29fa 29fc
2 29fe 2a00 2
2a02 2a04 2 2a06
2a08 1 2a13 2
2a12 2a13 1 2a1b
1 2a21 2 2a23
2a24 2 2a10 2a25
1 2a2c 1 2a27
2 2a3f 2a41 3
2a3d 2a3e 2a43 2
2a4c 2a4e 2 2a50
2a52 2 2a54 2a56
2 2a58 2a5a 2
2a5c 2a5e 3 2a63
2a64 2a65 2 2a60
2a67 2 2a4b 2a69
2 2a70 2a72 2
2a6f 2a74 2 2a6b
2a76 1 2a78 2
2a36 2a79 1 2a7d
1 2a31 2 2a2f
2a81 2 2a8e 2a90
2 2a92 2a94 2
2a96 2a98 2 2a9a
2a9c 2 2a9e 2aa0
2 2aa2 2aa4 2
2aa6 2aa8 2 2aaa
2aac 2 2aae 2ab0
2 2ab2 2ab4 2
2ab6 2ab8 2 2aba
2abc 2 2abe 2ac0
2 2ac2 2ac4 2
2ac6 2ac8 2 2aca
2acc 2 2ace 2ad0
2 2ad2 2ad4 2
2ad6 2ad8 2 2ada
2adc 2 2ade 2ae0
2 2ae2 2ae4 2
2ae6 2ae8 2 2aea
2aec 2 2aee 2af0
2 2af2 2af4 2
2af6 2af8 2 2afa
2afc 2 2afe 2b00
2 2b02 2b04 2
2b06 2b08 2 2b0a
2b0c 2 2b0e 2b10
2 2b12 2b14 2
2b16 2b18 2 2b1a
2b1c 2 2b1e 2b20
2 2b22 2b24 1
2b2f 2 2b2e 2b2f
1 2b37 1 2b3d
2 2b3f 2b40 2
2b2c 2b41 1 2b48
1 2b43 2 2b5b
2b5d 3 2b59 2b5a
2b5f 2 2b68 2b6a
2 2b6c 2b6e 2
2b70 2b72 2 2b74
2b76 2 2b78 2b7a
3 2b7f 2b80 2b81
2 2b7c 2b83 2
2b67 2b85 2 2b8c
2b8e 2 2b8b 2b90
2 2b87 2b92 1
2b94 2 2b52 2b95
1 2b99 1 2b4d
2 2b4b 2b9d 2
2ba3 2ba5 2 2ba7
2ba9 2 2bab 2bad
2 2bb3 2bb4 1
2bb6 2 2baf 2bb8
2 2bba 2bbc 2
2bbe 2bc0 2 2bc6
2bc7 1 2bc9 2
2bc2 2bcb 2 2bcd
2bcf 2 2bd1 2bd3
2 2bd5 2bd7 2
2bd9 2bdb 2 2bdd
2bdf 2 2be1 2be3
2 2be5 2be7 2
2be9 2beb 2 2bed
2bef 2 2bf5 2bf6
1 2bf8 2 2bf1
2bfa 2 2bfc 2bfe
2 2c00 2c02 2
2c08 2c09 1 2c0b
2 2c04 2c0d 2
2c0f 2c11 2 2c13
2c15 2 2c17 2c19
2 2c1b 2c1d 2
2c1f 2c21 c 291c
2921 2954 29cd 29d3
29d8 2a0b 2a84 2a8c
2b27 2ba0 2c24 1
2c29 2 2c2b 2c2c
1 2c30 2 2c2e
2c30 2 2c3e 2c40
2 2c42 2c44 2
2c46 2c48 2 2c4a
2c4c 2 2c4e 2c50
2 2c52 2c54 2
2c56 2c58 2 2c5a
2c5c 2 2c5e 2c60
2 2c62 2c64 1
2c6f 2 2c6e 2c6f
1 2c77 1 2c7d
2 2c7f 2c80 2
2c6c 2c81 1 2c88
1 2c83 2 2c9b
2c9d 3 2c99 2c9a
2c9f 2 2ca8 2caa
2 2cac 2cae 2
2cb0 2cb2 2 2cb4
2cb6 2 2cb8 2cba
3 2cbf 2cc0 2cc1
2 2cbc 2cc3 2
2ca7 2cc5 2 2ccc
2cce 2 2ccb 2cd0
2 2cc7 2cd2 1
2cd4 2 2c92 2cd5
1 2cd9 1 2c8d
2 2c8b 2cdd 2
2ced 2cef 2 2cf1
2cf3 2 2cf5 2cf7
2 2cf9 2cfb 2
2cfd 2cff 2 2d01
2d03 2 2d05 2d07
2 2d09 2d0b 2
2d0d 2d0f 1 2d1a
2 2d19 2d1a 1
2d22 1 2d28 2
2d2a 2d2b 2 2d17
2d2c 1 2d33 1
2d2e 2 2d46 2d48
3 2d44 2d45 2d4a
2 2d53 2d55 2
2d57 2d59 2 2d5b
2d5d 2 2d5f 2d61
2 2d63 2d65 3
2d6a 2d6b 2d6c 2
2d67 2d6e 2 2d52
2d70 2 2d77 2d79
2 2d76 2d7b 2
2d72 2d7d 1 2d7f
2 2d3d 2d80 1
2d84 1 2d38 2
2d36 2d88 2 2d95
2d97 2 2d99 2d9b
2 2d9d 2d9f 2
2da1 2da3 2 2da5
2da7 2 2da9 2dab
2 2dad 2daf 2
2db1 2db3 2 2db5
2db7 2 2db9 2dbb
2 2dbd 2dbf 2
2dc1 2dc3 2 2dc5
2dc7 2 2dc9 2dcb
2 2dcd 2dcf 2
2dd1 2dd3 2 2dd5
2dd7 2 2dd9 2ddb
1 2de6 2 2de5
2de6 1 2dee 1
2df4 2 2df6 2df7
2 2de3 2df8 1
2dff 1 2dfa 2
2e12 2e14 3 2e10
2e11 2e16 2 2e1f
2e21 2 2e23 2e25
2 2e27 2e29 2
2e2b 2e2d 2 2e2f
2e31 3 2e36 2e37
2e38 2 2e33 2e3a
2 2e1e 2e3c 2
2e43 2e45 2 2e42
2e47 2 2e3e 2e49
1 2e4b 2 2e09
2e4c 1 2e50 1
2e04 2 2e02 2e54
2 2e5a 2e5c 2
2e5e 2e60 2 2e62
2e64 2 2e6a 2e6b
1 2e6d 2 2e66
2e6f 2 2e71 2e73
2 2e79 2e7a 1
2e7c 2 2e75 2e7e
2 2e80 2e82 2
2e84 2e86 2 2e88
2e8a 2 2e8c 2e8e
2 2e90 2e92 c
2c37 2c3c 2c67 2ce0
2ce6 2ceb 2d12 2d8b
2d93 2dde 2e57 2e95
2 2ea3 2ea5 2
2ea7 2ea9 2 2eab
2ead 2 2eaf 2eb1
2 2eb3 2eb5 2
2eb7 2eb9 2 2ebb
2ebd 2 2ebf 2ec1
2 2ec3 2ec5 2
2ec7 2ec9 2 2ecb
2ecd 2 2ecf 2ed1
1 2edc 2 2edb
2edc 1 2ee4 1
2eea 2 2eec 2eed
2 2ed9 2eee 1
2ef5 1 2ef0 2
2f08 2f0a 3 2f06
2f07 2f0c 2 2f15
2f17 2 2f19 2f1b
2 2f1d 2f1f 2
2f21 2f23 2 2f25
2f27 3 2f2c 2f2d
2f2e 2 2f29 2f30
2 2f14 2f32 2
2f39 2f3b 2 2f38
2f3d 2 2f34 2f3f
1 2f41 2 2eff
2f42 1 2f46 1
2efa 2 2ef8 2f4a
2 2f5a 2f5c 2
2f5e 2f60 2 2f62
2f64 2 2f66 2f68
2 2f6a 2f6c 2
2f6e 2f70 2 2f72
2f74 2 2f76 2f78
2 2f7a 2f7c 1
2f87 2 2f86 2f87
1 2f8f 1 2f95
2 2f97 2f98 2
2f84 2f99 1 2fa0
1 2f9b 2 2fb3
2fb5 3 2fb1 2fb2
2fb7 2 2fc0 2fc2
2 2fc4 2fc6 2
2fc8 2fca 2 2fcc
2fce 2 2fd0 2fd2
3 2fd7 2fd8 2fd9
2 2fd4 2fdb 2
2fbf 2fdd 2 2fe4
2fe6 2 2fe3 2fe8
2 2fdf 2fea 1
2fec 2 2faa 2fed
1 2ff1 1 2fa5
2 2fa3 2ff5 2
3002 3004 2 3006
3008 2 300a 300c
2 300e 3010 2
3012 3014 2 3016
3018 2 301a 301c
2 301e 3020 2
3022 3024 2 3026
3028 2 302a 302c
2 302e 3030 2
3032 3034 2 3036
3038 2 303a 303c
2 303e 3040 1
304b 2 304a 304b
1 3053 1 3059
2 305b 305c 2
3048 305d 1 3064
1 305f 2 3077
3079 3 3075 3076
307b 2 3084 3086
2 3088 308a 2
308c 308e 2 3090
3092 2 3094 3096
3 309b 309c 309d
2 3098 309f 2
3083 30a1 2 30a8
30aa 2 30a7 30ac
2 30a3 30ae 1
30b0 2 306e 30b1
1 30b5 1 3069
2 3067 30b9 2
30bf 30c1 2 30c3
30c5 2 30c7 30c9
2 30cf 30d0 1
30d2 2 30cb 30d4
2 30d6 30d8 2
30da 30dc 2 30e2
30e3 1 30e5 2
30de 30e7 2 30e9
30eb 2 30ed 30ef
2 30f1 30f3 2
30f5 30f7 2 30f9
30fb c 2e9c 2ea1
2ed4 2f4d 2f53 2f58
2f7f 2ff8 3000 3043
30bc 30fe 3 3100
2e97 3101 2 310b
310d 3 3109 310a
310f 2 3118 311a
2 3117 311c 1
311e 1 3120 2
312a 312c 3 3128
3129 312e 2 3137
3139 2 3136 313b
1 313d 1 313f
1 3145 2d 2fa
319 31c 31f 322
325 328 32b 32e
331 334 337 33a
33d 340 343 346
349 34c 356 359
35c 35f 3c3 427
4a5 600 603 60e
616 61e 621 b58
d07 d0c d0f d12
d26 d33 2626 28fb
3102 3121 3140 3148
2 3155 3157 3
3153 3154 3159 2
3162 3164 2 3161
3166 2 316d 3171
2 3173 3175 2
3177 317b 2 316c
317d 2 3168 317f
1 3181 3 3182
318d 3191 1 314c
2 31a4 31a6 3
31a2 31a3 31a8 2
31b1 31b3 2 31b0
31b5 2 31bc 31be
2 31c0 31c2 3
31c7 31c8 31c9 2
31c4 31cb 2 31bb
31cd 2 31b7 31cf
1 31d1 4 319b
31d2 31db 31df 1
31e3 1 3196 2
3194 31e7 44 4b
52 59 60 67
6e 75 7c 83
8a 8f 94 9b
a2 a9 b0 b7
be c4 ca d0
d6 dc e2 e8
ee f4 fa 100
106 10c 112 118
11e 124 12a 130
136 13c 142 148
14e 154 15a 160
166 16d 174 17a
181 187 18e 195
19c 1a2 1a8 1ae
1b5 1bc 1c2 1c8
1cf 1d5 1db 1e1
1e7 1ec 1f3 
1
4
0 
31ee
0
1
a0
77
55
0 1 2 1 4 1 6 1
8 1 a 1 c 1 e 1
10 1 12 1 14 1 16 1
18 1 1a 1 1c 1 1e 1
20 1 22 1 24 1 26 1
28 1 2a 1 2c 1 2e 1
30 1 32 1 34 1 36 1
38 1 3a 1 3c 1 3e 1
40 41 40 43 40 45 40 47
40 49 40 4b 40 4d 40 4f
40 51 40 53 40 55 40 57
40 59 40 5b 40 5d 40 5f
1 61 1 63 1 65 1 67
1 69 1 6b 1 6d 1 6f
1 71 1 73 1 75 1 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0

3b 1 0
13e 1 0
2b 1 0
183 1 0
1dd 1 0
162 1 0
2f 1 0
1c4 1 0
ab 1 0
1ca 1 0
fc 1 0
15c 1 0
d8 1 0
1d7 1 0
27 1 0
189 1 0
1f 1 0
1b 1 0
1e3 1 0
96 1 0
2619 40 61
c0 1 0
9d 1 0
11a 1 0
17c 1 0
12c 1 0
8c 1 0
1e9 1 0
1a4 1 0
13 1 0
f6 1 0
1b0 1 0
ea 1 0
7e 1 0
16f 1 0
168 1 0
a4 1 0
77 1 0
f 1 0
e4 1 0
70 1 0
5b 1 0
3 1 0
c6 1 0
150 1 0
14a 1 0
1b7 1 0
85 1 0
19e 1 0
138 1 0
2 0 1
114 1 0
108 1 0
23 1 0
144 1 0
54 1 0
1d1 1 0
156 1 0
7 1 0
176 1 0
f0 1 0
de 1 0
132 1 0
d2 1 0
197 1 0
1be 1 0
1aa 1 0
cc 1 0
69 1 0
46 1 0
10e 1 0
37 1 0
4d 1 0
b2 1 0
120 1 0
17 1 0
91 1 0
b9 1 0
126 1 0
62 1 0
33 1 0
102 1 0
b 1 0
1ee 1 0
190 1 0
0

/
SHOW ERRORS FUNCTION SP_SEARCH_1;
ALTER FUNCTION SP_SEARCH_1 COMPILE DEBUG;
/*######################################################################################*/
/*######################################################################################*/
/*
#Stored Procedure  :SP_SEARCH_3
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_SEARCH_3 ==
CREATE OR REPLACE FUNCTION SP_SEARCH_3 wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
1a9
2 :e:
1FUNCTION:
1SP_SEARCH_3:
1PRJ_ID:
1VARCHAR2:
1START_DATE001:
1FINISH_DATE001:
1EFFORT001:
1PLS_INTEGER:
1WORK001:
1BITFLAG001:
1EUNIT:
1UNIT:
1CONTOUR001:
1OCCURRENCE:
1CONSIDER_PLANNED_SCHED:
1WORK_TYPE:
1TYPE001:
1AND_ROLES:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_PROJECT_ID:
132:
1H_TASK_ID:
1H_REC_USER:
1H_RES_ID:
1H_START_DATE:
126:
1H_START_DT:
1H_FINISH_DT:
1H_DAYOFYEAR:
1DATE:
1H_REC_STATUS:
11:
1H_RES_TYPE:
1H_PA_TIMEPHASE:
1H_ASSIGN_ID:
1H_PRJ_ASSIGNMENT_ID:
1H_AND_ROLES:
18000:
1H_SHW_ROLE:
1H_SHW_SKILL:
1H_SD:
1H_FD:
1H_FULL_NAME:
1255:
1H_COUNT:
10:
1H_BITFLAG:
1H_TYPE:
1H_UPDATE:
1H_WORK_TYPE:
1H_CRI:
1H_WORK:
1H_OCCURRENCE:
1H_EFFORT:
1H_EUNIT:
1H_UNIT:
1H_CONSIDER:
1H_CONTOUR:
1H_EVERY:
1H_PER_DAY:
1H_PER_WEEK:
1H_DAYS:
1H_DURATION:
1H_NUMBER:
1H_TOTAL:
1H_TOTPR:
1H_PRIORITY:
1H_WEEK:
1H_RES_COST:
1NUMBER:
1H_BILLABLE:
1H_CAPITAL_EXPENSE:
1H_TPH_DELETE:
1H_REG_COST:
10.00:
1H_REG_SALE:
1H_TEMP_PARTICIPANT:
1H_TEMP_SECURITY_ID:
1CHAR:
1H_TEMP2_SECURITY_ID:
1H_TEMP_BITFLAG:
1H_TEMP_SC_STATE:
1STMT:
17999:
1H_TABLE_TYPE:
1H_TEMP_VAR:
1I:
1II:
1I_RETURN:
1ISF:
1NUMITEMS:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_SEARCH_3(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
1103:
1+:
1FPUT:
1               ENTER:: SP_SEARCH_3(104) :: cplan.sql(4589) :: :
1GETTABLE:
1IS NULL:
18:
1140:
1      RETURN_INVALID:: SP_SEARCH_3(141) :: cplan.sql(4626) :: :
1 :
1STATE:: :
1PMO_CNST:
1ELEMENT_ID_REQUIRED:
1143:
1                EXIT:: SP_SEARCH_3(144) :: cplan.sql(4629) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1=:
15:
1160:
1      RETURN_INVALID:: SP_SEARCH_3(161) :: cplan.sql(4646) :: :
1INVALID_PARAMS:
1163:
1                EXIT:: SP_SEARCH_3(164) :: cplan.sql(4649) :: :
1!=:
1SC_SRCH_PARTICIPANT:
1IS NOT NULL:
1PMO_ORASTD:
1DAYS:
1DUAL:
1SELECT PMO_ORASTD.DAYS(finish_date001) - PMO_ORASTD.DAYS(start_date001):n    +
1     INTO h_total:n         FROM DUAL:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
1182:
1           EXCEPTION:: SP_SEARCH_3(183) :: cplan.sql(4668) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SC_SQL_SUCCESS:
1<:
1189:
1      RETURN_INVALID:: SP_SEARCH_3(190) :: cplan.sql(4675) :: :
1F_DATE_EARLIER_THAN_S_DATE:
1192:
1                EXIT:: SP_SEARCH_3(193) :: cplan.sql(4678) :: :
1ELSIF:
1SELECT SYSTIMESTAMP INTO h_start_date:n         FROM DUAL:
1210:
1           EXCEPTION:: SP_SEARCH_3(211) :: cplan.sql(4696) :: :
1PMO_FUNC:
1SHIFTRIGHT:
1BITAND:
1SC_C0000000:
130:
1SC_1FFFFFFF:
1MINUTE_PER_DAY:
1MINUTE_PER_WEEK:
1PRIORITY:
1TMT_WBS:
1ELEMENT_ID:
1SELECT COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORI+
1TY:n         INTO h_per_day, h_per_week, h_priority:n         FROM TMT_WBS WH+
1ERE ELEMENT_ID = h_project_id:
1236:
1           EXCEPTION:: SP_SEARCH_3(237) :: cplan.sql(4722) :: :
1SC_SQL_NO_DATA:
1TMT_POOLS:
1SELECT 480, 2400, 0 INTO h_per_day, h_per_week, h_priority FROM TMT_POOLS WHE+
1RE ELEMENT_ID = h_project_id:
1252:
1           EXCEPTION:: SP_SEARCH_3(253) :: cplan.sql(4738) :: :
1258:
1        RETURN_ERROR:: SP_SEARCH_3(259) :: cplan.sql(4744) :: :
1 ID:: :
1261:
1                EXIT:: SP_SEARCH_3(262) :: cplan.sql(4747) :: :
1PMO_RETURN_ERROR_SP:
1LEVEL_LOCK:
1PROJECT_DETAILS:
1PROJECT_ID:
1SELECT LEVEL_LOCK INTO h_temp_var:n         FROM PROJECT_DETAILS:n         WH+
1ERE PROJECT_ID = h_project_id:
1280:
1           EXCEPTION:: SP_SEARCH_3(281) :: cplan.sql(4766) :: :
1>:
1REC_STATUS:
1SELECT REC_STATUS INTO h_rec_status:n            FROM TMT_WBS:n            WH+
1ERE ELEMENT_ID = h_project_id:
1298:
1           EXCEPTION:: SP_SEARCH_3(299) :: cplan.sql(4784) :: :
1STRCMP:
1L:
1305:
1        RETURN_ERROR:: SP_SEARCH_3(306) :: cplan.sql(4791) :: :
1LEV_LOCKED_BY_COPY:
1308:
1                EXIT:: SP_SEARCH_3(309) :: cplan.sql(4794) :: :
1316:
1      RETURN_INVALID:: SP_SEARCH_3(317) :: cplan.sql(4802) :: :
1319:
1                EXIT:: SP_SEARCH_3(320) :: cplan.sql(4805) :: :
1PMO_STPU:
1STRNPCPY:
133:
1ISPROJECTTYPE:
1TYPE_ID:
1SC_PROJECT:
1SC_PROPOSAL:
1SC_PROJ_TYPE1:
1SC_PROJ_TYPE2:
1SC_PROJ_TYPE3:
1SELECT PROJECT_ID, COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2+
1400), PRIORITY:n            INTO h_project_id, h_per_day, h_per_week, h_prior+
1ity:n            FROM TMT_WBS WHERE ELEMENT_ID = h_task_id AND TYPE_ID IN (PM+
1O_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PRO+
1J_TYPE2,PMO_CNST.SC_PROJ_TYPE3):
1347:
1           EXCEPTION:: SP_SEARCH_3(348) :: cplan.sql(4833) :: :
1354:
1        RETURN_ERROR:: SP_SEARCH_3(355) :: cplan.sql(4840) :: :
1RECORD_NOT_FOUND:
1357:
1                EXIT:: SP_SEARCH_3(358) :: cplan.sql(4843) :: :
1363:
1        RETURN_ERROR:: SP_SEARCH_3(364) :: cplan.sql(4849) :: :
1366:
1                EXIT:: SP_SEARCH_3(367) :: cplan.sql(4852) :: :
1100:
1:
1FC_CALC_DURATION:
1389:
1      RETURN_INVALID:: SP_SEARCH_3(390) :: cplan.sql(4875) :: :
1392:
1                EXIT:: SP_SEARCH_3(393) :: cplan.sql(4878) :: :
1397:
1        RETURN_ERROR:: SP_SEARCH_3(398) :: cplan.sql(4883) :: :
1400:
1                EXIT:: SP_SEARCH_3(401) :: cplan.sql(4886) :: :
12:
13:
14:
1*:
15000:
1422:
1        RETURN_ERROR:: SP_SEARCH_3(423) :: cplan.sql(4908) :: :
1CANNOT_SEARCH_ASSIGN:
1425:
1                EXIT:: SP_SEARCH_3(426) :: cplan.sql(4911) :: :
150:
1TRUNC:
1TO_NUMBER:
11.5:
1FC_DECLARE_PMO_TMP_SEARCH:
1TSK_ASSIGNMENT_ID:
1TO_DATE:
1UNRESOLVED_START:
1UNRESOLVED_FINISH:
1BILLABLE:
1CAPITAL_EXPENSE:
1REGULAR_COST:
1REGULAR_RATE:
1TASK_ASSIGNMENTS:
1TASK_ID:
1RESOURCE_ID:
1RESOURCE_TYPE:
1BIT_FLAG:
1SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(S+
1ubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,:n            REG+
1ULAR_COST, REGULAR_RATE:n            INTO h_prj_assignment_id, h_SD, h_FD, h_+
1billable, h_capital_expense,:n            h_reg_cost, h_reg_sale:n           +
1 FROM TASK_ASSIGNMENTS:n            WHERE TASK_ID=h_project_id AND RESOURCE_I+
1D=h_res_id AND RESOURCE_TYPE='1' AND BIT_FLAG='0':
1478:
1           EXCEPTION:: SP_SEARCH_3(479) :: cplan.sql(4964) :: :
1PROFILE_NAME:
1PROFILES:
1PROFILE_ID:
1SELECT PROFILE_NAME INTO h_full_name FROM PROFILES WHERE PROFILE_ID = h_res_i+
1d:
1493:
1           EXCEPTION:: SP_SEARCH_3(494) :: cplan.sql(4979) :: :
1        RETURN_ERROR:: SP_SEARCH_3(501) :: cplan.sql(4986) :: :
1503:
1                EXIT:: SP_SEARCH_3(504) :: cplan.sql(4989) :: :
1RES_ASSIGNMENT_ID:
1PARTICIPANT:
1RESOURCE_ASSIGNMENTS:
1SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO h_assign_id, h_number:n           +
1 FROM RESOURCE_ASSIGNMENTS:n            WHERE ELEMENT_ID = h_task_id AND RESO+
1URCE_TYPE = '1' AND RESOURCE_ID = h_res_id:
1522:
1           EXCEPTION:: SP_SEARCH_3(523) :: cplan.sql(5008) :: :
1NICK_NAME:
1RESOURCES:
1SELECT NICK_NAME INTO h_full_name FROM RESOURCES WHERE RESOURCE_ID = h_res_id:
1537:
1           EXCEPTION:: SP_SEARCH_3(538) :: cplan.sql(5023) :: :
1544:
1        RETURN_ERROR:: SP_SEARCH_3(545) :: cplan.sql(5030) :: :
1547:
1                EXIT:: SP_SEARCH_3(548) :: cplan.sql(5033) :: :
1SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(S+
1ubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,:n            REG+
1ULAR_COST, REGULAR_RATE:n            INTO h_prj_assignment_id, h_SD, h_FD, h_+
1billable, h_capital_expense,:n            h_reg_cost, h_reg_sale:n           +
1 FROM TASK_ASSIGNMENTS:n            WHERE TASK_ID=h_project_id AND RESOURCE_I+
1D=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='0':
1569:
1           EXCEPTION:: SP_SEARCH_3(570) :: cplan.sql(5055) :: :
1SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(S+
1ubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,:n               +
1REGULAR_COST, REGULAR_RATE:n               INTO h_prj_assignment_id, h_SD, h_+
1FD, h_billable, h_capital_expense,:n               h_reg_cost, h_reg_sale:n  +
1             FROM TASK_ASSIGNMENTS:n               WHERE TASK_ID=h_project_id+
1 AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='2':
1602:
1           EXCEPTION:: SP_SEARCH_3(603) :: cplan.sql(5088) :: :
1SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO h_assign_id, h_number:n           +
1 FROM RESOURCE_ASSIGNMENTS:n            WHERE ELEMENT_ID = h_task_id AND RESO+
1URCE_TYPE = '0' AND RESOURCE_ID = h_res_id:
1621:
1           EXCEPTION:: SP_SEARCH_3(622) :: cplan.sql(5107) :: :
1627:
1      RETURN_INVALID:: SP_SEARCH_3(628) :: cplan.sql(5113) :: :
1630:
1                EXIT:: SP_SEARCH_3(631) :: cplan.sql(5116) :: :
1GETSECURITYID:
1SC_TEAMMEMBER_GROUP:
1SC_PARTICIPANT_GROUP:
1SECURITY_ID:
1ELEMENT_TYPE:
1SC_AFFECTED:
1SC_TEAMMEMBER:
1BIT_OR:
1UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT =:n            CASE:n            +
1WHEN SECURITY_ID = h_temp2_security_id AND ELEMENT_TYPE IN (PMO_CNST.SC_PROJE+
1CT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNS+
1T.SC_PROJ_TYPE3) THEN PMO_CNST.SC_AFFECTED:n            WHEN SECURITY_ID = h_+
1temp2_security_id THEN PMO_CNST.SC_TEAMMEMBER:n            WHEN SECURITY_ID =+
1 h_temp_security_id THEN PMO_CNST.SC_AFFECTED:n            ELSE PMO_STPU.BIT_+
1OR( PARTICIPANT, PMO_CNST.SC_TEAMMEMBER):n            END, SECURITY_ID =:n   +
1         CASE:n            WHEN SECURITY_ID = h_temp2_security_id THEN  h_tem+
1p_security_id:n            ELSE SECURITY_ID:n            END:n            WHE+
1RE RES_ASSIGNMENT_ID = h_assign_id:
1FOUND:
1667:
1           EXCEPTION:: SP_SEARCH_3(668) :: cplan.sql(5153) :: :
1FC_AFTER_U_RES_ASSIGN:
1SYS_GUID:
1FC_BEFORE_I_RES_ASSIGN:
1TABLE_TYPE:
1REC_USER:
1REC_DATETIME:
1INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, E+
1LEMENT_TYPE, TABLE_TYPE,:n            RESOURCE_TYPE, RESOURCE_ID, PARTICIPANT+
1, SECURITY_ID, REC_USER, REC_DATETIME):n            VALUES (h_assign_id, h_pr+
1oject_id, h_task_id, h_type, h_table_type,:n            h_res_type, h_res_id,+
1 PMO_CNST.SC_AFFECTED, h_temp_security_id, rec_user001, SYSTIMESTAMP):
1703:
1           EXCEPTION:: SP_SEARCH_3(704) :: cplan.sql(5189) :: :
1TIMEPHASE_PROJECT_ASSIGNMENT:
1SELECT TIMEPHASE_PROJECT_ASSIGNMENT INTO h_pa_timephase FROM PROJECT_DETAILS +
1WHERE PROJECT_ID = h_project_id:
1721:
1           EXCEPTION:: SP_SEARCH_3(722) :: cplan.sql(5207) :: :
1Y:
1-:
1FC_TPHASE_SCHED_ASSIGN:
1SC_SRCH_PROFILE:
1FC_SEARCH_2:
1PMO_TMP_SEARCH:
1PLANNED_SCHED_FLAG:
1RES_ID:
1UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id:
1770:
1           EXCEPTION:: SP_SEARCH_3(771) :: cplan.sql(5256) :: :
1LENGTH:
1/:
1WHILE:
1LOOP:
1',':
1BAK_START_DT:
1BAK_FINISH_DT:
1TO_TIMESTAMP:
1SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(+
1BAK_START_DT, 1, 10)) AND TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)):n            +
1      THEN BAK_START_DT + 1      ELSE TO_TIMESTAMP(h_start_date) END:n       +
1           INTO h_start_DT FROM TASK_ASSIGNMENTS:n                  WHERE TAS+
1K_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1':
1830:
1           EXCEPTION:: SP_SEARCH_3(831) :: cplan.sql(5316) :: :
1SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(+
1BAK_START_DT, 1, 10)) AND TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)):n            +
1      THEN BAK_START_DT + 1      ELSE TO_TIMESTAMP(h_start_date) END:n       +
1           INTO h_start_DT FROM TASK_ASSIGNMENTS:n                  WHERE TAS+
1K_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG+
1='0':
1852:
1           EXCEPTION:: SP_SEARCH_3(853) :: cplan.sql(5338) :: :
1SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(+
1BAK_START_DT, 1, 10)) AND TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)):n            +
1         THEN BAK_START_DT + 1      ELSE TO_TIMESTAMP(h_start_date) END:n    +
1                 INTO h_start_DT FROM TASK_ASSIGNMENTS:n                     +
1WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND+
1 BIT_FLAG='2':
1871:
1           EXCEPTION:: SP_SEARCH_3(872) :: cplan.sql(5357) :: :
1911:
1           EXCEPTION:: SP_SEARCH_3(912) :: cplan.sql(5397) :: :
1925:
1        RETURN_ERROR:: SP_SEARCH_3(926) :: cplan.sql(5411) :: :
1CANNOT_ASSIGN_FROM_PROJECT_AFF:
1928:
1                EXIT:: SP_SEARCH_3(929) :: cplan.sql(5414) :: :
1SELECT t.RESOURCE_ID AS ELEMENT_ID, t.RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEME+
1NT_NAME, 13 AS TYPE_ID, :
1p.E_MAIL_ADDRESS,b.PLANNED_START,b.PLANNED_FINISH,b.BEST AS PLANNED_EFFORT, :
1a.SCHED_START AS START_DATE, a.SCHED_FINISH AS FINISH_DATE, a.BEST AS WORK, a+
1.SCHED_WORK AS REMAINING, :
1rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS +
1CALENDAR,rs.PLANNED AS PLANNED_WORK,:
1rs.SCHED AS SCHED_WORK, :
1 AS WORK_EFFORT, :
1 AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILIT+
1Y,:
1 AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, rs.PLANNED_SCHED_FLAG, '' AS+
1 OBJECTIVES,'' AS RANK :
1FROM RESOURCE_ASSIGNMENTS t LEFT JOIN TASK_ASSIGNMENTS a ON a.RES_ASSIGNMENT_+
1ID=t.RES_ASSIGNMENT_ID AND a.BIT_FLAG='2' :
1LEFT JOIN TASK_ASSIGNMENTS b ON b.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND b+
1.BIT_FLAG='0' :
1JOIN RESOURCES p ON t.RESOURCE_ID=p.RESOURCE_ID :
1LEFT OUTER JOIN PMO_TMP_SEARCH rs ON t.RESOURCE_ID=rs.RES_ID :
1WHERE t.ELEMENT_ID=':
1' AND t.RESOURCE_TYPE='0' AND t.RESOURCE_ID IN (':
1') AND BITAND(t.PARTICIPANT, 4112) > 0 :
1UNION ALL :
1SELECT r.PROFILE_ID AS ELEMENT_ID, ta.RES_ASSIGNMENT_ID, r.PROFILE_NAME AS EL+
1EMENT_NAME, 41 AS TYPE_ID, :
1'' AS E_MAIL_ADDRESS,ta.PLANNED_START,ta.PLANNED_FINISH,ta.BEST AS PLANNED_EF+
1FORT,:
1STR_1899_12_31_00_00_00:
1 AS START_DATE, :
1 AS FINISH_DATE, 0 AS WORK, 0 AS REMAINING, :
1rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS +
1CALENDAR, :
1rs.PLANNED AS PLANNED_WORK,rs.SCHED AS SCHED_WORK, :
1 AS UNIT_EFFORT,:
1(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, :
1 AS MINUTE_PER_DAY , rs.CONTOUR AS ASSIGN_STRING, rs.PLANNED_SCHED_FLAG, '' A+
1S OBJECTIVES,'' AS RANK :
1FROM TASK_ASSIGNMENTS ta JOIN PROFILES r ON ta.TASK_ID=':
1' AND ta.RESOURCE_TYPE='1' AND ta.RESOURCE_ID IN (':
1') AND r.PROFILE_ID=ta.RESOURCE_ID :
1LEFT OUTER JOIN PMO_TMP_SEARCH rs ON ta.RESOURCE_ID=rs.RES_ID :
1ORDER BY TYPE_ID:
11014:
1                EXIT:: SP_SEARCH_3(1015) :: cplan.sql(5500) :: :
11017:
1                EXIT:: SP_SEARCH_3(1018) :: cplan.sql(5503) :: :
1PMO_SELECT_READ:
1RETURN_ERROR_EXCEPTION:
11025:
1           EXCEPTION:: SP_SEARCH_3(1026) :: cplan.sql(5511) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1ID:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
11037:
1           EXCEPTION:: SP_SEARCH_3(1038) :: cplan.sql(5523) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
1a7f
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 1c 81 b0 a3
a0 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 81
b0 a3 :2 a0 6b 1c 81 b0 :2 a0
6b :2 a0 6b 6e a5 b a 10
:2 a0 6b 6e :2 a0 6b 6e a5 b
6e 7e a0 b4 2e 6e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e :3 a0 a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e :3 a0 a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 d :2 a0 d :2 a0 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 d :4 a0 51
a5 b a5 b d a0 51 d
a0 51 d a0 7e b4 2e 5a
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b a0 a5 57 b7
19 3c :2 a0 51 a5 b 7e 51
b4 2e :2 a0 51 a5 b 7e 51
b4 2e a 10 :2 a0 51 a5 b
7e 51 b4 2e a 10 :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b a0 a5 57 b7 19 3c
:2 a0 51 a5 b a0 7e a0 6b
b4 2e 5a a0 7e b4 2e :2 5a
a 10 a0 7e b4 2e :2 5a a
10 :8 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a a 10 :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b a0 a5 57 b7 19
3c a0 51 d a0 b7 a0 7e
b4 2e :2 5a a0 7e b4 2e :2 5a
a 10 :3 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c :2 a0
d :3 a0 6b :3 a0 51 a5 b :2 a0
6b a5 b 5a 51 a5 b d
:4 a0 51 a5 b :2 a0 6b a5 b
5a d :2 a0 51 a5 b 7e 51
b4 2e :b a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a :6 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
6b a0 7e a0 6b b4 2e 5a
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 b7
19 3c b7 19 3c a0 51 d
:5 a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 51 a5 b 7e 51
b4 2e :5 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 6e
a5 b 7e 51 b4 2e :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b :2 a0 a5 57 b7 19 3c
b7 19 3c b7 a0 7e b4 2e
5a :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b a0 a5 57
b7 19 3c :2 a0 d :4 a0 :2 51 a5
b :2 51 a5 b d :3 a0 6b :3 51
a0 a5 b d :3 a0 51 a5 b
a5 b 7e 51 b4 2e :18 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 6b a0 7e a0 6b b4 2e
5a :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 19 3c b7 :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 :2 19 3c b7 :2 19
3c :2 a0 51 a5 b 7e 51 b4
2e 5a :2 a0 51 a5 b 7e 51
b4 2e 5a a 10 a0 51 d
b7 19 3c a0 7e 51 b4 2e
:2 a0 d a0 6e d :6 a0 51 a5
b :2 a0 51 a5 b :2 a0 51 a5
b :2 a0 51 a5 b :2 a0 51 a5
b :2 a0 51 a5 b :2 a0 51 a5
b :2 a0 51 a5 b :3 a0 a5 b
d :2 a0 51 a5 b a0 7e a0
6b b4 2e :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b a0
a5 57 a0 b7 :2 a0 51 a5 b
7e 51 b4 2e :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 51 a5
b :2 a0 a5 57 b7 :2 19 3c b7
19 3c :2 a0 51 a5 b 7e 51
b4 2e 5a :2 a0 51 a5 b 7e
51 b4 2e 5a a 10 :2 a0 51
a5 b 7e 51 b4 2e 5a :2 a0
51 a5 b 7e 51 b4 2e 5a
52 10 a0 51 d a0 b7 :2 a0
51 a5 b 7e 51 b4 2e a0
51 d a0 b7 19 :2 a0 51 a5
b 7e 51 b4 2e 5a :2 a0 51
a5 b 7e 51 b4 2e 5a 52
10 a0 51 d a0 b7 19 :2 a0
51 a5 b 7e 51 b4 2e :3 a0
51 a5 b 7e :2 a0 51 a5 b
b4 2e d a0 51 d b7 :2 19
3c b7 19 3c :2 a0 51 a5 b
7e 51 b4 2e :2 a0 51 a5 b
7e 51 b4 2e :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 19 3c :2 a0 51
a5 b 7e 51 b4 2e a0 51
d b7 :5 a0 51 a5 b a5 b
7e 51 b4 2e a5 b d b7
:2 19 3c b7 19 3c :2 a0 b4 2e
d a0 6e d a0 6e d a0
6e d a0 6e d :3 a0 :2 51 a5
b d :3 a0 :2 51 a5 b d :2 a0
51 a5 b 7e 51 b4 2e a0
4d d :2 a0 d :2 a0 :2 51 a5 b
7e 6e b4 2e :19 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :5 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 6b a0 7e a0 6b b4 2e
5a :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 19 3c :2 a0 d :a a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f a0
b7 :2 a0 :2 51 a5 b 7e 6e b4
2e :5 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 19 3c :2 a0 d
:19 a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b a0 7e a0 6b
b4 2e 5a :19 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 19 3c
:a a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 19 :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
a0 a5 57 b7 :2 19 3c :2 a0 6b
a0 7e a0 6b b4 2e 5a :2 a0
51 :2 a0 51 a5 b :2 a0 6b a0
a5 b d :2 a0 51 :2 a0 51 a5
b :2 a0 6b a0 a5 b d :25 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :3 a0
a5 b d b7 :2 a0 b4 2e d
:2 a0 51 :2 a0 51 a5 b :2 a0 6b
a0 a5 b d :3 a0 6b d a0
51 d :9 a0 a5 b d :18 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :2 19
3c :2 a0 51 a5 b 7e 51 b4
2e 5a a0 7e b4 2e :2 5a a
10 :5 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 :2 51 a5 b 7e
6e b4 2e a0 7e 51 b4 2e
d :2 a0 6b :b a0 a5 b d b7
19 3c b7 19 3c :2 a0 d :2 a0
:2 51 a5 b 7e 6e b4 2e :3 a0
6b d b7 :3 a0 51 a5 b d
b7 :2 19 3c :1e a0 :4 51 a5 b d
a0 7e 51 b4 2e :4 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 19 3c
b7 a0 6e d a0 6e d :2 a0
d :3 a0 a5 b 51 a5 b 7e
51 b4 2e a0 51 d b7 :5 a0
a5 b 51 a5 b 7e 51 b4
2e 5a 7e 51 b4 2e a5 b
d b7 :2 19 3c a0 7e 51 b4
2e a0 51 d :3 a0 51 a5 b
a0 7e a0 b4 2e 82 :3 a0 6b
:2 51 :2 a0 51 a5 b 7e 51 b4
2e 7e 51 b4 2e a0 a5 b
d :3 a0 6b :2 51 :2 a0 51 a5 b
7e 51 b4 2e 7e 51 b4 2e
7e 51 b4 2e a0 a5 b d
:2 a0 :2 51 a5 b 7e 6e b4 2e
:3 a0 6b d :3 a0 a5 b 51 a5
b 7e 51 b4 2e :2 a0 d b7
:3 a0 7e 6e b4 2e :2 51 a5 b
d :3 a0 7e a0 b4 2e :2 51 a5
b d b7 :2 19 3c b7 :3 a0 51
a5 b d :3 a0 a5 b 51 a5
b 7e 51 b4 2e :2 a0 d b7
:3 a0 7e 6e b4 2e :2 51 a5 b
d :3 a0 7e a0 b4 2e :2 51 a5
b d b7 :2 19 3c b7 :2 19 3c
:2 a0 :2 51 a5 b 7e 6e b4 2e
:13 a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f a0 51 d a0 51 d
b7 :14 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a :14 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 19
3c b7 :2 19 3c :2 a0 d :2 a0 :2 51
a5 b 7e 6e b4 2e :3 a0 6b
d b7 :3 a0 51 a5 b d b7
:2 19 3c :2 a0 d :1e a0 :4 51 a5 b
d a0 7e 51 b4 2e :4 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 19
3c :3 a0 51 a5 b 7e 51 b4
2e d b7 a0 47 b7 19 3c
b7 :2 19 3c a0 7e 51 b4 2e
5a :2 a0 51 a5 b 7e 51 b4
2e 5a a 10 :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 19 3c :2 a0 51
a5 b 7e 51 b4 2e :2 a0 d
b7 19 3c :3 a0 a5 b 51 a5
b 7e 51 b4 2e 5a :3 a0 a5
b 51 a5 b 7e 51 b4 2e
5a a 10 a0 6e 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :3 a0 51
a5 b a5 b b4 2e 7e 6e
b4 2e 7e :3 a0 51 a5 b a5
b b4 2e 7e 6e b4 2e 7e
:3 a0 51 a5 b a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :2 a0 6b b4
2e 7e 6e b4 2e 7e :2 a0 6b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :3 a0
51 a5 b a5 b b4 2e 7e
6e b4 2e 7e :3 a0 51 a5 b
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :3 a0 51 a5
b a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e 6e b4 2e d a0 b7
:3 a0 a5 b 51 a5 b 7e 51
b4 2e a0 6e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :3 a0 51 a5
b a5 b b4 2e 7e 6e b4
2e 7e :3 a0 51 a5 b a5 b
b4 2e 7e 6e b4 2e 7e :3 a0
51 a5 b a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e d a0 b7 19 :3 a0
a5 b 51 a5 b 7e 51 b4
2e a0 6e 7e 6e b4 2e 7e
:2 a0 6b b4 2e 7e 6e b4 2e
7e :2 a0 6b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :3 a0 51 a5 b a5 b
b4 2e 7e 6e b4 2e 7e :3 a0
51 a5 b a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:3 a0 51 a5 b a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
d b7 :2 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :3 a0 6b a0 a5 b 65
b7 :2 a0 6b :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 :2 a0 6b :2 6e 7e :2 a0
6b b4 2e 7e 6e b4 2e 7e
:2 a0 6b b4 2e a5 57 b7 19
3c :9 a0 12a e9 dd :2 a0 65 b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c :7 a0
12a e9 dd :2 a0 65 b7 a4 b1
11 4f b7 a6 9 a4 b1 11
68 4f 1d 17 b5 
1a7f
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 6f 6b 59 77 84
80 56 8c 95 91 7f 9d aa
a6 7c b2 bb b7 a5 c3 d0
cc a2 d8 e1 dd cb e9 f6
f2 c8 fe 107 103 f1 10f 11c
118 ee 124 12d 129 117 135 114
13a 13e 142 146 149 14d 16a 155
159 15c 15d 165 154 187 175 151
179 17a 182 174 1a4 192 171 196
197 19f 191 1c1 1af 18e 1b3 1b4
1bc 1ae 1de 1cc 1ab 1d0 1d1 1d9
1cb 1fb 1e9 1c8 1ed 1ee 1f6 1e8
218 206 1e5 20a 20b 213 205 234
223 227 22f 202 250 23b 23f 242
243 24b 222 26d 25b 21f 25f 260
268 25a 28a 278 257 27c 27d 285
277 2a7 295 274 299 29a 2a2 294
2c4 2b2 291 2b6 2b7 2bf 2b1 2e1
2cf 2ae 2d3 2d4 2dc 2ce 2fe 2ec
2cb 2f0 2f1 2f9 2eb 31b 309 2e8
30d 30e 316 308 337 326 32a 332
305 34f 33e 342 34a 325 36c 35a
322 35e 35f 367 359 388 377 37b
356 383 376 3a4 393 397 373 39f
392 3c0 3af 3b3 38f 3bb 3ae 3dc
3cb 3cf 3ab 3d7 3ca 3f8 3e7 3eb
3c7 3f3 3e6 414 403 407 3e3 40f
402 430 41f 423 3ff 42b 41e 44c
43b 43f 41b 447 43a 468 457 45b
437 463 456 484 473 477 453 47f
472 4a0 48f 493 46f 49b 48e 4bc
4ab 4af 48b 4b7 4aa 4d8 4c7 4cb
4a7 4d3 4c6 4f4 4e3 4e7 4c3 4ef
4e2 510 4ff 503 4df 50b 4fe 52c
51b 51f 4fb 527 51a 548 537 53b
517 543 536 564 553 557 533 55f
552 580 56f 573 54f 57b 56e 59c
58b 58f 56b 597 58a 5b8 5a7 5ab
587 5b3 5a6 5d4 5c3 5c7 5a3 5cf
5c2 5f0 5df 5e3 5bf 5eb 5de 60c
5fb 5ff 5db 607 5fa 628 617 61b
5f7 623 616 644 633 637 613 63f
632 660 64f 653 62f 65b 64e 67c
66b 66f 64b 677 66a 698 687 68b
667 693 686 6b4 6a3 6a7 683 6af
6a2 6d1 6bf 69f 6c3 6c4 6cc 6be
6ee 6dc 6bb 6e0 6e1 6e9 6db 70a
6f9 6fd 6d8 705 6f8 726 715 719
6f5 721 714 743 731 711 735 736
73e 730 75f 74e 752 72d 75a 74d
77b 76a 76e 74a 776 769 797 786
78a 766 792 785 7b3 7a2 7a6 782
7ae 7a1 7cf 7be 7c2 79e 7ca 7bd
7eb 7da 7de 7ba 7e6 7d9 807 7f6
7fa 802 7d6 826 80e 812 816 819
821 7f5 82d 831 7f2 835 839 83d
840 845 846 1 848 84d 851 855
858 85d 861 865 868 86d 86e 870
875 878 87c 87d 882 887 88a 88f
890 895 898 89c 89d 8a2 8a5 8aa
8ab 8b0 8b3 8b8 8b9 8be 8c1 8c6
8c7 8cc 8cf 8d3 8d4 8d9 8dc 8e1
8e2 8e7 8ea 8ef 8f0 8f5 8f8 8fd
8fe 903 906 90a 90b 910 913 918
919 91e 921 926 927 92c 92f 933
937 93b 93c 93e 943 944 946 947
94c 94f 954 955 95a 95d 961 965
969 96a 96c 971 972 974 975 97a
97d 982 983 988 98b 98f 993 997
998 99a 99f 9a0 9a2 9a3 9a8 9ab
9b0 9b1 9b6 9b9 9bd 9c1 9c5 9c6
9c8 9cd 9ce 9d0 9d1 9d6 9d9 9de
9df 9e4 9e7 9eb 9ef 9f3 9f4 9f6
9fb 9fc 9fe 9ff a04 a07 a0c a0d
a12 a15 a19 a1d a21 a22 a24 a29
a2a a2c a2d a32 a35 a3a a3b a40
a43 a47 a4b a4f a50 a52 a57 a58
a5a a5b a60 a63 a68 a69 a6e a71
a75 a79 a7d a7e a80 a85 a86 a88
a89 a8e a91 a96 a97 a9c a9f aa3
aa7 aab aac aae ab3 ab4 ab6 ab7
abc abf ac4 ac5 aca acd ad1 ad5
ad9 ada adc ae1 ae2 ae4 ae5 aea
aed af2 af3 af8 afb b00 b01 b06
b09 b0d b0e b13 b16 b1b b1c b21
b24 b29 b2a b2f b32 b37 b38 b3d
b40 b44 b45 b4a b4d b52 b53 b58
b5b b60 b61 b66 b67 b6c b6e b72
b75 b79 b7d b80 b84 b88 b8b b8e
b93 b96 b99 b9c b9d ba2 ba3 1
ba5 baa bae bb2 bb5 bba bbf bc2
bc6 bc7 bcc bcd bd2 bd4 bd8 bdb
bdf be3 be7 beb bef bf3 bf7 bfb
bff c03 c07 c0b c0f c13 c17 c1b
c1f c23 c27 c2b c2f c33 c37 c3b
c3f c43 c47 c4b c4f c53 c57 c5b
c5f c63 c67 c6b c6f c73 c77 c7b
c7f c83 c87 c8b c8f c93 c97 c9b
c9f ca2 ca3 ca5 ca6 ca8 cac cb0
cb3 cb7 cbb cbe cc2 cc6 cc9 cca
ccf cd2 cd6 cda cdd ce1 ce5 ce8
ceb cf0 cf3 cf6 cf9 cfa cff d00
1 d02 d07 d0b d0f d12 d17 d1c
d1f d23 d24 d29 d2c d31 d32 d37
d3a d3f d40 d45 d48 d4c d50 d54
d58 d5b d5c d5e d63 d64 d66 d67
d6c d6d d72 d74 d78 d7b d7f d83
d86 d8a d8e d91 d94 d99 d9c d9f
da2 da3 da8 da9 1 dab db0 db4
db8 dbb dc0 dc5 dc8 dcc dcd dd2
dd3 dd8 dda dde de1 de5 de9 dec
df0 df4 df7 dfb dfc e01 e03 e07
e0a e0e e12 e15 e16 e18 e1b e1e
e1f e24 e28 e2c e2f e30 e32 e35
e38 e39 1 e3e e43 e47 e4b e4e
e4f e51 e54 e57 e58 1 e5d e62
e66 e6a e6d e71 e75 e78 e7b e80
e83 e86 e89 e8a e8f e90 1 e92
e97 e9b e9f ea2 ea7 eac eaf eb3
eb4 eb9 ebc ec1 ec2 ec7 eca ecf
ed0 ed5 ed8 edc ee0 ee4 ee8 eeb
eec eee ef3 ef4 ef6 ef7 efc efd
f02 f04 f08 f0b f0f f13 f16 f1a
f1e f21 f24 f29 f2c f2f f32 f33
f38 f39 1 f3b f40 f44 f48 f4b
f50 f55 f58 f5c f5d f62 f63 f68
f6a f6e f71 f75 f79 f7c f80 f84
f87 f8b f8c f91 f93 f97 f9a f9e
fa2 fa5 fa6 fa8 fac faf fb3 fb6
fb7 fbc fbf fc3 fc6 fc7 fcc fcf
1 fd2 fd7 fdb fde fdf fe4 fe7
1 fea fef ff3 ff7 ffb fff 1003
1007 100b 100f 101b 101f 1023 1026 102a
102e 1030 1034 1038 103c 103f 1043 1047
1049 104a 104f 1 1053 1057 105b 105e
1062 1066 106a 106e 1071 1075 1079 107c
107f 1084 1087 108a 108d 108e 1093 1094
1 1096 109b 109f 10a3 10a6 10ab 10b0
10b3 10b7 10b8 10bd 10c0 10c5 10c6 10cb
10ce 10d3 10d4 10d9 10dc 10e0 10e1 10e6
10e9 10ee 10ef 10f4 10f7 10fb 10ff 1102
1105 1106 1108 1109 110e 110f 1114 1116
111a 111d 111f 1123 1125 1131 1133 1135
1136 113b 113f 1141 114d 114f 1153 1157
115a 115e 1161 1165 1168 1169 116e 1171
1175 1179 117c 117d 117f 1182 1185 1186
118b 1 118e 1193 1197 119b 119e 11a2
11a6 11a9 11ac 11b1 11b4 11b7 11ba 11bb
11c0 11c1 1 11c3 11c8 11cc 11d0 11d3
11d8 11dd 11e0 11e4 11e5 11ea 11ed 11f2
11f3 11f8 11fb 1200 1201 1206 1209 120d
1211 1215 1219 121c 121d 121f 1224 1225
1227 1228 122d 122e 1233 1235 1239 123c
1240 1244 1247 124b 124f 1252 1255 125a
125d 1260 1263 1264 1269 126a 1 126c
1271 1275 1279 127c 1281 1286 1289 128d
128e 1293 1294 1299 129b 129f 12a2 12a6
12aa 12ad 12b1 12b5 12b8 12bc 12bd 12c2
12c4 12c8 12cb 12cf 12d2 12d6 12da 12dc
12e0 12e3 12e4 12e9 12ec 12ef 12f3 12f6
12f7 12fc 12ff 1 1302 1307 130b 130f
1313 131f 1323 1327 132a 132e 1332 1334
1338 133c 1340 1343 1347 134b 134d 134e
1353 1 1357 135b 135f 1362 1366 136a
136e 1372 1375 1379 137d 1380 1383 1388
138b 138e 1391 1392 1397 1398 1 139a
139f 13a3 13a7 13aa 13af 13b4 13b7 13bb
13bc 13c1 13c4 13c9 13ca 13cf 13d2 13d7
13d8 13dd 13e0 13e4 13e5 13ea 13ed 13f2
13f3 13f8 13fb 13ff 1403 1406 1409 140a
140c 140d 1412 1413 1418 141a 141e 1421
1423 1427 1429 1435 1437 1439 143a 143f
1443 1445 1451 1453 1455 1459 145d 1460
1464 1468 146c 1470 1474 1478 147b 147f
1483 1487 148a 148b 148d 1491 1495 1498
1499 149b 149e 14a1 14a2 14a4 14a8 14ac
14b0 14b4 14b8 14bb 14bc 14be 14c2 14c6
14c9 14ca 14cc 14cf 14d3 14d7 14db 14de
14df 14e1 14e4 14e7 14e8 14ed 14f1 14f5
14f9 14fd 1501 1505 1509 150d 1511 1515
1519 1525 1529 152d 1530 1534 1538 153a
153e 1542 1546 1549 154d 1551 1553 1554
1559 1 155d 1561 1565 1568 156c 1570
1574 1578 157b 157f 1583 1586 1589 158e
1591 1594 1597 1598 159d 159e 1 15a0
15a5 15a9 15ad 15b0 15b5 15ba 15bd 15c1
15c2 15c7 15ca 15cf 15d0 15d5 15d8 15dd
15de 15e3 15e6 15ea 15eb 15f0 15f3 15f8
15f9 15fe 1601 1605 1609 160c 160f 1610
1612 1613 1618 1619 161e 1620 1624 1627
1629 162d 162f 163b 163d 163f 1640 1645
1649 164b 1657 1659 165d 1661 1664 1668
166b 166f 1672 1673 1678 167b 167f 1683
1687 168b 168f 1693 169f 16a3 16a7 16aa
16ae 16b2 16b4 16b8 16bc 16c0 16c3 16c7
16cb 16cd 16ce 16d3 1 16d7 16db 16df
16e2 16e6 16ea 16ee 16f2 16f5 16f9 16fd
1700 1703 1708 170b 170e 1711 1712 1717
1718 1 171a 171f 1723 1727 172a 172f
1734 1737 173b 173c 1741 1744 1749 174a
174f 1752 1757 1758 175d 1760 1764 1765
176a 176d 1772 1773 1778 177b 177f 1783
1786 1789 178a 178c 178d 1792 1793 1798
179a 179e 17a1 17a3 17a7 17a9 17b5 17b7
17b9 17ba 17bf 17c3 17c5 17d1 17d3 17d7
17db 17de 17e2 17e5 17e9 17ec 17ed 17f2
17f5 17f9 17fd 1800 1804 1808 180b 180e
1813 1816 1819 181c 181d 1822 1823 1
1825 182a 182e 1832 1835 183a 183f 1842
1846 1847 184c 184f 1854 1855 185a 185d
1862 1863 1868 186b 186f 1873 1877 187b
187e 187f 1881 1886 1887 1889 188a 188f
1892 1897 1898 189d 18a0 18a5 18a6 18ab
18ae 18b2 18b3 18b8 18bb 18c0 18c1 18c6
18c7 18cc 18ce 18d2 18d5 18d9 18dd 18e0
18e4 18e8 18eb 18ee 18f3 18f6 18f9 18fc
18fd 1902 1903 1 1905 190a 190e 1912
1915 191a 191f 1922 1926 1927 192c 192d
1932 1934 1938 193b 193f 1943 1946 194a
194e 1951 1955 1959 195a 195f 1961 1965
1968 196a 196e 1971 1975 1978 197c 1980
1984 1988 198c 1990 199c 19a0 19a4 19a7
19ab 19af 19b1 19b5 19b9 19bd 19c0 19c4
19c8 19ca 19cb 19d0 1 19d4 19d8 19dc
19df 19e3 19e7 19eb 19ef 19f2 19f6 19fa
19fd 1a00 1a05 1a08 1a0b 1a0e 1a0f 1a14
1a15 1 1a17 1a1c 1a20 1a24 1a27 1a2c
1a31 1a34 1a38 1a39 1a3e 1a41 1a46 1a47
1a4c 1a4f 1a54 1a55 1a5a 1a5d 1a61 1a62
1a67 1a6a 1a6f 1a70 1a75 1a78 1a7c 1a80
1a83 1a86 1a87 1a89 1a8a 1a8f 1a90 1a95
1a97 1a9b 1a9e 1aa0 1aa4 1aa6 1ab2 1ab4
1ab6 1ab7 1abc 1ac0 1ac2 1ace 1ad0 1ad4
1ad8 1adb 1adc 1ade 1ae1 1ae4 1ae5 1aea
1aee 1af2 1af6 1afa 1afe 1b0a 1b0e 1b12
1b15 1b19 1b1d 1b1f 1b23 1b27 1b2b 1b2e
1b32 1b36 1b38 1b39 1b3e 1 1b42 1b46
1b4a 1b4d 1b51 1b55 1b59 1b5d 1b60 1b64
1b68 1b6b 1b6e 1b73 1b76 1b79 1b7c 1b7d
1b82 1b83 1 1b85 1b8a 1b8e 1b92 1b95
1b9a 1b9f 1ba2 1ba6 1ba7 1bac 1baf 1bb4
1bb5 1bba 1bbd 1bc2 1bc3 1bc8 1bcb 1bcf
1bd0 1bd5 1bd8 1bdd 1bde 1be3 1be6 1bea
1bee 1bf1 1bf4 1bf5 1bf7 1bf8 1bfd 1bfe
1c03 1c05 1c09 1c0c 1c0e 1c12 1c14 1c20
1c22 1c24 1c25 1c2a 1c2e 1c30 1c3c 1c3e
1c42 1c46 1c49 1c4d 1c52 1c53 1c55 1c58
1c5b 1c5c 1c61 1c65 1c69 1c6c 1c70 1c74
1c77 1c7a 1c7f 1c82 1c85 1c88 1c89 1c8e
1c8f 1 1c91 1c96 1c9a 1c9e 1ca1 1ca6
1cab 1cae 1cb2 1cb3 1cb8 1cbb 1cc0 1cc1
1cc6 1cc9 1cce 1ccf 1cd4 1cd7 1cdb 1cdf
1ce3 1ce7 1cea 1ceb 1ced 1cf2 1cf3 1cf5
1cf6 1cfb 1cfe 1d03 1d04 1d09 1d0c 1d11
1d12 1d17 1d1a 1d1e 1d1f 1d24 1d27 1d2c
1d2d 1d32 1d33 1d38 1d3a 1d3e 1d41 1d45
1d49 1d4c 1d50 1d54 1d57 1d5a 1d5f 1d62
1d65 1d68 1d69 1d6e 1d6f 1 1d71 1d76
1d7a 1d7e 1d81 1d86 1d8b 1d8e 1d92 1d93
1d98 1d99 1d9e 1da0 1da4 1da7 1dab 1daf
1db2 1db6 1dba 1dbd 1dc1 1dc5 1dc6 1dcb
1dcd 1dd1 1dd4 1dd6 1dda 1ddd 1ddf 1de3
1de6 1de7 1dec 1def 1df3 1df7 1dfa 1dfe
1e02 1e05 1e08 1e0d 1e10 1e13 1e16 1e17
1e1c 1e1d 1 1e1f 1e24 1e28 1e2c 1e2f
1e34 1e39 1e3c 1e40 1e41 1e46 1e49 1e4e
1e4f 1e54 1e57 1e5c 1e5d 1e62 1e65 1e69
1e6d 1e71 1e75 1e78 1e79 1e7b 1e80 1e81
1e83 1e84 1e89 1e8a 1e8f 1e91 1e95 1e98
1e9c 1ea0 1ea3 1ea7 1eab 1eae 1eb1 1eb6
1eb9 1ebc 1ebf 1ec0 1ec5 1ec6 1 1ec8
1ecd 1ed1 1ed5 1ed8 1edd 1ee2 1ee5 1ee9
1eea 1eef 1ef0 1ef5 1ef7 1efb 1efe 1f02
1f06 1f09 1f0d 1f11 1f14 1f18 1f19 1f1e
1f20 1f24 1f27 1f2b 1f2f 1f33 1f37 1f3b
1f3f 1f43 1f46 1f49 1f4a 1f4c 1f4f 1f52
1f53 1f55 1f59 1f5d 1f61 1f65 1f68 1f6b
1f6e 1f71 1f75 1f76 1f78 1f7c 1f80 1f84
1f88 1f8b 1f8c 1f8e 1f8f 1f91 1f94 1f97
1f98 1f9d 1fa1 1fa5 1fa9 1fad 1fb1 1fb5
1fb9 1fbd 1fc1 1fc5 1fc9 1fcd 1fd1 1fd5
1fd9 1fdd 1fe1 1fe5 1fe9 1fed 1ff1 1ff5
1ff9 1ffd 2009 200d 2011 2014 2018 201c
201e 2022 2026 202a 202d 2031 2035 2037
2038 203d 1 2041 2045 2049 204c 2050
2054 2058 205c 205f 2063 2067 206a 206d
2072 2075 2078 207b 207c 2081 2082 1
2084 2089 208d 2091 2094 2099 209e 20a1
20a5 20a6 20ab 20ae 20b3 20b4 20b9 20bc
20c1 20c2 20c7 20ca 20ce 20cf 20d4 20d7
20dc 20dd 20e2 20e5 20e9 20ed 20f0 20f3
20f4 20f6 20f7 20fc 20fd 2102 2104 2108
210b 210d 2111 2113 211f 2121 2123 2124
2129 212d 212f 213b 213d 2141 2145 2148
214c 214f 2153 2156 2157 215c 215f 2163
2167 216a 216e 2172 2175 2178 217d 2180
2183 2186 2187 218c 218d 1 218f 2194
2198 219c 219f 21a4 21a9 21ac 21b0 21b1
21b6 21b9 21be 21bf 21c4 21c7 21cc 21cd
21d2 21d5 21d9 21dd 21e1 21e5 21e8 21e9
21eb 21f0 21f1 21f3 21f4 21f9 21fc 2201
2202 2207 220a 220f 2210 2215 2218 221c
221d 2222 2225 222a 222b 2230 2231 2236
2238 223c 223f 2243 2247 224a 224e 2252
2255 2258 225d 2260 2263 2266 2267 226c
226d 1 226f 2274 2278 227c 227f 2284
2289 228c 2290 2291 2296 2297 229c 229e
22a2 22a5 22a9 22ad 22b0 22b4 22b8 22bb
22bf 22c3 22c4 22c9 22cb 22cf 22d2 22d4
22d8 22dc 22df 22e3 22e7 22ea 22ed 22f2
22f5 22f8 22fb 22fc 2301 2302 1 2304
2309 230d 2311 2314 2319 231e 2321 2325
2326 232b 232e 2333 2334 2339 233c 2341
2342 2347 234a 234e 2352 2356 235a 235d
235e 2360 2365 2366 2368 2369 236e 2371
2376 2377 237c 237f 2384 2385 238a 238d
2391 2392 2397 239a 239f 23a0 23a5 23a6
23ab 23ad 23b1 23b4 23b8 23bc 23bf 23c3
23c7 23ca 23cd 23d2 23d5 23d8 23db 23dc
23e1 23e2 1 23e4 23e9 23ed 23f1 23f4
23f9 23fe 2401 2405 2406 240b 240c 2411
2413 2417 241a 241e 2422 2425 2429 242d
2430 2434 2438 2439 243e 2440 2444 2448
244b 244d 2451 2455 2458 245c 2460 2463
2464 2466 2469 246c 246d 2472 2475 2479
247d 2480 2481 2483 2486 2489 248a 248f
1 2492 2497 249b 249e 24a2 24a4 24a8
24ab 24af 24b2 24b5 24b6 24bb 24bf 24c3
24c7 24cb 24d0 24d4 24d8 24dc 24e0 24e4
24e8 24ec 24ef 24f0 24f2 24f6 24fa 24fd
24fe 2500 2504 2508 250b 250c 250e 2512
2516 2519 251a 251c 2520 2524 2527 2528
252a 252e 2532 2535 2536 2538 253c 2540
2543 2544 2546 254a 254e 2551 2552 2554
2558 255c 2560 2561 2563 2567 256b 256f
2572 2573 2575 2579 257c 2580 2583 2584
2589 258d 2591 2594 2598 259c 259f 25a2
25a7 25aa 25ad 25b0 25b1 25b6 25b7 1
25b9 25be 25c2 25c6 25c9 25ce 25d3 25d6
25da 25db 25e0 25e3 25e8 25e9 25ee 25f1
25f6 25f7 25fc 25ff 2603 2607 260b 260f
2612 2613 2615 261a 261b 261d 261e 2623
2624 2629 262b 262f 2632 2636 263a 263d
2641 2645 2648 264b 2650 2653 2656 2659
265a 265f 2660 1 2662 2667 266b 266f
2672 2677 267c 267f 2683 2684 2689 268a
268f 2691 2695 2698 269c 26a0 26a3 26a7
26ab 26ae 26b2 26b3 26b8 26bc 26be 26c2
26c6 26c9 26ca 26cc 26cf 26d2 26d3 26d8
26dc 26e0 26e3 26e7 26eb 26ee 26f1 26f6
26f9 26fc 26ff 2700 2705 2706 1 2708
270d 2711 2715 2718 271d 2722 2725 2729
272a 272f 2732 2737 2738 273d 2740 2745
2746 274b 274e 2752 2756 275a 275b 275d
2762 2763 2765 2766 276b 276e 2773 2774
2779 277c 2781 2782 2787 278a 278e 278f
2794 2797 279c 279d 27a2 27a3 27a8 27aa
27ae 27b1 27b5 27b9 27bc 27c0 27c4 27c7
27ca 27cf 27d2 27d5 27d8 27d9 27de 27df
1 27e1 27e6 27ea 27ee 27f1 27f6 27fb
27fe 2802 2803 2808 2809 280e 2810 2814
2817 281b 281f 2822 2826 282a 282d 282e
2830 2834 2838 2839 283e 2840 2844 2848
284b 284d 2851 2854 2858 285c 285f 2860
2862 2865 2868 2869 286e 2871 2875 2879
287c 287d 287f 2882 2885 2886 288b 1
288e 2893 2897 289b 289e 289f 28a1 28a4
28a7 28a8 28ad 28b0 28b4 28b8 28bb 28bc
28be 28c1 28c4 28c5 28ca 1 28cd 28d2
28d6 28d9 28dd 28e1 28e3 28e7 28eb 28ee
28ef 28f1 28f4 28f7 28f8 28fd 2901 2904
2908 290c 290e 2912 2916 291a 291d 291e
2920 2923 2926 2927 292c 292f 2933 2937
293a 293b 293d 2940 2943 2944 2949 1
294c 2951 2955 2958 295c 2960 2962 2966
296a 296e 2971 2972 2974 2977 297a 297b
2980 2984 2988 298c 298f 2990 2992 2995
2999 299d 29a0 29a1 29a3 29a4 29a9 29ad
29b1 29b4 29b8 29ba 29be 29c2 29c5 29c7
29cb 29ce 29d2 29d6 29d9 29da 29dc 29df
29e2 29e3 29e8 29ec 29f0 29f3 29f4 29f6
29f9 29fc 29fd 2a02 2a06 2a0a 2a0d 2a11
2a15 2a18 2a1b 2a20 2a23 2a26 2a29 2a2a
2a2f 2a30 1 2a32 2a37 2a3b 2a3f 2a42
2a47 2a4c 2a4f 2a53 2a54 2a59 2a5c 2a61
2a62 2a67 2a6a 2a6f 2a70 2a75 2a78 2a7c
2a80 2a84 2a88 2a8b 2a8c 2a8e 2a93 2a94
2a96 2a97 2a9c 2a9f 2aa4 2aa5 2aaa 2aad
2ab2 2ab3 2ab8 2abb 2abf 2ac0 2ac5 2ac8
2acd 2ace 2ad3 2ad4 2ad9 2adb 2adf 2ae2
2ae6 2aea 2aed 2af1 2af5 2af8 2afb 2b00
2b03 2b06 2b09 2b0a 2b0f 2b10 1 2b12
2b17 2b1b 2b1f 2b22 2b27 2b2c 2b2f 2b33
2b34 2b39 2b3a 2b3f 2b41 2b45 2b48 2b4c
2b50 2b53 2b57 2b5b 2b5e 2b62 2b66 2b67
2b6c 2b6e 2b72 2b75 2b79 2b7d 2b80 2b81
2b83 2b86 2b89 2b8a 2b8f 2b93 2b96 2b9a
2b9c 2ba0 2ba4 2ba8 2bac 2bb0 2bb3 2bb4
2bb6 2bb7 2bb9 2bbc 2bbf 2bc0 2bc5 2bc6
2bc8 2bcc 2bce 2bd2 2bd6 2bd9 2bdb 2bdf
2be2 2be6 2bea 2beb 2bf0 2bf4 2bf8 2bfd
2c01 2c05 2c0a 2c0e 2c12 2c17 2c1b 2c1f
2c24 2c28 2c2c 2c30 2c34 2c37 2c3a 2c3b
2c3d 2c41 2c45 2c49 2c4d 2c50 2c53 2c54
2c56 2c5a 2c5e 2c62 2c65 2c66 2c68 2c6b
2c6e 2c6f 2c74 2c78 2c79 2c7d 2c81 2c85
2c89 2c8d 2c91 2c94 2c97 2c98 2c9a 2c9d
2ca2 2ca3 2ca8 2cac 2cb0 2cb4 2cb8 2cbc
2cc0 2cc4 2cc8 2ccc 2cd0 2cd4 2cd8 2cdc
2ce0 2ce4 2ce8 2cec 2cf0 2cf4 2cf8 2cfc
2d00 2d04 2d08 2d0c 2d18 2d1c 2d20 2d23
2d27 2d2b 2d2d 2d31 2d35 2d39 2d3c 2d40
2d44 2d46 2d47 2d4c 1 2d50 2d54 2d58
2d5b 2d5f 2d63 2d67 2d6b 2d6e 2d72 2d76
2d79 2d7c 2d81 2d84 2d87 2d8a 2d8b 2d90
2d91 1 2d93 2d98 2d9c 2da0 2da3 2da8
2dad 2db0 2db4 2db5 2dba 2dbd 2dc2 2dc3
2dc8 2dcb 2dd0 2dd1 2dd6 2dd9 2ddd 2dde
2de3 2de6 2deb 2dec 2df1 2df4 2df8 2dfc
2dff 2e02 2e03 2e05 2e06 2e0b 2e0c 2e11
2e13 2e17 2e1a 2e1c 2e20 2e22 2e2e 2e30
2e32 2e33 2e38 2e3c 2e3e 2e4a 2e4c 2e50
2e54 2e58 2e5c 2e60 2e6c 2e70 2e74 2e77
2e7b 2e7f 2e81 2e85 2e89 2e8d 2e90 2e94
2e98 2e9a 2e9b 2ea0 1 2ea4 2ea8 2eac
2eaf 2eb3 2eb7 2ebb 2ebf 2ec2 2ec6 2eca
2ecd 2ed0 2ed5 2ed8 2edb 2ede 2edf 2ee4
2ee5 1 2ee7 2eec 2ef0 2ef4 2ef7 2efc
2f01 2f04 2f08 2f09 2f0e 2f11 2f16 2f17
2f1c 2f1f 2f24 2f25 2f2a 2f2d 2f31 2f32
2f37 2f3a 2f3f 2f40 2f45 2f48 2f4c 2f50
2f53 2f56 2f57 2f59 2f5a 2f5f 2f60 2f65
2f67 2f6b 2f6e 2f70 2f74 2f76 2f82 2f84
2f86 2f87 2f8c 2f90 2f92 2f9e 2fa0 2fa4
2fa8 2fab 2faf 2fb2 2fb6 2fb9 2fba 2fbf
2fc2 2fc6 2fca 2fcd 2fd1 2fd5 2fd8 2fdb
2fe0 2fe3 2fe6 2fe9 2fea 2fef 2ff0 1
2ff2 2ff7 2ffb 2fff 3002 3007 300c 300f
3013 3014 3019 301c 3021 3022 3027 302a
302f 3030 3035 3038 303c 3040 3044 3048
304b 304c 304e 3053 3054 3056 3057 305c
305f 3064 3065 306a 306d 3072 3073 3078
307b 307f 3080 3085 3088 308d 308e 3093
3094 3099 309b 309f 30a2 30a6 30aa 30ad
30b1 30b5 30b8 30bb 30c0 30c3 30c6 30c9
30ca 30cf 30d0 1 30d2 30d7 30db 30df
30e2 30e7 30ec 30ef 30f3 30f4 30f9 30fa
30ff 3101 3105 3108 310c 3110 3113 3117
311b 311e 3122 3126 3127 312c 312e 3132
3135 3139 313d 3141 3145 3149 314d 3151
3155 3159 315d 3161 3165 3169 3175 3179
317d 3180 3184 3188 318a 318e 3192 3196
3199 319d 31a1 31a3 31a4 31a9 1 31ad
31b1 31b5 31b8 31bc 31c0 31c4 31c8 31cb
31cf 31d3 31d6 31d9 31de 31e1 31e4 31e7
31e8 31ed 31ee 1 31f0 31f5 31f9 31fd
3200 3205 320a 320d 3211 3212 3217 321a
321f 3220 3225 3228 322d 322e 3233 3236
323a 323b 3240 3243 3248 3249 324e 3251
3255 3259 325c 325f 3260 3262 3263 3268
3269 326e 3270 3274 3277 3279 327d 327f
328b 328d 328f 3290 3295 3299 329b 32a7
32a9 32ad 32af 32b3 32b7 32ba 32bd 32be
32c0 32c3 32c8 32c9 32ce 32d2 32d6 32da
32de 32e2 32ee 32f2 32f6 32f9 32fd 3301
3303 3307 330b 330f 3312 3316 331a 331c
331d 3322 1 3326 332a 332e 3331 3335
3339 333d 3341 3344 3348 334c 334f 3352
3357 335a 335d 3360 3361 3366 3367 1
3369 336e 3372 3376 3379 337e 3383 3386
338a 338b 3390 3393 3398 3399 339e 33a1
33a6 33a7 33ac 33af 33b3 33b4 33b9 33bc
33c1 33c2 33c7 33ca 33ce 33d2 33d5 33d8
33d9 33db 33dc 33e1 33e2 33e7 33e9 33ed
33f0 33f2 33f6 33f8 3404 3406 3408 3409
340e 3412 3414 3420 3422 3426 342a 342d
3431 3434 3438 343b 343c 3441 3444 3448
344c 344f 3453 3457 345a 345d 3462 3465
3468 346b 346c 3471 3472 1 3474 3479
347d 3481 3484 3489 348e 3491 3495 3496
349b 349e 34a3 34a4 34a9 34ac 34b1 34b2
34b7 34ba 34be 34c2 34c6 34ca 34cd 34ce
34d0 34d5 34d6 34d8 34d9 34de 34e1 34e6
34e7 34ec 34ef 34f4 34f5 34fa 34fd 3501
3502 3507 350a 350f 3510 3515 3516 351b
351d 3521 3524 3528 352c 352f 3533 3537
353a 353d 3542 3545 3548 354b 354c 3551
3552 1 3554 3559 355d 3561 3564 3569
356e 3571 3575 3576 357b 357c 3581 3583
3587 358a 358e 3592 3595 3599 359d 35a0
35a4 35a8 35a9 35ae 35b0 35b4 35b7 35bb
35bf 35c3 35c7 35cb 35cf 35d3 35d7 35db
35df 35e3 35e7 35eb 35ef 35f3 35f7 35fb
35ff 3603 3607 360b 360f 3613 3617 361b
361f 3623 3627 3633 3637 363b 363e 3642
3646 3648 364c 3650 3654 3657 365b 365f
3661 3662 3667 1 366b 366f 3673 3676
367a 367e 3682 3686 3689 368d 3691 3694
3697 369c 369f 36a2 36a5 36a6 36ab 36ac
1 36ae 36b3 36b7 36bb 36be 36c3 36c8
36cb 36cf 36d0 36d5 36d8 36dd 36de 36e3
36e6 36eb 36ec 36f1 36f4 36f8 36f9 36fe
3701 3706 3707 370c 370f 3713 3717 371a
371d 371e 3720 3721 3726 3727 372c 372e
3732 3735 3737 373b 373d 3749 374b 374d
374e 3753 3757 3759 3765 3767 376b 376f
3772 3776 3779 377d 3780 3781 3786 3789
378d 3791 3795 3799 379d 37a1 37a5 37a9
37ad 37b1 37b5 37b9 37bd 37c1 37c5 37c9
37cd 37d1 37d5 37d9 37dd 37e1 37e5 37e9
37ed 37f9 37fd 3801 3804 3808 380c 380e
3812 3816 381a 381d 3821 3825 3827 3828
382d 1 3831 3835 3839 383c 3840 3844
3848 384c 384f 3853 3857 385a 385d 3862
3865 3868 386b 386c 3871 3872 1 3874
3879 387d 3881 3884 3889 388e 3891 3895
3896 389b 389e 38a3 38a4 38a9 38ac 38b1
38b2 38b7 38ba 38be 38bf 38c4 38c7 38cc
38cd 38d2 38d5 38d9 38dd 38e0 38e3 38e4
38e6 38e7 38ec 38ed 38f2 38f4 38f8 38fb
38fd 3901 3903 390f 3911 3913 3914 3919
391d 391f 392b 392d 392f 3933 3936 393a
393e 3942 3946 394a 394e 3952 3956 395a
395e 396a 396e 3972 3975 3979 397d 397f
3983 3987 398b 398e 3992 3996 3998 3999
399e 1 39a2 39a6 39aa 39ad 39b1 39b5
39b9 39bd 39c0 39c4 39c8 39cb 39ce 39d3
39d6 39d9 39dc 39dd 39e2 39e3 1 39e5
39ea 39ee 39f2 39f5 39fa 39ff 3a02 3a06
3a07 3a0c 3a0f 3a14 3a15 3a1a 3a1d 3a22
3a23 3a28 3a2b 3a2f 3a30 3a35 3a38 3a3d
3a3e 3a43 3a46 3a4a 3a4e 3a51 3a54 3a55
3a57 3a58 3a5d 3a5e 3a63 3a65 3a69 3a6c
3a6e 3a72 3a74 3a80 3a82 3a84 3a85 3a8a
3a8e 3a90 3a9c 3a9e 3aa0 3aa4 3aa8 3aac
3aaf 3ab3 3ab7 3aba 3abd 3ac2 3ac5 3ac8
3acb 3acc 3ad1 3ad2 1 3ad4 3ad9 3add
3ae1 3ae4 3ae9 3aee 3af1 3af5 3af6 3afb
3afe 3b03 3b04 3b09 3b0c 3b11 3b12 3b17
3b1a 3b1e 3b22 3b26 3b2a 3b2d 3b2e 3b30
3b35 3b36 3b38 3b39 3b3e 3b3f 3b44 3b46
3b4a 3b4d 3b51 3b55 3b58 3b5c 3b60 3b63
3b66 3b6b 3b6e 3b71 3b74 3b75 3b7a 3b7b
1 3b7d 3b82 3b86 3b8a 3b8d 3b92 3b97
3b9a 3b9e 3b9f 3ba4 3ba5 3baa 3bac 3bb0
3bb3 3bb7 3bbb 3bbe 3bc2 3bc6 3bc9 3bcd
3bce 3bd3 3bd5 3bd9 3bdd 3be0 3be4 3be8
3beb 3bef 3bf2 3bf6 3bf9 3bfa 3bff 3c02
3c06 3c0a 3c0d 3c11 3c15 3c18 3c19 3c1b
3c1f 3c23 3c26 3c2a 3c2b 3c2d 3c31 3c35
3c39 3c3c 3c40 3c44 3c47 3c48 3c4a 3c4e
3c52 3c55 3c59 3c5a 3c5c 3c60 3c64 3c68
3c6c 3c70 3c74 3c78 3c7c 3c80 3c84 3c88
3c8c 3c90 3c94 3c98 3c9c 3ca0 3ca4 3ca8
3cac 3cb0 3cb4 3cb8 3cbc 3cc0 3cc4 3cc8
3ccc 3cd0 3cd4 3cd8 3cdc 3ce0 3ce4 3ce8
3cec 3cf0 3cf4 3d00 3d04 3d09 3d0d 3d10
3d11 3d16 3d1a 3d1e 3d21 3d24 3d28 3d2a
3d2e 3d32 3d35 3d38 3d3c 3d3e 3d42 3d46
3d49 3d4b 3d4f 3d53 3d57 3d5a 3d5e 3d62
3d64 3d65 3d6a 1 3d6e 3d72 3d76 3d79
3d7d 3d81 3d85 3d89 3d8c 3d90 3d94 3d97
3d9a 3d9f 3da2 3da5 3da8 3da9 3dae 3daf
1 3db1 3db6 3dba 3dbe 3dc1 3dc6 3dcb
3dce 3dd2 3dd3 3dd8 3ddb 3de0 3de1 3de6
3de9 3dee 3def 3df4 3df7 3dfb 3dfc 3e01
3e04 3e09 3e0a 3e0f 3e12 3e16 3e1a 3e1d
3e20 3e21 3e23 3e24 3e29 3e2a 3e2f 3e31
3e35 3e38 3e3a 3e3e 3e40 3e4c 3e4e 3e50
3e51 3e56 3e5a 3e5c 3e68 3e6a 3e6e 3e72
3e76 3e77 3e79 3e7d 3e7f 3e83 3e87 3e88
3e8d 3e91 3e95 3e99 3e9c 3ea0 3ea4 3ea7
3ea8 3eaa 3eae 3eb2 3eb5 3eb9 3eba 3ebc
3ec0 3ec4 3ec8 3ecc 3ecf 3ed3 3ed7 3eda
3ede 3ee2 3ee6 3eea 3eee 3ef2 3ef6 3efa
3efe 3f02 3f03 3f05 3f09 3f0d 3f11 3f15
3f19 3f1d 3f21 3f25 3f29 3f2d 3f31 3f35
3f39 3f3d 3f41 3f45 3f49 3f4d 3f51 3f55
3f59 3f5d 3f61 3f65 3f69 3f75 3f79 3f7e
3f82 3f85 3f86 3f8b 3f8f 3f93 3f96 3f99
3f9d 3f9f 3fa3 3fa7 3faa 3fad 3fb1 3fb3
3fb7 3fbb 3fbe 3fc0 3fc4 3fc8 3fcc 3fcf
3fd3 3fd7 3fd9 3fda 3fdf 1 3fe3 3fe7
3feb 3fee 3ff2 3ff6 3ffa 3ffe 4001 4005
4009 400c 400f 4014 4017 401a 401d 401e
4023 4024 1 4026 402b 402f 4033 4036
403b 4040 4043 4047 4048 404d 4050 4055
4056 405b 405e 4063 4064 4069 406c 4070
4071 4076 4079 407e 407f 4084 4087 408b
408f 4092 4095 4096 4098 4099 409e 409f
40a4 40a6 40aa 40ad 40af 40b3 40b5 40c1
40c3 40c5 40c6 40cb 40cf 40d1 40dd 40df
40e1 40e5 40e9 40ec 40f0 40f4 40f7 40f8
40fa 40fd 4100 4101 4106 4109 410d 4110
4111 4116 4119 1 411c 4121 4125 4129
412d 4131 4135 4141 4145 4149 414c 4150
4154 4156 415a 415e 4162 4165 4169 416d
416f 4170 4175 1 4179 417d 4181 4184
4188 418c 4190 4194 4197 419b 419f 41a2
41a5 41aa 41ad 41b0 41b3 41b4 41b9 41ba
1 41bc 41c1 41c5 41c9 41cc 41d1 41d6
41d9 41dd 41de 41e3 41e6 41eb 41ec 41f1
41f4 41f9 41fa 41ff 4202 4206 4207 420c
420f 4214 4215 421a 421d 4221 4225 4228
422b 422c 422e 422f 4234 4235 423a 423c
4240 4243 4245 4249 424b 4257 4259 425b
425c 4261 4265 4267 4273 4275 4279 427d
4280 4283 4284 4286 4289 428e 428f 4294
4298 429b 429e 429f 42a4 42a8 42ac 42b0
42b3 42b7 42bb 42bf 42c3 42c7 42cb 42cf
42d3 42d7 42db 42df 42e0 42e2 42e6 42e8
42ec 42ef 42f1 42f5 42f8 42fc 4300 4304
4308 430c 430f 4312 4313 4315 4318 431d
431e 4323 4327 432b 432f 4332 4336 4338
433c 4340 4344 4347 4348 434a 434e 4350
4354 4358 435b 435f 4363 4367 436b 436f
4373 4377 437b 437f 4383 4387 438b 438f
4393 4397 439b 439f 43a3 43a7 43ab 43af
43b3 43b7 43bb 43bf 43c3 43c7 43cb 43cf
43d3 43d6 43d9 43dc 43df 43e0 43e2 43e6
43ea 43ed 43f0 43f1 43f6 43fa 43fe 4402
4406 4412 4416 441b 441f 4422 4423 4428
442c 4430 4433 4436 443a 443c 4440 4444
4447 444a 444e 4450 4454 4458 445b 445d
4461 4465 4469 446c 4470 4474 4476 4477
447c 1 4480 4484 4488 448b 448f 4493
4497 449b 449e 44a2 44a6 44a9 44ac 44b1
44b4 44b7 44ba 44bb 44c0 44c1 1 44c3
44c8 44cc 44d0 44d3 44d8 44dd 44e0 44e4
44e5 44ea 44ed 44f2 44f3 44f8 44fb 4500
4501 4506 4509 450d 450e 4513 4516 451b
451c 4521 4524 4528 452c 452f 4532 4533
4535 4536 453b 453c 4541 4543 4547 454a
454c 4550 4552 455e 4560 4562 4563 4568
456c 456e 457a 457c 457e 4582 4585 4587
458b 4590 4594 4598 459d 45a1 45a5 45a9
45ad 45b1 45b5 45b9 45ba 45bc 45bf 45c0
45c2 45c5 45c8 45c9 45ce 45d2 45d5 45d9
45db 45df 45e3 45e7 45eb 45ef 45f0 45f2
45f5 45f6 45f8 45fb 45fe 45ff 4604 4607
460a 460d 460e 4613 4614 4616 461a 461c
4620 4624 4627 462b 462e 4631 4632 4637
463b 463e 4642 4646 464a 464e 4651 4652
4654 4658 465b 465f 4660 4665 4667 466b
466f 4673 4676 4679 467c 4680 4684 4687
4688 468a 468d 4690 4691 4696 4699 469c
469d 46a2 46a6 46a7 46a9 46ad 46b1 46b5
46b9 46bc 46bf 46c2 46c6 46ca 46cd 46ce
46d0 46d3 46d6 46d7 46dc 46df 46e2 46e3
46e8 46eb 46ee 46ef 46f4 46f8 46f9 46fb
46ff 4703 4707 470a 470d 470e 4710 4713
4718 4719 471e 4722 4726 472a 472d 4731
4735 4739 473d 473e 4740 4743 4744 4746
4749 474c 474d 4752 4756 475a 475e 4760
4764 4768 476c 476f 4774 4775 477a 477d
4780 4781 4783 4787 478b 478f 4793 4796
479a 479b 47a0 47a3 47a6 47a7 47a9 47ad
47af 47b3 47b7 47ba 47bc 47c0 47c4 47c8
47cb 47cc 47ce 47d2 47d6 47da 47de 47df
47e1 47e4 47e5 47e7 47ea 47ed 47ee 47f3
47f7 47fb 47ff 4801 4805 4809 480d 4810
4815 4816 481b 481e 4821 4822 4824 4828
482c 4830 4834 4837 483b 483c 4841 4844
4847 4848 484a 484e 4850 4854 4858 485b
485d 4861 4865 4868 486c 4870 4873 4876
4877 4879 487c 4881 4882 4887 488b 488f
4893 4897 489b 489f 48a3 48a7 48ab 48af
48b3 48b7 48bb 48bf 48c3 48c7 48cb 48cf
48d3 48df 48e3 48e7 48ea 48ee 48f2 48f4
48f8 48fc 4900 4903 4907 490b 490d 490e
4913 1 4917 491b 491f 4922 4926 492a
492e 4932 4935 4939 493d 4940 4943 4948
494b 494e 4951 4952 4957 4958 1 495a
495f 4963 4967 496a 496f 4974 4977 497b
497c 4981 4984 4989 498a 498f 4992 4997
4998 499d 49a0 49a4 49a5 49aa 49ad 49b2
49b3 49b8 49bb 49bf 49c3 49c6 49c9 49ca
49cc 49cd 49d2 49d3 49d8 49da 49de 49e1
49e3 49e7 49e9 49f5 49f7 49f9 49fa 49ff
4a03 4a05 4a11 4a13 4a17 4a1a 4a1e 4a22
4a25 4a29 4a2b 4a2f 4a33 4a37 4a3b 4a3f
4a43 4a47 4a4b 4a4f 4a53 4a57 4a5b 4a5f
4a63 4a67 4a6b 4a6f 4a73 4a77 4a7b 4a87
4a8b 4a8f 4a92 4a96 4a9a 4a9c 4aa0 4aa4
4aa8 4aab 4aaf 4ab3 4ab5 4ab6 4abb 1
4abf 4ac3 4ac7 4aca 4ace 4ad2 4ad6 4ada
4add 4ae1 4ae5 4ae8 4aeb 4af0 4af3 4af6
4af9 4afa 4aff 4b00 1 4b02 4b07 4b0b
4b0f 4b12 4b17 4b1c 4b1f 4b23 4b24 4b29
4b2c 4b31 4b32 4b37 4b3a 4b3f 4b40 4b45
4b48 4b4c 4b4d 4b52 4b55 4b5a 4b5b 4b60
4b63 4b67 4b6b 4b6e 4b71 4b72 4b74 4b75
4b7a 4b7b 4b80 4b82 4b86 4b89 4b8b 4b8f
4b91 4b9d 4b9f 4ba1 4ba2 4ba7 4bab 4bad
4bb9 4bbb 4bbf 4bc3 4bc6 4bca 4bcd 4bd1
4bd4 4bd5 4bda 4bdd 4be1 4be5 4be9 4bed
4bf1 4bf5 4bf9 4bfd 4c01 4c05 4c09 4c0d
4c11 4c15 4c19 4c1d 4c21 4c25 4c29 4c2d
4c39 4c3d 4c41 4c44 4c48 4c4c 4c4e 4c52
4c56 4c5a 4c5d 4c61 4c65 4c67 4c68 4c6d
1 4c71 4c75 4c79 4c7c 4c80 4c84 4c88
4c8c 4c8f 4c93 4c97 4c9a 4c9d 4ca2 4ca5
4ca8 4cab 4cac 4cb1 4cb2 1 4cb4 4cb9
4cbd 4cc1 4cc4 4cc9 4cce 4cd1 4cd5 4cd6
4cdb 4cde 4ce3 4ce4 4ce9 4cec 4cf1 4cf2
4cf7 4cfa 4cfe 4cff 4d04 4d07 4d0c 4d0d
4d12 4d15 4d19 4d1d 4d20 4d23 4d24 4d26
4d27 4d2c 4d2d 4d32 4d34 4d38 4d3b 4d3d
4d41 4d43 4d4f 4d51 4d53 4d54 4d59 4d5d
4d5f 4d6b 4d6d 4d6f 4d73 4d76 4d78 4d7c
4d80 4d83 4d87 4d8b 4d8f 4d93 4d97 4d9a
4d9d 4d9e 4da0 4da3 4da8 4da9 4dae 4db2
4db6 4dba 4dbd 4dc1 4dc3 4dc7 4dcb 4dcf
4dd2 4dd3 4dd5 4dd9 4ddb 4ddf 4de3 4de6
4dea 4dee 4df2 4df6 4dfa 4dfe 4e02 4e06
4e0a 4e0e 4e12 4e16 4e1a 4e1e 4e22 4e26
4e2a 4e2e 4e32 4e36 4e3a 4e3e 4e42 4e46
4e4a 4e4e 4e52 4e56 4e5a 4e5e 4e62 4e66
4e6a 4e6d 4e70 4e73 4e76 4e77 4e79 4e7d
4e81 4e84 4e87 4e88 4e8d 4e91 4e95 4e99
4e9d 4ea9 4ead 4eb2 4eb6 4eb9 4eba 4ebf
4ec3 4ec7 4eca 4ecd 4ed1 4ed3 4ed7 4edb
4ede 4ee1 4ee5 4ee7 4eeb 4eef 4ef2 4ef4
4ef8 4efc 4f00 4f03 4f07 4f0b 4f0d 4f0e
4f13 1 4f17 4f1b 4f1f 4f22 4f26 4f2a
4f2e 4f32 4f35 4f39 4f3d 4f40 4f43 4f48
4f4b 4f4e 4f51 4f52 4f57 4f58 1 4f5a
4f5f 4f63 4f67 4f6a 4f6f 4f74 4f77 4f7b
4f7c 4f81 4f84 4f89 4f8a 4f8f 4f92 4f97
4f98 4f9d 4fa0 4fa4 4fa5 4faa 4fad 4fb2
4fb3 4fb8 4fbb 4fbf 4fc3 4fc6 4fc9 4fca
4fcc 4fcd 4fd2 4fd3 4fd8 4fda 4fde 4fe1
4fe3 4fe7 4fe9 4ff5 4ff7 4ff9 4ffa 4fff
5003 5005 5011 5013 5015 5019 501c 5020
5024 5028 502b 502c 502e 5031 5034 5035
503a 503e 5040 5044 504b 504d 5051 5054
5056 505a 505e 5061 5065 5068 506b 506c
5071 5074 5078 507c 507f 5080 5082 5085
5088 5089 508e 1 5091 5096 509a 509e
50a1 50a5 50a9 50ac 50af 50b4 50b7 50ba
50bd 50be 50c3 50c4 1 50c6 50cb 50cf
50d3 50d6 50db 50e0 50e3 50e7 50e8 50ed
50f0 50f5 50f6 50fb 50fe 5103 5104 5109
510c 5110 5114 5118 511c 511f 5120 5122
5127 5128 512a 512b 5130 5133 5138 5139
513e 5141 5146 5147 514c 514f 5153 5154
5159 515c 5161 5162 5167 5168 516d 516f
5173 5176 517a 517e 5181 5185 5189 518c
518f 5194 5197 519a 519d 519e 51a3 51a4
1 51a6 51ab 51af 51b3 51b6 51bb 51c0
51c3 51c7 51c8 51cd 51ce 51d3 51d5 51d9
51dc 51e0 51e4 51e7 51eb 51ef 51f2 51f6
51fa 51fb 5200 5202 5206 5209 520d 5211
5214 5215 5217 521a 521d 521e 5223 5227
522b 522f 5231 5235 5238 523c 5240 5244
5245 5247 524a 524b 524d 5250 5253 5254
5259 525c 5260 5264 5268 5269 526b 526e
526f 5271 5274 5277 5278 527d 1 5280
5285 5289 528e 5291 5296 5297 529c 529f
52a4 52a5 52aa 52ad 52b2 52b3 52b8 52bb
52c0 52c1 52c6 52c9 52cd 52d1 52d5 52d8
52d9 52db 52dc 52de 52df 52e4 52e7 52ec
52ed 52f2 52f5 52f9 52fd 5301 5304 5305
5307 5308 530a 530b 5310 5313 5318 5319
531e 5321 5325 5329 532d 5330 5331 5333
5334 5336 5337 533c 533f 5344 5345 534a
534d 5352 5353 5358 535b 5360 5361 5366
5369 536e 536f 5374 5377 537c 537d 5382
5385 538a 538b 5390 5393 5397 5398 539d
53a0 53a5 53a6 53ab 53ae 53b2 53b3 53b8
53bb 53c0 53c1 53c6 53c9 53ce 53cf 53d4
53d7 53dc 53dd 53e2 53e5 53ea 53eb 53f0
53f3 53f7 53fb 53fe 53ff 5404 5407 540c
540d 5412 5415 5419 541d 5420 5421 5426
5429 542e 542f 5434 5437 543c 543d 5442
5445 544a 544b 5450 5453 5457 545b 545f
5462 5463 5465 5466 5468 5469 546e 5471
5476 5477 547c 547f 5483 5487 548b 548e
548f 5491 5492 5494 5495 549a 549d 54a2
54a3 54a8 54ab 54b0 54b1 54b6 54b9 54bd
54c1 54c5 54c8 54c9 54cb 54cc 54ce 54cf
54d4 54d7 54dc 54dd 54e2 54e5 54ea 54eb
54f0 54f3 54f7 54f8 54fd 5500 5505 5506
550b 550e 5512 5513 5518 551b 5520 5521
5526 5529 552e 552f 5534 5537 553c 553d
5542 5546 554a 554c 5550 5554 5558 5559
555b 555e 555f 5561 5564 5567 5568 556d
5571 5576 5579 557e 557f 5584 5587 558c
558d 5592 5595 559a 559b 55a0 55a3 55a8
55a9 55ae 55b1 55b5 55b9 55bd 55c0 55c1
55c3 55c4 55c6 55c7 55cc 55cf 55d4 55d5
55da 55dd 55e1 55e5 55e9 55ec 55ed 55ef
55f0 55f2 55f3 55f8 55fb 5600 5601 5606
5609 560d 5611 5615 5618 5619 561b 561c
561e 561f 5624 5627 562c 562d 5632 5635
563a 563b 5640 5643 5648 5649 564e 5651
5656 5657 565c 565f 5664 5665 566a 566d
5672 5673 5678 567b 567f 5680 5685 5688
568d 568e 5693 5696 569a 569b 56a0 56a3
56a8 56a9 56ae 56b1 56b6 56b7 56bc 56c0
56c4 56c6 56ca 56ce 56d2 56d6 56d7 56d9
56dc 56dd 56df 56e2 56e5 56e6 56eb 56ef
56f4 56f7 56fc 56fd 5702 5705 5709 570d
5710 5711 5716 5719 571e 571f 5724 5727
572b 572f 5732 5733 5738 573b 5740 5741
5746 5749 574e 574f 5754 5757 575c 575d
5762 5765 5769 576d 5771 5774 5775 5777
5778 577a 577b 5780 5783 5788 5789 578e
5791 5795 5799 579d 57a0 57a1 57a3 57a4
57a6 57a7 57ac 57af 57b4 57b5 57ba 57bd
57c2 57c3 57c8 57cb 57cf 57d3 57d7 57da
57db 57dd 57de 57e0 57e1 57e6 57e9 57ee
57ef 57f4 57f7 57fc 57fd 5802 5805 5809
580a 580f 5812 5817 5818 581d 5820 5824
5825 582a 582d 5832 5833 5838 583b 5840
5841 5846 5849 584e 584f 5854 5858 585a
585e 5862 5865 5869 586d 5870 5874 5878
587b 587e 5883 5886 5889 588c 588d 5892
5893 1 5895 589a 589e 58a2 58a5 58aa
58af 58b2 58b6 58b7 58bc 58bd 58c2 58c4
58c8 58cb 58cf 58d3 58d6 58da 58de 58e1
58e4 58e9 58ec 58ef 58f2 58f3 58f8 58f9
1 58fb 5900 5904 5908 590b 5910 5915
5918 591c 591d 5922 5923 5928 592a 592e
5931 5935 5939 593d 5940 5944 5945 5947
594b 594d 5951 5955 5958 595c 5960 5963
5967 596b 596e 5971 5976 5979 597c 597f
5980 5985 5986 1 5988 598d 5991 5995
5998 599d 59a2 59a5 59a9 59aa 59af 59b0
59b5 59b9 59bd 59c0 59c5 59ca 59cd 59d1
59d5 59d8 59d9 59de 59e1 59e6 59e7 59ec
59ef 59f3 59f7 59fa 59fb 5a00 5a01 5a06
5a08 5a0c 5a0f 5a13 5a17 5a1b 5a1f 5a23
5a27 5a2b 5a2f 5a33 5a3f 5a44 5a48 5a4c
5a50 5a54 5a56 5a57 5a5c 1 5a60 5a64
5a68 5a6b 5a6f 5a73 5a77 5a7b 5a7e 5a82
5a86 5a89 5a8c 5a91 5a94 5a97 5a9a 5a9b
5aa0 5aa1 1 5aa3 5aa8 5aac 5ab0 5ab3
5ab8 5abd 5ac0 5ac4 5ac5 5aca 5acb 5ad0
5ad4 5ad8 5adb 5ae0 5ae5 5ae8 5aec 5aed
5af2 5af5 5afa 5afb 5b00 5b03 5b07 5b0b
5b0e 5b11 5b12 5b14 5b15 5b1a 5b1b 5b20
5b22 5b26 5b29 5b2d 5b31 5b35 5b39 5b3d
5b41 5b45 5b51 5b56 5b5a 5b5e 5b62 5b66
5b68 5b6c 5b6e 5b7a 5b7c 5b7e 5b7f 5b84
5b88 5b8a 5b96 5b9a 5b9c 5b9d 5ba6 
1a7f
2
0 1 a 6 d :3 6 14 :3 6
15 :3 6 10 :3 6 e :3 6 11 :3 6
c :3 6 b :3 6 11 :3 6 11 :3 6
1d :3 6 10 :3 6 e :3 6 10 :3 6
12 :2 6 1 3 a 13 a :2 1
4 11 1a 19 :2 11 :2 4 e 17
16 :2 e :2 4 f 18 17 :2 f :2 4
d 16 15 :2 d :2 4 11 1a 19
:2 11 :2 4 f 18 17 :2 f :2 4 10
19 18 :2 10 :2 4 :3 10 :2 4 11 1a
19 :2 11 :2 4 f 18 17 :2 f :2 4
13 1c 1b :2 13 :2 4 10 19 18
:2 10 :2 4 18 21 20 :2 18 :2 4 10
19 18 :2 10 :2 4 f 18 17 :2 f
:2 4 10 19 18 :2 10 :2 4 :3 9 :2 4
:3 9 :2 4 10 19 18 :2 10 :2 4 :2 c
1b c :2 4 :2 e 1d e :2 4 :2 b
1a b :2 4 :2 d 1c d :2 4 :2 10
1f 10 :2 4 :2 a 19 a :2 4 :2 b
1a b :2 4 :2 11 20 11 :2 4 :2 d
1c d :2 4 :2 c 1b c :2 4 :2 b
1a b :2 4 :2 f 1e f :2 4 :2 e
1d e :2 4 :2 c 1b c :2 4 :2 e
1d e :2 4 :2 f 1e f :2 4 :2 b
1a b :2 4 :2 f 1e f :2 4 :2 d
1c d :2 4 :2 c 1b c :2 4 :2 c
1b c :2 4 :2 f 1e f :2 4 :2 b
1a b :2 4 :2 f 19 f :2 4 :2 f
1e f :2 4 :2 16 25 16 :2 4 :2 11
20 11 :2 4 :2 f 19 f :2 4 :2 f
19 f :2 4 :2 17 26 17 :2 4 17
1c 1b :2 17 :2 4 18 1d 1c :2 18
:2 4 :2 13 22 13 :2 4 :2 14 23 14
:2 4 9 12 11 :2 9 :2 4 :2 11 20
11 :2 4 :2 f 1e f :2 4 :2 6 1e
6 :2 4 :2 7 1f 7 :2 4 :2 d 25
d :2 4 :2 8 20 8 :2 4 :3 d :2 4
e 17 :3 e 4 7 :2 11 23 :2 2d
3a :2 23 :3 7 :2 d 19 25 :2 2f 43
:2 25 53 59 5c :2 53 6a 7f 8
:2 6a d 10 :2 6a 17 1a :2 6a b
e :2 6a 13 16 :2 6a 1b 1e :2 6a
2c 2f :2 6a b e :2 6a 13 16
:2 6a 1b 1e :2 6a 2d 30 :2 6a b
e :2 6a 13 16 1f 27 :2 1f 33
:2 16 :2 6a b e :2 6a 13 16 1f
27 :2 1f 31 :2 16 :2 6a b e :2 6a
13 16 1f 27 :2 1f 34 :2 16 :2 6a
b e :2 6a 13 16 1f 27 :2 1f
2f :2 16 :2 6a b e :2 6a 13 16
1f 27 :2 1f 2e :2 16 :2 6a b e
:2 6a 13 16 1f 27 :2 1f 34 :2 16
:2 6a b e :2 6a 13 16 1f 27
:2 1f 34 :2 16 :2 6a b e :2 6a 13
16 1f 27 :2 1f 40 :2 16 :2 6a b
e :2 6a 13 16 1f 27 :2 1f 33
:2 16 :2 6a b e :2 6a 13 16 1f
27 :2 1f 31 :2 16 :2 6a b e :2 6a
13 16 :2 6a 1b 1e :2 6a 28 2b
:2 6a b e :2 6a 13 16 :2 6a 1b
1e :2 6a 2a 2d :2 6a 32 35 :2 6a
:2 7 49 :2 4 7 :2 11 23 :2 2d 39
3c 4b 4e 4f :2 4b :2 23 :3 7 :2 d
12 1e 5c 5f :2 1e :2 7 52 :3 4
14 :2 4 14 :2 4 13 :2 4 12 :2 4
10 :2 4 e :2 4 14 :2 4 f :2 4
12 :2 4 e :2 4 e :2 4 11 :2 4
11 :2 4 13 :2 4 11 :2 4 14 1d
26 2e :2 1d :2 14 :2 4 12 :2 4 10
4 :4 8 7 a :2 14 26 :2 30 3c
3f 4e 51 52 :2 4e :2 26 :3 a :2 10
15 21 5f 62 :2 21 6f 72 :2 21
76 79 :2 21 83 86 8f 97 :2 a0
:2 8f b6 :2 86 :2 21 :2 a 55 :2 7 a
:2 14 26 :2 30 3c 3f 4e 51 52
:2 4e :2 26 :3 a :2 10 15 21 5f 62
:2 21 :2 a 55 :3 7 :2 12 28 :2 31 46
:2 7 18 :2 4 7 10 1b :2 7 1e
20 :2 1e 26 2f 37 :2 26 3a 3c
:2 3a :2 7 42 4b 53 :2 42 56 58
:2 56 :2 7 a :2 14 26 :2 30 3c 3f
4e 51 52 :2 4e :2 26 :3 a :2 10 15
21 5f 62 :2 21 6f 72 :2 21 76
79 :2 21 83 86 8f 97 :2 a0 :2 8f
b1 :2 86 :2 21 :2 a 55 :2 7 a :2 14
26 :2 30 3c 3f 4e 51 52 :2 4e
:2 26 :3 a :2 10 15 21 5f 62 :2 21
:2 a 55 :3 7 :2 12 28 :2 31 41 :2 7
5a :2 4 8 11 1c :2 8 22 1f
:2 2b :2 1f 7 :4 46 45 44 :2 7 :4 14
13 12 :2 7 11 1c 21 33 3e
43 :2 f :2 a :2 13 1f a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 55 58 59 :2 55 :2 2c :3 10 :2 16
1b 27 65 68 :2 27 75 78 :2 27
7c 7f :2 27 8b 8e :2 27 96 99
:2 27 a5 a8 af b8 bb :2 a8 :2 27
:2 10 5c :2 d :2 a :4 16 :2 a 7 :3 31
b :2 14 1f 1d :2 28 :2 1d a 3d
46 4f :2 3d 52 54 :2 52 3c :2 a
d :2 17 29 :2 33 3f 42 51 54
55 :2 51 :2 29 :3 d :2 13 18 24 62
65 :2 24 72 75 :2 24 79 7c :2 24
86 89 92 9a :2 a3 :2 92 c0 :2 89
:2 24 :2 d 58 :2 a d :2 17 29 :2 33
3f 42 51 54 55 :2 51 :2 29 :3 d
:2 13 18 24 62 65 :2 24 :2 d 58
:3 a :2 15 2b :2 34 50 :2 a 57 :3 7
e 7 4 31 :4 c b a :4 2a
29 28 :2 a 11 23 f :2 a :2 13
1f a 7 f d :2 16 22 d
1d :2 a :2 f d :2 16 22 d 10
:2 1a 2c :2 36 42 46 55 58 59
:2 55 :2 2c :3 10 :2 16 1b 27 65 68
:2 27 75 78 :2 27 7c 7f :2 27 8b
8e :2 27 96 99 :2 27 a5 a8 af
b8 bb :2 a8 :2 27 :2 10 5c :2 d :2 a
:4 16 :2 a 7 :4 43 31 :3 4 12 :2 4
d :2 16 22 29 32 3e :2 29 42
:2 4b :2 22 21 5a :2 d :2 4 13 1a
23 2f :2 1a 33 :2 3c :2 13 12 4
7 10 18 :2 7 1b 1d :2 1b 11
1a 30 39 51 f 1a 26 f
1d 2a :2 a :2 13 1f a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 55 58 59 :2 55 :2 2c :3 10 :2 16
1b 27 65 68 :2 27 75 78 :2 27
7c 7f :2 27 8b 8e :2 27 96 99
:2 27 a5 a8 af b8 bb :2 a8 :2 27
:2 10 5c :2 d :2 a :4 16 :2 a 7 :3 1f
b :2 14 1f 1d :2 28 :2 1d a 26
31 3d 4d 5d 6a :2 d :2 16 22
d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 58 5b 5c :2 58
:2 2f :3 13 :2 19 1e 2a 68 6b :2 2a
78 7b :2 2a 7f 82 :2 2a 8e 91
:2 2a 99 9c :2 2a a8 ab b2 bb
be :2 ab :2 2a :2 13 5f :2 10 :2 d :4 19
:2 d a :3 38 e :2 17 22 20 :2 2b
:2 20 d 10 :2 1a 2c :2 36 42 46
55 58 59 :2 55 :2 2c :3 10 :2 16 1b
27 65 68 :2 27 75 78 :2 27 7c
7f :2 27 89 8c 95 9d :2 a6 :2 95
b7 :2 8c :2 27 bf c2 :2 27 ca cd
:2 27 d2 d5 :2 27 e2 e5 :2 27 :2 10
5c :2 d 10 :2 1a 2c :2 36 42 45
54 57 58 :2 54 :2 2c :3 10 :2 16 1b
27 65 68 :2 27 :2 10 5b :3 d :2 18
2c :2 35 45 53 :2 d 3b :2 a 38
:3 7 15 7 11 21 f 10 1d
:2 a :2 13 1f a 7 f d :2 16
22 d 1d :2 a :2 f d :2 16 22
d 10 :2 1a 2c :2 36 42 46 55
58 59 :2 55 :2 2c :3 10 :2 16 1b 27
65 68 :2 27 75 78 :2 27 7c 7f
:2 27 8b 8e :2 27 96 99 :2 27 a5
a8 af b8 bb :2 a8 :2 27 :2 10 5c
:2 d :2 a :4 16 :2 a 7 :3 1f a 13
1f :2 a 22 24 :2 22 14 24 12
13 20 :2 d :2 16 22 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 58 5b 5c :2 58 :2 2f :3 13 :2 19
1e 2a 68 6b :2 2a 78 7b :2 2a
7f 82 :2 2a 8e 91 :2 2a 99 9c
:2 2a a8 ab b2 bb be :2 ab :2 2a
:2 13 5f :2 10 :2 d :4 19 :2 d a :3 26
d :2 18 1f 2d :2 d 32 34 :2 32
10 :2 1a 2c :2 36 42 46 55 58
59 :2 55 :2 2c :3 10 :2 16 1b 27 65
68 :2 27 75 78 :2 27 7c 7f :2 27
89 8c 95 9d :2 a6 :2 95 bb :2 8c
:2 27 c3 c6 :2 27 ce d1 :2 27 d6
d9 :2 27 e6 e9 :2 27 :2 10 5c :2 d
10 :2 1a 2c :2 36 42 45 54 57
58 :2 54 :2 2c :3 10 :2 16 1b 27 65
68 :2 27 :2 10 5b :3 d :2 18 2c :2 35
49 57 :2 d 36 :2 a 26 :2 7 1f
:4 b a d :2 17 29 :2 33 3f 42
51 54 55 :2 51 :2 29 :3 d :2 13 18
24 62 65 :2 24 72 75 :2 24 79
7c :2 24 86 89 92 9a :2 a3 :2 92
b9 :2 89 :2 24 :2 d 58 :2 a d :2 17
29 :2 33 3f 42 51 54 55 :2 51
:2 29 :3 d :2 13 18 24 62 65 :2 24
:2 d 58 :3 a :2 15 2b :2 34 49 :2 a
1e :3 7 16 :2 7 13 1a 21 2e
31 :2 1a 36 39 :2 13 :2 7 15 :2 1e
27 2a 2d 31 :2 15 7 a 18
21 29 :2 18 :2 a 2d 30 :2 2d 14
20 29 3f 48 60 12 20 2b
37 12 20 2d 3b 47 50 5b
64 70 79 87 90 9e a7 :2 d
:2 16 22 d a 12 10 :2 19 25
10 20 :2 d :2 12 10 :2 19 25 10
13 :2 1d 2f :2 39 45 49 58 5b
5c :2 58 :2 2f :3 13 :2 19 1e 2a 68
6b :2 2a 78 7b :2 2a 7f 82 :2 2a
8e 91 :2 2a 99 9c :2 2a a8 ab
b2 bb be :2 ab :2 2a :2 13 5f :2 10
:2 d :4 19 :2 d a :3 32 e :2 17 22
20 :2 2b :2 20 d 10 :2 1a 2c :2 36
42 46 55 58 59 :2 55 :2 2c :3 10
:2 16 1b 27 65 68 :2 27 75 78
:2 27 7c 7f :2 27 89 8c 95 9d
:2 a6 :2 95 b9 :2 8c :2 27 c1 c4 :2 27
cc cf :2 27 d4 d7 :2 27 e1 e4
:2 27 :2 10 5c :2 d 10 :2 1a 2c :2 36
42 45 54 57 58 :2 54 :2 2c :3 10
:2 16 1b 27 65 68 :2 27 :2 10 5b
:3 d :2 18 2c :2 35 47 52 :2 d 3b
:2 a 32 d :2 17 29 :2 33 3f 43
52 55 56 :2 52 :2 29 :3 d :2 13 18
24 62 65 :2 24 72 75 :2 24 79
7c :2 24 86 89 92 9a :2 a3 :2 92
b6 :2 89 :2 24 be c1 :2 24 c9 cc
:2 24 d1 d4 :2 24 de e1 :2 24 :2 d
59 :2 a d :2 17 29 :2 33 3f 42
51 54 55 :2 51 :2 29 :3 d :2 13 18
24 62 65 :2 24 :2 d 58 :3 a :2 15
29 :2 32 44 4f :2 a :4 7 :4 4 8
11 1d :2 8 20 22 :2 20 7 2a
33 3b :2 2a 3e 41 :2 3e 29 :3 7
11 7 46 :2 4 7 b d :2 b
7 15 :2 7 16 :2 7 15 26 32
3f 48 51 :2 3f 55 5e 66 :2 55
22 2b 37 :2 22 3b 44 4f :2 3b
53 20 2b :2 53 2f 38 46 :2 2f
4a 53 5e :2 4a 22 2b 37 :2 22
3b 43 4d :2 15 7 a 13 1f
:2 a 24 22 :2 2d :2 22 d :2 17 29
:2 33 3f 42 51 54 55 :2 51 :2 29
:3 d :2 13 18 24 62 65 :2 24 72
75 :2 24 79 7c :2 24 86 89 92
9a :2 a3 :2 92 b4 :2 89 :2 24 :2 d 58
:2 a d :2 17 29 :2 33 3f 42 51
54 55 :2 51 :2 29 :3 d :2 13 18 24
62 65 :2 24 :2 d 58 :3 a :2 15 2b
:2 34 44 :2 a 7 3c d 16 22
:2 d 25 28 :2 25 d :2 17 29 :2 33
3f 43 52 55 56 :2 52 :2 29 :3 d
:2 13 18 24 62 65 :2 24 72 75
:2 24 79 7c :2 24 86 89 92 9a
:2 92 a7 :2 89 :2 24 af b2 :2 24 ba
bd :2 24 c2 c5 :2 24 d2 d5 :2 24
:2 d 59 :2 a d :2 17 29 :2 33 3f
42 51 54 55 :2 51 :2 29 :3 d :2 13
18 24 62 65 :2 24 :2 d 58 :3 a
:2 15 29 32 3e :2 29 42 50 :2 a
2a 3c :2 7 f :2 4 8 11 19
:2 8 1c 1e :2 1c 7 26 2f 37
:2 26 3a 3d :2 3a 25 :2 7 b 14
1c :2 b 1f 21 :2 1f a 28 31
39 :2 28 3c 3e :2 3c 27 :3 a 15
a 7 41 d 16 1e :2 d 21
23 :2 21 a 15 a 7 25 41
e 17 1f :2 e 22 24 :2 22 d
2b 34 3c :2 2b 3f 41 :2 3f 2a
:2 d a 15 a 7 44 41 d
16 1e :2 d 21 23 :2 21 a 14
1d 29 :2 14 2c 2e 37 3f :2 2e
:2 14 :2 a 15 a 25 41 :2 7 42
:2 4 7 10 1a :2 7 1d 1f :2 1d
a 13 1f :2 a 22 24 :2 22 d
:2 17 29 :2 33 3f 43 52 55 56
:2 52 :2 29 :3 d :2 13 18 24 62 65
:2 24 72 75 :2 24 79 7c :2 24 86
89 92 9a :2 a3 :2 92 ba :2 89 :2 24
c2 c5 :2 24 cd d0 :2 24 d5 d8
:2 24 e5 e8 :2 24 :2 d 59 :2 a d
:2 17 29 :2 33 3f 42 51 54 55
:2 51 :2 29 :3 d :2 13 18 24 62 65
:2 24 :2 d 58 :3 a :2 15 29 :2 32 48
56 :2 a 29 :2 7 a 13 1f :2 a
22 24 :2 22 a 18 a 27 a
18 1e 28 31 3d :2 28 :2 1e 41
43 :2 1e :2 18 a :4 7 21 :3 4 :3 17
:2 4 12 :2 4 13 :2 4 13 :2 4 13
:2 4 10 17 21 24 :2 10 :2 4 12
19 25 28 :2 12 4 7 10 18
:2 7 1b 1d :2 1b 7 f :2 7 15
7 a 11 1d 20 :2 a 23 25
:2 23 14 27 2f 36 51 59 60
7c 86 d 1b 12 27 2d 33
3f d 19 12 13 1b 2c 38
45 5b :2 d :2 16 22 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 58 5b 5c :2 58 :2 2f :3 13 :2 19
1e 2a 68 6b :2 2a 78 7b :2 2a
7f 82 :2 2a 8e 91 :2 2a 99 9c
:2 2a a8 ab b2 bb be :2 ab :2 2a
:2 13 5f :2 10 :2 d :4 19 :2 d a :3 29
14 26 37 46 53 :2 d :2 16 22
d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 58 5b 5c :2 58
:2 2f :3 13 :2 19 1e 2a 68 6b :2 2a
78 7b :2 2a 7f 82 :2 2a 8e 91
:2 2a 99 9c :2 2a a8 ab b2 bb
be :2 ab :2 2a :2 13 5f :2 10 :2 d :4 19
:2 d a :3 29 e :2 17 22 20 :2 2b
:2 20 d 10 :2 1a 2c :2 36 42 46
55 58 59 :2 55 :2 2c :3 10 :2 16 1b
27 65 68 :2 27 75 78 :2 27 7c
7f :2 27 89 8c 95 9d :2 a6 :2 95
b9 :2 8c :2 27 c1 c4 :2 27 cc cf
:2 27 d4 d7 :2 27 e0 e3 :2 27 :2 10
5c :2 d 10 :2 1a 2c :2 36 42 45
54 57 58 :2 54 :2 2c :3 10 :2 16 1b
27 65 68 :2 27 :2 10 5b :3 d :2 18
2c :2 35 47 51 :2 d 3b :3 a 19
a 14 27 38 45 12 13 20
2e 46 54 :2 d :2 16 22 d a
12 10 :2 19 25 10 20 :2 d :2 12
10 :2 19 25 10 13 :2 1d 2f :2 39
45 49 58 5b 5c :2 58 :2 2f :3 13
:2 19 1e 2a 68 6b :2 2a 78 7b
:2 2a 7f 82 :2 2a 8e 91 :2 2a 99
9c :2 2a a8 ab b2 bb be :2 ab
:2 2a :2 13 5f :2 10 :2 d :4 19 :2 d a
:3 29 7 29 d 14 20 23 :2 d
26 28 :2 26 14 23 34 44 52
:2 d :2 16 22 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 58
5b 5c :2 58 :2 2f :3 13 :2 19 1e 2a
68 6b :2 2a 78 7b :2 2a 7f 82
:2 2a 8e 91 :2 2a 99 9c :2 2a a8
ab b2 bb be :2 ab :2 2a :2 13 5f
:2 10 :2 d :4 19 :2 d a :3 2c e :2 17
22 20 :2 2b :2 20 d 10 :2 1a 2c
:2 36 42 46 55 58 59 :2 55 :2 2c
:3 10 :2 16 1b 27 65 68 :2 27 75
78 :2 27 7c 7f :2 27 89 8c 95
9d :2 a6 :2 95 b9 :2 8c :2 27 c1 c4
:2 27 cc cf :2 27 d4 d7 :2 27 e0
e3 :2 27 :2 10 5c :2 d 10 :2 1a 2c
:2 36 42 45 54 57 58 :2 54 :2 2c
:3 10 :2 16 1b 27 65 68 :2 27 :2 10
5b :3 d :2 18 2c :2 35 47 51 :2 d
3b :3 a 18 a 14 27 2f 36
51 59 60 7c 86 d 1b 12
27 2d 33 3f d 19 12 13
1b 2c 38 45 5b :2 d :2 16 22
d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 58 5b 5c :2 58
:2 2f :3 13 :2 19 1e 2a 68 6b :2 2a
78 7b :2 2a 7f 82 :2 2a 8e 91
:2 2a 99 9c :2 2a a8 ab b2 bb
be :2 ab :2 2a :2 13 5f :2 10 :2 d :4 19
:2 d a :3 2c e :2 17 22 20 :2 2b
:2 20 d 17 2a 32 39 54 5c
63 7f 89 10 1e 15 2a 30
36 42 10 1c 15 16 1e 2f
3b 48 5e :2 10 :2 19 25 10 d
15 13 :2 1c 28 13 23 :2 10 :2 15
13 :2 1c 28 13 16 :2 20 32 :2 3c
48 4c 5b 5e 5f :2 5b :2 32 :3 16
:2 1c 21 2d 6b 6e :2 2d 7b 7e
:2 2d 82 85 :2 2d 91 94 :2 2d 9c
9f :2 2d ab ae b5 be c1 :2 ae
:2 2d :2 16 62 :2 13 :2 10 :4 1c :2 10 d
:4 3b :2 a 14 27 38 45 12 13
20 2e 46 54 :2 d :2 16 22 d
a 12 10 :2 19 25 10 20 :2 d
:2 12 10 :2 19 25 10 13 :2 1d 2f
:2 39 45 49 58 5b 5c :2 58 :2 2f
:3 13 :2 19 1e 2a 68 6b :2 2a 78
7b :2 2a 7f 82 :2 2a 8e 91 :2 2a
99 9c :2 2a a8 ab b2 bb be
:2 ab :2 2a :2 13 5f :2 10 :2 d :4 19 :2 d
a :4 2c 29 d :2 17 29 :2 33 3f
42 51 54 55 :2 51 :2 29 :3 d :2 13
18 24 62 65 :2 24 72 75 :2 24
79 7c :2 24 86 89 92 9a :2 a3
:2 92 b4 :2 89 :2 24 :2 d 58 :2 a d
:2 17 29 :2 33 3f 42 51 54 55
:2 51 :2 29 :3 d :2 13 18 24 62 65
:2 24 :2 d 58 :3 a :2 15 2b :2 34 44
:2 a :4 7 b :2 14 1f 1d :2 28 :2 1d
:2 a 1d 2b 2e 37 3f :2 2e 43
:2 4c 24 :2 1d :2 a 1d 2b 2e 37
3f :2 2e 43 :2 4c 24 :2 1d a 14
2d 12 20 38 49 52 5d 66
72 7b 89 92 a0 a9 bd c6
12 20 39 42 12 20 38 41
12 1b 23 30 39 :2 12 20 3a
12 13 27 d 14 10 1c :3 1a
10 :2 19 25 10 21 10 :2 19 25
10 :4 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 58 5b 5c
:2 58 :2 2f :3 13 :2 19 1e 2a 68 6b
:2 2a 78 7b :2 2a 7f 82 :2 2a 8e
91 :2 2a 99 9c :2 2a a8 ab b2
bb be :2 ab :2 2a :2 13 5f :2 10 :2 d
:4 19 :2 d a :3 38 a 1d 33 :2 1d
a 38 a :3 19 :2 a 1d 2b 2e
37 3f :2 2e 43 :2 4c 24 :2 1d :2 a
20 :2 29 :2 a 1c :2 a 1d 34 42
4c 54 2d 41 4d :2 1d a 19
2e 41 4d 59 67 d 1c 29
36 43 4d 15 22 30 3b 43
d 19 23 2c 39 4d 5a d
14 10 1c :3 1a 10 :2 19 25 10
21 10 :2 19 25 10 :4 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 58 5b 5c :2 58 :2 2f :3 13 :2 19
1e 2a 68 6b :2 2a 78 7b :2 2a
7f 82 :2 2a 8e 91 :2 2a 99 9c
:2 2a a8 ab b2 bb be :2 ab :2 2a
:2 13 5f :2 10 :2 d :4 19 :2 d a :7 7
b 14 21 :2 b 24 26 :2 24 a
:4 2f 2e 2d :2 a 14 36 4a 60
6d :2 d :2 16 22 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
58 5b 5c :2 58 :2 2f :3 13 :2 19 1e
2a 68 6b :2 2a 78 7b :2 2a 7f
82 :2 2a 8e 91 :2 2a 99 9c :2 2a
a8 ab b2 bb be :2 ab :2 2a :2 13
5f :2 10 :2 d :4 19 :2 d a :3 42 d
14 24 27 :2 d 2a 2c :2 2a d
1d 1e :2 1d :2 d :2 16 22 39 47
55 6a 2f 35 41 4d 2f 42
:2 22 d 30 :2 a 42 :3 7 15 7
a 11 1d 20 :2 a 23 25 :2 23
a 15 :2 1e a 29 a 15 1e
2a :2 15 a :5 7 13 1f 2c 3a
45 4f 5b 1b 28 35 42 4f
1b 28 34 3e 46 4f 57 5e
1b 27 34 3f 4a 58 1b 26
32 3e 41 44 4a :2 13 7 a
13 15 :2 13 14 27 44 4b d
14 10 1c :3 1a 10 :2 19 25 10
21 10 :2 19 25 10 :4 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 58 5b 5c :2 58 :2 2f :3 13 :2 19
1e 2a 68 6b :2 2a 78 7b :2 2a
7f 82 :2 2a 8e 91 :2 2a 99 9c
:2 2a a8 ab b2 bb be :2 ab :2 2a
:2 13 5f :2 10 :2 d :4 19 :2 d a :4 17
:2 7 1f 7 15 :2 7 16 :2 7 16
7 a 13 1a :2 13 28 :2 a 2b
2d :2 2b a 16 a 30 a 16
1d 26 2d :2 26 3b :2 1d 3e 40
:2 1d 1c 43 45 :2 1c :2 16 a :4 7
a 13 15 :2 13 a 15 :2 a 10
19 22 :2 10 27 25 30 :2 25 a
d 19 :2 22 2b 2f 33 3c 45
:2 33 48 4a :2 33 4c 4d :2 33 50
:2 19 :2 d 1b :2 24 2d 30 33 3c
45 :2 33 48 4a :2 33 4d 4f :2 33
51 52 :2 33 55 :2 1b d 10 17
23 26 :2 10 29 2b :2 29 10 1b
:2 24 10 13 1c 23 :2 1c 31 :2 13
34 36 :2 34 13 22 13 38 13
22 29 35 38 :2 29 41 44 :2 22
:2 13 22 29 35 38 :2 29 42 45
:2 22 13 :4 10 2f 10 1b 24 30
:2 1b 10 13 1c 23 :2 1c 30 :2 13
33 35 :2 33 13 21 13 37 13
21 28 33 36 :2 28 3f 42 :2 21
:2 13 21 28 33 36 :2 28 40 43
:2 21 13 :4 10 :4 d 10 17 23 26
:2 10 29 2b :2 29 24 2c 33 51
59 60 7a 82 89 18 33 40
18 28 19 21 32 3e 4b :2 13
:2 1c 28 13 10 18 16 :2 1f 2b
16 26 :2 13 :2 18 16 :2 1f 2b 16
19 :2 23 35 :2 3f 4b 4f 5e 61
62 :2 5e :2 35 :3 19 :2 1f 24 30 6e
71 :2 30 7e 81 :2 30 85 88 :2 30
94 97 :2 30 9f a2 :2 30 ae b1
b8 c1 c4 :2 b1 :2 30 :2 19 65 :2 16
:2 13 :4 1f :2 13 10 :3 2f 10 1b :2 10
1f 10 2f 24 2c 33 51 59
60 7a 82 89 18 33 40 18
28 19 21 32 3e 4b 61 :2 13
:2 1c 28 13 10 18 16 :2 1f 2b
16 26 :2 13 :2 18 16 :2 1f 2b 16
19 :2 23 35 :2 3f 4b 4f 5e 61
62 :2 5e :2 35 :3 19 :2 1f 24 30 6e
71 :2 30 7e 81 :2 30 85 88 :2 30
94 97 :2 30 9f a2 :2 30 ae b1
b8 c1 c4 :2 b1 :2 30 :2 19 65 :2 16
:2 13 :4 1f :2 13 10 :3 d 14 :2 1d 28
26 :2 31 :2 26 13 27 2f 36 54
5c 63 7d 85 8c 1b 36 43
1b 2b 1c 24 35 41 4e 64
:2 16 :2 1f 2b 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 61
64 65 :2 61 :2 38 :3 1c :2 22 27 33
71 74 :2 33 81 84 :2 33 88 8b
:2 33 97 9a :2 33 a2 a5 :2 33 b1
b4 bb c4 c7 :2 b4 :2 33 :2 1c 68
:2 19 :2 16 :4 22 :2 16 13 :4 41 :2 10 :5 d
1b d 10 17 23 26 :2 10 29
2b :2 29 10 1b :2 24 10 2f 10
1b 24 30 :2 1b 10 :5 d 1b :2 d
19 25 32 40 4b 55 61 1b
28 35 42 4f 1b 28 34 3e
46 4f 57 5e 1b 27 34 3f
4a 58 1b 26 32 3e 41 44
4a :2 19 d 10 19 1b :2 19 1a
2d 4a 51 13 1a 16 22 :3 20
16 :2 1f 2b 16 27 16 :2 1f 2b
16 :4 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 5e 61 62
:2 5e :2 35 :3 19 :2 1f 24 30 6e 71
:2 30 7e 81 :2 30 85 88 :2 30 94
97 :2 30 9f a2 :2 30 ae b1 b8
c1 c4 :2 b1 :2 30 :2 19 65 :2 16 :2 13
:4 1f :2 13 10 :4 1d :3 d 18 21 2a
:2 18 2d 2f :2 18 d 30 e a
17 :2 7 :4 4 8 11 14 :2 11 7
1c 25 2d :2 1c 30 32 :2 30 1b
:2 7 a :2 14 26 :2 30 3c 40 4f
52 53 :2 4f :2 26 :3 a :2 10 15 21
5f 62 :2 21 6f 72 :2 21 76 79
:2 21 83 86 8f 97 :2 a0 :2 8f c1
:2 86 :2 21 c9 cc :2 21 d4 d7 :2 21
dc df :2 21 eb ee :2 21 :2 a 56
:2 7 a :2 14 26 :2 30 3c 3f 4e
51 52 :2 4e :2 26 :3 a :2 10 15 21
5f 62 :2 21 :2 a 55 :3 7 :2 12 26
:2 2f 4f f :2 7 35 :2 4 7 10
18 :2 7 1b 1d :2 1b 7 17 7
1f :2 4 8 11 18 :2 11 25 :2 8
28 2a :2 28 7 32 3b 42 :2 3b
50 :2 32 53 55 :2 53 31 :3 7 d
e 11 :2 d e 11 :2 d e 11
:2 d e 11 :2 d 2c 2f 37 40
4a :2 37 :2 2f :2 d 4e 51 :2 d 13
16 1e 27 30 :2 1e :2 16 :2 d 13
16 :2 d e 11 19 22 2d :2 19
:2 11 :2 d 13 16 :2 d e 11 :2 d
e 11 :2 d e 11 :2 d e 11
:2 d e 11 :2 d 29 2c :2 d 13
16 :2 d 4e 51 :2 d 13 16 :2 d
e 11 :2 d e 11 :2 d e 11
:2 d e 11 :2 1a :2 d 32 35 :2 d
48 4b :2 54 :2 d 13 16 :2 d e
11 :2 d e 11 :2 d 47 4a 52
5b 65 :2 52 :2 4a :2 d 13 16 :2 d
2a 2d 35 3e 47 :2 35 :2 2d :2 d
4b 4e :2 d e 11 :2 d 54 57
5f 13 1e :2 5f :2 57 :2 d 13 16
:2 d e 11 :2 d 4d 50 :2 d 13
16 :2 d 50 53 :2 d 13 16 :2 d
e 11 :2 d e 11 :2 d 7 4
58 a 13 1a :2 13 27 :2 a 2a
2c :2 2a 7 d e 11 :2 d e
11 :2 d e 11 :2 d e 11 :2 d
2c 2f 37 40 4a :2 37 :2 2f :2 d
4e 51 :2 d 13 16 1e 27 30
:2 1e :2 16 :2 d 13 16 :2 d e 11
19 22 2d :2 19 :2 11 :2 d 13 16
:2 d e 11 :2 d e 11 :2 d e
11 :2 d e 11 :2 d e 11 :2 d
29 2c :2 d 13 16 :2 d 4e 51
:2 d 13 16 :2 d e 11 :2 d 7
4 2e 58 a 13 1a :2 13 28
:2 a 2b 2d :2 2b 7 d e 11
:2 d e 11 :2 1a :2 d 32 35 :2 d
48 4b :2 54 :2 d 13 16 :2 d e
11 :2 d e 11 :2 d 47 4a 52
5b 65 :2 52 :2 4a :2 d 13 16 :2 d
2a 2d 35 3e 47 :2 35 :2 2d :2 d
4b 4e :2 d e 11 :2 d 54 57
5f 13 1e :2 5f :2 57 :2 d 13 16
:2 d e 11 :2 d 4d 50 :2 d 13
16 :2 d 50 53 :2 d 13 16 :2 d
e 11 :2 d e 11 :2 d 7 2f
58 :2 4 7 :2 11 23 :2 2d 39 3c
4b 4f 50 :2 4b :2 23 :3 7 :2 d 12
1e 5d 60 :2 1e :2 7 53 :2 4 7
:2 11 23 :2 2d 39 3c 4b 4f 50
:2 4b :2 23 :3 7 :2 d 12 1e 5d 60
:2 1e :2 7 53 :3 4 b :2 16 26 :2 b
4 1 9 12 9 a :2 14 26
:2 30 3c 40 4f 53 54 :2 4f :2 26
:3 a :2 10 15 21 60 63 :2 21 :3 a
:2 10 15 39 5c 5f :2 68 :2 39 76
79 :2 39 81 84 :2 8d :2 39 :2 a 57
:3 7 c 15 1e 2c a 13 1e
f e 7 e 7 e 7 29
:2 4 :2 9 7 :2 10 1c :2 7 :2 11 23
:2 2d 39 3d 4c 50 51 :2 4c :2 23
:3 7 :2 d 12 1e 5d 60 :2 1e :3 7
:2 d 12 36 42 45 :2 36 4d 50
:2 36 5c 5f 66 6f 72 :2 5f :2 36
:2 7 54 :2 4 7 c 1b a 13
1e f e 7 e 7 e 7
:2 4 :4 10 :2 4 :8 1 
1a7f
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 :4 7 :4 8
:4 9 :4 a :4 b :4 c
:4 d :4 e :4 f :4 10
:4 11 2 :4 12 :2 1
:7 14 :7 15 :7 16 :7 17
:7 18 :7 19 :7 1a :5 1b
:7 1c :7 1d :7 1e :7 1f
:7 20 :7 21 :7 22 :7 23
:5 24 :5 25 :7 26 :6 27
:6 28 :6 29 :6 2a :6 2b
:6 2c :6 2d :6 2e :6 2f
:6 30 :6 31 :6 32 :6 33
:6 34 :6 35 :6 36 :6 37
:6 38 :6 39 :6 3a :6 3b
:6 3c :6 3d :6 3e :6 3f
:6 40 :6 41 :6 42 :6 43
:6 44 :7 45 :7 46 :6 47
:6 48 :7 49 :6 4a :6 4b
:6 4c :6 4d :6 4e :6 4f
:5 50 :7 52 :b 54 :11 55
56 :2 55 :2 56 :2 55
:2 56 :2 55 :2 57 :2 55
:2 57 :2 55 :2 57 :2 55
:2 57 :2 55 :2 58 :2 55
:2 58 :2 55 :2 58 :2 55
:2 58 :2 55 :2 59 :2 55
:9 59 :2 55 :2 5a :2 55
:9 5a :2 55 :2 5b :2 55
:9 5b :2 55 :2 5c :2 55
:9 5c :2 55 :2 5d :2 55
:9 5d :2 55 :2 5e :2 55
:9 5e :2 55 :2 5f :2 55
:9 5f :2 55 :2 60 :2 55
:9 60 :2 55 :2 61 :2 55
:9 61 :2 55 :2 62 :2 55
:9 62 :2 55 :2 63 :2 55
:2 63 :2 55 :2 63 :2 55
:2 63 :2 55 :2 64 :2 55
:2 64 :2 55 :2 64 :2 55
:2 64 :2 55 :2 64 :4 55
:3 54 :11 67 :b 68 :3 67
:3 6f :3 70 :3 71 :3 72
:3 74 :3 76 :3 78 :3 7a
:3 7b :3 7c :3 7d :3 7e
:3 7f :3 80 :3 82 :a 83
:3 88 :3 89 :5 8b :11 8c
:20 8d :3 8c :11 8f :b 90
:3 8f :9 92 :3 8b :1f 9f
:11 a0 :20 a1 :3 a0 :11 a3
:b a4 :3 a3 :9 a6 :3 9f
:14 a9 :6 aa :2 a9 :6 ac
ad ae ac :5 af
ab b1 :5 b2 :3 b1
:2 b3 :5 b5 :11 b6 :24 b7
:3 b6 :2 b4 :6 b3 b0
:3 aa :16 bc :11 bd :20 be
:3 bd :11 c0 :b c1 :3 c0
:9 c3 :3 bc :3 c6 c7
aa :e c7 :2 c9 ca
c9 :5 cb c8 cd
:5 ce :3 cd :2 cf :5 d1
:11 d2 :24 d3 :3 d2 :2 d0
:6 cf cc :4 c7 aa
:2 a9 :3 d9 :14 db :e dd
:9 e0 :5 e2 :3 e3 :3 e4
e2 :5 e5 e1 e7
:5 e8 :3 e7 :2 e9 :5 eb
:11 ec :24 ed :3 ec :2 ea
:6 e9 e6 :3 e0 :a f2
:7 f4 :5 f5 f3 f7
:5 f8 :3 f7 :2 f9 :5 fb
:11 fc :24 fd :3 fc :2 fa
:6 f9 f6 :3 f2 :a 101
:11 102 :30 103 :3 102 :11 105
:b 106 :3 105 :a 108 :3 101
:3 f2 :3 10c :2 10e 10f
:2 110 10e :5 111 10d
113 :5 114 :3 113 :2 115
:5 117 :11 118 :24 119 :3 118
:2 116 :6 115 112 :3 e0
:9 11e :2 120 121 :2 122
120 :5 123 11f 125
:5 126 :3 125 :2 127 :5 129
:11 12a :24 12b :3 12a :2 128
:6 127 124 :3 11e :b 130
:11 131 :30 132 :3 131 :11 134
:b 135 :3 134 :a 137 :3 130
:3 11e e0 :5 13b :11 13c
:20 13d :3 13c :11 13f :b 140
:3 13f :9 142 :3 13b :3 145
:d 149 :b 14d :c 14f :6 151
:4 152 :e 153 151 :5 154
150 156 :5 157 :3 156
:2 158 :5 15a :11 15b :24 15c
:3 15b :2 159 :6 158 155
:3 14f :a 161 :11 162 :30 163
:3 162 :11 165 :b 166 :3 165
:a 168 :3 161 14f :11 16b
:30 16c :3 16b :11 16e :b 16f
:3 16e :a 171 16a :3 14f
13a :3 e0 :16 176 :3 177
:3 176 :5 17a :3 17c :3 17d
:e 17f :b 180 :2 181 :2 180
:a 181 :8 182 :3 17f :b 184
:11 185 :20 186 :3 185 :11 188
:b 189 :3 188 :9 18b 18c
184 :9 18c :11 18d :2e 18e
:3 18d :11 190 :b 191 :3 190
:c 193 18c :3 184 :3 17a
:16 196 :16 197 :3 198 199
197 :9 199 :3 19a 19b
199 197 :16 19b :3 19c
19d 19b 197 :9 19d
:f 19e :3 19f 19d :3 197
:3 196 :9 1a4 :9 1a5 :11 1a6
:30 1a7 :3 1a6 :11 1a9 :b 1aa
:3 1a9 :a 1ac :3 1a5 :9 1ae
:3 1af 1ae :11 1b1 1b0
:3 1ae :3 1a4 :5 1b5 :3 1b8
:3 1b9 :3 1ba :3 1bb :8 1bd
:8 1bf :9 1c4 :3 1c5 :3 1c6
:a 1c7 :9 1d1 :2 1d2 :5 1d3
:2 1d4 1d5 :6 1d6 1d1
:5 1d7 1d0 1d9 :5 1da
:3 1d9 :2 1db :5 1dd :11 1de
:24 1df :3 1de :2 1dc :6 1db
1d8 :3 1c7 :6 1e5 :5 1e6
1e4 1e8 :5 1e9 :3 1e8
:2 1ea :5 1ec :11 1ed :24 1ee
:3 1ed :2 1eb :6 1ea 1e7
:3 1c7 :a 1f3 :11 1f4 :30 1f5
:3 1f4 :11 1f7 :b 1f8 :3 1f7
:a 1fa :3 1f3 :3 1fd :4 200
201 :5 202 200 :5 203
1ff 205 :5 206 :3 205
:2 207 :5 209 :11 20a :24 20b
:3 20a :2 208 :6 207 204
:3 1c7 20f 1c7 :a 20f
:6 211 :5 212 210 214
:5 215 :3 214 :2 216 :5 218
:11 219 :24 21a :3 219 :2 217
:6 216 213 :3 20f :a 21f
:11 220 :30 221 :3 220 :11 223
:b 224 :3 223 :a 226 :3 21f
:3 229 :9 22c :2 22d :5 22e
:2 22f 230 :6 231 22c
:5 232 22b 234 :5 235
:3 234 :2 236 :5 238 :11 239
:24 23a :3 239 :2 237 :6 236
233 :3 20f :a 24b :9 24d
:2 24e :5 24f :2 250 251
:6 252 24d :5 253 24c
255 :5 256 :3 255 :2 257
:5 259 :11 25a :24 25b :3 25a
:2 258 :6 257 254 :6 24b
:4 263 264 :5 265 263
:5 266 262 268 :5 269
:3 268 :2 26a :5 26c :11 26d
:24 26e :3 26d :2 26b :6 26a
267 :4 20f 1c7 :11 273
:20 274 :3 273 :11 276 :b 277
:3 276 :9 279 272 :3 1c7
:a 27d :b 27e 27f :3 27e
:b 280 281 :3 280 :2 284
:f 286 :4 287 :4 288 :5 289
28a :3 28c 28d :2 28f
284 :6 290 :5 291 290
:5 293 292 :3 290 283
296 :5 297 :3 296 :2 298
:5 29a :11 29b :24 29c :3 29b
:2 299 :6 298 295 :3 27d
:6 2a1 27d :5 2a4 :b 2a6
2a7 :3 2a6 :5 2a9 :3 2aa
:6 2ac :3 2ad :3 2ac :6 2b0
:6 2b1 :5 2b2 :7 2b3 2b0
:6 2b4 :5 2b5 2b4 :5 2b7
2b6 :3 2b4 2af 2ba
:5 2bb :3 2ba :2 2bc :5 2be
:11 2bf :24 2c0 :3 2bf :2 2bd
:6 2bc 2b9 :4 2a2 :3 27d
:12 2c7 :6 2c9 :5 2ca 2c8
2cc :5 2cd :3 2cc :2 2ce
:5 2d0 :11 2d1 :24 2d2 :3 2d1
:2 2cf :6 2ce 2cb :3 2c7
:a 2d8 :6 2d9 :8 2da :4 2db
:2 2dc :3 2da :3 2d8 :3 2c7
:3 2e0 :a 2e5 :5 2e6 2e5
:7 2e9 2e7 :3 2e5 :8 2ed
:5 2ee :8 2ef :6 2f0 :7 2f1
:3 2ed :5 2f4 :5 2f6 :6 2f7
:5 2f8 2f7 :5 2fa 2f9
:3 2f7 2f5 2fd :5 2fe
:3 2fd :2 2ff :5 301 :11 302
:24 303 :3 302 :2 300 :6 2ff
2fc :6 2f4 1c4 :3 309
:3 30a :3 30c :c 30e :3 30f
30e :16 311 310 :3 30e
:5 314 :3 315 :c 316 :17 319
:1b 31b :a 31d :5 31e :c 320
:3 321 320 :c 323 :c 324
322 :3 320 31d :7 327
:c 329 :3 32a 329 :c 32c
:c 32d 32b :3 329 326
:3 31d :a 331 :9 333 :3 334
:2 335 :5 336 333 :5 337
332 339 :5 33a :3 339
:2 33b :5 33d :11 33e :24 33f
:3 33e :2 33c :6 33b 338
:3 331 :3 344 :3 345 331
:9 349 :3 34a :2 34b :6 34c
349 :5 34d 348 34f
:5 350 :3 34f :2 351 :5 353
:11 354 :24 355 :3 354 :2 352
:6 351 34e :3 347 :a 35a
:9 35c :3 35d :2 35e :6 35f
35c :5 360 35b 362
:5 363 :3 362 :2 364 :5 366
:11 367 :24 368 :3 367 :2 365
:6 364 361 :6 35a 347
:3 331 :3 370 :a 372 :5 373
372 :7 375 374 :3 372
:3 378 :8 37a :5 37b :8 37c
:6 37d :7 37e :3 37a :5 381
:5 383 :6 384 :5 385 384
:5 387 386 :3 384 382
38a :5 38b :3 38a :2 38c
:5 38e :11 38f :24 390 :3 38f
:2 38d :6 38c 389 :6 381
:b 396 316 397 316
:3 314 308 :3 1c4 :12 39c
:11 39d :30 39e :3 39d :11 3a0
:b 3a1 :3 3a0 :7 3a3 3a4
:2 3a3 :3 39c :9 3a7 :3 3a8
:3 3a7 :1c 3ad 3ae 3af
:2 3b0 :2 3af :2 3b1 :2 3af
:2 3b2 :2 3af :2 3b3 :2 3af
:9 3b3 :2 3af :2 3b3 :2 3af
:9 3b4 :2 3af :2 3b5 :2 3af
:9 3b6 :2 3af :2 3b7 :2 3af
:2 3b8 :2 3af :2 3b9 :2 3af
:2 3ba :2 3af :2 3bb :2 3af
:2 3bc :2 3af :2 3bc :2 3af
:2 3bd :2 3af :2 3bd :2 3af
:2 3be :2 3af :2 3bf :2 3af
:2 3c0 :2 3af :2 3c1 :2 3af
:4 3c2 :2 3af :2 3c2 :2 3af
:4 3c2 :2 3af :2 3c3 :2 3af
:2 3c4 :2 3af :2 3c5 :2 3af
:9 3c5 :2 3af :2 3c6 :2 3af
:9 3c6 :2 3af :2 3c6 :2 3af
:2 3c7 :2 3af :3 3c7 :2 3c8
:4 3c7 :2 3af :2 3c9 :2 3af
:2 3ca :2 3af :2 3ca :2 3af
:2 3cb :2 3af :2 3cb :2 3af
:2 3cc :2 3af :2 3cd :2 3af
:2 3ce :2 3af 3ae 3cf
3ad :c 3cf 3d0 3d1
:2 3d2 :2 3d1 :2 3d3 :2 3d1
:2 3d4 :2 3d1 :2 3d5 :2 3d1
:9 3d5 :2 3d1 :2 3d5 :2 3d1
:9 3d6 :2 3d1 :2 3d7 :2 3d1
:9 3d8 :2 3d1 :2 3d9 :2 3d1
:2 3da :2 3d1 :2 3db :2 3d1
:2 3dc :2 3d1 :2 3dd :2 3d1
:2 3de :2 3d1 :2 3de :2 3d1
:2 3df :2 3d1 :2 3df :2 3d1
:2 3e0 :2 3d1 :2 3e1 :2 3d1
3d0 3e2 3cf 3ad
:c 3e2 3e3 3e4 :2 3e5
:2 3e4 :4 3e6 :2 3e4 :2 3e6
:2 3e4 :4 3e6 :2 3e4 :2 3e7
:2 3e4 :2 3e8 :2 3e4 :2 3e9
:2 3e4 :9 3e9 :2 3e4 :2 3ea
:2 3e4 :9 3ea :2 3e4 :2 3ea
:2 3e4 :2 3eb :2 3e4 :3 3eb
:2 3ec :4 3eb :2 3e4 :2 3ed
:2 3e4 :2 3ee :2 3e4 :2 3ee
:2 3e4 :2 3ef :2 3e4 :2 3ef
:2 3e4 :2 3f0 :2 3e4 :2 3f1
:2 3e4 :2 3f2 :2 3e4 3e3
3e2 :3 3ad :11 3f6 :b 3f7
:3 3f6 :11 3f9 :b 3fa :3 3f9
:8 3fc 53 :3 400 :11 401
:b 402 :17 403 :3 401 :2 405
:3 406 :3 407 408 406
405 406 :3 409 :3 400
:2 40a :5 40c :11 40d :b 40e
:18 40f :3 40d :2 411 412
:3 413 414 412 411
412 :3 415 :2 40b :6 40a
3ff :7 1 
5ba8
4
:3 0 1 :3 0 2
:a 0 1a7a 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 4
:3 0 5 :7 0 9
8 :3 0 4 :3 0
6 :7 0 d c
:3 0 d 7c 0
b 8 :3 0 7
:7 0 11 10 :3 0
8 :3 0 9 :7 0
15 14 :3 0 11
a2 0 f 8
:3 0 a :7 0 19
18 :3 0 8 :3 0
b :7 0 1d 1c
:3 0 15 c8 0
13 8 :3 0 c
:7 0 21 20 :3 0
8 :3 0 d :7 0
25 24 :3 0 19
ee 0 17 8
:3 0 e :7 0 29
28 :3 0 8 :3 0
f :7 0 2d 2c
:3 0 1d 114 0
1b 8 :3 0 10
:7 0 31 30 :3 0
8 :3 0 11 :7 0
35 34 :3 0 21
:2 0 1f 4 :3 0
12 :7 0 39 38
:3 0 4 :3 0 13
:7 0 3d 3c :3 0
14 :3 0 15 :3 0
16 :2 0 4 41
42 0 3f 43
0 1a7a 2 44
:2 0 18 :2 0 33
4 :3 0 18 :2 0
31 47 49 :6 0
4c 4a 0 1a78
0 17 :6 0 18
:2 0 37 4 :3 0
35 4e 50 :6 0
53 51 0 1a78
0 19 :6 0 18
:2 0 3b 4 :3 0
39 55 57 :6 0
5a 58 0 1a78
0 1a :6 0 1d
:2 0 3f 4 :3 0
3d 5c 5e :6 0
61 5f 0 1a78
0 1b :6 0 1d
:2 0 43 4 :3 0
41 63 65 :6 0
68 66 0 1a78
0 1c :6 0 1d
:2 0 47 4 :3 0
45 6a 6c :6 0
6f 6d 0 1a78
0 1e :6 0 4d
21f 0 4b 4
:3 0 49 71 73
:6 0 76 74 0
1a78 0 1f :6 0
23 :2 0 51 21
:3 0 78 :7 0 7b
79 0 1a78 0
20 :6 0 4 :3 0
23 :2 0 4f 7d
7f :6 0 82 80
0 1a78 0 22
:6 0 23 :2 0 55
4 :3 0 53 84
86 :6 0 89 87
0 1a78 0 24
:6 0 18 :2 0 59
4 :3 0 57 8b
8d :6 0 90 8e
0 1a78 0 25
:6 0 18 :2 0 5d
4 :3 0 5b 92
94 :6 0 97 95
0 1a78 0 26
:6 0 29 :2 0 61
4 :3 0 5f 99
9b :6 0 9e 9c
0 1a78 0 27
:6 0 29 :2 0 65
4 :3 0 63 a0
a2 :6 0 a5 a3
0 1a78 0 28
:6 0 29 :2 0 69
4 :3 0 67 a7
a9 :6 0 ac aa
0 1a78 0 2a
:6 0 6f 322 0
6d 4 :3 0 6b
ae b0 :6 0 b3
b1 0 1a78 0
2b :6 0 2f :2 0
71 21 :3 0 b5
:7 0 b8 b6 0
1a78 0 2c :6 0
21 :3 0 ba :7 0
bd bb 0 1a78
0 2d :6 0 31
:2 0 75 4 :3 0
73 bf c1 :6 0
c4 c2 0 1a78
0 2e :6 0 31
:2 0 77 8 :3 0
c6 :7 0 ca c7
c8 1a78 0 30
:6 0 31 :2 0 79
8 :3 0 cc :7 0
d0 cd ce 1a78
0 32 :6 0 31
:2 0 7b 8 :3 0
d2 :7 0 d6 d3
d4 1a78 0 33
:6 0 31 :2 0 7d
8 :3 0 d8 :7 0
dc d9 da 1a78
0 34 :6 0 31
:2 0 7f 8 :3 0
de :7 0 e2 df
e0 1a78 0 35
:6 0 31 :2 0 81
8 :3 0 e4 :7 0
e8 e5 e6 1a78
0 36 :6 0 31
:2 0 83 8 :3 0
ea :7 0 ee eb
ec 1a78 0 37
:6 0 31 :2 0 85
8 :3 0 f0 :7 0
f4 f1 f2 1a78
0 38 :6 0 31
:2 0 87 8 :3 0
f6 :7 0 fa f7
f8 1a78 0 39
:6 0 31 :2 0 89
8 :3 0 fc :7 0
100 fd fe 1a78
0 3a :6 0 31
:2 0 8b 8 :3 0
102 :7 0 106 103
104 1a78 0 3b
:6 0 31 :2 0 8d
8 :3 0 108 :7 0
10c 109 10a 1a78
0 3c :6 0 31
:2 0 8f 8 :3 0
10e :7 0 112 10f
110 1a78 0 3d
:6 0 31 :2 0 91
8 :3 0 114 :7 0
118 115 116 1a78
0 3e :6 0 31
:2 0 93 8 :3 0
11a :7 0 11e 11b
11c 1a78 0 3f
:6 0 31 :2 0 95
8 :3 0 120 :7 0
124 121 122 1a78
0 40 :6 0 31
:2 0 97 8 :3 0
126 :7 0 12a 127
128 1a78 0 41
:6 0 31 :2 0 99
8 :3 0 12c :7 0
130 12d 12e 1a78
0 42 :6 0 31
:2 0 9b 8 :3 0
132 :7 0 136 133
134 1a78 0 43
:6 0 31 :2 0 9d
8 :3 0 138 :7 0
13c 139 13a 1a78
0 44 :6 0 31
:2 0 9f 8 :3 0
13e :7 0 142 13f
140 1a78 0 45
:6 0 31 :2 0 a1
8 :3 0 144 :7 0
148 145 146 1a78
0 46 :6 0 31
:2 0 a3 8 :3 0
14a :7 0 14e 14b
14c 1a78 0 47
:6 0 31 :2 0 a5
49 :3 0 150 :7 0
154 151 152 1a78
0 48 :6 0 31
:2 0 a7 8 :3 0
156 :7 0 15a 157
158 1a78 0 4a
:6 0 31 :2 0 a9
8 :3 0 15c :7 0
160 15d 15e 1a78
0 4b :6 0 4e
:2 0 ab 8 :3 0
162 :7 0 166 163
164 1a78 0 4c
:6 0 4e :2 0 ad
49 :3 0 168 :7 0
16c 169 16a 1a78
0 4d :6 0 31
:2 0 af 49 :3 0
16e :7 0 172 16f
170 1a78 0 4f
:6 0 18 :2 0 b1
8 :3 0 174 :7 0
178 175 176 1a78
0 50 :6 0 18
:2 0 b5 52 :3 0
b3 17a 17c :6 0
17f 17d 0 1a78
0 51 :6 0 31
:2 0 b9 52 :3 0
b7 181 183 :6 0
186 184 0 1a78
0 53 :6 0 31
:2 0 bb 8 :3 0
188 :7 0 18c 189
18a 1a78 0 54
:6 0 57 :2 0 bd
8 :3 0 18e :7 0
192 18f 190 1a78
0 55 :6 0 31
:2 0 c1 4 :3 0
bf 194 196 :6 0
199 197 0 1a78
0 56 :6 0 31
:2 0 c3 8 :3 0
19b :7 0 19f 19c
19d 1a78 0 58
:6 0 23 :2 0 c5
8 :3 0 1a1 :7 0
1a5 1a2 1a3 1a78
0 59 :6 0 23
:2 0 c7 8 :3 0
1a7 0 2 :5 0
1ab 1a8 1a9 1a78
0 5a :6 0 31
:2 0 c9 8 :3 0
1ad 0 2 :5 0
1b1 1ae 1af 1a78
0 5b :6 0 31
:2 0 cb 8 :3 0
1b3 0 2 :5 0
1b7 1b4 1b5 1a78
0 5c :6 0 cf
7f2 0 cd 8
:3 0 1b9 0 2
:5 0 1bd 1ba 1bb
1a78 0 5d :6 0
1ca 1cb 0 d1
8 :3 0 1bf :7 0
1c2 1c0 0 1a78
0 5e :6 0 15
:3 0 16 :2 0 4
1c4 1c5 0 1c6
:7 0 1c9 1c7 0
1a78 0 5f :6 0
60 :3 0 61 :3 0
60 :3 0 62 :3 0
1cd 1ce 0 2
:4 0 d3 1cf 1d1
1cc 1d3 1d2 :2 0
63 :3 0 64 :3 0
1d5 1d6 0 65
:4 0 60 :3 0 66
:3 0 1d9 1da 0
2 :4 0 d5 1db
1dd 67 :4 0 68
:2 0 69 :3 0 d7
1e0 1e2 :3 0 6a
:4 0 68 :2 0 6b
:4 0 da 1e5 1e7
:3 0 68 :2 0 3
:3 0 dd 1e9 1eb
:3 0 68 :2 0 6b
:4 0 e0 1ed 1ef
:3 0 68 :2 0 6c
:4 0 e3 1f1 1f3
:3 0 68 :2 0 6b
:4 0 e6 1f5 1f7
:3 0 68 :2 0 5
:3 0 e9 1f9 1fb
:3 0 68 :2 0 6b
:4 0 ec 1fd 1ff
:3 0 68 :2 0 6c
:4 0 ef 201 203
:3 0 68 :2 0 6b
:4 0 f2 205 207
:3 0 68 :2 0 6
:3 0 f5 209 20b
:3 0 68 :2 0 6b
:4 0 f8 20d 20f
:3 0 68 :2 0 6c
:4 0 fb 211 213
:3 0 68 :2 0 6d
:3 0 6e :3 0 7
:3 0 fe 217 219
6f :4 0 100 216
21c 103 215 21e
:3 0 68 :2 0 6c
:4 0 106 220 222
:3 0 68 :2 0 6d
:3 0 6e :3 0 9
:3 0 109 226 228
6f :4 0 10b 225
22b 10e 224 22d
:3 0 68 :2 0 6c
:4 0 111 22f 231
:3 0 68 :2 0 6d
:3 0 6e :3 0 a
:3 0 114 235 237
6f :4 0 116 234
23a 119 233 23c
:3 0 68 :2 0 6c
:4 0 11c 23e 240
:3 0 68 :2 0 6d
:3 0 6e :3 0 b
:3 0 11f 244 246
6f :4 0 121 243
249 124 242 24b
:3 0 68 :2 0 6c
:4 0 127 24d 24f
:3 0 68 :2 0 6d
:3 0 6e :3 0 c
:3 0 12a 253 255
6f :4 0 12c 252
258 12f 251 25a
:3 0 68 :2 0 6c
:4 0 132 25c 25e
:3 0 68 :2 0 6d
:3 0 6e :3 0 d
:3 0 135 262 264
6f :4 0 137 261
267 13a 260 269
:3 0 68 :2 0 6c
:4 0 13d 26b 26d
:3 0 68 :2 0 6d
:3 0 6e :3 0 e
:3 0 140 271 273
6f :4 0 142 270
276 145 26f 278
:3 0 68 :2 0 6c
:4 0 148 27a 27c
:3 0 68 :2 0 6d
:3 0 6e :3 0 f
:3 0 14b 280 282
6f :4 0 14d 27f
285 150 27e 287
:3 0 68 :2 0 6c
:4 0 153 289 28b
:3 0 68 :2 0 6d
:3 0 6e :3 0 10
:3 0 156 28f 291
6f :4 0 158 28e
294 15b 28d 296
:3 0 68 :2 0 6c
:4 0 15e 298 29a
:3 0 68 :2 0 6d
:3 0 6e :3 0 11
:3 0 161 29e 2a0
6f :4 0 163 29d
2a3 166 29c 2a5
:3 0 68 :2 0 6c
:4 0 169 2a7 2a9
:3 0 68 :2 0 6b
:4 0 16c 2ab 2ad
:3 0 68 :2 0 12
:3 0 16f 2af 2b1
:3 0 68 :2 0 6b
:4 0 172 2b3 2b5
:3 0 68 :2 0 6c
:4 0 175 2b7 2b9
:3 0 68 :2 0 6b
:4 0 178 2bb 2bd
:3 0 68 :2 0 13
:3 0 17b 2bf 2c1
:3 0 68 :2 0 6b
:4 0 17e 2c3 2c5
:3 0 68 :2 0 70
:4 0 181 2c7 2c9
:3 0 184 1d7 2cb
:2 0 2cd 189 2ce
1d4 2cd 0 2cf
18b 0 19d8 60
:3 0 61 :3 0 2d0
2d1 0 60 :3 0
71 :3 0 2d3 2d4
0 23 :2 0 2
:4 0 72 :2 0 73
:2 0 23 :2 0 18d
2d9 2db :3 0 190
2d5 2dd 2d2 2df
2de :2 0 63 :3 0
74 :3 0 2e1 2e2
0 65 :4 0 75
:4 0 68 :2 0 69
:3 0 194 2e6 2e8
:3 0 197 2e3 2ea
:2 0 2ec 19a 2ed
2e0 2ec 0 2ee
19c 0 19d8 17
:3 0 3 :3 0 2ef
2f0 0 19d8 1c
:3 0 5 :3 0 2f2
2f3 0 19d8 1f
:3 0 6 :3 0 2f5
2f6 0 19d8 1a
:3 0 13 :3 0 2f8
2f9 0 19d8 39
:3 0 7 :3 0 2fb
2fc 0 19d8 37
:3 0 9 :3 0 2fe
2ff 0 19d8 38
:3 0 e :3 0 301
302 0 19d8 3a
:3 0 b :3 0 304
305 0 19d8 3c
:3 0 f :3 0 307
308 0 19d8 3b
:3 0 c :3 0 30a
30b 0 19d8 33
:3 0 11 :3 0 30d
30e 0 19d8 32
:3 0 a :3 0 310
311 0 19d8 3d
:3 0 d :3 0 313
314 0 19d8 35
:3 0 10 :3 0 316
317 0 19d8 19
:3 0 17 :3 0 319
31a 0 19d8 58
:3 0 76 :3 0 6d
:3 0 33 :3 0 31
:2 0 19e 31e 321
1a1 31d 323 31c
324 0 19d8 48
:3 0 4e :2 0 326
327 0 19d8 34
:3 0 31 :2 0 329
32a 0 19d8 3
:3 0 77 :2 0 1a3
32d 32e :3 0 32f
:2 0 60 :3 0 61
:3 0 331 332 0
60 :3 0 71 :3 0
334 335 0 78
:2 0 2 :4 0 79
:2 0 73 :2 0 23
:2 0 1a5 33a 33c
:3 0 1a8 336 33e
333 340 33f :2 0
63 :3 0 74 :3 0
342 343 0 65
:4 0 7a :4 0 68
:2 0 69 :3 0 1ac
347 349 :3 0 68
:2 0 7b :4 0 1af
34b 34d :3 0 68
:2 0 7c :4 0 1b2
34f 351 :3 0 68
:2 0 6d :3 0 6e
:3 0 7d :3 0 7e
:3 0 356 357 0
1b5 355 359 6f
:4 0 1b7 354 35c
1ba 353 35e :3 0
1bd 344 360 :2 0
362 1c0 363 341
362 0 364 1c2
0 38d 60 :3 0
61 :3 0 365 366
0 60 :3 0 71
:3 0 368 369 0
23 :2 0 2 :4 0
7f :2 0 73 :2 0
23 :2 0 1c4 36e
370 :3 0 1c7 36a
372 367 374 373
:2 0 63 :3 0 74
:3 0 376 377 0
65 :4 0 80 :4 0
68 :2 0 69 :3 0
1cb 37b 37d :3 0
1ce 378 37f :2 0
381 1d1 382 375
381 0 383 1d3
0 38d 81 :3 0
82 :3 0 384 385
0 7d :3 0 7e
:3 0 387 388 0
83 :3 0 1d5 386
38b :2 0 38d 1d8
38e 330 38d 0
38f 1dc 0 19d8
6d :3 0 32 :3 0
31 :2 0 1de 390
393 84 :2 0 85
:2 0 1e3 395 397
:3 0 6d :3 0 3b
:3 0 31 :2 0 1e6
399 39c 84 :2 0
85 :2 0 1eb 39e
3a0 :3 0 398 3a2
3a1 :2 0 6d :3 0
37 :3 0 31 :2 0
1ee 3a4 3a7 84
:2 0 85 :2 0 1f3
3a9 3ab :3 0 3a3
3ad 3ac :2 0 60
:3 0 61 :3 0 3af
3b0 0 60 :3 0
71 :3 0 3b2 3b3
0 78 :2 0 2
:4 0 86 :2 0 73
:2 0 23 :2 0 1f6
3b8 3ba :3 0 1f9
3b4 3bc 3b1 3be
3bd :2 0 63 :3 0
74 :3 0 3c0 3c1
0 65 :4 0 87
:4 0 68 :2 0 69
:3 0 1fd 3c5 3c7
:3 0 68 :2 0 7b
:4 0 200 3c9 3cb
:3 0 68 :2 0 7c
:4 0 203 3cd 3cf
:3 0 68 :2 0 6d
:3 0 6e :3 0 7d
:3 0 88 :3 0 3d4
3d5 0 206 3d3
3d7 6f :4 0 208
3d2 3da 20b 3d1
3dc :3 0 20e 3c2
3de :2 0 3e0 211
3e1 3bf 3e0 0
3e2 213 0 40b
60 :3 0 61 :3 0
3e3 3e4 0 60
:3 0 71 :3 0 3e6
3e7 0 23 :2 0
2 :4 0 89 :2 0
73 :2 0 23 :2 0
215 3ec 3ee :3 0
218 3e8 3f0 3e5
3f2 3f1 :2 0 63
:3 0 74 :3 0 3f4
3f5 0 65 :4 0
8a :4 0 68 :2 0
69 :3 0 21c 3f9
3fb :3 0 21f 3f6
3fd :2 0 3ff 222
400 3f3 3ff 0
401 224 0 40b
81 :3 0 82 :3 0
402 403 0 7d
:3 0 88 :3 0 405
406 0 83 :3 0
226 404 409 :2 0
40b 229 40c 3ae
40b 0 40d 22d
0 19d8 6d :3 0
32 :3 0 31 :2 0
22f 40e 411 7d
:3 0 8b :2 0 8c
:3 0 413 415 0
234 414 417 :3 0
418 :2 0 5 :3 0
8d :2 0 237 41b
41c :3 0 41d :2 0
41e :2 0 419 420
41f :2 0 6 :3 0
8d :2 0 239 423
424 :3 0 425 :2 0
426 :2 0 421 428
427 :2 0 8e :3 0
8f :3 0 6 :3 0
8e :3 0 8f :3 0
5 :3 0 44 :3 0
90 :4 0 91 1
:8 0 438 15 :3 0
92 :3 0 433 434
0 93 :3 0 435
436 0 438 23b
48b 94 :3 0 15
:3 0 92 :3 0 43a
43b 0 93 :3 0
43c 43d 0 43f
23e 441 240 440
43f :2 0 489 95
:3 0 15 :3 0 92
:3 0 444 445 0
93 :3 0 446 447
0 481 60 :3 0
61 :3 0 449 44a
0 60 :3 0 71
:3 0 44c 44d 0
96 :2 0 2 :4 0
97 :2 0 73 :2 0
23 :2 0 242 452
454 :3 0 245 44e
456 44b 458 457
:2 0 63 :3 0 74
:3 0 45a 45b 0
65 :4 0 98 :4 0
68 :2 0 69 :3 0
249 45f 461 :3 0
68 :2 0 7b :4 0
24c 463 465 :3 0
68 :2 0 99 :4 0
24f 467 469 :3 0
68 :2 0 93 :3 0
252 46b 46d :3 0
68 :2 0 9a :4 0
255 46f 471 :3 0
68 :2 0 9b :3 0
9c :3 0 23 :2 0
9d :2 0 258 474
478 25c 473 47a
:3 0 25f 45c 47c
:2 0 47e 262 47f
459 47e 0 480
264 0 481 266
484 :3 0 484 0
484 483 481 482
:6 0 486 2 :3 0
269 488 26b 487
486 :2 0 489 26d
:2 0 48b 0 48b
48a 438 489 :6 0
506 1 :3 0 15
:3 0 92 :3 0 48d
48e 0 15 :3 0
84 :2 0 9e :3 0
490 492 0 272
491 494 :3 0 495
:2 0 6d :3 0 44
:3 0 31 :2 0 275
497 49a 9f :2 0
31 :2 0 27a 49c
49e :3 0 49f :2 0
496 4a1 4a0 :2 0
60 :3 0 61 :3 0
4a3 4a4 0 60
:3 0 71 :3 0 4a6
4a7 0 78 :2 0
2 :4 0 a0 :2 0
73 :2 0 23 :2 0
27d 4ac 4ae :3 0
280 4a8 4b0 4a5
4b2 4b1 :2 0 63
:3 0 74 :3 0 4b4
4b5 0 65 :4 0
a1 :4 0 68 :2 0
69 :3 0 284 4b9
4bb :3 0 68 :2 0
7b :4 0 287 4bd
4bf :3 0 68 :2 0
7c :4 0 28a 4c1
4c3 :3 0 68 :2 0
6d :3 0 6e :3 0
7d :3 0 a2 :3 0
4c8 4c9 0 28d
4c7 4cb 6f :4 0
28f 4c6 4ce 292
4c5 4d0 :3 0 295
4b6 4d2 :2 0 4d4
298 4d5 4b3 4d4
0 4d6 29a 0
4ff 60 :3 0 61
:3 0 4d7 4d8 0
60 :3 0 71 :3 0
4da 4db 0 23
:2 0 2 :4 0 a3
:2 0 73 :2 0 23
:2 0 29c 4e0 4e2
:3 0 29f 4dc 4e4
4d9 4e6 4e5 :2 0
63 :3 0 74 :3 0
4e8 4e9 0 65
:4 0 a4 :4 0 68
:2 0 69 :3 0 2a3
4ed 4ef :3 0 2a6
4ea 4f1 :2 0 4f3
2a9 4f4 4e7 4f3
0 4f5 2ab 0
4ff 81 :3 0 82
:3 0 4f6 4f7 0
7d :3 0 a2 :3 0
4f9 4fa 0 83
:3 0 2ad 4f8 4fd
:2 0 4ff 2b0 500
4a2 4ff 0 501
2b4 0 506 5d
:3 0 23 :2 0 502
503 0 506 a5
:3 0 2b6 575 5
:3 0 77 :2 0 2ba
508 509 :3 0 50a
:2 0 50b :2 0 6
:3 0 77 :2 0 2bc
50e 50f :3 0 510
:2 0 511 :2 0 50c
513 512 :2 0 69
:3 0 1c :3 0 90
:4 0 a6 1 :8 0
51e 15 :3 0 92
:3 0 519 51a 0
93 :3 0 51b 51c
0 51e 2be 571
94 :3 0 15 :3 0
92 :3 0 520 521
0 93 :3 0 522
523 0 525 2c1
527 2c3 526 525
:2 0 56f 95 :3 0
15 :3 0 92 :3 0
52a 52b 0 93
:3 0 52c 52d 0
567 60 :3 0 61
:3 0 52f 530 0
60 :3 0 71 :3 0
532 533 0 96
:2 0 2 :4 0 a7
:2 0 73 :2 0 23
:2 0 2c5 538 53a
:3 0 2c8 534 53c
531 53e 53d :2 0
63 :3 0 74 :3 0
540 541 0 65
:4 0 a8 :4 0 68
:2 0 69 :3 0 2cc
545 547 :3 0 68
:2 0 7b :4 0 2cf
549 54b :3 0 68
:2 0 99 :4 0 2d2
54d 54f :3 0 68
:2 0 93 :3 0 2d5
551 553 :3 0 68
:2 0 9a :4 0 2d8
555 557 :3 0 68
:2 0 9b :3 0 9c
:3 0 23 :2 0 9d
:2 0 2db 55a 55e
2df 559 560 :3 0
2e2 542 562 :2 0
564 2e5 565 53f
564 0 566 2e7
0 567 2e9 56a
:3 0 56a 0 56a
569 567 568 :6 0
56c 4 :3 0 2ec
56e 2ee 56d 56c
:2 0 56f 2f0 :2 0
571 0 571 570
51e 56f :6 0 573
1 :3 0 2f3 574
514 573 0 576
429 506 0 576
2f5 0 19d8 1e
:3 0 1c :3 0 577
578 0 19d8 36
:3 0 a9 :3 0 aa
:3 0 57b 57c 0
ab :3 0 6d :3 0
3c :3 0 31 :2 0
2f8 57f 582 7d
:3 0 ac :3 0 584
585 0 2fb 57e
587 588 :2 0 ad
:2 0 2fe 57d 58b
57a 58c 0 19d8
3c :3 0 ab :3 0
6d :3 0 3c :3 0
31 :2 0 301 590
593 7d :3 0 ae
:3 0 595 596 0
304 58f 598 599
:2 0 58e 59a 0
19d8 6d :3 0 33
:3 0 31 :2 0 307
59c 59f 84 :2 0
31 :2 0 30c 5a1
5a3 :3 0 6d :3 0
af :3 0 6d :3 0
b0 :3 0 b1 :3 0
3f :3 0 40 :3 0
46 :3 0 b2 :3 0
b3 :3 0 17 :4 0
b4 1 :8 0 5b6
15 :3 0 92 :3 0
5b1 5b2 0 93
:3 0 5b3 5b4 0
5b6 30f 609 94
:3 0 15 :3 0 92
:3 0 5b8 5b9 0
93 :3 0 5ba 5bb
0 5bd 312 5bf
314 5be 5bd :2 0
607 95 :3 0 15
:3 0 92 :3 0 5c2
5c3 0 93 :3 0
5c4 5c5 0 5ff
60 :3 0 61 :3 0
5c7 5c8 0 60
:3 0 71 :3 0 5ca
5cb 0 96 :2 0
2 :4 0 b5 :2 0
73 :2 0 23 :2 0
316 5d0 5d2 :3 0
319 5cc 5d4 5c9
5d6 5d5 :2 0 63
:3 0 74 :3 0 5d8
5d9 0 65 :4 0
b6 :4 0 68 :2 0
69 :3 0 31d 5dd
5df :3 0 68 :2 0
7b :4 0 320 5e1
5e3 :3 0 68 :2 0
99 :4 0 323 5e5
5e7 :3 0 68 :2 0
93 :3 0 326 5e9
5eb :3 0 68 :2 0
9a :4 0 329 5ed
5ef :3 0 68 :2 0
9b :3 0 9c :3 0
23 :2 0 9d :2 0
32c 5f2 5f6 330
5f1 5f8 :3 0 333
5da 5fa :2 0 5fc
336 5fd 5d7 5fc
0 5fe 338 0
5ff 33a 602 :3 0
602 0 602 601
5ff 600 :6 0 604
6 :3 0 33d 606
33f 605 604 :2 0
607 341 :2 0 609
0 609 608 5b6
607 :6 0 83d 1
:3 0 15 :3 0 92
:3 0 60b 60c 0
15 :3 0 84 :2 0
b7 :3 0 60e 610
0 346 60f 612
:3 0 613 :2 0 3f
:3 0 40 :3 0 46
:3 0 b8 :3 0 b3
:3 0 17 :4 0 b9
1 :8 0 621 15
:3 0 92 :3 0 61c
61d 0 93 :3 0
61e 61f 0 621
349 674 94 :3 0
15 :3 0 92 :3 0
623 624 0 93
:3 0 625 626 0
628 34c 62a 34e
629 628 :2 0 672
95 :3 0 15 :3 0
92 :3 0 62d 62e
0 93 :3 0 62f
630 0 66a 60
:3 0 61 :3 0 632
633 0 60 :3 0
71 :3 0 635 636
0 96 :2 0 2
:4 0 ba :2 0 73
:2 0 23 :2 0 350
63b 63d :3 0 353
637 63f 634 641
640 :2 0 63 :3 0
74 :3 0 643 644
0 65 :4 0 bb
:4 0 68 :2 0 69
:3 0 357 648 64a
:3 0 68 :2 0 7b
:4 0 35a 64c 64e
:3 0 68 :2 0 99
:4 0 35d 650 652
:3 0 68 :2 0 93
:3 0 360 654 656
:3 0 68 :2 0 9a
:4 0 363 658 65a
:3 0 68 :2 0 9b
:3 0 9c :3 0 23
:2 0 9d :2 0 366
65d 661 36a 65c
663 :3 0 36d 645
665 :2 0 667 370
668 642 667 0
669 372 0 66a
374 66d :3 0 66d
0 66d 66c 66a
66b :6 0 66f 8
:3 0 377 671 379
670 66f :2 0 672
37b :2 0 674 0
674 673 621 672
:6 0 6f0 1 :3 0
15 :3 0 92 :3 0
676 677 0 15
:3 0 84 :2 0 b7
:3 0 679 67b 0
380 67a 67d :3 0
67e :2 0 60 :3 0
61 :3 0 680 681
0 60 :3 0 71
:3 0 683 684 0
18 :2 0 2 :4 0
bc :2 0 73 :2 0
23 :2 0 383 689
68b :3 0 386 685
68d 682 68f 68e
:2 0 63 :3 0 74
:3 0 691 692 0
65 :4 0 bd :4 0
68 :2 0 69 :3 0
38a 696 698 :3 0
68 :2 0 7b :4 0
38d 69a 69c :3 0
68 :2 0 7c :4 0
390 69e 6a0 :3 0
68 :2 0 6d :3 0
6e :3 0 7d :3 0
88 :3 0 6a5 6a6
0 393 6a4 6a8
6f :4 0 395 6a3
6ab 398 6a2 6ad
:3 0 68 :2 0 be
:4 0 39b 6af 6b1
:3 0 68 :2 0 6b
:4 0 39e 6b3 6b5
:3 0 68 :2 0 17
:3 0 3a1 6b7 6b9
:3 0 68 :2 0 6b
:4 0 3a4 6bb 6bd
:3 0 3a7 693 6bf
:2 0 6c1 3aa 6c2
690 6c1 0 6c3
3ac 0 6ed 60
:3 0 61 :3 0 6c4
6c5 0 60 :3 0
71 :3 0 6c7 6c8
0 23 :2 0 2
:4 0 bf :2 0 73
:2 0 23 :2 0 3ae
6cd 6cf :3 0 3b1
6c9 6d1 6c6 6d3
6d2 :2 0 63 :3 0
74 :3 0 6d5 6d6
0 65 :4 0 c0
:4 0 68 :2 0 69
:3 0 3b5 6da 6dc
:3 0 3b8 6d7 6de
:2 0 6e0 3bb 6e1
6d4 6e0 0 6e2
3bd 0 6ed 81
:3 0 c1 :3 0 6e3
6e4 0 7d :3 0
88 :3 0 6e6 6e7
0 17 :3 0 83
:3 0 3bf 6e5 6eb
:2 0 6ed 3c3 6ee
67f 6ed 0 6ef
3c7 0 6f0 3c9
6f1 614 6f0 0
6f2 3cc 0 83d
59 :3 0 31 :2 0
6f3 6f4 0 83d
c2 :3 0 59 :3 0
c3 :3 0 c4 :3 0
17 :4 0 c5 1
:8 0 701 15 :3 0
92 :3 0 6fc 6fd
0 93 :3 0 6fe
6ff 0 701 3ce
754 94 :3 0 15
:3 0 92 :3 0 703
704 0 93 :3 0
705 706 0 708
3d1 70a 3d3 709
708 :2 0 752 95
:3 0 15 :3 0 92
:3 0 70d 70e 0
93 :3 0 70f 710
0 74a 60 :3 0
61 :3 0 712 713
0 60 :3 0 71
:3 0 715 716 0
96 :2 0 2 :4 0
c6 :2 0 73 :2 0
23 :2 0 3d5 71b
71d :3 0 3d8 717
71f 714 721 720
:2 0 63 :3 0 74
:3 0 723 724 0
65 :4 0 c7 :4 0
68 :2 0 69 :3 0
3dc 728 72a :3 0
68 :2 0 7b :4 0
3df 72c 72e :3 0
68 :2 0 99 :4 0
3e2 730 732 :3 0
68 :2 0 93 :3 0
3e5 734 736 :3 0
68 :2 0 9a :4 0
3e8 738 73a :3 0
68 :2 0 9b :3 0
9c :3 0 23 :2 0
9d :2 0 3eb 73d
741 3ef 73c 743
:3 0 3f2 725 745
:2 0 747 3f5 748
722 747 0 749
3f7 0 74a 3f9
74d :3 0 74d 0
74d 74c 74a 74b
:6 0 74f a :3 0
3fc 751 3fe 750
74f :2 0 752 400
:2 0 754 0 754
753 701 752 :6 0
83d 1 :3 0 6d
:3 0 59 :3 0 31
:2 0 403 756 759
c8 :2 0 31 :2 0
408 75b 75d :3 0
c9 :3 0 22 :3 0
b2 :3 0 b3 :3 0
17 :4 0 ca 1
:8 0 76a 15 :3 0
92 :3 0 765 766
0 93 :3 0 767
768 0 76a 40b
7bd 94 :3 0 15
:3 0 92 :3 0 76c
76d 0 93 :3 0
76e 76f 0 771
40e 773 410 772
771 :2 0 7bb 95
:3 0 15 :3 0 92
:3 0 776 777 0
93 :3 0 778 779
0 7b3 60 :3 0
61 :3 0 77b 77c
0 60 :3 0 71
:3 0 77e 77f 0
96 :2 0 2 :4 0
cb :2 0 73 :2 0
23 :2 0 412 784
786 :3 0 415 780
788 77d 78a 789
:2 0 63 :3 0 74
:3 0 78c 78d 0
65 :4 0 cc :4 0
68 :2 0 69 :3 0
419 791 793 :3 0
68 :2 0 7b :4 0
41c 795 797 :3 0
68 :2 0 99 :4 0
41f 799 79b :3 0
68 :2 0 93 :3 0
422 79d 79f :3 0
68 :2 0 9a :4 0
425 7a1 7a3 :3 0
68 :2 0 9b :3 0
9c :3 0 23 :2 0
9d :2 0 428 7a6
7aa 42c 7a5 7ac
:3 0 42f 78e 7ae
:2 0 7b0 432 7b1
78b 7b0 0 7b2
434 0 7b3 436
7b6 :3 0 7b6 0
7b6 7b5 7b3 7b4
:6 0 7b8 c :3 0
439 7ba 43b 7b9
7b8 :2 0 7bb 43d
:2 0 7bd 0 7bd
7bc 76a 7bb :6 0
83a 1 :3 0 8e
:3 0 cd :3 0 7bf
7c0 0 22 :3 0
ce :4 0 440 7c1
7c4 84 :2 0 31
:2 0 445 7c6 7c8
:3 0 60 :3 0 61
:3 0 7ca 7cb 0
60 :3 0 71 :3 0
7cd 7ce 0 18
:2 0 2 :4 0 cf
:2 0 73 :2 0 23
:2 0 448 7d3 7d5
:3 0 44b 7cf 7d7
7cc 7d9 7d8 :2 0
63 :3 0 74 :3 0
7db 7dc 0 65
:4 0 d0 :4 0 68
:2 0 69 :3 0 44f
7e0 7e2 :3 0 68
:2 0 7b :4 0 452
7e4 7e6 :3 0 68
:2 0 7c :4 0 455
7e8 7ea :3 0 68
:2 0 6d :3 0 6e
:3 0 7d :3 0 d1
:3 0 7ef 7f0 0
458 7ee 7f2 6f
:4 0 45a 7ed 7f5
45d 7ec 7f7 :3 0
68 :2 0 be :4 0
460 7f9 7fb :3 0
68 :2 0 6b :4 0
463 7fd 7ff :3 0
68 :2 0 17 :3 0
466 801 803 :3 0
68 :2 0 6b :4 0
469 805 807 :3 0
46c 7dd 809 :2 0
80b 46f 80c 7da
80b 0 80d 471
0 837 60 :3 0
61 :3 0 80e 80f
0 60 :3 0 71
:3 0 811 812 0
23 :2 0 2 :4 0
d2 :2 0 73 :2 0
23 :2 0 473 817
819 :3 0 476 813
81b 810 81d 81c
:2 0 63 :3 0 74
:3 0 81f 820 0
65 :4 0 d3 :4 0
68 :2 0 69 :3 0
47a 824 826 :3 0
47d 821 828 :2 0
82a 480 82b 81e
82a 0 82c 482
0 837 81 :3 0
c1 :3 0 82d 82e
0 7d :3 0 d1
:3 0 830 831 0
17 :3 0 83 :3 0
484 82f 835 :2 0
837 488 838 7c9
837 0 839 48c
0 83a 48e 83b
75e 83a 0 83c
491 0 83d 493
a29 12 :3 0 77
:2 0 499 83f 840
:3 0 841 :2 0 60
:3 0 61 :3 0 843
844 0 60 :3 0
71 :3 0 846 847
0 78 :2 0 2
:4 0 d4 :2 0 73
:2 0 23 :2 0 49b
84c 84e :3 0 49e
848 850 845 852
851 :2 0 63 :3 0
74 :3 0 854 855
0 65 :4 0 d5
:4 0 68 :2 0 69
:3 0 4a2 859 85b
:3 0 68 :2 0 7b
:4 0 4a5 85d 85f
:3 0 68 :2 0 7c
:4 0 4a8 861 863
:3 0 68 :2 0 6d
:3 0 6e :3 0 7d
:3 0 7e :3 0 868
869 0 4ab 867
86b 6f :4 0 4ad
866 86e 4b0 865
870 :3 0 4b3 856
872 :2 0 874 4b6
875 853 874 0
876 4b8 0 89f
60 :3 0 61 :3 0
877 878 0 60
:3 0 71 :3 0 87a
87b 0 23 :2 0
2 :4 0 d6 :2 0
73 :2 0 23 :2 0
4ba 880 882 :3 0
4bd 87c 884 879
886 885 :2 0 63
:3 0 74 :3 0 888
889 0 65 :4 0
d7 :4 0 68 :2 0
69 :3 0 4c1 88d
88f :3 0 4c4 88a
891 :2 0 893 4c7
894 887 893 0
895 4c9 0 89f
81 :3 0 82 :3 0
896 897 0 7d
:3 0 7e :3 0 899
89a 0 83 :3 0
4cb 898 89d :2 0
89f 4ce 8a0 842
89f 0 8a1 4d2
0 a28 28 :3 0
12 :3 0 8a2 8a3
0 a28 1b :3 0
9b :3 0 9b :3 0
28 :3 0 23 :2 0
18 :2 0 4d4 8a7
8ab 23 :2 0 18
:2 0 4d8 8a6 8af
8a5 8b0 0 a28
24 :3 0 d8 :3 0
d9 :3 0 8b3 8b4
0 23 :2 0 23
:2 0 da :2 0 28
:3 0 4dc 8b5 8ba
8b2 8bb 0 a28
db :3 0 6d :3 0
33 :3 0 31 :2 0
4e1 8be 8c1 4e4
8bd 8c3 8b :2 0
31 :2 0 4e8 8c5
8c7 :3 0 c4 :3 0
6d :3 0 af :3 0
6d :3 0 b0 :3 0
b1 :3 0 17 :3 0
3f :3 0 40 :3 0
46 :3 0 b2 :3 0
b3 :3 0 19 :3 0
dc :3 0 7d :3 0
dd :3 0 7d :3 0
de :3 0 7d :3 0
df :3 0 7d :3 0
e0 :3 0 7d :3 0
e1 :4 0 e2 1
:8 0 8e7 15 :3 0
92 :3 0 8e2 8e3
0 93 :3 0 8e4
8e5 0 8e7 4eb
93a 94 :3 0 15
:3 0 92 :3 0 8e9
8ea 0 93 :3 0
8eb 8ec 0 8ee
4ee 8f0 4f0 8ef
8ee :2 0 938 95
:3 0 15 :3 0 92
:3 0 8f3 8f4 0
93 :3 0 8f5 8f6
0 930 60 :3 0
61 :3 0 8f8 8f9
0 60 :3 0 71
:3 0 8fb 8fc 0
96 :2 0 2 :4 0
e3 :2 0 73 :2 0
23 :2 0 4f2 901
903 :3 0 4f5 8fd
905 8fa 907 906
:2 0 63 :3 0 74
:3 0 909 90a 0
65 :4 0 e4 :4 0
68 :2 0 69 :3 0
4f9 90e 910 :3 0
68 :2 0 7b :4 0
4fc 912 914 :3 0
68 :2 0 99 :4 0
4ff 916 918 :3 0
68 :2 0 93 :3 0
502 91a 91c :3 0
68 :2 0 9a :4 0
505 91e 920 :3 0
68 :2 0 9b :3 0
9c :3 0 23 :2 0
9d :2 0 508 923
927 50c 922 929
:3 0 50f 90b 92b
:2 0 92d 512 92e
908 92d 0 92f
514 0 930 516
933 :3 0 933 0
933 932 930 931
:6 0 935 e :3 0
519 937 51b 936
935 :2 0 938 51d
:2 0 93a 0 93a
939 8e7 938 :6 0
9b6 1 :3 0 15
:3 0 92 :3 0 93c
93d 0 15 :3 0
84 :2 0 b7 :3 0
93f 941 0 522
940 943 :3 0 944
:2 0 60 :3 0 61
:3 0 946 947 0
60 :3 0 71 :3 0
949 94a 0 18
:2 0 2 :4 0 e5
:2 0 73 :2 0 23
:2 0 525 94f 951
:3 0 528 94b 953
948 955 954 :2 0
63 :3 0 74 :3 0
957 958 0 65
:4 0 e6 :4 0 68
:2 0 69 :3 0 52c
95c 95e :3 0 68
:2 0 7b :4 0 52f
960 962 :3 0 68
:2 0 7c :4 0 532
964 966 :3 0 68
:2 0 6d :3 0 6e
:3 0 7d :3 0 e7
:3 0 96b 96c 0
535 96a 96e 6f
:4 0 537 969 971
53a 968 973 :3 0
68 :2 0 be :4 0
53d 975 977 :3 0
68 :2 0 6b :4 0
540 979 97b :3 0
68 :2 0 19 :3 0
543 97d 97f :3 0
68 :2 0 6b :4 0
546 981 983 :3 0
549 959 985 :2 0
987 54c 988 956
987 0 989 54e
0 9b3 60 :3 0
61 :3 0 98a 98b
0 60 :3 0 71
:3 0 98d 98e 0
23 :2 0 2 :4 0
e8 :2 0 73 :2 0
23 :2 0 550 993
995 :3 0 553 98f
997 98c 999 998
:2 0 63 :3 0 74
:3 0 99b 99c 0
65 :4 0 e9 :4 0
68 :2 0 69 :3 0
557 9a0 9a2 :3 0
55a 99d 9a4 :2 0
9a6 55d 9a7 99a
9a6 0 9a8 55f
0 9b3 81 :3 0
c1 :3 0 9a9 9aa
0 7d :3 0 e7
:3 0 9ac 9ad 0
19 :3 0 83 :3 0
561 9ab 9b1 :2 0
9b3 565 9b4 945
9b3 0 9b5 569
0 9b6 56b a25
60 :3 0 61 :3 0
9b7 9b8 0 60
:3 0 71 :3 0 9ba
9bb 0 18 :2 0
2 :4 0 ea :2 0
73 :2 0 23 :2 0
56e 9c0 9c2 :3 0
571 9bc 9c4 9b9
9c6 9c5 :2 0 63
:3 0 74 :3 0 9c8
9c9 0 65 :4 0
eb :4 0 68 :2 0
69 :3 0 575 9cd
9cf :3 0 68 :2 0
7b :4 0 578 9d1
9d3 :3 0 68 :2 0
7c :4 0 57b 9d5
9d7 :3 0 68 :2 0
6d :3 0 6e :3 0
7d :3 0 e7 :3 0
9dc 9dd 0 57e
9db 9df 6f :4 0
580 9da 9e2 583
9d9 9e4 :3 0 68
:2 0 be :4 0 586
9e6 9e8 :3 0 68
:2 0 6b :4 0 589
9ea 9ec :3 0 68
:2 0 19 :3 0 58c
9ee 9f0 :3 0 68
:2 0 6b :4 0 58f
9f2 9f4 :3 0 592
9ca 9f6 :2 0 9f8
595 9f9 9c7 9f8
0 9fa 597 0
a24 60 :3 0 61
:3 0 9fb 9fc 0
60 :3 0 71 :3 0
9fe 9ff 0 23
:2 0 2 :4 0 ec
:2 0 73 :2 0 23
:2 0 599 a04 a06
:3 0 59c a00 a08
9fd a0a a09 :2 0
63 :3 0 74 :3 0
a0c a0d 0 65
:4 0 ed :4 0 68
:2 0 69 :3 0 5a0
a11 a13 :3 0 5a3
a0e a15 :2 0 a17
5a6 a18 a0b a17
0 a19 5a8 0
a24 81 :3 0 c1
:3 0 a1a a1b 0
7d :3 0 e7 :3 0
a1d a1e 0 19
:3 0 83 :3 0 5aa
a1c a22 :2 0 a24
5ae a26 8c8 9b6
0 a27 0 a24
0 a27 5b2 0
a28 5b5 a2a 5a4
83d 0 a2b 0
a28 0 a2b 5bb
0 19d8 6d :3 0
d :3 0 31 :2 0
5be a2c a2f 84
:2 0 31 :2 0 5c3
a31 a33 :3 0 a34
:2 0 6d :3 0 3b
:3 0 31 :2 0 5c6
a36 a39 8b :2 0
ee :2 0 5cb a3b
a3d :3 0 a3e :2 0
a35 a40 a3f :2 0
37 :3 0 31 :2 0
a42 a43 0 a45
5ce a46 a41 a45
0 a47 5d0 0
19d8 5d :3 0 84
:2 0 31 :2 0 5d4
a49 a4b :3 0 1e
:3 0 5 :3 0 a4d
a4e 0 b68 1f
:3 0 ef :4 0 a50
a51 0 b68 59
:3 0 f0 :3 0 1e
:3 0 1f :3 0 6d
:3 0 3a :3 0 31
:2 0 5d7 a57 a5a
6d :3 0 3b :3 0
31 :2 0 5da a5c
a5f 6d :3 0 3c
:3 0 31 :2 0 5dd
a61 a64 6d :3 0
32 :3 0 31 :2 0
5e0 a66 a69 6d
:3 0 3d :3 0 31
:2 0 5e3 a6b a6e
6d :3 0 38 :3 0
31 :2 0 5e6 a70
a73 6d :3 0 3f
:3 0 31 :2 0 5e9
a75 a78 6d :3 0
40 :3 0 31 :2 0
5ec a7a a7d 37
:3 0 39 :3 0 42
:3 0 5ef a54 a82
a53 a83 0 b68
6d :3 0 59 :3 0
31 :2 0 5fd a85
a88 7d :3 0 84
:2 0 88 :3 0 a8a
a8c 0 602 a8b
a8e :3 0 60 :3 0
61 :3 0 a90 a91
0 60 :3 0 71
:3 0 a93 a94 0
78 :2 0 2 :4 0
f1 :2 0 73 :2 0
23 :2 0 605 a99
a9b :3 0 608 a95
a9d a92 a9f a9e
:2 0 63 :3 0 74
:3 0 aa1 aa2 0
65 :4 0 f2 :4 0
68 :2 0 69 :3 0
60c aa6 aa8 :3 0
68 :2 0 7b :4 0
60f aaa aac :3 0
68 :2 0 7c :4 0
612 aae ab0 :3 0
68 :2 0 6d :3 0
6e :3 0 7d :3 0
88 :3 0 ab5 ab6
0 615 ab4 ab8
6f :4 0 617 ab3
abb 61a ab2 abd
:3 0 61d aa3 abf
:2 0 ac1 620 ac2
aa0 ac1 0 ac3
622 0 aed 60
:3 0 61 :3 0 ac4
ac5 0 60 :3 0
71 :3 0 ac7 ac8
0 23 :2 0 2
:4 0 f3 :2 0 73
:2 0 23 :2 0 624
acd acf :3 0 627
ac9 ad1 ac6 ad3
ad2 :2 0 63 :3 0
74 :3 0 ad5 ad6
0 65 :4 0 f4
:4 0 68 :2 0 69
:3 0 62b ada adc
:3 0 62e ad7 ade
:2 0 ae0 631 ae1
ad4 ae0 0 ae2
633 0 aed 81
:3 0 82 :3 0 ae3
ae4 0 7d :3 0
88 :3 0 ae6 ae7
0 83 :3 0 635
ae5 aea :2 0 aed
a5 :3 0 638 b66
6d :3 0 59 :3 0
31 :2 0 63c aee
af1 8b :2 0 31
:2 0 641 af3 af5
:3 0 60 :3 0 61
:3 0 af7 af8 0
60 :3 0 71 :3 0
afa afb 0 18
:2 0 2 :4 0 f5
:2 0 73 :2 0 23
:2 0 644 b00 b02
:3 0 647 afc b04
af9 b06 b05 :2 0
63 :3 0 74 :3 0
b08 b09 0 65
:4 0 f6 :4 0 68
:2 0 69 :3 0 64b
b0d b0f :3 0 68
:2 0 7b :4 0 64e
b11 b13 :3 0 68
:2 0 7c :4 0 651
b15 b17 :3 0 68
:2 0 6d :3 0 6e
:3 0 59 :3 0 654
b1b b1d 6f :4 0
656 b1a b20 659
b19 b22 :3 0 68
:2 0 be :4 0 65c
b24 b26 :3 0 68
:2 0 6b :4 0 65f
b28 b2a :3 0 68
:2 0 17 :3 0 662
b2c b2e :3 0 68
:2 0 6b :4 0 665
b30 b32 :3 0 668
b0a b34 :2 0 b36
66b b37 b07 b36
0 b38 66d 0
b64 60 :3 0 61
:3 0 b39 b3a 0
60 :3 0 71 :3 0
b3c b3d 0 23
:2 0 2 :4 0 f7
:2 0 73 :2 0 23
:2 0 66f b42 b44
:3 0 672 b3e b46
b3b b48 b47 :2 0
63 :3 0 74 :3 0
b4a b4b 0 65
:4 0 f8 :4 0 68
:2 0 69 :3 0 676
b4f b51 :3 0 679
b4c b53 :2 0 b55
67c b56 b49 b55
0 b57 67e 0
b64 81 :3 0 c1
:3 0 b58 b59 0
6d :3 0 59 :3 0
31 :2 0 680 b5b
b5e 17 :3 0 83
:3 0 683 b5a b62
:2 0 b64 687 b65
af6 b64 0 b67
a8f aed 0 b67
68b 0 b68 68e
b69 a4c b68 0
b6a 693 0 19d8
6d :3 0 37 :3 0
31 :2 0 695 b6b
b6e c8 :2 0 31
:2 0 69a b70 b72
:3 0 b73 :2 0 6d
:3 0 3b :3 0 31
:2 0 69d b75 b78
8b :2 0 ee :2 0
6a2 b7a b7c :3 0
b7d :2 0 b74 b7f
b7e :2 0 6d :3 0
3b :3 0 31 :2 0
6a5 b81 b84 84
:2 0 23 :2 0 6aa
b86 b88 :3 0 b89
:2 0 6d :3 0 3b
:3 0 31 :2 0 6ad
b8b b8e 84 :2 0
f9 :2 0 6b2 b90
b92 :3 0 b93 :2 0
b8a b95 b94 :2 0
3a :3 0 f9 :2 0
b97 b98 0 b9b
a5 :3 0 6b5 be4
6d :3 0 3b :3 0
31 :2 0 6b7 b9c
b9f 84 :2 0 31
:2 0 6bc ba1 ba3
:3 0 3a :3 0 23
:2 0 ba5 ba6 0
ba9 a5 :3 0 6bf
baa ba4 ba9 0
be5 6d :3 0 3b
:3 0 31 :2 0 6c1
bab bae 84 :2 0
fa :2 0 6c6 bb0
bb2 :3 0 bb3 :2 0
6d :3 0 3b :3 0
31 :2 0 6c9 bb5
bb8 84 :2 0 fb
:2 0 6ce bba bbc
:3 0 bbd :2 0 bb4
bbf bbe :2 0 3a
:3 0 23 :2 0 bc1
bc2 0 bc5 a5
:3 0 6d1 bc6 bc0
bc5 0 be5 6d
:3 0 3b :3 0 31
:2 0 6d3 bc7 bca
84 :2 0 85 :2 0
6d8 bcc bce :3 0
37 :3 0 6d :3 0
40 :3 0 31 :2 0
6db bd1 bd4 fc
:2 0 6d :3 0 37
:3 0 31 :2 0 6de
bd7 bda 6e1 bd6
bdc :3 0 bd0 bdd
0 be2 3a :3 0
fa :2 0 bdf be0
0 be2 6e4 be3
bcf be2 0 be5
b96 b9b 0 be5
6e7 0 be6 6ec
be7 b80 be6 0
be8 6ee 0 19d8
6d :3 0 39 :3 0
31 :2 0 6f0 be9
bec c8 :2 0 31
:2 0 6f5 bee bf0
:3 0 6d :3 0 42
:3 0 31 :2 0 6f8
bf2 bf5 c8 :2 0
fd :2 0 6fd bf7
bf9 :3 0 60 :3 0
61 :3 0 bfb bfc
0 60 :3 0 71
:3 0 bfe bff 0
18 :2 0 2 :4 0
fe :2 0 73 :2 0
23 :2 0 700 c04
c06 :3 0 703 c00
c08 bfd c0a c09
:2 0 63 :3 0 74
:3 0 c0c c0d 0
65 :4 0 ff :4 0
68 :2 0 69 :3 0
707 c11 c13 :3 0
68 :2 0 7b :4 0
70a c15 c17 :3 0
68 :2 0 7c :4 0
70d c19 c1b :3 0
68 :2 0 6d :3 0
6e :3 0 7d :3 0
100 :3 0 c20 c21
0 710 c1f c23
6f :4 0 712 c1e
c26 715 c1d c28
:3 0 68 :2 0 be
:4 0 718 c2a c2c
:3 0 68 :2 0 6b
:4 0 71b c2e c30
:3 0 68 :2 0 17
:3 0 71e c32 c34
:3 0 68 :2 0 6b
:4 0 721 c36 c38
:3 0 724 c0e c3a
:2 0 c3c 727 c3d
c0b c3c 0 c3e
729 0 c68 60
:3 0 61 :3 0 c3f
c40 0 60 :3 0
71 :3 0 c42 c43
0 23 :2 0 2
:4 0 101 :2 0 73
:2 0 23 :2 0 72b
c48 c4a :3 0 72e
c44 c4c c41 c4e
c4d :2 0 63 :3 0
74 :3 0 c50 c51
0 65 :4 0 102
:4 0 68 :2 0 69
:3 0 732 c55 c57
:3 0 735 c52 c59
:2 0 c5b 738 c5c
c4f c5b 0 c5d
73a 0 c68 81
:3 0 c1 :3 0 c5e
c5f 0 7d :3 0
100 :3 0 c61 c62
0 17 :3 0 83
:3 0 73c c60 c66
:2 0 c68 740 c69
bfa c68 0 c6a
744 0 c8d 6d
:3 0 42 :3 0 31
:2 0 746 c6b c6e
9f :2 0 ad :2 0
74b c70 c72 :3 0
42 :3 0 103 :2 0
c74 c75 0 c77
74e c8a 42 :3 0
104 :3 0 105 :3 0
6d :3 0 42 :3 0
31 :2 0 750 c7b
c7e 753 c7a c80
fc :2 0 106 :2 0
755 c82 c84 :3 0
758 c79 c86 c78
c87 0 c89 75a
c8b c73 c77 0
c8c 0 c89 0
c8c 75c 0 c8d
75f c8e bf1 c8d
0 c8f 762 0
19d8 55 :3 0 107
:4 0 c91 c92 :3 0
c90 c93 0 19d8
2a :3 0 ef :4 0
c95 c96 0 19d8
2b :3 0 ef :4 0
c98 c99 0 19d8
20 :3 0 ef :4 0
c9b c9c 0 19d8
26 :3 0 ef :4 0
c9e c9f 0 19d8
1b :3 0 9b :3 0
1b :3 0 23 :2 0
18 :2 0 764 ca2
ca6 ca1 ca7 0
19d8 24 :3 0 9b
:3 0 24 :3 0 23
:2 0 23 :2 0 768
caa cae ca9 caf
0 19d8 6d :3 0
33 :3 0 31 :2 0
76c cb1 cb4 c8
:2 0 31 :2 0 771
cb6 cb8 :3 0 2c
:4 0 cba cbb 0
13e6 1e :3 0 1c
:3 0 cbd cbe 0
13e6 9b :3 0 24
:3 0 23 :2 0 23
:2 0 774 cc0 cc4
84 :2 0 23 :4 0
77a cc6 cc8 :3 0
108 :3 0 109 :3 0
9b :3 0 10a :3 0
109 :3 0 9b :3 0
10b :3 0 10c :3 0
10d :3 0 10e :3 0
10f :3 0 27 :3 0
2c :3 0 2d :3 0
4a :3 0 4b :3 0
4d :3 0 4f :3 0
110 :3 0 111 :3 0
17 :3 0 112 :3 0
1b :3 0 113 :3 0
114 :4 0 115 1
:8 0 ce9 15 :3 0
92 :3 0 ce4 ce5
0 93 :3 0 ce6
ce7 0 ce9 77d
d3c 94 :3 0 15
:3 0 92 :3 0 ceb
cec 0 93 :3 0
ced cee 0 cf0
780 cf2 782 cf1
cf0 :2 0 d3a 95
:3 0 15 :3 0 92
:3 0 cf5 cf6 0
93 :3 0 cf7 cf8
0 d32 60 :3 0
61 :3 0 cfa cfb
0 60 :3 0 71
:3 0 cfd cfe 0
96 :2 0 2 :4 0
116 :2 0 73 :2 0
23 :2 0 784 d03
d05 :3 0 787 cff
d07 cfc d09 d08
:2 0 63 :3 0 74
:3 0 d0b d0c 0
65 :4 0 117 :4 0
68 :2 0 69 :3 0
78b d10 d12 :3 0
68 :2 0 7b :4 0
78e d14 d16 :3 0
68 :2 0 99 :4 0
791 d18 d1a :3 0
68 :2 0 93 :3 0
794 d1c d1e :3 0
68 :2 0 9a :4 0
797 d20 d22 :3 0
68 :2 0 9b :3 0
9c :3 0 23 :2 0
9d :2 0 79a d25
d29 79e d24 d2b
:3 0 7a1 d0d d2d
:2 0 d2f 7a4 d30
d0a d2f 0 d31
7a6 0 d32 7a8
d35 :3 0 d35 0
d35 d34 d32 d33
:6 0 d37 10 :3 0
7ab d39 7ad d38
d37 :2 0 d3a 7af
:2 0 d3c 0 d3c
d3b ce9 d3a :6 0
e81 1 :3 0 118
:3 0 2e :3 0 119
:3 0 11a :3 0 1b
:4 0 11b 1 :8 0
d49 15 :3 0 92
:3 0 d44 d45 0
93 :3 0 d46 d47
0 d49 7b2 d9c
94 :3 0 15 :3 0
92 :3 0 d4b d4c
0 93 :3 0 d4d
d4e 0 d50 7b5
d52 7b7 d51 d50
:2 0 d9a 95 :3 0
15 :3 0 92 :3 0
d55 d56 0 93
:3 0 d57 d58 0
d92 60 :3 0 61
:3 0 d5a d5b 0
60 :3 0 71 :3 0
d5d d5e 0 96
:2 0 2 :4 0 11c
:2 0 73 :2 0 23
:2 0 7b9 d63 d65
:3 0 7bc d5f d67
d5c d69 d68 :2 0
63 :3 0 74 :3 0
d6b d6c 0 65
:4 0 11d :4 0 68
:2 0 69 :3 0 7c0
d70 d72 :3 0 68
:2 0 7b :4 0 7c3
d74 d76 :3 0 68
:2 0 99 :4 0 7c6
d78 d7a :3 0 68
:2 0 93 :3 0 7c9
d7c d7e :3 0 68
:2 0 9a :4 0 7cc
d80 d82 :3 0 68
:2 0 9b :3 0 9c
:3 0 23 :2 0 9d
:2 0 7cf d85 d89
7d3 d84 d8b :3 0
7d6 d6d d8d :2 0
d8f 7d9 d90 d6a
d8f 0 d91 7db
0 d92 7dd d95
:3 0 d95 0 d95
d94 d92 d93 :6 0
d97 12 :3 0 7e0
d99 7e2 d98 d97
:2 0 d9a 7e4 :2 0
d9c 0 d9c d9b
d49 d9a :6 0 e81
1 :3 0 15 :3 0
92 :3 0 d9e d9f
0 15 :3 0 84
:2 0 b7 :3 0 da1
da3 0 7e9 da2
da5 :3 0 da6 :2 0
60 :3 0 61 :3 0
da8 da9 0 60
:3 0 71 :3 0 dab
dac 0 18 :2 0
2 :4 0 9d :2 0
73 :2 0 23 :2 0
7ec db1 db3 :3 0
7ef dad db5 daa
db7 db6 :2 0 63
:3 0 74 :3 0 db9
dba 0 65 :4 0
11e :4 0 68 :2 0
69 :3 0 7f3 dbe
dc0 :3 0 68 :2 0
7b :4 0 7f6 dc2
dc4 :3 0 68 :2 0
7c :4 0 7f9 dc6
dc8 :3 0 68 :2 0
6d :3 0 6e :3 0
7d :3 0 e7 :3 0
dcd dce 0 7fc
dcc dd0 6f :4 0
7fe dcb dd3 801
dca dd5 :3 0 68
:2 0 be :4 0 804
dd7 dd9 :3 0 68
:2 0 6b :4 0 807
ddb ddd :3 0 68
:2 0 1b :3 0 80a
ddf de1 :3 0 68
:2 0 6b :4 0 80d
de3 de5 :3 0 810
dbb de7 :2 0 de9
813 dea db8 de9
0 deb 815 0
e15 60 :3 0 61
:3 0 dec ded 0
60 :3 0 71 :3 0
def df0 0 23
:2 0 2 :4 0 11f
:2 0 73 :2 0 23
:2 0 817 df5 df7
:3 0 81a df1 df9
dee dfb dfa :2 0
63 :3 0 74 :3 0
dfd dfe 0 65
:4 0 120 :4 0 68
:2 0 69 :3 0 81e
e02 e04 :3 0 821
dff e06 :2 0 e08
824 e09 dfc e08
0 e0a 826 0
e15 81 :3 0 c1
:3 0 e0b e0c 0
7d :3 0 e7 :3 0
e0e e0f 0 1b
:3 0 83 :3 0 828
e0d e13 :2 0 e15
82c e16 da7 e15
0 e17 830 0
e81 2b :3 0 1b
:3 0 e18 e19 0
e81 121 :3 0 122
:3 0 26 :3 0 43
:3 0 123 :3 0 b3
:3 0 19 :3 0 113
:3 0 112 :3 0 1b
:4 0 124 1 :8 0
e2b 15 :3 0 92
:3 0 e26 e27 0
93 :3 0 e28 e29
0 e2b 832 e7e
94 :3 0 15 :3 0
92 :3 0 e2d e2e
0 93 :3 0 e2f
e30 0 e32 835
e34 837 e33 e32
:2 0 e7c 95 :3 0
15 :3 0 92 :3 0
e37 e38 0 93
:3 0 e39 e3a 0
e74 60 :3 0 61
:3 0 e3c e3d 0
60 :3 0 71 :3 0
e3f e40 0 96
:2 0 2 :4 0 125
:2 0 73 :2 0 23
:2 0 839 e45 e47
:3 0 83c e41 e49
e3e e4b e4a :2 0
63 :3 0 74 :3 0
e4d e4e 0 65
:4 0 126 :4 0 68
:2 0 69 :3 0 840
e52 e54 :3 0 68
:2 0 7b :4 0 843
e56 e58 :3 0 68
:2 0 99 :4 0 846
e5a e5c :3 0 68
:2 0 93 :3 0 849
e5e e60 :3 0 68
:2 0 9a :4 0 84c
e62 e64 :3 0 68
:2 0 9b :3 0 9c
:3 0 23 :2 0 9d
:2 0 84f e67 e6b
853 e66 e6d :3 0
856 e4f e6f :2 0
e71 859 e72 e4c
e71 0 e73 85b
0 e74 85d e77
:3 0 e77 0 e77
e76 e74 e75 :6 0
e79 14 :3 0 860
e7b 862 e7a e79
:2 0 e7c 864 :2 0
e7e 0 e7e e7d
e2b e7c :6 0 e81
1 :3 0 a5 :3 0
867 1122 9b :3 0
24 :3 0 23 :2 0
23 :2 0 86d e82
e86 84 :2 0 31
:4 0 873 e88 e8a
:3 0 127 :3 0 2e
:3 0 128 :3 0 112
:3 0 1b :4 0 129
1 :8 0 e97 15
:3 0 92 :3 0 e92
e93 0 93 :3 0
e94 e95 0 e97
876 eea 94 :3 0
15 :3 0 92 :3 0
e99 e9a 0 93
:3 0 e9b e9c 0
e9e 879 ea0 87b
e9f e9e :2 0 ee8
95 :3 0 15 :3 0
92 :3 0 ea3 ea4
0 93 :3 0 ea5
ea6 0 ee0 60
:3 0 61 :3 0 ea8
ea9 0 60 :3 0
71 :3 0 eab eac
0 96 :2 0 2
:4 0 12a :2 0 73
:2 0 23 :2 0 87d
eb1 eb3 :3 0 880
ead eb5 eaa eb7
eb6 :2 0 63 :3 0
74 :3 0 eb9 eba
0 65 :4 0 12b
:4 0 68 :2 0 69
:3 0 884 ebe ec0
:3 0 68 :2 0 7b
:4 0 887 ec2 ec4
:3 0 68 :2 0 99
:4 0 88a ec6 ec8
:3 0 68 :2 0 93
:3 0 88d eca ecc
:3 0 68 :2 0 9a
:4 0 890 ece ed0
:3 0 68 :2 0 9b
:3 0 9c :3 0 23
:2 0 9d :2 0 893
ed3 ed7 897 ed2
ed9 :3 0 89a ebb
edb :2 0 edd 89d
ede eb8 edd 0
edf 89f 0 ee0
8a1 ee3 :3 0 ee3
0 ee3 ee2 ee0
ee1 :6 0 ee5 16
:3 0 8a4 ee7 8a6
ee6 ee5 :2 0 ee8
8a8 :2 0 eea 0
eea ee9 e97 ee8
:6 0 10c3 1 :3 0
15 :3 0 92 :3 0
eec eed 0 15
:3 0 84 :2 0 b7
:3 0 eef ef1 0
8ad ef0 ef3 :3 0
ef4 :2 0 60 :3 0
61 :3 0 ef6 ef7
0 60 :3 0 71
:3 0 ef9 efa 0
18 :2 0 2 :4 0
12c :2 0 73 :2 0
23 :2 0 8b0 eff
f01 :3 0 8b3 efb
f03 ef8 f05 f04
:2 0 63 :3 0 74
:3 0 f07 f08 0
65 :4 0 12d :4 0
68 :2 0 69 :3 0
8b7 f0c f0e :3 0
68 :2 0 7b :4 0
8ba f10 f12 :3 0
68 :2 0 7c :4 0
8bd f14 f16 :3 0
68 :2 0 6d :3 0
6e :3 0 7d :3 0
e7 :3 0 f1b f1c
0 8c0 f1a f1e
6f :4 0 8c2 f19
f21 8c5 f18 f23
:3 0 68 :2 0 be
:4 0 8c8 f25 f27
:3 0 68 :2 0 6b
:4 0 8cb f29 f2b
:3 0 68 :2 0 1b
:3 0 8ce f2d f2f
:3 0 68 :2 0 6b
:4 0 8d1 f31 f33
:3 0 8d4 f09 f35
:2 0 f37 8d7 f38
f06 f37 0 f39
8d9 0 f63 60
:3 0 61 :3 0 f3a
f3b 0 60 :3 0
71 :3 0 f3d f3e
0 23 :2 0 2
:4 0 12e :2 0 73
:2 0 23 :2 0 8db
f43 f45 :3 0 8de
f3f f47 f3c f49
f48 :2 0 63 :3 0
74 :3 0 f4b f4c
0 65 :4 0 12f
:4 0 68 :2 0 69
:3 0 8e2 f50 f52
:3 0 8e5 f4d f54
:2 0 f56 8e8 f57
f4a f56 0 f58
8ea 0 f63 81
:3 0 c1 :3 0 f59
f5a 0 7d :3 0
e7 :3 0 f5c f5d
0 1b :3 0 83
:3 0 8ec f5b f61
:2 0 f63 8f0 f64
ef5 f63 0 f65
8f4 0 10c3 2a
:3 0 1b :3 0 f66
f67 0 10c3 108
:3 0 109 :3 0 9b
:3 0 10a :3 0 109
:3 0 9b :3 0 10b
:3 0 10c :3 0 10d
:3 0 10e :3 0 10f
:3 0 27 :3 0 2c
:3 0 2d :3 0 4a
:3 0 4b :3 0 4d
:3 0 4f :3 0 110
:3 0 111 :3 0 17
:3 0 112 :3 0 1b
:3 0 113 :3 0 114
:4 0 130 1 :8 0
f88 15 :3 0 92
:3 0 f83 f84 0
93 :3 0 f85 f86
0 f88 8f6 fdb
94 :3 0 15 :3 0
92 :3 0 f8a f8b
0 93 :3 0 f8c
f8d 0 f8f 8f9
f91 8fb f90 f8f
:2 0 fd9 95 :3 0
15 :3 0 92 :3 0
f94 f95 0 93
:3 0 f96 f97 0
fd1 60 :3 0 61
:3 0 f99 f9a 0
60 :3 0 71 :3 0
f9c f9d 0 96
:2 0 2 :4 0 131
:2 0 73 :2 0 23
:2 0 8fd fa2 fa4
:3 0 900 f9e fa6
f9b fa8 fa7 :2 0
63 :3 0 74 :3 0
faa fab 0 65
:4 0 132 :4 0 68
:2 0 69 :3 0 904
faf fb1 :3 0 68
:2 0 7b :4 0 907
fb3 fb5 :3 0 68
:2 0 99 :4 0 90a
fb7 fb9 :3 0 68
:2 0 93 :3 0 90d
fbb fbd :3 0 68
:2 0 9a :4 0 910
fbf fc1 :3 0 68
:2 0 9b :3 0 9c
:3 0 23 :2 0 9d
:2 0 913 fc4 fc8
917 fc3 fca :3 0
91a fac fcc :2 0
fce 91d fcf fa9
fce 0 fd0 91f
0 fd1 921 fd4
:3 0 fd4 0 fd4
fd3 fd1 fd2 :6 0
fd6 18 :3 0 924
fd8 926 fd7 fd6
:2 0 fd9 928 :2 0
fdb 0 fdb fda
f88 fd9 :6 0 10c3
1 :3 0 15 :3 0
92 :3 0 fdd fde
0 15 :3 0 84
:2 0 b7 :3 0 fe0
fe2 0 92d fe1
fe4 :3 0 fe5 :2 0
108 :3 0 109 :3 0
9b :3 0 10a :3 0
109 :3 0 9b :3 0
10b :3 0 10c :3 0
10d :3 0 10e :3 0
10f :3 0 27 :3 0
2c :3 0 2d :3 0
4a :3 0 4b :3 0
4d :3 0 4f :3 0
110 :3 0 111 :3 0
17 :3 0 112 :3 0
1b :3 0 113 :3 0
114 :4 0 133 1
:8 0 1006 15 :3 0
92 :3 0 1001 1002
0 93 :3 0 1003
1004 0 1006 930
1059 94 :3 0 15
:3 0 92 :3 0 1008
1009 0 93 :3 0
100a 100b 0 100d
933 100f 935 100e
100d :2 0 1057 95
:3 0 15 :3 0 92
:3 0 1012 1013 0
93 :3 0 1014 1015
0 104f 60 :3 0
61 :3 0 1017 1018
0 60 :3 0 71
:3 0 101a 101b 0
96 :2 0 2 :4 0
134 :2 0 73 :2 0
23 :2 0 937 1020
1022 :3 0 93a 101c
1024 1019 1026 1025
:2 0 63 :3 0 74
:3 0 1028 1029 0
65 :4 0 135 :4 0
68 :2 0 69 :3 0
93e 102d 102f :3 0
68 :2 0 7b :4 0
941 1031 1033 :3 0
68 :2 0 99 :4 0
944 1035 1037 :3 0
68 :2 0 93 :3 0
947 1039 103b :3 0
68 :2 0 9a :4 0
94a 103d 103f :3 0
68 :2 0 9b :3 0
9c :3 0 23 :2 0
9d :2 0 94d 1042
1046 951 1041 1048
:3 0 954 102a 104a
:2 0 104c 957 104d
1027 104c 0 104e
959 0 104f 95b
1052 :3 0 1052 0
1052 1051 104f 1050
:6 0 1054 1a :3 0
95e 1056 960 1055
1054 :2 0 1057 962
:2 0 1059 0 1059
1058 1006 1057 :6 0
105b 1 :3 0 965
105c fe6 105b 0
105d 967 0 10c3
121 :3 0 122 :3 0
26 :3 0 43 :3 0
123 :3 0 b3 :3 0
19 :3 0 113 :3 0
112 :3 0 1b :4 0
136 1 :8 0 106e
15 :3 0 92 :3 0
1069 106a 0 93
:3 0 106b 106c 0
106e 969 10c1 94
:3 0 15 :3 0 92
:3 0 1070 1071 0
93 :3 0 1072 1073
0 1075 96c 1077
96e 1076 1075 :2 0
10bf 95 :3 0 15
:3 0 92 :3 0 107a
107b 0 93 :3 0
107c 107d 0 10b7
60 :3 0 61 :3 0
107f 1080 0 60
:3 0 71 :3 0 1082
1083 0 96 :2 0
2 :4 0 137 :2 0
73 :2 0 23 :2 0
970 1088 108a :3 0
973 1084 108c 1081
108e 108d :2 0 63
:3 0 74 :3 0 1090
1091 0 65 :4 0
138 :4 0 68 :2 0
69 :3 0 977 1095
1097 :3 0 68 :2 0
7b :4 0 97a 1099
109b :3 0 68 :2 0
99 :4 0 97d 109d
109f :3 0 68 :2 0
93 :3 0 980 10a1
10a3 :3 0 68 :2 0
9a :4 0 983 10a5
10a7 :3 0 68 :2 0
9b :3 0 9c :3 0
23 :2 0 9d :2 0
986 10aa 10ae 98a
10a9 10b0 :3 0 98d
1092 10b2 :2 0 10b4
990 10b5 108f 10b4
0 10b6 992 0
10b7 994 10ba :3 0
10ba 0 10ba 10b9
10b7 10b8 :6 0 10bc
1c :3 0 997 10be
999 10bd 10bc :2 0
10bf 99b :2 0 10c1
0 10c1 10c0 106e
10bf :6 0 10c3 1
:3 0 99e 10c4 e8b
10c3 0 1124 60
:3 0 61 :3 0 10c5
10c6 0 60 :3 0
71 :3 0 10c8 10c9
0 78 :2 0 2
:4 0 139 :2 0 73
:2 0 23 :2 0 9a5
10ce 10d0 :3 0 9a8
10ca 10d2 10c7 10d4
10d3 :2 0 63 :3 0
74 :3 0 10d6 10d7
0 65 :4 0 13a
:4 0 68 :2 0 69
:3 0 9ac 10db 10dd
:3 0 68 :2 0 7b
:4 0 9af 10df 10e1
:3 0 68 :2 0 7c
:4 0 9b2 10e3 10e5
:3 0 68 :2 0 6d
:3 0 6e :3 0 7d
:3 0 88 :3 0 10ea
10eb 0 9b5 10e9
10ed 6f :4 0 9b7
10e8 10f0 9ba 10e7
10f2 :3 0 9bd 10d8
10f4 :2 0 10f6 9c0
10f7 10d5 10f6 0
10f8 9c2 0 1121
60 :3 0 61 :3 0
10f9 10fa 0 60
:3 0 71 :3 0 10fc
10fd 0 23 :2 0
2 :4 0 13b :2 0
73 :2 0 23 :2 0
9c4 1102 1104 :3 0
9c7 10fe 1106 10fb
1108 1107 :2 0 63
:3 0 74 :3 0 110a
110b 0 65 :4 0
13c :4 0 68 :2 0
69 :3 0 9cb 110f
1111 :3 0 9ce 110c
1113 :2 0 1115 9d1
1116 1109 1115 0
1117 9d3 0 1121
81 :3 0 82 :3 0
1118 1119 0 7d
:3 0 88 :3 0 111b
111c 0 83 :3 0
9d5 111a 111f :2 0
1121 9d8 1123 cc9
e81 0 1124 0
1121 0 1124 9dc
0 13e6 15 :3 0
92 :3 0 1125 1126
0 15 :3 0 84
:2 0 9e :3 0 1128
112a 0 9e2 1129
112c :3 0 112d :2 0
55 :3 0 13d :3 0
31 :2 0 6d :3 0
33 :3 0 31 :2 0
9e5 1132 1135 7d
:3 0 13e :3 0 1137
1138 0 51 :3 0
9e8 1130 113b 112f
113c 0 11e3 55
:3 0 13d :3 0 31
:2 0 6d :3 0 33
:3 0 31 :2 0 9ed
1141 1144 7d :3 0
13f :3 0 1146 1147
0 53 :3 0 9f0
113f 114a 113e 114b
0 11e3 123 :3 0
122 :3 0 140 :3 0
53 :3 0 141 :3 0
7d :3 0 dd :3 0
7d :3 0 de :3 0
7d :3 0 df :3 0
7d :3 0 e0 :3 0
7d :3 0 e1 :3 0
7d :3 0 142 :3 0
140 :3 0 53 :3 0
7d :3 0 143 :3 0
140 :3 0 51 :3 0
7d :3 0 142 :3 0
d8 :3 0 144 :3 0
122 :3 0 7d :3 0
143 :3 0 140 :3 0
140 :3 0 53 :3 0
51 :3 0 140 :3 0
121 :3 0 26 :4 0
145 1 :8 0 1188
146 :4 0 1173 :3 0
83 :3 0 84 :2 0
9f7 1176 1177 :3 0
15 :3 0 92 :3 0
1179 117a 0 31
:2 0 117b 117c 0
117e 9fa 1185 15
:3 0 92 :3 0 117f
1180 0 ee :2 0
1181 1182 0 1184
9fc 1186 1178 117e
0 1187 0 1184
0 1187 9fe 0
1188 a01 11db 94
:3 0 15 :3 0 92
:3 0 118a 118b 0
93 :3 0 118c 118d
0 118f a04 1191
a06 1190 118f :2 0
11d9 95 :3 0 15
:3 0 92 :3 0 1194
1195 0 93 :3 0
1196 1197 0 11d1
60 :3 0 61 :3 0
1199 119a 0 60
:3 0 71 :3 0 119c
119d 0 96 :2 0
2 :4 0 147 :2 0
73 :2 0 23 :2 0
a08 11a2 11a4 :3 0
a0b 119e 11a6 119b
11a8 11a7 :2 0 63
:3 0 74 :3 0 11aa
11ab 0 65 :4 0
148 :4 0 68 :2 0
69 :3 0 a0f 11af
11b1 :3 0 68 :2 0
7b :4 0 a12 11b3
11b5 :3 0 68 :2 0
99 :4 0 a15 11b7
11b9 :3 0 68 :2 0
93 :3 0 a18 11bb
11bd :3 0 68 :2 0
9a :4 0 a1b 11bf
11c1 :3 0 68 :2 0
9b :3 0 9c :3 0
23 :2 0 9d :2 0
a1e 11c4 11c8 a22
11c3 11ca :3 0 a25
11ac 11cc :2 0 11ce
a28 11cf 11a9 11ce
0 11d0 a2a 0
11d1 a2c 11d4 :3 0
11d4 0 11d4 11d3
11d1 11d2 :6 0 11d6
1e :3 0 a2f 11d8
a31 11d7 11d6 :2 0
11d9 a33 :2 0 11db
0 11db 11da 1188
11d9 :6 0 11e3 1
:3 0 55 :3 0 149
:3 0 26 :3 0 a36
11de 11e0 11dd 11e1
0 11e3 a38 1290
26 :3 0 14a :4 0
11e5 11e6 :3 0 11e4
11e7 0 128f 55
:3 0 13d :3 0 31
:2 0 6d :3 0 33
:3 0 31 :2 0 a3d
11ec 11ef 7d :3 0
13e :3 0 11f1 11f2
0 51 :3 0 a40
11ea 11f5 11e9 11f6
0 128f 50 :3 0
7d :3 0 142 :3 0
11f9 11fa 0 11f8
11fb 0 128f 54
:3 0 23 :2 0 11fd
11fe 0 128f 55
:3 0 14b :3 0 17
:3 0 1b :3 0 33
:3 0 50 :3 0 51
:3 0 24 :3 0 54
:3 0 a45 1201 1209
1200 120a 0 128f
123 :3 0 121 :3 0
c4 :3 0 b3 :3 0
141 :3 0 14c :3 0
113 :3 0 112 :3 0
122 :3 0 140 :3 0
14d :3 0 14e :3 0
26 :3 0 17 :3 0
19 :3 0 33 :3 0
58 :3 0 24 :3 0
1b :3 0 7d :3 0
142 :3 0 51 :3 0
13 :3 0 69 :4 0
14f 1 :8 0 123a
146 :4 0 1225 :3 0
83 :3 0 84 :2 0
a4f 1228 1229 :3 0
15 :3 0 92 :3 0
122b 122c 0 31
:2 0 122d 122e 0
1230 a52 1237 15
:3 0 92 :3 0 1231
1232 0 ee :2 0
1233 1234 0 1236
a54 1238 122a 1230
0 1239 0 1236
0 1239 a56 0
123a a59 128d 94
:3 0 15 :3 0 92
:3 0 123c 123d 0
93 :3 0 123e 123f
0 1241 a5c 1243
a5e 1242 1241 :2 0
128b 95 :3 0 15
:3 0 92 :3 0 1246
1247 0 93 :3 0
1248 1249 0 1283
60 :3 0 61 :3 0
124b 124c 0 60
:3 0 71 :3 0 124e
124f 0 96 :2 0
2 :4 0 150 :2 0
73 :2 0 23 :2 0
a60 1254 1256 :3 0
a63 1250 1258 124d
125a 1259 :2 0 63
:3 0 74 :3 0 125c
125d 0 65 :4 0
151 :4 0 68 :2 0
69 :3 0 a67 1261
1263 :3 0 68 :2 0
7b :4 0 a6a 1265
1267 :3 0 68 :2 0
99 :4 0 a6d 1269
126b :3 0 68 :2 0
93 :3 0 a70 126d
126f :3 0 68 :2 0
9a :4 0 a73 1271
1273 :3 0 68 :2 0
9b :3 0 9c :3 0
23 :2 0 9d :2 0
a76 1276 127a a7a
1275 127c :3 0 a7d
125e 127e :2 0 1280
a80 1281 125b 1280
0 1282 a82 0
1283 a84 1286 :3 0
1286 0 1286 1285
1283 1284 :6 0 1288
20 :3 0 a87 128a
a89 1289 1288 :2 0
128b a8b :2 0 128d
0 128d 128c 123a
128b :6 0 128f 1
:3 0 a8e 1291 112e
11e3 0 1292 0
128f 0 1292 a95
0 13e6 6d :3 0
35 :3 0 31 :2 0
a98 1293 1296 84
:2 0 31 :2 0 a9d
1298 129a :3 0 129b
:2 0 2c :3 0 8d
:2 0 aa0 129e 129f
:3 0 12a0 :2 0 12a1
:2 0 129c 12a3 12a2
:2 0 152 :3 0 25
:3 0 c3 :3 0 c4
:3 0 17 :4 0 153
1 :8 0 12b0 15
:3 0 92 :3 0 12ab
12ac 0 93 :3 0
12ad 12ae 0 12b0
aa2 1303 94 :3 0
15 :3 0 92 :3 0
12b2 12b3 0 93
:3 0 12b4 12b5 0
12b7 aa5 12b9 aa7
12b8 12b7 :2 0 1301
95 :3 0 15 :3 0
92 :3 0 12bc 12bd
0 93 :3 0 12be
12bf 0 12f9 60
:3 0 61 :3 0 12c1
12c2 0 60 :3 0
71 :3 0 12c4 12c5
0 96 :2 0 2
:4 0 154 :2 0 73
:2 0 23 :2 0 aa9
12ca 12cc :3 0 aac
12c6 12ce 12c3 12d0
12cf :2 0 63 :3 0
74 :3 0 12d2 12d3
0 65 :4 0 155
:4 0 68 :2 0 69
:3 0 ab0 12d7 12d9
:3 0 68 :2 0 7b
:4 0 ab3 12db 12dd
:3 0 68 :2 0 99
:4 0 ab6 12df 12e1
:3 0 68 :2 0 93
:3 0 ab9 12e3 12e5
:3 0 68 :2 0 9a
:4 0 abc 12e7 12e9
:3 0 68 :2 0 9b
:3 0 9c :3 0 23
:2 0 9d :2 0 abf
12ec 12f0 ac3 12eb
12f2 :3 0 ac6 12d4
12f4 :2 0 12f6 ac9
12f7 12d1 12f6 0
12f8 acb 0 12f9
acd 12fc :3 0 12fc
0 12fc 12fb 12f9
12fa :6 0 12fe 22
:3 0 ad0 1300 ad2
12ff 12fe :2 0 1301
ad4 :2 0 1303 0
1303 1302 12b0 1301
:6 0 1329 1 :3 0
9b :3 0 25 :3 0
23 :2 0 23 :2 0
ad7 1305 1309 84
:2 0 156 :4 0 add
130b 130d :3 0 4c
:3 0 157 :2 0 23
:2 0 ae0 1310 1312
:3 0 130f 1313 0
1326 15 :3 0 92
:3 0 1315 1316 0
158 :3 0 17 :3 0
17 :3 0 27 :3 0
2c :3 0 2d :3 0
4d :3 0 4f :3 0
4c :3 0 4b :3 0
4a :3 0 ae2 1318
1323 1317 1324 0
1326 aed 1327 130e
1326 0 1328 af0
0 1329 af2 132a
12a4 1329 0 132b
af5 0 13e6 1e
:3 0 1e :3 0 132c
132d 0 13e6 9b
:3 0 24 :3 0 23
:2 0 23 :2 0 af7
132f 1333 84 :2 0
23 :4 0 afd 1335
1337 :3 0 45 :3 0
7d :3 0 159 :3 0
133a 133b 0 1339
133c 0 133e b00
1347 45 :3 0 6d
:3 0 3c :3 0 31
:2 0 b02 1340 1343
133f 1344 0 1346
b05 1348 1338 133e
0 1349 0 1346
0 1349 b07 0
13e6 5c :3 0 15a
:3 0 26 :3 0 17
:3 0 19 :3 0 1b
:3 0 48 :3 0 33
:3 0 20 :3 0 20
:3 0 20 :3 0 20
:3 0 1e :3 0 1f
:3 0 42 :3 0 39
:3 0 37 :3 0 3a
:3 0 3b :3 0 36
:3 0 45 :3 0 24
:3 0 35 :3 0 32
:3 0 3d :3 0 38
:3 0 34 :3 0 3f
:3 0 46 :3 0 1a
:3 0 31 :2 0 31
:2 0 4e :2 0 4e
:2 0 b0a 134b 136c
134a 136d 0 13e6
5c :3 0 84 :2 0
31 :2 0 b2d 1370
1372 :3 0 15b :3 0
15c :3 0 15d :3 0
1b :4 0 15e 1
:8 0 138e 146 :4 0
1379 :3 0 83 :3 0
84 :2 0 b32 137c
137d :3 0 15 :3 0
92 :3 0 137f 1380
0 31 :2 0 1381
1382 0 1384 b35
138b 15 :3 0 92
:3 0 1385 1386 0
ee :2 0 1387 1388
0 138a b37 138c
137e 1384 0 138d
0 138a 0 138d
b39 0 138e b3c
13e1 94 :3 0 15
:3 0 92 :3 0 1390
1391 0 93 :3 0
1392 1393 0 1395
b3f 1397 b41 1396
1395 :2 0 13df 95
:3 0 15 :3 0 92
:3 0 139a 139b 0
93 :3 0 139c 139d
0 13d7 60 :3 0
61 :3 0 139f 13a0
0 60 :3 0 71
:3 0 13a2 13a3 0
96 :2 0 2 :4 0
15f :2 0 73 :2 0
23 :2 0 b43 13a8
13aa :3 0 b46 13a4
13ac 13a1 13ae 13ad
:2 0 63 :3 0 74
:3 0 13b0 13b1 0
65 :4 0 160 :4 0
68 :2 0 69 :3 0
b4a 13b5 13b7 :3 0
68 :2 0 7b :4 0
b4d 13b9 13bb :3 0
68 :2 0 99 :4 0
b50 13bd 13bf :3 0
68 :2 0 93 :3 0
b53 13c1 13c3 :3 0
68 :2 0 9a :4 0
b56 13c5 13c7 :3 0
68 :2 0 9b :3 0
9c :3 0 23 :2 0
9d :2 0 b59 13ca
13ce b5d 13c9 13d0
:3 0 b60 13b2 13d2
:2 0 13d4 b63 13d5
13af 13d4 0 13d6
b65 0 13d7 b67
13da :3 0 13da 0
13da 13d9 13d7 13d8
:6 0 13dc 24 :3 0
b6a 13de b6c 13dd
13dc :2 0 13df b6e
:2 0 13e1 0 13e1
13e0 138e 13df :6 0
13e3 1 :3 0 b71
13e4 1373 13e3 0
13e5 b73 0 13e6
b75 1710 2a :3 0
ef :4 0 13e7 13e8
0 170f 2b :3 0
ef :4 0 13ea 13eb
0 170f 28 :3 0
12 :3 0 13ed 13ee
0 170f 6d :3 0
161 :3 0 28 :3 0
b7f 13f1 13f3 31
:2 0 b81 13f0 13f6
9f :2 0 18 :2 0
b86 13f8 13fa :3 0
5e :3 0 31 :2 0
13fc 13fd 0 13ff
b89 1417 5e :3 0
104 :3 0 6d :3 0
161 :3 0 28 :3 0
b8b 1403 1405 31
:2 0 b8d 1402 1408
73 :2 0 23 :2 0
b90 140a 140c :3 0
140d :2 0 162 :2 0
da :2 0 b93 140f
1411 :3 0 b96 1401
1413 1400 1414 0
1416 b98 1418 13fb
13ff 0 1419 0
1416 0 1419 b9a
0 170f 5e :3 0
c8 :2 0 31 :2 0
b9f 141b 141d :3 0
30 :3 0 31 :2 0
141f 1420 0 170c
163 :3 0 6d :3 0
30 :3 0 31 :2 0
ba2 1423 1426 5e
:3 0 9f :2 0 164
:3 0 ba7 1429 142b
:3 0 142c 170b 1b
:3 0 d8 :3 0 d9
:3 0 142f 1430 0
18 :2 0 18 :2 0
6d :3 0 30 :3 0
31 :2 0 baa 1434
1437 fc :2 0 da
:2 0 bad 1439 143b
:3 0 73 :2 0 23
:2 0 bb0 143d 143f
:3 0 28 :3 0 bb3
1431 1442 142e 1443
0 1709 24 :3 0
d8 :3 0 d9 :3 0
1446 1447 0 23
:2 0 23 :2 0 6d
:3 0 30 :3 0 31
:2 0 bb8 144b 144e
fc :2 0 da :2 0
bbb 1450 1452 :3 0
73 :2 0 18 :2 0
bbe 1454 1456 :3 0
73 :2 0 23 :2 0
bc1 1458 145a :3 0
28 :3 0 bc4 1448
145d 1445 145e 0
1709 9b :3 0 24
:3 0 23 :2 0 23
:2 0 bc9 1460 1464
84 :2 0 23 :4 0
bcf 1466 1468 :3 0
45 :3 0 7d :3 0
159 :3 0 146b 146c
0 146a 146d 0
149b 6d :3 0 161
:3 0 2b :3 0 bd2
1470 1472 31 :2 0
bd4 146f 1475 84
:2 0 31 :2 0 bd9
1477 1479 :3 0 2b
:3 0 1b :3 0 147b
147c 0 147e bdc
1498 2b :3 0 9b
:3 0 2b :3 0 68
:2 0 165 :4 0 bde
1482 1484 :3 0 23
:2 0 29 :2 0 be1
1480 1488 147f 1489
0 1497 2b :3 0
9b :3 0 2b :3 0
68 :2 0 1b :3 0
be5 148e 1490 :3 0
23 :2 0 29 :2 0
be8 148c 1494 148b
1495 0 1497 bec
1499 147a 147e 0
149a 0 1497 0
149a bef 0 149b
bf2 14d0 45 :3 0
6d :3 0 3c :3 0
31 :2 0 bf5 149d
14a0 149c 14a1 0
14cf 6d :3 0 161
:3 0 2a :3 0 bf8
14a4 14a6 31 :2 0
bfa 14a3 14a9 84
:2 0 31 :2 0 bff
14ab 14ad :3 0 2a
:3 0 1b :3 0 14af
14b0 0 14b2 c02
14cc 2a :3 0 9b
:3 0 2a :3 0 68
:2 0 165 :4 0 c04
14b6 14b8 :3 0 23
:2 0 29 :2 0 c07
14b4 14bc 14b3 14bd
0 14cb 2a :3 0
9b :3 0 2a :3 0
68 :2 0 1b :3 0
c0b 14c2 14c4 :3 0
23 :2 0 29 :2 0
c0e 14c0 14c8 14bf
14c9 0 14cb c12
14cd 14ae 14b2 0
14ce 0 14cb 0
14ce c15 0 14cf
c18 14d1 1469 149b
0 14d2 0 14cf
0 14d2 c1b 0
1709 9b :3 0 24
:3 0 23 :2 0 23
:2 0 c1e 14d3 14d7
84 :2 0 23 :4 0
c24 14d9 14db :3 0
109 :3 0 9b :3 0
1c :3 0 109 :3 0
9b :3 0 166 :3 0
109 :3 0 9b :3 0
167 :3 0 166 :3 0
168 :3 0 1c :3 0
1e :3 0 110 :3 0
111 :3 0 17 :3 0
112 :3 0 1b :3 0
113 :4 0 169 1
:8 0 14f6 15 :3 0
92 :3 0 14f1 14f2
0 93 :3 0 14f3
14f4 0 14f6 c27
1549 94 :3 0 15
:3 0 92 :3 0 14f8
14f9 0 93 :3 0
14fa 14fb 0 14fd
c2a 14ff c2c 14fe
14fd :2 0 1547 95
:3 0 15 :3 0 92
:3 0 1502 1503 0
93 :3 0 1504 1505
0 153f 60 :3 0
61 :3 0 1507 1508
0 60 :3 0 71
:3 0 150a 150b 0
96 :2 0 2 :4 0
16a :2 0 73 :2 0
23 :2 0 c2e 1510
1512 :3 0 c31 150c
1514 1509 1516 1515
:2 0 63 :3 0 74
:3 0 1518 1519 0
65 :4 0 16b :4 0
68 :2 0 69 :3 0
c35 151d 151f :3 0
68 :2 0 7b :4 0
c38 1521 1523 :3 0
68 :2 0 99 :4 0
c3b 1525 1527 :3 0
68 :2 0 93 :3 0
c3e 1529 152b :3 0
68 :2 0 9a :4 0
c41 152d 152f :3 0
68 :2 0 9b :3 0
9c :3 0 23 :2 0
9d :2 0 c44 1532
1536 c48 1531 1538
:3 0 c4b 151a 153a
:2 0 153c c4e 153d
1517 153c 0 153e
c50 0 153f c52
1542 :3 0 1542 0
1542 1541 153f 1540
:6 0 1544 27 :3 0
c55 1546 c57 1545
1544 :2 0 1547 c59
:2 0 1549 0 1549
1548 14f6 1547 :6 0
1551 26 :3 0 44
:3 0 31 :2 0 154b
154c 0 1551 35
:3 0 31 :2 0 154e
154f 0 1551 c5c
163e 109 :3 0 9b
:3 0 1c :3 0 109
:3 0 9b :3 0 166
:3 0 109 :3 0 9b
:3 0 167 :3 0 166
:3 0 168 :3 0 1c
:3 0 1e :3 0 110
:3 0 111 :3 0 17
:3 0 112 :3 0 1b
:3 0 113 :3 0 114
:4 0 16c 1 :8 0
156c 15 :3 0 92
:3 0 1567 1568 0
93 :3 0 1569 156a
0 156c c60 15bf
94 :3 0 15 :3 0
92 :3 0 156e 156f
0 93 :3 0 1570
1571 0 1573 c63
1575 c65 1574 1573
:2 0 15bd 95 :3 0
15 :3 0 92 :3 0
1578 1579 0 93
:3 0 157a 157b 0
15b5 60 :3 0 61
:3 0 157d 157e 0
60 :3 0 71 :3 0
1580 1581 0 96
:2 0 2 :4 0 16d
:2 0 73 :2 0 23
:2 0 c67 1586 1588
:3 0 c6a 1582 158a
157f 158c 158b :2 0
63 :3 0 74 :3 0
158e 158f 0 65
:4 0 16e :4 0 68
:2 0 69 :3 0 c6e
1593 1595 :3 0 68
:2 0 7b :4 0 c71
1597 1599 :3 0 68
:2 0 99 :4 0 c74
159b 159d :3 0 68
:2 0 93 :3 0 c77
159f 15a1 :3 0 68
:2 0 9a :4 0 c7a
15a3 15a5 :3 0 68
:2 0 9b :3 0 9c
:3 0 23 :2 0 9d
:2 0 c7d 15a8 15ac
c81 15a7 15ae :3 0
c84 1590 15b0 :2 0
15b2 c87 15b3 158d
15b2 0 15b4 c89
0 15b5 c8b 15b8
:3 0 15b8 0 15b8
15b7 15b5 15b6 :6 0
15ba 29 :3 0 c8e
15bc c90 15bb 15ba
:2 0 15bd c92 :2 0
15bf 0 15bf 15be
156c 15bd :6 0 163d
26 :3 0 15 :3 0
92 :3 0 15c1 15c2
0 15 :3 0 84
:2 0 b7 :3 0 15c4
15c6 0 c97 15c5
15c8 :3 0 15c9 :2 0
109 :3 0 9b :3 0
1c :3 0 109 :3 0
9b :3 0 166 :3 0
109 :3 0 9b :3 0
167 :3 0 166 :3 0
168 :3 0 1c :3 0
1e :3 0 110 :3 0
111 :3 0 17 :3 0
112 :3 0 1b :3 0
113 :3 0 114 :4 0
16f 1 :8 0 15e5
15 :3 0 92 :3 0
15e0 15e1 0 93
:3 0 15e2 15e3 0
15e5 c9a 1638 94
:3 0 15 :3 0 92
:3 0 15e7 15e8 0
93 :3 0 15e9 15ea
0 15ec c9d 15ee
c9f 15ed 15ec :2 0
1636 95 :3 0 15
:3 0 92 :3 0 15f1
15f2 0 93 :3 0
15f3 15f4 0 162e
60 :3 0 61 :3 0
15f6 15f7 0 60
:3 0 71 :3 0 15f9
15fa 0 96 :2 0
2 :4 0 170 :2 0
73 :2 0 23 :2 0
ca1 15ff 1601 :3 0
ca4 15fb 1603 15f8
1605 1604 :2 0 63
:3 0 74 :3 0 1607
1608 0 65 :4 0
171 :4 0 68 :2 0
69 :3 0 ca8 160c
160e :3 0 68 :2 0
7b :4 0 cab 1610
1612 :3 0 68 :2 0
99 :4 0 cae 1614
1616 :3 0 68 :2 0
93 :3 0 cb1 1618
161a :3 0 68 :2 0
9a :4 0 cb4 161c
161e :3 0 68 :2 0
9b :3 0 9c :3 0
23 :2 0 9d :2 0
cb7 1621 1625 cbb
1620 1627 :3 0 cbe
1609 1629 :2 0 162b
cc1 162c 1606 162b
0 162d cc3 0
162e cc5 1631 :3 0
1631 0 1631 1630
162e 162f :6 0 1633
2b :3 0 cc8 1635
cca 1634 1633 :2 0
1636 ccc :2 0 1638
0 1638 1637 15e5
1636 :6 0 163a 26
:3 0 ccf 163b 15ca
163a 0 163c cd1
0 163d cd3 163f
14dc 1551 0 1640
0 163d 0 1640
cd6 0 1709 1e
:3 0 1e :3 0 1641
1642 0 1709 9b
:3 0 24 :3 0 23
:2 0 23 :2 0 cd9
1644 1648 84 :2 0
23 :4 0 cdf 164a
164c :3 0 45 :3 0
7d :3 0 159 :3 0
164f 1650 0 164e
1651 0 1653 ce2
165c 45 :3 0 6d
:3 0 3c :3 0 31
:2 0 ce4 1655 1658
1654 1659 0 165b
ce7 165d 164d 1653
0 165e 0 165b
0 165e ce9 0
1709 1e :3 0 1c
:3 0 165f 1660 0
1709 5c :3 0 15a
:3 0 26 :3 0 17
:3 0 19 :3 0 1b
:3 0 48 :3 0 33
:3 0 20 :3 0 20
:3 0 20 :3 0 20
:3 0 1e :3 0 1f
:3 0 42 :3 0 39
:3 0 37 :3 0 3a
:3 0 3b :3 0 36
:3 0 45 :3 0 24
:3 0 35 :3 0 32
:3 0 3d :3 0 38
:3 0 34 :3 0 3f
:3 0 46 :3 0 1a
:3 0 31 :2 0 31
:2 0 4e :2 0 4e
:2 0 cec 1663 1684
1662 1685 0 1709
5c :3 0 84 :2 0
31 :2 0 d0f 1688
168a :3 0 15b :3 0
15c :3 0 15d :3 0
1b :4 0 15e 1
:8 0 16a6 146 :4 0
1691 :3 0 83 :3 0
84 :2 0 d14 1694
1695 :3 0 15 :3 0
92 :3 0 1697 1698
0 31 :2 0 1699
169a 0 169c d17
16a3 15 :3 0 92
:3 0 169d 169e 0
ee :2 0 169f 16a0
0 16a2 d19 16a4
1696 169c 0 16a5
0 16a2 0 16a5
d1b 0 16a6 d1e
16f9 94 :3 0 15
:3 0 92 :3 0 16a8
16a9 0 93 :3 0
16aa 16ab 0 16ad
d21 16af d23 16ae
16ad :2 0 16f7 95
:3 0 15 :3 0 92
:3 0 16b2 16b3 0
93 :3 0 16b4 16b5
0 16ef 60 :3 0
61 :3 0 16b7 16b8
0 60 :3 0 71
:3 0 16ba 16bb 0
96 :2 0 2 :4 0
172 :2 0 73 :2 0
23 :2 0 d25 16c0
16c2 :3 0 d28 16bc
16c4 16b9 16c6 16c5
:2 0 63 :3 0 74
:3 0 16c8 16c9 0
65 :4 0 173 :4 0
68 :2 0 69 :3 0
d2c 16cd 16cf :3 0
68 :2 0 7b :4 0
d2f 16d1 16d3 :3 0
68 :2 0 99 :4 0
d32 16d5 16d7 :3 0
68 :2 0 93 :3 0
d35 16d9 16db :3 0
68 :2 0 9a :4 0
d38 16dd 16df :3 0
68 :2 0 9b :3 0
9c :3 0 23 :2 0
9d :2 0 d3b 16e2
16e6 d3f 16e1 16e8
:3 0 d42 16ca 16ea
:2 0 16ec d45 16ed
16c7 16ec 0 16ee
d47 0 16ef d49
16f2 :3 0 16f2 0
16f2 16f1 16ef 16f0
:6 0 16f4 2d :3 0
d4c 16f6 d4e 16f5
16f4 :2 0 16f7 d50
:2 0 16f9 0 16f9
16f8 16a6 16f7 :6 0
16fb 26 :3 0 d53
16fc 168b 16fb 0
16fd d55 0 1709
30 :3 0 6d :3 0
30 :3 0 31 :2 0
d57 16ff 1702 73
:2 0 23 :2 0 d5a
1704 1706 :3 0 16fe
1707 0 1709 d5d
170b 164 :3 0 142d
1709 :4 0 170c d68
170d 141e 170c 0
170e d6b 0 170f
d6d 1711 cb9 13e6
0 1712 0 170f
0 1712 d73 0
19d8 5c :3 0 8b
:2 0 31 :2 0 d78
1714 1716 :3 0 1717
:2 0 6d :3 0 33
:3 0 31 :2 0 d7b
1719 171c c8 :2 0
31 :2 0 d80 171e
1720 :3 0 1721 :2 0
1718 1723 1722 :2 0
60 :3 0 61 :3 0
1725 1726 0 60
:3 0 71 :3 0 1728
1729 0 18 :2 0
2 :4 0 174 :2 0
73 :2 0 23 :2 0
d83 172e 1730 :3 0
d86 172a 1732 1727
1734 1733 :2 0 63
:3 0 74 :3 0 1736
1737 0 65 :4 0
175 :4 0 68 :2 0
69 :3 0 d8a 173b
173d :3 0 68 :2 0
7b :4 0 d8d 173f
1741 :3 0 68 :2 0
7c :4 0 d90 1743
1745 :3 0 68 :2 0
6d :3 0 6e :3 0
7d :3 0 176 :3 0
174a 174b 0 d93
1749 174d 6f :4 0
d95 1748 1750 d98
1747 1752 :3 0 68
:2 0 be :4 0 d9b
1754 1756 :3 0 68
:2 0 6b :4 0 d9e
1758 175a :3 0 68
:2 0 26 :3 0 da1
175c 175e :3 0 68
:2 0 6b :4 0 da4
1760 1762 :3 0 da7
1738 1764 :2 0 1766
daa 1767 1735 1766
0 1768 dac 0
1792 60 :3 0 61
:3 0 1769 176a 0
60 :3 0 71 :3 0
176c 176d 0 23
:2 0 2 :4 0 177
:2 0 73 :2 0 23
:2 0 dae 1772 1774
:3 0 db1 176e 1776
176b 1778 1777 :2 0
63 :3 0 74 :3 0
177a 177b 0 65
:4 0 178 :4 0 68
:2 0 69 :3 0 db5
177f 1781 :3 0 db8
177c 1783 :2 0 1785
dbb 1786 1779 1785
0 1787 dbd 0
1792 81 :3 0 c1
:3 0 1788 1789 0
7d :3 0 176 :3 0
178b 178c 0 26
:3 0 83 :3 0 dbf
178a 1790 :2 0 1792
dc3 1793 1724 1792
0 1794 dc7 0
19d8 6d :3 0 33
:3 0 31 :2 0 dc9
1795 1798 c8 :2 0
31 :2 0 dce 179a
179c :3 0 17 :3 0
19 :3 0 179e 179f
0 17a1 dd1 17a2
179d 17a1 0 17a3
dd3 0 19d8 6d
:3 0 161 :3 0 2a
:3 0 dd5 17a5 17a7
31 :2 0 dd7 17a4
17aa c8 :2 0 31
:2 0 ddc 17ac 17ae
:3 0 17af :2 0 6d
:3 0 161 :3 0 2b
:3 0 ddf 17b2 17b4
31 :2 0 de1 17b1
17b7 c8 :2 0 31
:2 0 de6 17b9 17bb
:3 0 17bc :2 0 17b0
17be 17bd :2 0 56
:3 0 179 :4 0 68
:2 0 17a :4 0 de9
17c2 17c4 :3 0 68
:2 0 17b :4 0 dec
17c6 17c8 :3 0 68
:2 0 17c :4 0 def
17ca 17cc :3 0 68
:2 0 17d :4 0 df2
17ce 17d0 :3 0 68
:2 0 6e :3 0 6d
:3 0 39 :3 0 31
:2 0 df5 17d4 17d7
df8 17d3 17d9 dfa
17d2 17db :3 0 68
:2 0 17e :4 0 dfd
17dd 17df :3 0 68
:2 0 6e :3 0 6d
:3 0 3a :3 0 31
:2 0 e00 17e3 17e6
e03 17e2 17e8 e05
17e1 17ea :3 0 68
:2 0 17f :4 0 e08
17ec 17ee :3 0 68
:2 0 6e :3 0 6d
:3 0 3f :3 0 31
:2 0 e0b 17f2 17f5
e0e 17f1 17f7 e10
17f0 17f9 :3 0 68
:2 0 180 :4 0 e13
17fb 17fd :3 0 68
:2 0 181 :4 0 e16
17ff 1801 :3 0 68
:2 0 182 :4 0 e19
1803 1805 :3 0 68
:2 0 183 :4 0 e1c
1807 1809 :3 0 68
:2 0 184 :4 0 e1f
180b 180d :3 0 68
:2 0 185 :4 0 e22
180f 1811 :3 0 68
:2 0 17 :3 0 e25
1813 1815 :3 0 68
:2 0 186 :4 0 e28
1817 1819 :3 0 68
:2 0 2a :3 0 e2b
181b 181d :3 0 68
:2 0 187 :4 0 e2e
181f 1821 :3 0 68
:2 0 188 :4 0 e31
1823 1825 :3 0 68
:2 0 189 :4 0 e34
1827 1829 :3 0 68
:2 0 18a :4 0 e37
182b 182d :3 0 68
:2 0 7d :3 0 18b
:3 0 1830 1831 0
e3a 182f 1833 :3 0
68 :2 0 18c :4 0
e3d 1835 1837 :3 0
68 :2 0 7d :3 0
18b :3 0 183a 183b
0 e40 1839 183d
:3 0 68 :2 0 18d
:4 0 e43 183f 1841
:3 0 68 :2 0 18e
:4 0 e46 1843 1845
:3 0 68 :2 0 18f
:4 0 e49 1847 1849
:3 0 68 :2 0 6e
:3 0 6d :3 0 39
:3 0 31 :2 0 e4c
184d 1850 e4f 184c
1852 e51 184b 1854
:3 0 68 :2 0 17e
:4 0 e54 1856 1858
:3 0 68 :2 0 6e
:3 0 6d :3 0 3a
:3 0 31 :2 0 e57
185c 185f e5a 185b
1861 e5c 185a 1863
:3 0 68 :2 0 190
:4 0 e5f 1865 1867
:3 0 68 :2 0 191
:4 0 e62 1869 186b
:3 0 68 :2 0 6e
:3 0 6d :3 0 3f
:3 0 31 :2 0 e65
186f 1872 e68 186e
1874 e6a 186d 1876
:3 0 68 :2 0 192
:4 0 e6d 1878 187a
:3 0 68 :2 0 193
:4 0 e70 187c 187e
:3 0 68 :2 0 17
:3 0 e73 1880 1882
:3 0 68 :2 0 194
:4 0 e76 1884 1886
:3 0 68 :2 0 2b
:3 0 e79 1888 188a
:3 0 68 :2 0 195
:4 0 e7c 188c 188e
:3 0 68 :2 0 196
:4 0 e7f 1890 1892
:3 0 68 :2 0 197
:4 0 e82 1894 1896
:3 0 17c0 1897 0
189a a5 :3 0 e85
1990 6d :3 0 161
:3 0 2a :3 0 e87
189c 189e 31 :2 0
e89 189b 18a1 c8
:2 0 31 :2 0 e8e
18a3 18a5 :3 0 56
:3 0 179 :4 0 68
:2 0 17a :4 0 e91
18a9 18ab :3 0 68
:2 0 17b :4 0 e94
18ad 18af :3 0 68
:2 0 17c :4 0 e97
18b1 18b3 :3 0 68
:2 0 17d :4 0 e9a
18b5 18b7 :3 0 68
:2 0 6e :3 0 6d
:3 0 39 :3 0 31
:2 0 e9d 18bb 18be
ea0 18ba 18c0 ea2
18b9 18c2 :3 0 68
:2 0 17e :4 0 ea5
18c4 18c6 :3 0 68
:2 0 6e :3 0 6d
:3 0 3a :3 0 31
:2 0 ea8 18ca 18cd
eab 18c9 18cf ead
18c8 18d1 :3 0 68
:2 0 17f :4 0 eb0
18d3 18d5 :3 0 68
:2 0 6e :3 0 6d
:3 0 3f :3 0 31
:2 0 eb3 18d9 18dc
eb6 18d8 18de eb8
18d7 18e0 :3 0 68
:2 0 180 :4 0 ebb
18e2 18e4 :3 0 68
:2 0 181 :4 0 ebe
18e6 18e8 :3 0 68
:2 0 182 :4 0 ec1
18ea 18ec :3 0 68
:2 0 183 :4 0 ec4
18ee 18f0 :3 0 68
:2 0 184 :4 0 ec7
18f2 18f4 :3 0 68
:2 0 185 :4 0 eca
18f6 18f8 :3 0 68
:2 0 17 :3 0 ecd
18fa 18fc :3 0 68
:2 0 186 :4 0 ed0
18fe 1900 :3 0 68
:2 0 2a :3 0 ed3
1902 1904 :3 0 68
:2 0 187 :4 0 ed6
1906 1908 :3 0 68
:2 0 197 :4 0 ed9
190a 190c :3 0 18a7
190d 0 1910 a5
:3 0 edc 1911 18a6
1910 0 1991 6d
:3 0 161 :3 0 2b
:3 0 ede 1913 1915
31 :2 0 ee0 1912
1918 c8 :2 0 31
:2 0 ee5 191a 191c
:3 0 56 :3 0 189
:4 0 68 :2 0 18a
:4 0 ee8 1920 1922
:3 0 68 :2 0 7d
:3 0 18b :3 0 1925
1926 0 eeb 1924
1928 :3 0 68 :2 0
18c :4 0 eee 192a
192c :3 0 68 :2 0
7d :3 0 18b :3 0
192f 1930 0 ef1
192e 1932 :3 0 68
:2 0 18d :4 0 ef4
1934 1936 :3 0 68
:2 0 18e :4 0 ef7
1938 193a :3 0 68
:2 0 18f :4 0 efa
193c 193e :3 0 68
:2 0 6e :3 0 6d
:3 0 39 :3 0 31
:2 0 efd 1942 1945
f00 1941 1947 f02
1940 1949 :3 0 68
:2 0 17e :4 0 f05
194b 194d :3 0 68
:2 0 6e :3 0 6d
:3 0 3a :3 0 31
:2 0 f08 1951 1954
f0b 1950 1956 f0d
194f 1958 :3 0 68
:2 0 190 :4 0 f10
195a 195c :3 0 68
:2 0 191 :4 0 f13
195e 1960 :3 0 68
:2 0 6e :3 0 6d
:3 0 3f :3 0 31
:2 0 f16 1964 1967
f19 1963 1969 f1b
1962 196b :3 0 68
:2 0 192 :4 0 f1e
196d 196f :3 0 68
:2 0 193 :4 0 f21
1971 1973 :3 0 68
:2 0 17 :3 0 f24
1975 1977 :3 0 68
:2 0 194 :4 0 f27
1979 197b :3 0 68
:2 0 2b :3 0 f2a
197d 197f :3 0 68
:2 0 195 :4 0 f2d
1981 1983 :3 0 68
:2 0 196 :4 0 f30
1985 1987 :3 0 68
:2 0 197 :4 0 f33
1989 198b :3 0 191e
198c 0 198e f36
198f 191d 198e 0
1991 17bf 189a 0
1991 f38 0 19d8
60 :3 0 61 :3 0
1992 1993 0 60
:3 0 71 :3 0 1995
1996 0 23 :2 0
2 :4 0 198 :2 0
73 :2 0 23 :2 0
f3c 199b 199d :3 0
f3f 1997 199f 1994
19a1 19a0 :2 0 63
:3 0 74 :3 0 19a3
19a4 0 65 :4 0
199 :4 0 68 :2 0
69 :3 0 f43 19a8
19aa :3 0 f46 19a5
19ac :2 0 19ae f49
19af 19a2 19ae 0
19b0 f4b 0 19d8
60 :3 0 61 :3 0
19b1 19b2 0 60
:3 0 71 :3 0 19b4
19b5 0 23 :2 0
2 :4 0 19a :2 0
73 :2 0 23 :2 0
f4d 19ba 19bc :3 0
f50 19b6 19be 19b3
19c0 19bf :2 0 63
:3 0 74 :3 0 19c2
19c3 0 65 :4 0
19b :4 0 68 :2 0
69 :3 0 f54 19c7
19c9 :3 0 f57 19c4
19cb :2 0 19cd f5a
19ce 19c1 19cd 0
19cf f5c 0 19d8
14 :3 0 81 :3 0
19c :3 0 19d1 19d2
0 56 :3 0 f5e
19d3 19d5 19d6 :2 0
19d8 f60 1a79 15
:3 0 19d :2 0 4
19d9 19da 0 60
:3 0 61 :3 0 19dc
19dd 0 60 :3 0
71 :3 0 19df 19e0
0 96 :2 0 2
:4 0 19e :2 0 73
:2 0 23 :2 0 f8e
19e5 19e7 :3 0 f91
19e1 19e9 19de 19eb
19ea :2 0 63 :3 0
74 :3 0 19ed 19ee
0 65 :4 0 19f
:4 0 68 :2 0 69
:3 0 f95 19f2 19f4
:3 0 f98 19ef 19f6
:2 0 1a0f 63 :3 0
74 :3 0 19f8 19f9
0 65 :4 0 1a0
:4 0 68 :2 0 15
:3 0 1a1 :3 0 19fe
19ff 0 f9b 19fd
1a01 :3 0 68 :2 0
be :4 0 f9e 1a03
1a05 :3 0 68 :2 0
15 :3 0 1a2 :3 0
1a08 1a09 0 fa1
1a07 1a0b :3 0 fa4
19fa 1a0d :2 0 1a0f
fa7 1a10 19ec 1a0f
0 1a11 faa 0
1a21 1a3 :3 0 5f
:3 0 15 :3 0 1a1
:3 0 1a4 :3 0 15
:3 0 1a2 :3 0 1a5
:3 0 90 :4 0 1a6
1 :a 0 1a1d :2 0
1a21 1a13 1a1b :2 0
14 :3 0 5f :3 0
1a1f :2 0 1a21 fac
1a23 fb0 1a22 1a21
:2 0 1a77 95 :3 0
15 :3 0 92 :3 0
1a26 1a27 0 93
:3 0 1a28 1a29 0
1a6f 60 :3 0 61
:3 0 1a2b 1a2c 0
60 :3 0 71 :3 0
1a2e 1a2f 0 96
:2 0 2 :4 0 1a7
:2 0 73 :2 0 23
:2 0 fb2 1a34 1a36
:3 0 fb5 1a30 1a38
1a2d 1a3a 1a39 :2 0
63 :3 0 74 :3 0
1a3c 1a3d 0 65
:4 0 1a8 :4 0 68
:2 0 69 :3 0 fb9
1a41 1a43 :3 0 fbc
1a3e 1a45 :2 0 1a5f
63 :3 0 74 :3 0
1a47 1a48 0 65
:4 0 99 :4 0 68
:2 0 93 :3 0 fbf
1a4c 1a4e :3 0 68
:2 0 9a :4 0 fc2
1a50 1a52 :3 0 68
:2 0 9b :3 0 9c
:3 0 23 :2 0 9d
:2 0 fc5 1a55 1a59
fc9 1a54 1a5b :3 0
fcc 1a49 1a5d :2 0
1a5f fcf 1a60 1a3b
1a5f 0 1a61 fd2
0 1a6f 1a3 :3 0
5f :3 0 1a4 :3 0
15 :3 0 1a2 :3 0
1a5 :3 0 90 :4 0
1a9 1 :a 0 1a6b
:2 0 1a6f 1a63 1a69
:2 0 14 :3 0 5f
:3 0 1a6d :2 0 1a6f
fd4 1a72 :3 0 1a72
0 1a72 1a71 1a6f
1a70 :6 0 1a74 1
:3 0 fd9 1a76 fdb
1a75 1a74 :2 0 1a77
fdd :2 0 1a79 fe0
1a79 1a78 19d8 1a77
:6 0 1a7a :2 0 2
44 1a79 1a7d :3 0
1a7c 1a7a 1a7e :8 0

101f
4
:3 0 1 3 1
7 1 b 1
f 1 13 1
17 1 1b 1
1f 1 23 1
27 1 2b 1
2f 1 33 1
37 1 3b f
6 a e 12
16 1a 1e 22
26 2a 2e 32
36 3a 3e 1
48 1 46 1
4f 1 4d 1
56 1 54 1
5d 1 5b 1
64 1 62 1
6b 1 69 1
72 1 70 1
77 1 7e 1
7c 1 85 1
83 1 8c 1
8a 1 93 1
91 1 9a 1
98 1 a1 1
9f 1 a8 1
a6 1 af 1
ad 1 b4 1
b9 1 c0 1
be 1 c5 1
cb 1 d1 1
d7 1 dd 1
e3 1 e9 1
ef 1 f5 1
fb 1 101 1
107 1 10d 1
113 1 119 1
11f 1 125 1
12b 1 131 1
137 1 13d 1
143 1 149 1
14f 1 155 1
15b 1 161 1
167 1 16d 1
173 1 17b 1
179 1 182 1
180 1 187 1
18d 1 195 1
193 1 19a 1
1a0 1 1a6 1
1ac 1 1b2 1
1b8 1 1be 1
1c3 1 1d0 1
1dc 2 1df 1e1
2 1e4 1e6 2
1e8 1ea 2 1ec
1ee 2 1f0 1f2
2 1f4 1f6 2
1f8 1fa 2 1fc
1fe 2 200 202
2 204 206 2
208 20a 2 20c
20e 2 210 212
1 218 2 21a
21b 2 214 21d
2 21f 221 1
227 2 229 22a
2 223 22c 2
22e 230 1 236
2 238 239 2
232 23b 2 23d
23f 1 245 2
247 248 2 241
24a 2 24c 24e
1 254 2 256
257 2 250 259
2 25b 25d 1
263 2 265 266
2 25f 268 2
26a 26c 1 272
2 274 275 2
26e 277 2 279
27b 1 281 2
283 284 2 27d
286 2 288 28a
1 290 2 292
293 2 28c 295
2 297 299 1
29f 2 2a1 2a2
2 29b 2a4 2
2a6 2a8 2 2aa
2ac 2 2ae 2b0
2 2b2 2b4 2
2b6 2b8 2 2ba
2bc 2 2be 2c0
2 2c2 2c4 2
2c6 2c8 4 1d8
1de 1e3 2ca 1
2cc 1 2ce 2
2d8 2da 3 2d6
2d7 2dc 2 2e5
2e7 2 2e4 2e9
1 2eb 1 2ed
2 31f 320 1
322 1 32c 2
339 33b 3 337
338 33d 2 346
348 2 34a 34c
2 34e 350 1
358 2 35a 35b
2 352 35d 2
345 35f 1 361
1 363 2 36d
36f 3 36b 36c
371 2 37a 37c
2 379 37e 1
380 1 382 2
389 38a 3 364
383 38c 1 38e
2 391 392 1
396 2 394 396
2 39a 39b 1
39f 2 39d 39f
2 3a5 3a6 1
3aa 2 3a8 3aa
2 3b7 3b9 3
3b5 3b6 3bb 2
3c4 3c6 2 3c8
3ca 2 3cc 3ce
1 3d6 2 3d8
3d9 2 3d0 3db
2 3c3 3dd 1
3df 1 3e1 2
3eb 3ed 3 3e9
3ea 3ef 2 3f8
3fa 2 3f7 3fc
1 3fe 1 400
2 407 408 3
3e2 401 40a 1
40c 2 40f 410
1 416 2 412
416 1 41a 1
422 2 432 437
1 43e 1 439
2 451 453 3
44f 450 455 2
45e 460 2 462
464 2 466 468
2 46a 46c 2
46e 470 3 475
476 477 2 472
479 2 45d 47b
1 47d 1 47f
2 448 480 1
484 1 443 2
441 488 1 493
2 48f 493 2
498 499 1 49d
2 49b 49d 2
4ab 4ad 3 4a9
4aa 4af 2 4b8
4ba 2 4bc 4be
2 4c0 4c2 1
4ca 2 4cc 4cd
2 4c4 4cf 2
4b7 4d1 1 4d3
1 4d5 2 4df
4e1 3 4dd 4de
4e3 2 4ec 4ee
2 4eb 4f0 1
4f2 1 4f4 2
4fb 4fc 3 4d6
4f5 4fe 1 500
3 48b 501 504
1 507 1 50d
2 518 51d 1
524 1 51f 2
537 539 3 535
536 53b 2 544
546 2 548 54a
2 54c 54e 2
550 552 2 554
556 3 55b 55c
55d 2 558 55f
2 543 561 1
563 1 565 2
52e 566 1 56a
1 529 2 527
56e 1 571 2
575 574 2 580
581 2 583 586
2 589 58a 2
591 592 2 594
597 2 59d 59e
1 5a2 2 5a0
5a2 2 5b0 5b5
1 5bc 1 5b7
2 5cf 5d1 3
5cd 5ce 5d3 2
5dc 5de 2 5e0
5e2 2 5e4 5e6
2 5e8 5ea 2
5ec 5ee 3 5f3
5f4 5f5 2 5f0
5f7 2 5db 5f9
1 5fb 1 5fd
2 5c6 5fe 1
602 1 5c1 2
5bf 606 1 611
2 60d 611 2
61b 620 1 627
1 622 2 63a
63c 3 638 639
63e 2 647 649
2 64b 64d 2
64f 651 2 653
655 2 657 659
3 65e 65f 660
2 65b 662 2
646 664 1 666
1 668 2 631
669 1 66d 1
62c 2 62a 671
1 67c 2 678
67c 2 688 68a
3 686 687 68c
2 695 697 2
699 69b 2 69d
69f 1 6a7 2
6a9 6aa 2 6a1
6ac 2 6ae 6b0
2 6b2 6b4 2
6b6 6b8 2 6ba
6bc 2 694 6be
1 6c0 1 6c2
2 6cc 6ce 3
6ca 6cb 6d0 2
6d9 6db 2 6d8
6dd 1 6df 1
6e1 3 6e8 6e9
6ea 3 6c3 6e2
6ec 1 6ee 2
674 6ef 1 6f1
2 6fb 700 1
707 1 702 2
71a 71c 3 718
719 71e 2 727
729 2 72b 72d
2 72f 731 2
733 735 2 737
739 3 73e 73f
740 2 73b 742
2 726 744 1
746 1 748 2
711 749 1 74d
1 70c 2 70a
751 2 757 758
1 75c 2 75a
75c 2 764 769
1 770 1 76b
2 783 785 3
781 782 787 2
790 792 2 794
796 2 798 79a
2 79c 79e 2
7a0 7a2 3 7a7
7a8 7a9 2 7a4
7ab 2 78f 7ad
1 7af 1 7b1
2 77a 7b2 1
7b6 1 775 2
773 7ba 2 7c2
7c3 1 7c7 2
7c5 7c7 2 7d2
7d4 3 7d0 7d1
7d6 2 7df 7e1
2 7e3 7e5 2
7e7 7e9 1 7f1
2 7f3 7f4 2
7eb 7f6 2 7f8
7fa 2 7fc 7fe
2 800 802 2
804 806 2 7de
808 1 80a 1
80c 2 816 818
3 814 815 81a
2 823 825 2
822 827 1 829
1 82b 3 832
833 834 3 80d
82c 836 1 838
2 7bd 839 1
83b 5 609 6f2
6f5 754 83c 1
83e 2 84b 84d
3 849 84a 84f
2 858 85a 2
85c 85e 2 860
862 1 86a 2
86c 86d 2 864
86f 2 857 871
1 873 1 875
2 87f 881 3
87d 87e 883 2
88c 88e 2 88b
890 1 892 1
894 2 89b 89c
3 876 895 89e
1 8a0 3 8a8
8a9 8aa 3 8ac
8ad 8ae 4 8b6
8b7 8b8 8b9 2
8bf 8c0 1 8c2
1 8c6 2 8c4
8c6 2 8e1 8e6
1 8ed 1 8e8
2 900 902 3
8fe 8ff 904 2
90d 90f 2 911
913 2 915 917
2 919 91b 2
91d 91f 3 924
925 926 2 921
928 2 90c 92a
1 92c 1 92e
2 8f7 92f 1
933 1 8f2 2
8f0 937 1 942
2 93e 942 2
94e 950 3 94c
94d 952 2 95b
95d 2 95f 961
2 963 965 1
96d 2 96f 970
2 967 972 2
974 976 2 978
97a 2 97c 97e
2 980 982 2
95a 984 1 986
1 988 2 992
994 3 990 991
996 2 99f 9a1
2 99e 9a3 1
9a5 1 9a7 3
9ae 9af 9b0 3
989 9a8 9b2 1
9b4 2 93a 9b5
2 9bf 9c1 3
9bd 9be 9c3 2
9cc 9ce 2 9d0
9d2 2 9d4 9d6
1 9de 2 9e0
9e1 2 9d8 9e3
2 9e5 9e7 2
9e9 9eb 2 9ed
9ef 2 9f1 9f3
2 9cb 9f5 1
9f7 1 9f9 2
a03 a05 3 a01
a02 a07 2 a10
a12 2 a0f a14
1 a16 1 a18
3 a1f a20 a21
3 9fa a19 a23
2 a25 a26 5
8a1 8a4 8b1 8bc
a27 2 a29 a2a
2 a2d a2e 1
a32 2 a30 a32
2 a37 a38 1
a3c 2 a3a a3c
1 a44 1 a46
1 a4a 2 a48
a4a 2 a58 a59
2 a5d a5e 2
a62 a63 2 a67
a68 2 a6c a6d
2 a71 a72 2
a76 a77 2 a7b
a7c d a55 a56
a5b a60 a65 a6a
a6f a74 a79 a7e
a7f a80 a81 2
a86 a87 1 a8d
2 a89 a8d 2
a98 a9a 3 a96
a97 a9c 2 aa5
aa7 2 aa9 aab
2 aad aaf 1
ab7 2 ab9 aba
2 ab1 abc 2
aa4 abe 1 ac0
1 ac2 2 acc
ace 3 aca acb
ad0 2 ad9 adb
2 ad8 add 1
adf 1 ae1 2
ae8 ae9 3 ac3
ae2 aeb 2 aef
af0 1 af4 2
af2 af4 2 aff
b01 3 afd afe
b03 2 b0c b0e
2 b10 b12 2
b14 b16 1 b1c
2 b1e b1f 2
b18 b21 2 b23
b25 2 b27 b29
2 b2b b2d 2
b2f b31 2 b0b
b33 1 b35 1
b37 2 b41 b43
3 b3f b40 b45
2 b4e b50 2
b4d b52 1 b54
1 b56 2 b5c
b5d 3 b5f b60
b61 3 b38 b57
b63 2 b66 b65
4 a4f a52 a84
b67 1 b69 2
b6c b6d 1 b71
2 b6f b71 2
b76 b77 1 b7b
2 b79 b7b 2
b82 b83 1 b87
2 b85 b87 2
b8c b8d 1 b91
2 b8f b91 1
b99 2 b9d b9e
1 ba2 2 ba0
ba2 1 ba7 2
bac bad 1 bb1
2 baf bb1 2
bb6 bb7 1 bbb
2 bb9 bbb 1
bc3 2 bc8 bc9
1 bcd 2 bcb
bcd 2 bd2 bd3
2 bd8 bd9 2
bd5 bdb 2 bde
be1 4 be4 baa
bc6 be3 1 be5
1 be7 2 bea
beb 1 bef 2
bed bef 2 bf3
bf4 1 bf8 2
bf6 bf8 2 c03
c05 3 c01 c02
c07 2 c10 c12
2 c14 c16 2
c18 c1a 1 c22
2 c24 c25 2
c1c c27 2 c29
c2b 2 c2d c2f
2 c31 c33 2
c35 c37 2 c0f
c39 1 c3b 1
c3d 2 c47 c49
3 c45 c46 c4b
2 c54 c56 2
c53 c58 1 c5a
1 c5c 3 c63
c64 c65 3 c3e
c5d c67 1 c69
2 c6c c6d 1
c71 2 c6f c71
1 c76 2 c7c
c7d 1 c7f 2
c81 c83 1 c85
1 c88 2 c8a
c8b 2 c6a c8c
1 c8e 3 ca3
ca4 ca5 3 cab
cac cad 2 cb2
cb3 1 cb7 2
cb5 cb7 3 cc1
cc2 cc3 1 cc7
2 cc5 cc7 2
ce3 ce8 1 cef
1 cea 2 d02
d04 3 d00 d01
d06 2 d0f d11
2 d13 d15 2
d17 d19 2 d1b
d1d 2 d1f d21
3 d26 d27 d28
2 d23 d2a 2
d0e d2c 1 d2e
1 d30 2 cf9
d31 1 d35 1
cf4 2 cf2 d39
2 d43 d48 1
d4f 1 d4a 2
d62 d64 3 d60
d61 d66 2 d6f
d71 2 d73 d75
2 d77 d79 2
d7b d7d 2 d7f
d81 3 d86 d87
d88 2 d83 d8a
2 d6e d8c 1
d8e 1 d90 2
d59 d91 1 d95
1 d54 2 d52
d99 1 da4 2
da0 da4 2 db0
db2 3 dae daf
db4 2 dbd dbf
2 dc1 dc3 2
dc5 dc7 1 dcf
2 dd1 dd2 2
dc9 dd4 2 dd6
dd8 2 dda ddc
2 dde de0 2
de2 de4 2 dbc
de6 1 de8 1
dea 2 df4 df6
3 df2 df3 df8
2 e01 e03 2
e00 e05 1 e07
1 e09 3 e10
e11 e12 3 deb
e0a e14 1 e16
2 e25 e2a 1
e31 1 e2c 2
e44 e46 3 e42
e43 e48 2 e51
e53 2 e55 e57
2 e59 e5b 2
e5d e5f 2 e61
e63 3 e68 e69
e6a 2 e65 e6c
2 e50 e6e 1
e70 1 e72 2
e3b e73 1 e77
1 e36 2 e34
e7b 5 d3c d9c
e17 e1a e7e 3
e83 e84 e85 1
e89 2 e87 e89
2 e91 e96 1
e9d 1 e98 2
eb0 eb2 3 eae
eaf eb4 2 ebd
ebf 2 ec1 ec3
2 ec5 ec7 2
ec9 ecb 2 ecd
ecf 3 ed4 ed5
ed6 2 ed1 ed8
2 ebc eda 1
edc 1 ede 2
ea7 edf 1 ee3
1 ea2 2 ea0
ee7 1 ef2 2
eee ef2 2 efe
f00 3 efc efd
f02 2 f0b f0d
2 f0f f11 2
f13 f15 1 f1d
2 f1f f20 2
f17 f22 2 f24
f26 2 f28 f2a
2 f2c f2e 2
f30 f32 2 f0a
f34 1 f36 1
f38 2 f42 f44
3 f40 f41 f46
2 f4f f51 2
f4e f53 1 f55
1 f57 3 f5e
f5f f60 3 f39
f58 f62 1 f64
2 f82 f87 1
f8e 1 f89 2
fa1 fa3 3 f9f
fa0 fa5 2 fae
fb0 2 fb2 fb4
2 fb6 fb8 2
fba fbc 2 fbe
fc0 3 fc5 fc6
fc7 2 fc2 fc9
2 fad fcb 1
fcd 1 fcf 2
f98 fd0 1 fd4
1 f93 2 f91
fd8 1 fe3 2
fdf fe3 2 1000
1005 1 100c 1
1007 2 101f 1021
3 101d 101e 1023
2 102c 102e 2
1030 1032 2 1034
1036 2 1038 103a
2 103c 103e 3
1043 1044 1045 2
1040 1047 2 102b
1049 1 104b 1
104d 2 1016 104e
1 1052 1 1011
2 100f 1056 1
1059 1 105c 2
1068 106d 1 1074
1 106f 2 1087
1089 3 1085 1086
108b 2 1094 1096
2 1098 109a 2
109c 109e 2 10a0
10a2 2 10a4 10a6
3 10ab 10ac 10ad
2 10a8 10af 2
1093 10b1 1 10b3
1 10b5 2 107e
10b6 1 10ba 1
1079 2 1077 10be
6 eea f65 f68
fdb 105d 10c1 2
10cd 10cf 3 10cb
10cc 10d1 2 10da
10dc 2 10de 10e0
2 10e2 10e4 1
10ec 2 10ee 10ef
2 10e6 10f1 2
10d9 10f3 1 10f5
1 10f7 2 1101
1103 3 10ff 1100
1105 2 110e 1110
2 110d 1112 1
1114 1 1116 2
111d 111e 3 10f8
1117 1120 3 1122
10c4 1123 1 112b
2 1127 112b 2
1133 1134 4 1131
1136 1139 113a 2
1142 1143 4 1140
1145 1148 1149 1
1175 2 1174 1175
1 117d 1 1183
2 1185 1186 2
1172 1187 1 118e
1 1189 2 11a1
11a3 3 119f 11a0
11a5 2 11ae 11b0
2 11b2 11b4 2
11b6 11b8 2 11ba
11bc 2 11be 11c0
3 11c5 11c6 11c7
2 11c2 11c9 2
11ad 11cb 1 11cd
1 11cf 2 1198
11d0 1 11d4 1
1193 2 1191 11d8
1 11df 4 113d
114c 11db 11e2 2
11ed 11ee 4 11eb
11f0 11f3 11f4 7
1202 1203 1204 1205
1206 1207 1208 1
1227 2 1226 1227
1 122f 1 1235
2 1237 1238 2
1224 1239 1 1240
1 123b 2 1253
1255 3 1251 1252
1257 2 1260 1262
2 1264 1266 2
1268 126a 2 126c
126e 2 1270 1272
3 1277 1278 1279
2 1274 127b 2
125f 127d 1 127f
1 1281 2 124a
1282 1 1286 1
1245 2 1243 128a
6 11e8 11f7 11fc
11ff 120b 128d 2
1290 1291 2 1294
1295 1 1299 2
1297 1299 1 129d
2 12aa 12af 1
12b6 1 12b1 2
12c9 12cb 3 12c7
12c8 12cd 2 12d6
12d8 2 12da 12dc
2 12de 12e0 2
12e2 12e4 2 12e6
12e8 3 12ed 12ee
12ef 2 12ea 12f1
2 12d5 12f3 1
12f5 1 12f7 2
12c0 12f8 1 12fc
1 12bb 2 12b9
1300 3 1306 1307
1308 1 130c 2
130a 130c 1 1311
a 1319 131a 131b
131c 131d 131e 131f
1320 1321 1322 2
1314 1325 1 1327
2 1303 1328 1
132a 3 1330 1331
1332 1 1336 2
1334 1336 1 133d
2 1341 1342 1
1345 2 1347 1348
20 134c 134d 134e
134f 1350 1351 1352
1353 1354 1355 1356
1357 1358 1359 135a
135b 135c 135d 135e
135f 1360 1361 1362
1363 1364 1365 1366
1367 1368 1369 136a
136b 1 1371 2
136f 1371 1 137b
2 137a 137b 1
1383 1 1389 2
138b 138c 2 1378
138d 1 1394 1
138f 2 13a7 13a9
3 13a5 13a6 13ab
2 13b4 13b6 2
13b8 13ba 2 13bc
13be 2 13c0 13c2
2 13c4 13c6 3
13cb 13cc 13cd 2
13c8 13cf 2 13b3
13d1 1 13d3 1
13d5 2 139e 13d6
1 13da 1 1399
2 1397 13de 1
13e1 1 13e4 9
cbc cbf 1124 1292
132b 132e 1349 136e
13e5 1 13f2 2
13f4 13f5 1 13f9
2 13f7 13f9 1
13fe 1 1404 2
1406 1407 2 1409
140b 2 140e 1410
1 1412 1 1415
2 1417 1418 1
141c 2 141a 141c
2 1424 1425 1
1428 2 1427 1428
2 1435 1436 2
1438 143a 2 143c
143e 4 1432 1433
1440 1441 2 144c
144d 2 144f 1451
2 1453 1455 2
1457 1459 4 1449
144a 145b 145c 3
1461 1462 1463 1
1467 2 1465 1467
1 1471 2 1473
1474 1 1478 2
1476 1478 1 147d
2 1481 1483 3
1485 1486 1487 2
148d 148f 3 1491
1492 1493 2 148a
1496 2 1498 1499
2 146e 149a 2
149e 149f 1 14a5
2 14a7 14a8 1
14ac 2 14aa 14ac
1 14b1 2 14b5
14b7 3 14b9 14ba
14bb 2 14c1 14c3
3 14c5 14c6 14c7
2 14be 14ca 2
14cc 14cd 2 14a2
14ce 2 14d0 14d1
3 14d4 14d5 14d6
1 14da 2 14d8
14da 2 14f0 14f5
1 14fc 1 14f7
2 150f 1511 3
150d 150e 1513 2
151c 151e 2 1520
1522 2 1524 1526
2 1528 152a 2
152c 152e 3 1533
1534 1535 2 1530
1537 2 151b 1539
1 153b 1 153d
2 1506 153e 1
1542 1 1501 2
14ff 1546 3 1549
154d 1550 2 1566
156b 1 1572 1
156d 2 1585 1587
3 1583 1584 1589
2 1592 1594 2
1596 1598 2 159a
159c 2 159e 15a0
2 15a2 15a4 3
15a9 15aa 15ab 2
15a6 15ad 2 1591
15af 1 15b1 1
15b3 2 157c 15b4
1 15b8 1 1577
2 1575 15bc 1
15c7 2 15c3 15c7
2 15df 15e4 1
15eb 1 15e6 2
15fe 1600 3 15fc
15fd 1602 2 160b
160d 2 160f 1611
2 1613 1615 2
1617 1619 2 161b
161d 3 1622 1623
1624 2 161f 1626
2 160a 1628 1
162a 1 162c 2
15f5 162d 1 1631
1 15f0 2 15ee
1635 1 1638 1
163b 2 15bf 163c
2 163e 163f 3
1645 1646 1647 1
164b 2 1649 164b
1 1652 2 1656
1657 1 165a 2
165c 165d 20 1664
1665 1666 1667 1668
1669 166a 166b 166c
166d 166e 166f 1670
1671 1672 1673 1674
1675 1676 1677 1678
1679 167a 167b 167c
167d 167e 167f 1680
1681 1682 1683 1
1689 2 1687 1689
1 1693 2 1692
1693 1 169b 1
16a1 2 16a3 16a4
2 1690 16a5 1
16ac 1 16a7 2
16bf 16c1 3 16bd
16be 16c3 2 16cc
16ce 2 16d0 16d2
2 16d4 16d6 2
16d8 16da 2 16dc
16de 3 16e3 16e4
16e5 2 16e0 16e7
2 16cb 16e9 1
16eb 1 16ed 2
16b6 16ee 1 16f2
1 16b1 2 16af
16f6 1 16f9 1
16fc 2 1700 1701
2 1703 1705 a
1444 145f 14d2 1640
1643 165e 1661 1686
16fd 1708 2 1421
170b 1 170d 5
13e9 13ec 13ef 1419
170e 2 1710 1711
1 1715 2 1713
1715 2 171a 171b
1 171f 2 171d
171f 2 172d 172f
3 172b 172c 1731
2 173a 173c 2
173e 1740 2 1742
1744 1 174c 2
174e 174f 2 1746
1751 2 1753 1755
2 1757 1759 2
175b 175d 2 175f
1761 2 1739 1763
1 1765 1 1767
2 1771 1773 3
176f 1770 1775 2
177e 1780 2 177d
1782 1 1784 1
1786 3 178d 178e
178f 3 1768 1787
1791 1 1793 2
1796 1797 1 179b
2 1799 179b 1
17a0 1 17a2 1
17a6 2 17a8 17a9
1 17ad 2 17ab
17ad 1 17b3 2
17b5 17b6 1 17ba
2 17b8 17ba 2
17c1 17c3 2 17c5
17c7 2 17c9 17cb
2 17cd 17cf 2
17d5 17d6 1 17d8
2 17d1 17da 2
17dc 17de 2 17e4
17e5 1 17e7 2
17e0 17e9 2 17eb
17ed 2 17f3 17f4
1 17f6 2 17ef
17f8 2 17fa 17fc
2 17fe 1800 2
1802 1804 2 1806
1808 2 180a 180c
2 180e 1810 2
1812 1814 2 1816
1818 2 181a 181c
2 181e 1820 2
1822 1824 2 1826
1828 2 182a 182c
2 182e 1832 2
1834 1836 2 1838
183c 2 183e 1840
2 1842 1844 2
1846 1848 2 184e
184f 1 1851 2
184a 1853 2 1855
1857 2 185d 185e
1 1860 2 1859
1862 2 1864 1866
2 1868 186a 2
1870 1871 1 1873
2 186c 1875 2
1877 1879 2 187b
187d 2 187f 1881
2 1883 1885 2
1887 1889 2 188b
188d 2 188f 1891
2 1893 1895 1
1898 1 189d 2
189f 18a0 1 18a4
2 18a2 18a4 2
18a8 18aa 2 18ac
18ae 2 18b0 18b2
2 18b4 18b6 2
18bc 18bd 1 18bf
2 18b8 18c1 2
18c3 18c5 2 18cb
18cc 1 18ce 2
18c7 18d0 2 18d2
18d4 2 18da 18db
1 18dd 2 18d6
18df 2 18e1 18e3
2 18e5 18e7 2
18e9 18eb 2 18ed
18ef 2 18f1 18f3
2 18f5 18f7 2
18f9 18fb 2 18fd
18ff 2 1901 1903
2 1905 1907 2
1909 190b 1 190e
1 1914 2 1916
1917 1 191b 2
1919 191b 2 191f
1921 2 1923 1927
2 1929 192b 2
192d 1931 2 1933
1935 2 1937 1939
2 193b 193d 2
1943 1944 1 1946
2 193f 1948 2
194a 194c 2 1952
1953 1 1955 2
194e 1957 2 1959
195b 2 195d 195f
2 1965 1966 1
1968 2 1961 196a
2 196c 196e 2
1970 1972 2 1974
1976 2 1978 197a
2 197c 197e 2
1980 1982 2 1984
1986 2 1988 198a
1 198d 3 1990
1911 198f 2 199a
199c 3 1998 1999
199e 2 19a7 19a9
2 19a6 19ab 1
19ad 1 19af 2
19b9 19bb 3 19b7
19b8 19bd 2 19c6
19c8 2 19c5 19ca
1 19cc 1 19ce
1 19d4 2d 2cf
2ee 2f1 2f4 2f7
2fa 2fd 300 303
306 309 30c 30f
312 315 318 31b
325 328 32b 38f
40d 576 579 58d
59b a2b a47 b6a
be8 c8f c94 c97
c9a c9d ca0 ca8
cb0 1712 1794 17a3
1991 19b0 19cf 19d7
2 19e4 19e6 3
19e2 19e3 19e8 2
19f1 19f3 2 19f0
19f5 2 19fc 1a00
2 1a02 1a04 2
1a06 1a0a 2 19fb
1a0c 2 19f7 1a0e
1 1a10 3 1a11
1a1c 1a20 1 19db
2 1a33 1a35 3
1a31 1a32 1a37 2
1a40 1a42 2 1a3f
1a44 2 1a4b 1a4d
2 1a4f 1a51 3
1a56 1a57 1a58 2
1a53 1a5a 2 1a4a
1a5c 2 1a46 1a5e
1 1a60 4 1a2a
1a61 1a6a 1a6e 1
1a72 1 1a25 2
1a23 1a76 3e 4b
52 59 60 67
6e 75 7a 81
88 8f 96 9d
a4 ab b2 b7
bc c3 c9 cf
d5 db e1 e7
ed f3 f9 ff
105 10b 111 117
11d 123 129 12f
135 13b 141 147
14d 153 159 15f
165 16b 171 177
17e 185 18b 191
198 19e 1a4 1aa
1b0 1b6 1bc 1c1
1c8 
1
4
0 
1a7d
0
1
50
2f
4e
0 1 2 1 4 1 6 1
8 1 a 1 c 1 e 1
10 1 12 1 14 1 16 1
18 1 1a 1 1c 1 1e 1
20 1 22 1 24 1 26 27
26 29 26 2b 26 2d 1 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0

3b 1 0
13d 1 0
2b 1 0
173 1 0
1b2 1 0
149 1 0
2f 1 0
1a0 1 0
a6 1 0
107 1 0
e3 1 0
1ac 1 0
8a 1 0
27 1 0
161 1 0
179 1 0
98 1 0
1f 1 0
1b 1 0
1b8 1 0
83 1 0
c5 1 0
16d 1 0
91 1 0
d7 1 0
193 1 0
77 1 0
1be 1 0
18d 1 0
13 1 0
101 1 0
f5 1 0
69 1 0
7c 1 0
131 1 0
9f 1 0
62 1 0
f 1 0
ef 1 0
155 1 0
3 1 0
cb 1 0
b9 1 0
70 1 0
187 1 0
137 1 0
15b 1 0
11f 1 0
113 1 0
23 1 0
2 0 1
4d 1 0
1a6 1 0
7 1 0
14f 1 0
fb 1 0
e9 1 0
143 1 0
dd 1 0
19a 1 0
d1 1 0
5b 1 0
46 1 0
119 1 0
37 1 0
ad 1 0
125 1 0
17 1 0
167 1 0
b4 1 0
be 1 0
12b 1 0
54 1 0
33 1 0
10d 1 0
b 1 0
1c3 1 0
180 1 0
0

/
SHOW ERRORS FUNCTION SP_SEARCH_3;
ALTER FUNCTION SP_SEARCH_3 COMPILE DEBUG;
/*
#Stored Procedure  :SP_I_PROFILE
#Short Description :Assign a profile to a task or project
#Description       :used tables: RESOURCE_ASSIGNMENTS, PROFILES
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_I_PROFILE ==
CREATE OR REPLACE FUNCTION SP_I_PROFILE wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
9b
2 :e:
1FUNCTION:
1SP_I_PROFILE:
1PRJ_ID:
1VARCHAR2:
1AND_ROLES:
1AND_SKILLS:
1ORG_ID001:
1GEO_ID001:
1POOL_ID001:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_UNIQUEID:
132:
1H_AND_ROLES:
18000:
1H_ROLE_ID:
1H_FULL_NAME:
1255:
1H_COUNT:
1PLS_INTEGER:
10:
1H_NUMBER:
1NUMITEMS:
1NUMITEMS001:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_I_PROFILE(:
1':
1, :
1);:
1CHECK_TRACE:
11:
134:
1+:
1FPUT:
1               ENTER:: SP_I_PROFILE(35) :: cplan.sql(5582) :: :
1IS NULL:
18:
141:
1      RETURN_INVALID:: SP_I_PROFILE(42) :: cplan.sql(5589) :: :
1 :
1STATE:: :
1COALESCE:
1TO_CHAR:
1PMO_CNST:
1INVALID_PARAMS:
1NULL:
144:
1                EXIT:: SP_I_PROFILE(45) :: cplan.sql(5592) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1SUBSTR:
1MAX:
1PROF_NUMBER:
1PROFILES:
1PROJECT_ID:
1ROLE_ID:
1SELECT COALESCE(MAX(PROF_NUMBER),0)+1 INTO h_number:n      FROM PROFILES:n   +
1   WHERE PROJECT_ID=prj_id/**prj_id*/ AND ROLE_ID=h_role_id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
167:
1           EXCEPTION:: SP_I_PROFILE(68) :: cplan.sql(5615) :: :
1SQLCODE:: :
1 SQLERR:: :
1SQLERRM:
1500:
1PMO_ORASTD:
1STRINGCONCAT:
1RTRIM:
1TCP:
1ELEMENT_NAME:
1TC:
1TMT_COMPETENCIES:
1ELEMENT_ID:
1PARENT_ID:
1SELECT PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConca+
1t(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(TCP.ELEMENT_NAME), ',+
1 '), RTRIM(TC.ELEMENT_NAME)), '('), RTRIM(TO_CHAR(h_number))), ')'):n      IN+
1TO h_full_name FROM TMT_COMPETENCIES TC, TMT_COMPETENCIES TCP:n      WHERE TC+
1.ELEMENT_ID = h_role_id AND TCP.ELEMENT_ID = TC.PARENT_ID:
184:
1           EXCEPTION:: SP_I_PROFILE(85) :: cplan.sql(5632) :: :
1=:
1SC_SQL_NO_DATA:
192:
1        RETURN_ERROR:: SP_I_PROFILE(93) :: cplan.sql(5640) :: :
1RECORD_NOT_FOUND:
1 ID:: :
195:
1                EXIT:: SP_I_PROFILE(96) :: cplan.sql(5643) :: :
1PMO_RETURN_ERROR_SP:
1SYS_GUID:
1PROFILE_ID:
1PROFILE_NAME:
1POOL_ID:
1GEO_ID:
1ORG_ID:
1INSERT INTO PROFILES(PROFILE_ID,PROJECT_ID,ROLE_ID,PROF_NUMBER, PROFILE_NAME,+
1 POOL_ID, GEO_ID, ORG_ID):n      VALUES(h_uniqueID, prj_id, h_role_id, h_numb+
1er, h_full_name, pool_id001, geo_id001, org_id001):
1FOUND:
1100:
1118:
1           EXCEPTION:: SP_I_PROFILE(119) :: cplan.sql(5666) :: :
1SC_SQL_SUCCESS:
1TRUNC:
1LENGTH:
1/:
1>:
1WHILE:
1<:
1LOOP:
1PMO_STPU:
1STRNPCPY:
1*:
1RESOURCE_ASSIGN_ROLES:
1PROFICIENCY_LEVEL_ID:
1PREFERENCE_LEVEL_ID:
1EXPERIENCE_ID:
1LAST_USED_DATETIME:
1INSERT INTO RESOURCE_ASSIGN_ROLES:n               SELECT h_uniqueID, ELEMENT_+
1ID, PROFICIENCY_LEVEL_ID, PREFERENCE_LEVEL_ID,:n               EXPERIENCE_ID,+
1 LAST_USED_DATETIME, TO_CHAR(h_count):n               FROM TMT_COMPETENCIES W+
1HERE ELEMENT_ID = h_role_id:
1151:
1           EXCEPTION:: SP_I_PROFILE(152) :: cplan.sql(5699) :: :
1IS NOT NULL:
1RESOURCE_ASSIGN_SKILLS:
1INSERT INTO RESOURCE_ASSIGN_SKILLS:n                  SELECT h_uniqueID, h_ro+
1le_id, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID,:n                  PREFERENCE_LEV+
1EL_ID, LAST_USED_DATETIME, TO_CHAR(h_count):n                  FROM TMT_COMPE+
1TENCIES WHERE ELEMENT_ID = h_role_id:
1190:
1           EXCEPTION:: SP_I_PROFILE(191) :: cplan.sql(5738) :: :
14:
1202:
1       RETURN_STATUS:: SP_I_PROFILE(203) :: cplan.sql(5750) :: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
1209:
1           EXCEPTION:: SP_I_PROFILE(210) :: cplan.sql(5757) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1ID:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1221:
1           EXCEPTION:: SP_I_PROFILE(222) :: cplan.sql(5769) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
603
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 81
b0 a3 a0 1c 81 b0 a3 :2 a0
6b 1c 81 b0 :2 a0 6b :2 a0 6b
6e a5 b a 10 :2 a0 6b 6e
:2 a0 6b 6e a5 b 6e 7e a0
b4 2e 6e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c a0 7e b4 2e :2 5a
a0 7e b4 2e :2 5a 52 10 :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b a0 a5 57 b7 19
3c :2 a0 d :4 a0 :2 51 a5 b :2 51
a5 b d :9 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :1f a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
6b a0 7e a0 6b b4 2e 5a
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 b7
19 3c :2 a0 b4 2e d :11 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a :5 a0
a5 b 51 a5 b 7e 51 b4
2e 5a 7e 51 b4 2e a5 b
d a0 7e 51 b4 2e a0 51
d :3 a0 51 a5 b a0 7e a0
b4 2e 82 :3 a0 6b :2 51 :2 a0 51
a5 b 7e 51 b4 2e 7e 51
b4 2e a0 a5 b d :c a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :3 a0 51
a5 b 7e 51 b4 2e d b7
a0 47 b7 19 3c a0 7e b4
2e 5a :2 a0 d :5 a0 a5 b 51
a5 b 7e 51 b4 2e 5a 7e
51 b4 2e a5 b d a0 7e
51 b4 2e a0 51 d :3 a0 51
a5 b a0 7e a0 b4 2e 82
:3 a0 6b :2 51 :2 a0 51 a5 b 7e
51 b4 2e 7e 51 b4 2e a0
a5 b d :c a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :3 a0 51 a5 b 7e
51 b4 2e d b7 a0 47 b7
19 3c b7 19 3c b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 :2 a0 6b :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 :2 a0
6b :2 6e 7e :2 a0 6b b4 2e 7e
6e b4 2e 7e :2 a0 6b b4 2e
a5 57 b7 19 3c :9 a0 12a e9
dd :2 a0 65 b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c :7 a0 12a e9 dd :2 a0
65 b7 a4 b1 11 4f b7 a6
9 a4 b1 11 68 4f 1d 17
b5 
603
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 6f 6b 59 77 84
80 56 8c 95 91 7f 9d 7c
a2 a6 aa ae b1 b5 d2 bd
c1 c4 c5 cd bc ef dd b9
e1 e2 ea dc 10c fa d9 fe
ff 107 f9 129 117 f6 11b 11c
124 116 145 134 138 113 140 133
161 150 154 130 15c 14f 17d 16c
170 178 14c 195 184 188 190 16b
1b5 1a0 1a4 168 1a8 1b0 19f 1bc
1c0 19c 1c4 1c8 1cc 1cf 1d4 1d5
1 1d7 1dc 1e0 1e4 1e7 1ec 1f0
1f4 1f7 1fc 1fd 1ff 204 207 20b
20c 211 216 219 21e 21f 224 227
22b 22c 231 234 239 23a 23f 242
247 248 24d 250 255 256 25b 25e
262 263 268 26b 270 271 276 279
27e 27f 284 287 28c 28d 292 295
299 29a 29f 2a2 2a7 2a8 2ad 2b0
2b5 2b6 2bb 2be 2c3 2c4 2c9 2cc
2d0 2d1 2d6 2d9 2de 2df 2e4 2e7
2ec 2ed 2f2 2f5 2fa 2fb 300 303
307 308 30d 310 315 316 31b 31e
323 324 329 32c 331 332 337 33a
33e 33f 344 347 34c 34d 352 355
35a 35b 360 363 368 369 36e 371
375 376 37b 37e 383 384 389 38c
391 392 397 398 39d 39f 3a3 3a6
3aa 3ae 3b1 3b5 3b9 3bc 3bf 3c4
3c7 3ca 3cd 3ce 3d3 3d4 1 3d6
3db 3df 3e3 3e6 3eb 3f0 3f3 3f7
3f8 3fd 3fe 403 405 409 40c 410
413 414 419 41c 41f 423 426 427
42c 42f 1 432 437 43b 43f 442
446 44a 44d 450 455 458 45b 45e
45f 464 465 1 467 46c 470 474
477 47c 481 484 488 489 48e 491
496 497 49c 49f 4a4 4a5 4aa 4ad
4b1 4b5 4b9 4bd 4c0 4c1 4c3 4c8
4c9 4cb 4cc 4d1 4d2 4d7 4d9 4dd
4e0 4e4 4e8 4eb 4ef 4f3 4f6 4f9
4fe 501 504 507 508 50d 50e 1
510 515 519 51d 520 525 52a 52d
531 532 537 538 53d 53f 543 546
54a 54e 551 555 559 55c 560 561
566 568 56c 56f 573 577 57b 57f
583 587 58b 58e 591 592 594 597
59a 59b 59d 5a1 5a5 5a9 5ad 5b1
5b5 5b9 5bd 5c1 5c5 5d1 5d5 5d9
5dc 5e0 5e4 5e6 5ea 5ee 5f2 5f5
5f9 5fd 5ff 600 605 1 609 60d
611 614 618 61c 620 624 627 62b
62f 632 635 63a 63d 640 643 644
649 64a 1 64c 651 655 659 65c
661 666 669 66d 66e 673 676 67b
67c 681 684 689 68a 68f 692 696
697 69c 69f 6a4 6a5 6aa 6ad 6b1
6b5 6b8 6bb 6bc 6be 6bf 6c4 6c5
6ca 6cc 6d0 6d3 6d5 6d9 6db 6e7
6e9 6eb 6ec 6f1 6f5 6f7 703 705
709 70d 711 715 719 71d 721 725
729 72d 731 735 739 73d 741 745
749 74d 751 755 759 75d 761 765
769 76d 771 775 779 77d 781 78d
791 795 798 79c 7a0 7a2 7a6 7aa
7ae 7b1 7b5 7b9 7bb 7bc 7c1 1
7c5 7c9 7cd 7d0 7d4 7d8 7dc 7e0
7e3 7e7 7eb 7ee 7f1 7f6 7f9 7fc
7ff 800 805 806 1 808 80d 811
815 818 81d 822 825 829 82a 82f
832 837 838 83d 840 845 846 84b
84e 852 853 858 85b 860 861 866
869 86d 871 874 877 878 87a 87b
880 881 886 888 88c 88f 891 895
897 8a3 8a5 8a7 8a8 8ad 8b1 8b3
8bf 8c1 8c5 8c9 8cc 8d0 8d3 8d7
8da 8db 8e0 8e3 8e7 8eb 8ee 8f2
8f6 8f9 8fc 901 904 907 90a 90b
910 911 1 913 918 91c 920 923
928 92d 930 934 935 93a 93d 942
943 948 94b 950 951 956 959 95d
961 965 969 96c 96d 96f 974 975
977 978 97d 980 985 986 98b 98e
993 994 999 99c 9a0 9a1 9a6 9a9
9ae 9af 9b4 9b5 9ba 9bc 9c0 9c3
9c7 9cb 9ce 9d2 9d6 9d9 9dc 9e1
9e4 9e7 9ea 9eb 9f0 9f1 1 9f3
9f8 9fc a00 a03 a08 a0d a10 a14
a15 a1a a1b a20 a22 a26 a29 a2d
a31 a34 a38 a3c a3f a43 a47 a48
a4d a4f a53 a56 a5a a5e a5f a64
a68 a6c a70 a74 a78 a7c a80 a84
a88 a8c a90 a94 a98 a9c aa0 aa4
aa8 aac ab8 abc ac1 ac5 ac8 ac9
ace ad2 ad6 ad9 adc ae0 ae2 ae6
aea aed af0 af4 af6 afa afe b01
b03 b07 b0b b0f b12 b16 b1a b1c
b1d b22 1 b26 b2a b2e b31 b35
b39 b3d b41 b44 b48 b4c b4f b52
b57 b5a b5d b60 b61 b66 b67 1
b69 b6e b72 b76 b79 b7e b83 b86
b8a b8b b90 b93 b98 b99 b9e ba1
ba6 ba7 bac baf bb3 bb4 bb9 bbc
bc1 bc2 bc7 bca bce bd2 bd5 bd8
bd9 bdb bdc be1 be2 be7 be9 bed
bf0 bf2 bf6 bf8 c04 c06 c08 c09
c0e c12 c14 c20 c22 c26 c2a c2d
c31 c34 c38 c3b c3c c41 c44 c48
c4c c50 c54 c58 c59 c5b c5e c5f
c61 c64 c67 c68 c6d c70 c73 c76
c77 c7c c7d c7f c83 c87 c8a c8d
c8e c93 c97 c9a c9e ca2 ca6 caa
cad cae cb0 cb4 cb7 cbb cbc cc1
cc3 cc7 ccb ccf cd2 cd5 cd8 cdc
ce0 ce3 ce4 ce6 ce9 cec ced cf2
cf5 cf8 cf9 cfe d02 d03 d05 d09
d0d d11 d15 d19 d1d d21 d25 d29
d2d d31 d35 d39 d45 d49 d4e d52
d55 d56 d5b d5f d63 d66 d69 d6d
d6f d73 d77 d7a d7d d81 d83 d87
d8b d8e d90 d94 d98 d9c d9f da3
da7 da9 daa daf 1 db3 db7 dbb
dbe dc2 dc6 dca dce dd1 dd5 dd9
ddc ddf de4 de7 dea ded dee df3
df4 1 df6 dfb dff e03 e06 e0b
e10 e13 e17 e18 e1d e20 e25 e26
e2b e2e e33 e34 e39 e3c e40 e41
e46 e49 e4e e4f e54 e57 e5b e5f
e62 e65 e66 e68 e69 e6e e6f e74
e76 e7a e7d e7f e83 e85 e91 e93
e95 e96 e9b e9f ea1 ead eaf eb3
eb7 ebb ebe ebf ec1 ec4 ec7 ec8
ecd ed1 ed3 ed7 ede ee0 ee4 ee7
eeb eee eef ef4 ef7 efb eff f03
f07 f0b f0f f13 f17 f18 f1a f1d
f1e f20 f23 f26 f27 f2c f2f f32
f35 f36 f3b f3c f3e f42 f46 f49
f4c f4d f52 f56 f59 f5d f61 f65
f69 f6c f6d f6f f73 f76 f7a f7b
f80 f82 f86 f8a f8e f91 f94 f97
f9b f9f fa2 fa3 fa5 fa8 fab fac
fb1 fb4 fb7 fb8 fbd fc1 fc2 fc4
fc8 fcc fd0 fd4 fd8 fdc fe0 fe4
fe8 fec ff0 ff4 ff8 1004 1008 100d
1011 1014 1015 101a 101e 1022 1025 1028
102c 102e 1032 1036 1039 103c 1040 1042
1046 104a 104d 104f 1053 1057 105b 105e
1062 1066 1068 1069 106e 1 1072 1076
107a 107d 1081 1085 1089 108d 1090 1094
1098 109b 109e 10a3 10a6 10a9 10ac 10ad
10b2 10b3 1 10b5 10ba 10be 10c2 10c5
10ca 10cf 10d2 10d6 10d7 10dc 10df 10e4
10e5 10ea 10ed 10f2 10f3 10f8 10fb 10ff
1100 1105 1108 110d 110e 1113 1116 111a
111e 1121 1124 1125 1127 1128 112d 112e
1133 1135 1139 113c 113e 1142 1144 1150
1152 1154 1155 115a 115e 1160 116c 116e
1172 1176 117a 117d 117e 1180 1183 1186
1187 118c 1190 1192 1196 119d 119f 11a3
11a6 11a8 11ac 11af 11b1 11b5 11b8 11bc
11c0 11c3 11c7 11cb 11ce 11d1 11d6 11d9
11dc 11df 11e0 11e5 11e6 1 11e8 11ed
11f1 11f5 11f8 11fd 1202 1205 1209 120a
120f 1212 1217 1218 121d 1220 1225 1226
122b 122e 1232 1236 123a 123e 1241 1242
1244 1249 124a 124c 124d 1252 1255 125a
125b 1260 1263 1268 1269 126e 1271 1275
1276 127b 127e 1283 1284 1289 128a 128f
1291 1295 1298 129c 12a0 12a3 12a7 12ab
12ae 12b2 12b6 12b7 12bc 12be 12c2 12c6
12c9 12cd 12d1 12d4 12d8 12dc 12df 12e2
12e7 12ea 12ed 12f0 12f1 12f6 12f7 1
12f9 12fe 1302 1306 1309 130e 1313 1316
131a 131b 1320 1321 1326 132a 132e 1331
1336 133b 133e 1342 1346 1349 134a 134f
1352 1357 1358 135d 1360 1364 1368 136b
136c 1371 1372 1377 1379 137d 1380 1384
1388 138c 1390 1394 1398 139c 13a0 13a4
13b0 13b5 13b9 13bd 13c1 13c5 13c7 13c8
13cd 1 13d1 13d5 13d9 13dc 13e0 13e4
13e8 13ec 13ef 13f3 13f7 13fa 13fd 1402
1405 1408 140b 140c 1411 1412 1 1414
1419 141d 1421 1424 1429 142e 1431 1435
1436 143b 143c 1441 1445 1449 144c 1451
1456 1459 145d 145e 1463 1466 146b 146c
1471 1474 1478 147c 147f 1482 1483 1485
1486 148b 148c 1491 1493 1497 149a 149e
14a2 14a6 14aa 14ae 14b2 14b6 14c2 14c7
14cb 14cf 14d3 14d7 14d9 14dd 14df 14eb
14ed 14ef 14f0 14f5 14f9 14fb 1507 150b
150d 150e 1517 
603
2
0 1 a 6 d :3 6 10 :3 6
11 :3 6 10 :3 6 10 :3 6 11 :3 6
12 :2 6 1 3 a 13 a :2 1
4 f 18 17 :2 f :2 4 10 19
18 :2 10 :2 4 e 17 16 :2 e :2 4
10 19 18 :2 10 :2 4 :2 c 1b c
:2 4 :2 d 1c d :2 4 :3 d :2 4 :3 10
:2 4 e 17 :3 e 4 7 :2 11 23
:2 2d 3a :2 23 :3 7 :2 d 19 25 :2 2f
43 :2 25 54 5a 5d :2 54 6b 81
8 :2 6b d 10 :2 6b 17 1a :2 6b
b e :2 6b 13 16 :2 6b 1b 1e
:2 6b 28 2b :2 6b b e :2 6b 13
16 :2 6b 1b 1e :2 6b 29 2c :2 6b
b e :2 6b 13 16 :2 6b 1b 1e
:2 6b 28 2b :2 6b b e :2 6b 13
16 :2 6b 1b 1e :2 6b 28 2b :2 6b
b e :2 6b 13 16 :2 6b 1b 1e
:2 6b 29 2c :2 6b b e :2 6b 13
16 :2 6b 1b 1e :2 6b 2a 2d :2 6b
32 35 :2 6b :2 7 4a :2 4 7 :2 11
23 :2 2d 39 3c 4c 4e 4f :2 4c
:2 23 :3 7 :2 d 12 1e 5c 5f :2 1e
:2 7 52 :2 4 :4 9 8 7 :4 1f 1e
1d :2 7 a :2 14 26 :2 30 3c 3f
4f 51 52 :2 4f :2 26 :3 a :2 10 15
21 5f 62 :2 21 6f 72 :2 21 76
79 :2 21 83 86 8f 97 :2 a0 :2 8f
b1 :2 86 :2 21 :2 a 55 :2 7 a :2 14
26 :2 30 3c 3f 4f 51 52 :2 4f
:2 26 :3 a :2 10 15 21 5f 62 :2 21
:2 a 55 :3 7 :2 12 28 :2 31 41 :2 7
33 :3 4 13 :2 4 11 18 1f 2c
2f :2 18 34 37 :2 11 4 e 17
1b 32 c d 18 2e 36 :2 7
:2 10 1c 7 4 c a :2 13 1f
a 1a :2 7 :2 c a :2 13 1f a
d :2 17 29 :2 33 3f 43 53 55
56 :2 53 :2 29 :3 d :2 13 18 24 62
65 :2 24 72 75 :2 24 79 7c :2 24
88 8b :2 24 93 96 :2 24 a2 a5
ac b5 b8 :2 a5 :2 24 :2 d 59 :2 a
:2 7 :4 13 :2 7 4 :3 1 e 19 26
31 3e 49 56 61 6e 79 86
8c 90 a6 ac af c5 cb d3
c 1d 2e 32 43 d 10 1d
2b 2f 3c 3f :2 7 :2 10 1c 7
4 c a :2 13 1f a 1a :2 7
:2 c a :2 13 1f a d :2 17 29
:2 33 3f 43 53 55 56 :2 53 :2 29
:3 d :2 13 18 24 62 65 :2 24 72
75 :2 24 79 7c :2 24 88 8b :2 24
93 96 :2 24 a2 a5 ac b5 b8
:2 a5 :2 24 :2 d 59 :2 a :2 7 :4 13 :2 7
4 :3 1 8 :2 11 1c 1a :2 25 :2 1a
7 a :2 14 26 :2 30 3c 40 50
52 53 :2 50 :2 26 :3 a :2 10 15 21
5f 62 :2 21 6f 72 :2 21 76 79
:2 21 83 86 8f 97 :2 a0 :2 8f b3
:2 86 :2 21 bb be :2 21 c6 c9 :2 21
ce d1 :2 21 db de :2 21 :2 a 56
:2 7 a :2 14 26 :2 30 3c 3f 4f
51 52 :2 4f :2 26 :3 a :2 10 15 21
5f 62 :2 21 :2 a 55 :3 7 :2 12 26
:2 2f 41 4c :2 7 35 :3 4 :3 12 4
13 1c 27 32 3a 47 55 5e
66 e 1a 22 2d 37 44 50
5b 7 e a 16 :3 14 a :2 13
1f a 1b a :2 13 1f a :4 7
4 c a :2 13 1f a 1a :2 7
:2 c a :2 13 1f a d :2 17 29
:2 33 3f 43 53 56 57 :2 53 :2 29
:3 d :2 13 18 24 63 66 :2 24 73
76 :2 24 7a 7d :2 24 89 8c :2 24
94 97 :2 24 a3 a6 ad b6 b9
:2 a6 :2 24 :2 d 5a :2 a :2 7 :4 13 :2 7
4 :3 1 8 :2 11 1c 1a :2 25 :2 1a
:2 7 13 1a 23 2a :2 23 38 :2 1a
3b 3d :2 1a 19 40 42 :2 19 :2 13
7 a 13 15 :2 13 a 15 :2 a
10 19 22 :2 10 27 25 30 :2 25
a d 1a :2 23 2c 30 34 3d
46 :2 34 49 4b :2 34 4d 4e :2 34
51 :2 1a d 1c 17 23 2f 45
10 1f 33 3b 15 2c 39 10
17 13 1f :3 1d 13 :2 1c 28 13
24 13 :2 1c 28 13 :4 10 d 15
13 :2 1c 28 13 23 :2 10 :2 15 13
:2 1c 28 13 16 :2 20 32 :2 3c 48
4c 5c 5f 60 :2 5c :2 32 :3 16 :2 1c
21 2d 6c 6f :2 2d 7c 7f :2 2d
83 86 :2 2d 92 95 :2 2d 9d a0
:2 2d ac af b6 bf c2 :2 af :2 2d
:2 16 63 :2 13 :2 10 :4 1c :2 10 d :3 30
d 18 21 2a :2 18 2d 2f :2 18
d 30 e a 17 :2 7 :4 b :2 a
19 :2 a 19 20 29 30 :2 29 3e
:2 20 41 43 :2 20 1f 46 48 :2 1f
:2 19 a d 19 1b :2 19 d 18
:2 d 13 1c 25 :2 13 2a 28 36
:2 28 d 10 1d :2 26 2f 33 37
40 49 :2 37 4c 4e :2 37 50 51
:2 37 54 :2 1d 10 1f 1a 26 31
47 13 28 3c 44 18 2f 3c
13 1a 16 22 :3 20 16 :2 1f 2b
16 27 16 :2 1f 2b 16 :4 13 10
18 16 :2 1f 2b 16 26 :2 13 :2 18
16 :2 1f 2b 16 19 :2 23 35 :2 3f
4b 4f 5f 62 63 :2 5f :2 35 :3 19
:2 1f 24 30 6f 72 :2 30 7f 82
:2 30 86 89 :2 30 95 98 :2 30 a0
a3 :2 30 af b2 b9 c2 c5 :2 b2
:2 30 :2 19 66 :2 16 :2 13 :4 1f :2 13 10
:3 36 10 1b 24 2d :2 1b 30 32
:2 1b 10 36 11 d 1d :2 a 23
:2 7 35 :2 4 7 :2 11 23 :2 2d 39
3c 4c 4f 50 :2 4c :2 23 :3 7 :2 d
12 1e 5d 60 :2 1e 6d 70 :2 1e
74 77 :2 1e 81 84 8d 95 :2 9e
:2 8d a9 :2 84 :2 1e b1 b4 :2 1e bc
bf :2 1e c4 c7 :2 1e d2 d5 :2 1e
:2 7 53 :3 4 :2 f 21 :2 2a 34 40
:2 4 1 9 12 9 a :2 14 26
:2 30 3c 40 50 53 54 :2 50 :2 26
:3 a :2 10 15 21 60 63 :2 21 :3 a
:2 10 15 39 5c 5f :2 68 :2 39 76
79 :2 39 81 84 :2 8d :2 39 :2 a 57
:3 7 c 15 1e 2c a 13 1e
f e 7 e 7 e 7 29
:2 4 :2 9 7 :2 10 1c :2 7 :2 11 23
:2 2d 39 3d 4d 50 51 :2 4d :2 23
:3 7 :2 d 12 1e 5d 60 :2 1e :3 7
:2 d 12 36 42 45 :2 36 4d 50
:2 36 5c 5f 66 6f 72 :2 5f :2 36
:2 7 54 :2 4 7 c 1b a 13
1e f e 7 e 7 e 7
:2 4 :4 10 :2 4 :8 1 
603
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 :4 7 :4 8
:4 9 2 :4 a :2 1
:7 c :7 d :7 e :7 f
:6 10 :6 11 :5 12 :5 13
:7 15 :b 17 :11 18 19
:2 18 :2 19 :2 18 :2 19
:2 18 :2 1a :2 18 :2 1a
:2 18 :2 1a :2 18 :2 1a
:2 18 :2 1b :2 18 :2 1b
:2 18 :2 1b :2 18 :2 1b
:2 18 :2 1c :2 18 :2 1c
:2 18 :2 1c :2 18 :2 1c
:2 18 :2 1d :2 18 :2 1d
:2 18 :2 1d :2 18 :2 1d
:2 18 :2 1e :2 18 :2 1e
:2 18 :2 1e :2 18 :2 1e
:2 18 :2 1f :2 18 :2 1f
:2 18 :2 1f :2 18 :2 1f
:2 18 :2 1f :4 18 :3 17
:11 22 :b 23 :3 22 :e 28
:11 29 :20 2a :3 29 :11 2c
:b 2d :3 2c :9 2f :3 28
:3 33 :d 36 :4 39 3a
:4 3b 39 :5 3c 38
3e :5 3f :3 3e :2 40
:5 42 :11 43 :24 44 :3 43
:2 41 :6 40 3d :3 16
:13 4a :5 4b :7 4c 4a
:5 4d 49 4f :5 50
:3 4f :2 51 :5 53 :11 54
:24 55 :3 54 :2 52 :6 51
4e :3 16 :a 5b :11 5c
:30 5d :3 5c :11 5f :b 60
:3 5f :a 62 :3 5b :5 66
:9 69 :8 6a 69 :6 6b
:5 6c 6b :5 6e 6d
:3 6b 68 71 :5 72
:3 71 :2 73 :5 75 :11 76
:24 77 :3 76 :2 74 :6 73
70 :3 16 :a 7c :16 7e
:5 7f :3 80 :c 81 :17 85
88 :4 89 :4 8a :3 8b
88 :6 8c :5 8d 8c
:5 8f 8e :3 8c 87
92 :5 93 :3 92 :2 94
:5 96 :11 97 :24 98 :3 97
:2 95 :6 94 91 :3 81
:b 9c 81 9d 81
:3 7f :5 a1 :3 a3 :16 a4
:5 a6 :3 a7 :c a8 :17 ac
af :4 b0 :4 b1 :3 b2
af :6 b3 :5 b4 b3
:5 b6 b5 :3 b3 ae
b9 :5 ba :3 b9 :2 bb
:5 bd :11 be :24 bf :3 be
:2 bc :6 bb b8 :3 a8
:b c4 a8 c5 a8
:3 a6 :3 a1 :3 7c :11 ca
:30 cb :3 ca :a cd 16
:3 d0 :11 d1 :b d2 :17 d3
:3 d1 :2 d5 :3 d6 :3 d7
d8 d6 d5 d6
:3 d9 :3 d0 :2 da :5 dc
:11 dd :b de :18 df :3 dd
:2 e1 e2 :3 e3 e4
e2 e1 e2 :3 e5
:2 db :6 da cf :7 1

1519
4
:3 0 1 :3 0 2
:a 0 5fe 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 4
:3 0 5 :7 0 9
8 :3 0 4 :3 0
6 :7 0 d c
:3 0 d 7c 0
b 4 :3 0 7
:7 0 11 10 :3 0
4 :3 0 8 :7 0
15 14 :3 0 11
:2 0 f 4 :3 0
9 :7 0 19 18
:3 0 4 :3 0 a
:7 0 1d 1c :3 0
b :3 0 c :3 0
d :2 0 4 21
22 0 1f 23
0 5fe 2 24
:2 0 11 :2 0 1b
4 :3 0 f :2 0
19 27 29 :6 0
2c 2a 0 5fc
0 e :6 0 f
:2 0 1f 4 :3 0
1d 2e 30 :6 0
33 31 0 5fc
0 10 :6 0 14
:2 0 23 4 :3 0
21 35 37 :6 0
3a 38 0 5fc
0 12 :6 0 17
:2 0 27 4 :3 0
25 3c 3e :6 0
41 3f 0 5fc
0 13 :6 0 17
:2 0 29 16 :3 0
43 :7 0 47 44
45 5fc 0 15
:6 0 2d 168 0
2b 16 :3 0 49
:7 0 4d 4a 4b
5fc 0 18 :6 0
59 5a 0 2f
16 :3 0 4f :7 0
52 50 0 5fc
0 19 :6 0 16
:3 0 54 :7 0 57
55 0 5fc 0
1a :6 0 5f 60
0 31 c :3 0
d :2 0 4 5b
:7 0 5e 5c 0
5fc 0 1b :6 0
1c :3 0 1d :3 0
1c :3 0 1e :3 0
62 63 0 2
:4 0 33 64 66
61 68 67 :2 0
1f :3 0 20 :3 0
6a 6b 0 21
:4 0 1c :3 0 22
:3 0 6e 6f 0
2 :4 0 35 70
72 23 :4 0 24
:2 0 25 :3 0 37
75 77 :3 0 26
:4 0 24 :2 0 27
:4 0 3a 7a 7c
:3 0 24 :2 0 3
:3 0 3d 7e 80
:3 0 24 :2 0 27
:4 0 40 82 84
:3 0 24 :2 0 28
:4 0 43 86 88
:3 0 24 :2 0 27
:4 0 46 8a 8c
:3 0 24 :2 0 5
:3 0 49 8e 90
:3 0 24 :2 0 27
:4 0 4c 92 94
:3 0 24 :2 0 28
:4 0 4f 96 98
:3 0 24 :2 0 27
:4 0 52 9a 9c
:3 0 24 :2 0 6
:3 0 55 9e a0
:3 0 24 :2 0 27
:4 0 58 a2 a4
:3 0 24 :2 0 28
:4 0 5b a6 a8
:3 0 24 :2 0 27
:4 0 5e aa ac
:3 0 24 :2 0 7
:3 0 61 ae b0
:3 0 24 :2 0 27
:4 0 64 b2 b4
:3 0 24 :2 0 28
:4 0 67 b6 b8
:3 0 24 :2 0 27
:4 0 6a ba bc
:3 0 24 :2 0 8
:3 0 6d be c0
:3 0 24 :2 0 27
:4 0 70 c2 c4
:3 0 24 :2 0 28
:4 0 73 c6 c8
:3 0 24 :2 0 27
:4 0 76 ca cc
:3 0 24 :2 0 9
:3 0 79 ce d0
:3 0 24 :2 0 27
:4 0 7c d2 d4
:3 0 24 :2 0 28
:4 0 7f d6 d8
:3 0 24 :2 0 27
:4 0 82 da dc
:3 0 24 :2 0 a
:3 0 85 de e0
:3 0 24 :2 0 27
:4 0 88 e2 e4
:3 0 24 :2 0 29
:4 0 8b e6 e8
:3 0 8e 6c ea
:2 0 ec 93 ed
69 ec 0 ee
95 0 55c 1c
:3 0 1d :3 0 ef
f0 0 1c :3 0
2a :3 0 f2 f3
0 2b :2 0 2
:4 0 2c :2 0 2d
:2 0 2b :2 0 97
f8 fa :3 0 9a
f4 fc f1 fe
fd :2 0 1f :3 0
2e :3 0 100 101
0 21 :4 0 2f
:4 0 24 :2 0 25
:3 0 9e 105 107
:3 0 a1 102 109
:2 0 10b a4 10c
ff 10b 0 10d
a6 0 55c 3
:3 0 30 :2 0 a8
10f 110 :3 0 111
:2 0 112 :2 0 5
:3 0 30 :2 0 aa
115 116 :3 0 117
:2 0 118 :2 0 113
11a 119 :2 0 1c
:3 0 1d :3 0 11c
11d 0 1c :3 0
2a :3 0 11f 120
0 31 :2 0 2
:4 0 32 :2 0 2d
:2 0 2b :2 0 ac
125 127 :3 0 af
121 129 11e 12b
12a :2 0 1f :3 0
2e :3 0 12d 12e
0 21 :4 0 33
:4 0 24 :2 0 25
:3 0 b3 132 134
:3 0 24 :2 0 34
:4 0 b6 136 138
:3 0 24 :2 0 35
:4 0 b9 13a 13c
:3 0 24 :2 0 36
:3 0 37 :3 0 38
:3 0 39 :3 0 141
142 0 bc 140
144 3a :4 0 be
13f 147 c1 13e
149 :3 0 c4 12f
14b :2 0 14d c7
14e 12c 14d 0
14f c9 0 178
1c :3 0 1d :3 0
150 151 0 1c
:3 0 2a :3 0 153
154 0 2b :2 0
2 :4 0 3b :2 0
2d :2 0 2b :2 0
cb 159 15b :3 0
ce 155 15d 152
15f 15e :2 0 1f
:3 0 2e :3 0 161
162 0 21 :4 0
3c :4 0 24 :2 0
25 :3 0 d2 166
168 :3 0 d5 163
16a :2 0 16c d8
16d 160 16c 0
16e da 0 178
3d :3 0 3e :3 0
16f 170 0 38
:3 0 39 :3 0 172
173 0 3f :3 0
dc 171 176 :2 0
178 df 179 11b
178 0 17a e3
0 55c 10 :3 0
5 :3 0 17b 17c
0 55c 12 :3 0
40 :3 0 40 :3 0
10 :3 0 2b :2 0
f :2 0 e5 180
184 2b :2 0 f
:2 0 e9 17f 188
17e 189 0 55c
36 :3 0 41 :3 0
42 :3 0 18 :3 0
43 :3 0 44 :3 0
3 :3 0 45 :3 0
12 :4 0 46 1
:8 0 19a c :3 0
47 :3 0 195 196
0 48 :3 0 197
198 0 19a ed
1ed 49 :3 0 c
:3 0 47 :3 0 19c
19d 0 48 :3 0
19e 19f 0 1a1
f0 1a3 f2 1a2
1a1 :2 0 1eb 4a
:3 0 c :3 0 47
:3 0 1a6 1a7 0
48 :3 0 1a8 1a9
0 1e3 1c :3 0
1d :3 0 1ab 1ac
0 1c :3 0 2a
:3 0 1ae 1af 0
4b :2 0 2 :4 0
4c :2 0 2d :2 0
2b :2 0 f4 1b4
1b6 :3 0 f7 1b0
1b8 1ad 1ba 1b9
:2 0 1f :3 0 2e
:3 0 1bc 1bd 0
21 :4 0 4d :4 0
24 :2 0 25 :3 0
fb 1c1 1c3 :3 0
24 :2 0 34 :4 0
fe 1c5 1c7 :3 0
24 :2 0 4e :4 0
101 1c9 1cb :3 0
24 :2 0 48 :3 0
104 1cd 1cf :3 0
24 :2 0 4f :4 0
107 1d1 1d3 :3 0
24 :2 0 40 :3 0
50 :3 0 2b :2 0
51 :2 0 10a 1d6
1da 10e 1d5 1dc
:3 0 111 1be 1de
:2 0 1e0 114 1e1
1bb 1e0 0 1e2
116 0 1e3 118
1e6 :3 0 1e6 0
1e6 1e5 1e3 1e4
:6 0 1e8 2 :3 0
11b 1ea 11d 1e9
1e8 :2 0 1eb 11f
:2 0 1ed 0 1ed
1ec 19a 1eb :6 0
55c 1 :3 0 52
:3 0 53 :3 0 52
:3 0 53 :3 0 52
:3 0 53 :3 0 52
:3 0 53 :3 0 52
:3 0 53 :3 0 54
:3 0 55 :3 0 56
:3 0 54 :3 0 57
:3 0 56 :3 0 54
:3 0 37 :3 0 18
:3 0 13 :3 0 58
:3 0 57 :3 0 58
:3 0 55 :3 0 57
:3 0 59 :3 0 12
:3 0 55 :3 0 59
:3 0 57 :3 0 5a
:4 0 5b 1 :8 0
214 c :3 0 47
:3 0 20f 210 0
48 :3 0 211 212
0 214 122 267
49 :3 0 c :3 0
47 :3 0 216 217
0 48 :3 0 218
219 0 21b 125
21d 127 21c 21b
:2 0 265 4a :3 0
c :3 0 47 :3 0
220 221 0 48
:3 0 222 223 0
25d 1c :3 0 1d
:3 0 225 226 0
1c :3 0 2a :3 0
228 229 0 4b
:2 0 2 :4 0 5c
:2 0 2d :2 0 2b
:2 0 129 22e 230
:3 0 12c 22a 232
227 234 233 :2 0
1f :3 0 2e :3 0
236 237 0 21
:4 0 5d :4 0 24
:2 0 25 :3 0 130
23b 23d :3 0 24
:2 0 34 :4 0 133
23f 241 :3 0 24
:2 0 4e :4 0 136
243 245 :3 0 24
:2 0 48 :3 0 139
247 249 :3 0 24
:2 0 4f :4 0 13c
24b 24d :3 0 24
:2 0 40 :3 0 50
:3 0 2b :2 0 51
:2 0 13f 250 254
143 24f 256 :3 0
146 238 258 :2 0
25a 149 25b 235
25a 0 25c 14b
0 25d 14d 260
:3 0 260 0 260
25f 25d 25e :6 0
262 4 :3 0 150
264 152 263 262
:2 0 265 154 :2 0
267 0 267 266
214 265 :6 0 55c
1 :3 0 c :3 0
47 :3 0 269 26a
0 c :3 0 5e
:2 0 5f :3 0 26c
26e 0 159 26d
270 :3 0 271 :2 0
1c :3 0 1d :3 0
273 274 0 1c
:3 0 2a :3 0 276
277 0 f :2 0
2 :4 0 60 :2 0
2d :2 0 2b :2 0
15c 27c 27e :3 0
15f 278 280 275
282 281 :2 0 1f
:3 0 2e :3 0 284
285 0 21 :4 0
61 :4 0 24 :2 0
25 :3 0 163 289
28b :3 0 24 :2 0
34 :4 0 166 28d
28f :3 0 24 :2 0
35 :4 0 169 291
293 :3 0 24 :2 0
36 :3 0 37 :3 0
38 :3 0 62 :3 0
298 299 0 16c
297 29b 3a :4 0
16e 296 29e 171
295 2a0 :3 0 24
:2 0 63 :4 0 174
2a2 2a4 :3 0 24
:2 0 27 :4 0 177
2a6 2a8 :3 0 24
:2 0 12 :3 0 17a
2aa 2ac :3 0 24
:2 0 27 :4 0 17d
2ae 2b0 :3 0 180
286 2b2 :2 0 2b4
183 2b5 283 2b4
0 2b6 185 0
2e0 1c :3 0 1d
:3 0 2b7 2b8 0
1c :3 0 2a :3 0
2ba 2bb 0 2b
:2 0 2 :4 0 64
:2 0 2d :2 0 2b
:2 0 187 2c0 2c2
:3 0 18a 2bc 2c4
2b9 2c6 2c5 :2 0
1f :3 0 2e :3 0
2c8 2c9 0 21
:4 0 65 :4 0 24
:2 0 25 :3 0 18e
2cd 2cf :3 0 191
2ca 2d1 :2 0 2d3
194 2d4 2c7 2d3
0 2d5 196 0
2e0 3d :3 0 66
:3 0 2d6 2d7 0
38 :3 0 62 :3 0
2d9 2da 0 12
:3 0 3f :3 0 198
2d8 2de :2 0 2e0
19c 2e1 272 2e0
0 2e2 1a0 0
55c e :3 0 67
:4 0 2e4 2e5 :3 0
2e3 2e6 0 55c
43 :3 0 68 :3 0
44 :3 0 45 :3 0
42 :3 0 69 :3 0
6a :3 0 6b :3 0
6c :3 0 e :3 0
3 :3 0 12 :3 0
18 :3 0 13 :3 0
9 :3 0 8 :3 0
7 :4 0 6d 1
:8 0 30f 6e :4 0
2fa :3 0 3f :3 0
5e :2 0 1a4 2fd
2fe :3 0 c :3 0
47 :3 0 300 301
0 17 :2 0 302
303 0 305 1a7
30c c :3 0 47
:3 0 306 307 0
6f :2 0 308 309
0 30b 1a9 30d
2ff 305 0 30e
0 30b 0 30e
1ab 0 30f 1ae
362 49 :3 0 c
:3 0 47 :3 0 311
312 0 48 :3 0
313 314 0 316
1b1 318 1b3 317
316 :2 0 360 4a
:3 0 c :3 0 47
:3 0 31b 31c 0
48 :3 0 31d 31e
0 358 1c :3 0
1d :3 0 320 321
0 1c :3 0 2a
:3 0 323 324 0
4b :2 0 2 :4 0
70 :2 0 2d :2 0
2b :2 0 1b5 329
32b :3 0 1b8 325
32d 322 32f 32e
:2 0 1f :3 0 2e
:3 0 331 332 0
21 :4 0 71 :4 0
24 :2 0 25 :3 0
1bc 336 338 :3 0
24 :2 0 34 :4 0
1bf 33a 33c :3 0
24 :2 0 4e :4 0
1c2 33e 340 :3 0
24 :2 0 48 :3 0
1c5 342 344 :3 0
24 :2 0 4f :4 0
1c8 346 348 :3 0
24 :2 0 40 :3 0
50 :3 0 2b :2 0
51 :2 0 1cb 34b
34f 1cf 34a 351
:3 0 1d2 333 353
:2 0 355 1d5 356
330 355 0 357
1d7 0 358 1d9
35b :3 0 35b 0
35b 35a 358 359
:6 0 35d 6 :3 0
1dc 35f 1de 35e
35d :2 0 360 1e0
:2 0 362 0 362
361 30f 360 :6 0
55c 1 :3 0 c
:3 0 47 :3 0 364
365 0 c :3 0
5e :2 0 72 :3 0
367 369 0 1e5
368 36b :3 0 36c
:2 0 19 :3 0 73
:3 0 36 :3 0 74
:3 0 10 :3 0 1e8
371 373 17 :2 0
1ea 370 376 2d
:2 0 2b :2 0 1ed
378 37a :3 0 37b
:2 0 75 :2 0 f
:2 0 1f0 37d 37f
:3 0 1f3 36f 381
36e 382 0 50b
19 :3 0 76 :2 0
2b :2 0 1f7 385
387 :3 0 15 :3 0
2b :2 0 389 38a
0 434 77 :3 0
36 :3 0 15 :3 0
17 :2 0 1fa 38d
390 19 :3 0 78
:2 0 79 :3 0 1ff
393 395 :3 0 396
433 12 :3 0 7a
:3 0 7b :3 0 399
39a 0 f :2 0
f :2 0 36 :3 0
15 :3 0 17 :2 0
202 39e 3a1 7c
:2 0 f :2 0 205
3a3 3a5 :3 0 2d
:2 0 2b :2 0 208
3a7 3a9 :3 0 10
:3 0 20b 39b 3ac
398 3ad 0 431
7d :3 0 e :3 0
59 :3 0 7e :3 0
7f :3 0 80 :3 0
81 :3 0 37 :3 0
15 :3 0 58 :3 0
59 :3 0 12 :4 0
82 1 :8 0 3d1
6e :4 0 3bc :3 0
3f :3 0 5e :2 0
212 3bf 3c0 :3 0
c :3 0 47 :3 0
3c2 3c3 0 17
:2 0 3c4 3c5 0
3c7 215 3ce c
:3 0 47 :3 0 3c8
3c9 0 6f :2 0
3ca 3cb 0 3cd
217 3cf 3c1 3c7
0 3d0 0 3cd
0 3d0 219 0
3d1 21c 424 49
:3 0 c :3 0 47
:3 0 3d3 3d4 0
48 :3 0 3d5 3d6
0 3d8 21f 3da
221 3d9 3d8 :2 0
422 4a :3 0 c
:3 0 47 :3 0 3dd
3de 0 48 :3 0
3df 3e0 0 41a
1c :3 0 1d :3 0
3e2 3e3 0 1c
:3 0 2a :3 0 3e5
3e6 0 4b :2 0
2 :4 0 83 :2 0
2d :2 0 2b :2 0
223 3eb 3ed :3 0
226 3e7 3ef 3e4
3f1 3f0 :2 0 1f
:3 0 2e :3 0 3f3
3f4 0 21 :4 0
84 :4 0 24 :2 0
25 :3 0 22a 3f8
3fa :3 0 24 :2 0
34 :4 0 22d 3fc
3fe :3 0 24 :2 0
4e :4 0 230 400
402 :3 0 24 :2 0
48 :3 0 233 404
406 :3 0 24 :2 0
4f :4 0 236 408
40a :3 0 24 :2 0
40 :3 0 50 :3 0
2b :2 0 51 :2 0
239 40d 411 23d
40c 413 :3 0 240
3f5 415 :2 0 417
243 418 3f2 417
0 419 245 0
41a 247 41d :3 0
41d 0 41d 41c
41a 41b :6 0 41f
9 :3 0 24a 421
24c 420 41f :2 0
422 24e :2 0 424
0 424 423 3d1
422 :6 0 431 8
:3 0 15 :3 0 36
:3 0 15 :3 0 17
:2 0 251 427 42a
2d :2 0 2b :2 0
254 42c 42e :3 0
426 42f 0 431
257 433 79 :3 0
397 431 :4 0 434
25b 435 388 434
0 436 25e 0
50b 6 :3 0 85
:2 0 260 438 439
:3 0 43a :2 0 10
:3 0 6 :3 0 43c
43d 0 508 1a
:3 0 73 :3 0 36
:3 0 74 :3 0 10
:3 0 262 442 444
17 :2 0 264 441
447 2d :2 0 2b
:2 0 267 449 44b
:3 0 44c :2 0 75
:2 0 f :2 0 26a
44e 450 :3 0 26d
440 452 43f 453
0 508 1a :3 0
76 :2 0 17 :2 0
271 456 458 :3 0
15 :3 0 17 :2 0
45a 45b 0 505
77 :3 0 36 :3 0
15 :3 0 17 :2 0
274 45e 461 1a
:3 0 78 :2 0 79
:3 0 279 464 466
:3 0 467 504 12
:3 0 7a :3 0 7b
:3 0 46a 46b 0
f :2 0 f :2 0
36 :3 0 15 :3 0
17 :2 0 27c 46f
472 7c :2 0 f
:2 0 27f 474 476
:3 0 2d :2 0 2b
:2 0 282 478 47a
:3 0 10 :3 0 285
46c 47d 469 47e
0 502 86 :3 0
e :3 0 12 :3 0
7e :3 0 80 :3 0
7f :3 0 81 :3 0
37 :3 0 15 :3 0
58 :3 0 59 :3 0
12 :4 0 87 1
:8 0 4a2 6e :4 0
48d :3 0 3f :3 0
5e :2 0 28c 490
491 :3 0 c :3 0
47 :3 0 493 494
0 17 :2 0 495
496 0 498 28f
49f c :3 0 47
:3 0 499 49a 0
6f :2 0 49b 49c
0 49e 291 4a0
492 498 0 4a1
0 49e 0 4a1
293 0 4a2 296
4f5 49 :3 0 c
:3 0 47 :3 0 4a4
4a5 0 48 :3 0
4a6 4a7 0 4a9
299 4ab 29b 4aa
4a9 :2 0 4f3 4a
:3 0 c :3 0 47
:3 0 4ae 4af 0
48 :3 0 4b0 4b1
0 4eb 1c :3 0
1d :3 0 4b3 4b4
0 1c :3 0 2a
:3 0 4b6 4b7 0
4b :2 0 2 :4 0
88 :2 0 2d :2 0
2b :2 0 29d 4bc
4be :3 0 2a0 4b8
4c0 4b5 4c2 4c1
:2 0 1f :3 0 2e
:3 0 4c4 4c5 0
21 :4 0 89 :4 0
24 :2 0 25 :3 0
2a4 4c9 4cb :3 0
24 :2 0 34 :4 0
2a7 4cd 4cf :3 0
24 :2 0 4e :4 0
2aa 4d1 4d3 :3 0
24 :2 0 48 :3 0
2ad 4d5 4d7 :3 0
24 :2 0 4f :4 0
2b0 4d9 4db :3 0
24 :2 0 40 :3 0
50 :3 0 2b :2 0
51 :2 0 2b3 4de
4e2 2b7 4dd 4e4
:3 0 2ba 4c6 4e6
:2 0 4e8 2bd 4e9
4c3 4e8 0 4ea
2bf 0 4eb 2c1
4ee :3 0 4ee 0
4ee 4ed 4eb 4ec
:6 0 4f0 c :3 0
2c4 4f2 2c6 4f1
4f0 :2 0 4f3 2c8
:2 0 4f5 0 4f5
4f4 4a2 4f3 :6 0
502 b :3 0 15
:3 0 36 :3 0 15
:3 0 17 :2 0 2cb
4f8 4fb 2d :2 0
2b :2 0 2ce 4fd
4ff :3 0 4f7 500
0 502 2d1 504
79 :3 0 468 502
:4 0 505 2d5 506
459 505 0 507
2d8 0 508 2da
509 43b 508 0
50a 2de 0 50b
2e0 50c 36d 50b
0 50d 2e4 0
55c 1c :3 0 1d
:3 0 50e 50f 0
1c :3 0 2a :3 0
511 512 0 8a
:2 0 2 :4 0 8b
:2 0 2d :2 0 2b
:2 0 2e6 517 519
:3 0 2e9 513 51b
510 51d 51c :2 0
1f :3 0 2e :3 0
51f 520 0 21
:4 0 8c :4 0 24
:2 0 25 :3 0 2ed
524 526 :3 0 24
:2 0 34 :4 0 2f0
528 52a :3 0 24
:2 0 35 :4 0 2f3
52c 52e :3 0 24
:2 0 36 :3 0 37
:3 0 c :3 0 47
:3 0 533 534 0
2f6 532 536 3a
:4 0 2f8 531 539
2fb 530 53b :3 0
24 :2 0 63 :4 0
2fe 53d 53f :3 0
24 :2 0 27 :4 0
301 541 543 :3 0
24 :2 0 e :3 0
304 545 547 :3 0
24 :2 0 27 :4 0
307 549 54b :3 0
30a 521 54d :2 0
54f 30d 550 51e
54f 0 551 30f
0 55c 3d :3 0
8d :3 0 552 553
0 c :3 0 47
:3 0 555 556 0
e :3 0 3f :3 0
311 554 55a :2 0
55c 315 5fd c
:3 0 8e :2 0 4
55d 55e 0 1c
:3 0 1d :3 0 560
561 0 1c :3 0
2a :3 0 563 564
0 4b :2 0 2
:4 0 8f :2 0 2d
:2 0 2b :2 0 323
569 56b :3 0 326
565 56d 562 56f
56e :2 0 1f :3 0
2e :3 0 571 572
0 21 :4 0 90
:4 0 24 :2 0 25
:3 0 32a 576 578
:3 0 32d 573 57a
:2 0 593 1f :3 0
2e :3 0 57c 57d
0 21 :4 0 91
:4 0 24 :2 0 c
:3 0 92 :3 0 582
583 0 330 581
585 :3 0 24 :2 0
63 :4 0 333 587
589 :3 0 24 :2 0
c :3 0 93 :3 0
58c 58d 0 336
58b 58f :3 0 339
57e 591 :2 0 593
33c 594 570 593
0 595 33f 0
5a5 94 :3 0 1b
:3 0 c :3 0 92
:3 0 95 :3 0 c
:3 0 93 :3 0 96
:3 0 97 :4 0 98
1 :a 0 5a1 :2 0
5a5 597 59f :2 0
b :3 0 1b :3 0
5a3 :2 0 5a5 341
5a7 345 5a6 5a5
:2 0 5fb 4a :3 0
c :3 0 47 :3 0
5aa 5ab 0 48
:3 0 5ac 5ad 0
5f3 1c :3 0 1d
:3 0 5af 5b0 0
1c :3 0 2a :3 0
5b2 5b3 0 4b
:2 0 2 :4 0 99
:2 0 2d :2 0 2b
:2 0 347 5b8 5ba
:3 0 34a 5b4 5bc
5b1 5be 5bd :2 0
1f :3 0 2e :3 0
5c0 5c1 0 21
:4 0 9a :4 0 24
:2 0 25 :3 0 34e
5c5 5c7 :3 0 351
5c2 5c9 :2 0 5e3
1f :3 0 2e :3 0
5cb 5cc 0 21
:4 0 4e :4 0 24
:2 0 48 :3 0 354
5d0 5d2 :3 0 24
:2 0 4f :4 0 357
5d4 5d6 :3 0 24
:2 0 40 :3 0 50
:3 0 2b :2 0 51
:2 0 35a 5d9 5dd
35e 5d8 5df :3 0
361 5cd 5e1 :2 0
5e3 364 5e4 5bf
5e3 0 5e5 367
0 5f3 94 :3 0
1b :3 0 95 :3 0
c :3 0 93 :3 0
96 :3 0 97 :4 0
9b 1 :a 0 5ef
:2 0 5f3 5e7 5ed
:2 0 b :3 0 1b
:3 0 5f1 :2 0 5f3
369 5f6 :3 0 5f6
0 5f6 5f5 5f3
5f4 :6 0 5f8 1
:3 0 36e 5fa 370
5f9 5f8 :2 0 5fb
372 :2 0 5fd 375
5fd 5fc 55c 5fb
:6 0 5fe :2 0 2
24 5fd 601 :3 0
600 5fe 602 :8 0

37f
4
:3 0 1 3 1
7 1 b 1
f 1 13 1
17 1 1b 7
6 a e 12
16 1a 1e 1
28 1 26 1
2f 1 2d 1
36 1 34 1
3d 1 3b 1
42 1 48 1
4e 1 53 1
58 1 65 1
71 2 74 76
2 79 7b 2
7d 7f 2 81
83 2 85 87
2 89 8b 2
8d 8f 2 91
93 2 95 97
2 99 9b 2
9d 9f 2 a1
a3 2 a5 a7
2 a9 ab 2
ad af 2 b1
b3 2 b5 b7
2 b9 bb 2
bd bf 2 c1
c3 2 c5 c7
2 c9 cb 2
cd cf 2 d1
d3 2 d5 d7
2 d9 db 2
dd df 2 e1
e3 2 e5 e7
4 6d 73 78
e9 1 eb 1
ed 2 f7 f9
3 f5 f6 fb
2 104 106 2
103 108 1 10a
1 10c 1 10e
1 114 2 124
126 3 122 123
128 2 131 133
2 135 137 2
139 13b 1 143
2 145 146 2
13d 148 2 130
14a 1 14c 1
14e 2 158 15a
3 156 157 15c
2 165 167 2
164 169 1 16b
1 16d 2 174
175 3 14f 16e
177 1 179 3
181 182 183 3
185 186 187 2
194 199 1 1a0
1 19b 2 1b3
1b5 3 1b1 1b2
1b7 2 1c0 1c2
2 1c4 1c6 2
1c8 1ca 2 1cc
1ce 2 1d0 1d2
3 1d7 1d8 1d9
2 1d4 1db 2
1bf 1dd 1 1df
1 1e1 2 1aa
1e2 1 1e6 1
1a5 2 1a3 1ea
2 20e 213 1
21a 1 215 2
22d 22f 3 22b
22c 231 2 23a
23c 2 23e 240
2 242 244 2
246 248 2 24a
24c 3 251 252
253 2 24e 255
2 239 257 1
259 1 25b 2
224 25c 1 260
1 21f 2 21d
264 1 26f 2
26b 26f 2 27b
27d 3 279 27a
27f 2 288 28a
2 28c 28e 2
290 292 1 29a
2 29c 29d 2
294 29f 2 2a1
2a3 2 2a5 2a7
2 2a9 2ab 2
2ad 2af 2 287
2b1 1 2b3 1
2b5 2 2bf 2c1
3 2bd 2be 2c3
2 2cc 2ce 2
2cb 2d0 1 2d2
1 2d4 3 2db
2dc 2dd 3 2b6
2d5 2df 1 2e1
1 2fc 2 2fb
2fc 1 304 1
30a 2 30c 30d
2 2f9 30e 1
315 1 310 2
328 32a 3 326
327 32c 2 335
337 2 339 33b
2 33d 33f 2
341 343 2 345
347 3 34c 34d
34e 2 349 350
2 334 352 1
354 1 356 2
31f 357 1 35b
1 31a 2 318
35f 1 36a 2
366 36a 1 372
2 374 375 2
377 379 2 37c
37e 1 380 1
386 2 384 386
2 38e 38f 1
392 2 391 392
2 39f 3a0 2
3a2 3a4 2 3a6
3a8 4 39c 39d
3aa 3ab 1 3be
2 3bd 3be 1
3c6 1 3cc 2
3ce 3cf 2 3bb
3d0 1 3d7 1
3d2 2 3ea 3ec
3 3e8 3e9 3ee
2 3f7 3f9 2
3fb 3fd 2 3ff
401 2 403 405
2 407 409 3
40e 40f 410 2
40b 412 2 3f6
414 1 416 1
418 2 3e1 419
1 41d 1 3dc
2 3da 421 2
428 429 2 42b
42d 3 3ae 424
430 2 38b 433
1 435 1 437
1 443 2 445
446 2 448 44a
2 44d 44f 1
451 1 457 2
455 457 2 45f
460 1 463 2
462 463 2 470
471 2 473 475
2 477 479 4
46d 46e 47b 47c
1 48f 2 48e
48f 1 497 1
49d 2 49f 4a0
2 48c 4a1 1
4a8 1 4a3 2
4bb 4bd 3 4b9
4ba 4bf 2 4c8
4ca 2 4cc 4ce
2 4d0 4d2 2
4d4 4d6 2 4d8
4da 3 4df 4e0
4e1 2 4dc 4e3
2 4c7 4e5 1
4e7 1 4e9 2
4b2 4ea 1 4ee
1 4ad 2 4ab
4f2 2 4f9 4fa
2 4fc 4fe 3
47f 4f5 501 2
45c 504 1 506
3 43e 454 507
1 509 3 383
436 50a 1 50c
2 516 518 3
514 515 51a 2
523 525 2 527
529 2 52b 52d
1 535 2 537
538 2 52f 53a
2 53c 53e 2
540 542 2 544
546 2 548 54a
2 522 54c 1
54e 1 550 3
557 558 559 d
ee 10d 17a 17d
18a 1ed 267 2e2
2e7 362 50d 551
55b 2 568 56a
3 566 567 56c
2 575 577 2
574 579 2 580
584 2 586 588
2 58a 58e 2
57f 590 2 57b
592 1 594 3
595 5a0 5a4 1
55f 2 5b7 5b9
3 5b5 5b6 5bb
2 5c4 5c6 2
5c3 5c8 2 5cf
5d1 2 5d3 5d5
3 5da 5db 5dc
2 5d7 5de 2
5ce 5e0 2 5ca
5e2 1 5e4 4
5ae 5e5 5ee 5f2
1 5f6 1 5a9
2 5a7 5fa 9
2b 32 39 40
46 4c 51 56
5d 
1
4
0 
601
0
1
14
e
11
0 1 2 1 4 1 6 1
8 9 1 b c 1 0 0
0 0 0 0 
1b 1 0
b 1 0
34 1 0
42 1 0
4e 1 0
17 1 0
48 1 0
2d 1 0
13 1 0
f 1 0
26 1 0
3 1 0
53 1 0
2 0 1
7 1 0
3b 1 0
58 1 0
0

/
SHOW ERRORS FUNCTION SP_I_PROFILE;
ALTER FUNCTION SP_I_PROFILE COMPILE DEBUG;
/*######################################################################################*/
/* assign a profile to a task or project, used tables: RESOURCE_ASSIGNMENTS, PROFILES   */
/****************************************************************************************/
/*  RES_ID is profile_id, role_id is role_id in TMT_COMPETENCIES    */
/****************************************************************************************/
/*
#Stored Procedure  :SP_A_TSKPROF
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_A_TSKPROF ==
CREATE OR REPLACE FUNCTION SP_A_TSKPROF wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
d7
2 :e:
1FUNCTION:
1SP_A_TSKPROF:
1TASK_ID001:
1VARCHAR2:
1RES_ID001:
1ROLE_ID001:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_UNIQUEID:
132:
1H_PROJECT_ID:
1H_RES_ID:
1H_AND_ROLES:
18000:
1H_ROLE_ID:
1H_REC_STATUS:
11:
1H_FULL_NAME:
1255:
1H_TYPE:
1PLS_INTEGER:
10:
1H_CRI:
1H_BITFLAG:
1H_NUMBER:
1H_TEMP_PARTICIPANT:
1H_TEMP_SECURITY_ID:
1H_TEMP_BITFLAG:
1H_TEMP_RES_TYPE:
1H_TEMP_SC_STATE:
1H_TABLE_TYPE:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_A_TSKPROF(:
1':
1, :
1);:
1CHECK_TRACE:
137:
1+:
1FPUT:
1               ENTER:: SP_A_TSKPROF(38) :: cplan.sql(5841) :: :
1IS NULL:
18:
144:
1      RETURN_INVALID:: SP_A_TSKPROF(45) :: cplan.sql(5848) :: :
1 :
1STATE:: :
1COALESCE:
1TO_CHAR:
1PMO_CNST:
1INVALID_PARAMS:
1NULL:
147:
1                EXIT:: SP_A_TSKPROF(48) :: cplan.sql(5851) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1PROJECT_ID:
1TYPE_ID:
1TMT_WBS:
1ELEMENT_ID:
1SELECT PROJECT_ID, TYPE_ID INTO h_project_id, h_type FROM TMT_WBS WHERE ELEME+
1NT_ID = task_id001:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
162:
1           EXCEPTION:: SP_A_TSKPROF(63) :: cplan.sql(5866) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1=:
1SC_SQL_NO_DATA:
1TMT_CRI:
1SELECT PROJECT_ID, TYPE_ID INTO h_project_id, h_type FROM TMT_CRI WHERE ELEME+
1NT_ID = task_id001:
178:
1           EXCEPTION:: SP_A_TSKPROF(79) :: cplan.sql(5882) :: :
185:
1        RETURN_ERROR:: SP_A_TSKPROF(86) :: cplan.sql(5889) :: :
1RECORD_NOT_FOUND:
1 ID:: :
188:
1                EXIT:: SP_A_TSKPROF(89) :: cplan.sql(5892) :: :
1PMO_RETURN_ERROR_SP:
1SC_TABLE_CRI:
1SC_TABLE_WBS:
1PROFILE_ID:
1PROFILE_NAME:
1REC_STATUS:
1PROFILES:
1SELECT PROFILE_ID, PROFILE_NAME, REC_STATUS INTO h_res_id, h_full_name, h_rec+
1_status:n      FROM PROFILES WHERE PROFILE_ID = h_res_id:
1113:
1           EXCEPTION:: SP_A_TSKPROF(114) :: cplan.sql(5917) :: :
1MAX:
1PROF_NUMBER:
1ROLE_ID:
1SELECT COALESCE(MAX(PROF_NUMBER),0)+1 INTO h_number:n         FROM PROFILES:n+
1         WHERE PROJECT_ID=h_project_id AND ROLE_ID=h_role_id:
1135:
1           EXCEPTION:: SP_A_TSKPROF(136) :: cplan.sql(5939) :: :
1PMO_ORASTD:
1STRINGCONCAT:
1RTRIM:
1TCP:
1ELEMENT_NAME:
1TC:
1TMT_COMPETENCIES:
1PARENT_ID:
1SELECT PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConca+
1t(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(TCP.ELEMENT_NAME), ',+
1 '), RTRIM(TC.ELEMENT_NAME)), '('), RTRIM(TO_CHAR(h_number))), ')') INTO h_fu+
1ll_name:n         FROM TMT_COMPETENCIES TC, TMT_COMPETENCIES TCP:n         WH+
1ERE TC.ELEMENT_ID = h_role_id AND TCP.ELEMENT_ID = TC.PARENT_ID:
1152:
1           EXCEPTION:: SP_A_TSKPROF(153) :: cplan.sql(5956) :: :
1159:
1        RETURN_ERROR:: SP_A_TSKPROF(160) :: cplan.sql(5963) :: :
1162:
1                EXIT:: SP_A_TSKPROF(163) :: cplan.sql(5966) :: :
1INSERT INTO PROFILES(PROFILE_ID,PROJECT_ID,ROLE_ID,PROF_NUMBER, PROFILE_NAME)+
1:n         VALUES(h_res_id, h_project_id, h_role_id, h_number, h_full_name):
1FOUND:
1100:
1182:
1           EXCEPTION:: SP_A_TSKPROF(183) :: cplan.sql(5986) :: :
1!=:
1SC_SQL_SUCCESS:
1189:
1        RETURN_ERROR:: SP_A_TSKPROF(190) :: cplan.sql(5993) :: :
1192:
1                EXIT:: SP_A_TSKPROF(193) :: cplan.sql(5996) :: :
1ELSIF:
1ISNOTPROJECTTYPE:
1A:
1RES_ASSIGNMENT_ID:
1RESOURCE_ASSIGNMENTS:
1RESOURCE_ID:
1RESOURCE_TYPE:
1SELECT a.RES_ASSIGNMENT_ID INTO h_uniqueID :n            FROM RESOURCE_ASSIGN+
1MENTS a:n            WHERE a.ELEMENT_ID = task_id001  :n            AND a.RES+
1OURCE_ID = res_id001:n            AND a.RESOURCE_TYPE = '1':
1212:
1           EXCEPTION:: SP_A_TSKPROF(213) :: cplan.sql(6016) :: :
1219:
1        RETURN_ERROR:: SP_A_TSKPROF(220) :: cplan.sql(6023) :: :
1RECORD_ALREADY_ASSIGNED:
1222:
1                EXIT:: SP_A_TSKPROF(223) :: cplan.sql(6026) :: :
1D:
1UPDATE PROFILES SET REC_STATUS = 'U' WHERE PROFILE_ID = h_res_id:
1243:
1           EXCEPTION:: SP_A_TSKPROF(244) :: cplan.sql(6047) :: :
1250:
1        RETURN_ERROR:: SP_A_TSKPROF(251) :: cplan.sql(6054) :: :
1253:
1                EXIT:: SP_A_TSKPROF(254) :: cplan.sql(6057) :: :
1SELECT RES_ASSIGNMENT_ID INTO h_uniqueID :n      FROM RESOURCE_ASSIGNMENTS:n +
1     WHERE ELEMENT_ID = task_id001 :n      AND RESOURCE_ID = h_res_id :n     +
1 AND RESOURCE_TYPE = '1':
1273:
1           EXCEPTION:: SP_A_TSKPROF(274) :: cplan.sql(6077) :: :
1ASSIGN_NAMES:
1INSTR:
1LENGTH:
1UPDATE TMT_WBS SET ASSIGN_NAMES =:n            CASE:n            WHEN InStr( +
1ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES:n            ELSE:n  +
1          CASE:n            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) >+
1 2040 THEN ASSIGN_NAMES:n            ELSE ASSIGN_NAMES || h_full_name || ';'+
1:n            END:n            END:n            WHERE ELEMENT_ID = task_id001:
1304:
1           EXCEPTION:: SP_A_TSKPROF(305) :: cplan.sql(6108) :: :
1UPDATE TMT_CRI SET ASSIGN_NAMES =:n            CASE:n            WHEN InStr( +
1ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES:n            ELSE:n  +
1          CASE:n            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) >+
1 2040 THEN ASSIGN_NAMES:n            ELSE ASSIGN_NAMES || h_full_name || ';'+
1:n            END:n            END:n            WHERE ELEMENT_ID = task_id001:
1332:
1           EXCEPTION:: SP_A_TSKPROF(333) :: cplan.sql(6136) :: :
1SYS_GUID:
1GETSECURITYID:
1SC_TEAMMEMBER_GROUP:
1ISPROJECTTYPE:
1SC_AFFECTED:
1SC_TEAMMEMBER:
1FC_BEFORE_I_RES_ASSIGN:
1ELEMENT_TYPE:
1PARTICIPANT:
1SECURITY_ID:
1REC_USER:
1REC_DATETIME:
1TABLE_TYPE:
1INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, E+
1LEMENT_TYPE,:n         RESOURCE_TYPE, RESOURCE_ID, PARTICIPANT, SECURITY_ID, +
1REC_USER, REC_DATETIME,TABLE_TYPE) :n         VALUES (h_uniqueID, h_project_i+
1d,task_id001, h_type, :n         '1', h_res_id, h_temp_participant, h_temp_se+
1curity_id, rec_user001, SYSTIMESTAMP,h_table_type):
1372:
1           EXCEPTION:: SP_A_TSKPROF(373) :: cplan.sql(6176) :: :
1PMO_STPU:
1BIT_OR:
1UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = PMO_STPU.BIT_OR(PARTICIPANT, PM+
1O_CNST.SC_TEAMMEMBER):n         WHERE RES_ASSIGNMENT_ID = h_uniqueID:
1393:
1           EXCEPTION:: SP_A_TSKPROF(394) :: cplan.sql(6197) :: :
1FC_AFTER_U_RES_ASSIGN:
14:
1402:
1       RETURN_STATUS:: SP_A_TSKPROF(403) :: cplan.sql(6206) :: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
1409:
1           EXCEPTION:: SP_A_TSKPROF(410) :: cplan.sql(6213) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1ID:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1421:
1           EXCEPTION:: SP_A_TSKPROF(422) :: cplan.sql(6225) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
b52
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d b4 :3 a0 6b 2c 6a
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
1c 51 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
:2 a0 6b 1c 81 b0 :2 a0 6b :2 a0
6b 6e a5 b a 10 :2 a0 6b
6e :2 a0 6b 6e a5 b 6e 7e
a0 b4 2e 6e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c a0 7e b4 2e
:2 5a a0 7e b4 2e :2 5a 52 10
a0 7e b4 2e :2 5a 52 10 :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b a0 a5 57 b7 19
3c :7 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a :7 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 19
3c a0 51 d :3 a0 6b d b7
a0 51 d :3 a0 6b d b7 :2 19
3c :2 a0 d :9 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b a0
7e a0 6b b4 2e 5a :2 a0 d
:3 a0 :2 51 a5 b d :9 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :1f a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 19 3c :b a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c a0
b7 :2 a0 6b a0 7e a0 6b b4
2e 5a :3 a0 51 a5 b a5 b
7e 51 b4 2e :d a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 19
3c b7 19 3c :2 a0 :2 51 a5 b
7e 6e b4 2e :4 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 19 3c b7
19 :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 :2 19 3c a0 51 d :8 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 6b a0 7e a0 6b b4 2e
5a :2 a0 51 a5 b 7e 51 b4
2e :f a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f a0 b7 :2 a0 51 a5 b 7e
51 b4 2e :f a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 :2 19 3c :2 a0 b4
2e d :4 a0 51 a5 b :2 a0 51
a5 b :2 a0 6b a0 a5 b d
:3 a0 51 a5 b a5 b 7e 51
b4 2e :3 a0 6b d b7 :3 a0 6b
d b7 :2 19 3c a0 6e d a0
51 d :9 a0 a5 b d :16 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f a0 b7
:2 a0 6b a0 7e a0 6b b4 2e
5a :9 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :3 a0 a5 b d b7 :2 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 :2 a0 6b :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 :2 a0
6b :2 6e 7e :2 a0 6b b4 2e 7e
6e b4 2e 7e :2 a0 6b b4 2e
a5 57 b7 19 3c :9 a0 12a e9
dd :2 a0 65 b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c :7 a0 12a e9 dd :2 a0
65 b7 a4 b1 11 4f b7 a6
9 a4 b1 11 68 4f 1d 17
b5 
b52
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 59 6b 6f 73 56
77 7b 98 83 87 8a 8b 93
82 b5 a3 7f a7 a8 b0 a2
d2 c0 9f c4 c5 cd bf ef
dd bc e1 e2 ea dc 10c fa
d9 fe ff 107 f9 129 117 f6
11b 11c 124 116 146 134 113 138
139 141 133 162 151 155 130 15d
150 17e 16d 171 14d 179 16c 19a
189 18d 169 195 188 1b6 1a5 1a9
185 1b1 1a4 1d2 1c1 1c5 1a1 1cd
1c0 1ef 1dd 1bd 1e1 1e2 1ea 1dc
20b 1fa 1fe 1d9 206 1f9 228 216
1f6 21a 21b 223 215 244 233 237
212 23f 232 260 24f 253 22f 25b
24e 280 26b 26f 24b 273 27b 26a
287 28b 267 28f 293 297 29a 29f
2a0 1 2a2 2a7 2ab 2af 2b2 2b7
2bb 2bf 2c2 2c7 2c8 2ca 2cf 2d2
2d6 2d7 2dc 2e1 2e4 2e9 2ea 2ef
2f2 2f6 2f7 2fc 2ff 304 305 30a
30d 312 313 318 31b 320 321 326
329 32d 32e 333 336 33b 33c 341
344 349 34a 34f 352 357 358 35d
360 364 365 36a 36d 372 373 378
37b 380 381 386 389 38e 38f 394
397 39b 39c 3a1 3a4 3a9 3aa 3af
3b2 3b7 3b8 3bd 3be 3c3 3c5 3c9
3cc 3d0 3d4 3d7 3db 3df 3e2 3e5
3ea 3ed 3f0 3f3 3f4 3f9 3fa 1
3fc 401 405 409 40c 411 416 419
41d 41e 423 424 429 42b 42f 432
436 439 43a 43f 442 445 449 44c
44d 452 455 1 458 45d 461 464
465 46a 46d 1 470 475 479 47d
480 484 488 48b 48e 493 496 499
49c 49d 4a2 4a3 1 4a5 4aa 4ae
4b2 4b5 4ba 4bf 4c2 4c6 4c7 4cc
4cf 4d4 4d5 4da 4dd 4e2 4e3 4e8
4eb 4ef 4f3 4f7 4fb 4fe 4ff 501
506 507 509 50a 50f 510 515 517
51b 51e 522 526 529 52d 531 534
537 53c 53f 542 545 546 54b 54c
1 54e 553 557 55b 55e 563 568
56b 56f 570 575 576 57b 57d 581
584 588 58c 58f 593 597 59a 59e
59f 5a4 5a6 5aa 5ad 5b1 5b5 5b9
5bd 5c1 5c5 5c9 5d5 5d9 5dd 5e0
5e4 5e8 5ea 5ee 5f2 5f6 5f9 5fd
601 603 604 609 1 60d 611 615
618 61c 620 624 628 62b 62f 633
636 639 63e 641 644 647 648 64d
64e 1 650 655 659 65d 660 665
66a 66d 671 672 677 67a 67f 680
685 688 68d 68e 693 696 69a 69b
6a0 6a3 6a8 6a9 6ae 6b1 6b5 6b9
6bc 6bf 6c0 6c2 6c3 6c8 6c9 6ce
6d0 6d4 6d7 6d9 6dd 6df 6eb 6ed
6ef 6f0 6f5 6f9 6fb 707 709 70d
711 714 718 71b 71f 722 723 728
72b 72f 733 737 73b 73f 743 747
753 757 75b 75e 762 766 768 76c
770 774 777 77b 77f 781 782 787
1 78b 78f 793 796 79a 79e 7a2
7a6 7a9 7ad 7b1 7b4 7b7 7bc 7bf
7c2 7c5 7c6 7cb 7cc 1 7ce 7d3
7d7 7db 7de 7e3 7e8 7eb 7ef 7f0
7f5 7f8 7fd 7fe 803 806 80b 80c
811 814 818 819 81e 821 826 827
82c 82f 833 837 83a 83d 83e 840
841 846 847 84c 84e 852 855 857
85b 85d 869 86b 86d 86e 873 877
879 885 887 88b 88f 892 896 899
89d 8a0 8a1 8a6 8a9 8ad 8b1 8b4
8b8 8bc 8bf 8c2 8c7 8ca 8cd 8d0
8d1 8d6 8d7 1 8d9 8de 8e2 8e6
8e9 8ee 8f3 8f6 8fa 8fb 900 903
908 909 90e 911 916 917 91c 91f
923 927 92b 92f 932 933 935 93a
93b 93d 93e 943 946 94b 94c 951
954 959 95a 95f 962 966 967 96c
96f 974 975 97a 97b 980 982 986
989 98d 991 994 998 99c 99f 9a2
9a7 9aa 9ad 9b0 9b1 9b6 9b7 1
9b9 9be 9c2 9c6 9c9 9ce 9d3 9d6
9da 9db 9e0 9e1 9e6 9e8 9ec 9ef
9f3 9f7 9fa 9fe a02 a05 a09 a0d
a0e a13 a15 a19 a1c a20 a23 a27
a2b a2f a33 a36 a3a a3c a40 a43
a47 a4b a4f a53 a56 a5a a5c a60
a64 a67 a6b a6f a73 a77 a7b a7f
a83 a87 a8b a8f a93 a97 aa3 aa7
aab aae ab2 ab6 ab8 abc ac0 ac4
ac7 acb acf ad1 ad2 ad7 1 adb
adf ae3 ae6 aea aee af2 af6 af9
afd b01 b04 b07 b0c b0f b12 b15
b16 b1b b1c 1 b1e b23 b27 b2b
b2e b33 b38 b3b b3f b40 b45 b48
b4d b4e b53 b56 b5b b5c b61 b64
b68 b69 b6e b71 b76 b77 b7c b7f
b83 b87 b8a b8d b8e b90 b91 b96
b97 b9c b9e ba2 ba5 ba7 bab bad
bb9 bbb bbd bbe bc3 bc7 bc9 bd5
bd7 bdb bdf be2 be6 be9 bed bf0
bf1 bf6 bf9 bfd c01 c05 c09 c0d
c11 c14 c17 c18 c1a c1e c22 c26
c2a c2e c32 c36 c3a c3e c42 c4e
c52 c56 c59 c5d c61 c63 c67 c6b
c6f c72 c76 c7a c7c c7d c82 1
c86 c8a c8e c91 c95 c99 c9d ca1
ca4 ca8 cac caf cb2 cb7 cba cbd
cc0 cc1 cc6 cc7 1 cc9 cce cd2
cd6 cd9 cde ce3 ce6 cea ceb cf0
cf3 cf8 cf9 cfe d01 d06 d07 d0c
d0f d13 d14 d19 d1c d21 d22 d27
d2a d2e d32 d35 d38 d39 d3b d3c
d41 d42 d47 d49 d4d d50 d52 d56
d58 d64 d66 d68 d69 d6e d72 d74
d80 d82 d86 d8a d8e d92 d96 d9a
d9e da2 da6 daa dae db2 db6 dba
dbe dc2 dc6 dca dce dd2 dd6 dda
dde de2 de6 dea dee df2 df6 dfa
dfe e0a e0e e12 e15 e19 e1d e1f
e23 e27 e2b e2e e32 e36 e38 e39
e3e 1 e42 e46 e4a e4d e51 e55
e59 e5d e60 e64 e68 e6b e6e e73
e76 e79 e7c e7d e82 e83 1 e85
e8a e8e e92 e95 e9a e9f ea2 ea6
ea7 eac eaf eb4 eb5 eba ebd ec2
ec3 ec8 ecb ecf ed0 ed5 ed8 edd
ede ee3 ee6 eea eee ef1 ef4 ef5
ef7 ef8 efd efe f03 f05 f09 f0c
f0e f12 f14 f20 f22 f24 f25 f2a
f2e f30 f3c f3e f42 f46 f49 f4d
f50 f54 f57 f58 f5d f60 f64 f68
f6b f6f f73 f76 f79 f7e f81 f84
f87 f88 f8d f8e 1 f90 f95 f99
f9d fa0 fa5 faa fad fb1 fb2 fb7
fba fbf fc0 fc5 fc8 fcd fce fd3
fd6 fda fde fe2 fe6 fe9 fea fec
ff1 ff2 ff4 ff5 ffa ffd 1002 1003
1008 100b 1010 1011 1016 1019 101d 101e
1023 1026 102b 102c 1031 1032 1037 1039
103d 1040 1044 1048 104b 104f 1053 1056
1059 105e 1061 1064 1067 1068 106d 106e
1 1070 1075 1079 107d 1080 1085 108a
108d 1091 1092 1097 1098 109d 109f 10a3
10a6 10aa 10ae 10b1 10b5 10b9 10bc 10c0
10c4 10c5 10ca 10cc 10d0 10d3 10d7 10db
10df 10e3 10e7 10eb 10ef 10f3 10f7 10fb
10ff 110b 110f 1114 1118 111b 111c 1121
1125 1129 112c 112f 1133 1135 1139 113d
1140 1143 1147 1149 114d 1151 1154 1156
115a 115e 1162 1165 1169 116d 116f 1170
1175 1 1179 117d 1181 1184 1188 118c
1190 1194 1197 119b 119f 11a2 11a5 11aa
11ad 11b0 11b3 11b4 11b9 11ba 1 11bc
11c1 11c5 11c9 11cc 11d1 11d6 11d9 11dd
11de 11e3 11e6 11eb 11ec 11f1 11f4 11f9
11fa 11ff 1202 1206 1207 120c 120f 1214
1215 121a 121d 1221 1225 1228 122b 122c
122e 122f 1234 1235 123a 123c 1240 1243
1245 1249 124b 1257 1259 125b 125c 1261
1265 1267 1273 1275 1279 127d 1280 1284
1287 128b 128e 128f 1294 1297 129b 129f
12a2 12a6 12aa 12ad 12b0 12b5 12b8 12bb
12be 12bf 12c4 12c5 1 12c7 12cc 12d0
12d4 12d7 12dc 12e1 12e4 12e8 12e9 12ee
12f1 12f6 12f7 12fc 12ff 1304 1305 130a
130d 1311 1315 1319 131d 1320 1321 1323
1328 1329 132b 132c 1331 1334 1339 133a
133f 1342 1347 1348 134d 1350 1354 1355
135a 135d 1362 1363 1368 1369 136e 1370
1374 1377 137b 137f 1382 1386 138a 138d
1390 1395 1398 139b 139e 139f 13a4 13a5
1 13a7 13ac 13b0 13b4 13b7 13bc 13c1
13c4 13c8 13c9 13ce 13cf 13d4 13d6 13da
13dd 13e1 13e5 13e8 13ec 13f0 13f3 13f7
13fb 13fc 1401 1403 1407 140a 140e 1410
1414 1418 141b 141f 1422 1426 1429 142a
142f 1432 1436 143a 143e 1441 1442 1444
1445 1447 144a 144d 144e 1453 1457 145b
145f 1463 1467 146b 146f 1473 1477 147b
147f 1483 1487 1493 1497 149b 149e 14a2
14a6 14a8 14ac 14b0 14b4 14b7 14bb 14bf
14c1 14c2 14c7 1 14cb 14cf 14d3 14d6
14da 14de 14e2 14e6 14e9 14ed 14f1 14f4
14f7 14fc 14ff 1502 1505 1506 150b 150c
1 150e 1513 1517 151b 151e 1523 1528
152b 152f 1530 1535 1538 153d 153e 1543
1546 154b 154c 1551 1554 1558 1559 155e
1561 1566 1567 156c 156f 1573 1577 157a
157d 157e 1580 1581 1586 1587 158c 158e
1592 1595 1597 159b 159d 15a9 15ab 15ad
15ae 15b3 15b7 15b9 15c5 15c7 15cb 15cf
15d2 15d6 15d9 15dd 15e0 15e1 15e6 15e9
15ed 15f1 15f4 15f8 15fc 15ff 1602 1607
160a 160d 1610 1611 1616 1617 1 1619
161e 1622 1626 1629 162e 1633 1636 163a
163b 1640 1643 1648 1649 164e 1651 1656
1657 165c 165f 1663 1667 166b 166f 1672
1673 1675 167a 167b 167d 167e 1683 1686
168b 168c 1691 1694 1699 169a 169f 16a2
16a6 16a7 16ac 16af 16b4 16b5 16ba 16bb
16c0 16c2 16c6 16c9 16cd 16d1 16d4 16d8
16dc 16df 16e2 16e7 16ea 16ed 16f0 16f1
16f6 16f7 1 16f9 16fe 1702 1706 1709
170e 1713 1716 171a 171b 1720 1721 1726
1728 172c 172f 1733 1737 173a 173e 1742
1745 1749 174d 174e 1753 1755 1759 175c
175e 1762 1765 1769 176d 1770 1773 1774
1776 1779 177e 177f 1784 1788 178c 1790
1794 17a0 17a4 17a9 17ad 17b0 17b1 17b6
17ba 17be 17c1 17c4 17c8 17ca 17ce 17d2
17d5 17d8 17dc 17de 17e2 17e6 17e9 17eb
17ef 17f3 17f7 17fa 17fe 1802 1804 1805
180a 1 180e 1812 1816 1819 181d 1821
1825 1829 182c 1830 1834 1837 183a 183f
1842 1845 1848 1849 184e 184f 1 1851
1856 185a 185e 1861 1866 186b 186e 1872
1873 1878 187b 1880 1881 1886 1889 188e
188f 1894 1897 189b 189c 18a1 18a4 18a9
18aa 18af 18b2 18b6 18ba 18bd 18c0 18c1
18c3 18c4 18c9 18ca 18cf 18d1 18d5 18d8
18da 18de 18e0 18ec 18ee 18f0 18f1 18f6
18fa 18fc 1908 190a 190c 1910 1913 1915
1919 191d 1921 1924 1928 192c 192f 1932
1937 193a 193d 1940 1941 1946 1947 1
1949 194e 1952 1956 1959 195e 1963 1966
196a 196b 1970 1973 1978 1979 197e 1981
1986 1987 198c 198f 1993 1997 199b 199f
19a2 19a3 19a5 19aa 19ab 19ad 19ae 19b3
19b6 19bb 19bc 19c1 19c4 19c9 19ca 19cf
19d2 19d6 19d7 19dc 19df 19e4 19e5 19ea
19eb 19f0 19f2 19f6 19f9 19fd 1a01 1a04
1a08 1a0c 1a0f 1a12 1a17 1a1a 1a1d 1a20
1a21 1a26 1a27 1 1a29 1a2e 1a32 1a36
1a39 1a3e 1a43 1a46 1a4a 1a4b 1a50 1a51
1a56 1a58 1a5c 1a5f 1a63 1a67 1a6a 1a6e
1a72 1a75 1a79 1a7d 1a7e 1a83 1a85 1a89
1a8d 1a90 1a94 1a97 1a9b 1a9f 1aa3 1aa7
1aab 1aaf 1ab3 1ab7 1abb 1ac7 1acb 1acf
1ad2 1ad6 1ada 1adc 1ae0 1ae4 1ae8 1aeb
1aef 1af3 1af5 1af6 1afb 1 1aff 1b03
1b07 1b0a 1b0e 1b12 1b16 1b1a 1b1d 1b21
1b25 1b28 1b2b 1b30 1b33 1b36 1b39 1b3a
1b3f 1b40 1 1b42 1b47 1b4b 1b4f 1b52
1b57 1b5c 1b5f 1b63 1b64 1b69 1b6c 1b71
1b72 1b77 1b7a 1b7f 1b80 1b85 1b88 1b8c
1b8d 1b92 1b95 1b9a 1b9b 1ba0 1ba3 1ba7
1bab 1bae 1bb1 1bb2 1bb4 1bb5 1bba 1bbb
1bc0 1bc2 1bc6 1bc9 1bcb 1bcf 1bd1 1bdd
1bdf 1be1 1be2 1be7 1beb 1bed 1bf9 1bfb
1bff 1c03 1c06 1c0a 1c0d 1c11 1c14 1c15
1c1a 1c1d 1c21 1c25 1c28 1c29 1c2b 1c2e
1c31 1c32 1c37 1c3b 1c3f 1c43 1c47 1c4b
1c4f 1c53 1c57 1c5b 1c5f 1c63 1c67 1c6b
1c6f 1c73 1c7f 1c83 1c88 1c8c 1c8f 1c90
1c95 1c99 1c9d 1ca0 1ca3 1ca7 1ca9 1cad
1cb1 1cb4 1cb7 1cbb 1cbd 1cc1 1cc5 1cc8
1cca 1cce 1cd2 1cd6 1cd9 1cdd 1ce1 1ce3
1ce4 1ce9 1 1ced 1cf1 1cf5 1cf8 1cfc
1d00 1d04 1d08 1d0b 1d0f 1d13 1d16 1d19
1d1e 1d21 1d24 1d27 1d28 1d2d 1d2e 1
1d30 1d35 1d39 1d3d 1d40 1d45 1d4a 1d4d
1d51 1d52 1d57 1d5a 1d5f 1d60 1d65 1d68
1d6d 1d6e 1d73 1d76 1d7a 1d7b 1d80 1d83
1d88 1d89 1d8e 1d91 1d95 1d99 1d9c 1d9f
1da0 1da2 1da3 1da8 1da9 1dae 1db0 1db4
1db7 1db9 1dbd 1dbf 1dcb 1dcd 1dcf 1dd0
1dd5 1dd9 1ddb 1de7 1de9 1ded 1def 1df3
1df7 1dfa 1dfb 1dfd 1e00 1e03 1e04 1e09
1e0d 1e11 1e15 1e19 1e1d 1e21 1e25 1e29
1e2d 1e31 1e35 1e39 1e3d 1e41 1e45 1e51
1e55 1e5a 1e5e 1e61 1e62 1e67 1e6b 1e6f
1e72 1e75 1e79 1e7b 1e7f 1e83 1e86 1e89
1e8d 1e8f 1e93 1e97 1e9a 1e9c 1ea0 1ea4
1ea8 1eab 1eaf 1eb3 1eb5 1eb6 1ebb 1
1ebf 1ec3 1ec7 1eca 1ece 1ed2 1ed6 1eda
1edd 1ee1 1ee5 1ee8 1eeb 1ef0 1ef3 1ef6
1ef9 1efa 1eff 1f00 1 1f02 1f07 1f0b
1f0f 1f12 1f17 1f1c 1f1f 1f23 1f24 1f29
1f2c 1f31 1f32 1f37 1f3a 1f3f 1f40 1f45
1f48 1f4c 1f4d 1f52 1f55 1f5a 1f5b 1f60
1f63 1f67 1f6b 1f6e 1f71 1f72 1f74 1f75
1f7a 1f7b 1f80 1f82 1f86 1f89 1f8b 1f8f
1f91 1f9d 1f9f 1fa1 1fa2 1fa7 1fab 1fad
1fb9 1fbb 1fbd 1fc1 1fc5 1fc8 1fcc 1fd0
1fd1 1fd6 1fda 1fde 1fe2 1fe6 1fea 1fed
1fee 1ff0 1ff4 1ff8 1ffb 1ffc 1ffe 2002
2006 2009 200d 200e 2010 2014 2018 201c
2020 2023 2024 2026 2027 2029 202c 202f
2030 2035 2039 203d 2041 2044 2048 204a
204e 2052 2056 2059 205d 205f 2063 2067
206a 206e 2073 2077 207b 207e 2082 2086
208a 208e 2092 2096 209a 209e 20a2 20a6
20a7 20a9 20ad 20b1 20b5 20b9 20bd 20c1
20c5 20c9 20cd 20d1 20d5 20d9 20dd 20e1
20e5 20e9 20ed 20f1 20f5 20f9 20fd 2101
2105 2111 2115 211a 211e 2121 2122 2127
212b 212f 2132 2135 2139 213b 213f 2143
2146 2149 214d 214f 2153 2157 215a 215c
2160 2164 2168 216b 216f 2173 2175 2176
217b 1 217f 2183 2187 218a 218e 2192
2196 219a 219d 21a1 21a5 21a8 21ab 21b0
21b3 21b6 21b9 21ba 21bf 21c0 1 21c2
21c7 21cb 21cf 21d2 21d7 21dc 21df 21e3
21e4 21e9 21ec 21f1 21f2 21f7 21fa 21ff
2200 2205 2208 220c 220d 2212 2215 221a
221b 2220 2223 2227 222b 222e 2231 2232
2234 2235 223a 223b 2240 2242 2246 2249
224b 224f 2251 225d 225f 2261 2262 2267
226b 226d 2279 227b 227f 2281 2285 2289
228c 2290 2293 2297 229a 229b 22a0 22a3
22a7 22ab 22af 22b3 22b7 22bb 22bf 22c3
22c7 22d3 22d7 22dc 22e0 22e3 22e4 22e9
22ed 22f1 22f4 22f7 22fb 22fd 2301 2305
2308 230b 230f 2311 2315 2319 231c 231e
2322 2326 232a 232d 2331 2335 2337 2338
233d 1 2341 2345 2349 234c 2350 2354
2358 235c 235f 2363 2367 236a 236d 2372
2375 2378 237b 237c 2381 2382 1 2384
2389 238d 2391 2394 2399 239e 23a1 23a5
23a6 23ab 23ae 23b3 23b4 23b9 23bc 23c1
23c2 23c7 23ca 23ce 23cf 23d4 23d7 23dc
23dd 23e2 23e5 23e9 23ed 23f0 23f3 23f4
23f6 23f7 23fc 23fd 2402 2404 2408 240b
240d 2411 2413 241f 2421 2423 2424 2429
242d 242f 243b 243d 2441 2445 2449 244a
244c 2450 2452 2456 245a 245d 2461 2465
2468 246c 2470 2473 2476 247b 247e 2481
2484 2485 248a 248b 1 248d 2492 2496
249a 249d 24a2 24a7 24aa 24ae 24af 24b4
24b7 24bc 24bd 24c2 24c5 24ca 24cb 24d0
24d3 24d7 24db 24df 24e3 24e6 24e7 24e9
24ee 24ef 24f1 24f2 24f7 24fa 24ff 2500
2505 2508 250d 250e 2513 2516 251a 251b
2520 2523 2528 2529 252e 252f 2534 2536
253a 253d 2541 2545 2548 254c 2550 2553
2557 255b 255c 2561 2563 2567 256b 256e
2572 2576 2579 257d 2581 2584 2587 258c
258f 2592 2595 2596 259b 259c 1 259e
25a3 25a7 25ab 25ae 25b3 25b8 25bb 25bf
25c0 25c5 25c6 25cb 25cf 25d3 25d6 25db
25e0 25e3 25e7 25eb 25ee 25ef 25f4 25f7
25fc 25fd 2602 2605 2609 260d 2610 2611
2616 2617 261c 261e 2622 2625 2629 262d
2631 2635 2639 263d 2641 2645 2649 2655
265a 265e 2662 2666 266a 266c 266d 2672
1 2676 267a 267e 2681 2685 2689 268d
2691 2694 2698 269c 269f 26a2 26a7 26aa
26ad 26b0 26b1 26b6 26b7 1 26b9 26be
26c2 26c6 26c9 26ce 26d3 26d6 26da 26db
26e0 26e1 26e6 26ea 26ee 26f1 26f6 26fb
26fe 2702 2703 2708 270b 2710 2711 2716
2719 271d 2721 2724 2727 2728 272a 272b
2730 2731 2736 2738 273c 273f 2743 2747
274b 274f 2753 2757 275b 2767 276c 2770
2774 2778 277c 277e 2782 2784 2790 2792
2794 2795 279a 279e 27a0 27ac 27b0 27b2
27b3 27bc 
b52
2
0 1 a 6 11 :3 6 10 :3 6
11 :3 6 12 :2 6 1 3 a 13
a :2 1 4 f 18 17 :2 f :2 4
11 1a 19 :2 11 :2 4 d 16 15
:2 d :2 4 10 19 18 :2 10 :2 4 e
17 16 :2 e :2 4 11 1a 19 :2 11
:2 4 10 19 18 :2 10 :2 4 :2 b 1a
b :2 4 :2 a 19 a :2 4 :2 e 1d
e :2 4 :2 d 1c d :2 4 :2 17 26
17 :2 4 17 20 1f :2 17 :2 4 :2 13
22 13 :2 4 14 1d 1c :2 14 :2 4
:2 14 23 14 :2 4 :2 11 20 11 :2 4
e 17 :3 e 4 7 :2 11 23 :2 2d
3a :2 23 :3 7 :2 d 19 25 :2 2f 43
:2 25 54 5a 5d :2 54 6b 81 8
:2 6b d 10 :2 6b 1b 1e :2 6b b
e :2 6b 13 16 :2 6b 1b 1e :2 6b
28 2b :2 6b b e :2 6b 13 16
:2 6b 1b 1e :2 6b 29 2c :2 6b b
e :2 6b 13 16 :2 6b 1b 1e :2 6b
2a 2d :2 6b 32 35 :2 6b :2 7 4a
:2 4 7 :2 11 23 :2 2d 39 3c 4c
4e 4f :2 4c :2 23 :3 7 :2 d 12 1e
5c 5f :2 1e :2 7 52 :2 4 :4 9 8
7 :4 23 22 21 :2 7 :4 3c 3b 3a
:2 7 a :2 14 26 :2 30 3c 3f 4f
51 52 :2 4f :2 26 :3 a :2 10 15 21
5f 62 :2 21 6f 72 :2 21 76 79
:2 21 83 86 8f 97 :2 a0 :2 8f b1
:2 86 :2 21 :2 a 55 :2 7 a :2 14 26
:2 30 3c 3f 4f 51 52 :2 4f :2 26
:3 a :2 10 15 21 5f 62 :2 21 :2 a
55 :3 7 :2 12 28 :2 31 41 :2 7 51
:2 4 e 1a 27 35 41 4f 5c
:2 7 :2 10 1c 7 4 c a :2 13
1f a 1a :2 7 :2 c a :2 13 1f
a d :2 17 29 :2 33 3f 43 53
55 56 :2 53 :2 29 :3 d :2 13 18 24
62 65 :2 24 72 75 :2 24 79 7c
:2 24 88 8b :2 24 93 96 :2 24 a2
a5 ac b5 b8 :2 a5 :2 24 :2 d 59
:2 a :2 7 :4 13 :2 7 4 :3 1 8 :2 11
1c 1a :2 25 :2 1a 7 11 1d 2a
38 44 52 5f :2 a :2 13 1f a
7 f d :2 16 22 d 1d :2 a
:2 f d :2 16 22 d 10 :2 1a 2c
:2 36 42 46 56 58 59 :2 56 :2 2c
:3 10 :2 16 1b 27 65 68 :2 27 75
78 :2 27 7c 7f :2 27 8b 8e :2 27
96 99 :2 27 a5 a8 af b8 bb
:2 a8 :2 27 :2 10 5c :2 d :2 a :4 16 :2 a
7 :3 35 b :2 14 1f 1d :2 28 :2 1d
a d :2 17 29 :2 33 3f 43 53
55 56 :2 53 :2 29 :3 d :2 13 18 24
62 65 :2 24 72 75 :2 24 79 7c
:2 24 86 89 92 9a :2 a3 :2 92 b6
:2 89 :2 24 be c1 :2 24 c9 cc :2 24
d1 d4 :2 24 df e2 :2 24 :2 d 59
:2 a d :2 17 29 :2 33 3f 42 52
54 55 :2 52 :2 29 :3 d :2 13 18 24
62 65 :2 24 :2 d 58 :3 a :2 15 29
:2 32 44 50 :2 a 38 :3 7 10 :2 7
17 :2 20 7 35 7 10 :2 7 17
:2 20 7 :5 4 10 4 e 1a 28
38 42 4f c 1b 28 :2 7 :2 10
1c 7 4 c a :2 13 1f a
1a :2 7 :2 c a :2 13 1f a d
:2 17 29 :2 33 3f 43 53 56 57
:2 53 :2 29 :3 d :2 13 18 24 63 66
:2 24 73 76 :2 24 7a 7d :2 24 89
8c :2 24 94 97 :2 24 a3 a6 ad
b6 b9 :2 a6 :2 24 :2 d 5a :2 a :2 7
:4 13 :2 7 4 :3 1 8 :2 11 1c 1a
:2 25 :2 1a :2 7 16 :2 7 14 1b 28
2b :2 14 7 11 1a 1e 35 f
10 1b 2c 34 :2 a :2 13 1f a
7 f d :2 16 22 d 1d :2 a
:2 f d :2 16 22 d 10 :2 1a 2c
:2 36 42 46 56 59 5a :2 56 :2 2c
:3 10 :2 16 1b 27 66 69 :2 27 76
79 :2 27 7d 80 :2 27 8c 8f :2 27
97 9a :2 27 a6 a9 b0 b9 bc
:2 a9 :2 27 :2 10 5d :2 d :2 a :4 16 :2 a
7 :3 35 11 1c 29 34 41 4c
59 64 71 7c 89 8f 93 a9
af b2 c8 ce d6 ed f 20
24 35 10 13 20 2e 32 3f
42 :2 a :2 13 1f a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
56 59 5a :2 56 :2 2c :3 10 :2 16 1b
27 66 69 :2 27 76 79 :2 27 7d
80 :2 27 8c 8f :2 27 97 9a :2 27
a6 a9 b0 b9 bc :2 a9 :2 27 :2 10
5d :2 d :2 a :4 16 :2 a 7 :3 35 b
:2 14 1f 1d :2 28 :2 1d a d :2 17
29 :2 33 3f 43 53 56 57 :2 53
:2 29 :3 d :2 13 18 24 63 66 :2 24
73 76 :2 24 7a 7d :2 24 87 8a
93 9b :2 a4 :2 93 b7 :2 8a :2 24 bf
c2 :2 24 ca cd :2 24 d2 d5 :2 24
de e1 :2 24 :2 d 5a :2 a d :2 17
29 :2 33 3f 42 52 55 56 :2 52
:2 29 :3 d :2 13 18 24 63 66 :2 24
:2 d 59 :3 a :2 15 29 :2 32 44 4e
:2 a 38 :2 7 16 1f 2a 35 3d
4a 11 1b 29 34 3e a 11
d 19 :3 17 d :2 16 22 d 1e
d :2 16 22 d :4 a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
56 59 5a :2 56 :2 2c :3 10 :2 16 1b
27 66 69 :2 27 76 79 :2 27 7d
80 :2 27 8c 8f :2 27 97 9a :2 27
a6 a9 b0 b9 bc :2 a9 :2 27 :2 10
5d :2 d :2 a :4 16 :2 a 7 :3 35 b
:2 14 20 1d :2 29 :2 1d a d :2 17
29 :2 33 3f 43 53 56 57 :2 53
:2 29 :3 d :2 13 18 24 63 66 :2 24
73 76 :2 24 7a 7d :2 24 87 8a
93 9b :2 a4 :2 93 af :2 8a :2 24 b7
ba :2 24 c2 c5 :2 24 ca cd :2 24
d7 da :2 24 :2 d 5a :2 a d :2 17
29 :2 33 3f 42 52 55 56 :2 52
:2 29 :3 d :2 13 18 24 63 66 :2 24
:2 d 59 :3 a :2 15 29 :2 32 3c 47
:2 a 39 :2 7 4 35 b :2 14 1f
1d :2 28 :2 1d :2 a 1b 24 2c :2 1b
:2 a 30 33 :2 30 14 16 2d 12
27 13 15 22 11 13 21 11
13 :2 d :2 16 22 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
59 5c 5d :2 59 :2 2f :3 13 :2 19 1e
2a 69 6c :2 2a 79 7c :2 2a 80
83 :2 2a 8f 92 :2 2a 9a 9d :2 2a
a9 ac b3 bc bf :2 ac :2 2a :2 13
60 :2 10 :2 d :4 19 :2 d a :3 35 e
:2 17 22 20 :2 2b :2 20 d 10 :2 1a
2c :2 36 42 46 56 59 5a :2 56
:2 2c :3 10 :2 16 1b 27 66 69 :2 27
76 79 :2 27 7d 80 :2 27 8a 8d
96 9e :2 a7 :2 96 c1 :2 8d :2 27 c9
cc :2 27 d4 d7 :2 27 dc df :2 27
e9 ec :2 27 :2 10 5d :2 d 10 :2 1a
2c :2 36 42 45 55 58 59 :2 55
:2 2c :3 10 :2 16 1b 27 66 69 :2 27
:2 10 5c :3 d :2 18 2c :2 35 4e 59
:2 d 3b :2 a 35 :2 7 a 11 1f
22 :2 a 25 27 :2 25 14 21 38
45 d 14 10 1c :3 1a 10 :2 19
25 10 21 10 :2 19 25 10 :4 d
a 12 10 :2 19 25 10 20 :2 d
:2 12 10 :2 19 25 10 13 :2 1d 2f
:2 39 45 49 59 5c 5d :2 59 :2 2f
:3 13 :2 19 1e 2a 69 6c :2 2a 79
7c :2 2a 80 83 :2 2a 8f 92 :2 2a
9a 9d :2 2a a9 ac b3 bc bf
:2 ac :2 2a :2 13 60 :2 10 :2 d :4 19 :2 d
a :4 2b :2 7 38 35 a :2 14 26
:2 30 3c 40 50 53 54 :2 50 :2 26
:3 a :2 10 15 21 60 63 :2 21 70
73 :2 21 77 7a :2 21 84 87 90
98 :2 a1 :2 90 b4 :2 87 :2 21 bc bf
:2 21 c7 ca :2 21 cf d2 :2 21 dc
df :2 21 :2 a 57 :2 7 a :2 14 26
:2 30 3c 3f 4f 52 53 :2 4f :2 26
:3 a :2 10 15 21 60 63 :2 21 :2 a
56 :3 7 :2 12 26 :2 2f 41 4c :2 7
:5 4 11 4 e 25 c d 1a
b 19 b :2 7 :2 10 1c 7 4
c a :2 13 1f a 1a :2 7 :2 c
a :2 13 1f a d :2 17 29 :2 33
3f 43 53 56 57 :2 53 :2 29 :3 d
:2 13 18 24 63 66 :2 24 73 76
:2 24 7a 7d :2 24 89 8c :2 24 94
97 :2 24 a3 a6 ad b6 b9 :2 a6
:2 24 :2 d 5a :2 a :2 7 :4 13 :2 7 4
:3 1 8 :2 11 1c 1a :2 25 :2 1a 7
a 13 1a :2 a 1d 1f :2 1d 14
20 12 19 27 44 12 19 28
2f 49 12 22 13 20 d 14
10 1c :3 1a 10 :2 19 25 10 21
10 :2 19 25 10 :4 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
59 5c 5d :2 59 :2 2f :3 13 :2 19 1e
2a 69 6c :2 2a 79 7c :2 2a 80
83 :2 2a 8f 92 :2 2a 9a 9d :2 2a
a9 ac b3 bc bf :2 ac :2 2a :2 13
60 :2 10 :2 d :4 19 :2 d a :3 21 7
21 d 16 1d :2 d 20 22 :2 20
14 20 12 19 27 44 12 19
28 2f 49 12 22 13 20 d
14 10 1c :3 1a 10 :2 19 25 10
21 10 :2 19 25 10 :4 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 59 5c 5d :2 59 :2 2f :3 13 :2 19
1e 2a 69 6c :2 2a 79 7c :2 2a
80 83 :2 2a 8f 92 :2 2a 9a 9d
:2 2a a9 ac b3 bc bf :2 ac :2 2a
:2 13 60 :2 10 :2 d :4 19 :2 d a :4 24
21 :3 7 :3 15 :2 7 1a 28 31 3f
:2 28 43 4c 54 :2 43 24 :2 2d 42
:2 1a 7 a 18 21 29 :2 18 :2 a
2d 30 :2 2d a 20 :2 29 a 32
a 20 :2 29 a :5 7 1a :2 7 19
:2 7 1a 31 3f 49 51 2d 41
52 :2 1a 7 16 2b 3e 4a 56
a 19 26 33 40 4a 57 12
1e 2b 37 f 19 2d 41 4e
5b a 11 d 19 :3 17 d :2 16
22 d 1e d :2 16 22 d :4 a
7 f d :2 16 22 d 1d :2 a
:2 f d :2 16 22 d 10 :2 1a 2c
:2 36 42 46 56 59 5a :2 56 :2 2c
:3 10 :2 16 1b 27 66 69 :2 27 76
79 :2 27 7d 80 :2 27 8c 8f :2 27
97 9a :2 27 a6 a9 b0 b9 bc
:2 a9 :2 27 :2 10 5d :2 d :2 a :4 16 :2 a
7 :3 35 4 35 b :2 14 1f 1d
:2 28 :2 1d a 11 2a 38 41 48
55 5e 10 24 a 11 d 19
:3 17 d :2 16 22 d 1e d :2 16
22 d :4 a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 56 59
5a :2 56 :2 2c :3 10 :2 16 1b 27 66
69 :2 27 76 79 :2 27 7d 80 :2 27
8c 8f :2 27 97 9a :2 27 a6 a9
b0 b9 bc :2 a9 :2 27 :2 10 5d :2 d
:2 a :4 16 :2 a 7 :3 38 7 1a 30
:2 1a 7 38 35 :2 4 7 :2 11 23
:2 2d 39 3c 4c 4f 50 :2 4c :2 23
:3 7 :2 d 12 1e 5d 60 :2 1e 6d
70 :2 1e 74 77 :2 1e 81 84 8d
95 :2 9e :2 8d a9 :2 84 :2 1e b1 b4
:2 1e bc bf :2 1e c4 c7 :2 1e d2
d5 :2 1e :2 7 53 :3 4 :2 f 21 :2 2a
34 40 :2 4 1 9 12 9 a
:2 14 26 :2 30 3c 40 50 53 54
:2 50 :2 26 :3 a :2 10 15 21 60 63
:2 21 :3 a :2 10 15 39 5c 5f :2 68
:2 39 76 79 :2 39 81 84 :2 8d :2 39
:2 a 57 :3 7 c 15 1e 2c a
13 1e f e 7 e 7 e
7 29 :2 4 :2 9 7 :2 10 1c :2 7
:2 11 23 :2 2d 39 3d 4d 50 51
:2 4d :2 23 :3 7 :2 d 12 1e 5d 60
:2 1e :3 7 :2 d 12 36 42 45 :2 36
4d 50 :2 36 5c 5f 66 6f 72
:2 5f :2 36 :2 7 54 :2 4 7 c 1b
a 13 1e f e 7 e 7
e 7 :2 4 :4 10 :2 4 :8 1 
b52
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 2 :4 7
:2 1 :7 9 :7 a :7 b
:7 c :7 d :7 e :7 f
:6 10 :6 11 :6 12 :6 13
:6 14 :7 15 :6 16 :7 17
:6 18 :6 19 :7 1b :b 1d
:11 1e 1f :2 1e :2 1f
:2 1e :2 1f :2 1e :2 20
:2 1e :2 20 :2 1e :2 20
:2 1e :2 20 :2 1e :2 21
:2 1e :2 21 :2 1e :2 21
:2 1e :2 21 :2 1e :2 22
:2 1e :2 22 :2 1e :2 22
:2 1e :2 22 :2 1e :2 22
:4 1e :3 1d :11 25 :b 26
:3 25 :16 2b :11 2c :20 2d
:3 2c :11 2f :b 30 :3 2f
:9 32 :3 2b :8 36 :5 37
35 39 :5 3a :3 39
:2 3b :5 3d :11 3e :24 3f
:3 3e :2 3c :6 3b 38
:3 1c :a 44 :8 46 :5 47
45 49 :5 4a :3 49
:2 4b :5 4d :11 4e :24 4f
:3 4e :2 4c :6 4b 48
:3 44 :a 54 :11 55 :30 56
:3 55 :11 58 :b 59 :3 58
:a 5b :3 54 :3 5d :5 5e
44 :3 60 :5 61 5f
:3 44 :3 64 :6 68 :3 69
68 :5 6a 67 6c
:5 6d :3 6c :2 6e :5 70
:11 71 :24 72 :3 71 :2 6f
:6 6e 6b :3 1c :a 77
:3 78 :8 7a :4 7d 7e
:4 7f 7d :5 80 7c
82 :5 83 :3 82 :2 84
:5 86 :11 87 :24 88 :3 87
:2 85 :6 84 81 :3 77
:14 8e :4 8f :7 90 8e
:5 91 8d 93 :5 94
:3 93 :2 95 :5 97 :11 98
:24 99 :3 98 :2 96 :6 95
92 :3 77 :a 9e :11 9f
:30 a0 :3 9f :11 a2 :b a3
:3 a2 :a a5 :3 9e :6 a9
:5 aa a9 :6 ab :5 ac
ab :5 ae ad :3 ab
a8 b1 :5 b2 :3 b1
:2 b3 :5 b5 :11 b6 :24 b7
:3 b6 :2 b4 :6 b3 b0
:3 77 :a bc :11 bd :30 be
:3 bd :11 c0 :b c1 :3 c0
:a c3 :3 bc c5 77
:a c5 :c c6 :3 c8 :2 c9
:3 ca :3 cb :2 cc c8
:5 cd c7 cf :5 d0
:3 cf :2 d1 :5 d3 :11 d4
:24 d5 :3 d4 :2 d2 :6 d1
ce :3 c6 :a da :11 db
:30 dc :3 db :11 de :b df
:3 de :a e1 :3 da :3 c6
:a e5 :5 e7 :6 e8 :5 e9
e8 :5 eb ea :3 e8
e6 ee :5 ef :3 ee
:2 f0 :5 f2 :11 f3 :24 f4
:3 f3 :2 f1 :6 f0 ed
:6 e5 c5 77 :11 fa
:30 fb :3 fa :11 fd :b fe
:3 fd :a 100 f9 :3 77
:3 103 :2 105 106 :2 107
:2 108 109 105 :5 10a
104 10c :5 10d :3 10c
:2 10e :5 110 :11 111 :24 112
:3 111 :2 10f :6 10e 10b
:3 1c :a 118 :9 119 :2 11b
:4 11d :5 120 :2 121 :2 124
11b :6 125 :5 126 125
:5 128 127 :3 125 11a
12b :5 12c :3 12b :2 12d
:5 12f :11 130 :24 131 :3 130
:2 12e :6 12d 12a :3 119
135 119 :9 135 :2 137
:4 139 :5 13c :2 13d :2 140
137 :6 141 :5 142 141
:5 144 143 :3 141 136
147 :5 148 :3 147 :2 149
:5 14b :11 14c :24 14d :3 14c
:2 14a :6 149 146 :4 135
:3 119 :5 154 :c 155 :4 156
:3 155 :c 158 :5 159 158
:5 15b 15a :3 158 :3 15e
:3 15f :6 161 :3 162 :3 161
:5 165 :7 166 :4 167 :6 168
165 :6 169 :5 16a 169
:5 16c 16b :3 169 164
16f :5 170 :3 16f :2 171
:5 173 :11 174 :24 175 :3 174
:2 172 :6 171 16e :3 118
17a 118 :a 17a :7 17c
:2 17d 17c :6 17e :5 17f
17e :5 181 180 :3 17e
17b 184 :5 185 :3 184
:2 186 :5 188 :11 189 :24 18a
:3 189 :2 187 :6 186 183
:3 17a :6 18f 17a :3 118
:11 192 :30 193 :3 192 :a 195
1c :3 198 :11 199 :b 19a
:17 19b :3 199 :2 19d :3 19e
:3 19f 1a0 19e 19d
19e :3 1a1 :3 198 :2 1a2
:5 1a4 :11 1a5 :b 1a6 :18 1a7
:3 1a5 :2 1a9 1aa :3 1ab
1ac 1aa 1a9 1aa
:3 1ad :2 1a3 :6 1a2 197
:7 1 
27be
4
:3 0 1 :3 0 2
:a 0 b4d 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 4
:3 0 5 :7 0 9
8 :3 0 4 :3 0
6 :7 0 d c
:3 0 15 16 0
b 4 :3 0 7
:7 0 11 10 :3 0
8 :3 0 9 :3 0
a :2 0 4 13
17 0 b4d 2
18 :2 0 c :2 0
12 4 :3 0 c
:2 0 10 1b 1d
:6 0 20 1e 0
b4b 0 b :6 0
c :2 0 16 4
:3 0 14 22 24
:6 0 27 25 0
b4b 0 d :6 0
10 :2 0 1a 4
:3 0 18 29 2b
:6 0 2e 2c 0
b4b 0 e :6 0
c :2 0 1e 4
:3 0 1c 30 32
:6 0 35 33 0
b4b 0 f :6 0
13 :2 0 22 4
:3 0 20 37 39
:6 0 3c 3a 0
b4b 0 11 :6 0
15 :2 0 26 4
:3 0 24 3e 40
:6 0 43 41 0
b4b 0 12 :6 0
18 :2 0 2a 4
:3 0 28 45 47
:6 0 4a 48 0
b4b 0 14 :6 0
18 :2 0 2c 17
:3 0 4c :7 0 50
4d 4e b4b 0
16 :6 0 18 :2 0
2e 17 :3 0 52
:7 0 56 53 54
b4b 0 19 :6 0
18 :2 0 30 17
:3 0 58 :7 0 5c
59 5a b4b 0
1a :6 0 18 :2 0
32 17 :3 0 5e
:7 0 62 5f 60
b4b 0 1b :6 0
c :2 0 34 17
:3 0 64 :7 0 68
65 66 b4b 0
1c :6 0 18 :2 0
38 4 :3 0 36
6a 6c :6 0 6f
6d 0 b4b 0
1d :6 0 13 :2 0
3a 17 :3 0 71
:7 0 75 72 73
b4b 0 1e :6 0
18 :2 0 3e 4
:3 0 3c 77 79
:6 0 7c 7a 0
b4b 0 1f :6 0
18 :2 0 40 17
:3 0 7e :7 0 82
7f 80 b4b 0
20 :6 0 8a 8b
0 42 17 :3 0
84 :7 0 88 85
86 b4b 0 21
:6 0 90 91 0
44 9 :3 0 a
:2 0 4 8c :7 0
8f 8d 0 b4b
0 22 :6 0 23
:3 0 24 :3 0 23
:3 0 25 :3 0 93
94 0 2 :4 0
46 95 97 92
99 98 :2 0 26
:3 0 27 :3 0 9b
9c 0 28 :4 0
23 :3 0 29 :3 0
9f a0 0 2
:4 0 48 a1 a3
2a :4 0 2b :2 0
2c :3 0 4a a6
a8 :3 0 2d :4 0
2b :2 0 2e :4 0
4d ab ad :3 0
2b :2 0 3 :3 0
50 af b1 :3 0
2b :2 0 2e :4 0
53 b3 b5 :3 0
2b :2 0 2f :4 0
56 b7 b9 :3 0
2b :2 0 2e :4 0
59 bb bd :3 0
2b :2 0 5 :3 0
5c bf c1 :3 0
2b :2 0 2e :4 0
5f c3 c5 :3 0
2b :2 0 2f :4 0
62 c7 c9 :3 0
2b :2 0 2e :4 0
65 cb cd :3 0
2b :2 0 6 :3 0
68 cf d1 :3 0
2b :2 0 2e :4 0
6b d3 d5 :3 0
2b :2 0 2f :4 0
6e d7 d9 :3 0
2b :2 0 2e :4 0
71 db dd :3 0
2b :2 0 7 :3 0
74 df e1 :3 0
2b :2 0 2e :4 0
77 e3 e5 :3 0
2b :2 0 30 :4 0
7a e7 e9 :3 0
7d 9d eb :2 0
ed 82 ee 9a
ed 0 ef 84
0 aab 23 :3 0
24 :3 0 f0 f1
0 23 :3 0 31
:3 0 f3 f4 0
13 :2 0 2 :4 0
32 :2 0 33 :2 0
13 :2 0 86 f9
fb :3 0 89 f5
fd f2 ff fe
:2 0 26 :3 0 34
:3 0 101 102 0
28 :4 0 35 :4 0
2b :2 0 2c :3 0
8d 106 108 :3 0
90 103 10a :2 0
10c 93 10d 100
10c 0 10e 95
0 aab 3 :3 0
36 :2 0 97 110
111 :3 0 112 :2 0
113 :2 0 5 :3 0
36 :2 0 99 116
117 :3 0 118 :2 0
119 :2 0 114 11b
11a :2 0 6 :3 0
36 :2 0 9b 11e
11f :3 0 120 :2 0
121 :2 0 11c 123
122 :2 0 23 :3 0
24 :3 0 125 126
0 23 :3 0 31
:3 0 128 129 0
37 :2 0 2 :4 0
38 :2 0 33 :2 0
13 :2 0 9d 12e
130 :3 0 a0 12a
132 127 134 133
:2 0 26 :3 0 34
:3 0 136 137 0
28 :4 0 39 :4 0
2b :2 0 2c :3 0
a4 13b 13d :3 0
2b :2 0 3a :4 0
a7 13f 141 :3 0
2b :2 0 3b :4 0
aa 143 145 :3 0
2b :2 0 3c :3 0
3d :3 0 3e :3 0
3f :3 0 14a 14b
0 ad 149 14d
40 :4 0 af 148
150 b2 147 152
:3 0 b5 138 154
:2 0 156 b8 157
135 156 0 158
ba 0 181 23
:3 0 24 :3 0 159
15a 0 23 :3 0
31 :3 0 15c 15d
0 13 :2 0 2
:4 0 41 :2 0 33
:2 0 13 :2 0 bc
162 164 :3 0 bf
15e 166 15b 168
167 :2 0 26 :3 0
34 :3 0 16a 16b
0 28 :4 0 42
:4 0 2b :2 0 2c
:3 0 c3 16f 171
:3 0 c6 16c 173
:2 0 175 c9 176
169 175 0 177
cb 0 181 43
:3 0 44 :3 0 178
179 0 3e :3 0
3f :3 0 17b 17c
0 45 :3 0 cd
17a 17f :2 0 181
d0 182 124 181
0 183 d4 0
aab 46 :3 0 47
:3 0 d :3 0 16
:3 0 48 :3 0 49
:3 0 3 :4 0 4a
1 :8 0 191 9
:3 0 4b :3 0 18c
18d 0 4c :3 0
18e 18f 0 191
d6 1e4 4d :3 0
9 :3 0 4b :3 0
193 194 0 4c
:3 0 195 196 0
198 d9 19a db
199 198 :2 0 1e2
4e :3 0 9 :3 0
4b :3 0 19d 19e
0 4c :3 0 19f
1a0 0 1da 23
:3 0 24 :3 0 1a2
1a3 0 23 :3 0
31 :3 0 1a5 1a6
0 4f :2 0 2
:4 0 50 :2 0 33
:2 0 13 :2 0 dd
1ab 1ad :3 0 e0
1a7 1af 1a4 1b1
1b0 :2 0 26 :3 0
34 :3 0 1b3 1b4
0 28 :4 0 51
:4 0 2b :2 0 2c
:3 0 e4 1b8 1ba
:3 0 2b :2 0 3a
:4 0 e7 1bc 1be
:3 0 2b :2 0 52
:4 0 ea 1c0 1c2
:3 0 2b :2 0 4c
:3 0 ed 1c4 1c6
:3 0 2b :2 0 53
:4 0 f0 1c8 1ca
:3 0 2b :2 0 54
:3 0 55 :3 0 13
:2 0 56 :2 0 f3
1cd 1d1 f7 1cc
1d3 :3 0 fa 1b5
1d5 :2 0 1d7 fd
1d8 1b2 1d7 0
1d9 ff 0 1da
101 1dd :3 0 1dd
0 1dd 1dc 1da
1db :6 0 1df 2
:3 0 104 1e1 106
1e0 1df :2 0 1e2
108 :2 0 1e4 0
1e4 1e3 191 1e2
:6 0 aab 1 :3 0
9 :3 0 4b :3 0
1e6 1e7 0 9
:3 0 57 :2 0 58
:3 0 1e9 1eb 0
10d 1ea 1ed :3 0
1ee :2 0 46 :3 0
47 :3 0 d :3 0
16 :3 0 59 :3 0
49 :3 0 3 :4 0
5a 1 :8 0 1fd
9 :3 0 4b :3 0
1f8 1f9 0 4c
:3 0 1fa 1fb 0
1fd 110 250 4d
:3 0 9 :3 0 4b
:3 0 1ff 200 0
4c :3 0 201 202
0 204 113 206
115 205 204 :2 0
24e 4e :3 0 9
:3 0 4b :3 0 209
20a 0 4c :3 0
20b 20c 0 246
23 :3 0 24 :3 0
20e 20f 0 23
:3 0 31 :3 0 211
212 0 4f :2 0
2 :4 0 5b :2 0
33 :2 0 13 :2 0
117 217 219 :3 0
11a 213 21b 210
21d 21c :2 0 26
:3 0 34 :3 0 21f
220 0 28 :4 0
5c :4 0 2b :2 0
2c :3 0 11e 224
226 :3 0 2b :2 0
3a :4 0 121 228
22a :3 0 2b :2 0
52 :4 0 124 22c
22e :3 0 2b :2 0
4c :3 0 127 230
232 :3 0 2b :2 0
53 :4 0 12a 234
236 :3 0 2b :2 0
54 :3 0 55 :3 0
13 :2 0 56 :2 0
12d 239 23d 131
238 23f :3 0 134
221 241 :2 0 243
137 244 21e 243
0 245 139 0
246 13b 249 :3 0
249 0 249 248
246 247 :6 0 24b
4 :3 0 13e 24d
140 24c 24b :2 0
24e 142 :2 0 250
0 250 24f 1fd
24e :6 0 2d4 1
:3 0 9 :3 0 4b
:3 0 252 253 0
9 :3 0 57 :2 0
58 :3 0 255 257
0 147 256 259
:3 0 25a :2 0 23
:3 0 24 :3 0 25c
25d 0 23 :3 0
31 :3 0 25f 260
0 c :2 0 2
:4 0 5d :2 0 33
:2 0 13 :2 0 14a
265 267 :3 0 14d
261 269 25e 26b
26a :2 0 26 :3 0
34 :3 0 26d 26e
0 28 :4 0 5e
:4 0 2b :2 0 2c
:3 0 151 272 274
:3 0 2b :2 0 3a
:4 0 154 276 278
:3 0 2b :2 0 3b
:4 0 157 27a 27c
:3 0 2b :2 0 3c
:3 0 3d :3 0 3e
:3 0 5f :3 0 281
282 0 15a 280
284 40 :4 0 15c
27f 287 15f 27e
289 :3 0 2b :2 0
60 :4 0 162 28b
28d :3 0 2b :2 0
2e :4 0 165 28f
291 :3 0 2b :2 0
3 :3 0 168 293
295 :3 0 2b :2 0
2e :4 0 16b 297
299 :3 0 16e 26f
29b :2 0 29d 171
29e 26c 29d 0
29f 173 0 2c9
23 :3 0 24 :3 0
2a0 2a1 0 23
:3 0 31 :3 0 2a3
2a4 0 13 :2 0
2 :4 0 61 :2 0
33 :2 0 13 :2 0
175 2a9 2ab :3 0
178 2a5 2ad 2a2
2af 2ae :2 0 26
:3 0 34 :3 0 2b1
2b2 0 28 :4 0
62 :4 0 2b :2 0
2c :3 0 17c 2b6
2b8 :3 0 17f 2b3
2ba :2 0 2bc 182
2bd 2b0 2bc 0
2be 184 0 2c9
43 :3 0 63 :3 0
2bf 2c0 0 3e
:3 0 5f :3 0 2c2
2c3 0 3 :3 0
45 :3 0 186 2c1
2c7 :2 0 2c9 18a
2ca 25b 2c9 0
2cb 18e 0 2d4
19 :3 0 13 :2 0
2cc 2cd 0 2d4
21 :3 0 3e :3 0
64 :3 0 2d0 2d1
0 2cf 2d2 0
2d4 190 2de 19
:3 0 18 :2 0 2d5
2d6 0 2dd 21
:3 0 3e :3 0 65
:3 0 2d9 2da 0
2d8 2db 0 2dd
195 2df 1ef 2d4
0 2e0 0 2dd
0 2e0 198 0
aab e :3 0 5
:3 0 2e1 2e2 0
aab 66 :3 0 67
:3 0 68 :3 0 e
:3 0 14 :3 0 12
:3 0 69 :3 0 66
:3 0 e :4 0 6a
1 :8 0 2f3 9
:3 0 4b :3 0 2ee
2ef 0 4c :3 0
2f0 2f1 0 2f3
19b 346 4d :3 0
9 :3 0 4b :3 0
2f5 2f6 0 4c
:3 0 2f7 2f8 0
2fa 19e 2fc 1a0
2fb 2fa :2 0 344
4e :3 0 9 :3 0
4b :3 0 2ff 300
0 4c :3 0 301
302 0 33c 23
:3 0 24 :3 0 304
305 0 23 :3 0
31 :3 0 307 308
0 4f :2 0 2
:4 0 6b :2 0 33
:2 0 13 :2 0 1a2
30d 30f :3 0 1a5
309 311 306 313
312 :2 0 26 :3 0
34 :3 0 315 316
0 28 :4 0 6c
:4 0 2b :2 0 2c
:3 0 1a9 31a 31c
:3 0 2b :2 0 3a
:4 0 1ac 31e 320
:3 0 2b :2 0 52
:4 0 1af 322 324
:3 0 2b :2 0 4c
:3 0 1b2 326 328
:3 0 2b :2 0 53
:4 0 1b5 32a 32c
:3 0 2b :2 0 54
:3 0 55 :3 0 13
:2 0 56 :2 0 1b8
32f 333 1bc 32e
335 :3 0 1bf 317
337 :2 0 339 1c2
33a 314 339 0
33b 1c4 0 33c
1c6 33f :3 0 33f
0 33f 33e 33c
33d :6 0 341 6
:3 0 1c9 343 1cb
342 341 :2 0 344
1cd :2 0 346 0
346 345 2f3 344
:6 0 aab 1 :3 0
9 :3 0 4b :3 0
348 349 0 9
:3 0 57 :2 0 58
:3 0 34b 34d 0
1d2 34c 34f :3 0
350 :2 0 f :3 0
6 :3 0 352 353
0 5a6 11 :3 0
54 :3 0 f :3 0
13 :2 0 c :2 0
1d5 356 35a 355
35b 0 5a6 3c
:3 0 6d :3 0 6e
:3 0 1b :3 0 69
:3 0 46 :3 0 d
:3 0 6f :3 0 11
:4 0 70 1 :8 0
36c 9 :3 0 4b
:3 0 367 368 0
4c :3 0 369 36a
0 36c 1d9 3bf
4d :3 0 9 :3 0
4b :3 0 36e 36f
0 4c :3 0 370
371 0 373 1dc
375 1de 374 373
:2 0 3bd 4e :3 0
9 :3 0 4b :3 0
378 379 0 4c
:3 0 37a 37b 0
3b5 23 :3 0 24
:3 0 37d 37e 0
23 :3 0 31 :3 0
380 381 0 4f
:2 0 2 :4 0 71
:2 0 33 :2 0 13
:2 0 1e0 386 388
:3 0 1e3 382 38a
37f 38c 38b :2 0
26 :3 0 34 :3 0
38e 38f 0 28
:4 0 72 :4 0 2b
:2 0 2c :3 0 1e7
393 395 :3 0 2b
:2 0 3a :4 0 1ea
397 399 :3 0 2b
:2 0 52 :4 0 1ed
39b 39d :3 0 2b
:2 0 4c :3 0 1f0
39f 3a1 :3 0 2b
:2 0 53 :4 0 1f3
3a3 3a5 :3 0 2b
:2 0 54 :3 0 55
:3 0 13 :2 0 56
:2 0 1f6 3a8 3ac
1fa 3a7 3ae :3 0
1fd 390 3b0 :2 0
3b2 200 3b3 38d
3b2 0 3b4 202
0 3b5 204 3b8
:3 0 3b8 0 3b8
3b7 3b5 3b6 :6 0
3ba 8 :3 0 207
3bc 209 3bb 3ba
:2 0 3bd 20b :2 0
3bf 0 3bf 3be
36c 3bd :6 0 5a6
1 :3 0 73 :3 0
74 :3 0 73 :3 0
74 :3 0 73 :3 0
74 :3 0 73 :3 0
74 :3 0 73 :3 0
74 :3 0 75 :3 0
76 :3 0 77 :3 0
75 :3 0 78 :3 0
77 :3 0 75 :3 0
3d :3 0 1b :3 0
14 :3 0 79 :3 0
78 :3 0 79 :3 0
76 :3 0 78 :3 0
49 :3 0 11 :3 0
76 :3 0 49 :3 0
78 :3 0 7a :4 0
7b 1 :8 0 3e6
9 :3 0 4b :3 0
3e1 3e2 0 4c
:3 0 3e3 3e4 0
3e6 20e 439 4d
:3 0 9 :3 0 4b
:3 0 3e8 3e9 0
4c :3 0 3ea 3eb
0 3ed 211 3ef
213 3ee 3ed :2 0
437 4e :3 0 9
:3 0 4b :3 0 3f2
3f3 0 4c :3 0
3f4 3f5 0 42f
23 :3 0 24 :3 0
3f7 3f8 0 23
:3 0 31 :3 0 3fa
3fb 0 4f :2 0
2 :4 0 7c :2 0
33 :2 0 13 :2 0
215 400 402 :3 0
218 3fc 404 3f9
406 405 :2 0 26
:3 0 34 :3 0 408
409 0 28 :4 0
7d :4 0 2b :2 0
2c :3 0 21c 40d
40f :3 0 2b :2 0
3a :4 0 21f 411
413 :3 0 2b :2 0
52 :4 0 222 415
417 :3 0 2b :2 0
4c :3 0 225 419
41b :3 0 2b :2 0
53 :4 0 228 41d
41f :3 0 2b :2 0
54 :3 0 55 :3 0
13 :2 0 56 :2 0
22b 422 426 22f
421 428 :3 0 232
40a 42a :2 0 42c
235 42d 407 42c
0 42e 237 0
42f 239 432 :3 0
432 0 432 431
42f 430 :6 0 434
a :3 0 23c 436
23e 435 434 :2 0
437 240 :2 0 439
0 439 438 3e6
437 :6 0 5a6 1
:3 0 9 :3 0 4b
:3 0 43b 43c 0
9 :3 0 57 :2 0
58 :3 0 43e 440
0 245 43f 442
:3 0 443 :2 0 23
:3 0 24 :3 0 445
446 0 23 :3 0
31 :3 0 448 449
0 c :2 0 2
:4 0 7e :2 0 33
:2 0 13 :2 0 248
44e 450 :3 0 24b
44a 452 447 454
453 :2 0 26 :3 0
34 :3 0 456 457
0 28 :4 0 7f
:4 0 2b :2 0 2c
:3 0 24f 45b 45d
:3 0 2b :2 0 3a
:4 0 252 45f 461
:3 0 2b :2 0 3b
:4 0 255 463 465
:3 0 2b :2 0 3c
:3 0 3d :3 0 3e
:3 0 5f :3 0 46a
46b 0 258 469
46d 40 :4 0 25a
468 470 25d 467
472 :3 0 2b :2 0
60 :4 0 260 474
476 :3 0 2b :2 0
2e :4 0 263 478
47a :3 0 2b :2 0
e :3 0 266 47c
47e :3 0 2b :2 0
2e :4 0 269 480
482 :3 0 26c 458
484 :2 0 486 26f
487 455 486 0
488 271 0 4b2
23 :3 0 24 :3 0
489 48a 0 23
:3 0 31 :3 0 48c
48d 0 13 :2 0
2 :4 0 80 :2 0
33 :2 0 13 :2 0
273 492 494 :3 0
276 48e 496 48b
498 497 :2 0 26
:3 0 34 :3 0 49a
49b 0 28 :4 0
81 :4 0 2b :2 0
2c :3 0 27a 49f
4a1 :3 0 27d 49c
4a3 :2 0 4a5 280
4a6 499 4a5 0
4a7 282 0 4b2
43 :3 0 63 :3 0
4a8 4a9 0 3e
:3 0 5f :3 0 4ab
4ac 0 e :3 0
45 :3 0 284 4aa
4b0 :2 0 4b2 288
4b3 444 4b2 0
4b4 28c 0 5a6
69 :3 0 66 :3 0
46 :3 0 6f :3 0
6e :3 0 67 :3 0
e :3 0 d :3 0
11 :3 0 1b :3 0
14 :4 0 82 1
:8 0 4d6 83 :4 0
4c1 :3 0 45 :3 0
57 :2 0 290 4c4
4c5 :3 0 9 :3 0
4b :3 0 4c7 4c8
0 18 :2 0 4c9
4ca 0 4cc 293
4d3 9 :3 0 4b
:3 0 4cd 4ce 0
84 :2 0 4cf 4d0
0 4d2 295 4d4
4c6 4cc 0 4d5
0 4d2 0 4d5
297 0 4d6 29a
529 4d :3 0 9
:3 0 4b :3 0 4d8
4d9 0 4c :3 0
4da 4db 0 4dd
29d 4df 29f 4de
4dd :2 0 527 4e
:3 0 9 :3 0 4b
:3 0 4e2 4e3 0
4c :3 0 4e4 4e5
0 51f 23 :3 0
24 :3 0 4e7 4e8
0 23 :3 0 31
:3 0 4ea 4eb 0
4f :2 0 2 :4 0
85 :2 0 33 :2 0
13 :2 0 2a1 4f0
4f2 :3 0 2a4 4ec
4f4 4e9 4f6 4f5
:2 0 26 :3 0 34
:3 0 4f8 4f9 0
28 :4 0 86 :4 0
2b :2 0 2c :3 0
2a8 4fd 4ff :3 0
2b :2 0 3a :4 0
2ab 501 503 :3 0
2b :2 0 52 :4 0
2ae 505 507 :3 0
2b :2 0 4c :3 0
2b1 509 50b :3 0
2b :2 0 53 :4 0
2b4 50d 50f :3 0
2b :2 0 54 :3 0
55 :3 0 13 :2 0
56 :2 0 2b7 512
516 2bb 511 518
:3 0 2be 4fa 51a
:2 0 51c 2c1 51d
4f7 51c 0 51e
2c3 0 51f 2c5
522 :3 0 522 0
522 521 51f 520
:6 0 524 c :3 0
2c8 526 2ca 525
524 :2 0 527 2cc
:2 0 529 0 529
528 4d6 527 :6 0
5a6 1 :3 0 9
:3 0 4b :3 0 52b
52c 0 9 :3 0
87 :2 0 88 :3 0
52e 530 0 2d1
52f 532 :3 0 533
:2 0 23 :3 0 24
:3 0 535 536 0
23 :3 0 31 :3 0
538 539 0 c
:2 0 2 :4 0 89
:2 0 33 :2 0 13
:2 0 2d4 53e 540
:3 0 2d7 53a 542
537 544 543 :2 0
26 :3 0 34 :3 0
546 547 0 28
:4 0 8a :4 0 2b
:2 0 2c :3 0 2db
54b 54d :3 0 2b
:2 0 3a :4 0 2de
54f 551 :3 0 2b
:2 0 3b :4 0 2e1
553 555 :3 0 2b
:2 0 3c :3 0 3d
:3 0 9 :3 0 4b
:3 0 55a 55b 0
2e4 559 55d 40
:4 0 2e6 558 560
2e9 557 562 :3 0
2b :2 0 60 :4 0
2ec 564 566 :3 0
2b :2 0 2e :4 0
2ef 568 56a :3 0
2b :2 0 5 :3 0
2f2 56c 56e :3 0
2b :2 0 2e :4 0
2f5 570 572 :3 0
2f8 548 574 :2 0
576 2fb 577 545
576 0 578 2fd
0 5a2 23 :3 0
24 :3 0 579 57a
0 23 :3 0 31
:3 0 57c 57d 0
13 :2 0 2 :4 0
8b :2 0 33 :2 0
13 :2 0 2ff 582
584 :3 0 302 57e
586 57b 588 587
:2 0 26 :3 0 34
:3 0 58a 58b 0
28 :4 0 8c :4 0
2b :2 0 2c :3 0
306 58f 591 :3 0
309 58c 593 :2 0
595 30c 596 589
595 0 597 30e
0 5a2 43 :3 0
63 :3 0 598 599
0 9 :3 0 4b
:3 0 59b 59c 0
5 :3 0 45 :3 0
310 59a 5a0 :2 0
5a2 314 5a3 534
5a2 0 5a4 318
0 5a6 8d :3 0
31a 78e 9 :3 0
4b :3 0 5a7 5a8
0 9 :3 0 57
:2 0 88 :3 0 5aa
5ac 0 324 5ab
5ae :3 0 5af :2 0
8e :3 0 3c :3 0
16 :3 0 18 :2 0
327 5b2 5b5 32a
5b1 5b7 87 :2 0
18 :2 0 32e 5b9
5bb :3 0 8f :3 0
90 :3 0 b :3 0
91 :3 0 8f :3 0
8f :3 0 49 :3 0
3 :3 0 8f :3 0
92 :3 0 5 :3 0
8f :3 0 93 :4 0
94 1 :8 0 5d0
9 :3 0 4b :3 0
5cb 5cc 0 4c
:3 0 5cd 5ce 0
5d0 331 623 4d
:3 0 9 :3 0 4b
:3 0 5d2 5d3 0
4c :3 0 5d4 5d5
0 5d7 334 5d9
336 5d8 5d7 :2 0
621 4e :3 0 9
:3 0 4b :3 0 5dc
5dd 0 4c :3 0
5de 5df 0 619
23 :3 0 24 :3 0
5e1 5e2 0 23
:3 0 31 :3 0 5e4
5e5 0 4f :2 0
2 :4 0 95 :2 0
33 :2 0 13 :2 0
338 5ea 5ec :3 0
33b 5e6 5ee 5e3
5f0 5ef :2 0 26
:3 0 34 :3 0 5f2
5f3 0 28 :4 0
96 :4 0 2b :2 0
2c :3 0 33f 5f7
5f9 :3 0 2b :2 0
3a :4 0 342 5fb
5fd :3 0 2b :2 0
52 :4 0 345 5ff
601 :3 0 2b :2 0
4c :3 0 348 603
605 :3 0 2b :2 0
53 :4 0 34b 607
609 :3 0 2b :2 0
54 :3 0 55 :3 0
13 :2 0 56 :2 0
34e 60c 610 352
60b 612 :3 0 355
5f4 614 :2 0 616
358 617 5f1 616
0 618 35a 0
619 35c 61c :3 0
61c 0 61c 61b
619 61a :6 0 61e
e :3 0 35f 620
361 61f 61e :2 0
621 363 :2 0 623
0 623 622 5d0
621 :6 0 69f 1
:3 0 9 :3 0 4b
:3 0 625 626 0
9 :3 0 57 :2 0
88 :3 0 628 62a
0 368 629 62c
:3 0 62d :2 0 23
:3 0 24 :3 0 62f
630 0 23 :3 0
31 :3 0 632 633
0 c :2 0 2
:4 0 97 :2 0 33
:2 0 13 :2 0 36b
638 63a :3 0 36e
634 63c 631 63e
63d :2 0 26 :3 0
34 :3 0 640 641
0 28 :4 0 98
:4 0 2b :2 0 2c
:3 0 372 645 647
:3 0 2b :2 0 3a
:4 0 375 649 64b
:3 0 2b :2 0 3b
:4 0 378 64d 64f
:3 0 2b :2 0 3c
:3 0 3d :3 0 3e
:3 0 99 :3 0 654
655 0 37b 653
657 40 :4 0 37d
652 65a 380 651
65c :3 0 2b :2 0
60 :4 0 383 65e
660 :3 0 2b :2 0
2e :4 0 386 662
664 :3 0 2b :2 0
5 :3 0 389 666
668 :3 0 2b :2 0
2e :4 0 38c 66a
66c :3 0 38f 642
66e :2 0 670 392
671 63f 670 0
672 394 0 69c
23 :3 0 24 :3 0
673 674 0 23
:3 0 31 :3 0 676
677 0 13 :2 0
2 :4 0 9a :2 0
33 :2 0 13 :2 0
396 67c 67e :3 0
399 678 680 675
682 681 :2 0 26
:3 0 34 :3 0 684
685 0 28 :4 0
9b :4 0 2b :2 0
2c :3 0 39d 689
68b :3 0 3a0 686
68d :2 0 68f 3a3
690 683 68f 0
691 3a5 0 69c
43 :3 0 63 :3 0
692 693 0 3e
:3 0 99 :3 0 695
696 0 5 :3 0
45 :3 0 3a7 694
69a :2 0 69c 3ab
69d 62e 69c 0
69e 3af 0 69f
3b1 6a0 5bc 69f
0 6a1 3b4 0
71e 54 :3 0 12
:3 0 13 :2 0 13
:2 0 3b6 6a2 6a6
57 :2 0 9c :4 0
3bc 6a8 6aa :3 0
69 :3 0 68 :3 0
66 :3 0 e :4 0
9d 1 :8 0 6c6
83 :4 0 6b1 :3 0
45 :3 0 57 :2 0
3c1 6b4 6b5 :3 0
9 :3 0 4b :3 0
6b7 6b8 0 18
:2 0 6b9 6ba 0
6bc 3c4 6c3 9
:3 0 4b :3 0 6bd
6be 0 84 :2 0
6bf 6c0 0 6c2
3c6 6c4 6b6 6bc
0 6c5 0 6c2
0 6c5 3c8 0
6c6 3cb 719 4d
:3 0 9 :3 0 4b
:3 0 6c8 6c9 0
4c :3 0 6ca 6cb
0 6cd 3ce 6cf
3d0 6ce 6cd :2 0
717 4e :3 0 9
:3 0 4b :3 0 6d2
6d3 0 4c :3 0
6d4 6d5 0 70f
23 :3 0 24 :3 0
6d7 6d8 0 23
:3 0 31 :3 0 6da
6db 0 4f :2 0
2 :4 0 9e :2 0
33 :2 0 13 :2 0
3d2 6e0 6e2 :3 0
3d5 6dc 6e4 6d9
6e6 6e5 :2 0 26
:3 0 34 :3 0 6e8
6e9 0 28 :4 0
9f :4 0 2b :2 0
2c :3 0 3d9 6ed
6ef :3 0 2b :2 0
3a :4 0 3dc 6f1
6f3 :3 0 2b :2 0
52 :4 0 3df 6f5
6f7 :3 0 2b :2 0
4c :3 0 3e2 6f9
6fb :3 0 2b :2 0
53 :4 0 3e5 6fd
6ff :3 0 2b :2 0
54 :3 0 55 :3 0
13 :2 0 56 :2 0
3e8 702 706 3ec
701 708 :3 0 3ef
6ea 70a :2 0 70c
3f2 70d 6e7 70c
0 70e 3f4 0
70f 3f6 712 :3 0
712 0 712 711
70f 710 :6 0 714
10 :3 0 3f9 716
3fb 715 714 :2 0
717 3fd :2 0 719
0 719 718 6c6
717 :6 0 71b 1
:3 0 400 71c 6ab
71b 0 71d 402
0 71e 404 71f
5b0 71e 0 790
23 :3 0 24 :3 0
720 721 0 23
:3 0 31 :3 0 723
724 0 c :2 0
2 :4 0 a0 :2 0
33 :2 0 13 :2 0
407 729 72b :3 0
40a 725 72d 722
72f 72e :2 0 26
:3 0 34 :3 0 731
732 0 28 :4 0
a1 :4 0 2b :2 0
2c :3 0 40e 736
738 :3 0 2b :2 0
3a :4 0 411 73a
73c :3 0 2b :2 0
3b :4 0 414 73e
740 :3 0 2b :2 0
3c :3 0 3d :3 0
3e :3 0 5f :3 0
745 746 0 417
744 748 40 :4 0
419 743 74b 41c
742 74d :3 0 2b
:2 0 60 :4 0 41f
74f 751 :3 0 2b
:2 0 2e :4 0 422
753 755 :3 0 2b
:2 0 5 :3 0 425
757 759 :3 0 2b
:2 0 2e :4 0 428
75b 75d :3 0 42b
733 75f :2 0 761
42e 762 730 761
0 763 430 0
78d 23 :3 0 24
:3 0 764 765 0
23 :3 0 31 :3 0
767 768 0 13
:2 0 2 :4 0 a2
:2 0 33 :2 0 13
:2 0 432 76d 76f
:3 0 435 769 771
766 773 772 :2 0
26 :3 0 34 :3 0
775 776 0 28
:4 0 a3 :4 0 2b
:2 0 2c :3 0 439
77a 77c :3 0 43c
777 77e :2 0 780
43f 781 774 780
0 782 441 0
78d 43 :3 0 63
:3 0 783 784 0
3e :3 0 5f :3 0
786 787 0 5
:3 0 45 :3 0 443
785 78b :2 0 78d
447 78f 351 5a6
0 790 0 78d
0 790 44b 0
aab 1a :3 0 18
:2 0 791 792 0
aab 90 :3 0 b
:3 0 91 :3 0 49
:3 0 3 :3 0 92
:3 0 e :3 0 93
:4 0 a4 1 :8 0
7a2 9 :3 0 4b
:3 0 79d 79e 0
4c :3 0 79f 7a0
0 7a2 44f 7f5
4d :3 0 9 :3 0
4b :3 0 7a4 7a5
0 4c :3 0 7a6
7a7 0 7a9 452
7ab 454 7aa 7a9
:2 0 7f3 4e :3 0
9 :3 0 4b :3 0
7ae 7af 0 4c
:3 0 7b0 7b1 0
7eb 23 :3 0 24
:3 0 7b3 7b4 0
23 :3 0 31 :3 0
7b6 7b7 0 4f
:2 0 2 :4 0 a5
:2 0 33 :2 0 13
:2 0 456 7bc 7be
:3 0 459 7b8 7c0
7b5 7c2 7c1 :2 0
26 :3 0 34 :3 0
7c4 7c5 0 28
:4 0 a6 :4 0 2b
:2 0 2c :3 0 45d
7c9 7cb :3 0 2b
:2 0 3a :4 0 460
7cd 7cf :3 0 2b
:2 0 52 :4 0 463
7d1 7d3 :3 0 2b
:2 0 4c :3 0 466
7d5 7d7 :3 0 2b
:2 0 53 :4 0 469
7d9 7db :3 0 2b
:2 0 54 :3 0 55
:3 0 13 :2 0 56
:2 0 46c 7de 7e2
470 7dd 7e4 :3 0
473 7c6 7e6 :2 0
7e8 476 7e9 7c3
7e8 0 7ea 478
0 7eb 47a 7ee
:3 0 7ee 0 7ee
7ed 7eb 7ec :6 0
7f0 12 :3 0 47d
7f2 47f 7f1 7f0
:2 0 7f3 481 :2 0
7f5 0 7f5 7f4
7a2 7f3 :6 0 aab
1 :3 0 9 :3 0
4b :3 0 7f7 7f8
0 9 :3 0 57
:2 0 58 :3 0 7fa
7fc 0 486 7fb
7fe :3 0 7ff :2 0
3c :3 0 19 :3 0
18 :2 0 489 801
804 57 :2 0 18
:2 0 48e 806 808
:3 0 48 :3 0 a7
:3 0 a8 :3 0 a7
:3 0 14 :3 0 a7
:3 0 a9 :3 0 14
:3 0 a9 :3 0 a7
:3 0 a7 :3 0 a7
:3 0 14 :3 0 49
:3 0 3 :4 0 aa
1 :8 0 82f 83
:4 0 81a :3 0 45
:3 0 57 :2 0 493
81d 81e :3 0 9
:3 0 4b :3 0 820
821 0 18 :2 0
822 823 0 825
496 82c 9 :3 0
4b :3 0 826 827
0 84 :2 0 828
829 0 82b 498
82d 81f 825 0
82e 0 82b 0
82e 49a 0 82f
49d 882 4d :3 0
9 :3 0 4b :3 0
831 832 0 4c
:3 0 833 834 0
836 4a0 838 4a2
837 836 :2 0 880
4e :3 0 9 :3 0
4b :3 0 83b 83c
0 4c :3 0 83d
83e 0 878 23
:3 0 24 :3 0 840
841 0 23 :3 0
31 :3 0 843 844
0 4f :2 0 2
:4 0 ab :2 0 33
:2 0 13 :2 0 4a4
849 84b :3 0 4a7
845 84d 842 84f
84e :2 0 26 :3 0
34 :3 0 851 852
0 28 :4 0 ac
:4 0 2b :2 0 2c
:3 0 4ab 856 858
:3 0 2b :2 0 3a
:4 0 4ae 85a 85c
:3 0 2b :2 0 52
:4 0 4b1 85e 860
:3 0 2b :2 0 4c
:3 0 4b4 862 864
:3 0 2b :2 0 53
:4 0 4b7 866 868
:3 0 2b :2 0 54
:3 0 55 :3 0 13
:2 0 56 :2 0 4ba
86b 86f 4be 86a
871 :3 0 4c1 853
873 :2 0 875 4c4
876 850 875 0
877 4c6 0 878
4c8 87b :3 0 87b
0 87b 87a 878
879 :6 0 87d 14
:3 0 4cb 87f 4cd
87e 87d :2 0 880
4cf :2 0 882 0
882 881 82f 880
:6 0 885 1 :3 0
8d :3 0 4d2 90b
3c :3 0 19 :3 0
18 :2 0 4d4 886
889 57 :2 0 13
:2 0 4d9 88b 88d
:3 0 59 :3 0 a7
:3 0 a8 :3 0 a7
:3 0 14 :3 0 a7
:3 0 a9 :3 0 14
:3 0 a9 :3 0 a7
:3 0 a7 :3 0 a7
:3 0 14 :3 0 49
:3 0 3 :4 0 ad
1 :8 0 8b4 83
:4 0 89f :3 0 45
:3 0 57 :2 0 4de
8a2 8a3 :3 0 9
:3 0 4b :3 0 8a5
8a6 0 18 :2 0
8a7 8a8 0 8aa
4e1 8b1 9 :3 0
4b :3 0 8ab 8ac
0 84 :2 0 8ad
8ae 0 8b0 4e3
8b2 8a4 8aa 0
8b3 0 8b0 0
8b3 4e5 0 8b4
4e8 907 4d :3 0
9 :3 0 4b :3 0
8b6 8b7 0 4c
:3 0 8b8 8b9 0
8bb 4eb 8bd 4ed
8bc 8bb :2 0 905
4e :3 0 9 :3 0
4b :3 0 8c0 8c1
0 4c :3 0 8c2
8c3 0 8fd 23
:3 0 24 :3 0 8c5
8c6 0 23 :3 0
31 :3 0 8c8 8c9
0 4f :2 0 2
:4 0 ae :2 0 33
:2 0 13 :2 0 4ef
8ce 8d0 :3 0 4f2
8ca 8d2 8c7 8d4
8d3 :2 0 26 :3 0
34 :3 0 8d6 8d7
0 28 :4 0 af
:4 0 2b :2 0 2c
:3 0 4f6 8db 8dd
:3 0 2b :2 0 3a
:4 0 4f9 8df 8e1
:3 0 2b :2 0 52
:4 0 4fc 8e3 8e5
:3 0 2b :2 0 4c
:3 0 4ff 8e7 8e9
:3 0 2b :2 0 53
:4 0 502 8eb 8ed
:3 0 2b :2 0 54
:3 0 55 :3 0 13
:2 0 56 :2 0 505
8f0 8f4 509 8ef
8f6 :3 0 50c 8d8
8f8 :2 0 8fa 50f
8fb 8d5 8fa 0
8fc 511 0 8fd
513 900 :3 0 900
0 900 8ff 8fd
8fe :6 0 902 16
:3 0 516 904 518
903 902 :2 0 905
51a :2 0 907 0
907 906 8b4 905
:6 0 909 1 :3 0
51d 90a 88e 909
0 90c 809 885
0 90c 51f 0
9d4 b :3 0 b0
:4 0 90e 90f :3 0
90d 910 0 9d4
20 :3 0 b1 :3 0
3c :3 0 21 :3 0
18 :2 0 522 914
917 3c :3 0 16
:3 0 18 :2 0 525
919 91c 3e :3 0
b2 :3 0 91e 91f
0 1d :3 0 528
913 922 912 923
0 9d4 b3 :3 0
3c :3 0 16 :3 0
18 :2 0 52d 926
929 530 925 92b
87 :2 0 18 :2 0
534 92d 92f :3 0
1c :3 0 3e :3 0
b4 :3 0 932 933
0 931 934 0
936 537 93d 1c
:3 0 3e :3 0 b5
:3 0 938 939 0
937 93a 0 93c
539 93e 930 936
0 93f 0 93c
0 93f 53b 0
9d4 1f :3 0 13
:4 0 940 941 0
9d4 1e :3 0 13
:2 0 943 944 0
9d4 20 :3 0 b6
:3 0 d :3 0 e
:3 0 16 :3 0 1c
:3 0 1d :3 0 1f
:3 0 1e :3 0 53e
947 94f 946 950
0 9d4 91 :3 0
90 :3 0 46 :3 0
49 :3 0 b7 :3 0
93 :3 0 92 :3 0
b8 :3 0 b9 :3 0
ba :3 0 bb :3 0
bc :3 0 b :3 0
d :3 0 3 :3 0
16 :3 0 e :3 0
1c :3 0 1d :3 0
7 :3 0 2c :3 0
21 :4 0 bd 1
:8 0 97e 83 :4 0
969 :3 0 45 :3 0
57 :2 0 548 96c
96d :3 0 9 :3 0
4b :3 0 96f 970
0 18 :2 0 971
972 0 974 54b
97b 9 :3 0 4b
:3 0 975 976 0
84 :2 0 977 978
0 97a 54d 97c
96e 974 0 97d
0 97a 0 97d
54f 0 97e 552
9d1 4d :3 0 9
:3 0 4b :3 0 980
981 0 4c :3 0
982 983 0 985
555 987 557 986
985 :2 0 9cf 4e
:3 0 9 :3 0 4b
:3 0 98a 98b 0
4c :3 0 98c 98d
0 9c7 23 :3 0
24 :3 0 98f 990
0 23 :3 0 31
:3 0 992 993 0
4f :2 0 2 :4 0
be :2 0 33 :2 0
13 :2 0 559 998
99a :3 0 55c 994
99c 991 99e 99d
:2 0 26 :3 0 34
:3 0 9a0 9a1 0
28 :4 0 bf :4 0
2b :2 0 2c :3 0
560 9a5 9a7 :3 0
2b :2 0 3a :4 0
563 9a9 9ab :3 0
2b :2 0 52 :4 0
566 9ad 9af :3 0
2b :2 0 4c :3 0
569 9b1 9b3 :3 0
2b :2 0 53 :4 0
56c 9b5 9b7 :3 0
2b :2 0 54 :3 0
55 :3 0 13 :2 0
56 :2 0 56f 9ba
9be 573 9b9 9c0
:3 0 576 9a2 9c2
:2 0 9c4 579 9c5
99f 9c4 0 9c6
57b 0 9c7 57d
9ca :3 0 9ca 0
9ca 9c9 9c7 9c8
:6 0 9cc 18 :3 0
580 9ce 582 9cd
9cc :2 0 9cf 584
:2 0 9d1 0 9d1
9d0 97e 9cf :6 0
9d4 1 :3 0 8d
:3 0 587 a5b 9
:3 0 4b :3 0 9d5
9d6 0 9 :3 0
57 :2 0 88 :3 0
9d8 9da 0 592
9d9 9dc :3 0 9dd
:2 0 91 :3 0 b8
:3 0 c0 :3 0 c1
:3 0 b8 :3 0 3e
:3 0 b5 :3 0 90
:3 0 b :4 0 c2
1 :8 0 9fe 83
:4 0 9e9 :3 0 45
:3 0 57 :2 0 597
9ec 9ed :3 0 9
:3 0 4b :3 0 9ef
9f0 0 18 :2 0
9f1 9f2 0 9f4
59a 9fb 9 :3 0
4b :3 0 9f5 9f6
0 84 :2 0 9f7
9f8 0 9fa 59c
9fc 9ee 9f4 0
9fd 0 9fa 0
9fd 59e 0 9fe
5a1 a51 4d :3 0
9 :3 0 4b :3 0
a00 a01 0 4c
:3 0 a02 a03 0
a05 5a4 a07 5a6
a06 a05 :2 0 a4f
4e :3 0 9 :3 0
4b :3 0 a0a a0b
0 4c :3 0 a0c
a0d 0 a47 23
:3 0 24 :3 0 a0f
a10 0 23 :3 0
31 :3 0 a12 a13
0 4f :2 0 2
:4 0 c3 :2 0 33
:2 0 13 :2 0 5a8
a18 a1a :3 0 5ab
a14 a1c a11 a1e
a1d :2 0 26 :3 0
34 :3 0 a20 a21
0 28 :4 0 c4
:4 0 2b :2 0 2c
:3 0 5af a25 a27
:3 0 2b :2 0 3a
:4 0 5b2 a29 a2b
:3 0 2b :2 0 52
:4 0 5b5 a2d a2f
:3 0 2b :2 0 4c
:3 0 5b8 a31 a33
:3 0 2b :2 0 53
:4 0 5bb a35 a37
:3 0 2b :2 0 54
:3 0 55 :3 0 13
:2 0 56 :2 0 5be
a3a a3e 5c2 a39
a40 :3 0 5c5 a22
a42 :2 0 a44 5c8
a45 a1f a44 0
a46 5ca 0 a47
5cc a4a :3 0 a4a
0 a4a a49 a47
a48 :6 0 a4c 1a
:3 0 5cf a4e 5d1
a4d a4c :2 0 a4f
5d3 :2 0 a51 0
a51 a50 9fe a4f
:6 0 a59 1 :3 0
20 :3 0 c5 :3 0
b :3 0 5d6 a54
a56 a53 a57 0
a59 5d8 a5a 9de
a59 0 a5c 800
9d4 0 a5c 5db
0 aab 23 :3 0
24 :3 0 a5d a5e
0 23 :3 0 31
:3 0 a60 a61 0
c6 :2 0 2 :4 0
c7 :2 0 33 :2 0
13 :2 0 5de a66
a68 :3 0 5e1 a62
a6a a5f a6c a6b
:2 0 26 :3 0 34
:3 0 a6e a6f 0
28 :4 0 c8 :4 0
2b :2 0 2c :3 0
5e5 a73 a75 :3 0
2b :2 0 3a :4 0
5e8 a77 a79 :3 0
2b :2 0 3b :4 0
5eb a7b a7d :3 0
2b :2 0 3c :3 0
3d :3 0 9 :3 0
4b :3 0 a82 a83
0 5ee a81 a85
40 :4 0 5f0 a80
a88 5f3 a7f a8a
:3 0 2b :2 0 60
:4 0 5f6 a8c a8e
:3 0 2b :2 0 2e
:4 0 5f9 a90 a92
:3 0 2b :2 0 b
:3 0 5fc a94 a96
:3 0 2b :2 0 2e
:4 0 5ff a98 a9a
:3 0 602 a70 a9c
:2 0 a9e 605 a9f
a6d a9e 0 aa0
607 0 aab 43
:3 0 c9 :3 0 aa1
aa2 0 9 :3 0
4b :3 0 aa4 aa5
0 b :3 0 45
:3 0 609 aa3 aa9
:2 0 aab 60d b4c
9 :3 0 ca :2 0
4 aac aad 0
23 :3 0 24 :3 0
aaf ab0 0 23
:3 0 31 :3 0 ab2
ab3 0 4f :2 0
2 :4 0 cb :2 0
33 :2 0 13 :2 0
61b ab8 aba :3 0
61e ab4 abc ab1
abe abd :2 0 26
:3 0 34 :3 0 ac0
ac1 0 28 :4 0
cc :4 0 2b :2 0
2c :3 0 622 ac5
ac7 :3 0 625 ac2
ac9 :2 0 ae2 26
:3 0 34 :3 0 acb
acc 0 28 :4 0
cd :4 0 2b :2 0
9 :3 0 ce :3 0
ad1 ad2 0 628
ad0 ad4 :3 0 2b
:2 0 60 :4 0 62b
ad6 ad8 :3 0 2b
:2 0 9 :3 0 cf
:3 0 adb adc 0
62e ada ade :3 0
631 acd ae0 :2 0
ae2 634 ae3 abf
ae2 0 ae4 637
0 af4 d0 :3 0
22 :3 0 9 :3 0
ce :3 0 d1 :3 0
9 :3 0 cf :3 0
d2 :3 0 d3 :4 0
d4 1 :a 0 af0
:2 0 af4 ae6 aee
:2 0 8 :3 0 22
:3 0 af2 :2 0 af4
639 af6 63d af5
af4 :2 0 b4a 4e
:3 0 9 :3 0 4b
:3 0 af9 afa 0
4c :3 0 afb afc
0 b42 23 :3 0
24 :3 0 afe aff
0 23 :3 0 31
:3 0 b01 b02 0
4f :2 0 2 :4 0
d5 :2 0 33 :2 0
13 :2 0 63f b07
b09 :3 0 642 b03
b0b b00 b0d b0c
:2 0 26 :3 0 34
:3 0 b0f b10 0
28 :4 0 d6 :4 0
2b :2 0 2c :3 0
646 b14 b16 :3 0
649 b11 b18 :2 0
b32 26 :3 0 34
:3 0 b1a b1b 0
28 :4 0 52 :4 0
2b :2 0 4c :3 0
64c b1f b21 :3 0
2b :2 0 53 :4 0
64f b23 b25 :3 0
2b :2 0 54 :3 0
55 :3 0 13 :2 0
56 :2 0 652 b28
b2c 656 b27 b2e
:3 0 659 b1c b30
:2 0 b32 65c b33
b0e b32 0 b34
65f 0 b42 d0
:3 0 22 :3 0 d1
:3 0 9 :3 0 cf
:3 0 d2 :3 0 d3
:4 0 d7 1 :a 0
b3e :2 0 b42 b36
b3c :2 0 8 :3 0
22 :3 0 b40 :2 0
b42 661 b45 :3 0
b45 0 b45 b44
b42 b43 :6 0 b47
1 :3 0 666 b49
668 b48 b47 :2 0
b4a 66a :2 0 b4c
66d b4c b4b aab
b4a :6 0 b4d :2 0
2 18 b4c b50
:3 0 b4f b4d b51
:8 0 
680
4
:3 0 1 3 1
7 1 b 1
f 4 6 a
e 12 1 1c
1 1a 1 23
1 21 1 2a
1 28 1 31
1 2f 1 38
1 36 1 3f
1 3d 1 46
1 44 1 4b
1 51 1 57
1 5d 1 63
1 6b 1 69
1 70 1 78
1 76 1 7d
1 83 1 89
1 96 1 a2
2 a5 a7 2
aa ac 2 ae
b0 2 b2 b4
2 b6 b8 2
ba bc 2 be
c0 2 c2 c4
2 c6 c8 2
ca cc 2 ce
d0 2 d2 d4
2 d6 d8 2
da dc 2 de
e0 2 e2 e4
2 e6 e8 4
9e a4 a9 ea
1 ec 1 ee
2 f8 fa 3
f6 f7 fc 2
105 107 2 104
109 1 10b 1
10d 1 10f 1
115 1 11d 2
12d 12f 3 12b
12c 131 2 13a
13c 2 13e 140
2 142 144 1
14c 2 14e 14f
2 146 151 2
139 153 1 155
1 157 2 161
163 3 15f 160
165 2 16e 170
2 16d 172 1
174 1 176 2
17d 17e 3 158
177 180 1 182
2 18b 190 1
197 1 192 2
1aa 1ac 3 1a8
1a9 1ae 2 1b7
1b9 2 1bb 1bd
2 1bf 1c1 2
1c3 1c5 2 1c7
1c9 3 1ce 1cf
1d0 2 1cb 1d2
2 1b6 1d4 1
1d6 1 1d8 2
1a1 1d9 1 1dd
1 19c 2 19a
1e1 1 1ec 2
1e8 1ec 2 1f7
1fc 1 203 1
1fe 2 216 218
3 214 215 21a
2 223 225 2
227 229 2 22b
22d 2 22f 231
2 233 235 3
23a 23b 23c 2
237 23e 2 222
240 1 242 1
244 2 20d 245
1 249 1 208
2 206 24d 1
258 2 254 258
2 264 266 3
262 263 268 2
271 273 2 275
277 2 279 27b
1 283 2 285
286 2 27d 288
2 28a 28c 2
28e 290 2 292
294 2 296 298
2 270 29a 1
29c 1 29e 2
2a8 2aa 3 2a6
2a7 2ac 2 2b5
2b7 2 2b4 2b9
1 2bb 1 2bd
3 2c4 2c5 2c6
3 29f 2be 2c8
1 2ca 4 250
2cb 2ce 2d3 2
2d7 2dc 2 2de
2df 2 2ed 2f2
1 2f9 1 2f4
2 30c 30e 3
30a 30b 310 2
319 31b 2 31d
31f 2 321 323
2 325 327 2
329 32b 3 330
331 332 2 32d
334 2 318 336
1 338 1 33a
2 303 33b 1
33f 1 2fe 2
2fc 343 1 34e
2 34a 34e 3
357 358 359 2
366 36b 1 372
1 36d 2 385
387 3 383 384
389 2 392 394
2 396 398 2
39a 39c 2 39e
3a0 2 3a2 3a4
3 3a9 3aa 3ab
2 3a6 3ad 2
391 3af 1 3b1
1 3b3 2 37c
3b4 1 3b8 1
377 2 375 3bc
2 3e0 3e5 1
3ec 1 3e7 2
3ff 401 3 3fd
3fe 403 2 40c
40e 2 410 412
2 414 416 2
418 41a 2 41c
41e 3 423 424
425 2 420 427
2 40b 429 1
42b 1 42d 2
3f6 42e 1 432
1 3f1 2 3ef
436 1 441 2
43d 441 2 44d
44f 3 44b 44c
451 2 45a 45c
2 45e 460 2
462 464 1 46c
2 46e 46f 2
466 471 2 473
475 2 477 479
2 47b 47d 2
47f 481 2 459
483 1 485 1
487 2 491 493
3 48f 490 495
2 49e 4a0 2
49d 4a2 1 4a4
1 4a6 3 4ad
4ae 4af 3 488
4a7 4b1 1 4b3
1 4c3 2 4c2
4c3 1 4cb 1
4d1 2 4d3 4d4
2 4c0 4d5 1
4dc 1 4d7 2
4ef 4f1 3 4ed
4ee 4f3 2 4fc
4fe 2 500 502
2 504 506 2
508 50a 2 50c
50e 3 513 514
515 2 510 517
2 4fb 519 1
51b 1 51d 2
4e6 51e 1 522
1 4e1 2 4df
526 1 531 2
52d 531 2 53d
53f 3 53b 53c
541 2 54a 54c
2 54e 550 2
552 554 1 55c
2 55e 55f 2
556 561 2 563
565 2 567 569
2 56b 56d 2
56f 571 2 549
573 1 575 1
577 2 581 583
3 57f 580 585
2 58e 590 2
58d 592 1 594
1 596 3 59d
59e 59f 3 578
597 5a1 1 5a3
7 354 35c 3bf
439 4b4 529 5a4
1 5ad 2 5a9
5ad 2 5b3 5b4
1 5b6 1 5ba
2 5b8 5ba 2
5ca 5cf 1 5d6
1 5d1 2 5e9
5eb 3 5e7 5e8
5ed 2 5f6 5f8
2 5fa 5fc 2
5fe 600 2 602
604 2 606 608
3 60d 60e 60f
2 60a 611 2
5f5 613 1 615
1 617 2 5e0
618 1 61c 1
5db 2 5d9 620
1 62b 2 627
62b 2 637 639
3 635 636 63b
2 644 646 2
648 64a 2 64c
64e 1 656 2
658 659 2 650
65b 2 65d 65f
2 661 663 2
665 667 2 669
66b 2 643 66d
1 66f 1 671
2 67b 67d 3
679 67a 67f 2
688 68a 2 687
68c 1 68e 1
690 3 697 698
699 3 672 691
69b 1 69d 2
623 69e 1 6a0
3 6a3 6a4 6a5
1 6a9 2 6a7
6a9 1 6b3 2
6b2 6b3 1 6bb
1 6c1 2 6c3
6c4 2 6b0 6c5
1 6cc 1 6c7
2 6df 6e1 3
6dd 6de 6e3 2
6ec 6ee 2 6f0
6f2 2 6f4 6f6
2 6f8 6fa 2
6fc 6fe 3 703
704 705 2 700
707 2 6eb 709
1 70b 1 70d
2 6d6 70e 1
712 1 6d1 2
6cf 716 1 719
1 71c 2 6a1
71d 2 728 72a
3 726 727 72c
2 735 737 2
739 73b 2 73d
73f 1 747 2
749 74a 2 741
74c 2 74e 750
2 752 754 2
756 758 2 75a
75c 2 734 75e
1 760 1 762
2 76c 76e 3
76a 76b 770 2
779 77b 2 778
77d 1 77f 1
781 3 788 789
78a 3 763 782
78c 3 78e 71f
78f 2 79c 7a1
1 7a8 1 7a3
2 7bb 7bd 3
7b9 7ba 7bf 2
7c8 7ca 2 7cc
7ce 2 7d0 7d2
2 7d4 7d6 2
7d8 7da 3 7df
7e0 7e1 2 7dc
7e3 2 7c7 7e5
1 7e7 1 7e9
2 7b2 7ea 1
7ee 1 7ad 2
7ab 7f2 1 7fd
2 7f9 7fd 2
802 803 1 807
2 805 807 1
81c 2 81b 81c
1 824 1 82a
2 82c 82d 2
819 82e 1 835
1 830 2 848
84a 3 846 847
84c 2 855 857
2 859 85b 2
85d 85f 2 861
863 2 865 867
3 86c 86d 86e
2 869 870 2
854 872 1 874
1 876 2 83f
877 1 87b 1
83a 2 838 87f
1 882 2 887
888 1 88c 2
88a 88c 1 8a1
2 8a0 8a1 1
8a9 1 8af 2
8b1 8b2 2 89e
8b3 1 8ba 1
8b5 2 8cd 8cf
3 8cb 8cc 8d1
2 8da 8dc 2
8de 8e0 2 8e2
8e4 2 8e6 8e8
2 8ea 8ec 3
8f1 8f2 8f3 2
8ee 8f5 2 8d9
8f7 1 8f9 1
8fb 2 8c4 8fc
1 900 1 8bf
2 8bd 904 1
907 2 90b 90a
2 915 916 2
91a 91b 4 918
91d 920 921 2
927 928 1 92a
1 92e 2 92c
92e 1 935 1
93b 2 93d 93e
7 948 949 94a
94b 94c 94d 94e
1 96b 2 96a
96b 1 973 1
979 2 97b 97c
2 968 97d 1
984 1 97f 2
997 999 3 995
996 99b 2 9a4
9a6 2 9a8 9aa
2 9ac 9ae 2
9b0 9b2 2 9b4
9b6 3 9bb 9bc
9bd 2 9b8 9bf
2 9a3 9c1 1
9c3 1 9c5 2
98e 9c6 1 9ca
1 989 2 987
9ce 8 90c 911
924 93f 942 945
951 9d1 1 9db
2 9d7 9db 1
9eb 2 9ea 9eb
1 9f3 1 9f9
2 9fb 9fc 2
9e8 9fd 1 a04
1 9ff 2 a17
a19 3 a15 a16
a1b 2 a24 a26
2 a28 a2a 2
a2c a2e 2 a30
a32 2 a34 a36
3 a3b a3c a3d
2 a38 a3f 2
a23 a41 1 a43
1 a45 2 a0e
a46 1 a4a 1
a09 2 a07 a4e
1 a55 2 a51
a58 2 a5b a5a
2 a65 a67 3
a63 a64 a69 2
a72 a74 2 a76
a78 2 a7a a7c
1 a84 2 a86
a87 2 a7e a89
2 a8b a8d 2
a8f a91 2 a93
a95 2 a97 a99
2 a71 a9b 1
a9d 1 a9f 3
aa6 aa7 aa8 d
ef 10e 183 1e4
2e0 2e3 346 790
793 7f5 a5c aa0
aaa 2 ab7 ab9
3 ab5 ab6 abb
2 ac4 ac6 2
ac3 ac8 2 acf
ad3 2 ad5 ad7
2 ad9 add 2
ace adf 2 aca
ae1 1 ae3 3
ae4 aef af3 1
aae 2 b06 b08
3 b04 b05 b0a
2 b13 b15 2
b12 b17 2 b1e
b20 2 b22 b24
3 b29 b2a b2b
2 b26 b2d 2
b1d b2f 2 b19
b31 1 b33 4
afd b34 b3d b41
1 b45 1 af8
2 af6 b49 12
1f 26 2d 34
3b 42 49 4f
55 5b 61 67
6e 74 7b 81
87 8e 
1
4
0 
b50
0
1
28
1c
17
0 1 2 1 4 1 6 1
8 1 a 1 c 1 e 1
10 1 12 1 14 1 16 1
18 1 1a 1 0 0 0 0
0 0 0 0 0 0 0 0

f 1 0
63 1 0
51 1 0
69 1 0
7 1 0
36 1 0
76 1 0
7d 1 0
b 1 0
3d 1 0
5d 1 0
2f 1 0
1a 1 0
57 1 0
3 1 0
70 1 0
2 0 1
83 1 0
4b 1 0
28 1 0
21 1 0
44 1 0
89 1 0
0

/
SHOW ERRORS FUNCTION SP_A_TSKPROF;
ALTER FUNCTION SP_A_TSKPROF COMPILE DEBUG;
/***************************************************************************************************/
/*
#Stored Procedure  :SP_A_TSKRES
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_A_TSKRES ==
CREATE OR REPLACE FUNCTION SP_A_TSKRES wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
5e
2 :e:
1FUNCTION:
1SP_A_TSKRES:
1ID:
1VARCHAR2:
1RES_ID001:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_ELEMENT_ID:
132:
1H_RA_ID:
1H_PROJECT_ID:
1H_REC_USER:
1H_RESOURCE_ID:
1H_TABLE_TYPE:
1PLS_INTEGER:
10:
1H_TYPE_ID:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_A_TSKRES(:
1':
1, :
1);:
1CHECK_TRACE:
11:
125:
1+:
1FPUT:
1               ENTER:: SP_A_TSKRES(26) :: cplan.sql(6277) :: :
1IS NULL:
18:
130:
1      RETURN_INVALID:: SP_A_TSKRES(31) :: cplan.sql(6282) :: :
1 :
1STATE:: :
1COALESCE:
1TO_CHAR:
1PMO_CNST:
1INVALID_PARAMS:
1NULL:
133:
1                EXIT:: SP_A_TSKRES(34) :: cplan.sql(6285) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1SC_TABLE_WBS:
1TYPE_ID:
1PROJECT_ID:
1TMT_WBS:
1ELEMENT_ID:
1SELECT TYPE_ID, PROJECT_ID INTO h_type_id, h_project_id:n      FROM TMT_WBS W+
1HERE ELEMENT_ID = h_element_id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
156:
1           EXCEPTION:: SP_A_TSKRES(57) :: cplan.sql(6308) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SYS_GUID:
1ASSIGNRESOURCE:
14:
167:
1       RETURN_STATUS:: SP_A_TSKRES(68) :: cplan.sql(6319) :: :
1 ID:: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
174:
1           EXCEPTION:: SP_A_TSKRES(75) :: cplan.sql(6326) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
186:
1           EXCEPTION:: SP_A_TSKRES(87) :: cplan.sql(6338) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
2a7
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 :2 a0 6b 1c 81
b0 :2 a0 6b :2 a0 6b 6e a5 b
a 10 :2 a0 6b 6e :2 a0 6b 6e
a5 b 6e 7e a0 b4 2e 6e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
a0 7e b4 2e :2 5a a0 7e b4
2e :2 5a 52 10 a0 7e b4 2e
:2 5a 52 10 :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b a0
a5 57 b7 19 3c :2 a0 d :2 a0
d :2 a0 d :3 a0 6b d :7 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 b4 2e d :2 a0 6b :7 a0 a5
b d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 :2 a0
6b :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 :2 a0 6b :2 6e 7e :2 a0 6b b4
2e 7e 6e b4 2e 7e :2 a0 6b
b4 2e a5 57 b7 19 3c :9 a0
12a e9 dd :2 a0 65 b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c :7 a0 12a e9
dd :2 a0 65 b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 68 4f
1d 17 b5 
2a7
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 30
56 5a 5e 62 65 69 86 71
75 78 79 81 70 a3 91 6d
95 96 9e 90 c0 ae 8d b2
b3 bb ad dd cb aa cf d0
d8 ca fa e8 c7 ec ed f5
e7 116 105 109 e4 111 104 132
121 125 101 12d 120 152 13d 141
11d 145 14d 13c 159 15d 139 161
165 169 16c 171 172 1 174 179
17d 181 184 189 18d 191 194 199
19a 19c 1a1 1a4 1a8 1a9 1ae 1b3
1b6 1bb 1bc 1c1 1c4 1c8 1c9 1ce
1d1 1d6 1d7 1dc 1df 1e4 1e5 1ea
1ed 1f2 1f3 1f8 1fb 1ff 200 205
208 20d 20e 213 216 21b 21c 221
224 229 22a 22f 232 236 237 23c
23f 244 245 24a 24d 252 253 258
259 25e 260 264 267 26b 26f 272
276 27a 27d 280 285 288 28b 28e
28f 294 295 1 297 29c 2a0 2a4
2a7 2ac 2b1 2b4 2b8 2b9 2be 2bf
2c4 2c6 2ca 2cd 2d1 2d4 2d5 2da
2dd 2e0 2e4 2e7 2e8 2ed 2f0 1
2f3 2f8 2fc 2ff 300 305 308 1
30b 310 314 318 31b 31f 323 326
329 32e 331 334 337 338 33d 33e
1 340 345 349 34d 350 355 35a
35d 361 362 367 36a 36f 370 375
378 37d 37e 383 386 38a 38e 392
396 399 39a 39c 3a1 3a2 3a4 3a5
3aa 3ab 3b0 3b2 3b6 3b9 3bd 3c1
3c4 3c8 3cc 3cf 3d2 3d7 3da 3dd
3e0 3e1 3e6 3e7 1 3e9 3ee 3f2
3f6 3f9 3fe 403 406 40a 40b 410
411 416 418 41c 41f 423 427 42a
42e 432 435 439 43a 43f 441 445
448 44c 450 454 458 45c 460 464
468 46c 470 474 478 47b 47f 483
487 48b 48f 493 497 49b 4a7 4ab
4af 4b2 4b6 4ba 4bc 4c0 4c4 4c8
4cb 4cf 4d3 4d5 4d6 4db 1 4df
4e3 4e7 4ea 4ee 4f2 4f6 4fa 4fd
501 505 508 50b 510 513 516 519
51a 51f 520 1 522 527 52b 52f
532 537 53c 53f 543 544 549 54c
551 552 557 55a 55f 560 565 568
56c 56d 572 575 57a 57b 580 583
587 58b 58e 591 592 594 595 59a
59b 5a0 5a2 5a6 5a9 5ab 5af 5b1
5bd 5bf 5c1 5c2 5c7 5cb 5cd 5d9
5db 5df 5e3 5e4 5e9 5ed 5f1 5f5
5f8 5fc 600 604 608 60c 610 614
615 617 61b 61f 623 626 62a 62e
631 634 639 63c 63f 642 643 648
649 1 64b 650 654 658 65b 660
665 668 66c 66d 672 675 67a 67b
680 683 688 689 68e 691 695 699
69d 6a1 6a4 6a5 6a7 6ac 6ad 6af
6b0 6b5 6b8 6bd 6be 6c3 6c6 6cb
6cc 6d1 6d4 6d8 6d9 6de 6e1 6e6
6e7 6ec 6ed 6f2 6f4 6f8 6fb 6ff
703 706 70a 70e 711 715 719 71a
71f 721 725 729 72c 730 734 737
73b 73f 742 745 74a 74d 750 753
754 759 75a 1 75c 761 765 769
76c 771 776 779 77d 77e 783 784
789 78d 791 794 799 79e 7a1 7a5
7a9 7ac 7ad 7b2 7b5 7ba 7bb 7c0
7c3 7c7 7cb 7ce 7cf 7d4 7d5 7da
7dc 7e0 7e3 7e7 7eb 7ef 7f3 7f7
7fb 7ff 803 807 813 818 81c 820
824 828 82a 82b 830 1 834 838
83c 83f 843 847 84b 84f 852 856
85a 85d 860 865 868 86b 86e 86f
874 875 1 877 87c 880 884 887
88c 891 894 898 899 89e 89f 8a4
8a8 8ac 8af 8b4 8b9 8bc 8c0 8c1
8c6 8c9 8ce 8cf 8d4 8d7 8db 8df
8e2 8e5 8e6 8e8 8e9 8ee 8ef 8f4
8f6 8fa 8fd 901 905 909 90d 911
915 919 925 92a 92e 932 936 93a
93c 940 942 94e 950 952 953 958
95c 95e 96a 96e 970 971 97a 
2a7
2
0 1 a 6 9 :3 6 10 :3 6
12 :2 6 1 3 a 13 a :2 1
4 11 1a 19 :2 11 :2 4 c 15
14 :2 c :2 4 11 1a 19 :2 11 :2 4
f 18 17 :2 f :2 4 12 1b 1a
:2 12 :2 4 :2 11 20 11 :2 4 :2 e 1d
e :2 4 e 17 :3 e 4 7 :2 11
23 :2 2d 3a :2 23 :3 7 :2 d 19 25
:2 2f 43 :2 25 53 59 5c :2 53 6a
7f 8 :2 6a d 10 :2 6a 13 16
:2 6a b e :2 6a 13 16 :2 6a 1b
1e :2 6a 28 2b :2 6a b e :2 6a
13 16 :2 6a 1b 1e :2 6a 2a 2d
:2 6a 32 35 :2 6a :2 7 49 :2 4 7
:2 11 23 :2 2d 39 3c 4b 4d 4e
:2 4b :2 23 :3 7 :2 d 12 1e 5b 5e
:2 1e :2 7 51 :2 4 :4 9 8 7 :4 1b
1a 19 :2 7 :4 34 33 32 :2 7 a
:2 14 26 :2 30 3c 3f 4e 50 51
:2 4e :2 26 :3 a :2 10 15 21 5e 61
:2 21 6e 71 :2 21 75 78 :2 21 82
85 8e 96 :2 9f :2 8e b0 :2 85 :2 21
:2 a 54 :2 7 a :2 14 26 :2 30 3c
3f 4e 50 51 :2 4e :2 26 :3 a :2 10
15 21 5e 61 :2 21 :2 a 54 :3 7
:2 12 28 :2 31 41 :2 7 4a :3 4 14
:2 4 15 :2 4 12 :2 4 14 :2 1d 4
e 17 27 32 c 1a 27 :2 7
:2 10 1c 7 4 c a :2 13 1f
a 1a :2 7 :2 c a :2 13 1f a
d :2 17 29 :2 33 3f 43 52 54
55 :2 52 :2 29 :3 d :2 13 18 24 61
64 :2 24 71 74 :2 24 78 7b :2 24
87 8a :2 24 92 95 :2 24 a1 a4
ab b4 b7 :2 a4 :2 24 :2 d 58 :2 a
:2 7 :4 13 :2 7 4 :3 1 4 :3 f :2 4
:2 d 19 28 36 41 4f 5e 27
:2 19 4 7 :2 11 23 :2 2d 39 3c
4b 4d 4e :2 4b :2 23 :3 7 :2 d 12
1e 5b 5e :2 1e 6b 6e :2 1e 72
75 :2 1e 7f 82 8b 93 :2 9c :2 8b
a7 :2 82 :2 1e af b2 :2 1e ba bd
:2 1e c2 c5 :2 1e cd d0 :2 1e :2 7
51 :3 4 :2 f 21 :2 2a 34 3d :2 4
1 9 12 9 a :2 14 26 :2 30
3c 40 4f 51 52 :2 4f :2 26 :3 a
:2 10 15 21 5e 61 :2 21 :3 a :2 10
15 39 5c 5f :2 68 :2 39 76 79
:2 39 81 84 :2 8d :2 39 :2 a 55 :3 7
c 15 1e 2c a 13 1e f
e 7 e 7 e 7 29 :2 4
:2 9 7 :2 10 1c :2 7 :2 11 23 :2 2d
39 3d 4c 4e 4f :2 4c :2 23 :3 7
:2 d 12 1e 5b 5e :2 1e :3 7 :2 d
12 36 42 45 :2 36 4d 50 :2 36
5c 5f 66 6f 72 :2 5f :2 36 :2 7
52 :2 4 7 c 1b a 13 1e
f e 7 e 7 e 7 :2 4
:4 10 :2 4 :8 1 
2a7
4
0 :2 1 :4 3 :4 4
:4 5 2 :4 6 :2 1
:7 8 :7 9 :7 a :7 b
:7 c :6 d :6 e :7 10
:b 12 :11 13 14 :2 13
:2 14 :2 13 :2 14 :2 13
:2 15 :2 13 :2 15 :2 13
:2 15 :2 13 :2 15 :2 13
:2 16 :2 13 :2 16 :2 13
:2 16 :2 13 :2 16 :2 13
:2 16 :4 13 :3 12 :11 19
:b 1a :3 19 :16 1d :11 1e
:20 1f :3 1e :11 21 :b 22
:3 21 :9 24 :3 1d :3 29
:3 2a :3 2b :5 2c :4 2f
:3 30 2f :5 31 2e
33 :5 34 :3 33 :2 35
:5 37 :11 38 :24 39 :3 38
:2 36 :6 35 32 :3 11
:5 3e :9 40 41 :3 40
:11 43 :30 44 :3 43 :a 46
11 :3 49 :11 4a :b 4b
:17 4c :3 4a :2 4e :3 4f
:3 50 51 4f 4e
4f :3 52 :3 49 :2 53
:5 55 :11 56 :b 57 :18 58
:3 56 :2 5a 5b :3 5c
5d 5b 5a 5b
:3 5e :2 54 :6 53 48
:7 1 
97c
4
:3 0 1 :3 0 2
:a 0 2a2 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
:2 0 7 4 :3 0
5 :7 0 9 8
:3 0 4 :3 0 6
:7 0 d c :3 0
7 :3 0 8 :3 0
9 :2 0 4 11
12 0 f 13
0 2a2 2 14
:2 0 b :2 0 f
4 :3 0 b :2 0
d 17 19 :6 0
1c 1a 0 2a0
0 a :6 0 b
:2 0 13 4 :3 0
11 1e 20 :6 0
23 21 0 2a0
0 c :6 0 b
:2 0 17 4 :3 0
15 25 27 :6 0
2a 28 0 2a0
0 d :6 0 b
:2 0 1b 4 :3 0
19 2c 2e :6 0
31 2f 0 2a0
0 e :6 0 12
:2 0 1f 4 :3 0
1d 33 35 :6 0
38 36 0 2a0
0 f :6 0 12
:2 0 21 11 :3 0
3a :7 0 3e 3b
3c 2a0 0 10
:6 0 46 47 0
23 11 :3 0 40
:7 0 44 41 42
2a0 0 13 :6 0
4c 4d 0 25
8 :3 0 9 :2 0
4 48 :7 0 4b
49 0 2a0 0
14 :6 0 15 :3 0
16 :3 0 15 :3 0
17 :3 0 4f 50
0 2 :4 0 27
51 53 4e 55
54 :2 0 18 :3 0
19 :3 0 57 58
0 1a :4 0 15
:3 0 1b :3 0 5b
5c 0 2 :4 0
29 5d 5f 1c
:4 0 1d :2 0 1e
:3 0 2b 62 64
:3 0 1f :4 0 1d
:2 0 20 :4 0 2e
67 69 :3 0 1d
:2 0 3 :3 0 31
6b 6d :3 0 1d
:2 0 20 :4 0 34
6f 71 :3 0 1d
:2 0 21 :4 0 37
73 75 :3 0 1d
:2 0 20 :4 0 3a
77 79 :3 0 1d
:2 0 5 :3 0 3d
7b 7d :3 0 1d
:2 0 20 :4 0 40
7f 81 :3 0 1d
:2 0 21 :4 0 43
83 85 :3 0 1d
:2 0 20 :4 0 46
87 89 :3 0 1d
:2 0 6 :3 0 49
8b 8d :3 0 1d
:2 0 20 :4 0 4c
8f 91 :3 0 1d
:2 0 22 :4 0 4f
93 95 :3 0 52
59 97 :2 0 99
57 9a 56 99
0 9b 59 0
200 15 :3 0 16
:3 0 9c 9d 0
15 :3 0 23 :3 0
9f a0 0 24
:2 0 2 :4 0 25
:2 0 26 :2 0 24
:2 0 5b a5 a7
:3 0 5e a1 a9
9e ab aa :2 0
18 :3 0 27 :3 0
ad ae 0 1a
:4 0 28 :4 0 1d
:2 0 1e :3 0 62
b2 b4 :3 0 65
af b6 :2 0 b8
68 b9 ac b8
0 ba 6a 0
200 3 :3 0 29
:2 0 6c bc bd
:3 0 be :2 0 bf
:2 0 5 :3 0 29
:2 0 6e c2 c3
:3 0 c4 :2 0 c5
:2 0 c0 c7 c6
:2 0 6 :3 0 29
:2 0 70 ca cb
:3 0 cc :2 0 cd
:2 0 c8 cf ce
:2 0 15 :3 0 16
:3 0 d1 d2 0
15 :3 0 23 :3 0
d4 d5 0 2a
:2 0 2 :4 0 2b
:2 0 26 :2 0 24
:2 0 72 da dc
:3 0 75 d6 de
d3 e0 df :2 0
18 :3 0 27 :3 0
e2 e3 0 1a
:4 0 2c :4 0 1d
:2 0 1e :3 0 79
e7 e9 :3 0 1d
:2 0 2d :4 0 7c
eb ed :3 0 1d
:2 0 2e :4 0 7f
ef f1 :3 0 1d
:2 0 2f :3 0 30
:3 0 31 :3 0 32
:3 0 f6 f7 0
82 f5 f9 33
:4 0 84 f4 fc
87 f3 fe :3 0
8a e4 100 :2 0
102 8d 103 e1
102 0 104 8f
0 12d 15 :3 0
16 :3 0 105 106
0 15 :3 0 23
:3 0 108 109 0
24 :2 0 2 :4 0
34 :2 0 26 :2 0
24 :2 0 91 10e
110 :3 0 94 10a
112 107 114 113
:2 0 18 :3 0 27
:3 0 116 117 0
1a :4 0 35 :4 0
1d :2 0 1e :3 0
98 11b 11d :3 0
9b 118 11f :2 0
121 9e 122 115
121 0 123 a0
0 12d 36 :3 0
37 :3 0 124 125
0 31 :3 0 32
:3 0 127 128 0
38 :3 0 a2 126
12b :2 0 12d a5
12e d0 12d 0
12f a9 0 200
a :3 0 3 :3 0
130 131 0 200
f :3 0 5 :3 0
133 134 0 200
e :3 0 6 :3 0
136 137 0 200
10 :3 0 31 :3 0
39 :3 0 13a 13b
0 139 13c 0
200 3a :3 0 3b
:3 0 13 :3 0 d
:3 0 3c :3 0 3d
:3 0 a :4 0 3e
1 :8 0 14b 8
:3 0 3f :3 0 146
147 0 40 :3 0
148 149 0 14b
ab 19e 41 :3 0
8 :3 0 3f :3 0
14d 14e 0 40
:3 0 14f 150 0
152 ae 154 b0
153 152 :2 0 19c
42 :3 0 8 :3 0
3f :3 0 157 158
0 40 :3 0 159
15a 0 194 15
:3 0 16 :3 0 15c
15d 0 15 :3 0
23 :3 0 15f 160
0 43 :2 0 2
:4 0 44 :2 0 26
:2 0 24 :2 0 b2
165 167 :3 0 b5
161 169 15e 16b
16a :2 0 18 :3 0
27 :3 0 16d 16e
0 1a :4 0 45
:4 0 1d :2 0 1e
:3 0 b9 172 174
:3 0 1d :2 0 2d
:4 0 bc 176 178
:3 0 1d :2 0 46
:4 0 bf 17a 17c
:3 0 1d :2 0 40
:3 0 c2 17e 180
:3 0 1d :2 0 47
:4 0 c5 182 184
:3 0 1d :2 0 48
:3 0 49 :3 0 24
:2 0 4a :2 0 c8
187 18b cc 186
18d :3 0 cf 16f
18f :2 0 191 d2
192 16c 191 0
193 d4 0 194
d6 197 :3 0 197
0 197 196 194
195 :6 0 199 2
:3 0 d9 19b db
19a 199 :2 0 19c
dd :2 0 19e 0
19e 19d 14b 19c
:6 0 200 1 :3 0
c :3 0 4b :4 0
1a1 1a2 :3 0 1a0
1a3 0 200 8
:3 0 3f :3 0 1a5
1a6 0 4c :3 0
10 :3 0 13 :3 0
a :3 0 f :3 0
e :3 0 c :3 0
e0 1a8 1af 1a7
1b0 0 200 15
:3 0 16 :3 0 1b2
1b3 0 15 :3 0
23 :3 0 1b5 1b6
0 4d :2 0 2
:4 0 4e :2 0 26
:2 0 24 :2 0 e7
1bb 1bd :3 0 ea
1b7 1bf 1b4 1c1
1c0 :2 0 18 :3 0
27 :3 0 1c3 1c4
0 1a :4 0 4f
:4 0 1d :2 0 1e
:3 0 ee 1c8 1ca
:3 0 1d :2 0 2d
:4 0 f1 1cc 1ce
:3 0 1d :2 0 2e
:4 0 f4 1d0 1d2
:3 0 1d :2 0 2f
:3 0 30 :3 0 8
:3 0 3f :3 0 1d7
1d8 0 f7 1d6
1da 33 :4 0 f9
1d5 1dd fc 1d4
1df :3 0 1d :2 0
50 :4 0 ff 1e1
1e3 :3 0 1d :2 0
20 :4 0 102 1e5
1e7 :3 0 1d :2 0
c :3 0 105 1e9
1eb :3 0 1d :2 0
20 :4 0 108 1ed
1ef :3 0 10b 1c5
1f1 :2 0 1f3 10e
1f4 1c2 1f3 0
1f5 110 0 200
36 :3 0 51 :3 0
1f6 1f7 0 8
:3 0 3f :3 0 1f9
1fa 0 c :3 0
38 :3 0 112 1f8
1fe :2 0 200 116
2a1 8 :3 0 52
:2 0 4 201 202
0 15 :3 0 16
:3 0 204 205 0
15 :3 0 23 :3 0
207 208 0 43
:2 0 2 :4 0 53
:2 0 26 :2 0 24
:2 0 123 20d 20f
:3 0 126 209 211
206 213 212 :2 0
18 :3 0 27 :3 0
215 216 0 1a
:4 0 54 :4 0 1d
:2 0 1e :3 0 12a
21a 21c :3 0 12d
217 21e :2 0 237
18 :3 0 27 :3 0
220 221 0 1a
:4 0 55 :4 0 1d
:2 0 8 :3 0 56
:3 0 226 227 0
130 225 229 :3 0
1d :2 0 50 :4 0
133 22b 22d :3 0
1d :2 0 8 :3 0
57 :3 0 230 231
0 136 22f 233
:3 0 139 222 235
:2 0 237 13c 238
214 237 0 239
13f 0 249 58
:3 0 14 :3 0 8
:3 0 56 :3 0 59
:3 0 8 :3 0 57
:3 0 3 :3 0 5a
:4 0 5b 1 :a 0
245 :2 0 249 23b
243 :2 0 7 :3 0
14 :3 0 247 :2 0
249 141 24b 145
24a 249 :2 0 29f
42 :3 0 8 :3 0
3f :3 0 24e 24f
0 40 :3 0 250
251 0 297 15
:3 0 16 :3 0 253
254 0 15 :3 0
23 :3 0 256 257
0 43 :2 0 2
:4 0 5c :2 0 26
:2 0 24 :2 0 147
25c 25e :3 0 14a
258 260 255 262
261 :2 0 18 :3 0
27 :3 0 264 265
0 1a :4 0 5d
:4 0 1d :2 0 1e
:3 0 14e 269 26b
:3 0 151 266 26d
:2 0 287 18 :3 0
27 :3 0 26f 270
0 1a :4 0 46
:4 0 1d :2 0 40
:3 0 154 274 276
:3 0 1d :2 0 47
:4 0 157 278 27a
:3 0 1d :2 0 48
:3 0 49 :3 0 24
:2 0 4a :2 0 15a
27d 281 15e 27c
283 :3 0 161 271
285 :2 0 287 164
288 263 287 0
289 167 0 297
58 :3 0 14 :3 0
59 :3 0 8 :3 0
57 :3 0 3 :3 0
5a :4 0 5e 1
:a 0 293 :2 0 297
28b 291 :2 0 7
:3 0 14 :3 0 295
:2 0 297 169 29a
:3 0 29a 0 29a
299 297 298 :6 0
29c 1 :3 0 16e
29e 170 29d 29c
:2 0 29f 172 :2 0
2a1 175 2a1 2a0
200 29f :6 0 2a2
:2 0 2 14 2a1
2a5 :3 0 2a4 2a2
2a6 :8 0 
17e
4
:3 0 1 3 1
7 1 b 3
6 a e 1
18 1 16 1
1f 1 1d 1
26 1 24 1
2d 1 2b 1
34 1 32 1
39 1 3f 1
45 1 52 1
5e 2 61 63
2 66 68 2
6a 6c 2 6e
70 2 72 74
2 76 78 2
7a 7c 2 7e
80 2 82 84
2 86 88 2
8a 8c 2 8e
90 2 92 94
4 5a 60 65
96 1 98 1
9a 2 a4 a6
3 a2 a3 a8
2 b1 b3 2
b0 b5 1 b7
1 b9 1 bb
1 c1 1 c9
2 d9 db 3
d7 d8 dd 2
e6 e8 2 ea
ec 2 ee f0
1 f8 2 fa
fb 2 f2 fd
2 e5 ff 1
101 1 103 2
10d 10f 3 10b
10c 111 2 11a
11c 2 119 11e
1 120 1 122
2 129 12a 3
104 123 12c 1
12e 2 145 14a
1 151 1 14c
2 164 166 3
162 163 168 2
171 173 2 175
177 2 179 17b
2 17d 17f 2
181 183 3 188
189 18a 2 185
18c 2 170 18e
1 190 1 192
2 15b 193 1
197 1 156 2
154 19b 6 1a9
1aa 1ab 1ac 1ad
1ae 2 1ba 1bc
3 1b8 1b9 1be
2 1c7 1c9 2
1cb 1cd 2 1cf
1d1 1 1d9 2
1db 1dc 2 1d3
1de 2 1e0 1e2
2 1e4 1e6 2
1e8 1ea 2 1ec
1ee 2 1c6 1f0
1 1f2 1 1f4
3 1fb 1fc 1fd
c 9b ba 12f
132 135 138 13d
19e 1a4 1b1 1f5
1ff 2 20c 20e
3 20a 20b 210
2 219 21b 2
218 21d 2 224
228 2 22a 22c
2 22e 232 2
223 234 2 21f
236 1 238 3
239 244 248 1
203 2 25b 25d
3 259 25a 25f
2 268 26a 2
267 26c 2 273
275 2 277 279
3 27e 27f 280
2 27b 282 2
272 284 2 26e
286 1 288 4
252 289 292 296
1 29a 1 24d
2 24b 29e 8
1b 22 29 30
37 3d 43 4a

1
4
0 
2a5
0
1
14
4
c
0 1 2 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 
b 1 0
16 1 0
32 1 0
3 1 0
7 1 0
2 0 1
3f 1 0
1d 1 0
39 1 0
24 1 0
2b 1 0
45 1 0
0

/
SHOW ERRORS FUNCTION SP_A_TSKRES;
ALTER FUNCTION SP_A_TSKRES COMPILE DEBUG;
/*
#Stored Procedure  :SP_A_CRIRES
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_A_CRIRES ==
CREATE OR REPLACE FUNCTION SP_A_CRIRES wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
62
2 :e:
1FUNCTION:
1SP_A_CRIRES:
1ID:
1VARCHAR2:
1RES_ID001:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_ELEMENT_ID:
132:
1H_RA_ID:
1H_REC_USER:
1H_RESOURCE_ID:
1H_TABLE_TYPE:
1PLS_INTEGER:
10:
1H_TYPE_ID:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_A_CRIRES(:
1':
1, :
1);:
1CHECK_TRACE:
11:
124:
1+:
1FPUT:
1               ENTER:: SP_A_CRIRES(25) :: cplan.sql(6387) :: :
1IS NULL:
18:
129:
1      RETURN_INVALID:: SP_A_CRIRES(30) :: cplan.sql(6392) :: :
1 :
1STATE:: :
1COALESCE:
1TO_CHAR:
1PMO_CNST:
1INVALID_PARAMS:
1NULL:
1                EXIT:: SP_A_CRIRES(33) :: cplan.sql(6395) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1TYPE_ID:
1TMT_WBS:
1ELEMENT_ID:
1SELECT TYPE_ID INTO h_type_id FROM TMT_WBS WHERE ELEMENT_ID = h_element_id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
153:
1           EXCEPTION:: SP_A_CRIRES(54) :: cplan.sql(6416) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1=:
1SC_SQL_NO_DATA:
1TMT_CRI:
1SELECT TYPE_ID INTO h_type_id FROM TMT_CRI WHERE ELEMENT_ID = h_element_id:
170:
1           EXCEPTION:: SP_A_CRIRES(71) :: cplan.sql(6433) :: :
1SC_TABLE_CRI:
1SC_TABLE_WBS:
1SYS_GUID:
1ASSIGNRESOURCE:
14:
186:
1       RETURN_STATUS:: SP_A_CRIRES(87) :: cplan.sql(6449) :: :
1 ID:: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
193:
1           EXCEPTION:: SP_A_CRIRES(94) :: cplan.sql(6456) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1105:
1           EXCEPTION:: SP_A_CRIRES(106) :: cplan.sql(6468) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
312
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 :2 a0 6b 1c
81 b0 :2 a0 6b :2 a0 6b 6e a5
b a 10 :2 a0 6b 6e :2 a0 6b
6e a5 b 6e 7e a0 b4 2e
6e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c a0 7e b4 2e :2 5a a0 7e
b4 2e :2 5a 52 10 a0 7e b4
2e :2 5a 52 10 :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
a0 a5 57 b7 19 3c :2 a0 d
:2 a0 d :2 a0 d :5 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a :5 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :3 a0 6b d b7 :3 a0 6b d
b7 :2 19 3c :2 a0 b4 2e d :2 a0
6b :7 a0 a5 b d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 :2 a0 6b :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 :2 a0 6b :2 6e 7e
:2 a0 6b b4 2e 7e 6e b4 2e
7e :2 a0 6b b4 2e a5 57 b7
19 3c :9 a0 12a e9 dd :2 a0 65
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
:7 a0 12a e9 dd :2 a0 65 b7 a4
b1 11 4f b7 a6 9 a4 b1
11 68 4f 1d 17 b5 
312
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 30
56 5a 5e 62 65 69 86 71
75 78 79 81 70 a3 91 6d
95 96 9e 90 c0 ae 8d b2
b3 bb ad dd cb aa cf d0
d8 ca f9 e8 ec c7 f4 e7
115 104 108 e4 110 103 135 120
124 100 128 130 11f 13c 140 11c
144 148 14c 14f 154 155 1 157
15c 160 164 167 16c 170 174 177
17c 17d 17f 184 187 18b 18c 191
196 199 19e 19f 1a4 1a7 1ab 1ac
1b1 1b4 1b9 1ba 1bf 1c2 1c7 1c8
1cd 1d0 1d5 1d6 1db 1de 1e2 1e3
1e8 1eb 1f0 1f1 1f6 1f9 1fe 1ff
204 207 20c 20d 212 215 219 21a
21f 222 227 228 22d 230 235 236
23b 23c 241 243 247 24a 24e 252
255 259 25d 260 263 268 26b 26e
271 272 277 278 1 27a 27f 283
287 28a 28f 294 297 29b 29c 2a1
2a2 2a7 2a9 2ad 2b0 2b4 2b7 2b8
2bd 2c0 2c3 2c7 2ca 2cb 2d0 2d3
1 2d6 2db 2df 2e2 2e3 2e8 2eb
1 2ee 2f3 2f7 2fb 2fe 302 306
309 30c 311 314 317 31a 31b 320
321 1 323 328 32c 330 333 338
33d 340 344 345 34a 34d 352 353
358 35b 360 361 366 369 36d 371
375 379 37c 37d 37f 384 385 387
388 38d 38e 393 395 399 39c 3a0
3a4 3a7 3ab 3af 3b2 3b5 3ba 3bd
3c0 3c3 3c4 3c9 3ca 1 3cc 3d1
3d5 3d9 3dc 3e1 3e6 3e9 3ed 3ee
3f3 3f4 3f9 3fb 3ff 402 406 40a
40d 411 415 418 41c 41d 422 424
428 42b 42f 433 437 43b 43f 443
447 44b 44f 453 457 45b 45f 463
46f 473 477 47a 47e 482 484 488
48c 490 493 497 49b 49d 49e 4a3
1 4a7 4ab 4af 4b2 4b6 4ba 4be
4c2 4c5 4c9 4cd 4d0 4d3 4d8 4db
4de 4e1 4e2 4e7 4e8 1 4ea 4ef
4f3 4f7 4fa 4ff 504 507 50b 50c
511 514 519 51a 51f 522 527 528
52d 530 534 535 53a 53d 542 543
548 54b 54f 553 556 559 55a 55c
55d 562 563 568 56a 56e 571 573
577 579 585 587 589 58a 58f 593
595 5a1 5a3 5a7 5ab 5ae 5b2 5b5
5b9 5bc 5bd 5c2 5c5 5c9 5cd 5d1
5d5 5d9 5e5 5e9 5ed 5f0 5f4 5f8
5fa 5fe 602 606 609 60d 611 613
614 619 1 61d 621 625 628 62c
630 634 638 63b 63f 643 646 649
64e 651 654 657 658 65d 65e 1
660 665 669 66d 670 675 67a 67d
681 682 687 68a 68f 690 695 698
69d 69e 6a3 6a6 6aa 6ab 6b0 6b3
6b8 6b9 6be 6c1 6c5 6c9 6cc 6cf
6d0 6d2 6d3 6d8 6d9 6de 6e0 6e4
6e7 6e9 6ed 6ef 6fb 6fd 6ff 700
705 709 70b 717 719 71d 721 725
728 72c 72e 732 736 73a 73d 741
743 747 74b 74e 752 756 757 75c
760 764 768 76b 76f 773 777 77b
77f 783 787 788 78a 78e 792 796
799 79d 7a1 7a4 7a7 7ac 7af 7b2
7b5 7b6 7bb 7bc 1 7be 7c3 7c7
7cb 7ce 7d3 7d8 7db 7df 7e0 7e5
7e8 7ed 7ee 7f3 7f6 7fb 7fc 801
804 808 80c 810 814 817 818 81a
81f 820 822 823 828 82b 830 831
836 839 83e 83f 844 847 84b 84c
851 854 859 85a 85f 860 865 867
86b 86e 872 876 879 87d 881 884
888 88c 88d 892 894 898 89c 89f
8a3 8a7 8aa 8ae 8b2 8b5 8b8 8bd
8c0 8c3 8c6 8c7 8cc 8cd 1 8cf
8d4 8d8 8dc 8df 8e4 8e9 8ec 8f0
8f1 8f6 8f7 8fc 900 904 907 90c
911 914 918 91c 91f 920 925 928
92d 92e 933 936 93a 93e 941 942
947 948 94d 94f 953 956 95a 95e
962 966 96a 96e 972 976 97a 986
98b 98f 993 997 99b 99d 99e 9a3
1 9a7 9ab 9af 9b2 9b6 9ba 9be
9c2 9c5 9c9 9cd 9d0 9d3 9d8 9db
9de 9e1 9e2 9e7 9e8 1 9ea 9ef
9f3 9f7 9fa 9ff a04 a07 a0b a0c
a11 a12 a17 a1b a1f a22 a27 a2c
a2f a33 a34 a39 a3c a41 a42 a47
a4a a4e a52 a55 a58 a59 a5b a5c
a61 a62 a67 a69 a6d a70 a74 a78
a7c a80 a84 a88 a8c a98 a9d aa1
aa5 aa9 aad aaf ab3 ab5 ac1 ac3
ac5 ac6 acb acf ad1 add ae1 ae3
ae4 aed 
312
2
0 1 a 6 9 :3 6 10 :3 6
12 :2 6 1 3 a 13 a :2 1
4 11 1a 19 :2 11 :2 4 c 15
14 :2 c :2 4 f 18 17 :2 f :2 4
12 1b 1a :2 12 :2 4 :2 11 20 11
:2 4 :2 e 1d e :2 4 e 17 :3 e
4 7 :2 11 23 :2 2d 3a :2 23 :3 7
:2 d 19 25 :2 2f 43 :2 25 53 59
5c :2 53 6a 7f 8 :2 6a d 10
:2 6a 13 16 :2 6a b e :2 6a 13
16 :2 6a 1b 1e :2 6a 28 2b :2 6a
b e :2 6a 13 16 :2 6a 1b 1e
:2 6a 2a 2d :2 6a 32 35 :2 6a :2 7
49 :2 4 7 :2 11 23 :2 2d 39 3c
4b 4d 4e :2 4b :2 23 :3 7 :2 d 12
1e 5b 5e :2 1e :2 7 51 :2 4 :4 9
8 7 :4 1b 1a 19 :2 7 :4 34 33
32 :2 7 a :2 14 26 :2 30 3c 3f
4e 50 51 :2 4e :2 26 :3 a :2 10 15
21 5e 61 :2 21 6e 71 :2 21 75
78 :2 21 82 85 8e 96 :2 9f :2 8e
b0 :2 85 :2 21 :2 a 54 :2 7 a :2 14
26 :2 30 3c 3f 4e 50 51 :2 4e
:2 26 :3 a :2 10 15 21 5e 61 :2 21
:2 a 54 :3 7 :2 12 28 :2 31 41 :2 7
4a :3 4 14 :2 4 15 :2 4 12 4
e 1b 2a 38 45 :2 7 :2 10 1c
7 4 c a :2 13 1f a 1a
:2 7 :2 c a :2 13 1f a d :2 17
29 :2 33 3f 43 52 54 55 :2 52
:2 29 :3 d :2 13 18 24 61 64 :2 24
71 74 :2 24 78 7b :2 24 87 8a
:2 24 92 95 :2 24 a1 a4 ab b4
b7 :2 a4 :2 24 :2 d 58 :2 a :2 7 :4 13
:2 7 4 :3 1 8 :2 11 1c 1a :2 25
:2 1a 7 11 1e 2d 3b 48 :2 a
:2 13 1f a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 55 57
58 :2 55 :2 2c :3 10 :2 16 1b 27 64
67 :2 27 74 77 :2 27 7b 7e :2 27
8a 8d :2 27 95 98 :2 27 a4 a7
ae b7 ba :2 a7 :2 27 :2 10 5b :2 d
:2 a :4 16 :2 a 7 :3 35 7 17 :2 20
7 35 7 17 :2 20 7 :5 4 :3 f
:2 4 :2 d 19 28 36 41 4f 5e
27 :2 19 4 7 :2 11 23 :2 2d 39
3c 4b 4d 4e :2 4b :2 23 :3 7 :2 d
12 1e 5b 5e :2 1e 6b 6e :2 1e
72 75 :2 1e 7f 82 8b 93 :2 9c
:2 8b a7 :2 82 :2 1e af b2 :2 1e ba
bd :2 1e c2 c5 :2 1e cd d0 :2 1e
:2 7 51 :3 4 :2 f 21 :2 2a 34 3d
:2 4 1 9 12 9 a :2 14 26
:2 30 3c 40 4f 51 52 :2 4f :2 26
:3 a :2 10 15 21 5e 61 :2 21 :3 a
:2 10 15 39 5c 5f :2 68 :2 39 76
79 :2 39 81 84 :2 8d :2 39 :2 a 55
:3 7 c 15 1e 2c a 13 1e
f e 7 e 7 e 7 29
:2 4 :2 9 7 :2 10 1c :2 7 :2 11 23
:2 2d 39 3d 4c 4f 50 :2 4c :2 23
:3 7 :2 d 12 1e 5c 5f :2 1e :3 7
:2 d 12 36 42 45 :2 36 4d 50
:2 36 5c 5f 66 6f 72 :2 5f :2 36
:2 7 53 :2 4 7 c 1b a 13
1e f e 7 e 7 e 7
:2 4 :4 10 :2 4 :8 1 
312
4
0 :2 1 :4 3 :4 4
:4 5 2 :4 6 :2 1
:7 8 :7 9 :7 a :7 b
:6 c :6 d :7 f :b 11
:11 12 13 :2 12 :2 13
:2 12 :2 13 :2 12 :2 14
:2 12 :2 14 :2 12 :2 14
:2 12 :2 14 :2 12 :2 15
:2 12 :2 15 :2 12 :2 15
:2 12 :2 15 :2 12 :2 15
:4 12 :3 11 :11 18 :b 19
:3 18 :16 1c :11 1d :20 1e
:3 1d :11 20 :b 21 :3 20
:9 23 :3 1c :3 28 :3 29
:3 2a :6 2d :5 2e 2c
30 :5 31 :3 30 :2 32
:5 34 :11 35 :24 36 :3 35
:2 33 :6 32 2f :3 10
:a 3c :6 3e :5 3f 3d
41 :5 42 :3 41 :2 43
:5 45 :11 46 :24 47 :3 46
:2 44 :6 43 40 :3 3c
:5 4c 3c :5 4e 4d
:3 3c :5 51 :9 53 54
:3 53 :11 56 :30 57 :3 56
:a 59 10 :3 5c :11 5d
:b 5e :17 5f :3 5d :2 61
:3 62 :3 63 64 62
61 62 :3 65 :3 5c
:2 66 :5 68 :11 69 :b 6a
:18 6b :3 69 :2 6d 6e
:3 6f 70 6e 6d
6e :3 71 :2 67 :6 66
5b :7 1 
aef
4
:3 0 1 :3 0 2
:a 0 30d 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
:2 0 7 4 :3 0
5 :7 0 9 8
:3 0 4 :3 0 6
:7 0 d c :3 0
7 :3 0 8 :3 0
9 :2 0 4 11
12 0 f 13
0 30d 2 14
:2 0 b :2 0 f
4 :3 0 b :2 0
d 17 19 :6 0
1c 1a 0 30b
0 a :6 0 b
:2 0 13 4 :3 0
11 1e 20 :6 0
23 21 0 30b
0 c :6 0 b
:2 0 17 4 :3 0
15 25 27 :6 0
2a 28 0 30b
0 d :6 0 11
:2 0 1b 4 :3 0
19 2c 2e :6 0
31 2f 0 30b
0 e :6 0 11
:2 0 1d 10 :3 0
33 :7 0 37 34
35 30b 0 f
:6 0 3f 40 0
1f 10 :3 0 39
:7 0 3d 3a 3b
30b 0 12 :6 0
45 46 0 21
8 :3 0 9 :2 0
4 41 :7 0 44
42 0 30b 0
13 :6 0 14 :3 0
15 :3 0 14 :3 0
16 :3 0 48 49
0 2 :4 0 23
4a 4c 47 4e
4d :2 0 17 :3 0
18 :3 0 50 51
0 19 :4 0 14
:3 0 1a :3 0 54
55 0 2 :4 0
25 56 58 1b
:4 0 1c :2 0 1d
:3 0 27 5b 5d
:3 0 1e :4 0 1c
:2 0 1f :4 0 2a
60 62 :3 0 1c
:2 0 3 :3 0 2d
64 66 :3 0 1c
:2 0 1f :4 0 30
68 6a :3 0 1c
:2 0 20 :4 0 33
6c 6e :3 0 1c
:2 0 1f :4 0 36
70 72 :3 0 1c
:2 0 5 :3 0 39
74 76 :3 0 1c
:2 0 1f :4 0 3c
78 7a :3 0 1c
:2 0 20 :4 0 3f
7c 7e :3 0 1c
:2 0 1f :4 0 42
80 82 :3 0 1c
:2 0 6 :3 0 45
84 86 :3 0 1c
:2 0 1f :4 0 48
88 8a :3 0 1c
:2 0 21 :4 0 4b
8c 8e :3 0 4e
52 90 :2 0 92
53 93 4f 92
0 94 55 0
26b 14 :3 0 15
:3 0 95 96 0
14 :3 0 22 :3 0
98 99 0 23
:2 0 2 :4 0 24
:2 0 25 :2 0 23
:2 0 57 9e a0
:3 0 5a 9a a2
97 a4 a3 :2 0
17 :3 0 26 :3 0
a6 a7 0 19
:4 0 27 :4 0 1c
:2 0 1d :3 0 5e
ab ad :3 0 61
a8 af :2 0 b1
64 b2 a5 b1
0 b3 66 0
26b 3 :3 0 28
:2 0 68 b5 b6
:3 0 b7 :2 0 b8
:2 0 5 :3 0 28
:2 0 6a bb bc
:3 0 bd :2 0 be
:2 0 b9 c0 bf
:2 0 6 :3 0 28
:2 0 6c c3 c4
:3 0 c5 :2 0 c6
:2 0 c1 c8 c7
:2 0 14 :3 0 15
:3 0 ca cb 0
14 :3 0 22 :3 0
cd ce 0 29
:2 0 2 :4 0 2a
:2 0 25 :2 0 23
:2 0 6e d3 d5
:3 0 71 cf d7
cc d9 d8 :2 0
17 :3 0 26 :3 0
db dc 0 19
:4 0 2b :4 0 1c
:2 0 1d :3 0 75
e0 e2 :3 0 1c
:2 0 2c :4 0 78
e4 e6 :3 0 1c
:2 0 2d :4 0 7b
e8 ea :3 0 1c
:2 0 2e :3 0 2f
:3 0 30 :3 0 31
:3 0 ef f0 0
7e ee f2 32
:4 0 80 ed f5
83 ec f7 :3 0
86 dd f9 :2 0
fb 89 fc da
fb 0 fd 8b
0 126 14 :3 0
15 :3 0 fe ff
0 14 :3 0 22
:3 0 101 102 0
23 :2 0 2 :4 0
b :2 0 25 :2 0
23 :2 0 8d 107
109 :3 0 90 103
10b 100 10d 10c
:2 0 17 :3 0 26
:3 0 10f 110 0
19 :4 0 33 :4 0
1c :2 0 1d :3 0
94 114 116 :3 0
97 111 118 :2 0
11a 9a 11b 10e
11a 0 11c 9c
0 126 34 :3 0
35 :3 0 11d 11e
0 30 :3 0 31
:3 0 120 121 0
36 :3 0 9e 11f
124 :2 0 126 a1
127 c9 126 0
128 a5 0 26b
a :3 0 3 :3 0
129 12a 0 26b
e :3 0 5 :3 0
12c 12d 0 26b
d :3 0 6 :3 0
12f 130 0 26b
37 :3 0 12 :3 0
38 :3 0 39 :3 0
a :4 0 3a 1
:8 0 13d 8 :3 0
3b :3 0 138 139
0 3c :3 0 13a
13b 0 13d a7
190 3d :3 0 8
:3 0 3b :3 0 13f
140 0 3c :3 0
141 142 0 144
aa 146 ac 145
144 :2 0 18e 3e
:3 0 8 :3 0 3b
:3 0 149 14a 0
3c :3 0 14b 14c
0 186 14 :3 0
15 :3 0 14e 14f
0 14 :3 0 22
:3 0 151 152 0
3f :2 0 2 :4 0
40 :2 0 25 :2 0
23 :2 0 ae 157
159 :3 0 b1 153
15b 150 15d 15c
:2 0 17 :3 0 26
:3 0 15f 160 0
19 :4 0 41 :4 0
1c :2 0 1d :3 0
b5 164 166 :3 0
1c :2 0 2c :4 0
b8 168 16a :3 0
1c :2 0 42 :4 0
bb 16c 16e :3 0
1c :2 0 3c :3 0
be 170 172 :3 0
1c :2 0 43 :4 0
c1 174 176 :3 0
1c :2 0 44 :3 0
45 :3 0 23 :2 0
46 :2 0 c4 179
17d c8 178 17f
:3 0 cb 161 181
:2 0 183 ce 184
15e 183 0 185
d0 0 186 d2
189 :3 0 189 0
189 188 186 187
:6 0 18b 2 :3 0
d5 18d d7 18c
18b :2 0 18e d9
:2 0 190 0 190
18f 13d 18e :6 0
26b 1 :3 0 8
:3 0 3b :3 0 192
193 0 8 :3 0
47 :2 0 48 :3 0
195 197 0 de
196 199 :3 0 19a
:2 0 37 :3 0 12
:3 0 49 :3 0 39
:3 0 a :4 0 4a
1 :8 0 1a7 8
:3 0 3b :3 0 1a2
1a3 0 3c :3 0
1a4 1a5 0 1a7
e1 1fa 3d :3 0
8 :3 0 3b :3 0
1a9 1aa 0 3c
:3 0 1ab 1ac 0
1ae e4 1b0 e6
1af 1ae :2 0 1f8
3e :3 0 8 :3 0
3b :3 0 1b3 1b4
0 3c :3 0 1b5
1b6 0 1f0 14
:3 0 15 :3 0 1b8
1b9 0 14 :3 0
22 :3 0 1bb 1bc
0 3f :2 0 2
:4 0 4b :2 0 25
:2 0 23 :2 0 e8
1c1 1c3 :3 0 eb
1bd 1c5 1ba 1c7
1c6 :2 0 17 :3 0
26 :3 0 1c9 1ca
0 19 :4 0 4c
:4 0 1c :2 0 1d
:3 0 ef 1ce 1d0
:3 0 1c :2 0 2c
:4 0 f2 1d2 1d4
:3 0 1c :2 0 42
:4 0 f5 1d6 1d8
:3 0 1c :2 0 3c
:3 0 f8 1da 1dc
:3 0 1c :2 0 43
:4 0 fb 1de 1e0
:3 0 1c :2 0 44
:3 0 45 :3 0 23
:2 0 46 :2 0 fe
1e3 1e7 102 1e2
1e9 :3 0 105 1cb
1eb :2 0 1ed 108
1ee 1c8 1ed 0
1ef 10a 0 1f0
10c 1f3 :3 0 1f3
0 1f3 1f2 1f0
1f1 :6 0 1f5 4
:3 0 10f 1f7 111
1f6 1f5 :2 0 1f8
113 :2 0 1fa 0
1fa 1f9 1a7 1f8
:6 0 201 1 :3 0
f :3 0 30 :3 0
4d :3 0 1fd 1fe
0 1fc 1ff 0
201 116 208 f
:3 0 30 :3 0 4e
:3 0 203 204 0
202 205 0 207
119 209 19b 201
0 20a 0 207
0 20a 11b 0
26b c :3 0 4f
:4 0 20c 20d :3 0
20b 20e 0 26b
8 :3 0 3b :3 0
210 211 0 50
:3 0 f :3 0 12
:3 0 a :3 0 e
:3 0 d :3 0 c
:3 0 11e 213 21a
212 21b 0 26b
14 :3 0 15 :3 0
21d 21e 0 14
:3 0 22 :3 0 220
221 0 51 :2 0
2 :4 0 52 :2 0
25 :2 0 23 :2 0
125 226 228 :3 0
128 222 22a 21f
22c 22b :2 0 17
:3 0 26 :3 0 22e
22f 0 19 :4 0
53 :4 0 1c :2 0
1d :3 0 12c 233
235 :3 0 1c :2 0
2c :4 0 12f 237
239 :3 0 1c :2 0
2d :4 0 132 23b
23d :3 0 1c :2 0
2e :3 0 2f :3 0
8 :3 0 3b :3 0
242 243 0 135
241 245 32 :4 0
137 240 248 13a
23f 24a :3 0 1c
:2 0 54 :4 0 13d
24c 24e :3 0 1c
:2 0 1f :4 0 140
250 252 :3 0 1c
:2 0 c :3 0 143
254 256 :3 0 1c
:2 0 1f :4 0 146
258 25a :3 0 149
230 25c :2 0 25e
14c 25f 22d 25e
0 260 14e 0
26b 34 :3 0 55
:3 0 261 262 0
8 :3 0 3b :3 0
264 265 0 c
:3 0 36 :3 0 150
263 269 :2 0 26b
154 30c 8 :3 0
56 :2 0 4 26c
26d 0 14 :3 0
15 :3 0 26f 270
0 14 :3 0 22
:3 0 272 273 0
3f :2 0 2 :4 0
57 :2 0 25 :2 0
23 :2 0 161 278
27a :3 0 164 274
27c 271 27e 27d
:2 0 17 :3 0 26
:3 0 280 281 0
19 :4 0 58 :4 0
1c :2 0 1d :3 0
168 285 287 :3 0
16b 282 289 :2 0
2a2 17 :3 0 26
:3 0 28b 28c 0
19 :4 0 59 :4 0
1c :2 0 8 :3 0
5a :3 0 291 292
0 16e 290 294
:3 0 1c :2 0 54
:4 0 171 296 298
:3 0 1c :2 0 8
:3 0 5b :3 0 29b
29c 0 174 29a
29e :3 0 177 28d
2a0 :2 0 2a2 17a
2a3 27f 2a2 0
2a4 17d 0 2b4
5c :3 0 13 :3 0
8 :3 0 5a :3 0
5d :3 0 8 :3 0
5b :3 0 3 :3 0
5e :4 0 5f 1
:a 0 2b0 :2 0 2b4
2a6 2ae :2 0 7
:3 0 13 :3 0 2b2
:2 0 2b4 17f 2b6
183 2b5 2b4 :2 0
30a 3e :3 0 8
:3 0 3b :3 0 2b9
2ba 0 3c :3 0
2bb 2bc 0 302
14 :3 0 15 :3 0
2be 2bf 0 14
:3 0 22 :3 0 2c1
2c2 0 3f :2 0
2 :4 0 60 :2 0
25 :2 0 23 :2 0
185 2c7 2c9 :3 0
188 2c3 2cb 2c0
2cd 2cc :2 0 17
:3 0 26 :3 0 2cf
2d0 0 19 :4 0
61 :4 0 1c :2 0
1d :3 0 18c 2d4
2d6 :3 0 18f 2d1
2d8 :2 0 2f2 17
:3 0 26 :3 0 2da
2db 0 19 :4 0
42 :4 0 1c :2 0
3c :3 0 192 2df
2e1 :3 0 1c :2 0
43 :4 0 195 2e3
2e5 :3 0 1c :2 0
44 :3 0 45 :3 0
23 :2 0 46 :2 0
198 2e8 2ec 19c
2e7 2ee :3 0 19f
2dc 2f0 :2 0 2f2
1a2 2f3 2ce 2f2
0 2f4 1a5 0
302 5c :3 0 13
:3 0 5d :3 0 8
:3 0 5b :3 0 3
:3 0 5e :4 0 62
1 :a 0 2fe :2 0
302 2f6 2fc :2 0
7 :3 0 13 :3 0
300 :2 0 302 1a7
305 :3 0 305 0
305 304 302 303
:6 0 307 1 :3 0
1ac 309 1ae 308
307 :2 0 30a 1b0
:2 0 30c 1b3 30c
30b 26b 30a :6 0
30d :2 0 2 14
30c 310 :3 0 30f
30d 311 :8 0 
1bb
4
:3 0 1 3 1
7 1 b 3
6 a e 1
18 1 16 1
1f 1 1d 1
26 1 24 1
2d 1 2b 1
32 1 38 1
3e 1 4b 1
57 2 5a 5c
2 5f 61 2
63 65 2 67
69 2 6b 6d
2 6f 71 2
73 75 2 77
79 2 7b 7d
2 7f 81 2
83 85 2 87
89 2 8b 8d
4 53 59 5e
8f 1 91 1
93 2 9d 9f
3 9b 9c a1
2 aa ac 2
a9 ae 1 b0
1 b2 1 b4
1 ba 1 c2
2 d2 d4 3
d0 d1 d6 2
df e1 2 e3
e5 2 e7 e9
1 f1 2 f3
f4 2 eb f6
2 de f8 1
fa 1 fc 2
106 108 3 104
105 10a 2 113
115 2 112 117
1 119 1 11b
2 122 123 3
fd 11c 125 1
127 2 137 13c
1 143 1 13e
2 156 158 3
154 155 15a 2
163 165 2 167
169 2 16b 16d
2 16f 171 2
173 175 3 17a
17b 17c 2 177
17e 2 162 180
1 182 1 184
2 14d 185 1
189 1 148 2
146 18d 1 198
2 194 198 2
1a1 1a6 1 1ad
1 1a8 2 1c0
1c2 3 1be 1bf
1c4 2 1cd 1cf
2 1d1 1d3 2
1d5 1d7 2 1d9
1db 2 1dd 1df
3 1e4 1e5 1e6
2 1e1 1e8 2
1cc 1ea 1 1ec
1 1ee 2 1b7
1ef 1 1f3 1
1b2 2 1b0 1f7
2 1fa 200 1
206 2 208 209
6 214 215 216
217 218 219 2
225 227 3 223
224 229 2 232
234 2 236 238
2 23a 23c 1
244 2 246 247
2 23e 249 2
24b 24d 2 24f
251 2 253 255
2 257 259 2
231 25b 1 25d
1 25f 3 266
267 268 c 94
b3 128 12b 12e
131 190 20a 20f
21c 260 26a 2
277 279 3 275
276 27b 2 284
286 2 283 288
2 28f 293 2
295 297 2 299
29d 2 28e 29f
2 28a 2a1 1
2a3 3 2a4 2af
2b3 1 26e 2
2c6 2c8 3 2c4
2c5 2ca 2 2d3
2d5 2 2d2 2d7
2 2de 2e0 2
2e2 2e4 3 2e9
2ea 2eb 2 2e6
2ed 2 2dd 2ef
2 2d9 2f1 1
2f3 4 2bd 2f4
2fd 301 1 305
1 2b8 2 2b6
309 7 1b 22
29 30 36 3c
43 
1
4
0 
310
0
1
14
6
b
0 1 2 1 4 1 0 0
0 0 0 0 0 0 0 0
0 0 0 0 
b 1 0
16 1 0
2b 1 0
3 1 0
7 1 0
38 1 0
2 0 1
1d 1 0
32 1 0
24 1 0
3e 1 0
0

/
SHOW ERRORS FUNCTION SP_A_CRIRES;
ALTER FUNCTION SP_A_CRIRES COMPILE DEBUG;
/***************************************************************************************************/
/*
#Stored Procedure  :SP_A_REQRES
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_A_REQRES ==
CREATE OR REPLACE FUNCTION SP_A_REQRES wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
59
2 :e:
1FUNCTION:
1SP_A_REQRES:
1ID:
1VARCHAR2:
1RES_ID001:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_ELEMENT_ID:
132:
1H_RA_ID:
1H_REC_USER:
1H_RESOURCE_ID:
1H_TYPE_ID:
1PLS_INTEGER:
10:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_A_REQRES(:
1':
1, :
1);:
1CHECK_TRACE:
11:
123:
1+:
1FPUT:
1               ENTER:: SP_A_REQRES(24) :: cplan.sql(6518) :: :
1IS NULL:
18:
128:
1      RETURN_INVALID:: SP_A_REQRES(29) :: cplan.sql(6523) :: :
1 :
1STATE:: :
1COALESCE:
1TO_CHAR:
1PMO_CNST:
1INVALID_PARAMS:
1NULL:
131:
1                EXIT:: SP_A_REQRES(32) :: cplan.sql(6526) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1TYPE_ID:
1TMT_WBS:
1ELEMENT_ID:
1SELECT TYPE_ID INTO h_type_id FROM TMT_WBS WHERE ELEMENT_ID = h_element_id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
152:
1           EXCEPTION:: SP_A_REQRES(53) :: cplan.sql(6547) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SYS_GUID:
14:
164:
1       RETURN_STATUS:: SP_A_REQRES(65) :: cplan.sql(6559) :: :
1 ID:: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
171:
1           EXCEPTION:: SP_A_REQRES(72) :: cplan.sql(6566) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
183:
1           EXCEPTION:: SP_A_REQRES(84) :: cplan.sql(6578) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
284
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 1c 51 81 b0 a3 :2 a0
6b 1c 81 b0 :2 a0 6b :2 a0 6b
6e a5 b a 10 :2 a0 6b 6e
:2 a0 6b 6e a5 b 6e 7e a0
b4 2e 6e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c a0 7e b4 2e 5a
a0 7e b4 2e 5a 52 10 a0
7e b4 2e :2 5a 52 10 :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b a0 a5 57 b7 19 3c
:2 a0 d :2 a0 d :2 a0 d :5 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 b4 2e d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 :2 a0 6b :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 :2 a0 6b :2 6e 7e :2 a0
6b b4 2e 7e 6e b4 2e 7e
:2 a0 6b b4 2e a5 57 b7 19
3c :9 a0 12a e9 dd :2 a0 65 b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c :7 a0
12a e9 dd :2 a0 65 b7 a4 b1
11 4f b7 a6 9 a4 b1 11
68 4f 1d 17 b5 
284
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 30
56 5a 5e 62 65 69 86 71
75 78 79 81 70 a3 91 6d
95 96 9e 90 c0 ae 8d b2
b3 bb ad dd cb aa cf d0
d8 ca f9 e8 ec c7 f4 e7
119 104 108 e4 10c 114 103 120
124 100 128 12c 130 133 138 139
1 13b 140 144 148 14b 150 154
158 15b 160 161 163 168 16b 16f
170 175 17a 17d 182 183 188 18b
18f 190 195 198 19d 19e 1a3 1a6
1ab 1ac 1b1 1b4 1b9 1ba 1bf 1c2
1c6 1c7 1cc 1cf 1d4 1d5 1da 1dd
1e2 1e3 1e8 1eb 1f0 1f1 1f6 1f9
1fd 1fe 203 206 20b 20c 211 214
219 21a 21f 220 225 227 22b 22e
232 236 239 23d 241 244 247 24c
24f 252 255 256 25b 25c 1 25e
263 267 26b 26e 273 278 27b 27f
280 285 286 28b 28d 291 294 298
29b 29c 2a1 2a4 2a8 2ab 2ac 2b1
1 2b4 2b9 2bd 2c0 2c1 2c6 2c9
1 2cc 2d1 2d5 2d9 2dc 2e0 2e4
2e7 2ea 2ef 2f2 2f5 2f8 2f9 2fe
2ff 1 301 306 30a 30e 311 316
31b 31e 322 323 328 32b 330 331
336 339 33e 33f 344 347 34b 34f
353 357 35a 35b 35d 362 363 365
366 36b 36c 371 373 377 37a 37e
382 385 389 38d 390 393 398 39b
39e 3a1 3a2 3a7 3a8 1 3aa 3af
3b3 3b7 3ba 3bf 3c4 3c7 3cb 3cc
3d1 3d2 3d7 3d9 3dd 3e0 3e4 3e8
3eb 3ef 3f3 3f6 3fa 3fb 400 402
406 409 40d 411 415 419 41d 421
425 429 42d 431 435 439 43d 441
44d 451 455 458 45c 460 462 466
46a 46e 471 475 479 47b 47c 481
1 485 489 48d 490 494 498 49c
4a0 4a3 4a7 4ab 4ae 4b1 4b6 4b9
4bc 4bf 4c0 4c5 4c6 1 4c8 4cd
4d1 4d5 4d8 4dd 4e2 4e5 4e9 4ea
4ef 4f2 4f7 4f8 4fd 500 505 506
50b 50e 512 513 518 51b 520 521
526 529 52d 531 534 537 538 53a
53b 540 541 546 548 54c 54f 551
555 557 563 565 567 568 56d 571
573 57f 581 585 589 58a 58f 593
597 59b 59e 5a2 5a6 5a9 5ac 5b1
5b4 5b7 5ba 5bb 5c0 5c1 1 5c3
5c8 5cc 5d0 5d3 5d8 5dd 5e0 5e4
5e5 5ea 5ed 5f2 5f3 5f8 5fb 600
601 606 609 60d 611 615 619 61c
61d 61f 624 625 627 628 62d 630
635 636 63b 63e 643 644 649 64c
650 651 656 659 65e 65f 664 665
66a 66c 670 673 677 67b 67e 682
686 689 68d 691 692 697 699 69d
6a1 6a4 6a8 6ac 6af 6b3 6b7 6ba
6bd 6c2 6c5 6c8 6cb 6cc 6d1 6d2
1 6d4 6d9 6dd 6e1 6e4 6e9 6ee
6f1 6f5 6f6 6fb 6fc 701 705 709
70c 711 716 719 71d 721 724 725
72a 72d 732 733 738 73b 73f 743
746 747 74c 74d 752 754 758 75b
75f 763 767 76b 76f 773 777 77b
77f 78b 790 794 798 79c 7a0 7a2
7a3 7a8 1 7ac 7b0 7b4 7b7 7bb
7bf 7c3 7c7 7ca 7ce 7d2 7d5 7d8
7dd 7e0 7e3 7e6 7e7 7ec 7ed 1
7ef 7f4 7f8 7fc 7ff 804 809 80c
810 811 816 817 81c 820 824 827
82c 831 834 838 839 83e 841 846
847 84c 84f 853 857 85a 85d 85e
860 861 866 867 86c 86e 872 875
879 87d 881 885 889 88d 891 89d
8a2 8a6 8aa 8ae 8b2 8b4 8b8 8ba
8c6 8c8 8ca 8cb 8d0 8d4 8d6 8e2
8e6 8e8 8e9 8f2 
284
2
0 1 a 6 9 :3 6 10 :3 6
12 :2 6 1 3 a 13 a :2 1
4 11 1a 19 :2 11 :2 4 c 15
14 :2 c :2 4 f 18 17 :2 f :2 4
12 1b 1a :2 12 :2 4 :2 e 1d e
:2 4 e 17 :3 e 4 7 :2 11 23
:2 2d 3a :2 23 :3 7 :2 d 19 25 :2 2f
43 :2 25 53 59 5c :2 53 6a 7f
8 :2 6a d 10 :2 6a 13 16 :2 6a
b e :2 6a 13 16 :2 6a 1b 1e
:2 6a 28 2b :2 6a b e :2 6a 13
16 :2 6a 1b 1e :2 6a 2a 2d :2 6a
32 35 :2 6a :2 7 49 :2 4 7 :2 11
23 :2 2d 39 3c 4b 4d 4e :2 4b
:2 23 :3 7 :2 d 12 1e 5b 5e :2 1e
:2 7 51 :2 4 :4 8 7 :4 18 17 :2 7
:4 30 2f 2e :2 7 a :2 14 26 :2 30
3c 3f 4e 50 51 :2 4e :2 26 :3 a
:2 10 15 21 5e 61 :2 21 6e 71
:2 21 75 78 :2 21 82 85 8e 96
:2 9f :2 8e b0 :2 85 :2 21 :2 a 54 :2 7
a :2 14 26 :2 30 3c 3f 4e 50
51 :2 4e :2 26 :3 a :2 10 15 21 5e
61 :2 21 :2 a 54 :3 7 :2 12 28 :2 31
41 :2 7 46 :3 4 14 :2 4 15 :2 4
12 4 e 1b 2a 38 45 :2 7
:2 10 1c 7 4 c a :2 13 1f
a 1a :2 7 :2 c a :2 13 1f a
d :2 17 29 :2 33 3f 43 52 54
55 :2 52 :2 29 :3 d :2 13 18 24 61
64 :2 24 71 74 :2 24 78 7b :2 24
87 8a :2 24 92 95 :2 24 a1 a4
ab b4 b7 :2 a4 :2 24 :2 d 58 :2 a
:2 7 :4 13 :2 7 4 :3 1 4 :3 f 4
7 :2 11 23 :2 2d 39 3c 4b 4d
4e :2 4b :2 23 :3 7 :2 d 12 1e 5b
5e :2 1e 6b 6e :2 1e 72 75 :2 1e
7f 82 8b 93 :2 9c :2 8b a7 :2 82
:2 1e af b2 :2 1e ba bd :2 1e c2
c5 :2 1e cd d0 :2 1e :2 7 51 :3 4
:2 f 21 :2 2a 34 3d :2 4 1 9
12 9 a :2 14 26 :2 30 3c 40
4f 51 52 :2 4f :2 26 :3 a :2 10 15
21 5e 61 :2 21 :3 a :2 10 15 39
5c 5f :2 68 :2 39 76 79 :2 39 81
84 :2 8d :2 39 :2 a 55 :3 7 c 15
1e 2c a 13 1e f e 7
e 7 e 7 29 :2 4 :2 9 7
:2 10 1c :2 7 :2 11 23 :2 2d 39 3d
4c 4e 4f :2 4c :2 23 :3 7 :2 d 12
1e 5b 5e :2 1e :3 7 :2 d 12 36
42 45 :2 36 4d 50 :2 36 5c 5f
66 6f 72 :2 5f :2 36 :2 7 52 :2 4
7 c 1b a 13 1e f e
7 e 7 e 7 :2 4 :4 10 :2 4
:8 1 
284
4
0 :2 1 :4 3 :4 4
:4 5 2 :4 6 :2 1
:7 8 :7 9 :7 a :7 b
:6 c :7 e :b 10 :11 11
12 :2 11 :2 12 :2 11
:2 12 :2 11 :2 13 :2 11
:2 13 :2 11 :2 13 :2 11
:2 13 :2 11 :2 14 :2 11
:2 14 :2 11 :2 14 :2 11
:2 14 :2 11 :2 14 :4 11
:3 10 :11 17 :b 18 :3 17
:14 1b :11 1c :20 1d :3 1c
:11 1f :b 20 :3 1f :9 22
:3 1b :3 27 :3 28 :3 29
:6 2c :5 2d 2b 2f
:5 30 :3 2f :2 31 :5 33
:11 34 :24 35 :3 34 :2 32
:6 31 2e :3 f :5 3c
:11 40 :30 41 :3 40 :a 43
f :3 46 :11 47 :b 48
:17 49 :3 47 :2 4b :3 4c
:3 4d 4e 4c 4b
4c :3 4f :3 46 :2 50
:5 52 :11 53 :b 54 :18 55
:3 53 :2 57 58 :3 59
5a 58 57 58
:3 5b :2 51 :6 50 45
:7 1 
8f4
4
:3 0 1 :3 0 2
:a 0 27f 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
:2 0 7 4 :3 0
5 :7 0 9 8
:3 0 4 :3 0 6
:7 0 d c :3 0
7 :3 0 8 :3 0
9 :2 0 4 11
12 0 f 13
0 27f 2 14
:2 0 b :2 0 f
4 :3 0 b :2 0
d 17 19 :6 0
1c 1a 0 27d
0 a :6 0 b
:2 0 13 4 :3 0
11 1e 20 :6 0
23 21 0 27d
0 c :6 0 b
:2 0 17 4 :3 0
15 25 27 :6 0
2a 28 0 27d
0 d :6 0 11
:2 0 1b 4 :3 0
19 2c 2e :6 0
31 2f 0 27d
0 e :6 0 39
3a 0 1d 10
:3 0 33 :7 0 37
34 35 27d 0
f :6 0 3f 40
0 1f 8 :3 0
9 :2 0 4 3b
:7 0 3e 3c 0
27d 0 12 :6 0
13 :3 0 14 :3 0
13 :3 0 15 :3 0
42 43 0 2
:4 0 21 44 46
41 48 47 :2 0
16 :3 0 17 :3 0
4a 4b 0 18
:4 0 13 :3 0 19
:3 0 4e 4f 0
2 :4 0 23 50
52 1a :4 0 1b
:2 0 1c :3 0 25
55 57 :3 0 1d
:4 0 1b :2 0 1e
:4 0 28 5a 5c
:3 0 1b :2 0 3
:3 0 2b 5e 60
:3 0 1b :2 0 1e
:4 0 2e 62 64
:3 0 1b :2 0 1f
:4 0 31 66 68
:3 0 1b :2 0 1e
:4 0 34 6a 6c
:3 0 1b :2 0 5
:3 0 37 6e 70
:3 0 1b :2 0 1e
:4 0 3a 72 74
:3 0 1b :2 0 1f
:4 0 3d 76 78
:3 0 1b :2 0 1e
:4 0 40 7a 7c
:3 0 1b :2 0 6
:3 0 43 7e 80
:3 0 1b :2 0 1e
:4 0 46 82 84
:3 0 1b :2 0 20
:4 0 49 86 88
:3 0 :2 4c 8a :2 0
8c 51 8d 49
8c 0 8e 53
0 1dd 13 :3 0
14 :3 0 8f 90
0 13 :3 0 21
:3 0 92 93 0
22 :2 0 2 :4 0
23 :2 0 24 :2 0
22 :2 0 55 98
9a :3 0 58 94
9c 91 9e 9d
:2 0 16 :3 0 25
:3 0 a0 a1 0
18 :4 0 26 :4 0
1b :2 0 1c :3 0
5c a5 a7 :3 0
5f a2 a9 :2 0
ab 62 ac 9f
ab 0 ad 64
0 1dd 3 :3 0
27 :2 0 66 af
b0 :3 0 b1 :2 0
5 :3 0 27 :2 0
68 b4 b5 :3 0
b6 :2 0 b2 b8
b7 :2 0 6 :3 0
27 :2 0 6a bb
bc :3 0 bd :2 0
be :2 0 b9 c0
bf :2 0 13 :3 0
14 :3 0 c2 c3
0 13 :3 0 21
:3 0 c5 c6 0
28 :2 0 2 :4 0
29 :2 0 24 :2 0
22 :2 0 6c cb
cd :3 0 6f c7
cf c4 d1 d0
:2 0 16 :3 0 25
:3 0 d3 d4 0
18 :4 0 2a :4 0
1b :2 0 1c :3 0
73 d8 da :3 0
1b :2 0 2b :4 0
76 dc de :3 0
1b :2 0 2c :4 0
79 e0 e2 :3 0
1b :2 0 2d :3 0
2e :3 0 2f :3 0
30 :3 0 e7 e8
0 7c e6 ea
31 :4 0 7e e5
ed 81 e4 ef
:3 0 84 d5 f1
:2 0 f3 87 f4
d2 f3 0 f5
89 0 11e 13
:3 0 14 :3 0 f6
f7 0 13 :3 0
21 :3 0 f9 fa
0 22 :2 0 2
:4 0 32 :2 0 24
:2 0 22 :2 0 8b
ff 101 :3 0 8e
fb 103 f8 105
104 :2 0 16 :3 0
25 :3 0 107 108
0 18 :4 0 33
:4 0 1b :2 0 1c
:3 0 92 10c 10e
:3 0 95 109 110
:2 0 112 98 113
106 112 0 114
9a 0 11e 34
:3 0 35 :3 0 115
116 0 2f :3 0
30 :3 0 118 119
0 36 :3 0 9c
117 11c :2 0 11e
9f 11f c1 11e
0 120 a3 0
1dd a :3 0 3
:3 0 121 122 0
1dd e :3 0 5
:3 0 124 125 0
1dd d :3 0 6
:3 0 127 128 0
1dd 37 :3 0 f
:3 0 38 :3 0 39
:3 0 a :4 0 3a
1 :8 0 135 8
:3 0 3b :3 0 130
131 0 3c :3 0
132 133 0 135
a5 188 3d :3 0
8 :3 0 3b :3 0
137 138 0 3c
:3 0 139 13a 0
13c a8 13e aa
13d 13c :2 0 186
3e :3 0 8 :3 0
3b :3 0 141 142
0 3c :3 0 143
144 0 17e 13
:3 0 14 :3 0 146
147 0 13 :3 0
21 :3 0 149 14a
0 3f :2 0 2
:4 0 40 :2 0 24
:2 0 22 :2 0 ac
14f 151 :3 0 af
14b 153 148 155
154 :2 0 16 :3 0
25 :3 0 157 158
0 18 :4 0 41
:4 0 1b :2 0 1c
:3 0 b3 15c 15e
:3 0 1b :2 0 2b
:4 0 b6 160 162
:3 0 1b :2 0 42
:4 0 b9 164 166
:3 0 1b :2 0 3c
:3 0 bc 168 16a
:3 0 1b :2 0 43
:4 0 bf 16c 16e
:3 0 1b :2 0 44
:3 0 45 :3 0 22
:2 0 46 :2 0 c2
171 175 c6 170
177 :3 0 c9 159
179 :2 0 17b cc
17c 156 17b 0
17d ce 0 17e
d0 181 :3 0 181
0 181 180 17e
17f :6 0 183 2
:3 0 d3 185 d5
184 183 :2 0 186
d7 :2 0 188 0
188 187 135 186
:6 0 1dd 1 :3 0
c :3 0 47 :4 0
18b 18c :3 0 18a
18d 0 1dd 13
:3 0 14 :3 0 18f
190 0 13 :3 0
21 :3 0 192 193
0 48 :2 0 2
:4 0 49 :2 0 24
:2 0 22 :2 0 da
198 19a :3 0 dd
194 19c 191 19e
19d :2 0 16 :3 0
25 :3 0 1a0 1a1
0 18 :4 0 4a
:4 0 1b :2 0 1c
:3 0 e1 1a5 1a7
:3 0 1b :2 0 2b
:4 0 e4 1a9 1ab
:3 0 1b :2 0 2c
:4 0 e7 1ad 1af
:3 0 1b :2 0 2d
:3 0 2e :3 0 8
:3 0 3b :3 0 1b4
1b5 0 ea 1b3
1b7 31 :4 0 ec
1b2 1ba ef 1b1
1bc :3 0 1b :2 0
4b :4 0 f2 1be
1c0 :3 0 1b :2 0
1e :4 0 f5 1c2
1c4 :3 0 1b :2 0
c :3 0 f8 1c6
1c8 :3 0 1b :2 0
1e :4 0 fb 1ca
1cc :3 0 fe 1a2
1ce :2 0 1d0 101
1d1 19f 1d0 0
1d2 103 0 1dd
34 :3 0 4c :3 0
1d3 1d4 0 8
:3 0 3b :3 0 1d6
1d7 0 c :3 0
36 :3 0 105 1d5
1db :2 0 1dd 109
27e 8 :3 0 4d
:2 0 4 1de 1df
0 13 :3 0 14
:3 0 1e1 1e2 0
13 :3 0 21 :3 0
1e4 1e5 0 3f
:2 0 2 :4 0 4e
:2 0 24 :2 0 22
:2 0 114 1ea 1ec
:3 0 117 1e6 1ee
1e3 1f0 1ef :2 0
16 :3 0 25 :3 0
1f2 1f3 0 18
:4 0 4f :4 0 1b
:2 0 1c :3 0 11b
1f7 1f9 :3 0 11e
1f4 1fb :2 0 214
16 :3 0 25 :3 0
1fd 1fe 0 18
:4 0 50 :4 0 1b
:2 0 8 :3 0 51
:3 0 203 204 0
121 202 206 :3 0
1b :2 0 4b :4 0
124 208 20a :3 0
1b :2 0 8 :3 0
52 :3 0 20d 20e
0 127 20c 210
:3 0 12a 1ff 212
:2 0 214 12d 215
1f1 214 0 216
130 0 226 53
:3 0 12 :3 0 8
:3 0 51 :3 0 54
:3 0 8 :3 0 52
:3 0 3 :3 0 55
:4 0 56 1 :a 0
222 :2 0 226 218
220 :2 0 7 :3 0
12 :3 0 224 :2 0
226 132 228 136
227 226 :2 0 27c
3e :3 0 8 :3 0
3b :3 0 22b 22c
0 3c :3 0 22d
22e 0 274 13
:3 0 14 :3 0 230
231 0 13 :3 0
21 :3 0 233 234
0 3f :2 0 2
:4 0 57 :2 0 24
:2 0 22 :2 0 138
239 23b :3 0 13b
235 23d 232 23f
23e :2 0 16 :3 0
25 :3 0 241 242
0 18 :4 0 58
:4 0 1b :2 0 1c
:3 0 13f 246 248
:3 0 142 243 24a
:2 0 264 16 :3 0
25 :3 0 24c 24d
0 18 :4 0 42
:4 0 1b :2 0 3c
:3 0 145 251 253
:3 0 1b :2 0 43
:4 0 148 255 257
:3 0 1b :2 0 44
:3 0 45 :3 0 22
:2 0 46 :2 0 14b
25a 25e 14f 259
260 :3 0 152 24e
262 :2 0 264 155
265 240 264 0
266 158 0 274
53 :3 0 12 :3 0
54 :3 0 8 :3 0
52 :3 0 3 :3 0
55 :4 0 59 1
:a 0 270 :2 0 274
268 26e :2 0 7
:3 0 12 :3 0 272
:2 0 274 15a 277
:3 0 277 0 277
276 274 275 :6 0
279 1 :3 0 15f
27b 161 27a 279
:2 0 27c 163 :2 0
27e 166 27e 27d
1dd 27c :6 0 27f
:2 0 2 14 27e
282 :3 0 281 27f
283 :8 0 
16d
4
:3 0 1 3 1
7 1 b 3
6 a e 1
18 1 16 1
1f 1 1d 1
26 1 24 1
2d 1 2b 1
32 1 38 1
45 1 51 2
54 56 2 59
5b 2 5d 5f
2 61 63 2
65 67 2 69
6b 2 6d 6f
2 71 73 2
75 77 2 79
7b 2 7d 7f
2 81 83 2
85 87 4 4d
53 58 89 1
8b 1 8d 2
97 99 3 95
96 9b 2 a4
a6 2 a3 a8
1 aa 1 ac
1 ae 1 b3
1 ba 2 ca
cc 3 c8 c9
ce 2 d7 d9
2 db dd 2
df e1 1 e9
2 eb ec 2
e3 ee 2 d6
f0 1 f2 1
f4 2 fe 100
3 fc fd 102
2 10b 10d 2
10a 10f 1 111
1 113 2 11a
11b 3 f5 114
11d 1 11f 2
12f 134 1 13b
1 136 2 14e
150 3 14c 14d
152 2 15b 15d
2 15f 161 2
163 165 2 167
169 2 16b 16d
3 172 173 174
2 16f 176 2
15a 178 1 17a
1 17c 2 145
17d 1 181 1
140 2 13e 185
2 197 199 3
195 196 19b 2
1a4 1a6 2 1a8
1aa 2 1ac 1ae
1 1b6 2 1b8
1b9 2 1b0 1bb
2 1bd 1bf 2
1c1 1c3 2 1c5
1c7 2 1c9 1cb
2 1a3 1cd 1
1cf 1 1d1 3
1d8 1d9 1da a
8e ad 120 123
126 129 188 18e
1d2 1dc 2 1e9
1eb 3 1e7 1e8
1ed 2 1f6 1f8
2 1f5 1fa 2
201 205 2 207
209 2 20b 20f
2 200 211 2
1fc 213 1 215
3 216 221 225
1 1e0 2 238
23a 3 236 237
23c 2 245 247
2 244 249 2
250 252 2 254
256 3 25b 25c
25d 2 258 25f
2 24f 261 2
24b 263 1 265
4 22f 266 26f
273 1 277 1
22a 2 228 27b
6 1b 22 29
30 36 3d 
1
4
0 
282
0
1
14
4
a
0 1 2 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 
b 1 0
16 1 0
2b 1 0
3 1 0
7 1 0
32 1 0
1d 1 0
24 1 0
2 0 1
38 1 0
0

/
SHOW ERRORS FUNCTION SP_A_REQRES;
ALTER FUNCTION SP_A_REQRES COMPILE DEBUG;
/*
#Stored Procedure  :SP_A_POOLRES
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_A_POOLRES ==
CREATE OR REPLACE FUNCTION SP_A_POOLRES wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
5b
2 :e:
1FUNCTION:
1SP_A_POOLRES:
1ID:
1VARCHAR2:
1RES_ID001:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_ELEMENT_ID:
132:
1H_RA_ID:
1H_REC_USER:
1H_RESOURCE_ID:
1H_TABLE_TYPE:
1PLS_INTEGER:
10:
1H_TYPE_ID:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_A_POOLRES(:
1':
1, :
1);:
1CHECK_TRACE:
11:
124:
1+:
1FPUT:
1               ENTER:: SP_A_POOLRES(25) :: cplan.sql(6627) :: :
1IS NULL:
18:
129:
1      RETURN_INVALID:: SP_A_POOLRES(30) :: cplan.sql(6632) :: :
1 :
1STATE:: :
1COALESCE:
1TO_CHAR:
1PMO_CNST:
1INVALID_PARAMS:
1NULL:
1                EXIT:: SP_A_POOLRES(33) :: cplan.sql(6635) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1SC_TABLE_POL:
1TYPE_ID:
1TMT_POOLS:
1ELEMENT_ID:
1SELECT TYPE_ID INTO h_type_id FROM TMT_POOLS WHERE ELEMENT_ID = h_element_id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
154:
1           EXCEPTION:: SP_A_POOLRES(55) :: cplan.sql(6657) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SYS_GUID:
1ASSIGNRESOURCE:
14:
165:
1       RETURN_STATUS:: SP_A_POOLRES(66) :: cplan.sql(6668) :: :
1 ID:: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
172:
1           EXCEPTION:: SP_A_POOLRES(73) :: cplan.sql(6675) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
184:
1           EXCEPTION:: SP_A_POOLRES(85) :: cplan.sql(6687) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
29e
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 :2 a0 6b 1c
81 b0 :2 a0 6b :2 a0 6b 6e a5
b a 10 :2 a0 6b 6e :2 a0 6b
6e a5 b 6e 7e a0 b4 2e
6e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c a0 7e b4 2e :2 5a a0 7e
b4 2e :2 5a 52 10 a0 7e b4
2e :2 5a 52 10 :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
a0 a5 57 b7 19 3c :2 a0 d
:2 a0 d :2 a0 d :3 a0 6b d :5 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 b4 2e d :2 a0 6b :7 a0
a5 b d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 b7
:2 a0 6b :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 :2 a0 6b :2 6e 7e :2 a0 6b
b4 2e 7e 6e b4 2e 7e :2 a0
6b b4 2e a5 57 b7 19 3c
:9 a0 12a e9 dd :2 a0 65 b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c :7 a0 12a
e9 dd :2 a0 65 b7 a4 b1 11
4f b7 a6 9 a4 b1 11 68
4f 1d 17 b5 
29e
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 30
56 5a 5e 62 65 69 86 71
75 78 79 81 70 a3 91 6d
95 96 9e 90 c0 ae 8d b2
b3 bb ad dd cb aa cf d0
d8 ca f9 e8 ec c7 f4 e7
115 104 108 e4 110 103 135 120
124 100 128 130 11f 13c 140 11c
144 148 14c 14f 154 155 1 157
15c 160 164 167 16c 170 174 177
17c 17d 17f 184 187 18b 18c 191
196 199 19e 19f 1a4 1a7 1ab 1ac
1b1 1b4 1b9 1ba 1bf 1c2 1c7 1c8
1cd 1d0 1d5 1d6 1db 1de 1e2 1e3
1e8 1eb 1f0 1f1 1f6 1f9 1fe 1ff
204 207 20c 20d 212 215 219 21a
21f 222 227 228 22d 230 235 236
23b 23c 241 243 247 24a 24e 252
255 259 25d 260 263 268 26b 26e
271 272 277 278 1 27a 27f 283
287 28a 28f 294 297 29b 29c 2a1
2a2 2a7 2a9 2ad 2b0 2b4 2b7 2b8
2bd 2c0 2c3 2c7 2ca 2cb 2d0 2d3
1 2d6 2db 2df 2e2 2e3 2e8 2eb
1 2ee 2f3 2f7 2fb 2fe 302 306
309 30c 311 314 317 31a 31b 320
321 1 323 328 32c 330 333 338
33d 340 344 345 34a 34d 352 353
358 35b 360 361 366 369 36d 371
375 379 37c 37d 37f 384 385 387
388 38d 38e 393 395 399 39c 3a0
3a4 3a7 3ab 3af 3b2 3b5 3ba 3bd
3c0 3c3 3c4 3c9 3ca 1 3cc 3d1
3d5 3d9 3dc 3e1 3e6 3e9 3ed 3ee
3f3 3f4 3f9 3fb 3ff 402 406 40a
40d 411 415 418 41c 41d 422 424
428 42b 42f 433 437 43b 43f 443
447 44b 44f 453 457 45b 45e 462
466 46a 46e 472 476 482 486 48a
48d 491 495 497 49b 49f 4a3 4a6
4aa 4ae 4b0 4b1 4b6 1 4ba 4be
4c2 4c5 4c9 4cd 4d1 4d5 4d8 4dc
4e0 4e3 4e6 4eb 4ee 4f1 4f4 4f5
4fa 4fb 1 4fd 502 506 50a 50d
512 517 51a 51e 51f 524 527 52c
52d 532 535 53a 53b 540 543 547
548 54d 550 555 556 55b 55e 562
566 569 56c 56d 56f 570 575 576
57b 57d 581 584 586 58a 58c 598
59a 59c 59d 5a2 5a6 5a8 5b4 5b6
5ba 5be 5bf 5c4 5c8 5cc 5d0 5d3
5d7 5db 5df 5e3 5e7 5eb 5ef 5f0
5f2 5f6 5fa 5fe 601 605 609 60c
60f 614 617 61a 61d 61e 623 624
1 626 62b 62f 633 636 63b 640
643 647 648 64d 650 655 656 65b
65e 663 664 669 66c 670 674 678
67c 67f 680 682 687 688 68a 68b
690 693 698 699 69e 6a1 6a6 6a7
6ac 6af 6b3 6b4 6b9 6bc 6c1 6c2
6c7 6c8 6cd 6cf 6d3 6d6 6da 6de
6e1 6e5 6e9 6ec 6f0 6f4 6f5 6fa
6fc 700 704 707 70b 70f 712 716
71a 71d 720 725 728 72b 72e 72f
734 735 1 737 73c 740 744 747
74c 751 754 758 759 75e 75f 764
768 76c 76f 774 779 77c 780 784
787 788 78d 790 795 796 79b 79e
7a2 7a6 7a9 7aa 7af 7b0 7b5 7b7
7bb 7be 7c2 7c6 7ca 7ce 7d2 7d6
7da 7de 7e2 7ee 7f3 7f7 7fb 7ff
803 805 806 80b 1 80f 813 817
81a 81e 822 826 82a 82d 831 835
838 83b 840 843 846 849 84a 84f
850 1 852 857 85b 85f 862 867
86c 86f 873 874 879 87a 87f 883
887 88a 88f 894 897 89b 89c 8a1
8a4 8a9 8aa 8af 8b2 8b6 8ba 8bd
8c0 8c1 8c3 8c4 8c9 8ca 8cf 8d1
8d5 8d8 8dc 8e0 8e4 8e8 8ec 8f0
8f4 900 905 909 90d 911 915 917
91b 91d 929 92b 92d 92e 933 937
939 945 949 94b 94c 955 
29e
2
0 1 a 6 9 :3 6 10 :3 6
12 :2 6 1 3 a 13 a :2 1
4 11 1a 19 :2 11 :2 4 c 15
14 :2 c :2 4 f 18 17 :2 f :2 4
12 1b 1a :2 12 :2 4 :2 11 20 11
:2 4 :2 e 1d e :2 4 e 17 :3 e
4 7 :2 11 23 :2 2d 3a :2 23 :3 7
:2 d 19 25 :2 2f 43 :2 25 54 5a
5d :2 54 6b 81 8 :2 6b d 10
:2 6b 13 16 :2 6b b e :2 6b 13
16 :2 6b 1b 1e :2 6b 28 2b :2 6b
b e :2 6b 13 16 :2 6b 1b 1e
:2 6b 2a 2d :2 6b 32 35 :2 6b :2 7
4a :2 4 7 :2 11 23 :2 2d 39 3c
4c 4e 4f :2 4c :2 23 :3 7 :2 d 12
1e 5c 5f :2 1e :2 7 52 :2 4 :4 9
8 7 :4 1b 1a 19 :2 7 :4 34 33
32 :2 7 a :2 14 26 :2 30 3c 3f
4f 51 52 :2 4f :2 26 :3 a :2 10 15
21 5f 62 :2 21 6f 72 :2 21 76
79 :2 21 83 86 8f 97 :2 a0 :2 8f
b1 :2 86 :2 21 :2 a 55 :2 7 a :2 14
26 :2 30 3c 3f 4f 51 52 :2 4f
:2 26 :3 a :2 10 15 21 5f 62 :2 21
:2 a 55 :3 7 :2 12 28 :2 31 41 :2 7
4a :3 4 14 :2 4 15 :2 4 12 :2 4
14 :2 1d 4 e 1b 2a 3a 47
:2 7 :2 10 1c 7 4 c a :2 13
1f a 1a :2 7 :2 c a :2 13 1f
a d :2 17 29 :2 33 3f 43 53
55 56 :2 53 :2 29 :3 d :2 13 18 24
62 65 :2 24 72 75 :2 24 79 7c
:2 24 88 8b :2 24 93 96 :2 24 a2
a5 ac b5 b8 :2 a5 :2 24 :2 d 59
:2 a :2 7 :4 13 :2 7 4 :3 1 4 :3 f
:2 4 :2 d 19 28 36 41 4f 5e
27 :2 19 4 7 :2 11 23 :2 2d 39
3c 4c 4e 4f :2 4c :2 23 :3 7 :2 d
12 1e 5c 5f :2 1e 6c 6f :2 1e
73 76 :2 1e 80 83 8c 94 :2 9d
:2 8c a8 :2 83 :2 1e b0 b3 :2 1e bb
be :2 1e c3 c6 :2 1e ce d1 :2 1e
:2 7 52 :3 4 :2 f 21 :2 2a 34 3d
:2 4 1 9 12 9 a :2 14 26
:2 30 3c 40 50 52 53 :2 50 :2 26
:3 a :2 10 15 21 5f 62 :2 21 :3 a
:2 10 15 39 5c 5f :2 68 :2 39 76
79 :2 39 81 84 :2 8d :2 39 :2 a 56
:3 7 c 15 1e 2c a 13 1e
f e 7 e 7 e 7 29
:2 4 :2 9 7 :2 10 1c :2 7 :2 11 23
:2 2d 39 3d 4d 4f 50 :2 4d :2 23
:3 7 :2 d 12 1e 5c 5f :2 1e :3 7
:2 d 12 36 42 45 :2 36 4d 50
:2 36 5c 5f 66 6f 72 :2 5f :2 36
:2 7 53 :2 4 7 c 1b a 13
1e f e 7 e 7 e 7
:2 4 :4 10 :2 4 :8 1 
29e
4
0 :2 1 :4 3 :4 4
:4 5 2 :4 6 :2 1
:7 8 :7 9 :7 a :7 b
:6 c :6 d :7 f :b 11
:11 12 13 :2 12 :2 13
:2 12 :2 13 :2 12 :2 14
:2 12 :2 14 :2 12 :2 14
:2 12 :2 14 :2 12 :2 15
:2 12 :2 15 :2 12 :2 15
:2 12 :2 15 :2 12 :2 15
:4 12 :3 11 :11 18 :b 19
:3 18 :16 1c :11 1d :20 1e
:3 1d :11 20 :b 21 :3 20
:9 23 :3 1c :3 28 :3 29
:3 2a :5 2b :6 2e :5 2f
2d 31 :5 32 :3 31
:2 33 :5 35 :11 36 :24 37
:3 36 :2 34 :6 33 30
:3 10 :5 3c :9 3e 3f
:3 3e :11 41 :30 42 :3 41
:a 44 10 :3 47 :11 48
:b 49 :17 4a :3 48 :2 4c
:3 4d :3 4e 4f 4d
4c 4d :3 50 :3 47
:2 51 :5 53 :11 54 :b 55
:18 56 :3 54 :2 58 59
:3 5a 5b 59 58
59 :3 5c :2 52 :6 51
46 :7 1 
957
4
:3 0 1 :3 0 2
:a 0 299 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
:2 0 7 4 :3 0
5 :7 0 9 8
:3 0 4 :3 0 6
:7 0 d c :3 0
7 :3 0 8 :3 0
9 :2 0 4 11
12 0 f 13
0 299 2 14
:2 0 b :2 0 f
4 :3 0 b :2 0
d 17 19 :6 0
1c 1a 0 297
0 a :6 0 b
:2 0 13 4 :3 0
11 1e 20 :6 0
23 21 0 297
0 c :6 0 b
:2 0 17 4 :3 0
15 25 27 :6 0
2a 28 0 297
0 d :6 0 11
:2 0 1b 4 :3 0
19 2c 2e :6 0
31 2f 0 297
0 e :6 0 11
:2 0 1d 10 :3 0
33 :7 0 37 34
35 297 0 f
:6 0 3f 40 0
1f 10 :3 0 39
:7 0 3d 3a 3b
297 0 12 :6 0
45 46 0 21
8 :3 0 9 :2 0
4 41 :7 0 44
42 0 297 0
13 :6 0 14 :3 0
15 :3 0 14 :3 0
16 :3 0 48 49
0 2 :4 0 23
4a 4c 47 4e
4d :2 0 17 :3 0
18 :3 0 50 51
0 19 :4 0 14
:3 0 1a :3 0 54
55 0 2 :4 0
25 56 58 1b
:4 0 1c :2 0 1d
:3 0 27 5b 5d
:3 0 1e :4 0 1c
:2 0 1f :4 0 2a
60 62 :3 0 1c
:2 0 3 :3 0 2d
64 66 :3 0 1c
:2 0 1f :4 0 30
68 6a :3 0 1c
:2 0 20 :4 0 33
6c 6e :3 0 1c
:2 0 1f :4 0 36
70 72 :3 0 1c
:2 0 5 :3 0 39
74 76 :3 0 1c
:2 0 1f :4 0 3c
78 7a :3 0 1c
:2 0 20 :4 0 3f
7c 7e :3 0 1c
:2 0 1f :4 0 42
80 82 :3 0 1c
:2 0 6 :3 0 45
84 86 :3 0 1c
:2 0 1f :4 0 48
88 8a :3 0 1c
:2 0 21 :4 0 4b
8c 8e :3 0 4e
52 90 :2 0 92
53 93 4f 92
0 94 55 0
1f7 14 :3 0 15
:3 0 95 96 0
14 :3 0 22 :3 0
98 99 0 23
:2 0 2 :4 0 24
:2 0 25 :2 0 23
:2 0 57 9e a0
:3 0 5a 9a a2
97 a4 a3 :2 0
17 :3 0 26 :3 0
a6 a7 0 19
:4 0 27 :4 0 1c
:2 0 1d :3 0 5e
ab ad :3 0 61
a8 af :2 0 b1
64 b2 a5 b1
0 b3 66 0
1f7 3 :3 0 28
:2 0 68 b5 b6
:3 0 b7 :2 0 b8
:2 0 5 :3 0 28
:2 0 6a bb bc
:3 0 bd :2 0 be
:2 0 b9 c0 bf
:2 0 6 :3 0 28
:2 0 6c c3 c4
:3 0 c5 :2 0 c6
:2 0 c1 c8 c7
:2 0 14 :3 0 15
:3 0 ca cb 0
14 :3 0 22 :3 0
cd ce 0 29
:2 0 2 :4 0 2a
:2 0 25 :2 0 23
:2 0 6e d3 d5
:3 0 71 cf d7
cc d9 d8 :2 0
17 :3 0 26 :3 0
db dc 0 19
:4 0 2b :4 0 1c
:2 0 1d :3 0 75
e0 e2 :3 0 1c
:2 0 2c :4 0 78
e4 e6 :3 0 1c
:2 0 2d :4 0 7b
e8 ea :3 0 1c
:2 0 2e :3 0 2f
:3 0 30 :3 0 31
:3 0 ef f0 0
7e ee f2 32
:4 0 80 ed f5
83 ec f7 :3 0
86 dd f9 :2 0
fb 89 fc da
fb 0 fd 8b
0 126 14 :3 0
15 :3 0 fe ff
0 14 :3 0 22
:3 0 101 102 0
23 :2 0 2 :4 0
b :2 0 25 :2 0
23 :2 0 8d 107
109 :3 0 90 103
10b 100 10d 10c
:2 0 17 :3 0 26
:3 0 10f 110 0
19 :4 0 33 :4 0
1c :2 0 1d :3 0
94 114 116 :3 0
97 111 118 :2 0
11a 9a 11b 10e
11a 0 11c 9c
0 126 34 :3 0
35 :3 0 11d 11e
0 30 :3 0 31
:3 0 120 121 0
36 :3 0 9e 11f
124 :2 0 126 a1
127 c9 126 0
128 a5 0 1f7
a :3 0 3 :3 0
129 12a 0 1f7
e :3 0 5 :3 0
12c 12d 0 1f7
d :3 0 6 :3 0
12f 130 0 1f7
f :3 0 30 :3 0
37 :3 0 133 134
0 132 135 0
1f7 38 :3 0 12
:3 0 39 :3 0 3a
:3 0 a :4 0 3b
1 :8 0 142 8
:3 0 3c :3 0 13d
13e 0 3d :3 0
13f 140 0 142
a7 195 3e :3 0
8 :3 0 3c :3 0
144 145 0 3d
:3 0 146 147 0
149 aa 14b ac
14a 149 :2 0 193
3f :3 0 8 :3 0
3c :3 0 14e 14f
0 3d :3 0 150
151 0 18b 14
:3 0 15 :3 0 153
154 0 14 :3 0
22 :3 0 156 157
0 40 :2 0 2
:4 0 41 :2 0 25
:2 0 23 :2 0 ae
15c 15e :3 0 b1
158 160 155 162
161 :2 0 17 :3 0
26 :3 0 164 165
0 19 :4 0 42
:4 0 1c :2 0 1d
:3 0 b5 169 16b
:3 0 1c :2 0 2c
:4 0 b8 16d 16f
:3 0 1c :2 0 43
:4 0 bb 171 173
:3 0 1c :2 0 3d
:3 0 be 175 177
:3 0 1c :2 0 44
:4 0 c1 179 17b
:3 0 1c :2 0 45
:3 0 46 :3 0 23
:2 0 47 :2 0 c4
17e 182 c8 17d
184 :3 0 cb 166
186 :2 0 188 ce
189 163 188 0
18a d0 0 18b
d2 18e :3 0 18e
0 18e 18d 18b
18c :6 0 190 2
:3 0 d5 192 d7
191 190 :2 0 193
d9 :2 0 195 0
195 194 142 193
:6 0 1f7 1 :3 0
c :3 0 48 :4 0
198 199 :3 0 197
19a 0 1f7 8
:3 0 3c :3 0 19c
19d 0 49 :3 0
f :3 0 12 :3 0
a :3 0 e :3 0
d :3 0 c :3 0
dc 19f 1a6 19e
1a7 0 1f7 14
:3 0 15 :3 0 1a9
1aa 0 14 :3 0
22 :3 0 1ac 1ad
0 4a :2 0 2
:4 0 4b :2 0 25
:2 0 23 :2 0 e3
1b2 1b4 :3 0 e6
1ae 1b6 1ab 1b8
1b7 :2 0 17 :3 0
26 :3 0 1ba 1bb
0 19 :4 0 4c
:4 0 1c :2 0 1d
:3 0 ea 1bf 1c1
:3 0 1c :2 0 2c
:4 0 ed 1c3 1c5
:3 0 1c :2 0 2d
:4 0 f0 1c7 1c9
:3 0 1c :2 0 2e
:3 0 2f :3 0 8
:3 0 3c :3 0 1ce
1cf 0 f3 1cd
1d1 32 :4 0 f5
1cc 1d4 f8 1cb
1d6 :3 0 1c :2 0
4d :4 0 fb 1d8
1da :3 0 1c :2 0
1f :4 0 fe 1dc
1de :3 0 1c :2 0
c :3 0 101 1e0
1e2 :3 0 1c :2 0
1f :4 0 104 1e4
1e6 :3 0 107 1bc
1e8 :2 0 1ea 10a
1eb 1b9 1ea 0
1ec 10c 0 1f7
34 :3 0 4e :3 0
1ed 1ee 0 8
:3 0 3c :3 0 1f0
1f1 0 c :3 0
36 :3 0 10e 1ef
1f5 :2 0 1f7 112
298 8 :3 0 4f
:2 0 4 1f8 1f9
0 14 :3 0 15
:3 0 1fb 1fc 0
14 :3 0 22 :3 0
1fe 1ff 0 40
:2 0 2 :4 0 50
:2 0 25 :2 0 23
:2 0 11f 204 206
:3 0 122 200 208
1fd 20a 209 :2 0
17 :3 0 26 :3 0
20c 20d 0 19
:4 0 51 :4 0 1c
:2 0 1d :3 0 126
211 213 :3 0 129
20e 215 :2 0 22e
17 :3 0 26 :3 0
217 218 0 19
:4 0 52 :4 0 1c
:2 0 8 :3 0 53
:3 0 21d 21e 0
12c 21c 220 :3 0
1c :2 0 4d :4 0
12f 222 224 :3 0
1c :2 0 8 :3 0
54 :3 0 227 228
0 132 226 22a
:3 0 135 219 22c
:2 0 22e 138 22f
20b 22e 0 230
13b 0 240 55
:3 0 13 :3 0 8
:3 0 53 :3 0 56
:3 0 8 :3 0 54
:3 0 3 :3 0 57
:4 0 58 1 :a 0
23c :2 0 240 232
23a :2 0 7 :3 0
13 :3 0 23e :2 0
240 13d 242 141
241 240 :2 0 296
3f :3 0 8 :3 0
3c :3 0 245 246
0 3d :3 0 247
248 0 28e 14
:3 0 15 :3 0 24a
24b 0 14 :3 0
22 :3 0 24d 24e
0 40 :2 0 2
:4 0 59 :2 0 25
:2 0 23 :2 0 143
253 255 :3 0 146
24f 257 24c 259
258 :2 0 17 :3 0
26 :3 0 25b 25c
0 19 :4 0 5a
:4 0 1c :2 0 1d
:3 0 14a 260 262
:3 0 14d 25d 264
:2 0 27e 17 :3 0
26 :3 0 266 267
0 19 :4 0 43
:4 0 1c :2 0 3d
:3 0 150 26b 26d
:3 0 1c :2 0 44
:4 0 153 26f 271
:3 0 1c :2 0 45
:3 0 46 :3 0 23
:2 0 47 :2 0 156
274 278 15a 273
27a :3 0 15d 268
27c :2 0 27e 160
27f 25a 27e 0
280 163 0 28e
55 :3 0 13 :3 0
56 :3 0 8 :3 0
54 :3 0 3 :3 0
57 :4 0 5b 1
:a 0 28a :2 0 28e
282 288 :2 0 7
:3 0 13 :3 0 28c
:2 0 28e 165 291
:3 0 291 0 291
290 28e 28f :6 0
293 1 :3 0 16a
295 16c 294 293
:2 0 296 16e :2 0
298 171 298 297
1f7 296 :6 0 299
:2 0 2 14 298
29c :3 0 29b 299
29d :8 0 
179
4
:3 0 1 3 1
7 1 b 3
6 a e 1
18 1 16 1
1f 1 1d 1
26 1 24 1
2d 1 2b 1
32 1 38 1
3e 1 4b 1
57 2 5a 5c
2 5f 61 2
63 65 2 67
69 2 6b 6d
2 6f 71 2
73 75 2 77
79 2 7b 7d
2 7f 81 2
83 85 2 87
89 2 8b 8d
4 53 59 5e
8f 1 91 1
93 2 9d 9f
3 9b 9c a1
2 aa ac 2
a9 ae 1 b0
1 b2 1 b4
1 ba 1 c2
2 d2 d4 3
d0 d1 d6 2
df e1 2 e3
e5 2 e7 e9
1 f1 2 f3
f4 2 eb f6
2 de f8 1
fa 1 fc 2
106 108 3 104
105 10a 2 113
115 2 112 117
1 119 1 11b
2 122 123 3
fd 11c 125 1
127 2 13c 141
1 148 1 143
2 15b 15d 3
159 15a 15f 2
168 16a 2 16c
16e 2 170 172
2 174 176 2
178 17a 3 17f
180 181 2 17c
183 2 167 185
1 187 1 189
2 152 18a 1
18e 1 14d 2
14b 192 6 1a0
1a1 1a2 1a3 1a4
1a5 2 1b1 1b3
3 1af 1b0 1b5
2 1be 1c0 2
1c2 1c4 2 1c6
1c8 1 1d0 2
1d2 1d3 2 1ca
1d5 2 1d7 1d9
2 1db 1dd 2
1df 1e1 2 1e3
1e5 2 1bd 1e7
1 1e9 1 1eb
3 1f2 1f3 1f4
c 94 b3 128
12b 12e 131 136
195 19b 1a8 1ec
1f6 2 203 205
3 201 202 207
2 210 212 2
20f 214 2 21b
21f 2 221 223
2 225 229 2
21a 22b 2 216
22d 1 22f 3
230 23b 23f 1
1fa 2 252 254
3 250 251 256
2 25f 261 2
25e 263 2 26a
26c 2 26e 270
3 275 276 277
2 272 279 2
269 27b 2 265
27d 1 27f 4
249 280 289 28d
1 291 1 244
2 242 295 7
1b 22 29 30
36 3c 43 
1
4
0 
29c
0
1
14
4
b
0 1 2 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 
b 1 0
16 1 0
2b 1 0
2 0 1
3 1 0
7 1 0
38 1 0
1d 1 0
32 1 0
24 1 0
3e 1 0
0

/
SHOW ERRORS FUNCTION SP_A_POOLRES;
ALTER FUNCTION SP_A_POOLRES COMPILE DEBUG;
/***************************************************************************************************/
/*
#Stored Procedure  :SP_A_DOCRES
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_A_DOCRES ==
CREATE OR REPLACE FUNCTION SP_A_DOCRES wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
7d
2 :e:
1FUNCTION:
1SP_A_DOCRES:
1TABLE_TYPE001:
1PLS_INTEGER:
1ID:
1VARCHAR2:
1RES_ID001:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_ELEMENT_ID:
132:
1H_RA_ID:
1H_REC_USER:
1H_RESOURCE_ID:
1H_TABLE_TYPE:
10:
1H_TYPE_ID:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_A_DOCRES(:
1COALESCE:
1TO_CHAR:
1NULL:
1, :
1':
1);:
1CHECK_TRACE:
11:
126:
1+:
1FPUT:
1               ENTER:: SP_A_DOCRES(27) :: cplan.sql(6740) :: :
1IS NULL:
18:
131:
1      RETURN_INVALID:: SP_A_DOCRES(32) :: cplan.sql(6745) :: :
1 :
1STATE:: :
1PMO_CNST:
1INVALID_PARAMS:
134:
1                EXIT:: SP_A_DOCRES(35) :: cplan.sql(6748) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1=:
1SC_TABLE_DOC:
1TYPE_ID:
1TMT_DOCUMENTS:
1ELEMENT_ID:
1SELECT TYPE_ID INTO h_type_id FROM TMT_DOCUMENTS WHERE ELEMENT_ID = h_element+
1_id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
157:
1           EXCEPTION:: SP_A_DOCRES(58) :: cplan.sql(6771) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1ELSIF:
1SC_TABLE_SCD:
1TMT_ASSIGNED_SCORECARDS:
1SELECT TYPE_ID INTO h_type_id FROM TMT_ASSIGNED_SCORECARDS WHERE ELEMENT_ID =+
1 h_element_id:
172:
1           EXCEPTION:: SP_A_DOCRES(73) :: cplan.sql(6786) :: :
1SC_TABLE_POL_DOC:
1TMT_POOL_DOCUMENTS:
1SELECT TYPE_ID INTO h_type_id FROM TMT_POOL_DOCUMENTS WHERE ELEMENT_ID = h_el+
1ement_id:
187:
1           EXCEPTION:: SP_A_DOCRES(88) :: cplan.sql(6801) :: :
1SC_TABLE_RES_DOC:
1TMT_RESOURCE_DOCUMENTS:
1SELECT TYPE_ID INTO h_type_id FROM TMT_RESOURCE_DOCUMENTS WHERE ELEMENT_ID = +
1h_element_id:
1102:
1           EXCEPTION:: SP_A_DOCRES(103) :: cplan.sql(6816) :: :
1SC_TABLE_CLN_DOC:
1TMT_CLIENT_DOCUMENTS:
1SELECT TYPE_ID INTO h_type_id FROM TMT_CLIENT_DOCUMENTS WHERE ELEMENT_ID = h_+
1element_id:
1117:
1           EXCEPTION:: SP_A_DOCRES(118) :: cplan.sql(6831) :: :
1SC_TABLE_AST_DOC:
1TMT_ASSET_DOCUMENTS:
1SELECT TYPE_ID INTO h_type_id FROM TMT_ASSET_DOCUMENTS WHERE ELEMENT_ID = h_e+
1lement_id:
1132:
1           EXCEPTION:: SP_A_DOCRES(133) :: cplan.sql(6846) :: :
1SC_TABLE_VDR_DOC:
1TMT_VENDOR_DOCUMENTS:
1SELECT TYPE_ID INTO h_type_id FROM TMT_VENDOR_DOCUMENTS WHERE ELEMENT_ID = h_+
1element_id:
1147:
1           EXCEPTION:: SP_A_DOCRES(148) :: cplan.sql(6861) :: :
1SYS_GUID:
1ASSIGNRESOURCE:
14:
1159:
1       RETURN_STATUS:: SP_A_DOCRES(160) :: cplan.sql(6873) :: :
1 ID:: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
1166:
1           EXCEPTION:: SP_A_DOCRES(167) :: cplan.sql(6880) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1178:
1           EXCEPTION:: SP_A_DOCRES(179) :: cplan.sql(6892) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
551
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d b4 :3 a0 6b 2c 6a
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 :2 a0 6b 1c 81 b0 :2 a0 6b
:2 a0 6b 6e a5 b a 10 :2 a0
6b 6e :2 a0 6b 6e a5 b 6e
7e a0 b4 2e 6e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c a0 7e b4 2e :2 5a a0
7e b4 2e :2 5a 52 10 a0 7e
b4 2e :2 5a 52 10 :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b a0 a5 57 b7 19 3c :2 a0
d :2 a0 d :2 a0 d :2 a0 d :2 a0
51 a5 b a0 7e a0 6b b4
2e :5 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f a0 b7 :2 a0 51 a5
b a0 7e a0 6b b4 2e :5 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f a0 b7 19 :2 a0 51 a5 b
a0 7e a0 6b b4 2e :5 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
a0 b7 19 :2 a0 51 a5 b a0
7e a0 6b b4 2e :5 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f a0
b7 19 :2 a0 51 a5 b a0 7e
a0 6b b4 2e :5 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f a0 b7
19 :2 a0 51 a5 b a0 7e a0
6b b4 2e :5 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f a0 b7 19
:2 a0 51 a5 b a0 7e a0 6b
b4 2e :5 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c :2 a0
b4 2e d :2 a0 6b :7 a0 a5 b
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b :2 a0 a5 57 b7 :2 a0 6b
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
:2 a0 6b :2 6e 7e :2 a0 6b b4 2e
7e 6e b4 2e 7e :2 a0 6b b4
2e a5 57 b7 19 3c :9 a0 12a
e9 dd :2 a0 65 b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c :7 a0 12a e9 dd
:2 a0 65 b7 a4 b1 11 4f b7
a6 9 a4 b1 11 68 4f 1d
17 b5 
551
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 59 6b 6f 73 56
77 7b 98 83 87 8a 8b 93
82 b5 a3 7f a7 a8 b0 a2
d2 c0 9f c4 c5 cd bf ef
dd bc e1 e2 ea dc 10b fa
fe d9 106 f9 127 116 11a f6
122 115 147 132 136 112 13a 142
131 14e 152 12e 156 15a 15e 161
166 167 1 169 16e 172 176 179
17e 182 186 189 18e 18f 191 196
199 19d 19e 1a3 1a8 1ab 1af 1b3
1b7 1b8 1ba 1bf 1c0 1c2 1c3 1c8
1cb 1d0 1d1 1d6 1d9 1de 1df 1e4
1e7 1eb 1ec 1f1 1f4 1f9 1fa 1ff
202 207 208 20d 210 215 216 21b
21e 222 223 228 22b 230 231 236
239 23e 23f 244 247 24c 24d 252
255 259 25a 25f 262 267 268 26d
270 275 276 27b 27c 281 283 287
28a 28e 292 295 299 29d 2a0 2a3
2a8 2ab 2ae 2b1 2b2 2b7 2b8 1
2ba 2bf 2c3 2c7 2ca 2cf 2d4 2d7
2db 2dc 2e1 2e2 2e7 2e9 2ed 2f0
2f4 2f7 2f8 2fd 300 303 307 30a
30b 310 313 1 316 31b 31f 322
323 328 32b 1 32e 333 337 33b
33e 342 346 349 34c 351 354 357
35a 35b 360 361 1 363 368 36c
370 373 378 37d 380 384 385 38a
38d 392 393 398 39b 3a0 3a1 3a6
3a9 3ad 3b1 3b5 3b9 3bc 3bd 3bf
3c4 3c5 3c7 3c8 3cd 3ce 3d3 3d5
3d9 3dc 3e0 3e4 3e7 3eb 3ef 3f2
3f5 3fa 3fd 400 403 404 409 40a
1 40c 411 415 419 41c 421 426
429 42d 42e 433 434 439 43b 43f
442 446 44a 44d 451 455 458 45c
45d 462 464 468 46b 46f 473 477
47b 47f 483 487 48b 48f 493 497
49b 49f 4a3 4a6 4a7 4a9 4ad 4b0
4b4 4b7 4b8 4bd 4c1 4c5 4c9 4cd
4d1 4dd 4e1 4e5 4e8 4ec 4f0 4f2
4f6 4fa 4fe 501 505 509 50b 50c
511 1 515 519 51d 520 524 528
52c 530 533 537 53b 53e 541 546
549 54c 54f 550 555 556 1 558
55d 561 565 568 56d 572 575 579
57a 57f 582 587 588 58d 590 595
596 59b 59e 5a2 5a3 5a8 5ab 5b0
5b1 5b6 5b9 5bd 5c1 5c4 5c7 5c8
5ca 5cb 5d0 5d1 5d6 5d8 5dc 5df
5e1 5e5 5e7 5f3 5f5 5f7 5f8 5fd
601 603 60f 611 615 617 61b 61f
622 623 625 629 62c 630 633 634
639 63d 641 645 649 64d 659 65d
661 664 668 66c 66e 672 676 67a
67d 681 685 687 688 68d 1 691
695 699 69c 6a0 6a4 6a8 6ac 6af
6b3 6b7 6ba 6bd 6c2 6c5 6c8 6cb
6cc 6d1 6d2 1 6d4 6d9 6dd 6e1
6e4 6e9 6ee 6f1 6f5 6f6 6fb 6fe
703 704 709 70c 711 712 717 71a
71e 71f 724 727 72c 72d 732 735
739 73d 740 743 744 746 747 74c
74d 752 754 758 75b 75d 761 763
76f 771 773 774 779 77d 77f 78b
78d 791 793 797 79b 79f 7a2 7a3
7a5 7a9 7ac 7b0 7b3 7b4 7b9 7bd
7c1 7c5 7c9 7cd 7d9 7dd 7e1 7e4
7e8 7ec 7ee 7f2 7f6 7fa 7fd 801
805 807 808 80d 1 811 815 819
81c 820 824 828 82c 82f 833 837
83a 83d 842 845 848 84b 84c 851
852 1 854 859 85d 861 864 869
86e 871 875 876 87b 87e 883 884
889 88c 891 892 897 89a 89e 89f
8a4 8a7 8ac 8ad 8b2 8b5 8b9 8bd
8c0 8c3 8c4 8c6 8c7 8cc 8cd 8d2
8d4 8d8 8db 8dd 8e1 8e3 8ef 8f1
8f3 8f4 8f9 8fd 8ff 90b 90d 911
913 917 91b 91f 922 923 925 929
92c 930 933 934 939 93d 941 945
949 94d 959 95d 961 964 968 96c
96e 972 976 97a 97d 981 985 987
988 98d 1 991 995 999 99c 9a0
9a4 9a8 9ac 9af 9b3 9b7 9ba 9bd
9c2 9c5 9c8 9cb 9cc 9d1 9d2 1
9d4 9d9 9dd 9e1 9e4 9e9 9ee 9f1
9f5 9f6 9fb 9fe a03 a04 a09 a0c
a11 a12 a17 a1a a1e a1f a24 a27
a2c a2d a32 a35 a39 a3d a40 a43
a44 a46 a47 a4c a4d a52 a54 a58
a5b a5d a61 a63 a6f a71 a73 a74
a79 a7d a7f a8b a8d a91 a93 a97
a9b a9f aa2 aa3 aa5 aa9 aac ab0
ab3 ab4 ab9 abd ac1 ac5 ac9 acd
ad9 add ae1 ae4 ae8 aec aee af2
af6 afa afd b01 b05 b07 b08 b0d
1 b11 b15 b19 b1c b20 b24 b28
b2c b2f b33 b37 b3a b3d b42 b45
b48 b4b b4c b51 b52 1 b54 b59
b5d b61 b64 b69 b6e b71 b75 b76
b7b b7e b83 b84 b89 b8c b91 b92
b97 b9a b9e b9f ba4 ba7 bac bad
bb2 bb5 bb9 bbd bc0 bc3 bc4 bc6
bc7 bcc bcd bd2 bd4 bd8 bdb bdd
be1 be3 bef bf1 bf3 bf4 bf9 bfd
bff c0b c0d c11 c13 c17 c1b c1f
c22 c23 c25 c29 c2c c30 c33 c34
c39 c3d c41 c45 c49 c4d c59 c5d
c61 c64 c68 c6c c6e c72 c76 c7a
c7d c81 c85 c87 c88 c8d 1 c91
c95 c99 c9c ca0 ca4 ca8 cac caf
cb3 cb7 cba cbd cc2 cc5 cc8 ccb
ccc cd1 cd2 1 cd4 cd9 cdd ce1
ce4 ce9 cee cf1 cf5 cf6 cfb cfe
d03 d04 d09 d0c d11 d12 d17 d1a
d1e d1f d24 d27 d2c d2d d32 d35
d39 d3d d40 d43 d44 d46 d47 d4c
d4d d52 d54 d58 d5b d5d d61 d63
d6f d71 d73 d74 d79 d7d d7f d8b
d8d d91 d93 d97 d9b d9f da2 da3
da5 da9 dac db0 db3 db4 db9 dbd
dc1 dc5 dc9 dcd dd9 ddd de1 de4
de8 dec dee df2 df6 dfa dfd e01
e05 e07 e08 e0d 1 e11 e15 e19
e1c e20 e24 e28 e2c e2f e33 e37
e3a e3d e42 e45 e48 e4b e4c e51
e52 1 e54 e59 e5d e61 e64 e69
e6e e71 e75 e76 e7b e7e e83 e84
e89 e8c e91 e92 e97 e9a e9e e9f
ea4 ea7 eac ead eb2 eb5 eb9 ebd
ec0 ec3 ec4 ec6 ec7 ecc ecd ed2
ed4 ed8 edb edd ee1 ee3 eef ef1
ef3 ef4 ef9 efd eff f0b f0d f0f
f13 f17 f1a f1e f22 f23 f28 f2c
f30 f34 f37 f3b f3f f43 f47 f4b
f4f f53 f54 f56 f5a f5e f62 f65
f69 f6d f70 f73 f78 f7b f7e f81
f82 f87 f88 1 f8a f8f f93 f97
f9a f9f fa4 fa7 fab fac fb1 fb4
fb9 fba fbf fc2 fc7 fc8 fcd fd0
fd4 fd8 fdc fe0 fe3 fe4 fe6 feb
fec fee fef ff4 ff7 ffc ffd 1002
1005 100a 100b 1010 1013 1017 1018 101d
1020 1025 1026 102b 102c 1031 1033 1037
103a 103e 1042 1045 1049 104d 1050 1054
1058 1059 105e 1060 1064 1068 106b 106f
1073 1076 107a 107e 1081 1084 1089 108c
108f 1092 1093 1098 1099 1 109b 10a0
10a4 10a8 10ab 10b0 10b5 10b8 10bc 10bd
10c2 10c3 10c8 10cc 10d0 10d3 10d8 10dd
10e0 10e4 10e8 10eb 10ec 10f1 10f4 10f9
10fa 10ff 1102 1106 110a 110d 110e 1113
1114 1119 111b 111f 1122 1126 112a 112e
1132 1136 113a 113e 1142 1146 1152 1157
115b 115f 1163 1167 1169 116a 116f 1
1173 1177 117b 117e 1182 1186 118a 118e
1191 1195 1199 119c 119f 11a4 11a7 11aa
11ad 11ae 11b3 11b4 1 11b6 11bb 11bf
11c3 11c6 11cb 11d0 11d3 11d7 11d8 11dd
11de 11e3 11e7 11eb 11ee 11f3 11f8 11fb
11ff 1200 1205 1208 120d 120e 1213 1216
121a 121e 1221 1224 1225 1227 1228 122d
122e 1233 1235 1239 123c 1240 1244 1248
124c 1250 1254 1258 1264 1269 126d 1271
1275 1279 127b 127f 1281 128d 128f 1291
1292 1297 129b 129d 12a9 12ad 12af 12b0
12b9 
551
2
0 1 a 6 14 :3 6 9 :3 6
10 :3 6 12 :2 6 1 3 a 13
a :2 1 4 11 1a 19 :2 11 :2 4
c 15 14 :2 c :2 4 f 18 17
:2 f :2 4 12 1b 1a :2 12 :2 4 :2 11
20 11 :2 4 :2 e 1d e :2 4 e
17 :3 e 4 7 :2 11 23 :2 2d 3a
:2 23 :3 7 :2 d 19 25 :2 2f 43 :2 25
53 59 5c :2 53 6a 7f 8 11
19 :2 11 29 :2 8 :2 6a b e :2 6a
13 16 :2 6a 1b 1e :2 6a 21 24
:2 6a b e :2 6a 13 16 :2 6a 1b
1e :2 6a 28 2b :2 6a b e :2 6a
13 16 :2 6a 1b 1e :2 6a 2a 2d
:2 6a 32 35 :2 6a :2 7 49 :2 4 7
:2 11 23 :2 2d 39 3c 4b 4d 4e
:2 4b :2 23 :3 7 :2 d 12 1e 5b 5e
:2 1e :2 7 51 :2 4 :4 9 8 7 :4 1b
1a 19 :2 7 :4 34 33 32 :2 7 a
:2 14 26 :2 30 3c 3f 4e 50 51
:2 4e :2 26 :3 a :2 10 15 21 5e 61
:2 21 6e 71 :2 21 75 78 :2 21 82
85 8e 96 :2 9f :2 8e b0 :2 85 :2 21
:2 a 54 :2 7 a :2 14 26 :2 30 3c
3f 4e 50 51 :2 4e :2 26 :3 a :2 10
15 21 5e 61 :2 21 :2 a 54 :3 7
:2 12 28 :2 31 41 :2 7 4a :3 4 14
:2 4 15 :2 4 12 :2 4 14 4 7
10 1e :2 7 23 21 :2 2c :2 21 11
1e 2d 41 4e :2 a :2 13 1f a
7 f d :2 16 22 d 1d :2 a
:2 f d :2 16 22 d 10 :2 1a 2c
:2 36 42 46 55 57 58 :2 55 :2 2c
:3 10 :2 16 1b 27 64 67 :2 27 74
77 :2 27 7b 7e :2 27 8a 8d :2 27
95 98 :2 27 a4 a7 ae b7 ba
:2 a7 :2 27 :2 10 5b :2 d :2 a :4 16 :2 a
7 :3 39 4 39 a 13 21 :2 a
26 24 :2 2f :2 24 11 1e 2d 4b
58 :2 a :2 13 1f a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
55 57 58 :2 55 :2 2c :3 10 :2 16 1b
27 64 67 :2 27 74 77 :2 27 7b
7e :2 27 8a 8d :2 27 95 98 :2 27
a4 a7 ae b7 ba :2 a7 :2 27 :2 10
5b :2 d :2 a :4 16 :2 a 7 :3 3c 4
3c 39 a 13 21 :2 a 26 24
:2 2f :2 24 11 1e 2d 46 53 :2 a
:2 13 1f a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 55 57
58 :2 55 :2 2c :3 10 :2 16 1b 27 64
67 :2 27 74 77 :2 27 7b 7e :2 27
8a 8d :2 27 95 98 :2 27 a4 a7
ae b7 ba :2 a7 :2 27 :2 10 5b :2 d
:2 a :4 16 :2 a 7 :3 40 4 40 39
a 13 21 :2 a 26 24 :2 2f :2 24
11 1e 2d 4a 57 :2 a :2 13 1f
a 7 f d :2 16 22 d 1d
:2 a :2 f d :2 16 22 d 10 :2 1a
2c :2 36 42 46 55 58 59 :2 55
:2 2c :3 10 :2 16 1b 27 65 68 :2 27
75 78 :2 27 7c 7f :2 27 8b 8e
:2 27 96 99 :2 27 a5 a8 af b8
bb :2 a8 :2 27 :2 10 5c :2 d :2 a :4 16
:2 a 7 :3 40 4 40 39 a 13
21 :2 a 26 24 :2 2f :2 24 11 1e
2d 48 55 :2 a :2 13 1f a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 55 58 59 :2 55 :2 2c :3 10
:2 16 1b 27 65 68 :2 27 75 78
:2 27 7c 7f :2 27 8b 8e :2 27 96
99 :2 27 a5 a8 af b8 bb :2 a8
:2 27 :2 10 5c :2 d :2 a :4 16 :2 a 7
:3 40 4 40 39 a 13 21 :2 a
26 24 :2 2f :2 24 11 1e 2d 47
54 :2 a :2 13 1f a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
55 58 59 :2 55 :2 2c :3 10 :2 16 1b
27 65 68 :2 27 75 78 :2 27 7c
7f :2 27 8b 8e :2 27 96 99 :2 27
a5 a8 af b8 bb :2 a8 :2 27 :2 10
5c :2 d :2 a :4 16 :2 a 7 :3 40 4
40 39 a 13 21 :2 a 26 24
:2 2f :2 24 11 1e 2d 48 55 :2 a
:2 13 1f a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 55 58
59 :2 55 :2 2c :3 10 :2 16 1b 27 65
68 :2 27 75 78 :2 27 7c 7f :2 27
8b 8e :2 27 96 99 :2 27 a5 a8
af b8 bb :2 a8 :2 27 :2 10 5c :2 d
:2 a :4 16 :2 a 7 :4 40 39 :3 4 :3 f
:2 4 :2 d 19 28 36 41 4f 5e
27 :2 19 4 7 :2 11 23 :2 2d 39
3c 4b 4e 4f :2 4b :2 23 :3 7 :2 d
12 1e 5c 5f :2 1e 6c 6f :2 1e
73 76 :2 1e 80 83 8c 94 :2 9d
:2 8c a8 :2 83 :2 1e b0 b3 :2 1e bb
be :2 1e c3 c6 :2 1e ce d1 :2 1e
:2 7 52 :3 4 :2 f 21 :2 2a 34 3d
:2 4 1 9 12 9 a :2 14 26
:2 30 3c 40 4f 52 53 :2 4f :2 26
:3 a :2 10 15 21 5f 62 :2 21 :3 a
:2 10 15 39 5c 5f :2 68 :2 39 76
79 :2 39 81 84 :2 8d :2 39 :2 a 56
:3 7 c 15 1e 2c a 13 1e
f e 7 e 7 e 7 29
:2 4 :2 9 7 :2 10 1c :2 7 :2 11 23
:2 2d 39 3d 4c 4f 50 :2 4c :2 23
:3 7 :2 d 12 1e 5c 5f :2 1e :3 7
:2 d 12 36 42 45 :2 36 4d 50
:2 36 5c 5f 66 6f 72 :2 5f :2 36
:2 7 53 :2 4 7 c 1b a 13
1e f e 7 e 7 e 7
:2 4 :4 10 :2 4 :8 1 
551
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 2 :4 7
:2 1 :7 9 :7 a :7 b
:7 c :6 d :6 e :7 10
:b 12 :11 13 :8 14 :2 13
:2 15 :2 13 :2 15 :2 13
:2 15 :2 13 :2 15 :2 13
:2 16 :2 13 :2 16 :2 13
:2 16 :2 13 :2 16 :2 13
:2 17 :2 13 :2 17 :2 13
:2 17 :2 13 :2 17 :2 13
:2 17 :4 13 :3 12 :11 1a
:b 1b :3 1a :16 1e :11 1f
:20 20 :3 1f :11 22 :b 23
:3 22 :9 25 :3 1e :3 2a
:3 2b :3 2c :3 2d :b 2f
:6 31 :5 32 30 34
:5 35 :3 34 :2 36 :5 38
:11 39 :24 3a :3 39 :2 37
:6 36 33 :3 2f 3e
2f :b 3e :6 40 :5 41
3f 43 :5 44 :3 43
:2 45 :5 47 :11 48 :24 49
:3 48 :2 46 :6 45 42
:3 3e 4d 3e 2f
:b 4d :6 4f :5 50 4e
52 :5 53 :3 52 :2 54
:5 56 :11 57 :24 58 :3 57
:2 55 :6 54 51 :3 4d
5c 4d 2f :b 5c
:6 5e :5 5f 5d 61
:5 62 :3 61 :2 63 :5 65
:11 66 :24 67 :3 66 :2 64
:6 63 60 :3 5c 6b
5c 2f :b 6b :6 6d
:5 6e 6c 70 :5 71
:3 70 :2 72 :5 74 :11 75
:24 76 :3 75 :2 73 :6 72
6f :3 6b 7a 6b
2f :b 7a :6 7c :5 7d
7b 7f :5 80 :3 7f
:2 81 :5 83 :11 84 :24 85
:3 84 :2 82 :6 81 7e
:3 7a 89 7a 2f
:b 89 :6 8b :5 8c 8a
8e :5 8f :3 8e :2 90
:5 92 :11 93 :24 94 :3 93
:2 91 :6 90 8d :4 89
:3 2f :5 9a :9 9c 9d
:3 9c :11 9f :30 a0 :3 9f
:a a2 11 :3 a5 :11 a6
:b a7 :17 a8 :3 a6 :2 aa
:3 ab :3 ac ad ab
aa ab :3 ae :3 a5
:2 af :5 b1 :11 b2 :b b3
:18 b4 :3 b2 :2 b6 b7
:3 b8 b9 b7 b6
b7 :3 ba :2 b0 :6 af
a4 :7 1 
12bb
4
:3 0 1 :3 0 2
:a 0 54c 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 6
:3 0 5 :7 0 9
8 :3 0 6 :3 0
7 :7 0 d c
:3 0 15 16 0
b 6 :3 0 8
:7 0 11 10 :3 0
9 :3 0 a :3 0
b :2 0 4 13
17 0 54c 2
18 :2 0 d :2 0
12 6 :3 0 d
:2 0 10 1b 1d
:6 0 20 1e 0
54a 0 c :6 0
d :2 0 16 6
:3 0 14 22 24
:6 0 27 25 0
54a 0 e :6 0
d :2 0 1a 6
:3 0 18 29 2b
:6 0 2e 2c 0
54a 0 f :6 0
12 :2 0 1e 6
:3 0 1c 30 32
:6 0 35 33 0
54a 0 10 :6 0
12 :2 0 20 4
:3 0 37 :7 0 3b
38 39 54a 0
11 :6 0 43 44
0 22 4 :3 0
3d :7 0 41 3e
3f 54a 0 13
:6 0 49 4a 0
24 a :3 0 b
:2 0 4 45 :7 0
48 46 0 54a
0 14 :6 0 15
:3 0 16 :3 0 15
:3 0 17 :3 0 4c
4d 0 2 :4 0
26 4e 50 4b
52 51 :2 0 18
:3 0 19 :3 0 54
55 0 1a :4 0
15 :3 0 1b :3 0
58 59 0 2
:4 0 28 5a 5c
1c :4 0 1d :2 0
1e :3 0 2a 5f
61 :3 0 1f :4 0
1d :2 0 20 :3 0
21 :3 0 3 :3 0
2d 66 68 22
:4 0 2f 65 6b
32 64 6d :3 0
1d :2 0 23 :4 0
35 6f 71 :3 0
1d :2 0 24 :4 0
38 73 75 :3 0
1d :2 0 5 :3 0
3b 77 79 :3 0
1d :2 0 24 :4 0
3e 7b 7d :3 0
1d :2 0 23 :4 0
41 7f 81 :3 0
1d :2 0 24 :4 0
44 83 85 :3 0
1d :2 0 7 :3 0
47 87 89 :3 0
1d :2 0 24 :4 0
4a 8b 8d :3 0
1d :2 0 23 :4 0
4d 8f 91 :3 0
1d :2 0 24 :4 0
50 93 95 :3 0
1d :2 0 8 :3 0
53 97 99 :3 0
1d :2 0 24 :4 0
56 9b 9d :3 0
1d :2 0 25 :4 0
59 9f a1 :3 0
5c 56 a3 :2 0
a5 61 a6 53
a5 0 a7 63
0 4aa 15 :3 0
16 :3 0 a8 a9
0 15 :3 0 26
:3 0 ab ac 0
27 :2 0 2 :4 0
28 :2 0 29 :2 0
27 :2 0 65 b1
b3 :3 0 68 ad
b5 aa b7 b6
:2 0 18 :3 0 2a
:3 0 b9 ba 0
1a :4 0 2b :4 0
1d :2 0 1e :3 0
6c be c0 :3 0
6f bb c2 :2 0
c4 72 c5 b8
c4 0 c6 74
0 4aa 5 :3 0
2c :2 0 76 c8
c9 :3 0 ca :2 0
cb :2 0 7 :3 0
2c :2 0 78 ce
cf :3 0 d0 :2 0
d1 :2 0 cc d3
d2 :2 0 8 :3 0
2c :2 0 7a d6
d7 :3 0 d8 :2 0
d9 :2 0 d4 db
da :2 0 15 :3 0
16 :3 0 dd de
0 15 :3 0 26
:3 0 e0 e1 0
2d :2 0 2 :4 0
2e :2 0 29 :2 0
27 :2 0 7c e6
e8 :3 0 7f e2
ea df ec eb
:2 0 18 :3 0 2a
:3 0 ee ef 0
1a :4 0 2f :4 0
1d :2 0 1e :3 0
83 f3 f5 :3 0
1d :2 0 30 :4 0
86 f7 f9 :3 0
1d :2 0 31 :4 0
89 fb fd :3 0
1d :2 0 20 :3 0
21 :3 0 32 :3 0
33 :3 0 102 103
0 8c 101 105
22 :4 0 8e 100
108 91 ff 10a
:3 0 94 f0 10c
:2 0 10e 97 10f
ed 10e 0 110
99 0 139 15
:3 0 16 :3 0 111
112 0 15 :3 0
26 :3 0 114 115
0 27 :2 0 2
:4 0 34 :2 0 29
:2 0 27 :2 0 9b
11a 11c :3 0 9e
116 11e 113 120
11f :2 0 18 :3 0
2a :3 0 122 123
0 1a :4 0 35
:4 0 1d :2 0 1e
:3 0 a2 127 129
:3 0 a5 124 12b
:2 0 12d a8 12e
121 12d 0 12f
aa 0 139 36
:3 0 37 :3 0 130
131 0 32 :3 0
33 :3 0 133 134
0 38 :3 0 ac
132 137 :2 0 139
af 13a dc 139
0 13b b3 0
4aa c :3 0 5
:3 0 13c 13d 0
4aa 10 :3 0 7
:3 0 13f 140 0
4aa f :3 0 8
:3 0 142 143 0
4aa 11 :3 0 3
:3 0 145 146 0
4aa 20 :3 0 11
:3 0 12 :2 0 b5
148 14b 32 :3 0
39 :2 0 3a :3 0
14d 14f 0 ba
14e 151 :3 0 3b
:3 0 13 :3 0 3c
:3 0 3d :3 0 c
:4 0 3e 1 :8 0
15e a :3 0 3f
:3 0 159 15a 0
40 :3 0 15b 15c
0 15e bd 1b1
41 :3 0 a :3 0
3f :3 0 160 161
0 40 :3 0 162
163 0 165 c0
167 c2 166 165
:2 0 1af 42 :3 0
a :3 0 3f :3 0
16a 16b 0 40
:3 0 16c 16d 0
1a7 15 :3 0 16
:3 0 16f 170 0
15 :3 0 26 :3 0
172 173 0 43
:2 0 2 :4 0 44
:2 0 29 :2 0 27
:2 0 c4 178 17a
:3 0 c7 174 17c
171 17e 17d :2 0
18 :3 0 2a :3 0
180 181 0 1a
:4 0 45 :4 0 1d
:2 0 1e :3 0 cb
185 187 :3 0 1d
:2 0 30 :4 0 ce
189 18b :3 0 1d
:2 0 46 :4 0 d1
18d 18f :3 0 1d
:2 0 40 :3 0 d4
191 193 :3 0 1d
:2 0 47 :4 0 d7
195 197 :3 0 1d
:2 0 48 :3 0 49
:3 0 27 :2 0 4a
:2 0 da 19a 19e
de 199 1a0 :3 0
e1 182 1a2 :2 0
1a4 e4 1a5 17f
1a4 0 1a6 e6
0 1a7 e8 1aa
:3 0 1aa 0 1aa
1a9 1a7 1a8 :6 0
1ac 2 :3 0 eb
1ae ed 1ad 1ac
:2 0 1af ef :2 0
1b1 0 1b1 1b0
15e 1af :6 0 1b4
1 :3 0 4b :3 0
f2 448 20 :3 0
11 :3 0 12 :2 0
f4 1b5 1b8 32
:3 0 39 :2 0 4c
:3 0 1ba 1bc 0
f9 1bb 1be :3 0
3b :3 0 13 :3 0
4d :3 0 3d :3 0
c :4 0 4e 1
:8 0 1cb a :3 0
3f :3 0 1c6 1c7
0 40 :3 0 1c8
1c9 0 1cb fc
21e 41 :3 0 a
:3 0 3f :3 0 1cd
1ce 0 40 :3 0
1cf 1d0 0 1d2
ff 1d4 101 1d3
1d2 :2 0 21c 42
:3 0 a :3 0 3f
:3 0 1d7 1d8 0
40 :3 0 1d9 1da
0 214 15 :3 0
16 :3 0 1dc 1dd
0 15 :3 0 26
:3 0 1df 1e0 0
43 :2 0 2 :4 0
4f :2 0 29 :2 0
27 :2 0 103 1e5
1e7 :3 0 106 1e1
1e9 1de 1eb 1ea
:2 0 18 :3 0 2a
:3 0 1ed 1ee 0
1a :4 0 50 :4 0
1d :2 0 1e :3 0
10a 1f2 1f4 :3 0
1d :2 0 30 :4 0
10d 1f6 1f8 :3 0
1d :2 0 46 :4 0
110 1fa 1fc :3 0
1d :2 0 40 :3 0
113 1fe 200 :3 0
1d :2 0 47 :4 0
116 202 204 :3 0
1d :2 0 48 :3 0
49 :3 0 27 :2 0
4a :2 0 119 207
20b 11d 206 20d
:3 0 120 1ef 20f
:2 0 211 123 212
1ec 211 0 213
125 0 214 127
217 :3 0 217 0
217 216 214 215
:6 0 219 4 :3 0
12a 21b 12c 21a
219 :2 0 21c 12e
:2 0 21e 0 21e
21d 1cb 21c :6 0
221 1 :3 0 4b
:3 0 131 222 1bf
221 0 449 20
:3 0 11 :3 0 12
:2 0 133 223 226
32 :3 0 39 :2 0
51 :3 0 228 22a
0 138 229 22c
:3 0 3b :3 0 13
:3 0 52 :3 0 3d
:3 0 c :4 0 53
1 :8 0 239 a
:3 0 3f :3 0 234
235 0 40 :3 0
236 237 0 239
13b 28c 41 :3 0
a :3 0 3f :3 0
23b 23c 0 40
:3 0 23d 23e 0
240 13e 242 140
241 240 :2 0 28a
42 :3 0 a :3 0
3f :3 0 245 246
0 40 :3 0 247
248 0 282 15
:3 0 16 :3 0 24a
24b 0 15 :3 0
26 :3 0 24d 24e
0 43 :2 0 2
:4 0 54 :2 0 29
:2 0 27 :2 0 142
253 255 :3 0 145
24f 257 24c 259
258 :2 0 18 :3 0
2a :3 0 25b 25c
0 1a :4 0 55
:4 0 1d :2 0 1e
:3 0 149 260 262
:3 0 1d :2 0 30
:4 0 14c 264 266
:3 0 1d :2 0 46
:4 0 14f 268 26a
:3 0 1d :2 0 40
:3 0 152 26c 26e
:3 0 1d :2 0 47
:4 0 155 270 272
:3 0 1d :2 0 48
:3 0 49 :3 0 27
:2 0 4a :2 0 158
275 279 15c 274
27b :3 0 15f 25d
27d :2 0 27f 162
280 25a 27f 0
281 164 0 282
166 285 :3 0 285
0 285 284 282
283 :6 0 287 6
:3 0 169 289 16b
288 287 :2 0 28a
16d :2 0 28c 0
28c 28b 239 28a
:6 0 28f 1 :3 0
4b :3 0 170 290
22d 28f 0 449
20 :3 0 11 :3 0
12 :2 0 172 291
294 32 :3 0 39
:2 0 56 :3 0 296
298 0 177 297
29a :3 0 3b :3 0
13 :3 0 57 :3 0
3d :3 0 c :4 0
58 1 :8 0 2a7
a :3 0 3f :3 0
2a2 2a3 0 40
:3 0 2a4 2a5 0
2a7 17a 2fa 41
:3 0 a :3 0 3f
:3 0 2a9 2aa 0
40 :3 0 2ab 2ac
0 2ae 17d 2b0
17f 2af 2ae :2 0
2f8 42 :3 0 a
:3 0 3f :3 0 2b3
2b4 0 40 :3 0
2b5 2b6 0 2f0
15 :3 0 16 :3 0
2b8 2b9 0 15
:3 0 26 :3 0 2bb
2bc 0 43 :2 0
2 :4 0 59 :2 0
29 :2 0 27 :2 0
181 2c1 2c3 :3 0
184 2bd 2c5 2ba
2c7 2c6 :2 0 18
:3 0 2a :3 0 2c9
2ca 0 1a :4 0
5a :4 0 1d :2 0
1e :3 0 188 2ce
2d0 :3 0 1d :2 0
30 :4 0 18b 2d2
2d4 :3 0 1d :2 0
46 :4 0 18e 2d6
2d8 :3 0 1d :2 0
40 :3 0 191 2da
2dc :3 0 1d :2 0
47 :4 0 194 2de
2e0 :3 0 1d :2 0
48 :3 0 49 :3 0
27 :2 0 4a :2 0
197 2e3 2e7 19b
2e2 2e9 :3 0 19e
2cb 2eb :2 0 2ed
1a1 2ee 2c8 2ed
0 2ef 1a3 0
2f0 1a5 2f3 :3 0
2f3 0 2f3 2f2
2f0 2f1 :6 0 2f5
8 :3 0 1a8 2f7
1aa 2f6 2f5 :2 0
2f8 1ac :2 0 2fa
0 2fa 2f9 2a7
2f8 :6 0 2fd 1
:3 0 4b :3 0 1af
2fe 29b 2fd 0
449 20 :3 0 11
:3 0 12 :2 0 1b1
2ff 302 32 :3 0
39 :2 0 5b :3 0
304 306 0 1b6
305 308 :3 0 3b
:3 0 13 :3 0 5c
:3 0 3d :3 0 c
:4 0 5d 1 :8 0
315 a :3 0 3f
:3 0 310 311 0
40 :3 0 312 313
0 315 1b9 368
41 :3 0 a :3 0
3f :3 0 317 318
0 40 :3 0 319
31a 0 31c 1bc
31e 1be 31d 31c
:2 0 366 42 :3 0
a :3 0 3f :3 0
321 322 0 40
:3 0 323 324 0
35e 15 :3 0 16
:3 0 326 327 0
15 :3 0 26 :3 0
329 32a 0 43
:2 0 2 :4 0 5e
:2 0 29 :2 0 27
:2 0 1c0 32f 331
:3 0 1c3 32b 333
328 335 334 :2 0
18 :3 0 2a :3 0
337 338 0 1a
:4 0 5f :4 0 1d
:2 0 1e :3 0 1c7
33c 33e :3 0 1d
:2 0 30 :4 0 1ca
340 342 :3 0 1d
:2 0 46 :4 0 1cd
344 346 :3 0 1d
:2 0 40 :3 0 1d0
348 34a :3 0 1d
:2 0 47 :4 0 1d3
34c 34e :3 0 1d
:2 0 48 :3 0 49
:3 0 27 :2 0 4a
:2 0 1d6 351 355
1da 350 357 :3 0
1dd 339 359 :2 0
35b 1e0 35c 336
35b 0 35d 1e2
0 35e 1e4 361
:3 0 361 0 361
360 35e 35f :6 0
363 a :3 0 1e7
365 1e9 364 363
:2 0 366 1eb :2 0
368 0 368 367
315 366 :6 0 36b
1 :3 0 4b :3 0
1ee 36c 309 36b
0 449 20 :3 0
11 :3 0 12 :2 0
1f0 36d 370 32
:3 0 39 :2 0 60
:3 0 372 374 0
1f5 373 376 :3 0
3b :3 0 13 :3 0
61 :3 0 3d :3 0
c :4 0 62 1
:8 0 383 a :3 0
3f :3 0 37e 37f
0 40 :3 0 380
381 0 383 1f8
3d6 41 :3 0 a
:3 0 3f :3 0 385
386 0 40 :3 0
387 388 0 38a
1fb 38c 1fd 38b
38a :2 0 3d4 42
:3 0 a :3 0 3f
:3 0 38f 390 0
40 :3 0 391 392
0 3cc 15 :3 0
16 :3 0 394 395
0 15 :3 0 26
:3 0 397 398 0
43 :2 0 2 :4 0
63 :2 0 29 :2 0
27 :2 0 1ff 39d
39f :3 0 202 399
3a1 396 3a3 3a2
:2 0 18 :3 0 2a
:3 0 3a5 3a6 0
1a :4 0 64 :4 0
1d :2 0 1e :3 0
206 3aa 3ac :3 0
1d :2 0 30 :4 0
209 3ae 3b0 :3 0
1d :2 0 46 :4 0
20c 3b2 3b4 :3 0
1d :2 0 40 :3 0
20f 3b6 3b8 :3 0
1d :2 0 47 :4 0
212 3ba 3bc :3 0
1d :2 0 48 :3 0
49 :3 0 27 :2 0
4a :2 0 215 3bf
3c3 219 3be 3c5
:3 0 21c 3a7 3c7
:2 0 3c9 21f 3ca
3a4 3c9 0 3cb
221 0 3cc 223
3cf :3 0 3cf 0
3cf 3ce 3cc 3cd
:6 0 3d1 c :3 0
226 3d3 228 3d2
3d1 :2 0 3d4 22a
:2 0 3d6 0 3d6
3d5 383 3d4 :6 0
3d9 1 :3 0 4b
:3 0 22d 3da 377
3d9 0 449 20
:3 0 11 :3 0 12
:2 0 22f 3db 3de
32 :3 0 39 :2 0
65 :3 0 3e0 3e2
0 234 3e1 3e4
:3 0 3b :3 0 13
:3 0 66 :3 0 3d
:3 0 c :4 0 67
1 :8 0 3f1 a
:3 0 3f :3 0 3ec
3ed 0 40 :3 0
3ee 3ef 0 3f1
237 444 41 :3 0
a :3 0 3f :3 0
3f3 3f4 0 40
:3 0 3f5 3f6 0
3f8 23a 3fa 23c
3f9 3f8 :2 0 442
42 :3 0 a :3 0
3f :3 0 3fd 3fe
0 40 :3 0 3ff
400 0 43a 15
:3 0 16 :3 0 402
403 0 15 :3 0
26 :3 0 405 406
0 43 :2 0 2
:4 0 68 :2 0 29
:2 0 27 :2 0 23e
40b 40d :3 0 241
407 40f 404 411
410 :2 0 18 :3 0
2a :3 0 413 414
0 1a :4 0 69
:4 0 1d :2 0 1e
:3 0 245 418 41a
:3 0 1d :2 0 30
:4 0 248 41c 41e
:3 0 1d :2 0 46
:4 0 24b 420 422
:3 0 1d :2 0 40
:3 0 24e 424 426
:3 0 1d :2 0 47
:4 0 251 428 42a
:3 0 1d :2 0 48
:3 0 49 :3 0 27
:2 0 4a :2 0 254
42d 431 258 42c
433 :3 0 25b 415
435 :2 0 437 25e
438 412 437 0
439 260 0 43a
262 43d :3 0 43d
0 43d 43c 43a
43b :6 0 43f e
:3 0 265 441 267
440 43f :2 0 442
269 :2 0 444 0
444 443 3f1 442
:6 0 446 1 :3 0
26c 447 3e5 446
0 449 152 1b4
0 449 26e 0
4aa e :3 0 6a
:4 0 44b 44c :3 0
44a 44d 0 4aa
a :3 0 3f :3 0
44f 450 0 6b
:3 0 11 :3 0 13
:3 0 c :3 0 10
:3 0 f :3 0 e
:3 0 276 452 459
451 45a 0 4aa
15 :3 0 16 :3 0
45c 45d 0 15
:3 0 26 :3 0 45f
460 0 6c :2 0
2 :4 0 6d :2 0
29 :2 0 27 :2 0
27d 465 467 :3 0
280 461 469 45e
46b 46a :2 0 18
:3 0 2a :3 0 46d
46e 0 1a :4 0
6e :4 0 1d :2 0
1e :3 0 284 472
474 :3 0 1d :2 0
30 :4 0 287 476
478 :3 0 1d :2 0
31 :4 0 28a 47a
47c :3 0 1d :2 0
20 :3 0 21 :3 0
a :3 0 3f :3 0
481 482 0 28d
480 484 22 :4 0
28f 47f 487 292
47e 489 :3 0 1d
:2 0 6f :4 0 295
48b 48d :3 0 1d
:2 0 24 :4 0 298
48f 491 :3 0 1d
:2 0 e :3 0 29b
493 495 :3 0 1d
:2 0 24 :4 0 29e
497 499 :3 0 2a1
46f 49b :2 0 49d
2a4 49e 46c 49d
0 49f 2a6 0
4aa 36 :3 0 70
:3 0 4a0 4a1 0
a :3 0 3f :3 0
4a3 4a4 0 e
:3 0 38 :3 0 2a8
4a2 4a8 :2 0 4aa
2ac 54b a :3 0
71 :2 0 4 4ab
4ac 0 15 :3 0
16 :3 0 4ae 4af
0 15 :3 0 26
:3 0 4b1 4b2 0
43 :2 0 2 :4 0
72 :2 0 29 :2 0
27 :2 0 2b9 4b7
4b9 :3 0 2bc 4b3
4bb 4b0 4bd 4bc
:2 0 18 :3 0 2a
:3 0 4bf 4c0 0
1a :4 0 73 :4 0
1d :2 0 1e :3 0
2c0 4c4 4c6 :3 0
2c3 4c1 4c8 :2 0
4e1 18 :3 0 2a
:3 0 4ca 4cb 0
1a :4 0 74 :4 0
1d :2 0 a :3 0
75 :3 0 4d0 4d1
0 2c6 4cf 4d3
:3 0 1d :2 0 6f
:4 0 2c9 4d5 4d7
:3 0 1d :2 0 a
:3 0 76 :3 0 4da
4db 0 2cc 4d9
4dd :3 0 2cf 4cc
4df :2 0 4e1 2d2
4e2 4be 4e1 0
4e3 2d5 0 4f3
77 :3 0 14 :3 0
a :3 0 75 :3 0
78 :3 0 a :3 0
76 :3 0 5 :3 0
79 :4 0 7a 1
:a 0 4ef :2 0 4f3
4e5 4ed :2 0 9
:3 0 14 :3 0 4f1
:2 0 4f3 2d7 4f5
2db 4f4 4f3 :2 0
549 42 :3 0 a
:3 0 3f :3 0 4f8
4f9 0 40 :3 0
4fa 4fb 0 541
15 :3 0 16 :3 0
4fd 4fe 0 15
:3 0 26 :3 0 500
501 0 43 :2 0
2 :4 0 7b :2 0
29 :2 0 27 :2 0
2dd 506 508 :3 0
2e0 502 50a 4ff
50c 50b :2 0 18
:3 0 2a :3 0 50e
50f 0 1a :4 0
7c :4 0 1d :2 0
1e :3 0 2e4 513
515 :3 0 2e7 510
517 :2 0 531 18
:3 0 2a :3 0 519
51a 0 1a :4 0
46 :4 0 1d :2 0
40 :3 0 2ea 51e
520 :3 0 1d :2 0
47 :4 0 2ed 522
524 :3 0 1d :2 0
48 :3 0 49 :3 0
27 :2 0 4a :2 0
2f0 527 52b 2f4
526 52d :3 0 2f7
51b 52f :2 0 531
2fa 532 50d 531
0 533 2fd 0
541 77 :3 0 14
:3 0 78 :3 0 a
:3 0 76 :3 0 5
:3 0 79 :4 0 7d
1 :a 0 53d :2 0
541 535 53b :2 0
9 :3 0 14 :3 0
53f :2 0 541 2ff
544 :3 0 544 0
544 543 541 542
:6 0 546 1 :3 0
304 548 306 547
546 :2 0 549 308
:2 0 54b 30b 54b
54a 4aa 549 :6 0
54c :2 0 2 18
54b 54f :3 0 54e
54c 550 :8 0 
313
4
:3 0 1 3 1
7 1 b 1
f 4 6 a
e 12 1 1c
1 1a 1 23
1 21 1 2a
1 28 1 31
1 2f 1 36
1 3c 1 42
1 4f 1 5b
2 5e 60 1
67 2 69 6a
2 63 6c 2
6e 70 2 72
74 2 76 78
2 7a 7c 2
7e 80 2 82
84 2 86 88
2 8a 8c 2
8e 90 2 92
94 2 96 98
2 9a 9c 2
9e a0 4 57
5d 62 a2 1
a4 1 a6 2
b0 b2 3 ae
af b4 2 bd
bf 2 bc c1
1 c3 1 c5
1 c7 1 cd
1 d5 2 e5
e7 3 e3 e4
e9 2 f2 f4
2 f6 f8 2
fa fc 1 104
2 106 107 2
fe 109 2 f1
10b 1 10d 1
10f 2 119 11b
3 117 118 11d
2 126 128 2
125 12a 1 12c
1 12e 2 135
136 3 110 12f
138 1 13a 2
149 14a 1 150
2 14c 150 2
158 15d 1 164
1 15f 2 177
179 3 175 176
17b 2 184 186
2 188 18a 2
18c 18e 2 190
192 2 194 196
3 19b 19c 19d
2 198 19f 2
183 1a1 1 1a3
1 1a5 2 16e
1a6 1 1aa 1
169 2 167 1ae
1 1b1 2 1b6
1b7 1 1bd 2
1b9 1bd 2 1c5
1ca 1 1d1 1
1cc 2 1e4 1e6
3 1e2 1e3 1e8
2 1f1 1f3 2
1f5 1f7 2 1f9
1fb 2 1fd 1ff
2 201 203 3
208 209 20a 2
205 20c 2 1f0
20e 1 210 1
212 2 1db 213
1 217 1 1d6
2 1d4 21b 1
21e 2 224 225
1 22b 2 227
22b 2 233 238
1 23f 1 23a
2 252 254 3
250 251 256 2
25f 261 2 263
265 2 267 269
2 26b 26d 2
26f 271 3 276
277 278 2 273
27a 2 25e 27c
1 27e 1 280
2 249 281 1
285 1 244 2
242 289 1 28c
2 292 293 1
299 2 295 299
2 2a1 2a6 1
2ad 1 2a8 2
2c0 2c2 3 2be
2bf 2c4 2 2cd
2cf 2 2d1 2d3
2 2d5 2d7 2
2d9 2db 2 2dd
2df 3 2e4 2e5
2e6 2 2e1 2e8
2 2cc 2ea 1
2ec 1 2ee 2
2b7 2ef 1 2f3
1 2b2 2 2b0
2f7 1 2fa 2
300 301 1 307
2 303 307 2
30f 314 1 31b
1 316 2 32e
330 3 32c 32d
332 2 33b 33d
2 33f 341 2
343 345 2 347
349 2 34b 34d
3 352 353 354
2 34f 356 2
33a 358 1 35a
1 35c 2 325
35d 1 361 1
320 2 31e 365
1 368 2 36e
36f 1 375 2
371 375 2 37d
382 1 389 1
384 2 39c 39e
3 39a 39b 3a0
2 3a9 3ab 2
3ad 3af 2 3b1
3b3 2 3b5 3b7
2 3b9 3bb 3
3c0 3c1 3c2 2
3bd 3c4 2 3a8
3c6 1 3c8 1
3ca 2 393 3cb
1 3cf 1 38e
2 38c 3d3 1
3d6 2 3dc 3dd
1 3e3 2 3df
3e3 2 3eb 3f0
1 3f7 1 3f2
2 40a 40c 3
408 409 40e 2
417 419 2 41b
41d 2 41f 421
2 423 425 2
427 429 3 42e
42f 430 2 42b
432 2 416 434
1 436 1 438
2 401 439 1
43d 1 3fc 2
3fa 441 1 444
7 448 222 290
2fe 36c 3da 447
6 453 454 455
456 457 458 2
464 466 3 462
463 468 2 471
473 2 475 477
2 479 47b 1
483 2 485 486
2 47d 488 2
48a 48c 2 48e
490 2 492 494
2 496 498 2
470 49a 1 49c
1 49e 3 4a5
4a6 4a7 c a7
c6 13b 13e 141
144 147 449 44e
45b 49f 4a9 2
4b6 4b8 3 4b4
4b5 4ba 2 4c3
4c5 2 4c2 4c7
2 4ce 4d2 2
4d4 4d6 2 4d8
4dc 2 4cd 4de
2 4c9 4e0 1
4e2 3 4e3 4ee
4f2 1 4ad 2
505 507 3 503
504 509 2 512
514 2 511 516
2 51d 51f 2
521 523 3 528
529 52a 2 525
52c 2 51c 52e
2 518 530 1
532 4 4fc 533
53c 540 1 544
1 4f7 2 4f5
548 7 1f 26
2d 34 3a 40
47 
1
4
0 
54f
0
1
14
10
c
0 1 2 1 4 1 6 1
8 1 a 1 c 1 e 1
0 0 0 0 
f 1 0
1a 1 0
2f 1 0
7 1 0
b 1 0
3c 1 0
21 1 0
36 1 0
28 1 0
3 1 0
2 0 1
42 1 0
0

/
SHOW ERRORS FUNCTION SP_A_DOCRES;
ALTER FUNCTION SP_A_DOCRES COMPILE DEBUG;
/*****************************************************************************************/
Prompt == FUNCTION FC_U_CREATETMPTABLE ==
CREATE OR REPLACE FUNCTION FC_U_CREATETMPTABLE wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
1f
2 :e:
1FUNCTION:
1FC_U_CREATETMPTABLE:
1RETURN:
1PLS_INTEGER:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= FC_U_CREATETMPTABLE();:
1CHECK_TRACE:
11:
18:
1+:
1FPUT:
1               ENTER:: FC_U_CREATETMPTABLE(9) :: cplan.sql(6917) :: :
1EXECUTE:
1IMMEDIATE:
1DELETE FROM PMO_TMP_CPLN01_TEMP:
1PMO_DB2M:
1SC_STATE:
10:
117:
1                EXIT:: FC_U_CREATETMPTABLE(18) :: cplan.sql(6926) :: :
1 :
1STATE:: :
0

0
0
8c
2
0 a0 8d a0 b4 a0 2c 6a
:2 a0 6b :2 a0 6b 6e a5 b a
10 :2 a0 6b 6e :2 a0 6b 6e a5
b 6e 7e a0 b4 2e 6e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6e 11e 11d :2 a0 6b 51 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :2 a0 6b
b4 2e a5 57 b7 19 3c :3 a0
6b 65 b7 a4 b1 11 68 4f
1d 17 b5 
8c
2
0 3 f b a 23 27 2b
2f 33 7 37 3b 3f 42 47
48 1 4a 4f 53 57 5a 5f
63 67 6a 6f 70 72 77 7a
7e 7f 84 89 8a 8f 91 95
98 9c a0 a3 a7 ab ae b1
b6 b9 bc bf c0 c5 c6 1
c8 cd d1 d5 d8 dd e2 e5
e9 ea ef f0 f5 f7 fb fe
102 106 10b 110 114 118 11c 11f
122 126 12a 12e 131 135 139 13c
13f 144 147 14a 14d 14e 153 154
1 156 15b 15f 163 166 16b 170
173 177 178 17d 180 185 186 18b
18e 193 194 199 19c 1a0 1a4 1a7
1a8 1ad 1ae 1b3 1b5 1b9 1bc 1c0
1c4 1c8 1cb 1cf 1d1 1d5 1d7 1e3
1e7 1e9 1ea 1f3 
8c
2
0 1 a 1e 0 25 :2 1 7
:2 11 23 :2 2d 3a :2 23 :3 7 :2 d 19
25 :2 2f 43 :2 25 5b 61 64 :2 5b
72 :2 7 51 :2 4 7 :2 11 23 :2 2d
39 3c 53 54 55 :2 53 :2 23 :3 7
:2 d 12 1e 62 65 :2 1e :2 7 58
:3 4 c 16 :3 4 :2 d 19 4 7
:2 11 23 :2 2d 39 3c 53 55 56
:2 53 :2 23 :3 7 :2 d 12 1e 63 66
:2 1e 73 76 :2 1e 7a 7d :2 1e 87
8a :2 93 :2 1e :2 7 59 :3 4 b :2 14
4 :9 1 
8c
4
0 :3 1 0 :3 1
:b 4 :12 5 :3 4 :11 8
:b 9 :3 8 :5 e :5 f
:11 11 :19 12 :3 11 :5 14
:2 3 :7 1 
1f5
4
:3 0 1 :3 0 8
9 :2 0 3 :3 0
2 :a 0 87 1
:7 0 4 :3 0 4
5 0 87 2
6 :2 0 5 :3 0
6 :3 0 5 :3 0
7 :3 0 b c
0 2 :4 0 3
d f a 11
10 :2 0 8 :3 0
9 :3 0 13 14
0 a :4 0 5
:3 0 b :3 0 17
18 0 2 :4 0
5 19 1b c
:4 0 d :2 0 e
:3 0 7 1e 20
:3 0 f :4 0 a
15 23 :2 0 25
f 26 12 25
0 27 11 0
83 5 :3 0 6
:3 0 28 29 0
5 :3 0 10 :3 0
2b 2c 0 11
:2 0 2 :4 0 12
:2 0 13 :2 0 11
:2 0 13 31 33
:3 0 16 2d 35
2a 37 36 :2 0
8 :3 0 14 :3 0
39 3a 0 a
:4 0 15 :4 0 d
:2 0 e :3 0 1a
3e 40 :3 0 1d
3b 42 :2 0 44
20 45 38 44
0 46 22 0
83 16 :3 0 17
:3 0 18 :4 0 49
:4 0 4a :2 0 83
19 :3 0 1a :3 0
4c 4d 0 1b
:2 0 4e 4f 0
83 5 :3 0 6
:3 0 51 52 0
5 :3 0 10 :3 0
54 55 0 11
:2 0 2 :4 0 1c
:2 0 13 :2 0 11
:2 0 24 5a 5c
:3 0 27 56 5e
53 60 5f :2 0
8 :3 0 14 :3 0
62 63 0 a
:4 0 1d :4 0 d
:2 0 e :3 0 2b
67 69 :3 0 d
:2 0 1e :4 0 2e
6b 6d :3 0 d
:2 0 1f :4 0 31
6f 71 :3 0 d
:2 0 19 :3 0 1a
:3 0 74 75 0
34 73 77 :3 0
37 64 79 :2 0
7b 3a 7c 61
7b 0 7d 3c
0 83 3 :3 0
19 :3 0 1a :3 0
7f 80 0 81
:2 0 83 3e 86
:3 0 86 0 86
85 83 84 :6 0
87 :2 0 2 6
86 8a :3 0 89
87 8b :8 0 
45
4
:3 0 1 e 1
1a 2 1d 1f
4 16 1c 21
22 1 24 1
26 2 30 32
3 2e 2f 34
2 3d 3f 2
3c 41 1 43
1 45 2 59
5b 3 57 58
5d 2 66 68
2 6a 6c 2
6e 70 2 72
76 2 65 78
1 7a 1 7c
6 27 46 4b
50 7d 82 
1
4
0 
8a
0
1
14
1
1
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 
2 0 1
0

/
SHOW ERRORS FUNCTION FC_U_CREATETMPTABLE;
ALTER FUNCTION FC_U_CREATETMPTABLE COMPILE DEBUG;
/************************************************************************************************/
/*****************************************************************************************/
Prompt == FUNCTION FC_U_TSKRES2 ==
CREATE OR REPLACE FUNCTION FC_U_TSKRES2 wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
ee
2 :e:
1FUNCTION:
1FC_U_TSKRES2:
1H_ASSIGN_ID:
1OUT:
1NOCOPY:
1VARCHAR2:
1H_AND_ROLES:
1H_AND_SKILLS:
1SH_H_WORK_LOCATION:
1PLS_INTEGER:
1H_START_DATE:
1H_FINISH_DATE:
1SH_H_EFFORT:
1SH_H_WORK:
1SH_H_EUNIT:
1SH_H_UNIT:
1SH_H_WORK_TYPE:
1SH_H_BITFLAG:
1SH_H_CONTOUR:
1SH_H_CONTOUR_TYPE:
1SH_H_OCCURRENCE:
1H_ASSIGN_STRING:
1H_REC_USER:
1RETURN:
1H_WORK_LOCATION:
1H_EFFORT:
1H_WORK:
1H_EUNIT:
1H_UNIT:
1H_WORK_TYPE:
1H_BITFLAG:
1H_CONTOUR:
1H_CONTOUR_TYPE:
1H_OCCURRENCE:
1H_TASK_ID:
132:
1H_RES_ID:
1H_START_DT:
126:
1H_FINISH_DT:
1H_DAYOFYEAR:
1DATE:
1H_LAST_SUBMITTED:
1H_TSK_ASSIGNMENT_ID:
1H_PROJECT_ID:
1H_CALENDAR_ID:
1H_SECURITY_ID:
1H_RES_TYPE:
11:
1H_BIT_FLAG:
1H_FULL_NAME:
1255:
1H_NULL_STR:
1H_MIN_PER_DAY:
10:
1H_TYPE:
1H_PER_WEEK:
1H_CONSIDER:
1H_ROLLUP:
1H_CRI:
1H_TEMP_SC_STATE:
1H_TRANSFED_CRI:
1H_WORK_TODATE:
1H_TEMP_VAR:
1H_TEMP_SECURITY_ID:
1H_TEMP2_SECURITY_ID:
1H_SD:
1H_FD:
1H_REG_COST:
1NUMBER:
1H_REG_SALE:
1H_BILLABLE:
1H_CAPITAL_EXPENSE:
1H_TPH_DELETE:
1H_DURATION:
1H_PERCENT_UNITS:
1H_SCHED_WORK:
1H_PRIORITY:
1H_UPDATE:
1H_ZERO:
1H_RES_COST:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= FC_U_TSKRES2(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
194:
1+:
1FPUT:
1               ENTER:: FC_U_TSKRES2(95) :: cplan.sql(7033) :: :
1PMO_CNST:
1>=:
1SC_BIT_30:
1PMO_FUNC:
1SHIFTRIGHT:
1BITAND:
1SC_C0000000:
130:
1SC_1FFFFFFF:
1ELSIF:
1SC_BIT_28:
1SC_BIT_NOT_28:
10.00:
1=:
1PROJECT_ID:
1TASK_ID:
1RESOURCE_ID:
1RESOURCE_TYPE:
1LAST_SUBMITTED:
1WORK_TODATE:
1TO_DATE:
1SUBSTR:
1SCHED_START:
1SCHED_FINISH:
1BILLABLE:
1CAPITAL_EXPENSE:
1REGULAR_COST:
1REGULAR_RATE:
1TASK_ASSIGNMENTS:
1TSK_ASSIGNMENT_ID:
1SELECT:n         PROJECT_ID,:n         TASK_ID,:n         RESOURCE_ID,:n     +
1    RESOURCE_TYPE,:n         LAST_SUBMITTED,:n         WORK_TODATE,:n        +
1 TO_DATE(SubStr(SCHED_START, 1, 10)),:n         TO_DATE(SubStr(SCHED_FINISH, +
11, 10)),:n         BILLABLE,:n         CAPITAL_EXPENSE,:n         REGULAR_COS+
1T,:n         REGULAR_RATE:n         INTO:n         h_project_id,:n         h_+
1task_id,:n         h_res_id,:n         h_res_type,:n         h_last_submitted+
1,:n         h_work_todate,:n         h_SD,:n         h_FD,:n         h_billab+
1le,:n         h_capital_expense,:n         h_reg_cost,:n         h_reg_sale:n+
1         FROM TASK_ASSIGNMENTS:n         WHERE TSK_ASSIGNMENT_ID = h_tsk_assi+
1gnment_id:
1PMO_DB2M:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
1165:
1           EXCEPTION:: FC_U_TSKRES2(166) :: cplan.sql(7104) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SQLERRM:
1500:
1SC_SQL_NO_DATA:
1172:
1                EXIT:: FC_U_TSKRES2(173) :: cplan.sql(7111) :: :
1RECORD_NOT_FOUND:
1ELEMENT_ID:
1ELEMENT_TYPE:
1SECURITY_ID:
1CRI_SECURITY_ID:
1RESOURCE_ASSIGNMENTS:
1RES_ASSIGNMENT_ID:
1SELECT PROJECT_ID, ELEMENT_ID, RESOURCE_ID, ELEMENT_TYPE, :n         RESOURCE+
1_TYPE, SECURITY_ID, CASE WHEN CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END:n    +
1     INTO h_project_id, h_task_id, h_res_id, h_type,:n         h_res_type, h_+
1security_id, h_transfed_cri:n         FROM RESOURCE_ASSIGNMENTS :n         WH+
1ERE RES_ASSIGNMENT_ID = h_assign_id:
1192:
1           EXCEPTION:: FC_U_TSKRES2(193) :: cplan.sql(7131) :: :
1199:
1                EXIT:: FC_U_TSKRES2(200) :: cplan.sql(7138) :: :
1ISWBSTRANSFEREDDELIV:
1!=:
1204:
1                EXIT:: FC_U_TSKRES2(205) :: cplan.sql(7143) :: :
1TASK_ASSIGNMENT_ERROR:
1WORK_LOCATION:
1UPDATE RESOURCE_ASSIGNMENTS SET WORK_LOCATION = h_work_location:n         WHE+
1RE RES_ASSIGNMENT_ID = h_assign_id:
1FOUND:
1TRUE:
1100:
1224:
1           EXCEPTION:: FC_U_TSKRES2(225) :: cplan.sql(7163) :: :
1ISPROJECTTYPE:
1NOT:
1SELECT TSK_ASSIGNMENT_ID INTO h_tsk_assignment_id:n            FROM TASK_ASSI+
1GNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id:
1241:
1           EXCEPTION:: FC_U_TSKRES2(242) :: cplan.sql(7180) :: :
1SC_SQL_SUCCESS:
1248:
1                EXIT:: FC_U_TSKRES2(249) :: cplan.sql(7187) :: :
1RECORD_ALREADY_ASSIGNED:
1FC_AFTER_U_RES_ASSIGN:
1MINUTE_PER_DAY:
1MINUTE_PER_WEEK:
1CALENDAR_ID:
1PRIORITY:
1TMT_WBS:
1SELECT COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), CALEND+
1AR_ID, PRIORITY:n      INTO h_min_per_day, h_per_week, h_calendar_id, h_prior+
1ity:n      FROM TMT_WBS WHERE ELEMENT_ID = h_project_id:
1270:
1           EXCEPTION:: FC_U_TSKRES2(271) :: cplan.sql(7209) :: :
1>:
15:
1*:
1IS NULL:
1SUM:
1AVAILABLE_MIN:
1CALENDAR_DAY_OF_YEAR:
1DAY_YEAR:
1SELECT SUM(AVAILABLE_MIN) INTO h_effort FROM CALENDAR_DAY_OF_YEAR:n          +
1     WHERE ELEMENT_ID=h_project_id AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_star+
1t_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10)):
1298:
1           EXCEPTION:: FC_U_TSKRES2(299) :: cplan.sql(7237) :: :
160:
12:
13:
1315:
1                EXIT:: FC_U_TSKRES2(316) :: cplan.sql(7254) :: :
1SC_BIT_NOT_30:
1UNRESOLVED_START:
1UNRESOLVED_FINISH:
1PLANNED_WORK:
1SCHED_WORK:
1BIT_FLAG:
1SELECT TSK_ASSIGNMENT_ID, UNRESOLVED_START, UNRESOLVED_FINISH, PLANNED_WORK+S+
1CHED_WORK, BIT_FLAG:n         INTO h_tsk_assignment_id, h_start_DT, h_finish_+
1DT, h_sched_work, h_bit_flag:n         FROM TASK_ASSIGNMENTS:n         WHERE +
1PROJECT_ID = h_project_id AND TASK_ID = h_project_id AND RESOURCE_ID = h_res_+
1id:
1338:
1           EXCEPTION:: FC_U_TSKRES2(339) :: cplan.sql(7277) :: :
1IS NOT NULL:
1PMO_ORASTD:
1STRNCMP:
110:
1BIT_OR:
1<:
1FC_DECLARE_PMO_TMP_SEARCH:
1SC_SRCH_FROM_PROJECT:
1FC_SEARCH_2:
1ETC:
1UPDATE TASK_ASSIGNMENTS SET ETC = h_effort:n                  WHERE RES_ASSIG+
1NMENT_ID = h_assign_id:
1389:
1           EXCEPTION:: FC_U_TSKRES2(390) :: cplan.sql(7328) :: :
1395:
1                EXIT:: FC_U_TSKRES2(396) :: cplan.sql(7334) :: :
1U_TSKRES2:
1110:
1RESOURCE_UNAVAILABLE:
1489:
1                EXIT:: FC_U_TSKRES2(490) :: cplan.sql(7428) :: :
1STATE:: :
0

0
0
af0
2
0 a0 8d 90 :3 a0 b0 3f 90
:3 a0 b0 3f 90 :3 a0 b0 3f 8f
a0 b0 3d 90 :3 a0 b0 3f 90
:3 a0 b0 3f 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 90
:3 a0 b0 3f 90 :3 a0 b0 3f b4
:2 a0 2c 6a a3 a0 1c a0 81
b0 a3 a0 1c a0 81 b0 a3
a0 1c a0 81 b0 a3 a0 1c
a0 81 b0 a3 a0 1c a0 81
b0 a3 a0 1c a0 81 b0 a3
a0 1c a0 81 b0 a3 a0 1c
a0 81 b0 a3 a0 1c a0 81
b0 a3 a0 1c a0 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 1c 81 b0
a3 a0 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 1c 81 b0 a3 a0 1c 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 :2 a0 6b :2 a0 6b 6e a5 b
a 10 :2 a0 6b 6e :2 a0 6b 6e
a5 b 6e 7e a0 b4 2e 6e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e :3 a0 a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 51 a5 b a0 7e
a0 6b b4 2e :3 a0 6b :3 a0 51
a5 b :2 a0 6b a5 b 5a 51
a5 b d :4 a0 51 a5 b :2 a0
6b a5 b d a0 51 d a0
51 d a0 b7 :2 a0 51 a5 b
a0 7e a0 6b b4 2e :4 a0 51
a5 b :2 a0 6b a5 b 5a d
:4 a0 51 a5 b :2 a0 6b a5 b
5a d a0 51 d b7 19 :3 a0
51 a5 b d a0 51 d a0
51 d b7 :2 19 3c a0 51 d
a0 6e d a0 51 d a0 4d
d a0 4d d a0 4d d a0
4d d a0 51 d :2 a0 51 a5
b 7e 51 b4 2e :2 a0 d :1f a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :3 a0 6b 65
b7 19 3c b7 :11 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :3 a0 6b 65 a0 b7 :3 a0
51 a5 b a5 b 7e 51 b4
2e :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :3 a0 6b 65 b7
:2 19 3c :5 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :3 a0 51 a5 b a5 b
5a 7e 51 b4 2e 7e b4 2e
:5 a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b a0 7e a0 6b
b4 2e 5a :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :3 a0 6b
65 b7 19 3c b7 19 3c :3 a0
a5 b d b7 :2 19 3c :d a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 51 a5 b 7e 51 b4 2e
5a :2 a0 51 a5 b 7e 51 b4
2e 5a a 10 :3 a0 51 a5 b
7e :2 a0 51 a5 b b4 2e d
b7 19 3c :2 a0 51 a5 b 7e
51 b4 2e a0 7e b4 2e 5a
:2 a0 51 a5 b 7e 51 b4 2e
:3 a0 51 a5 b 7e :2 a0 51 a5
b b4 2e d b7 19 3c b7
:d a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 :2 19 3c b7 :2 a0 51
a5 b 7e 51 b4 2e :3 a0 51
a5 b 7e 51 b4 2e d a0
b7 :2 a0 51 a5 b 7e 51 b4
2e :3 a0 51 a5 b 7e :2 a0 51
a5 b b4 2e d a0 b7 19
:2 a0 51 a5 b 7e 51 b4 2e
:3 a0 51 a5 b 7e :2 a0 51 a5
b b4 2e d b7 :2 19 3c b7
:2 19 3c :2 a0 51 a5 b 7e 51
b4 2e :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :3 a0 6b 65
b7 19 3c b7 19 3c :2 a0 51
a5 b 7e 51 b4 2e :4 a0 51
a5 b :2 a0 6b a5 b 5a d
a0 b7 :2 a0 :2 51 a5 b 7e 6e
b4 2e 5a :2 a0 51 a5 b 7e
51 b4 2e 5a a 10 :12 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 6b a0 7e a0 6b b4 2e
5a a0 7e b4 2e 5a a 10
:4 a0 51 a5 b :2 a0 6b a5 b
5a d :2 a0 6b :2 a0 51 a5 b
7e 51 b4 2e 5a :2 a0 6b :2 a0
51 a5 b 7e 51 b4 2e 5a
a 10 :2 a0 :2 51 a5 b 7e 6e
b4 2e :2 a0 51 a5 b 7e 51
b4 2e a0 51 d b7 19 3c
a0 51 d b7 19 3c b7 :2 a0
51 a5 b 7e 51 b4 2e :3 a0
6b :2 a0 51 a5 b :2 a0 6b a5
b 5a d b7 19 3c :2 a0 51
a5 b a0 7e a0 51 a5 b
b4 2e :2 a0 51 a5 b 7e 51
b4 2e :2 a0 6b :2 a0 51 a5 b
7e 51 b4 2e a 10 :2 a0 :2 51
a5 b 7e 6e b4 2e a0 51
d b7 19 3c :2 a0 b4 2e d
:3 a0 6b d :20 a0 51 a5 b :3 51
a5 b d b7 19 3c :5 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c a0 51 65 b7 19 3c b7
:2 19 3c b7 19 3c b7 :2 19 3c
:2 a0 51 a5 b 7e 51 b4 2e
5a :2 a0 51 a5 b a0 7e a0
51 a5 b b4 2e 5a 52 10
:2 a0 51 a5 b 7e 51 b4 2e
:3 a0 6b :2 a0 51 a5 b :2 a0 51
a5 b a5 b 5a d b7 19
3c :18 a0 :2 51 a5 b d a0 b7
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 51 a5 b 7e 51 b4 2e
:3 a0 6b :2 a0 51 a5 b :2 a0 51
a5 b a5 b 5a d b7 19
3c :18 a0 :2 51 a5 b d b7 :2 19
3c :2 a0 51 a5 b 7e 51 b4
2e :2 a0 6b :2 a0 6b d a0 b7
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 51 a5 b 7e 51 b4 2e
a 10 :2 a0 6b :2 a0 6b d b7
:2 19 3c :2 a0 6b 7e 51 b4 2e
:2 a0 6b 51 d b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :2 a0 6b
b4 2e a5 57 b7 19 3c :3 a0
6b 65 b7 a4 b1 11 68 4f
1d 17 b5 
af0
2
0 3 7 2b 1f 23 27 1e
32 47 3b 3f 43 1b 4e 5f
53 57 5b 3a 66 73 6f 37
7b 8c 80 84 88 6e 93 a8
9c a0 a4 6b af b8 b4 9b
c0 cd c9 98 d5 de da c8
e6 f3 ef c5 fb 104 100 ee
10c 119 115 eb 121 12a 126 114
132 13f 13b 111 147 150 14c 13a
158 16d 161 165 169 137 174 185
179 17d 181 160 18c 15d 191 195
199 19d 1ba 1a5 1a9 1b1 1b5 1a4
1da 1c5 1c9 1d1 1d5 1a1 1f6 1e1
1e5 1ed 1f1 1c4 216 201 205 20d
211 1c1 232 21d 221 229 22d 200
252 23d 241 249 24d 1fd 26e 259
25d 265 269 23c 28e 279 27d 285
289 239 2aa 295 299 2a1 2a5 278
2ca 2b5 2b9 2c1 2c5 275 2e6 2d1
2d5 2d8 2d9 2e1 2b4 303 2f1 2b1
2f5 2f6 2fe 2f0 320 30e 2ed 312
313 31b 30d 33d 32b 30a 32f 330
338 32a 359 348 34c 354 327 371
360 364 36c 347 38e 37c 344 380
381 389 37b 3ab 399 378 39d 39e
3a6 398 3c8 3b6 395 3ba 3bb 3c3
3b5 3e5 3d3 3b2 3d7 3d8 3e0 3d2
402 3f0 3cf 3f4 3f5 3fd 3ef 41f
40d 3ec 411 412 41a 40c 43c 42a
409 42e 42f 437 429 459 447 426
44b 44c 454 446 475 464 468 443
470 463 491 480 484 460 48c 47f
4ad 49c 4a0 47c 4a8 49b 4c9 4b8
4bc 498 4c4 4b7 4e5 4d4 4d8 4b4
4e0 4d3 501 4f0 4f4 4d0 4fc 4ef
51d 50c 510 4ec 518 50b 539 528
52c 508 534 527 555 544 548 524
550 543 571 560 564 540 56c 55f
58e 57c 55c 580 581 589 57b 5ab
599 578 59d 59e 5a6 598 5c7 5b6
5ba 5c2 595 5df 5ce 5d2 5da 5b5
5fb 5ea 5ee 5b2 5f6 5e9 617 606
60a 5e6 612 605 633 622 626 602
62e 621 64f 63e 642 61e 64a 63d
66b 65a 65e 63a 666 659 687 676
67a 656 682 675 6a3 692 696 672
69e 691 6bf 6ae 6b2 68e 6ba 6ad
6db 6ca 6ce 6aa 6d6 6c9 6f7 6e6
6ea 6c6 6f2 6e5 713 702 706 6e2
70e 701 72f 71e 722 6fe 72a 71d
736 73a 71a 73e 742 746 749 74e
74f 1 751 756 75a 75e 761 766
76a 76e 771 776 777 779 77e 781
785 786 78b 790 793 798 799 79e
7a1 7a5 7a6 7ab 7ae 7b3 7b4 7b9
7bc 7c1 7c2 7c7 7ca 7cf 7d0 7d5
7d8 7dc 7dd 7e2 7e5 7ea 7eb 7f0
7f3 7f8 7f9 7fe 801 806 807 80c
80f 813 814 819 81c 821 822 827
82a 82f 830 835 838 83c 840 844
845 847 84c 84d 84f 850 855 858
85d 85e 863 866 86b 86c 871 874
878 879 87e 881 886 887 88c 88f
894 895 89a 89d 8a2 8a3 8a8 8ab
8af 8b0 8b5 8b8 8bd 8be 8c3 8c6
8cb 8cc 8d1 8d4 8d8 8dc 8e0 8e1
8e3 8e8 8e9 8eb 8ec 8f1 8f4 8f9
8fa 8ff 902 906 90a 90e 90f 911
916 917 919 91a 91f 922 927 928
92d 930 934 938 93c 93d 93f 944
945 947 948 94d 950 955 956 95b
95e 962 966 96a 96b 96d 972 973
975 976 97b 97e 983 984 989 98c
990 994 998 999 99b 9a0 9a1 9a3
9a4 9a9 9ac 9b1 9b2 9b7 9ba 9be
9c2 9c6 9c7 9c9 9ce 9cf 9d1 9d2
9d7 9da 9df 9e0 9e5 9e8 9ec 9f0
9f4 9f5 9f7 9fc 9fd 9ff a00 a05
a08 a0d a0e a13 a16 a1a a1e a22
a23 a25 a2a a2b a2d a2e a33 a36
a3b a3c a41 a44 a48 a4c a50 a51
a53 a58 a59 a5b a5c a61 a64 a69
a6a a6f a72 a77 a78 a7d a80 a84
a85 a8a a8d a92 a93 a98 a9b aa0
aa1 aa6 aa9 aae aaf ab4 ab7 abb
abc ac1 ac4 ac9 aca acf ad2 ad7
ad8 add ade ae3 ae5 ae9 aec af0
af4 af7 afb aff b02 b05 b0a b0d
b10 b13 b14 b19 b1a 1 b1c b21
b25 b29 b2c b31 b36 b39 b3d b3e
b43 b44 b49 b4b b4f b52 b56 b5a
b5d b5e b60 b64 b67 b6b b6e b6f
b74 b78 b7c b80 b83 b87 b8b b8f
b92 b93 b95 b99 b9d ba0 ba1 ba3
ba6 ba9 baa bac bb0 bb4 bb8 bbc
bc0 bc3 bc4 bc6 bca bce bd1 bd2
bd4 bd8 bdc bdf be3 be7 bea bee
bf2 bf4 bf8 bfc bff c00 c02 c06
c09 c0d c10 c11 c16 c1a c1e c22
c26 c29 c2a c2c c30 c34 c37 c38
c3a c3d c41 c45 c49 c4d c51 c54
c55 c57 c5b c5f c62 c63 c65 c68
c6c c70 c73 c77 c79 c7d c81 c85
c89 c8c c8d c8f c93 c97 c9a c9e
ca2 ca5 ca9 cab caf cb3 cb6 cba
cbd cc1 cc5 cca cce cd2 cd5 cd9
cdd cde ce2 ce6 ce7 ceb cef cf0
cf4 cf8 cf9 cfd d01 d04 d08 d0c
d10 d13 d14 d16 d19 d1c d1d d22
d26 d2a d2e d32 d36 d3a d3e d42
d46 d4a d4e d52 d56 d5a d5e d62
d66 d6a d6e d72 d76 d7a d7e d82
d86 d8a d8e d92 d96 d9a d9e da2
da6 daa db6 dba dbe dc1 dc5 dc9
dcb dcf dd3 dd7 dda dde de2 de4
de5 dea 1 dee df2 df6 df9 dfd
e01 e05 e09 e0c e10 e14 e17 e1a
e1f e22 e25 e28 e29 e2e e2f 1
e31 e36 e3a e3e e41 e46 e4b e4e
e52 e53 e58 e5b e60 e61 e66 e69
e6e e6f e74 e77 e7b e7c e81 e84
e89 e8a e8f e92 e96 e9a e9d ea0
ea1 ea3 ea4 ea9 eaa eaf eb1 eb5
eb8 eba ebe ec0 ecc ece ed0 ed1
ed6 eda edc ee8 eea eee ef2 ef5
ef9 efc f00 f03 f04 f09 f0c f10
f14 f17 f1b f1f f22 f25 f2a f2d
f30 f33 f34 f39 f3a 1 f3c f41
f45 f49 f4c f51 f56 f59 f5d f5e
f63 f64 f69 f6b f6f f72 f76 f7a
f7e f81 f85 f87 f8b f8e f90 f94
f98 f9c fa0 fa4 fa8 fac fb0 fb4
fb8 fbc fc0 fc4 fc8 fcc fd0 fd4
fe0 fe4 fe8 feb fef ff3 ff5 ff9
ffd 1001 1004 1008 100c 100e 100f 1014
1 1018 101c 1020 1023 1027 102b 102f
1033 1036 103a 103e 1041 1044 1049 104c
104f 1052 1053 1058 1059 1 105b 1060
1064 1068 106b 1070 1075 1078 107c 107d
1082 1085 108a 108b 1090 1093 1098 1099
109e 10a1 10a5 10a6 10ab 10ae 10b3 10b4
10b9 10bc 10c0 10c4 10c7 10ca 10cb 10cd
10ce 10d3 10d4 10d9 10db 10df 10e2 10e4
10e8 10ea 10f6 10f8 10fa 10fb 1100 1104
1106 1112 1114 1118 111c 111f 1123 1126
112a 112d 112e 1133 1136 113a 113e 1141
1145 1149 114c 114f 1154 1157 115a 115d
115e 1163 1164 1 1166 116b 116f 1173
1176 117b 1180 1183 1187 1188 118d 118e
1193 1195 1199 119c 11a0 11a4 11a8 11ab
11af 11b3 11b5 11b9 11bd 11c1 11c4 11c5
11c7 11c8 11ca 11cd 11d0 11d1 11d6 11da
11de 11e1 11e5 11e9 11ec 11ef 11f4 11f7
11fa 11fd 11fe 1203 1204 1 1206 120b
120f 1213 1216 121b 1220 1223 1227 1228
122d 122e 1233 1235 1239 123c 1240 1244
1248 124b 124f 1251 1255 1259 125c 1260
1264 1268 126c 1270 127c 1280 1285 1289
128c 128d 1292 1296 129a 129d 12a0 12a4
12a6 12aa 12ae 12b1 12b4 12b8 12ba 12be
12c2 12c5 12c7 12cb 12cf 12d3 12d6 12da
12de 12e0 12e1 12e6 1 12ea 12ee 12f2
12f5 12f9 12fd 1301 1305 1308 130c 1310
1313 1316 131b 131e 1321 1324 1325 132a
132b 1 132d 1332 1336 133a 133d 1342
1347 134a 134e 134f 1354 1357 135c 135d
1362 1365 136a 136b 1370 1373 1377 1378
137d 1380 1385 1386 138b 138e 1392 1396
1399 139c 139d 139f 13a0 13a5 13a6 13ab
13ad 13b1 13b4 13b6 13ba 13bc 13c8 13ca
13cc 13cd 13d2 13d6 13d8 13e4 13e6 13ea
13ee 13f2 13f5 13f6 13f8 13f9 13fb 13fe
1401 1404 1405 140a 140d 140e 1413 1417
141b 141f 1423 1427 1433 1437 143b 143e
1442 1446 1448 144c 1450 1454 1457 145b
145f 1461 1462 1467 1 146b 146f 1473
1476 147a 147e 1482 1486 1489 148d 1491
1494 1497 149c 149f 14a2 14a5 14a6 14ab
14ac 1 14ae 14b3 14b7 14bb 14be 14c3
14c8 14cb 14cf 14d0 14d5 14d8 14dd 14de
14e3 14e6 14eb 14ec 14f1 14f4 14f8 14f9
14fe 1501 1506 1507 150c 150f 1513 1517
151a 151d 151e 1520 1521 1526 1527 152c
152e 1532 1535 1537 153b 153d 1549 154b
154d 154e 1553 1557 1559 1565 1567 156b
156f 1572 1576 1579 157d 1580 1581 1586
1589 158d 1591 1594 1598 159c 159f 15a2
15a7 15aa 15ad 15b0 15b1 15b6 15b7 1
15b9 15be 15c2 15c6 15c9 15ce 15d3 15d6
15da 15db 15e0 15e1 15e6 15e8 15ec 15ef
15f3 15f7 15fb 15fe 1602 1604 1608 160b
160d 1611 1614 1618 161c 1620 1621 1623
1627 1629 162d 1631 1634 1638 163c 1640
1644 1648 164c 1650 1654 1658 165c 1660
1664 1668 1674 1678 167c 167f 1683 1687
1689 168d 1691 1695 1698 169c 16a0 16a2
16a3 16a8 1 16ac 16b0 16b4 16b7 16bb
16bf 16c3 16c7 16ca 16ce 16d2 16d5 16d8
16dd 16e0 16e3 16e6 16e7 16ec 16ed 1
16ef 16f4 16f8 16fc 16ff 1704 1709 170c
1710 1711 1716 1719 171e 171f 1724 1727
172c 172d 1732 1735 1739 173a 173f 1742
1747 1748 174d 1750 1754 1758 175b 175e
175f 1761 1762 1767 1768 176d 176f 1773
1776 1778 177c 177e 178a 178c 178e 178f
1794 1798 179a 17a6 17a8 17ac 17b0 17b3
17b4 17b6 17b9 17bc 17bd 17c2 17c6 17ca
17cd 17ce 17d0 17d3 17d6 17d7 17dc 17df
17e3 17e7 17ea 17eb 17ed 17f0 17f3 17f4
17f9 1 17fc 1801 1805 1809 180d 1810
1811 1813 1816 181a 181e 1821 1822 1824
1825 182a 182e 1830 1834 1837 183b 183f
1842 1843 1845 1848 184b 184c 1851 1855
1858 1859 185e 1861 1865 1869 186c 186d
186f 1872 1875 1876 187b 187f 1883 1887
188a 188b 188d 1890 1894 1898 189b 189c
189e 189f 18a4 18a8 18aa 18ae 18b1 18b3
18b7 18bb 18bf 18c3 18c7 18cb 18cf 18d3
18d7 18db 18df 18e3 18e7 18f3 18f7 18fb
18fe 1902 1906 1908 190c 1910 1914 1917
191b 191f 1921 1922 1927 1 192b 192f
1933 1936 193a 193e 1942 1946 1949 194d
1951 1954 1957 195c 195f 1962 1965 1966
196b 196c 1 196e 1973 1977 197b 197e
1983 1988 198b 198f 1990 1995 1998 199d
199e 19a3 19a6 19ab 19ac 19b1 19b4 19b8
19b9 19be 19c1 19c6 19c7 19cc 19cf 19d3
19d7 19da 19dd 19de 19e0 19e1 19e6 19e7
19ec 19ee 19f2 19f5 19f7 19fb 19fd 1a09
1a0b 1a0d 1a0e 1a13 1a17 1a19 1a25 1a27
1a29 1a2d 1a31 1a34 1a36 1a3a 1a3e 1a41
1a42 1a44 1a47 1a4a 1a4b 1a50 1a54 1a58
1a5c 1a5f 1a60 1a62 1a65 1a68 1a69 1a6e
1a72 1a76 1a78 1a7c 1a80 1a83 1a84 1a86
1a89 1a8c 1a8d 1a92 1a96 1a9a 1a9e 1aa1
1aa2 1aa4 1aa7 1aab 1aaf 1ab2 1ab3 1ab5
1ab6 1abb 1abf 1ac3 1ac5 1ac9 1acd 1ad1
1ad4 1ad5 1ad7 1ada 1add 1ade 1ae3 1ae7
1aeb 1aef 1af2 1af3 1af5 1af8 1afc 1b00
1b03 1b04 1b06 1b07 1b0c 1b10 1b12 1b16
1b1a 1b1d 1b1f 1b23 1b27 1b2a 1b2e 1b32
1b35 1b36 1b38 1b3b 1b3e 1b3f 1b44 1b48
1b4c 1b4f 1b53 1b57 1b5a 1b5d 1b62 1b65
1b68 1b6b 1b6c 1b71 1b72 1 1b74 1b79
1b7d 1b81 1b84 1b89 1b8e 1b91 1b95 1b96
1b9b 1b9c 1ba1 1ba3 1ba7 1baa 1bae 1bb2
1bb6 1bb9 1bbd 1bbf 1bc3 1bc6 1bc8 1bcc
1bcf 1bd3 1bd7 1bda 1bdb 1bdd 1be0 1be3
1be4 1be9 1bed 1bf1 1bf5 1bf9 1bfc 1bfd
1bff 1c03 1c07 1c0a 1c0b 1c0d 1c10 1c14
1c18 1c1a 1c1e 1c22 1c25 1c28 1c29 1c2b
1c2e 1c33 1c34 1c39 1c3c 1c40 1c44 1c47
1c48 1c4a 1c4d 1c50 1c51 1c56 1 1c59
1c5e 1c62 1c66 1c6a 1c6e 1c72 1c76 1c7a
1c7e 1c82 1c86 1c8a 1c8e 1c92 1c96 1c9a
1c9e 1ca2 1ca6 1cb2 1cb6 1cba 1cbd 1cc1
1cc5 1cc7 1ccb 1ccf 1cd3 1cd6 1cda 1cde
1ce0 1ce1 1ce6 1 1cea 1cee 1cf2 1cf5
1cf9 1cfd 1d01 1d05 1d08 1d0c 1d10 1d13
1d16 1d1b 1d1e 1d21 1d24 1d25 1d2a 1d2b
1 1d2d 1d32 1d36 1d3a 1d3d 1d42 1d47
1d4a 1d4e 1d4f 1d54 1d57 1d5c 1d5d 1d62
1d65 1d6a 1d6b 1d70 1d73 1d77 1d78 1d7d
1d80 1d85 1d86 1d8b 1d8e 1d92 1d96 1d99
1d9c 1d9d 1d9f 1da0 1da5 1da6 1dab 1dad
1db1 1db4 1db6 1dba 1dbc 1dc8 1dca 1dcc
1dcd 1dd2 1dd6 1dd8 1de4 1de6 1dea 1dee
1df1 1df5 1df8 1dfc 1dff 1e00 1e05 1e08
1e0c 1e0f 1e10 1e15 1 1e18 1e1d 1e21
1e25 1e29 1e2d 1e30 1e31 1e33 1e37 1e3b
1e3e 1e3f 1e41 1e44 1e48 1e4c 1e50 1e53
1e57 1e5b 1e5e 1e5f 1e61 1e64 1e67 1e68
1e6d 1e70 1e74 1e78 1e7b 1e7f 1e83 1e86
1e87 1e89 1e8c 1e8f 1e90 1e95 1 1e98
1e9d 1ea1 1ea5 1ea8 1eab 1eac 1eae 1eb1
1eb6 1eb7 1ebc 1ec0 1ec4 1ec7 1ec8 1eca
1ecd 1ed0 1ed1 1ed6 1eda 1edd 1ee1 1ee3
1ee7 1eea 1eee 1ef1 1ef5 1ef7 1efb 1efe
1f00 1f04 1f08 1f0b 1f0c 1f0e 1f11 1f14
1f15 1f1a 1f1e 1f22 1f26 1f29 1f2d 1f31
1f34 1f35 1f37 1f3b 1f3f 1f42 1f43 1f45
1f48 1f4c 1f4e 1f52 1f55 1f59 1f5d 1f60
1f61 1f63 1f67 1f6a 1f6e 1f71 1f72 1f74
1f75 1f7a 1f7e 1f82 1f85 1f86 1f88 1f8b
1f8e 1f8f 1f94 1f98 1f9c 1f9f 1fa3 1fa7
1faa 1fab 1fad 1fb0 1fb3 1fb4 1 1fb9
1fbe 1fc2 1fc6 1fc9 1fcc 1fcd 1fcf 1fd2
1fd7 1fd8 1fdd 1fe1 1fe4 1fe8 1fea 1fee
1ff1 1ff5 1ff9 1ffa 1fff 2003 2007 200b
200f 2012 2016 201a 201e 2022 2026 202a
202e 2032 2036 203a 203e 2042 2046 204a
204e 2052 2056 205a 205e 2062 2066 206a
206e 2072 2076 207a 207e 2082 2086 208a
208e 2092 2096 2099 209a 209c 209f 20a2
20a5 20a6 20a8 20ac 20ae 20b2 20b5 20b9
20bd 20c1 20c5 20c9 20d5 20d9 20de 20e2
20e5 20e6 20eb 20ef 20f3 20f6 20f9 20fd
20ff 2103 2107 210a 210d 2111 2113 2117
211b 211e 2120 2124 2128 212c 212f 2133
2137 2139 213a 213f 1 2143 2147 214b
214e 2152 2156 215a 215e 2161 2165 2169
216c 216f 2174 2177 217a 217d 217e 2183
2184 1 2186 218b 218f 2193 2196 219b
21a0 21a3 21a7 21a8 21ad 21b0 21b5 21b6
21bb 21be 21c3 21c4 21c9 21cc 21d0 21d1
21d6 21d9 21de 21df 21e4 21e7 21eb 21ef
21f2 21f5 21f6 21f8 21f9 21fe 21ff 2204
2206 220a 220d 220f 2213 2215 2221 2223
2225 2226 222b 222f 2231 223d 223f 2243
2247 224a 224e 2252 2255 2258 225d 2260
2263 2266 2267 226c 226d 1 226f 2274
2278 227c 227f 2284 2289 228c 2290 2291
2296 2297 229c 229e 22a2 22a5 22a9 22ac
22b0 22b2 22b6 22b9 22bb 22bf 22c3 22c6
22c8 22cc 22cf 22d1 22d5 22d9 22dc 22e0
22e4 22e7 22e8 22ea 22ed 22f0 22f1 22f6
22f9 22fd 2301 2304 2305 2307 230b 230e
2312 2315 2316 2318 2319 231e 1 2321
2326 232a 232e 2331 2332 2334 2337 233a
233b 2340 2344 2348 234c 234f 2353 2357
235a 235b 235d 2361 2365 2368 2369 236b
236c 236e 2371 2375 2377 237b 237e 2382
2386 238a 238e 2392 2396 239a 239e 23a2
23a6 23aa 23ae 23b2 23b6 23ba 23be 23c2
23c6 23ca 23ce 23d2 23d6 23da 23de 23e1
23e4 23e5 23e7 23eb 23ef 23f1 23f5 23f9
23fc 23fd 23ff 2402 2405 2406 240b 240f
2413 2416 2417 2419 241c 241f 2420 2425
2429 242d 2431 2434 2438 243c 243f 2440
2442 2446 244a 244d 244e 2450 2451 2453
2456 245a 245c 2460 2463 2467 246b 246f
2473 2477 247b 247f 2483 2487 248b 248f
2493 2497 249b 249f 24a3 24a7 24ab 24af
24b3 24b7 24bb 24bf 24c3 24c6 24c9 24ca
24cc 24d0 24d2 24d6 24da 24dd 24e1 24e5
24e8 24e9 24eb 24ee 24f1 24f2 24f7 24fb
24ff 2502 2506 250a 250d 2511 2515 2517
251b 251f 2522 2523 2525 2528 252b 252c
2531 2535 2539 253c 253d 253f 2542 2545
2546 1 254b 2550 2554 2558 255b 255f
2563 2566 256a 256c 2570 2574 2577 257b
257f 2582 2585 2588 2589 258e 2592 2596
2599 259c 25a0 25a2 25a6 25a9 25ad 25b1
25b4 25b8 25bc 25bf 25c2 25c7 25ca 25cd
25d0 25d1 25d6 25d7 1 25d9 25de 25e2
25e6 25e9 25ee 25f3 25f6 25fa 25fb 2600
2603 2608 2609 260e 2611 2616 2617 261c
261f 2623 2627 262a 262b 2630 2631 2636
2638 263c 263f 2643 2647 264b 264e 2652
2654 2658 265a 2666 266a 266c 266d 2676 
af0
2
0 1 a 6 15 19 20 :3 6
15 19 20 :3 6 16 1a 21 :3 6
19 :3 6 16 1a 21 :3 6 17 1b
22 :3 6 12 :3 6 10 :3 6 11 :3 6
10 :3 6 15 :3 6 13 :3 6 13 :3 6
18 :3 6 16 :3 6 19 1d 24 :3 6
14 18 1f :2 6 1 3 a :2 1
4 :2 14 23 14 :2 4 :2 d 1c d
:2 4 :2 b 1a b :2 4 :2 c 1b c
:2 4 :2 b 1a b :2 4 :2 10 1f 10
:2 4 :2 e 1d e :2 4 :2 e 1d e
:2 4 :2 13 22 13 :2 4 :2 11 20 11
:2 4 e 17 16 :2 e :2 4 d 16
15 :2 d :2 4 f 18 17 :2 f :2 4
10 19 18 :2 10 :2 4 :3 10 :2 4 :3 15
:2 4 18 21 20 :2 18 :2 4 11 1a
19 :2 11 :2 4 12 1b 1a :2 12 :2 4
12 1b 1a :2 12 :2 4 f 18 17
:2 f :2 4 f 18 17 :2 f :2 4 10
19 18 :2 10 :2 4 f 18 17 :2 f
:2 4 :2 12 21 12 :2 4 :2 b 1a b
:2 4 :2 f 1e f :2 4 :2 f 1e f
:2 4 :2 d 1c d :2 4 :2 a 19 a
:2 4 :2 14 23 14 :2 4 :2 13 22 13
:2 4 :2 12 21 12 :2 4 :2 f 1e f
:2 4 17 20 1f :2 17 :2 4 18 21
20 :2 18 :2 4 :3 9 :2 4 :3 9 :2 4 :2 f
19 f :2 4 :2 f 19 f :2 4 :2 f
1e f :2 4 :2 16 25 16 :2 4 :2 11
20 11 :2 4 :2 f 1e f :2 4 :2 14
23 14 :2 4 :2 11 20 11 :2 4 :2 f
1e f :2 4 :2 d 1c d :2 4 :2 b
1a b :2 4 :2 f 19 f 4 7
:2 11 23 :2 2d 3a :2 23 :3 7 :2 d 19
25 :2 2f 43 :2 25 54 5a 5d :2 54
6b 81 8 :2 6b d 10 :2 6b 1c
1f :2 6b b e :2 6b 13 16 :2 6b
1b 1e :2 6b 2a 2d :2 6b b e
:2 6b 13 16 :2 6b 1b 1e :2 6b 2b
2e :2 6b b e :2 6b 13 16 1f
27 :2 1f 3c :2 16 :2 6b b e :2 6b
13 16 :2 6b 1b 1e :2 6b 2b 2e
:2 6b b e :2 6b 13 16 :2 6b 1b
1e :2 6b 2c 2f :2 6b b e :2 6b
13 16 1f 27 :2 1f 35 :2 16 :2 6b
b e :2 6b 13 16 1f 27 :2 1f
33 :2 16 :2 6b b e :2 6b 13 16
1f 27 :2 1f 34 :2 16 :2 6b b e
:2 6b 13 16 1f 27 :2 1f 33 :2 16
:2 6b b e :2 6b 13 16 1f 27
:2 1f 38 :2 16 :2 6b b e :2 6b 13
16 1f 27 :2 1f 36 :2 16 :2 6b b
e :2 6b 13 16 1f 27 :2 1f 36
:2 16 :2 6b b e :2 6b 13 16 1f
27 :2 1f 3b :2 16 :2 6b b e :2 6b
13 16 1f 27 :2 1f 39 :2 16 :2 6b
b e :2 6b 13 16 :2 6b 1b 1e
:2 6b 2e 31 :2 6b b e :2 6b 13
16 :2 6b 1b 1e :2 6b 29 2c :2 6b
31 34 :2 6b :2 7 4a :2 4 7 :2 11
23 :2 2d 39 3c 4c 4e 4f :2 4c
:2 23 :3 7 :2 d 12 1e 5c 5f :2 1e
:2 7 52 :2 4 7 10 1b :2 7 21
1e :2 2a :2 1e 7 10 :2 19 25 2c
35 40 :2 2c 44 :2 4d :2 25 24 5c
:2 10 :2 7 15 1c 25 30 :2 1c 34
:2 3d :2 15 :2 7 14 :2 7 13 7 4
34 a 13 1e :2 a 24 21 :2 2d
:2 21 7 14 1b 24 2f :2 1b 33
:2 3c :2 14 13 :2 7 15 1c 25 30
:2 1c 34 :2 3d :2 15 14 :2 7 15 7
37 34 7 15 1e 29 :2 15 :2 7
14 :2 7 13 7 :5 4 12 :2 4 12
:2 4 15 :2 4 18 :2 4 c :2 4 c
:2 4 12 :2 4 12 4 7 10 1b
:2 7 1e 20 :2 1e 7 1e 7 :7 a
12 19 a 12 19 :10 a f 10
24 fffa a :2 13 1f a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 56 59 5a :2 56 :2 2c :3 10 :2 16
1b 27 66 69 :2 27 76 79 :2 27
7d 80 :2 27 8c 8f :2 27 97 9a
:2 27 a6 a9 b0 b9 bc :2 a9 :2 27
:2 10 5d :2 d :2 a :4 16 :2 a 7 :3 22
b :2 14 1f 1d :2 28 :2 1d a d
:2 17 29 :2 33 3f 42 52 55 56
:2 52 :2 29 :3 d :2 13 18 24 63 66
:2 24 :2 d 59 :3 a 11 :2 1a a 38
:2 7 22 11 1d 29 36 a 19
30 f 1d 28 32 a 16 25
f 10 24 :2 a :2 13 1f a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 56 59 5a :2 56 :2 2c :3 10
:2 16 1b 27 66 69 :2 27 76 79
:2 27 7d 80 :2 27 8c 8f :2 27 97
9a :2 27 a6 a9 b0 b9 bc :2 a9
:2 27 :2 10 5d :2 d :2 a :4 16 :2 a 7
:3 4 b :2 14 1f 1d :2 28 :2 1d a
d :2 17 29 :2 33 3f 42 52 55
56 :2 52 :2 29 :3 d :2 13 18 24 63
66 :2 24 :2 d 59 :3 a 11 :2 1a a
7 38 d 22 2b 33 :2 22 :2 d
37 3a :2 37 d :2 17 29 :2 33 3f
42 52 55 56 :2 52 :2 29 :3 d :2 13
18 24 63 66 :2 24 :2 d 59 :3 a
11 :2 1a a 3c 38 :2 7 11 2a
3a 10 24 a 11 d 19 :3 17
d :2 16 22 d 1e d :2 16 22
d :4 a 7 f d :2 16 22 d
1d :2 a :2 f d :2 16 22 d 10
:2 1a 2c :2 36 42 46 56 59 5a
:2 56 :2 2c :3 10 :2 16 1b 27 66 69
:2 27 76 79 :2 27 7d 80 :2 27 8c
8f :2 27 97 9a :2 27 a6 a9 b0
b9 bc :2 a9 :2 27 :2 10 5d :2 d :2 a
:4 16 :2 a 7 :3 4 f 1d 26 2e
:2 1d :2 f e 33 36 :2 33 :3 a 14
2b 12 29 3d :2 d :2 16 22 d
a 12 10 :2 19 25 10 20 :2 d
:2 12 10 :2 19 25 10 13 :2 1d 2f
:2 39 45 49 59 5c 5d :2 59 :2 2f
:3 13 :2 19 1e 2a 69 6c :2 2a 79
7c :2 2a 80 83 :2 2a 8f 92 :2 2a
9a 9d :2 2a a9 ac b3 bc bf
:2 ac :2 2a :2 13 60 :2 10 :2 d :4 19 :2 d
a :3 38 e :2 17 22 20 :2 2b :2 20
d 10 :2 1a 2c :2 36 42 45 55
58 59 :2 55 :2 2c :3 10 :2 16 1b 27
66 69 :2 27 :2 10 5c :3 d 14 :2 1d
d 3b :2 a 38 :3 7 1a 30 :2 1a
7 :4 4 e 17 2d 36 4e 5b
c 1b 27 36 c 1a 27 :2 7
:2 10 1c 7 4 c a :2 13 1f
a 1a :2 7 :2 c a :2 13 1f a
d :2 17 29 :2 33 3f 43 53 56
57 :2 53 :2 29 :3 d :2 13 18 24 63
66 :2 24 73 76 :2 24 7a 7d :2 24
89 8c :2 24 94 97 :2 24 a3 a6
ad b6 b9 :2 a6 :2 24 :2 d 5a :2 a
:2 7 :4 13 :2 7 4 :3 1 7 10 1a
:2 7 1d 1f :2 1d b 14 1c :2 b
1f 21 :2 1f a 29 32 3a :2 29
3d 3f :2 3d 28 :3 a 14 1d 2c
:2 14 2f 31 3a 42 :2 31 :2 14 a
42 :2 7 a 13 1d :2 a 20 22
:2 20 :4 e d 10 19 21 :2 10 24
26 :2 24 10 1c 25 33 :2 1c 36
38 41 49 :2 38 :2 1c 10 28 :2 d
25 17 1b 2f 3d 16 21 32
43 4b 52 6c 74 7b :2 10 :2 19
25 10 d 15 13 :2 1c 28 13
23 :2 10 :2 15 13 :2 1c 28 13 16
:2 20 32 :2 3c 48 4c 5c 5f 60
:2 5c :2 32 :3 16 :2 1c 21 2d 6c 6f
:2 2d 7c 7f :2 2d 83 86 :2 2d 92
95 :2 2d 9d a0 :2 2d ac af b6
bf c2 :2 af :2 2d :2 16 63 :2 13 :2 10
:4 1c :2 10 d :7 a 24 d 16 1f
:2 d 22 24 :2 22 d 19 22 2c
:2 19 2f 31 :2 19 d a 26 10
19 22 :2 10 25 27 :2 25 d 19
22 31 :2 19 34 36 3f 49 :2 36
:2 19 d a 29 26 10 19 22
:2 10 25 27 :2 25 d 19 22 2e
:2 19 31 33 3c 46 :2 33 :2 19 d
29 26 :2 a :4 7 a 13 1d :2 a
20 22 :2 20 d :2 17 29 :2 33 3f
42 52 55 56 :2 52 :2 29 :3 d :2 13
18 24 63 66 :2 24 :2 d 59 :3 a
11 :2 1a a 24 :2 7 21 :2 4 7
10 1b :2 7 1e 20 :2 1e 7 15
1c 25 30 :2 1c 34 :2 3d :2 15 14
7 4 22 b 12 1e 21 :2 b
24 26 :2 24 a 30 39 46 :2 30
49 4b :2 49 2f :2 a 11 24 36
49 56 62 f 24 30 3d 4b
f 10 1d 2e 38 49 57 :2 a
:2 13 1f a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 56 59
5a :2 56 :2 2c :3 10 :2 16 1b 27 66
69 :2 27 76 79 :2 27 7d 80 :2 27
8c 8f :2 27 97 9a :2 27 a6 a9
b0 b9 bc :2 a9 :2 27 :2 10 5d :2 d
:2 a :4 16 :2 a 7 :3 4e b :2 14 1f
1d :2 28 :2 1d a :4 3d 3c :3 a 18
1f 28 33 :2 1f 37 :2 40 :2 18 17
a e :2 19 21 2f 3b :2 e 3f
42 :2 3f d 4a :2 55 5d 12 21
:2 4a 25 28 :2 25 49 :2 d 10 17
23 26 :2 10 29 2b :2 29 13 1c
26 :2 13 29 2b :2 29 13 20 13
2d :3 10 1f 10 2f :2 d 2b 10
19 24 :2 10 27 29 :2 27 10 1f
:2 28 2f 38 44 :2 2f 48 :2 51 :2 1f
1e 10 2b :2 d 10 19 27 :2 10
2c 2a 35 3f :2 2c :2 2a 13 1c
2a :2 13 2d 2f :2 2d 35 :2 40 48
56 63 :2 35 67 6a :2 67 :2 13 16
1d 29 2c :2 16 2f 31 :2 2f 16
22 16 35 :3 13 :3 26 :2 13 21 :2 2a
:2 13 26 32 3f 4d 58 62 6e
22 2f 3d 49 55 22 2f 37
45 4d 56 5e 22 2e 3a 47
52 22 30 3a 49 55 22 2b
3a :2 22 3e 41 47 :2 26 13 6c
:2 10 1a 2f 35 19 2d 13 1a
16 22 :3 20 16 :2 1f 2b 16 27
16 :2 1f 2b 16 :4 13 10 18 16
:2 1f 2b 16 26 :2 13 :2 18 16 :2 1f
2b 16 19 :2 23 35 :2 3f 4b 4f
5f 62 63 :2 5f :2 35 :3 19 :2 1f 24
30 6f 72 :2 30 7f 82 :2 30 86
89 :2 30 95 98 :2 30 a0 a3 :2 30
af b2 b9 c2 c5 :2 b2 :2 30 :2 19
66 :2 16 :2 13 :4 1f :2 13 10 :3 42 13
:2 1d 2f :2 39 45 48 58 5b 5c
:2 58 :2 2f :3 13 :2 19 1e 2a 69 6c
:2 2a :2 13 5f :3 10 17 10 42 :2 d
:4 a 5e :2 7 4e 22 :2 4 8 11
1c :2 8 1f 21 :2 1f 7 28 31
3f :2 28 44 42 4d 5c :2 44 :2 42
27 :2 7 a 13 1d :2 a 20 23
:2 20 a 18 :2 21 28 31 3c :2 28
40 49 53 :2 40 :2 18 17 a 25
:3 7 1a 24 31 3f 4a 54 20
2e 3c 4b 55 5d 20 28 34
41 4c 57 20 2e 3a 4b 57
20 26 :2 1a 7 4 60 a 13
1e :2 a 21 23 :2 21 a 13 1d
:2 a 20 23 :2 20 a 18 :2 21 28
31 3c :2 28 40 49 53 :2 40 :2 18
17 a 25 :3 7 1a 24 39 47
52 5c 20 2e 3c 4b 55 5d
20 28 34 41 4c 57 20 2e
3a 4b 57 20 26 :2 1a 7 25
60 :2 4 7 10 21 :2 7 24 26
:2 24 7 :2 10 1c :2 25 7 4 2a
a 13 24 :2 a 27 2a :2 27 30
39 4a :2 30 4d 50 :2 4d :2 a 7
:2 10 1c :2 25 7 54 2a :2 4 7
:2 10 19 1b :2 19 7 :2 10 1c 7
1f :2 4 7 :2 11 23 :2 2d 39 3c
4c 4f 50 :2 4c :2 23 :3 7 :2 d 12
1e 5d 60 :2 1e 6d 70 :2 1e 74
77 :2 1e 81 84 :2 8d :2 1e :2 7 53
:3 4 b :2 14 4 :9 1 
af0
4
0 :2 1 :6 3 :6 4
:6 5 :4 6 :6 7 :6 8
:4 9 :4 a :4 b :4 c
:4 d :4 e :4 f :4 10
:4 11 :6 12 :6 13 2
:2 14 :2 1 :6 16 :6 17
:6 18 :6 19 :6 1a :6 1b
:6 1c :6 1d :6 1e :6 1f
:7 20 :7 21 :7 22 :7 23
:5 24 :5 25 :7 26 :7 27
:7 28 :7 29 :7 2a :7 2b
:7 2c :7 2d :6 2e :6 2f
:6 30 :6 31 :6 32 :6 33
:6 34 :6 35 :6 36 :6 37
:7 38 :7 39 :5 3a :5 3b
:6 3c :6 3d :6 3e :6 3f
:6 40 :6 41 :6 42 :6 43
:6 44 :6 45 :6 46 :6 47
:b 49 :11 4a 4b :2 4a
:2 4b :2 4a :2 4b :2 4a
:2 4c :2 4a :2 4c :2 4a
:2 4c :2 4a :2 4c :2 4a
:2 4d :2 4a :2 4d :2 4a
:2 4d :2 4a :2 4d :2 4a
:2 4e :2 4a :9 4e :2 4a
:2 4f :2 4a :2 4f :2 4a
:2 4f :2 4a :2 4f :2 4a
:2 50 :2 4a :2 50 :2 4a
:2 50 :2 4a :2 50 :2 4a
:2 51 :2 4a :9 51 :2 4a
:2 52 :2 4a :9 52 :2 4a
:2 53 :2 4a :9 53 :2 4a
:2 54 :2 4a :9 54 :2 4a
:2 55 :2 4a :9 55 :2 4a
:2 56 :2 4a :9 56 :2 4a
:2 57 :2 4a :9 57 :2 4a
:2 58 :2 4a :9 58 :2 4a
:2 59 :2 4a :9 59 :2 4a
:2 5a :2 4a :2 5a :2 4a
:2 5a :2 4a :2 5a :2 4a
:2 5b :2 4a :2 5b :2 4a
:2 5b :2 4a :2 5b :2 4a
:2 5b :4 4a :3 49 :11 5e
:b 5f :3 5e :b 64 :14 65
:d 67 :3 68 :3 69 6a
64 :b 6a :e 6b :e 6d
:3 6f 6a 64 :7 71
:3 72 :3 73 70 :3 64
:3 76 :3 77 :3 78 :3 79
:3 7a :3 7b :3 7c :3 7d
:9 7f :3 80 83 84
85 86 87 88
:3 89 :3 8a 8b 8c
8d 8e 90 91
92 93 94 95
96 97 98 99
9a 9b 9c :2 9d
83 :5 9e 81 a0
:5 a1 :3 a0 :2 a2 :5 a4
:11 a5 :24 a6 :3 a5 :2 a3
:6 a2 9f :3 7f :a ab
:11 ac :b ad :3 ac :5 af
:3 ab 7f :4 b3 :3 b4
:4 b5 :3 b6 b7 :2 b8
b3 :5 b9 b2 bb
:5 bc :3 bb :2 bd :5 bf
:11 c0 :24 c1 :3 c0 :2 be
:6 bd ba :3 b1 :a c6
:11 c7 :b c8 :3 c7 :5 ca
cb c6 :c cb :11 cc
:b cd :3 cc :5 cf cb
:3 c6 :3 d3 :2 d4 d3
:6 d5 :5 d6 d5 :5 d8
d7 :3 d5 d2 db
:5 dc :3 db :2 dd :5 df
:11 e0 :24 e1 :3 e0 :2 de
:6 dd da :3 b1 :10 e6
:2 e8 :3 e9 e8 :5 ea
e7 ec :5 ed :3 ec
:2 ee :5 f0 :11 f1 :24 f2
:3 f1 :2 ef :6 ee eb
:3 e6 :a f7 :11 f8 :b f9
:3 f8 :5 fb :3 f7 :3 e6
:6 ff b1 :3 7f :6 104
:4 105 :3 106 104 :5 107
103 109 :5 10a :3 109
:2 10b :5 10d :11 10e :24 10f
:3 10e :2 10c :6 10b 108
:3 48 :9 114 :16 115 :f 116
:3 115 :9 119 :5 11b :9 11c
:f 11d :3 11c 11b :4 121
:9 122 121 :5 123 120
125 :5 126 :3 125 :2 127
:5 129 :11 12a :24 12b :3 12a
:2 128 :6 127 124 :4 11f
:3 11b 119 :9 131 :b 132
133 131 :9 133 :f 134
135 133 131 :9 135
:f 136 135 :3 131 130
:3 119 :9 13a :11 13b :b 13c
:3 13b :5 13e :3 13a :3 114
:9 142 :e 143 145 142
:17 145 :6 147 :5 148 149
:6 14a 147 :5 14b 146
14d :5 14e :3 14d :2 14f
:5 151 :11 152 :24 153 :3 152
:2 150 :6 14f 14c :3 145
:11 158 :e 159 :11 15b :2 15c
:2 15b :4 15c :3 15b :a 15d
:9 15e :3 15f :3 15e :3 161
:3 15d 15c :9 164 :10 165
:3 164 :d 168 :17 169 :a 16a
:3 16b :3 16a :5 16d :5 16e
:8 16f :5 170 :7 171 :5 172
:5 173 :8 174 :3 16f :3 169
:3 178 :2 179 178 :6 17a
:5 17b 17a :5 17d 17c
:3 17a 177 180 :5 181
:3 180 :2 182 :5 184 :11 185
:24 186 :3 185 :2 183 :6 182
17f :3 168 :11 18b :b 18c
:3 18b :3 18e :3 168 163
:3 15b :3 158 145 :3 142
:1a 194 :9 195 :12 196 :3 195
:7 19a :6 19b :6 19c :5 19d
:2 19e :3 19a 19f 194
:9 19f :9 1a0 :12 1a1 :3 1a0
:7 1d7 :6 1d8 :6 1d9 :5 1da
:2 1db :3 1d7 19f :3 194
:9 1df :7 1e0 1e1 1df
:14 1e1 :7 1e2 1e1 :3 1df
:7 1e5 :5 1e6 :3 1e5 :11 1e9
:19 1ea :3 1e9 :5 1ec :2 48
:7 1 
2678
4
:3 0 1 :3 0 2
:a 0 aeb 1 :7 0
5 37 0 3
4 :3 0 5 :3 0
6 :3 0 3 :5 0
1 7 6 :3 0
9 6b 0 7
4 :3 0 5 :3 0
6 :3 0 7 :5 0
1 d c :3 0
4 :3 0 5 :3 0
6 :3 0 8 :5 0
1 13 12 :3 0
d 98 0 b
a :3 0 9 :7 0
17 16 :3 0 4
:3 0 5 :3 0 6
:3 0 b :5 0 1
1d 1c :3 0 11
c5 0 f 4
:3 0 5 :3 0 6
:3 0 c :5 0 1
23 22 :3 0 a
:3 0 d :7 0 27
26 :3 0 15 eb
0 13 a :3 0
e :7 0 2b 2a
:3 0 a :3 0 f
:7 0 2f 2e :3 0
19 111 0 17
a :3 0 10 :7 0
33 32 :3 0 a
:3 0 11 :7 0 37
36 :3 0 1d 137
0 1b a :3 0
12 :7 0 3b 3a
:3 0 a :3 0 13
:7 0 3f 3e :3 0
21 15d 0 1f
a :3 0 14 :7 0
43 42 :3 0 a
:3 0 15 :7 0 47
46 :3 0 25 :2 0
23 4 :3 0 5
:3 0 6 :3 0 16
:5 0 1 4d 4c
:3 0 4 :3 0 5
:3 0 6 :3 0 17
:5 0 1 53 52
:3 0 18 :3 0 a
:3 0 55 57 0
aeb 2 58 :2 0
39 1c1 0 37
a :3 0 5b :7 0
9 :3 0 5f 5c
5d ae9 0 19
:6 0 3d 1fd 0
3b a :3 0 61
:7 0 d :3 0 65
62 63 ae9 0
1a :6 0 a :3 0
67 :7 0 e :3 0
6b 68 69 ae9
0 1b :6 0 41
239 0 3f a
:3 0 6d :7 0 f
:3 0 71 6e 6f
ae9 0 1c :6 0
a :3 0 73 :7 0
10 :3 0 77 74
75 ae9 0 1d
:6 0 45 275 0
43 a :3 0 79
:7 0 11 :3 0 7d
7a 7b ae9 0
1e :6 0 a :3 0
7f :7 0 12 :3 0
83 80 81 ae9
0 1f :6 0 49
2b1 0 47 a
:3 0 85 :7 0 13
:3 0 89 86 87
ae9 0 20 :6 0
a :3 0 8b :7 0
14 :3 0 8f 8c
8d ae9 0 21
:6 0 24 :2 0 4d
a :3 0 91 :7 0
15 :3 0 95 92
93 ae9 0 22
:6 0 6 :3 0 24
:2 0 4b 97 99
:6 0 9c 9a 0
ae9 0 23 :6 0
27 :2 0 51 6
:3 0 4f 9e a0
:6 0 a3 a1 0
ae9 0 25 :6 0
27 :2 0 55 6
:3 0 53 a5 a7
:6 0 aa a8 0
ae9 0 26 :6 0
5b 344 0 59
6 :3 0 57 ac
ae :6 0 b1 af
0 ae9 0 28
:6 0 24 :2 0 5d
2a :3 0 b3 :7 0
b6 b4 0 ae9
0 29 :6 0 2a
:3 0 b8 :7 0 bb
b9 0 ae9 0
2b :6 0 24 :2 0
61 6 :3 0 5f
bd bf :6 0 c2
c0 0 ae9 0
2c :6 0 24 :2 0
65 6 :3 0 63
c4 c6 :6 0 c9
c7 0 ae9 0
2d :6 0 24 :2 0
69 6 :3 0 67
cb cd :6 0 d0
ce 0 ae9 0
2e :6 0 31 :2 0
6d 6 :3 0 6b
d2 d4 :6 0 d7
d5 0 ae9 0
2f :6 0 31 :2 0
71 6 :3 0 6f
d9 db :6 0 de
dc 0 ae9 0
30 :6 0 34 :2 0
75 6 :3 0 73
e0 e2 :6 0 e5
e3 0 ae9 0
32 :6 0 31 :2 0
79 6 :3 0 77
e7 e9 :6 0 ec
ea 0 ae9 0
33 :6 0 37 :2 0
7d 6 :3 0 7b
ee f0 :6 0 f3
f1 0 ae9 0
35 :6 0 37 :2 0
7f a :3 0 f5
:7 0 f9 f6 f7
ae9 0 36 :6 0
37 :2 0 81 a
:3 0 fb :7 0 ff
fc fd ae9 0
38 :6 0 37 :2 0
83 a :3 0 101
:7 0 105 102 103
ae9 0 39 :6 0
37 :2 0 85 a
:3 0 107 :7 0 10b
108 109 ae9 0
3a :6 0 37 :2 0
87 a :3 0 10d
:7 0 111 10e 10f
ae9 0 3b :6 0
37 :2 0 89 a
:3 0 113 :7 0 117
114 115 ae9 0
3c :6 0 37 :2 0
8b a :3 0 119
:7 0 11d 11a 11b
ae9 0 3d :6 0
37 :2 0 8d a
:3 0 11f :7 0 123
120 121 ae9 0
3e :6 0 37 :2 0
8f a :3 0 125
:7 0 129 126 127
ae9 0 3f :6 0
24 :2 0 91 a
:3 0 12b :7 0 12f
12c 12d ae9 0
40 :6 0 24 :2 0
95 6 :3 0 93
131 133 :6 0 136
134 0 ae9 0
41 :6 0 9b 5b2
0 99 6 :3 0
97 138 13a :6 0
13d 13b 0 ae9
0 42 :6 0 37
:2 0 9d 2a :3 0
13f :7 0 142 140
0 ae9 0 43
:6 0 2a :3 0 144
:7 0 147 145 0
ae9 0 44 :6 0
37 :2 0 9f 46
:3 0 149 :7 0 14d
14a 14b ae9 0
45 :6 0 37 :2 0
a1 46 :3 0 14f
:7 0 153 150 151
ae9 0 47 :6 0
37 :2 0 a3 a
:3 0 155 :7 0 159
156 157 ae9 0
48 :6 0 37 :2 0
a5 a :3 0 15b
:7 0 15f 15c 15d
ae9 0 49 :6 0
37 :2 0 a7 a
:3 0 161 :7 0 165
162 163 ae9 0
4a :6 0 37 :2 0
a9 a :3 0 167
:7 0 16b 168 169
ae9 0 4b :6 0
37 :2 0 ab a
:3 0 16d :7 0 171
16e 16f ae9 0
4c :6 0 37 :2 0
ad a :3 0 173
:7 0 177 174 175
ae9 0 4d :6 0
37 :2 0 af a
:3 0 179 :7 0 17d
17a 17b ae9 0
4e :6 0 37 :2 0
b1 a :3 0 17f
:7 0 183 180 181
ae9 0 4f :6 0
37 :2 0 b3 a
:3 0 185 :7 0 189
186 187 ae9 0
50 :6 0 190 191
0 b5 46 :3 0
18b :7 0 18f 18c
18d ae9 0 51
:6 0 52 :3 0 53
:3 0 52 :3 0 54
:3 0 193 194 0
2 :4 0 b7 195
197 192 199 198
:2 0 55 :3 0 56
:3 0 19b 19c 0
57 :4 0 52 :3 0
58 :3 0 19f 1a0
0 2 :4 0 b9
1a1 1a3 59 :4 0
5a :2 0 5b :3 0
bb 1a6 1a8 :3 0
5c :4 0 5a :2 0
5d :4 0 be 1ab
1ad :3 0 5a :2 0
3 :3 0 c1 1af
1b1 :3 0 5a :2 0
5d :4 0 c4 1b3
1b5 :3 0 5a :2 0
5e :4 0 c7 1b7
1b9 :3 0 5a :2 0
5d :4 0 ca 1bb
1bd :3 0 5a :2 0
7 :3 0 cd 1bf
1c1 :3 0 5a :2 0
5d :4 0 d0 1c3
1c5 :3 0 5a :2 0
5e :4 0 d3 1c7
1c9 :3 0 5a :2 0
5d :4 0 d6 1cb
1cd :3 0 5a :2 0
8 :3 0 d9 1cf
1d1 :3 0 5a :2 0
5d :4 0 dc 1d3
1d5 :3 0 5a :2 0
5e :4 0 df 1d7
1d9 :3 0 5a :2 0
5f :3 0 60 :3 0
9 :3 0 e2 1dd
1df 61 :4 0 e4
1dc 1e2 e7 1db
1e4 :3 0 5a :2 0
5e :4 0 ea 1e6
1e8 :3 0 5a :2 0
5d :4 0 ed 1ea
1ec :3 0 5a :2 0
b :3 0 f0 1ee
1f0 :3 0 5a :2 0
5d :4 0 f3 1f2
1f4 :3 0 5a :2 0
5e :4 0 f6 1f6
1f8 :3 0 5a :2 0
5d :4 0 f9 1fa
1fc :3 0 5a :2 0
c :3 0 fc 1fe
200 :3 0 5a :2 0
5d :4 0 ff 202
204 :3 0 5a :2 0
5e :4 0 102 206
208 :3 0 5a :2 0
5f :3 0 60 :3 0
d :3 0 105 20c
20e 61 :4 0 107
20b 211 10a 20a
213 :3 0 5a :2 0
5e :4 0 10d 215
217 :3 0 5a :2 0
5f :3 0 60 :3 0
e :3 0 110 21b
21d 61 :4 0 112
21a 220 115 219
222 :3 0 5a :2 0
5e :4 0 118 224
226 :3 0 5a :2 0
5f :3 0 60 :3 0
f :3 0 11b 22a
22c 61 :4 0 11d
229 22f 120 228
231 :3 0 5a :2 0
5e :4 0 123 233
235 :3 0 5a :2 0
5f :3 0 60 :3 0
10 :3 0 126 239
23b 61 :4 0 128
238 23e 12b 237
240 :3 0 5a :2 0
5e :4 0 12e 242
244 :3 0 5a :2 0
5f :3 0 60 :3 0
11 :3 0 131 248
24a 61 :4 0 133
247 24d 136 246
24f :3 0 5a :2 0
5e :4 0 139 251
253 :3 0 5a :2 0
5f :3 0 60 :3 0
12 :3 0 13c 257
259 61 :4 0 13e
256 25c 141 255
25e :3 0 5a :2 0
5e :4 0 144 260
262 :3 0 5a :2 0
5f :3 0 60 :3 0
13 :3 0 147 266
268 61 :4 0 149
265 26b 14c 264
26d :3 0 5a :2 0
5e :4 0 14f 26f
271 :3 0 5a :2 0
5f :3 0 60 :3 0
14 :3 0 152 275
277 61 :4 0 154
274 27a 157 273
27c :3 0 5a :2 0
5e :4 0 15a 27e
280 :3 0 5a :2 0
5f :3 0 60 :3 0
15 :3 0 15d 284
286 61 :4 0 15f
283 289 162 282
28b :3 0 5a :2 0
5e :4 0 165 28d
28f :3 0 5a :2 0
5d :4 0 168 291
293 :3 0 5a :2 0
16 :3 0 16b 295
297 :3 0 5a :2 0
5d :4 0 16e 299
29b :3 0 5a :2 0
5e :4 0 171 29d
29f :3 0 5a :2 0
5d :4 0 174 2a1
2a3 :3 0 5a :2 0
17 :3 0 177 2a5
2a7 :3 0 5a :2 0
5d :4 0 17a 2a9
2ab :3 0 5a :2 0
62 :4 0 17d 2ad
2af :3 0 180 19d
2b1 :2 0 2b3 185
2b4 19a 2b3 0
2b5 187 0 ae7
52 :3 0 53 :3 0
2b6 2b7 0 52
:3 0 63 :3 0 2b9
2ba 0 31 :2 0
2 :4 0 64 :2 0
65 :2 0 31 :2 0
189 2bf 2c1 :3 0
18c 2bb 2c3 2b8
2c5 2c4 :2 0 55
:3 0 66 :3 0 2c7
2c8 0 57 :4 0
67 :4 0 5a :2 0
5b :3 0 190 2cc
2ce :3 0 193 2c9
2d0 :2 0 2d2 196
2d3 2c6 2d2 0
2d4 198 0 ae7
5f :3 0 1f :3 0
37 :2 0 19a 2d5
2d8 68 :3 0 69
:2 0 6a :3 0 2da
2dc 0 19f 2db
2de :3 0 3c :3 0
6b :3 0 6c :3 0
2e1 2e2 0 6d
:3 0 5f :3 0 1f
:3 0 37 :2 0 1a2
2e5 2e8 68 :3 0
6e :3 0 2ea 2eb
0 1a5 2e4 2ed
2ee :2 0 6f :2 0
1a8 2e3 2f1 2e0
2f2 0 308 3a
:3 0 6d :3 0 5f
:3 0 1f :3 0 37
:2 0 1ab 2f6 2f9
68 :3 0 70 :3 0
2fb 2fc 0 1ae
2f5 2fe 2f4 2ff
0 308 1f :3 0
37 :2 0 301 302
0 308 3b :3 0
37 :2 0 304 305
0 308 71 :3 0
1b1 343 5f :3 0
1f :3 0 37 :2 0
1b6 309 30c 68
:3 0 69 :2 0 72
:3 0 30e 310 0
1bb 30f 312 :3 0
3b :3 0 6d :3 0
5f :3 0 1f :3 0
37 :2 0 1be 316
319 68 :3 0 72
:3 0 31b 31c 0
1c1 315 31e 31f
:2 0 314 320 0
333 1f :3 0 6d
:3 0 5f :3 0 1f
:3 0 37 :2 0 1c4
324 327 68 :3 0
73 :3 0 329 32a
0 1c7 323 32c
32d :2 0 322 32e
0 333 3a :3 0
37 :2 0 330 331
0 333 1ca 334
313 333 0 345
3a :3 0 5f :3 0
1f :3 0 37 :2 0
1ce 336 339 335
33a 0 342 1f
:3 0 37 :2 0 33c
33d 0 342 3b
:3 0 37 :2 0 33f
340 0 342 1d1
344 2df 308 0
345 0 342 0
345 1d5 0 ae7
4b :3 0 37 :2 0
346 347 0 ae7
30 :3 0 37 :4 0
349 34a 0 ae7
3f :3 0 37 :2 0
34c 34d 0 ae7
2b :4 0 34f 350
0 ae7 43 :4 0
352 353 0 ae7
44 :4 0 355 356
0 ae7 35 :4 0
358 359 0 ae7
51 :3 0 74 :2 0
35b 35c 0 ae7
5f :3 0 1f :3 0
37 :2 0 1d9 35e
361 75 :2 0 31
:2 0 1de 363 365
:3 0 2c :3 0 3
:3 0 367 368 0
415 76 :3 0 77
:3 0 78 :3 0 79
:3 0 7a :3 0 7b
:3 0 7c :3 0 7d
:3 0 7e :3 0 7c
:3 0 7d :3 0 7f
:3 0 80 :3 0 81
:3 0 82 :3 0 83
:3 0 2d :3 0 23
:3 0 25 :3 0 30
:3 0 2b :3 0 3f
:3 0 43 :3 0 44
:3 0 48 :3 0 49
:3 0 45 :3 0 47
:3 0 84 :3 0 85
:3 0 2c :4 0 86
1 :8 0 38f 87
:3 0 88 :3 0 38a
38b 0 89 :3 0
38c 38d 0 38f
1e1 3e2 8a :3 0
87 :3 0 88 :3 0
391 392 0 89
:3 0 393 394 0
396 1e4 398 1e6
397 396 :2 0 3e0
8b :3 0 87 :3 0
88 :3 0 39b 39c
0 89 :3 0 39d
39e 0 3d8 52
:3 0 53 :3 0 3a0
3a1 0 52 :3 0
63 :3 0 3a3 3a4
0 8c :2 0 2
:4 0 8d :2 0 65
:2 0 31 :2 0 1e8
3a9 3ab :3 0 1eb
3a5 3ad 3a2 3af
3ae :2 0 55 :3 0
66 :3 0 3b1 3b2
0 57 :4 0 8e
:4 0 5a :2 0 5b
:3 0 1ef 3b6 3b8
:3 0 5a :2 0 8f
:4 0 1f2 3ba 3bc
:3 0 5a :2 0 90
:4 0 1f5 3be 3c0
:3 0 5a :2 0 89
:3 0 1f8 3c2 3c4
:3 0 5a :2 0 91
:4 0 1fb 3c6 3c8
:3 0 5a :2 0 7d
:3 0 92 :3 0 31
:2 0 93 :2 0 1fe
3cb 3cf 202 3ca
3d1 :3 0 205 3b3
3d3 :2 0 3d5 208
3d6 3b0 3d5 0
3d7 20a 0 3d8
20c 3db :3 0 3db
0 3db 3da 3d8
3d9 :6 0 3dd 2
:3 0 20f 3df 211
3de 3dd :2 0 3e0
213 :2 0 3e2 0
3e2 3e1 38f 3e0
:6 0 415 1 :3 0
87 :3 0 88 :3 0
3e4 3e5 0 87
:3 0 75 :2 0 94
:3 0 3e7 3e9 0
218 3e8 3eb :3 0
3ec :2 0 52 :3 0
53 :3 0 3ee 3ef
0 52 :3 0 63
:3 0 3f1 3f2 0
31 :2 0 2 :4 0
95 :2 0 65 :2 0
31 :2 0 21b 3f7
3f9 :3 0 21e 3f3
3fb 3f0 3fd 3fc
:2 0 55 :3 0 66
:3 0 3ff 400 0
57 :4 0 96 :4 0
5a :2 0 5b :3 0
222 404 406 :3 0
225 401 408 :2 0
40a 228 40b 3fe
40a 0 40c 22a
0 412 18 :3 0
68 :3 0 97 :3 0
40e 40f 0 410
:2 0 412 22c 413
3ed 412 0 414
22f 0 415 231
601 76 :3 0 98
:3 0 78 :3 0 99
:3 0 79 :3 0 9a
:3 0 9b :3 0 2d
:3 0 23 :3 0 25
:3 0 38 :3 0 30
:3 0 2f :3 0 3e
:3 0 9c :3 0 9d
:3 0 3 :4 0 9e
1 :8 0 42d 87
:3 0 88 :3 0 428
429 0 89 :3 0
42a 42b 0 42d
235 480 8a :3 0
87 :3 0 88 :3 0
42f 430 0 89
:3 0 431 432 0
434 238 436 23a
435 434 :2 0 47e
8b :3 0 87 :3 0
88 :3 0 439 43a
0 89 :3 0 43b
43c 0 476 52
:3 0 53 :3 0 43e
43f 0 52 :3 0
63 :3 0 441 442
0 8c :2 0 2
:4 0 9f :2 0 65
:2 0 31 :2 0 23c
447 449 :3 0 23f
443 44b 440 44d
44c :2 0 55 :3 0
66 :3 0 44f 450
0 57 :4 0 a0
:4 0 5a :2 0 5b
:3 0 243 454 456
:3 0 5a :2 0 8f
:4 0 246 458 45a
:3 0 5a :2 0 90
:4 0 249 45c 45e
:3 0 5a :2 0 89
:3 0 24c 460 462
:3 0 5a :2 0 91
:4 0 24f 464 466
:3 0 5a :2 0 7d
:3 0 92 :3 0 31
:2 0 93 :2 0 252
469 46d 256 468
46f :3 0 259 451
471 :2 0 473 25c
474 44e 473 0
475 25e 0 476
260 479 :3 0 479
0 479 478 476
477 :6 0 47b 4
:3 0 263 47d 265
47c 47b :2 0 47e
267 :2 0 480 0
480 47f 42d 47e
:6 0 600 1 :3 0
87 :3 0 88 :3 0
482 483 0 87
:3 0 75 :2 0 94
:3 0 485 487 0
26c 486 489 :3 0
48a :2 0 52 :3 0
53 :3 0 48c 48d
0 52 :3 0 63
:3 0 48f 490 0
31 :2 0 2 :4 0
a1 :2 0 65 :2 0
31 :2 0 26f 495
497 :3 0 272 491
499 48e 49b 49a
:2 0 55 :3 0 66
:3 0 49d 49e 0
57 :4 0 a2 :4 0
5a :2 0 5b :3 0
276 4a2 4a4 :3 0
279 49f 4a6 :2 0
4a8 27c 4a9 49c
4a8 0 4aa 27e
0 4b1 18 :3 0
68 :3 0 97 :3 0
4ac 4ad 0 4ae
:2 0 4b1 71 :3 0
280 4e4 a3 :3 0
5f :3 0 38 :3 0
37 :2 0 283 4b3
4b6 286 4b2 4b8
a4 :2 0 37 :2 0
28a 4ba 4bc :3 0
52 :3 0 53 :3 0
4be 4bf 0 52
:3 0 63 :3 0 4c1
4c2 0 31 :2 0
2 :4 0 a5 :2 0
65 :2 0 31 :2 0
28d 4c7 4c9 :3 0
290 4c3 4cb 4c0
4cd 4cc :2 0 55
:3 0 66 :3 0 4cf
4d0 0 57 :4 0
a6 :4 0 5a :2 0
5b :3 0 294 4d4
4d6 :3 0 297 4d1
4d8 :2 0 4da 29a
4db 4ce 4da 0
4dc 29c 0 4e2
18 :3 0 68 :3 0
a7 :3 0 4de 4df
0 4e0 :2 0 4e2
29e 4e3 4bd 4e2
0 4e5 48b 4b1
0 4e5 2a1 0
600 9c :3 0 a8
:3 0 19 :3 0 9d
:3 0 3 :4 0 a9
1 :8 0 501 aa
:4 0 4ec :3 0 ab
:3 0 75 :2 0 2a6
4ef 4f0 :3 0 87
:3 0 88 :3 0 4f2
4f3 0 37 :2 0
4f4 4f5 0 4f7
2a9 4fe 87 :3 0
88 :3 0 4f8 4f9
0 ac :2 0 4fa
4fb 0 4fd 2ab
4ff 4f1 4f7 0
500 0 4fd 0
500 2ad 0 501
2b0 554 8a :3 0
87 :3 0 88 :3 0
503 504 0 89
:3 0 505 506 0
508 2b3 50a 2b5
509 508 :2 0 552
8b :3 0 87 :3 0
88 :3 0 50d 50e
0 89 :3 0 50f
510 0 54a 52
:3 0 53 :3 0 512
513 0 52 :3 0
63 :3 0 515 516
0 8c :2 0 2
:4 0 ad :2 0 65
:2 0 31 :2 0 2b7
51b 51d :3 0 2ba
517 51f 514 521
520 :2 0 55 :3 0
66 :3 0 523 524
0 57 :4 0 ae
:4 0 5a :2 0 5b
:3 0 2be 528 52a
:3 0 5a :2 0 8f
:4 0 2c1 52c 52e
:3 0 5a :2 0 90
:4 0 2c4 530 532
:3 0 5a :2 0 89
:3 0 2c7 534 536
:3 0 5a :2 0 91
:4 0 2ca 538 53a
:3 0 5a :2 0 7d
:3 0 92 :3 0 31
:2 0 93 :2 0 2cd
53d 541 2d1 53c
543 :3 0 2d4 525
545 :2 0 547 2d7
548 522 547 0
549 2d9 0 54a
2db 54d :3 0 54d
0 54d 54c 54a
54b :6 0 54f 6
:3 0 2de 551 2e0
550 54f :2 0 552
2e2 :2 0 554 0
554 553 501 552
:6 0 600 1 :3 0
af :3 0 5f :3 0
38 :3 0 37 :2 0
2e5 557 55a 2e8
556 55c 55d :2 0
a4 :2 0 37 :2 0
2ec 55f 561 :3 0
b0 :2 0 2ef 563
564 :3 0 85 :3 0
2c :3 0 84 :3 0
9d :3 0 3 :4 0
b1 1 :8 0 571
87 :3 0 88 :3 0
56c 56d 0 89
:3 0 56e 56f 0
571 2f1 5c4 8a
:3 0 87 :3 0 88
:3 0 573 574 0
89 :3 0 575 576
0 578 2f4 57a
2f6 579 578 :2 0
5c2 8b :3 0 87
:3 0 88 :3 0 57d
57e 0 89 :3 0
57f 580 0 5ba
52 :3 0 53 :3 0
582 583 0 52
:3 0 63 :3 0 585
586 0 8c :2 0
2 :4 0 b2 :2 0
65 :2 0 31 :2 0
2f8 58b 58d :3 0
2fb 587 58f 584
591 590 :2 0 55
:3 0 66 :3 0 593
594 0 57 :4 0
b3 :4 0 5a :2 0
5b :3 0 2ff 598
59a :3 0 5a :2 0
8f :4 0 302 59c
59e :3 0 5a :2 0
90 :4 0 305 5a0
5a2 :3 0 5a :2 0
89 :3 0 308 5a4
5a6 :3 0 5a :2 0
91 :4 0 30b 5a8
5aa :3 0 5a :2 0
7d :3 0 92 :3 0
31 :2 0 93 :2 0
30e 5ad 5b1 312
5ac 5b3 :3 0 315
595 5b5 :2 0 5b7
318 5b8 592 5b7
0 5b9 31a 0
5ba 31c 5bd :3 0
5bd 0 5bd 5bc
5ba 5bb :6 0 5bf
8 :3 0 31f 5c1
321 5c0 5bf :2 0
5c2 323 :2 0 5c4
0 5c4 5c3 571
5c2 :6 0 5f7 1
:3 0 87 :3 0 88
:3 0 5c6 5c7 0
87 :3 0 75 :2 0
b4 :3 0 5c9 5cb
0 328 5ca 5cd
:3 0 5ce :2 0 52
:3 0 53 :3 0 5d0
5d1 0 52 :3 0
63 :3 0 5d3 5d4
0 31 :2 0 2
:4 0 b5 :2 0 65
:2 0 31 :2 0 32b
5d9 5db :3 0 32e
5d5 5dd 5d2 5df
5de :2 0 55 :3 0
66 :3 0 5e1 5e2
0 57 :4 0 b6
:4 0 5a :2 0 5b
:3 0 332 5e6 5e8
:3 0 335 5e3 5ea
:2 0 5ec 338 5ed
5e0 5ec 0 5ee
33a 0 5f4 18
:3 0 68 :3 0 b7
:3 0 5f0 5f1 0
5f2 :2 0 5f4 33c
5f5 5cf 5f4 0
5f6 33f 0 5f7
341 5f8 565 5f7
0 5f9 344 0
600 3d :3 0 b8
:3 0 3 :3 0 346
5fb 5fd 5fa 5fe
0 600 348 602
366 415 0 603
0 600 0 603
34e 0 ae7 5f
:3 0 b9 :3 0 5f
:3 0 ba :3 0 bb
:3 0 bc :3 0 36
:3 0 39 :3 0 2e
:3 0 4e :3 0 bd
:3 0 98 :3 0 2d
:4 0 be 1 :8 0
617 87 :3 0 88
:3 0 612 613 0
89 :3 0 614 615
0 617 351 66a
8a :3 0 87 :3 0
88 :3 0 619 61a
0 89 :3 0 61b
61c 0 61e 354
620 356 61f 61e
:2 0 668 8b :3 0
87 :3 0 88 :3 0
623 624 0 89
:3 0 625 626 0
660 52 :3 0 53
:3 0 628 629 0
52 :3 0 63 :3 0
62b 62c 0 8c
:2 0 2 :4 0 bf
:2 0 65 :2 0 31
:2 0 358 631 633
:3 0 35b 62d 635
62a 637 636 :2 0
55 :3 0 66 :3 0
639 63a 0 57
:4 0 c0 :4 0 5a
:2 0 5b :3 0 35f
63e 640 :3 0 5a
:2 0 8f :4 0 362
642 644 :3 0 5a
:2 0 90 :4 0 365
646 648 :3 0 5a
:2 0 89 :3 0 368
64a 64c :3 0 5a
:2 0 91 :4 0 36b
64e 650 :3 0 5a
:2 0 7d :3 0 92
:3 0 31 :2 0 93
:2 0 36e 653 657
372 652 659 :3 0
375 63b 65b :2 0
65d 378 65e 638
65d 0 65f 37a
0 660 37c 663
:3 0 663 0 663
662 660 661 :6 0
665 a :3 0 37f
667 381 666 665
:2 0 668 383 :2 0
66a 0 66a 669
617 668 :6 0 ae7
1 :3 0 5f :3 0
3b :3 0 37 :2 0
386 66c 66f 75
:2 0 37 :2 0 38b
671 673 :3 0 5f
:3 0 1b :3 0 37
:2 0 38e 675 678
c1 :2 0 37 :2 0
393 67a 67c :3 0
67d :2 0 5f :3 0
1d :3 0 37 :2 0
396 67f 682 75
:2 0 c2 :2 0 39b
684 686 :3 0 687
:2 0 67e 689 688
:2 0 1b :3 0 5f
:3 0 36 :3 0 37
:2 0 39e 68c 68f
c3 :2 0 5f :3 0
1b :3 0 37 :2 0
3a1 692 695 3a4
691 697 :3 0 68b
698 0 69a 3a7
69b 68a 69a 0
69c 3a9 0 7b5
5f :3 0 1a :3 0
37 :2 0 3ab 69d
6a0 75 :2 0 37
:2 0 3b0 6a2 6a4
:3 0 c :3 0 c4
:2 0 3b3 6a7 6a8
:3 0 6a9 :2 0 5f
:3 0 1b :3 0 37
:2 0 3b5 6ab 6ae
c1 :2 0 37 :2 0
3ba 6b0 6b2 :3 0
1a :3 0 5f :3 0
22 :3 0 37 :2 0
3bd 6b5 6b8 c3
:2 0 5f :3 0 1b
:3 0 37 :2 0 3c0
6bb 6be 3c3 6ba
6c0 :3 0 6b4 6c1
0 6c3 3c6 6c4
6b3 6c3 0 6c5
3c8 0 6c6 3ca
730 c5 :3 0 c6
:3 0 1a :3 0 c7
:3 0 98 :3 0 2d
:3 0 c8 :3 0 7c
:3 0 7d :3 0 b
:3 0 7c :3 0 7d
:3 0 c :4 0 c9
1 :8 0 6da 87
:3 0 88 :3 0 6d5
6d6 0 89 :3 0
6d7 6d8 0 6da
3cc 72d 8a :3 0
87 :3 0 88 :3 0
6dc 6dd 0 89
:3 0 6de 6df 0
6e1 3cf 6e3 3d1
6e2 6e1 :2 0 72b
8b :3 0 87 :3 0
88 :3 0 6e6 6e7
0 89 :3 0 6e8
6e9 0 723 52
:3 0 53 :3 0 6eb
6ec 0 52 :3 0
63 :3 0 6ee 6ef
0 8c :2 0 2
:4 0 ca :2 0 65
:2 0 31 :2 0 3d3
6f4 6f6 :3 0 3d6
6f0 6f8 6ed 6fa
6f9 :2 0 55 :3 0
66 :3 0 6fc 6fd
0 57 :4 0 cb
:4 0 5a :2 0 5b
:3 0 3da 701 703
:3 0 5a :2 0 8f
:4 0 3dd 705 707
:3 0 5a :2 0 90
:4 0 3e0 709 70b
:3 0 5a :2 0 89
:3 0 3e3 70d 70f
:3 0 5a :2 0 91
:4 0 3e6 711 713
:3 0 5a :2 0 7d
:3 0 92 :3 0 31
:2 0 93 :2 0 3e9
716 71a 3ed 715
71c :3 0 3f0 6fe
71e :2 0 720 3f3
721 6fb 720 0
722 3f5 0 723
3f7 726 :3 0 726
0 726 725 723
724 :6 0 728 c
:3 0 3fa 72a 3fc
729 728 :2 0 72b
3fe :2 0 72d 0
72d 72c 6da 72b
:6 0 72f 1 :3 0
401 731 6aa 6c6
0 732 0 72f
0 732 403 0
733 406 782 5f
:3 0 1c :3 0 37
:2 0 408 734 737
75 :2 0 31 :2 0
40d 739 73b :3 0
1a :3 0 5f :3 0
1a :3 0 37 :2 0
410 73e 741 c3
:2 0 cc :2 0 413
743 745 :3 0 73d
746 0 749 71
:3 0 416 77f 5f
:3 0 1c :3 0 37
:2 0 418 74a 74d
75 :2 0 cd :2 0
41d 74f 751 :3 0
1a :3 0 5f :3 0
36 :3 0 37 :2 0
420 754 757 c3
:2 0 5f :3 0 1a
:3 0 37 :2 0 423
75a 75d 426 759
75f :3 0 753 760
0 763 71 :3 0
429 764 752 763
0 780 5f :3 0
1c :3 0 37 :2 0
42b 765 768 75
:2 0 ce :2 0 430
76a 76c :3 0 1a
:3 0 5f :3 0 39
:3 0 37 :2 0 433
76f 772 c3 :2 0
5f :3 0 1a :3 0
37 :2 0 436 775
778 439 774 77a
:3 0 76e 77b 0
77d 43c 77e 76d
77d 0 780 73c
749 0 780 43e
0 781 442 783
6a5 733 0 784
0 781 0 784
444 0 7b5 5f
:3 0 1a :3 0 37
:2 0 447 785 788
75 :2 0 37 :2 0
44c 78a 78c :3 0
52 :3 0 53 :3 0
78e 78f 0 52
:3 0 63 :3 0 791
792 0 31 :2 0
2 :4 0 cf :2 0
65 :2 0 31 :2 0
44f 797 799 :3 0
452 793 79b 790
79d 79c :2 0 55
:3 0 66 :3 0 79f
7a0 0 57 :4 0
d0 :4 0 5a :2 0
5b :3 0 456 7a4
7a6 :3 0 459 7a1
7a8 :2 0 7aa 45c
7ab 79e 7aa 0
7ac 45e 0 7b2
18 :3 0 68 :3 0
a7 :3 0 7ae 7af
0 7b0 :2 0 7b2
460 7b3 78d 7b2
0 7b4 463 0
7b5 465 7b6 674
7b5 0 7b7 469
0 ae7 5f :3 0
1f :3 0 37 :2 0
46b 7b8 7bb 75
:2 0 cd :2 0 470
7bd 7bf :3 0 1f
:3 0 6d :3 0 5f
:3 0 1f :3 0 37
:2 0 473 7c3 7c6
68 :3 0 d1 :3 0
7c8 7c9 0 476
7c2 7cb 7cc :2 0
7c1 7cd 0 7d0
71 :3 0 479 9d4
7d :3 0 30 :3 0
31 :2 0 31 :2 0
47b 7d1 7d5 75
:2 0 37 :4 0 481
7d7 7d9 :3 0 7da
:2 0 5f :3 0 1e
:3 0 37 :2 0 484
7dc 7df 75 :2 0
37 :2 0 489 7e1
7e3 :3 0 7e4 :2 0
7db 7e6 7e5 :2 0
85 :3 0 d2 :3 0
d3 :3 0 d4 :3 0
d5 :3 0 d6 :3 0
2c :3 0 26 :3 0
28 :3 0 4d :3 0
32 :3 0 84 :3 0
76 :3 0 2d :3 0
77 :3 0 2d :3 0
78 :3 0 25 :4 0
d7 1 :8 0 800
87 :3 0 88 :3 0
7fb 7fc 0 89
:3 0 7fd 7fe 0
800 48c 853 8a
:3 0 87 :3 0 88
:3 0 802 803 0
89 :3 0 804 805
0 807 48f 809
491 808 807 :2 0
851 8b :3 0 87
:3 0 88 :3 0 80c
80d 0 89 :3 0
80e 80f 0 849
52 :3 0 53 :3 0
811 812 0 52
:3 0 63 :3 0 814
815 0 8c :2 0
2 :4 0 d8 :2 0
65 :2 0 31 :2 0
493 81a 81c :3 0
496 816 81e 813
820 81f :2 0 55
:3 0 66 :3 0 822
823 0 57 :4 0
d9 :4 0 5a :2 0
5b :3 0 49a 827
829 :3 0 5a :2 0
8f :4 0 49d 82b
82d :3 0 5a :2 0
90 :4 0 4a0 82f
831 :3 0 5a :2 0
89 :3 0 4a3 833
835 :3 0 5a :2 0
91 :4 0 4a6 837
839 :3 0 5a :2 0
7d :3 0 92 :3 0
31 :2 0 93 :2 0
4a9 83c 840 4ad
83b 842 :3 0 4b0
824 844 :2 0 846
4b3 847 821 846
0 848 4b5 0
849 4b7 84c :3 0
84c 0 84c 84b
849 84a :6 0 84e
e :3 0 4ba 850
4bc 84f 84e :2 0
851 4be :2 0 853
0 853 852 800
851 :6 0 9d2 1
:3 0 87 :3 0 88
:3 0 855 856 0
87 :3 0 75 :2 0
b4 :3 0 858 85a
0 4c3 859 85c
:3 0 85d :2 0 2c
:3 0 da :2 0 4c6
860 861 :3 0 862
:2 0 85e 864 863
:2 0 1f :3 0 6d
:3 0 5f :3 0 1f
:3 0 37 :2 0 4c8
868 86b 68 :3 0
d1 :3 0 86d 86e
0 4cb 867 870
871 :2 0 866 872
0 9cf db :3 0
dc :3 0 874 875
0 b :3 0 26
:3 0 dd :2 0 4ce
876 87a 69 :2 0
37 :2 0 4d4 87c
87e :3 0 87f :2 0
db :3 0 dc :3 0
881 882 0 28
:3 0 c :3 0 dd
:2 0 4d7 883 887
69 :2 0 37 :2 0
4dd 889 88b :3 0
88c :2 0 880 88e
88d :2 0 7d :3 0
32 :3 0 31 :2 0
31 :2 0 4e0 890
894 75 :2 0 cd
:4 0 4e6 896 898
:3 0 5f :3 0 3b
:3 0 37 :2 0 4e9
89a 89d 75 :2 0
37 :2 0 4ee 89f
8a1 :3 0 1f :3 0
cd :2 0 8a3 8a4
0 8a6 4f1 8a7
8a2 8a6 0 8a8
4f3 0 8ac 1e
:3 0 cd :2 0 8a9
8aa 0 8ac 4f5
8ad 899 8ac 0
8ae 4f8 0 8af
4fa 9cc 5f :3 0
1f :3 0 37 :2 0
4fc 8b0 8b3 75
:2 0 31 :2 0 501
8b5 8b7 :3 0 3a
:3 0 6b :3 0 de
:3 0 8ba 8bb 0
5f :3 0 3a :3 0
37 :2 0 504 8bd
8c0 68 :3 0 6a
:3 0 8c2 8c3 0
507 8bc 8c5 8c6
:2 0 8b9 8c7 0
8c9 50a 8ca 8b8
8c9 0 8cb 50c
0 9cb 5f :3 0
4d :3 0 37 :2 0
50e 8cc 8cf 5f
:3 0 df :2 0 1a
:3 0 37 :2 0 511
8d1 8d5 516 8d2
8d7 :3 0 5f :3 0
4d :3 0 37 :2 0
519 8d9 8dc c1
:2 0 37 :2 0 51e
8de 8e0 :3 0 db
:3 0 dc :3 0 8e2
8e3 0 b :3 0
28 :3 0 dd :2 0
521 8e4 8e8 69
:2 0 37 :2 0 527
8ea 8ec :3 0 8e1
8ee 8ed :2 0 7d
:3 0 32 :3 0 31
:2 0 31 :2 0 52a
8f0 8f4 75 :2 0
37 :4 0 530 8f6
8f8 :3 0 4f :3 0
ce :2 0 8fa 8fb
0 8fd 533 8fe
8f9 8fd 0 8ff
535 0 933 3d
:3 0 e0 :4 0 901
902 :3 0 900 903
0 933 3a :3 0
68 :3 0 e1 :3 0
906 907 0 905
908 0 933 3d
:3 0 e2 :3 0 3
:3 0 2d :3 0 23
:3 0 25 :3 0 51
:3 0 38 :3 0 7
:3 0 8 :3 0 35
:3 0 35 :3 0 b
:3 0 28 :3 0 50
:3 0 4d :3 0 1b
:3 0 1c :3 0 1d
:3 0 50 :3 0 3a
:3 0 30 :3 0 1e
:3 0 1f :3 0 20
:3 0 22 :3 0 4f
:3 0 36 :3 0 4e
:3 0 17 :3 0 5f
:3 0 36 :3 0 37
:2 0 537 928 92b
37 :2 0 74 :2 0
74 :2 0 53a 90b
930 90a 931 0
933 55b 934 8ef
933 0 935 560
0 9c8 84 :3 0
e3 :3 0 1a :3 0
9d :3 0 3 :4 0
e4 1 :8 0 951
aa :4 0 93c :3 0
ab :3 0 75 :2 0
564 93f 940 :3 0
87 :3 0 88 :3 0
942 943 0 37
:2 0 944 945 0
947 567 94e 87
:3 0 88 :3 0 948
949 0 ac :2 0
94a 94b 0 94d
569 94f 941 947
0 950 0 94d
0 950 56b 0
951 56e 9a4 8a
:3 0 87 :3 0 88
:3 0 953 954 0
89 :3 0 955 956
0 958 571 95a
573 959 958 :2 0
9a2 8b :3 0 87
:3 0 88 :3 0 95d
95e 0 89 :3 0
95f 960 0 99a
52 :3 0 53 :3 0
962 963 0 52
:3 0 63 :3 0 965
966 0 8c :2 0
2 :4 0 e5 :2 0
65 :2 0 31 :2 0
575 96b 96d :3 0
578 967 96f 964
971 970 :2 0 55
:3 0 66 :3 0 973
974 0 57 :4 0
e6 :4 0 5a :2 0
5b :3 0 57c 978
97a :3 0 5a :2 0
8f :4 0 57f 97c
97e :3 0 5a :2 0
90 :4 0 582 980
982 :3 0 5a :2 0
89 :3 0 585 984
986 :3 0 5a :2 0
91 :4 0 588 988
98a :3 0 5a :2 0
7d :3 0 92 :3 0
31 :2 0 93 :2 0
58b 98d 991 58f
98c 993 :3 0 592
975 995 :2 0 997
595 998 972 997
0 999 597 0
99a 599 99d :3 0
99d 0 99d 99c
99a 99b :6 0 99f
10 :3 0 59c 9a1
59e 9a0 99f :2 0
9a2 5a0 :2 0 9a4
0 9a4 9a3 951
9a2 :6 0 9c8 1
:3 0 52 :3 0 53
:3 0 9a6 9a7 0
52 :3 0 63 :3 0
9a9 9aa 0 31
:2 0 2 :4 0 e7
:2 0 65 :2 0 31
:2 0 5a3 9af 9b1
:3 0 5a6 9ab 9b3
9a8 9b5 9b4 :2 0
55 :3 0 66 :3 0
9b7 9b8 0 57
:4 0 e8 :4 0 5a
:2 0 5b :3 0 5aa
9bc 9be :3 0 5ad
9b9 9c0 :2 0 9c2
5b0 9c3 9b6 9c2
0 9c4 5b2 0
9c8 18 :3 0 37
:2 0 9c6 :2 0 9c8
5b4 9c9 8d8 9c8
0 9ca 5b9 0
9cb 5bb 9cd 88f
8af 0 9ce 0
9cb 0 9ce 5be
0 9cf 5c1 9d0
865 9cf 0 9d1
5c4 0 9d2 5c6
9d3 7e7 9d2 0
9d5 7c0 7d0 0
9d5 5c9 0 ae7
5f :3 0 1f :3 0
37 :2 0 5cc 9d6
9d9 75 :2 0 37
:2 0 5d1 9db 9dd
:3 0 9de :2 0 5f
:3 0 22 :3 0 37
:2 0 5d4 9e0 9e3
5f :3 0 75 :2 0
3f :3 0 37 :2 0
5d7 9e5 9e9 5dc
9e6 9eb :3 0 9ec
:2 0 9df 9ee 9ed
:2 0 5f :3 0 3b
:3 0 37 :2 0 5df
9f0 9f3 a4 :2 0
37 :2 0 5e4 9f5
9f7 :3 0 1f :3 0
6b :3 0 de :3 0
9fa 9fb 0 5f
:3 0 1f :3 0 37
:2 0 5e7 9fd a00
5f :3 0 3b :3 0
37 :2 0 5ea a02
a05 5ed 9fc a07
a08 :2 0 9f9 a09
0 a0b 5f0 a0c
9f8 a0b 0 a0d
5f2 0 a2c 3d
:3 0 e9 :3 0 3
:3 0 2d :3 0 23
:3 0 25 :3 0 7
:3 0 8 :3 0 b
:3 0 c :3 0 1a
:3 0 1b :3 0 1c
:3 0 1d :3 0 30
:3 0 1e :3 0 1f
:3 0 20 :3 0 21
:3 0 22 :3 0 3a
:3 0 16 :3 0 17
:3 0 36 :3 0 74
:2 0 74 :2 0 5f4
a0f a28 a0e a29
0 a2c 71 :3 0
60d a73 5f :3 0
1f :3 0 37 :2 0
610 a2d a30 75
:2 0 31 :2 0 615
a32 a34 :3 0 5f
:3 0 3b :3 0 37
:2 0 618 a36 a39
a4 :2 0 37 :2 0
61d a3b a3d :3 0
1f :3 0 6b :3 0
de :3 0 a40 a41
0 5f :3 0 1f
:3 0 37 :2 0 620
a43 a46 5f :3 0
3b :3 0 37 :2 0
623 a48 a4b 626
a42 a4d a4e :2 0
a3f a4f 0 a51
629 a52 a3e a51
0 a53 62b 0
a71 3d :3 0 e9
:3 0 2c :3 0 2d
:3 0 23 :3 0 25
:3 0 7 :3 0 8
:3 0 b :3 0 c
:3 0 1a :3 0 1b
:3 0 1c :3 0 1d
:3 0 30 :3 0 1e
:3 0 1f :3 0 20
:3 0 21 :3 0 22
:3 0 3a :3 0 16
:3 0 17 :3 0 36
:3 0 74 :2 0 74
:2 0 62d a55 a6e
a54 a6f 0 a71
646 a72 a35 a71
0 a74 9ef a2c
0 a74 649 0
ae7 5f :3 0 3d
:3 0 37 :2 0 64c
a75 a78 75 :2 0
ea :2 0 651 a7a
a7c :3 0 87 :3 0
88 :3 0 a7e a7f
0 68 :3 0 eb
:3 0 a81 a82 0
a80 a83 0 a86
71 :3 0 654 aa4
5f :3 0 3d :3 0
37 :2 0 656 a87
a8a a4 :2 0 37
:2 0 65b a8c a8e
:3 0 5f :3 0 3d
:3 0 37 :2 0 65e
a90 a93 a4 :2 0
ac :2 0 663 a95
a97 :3 0 a8f a99
a98 :2 0 87 :3 0
88 :3 0 a9b a9c
0 68 :3 0 a7
:3 0 a9e a9f 0
a9d aa0 0 aa2
666 aa3 a9a aa2
0 aa5 a7d a86
0 aa5 668 0
ae7 87 :3 0 88
:3 0 aa6 aa7 0
75 :2 0 ac :2 0
66d aa9 aab :3 0
87 :3 0 88 :3 0
aad aae 0 37
:2 0 aaf ab0 0
ab2 670 ab3 aac
ab2 0 ab4 672
0 ae7 52 :3 0
53 :3 0 ab5 ab6
0 52 :3 0 63
:3 0 ab8 ab9 0
31 :2 0 2 :4 0
ec :2 0 65 :2 0
31 :2 0 674 abe
ac0 :3 0 677 aba
ac2 ab7 ac4 ac3
:2 0 55 :3 0 66
:3 0 ac6 ac7 0
57 :4 0 ed :4 0
5a :2 0 5b :3 0
67b acb acd :3 0
5a :2 0 8f :4 0
67e acf ad1 :3 0
5a :2 0 ee :4 0
681 ad3 ad5 :3 0
5a :2 0 87 :3 0
88 :3 0 ad8 ad9
0 684 ad7 adb
:3 0 687 ac8 add
:2 0 adf 68a ae0
ac5 adf 0 ae1
68c 0 ae7 18
:3 0 87 :3 0 88
:3 0 ae3 ae4 0
ae5 :2 0 ae7 68e
aea :3 0 aea 6a3
aea ae9 ae7 ae8
:6 0 aeb :2 0 2
58 aea aee :3 0
aed aeb aef :8 0

6d6
4
:3 0 1 3 1
9 1 f 1
15 1 19 1
1f 1 25 1
29 1 2d 1
31 1 35 1
39 1 3d 1
41 1 45 1
49 1 4f 11
8 e 14 18
1e 24 28 2c
30 34 38 3c
40 44 48 4e
54 1 5a 1
60 1 66 1
6c 1 72 1
78 1 7e 1
84 1 8a 1
90 1 98 1
96 1 9f 1
9d 1 a6 1
a4 1 ad 1
ab 1 b2 1
b7 1 be 1
bc 1 c5 1
c3 1 cc 1
ca 1 d3 1
d1 1 da 1
d8 1 e1 1
df 1 e8 1
e6 1 ef 1
ed 1 f4 1
fa 1 100 1
106 1 10c 1
112 1 118 1
11e 1 124 1
12a 1 132 1
130 1 139 1
137 1 13e 1
143 1 148 1
14e 1 154 1
15a 1 160 1
166 1 16c 1
172 1 178 1
17e 1 184 1
18a 1 196 1
1a2 2 1a5 1a7
2 1aa 1ac 2
1ae 1b0 2 1b2
1b4 2 1b6 1b8
2 1ba 1bc 2
1be 1c0 2 1c2
1c4 2 1c6 1c8
2 1ca 1cc 2
1ce 1d0 2 1d2
1d4 2 1d6 1d8
1 1de 2 1e0
1e1 2 1da 1e3
2 1e5 1e7 2
1e9 1eb 2 1ed
1ef 2 1f1 1f3
2 1f5 1f7 2
1f9 1fb 2 1fd
1ff 2 201 203
2 205 207 1
20d 2 20f 210
2 209 212 2
214 216 1 21c
2 21e 21f 2
218 221 2 223
225 1 22b 2
22d 22e 2 227
230 2 232 234
1 23a 2 23c
23d 2 236 23f
2 241 243 1
249 2 24b 24c
2 245 24e 2
250 252 1 258
2 25a 25b 2
254 25d 2 25f
261 1 267 2
269 26a 2 263
26c 2 26e 270
1 276 2 278
279 2 272 27b
2 27d 27f 1
285 2 287 288
2 281 28a 2
28c 28e 2 290
292 2 294 296
2 298 29a 2
29c 29e 2 2a0
2a2 2 2a4 2a6
2 2a8 2aa 2
2ac 2ae 4 19e
1a4 1a9 2b0 1
2b2 1 2b4 2
2be 2c0 3 2bc
2bd 2c2 2 2cb
2cd 2 2ca 2cf
1 2d1 1 2d3
2 2d6 2d7 1
2dd 2 2d9 2dd
2 2e6 2e7 2
2e9 2ec 2 2ef
2f0 2 2f7 2f8
2 2fa 2fd 4
2f3 300 303 306
2 30a 30b 1
311 2 30d 311
2 317 318 2
31a 31d 2 325
326 2 328 32b
3 321 32f 332
2 337 338 3
33b 33e 341 3
343 334 344 2
35f 360 1 364
2 362 364 2
389 38e 1 395
1 390 2 3a8
3aa 3 3a6 3a7
3ac 2 3b5 3b7
2 3b9 3bb 2
3bd 3bf 2 3c1
3c3 2 3c5 3c7
3 3cc 3cd 3ce
2 3c9 3d0 2
3b4 3d2 1 3d4
1 3d6 2 39f
3d7 1 3db 1
39a 2 398 3df
1 3ea 2 3e6
3ea 2 3f6 3f8
3 3f4 3f5 3fa
2 403 405 2
402 407 1 409
1 40b 2 40c
411 1 413 3
369 3e2 414 2
427 42c 1 433
1 42e 2 446
448 3 444 445
44a 2 453 455
2 457 459 2
45b 45d 2 45f
461 2 463 465
3 46a 46b 46c
2 467 46e 2
452 470 1 472
1 474 2 43d
475 1 479 1
438 2 436 47d
1 488 2 484
488 2 494 496
3 492 493 498
2 4a1 4a3 2
4a0 4a5 1 4a7
1 4a9 2 4aa
4af 2 4b4 4b5
1 4b7 1 4bb
2 4b9 4bb 2
4c6 4c8 3 4c4
4c5 4ca 2 4d3
4d5 2 4d2 4d7
1 4d9 1 4db
2 4dc 4e1 2
4e4 4e3 1 4ee
2 4ed 4ee 1
4f6 1 4fc 2
4fe 4ff 2 4eb
500 1 507 1
502 2 51a 51c
3 518 519 51e
2 527 529 2
52b 52d 2 52f
531 2 533 535
2 537 539 3
53e 53f 540 2
53b 542 2 526
544 1 546 1
548 2 511 549
1 54d 1 50c
2 50a 551 2
558 559 1 55b
1 560 2 55e
560 1 562 2
56b 570 1 577
1 572 2 58a
58c 3 588 589
58e 2 597 599
2 59b 59d 2
59f 5a1 2 5a3
5a5 2 5a7 5a9
3 5ae 5af 5b0
2 5ab 5b2 2
596 5b4 1 5b6
1 5b8 2 581
5b9 1 5bd 1
57c 2 57a 5c1
1 5cc 2 5c8
5cc 2 5d8 5da
3 5d6 5d7 5dc
2 5e5 5e7 2
5e4 5e9 1 5eb
1 5ed 2 5ee
5f3 1 5f5 2
5c4 5f6 1 5f8
1 5fc 5 480
4e5 554 5f9 5ff
2 601 602 2
611 616 1 61d
1 618 2 630
632 3 62e 62f
634 2 63d 63f
2 641 643 2
645 647 2 649
64b 2 64d 64f
3 654 655 656
2 651 658 2
63c 65a 1 65c
1 65e 2 627
65f 1 663 1
622 2 620 667
2 66d 66e 1
672 2 670 672
2 676 677 1
67b 2 679 67b
2 680 681 1
685 2 683 685
2 68d 68e 2
693 694 2 690
696 1 699 1
69b 2 69e 69f
1 6a3 2 6a1
6a3 1 6a6 2
6ac 6ad 1 6b1
2 6af 6b1 2
6b6 6b7 2 6bc
6bd 2 6b9 6bf
1 6c2 1 6c4
1 6c5 2 6d4
6d9 1 6e0 1
6db 2 6f3 6f5
3 6f1 6f2 6f7
2 700 702 2
704 706 2 708
70a 2 70c 70e
2 710 712 3
717 718 719 2
714 71b 2 6ff
71d 1 71f 1
721 2 6ea 722
1 726 1 6e5
2 6e3 72a 1
72d 2 730 731
1 732 2 735
736 1 73a 2
738 73a 2 73f
740 2 742 744
1 747 2 74b
74c 1 750 2
74e 750 2 755
756 2 75b 75c
2 758 75e 1
761 2 766 767
1 76b 2 769
76b 2 770 771
2 776 777 2
773 779 1 77c
3 77f 764 77e
1 780 2 782
783 2 786 787
1 78b 2 789
78b 2 796 798
3 794 795 79a
2 7a3 7a5 2
7a2 7a7 1 7a9
1 7ab 2 7ac
7b1 1 7b3 3
69c 784 7b4 1
7b6 2 7b9 7ba
1 7be 2 7bc
7be 2 7c4 7c5
2 7c7 7ca 1
7ce 3 7d2 7d3
7d4 1 7d8 2
7d6 7d8 2 7dd
7de 1 7e2 2
7e0 7e2 2 7fa
7ff 1 806 1
801 2 819 81b
3 817 818 81d
2 826 828 2
82a 82c 2 82e
830 2 832 834
2 836 838 3
83d 83e 83f 2
83a 841 2 825
843 1 845 1
847 2 810 848
1 84c 1 80b
2 809 850 1
85b 2 857 85b
1 85f 2 869
86a 2 86c 86f
3 877 878 879
1 87d 2 87b
87d 3 884 885
886 1 88a 2
888 88a 3 891
892 893 1 897
2 895 897 2
89b 89c 1 8a0
2 89e 8a0 1
8a5 1 8a7 2
8a8 8ab 1 8ad
1 8ae 2 8b1
8b2 1 8b6 2
8b4 8b6 2 8be
8bf 2 8c1 8c4
1 8c8 1 8ca
2 8cd 8ce 2
8d3 8d4 1 8d6
2 8d0 8d6 2
8da 8db 1 8df
2 8dd 8df 3
8e5 8e6 8e7 1
8eb 2 8e9 8eb
3 8f1 8f2 8f3
1 8f7 2 8f5
8f7 1 8fc 1
8fe 2 929 92a
20 90c 90d 90e
90f 910 911 912
913 914 915 916
917 918 919 91a
91b 91c 91d 91e
91f 920 921 922
923 924 925 926
927 92c 92d 92e
92f 4 8ff 904
909 932 1 934
1 93e 2 93d
93e 1 946 1
94c 2 94e 94f
2 93b 950 1
957 1 952 2
96a 96c 3 968
969 96e 2 977
979 2 97b 97d
2 97f 981 2
983 985 2 987
989 3 98e 98f
990 2 98b 992
2 976 994 1
996 1 998 2
961 999 1 99d
1 95c 2 95a
9a1 2 9ae 9b0
3 9ac 9ad 9b2
2 9bb 9bd 2
9ba 9bf 1 9c1
1 9c3 4 935
9a4 9c4 9c7 1
9c9 2 8cb 9ca
2 9cc 9cd 2
873 9ce 1 9d0
2 853 9d1 2
9d4 9d3 2 9d7
9d8 1 9dc 2
9da 9dc 2 9e1
9e2 2 9e7 9e8
1 9ea 2 9e4
9ea 2 9f1 9f2
1 9f6 2 9f4
9f6 2 9fe 9ff
2 a03 a04 2
a01 a06 1 a0a
1 a0c 18 a10
a11 a12 a13 a14
a15 a16 a17 a18
a19 a1a a1b a1c
a1d a1e a1f a20
a21 a22 a23 a24
a25 a26 a27 2
a0d a2a 2 a2e
a2f 1 a33 2
a31 a33 2 a37
a38 1 a3c 2
a3a a3c 2 a44
a45 2 a49 a4a
2 a47 a4c 1
a50 1 a52 18
a56 a57 a58 a59
a5a a5b a5c a5d
a5e a5f a60 a61
a62 a63 a64 a65
a66 a67 a68 a69
a6a a6b a6c a6d
2 a53 a70 2
a73 a72 2 a76
a77 1 a7b 2
a79 a7b 1 a84
2 a88 a89 1
a8d 2 a8b a8d
2 a91 a92 1
a96 2 a94 a96
1 aa1 2 aa4
aa3 1 aaa 2
aa8 aaa 1 ab1
1 ab3 2 abd
abf 3 abb abc
ac1 2 aca acc
2 ace ad0 2
ad2 ad4 2 ad6
ada 2 ac9 adc
1 ade 1 ae0
14 2b5 2d4 345
348 34b 34e 351
354 357 35a 35d
603 66a 7b7 9d5
a74 aa5 ab4 ae1
ae6 32 5e 64
6a 70 76 7c
82 88 8e 94
9b a2 a9 b0
b5 ba c1 c8
cf d6 dd e4
eb f2 f8 fe
104 10a 110 116
11c 122 128 12e
135 13c 141 146
14c 152 158 15e
164 16a 170 176
17c 182 188 18e

1
4
0 
aee
0
1
14
11
44
0 1 2 1 4 1 6 1
8 1 a 1 c 1 e 1
10 0 0 0 
ca 1 0
2 0 1
12a 1 0
41 1 0
39 1 0
f4 1 0
106 1 0
15 1 0
112 1 0
1f 1 0
bc 1 0
29 1 0
160 1 0
130 1 0
25 1 0
d8 1 0
14e 1 0
3 1 0
17e 1 0
b2 1 0
118 1 0
16c 1 0
10c 1 0
b7 1 0
72 1 0
f 1 0
a4 1 0
60 1 0
3d 1 0
8a 1 0
19 1 0
9 1 0
90 1 0
154 1 0
11e 1 0
d1 1 0
7e 1 0
2d 1 0
184 1 0
172 1 0
143 1 0
ab 1 0
15a 1 0
100 1 0
96 1 0
45 1 0
124 1 0
18a 1 0
6c 1 0
66 1 0
178 1 0
df 1 0
78 1 0
fa 1 0
c3 1 0
9d 1 0
35 1 0
5a 1 0
148 1 0
13e 1 0
31 1 0
e6 1 0
166 1 0
4f 1 0
49 1 0
84 1 0
137 1 0
ed 1 0
0

/
SHOW ERRORS FUNCTION FC_U_TSKRES2;
ALTER FUNCTION FC_U_TSKRES2 COMPILE DEBUG;
/*
#Stored Procedure  :SP_U_TSKRES2
#Short Description :Update resource/profile assignment
#Description       :Gets call from FC_SEARCH_2
#Table Name        :
#Parameters        :
 $$if bitflag  = 0 then insert new assignment (assign_id = RES_ASSIGNMENT_ID)
               = 1 then update an existing assignment (assign_id = TSK_ASSIGNMENT_ID)
               = 2 then insert new assignment using project assignment (assign_id = RES_ASSIGNMENT_ID)
               = 3 then reject a profile replacement, uncommit everything
               = 9 MSP
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_U_TSKRES2 ==
CREATE OR REPLACE FUNCTION SP_U_TSKRES2 wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
71
2 :e:
1FUNCTION:
1SP_U_TSKRES2:
1ASSIGN_ID:
1VARCHAR2:
1AND_ROLES:
1AND_SKILLS:
1WORK_LOCATION001:
1PLS_INTEGER:
1START_DATE001:
1FINISH_DATE001:
1EFFORT001:
1WORK001:
1EUNIT:
1UNIT:
1WORK_TYPE:
1BITFLAG001:
1CONTOUR001:
1CONTOUR_TYPE001:
1OCCURRENCE:
1ASSIGN_STRING001:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_ASSIGN_ID:
132:
1H_AND_ROLES:
18000:
1H_AND_SKILLS:
1H_WORK_LOCATION:
10:
1H_START_DATE:
126:
1H_FINISH_DATE:
1H_EFFORT:
1H_WORK:
1H_EUNIT:
1H_UNIT:
1H_WORK_TYPE:
1H_BITFLAG:
1H_CONTOUR:
1H_CONTOUR_TYPE:
1H_OCCURRENCE:
1H_ASSIGN_STRING:
18192:
1H_REC_USER:
1H_TEMP_SC_STATE:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_U_TSKRES2(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
11:
164:
1+:
1FPUT:
1               ENTER:: SP_U_TSKRES2(65) :: cplan.sql(7516) :: :
1IS NULL:
18:
172:
1      RETURN_INVALID:: SP_U_TSKRES2(73) :: cplan.sql(7524) :: :
1 :
1STATE:: :
1PMO_CNST:
1INVALID_PARAMS:
175:
1                EXIT:: SP_U_TSKRES2(76) :: cplan.sql(7527) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1SC_STATE:
1FC_U_TSKRES2:
14:
1105:
1       RETURN_STATUS:: SP_U_TSKRES2(106) :: cplan.sql(7557) :: :
1 ID:: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
116:
1113:
1           EXCEPTION:: SP_U_TSKRES2(114) :: cplan.sql(7565) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1ID:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1OTHERS:
1SQLCODE:
1125:
1           EXCEPTION:: SP_U_TSKRES2(126) :: cplan.sql(7577) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
3ca
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 1c 51 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 1c 51 81 b0 a3
:2 a0 6b 1c 81 b0 :2 a0 6b :2 a0
6b 6e a5 b a 10 :2 a0 6b
6e :2 a0 6b 6e a5 b 6e 7e
a0 b4 2e 6e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e :3 a0 a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c a0 7e
b4 2e :2 5a a0 7e b4 2e :2 5a
52 10 a0 7e b4 2e :2 5a 52
10 a0 7e b4 2e :2 5a 52 10
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b a0 a5 57 b7
19 3c :2 a0 d :2 a0 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 6b :12 a0 a5
b d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 :2 a0
6b :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 :2 a0 6b :2 6e 7e :2 a0 6b b4
2e 7e 6e b4 2e 7e :2 a0 6b
b4 2e a5 57 b7 19 3c :9 a0
12a e9 dd :2 a0 65 b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c :7 a0 12a e9
dd :2 a0 65 b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 68 4f
1d 17 b5 
3ca
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 6f 6b 59 77 84
80 56 8c 95 91 7f 9d aa
a6 7c b2 bb b7 a5 c3 d0
cc a2 d8 e1 dd cb e9 f6
f2 c8 fe 107 103 f1 10f 11c
118 ee 124 12d 129 117 135 142
13e 114 14a 153 14f 13d 15b 13a
160 164 168 16c 16f 173 190 17b
17f 182 183 18b 17a 1ad 19b 177
19f 1a0 1a8 19a 1ca 1b8 197 1bc
1bd 1c5 1b7 1e6 1d5 1d9 1b4 1e1
1d4 203 1f1 1d1 1f5 1f6 1fe 1f0
220 20e 1ed 212 213 21b 20d 23c
22b 22f 20a 237 22a 258 247 24b
227 253 246 274 263 267 243 26f
262 290 27f 283 25f 28b 27e 2ac
29b 29f 27b 2a7 29a 2c8 2b7 2bb
297 2c3 2b6 2e4 2d3 2d7 2b3 2df
2d2 300 2ef 2f3 2cf 2fb 2ee 31c
30b 30f 2eb 317 30a 339 327 307
32b 32c 334 326 356 344 323 348
349 351 343 372 361 365 340 36d
360 392 37d 381 35d 385 38d 37c
399 39d 379 3a1 3a5 3a9 3ac 3b1
3b2 1 3b4 3b9 3bd 3c1 3c4 3c9
3cd 3d1 3d4 3d9 3da 3dc 3e1 3e4
3e8 3e9 3ee 3f3 3f6 3fb 3fc 401
404 408 409 40e 411 416 417 41c
41f 424 425 42a 42d 432 433 438
43b 43f 440 445 448 44d 44e 453
456 45b 45c 461 464 469 46a 46f
472 476 477 47c 47f 484 485 48a
48d 492 493 498 49b 49f 4a3 4a7
4a8 4aa 4af 4b0 4b2 4b3 4b8 4bb
4c0 4c1 4c6 4c9 4ce 4cf 4d4 4d7
4db 4dc 4e1 4e4 4e9 4ea 4ef 4f2
4f7 4f8 4fd 500 505 506 50b 50e
512 513 518 51b 520 521 526 529
52e 52f 534 537 53b 53f 543 544
546 54b 54c 54e 54f 554 557 55c
55d 562 565 569 56d 571 572 574
579 57a 57c 57d 582 585 58a 58b
590 593 597 59b 59f 5a0 5a2 5a7
5a8 5aa 5ab 5b0 5b3 5b8 5b9 5be
5c1 5c5 5c9 5cd 5ce 5d0 5d5 5d6
5d8 5d9 5de 5e1 5e6 5e7 5ec 5ef
5f3 5f7 5fb 5fc 5fe 603 604 606
607 60c 60f 614 615 61a 61d 621
625 629 62a 62c 631 632 634 635
63a 63d 642 643 648 64b 64f 653
657 658 65a 65f 660 662 663 668
66b 670 671 676 679 67d 681 685
686 688 68d 68e 690 691 696 699
69e 69f 6a4 6a7 6ab 6af 6b3 6b4
6b6 6bb 6bc 6be 6bf 6c4 6c7 6cc
6cd 6d2 6d5 6da 6db 6e0 6e3 6e7
6e8 6ed 6f0 6f5 6f6 6fb 6fe 703
704 709 70c 711 712 717 71a 71e
71f 724 727 72c 72d 732 735 73a
73b 740 741 746 748 74c 74f 753
757 75a 75e 762 765 768 76d 770
773 776 777 77c 77d 1 77f 784
788 78c 78f 794 799 79c 7a0 7a1
7a6 7a7 7ac 7ae 7b2 7b5 7b9 7bc
7bd 7c2 7c5 7c8 7cc 7cf 7d0 7d5
7d8 1 7db 7e0 7e4 7e7 7e8 7ed
7f0 1 7f3 7f8 7fc 7ff 800 805
808 1 80b 810 814 818 81b 81f
823 826 829 82e 831 834 837 838
83d 83e 1 840 845 849 84d 850
855 85a 85d 861 862 867 86a 86f
870 875 878 87d 87e 883 886 88a
88e 892 896 899 89a 89c 8a1 8a2
8a4 8a5 8aa 8ab 8b0 8b2 8b6 8b9
8bd 8c1 8c4 8c8 8cc 8cf 8d2 8d7
8da 8dd 8e0 8e1 8e6 8e7 1 8e9
8ee 8f2 8f6 8f9 8fe 903 906 90a
90b 910 911 916 918 91c 91f 923
927 92a 92e 932 935 939 93a 93f
941 945 948 94c 950 954 958 95c
960 964 968 96c 970 974 978 97c
980 984 988 98c 990 994 998 99c
9a0 9a4 9a8 9ac 9b0 9b4 9b8 9bc
9c0 9c4 9c8 9cc 9d0 9d4 9d8 9dc
9e0 9e4 9e8 9ec 9f0 9f4 9f8 9fc
a00 a04 a08 a0c a10 a14 a18 a1c
a1f a23 a27 a2b a2f a33 a37 a3b
a3f a43 a47 a4b a4f a53 a57 a5b
a5f a63 a67 a68 a6a a6e a72 a76
a79 a7d a81 a84 a87 a8c a8f a92
a95 a96 a9b a9c 1 a9e aa3 aa7
aab aae ab3 ab8 abb abf ac0 ac5
ac8 acd ace ad3 ad6 adb adc ae1
ae4 ae8 aec af0 af4 af7 af8 afa
aff b00 b02 b03 b08 b0b b10 b11
b16 b19 b1e b1f b24 b27 b2b b2c
b31 b34 b39 b3a b3f b40 b45 b47
b4b b4e b52 b56 b59 b5d b61 b64
b68 b6c b6d b72 b74 b78 b7c b7f
b83 b87 b8a b8e b92 b95 b98 b9d
ba0 ba3 ba6 ba7 bac bad 1 baf
bb4 bb8 bbc bbf bc4 bc9 bcc bd0
bd1 bd6 bd7 bdc be0 be4 be7 bec
bf1 bf4 bf8 bfc bff c00 c05 c08
c0d c0e c13 c16 c1a c1e c21 c22
c27 c28 c2d c2f c33 c36 c3a c3e
c42 c46 c4a c4e c52 c56 c5a c66
c6b c6f c73 c77 c7b c7d c7e c83
1 c87 c8b c8f c92 c96 c9a c9e
ca2 ca5 ca9 cad cb0 cb3 cb8 cbb
cbe cc1 cc2 cc7 cc8 1 cca ccf
cd3 cd7 cda cdf ce4 ce7 ceb cec
cf1 cf2 cf7 cfb cff d02 d07 d0c
d0f d13 d14 d19 d1c d21 d22 d27
d2a d2e d32 d35 d38 d39 d3b d3c
d41 d42 d47 d49 d4d d50 d54 d58
d5c d60 d64 d68 d6c d78 d7d d81
d85 d89 d8d d8f d93 d95 da1 da3
da5 da6 dab daf db1 dbd dc1 dc3
dc4 dcd 
3ca
2
0 1 a 6 10 :3 6 10 :3 6
11 :3 6 17 :3 6 14 :3 6 15 :3 6
10 :3 6 e :3 6 c :3 6 b :3 6
10 :3 6 11 :3 6 11 :3 6 16 :3 6
11 :3 6 17 :3 6 12 :2 6 1 3
a 13 a :2 1 4 10 19 18
:2 10 :2 4 10 19 18 :2 10 :2 4 11
1a 19 :2 11 :2 4 :2 14 23 14 :2 4
11 1a 19 :2 11 :2 4 12 1b 1a
:2 12 :2 4 :2 d 1c d :2 4 :2 b 1a
b :2 4 :2 c 1b c :2 4 :2 b 1a
b :2 4 :2 10 1f 10 :2 4 :2 e 1d
e :2 4 :2 e 1d e :2 4 :2 13 22
13 :2 4 :2 11 20 11 :2 4 14 1d
1c :2 14 :2 4 f 18 17 :2 f :2 4
:2 14 23 14 :2 4 e 17 :3 e 4
7 :2 11 23 :2 2d 3a :2 23 :3 7 :2 d
19 25 :2 2f 43 :2 25 54 5a 5d
:2 54 6b 81 8 :2 6b d 10 :2 6b
1a 1d :2 6b b e :2 6b 13 16
:2 6b 1b 1e :2 6b 28 2b :2 6b b
e :2 6b 13 16 :2 6b 1b 1e :2 6b
29 2c :2 6b b e :2 6b 13 16
1f 27 :2 1f 3a :2 16 :2 6b b e
:2 6b 13 16 :2 6b 1b 1e :2 6b 2c
2f :2 6b b e :2 6b 13 16 :2 6b
1b 1e :2 6b 2d 30 :2 6b b e
:2 6b 13 16 1f 27 :2 1f 33 :2 16
:2 6b b e :2 6b 13 16 1f 27
:2 1f 31 :2 16 :2 6b b e :2 6b 13
16 1f 27 :2 1f 2f :2 16 :2 6b b
e :2 6b 13 16 1f 27 :2 1f 2e
:2 16 :2 6b b e :2 6b 13 16 1f
27 :2 1f 33 :2 16 :2 6b b e :2 6b
13 16 1f 27 :2 1f 34 :2 16 :2 6b
b e :2 6b 13 16 1f 27 :2 1f
34 :2 16 :2 6b b e :2 6b 13 16
1f 27 :2 1f 39 :2 16 :2 6b b e
:2 6b 13 16 1f 27 :2 1f 34 :2 16
:2 6b b e :2 6b 13 16 :2 6b 1b
1e :2 6b 2f 32 :2 6b b e :2 6b
13 16 :2 6b 1b 1e :2 6b 2a 2d
:2 6b 32 35 :2 6b :2 7 4a :2 4 7
:2 11 23 :2 2d 39 3c 4c 4e 4f
:2 4c :2 23 :3 7 :2 d 12 1e 5c 5f
:2 1e :2 7 52 :2 4 :4 9 8 7 :4 22
21 20 :2 7 :4 3f 3e 3d :2 7 :4 f
e d :2 7 a :2 14 26 :2 30 3c
3f 4f 51 52 :2 4f :2 26 :3 a :2 10
15 21 5f 62 :2 21 6f 72 :2 21
76 79 :2 21 83 86 8f 97 :2 a0
:2 8f b1 :2 86 :2 21 :2 a 55 :2 7 a
:2 14 26 :2 30 3c 3f 4f 51 52
:2 4f :2 26 :3 a :2 10 15 21 5f 62
:2 21 :2 a 55 :3 7 :2 12 28 :2 31 41
:2 7 2a :3 4 13 :2 4 13 :2 4 14
:2 4 14 :2 4 15 :2 4 17 :2 4 12
:2 4 17 :2 4 10 :2 4 e :2 4 f
:2 4 e :2 4 13 :2 4 11 :2 4 11
:2 4 16 :2 4 14 :2 4 :2 d 19 26
33 40 4e 25 33 42 4c 54
5d 25 32 3d 48 58 25 36
:2 19 4 7 :2 11 23 :2 2d 39 3c
4c 4f 50 :2 4c :2 23 :3 7 :2 d 12
1e 5d 60 :2 1e 6d 70 :2 1e 74
77 :2 1e 81 84 8d 95 :2 9e :2 8d
a9 :2 84 :2 1e b1 b4 :2 1e bc bf
:2 1e c4 c7 :2 1e d3 d6 :2 1e :2 7
53 :3 4 :2 f 21 :2 2a 34 41 :2 4
1 9 12 9 a :2 14 26 :2 30
3c 40 50 53 54 :2 50 :2 26 :3 a
:2 10 15 21 60 63 :2 21 :3 a :2 10
15 39 5c 5f :2 68 :2 39 76 79
:2 39 81 84 :2 8d :2 39 :2 a 57 :3 7
c 15 1e 2c a 13 1e f
e 7 e 7 e 7 29 :2 4
:2 9 7 :2 10 1c :2 7 :2 11 23 :2 2d
39 3d 4d 50 51 :2 4d :2 23 :3 7
:2 d 12 1e 5d 60 :2 1e :3 7 :2 d
12 36 42 45 :2 36 4d 50 :2 36
5c 5f 66 6f 72 :2 5f :2 36 :2 7
54 :2 4 7 c 1b a 13 1e
f e 7 e 7 e 7 :2 4
:4 10 :2 4 :8 1 
3ca
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 :4 7 :4 8
:4 9 :4 a :4 b :4 c
:4 d :4 e :4 f :4 10
:4 11 :4 12 :4 13 2
:4 14 :2 1 :7 16 :7 17
:7 18 :6 19 :7 1a :7 1b
:6 1c :6 1d :6 1e :6 1f
:6 20 :6 21 :6 22 :6 23
:6 24 :7 25 :7 26 :6 27
:7 29 :b 2b :11 2c 2d
:2 2c :2 2d :2 2c :2 2d
:2 2c :2 2e :2 2c :2 2e
:2 2c :2 2e :2 2c :2 2e
:2 2c :2 2f :2 2c :2 2f
:2 2c :2 2f :2 2c :2 2f
:2 2c :2 30 :2 2c :9 30
:2 2c :2 31 :2 2c :2 31
:2 2c :2 31 :2 2c :2 31
:2 2c :2 32 :2 2c :2 32
:2 2c :2 32 :2 2c :2 32
:2 2c :2 33 :2 2c :9 33
:2 2c :2 34 :2 2c :9 34
:2 2c :2 35 :2 2c :9 35
:2 2c :2 36 :2 2c :9 36
:2 2c :2 37 :2 2c :9 37
:2 2c :2 38 :2 2c :9 38
:2 2c :2 39 :2 2c :9 39
:2 2c :2 3a :2 2c :9 3a
:2 2c :2 3b :2 2c :9 3b
:2 2c :2 3c :2 2c :2 3c
:2 2c :2 3c :2 2c :2 3c
:2 2c :2 3d :2 2c :2 3d
:2 2c :2 3d :2 2c :2 3d
:2 2c :2 3d :4 2c :3 2b
:11 40 :b 41 :3 40 :16 46
:6 47 :2 46 :11 48 :20 49
:3 48 :11 4b :b 4c :3 4b
:9 4e 47 :2 46 :3 51
:3 52 :3 53 :3 54 :3 55
:3 56 :3 57 :3 59 :3 5a
:3 5b :3 5c :3 5d :3 5e
:3 5f :3 60 :3 61 :3 62
:8 64 :6 65 :5 66 :2 67
:3 64 :11 69 :30 6a :3 69
:a 6c 2a :3 70 :11 71
:b 72 :17 73 :3 71 :2 75
:3 76 :3 77 78 76
75 76 :3 79 :3 70
:2 7a :5 7c :11 7d :b 7e
:18 7f :3 7d :2 81 82
:3 83 84 82 81
82 :3 85 :2 7b :6 7a
6f :7 1 
dcf
4
:3 0 1 :3 0 2
:a 0 3c5 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 4
:3 0 5 :7 0 9
8 :3 0 4 :3 0
6 :7 0 d c
:3 0 d 7c 0
b 8 :3 0 7
:7 0 11 10 :3 0
4 :3 0 9 :7 0
15 14 :3 0 11
a2 0 f 4
:3 0 a :7 0 19
18 :3 0 8 :3 0
b :7 0 1d 1c
:3 0 15 c8 0
13 8 :3 0 c
:7 0 21 20 :3 0
8 :3 0 d :7 0
25 24 :3 0 19
ee 0 17 8
:3 0 e :7 0 29
28 :3 0 8 :3 0
f :7 0 2d 2c
:3 0 1d 114 0
1b 8 :3 0 10
:7 0 31 30 :3 0
8 :3 0 11 :7 0
35 34 :3 0 21
13a 0 1f 8
:3 0 12 :7 0 39
38 :3 0 8 :3 0
13 :7 0 3d 3c
:3 0 25 :2 0 23
4 :3 0 14 :7 0
41 40 :3 0 4
:3 0 15 :7 0 45
44 :3 0 16 :3 0
17 :3 0 18 :2 0
4 49 4a 0
47 4b 0 3c5
2 4c :2 0 1c
:2 0 39 4 :3 0
1a :2 0 37 4f
51 :6 0 54 52
0 3c3 0 19
:6 0 1c :2 0 3d
4 :3 0 3b 56
58 :6 0 5b 59
0 3c3 0 1b
:6 0 1f :2 0 41
4 :3 0 3f 5d
5f :6 0 62 60
0 3c3 0 1d
:6 0 21 :2 0 43
8 :3 0 64 :7 0
68 65 66 3c3
0 1e :6 0 21
:2 0 47 4 :3 0
45 6a 6c :6 0
6f 6d 0 3c3
0 20 :6 0 1f
:2 0 4b 4 :3 0
49 71 73 :6 0
76 74 0 3c3
0 22 :6 0 1f
:2 0 4d 8 :3 0
78 :7 0 7c 79
7a 3c3 0 23
:6 0 1f :2 0 4f
8 :3 0 7e :7 0
82 7f 80 3c3
0 24 :6 0 1f
:2 0 51 8 :3 0
84 :7 0 88 85
86 3c3 0 25
:6 0 1f :2 0 53
8 :3 0 8a :7 0
8e 8b 8c 3c3
0 26 :6 0 1f
:2 0 55 8 :3 0
90 :7 0 94 91
92 3c3 0 27
:6 0 1f :2 0 57
8 :3 0 96 :7 0
9a 97 98 3c3
0 28 :6 0 1f
:2 0 59 8 :3 0
9c :7 0 a0 9d
9e 3c3 0 29
:6 0 1f :2 0 5b
8 :3 0 a2 :7 0
a6 a3 a4 3c3
0 2a :6 0 2d
:2 0 5d 8 :3 0
a8 :7 0 ac a9
aa 3c3 0 2b
:6 0 1a :2 0 61
4 :3 0 5f ae
b0 :6 0 b3 b1
0 3c3 0 2c
:6 0 1f :2 0 65
4 :3 0 63 b5
b7 :6 0 ba b8
0 3c3 0 2e
:6 0 c2 c3 0
67 8 :3 0 bc
:7 0 c0 bd be
3c3 0 2f :6 0
c8 c9 0 69
17 :3 0 18 :2 0
4 c4 :7 0 c7
c5 0 3c3 0
30 :6 0 31 :3 0
32 :3 0 31 :3 0
33 :3 0 cb cc
0 2 :4 0 6b
cd cf ca d1
d0 :2 0 34 :3 0
35 :3 0 d3 d4
0 36 :4 0 31
:3 0 37 :3 0 d7
d8 0 2 :4 0
6d d9 db 38
:4 0 39 :2 0 3a
:3 0 6f de e0
:3 0 3b :4 0 39
:2 0 3c :4 0 72
e3 e5 :3 0 39
:2 0 3 :3 0 75
e7 e9 :3 0 39
:2 0 3c :4 0 78
eb ed :3 0 39
:2 0 3d :4 0 7b
ef f1 :3 0 39
:2 0 3c :4 0 7e
f3 f5 :3 0 39
:2 0 5 :3 0 81
f7 f9 :3 0 39
:2 0 3c :4 0 84
fb fd :3 0 39
:2 0 3d :4 0 87
ff 101 :3 0 39
:2 0 3c :4 0 8a
103 105 :3 0 39
:2 0 6 :3 0 8d
107 109 :3 0 39
:2 0 3c :4 0 90
10b 10d :3 0 39
:2 0 3d :4 0 93
10f 111 :3 0 39
:2 0 3e :3 0 3f
:3 0 7 :3 0 96
115 117 40 :4 0
98 114 11a 9b
113 11c :3 0 39
:2 0 3d :4 0 9e
11e 120 :3 0 39
:2 0 3c :4 0 a1
122 124 :3 0 39
:2 0 9 :3 0 a4
126 128 :3 0 39
:2 0 3c :4 0 a7
12a 12c :3 0 39
:2 0 3d :4 0 aa
12e 130 :3 0 39
:2 0 3c :4 0 ad
132 134 :3 0 39
:2 0 a :3 0 b0
136 138 :3 0 39
:2 0 3c :4 0 b3
13a 13c :3 0 39
:2 0 3d :4 0 b6
13e 140 :3 0 39
:2 0 3e :3 0 3f
:3 0 b :3 0 b9
144 146 40 :4 0
bb 143 149 be
142 14b :3 0 39
:2 0 3d :4 0 c1
14d 14f :3 0 39
:2 0 3e :3 0 3f
:3 0 c :3 0 c4
153 155 40 :4 0
c6 152 158 c9
151 15a :3 0 39
:2 0 3d :4 0 cc
15c 15e :3 0 39
:2 0 3e :3 0 3f
:3 0 d :3 0 cf
162 164 40 :4 0
d1 161 167 d4
160 169 :3 0 39
:2 0 3d :4 0 d7
16b 16d :3 0 39
:2 0 3e :3 0 3f
:3 0 e :3 0 da
171 173 40 :4 0
dc 170 176 df
16f 178 :3 0 39
:2 0 3d :4 0 e2
17a 17c :3 0 39
:2 0 3e :3 0 3f
:3 0 f :3 0 e5
180 182 40 :4 0
e7 17f 185 ea
17e 187 :3 0 39
:2 0 3d :4 0 ed
189 18b :3 0 39
:2 0 3e :3 0 3f
:3 0 10 :3 0 f0
18f 191 40 :4 0
f2 18e 194 f5
18d 196 :3 0 39
:2 0 3d :4 0 f8
198 19a :3 0 39
:2 0 3e :3 0 3f
:3 0 11 :3 0 fb
19e 1a0 40 :4 0
fd 19d 1a3 100
19c 1a5 :3 0 39
:2 0 3d :4 0 103
1a7 1a9 :3 0 39
:2 0 3e :3 0 3f
:3 0 12 :3 0 106
1ad 1af 40 :4 0
108 1ac 1b2 10b
1ab 1b4 :3 0 39
:2 0 3d :4 0 10e
1b6 1b8 :3 0 39
:2 0 3e :3 0 3f
:3 0 13 :3 0 111
1bc 1be 40 :4 0
113 1bb 1c1 116
1ba 1c3 :3 0 39
:2 0 3d :4 0 119
1c5 1c7 :3 0 39
:2 0 3c :4 0 11c
1c9 1cb :3 0 39
:2 0 14 :3 0 11f
1cd 1cf :3 0 39
:2 0 3c :4 0 122
1d1 1d3 :3 0 39
:2 0 3d :4 0 125
1d5 1d7 :3 0 39
:2 0 3c :4 0 128
1d9 1db :3 0 39
:2 0 15 :3 0 12b
1dd 1df :3 0 39
:2 0 3c :4 0 12e
1e1 1e3 :3 0 39
:2 0 41 :4 0 131
1e5 1e7 :3 0 134
d5 1e9 :2 0 1eb
139 1ec d2 1eb
0 1ed 13b 0
323 31 :3 0 32
:3 0 1ee 1ef 0
31 :3 0 42 :3 0
1f1 1f2 0 43
:2 0 2 :4 0 44
:2 0 45 :2 0 43
:2 0 13d 1f7 1f9
:3 0 140 1f3 1fb
1f0 1fd 1fc :2 0
34 :3 0 46 :3 0
1ff 200 0 36
:4 0 47 :4 0 39
:2 0 3a :3 0 144
204 206 :3 0 147
201 208 :2 0 20a
14a 20b 1fe 20a
0 20c 14c 0
323 3 :3 0 48
:2 0 14e 20e 20f
:3 0 210 :2 0 211
:2 0 9 :3 0 48
:2 0 150 214 215
:3 0 216 :2 0 217
:2 0 212 219 218
:2 0 a :3 0 48
:2 0 152 21c 21d
:3 0 21e :2 0 21f
:2 0 21a 221 220
:2 0 14 :3 0 48
:2 0 154 224 225
:3 0 226 :2 0 227
:2 0 222 229 228
:2 0 31 :3 0 32
:3 0 22b 22c 0
31 :3 0 42 :3 0
22e 22f 0 49
:2 0 2 :4 0 4a
:2 0 45 :2 0 43
:2 0 156 234 236
:3 0 159 230 238
22d 23a 239 :2 0
34 :3 0 46 :3 0
23c 23d 0 36
:4 0 4b :4 0 39
:2 0 3a :3 0 15d
241 243 :3 0 39
:2 0 4c :4 0 160
245 247 :3 0 39
:2 0 4d :4 0 163
249 24b :3 0 39
:2 0 3e :3 0 3f
:3 0 4e :3 0 4f
:3 0 250 251 0
166 24f 253 40
:4 0 168 24e 256
16b 24d 258 :3 0
16e 23e 25a :2 0
25c 171 25d 23b
25c 0 25e 173
0 287 31 :3 0
32 :3 0 25f 260
0 31 :3 0 42
:3 0 262 263 0
43 :2 0 2 :4 0
50 :2 0 45 :2 0
43 :2 0 175 268
26a :3 0 178 264
26c 261 26e 26d
:2 0 34 :3 0 46
:3 0 270 271 0
36 :4 0 51 :4 0
39 :2 0 3a :3 0
17c 275 277 :3 0
17f 272 279 :2 0
27b 182 27c 26f
27b 0 27d 184
0 287 52 :3 0
53 :3 0 27e 27f
0 4e :3 0 4f
:3 0 281 282 0
54 :3 0 186 280
285 :2 0 287 189
288 22a 287 0
289 18d 0 323
19 :3 0 3 :3 0
28a 28b 0 323
1b :3 0 5 :3 0
28d 28e 0 323
1d :3 0 6 :3 0
290 291 0 323
20 :3 0 9 :3 0
293 294 0 323
22 :3 0 a :3 0
296 297 0 323
2c :3 0 14 :3 0
299 29a 0 323
2e :3 0 15 :3 0
29c 29d 0 323
1e :3 0 7 :3 0
29f 2a0 0 323
23 :3 0 b :3 0
2a2 2a3 0 323
24 :3 0 c :3 0
2a5 2a6 0 323
25 :3 0 d :3 0
2a8 2a9 0 323
26 :3 0 e :3 0
2ab 2ac 0 323
27 :3 0 f :3 0
2ae 2af 0 323
28 :3 0 10 :3 0
2b1 2b2 0 323
29 :3 0 11 :3 0
2b4 2b5 0 323
2a :3 0 12 :3 0
2b7 2b8 0 323
2b :3 0 13 :3 0
2ba 2bb 0 323
17 :3 0 55 :3 0
2bd 2be 0 56
:3 0 19 :3 0 1b
:3 0 1d :3 0 1e
:3 0 20 :3 0 22
:3 0 23 :3 0 24
:3 0 25 :3 0 26
:3 0 27 :3 0 28
:3 0 29 :3 0 2a
:3 0 2b :3 0 2c
:3 0 2e :3 0 18f
2c0 2d2 2bf 2d3
0 323 31 :3 0
32 :3 0 2d5 2d6
0 31 :3 0 42
:3 0 2d8 2d9 0
57 :2 0 2 :4 0
58 :2 0 45 :2 0
43 :2 0 1a1 2de
2e0 :3 0 1a4 2da
2e2 2d7 2e4 2e3
:2 0 34 :3 0 46
:3 0 2e6 2e7 0
36 :4 0 59 :4 0
39 :2 0 3a :3 0
1a8 2eb 2ed :3 0
39 :2 0 4c :4 0
1ab 2ef 2f1 :3 0
39 :2 0 4d :4 0
1ae 2f3 2f5 :3 0
39 :2 0 3e :3 0
3f :3 0 17 :3 0
55 :3 0 2fa 2fb
0 1b1 2f9 2fd
40 :4 0 1b3 2f8
300 1b6 2f7 302
:3 0 39 :2 0 5a
:4 0 1b9 304 306
:3 0 39 :2 0 3c
:4 0 1bc 308 30a
:3 0 39 :2 0 19
:3 0 1bf 30c 30e
:3 0 39 :2 0 3c
:4 0 1c2 310 312
:3 0 1c5 2e8 314
:2 0 316 1c8 317
2e5 316 0 318
1ca 0 323 52
:3 0 5b :3 0 319
31a 0 17 :3 0
55 :3 0 31c 31d
0 19 :3 0 54
:3 0 1cc 31b 321
:2 0 323 1d0 3c4
17 :3 0 5c :2 0
4 324 325 0
31 :3 0 32 :3 0
327 328 0 31
:3 0 42 :3 0 32a
32b 0 5d :2 0
2 :4 0 5e :2 0
45 :2 0 43 :2 0
1e8 330 332 :3 0
1eb 32c 334 329
336 335 :2 0 34
:3 0 46 :3 0 338
339 0 36 :4 0
5f :4 0 39 :2 0
3a :3 0 1ef 33d
33f :3 0 1f2 33a
341 :2 0 35a 34
:3 0 46 :3 0 343
344 0 36 :4 0
60 :4 0 39 :2 0
17 :3 0 61 :3 0
349 34a 0 1f5
348 34c :3 0 39
:2 0 5a :4 0 1f8
34e 350 :3 0 39
:2 0 17 :3 0 62
:3 0 353 354 0
1fb 352 356 :3 0
1fe 345 358 :2 0
35a 201 35b 337
35a 0 35c 204
0 36c 63 :3 0
30 :3 0 17 :3 0
61 :3 0 64 :3 0
17 :3 0 62 :3 0
65 :3 0 66 :4 0
67 1 :a 0 368
:2 0 36c 35e 366
:2 0 16 :3 0 30
:3 0 36a :2 0 36c
206 36e 20a 36d
36c :2 0 3c2 68
:3 0 17 :3 0 55
:3 0 371 372 0
69 :3 0 373 374
0 3ba 31 :3 0
32 :3 0 376 377
0 31 :3 0 42
:3 0 379 37a 0
5d :2 0 2 :4 0
6a :2 0 45 :2 0
43 :2 0 20c 37f
381 :3 0 20f 37b
383 378 385 384
:2 0 34 :3 0 46
:3 0 387 388 0
36 :4 0 6b :4 0
39 :2 0 3a :3 0
213 38c 38e :3 0
216 389 390 :2 0
3aa 34 :3 0 46
:3 0 392 393 0
36 :4 0 6c :4 0
39 :2 0 69 :3 0
219 397 399 :3 0
39 :2 0 6d :4 0
21c 39b 39d :3 0
39 :2 0 6e :3 0
6f :3 0 43 :2 0
70 :2 0 21f 3a0
3a4 223 39f 3a6
:3 0 226 394 3a8
:2 0 3aa 229 3ab
386 3aa 0 3ac
22c 0 3ba 63
:3 0 30 :3 0 64
:3 0 17 :3 0 62
:3 0 65 :3 0 66
:4 0 71 1 :a 0
3b6 :2 0 3ba 3ae
3b4 :2 0 16 :3 0
30 :3 0 3b8 :2 0
3ba 22e 3bd :3 0
3bd 0 3bd 3bc
3ba 3bb :6 0 3bf
1 :3 0 233 3c1
235 3c0 3bf :2 0
3c2 237 :2 0 3c4
23a 3c4 3c3 323
3c2 :6 0 3c5 :2 0
2 4c 3c4 3c8
:3 0 3c7 3c5 3c9
:8 0 
24e
4
:3 0 1 3 1
7 1 b 1
f 1 13 1
17 1 1b 1
1f 1 23 1
27 1 2b 1
2f 1 33 1
37 1 3b 1
3f 1 43 11
6 a e 12
16 1a 1e 22
26 2a 2e 32
36 3a 3e 42
46 1 50 1
4e 1 57 1
55 1 5e 1
5c 1 63 1
6b 1 69 1
72 1 70 1
77 1 7d 1
83 1 89 1
8f 1 95 1
9b 1 a1 1
a7 1 af 1
ad 1 b6 1
b4 1 bb 1
c1 1 ce 1
da 2 dd df
2 e2 e4 2
e6 e8 2 ea
ec 2 ee f0
2 f2 f4 2
f6 f8 2 fa
fc 2 fe 100
2 102 104 2
106 108 2 10a
10c 2 10e 110
1 116 2 118
119 2 112 11b
2 11d 11f 2
121 123 2 125
127 2 129 12b
2 12d 12f 2
131 133 2 135
137 2 139 13b
2 13d 13f 1
145 2 147 148
2 141 14a 2
14c 14e 1 154
2 156 157 2
150 159 2 15b
15d 1 163 2
165 166 2 15f
168 2 16a 16c
1 172 2 174
175 2 16e 177
2 179 17b 1
181 2 183 184
2 17d 186 2
188 18a 1 190
2 192 193 2
18c 195 2 197
199 1 19f 2
1a1 1a2 2 19b
1a4 2 1a6 1a8
1 1ae 2 1b0
1b1 2 1aa 1b3
2 1b5 1b7 1
1bd 2 1bf 1c0
2 1b9 1c2 2
1c4 1c6 2 1c8
1ca 2 1cc 1ce
2 1d0 1d2 2
1d4 1d6 2 1d8
1da 2 1dc 1de
2 1e0 1e2 2
1e4 1e6 4 d6
dc e1 1e8 1
1ea 1 1ec 2
1f6 1f8 3 1f4
1f5 1fa 2 203
205 2 202 207
1 209 1 20b
1 20d 1 213
1 21b 1 223
2 233 235 3
231 232 237 2
240 242 2 244
246 2 248 24a
1 252 2 254
255 2 24c 257
2 23f 259 1
25b 1 25d 2
267 269 3 265
266 26b 2 274
276 2 273 278
1 27a 1 27c
2 283 284 3
25e 27d 286 1
288 11 2c1 2c2
2c3 2c4 2c5 2c6
2c7 2c8 2c9 2ca
2cb 2cc 2cd 2ce
2cf 2d0 2d1 2
2dd 2df 3 2db
2dc 2e1 2 2ea
2ec 2 2ee 2f0
2 2f2 2f4 1
2fc 2 2fe 2ff
2 2f6 301 2
303 305 2 307
309 2 30b 30d
2 30f 311 2
2e9 313 1 315
1 317 3 31e
31f 320 17 1ed
20c 289 28c 28f
292 295 298 29b
29e 2a1 2a4 2a7
2aa 2ad 2b0 2b3
2b6 2b9 2bc 2d4
318 322 2 32f
331 3 32d 32e
333 2 33c 33e
2 33b 340 2
347 34b 2 34d
34f 2 351 355
2 346 357 2
342 359 1 35b
3 35c 367 36b
1 326 2 37e
380 3 37c 37d
382 2 38b 38d
2 38a 38f 2
396 398 2 39a
39c 3 3a1 3a2
3a3 2 39e 3a5
2 395 3a7 2
391 3a9 1 3ab
4 375 3ac 3b5
3b9 1 3bd 1
370 2 36e 3c1
13 53 5a 61
67 6e 75 7b
81 87 8d 93
99 9f a5 ab
b2 b9 bf c6

1
4
0 
3c8
0
1
14
2
25
0 1 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 
43 1 0
2b 1 0
b 1 0
70 1 0
3b 1 0
27 1 0
23 1 0
4e 1 0
f 1 0
bb 1 0
2 0 1
1f 1 0
89 1 0
5c 1 0
77 1 0
a1 1 0
69 1 0
55 1 0
1b 1 0
a7 1 0
95 1 0
3 1 0
33 1 0
13 1 0
83 1 0
7d 1 0
8f 1 0
37 1 0
7 1 0
63 1 0
2f 1 0
b4 1 0
3f 1 0
ad 1 0
9b 1 0
17 1 0
c1 1 0
0

/
SHOW ERRORS FUNCTION SP_U_TSKRES2;
ALTER FUNCTION SP_U_TSKRES2 COMPILE DEBUG;
/*
#Stored Procedure  :SP_ASSIGN_MSP
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_ASSIGN_MSP ==
CREATE OR REPLACE FUNCTION SP_ASSIGN_MSP wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
19e
2 :e:
1FUNCTION:
1SP_ASSIGN_MSP:
1ID:
1VARCHAR2:
1ACTION_TYPE:
1PLS_INTEGER:
1TASK_ID001:
1START_DT001:
1FINISH_DT001:
1EFFORT001:
1ASSIGN_STRING001:
1WORK_TODATE001:
1PERCENT_COMPLETE001:
1EETC:
1DATA_FLAG:
1UNITS:
1CONTOUR_TYPE001:
1BASELINE_START001:
1BASELINE_FINISH001:
1BASELINE_DURATION001:
1BASELINE_WORK001:
1ACTUAL_START001:
1ACTUAL_FINISH001:
1ACTUAL_EFFORT001:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_ELEMENT_NAME:
1255:
1H_FULL_NAME:
1H_MESSAGE:
18192:
1H_SUBJECT:
12048:
1H_REVISION:
116:
1H_TEMP_ID:
132:
1H_TEMP_ID2:
1H_WKF_DEF_ID:
1H_UNIQUEID:
1H_RA_ID:
1H_RESOURCE_ID:
1H_PROJECT_ID:
1H_TASK_ID:
1H_REC_USER:
1H_TYPE_ID:
10:
1H_WORK_LOCATION:
1H_ZERO:
1H_TWO:
12:
1H_WORK:
1H_CONTOUR:
1H_CONTOUR_TYPE:
1H_UNITS:
1H_PER_DAY:
1480:
1H_IMPEX_ROLLUP:
1H_EFFORT:
1H_COUNT:
1H_TOTAL:
1H_MINUTES:
1H_DATA_FLAG:
1H_WORK_TODATE:
1H_DAYOFYEAR:
1DATE:
1H_FINISH_D:
1H_TSK_ASSIGNMENT_ID:
1H_PRJ_ASSIGNMENT_ID:
1H_NULL_STR:
11:
1H_START_DATE:
1TIMESTAMP:
1H_START_DT:
1H_ACTUAL_START:
1H_FINISH_DT:
1H_REC_DT:
1H_LOCK_DT:
1H_ASSIGN_STRING:
1H_TABLE_TYPE:
1H_TEMP_VAR:
1H_TEMP_SC_STATE:
1H_OCCURRENCE:
1H_BIT_MINUTES:
1H_ACTION_TYPE:
1H_ETC:
1H_SD:
1H_FD:
1H_REG_COST:
1NUMBER:
1H_REG_SALE:
1H_BILLABLE:
1H_CAPITAL_EXPENSE:
1H_TPH_DELETE:
1H_PERCENT_UNITS:
1CURSOR:
1C_H_WBS:
1ELEMENT_NAME:
1RTRIM:
1TO_CHAR:
1MAJOR_REVISION:
1MINOR_REVISION:
1TMT_WBS:
1PROJECT_ID:
1ELEMENT_ID:
1REC_DATETIME:
1ROWNUM:
1SELECT * FROM (SELECT ELEMENT_NAME,RTRIM(TO_CHAR(MAJOR_REVISION)) || '.' || R+
1TRIM(TO_CHAR(MINOR_REVISION)):n         FROM TMT_WBS:n         WHERE PROJECT_+
1ID = h_project_id AND ELEMENT_ID = h_task_id:n         ORDER BY REC_DATETIME +
1DESC ) WHERE ROWNUM<=1:
1CAF:
1SCHED_MIN:
1DAY_YEAR:
1ACTUAL_FLAG:
1TASK_DAY_OF_YEAR:
1TSK_ASSIGNMENT_ID:
1ACTUAL_MIN:
1SELECT SCHED_MIN, DAY_YEAR, ACTUAL_FLAG:n         FROM TASK_DAY_OF_YEAR:n    +
1     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment+
1_id:n         FOR UPDATE OF ACTUAL_MIN,ACTUAL_FLAG:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_ASSIGN_MSP(:
1':
1, :
1COALESCE:
1NULL:
1);:
1CHECK_TRACE:
1123:
1+:
1FPUT:
1               ENTER:: SP_ASSIGN_MSP(124) :: cplan.sql(7725) :: :
1IS NULL:
1<:
1>:
13:
18:
1129:
1      RETURN_INVALID:: SP_ASSIGN_MSP(130) :: cplan.sql(7731) :: :
1 :
1STATE:: :
1PMO_CNST:
1INVALID_PARAMS:
1132:
1                EXIT:: SP_ASSIGN_MSP(133) :: cplan.sql(7734) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1139:
1      RETURN_INVALID:: SP_ASSIGN_MSP(140) :: cplan.sql(7741) :: :
1142:
1                EXIT:: SP_ASSIGN_MSP(143) :: cplan.sql(7744) :: :
1265:
1LENGTH:
1SC_ASSIGN_STRING_LENGTH:
1=:
1171:
1      RETURN_INVALID:: SP_ASSIGN_MSP(172) :: cplan.sql(7773) :: :
1174:
1                EXIT:: SP_ASSIGN_MSP(175) :: cplan.sql(7776) :: :
1TYPE_ID:
1SELECT PROJECT_ID, TYPE_ID INTO h_project_id, h_type_id:n            FROM TMT+
1_WBS WHERE ELEMENT_ID = h_task_id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
1194:
1           EXCEPTION:: SP_ASSIGN_MSP(195) :: cplan.sql(7796) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SYS_GUID:
1GETTABLE:
1ASSIGNRESOURCE:
1COUNT:
1DUAL:
1PROFILES:
1PROFILE_ID:
1SELECT COUNT('X') INTO h_temp_var FROM DUAL:n               WHERE EXISTS (SEL+
1ECT 'X' FROM PROFILES:n               WHERE PROJECT_ID = h_project_id AND PRO+
1FILE_ID = h_resource_id):
1220:
1           EXCEPTION:: SP_ASSIGN_MSP(221) :: cplan.sql(7822) :: :
1!=:
15:
11*5:
1IS NOT NULL:
1TO_TIMESTAMP:
1PMO_ORASTD:
1DB2TIMESTAMP:
1SELECT TO_TIMESTAMP(PMO_ORASTD.DB2TIMESTAMP(h_start_DT)) + (5 / 1440.0) INTO +
1h_finish_DT FROM DUAL:
1241:
1           EXCEPTION:: SP_ASSIGN_MSP(242) :: cplan.sql(7843) :: :
1ISWBSDELIV:
1TASK_ID:
1RESOURCE_ID:
1WORK_TODATE:
1PROJECT_ASSIGNMENT_ID:
1UNITS_MSP:
1C_EFFORT:
1C_CONTOUR:
1CONTOUR_TYPE:
1TASK_ASSIGNMENTS:
1SELECT PROJECT_ID, TASK_ID, RESOURCE_ID, WORK_TODATE, PROJECT_ASSIGNMENT_ID,+
1:n            UNITS_MSP, C_EFFORT, C_CONTOUR, CONTOUR_TYPE, REC_DATETIME:n   +
1         INTO h_project_id, h_task_id, h_resource_id, h_work_todate, h_prj_as+
1signment_id,:n            h_percent_units, h_work, h_contour, h_contour_type,+
1 h_rec_DT:n            FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_ra_i+
1d:
1270:
1           EXCEPTION:: SP_ASSIGN_MSP(271) :: cplan.sql(7872) :: :
1SC_SQL_SUCCESS:
1-:
1287:
1        RETURN_ERROR:: SP_ASSIGN_MSP(288) :: cplan.sql(7889) :: :
1 ID:: :
1290:
1                EXIT:: SP_ASSIGN_MSP(291) :: cplan.sql(7892) :: :
1PMO_RETURN_ERROR_SP:
1299:
1      RETURN_INVALID:: SP_ASSIGN_MSP(300) :: cplan.sql(7901) :: :
1302:
1                EXIT:: SP_ASSIGN_MSP(303) :: cplan.sql(7904) :: :
1SC_BIT_28:
1103079040:
1MINUTE_PER_DAY:
1SELECT COALESCE(MINUTE_PER_DAY, 480) INTO h_per_day FROM TMT_WBS WHERE ELEMEN+
1T_ID = h_task_id:
1327:
1           EXCEPTION:: SP_ASSIGN_MSP(328) :: cplan.sql(7929) :: :
1TRUNC:
1TO_NUMBER:
1/:
1*:
110000:
10.4:
1BITAND:
1FC_U_TSKRES2:
1ETC:
1PROPOSED_START:
1SCHED_START:
1PROPOSED_FINISH:
1SCHED_FINISH:
1PROPOSED_WORK:
1SCHED_WORK:
1UPDATE TASK_ASSIGNMENTS SET ETC = eetc,:n                        PROPOSED_STA+
1RT = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK:n +
1                       WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_i+
1d AND RESOURCE_ID = h_resource_id:
1FOUND:
1100:
1398:
1           EXCEPTION:: SP_ASSIGN_MSP(399) :: cplan.sql(8000) :: :
1UPDATE TASK_ASSIGNMENTS SET ETC = eetc,:n                        PROPOSED_STA+
1RT = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK:n +
1                       WHERE TSK_ASSIGNMENT_ID = h_ra_id:
1436:
1           EXCEPTION:: SP_ASSIGN_MSP(437) :: cplan.sql(8038) :: :
1MILESTONE_PERCENT:
1ACTUAL_START:
1ACTUAL_FINISH:
1UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,:n       +
1              UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,:n          +
1           PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOS+
1ED_WORK = SCHED_WORK,:n                     ACTUAL_START = h_start_DT, ACTUAL+
1_FINISH = h_finish_DT,:n                     WORK_TODATE = work_todate001,:n +
1                    ETC = eetc:n                     WHERE TSK_ASSIGNMENT_ID +
1= h_ra_id:
1473:
1           EXCEPTION:: SP_ASSIGN_MSP(474) :: cplan.sql(8075) :: :
1UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = SCHED_MIN, ACTUAL_FLAG = 1:n        +
1             WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ra_id:
1493:
1           EXCEPTION:: SP_ASSIGN_MSP(494) :: cplan.sql(8095) :: :
1LAST_SUBMITTED:
1UPDATE TASK_ASSIGNMENTS SET LAST_SUBMITTED = h_finish_DT:n                   +
1  WHERE TSK_ASSIGNMENT_ID = h_ra_id:
1513:
1           EXCEPTION:: SP_ASSIGN_MSP(514) :: cplan.sql(8115) :: :
1UPDATE TASK_ASSIGNMENTS SET ETC = eetc,:n                     PROPOSED_START +
1= SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK:n    +
1                 WHERE TSK_ASSIGNMENT_ID = h_ra_id:
1534:
1           EXCEPTION:: SP_ASSIGN_MSP(535) :: cplan.sql(8136) :: :
1SC_NOT_32:
1UPDATE TASK_ASSIGNMENTS SET ETC = eetc,:n                     PROPOSED_START +
1= SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK:n    +
1                 WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND +
1RESOURCE_ID = h_resource_id:
1567:
1           EXCEPTION:: SP_ASSIGN_MSP(568) :: cplan.sql(8169) :: :
1SELECT SUBSTR(h_lock_DT,1,10) || '-' || SUBSTR(h_lock_DT,12,2) || '.' || SUBS+
1TR(h_lock_DT,15,2) || '.' || SUBSTR(h_lock_DT,18,9) INTO h_lock_DT FROM DUAL:
1589:
1           EXCEPTION:: SP_ASSIGN_MSP(590) :: cplan.sql(8191) :: :
1STRNCMP:
119:
1SC_TASK:
1TMT_WORKFLOW_CYCLES:
1PARENT_ID:
1SELECT ELEMENT_ID INTO h_wkf_def_id:n                        FROM TMT_WORKFLO+
1W_CYCLES:n                        WHERE PARENT_ID = 'NOTIFICATION':
1610:
1           EXCEPTION:: SP_ASSIGN_MSP(611) :: cplan.sql(8212) :: :
1FC_A_NOTIFICATION:
1OPEN:
1CLOSE:
1644:
1           EXCEPTION:: SP_ASSIGN_MSP(645) :: cplan.sql(8246) :: :
1FULL_NAME:
1RESOURCES:
1SELECT FULL_NAME INTO h_full_name FROM RESOURCES:n                           +
1WHERE RESOURCE_ID = h_resource_id:
1660:
1           EXCEPTION:: SP_ASSIGN_MSP(661) :: cplan.sql(8262) :: :
1Import of assignment failed.:
1Import cannot modify assignment for resource:: :
1 on the task:::
1. Assignment was modified in Timesheets during the time of export of the proj+
1ect.:
1FC_U_WORKFLOW:
1681:
1        RETURN_ERROR:: SP_ASSIGN_MSP(682) :: cplan.sql(8283) :: :
1684:
1                EXIT:: SP_ASSIGN_MSP(685) :: cplan.sql(8286) :: :
1716:
1           EXCEPTION:: SP_ASSIGN_MSP(717) :: cplan.sql(8318) :: :
1727:
1        RETURN_ERROR:: SP_ASSIGN_MSP(728) :: cplan.sql(8329) :: :
1730:
1                EXIT:: SP_ASSIGN_MSP(731) :: cplan.sql(8332) :: :
1RES_ASSIGNMENT_ID:
1UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,:n       +
1                 UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,:n       +
1                 PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,+
1PROPOSED_WORK = SCHED_WORK,:n                        ACTUAL_START = actual_st+
1art001,:n                        ACTUAL_FINISH = actual_finish001,:n         +
1               WORK_TODATE = actual_effort001:n                        WHERE +
1RES_ASSIGNMENT_ID = h_ra_id:
1765:
1           EXCEPTION:: SP_ASSIGN_MSP(766) :: cplan.sql(8367) :: :
1UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,:n       +
1                 UNITS_MSP = h_units,:n                        CONTOUR_TYPE =+
1 h_contour_type:n                        WHERE RES_ASSIGNMENT_ID = h_ra_id:
1787:
1           EXCEPTION:: SP_ASSIGN_MSP(788) :: cplan.sql(8389) :: :
1BASELINE3_START:
1BASELINE3_SCH_FLAG:
1BASELINE3_FINISH:
1BASELINE3_DURATION:
1BASELINE3_WORK:
1BASELINE2_START:
1BASELINE1_START:
1BASELINE2_FINISH:
1BASELINE1_FINISH:
1BASELINE2_DURATION:
1BASELINE1_DURATION:
1BASELINE2_WORK:
1BASELINE1_WORK:
1B_REGULAR_COST:
1REGULAR_COST:
1B_REGULAR_RATE:
1REGULAR_RATE:
1B_SPECIAL_COST:
1SPECIAL_COST:
1B_SPECIAL_RATE:
1SPECIAL_RATE:
1UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,:n       +
1                 UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,:n       +
1                 BASELINE3_START = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_T+
1IMESTAMP(baseline_start001) ELSE BASELINE3_START END,:n                      +
1  BASELINE3_FINISH = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(basel+
1ine_finish001) ELSE BASELINE3_FINISH END,:n                        BASELINE3_+
1DURATION = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_duration001 ELSE BA+
1SELINE3_DURATION END,:n                        BASELINE3_WORK = CASE WHEN BAS+
1ELINE3_SCH_FLAG = 0 THEN baseline_work001 ELSE BASELINE3_WORK END,:n         +
1               :n                        BASELINE2_START = BASELINE1_START,:n+
1                        BASELINE2_FINISH = BASELINE1_FINISH,:n               +
1         BASELINE2_DURATION = BASELINE1_DURATION,:n                        BA+
1SELINE2_WORK = BASELINE1_WORK,:n                        :n                   +
1     BASELINE1_START = baseline_start001,:n                        BASELINE1_+
1FINISH = baseline_finish001,:n                        BASELINE1_DURATION = ba+
1seline_duration001,:n                        BASELINE1_WORK = baseline_work00+
11,:n                        B_REGULAR_COST = REGULAR_COST, B_REGULAR_RATE = R+
1EGULAR_RATE, B_SPECIAL_COST = SPECIAL_COST, B_SPECIAL_RATE = SPECIAL_RATE,:n +
1                       :n                        BASELINE3_SCH_FLAG = 1,:n   +
1                     PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FIN+
1ISH,PROPOSED_WORK = SCHED_WORK,:n                        ACTUAL_START = actua+
1l_start001,:n                        ACTUAL_FINISH = actual_finish001,:n     +
1                   WORK_TODATE = actual_effort001:n                        WH+
1ERE RES_ASSIGNMENT_ID = h_ra_id:
1832:
1           EXCEPTION:: SP_ASSIGN_MSP(833) :: cplan.sql(8434) :: :
1UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,:n       +
1                 UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,:n       +
1                 BASELINE3_START = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_T+
1IMESTAMP(baseline_start001) ELSE BASELINE3_START END,:n                      +
1  BASELINE3_FINISH = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(basel+
1ine_finish001) ELSE BASELINE3_FINISH END,:n                        BASELINE3_+
1DURATION = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_duration001 ELSE BA+
1SELINE3_DURATION END,:n                        BASELINE3_WORK = CASE WHEN BAS+
1ELINE3_SCH_FLAG = 0 THEN baseline_work001 ELSE BASELINE3_WORK END,:n         +
1               :n                        BASELINE2_START = BASELINE1_START,:n+
1                        BASELINE2_FINISH = BASELINE1_FINISH,:n               +
1         BASELINE2_DURATION = BASELINE1_DURATION,:n                        BA+
1SELINE2_WORK = BASELINE1_WORK,:n                        :n                   +
1     BASELINE1_START = baseline_start001,:n                        BASELINE1_+
1FINISH = baseline_finish001,:n                        BASELINE1_DURATION = ba+
1seline_duration001,:n                        BASELINE1_WORK = baseline_work00+
11,:n                        B_REGULAR_COST = REGULAR_COST, B_REGULAR_RATE = R+
1EGULAR_RATE, B_SPECIAL_COST = SPECIAL_COST, B_SPECIAL_RATE = SPECIAL_RATE,:n +
1                       :n                        BASELINE3_SCH_FLAG = 1:n    +
1                    WHERE RES_ASSIGNMENT_ID = h_ra_id:
1871:
1           EXCEPTION:: SP_ASSIGN_MSP(872) :: cplan.sql(8473) :: :
1ELSIF:
1UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,:n       +
1           UNITS_MSP = h_units,:n                  CONTOUR_TYPE = h_contour_t+
1ype:n                  WHERE RES_ASSIGNMENT_ID = h_ra_id:
1895:
1           EXCEPTION:: SP_ASSIGN_MSP(896) :: cplan.sql(8497) :: :
1UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,:n       +
1                 UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,:n       +
1                 PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,+
1PROPOSED_WORK = SCHED_WORK,:n                        ACTUAL_START = actual_st+
1art001,:n                        ACTUAL_FINISH = actual_finish001,:n         +
1               WORK_TODATE = actual_effort001:n                        WHERE +
1TSK_ASSIGNMENT_ID = h_ra_id:
1925:
1           EXCEPTION:: SP_ASSIGN_MSP(926) :: cplan.sql(8527) :: :
1UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,:n       +
1                 UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,:n       +
1                 WORK_TODATE = work_todate001:n                        WHERE +
1TSK_ASSIGNMENT_ID = h_ra_id:
1947:
1           EXCEPTION:: SP_ASSIGN_MSP(948) :: cplan.sql(8549) :: :
1UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,:n       +
1                 UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,:n       +
1                 BASELINE3_START = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_T+
1IMESTAMP(baseline_start001) ELSE BASELINE3_START END,:n                      +
1  BASELINE3_FINISH = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(basel+
1ine_finish001) ELSE BASELINE3_FINISH END,:n                        BASELINE3_+
1DURATION = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_duration001 ELSE BA+
1SELINE3_DURATION END,:n                        BASELINE3_WORK = CASE WHEN BAS+
1ELINE3_SCH_FLAG = 0 THEN baseline_work001 ELSE BASELINE3_WORK END,:n         +
1               :n                        BASELINE2_START = BASELINE1_START,:n+
1                        BASELINE2_FINISH = BASELINE1_FINISH,:n               +
1         BASELINE2_DURATION = BASELINE1_DURATION,:n                        BA+
1SELINE2_WORK = BASELINE1_WORK,:n                        :n                   +
1     BASELINE1_START = baseline_start001,:n                        BASELINE1_+
1FINISH = baseline_finish001,:n                        BASELINE1_DURATION = ba+
1seline_duration001,:n                        BASELINE1_WORK = baseline_work00+
11,:n                        B_REGULAR_COST = REGULAR_COST, B_REGULAR_RATE = R+
1EGULAR_RATE, B_SPECIAL_COST = SPECIAL_COST, B_SPECIAL_RATE = SPECIAL_RATE,:n +
1                       :n                        BASELINE3_SCH_FLAG = 1,:n   +
1                     PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FIN+
1ISH,PROPOSED_WORK = SCHED_WORK,:n                        ACTUAL_START = actua+
1l_start001,:n                        ACTUAL_FINISH = actual_finish001,:n     +
1                   WORK_TODATE = actual_effort001:n                        WH+
1ERE TSK_ASSIGNMENT_ID = h_ra_id:
1992:
1           EXCEPTION:: SP_ASSIGN_MSP(993) :: cplan.sql(8594) :: :
1UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,:n       +
1                 UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,:n       +
1                 WORK_TODATE = work_todate001,:n                        BASEL+
1INE3_START = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_star+
1t001) ELSE BASELINE3_START END,:n                        BASELINE3_FINISH = C+
1ASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_finish001) ELSE BA+
1SELINE3_FINISH END,:n                        BASELINE3_DURATION = CASE WHEN B+
1ASELINE3_SCH_FLAG = 0 THEN baseline_duration001 ELSE BASELINE3_DURATION END,+
1:n                        BASELINE3_WORK = CASE WHEN BASELINE3_SCH_FLAG = 0 T+
1HEN baseline_work001 ELSE BASELINE3_WORK END,:n                        :n    +
1                    BASELINE2_START = BASELINE1_START,:n                     +
1   BASELINE2_FINISH = BASELINE1_FINISH,:n                        BASELINE2_DU+
1RATION = BASELINE1_DURATION,:n                        BASELINE2_WORK = BASELI+
1NE1_WORK,:n                        :n                        BASELINE1_START +
1= baseline_start001,:n                        BASELINE1_FINISH = baseline_fin+
1ish001,:n                        BASELINE1_DURATION = baseline_duration001,:n+
1                        BASELINE1_WORK = baseline_work001,:n                 +
1       B_REGULAR_COST = REGULAR_COST, B_REGULAR_RATE = REGULAR_RATE, B_SPECIA+
1L_COST = SPECIAL_COST, B_SPECIAL_RATE = SPECIAL_RATE,:n                      +
1  :n                        BASELINE3_SCH_FLAG = 1:n                        W+
1HERE TSK_ASSIGNMENT_ID = h_ra_id:
11031:
1           EXCEPTION:: SP_ASSIGN_MSP(1032) :: cplan.sql(8633) :: :
1UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,:n       +
1                 UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,:n       +
1                 WORK_TODATE = work_todate001,:n                        ETC =+
1 h_effort - work_todate001:n                        WHERE TSK_ASSIGNMENT_ID =+
1 h_ra_id:
11058:
1           EXCEPTION:: SP_ASSIGN_MSP(1059) :: cplan.sql(8660) :: :
11080:
1           EXCEPTION:: SP_ASSIGN_MSP(1081) :: cplan.sql(8682) :: :
1UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,:n       +
1              UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,:n          +
1           WORK_TODATE = work_todate001:n                     WHERE TSK_ASSIG+
1NMENT_ID = h_ra_id:
11103:
1           EXCEPTION:: SP_ASSIGN_MSP(1104) :: cplan.sql(8705) :: :
1:
1TO_DATE:
1BILLABLE:
1CAPITAL_EXPENSE:
1SELECT TSK_ASSIGNMENT_ID, SCHED_WORK, TASK_ID, TO_DATE(SubStr(SCHED_START, 1,+
1 10)), TO_DATE(SubStr(SCHED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGUL+
1AR_COST, REGULAR_RATE:n                  INTO h_tsk_assignment_id, h_effort, +
1h_task_id, h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale+
1:n                  FROM TASK_ASSIGNMENTS:n                  WHERE RES_ASSIGN+
1MENT_ID = h_ra_id:
11134:
1           EXCEPTION:: SP_ASSIGN_MSP(1135) :: cplan.sql(8736) :: :
1SELECT SCHED_WORK, TASK_ID, TO_DATE(SubStr(SCHED_START, 1, 10)), TO_DATE(SubS+
1tr(SCHED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RA+
1TE:n                  INTO h_effort, h_task_id, h_SD, h_FD, h_billable, h_cap+
1ital_expense, h_reg_cost, h_reg_sale:n                  FROM TASK_ASSIGNMENTS+
1:n                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
11153:
1           EXCEPTION:: SP_ASSIGN_MSP(1154) :: cplan.sql(8755) :: :
1UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = SCHED_MIN, ACTUAL_FLAG = 1:n        +
1             WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_as+
1signment_id:
11178:
1           EXCEPTION:: SP_ASSIGN_MSP(1179) :: cplan.sql(8780) :: :
1MAX:
1UPDATE TASK_ASSIGNMENTS SET WORK_TODATE = SCHED_WORK,:n                     L+
1AST_SUBMITTED = (SELECT MAX(DAY_YEAR) FROM TASK_DAY_OF_YEAR:n                +
1     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment+
1_id):n                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
11200:
1           EXCEPTION:: SP_ASSIGN_MSP(1201) :: cplan.sql(8802) :: :
1LOOP:
1EXIT:
1>=:
1UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = h_minutes, ACTUAL_FLAG = 1 WHERE CUR+
1RENT OF CAF:
11259:
1           EXCEPTION:: SP_ASSIGN_MSP(1260) :: cplan.sql(8861) :: :
11282:
1           EXCEPTION:: SP_ASSIGN_MSP(1283) :: cplan.sql(8884) :: :
1UPDATE TASK_ASSIGNMENTS SET LAST_SUBMITTED = h_finish_D WHERE TSK_ASSIGNMENT_+
1ID = h_tsk_assignment_id:
11303:
1           EXCEPTION:: SP_ASSIGN_MSP(1304) :: cplan.sql(8905) :: :
1FC_GENERATE_TS_ALL:
1FC_GENERATE_TS_MSP:
14:
11399:
1       RETURN_STATUS:: SP_ASSIGN_MSP(1400) :: cplan.sql(9001) :: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
11407:
1           EXCEPTION:: SP_ASSIGN_MSP(1408) :: cplan.sql(9009) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1ERROR:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
11419:
1           EXCEPTION:: SP_ASSIGN_MSP(1420) :: cplan.sql(9021) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
1e0f
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 81 b0 a3 a0 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 1c
81 b0 a3 a0 1c 81 b0 a3
a0 1c 81 b0 a3 a0 1c 81
b0 a3 a0 1c 81 b0 a3 a0
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 81
b0 a3 a0 1c 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a0 f4 b4 bf c8 :e a0
12a bd b7 11 a4 b1 a0 f4
b4 bf c8 :a a0 12a bd b7 11
a4 b1 a3 :2 a0 6b 1c 81 b0
:2 a0 6b :2 a0 6b 6e a5 b a
10 :2 a0 6b 6e :2 a0 6b 6e a5
b 6e 7e a0 b4 2e 6e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e :3 a0 a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c a0 7e
b4 2e :2 5a :2 a0 51 a5 b 7e
51 b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a 52 10 5a
52 10 :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b a0 a5
57 b7 19 3c a0 7e b4 2e
:2 5a a0 7e b4 2e :2 5a 52 10
a0 7e b4 2e :2 5a 52 10 :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b a0 a5 57 b7 19
3c a0 4d d a0 4d d a0
51 d :2 a0 d :2 a0 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 d a0 51
d :3 a0 a5 b 51 a5 b a0
7e a0 6b b4 2e :2 a0 51 a5
b 7e 51 b4 2e 5a :2 a0 51
a5 b 7e 51 b4 2e 5a 52
10 a0 7e b4 2e 5a :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b a0 a5 57 b7 19 3c
:2 a0 d :2 a0 d :2 a0 d :7 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 b4 2e d :4 a0 51 a5 b
a5 b d :2 a0 6b :7 a0 a5 b
d :2 a0 6b 7e 51 b4 2e 5a
:2 a0 51 a5 b 7e 51 b4 2e
5a a 10 :2 a0 51 a5 b 7e
51 b4 2e 5a a 10 a0 51
d :8 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 51 a5 b 7e
51 b4 2e a0 51 d a0 51
d a0 6e d a0 7e b4 2e
5a :6 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 19 3c b7 19
3c b7 19 3c :2 a0 51 a5 b
7e 51 b4 2e 5a :3 a0 51 a5
b a5 b 5a 7e 51 b4 2e
a 10 a0 51 d b7 19 3c
b7 :2 a0 d :17 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b a0
7e a0 6b b4 2e 5a :2 a0 51
a5 b a0 7e a0 51 a5 b
b4 2e 5a a 10 a0 4d d
a0 7e 51 b4 2e d b7 19
3c b7 :2 19 3c :2 a0 51 a5 b
7e 51 b4 2e :2 a0 6b 7e 51
b4 2e :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 19 3c a0 7e b4 2e
5a a0 7e b4 2e 5a 52 10
a0 7e b4 2e 5a 52 10 a0
7e b4 2e 5a 52 10 :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b a0 a5 57 b7 19 3c
:3 a0 6b d :2 a0 51 a5 b 7e
51 b4 2e a0 51 d b7 19
3c :6 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 51 a5 b 7e
51 b4 2e :5 a0 a5 b 51 a5
b 7e :3 a0 a5 b 51 a5 b
b4 2e 5a 7e 51 b4 2e 5a
7e 51 b4 2e a5 b d :2 a0
51 a5 b a0 7e a0 51 a5
b b4 2e a0 51 d a0 51
d b7 a0 51 d b7 :2 19 3c
b7 19 3c a0 4d d :3 a0 51
a5 b 51 a5 b 5a 7e 51
b4 2e :2 a0 51 a5 b 7e 51
b4 2e :2 a0 51 a5 b 7e 51
b4 2e :2 a0 6b :12 a0 a5 b d
:2 a0 6b 7e 51 b4 2e :f a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 19
3c b7 :2 a0 6b :12 a0 a5 b d
b7 :2 19 3c b7 :3 a0 51 a5 b
7e 51 b4 2e d :2 a0 51 a5
b 7e 51 b4 2e :2 a0 6b :12 a0
a5 b d :2 a0 6b 7e 51 b4
2e :b a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 19 3c b7 :2 a0 6b :12 a0
a5 b d b7 :2 19 3c b7 :2 19
3c :2 a0 6b 7e 51 b4 2e 5a
:3 a0 51 a5 b 51 a5 b 5a
7e 51 b4 2e 5a a 10 :2 a0
51 a5 b a0 7e a0 51 a5
b b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a a 10 :2 a0
51 a5 b 7e 51 b4 2e 5a
a 10 :17 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :8 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :5 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 :b a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 :2 19 3c b7 19
3c b7 :4 a0 51 a5 b :2 a0 6b
a5 b d :2 a0 51 a5 b 7e
51 b4 2e :2 a0 6b :12 a0 a5 b
d :2 a0 6b 7e 51 b4 2e 5a
:2 a0 51 a5 b 7e 51 b4 2e
5a a 10 :2 a0 51 a5 b 7e
51 b4 2e 5a a 10 :f a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 19
3c b7 a0 7e b4 2e 5a :2 a0
d :a a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b :2 a0 51 a5
b 7e 51 b4 2e :3 a0 6b d
a0 51 d :4 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :8 a0 a5 b
d :2 a0 51 a5 b 7e 51 b4
2e a0 4d d a0 4d d a0
4d d a0 4d d a0 4d d
:2 a0 e9 dd b3 :3 a0 e9 d3 5
:2 a0 f :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c :2 a0 e9
c1 :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :5 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f a0 6e d a0 6e
d :3 a0 7e a0 b4 2e :2 51 a5
b d :3 a0 7e 6e b4 2e :2 51
a5 b d :3 a0 7e a0 b4 2e
:2 51 a5 b d :3 a0 7e 6e b4
2e :2 51 a5 b d :11 a0 a5 b
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:2 a0 51 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b 51 :2 a0 a5 57 b7
19 3c b7 19 3c b7 19 3c
:3 a0 51 a5 b 7e 51 b4 2e
d :2 a0 6b :12 a0 a5 b d :2 a0
6b 7e 51 b4 2e 5a :2 a0 51
a5 b 7e 51 b4 2e 5a a
10 :2 a0 51 a5 b 7e 51 b4
2e 5a a 10 :b a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 19 3c b7
:2 19 3c b7 :2 19 3c :2 a0 6b 7e
51 b4 2e :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b :2 a0
a5 57 b7 19 3c :2 a0 d :2 a0
51 a5 b 7e 51 b4 2e 5a
:2 a0 51 a5 b 7e 51 b4 2e
5a a 10 a0 51 d b7 19
3c :2 a0 51 a5 b 7e 51 b4
2e :3 a0 51 a5 b 51 a5 b
5a 7e 51 b4 2e a0 7e b4
2e 5a a0 7e b4 2e 5a :15 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:9 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c b7 a0 7e b4 2e
5a :40 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 :34 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 :2 19 3c b7 :2 19 3c
a0 b7 :2 a0 51 a5 b 7e 51
b4 2e :9 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 :2 19 3c b7 :3 a0 51
a5 b 51 a5 b 5a 7e 51
b4 2e a0 7e b4 2e 5a a0
7e b4 2e 5a :15 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :b a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :2 19 3c
b7 a0 7e b4 2e 5a :40 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :36 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:2 19 3c b7 :2 19 3c a0 b7 :2 a0
51 a5 b 7e 51 b4 2e a0
7e b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e :e a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :b a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :2 19 3c
b7 :b a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 :2 19 3c b7 :2 19 3c b7
:2 19 3c a0 7e b4 2e 5a :2 a0
51 a5 b 7e 51 b4 2e 5a
a 10 5a :2 a0 51 a5 b 7e
51 b4 2e 5a 52 10 a0 6e
d a0 4d d a0 51 d a0
51 d :2 a0 51 a5 b 7e 51
b4 2e :19 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 a0 d :17 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 :2 19 3c :2 a0 6b a0 7e
a0 6b b4 2e 5a :2 a0 51 a5
b a0 7e a0 51 a5 b b4
2e 5a :2 a0 51 a5 b 7e 51
b4 2e 5a a 10 :2 a0 51 a5
b 7e 51 b4 2e 5a a 10
:8 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:d a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 a0 e9 dd b3 :5 a0 e9 d3
5 :2 a0 f :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c :2 a0
6b 7e 51 b4 2e a0 2b b7
19 3c :3 a0 51 a5 b 7e :2 a0
51 a5 b b4 2e d :2 a0 51
a5 b 7e 51 b4 2e :2 a0 51
a5 b a0 7e a0 51 a5 b
b4 2e :2 a0 51 a5 b a0 7e
a0 51 a5 b b4 2e :3 a0 51
a5 b d b7 :3 a0 51 a5 b
7e :2 a0 51 a5 b b4 2e 7e
:2 a0 51 a5 b b4 2e d b7
:2 19 3c b7 19 3c :5 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 d b7
19 3c :2 a0 51 a5 b a0 7e
a0 51 a5 b b4 2e a0 2b
b7 19 3c b7 a0 47 :2 a0 e9
c1 :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 51 a5 b 7e 51 b4
2e 5a a0 7e b4 2e :2 5a a
10 :5 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 19 3c b7 :2 19 3c :2 a0
51 a5 b 7e 51 b4 2e :3 a0
51 a5 b 51 a5 b 5a 7e
51 b4 2e :4 a0 a5 b d b7
:4 a0 a5 b d b7 :2 19 3c b7
19 3c b7 19 3c b7 19 3c
b7 19 3c :2 a0 6b 7e 51 b4
2e :2 a0 6b 51 d b7 19 3c
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 b7
:2 a0 6b :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 :2 a0 6b :2 6e 7e :2 a0 6b
b4 2e 7e 6e b4 2e 7e :2 a0
6b b4 2e a5 57 b7 19 3c
:9 a0 12a e9 dd :2 a0 65 b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c :7 a0 12a
e9 dd :2 a0 65 b7 a4 b1 11
4f b7 a6 9 a4 b1 11 68
4f 1d 17 b5 
1e0f
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 6f 6b 59 77 84
80 56 8c 95 91 7f 9d aa
a6 7c b2 bb b7 a5 c3 d0
cc a2 d8 e1 dd cb e9 f6
f2 c8 fe 107 103 f1 10f 11c
118 ee 124 12d 129 117 135 142
13e 114 14a 153 14f 13d 15b 168
164 13a 170 179 175 163 181 18e
18a 160 196 19f 19b 189 1a7 186
1ac 1b0 1b4 1b8 1bb 1bf 1dc 1c7
1cb 1ce 1cf 1d7 1c6 1f9 1e7 1c3
1eb 1ec 1f4 1e6 216 204 1e3 208
209 211 203 233 221 200 225 226
22e 220 250 23e 21d 242 243 24b
23d 26d 25b 23a 25f 260 268 25a
28a 278 257 27c 27d 285 277 2a7
295 274 299 29a 2a2 294 2c4 2b2
291 2b6 2b7 2bf 2b1 2e1 2cf 2ae
2d3 2d4 2dc 2ce 2fe 2ec 2cb 2f0
2f1 2f9 2eb 31b 309 2e8 30d 30e
316 308 338 326 305 32a 32b 333
325 355 343 322 347 348 350 342
371 360 364 33f 36c 35f 38d 37c
380 35c 388 37b 3a9 398 39c 378
3a4 397 3c5 3b4 3b8 394 3c0 3b3
3e1 3d0 3d4 3b0 3dc 3cf 3fd 3ec
3f0 3cc 3f8 3eb 419 408 40c 3e8
414 407 435 424 428 404 430 423
451 440 444 420 44c 43f 46d 45c
460 43c 468 45b 489 478 47c 458
484 477 4a5 494 498 474 4a0 493
4c1 4b0 4b4 490 4bc 4af 4dd 4cc
4d0 4ac 4d8 4cb 4f9 4e8 4ec 4c8
4f4 4e7 515 504 508 4e4 510 503
531 520 524 52c 500 549 538 53c
544 51f 566 554 51c 558 559 561
553 583 571 550 575 576 57e 570
5a0 58e 56d 592 593 59b 58d 5bc
5ab 5af 5b7 58a 5d4 5c3 5c7 5cf
5aa 5f0 5df 5e3 5eb 5a7 608 5f7
5fb 603 5de 624 613 617 61f 5db
63c 62b 62f 637 612 659 647 60f
64b 64c 654 646 675 664 668 643
670 663 691 680 684 660 68c 67f
6ad 69c 6a0 67c 6a8 69b 6c9 6b8
6bc 698 6c4 6b7 6e5 6d4 6d8 6b4
6e0 6d3 701 6f0 6f4 6d0 6fc 6ef
71d 70c 710 6ec 718 70b 739 728
72c 734 708 751 740 744 74c 727
76d 75c 760 724 768 75b 789 778
77c 758 784 777 7a5 794 798 774
7a0 793 7c1 7b0 7b4 790 7bc 7af
7dd 7cc 7d0 7ac 7d8 7cb 7f9 7e8
7ec 7c8 7f4 7e7 800 804 7e4 815
818 81c 820 824 828 82c 830 834
838 83c 840 844 848 84c 850 854
860 865 867 873 877 879 87d 88e
88f 892 896 89a 89e 8a2 8a6 8aa
8ae 8b2 8b6 8ba 8be 8ca 8cf 8d1
8dd 8e1 8ff 8e7 8eb 8ef 8f2 8fa
8e6 906 90a 8e3 90e 912 916 919
91e 91f 1 921 926 92a 92e 931
936 93a 93e 941 946 947 949 94e
951 955 956 95b 960 963 968 969
96e 971 975 976 97b 97e 983 984
989 98c 991 992 997 99a 99e 9a2
9a6 9a7 9a9 9ae 9af 9b1 9b2 9b7
9ba 9bf 9c0 9c5 9c8 9cd 9ce 9d3
9d6 9da 9db 9e0 9e3 9e8 9e9 9ee
9f1 9f6 9f7 9fc 9ff a04 a05 a0a
a0d a11 a12 a17 a1a a1f a20 a25
a28 a2d a2e a33 a36 a3b a3c a41
a44 a48 a49 a4e a51 a56 a57 a5c
a5f a64 a65 a6a a6d a71 a75 a79
a7a a7c a81 a82 a84 a85 a8a a8d
a92 a93 a98 a9b aa0 aa1 aa6 aa9
aad aae ab3 ab6 abb abc ac1 ac4
ac9 aca acf ad2 ad6 ada ade adf
ae1 ae6 ae7 ae9 aea aef af2 af7
af8 afd b00 b04 b08 b0c b0d b0f
b14 b15 b17 b18 b1d b20 b25 b26
b2b b2e b32 b36 b3a b3b b3d b42
b43 b45 b46 b4b b4e b53 b54 b59
b5c b60 b64 b68 b69 b6b b70 b71
b73 b74 b79 b7c b81 b82 b87 b8a
b8e b92 b96 b97 b99 b9e b9f ba1
ba2 ba7 baa baf bb0 bb5 bb8 bbc
bc0 bc4 bc5 bc7 bcc bcd bcf bd0
bd5 bd8 bdd bde be3 be6 beb bec
bf1 bf4 bf8 bf9 bfe c01 c06 c07
c0c c0f c14 c15 c1a c1d c22 c23
c28 c2b c2f c30 c35 c38 c3d c3e
c43 c46 c4b c4c c51 c54 c58 c5c
c60 c61 c63 c68 c69 c6b c6c c71
c74 c79 c7a c7f c82 c86 c8a c8e
c8f c91 c96 c97 c99 c9a c9f ca2
ca7 ca8 cad cb0 cb5 cb6 cbb cbe
cc2 cc3 cc8 ccb cd0 cd1 cd6 cd9
cde cdf ce4 ce7 cec ced cf2 cf5
cf9 cfa cff d02 d07 d08 d0d d10
d15 d16 d1b d1e d22 d26 d2a d2b
d2d d32 d33 d35 d36 d3b d3e d43
d44 d49 d4c d51 d52 d57 d5a d5e
d5f d64 d67 d6c d6d d72 d75 d7a
d7b d80 d81 d86 d88 d8c d8f d93
d97 d9a d9e da2 da5 da8 dad db0
db3 db6 db7 dbc dbd 1 dbf dc4
dc8 dcc dcf dd4 dd9 ddc de0 de1
de6 de7 dec dee df2 df5 df9 dfc
dfd e02 e05 e08 e0c e10 e13 e14
e16 e19 e1c e1d e22 e25 e29 e2d
e30 e31 e33 e36 e39 e3a e3f 1
e42 e47 1 e4a e4f e53 e57 e5a
e5e e62 e65 e68 e6d e70 e73 e76
e77 e7c e7d 1 e7f e84 e88 e8c
e8f e94 e99 e9c ea0 ea1 ea6 ea9
eae eaf eb4 eb7 ebc ebd ec2 ec5
ec9 ecd ed1 ed5 ed8 ed9 edb ee0
ee1 ee3 ee4 ee9 eea eef ef1 ef5
ef8 efc f00 f03 f07 f0b f0e f11
f16 f19 f1c f1f f20 f25 f26 1
f28 f2d f31 f35 f38 f3d f42 f45
f49 f4a f4f f50 f55 f57 f5b f5e
f62 f66 f69 f6d f71 f74 f78 f79
f7e f80 f84 f87 f8b f8e f8f f94
f97 f9a f9e fa1 fa2 fa7 faa 1
fad fb2 fb6 fb9 fba fbf fc2 1
fc5 fca fce fd2 fd5 fd9 fdd fe0
fe3 fe8 feb fee ff1 ff2 ff7 ff8
1 ffa fff 1003 1007 100a 100f 1014
1017 101b 101c 1021 1024 1029 102a 102f
1032 1037 1038 103d 1040 1044 1048 104c
1050 1053 1054 1056 105b 105c 105e 105f
1064 1065 106a 106c 1070 1073 1077 107b
107e 1082 1086 1089 108c 1091 1094 1097
109a 109b 10a0 10a1 1 10a3 10a8 10ac
10b0 10b3 10b8 10bd 10c0 10c4 10c5 10ca
10cb 10d0 10d2 10d6 10d9 10dd 10e1 10e4
10e8 10ec 10ef 10f3 10f4 10f9 10fb 10ff
1102 1106 1107 110b 110f 1110 1114 1118
111b 111f 1123 1127 112b 112f 1133 1137
113b 113f 1143 1147 114b 114f 1153 1157
115b 115f 1163 1167 116b 116f 1173 1177
117b 117f 1183 1187 118b 118f 1193 1197
119b 119e 11a2 11a6 11aa 11ae 11af 11b1
11b4 11b5 11b7 11bb 11be 11c2 11c5 11c6
11cb 11cf 11d3 11d6 11d7 11d9 11dc 11df
11e0 11e5 11e8 11ec 11f0 11f3 11f4 11f6
11f9 11fc 11fd 1202 1 1205 120a 120e
1211 1212 1217 121a 121e 1222 1225 1229
122d 1230 1233 1238 123b 123e 1241 1242
1247 1248 1 124a 124f 1253 1257 125a
125f 1264 1267 126b 126c 1271 1274 1279
127a 127f 1282 1287 1288 128d 1290 1294
1298 129c 12a0 12a3 12a4 12a6 12ab 12ac
12ae 12af 12b4 12b5 12ba 12bc 12c0 12c3
12c7 12cb 12ce 12d2 12d6 12d9 12dc 12e1
12e4 12e7 12ea 12eb 12f0 12f1 1 12f3
12f8 12fc 1300 1303 1308 130d 1310 1314
1315 131a 131b 1320 1322 1326 1329 132d
1331 1334 1338 133c 133f 1343 1344 1349
134b 134f 1352 1356 135a 135e 1362 1366
136a 136e 1372 1376 137a 137e 1382 1386
138a 138e 1392 139e 13a2 13a6 13a9 13ad
13b1 13b3 13b7 13bb 13bf 13c2 13c6 13ca
13cc 13cd 13d2 1 13d6 13da 13de 13e1
13e5 13e9 13ed 13f1 13f4 13f8 13fc 13ff
1402 1407 140a 140d 1410 1411 1416 1417
1 1419 141e 1422 1426 1429 142e 1433
1436 143a 143b 1440 1443 1448 1449 144e
1451 1456 1457 145c 145f 1463 1464 1469
146c 1471 1472 1477 147a 147e 1482 1485
1488 1489 148b 148c 1491 1492 1497 1499
149d 14a0 14a2 14a6 14a8 14b4 14b6 14b8
14b9 14be 14c2 14c4 14d0 14d2 14d6 14da
14db 14e0 14e4 14e8 14ec 14f0 14f4 14f7
14f8 14fa 14fb 14fd 1501 1505 1509 150c
1510 1514 1518 151c 1520 1524 1528 1529
152b 152f 1533 1537 153a 153d 1540 1541
1546 1549 154d 1551 1554 1555 1557 155a
155d 155e 1563 1 1566 156b 156f 1573
1576 1577 1579 157c 157f 1580 1585 1
1588 158d 1591 1594 1598 159c 15a0 15a4
15a8 15ac 15b0 15b4 15b8 15c4 15c8 15cc
15cf 15d3 15d7 15d9 15dd 15e1 15e5 15e8
15ec 15f0 15f2 15f3 15f8 1 15fc 1600
1604 1607 160b 160f 1613 1617 161a 161e
1622 1625 1628 162d 1630 1633 1636 1637
163c 163d 1 163f 1644 1648 164c 164f
1654 1659 165c 1660 1661 1666 1669 166e
166f 1674 1677 167c 167d 1682 1685 1689
168a 168f 1692 1697 1698 169d 16a0 16a4
16a8 16ab 16ae 16af 16b1 16b2 16b7 16b8
16bd 16bf 16c3 16c6 16c8 16cc 16ce 16da
16dc 16de 16df 16e4 16e8 16ea 16f6 16f8
16fc 1700 1703 1704 1706 1709 170c 170d
1712 1716 1719 171d 1721 1724 1728 172c
1731 1735 1739 173c 173d 1742 1745 1749
174d 1751 1755 1759 175d 1769 176d 1771
1774 1778 177c 177e 1782 1786 178a 178d
1791 1795 1797 1798 179d 1 17a1 17a5
17a9 17ac 17b0 17b4 17b8 17bc 17bf 17c3
17c7 17ca 17cd 17d2 17d5 17d8 17db 17dc
17e1 17e2 1 17e4 17e9 17ed 17f1 17f4
17f9 17fe 1801 1805 1806 180b 180e 1813
1814 1819 181c 1821 1822 1827 182a 182e
182f 1834 1837 183c 183d 1842 1845 1849
184d 1850 1853 1854 1856 1857 185c 185d
1862 1864 1868 186b 186d 1871 1873 187f
1881 1883 1884 1889 188d 188f 189b 189d
189f 18a3 18a6 18a8 18ac 18af 18b1 18b5
18b8 18bc 18c0 18c3 18c4 18c6 18c9 18cc
18cd 18d2 18d5 18d9 18dd 18e1 18e4 18e5
18e7 18e8 18ea 18ed 18f0 18f3 18f4 1
18f9 18fe 1902 1905 1909 190b 190f 1912
1914 1918 191c 1920 1924 1928 192c 1930
1934 1938 193c 1940 1944 1948 194c 1950
1954 1958 195c 1960 1964 1968 196c 1970
1974 1978 197c 1988 198c 1990 1993 1997
199b 199d 19a1 19a5 19a9 19ac 19b0 19b4
19b6 19b7 19bc 1 19c0 19c4 19c8 19cb
19cf 19d3 19d7 19db 19de 19e2 19e6 19e9
19ec 19f1 19f4 19f7 19fa 19fb 1a00 1a01
1 1a03 1a08 1a0c 1a10 1a13 1a18 1a1d
1a20 1a24 1a25 1a2a 1a2d 1a32 1a33 1a38
1a3b 1a40 1a41 1a46 1a49 1a4d 1a4e 1a53
1a56 1a5b 1a5c 1a61 1a64 1a68 1a6c 1a6f
1a72 1a73 1a75 1a76 1a7b 1a7c 1a81 1a83
1a87 1a8a 1a8c 1a90 1a92 1a9e 1aa0 1aa2
1aa3 1aa8 1aac 1aae 1aba 1abc 1ac0 1ac4
1ac7 1acb 1ace 1ad2 1ad5 1ad6 1adb 1ade
1ae2 1ae6 1ae9 1aea 1aec 1af0 1af3 1af7
1afa 1afb 1afd 1afe 1b03 1 1b06 1b0b
1b0f 1b10 1b14 1b18 1b1b 1b1e 1b1f 1b24
1b28 1b2a 1b2e 1b31 1b33 1b37 1b3b 1b3e
1b42 1b46 1b49 1b4a 1b4c 1b4f 1b52 1b53
1b58 1b5c 1b60 1b63 1b66 1b69 1b6a 1b6f
1b73 1b77 1b7a 1b7e 1b82 1b85 1b88 1b8d
1b90 1b93 1b96 1b97 1b9c 1b9d 1 1b9f
1ba4 1ba8 1bac 1baf 1bb4 1bb9 1bbc 1bc0
1bc1 1bc6 1bc9 1bce 1bcf 1bd4 1bd7 1bdc
1bdd 1be2 1be5 1be9 1bed 1bf1 1bf5 1bf8
1bf9 1bfb 1c00 1c01 1c03 1c04 1c09 1c0c
1c11 1c12 1c17 1c1a 1c1f 1c20 1c25 1c28
1c2c 1c2d 1c32 1c35 1c3a 1c3b 1c40 1c41
1c46 1c48 1c4c 1c4f 1c53 1c57 1c5a 1c5e
1c62 1c65 1c68 1c6d 1c70 1c73 1c76 1c77
1c7c 1c7d 1 1c7f 1c84 1c88 1c8c 1c8f
1c94 1c99 1c9c 1ca0 1ca1 1ca6 1ca7 1cac
1cae 1cb2 1cb5 1cb9 1cbd 1cc0 1cc4 1cc8
1ccb 1ccf 1cd3 1cd4 1cd9 1cdb 1cdf 1ce2
1ce6 1ce9 1cea 1cef 1cf2 1cf6 1cf9 1cfa
1cff 1 1d02 1d07 1d0b 1d0e 1d0f 1d14
1 1d17 1d1c 1d20 1d23 1d24 1d29 1
1d2c 1d31 1d35 1d39 1d3c 1d40 1d44 1d47
1d4a 1d4f 1d52 1d55 1d58 1d59 1d5e 1d5f
1 1d61 1d66 1d6a 1d6e 1d71 1d76 1d7b
1d7e 1d82 1d83 1d88 1d8b 1d90 1d91 1d96
1d99 1d9e 1d9f 1da4 1da7 1dab 1daf 1db3
1db7 1dba 1dbb 1dbd 1dc2 1dc3 1dc5 1dc6
1dcb 1dcc 1dd1 1dd3 1dd7 1dda 1dde 1de2
1de5 1de9 1ded 1df0 1df3 1df8 1dfb 1dfe
1e01 1e02 1e07 1e08 1 1e0a 1e0f 1e13
1e17 1e1a 1e1f 1e24 1e27 1e2b 1e2c 1e31
1e32 1e37 1e39 1e3d 1e40 1e44 1e48 1e4b
1e4f 1e53 1e56 1e5a 1e5b 1e60 1e62 1e66
1e69 1e6d 1e71 1e75 1e78 1e7c 1e80 1e84
1e87 1e88 1e8a 1e8d 1e90 1e91 1e96 1e9a
1e9d 1ea1 1ea3 1ea7 1eaa 1eae 1eb2 1eb6
1eba 1ebe 1ec2 1ece 1ed2 1ed6 1ed9 1edd
1ee1 1ee3 1ee7 1eeb 1eef 1ef2 1ef6 1efa
1efc 1efd 1f02 1 1f06 1f0a 1f0e 1f11
1f15 1f19 1f1d 1f21 1f24 1f28 1f2c 1f2f
1f32 1f37 1f3a 1f3d 1f40 1f41 1f46 1f47
1 1f49 1f4e 1f52 1f56 1f59 1f5e 1f63
1f66 1f6a 1f6b 1f70 1f73 1f78 1f79 1f7e
1f81 1f86 1f87 1f8c 1f8f 1f93 1f94 1f99
1f9c 1fa1 1fa2 1fa7 1faa 1fae 1fb2 1fb5
1fb8 1fb9 1fbb 1fbc 1fc1 1fc2 1fc7 1fc9
1fcd 1fd0 1fd2 1fd6 1fd8 1fe4 1fe6 1fe8
1fe9 1fee 1ff2 1ff4 2000 2002 2006 200a
200d 200e 2010 2013 2016 2017 201c 2020
2024 2028 202c 2030 2031 2033 2036 2037
2039 203c 2040 2044 2048 2049 204b 204e
204f 2051 2052 2057 205a 205d 2060 2061
2066 2069 206c 206f 2070 2075 2076 2078
207c 2080 2084 2087 2088 208a 208e 2091
2095 2098 2099 209b 209c 20a1 20a5 20a8
20ac 20b0 20b3 20b7 20b9 20bd 20c0 20c4
20c6 20ca 20ce 20d1 20d3 20d7 20da 20de
20df 20e3 20e7 20eb 20ef 20f2 20f3 20f5
20f8 20f9 20fb 20fe 2101 2104 2105 210a
210e 2112 2115 2116 2118 211b 211e 211f
2124 2128 212c 212f 2130 2132 2135 2138
2139 213e 2142 2146 2149 214d 2151 2155
2159 215d 2161 2165 2169 216d 2171 2175
2179 217d 2181 2185 2189 218d 2191 2192
2194 2198 219c 21a0 21a3 21a6 21a9 21aa
21af 21b3 21b7 21bb 21bf 21c3 21c7 21cb
21cf 21d3 21d7 21db 21df 21e3 21e7 21eb
21f7 21fb 2200 2204 2207 2208 220d 2211
2215 2218 221b 221f 2221 2225 2229 222c
222f 2233 2235 2239 223d 2240 2242 2246
224a 224e 2251 2255 2259 225b 225c 2261
1 2265 2269 226d 2270 2274 2278 227c
2280 2283 2287 228b 228e 2291 2296 2299
229c 229f 22a0 22a5 22a6 1 22a8 22ad
22b1 22b5 22b8 22bd 22c2 22c5 22c9 22ca
22cf 22d2 22d7 22d8 22dd 22e0 22e5 22e6
22eb 22ee 22f2 22f3 22f8 22fb 2300 2301
2306 2309 230d 2311 2314 2317 2318 231a
231b 2320 2321 2326 2328 232c 232f 2331
2335 2337 2343 2345 2347 2348 234d 2351
2353 235f 2361 2363 2367 236a 236c 2370
2374 2377 237b 237f 2383 2387 238b 238f
2393 2397 239b 239f 23a3 23a7 23ab 23af
23b3 23b7 23bb 23bf 23c0 23c2 23c6 23c8
23cc 23d0 23d3 23d5 23d9 23dd 23e1 23e4
23e5 23e7 23ea 23ed 23ee 23f3 23f7 23fb
23ff 2402 2403 2405 2408 240b 240c 2411
2415 2419 241c 2420 2424 2428 242c 2430
2434 2438 243c 2440 2444 2448 244c 2450
2454 2458 245c 2460 2464 2465 2467 246b
246f 2473 2476 2479 247c 247d 2482 2486
248a 248e 2492 2496 249a 249e 24a2 24a6
24aa 24ae 24ba 24be 24c3 24c7 24ca 24cb
24d0 24d4 24d8 24db 24de 24e2 24e4 24e8
24ec 24ef 24f2 24f6 24f8 24fc 2500 2503
2505 2509 250d 2511 2514 2518 251c 251e
251f 2524 1 2528 252c 2530 2533 2537
253b 253f 2543 2546 254a 254e 2551 2554
2559 255c 255f 2562 2563 2568 2569 1
256b 2570 2574 2578 257b 2580 2585 2588
258c 258d 2592 2595 259a 259b 25a0 25a3
25a8 25a9 25ae 25b1 25b5 25b6 25bb 25be
25c3 25c4 25c9 25cc 25d0 25d4 25d7 25da
25db 25dd 25de 25e3 25e4 25e9 25eb 25ef
25f2 25f4 25f8 25fa 2606 2608 260a 260b
2610 2614 2616 2622 2624 2626 262a 262d
262f 2633 2637 263a 263e 2642 2646 264a
264e 2652 2656 265a 265e 2662 2666 266a
266e 2672 2676 267a 267e 2682 2683 2685
2689 268b 268f 2693 2696 2698 269c 26a0
26a3 26a7 26ab 26ae 26b1 26b4 26b5 26ba
26bd 26c1 26c5 26c9 26cc 26cd 26cf 26d2
26d3 26d5 26d8 26db 26de 26df 26e4 1
26e7 26ec 26f0 26f4 26f7 26f8 26fa 26fe
2701 2705 2708 2709 270b 270c 2711 2714
2718 271c 271f 2720 2722 2725 2728 2729
272e 1 2731 2736 273a 273e 2741 2742
2744 2747 274a 274b 2750 1 2753 2758
275c 2760 2764 2768 276c 2770 2774 2778
277c 2780 2784 2788 278c 2790 2794 2798
279c 27a0 27a4 27a8 27ac 27b0 27b4 27c0
27c4 27c9 27cd 27d0 27d1 27d6 27da 27de
27e1 27e4 27e8 27ea 27ee 27f2 27f5 27f8
27fc 27fe 2802 2806 2809 280b 280f 2813
2817 281a 281e 2822 2824 2825 282a 1
282e 2832 2836 2839 283d 2841 2845 2849
284c 2850 2854 2857 285a 285f 2862 2865
2868 2869 286e 286f 1 2871 2876 287a
287e 2881 2886 288b 288e 2892 2893 2898
289b 28a0 28a1 28a6 28a9 28ae 28af 28b4
28b7 28bb 28bc 28c1 28c4 28c9 28ca 28cf
28d2 28d6 28da 28dd 28e0 28e1 28e3 28e4
28e9 28ea 28ef 28f1 28f5 28f8 28fa 28fe
2900 290c 290e 2910 2911 2916 291a 291c
2928 292a 292e 2932 2936 293a 293e 2942
2946 294a 2956 295a 295f 2963 2966 2967
296c 2970 2974 2977 297a 297e 2980 2984
2988 298b 298e 2992 2994 2998 299c 299f
29a1 29a5 29a9 29ad 29b0 29b4 29b8 29ba
29bb 29c0 1 29c4 29c8 29cc 29cf 29d3
29d7 29db 29df 29e2 29e6 29ea 29ed 29f0
29f5 29f8 29fb 29fe 29ff 2a04 2a05 1
2a07 2a0c 2a10 2a14 2a17 2a1c 2a21 2a24
2a28 2a29 2a2e 2a31 2a36 2a37 2a3c 2a3f
2a44 2a45 2a4a 2a4d 2a51 2a52 2a57 2a5a
2a5f 2a60 2a65 2a68 2a6c 2a70 2a73 2a76
2a77 2a79 2a7a 2a7f 2a80 2a85 2a87 2a8b
2a8e 2a90 2a94 2a96 2aa2 2aa4 2aa6 2aa7
2aac 2ab0 2ab2 2abe 2ac0 2ac4 2ac8 2acc
2ad0 2ad4 2ae0 2ae4 2ae9 2aed 2af0 2af1
2af6 2afa 2afe 2b01 2b04 2b08 2b0a 2b0e
2b12 2b15 2b18 2b1c 2b1e 2b22 2b26 2b29
2b2b 2b2f 2b33 2b37 2b3a 2b3e 2b42 2b44
2b45 2b4a 1 2b4e 2b52 2b56 2b59 2b5d
2b61 2b65 2b69 2b6c 2b70 2b74 2b77 2b7a
2b7f 2b82 2b85 2b88 2b89 2b8e 2b8f 1
2b91 2b96 2b9a 2b9e 2ba1 2ba6 2bab 2bae
2bb2 2bb3 2bb8 2bbb 2bc0 2bc1 2bc6 2bc9
2bce 2bcf 2bd4 2bd7 2bdb 2bdc 2be1 2be4
2be9 2bea 2bef 2bf2 2bf6 2bfa 2bfd 2c00
2c01 2c03 2c04 2c09 2c0a 2c0f 2c11 2c15
2c18 2c1a 2c1e 2c20 2c2c 2c2e 2c30 2c31
2c36 2c3a 2c3c 2c48 2c4a 2c4c 2c50 2c54
2c58 2c5c 2c60 2c64 2c68 2c6c 2c70 2c74
2c78 2c84 2c88 2c8d 2c91 2c94 2c95 2c9a
2c9e 2ca2 2ca5 2ca8 2cac 2cae 2cb2 2cb6
2cb9 2cbc 2cc0 2cc2 2cc6 2cca 2ccd 2ccf
2cd3 2cd7 2cdb 2cde 2ce2 2ce6 2ce8 2ce9
2cee 1 2cf2 2cf6 2cfa 2cfd 2d01 2d05
2d09 2d0d 2d10 2d14 2d18 2d1b 2d1e 2d23
2d26 2d29 2d2c 2d2d 2d32 2d33 1 2d35
2d3a 2d3e 2d42 2d45 2d4a 2d4f 2d52 2d56
2d57 2d5c 2d5f 2d64 2d65 2d6a 2d6d 2d72
2d73 2d78 2d7b 2d7f 2d80 2d85 2d88 2d8d
2d8e 2d93 2d96 2d9a 2d9e 2da1 2da4 2da5
2da7 2da8 2dad 2dae 2db3 2db5 2db9 2dbc
2dbe 2dc2 2dc4 2dd0 2dd2 2dd4 2dd5 2dda
2dde 2de0 2dec 2dee 2df0 2df4 2df8 2dfb
2dfd 2e01 2e04 2e06 2e0a 2e0e 2e12 2e16
2e19 2e1a 2e1c 2e20 2e24 2e27 2e28 2e2a
2e2e 2e32 2e36 2e39 2e3a 2e3c 2e3f 2e42
2e43 2e48 2e4c 2e50 2e53 2e57 2e5b 2e5f
2e63 2e67 2e6b 2e6f 2e73 2e77 2e7b 2e7f
2e83 2e87 2e8b 2e8f 2e93 2e97 2e9b 2e9c
2e9e 2ea2 2ea6 2eaa 2ead 2eb0 2eb3 2eb4
2eb9 2ebc 2ec0 2ec4 2ec7 2ec8 2eca 2ecd
2ed0 2ed1 2ed6 1 2ed9 2ede 2ee2 2ee6
2ee9 2eea 2eec 2eef 2ef2 2ef3 2ef8 1
2efb 2f00 2f04 2f08 2f0c 2f10 2f14 2f18
2f1c 2f20 2f24 2f28 2f2c 2f30 2f34 2f38
2f3c 2f48 2f4c 2f51 2f55 2f58 2f59 2f5e
2f62 2f66 2f69 2f6c 2f70 2f72 2f76 2f7a
2f7d 2f80 2f84 2f86 2f8a 2f8e 2f91 2f93
2f97 2f9b 2f9f 2fa2 2fa6 2faa 2fac 2fad
2fb2 1 2fb6 2fba 2fbe 2fc1 2fc5 2fc9
2fcd 2fd1 2fd4 2fd8 2fdc 2fdf 2fe2 2fe7
2fea 2fed 2ff0 2ff1 2ff6 2ff7 1 2ff9
2ffe 3002 3006 3009 300e 3013 3016 301a
301b 3020 3023 3028 3029 302e 3031 3036
3037 303c 303f 3043 3044 3049 304c 3051
3052 3057 305a 305e 3062 3065 3068 3069
306b 306c 3071 3072 3077 3079 307d 3080
3082 3086 3088 3094 3096 3098 3099 309e
30a2 30a4 30b0 30b2 30b4 30b8 30bb 30bd
30c1 30c4 30c5 30ca 30cd 30d1 30d5 30d9
30dd 30e1 30e5 30e9 30ed 30f1 30f5 30f9
30fd 3101 310d 3111 3115 3118 311c 3120
3122 3126 312a 312e 3131 3135 3139 313b
313c 3141 1 3145 3149 314d 3150 3154
3158 315c 3160 3163 3167 316b 316e 3171
3176 3179 317c 317f 3180 3185 3186 1
3188 318d 3191 3195 3198 319d 31a2 31a5
31a9 31aa 31af 31b2 31b7 31b8 31bd 31c0
31c5 31c6 31cb 31ce 31d2 31d3 31d8 31db
31e0 31e1 31e6 31e9 31ed 31f1 31f4 31f7
31f8 31fa 31fb 3200 3201 3206 3208 320c
320f 3211 3215 3217 3223 3225 3227 3228
322d 3231 3233 323f 3241 3245 3249 324c
3250 3254 3257 3258 325a 325d 3260 3261
3266 326a 326e 3272 3275 3279 327d 3280
3284 3288 328c 3290 3294 32a0 32a4 32a8
32ab 32af 32b3 32b5 32b9 32bd 32c1 32c4
32c8 32cc 32ce 32cf 32d4 1 32d8 32dc
32e0 32e3 32e7 32eb 32ef 32f3 32f6 32fa
32fe 3301 3304 3309 330c 330f 3312 3313
3318 3319 1 331b 3320 3324 3328 332b
3330 3335 3338 333c 333d 3342 3345 334a
334b 3350 3353 3358 3359 335e 3361 3365
3366 336b 336e 3373 3374 3379 337c 3380
3384 3387 338a 338b 338d 338e 3393 3394
3399 339b 339f 33a2 33a4 33a8 33aa 33b6
33b8 33ba 33bb 33c0 33c4 33c6 33d2 33d4
33d8 33dc 33e0 33e4 33e8 33ec 33f0 33f4
33f5 33f7 33fb 33ff 3403 3406 3407 3409
340c 340f 3410 3415 3419 341a 341e 3422
3423 3427 342b 342c 3430 3434 3435 3439
343d 343e 3442 3446 344a 344f 3453 3454
3458 345c 3460 3465 346a 346e 3472 3476
347b 347f 3483 3486 3489 348d 348f 3493
3497 349a 349d 34a1 34a3 34a7 34ab 34ae
34b2 34b6 34bb 34bd 34c1 34c5 34c8 34cc
34d0 34d2 34d6 34da 34de 34e1 34e5 34e9
34eb 34ec 34f1 1 34f5 34f9 34fd 3500
3504 3508 350c 3510 3513 3517 351b 351e
3521 3526 3529 352c 352f 3530 3535 3536
1 3538 353d 3541 3545 3548 354d 3552
3555 3559 355a 355f 3562 3567 3568 356d
3570 3575 3576 357b 357e 3582 3583 3588
358b 3590 3591 3596 3599 359d 35a1 35a4
35a7 35a8 35aa 35ab 35b0 35b1 35b6 35b8
35bc 35bf 35c1 35c5 35c7 35d3 35d5 35d7
35d8 35dd 35e1 35e3 35ef 35f1 35f5 35f9
35fd 3601 3605 3611 3615 3619 361c 3620
3624 3626 362a 362e 3632 3635 3639 363d
363f 3640 3645 1 3649 364d 3651 3654
3658 365c 3660 3664 3667 366b 366f 3672
3675 367a 367d 3680 3683 3684 3689 368a
1 368c 3691 3695 3699 369c 36a1 36a6
36a9 36ad 36ae 36b3 36b6 36bb 36bc 36c1
36c4 36c9 36ca 36cf 36d2 36d6 36d7 36dc
36df 36e4 36e5 36ea 36ed 36f1 36f5 36f8
36fb 36fc 36fe 36ff 3704 3705 370a 370c
3710 3713 3715 3719 371b 3727 3729 372b
372c 3731 3735 3737 3743 3745 3749 374e
3752 3756 375b 375f 3763 3767 376b 376e
3772 3773 3778 377b 377e 377f 3781 3785
3789 378d 3791 3794 3799 379a 379f 37a2
37a5 37a6 37a8 37ac 37b0 37b4 37b8 37bb
37bf 37c0 37c5 37c8 37cb 37cc 37ce 37d2
37d6 37da 37de 37e1 37e6 37e7 37ec 37ef
37f2 37f3 37f5 37f9 37fd 3801 3805 3809
380d 3811 3815 3819 381d 3821 3825 3829
382d 3831 3835 3839 383d 383e 3840 3844
3848 384c 384f 3853 3857 385a 385d 3862
3865 3868 386b 386c 3871 3872 1 3874
3879 387d 3881 3884 3889 388e 3891 3895
3896 389b 389e 38a3 38a4 38a9 38ac 38b1
38b2 38b7 38ba 38be 38c2 38c5 38c6 38c8
38cd 38ce 38d0 38d1 38d6 38d9 38de 38df
38e4 38e7 38ec 38ed 38f2 38f5 38f9 38fa
38ff 3902 3907 3908 390d 390e 3913 3915
3919 391c 3920 3924 3927 392b 392f 3932
3935 393a 393d 3940 3943 3944 3949 394a
1 394c 3951 3955 3959 395c 3961 3966
3969 396d 396e 3973 3974 3979 397b 397f
3982 3986 398a 398d 3990 3994 3998 3999
399e 39a0 39a4 39a7 39a9 39ad 39b0 39b2
39b6 39b9 39bd 39c1 39c5 39c8 39c9 39cb
39ce 39d1 39d2 39d7 39db 39df 39e3 39e6
39ea 39ee 39f2 39f6 39fa 39fe 3a02 3a06
3a0a 3a0e 3a12 3a16 3a1a 3a1e 3a22 3a26
3a2a 3a2e 3a2f 3a31 3a35 3a39 3a3d 3a40
3a43 3a46 3a47 3a4c 3a4f 3a53 3a57 3a5a
3a5b 3a5d 3a60 3a63 3a64 3a69 1 3a6c
3a71 3a75 3a79 3a7c 3a7d 3a7f 3a82 3a85
3a86 3a8b 1 3a8e 3a93 3a97 3a9b 3a9f
3aa3 3aa7 3aab 3aaf 3ab3 3ab7 3abb 3abf
3acb 3acf 3ad4 3ad8 3adb 3adc 3ae1 3ae5
3ae9 3aec 3aef 3af3 3af5 3af9 3afd 3b00
3b03 3b07 3b09 3b0d 3b11 3b14 3b16 3b1a
3b1e 3b22 3b25 3b29 3b2d 3b2f 3b30 3b35
1 3b39 3b3d 3b41 3b44 3b48 3b4c 3b50
3b54 3b57 3b5b 3b5f 3b62 3b65 3b6a 3b6d
3b70 3b73 3b74 3b79 3b7a 1 3b7c 3b81
3b85 3b89 3b8c 3b91 3b96 3b99 3b9d 3b9e
3ba3 3ba6 3bab 3bac 3bb1 3bb4 3bb9 3bba
3bbf 3bc2 3bc6 3bc7 3bcc 3bcf 3bd4 3bd5
3bda 3bdd 3be1 3be5 3be8 3beb 3bec 3bee
3bef 3bf4 3bf5 3bfa 3bfc 3c00 3c03 3c05
3c09 3c0b 3c17 3c19 3c1b 3c1c 3c21 3c25
3c27 3c33 3c35 3c37 3c3b 3c3e 3c40 3c44
3c48 3c4b 3c4d 3c51 3c55 3c58 3c5c 3c60
3c63 3c66 3c69 3c6a 3c6f 3c73 3c77 3c7a
3c7e 3c82 3c85 3c88 3c8d 3c90 3c93 3c96
3c97 3c9c 3c9d 1 3c9f 3ca4 3ca8 3cac
3caf 3cb4 3cb9 3cbc 3cc0 3cc1 3cc6 3cc9
3cce 3ccf 3cd4 3cd7 3cdc 3cdd 3ce2 3ce5
3ce9 3ced 3cf1 3cf5 3cf8 3cf9 3cfb 3d00
3d01 3d03 3d04 3d09 3d0c 3d11 3d12 3d17
3d1a 3d1f 3d20 3d25 3d28 3d2c 3d2d 3d32
3d35 3d3a 3d3b 3d40 3d41 3d46 3d48 3d4c
3d4f 3d53 3d57 3d5a 3d5e 3d62 3d65 3d68
3d6d 3d70 3d73 3d76 3d77 3d7c 3d7d 1
3d7f 3d84 3d88 3d8c 3d8f 3d94 3d99 3d9c
3da0 3da1 3da6 3da7 3dac 3dae 3db2 3db5
3db9 3dbd 3dc0 3dc4 3dc8 3dcb 3dcf 3dd3
3dd4 3dd9 3ddb 3ddf 3de2 3de6 3dea 3dee
3df2 3df6 3df9 3dfa 3dfc 3dff 3e02 3e03
3e08 3e0b 3e0f 3e13 3e16 3e17 3e19 3e1c
3e1f 3e20 3e25 1 3e28 3e2d 3e31 3e34
3e38 3e3a 3e3e 3e41 3e45 3e49 3e4c 3e4d
3e4f 3e52 3e55 3e56 3e5b 3e5f 3e63 3e67
3e6a 3e6b 3e6d 3e70 3e71 3e73 3e76 3e79
3e7c 3e7d 3e82 3e86 3e89 3e8a 3e8f 3e92
3e96 3e99 3e9a 3e9f 3ea2 3ea6 3eaa 3eae
3eb2 3eb6 3eba 3ebe 3ec2 3ec6 3eca 3ece
3ed2 3ed6 3eda 3ede 3ee2 3ee6 3eea 3eee
3ef2 3ef6 3f02 3f06 3f0b 3f0f 3f12 3f13
3f18 3f1c 3f20 3f23 3f26 3f2a 3f2c 3f30
3f34 3f37 3f3a 3f3e 3f40 3f44 3f48 3f4b
3f4d 3f51 3f55 3f59 3f5c 3f60 3f64 3f66
3f67 3f6c 1 3f70 3f74 3f78 3f7b 3f7f
3f83 3f87 3f8b 3f8e 3f92 3f96 3f99 3f9c
3fa1 3fa4 3fa7 3faa 3fab 3fb0 3fb1 1
3fb3 3fb8 3fbc 3fc0 3fc3 3fc8 3fcd 3fd0
3fd4 3fd5 3fda 3fdd 3fe2 3fe3 3fe8 3feb
3ff0 3ff1 3ff6 3ff9 3ffd 3ffe 4003 4006
400b 400c 4011 4014 4018 401c 401f 4022
4023 4025 4026 402b 402c 4031 4033 4037
403a 403c 4040 4042 404e 4050 4052 4053
4058 405c 405e 406a 406c 406e 4072 4076
407a 407e 4082 4086 408a 408e 4092 409e
40a2 40a7 40ab 40ae 40af 40b4 40b8 40bc
40bf 40c2 40c6 40c8 40cc 40d0 40d3 40d6
40da 40dc 40e0 40e4 40e7 40e9 40ed 40f1
40f5 40f8 40fc 4100 4102 4103 4108 1
410c 4110 4114 4117 411b 411f 4123 4127
412a 412e 4132 4135 4138 413d 4140 4143
4146 4147 414c 414d 1 414f 4154 4158
415c 415f 4164 4169 416c 4170 4171 4176
4179 417e 417f 4184 4187 418c 418d 4192
4195 4199 419a 419f 41a2 41a7 41a8 41ad
41b0 41b4 41b8 41bb 41be 41bf 41c1 41c2
41c7 41c8 41cd 41cf 41d3 41d6 41d8 41dc
41de 41ea 41ec 41ee 41ef 41f4 41f8 41fa
4206 4208 420a 420e 4212 4215 4217 421b
421e 421f 4224 4227 422b 422f 4233 4237
423b 423f 4243 4247 424b 424f 4253 4257
425b 425f 4263 4267 426b 426f 4273 4277
427b 427f 4283 4287 428b 428f 4293 4297
429b 429f 42a3 42a7 42ab 42af 42b3 42b7
42bb 42bf 42c3 42c7 42cb 42cf 42d3 42d7
42db 42df 42e3 42e7 42eb 42ef 42f3 42f7
42fb 42ff 4303 4307 430b 430f 4313 4317
431b 431f 4323 4327 4333 4337 433c 4340
4343 4344 4349 434d 4351 4354 4357 435b
435d 4361 4365 4368 436b 436f 4371 4375
4379 437c 437e 4382 4386 438a 438d 4391
4395 4397 4398 439d 1 43a1 43a5 43a9
43ac 43b0 43b4 43b8 43bc 43bf 43c3 43c7
43ca 43cd 43d2 43d5 43d8 43db 43dc 43e1
43e2 1 43e4 43e9 43ed 43f1 43f4 43f9
43fe 4401 4405 4406 440b 440e 4413 4414
4419 441c 4421 4422 4427 442a 442e 442f
4434 4437 443c 443d 4442 4445 4449 444d
4450 4453 4454 4456 4457 445c 445d 4462
4464 4468 446b 446d 4471 4473 447f 4481
4483 4484 4489 448d 448f 449b 449d 449f
44a3 44a7 44ab 44af 44b3 44b7 44bb 44bf
44c3 44c7 44cb 44cf 44d3 44d7 44db 44df
44e3 44e7 44eb 44ef 44f3 44f7 44fb 44ff
4503 4507 450b 450f 4513 4517 451b 451f
4523 4527 452b 452f 4533 4537 453b 453f
4543 4547 454b 454f 4553 4557 455b 455f
4563 4567 456b 456f 457b 457f 4584 4588
458b 458c 4591 4595 4599 459c 459f 45a3
45a5 45a9 45ad 45b0 45b3 45b7 45b9 45bd
45c1 45c4 45c6 45ca 45ce 45d2 45d5 45d9
45dd 45df 45e0 45e5 1 45e9 45ed 45f1
45f4 45f8 45fc 4600 4604 4607 460b 460f
4612 4615 461a 461d 4620 4623 4624 4629
462a 1 462c 4631 4635 4639 463c 4641
4646 4649 464d 464e 4653 4656 465b 465c
4661 4664 4669 466a 466f 4672 4676 4677
467c 467f 4684 4685 468a 468d 4691 4695
4698 469b 469c 469e 469f 46a4 46a5 46aa
46ac 46b0 46b3 46b5 46b9 46bb 46c7 46c9
46cb 46cc 46d1 46d5 46d7 46e3 46e5 46e7
46eb 46ef 46f2 46f4 46f8 46fc 46ff 4703
4705 4709 470d 4710 4711 4713 4716 4719
471a 471f 4723 4727 472b 472f 4733 4737
473b 473f 4743 474f 4753 4758 475c 475f
4760 4765 4769 476d 4770 4773 4777 4779
477d 4781 4784 4787 478b 478d 4791 4795
4798 479a 479e 47a2 47a6 47a9 47ad 47b1
47b3 47b4 47b9 1 47bd 47c1 47c5 47c8
47cc 47d0 47d4 47d8 47db 47df 47e3 47e6
47e9 47ee 47f1 47f4 47f7 47f8 47fd 47fe
1 4800 4805 4809 480d 4810 4815 481a
481d 4821 4822 4827 482a 482f 4830 4835
4838 483d 483e 4843 4846 484a 484b 4850
4853 4858 4859 485e 4861 4865 4869 486c
486f 4870 4872 4873 4878 4879 487e 4880
4884 4887 4889 488d 488f 489b 489d 489f
48a0 48a5 48a9 48ab 48b7 48b9 48bb 48bf
48c3 48c6 48c8 48cc 48d0 48d4 48d7 48d8
48da 48dd 48de 48e0 48e3 48e6 48e9 48ea
48ef 48f3 48f6 48f7 48fc 48ff 4903 4906
4907 490c 490f 4913 4917 491b 491f 4923
4927 492b 492f 4933 4937 493b 493f 4943
4947 494b 494f 4953 4957 495b 495f 4963
496f 4973 4978 497c 497f 4980 4985 4989
498d 4990 4993 4997 4999 499d 49a1 49a4
49a7 49ab 49ad 49b1 49b5 49b8 49ba 49be
49c2 49c6 49c9 49cd 49d1 49d3 49d4 49d9
1 49dd 49e1 49e5 49e8 49ec 49f0 49f4
49f8 49fb 49ff 4a03 4a06 4a09 4a0e 4a11
4a14 4a17 4a18 4a1d 4a1e 1 4a20 4a25
4a29 4a2d 4a30 4a35 4a3a 4a3d 4a41 4a42
4a47 4a4a 4a4f 4a50 4a55 4a58 4a5d 4a5e
4a63 4a66 4a6a 4a6b 4a70 4a73 4a78 4a79
4a7e 4a81 4a85 4a89 4a8c 4a8f 4a90 4a92
4a93 4a98 4a99 4a9e 4aa0 4aa4 4aa7 4aa9
4aad 4aaf 4abb 4abd 4abf 4ac0 4ac5 4ac9
4acb 4ad7 4ad9 4adb 4adf 4ae3 4ae7 4aeb
4aef 4af3 4af7 4afb 4aff 4b03 4b07 4b13
4b17 4b1c 4b20 4b23 4b24 4b29 4b2d 4b31
4b34 4b37 4b3b 4b3d 4b41 4b45 4b48 4b4b
4b4f 4b51 4b55 4b59 4b5c 4b5e 4b62 4b66
4b6a 4b6d 4b71 4b75 4b77 4b78 4b7d 1
4b81 4b85 4b89 4b8c 4b90 4b94 4b98 4b9c
4b9f 4ba3 4ba7 4baa 4bad 4bb2 4bb5 4bb8
4bbb 4bbc 4bc1 4bc2 1 4bc4 4bc9 4bcd
4bd1 4bd4 4bd9 4bde 4be1 4be5 4be6 4beb
4bee 4bf3 4bf4 4bf9 4bfc 4c01 4c02 4c07
4c0a 4c0e 4c0f 4c14 4c17 4c1c 4c1d 4c22
4c25 4c29 4c2d 4c30 4c33 4c34 4c36 4c37
4c3c 4c3d 4c42 4c44 4c48 4c4b 4c4d 4c51
4c53 4c5f 4c61 4c63 4c64 4c69 4c6d 4c6f
4c7b 4c7d 4c7f 4c83 4c87 4c8a 4c8c 4c90
4c93 4c94 4c99 4c9c 4ca0 4ca4 4ca8 4cac
4cb0 4cb4 4cb8 4cbc 4cc0 4cc4 4cc8 4ccc
4cd0 4cd4 4cd8 4cdc 4ce0 4ce4 4ce8 4cec
4cf0 4cf4 4cf8 4cfc 4d00 4d04 4d08 4d0c
4d10 4d14 4d18 4d1c 4d20 4d24 4d28 4d2c
4d30 4d34 4d38 4d3c 4d40 4d44 4d48 4d4c
4d50 4d54 4d58 4d5c 4d60 4d64 4d68 4d6c
4d70 4d74 4d78 4d7c 4d80 4d84 4d88 4d8c
4d90 4d94 4d98 4d9c 4da8 4dac 4db1 4db5
4db8 4db9 4dbe 4dc2 4dc6 4dc9 4dcc 4dd0
4dd2 4dd6 4dda 4ddd 4de0 4de4 4de6 4dea
4dee 4df1 4df3 4df7 4dfb 4dff 4e02 4e06
4e0a 4e0c 4e0d 4e12 1 4e16 4e1a 4e1e
4e21 4e25 4e29 4e2d 4e31 4e34 4e38 4e3c
4e3f 4e42 4e47 4e4a 4e4d 4e50 4e51 4e56
4e57 1 4e59 4e5e 4e62 4e66 4e69 4e6e
4e73 4e76 4e7a 4e7b 4e80 4e83 4e88 4e89
4e8e 4e91 4e96 4e97 4e9c 4e9f 4ea3 4ea4
4ea9 4eac 4eb1 4eb2 4eb7 4eba 4ebe 4ec2
4ec5 4ec8 4ec9 4ecb 4ecc 4ed1 4ed2 4ed7
4ed9 4edd 4ee0 4ee2 4ee6 4ee8 4ef4 4ef6
4ef8 4ef9 4efe 4f02 4f04 4f10 4f12 4f14
4f18 4f1c 4f20 4f24 4f28 4f2c 4f30 4f34
4f38 4f3c 4f40 4f44 4f48 4f4c 4f50 4f54
4f58 4f5c 4f60 4f64 4f68 4f6c 4f70 4f74
4f78 4f7c 4f80 4f84 4f88 4f8c 4f90 4f94
4f98 4f9c 4fa0 4fa4 4fa8 4fac 4fb0 4fb4
4fb8 4fbc 4fc0 4fc4 4fc8 4fcc 4fd0 4fd4
4fd8 4fdc 4fe0 4fe4 4fe8 4fec 4ff8 4ffc
5001 5005 5008 5009 500e 5012 5016 5019
501c 5020 5022 5026 502a 502d 5030 5034
5036 503a 503e 5041 5043 5047 504b 504f
5052 5056 505a 505c 505d 5062 1 5066
506a 506e 5071 5075 5079 507d 5081 5084
5088 508c 508f 5092 5097 509a 509d 50a0
50a1 50a6 50a7 1 50a9 50ae 50b2 50b6
50b9 50be 50c3 50c6 50ca 50cb 50d0 50d3
50d8 50d9 50de 50e1 50e6 50e7 50ec 50ef
50f3 50f4 50f9 50fc 5101 5102 5107 510a
510e 5112 5115 5118 5119 511b 511c 5121
5122 5127 5129 512d 5130 5132 5136 5138
5144 5146 5148 5149 514e 5152 5154 5160
5162 5164 5168 516c 516f 5171 5175 5179
517c 5180 5182 5186 518a 518d 518e 5190
5193 5196 5197 519c 51a0 51a3 51a4 51a9
51ac 51b0 51b4 51b7 51b8 51ba 51bd 51c0
51c1 51c6 51ca 51ce 51d2 51d6 51da 51de
51e2 51e6 51ea 51ee 51f2 51f6 51fa 51fe
520a 520e 5213 5217 521a 521b 5220 5224
5228 522b 522e 5232 5234 5238 523c 523f
5242 5246 5248 524c 5250 5253 5255 5259
525d 5261 5264 5268 526c 526e 526f 5274
1 5278 527c 5280 5283 5287 528b 528f
5293 5296 529a 529e 52a1 52a4 52a9 52ac
52af 52b2 52b3 52b8 52b9 1 52bb 52c0
52c4 52c8 52cb 52d0 52d5 52d8 52dc 52dd
52e2 52e5 52ea 52eb 52f0 52f3 52f8 52f9
52fe 5301 5305 5306 530b 530e 5313 5314
5319 531c 5320 5324 5327 532a 532b 532d
532e 5333 5334 5339 533b 533f 5342 5344
5348 534a 5356 5358 535a 535b 5360 5364
5366 5372 5374 5376 537a 537e 5382 5386
538a 538e 5392 5396 539a 539e 53a2 53ae
53b2 53b7 53bb 53be 53bf 53c4 53c8 53cc
53cf 53d2 53d6 53d8 53dc 53e0 53e3 53e6
53ea 53ec 53f0 53f4 53f7 53f9 53fd 5401
5405 5408 540c 5410 5412 5413 5418 1
541c 5420 5424 5427 542b 542f 5433 5437
543a 543e 5442 5445 5448 544d 5450 5453
5456 5457 545c 545d 1 545f 5464 5468
546c 546f 5474 5479 547c 5480 5481 5486
5489 548e 548f 5494 5497 549c 549d 54a2
54a5 54a9 54aa 54af 54b2 54b7 54b8 54bd
54c0 54c4 54c8 54cb 54ce 54cf 54d1 54d2
54d7 54d8 54dd 54df 54e3 54e6 54e8 54ec
54ee 54fa 54fc 54fe 54ff 5504 5508 550a
5516 5518 551a 551e 5522 5525 5527 552b
552f 5533 5537 553b 553f 5543 5547 554b
554f 5553 555f 5563 5568 556c 556f 5570
5575 5579 557d 5580 5583 5587 5589 558d
5591 5594 5597 559b 559d 55a1 55a5 55a8
55aa 55ae 55b2 55b6 55b9 55bd 55c1 55c3
55c4 55c9 1 55cd 55d1 55d5 55d8 55dc
55e0 55e4 55e8 55eb 55ef 55f3 55f6 55f9
55fe 5601 5604 5607 5608 560d 560e 1
5610 5615 5619 561d 5620 5625 562a 562d
5631 5632 5637 563a 563f 5640 5645 5648
564d 564e 5653 5656 565a 565b 5660 5663
5668 5669 566e 5671 5675 5679 567c 567f
5680 5682 5683 5688 5689 568e 5690 5694
5697 5699 569d 569f 56ab 56ad 56af 56b0
56b5 56b9 56bb 56c7 56c9 56cb 56cf 56d3
56d6 56d8 56dc 56e0 56e3 56e5 56e9 56ed
56f0 56f4 56f7 56f8 56fd 5700 5704 5708
570b 570c 570e 5711 5714 5715 571a 1
571d 5722 5725 5729 572d 5730 5731 5733
5736 5739 573a 573f 1 5742 5747 574b
5750 5754 5758 5759 575d 5761 5764 5768
576c 576f 5773 5777 577b 577e 577f 5781
5784 5787 5788 578d 5791 5795 5799 579d
57a1 57a5 57a9 57ad 57b1 57b5 57b9 57bd
57c1 57c5 57c9 57cd 57d1 57d5 57d9 57dd
57e1 57e5 57e9 57ed 57f1 57fd 5801 5805
5808 580c 5810 5812 5816 581a 581e 5821
5825 5829 582b 582c 5831 1 5835 5839
583d 5840 5844 5848 584c 5850 5853 5857
585b 585e 5861 5866 5869 586c 586f 5870
5875 5876 1 5878 587d 5881 5885 5888
588d 5892 5895 5899 589a 589f 58a2 58a7
58a8 58ad 58b0 58b5 58b6 58bb 58be 58c2
58c3 58c8 58cb 58d0 58d1 58d6 58d9 58dd
58e1 58e4 58e7 58e8 58ea 58eb 58f0 58f1
58f6 58f8 58fc 58ff 5901 5905 5907 5913
5915 5917 5918 591d 5921 5923 592f 5931
5933 5937 593b 593f 5943 5947 594b 594f
5953 5957 595b 595f 5963 5967 596b 596f
5973 5977 597b 597f 5983 5987 598b 598f
5993 5997 599b 59a7 59ab 59af 59b2 59b6
59ba 59bc 59c0 59c4 59c8 59cb 59cf 59d3
59d5 59d6 59db 1 59df 59e3 59e7 59ea
59ee 59f2 59f6 59fa 59fd 5a01 5a05 5a08
5a0b 5a10 5a13 5a16 5a19 5a1a 5a1f 5a20
1 5a22 5a27 5a2b 5a2f 5a32 5a37 5a3c
5a3f 5a43 5a44 5a49 5a4c 5a51 5a52 5a57
5a5a 5a5f 5a60 5a65 5a68 5a6c 5a6d 5a72
5a75 5a7a 5a7b 5a80 5a83 5a87 5a8b 5a8e
5a91 5a92 5a94 5a95 5a9a 5a9b 5aa0 5aa2
5aa6 5aa9 5aab 5aaf 5ab1 5abd 5abf 5ac1
5ac2 5ac7 5acb 5acd 5ad9 5adb 5add 5ae1
5ae5 5ae8 5aec 5af0 5af3 5af7 5afa 5afe
5b01 5b02 5b07 5b0a 5b0e 5b12 5b15 5b16
5b18 5b1c 5b1f 5b23 5b26 5b27 5b29 5b2a
5b2f 5b32 5b36 5b3a 5b3d 5b3e 5b40 5b43
5b46 5b47 5b4c 1 5b4f 5b54 5b58 5b5c
5b5f 5b60 5b62 5b65 5b68 5b69 5b6e 1
5b71 5b76 5b7a 5b7e 5b82 5b86 5b8a 5b8e
5b92 5b96 5ba2 5ba6 5bab 5baf 5bb2 5bb3
5bb8 5bbc 5bc0 5bc3 5bc6 5bca 5bcc 5bd0
5bd4 5bd7 5bda 5bde 5be0 5be4 5be8 5beb
5bed 5bf1 5bf5 5bf9 5bfc 5c00 5c04 5c06
5c07 5c0c 1 5c10 5c14 5c18 5c1b 5c1f
5c23 5c27 5c2b 5c2e 5c32 5c36 5c39 5c3c
5c41 5c44 5c47 5c4a 5c4b 5c50 5c51 1
5c53 5c58 5c5c 5c60 5c63 5c68 5c6d 5c70
5c74 5c75 5c7a 5c7d 5c82 5c83 5c88 5c8b
5c90 5c91 5c96 5c99 5c9d 5c9e 5ca3 5ca6
5cab 5cac 5cb1 5cb4 5cb8 5cbc 5cbf 5cc2
5cc3 5cc5 5cc6 5ccb 5ccc 5cd1 5cd3 5cd7
5cda 5cdc 5ce0 5ce2 5cee 5cf0 5cf2 5cf3
5cf8 5cfc 5cfe 5d0a 5d0c 5d10 5d14 5d18
5d1c 5d20 5d24 5d28 5d2c 5d30 5d34 5d38
5d3c 5d40 5d4c 5d50 5d55 5d59 5d5c 5d5d
5d62 5d66 5d6a 5d6d 5d70 5d74 5d76 5d7a
5d7e 5d81 5d84 5d88 5d8a 5d8e 5d92 5d95
5d97 5d9b 5d9f 5da3 5da6 5daa 5dae 5db0
5db1 5db6 1 5dba 5dbe 5dc2 5dc5 5dc9
5dcd 5dd1 5dd5 5dd8 5ddc 5de0 5de3 5de6
5deb 5dee 5df1 5df4 5df5 5dfa 5dfb 1
5dfd 5e02 5e06 5e0a 5e0d 5e12 5e17 5e1a
5e1e 5e1f 5e24 5e27 5e2c 5e2d 5e32 5e35
5e3a 5e3b 5e40 5e43 5e47 5e48 5e4d 5e50
5e55 5e56 5e5b 5e5e 5e62 5e66 5e69 5e6c
5e6d 5e6f 5e70 5e75 5e76 5e7b 5e7d 5e81
5e84 5e86 5e8a 5e8c 5e98 5e9a 5e9c 5e9d
5ea2 5ea6 5ea8 5eb4 5eb6 5eb8 5ebc 5ec0
5ec5 5ec9 5eca 5ece 5ed2 5ed6 5eda 5ede
5ee3 5ee8 5eec 5ef0 5ef4 5ef9 5efd 5f01
5f04 5f07 5f0b 5f0d 5f11 5f15 5f18 5f1b
5f1f 5f21 5f25 5f29 5f2c 5f30 5f34 5f37
5f3a 5f3d 5f3e 5f43 5f47 5f4d 5f4f 5f53
5f56 5f5a 5f5e 5f62 5f65 5f66 5f68 5f6b
5f6f 5f73 5f76 5f77 5f79 5f7a 5f7f 5f83
5f87 5f8b 5f8e 5f8f 5f91 5f94 5f97 5f98
5f9d 5fa1 5fa5 5fa8 5fa9 5fab 5faf 5fb2
5fb6 5fb9 5fba 5fbc 5fbd 5fc2 5fc6 5fca
5fcd 5fce 5fd0 5fd4 5fd7 5fdb 5fde 5fdf
5fe1 5fe2 5fe7 5feb 5fef 5ff3 5ff6 5ff7
5ff9 5ffd 5fff 6003 6007 600b 600e 600f
6011 6014 6018 601c 601f 6020 6022 6023
6028 602b 602f 6033 6036 6037 6039 603a
603f 6043 6045 6049 604d 6050 6052 6056
6059 605d 6061 6065 6069 606d 6079 607d
6082 6086 6089 608a 608f 6093 6097 609a
609d 60a1 60a3 60a7 60ab 60ae 60b1 60b5
60b7 60bb 60bf 60c2 60c4 60c8 60cc 60d0
60d3 60d7 60db 60dd 60de 60e3 1 60e7
60eb 60ef 60f2 60f6 60fa 60fe 6102 6105
6109 610d 6110 6113 6118 611b 611e 6121
6122 6127 6128 1 612a 612f 6133 6137
613a 613f 6144 6147 614b 614c 6151 6154
6159 615a 615f 6162 6167 6168 616d 6170
6174 6175 617a 617d 6182 6183 6188 618b
618f 6193 6196 6199 619a 619c 619d 61a2
61a3 61a8 61aa 61ae 61b1 61b3 61b7 61b9
61c5 61c7 61c9 61ca 61cf 61d3 61d5 61e1
61e3 61e7 61eb 61ef 61f1 61f5 61f8 61fc
6200 6203 6204 6206 620a 620d 6211 6214
6215 6217 6218 621d 6221 6227 6229 622d
6230 6232 6236 623d 6241 6245 624a 624c
6250 6254 6257 625b 625f 6261 6265 6269
626d 6270 6274 6278 627a 627b 6280 1
6284 6288 628c 628f 6293 6297 629b 629f
62a2 62a6 62aa 62ad 62b0 62b5 62b8 62bb
62be 62bf 62c4 62c5 1 62c7 62cc 62d0
62d4 62d7 62dc 62e1 62e4 62e8 62e9 62ee
62f1 62f6 62f7 62fc 62ff 6304 6305 630a
630d 6311 6312 6317 631a 631f 6320 6325
6328 632c 6330 6333 6336 6337 6339 633a
633f 6340 6345 6347 634b 634e 6350 6354
6356 6362 6364 6366 6367 636c 6370 6372
637e 6380 6384 6388 638b 638c 638e 6391
6394 6395 639a 639d 63a1 63a4 63a5 63aa
63ad 1 63b0 63b5 63b9 63bd 63c1 63c5
63c9 63d5 63d9 63de 63e2 63e5 63e6 63eb
63ef 63f3 63f6 63f9 63fd 63ff 6403 6407
640a 640d 6411 6413 6417 641b 641e 6420
6424 6428 642c 642f 6433 6437 6439 643a
643f 1 6443 6447 644b 644e 6452 6456
645a 645e 6461 6465 6469 646c 646f 6474
6477 647a 647d 647e 6483 6484 1 6486
648b 648f 6493 6496 649b 64a0 64a3 64a7
64a8 64ad 64b0 64b5 64b6 64bb 64be 64c3
64c4 64c9 64cc 64d0 64d1 64d6 64d9 64de
64df 64e4 64e7 64eb 64ef 64f2 64f5 64f6
64f8 64f9 64fe 64ff 6504 6506 650a 650d
650f 6513 6515 6521 6523 6525 6526 652b
652f 6531 653d 653f 6541 6545 6548 654a
654e 6552 6555 6559 655d 6560 6561 6563
6566 6569 656a 656f 6573 6577 657b 657e
657f 6581 6584 6585 6587 658a 658d 6590
6591 6596 659a 659e 65a2 65a6 65a7 65a9
65ad 65af 65b3 65b7 65bb 65bf 65c0 65c2
65c6 65c8 65cc 65d0 65d3 65d5 65d9 65dc
65de 65e2 65e5 65e7 65eb 65ee 65f0 65f4
65f7 65fb 65ff 6602 6605 6608 6609 660e
6612 6616 6619 661c 6620 6622 6626 6629
662b 662f 6632 6636 663a 663d 6641 6645
6648 664b 6650 6653 6656 6659 665a 665f
6660 1 6662 6667 666b 666f 6672 6677
667c 667f 6683 6684 6689 668c 6691 6692
6697 669a 669f 66a0 66a5 66a8 66ac 66b0
66b4 66b8 66bb 66bc 66be 66c3 66c4 66c6
66c7 66cc 66cf 66d4 66d5 66da 66dd 66e2
66e3 66e8 66eb 66ef 66f0 66f5 66f8 66fd
66fe 6703 6704 6709 670b 670f 6712 6716
671a 671d 6721 6725 6728 672c 6730 6731
6736 6738 673c 6740 6743 6747 674b 674e
6752 6756 6759 675c 6761 6764 6767 676a
676b 6770 6771 1 6773 6778 677c 6780
6783 6788 678d 6790 6794 6795 679a 679b
67a0 67a4 67a8 67ab 67b0 67b5 67b8 67bc
67c0 67c3 67c4 67c9 67cc 67d1 67d2 67d7
67da 67de 67e2 67e5 67e6 67eb 67ec 67f1
67f3 67f7 67fa 67fe 6802 6806 680a 680e
6812 6816 681a 681e 682a 682f 6833 6837
683b 683f 6841 6842 6847 1 684b 684f
6853 6856 685a 685e 6862 6866 6869 686d
6871 6874 6877 687c 687f 6882 6885 6886
688b 688c 1 688e 6893 6897 689b 689e
68a3 68a8 68ab 68af 68b0 68b5 68b6 68bb
68bf 68c3 68c6 68cb 68d0 68d3 68d7 68d8
68dd 68e0 68e5 68e6 68eb 68ee 68f2 68f6
68f9 68fc 68fd 68ff 6900 6905 6906 690b
690d 6911 6914 6918 691c 6920 6924 6928
692c 6930 693c 6941 6945 6949 694d 6951
6953 6957 6959 6965 6967 6969 696a 696f
6973 6975 6981 6985 6987 6988 6991 
1e0f
2
0 1 a 6 9 :3 6 12 :3 6
11 :3 6 12 :3 6 13 :3 6 10 :3 6
17 :3 6 15 :3 6 1a :3 6 b :3 6
10 :3 6 c :3 6 16 :3 6 18 :3 6
19 :3 6 1b :3 6 17 :3 6 16 :3 6
17 :3 6 17 :3 6 12 :2 6 1 3
a 13 a :2 1 4 13 1c 1b
:2 13 :2 4 10 19 18 :2 10 :2 4 e
17 16 :2 e :2 4 e 17 16 :2 e
:2 4 f 18 17 :2 f :2 4 e 17
16 :2 e :2 4 f 18 17 :2 f :2 4
11 1a 19 :2 11 :2 4 f 18 17
:2 f :2 4 c 15 14 :2 c :2 4 12
1b 1a :2 12 :2 4 11 1a 19 :2 11
:2 4 e 17 16 :2 e :2 4 f 18
17 :2 f :2 4 :2 e 1d e :2 4 :2 14
23 14 :2 4 :2 b 1a b :2 4 :2 a
19 a :2 4 :2 b 1a b :2 4 :2 e
1d e :2 4 :2 13 22 13 :2 4 :2 c
1b c :2 4 :2 e 1d e :2 4 :2 13
22 13 :2 4 :2 d 1c d :2 4 :2 c
1b c :2 4 :2 c 1b c :2 4 :2 e
1d e :2 4 :2 10 1f 10 :2 4 :2 12
21 12 :2 4 :3 10 :2 4 :3 f :2 4 18
21 20 :2 18 :2 4 18 21 20 :2 18
:2 4 f 18 17 :2 f :2 4 :3 11 :2 4
:3 f :2 4 :3 13 :2 4 :3 10 :2 4 :3 d :2 4
:3 e :2 4 14 1d 1c :2 14 :2 4 :2 11
20 11 :2 4 :2 f 1e f :2 4 :2 14
23 14 :2 4 :2 11 20 11 :2 4 :2 12
21 12 :2 4 :2 12 21 12 :2 4 :2 a
19 a :2 4 :3 9 :2 4 :3 9 :2 4 :2 f
19 f :2 4 :2 f 19 f :2 4 :2 f
1e f :2 4 :2 16 25 16 :2 4 :2 11
20 11 :2 4 :2 14 23 14 4 7
e 0 :2 7 20 2d 33 3b 56
5c 64 f 10 1d 2e 3b 13
2d a :6 7 e 0 :2 7 11 1c
26 f 10 1d 2e 42 18 23
a :5 7 4 e 17 :3 e 4 7
:2 11 23 :2 2d 3a :2 23 :3 7 :2 d 19
25 :2 2f 43 :2 25 55 5b 5e :2 55
6c 83 8 :2 6c d 10 :2 6c 13
16 :2 6c b e :2 6c 13 16 1f
27 :2 1f 35 :2 16 :2 6c b e :2 6c
13 16 :2 6c 1b 1e :2 6c 29 2c
:2 6c b e :2 6c 13 16 :2 6c 1b
1e :2 6c 2a 2d :2 6c b e :2 6c
13 16 :2 6c 1b 1e :2 6c 2b 2e
:2 6c b e :2 6c 13 16 1f 27
:2 1f 33 :2 16 :2 6c b e :2 6c 13
16 :2 6c 1b 1e :2 6c 2f 32 :2 6c
b e :2 6c 13 16 1f 27 :2 1f
38 :2 16 :2 6c b e :2 6c 13 16
1f 27 :2 1f 3d :2 16 :2 6c b e
:2 6c 13 16 1f 27 :2 1f 2e :2 16
:2 6c b e :2 6c 13 16 1f 27
:2 1f 33 :2 16 :2 6c b e :2 6c 13
16 1f 27 :2 1f 2f :2 16 :2 6c b
e :2 6c 13 16 1f 27 :2 1f 39
:2 16 :2 6c b e :2 6c 13 16 :2 6c
1b 1e :2 6c 30 33 :2 6c b e
:2 6c 13 16 :2 6c 1b 1e :2 6c 31
34 :2 6c b e :2 6c 13 16 1f
27 :2 1f 3e :2 16 :2 6c b e :2 6c
13 16 1f 27 :2 1f 3a :2 16 :2 6c
b e :2 6c 13 16 :2 6c 1b 1e
:2 6c 2e 31 :2 6c b e :2 6c 13
16 :2 6c 1b 1e :2 6c 2f 32 :2 6c
b e :2 6c 13 16 1f 27 :2 1f
3a :2 16 :2 6c b e :2 6c 13 16
:2 6c 1b 1e :2 6c 2a 2d :2 6c 32
35 :2 6c :2 7 4b :2 4 7 :2 11 23
:2 2d 39 3c 4d 50 51 :2 4d :2 23
:3 7 :2 d 12 1e 5e 61 :2 1e :2 7
54 :2 4 :4 9 8 7 24 2d 3a
:2 24 3d 3f :2 3d 23 46 a 17
:2 46 1a 1c :2 1a 45 :2 23 22 :2 7
a :2 14 26 :2 30 3c 3f 50 53
54 :2 50 :2 26 :3 a :2 10 15 21 61
64 :2 21 71 74 :2 21 78 7b :2 21
85 88 91 99 :2 a2 :2 91 b3 :2 88
:2 21 :2 a 57 :2 7 a :2 14 26 :2 30
3c 3f 50 53 54 :2 50 :2 26 :3 a
:2 10 15 21 61 64 :2 21 :2 a 57
:3 7 :2 12 28 :2 31 41 :2 7 20 :2 4
:4 9 8 7 :4 1b 1a 19 :2 7 :4 34
33 32 :2 7 a :2 14 26 :2 30 3c
3f 50 53 54 :2 50 :2 26 :3 a :2 10
15 21 61 64 :2 21 71 74 :2 21
78 7b :2 21 85 88 91 99 :2 a2
:2 91 b3 :2 88 :2 21 :2 a 57 :2 7 a
:2 14 26 :2 30 3c 3f 50 53 54
:2 50 :2 26 :3 a :2 10 15 21 61 64
:2 21 :2 a 57 :3 7 :2 12 28 :2 31 41
:2 7 4a :3 4 16 :2 4 10 :2 4 17
:2 4 12 :2 4 13 :2 4 17 :2 4 12
:2 4 10 :2 4 13 :2 4 15 :2 4 f
:2 4 16 :2 4 15 :2 4 14 4 7
10 17 :2 10 29 :2 7 2e 2c :2 37
:2 2c b 14 23 :2 b 26 28 :2 26
a 2f 38 47 :2 2f 4a 4c :2 4a
2e :2 a :4 e d 10 :2 1a 2c :2 36
42 45 56 59 5a :2 56 :2 2c :3 10
:2 16 1b 27 67 6a :2 27 77 7a
:2 27 7e 81 :2 27 8b 8e 97 9f
:2 a8 :2 97 b9 :2 8e :2 27 :2 10 5d :2 d
10 :2 1a 2c :2 36 42 45 56 59
5a :2 56 :2 2c :3 10 :2 16 1b 27 67
6a :2 27 :2 10 5d :3 d :2 18 2e :2 37
47 :2 d 22 :3 a 1b :2 a 17 :2 a
18 a 14 20 2d 3b 12 20
2d :2 d :2 16 22 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
5a 5d 5e :2 5a :2 2f :3 13 :2 19 1e
2a 6a 6d :2 2a 7a 7d :2 2a 81
84 :2 2a 90 93 :2 2a 9b 9e :2 2a
aa ad b4 bd c0 :2 ad :2 2a :2 13
61 :2 10 :2 d :4 19 :2 d a :3 4f a
:3 15 :2 a 1a 23 2c 37 :2 23 :2 1a
:2 a :2 13 1f 2e 3c 47 52 61
27 :2 1f a e :2 17 20 22 :2 20
d 2a 33 42 :2 2a 45 47 :2 45
29 :2 d 4f 58 62 :2 4f 65 67
:2 65 4e :3 d 1b d 17 27 37
2e 16 23 34 41 :2 10 :2 19 25
10 d 15 13 :2 1c 28 13 23
:2 10 :2 15 13 :2 1c 28 13 16 :2 20
32 :2 3c 48 4c 5d 60 61 :2 5d
:2 32 :3 16 :2 1c 21 2d 6d 70 :2 2d
7d 80 :2 2d 84 87 :2 2d 93 96
:2 2d 9e a1 :2 2d ad b0 b7 c0
c3 :2 b0 :2 2d :2 16 64 :2 13 :2 10 :4 1c
:2 10 d :3 6a 10 19 25 :2 10 28
2b :2 28 10 21 :2 10 1c :2 10 23
10 :4 14 13 1d 2a 35 42 63
74 :2 16 :2 1f 2b 16 13 1b 19
:2 22 2e 19 29 :2 16 :2 1b 19 :2 22
2e 19 1c :2 26 38 :2 42 4e 52
63 66 67 :2 63 :2 38 :3 1c :2 22 27
33 73 76 :2 33 83 86 :2 33 8a
8d :2 33 99 9c :2 33 a4 a7 :2 33
b3 b6 bd c6 c9 :2 b6 :2 33 :2 1c
6a :2 19 :2 16 :4 22 :2 16 13 :4 2c :2 10
2d :2 d 6a :2 a e 17 26 :2 e
29 2b :2 29 d 33 3e 47 52
:2 3e :2 33 32 57 5a :2 57 :3 d 1e
d 5c :2 a 4f a 15 a 14
20 29 36 43 d 18 22 2d
3b 12 20 2b 3a 49 d 1e
26 31 41 12 29 3d :2 d :2 16
22 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 5a 5d 5e
:2 5a :2 2f :3 13 :2 19 1e 2a 6a 6d
:2 2a 7a 7d :2 2a 81 84 :2 2a 90
93 :2 2a 9b 9e :2 2a aa ad b4
bd c0 :2 ad :2 2a :2 13 61 :2 10 :2 d
:4 19 :2 d a :3 7 e :2 17 22 20
:2 2b :2 20 d 40 49 59 :2 40 5f
5c 10 1f :2 5f :2 5c 3f :3 d 1f
:2 d 18 19 :2 18 d 23 :2 a :4 7
a 13 22 :2 a 25 28 :2 25 d
:2 16 1f 22 :2 1f 10 :2 1a 2c :2 36
42 46 57 5a 5b :2 57 :2 2c :3 10
:2 16 1b 27 67 6a :2 27 77 7a
:2 27 7e 81 :2 27 8b 8e 97 9f
:2 a8 :2 97 b3 :2 8e :2 27 bb be :2 27
c6 c9 :2 27 ce d1 :2 27 d9 dc
:2 27 :2 10 5e :2 d 10 :2 1a 2c :2 36
42 45 56 59 5a :2 56 :2 2c :3 10
:2 16 1b 27 67 6a :2 27 :2 10 5d
:3 d :2 18 2c :2 35 3f 48 :2 d 24
:2 a :4 e d :4 26 25 :2 d :4 3f 3e
:2 d :4 14 13 :2 d 10 :2 1a 2c :2 36
42 45 56 59 5a :2 56 :2 2c :3 10
:2 16 1b 27 67 6a :2 27 77 7a
:2 27 7e 81 :2 27 8b 8e 97 9f
:2 a8 :2 97 b9 :2 8e :2 27 :2 10 5d :2 d
10 :2 1a 2c :2 36 42 45 56 59
5a :2 56 :2 2c :3 10 :2 16 1b 27 67
6a :2 27 :2 10 5d :3 d :2 18 2e :2 37
47 :2 d 2d :3 a 1c :2 25 a d
16 1f :2 d 22 24 :2 22 d 18
d 2e :2 a 14 1d 37 46 54
61 :2 d :2 16 22 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
5a 5d 5e :2 5a :2 2f :3 13 :2 19 1e
2a 6a 6d :2 2a 7a 7d :2 2a 81
84 :2 2a 90 93 :2 2a 9b 9e :2 2a
aa ad b4 bd c0 :2 ad :2 2a :2 13
61 :2 10 :2 d :4 19 :2 d a :3 2a d
16 1e :2 d 21 23 :2 21 d 1b
23 2c 36 :2 2c 3f :2 23 42 44
4d 57 :2 4d 63 :2 44 :2 23 22 67
69 :2 22 21 20 22 :2 21 :2 1b d
10 19 22 :2 10 28 25 31 3d
:2 28 :2 25 10 1a :2 10 1d 10 40
10 1b 10 :4 d 25 :3 a 18 a
e 15 1e 29 :2 15 2d :2 e d
32 34 :2 32 10 19 28 :2 10 2b
2d :2 2b 13 1c 2c :2 13 2f 31
:2 2f 13 :2 1c 28 35 3e 4a 56
67 25 32 3c 44 4c 54 25
35 40 50 25 36 :2 28 13 16
:2 1f 28 2a :2 28 20 35 3b 19
2a 36 48 55 65 1f 2c 3d
47 55 63 19 20 1c 28 :3 26
1c :2 25 31 1c 2d 1c :2 25 31
1c :4 19 16 1e 1c :2 25 31 1c
2c :2 19 :2 1e 1c :2 25 31 1c 1f
:2 29 3b :2 45 51 55 66 69 6a
:2 66 :2 3b :3 1f :2 25 2a 36 76 79
:2 36 86 89 :2 36 8d 90 :2 36 9c
9f :2 36 a7 aa :2 36 b6 b9 c0
c9 cc :2 b9 :2 36 :2 1f 6d :2 1c :2 19
:4 25 :2 19 16 :4 2c :2 13 33 13 :2 1c
28 35 3e 4a 56 67 25 32
3c 44 4c 54 25 35 40 50
25 36 :2 28 13 :4 10 2f 10 22
2b 3b :2 22 3e 40 :2 22 10 13
1c 2c :2 13 2f 31 :2 2f 13 :2 1c
28 35 3e 4a 56 67 25 32
3c 44 4c 54 25 35 40 50
25 36 :2 28 13 16 :2 1f 28 2a
:2 28 20 35 3b 19 2a 36 48
55 65 1f 33 19 20 1c 28
:3 26 1c :2 25 31 1c 2d 1c :2 25
31 1c :4 19 16 1e 1c :2 25 31
1c 2c :2 19 :2 1e 1c :2 25 31 1c
1f :2 29 3b :2 45 51 55 66 69
6a :2 66 :2 3b :3 1f :2 25 2a 36 76
79 :2 36 86 89 :2 36 8d 90 :2 36
9c 9f :2 36 a7 aa :2 36 b6 b9
c0 c9 cc :2 b9 :2 36 :2 1f 6d :2 1c
:2 19 :4 25 :2 19 16 :4 2c :2 13 33 13
:2 1c 28 35 3e 4a 56 67 25
32 3c 44 4c 54 25 35 40
50 25 36 :2 28 13 :4 10 :4 d 11
:2 1a 23 25 :2 23 10 2e 35 3e
49 :2 35 4d :2 2e 2d 51 53 :2 51
2c :2 10 14 1d 2d :2 14 32 30
3b 46 :2 32 :2 30 13 4f 58 5e
:2 4f 61 63 :2 61 4e :2 13 6b 2c
41 :2 6b 44 46 :2 44 6a :2 13 1d
32 46 16 22 2b 3a 16 27
33 45 52 62 16 25 31 41
16 24 16 :2 1c 30 16 1d 19
25 :3 23 19 :2 22 2e 19 2a 19
:2 22 2e 19 :4 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 63
66 67 :2 63 :2 38 :3 1c :2 22 27 33
73 76 :2 33 83 86 :2 33 8a 8d
:2 33 99 9c :2 33 a4 a7 :2 33 b3
b6 bd c6 c9 :2 b6 :2 33 :2 1c 6a
:2 19 :2 16 :4 22 :2 16 13 :3 4d 1d 32
3f 4a 1c 29 3a 4e 16 1d
19 25 :3 23 19 :2 22 2e 19 2a
19 :2 22 2e 19 :4 16 13 1b 19
:2 22 2e 19 29 :2 16 :2 1b 19 :2 22
2e 19 1c :2 26 38 :2 42 4e 52
63 66 67 :2 63 :2 38 :3 1c :2 22 27
33 73 76 :2 33 83 86 :2 33 8a
8d :2 33 99 9c :2 33 a4 a7 :2 33
b3 b6 bd c6 c9 :2 b6 :2 33 :2 1c
6a :2 19 :2 16 :4 22 :2 16 13 :3 4d 1d
32 43 1c 30 16 1d 19 25
:3 23 19 :2 22 2e 19 2a 19 :2 22
2e 19 :4 16 13 1b 19 :2 22 2e
19 29 :2 16 :2 1b 19 :2 22 2e 19
1c :2 26 38 :2 42 4e 52 63 66
67 :2 63 :2 38 :3 1c :2 22 27 33 73
76 :2 33 83 86 :2 33 8a 8d :2 33
99 9c :2 33 a4 a7 :2 33 b3 b6
bd c6 c9 :2 b6 :2 33 :2 1c 6a :2 19
:2 16 :4 22 :2 16 13 :4 4d 1d 32 38
16 27 33 45 52 62 1c 30
16 1d 19 25 :3 23 19 :2 22 2e
19 2a 19 :2 22 2e 19 :4 16 13
1b 19 :2 22 2e 19 29 :2 16 :2 1b
19 :2 22 2e 19 1c :2 26 38 :2 42
4e 52 63 66 67 :2 63 :2 38 :3 1c
:2 22 27 33 73 76 :2 33 83 86
:2 33 8a 8d :2 33 99 9c :2 33 a4
a7 :2 33 b3 b6 bd c6 c9 :2 b6
:2 33 :2 1c 6a :2 19 :2 16 :4 22 :2 16 13
:7 10 56 :2 d 37 d 1c 23 2c
39 :2 23 3d :2 46 :2 1c d 10 19
28 :2 10 2b 2d :2 2b 10 :2 19 25
32 3b 47 53 64 25 32 3c
44 4c 54 25 35 40 50 25
36 :2 25 10 14 :2 1d 26 28 :2 26
13 30 39 44 :2 30 47 49 :2 47
2f :2 13 51 5a 63 :2 51 66 68
:2 66 50 :2 13 1d 32 38 16 27
33 45 52 62 1c 29 3a 44
52 60 16 1d 19 25 :3 23 19
:2 22 2e 19 2a 19 :2 22 2e 19
:4 16 13 1b 19 :2 22 2e 19 29
:2 16 :2 1b 19 :2 22 2e 19 1c :2 26
38 :2 42 4e 52 63 66 67 :2 63
:2 38 :3 1c :2 22 27 33 73 76 :2 33
83 86 :2 33 8a 8d :2 33 99 9c
:2 33 a4 a7 :2 33 b3 b6 bd c6
c9 :2 b6 :2 33 :2 1c 6a :2 19 :2 16 :4 22
:2 16 13 :4 6b :2 10 2f :4 14 :2 13 20
13 1d 24 3e 45 5f 66 80
87 9c ab :2 16 :2 1f 2b 16 13
1b 19 :2 22 2e 19 29 :2 16 :2 1b
19 :2 22 2e 19 1c :2 26 38 :2 42
4e 52 63 66 67 :2 63 :2 38 :3 1c
:2 22 27 33 73 76 :2 33 83 86
:2 33 8a 8d :2 33 99 9c :2 33 a4
a7 :2 33 b3 b6 bd c6 c9 :2 b6
:2 33 :2 1c 6a :2 19 :2 16 :4 22 :2 16 13
:3 33 16 :2 21 29 34 3e :2 16 42
44 :2 42 16 23 :2 2c :2 16 26 16
20 30 1e 1f :2 19 :2 22 2e 19
16 1e 1c :2 25 31 1c 2c :2 19
:2 1e 1c :2 25 31 1c 1f :2 29 3b
:2 45 51 55 66 69 6a :2 66 :2 3b
:3 1f :2 25 2a 36 76 79 :2 36 86
89 :2 36 8d 90 :2 36 9c 9f :2 36
a7 aa :2 36 b6 b9 c0 c9 cc
:2 b9 :2 36 :2 1f 6d :2 1c :2 19 :4 25 :2 19
16 :3 46 16 29 3b 49 54 62
6d 28 :2 29 16 19 22 33 :2 19
36 38 :2 36 19 26 :2 19 27 :2 19
2b :2 19 27 :2 19 28 :2 19 1e :3 19
1f 2c 3c :3 19 1c 24 :2 1c :2 25
31 1c 2a 1c :2 25 31 1c :4 19
1c 22 :3 1c :2 25 31 1c 19 21
1f :2 28 34 1f 2f :2 1c :2 21 1f
:2 28 34 1f 22 :2 2c 3e :2 48 54
58 69 6c 6d :2 69 :2 3e :3 22 :2 28
2d 39 79 7c :2 39 89 8c :2 39
90 93 :2 39 9f a2 :2 39 aa ad
:2 39 b9 bc c3 cc cf :2 bc :2 39
:2 22 70 :2 1f :2 1c :4 28 :2 1c 19 :3 3a
23 32 43 22 30 :2 1c :2 25 31
1c 19 21 1f :2 28 34 1f 2f
:2 1c :2 21 1f :2 28 34 1f 22 :2 2c
3e :2 48 54 58 69 6c 6d :2 69
:2 3e :3 22 :2 28 2d 39 79 7c :2 39
89 8c :2 39 90 93 :2 39 9f a2
:2 39 aa ad :2 39 b9 bc c3 cc
cf :2 bc :2 39 :2 22 70 :2 1f :2 1c :4 28
:2 1c 19 :3 3a 19 26 :2 19 26 :2 19
26 2d 37 3a :2 2d 47 4a :2 26
:2 19 26 2d 37 3a :2 2d 4b 4e
:2 26 :2 19 26 2d 37 3a :2 2d 4a
4d :2 26 :2 19 26 2d 37 22 :2 2d
24 27 :2 26 :2 19 2c 3a 46 52
5a 62 6e 24 2f 3a 46 52
5e 24 2f 3b :2 2c 19 1c :2 26
38 :2 42 4e 52 63 66 67 :2 63
:2 38 :3 1c :2 22 27 33 73 76 :2 33
83 86 :2 33 8a 8d :2 33 97 9a
a3 ab :2 a3 af :2 9a :2 33 b7 ba
:2 33 c2 c5 :2 33 ca cd :2 33 d5
d8 :2 33 :2 1c 6a :2 19 1c :2 26 38
:2 42 4e 51 62 65 66 :2 62 :2 38
:3 1c :2 22 27 33 73 76 :2 33 :2 1c
69 :3 19 :2 24 38 3b 44 :2 19 3a
:2 16 46 :2 13 33 :3 10 22 2b 3b
:2 22 3e 40 :2 22 :2 10 :2 19 25 32
3b 47 53 64 25 32 3c 44
4c 54 25 35 40 50 25 36
:2 25 10 14 :2 1d 26 28 :2 26 13
30 39 44 :2 30 47 49 :2 47 2f
:2 13 51 5a 63 :2 51 66 68 :2 66
50 :2 13 1d 32 38 16 27 33
45 52 62 1c 30 16 1d 19
25 :3 23 19 :2 22 2e 19 2a 19
:2 22 2e 19 :4 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 63
66 67 :2 63 :2 38 :3 1c :2 22 27 33
73 76 :2 33 83 86 :2 33 8a 8d
:2 33 99 9c :2 33 a4 a7 :2 33 b3
b6 bd c6 c9 :2 b6 :2 33 :2 1c 6a
:2 19 :2 16 :4 22 :2 16 13 :4 6b :2 10 :4 d
:4 a d :2 16 1f 22 :2 1f 10 :2 1a
2c :2 36 42 46 57 5a 5b :2 57
:2 2c :3 10 :2 16 1b 27 67 6a :2 27
77 7a :2 27 7e 81 :2 27 8b 8e
97 9f :2 a8 :2 97 b3 :2 8e :2 27 bb
be :2 27 c6 c9 :2 27 ce d1 :2 27
d9 dc :2 27 :2 10 5e :2 d 10 :2 1a
2c :2 36 42 45 56 59 5a :2 56
:2 2c :3 10 :2 16 1b 27 67 6a :2 27
:2 10 5d :3 d :2 18 2c :2 35 3f 48
:2 d 24 :3 a 1c a e 17 20
:2 e 23 26 :2 23 d 32 3b 4b
:2 32 4e 50 :2 4e 31 :3 d 1f d
53 :2 a d 16 25 :2 d 28 2a
:2 28 11 18 21 2c :2 18 30 :2 11
10 35 37 :2 35 :4 14 13 :4 17 16
20 35 49 19 25 2e 3d 19
2a 36 48 55 65 19 28 19
29 19 27 1f 33 19 20 1c
28 :3 26 1c :2 25 31 1c 2d 1c
:2 25 31 1c :4 19 16 1e 1c :2 25
31 1c 2c :2 19 :2 1e 1c :2 25 31
1c 1f :2 29 3b :2 45 51 55 66
69 6a :2 66 :2 3b :3 1f :2 25 2a 36
76 79 :2 36 86 89 :2 36 8d 90
:2 36 9c 9f :2 36 a7 aa :2 36 b6
b9 c0 c9 cc :2 b9 :2 36 :2 1f 6d
:2 1c :2 19 :4 25 :2 19 16 :4 33 20 35
49 19 25 19 28 1f 33 19
20 1c 28 :3 26 1c :2 25 31 1c
2d 1c :2 25 31 1c :4 19 16 1e
1c :2 25 31 1c 2c :2 19 :2 1e 1c
:2 25 31 1c 1f :2 29 3b :2 45 51
55 66 69 6a :2 66 :2 3b :3 1f :2 25
2a 36 76 79 :2 36 86 89 :2 36
8d 90 :2 36 9c 9f :2 36 a7 aa
:2 36 b6 b9 c0 c9 cc :2 b9 :2 36
:2 1f 6d :2 1c :2 19 :4 25 :2 19 16 :7 13
2f :4 17 16 20 35 49 19 25
2e 3d 19 35 51 5e 76 19
36 52 5f 78 19 38 54 6e
19 34 50 66 19 2b 19 2c
19 2e 19 2a 19 2b 19 2c
19 2e 19 2a 19 2a 38 49
57 68 76 87 :2 19 2a 36 48
55 65 19 28 19 29 19 27
1f 33 19 20 1c 28 :3 26 1c
:2 25 31 1c 2d 1c :2 25 31 1c
:4 19 16 1e 1c :2 25 31 1c 2c
:2 19 :2 1e 1c :2 25 31 1c 1f :2 29
3b :2 45 51 55 66 69 6a :2 66
:2 3b :3 1f :2 25 2a 36 76 79 :2 36
86 89 :2 36 8d 90 :2 36 9c 9f
:2 36 a7 aa :2 36 b6 b9 c0 c9
cc :2 b9 :2 36 :2 1f 6d :2 1c :2 19 :4 25
:2 19 16 :4 33 20 35 49 19 25
2e 3d 19 35 51 5e 76 19
36 52 5f 78 19 38 54 6e
19 34 50 66 19 2b 19 2c
19 2e 19 2a 19 2b 19 2c
19 2e 19 2a 19 2a 38 49
57 68 76 87 19 1f 33 19
20 1c 28 :3 26 1c :2 25 31 1c
2d 1c :2 25 31 1c :4 19 16 1e
1c :2 25 31 1c 2c :2 19 :2 1e 1c
:2 25 31 1c 1f :2 29 3b :2 45 51
55 66 69 6a :2 66 :2 3b :3 1f :2 25
2a 36 76 79 :2 36 86 89 :2 36
8d 90 :2 36 9c 9f :2 36 a7 aa
:2 36 b6 b9 c0 c9 cc :2 b9 :2 36
:2 1f 6d :2 1c :2 19 :4 25 :2 19 16 :7 13
:4 10 d 3a 13 1c 25 :2 13 28
2a :2 28 1a 2f 43 13 1f 13
22 19 2d 13 1a 16 22 :3 20
16 :2 1f 2b 16 27 16 :2 1f 2b
16 :4 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 60 63 64
:2 60 :2 35 :3 19 :2 1f 24 30 70 73
:2 30 80 83 :2 30 87 8a :2 30 96
99 :2 30 a1 a4 :2 30 b0 b3 ba
c3 c6 :2 b3 :2 30 :2 19 67 :2 16 :2 13
:4 1f :2 13 10 :4 2c 3a :2 d 2c 11
18 21 2c :2 18 30 :2 11 10 35
37 :2 35 :4 14 13 :4 17 16 20 35
49 19 25 2e 3d 19 2a 36
48 55 65 19 28 19 29 19
27 1f 33 19 20 1c 28 :3 26
1c :2 25 31 1c 2d 1c :2 25 31
1c :4 19 16 1e 1c :2 25 31 1c
2c :2 19 :2 1e 1c :2 25 31 1c 1f
:2 29 3b :2 45 51 55 66 69 6a
:2 66 :2 3b :3 1f :2 25 2a 36 76 79
:2 36 86 89 :2 36 8d 90 :2 36 9c
9f :2 36 a7 aa :2 36 b6 b9 c0
c9 cc :2 b9 :2 36 :2 1f 6d :2 1c :2 19
:4 25 :2 19 16 :4 33 20 35 49 19
25 2e 3d 19 27 1f 33 19
20 1c 28 :3 26 1c :2 25 31 1c
2d 1c :2 25 31 1c :4 19 16 1e
1c :2 25 31 1c 2c :2 19 :2 1e 1c
:2 25 31 1c 1f :2 29 3b :2 45 51
55 66 69 6a :2 66 :2 3b :3 1f :2 25
2a 36 76 79 :2 36 86 89 :2 36
8d 90 :2 36 9c 9f :2 36 a7 aa
:2 36 b6 b9 c0 c9 cc :2 b9 :2 36
:2 1f 6d :2 1c :2 19 :4 25 :2 19 16 :7 13
2f :4 17 16 20 35 49 19 25
2e 3d 19 35 51 5e 76 19
36 52 5f 78 19 38 54 6e
19 34 50 66 19 2b 19 2c
19 2e 19 2a 19 2b 19 2c
19 2e 19 2a 19 2a 38 49
57 68 76 87 :2 19 2a 36 48
55 65 19 28 19 29 19 27
1f 33 19 20 1c 28 :3 26 1c
:2 25 31 1c 2d 1c :2 25 31 1c
:4 19 16 1e 1c :2 25 31 1c 2c
:2 19 :2 1e 1c :2 25 31 1c 1f :2 29
3b :2 45 51 55 66 69 6a :2 66
:2 3b :3 1f :2 25 2a 36 76 79 :2 36
86 89 :2 36 8d 90 :2 36 9c 9f
:2 36 a7 aa :2 36 b6 b9 c0 c9
cc :2 b9 :2 36 :2 1f 6d :2 1c :2 19 :4 25
:2 19 16 :4 33 20 35 49 19 25
2e 3d 19 27 19 35 51 5e
76 19 36 52 5f 78 19 38
54 6e 19 34 50 66 19 2b
19 2c 19 2e 19 2a 19 2b
19 2c 19 2e 19 2a 19 2a
38 49 57 68 76 87 19 1f
33 19 20 1c 28 :3 26 1c :2 25
31 1c 2d 1c :2 25 31 1c :4 19
16 1e 1c :2 25 31 1c 2c :2 19
:2 1e 1c :2 25 31 1c 1f :2 29 3b
:2 45 51 55 66 6a 6b :2 66 :2 3b
:3 1f :2 25 2a 36 77 7a :2 36 87
8a :2 36 8e 91 :2 36 9d a0 :2 36
a8 ab :2 36 b7 ba c1 ca cd
:2 ba :2 36 :2 1f 6e :2 1c :2 19 :4 25 :2 19
16 :7 13 :4 10 d 3a 13 1c 25
:2 13 28 2a :2 28 :4 14 13 16 1f
25 :2 16 28 2a :2 28 20 35 49
19 25 2e 3d 19 27 19 1f
2a 1f 33 19 20 1c 28 :3 26
1c :2 25 31 1c 2d 1c :2 25 31
1c :4 19 16 1e 1c :2 25 31 1c
2c :2 19 :2 1e 1c :2 25 31 1c 1f
:2 29 3b :2 45 51 55 66 6a 6b
:2 66 :2 3b :3 1f :2 25 2a 36 77 7a
:2 36 87 8a :2 36 8e 91 :2 36 9d
a0 :2 36 a8 ab :2 36 b7 ba c1
ca cd :2 ba :2 36 :2 1f 6e :2 1c :2 19
:4 25 :2 19 16 :4 2c 20 35 49 19
25 2e 3d 19 27 1f 33 19
20 1c 28 :3 26 1c :2 25 31 1c
2d 1c :2 25 31 1c :4 19 16 1e
1c :2 25 31 1c 2c :2 19 :2 1e 1c
:2 25 31 1c 1f :2 29 3b :2 45 51
55 66 6a 6b :2 66 :2 3b :3 1f :2 25
2a 36 77 7a :2 36 87 8a :2 36
8e 91 :2 36 9d a0 :2 36 a8 ab
:2 36 b7 ba c1 ca cd :2 ba :2 36
:2 1f 6e :2 1c :2 19 :4 25 :2 19 16 :7 13
31 1d 32 46 16 22 2b 3a
16 24 1c 30 16 1d 19 25
:3 23 19 :2 22 2e 19 2a 19 :2 22
2e 19 :4 16 13 1b 19 :2 22 2e
19 29 :2 16 :2 1b 19 :2 22 2e 19
1c :2 26 38 :2 42 4e 52 63 67
68 :2 63 :2 38 :3 1c :2 22 27 33 74
77 :2 33 84 87 :2 33 8b 8e :2 33
9a 9d :2 33 a5 a8 :2 33 b4 b7
be c7 ca :2 b7 :2 33 :2 1c 6b :2 19
:2 16 :4 22 :2 16 13 :7 10 2c 3a :2 d
:4 a :4 f e 30 39 4b :2 30 4e
50 :2 4e 2f :2 e d 58 10 20
:2 58 23 25 :2 23 57 :3 d 1b :2 d
1b :2 d 18 :2 d 18 d 10 19
28 :2 10 2b 2d :2 2b 1a 2d 39
42 4a 51 67 6f 76 8d 97
a8 b6 18 2d 37 42 48 4e
5a 6d 79 18 19 2d :2 13 :2 1c
28 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 60 64 65
:2 60 :2 35 :3 19 :2 1f 24 30 71 74
:2 30 81 84 :2 30 88 8b :2 30 97
9a :2 30 a2 a5 :2 30 b1 b4 bb
c4 c7 :2 b4 :2 30 :2 19 68 :2 16 :2 13
:4 1f :2 13 10 :4 2f 10 27 10 1a
26 2f 37 3e 54 5c 63 7a
84 95 a3 18 22 2d 33 39
45 58 64 18 19 2d :2 13 :2 1c
28 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 60 64 65
:2 60 :2 35 :3 19 :2 1f 24 30 71 74
:2 30 81 84 :2 30 88 8b :2 30 97
9a :2 30 a2 a5 :2 30 b1 b4 bb
c4 c7 :2 b4 :2 30 :2 19 68 :2 16 :2 13
:4 1f :2 13 10 :7 d 11 :2 1a 25 23
:2 2e :2 23 10 14 1d 2d :2 14 32
30 3b 46 :2 32 :2 30 13 4f 58
5e :2 4f 61 63 :2 61 4e :2 13 6b
2c 41 :2 6b 44 46 :2 44 6a :2 13
1d 32 3f 4a 1c 29 3a 4e
16 1d 19 25 :3 23 19 :2 22 2e
19 2a 19 :2 22 2e 19 :4 16 13
1b 19 :2 22 2e 19 29 :2 16 :2 1b
19 :2 22 2e 19 1c :2 26 38 :2 42
4e 52 63 67 68 :2 63 :2 38 :3 1c
:2 22 27 33 74 77 :2 33 84 87
:2 33 8b 8e :2 33 9a 9d :2 33 a5
a8 :2 33 b4 b7 be c7 ca :2 b7
:2 33 :2 1c 6b :2 19 :2 16 :4 22 :2 16 13
:3 4d 1d 32 40 16 2f 33 42
1c 29 3a 4e 1c 30 16 1d
19 25 :3 23 19 :2 22 2e 19 2a
19 :2 22 2e 19 :4 16 13 1b 19
:2 22 2e 19 29 :2 16 :2 1b 19 :2 22
2e 19 1c :2 26 38 :2 42 4e 52
63 67 68 :2 63 :2 38 :3 1c :2 22 27
33 74 77 :2 33 84 87 :2 33 8b
8e :2 33 9a 9d :2 33 a5 a8 :2 33
b4 b7 be c7 ca :2 b7 :2 33 :2 1c
6b :2 19 :2 16 :4 22 :2 16 13 :4 4d 13
18 :4 13 1c 25 30 3d :3 16 19
1d :2 19 :2 22 2e 19 23 19 :2 22
2e 19 :4 16 19 :2 22 2b 2e :2 2b
:2 19 30 :3 16 21 2a 33 :2 21 36
38 41 4c :2 38 :2 21 16 19 22
2b :2 19 2e 30 :2 2e 1c 25 2e
:2 1c 33 31 3c 4b :2 33 :2 31 1f
28 33 :2 1f 39 36 42 51 :2 39
:2 36 1f 2c 35 44 :2 2c 1f 54
1f 2c 35 44 :2 2c 47 49 52
5b :2 49 :2 2c 5e 60 69 74 :2 60
:2 2c 1f :4 1c 4e :2 19 23 38 45
50 71 1c 23 1f 2b :3 29 1f
:2 28 34 1f 30 1f :2 28 34 1f
:4 1c 19 21 1f :2 28 34 1f 2f
:2 1c :2 21 1f :2 28 34 1f 22 :2 2c
3e :2 48 54 58 69 6d 6e :2 69
:2 3e :3 22 :2 28 2d 39 7a 7d :2 39
8a 8d :2 39 91 94 :2 39 a0 a3
:2 39 ab ae :2 39 ba bd c4 cd
d0 :2 bd :2 39 :2 22 71 :2 1f :2 1c :4 28
:2 1c 19 :3 32 19 27 19 32 :2 16
19 22 2b :2 19 31 2e 3a 49
:2 31 :2 2e :2 19 4c :2 16 13 17 10
16 1c :3 16 :2 1f 2b 16 13 1b
19 :2 22 2e 19 29 :2 16 :2 1b 19
:2 22 2e 19 1c :2 26 38 :2 42 4e
52 63 67 68 :2 63 :2 38 :3 1c :2 22
27 33 74 77 :2 33 84 87 :2 33
8b 8e :2 33 9a 9d :2 33 a5 a8
:2 33 b4 b7 be c7 ca :2 b7 :2 33
:2 1c 6b :2 19 :2 16 :4 22 :2 16 13 :3 10
17 20 29 :2 17 2c 2e :2 2c 16
:4 37 36 35 :2 16 20 35 46 57
6b 19 20 1c 28 :3 26 1c :2 25
31 1c 2d 1c :2 25 31 1c :4 19
16 1e 1c :2 25 31 1c 2c :2 19
:2 1e 1c :2 25 31 1c 1f :2 29 3b
:2 45 51 55 66 6a 6b :2 66 :2 3b
:3 1f :2 25 2a 36 77 7a :2 36 87
8a :2 36 8e 91 :2 36 9d a0 :2 36
a8 ab :2 36 b7 ba c1 ca cd
:2 ba :2 36 :2 1f 6e :2 1c :2 19 :4 25 :2 19
16 :4 50 :2 13 :4 10 13 1c 2b :2 13
2e 30 :2 2e 17 1e 27 32 :2 1e
36 :2 17 16 3b 3d :2 3b 16 29
3c 51 :2 29 16 40 16 29 3c
51 :2 29 16 :4 13 32 :2 10 3e :2 d
28 :2 a 2a :2 7 a :2 13 1c 1e
:2 1c a :2 13 1f a 22 :2 7 4f
:2 4 7 :2 11 23 :2 2d 39 3c 4d
51 52 :2 4d :2 23 :3 7 :2 d 12 1e
5f 62 :2 1e 6f 72 :2 1e 76 79
:2 1e 83 86 8f 97 :2 a0 :2 8f ab
:2 86 :2 1e b3 b6 :2 1e be c1 :2 1e
c6 c9 :2 1e cc cf :2 1e :2 7 55
:3 4 :2 f 21 :2 2a 34 38 :2 4 1
9 12 9 a :2 14 26 :2 30 3c
40 51 55 56 :2 51 :2 26 :3 a :2 10
15 21 62 65 :2 21 :3 a :2 10 15
39 5c 5f :2 68 :2 39 76 79 :2 39
81 84 :2 8d :2 39 :2 a 59 :3 7 c
15 1e 2c a 13 1e f e
7 e 7 e 7 29 :2 4 :2 9
7 :2 10 1c :2 7 :2 11 23 :2 2d 39
3d 4e 52 53 :2 4e :2 23 :3 7 :2 d
12 1e 5f 62 :2 1e :3 7 :2 d 12
36 42 45 :2 36 4d 50 :2 36 5c
5f 66 6f 72 :2 5f :2 36 :2 7 56
:2 4 7 c 1b a 13 1e f
e 7 e 7 e 7 :2 4 :4 10
:2 4 :8 1 
1e0f
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 :4 7 :4 8
:4 9 :4 a :4 b :4 c
:4 d :4 e :4 f :4 10
:4 11 :4 12 :4 13 :4 14
:4 15 :4 16 :4 17 2
:4 18 :2 1 :7 1a :7 1b
:7 1c :7 1d :7 1e :7 1f
:7 20 :7 21 :7 22 :7 23
:7 24 :7 25 :7 26 :7 27
:6 28 :6 29 :6 2a :6 2b
:6 2c :6 2d :6 2e :6 2f
:6 30 :6 31 :6 32 :6 33
:6 34 :6 35 :6 36 :6 37
:5 38 :5 39 :7 3a :7 3b
:7 3c :5 3d :5 3e :5 3f
:5 40 :5 41 :5 42 :7 43
:6 44 :6 45 :6 46 :6 47
:6 48 :6 49 :6 4a :5 4b
:5 4c :6 4d :6 4e :6 4f
:6 50 :6 51 :6 52 :2 54
0 :2 54 :7 55 56
:4 57 :2 58 55 :5 54
:2 5a 0 :2 5a :3 5b
5c :4 5d :2 5e 5b
:5 5a :7 60 :b 62 :11 63
64 :2 63 :2 64 :2 63
:2 64 :2 63 :2 65 :2 63
:9 65 :2 63 :2 66 :2 63
:2 66 :2 63 :2 66 :2 63
:2 66 :2 63 :2 67 :2 63
:2 67 :2 63 :2 67 :2 63
:2 67 :2 63 :2 68 :2 63
:2 68 :2 63 :2 68 :2 63
:2 68 :2 63 :2 69 :2 63
:9 69 :2 63 :2 6a :2 63
:2 6a :2 63 :2 6a :2 63
:2 6a :2 63 :2 6b :2 63
:9 6b :2 63 :2 6c :2 63
:9 6c :2 63 :2 6d :2 63
:9 6d :2 63 :2 6e :2 63
:9 6e :2 63 :2 6f :2 63
:9 6f :2 63 :2 70 :2 63
:9 70 :2 63 :2 71 :2 63
:2 71 :2 63 :2 71 :2 63
:2 71 :2 63 :2 72 :2 63
:2 72 :2 63 :2 72 :2 63
:2 72 :2 63 :2 73 :2 63
:9 73 :2 63 :2 74 :2 63
:9 74 :2 63 :2 75 :2 63
:2 75 :2 63 :2 75 :2 63
:2 75 :2 63 :2 76 :2 63
:2 76 :2 63 :2 76 :2 63
:2 76 :2 63 :2 77 :2 63
:9 77 :2 63 :2 78 :2 63
:2 78 :2 63 :2 78 :2 63
:2 78 :2 63 :2 78 :4 63
:3 62 :11 7b :b 7c :3 7b
:11 7f :2 80 :2 7f :4 80
:6 7f :11 81 :20 82 :3 81
:11 84 :b 85 :3 84 :9 87
80 :2 7f :16 8a :11 8b
:20 8c :3 8b :11 8e :b 8f
:3 8e :9 91 :3 8a :3 96
:3 97 :3 98 :3 99 :3 9a
:3 9b :3 9c :3 9d :3 9e
:3 9f :3 a0 :3 a1 :3 a2
:3 a3 :e a7 :16 a8 :5 aa
:11 ab :20 ac :3 ab :11 ae
:b af :3 ae :9 b1 :3 aa
:3 b4 :3 b5 :3 b6 :4 b9
:3 ba b9 :5 bb b8
bd :5 be :3 bd :2 bf
:5 c1 :11 c2 :24 c3 :3 c2
:2 c0 :6 bf bc :3 a8
:5 c8 :a c9 :9 cb cc
:3 cb :20 cf :3 d0 :3 d2
d3 :4 d4 d2 :5 d5
d1 d7 :5 d8 :3 d7
:2 d9 :5 db :11 dc :24 dd
:3 dc :2 da :6 d9 d6
:3 cf :9 e2 :3 e3 :3 e4
:3 e5 :5 e7 :7 e9 :5 ea
e8 ec :5 ed :3 ec
:2 ee :5 f0 :11 f1 :24 f2
:3 f1 :2 ef :6 ee eb
:6 e7 :3 e2 :3 cf :19 fb
:3 fc :3 fb a8 :3 ff
:5 102 :5 103 :5 104 :5 105
:3 106 102 :5 107 101
109 :5 10a :3 109 :2 10b
:5 10d :11 10e :24 10f :3 10e
:2 10c :6 10b 108 :3 fe
:11 115 :2 116 :7 115 :3 117
:6 118 116 :2 115 fe
:3 a8 :9 11d :7 11e :11 11f
:30 120 :3 11f :11 122 :b 123
:3 122 :a 125 :3 11e :13 129
:5 12a :2 129 :11 12b :20 12c
:3 12b :11 12e :b 12f :3 12e
:9 131 12a :2 129 :5 134
:9 137 :3 138 :3 137 :7 13f
:5 140 13e 142 :5 143
:3 142 :2 144 :5 146 :11 147
:24 148 :3 147 :2 145 :6 144
141 :3 11d :9 14d :1b 14f
:2 150 :5 14f :d 152 :3 153
:3 154 152 :3 156 155
:3 152 :3 14d :3 173 :e 175
:9 176 :9 177 :9 178 :6 179
:4 17a :2 17b :3 178 :7 17e
:3 180 :6 181 :6 182 180
:6 183 :5 184 183 :5 186
185 :3 183 17f 189
:5 18a :3 189 :2 18b :5 18d
:11 18e :24 18f :3 18e :2 18c
:6 18b 188 :6 17e 177
:9 195 :6 196 :4 197 :2 198
:3 195 194 :3 177 176
:b 19c :9 19d :9 19e :6 19f
:4 1a0 :2 1a1 :3 19e :7 1a4
:3 1a6 :6 1a7 :2 1a8 1a6
:6 1a9 :5 1aa 1a9 :5 1ac
1ab :3 1a9 1a5 1af
:5 1b0 :3 1af :2 1b1 :5 1b3
:11 1b4 :24 1b5 :3 1b4 :2 1b2
:6 1b1 1ae :6 1a4 19d
:9 1bb :6 1bc :4 1bd :2 1be
:3 1bb 1ba :3 19d 19b
:3 176 :19 1c3 :1b 1c4 :2 1c5
:2 1c4 :4 1c5 :3 1c4 :3 1c7
:4 1c8 :6 1c9 :4 1ca :2 1cb
:2 1cc :2 1cd 1c7 :6 1ce
:5 1cf 1ce :5 1d1 1d0
:3 1ce 1c6 1d4 :5 1d5
:3 1d4 :2 1d6 :5 1d8 :11 1d9
:24 1da :3 1d9 :2 1d7 :6 1d6
1d3 :3 1c5 :4 1e0 :4 1e1
1e0 :6 1e2 :5 1e3 1e2
:5 1e5 1e4 :3 1e2 1df
1e8 :5 1e9 :3 1e8 :2 1ea
:5 1ec :11 1ed :24 1ee :3 1ed
:2 1eb :6 1ea 1e7 :3 1c5
:3 1f4 :2 1f5 1f4 :6 1f6
:5 1f7 1f6 :5 1f9 1f8
:3 1f6 1f3 1fc :5 1fd
:3 1fc :2 1fe :5 200 :11 201
:24 202 :3 201 :2 1ff :6 1fe
1fb :4 1c5 :3 208 :6 209
:2 20a 208 :6 20b :5 20c
20b :5 20e 20d :3 20b
207 211 :5 212 :3 211
:2 213 :5 215 :11 216 :24 217
:3 216 :2 214 :6 213 210
:4 206 :3 1c4 :3 1c3 175
:d 21e :9 220 :9 221 :6 222
:4 223 :2 224 :3 221 :20 227
:3 229 :6 22a :6 22b 229
:6 22c :5 22d 22c :5 22f
22e :3 22c 228 232
:5 233 :3 232 :2 234 :5 236
:11 237 :24 238 :3 237 :2 235
:6 234 231 :6 227 220
:5 23e :3 23f :b 245 :5 246
244 248 :5 249 :3 248
:2 24a :5 24c :11 24d :24 24e
:3 24d :2 24b :6 24a 247
:3 23e :c 253 :5 254 :3 255
:2 258 259 25a 258
:5 25b 257 25d :5 25e
:3 25d :2 25f :5 261 :11 262
:24 263 :3 262 :2 260 :6 25f
25c :3 253 :7 268 269
:3 268 :9 26b :3 26c :3 26d
:3 26e :3 26f :3 270 :5 272
:6 274 :3 275 :5 276 275
:5 278 277 :3 275 :4 27c
:5 27d 27b 27f :5 280
:3 27f :2 281 :5 283 :11 284
:24 285 :3 284 :2 282 :6 281
27e :3 26b :3 28b :2 28c
28b :5 28d 28a 28f
:5 290 :3 28f :2 291 :5 293
:11 294 :24 295 :3 294 :2 292
:6 291 28e :3 26b :3 29b
:3 29d :c 29e :c 29f :c 2a0
:4 2a1 2a2 :2 2a1 :2 2a3
:3 2a1 :8 2a5 :6 2a6 :3 2a7
:3 2a5 :11 2a9 :2e 2aa :3 2a9
:11 2ac :b 2ad :3 2ac :8 2af
:3 26b :3 253 :3 23e :b 2b4
:9 2b5 :6 2b6 :4 2b7 :2 2b8
:3 2b5 :20 2bc :3 2be :6 2bf
:2 2c0 2be :6 2c1 :5 2c2
2c1 :5 2c4 2c3 :3 2c1
2bd 2c7 :5 2c8 :3 2c7
:2 2c9 :5 2cb :11 2cc :24 2cd
:3 2cc :2 2ca :6 2c9 2c6
:6 2bc 23d :3 220 21d
:3 175 :7 2d6 :11 2d7 :30 2d8
:3 2d7 :11 2da :b 2db :3 2da
:a 2dd :3 2d6 :3 2e0 :16 2e1
:3 2e2 :3 2e1 :9 2e5 :e 2e6
:5 2e7 :5 2e9 :3 2eb :4 2ec
:6 2ed :2 2ee :2 2ef :2 2f0
:2 2f1 2eb :6 2f2 :5 2f3
2f2 :5 2f5 2f4 :3 2f2
2ea 2f8 :5 2f9 :3 2f8
:2 2fa :5 2fc :11 2fd :24 2fe
:3 2fd :2 2fb :6 2fa 2f7
:4 2e9 :3 304 :2 305 :2 306
:2 307 304 :6 308 :5 309
308 :5 30b 30a :3 308
303 30e :5 30f :3 30e
:2 310 :5 312 :11 313 :24 314
:3 313 :2 311 :6 310 30d
:4 302 :3 2e9 2e7 :5 31b
:3 31d :4 31e :5 31f :5 320
:4 321 :4 322 :2 324 :2 325
:2 326 :2 327 :2 329 :2 32a
:2 32b :2 32c :8 32d 32f
:6 330 :2 331 :2 332 :2 333
:2 334 31d :6 335 :5 336
335 :5 338 337 :3 335
31c 33b :5 33c :3 33b
:2 33d :5 33f :11 340 :24 341
:3 340 :2 33e :6 33d 33a
:4 31b :3 348 :4 349 :5 34a
:5 34b :4 34c :4 34d :2 34f
:2 350 :2 351 :2 352 :2 354
:2 355 :2 356 :2 357 :8 358
35a :2 35b 348 :6 35c
:5 35d 35c :5 35f 35e
:3 35c 347 362 :5 363
:3 362 :2 364 :5 366 :11 367
:24 368 :3 367 :2 365 :6 364
361 :4 346 :3 31b 319
:3 2e7 36e 2e6 :9 36e
:3 370 :2 371 :2 372 :2 373
370 :6 374 :5 375 374
:5 377 376 :3 374 36f
37a :5 37b :3 37a :2 37c
:5 37e :11 37f :24 380 :3 37f
:2 37d :6 37c 379 :4 36e
:3 2e6 2e5 :e 386 :5 387
:5 389 :3 38b :4 38c :6 38d
:2 38e :2 38f :2 390 :2 391
38b :6 392 :5 393 392
:5 395 394 :3 392 38a
398 :5 399 :3 398 :2 39a
:5 39c :11 39d :24 39e :3 39d
:2 39b :6 39a 397 :4 389
:3 3a4 :4 3a5 :2 3a6 :2 3a7
3a4 :6 3a8 :5 3a9 3a8
:5 3ab 3aa :3 3a8 3a3
3ae :5 3af :3 3ae :2 3b0
:5 3b2 :11 3b3 :24 3b4 :3 3b3
:2 3b1 :6 3b0 3ad :4 3a2
:3 389 387 :5 3bb :3 3bd
:4 3be :5 3bf :5 3c0 :4 3c1
:4 3c2 :2 3c4 :2 3c5 :2 3c6
:2 3c7 :2 3c9 :2 3ca :2 3cb
:2 3cc :8 3cd 3cf :6 3d0
:2 3d1 :2 3d2 :2 3d3 :2 3d4
3bd :6 3d5 :5 3d6 3d5
:5 3d8 3d7 :3 3d5 3bc
3db :5 3dc :3 3db :2 3dd
:5 3df :11 3e0 :24 3e1 :3 3e0
:2 3de :6 3dd 3da :4 3bb
:3 3e7 :4 3e8 :2 3e9 :5 3ea
:5 3eb :4 3ec :4 3ed :2 3ef
:2 3f0 :2 3f1 :2 3f2 :2 3f4
:2 3f5 :2 3f6 :2 3f7 :8 3f8
3fa :2 3fb 3e7 :6 3fc
:5 3fd 3fc :5 3ff 3fe
:3 3fc 3e6 402 :5 403
:3 402 :2 404 :5 406 :11 407
:24 408 :3 407 :2 405 :6 404
401 :4 3e5 :3 3bb 3b9
:3 387 40e 386 :9 40e
:5 40f :9 410 :3 412 :4 413
:2 414 :3 415 :2 416 412
:6 417 :5 418 417 :5 41a
419 :3 417 411 41d
:5 41e :3 41d :2 41f :5 421
:11 422 :24 423 :3 422 :2 420
:6 41f 41c :4 410 :3 429
:4 42a :2 42b :2 42c 429
:6 42d :5 42e 42d :5 430
42f :3 42d 428 433
:5 434 :3 433 :2 435 :5 437
:11 438 :24 439 :3 438 :2 436
:6 435 432 :4 427 :3 410
40f :3 440 :4 441 :2 442
:2 443 440 :6 444 :5 445
444 :5 447 446 :3 444
43f 44a :5 44b :3 44a
:2 44c :5 44e :11 44f :24 450
:3 44f :2 44d :6 44c 449
:4 43e :3 40f 40e :3 386
385 :3 2e5 :13 45a :2 45b
:2 45a :4 45b :3 45a :3 45c
:3 45d :3 45e :3 45f :9 461
:d 463 :9 464 465 :2 466
463 :5 467 462 469
:5 46a :3 469 :2 46b :5 46d
:11 46e :24 46f :3 46e :2 46c
:6 46b 468 :4 461 :3 474
:c 476 :8 477 478 :2 479
476 :5 47a 475 47c
:5 47d :3 47c :2 47e :5 480
:11 481 :24 482 :3 481 :2 47f
:6 47e 47b :4 473 :3 461
:a 489 :1b 48a :2 48b :2 48a
:4 48b :3 48a :4 48d :4 48e
48d :6 48f :5 490 48f
:5 492 491 :3 48f 48c
495 :5 496 :3 495 :2 497
:5 499 :11 49a :24 49b :3 49a
:2 498 :6 497 494 :3 48b
:3 4a1 :4 4a2 :4 4a3 :2 4a4
4a1 :6 4a5 :5 4a6 4a5
:5 4a8 4a7 :3 4a5 4a0
4ab :5 4ac :3 4ab :2 4ad
:5 4af :11 4b0 :24 4b1 :3 4b0
:2 4ae :6 4ad 4aa :4 48b
:5 4c5 4c7 :7 4c8 :3 4c9
:5 4ca 4c9 :5 4cc 4cb
:3 4c9 :7 4ce :2 4cf :3 4ce
:f 4d3 :9 4d5 :d 4d6 :d 4d7
:7 4d8 4d7 :17 4da 4d9
:3 4d7 :3 4d6 :6 4df :6 4e0
:5 4e1 4e0 :5 4e3 4e2
:3 4e0 4de 4e6 :5 4e7
:3 4e6 :2 4e8 :5 4ea :11 4eb
:24 4ec :3 4eb :2 4e9 :6 4e8
4e5 :3 4d5 :3 4f1 :3 4d5
:d 4f4 :2 4f5 :3 4f4 4c7
4f7 4c4 :4 4fa :5 4fb
4f9 4fd :5 4fe :3 4fd
:2 4ff :5 501 :11 502 :24 503
:3 502 :2 500 :6 4ff 4fc
:3 4c4 :12 509 :6 50b :6 50c
:5 50d 50c :5 50f 50e
:3 50c 50a 512 :5 513
:3 512 :2 514 :5 516 :11 517
:24 518 :3 517 :2 515 :6 514
511 :6 509 4c4 :3 48a
:9 565 :e 566 :7 567 566
:7 569 568 :3 566 :3 565
:3 489 45b :2 45a :3 11d
:7 570 :5 571 :3 570 :3 a7
:11 577 :30 578 :3 577 :a 57a
61 :3 57e :11 57f :b 580
:17 581 :3 57f :2 583 :3 584
:3 585 586 584 583
584 :3 587 :3 57e :2 588
:5 58a :11 58b :b 58c :18 58d
:3 58b :2 58f 590 :3 591
592 590 58f 590
:3 593 :2 589 :6 588 57d
:7 1 
6993
4
:3 0 1 :3 0 2
:a 0 1e0a 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 6
:3 0 5 :7 0 9
8 :3 0 4 :3 0
7 :7 0 d c
:3 0 d 7c 0
b 4 :3 0 8
:7 0 11 10 :3 0
4 :3 0 9 :7 0
15 14 :3 0 11
a2 0 f 6
:3 0 a :7 0 19
18 :3 0 4 :3 0
b :7 0 1d 1c
:3 0 15 c8 0
13 6 :3 0 c
:7 0 21 20 :3 0
6 :3 0 d :7 0
25 24 :3 0 19
ee 0 17 6
:3 0 e :7 0 29
28 :3 0 6 :3 0
f :7 0 2d 2c
:3 0 1d 114 0
1b 6 :3 0 10
:7 0 31 30 :3 0
6 :3 0 11 :7 0
35 34 :3 0 21
13a 0 1f 4
:3 0 12 :7 0 39
38 :3 0 4 :3 0
13 :7 0 3d 3c
:3 0 25 160 0
23 6 :3 0 14
:7 0 41 40 :3 0
6 :3 0 15 :7 0
45 44 :3 0 29
186 0 27 4
:3 0 16 :7 0 49
48 :3 0 4 :3 0
17 :7 0 4d 4c
:3 0 2d :2 0 2b
6 :3 0 18 :7 0
51 50 :3 0 4
:3 0 19 :7 0 55
54 :3 0 1a :3 0
1b :3 0 1c :2 0
4 59 5a 0
57 5b 0 1e0a
2 5c :2 0 1e
:2 0 45 4 :3 0
1e :2 0 43 5f
61 :6 0 64 62
0 1e08 0 1d
:6 0 21 :2 0 49
4 :3 0 47 66
68 :6 0 6b 69
0 1e08 0 1f
:6 0 23 :2 0 4d
4 :3 0 4b 6d
6f :6 0 72 70
0 1e08 0 20
:6 0 25 :2 0 51
4 :3 0 4f 74
76 :6 0 79 77
0 1e08 0 22
:6 0 27 :2 0 55
4 :3 0 53 7b
7d :6 0 80 7e
0 1e08 0 24
:6 0 27 :2 0 59
4 :3 0 57 82
84 :6 0 87 85
0 1e08 0 26
:6 0 27 :2 0 5d
4 :3 0 5b 89
8b :6 0 8e 8c
0 1e08 0 28
:6 0 27 :2 0 61
4 :3 0 5f 90
92 :6 0 95 93
0 1e08 0 29
:6 0 27 :2 0 65
4 :3 0 63 97
99 :6 0 9c 9a
0 1e08 0 2a
:6 0 27 :2 0 69
4 :3 0 67 9e
a0 :6 0 a3 a1
0 1e08 0 2b
:6 0 27 :2 0 6d
4 :3 0 6b a5
a7 :6 0 aa a8
0 1e08 0 2c
:6 0 27 :2 0 71
4 :3 0 6f ac
ae :6 0 b1 af
0 1e08 0 2d
:6 0 27 :2 0 75
4 :3 0 73 b3
b5 :6 0 b8 b6
0 1e08 0 2e
:6 0 31 :2 0 79
4 :3 0 77 ba
bc :6 0 bf bd
0 1e08 0 2f
:6 0 31 :2 0 7b
6 :3 0 c1 :7 0
c5 c2 c3 1e08
0 30 :6 0 31
:2 0 7d 6 :3 0
c7 :7 0 cb c8
c9 1e08 0 32
:6 0 35 :2 0 7f
6 :3 0 cd :7 0
d1 ce cf 1e08
0 33 :6 0 31
:2 0 81 6 :3 0
d3 :7 0 d7 d4
d5 1e08 0 34
:6 0 31 :2 0 83
6 :3 0 d9 :7 0
dd da db 1e08
0 36 :6 0 31
:2 0 85 6 :3 0
df :7 0 e3 e0
e1 1e08 0 37
:6 0 31 :2 0 87
6 :3 0 e5 :7 0
e9 e6 e7 1e08
0 38 :6 0 3b
:2 0 89 6 :3 0
eb :7 0 ef ec
ed 1e08 0 39
:6 0 31 :2 0 8b
6 :3 0 f1 :7 0
f5 f2 f3 1e08
0 3a :6 0 31
:2 0 8d 6 :3 0
f7 :7 0 fb f8
f9 1e08 0 3c
:6 0 31 :2 0 8f
6 :3 0 fd :7 0
101 fe ff 1e08
0 3d :6 0 31
:2 0 91 6 :3 0
103 :7 0 107 104
105 1e08 0 3e
:6 0 31 :2 0 93
6 :3 0 109 :7 0
10d 10a 10b 1e08
0 3f :6 0 31
:2 0 95 6 :3 0
10f :7 0 113 110
111 1e08 0 40
:6 0 31 :2 0 97
6 :3 0 115 :7 0
119 116 117 1e08
0 41 :6 0 9b
51c 0 99 6
:3 0 11b :7 0 11f
11c 11d 1e08 0
42 :6 0 27 :2 0
9d 44 :3 0 121
:7 0 124 122 0
1e08 0 43 :6 0
44 :3 0 126 :7 0
129 127 0 1e08
0 45 :6 0 27
:2 0 a1 4 :3 0
9f 12b 12d :6 0
130 12e 0 1e08
0 46 :6 0 49
:2 0 a5 4 :3 0
a3 132 134 :6 0
137 135 0 1e08
0 47 :6 0 ab
5a7 0 a9 4
:3 0 a7 139 13b
:6 0 13e 13c 0
1e08 0 48 :6 0
af 5db 0 ad
4b :3 0 140 :7 0
143 141 0 1e08
0 4a :6 0 4b
:3 0 145 :7 0 148
146 0 1e08 0
4c :6 0 b3 60f
0 b1 4b :3 0
14a :7 0 14d 14b
0 1e08 0 4d
:6 0 4b :3 0 14f
:7 0 152 150 0
1e08 0 4e :6 0
21 :2 0 b5 4b
:3 0 154 :7 0 157
155 0 1e08 0
4f :6 0 4b :3 0
159 :7 0 15c 15a
0 1e08 0 50
:6 0 31 :2 0 b9
4 :3 0 b7 15e
160 :6 0 163 161
0 1e08 0 51
:6 0 31 :2 0 bb
6 :3 0 165 :7 0
169 166 167 1e08
0 52 :6 0 31
:2 0 bd 6 :3 0
16b :7 0 16f 16c
16d 1e08 0 53
:6 0 31 :2 0 bf
6 :3 0 171 :7 0
175 172 173 1e08
0 54 :6 0 31
:2 0 c1 6 :3 0
177 :7 0 17b 178
179 1e08 0 55
:6 0 31 :2 0 c3
6 :3 0 17d :7 0
181 17e 17f 1e08
0 56 :6 0 31
:2 0 c5 6 :3 0
183 :7 0 187 184
185 1e08 0 57
:6 0 c9 724 0
c7 6 :3 0 189
:7 0 18d 18a 18b
1e08 0 58 :6 0
31 :2 0 cb 44
:3 0 18f :7 0 192
190 0 1e08 0
59 :6 0 44 :3 0
194 :7 0 197 195
0 1e08 0 5a
:6 0 31 :2 0 cd
5c :3 0 199 :7 0
19d 19a 19b 1e08
0 5b :6 0 31
:2 0 cf 5c :3 0
19f :7 0 1a3 1a0
1a1 1e08 0 5d
:6 0 31 :2 0 d1
6 :3 0 1a5 :7 0
1a9 1a6 1a7 1e08
0 5e :6 0 31
:2 0 d3 6 :3 0
1ab :7 0 1af 1ac
1ad 1e08 0 5f
:6 0 31 :2 0 d5
6 :3 0 1b1 :7 0
1b5 1b2 1b3 1e08
0 60 :9 0 d7
6 :3 0 1b7 :7 0
1bb 1b8 1b9 1e08
0 61 :6 0 62
:3 0 63 :a 0 2
1d0 :4 0 1bd 1c0
0 1be :3 0 64
:3 0 65 :3 0 66
:3 0 67 :3 0 65
:3 0 66 :3 0 68
:3 0 69 :3 0 6a
:3 0 2d :3 0 6b
:3 0 2e :3 0 6c
:3 0 6d :4 0 6e
1 :8 0 1d1 1bd
1c0 1d2 0 1e08
d9 1d2 1d4 1d1
1d3 :6 0 1d0 :7 0
1d2 62 :3 0 6f
:a 0 3 1e5 :5 0
1d6 1d9 0 1d7
:3 0 70 :3 0 71
:3 0 72 :3 0 73
:3 0 6a :3 0 2d
:3 0 74 :3 0 46
:3 0 75 :3 0 72
:4 0 76 1 :8 0
1e6 1d6 1d9 1e7
0 1e08 db 1e7
1e9 1e6 1e8 :6 0
1e5 :7 0 1e7 1f1
1f2 0 dd 1b
:3 0 1c :2 0 4
1eb 1ec 0 1ed
:7 0 1f0 1ee 0
1e08 0 77 :6 0
78 :3 0 79 :3 0
78 :3 0 7a :3 0
1f4 1f5 0 2
:4 0 df 1f6 1f8
1f3 1fa 1f9 :2 0
7b :3 0 7c :3 0
1fc 1fd 0 7d
:4 0 78 :3 0 7e
:3 0 200 201 0
2 :4 0 e1 202
204 7f :4 0 80
:2 0 81 :3 0 e3
207 209 :3 0 82
:4 0 80 :2 0 83
:4 0 e6 20c 20e
:3 0 80 :2 0 3
:3 0 e9 210 212
:3 0 80 :2 0 83
:4 0 ec 214 216
:3 0 80 :2 0 84
:4 0 ef 218 21a
:3 0 80 :2 0 85
:3 0 66 :3 0 5
:3 0 f2 21e 220
86 :4 0 f4 21d
223 f7 21c 225
:3 0 80 :2 0 84
:4 0 fa 227 229
:3 0 80 :2 0 83
:4 0 fd 22b 22d
:3 0 80 :2 0 7
:3 0 100 22f 231
:3 0 80 :2 0 83
:4 0 103 233 235
:3 0 80 :2 0 84
:4 0 106 237 239
:3 0 80 :2 0 83
:4 0 109 23b 23d
:3 0 80 :2 0 8
:3 0 10c 23f 241
:3 0 80 :2 0 83
:4 0 10f 243 245
:3 0 80 :2 0 84
:4 0 112 247 249
:3 0 80 :2 0 83
:4 0 115 24b 24d
:3 0 80 :2 0 9
:3 0 118 24f 251
:3 0 80 :2 0 83
:4 0 11b 253 255
:3 0 80 :2 0 84
:4 0 11e 257 259
:3 0 80 :2 0 85
:3 0 66 :3 0 a
:3 0 121 25d 25f
86 :4 0 123 25c
262 126 25b 264
:3 0 80 :2 0 84
:4 0 129 266 268
:3 0 80 :2 0 83
:4 0 12c 26a 26c
:3 0 80 :2 0 b
:3 0 12f 26e 270
:3 0 80 :2 0 83
:4 0 132 272 274
:3 0 80 :2 0 84
:4 0 135 276 278
:3 0 80 :2 0 85
:3 0 66 :3 0 c
:3 0 138 27c 27e
86 :4 0 13a 27b
281 13d 27a 283
:3 0 80 :2 0 84
:4 0 140 285 287
:3 0 80 :2 0 85
:3 0 66 :3 0 d
:3 0 143 28b 28d
86 :4 0 145 28a
290 148 289 292
:3 0 80 :2 0 84
:4 0 14b 294 296
:3 0 80 :2 0 85
:3 0 66 :3 0 e
:3 0 14e 29a 29c
86 :4 0 150 299
29f 153 298 2a1
:3 0 80 :2 0 84
:4 0 156 2a3 2a5
:3 0 80 :2 0 85
:3 0 66 :3 0 f
:3 0 159 2a9 2ab
86 :4 0 15b 2a8
2ae 15e 2a7 2b0
:3 0 80 :2 0 84
:4 0 161 2b2 2b4
:3 0 80 :2 0 85
:3 0 66 :3 0 10
:3 0 164 2b8 2ba
86 :4 0 166 2b7
2bd 169 2b6 2bf
:3 0 80 :2 0 84
:4 0 16c 2c1 2c3
:3 0 80 :2 0 85
:3 0 66 :3 0 11
:3 0 16f 2c7 2c9
86 :4 0 171 2c6
2cc 174 2c5 2ce
:3 0 80 :2 0 84
:4 0 177 2d0 2d2
:3 0 80 :2 0 83
:4 0 17a 2d4 2d6
:3 0 80 :2 0 12
:3 0 17d 2d8 2da
:3 0 80 :2 0 83
:4 0 180 2dc 2de
:3 0 80 :2 0 84
:4 0 183 2e0 2e2
:3 0 80 :2 0 83
:4 0 186 2e4 2e6
:3 0 80 :2 0 13
:3 0 189 2e8 2ea
:3 0 80 :2 0 83
:4 0 18c 2ec 2ee
:3 0 80 :2 0 84
:4 0 18f 2f0 2f2
:3 0 80 :2 0 85
:3 0 66 :3 0 14
:3 0 192 2f6 2f8
86 :4 0 194 2f5
2fb 197 2f4 2fd
:3 0 80 :2 0 84
:4 0 19a 2ff 301
:3 0 80 :2 0 85
:3 0 66 :3 0 15
:3 0 19d 305 307
86 :4 0 19f 304
30a 1a2 303 30c
:3 0 80 :2 0 84
:4 0 1a5 30e 310
:3 0 80 :2 0 83
:4 0 1a8 312 314
:3 0 80 :2 0 16
:3 0 1ab 316 318
:3 0 80 :2 0 83
:4 0 1ae 31a 31c
:3 0 80 :2 0 84
:4 0 1b1 31e 320
:3 0 80 :2 0 83
:4 0 1b4 322 324
:3 0 80 :2 0 17
:3 0 1b7 326 328
:3 0 80 :2 0 83
:4 0 1ba 32a 32c
:3 0 80 :2 0 84
:4 0 1bd 32e 330
:3 0 80 :2 0 85
:3 0 66 :3 0 18
:3 0 1c0 334 336
86 :4 0 1c2 333
339 1c5 332 33b
:3 0 80 :2 0 84
:4 0 1c8 33d 33f
:3 0 80 :2 0 83
:4 0 1cb 341 343
:3 0 80 :2 0 19
:3 0 1ce 345 347
:3 0 80 :2 0 83
:4 0 1d1 349 34b
:3 0 80 :2 0 87
:4 0 1d4 34d 34f
:3 0 1d7 1fe 351
:2 0 353 1dc 354
1fb 353 0 355
1de 0 1d68 78
:3 0 79 :3 0 356
357 0 78 :3 0
88 :3 0 359 35a
0 49 :2 0 2
:4 0 89 :2 0 8a
:2 0 49 :2 0 1e0
35f 361 :3 0 1e3
35b 363 358 365
364 :2 0 7b :3 0
8b :3 0 367 368
0 7d :4 0 8c
:4 0 80 :2 0 81
:3 0 1e7 36c 36e
:3 0 1ea 369 370
:2 0 372 1ed 373
366 372 0 374
1ef 0 1d68 5
:3 0 8d :2 0 1f1
376 377 :3 0 378
:2 0 379 :2 0 85
:3 0 5 :3 0 31
:2 0 1f3 37b 37e
8e :2 0 49 :2 0
1f8 380 382 :3 0
383 :2 0 85 :3 0
5 :3 0 31 :2 0
1fb 385 388 8f
:2 0 90 :2 0 200
38a 38c :3 0 38d
:2 0 384 38f 38e
:2 0 390 :2 0 37a
392 391 :2 0 78
:3 0 79 :3 0 394
395 0 78 :3 0
88 :3 0 397 398
0 91 :2 0 2
:4 0 92 :2 0 8a
:2 0 49 :2 0 203
39d 39f :3 0 206
399 3a1 396 3a3
3a2 :2 0 7b :3 0
8b :3 0 3a5 3a6
0 7d :4 0 93
:4 0 80 :2 0 81
:3 0 20a 3aa 3ac
:3 0 80 :2 0 94
:4 0 20d 3ae 3b0
:3 0 80 :2 0 95
:4 0 210 3b2 3b4
:3 0 80 :2 0 85
:3 0 66 :3 0 96
:3 0 97 :3 0 3b9
3ba 0 213 3b8
3bc 86 :4 0 215
3b7 3bf 218 3b6
3c1 :3 0 21b 3a7
3c3 :2 0 3c5 21e
3c6 3a4 3c5 0
3c7 220 0 3f0
78 :3 0 79 :3 0
3c8 3c9 0 78
:3 0 88 :3 0 3cb
3cc 0 49 :2 0
2 :4 0 98 :2 0
8a :2 0 49 :2 0
222 3d1 3d3 :3 0
225 3cd 3d5 3ca
3d7 3d6 :2 0 7b
:3 0 8b :3 0 3d9
3da 0 7d :4 0
99 :4 0 80 :2 0
81 :3 0 229 3de
3e0 :3 0 22c 3db
3e2 :2 0 3e4 22f
3e5 3d8 3e4 0
3e6 231 0 3f0
9a :3 0 9b :3 0
3e7 3e8 0 96
:3 0 97 :3 0 3ea
3eb 0 9c :3 0
233 3e9 3ee :2 0
3f0 236 3f1 393
3f0 0 3f2 23a
0 1d68 3 :3 0
8d :2 0 23c 3f4
3f5 :3 0 3f6 :2 0
3f7 :2 0 f :3 0
8d :2 0 23e 3fa
3fb :3 0 3fc :2 0
3fd :2 0 3f8 3ff
3fe :2 0 19 :3 0
8d :2 0 240 402
403 :3 0 404 :2 0
405 :2 0 400 407
406 :2 0 78 :3 0
79 :3 0 409 40a
0 78 :3 0 88
:3 0 40c 40d 0
91 :2 0 2 :4 0
9d :2 0 8a :2 0
49 :2 0 242 412
414 :3 0 245 40e
416 40b 418 417
:2 0 7b :3 0 8b
:3 0 41a 41b 0
7d :4 0 9e :4 0
80 :2 0 81 :3 0
249 41f 421 :3 0
80 :2 0 94 :4 0
24c 423 425 :3 0
80 :2 0 95 :4 0
24f 427 429 :3 0
80 :2 0 85 :3 0
66 :3 0 96 :3 0
97 :3 0 42e 42f
0 252 42d 431
86 :4 0 254 42c
434 257 42b 436
:3 0 25a 41c 438
:2 0 43a 25d 43b
419 43a 0 43c
25f 0 465 78
:3 0 79 :3 0 43d
43e 0 78 :3 0
88 :3 0 440 441
0 49 :2 0 2
:4 0 9f :2 0 8a
:2 0 49 :2 0 261
446 448 :3 0 264
442 44a 43f 44c
44b :2 0 7b :3 0
8b :3 0 44e 44f
0 7d :4 0 a0
:4 0 80 :2 0 81
:3 0 268 453 455
:3 0 26b 450 457
:2 0 459 26e 45a
44d 459 0 45b
270 0 465 9a
:3 0 9b :3 0 45c
45d 0 96 :3 0
97 :3 0 45f 460
0 9c :3 0 272
45e 463 :2 0 465
275 466 408 465
0 467 279 0
1d68 4d :4 0 468
469 0 1d68 4f
:4 0 46b 46c 0
1d68 32 :3 0 a1
:2 0 46e 46f 0
1d68 4c :3 0 8
:3 0 471 472 0
1d68 4e :3 0 9
:3 0 474 475 0
1d68 51 :3 0 b
:3 0 477 478 0
1d68 2f :3 0 19
:3 0 47a 47b 0
1d68 3d :3 0 a
:3 0 47d 47e 0
1d68 41 :3 0 f
:3 0 480 481 0
1d68 57 :3 0 5
:3 0 483 484 0
1d68 39 :3 0 10
:3 0 486 487 0
1d68 4d :3 0 16
:3 0 489 48a 0
1d68 42 :3 0 c
:3 0 48c 48d 0
1d68 55 :3 0 31
:2 0 48f 490 0
1d68 85 :3 0 a2
:3 0 51 :3 0 27b
493 495 31 :2 0
27d 492 498 96
:3 0 8e :2 0 a3
:3 0 49a 49c 0
282 49b 49e :3 0
85 :3 0 57 :3 0
31 :2 0 285 4a0
4a3 a4 :2 0 49
:2 0 28a 4a5 4a7
:3 0 4a8 :2 0 85
:3 0 57 :3 0 31
:2 0 28d 4aa 4ad
a4 :2 0 90 :2 0
292 4af 4b1 :3 0
4b2 :2 0 4a9 4b4
4b3 :2 0 7 :3 0
8d :2 0 295 4b7
4b8 :3 0 4b9 :2 0
78 :3 0 79 :3 0
4bb 4bc 0 78
:3 0 88 :3 0 4be
4bf 0 91 :2 0
2 :4 0 a5 :2 0
8a :2 0 49 :2 0
297 4c4 4c6 :3 0
29a 4c0 4c8 4bd
4ca 4c9 :2 0 7b
:3 0 8b :3 0 4cc
4cd 0 7d :4 0
a6 :4 0 80 :2 0
81 :3 0 29e 4d1
4d3 :3 0 80 :2 0
94 :4 0 2a1 4d5
4d7 :3 0 80 :2 0
95 :4 0 2a4 4d9
4db :3 0 80 :2 0
85 :3 0 66 :3 0
96 :3 0 97 :3 0
4e0 4e1 0 2a7
4df 4e3 86 :4 0
2a9 4de 4e6 2ac
4dd 4e8 :3 0 2af
4ce 4ea :2 0 4ec
2b2 4ed 4cb 4ec
0 4ee 2b4 0
517 78 :3 0 79
:3 0 4ef 4f0 0
78 :3 0 88 :3 0
4f2 4f3 0 49
:2 0 2 :4 0 a7
:2 0 8a :2 0 49
:2 0 2b6 4f8 4fa
:3 0 2b9 4f4 4fc
4f1 4fe 4fd :2 0
7b :3 0 8b :3 0
500 501 0 7d
:4 0 a8 :4 0 80
:2 0 81 :3 0 2bd
505 507 :3 0 2c0
502 509 :2 0 50b
2c3 50c 4ff 50b
0 50d 2c5 0
517 9a :3 0 9b
:3 0 50e 50f 0
96 :3 0 97 :3 0
511 512 0 9c
:3 0 2c7 510 515
:2 0 517 2ca 518
4ba 517 0 519
2ce 0 6c7 2c
:3 0 3 :3 0 51a
51b 0 6c7 2e
:3 0 7 :3 0 51d
51e 0 6c7 2f
:3 0 19 :3 0 520
521 0 6c7 6a
:3 0 a9 :3 0 2d
:3 0 30 :3 0 69
:3 0 6b :3 0 2e
:4 0 aa 1 :8 0
530 1b :3 0 ab
:3 0 52b 52c 0
ac :3 0 52d 52e
0 530 2d0 583
ad :3 0 1b :3 0
ab :3 0 532 533
0 ac :3 0 534
535 0 537 2d3
539 2d5 538 537
:2 0 581 ae :3 0
1b :3 0 ab :3 0
53c 53d 0 ac
:3 0 53e 53f 0
579 78 :3 0 79
:3 0 541 542 0
78 :3 0 88 :3 0
544 545 0 25
:2 0 2 :4 0 af
:2 0 8a :2 0 49
:2 0 2d7 54a 54c
:3 0 2da 546 54e
543 550 54f :2 0
7b :3 0 8b :3 0
552 553 0 7d
:4 0 b0 :4 0 80
:2 0 81 :3 0 2de
557 559 :3 0 80
:2 0 94 :4 0 2e1
55b 55d :3 0 80
:2 0 b1 :4 0 2e4
55f 561 :3 0 80
:2 0 ac :3 0 2e7
563 565 :3 0 80
:2 0 b2 :4 0 2ea
567 569 :3 0 80
:2 0 b3 :3 0 b4
:3 0 49 :2 0 b5
:2 0 2ed 56c 570
2f1 56b 572 :3 0
2f4 554 574 :2 0
576 2f7 577 551
576 0 578 2f9
0 579 2fb 57c
:3 0 57c 0 57c
57b 579 57a :6 0
57e 4 :3 0 2fe
580 300 57f 57e
:2 0 581 302 :2 0
583 0 583 582
530 581 :6 0 6c7
1 :3 0 2b :3 0
b6 :4 0 586 587
:3 0 585 588 0
6c7 52 :3 0 b7
:3 0 85 :3 0 30
:3 0 31 :2 0 305
58c 58f 308 58b
591 58a 592 0
6c7 1b :3 0 ab
:3 0 594 595 0
b8 :3 0 52 :3 0
30 :3 0 2e :3 0
2c :3 0 2f :3 0
2b :3 0 30a 597
59e 596 59f 0
6c7 1b :3 0 ab
:3 0 5a1 5a2 0
a4 :2 0 31 :2 0
313 5a4 5a6 :3 0
5a7 :2 0 85 :3 0
57 :3 0 31 :2 0
316 5a9 5ac a4
:2 0 90 :2 0 31b
5ae 5b0 :3 0 5b1
:2 0 5a8 5b3 5b2
:2 0 85 :3 0 3d
:3 0 31 :2 0 31e
5b5 5b8 a4 :2 0
31 :2 0 323 5ba
5bc :3 0 5bd :2 0
5b4 5bf 5be :2 0
53 :3 0 31 :2 0
5c1 5c2 0 6a5
b9 :3 0 53 :3 0
ba :3 0 bb :3 0
6a :3 0 2d :3 0
bc :3 0 2c :4 0
bd 1 :8 0 5d2
1b :3 0 ab :3 0
5cd 5ce 0 ac
:3 0 5cf 5d0 0
5d2 326 625 ad
:3 0 1b :3 0 ab
:3 0 5d4 5d5 0
ac :3 0 5d6 5d7
0 5d9 329 5db
32b 5da 5d9 :2 0
623 ae :3 0 1b
:3 0 ab :3 0 5de
5df 0 ac :3 0
5e0 5e1 0 61b
78 :3 0 79 :3 0
5e3 5e4 0 78
:3 0 88 :3 0 5e6
5e7 0 25 :2 0
2 :4 0 be :2 0
8a :2 0 49 :2 0
32d 5ec 5ee :3 0
330 5e8 5f0 5e5
5f2 5f1 :2 0 7b
:3 0 8b :3 0 5f4
5f5 0 7d :4 0
bf :4 0 80 :2 0
81 :3 0 334 5f9
5fb :3 0 80 :2 0
94 :4 0 337 5fd
5ff :3 0 80 :2 0
b1 :4 0 33a 601
603 :3 0 80 :2 0
ac :3 0 33d 605
607 :3 0 80 :2 0
b2 :4 0 340 609
60b :3 0 80 :2 0
b3 :3 0 b4 :3 0
49 :2 0 b5 :2 0
343 60e 612 347
60d 614 :3 0 34a
5f6 616 :2 0 618
34d 619 5f3 618
0 61a 34f 0
61b 351 61e :3 0
61e 0 61e 61d
61b 61c :6 0 620
6 :3 0 354 622
356 621 620 :2 0
623 358 :2 0 625
0 625 624 5d2
623 :6 0 6a5 1
:3 0 85 :3 0 53
:3 0 31 :2 0 35b
627 62a c0 :2 0
31 :2 0 360 62c
62e :3 0 57 :3 0
49 :2 0 630 631
0 6a2 3d :3 0
c1 :2 0 633 634
0 6a2 51 :3 0
c2 :4 0 636 637
0 6a2 4c :3 0
c3 :2 0 363 63a
63b :3 0 63c :2 0
c4 :3 0 c5 :3 0
c6 :3 0 4c :3 0
4e :3 0 ba :4 0
c7 1 :8 0 64a
1b :3 0 ab :3 0
645 646 0 ac
:3 0 647 648 0
64a 365 69d ad
:3 0 1b :3 0 ab
:3 0 64c 64d 0
ac :3 0 64e 64f
0 651 368 653
36a 652 651 :2 0
69b ae :3 0 1b
:3 0 ab :3 0 656
657 0 ac :3 0
658 659 0 693
78 :3 0 79 :3 0
65b 65c 0 78
:3 0 88 :3 0 65e
65f 0 25 :2 0
2 :4 0 c8 :2 0
8a :2 0 49 :2 0
36c 664 666 :3 0
36f 660 668 65d
66a 669 :2 0 7b
:3 0 8b :3 0 66c
66d 0 7d :4 0
c9 :4 0 80 :2 0
81 :3 0 373 671
673 :3 0 80 :2 0
94 :4 0 376 675
677 :3 0 80 :2 0
b1 :4 0 379 679
67b :3 0 80 :2 0
ac :3 0 37c 67d
67f :3 0 80 :2 0
b2 :4 0 37f 681
683 :3 0 80 :2 0
b3 :3 0 b4 :3 0
49 :2 0 b5 :2 0
382 686 68a 386
685 68c :3 0 389
66e 68e :2 0 690
38c 691 66b 690
0 692 38e 0
693 390 696 :3 0
696 0 696 695
693 694 :6 0 698
8 :3 0 393 69a
395 699 698 :2 0
69b 397 :2 0 69d
0 69d 69c 64a
69b :6 0 69f 1
:3 0 39a 6a0 63d
69f 0 6a1 39c
0 6a2 39e 6a3
62f 6a2 0 6a4
3a3 0 6a5 3a5
6a6 5c0 6a5 0
6a7 3a9 0 6c7
85 :3 0 57 :3 0
31 :2 0 3ab 6a8
6ab a4 :2 0 49
:2 0 3b0 6ad 6af
:3 0 6b0 :2 0 ca
:3 0 85 :3 0 30
:3 0 31 :2 0 3b3
6b3 6b6 3b6 6b2
6b8 6b9 :2 0 c0
:2 0 31 :2 0 3ba
6bb 6bd :3 0 6b1
6bf 6be :2 0 57
:3 0 90 :2 0 6c1
6c2 0 6c4 3bd
6c5 6c0 6c4 0
6c6 3bf 0 6c7
3c1 764 2b :3 0
3 :3 0 6c8 6c9
0 763 6a :3 0
cb :3 0 cc :3 0
cd :3 0 ce :3 0
cf :3 0 d0 :3 0
d1 :3 0 d2 :3 0
6c :3 0 2d :3 0
2e :3 0 2c :3 0
42 :3 0 47 :3 0
61 :3 0 36 :3 0
37 :3 0 38 :3 0
4f :3 0 d3 :3 0
74 :3 0 2b :4 0
d4 1 :8 0 6e8
1b :3 0 ab :3 0
6e3 6e4 0 ac
:3 0 6e5 6e6 0
6e8 3cc 73b ad
:3 0 1b :3 0 ab
:3 0 6ea 6eb 0
ac :3 0 6ec 6ed
0 6ef 3cf 6f1
3d1 6f0 6ef :2 0
739 ae :3 0 1b
:3 0 ab :3 0 6f4
6f5 0 ac :3 0
6f6 6f7 0 731
78 :3 0 79 :3 0
6f9 6fa 0 78
:3 0 88 :3 0 6fc
6fd 0 25 :2 0
2 :4 0 d5 :2 0
8a :2 0 49 :2 0
3d3 702 704 :3 0
3d6 6fe 706 6fb
708 707 :2 0 7b
:3 0 8b :3 0 70a
70b 0 7d :4 0
d6 :4 0 80 :2 0
81 :3 0 3da 70f
711 :3 0 80 :2 0
94 :4 0 3dd 713
715 :3 0 80 :2 0
b1 :4 0 3e0 717
719 :3 0 80 :2 0
ac :3 0 3e3 71b
71d :3 0 80 :2 0
b2 :4 0 3e6 71f
721 :3 0 80 :2 0
b3 :3 0 b4 :3 0
49 :2 0 b5 :2 0
3e9 724 728 3ed
723 72a :3 0 3f0
70c 72c :2 0 72e
3f3 72f 709 72e
0 730 3f5 0
731 3f7 734 :3 0
734 0 734 733
731 732 :6 0 736
a :3 0 3fa 738
3fc 737 736 :2 0
739 3fe :2 0 73b
0 73b 73a 6e8
739 :6 0 763 1
:3 0 1b :3 0 ab
:3 0 73d 73e 0
1b :3 0 a4 :2 0
d7 :3 0 740 742
0 403 741 744
:3 0 745 :2 0 85
:3 0 c :3 0 31
:2 0 406 747 74a
85 :3 0 c0 :2 0
42 :3 0 31 :2 0
409 74c 750 40e
74d 752 :3 0 753
:2 0 746 755 754
:2 0 4d :4 0 757
758 0 760 3e
:3 0 d8 :2 0 49
:2 0 411 75b 75d
:3 0 75a 75e 0
760 413 761 756
760 0 762 416
0 763 418 765
4b5 6c7 0 766
0 763 0 766
41c 0 1d17 85
:3 0 57 :3 0 31
:2 0 41f 767 76a
c0 :2 0 90 :2 0
424 76c 76e :3 0
1b :3 0 ab :3 0
770 771 0 c0
:2 0 31 :2 0 429
773 775 :3 0 78
:3 0 79 :3 0 777
778 0 78 :3 0
88 :3 0 77a 77b
0 27 :2 0 2
:4 0 d9 :2 0 8a
:2 0 49 :2 0 42c
780 782 :3 0 42f
77c 784 779 786
785 :2 0 7b :3 0
8b :3 0 788 789
0 7d :4 0 da
:4 0 80 :2 0 81
:3 0 433 78d 78f
:3 0 80 :2 0 94
:4 0 436 791 793
:3 0 80 :2 0 95
:4 0 439 795 797
:3 0 80 :2 0 85
:3 0 66 :3 0 1b
:3 0 ab :3 0 79c
79d 0 43c 79b
79f 86 :4 0 43e
79a 7a2 441 799
7a4 :3 0 80 :2 0
db :4 0 444 7a6
7a8 :3 0 80 :2 0
83 :4 0 447 7aa
7ac :3 0 80 :2 0
2b :3 0 44a 7ae
7b0 :3 0 80 :2 0
83 :4 0 44d 7b2
7b4 :3 0 450 78a
7b6 :2 0 7b8 453
7b9 787 7b8 0
7ba 455 0 7e4
78 :3 0 79 :3 0
7bb 7bc 0 78
:3 0 88 :3 0 7be
7bf 0 49 :2 0
2 :4 0 dc :2 0
8a :2 0 49 :2 0
457 7c4 7c6 :3 0
45a 7c0 7c8 7bd
7ca 7c9 :2 0 7b
:3 0 8b :3 0 7cc
7cd 0 7d :4 0
dd :4 0 80 :2 0
81 :3 0 45e 7d1
7d3 :3 0 461 7ce
7d5 :2 0 7d7 464
7d8 7cb 7d7 0
7d9 466 0 7e4
9a :3 0 de :3 0
7da 7db 0 1b
:3 0 ab :3 0 7dd
7de 0 2b :3 0
9c :3 0 468 7dc
7e2 :2 0 7e4 46c
7e5 776 7e4 0
7e6 470 0 1d05
4c :3 0 8d :2 0
472 7e8 7e9 :3 0
7ea :2 0 4e :3 0
8d :2 0 474 7ed
7ee :3 0 7ef :2 0
7eb 7f1 7f0 :2 0
3d :3 0 8d :2 0
476 7f4 7f5 :3 0
7f6 :2 0 7f2 7f8
7f7 :2 0 51 :3 0
8d :2 0 478 7fb
7fc :3 0 7fd :2 0
7f9 7ff 7fe :2 0
78 :3 0 79 :3 0
801 802 0 78
:3 0 88 :3 0 804
805 0 91 :2 0
2 :4 0 df :2 0
8a :2 0 49 :2 0
47a 80a 80c :3 0
47d 806 80e 803
810 80f :2 0 7b
:3 0 8b :3 0 812
813 0 7d :4 0
e0 :4 0 80 :2 0
81 :3 0 481 817
819 :3 0 80 :2 0
94 :4 0 484 81b
81d :3 0 80 :2 0
95 :4 0 487 81f
821 :3 0 80 :2 0
85 :3 0 66 :3 0
96 :3 0 97 :3 0
826 827 0 48a
825 829 86 :4 0
48c 824 82c 48f
823 82e :3 0 492
814 830 :2 0 832
495 833 811 832
0 834 497 0
85d 78 :3 0 79
:3 0 835 836 0
78 :3 0 88 :3 0
838 839 0 49
:2 0 2 :4 0 e1
:2 0 8a :2 0 49
:2 0 499 83e 840
:3 0 49c 83a 842
837 844 843 :2 0
7b :3 0 8b :3 0
846 847 0 7d
:4 0 e2 :4 0 80
:2 0 81 :3 0 4a0
84b 84d :3 0 4a3
848 84f :2 0 851
4a6 852 845 851
0 853 4a8 0
85d 9a :3 0 9b
:3 0 854 855 0
96 :3 0 97 :3 0
857 858 0 9c
:3 0 4aa 856 85b
:2 0 85d 4ad 85e
800 85d 0 85f
4b1 0 1d05 3c
:3 0 96 :3 0 e3
:3 0 861 862 0
860 863 0 1d05
85 :3 0 39 :3 0
31 :2 0 4b3 865
868 8f :2 0 e4
:2 0 4b8 86a 86c
:3 0 39 :3 0 e4
:2 0 86e 86f 0
871 4bb 872 86d
871 0 873 4bd
0 1d05 85 :3 0
e5 :3 0 3a :3 0
69 :3 0 6b :3 0
2e :4 0 e6 1
:8 0 880 1b :3 0
ab :3 0 87b 87c
0 ac :3 0 87d
87e 0 880 4bf
8d3 ad :3 0 1b
:3 0 ab :3 0 882
883 0 ac :3 0
884 885 0 887
4c2 889 4c4 888
887 :2 0 8d1 ae
:3 0 1b :3 0 ab
:3 0 88c 88d 0
ac :3 0 88e 88f
0 8c9 78 :3 0
79 :3 0 891 892
0 78 :3 0 88
:3 0 894 895 0
25 :2 0 2 :4 0
e7 :2 0 8a :2 0
49 :2 0 4c6 89a
89c :3 0 4c9 896
89e 893 8a0 89f
:2 0 7b :3 0 8b
:3 0 8a2 8a3 0
7d :4 0 e8 :4 0
80 :2 0 81 :3 0
4cd 8a7 8a9 :3 0
80 :2 0 94 :4 0
4d0 8ab 8ad :3 0
80 :2 0 b1 :4 0
4d3 8af 8b1 :3 0
80 :2 0 ac :3 0
4d6 8b3 8b5 :3 0
80 :2 0 b2 :4 0
4d9 8b7 8b9 :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 4dc 8bc
8c0 4e0 8bb 8c2
:3 0 4e3 8a4 8c4
:2 0 8c6 4e6 8c7
8a1 8c6 0 8c8
4e8 0 8c9 4ea
8cc :3 0 8cc 0
8cc 8cb 8c9 8ca
:6 0 8ce c :3 0
4ed 8d0 4ef 8cf
8ce :2 0 8d1 4f1
:2 0 8d3 0 8d3
8d2 880 8d1 :6 0
1d05 1 :3 0 85
:3 0 36 :3 0 31
:2 0 4f4 8d5 8d8
8f :2 0 31 :2 0
4f9 8da 8dc :3 0
53 :3 0 e9 :3 0
85 :3 0 ea :3 0
36 :3 0 4fc 8e1
8e3 31 :2 0 4fe
8e0 8e6 eb :2 0
85 :3 0 ea :3 0
3a :3 0 501 8ea
8ec 31 :2 0 503
8e9 8ef 506 8e8
8f1 :3 0 8f2 :2 0
ec :2 0 ed :2 0
509 8f4 8f6 :3 0
8f7 :2 0 8a :2 0
ee :2 0 50c 8f9
8fb :3 0 50f 8df
8fd 8de 8fe 0
91b 85 :3 0 39
:3 0 31 :2 0 511
900 903 85 :3 0
c0 :2 0 53 :3 0
31 :2 0 514 905
909 519 906 90b
:3 0 36 :3 0 31
:2 0 90d 90e 0
913 37 :3 0 31
:2 0 910 911 0
913 51c 918 39
:3 0 31 :2 0 914
915 0 917 51f
919 90c 913 0
91a 0 917 0
91a 521 0 91b
524 91c 8dd 91b
0 91d 527 0
1d05 48 :4 0 91e
91f 0 1d05 ef
:3 0 85 :3 0 f
:3 0 31 :2 0 529
922 925 25 :2 0
52c 921 928 929
:2 0 a4 :2 0 25
:2 0 531 92b 92d
:3 0 85 :3 0 57
:3 0 31 :2 0 534
92f 932 a4 :2 0
49 :2 0 539 934
936 :3 0 85 :3 0
c :3 0 31 :2 0
53c 938 93b 8f
:2 0 31 :2 0 541
93d 93f :3 0 1b
:3 0 ab :3 0 941
942 0 f0 :3 0
2b :3 0 48 :3 0
48 :3 0 32 :3 0
4c :3 0 4e :3 0
3d :3 0 36 :3 0
33 :3 0 33 :3 0
34 :3 0 3c :3 0
37 :3 0 38 :3 0
55 :3 0 51 :3 0
2f :3 0 544 944
956 943 957 0
9dd 1b :3 0 ab
:3 0 959 95a 0
a4 :2 0 31 :2 0
558 95c 95e :3 0
d3 :3 0 f1 :3 0
e :3 0 f2 :3 0
f3 :3 0 f4 :3 0
f5 :3 0 f6 :3 0
f7 :3 0 6a :3 0
2d :3 0 cb :3 0
2e :3 0 cc :3 0
2c :4 0 f8 1
:8 0 985 f9 :4 0
970 :3 0 9c :3 0
a4 :2 0 55d 973
974 :3 0 1b :3 0
ab :3 0 976 977
0 31 :2 0 978
979 0 97b 560
982 1b :3 0 ab
:3 0 97c 97d 0
fa :2 0 97e 97f
0 981 562 983
975 97b 0 984
0 981 0 984
564 0 985 567
9d8 ad :3 0 1b
:3 0 ab :3 0 987
988 0 ac :3 0
989 98a 0 98c
56a 98e 56c 98d
98c :2 0 9d6 ae
:3 0 1b :3 0 ab
:3 0 991 992 0
ac :3 0 993 994
0 9ce 78 :3 0
79 :3 0 996 997
0 78 :3 0 88
:3 0 999 99a 0
25 :2 0 2 :4 0
fb :2 0 8a :2 0
49 :2 0 56e 99f
9a1 :3 0 571 99b
9a3 998 9a5 9a4
:2 0 7b :3 0 8b
:3 0 9a7 9a8 0
7d :4 0 fc :4 0
80 :2 0 81 :3 0
575 9ac 9ae :3 0
80 :2 0 94 :4 0
578 9b0 9b2 :3 0
80 :2 0 b1 :4 0
57b 9b4 9b6 :3 0
80 :2 0 ac :3 0
57e 9b8 9ba :3 0
80 :2 0 b2 :4 0
581 9bc 9be :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 584 9c1
9c5 588 9c0 9c7
:3 0 58b 9a9 9c9
:2 0 9cb 58e 9cc
9a6 9cb 0 9cd
590 0 9ce 592
9d1 :3 0 9d1 0
9d1 9d0 9ce 9cf
:6 0 9d3 e :3 0
595 9d5 597 9d4
9d3 :2 0 9d6 599
:2 0 9d8 0 9d8
9d7 985 9d6 :6 0
9da 1 :3 0 59c
9db 95f 9da 0
9dc 59e 0 9dd
5a0 9f7 1b :3 0
ab :3 0 9de 9df
0 f0 :3 0 2b
:3 0 48 :3 0 48
:3 0 32 :3 0 4c
:3 0 4e :3 0 3d
:3 0 36 :3 0 33
:3 0 33 :3 0 33
:3 0 3c :3 0 37
:3 0 38 :3 0 55
:3 0 51 :3 0 2f
:3 0 5a3 9e1 9f3
9e0 9f4 0 9f6
5b5 9f8 940 9dd
0 9f9 0 9f6
0 9f9 5b7 0
9fa 5ba ac5 3c
:3 0 85 :3 0 3c
:3 0 31 :2 0 5bc
9fc 9ff 8a :2 0
49 :2 0 5bf a01
a03 :3 0 9fb a04
0 ac4 85 :3 0
c :3 0 31 :2 0
5c2 a06 a09 8f
:2 0 31 :2 0 5c7
a0b a0d :3 0 1b
:3 0 ab :3 0 a0f
a10 0 f0 :3 0
2b :3 0 48 :3 0
48 :3 0 32 :3 0
4c :3 0 4e :3 0
3d :3 0 36 :3 0
33 :3 0 33 :3 0
34 :3 0 3c :3 0
37 :3 0 38 :3 0
55 :3 0 51 :3 0
2f :3 0 5ca a12
a24 a11 a25 0
aa7 1b :3 0 ab
:3 0 a27 a28 0
a4 :2 0 31 :2 0
5de a2a a2c :3 0
d3 :3 0 f1 :3 0
e :3 0 f2 :3 0
f3 :3 0 f4 :3 0
f5 :3 0 f6 :3 0
f7 :3 0 74 :3 0
2b :4 0 fd 1
:8 0 a4f f9 :4 0
a3a :3 0 9c :3 0
a4 :2 0 5e3 a3d
a3e :3 0 1b :3 0
ab :3 0 a40 a41
0 31 :2 0 a42
a43 0 a45 5e6
a4c 1b :3 0 ab
:3 0 a46 a47 0
fa :2 0 a48 a49
0 a4b 5e8 a4d
a3f a45 0 a4e
0 a4b 0 a4e
5ea 0 a4f 5ed
aa2 ad :3 0 1b
:3 0 ab :3 0 a51
a52 0 ac :3 0
a53 a54 0 a56
5f0 a58 5f2 a57
a56 :2 0 aa0 ae
:3 0 1b :3 0 ab
:3 0 a5b a5c 0
ac :3 0 a5d a5e
0 a98 78 :3 0
79 :3 0 a60 a61
0 78 :3 0 88
:3 0 a63 a64 0
25 :2 0 2 :4 0
fe :2 0 8a :2 0
49 :2 0 5f4 a69
a6b :3 0 5f7 a65
a6d a62 a6f a6e
:2 0 7b :3 0 8b
:3 0 a71 a72 0
7d :4 0 ff :4 0
80 :2 0 81 :3 0
5fb a76 a78 :3 0
80 :2 0 94 :4 0
5fe a7a a7c :3 0
80 :2 0 b1 :4 0
601 a7e a80 :3 0
80 :2 0 ac :3 0
604 a82 a84 :3 0
80 :2 0 b2 :4 0
607 a86 a88 :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 60a a8b
a8f 60e a8a a91
:3 0 611 a73 a93
:2 0 a95 614 a96
a70 a95 0 a97
616 0 a98 618
a9b :3 0 a9b 0
a9b a9a a98 a99
:6 0 a9d 10 :3 0
61b a9f 61d a9e
a9d :2 0 aa0 61f
:2 0 aa2 0 aa2
aa1 a4f aa0 :6 0
aa4 1 :3 0 622
aa5 a2d aa4 0
aa6 624 0 aa7
626 ac1 1b :3 0
ab :3 0 aa8 aa9
0 f0 :3 0 2b
:3 0 48 :3 0 48
:3 0 32 :3 0 4c
:3 0 4e :3 0 3d
:3 0 36 :3 0 33
:3 0 33 :3 0 33
:3 0 3c :3 0 37
:3 0 38 :3 0 55
:3 0 51 :3 0 2f
:3 0 629 aab abd
aaa abe 0 ac0
63b ac2 a0e aa7
0 ac3 0 ac0
0 ac3 63d 0
ac4 640 ac6 937
9fa 0 ac7 0
ac4 0 ac7 643
0 cea 1b :3 0
ab :3 0 ac8 ac9
0 a4 :2 0 31
:2 0 648 acb acd
:3 0 ace :2 0 ef
:3 0 85 :3 0 f
:3 0 31 :2 0 64b
ad1 ad4 35 :2 0
64e ad0 ad7 ad8
:2 0 a4 :2 0 35
:2 0 653 ada adc
:3 0 add :2 0 acf
adf ade :2 0 85
:3 0 c :3 0 31
:2 0 656 ae1 ae4
85 :3 0 a4 :2 0
a :3 0 31 :2 0
659 ae6 aea 65e
ae7 aec :3 0 aed
:2 0 85 :3 0 e
:3 0 31 :2 0 661
aef af2 a4 :2 0
31 :2 0 666 af4
af6 :3 0 af7 :2 0
aee af9 af8 :2 0
85 :3 0 d :3 0
31 :2 0 669 afb
afe a4 :2 0 ed
:2 0 66e b00 b02
:3 0 b03 :2 0 afa
b05 b04 :2 0 d3
:3 0 100 :3 0 d
:3 0 cf :3 0 39
:3 0 d2 :3 0 38
:3 0 f2 :3 0 f3
:3 0 f4 :3 0 f5
:3 0 f6 :3 0 f7
:3 0 101 :3 0 4c
:3 0 102 :3 0 4e
:3 0 cd :3 0 c
:3 0 f1 :3 0 e
:3 0 74 :3 0 2b
:4 0 103 1 :8 0
b34 f9 :4 0 b1f
:3 0 9c :3 0 a4
:2 0 673 b22 b23
:3 0 1b :3 0 ab
:3 0 b25 b26 0
31 :2 0 b27 b28
0 b2a 676 b31
1b :3 0 ab :3 0
b2b b2c 0 fa
:2 0 b2d b2e 0
b30 678 b32 b24
b2a 0 b33 0
b30 0 b33 67a
0 b34 67d b87
ad :3 0 1b :3 0
ab :3 0 b36 b37
0 ac :3 0 b38
b39 0 b3b 680
b3d 682 b3c b3b
:2 0 b85 ae :3 0
1b :3 0 ab :3 0
b40 b41 0 ac
:3 0 b42 b43 0
b7d 78 :3 0 79
:3 0 b45 b46 0
78 :3 0 88 :3 0
b48 b49 0 25
:2 0 2 :4 0 104
:2 0 8a :2 0 49
:2 0 684 b4e b50
:3 0 687 b4a b52
b47 b54 b53 :2 0
7b :3 0 8b :3 0
b56 b57 0 7d
:4 0 105 :4 0 80
:2 0 81 :3 0 68b
b5b b5d :3 0 80
:2 0 94 :4 0 68e
b5f b61 :3 0 80
:2 0 b1 :4 0 691
b63 b65 :3 0 80
:2 0 ac :3 0 694
b67 b69 :3 0 80
:2 0 b2 :4 0 697
b6b b6d :3 0 80
:2 0 b3 :3 0 b4
:3 0 49 :2 0 b5
:2 0 69a b70 b74
69e b6f b76 :3 0
6a1 b58 b78 :2 0
b7a 6a4 b7b b55
b7a 0 b7c 6a6
0 b7d 6a8 b80
:3 0 b80 0 b80
b7f b7d b7e :6 0
b82 12 :3 0 6ab
b84 6ad b83 b82
:2 0 b85 6af :2 0
b87 0 b87 b86
b34 b85 :6 0 c6c
1 :3 0 73 :3 0
75 :3 0 70 :3 0
72 :3 0 6a :3 0
2d :3 0 74 :3 0
2b :4 0 106 1
:8 0 ba7 f9 :4 0
b92 :3 0 9c :3 0
a4 :2 0 6b4 b95
b96 :3 0 1b :3 0
ab :3 0 b98 b99
0 31 :2 0 b9a
b9b 0 b9d 6b7
ba4 1b :3 0 ab
:3 0 b9e b9f 0
fa :2 0 ba0 ba1
0 ba3 6b9 ba5
b97 b9d 0 ba6
0 ba3 0 ba6
6bb 0 ba7 6be
bfa ad :3 0 1b
:3 0 ab :3 0 ba9
baa 0 ac :3 0
bab bac 0 bae
6c1 bb0 6c3 baf
bae :2 0 bf8 ae
:3 0 1b :3 0 ab
:3 0 bb3 bb4 0
ac :3 0 bb5 bb6
0 bf0 78 :3 0
79 :3 0 bb8 bb9
0 78 :3 0 88
:3 0 bbb bbc 0
25 :2 0 2 :4 0
107 :2 0 8a :2 0
49 :2 0 6c5 bc1
bc3 :3 0 6c8 bbd
bc5 bba bc7 bc6
:2 0 7b :3 0 8b
:3 0 bc9 bca 0
7d :4 0 108 :4 0
80 :2 0 81 :3 0
6cc bce bd0 :3 0
80 :2 0 94 :4 0
6cf bd2 bd4 :3 0
80 :2 0 b1 :4 0
6d2 bd6 bd8 :3 0
80 :2 0 ac :3 0
6d5 bda bdc :3 0
80 :2 0 b2 :4 0
6d8 bde be0 :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 6db be3
be7 6df be2 be9
:3 0 6e2 bcb beb
:2 0 bed 6e5 bee
bc8 bed 0 bef
6e7 0 bf0 6e9
bf3 :3 0 bf3 0
bf3 bf2 bf0 bf1
:6 0 bf5 14 :3 0
6ec bf7 6ee bf6
bf5 :2 0 bf8 6f0
:2 0 bfa 0 bfa
bf9 ba7 bf8 :6 0
c6c 1 :3 0 d3
:3 0 109 :3 0 4e
:3 0 74 :3 0 2b
:4 0 10a 1 :8 0
c17 f9 :4 0 c02
:3 0 9c :3 0 a4
:2 0 6f5 c05 c06
:3 0 1b :3 0 ab
:3 0 c08 c09 0
31 :2 0 c0a c0b
0 c0d 6f8 c14
1b :3 0 ab :3 0
c0e c0f 0 fa
:2 0 c10 c11 0
c13 6fa c15 c07
c0d 0 c16 0
c13 0 c16 6fc
0 c17 6ff c6a
ad :3 0 1b :3 0
ab :3 0 c19 c1a
0 ac :3 0 c1b
c1c 0 c1e 702
c20 704 c1f c1e
:2 0 c68 ae :3 0
1b :3 0 ab :3 0
c23 c24 0 ac
:3 0 c25 c26 0
c60 78 :3 0 79
:3 0 c28 c29 0
78 :3 0 88 :3 0
c2b c2c 0 25
:2 0 2 :4 0 10b
:2 0 8a :2 0 49
:2 0 706 c31 c33
:3 0 709 c2d c35
c2a c37 c36 :2 0
7b :3 0 8b :3 0
c39 c3a 0 7d
:4 0 10c :4 0 80
:2 0 81 :3 0 70d
c3e c40 :3 0 80
:2 0 94 :4 0 710
c42 c44 :3 0 80
:2 0 b1 :4 0 713
c46 c48 :3 0 80
:2 0 ac :3 0 716
c4a c4c :3 0 80
:2 0 b2 :4 0 719
c4e c50 :3 0 80
:2 0 b3 :3 0 b4
:3 0 49 :2 0 b5
:2 0 71c c53 c57
720 c52 c59 :3 0
723 c3b c5b :2 0
c5d 726 c5e c38
c5d 0 c5f 728
0 c60 72a c63
:3 0 c63 0 c63
c62 c60 c61 :6 0
c65 16 :3 0 72d
c67 72f c66 c65
:2 0 c68 731 :2 0
c6a 0 c6a c69
c17 c68 :6 0 c6c
1 :3 0 734 ce4
d3 :3 0 f1 :3 0
e :3 0 f2 :3 0
f3 :3 0 f4 :3 0
f5 :3 0 f6 :3 0
f7 :3 0 74 :3 0
2b :4 0 10d 1
:8 0 c8e f9 :4 0
c79 :3 0 9c :3 0
a4 :2 0 73a c7c
c7d :3 0 1b :3 0
ab :3 0 c7f c80
0 31 :2 0 c81
c82 0 c84 73d
c8b 1b :3 0 ab
:3 0 c85 c86 0
fa :2 0 c87 c88
0 c8a 73f c8c
c7e c84 0 c8d
0 c8a 0 c8d
741 0 c8e 744
ce1 ad :3 0 1b
:3 0 ab :3 0 c90
c91 0 ac :3 0
c92 c93 0 c95
747 c97 749 c96
c95 :2 0 cdf ae
:3 0 1b :3 0 ab
:3 0 c9a c9b 0
ac :3 0 c9c c9d
0 cd7 78 :3 0
79 :3 0 c9f ca0
0 78 :3 0 88
:3 0 ca2 ca3 0
25 :2 0 2 :4 0
10e :2 0 8a :2 0
49 :2 0 74b ca8
caa :3 0 74e ca4
cac ca1 cae cad
:2 0 7b :3 0 8b
:3 0 cb0 cb1 0
7d :4 0 10f :4 0
80 :2 0 81 :3 0
752 cb5 cb7 :3 0
80 :2 0 94 :4 0
755 cb9 cbb :3 0
80 :2 0 b1 :4 0
758 cbd cbf :3 0
80 :2 0 ac :3 0
75b cc1 cc3 :3 0
80 :2 0 b2 :4 0
75e cc5 cc7 :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 761 cca
cce 765 cc9 cd0
:3 0 768 cb2 cd2
:2 0 cd4 76b cd5
caf cd4 0 cd6
76d 0 cd7 76f
cda :3 0 cda 0
cda cd9 cd7 cd8
:6 0 cdc 18 :3 0
772 cde 774 cdd
cdc :2 0 cdf 776
:2 0 ce1 0 ce1
ce0 c8e cdf :6 0
ce3 1 :3 0 779
ce5 b06 c6c 0
ce6 0 ce3 0
ce6 77b 0 ce7
77e ce8 ae0 ce7
0 ce9 780 0
cea 782 1112 41
:3 0 ef :3 0 85
:3 0 41 :3 0 31
:2 0 785 ced cf0
96 :3 0 110 :3 0
cf2 cf3 0 788
cec cf5 ceb cf6
0 1111 85 :3 0
57 :3 0 31 :2 0
78b cf8 cfb a4
:2 0 49 :2 0 790
cfd cff :3 0 1b
:3 0 ab :3 0 d01
d02 0 f0 :3 0
2b :3 0 48 :3 0
48 :3 0 32 :3 0
4c :3 0 4e :3 0
3d :3 0 36 :3 0
33 :3 0 33 :3 0
41 :3 0 3c :3 0
37 :3 0 38 :3 0
55 :3 0 51 :3 0
2f :3 0 793 d04
d16 d03 d17 0
db6 1b :3 0 ab
:3 0 d19 d1a 0
a4 :2 0 31 :2 0
7a7 d1c d1e :3 0
d1f :2 0 85 :3 0
f :3 0 31 :2 0
7aa d21 d24 a4
:2 0 35 :2 0 7af
d26 d28 :3 0 d29
:2 0 d20 d2b d2a
:2 0 85 :3 0 3e
:3 0 31 :2 0 7b2
d2d d30 a4 :2 0
31 :2 0 7b7 d32
d34 :3 0 d35 :2 0
d2c d37 d36 :2 0
d3 :3 0 f1 :3 0
e :3 0 f2 :3 0
f3 :3 0 f4 :3 0
f5 :3 0 f6 :3 0
f7 :3 0 6a :3 0
2d :3 0 cb :3 0
2e :3 0 cc :3 0
2c :4 0 111 1
:8 0 d5e f9 :4 0
d49 :3 0 9c :3 0
a4 :2 0 7bc d4c
d4d :3 0 1b :3 0
ab :3 0 d4f d50
0 31 :2 0 d51
d52 0 d54 7bf
d5b 1b :3 0 ab
:3 0 d55 d56 0
fa :2 0 d57 d58
0 d5a 7c1 d5c
d4e d54 0 d5d
0 d5a 0 d5d
7c3 0 d5e 7c6
db1 ad :3 0 1b
:3 0 ab :3 0 d60
d61 0 ac :3 0
d62 d63 0 d65
7c9 d67 7cb d66
d65 :2 0 daf ae
:3 0 1b :3 0 ab
:3 0 d6a d6b 0
ac :3 0 d6c d6d
0 da7 78 :3 0
79 :3 0 d6f d70
0 78 :3 0 88
:3 0 d72 d73 0
25 :2 0 2 :4 0
112 :2 0 8a :2 0
49 :2 0 7cd d78
d7a :3 0 7d0 d74
d7c d71 d7e d7d
:2 0 7b :3 0 8b
:3 0 d80 d81 0
7d :4 0 113 :4 0
80 :2 0 81 :3 0
7d4 d85 d87 :3 0
80 :2 0 94 :4 0
7d7 d89 d8b :3 0
80 :2 0 b1 :4 0
7da d8d d8f :3 0
80 :2 0 ac :3 0
7dd d91 d93 :3 0
80 :2 0 b2 :4 0
7e0 d95 d97 :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 7e3 d9a
d9e 7e7 d99 da0
:3 0 7ea d82 da2
:2 0 da4 7ed da5
d7f da4 0 da6
7ef 0 da7 7f1
daa :3 0 daa 0
daa da9 da7 da8
:6 0 dac 1a :3 0
7f4 dae 7f6 dad
dac :2 0 daf 7f8
:2 0 db1 0 db1
db0 d5e daf :6 0
db3 1 :3 0 7fb
db4 d38 db3 0
db5 7fd 0 db6
7ff 110e 12 :3 0
c3 :2 0 802 db8
db9 :3 0 dba :2 0
50 :3 0 12 :3 0
dbc dbd 0 104e
b3 :3 0 50 :3 0
b3 :3 0 50 :3 0
b3 :3 0 50 :3 0
b3 :3 0 50 :3 0
50 :3 0 ba :4 0
114 1 :8 0 dcf
1b :3 0 ab :3 0
dca dcb 0 ac
:3 0 dcc dcd 0
dcf 804 e22 ad
:3 0 1b :3 0 ab
:3 0 dd1 dd2 0
ac :3 0 dd3 dd4
0 dd6 807 dd8
809 dd7 dd6 :2 0
e20 ae :3 0 1b
:3 0 ab :3 0 ddb
ddc 0 ac :3 0
ddd dde 0 e18
78 :3 0 79 :3 0
de0 de1 0 78
:3 0 88 :3 0 de3
de4 0 25 :2 0
2 :4 0 115 :2 0
8a :2 0 49 :2 0
80b de9 deb :3 0
80e de5 ded de2
def dee :2 0 7b
:3 0 8b :3 0 df1
df2 0 7d :4 0
116 :4 0 80 :2 0
81 :3 0 812 df6
df8 :3 0 80 :2 0
94 :4 0 815 dfa
dfc :3 0 80 :2 0
b1 :4 0 818 dfe
e00 :3 0 80 :2 0
ac :3 0 81b e02
e04 :3 0 80 :2 0
b2 :4 0 81e e06
e08 :3 0 80 :2 0
b3 :3 0 b4 :3 0
49 :2 0 b5 :2 0
821 e0b e0f 825
e0a e11 :3 0 828
df3 e13 :2 0 e15
82b e16 df0 e15
0 e17 82d 0
e18 82f e1b :3 0
e1b 0 e1b e1a
e18 e19 :6 0 e1d
1c :3 0 832 e1f
834 e1e e1d :2 0
e20 836 :2 0 e22
0 e22 e21 dcf
e20 :6 0 104e 1
:3 0 c5 :3 0 117
:3 0 e24 e25 0
50 :3 0 4f :3 0
118 :2 0 839 e26
e2a 8e :2 0 31
:2 0 83f e2c e2e
:3 0 30 :3 0 96
:3 0 119 :3 0 e31
e32 0 e30 e33
0 104b 52 :3 0
49 :2 0 e35 e36
0 104b 6b :3 0
29 :3 0 11a :3 0
11b :4 0 11c 1
:8 0 e42 1b :3 0
ab :3 0 e3d e3e
0 ac :3 0 e3f
e40 0 e42 842
e95 ad :3 0 1b
:3 0 ab :3 0 e44
e45 0 ac :3 0
e46 e47 0 e49
845 e4b 847 e4a
e49 :2 0 e93 ae
:3 0 1b :3 0 ab
:3 0 e4e e4f 0
ac :3 0 e50 e51
0 e8b 78 :3 0
79 :3 0 e53 e54
0 78 :3 0 88
:3 0 e56 e57 0
25 :2 0 2 :4 0
11d :2 0 8a :2 0
49 :2 0 849 e5c
e5e :3 0 84c e58
e60 e55 e62 e61
:2 0 7b :3 0 8b
:3 0 e64 e65 0
7d :4 0 11e :4 0
80 :2 0 81 :3 0
850 e69 e6b :3 0
80 :2 0 94 :4 0
853 e6d e6f :3 0
80 :2 0 b1 :4 0
856 e71 e73 :3 0
80 :2 0 ac :3 0
859 e75 e77 :3 0
80 :2 0 b2 :4 0
85c e79 e7b :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 85f e7e
e82 863 e7d e84
:3 0 866 e66 e86
:2 0 e88 869 e89
e63 e88 0 e8a
86b 0 e8b 86d
e8e :3 0 e8e 0
e8e e8d e8b e8c
:6 0 e90 1e :3 0
870 e92 872 e91
e90 :2 0 e93 874
:2 0 e95 0 e95
e94 e42 e93 :6 0
104b 1 :3 0 54
:3 0 11f :3 0 52
:3 0 2e :3 0 29
:3 0 30 :3 0 2f
:3 0 2a :3 0 877
e98 e9f e97 ea0
0 104b 85 :3 0
54 :3 0 31 :2 0
87e ea2 ea5 a4
:2 0 31 :2 0 883
ea7 ea9 :3 0 26
:4 0 eab eac 0
1048 28 :4 0 eae
eaf 0 1048 1d
:4 0 eb1 eb2 0
1048 24 :4 0 eb4
eb5 0 1048 1f
:4 0 eb7 eb8 0
1048 120 :3 0 63
:4 0 ebd :2 0 1048
ebb ebe :3 0 63
:3 0 1d :3 0 24
:4 0 ec3 :2 0 1048
ebf ec4 :3 0 886
:3 0 63 :3 0 f9
:3 0 ec5 ec6 :3 0
1b :3 0 ab :3 0
ec8 ec9 0 31
:2 0 eca ecb 0
ecd 889 ed4 1b
:3 0 ab :3 0 ece
ecf 0 fa :2 0
ed0 ed1 0 ed3
88b ed5 ec7 ecd
0 ed6 0 ed3
0 ed6 88d 0
1048 121 :3 0 63
:4 0 eda :2 0 ee0
ed8 0 1b :3 0
ab :3 0 edb edc
0 ac :3 0 edd
ede 0 ee0 890
f33 ad :3 0 1b
:3 0 ab :3 0 ee2
ee3 0 ac :3 0
ee4 ee5 0 ee7
893 ee9 895 ee8
ee7 :2 0 f31 ae
:3 0 1b :3 0 ab
:3 0 eec eed 0
ac :3 0 eee eef
0 f29 78 :3 0
79 :3 0 ef1 ef2
0 78 :3 0 88
:3 0 ef4 ef5 0
25 :2 0 2 :4 0
122 :2 0 8a :2 0
49 :2 0 897 efa
efc :3 0 89a ef6
efe ef3 f00 eff
:2 0 7b :3 0 8b
:3 0 f02 f03 0
7d :4 0 123 :4 0
80 :2 0 81 :3 0
89e f07 f09 :3 0
80 :2 0 94 :4 0
8a1 f0b f0d :3 0
80 :2 0 b1 :4 0
8a4 f0f f11 :3 0
80 :2 0 ac :3 0
8a7 f13 f15 :3 0
80 :2 0 b2 :4 0
8aa f17 f19 :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 8ad f1c
f20 8b1 f1b f22
:3 0 8b4 f04 f24
:2 0 f26 8b7 f27
f01 f26 0 f28
8b9 0 f29 8bb
f2c :3 0 f2c 0
f2c f2b f29 f2a
:6 0 f2e 20 :3 0
8be f30 8c0 f2f
f2e :2 0 f31 8c2
:2 0 f33 0 f33
f32 ee0 f31 :6 0
1048 1 :3 0 124
:3 0 1f :3 0 125
:3 0 cc :3 0 2c
:4 0 126 1 :8 0
f40 1b :3 0 ab
:3 0 f3b f3c 0
ac :3 0 f3d f3e
0 f40 8c5 f93
ad :3 0 1b :3 0
ab :3 0 f42 f43
0 ac :3 0 f44
f45 0 f47 8c8
f49 8ca f48 f47
:2 0 f91 ae :3 0
1b :3 0 ab :3 0
f4c f4d 0 ac
:3 0 f4e f4f 0
f89 78 :3 0 79
:3 0 f51 f52 0
78 :3 0 88 :3 0
f54 f55 0 25
:2 0 2 :4 0 127
:2 0 8a :2 0 49
:2 0 8cc f5a f5c
:3 0 8cf f56 f5e
f53 f60 f5f :2 0
7b :3 0 8b :3 0
f62 f63 0 7d
:4 0 128 :4 0 80
:2 0 81 :3 0 8d3
f67 f69 :3 0 80
:2 0 94 :4 0 8d6
f6b f6d :3 0 80
:2 0 b1 :4 0 8d9
f6f f71 :3 0 80
:2 0 ac :3 0 8dc
f73 f75 :3 0 80
:2 0 b2 :4 0 8df
f77 f79 :3 0 80
:2 0 b3 :3 0 b4
:3 0 49 :2 0 b5
:2 0 8e2 f7c f80
8e6 f7b f82 :3 0
8e9 f64 f84 :2 0
f86 8ec f87 f61
f86 0 f88 8ee
0 f89 8f0 f8c
:3 0 f8c 0 f8c
f8b f89 f8a :6 0
f8e 22 :3 0 8f3
f90 8f5 f8f f8e
:2 0 f91 8f7 :2 0
f93 0 f93 f92
f40 f91 :6 0 1048
1 :3 0 22 :3 0
129 :4 0 f95 f96
0 1048 20 :3 0
12a :4 0 f98 f99
0 1048 20 :3 0
b3 :3 0 20 :3 0
80 :2 0 1f :3 0
8fa f9e fa0 :3 0
49 :2 0 21 :2 0
8fd f9c fa4 f9b
fa5 0 1048 20
:3 0 b3 :3 0 20
:3 0 80 :2 0 12b
:4 0 901 faa fac
:3 0 49 :2 0 21
:2 0 904 fa8 fb0
fa7 fb1 0 1048
20 :3 0 b3 :3 0
20 :3 0 80 :2 0
1d :3 0 908 fb6
fb8 :3 0 49 :2 0
21 :2 0 90b fb4
fbc fb3 fbd 0
1048 20 :3 0 b3
:3 0 20 :3 0 80
:2 0 12c :4 0 90f
fc2 fc4 :3 0 49
:2 0 21 :2 0 912
fc0 fc8 fbf fc9
0 1048 54 :3 0
12d :3 0 2a :3 0
24 :3 0 33 :3 0
33 :3 0 48 :3 0
33 :3 0 22 :3 0
20 :3 0 2f :3 0
48 :3 0 48 :3 0
33 :3 0 26 :3 0
28 :3 0 2f :3 0
916 fcc fdc fcb
fdd 0 1048 78
:3 0 79 :3 0 fdf
fe0 0 78 :3 0
88 :3 0 fe2 fe3
0 27 :2 0 2
:4 0 12e :2 0 8a
:2 0 49 :2 0 926
fe8 fea :3 0 929
fe4 fec fe1 fee
fed :2 0 7b :3 0
8b :3 0 ff0 ff1
0 7d :4 0 12f
:4 0 80 :2 0 81
:3 0 92d ff5 ff7
:3 0 80 :2 0 94
:4 0 930 ff9 ffb
:3 0 80 :2 0 95
:4 0 933 ffd fff
:3 0 80 :2 0 85
:3 0 66 :3 0 31
:2 0 936 1003 1005
86 :4 0 938 1002
1008 93b 1001 100a
:3 0 80 :2 0 db
:4 0 93e 100c 100e
:3 0 80 :2 0 83
:4 0 941 1010 1012
:3 0 80 :2 0 2b
:3 0 944 1014 1016
:3 0 80 :2 0 83
:4 0 947 1018 101a
:3 0 94a ff2 101c
:2 0 101e 94d 101f
fef 101e 0 1020
94f 0 1048 78
:3 0 79 :3 0 1021
1022 0 78 :3 0
88 :3 0 1024 1025
0 49 :2 0 2
:4 0 130 :2 0 8a
:2 0 49 :2 0 951
102a 102c :3 0 954
1026 102e 1023 1030
102f :2 0 7b :3 0
8b :3 0 1032 1033
0 7d :4 0 131
:4 0 80 :2 0 81
:3 0 958 1037 1039
:3 0 95b 1034 103b
:2 0 103d 95e 103e
1031 103d 0 103f
960 0 1048 9a
:3 0 de :3 0 1040
1041 0 31 :2 0
2b :3 0 9c :3 0
962 1042 1046 :2 0
1048 966 1049 eaa
1048 0 104a 97b
0 104b 97d 104c
e2f 104b 0 104d
983 0 104e 985
104f dbb 104e 0
1050 989 0 110d
3c :3 0 85 :3 0
3c :3 0 31 :2 0
98b 1052 1055 8a
:2 0 49 :2 0 98e
1057 1059 :3 0 1051
105a 0 110d 1b
:3 0 ab :3 0 105c
105d 0 f0 :3 0
2b :3 0 48 :3 0
48 :3 0 32 :3 0
4c :3 0 4e :3 0
3d :3 0 36 :3 0
33 :3 0 33 :3 0
41 :3 0 3c :3 0
37 :3 0 38 :3 0
55 :3 0 51 :3 0
2f :3 0 991 105f
1071 105e 1072 0
110d 1b :3 0 ab
:3 0 1074 1075 0
a4 :2 0 31 :2 0
9a5 1077 1079 :3 0
107a :2 0 85 :3 0
f :3 0 31 :2 0
9a8 107c 107f a4
:2 0 35 :2 0 9ad
1081 1083 :3 0 1084
:2 0 107b 1086 1085
:2 0 85 :3 0 3e
:3 0 31 :2 0 9b0
1088 108b a4 :2 0
31 :2 0 9b5 108d
108f :3 0 1090 :2 0
1087 1092 1091 :2 0
d3 :3 0 f1 :3 0
e :3 0 f2 :3 0
f3 :3 0 f4 :3 0
f5 :3 0 f6 :3 0
f7 :3 0 74 :3 0
2b :4 0 10d 1
:8 0 10b5 f9 :4 0
10a0 :3 0 9c :3 0
a4 :2 0 9ba 10a3
10a4 :3 0 1b :3 0
ab :3 0 10a6 10a7
0 31 :2 0 10a8
10a9 0 10ab 9bd
10b2 1b :3 0 ab
:3 0 10ac 10ad 0
fa :2 0 10ae 10af
0 10b1 9bf 10b3
10a5 10ab 0 10b4
0 10b1 0 10b4
9c1 0 10b5 9c4
1108 ad :3 0 1b
:3 0 ab :3 0 10b7
10b8 0 ac :3 0
10b9 10ba 0 10bc
9c7 10be 9c9 10bd
10bc :2 0 1106 ae
:3 0 1b :3 0 ab
:3 0 10c1 10c2 0
ac :3 0 10c3 10c4
0 10fe 78 :3 0
79 :3 0 10c6 10c7
0 78 :3 0 88
:3 0 10c9 10ca 0
25 :2 0 2 :4 0
132 :2 0 8a :2 0
49 :2 0 9cb 10cf
10d1 :3 0 9ce 10cb
10d3 10c8 10d5 10d4
:2 0 7b :3 0 8b
:3 0 10d7 10d8 0
7d :4 0 133 :4 0
80 :2 0 81 :3 0
9d2 10dc 10de :3 0
80 :2 0 94 :4 0
9d5 10e0 10e2 :3 0
80 :2 0 b1 :4 0
9d8 10e4 10e6 :3 0
80 :2 0 ac :3 0
9db 10e8 10ea :3 0
80 :2 0 b2 :4 0
9de 10ec 10ee :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 9e1 10f1
10f5 9e5 10f0 10f7
:3 0 9e8 10d9 10f9
:2 0 10fb 9eb 10fc
10d6 10fb 0 10fd
9ed 0 10fe 9ef
1101 :3 0 1101 0
1101 1100 10fe 10ff
:6 0 1103 24 :3 0
9f2 1105 9f4 1104
1103 :2 0 1106 9f6
:2 0 1108 0 1108
1107 10b5 1106 :6 0
110a 1 :3 0 9f9
110b 1093 110a 0
110c 9fb 0 110d
9fd 110f d00 db6
0 1110 0 110d
0 1110 a02 0
1111 a05 1113 92e
cea 0 1114 0
1111 0 1114 a08
0 1d05 1b :3 0
ab :3 0 1115 1116
0 c0 :2 0 31
:2 0 a0d 1118 111a
:3 0 78 :3 0 79
:3 0 111c 111d 0
78 :3 0 88 :3 0
111f 1120 0 27
:2 0 2 :4 0 134
:2 0 8a :2 0 49
:2 0 a10 1125 1127
:3 0 a13 1121 1129
111e 112b 112a :2 0
7b :3 0 8b :3 0
112d 112e 0 7d
:4 0 135 :4 0 80
:2 0 81 :3 0 a17
1132 1134 :3 0 80
:2 0 94 :4 0 a1a
1136 1138 :3 0 80
:2 0 95 :4 0 a1d
113a 113c :3 0 80
:2 0 85 :3 0 66
:3 0 1b :3 0 ab
:3 0 1141 1142 0
a20 1140 1144 86
:4 0 a22 113f 1147
a25 113e 1149 :3 0
80 :2 0 db :4 0
a28 114b 114d :3 0
80 :2 0 83 :4 0
a2b 114f 1151 :3 0
80 :2 0 2b :3 0
a2e 1153 1155 :3 0
80 :2 0 83 :4 0
a31 1157 1159 :3 0
a34 112f 115b :2 0
115d a37 115e 112c
115d 0 115f a39
0 1189 78 :3 0
79 :3 0 1160 1161
0 78 :3 0 88
:3 0 1163 1164 0
49 :2 0 2 :4 0
136 :2 0 8a :2 0
49 :2 0 a3b 1169
116b :3 0 a3e 1165
116d 1162 116f 116e
:2 0 7b :3 0 8b
:3 0 1171 1172 0
7d :4 0 137 :4 0
80 :2 0 81 :3 0
a42 1176 1178 :3 0
a45 1173 117a :2 0
117c a48 117d 1170
117c 0 117e a4a
0 1189 9a :3 0
de :3 0 117f 1180
0 1b :3 0 ab
:3 0 1182 1183 0
2b :3 0 9c :3 0
a4c 1181 1187 :2 0
1189 a50 118a 111b
1189 0 118b a54
0 1d05 38 :3 0
11 :3 0 118c 118d
0 1d05 85 :3 0
39 :3 0 31 :2 0
a56 118f 1192 c0
:2 0 ed :2 0 a5b
1194 1196 :3 0 1197
:2 0 85 :3 0 38
:3 0 31 :2 0 a5e
1199 119c a4 :2 0
31 :2 0 a63 119e
11a0 :3 0 11a1 :2 0
1198 11a3 11a2 :2 0
38 :3 0 49 :2 0
11a5 11a6 0 11a8
a66 11a9 11a4 11a8
0 11aa a68 0
1d05 85 :3 0 57
:3 0 31 :2 0 a6a
11ab 11ae a4 :2 0
49 :2 0 a6f 11b0
11b2 :3 0 ef :3 0
85 :3 0 f :3 0
31 :2 0 a72 11b5
11b8 25 :2 0 a75
11b4 11bb 11bc :2 0
a4 :2 0 25 :2 0
a7a 11be 11c0 :3 0
12 :3 0 8d :2 0
a7d 11c3 11c4 :3 0
11c5 :2 0 4d :3 0
c3 :2 0 a7f 11c8
11c9 :3 0 11ca :2 0
d3 :3 0 100 :3 0
d :3 0 cf :3 0
39 :3 0 d2 :3 0
38 :3 0 f2 :3 0
f3 :3 0 f4 :3 0
f5 :3 0 f6 :3 0
f7 :3 0 101 :3 0
16 :3 0 102 :3 0
17 :3 0 cd :3 0
18 :3 0 138 :3 0
2b :4 0 139 1
:8 0 11f7 f9 :4 0
11e2 :3 0 9c :3 0
a4 :2 0 a83 11e5
11e6 :3 0 1b :3 0
ab :3 0 11e8 11e9
0 31 :2 0 11ea
11eb 0 11ed a86
11f4 1b :3 0 ab
:3 0 11ee 11ef 0
fa :2 0 11f0 11f1
0 11f3 a88 11f5
11e7 11ed 0 11f6
0 11f3 0 11f6
a8a 0 11f7 a8d
124a ad :3 0 1b
:3 0 ab :3 0 11f9
11fa 0 ac :3 0
11fb 11fc 0 11fe
a90 1200 a92 11ff
11fe :2 0 1248 ae
:3 0 1b :3 0 ab
:3 0 1203 1204 0
ac :3 0 1205 1206
0 1240 78 :3 0
79 :3 0 1208 1209
0 78 :3 0 88
:3 0 120b 120c 0
25 :2 0 2 :4 0
13a :2 0 8a :2 0
49 :2 0 a94 1211
1213 :3 0 a97 120d
1215 120a 1217 1216
:2 0 7b :3 0 8b
:3 0 1219 121a 0
7d :4 0 13b :4 0
80 :2 0 81 :3 0
a9b 121e 1220 :3 0
80 :2 0 94 :4 0
a9e 1222 1224 :3 0
80 :2 0 b1 :4 0
aa1 1226 1228 :3 0
80 :2 0 ac :3 0
aa4 122a 122c :3 0
80 :2 0 b2 :4 0
aa7 122e 1230 :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 aaa 1233
1237 aae 1232 1239
:3 0 ab1 121b 123b
:2 0 123d ab4 123e
1218 123d 0 123f
ab6 0 1240 ab8
1243 :3 0 1243 0
1243 1242 1240 1241
:6 0 1245 26 :3 0
abb 1247 abd 1246
1245 :2 0 1248 abf
:2 0 124a 0 124a
1249 11f7 1248 :6 0
124c 1 :3 0 ac2
12c2 d3 :3 0 100
:3 0 d :3 0 cf
:3 0 39 :3 0 d2
:3 0 38 :3 0 138
:3 0 2b :4 0 13c
1 :8 0 126c f9
:4 0 1257 :3 0 9c
:3 0 a4 :2 0 ac6
125a 125b :3 0 1b
:3 0 ab :3 0 125d
125e 0 31 :2 0
125f 1260 0 1262
ac9 1269 1b :3 0
ab :3 0 1263 1264
0 fa :2 0 1265
1266 0 1268 acb
126a 125c 1262 0
126b 0 1268 0
126b acd 0 126c
ad0 12bf ad :3 0
1b :3 0 ab :3 0
126e 126f 0 ac
:3 0 1270 1271 0
1273 ad3 1275 ad5
1274 1273 :2 0 12bd
ae :3 0 1b :3 0
ab :3 0 1278 1279
0 ac :3 0 127a
127b 0 12b5 78
:3 0 79 :3 0 127d
127e 0 78 :3 0
88 :3 0 1280 1281
0 25 :2 0 2
:4 0 13d :2 0 8a
:2 0 49 :2 0 ad7
1286 1288 :3 0 ada
1282 128a 127f 128c
128b :2 0 7b :3 0
8b :3 0 128e 128f
0 7d :4 0 13e
:4 0 80 :2 0 81
:3 0 ade 1293 1295
:3 0 80 :2 0 94
:4 0 ae1 1297 1299
:3 0 80 :2 0 b1
:4 0 ae4 129b 129d
:3 0 80 :2 0 ac
:3 0 ae7 129f 12a1
:3 0 80 :2 0 b2
:4 0 aea 12a3 12a5
:3 0 80 :2 0 b3
:3 0 b4 :3 0 49
:2 0 b5 :2 0 aed
12a8 12ac af1 12a7
12ae :3 0 af4 1290
12b0 :2 0 12b2 af7
12b3 128d 12b2 0
12b4 af9 0 12b5
afb 12b8 :3 0 12b8
0 12b8 12b7 12b5
12b6 :6 0 12ba 28
:3 0 afe 12bc b00
12bb 12ba :2 0 12bd
b02 :2 0 12bf 0
12bf 12be 126c 12bd
:6 0 12c1 1 :3 0
b05 12c3 11cb 124c
0 12c4 0 12c1
0 12c4 b07 0
12c5 b0a 141b 4d
:3 0 c3 :2 0 b0c
12c7 12c8 :3 0 12c9
:2 0 d3 :3 0 100
:3 0 d :3 0 cf
:3 0 39 :3 0 d2
:3 0 38 :3 0 13f
:3 0 140 :3 0 c4
:3 0 12 :3 0 13f
:3 0 141 :3 0 140
:3 0 c4 :3 0 13
:3 0 141 :3 0 142
:3 0 140 :3 0 14
:3 0 142 :3 0 143
:3 0 140 :3 0 15
:3 0 143 :3 0 144
:3 0 145 :3 0 146
:3 0 147 :3 0 148
:3 0 149 :3 0 14a
:3 0 14b :3 0 145
:3 0 12 :3 0 147
:3 0 13 :3 0 149
:3 0 14 :3 0 14b
:3 0 15 :3 0 14c
:3 0 14d :3 0 14e
:3 0 14f :3 0 150
:3 0 151 :3 0 152
:3 0 153 :3 0 140
:3 0 f2 :3 0 f3
:3 0 f4 :3 0 f5
:3 0 f6 :3 0 f7
:3 0 101 :3 0 16
:3 0 102 :3 0 17
:3 0 cd :3 0 18
:3 0 138 :3 0 2b
:4 0 154 1 :8 0
1321 f9 :4 0 130c
:3 0 9c :3 0 a4
:2 0 b10 130f 1310
:3 0 1b :3 0 ab
:3 0 1312 1313 0
31 :2 0 1314 1315
0 1317 b13 131e
1b :3 0 ab :3 0
1318 1319 0 fa
:2 0 131a 131b 0
131d b15 131f 1311
1317 0 1320 0
131d 0 1320 b17
0 1321 b1a 1374
ad :3 0 1b :3 0
ab :3 0 1323 1324
0 ac :3 0 1325
1326 0 1328 b1d
132a b1f 1329 1328
:2 0 1372 ae :3 0
1b :3 0 ab :3 0
132d 132e 0 ac
:3 0 132f 1330 0
136a 78 :3 0 79
:3 0 1332 1333 0
78 :3 0 88 :3 0
1335 1336 0 25
:2 0 2 :4 0 155
:2 0 8a :2 0 49
:2 0 b21 133b 133d
:3 0 b24 1337 133f
1334 1341 1340 :2 0
7b :3 0 8b :3 0
1343 1344 0 7d
:4 0 156 :4 0 80
:2 0 81 :3 0 b28
1348 134a :3 0 80
:2 0 94 :4 0 b2b
134c 134e :3 0 80
:2 0 b1 :4 0 b2e
1350 1352 :3 0 80
:2 0 ac :3 0 b31
1354 1356 :3 0 80
:2 0 b2 :4 0 b34
1358 135a :3 0 80
:2 0 b3 :3 0 b4
:3 0 49 :2 0 b5
:2 0 b37 135d 1361
b3b 135c 1363 :3 0
b3e 1345 1365 :2 0
1367 b41 1368 1342
1367 0 1369 b43
0 136a b45 136d
:3 0 136d 0 136d
136c 136a 136b :6 0
136f 2a :3 0 b48
1371 b4a 1370 136f
:2 0 1372 b4c :2 0
1374 0 1374 1373
1321 1372 :6 0 1376
1 :3 0 b4f 1417
d3 :3 0 100 :3 0
d :3 0 cf :3 0
39 :3 0 d2 :3 0
38 :3 0 13f :3 0
140 :3 0 c4 :3 0
12 :3 0 13f :3 0
141 :3 0 140 :3 0
c4 :3 0 13 :3 0
141 :3 0 142 :3 0
140 :3 0 14 :3 0
142 :3 0 143 :3 0
140 :3 0 15 :3 0
143 :3 0 144 :3 0
145 :3 0 146 :3 0
147 :3 0 148 :3 0
149 :3 0 14a :3 0
14b :3 0 145 :3 0
12 :3 0 147 :3 0
13 :3 0 149 :3 0
14 :3 0 14b :3 0
15 :3 0 14c :3 0
14d :3 0 14e :3 0
14f :3 0 150 :3 0
151 :3 0 152 :3 0
153 :3 0 140 :3 0
138 :3 0 2b :4 0
157 1 :8 0 13c1
f9 :4 0 13ac :3 0
9c :3 0 a4 :2 0
b53 13af 13b0 :3 0
1b :3 0 ab :3 0
13b2 13b3 0 31
:2 0 13b4 13b5 0
13b7 b56 13be 1b
:3 0 ab :3 0 13b8
13b9 0 fa :2 0
13ba 13bb 0 13bd
b58 13bf 13b1 13b7
0 13c0 0 13bd
0 13c0 b5a 0
13c1 b5d 1414 ad
:3 0 1b :3 0 ab
:3 0 13c3 13c4 0
ac :3 0 13c5 13c6
0 13c8 b60 13ca
b62 13c9 13c8 :2 0
1412 ae :3 0 1b
:3 0 ab :3 0 13cd
13ce 0 ac :3 0
13cf 13d0 0 140a
78 :3 0 79 :3 0
13d2 13d3 0 78
:3 0 88 :3 0 13d5
13d6 0 25 :2 0
2 :4 0 158 :2 0
8a :2 0 49 :2 0
b64 13db 13dd :3 0
b67 13d7 13df 13d4
13e1 13e0 :2 0 7b
:3 0 8b :3 0 13e3
13e4 0 7d :4 0
159 :4 0 80 :2 0
81 :3 0 b6b 13e8
13ea :3 0 80 :2 0
94 :4 0 b6e 13ec
13ee :3 0 80 :2 0
b1 :4 0 b71 13f0
13f2 :3 0 80 :2 0
ac :3 0 b74 13f4
13f6 :3 0 80 :2 0
b2 :4 0 b77 13f8
13fa :3 0 80 :2 0
b3 :3 0 b4 :3 0
49 :2 0 b5 :2 0
b7a 13fd 1401 b7e
13fc 1403 :3 0 b81
13e5 1405 :2 0 1407
b84 1408 13e2 1407
0 1409 b86 0
140a b88 140d :3 0
140d 0 140d 140c
140a 140b :6 0 140f
2c :3 0 b8b 1411
b8d 1410 140f :2 0
1412 b8f :2 0 1414
0 1414 1413 13c1
1412 :6 0 1416 1
:3 0 b92 1418 12ca
1376 0 1419 0
1416 0 1419 b94
0 141a b97 141c
11c6 12c5 0 141d
0 141a 0 141d
b99 0 141f 15a
:3 0 b9c 149f 85
:3 0 3e :3 0 31
:2 0 b9e 1420 1423
a4 :2 0 31 :2 0
ba3 1425 1427 :3 0
d3 :3 0 100 :3 0
d :3 0 cf :3 0
39 :3 0 d2 :3 0
38 :3 0 138 :3 0
2b :4 0 15b 1
:8 0 1448 f9 :4 0
1433 :3 0 9c :3 0
a4 :2 0 ba8 1436
1437 :3 0 1b :3 0
ab :3 0 1439 143a
0 31 :2 0 143b
143c 0 143e bab
1445 1b :3 0 ab
:3 0 143f 1440 0
fa :2 0 1441 1442
0 1444 bad 1446
1438 143e 0 1447
0 1444 0 1447
baf 0 1448 bb2
149b ad :3 0 1b
:3 0 ab :3 0 144a
144b 0 ac :3 0
144c 144d 0 144f
bb5 1451 bb7 1450
144f :2 0 1499 ae
:3 0 1b :3 0 ab
:3 0 1454 1455 0
ac :3 0 1456 1457
0 1491 78 :3 0
79 :3 0 1459 145a
0 78 :3 0 88
:3 0 145c 145d 0
25 :2 0 2 :4 0
15c :2 0 8a :2 0
49 :2 0 bb9 1462
1464 :3 0 bbc 145e
1466 145b 1468 1467
:2 0 7b :3 0 8b
:3 0 146a 146b 0
7d :4 0 15d :4 0
80 :2 0 81 :3 0
bc0 146f 1471 :3 0
80 :2 0 94 :4 0
bc3 1473 1475 :3 0
80 :2 0 b1 :4 0
bc6 1477 1479 :3 0
80 :2 0 ac :3 0
bc9 147b 147d :3 0
80 :2 0 b2 :4 0
bcc 147f 1481 :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 bcf 1484
1488 bd3 1483 148a
:3 0 bd6 146c 148c
:2 0 148e bd9 148f
1469 148e 0 1490
bdb 0 1491 bdd
1494 :3 0 1494 0
1494 1493 1491 1492
:6 0 1496 2e :3 0
be0 1498 be2 1497
1496 :2 0 1499 be4
:2 0 149b 0 149b
149a 1448 1499 :6 0
149d 1 :3 0 be7
149e 1428 149d 0
14a0 11c1 141f 0
14a0 be9 0 14a1
bec 189d ef :3 0
85 :3 0 f :3 0
31 :2 0 bee 14a3
14a6 25 :2 0 bf1
14a2 14a9 14aa :2 0
a4 :2 0 25 :2 0
bf6 14ac 14ae :3 0
12 :3 0 8d :2 0
bf9 14b1 14b2 :3 0
14b3 :2 0 4d :3 0
c3 :2 0 bfb 14b6
14b7 :3 0 14b8 :2 0
d3 :3 0 100 :3 0
d :3 0 cf :3 0
39 :3 0 d2 :3 0
38 :3 0 f2 :3 0
f3 :3 0 f4 :3 0
f5 :3 0 f6 :3 0
f7 :3 0 101 :3 0
16 :3 0 102 :3 0
17 :3 0 cd :3 0
18 :3 0 74 :3 0
2b :4 0 15e 1
:8 0 14e5 f9 :4 0
14d0 :3 0 9c :3 0
a4 :2 0 bff 14d3
14d4 :3 0 1b :3 0
ab :3 0 14d6 14d7
0 31 :2 0 14d8
14d9 0 14db c02
14e2 1b :3 0 ab
:3 0 14dc 14dd 0
fa :2 0 14de 14df
0 14e1 c04 14e3
14d5 14db 0 14e4
0 14e1 0 14e4
c06 0 14e5 c09
1538 ad :3 0 1b
:3 0 ab :3 0 14e7
14e8 0 ac :3 0
14e9 14ea 0 14ec
c0c 14ee c0e 14ed
14ec :2 0 1536 ae
:3 0 1b :3 0 ab
:3 0 14f1 14f2 0
ac :3 0 14f3 14f4
0 152e 78 :3 0
79 :3 0 14f6 14f7
0 78 :3 0 88
:3 0 14f9 14fa 0
25 :2 0 2 :4 0
15f :2 0 8a :2 0
49 :2 0 c10 14ff
1501 :3 0 c13 14fb
1503 14f8 1505 1504
:2 0 7b :3 0 8b
:3 0 1507 1508 0
7d :4 0 160 :4 0
80 :2 0 81 :3 0
c17 150c 150e :3 0
80 :2 0 94 :4 0
c1a 1510 1512 :3 0
80 :2 0 b1 :4 0
c1d 1514 1516 :3 0
80 :2 0 ac :3 0
c20 1518 151a :3 0
80 :2 0 b2 :4 0
c23 151c 151e :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 c26 1521
1525 c2a 1520 1527
:3 0 c2d 1509 1529
:2 0 152b c30 152c
1506 152b 0 152d
c32 0 152e c34
1531 :3 0 1531 0
1531 1530 152e 152f
:6 0 1533 30 :3 0
c37 1535 c39 1534
1533 :2 0 1536 c3b
:2 0 1538 0 1538
1537 14e5 1536 :6 0
153a 1 :3 0 c3e
15b2 d3 :3 0 100
:3 0 d :3 0 cf
:3 0 39 :3 0 d2
:3 0 38 :3 0 cd
:3 0 c :3 0 74
:3 0 2b :4 0 161
1 :8 0 155c f9
:4 0 1547 :3 0 9c
:3 0 a4 :2 0 c42
154a 154b :3 0 1b
:3 0 ab :3 0 154d
154e 0 31 :2 0
154f 1550 0 1552
c45 1559 1b :3 0
ab :3 0 1553 1554
0 fa :2 0 1555
1556 0 1558 c47
155a 154c 1552 0
155b 0 1558 0
155b c49 0 155c
c4c 15af ad :3 0
1b :3 0 ab :3 0
155e 155f 0 ac
:3 0 1560 1561 0
1563 c4f 1565 c51
1564 1563 :2 0 15ad
ae :3 0 1b :3 0
ab :3 0 1568 1569
0 ac :3 0 156a
156b 0 15a5 78
:3 0 79 :3 0 156d
156e 0 78 :3 0
88 :3 0 1570 1571
0 25 :2 0 2
:4 0 162 :2 0 8a
:2 0 49 :2 0 c53
1576 1578 :3 0 c56
1572 157a 156f 157c
157b :2 0 7b :3 0
8b :3 0 157e 157f
0 7d :4 0 163
:4 0 80 :2 0 81
:3 0 c5a 1583 1585
:3 0 80 :2 0 94
:4 0 c5d 1587 1589
:3 0 80 :2 0 b1
:4 0 c60 158b 158d
:3 0 80 :2 0 ac
:3 0 c63 158f 1591
:3 0 80 :2 0 b2
:4 0 c66 1593 1595
:3 0 80 :2 0 b3
:3 0 b4 :3 0 49
:2 0 b5 :2 0 c69
1598 159c c6d 1597
159e :3 0 c70 1580
15a0 :2 0 15a2 c73
15a3 157d 15a2 0
15a4 c75 0 15a5
c77 15a8 :3 0 15a8
0 15a8 15a7 15a5
15a6 :6 0 15aa 32
:3 0 c7a 15ac c7c
15ab 15aa :2 0 15ad
c7e :2 0 15af 0
15af 15ae 155c 15ad
:6 0 15b1 1 :3 0
c81 15b3 14b9 153a
0 15b4 0 15b1
0 15b4 c83 0
15b5 c86 170d 4d
:3 0 c3 :2 0 c88
15b7 15b8 :3 0 15b9
:2 0 d3 :3 0 100
:3 0 d :3 0 cf
:3 0 39 :3 0 d2
:3 0 38 :3 0 13f
:3 0 140 :3 0 c4
:3 0 12 :3 0 13f
:3 0 141 :3 0 140
:3 0 c4 :3 0 13
:3 0 141 :3 0 142
:3 0 140 :3 0 14
:3 0 142 :3 0 143
:3 0 140 :3 0 15
:3 0 143 :3 0 144
:3 0 145 :3 0 146
:3 0 147 :3 0 148
:3 0 149 :3 0 14a
:3 0 14b :3 0 145
:3 0 12 :3 0 147
:3 0 13 :3 0 149
:3 0 14 :3 0 14b
:3 0 15 :3 0 14c
:3 0 14d :3 0 14e
:3 0 14f :3 0 150
:3 0 151 :3 0 152
:3 0 153 :3 0 140
:3 0 f2 :3 0 f3
:3 0 f4 :3 0 f5
:3 0 f6 :3 0 f7
:3 0 101 :3 0 16
:3 0 102 :3 0 17
:3 0 cd :3 0 18
:3 0 74 :3 0 2b
:4 0 164 1 :8 0
1611 f9 :4 0 15fc
:3 0 9c :3 0 a4
:2 0 c8c 15ff 1600
:3 0 1b :3 0 ab
:3 0 1602 1603 0
31 :2 0 1604 1605
0 1607 c8f 160e
1b :3 0 ab :3 0
1608 1609 0 fa
:2 0 160a 160b 0
160d c91 160f 1601
1607 0 1610 0
160d 0 1610 c93
0 1611 c96 1664
ad :3 0 1b :3 0
ab :3 0 1613 1614
0 ac :3 0 1615
1616 0 1618 c99
161a c9b 1619 1618
:2 0 1662 ae :3 0
1b :3 0 ab :3 0
161d 161e 0 ac
:3 0 161f 1620 0
165a 78 :3 0 79
:3 0 1622 1623 0
78 :3 0 88 :3 0
1625 1626 0 25
:2 0 2 :4 0 165
:2 0 8a :2 0 49
:2 0 c9d 162b 162d
:3 0 ca0 1627 162f
1624 1631 1630 :2 0
7b :3 0 8b :3 0
1633 1634 0 7d
:4 0 166 :4 0 80
:2 0 81 :3 0 ca4
1638 163a :3 0 80
:2 0 94 :4 0 ca7
163c 163e :3 0 80
:2 0 b1 :4 0 caa
1640 1642 :3 0 80
:2 0 ac :3 0 cad
1644 1646 :3 0 80
:2 0 b2 :4 0 cb0
1648 164a :3 0 80
:2 0 b3 :3 0 b4
:3 0 49 :2 0 b5
:2 0 cb3 164d 1651
cb7 164c 1653 :3 0
cba 1635 1655 :2 0
1657 cbd 1658 1632
1657 0 1659 cbf
0 165a cc1 165d
:3 0 165d 0 165d
165c 165a 165b :6 0
165f 34 :3 0 cc4
1661 cc6 1660 165f
:2 0 1662 cc8 :2 0
1664 0 1664 1663
1611 1662 :6 0 1666
1 :3 0 ccb 1709
d3 :3 0 100 :3 0
d :3 0 cf :3 0
39 :3 0 d2 :3 0
38 :3 0 cd :3 0
c :3 0 13f :3 0
140 :3 0 c4 :3 0
12 :3 0 13f :3 0
141 :3 0 140 :3 0
c4 :3 0 13 :3 0
141 :3 0 142 :3 0
140 :3 0 14 :3 0
142 :3 0 143 :3 0
140 :3 0 15 :3 0
143 :3 0 144 :3 0
145 :3 0 146 :3 0
147 :3 0 148 :3 0
149 :3 0 14a :3 0
14b :3 0 145 :3 0
12 :3 0 147 :3 0
13 :3 0 149 :3 0
14 :3 0 14b :3 0
15 :3 0 14c :3 0
14d :3 0 14e :3 0
14f :3 0 150 :3 0
151 :3 0 152 :3 0
153 :3 0 140 :3 0
74 :3 0 2b :4 0
167 1 :8 0 16b3
f9 :4 0 169e :3 0
9c :3 0 a4 :2 0
ccf 16a1 16a2 :3 0
1b :3 0 ab :3 0
16a4 16a5 0 31
:2 0 16a6 16a7 0
16a9 cd2 16b0 1b
:3 0 ab :3 0 16aa
16ab 0 fa :2 0
16ac 16ad 0 16af
cd4 16b1 16a3 16a9
0 16b2 0 16af
0 16b2 cd6 0
16b3 cd9 1706 ad
:3 0 1b :3 0 ab
:3 0 16b5 16b6 0
ac :3 0 16b7 16b8
0 16ba cdc 16bc
cde 16bb 16ba :2 0
1704 ae :3 0 1b
:3 0 ab :3 0 16bf
16c0 0 ac :3 0
16c1 16c2 0 16fc
78 :3 0 79 :3 0
16c4 16c5 0 78
:3 0 88 :3 0 16c7
16c8 0 25 :2 0
2 :4 0 168 :2 0
8a :2 0 49 :2 0
ce0 16cd 16cf :3 0
ce3 16c9 16d1 16c6
16d3 16d2 :2 0 7b
:3 0 8b :3 0 16d5
16d6 0 7d :4 0
169 :4 0 80 :2 0
81 :3 0 ce7 16da
16dc :3 0 80 :2 0
94 :4 0 cea 16de
16e0 :3 0 80 :2 0
b1 :4 0 ced 16e2
16e4 :3 0 80 :2 0
ac :3 0 cf0 16e6
16e8 :3 0 80 :2 0
b2 :4 0 cf3 16ea
16ec :3 0 80 :2 0
b3 :3 0 b4 :3 0
49 :2 0 b5 :2 0
cf6 16ef 16f3 cfa
16ee 16f5 :3 0 cfd
16d7 16f7 :2 0 16f9
d00 16fa 16d4 16f9
0 16fb d02 0
16fc d04 16ff :3 0
16ff 0 16ff 16fe
16fc 16fd :6 0 1701
36 :3 0 d07 1703
d09 1702 1701 :2 0
1704 d0b :2 0 1706
0 1706 1705 16b3
1704 :6 0 1708 1
:3 0 d0e 170a 15ba
1666 0 170b 0
1708 0 170b d10
0 170c d13 170e
14b4 15b5 0 170f
0 170c 0 170f
d15 0 1711 15a
:3 0 d18 189a 85
:3 0 3e :3 0 31
:2 0 d1a 1712 1715
a4 :2 0 31 :2 0
d1f 1717 1719 :3 0
47 :3 0 8d :2 0
d22 171c 171d :3 0
171e :2 0 85 :3 0
e :3 0 31 :2 0
d24 1720 1723 a4
:2 0 31 :2 0 d29
1725 1727 :3 0 d3
:3 0 100 :3 0 d
:3 0 cf :3 0 39
:3 0 d2 :3 0 38
:3 0 cd :3 0 c
:3 0 f1 :3 0 3d
:3 0 c :3 0 74
:3 0 2b :4 0 16a
1 :8 0 174d f9
:4 0 1738 :3 0 9c
:3 0 a4 :2 0 d2e
173b 173c :3 0 1b
:3 0 ab :3 0 173e
173f 0 31 :2 0
1740 1741 0 1743
d31 174a 1b :3 0
ab :3 0 1744 1745
0 fa :2 0 1746
1747 0 1749 d33
174b 173d 1743 0
174c 0 1749 0
174c d35 0 174d
d38 17a0 ad :3 0
1b :3 0 ab :3 0
174f 1750 0 ac
:3 0 1751 1752 0
1754 d3b 1756 d3d
1755 1754 :2 0 179e
ae :3 0 1b :3 0
ab :3 0 1759 175a
0 ac :3 0 175b
175c 0 1796 78
:3 0 79 :3 0 175e
175f 0 78 :3 0
88 :3 0 1761 1762
0 25 :2 0 2
:4 0 16b :2 0 8a
:2 0 49 :2 0 d3f
1767 1769 :3 0 d42
1763 176b 1760 176d
176c :2 0 7b :3 0
8b :3 0 176f 1770
0 7d :4 0 16c
:4 0 80 :2 0 81
:3 0 d46 1774 1776
:3 0 80 :2 0 94
:4 0 d49 1778 177a
:3 0 80 :2 0 b1
:4 0 d4c 177c 177e
:3 0 80 :2 0 ac
:3 0 d4f 1780 1782
:3 0 80 :2 0 b2
:4 0 d52 1784 1786
:3 0 80 :2 0 b3
:3 0 b4 :3 0 49
:2 0 b5 :2 0 d55
1789 178d d59 1788
178f :3 0 d5c 1771
1791 :2 0 1793 d5f
1794 176e 1793 0
1795 d61 0 1796
d63 1799 :3 0 1799
0 1799 1798 1796
1797 :6 0 179b 38
:3 0 d66 179d d68
179c 179b :2 0 179e
d6a :2 0 17a0 0
17a0 179f 174d 179e
:6 0 17a2 1 :3 0
d6d 181a d3 :3 0
100 :3 0 d :3 0
cf :3 0 39 :3 0
d2 :3 0 38 :3 0
cd :3 0 c :3 0
74 :3 0 2b :4 0
161 1 :8 0 17c4
f9 :4 0 17af :3 0
9c :3 0 a4 :2 0
d71 17b2 17b3 :3 0
1b :3 0 ab :3 0
17b5 17b6 0 31
:2 0 17b7 17b8 0
17ba d74 17c1 1b
:3 0 ab :3 0 17bb
17bc 0 fa :2 0
17bd 17be 0 17c0
d76 17c2 17b4 17ba
0 17c3 0 17c0
0 17c3 d78 0
17c4 d7b 1817 ad
:3 0 1b :3 0 ab
:3 0 17c6 17c7 0
ac :3 0 17c8 17c9
0 17cb d7e 17cd
d80 17cc 17cb :2 0
1815 ae :3 0 1b
:3 0 ab :3 0 17d0
17d1 0 ac :3 0
17d2 17d3 0 180d
78 :3 0 79 :3 0
17d5 17d6 0 78
:3 0 88 :3 0 17d8
17d9 0 25 :2 0
2 :4 0 16d :2 0
8a :2 0 49 :2 0
d82 17de 17e0 :3 0
d85 17da 17e2 17d7
17e4 17e3 :2 0 7b
:3 0 8b :3 0 17e6
17e7 0 7d :4 0
16e :4 0 80 :2 0
81 :3 0 d89 17eb
17ed :3 0 80 :2 0
94 :4 0 d8c 17ef
17f1 :3 0 80 :2 0
b1 :4 0 d8f 17f3
17f5 :3 0 80 :2 0
ac :3 0 d92 17f7
17f9 :3 0 80 :2 0
b2 :4 0 d95 17fb
17fd :3 0 80 :2 0
b3 :3 0 b4 :3 0
49 :2 0 b5 :2 0
d98 1800 1804 d9c
17ff 1806 :3 0 d9f
17e8 1808 :2 0 180a
da2 180b 17e5 180a
0 180c da4 0
180d da6 1810 :3 0
1810 0 1810 180f
180d 180e :6 0 1812
3a :3 0 da9 1814
dab 1813 1812 :2 0
1815 dad :2 0 1817
0 1817 1816 17c4
1815 :6 0 1819 1
:3 0 db0 181b 1728
17a2 0 181c 0
1819 0 181c db2
0 181d db5 1895
d3 :3 0 100 :3 0
d :3 0 cf :3 0
39 :3 0 d2 :3 0
38 :3 0 cd :3 0
c :3 0 74 :3 0
2b :4 0 16f 1
:8 0 183f f9 :4 0
182a :3 0 9c :3 0
a4 :2 0 db9 182d
182e :3 0 1b :3 0
ab :3 0 1830 1831
0 31 :2 0 1832
1833 0 1835 dbc
183c 1b :3 0 ab
:3 0 1836 1837 0
fa :2 0 1838 1839
0 183b dbe 183d
182f 1835 0 183e
0 183b 0 183e
dc0 0 183f dc3
1892 ad :3 0 1b
:3 0 ab :3 0 1841
1842 0 ac :3 0
1843 1844 0 1846
dc6 1848 dc8 1847
1846 :2 0 1890 ae
:3 0 1b :3 0 ab
:3 0 184b 184c 0
ac :3 0 184d 184e
0 1888 78 :3 0
79 :3 0 1850 1851
0 78 :3 0 88
:3 0 1853 1854 0
25 :2 0 2 :4 0
170 :2 0 8a :2 0
49 :2 0 dca 1859
185b :3 0 dcd 1855
185d 1852 185f 185e
:2 0 7b :3 0 8b
:3 0 1861 1862 0
7d :4 0 171 :4 0
80 :2 0 81 :3 0
dd1 1866 1868 :3 0
80 :2 0 94 :4 0
dd4 186a 186c :3 0
80 :2 0 b1 :4 0
dd7 186e 1870 :3 0
80 :2 0 ac :3 0
dda 1872 1874 :3 0
80 :2 0 b2 :4 0
ddd 1876 1878 :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 de0 187b
187f de4 187a 1881
:3 0 de7 1863 1883
:2 0 1885 dea 1886
1860 1885 0 1887
dec 0 1888 dee
188b :3 0 188b 0
188b 188a 1888 1889
:6 0 188d 3c :3 0
df1 188f df3 188e
188d :2 0 1890 df5
:2 0 1892 0 1892
1891 183f 1890 :6 0
1894 1 :3 0 df8
1896 171f 181d 0
1897 0 1894 0
1897 dfa 0 1898
dfd 1899 171a 1898
0 189b 14af 1711
0 189b dff 0
189c e02 189e 11b3
14a1 0 189f 0
189c 0 189f e04
0 1d05 4d :3 0
c3 :2 0 e07 18a1
18a2 :3 0 18a3 :2 0
85 :3 0 18 :3 0
31 :2 0 e09 18a5
18a8 8f :2 0 31
:2 0 e0e 18aa 18ac
:3 0 18ad :2 0 18a4
18af 18ae :2 0 18b0
:2 0 85 :3 0 c
:3 0 31 :2 0 e11
18b2 18b5 8f :2 0
31 :2 0 e16 18b7
18b9 :3 0 18ba :2 0
18b1 18bc 18bb :2 0
45 :3 0 172 :4 0
18be 18bf 0 1d02
45 :4 0 18c1 18c2
0 1d02 3f :3 0
31 :2 0 18c4 18c5
0 1d02 3e :3 0
31 :2 0 18c7 18c8
0 1d02 85 :3 0
57 :3 0 31 :2 0
e19 18ca 18cd a4
:2 0 49 :2 0 e1e
18cf 18d1 :3 0 74
:3 0 f7 :3 0 cb
:3 0 173 :3 0 b3
:3 0 f3 :3 0 173
:3 0 b3 :3 0 f5
:3 0 174 :3 0 175
:3 0 14d :3 0 14f
:3 0 46 :3 0 3d
:3 0 2e :3 0 59
:3 0 5a :3 0 5e
:3 0 5f :3 0 5b
:3 0 5d :3 0 d3
:3 0 138 :3 0 2b
:4 0 176 1 :8 0
18f2 1b :3 0 ab
:3 0 18ed 18ee 0
ac :3 0 18ef 18f0
0 18f2 e21 1945
ad :3 0 1b :3 0
ab :3 0 18f4 18f5
0 ac :3 0 18f6
18f7 0 18f9 e24
18fb e26 18fa 18f9
:2 0 1943 ae :3 0
1b :3 0 ab :3 0
18fe 18ff 0 ac
:3 0 1900 1901 0
193b 78 :3 0 79
:3 0 1903 1904 0
78 :3 0 88 :3 0
1906 1907 0 25
:2 0 2 :4 0 177
:2 0 8a :2 0 49
:2 0 e28 190c 190e
:3 0 e2b 1908 1910
1905 1912 1911 :2 0
7b :3 0 8b :3 0
1914 1915 0 7d
:4 0 178 :4 0 80
:2 0 81 :3 0 e2f
1919 191b :3 0 80
:2 0 94 :4 0 e32
191d 191f :3 0 80
:2 0 b1 :4 0 e35
1921 1923 :3 0 80
:2 0 ac :3 0 e38
1925 1927 :3 0 80
:2 0 b2 :4 0 e3b
1929 192b :3 0 80
:2 0 b3 :3 0 b4
:3 0 49 :2 0 b5
:2 0 e3e 192e 1932
e42 192d 1934 :3 0
e45 1916 1936 :2 0
1938 e48 1939 1913
1938 0 193a e4a
0 193b e4c 193e
:3 0 193e 0 193e
193d 193b 193c :6 0
1940 3e :3 0 e4f
1942 e51 1941 1940
:2 0 1943 e53 :2 0
1945 0 1945 1944
18f2 1943 :6 0 1947
1 :3 0 e56 19be
46 :3 0 2b :3 0
1948 1949 0 19bd
f7 :3 0 cb :3 0
173 :3 0 b3 :3 0
f3 :3 0 173 :3 0
b3 :3 0 f5 :3 0
174 :3 0 175 :3 0
14d :3 0 14f :3 0
3d :3 0 2e :3 0
59 :3 0 5a :3 0
5e :3 0 5f :3 0
5b :3 0 5d :3 0
d3 :3 0 74 :3 0
46 :4 0 179 1
:8 0 1968 1b :3 0
ab :3 0 1963 1964
0 ac :3 0 1965
1966 0 1968 e58
19bb ad :3 0 1b
:3 0 ab :3 0 196a
196b 0 ac :3 0
196c 196d 0 196f
e5b 1971 e5d 1970
196f :2 0 19b9 ae
:3 0 1b :3 0 ab
:3 0 1974 1975 0
ac :3 0 1976 1977
0 19b1 78 :3 0
79 :3 0 1979 197a
0 78 :3 0 88
:3 0 197c 197d 0
25 :2 0 2 :4 0
17a :2 0 8a :2 0
49 :2 0 e5f 1982
1984 :3 0 e62 197e
1986 197b 1988 1987
:2 0 7b :3 0 8b
:3 0 198a 198b 0
7d :4 0 17b :4 0
80 :2 0 81 :3 0
e66 198f 1991 :3 0
80 :2 0 94 :4 0
e69 1993 1995 :3 0
80 :2 0 b1 :4 0
e6c 1997 1999 :3 0
80 :2 0 ac :3 0
e6f 199b 199d :3 0
80 :2 0 b2 :4 0
e72 199f 19a1 :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 e75 19a4
19a8 e79 19a3 19aa
:3 0 e7c 198c 19ac
:2 0 19ae e7f 19af
1989 19ae 0 19b0
e81 0 19b1 e83
19b4 :3 0 19b4 0
19b4 19b3 19b1 19b2
:6 0 19b6 40 :3 0
e86 19b8 e88 19b7
19b6 :2 0 19b9 e8a
:2 0 19bb 0 19bb
19ba 1968 19b9 :6 0
19bd 1 :3 0 e8d
19bf 18d2 1947 0
19c0 0 19bd 0
19c0 e90 0 1d02
1b :3 0 ab :3 0
19c1 19c2 0 1b
:3 0 a4 :2 0 d7
:3 0 19c4 19c6 0
e95 19c5 19c8 :3 0
19c9 :2 0 85 :3 0
c :3 0 31 :2 0
e98 19cb 19ce 85
:3 0 a4 :2 0 a
:3 0 31 :2 0 e9b
19d0 19d4 ea0 19d1
19d6 :3 0 19d7 :2 0
85 :3 0 e :3 0
31 :2 0 ea3 19d9
19dc a4 :2 0 31
:2 0 ea8 19de 19e0
:3 0 19e1 :2 0 19d8
19e3 19e2 :2 0 85
:3 0 d :3 0 31
:2 0 eab 19e5 19e8
a4 :2 0 ed :2 0
eb0 19ea 19ec :3 0
19ed :2 0 19e4 19ef
19ee :2 0 73 :3 0
75 :3 0 70 :3 0
72 :3 0 6a :3 0
2d :3 0 74 :3 0
46 :4 0 17c 1
:8 0 1a0f f9 :4 0
19fa :3 0 9c :3 0
a4 :2 0 eb5 19fd
19fe :3 0 1b :3 0
ab :3 0 1a00 1a01
0 31 :2 0 1a02
1a03 0 1a05 eb8
1a0c 1b :3 0 ab
:3 0 1a06 1a07 0
fa :2 0 1a08 1a09
0 1a0b eba 1a0d
19ff 1a05 0 1a0e
0 1a0b 0 1a0e
ebc 0 1a0f ebf
1a62 ad :3 0 1b
:3 0 ab :3 0 1a11
1a12 0 ac :3 0
1a13 1a14 0 1a16
ec2 1a18 ec4 1a17
1a16 :2 0 1a60 ae
:3 0 1b :3 0 ab
:3 0 1a1b 1a1c 0
ac :3 0 1a1d 1a1e
0 1a58 78 :3 0
79 :3 0 1a20 1a21
0 78 :3 0 88
:3 0 1a23 1a24 0
25 :2 0 2 :4 0
17d :2 0 8a :2 0
49 :2 0 ec6 1a29
1a2b :3 0 ec9 1a25
1a2d 1a22 1a2f 1a2e
:2 0 7b :3 0 8b
:3 0 1a31 1a32 0
7d :4 0 17e :4 0
80 :2 0 81 :3 0
ecd 1a36 1a38 :3 0
80 :2 0 94 :4 0
ed0 1a3a 1a3c :3 0
80 :2 0 b1 :4 0
ed3 1a3e 1a40 :3 0
80 :2 0 ac :3 0
ed6 1a42 1a44 :3 0
80 :2 0 b2 :4 0
ed9 1a46 1a48 :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 edc 1a4b
1a4f ee0 1a4a 1a51
:3 0 ee3 1a33 1a53
:2 0 1a55 ee6 1a56
1a30 1a55 0 1a57
ee8 0 1a58 eea
1a5b :3 0 1a5b 0
1a5b 1a5a 1a58 1a59
:6 0 1a5d 42 :3 0
eed 1a5f eef 1a5e
1a5d :2 0 1a60 ef1
:2 0 1a62 0 1a62
1a61 1a0f 1a60 :6 0
1adc 1 :3 0 d3
:3 0 cd :3 0 f7
:3 0 109 :3 0 17f
:3 0 71 :3 0 73
:3 0 6a :3 0 2d
:3 0 74 :3 0 46
:3 0 74 :3 0 46
:4 0 180 1 :8 0
1a87 f9 :4 0 1a72
:3 0 9c :3 0 a4
:2 0 ef6 1a75 1a76
:3 0 1b :3 0 ab
:3 0 1a78 1a79 0
31 :2 0 1a7a 1a7b
0 1a7d ef9 1a84
1b :3 0 ab :3 0
1a7e 1a7f 0 fa
:2 0 1a80 1a81 0
1a83 efb 1a85 1a77
1a7d 0 1a86 0
1a83 0 1a86 efd
0 1a87 f00 1ada
ad :3 0 1b :3 0
ab :3 0 1a89 1a8a
0 ac :3 0 1a8b
1a8c 0 1a8e f03
1a90 f05 1a8f 1a8e
:2 0 1ad8 ae :3 0
1b :3 0 ab :3 0
1a93 1a94 0 ac
:3 0 1a95 1a96 0
1ad0 78 :3 0 79
:3 0 1a98 1a99 0
78 :3 0 88 :3 0
1a9b 1a9c 0 25
:2 0 2 :4 0 181
:2 0 8a :2 0 49
:2 0 f07 1aa1 1aa3
:3 0 f0a 1a9d 1aa5
1a9a 1aa7 1aa6 :2 0
7b :3 0 8b :3 0
1aa9 1aaa 0 7d
:4 0 182 :4 0 80
:2 0 81 :3 0 f0e
1aae 1ab0 :3 0 80
:2 0 94 :4 0 f11
1ab2 1ab4 :3 0 80
:2 0 b1 :4 0 f14
1ab6 1ab8 :3 0 80
:2 0 ac :3 0 f17
1aba 1abc :3 0 80
:2 0 b2 :4 0 f1a
1abe 1ac0 :3 0 80
:2 0 b3 :3 0 b4
:3 0 49 :2 0 b5
:2 0 f1d 1ac3 1ac7
f21 1ac2 1ac9 :3 0
f24 1aab 1acb :2 0
1acd f27 1ace 1aa8
1acd 0 1acf f29
0 1ad0 f2b 1ad3
:3 0 1ad3 0 1ad3
1ad2 1ad0 1ad1 :6 0
1ad5 44 :3 0 f2e
1ad7 f30 1ad6 1ad5
:2 0 1ad8 f32 :2 0
1ada 0 1ada 1ad9
1a87 1ad8 :6 0 1adc
1 :3 0 f35 1ccf
120 :3 0 6f :4 0
1ae0 :2 0 1cce 1ade
1ae1 :3 0 183 :3 0
6f :3 0 40 :3 0
43 :3 0 3e :4 0
1ae8 :2 0 1be8 1ae3
1ae9 :3 0 f38 :3 0
6f :3 0 f9 :3 0
1aea 1aeb :3 0 1b
:3 0 ab :3 0 1aed
1aee 0 31 :2 0
1aef 1af0 0 1af2
f3c 1af9 1b :3 0
ab :3 0 1af3 1af4
0 fa :2 0 1af5
1af6 0 1af8 f3e
1afa 1aec 1af2 0
1afb 0 1af8 0
1afb f40 0 1be8
1b :3 0 ab :3 0
1afc 1afd 0 c0
:2 0 31 :2 0 f45
1aff 1b01 :3 0 184
:8 0 1b05 f48 1b06
1b02 1b05 0 1b07
f4a 0 1be8 3f
:3 0 85 :3 0 3f
:3 0 31 :2 0 f4c
1b09 1b0c 8a :2 0
85 :3 0 40 :3 0
31 :2 0 f4f 1b0f
1b12 f52 1b0e 1b14
:3 0 1b08 1b15 0
1be8 85 :3 0 3e
:3 0 31 :2 0 f55
1b17 1b1a a4 :2 0
31 :2 0 f5a 1b1c
1b1e :3 0 85 :3 0
3f :3 0 31 :2 0
f5d 1b20 1b23 85
:3 0 8f :2 0 42
:3 0 31 :2 0 f60
1b25 1b29 f65 1b26
1b2b :3 0 85 :3 0
40 :3 0 31 :2 0
f68 1b2d 1b30 85
:3 0 185 :2 0 42
:3 0 31 :2 0 f6b
1b32 1b36 f70 1b33
1b38 :3 0 40 :3 0
85 :3 0 42 :3 0
31 :2 0 f73 1b3b
1b3e 1b3a 1b3f 0
1b41 f76 1b5a 40
:3 0 85 :3 0 42
:3 0 31 :2 0 f78
1b43 1b46 d8 :2 0
85 :3 0 3f :3 0
31 :2 0 f7b 1b49
1b4c f7e 1b48 1b4e
:3 0 8a :2 0 85
:3 0 40 :3 0 31
:2 0 f81 1b51 1b54
f84 1b50 1b56 :3 0
1b42 1b57 0 1b59
f87 1b5b 1b39 1b41
0 1b5c 0 1b59
0 1b5c f89 0
1b5d f8c 1b5e 1b2c
1b5d 0 1b5f f8e
0 1bd3 73 :3 0
75 :3 0 40 :3 0
72 :3 0 6f :4 0
186 1 :8 0 1b7b
f9 :4 0 1b66 :3 0
9c :3 0 a4 :2 0
f92 1b69 1b6a :3 0
1b :3 0 ab :3 0
1b6c 1b6d 0 31
:2 0 1b6e 1b6f 0
1b71 f95 1b78 1b
:3 0 ab :3 0 1b72
1b73 0 fa :2 0
1b74 1b75 0 1b77
f97 1b79 1b6b 1b71
0 1b7a 0 1b77
0 1b7a f99 0
1b7b f9c 1bce ad
:3 0 1b :3 0 ab
:3 0 1b7d 1b7e 0
ac :3 0 1b7f 1b80
0 1b82 f9f 1b84
fa1 1b83 1b82 :2 0
1bcc ae :3 0 1b
:3 0 ab :3 0 1b87
1b88 0 ac :3 0
1b89 1b8a 0 1bc4
78 :3 0 79 :3 0
1b8c 1b8d 0 78
:3 0 88 :3 0 1b8f
1b90 0 25 :2 0
2 :4 0 187 :2 0
8a :2 0 49 :2 0
fa3 1b95 1b97 :3 0
fa6 1b91 1b99 1b8e
1b9b 1b9a :2 0 7b
:3 0 8b :3 0 1b9d
1b9e 0 7d :4 0
188 :4 0 80 :2 0
81 :3 0 faa 1ba2
1ba4 :3 0 80 :2 0
94 :4 0 fad 1ba6
1ba8 :3 0 80 :2 0
b1 :4 0 fb0 1baa
1bac :3 0 80 :2 0
ac :3 0 fb3 1bae
1bb0 :3 0 80 :2 0
b2 :4 0 fb6 1bb2
1bb4 :3 0 80 :2 0
b3 :3 0 b4 :3 0
49 :2 0 b5 :2 0
fb9 1bb7 1bbb fbd
1bb6 1bbd :3 0 fc0
1b9f 1bbf :2 0 1bc1
fc3 1bc2 1b9c 1bc1
0 1bc3 fc5 0
1bc4 fc7 1bc7 :3 0
1bc7 0 1bc7 1bc6
1bc4 1bc5 :6 0 1bc9
47 :3 0 fca 1bcb
fcc 1bca 1bc9 :2 0
1bcc fce :2 0 1bce
0 1bce 1bcd 1b7b
1bcc :6 0 1bd3 46
:3 0 45 :3 0 43
:3 0 1bd0 1bd1 0
1bd3 fd1 1bd4 1b1f
1bd3 0 1bd5 fd5
0 1be8 85 :3 0
3f :3 0 31 :2 0
fd7 1bd6 1bd9 85
:3 0 185 :2 0 42
:3 0 31 :2 0 fda
1bdb 1bdf fdf 1bdc
1be1 :3 0 184 :8 0
1be5 fe2 1be6 1be2
1be5 0 1be7 fe4
0 1be8 fe6 1bea
183 :4 0 1be8 :4 0
1cce 121 :3 0 6f
:4 0 1bee :2 0 1bf4
1bec 0 1b :3 0
ab :3 0 1bef 1bf0
0 ac :3 0 1bf1
1bf2 0 1bf4 fed
1c47 ad :3 0 1b
:3 0 ab :3 0 1bf6
1bf7 0 ac :3 0
1bf8 1bf9 0 1bfb
ff0 1bfd ff2 1bfc
1bfb :2 0 1c45 ae
:3 0 1b :3 0 ab
:3 0 1c00 1c01 0
ac :3 0 1c02 1c03
0 1c3d 78 :3 0
79 :3 0 1c05 1c06
0 78 :3 0 88
:3 0 1c08 1c09 0
25 :2 0 2 :4 0
189 :2 0 8a :2 0
49 :2 0 ff4 1c0e
1c10 :3 0 ff7 1c0a
1c12 1c07 1c14 1c13
:2 0 7b :3 0 8b
:3 0 1c16 1c17 0
7d :4 0 18a :4 0
80 :2 0 81 :3 0
ffb 1c1b 1c1d :3 0
80 :2 0 94 :4 0
ffe 1c1f 1c21 :3 0
80 :2 0 b1 :4 0
1001 1c23 1c25 :3 0
80 :2 0 ac :3 0
1004 1c27 1c29 :3 0
80 :2 0 b2 :4 0
1007 1c2b 1c2d :3 0
80 :2 0 b3 :3 0
b4 :3 0 49 :2 0
b5 :2 0 100a 1c30
1c34 100e 1c2f 1c36
:3 0 1011 1c18 1c38
:2 0 1c3a 1014 1c3b
1c15 1c3a 0 1c3c
1016 0 1c3d 1018
1c40 :3 0 1c40 0
1c40 1c3f 1c3d 1c3e
:6 0 1c42 49 :3 0
101b 1c44 101d 1c43
1c42 :2 0 1c45 101f
:2 0 1c47 0 1c47
1c46 1bf4 1c45 :6 0
1cce 1 :3 0 85
:3 0 3f :3 0 31
:2 0 1022 1c49 1c4c
8f :2 0 31 :2 0
1027 1c4e 1c50 :3 0
1c51 :2 0 45 :3 0
c3 :2 0 102a 1c54
1c55 :3 0 1c56 :2 0
1c57 :2 0 1c52 1c59
1c58 :2 0 d3 :3 0
109 :3 0 45 :3 0
74 :3 0 46 :4 0
18b 1 :8 0 1c76
f9 :4 0 1c61 :3 0
9c :3 0 a4 :2 0
102e 1c64 1c65 :3 0
1b :3 0 ab :3 0
1c67 1c68 0 31
:2 0 1c69 1c6a 0
1c6c 1031 1c73 1b
:3 0 ab :3 0 1c6d
1c6e 0 fa :2 0
1c6f 1c70 0 1c72
1033 1c74 1c66 1c6c
0 1c75 0 1c72
0 1c75 1035 0
1c76 1038 1cc9 ad
:3 0 1b :3 0 ab
:3 0 1c78 1c79 0
ac :3 0 1c7a 1c7b
0 1c7d 103b 1c7f
103d 1c7e 1c7d :2 0
1cc7 ae :3 0 1b
:3 0 ab :3 0 1c82
1c83 0 ac :3 0
1c84 1c85 0 1cbf
78 :3 0 79 :3 0
1c87 1c88 0 78
:3 0 88 :3 0 1c8a
1c8b 0 25 :2 0
2 :4 0 18c :2 0
8a :2 0 49 :2 0
103f 1c90 1c92 :3 0
1042 1c8c 1c94 1c89
1c96 1c95 :2 0 7b
:3 0 8b :3 0 1c98
1c99 0 7d :4 0
18d :4 0 80 :2 0
81 :3 0 1046 1c9d
1c9f :3 0 80 :2 0
94 :4 0 1049 1ca1
1ca3 :3 0 80 :2 0
b1 :4 0 104c 1ca5
1ca7 :3 0 80 :2 0
ac :3 0 104f 1ca9
1cab :3 0 80 :2 0
b2 :4 0 1052 1cad
1caf :3 0 80 :2 0
b3 :3 0 b4 :3 0
49 :2 0 b5 :2 0
1055 1cb2 1cb6 1059
1cb1 1cb8 :3 0 105c
1c9a 1cba :2 0 1cbc
105f 1cbd 1c97 1cbc
0 1cbe 1061 0
1cbf 1063 1cc2 :3 0
1cc2 0 1cc2 1cc1
1cbf 1cc0 :6 0 1cc4
4b :3 0 1066 1cc6
1068 1cc5 1cc4 :2 0
1cc7 106a :2 0 1cc9
0 1cc9 1cc8 1c76
1cc7 :6 0 1ccb 1
:3 0 106d 1ccc 1c5a
1ccb 0 1ccd 106f
0 1cce 1071 1cd0
19f0 1adc 0 1cd1
0 1cce 0 1cd1
1076 0 1cff 85
:3 0 57 :3 0 31
:2 0 1079 1cd2 1cd5
a4 :2 0 49 :2 0
107e 1cd7 1cd9 :3 0
ef :3 0 85 :3 0
f :3 0 31 :2 0
1081 1cdc 1cdf 27
:2 0 1084 1cdb 1ce2
1ce3 :2 0 a4 :2 0
27 :2 0 1089 1ce5
1ce7 :3 0 54 :3 0
18e :3 0 46 :3 0
2f :3 0 108c 1cea
1ced 1ce9 1cee 0
1cf0 108f 1cf9 54
:3 0 18f :3 0 46
:3 0 2f :3 0 1091
1cf2 1cf5 1cf1 1cf6
0 1cf8 1094 1cfa
1ce8 1cf0 0 1cfb
0 1cf8 0 1cfb
1096 0 1cfc 1099
1cfd 1cda 1cfc 0
1cfe 109b 0 1cff
109d 1d00 19ca 1cff
0 1d01 10a0 0
1d02 10a2 1d03 18bd
1d02 0 1d04 10a9
0 1d05 10ab 1d06
76f 1d05 0 1d07
10b9 0 1d17 1b
:3 0 ab :3 0 1d08
1d09 0 a4 :2 0
fa :2 0 10bd 1d0b
1d0d :3 0 1b :3 0
ab :3 0 1d0f 1d10
0 31 :2 0 1d11
1d12 0 1d14 10c0
1d15 1d0e 1d14 0
1d16 10c2 0 1d17
10c4 1d18 49f 1d17
0 1d19 10c8 0
1d68 78 :3 0 79
:3 0 1d1a 1d1b 0
78 :3 0 88 :3 0
1d1d 1d1e 0 190
:2 0 2 :4 0 191
:2 0 8a :2 0 49
:2 0 10ca 1d23 1d25
:3 0 10cd 1d1f 1d27
1d1c 1d29 1d28 :2 0
7b :3 0 8b :3 0
1d2b 1d2c 0 7d
:4 0 192 :4 0 80
:2 0 81 :3 0 10d1
1d30 1d32 :3 0 80
:2 0 94 :4 0 10d4
1d34 1d36 :3 0 80
:2 0 95 :4 0 10d7
1d38 1d3a :3 0 80
:2 0 85 :3 0 66
:3 0 1b :3 0 ab
:3 0 1d3f 1d40 0
10da 1d3e 1d42 86
:4 0 10dc 1d3d 1d45
10df 1d3c 1d47 :3 0
80 :2 0 db :4 0
10e2 1d49 1d4b :3 0
80 :2 0 83 :4 0
10e5 1d4d 1d4f :3 0
80 :2 0 3 :3 0
10e8 1d51 1d53 :3 0
80 :2 0 83 :4 0
10eb 1d55 1d57 :3 0
10ee 1d2d 1d59 :2 0
1d5b 10f1 1d5c 1d2a
1d5b 0 1d5d 10f3
0 1d68 9a :3 0
193 :3 0 1d5e 1d5f
0 1b :3 0 ab
:3 0 1d61 1d62 0
3 :3 0 9c :3 0
10f5 1d60 1d66 :2 0
1d68 10f9 1e09 1b
:3 0 194 :2 0 4
1d69 1d6a 0 78
:3 0 79 :3 0 1d6c
1d6d 0 78 :3 0
88 :3 0 1d6f 1d70
0 25 :2 0 2
:4 0 195 :2 0 8a
:2 0 49 :2 0 110f
1d75 1d77 :3 0 1112
1d71 1d79 1d6e 1d7b
1d7a :2 0 7b :3 0
8b :3 0 1d7d 1d7e
0 7d :4 0 196
:4 0 80 :2 0 81
:3 0 1116 1d82 1d84
:3 0 1119 1d7f 1d86
:2 0 1d9f 7b :3 0
8b :3 0 1d88 1d89
0 7d :4 0 197
:4 0 80 :2 0 1b
:3 0 198 :3 0 1d8e
1d8f 0 111c 1d8d
1d91 :3 0 80 :2 0
db :4 0 111f 1d93
1d95 :3 0 80 :2 0
1b :3 0 199 :3 0
1d98 1d99 0 1122
1d97 1d9b :3 0 1125
1d8a 1d9d :2 0 1d9f
1128 1da0 1d7c 1d9f
0 1da1 112b 0
1db1 120 :3 0 77
:3 0 1b :3 0 198
:3 0 19a :3 0 1b
:3 0 199 :3 0 3
:3 0 ba :4 0 19b
1 :a 0 1dad :2 0
1db1 1da3 1dab :2 0
1a :3 0 77 :3 0
1daf :2 0 1db1 112d
1db3 1131 1db2 1db1
:2 0 1e07 ae :3 0
1b :3 0 ab :3 0
1db6 1db7 0 ac
:3 0 1db8 1db9 0
1dff 78 :3 0 79
:3 0 1dbb 1dbc 0
78 :3 0 88 :3 0
1dbe 1dbf 0 25
:2 0 2 :4 0 19c
:2 0 8a :2 0 49
:2 0 1133 1dc4 1dc6
:3 0 1136 1dc0 1dc8
1dbd 1dca 1dc9 :2 0
7b :3 0 8b :3 0
1dcc 1dcd 0 7d
:4 0 19d :4 0 80
:2 0 81 :3 0 113a
1dd1 1dd3 :3 0 113d
1dce 1dd5 :2 0 1def
7b :3 0 8b :3 0
1dd7 1dd8 0 7d
:4 0 b1 :4 0 80
:2 0 ac :3 0 1140
1ddc 1dde :3 0 80
:2 0 b2 :4 0 1143
1de0 1de2 :3 0 80
:2 0 b3 :3 0 b4
:3 0 49 :2 0 b5
:2 0 1146 1de5 1de9
114a 1de4 1deb :3 0
114d 1dd9 1ded :2 0
1def 1150 1df0 1dcb
1def 0 1df1 1153
0 1dff 120 :3 0
77 :3 0 19a :3 0
1b :3 0 199 :3 0
3 :3 0 ba :4 0
19e 1 :a 0 1dfb
:2 0 1dff 1df3 1df9
:2 0 1a :3 0 77
:3 0 1dfd :2 0 1dff
1155 1e02 :3 0 1e02
0 1e02 1e01 1dff
1e00 :6 0 1e04 1
:3 0 115a 1e06 115c
1e05 1e04 :2 0 1e07
115e :2 0 1e09 1161
1e09 1e08 1d68 1e07
:6 0 1e0a :2 0 2
5c 1e09 1e0d :3 0
1e0c 1e0a 1e0e :8 0

119e
4
:3 0 1 3 1
7 1 b 1
f 1 13 1
17 1 1b 1
1f 1 23 1
27 1 2b 1
2f 1 33 1
37 1 3b 1
3f 1 43 1
47 1 4b 1
4f 1 53 15
6 a e 12
16 1a 1e 22
26 2a 2e 32
36 3a 3e 42
46 4a 4e 52
56 1 60 1
5e 1 67 1
65 1 6e 1
6c 1 75 1
73 1 7c 1
7a 1 83 1
81 1 8a 1
88 1 91 1
8f 1 98 1
96 1 9f 1
9d 1 a6 1
a4 1 ad 1
ab 1 b4 1
b2 1 bb 1
b9 1 c0 1
c6 1 cc 1
d2 1 d8 1
de 1 e4 1
ea 1 f0 1
f6 1 fc 1
102 1 108 1
10e 1 114 1
11a 1 120 1
125 1 12c 1
12a 1 133 1
131 1 13a 1
138 1 13f 1
144 1 149 1
14e 1 153 1
158 1 15f 1
15d 1 164 1
16a 1 170 1
176 1 17c 1
182 1 188 1
18e 1 193 1
198 1 19e 1
1a4 1 1aa 1
1b0 1 1b6 1
1cf 1 1e4 1
1ea 1 1f7 1
203 2 206 208
2 20b 20d 2
20f 211 2 213
215 2 217 219
1 21f 2 221
222 2 21b 224
2 226 228 2
22a 22c 2 22e
230 2 232 234
2 236 238 2
23a 23c 2 23e
240 2 242 244
2 246 248 2
24a 24c 2 24e
250 2 252 254
2 256 258 1
25e 2 260 261
2 25a 263 2
265 267 2 269
26b 2 26d 26f
2 271 273 2
275 277 1 27d
2 27f 280 2
279 282 2 284
286 1 28c 2
28e 28f 2 288
291 2 293 295
1 29b 2 29d
29e 2 297 2a0
2 2a2 2a4 1
2aa 2 2ac 2ad
2 2a6 2af 2
2b1 2b3 1 2b9
2 2bb 2bc 2
2b5 2be 2 2c0
2c2 1 2c8 2
2ca 2cb 2 2c4
2cd 2 2cf 2d1
2 2d3 2d5 2
2d7 2d9 2 2db
2dd 2 2df 2e1
2 2e3 2e5 2
2e7 2e9 2 2eb
2ed 2 2ef 2f1
1 2f7 2 2f9
2fa 2 2f3 2fc
2 2fe 300 1
306 2 308 309
2 302 30b 2
30d 30f 2 311
313 2 315 317
2 319 31b 2
31d 31f 2 321
323 2 325 327
2 329 32b 2
32d 32f 1 335
2 337 338 2
331 33a 2 33c
33e 2 340 342
2 344 346 2
348 34a 2 34c
34e 4 1ff 205
20a 350 1 352
1 354 2 35e
360 3 35c 35d
362 2 36b 36d
2 36a 36f 1
371 1 373 1
375 2 37c 37d
1 381 2 37f
381 2 386 387
1 38b 2 389
38b 2 39c 39e
3 39a 39b 3a0
2 3a9 3ab 2
3ad 3af 2 3b1
3b3 1 3bb 2
3bd 3be 2 3b5
3c0 2 3a8 3c2
1 3c4 1 3c6
2 3d0 3d2 3
3ce 3cf 3d4 2
3dd 3df 2 3dc
3e1 1 3e3 1
3e5 2 3ec 3ed
3 3c7 3e6 3ef
1 3f1 1 3f3
1 3f9 1 401
2 411 413 3
40f 410 415 2
41e 420 2 422
424 2 426 428
1 430 2 432
433 2 42a 435
2 41d 437 1
439 1 43b 2
445 447 3 443
444 449 2 452
454 2 451 456
1 458 1 45a
2 461 462 3
43c 45b 464 1
466 1 494 2
496 497 1 49d
2 499 49d 2
4a1 4a2 1 4a6
2 4a4 4a6 2
4ab 4ac 1 4b0
2 4ae 4b0 1
4b6 2 4c3 4c5
3 4c1 4c2 4c7
2 4d0 4d2 2
4d4 4d6 2 4d8
4da 1 4e2 2
4e4 4e5 2 4dc
4e7 2 4cf 4e9
1 4eb 1 4ed
2 4f7 4f9 3
4f5 4f6 4fb 2
504 506 2 503
508 1 50a 1
50c 2 513 514
3 4ee 50d 516
1 518 2 52a
52f 1 536 1
531 2 549 54b
3 547 548 54d
2 556 558 2
55a 55c 2 55e
560 2 562 564
2 566 568 3
56d 56e 56f 2
56a 571 2 555
573 1 575 1
577 2 540 578
1 57c 1 53b
2 539 580 2
58d 58e 1 590
6 598 599 59a
59b 59c 59d 1
5a5 2 5a3 5a5
2 5aa 5ab 1
5af 2 5ad 5af
2 5b6 5b7 1
5bb 2 5b9 5bb
2 5cc 5d1 1
5d8 1 5d3 2
5eb 5ed 3 5e9
5ea 5ef 2 5f8
5fa 2 5fc 5fe
2 600 602 2
604 606 2 608
60a 3 60f 610
611 2 60c 613
2 5f7 615 1
617 1 619 2
5e2 61a 1 61e
1 5dd 2 5db
622 2 628 629
1 62d 2 62b
62d 1 639 2
644 649 1 650
1 64b 2 663
665 3 661 662
667 2 670 672
2 674 676 2
678 67a 2 67c
67e 2 680 682
3 687 688 689
2 684 68b 2
66f 68d 1 68f
1 691 2 65a
692 1 696 1
655 2 653 69a
1 69d 1 6a0
4 632 635 638
6a1 1 6a3 3
5c3 625 6a4 1
6a6 2 6a9 6aa
1 6ae 2 6ac
6ae 2 6b4 6b5
1 6b7 1 6bc
2 6ba 6bc 1
6c3 1 6c5 a
519 51c 51f 522
583 589 593 5a0
6a7 6c6 2 6e2
6e7 1 6ee 1
6e9 2 701 703
3 6ff 700 705
2 70e 710 2
712 714 2 716
718 2 71a 71c
2 71e 720 3
725 726 727 2
722 729 2 70d
72b 1 72d 1
72f 2 6f8 730
1 734 1 6f3
2 6f1 738 1
743 2 73f 743
2 748 749 2
74e 74f 1 751
2 74b 751 1
75c 2 759 75f
1 761 3 6ca
73b 762 2 764
765 2 768 769
1 76d 2 76b
76d 1 774 2
772 774 2 77f
781 3 77d 77e
783 2 78c 78e
2 790 792 2
794 796 1 79e
2 7a0 7a1 2
798 7a3 2 7a5
7a7 2 7a9 7ab
2 7ad 7af 2
7b1 7b3 2 78b
7b5 1 7b7 1
7b9 2 7c3 7c5
3 7c1 7c2 7c7
2 7d0 7d2 2
7cf 7d4 1 7d6
1 7d8 3 7df
7e0 7e1 3 7ba
7d9 7e3 1 7e5
1 7e7 1 7ec
1 7f3 1 7fa
2 809 80b 3
807 808 80d 2
816 818 2 81a
81c 2 81e 820
1 828 2 82a
82b 2 822 82d
2 815 82f 1
831 1 833 2
83d 83f 3 83b
83c 841 2 84a
84c 2 849 84e
1 850 1 852
2 859 85a 3
834 853 85c 1
85e 2 866 867
1 86b 2 869
86b 1 870 1
872 2 87a 87f
1 886 1 881
2 899 89b 3
897 898 89d 2
8a6 8a8 2 8aa
8ac 2 8ae 8b0
2 8b2 8b4 2
8b6 8b8 3 8bd
8be 8bf 2 8ba
8c1 2 8a5 8c3
1 8c5 1 8c7
2 890 8c8 1
8cc 1 88b 2
889 8d0 2 8d6
8d7 1 8db 2
8d9 8db 1 8e2
2 8e4 8e5 1
8eb 2 8ed 8ee
2 8e7 8f0 2
8f3 8f5 2 8f8
8fa 1 8fc 2
901 902 2 907
908 1 90a 2
904 90a 2 90f
912 1 916 2
918 919 2 8ff
91a 1 91c 2
923 924 2 926
927 1 92c 2
92a 92c 2 930
931 1 935 2
933 935 2 939
93a 1 93e 2
93c 93e 11 945
946 947 948 949
94a 94b 94c 94d
94e 94f 950 951
952 953 954 955
1 95d 2 95b
95d 1 972 2
971 972 1 97a
1 980 2 982
983 2 96f 984
1 98b 1 986
2 99e 9a0 3
99c 99d 9a2 2
9ab 9ad 2 9af
9b1 2 9b3 9b5
2 9b7 9b9 2
9bb 9bd 3 9c2
9c3 9c4 2 9bf
9c6 2 9aa 9c8
1 9ca 1 9cc
2 995 9cd 1
9d1 1 990 2
98e 9d5 1 9d8
1 9db 2 958
9dc 11 9e2 9e3
9e4 9e5 9e6 9e7
9e8 9e9 9ea 9eb
9ec 9ed 9ee 9ef
9f0 9f1 9f2 1
9f5 2 9f7 9f8
1 9f9 2 9fd
9fe 2 a00 a02
2 a07 a08 1
a0c 2 a0a a0c
11 a13 a14 a15
a16 a17 a18 a19
a1a a1b a1c a1d
a1e a1f a20 a21
a22 a23 1 a2b
2 a29 a2b 1
a3c 2 a3b a3c
1 a44 1 a4a
2 a4c a4d 2
a39 a4e 1 a55
1 a50 2 a68
a6a 3 a66 a67
a6c 2 a75 a77
2 a79 a7b 2
a7d a7f 2 a81
a83 2 a85 a87
3 a8c a8d a8e
2 a89 a90 2
a74 a92 1 a94
1 a96 2 a5f
a97 1 a9b 1
a5a 2 a58 a9f
1 aa2 1 aa5
2 a26 aa6 11
aac aad aae aaf
ab0 ab1 ab2 ab3
ab4 ab5 ab6 ab7
ab8 ab9 aba abb
abc 1 abf 2
ac1 ac2 2 a05
ac3 2 ac5 ac6
1 acc 2 aca
acc 2 ad2 ad3
2 ad5 ad6 1
adb 2 ad9 adb
2 ae2 ae3 2
ae8 ae9 1 aeb
2 ae5 aeb 2
af0 af1 1 af5
2 af3 af5 2
afc afd 1 b01
2 aff b01 1
b21 2 b20 b21
1 b29 1 b2f
2 b31 b32 2
b1e b33 1 b3a
1 b35 2 b4d
b4f 3 b4b b4c
b51 2 b5a b5c
2 b5e b60 2
b62 b64 2 b66
b68 2 b6a b6c
3 b71 b72 b73
2 b6e b75 2
b59 b77 1 b79
1 b7b 2 b44
b7c 1 b80 1
b3f 2 b3d b84
1 b94 2 b93
b94 1 b9c 1
ba2 2 ba4 ba5
2 b91 ba6 1
bad 1 ba8 2
bc0 bc2 3 bbe
bbf bc4 2 bcd
bcf 2 bd1 bd3
2 bd5 bd7 2
bd9 bdb 2 bdd
bdf 3 be4 be5
be6 2 be1 be8
2 bcc bea 1
bec 1 bee 2
bb7 bef 1 bf3
1 bb2 2 bb0
bf7 1 c04 2
c03 c04 1 c0c
1 c12 2 c14
c15 2 c01 c16
1 c1d 1 c18
2 c30 c32 3
c2e c2f c34 2
c3d c3f 2 c41
c43 2 c45 c47
2 c49 c4b 2
c4d c4f 3 c54
c55 c56 2 c51
c58 2 c3c c5a
1 c5c 1 c5e
2 c27 c5f 1
c63 1 c22 2
c20 c67 3 b87
bfa c6a 1 c7b
2 c7a c7b 1
c83 1 c89 2
c8b c8c 2 c78
c8d 1 c94 1
c8f 2 ca7 ca9
3 ca5 ca6 cab
2 cb4 cb6 2
cb8 cba 2 cbc
cbe 2 cc0 cc2
2 cc4 cc6 3
ccb ccc ccd 2
cc8 ccf 2 cb3
cd1 1 cd3 1
cd5 2 c9e cd6
1 cda 1 c99
2 c97 cde 1
ce1 2 ce4 ce5
1 ce6 1 ce8
2 ac7 ce9 2
cee cef 2 cf1
cf4 2 cf9 cfa
1 cfe 2 cfc
cfe 11 d05 d06
d07 d08 d09 d0a
d0b d0c d0d d0e
d0f d10 d11 d12
d13 d14 d15 1
d1d 2 d1b d1d
2 d22 d23 1
d27 2 d25 d27
2 d2e d2f 1
d33 2 d31 d33
1 d4b 2 d4a
d4b 1 d53 1
d59 2 d5b d5c
2 d48 d5d 1
d64 1 d5f 2
d77 d79 3 d75
d76 d7b 2 d84
d86 2 d88 d8a
2 d8c d8e 2
d90 d92 2 d94
d96 3 d9b d9c
d9d 2 d98 d9f
2 d83 da1 1
da3 1 da5 2
d6e da6 1 daa
1 d69 2 d67
dae 1 db1 1
db4 2 d18 db5
1 db7 2 dc9
dce 1 dd5 1
dd0 2 de8 dea
3 de6 de7 dec
2 df5 df7 2
df9 dfb 2 dfd
dff 2 e01 e03
2 e05 e07 3
e0c e0d e0e 2
e09 e10 2 df4
e12 1 e14 1
e16 2 ddf e17
1 e1b 1 dda
2 dd8 e1f 3
e27 e28 e29 1
e2d 2 e2b e2d
2 e3c e41 1
e48 1 e43 2
e5b e5d 3 e59
e5a e5f 2 e68
e6a 2 e6c e6e
2 e70 e72 2
e74 e76 2 e78
e7a 3 e7f e80
e81 2 e7c e83
2 e67 e85 1
e87 1 e89 2
e52 e8a 1 e8e
1 e4d 2 e4b
e92 6 e99 e9a
e9b e9c e9d e9e
2 ea3 ea4 1
ea8 2 ea6 ea8
2 ec0 ec1 1
ecc 1 ed2 2
ed4 ed5 2 ed9
edf 1 ee6 1
ee1 2 ef9 efb
3 ef7 ef8 efd
2 f06 f08 2
f0a f0c 2 f0e
f10 2 f12 f14
2 f16 f18 3
f1d f1e f1f 2
f1a f21 2 f05
f23 1 f25 1
f27 2 ef0 f28
1 f2c 1 eeb
2 ee9 f30 2
f3a f3f 1 f46
1 f41 2 f59
f5b 3 f57 f58
f5d 2 f66 f68
2 f6a f6c 2
f6e f70 2 f72
f74 2 f76 f78
3 f7d f7e f7f
2 f7a f81 2
f65 f83 1 f85
1 f87 2 f50
f88 1 f8c 1
f4b 2 f49 f90
2 f9d f9f 3
fa1 fa2 fa3 2
fa9 fab 3 fad
fae faf 2 fb5
fb7 3 fb9 fba
fbb 2 fc1 fc3
3 fc5 fc6 fc7
f fcd fce fcf
fd0 fd1 fd2 fd3
fd4 fd5 fd6 fd7
fd8 fd9 fda fdb
2 fe7 fe9 3
fe5 fe6 feb 2
ff4 ff6 2 ff8
ffa 2 ffc ffe
1 1004 2 1006
1007 2 1000 1009
2 100b 100d 2
100f 1011 2 1013
1015 2 1017 1019
2 ff3 101b 1
101d 1 101f 2
1029 102b 3 1027
1028 102d 2 1036
1038 2 1035 103a
1 103c 1 103e
3 1043 1044 1045
14 ead eb0 eb3
eb6 eb9 ebc ec2
ed6 f33 f93 f97
f9a fa6 fb2 fbe
fca fde 1020 103f
1047 1 1049 5
e34 e37 e95 ea1
104a 1 104c 3
dbe e22 104d 1
104f 2 1053 1054
2 1056 1058 11
1060 1061 1062 1063
1064 1065 1066 1067
1068 1069 106a 106b
106c 106d 106e 106f
1070 1 1078 2
1076 1078 2 107d
107e 1 1082 2
1080 1082 2 1089
108a 1 108e 2
108c 108e 1 10a2
2 10a1 10a2 1
10aa 1 10b0 2
10b2 10b3 2 109f
10b4 1 10bb 1
10b6 2 10ce 10d0
3 10cc 10cd 10d2
2 10db 10dd 2
10df 10e1 2 10e3
10e5 2 10e7 10e9
2 10eb 10ed 3
10f2 10f3 10f4 2
10ef 10f6 2 10da
10f8 1 10fa 1
10fc 2 10c5 10fd
1 1101 1 10c0
2 10be 1105 1
1108 1 110b 4
1050 105b 1073 110c
2 110e 110f 2
cf7 1110 2 1112
1113 1 1119 2
1117 1119 2 1124
1126 3 1122 1123
1128 2 1131 1133
2 1135 1137 2
1139 113b 1 1143
2 1145 1146 2
113d 1148 2 114a
114c 2 114e 1150
2 1152 1154 2
1156 1158 2 1130
115a 1 115c 1
115e 2 1168 116a
3 1166 1167 116c
2 1175 1177 2
1174 1179 1 117b
1 117d 3 1184
1185 1186 3 115f
117e 1188 1 118a
2 1190 1191 1
1195 2 1193 1195
2 119a 119b 1
119f 2 119d 119f
1 11a7 1 11a9
2 11ac 11ad 1
11b1 2 11af 11b1
2 11b6 11b7 2
11b9 11ba 1 11bf
2 11bd 11bf 1
11c2 1 11c7 1
11e4 2 11e3 11e4
1 11ec 1 11f2
2 11f4 11f5 2
11e1 11f6 1 11fd
1 11f8 2 1210
1212 3 120e 120f
1214 2 121d 121f
2 1221 1223 2
1225 1227 2 1229
122b 2 122d 122f
3 1234 1235 1236
2 1231 1238 2
121c 123a 1 123c
1 123e 2 1207
123f 1 1243 1
1202 2 1200 1247
1 124a 1 1259
2 1258 1259 1
1261 1 1267 2
1269 126a 2 1256
126b 1 1272 1
126d 2 1285 1287
3 1283 1284 1289
2 1292 1294 2
1296 1298 2 129a
129c 2 129e 12a0
2 12a2 12a4 3
12a9 12aa 12ab 2
12a6 12ad 2 1291
12af 1 12b1 1
12b3 2 127c 12b4
1 12b8 1 1277
2 1275 12bc 1
12bf 2 12c2 12c3
1 12c4 1 12c6
1 130e 2 130d
130e 1 1316 1
131c 2 131e 131f
2 130b 1320 1
1327 1 1322 2
133a 133c 3 1338
1339 133e 2 1347
1349 2 134b 134d
2 134f 1351 2
1353 1355 2 1357
1359 3 135e 135f
1360 2 135b 1362
2 1346 1364 1
1366 1 1368 2
1331 1369 1 136d
1 132c 2 132a
1371 1 1374 1
13ae 2 13ad 13ae
1 13b6 1 13bc
2 13be 13bf 2
13ab 13c0 1 13c7
1 13c2 2 13da
13dc 3 13d8 13d9
13de 2 13e7 13e9
2 13eb 13ed 2
13ef 13f1 2 13f3
13f5 2 13f7 13f9
3 13fe 13ff 1400
2 13fb 1402 2
13e6 1404 1 1406
1 1408 2 13d1
1409 1 140d 1
13cc 2 13ca 1411
1 1414 2 1417
1418 1 1419 2
141b 141c 1 141d
2 1421 1422 1
1426 2 1424 1426
1 1435 2 1434
1435 1 143d 1
1443 2 1445 1446
2 1432 1447 1
144e 1 1449 2
1461 1463 3 145f
1460 1465 2 146e
1470 2 1472 1474
2 1476 1478 2
147a 147c 2 147e
1480 3 1485 1486
1487 2 1482 1489
2 146d 148b 1
148d 1 148f 2
1458 1490 1 1494
1 1453 2 1451
1498 1 149b 2
149f 149e 1 14a0
2 14a4 14a5 2
14a7 14a8 1 14ad
2 14ab 14ad 1
14b0 1 14b5 1
14d2 2 14d1 14d2
1 14da 1 14e0
2 14e2 14e3 2
14cf 14e4 1 14eb
1 14e6 2 14fe
1500 3 14fc 14fd
1502 2 150b 150d
2 150f 1511 2
1513 1515 2 1517
1519 2 151b 151d
3 1522 1523 1524
2 151f 1526 2
150a 1528 1 152a
1 152c 2 14f5
152d 1 1531 1
14f0 2 14ee 1535
1 1538 1 1549
2 1548 1549 1
1551 1 1557 2
1559 155a 2 1546
155b 1 1562 1
155d 2 1575 1577
3 1573 1574 1579
2 1582 1584 2
1586 1588 2 158a
158c 2 158e 1590
2 1592 1594 3
1599 159a 159b 2
1596 159d 2 1581
159f 1 15a1 1
15a3 2 156c 15a4
1 15a8 1 1567
2 1565 15ac 1
15af 2 15b2 15b3
1 15b4 1 15b6
1 15fe 2 15fd
15fe 1 1606 1
160c 2 160e 160f
2 15fb 1610 1
1617 1 1612 2
162a 162c 3 1628
1629 162e 2 1637
1639 2 163b 163d
2 163f 1641 2
1643 1645 2 1647
1649 3 164e 164f
1650 2 164b 1652
2 1636 1654 1
1656 1 1658 2
1621 1659 1 165d
1 161c 2 161a
1661 1 1664 1
16a0 2 169f 16a0
1 16a8 1 16ae
2 16b0 16b1 2
169d 16b2 1 16b9
1 16b4 2 16cc
16ce 3 16ca 16cb
16d0 2 16d9 16db
2 16dd 16df 2
16e1 16e3 2 16e5
16e7 2 16e9 16eb
3 16f0 16f1 16f2
2 16ed 16f4 2
16d8 16f6 1 16f8
1 16fa 2 16c3
16fb 1 16ff 1
16be 2 16bc 1703
1 1706 2 1709
170a 1 170b 2
170d 170e 1 170f
2 1713 1714 1
1718 2 1716 1718
1 171b 2 1721
1722 1 1726 2
1724 1726 1 173a
2 1739 173a 1
1742 1 1748 2
174a 174b 2 1737
174c 1 1753 1
174e 2 1766 1768
3 1764 1765 176a
2 1773 1775 2
1777 1779 2 177b
177d 2 177f 1781
2 1783 1785 3
178a 178b 178c 2
1787 178e 2 1772
1790 1 1792 1
1794 2 175d 1795
1 1799 1 1758
2 1756 179d 1
17a0 1 17b1 2
17b0 17b1 1 17b9
1 17bf 2 17c1
17c2 2 17ae 17c3
1 17ca 1 17c5
2 17dd 17df 3
17db 17dc 17e1 2
17ea 17ec 2 17ee
17f0 2 17f2 17f4
2 17f6 17f8 2
17fa 17fc 3 1801
1802 1803 2 17fe
1805 2 17e9 1807
1 1809 1 180b
2 17d4 180c 1
1810 1 17cf 2
17cd 1814 1 1817
2 181a 181b 1
181c 1 182c 2
182b 182c 1 1834
1 183a 2 183c
183d 2 1829 183e
1 1845 1 1840
2 1858 185a 3
1856 1857 185c 2
1865 1867 2 1869
186b 2 186d 186f
2 1871 1873 2
1875 1877 3 187c
187d 187e 2 1879
1880 2 1864 1882
1 1884 1 1886
2 184f 1887 1
188b 1 184a 2
1848 188f 1 1892
2 1895 1896 1
1897 2 189a 1899
1 189b 2 189d
189e 1 18a0 2
18a6 18a7 1 18ab
2 18a9 18ab 2
18b3 18b4 1 18b8
2 18b6 18b8 2
18cb 18cc 1 18d0
2 18ce 18d0 2
18ec 18f1 1 18f8
1 18f3 2 190b
190d 3 1909 190a
190f 2 1918 191a
2 191c 191e 2
1920 1922 2 1924
1926 2 1928 192a
3 192f 1930 1931
2 192c 1933 2
1917 1935 1 1937
1 1939 2 1902
193a 1 193e 1
18fd 2 18fb 1942
1 1945 2 1962
1967 1 196e 1
1969 2 1981 1983
3 197f 1980 1985
2 198e 1990 2
1992 1994 2 1996
1998 2 199a 199c
2 199e 19a0 3
19a5 19a6 19a7 2
19a2 19a9 2 198d
19ab 1 19ad 1
19af 2 1978 19b0
1 19b4 1 1973
2 1971 19b8 2
194a 19bb 2 19be
19bf 1 19c7 2
19c3 19c7 2 19cc
19cd 2 19d2 19d3
1 19d5 2 19cf
19d5 2 19da 19db
1 19df 2 19dd
19df 2 19e6 19e7
1 19eb 2 19e9
19eb 1 19fc 2
19fb 19fc 1 1a04
1 1a0a 2 1a0c
1a0d 2 19f9 1a0e
1 1a15 1 1a10
2 1a28 1a2a 3
1a26 1a27 1a2c 2
1a35 1a37 2 1a39
1a3b 2 1a3d 1a3f
2 1a41 1a43 2
1a45 1a47 3 1a4c
1a4d 1a4e 2 1a49
1a50 2 1a34 1a52
1 1a54 1 1a56
2 1a1f 1a57 1
1a5b 1 1a1a 2
1a18 1a5f 1 1a74
2 1a73 1a74 1
1a7c 1 1a82 2
1a84 1a85 2 1a71
1a86 1 1a8d 1
1a88 2 1aa0 1aa2
3 1a9e 1a9f 1aa4
2 1aad 1aaf 2
1ab1 1ab3 2 1ab5
1ab7 2 1ab9 1abb
2 1abd 1abf 3
1ac4 1ac5 1ac6 2
1ac1 1ac8 2 1aac
1aca 1 1acc 1
1ace 2 1a97 1acf
1 1ad3 1 1a92
2 1a90 1ad7 2
1a62 1ada 3 1ae4
1ae5 1ae6 1 1af1
1 1af7 2 1af9
1afa 1 1b00 2
1afe 1b00 1 1b04
1 1b06 2 1b0a
1b0b 2 1b10 1b11
2 1b0d 1b13 2
1b18 1b19 1 1b1d
2 1b1b 1b1d 2
1b21 1b22 2 1b27
1b28 1 1b2a 2
1b24 1b2a 2 1b2e
1b2f 2 1b34 1b35
1 1b37 2 1b31
1b37 2 1b3c 1b3d
1 1b40 2 1b44
1b45 2 1b4a 1b4b
2 1b47 1b4d 2
1b52 1b53 2 1b4f
1b55 1 1b58 2
1b5a 1b5b 1 1b5c
1 1b5e 1 1b68
2 1b67 1b68 1
1b70 1 1b76 2
1b78 1b79 2 1b65
1b7a 1 1b81 1
1b7c 2 1b94 1b96
3 1b92 1b93 1b98
2 1ba1 1ba3 2
1ba5 1ba7 2 1ba9
1bab 2 1bad 1baf
2 1bb1 1bb3 3
1bb8 1bb9 1bba 2
1bb5 1bbc 2 1ba0
1bbe 1 1bc0 1
1bc2 2 1b8b 1bc3
1 1bc7 1 1b86
2 1b84 1bcb 3
1b5f 1bce 1bd2 1
1bd4 2 1bd7 1bd8
2 1bdd 1bde 1
1be0 2 1bda 1be0
1 1be4 1 1be6
6 1ae7 1afb 1b07
1b16 1bd5 1be7 2
1bed 1bf3 1 1bfa
1 1bf5 2 1c0d
1c0f 3 1c0b 1c0c
1c11 2 1c1a 1c1c
2 1c1e 1c20 2
1c22 1c24 2 1c26
1c28 2 1c2a 1c2c
3 1c31 1c32 1c33
2 1c2e 1c35 2
1c19 1c37 1 1c39
1 1c3b 2 1c04
1c3c 1 1c40 1
1bff 2 1bfd 1c44
2 1c4a 1c4b 1
1c4f 2 1c4d 1c4f
1 1c53 1 1c63
2 1c62 1c63 1
1c6b 1 1c71 2
1c73 1c74 2 1c60
1c75 1 1c7c 1
1c77 2 1c8f 1c91
3 1c8d 1c8e 1c93
2 1c9c 1c9e 2
1ca0 1ca2 2 1ca4
1ca6 2 1ca8 1caa
2 1cac 1cae 3
1cb3 1cb4 1cb5 2
1cb0 1cb7 2 1c9b
1cb9 1 1cbb 1
1cbd 2 1c86 1cbe
1 1cc2 1 1c81
2 1c7f 1cc6 1
1cc9 1 1ccc 4
1adf 1bea 1c47 1ccd
2 1ccf 1cd0 2
1cd3 1cd4 1 1cd8
2 1cd6 1cd8 2
1cdd 1cde 2 1ce0
1ce1 1 1ce6 2
1ce4 1ce6 2 1ceb
1cec 1 1cef 2
1cf3 1cf4 1 1cf7
2 1cf9 1cfa 1
1cfb 1 1cfd 2
1cd1 1cfe 1 1d00
6 18c0 18c3 18c6
18c9 19c0 1d01 1
1d03 d 7e6 85f
864 873 8d3 91d
920 1114 118b 118e
11aa 189f 1d04 1
1d06 1 1d0c 2
1d0a 1d0c 1 1d13
1 1d15 3 766
1d07 1d16 1 1d18
2 1d22 1d24 3
1d20 1d21 1d26 2
1d2f 1d31 2 1d33
1d35 2 1d37 1d39
1 1d41 2 1d43
1d44 2 1d3b 1d46
2 1d48 1d4a 2
1d4c 1d4e 2 1d50
1d52 2 1d54 1d56
2 1d2e 1d58 1
1d5a 1 1d5c 3
1d63 1d64 1d65 15
355 374 3f2 467
46a 46d 470 473
476 479 47c 47f
482 485 488 48b
48e 491 1d19 1d5d
1d67 2 1d74 1d76
3 1d72 1d73 1d78
2 1d81 1d83 2
1d80 1d85 2 1d8c
1d90 2 1d92 1d94
2 1d96 1d9a 2
1d8b 1d9c 2 1d87
1d9e 1 1da0 3
1da1 1dac 1db0 1
1d6b 2 1dc3 1dc5
3 1dc1 1dc2 1dc7
2 1dd0 1dd2 2
1dcf 1dd4 2 1ddb
1ddd 2 1ddf 1de1
3 1de6 1de7 1de8
2 1de3 1dea 2
1dda 1dec 2 1dd6
1dee 1 1df0 4
1dba 1df1 1dfa 1dfe
1 1e02 1 1db5
2 1db3 1e06 3c
63 6a 71 78
7f 86 8d 94
9b a2 a9 b0
b7 be c4 ca
d0 d6 dc e2
e8 ee f4 fa
100 106 10c 112
118 11e 123 128
12f 136 13d 142
147 14c 151 156
15b 162 168 16e
174 17a 180 186
18c 191 196 19c
1a2 1a8 1ae 1b4
1ba 1d0 1e5 1ef

1
4
0 
1e0d
0
1
50
4d
52
0 1 1 1 4 1 6 1
8 1 a 1 c 1 e 1
10 1 12 1 14 1 16 1
18 1 1a 1 1c 1 1e 1
20 1 22 1 24 1 26 1
28 1 2a 1 2c 1 2e 1
30 1 32 1 34 1 36 1
38 1 3a 1 3c 1 3e 1
40 1 42 1 44 1 46 47
1 49 1 4b 1 4c4c 2929 4f20

d2 1 0
53 1 0
1bd 1 2
3b 1 0
23 1 0
13 1 0
7a 1 0
a4 1 0
16a 1 0
182 1 0
43 1 0
f6 1 0
3 1 0
12a 1 0
1b0 1 0
131 1 0
188 1 0
2f 1 0
102 1 0
19e 1 0
88 1 0
114 1 0
37 1 0
120 1 0
170 1 0
10e 1 0
7 1 0
c0 1 0
81 1 0
1b6 1 0
17c 1 0
1f 1 0
2 0 1
144 1 0
fc 1 0
e4 1 0
f 1 0
13f 1 0
4b 1 0
17 1 0
8f 1 0
176 1 0
158 1 0
1a4 1 0
96 1 0
5e 1 0
153 1 0
47 1 0
b 1 0
cc 1 0
27 1 0
193 1 0
14e 1 0
108 1 0
73 1 0
1aa 1 0
1d6 1 3
2b 1 0
b2 1 0
149 1 0
11a 1 0
4f 1 0
9d 1 0
ea 1 0
d8 1 0
33 1 0
164 1 0
ab 1 0
f0 1 0
c6 1 0
3f 1 0
198 1 0
18e 1 0
125 1 0
6c 1 0
65 1 0
b9 1 0
1b 1 0
15d 1 0
de 1 0
1ea 1 0
138 1 0
0

/
SHOW ERRORS FUNCTION SP_ASSIGN_MSP;
ALTER FUNCTION SP_ASSIGN_MSP COMPILE DEBUG;
/*
#Stored Procedure  :SP_A_RESTOPROF
#Short Description :Replace a profile with a resource
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_A_RESTOPROF ==
CREATE OR REPLACE FUNCTION SP_A_RESTOPROF wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
343
2 :e:
1FUNCTION:
1SP_A_RESTOPROF:
1PRJ_ID:
1VARCHAR2:
1PROFILE_ID001:
1RES_ID001:
1START_DATE001:
1FINISH_DATE001:
1EFFORT001:
1PLS_INTEGER:
1WORK001:
1BITFLAG001:
1EUNIT:
1UNIT:
1CONTOUR001:
1OCCURRENCE:
1CONSIDER_PLANNED_SCHED:
1WORK_TYPE:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_ID:
132:
1H_SECURITY_ID:
1H_REC_USER:
1H_ASSIGN_ID:
1H_PRJ_ASSIGN_ID:
1H_TSK_ASSIGNMENT_ID:
1H_PRJ_ASSIGNMENT_ID:
1H_CUR_PRJ_ASSIGN_ID:
1H_TASK_ID:
1H_RES_ID:
1H_PROJECT_ID:
1H_PROFILE_ID:
1H_START_DATE:
1TIMESTAMP:
1H_FINISH_DATE:
1H_TASK_START_DT:
1H_TASK_FINISH_DT:
1H_NULL_STR:
11:
1H_START_D:
1DATE:
1H_FINISH_D:
1H_RES_TYPE:
1H_RANK:
1200:
1H_START_DT:
1H_FINISH_DT:
1H_SCHED_START:
1H_SCHED_FINISH:
1H_ROLE_ID:
1H_FULL_NAME:
1255:
1H_IN:
1H_PROFILE_NAME:
1H_LABOR_CODE:
150:
1H_PARTICIPANT:
10:
1H_EVERY:
1H_EFFORT:
1H_CRI:
1H_TYPE:
1H_TYPE_ID:
1H_TOTAL:
1H_WORK:
1H_OCCURRENCE:
1H_EUNIT:
1H_UNIT:
1H_CONTOUR:
1H_DURATION:
1H_WORK_TYPE:
1H_PASSED:
1H_ASSIGNED:
1H_REMAINING:
1H_BITFLAG:
1H_PER_DAY:
1H_PER_WEEK:
1H_UPDATE:
1H_CONSIDER:
1H_PRIORITY:
1H_DAYS:
1H_OWNER:
1H_COUNT:
1H_CONTOUR_CHANGED:
1H_REG_COST:
1NUMBER:
10.00:
1H_REG_SALE:
1H_SPEC_COST:
1H_SPEC_SALE:
1H_RATE_ECHANGE:
1H_CURRENCY_ID:
1H_DONOT_REPLACE_BITS:
1H_DONOT_REPLACE_RATE:
1H_DONOT_REPLACE_COST:
1H_FROM_PRJ:
1H_SAME_LABOR:
1H_FS_PROJECT:
1I_RETURN:
1H_AND_ROLES:
14000:
1H_AND_SKILLS:
1H_TEMP_RES_COST:
1H_TEMP_VAR:
1H_TEMP_RES_TYPE:
1H_TEMP_PARTICIPANT:
1H_TEMP_SECURITY_ID:
1H_PA_TIMEPHASE:
1H_TEMP_BITFLAG:
1H_TEMP_SC_STATE:
1H_SD:
1H_FD:
1H_OLD_COST:
1H_OLD_SALE:
1H_PA_COUNT:
1H_TA_COUNT:
1H_BILLABLE:
1H_CAPITAL_EXPENSE:
1H_TPH_DELETE:
1H_TABLE_TYPE:
1H_PLANNED_EFFORT:
1H_TASK_TYPE:
1H_PERCENT_UNITS:
1H_TIMECODE_1_ID:
1H_TIMECODE_2_ID:
1H_OLD_PRJ_ASSIGN_ID:
1I_ISTASK:
1ISWBSTASK:
1COALESCE:
1L_DATA_FLAG:
1I_MANY:
1CURSOR:
1CRCAL1:
1AVAILABLE_MIN:
1CALENDAR_DAY_OF_YEAR:
1ROWNUM:
1ELEMENT_ID:
1SELECT AVAILABLE_MIN FROM CALENDAR_DAY_OF_YEAR:n         WHERE  ROWNUM <= 1 A+
1ND  ELEMENT_ID = h_res_id:
1CSA1:
1BAK_WORK:
1TASK_ASSIGNMENTS:
1PROJECT_ID:
1RESOURCE_ID:
1RESOURCE_TYPE:
1PROJECT_ASSIGNMENT_ID:
1SELECT BAK_WORK FROM TASK_ASSIGNMENTS:n         WHERE  ROWNUM <= 1 AND  PROJE+
1CT_ID = h_project_id AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0' AND:n+
1         PROJECT_ASSIGNMENT_ID IS NuLL:
1C6A:
1R:
1RES_ASSIGNMENT_ID:
1T:
1TSK_ASSIGNMENT_ID:
1TASK_ID:
1ELEMENT_TYPE:
1PLANNED_WORK:
1UNITS_MSP:
1TO_DATE:
1SUBSTR:
1PLANNED_START:
1PLANNED_FINISH:
1PLANNED_DURATION:
1W:
1BILLABLE:
1CAPITAL_EXPENSE:
1REGULAR_COST:
1REGULAR_RATE:
1C_EFFORT:
1T_UNIT:
1C_UNIT:
1C_CONTOUR:
1CONTOUR_TYPE:
1TMT_WBS:
1RESOURCE_ASSIGNMENTS:
1PMO_CNST:
1SC_TASK:
1SC_TASK_TYPE1:
1SC_TASK_TYPE2:
1SC_TASK_TYPE3:
1SC_MILESTONE:
1SC_PERSONAL_TASK:
1SC_ACTIVE_ADM_TASK:
1SC_INACTIVE_ADM_TASK:
1SC_WBS_CHANGE_REQUEST_TASK:
1SC_WBS_ACTION_ITEM_TASK:
1SC_WBS_RISK_TASK:
1SC_WBS_ISSUE_TASK:
1SC_WBS_REQUIREMENT_TASK:
1SC_WBS_DEFECT_TASK:
1SC_WBS_SERVICE_REQUEST_TASK:
1RANK:
1SELECT r.RES_ASSIGNMENT_ID, t.TSK_ASSIGNMENT_ID, t.TASK_ID, r.ELEMENT_TYPE, t+
1.PLANNED_WORK, t.UNITS_MSP,:n         TO_DATE(SubStr(t.PLANNED_START, 1, 10))+
1, TO_DATE(SubStr(t.PLANNED_FINISH, 1, 10)),t.PLANNED_START, t.PLANNED_FINISH,+
1 CASE WHEN t.PLANNED_DURATION = 0 THEN w.PLANNED_DURATION ELSE t.PLANNED_DURA+
1TION END,:n         t.PROJECT_ASSIGNMENT_ID, t.BILLABLE, t.CAPITAL_EXPENSE, t+
1.REGULAR_COST, t.REGULAR_RATE,:n         t.C_EFFORT, t.T_UNIT, t.C_UNIT, t.C_+
1CONTOUR, t.CONTOUR_TYPE:n         FROM TMT_WBS w, RESOURCE_ASSIGNMENTS r, TAS+
1K_ASSIGNMENTS t:n         WHERE r.PROJECT_ID = h_project_id AND r.RESOURCE_ID+
1 = h_profile_id AND r.RESOURCE_TYPE = '1' :n         AND t.RES_ASSIGNMENT_ID +
1= r.RES_ASSIGNMENT_ID AND r.ELEMENT_ID = w.ELEMENT_ID :n         AND r.ELEMEN+
1T_TYPE IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO+
1_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.+
1SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUE+
1ST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.S+
1C_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK+
1,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) AND w.RANK LIKE h_rank:n         ORDER+
1 BY t.PLANNED_START:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_A_RESTOPROF(:
1':
1, :
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
1161:
1+:
1FPUT:
1               ENTER:: SP_A_RESTOPROF(162) :: cplan.sql(9207) :: :
1BITAND:
1SC_1FFFFFFF:
1=:
1SC_SRCH_PROFILE:
1SC_SRCH_FROM_PROJECT:
17:
1:
1OPEN:
1FOUND:
1SC_STATE:
1100:
1CLOSE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
1212:
1           EXCEPTION:: SP_A_RESTOPROF(213) :: cplan.sql(9258) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SQLERRM:
1500:
1219:
1        RETURN_ERROR:: SP_A_RESTOPROF(220) :: cplan.sql(9265) :: :
1STATE:: :
1RECORD_NOT_FOUND:
1 ID:: :
1222:
1                EXIT:: SP_A_RESTOPROF(223) :: cplan.sql(9268) :: :
1PMODEFINES:
1PMO_RETURN_ERROR_SP:
1TRUE:
1TYPE_ID:
1TIMECODE_1_ID:
1TIMECODE_2_ID:
1TASK_TYPE:
1UNRESOLVED_DURATION:
1SELECT TYPE_ID, RANK, PROJECT_ID, TIMECODE_1_ID, TIMECODE_2_ID, TASK_TYPE, UN+
1RESOLVED_DURATION:n      INTO h_type_id, h_rank, h_project_id, h_timecode_1_i+
1d, h_timecode_2_id, h_task_type, h_duration:n      FROM TMT_WBS WHERE ELEMENT+
1_ID = h_task_id:
1239:
1           EXCEPTION:: SP_A_RESTOPROF(240) :: cplan.sql(9285) :: :
1SC_SQL_NO_DATA:
1247:
1        RETURN_ERROR:: SP_A_RESTOPROF(248) :: cplan.sql(9293) :: :
1250:
1                EXIT:: SP_A_RESTOPROF(251) :: cplan.sql(9296) :: :
1LEVEL_LOCK:
1PROJECT_DETAILS:
1SELECT LEVEL_LOCK INTO h_temp_var:n      FROM PROJECT_DETAILS:n      WHERE PR+
1OJECT_ID = h_project_id:
1268:
1           EXCEPTION:: SP_A_RESTOPROF(269) :: cplan.sql(9314) :: :
1>:
1275:
1        RETURN_ERROR:: SP_A_RESTOPROF(276) :: cplan.sql(9321) :: :
1LEV_LOCKED_BY_LEVEL:
1278:
1                EXIT:: SP_A_RESTOPROF(279) :: cplan.sql(9324) :: :
1TW:
1FS_CODE:
1MINUTE_PER_DAY:
1MINUTE_PER_WEEK:
1PRIORITY:
1CURRENCY_ID:
1PMO_ORASTD:
1TODECIMAL:
1TD:
1ELEMENT_NAME:
1JOIN:
1TMT_DATAFIELDS:
1SELECT tw.TYPE_ID, CASE WHEN tw.FS_CODE IS NuLL THEN 0 ELSE 1 END, COALESCE(t+
1w.MINUTE_PER_DAY, 480), COALESCE(tw.MINUTE_PER_WEEK, 2400), tw.PRIORITY, tw.C+
1URRENCY_ID, PMO_ORASTD.TODECIMAL(td.ELEMENT_NAME,20,4):n      INTO h_type, h_+
1fs_project, h_per_day, h_per_week, h_priority, h_currency_id, h_rate_echange+
1:n      FROM TMT_WBS tw JOIN TMT_DATAFIELDS td on tw.CURRENCY_ID = td.ELEMENT+
1_ID WHERE tw.ELEMENT_ID = h_project_id:
1295:
1           EXCEPTION:: SP_A_RESTOPROF(296) :: cplan.sql(9341) :: :
1302:
1        RETURN_ERROR:: SP_A_RESTOPROF(303) :: cplan.sql(9348) :: :
1305:
1                EXIT:: SP_A_RESTOPROF(306) :: cplan.sql(9351) :: :
1NICK_NAME:
1FS_LABOR_CODE:
1RESOURCES:
1SELECT NICK_NAME,FS_LABOR_CODE INTO h_full_name,h_labor_code FROM RESOURCES W+
1HERE RESOURCE_ID = h_res_id:
1320:
1           EXCEPTION:: SP_A_RESTOPROF(321) :: cplan.sql(9366) :: :
1327:
1        RETURN_ERROR:: SP_A_RESTOPROF(328) :: cplan.sql(9373) :: :
1330:
1                EXIT:: SP_A_RESTOPROF(331) :: cplan.sql(9376) :: :
1ROLE_ID:
1PROFILES:
1PROFILE_ID:
1SELECT ROLE_ID INTO h_role_id FROM PROFILES WHERE PROFILE_ID = h_profile_id:
1345:
1           EXCEPTION:: SP_A_RESTOPROF(346) :: cplan.sql(9391) :: :
1STRCMP:
1DEFAULTLEVEL:
1356:
1        RETURN_ERROR:: SP_A_RESTOPROF(357) :: cplan.sql(9402) :: :
1359:
1                EXIT:: SP_A_RESTOPROF(360) :: cplan.sql(9405) :: :
14:
12:
1!=:
1COUNT:
1SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_temp_var FROM TASK_ASSIGNMENTS:n      +
1      WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id AND:n       +
1     RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0':
1391:
1           EXCEPTION:: SP_A_RESTOPROF(392) :: cplan.sql(9437) :: :
1ELSIF:
1SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_temp_var FROM TASK_ASSIGNMENTS:n      +
1      WHERE PROJECT_ID = h_project_id AND TASK_ID <> PROJECT_ID AND:n        +
1    RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0':
1408:
1           EXCEPTION:: SP_A_RESTOPROF(409) :: cplan.sql(9454) :: :
1416:
1        RETURN_ERROR:: SP_A_RESTOPROF(417) :: cplan.sql(9462) :: :
1CHILD_TASK_ASSIGNMENT_ALREADY_:
1419:
1                EXIT:: SP_A_RESTOPROF(420) :: cplan.sql(9465) :: :
1SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_ta_count FROM TASK_ASSIGNMENTS:n      +
1   WHERE PROJECT_ID = h_project_id AND PROJECT_ID <> TASK_ID AND RESOURCE_ID +
1= h_profile_id AND RESOURCE_TYPE = '1':
1436:
1           EXCEPTION:: SP_A_RESTOPROF(437) :: cplan.sql(9482) :: :
1SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_pa_count FROM TASK_ASSIGNMENTS:n      +
1   WHERE PROJECT_ID = h_project_id AND PROJECT_ID = TASK_ID AND RESOURCE_ID =+
1 h_profile_id AND RESOURCE_TYPE = '1':
1452:
1           EXCEPTION:: SP_A_RESTOPROF(453) :: cplan.sql(9498) :: :
1ISPROJECTTYPE:
1SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_temp_var FROM TASK_ASSIGNMENTS:n      +
1      WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id AND RESOURCE+
1_TYPE = '0':n            AND PROJECT_ASSIGNMENT_ID IS NuLL:
1471:
1           EXCEPTION:: SP_A_RESTOPROF(472) :: cplan.sql(9517) :: :
1478:
1        RETURN_ERROR:: SP_A_RESTOPROF(479) :: cplan.sql(9524) :: :
1481:
1                EXIT:: SP_A_RESTOPROF(482) :: cplan.sql(9527) :: :
1SC_SQL_SUCCESS:
1540:
1           EXCEPTION:: SP_A_RESTOPROF(541) :: cplan.sql(9586) :: :
1SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_count FROM TASK_ASSIGNMENTS WHERE PROJ+
1ECT_ID = h_project_id AND:n         RESOURCE_ID = h_res_id AND RESOURCE_TYPE +
1= '0' AND TASK_ID = h_project_id:
1558:
1           EXCEPTION:: SP_A_RESTOPROF(559) :: cplan.sql(9604) :: :
1BIT_FLAG:
1SELECT PLANNED_START, PLANNED_FINISH INTO h_sched_start, h_sched_finish:n    +
1        FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND BIT_FLAG = +
1'0' AND:n            RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0' AND TASK_+
1ID = h_project_id:
1577:
1           EXCEPTION:: SP_A_RESTOPROF(578) :: cplan.sql(9623) :: :
1585:
1        RETURN_ERROR:: SP_A_RESTOPROF(586) :: cplan.sql(9631) :: :
1588:
1                EXIT:: SP_A_RESTOPROF(589) :: cplan.sql(9634) :: :
1IS NOT NULL:
1STRNCMP:
110:
1<:
1596:
1        RETURN_ERROR:: SP_A_RESTOPROF(597) :: cplan.sql(9642) :: :
1599:
1                EXIT:: SP_A_RESTOPROF(600) :: cplan.sql(9645) :: :
1PARTICIPANT:
1SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO h_prj_assign_id, h_owner:n        +
1    FROM RESOURCE_ASSIGNMENTS:n            WHERE ELEMENT_ID = h_project_id :n+
1            AND RESOURCE_TYPE = h_res_type :n            AND RESOURCE_ID = h_+
1res_id:
1627:
1           EXCEPTION:: SP_A_RESTOPROF(628) :: cplan.sql(9673) :: :
1SYS_GUID:
1GETSECURITYID:
1SC_TEAMMEMBER_GROUP:
1SC_AFFECTED:
1FC_BEFORE_I_RES_ASSIGN:
1GETTABLE:
1SECURITY_ID:
1TABLE_TYPE:
1REC_USER:
1REC_DATETIME:
1INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, E+
1LEMENT_TYPE,:n               RESOURCE_TYPE, RESOURCE_ID, PROFILE_ID, PARTICIP+
1ANT, SECURITY_ID, TABLE_TYPE,:n               REC_USER, REC_DATETIME):n      +
1         VALUES (h_prj_assign_id, h_project_id, h_project_id, h_type,:n      +
1         h_res_type, h_res_id, h_profile_id, PMO_CNST.SC_AFFECTED, h_temp_sec+
1urity_id, h_table_type,:n               rec_user001, SYSTIMESTAMP):
1666:
1           EXCEPTION:: SP_A_RESTOPROF(667) :: cplan.sql(9712) :: :
1PROFILE_NAME:
1SELECT PROFILE_NAME INTO h_profile_name FROM PROFILES WHERE PROFILE_ID = h_pr+
1ofile_id:
1682:
1           EXCEPTION:: SP_A_RESTOPROF(683) :: cplan.sql(9728) :: :
1SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_total FROM TASK_ASSIGNMENTS:n         +
1         WHERE PROJECT_ID = h_project_id AND TASK_ID <> h_task_id AND RESOURC+
1E_TYPE = '1' AND RESOURCE_ID = h_profile_id AND BIT_FLAG = '0':
1701:
1           EXCEPTION:: SP_A_RESTOPROF(702) :: cplan.sql(9747) :: :
1ASSIGN_NAMES:
1REPLACE:
1UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE (ASSIGN_NAMES, h_profile_name, h_fu+
1ll_name):n                  WHERE ELEMENT_ID = h_project_id AND PROJECT_ID = +
1h_project_id:
1725:
1           EXCEPTION:: SP_A_RESTOPROF(726) :: cplan.sql(9771) :: :
1INSTR:
1LENGTH:
1UPDATE TMT_WBS SET ASSIGN_NAMES =:n                  CASE:n                  +
1WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES:n        +
1          ELSE:n                  CASE:n                  WHEN LENGTH(h_full_+
1name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES:n                  ELSE+
1 ASSIGN_NAMES || h_full_name || ';':n                  END:n                 +
1 END:n                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_pr+
1oject_id:
1754:
1           EXCEPTION:: SP_A_RESTOPROF(755) :: cplan.sql(9800) :: :
1783:
1           EXCEPTION:: SP_A_RESTOPROF(784) :: cplan.sql(9829) :: :
1SELECT TSK_ASSIGNMENT_ID INTO h_prj_assignment_id FROM TASK_ASSIGNMENTS:n    +
1           WHERE RES_ASSIGNMENT_ID = h_prj_assign_id AND BIT_FLAG = '0':
1800:
1           EXCEPTION:: SP_A_RESTOPROF(801) :: cplan.sql(9846) :: :
1SC_PARTICIPANT:
1UPDATE RESOURCE_ASSIGNMENTS SET PROFILE_ID = h_profile_id,:n                 +
1 PARTICIPANT = PMO_CNST.SC_AFFECTED, SECURITY_ID = h_temp_security_id:n      +
1            WHERE RES_ASSIGNMENT_ID = h_prj_assign_id:
1825:
1           EXCEPTION:: SP_A_RESTOPROF(826) :: cplan.sql(9871) :: :
1FC_AFTER_U_RES_ASSIGN:
1PMO_STPU:
1BIT_OR:
1UPDATE RESOURCE_ASSIGNMENTS SET PROFILE_ID = h_profile_id,:n                 +
1 PARTICIPANT=PMO_STPU.BIT_OR(PARTICIPANT, PMO_CNST.SC_AFFECTED):n            +
1      WHERE RES_ASSIGNMENT_ID = h_prj_assign_id:
1848:
1           EXCEPTION:: SP_A_RESTOPROF(849) :: cplan.sql(9894) :: :
1SC_TEAMMEMBER:
1UPDATE RESOURCE_ASSIGNMENTS SET PROFILE_ID = h_profile_id,:n                 +
1 PARTICIPANT=PMO_STPU.BIT_OR(PARTICIPANT, PMO_CNST.SC_TEAMMEMBER):n          +
1        WHERE RES_ASSIGNMENT_ID = h_prj_assign_id:
1871:
1           EXCEPTION:: SP_A_RESTOPROF(872) :: cplan.sql(9917) :: :
1FC_DECLARE_PMO_TMP_SEARCH:
1RS:
1SELECT CASE WHEN h_currency_id = rs.CURRENCY_ID THEN 1:n      ELSE h_rate_ech+
1ange / PMO_ORASTD.TODECIMAL(td.ELEMENT_NAME,20,4) END:n      INTO h_rate_echa+
1nge:n      FROM RESOURCES rs JOIN TMT_DATAFIELDS td ON rs.CURRENCY_ID = td.EL+
1EMENT_ID AND rs.CURRENCY_ID > 0:n      WHERE RESOURCE_ID=h_res_id:
1908:
1           EXCEPTION:: SP_A_RESTOPROF(909) :: cplan.sql(9954) :: :
11.00:
1FC_GET_RES_RATES:
1*:
1ASSIGNMENT_BILLABLE_FLAG:
1TIMEPHASE_PROJECT_ASSIGNMENT:
1SELECT ASSIGNMENT_BILLABLE_FLAG, TIMEPHASE_PROJECT_ASSIGNMENT INTO h_billable+
1, h_pa_timephase:n      FROM PROJECT_DETAILS WHERE PROJECT_ID = h_project_id:
1949:
1           EXCEPTION:: SP_A_RESTOPROF(950) :: cplan.sql(9995) :: :
1SUM:
1SCHED_MIN:
1TASK_DAY_OF_YEAR:
1SELECT SUM(SCHED_MIN) INTO h_total FROM TASK_DAY_OF_YEAR:n         WHERE PROJ+
1ECT_ID = h_project_id AND TASK_ID = h_project_id AND RESOURCE_ID = h_profile_+
1id:
1967:
1           EXCEPTION:: SP_A_RESTOPROF(968) :: cplan.sql(10013) :: :
1SELECT TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,PLANNED_WORK, TO_DATE(SubStr(PLANN+
1ED_START, 1, 10)), TO_DATE(SubStr(PLANNED_FINISH, 1, 10)), BILLABLE, CAPITAL_+
1EXPENSE,:n            REGULAR_COST, REGULAR_RATE, TASK_ID, UNITS_MSP, C_EFFOR+
1T, T_UNIT, C_UNIT, C_CONTOUR:n            INTO h_prj_assignment_id, h_id, h_p+
1lanned_effort, h_start_D, h_finish_D, h_billable, h_capital_expense,:n       +
1     h_old_cost, h_old_sale, h_task_id, h_percent_units, h_work, h_eunit, h_u+
1nit, h_contour:n            FROM TASK_ASSIGNMENTS:n            WHERE TASK_ID +
1= h_project_id AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1':
1998:
1           EXCEPTION:: SP_A_RESTOPROF(999) :: cplan.sql(10044) :: :
1SC_0000FFFF:
11014:
1        RETURN_ERROR:: SP_A_RESTOPROF(1015) :: cplan.sql(10060) :: :
1TASK_ASSIGNMENT_ERROR:
11017:
1                EXIT:: SP_A_RESTOPROF(1018) :: cplan.sql(10063) :: :
1Y:
1-:
1FC_TPHASE_SCHED_ASSIGN:
1DELETE FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id:
11066:
1           EXCEPTION:: SP_A_RESTOPROF(1067) :: cplan.sql(10112) :: :
1DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNME+
1NT_ID = h_prj_assignment_id:
11086:
1           EXCEPTION:: SP_A_RESTOPROF(1087) :: cplan.sql(10132) :: :
1MAX:
1DAY_YEAR:
1SELECT MAX(DAY_YEAR) INTO h_start_D FROM TASK_DAY_OF_YEAR:n                  +
1WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id:
11104:
1           EXCEPTION:: SP_A_RESTOPROF(1105) :: cplan.sql(10150) :: :
1<=:
1DB2TIMESTAMP:
1TIME:
1DUAL:
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_D, 1, 10)) + 1     , PM+
1O_ORASTD.TIME(h_start_date)):n                     INTO h_start_date:n       +
1              FROM DUAL:
11122:
1           EXCEPTION:: SP_A_RESTOPROF(1123) :: cplan.sql(10168) :: :
1SPECIAL_COST:
1SPECIAL_RATE:
1ESTIMATION_LABOR_COST:
1SCHED_WORK:
1ROLLUP_BITS:
1SC_LABOR_BIT:
1UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,:n           +
1             RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',:n                  +
1      REGULAR_COST = h_reg_cost,:n                        SPECIAL_COST = h_sp+
1ec_cost,:n                        REGULAR_RATE = h_reg_sale,:n               +
1         SPECIAL_RATE = h_spec_sale,:n                        UNITS_MSP = CAS+
1E WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,:n      +
1                  ESTIMATION_LABOR_COST = ((PLANNED_WORK+SCHED_WORK)/60.00) *+
1 h_reg_cost,:n                        ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BI+
1TS, PMO_CNST.SC_LABOR_BIT):n                        WHERE TSK_ASSIGNMENT_ID =+
1 h_prj_assignment_id:
11153:
1           EXCEPTION:: SP_A_RESTOPROF(1154) :: cplan.sql(10199) :: :
1ESTIMATION_LABOR_REVENUE:
1UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,:n           +
1             RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',:n                  +
1      REGULAR_COST = h_reg_cost,:n                        SPECIAL_COST = h_sp+
1ec_cost,:n                        REGULAR_RATE = h_reg_sale,:n               +
1         SPECIAL_RATE = h_spec_sale,:n                        UNITS_MSP = CAS+
1E WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,:n      +
1                  ESTIMATION_LABOR_COST = ((PLANNED_WORK+SCHED_WORK)/60.00) *+
1 h_reg_cost,:n                        ESTIMATION_LABOR_REVENUE = ((PLANNED_WO+
1RK+PLANNED_WORK)/60.00) * h_reg_sale,:n                        ROLLUP_BITS = +
1PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT):n                        +
1WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id:
11182:
1           EXCEPTION:: SP_A_RESTOPROF(1183) :: cplan.sql(10228) :: :
1UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,:n           +
1          RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',:n                     +
1REGULAR_COST = h_reg_cost,:n                     SPECIAL_COST = h_spec_cost,+
1:n                     UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS+
1_MSP ELSE h_percent_units END,:n                     ESTIMATION_LABOR_COST = +
1((PLANNED_WORK+SCHED_WORK)/60.00) * h_reg_cost,:n                     ROLLUP_+
1BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT):n                 +
1    WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id:
11209:
1           EXCEPTION:: SP_A_RESTOPROF(1210) :: cplan.sql(10255) :: :
1UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,:n           +
1             RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',:n                  +
1      REGULAR_RATE = h_reg_sale,:n                        SPECIAL_RATE = h_sp+
1ec_sale,:n                        UNITS_MSP = CASE WHEN h_contour_changed = 0+
1 THEN UNITS_MSP ELSE h_percent_units END,:n                        ROLLUP_BIT+
1S = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT):n                    +
1    WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id:
11238:
1           EXCEPTION:: SP_A_RESTOPROF(1239) :: cplan.sql(10284) :: :
1UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,:n           +
1             RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',:n                  +
1      REGULAR_RATE = h_reg_sale,:n                        SPECIAL_RATE = h_sp+
1ec_sale,:n                        UNITS_MSP = CASE WHEN h_contour_changed = 0+
1 THEN UNITS_MSP ELSE h_percent_units END,:n                        ESTIMATION+
1_LABOR_REVENUE = ((PLANNED_WORK+PLANNED_WORK)/60.00) * h_reg_sale,:n         +
1               ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_B+
1IT):n                        WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id:
11264:
1           EXCEPTION:: SP_A_RESTOPROF(1265) :: cplan.sql(10310) :: :
1UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,:n           +
1          UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_p+
1ercent_units END,:n                     RESOURCE_ID = h_res_id, RESOURCE_TYPE+
1 = '0':n                     WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id:
11287:
1           EXCEPTION:: SP_A_RESTOPROF(1288) :: cplan.sql(10333) :: :
11307:
1           EXCEPTION:: SP_A_RESTOPROF(1308) :: cplan.sql(10353) :: :
1RESOURCE_ASSIGN_SKILLS:
1SKILL_ID:
1PROFICIENCY_LEVEL_ID:
1EXPERIENCE_ID:
1PREFERENCE_LEVEL_ID:
1LAST_USED_DATETIME:
1PRIMARY_SKILL:
1INSERT INTO RESOURCE_ASSIGN_SKILLS:n                  SELECT h_prj_assignment+
1_id, SKILL_ID, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID, PREFERENCE_LEVEL_ID, LAST+
1_USED_DATETIME, PRIMARY_SKILL:n                  FROM RESOURCE_ASSIGN_SKILLS +
1WHERE RES_ASSIGNMENT_ID = h_profile_id:
11328:
1           EXCEPTION:: SP_A_RESTOPROF(1329) :: cplan.sql(10374) :: :
1DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_id:
11352:
1           EXCEPTION:: SP_A_RESTOPROF(1353) :: cplan.sql(10398) :: :
1PROJECT_ASSIGNMENTS:
1DELETE FROM PROJECT_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND RESOURCE_+
1ID = h_profile_id:
11372:
1           EXCEPTION:: SP_A_RESTOPROF(1373) :: cplan.sql(10418) :: :
1PMO_FUNC:
1ASSIGNLONG:
1FC_CALC_DURATION:
1INVALID_PARAMS:
18:
11397:
1      RETURN_INVALID:: SP_A_RESTOPROF(1398) :: cplan.sql(10443) :: :
11400:
1                EXIT:: SP_A_RESTOPROF(1401) :: cplan.sql(10446) :: :
1PMO_RETURN_INVALID_SP:
11405:
1        RETURN_ERROR:: SP_A_RESTOPROF(1406) :: cplan.sql(10451) :: :
11408:
1                EXIT:: SP_A_RESTOPROF(1409) :: cplan.sql(10454) :: :
1SC_BIT_27:
1FC_SEARCH_2:
1110:
11436:
1        RETURN_ERROR:: SP_A_RESTOPROF(1437) :: cplan.sql(10482) :: :
1RESOURCE_UNAVAILABLE:
11439:
1                EXIT:: SP_A_RESTOPROF(1440) :: cplan.sql(10485) :: :
11444:
1        RETURN_ERROR:: SP_A_RESTOPROF(1445) :: cplan.sql(10490) :: :
11447:
1                EXIT:: SP_A_RESTOPROF(1448) :: cplan.sql(10493) :: :
1U_ASSIGN_STRING:
1SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(PLANNED_START, 1, 10)), TO_DATE(SubS+
1tr(PLANNED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_+
1RATE:n            INTO h_old_prj_assign_id, h_start_D, h_finish_D, h_billable+
1, h_capital_expense, h_old_cost, h_old_sale:n            FROM TASK_ASSIGNMENT+
1S:n            WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id :n +
1           AND RESOURCE_TYPE = '1' AND RESOURCE_ID = h_profile_id:
11474:
1           EXCEPTION:: SP_A_RESTOPROF(1475) :: cplan.sql(10520) :: :
1SELECT TSK_ASSIGNMENT_ID, BILLABLE INTO h_old_prj_assign_id, h_billable:n    +
1        FROM TASK_ASSIGNMENTS:n            WHERE PROJECT_ID = h_project_id AN+
1D TASK_ID = h_project_id :n            AND RESOURCE_ID = h_profile_id AND RES+
1OURCE_TYPE = '1':
11501:
1           EXCEPTION:: SP_A_RESTOPROF(1502) :: cplan.sql(10547) :: :
1SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(PLANNED_START, 1, 10)), TO_DATE(SubS+
1tr(PLANNED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_+
1RATE:n         INTO h_old_prj_assign_id, h_start_D, h_finish_D, h_billable, h+
1_capital_expense, h_old_cost, h_old_sale:n         FROM TASK_ASSIGNMENTS:n   +
1      WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id :n         A+
1ND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1':
11523:
1           EXCEPTION:: SP_A_RESTOPROF(1524) :: cplan.sql(10569) :: :
1SELECT TSK_ASSIGNMENT_ID, BILLABLE INTO h_old_prj_assign_id, h_billable:n    +
1     FROM TASK_ASSIGNMENTS:n         WHERE PROJECT_ID = h_project_id AND TASK+
1_ID = h_project_id :n         AND RESOURCE_ID = h_profile_id AND RESOURCE_TYP+
1E = '1':
11550:
1           EXCEPTION:: SP_A_RESTOPROF(1551) :: cplan.sql(10596) :: :
1IS NULL:
1SD:
1FD:
1START_DT:
1FINISH_DT:
1BAK_START_DT:
1BAK_FINISH_DT:
1SC_NS:
1SC_NF:
1INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,TASK_ID,PROJ+
1ECT_ID,SD,FD,START_DT,FINISH_DT,:n               PLANNED_START, PLANNED_FINIS+
1H, BIT_FLAG, BAK_START_DT, BAK_FINISH_DT, RESOURCE_ID,RESOURCE_TYPE,PROJECT_A+
1SSIGNMENT_ID,:n               BILLABLE, CAPITAL_EXPENSE, REGULAR_COST,SPECIAL+
1_COST,REGULAR_RATE,SPECIAL_RATE, TIMECODE_1_ID, TIMECODE_2_ID, TABLE_TYPE):n +
1              VALUES(h_prj_assignment_id, h_prj_assign_id, h_project_id, h_pr+
1oject_id, PMO_CNST.SC_NS,PMO_CNST.SC_NF,h_start_DT, h_finish_DT,:n           +
1    h_start_DT, h_finish_DT, '0', h_start_DT, h_finish_DT, h_res_id,h_res_typ+
1e, h_prj_assignment_id,:n               h_billable, h_capital_expense,h_reg_c+
1ost,h_spec_cost,h_reg_sale,h_spec_sale,h_timecode_1_id, h_timecode_2_id, h_ta+
1ble_type):
11593:
1           EXCEPTION:: SP_A_RESTOPROF(1594) :: cplan.sql(10639) :: :
1INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,TASK_ID,PROJ+
1ECT_ID,SD,FD,START_DT,FINISH_DT,:n               PLANNED_START, PLANNED_FINIS+
1H, BIT_FLAG, BAK_START_DT, BAK_FINISH_DT, RESOURCE_ID,RESOURCE_TYPE,PROJECT_A+
1SSIGNMENT_ID,:n               BILLABLE, CAPITAL_EXPENSE, REGULAR_COST,SPECIAL+
1_COST,REGULAR_RATE,SPECIAL_RATE, TIMECODE_1_ID, TIMECODE_2_ID, TABLE_TYPE):n +
1              VALUES(h_prj_assignment_id, h_prj_assign_id, h_project_id, h_pr+
1oject_id, PMO_CNST.SC_NS,PMO_CNST.SC_NF,h_start_DT, h_finish_DT,:n           +
1    h_start_DT, h_finish_DT, '0', h_start_DT, h_finish_DT, h_res_id,h_res_typ+
1e, h_prj_assignment_id,:n               h_billable, h_capital_expense,h_reg_c+
1ost,h_spec_cost,h_reg_sale,h_spec_sale, h_timecode_1_id, h_timecode_2_id, h_t+
1able_type):
11617:
1           EXCEPTION:: SP_A_RESTOPROF(1618) :: cplan.sql(10663) :: :
11641:
1           EXCEPTION:: SP_A_RESTOPROF(1642) :: cplan.sql(10687) :: :
1INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,TASK_ID,PROJ+
1ECT_ID,SD,FD,START_DT,FINISH_DT,:n            PLANNED_START,PLANNED_FINISH,BI+
1T_FLAG,BAK_START_DT,BAK_FINISH_DT,RESOURCE_ID,RESOURCE_TYPE,PROJECT_ASSIGNMEN+
1T_ID,:n            BILLABLE, CAPITAL_EXPENSE, REGULAR_COST,SPECIAL_COST,REGUL+
1AR_RATE,SPECIAL_RATE, TIMECODE_1_ID, TIMECODE_2_ID, TABLE_TYPE):n            +
1VALUES(h_prj_assignment_id, h_prj_assign_id, h_project_id, h_project_id, PMO_+
1CNST.SC_NS,PMO_CNST.SC_NF,h_start_DT, h_finish_DT,:n            h_start_DT, h+
1_finish_DT, '0', h_start_DT, h_finish_DT, h_res_id,h_res_type, h_prj_assignme+
1nt_id,:n            h_billable, h_capital_expense,h_reg_cost,h_spec_cost,h_re+
1g_sale,h_spec_sale, h_timecode_1_id, h_timecode_2_id, h_table_type):
11666:
1           EXCEPTION:: SP_A_RESTOPROF(1667) :: cplan.sql(10712) :: :
1%:
11694:
1           EXCEPTION:: SP_A_RESTOPROF(1695) :: cplan.sql(10740) :: :
1MIN:
1SELECT MIN(t.PLANNED_START) INTO h_start_date:n         FROM TMT_WBS w, TASK_+
1ASSIGNMENTS t:n         WHERE t.PROJECT_ID = h_project_id AND t.RESOURCE_ID =+
1 h_profile_id AND:n         t.TASK_ID = w.ELEMENT_ID AND w.TYPE_ID IN (PMO_CN+
1ST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYP+
1E3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TAS+
1K,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.+
1SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,+
1PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_+
1SERVICE_REQUEST_TASK) AND w.RANK LIKE h_rank:
11718:
1           EXCEPTION:: SP_A_RESTOPROF(1719) :: cplan.sql(10764) :: :
1LOOP:
1EXIT:
1FC_BEFORE_D_RES_ASSIGN:
1DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id:
11770:
1           EXCEPTION:: SP_A_RESTOPROF(1771) :: cplan.sql(10816) :: :
1DELETE FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
11789:
1           EXCEPTION:: SP_A_RESTOPROF(1790) :: cplan.sql(10835) :: :
1EFFORT_WEEK:
1DELETE FROM EFFORT_WEEK WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
11808:
1           EXCEPTION:: SP_A_RESTOPROF(1809) :: cplan.sql(10854) :: :
1DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNME+
1NT_ID = h_tsk_assignment_id:
11827:
1           EXCEPTION:: SP_A_RESTOPROF(1828) :: cplan.sql(10873) :: :
1RESOURCE_ASSIGN_ROLES:
1DELETE FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID = h_tsk_assignment_+
1id:
11846:
1           EXCEPTION:: SP_A_RESTOPROF(1847) :: cplan.sql(10892) :: :
1DELETE FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID = h_tsk_assignment+
1_id:
11865:
1           EXCEPTION:: SP_A_RESTOPROF(1866) :: cplan.sql(10911) :: :
1SUMMARY_TIMESHEETS:
1UPDATE SUMMARY_TIMESHEETS SET RESOURCE_ID = h_res_id :n            WHERE TSK_+
1ASSIGNMENT_ID = h_tsk_assignment_id :n            AND PROJECT_ID = h_project_+
1id :n            AND RESOURCE_ID = h_profile_id:
11887:
1           EXCEPTION:: SP_A_RESTOPROF(1888) :: cplan.sql(10933) :: :
1SELECT TSK_ASSIGNMENT_ID INTO h_id FROM TASK_ASSIGNMENTS:n            WHERE P+
1ROJECT_ID = h_project_id AND TASK_ID = h_task_id AND RESOURCE_ID = h_res_id:n+
1            AND RESOURCE_TYPE ='0':
11904:
1           EXCEPTION:: SP_A_RESTOPROF(1905) :: cplan.sql(10950) :: :
1SELECT RES_ASSIGNMENT_ID, PARTICIPANT, SECURITY_ID INTO h_id, h_participant, +
1h_security_id:n               FROM RESOURCE_ASSIGNMENTS :n               WHER+
1E ELEMENT_ID = h_task_id AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE='0':
11929:
1           EXCEPTION:: SP_A_RESTOPROF(1930) :: cplan.sql(10975) :: :
1UPDATE RESOURCE_ASSIGNMENTS SET :n                     PROFILE_ID		= h_profil+
1e_id,:n                     RESOURCE_ID		= h_res_id, :n                     R+
1ESOURCE_TYPE	= '0':n                     WHERE RES_ASSIGNMENT_ID = h_assign_i+
1d:
11957:
1           EXCEPTION:: SP_A_RESTOPROF(1958) :: cplan.sql(11003) :: :
1UPDATE RESOURCE_ASSIGNMENTS SET :n                     PARTICIPANT		= PMO_CNS+
1T.SC_TEAMMEMBER, :n                     SECURITY_ID		= h_temp_security_id, :n+
1                     PROFILE_ID		= h_profile_id,:n                     RESOUR+
1CE_ID		= h_res_id, :n                     RESOURCE_TYPE	= '0':n              +
1       WHERE RES_ASSIGNMENT_ID = h_assign_id:
11985:
1           EXCEPTION:: SP_A_RESTOPROF(1986) :: cplan.sql(11031) :: :
1UPDATE RESOURCE_ASSIGNMENTS SET :n                     PARTICIPANT		= PMO_STP+
1U.BIT_OR(h_participant,PMO_CNST.SC_TEAMMEMBER), :n                     SECURI+
1TY_ID		= h_security_id, :n                     PROFILE_ID		= h_profile_id,:n +
1                    RESOURCE_ID		= h_res_id, :n                     RESOURCE_+
1TYPE	= '0':n                     WHERE RES_ASSIGNMENT_ID = h_assign_id:
12010:
1           EXCEPTION:: SP_A_RESTOPROF(2011) :: cplan.sql(11056) :: :
12031:
1           EXCEPTION:: SP_A_RESTOPROF(2032) :: cplan.sql(11077) :: :
1RA:
1CREATED_BY:
1TA:
1SELECT ra.RES_ASSIGNMENT_ID, SUBSTR(tw.ASSIGN_NAMES,1,InStr(tw.ASSIGN_NAMES, +
1';') - 1):n                  INTO h_id,h_in FROM TMT_WBS tw, RESOURCE_ASSIGNM+
1ENTS ra:n                  WHERE tw.ELEMENT_ID = h_task_id AND tw.ELEMENT_ID +
1= ra.ELEMENT_ID AND ra.PARTICIPANT = 32 :n                  AND ra.RESOURCE_I+
1D = tw.CREATED_BY AND NOT EXISTS(:n                  SELECT TSK_ASSIGNMENT_ID+
1 FROM TASK_ASSIGNMENTS ta WHERE ta.RES_ASSIGNMENT_ID = ra.RES_ASSIGNMENT_ID):
12051:
1           EXCEPTION:: SP_A_RESTOPROF(2052) :: cplan.sql(11097) :: :
1SC_RESPONSIBLE_GROUP:
12075:
1           EXCEPTION:: SP_A_RESTOPROF(2076) :: cplan.sql(11121) :: :
1UPDATE RESOURCE_ASSIGNMENTS SET RESOURCE_ID = h_res_id, RESOURCE_TYPE='0',:n +
1                    PROFILE_ID = h_profile_id, PARTICIPANT = 48, SECURITY_ID=+
1h_temp_security_id:n                     WHERE RES_ASSIGNMENT_ID = h_assign_i+
1d:
12096:
1           EXCEPTION:: SP_A_RESTOPROF(2097) :: cplan.sql(11142) :: :
1UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE (ASSIGN_NAMES, h_in, h_full_name) +
1:n                     WHERE ELEMENT_ID = h_task_id:
12116:
1           EXCEPTION:: SP_A_RESTOPROF(2117) :: cplan.sql(11162) :: :
1UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE (ASSIGN_NAMES, h_profile_name, h_fu+
1ll_name) WHERE ELEMENT_ID = h_task_id:
12138:
1           EXCEPTION:: SP_A_RESTOPROF(2139) :: cplan.sql(11184) :: :
1UPDATE RESOURCE_ASSIGNMENTS SET RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',+
1:n                     PROFILE_ID = h_profile_id, PARTICIPANT = 16, SECURITY_+
1ID=h_temp_security_id:n                     WHERE RES_ASSIGNMENT_ID = h_assig+
1n_id:
12159:
1           EXCEPTION:: SP_A_RESTOPROF(2160) :: cplan.sql(11205) :: :
1SELECT CASE WHEN FS_LABOR_CODE IS NuLL THEN 0 WHEN FS_LABOR_CODE = h_labor_co+
1de THEN 1 ELSE 2 END:n                     INTO h_same_labor FROM TMT_WBS WHE+
1RE ELEMENT_ID=h_task_id:
12181:
1           EXCEPTION:: SP_A_RESTOPROF(2182) :: cplan.sql(11227) :: :
1UPDATE TMT_WBS SET FS_LABOR_CODE = h_labor_code WHERE ELEMENT_ID=h_task_id:
12202:
1           EXCEPTION:: SP_A_RESTOPROF(2203) :: cplan.sql(11248) :: :
12217:
1           EXCEPTION:: SP_A_RESTOPROF(2218) :: cplan.sql(11263) :: :
12223:
1        RETURN_ERROR:: SP_A_RESTOPROF(2224) :: cplan.sql(11269) :: :
1CANNOT_ASSIGN_LABOR_CODE:
12226:
1                EXIT:: SP_A_RESTOPROF(2227) :: cplan.sql(11272) :: :
12305:
1           EXCEPTION:: SP_A_RESTOPROF(2306) :: cplan.sql(11351) :: :
1ETC:
1UPDATE TASK_ASSIGNMENTS SET:n                           RESOURCE_ID = h_res_i+
1d, RESOURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =:n                           CAS+
1E WHEN h_assigned = 0 THEN NuLL ELSE h_prj_assignment_id END,:n              +
1             REGULAR_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_cos+
1t  ELSE REGULAR_COST END,:n                           SPECIAL_COST = CASE WHE+
1N h_donot_replace_cost = 0 THEN h_spec_cost ELSE SPECIAL_COST END,:n         +
1                  REGULAR_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_re+
1g_sale  ELSE REGULAR_RATE END,:n                           SPECIAL_RATE = CAS+
1E WHEN h_donot_replace_rate = 0 THEN h_spec_sale ELSE SPECIAL_RATE END,:n    +
1                       UNITS_MSP    = work001, ETC = h_effort,:n             +
1              C_CONTOUR    = contour001,:n                           ROLLUP_B+
1ITS  = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT):n                 +
1          WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
12355:
1           EXCEPTION:: SP_A_RESTOPROF(2356) :: cplan.sql(11401) :: :
1UPDATE TASK_ASSIGNMENTS SET:n                           RESOURCE_ID = h_res_i+
1d, RESOURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =:n                           CAS+
1E WHEN h_assigned = 0 THEN NuLL ELSE h_prj_assignment_id END,:n              +
1             REGULAR_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_cos+
1t  ELSE REGULAR_COST END,:n                           SPECIAL_COST = CASE WHE+
1N h_donot_replace_cost = 0 THEN h_spec_cost ELSE SPECIAL_COST END,:n         +
1                  REGULAR_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_re+
1g_sale  ELSE REGULAR_RATE END,:n                           SPECIAL_RATE = CAS+
1E WHEN h_donot_replace_rate = 0 THEN h_spec_sale ELSE SPECIAL_RATE END,:n    +
1                       C_EFFORT     = work001, ETC = h_effort,:n             +
1              C_UNIT       = unit,:n                           C_CONTOUR    =+
1 contour001,:n                           ROLLUP_BITS  = PMO_STPU.BIT_OR(ROLLU+
1P_BITS, PMO_CNST.SC_LABOR_BIT):n                           WHERE TSK_ASSIGNME+
1NT_ID = h_tsk_assignment_id:
12385:
1           EXCEPTION:: SP_A_RESTOPROF(2386) :: cplan.sql(11431) :: :
1UPDATE TASK_ASSIGNMENTS SET:n                        RESOURCE_ID = h_res_id, +
1RESOURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =:n                        CASE WHEN+
1 h_assigned = 0 THEN NuLL ELSE h_prj_assignment_id END,:n                    +
1    REGULAR_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_cost  ELSE R+
1EGULAR_COST END,:n                        SPECIAL_COST = CASE WHEN h_donot_re+
1place_cost = 0 THEN h_spec_cost ELSE SPECIAL_COST END,:n                     +
1   REGULAR_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_reg_sale  ELSE RE+
1GULAR_RATE END,:n                        SPECIAL_RATE = CASE WHEN h_donot_rep+
1lace_rate = 0 THEN h_spec_sale ELSE SPECIAL_RATE END,:n                      +
1  ROLLUP_BITS  = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT), ETC = h+
1_effort:n                        WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_i+
1d:
12415:
1           EXCEPTION:: SP_A_RESTOPROF(2416) :: cplan.sql(11461) :: :
12443:
1           EXCEPTION:: SP_A_RESTOPROF(2444) :: cplan.sql(11489) :: :
12449:
1      RETURN_INVALID:: SP_A_RESTOPROF(2450) :: cplan.sql(11495) :: :
12452:
1                EXIT:: SP_A_RESTOPROF(2453) :: cplan.sql(11498) :: :
12466:
1           EXCEPTION:: SP_A_RESTOPROF(2467) :: cplan.sql(11512) :: :
12472:
1        RETURN_ERROR:: SP_A_RESTOPROF(2473) :: cplan.sql(11518) :: :
12475:
1                EXIT:: SP_A_RESTOPROF(2476) :: cplan.sql(11521) :: :
1TRUNC:
1/:
130:
1TO_NUMBER:
11.5:
1UPDATE TASK_ASSIGNMENTS SET:n                     RESOURCE_ID = h_res_id, RES+
1OURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =:n                     CASE WHEN h_ass+
1igned = 0 THEN NuLL ELSE h_prj_assignment_id END,:n                     REGUL+
1AR_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_cost  ELSE REGULAR_CO+
1ST END,:n                     SPECIAL_COST = CASE WHEN h_donot_replace_cost =+
1 0 THEN h_spec_cost ELSE SPECIAL_COST END,:n                     REGULAR_RATE+
1 = CASE WHEN h_donot_replace_rate = 0 THEN h_reg_sale  ELSE REGULAR_RATE END,+
1:n                     SPECIAL_RATE = CASE WHEN h_donot_replace_rate = 0 THEN+
1 h_spec_sale ELSE SPECIAL_RATE END,:n                     ROLLUP_BITS  = PMO_+
1STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT):n                     WHERE T+
1SK_ASSIGNMENT_ID = h_tsk_assignment_id:
12531:
1           EXCEPTION:: SP_A_RESTOPROF(2532) :: cplan.sql(11577) :: :
12575:
1           EXCEPTION:: SP_A_RESTOPROF(2576) :: cplan.sql(11621) :: :
12581:
1        RETURN_ERROR:: SP_A_RESTOPROF(2582) :: cplan.sql(11627) :: :
12584:
1                EXIT:: SP_A_RESTOPROF(2585) :: cplan.sql(11630) :: :
12598:
1           EXCEPTION:: SP_A_RESTOPROF(2599) :: cplan.sql(11644) :: :
12604:
1        RETURN_ERROR:: SP_A_RESTOPROF(2605) :: cplan.sql(11650) :: :
12607:
1                EXIT:: SP_A_RESTOPROF(2608) :: cplan.sql(11653) :: :
1UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE(ASSIGN_NAMES,h_profile_name || ';',+
1'') WHERE ELEMENT_ID=h_task_id:
12633:
1           EXCEPTION:: SP_A_RESTOPROF(2634) :: cplan.sql(11679) :: :
1C:
1TMT_COMPETENCIES:
1P:
1INSERT INTO RESOURCE_ASSIGN_ROLES:n                  SELECT h_tsk_assignment_+
1id, c.ELEMENT_ID, c.PROFICIENCY_LEVEL_ID, c.EXPERIENCE_ID, c.PREFERENCE_LEVEL+
1_ID, c.LAST_USED_DATETIME, '0':n                  FROM TMT_COMPETENCIES c JOI+
1N PROFILES p ON c.ELEMENT_ID = p.ROLE_ID WHERE p.PROFILE_ID = h_profile_id:
12655:
1           EXCEPTION:: SP_A_RESTOPROF(2656) :: cplan.sql(11701) :: :
1PRIMARY_ROLE:
1INSERT INTO RESOURCE_ASSIGN_ROLES:n                  SELECT h_tsk_assignment_+
1id, ROLE_ID, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID, PREFERENCE_LEVEL_ID, LAST_U+
1SED_DATETIME, PRIMARY_ROLE:n                  FROM RESOURCE_ASSIGN_ROLES WHER+
1E RES_ASSIGNMENT_ID = h_profile_id:
12677:
1           EXCEPTION:: SP_A_RESTOPROF(2678) :: cplan.sql(11723) :: :
1INSERT INTO RESOURCE_ASSIGN_SKILLS:n                  SELECT h_tsk_assignment+
1_id, SKILL_ID, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID, PREFERENCE_LEVEL_ID, LAST+
1_USED_DATETIME, PRIMARY_SKILL:n                  FROM RESOURCE_ASSIGN_SKILLS +
1WHERE RES_ASSIGNMENT_ID = h_profile_id:
12698:
1           EXCEPTION:: SP_A_RESTOPROF(2699) :: cplan.sql(11744) :: :
1FC_AFTER_REPLACE_RES_ASSIGN:
1UPDATE EFFORT_WEEK SET RESOURCE_ID = h_res_id:n                  WHERE PROJEC+
1T_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
12727:
1           EXCEPTION:: SP_A_RESTOPROF(2728) :: cplan.sql(11773) :: :
1ACTUAL_REGULAR_COST:
1ACTUAL_SPECIAL_COST:
1ACTUAL_REGULAR_RATE:
1ACTUAL_SPECIAL_RATE:
1UPDATE SUMMARY_TIMESHEETS SET RESOURCE_ID = h_res_id,:n                     A+
1CTUAL_REGULAR_COST = h_reg_cost  ,:n                     ACTUAL_SPECIAL_COST +
1= h_spec_cost ,:n                     ACTUAL_REGULAR_RATE = h_reg_sale  ,:n  +
1                   ACTUAL_SPECIAL_RATE = h_spec_sale ,:n                     +
1REC_USER = h_rec_user, REC_DATETIME = SYSTIMESTAMP:n                     WHER+
1E TSK_ASSIGNMENT_ID = h_id:
12754:
1           EXCEPTION:: SP_A_RESTOPROF(2755) :: cplan.sql(11800) :: :
1UPDATE SUMMARY_TIMESHEETS SET RESOURCE_ID = h_res_id,:n                     A+
1CTUAL_REGULAR_COST = h_reg_cost  ,:n                     ACTUAL_SPECIAL_COST +
1= h_spec_cost ,:n                     REC_USER = h_rec_user, REC_DATETIME = S+
1YSTIMESTAMP:n                     WHERE TSK_ASSIGNMENT_ID = h_id:
12777:
1           EXCEPTION:: SP_A_RESTOPROF(2778) :: cplan.sql(11823) :: :
1UPDATE SUMMARY_TIMESHEETS SET RESOURCE_ID = h_res_id,:n                     A+
1CTUAL_REGULAR_RATE = h_reg_sale  ,:n                     ACTUAL_SPECIAL_RATE +
1= h_spec_sale ,:n                     REC_USER = h_rec_user, REC_DATETIME = S+
1YSTIMESTAMP:n                     WHERE TSK_ASSIGNMENT_ID = h_id:
12802:
1           EXCEPTION:: SP_A_RESTOPROF(2803) :: cplan.sql(11848) :: :
1UPDATE SUMMARY_TIMESHEETS SET RESOURCE_ID = h_res_id,:n                     R+
1EC_USER = h_rec_user, REC_DATETIME = SYSTIMESTAMP:n                     WHERE+
1 TSK_ASSIGNMENT_ID = h_id:
12825:
1           EXCEPTION:: SP_A_RESTOPROF(2826) :: cplan.sql(11871) :: :
1FC_D_RA_AFTER_REPLACE:
12865:
1           EXCEPTION:: SP_A_RESTOPROF(2866) :: cplan.sql(11911) :: :
1>=:
12911:
1      RETURN_INVALID:: SP_A_RESTOPROF(2912) :: cplan.sql(11957) :: :
12914:
1                EXIT:: SP_A_RESTOPROF(2915) :: cplan.sql(11960) :: :
12919:
1        RETURN_ERROR:: SP_A_RESTOPROF(2920) :: cplan.sql(11965) :: :
12922:
1                EXIT:: SP_A_RESTOPROF(2923) :: cplan.sql(11968) :: :
1EXECUTE:
1IMMEDIATE:
1DELETE FROM PMO_TMP_PA_SUM:
1PMO_TMP_PA_SUM:
1PARENT_ID:
1INSERT INTO PMO_TMP_PA_SUM SELECT DAY_YEAR, SUM(SCHED_MIN):n               FR+
1OM TASK_DAY_OF_YEAR:n               WHERE PROJECT_ID = h_project_id AND PAREN+
1T_ID = h_prj_assignment_id:n               GROUP BY DAY_YEAR:
12948:
1           EXCEPTION:: SP_A_RESTOPROF(2949) :: cplan.sql(11994) :: :
1SC_BIT_30:
12987:
1        RETURN_ERROR:: SP_A_RESTOPROF(2988) :: cplan.sql(12033) :: :
12990:
1                EXIT:: SP_A_RESTOPROF(2991) :: cplan.sql(12036) :: :
12995:
1        RETURN_ERROR:: SP_A_RESTOPROF(2996) :: cplan.sql(12041) :: :
12998:
1                EXIT:: SP_A_RESTOPROF(2999) :: cplan.sql(12044) :: :
1UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,:n           +
1       RESOURCE_ID  = h_res_id, RESOURCE_TYPE = '0',:n                  REGUL+
1AR_COST = CASE WHEN h_donot_replace_rate = 0 THEN h_reg_cost  * h_rate_echang+
1e ELSE REGULAR_COST END,:n                  SPECIAL_COST = CASE WHEN h_donot_+
1replace_rate = 0 THEN h_spec_cost * h_rate_echange ELSE SPECIAL_COST END,:n  +
1                REGULAR_RATE = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_+
1sale  * h_rate_echange ELSE REGULAR_RATE END,:n                  SPECIAL_RATE+
1 = CASE WHEN h_donot_replace_cost = 0 THEN h_spec_sale * h_rate_echange ELSE +
1SPECIAL_RATE END:n                  WHERE TSK_ASSIGNMENT_ID = h_prj_assignmen+
1t_id:
13023:
1           EXCEPTION:: SP_A_RESTOPROF(3024) :: cplan.sql(12069) :: :
1SELECT COUNT(*) INTO h_count FROM RESOURCE_ASSIGNMENTS:n         WHERE ELEMEN+
1T_ID = h_project_id AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1':
13042:
1           EXCEPTION:: SP_A_RESTOPROF(3043) :: cplan.sql(12088) :: :
1UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE(ASSIGN_NAMES,h_profile_name || ';',+
1''):n            WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_i+
1d:
13064:
1           EXCEPTION:: SP_A_RESTOPROF(3065) :: cplan.sql(12110) :: :
1SC_ASSIGNMENT_BIT:
1UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_ASS+
1IGNMENT_BIT) WHERE ELEMENT_ID = h_project_id:
13089:
1           EXCEPTION:: SP_A_RESTOPROF(3090) :: cplan.sql(12135) :: :
1SELECT COUNT(*) INTO h_count FROM TASK_ASSIGNMENTS:n            WHERE PROJECT+
1_ID = h_project_id AND PROJECT_ASSIGNMENT_ID = h_old_prj_assign_id:
13107:
1           EXCEPTION:: SP_A_RESTOPROF(3108) :: cplan.sql(12153) :: :
1DELETE FROM RESOURCE_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND:n       +
1        ELEMENT_ID = h_project_id AND RESOURCE_ID = h_profile_id AND RESOURCE+
1_TYPE = '1':
13129:
1           EXCEPTION:: SP_A_RESTOPROF(3130) :: cplan.sql(12175) :: :
13148:
1           EXCEPTION:: SP_A_RESTOPROF(3149) :: cplan.sql(12194) :: :
13:
1BEST:
1SELECT PLANNED_START, PLANNED_FINISH, BEST, TO_DATE(SubStr(BAK_START_DT, 1, 1+
10)), TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULA+
1R_COST, REGULAR_RATE:n            INTO h_start_DT, h_finish_DT, h_effort, h_s+
1tart_D, h_finish_D, h_billable, h_capital_expense, h_old_cost, h_old_sale:n  +
1          FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_old_prj_assign_id:
13172:
1           EXCEPTION:: SP_A_RESTOPROF(3173) :: cplan.sql(12218) :: :
1ESTIMATED_PLANNED_START:
1ESTIMATED_PLANNED_FINISH:
1ESTIMATED_PLANNED_EFFORT:
1ESTIMATED_PLANNED_COST:
1ESTIMATED_PLANNED_REVENUE:
1UPDATE PROJECT_ASSIGNMENTS SET:n                  ESTIMATED_PLANNED_START = h+
1_start_DT,:n                  ESTIMATED_PLANNED_FINISH = h_finish_DT,:n      +
1            ESTIMATED_PLANNED_EFFORT = h_effort,:n                  ESTIMATED+
1_PLANNED_COST = (h_effort/60.00) * h_old_cost,:n                  ESTIMATED_P+
1LANNED_REVENUE = 0.00:n                  WHERE PROJECT_ID = h_project_id AND +
1RESOURCE_ID = h_profile_id:
13200:
1           EXCEPTION:: SP_A_RESTOPROF(3201) :: cplan.sql(12246) :: :
1UPDATE PROJECT_ASSIGNMENTS SET:n                  ESTIMATED_PLANNED_START = h+
1_start_DT,:n                  ESTIMATED_PLANNED_FINISH = h_finish_DT,:n      +
1            ESTIMATED_PLANNED_EFFORT = h_effort,:n                  ESTIMATED+
1_PLANNED_COST = (h_effort/60.00) * h_old_cost,:n                  ESTIMATED_P+
1LANNED_REVENUE = (h_effort/60.00) * h_old_sale:n                  WHERE PROJE+
1CT_ID = h_project_id AND RESOURCE_ID = h_profile_id:
13225:
1           EXCEPTION:: SP_A_RESTOPROF(3226) :: cplan.sql(12271) :: :
1SELECT COUNT(*) INTO h_count FROM TASK_ASSIGNMENTS:n            WHERE PROJECT+
1_ID = h_project_id AND PROJECT_ASSIGNMENT_ID = h_prj_assignment_id:n         +
1   AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1':
13264:
1           EXCEPTION:: SP_A_RESTOPROF(3265) :: cplan.sql(12310) :: :
1UPDATE TASK_ASSIGNMENTS:n               SET PROJECT_ASSIGNMENT_ID = Null:n   +
1            WHERE PROJECT_ID = h_project_id AND PROJECT_ASSIGNMENT_ID = h_prj+
1_assignment_id:n               AND RESOURCE_ID = h_profile_id AND RESOURCE_TY+
1PE = '1':
13288:
1           EXCEPTION:: SP_A_RESTOPROF(3289) :: cplan.sql(12334) :: :
1UPDATE TASK_DAY_OF_YEAR:n               SET PARENT_ID = TSK_ASSIGNMENT_ID:n  +
1             WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment+
1_id:n               AND RESOURCE_ID = h_profile_id:
13310:
1           EXCEPTION:: SP_A_RESTOPROF(3311) :: cplan.sql(12356) :: :
1SELECT PLANNED_START, PLANNED_FINISH, BEST, TO_DATE(SubStr(BAK_START_DT, 1, 1+
10)), TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULA+
1R_COST, REGULAR_RATE:n         INTO h_start_DT, h_finish_DT, h_effort, h_star+
1t_D, h_finish_D, h_billable, h_capital_expense, h_reg_cost, h_reg_sale:n     +
1    FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id:
13333:
1           EXCEPTION:: SP_A_RESTOPROF(3334) :: cplan.sql(12379) :: :
1UPDATE PROJECT_ASSIGNMENTS SET:n               ESTIMATED_PLANNED_START  = h_s+
1tart_DT,:n               ESTIMATED_PLANNED_FINISH = h_finish_DT,:n           +
1    ESTIMATED_PLANNED_EFFORT = h_effort,:n               ESTIMATED_PLANNED_CO+
1ST   = (h_effort/60.00) * h_reg_cost,:n               ESTIMATED_PLANNED_REVEN+
1UE = 0.00:n               WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h+
1_res_id:
13361:
1           EXCEPTION:: SP_A_RESTOPROF(3362) :: cplan.sql(12407) :: :
1UPDATE PROJECT_ASSIGNMENTS SET:n               ESTIMATED_PLANNED_START  = h_s+
1tart_DT,:n               ESTIMATED_PLANNED_FINISH = h_finish_DT,:n           +
1    ESTIMATED_PLANNED_EFFORT = h_effort,:n               ESTIMATED_PLANNED_CO+
1ST   = (h_effort/60.00) * h_reg_cost,:n               ESTIMATED_PLANNED_REVEN+
1UE = (h_effort/60.00) * h_reg_sale:n               WHERE PROJECT_ID = h_proje+
1ct_id AND RESOURCE_ID = h_res_id:
13386:
1           EXCEPTION:: SP_A_RESTOPROF(3387) :: cplan.sql(12432) :: :
1FC_ROLLUP:
13412:
1       RETURN_STATUS:: SP_A_RESTOPROF(3413) :: cplan.sql(12458) :: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
13419:
1           EXCEPTION:: SP_A_RESTOPROF(3420) :: cplan.sql(12465) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1ERROR:
1ID:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
13431:
1           EXCEPTION:: SP_A_RESTOPROF(3432) :: cplan.sql(12477) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
4f32
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 1c 81 b0 a3 a0 1c
81 b0 a3 a0 1c 81 b0 a3
a0 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 1c 81 b0
a3 a0 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 1c 81 b0
a3 a0 1c 81 b0 a3 a0 1c
81 b0 a3 a0 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 1c 51 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 81 b0
a3 a0 1c 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 1c :3 a0
51 a5 b a5 b 5a 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a0 f4 b4 bf
c8 :5 a0 12a bd b7 11 a4 b1
a0 f4 b4 bf c8 :9 a0 12a bd
b7 11 a4 b1 a0 f4 b4 bf
c8 :6d a0 12a bd b7 11 a4 b1
a3 :2 a0 6b 1c 81 b0 :2 a0 6b
:2 a0 6b 6e a5 b a 10 :2 a0
6b 6e :2 a0 6b 6e a5 b 6e
7e a0 b4 2e 6e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e :3 a0 a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 d :2 a0 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 d :4 a0 51
a5 b :2 a0 6b a5 b d :2 a0
51 a5 b a0 7e a0 6b b4
2e 5a :2 a0 51 a5 b a0 7e
a0 6b b4 2e 5a 52 10 a0
51 d b7 19 3c :2 a0 d a0
6e d a0 6e d a0 51 d
a0 6e d a0 4d d a0 4d
d :2 a0 e9 dd b3 :2 a0 e9 d3
:2 a0 f :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c :3 a0 6b
d :2 a0 e9 c1 :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 51 a5 b
7e 51 b4 2e :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 19 3c :11 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 6b a0 7e a0 6b b4 2e
5a :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 19 3c a0 51 d :5 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 b7
19 3c :25 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c :7 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 19 3c :5 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 6e a5 b 7e 51 b4 2e
a0 6e d b7 19 3c :2 a0 6b
a0 7e a0 6b b4 2e 5a :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 19
3c :2 a0 d :4 a0 51 a5 b 51
a5 b d :4 a0 51 a5 b 51
a5 b d :4 a0 51 a5 b 51
a5 b d a0 7e 51 b4 2e
:2 a0 51 a5 b 7e 51 b4 2e
:b a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f a0 b7 :2 a0 51 a5 b
7e 51 b4 2e :b a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :2 19
3c :2 a0 51 a5 b 7e 51 b4
2e :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 19 3c b7 :b a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :b a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:3 a0 51 a5 b a5 b 5a 7e
51 b4 2e :2 a0 51 a5 b 7e
51 b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a 52 10 5a
a 10 :a a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 51 a5 b
7e 51 b4 2e :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 19 3c b7 19
3c b7 :2 19 3c a0 6e d a0
6e d a0 6e d a0 4d d
a0 4d d :2 a0 e9 dd b3 :2 a0
e9 d3 :2 a0 f :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
:2 a0 6b a0 7e a0 6b b4 2e
5a a0 51 d b7 a0 51 d
b7 :2 19 3c :2 a0 e9 c1 :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 51
a5 b 7e 51 b4 2e :b a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 51 a5 b 7e 51 b4 2e
a0 51 d :d a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b a0
7e a0 6b b4 2e 5a :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b :2 a0 a5 57 a0 b7 a0
7e b4 2e :2 5a a0 7e b4 2e
:2 5a a 10 :2 a0 6b :2 a0 51 a5
b 7e 51 b4 2e 5a :2 a0 6b
:2 a0 51 a5 b 7e 51 b4 2e
5a 52 10 :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b :2 a0
a5 57 b7 19 3c b7 :2 19 3c
b7 19 3c :2 a0 51 a5 b 7e
51 b4 2e a0 51 d :2 a0 d
b7 :b a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a a0 6e d :2 a0
b4 2e d :2 a0 51 :2 a0 51 a5
b :2 a0 6b a0 a5 b d :3 a0
6b d a0 6e d a0 51 d
:9 a0 a5 b d :4 a0 51 a5 b
a5 b d :1a a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :5 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b :2 a0
a5 b 7e 51 b4 2e a0 51
d b7 :c a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c :2 a0
51 a5 b 7e 51 b4 2e :a a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :11 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:11 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c a0 b7 :2 a0 6b a0
7e a0 6b b4 2e 5a :6 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 51 a5 b a0 7e a0 6b
b4 2e :2 a0 51 :2 a0 51 a5 b
:2 a0 6b a0 a5 b d :a a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :3 a0 a5
b d a0 b7 :3 a0 51 a5 b
:2 a0 6b a5 b 5a a0 7e a0
6b b4 2e :b a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :3 a0 a5 b d b7
19 :b a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :3 a0 a5 b d b7 :2 19 3c
b7 :2 19 3c b7 :2 19 3c b7 :2 a0
d b7 :2 19 3c a0 6e d a0
6e d a0 6e d :2 a0 b4 2e
d :16 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a a0 51 d b7
19 3c :2 a0 51 a5 b 7e 51
b4 2e 5a :2 a0 51 a5 b 7e
51 b4 2e 5a 52 10 :c a0 a5
b d :2 a0 51 a5 b 7e 51
b4 2e :3 a0 51 a5 b 7e :2 a0
51 a5 b b4 2e d :3 a0 51
a5 b 7e :2 a0 51 a5 b b4
2e d b7 19 3c :2 a0 51 a5
b 7e 51 b4 2e :3 a0 51 a5
b 7e :2 a0 51 a5 b b4 2e
d :3 a0 51 a5 b 7e :2 a0 51
a5 b b4 2e d b7 19 3c
b7 19 3c a0 51 d a0 51
d a0 4d d :7 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
:2 a0 a5 b 7e 51 b4 2e :a a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 51 a5 b 7e 51 b4
2e :2 a0 d b7 a0 51 d b7
:2 19 3c b7 19 3c :2 a0 51 a5
b 7e 51 b4 2e :2 a0 6b :2 a0
a5 b 7e 51 b4 2e :28 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 d a0 51 d :2 a0 51 a5
b 7e 51 b4 2e :2 a0 51 a5
b a0 7e a0 51 a5 b b4
2e 5a :2 a0 51 a5 b a0 7e
a0 51 a5 b b4 2e 5a 52
10 :2 a0 51 a5 b 7e 51 b4
2e 5a :3 a0 51 a5 b :2 a0 6b
a5 b 5a 7e 51 b4 2e 5a
a 10 :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 a0 51 d :2 a0 51 a5
b 7e 51 b4 2e a0 51 d
b7 19 3c b7 :2 19 3c b7 19
3c :2 a0 d :2 a0 d :2 a0 d :2 a0
d :2 a0 d :2 a0 d :2 a0 d :2 a0
d :2 a0 d b7 19 3c :2 a0 6b
a0 7e a0 6b b4 2e 5a a0
7e b4 2e :2 5a a 10 :2 a0 :2 51
a5 b 7e 6e b4 2e a0 7e
51 b4 2e d :2 a0 6b :b a0 a5
b d b7 19 3c :2 a0 51 a5
b 7e 51 b4 2e :3 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :5 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 d :8 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b :2 a0 51 a5 b 7e
51 b4 2e :a a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 19 3c
b7 :2 a0 51 a5 b 7e 51 b4
2e 5a :2 a0 51 a5 b 7e 51
b4 2e 5a a 10 :2 a0 51 a5
b 7e 51 b4 2e :1e a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :22 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :2 19
3c a0 b7 :2 a0 51 a5 b 7e
51 b4 2e :1a a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f a0 b7 19 :2 a0 51
a5 b 7e 51 b4 2e :3 a0 51
a5 b 7e :2 a0 51 a5 b b4
2e d :3 a0 51 a5 b 7e :2 a0
51 a5 b b4 2e d :2 a0 51
a5 b 7e 51 b4 2e :16 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :1a a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:2 19 3c b7 19 :c a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c :5 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :b a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:2 19 3c a0 51 d :3 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :5 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 51 a5
b 7e 51 b4 2e 5a :2 a0 51
a5 b 7e 51 b4 2e 5a a
10 a0 51 d a0 51 d :2 a0
6b a0 51 a5 b 7e 51 b4
2e a0 51 d b7 a0 51 d
b7 :2 19 3c :2 a0 51 a5 b 7e
51 b4 2e a0 51 d b7 19
3c :6 a0 51 a5 b :2 a0 51 a5
b :2 a0 51 a5 b :2 a0 51 a5
b :2 a0 51 a5 b :2 a0 51 a5
b :2 a0 51 a5 b :2 a0 51 a5
b :3 a0 a5 b d :2 a0 51 a5
b a0 7e a0 6b b4 2e :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b a0 a5 57 a0 b7
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 51 a5 b :2 a0 a5 57
b7 :2 19 3c :2 a0 51 a5 b 7e
51 b4 2e :3 a0 6b d b7 :3 a0
6b :2 a0 51 a5 b :2 a0 6b a5
b d b7 :2 19 3c a0 51 d
:1e a0 :4 51 a5 b d :2 a0 51 a5
b 7e 51 b4 2e :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 a0 b7 :2 a0 51
a5 b 7e 51 b4 2e :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b :2 a0 a5 57 b7 19 a0
51 d :7 a0 51 a5 b :2 a0 51
a5 b a5 b d b7 :2 19 3c
b7 19 3c b7 19 3c a0 b7
:2 a0 :2 51 a5 b 7e 6e b4 2e
:1a a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b a0 7e a0 6b
b4 2e 5a a0 7e b4 2e :2 5a
a 10 a0 7e 51 b4 2e d
:2 a0 6b :b a0 a5 b d b7 19
3c b7 19 :c a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :2 19 3c
a0 b7 :2 a0 :2 51 a5 b 7e 6e
b4 2e :1a a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a a0 7e b4
2e :2 5a a 10 a0 7e 51 b4
2e d :2 a0 6b :b a0 a5 b d
b7 19 3c b7 19 :c a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:2 19 3c a0 7e b4 2e :2 5a :2 a0
51 a5 b 7e 51 b4 2e 5a
a 10 :2 a0 51 a5 b 7e 51
b4 2e 5a a 10 a0 6e d
:2 a0 b4 2e d :2 a0 51 a5 b
7e 51 b4 2e :3 a0 51 a5 b
d b7 19 3c :2 a0 51 a5 b
7e 51 b4 2e 5a :2 a0 51 a5
b 7e 51 b4 2e 5a 52 10
:2 a0 51 a5 b 7e 51 b4 2e
5a :2 a0 51 a5 b 7e 51 b4
2e 5a a 10 :34 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f a0 b7 :2 a0 51
a5 b 7e 51 b4 2e :34 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 19
:34 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c b7 :34 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c a0
51 d b7 19 3c :3 a0 7e 6e
b4 2e :2 51 a5 b d a0 6e
d :2 a0 d :5 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f a0 51 d
a0 51 d a0 51 d a0 7e
51 b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a a 10 :35 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 19 3c :2 a0 d :2 a0 e9
dd b3 :17 a0 e9 d3 5 :2 a0 f
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c :2 a0 6b a0 7e
a0 6b b4 2e 5a a0 2b b7
19 3c :2 a0 51 a5 b 7e 51
b4 2e :3 a0 51 a5 b d :3 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :3 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :3 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :5 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :3 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :3 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:9 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:a a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b a0 7e a0 6b
b4 2e 5a :2 a0 51 a5 b 7e
51 b4 2e :2 a0 d b7 19 3c
:c a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b a0 7e a0 6b
b4 2e 5a :3 a0 51 a5 b :2 a0
6b a5 b 5a a0 7e a0 6b
b4 2e a0 51 d :8 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f a0 b7 :2 a0
51 a5 b a0 7e a0 6b b4
2e :2 a0 51 :2 a0 51 a5 b :2 a0
6b a0 a5 b d :d a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 19 :10 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:2 19 3c :2 a0 6b a0 7e a0 6b
b4 2e 5a :2 a0 51 a5 b 7e
51 b4 2e 5a a 10 :3 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 19
3c b7 :22 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a a0 7e b4
2e :2 5a a 10 :2 a0 51 :2 a0 51
a5 b :2 a0 6b a0 a5 b d
:3 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:b a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:8 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 a0 51 :2 a0 51 a5 b :2 a0
6b a0 a5 b d :8 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :b a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :2 19 3c
b7 :2 19 3c :2 a0 51 a5 b 7e
51 b4 2e :2 a0 51 a5 b 7e
51 b4 2e 5a a0 7e b4 2e
:2 5a a 10 :7 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b a0
7e a0 6b b4 2e 5a :2 a0 51
a5 b 7e 51 b4 2e 5a a
10 :5 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f a0 b7 :2 a0 6b a0 7e a0
6b b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a a 10 :2 a0
e9 c1 :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 :2 19 3c b7 19 3c a0
7e 51 b4 2e d :2 a0 6b :b a0
a5 b d :5 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f a0 6e d a0 7e
51 b4 2e :2 a0 d :2 a0 d :2 a0
d :2 a0 d :2 a0 d :2 a0 d :2 a0
d :2 a0 d :2 a0 d :3 a0 51 a5
b :2 a0 6b a5 b 5a 7e 51
b4 2e 5a :2 a0 51 a5 b a0
7e a0 51 a5 b b4 2e 5a
:2 a0 51 a5 b a0 7e a0 51
a5 b b4 2e 5a 52 10 5a
a 10 5a :2 a0 51 a5 b 7e
51 b4 2e :25 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 :27 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c b7
:21 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c :2 a0 51 a5 b 7e
51 b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a a 10 a0
51 d a0 b7 :2 a0 51 a5 b
7e 51 b4 2e :3 a0 51 a5 b
d b7 :2 19 3c :6 a0 51 a5 b
:2 a0 51 a5 b :2 a0 51 a5 b
:2 a0 51 a5 b :2 a0 51 a5 b
:2 a0 51 a5 b :2 a0 51 a5 b
:2 a0 51 a5 b :3 a0 a5 b d
:2 a0 51 a5 b a0 7e a0 6b
b4 2e :2 a0 e9 c1 :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b a0 a5 57 a0 b7 :2 a0 51
a5 b 7e 51 b4 2e :2 a0 e9
c1 :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 51 a5 b :2 a0 a5
57 b7 :2 19 3c b7 :2 a0 51 a5
b 7e 51 b4 2e :2 a0 51 a5
b 7e 51 b4 2e a 10 :2 a0
51 a5 b 7e 51 b4 2e a
10 a0 51 d :3 a0 51 a5 b
d b7 19 3c :2 a0 51 a5 b
7e 51 b4 2e 5a :2 a0 51 a5
b a0 7e a0 51 a5 b b4
2e 5a 52 10 :3 a0 51 a5 b
d b7 :4 a0 51 a5 b 7e :2 a0
51 a5 b b4 2e a5 b d
b7 :2 19 3c :2 a0 51 a5 b 7e
51 b4 2e :3 a0 51 a5 b 7e
51 b4 2e d b7 :5 a0 51 a5
b a5 b 7e 51 b4 2e a5
b d b7 :2 19 3c :1f a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :2 19 3c
a0 51 d a0 51 d a0 51
d :2 a0 51 a5 b 7e 51 b4
2e :2 a0 51 a5 b 7e 51 b4
2e :2 a0 51 a5 b 7e 51 b4
2e 52 10 5a a 10 :3 a0 6b
d b7 :3 a0 6b :2 a0 51 a5 b
:2 a0 6b a5 b d b7 :2 19 3c
a0 51 d a0 51 d :1e a0 :4 51
a5 b d :2 a0 51 a5 b 7e
51 b4 2e :2 a0 e9 c1 :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b :2 a0 a5 57 a0 b7 :2 a0
51 a5 b 7e 51 b4 2e :2 a0
e9 c1 :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 :2 19 3c :7 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :18 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :b a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :b a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :3 a0 51 a5 b
7e 51 b4 2e d :3 a0 51 a5
b 7e :2 a0 51 a5 b b4 2e
d a0 6e d :5 a0 a5 b d
a0 6e d :5 a0 a5 b d :7 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
51 a5 b 7e 51 b4 2e 5a
:2 a0 51 a5 b 7e 51 b4 2e
5a a 10 :11 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f a0 b7 :2 a0 51 a5
b 7e 51 b4 2e :d a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :3 a0 51 a5
b d a0 b7 19 :2 a0 51 a5
b 7e 51 b4 2e :d a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :3 a0 51 a5
b d b7 19 :9 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :3 a0 51 a5 b
d :3 a0 51 a5 b d b7 :2 19
3c b7 19 3c b7 19 3c b7
:2 19 3c :2 a0 6b :4 a0 a5 b d
b7 a0 47 :2 a0 e9 c1 :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f a0 51
d :2 a0 51 a5 b 7e 51 b4
2e :2 a0 d :2 a0 51 a5 b a0
7e a0 51 a5 b b4 2e :3 a0
51 a5 b 7e :2 a0 51 a5 b
b4 2e d :2 a0 51 a5 b 7e
51 b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a a 10 a0
7e b4 2e :2 5a a 10 :2 a0 d
:2 a0 d :2 a0 d :2 a0 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 d :2 a0 51
a5 b 7e 51 b4 2e :2 a0 51
a5 b 7e 51 b4 2e a 10
a0 51 d a0 b7 :2 a0 51 a5
b 7e 51 b4 2e :3 a0 51 a5
b d b7 :2 19 3c :6 a0 51 a5
b :2 a0 51 a5 b :2 a0 51 a5
b :2 a0 51 a5 b :2 a0 51 a5
b :2 a0 51 a5 b :2 a0 51 a5
b :2 a0 51 a5 b :3 a0 a5 b
d :2 a0 51 a5 b a0 7e a0
6b b4 2e :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b a0
a5 57 a0 b7 :2 a0 51 a5 b
7e 51 b4 2e :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 51 a5
b :2 a0 a5 57 b7 :2 19 3c :2 a0
6e 11e 11d :2 a0 6b 51 d :a a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f a0
6e d a0 51 d a0 51 d
a0 51 d a0 51 d :2 a0 51
a5 b 7e 51 b4 2e :2 a0 51
a5 b 7e 51 b4 2e :2 a0 51
a5 b 7e 51 b4 2e a 10
5a 52 10 :2 a0 51 a5 b 7e
51 b4 2e :2 a0 51 a5 b 7e
51 b4 2e a 10 :2 a0 51 a5
b 7e 51 b4 2e a 10 a0
51 d :3 a0 51 a5 b d b7
19 3c :3 a0 6b :2 a0 51 a5 b
:2 a0 6b a5 b d a0 51 d
b7 a0 51 d b7 :2 19 3c a0
51 d :1e a0 :4 51 a5 b d :2 a0
51 a5 b 7e 51 b4 2e :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 a0 b7
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 b7
19 :1c a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 :2 19 3c b7 19 3c b7
19 3c :8 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 51 a5 b
7e 51 b4 2e :9 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 19 3c b7
19 3c a0 51 d a0 7e b4
2e 5a :9 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 51 a5 b 7e 51
b4 2e :7 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 51 a5 b
7e 51 b4 2e :8 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :5 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 19 3c b7
19 3c :2 a0 51 a5 b 7e 51
b4 2e a0 51 d :5 a0 :2 51 a5
b d :19 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a :2 a0 51 a5
b 7e 51 b4 2e :f a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :11 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :2 19
3c :2 a0 :2 51 a5 b 7e 6e b4
2e 5a a0 51 d :2 a0 6b :b a0
a5 b d b7 19 3c b7 19
3c a0 7e b4 2e :2 5a :2 a0 6b
:2 a0 a5 b 7e 51 b4 2e 5a
a 10 a0 51 d b7 19 3c
b7 19 3c b7 19 3c :2 a0 51
a5 b 7e 51 b4 2e 5a a0
7e b4 2e :2 5a a 10 :2 a0 51
a5 b 7e 51 b4 2e :a a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 51 a5 b 7e 51 b4 2e
:9 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:9 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 19 3c b7 19 3c a0 51
d :5 a0 :2 51 a5 b d :19 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 6b a0 7e a0 6b b4 2e
5a :2 a0 51 a5 b 7e 51 b4
2e :f a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 :11 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 :2 19 3c :2 a0 :2 51 a5
b 7e 6e b4 2e 5a :2 a0 51
a5 b 7e 51 b4 2e 5a a
10 a0 51 d :2 a0 6b :b a0 a5
b d b7 19 3c b7 19 3c
b7 19 3c :3 a0 a5 b d :2 a0
6b 7e 51 b4 2e :2 a0 6b 51
d b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 :2 a0 6b :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 :2 a0 6b :2 6e 7e :2 a0
6b b4 2e 7e 6e b4 2e 7e
:2 a0 6b b4 2e a5 57 b7 19
3c :9 a0 12a e9 dd :2 a0 65 b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c :7 a0
12a e9 dd :2 a0 65 b7 a4 b1
11 4f b7 a6 9 a4 b1 11
68 4f 1d 17 b5 
4f32
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 6f 6b 59 77 84
80 56 8c 95 91 7f 9d aa
a6 7c b2 bb b7 a5 c3 d0
cc a2 d8 e1 dd cb e9 f6
f2 c8 fe 107 103 f1 10f 11c
118 ee 124 12d 129 117 135 114
13a 13e 142 146 149 14d 16a 155
159 15c 15d 165 154 187 175 151
179 17a 182 174 1a4 192 171 196
197 19f 191 1c1 1af 18e 1b3 1b4
1bc 1ae 1de 1cc 1ab 1d0 1d1 1d9
1cb 1fb 1e9 1c8 1ed 1ee 1f6 1e8
218 206 1e5 20a 20b 213 205 235
223 202 227 228 230 222 252 240
21f 244 245 24d 23f 26f 25d 23c
261 262 26a 25c 28c 27a 259 27e
27f 287 279 2a9 297 276 29b 29c
2a4 296 2c5 2b4 2b8 2c0 293 2dd
2cc 2d0 2d8 2b3 2f9 2e8 2ec 2f4
2b0 311 300 304 30c 2e7 32e 31c
2e4 320 321 329 31b 34a 339 33d
345 318 362 351 355 35d 338 37f
36d 335 371 372 37a 36c 39c 38a
369 38e 38f 397 389 3b8 3a7 3ab
3b3 386 3d0 3bf 3c3 3cb 3a6 3ec
3db 3df 3e7 3a3 404 3f3 3f7 3ff
3da 421 40f 3d7 413 414 41c 40e
43e 42c 40b 430 431 439 42b 45b
449 428 44d 44e 456 448 478 466
445 46a 46b 473 465 495 483 462
487 488 490 482 4b1 4a0 4a4 47f
4ac 49f 4cd 4bc 4c0 49c 4c8 4bb
4e9 4d8 4dc 4b8 4e4 4d7 505 4f4
4f8 4d4 500 4f3 521 510 514 4f0
51c 50f 53d 52c 530 50c 538 52b
559 548 54c 528 554 547 575 564
568 544 570 563 591 580 584 560
58c 57f 5ad 59c 5a0 57c 5a8 59b
5c9 5b8 5bc 598 5c4 5b7 5e5 5d4
5d8 5b4 5e0 5d3 601 5f0 5f4 5d0
5fc 5ef 61d 60c 610 5ec 618 60b
639 628 62c 608 634 627 655 644
648 624 650 643 671 660 664 640
66c 65f 68d 67c 680 65c 688 67b
6a9 698 69c 678 6a4 697 6c5 6b4
6b8 694 6c0 6b3 6e1 6d0 6d4 6b0
6dc 6cf 6fd 6ec 6f0 6cc 6f8 6eb
719 708 70c 6e8 714 707 735 724
728 704 730 723 751 740 744 720
74c 73f 76d 75c 760 73c 768 75b
789 778 77c 758 784 777 7a5 794
798 774 7a0 793 7c1 7b0 7b4 790
7bc 7af 7dd 7cc 7d0 7ac 7d8 7cb
7f9 7e8 7ec 7c8 7f4 7e7 815 804
808 7e4 810 803 831 820 824 800
82c 81f 84d 83c 840 81c 848 83b
869 858 85c 838 864 857 885 874
878 854 880 873 8a1 890 894 870
89c 88f 8bd 8ac 8b0 88c 8b8 8ab
8d9 8c8 8cc 8a8 8d4 8c7 8f5 8e4
8e8 8c4 8f0 8e3 912 900 8e0 904
905 90d 8ff 92f 91d 8fc 921 922
92a 91c 94b 93a 93e 919 946 939
967 956 95a 936 962 955 984 972
952 976 977 97f 971 9a0 98f 993
96e 99b 98e 9bd 9ab 98b 9af 9b0
9b8 9aa 9da 9c8 9a7 9cc 9cd 9d5
9c7 9f6 9e5 9e9 9c4 9f1 9e4 a12
a01 a05 9e1 a0d a00 a2e a1d a21
a29 9fd a46 a35 a39 a41 a1c a62
a51 a55 a19 a5d a50 a7e a6d a71
a4d a79 a6c a9a a89 a8d a69 a95
a88 ab6 aa5 aa9 a85 ab1 aa4 ad2
ac1 ac5 aa1 acd ac0 aee add ae1
abd ae9 adc b0a af9 afd ad9 b05
af8 b26 b15 b19 af5 b21 b14 b42
b31 b35 b11 b3d b30 b5e b4d b51
b2d b59 b4c b7a b69 b6d b49 b75
b68 b97 b85 b65 b89 b8a b92 b84
bb4 ba2 b81 ba6 ba7 baf ba1 bd1
bbf b9e bc3 bc4 bcc bbe c02 bdc
be0 be8 bec bf0 bbb bf4 bf5 bf7
bf8 bfa bfd bdb c1e c0d c11 bd8
c19 c0c c3a c29 c2d c09 c35 c28
c41 c45 c25 c56 c59 c5d c61 c65
c69 c6d c71 c7d c82 c84 c90 c94
c96 c9a cab cac caf cb3 cb7 cbb
cbf cc3 cc7 ccb ccf cd3 cd7 ce3
ce8 cea cf6 cfa cfc d00 d11 d12
d15 d19 d1d d21 d25 d29 d2d d31
d35 d39 d3d d41 d45 d49 d4d d51
d55 d59 d5d d61 d65 d69 d6d d71
d75 d79 d7d d81 d85 d89 d8d d91
d95 d99 d9d da1 da5 da9 dad db1
db5 db9 dbd dc1 dc5 dc9 dcd dd1
dd5 dd9 ddd de1 de5 de9 ded df1
df5 df9 dfd e01 e05 e09 e0d e11
e15 e19 e1d e21 e25 e29 e2d e31
e35 e39 e3d e41 e45 e49 e4d e51
e55 e59 e5d e61 e65 e69 e6d e71
e75 e79 e7d e81 e85 e89 e8d e91
e95 e99 e9d ea1 ea5 ea9 ead eb1
eb5 eb9 ebd ec1 ec5 ec9 ecd ed9
ede ee0 eec ef0 f0e ef6 efa efe
f01 f09 ef5 f15 f19 ef2 f1d f21
f25 f28 f2d f2e 1 f30 f35 f39
f3d f40 f45 f49 f4d f50 f55 f56
f58 f5d f60 f64 f65 f6a f6f f72
f77 f78 f7d f80 f84 f85 f8a f8d
f92 f93 f98 f9b fa0 fa1 fa6 fa9
fae faf fb4 fb7 fbb fbc fc1 fc4
fc9 fca fcf fd2 fd7 fd8 fdd fe0
fe5 fe6 feb fee ff2 ff3 ff8 ffb
1000 1001 1006 1009 100e 100f 1014 1017
101c 101d 1022 1025 1029 102a 102f 1032
1037 1038 103d 1040 1045 1046 104b 104e
1053 1054 1059 105c 1060 1061 1066 1069
106e 106f 1074 1077 107c 107d 1082 1085
1089 108d 1091 1092 1094 1099 109a 109c
109d 10a2 10a5 10aa 10ab 10b0 10b3 10b7
10bb 10bf 10c0 10c2 10c7 10c8 10ca 10cb
10d0 10d3 10d8 10d9 10de 10e1 10e5 10e9
10ed 10ee 10f0 10f5 10f6 10f8 10f9 10fe
1101 1106 1107 110c 110f 1113 1117 111b
111c 111e 1123 1124 1126 1127 112c 112f
1134 1135 113a 113d 1141 1145 1149 114a
114c 1151 1152 1154 1155 115a 115d 1162
1163 1168 116b 116f 1173 1177 1178 117a
117f 1180 1182 1183 1188 118b 1190 1191
1196 1199 119d 11a1 11a5 11a6 11a8 11ad
11ae 11b0 11b1 11b6 11b9 11be 11bf 11c4
11c7 11cb 11cf 11d3 11d4 11d6 11db 11dc
11de 11df 11e4 11e7 11ec 11ed 11f2 11f5
11f9 11fd 1201 1202 1204 1209 120a 120c
120d 1212 1215 121a 121b 1220 1223 1228
1229 122e 1231 1235 1236 123b 123e 1243
1244 1249 124c 1251 1252 1257 1258 125d
125f 1263 1266 126a 126e 1271 1275 1279
127c 127f 1284 1287 128a 128d 128e 1293
1294 1 1296 129b 129f 12a3 12a6 12ab
12b0 12b3 12b7 12b8 12bd 12be 12c3 12c5
12c9 12cc 12d0 12d4 12d8 12dc 12e0 12e4
12e8 12ec 12f0 12f4 12f8 12fc 1300 1304
1308 130c 1310 1314 1318 131c 1320 1324
1327 1328 132a 132e 1332 1335 1336 1338
133c 1340 1344 1347 1348 134a 134e 1351
1355 1358 1359 135e 1361 1365 1369 136c
136d 136f 1373 1376 137a 137d 137e 1383
1 1386 138b 138f 1392 1396 1398 139c
139f 13a3 13a7 13ab 13af 13b4 13b8 13bc
13c1 13c5 13c9 13cc 13d0 13d4 13d9 13dd
13e1 13e2 13e6 13ea 13eb 13ef 13f3 13f7
13fc 1400 1401 1405 1409 140e 1413 1417
141b 1420 1424 1428 142b 142e 1432 1434
1438 143c 143f 1442 1446 1448 144c 1450
1453 1457 145b 145f 1462 1466 146a 146e
1473 1475 1479 147d 1480 1484 1488 148a
148e 1492 1496 1499 149d 14a1 14a3 14a4
14a9 1 14ad 14b1 14b5 14b8 14bc 14c0
14c4 14c8 14cb 14cf 14d3 14d6 14d9 14de
14e1 14e4 14e7 14e8 14ed 14ee 1 14f0
14f5 14f9 14fd 1500 1505 150a 150d 1511
1512 1517 151a 151f 1520 1525 1528 152d
152e 1533 1536 153a 153b 1540 1543 1548
1549 154e 1551 1555 1559 155c 155f 1560
1562 1563 1568 1569 156e 1570 1574 1577
1579 157d 157f 158b 158d 158f 1590 1595
1599 159b 15a7 15a9 15ad 15b1 15b4 15b5
15b7 15ba 15bd 15be 15c3 15c7 15cb 15ce
15d2 15d6 15d9 15dc 15e1 15e4 15e7 15ea
15eb 15f0 15f1 1 15f3 15f8 15fc 1600
1603 1608 160d 1610 1614 1615 161a 161d
1622 1623 1628 162b 1630 1631 1636 1639
163d 1641 1645 1649 164c 164d 164f 1654
1655 1657 1658 165d 1660 1665 1666 166b
166e 1673 1674 1679 167c 1680 1681 1686
1689 168e 168f 1694 1695 169a 169c 16a0
16a3 16a7 16ab 16ae 16b2 16b6 16b9 16bc
16c1 16c4 16c7 16ca 16cb 16d0 16d1 1
16d3 16d8 16dc 16e0 16e3 16e8 16ed 16f0
16f4 16f5 16fa 16fb 1700 1702 1706 1709
170d 1711 1714 1718 171c 171f 1723 1727
1728 172d 172f 1733 1736 173a 173e 1742
1746 174a 174e 1752 1756 175a 175e 1762
1766 176a 176e 1772 1776 177a 1786 178a
178e 1791 1795 1799 179b 179f 17a3 17a7
17aa 17ae 17b2 17b4 17b5 17ba 1 17be
17c2 17c6 17c9 17cd 17d1 17d5 17d9 17dc
17e0 17e4 17e7 17ea 17ef 17f2 17f5 17f8
17f9 17fe 17ff 1 1801 1806 180a 180e
1811 1816 181b 181e 1822 1823 1828 182b
1830 1831 1836 1839 183e 183f 1844 1847
184b 184c 1851 1854 1859 185a 185f 1862
1866 186a 186d 1870 1871 1873 1874 1879
187a 187f 1881 1885 1888 188a 188e 1890
189c 189e 18a0 18a1 18a6 18aa 18ac 18b8
18ba 18be 18c2 18c5 18c9 18cc 18d0 18d3
18d4 18d9 18dc 18e0 18e4 18e7 18eb 18ef
18f2 18f5 18fa 18fd 1900 1903 1904 1909
190a 1 190c 1911 1915 1919 191c 1921
1926 1929 192d 192e 1933 1936 193b 193c
1941 1944 1949 194a 194f 1952 1956 195a
195e 1962 1965 1966 1968 196d 196e 1970
1971 1976 1979 197e 197f 1984 1987 198c
198d 1992 1995 1999 199a 199f 19a2 19a7
19a8 19ad 19ae 19b3 19b5 19b9 19bc 19c0
19c4 19c7 19cb 19cf 19d2 19d5 19da 19dd
19e0 19e3 19e4 19e9 19ea 1 19ec 19f1
19f5 19f9 19fc 1a01 1a06 1a09 1a0d 1a0e
1a13 1a14 1a19 1a1b 1a1f 1a22 1a26 1a2a
1a2d 1a31 1a35 1a38 1a3c 1a40 1a41 1a46
1a48 1a4c 1a4f 1a53 1a56 1a5a 1a5e 1a62
1a66 1a6a 1a6e 1a7a 1a7e 1a82 1a85 1a89
1a8d 1a8f 1a93 1a97 1a9b 1a9e 1aa2 1aa6
1aa8 1aa9 1aae 1 1ab2 1ab6 1aba 1abd
1ac1 1ac5 1ac9 1acd 1ad0 1ad4 1ad8 1adb
1ade 1ae3 1ae6 1ae9 1aec 1aed 1af2 1af3
1 1af5 1afa 1afe 1b02 1b05 1b0a 1b0f
1b12 1b16 1b17 1b1c 1b1f 1b24 1b25 1b2a
1b2d 1b32 1b33 1b38 1b3b 1b3f 1b40 1b45
1b48 1b4d 1b4e 1b53 1b56 1b5a 1b5e 1b61
1b64 1b65 1b67 1b68 1b6d 1b6e 1b73 1b75
1b79 1b7c 1b7e 1b82 1b84 1b90 1b92 1b94
1b95 1b9a 1b9e 1ba0 1bac 1bae 1bb2 1bb6
1bb9 1bba 1bbc 1bbf 1bc2 1bc3 1bc8 1bcc
1bd0 1bd3 1bd7 1bdb 1bde 1be1 1be6 1be9
1bec 1bef 1bf0 1bf5 1bf6 1 1bf8 1bfd
1c01 1c05 1c08 1c0d 1c12 1c15 1c19 1c1a
1c1f 1c22 1c27 1c28 1c2d 1c30 1c35 1c36
1c3b 1c3e 1c42 1c46 1c4a 1c4e 1c51 1c52
1c54 1c59 1c5a 1c5c 1c5d 1c62 1c65 1c6a
1c6b 1c70 1c73 1c78 1c79 1c7e 1c81 1c85
1c86 1c8b 1c8e 1c93 1c94 1c99 1c9a 1c9f
1ca1 1ca5 1ca8 1cac 1cb0 1cb3 1cb7 1cbb
1cbe 1cc1 1cc6 1cc9 1ccc 1ccf 1cd0 1cd5
1cd6 1 1cd8 1cdd 1ce1 1ce5 1ce8 1ced
1cf2 1cf5 1cf9 1cfa 1cff 1d00 1d05 1d07
1d0b 1d0e 1d12 1d16 1d19 1d1d 1d21 1d24
1d28 1d2c 1d2d 1d32 1d34 1d38 1d3b 1d3f
1d43 1d47 1d4b 1d4f 1d53 1d57 1d5b 1d5f
1d63 1d67 1d6b 1d6f 1d73 1d77 1d7b 1d7f
1d83 1d87 1d8b 1d8f 1d93 1d97 1d9b 1d9f
1da3 1da7 1dab 1daf 1db3 1db7 1dbb 1dbf
1dc3 1dc7 1dcb 1dcf 1ddb 1ddf 1de3 1de6
1dea 1dee 1df0 1df4 1df8 1dfc 1dff 1e03
1e07 1e09 1e0a 1e0f 1 1e13 1e17 1e1b
1e1e 1e22 1e26 1e2a 1e2e 1e31 1e35 1e39
1e3c 1e3f 1e44 1e47 1e4a 1e4d 1e4e 1e53
1e54 1 1e56 1e5b 1e5f 1e63 1e66 1e6b
1e70 1e73 1e77 1e78 1e7d 1e80 1e85 1e86
1e8b 1e8e 1e93 1e94 1e99 1e9c 1ea0 1ea1
1ea6 1ea9 1eae 1eaf 1eb4 1eb7 1ebb 1ebf
1ec2 1ec5 1ec6 1ec8 1ec9 1ece 1ecf 1ed4
1ed6 1eda 1edd 1edf 1ee3 1ee5 1ef1 1ef3
1ef5 1ef6 1efb 1eff 1f01 1f0d 1f0f 1f13
1f17 1f1a 1f1e 1f21 1f25 1f28 1f29 1f2e
1f31 1f35 1f39 1f3c 1f40 1f44 1f47 1f4a
1f4f 1f52 1f55 1f58 1f59 1f5e 1f5f 1
1f61 1f66 1f6a 1f6e 1f71 1f76 1f7b 1f7e
1f82 1f83 1f88 1f8b 1f90 1f91 1f96 1f99
1f9e 1f9f 1fa4 1fa7 1fab 1faf 1fb3 1fb7
1fba 1fbb 1fbd 1fc2 1fc3 1fc5 1fc6 1fcb
1fce 1fd3 1fd4 1fd9 1fdc 1fe1 1fe2 1fe7
1fea 1fee 1fef 1ff4 1ff7 1ffc 1ffd 2002
2003 2008 200a 200e 2011 2015 2019 201c
2020 2024 2027 202a 202f 2032 2035 2038
2039 203e 203f 1 2041 2046 204a 204e
2051 2056 205b 205e 2062 2063 2068 2069
206e 2070 2074 2077 207b 207f 2082 2086
208a 208d 2091 2095 2096 209b 209d 20a1
20a4 20a8 20ac 20b0 20b4 20b8 20bc 20c0
20cc 20d0 20d4 20d7 20db 20df 20e1 20e5
20e9 20ed 20f0 20f4 20f8 20fa 20fb 2100
1 2104 2108 210c 210f 2113 2117 211b
211f 2122 2126 212a 212d 2130 2135 2138
213b 213e 213f 2144 2145 1 2147 214c
2150 2154 2157 215c 2161 2164 2168 2169
216e 2171 2176 2177 217c 217f 2184 2185
218a 218d 2191 2192 2197 219a 219f 21a0
21a5 21a8 21ac 21b0 21b3 21b6 21b7 21b9
21ba 21bf 21c0 21c5 21c7 21cb 21ce 21d0
21d4 21d6 21e2 21e4 21e6 21e7 21ec 21f0
21f2 21fe 2200 2204 2208 220b 220f 2212
2216 2219 221a 221f 2222 2226 222a 222d
2231 2235 2238 223b 2240 2243 2246 2249
224a 224f 2250 1 2252 2257 225b 225f
2262 2267 226c 226f 2273 2274 2279 227c
2281 2282 2287 228a 228f 2290 2295 2298
229c 22a0 22a4 22a8 22ab 22ac 22ae 22b3
22b4 22b6 22b7 22bc 22bf 22c4 22c5 22ca
22cd 22d2 22d3 22d8 22db 22df 22e0 22e5
22e8 22ed 22ee 22f3 22f4 22f9 22fb 22ff
2302 2306 230a 230d 2311 2315 2318 231b
2320 2323 2326 2329 232a 232f 2330 1
2332 2337 233b 233f 2342 2347 234c 234f
2353 2354 2359 235a 235f 2361 2365 2368
236c 2370 2373 2377 237b 237e 2382 2386
2387 238c 238e 2392 2395 2399 239d 23a1
23a5 23a9 23b5 23b9 23bd 23c0 23c4 23c8
23ca 23ce 23d2 23d6 23d9 23dd 23e1 23e3
23e4 23e9 1 23ed 23f1 23f5 23f8 23fc
2400 2404 2408 240b 240f 2413 2416 2419
241e 2421 2424 2427 2428 242d 242e 1
2430 2435 2439 243d 2440 2445 244a 244d
2451 2452 2457 245a 245f 2460 2465 2468
246d 246e 2473 2476 247a 247b 2480 2483
2488 2489 248e 2491 2495 2499 249c 249f
24a0 24a2 24a3 24a8 24a9 24ae 24b0 24b4
24b7 24b9 24bd 24bf 24cb 24cd 24cf 24d0
24d5 24d9 24db 24e7 24e9 24ed 24f1 24f4
24f8 24fd 24fe 2500 2503 2506 2507 250c
2510 2515 2519 251b 251f 2522 2526 252a
252d 2531 2534 2538 253b 253c 2541 2544
2548 254c 254f 2553 2557 255a 255d 2562
2565 2568 256b 256c 2571 2572 1 2574
2579 257d 2581 2584 2589 258e 2591 2595
2596 259b 259e 25a3 25a4 25a9 25ac 25b1
25b2 25b7 25ba 25be 25c2 25c6 25ca 25cd
25ce 25d0 25d5 25d6 25d8 25d9 25de 25e1
25e6 25e7 25ec 25ef 25f4 25f5 25fa 25fd
2601 2602 2607 260a 260f 2610 2615 2616
261b 261d 2621 2624 2628 262c 262f 2633
2637 263a 263d 2642 2645 2648 264b 264c
2651 2652 1 2654 2659 265d 2661 2664
2669 266e 2671 2675 2676 267b 267c 2681
2683 2687 268a 268e 2692 2695 2699 269d
26a0 26a4 26a8 26a9 26ae 26b0 26b4 26b7
26bb 26bf 26c3 26c7 26cb 26cf 26d3 26d6
26d7 26d9 26dc 26dd 26df 26e3 26e7 26eb
26ef 26f3 26f6 26f7 26f9 26fc 26fd 26ff
2703 2707 270b 270f 2713 2716 2717 2719
271c 271d 271f 2723 2727 272a 272d 272e
2733 2737 273b 273e 273f 2741 2744 2747
2748 274d 2751 2755 2759 275d 2761 2765
2769 276d 2771 2775 2779 2785 2789 278d
2790 2794 2798 279a 279e 27a2 27a6 27a9
27ad 27b1 27b3 27b4 27b9 1 27bd 27c1
27c5 27c8 27cc 27d0 27d4 27d8 27db 27df
27e3 27e6 27e9 27ee 27f1 27f4 27f7 27f8
27fd 27fe 1 2800 2805 2809 280d 2810
2815 281a 281d 2821 2822 2827 282a 282f
2830 2835 2838 283d 283e 2843 2846 284a
284b 2850 2853 2858 2859 285e 2861 2865
2869 286c 286f 2870 2872 2873 2878 2879
287e 2880 2884 2887 2889 288d 288f 289b
289d 289f 28a0 28a5 28a9 28ab 28b7 28b9
28bd 28bf 28c3 28c7 28ca 28cb 28cd 28d0
28d3 28d4 28d9 28dd 28e1 28e5 28e9 28ed
28f1 28f5 28f9 28fd 2901 2905 2911 2915
2919 291c 2920 2924 2926 292a 292e 2932
2935 2939 293d 293f 2940 2945 1 2949
294d 2951 2954 2958 295c 2960 2964 2967
296b 296f 2972 2975 297a 297d 2980 2983
2984 2989 298a 1 298c 2991 2995 2999
299c 29a1 29a6 29a9 29ad 29ae 29b3 29b6
29bb 29bc 29c1 29c4 29c9 29ca 29cf 29d2
29d6 29d7 29dc 29df 29e4 29e5 29ea 29ed
29f1 29f5 29f8 29fb 29fc 29fe 29ff 2a04
2a05 2a0a 2a0c 2a10 2a13 2a15 2a19 2a1b
2a27 2a29 2a2b 2a2c 2a31 2a35 2a37 2a43
2a45 2a47 2a4b 2a4f 2a52 2a56 2a5a 2a5d
2a5e 2a60 2a63 2a66 2a67 2a6c 2a70 2a74
2a77 2a7b 2a7f 2a82 2a85 2a8a 2a8d 2a90
2a93 2a94 2a99 2a9a 1 2a9c 2aa1 2aa5
2aa9 2aac 2ab1 2ab6 2ab9 2abd 2abe 2ac3
2ac6 2acb 2acc 2ad1 2ad4 2ad9 2ada 2adf
2ae2 2ae6 2aea 2aee 2af2 2af5 2af6 2af8
2afd 2afe 2b00 2b01 2b06 2b09 2b0e 2b0f
2b14 2b17 2b1c 2b1d 2b22 2b25 2b29 2b2a
2b2f 2b32 2b37 2b38 2b3d 2b3e 2b43 2b45
2b49 2b4c 2b50 2b54 2b57 2b5b 2b5f 2b62
2b65 2b6a 2b6d 2b70 2b73 2b74 2b79 2b7a
1 2b7c 2b81 2b85 2b89 2b8c 2b91 2b96
2b99 2b9d 2b9e 2ba3 2ba4 2ba9 2bab 2baf
2bb2 2bb6 2bba 2bbd 2bc1 2bc5 2bc8 2bcc
2bd0 2bd1 2bd6 2bd8 2bdc 2bdf 2be1 2be5
2be9 2bed 2bf1 2bf5 2bf9 2bfd 2c01 2c05
2c09 2c0d 2c19 2c1d 2c21 2c24 2c28 2c2c
2c2e 2c32 2c36 2c3a 2c3d 2c41 2c45 2c47
2c48 2c4d 1 2c51 2c55 2c59 2c5c 2c60
2c64 2c68 2c6c 2c6f 2c73 2c77 2c7a 2c7d
2c82 2c85 2c88 2c8b 2c8c 2c91 2c92 1
2c94 2c99 2c9d 2ca1 2ca4 2ca9 2cae 2cb1
2cb5 2cb6 2cbb 2cbe 2cc3 2cc4 2cc9 2ccc
2cd1 2cd2 2cd7 2cda 2cde 2cdf 2ce4 2ce7
2cec 2ced 2cf2 2cf5 2cf9 2cfd 2d00 2d03
2d04 2d06 2d07 2d0c 2d0d 2d12 2d14 2d18
2d1b 2d1d 2d21 2d23 2d2f 2d31 2d33 2d34
2d39 2d3d 2d3f 2d4b 2d4d 2d51 2d55 2d59
2d5d 2d61 2d65 2d69 2d6d 2d71 2d75 2d79
2d85 2d89 2d8d 2d90 2d94 2d98 2d9a 2d9e
2da2 2da6 2da9 2dad 2db1 2db3 2db4 2db9
1 2dbd 2dc1 2dc5 2dc8 2dcc 2dd0 2dd4
2dd8 2ddb 2ddf 2de3 2de6 2de9 2dee 2df1
2df4 2df7 2df8 2dfd 2dfe 1 2e00 2e05
2e09 2e0d 2e10 2e15 2e1a 2e1d 2e21 2e22
2e27 2e2a 2e2f 2e30 2e35 2e38 2e3d 2e3e
2e43 2e46 2e4a 2e4b 2e50 2e53 2e58 2e59
2e5e 2e61 2e65 2e69 2e6c 2e6f 2e70 2e72
2e73 2e78 2e79 2e7e 2e80 2e84 2e87 2e89
2e8d 2e8f 2e9b 2e9d 2e9f 2ea0 2ea5 2ea9
2eab 2eb7 2eb9 2ebd 2ec1 2ec5 2ec8 2ec9
2ecb 2ecc 2ece 2ed1 2ed4 2ed7 2ed8 2edd
2ee1 2ee5 2ee8 2ee9 2eeb 2eee 2ef1 2ef2
2ef7 2efa 2efe 2f02 2f05 2f06 2f08 2f0b
2f0e 2f0f 2f14 1 2f17 2f1c 1 2f1f
2f24 2f28 2f2c 2f30 2f34 2f38 2f3c 2f40
2f44 2f48 2f4c 2f58 2f5c 2f60 2f63 2f67
2f6b 2f6d 2f71 2f75 2f79 2f7c 2f80 2f84
2f86 2f87 2f8c 1 2f90 2f94 2f98 2f9b
2f9f 2fa3 2fa7 2fab 2fae 2fb2 2fb6 2fb9
2fbc 2fc1 2fc4 2fc7 2fca 2fcb 2fd0 2fd1
1 2fd3 2fd8 2fdc 2fe0 2fe3 2fe8 2fed
2ff0 2ff4 2ff5 2ffa 2ffd 3002 3003 3008
300b 3010 3011 3016 3019 301d 301e 3023
3026 302b 302c 3031 3034 3038 303c 303f
3042 3043 3045 3046 304b 304c 3051 3053
3057 305a 305c 3060 3062 306e 3070 3072
3073 3078 307c 307e 308a 308c 3090 3094
3097 3098 309a 309d 30a0 30a1 30a6 30aa
30ae 30b1 30b5 30b9 30bc 30bf 30c4 30c7
30ca 30cd 30ce 30d3 30d4 1 30d6 30db
30df 30e3 30e6 30eb 30f0 30f3 30f7 30f8
30fd 3100 3105 3106 310b 310e 3113 3114
3119 311c 3120 3124 3128 312c 312f 3130
3132 3137 3138 313a 313b 3140 3143 3148
3149 314e 3151 3156 3157 315c 315f 3163
3164 3169 316c 3171 3172 3177 3178 317d
317f 3183 3186 318a 318e 3191 3195 3199
319c 319f 31a4 31a7 31aa 31ad 31ae 31b3
31b4 1 31b6 31bb 31bf 31c3 31c6 31cb
31d0 31d3 31d7 31d8 31dd 31de 31e3 31e5
31e9 31ec 31f0 31f4 31f7 31fb 31ff 3202
3206 320a 320b 3210 3212 3216 3219 321b
321f 3222 3224 3228 322c 322f 3233 3238
323c 3240 3245 3249 324d 3252 3256 325a
325b 325f 3263 3264 3268 326c 3270 3275
3279 327a 327e 3282 3287 328c 3290 3294
3299 329d 32a1 32a4 32a7 32ab 32ad 32b1
32b5 32b8 32bb 32bf 32c1 32c5 32c9 32cc
32d0 32d4 32d7 32db 32de 32e2 32e5 32e6
32eb 32ee 32f2 32f5 32f9 32fb 32ff 3302
3306 3308 330c 3310 3313 3317 331b 3320
3322 3326 332a 332d 3331 3335 3337 333b
333f 3343 3346 334a 334e 3350 3351 3356
1 335a 335e 3362 3365 3369 336d 3371
3375 3378 337c 3380 3383 3386 338b 338e
3391 3394 3395 339a 339b 1 339d 33a2
33a6 33aa 33ad 33b2 33b7 33ba 33be 33bf
33c4 33c7 33cc 33cd 33d2 33d5 33da 33db
33e0 33e3 33e7 33e8 33ed 33f0 33f5 33f6
33fb 33fe 3402 3406 3409 340c 340d 340f
3410 3415 3416 341b 341d 3421 3424 3426
342a 342c 3438 343a 343c 343d 3442 3446
3448 3454 3456 345a 345e 3461 3462 3464
3467 346a 346b 3470 3474 3478 347c 3480
3484 3488 348c 3490 3494 3498 349c 34a8
34ac 34b0 34b3 34b7 34bb 34bd 34c1 34c5
34c9 34cc 34d0 34d4 34d6 34d7 34dc 1
34e0 34e4 34e8 34eb 34ef 34f3 34f7 34fb
34fe 3502 3506 3509 350c 3511 3514 3517
351a 351b 3520 3521 1 3523 3528 352c
3530 3533 3538 353d 3540 3544 3545 354a
354d 3552 3553 3558 355b 3560 3561 3566
3569 356d 356e 3573 3576 357b 357c 3581
3584 3588 358c 358f 3592 3593 3595 3596
359b 359c 35a1 35a3 35a7 35aa 35ac 35b0
35b2 35be 35c0 35c2 35c3 35c8 35cc 35ce
35da 35dc 35e0 35e4 35e7 35e8 35ea 35ed
35f0 35f1 35f6 35fa 35fd 3601 3605 3609
360d 3611 3615 3619 361d 3621 3625 3629
362d 3631 3635 3641 3645 3649 364c 3650
3654 3656 365a 365e 3662 3665 3669 366d
366f 3670 3675 1 3679 367d 3681 3684
3688 368c 3690 3694 3697 369b 369f 36a2
36a5 36aa 36ad 36b0 36b3 36b4 36b9 36ba
1 36bc 36c1 36c5 36c9 36cc 36d1 36d6
36d9 36dd 36de 36e3 36e6 36eb 36ec 36f1
36f4 36f9 36fa 36ff 3702 3706 3707 370c
370f 3714 3715 371a 371d 3721 3725 3728
372b 372c 372e 372f 3734 3735 373a 373c
3740 3743 3745 3749 374b 3757 3759 375b
375c 3761 3765 3767 3773 3775 3779 377d
3780 3784 3787 378b 378e 378f 3794 3797
379b 379f 37a2 37a6 37aa 37ad 37b0 37b5
37b8 37bb 37be 37bf 37c4 37c5 1 37c7
37cc 37d0 37d4 37d7 37dc 37e1 37e4 37e8
37e9 37ee 37f1 37f6 37f7 37fc 37ff 3804
3805 380a 380d 3811 3815 3819 381d 3820
3821 3823 3828 3829 382b 382c 3831 3834
3839 383a 383f 3842 3847 3848 384d 3850
3854 3855 385a 385d 3862 3863 3868 3869
386e 3870 3874 3877 387b 387f 3882 3886
388a 388d 3890 3895 3898 389b 389e 389f
38a4 38a5 1 38a7 38ac 38b0 38b4 38b7
38bc 38c1 38c4 38c8 38c9 38ce 38cf 38d4
38d6 38da 38dd 38e1 38e5 38e8 38ec 38f0
38f3 38f7 38fb 38fc 3901 3905 3907 390b
390e 390f 3914 3917 391a 391e 3921 3922
3927 392a 1 392d 3932 3936 393a 393d
3941 3945 3948 3949 394b 394e 3951 3952
3957 395a 395e 3962 3965 3969 396d 3970
3971 3973 3976 3979 397a 397f 1 3982
3987 398b 398f 3992 3996 399a 399d 39a0
39a5 39a8 39ab 39ae 39af 39b4 39b5 1
39b7 39bc 39c0 39c4 39c7 39cc 39d1 39d4
39d8 39d9 39de 39e1 39e6 39e7 39ec 39ef
39f4 39f5 39fa 39fd 3a01 3a05 3a09 3a0d
3a10 3a11 3a13 3a18 3a19 3a1b 3a1c 3a21
3a24 3a29 3a2a 3a2f 3a32 3a37 3a38 3a3d
3a40 3a44 3a45 3a4a 3a4d 3a52 3a53 3a58
3a59 3a5e 3a60 3a64 3a67 3a6b 3a6f 3a72
3a76 3a7a 3a7d 3a80 3a85 3a88 3a8b 3a8e
3a8f 3a94 3a95 1 3a97 3a9c 3aa0 3aa4
3aa7 3aac 3ab1 3ab4 3ab8 3ab9 3abe 3abf
3ac4 3ac6 3aca 3acd 3ad1 3ad5 3ad8 3adc
3ae0 3ae3 3ae7 3aeb 3aec 3af1 3af3 3af7
3afa 3afc 3b00 3b04 3b07 3b09 3b0d 3b10
3b14 3b18 3b1b 3b1c 3b1e 3b21 3b24 3b25
3b2a 3b2e 3b31 3b35 3b39 3b3d 3b41 3b43
3b47 3b4b 3b4f 3b53 3b57 3b5b 3b5f 3b63
3b67 3b6b 3b6f 3b7b 3b7f 3b83 3b86 3b8a
3b8e 3b90 3b94 3b98 3b9c 3b9f 3ba3 3ba7
3ba9 3baa 3baf 1 3bb3 3bb7 3bbb 3bbe
3bc2 3bc6 3bca 3bce 3bd1 3bd5 3bd9 3bdc
3bdf 3be4 3be7 3bea 3bed 3bee 3bf3 3bf4
1 3bf6 3bfb 3bff 3c03 3c06 3c0b 3c10
3c13 3c17 3c18 3c1d 3c20 3c25 3c26 3c2b
3c2e 3c33 3c34 3c39 3c3c 3c40 3c41 3c46
3c49 3c4e 3c4f 3c54 3c57 3c5b 3c5f 3c62
3c65 3c66 3c68 3c69 3c6e 3c6f 3c74 3c76
3c7a 3c7d 3c7f 3c83 3c85 3c91 3c93 3c95
3c96 3c9b 3c9f 3ca1 3cad 3caf 3cb3 3cb7
3cba 3cbe 3cc1 3cc5 3cc8 3cc9 3cce 3cd1
3cd5 3cda 3cde 3ce2 3ce6 3ce7 3cec 3cf0
3cf4 3cf8 3cfb 3cff 3d03 3d06 3d07 3d09
3d0d 3d11 3d14 3d18 3d19 3d1b 3d1f 3d23
3d27 3d2b 3d2e 3d32 3d36 3d3b 3d3f 3d43
3d46 3d4a 3d4e 3d52 3d56 3d5a 3d5e 3d62
3d66 3d6a 3d6e 3d6f 3d71 3d75 3d79 3d7d
3d81 3d85 3d88 3d89 3d8b 3d8c 3d8e 3d92
3d96 3d9a 3d9e 3da2 3da6 3daa 3dae 3db2
3db6 3dba 3dbe 3dc2 3dc6 3dca 3dce 3dd2
3dd6 3dda 3dde 3de2 3de6 3dea 3dee 3df2
3df6 3dfa 3e06 3e0a 3e0f 3e13 3e16 3e17
3e1c 3e20 3e24 3e27 3e2a 3e2e 3e30 3e34
3e38 3e3b 3e3e 3e42 3e44 3e48 3e4c 3e4f
3e51 3e55 3e59 3e5d 3e60 3e64 3e68 3e6a
3e6b 3e70 1 3e74 3e78 3e7c 3e7f 3e83
3e87 3e8b 3e8f 3e92 3e96 3e9a 3e9d 3ea0
3ea5 3ea8 3eab 3eae 3eaf 3eb4 3eb5 1
3eb7 3ebc 3ec0 3ec4 3ec7 3ecc 3ed1 3ed4
3ed8 3ed9 3ede 3ee1 3ee6 3ee7 3eec 3eef
3ef4 3ef5 3efa 3efd 3f01 3f02 3f07 3f0a
3f0f 3f10 3f15 3f18 3f1c 3f20 3f23 3f26
3f27 3f29 3f2a 3f2f 3f30 3f35 3f37 3f3b
3f3e 3f40 3f44 3f46 3f52 3f54 3f56 3f57
3f5c 3f60 3f62 3f6e 3f70 3f74 3f78 3f7c
3f80 3f84 3f90 3f94 3f98 3f9b 3f9f 3fa3
3fa5 3fa9 3fad 3fb1 3fb4 3fb8 3fbc 3fbe
3fbf 3fc4 1 3fc8 3fcc 3fd0 3fd3 3fd7
3fdb 3fdf 3fe3 3fe6 3fea 3fee 3ff1 3ff4
3ff9 3ffc 3fff 4002 4003 4008 4009 1
400b 4010 4014 4018 401b 4020 4025 4028
402c 402d 4032 4035 403a 403b 4040 4043
4048 4049 404e 4051 4055 4056 405b 405e
4063 4064 4069 406c 4070 4074 4077 407a
407b 407d 407e 4083 4084 4089 408b 408f
4092 4094 4098 409a 40a6 40a8 40aa 40ab
40b0 40b4 40b6 40c2 40c4 40c8 40cc 40cf
40d3 40d7 40d8 40da 40dd 40e0 40e1 40e6
40ea 40ed 40f1 40f3 40f7 40fb 40ff 4103
4107 410b 410f 4113 4117 411b 411f 4123
412f 4133 4137 413a 413e 4142 4144 4148
414c 4150 4153 4157 415b 415d 415e 4163
1 4167 416b 416f 4172 4176 417a 417e
4182 4185 4189 418d 4190 4193 4198 419b
419e 41a1 41a2 41a7 41a8 1 41aa 41af
41b3 41b7 41ba 41bf 41c4 41c7 41cb 41cc
41d1 41d4 41d9 41da 41df 41e2 41e7 41e8
41ed 41f0 41f4 41f5 41fa 41fd 4202 4203
4208 420b 420f 4213 4216 4219 421a 421c
421d 4222 4223 4228 422a 422e 4231 4233
4237 4239 4245 4247 4249 424a 424f 4253
4255 4261 4263 4265 4269 426d 4270 4274
4278 427b 427c 427e 4281 4284 4285 428a
428e 4292 4296 429a 429e 42a2 42a6 42aa
42ae 42b2 42be 42c2 42c7 42cb 42ce 42cf
42d4 42d8 42dc 42df 42e2 42e6 42e8 42ec
42f0 42f3 42f6 42fa 42fc 4300 4304 4307
4309 430d 4311 4315 4318 431c 4320 4322
4323 4328 1 432c 4330 4334 4337 433b
433f 4343 4347 434a 434e 4352 4355 4358
435d 4360 4363 4366 4367 436c 436d 1
436f 4374 4378 437c 437f 4384 4389 438c
4390 4391 4396 4399 439e 439f 43a4 43a7
43ac 43ad 43b2 43b5 43b9 43ba 43bf 43c2
43c7 43c8 43cd 43d0 43d4 43d8 43db 43de
43df 43e1 43e2 43e7 43e8 43ed 43ef 43f3
43f6 43f8 43fc 43fe 440a 440c 440e 440f
4414 4418 441a 4426 4428 442c 4430 4434
4438 443c 4440 4444 4448 444c 4450 4454
4458 445c 4460 4464 4468 446c 4478 447c
4481 4485 4488 4489 448e 4492 4496 4499
449c 44a0 44a2 44a6 44aa 44ad 44b0 44b4
44b6 44ba 44be 44c1 44c3 44c7 44cb 44cf
44d2 44d6 44da 44dc 44dd 44e2 1 44e6
44ea 44ee 44f1 44f5 44f9 44fd 4501 4504
4508 450c 450f 4512 4517 451a 451d 4520
4521 4526 4527 1 4529 452e 4532 4536
4539 453e 4543 4546 454a 454b 4550 4553
4558 4559 455e 4561 4566 4567 456c 456f
4573 4574 4579 457c 4581 4582 4587 458a
458e 4592 4595 4598 4599 459b 459c 45a1
45a2 45a7 45a9 45ad 45b0 45b2 45b6 45b8
45c4 45c6 45c8 45c9 45ce 45d2 45d4 45e0
45e2 45e4 45e8 45ec 45f0 45f4 45f8 45fc
4600 4604 4608 460c 4610 4614 4618 461c
4620 4624 4628 4634 4638 463d 4641 4644
4645 464a 464e 4652 4655 4658 465c 465e
4662 4666 4669 466c 4670 4672 4676 467a
467d 467f 4683 4687 468b 468e 4692 4696
4698 4699 469e 1 46a2 46a6 46aa 46ad
46b1 46b5 46b9 46bd 46c0 46c4 46c8 46cb
46ce 46d3 46d6 46d9 46dc 46dd 46e2 46e3
1 46e5 46ea 46ee 46f2 46f5 46fa 46ff
4702 4706 4707 470c 470f 4714 4715 471a
471d 4722 4723 4728 472b 472f 4730 4735
4738 473d 473e 4743 4746 474a 474e 4751
4754 4755 4757 4758 475d 475e 4763 4765
4769 476c 476e 4772 4774 4780 4782 4784
4785 478a 478e 4790 479c 479e 47a0 47a4
47a8 47ab 47af 47b1 47b5 47b9 47bc 47c0
47c3 47c7 47ca 47cb 47d0 47d3 47d7 47db
47df 47e3 47e7 47eb 47f7 47fb 47ff 4802
4806 480a 480c 4810 4814 4818 481b 481f
4823 4825 4826 482b 1 482f 4833 4837
483a 483e 4842 4846 484a 484d 4851 4855
4858 485b 4860 4863 4866 4869 486a 486f
4870 1 4872 4877 487b 487f 4882 4887
488c 488f 4893 4894 4899 489c 48a1 48a2
48a7 48aa 48af 48b0 48b5 48b8 48bc 48bd
48c2 48c5 48ca 48cb 48d0 48d3 48d7 48db
48de 48e1 48e2 48e4 48e5 48ea 48eb 48f0
48f2 48f6 48f9 48fb 48ff 4901 490d 490f
4911 4912 4917 491b 491d 4929 492b 492f
4933 4936 4937 4939 493d 4940 4944 4947
4948 494d 4951 4955 4958 495c 4960 4963
4964 4966 496a 496e 4971 4975 4976 4978
497c 4980 4984 4988 498c 4990 4994 4998
499c 49a0 49a4 49b0 49b4 49b9 49bd 49c0
49c1 49c6 49ca 49ce 49d1 49d4 49d8 49da
49de 49e2 49e5 49e8 49ec 49ee 49f2 49f6
49f9 49fb 49ff 4a03 4a07 4a0a 4a0e 4a12
4a14 4a15 4a1a 1 4a1e 4a22 4a26 4a29
4a2d 4a31 4a35 4a39 4a3c 4a40 4a44 4a47
4a4a 4a4f 4a52 4a55 4a58 4a59 4a5e 4a5f
1 4a61 4a66 4a6a 4a6e 4a71 4a76 4a7b
4a7e 4a82 4a83 4a88 4a8b 4a90 4a91 4a96
4a99 4a9e 4a9f 4aa4 4aa7 4aab 4aac 4ab1
4ab4 4ab9 4aba 4abf 4ac2 4ac6 4aca 4acd
4ad0 4ad1 4ad3 4ad4 4ad9 4ada 4adf 4ae1
4ae5 4ae8 4aea 4aee 4af0 4afc 4afe 4b00
4b01 4b06 4b0a 4b0c 4b18 4b1a 4b1e 4b22
4b26 4b27 4b29 4b2d 4b31 4b33 4b37 4b3b
4b3f 4b42 4b43 4b45 4b49 4b4d 4b50 4b51
4b53 4b56 4b5a 4b5d 4b61 4b64 4b65 4b6a
4b6e 4b72 4b76 4b7a 4b7e 4b82 4b86 4b8a
4b8e 4b92 4b96 4ba2 4ba6 4bab 4baf 4bb2
4bb3 4bb8 4bbc 4bc0 4bc3 4bc6 4bca 4bcc
4bd0 4bd4 4bd7 4bda 4bde 4be0 4be4 4be8
4beb 4bed 4bf1 4bf5 4bf9 4bfc 4c00 4c04
4c06 4c07 4c0c 1 4c10 4c14 4c18 4c1b
4c1f 4c23 4c27 4c2b 4c2e 4c32 4c36 4c39
4c3c 4c41 4c44 4c47 4c4a 4c4b 4c50 4c51
1 4c53 4c58 4c5c 4c60 4c63 4c68 4c6d
4c70 4c74 4c75 4c7a 4c7d 4c82 4c83 4c88
4c8b 4c90 4c91 4c96 4c99 4c9d 4c9e 4ca3
4ca6 4cab 4cac 4cb1 4cb4 4cb8 4cbc 4cbf
4cc2 4cc3 4cc5 4cc6 4ccb 4ccc 4cd1 4cd3
4cd7 4cda 4cdc 4ce0 4ce2 4cee 4cf0 4cf2
4cf3 4cf8 4cfc 4cfe 4d0a 4d0c 4d10 4d14
4d18 4d19 4d1b 4d1f 4d21 4d25 4d29 4d2d
4d31 4d35 4d39 4d3d 4d41 4d45 4d49 4d4d
4d51 4d5d 4d61 4d66 4d6a 4d6d 4d6e 4d73
4d77 4d7b 4d7e 4d81 4d85 4d87 4d8b 4d8f
4d92 4d95 4d99 4d9b 4d9f 4da3 4da6 4da8
4dac 4db0 4db4 4db7 4dbb 4dbf 4dc1 4dc2
4dc7 1 4dcb 4dcf 4dd3 4dd6 4dda 4dde
4de2 4de6 4de9 4ded 4df1 4df4 4df7 4dfc
4dff 4e02 4e05 4e06 4e0b 4e0c 1 4e0e
4e13 4e17 4e1b 4e1e 4e23 4e28 4e2b 4e2f
4e30 4e35 4e38 4e3d 4e3e 4e43 4e46 4e4b
4e4c 4e51 4e54 4e58 4e59 4e5e 4e61 4e66
4e67 4e6c 4e6f 4e73 4e77 4e7a 4e7d 4e7e
4e80 4e81 4e86 4e87 4e8c 4e8e 4e92 4e95
4e97 4e9b 4e9d 4ea9 4eab 4ead 4eae 4eb3
4eb7 4eb9 4ec5 4ec7 4ecb 4ecf 4ed3 4ed4
4ed6 4eda 4edc 4ee0 4ee4 4ee7 4ee9 4eed
4ef1 4ef4 4ef6 4efa 4efe 4f01 4f03 4f07
4f0b 4f0f 4f11 4f15 4f19 4f1c 4f20 4f25
4f29 4f2d 4f32 4f36 4f3a 4f3f 4f43 4f47
4f4b 4f4c 4f51 4f55 4f59 4f5d 4f61 4f65
4f69 4f6d 4f71 4f75 4f79 4f7d 4f81 4f85
4f89 4f8d 4f91 4f95 4f99 4f9d 4fa1 4fa5
4fa9 4fad 4fb9 4fbd 4fc1 4fc4 4fc8 4fcc
4fce 4fd2 4fd6 4fda 4fdd 4fe1 4fe5 4fe7
4fe8 4fed 1 4ff1 4ff5 4ff9 4ffc 5000
5004 5008 500c 500f 5013 5017 501a 501d
5022 5025 5028 502b 502c 5031 5032 1
5034 5039 503d 5041 5044 5049 504e 5051
5055 5056 505b 505e 5063 5064 5069 506c
5071 5072 5077 507a 507e 507f 5084 5087
508c 508d 5092 5095 5099 509d 50a0 50a3
50a4 50a6 50a7 50ac 50ad 50b2 50b4 50b8
50bb 50bd 50c1 50c3 50cf 50d1 50d3 50d4
50d9 50dd 50df 50eb 50ed 50f1 50f5 50f8
50fc 50ff 5103 5106 5107 510c 510f 5113
5116 511a 511c 5120 5123 5127 512b 512e
512f 5131 5134 5137 5138 513d 5140 5144
5148 514b 514c 514e 5151 5154 5155 515a
1 515d 5162 5166 516a 516e 5172 5176
517a 517e 5182 5186 518a 518e 5192 5193
5195 5199 519d 51a1 51a4 51a5 51a7 51aa
51ad 51ae 51b3 51b7 51bb 51bf 51c2 51c3
51c5 51c8 51cc 51d0 51d3 51d4 51d6 51d7
51dc 51e0 51e4 51e8 51ec 51ef 51f0 51f2
51f5 51f9 51fd 5200 5201 5203 5204 5209
520d 520f 5213 5216 521a 521e 5221 5222
5224 5227 522a 522b 5230 5234 5238 523c
523f 5240 5242 5245 5249 524d 5250 5251
5253 5254 5259 525d 5261 5265 5269 526c
526d 526f 5272 5276 527a 527d 527e 5280
5281 5286 528a 528c 5290 5293 5295 5299
529c 52a0 52a3 52a7 52ab 52ae 52b2 52b6
52b7 52bb 52bf 52c3 52c7 52cb 52cf 52d3
52d7 52e3 52e7 52eb 52ee 52f2 52f6 52f8
52fc 5300 5304 5307 530b 530f 5311 5312
5317 1 531b 531f 5323 5326 532a 532e
5332 5336 5339 533d 5341 5344 5347 534c
534f 5352 5355 5356 535b 535c 1 535e
5363 5367 536b 536e 5373 5378 537b 537f
5380 5385 5388 538d 538e 5393 5396 539b
539c 53a1 53a4 53a8 53a9 53ae 53b1 53b6
53b7 53bc 53bf 53c3 53c7 53ca 53cd 53ce
53d0 53d1 53d6 53d7 53dc 53de 53e2 53e5
53e7 53eb 53ed 53f9 53fb 53fd 53fe 5403
5407 5409 5415 5417 541b 541f 5422 5426
542a 542b 542d 5430 5433 5434 5439 543d
5441 5445 5449 544d 5451 5455 5459 545d
5461 546d 5471 5475 5478 547c 5480 5482
5486 548a 548e 5491 5495 5499 549b 549c
54a1 1 54a5 54a9 54ad 54b0 54b4 54b8
54bc 54c0 54c3 54c7 54cb 54ce 54d1 54d6
54d9 54dc 54df 54e0 54e5 54e6 1 54e8
54ed 54f1 54f5 54f8 54fd 5502 5505 5509
550a 550f 5512 5517 5518 551d 5520 5525
5526 552b 552e 5532 5533 5538 553b 5540
5541 5546 5549 554d 5551 5554 5557 5558
555a 555b 5560 5561 5566 5568 556c 556f
5571 5575 5577 5583 5585 5587 5588 558d
5591 5593 559f 55a1 55a5 55a9 55ac 55ad
55af 55b2 55b5 55b6 55bb 55bf 55c3 55c7
55c9 55cd 55d0 55d4 55d6 55da 55de 55e1
55e3 55e7 55ea 55ee 55f2 55f5 55f6 55f8
55fb 55fe 55ff 5604 5608 560c 560f 5613
5617 5618 561a 561d 5620 5621 5626 562a
562e 5632 5636 563a 563e 5642 5646 564a
564e 5652 5656 565a 565e 5662 5666 566a
566e 5672 5676 567a 567e 5682 5686 568a
568e 5692 5696 569a 569e 56a2 56a6 56aa
56ae 56b2 56b6 56ba 56be 56c2 56c6 56d2
56d6 56da 56dd 56e1 56e5 56e7 56eb 56ef
56f3 56f6 56fa 56fe 5700 5701 5706 1
570a 570e 5712 5715 5719 571d 5721 5725
5728 572c 5730 5733 5736 573b 573e 5741
5744 5745 574a 574b 1 574d 5752 5756
575a 575d 5762 5767 576a 576e 576f 5774
5777 577c 577d 5782 5785 578a 578b 5790
5793 5797 5798 579d 57a0 57a5 57a6 57ab
57ae 57b2 57b6 57b9 57bc 57bd 57bf 57c0
57c5 57c6 57cb 57cd 57d1 57d4 57d6 57da
57dc 57e8 57ea 57ec 57ed 57f2 57f6 57f8
5804 5806 580a 580e 5812 5816 5819 581d
5821 5825 5828 5829 582b 582e 5831 5832
5837 583b 583f 5842 5843 5845 5849 584c
5850 5853 5854 5856 5857 585c 585f 5863
5867 586a 586b 586d 5871 5874 5878 587b
587c 587e 587f 5884 1 5887 588c 5890
5894 5897 5898 589a 589d 58a0 58a1 58a6
58a9 58ad 58b1 58b5 58b8 58b9 58bb 58bf
58c3 58c6 58c7 58c9 58cc 58cf 58d2 58d3
58d8 1 58db 58e0 58e4 58e8 58eb 58ef
58f3 58f6 58f9 58fe 5901 5904 5907 5908
590d 590e 1 5910 5915 5919 591d 5920
5925 592a 592d 5931 5932 5937 593a 593f
5940 5945 5948 594d 594e 5953 5956 595a
595e 5962 5966 5969 596a 596c 5971 5972
5974 5975 597a 597d 5982 5983 5988 598b
5990 5991 5996 5999 599d 599e 59a3 59a6
59ab 59ac 59b1 59b2 59b7 59b9 59bd 59c0
59c4 59c8 59cb 59cf 59d3 59d6 59d9 59de
59e1 59e4 59e7 59e8 59ed 59ee 1 59f0
59f5 59f9 59fd 5a00 5a05 5a0a 5a0d 5a11
5a12 5a17 5a18 5a1d 5a1f 5a23 5a26 5a2a
5a2e 5a31 5a35 5a39 5a3c 5a40 5a44 5a45
5a4a 5a4c 5a50 5a53 5a57 5a5b 5a5f 5a62
5a63 5a65 5a68 5a6b 5a6c 5a71 5a75 5a78
5a7c 5a7e 5a82 5a85 5a87 5a8b 5a8f 5a92
5a94 5a98 5a9b 5a9f 5aa3 5aa7 5aab 5aaf
5ab3 5ab7 5abb 5abf 5ac3 5ac7 5acb 5acf
5ad3 5ad7 5adb 5adf 5ae3 5ae7 5aeb 5aef
5af3 5af7 5afb 5aff 5b03 5b07 5b09 5b0d
5b10 5b14 5b18 5b1b 5b1f 5b22 5b26 5b29
5b2a 5b2f 5b32 5b36 5b39 5b3a 5b3f 5b42
1 5b45 5b4a 5b4e 5b52 5b55 5b58 5b59
5b5b 5b5e 5b63 5b64 5b69 5b6d 5b70 5b73
5b74 5b79 5b7d 5b81 5b85 5b88 5b8c 5b90
5b94 5b98 5b9c 5ba0 5ba4 5ba8 5bac 5bb0
5bb4 5bb5 5bb7 5bbb 5bbd 5bc1 5bc4 5bc8
5bcc 5bcf 5bd0 5bd2 5bd5 5bd8 5bd9 5bde
5be2 5be6 5bea 5bf6 5bfa 5bff 5c03 5c06
5c07 5c0c 5c10 5c14 5c17 5c1a 5c1e 5c20
5c24 5c28 5c2b 5c2e 5c32 5c34 5c38 5c3c
5c3f 5c41 5c45 5c49 5c4d 5c50 5c54 5c58
5c5a 5c5b 5c60 1 5c64 5c68 5c6c 5c6f
5c73 5c77 5c7b 5c7f 5c82 5c86 5c8a 5c8d
5c90 5c95 5c98 5c9b 5c9e 5c9f 5ca4 5ca5
1 5ca7 5cac 5cb0 5cb4 5cb7 5cbc 5cc1
5cc4 5cc8 5cc9 5cce 5cd1 5cd6 5cd7 5cdc
5cdf 5ce4 5ce5 5cea 5ced 5cf1 5cf2 5cf7
5cfa 5cff 5d00 5d05 5d08 5d0c 5d10 5d13
5d16 5d17 5d19 5d1a 5d1f 5d20 5d25 5d27
5d2b 5d2e 5d30 5d34 5d36 5d42 5d44 5d46
5d47 5d4c 5d50 5d52 5d5e 5d60 5d64 5d68
5d6c 5d70 5d74 5d80 5d84 5d89 5d8d 5d90
5d91 5d96 5d9a 5d9e 5da1 5da4 5da8 5daa
5dae 5db2 5db5 5db8 5dbc 5dbe 5dc2 5dc6
5dc9 5dcb 5dcf 5dd3 5dd7 5dda 5dde 5de2
5de4 5de5 5dea 1 5dee 5df2 5df6 5df9
5dfd 5e01 5e05 5e09 5e0c 5e10 5e14 5e17
5e1a 5e1f 5e22 5e25 5e28 5e29 5e2e 5e2f
1 5e31 5e36 5e3a 5e3e 5e41 5e46 5e4b
5e4e 5e52 5e53 5e58 5e5b 5e60 5e61 5e66
5e69 5e6e 5e6f 5e74 5e77 5e7b 5e7c 5e81
5e84 5e89 5e8a 5e8f 5e92 5e96 5e9a 5e9d
5ea0 5ea1 5ea3 5ea4 5ea9 5eaa 5eaf 5eb1
5eb5 5eb8 5eba 5ebe 5ec0 5ecc 5ece 5ed0
5ed1 5ed6 5eda 5edc 5ee8 5eea 5eee 5ef2
5ef6 5efa 5efe 5f02 5f06 5f0a 5f0e 5f12
5f16 5f22 5f26 5f2a 5f2d 5f31 5f35 5f37
5f3b 5f3f 5f43 5f46 5f4a 5f4e 5f50 5f51
5f56 1 5f5a 5f5e 5f62 5f65 5f69 5f6d
5f71 5f75 5f78 5f7c 5f80 5f83 5f86 5f8b
5f8e 5f91 5f94 5f95 5f9a 5f9b 1 5f9d
5fa2 5fa6 5faa 5fad 5fb2 5fb7 5fba 5fbe
5fbf 5fc4 5fc7 5fcc 5fcd 5fd2 5fd5 5fda
5fdb 5fe0 5fe3 5fe7 5fe8 5fed 5ff0 5ff5
5ff6 5ffb 5ffe 6002 6006 6009 600c 600d
600f 6010 6015 6016 601b 601d 6021 6024
6026 602a 602c 6038 603a 603c 603d 6042
6046 6048 6054 6056 605a 605e 6061 6065
6069 606c 606d 606f 6072 6075 6076 607b
607f 6083 6087 608b 608f 6093 6097 609b
609f 60a3 60af 60b3 60b7 60ba 60be 60c2
60c4 60c8 60cc 60d0 60d3 60d7 60db 60dd
60de 60e3 1 60e7 60eb 60ef 60f2 60f6
60fa 60fe 6102 6105 6109 610d 6110 6113
6118 611b 611e 6121 6122 6127 6128 1
612a 612f 6133 6137 613a 613f 6144 6147
614b 614c 6151 6154 6159 615a 615f 6162
6167 6168 616d 6170 6174 6175 617a 617d
6182 6183 6188 618b 618f 6193 6196 6199
619a 619c 619d 61a2 61a3 61a8 61aa 61ae
61b1 61b3 61b7 61b9 61c5 61c7 61c9 61ca
61cf 61d3 61d5 61e1 61e3 61e5 61e9 61ec
61ee 61f2 61f6 61f9 61fa 61fc 61ff 6202
6203 6208 620b 620f 6213 6216 6217 6219
621c 621f 6220 6225 1 6228 622d 6231
6235 6238 6239 623b 623e 6241 6242 6247
624b 624f 6253 6257 625b 625f 6263 6267
626b 626f 6273 6277 627b 627f 6283 6287
628b 628f 6293 6297 629b 629f 62a3 62a7
62ab 62af 62b3 62b7 62bb 62bf 62cb 62cf
62d4 62d8 62db 62dc 62e1 62e5 62e9 62ec
62ef 62f3 62f5 62f9 62fd 6300 6303 6307
6309 630d 6311 6314 6316 631a 631e 6322
6325 6329 632d 632f 6330 6335 1 6339
633d 6341 6344 6348 634c 6350 6354 6357
635b 635f 6362 6365 636a 636d 6370 6373
6374 6379 637a 1 637c 6381 6385 6389
638c 6391 6396 6399 639d 639e 63a3 63a6
63ab 63ac 63b1 63b4 63b9 63ba 63bf 63c2
63c6 63c7 63cc 63cf 63d4 63d5 63da 63dd
63e1 63e5 63e8 63eb 63ec 63ee 63ef 63f4
63f5 63fa 63fc 6400 6403 6405 6409 640b
6417 6419 641b 641c 6421 6425 6427 6433
6435 6437 643b 643f 6443 6447 644b 644f
6453 6457 645b 645f 6463 6467 646b 646f
6473 6477 647b 647f 6483 6487 648b 648f
6493 6497 649b 649f 64a3 64a7 64ab 64af
64b3 64b7 64bb 64bf 64cb 64cf 64d4 64d8
64db 64dc 64e1 64e5 64e9 64ec 64ef 64f3
64f5 64f9 64fd 6500 6503 6507 6509 650d
6511 6514 6516 651a 651e 6522 6525 6529
652d 652f 6530 6535 1 6539 653d 6541
6544 6548 654c 6550 6554 6557 655b 655f
6562 6565 656a 656d 6570 6573 6574 6579
657a 1 657c 6581 6585 6589 658c 6591
6596 6599 659d 659e 65a3 65a6 65ab 65ac
65b1 65b4 65b9 65ba 65bf 65c2 65c6 65c7
65cc 65cf 65d4 65d5 65da 65dd 65e1 65e5
65e8 65eb 65ec 65ee 65ef 65f4 65f5 65fa
65fc 6600 6603 6605 6609 660b 6617 6619
661b 661c 6621 6625 6627 6633 6635 6637
663b 663f 6642 6646 6648 664c 6650 6653
6654 6656 6659 665c 665d 6662 6666 666a
666e 6672 6676 667a 667e 6682 6686 668a
668e 6692 6696 669a 669e 66a2 66a6 66aa
66ae 66b2 66b6 66ba 66be 66c2 66c6 66ca
66d6 66da 66df 66e3 66e6 66e7 66ec 66f0
66f4 66f7 66fa 66fe 6700 6704 6708 670b
670e 6712 6714 6718 671c 671f 6721 6725
6729 672d 6730 6734 6738 673a 673b 6740
1 6744 6748 674c 674f 6753 6757 675b
675f 6762 6766 676a 676d 6770 6775 6778
677b 677e 677f 6784 6785 1 6787 678c
6790 6794 6797 679c 67a1 67a4 67a8 67a9
67ae 67b1 67b6 67b7 67bc 67bf 67c4 67c5
67ca 67cd 67d1 67d2 67d7 67da 67df 67e0
67e5 67e8 67ec 67f0 67f3 67f6 67f7 67f9
67fa 67ff 6800 6805 6807 680b 680e 6810
6814 6816 6822 6824 6826 6827 682c 6830
6832 683e 6840 6844 6846 684a 684e 6852
6855 6856 6858 685b 685e 685f 6864 6868
686c 6870 6873 6874 6876 6879 687d 6881
6884 6885 6887 6888 688d 6891 6895 6899
689d 68a0 68a1 68a3 68a6 68aa 68ae 68b1
68b2 68b4 68b5 68ba 68be 68c2 68c6 68c9
68ca 68cc 68cf 68d2 68d3 68d8 68dc 68e0
68e4 68e8 68ec 68f0 68f4 68f8 68fc 6900
6904 6908 690c 6910 6914 6918 691c 6920
6924 6928 692c 6930 693c 6940 6945 6949
694c 694d 6952 6956 695a 695d 6960 6964
6966 696a 696e 6971 6974 6978 697a 697e
6982 6985 6987 698b 698f 6993 6996 699a
699e 69a0 69a1 69a6 1 69aa 69ae 69b2
69b5 69b9 69bd 69c1 69c5 69c8 69cc 69d0
69d3 69d6 69db 69de 69e1 69e4 69e5 69ea
69eb 1 69ed 69f2 69f6 69fa 69fd 6a02
6a07 6a0a 6a0e 6a0f 6a14 6a17 6a1c 6a1d
6a22 6a25 6a2a 6a2b 6a30 6a33 6a37 6a38
6a3d 6a40 6a45 6a46 6a4b 6a4e 6a52 6a56
6a59 6a5c 6a5d 6a5f 6a60 6a65 6a66 6a6b
6a6d 6a71 6a74 6a76 6a7a 6a7c 6a88 6a8a
6a8c 6a8d 6a92 6a96 6a98 6aa4 6aa6 6aa8
6aac 6ab0 6ab4 6ab8 6abc 6ac0 6ac4 6ac8
6acc 6ad0 6ad4 6ad8 6adc 6ae0 6ae4 6ae8
6aec 6af0 6af4 6af8 6afc 6b00 6b04 6b08
6b0c 6b10 6b1c 6b20 6b25 6b29 6b2c 6b2d
6b32 6b36 6b3a 6b3d 6b40 6b44 6b46 6b4a
6b4e 6b51 6b54 6b58 6b5a 6b5e 6b62 6b65
6b67 6b6b 6b6f 6b73 6b76 6b7a 6b7e 6b80
6b81 6b86 1 6b8a 6b8e 6b92 6b95 6b99
6b9d 6ba1 6ba5 6ba8 6bac 6bb0 6bb3 6bb6
6bbb 6bbe 6bc1 6bc4 6bc5 6bca 6bcb 1
6bcd 6bd2 6bd6 6bda 6bdd 6be2 6be7 6bea
6bee 6bef 6bf4 6bf7 6bfc 6bfd 6c02 6c05
6c0a 6c0b 6c10 6c13 6c17 6c18 6c1d 6c20
6c25 6c26 6c2b 6c2e 6c32 6c36 6c39 6c3c
6c3d 6c3f 6c40 6c45 6c46 6c4b 6c4d 6c51
6c54 6c56 6c5a 6c5c 6c68 6c6a 6c6c 6c6d
6c72 6c76 6c78 6c84 6c86 6c88 6c8c 6c90
6c93 6c95 6c99 6c9d 6ca1 6ca5 6ca9 6cad
6cb1 6cb5 6cb9 6cbd 6cc1 6cc5 6cc9 6cd5
6cd9 6cde 6ce2 6ce5 6ce6 6ceb 6cef 6cf3
6cf6 6cf9 6cfd 6cff 6d03 6d07 6d0a 6d0d
6d11 6d13 6d17 6d1b 6d1e 6d20 6d24 6d28
6d2c 6d2f 6d33 6d37 6d39 6d3a 6d3f 1
6d43 6d47 6d4b 6d4e 6d52 6d56 6d5a 6d5e
6d61 6d65 6d69 6d6c 6d6f 6d74 6d77 6d7a
6d7d 6d7e 6d83 6d84 1 6d86 6d8b 6d8f
6d93 6d96 6d9b 6da0 6da3 6da7 6da8 6dad
6db0 6db5 6db6 6dbb 6dbe 6dc3 6dc4 6dc9
6dcc 6dd0 6dd1 6dd6 6dd9 6dde 6ddf 6de4
6de7 6deb 6def 6df2 6df5 6df6 6df8 6df9
6dfe 6dff 6e04 6e06 6e0a 6e0d 6e0f 6e13
6e15 6e21 6e23 6e25 6e26 6e2b 6e2f 6e31
6e3d 6e3f 6e41 6e45 6e49 6e4c 6e50 6e54
6e58 6e5c 6e60 6e6c 6e70 6e75 6e79 6e7c
6e7d 6e82 6e86 6e8a 6e8d 6e90 6e94 6e96
6e9a 6e9e 6ea1 6ea4 6ea8 6eaa 6eae 6eb2
6eb5 6eb7 6ebb 6ebf 6ec3 6ec6 6eca 6ece
6ed0 6ed1 6ed6 1 6eda 6ede 6ee2 6ee5
6ee9 6eed 6ef1 6ef5 6ef8 6efc 6f00 6f03
6f06 6f0b 6f0e 6f11 6f14 6f15 6f1a 6f1b
1 6f1d 6f22 6f26 6f2a 6f2d 6f32 6f37
6f3a 6f3e 6f3f 6f44 6f47 6f4c 6f4d 6f52
6f55 6f5a 6f5b 6f60 6f63 6f67 6f68 6f6d
6f70 6f75 6f76 6f7b 6f7e 6f82 6f86 6f89
6f8c 6f8d 6f8f 6f90 6f95 6f96 6f9b 6f9d
6fa1 6fa4 6fa6 6faa 6fac 6fb8 6fba 6fbc
6fbd 6fc2 6fc6 6fc8 6fd4 6fd6 6fda 6fde
6fe2 6fe6 6fea 6fee 6ff2 6ff6 6ffa 6ffe
7002 700e 7012 7017 701b 701e 701f 7024
7028 702c 702f 7032 7036 7038 703c 7040
7043 7046 704a 704c 7050 7054 7057 7059
705d 7061 7065 7068 706c 7070 7072 7073
7078 1 707c 7080 7084 7087 708b 708f
7093 7097 709a 709e 70a2 70a5 70a8 70ad
70b0 70b3 70b6 70b7 70bc 70bd 1 70bf
70c4 70c8 70cc 70cf 70d4 70d9 70dc 70e0
70e1 70e6 70e9 70ee 70ef 70f4 70f7 70fc
70fd 7102 7105 7109 710a 710f 7112 7117
7118 711d 7120 7124 7128 712b 712e 712f
7131 7132 7137 7138 713d 713f 7143 7146
7148 714c 714e 715a 715c 715e 715f 7164
7168 716a 7176 7178 717a 717e 7182 7185
7189 718c 7190 7194 7198 719c 71a8 71ac
71b1 71b5 71b8 71b9 71be 71c2 71c6 71c9
71cc 71d0 71d2 71d6 71da 71dd 71e0 71e4
71e6 71ea 71ee 71f1 71f3 71f7 71fb 71ff
7202 7206 720a 720c 720d 7212 1 7216
721a 721e 7221 7225 7229 722d 7231 7234
7238 723c 723f 7242 7247 724a 724d 7250
7251 7256 7257 1 7259 725e 7262 7266
7269 726e 7273 7276 727a 727b 7280 7283
7288 7289 728e 7291 7296 7297 729c 729f
72a3 72a4 72a9 72ac 72b1 72b2 72b7 72ba
72be 72c2 72c5 72c8 72c9 72cb 72cc 72d1
72d2 72d7 72d9 72dd 72e0 72e2 72e6 72e8
72f4 72f6 72f8 72f9 72fe 7302 7304 7310
7312 7316 731a 731e 7322 7326 7332 7336
733b 733f 7342 7343 7348 734c 7350 7353
7356 735a 735c 7360 7364 7367 736a 736e
7370 7374 7378 737b 737d 7381 7385 7389
738c 7390 7394 7396 7397 739c 1 73a0
73a4 73a8 73ab 73af 73b3 73b7 73bb 73be
73c2 73c6 73c9 73cc 73d1 73d4 73d7 73da
73db 73e0 73e1 1 73e3 73e8 73ec 73f0
73f3 73f8 73fd 7400 7404 7405 740a 740d
7412 7413 7418 741b 7420 7421 7426 7429
742d 742e 7433 7436 743b 743c 7441 7444
7448 744c 744f 7452 7453 7455 7456 745b
745c 7461 7463 7467 746a 746c 7470 7472
747e 7480 7482 7483 7488 748c 748e 749a
749c 74a0 74a4 74a7 74a8 74aa 74ad 74b0
74b1 74b6 74b9 74bd 74c1 74c4 74c5 74c7
74ca 74cd 74ce 74d3 1 74d6 74db 74df
74e2 74e6 74ea 74ed 74f1 74f5 74f9 74fc
7500 7503 7504 7506 7509 750c 750d 7512
7516 7519 751d 751f 7523 7526 752a 752c
7530 7534 7537 753b 753f 7542 7543 7545
7548 754b 754c 7551 7555 7558 755c 755e
7562 7565 7569 756d 7571 7575 7579 757d
7580 7581 7583 7587 758b 758e 758f 7591
7595 7599 759c 759d 759f 75a3 75a7 75aa
75ab 75ad 75b1 75b5 75b8 75b9 75bb 75bf
75c3 75c6 75c7 75c9 75cd 75d1 75d4 75d5
75d7 75db 75df 75e2 75e3 75e5 75e9 75ed
75f1 75f2 75f4 75f8 75fc 7600 7603 7604
7606 760a 760d 7611 7614 7615 761a 761e
7622 7625 7629 762d 7630 7633 7638 763b
763e 7641 7642 7647 7648 1 764a 764f
7653 7657 765a 765f 7664 7667 766b 766c
7671 7674 7679 767a 767f 7682 7687 7688
768d 7690 7694 7698 769c 76a0 76a3 76a4
76a6 76ab 76ac 76ae 76af 76b4 76b5 76ba
76bc 76c0 76c3 76c7 76cb 76ce 76d2 76d6
76d9 76dc 76e1 76e4 76e7 76ea 76eb 76f0
76f1 1 76f3 76f8 76fc 7700 7703 7708
770d 7710 7714 7715 771a 771b 7720 7722
7726 7729 772d 7731 7734 7738 773c 773f
7743 7744 7749 774d 774f 7753 7757 775a
775b 775d 7760 7763 7764 7769 776d 7771
7774 7778 777c 777f 7782 7787 778a 778d
7790 7791 7796 7797 1 7799 779e 77a2
77a6 77a9 77ae 77b3 77b6 77ba 77bb 77c0
77c3 77c8 77c9 77ce 77d1 77d6 77d7 77dc
77df 77e3 77e7 77eb 77ec 77ee 77f3 77f4
77f6 77f7 77fc 77ff 7804 7805 780a 780d
7812 7813 7818 781b 781f 7820 7825 7828
782d 782e 7833 7834 7839 783b 783f 7842
7846 784a 784d 7851 7855 7858 785b 7860
7863 7866 7869 786a 786f 7870 1 7872
7877 787b 787f 7882 7887 788c 788f 7893
7894 7899 789a 789f 78a1 78a5 78a8 78ac
78b0 78b3 78b7 78bb 78be 78bf 78c1 78c5
78c9 78ca 78cf 78d1 78d5 78d9 78dc 78e0
78e4 78e7 78e8 78ea 78ed 78f0 78f1 78f6
78fa 78fe 7902 7905 7909 790b 790f 7913
7917 791a 791e 7922 7925 7926 7928 792c
7930 7933 7934 7936 793a 793c 7940 7944
7947 794b 794e 7952 7956 795a 795e 7962
7966 796a 796e 7972 7976 797a 797e 7982
7986 798a 798e 7992 7996 799a 799e 79a2
79a6 79aa 79ae 79b2 79b6 79ba 79be 79c2
79c6 79ca 79cd 79d0 79d3 79d6 79d7 79d9
79dd 79e1 79e5 79e8 79e9 79eb 79ee 79f1
79f2 79f7 79fb 79ff 7a02 7a06 7a0a 7a0d
7a10 7a15 7a18 7a1b 7a1e 7a1f 7a24 7a25
1 7a27 7a2c 7a30 7a34 7a37 7a3c 7a41
7a44 7a48 7a49 7a4e 7a51 7a56 7a57 7a5c
7a5f 7a64 7a65 7a6a 7a6d 7a71 7a75 7a79
7a7d 7a80 7a81 7a83 7a88 7a89 7a8b 7a8c
7a91 7a94 7a99 7a9a 7a9f 7aa2 7aa7 7aa8
7aad 7ab0 7ab4 7ab5 7aba 7abd 7ac2 7ac3
7ac8 7ac9 7ace 7ad0 7ad4 7ad7 7adb 7adf
7ae2 7ae6 7aea 7aed 7af0 7af5 7af8 7afb
7afe 7aff 7b04 7b05 1 7b07 7b0c 7b10
7b14 7b17 7b1c 7b21 7b24 7b28 7b29 7b2e
7b2f 7b34 7b36 7b3a 7b3d 7b41 7b45 7b48
7b4c 7b50 7b53 7b57 7b5b 7b5c 7b61 7b65
7b67 7b6b 7b6f 7b72 7b73 7b75 7b78 7b7b
7b7c 7b81 7b85 7b89 7b8c 7b90 7b94 7b97
7b9a 7b9f 7ba2 7ba5 7ba8 7ba9 7bae 7baf
1 7bb1 7bb6 7bba 7bbe 7bc1 7bc6 7bcb
7bce 7bd2 7bd3 7bd8 7bdb 7be0 7be1 7be6
7be9 7bee 7bef 7bf4 7bf7 7bfb 7bff 7c03
7c07 7c0a 7c0b 7c0d 7c12 7c13 7c15 7c16
7c1b 7c1e 7c23 7c24 7c29 7c2c 7c31 7c32
7c37 7c3a 7c3e 7c3f 7c44 7c47 7c4c 7c4d
7c52 7c53 7c58 7c5a 7c5e 7c61 7c65 7c69
7c6c 7c70 7c74 7c77 7c7a 7c7f 7c82 7c85
7c88 7c89 7c8e 7c8f 1 7c91 7c96 7c9a
7c9e 7ca1 7ca6 7cab 7cae 7cb2 7cb3 7cb8
7cb9 7cbe 7cc0 7cc4 7cc7 7ccb 7ccf 7cd2
7cd6 7cda 7cdd 7ce1 7ce5 7ce6 7ceb 7ced
7cf1 7cf5 7cf8 7cfc 7d00 7d04 7d08 7d0c
7d10 7d14 7d18 7d1b 7d1c 7d1e 7d22 7d26
7d29 7d2a 7d2c 7d2d 7d2f 7d33 7d35 7d39
7d3d 7d40 7d42 7d46 7d49 7d4b 7d4f 7d52
7d56 7d58 7d5c 7d60 7d63 7d66 7d67 7d69
7d6c 7d71 7d72 7d77 7d7b 7d7f 7d83 7d87
7d8b 7d8f 7d93 7d97 7d9b 7d9f 7da3 7da7
7dab 7daf 7db3 7db7 7dbb 7dbf 7dc3 7dc7
7dcb 7dcf 7dd3 7dd7 7ddb 7ddf 7deb 7def
7df3 7df6 7dfa 7dfe 7e00 7e04 7e08 7e0c
7e0f 7e13 7e17 7e19 7e1a 7e1f 1 7e23
7e27 7e2b 7e2e 7e32 7e36 7e3a 7e3e 7e41
7e45 7e49 7e4c 7e4f 7e54 7e57 7e5a 7e5d
7e5e 7e63 7e64 1 7e66 7e6b 7e6f 7e73
7e76 7e7b 7e80 7e83 7e87 7e88 7e8d 7e90
7e95 7e96 7e9b 7e9e 7ea3 7ea4 7ea9 7eac
7eb0 7eb1 7eb6 7eb9 7ebe 7ebf 7ec4 7ec7
7ecb 7ecf 7ed2 7ed5 7ed6 7ed8 7ed9 7ede
7edf 7ee4 7ee6 7eea 7eed 7eef 7ef3 7ef5
7f01 7f03 7f05 7f06 7f0b 7f0f 7f11 7f1d
7f1f 7f23 7f27 7f2a 7f2e 7f31 7f35 7f38
7f39 7f3e 7f41 7f45 7f48 7f49 7f4e 7f51
1 7f54 7f59 7f5d 7f60 7f63 7f64 7f69
7f6d 7f71 7f75 7f78 7f7c 7f80 7f84 7f88
7f8c 7f90 7f94 7f98 7f9c 7fa0 7fa4 7fa5
7fa7 7fab 7fad 7fb1 7fb4 7fb6 7fba 7fbe
7fc2 7fc6 7fca 7fce 7fd2 7fd6 7fda 7fde
7fe2 7fe6 7fea 7ff6 7ffa 7ffe 8001 8005
8009 800b 800f 8013 8017 801a 801e 8022
8024 8025 802a 1 802e 8032 8036 8039
803d 8041 8045 8049 804c 8050 8054 8057
805a 805f 8062 8065 8068 8069 806e 806f
1 8071 8076 807a 807e 8081 8086 808b
808e 8092 8093 8098 809b 80a0 80a1 80a6
80a9 80ae 80af 80b4 80b7 80bb 80bc 80c1
80c4 80c9 80ca 80cf 80d2 80d6 80da 80dd
80e0 80e1 80e3 80e4 80e9 80ea 80ef 80f1
80f5 80f8 80fa 80fe 8100 810c 810e 8110
8111 8116 811a 811c 8128 812a 812c 8130
8134 8137 813b 813d 8141 8145 8148 814b
814c 814e 8151 8156 8157 815c 8160 8164
8168 816c 8170 8174 8178 817c 8180 8184
8188 818c 8190 8194 8198 819c 81a0 81a4
81a8 81ac 81b0 81b4 81b8 81bc 81c0 81c4
81d0 81d4 81d8 81db 81df 81e3 81e5 81e9
81ed 81f1 81f4 81f8 81fc 81fe 81ff 8204
1 8208 820c 8210 8213 8217 821b 821f
8223 8226 822a 822e 8231 8234 8239 823c
823f 8242 8243 8248 8249 1 824b 8250
8254 8258 825b 8260 8265 8268 826c 826d
8272 8275 827a 827b 8280 8283 8288 8289
828e 8291 8295 8296 829b 829e 82a3 82a4
82a9 82ac 82b0 82b4 82b7 82ba 82bb 82bd
82be 82c3 82c4 82c9 82cb 82cf 82d2 82d4
82d8 82da 82e6 82e8 82ea 82eb 82f0 82f4
82f6 8302 8304 8308 830c 830f 8313 8316
831a 831d 831e 8323 8326 832a 832d 832e
8333 8336 1 8339 833e 8342 8345 8348
8349 834e 8352 8356 835a 835d 8361 8365
8369 836d 8371 8375 8379 837d 8381 8385
8389 838a 838c 8390 8392 8396 8399 839b
839f 83a3 83a7 83ab 83af 83b3 83b7 83bb
83bf 83c3 83c7 83cb 83cf 83db 83df 83e3
83e6 83ea 83ee 83f0 83f4 83f8 83fc 83ff
8403 8407 8409 840a 840f 1 8413 8417
841b 841e 8422 8426 842a 842e 8431 8435
8439 843c 843f 8444 8447 844a 844d 844e
8453 8454 1 8456 845b 845f 8463 8466
846b 8470 8473 8477 8478 847d 8480 8485
8486 848b 848e 8493 8494 8499 849c 84a0
84a1 84a6 84a9 84ae 84af 84b4 84b7 84bb
84bf 84c2 84c5 84c6 84c8 84c9 84ce 84cf
84d4 84d6 84da 84dd 84df 84e3 84e5 84f1
84f3 84f5 84f6 84fb 84ff 8501 850d 850f
8511 8515 8519 851c 8520 8523 8524 8529
852c 852f 8533 8537 853a 853b 853d 8540
8543 8544 8549 1 854c 8551 8555 8559
855c 855d 855f 8562 8565 8566 856b 1
856e 8573 8577 857c 8580 8584 8588 8589
858e 8592 8596 859a 859d 859e 85a0 85a3
85a6 85a7 85ac 85b0 85b4 85b8 85bb 85bc
85be 85c2 85c4 85c8 85cb 85cf 85d3 85d6
85d7 85d9 85dc 85df 85e0 85e5 85e8 85ec
85f0 85f3 85f4 85f6 85f9 85fc 85fd 8602
1 8605 860a 860e 8612 8615 8616 8618
861b 861e 861f 8624 8627 862b 862f 8632
8633 8635 8638 863b 863c 8641 1 8644
8649 864d 8651 8655 8659 865d 8661 8665
8669 866d 8671 8675 8679 867d 8681 8685
8689 868d 8691 8695 8699 869d 86a1 86a5
86a9 86ad 86b1 86b5 86b9 86bd 86c1 86c5
86c9 86cd 86d1 86d5 86d9 86dd 86e1 86e5
86e9 86ed 86f1 86f5 86f9 86fd 8701 8705
8709 870d 8711 8715 8719 8725 8729 872e
8732 8735 8736 873b 873f 8743 8746 8749
874d 874f 8753 8757 875a 875d 8761 8763
8767 876b 876e 8770 8774 8778 877c 877f
8783 8787 8789 878a 878f 1 8793 8797
879b 879e 87a2 87a6 87aa 87ae 87b1 87b5
87b9 87bc 87bf 87c4 87c7 87ca 87cd 87ce
87d3 87d4 1 87d6 87db 87df 87e3 87e6
87eb 87f0 87f3 87f7 87f8 87fd 8800 8805
8806 880b 880e 8813 8814 8819 881c 8820
8821 8826 8829 882e 882f 8834 8837 883b
883f 8842 8845 8846 8848 8849 884e 884f
8854 8856 885a 885d 885f 8863 8865 8871
8873 8875 8876 887b 887f 8881 888d 888f
8893 8895 8899 889d 88a0 88a1 88a3 88a6
88a9 88aa 88af 88b3 88b7 88bb 88bf 88c3
88c7 88cb 88cf 88d3 88d7 88db 88df 88e3
88e7 88eb 88ef 88f3 88f7 88fb 88ff 8903
8907 890b 890f 8913 8917 891b 891f 8923
8927 892b 892f 8933 8937 893b 893f 8943
8947 894b 894f 8953 8957 895b 895f 8963
8967 896b 896f 8973 8977 897b 897f 898b
898f 8994 8998 899b 899c 89a1 89a5 89a9
89ac 89af 89b3 89b5 89b9 89bd 89c0 89c3
89c7 89c9 89cd 89d1 89d4 89d6 89da 89de
89e2 89e5 89e9 89ed 89ef 89f0 89f5 1
89f9 89fd 8a01 8a04 8a08 8a0c 8a10 8a14
8a17 8a1b 8a1f 8a22 8a25 8a2a 8a2d 8a30
8a33 8a34 8a39 8a3a 1 8a3c 8a41 8a45
8a49 8a4c 8a51 8a56 8a59 8a5d 8a5e 8a63
8a66 8a6b 8a6c 8a71 8a74 8a79 8a7a 8a7f
8a82 8a86 8a87 8a8c 8a8f 8a94 8a95 8a9a
8a9d 8aa1 8aa5 8aa8 8aab 8aac 8aae 8aaf
8ab4 8ab5 8aba 8abc 8ac0 8ac3 8ac5 8ac9
8acb 8ad7 8ad9 8adb 8adc 8ae1 8ae5 8ae7
8af3 8af5 8af7 8afb 8aff 8b03 8b07 8b0b
8b0f 8b13 8b17 8b1b 8b1f 8b23 8b27 8b2b
8b2f 8b33 8b37 8b3b 8b3f 8b43 8b47 8b4b
8b4f 8b53 8b57 8b5b 8b5f 8b63 8b67 8b6b
8b6f 8b73 8b77 8b7b 8b7f 8b83 8b87 8b8b
8b8f 8b93 8b97 8b9b 8b9f 8ba3 8ba7 8bab
8baf 8bb3 8bb7 8bbb 8bbf 8bc3 8bc7 8bcb
8bd7 8bdb 8be0 8be4 8be7 8be8 8bed 8bf1
8bf5 8bf8 8bfb 8bff 8c01 8c05 8c09 8c0c
8c0f 8c13 8c15 8c19 8c1d 8c20 8c22 8c26
8c2a 8c2e 8c31 8c35 8c39 8c3b 8c3c 8c41
1 8c45 8c49 8c4d 8c50 8c54 8c58 8c5c
8c60 8c63 8c67 8c6b 8c6e 8c71 8c76 8c79
8c7c 8c7f 8c80 8c85 8c86 1 8c88 8c8d
8c91 8c95 8c98 8c9d 8ca2 8ca5 8ca9 8caa
8caf 8cb2 8cb7 8cb8 8cbd 8cc0 8cc5 8cc6
8ccb 8cce 8cd2 8cd3 8cd8 8cdb 8ce0 8ce1
8ce6 8ce9 8ced 8cf1 8cf4 8cf7 8cf8 8cfa
8cfb 8d00 8d01 8d06 8d08 8d0c 8d0f 8d11
8d15 8d17 8d23 8d25 8d27 8d28 8d2d 8d31
8d33 8d3f 8d41 8d43 8d47 8d4b 8d4e 8d50
8d54 8d58 8d5c 8d60 8d64 8d68 8d6c 8d70
8d74 8d78 8d7c 8d80 8d84 8d88 8d8c 8d90
8d94 8d98 8d9c 8da0 8da4 8da8 8dac 8db0
8db4 8db8 8dbc 8dc0 8dc4 8dc8 8dcc 8dd0
8dd4 8dd8 8ddc 8de0 8de4 8de8 8dec 8df0
8df4 8df8 8dfc 8e00 8e04 8e08 8e0c 8e10
8e14 8e18 8e1c 8e20 8e2c 8e30 8e35 8e39
8e3c 8e3d 8e42 8e46 8e4a 8e4d 8e50 8e54
8e56 8e5a 8e5e 8e61 8e64 8e68 8e6a 8e6e
8e72 8e75 8e77 8e7b 8e7f 8e83 8e86 8e8a
8e8e 8e90 8e91 8e96 1 8e9a 8e9e 8ea2
8ea5 8ea9 8ead 8eb1 8eb5 8eb8 8ebc 8ec0
8ec3 8ec6 8ecb 8ece 8ed1 8ed4 8ed5 8eda
8edb 1 8edd 8ee2 8ee6 8eea 8eed 8ef2
8ef7 8efa 8efe 8eff 8f04 8f07 8f0c 8f0d
8f12 8f15 8f1a 8f1b 8f20 8f23 8f27 8f28
8f2d 8f30 8f35 8f36 8f3b 8f3e 8f42 8f46
8f49 8f4c 8f4d 8f4f 8f50 8f55 8f56 8f5b
8f5d 8f61 8f64 8f66 8f6a 8f6c 8f78 8f7a
8f7c 8f7d 8f82 8f86 8f88 8f94 8f96 8f98
8f9c 8fa0 8fa3 8fa7 8faa 8fae 8fb0 8fb4
8fb7 8fbb 8fbf 8fc3 8fc6 8fcb 8fcc 8fd1
8fd4 8fd7 8fd8 8fda 8fde 8fe2 8fe7 8feb
8fef 8ff3 8ff7 8ffb 8fff 9003 9007 900b
9017 901b 901f 9022 9026 902a 902c 9030
9034 9038 903b 903f 9043 9045 9046 904b
1 904f 9053 9057 905a 905e 9062 9066
906a 906d 9071 9075 9078 907b 9080 9083
9086 9089 908a 908f 9090 1 9092 9097
909b 909f 90a2 90a7 90ac 90af 90b3 90b4
90b9 90bc 90c1 90c2 90c7 90ca 90cf 90d0
90d5 90d8 90dc 90dd 90e2 90e5 90ea 90eb
90f0 90f3 90f7 90fb 90fe 9101 9102 9104
9105 910a 910b 9110 9112 9116 9119 911b
911f 9121 912d 912f 9131 9132 9137 913b
913d 9149 914b 914f 9152 9156 915a 915d
9161 9165 9168 916c 9170 9173 9176 9177
917c 917f 9183 9187 918a 918b 918d 9190
9193 9194 9199 1 919c 91a1 91a5 91a9
91ad 91b1 91b5 91b9 91bd 91c1 91c5 91c9
91cd 91d1 91d5 91d9 91dd 91e1 91e5 91e9
91ed 91f1 91f5 91f9 91fd 9201 9205 9209
920d 9211 9215 9219 921d 9221 9225 9229
922d 9231 9235 9239 923d 9241 9245 9249
924d 9251 9255 9259 925d 9261 9265 9269
926d 9271 9275 9281 9285 9289 928c 9290
9294 9296 929a 929e 92a2 92a5 92a9 92ad
92af 92b0 92b5 1 92b9 92bd 92c1 92c4
92c8 92cc 92d0 92d4 92d7 92db 92df 92e2
92e5 92ea 92ed 92f0 92f3 92f4 92f9 92fa
1 92fc 9301 9305 9309 930c 9311 9316
9319 931d 931e 9323 9326 932b 932c 9331
9334 9339 933a 933f 9342 9346 9347 934c
934f 9354 9355 935a 935d 9361 9365 9368
936b 936c 936e 936f 9374 9375 937a 937c
9380 9383 9385 9389 938b 9397 9399 939b
939c 93a1 93a5 93a7 93b3 93b5 93b7 93bb
93be 93c2 93c6 93ca 93ce 93d2 93d7 93db
93dc 93e0 93e4 93e8 93ec 93f0 93f4 93f8
93fc 9400 9404 9408 940c 9410 9414 9418
941c 9420 9424 9428 942c 9430 9434 9438
943d 9442 9446 944a 944e 9453 9457 945b
945e 9461 9465 9467 946b 946f 9472 9475
9479 947b 947f 9483 9486 948a 948e 9491
9495 9498 949c 949f 94a0 94a5 94a8 94ac
94b2 94b4 94b8 94bb 94bf 94c3 94c6 94c7
94c9 94cc 94cf 94d0 94d5 94d9 94dd 94e1
94e4 94e5 94e7 94eb 94ef 94f3 94f7 9503
9507 950c 9510 9513 9514 9519 951d 9521
9524 9527 952b 952d 9531 9535 9538 953b
953f 9541 9545 9549 954c 954e 9552 9556
955a 955d 9561 9565 9567 9568 956d 1
9571 9575 9579 957c 9580 9584 9588 958c
958f 9593 9597 959a 959d 95a2 95a5 95a8
95ab 95ac 95b1 95b2 1 95b4 95b9 95bd
95c1 95c4 95c9 95ce 95d1 95d5 95d6 95db
95de 95e3 95e4 95e9 95ec 95f1 95f2 95f7
95fa 95fe 95ff 9604 9607 960c 960d 9612
9615 9619 961d 9620 9623 9624 9626 9627
962c 962d 9632 9634 9638 963b 963d 9641
9643 964f 9651 9653 9654 9659 965d 965f
966b 966d 9671 9675 9679 9685 9689 968e
9692 9695 9696 969b 969f 96a3 96a6 96a9
96ad 96af 96b3 96b7 96ba 96bd 96c1 96c3
96c7 96cb 96ce 96d0 96d4 96d8 96dc 96df
96e3 96e7 96e9 96ea 96ef 1 96f3 96f7
96fb 96fe 9702 9706 970a 970e 9711 9715
9719 971c 971f 9724 9727 972a 972d 972e
9733 9734 1 9736 973b 973f 9743 9746
974b 9750 9753 9757 9758 975d 9760 9765
9766 976b 976e 9773 9774 9779 977c 9780
9781 9786 9789 978e 978f 9794 9797 979b
979f 97a2 97a5 97a6 97a8 97a9 97ae 97af
97b4 97b6 97ba 97bd 97bf 97c3 97c5 97d1
97d3 97d5 97d6 97db 97df 97e1 97ed 97ef
97f3 97f7 97fb 9807 980b 9810 9814 9817
9818 981d 9821 9825 9828 982b 982f 9831
9835 9839 983c 983f 9843 9845 9849 984d
9850 9852 9856 985a 985e 9861 9865 9869
986b 986c 9871 1 9875 9879 987d 9880
9884 9888 988c 9890 9893 9897 989b 989e
98a1 98a6 98a9 98ac 98af 98b0 98b5 98b6
1 98b8 98bd 98c1 98c5 98c8 98cd 98d2
98d5 98d9 98da 98df 98e2 98e7 98e8 98ed
98f0 98f5 98f6 98fb 98fe 9902 9903 9908
990b 9910 9911 9916 9919 991d 9921 9924
9927 9928 992a 992b 9930 9931 9936 9938
993c 993f 9941 9945 9947 9953 9955 9957
9958 995d 9961 9963 996f 9971 9975 9979
997d 9981 9985 9991 9995 999a 999e 99a1
99a2 99a7 99ab 99af 99b2 99b5 99b9 99bb
99bf 99c3 99c6 99c9 99cd 99cf 99d3 99d7
99da 99dc 99e0 99e4 99e8 99eb 99ef 99f3
99f5 99f6 99fb 1 99ff 9a03 9a07 9a0a
9a0e 9a12 9a16 9a1a 9a1d 9a21 9a25 9a28
9a2b 9a30 9a33 9a36 9a39 9a3a 9a3f 9a40
1 9a42 9a47 9a4b 9a4f 9a52 9a57 9a5c
9a5f 9a63 9a64 9a69 9a6c 9a71 9a72 9a77
9a7a 9a7f 9a80 9a85 9a88 9a8c 9a8d 9a92
9a95 9a9a 9a9b 9aa0 9aa3 9aa7 9aab 9aae
9ab1 9ab2 9ab4 9ab5 9aba 9abb 9ac0 9ac2
9ac6 9ac9 9acb 9acf 9ad1 9add 9adf 9ae1
9ae2 9ae7 9aeb 9aed 9af9 9afb 9aff 9b03
9b07 9b13 9b17 9b1c 9b20 9b23 9b24 9b29
9b2d 9b31 9b34 9b37 9b3b 9b3d 9b41 9b45
9b48 9b4b 9b4f 9b51 9b55 9b59 9b5c 9b5e
9b62 9b66 9b6a 9b6d 9b71 9b75 9b77 9b78
9b7d 1 9b81 9b85 9b89 9b8c 9b90 9b94
9b98 9b9c 9b9f 9ba3 9ba7 9baa 9bad 9bb2
9bb5 9bb8 9bbb 9bbc 9bc1 9bc2 1 9bc4
9bc9 9bcd 9bd1 9bd4 9bd9 9bde 9be1 9be5
9be6 9beb 9bee 9bf3 9bf4 9bf9 9bfc 9c01
9c02 9c07 9c0a 9c0e 9c0f 9c14 9c17 9c1c
9c1d 9c22 9c25 9c29 9c2d 9c30 9c33 9c34
9c36 9c37 9c3c 9c3d 9c42 9c44 9c48 9c4b
9c4d 9c51 9c53 9c5f 9c61 9c63 9c64 9c69
9c6d 9c6f 9c7b 9c7d 9c81 9c85 9c89 9c95
9c99 9c9e 9ca2 9ca5 9ca6 9cab 9caf 9cb3
9cb6 9cb9 9cbd 9cbf 9cc3 9cc7 9cca 9ccd
9cd1 9cd3 9cd7 9cdb 9cde 9ce0 9ce4 9ce8
9cec 9cef 9cf3 9cf7 9cf9 9cfa 9cff 1
9d03 9d07 9d0b 9d0e 9d12 9d16 9d1a 9d1e
9d21 9d25 9d29 9d2c 9d2f 9d34 9d37 9d3a
9d3d 9d3e 9d43 9d44 1 9d46 9d4b 9d4f
9d53 9d56 9d5b 9d60 9d63 9d67 9d68 9d6d
9d70 9d75 9d76 9d7b 9d7e 9d83 9d84 9d89
9d8c 9d90 9d91 9d96 9d99 9d9e 9d9f 9da4
9da7 9dab 9daf 9db2 9db5 9db6 9db8 9db9
9dbe 9dbf 9dc4 9dc6 9dca 9dcd 9dcf 9dd3
9dd5 9de1 9de3 9de5 9de6 9deb 9def 9df1
9dfd 9dff 9e01 9e05 9e09 9e0d 9e11 9e15
9e19 9e1d 9e21 9e25 9e31 9e35 9e3a 9e3e
9e41 9e42 9e47 9e4b 9e4f 9e52 9e55 9e59
9e5b 9e5f 9e63 9e66 9e69 9e6d 9e6f 9e73
9e77 9e7a 9e7c 9e80 9e84 9e88 9e8b 9e8f
9e93 9e95 9e96 9e9b 1 9e9f 9ea3 9ea7
9eaa 9eae 9eb2 9eb6 9eba 9ebd 9ec1 9ec5
9ec8 9ecb 9ed0 9ed3 9ed6 9ed9 9eda 9edf
9ee0 1 9ee2 9ee7 9eeb 9eef 9ef2 9ef7
9efc 9eff 9f03 9f04 9f09 9f0c 9f11 9f12
9f17 9f1a 9f1f 9f20 9f25 9f28 9f2c 9f2d
9f32 9f35 9f3a 9f3b 9f40 9f43 9f47 9f4b
9f4e 9f51 9f52 9f54 9f55 9f5a 9f5b 9f60
9f62 9f66 9f69 9f6b 9f6f 9f71 9f7d 9f7f
9f81 9f82 9f87 9f8b 9f8d 9f99 9f9b 9f9f
9fa3 9fa7 9fab 9faf 9fb3 9fb7 9fbb 9fbf
9fc3 9fcf 9fd3 9fd7 9fda 9fde 9fe2 9fe4
9fe8 9fec 9ff0 9ff3 9ff7 9ffb 9ffd 9ffe
a003 1 a007 a00b a00f a012 a016 a01a
a01e a022 a025 a029 a02d a030 a033 a038
a03b a03e a041 a042 a047 a048 1 a04a
a04f a053 a057 a05a a05f a064 a067 a06b
a06c a071 a074 a079 a07a a07f a082 a087
a088 a08d a090 a094 a095 a09a a09d a0a2
a0a3 a0a8 a0ab a0af a0b3 a0b6 a0b9 a0ba
a0bc a0bd a0c2 a0c3 a0c8 a0ca a0ce a0d1
a0d3 a0d7 a0d9 a0e5 a0e7 a0e9 a0ea a0ef
a0f3 a0f5 a101 a103 a107 a10b a10e a112
a115 a119 a11c a11d a122 a125 a129 a12d
a130 a131 a133 a136 a139 a13a a13f a143
a147 a14b a14d a151 a154 a158 a15c a160
a164 a168 a16c a170 a174 a178 a17c a180
a184 a190 a194 a198 a19b a19f a1a3 a1a5
a1a9 a1ad a1b1 a1b4 a1b8 a1bc a1be a1bf
a1c4 1 a1c8 a1cc a1d0 a1d3 a1d7 a1db
a1df a1e3 a1e6 a1ea a1ee a1f1 a1f4 a1f9
a1fc a1ff a202 a203 a208 a209 1 a20b
a210 a214 a218 a21b a220 a225 a228 a22c
a22d a232 a235 a23a a23b a240 a243 a248
a249 a24e a251 a255 a256 a25b a25e a263
a264 a269 a26c a270 a274 a277 a27a a27b
a27d a27e a283 a284 a289 a28b a28f a292
a294 a298 a29a a2a6 a2a8 a2aa a2ab a2b0
a2b4 a2b6 a2c2 a2c4 a2c8 a2cc a2cf a2d3
a2d6 a2da a2dd a2de a2e3 a2e6 a2ea a2ee
a2f2 a2f5 a2f6 a2f8 a2fc a300 a303 a304
a306 a309 a30d a310 a314 a317 a318 a31d
a321 a324 a328 a32c a330 a334 a338 a33c
a340 a344 a348 a354 a358 a35d a361 a364
a365 a36a a36e a372 a375 a378 a37c a37e
a382 a386 a389 a38c a390 a392 a396 a39a
a39d a39f a3a3 a3a7 a3ab a3ae a3b2 a3b6
a3b8 a3b9 a3be 1 a3c2 a3c6 a3ca a3cd
a3d1 a3d5 a3d9 a3dd a3e0 a3e4 a3e8 a3eb
a3ee a3f3 a3f6 a3f9 a3fc a3fd a402 a403
1 a405 a40a a40e a412 a415 a41a a41f
a422 a426 a427 a42c a42f a434 a435 a43a
a43d a442 a443 a448 a44b a44f a450 a455
a458 a45d a45e a463 a466 a46a a46e a471
a474 a475 a477 a478 a47d a47e a483 a485
a489 a48c a48e a492 a494 a4a0 a4a2 a4a4
a4a5 a4aa a4ae a4b0 a4bc a4be a4c2 a4c4
a4c8 a4cc a4cf a4d0 a4d2 a4d6 a4d9 a4dd
a4e0 a4e1 a4e6 a4ea a4ee a4f1 a4f5 a4f9
a4fc a4fd a4ff a503 a507 a50a a50e a50f
a511 a515 a519 a51d a521 a525 a529 a52d
a531 a535 a539 a53d a541 a545 a549 a555
a559 a55e a562 a565 a566 a56b a56f a573
a576 a579 a57d a57f a583 a587 a58a a58d
a591 a593 a597 a59b a59e a5a0 a5a4 a5a8
a5ac a5af a5b3 a5b7 a5b9 a5ba a5bf 1
a5c3 a5c7 a5cb a5ce a5d2 a5d6 a5da a5de
a5e1 a5e5 a5e9 a5ec a5ef a5f4 a5f7 a5fa
a5fd a5fe a603 a604 1 a606 a60b a60f
a613 a616 a61b a620 a623 a627 a628 a62d
a630 a635 a636 a63b a63e a643 a644 a649
a64c a650 a651 a656 a659 a65e a65f a664
a667 a66b a66f a672 a675 a676 a678 a679
a67e a67f a684 a686 a68a a68d a68f a693
a695 a6a1 a6a3 a6a5 a6a6 a6ab a6af a6b1
a6bd a6bf a6c1 a6c5 a6c9 a6cd a6d1 a6d5
a6d9 a6dd a6e1 a6e5 a6e9 a6ed a6f1 a6f5
a6f9 a6fd a701 a705 a711 a715 a71a a71e
a721 a722 a727 a72b a72f a732 a735 a739
a73b a73f a743 a746 a749 a74d a74f a753
a757 a75a a75c a760 a764 a768 a76b a76f
a773 a775 a776 a77b 1 a77f a783 a787
a78a a78e a792 a796 a79a a79d a7a1 a7a5
a7a8 a7ab a7b0 a7b3 a7b6 a7b9 a7ba a7bf
a7c0 1 a7c2 a7c7 a7cb a7cf a7d2 a7d7
a7dc a7df a7e3 a7e4 a7e9 a7ec a7f1 a7f2
a7f7 a7fa a7ff a800 a805 a808 a80c a80d
a812 a815 a81a a81b a820 a823 a827 a82b
a82e a831 a832 a834 a835 a83a a83b a840
a842 a846 a849 a84b a84f a851 a85d a85f
a861 a862 a867 a86b a86d a879 a87b a87d
a881 a885 a888 a88c a890 a893 a897 a89a
a89e a8a1 a8a2 a8a7 a8aa a8ae a8b2 a8b5
a8b6 a8b8 a8bb a8be a8bf a8c4 1 a8c7
a8cc a8d0 a8d4 a8d8 a8e4 a8e8 a8ed a8f1
a8f4 a8f5 a8fa a8fe a902 a905 a908 a90c
a90e a912 a916 a919 a91c a920 a922 a926
a92a a92d a92f a933 a937 a93b a93e a942
a946 a948 a949 a94e 1 a952 a956 a95a
a95d a961 a965 a969 a96d a970 a974 a978
a97b a97e a983 a986 a989 a98c a98d a992
a993 1 a995 a99a a99e a9a2 a9a5 a9aa
a9af a9b2 a9b6 a9b7 a9bc a9bf a9c4 a9c5
a9ca a9cd a9d2 a9d3 a9d8 a9db a9df a9e0
a9e5 a9e8 a9ed a9ee a9f3 a9f6 a9fa a9fe
aa01 aa04 aa05 aa07 aa08 aa0d aa0e aa13
aa15 aa19 aa1c aa1e aa22 aa24 aa30 aa32
aa34 aa35 aa3a aa3e aa40 aa4c aa4e aa50
aa54 aa57 aa59 aa5d aa61 aa65 aa69 aa6d
aa71 aa75 aa79 aa7d aa81 aa85 aa89 aa8d
aa91 aa95 aa99 aa9d aaa1 aaa5 aaa9 aaad
aab1 aab5 aab9 aabd aac1 aac5 aac9 aacd
aad1 aad5 aad9 aadd aae1 aaed aaf1 aaf5
aaf8 aafc ab00 ab02 ab06 ab0a ab0e ab11
ab15 ab19 ab1b ab1c ab21 1 ab25 ab29
ab2d ab30 ab34 ab38 ab3c ab40 ab43 ab47
ab4b ab4e ab51 ab56 ab59 ab5c ab5f ab60
ab65 ab66 1 ab68 ab6d ab71 ab75 ab78
ab7d ab82 ab85 ab89 ab8a ab8f ab92 ab97
ab98 ab9d aba0 aba5 aba6 abab abae abb2
abb3 abb8 abbb abc0 abc1 abc6 abc9 abcd
abd1 abd4 abd7 abd8 abda abdb abe0 abe1
abe6 abe8 abec abef abf1 abf5 abf7 ac03
ac05 ac07 ac08 ac0d ac11 ac13 ac1f ac21
ac25 ac29 ac2c ac30 ac33 ac37 ac3a ac3b
ac40 ac43 ac47 ac4a ac4b ac50 ac53 1
ac56 ac5b ac5f ac63 ac66 ac6a ac6e ac71
ac72 ac74 ac78 ac7c ac7f ac83 ac84 ac86
ac8a ac8e ac92 ac96 aca2 aca6 acab acaf
acb2 acb3 acb8 acbc acc0 acc3 acc6 acca
accc acd0 acd4 acd7 acda acde ace0 ace4
ace8 aceb aced acf1 acf5 acf9 acfc ad00
ad04 ad06 ad07 ad0c 1 ad10 ad14 ad18
ad1b ad1f ad23 ad27 ad2b ad2e ad32 ad36
ad39 ad3c ad41 ad44 ad47 ad4a ad4b ad50
ad51 1 ad53 ad58 ad5c ad60 ad63 ad68
ad6d ad70 ad74 ad75 ad7a ad7d ad82 ad83
ad88 ad8b ad90 ad91 ad96 ad99 ad9d ad9e
ada3 ada6 adab adac adb1 adb4 adb8 adbc
adbf adc2 adc3 adc5 adc6 adcb adcc add1
add3 add7 adda addc ade0 ade2 adee adf0
adf2 adf3 adf8 adfc adfe ae0a ae0c ae10
ae14 ae18 ae1c ae20 ae24 ae28 ae2c ae30
ae34 ae38 ae44 ae48 ae4d ae51 ae54 ae55
ae5a ae5e ae62 ae65 ae68 ae6c ae6e ae72
ae76 ae79 ae7c ae80 ae82 ae86 ae8a ae8d
ae8f ae93 ae97 ae9b ae9e aea2 aea6 aea8
aea9 aeae 1 aeb2 aeb6 aeba aebd aec1
aec5 aec9 aecd aed0 aed4 aed8 aedb aede
aee3 aee6 aee9 aeec aeed aef2 aef3 1
aef5 aefa aefe af02 af05 af0a af0f af12
af16 af17 af1c af1f af24 af25 af2a af2d
af32 af33 af38 af3b af3f af40 af45 af48
af4d af4e af53 af56 af5a af5e af61 af64
af65 af67 af68 af6d af6e af73 af75 af79
af7c af7e af82 af84 af90 af92 af94 af95
af9a af9e afa0 afac afae afb2 afb6 afba
afbe afc2 afc6 afca afce afda afde afe3
afe7 afea afeb aff0 aff4 aff8 affb affe
b002 b004 b008 b00c b00f b012 b016 b018
b01c b020 b023 b025 b029 b02d b031 b034
b038 b03c b03e b03f b044 1 b048 b04c
b050 b053 b057 b05b b05f b063 b066 b06a
b06e b071 b074 b079 b07c b07f b082 b083
b088 b089 1 b08b b090 b094 b098 b09b
b0a0 b0a5 b0a8 b0ac b0ad b0b2 b0b5 b0ba
b0bb b0c0 b0c3 b0c8 b0c9 b0ce b0d1 b0d5
b0d6 b0db b0de b0e3 b0e4 b0e9 b0ec b0f0
b0f4 b0f7 b0fa b0fb b0fd b0fe b103 b104
b109 b10b b10f b112 b114 b118 b11a b126
b128 b12a b12b b130 b134 b136 b142 b144
b146 b14a b14e b151 b155 b159 b15c b15d
b15f b163 b167 b16a b16e b16f b171 b175
b179 b17d b181 b185 b189 b18d b191 b195
b1a1 b1a5 b1aa b1ae b1b1 b1b2 b1b7 b1bb
b1bf b1c2 b1c5 b1c9 b1cb b1cf b1d3 b1d6
b1d9 b1dd b1df b1e3 b1e7 b1ea b1ec b1f0
b1f4 b1f8 b1fb b1ff b203 b205 b206 b20b
1 b20f b213 b217 b21a b21e b222 b226
b22a b22d b231 b235 b238 b23b b240 b243
b246 b249 b24a b24f b250 1 b252 b257
b25b b25f b262 b267 b26c b26f b273 b274
b279 b27c b281 b282 b287 b28a b28f b290
b295 b298 b29c b29d b2a2 b2a5 b2aa b2ab
b2b0 b2b3 b2b7 b2bb b2be b2c1 b2c2 b2c4
b2c5 b2ca b2cb b2d0 b2d2 b2d6 b2d9 b2db
b2df b2e1 b2ed b2ef b2f1 b2f2 b2f7 b2fb
b2fd b309 b30b b30f b313 b317 b31b b31f
b323 b327 b32b b32f b333 b337 b343 b347
b34c b350 b353 b354 b359 b35d b361 b364
b367 b36b b36d b371 b375 b378 b37b b37f
b381 b385 b389 b38c b38e b392 b396 b39a
b39d b3a1 b3a5 b3a7 b3a8 b3ad 1 b3b1
b3b5 b3b9 b3bc b3c0 b3c4 b3c8 b3cc b3cf
b3d3 b3d7 b3da b3dd b3e2 b3e5 b3e8 b3eb
b3ec b3f1 b3f2 1 b3f4 b3f9 b3fd b401
b404 b409 b40e b411 b415 b416 b41b b41e
b423 b424 b429 b42c b431 b432 b437 b43a
b43e b43f b444 b447 b44c b44d b452 b455
b459 b45d b460 b463 b464 b466 b467 b46c
b46d b472 b474 b478 b47b b47d b481 b483
b48f b491 b493 b494 b499 b49d b49f b4ab
b4ad b4af b4b3 b4b7 b4ba b4bc b4c0 b4c4
b4c7 b4cb b4cf b4d2 b4d3 b4d5 b4d8 b4db
b4dc b4e1 b4e5 b4e9 b4ec b4ed b4ef b4f2
b4f5 b4f6 b4fb b4fe b502 b505 b506 b50b
b50e 1 b511 b516 b51a b51e b522 b526
b52a b52e b532 b53e b542 b546 b549 b54d
b551 b553 b557 b55b b55f b562 b566 b56a
b56c b56d b572 1 b576 b57a b57e b581
b585 b589 b58d b591 b594 b598 b59c b59f
b5a2 b5a7 b5aa b5ad b5b0 b5b1 b5b6 b5b7
1 b5b9 b5be b5c2 b5c6 b5c9 b5ce b5d3
b5d6 b5da b5db b5e0 b5e3 b5e8 b5e9 b5ee
b5f1 b5f6 b5f7 b5fc b5ff b603 b604 b609
b60c b611 b612 b617 b61a b61e b622 b625
b628 b629 b62b b62c b631 b632 b637 b639
b63d b640 b642 b646 b648 b654 b656 b658
b659 b65e b662 b664 b670 b672 b676 b67a
b67d b681 b684 b688 b68b b68c b691 b694
b698 b69c b69f b6a0 b6a2 b6a5 b6a8 b6a9
b6ae 1 b6b1 b6b6 b6ba b6be b6c2 b6c6
b6ca b6d6 b6da b6df b6e3 b6e6 b6e7 b6ec
b6f0 b6f4 b6f7 b6fa b6fe b700 b704 b708
b70b b70e b712 b714 b718 b71c b71f b721
b725 b729 b72d b730 b734 b738 b73a b73b
b740 1 b744 b748 b74c b74f b753 b757
b75b b75f b762 b766 b76a b76d b770 b775
b778 b77b b77e b77f b784 b785 1 b787
b78c b790 b794 b797 b79c b7a1 b7a4 b7a8
b7a9 b7ae b7b1 b7b6 b7b7 b7bc b7bf b7c4
b7c5 b7ca b7cd b7d1 b7d2 b7d7 b7da b7df
b7e0 b7e5 b7e8 b7ec b7f0 b7f3 b7f6 b7f7
b7f9 b7fa b7ff b800 b805 b807 b80b b80e
b810 b814 b816 b822 b824 b826 b827 b82c
b830 b832 b83e b840 b844 b846 b84a b84e
b851 b855 b858 b85c b85f b860 b865 b868
b86c b870 b873 b874 b876 b879 b87c b87d
b882 1 b885 b88a b88e b892 b897 b899
b89d b8a1 b8a4 b8a8 b8ac b8ae b8b2 b8b6
b8ba b8bd b8c1 b8c5 b8c7 b8c8 b8cd 1
b8d1 b8d5 b8d9 b8dc b8e0 b8e4 b8e8 b8ec
b8ef b8f3 b8f7 b8fa b8fd b902 b905 b908
b90b b90c b911 b912 1 b914 b919 b91d
b921 b924 b929 b92e b931 b935 b936 b93b
b93e b943 b944 b949 b94c b951 b952 b957
b95a b95e b95f b964 b967 b96c b96d b972
b975 b979 b97d b980 b983 b984 b986 b987
b98c b98d b992 b994 b998 b99b b99d b9a1
b9a3 b9af b9b1 b9b3 b9b4 b9b9 b9bd b9bf
b9cb b9cd b9d1 b9d5 b9d8 b9dc b9e0 b9e3
b9e6 b9eb b9ee b9f1 b9f4 b9f5 b9fa b9fb
1 b9fd ba02 ba06 ba0a ba0d ba12 ba17
ba1a ba1e ba1f ba24 ba27 ba2c ba2d ba32
ba35 ba3a ba3b ba40 ba43 ba47 ba4b ba4f
ba53 ba56 ba57 ba59 ba5e ba5f ba61 ba62
ba67 ba6a ba6f ba70 ba75 ba78 ba7d ba7e
ba83 ba86 ba8a ba8b ba90 ba93 ba98 ba99
ba9e ba9f baa4 baa6 baaa baad bab1 bab5
bab8 babc bac0 bac3 bac6 bacb bace bad1
bad4 bad5 bada badb 1 badd bae2 bae6
baea baed baf2 baf7 bafa bafe baff bb04
bb05 bb0a bb0c bb10 bb13 bb17 bb1b bb1e
bb22 bb26 bb29 bb2d bb31 bb32 bb37 bb39
bb3d bb41 bb44 bb46 bb4a bb4d bb51 bb54
bb57 bb58 bb5d bb61 bb65 bb69 bb6c bb70
bb74 bb78 bb7c bb80 bb84 bb88 bb8c bb90
bb94 bb98 bb99 bb9b bb9f bba3 bba7 bbab
bbaf bbb3 bbbf bbc3 bbc8 bbcc bbcf bbd0
bbd5 bbd9 bbdd bbe0 bbe3 bbe7 bbe9 bbed
bbf1 bbf4 bbf7 bbfb bbfd bc01 bc05 bc08
bc0a bc0e bc12 bc16 bc19 bc1d bc21 bc23
bc24 bc29 1 bc2d bc31 bc35 bc38 bc3c
bc40 bc44 bc48 bc4b bc4f bc53 bc56 bc59
bc5e bc61 bc64 bc67 bc68 bc6d bc6e 1
bc70 bc75 bc79 bc7d bc80 bc85 bc8a bc8d
bc91 bc92 bc97 bc9a bc9f bca0 bca5 bca8
bcad bcae bcb3 bcb6 bcba bcbb bcc0 bcc3
bcc8 bcc9 bcce bcd1 bcd5 bcd9 bcdc bcdf
bce0 bce2 bce3 bce8 bce9 bcee bcf0 bcf4
bcf7 bcf9 bcfd bcff bd0b bd0d bd0f bd10
bd15 bd19 bd1b bd27 bd29 bd2d bd32 bd36
bd3a bd3d bd40 bd41 bd46 bd4a bd4e bd52
bd56 bd5a bd5e bd62 bd66 bd6a bd6e bd72
bd76 bd7a bd7e bd82 bd86 bd8a bd8e bd92
bd96 bd9a bd9e bda2 bda6 bdaa bdae bdb2
bdb6 bdba bdbe bdc1 bdc2 bdc4 bdc8 bdcc
bdcf bdd0 bdd2 bdd5 bdd8 bddb bddc bde1
bde4 bde8 bdec bdef bdf0 bdf2 bdf6 bdf9
bdfd be00 be01 be03 be04 be09 be0c be10
be14 be17 be18 be1a be1e be21 be25 be28
be29 be2b be2c be31 1 be34 be39 1
be3c be41 be44 be48 be4c be4f be50 be52
be55 be58 be59 be5e be62 be66 be6a be6e
be72 be76 be7a be7e be82 be86 be8a be8e
be92 be96 be9a be9e bea2 bea6 beaa beae
beb2 beb6 beba bebe bec2 bec6 beca bece
bed2 bed6 beda bede bee2 bee6 beea beee
bef2 befe bf02 bf07 bf0b bf0e bf0f bf14
bf18 bf1c bf1f bf22 bf26 bf28 bf2c bf30
bf33 bf36 bf3a bf3c bf40 bf44 bf47 bf49
bf4d bf51 bf55 bf58 bf5c bf60 bf62 bf63
bf68 1 bf6c bf70 bf74 bf77 bf7b bf7f
bf83 bf87 bf8a bf8e bf92 bf95 bf98 bf9d
bfa0 bfa3 bfa6 bfa7 bfac bfad 1 bfaf
bfb4 bfb8 bfbc bfbf bfc4 bfc9 bfcc bfd0
bfd1 bfd6 bfd9 bfde bfdf bfe4 bfe7 bfec
bfed bff2 bff5 bff9 bffa bfff c002 c007
c008 c00d c010 c014 c018 c01b c01e c01f
c021 c022 c027 c028 c02d c02f c033 c036
c038 c03c c03e c04a c04c c04e c04f c054
c058 c05a c066 c068 c06a c06e c072 c076
c07a c07e c082 c086 c08a c08e c092 c096
c09a c09e c0a2 c0a6 c0aa c0ae c0b2 c0b6
c0ba c0be c0c2 c0c6 c0ca c0ce c0d2 c0d6
c0da c0de c0e2 c0e6 c0ea c0ee c0f2 c0f6
c0fa c0fe c102 c106 c112 c116 c11b c11f
c122 c123 c128 c12c c130 c133 c136 c13a
c13c c140 c144 c147 c14a c14e c150 c154
c158 c15b c15d c161 c165 c169 c16c c170
c174 c176 c177 c17c 1 c180 c184 c188
c18b c18f c193 c197 c19b c19e c1a2 c1a6
c1a9 c1ac c1b1 c1b4 c1b7 c1ba c1bb c1c0
c1c1 1 c1c3 c1c8 c1cc c1d0 c1d3 c1d8
c1dd c1e0 c1e4 c1e5 c1ea c1ed c1f2 c1f3
c1f8 c1fb c200 c201 c206 c209 c20d c20e
c213 c216 c21b c21c c221 c224 c228 c22c
c22f c232 c233 c235 c236 c23b c23c c241
c243 c247 c24a c24c c250 c252 c25e c260
c262 c263 c268 c26c c26e c27a c27c c27e
c282 c286 c289 c28b c28f c293 c297 c29b
c29f c2a3 c2a7 c2ab c2af c2b3 c2b7 c2bb
c2bf c2c3 c2c7 c2cb c2cf c2d3 c2d7 c2db
c2df c2e3 c2e7 c2eb c2ef c2f3 c2f7 c2fb
c2ff c303 c307 c30b c30f c31b c31f c324
c328 c32b c32c c331 c335 c339 c33c c33f
c343 c345 c349 c34d c350 c353 c357 c359
c35d c361 c364 c366 c36a c36e c372 c375
c379 c37d c37f c380 c385 1 c389 c38d
c391 c394 c398 c39c c3a0 c3a4 c3a7 c3ab
c3af c3b2 c3b5 c3ba c3bd c3c0 c3c3 c3c4
c3c9 c3ca 1 c3cc c3d1 c3d5 c3d9 c3dc
c3e1 c3e6 c3e9 c3ed c3ee c3f3 c3f6 c3fb
c3fc c401 c404 c409 c40a c40f c412 c416
c417 c41c c41f c424 c425 c42a c42d c431
c435 c438 c43b c43c c43e c43f c444 c445
c44a c44c c450 c453 c455 c459 c45b c467
c469 c46b c46c c471 c475 c477 c483 c485
c487 c48b c48f c492 c496 c49a c49d c49e
c4a0 c4a3 c4a6 c4a7 c4ac c4af c4b3 c4b7
c4ba c4bb c4bd c4c0 c4c3 c4c4 c4c9 1
c4cc c4d1 c4d5 c4d8 c4dc c4e0 c4e2 c4e6
c4ea c4ed c4ee c4f0 c4f3 c4f6 c4f7 c4fc
c500 c504 c508 c50b c50c c50e c512 c514
c518 c51c c51f c523 c527 c52b c52f c533
c537 c53a c53b c53d c541 c545 c548 c549
c54b c54f c553 c556 c557 c559 c55d c561
c564 c565 c567 c56b c56f c572 c573 c575
c579 c57d c580 c581 c583 c587 c58b c58e
c58f c591 c595 c599 c59c c59d c59f c5a3
c5a7 c5ab c5ac c5ae c5b2 c5b6 c5ba c5bd
c5be c5c0 c5c4 c5c7 c5cb c5ce c5cf c5d4
c5d8 c5dc c5e1 c5e3 c5e7 c5eb c5ee c5f2
c5f6 c5f8 c5fc c600 c604 c607 c60b c60f
c611 c612 c617 1 c61b c61f c623 c626
c62a c62e c632 c636 c639 c63d c641 c644
c647 c64c c64f c652 c655 c656 c65b c65c
1 c65e c663 c667 c66b c66e c673 c678
c67b c67f c680 c685 c688 c68d c68e c693
c696 c69b c69c c6a1 c6a4 c6a8 c6a9 c6ae
c6b1 c6b6 c6b7 c6bc c6bf c6c3 c6c7 c6ca
c6cd c6ce c6d0 c6d1 c6d6 c6d7 c6dc c6de
c6e2 c6e5 c6e7 c6eb c6ed c6f9 c6fb c6fd
c6fe c703 c707 c709 c715 c717 c71b c71f
c722 c726 c72a c72d c730 c735 c738 c73b
c73e c73f c744 c745 1 c747 c74c c750
c754 c757 c75c c761 c764 c768 c769 c76e
c771 c776 c777 c77c c77f c784 c785 c78a
c78d c791 c795 c799 c79d c7a0 c7a1 c7a3
c7a8 c7a9 c7ab c7ac c7b1 c7b2 c7b7 c7b9
c7bd c7c0 c7c4 c7c8 c7cb c7cf c7d3 c7d6
c7d9 c7de c7e1 c7e4 c7e7 c7e8 c7ed c7ee
1 c7f0 c7f5 c7f9 c7fd c800 c805 c80a
c80d c811 c812 c817 c818 c81d c81f c823
c826 c82a c82e c831 c835 c839 c83c c840
c841 c846 c84a c84c c850 c854 c857 c858
c85a c85d c860 c861 c866 c86a c86e c873
c875 c879 c87d c880 c884 c888 c88a c88e
c892 c896 c899 c89d c8a1 c8a3 c8a4 c8a9
1 c8ad c8b1 c8b5 c8b8 c8bc c8c0 c8c4
c8c8 c8cb c8cf c8d3 c8d6 c8d9 c8de c8e1
c8e4 c8e7 c8e8 c8ed c8ee 1 c8f0 c8f5
c8f9 c8fd c900 c905 c90a c90d c911 c912
c917 c91a c91f c920 c925 c928 c92d c92e
c933 c936 c93a c93b c940 c943 c948 c949
c94e c951 c955 c959 c95c c95f c960 c962
c963 c968 c969 c96e c970 c974 c977 c979
c97d c97f c98b c98d c98f c990 c995 c999
c99b c9a7 c9a9 c9ad c9b1 c9b4 c9b8 c9bc
c9bf c9c2 c9c7 c9ca c9cd c9d0 c9d1 c9d6
c9d7 1 c9d9 c9de c9e2 c9e6 c9e9 c9ee
c9f3 c9f6 c9fa c9fb ca00 ca03 ca08 ca09
ca0e ca11 ca16 ca17 ca1c ca1f ca23 ca27
ca2b ca2c ca2e ca33 ca34 ca36 ca37 ca3c
ca3f ca44 ca45 ca4a ca4d ca52 ca53 ca58
ca5b ca5f ca60 ca65 ca68 ca6d ca6e ca73
ca74 ca79 ca7b ca7f ca82 ca86 ca8a ca8d
ca91 ca95 ca98 ca9b caa0 caa3 caa6 caa9
caaa caaf cab0 1 cab2 cab7 cabb cabf
cac2 cac7 cacc cacf cad3 cad4 cad9 cada
cadf cae1 cae5 cae8 caec caf0 caf3 caf7
cafb cafe caff cb01 cb05 cb09 cb0a cb0f
cb11 cb15 cb19 cb1c cb1e cb22 cb26 cb29
cb2a cb2c cb2f cb32 cb33 cb38 cb3c cb40
cb43 cb44 cb46 cb49 cb4c cb4d 1 cb52
cb57 cb5b cb5f cb62 cb63 cb65 cb68 cb6b
cb6c 1 cb71 cb76 cb7a cb7d cb81 cb85
cb89 cb8d cb90 cb91 cb93 cb97 cb99 cb9d
cba0 cba4 cba8 cbab cbac cbae cbb1 cbb4
cbb5 cbba cbbd cbc1 cbc5 cbc8 cbc9 cbcb
cbcf cbd2 cbd6 cbd9 cbda cbdc cbdd cbe2
1 cbe5 cbea cbee cbf2 cbf6 cbf9 cbfa
cbfc cc00 cc02 cc06 cc0a cc0e cc12 cc15
cc16 cc18 cc1b cc1f cc23 cc26 cc27 cc29
cc2a cc2f cc30 cc32 cc36 cc38 cc3c cc40
cc43 cc47 cc4b cc4e cc4f cc51 cc54 cc57
cc58 cc5d cc61 cc65 cc69 cc6c cc6d cc6f
cc72 cc75 cc76 cc7b cc7f cc81 cc85 cc89
cc8d cc91 cc95 cc98 cc99 cc9b cc9c cc9e
cca1 cca4 cca5 ccaa ccab ccad ccb1 ccb3
ccb7 ccbb ccbe ccc2 ccc6 ccca ccce ccd2
ccd6 ccda ccde cce2 cce6 ccea ccee ccf2
ccf6 ccfa ccfe cd02 cd06 cd0a cd0e cd12
cd16 cd1a cd1e cd22 cd26 cd2a cd2e cd32
cd36 cd3a cd46 cd4a cd4f cd53 cd56 cd57
cd5c cd60 cd64 cd67 cd6a cd6e cd70 cd74
cd78 cd7b cd7e cd82 cd84 cd88 cd8c cd8f
cd91 cd95 cd99 cd9d cda0 cda4 cda8 cdaa
cdab cdb0 1 cdb4 cdb8 cdbc cdbf cdc3
cdc7 cdcb cdcf cdd2 cdd6 cdda cddd cde0
cde5 cde8 cdeb cdee cdef cdf4 cdf5 1
cdf7 cdfc ce00 ce04 ce07 ce0c ce11 ce14
ce18 ce19 ce1e ce21 ce26 ce27 ce2c ce2f
ce34 ce35 ce3a ce3d ce41 ce42 ce47 ce4a
ce4f ce50 ce55 ce58 ce5c ce60 ce63 ce66
ce67 ce69 ce6a ce6f ce70 ce75 ce77 ce7b
ce7e ce80 ce84 ce86 ce92 ce94 ce96 ce97
ce9c cea0 cea2 ceae ceb0 ceb2 ceb6 ceba
cebd cec1 cec4 cec8 cecc cecf ced3 ced7
ceda cede cee2 cee6 cee9 ceea ceec ceef
cef2 cef3 cef8 cefc cf00 cf03 cf04 cf06
cf09 cf0c cf0d cf12 cf16 cf1a cf1d cf1e
cf20 cf23 cf26 cf27 1 cf2c cf31 1
cf34 cf39 cf3d cf41 cf45 cf48 cf4c cf4e
cf52 cf56 cf5a cf5d cf61 cf65 cf68 cf69
cf6b cf6f cf73 cf76 cf77 cf79 cf7d cf7f
cf83 cf87 cf8a cf8e cf91 cf95 cf99 cf9c
cfa0 cfa4 cfa8 cfac cfb0 cfb4 cfb8 cfbc
cfc0 cfc4 cfc8 cfcc cfd0 cfd4 cfd8 cfdc
cfe0 cfe4 cfe8 cfec cff0 cff4 cff8 cffc
d000 d004 d008 d00c d010 d014 d018 d01b
d01e d021 d024 d025 d027 d02b d02f d033
d036 d037 d039 d03c d03f d040 d045 d049
d04d d052 d054 d058 d05c d05f d063 d067
d069 d06d d071 d075 d078 d07c d080 d082
d083 d088 1 d08c d090 d094 d097 d09b
d09f d0a3 d0a7 d0aa d0ae d0b2 d0b5 d0b8
d0bd d0c0 d0c3 d0c6 d0c7 d0cc d0cd 1
d0cf d0d4 d0d8 d0dc d0df d0e4 d0e9 d0ec
d0f0 d0f1 d0f6 d0f9 d0fe d0ff d104 d107
d10c d10d d112 d115 d119 d11a d11f d122
d127 d128 d12d d130 d134 d138 d13b d13e
d13f d141 d142 d147 d148 d14d d14f d153
d156 d158 d15c d15e d16a d16c d16e d16f
d174 d178 d17a d186 d188 d18c d190 d193
d197 d19b d19e d1a1 d1a6 d1a9 d1ac d1af
d1b0 d1b5 d1b6 1 d1b8 d1bd d1c1 d1c5
d1c8 d1cd d1d2 d1d5 d1d9 d1da d1df d1e2
d1e7 d1e8 d1ed d1f0 d1f5 d1f6 d1fb d1fe
d202 d206 d20a d20e d211 d212 d214 d219
d21a d21c d21d d222 d225 d22a d22b d230
d233 d238 d239 d23e d241 d245 d246 d24b
d24e d253 d254 d259 d25a d25f d261 d265
d268 d26c d270 d273 d277 d27b d27e d281
d286 d289 d28c d28f d290 d295 d296 1
d298 d29d d2a1 d2a5 d2a8 d2ad d2b2 d2b5
d2b9 d2ba d2bf d2c0 d2c5 d2c7 d2cb d2ce
d2d2 d2d6 d2d9 d2dd d2e1 d2e4 d2e8 d2ec
d2ed d2f2 d2f6 d2f8 d2fc d300 d303 d304
d306 d309 d30c d30d d312 d316 d31a d31f
d321 d325 d329 d32c d330 d334 d336 d33a
d33e d342 d345 d349 d34d d34f d350 d355
1 d359 d35d d361 d364 d368 d36c d370
d374 d377 d37b d37f d382 d385 d38a d38d
d390 d393 d394 d399 d39a 1 d39c d3a1
d3a5 d3a9 d3ac d3b1 d3b6 d3b9 d3bd d3be
d3c3 d3c6 d3cb d3cc d3d1 d3d4 d3d9 d3da
d3df d3e2 d3e6 d3e7 d3ec d3ef d3f4 d3f5
d3fa d3fd d401 d405 d408 d40b d40c d40e
d40f d414 d415 d41a d41c d420 d423 d425
d429 d42b d437 d439 d43b d43c d441 d445
d447 d453 d455 d459 d45d d460 d464 d468
d46b d46e d473 d476 d479 d47c d47d d482
d483 1 d485 d48a d48e d492 d495 d49a
d49f d4a2 d4a6 d4a7 d4ac d4af d4b4 d4b5
d4ba d4bd d4c2 d4c3 d4c8 d4cb d4cf d4d3
d4d7 d4db d4de d4df d4e1 d4e6 d4e7 d4e9
d4ea d4ef d4f2 d4f7 d4f8 d4fd d500 d505
d506 d50b d50e d512 d513 d518 d51b d520
d521 d526 d527 d52c d52e d532 d535 d539
d53d d540 d544 d548 d54b d54e d553 d556
d559 d55c d55d d562 d563 1 d565 d56a
d56e d572 d575 d57a d57f d582 d586 d587
d58c d58d d592 d594 d598 d59b d59f d5a3
d5a6 d5aa d5ae d5b1 d5b5 d5b9 d5ba d5bf
d5c1 d5c5 d5c9 d5cc d5d0 d5d4 d5d8 d5dc
d5e0 d5e4 d5e8 d5f4 d5f8 d5fd d601 d604
d605 d60a d60e d612 d615 d618 d61c d61e
d622 d626 d629 d62c d630 d632 d636 d63a
d63d d63f d643 d647 d64b d64e d652 d656
d658 d659 d65e 1 d662 d666 d66a d66d
d671 d675 d679 d67d d680 d684 d688 d68b
d68e d693 d696 d699 d69c d69d d6a2 d6a3
1 d6a5 d6aa d6ae d6b2 d6b5 d6ba d6bf
d6c2 d6c6 d6c7 d6cc d6cf d6d4 d6d5 d6da
d6dd d6e2 d6e3 d6e8 d6eb d6ef d6f0 d6f5
d6f8 d6fd d6fe d703 d706 d70a d70e d711
d714 d715 d717 d718 d71d d71e d723 d725
d729 d72c d72e d732 d734 d740 d742 d744
d745 d74a d74e d750 d75c d75e d762 d766
d76a d76e d772 d776 d77a d77e d782 d786
d78a d78e d792 d796 d79a d79e d7a2 d7a6
d7aa d7ae d7b2 d7b6 d7ba d7be d7ca d7ce
d7d3 d7d7 d7da d7db d7e0 d7e4 d7e8 d7eb
d7ee d7f2 d7f4 d7f8 d7fc d7ff d802 d806
d808 d80c d810 d813 d815 d819 d81d d821
d824 d828 d82c d82e d82f d834 1 d838
d83c d840 d843 d847 d84b d84f d853 d856
d85a d85e d861 d864 d869 d86c d86f d872
d873 d878 d879 1 d87b d880 d884 d888
d88b d890 d895 d898 d89c d89d d8a2 d8a5
d8aa d8ab d8b0 d8b3 d8b8 d8b9 d8be d8c1
d8c5 d8c6 d8cb d8ce d8d3 d8d4 d8d9 d8dc
d8e0 d8e4 d8e7 d8ea d8eb d8ed d8ee d8f3
d8f4 d8f9 d8fb d8ff d902 d904 d908 d90a
d916 d918 d91a d91b d920 d924 d926 d932
d934 d938 d93c d940 d944 d948 d94c d950
d954 d958 d95c d960 d96c d970 d975 d979
d97c d97d d982 d986 d98a d98d d990 d994
d996 d99a d99e d9a1 d9a4 d9a8 d9aa d9ae
d9b2 d9b5 d9b7 d9bb d9bf d9c3 d9c6 d9ca
d9ce d9d0 d9d1 d9d6 1 d9da d9de d9e2
d9e5 d9e9 d9ed d9f1 d9f5 d9f8 d9fc da00
da03 da06 da0b da0e da11 da14 da15 da1a
da1b 1 da1d da22 da26 da2a da2d da32
da37 da3a da3e da3f da44 da47 da4c da4d
da52 da55 da5a da5b da60 da63 da67 da68
da6d da70 da75 da76 da7b da7e da82 da86
da89 da8c da8d da8f da90 da95 da96 da9b
da9d daa1 daa4 daa6 daaa daac dab8 daba
dabc dabd dac2 dac6 dac8 dad4 dad6 dada
dade dae2 dae6 daea daee daf2 daf6 dafa
dafe db02 db0e db12 db17 db1b db1e db1f
db24 db28 db2c db2f db32 db36 db38 db3c
db40 db43 db46 db4a db4c db50 db54 db57
db59 db5d db61 db65 db68 db6c db70 db72
db73 db78 1 db7c db80 db84 db87 db8b
db8f db93 db97 db9a db9e dba2 dba5 dba8
dbad dbb0 dbb3 dbb6 dbb7 dbbc dbbd 1
dbbf dbc4 dbc8 dbcc dbcf dbd4 dbd9 dbdc
dbe0 dbe1 dbe6 dbe9 dbee dbef dbf4 dbf7
dbfc dbfd dc02 dc05 dc09 dc0a dc0f dc12
dc17 dc18 dc1d dc20 dc24 dc28 dc2b dc2e
dc2f dc31 dc32 dc37 dc38 dc3d dc3f dc43
dc46 dc48 dc4c dc4e dc5a dc5c dc5e dc5f
dc64 dc68 dc6a dc76 dc78 dc7c dc80 dc84
dc87 dc88 dc8a dc8d dc90 dc91 dc96 dc9a
dc9e dca2 dca6 dca9 dcaa dcac dcaf dcb3
dcb7 dcba dcbb dcbd dcbe dcc3 dcc7 dccb
dcd0 dcd4 dcd8 dcdc dce0 dce4 dce8 dce9
dceb dcef dcf3 dcf8 dcfc dd00 dd04 dd08
dd0c dd10 dd11 dd13 dd17 dd1b dd1f dd23
dd27 dd2b dd2f dd33 dd3f dd43 dd48 dd4c
dd4f dd50 dd55 dd59 dd5d dd60 dd63 dd67
dd69 dd6d dd71 dd74 dd77 dd7b dd7d dd81
dd85 dd88 dd8a dd8e dd92 dd96 dd99 dd9d
dda1 dda3 dda4 dda9 1 ddad ddb1 ddb5
ddb8 ddbc ddc0 ddc4 ddc8 ddcb ddcf ddd3
ddd6 ddd9 ddde dde1 dde4 dde7 dde8 dded
ddee 1 ddf0 ddf5 ddf9 ddfd de00 de05
de0a de0d de11 de12 de17 de1a de1f de20
de25 de28 de2d de2e de33 de36 de3a de3b
de40 de43 de48 de49 de4e de51 de55 de59
de5c de5f de60 de62 de63 de68 de69 de6e
de70 de74 de77 de79 de7d de7f de8b de8d
de8f de90 de95 de99 de9b dea7 dea9 dead
deb1 deb4 deb5 deb7 deba debd debe dec3
dec6 deca dece ded1 ded2 ded4 ded7 deda
dedb dee0 1 dee3 dee8 deec def0 def4
def8 defc df00 df04 df08 df0c df10 df14
df18 df1c df20 df24 df28 df2c df38 df3c
df41 df45 df48 df49 df4e df52 df56 df59
df5c df60 df62 df66 df6a df6d df70 df74
df76 df7a df7e df81 df83 df87 df8b df8f
df92 df96 df9a df9c df9d dfa2 1 dfa6
dfaa dfae dfb1 dfb5 dfb9 dfbd dfc1 dfc4
dfc8 dfcc dfcf dfd2 dfd7 dfda dfdd dfe0
dfe1 dfe6 dfe7 1 dfe9 dfee dff2 dff6
dff9 dffe e003 e006 e00a e00b e010 e013
e018 e019 e01e e021 e026 e027 e02c e02f
e033 e034 e039 e03c e041 e042 e047 e04a
e04e e052 e055 e058 e059 e05b e05c e061
e062 e067 e069 e06d e070 e072 e076 e078
e084 e086 e088 e089 e08e e092 e094 e0a0
e0a2 e0a6 e0a8 e0ac e0b0 e0b3 e0b4 e0b6
e0b9 e0bc e0bd e0c2 e0c6 e0ca e0ce e0d2
e0d6 e0da e0de e0e2 e0e6 e0ea e0ee e0f2
e0f6 e102 e106 e10b e10f e112 e113 e118
e11c e120 e123 e126 e12a e12c e130 e134
e137 e13a e13e e140 e144 e148 e14b e14d
e151 e155 e159 e15c e160 e164 e166 e167
e16c 1 e170 e174 e178 e17b e17f e183
e187 e18b e18e e192 e196 e199 e19c e1a1
e1a4 e1a7 e1aa e1ab e1b0 e1b1 1 e1b3
e1b8 e1bc e1c0 e1c3 e1c8 e1cd e1d0 e1d4
e1d5 e1da e1dd e1e2 e1e3 e1e8 e1eb e1f0
e1f1 e1f6 e1f9 e1fd e1fe e203 e206 e20b
e20c e211 e214 e218 e21c e21f e222 e223
e225 e226 e22b e22c e231 e233 e237 e23a
e23c e240 e242 e24e e250 e252 e253 e258
e25c e25e e26a e26c e270 e274 e278 e27b
e27c e27e e282 e286 e288 e28c e290 e294
e297 e298 e29a e29d e2a0 e2a1 e2a6 e2aa
e2ae e2b2 e2b6 e2ba e2be e2c2 e2c6 e2ca
e2ce e2d2 e2d6 e2da e2e6 e2ea e2ef e2f3
e2f6 e2f7 e2fc e300 e304 e307 e30a e30e
e310 e314 e318 e31b e31e e322 e324 e328
e32c e32f e331 e335 e339 e33d e340 e344
e348 e34a e34b e350 1 e354 e358 e35c
e35f e363 e367 e36b e36f e372 e376 e37a
e37d e380 e385 e388 e38b e38e e38f e394
e395 1 e397 e39c e3a0 e3a4 e3a7 e3ac
e3b1 e3b4 e3b8 e3b9 e3be e3c1 e3c6 e3c7
e3cc e3cf e3d4 e3d5 e3da e3dd e3e1 e3e2
e3e7 e3ea e3ef e3f0 e3f5 e3f8 e3fc e400
e403 e406 e407 e409 e40a e40f e410 e415
e417 e41b e41e e420 e424 e426 e432 e434
e436 e437 e43c e440 e442 e44e e450 e454
e458 e45c e45f e460 e462 e466 e468 e46c
e470 e474 e478 e47c e480 e484 e488 e48c
e490 e49c e4a0 e4a5 e4a9 e4ac e4ad e4b2
e4b6 e4ba e4bd e4c0 e4c4 e4c6 e4ca e4ce
e4d1 e4d4 e4d8 e4da e4de e4e2 e4e5 e4e7
e4eb e4ef e4f3 e4f6 e4fa e4fe e500 e501
e506 1 e50a e50e e512 e515 e519 e51d
e521 e525 e528 e52c e530 e533 e536 e53b
e53e e541 e544 e545 e54a e54b 1 e54d
e552 e556 e55a e55d e562 e567 e56a e56e
e56f e574 e577 e57c e57d e582 e585 e58a
e58b e590 e593 e597 e598 e59d e5a0 e5a5
e5a6 e5ab e5ae e5b2 e5b6 e5b9 e5bc e5bd
e5bf e5c0 e5c5 e5c6 e5cb e5cd e5d1 e5d4
e5d6 e5da e5dc e5e8 e5ea e5ec e5ed e5f2
e5f6 e5f8 e604 e606 e60a e60e e612 e615
e616 e618 e61c e620 e624 e628 e62b e62c
e62e e632 e634 e638 e63c e63f e641 e645
e648 e64a e64e e651 e653 e657 e65b e65e
e662 e666 e669 e66d e671 e675 e679 e67a
e67c e680 e682 e686 e68d e691 e695 e69a
e69c e6a0 e6a4 e6a7 e6ab e6af e6b1 e6b5
e6b9 e6bd e6c0 e6c4 e6c8 e6ca e6cb e6d0
1 e6d4 e6d8 e6dc e6df e6e3 e6e7 e6eb
e6ef e6f2 e6f6 e6fa e6fd e700 e705 e708
e70b e70e e70f e714 e715 1 e717 e71c
e720 e724 e727 e72c e731 e734 e738 e739
e73e e741 e746 e747 e74c e74f e754 e755
e75a e75d e761 e762 e767 e76a e76f e770
e775 e778 e77c e780 e783 e786 e787 e789
e78a e78f e790 e795 e797 e79b e79e e7a0
e7a4 e7a6 e7b2 e7b4 e7b6 e7b7 e7bc e7c0
e7c2 e7ce e7d0 e7d4 e7d7 e7db e7df e7e3
e7e6 e7e7 e7e9 e7ec e7ef e7f0 e7f5 e7f9
e7fd e801 e805 e809 e80c e80d e80f e813
e816 e81a e81d e81e e820 e821 e826 e82a
e82e e832 e835 e836 e838 e83b e83f e843
e846 e847 e849 e84a e84f e853 e857 e85b
e85e e85f e861 e864 e867 e868 e86d e870
e874 e878 e87b e87c e87e e881 e884 e885
e88a 1 e88d e892 e896 e899 e89a e89f
e8a2 1 e8a5 e8aa e8ae e8b2 e8b6 e8ba
e8be e8c2 e8c6 e8ca e8ce e8d2 e8d6 e8da
e8de e8e2 e8e6 e8ea e8ee e8f2 e8f6 e8fa
e8fe e902 e906 e90a e90e e912 e915 e916
e918 e91b e91e e91f e924 e928 e92c e92f
e930 e932 e935 e938 e939 1 e93e e943
e947 e94a e94e e952 e954 e958 e95c e95f
e960 e962 e965 e968 e969 e96e e972 e976
e97a e97d e97e e980 e984 e986 e98a e98e
e991 e995 e999 e99d e9a1 e9a5 e9a9 e9ac
e9ad e9af e9b3 e9b7 e9ba e9bb e9bd e9c1
e9c5 e9c8 e9c9 e9cb e9cf e9d3 e9d6 e9d7
e9d9 e9dd e9e1 e9e4 e9e5 e9e7 e9eb e9ef
e9f2 e9f3 e9f5 e9f9 e9fd ea00 ea01 ea03
ea07 ea0b ea0e ea0f ea11 ea15 ea19 ea1d
ea1e ea20 ea24 ea28 ea2c ea2f ea30 ea32
ea36 ea39 ea3d ea40 ea41 ea46 ea4a ea4e
ea51 ea55 ea59 ea5c ea5f ea64 ea67 ea6a
ea6d ea6e ea73 ea74 1 ea76 ea7b ea7f
ea83 ea86 ea8b ea90 ea93 ea97 ea98 ea9d
eaa0 eaa5 eaa6 eaab eaae eab3 eab4 eab9
eabc eac0 eac4 eac8 eacc eacf ead0 ead2
ead7 ead8 eada eadb eae0 eae1 eae6 eae8
eaec eaef eaf3 eaf7 eafa eafe eb02 eb05
eb08 eb0d eb10 eb13 eb16 eb17 eb1c eb1d
1 eb1f eb24 eb28 eb2c eb2f eb34 eb39
eb3c eb40 eb41 eb46 eb47 eb4c eb4e eb52
eb55 eb59 eb5d eb60 eb64 eb68 eb6b eb6f
eb70 eb75 eb79 eb7b eb7f eb83 eb86 eb87
eb89 eb8c eb8f eb90 eb95 eb99 eb9d eba0
eba4 eba8 ebab ebae ebb3 ebb6 ebb9 ebbc
ebbd ebc2 ebc3 1 ebc5 ebca ebce ebd2
ebd5 ebda ebdf ebe2 ebe6 ebe7 ebec ebef
ebf4 ebf5 ebfa ebfd ec02 ec03 ec08 ec0b
ec0f ec13 ec17 ec18 ec1a ec1f ec20 ec22
ec23 ec28 ec2b ec30 ec31 ec36 ec39 ec3e
ec3f ec44 ec47 ec4b ec4c ec51 ec54 ec59
ec5a ec5f ec60 ec65 ec67 ec6b ec6e ec72
ec76 ec79 ec7d ec81 ec84 ec87 ec8c ec8f
ec92 ec95 ec96 ec9b ec9c 1 ec9e eca3
eca7 ecab ecae ecb3 ecb8 ecbb ecbf ecc0
ecc5 ecc6 eccb eccd ecd1 ecd4 ecd8 ecdc
ecdf ece3 ece7 ecea eceb eced ecf1 ecf5
ecf6 ecfb ecfd ed01 ed05 ed08 ed0c ed10
ed15 ed1a ed1e ed22 ed26 ed29 ed2c ed30
ed34 ed38 ed3c ed40 ed44 ed48 ed4c ed50
ed54 ed58 ed64 ed68 ed6d ed71 ed74 ed75
ed7a ed7e ed82 ed85 ed88 ed8c ed8e ed92
ed96 ed99 ed9c eda0 eda2 eda6 edaa edad
edaf edb3 edb7 edbb edbe edc2 edc6 edc8
edc9 edce 1 edd2 edd6 edda eddd ede1
ede5 ede9 eded edf0 edf4 edf8 edfb edfe
ee03 ee06 ee09 ee0c ee0d ee12 ee13 1
ee15 ee1a ee1e ee22 ee25 ee2a ee2f ee32
ee36 ee37 ee3c ee3f ee44 ee45 ee4a ee4d
ee52 ee53 ee58 ee5b ee5f ee60 ee65 ee68
ee6d ee6e ee73 ee76 ee7a ee7e ee81 ee84
ee85 ee87 ee88 ee8d ee8e ee93 ee95 ee99
ee9c ee9e eea2 eea4 eeb0 eeb2 eeb4 eeb5
eeba eebe eec0 eecc eece eed2 eed7 eedb
eedf eee2 eee6 eeea eeed eef1 eef5 eef8
eefc ef00 ef03 ef07 ef0b ef0f ef12 ef13
ef15 ef18 ef1b ef1c ef21 ef25 ef29 ef2c
ef2d ef2f ef32 ef35 ef36 ef3b ef3f ef43
ef46 ef47 ef49 ef4c ef4f ef50 1 ef55
ef5a 1 ef5d ef62 ef66 ef6a ef6d ef6e
ef70 ef73 ef76 ef77 ef7c ef80 ef84 ef87
ef88 ef8a ef8d ef90 ef91 1 ef96 ef9b
ef9f efa3 efa6 efa7 efa9 efac efaf efb0
1 efb5 efba efbe efc1 efc5 efc9 efcd
efd1 efd4 efd5 efd7 efdb efdd efe1 efe4
efe8 efec eff0 eff3 eff7 effb effe efff
f001 f005 f009 f00c f00d f00f f013 f017
f01a f01e f020 f024 f027 f02b f02d f031
f035 f038 f03c f03f f043 f047 f04b f04f
f053 f057 f05b f05f f063 f067 f06b f06f
f073 f077 f07b f07f f083 f087 f08b f08f
f093 f097 f09b f09f f0a3 f0a7 f0ab f0af
f0b3 f0b7 f0bb f0be f0c1 f0c4 f0c7 f0c8
f0ca f0ce f0d2 f0d6 f0d9 f0da f0dc f0df
f0e2 f0e3 f0e8 f0ec f0f0 f0f3 f0f7 f0fb
f0fe f101 f106 f109 f10c f10f f110 f115
f116 1 f118 f11d f121 f125 f128 f12d
f132 f135 f139 f13a f13f f142 f147 f148
f14d f150 f155 f156 f15b f15e f162 f166
f16a f16e f171 f172 f174 f179 f17a f17c
f17d f182 f185 f18a f18b f190 f193 f198
f199 f19e f1a1 f1a5 f1a6 f1ab f1ae f1b3
f1b4 f1b9 f1ba f1bf f1c1 f1c5 f1c8 f1cc
f1d0 f1d3 f1d7 f1db f1de f1e1 f1e6 f1e9
f1ec f1ef f1f0 f1f5 f1f6 1 f1f8 f1fd
f201 f205 f208 f20d f212 f215 f219 f21a
f21f f220 f225 f227 f22b f22e f232 f236
f239 f23d f241 f244 f248 f24c f24d f252
f256 f258 f25c f260 f263 f264 f266 f269
f26c f26d f272 f276 f27a f27d f281 f285
f288 f28b f290 f293 f296 f299 f29a f29f
f2a0 1 f2a2 f2a7 f2ab f2af f2b2 f2b7
f2bc f2bf f2c3 f2c4 f2c9 f2cc f2d1 f2d2
f2d7 f2da f2df f2e0 f2e5 f2e8 f2ec f2f0
f2f4 f2f8 f2fb f2fc f2fe f303 f304 f306
f307 f30c f30f f314 f315 f31a f31d f322
f323 f328 f32b f32f f330 f335 f338 f33d
f33e f343 f344 f349 f34b f34f f352 f356
f35a f35d f361 f365 f368 f36b f370 f373
f376 f379 f37a f37f f380 1 f382 f387
f38b f38f f392 f397 f39c f39f f3a3 f3a4
f3a9 f3aa f3af f3b1 f3b5 f3b8 f3bc f3c0
f3c3 f3c7 f3cb f3ce f3d2 f3d6 f3d7 f3dc
f3de f3e2 f3e6 f3ea f3ee f3f2 f3f6 f3fa
f3fe f402 f406 f40a f40e f412 f416 f41a
f41e f422 f426 f42a f42e f432 f436 f43a
f43e f442 f446 f44a f44e f452 f45e f462
f467 f46b f46e f46f f474 f478 f47c f47f
f482 f486 f488 f48c f490 f493 f496 f49a
f49c f4a0 f4a4 f4a7 f4a9 f4ad f4b1 f4b5
f4b8 f4bc f4c0 f4c2 f4c3 f4c8 1 f4cc
f4d0 f4d4 f4d7 f4db f4df f4e3 f4e7 f4ea
f4ee f4f2 f4f5 f4f8 f4fd f500 f503 f506
f507 f50c f50d 1 f50f f514 f518 f51c
f51f f524 f529 f52c f530 f531 f536 f539
f53e f53f f544 f547 f54c f54d f552 f555
f559 f55a f55f f562 f567 f568 f56d f570
f574 f578 f57b f57e f57f f581 f582 f587
f588 f58d f58f f593 f596 f598 f59c f59e
f5aa f5ac f5ae f5af f5b4 f5b8 f5ba f5c6
f5c8 f5ca f5ce f5d2 f5d5 f5d7 f5db f5de
f5e0 f5e4 f5e7 f5eb f5ef f5f3 f5f7 f5fb
f5ff f603 f607 f613 f617 f61b f61e f622
f626 f628 f62c f630 f634 f637 f63b f63f
f641 f642 f647 1 f64b f64f f653 f656
f65a f65e f662 f666 f669 f66d f671 f674
f677 f67c f67f f682 f685 f686 f68b f68c
1 f68e f693 f697 f69b f69e f6a3 f6a8
f6ab f6af f6b0 f6b5 f6b8 f6bd f6be f6c3
f6c6 f6cb f6cc f6d1 f6d4 f6d8 f6d9 f6de
f6e1 f6e6 f6e7 f6ec f6ef f6f3 f6f7 f6fa
f6fd f6fe f700 f701 f706 f707 f70c f70e
f712 f715 f717 f71b f71d f729 f72b f72d
f72e f733 f737 f739 f745 f747 f74b f74f
f752 f753 f755 f758 f75b f75c f761 f765
f769 f76d f771 f775 f779 f77d f781 f785
f791 f795 f79a f79e f7a1 f7a2 f7a7 f7ab
f7af f7b2 f7b5 f7b9 f7bb f7bf f7c3 f7c6
f7c9 f7cd f7cf f7d3 f7d7 f7da f7dc f7e0
f7e4 f7e8 f7eb f7ef f7f3 f7f5 f7f6 f7fb
1 f7ff f803 f807 f80a f80e f812 f816
f81a f81d f821 f825 f828 f82b f830 f833
f836 f839 f83a f83f f840 1 f842 f847
f84b f84f f852 f857 f85c f85f f863 f864
f869 f86c f871 f872 f877 f87a f87f f880
f885 f888 f88c f88d f892 f895 f89a f89b
f8a0 f8a3 f8a7 f8ab f8ae f8b1 f8b2 f8b4
f8b5 f8ba f8bb f8c0 f8c2 f8c6 f8c9 f8cb
f8cf f8d1 f8dd f8df f8e1 f8e2 f8e7 f8eb
f8ed f8f9 f8fb f8fd f901 f904 f906 f90a
f90d f911 f914 f918 f91c f91f f920 f925
f928 f92c f930 f934 f938 f93c f940 f944
f948 f94c f958 f95c f961 f965 f968 f969
f96e f972 f976 f979 f97c f980 f982 f986
f98a f98d f990 f994 f996 f99a f99e f9a1
f9a3 f9a7 f9ab f9af f9b2 f9b6 f9ba f9bc
f9bd f9c2 1 f9c6 f9ca f9ce f9d1 f9d5
f9d9 f9dd f9e1 f9e4 f9e8 f9ec f9ef f9f2
f9f7 f9fa f9fd fa00 fa01 fa06 fa07 1
fa09 fa0e fa12 fa16 fa19 fa1e fa23 fa26
fa2a fa2b fa30 fa33 fa38 fa39 fa3e fa41
fa46 fa47 fa4c fa4f fa53 fa54 fa59 fa5c
fa61 fa62 fa67 fa6a fa6e fa72 fa75 fa78
fa79 fa7b fa7c fa81 fa82 fa87 fa89 fa8d
fa90 fa92 fa96 fa98 faa4 faa6 faa8 faa9
faae fab2 fab4 fac0 fac2 fac6 faca facd
face fad0 fad3 fad6 fad7 fadc fae0 fae4
fae8 faec faf0 faf4 faf8 fb04 fb08 fb0c
fb0f fb13 fb17 fb19 fb1d fb21 fb25 fb28
fb2c fb30 fb32 fb33 fb38 1 fb3c fb40
fb44 fb47 fb4b fb4f fb53 fb57 fb5a fb5e
fb62 fb65 fb68 fb6d fb70 fb73 fb76 fb77
fb7c fb7d 1 fb7f fb84 fb88 fb8c fb8f
fb94 fb99 fb9c fba0 fba1 fba6 fba9 fbae
fbaf fbb4 fbb7 fbbc fbbd fbc2 fbc5 fbc9
fbca fbcf fbd2 fbd7 fbd8 fbdd fbe0 fbe4
fbe8 fbeb fbee fbef fbf1 fbf2 fbf7 fbf8
fbfd fbff fc03 fc06 fc08 fc0c fc0e fc1a
fc1c fc1e fc1f fc24 fc28 fc2a fc36 fc38
fc3c fc40 fc43 fc44 fc46 fc49 fc4c fc4d
fc52 fc56 fc5a fc5e fc62 fc66 fc6a fc6e
fc72 fc7e fc82 fc87 fc8b fc8e fc8f fc94
fc98 fc9c fc9f fca2 fca6 fca8 fcac fcb0
fcb3 fcb6 fcba fcbc fcc0 fcc4 fcc7 fcc9
fccd fcd1 fcd5 fcd8 fcdc fce0 fce2 fce3
fce8 1 fcec fcf0 fcf4 fcf7 fcfb fcff
fd03 fd07 fd0a fd0e fd12 fd15 fd18 fd1d
fd20 fd23 fd26 fd27 fd2c fd2d 1 fd2f
fd34 fd38 fd3c fd3f fd44 fd49 fd4c fd50
fd51 fd56 fd59 fd5e fd5f fd64 fd67 fd6c
fd6d fd72 fd75 fd79 fd7a fd7f fd82 fd87
fd88 fd8d fd90 fd94 fd98 fd9b fd9e fd9f
fda1 fda2 fda7 fda8 fdad fdaf fdb3 fdb6
fdb8 fdbc fdbe fdca fdcc fdce fdcf fdd4
fdd8 fdda fde6 fde8 fdec fdf0 fdf4 fdf8
fdfc fe08 fe0c fe11 fe15 fe18 fe19 fe1e
fe22 fe26 fe29 fe2c fe30 fe32 fe36 fe3a
fe3d fe40 fe44 fe46 fe4a fe4e fe51 fe53
fe57 fe5b fe5f fe62 fe66 fe6a fe6c fe6d
fe72 1 fe76 fe7a fe7e fe81 fe85 fe89
fe8d fe91 fe94 fe98 fe9c fe9f fea2 fea7
feaa fead feb0 feb1 feb6 feb7 1 feb9
febe fec2 fec6 fec9 fece fed3 fed6 feda
fedb fee0 fee3 fee8 fee9 feee fef1 fef6
fef7 fefc feff ff03 ff04 ff09 ff0c ff11
ff12 ff17 ff1a ff1e ff22 ff25 ff28 ff29
ff2b ff2c ff31 ff32 ff37 ff39 ff3d ff40
ff42 ff46 ff48 ff54 ff56 ff58 ff59 ff5e
ff62 ff64 ff70 ff72 ff74 ff78 ff7b ff7d
ff81 ff84 ff88 ff8c ff8f ff90 ff92 ff95
ff98 ff99 ff9e ffa2 ffa5 ffa9 ffad ffb1
ffb5 ffb9 ffbd ffc0 ffc3 ffc4 ffc6 ffca
ffce ffd2 ffd6 ffda ffde ffe2 ffe6 ffea
ffee fff2 fff6 fffa fffe 10002 10006 1000a
1000e 10012 10016 1001a 1001e 10022 10026 1002a
1002e 1003a 1003e 10042 10045 10049 1004d 1004f
10053 10057 1005b 1005e 10062 10066 10068 10069
1006e 1 10072 10076 1007a 1007d 10081 10085
10089 1008d 10090 10094 10098 1009b 1009e 100a3
100a6 100a9 100ac 100ad 100b2 100b3 1 100b5
100ba 100be 100c2 100c5 100ca 100cf 100d2 100d6
100d7 100dc 100df 100e4 100e5 100ea 100ed 100f2
100f3 100f8 100fb 100ff 10100 10105 10108 1010d
1010e 10113 10116 1011a 1011e 10121 10124 10125
10127 10128 1012d 1012e 10133 10135 10139 1013c
1013e 10142 10144 10150 10152 10154 10155 1015a
1015e 10160 1016c 1016e 10172 10176 10179 1017d
10180 10184 10187 10188 1018d 10190 10194 10198
1019b 1019c 1019e 101a1 101a4 101a5 101aa 101ae
101b2 101b6 101ba 101be 101c2 101c6 101ca 101ce
101d2 101d6 101da 101de 101e2 101e6 101f2 101f6
101fb 101ff 10202 10203 10208 1020c 10210 10213
10216 1021a 1021c 10220 10224 10227 1022a 1022e
10230 10234 10238 1023b 1023d 10241 10245 10249
1024c 10250 10254 10256 10257 1025c 1 10260
10264 10268 1026b 1026f 10273 10277 1027b 1027e
10282 10286 10289 1028c 10291 10294 10297 1029a
1029b 102a0 102a1 1 102a3 102a8 102ac 102b0
102b3 102b8 102bd 102c0 102c4 102c5 102ca 102cd
102d2 102d3 102d8 102db 102e0 102e1 102e6 102e9
102ed 102ee 102f3 102f6 102fb 102fc 10301 10304
10308 1030c 1030f 10312 10313 10315 10316 1031b
1031c 10321 10323 10327 1032a 1032c 10330 10332
1033e 10340 10342 10343 10348 1034c 1034e 1035a
1035c 1035e 10362 10366 1036a 1036e 10372 10376
1037a 1037e 10382 10386 1038a 1038e 10392 10396
1039a 1039e 103a2 103ae 103b2 103b7 103bb 103be
103bf 103c4 103c8 103cc 103cf 103d2 103d6 103d8
103dc 103e0 103e3 103e6 103ea 103ec 103f0 103f4
103f7 103f9 103fd 10401 10405 10408 1040c 10410
10412 10413 10418 1 1041c 10420 10424 10427
1042b 1042f 10433 10437 1043a 1043e 10442 10445
10448 1044d 10450 10453 10456 10457 1045c 1045d
1 1045f 10464 10468 1046c 1046f 10474 10479
1047c 10480 10481 10486 10489 1048e 1048f 10494
10497 1049c 1049d 104a2 104a5 104a9 104aa 104af
104b2 104b7 104b8 104bd 104c0 104c4 104c8 104cb
104ce 104cf 104d1 104d2 104d7 104d8 104dd 104df
104e3 104e6 104e8 104ec 104ee 104fa 104fc 104fe
104ff 10504 10508 1050a 10516 10518 1051a 1051e
10522 10525 10529 1052d 10530 10533 10534 10536
10539 1053e 1053f 10544 10547 1054b 1054e 10552
10556 1055a 1055d 10561 10565 10569 1056d 10571
10575 10579 1057d 10581 10585 10589 1058a 1058c
10590 10592 10596 10599 1059b 1059f 105a2 105a6
105a9 105aa 105af 105b2 105b5 105b9 105bd 105c0
105c4 105c8 105c9 105cb 105ce 105d1 105d2 105d7
1 105da 105df 105e3 105e6 105ea 105ec 105f0
105f3 105f5 105f9 105fc 105fe 10602 10605 10609
1060d 10610 10611 10613 10616 10619 1061a 1061f
10622 10626 10629 1062a 1062f 10632 1 10635
1063a 1063e 10642 10645 10646 10648 1064b 1064e
1064f 10654 10658 1065c 10660 10664 10668 1066c
10670 10674 10678 1067c 10688 1068c 10690 10693
10697 1069b 1069d 106a1 106a5 106a9 106ac 106b0
106b4 106b6 106b7 106bc 1 106c0 106c4 106c8
106cb 106cf 106d3 106d7 106db 106de 106e2 106e6
106e9 106ec 106f1 106f4 106f7 106fa 106fb 10700
10701 1 10703 10708 1070c 10710 10713 10718
1071d 10720 10724 10725 1072a 1072d 10732 10733
10738 1073b 10740 10741 10746 10749 1074d 1074e
10753 10756 1075b 1075c 10761 10764 10768 1076c
1076f 10772 10773 10775 10776 1077b 1077c 10781
10783 10787 1078a 1078c 10790 10792 1079e 107a0
107a2 107a3 107a8 107ac 107ae 107ba 107bc 107c0
107c4 107c7 107c8 107ca 107cd 107d0 107d1 107d6
107da 107de 107e2 107e6 107ea 107ee 107f2 107f6
107fa 10806 1080a 1080f 10813 10816 10817 1081c
10820 10824 10827 1082a 1082e 10830 10834 10838
1083b 1083e 10842 10844 10848 1084c 1084f 10851
10855 10859 1085d 10860 10864 10868 1086a 1086b
10870 1 10874 10878 1087c 1087f 10883 10887
1088b 1088f 10892 10896 1089a 1089d 108a0 108a5
108a8 108ab 108ae 108af 108b4 108b5 1 108b7
108bc 108c0 108c4 108c7 108cc 108d1 108d4 108d8
108d9 108de 108e1 108e6 108e7 108ec 108ef 108f4
108f5 108fa 108fd 10901 10902 10907 1090a 1090f
10910 10915 10918 1091c 10920 10923 10926 10927
10929 1092a 1092f 10930 10935 10937 1093b 1093e
10940 10944 10946 10952 10954 10956 10957 1095c
10960 10962 1096e 10970 10974 10978 1097c 10980
10984 10988 1098c 10990 10994 109a0 109a4 109a9
109ad 109b0 109b1 109b6 109ba 109be 109c1 109c4
109c8 109ca 109ce 109d2 109d5 109d8 109dc 109de
109e2 109e6 109e9 109eb 109ef 109f3 109f7 109fa
109fe 10a02 10a04 10a05 10a0a 1 10a0e 10a12
10a16 10a19 10a1d 10a21 10a25 10a29 10a2c 10a30
10a34 10a37 10a3a 10a3f 10a42 10a45 10a48 10a49
10a4e 10a4f 1 10a51 10a56 10a5a 10a5e 10a61
10a66 10a6b 10a6e 10a72 10a73 10a78 10a7b 10a80
10a81 10a86 10a89 10a8e 10a8f 10a94 10a97 10a9b
10a9c 10aa1 10aa4 10aa9 10aaa 10aaf 10ab2 10ab6
10aba 10abd 10ac0 10ac1 10ac3 10ac4 10ac9 10aca
10acf 10ad1 10ad5 10ad8 10ada 10ade 10ae0 10aec
10aee 10af0 10af1 10af6 10afa 10afc 10b08 10b0a
10b0c 10b10 10b13 10b15 10b19 10b1c 10b20 10b23
10b27 10b2b 10b2f 10b33 10b37 10b3b 10b3e 10b41
10b42 10b44 10b48 10b4c 10b50 10b54 10b58 10b5c
10b60 10b64 10b68 10b6c 10b70 10b74 10b78 10b7c
10b80 10b84 10b88 10b8c 10b90 10b94 10b98 10b9c
10ba0 10ba4 10ba8 10bac 10bb8 10bbc 10bc0 10bc3
10bc7 10bcb 10bcd 10bd1 10bd5 10bd9 10bdc 10be0
10be4 10be6 10be7 10bec 1 10bf0 10bf4 10bf8
10bfb 10bff 10c03 10c07 10c0b 10c0e 10c12 10c16
10c19 10c1c 10c21 10c24 10c27 10c2a 10c2b 10c30
10c31 1 10c33 10c38 10c3c 10c40 10c43 10c48
10c4d 10c50 10c54 10c55 10c5a 10c5d 10c62 10c63
10c68 10c6b 10c70 10c71 10c76 10c79 10c7d 10c7e
10c83 10c86 10c8b 10c8c 10c91 10c94 10c98 10c9c
10c9f 10ca2 10ca3 10ca5 10ca6 10cab 10cac 10cb1
10cb3 10cb7 10cba 10cbc 10cc0 10cc2 10cce 10cd0
10cd2 10cd3 10cd8 10cdc 10cde 10cea 10cec 10cf0
10cf4 10cf7 10cfb 10cfe 10d02 10d05 10d06 10d0b
10d0e 10d12 10d16 10d19 10d1a 10d1c 10d1f 10d22
10d23 10d28 10d2c 10d30 10d34 10d38 10d3c 10d40
10d44 10d48 10d4c 10d50 10d54 10d58 10d5c 10d60
10d64 10d70 10d74 10d79 10d7d 10d80 10d81 10d86
10d8a 10d8e 10d91 10d94 10d98 10d9a 10d9e 10da2
10da5 10da8 10dac 10dae 10db2 10db6 10db9 10dbb
10dbf 10dc3 10dc7 10dca 10dce 10dd2 10dd4 10dd5
10dda 1 10dde 10de2 10de6 10de9 10ded 10df1
10df5 10df9 10dfc 10e00 10e04 10e07 10e0a 10e0f
10e12 10e15 10e18 10e19 10e1e 10e1f 1 10e21
10e26 10e2a 10e2e 10e31 10e36 10e3b 10e3e 10e42
10e43 10e48 10e4b 10e50 10e51 10e56 10e59 10e5e
10e5f 10e64 10e67 10e6b 10e6c 10e71 10e74 10e79
10e7a 10e7f 10e82 10e86 10e8a 10e8d 10e90 10e91
10e93 10e94 10e99 10e9a 10e9f 10ea1 10ea5 10ea8
10eaa 10eae 10eb0 10ebc 10ebe 10ec0 10ec1 10ec6
10eca 10ecc 10ed8 10eda 10edc 10ee0 10ee4 10ee8
10eec 10ef0 10ef4 10ef8 10efc 10f00 10f04 10f08
10f0c 10f10 10f14 10f18 10f1c 10f20 10f2c 10f30
10f35 10f39 10f3c 10f3d 10f42 10f46 10f4a 10f4d
10f50 10f54 10f56 10f5a 10f5e 10f61 10f64 10f68
10f6a 10f6e 10f72 10f75 10f77 10f7b 10f7f 10f83
10f86 10f8a 10f8e 10f90 10f91 10f96 1 10f9a
10f9e 10fa2 10fa5 10fa9 10fad 10fb1 10fb5 10fb8
10fbc 10fc0 10fc3 10fc6 10fcb 10fce 10fd1 10fd4
10fd5 10fda 10fdb 1 10fdd 10fe2 10fe6 10fea
10fed 10ff2 10ff7 10ffa 10ffe 10fff 11004 11007
1100c 1100d 11012 11015 1101a 1101b 11020 11023
11027 11028 1102d 11030 11035 11036 1103b 1103e
11042 11046 11049 1104c 1104d 1104f 11050 11055
11056 1105b 1105d 11061 11064 11066 1106a 1106c
11078 1107a 1107c 1107d 11082 11086 11088 11094
11096 11098 1109c 110a0 110a3 110a7 110ab 110ae
110b1 110b2 110b4 110b7 110bc 110bd 110c2 110c5
110c9 110cd 110d0 110d1 110d3 110d6 110d9 110da
110df 1 110e2 110e7 110eb 110ee 110f2 110f6
110fa 110fd 11101 11105 11109 1110d 11111 11115
11119 1111d 11121 11125 11129 1112a 1112c 11130
11132 11136 11139 1113b 1113f 11142 11144 11148
1114b 1114f 11153 11157 11158 1115a 1115e 11162
11166 11169 1116c 1116f 11170 11175 11179 1117d
11180 11183 11187 11189 1118d 11190 11194 11198
1119b 1119f 111a3 111a6 111a9 111ae 111b1 111b4
111b7 111b8 111bd 111be 1 111c0 111c5 111c9
111cd 111d0 111d5 111da 111dd 111e1 111e2 111e7
111ea 111ef 111f0 111f5 111f8 111fd 111fe 11203
11206 1120a 1120e 11212 11216 11219 1121a 1121c
11221 11222 11224 11225 1122a 1122d 11232 11233
11238 1123b 11240 11241 11246 11249 1124d 1124e
11253 11256 1125b 1125c 11261 11262 11267 11269
1126d 11270 11274 11278 1127b 1127f 11283 11286
1128a 1128e 1128f 11294 11296 1129a 1129e 112a1
112a5 112a9 112ac 112b0 112b4 112b7 112ba 112bf
112c2 112c5 112c8 112c9 112ce 112cf 1 112d1
112d6 112da 112de 112e1 112e6 112eb 112ee 112f2
112f3 112f8 112f9 112fe 11302 11306 11309 1130e
11313 11316 1131a 1131e 11321 11322 11327 1132a
1132f 11330 11335 11338 1133c 11340 11343 11344
11349 1134a 1134f 11351 11355 11358 1135c 11360
11364 11368 1136c 11370 11374 11378 1137c 11388
1138d 11391 11395 11399 1139d 1139f 113a0 113a5
1 113a9 113ad 113b1 113b4 113b8 113bc 113c0
113c4 113c7 113cb 113cf 113d2 113d5 113da 113dd
113e0 113e3 113e4 113e9 113ea 1 113ec 113f1
113f5 113f9 113fc 11401 11406 11409 1140d 1140e
11413 11414 11419 1141d 11421 11424 11429 1142e
11431 11435 11436 1143b 1143e 11443 11444 11449
1144c 11450 11454 11457 1145a 1145b 1145d 1145e
11463 11464 11469 1146b 1146f 11472 11476 1147a
1147e 11482 11486 1148a 1148e 1149a 1149f 114a3
114a7 114ab 114af 114b1 114b5 114b7 114c3 114c5
114c7 114c8 114cd 114d1 114d3 114df 114e3 114e5
114e6 114ef 
4f32
2
0 1 a 6 d :3 6 14 :3 6
10 :3 6 14 :3 6 15 :3 6 10 :3 6
e :3 6 11 :3 6 c :3 6 b :3 6
11 :3 6 11 :3 6 1d :3 6 10 :3 6
12 :2 6 1 3 a 13 a :2 1
4 9 12 11 :2 9 :2 4 12 1b
1a :2 12 :2 4 f 18 17 :2 f :2 4
10 19 18 :2 10 :2 4 14 1d 1c
:2 14 :2 4 18 21 20 :2 18 :2 4 18
21 20 :2 18 :2 4 18 21 20 :2 18
:2 4 e 17 16 :2 e :2 4 d 16
15 :2 d :2 4 11 1a 19 :2 11 :2 4
11 1a 19 :2 11 :2 4 :3 11 :2 4 :3 12
:2 4 :3 14 :2 4 :3 15 :2 4 f 18 17
:2 f :2 4 :3 e :2 4 :3 f :2 4 f 18
17 :2 f :2 4 b 14 13 :2 b :2 4
:3 f :2 4 :3 10 :2 4 :3 12 :2 4 :3 13 :2 4
e 17 16 :2 e :2 4 10 19 18
:2 10 :2 4 9 12 11 :2 9 :2 4 13
1c 1b :2 13 :2 4 11 1a 19 :2 11
:2 4 :2 12 21 12 :2 4 :2 c 1b c
:2 4 :2 d 1c d :2 4 :2 a 19 a
:2 4 :2 b 1a b :2 4 :2 e 1d e
:2 4 :2 c 1b c :2 4 :2 b 1a b
:2 4 :2 11 20 11 :2 4 :2 c 1b c
:2 4 :2 b 1a b :2 4 :2 e 1d e
:2 4 :2 f 1e f :2 4 :2 10 1f 10
:2 4 :2 d 1c d :2 4 :2 f 1e f
:2 4 :2 10 1f 10 :2 4 :2 e 1d e
:2 4 :2 e 1d e :2 4 :2 f 1e f
:2 4 :2 d 1c d :2 4 :2 f 1e f
:2 4 :2 f 1e f :2 4 :2 b 1a b
:2 4 :2 c 1b c :2 4 :2 c 1b c
:2 4 :2 16 25 16 :2 4 :2 f 19 f
:2 4 :2 f 19 f :2 4 :2 10 1a 10
:2 4 :2 10 1a 10 :2 4 :2 13 1d 13
:2 4 :2 12 21 12 :2 4 :2 19 28 19
:2 4 :2 19 28 19 :2 4 :2 19 28 19
:2 4 :2 f 1e f :2 4 :2 11 20 11
:2 4 :2 11 20 11 :2 4 :2 d 1c d
:2 4 10 19 18 :2 10 :2 4 11 1a
19 :2 11 :2 4 :2 14 1e 14 :2 4 :2 f
1e f :2 4 14 1d 1c :2 14 :2 4
:2 17 26 17 :2 4 17 20 1f :2 17
:2 4 13 1c 1b :2 13 :2 4 :2 13 22
13 :2 4 :2 14 23 14 :2 4 :3 9 :2 4
:3 9 :2 4 :2 f 19 f :2 4 :2 f 19
f :2 4 :2 f 1e f :2 4 :2 f 1e
f :2 4 :2 f 1e f :2 4 :2 16 25
16 :2 4 :2 11 20 11 :2 4 :2 11 20
11 :2 4 :2 15 24 15 :2 4 :2 10 1f
10 :2 4 :2 14 23 14 :2 4 14 1d
1c :2 14 :2 4 14 1d 1c :2 14 :2 4
18 21 20 :2 18 :2 4 :2 d 26 30
39 44 :2 30 :2 26 25 d :2 4 :2 10
28 10 :2 4 :2 b 23 b 4 7
e 0 :2 7 11 24 11 22 2f
a :6 7 e 0 :2 7 11 1f 11
22 2f 40 4e 5b :2 a :6 7 e
0 :2 7 11 13 26 28 3b 3d
46 48 56 58 66 68 a 12
19 1b 33 3b 42 44 5c 5e
6d 6f 89 8b a5 a7 bd bf
a c 23 25 2f 31 42 44
52 54 a c 16 18 20 22
2a 2c 37 39 f 17 1a 2f
32 43 10 12 1f 30 32 40
51 53 e 10 24 26 3c 3e
4b 4d e 10 21 2a 32 3b
49 52 60 69 77 80 8d 96
a7 b0 c3 cc e1 ea 105 10e
126 12f 140 149 15b 164 17c 185
198 1a1 1c2 1c4 1ce 13 15 a
:5 7 4 e 17 :3 e 4 7 :2 11
23 :2 2d 3a :2 23 :3 7 :2 d 19 25
:2 2f 43 :2 25 56 5c 5f :2 56 6d
85 8 :2 6d d 10 :2 6d 17 1a
:2 6d b e :2 6d 13 16 :2 6d 1b
1e :2 6d 2c 2f :2 6d b e :2 6d
13 16 :2 6d 1b 1e :2 6d 28 2b
:2 6d b e :2 6d 13 16 :2 6d 1b
1e :2 6d 2c 2f :2 6d b e :2 6d
13 16 :2 6d 1b 1e :2 6d 2d 30
:2 6d b e :2 6d 13 16 1f 27
:2 1f 33 :2 16 :2 6d b e :2 6d 13
16 1f 27 :2 1f 31 :2 16 :2 6d b
e :2 6d 13 16 1f 27 :2 1f 34
:2 16 :2 6d b e :2 6d 13 16 1f
27 :2 1f 2f :2 16 :2 6d b e :2 6d
13 16 1f 27 :2 1f 2e :2 16 :2 6d
b e :2 6d 13 16 1f 27 :2 1f
34 :2 16 :2 6d b e :2 6d 13 16
1f 27 :2 1f 34 :2 16 :2 6d b e
:2 6d 13 16 1f 27 :2 1f 40 :2 16
:2 6d b e :2 6d 13 16 1f 27
:2 1f 33 :2 16 :2 6d b e :2 6d 13
16 :2 6d 1b 1e :2 6d 2a 2d :2 6d
32 35 :2 6d :2 7 4c :2 4 7 :2 11
23 :2 2d 39 3c 4e 51 52 :2 4e
:2 23 :3 7 :2 d 12 1e 5f 62 :2 1e
:2 7 55 :3 4 11 :2 4 14 :2 4 10
:2 4 12 :2 4 12 :2 4 12 :2 4 12
19 22 2e :2 19 32 :2 3b :2 12 4
8 11 1d :2 8 22 20 :2 2b :2 20
7 40 e 1a :2 40 1f 1d :2 28
:2 1d 3f :3 7 15 7 3e :3 4 13
:2 4 14 :2 4 12 :2 4 14 :2 4 15
:2 4 15 :2 4 16 :2 4 9 :3 4 a
16 :2 4 7 e :2 7 :2 10 1c 7
14 7 :2 10 1c 7 :5 4 f :2 18
4 7 d :3 7 :2 10 1c 7 4
c a :2 13 1f a 1a :2 7 :2 c
a :2 13 1f a d :2 17 29 :2 33
3f 43 55 58 59 :2 55 :2 29 :3 d
:2 13 18 24 65 68 :2 24 75 78
:2 24 7c 7f :2 24 8b 8e :2 24 96
99 :2 24 a5 a8 af b8 bb :2 a8
:2 24 :2 d 5c :2 a :2 7 :4 13 :2 7 4
:3 1 7 10 19 :2 7 1c 1e :2 1c
a :2 14 26 :2 30 3c 40 52 55
56 :2 52 :2 26 :3 a :2 10 15 21 62
65 :2 21 72 75 :2 21 79 7c :2 21
86 89 92 9a :2 a3 :2 92 b6 :2 89
:2 21 be c1 :2 21 c9 cc :2 21 d1
d4 :2 21 dd e0 :2 21 :2 a 59 :2 7
a :2 14 26 :2 30 3c 3f 51 54
55 :2 51 :2 26 :3 a :2 10 15 21 62
65 :2 21 :2 a 58 :3 7 :2 12 26 :2 2f
41 4b :2 7 22 :2 4 e 17 1d
29 38 47 52 c 17 1f 2d
3e 4f 5c c 1a 27 :2 7 :2 10
1c 7 4 c a :2 13 1f a
1a :2 7 :2 c a :2 13 1f a d
:2 17 29 :2 33 3f 43 55 58 59
:2 55 :2 29 :3 d :2 13 18 24 65 68
:2 24 75 78 :2 24 7c 7f :2 24 8b
8e :2 24 96 99 :2 24 a5 a8 af
b8 bb :2 a8 :2 24 :2 d 5c :2 a :2 7
:4 13 :2 7 4 :3 1 8 :2 11 1c 1a
:2 25 :2 1a 7 a :2 14 26 :2 30 3c
40 52 55 56 :2 52 :2 26 :3 a :2 10
15 21 62 65 :2 21 72 75 :2 21
79 7c :2 21 86 89 92 9a :2 a3
:2 92 b6 :2 89 :2 21 be c1 :2 21 c9
cc :2 21 d1 d4 :2 21 de e1 :2 21
:2 a 59 :2 7 a :2 14 26 :2 30 3c
3f 51 54 55 :2 51 :2 26 :3 a :2 10
15 21 62 65 :2 21 :2 a 58 :3 7
:2 12 26 :2 2f 41 4c :2 7 35 :3 4
12 4 e 1e c d 1a :2 7
:2 10 1c 7 4 c a :2 13 1f
a 1a :2 7 :2 c a :2 13 1f a
d :2 17 29 :2 33 3f 43 55 58
59 :2 55 :2 29 :3 d :2 13 18 24 65
68 :2 24 75 78 :2 24 7c 7f :2 24
8b 8e :2 24 96 99 :2 24 a5 a8
af b8 bb :2 a8 :2 24 :2 d 5c :2 a
:2 7 :4 13 :2 7 4 :3 1 7 10 1c
:2 7 1f 21 :2 1f a :2 14 26 :2 30
3c 40 52 55 56 :2 52 :2 26 :3 a
:2 10 15 21 62 65 :2 21 72 75
:2 21 79 7c :2 21 86 89 92 9a
:2 a3 :2 92 b9 :2 89 :2 21 c1 c4 :2 21
cc cf :2 21 d4 d7 :2 21 e4 e7
:2 21 :2 a 59 :2 7 a :2 14 26 :2 30
3c 3f 51 54 55 :2 51 :2 26 :3 a
:2 10 15 21 62 65 :2 21 :2 a 58
:3 7 :2 12 26 :2 2f 44 52 :2 7 23
:2 4 e 11 24 27 4a 53 56
6c 75 78 90 93 9d a0 ad
b8 c2 c5 c 14 22 2d 39
45 54 c 14 17 1c 2b 31
34 42 45 56 59 66 :2 7 :2 10
1c 7 4 c a :2 13 1f a
1a :2 7 :2 c a :2 13 1f a d
:2 17 29 :2 33 3f 43 55 58 59
:2 55 :2 29 :3 d :2 13 18 24 65 68
:2 24 75 78 :2 24 7c 7f :2 24 8b
8e :2 24 96 99 :2 24 a5 a8 af
b8 bb :2 a8 :2 24 :2 d 5c :2 a :2 7
:4 13 :2 7 4 :3 1 8 :2 11 1c 1a
:2 25 :2 1a 7 a :2 14 26 :2 30 3c
40 52 55 56 :2 52 :2 26 :3 a :2 10
15 21 62 65 :2 21 72 75 :2 21
79 7c :2 21 86 89 92 9a :2 a3
:2 92 b6 :2 89 :2 21 be c1 :2 21 c9
cc :2 21 d1 d4 :2 21 de e1 :2 21
:2 a 59 :2 7 a :2 14 26 :2 30 3c
3f 51 54 55 :2 51 :2 26 :3 a :2 10
15 21 62 65 :2 21 :2 a 58 :3 7
:2 12 26 :2 2f 41 4c :2 7 35 :2 4
e 18 2b 37 49 59 67 :2 7
:2 10 1c 7 4 c a :2 13 1f
a 1a :2 7 :2 c a :2 13 1f a
d :2 17 29 :2 33 3f 43 55 58
59 :2 55 :2 29 :3 d :2 13 18 24 65
68 :2 24 75 78 :2 24 7c 7f :2 24
8b 8e :2 24 96 99 :2 24 a5 a8
af b8 bb :2 a8 :2 24 :2 d 5c :2 a
:2 7 :4 13 :2 7 4 :3 1 8 :2 11 1c
1a :2 25 :2 1a 7 a :2 14 26 :2 30
3c 40 52 55 56 :2 52 :2 26 :3 a
:2 10 15 21 62 65 :2 21 72 75
:2 21 79 7c :2 21 86 89 92 9a
:2 a3 :2 92 b6 :2 89 :2 21 be c1 :2 21
c9 cc :2 21 d1 d4 :2 21 dd e0
:2 21 :2 a 59 :2 7 a :2 14 26 :2 30
3c 3f 51 54 55 :2 51 :2 26 :3 a
:2 10 15 21 62 65 :2 21 :2 a 58
:3 7 :2 12 26 :2 2f 41 4b :2 7 35
:2 4 e 1b 2a 39 46 :2 7 :2 10
1c 7 4 c a :2 13 1f a
1a :2 7 :2 c a :2 13 1f a d
:2 17 29 :2 33 3f 43 55 58 59
:2 55 :2 29 :3 d :2 13 18 24 65 68
:2 24 75 78 :2 24 7c 7f :2 24 8b
8e :2 24 96 99 :2 24 a5 a8 af
b8 bb :2 a8 :2 24 :2 d 5c :2 a :2 7
:4 13 :2 7 4 :3 1 7 :2 12 19 24
:2 7 34 36 :2 34 7 14 7 38
:2 4 8 :2 11 1c 1a :2 25 :2 1a 7
a :2 14 26 :2 30 3c 40 52 55
56 :2 52 :2 26 :3 a :2 10 15 21 62
65 :2 21 72 75 :2 21 79 7c :2 21
86 89 92 9a :2 a3 :2 92 b6 :2 89
:2 21 be c1 :2 21 c9 cc :2 21 d1
d4 :2 21 e1 e4 :2 21 :2 a 59 :2 7
a :2 14 26 :2 30 3c 3f 51 54
55 :2 51 :2 26 :3 a :2 10 15 21 62
65 :2 21 :2 a 58 :3 7 :2 12 26 :2 2f
41 4f :2 7 35 :3 4 1c :2 4 1c
23 2c 42 :2 23 46 :2 1c :2 4 12
19 22 38 :2 19 3c :2 12 :2 4 1c
23 2c 42 :2 23 46 :2 1c 4 7
10 13 :2 10 a 13 1f :2 a 22
24 :2 22 14 1a 32 42 13 20
31 3b d 1b 28 :2 d :2 16 22
d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 5b 5e 5f :2 5b
:2 2f :3 13 :2 19 1e 2a 6b 6e :2 2a
7b 7e :2 2a 82 85 :2 2a 91 94
:2 2a 9c 9f :2 2a ab ae b5 be
c1 :2 ae :2 2a :2 13 62 :2 10 :2 d :4 19
:2 d a :3 26 7 26 d 16 22
:2 d 25 27 :2 25 14 1a 32 42
13 20 31 3c d 1b 28 :2 d
:2 16 22 d a 12 10 :2 19 25
10 20 :2 d :2 12 10 :2 19 25 10
13 :2 1d 2f :2 39 45 49 5b 5e
5f :2 5b :2 2f :3 13 :2 19 1e 2a 6b
6e :2 2a 7b 7e :2 2a 82 85 :2 2a
91 94 :2 2a 9c 9f :2 2a ab ae
b5 be c1 :2 ae :2 2a :2 13 62 :2 10
:2 d :4 19 :2 d a :4 29 26 :2 7 a
13 1f :2 a 22 24 :2 22 d :2 17
29 :2 33 3f 43 55 58 59 :2 55
:2 29 :3 d :2 13 18 24 65 68 :2 24
75 78 :2 24 7c 7f :2 24 89 8c
95 9d :2 a6 :2 95 c7 :2 8c :2 24 cf
d2 :2 24 da dd :2 24 e2 e5 :2 24
ee f1 :2 24 :2 d 5c :2 a d :2 17
29 :2 33 3f 42 54 57 58 :2 54
:2 29 :3 d :2 13 18 24 65 68 :2 24
:2 d 5b :3 a :2 15 29 :2 32 52 12
:2 a 26 :2 7 15 11 17 2f 3f
10 1d 2e 3c 48 56 67 :2 a
:2 13 1f a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 58 5b
5c :2 58 :2 2c :3 10 :2 16 1b 27 68
6b :2 27 78 7b :2 27 7f 82 :2 27
8e 91 :2 27 99 9c :2 27 a8 ab
b2 bb be :2 ab :2 27 :2 10 5f :2 d
:2 a :4 16 :2 a 7 :3 4 11 17 2f
3f 10 1d 2e 3b 47 55 66
:2 a :2 13 1f a 7 f d :2 16
22 d 1d :2 a :2 f d :2 16 22
d 10 :2 1a 2c :2 36 42 46 58
5b 5c :2 58 :2 2c :3 10 :2 16 1b 27
68 6b :2 27 78 7b :2 27 7f 82
:2 27 8e 91 :2 27 99 9c :2 27 a8
ab b2 bb be :2 ab :2 27 :2 10 5f
:2 d :2 a :4 16 :2 a 7 :3 4 b 19
22 2d :2 19 :2 b a 32 35 :2 32
3d 46 52 :2 3d 55 57 :2 55 3c
5e d 19 :2 5e 1c 1e :2 1c 5d
:2 3c 3b :2 a 14 1a 32 42 13
20 31 3f 4c 11 :2 d :2 16 22
d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 5b 5e 5f :2 5b
:2 2f :3 13 :2 19 1e 2a 6b 6e :2 2a
7b 7e :2 2a 82 85 :2 2a 91 94
:2 2a 9c 9f :2 2a ab ae b5 be
c1 :2 ae :2 2a :2 13 62 :2 10 :2 d :4 19
:2 d a :3 22 d 16 22 :2 d 25
27 :2 25 10 :2 1a 2c :2 36 42 46
58 5b 5c :2 58 :2 2c :3 10 :2 16 1b
27 68 6b :2 27 78 7b :2 27 7f
82 :2 27 8c 8f 98 a0 :2 a9 :2 98
ca :2 8f :2 27 d2 d5 :2 27 dd e0
:2 27 e5 e8 :2 27 f1 f4 :2 27 :2 10
5f :2 d 10 :2 1a 2c :2 36 42 45
57 5a 5b :2 57 :2 2c :3 10 :2 16 1b
27 68 6b :2 27 :2 10 5e :3 d :2 18
2c :2 35 55 15 :2 d 29 :2 a 22
:2 7 :5 4 12 :2 4 13 :2 4 17 :2 4
1b :2 4 14 :2 4 9 :3 4 a 14
:2 4 7 c :2 7 :2 10 1c 7 12
7 :2 10 1c 7 :4 4 8 :2 11 1c
1a :2 25 :2 1a :2 7 15 7 35 7
15 7 :4 4 7 d :3 7 :2 10 1c
7 4 c a :2 13 1f a 1a
:2 7 :2 c a :2 13 1f a d :2 17
29 :2 33 3f 43 55 58 59 :2 55
:2 29 :3 d :2 13 18 24 65 68 :2 24
75 78 :2 24 7c 7f :2 24 8b 8e
:2 24 96 99 :2 24 a5 a8 af b8
bb :2 a8 :2 24 :2 d 5c :2 a :2 7 :4 13
:2 7 4 :3 1 7 10 1c :2 7 1f
21 :2 1f 11 17 2f 3c 53 60
a 18 25 3d 47 :2 a :2 13 1f
a 7 f d :2 16 22 d 1d
:2 a :2 f d :2 16 22 d 10 :2 1a
2c :2 36 42 46 58 5b 5c :2 58
:2 2c :3 10 :2 16 1b 27 68 6b :2 27
78 7b :2 27 7f 82 :2 27 8e 91
:2 27 99 9c :2 27 a8 ab b2 bb
be :2 ab :2 27 :2 10 5f :2 d :2 a :4 16
:2 a 7 :3 23 a 13 1c :2 a 1f
21 :2 1f a 18 a 14 23 37
46 12 29 36 47 d 1b 28
40 4a :2 d :2 16 22 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 5b 5e 5f :2 5b :2 2f :3 13 :2 19
1e 2a 6b 6e :2 2a 7b 7e :2 2a
82 85 :2 2a 91 94 :2 2a 9c 9f
:2 2a ab ae b5 be c1 :2 ae :2 2a
:2 13 62 :2 10 :2 d :4 19 :2 d a :3 23
e :2 17 22 20 :2 2b :2 20 d 10
:2 1a 2c :2 36 42 46 58 5b 5c
:2 58 :2 2c :3 10 :2 16 1b 27 68 6b
:2 27 78 7b :2 27 7f 82 :2 27 8c
8f 98 a0 :2 a9 :2 98 ca :2 8f :2 27
d2 d5 :2 27 dd e0 :2 27 e5 e8
:2 27 f1 f4 :2 27 :2 10 5f :2 d 10
:2 1a 2c :2 36 42 45 57 5a 5b
:2 57 :2 2c :3 10 :2 16 1b 27 68 6b
:2 27 :2 10 5e :3 d :2 18 2c :2 35 55
15 :2 d a 3b :4 12 11 10 :4 34
33 32 :2 10 11 :2 1c 24 33 41
:2 11 45 47 :2 45 10 4e :2 59 61
15 23 :2 4e 27 29 :2 27 4d :2 10
13 :2 1d 2f :2 39 45 49 5b 5e
5f :2 5b :2 2f :3 13 :2 19 1e 2a 6b
6e :2 2a 7b 7e :2 2a 82 85 :2 2a
8f 92 9b a3 :2 ac :2 9b cd :2 92
:2 2a d5 d8 :2 2a e0 e3 :2 2a e8
eb :2 2a f4 f7 :2 2a :2 13 62 :2 10
13 :2 1d 2f :2 39 45 48 5a 5d
5e :2 5a :2 2f :3 13 :2 19 1e 2a 6b
6e :2 2a :2 13 61 :3 10 :2 1b 2f :2 38
58 18 :2 10 2c :2 d 51 3b :2 a
23 :2 7 a 13 1f :2 a 22 24
:2 22 a 18 :2 a 21 a 26 14
27 38 49 12 13 20 11 21
11 1f :2 d :2 16 22 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 5b 5e 5f :2 5b :2 2f :3 13 :2 19
1e 2a 6b 6e :2 2a 7b 7e :2 2a
82 85 :2 2a 91 94 :2 2a 9c 9f
:2 2a ab ae b5 be c1 :2 ae :2 2a
:2 13 62 :2 10 :2 d :4 19 :2 d a :3 7
e :2 17 22 20 :2 2b :2 20 :2 d 20
:2 d :3 20 :2 d 20 2e 31 3a 42
:2 31 46 :2 4f 24 :2 20 :2 d 23 :2 2c
:2 d 20 :2 d 1f :2 d 20 37 45
4f 57 2d 41 52 :2 20 :2 d 1d
26 2f 37 :2 26 :2 1d d 1c 31
44 50 5c 10 1f 2c 38 45
52 10 1a 18 29 37 45 10
1c 26 34 3d 4a 5e 10 1d
10 17 13 1f :3 1d 13 :2 1c 28
13 24 13 :2 1c 28 13 :4 10 d
15 13 :2 1c 28 13 23 :2 10 :2 15
13 :2 1c 28 13 16 :2 20 32 :2 3c
48 4c 5e 61 62 :2 5e :2 32 :3 16
:2 1c 21 2d 6e 71 :2 2d 7e 81
:2 2d 85 88 :2 2d 94 97 :2 2d 9f
a2 :2 2d ae b1 b8 c1 c4 :2 b1
:2 2d :2 16 65 :2 13 :2 10 :4 1c :2 10 d
:3 3b 17 29 3d 4c 59 :2 10 :2 19
25 10 d 15 13 :2 1c 28 13
23 :2 10 :2 15 13 :2 1c 28 13 16
:2 20 32 :2 3c 48 4c 5e 61 62
:2 5e :2 32 :3 16 :2 1c 21 2d 6e 71
:2 2d 7e 81 :2 2d 85 88 :2 2d 94
97 :2 2d 9f a2 :2 2d ae b1 b8
c1 c4 :2 b1 :2 2d :2 16 65 :2 13 :2 10
:4 1c :2 10 d :3 3b 10 :2 1b 22 30
:2 10 3b 3d :2 3b 10 1b 10 3f
1a 20 38 45 19 26 37 42
50 68 76 87 :2 13 :2 1c 28 13
10 18 16 :2 1f 2b 16 26 :2 13
:2 18 16 :2 1f 2b 16 19 :2 23 35
:2 3f 4b 4f 61 64 65 :2 61 :2 35
:3 19 :2 1f 24 30 71 74 :2 30 81
84 :2 30 88 8b :2 30 97 9a :2 30
a2 a5 :2 30 b1 b4 bb c4 c7
:2 b4 :2 30 :2 19 68 :2 16 :2 13 :4 1f :2 13
10 :7 d 10 19 22 :2 10 25 27
:2 25 1a 26 35 3e 4c 5c 19
26 37 44 13 1a 16 22 :3 20
16 :2 1f 2b 16 27 16 :2 1f 2b
16 :4 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 61 64 65
:2 61 :2 35 :3 19 :2 1f 24 30 71 74
:2 30 81 84 :2 30 88 8b :2 30 97
9a :2 30 a2 a5 :2 30 b1 b4 bb
c4 c7 :2 b4 :2 30 :2 19 68 :2 16 :2 13
:4 1f :2 13 10 :3 29 1a 26 18 1f
2d 4a 18 1f 2e 35 4f 18
28 19 26 37 44 13 1a 16
22 :3 20 16 :2 1f 2b 16 27 16
:2 1f 2b 16 :4 13 10 18 16 :2 1f
2b 16 26 :2 13 :2 18 16 :2 1f 2b
16 19 :2 23 35 :2 3f 4b 4f 61
64 65 :2 61 :2 35 :3 19 :2 1f 24 30
71 74 :2 30 81 84 :2 30 88 8b
:2 30 97 9a :2 30 a2 a5 :2 30 b1
b4 bb c4 c7 :2 b4 :2 30 :2 19 68
:2 16 :2 13 :4 1f :2 13 10 :4 29 1a 26
18 1f 2d 4a 18 1f 2e 35
4f 18 28 19 26 37 44 13
1a 16 22 :3 20 16 :2 1f 2b 16
27 16 :2 1f 2b 16 :4 13 10 18
16 :2 1f 2b 16 26 :2 13 :2 18 16
:2 1f 2b 16 19 :2 23 35 :2 3f 4b
4f 61 64 65 :2 61 :2 35 :3 19 :2 1f
24 30 71 74 :2 30 81 84 :2 30
88 8b :2 30 97 9a :2 30 a2 a5
:2 30 b1 b4 bb c4 c7 :2 b4 :2 30
:2 19 68 :2 16 :2 13 :4 1f :2 13 10 :7 d
a 3b 11 :2 1a 25 23 :2 2e :2 23
10 17 2e 47 16 2a 3e :2 10
:2 19 25 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 5e 61
62 :2 5e :2 32 :3 16 :2 1c 21 2d 6e
71 :2 2d 7e 81 :2 2d 85 88 :2 2d
94 97 :2 2d 9f a2 :2 2d ae b1
b8 c1 c4 :2 b1 :2 2d :2 16 65 :2 13
:2 10 :4 1c :2 10 d :3 3e 10 19 22
:2 10 27 25 :2 30 :2 25 10 23 31
34 3d 45 :2 34 49 :2 52 24 :2 23
10 1a 33 40 13 21 2a 37
45 19 2d 13 1a 16 22 :3 20
16 :2 1f 2b 16 27 16 :2 1f 2b
16 :4 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 61 64 65
:2 61 :2 35 :3 19 :2 1f 24 30 71 74
:2 30 81 84 :2 30 88 8b :2 30 97
9a :2 30 a2 a5 :2 30 b1 b4 bb
c4 c7 :2 b4 :2 30 :2 19 68 :2 16 :2 13
:4 1f :2 13 10 :3 3f 10 23 39 :2 23
10 d 3f 14 1b 24 2d :2 1b
31 :2 3a :2 14 13 4b 48 :2 54 :2 48
1a 33 40 13 1f 28 2f 3c
45 19 2d 13 1a 16 22 :3 20
16 :2 1f 2b 16 27 16 :2 1f 2b
16 :4 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 61 64 65
:2 61 :2 35 :3 19 :2 1f 24 30 71 74
:2 30 81 84 :2 30 88 8b :2 30 97
9a :2 30 a2 a5 :2 30 b1 b4 bb
c4 c7 :2 b4 :2 30 :2 19 68 :2 16 :2 13
:4 1f :2 13 10 :3 60 10 23 39 :2 23
10 60 3f 1a 33 40 13 1f
28 2f 3c 45 19 2d 13 1a
16 22 :3 20 16 :2 1f 2b 16 27
16 :2 1f 2b 16 :4 13 10 18 16
:2 1f 2b 16 26 :2 13 :2 18 16 :2 1f
2b 16 19 :2 23 35 :2 3f 4b 4f
61 64 65 :2 61 :2 35 :3 19 :2 1f 24
30 71 74 :2 30 81 84 :2 30 88
8b :2 30 97 9a :2 30 a2 a5 :2 30
b1 b4 bb c4 c7 :2 b4 :2 30 :2 19
68 :2 16 :2 13 :4 1f :2 13 10 :3 d 10
23 39 :2 23 10 :4 d 3e 3b :2 a
:4 7 23 7 1e 7 :5 4 13 :2 4
14 :2 4 15 :2 4 :3 17 4 18 28
2b c 1d 28 32 35 :2 c 16
19 1e 2d 33 36 44 47 56
59 d 19 :2 7 :2 10 1c 7 4
c a :2 13 1f a 1a :2 7 :2 c
a :2 13 1f a d :2 17 29 :2 33
3f 43 55 58 59 :2 55 :2 29 :3 d
:2 13 18 24 65 68 :2 24 75 78
:2 24 7c 7f :2 24 8b 8e :2 24 96
99 :2 24 a5 a8 af b8 bb :2 a8
:2 24 :2 d 5c :2 a :2 7 :4 13 :2 7 4
:3 1 8 :2 11 1d 1a :2 26 :2 1a :2 7
19 7 36 :2 4 8 11 27 :2 8
2a 2c :2 2a 7 33 3c 52 :2 33
55 57 :2 55 32 :3 7 1a 2b 39
43 4f 5c 27 34 40 4d 59
:2 1a 7 a 13 29 :2 a 2c 2e
:2 2c a 18 21 2d :2 18 30 32
3b 4b :2 32 :2 18 :2 a 19 22 2f
:2 19 32 34 3d 4d :2 34 :2 19 a
30 :2 7 a 13 29 :2 a 2c 2e
:2 2c a 18 21 2d :2 18 30 32
3b 4b :2 32 :2 18 :2 a 19 22 2f
:2 19 32 34 3d 4d :2 34 :2 19 a
30 :2 7 5a :3 4 10 :2 4 f :2 4
1b 4 e 28 4a 56 c 22
2f :2 7 :2 10 1c 7 4 c a
:2 13 1f a 1a :2 7 :2 c a :2 13
1f a d :2 17 29 :2 33 3f 43
55 58 59 :2 55 :2 29 :3 d :2 13 18
24 65 68 :2 24 75 78 :2 24 7c
7f :2 24 8b 8e :2 24 96 99 :2 24
a5 a8 af b8 bb :2 a8 :2 24 :2 d
5c :2 a :2 7 :4 13 :2 7 4 :3 1 7
:2 12 19 27 :2 7 32 35 :2 32 11
15 25 32 10 1d 2e 38 49
57 :2 a :2 13 1f a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
58 5b 5c :2 58 :2 2c :3 10 :2 16 1b
27 69 6c :2 27 79 7c :2 27 80
83 :2 27 8f 92 :2 27 9a 9d :2 27
a9 ac b3 bc bf :2 ac :2 27 :2 10
5f :2 d :2 a :4 16 :2 a 7 :3 37 a
13 1c :2 a 1f 21 :2 1f a 17
a 23 a 15 a :4 7 37 :2 4
7 10 1c :2 7 1f 21 :2 1f a
:2 15 1c 2a :2 a 35 37 :2 35 14
26 38 46 4e 55 6d 75 7c
95 9f d 1b 29 32 3d 47
4f 57 12 27 2d 3f 4a 56
62 d 19 25 30 41 49 52
5a 12 13 1d 2e 3c 4d :2 d
:2 16 22 d a 12 10 :2 19 25
10 20 :2 d :2 12 10 :2 19 25 10
13 :2 1d 2f :2 39 45 49 5b 5e
5f :2 5b :2 2f :3 13 :2 19 1e 2a 6c
6f :2 2a 7c 7f :2 2a 83 86 :2 2a
92 95 :2 2a 9d a0 :2 2a ac af
b6 bf c2 :2 af :2 2a :2 13 62 :2 10
:2 d :4 19 :2 d a :3 39 a 21 :2 a
1f a d 16 22 :2 d 25 27
:2 25 11 1a 25 :2 11 2b 28 34
40 :2 2b :2 28 10 48 51 59 :2 48
5f 5c 28 31 :2 5f :2 5c 47 :2 10
14 1d 29 :2 14 2c 2e :2 2c 13
37 3e 16 22 :2 3e 26 :2 2f :2 37
36 3d 40 :2 3d 35 :2 13 16 :2 20
32 :2 3c 48 4c 5e 62 63 :2 5e
:2 32 :3 16 :2 1c 21 2d 70 73 :2 2d
80 83 :2 2d 87 8a :2 2d 94 97
a0 a8 :2 b1 :2 a0 c9 :2 97 :2 2d d1
d4 :2 2d dc df :2 2d e4 e7 :2 2d
f0 f3 :2 2d :2 16 66 :2 13 16 :2 20
32 :2 3c 48 4b 5d 61 62 :2 5d
:2 32 :3 16 :2 1c 21 2d 70 73 :2 2d
:2 16 65 :3 13 :2 1e 32 :2 3b 52 5c
:2 13 43 13 28 13 16 1f 2b
:2 16 2e 30 :2 2e 16 29 16 32
:2 13 :4 10 35 :3 d 19 :2 d 1d :2 d
1e :2 d 17 :2 d 1d :2 d 18 :2 d
17 :2 d 1a :2 d 1a d 29 :2 a
e :2 17 22 20 :2 2b :2 20 d :4 41
40 3f :2 d 10 17 27 2a :2 10
2d 2f :2 2d 10 20 21 :2 20 :2 10
:2 19 25 3c 4a 58 2f 3a 46
52 2f 3d 50 :2 25 10 33 :2 d
10 19 25 :2 10 28 2a :2 28 1f
36 4a 13 1a 16 22 :3 20 16
:2 1f 2b 16 27 16 :2 1f 2b 16
:4 13 10 18 16 :2 1f 2b 16 26
:2 13 :2 18 16 :2 1f 2b 16 19 :2 23
35 :2 3f 4b 4f 61 65 66 :2 61
:2 35 :3 19 :2 1f 24 30 73 76 :2 30
83 86 :2 30 8a 8d :2 30 99 9c
:2 30 a4 a7 :2 30 b3 b6 bd c6
c9 :2 b6 :2 30 :2 19 69 :2 16 :2 13 :4 1f
:2 13 10 :3 2c 1f 36 43 54 68
13 1a 16 22 :3 20 16 :2 1f 2b
16 27 16 :2 1f 2b 16 :4 13 10
18 16 :2 1f 2b 16 26 :2 13 :2 18
16 :2 1f 2b 16 19 :2 23 35 :2 3f
4b 4f 61 65 66 :2 61 :2 35 :3 19
:2 1f 24 30 73 76 :2 30 83 86
:2 30 8a 8d :2 30 99 9c :2 30 a4
a7 :2 30 b3 b6 bd c6 c9 :2 b6
:2 30 :2 19 69 :2 16 :2 13 :4 1f :2 13 10
:3 2c 10 27 10 1a 1e 2d 3c
19 26 37 4b :2 13 :2 1c 28 13
10 18 16 :2 1f 2b 16 26 :2 13
:2 18 16 :2 1f 2b 16 19 :2 23 35
:2 3f 4b 4f 61 65 66 :2 61 :2 35
:3 19 :2 1f 24 30 73 76 :2 30 83
86 :2 30 8a 8d :2 30 99 9c :2 30
a4 a7 :2 30 b3 b6 bd c6 c9
:2 b6 :2 30 :2 19 69 :2 16 :2 13 :4 1f :2 13
10 :3 2c 13 :2 1e 26 34 3f :2 13
43 46 :2 43 1d 28 35 3d 44
61 6c 71 :2 1b :2 16 :2 1f 2b 16
13 1b 19 :2 22 2e 19 29 :2 16
:2 1b 19 :2 22 2e 19 1c :2 26 38
:2 42 4e 52 64 68 69 :2 64 :2 38
:3 1c :2 22 27 33 76 79 :2 33 86
89 :2 33 8d 90 :2 33 9c 9f :2 33
a7 aa :2 33 b6 b9 c0 c9 cc
:2 b9 :2 33 :2 1c 6c :2 19 :2 16 :4 22 :2 16
13 :4 48 :2 10 2c 14 1d 33 :2 14
36 38 :2 36 13 40 49 5f :2 40
62 64 :2 62 3f :2 13 16 1f 2b
:2 16 2e 30 :2 2e 20 35 49 19
27 31 19 28 19 28 19 28
19 28 19 2f 4a 59 19 33
40 55 19 27 30 37 44 4d
1f 33 19 20 1c 28 :3 26 1c
:2 25 31 1c 2d 1c :2 25 31 1c
:4 19 16 1e 1c :2 25 31 1c 2c
:2 19 :2 1e 1c :2 25 31 1c 1f :2 29
3b :2 45 51 55 67 6b 6c :2 67
:2 3b :3 1f :2 25 2a 36 79 7c :2 36
89 8c :2 36 90 93 :2 36 9f a2
:2 36 aa ad :2 36 b9 bc c3 cc
cf :2 bc :2 36 :2 1f 6f :2 1c :2 19 :4 25
:2 19 16 :4 32 20 35 49 19 27
31 19 28 19 28 19 28 19
28 19 2f 4a 59 19 33 40
55 19 36 43 5a 19 27 30
37 44 4d 1f 33 19 20 1c
28 :3 26 1c :2 25 31 1c 2d 1c
:2 25 31 1c :4 19 16 1e 1c :2 25
31 1c 2c :2 19 :2 1e 1c :2 25 31
1c 1f :2 29 3b :2 45 51 55 67
6b 6c :2 67 :2 3b :3 1f :2 25 2a 36
79 7c :2 36 89 8c :2 36 90 93
:2 36 9f a2 :2 36 aa ad :2 36 b9
bc c3 cc cf :2 bc :2 36 :2 1f 6f
:2 1c :2 19 :4 25 :2 19 16 :7 13 10 67
16 1f 35 :2 16 38 3a :2 38 1d
32 46 16 24 2e 16 25 16
25 16 2c 47 56 16 30 3d
52 16 24 2d 34 41 4a 1c
30 16 1d 19 25 :3 23 19 :2 22
2e 19 2a 19 :2 22 2e 19 :4 16
13 1b 19 :2 22 2e 19 29 :2 16
:2 1b 19 :2 22 2e 19 1c :2 26 38
:2 42 4e 52 64 68 69 :2 64 :2 38
:3 1c :2 22 27 33 76 79 :2 33 86
89 :2 33 8d 90 :2 33 9c 9f :2 33
a7 aa :2 33 b6 b9 c0 c9 cc
:2 b9 :2 33 :2 1c 6c :2 19 :2 16 :4 22 :2 16
13 :3 3c 10 3c 67 16 1f 35
:2 16 38 3a :2 38 13 21 2a 36
:2 21 39 3b 44 54 :2 3b :2 21 :2 13
22 2b 38 :2 22 3b 3d 46 56
:2 3d :2 22 13 16 1f 2b :2 16 2e
30 :2 2e 20 35 49 19 27 31
19 28 19 28 19 2f 4a 59
19 27 30 37 44 4d 1f 33
19 20 1c 28 :3 26 1c :2 25 31
1c 2d 1c :2 25 31 1c :4 19 16
1e 1c :2 25 31 1c 2c :2 19 :2 1e
1c :2 25 31 1c 1f :2 29 3b :2 45
51 55 67 6b 6c :2 67 :2 3b :3 1f
:2 25 2a 36 79 7c :2 36 89 8c
:2 36 90 93 :2 36 9f a2 :2 36 aa
ad :2 36 b9 bc c3 cc cf :2 bc
:2 36 :2 1f 6f :2 1c :2 19 :4 25 :2 19 16
:4 32 20 35 49 19 27 31 19
28 19 28 19 2f 4a 59 19
36 43 5a 19 27 30 37 44
4d 1f 33 19 20 1c 28 :3 26
1c :2 25 31 1c 2d 1c :2 25 31
1c :4 19 16 1e 1c :2 25 31 1c
2c :2 19 :2 1e 1c :2 25 31 1c 1f
:2 29 3b :2 45 51 55 67 6b 6c
:2 67 :2 3b :3 1f :2 25 2a 36 79 7c
:2 36 89 8c :2 36 90 93 :2 36 9f
a2 :2 36 aa ad :2 36 b9 bc c3
cc cf :2 bc :2 36 :2 1f 6f :2 1c :2 19
:4 25 :2 19 16 :7 13 3c 67 1d 32
46 16 2c 47 56 16 24 2e
1c 30 16 1d 19 25 :3 23 19
:2 22 2e 19 2a 19 :2 22 2e 19
:4 16 13 1b 19 :2 22 2e 19 29
:2 16 :2 1b 19 :2 22 2e 19 1c :2 26
38 :2 42 4e 52 64 68 69 :2 64
:2 38 :3 1c :2 22 27 33 76 79 :2 33
86 89 :2 33 8d 90 :2 33 9c 9f
:2 33 a7 aa :2 33 b6 b9 c0 c9
cc :2 b9 :2 33 :2 1c 6c :2 19 :2 16 :4 22
:2 16 13 :7 10 1f 36 43 54 68
13 1a 16 22 :3 20 16 :2 1f 2b
16 27 16 :2 1f 2b 16 :4 13 10
18 16 :2 1f 2b 16 26 :2 13 :2 18
16 :2 1f 2b 16 19 :2 23 35 :2 3f
4b 4f 61 65 66 :2 61 :2 35 :3 19
:2 1f 24 30 73 76 :2 30 83 86
:2 30 8a 8d :2 30 99 9c :2 30 a4
a7 :2 30 b3 b6 bd c6 c9 :2 b6
:2 30 :2 19 69 :2 16 :2 13 :4 1f :2 13 10
:3 d 1f 1a 2f 39 4f 5e 73
87 18 35 49 13 1a 16 22
:3 20 16 :2 1f 2b 16 27 16 :2 1f
2b 16 :4 13 10 18 16 :2 1f 2b
16 26 :2 13 :2 18 16 :2 1f 2b 16
19 :2 23 35 :2 3f 4b 4f 61 65
66 :2 61 :2 35 :3 19 :2 1f 24 30 73
76 :2 30 83 86 :2 30 8a 8d :2 30
99 9c :2 30 a4 a7 :2 30 b3 b6
bd c6 c9 :2 b6 :2 30 :2 19 69 :2 16
:2 13 :4 1f :2 13 10 :8 d 18 d 1c
37 4b 10 17 13 1f :3 1d 13
:2 1c 28 13 24 13 :2 1c 28 13
:4 10 d 15 13 :2 1c 28 13 23
:2 10 :2 15 13 :2 1c 28 13 16 :2 20
32 :2 3c 48 4c 5e 62 63 :2 5e
:2 32 :3 16 :2 1c 21 2d 70 73 :2 2d
80 83 :2 2d 87 8a :2 2d 96 99
:2 2d a1 a4 :2 2d b0 b3 ba c3
c6 :2 b3 :2 2d :2 16 66 :2 13 :2 10 :4 1c
:2 10 d :3 63 1c 36 43 54 62
10 17 13 1f :3 1d 13 :2 1c 28
13 24 13 :2 1c 28 13 :4 10 d
15 13 :2 1c 28 13 23 :2 10 :2 15
13 :2 1c 28 13 16 :2 20 32 :2 3c
48 4c 5e 62 63 :2 5e :2 32 :3 16
:2 1c 21 2d 70 73 :2 2d 80 83
:2 2d 87 8a :2 2d 96 99 :2 2d a1
a4 :2 2d b0 b3 ba c3 c6 :2 b3
:2 2d :2 16 66 :2 13 :2 10 :4 1c :2 10 d
:3 63 11 1a 26 :2 11 29 2b :2 29
10 33 3c 48 :2 33 4b 4d :2 4b
32 :3 10 1f :2 10 19 10 13 :2 1c
27 3a :2 13 3d 40 :2 3d 13 1f
13 42 13 1f 13 :4 10 13 1c
28 :2 13 2b 2d :2 2b 13 1d 13
2f :3 10 1e 2f 3b 48 51 5a
:2 48 5e 67 6f :2 5e 22 2b 37
:2 22 3b 44 4f :2 3b 53 20 2b
:2 53 2f 38 46 :2 2f 4a 53 5e
:2 4a 22 2b 37 :2 22 3b 43 4d
:2 1e 10 13 1c 28 :2 13 2d 2b
:2 36 :2 2b 16 :2 20 32 :2 3c 48 4b
5d 61 62 :2 5d :2 32 :3 16 :2 1c 21
2d 70 73 :2 2d 80 83 :2 2d 87
8a :2 2d 94 97 a0 a8 :2 b1 :2 a0
c2 :2 97 :2 2d :2 16 65 :2 13 16 :2 20
32 :2 3c 48 4b 5d 61 62 :2 5d
:2 32 :3 16 :2 1c 21 2d 70 73 :2 2d
:2 16 65 :3 13 :2 1e 34 :2 3d 4d :2 13
10 45 16 1f 2b :2 16 2e 31
:2 2e 16 :2 20 32 :2 3c 48 4c 5e
62 63 :2 5e :2 32 :3 16 :2 1c 21 2d
70 73 :2 2d 80 83 :2 2d 87 8a
:2 2d 94 97 a0 a8 :2 a0 b5 :2 97
:2 2d bd c0 :2 2d c8 cb :2 2d d0
d3 :2 2d e0 e3 :2 2d :2 16 66 :2 13
16 :2 20 32 :2 3c 48 4b 5d 61
62 :2 5d :2 32 :3 16 :2 1c 21 2d 70
73 :2 2d :2 16 65 :3 13 :2 1e 32 3b
47 :2 32 4b 59 :2 13 33 45 :2 10
13 1c 24 :2 13 27 29 :2 27 13
20 :2 29 13 2b 13 20 :2 29 30
39 44 :2 30 48 :2 51 :2 20 13 :5 10
23 :2 10 1c 28 3d 4b 59 63
1b 23 2f 3b 47 53 1b 2a
36 40 48 51 59 1b 27 33
40 4b 56 1b 25 30 3c 48
4b 4e 54 :2 1c 10 13 1c 26
:2 13 29 2b :2 29 16 :2 20 32 :2 3c
48 4c 5e 62 63 :2 5e :2 32 :3 16
:2 1c 21 2d 70 73 :2 2d 80 83
:2 2d 87 8a :2 2d 94 97 a0 a8
:2 b1 :2 a0 c8 :2 97 :2 2d d0 d3 :2 2d
db de :2 2d e3 e6 :2 2d ef f2
:2 2d :2 16 66 :2 13 16 :2 20 32 :2 3c
48 4b 5d 61 62 :2 5d :2 32 :3 16
:2 1c 21 2d 70 73 :2 2d :2 16 65
:3 13 :2 1e 32 :2 3b 51 5b :2 13 10
2f 16 1f 29 :2 16 2c 2f :2 2c
16 :2 20 32 :2 3c 48 4c 5e 62
63 :2 5e :2 32 :3 16 :2 1c 21 2d 70
73 :2 2d 80 83 :2 2d 87 8a :2 2d
94 97 a0 a8 :2 b1 :2 a0 c9 :2 97
:2 2d d1 d4 :2 2d dc df :2 2d e4
e7 :2 2d f7 fa :2 2d :2 16 66 :2 13
16 :2 20 32 :2 3c 48 4b 5d 61
62 :2 5d :2 32 :3 16 :2 1c 21 2d 70
73 :2 2d :2 16 65 :3 13 :2 1e 32 :2 3b
52 1b :2 13 31 2f 13 21 :2 13
21 31 3f 54 60 69 74 :2 60
21 2a 34 :4 21 13 :4 10 50 :2 d
63 :2 a 7 39 d 14 24 27
:2 d 2a 2c :2 2a 14 27 2f 36
4e 56 5d 76 80 91 9f 12
27 32 3e 4a 5d 69 12 13
20 31 3b 11 29 37 :2 d :2 16
22 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 5b 5f 60
:2 5b :2 2f :3 13 :2 19 1e 2a 6d 70
:2 2a 7d 80 :2 2a 84 87 :2 2a 93
96 :2 2a 9e a1 :2 2a ad b0 b7
c0 c3 :2 b0 :2 2a :2 13 63 :2 10 :2 d
:4 19 :2 d a :3 30 e :2 17 22 20
:2 2b :2 20 d :4 41 40 3f :3 d 1d
1e :2 1d :2 d :2 16 22 39 47 55
2f 3a 46 52 2f 3d 50 :2 22
d 63 :2 a 30 39 14 27 35
4a 12 13 20 31 3b 11 1f
30 :2 d :2 16 22 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
5b 5f 60 :2 5b :2 2f :3 13 :2 19 1e
2a 6d 70 :2 2a 7d 80 :2 2a 84
87 :2 2a 93 96 :2 2a 9e a1 :2 2a
ad b0 b7 c0 c3 :2 b0 :2 2a :2 13
63 :2 10 :2 d :4 19 :2 d a :7 7 4
23 a 11 21 24 :2 a 27 29
:2 27 11 24 2c 33 4b 53 5a
73 7d 8e 9c f 24 2f 3b
47 5a 66 f 10 1d 2e 38
e 1c 2d :2 a :2 13 1f a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 58 5c 5d :2 58 :2 2c :3 10
:2 16 1b 27 6a 6d :2 27 7a 7d
:2 27 81 84 :2 27 90 93 :2 27 9b
9e :2 27 aa ad b4 bd c0 :2 ad
:2 27 :2 10 60 :2 d :2 a :4 16 :2 a 7
:3 2d b :2 14 1f 1d :2 28 :2 1d a
:4 3e 3d 3c :3 a 1a 1b :2 1a :2 a
:2 13 1f 36 44 52 2f 3a 46
52 2f 3d 50 :2 1f a 60 :2 7
2d 23 11 24 32 47 f 10
1d 2e 38 e 1c 2d :2 a :2 13
1f a 7 f d :2 16 22 d
1d :2 a :2 f d :2 16 22 d 10
:2 1a 2c :2 36 42 46 58 5c 5d
:2 58 :2 2c :3 10 :2 16 1b 27 6a 6d
:2 27 7a 7d :2 27 81 84 :2 27 90
93 :2 27 9b 9e :2 27 aa ad b4
bd c0 :2 ad :2 27 :2 10 60 :2 d :2 a
:4 16 :2 a 7 :7 4 :4 9 8 7 2c
35 41 :2 2c 44 46 :2 44 2b :2 7
4e a 16 :2 4e 19 1b :2 19 4d
:3 7 1e :2 7 :3 1e 7 a 13 19
:2 a 1c 1e :2 1c a 1d 26 2e
:2 1d a 22 :2 7 b 14 2a :2 b
2d 2f :2 2d a 36 3f 55 :2 36
58 5a :2 58 35 :2 a e 17 2d
:2 e 30 32 :2 30 d 3a 43 59
:2 3a 5c 5e :2 5c 39 :2 d 1c 2d
3f 51 59 64 67 6a 73 10
1f 2f 39 47 56 62 70 10
1a 2b 38 45 52 60 6f 7e
17 2c 3d 4b 59 62 68 71
77 83 10 1c 2e 3a 47 50
5c 10 1c 2e 39 45 50 5c
6d 7e 10 17 13 1f :3 1d 13
:2 1c 28 13 24 13 :2 1c 28 13
:4 10 d 15 13 :2 1c 28 13 23
:2 10 :2 15 13 :2 1c 28 13 16 :2 20
32 :2 3c 48 4c 5e 62 63 :2 5e
:2 32 :3 16 :2 1c 21 2d 70 73 :2 2d
80 83 :2 2d 87 8a :2 2d 96 99
:2 2d a1 a4 :2 2d b0 b3 ba c3
c6 :2 b3 :2 2d :2 16 66 :2 13 :2 10 :4 1c
:2 10 d :3 61 a 61 10 19 2f
:2 10 32 34 :2 32 1c 2d 3f 51
59 64 67 6a 73 10 1f 2f
39 47 56 62 70 10 1a 2b
38 45 52 60 6f 7e 17 2c
3d 4b 59 62 68 71 77 83
10 1c 2e 3a 47 50 5c 10
1c 2e 39 45 50 5d 6e 7f
10 17 13 1f :3 1d 13 :2 1c 28
13 24 13 :2 1c 28 13 :4 10 d
15 13 :2 1c 28 13 23 :2 10 :2 15
13 :2 1c 28 13 16 :2 20 32 :2 3c
48 4c 5e 62 63 :2 5e :2 32 :3 16
:2 1c 21 2d 70 73 :2 2d 80 83
:2 2d 87 8a :2 2d 96 99 :2 2d a1
a4 :2 2d b0 b3 ba c3 c6 :2 b3
:2 2d :2 16 66 :2 13 :2 10 :4 1c :2 10 d
:4 36 61 1c 2d 3f 51 59 64
67 6a 73 10 1f 2f 39 47
56 62 70 10 1a 2b 38 45
52 60 6f 7e 17 2c 3d 4b
59 62 68 71 77 83 10 1c
2e 3a 47 50 5c 10 1c 2e
39 45 50 5d 6e 7f 10 17
13 1f :3 1d 13 :2 1c 28 13 24
13 :2 1c 28 13 :4 10 d 15 13
:2 1c 28 13 23 :2 10 :2 15 13 :2 1c
28 13 16 :2 20 32 :2 3c 48 4c
5e 62 63 :2 5e :2 32 :3 16 :2 1c 21
2d 70 73 :2 2d 80 83 :2 2d 87
8a :2 2d 96 99 :2 2d a1 a4 :2 2d
b0 b3 ba c3 c6 :2 b3 :2 2d :2 16
66 :2 13 :2 10 :4 1c :2 10 d :7 a 5d
19 2a 3c 4e 56 61 64 67
70 d 1b 2a 33 40 4e 5a
68 d 17 28 35 42 4f 5d
6c 7b 14 29 3a 48 56 5f
65 6e 74 80 d 19 2b 37
44 4d 59 d 19 2b 36 42
4d 5a 6b 7c d 14 10 1c
:3 1a 10 :2 19 25 10 21 10 :2 19
25 10 :4 d a 12 10 :2 19 25
10 20 :2 d :2 12 10 :2 19 25 10
13 :2 1d 2f :2 39 45 49 5b 5f
60 :2 5b :2 2f :3 13 :2 19 1e 2a 6d
70 :2 2a 7d 80 :2 2a 84 87 :2 2a
93 96 :2 2a 9e a1 :2 2a ad b0
b7 c0 c3 :2 b0 :2 2a :2 13 63 :2 10
:2 d :4 19 :2 d a :8 7 12 7 1e
:3 4 e 15 1c 1f :2 15 24 27
:2 e :2 4 1b :2 4 10 4 e 20
34 43 50 :2 7 :2 10 1c 7 4
c a :2 13 1f a 1a :2 7 :2 c
a :2 13 1f a d :2 17 29 :2 33
3f 43 55 59 5a :2 55 :2 29 :3 d
:2 13 18 24 67 6a :2 24 77 7a
:2 24 7e 81 :2 24 8d 90 :2 24 98
9b :2 24 a7 aa b1 ba bd :2 aa
:2 24 :2 d 5d :2 a :2 7 :4 13 :2 7 4
:3 1 4 12 :2 4 12 :2 4 f 4
8 11 13 :2 11 7 1b 24 30
:2 1b 33 35 :2 33 1a :2 7 11 15
17 2b f 17 1a 2b 10 12
1f 30 32 40 a c 16 18
27 29 35 3e 46 4f 5d 66
74 7d 8b 94 a1 aa bb c4
d7 e0 f5 fe 119 122 13a 143
154 15d 16f 178 190 199 1ac 1b5
1d6 1d8 1e2 :2 a :2 13 1f a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 58 5c 5d :2 58 :2 2c :3 10
:2 16 1b 27 6a 6d :2 27 7a 7d
:2 27 81 84 :2 27 90 93 :2 27 9b
9e :2 27 aa ad b4 bd c0 :2 ad
:2 27 :2 10 60 :2 d :2 a :4 16 :2 a 7
:4 38 :3 4 14 :2 4 9 :4 4 d 16
23 38 43 4b 55 7 c 11
21 32 7 1c 27 39 44 7
f 18 20 2b :3 7 a e :2 a
:2 13 1f a 14 a :2 13 1f a
:4 7 b :2 14 20 1d :2 29 :2 1d :3 a
39 :2 7 a 13 1d :2 a 20 22
:2 20 a 1d 34 41 :2 1d a 19
34 48 d 14 10 1c :3 1a 10
:2 19 25 10 21 10 :2 19 25 10
:4 d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 5b 5f 60 :2 5b
:2 2f :3 13 :2 19 1e 2a 6d 70 :2 2a
7d 80 :2 2a 84 87 :2 2a 93 96
:2 2a 9e a1 :2 2a ad b0 b7 c0
c3 :2 b0 :2 2a :2 13 63 :2 10 :2 d :4 19
:2 d a :3 24 19 30 44 d 14
10 1c :3 1a 10 :2 19 25 10 21
10 :2 19 25 10 :4 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
5b 5f 60 :2 5b :2 2f :3 13 :2 19 1e
2a 6d 70 :2 2a 7d 80 :2 2a 84
87 :2 2a 93 96 :2 2a 9e a1 :2 2a
ad b0 b7 c0 c3 :2 b0 :2 2a :2 13
63 :2 10 :2 d :4 19 :2 d a :3 24 19
2b 3f d 14 10 1c :3 1a 10
:2 19 25 10 21 10 :2 19 25 10
:4 d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 5b 5f 60 :2 5b
:2 2f :3 13 :2 19 1e 2a 6d 70 :2 2a
7d 80 :2 2a 84 87 :2 2a 93 96
:2 2a 9e a1 :2 2a ad b0 b7 c0
c3 :2 b0 :2 2a :2 13 63 :2 10 :2 d :4 19
:2 d a :3 24 19 30 3d 4e 62
d 14 10 1c :3 1a 10 :2 19 25
10 21 10 :2 19 25 10 :4 d a
12 10 :2 19 25 10 20 :2 d :2 12
10 :2 19 25 10 13 :2 1d 2f :2 39
45 49 5b 5f 60 :2 5b :2 2f :3 13
:2 19 1e 2a 6d 70 :2 2a 7d 80
:2 2a 84 87 :2 2a 93 96 :2 2a 9e
a1 :2 2a ad b0 b7 c0 c3 :2 b0
:2 2a :2 13 63 :2 10 :2 d :4 19 :2 d a
:3 24 19 35 49 d 14 10 1c
:3 1a 10 :2 19 25 10 21 10 :2 19
25 10 :4 d a 12 10 :2 19 25
10 20 :2 d :2 12 10 :2 19 25 10
13 :2 1d 2f :2 39 45 49 5b 5f
60 :2 5b :2 2f :3 13 :2 19 1e 2a 6d
70 :2 2a 7d 80 :2 2a 84 87 :2 2a
93 96 :2 2a 9e a1 :2 2a ad b0
b7 c0 c3 :2 b0 :2 2a :2 13 63 :2 10
:2 d :4 19 :2 d a :3 24 19 36 4a
d 14 10 1c :3 1a 10 :2 19 25
10 21 10 :2 19 25 10 :4 d a
12 10 :2 19 25 10 20 :2 d :2 12
10 :2 19 25 10 13 :2 1d 2f :2 39
45 49 5b 5f 60 :2 5b :2 2f :3 13
:2 19 1e 2a 6d 70 :2 2a 7d 80
:2 2a 84 87 :2 2a 93 96 :2 2a 9e
a1 :2 2a ad b0 b7 c0 c3 :2 b0
:2 2a :2 13 63 :2 10 :2 d :4 19 :2 d a
:4 24 14 2b 39 13 27 11 1e
11 1f d 14 10 1c :3 1a 10
:2 19 25 10 21 10 :2 19 25 10
:4 d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 5b 5f 60 :2 5b
:2 2f :3 13 :2 19 1e 2a 6d 70 :2 2a
7d 80 :2 2a 84 87 :2 2a 93 96
:2 2a 9e a1 :2 2a ad b0 b7 c0
c3 :2 b0 :2 2a :2 13 63 :2 10 :2 d :4 19
:2 d a :3 7 14 2b 35 13 20
31 3b 49 57 11 :2 d :2 16 22
d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 5b 5f 60 :2 5b
:2 2f :3 13 :2 19 1e 2a 6d 70 :2 2a
7d 80 :2 2a 84 87 :2 2a 93 96
:2 2a 9e a1 :2 2a ad b0 b7 c0
c3 :2 b0 :2 2a :2 13 63 :2 10 :2 d :4 19
:2 d a :3 7 e :2 17 22 20 :2 2b
:2 20 d 10 19 25 :2 10 28 2a
:2 28 10 27 10 2c :2 d 17 2a
37 48 4e 5d 15 16 23 31
3f 4c :2 10 :2 19 25 10 d 15
13 :2 1c 28 13 23 :2 10 :2 15 13
:2 1c 28 13 16 :2 20 32 :2 3c 48
4c 5e 62 63 :2 5e :2 32 :3 16 :2 1c
21 2d 70 73 :2 2d 80 83 :2 2d
87 8a :2 2d 96 99 :2 2d a1 a4
:2 2d b0 b3 ba c3 c6 :2 b3 :2 2d
:2 16 66 :2 13 :2 10 :4 1c :2 10 d :3 3b
11 :2 1a 25 23 :2 2e :2 23 10 14
1b 24 33 :2 1b 37 :2 40 :2 14 13
52 50 :2 5b :2 50 13 1f 13 1d
16 24 16 25 16 1c 30 16
1d 19 25 :3 23 19 :2 22 2e 19
2a 19 :2 22 2e 19 :4 16 13 1b
19 :2 22 2e 19 29 :2 16 :2 1b 19
:2 22 2e 19 1c :2 26 38 :2 42 4e
52 64 68 69 :2 64 :2 38 :3 1c :2 22
27 33 76 79 :2 33 86 89 :2 33
8d 90 :2 33 9c 9f :2 33 a7 aa
:2 33 b6 b9 c0 c9 cc :2 b9 :2 33
:2 1c 6c :2 19 :2 16 :4 22 :2 16 13 :3 69
10 69 16 1f 2e :2 16 33 31
:2 3c :2 31 13 26 34 37 40 48
:2 37 4c :2 55 24 :2 26 13 1d 16
25 2e 16 25 16 24 16 25
16 1c 30 16 1d 19 25 :3 23
19 :2 22 2e 19 2a 19 :2 22 2e
19 :4 16 13 1b 19 :2 22 2e 19
29 :2 16 :2 1b 19 :2 22 2e 19 1c
:2 26 38 :2 42 4e 52 64 68 69
:2 64 :2 38 :3 1c :2 22 27 33 76 79
:2 33 86 89 :2 33 8d 90 :2 33 9c
9f :2 33 a7 aa :2 33 b6 b9 c0
c9 cc :2 b9 :2 33 :2 1c 6c :2 19 :2 16
:4 22 :2 16 13 :4 4b 69 1d 16 25
2e 35 43 4c 16 25 16 24
16 25 16 1c 30 16 1d 19
25 :3 23 19 :2 22 2e 19 2a 19
:2 22 2e 19 :4 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 64
68 69 :2 64 :2 38 :3 1c :2 22 27 33
76 79 :2 33 86 89 :2 33 8d 90
:2 33 9c 9f :2 33 a7 aa :2 33 b6
b9 c0 c9 cc :2 b9 :2 33 :2 1c 6c
:2 19 :2 16 :4 22 :2 16 13 :7 10 14 :2 1d
28 26 :2 31 :2 26 13 46 4f 59
:2 46 5c 5e :2 5c 45 :2 13 22 3d
51 16 1d 19 25 :3 23 19 :2 22
2e 19 2a 19 :2 22 2e 19 :4 16
13 1b 19 :2 22 2e 19 29 :2 16
:2 1b 19 :2 22 2e 19 1c :2 26 38
:2 42 4e 52 64 68 69 :2 64 :2 38
:3 1c :2 22 27 33 76 79 :2 33 86
89 :2 33 8d 90 :2 33 9c 9f :2 33
a7 aa :2 33 b6 b9 c0 c9 cc
:2 b9 :2 33 :2 1c 6c :2 19 :2 16 :4 22 :2 16
13 :4 61 :2 10 3e 1a 1d 30 37
3a 49 4f 52 18 1d 27 2f
33 48 19 1c 29 37 3a 47
4a 59 5c 17 1a 28 2b 1a
31 42 4b 4e 62 65 :2 13 :2 1c
28 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 61 65 66
:2 61 :2 35 :3 19 :2 1f 24 30 73 76
:2 30 83 86 :2 30 8a 8d :2 30 99
9c :2 30 a4 a7 :2 30 b3 b6 bd
c6 c9 :2 b6 :2 30 :2 19 69 :2 16 :2 13
:4 1f :2 13 10 :3 d 14 :2 1d 28 26
:2 31 :2 26 13 :4 47 46 45 :3 13 26
34 37 40 48 :2 37 4c :2 55 24
:2 26 13 22 3d 51 16 1d 19
25 :3 23 19 :2 22 2e 19 2a 19
:2 22 2e 19 :4 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 64
68 69 :2 64 :2 38 :3 1c :2 22 27 33
76 79 :2 33 86 89 :2 33 8d 90
:2 33 9c 9f :2 33 a7 aa :2 33 b6
b9 c0 c9 cc :2 b9 :2 33 :2 1c 6c
:2 19 :2 16 :4 22 :2 16 13 :3 5a 1d 36
44 4e 16 23 31 43 4f 1c
30 16 1d 19 25 :3 23 19 :2 22
2e 19 2a 19 :2 22 2e 19 :4 16
13 1b 19 :2 22 2e 19 29 :2 16
:2 1b 19 :2 22 2e 19 1c :2 26 38
:2 42 4e 52 64 68 69 :2 64 :2 38
:3 1c :2 22 27 33 76 79 :2 33 86
89 :2 33 8d 90 :2 33 9c 9f :2 33
a7 aa :2 33 b6 b9 c0 c9 cc
:2 b9 :2 33 :2 1c 6c :2 19 :2 16 :4 22 :2 16
13 :3 5a 1d 29 38 41 4f 55
1c 29 16 1d 19 25 :3 23 19
:2 22 2e 19 2a 19 :2 22 2e 19
:4 16 13 1b 19 :2 22 2e 19 29
:2 16 :2 1b 19 :2 22 2e 19 1c :2 26
38 :2 42 4e 52 64 68 69 :2 64
:2 38 :3 1c :2 22 27 33 76 79 :2 33
86 89 :2 33 8d 90 :2 33 9c 9f
:2 33 a7 aa :2 33 b6 b9 c0 c9
cc :2 b9 :2 33 :2 1c 6c :2 19 :2 16 :4 22
:2 16 13 :4 5a 13 26 34 37 40
48 :2 37 4c :2 55 24 :2 26 13 1d
29 38 41 4f 5f 72 7f 16
1d 19 25 :3 23 19 :2 22 2e 19
2a 19 :2 22 2e 19 :4 16 13 1b
19 :2 22 2e 19 29 :2 16 :2 1b 19
:2 22 2e 19 1c :2 26 38 :2 42 4e
52 64 68 69 :2 64 :2 38 :3 1c :2 22
27 33 76 79 :2 33 86 89 :2 33
8d 90 :2 33 9c 9f :2 33 a7 aa
:2 33 b6 b9 c0 c9 cc :2 b9 :2 33
:2 1c 6c :2 19 :2 16 :4 22 :2 16 13 :3 10
1d 36 44 4e 16 23 31 43
4f 1c 30 16 1d 19 25 :3 23
19 :2 22 2e 19 2a 19 :2 22 2e
19 :4 16 13 1b 19 :2 22 2e 19
29 :2 16 :2 1b 19 :2 22 2e 19 1c
:2 26 38 :2 42 4e 52 64 68 69
:2 64 :2 38 :3 1c :2 22 27 33 76 79
:2 33 86 89 :2 33 8d 90 :2 33 9c
9f :2 33 a7 aa :2 33 b6 b9 c0
c9 cc :2 b9 :2 33 :2 1c 6c :2 19 :2 16
:4 22 :2 16 13 :7 10 :4 d 10 19 23
:2 10 26 28 :2 26 14 1d 2b :2 14
2e 30 :2 2e 13 :4 39 38 37 :2 13
27 49 59 1b 2d 3b 46 :2 16
:2 1f 2b 16 13 1b 19 :2 22 2e
19 29 :2 16 :2 1b 19 :2 22 2e 19
1c :2 26 38 :2 42 4e 52 64 68
69 :2 64 :2 38 :3 1c :2 22 27 33 76
79 :2 33 86 89 :2 33 8d 90 :2 33
9c 9f :2 33 a7 aa :2 33 b6 b9
c0 c9 cc :2 b9 :2 33 :2 1c 6c :2 19
:2 16 :4 22 :2 16 13 :3 54 17 :2 20 2b
29 :2 34 :2 29 16 49 52 60 :2 49
63 65 :2 63 48 :2 16 20 2c 3c
4f 5a 19 20 1c 28 :3 26 1c
:2 25 31 1c 2d 1c :2 25 31 1c
:4 19 16 1e 1c :2 25 31 1c 2c
:2 19 :2 1e 1c :2 25 31 1c 1f :2 29
3b :2 45 51 55 67 6b 6c :2 67
:2 3b :3 1f :2 25 2a 36 79 7c :2 36
89 8c :2 36 90 93 :2 36 9f a2
:2 36 aa ad :2 36 b9 bc c3 cc
cf :2 bc :2 36 :2 1f 6f :2 1c :2 19 :4 25
:2 19 16 :3 68 13 68 1a :2 23 2e
2c :2 37 :2 2c 19 4c 55 63 :2 4c
66 68 :2 66 4b :3 19 1f :3 19 :2 22
2e 19 16 1e 1c :2 25 31 1c
2c :2 19 :2 1e 1c :2 25 31 1c 1f
:2 29 3b :2 45 51 55 67 6b 6c
:2 67 :2 3b :3 1f :2 25 2a 36 79 7c
:2 36 89 8c :2 36 90 93 :2 36 9f
a2 :2 36 aa ad :2 36 b9 bc c3
cc cf :2 bc :2 36 :2 1f 6f :2 1c :2 19
:4 25 :2 19 16 :3 6b 19 :2 23 35 :2 3f
4b 4f 61 65 66 :2 61 :2 35 :3 19
:2 1f 24 30 73 76 :2 30 83 86
:2 30 8a 8d :2 30 97 9a a3 ab
:2 b4 :2 a3 cf :2 9a :2 30 d7 da :2 30
e2 e5 :2 30 ea ed :2 30 f7 fa
:2 30 :2 19 69 :2 16 19 :2 23 35 :2 3f
4b 4e 60 64 65 :2 60 :2 35 :3 19
:2 1f 24 30 73 76 :2 30 :2 19 68
:3 16 :2 21 35 :2 3e 58 1e :2 16 6b
68 :2 13 54 :3 10 20 21 :2 20 :2 10
:2 19 25 3c 4a 55 6a 70 2f
3b 47 2f 42 :2 25 10 1f 36
43 54 68 13 1a 16 22 :3 20
16 :2 1f 2b 16 27 16 :2 1f 2b
16 :4 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 61 65 66
:2 61 :2 35 :3 19 :2 1f 24 30 73 76
:2 30 83 86 :2 30 8a 8d :2 30 99
9c :2 30 a4 a7 :2 30 b3 b6 bd
c6 c9 :2 b6 :2 30 :2 19 69 :2 16 :2 13
:4 1f :2 13 10 :3 2a 10 21 10 13
1c 1f :2 1c 13 1f :2 13 23 :2 13
24 :2 13 1d :2 13 23 :2 13 1e :2 13
1d :2 13 20 :2 13 20 13 19 20
29 35 :2 20 39 :2 42 :2 19 18 50
53 :2 50 17 5c 19 24 :2 5c 2a
27 33 3f :2 2a :2 27 5b 47 19
21 :2 47 27 24 30 39 :2 27 :2 24
46 :2 5b 5a :2 17 16 19 22 28
:2 19 2b 2d :2 2b 23 1c 2a 34
47 26 44 1c 35 53 64 1c
35 53 64 1c 35 53 64 1c
35 53 64 1c 2b 34 3a 1c
2b 1c 2b 34 3b 48 51 22
36 1c 23 1f 2b :3 29 1f :2 28
34 1f 30 1f :2 28 34 1f :4 1c
19 21 1f :2 28 34 1f 2f :2 1c
:2 21 1f :2 28 34 1f 22 :2 2c 3e
:2 48 54 58 6a 6e 6f :2 6a :2 3e
:3 22 :2 28 2d 39 7c 7f :2 39 8c
8f :2 39 93 96 :2 39 a2 a5 :2 39
ad b0 :2 39 bc bf c6 cf d2
:2 bf :2 39 :2 22 72 :2 1f :2 1c :4 28 :2 1c
19 :4 31 23 1c 2a 34 47 26
44 1c 35 53 64 1c 35 53
64 1c 35 53 64 1c 35 53
64 1c 2b 34 3a 1c 2b 1c
2b 1c 2b 34 3b 48 51 22
36 1c 23 1f 2b :3 29 1f :2 28
34 1f 30 1f :2 28 34 1f :4 1c
19 21 1f :2 28 34 1f 2f :2 1c
:2 21 1f :2 28 34 1f 22 :2 2c 3e
:2 48 54 58 6a 6e 6f :2 6a :2 3e
:3 22 :2 28 2d 39 7c 7f :2 39 8c
8f :2 39 93 96 :2 39 a2 a5 :2 39
ad b0 :2 39 bc bf c6 cf d2
:2 bf :2 39 :2 22 72 :2 1f :2 1c :4 28 :2 1c
19 :7 16 3f 20 19 27 31 44
23 41 19 32 50 61 19 32
50 61 19 32 50 61 19 32
50 61 19 28 31 38 45 4e
5d 63 1f 33 19 20 1c 28
:3 26 1c :2 25 31 1c 2d 1c :2 25
31 1c :4 19 16 1e 1c :2 25 31
1c 2c :2 19 :2 1e 1c :2 25 31 1c
1f :2 29 3b :2 45 51 55 67 6b
6c :2 67 :2 3b :3 1f :2 25 2a 36 79
7c :2 36 89 8c :2 36 90 93 :2 36
9f a2 :2 36 aa ad :2 36 b9 bc
c3 cc cf :2 bc :2 36 :2 1f 6f :2 1c
:2 19 :4 25 :2 19 16 :7 13 17 20 2b
:2 17 2e 30 :2 2e 16 38 41 49
:2 38 4c 4f :2 4c 37 :3 16 20 16
13 54 19 22 2a :2 19 2d 2f
:2 2d 16 29 32 3a :2 29 16 33
54 :3 13 21 32 3e 4b 54 5d
:2 4b 61 6a 72 :2 61 22 2b 37
:2 22 3b 44 4f :2 3b 53 20 2b
:2 53 2f 38 46 :2 2f 4a 53 5e
:2 4a 22 2b 37 :2 22 3b 43 4d
:2 21 13 16 1f 2b :2 16 30 2e
:2 39 :2 2e 19 1f :3 19 :2 22 2e 19
16 1e 1c :2 25 31 1c 2c :2 19
:2 1e 1c :2 25 31 1c 1f :2 29 3b
:2 45 51 55 67 6b 6c :2 67 :2 3b
:3 1f :2 25 2a 36 79 7c :2 36 89
8c :2 36 90 93 :2 36 9f a2 :2 36
aa ad :2 36 b9 bc c3 cc cf
:2 bc :2 36 :2 1f 6f :2 1c :2 19 :4 25 :2 19
16 :3 48 19 :2 23 35 :2 3f 4b 4e
60 64 65 :2 60 :2 35 :3 19 :2 1f 24
30 73 76 :2 30 83 86 :2 30 8a
8d :2 30 97 9a a3 ab :2 b4 :2 a3
c5 :2 9a :2 30 :2 19 68 :2 16 19 :2 23
35 :2 3f 4b 4e 60 64 65 :2 60
:2 35 :3 19 :2 1f 24 30 73 76 :2 30
:2 19 68 :3 16 :2 21 37 :2 40 50 :2 16
13 48 19 22 2e :2 19 31 34
:2 31 19 1f :3 19 :2 22 2e 19 16
1e 1c :2 25 31 1c 2c :2 19 :2 1e
1c :2 25 31 1c 1f :2 29 3b :2 45
51 55 67 6b 6c :2 67 :2 3b :3 1f
:2 25 2a 36 79 7c :2 36 89 8c
:2 36 90 93 :2 36 9f a2 :2 36 aa
ad :2 36 b9 bc c3 cc cf :2 bc
:2 36 :2 1f 6f :2 1c :2 19 :4 25 :2 19 16
:3 36 19 :2 23 35 :2 3f 4b 4f 61
65 66 :2 61 :2 35 :3 19 :2 1f 24 30
73 76 :2 30 83 86 :2 30 8a 8d
:2 30 97 9a a3 ab :2 a3 b8 :2 9a
:2 30 c0 c3 :2 30 cb ce :2 30 d3
d6 :2 30 e3 e6 :2 30 :2 19 69 :2 16
19 :2 23 35 :2 3f 4b 4e 60 64
65 :2 60 :2 35 :3 19 :2 1f 24 30 73
76 :2 30 :2 19 68 :3 16 :2 21 35 3e
4a :2 35 4e 5c :2 16 36 48 :2 13
21 16 1f 27 :2 16 2a 2c :2 2a
32 3b 4c :2 32 4f 52 :2 4f :2 16
5a 19 2a :2 5a 2d 30 :2 2d :3 16
20 :2 16 20 29 3a :2 20 16 32
:2 13 17 20 2c :2 17 2f 31 :2 2f
16 38 41 4d :2 38 52 50 5b
66 :2 52 :2 50 37 :3 16 24 2d 38
:2 24 16 6a 16 24 2a 33 3f
:2 2a 42 44 4d 58 :2 44 :2 2a :2 24
16 :4 13 16 1f 2b :2 16 2e 30
:2 2e 16 24 2d 39 :2 24 3c 3e
:2 24 16 33 16 24 2a 34 3d
49 :2 34 :2 2a 4d 4f :2 2a :2 24 16
:4 13 1d 16 24 2e 41 20 3e
16 2f 4d 5e 16 2f 4d 5e
16 2f 4d 5e 16 2f 4d 5e
16 25 2e 35 42 4b 1c 30
16 1d 19 25 :3 23 19 :2 22 2e
19 2a 19 :2 22 2e 19 :4 16 13
1b 19 :2 22 2e 19 29 :2 16 :2 1b
19 :2 22 2e 19 1c :2 26 38 :2 42
4e 52 64 68 69 :2 64 :2 38 :3 1c
:2 22 27 33 76 79 :2 33 86 89
:2 33 8d 90 :2 33 9c 9f :2 33 a7
aa :2 33 b6 b9 c0 c9 cc :2 b9
:2 33 :2 1c 6c :2 19 :2 16 :4 22 :2 16 13
:8 10 1f :2 10 1c :2 10 19 10 13
1c 24 :2 13 27 29 :2 27 30 39
4a :2 30 4d 4f :2 4d 56 16 27
:2 56 2a 2c :2 2a :2 30 2f :3 13 20
:2 29 13 2f 13 20 :2 29 30 39
44 :2 30 48 :2 51 :2 20 13 :5 10 20
:2 10 23 :2 10 1c 28 3d 4b 56
60 1b 23 2f 3b 47 53 1b
2a 36 40 48 51 59 1b 27
33 40 4b 56 1b 25 30 3c
48 4b 4e 54 :2 1c 10 13 1c
26 :2 13 29 2b :2 29 16 1c :3 16
:2 1f 2b 16 13 1b 19 :2 22 2e
19 29 :2 16 :2 1b 19 :2 22 2e 19
1c :2 26 38 :2 42 4e 52 64 68
69 :2 64 :2 38 :3 1c :2 22 27 33 76
79 :2 33 86 89 :2 33 8d 90 :2 33
9c 9f :2 33 a7 aa :2 33 b6 b9
c0 c9 cc :2 b9 :2 33 :2 1c 6c :2 19
:2 16 :4 22 :2 16 13 :3 2f 16 :2 20 32
:2 3c 48 4c 5e 62 63 :2 5e :2 32
:3 16 :2 1c 21 2d 70 73 :2 2d 80
83 :2 2d 87 8a :2 2d 94 97 a0
a8 :2 b1 :2 a0 c8 :2 97 :2 2d d0 d3
:2 2d db de :2 2d e3 e6 :2 2d ef
f2 :2 2d :2 16 66 :2 13 16 :2 20 32
:2 3c 48 4b 5d 61 62 :2 5d :2 32
:3 16 :2 1c 21 2d 70 73 :2 2d :2 16
65 :3 13 :2 1e 32 :2 3b 51 5b :2 13
10 2f 16 1f 29 :2 16 2c 2f
:2 2c 16 1c :3 16 :2 1f 2b 16 13
1b 19 :2 22 2e 19 29 :2 16 :2 1b
19 :2 22 2e 19 1c :2 26 38 :2 42
4e 52 64 68 69 :2 64 :2 38 :3 1c
:2 22 27 33 76 79 :2 33 86 89
:2 33 8d 90 :2 33 9c 9f :2 33 a7
aa :2 33 b6 b9 c0 c9 cc :2 b9
:2 33 :2 1c 6c :2 19 :2 16 :4 22 :2 16 13
:3 31 16 :2 20 32 :2 3c 48 4c 5e
62 63 :2 5e :2 32 :3 16 :2 1c 21 2d
70 73 :2 2d 80 83 :2 2d 87 8a
:2 2d 94 97 a0 a8 :2 b1 :2 a0 c9
:2 97 :2 2d d1 d4 :2 2d dc df :2 2d
e4 e7 :2 2d f7 fa :2 2d :2 16 66
:2 13 16 :2 20 32 :2 3c 48 4b 5d
61 62 :2 5d :2 32 :3 16 :2 1c 21 2d
70 73 :2 2d :2 16 65 :3 13 :2 1e 32
:2 3b 52 1b :2 13 31 2f :2 10 1a
26 35 3d 4a 6a 75 13 1a
16 22 :3 20 16 :2 1f 2b 16 27
16 :2 1f 2b 16 :4 13 10 18 16
:2 1f 2b 16 26 :2 13 :2 18 16 :2 1f
2b 16 19 :2 23 35 :2 3f 4b 4f
61 65 66 :2 61 :2 35 :3 19 :2 1f 24
30 73 76 :2 30 83 86 :2 30 8a
8d :2 30 99 9c :2 30 a4 a7 :2 30
b3 b6 bd c6 c9 :2 b6 :2 30 :2 19
69 :2 16 :2 13 :4 1f :2 13 10 :3 2a 1f
1a 2f 31 3d 3f 55 57 66
68 7d 7f 18 29 2b 30 39
3e 40 4d 4f 5d 5f 6c 13
1a 16 22 :3 20 16 :2 1f 2b 16
27 16 :2 1f 2b 16 :4 13 10 18
16 :2 1f 2b 16 26 :2 13 :2 18 16
:2 1f 2b 16 19 :2 23 35 :2 3f 4b
4f 61 65 66 :2 61 :2 35 :3 19 :2 1f
24 30 73 76 :2 30 83 86 :2 30
8a 8d :2 30 99 9c :2 30 a4 a7
:2 30 b3 b6 bd c6 c9 :2 b6 :2 30
:2 19 69 :2 16 :2 13 :4 1f :2 13 10 :3 2a
1f 1a 2f 38 4e 5d 72 86
18 34 48 13 1a 16 22 :3 20
16 :2 1f 2b 16 27 16 :2 1f 2b
16 :4 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 61 65 66
:2 61 :2 35 :3 19 :2 1f 24 30 73 76
:2 30 83 86 :2 30 8a 8d :2 30 99
9c :2 30 a4 a7 :2 30 b3 b6 bd
c6 c9 :2 b6 :2 30 :2 19 69 :2 16 :2 13
:4 1f :2 13 10 :3 2a 1f 1a 2f 39
4f 5e 73 87 18 35 49 13
1a 16 22 :3 20 16 :2 1f 2b 16
27 16 :2 1f 2b 16 :4 13 10 18
16 :2 1f 2b 16 26 :2 13 :2 18 16
:2 1f 2b 16 19 :2 23 35 :2 3f 4b
4f 61 65 66 :2 61 :2 35 :3 19 :2 1f
24 30 73 76 :2 30 83 86 :2 30
8a 8d :2 30 99 9c :2 30 a4 a7
:2 30 b3 b6 bd c6 c9 :2 b6 :2 30
:2 19 69 :2 16 :2 13 :4 1f :2 13 10 :3 2a
10 1c 25 2f :2 1c 32 34 :2 1c
:2 10 1b 24 2d :2 1b 30 32 3b
45 :2 32 :2 1b :2 10 23 :2 10 23 3f
4d 5b :2 23 :2 10 23 :2 10 23 3f
4d 57 :2 23 10 1a 2a 38 19
26 37 4b 13 1a 16 22 :3 20
16 :2 1f 2b 16 27 16 :2 1f 2b
16 :4 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 61 65 66
:2 61 :2 35 :3 19 :2 1f 24 30 73 76
:2 30 83 86 :2 30 8a 8d :2 30 99
9c :2 30 a4 a7 :2 30 b3 b6 bd
c6 c9 :2 b6 :2 30 :2 19 69 :2 16 :2 13
:4 1f :2 13 10 :3 2a 14 1d 33 :2 14
36 38 :2 36 13 40 49 5f :2 40
62 64 :2 62 3f :2 13 1d 34 42
16 2c 16 2c 16 2c 16 2c
16 21 2d 3c 1c 30 16 1d
19 25 :3 23 19 :2 22 2e 19 2a
19 :2 22 2e 19 :4 16 13 1b 19
:2 22 2e 19 29 :2 16 :2 1b 19 :2 22
2e 19 1c :2 26 38 :2 42 4e 52
64 68 69 :2 64 :2 38 :3 1c :2 22 27
33 76 79 :2 33 86 89 :2 33 8d
90 :2 33 9c 9f :2 33 a7 aa :2 33
b6 b9 c0 c9 cc :2 b9 :2 33 :2 1c
6c :2 19 :2 16 :4 22 :2 16 13 :3 67 10
67 16 1f 35 :2 16 38 3a :2 38
1d 34 42 16 2c 16 2c 16
21 2d 3c 1c 30 16 1d 19
25 :3 23 19 :2 22 2e 19 2a 19
:2 22 2e 19 :4 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 64
68 69 :2 64 :2 38 :3 1c :2 22 27 33
76 79 :2 33 86 89 :2 33 8d 90
:2 33 9c 9f :2 33 a7 aa :2 33 b6
b9 c0 c9 cc :2 b9 :2 33 :2 1c 6c
:2 19 :2 16 :4 22 :2 16 13 :3 3c 13 21
2a 36 :2 21 13 10 3c 67 16
1f 35 :2 16 38 3a :2 38 1d 34
42 16 2c 16 2c 16 21 2d
3c 1c 30 16 1d 19 25 :3 23
19 :2 22 2e 19 2a 19 :2 22 2e
19 :4 16 13 1b 19 :2 22 2e 19
29 :2 16 :2 1b 19 :2 22 2e 19 1c
:2 26 38 :2 42 4e 52 64 68 69
:2 64 :2 38 :3 1c :2 22 27 33 76 79
:2 33 86 89 :2 33 8d 90 :2 33 9c
9f :2 33 a7 aa :2 33 b6 b9 c0
c9 cc :2 b9 :2 33 :2 1c 6c :2 19 :2 16
:4 22 :2 16 13 :3 3c 13 21 2a 36
:2 21 13 3c 67 1d 34 42 16
21 2d 3c 1c 30 16 1d 19
25 :3 23 19 :2 22 2e 19 2a 19
:2 22 2e 19 :4 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 64
68 69 :2 64 :2 38 :3 1c :2 22 27 33
76 79 :2 33 86 89 :2 33 8d 90
:2 33 9c 9f :2 33 a7 aa :2 33 b6
b9 c0 c9 cc :2 b9 :2 33 :2 1c 6c
:2 19 :2 16 :4 22 :2 16 13 :3 10 13 21
2a 36 :2 21 :2 13 21 2a 36 :2 21
13 :4 10 2a :2 d 3b :2 a :5 7 :2 10
1c 32 40 4e :2 1c 7 4 8
1 7 d :3 7 :2 10 1c 7 4
c a :2 13 1f a 1a :2 7 :2 c
a :2 13 1f a d :2 17 29 :2 33
3f 43 55 59 5a :2 55 :2 29 :3 d
:2 13 18 24 67 6a :2 24 77 7a
:2 24 7e 81 :2 24 8d 90 :2 24 98
9b :2 24 a7 aa b1 ba bd :2 aa
:2 24 :2 d 5d :2 a :2 7 :4 13 :2 7 4
:3 1 4 f 4 7 10 1a :2 7
1d 1f :2 1d 7 13 7 a 13
1d :2 a 23 20 2c 35 :2 23 :2 20
a 16 1f 29 :2 16 2c 2e 37
40 :2 2e :2 16 a e 17 21 :2 e
24 26 :2 24 d 2e 37 43 :2 2e
46 48 :2 46 2d :2 d :4 16 15 14
:3 d 1d :2 d 1e :2 d 17 :2 d 1d
:2 d 18 :2 d 17 :2 d 1a :2 d 1a
d 10 19 24 :2 10 27 29 :2 27
2f 38 40 :2 2f 43 46 :2 43 :3 10
1a 10 d 4a 13 1c 24 :2 13
27 29 :2 27 10 23 2c 34 :2 23
10 2d 4a :3 d 1b 2c 38 45
4e 57 :2 45 5b 64 6c :2 5b 22
2b 37 :2 22 3b 44 4f :2 3b 53
20 2b :2 53 2f 38 46 :2 2f 4a
53 5e :2 4a 22 2b 37 :2 22 3b
43 4d :2 1b d 10 19 25 :2 10
2a 28 :2 33 :2 28 13 :2 1d 2f :2 39
45 48 5a 5e 5f :2 5a :2 2f :3 13
:2 19 1e 2a 6d 70 :2 2a 7d 80
:2 2a 84 87 :2 2a 91 94 9d a5
:2 ae :2 9d bf :2 94 :2 2a :2 13 62 :2 10
13 :2 1d 2f :2 39 45 48 5a 5e
5f :2 5a :2 2f :3 13 :2 19 1e 2a 6d
70 :2 2a :2 13 62 :3 10 :2 1b 31 :2 3a
4a :2 10 d 42 13 1c 28 :2 13
2b 2e :2 2b 13 :2 1d 2f :2 39 45
49 5b 5f 60 :2 5b :2 2f :3 13 :2 19
1e 2a 6d 70 :2 2a 7d 80 :2 2a
84 87 :2 2a 91 94 9d a5 :2 9d
b2 :2 94 :2 2a ba bd :2 2a c5 c8
:2 2a cd d0 :2 2a dd e0 :2 2a :2 13
63 :2 10 13 :2 1d 2f :2 39 45 48
5a 5e 5f :2 5a :2 2f :3 13 :2 19 1e
2a 6d 70 :2 2a :2 13 62 :3 10 :2 1b
2f 38 44 :2 2f 48 56 :2 10 30
42 :3 d 15 1f :3 d :2 16 22 d
1c 32 3c 40 15 16 23 34
40 19 10 17 13 1f :3 1d 13
:2 1c 28 13 24 13 :2 1c 28 13
:4 10 d 15 13 :2 1c 28 13 23
:2 10 :2 15 13 :2 1c 28 13 16 :2 20
32 :2 3c 48 4c 5e 62 63 :2 5e
:2 32 :3 16 :2 1c 21 2d 70 73 :2 2d
80 83 :2 2d 87 8a :2 2d 96 99
:2 2d a1 a4 :2 2d b0 b3 ba c3
c6 :2 b3 :2 2d :2 16 66 :2 13 :2 10 :4 1c
:2 10 d :3 38 d 1e :2 d 1c :2 d
19 :2 d 16 :2 d 17 d 10 19
21 :2 10 24 26 :2 24 2c 35 46
:2 2c 49 4c :2 49 54 13 24 :2 54
27 2a :2 27 :2 2c 2b :2 10 13 1c
24 :2 13 27 29 :2 27 2f 38 49
:2 2f 4c 4f :2 4c :2 13 57 16 27
:2 57 2a 2d :2 2a :3 13 1d :2 13 1d
26 37 :2 1d 13 2f :3 10 1a :2 23
2a 33 3b :2 2a 3f :2 48 :2 1a :2 10
1d 10 2d 10 1d 10 :5 d 20
:2 d 19 25 36 44 52 5c 1b
23 2f 3b 47 53 1b 2a 36
40 48 51 59 1b 27 33 40
4b 56 1b 25 30 3c 48 4b
4e 54 :2 19 d 10 19 23 :2 10
26 28 :2 26 13 :2 1d 2f :2 39 45
49 5b 5f 60 :2 5b :2 2f :3 13 :2 19
1e 2a 6d 70 :2 2a 7d 80 :2 2a
84 87 :2 2a 91 94 9d a5 :2 ae
:2 9d c5 :2 94 :2 2a cd d0 :2 2a d8
db :2 2a e0 e3 :2 2a ec ef :2 2a
:2 13 63 :2 10 13 :2 1d 2f :2 39 45
48 5a 5e 5f :2 5a :2 2f :3 13 :2 19
1e 2a 6d 70 :2 2a :2 13 62 :3 10
:2 1b 2f :2 38 4e 58 :2 10 d 2c
13 1c 26 :2 13 29 2c :2 29 13
:2 1d 2f :2 39 45 49 5b 5f 60
:2 5b :2 2f :3 13 :2 19 1e 2a 6d 70
:2 2a 7d 80 :2 2a 84 87 :2 2a 91
94 9d a5 :2 ae :2 9d c6 :2 94 :2 2a
ce d1 :2 2a d9 dc :2 2a e1 e4
:2 2a f4 f7 :2 2a :2 13 63 :2 10 13
:2 1d 2f :2 39 45 48 5a 5e 5f
:2 5a :2 2f :3 13 :2 19 1e 2a 6d 70
:2 2a :2 13 62 :3 10 :2 1b 2f :2 38 4f
18 :2 10 2e 2c 1a 2f 43 13
22 2c 13 2c 4a 58 6c 13
2c 4a 58 6c 13 2c 4a 58
6c 13 2c 4a 58 6c 19 2d
13 1a 16 22 :3 20 16 :2 1f 2b
16 27 16 :2 1f 2b 16 :4 13 10
18 16 :2 1f 2b 16 26 :2 13 :2 18
16 :2 1f 2b 16 19 :2 23 35 :2 3f
4b 4f 61 65 66 :2 61 :2 35 :3 19
:2 1f 24 30 73 76 :2 30 83 86
:2 30 8a 8d :2 30 99 9c :2 30 a4
a7 :2 30 b3 b6 bd c6 c9 :2 b6
:2 30 :2 19 69 :2 16 :2 13 :4 1f :2 13 10
:7 d 38 :2 a 38 :2 7 11 1f 2c
10 1d 2e 3c 4d :2 a :2 13 1f
a 7 f d :2 16 22 d 1d
:2 a :2 f d :2 16 22 d 10 :2 1a
2c :2 36 42 46 58 5c 5d :2 58
:2 2c :3 10 :2 16 1b 27 6a 6d :2 27
7a 7d :2 27 81 84 :2 27 90 93
:2 27 9b 9e :2 27 aa ad b4 bd
c0 :2 ad :2 27 :2 10 60 :2 d :2 a :4 16
:2 a 7 :3 21 a 13 1c :2 a 1f
21 :2 1f 14 20 2f 37 44 13
20 31 3e d 14 10 1c :3 1a
10 :2 19 25 10 21 10 :2 19 25
10 :4 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 5b 5f 60
:2 5b :2 2f :3 13 :2 19 1e 2a 6d 70
:2 2a 7d 80 :2 2a 84 87 :2 2a 93
96 :2 2a 9e a1 :2 2a ad b0 b7
c0 c3 :2 b0 :2 2a :2 13 63 :2 10 :2 d
:4 19 :2 d a :4 23 :2 7 21 :3 4 12
4 :4 8 7 11 1d 2b 34 3b
48 51 6a 77 a 11 d 19
:3 17 d :2 16 22 d 1e d :2 16
22 d :4 a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 58 5c
5d :2 58 :2 2c :3 10 :2 16 1b 27 6a
6d :2 27 7a 7d :2 27 81 84 :2 27
90 93 :2 27 9b 9e :2 27 aa ad
b4 bd c0 :2 ad :2 27 :2 10 60 :2 d
:2 a :4 16 :2 a 7 :3 29 a 13 1f
:2 a 22 24 :2 22 14 22 2f 13
20 31 49 :2 d :2 16 22 d a
12 10 :2 19 25 10 20 :2 d :2 12
10 :2 19 25 10 13 :2 1d 2f :2 39
45 49 5b 5f 60 :2 5b :2 2f :3 13
:2 19 1e 2a 6d 70 :2 2a 7d 80
:2 2a 84 87 :2 2a 93 96 :2 2a 9e
a1 :2 2a ad b0 b7 c0 c3 :2 b0
:2 2a :2 13 63 :2 10 :2 d :4 19 :2 d a
:3 26 d 16 1f :2 d 22 24 :2 22
1c 37 44 10 1d 2e 3c 4d
10 17 13 1f :3 1d 13 :2 1c 28
13 24 13 :2 1c 28 13 :4 10 d
15 13 :2 1c 28 13 23 :2 10 :2 15
13 :2 1c 28 13 16 :2 20 32 :2 3c
48 4c 5e 62 63 :2 5e :2 32 :3 16
:2 1c 21 2d 70 73 :2 2d 80 83
:2 2d 87 8a :2 2d 96 99 :2 2d a1
a4 :2 2d b0 b3 ba c3 c6 :2 b3
:2 2d :2 16 66 :2 13 :2 10 :4 1c :2 10 d
:3 26 1c 36 43 54 62 10 17
13 1f :3 1d 13 :2 1c 28 13 24
13 :2 1c 28 13 :4 10 d 15 13
:2 1c 28 13 23 :2 10 :2 15 13 :2 1c
28 13 16 :2 20 32 :2 3c 48 4c
5e 62 63 :2 5e :2 32 :3 16 :2 1c 21
2d 70 73 :2 2d 80 83 :2 2d 87
8a :2 2d 96 99 :2 2d a1 a4 :2 2d
b0 b3 ba c3 c6 :2 b3 :2 2d :2 16
66 :2 13 :2 10 :4 1c :2 10 d :4 26 :2 a
26 :2 7 a 13 1c :2 a 1f 21
:2 1f a 18 :2 a 1d 2d 3b 50
5c 5f :2 1d a 14 23 33 39
41 48 5f 67 6e 86 90 a1
af 12 1e 2b 35 40 4c 58
6b 77 12 29 3d :2 d :2 16 22
d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 5b 5f 60 :2 5b
:2 2f :3 13 :2 19 1e 2a 6d 70 :2 2a
7d 80 :2 2a 84 87 :2 2a 93 96
:2 2a 9e a1 :2 2a ad b0 b7 c0
c3 :2 b0 :2 2a :2 13 63 :2 10 :2 d :4 19
:2 d a :3 23 e :2 17 22 20 :2 2b
:2 20 d 10 19 25 :2 10 28 2a
:2 28 1a 13 2d 13 2e 13 2e
13 2d 3f 13 19 26 37 45
13 1a 16 22 :3 20 16 :2 1f 2b
16 27 16 :2 1f 2b 16 :4 13 10
18 16 :2 1f 2b 16 26 :2 13 :2 18
16 :2 1f 2b 16 19 :2 23 35 :2 3f
4b 4f 61 65 66 :2 61 :2 35 :3 19
:2 1f 24 30 73 76 :2 30 83 86
:2 30 8a 8d :2 30 99 9c :2 30 a4
a7 :2 30 b3 b6 bd c6 c9 :2 b6
:2 30 :2 19 69 :2 16 :2 13 :4 1f :2 13 10
:4 2c 1a 13 2d 13 2e 13 2e
13 2d 3f 13 30 42 19 26
37 45 13 1a 16 22 :3 20 16
:2 1f 2b 16 27 16 :2 1f 2b 16
:4 13 10 18 16 :2 1f 2b 16 26
:2 13 :2 18 16 :2 1f 2b 16 19 :2 23
35 :2 3f 4b 4f 61 65 66 :2 61
:2 35 :3 19 :2 1f 24 30 73 76 :2 30
83 86 :2 30 8a 8d :2 30 99 9c
:2 30 a4 a7 :2 30 b3 b6 bd c6
c9 :2 b6 :2 30 :2 19 69 :2 16 :2 13 :4 1f
:2 13 10 :7 d 11 18 28 2b :2 11
2e 30 :2 2e :2 10 20 :2 10 :2 19 25
3c 4a 58 2f 3a 46 52 2f
3d 50 :2 25 10 35 :2 d 3b :2 a
:4 f e d 36 :2 41 48 12 :2 36
27 29 :2 27 35 :3 d 1b d 2c
:2 a 23 :2 7 29 :2 4 8 11 1d
:2 8 20 22 :2 20 7 :4 2b 2a 29
:2 7 a 13 29 :2 a 2c 2e :2 2c
14 22 2f 13 20 31 49 11
1f 30 :2 d :2 16 22 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 5b 5f 60 :2 5b :2 2f :3 13 :2 19
1e 2a 6d 70 :2 2a 7d 80 :2 2a
84 87 :2 2a 93 96 :2 2a 9e a1
:2 2a ad b0 b7 c0 c3 :2 b0 :2 2a
:2 13 63 :2 10 :2 d :4 19 :2 d a :3 30
d 16 1f :2 d 22 24 :2 22 17
14 16 23 34 4c 14 22 33
10 17 13 1f :3 1d 13 :2 1c 28
13 24 13 :2 1c 28 13 :4 10 d
15 13 :2 1c 28 13 23 :2 10 :2 15
13 :2 1c 28 13 16 :2 20 32 :2 3c
48 4c 5e 62 63 :2 5e :2 32 :3 16
:2 1c 21 2d 70 73 :2 2d 80 83
:2 2d 87 8a :2 2d 96 99 :2 2d a1
a4 :2 2d b0 b3 ba c3 c6 :2 b3
:2 2d :2 16 66 :2 13 :2 10 :4 1c :2 10 d
:3 26 17 14 20 16 23 34 40
14 22 10 17 13 1f :3 1d 13
:2 1c 28 13 24 13 :2 1c 28 13
:4 10 d 15 13 :2 1c 28 13 23
:2 10 :2 15 13 :2 1c 28 13 16 :2 20
32 :2 3c 48 4c 5e 62 63 :2 5e
:2 32 :3 16 :2 1c 21 2d 70 73 :2 2d
80 83 :2 2d 87 8a :2 2d 96 99
:2 2d a1 a4 :2 2d b0 b3 ba c3
c6 :2 b3 :2 2d :2 16 66 :2 13 :2 10 :4 1c
:2 10 d :4 26 :2 a 30 :3 7 15 :2 7
1a 2a 38 4d 59 5c :2 1a 7
11 20 30 36 3e 45 5c 64
6b 83 8d 9e ac f 1b 28
32 3d 49 55 68 74 f 26
3a :2 a :2 13 1f a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
58 5c 5d :2 58 :2 2c :3 10 :2 16 1b
27 6a 6d :2 27 7a 7d :2 27 81
84 :2 27 90 93 :2 27 9b 9e :2 27
aa ad b4 bd c0 :2 ad :2 27 :2 10
60 :2 d :2 a :4 16 :2 a 7 :3 4d b
:2 14 1f 1d :2 28 :2 1d a d 16
22 :2 d 25 27 :2 25 17 10 2b
10 2b 10 2b 10 2c 3e 10
16 23 34 42 10 17 13 1f
:3 1d 13 :2 1c 28 13 24 13 :2 1c
28 13 :4 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 5e 62
63 :2 5e :2 32 :3 16 :2 1c 21 2d 70
73 :2 2d 80 83 :2 2d 87 8a :2 2d
96 99 :2 2d a1 a4 :2 2d b0 b3
ba c3 c6 :2 b3 :2 2d :2 16 66 :2 13
:2 10 :4 1c :2 10 d :4 29 17 10 2b
10 2b 10 2b 10 2c 3e 10
2d 3f 16 23 34 42 10 17
13 1f :3 1d 13 :2 1c 28 13 24
13 :2 1c 28 13 :4 10 d 15 13
:2 1c 28 13 23 :2 10 :2 15 13 :2 1c
28 13 16 :2 20 32 :2 3c 48 4c
5e 62 63 :2 5e :2 32 :3 16 :2 1c 21
2d 70 73 :2 2d 80 83 :2 2d 87
8a :2 2d 96 99 :2 2d a1 a4 :2 2d
b0 b3 ba c3 c6 :2 b3 :2 2d :2 16
66 :2 13 :2 10 :4 1c :2 10 d :7 a e
15 25 28 :2 e 2b 2d :2 2b d
37 40 4c :2 37 4f 51 :2 4f 36
:3 d 1d :2 d :2 16 22 39 47 55
2f 3a 46 52 2f 3d 50 :2 22
d 54 :2 a 38 :2 7 4d :3 4 17
21 :2 17 4 7 :2 10 19 1b :2 19
7 :2 10 1c 7 1f :2 4 7 :2 11
23 :2 2d 39 3c 4e 52 53 :2 4e
:2 23 :3 7 :2 d 12 1e 61 64 :2 1e
71 74 :2 1e 78 7b :2 1e 85 88
91 99 :2 a2 :2 91 ad :2 88 :2 1e b5
b8 :2 1e c0 c3 :2 1e c8 cb :2 1e
df e2 :2 1e :2 7 56 :3 4 :2 f 21
:2 2a 34 49 :2 4 1 9 12 9
a :2 14 26 :2 30 3c 40 52 56
57 :2 52 :2 26 :3 a :2 10 15 21 64
67 :2 21 :3 a :2 10 15 39 5c 5f
:2 68 :2 39 76 79 :2 39 81 84 :2 8d
:2 39 :2 a 5a :3 7 c 15 1e 2c
a 13 1e f e 7 e 7
e 7 29 :2 4 :2 9 7 :2 10 1c
:2 7 :2 11 23 :2 2d 39 3d 4f 53
54 :2 4f :2 23 :3 7 :2 d 12 1e 61
64 :2 1e :3 7 :2 d 12 36 42 45
:2 36 4d 50 :2 36 5c 5f 66 6f
72 :2 5f :2 36 :2 7 57 :2 4 7 c
1b a 13 1e f e 7 e
7 e 7 :2 4 :4 10 :2 4 :8 1 
4f32
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 :4 7 :4 8
:4 9 :4 a :4 b :4 c
:4 d :4 e :4 f :4 10
:4 11 2 :4 12 :2 1
:7 14 :7 15 :7 16 :7 17
:7 18 :7 19 :7 1a :7 1b
:7 1c :7 1d :7 1e :7 1f
:5 20 :5 21 :5 22 :5 23
:7 24 :5 25 :5 26 :7 27
:7 28 :5 29 :5 2a :5 2b
:5 2c :7 2d :7 2e :7 2f
:7 30 :7 31 :6 32 :6 33
:6 34 :6 35 :6 36 :6 37
:6 38 :6 39 :6 3a :6 3b
:6 3c :6 3d :6 3e :6 3f
:6 40 :6 41 :6 42 :6 43
:6 44 :6 45 :6 46 :6 47
:6 48 :6 49 :6 4a :6 4b
:6 4c :6 4d :6 4e :6 4f
:6 50 :6 51 :6 52 :6 53
:6 54 :6 55 :6 56 :6 57
:6 58 :6 59 :7 5a :7 5b
:6 5c :6 5d :7 5e :6 5f
:7 60 :7 61 :6 62 :6 63
:5 64 :5 65 :6 66 :6 67
:6 68 :6 69 :6 6a :6 6b
:6 6c :6 6d :6 6e :6 6f
:6 70 :7 71 :7 72 :7 73
:e 74 :6 75 :6 76 :2 78
0 :2 78 :2 79 :3 7a
79 :5 78 :2 7c 0
:2 7c :2 7d :6 7e 7f
7d :5 7c :2 81 0
:2 81 :c 82 :12 83 :a 84
:a 85 :6 86 :8 87 :8 88
:23 89 :2 8a 82 :5 81
:7 8c :b 8e :11 8f 90
:2 8f :2 90 :2 8f :2 90
:2 8f :2 91 :2 8f :2 91
:2 8f :2 91 :2 8f :2 91
:2 8f :2 92 :2 8f :2 92
:2 8f :2 92 :2 8f :2 92
:2 8f :2 93 :2 8f :2 93
:2 8f :2 93 :2 8f :2 93
:2 8f :2 94 :2 8f :2 94
:2 8f :2 94 :2 8f :2 94
:2 8f :2 95 :2 8f :9 95
:2 8f :2 96 :2 8f :9 96
:2 8f :2 97 :2 8f :9 97
:2 8f :2 98 :2 8f :9 98
:2 8f :2 99 :2 8f :9 99
:2 8f :2 9a :2 8f :9 9a
:2 8f :2 9b :2 8f :9 9b
:2 8f :2 9c :2 8f :9 9c
:2 8f :2 9d :2 8f :9 9d
:2 8f :2 9e :2 8f :2 9e
:2 8f :2 9e :2 8f :2 9e
:2 8f :2 9e :4 8f :3 8e
:11 a1 :b a2 :3 a1 :3 a9
:3 aa :3 ab :3 ac :3 ad
:3 ae :d af :d b0 :2 b1
:2 b0 :6 b1 :3 b0 :3 b2
b1 :2 b0 :3 b5 :3 b6
:3 b8 :3 b9 :3 bb :3 bd
:3 be :5 c0 :4 c2 :3 c3
:5 c4 c3 :5 c6 c5
:3 c3 :5 c9 :4 cc :5 cd
cb cf :5 d0 :3 cf
:2 d1 :5 d3 :11 d4 :24 d5
:3 d4 :2 d2 :6 d1 ce
:3 8d :9 da :11 db :30 dc
:3 db :11 de :b df :3 de
:a e1 :3 da :7 e5 :7 e6
:3 e7 e5 :5 e8 e4
ea :5 eb :3 ea :2 ec
:5 ee :11 ef :24 f0 :3 ef
:2 ed :6 ec e9 :3 8d
:a f6 :11 f7 :30 f8 :3 f7
:11 fa :b fb :3 fa :a fd
:3 f6 :3 100 :2 102 103
:2 104 102 :5 105 101
107 :5 108 :3 107 :2 109
:5 10b :11 10c :24 10d :3 10c
:2 10a :6 109 106 :3 8d
:9 112 :11 113 :30 114 :3 113
:11 116 :b 117 :3 116 :a 119
:3 112 :12 11d :7 11e :c 11f
11d :5 120 11c 122
:5 123 :3 122 :2 124 :5 126
:11 127 :24 128 :3 127 :2 125
:6 124 121 :3 8d :a 12d
:11 12e :30 12f :3 12e :11 131
:b 132 :3 131 :a 134 :3 12d
:8 138 :5 139 137 13b
:5 13c :3 13b :2 13d :5 13f
:11 140 :24 141 :3 140 :2 13e
:6 13d 13a :3 8d :a 146
:11 147 :30 148 :3 147 :11 14a
:b 14b :3 14a :a 14d :3 146
:6 151 :5 152 150 154
:5 155 :3 154 :2 156 :5 158
:11 159 :24 15a :3 159 :2 157
:6 156 153 :3 8d :b 15f
:3 160 :3 15f :a 163 :11 164
:30 165 :3 164 :11 167 :b 168
:3 167 :a 16a :3 163 :3 16d
:b 16f :b 170 :b 171 :5 17a
:9 17b :4 17d :4 17e :3 17f
17d :5 180 17c 182
:5 183 :3 182 :2 184 :5 186
:11 187 :24 188 :3 187 :2 185
:6 184 181 :3 17b 18c
17b :9 18c :4 18e :4 18f
:3 190 18e :5 191 18d
193 :5 194 :3 193 :2 195
:5 197 :11 198 :24 199 :3 198
:2 196 :6 195 192 :4 18c
:3 17b :9 19f :11 1a0 :30 1a1
:3 1a0 :11 1a3 :b 1a4 :3 1a3
:7 1a6 1a7 :2 1a6 :3 19f
17a :4 1ab :7 1ac 1ab
:5 1ad 1aa 1af :5 1b0
:3 1af :2 1b1 :5 1b3 :11 1b4
:24 1b5 :3 1b4 :2 1b2 :6 1b1
1ae :3 1a9 :4 1bb :7 1bc
1bb :5 1bd 1ba 1bf
:5 1c0 :3 1bf :2 1c1 :5 1c3
:11 1c4 :24 1c5 :3 1c4 :2 1c2
:6 1c1 1be :3 1a9 :18 1ca
:2 1cb :2 1ca :4 1cb :6 1ca
:4 1cd :5 1ce 1cf 1cd
:5 1d0 1cc 1d2 :5 1d3
:3 1d2 :2 1d4 :5 1d6 :11 1d7
:24 1d8 :3 1d7 :2 1d5 :6 1d4
1d1 :3 1cb :9 1dd :11 1de
:30 1df :3 1de :11 1e1 :b 1e2
:3 1e1 :7 1e4 1e5 :2 1e4
:3 1dd 1cb :2 1ca 1a9
:3 17a :3 1f7 :3 1f8 :3 1f9
:3 1fa :3 1fb :5 204 :4 206
:3 207 :5 208 207 :5 20a
209 :3 207 :a 20d :3 20e
20d :3 210 20f :3 20d
:4 214 :5 215 213 217
:5 218 :3 217 :2 219 :5 21b
:11 21c :24 21d :3 21c :2 21a
:6 219 216 :3 8d :9 223
:6 225 :5 226 225 :5 227
224 229 :5 22a :3 229
:2 22b :5 22d :11 22e :24 22f
:3 22e :2 22c :6 22b 228
:3 223 :9 234 :3 235 :4 237
:4 238 :5 239 237 :5 23a
236 23c :5 23d :3 23c
:2 23e :5 240 :11 241 :24 242
:3 241 :2 23f :6 23e 23b
:3 234 :a 248 :11 249 :30 24a
:3 249 :11 24c :b 24d :3 24c
:7 24f 250 :2 24f 251
248 :e 251 :11 252 :2 253
:2 252 :4 253 :3 252 :11 254
:30 255 :3 254 :11 257 :b 258
:3 257 :7 25a 25b :2 25a
253 :2 252 251 :3 248
:3 234 :9 261 :3 262 :3 263
261 :4 267 268 :2 269
:2 26a :2 26b 267 :5 26c
266 26e :5 26f :3 26e
:2 270 :5 272 :11 273 :24 274
:3 273 :2 271 :6 270 26d
:3 265 :a 27a :3 27b :5 27c
:b 27e 27f :3 27e :5 280
:3 281 :3 282 :6 284 :3 285
:3 284 :a 287 :5 289 :6 28a
:2 28b :4 28c :7 28d :2 28e
289 :6 28f :5 290 28f
:5 292 291 :3 28f 288
295 :5 296 :3 295 :2 297
:5 299 :11 29a :24 29b :3 29a
:2 298 :6 297 294 :3 27a
:6 2a2 :5 2a3 2a1 2a5
:5 2a6 :3 2a5 :2 2a7 :5 2a9
:11 2aa :24 2ab :3 2aa :2 2a8
:6 2a7 2a4 :3 27a :b 2b0
:3 2b1 2b0 :4 2b4 :8 2b5
2b4 :5 2b6 2b3 2b8
:5 2b9 :3 2b8 :2 2ba :5 2bc
:11 2bd :24 2be :3 2bd :2 2bb
:6 2ba 2b7 :4 2b2 :3 2b0
:9 2c5 :6 2c8 :4 2c9 2c8
:6 2ca :5 2cb 2ca :5 2cd
2cc :3 2ca 2c7 2d0
:5 2d1 :3 2d0 :2 2d2 :5 2d4
:11 2d5 :24 2d6 :3 2d5 :2 2d3
:6 2d2 2cf :3 2c5 :2 2dd
:4 2df :5 2e2 :2 2e3 :4 2e6
2dd :6 2e7 :5 2e8 2e7
:5 2ea 2e9 :3 2e7 2dc
2ed :5 2ee :3 2ed :2 2ef
:5 2f1 :11 2f2 :24 2f3 :3 2f2
:2 2f0 :6 2ef 2ec :4 2c5
:2 2fa :4 2fc :5 2ff :2 300
:4 303 2fa :6 304 :5 305
304 :5 307 306 :3 304
2f9 30a :5 30b :3 30a
:2 30c :5 30e :11 30f :24 310
:3 30f :2 30d :6 30c 309
:4 2f7 :3 2c5 315 27a
:a 315 :3 317 :3 318 317
:5 319 316 31b :5 31c
:3 31b :2 31d :5 31f :11 320
:24 321 :3 320 :2 31e :6 31d
31a :3 315 :b 327 :b 328
329 :3 328 :3 32b :5 32c
:2 32d 32b :6 32e :5 32f
32e :5 331 330 :3 32e
32a 334 :5 335 :3 334
:2 336 :5 338 :11 339 :24 33a
:3 339 :2 337 :6 336 333
:3 327 :6 33f 340 327
:12 340 :3 342 :6 343 :2 344
342 :6 345 :5 346 345
:5 348 347 :3 345 341
34b :5 34c :3 34b :2 34d
:5 34f :11 350 :24 351 :3 350
:2 34e :6 34d 34a :3 340
:6 356 340 327 :3 359
:6 35a :2 35b 359 :6 35c
:5 35d 35c :5 35f 35e
:3 35c 358 362 :5 363
:3 362 :2 364 :5 366 :11 367
:24 368 :3 367 :2 365 :6 364
361 :3 357 :6 36d 357
:3 327 315 :3 27a 265
:3 261 223 :3 373 372
:3 223 :3 379 :3 37a :3 37b
:5 37d :3 380 :5 381 382
:b 383 :2 384 380 :5 385
37f 387 :5 388 :3 387
:2 389 :5 38b :11 38c :24 38d
:3 38c :2 38a :6 389 386
:3 8d :a 392 :3 393 :3 392
:16 396 :7 397 :5 398 :3 397
:9 39a :f 39b :f 39c :3 39a
:9 39e :f 39f :f 3a0 :3 39e
:3 396 :3 3a6 :3 3a7 :3 3a8
:4 3ac :3 3ad 3ac :5 3ae
3ab 3b0 :5 3b1 :3 3b0
:2 3b2 :5 3b4 :11 3b5 :24 3b6
:3 3b5 :2 3b3 :6 3b2 3af
:3 8d :b 3bc :4 3be :6 3bf
3be :5 3c0 3bd 3c2
:5 3c3 :3 3c2 :2 3c4 :5 3c6
:11 3c7 :24 3c8 :3 3c7 :2 3c5
:6 3c4 3c1 :3 3bc :9 3ce
:3 3d0 3ce :3 3d2 3d1
:3 3ce :3 3bc :9 3d6 :b 3d7
:b 3d9 :8 3da :7 3db :8 3dc
3dd :5 3de 3d9 :5 3df
3d8 3e1 :5 3e2 :3 3e1
:2 3e3 :5 3e5 :11 3e6 :24 3e7
:3 3e6 :2 3e4 :6 3e3 3e0
:3 3d7 :3 3ed :3 3f0 :9 3f1
:15 3f2 :2 3f3 :7 3f2 :c 3f4
:2 3f5 :2 3f4 :3 3f5 :3 3f4
:4 3f5 :3 3f4 :11 3f6 :30 3f7
:3 3f6 :11 3f9 :b 3fa :3 3f9
:a 3fc 3f5 :3 3fe :9 400
:3 401 :3 400 3fd :3 3f4
3f3 :2 3f2 :3 406 :3 407
:3 408 :3 409 :3 40b :3 40d
:3 40e :3 40f :3 410 :3 3f1
:12 414 :a 415 :6 416 :7 417
:4 418 :3 419 :3 417 :3 415
:9 41c :4 41e :6 41f :5 420
41f :5 422 421 :3 41f
41d 425 :5 426 :3 425
:2 427 :5 429 :11 42a :24 42b
:3 42a :2 428 :6 427 424
:3 41c :6 432 :6 433 :5 434
433 :5 436 435 :3 433
431 439 :5 43a :3 439
:2 43b :5 43d :11 43e :24 43f
:3 43e :2 43c :6 43b 438
:3 41c :3 445 :4 447 :4 448
447 :5 449 446 44b
:5 44c :3 44b :2 44d :5 44f
:11 450 :24 451 :3 450 :2 44e
:6 44d 44a :3 41c :c 456
:8 458 459 45a 458
:5 45b 457 45d :5 45e
:3 45d :2 45f :5 461 :11 462
:24 463 :3 462 :2 460 :6 45f
45c :6 456 41c :16 469
:9 46a :3 46c :3 46d :2 46e
:2 46f :2 470 :2 471 :4 472
:4 473 :6 474 :2 475 46c
:6 476 :5 477 476 :5 479
478 :3 476 46b 47c
:5 47d :3 47c :2 47e :5 480
:11 481 :24 482 :3 481 :2 47f
:6 47e 47b :4 46a :3 488
:3 489 :2 48a :2 48b :2 48c
:2 48d :4 48e :4 48f :4 490
:6 491 :2 492 488 :6 493
:5 494 493 :5 496 495
:3 493 487 499 :5 49a
:3 499 :2 49b :5 49d :11 49e
:24 49f :3 49e :2 49c :6 49b
498 :4 486 :3 46a 4a4
469 :9 4a4 :3 4a6 :3 4a7
:2 4a8 :2 4a9 :4 4aa :4 4ab
:6 4ac :2 4ad 4a6 :6 4ae
:5 4af 4ae :5 4b1 4b0
:3 4ae 4a5 4b4 :5 4b5
:3 4b4 :2 4b6 :5 4b8 :11 4b9
:24 4ba :3 4b9 :2 4b7 :6 4b6
4b3 :3 4a4 4be 4a4
469 :9 4be :f 4bf :f 4c0
:9 4c2 :3 4c4 :3 4c5 :2 4c6
:2 4c7 :4 4c8 :6 4c9 :2 4ca
4c4 :6 4cb :5 4cc 4cb
:5 4ce 4cd :3 4cb 4c3
4d1 :5 4d2 :3 4d1 :2 4d3
:5 4d5 :11 4d6 :24 4d7 :3 4d6
:2 4d4 :6 4d3 4d0 :4 4c2
:3 4dd :3 4de :2 4df :2 4e0
:4 4e1 :4 4e2 :6 4e3 :2 4e4
4dd :6 4e5 :5 4e6 4e5
:5 4e8 4e7 :3 4e5 4dc
4eb :5 4ec :3 4eb :2 4ed
:5 4ef :11 4f0 :24 4f1 :3 4f0
:2 4ee :6 4ed 4ea :4 4db
:3 4c2 4be 469 :3 4f8
:4 4f9 :3 4fa :2 4fb 4f8
:6 4fc :5 4fd 4fc :5 4ff
4fe :3 4fc 4f7 502
:5 503 :3 502 :2 504 :5 506
:11 507 :24 508 :3 507 :2 505
:6 504 501 :4 4f6 :3 469
:6 50f :6 510 :5 511 510
:5 513 512 :3 510 50e
516 :5 517 :3 516 :2 518
:5 51a :11 51b :24 51c :3 51b
:2 519 :6 518 515 :3 468
522 :7 523 :3 524 522
:6 525 :5 526 525 :5 528
527 :3 525 521 52b
:5 52c :3 52b :2 52d :5 52f
:11 530 :24 531 :3 530 :2 52e
:6 52d 52a :4 468 :3 41c
:3 538 :4 53c :6 53d :5 53e
53d :5 540 53f :3 53d
53b 543 :5 544 :3 543
:2 545 :5 547 :11 548 :24 549
:3 548 :2 546 :6 545 542
:3 414 :6 550 :6 551 :5 552
551 :5 554 553 :3 551
54f 557 :5 558 :3 557
:2 559 :5 55b :11 55c :24 55d
:3 55c :2 55a :6 559 556
:3 414 :16 562 :3 563 :3 564
:b 565 :3 566 565 :3 568
567 :3 565 :9 56b :3 56c
:3 56b :e 56f :b 570 :2 571
:2 570 :a 571 :8 572 :3 56f
:b 574 :11 575 :20 576 :3 575
:11 578 :b 579 :3 578 :9 57b
57c 574 :9 57c :11 57d
:2e 57e :3 57d :11 580 :b 581
:3 580 :c 583 57c :3 574
:9 588 :5 589 588 :f 58b
58a :3 588 :3 592 :7 593
:6 594 :7 595 :6 596 :8 597
:3 593 :9 59b :11 59c :30 59d
:3 59c :11 59f :b 5a0 :3 59f
:a 5a2 5a3 59b :9 5a3
:11 5a4 :30 5a5 :3 5a4 :11 5a7
:b 5a8 :3 5a7 :7 5aa 5ab
:2 5aa 5a3 59b :3 5ad
:a 5ae :5 5af :3 5ae 5ac
:3 59b :3 562 :3 414 5b3
3d7 :a 5b3 :b 5b6 :7 5b7
5b8 :4 5b9 :3 5ba 5b6
:5 5bb 5b5 5bd :5 5be
:3 5bd :2 5bf :5 5c1 :11 5c2
:24 5c3 :3 5c2 :2 5c0 :6 5bf
5bc :3 5b3 :12 5c9 :6 5ca
:7 5cb :4 5cc :3 5cd :3 5cb
:3 5c9 5b3 3d7 :4 5d2
5d3 :4 5d4 :3 5d5 5d2
:5 5d6 5d1 5d8 :5 5d9
:3 5d8 :2 5da :5 5dc :11 5dd
:24 5de :3 5dd :2 5db :6 5da
5d7 :4 5cf :3 3d7 5e4
3d6 :a 5e4 :b 5e7 :7 5e8
5e9 :4 5ea :3 5eb 5e7
:5 5ec 5e6 5ee :5 5ef
:3 5ee :2 5f0 :5 5f2 :11 5f3
:24 5f4 :3 5f3 :2 5f1 :6 5f0
5ed :3 5e4 :12 5fa :6 5fb
:7 5fc :4 5fd :3 5fe :3 5fc
:3 5fa 5e4 3d6 :4 603
604 :4 605 :3 606 603
:5 607 602 609 :5 60a
:3 609 :2 60b :5 60d :11 60e
:24 60f :3 60e :2 60c :6 60b
608 :4 600 :3 3d6 :13 61a
:2 61b :2 61a :4 61b :3 61a
:3 61c :5 61d :9 621 :7 622
:3 621 :16 625 :16 626 :9 628
:8 629 :9 62a :a 62b :7 62c
:9 62d 628 :6 62e :5 62f
62e :5 631 630 :3 62e
627 634 :5 635 :3 634
:2 636 :5 638 :11 639 :24 63a
:3 639 :2 637 :6 636 633
:3 626 63e 626 :9 63e
:9 640 :8 641 :9 642 :a 643
:7 644 :9 645 640 :6 646
:5 647 646 :5 649 648
:3 646 63f 64c :5 64d
:3 64c :2 64e :5 650 :11 651
:24 652 :3 651 :2 64f :6 64e
64b :4 63e 626 :9 658
:8 659 :9 65a :a 65b :7 65c
:9 65d 658 :6 65e :5 65f
65e :5 661 660 :3 65e
657 664 :5 665 :3 664
:2 666 :5 668 :11 669 :24 66a
:3 669 :2 667 :6 666 663
:4 656 :3 626 625 :9 671
:8 672 :9 673 :a 674 :7 675
:9 676 671 :6 677 :5 678
677 :5 67a 679 :3 677
670 67d :5 67e :3 67d
:2 67f :5 681 :11 682 :24 683
:3 682 :2 680 :6 67f 67c
:4 66f :3 625 :3 68b 61b
:2 61a :c 68d :3 68e :3 68f
:6 696 :5 697 695 699
:5 69a :3 699 :2 69b :5 69d
:11 69e :24 69f :3 69e :2 69c
:6 69b 698 :3 8d :3 6a4
:3 6a5 :3 6a6 :12 6a9 :4 6ab
:4 6ac :6 6ad :27 6ae 6ab
:5 6af 6aa 6b1 :5 6b2
:3 6b1 :2 6b3 :5 6b5 :11 6b6
:24 6b7 :3 6b6 :2 6b4 :6 6b3
6b0 :6 6a9 :3 6c2 :5 6c6
6c8 :7 6c9 :5 6ca :5 6cb
:5 6cc :3 6c9 :3 6cd :5 6ce
6cd :5 6d0 6cf :3 6cd
:a 6d3 :2 6d4 :3 6d3 :9 6da
:7 6db :4 6de :6 6df :5 6e0
6df :5 6e2 6e1 :3 6df
6dd 6e5 :5 6e6 :3 6e5
:2 6e7 :5 6e9 :11 6ea :24 6eb
:3 6ea :2 6e8 :6 6e7 6e4
:3 6da :4 6f1 :6 6f2 :5 6f3
6f2 :5 6f5 6f4 :3 6f2
6f0 6f8 :5 6f9 :3 6f8
:2 6fa :5 6fc :11 6fd :24 6fe
:3 6fd :2 6fb :6 6fa 6f7
:3 6da :4 704 :6 705 :5 706
705 :5 708 707 :3 705
703 70b :5 70c :3 70b
:2 70d :5 70f :11 710 :24 711
:3 710 :2 70e :6 70d 70a
:3 6da :6 717 :6 718 :5 719
718 :5 71b 71a :3 718
716 71e :5 71f :3 71e
:2 720 :5 722 :11 723 :24 724
:3 723 :2 721 :6 720 71d
:3 6da :4 72a :6 72b :5 72c
72b :5 72e 72d :3 72b
729 731 :5 732 :3 731
:2 733 :5 735 :11 736 :24 737
:3 736 :2 734 :6 733 730
:3 6da :4 73d :6 73e :5 73f
73e :5 741 740 :3 73e
73c 744 :5 745 :3 744
:2 746 :5 748 :11 749 :24 74a
:3 749 :2 747 :6 746 743
:4 6da :3 750 :2 751 :2 752
:2 753 750 :6 754 :5 755
754 :5 757 756 :3 754
74f 75a :5 75b :3 75a
:2 75c :5 75e :11 75f :24 760
:3 75f :2 75d :6 75c 759
:3 74e :3 766 :6 767 768
766 :5 769 765 76b
:5 76c :3 76b :2 76d :5 76f
:11 770 :24 771 :3 770 :2 76e
:6 76d 76a :3 74e :a 778
:9 779 :3 77a :3 779 :6 77f
780 :5 781 77f :5 782
77e 784 :5 785 :3 784
:2 786 :5 788 :11 789 :24 78a
:3 789 :2 787 :6 786 783
:3 778 :a 791 :12 792 :3 793
795 :2 796 :2 797 798
:2 799 795 :6 79a :5 79b
79a :5 79d 79c :3 79a
794 7a0 :5 7a1 :3 7a0
:2 7a2 :5 7a4 :11 7a5 :24 7a6
:3 7a5 :2 7a3 :6 7a2 79f
:3 792 7aa 792 :b 7aa
:b 7ab 7ac :3 7ab 7af
:3 7b0 :2 7b1 :2 7b2 :2 7b3
7b4 :2 7b5 7af :6 7b6
:5 7b7 7b6 :5 7b9 7b8
:3 7b6 7ae 7bc :5 7bd
:3 7bc :2 7be :5 7c0 :11 7c1
:24 7c2 :3 7c1 :2 7bf :6 7be
7bb :4 7aa 792 7c8
:6 7c9 :2 7ca :2 7cb :2 7cc
7cd :2 7ce 7c8 :6 7cf
:5 7d0 7cf :5 7d2 7d1
:3 7cf 7c7 7d5 :5 7d6
:3 7d5 :2 7d7 :5 7d9 :11 7da
:24 7db :3 7da :2 7d8 :6 7d7
7d4 :4 7c6 :3 792 :16 7e1
:4 7e3 :6 7e4 :5 7e5 7e4
:5 7e7 7e6 :3 7e4 7e2
7ea :5 7eb :3 7ea :2 7ec
:5 7ee :11 7ef :24 7f0 :3 7ef
:2 7ed :6 7ec 7e9 :6 7e1
791 :8 7f7 :6 7f8 :9 7f9
:4 7fa :7 7fb 7f7 :5 7fc
7f6 7fe :5 7ff :3 7fe
:2 800 :5 802 :11 803 :24 804
:3 803 :2 801 :6 800 7fd
:3 7f5 :12 80a :b 80b 80c
:3 80b :4 80f :6 810 :5 811
810 :5 813 812 :3 810
80e 816 :5 817 :3 816
:2 818 :5 81a :11 81b :24 81c
:3 81b :2 819 :6 818 815
:3 80a :4 822 :5 823 :2 824
822 :6 825 :5 826 825
:5 828 827 :3 825 821
82b :5 82c :3 82b :2 82d
:5 82f :11 830 :24 831 :3 830
:2 82e :6 82d 82a :3 80a
:6 837 :2 838 837 :6 839
:5 83a 839 :5 83c 83b
:3 839 836 83f :5 840
:3 83f :2 841 :5 843 :11 844
:24 845 :3 844 :2 842 :6 841
83e :4 80a :b 84a 84b
:3 84a :9 84e :6 84f :5 850
84f :5 852 851 :3 84f
84d 855 :5 856 :3 855
:2 857 :5 859 :11 85a :24 85b
:3 85a :2 858 :6 857 854
:3 849 :4 861 :5 862 :2 863
861 :6 864 :5 865 864
:5 867 866 :3 864 860
86a :5 86b :3 86a :2 86c
:5 86e :11 86f :24 870 :3 86f
:2 86d :6 86c 869 :4 849
:3 80a 7f5 :3 791 :9 879
:12 87a :3 87c :4 87d 87c
:5 87e 87b 880 :5 881
:3 880 :2 882 :5 884 :11 885
:24 886 :3 885 :2 883 :6 882
87f :3 87a :16 88c :6 88e
:6 88f :5 890 88f :5 892
891 :3 88f 88d 895
:5 896 :3 895 :2 897 :5 899
:11 89a :24 89b :3 89a :2 898
:6 897 894 :3 88c 89f
88c :16 89f :4 8a1 :5 8a2
8a0 8a4 :5 8a5 :3 8a4
:2 8a6 :5 8a8 :11 8a9 :24 8aa
:3 8a9 :2 8a7 :6 8a6 8a3
:3 89f :11 8af :30 8b0 :3 8af
:11 8b2 :b 8b3 :3 8b2 :7 8b5
8b6 :2 8b5 89f :3 88c
:3 87a :6 8ba :9 8bb :3 8bc
:2 8bd :3 8bb :6 8f5 :6 8f6
:5 8f7 8f6 :5 8f9 8f8
:3 8f6 8f4 8fc :5 8fd
:3 8fc :2 8fe :5 900 :11 901
:24 902 :3 901 :2 8ff :6 8fe
8fb :3 879 :3 907 :5 90a
:3 90c :3 90d :3 90e :3 90f
:3 911 :3 913 :3 914 :3 915
:3 916 :12 918 :2 919 :2 918
:8 919 918 919 :2 91a
:2 919 :8 91a 919 :6 918
:9 91b 91d :4 91e :2 91f
:4 920 :4 921 :4 922 :4 923
:4 924 :2 925 :6 926 :2 927
91d :6 928 :5 929 928
:5 92b 92a :3 928 91c
92e :5 92f :3 92e :2 930
:5 932 :11 933 :24 934 :3 933
:2 931 :6 930 92d :4 91b
93a :4 93b :2 93c :4 93d
:4 93e :4 93f :4 940 :4 941
:2 942 :2 943 :6 944 :2 945
93a :6 946 :5 947 946
:5 949 948 :3 946 939
94c :5 94d :3 94c :2 94e
:5 950 :11 951 :24 952 :3 951
:2 94f :6 94e 94b :4 938
:3 91b 91a 95b :4 95c
:2 95d :4 95e :4 95f :4 960
:4 961 :8 962 :2 963 95b
:6 964 :5 965 964 :5 967
966 :3 964 95a 96a
:5 96b :3 96a :2 96c :5 96e
:11 96f :24 970 :3 96f :2 96d
:6 96c 969 :4 959 :3 918
:16 976 :3 977 978 976
:9 978 :7 979 978 :3 976
:e 97c :b 97d :2 97e :2 97d
:a 97e :8 97f :3 97c :b 981
:4 983 :5 984 982 986
:5 987 :3 986 :2 988 :5 98a
:11 98b :24 98c :3 98b :2 989
:6 988 985 :3 981 :11 991
:20 992 :3 991 :11 994 :b 995
:3 994 :9 997 998 981
:9 998 :4 99a :5 99b 999
99d :5 99e :3 99d :2 99f
:5 9a1 :11 9a2 :24 9a3 :3 9a2
:2 9a0 :6 99f 99c :3 998
:11 9a8 :2e 9a9 :3 9a8 :11 9ab
:b 9ac :3 9ab :c 9ae 998
:3 981 90a :15 9b3 :2 9b4
:2 9b3 :4 9b4 :2 9b3 :3 9b5
:7 9b6 9b4 :2 9b3 :1a 9b9
:7 9ba 9b9 :12 9bc 9bb
:3 9b9 :9 9bf :b 9c0 9bf
:11 9c2 9c1 :3 9bf 9cf
:4 9d0 :2 9d1 :4 9d2 :4 9d3
:4 9d4 :4 9d5 :6 9d6 :2 9d7
9cf :6 9d8 :5 9d9 9d8
:5 9db 9da :3 9d8 9ce
9de :5 9df :3 9de :2 9e0
:5 9e2 :11 9e3 :24 9e4 :3 9e3
:2 9e1 :6 9e0 9dd :4 9b0
:3 90a :3 9ea :3 9eb :3 9ec
:13 9ee :2 9ef :2 9ee :4 9ef
:5 9ee :5 9f0 9ef :f 9f2
9f1 :3 9ee :3 9f5 :3 9fc
:7 9fd :6 9fe :7 9ff :6 a00
:8 a01 :3 9fd :9 a05 :4 a07
:5 a08 a06 a0a :5 a0b
:3 a0a :2 a0c :5 a0e :11 a0f
:24 a10 :3 a0f :2 a0d :6 a0c
a09 :3 a05 :11 a15 :30 a16
:3 a15 :11 a18 :b a19 :3 a18
:a a1b a1c a05 :9 a1c
:4 a1e :5 a1f a1d a21
:5 a22 :3 a21 :2 a23 :5 a25
:11 a26 :24 a27 :3 a26 :2 a24
:6 a23 a20 :3 a1c :11 a2c
:30 a2d :3 a2c :11 a2f :b a30
:3 a2f :7 a32 a33 :2 a32
a1c :3 a05 :8 a3d :6 a3e
:5 a3f a3e :5 a41 a40
:3 a3e a3c a44 :5 a45
:3 a44 :2 a46 :5 a48 :11 a49
:24 a4a :3 a49 :2 a47 :6 a46
a43 :3 879 a51 :b a52
:c a53 a51 :6 a54 :5 a55
a54 :5 a57 a56 :3 a54
a50 a5a :5 a5b :3 a5a
:2 a5c :5 a5e :11 a5f :24 a60
:3 a5f :2 a5d :6 a5c a59
:3 879 a67 :7 a68 :3 a69
a67 :6 a6a :5 a6b a6a
:5 a6d a6c :3 a6a a66
a70 :5 a71 :3 a70 :2 a72
:5 a74 :11 a75 :24 a76 :3 a75
:2 a73 :6 a72 a6f :3 879
a7c :7 a7d :3 a7e a7c
:6 a7f :5 a80 a7f :5 a82
a81 :3 a7f a7b a85
:5 a86 :3 a85 :2 a87 :5 a89
:11 a8a :24 a8b :3 a8a :2 a88
:6 a87 a84 :3 879 :b a90
:f a91 :3 a93 :8 a94 :3 a96
:8 a97 :3 a9a :4 a9b a9a
:6 a9c :5 a9d a9c :5 a9f
a9e :3 a9c a99 aa2
:5 aa3 :3 aa2 :2 aa4 :5 aa6
:11 aa7 :24 aa8 :3 aa7 :2 aa5
:6 aa4 aa1 :3 879 :16 aae
:3 ab0 :2 ab1 :2 ab2 :2 ab3
:2 ab4 :4 ab5 :2 ab6 ab0
:6 ab7 :5 ab8 ab7 :5 aba
ab9 :3 ab7 aaf abd
:5 abe :3 abd :2 abf :5 ac1
:11 ac2 :24 ac3 :3 ac2 :2 ac0
:6 abf abc :3 aae ac7
aae :9 ac7 :3 ac9 :2 aca
:2 acb :4 acc :2 acd ac9
:6 ace :5 acf ace :5 ad1
ad0 :3 ace ac8 ad4
:5 ad5 :3 ad4 :2 ad6 :5 ad8
:11 ad9 :24 ada :3 ad9 :2 ad7
:6 ad6 ad3 :3 ac7 :7 adf
ae0 ac7 aae :9 ae0
:3 ae2 :2 ae3 :2 ae4 :4 ae5
:2 ae6 ae2 :6 ae7 :5 ae8
ae7 :5 aea ae9 :3 ae7
ae1 aed :5 aee :3 aed
:2 aef :5 af1 :11 af2 :24 af3
:3 af2 :2 af0 :6 aef aec
:3 ae0 :7 af8 ae0 aae
:3 afb :4 afc :2 afd afb
:6 afe :5 aff afe :5 b01
b00 :3 afe afa b04
:5 b05 :3 b04 :2 b06 :5 b08
:11 b09 :24 b0a :3 b09 :2 b07
:6 b06 b03 :3 af9 :7 b0f
:7 b10 af9 :3 aae :3 879
:3 778 74e :3 6da :a b24
6c8 b25 8d :4 b29
:5 b2a b28 b2c :5 b2d
:3 b2c :2 b2e :5 b30 :11 b31
:24 b32 :3 b31 :2 b2f :6 b2e
b2b :3 8d :3 b38 :9 b39
:3 b3a :d b3e :f b3f :16 b46
:6 b47 :2 b46 :3 b48 :3 b49
:3 b4a :3 b4c :3 b4e :3 b4f
:3 b50 :3 b51 :14 b53 :3 b54
b55 b53 :9 b55 :7 b56
b55 :3 b53 :e b59 :b b5a
:2 b5b :2 b5a :a b5b :8 b5c
:3 b59 :b b5e :11 b5f :20 b60
:3 b5f :11 b62 :b b63 :3 b62
:9 b65 b66 b5e :9 b66
:11 b67 :2e b68 :3 b67 :11 b6a
:b b6b :3 b6a :c b6d b66
:3 b5e :5 b71 :5 b72 :4 b75
b76 :4 b77 b78 b75
:6 b79 :5 b7a b79 :5 b7c
b7b :3 b79 b74 b7f
:5 b80 :3 b7f :2 b81 :5 b83
:11 b84 :24 b85 :3 b84 :2 b82
:6 b81 b7e :3 b47 :3 b8b
:3 b8e :3 b8f :3 b90 :3 b91
:13 b93 :2 b94 :2 b93 :4 b94
:5 b93 :15 b95 :2 b96 :2 b95
:4 b96 :2 b95 :3 b97 :7 b98
b96 :2 b95 :f b9a :3 b9b
b94 :3 b9d b9c :3 b93
:3 ba1 :7 ba2 :6 ba3 :7 ba4
:6 ba5 :8 ba6 :3 ba2 :9 baa
:11 bab :30 bac :3 bab :11 bae
:b baf :3 bae :a bb1 bb2
baa :9 bb2 :11 bb3 :30 bb4
:3 bb3 :11 bb6 :b bb7 :3 bb6
:7 bb9 bba :2 bb9 bb2
baa :3 bbd :3 bbe :5 bbf
:5 bc0 :5 bc1 :5 bc2 :2 bc3
bbd :6 bc4 :5 bc5 bc4
:5 bc7 bc6 :3 bc4 bbc
bca :5 bcb :3 bca :2 bcc
:5 bce :11 bcf :24 bd0 :3 bcf
:2 bcd :6 bcc bc9 :4 bbb
:3 baa b47 :2 b46 :3 b3e
:3 bd9 :5 bda bd9 :5 bdb
bd8 bdd :5 bde :3 bdd
:2 bdf :5 be1 :11 be2 :24 be3
:3 be2 :2 be0 :6 bdf bdc
:3 b39 :9 be9 :5 beb :4 bec
beb :6 bed :5 bee bed
:5 bf0 bef :3 bed bea
bf3 :5 bf4 :3 bf3 :2 bf5
:5 bf7 :11 bf8 :24 bf9 :3 bf8
:2 bf6 :6 bf5 bf2 :6 be9
:3 b39 :3 c00 :5 c03 :a c05
:6 c06 :5 c07 c06 :5 c09
c08 :3 c06 c04 c0c
:5 c0d :3 c0c :2 c0e :5 c10
:11 c11 :24 c12 :3 c11 :2 c0f
:6 c0e c0b :3 c03 :9 c18
:3 c1a :4 c1b c1a :5 c1c
c19 c1e :5 c1f :3 c1e
:2 c20 :5 c22 :11 c23 :24 c24
:3 c23 :2 c21 :6 c20 c1d
:3 c18 :9 c2a :3 c2c :5 c2d
c2c :6 c2e :5 c2f c2e
:5 c31 c30 :3 c2e c2b
c34 :5 c35 :3 c34 :2 c36
:5 c38 :11 c39 :24 c3a :3 c39
:2 c37 :6 c36 c33 :3 c2a
:6 c40 :6 c41 :5 c42 c41
:5 c44 c43 :3 c41 c3f
c47 :5 c48 :3 c47 :2 c49
:5 c4b :11 c4c :24 c4d :3 c4c
:2 c4a :6 c49 c46 :6 c2a
:3 c18 :9 c54 :3 c56 :a c57
:d c5a :9 c5b :3 c5c c5a
:5 c5d c59 c5f :5 c60
:3 c5f :2 c61 :5 c63 :11 c64
:24 c65 :3 c64 :2 c62 :6 c61
c5e :3 c54 :a c6b :9 c6c
c6e :2 c6f :2 c70 :2 c71
:3 c72 c73 :4 c74 c6e
:6 c75 :5 c76 c75 :5 c78
c77 :3 c75 c6d c7b
:5 c7c :3 c7b :2 c7d :5 c7f
:11 c80 :24 c81 :3 c80 :2 c7e
:6 c7d c7a :4 c6c c87
:2 c88 :2 c89 :2 c8a :3 c8b
:3 c8c :4 c8d c87 :6 c8e
:5 c8f c8e :5 c91 c90
:3 c8e c86 c94 :5 c95
:3 c94 :2 c96 :5 c98 :11 c99
:24 c9a :3 c99 :2 c97 :6 c96
c93 :4 c85 :3 c6c :b ca1
:3 ca2 :7 ca3 :4 ca4 :3 ca5
:3 ca3 :3 ca1 :3 c6b :a caa
cab :2 caa :4 cab :3 caa
:3 cac cab :2 caa :3 c54
:3 c03 :12 cb2 :9 cb4 :3 cb6
:4 cb7 :3 cb8 cb6 :5 cb9
cb5 cbb :5 cbc :3 cbb
:2 cbd :5 cbf :11 cc0 :24 cc1
:3 cc0 :2 cbe :6 cbd cba
:3 cb4 :9 cc6 cc9 cca
:4 ccb :3 ccc cc9 :6 ccd
:5 cce ccd :5 cd0 ccf
:3 ccd cc8 cd3 :5 cd4
:3 cd3 :2 cd5 :5 cd7 :11 cd8
:24 cd9 :3 cd8 :2 cd6 :6 cd5
cd2 :3 cc6 cdf :2 ce0
:4 ce1 :2 ce2 cdf :6 ce3
:5 ce4 ce3 :5 ce6 ce5
:3 ce3 cde ce9 :5 cea
:3 ce9 :2 ceb :5 ced :11 cee
:24 cef :3 cee :2 cec :6 ceb
ce8 :6 cc6 :3 cb4 :3 cf7
:a cf8 :d cfb :9 cfc :3 cfd
cfb :5 cfe cfa d00
:5 d01 :3 d00 :2 d02 :5 d04
:11 d05 :24 d06 :3 d05 :2 d03
:6 d02 cff :3 cb2 :a d0c
:9 d0d d0f :2 d10 :2 d11
:2 d12 :3 d13 d14 :4 d15
d0f :6 d16 :5 d17 d16
:5 d19 d18 :3 d16 d0e
d1c :5 d1d :3 d1c :2 d1e
:5 d20 :11 d21 :24 d22 :3 d21
:2 d1f :6 d1e d1b :4 d0d
d28 :2 d29 :2 d2a :2 d2b
:3 d2c :3 d2d :4 d2e d28
:6 d2f :5 d30 d2f :5 d32
d31 :3 d2f d27 d35
:5 d36 :3 d35 :2 d37 :5 d39
:11 d3a :24 d3b :3 d3a :2 d38
:6 d37 d34 :4 d26 :3 d0d
:17 d42 :3 d43 :7 d44 :4 d45
:3 d46 :3 d44 :3 d42 :3 d0c
:3 cb2 :6 d4c :7 d4f :5 d50
:3 d4f :11 d54 :30 d55 :3 d54
:a d57 8d :3 d5a :11 d5b
:b d5c :17 d5d :3 d5b :2 d5f
:3 d60 :3 d61 d62 d60
d5f d60 :3 d63 :3 d5a
:2 d64 :5 d66 :11 d67 :b d68
:18 d69 :3 d67 :2 d6b d6c
:3 d6d d6e d6c d6b
d6c :3 d6f :2 d65 :6 d64
d59 :7 1 
114f1
4
:3 0 1 :3 0 2
:a 0 4f2d 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 4
:3 0 5 :7 0 9
8 :3 0 4 :3 0
6 :7 0 d c
:3 0 d 7c 0
b 4 :3 0 7
:7 0 11 10 :3 0
4 :3 0 8 :7 0
15 14 :3 0 11
a2 0 f a
:3 0 9 :7 0 19
18 :3 0 a :3 0
b :7 0 1d 1c
:3 0 15 c8 0
13 a :3 0 c
:7 0 21 20 :3 0
a :3 0 d :7 0
25 24 :3 0 19
ee 0 17 a
:3 0 e :7 0 29
28 :3 0 a :3 0
f :7 0 2d 2c
:3 0 1d 114 0
1b a :3 0 10
:7 0 31 30 :3 0
a :3 0 11 :7 0
35 34 :3 0 21
:2 0 1f a :3 0
12 :7 0 39 38
:3 0 4 :3 0 13
:7 0 3d 3c :3 0
14 :3 0 15 :3 0
16 :2 0 4 41
42 0 3f 43
0 4f2d 2 44
:2 0 18 :2 0 33
4 :3 0 18 :2 0
31 47 49 :6 0
4c 4a 0 4f2b
0 17 :6 0 18
:2 0 37 4 :3 0
35 4e 50 :6 0
53 51 0 4f2b
0 19 :6 0 18
:2 0 3b 4 :3 0
39 55 57 :6 0
5a 58 0 4f2b
0 1a :6 0 18
:2 0 3f 4 :3 0
3d 5c 5e :6 0
61 5f 0 4f2b
0 1b :6 0 18
:2 0 43 4 :3 0
41 63 65 :6 0
68 66 0 4f2b
0 1c :6 0 18
:2 0 47 4 :3 0
45 6a 6c :6 0
6f 6d 0 4f2b
0 1d :6 0 18
:2 0 4b 4 :3 0
49 71 73 :6 0
76 74 0 4f2b
0 1e :6 0 18
:2 0 4f 4 :3 0
4d 78 7a :6 0
7d 7b 0 4f2b
0 1f :6 0 18
:2 0 53 4 :3 0
51 7f 81 :6 0
84 82 0 4f2b
0 20 :6 0 18
:2 0 57 4 :3 0
55 86 88 :6 0
8b 89 0 4f2b
0 21 :6 0 18
:2 0 5b 4 :3 0
59 8d 8f :6 0
92 90 0 4f2b
0 22 :6 0 61
2b0 0 5f 4
:3 0 5d 94 96
:6 0 99 97 0
4f2b 0 23 :6 0
65 2e4 0 63
25 :3 0 9b :7 0
9e 9c 0 4f2b
0 24 :6 0 25
:3 0 a0 :7 0 a3
a1 0 4f2b 0
26 :6 0 2a :2 0
67 25 :3 0 a5
:7 0 a8 a6 0
4f2b 0 27 :6 0
25 :3 0 aa :7 0
ad ab 0 4f2b
0 28 :6 0 6d
335 0 6b 4
:3 0 69 af b1
:6 0 b4 b2 0
4f2b 0 29 :6 0
2a :2 0 6f 2c
:3 0 b6 :7 0 b9
b7 0 4f2b 0
2b :6 0 2c :3 0
bb :7 0 be bc
0 4f2b 0 2d
:6 0 30 :2 0 73
4 :3 0 71 c0
c2 :6 0 c5 c3
0 4f2b 0 2e
:6 0 79 3a3 0
77 4 :3 0 75
c7 c9 :6 0 cc
ca 0 4f2b 0
2f :6 0 7d 3d7
0 7b 25 :3 0
ce :7 0 d1 cf
0 4f2b 0 31
:6 0 25 :3 0 d3
:7 0 d6 d4 0
4f2b 0 32 :6 0
18 :2 0 7f 25
:3 0 d8 :7 0 db
d9 0 4f2b 0
33 :6 0 25 :3 0
dd :7 0 e0 de
0 4f2b 0 34
:6 0 37 :2 0 83
4 :3 0 81 e2
e4 :6 0 e7 e5
0 4f2b 0 35
:6 0 37 :2 0 87
4 :3 0 85 e9
eb :6 0 ee ec
0 4f2b 0 36
:6 0 37 :2 0 8b
4 :3 0 89 f0
f2 :6 0 f5 f3
0 4f2b 0 38
:6 0 3b :2 0 8f
4 :3 0 8d f7
f9 :6 0 fc fa
0 4f2b 0 39
:6 0 3d :2 0 93
4 :3 0 91 fe
100 :6 0 103 101
0 4f2b 0 3a
:6 0 3d :2 0 95
a :3 0 105 :7 0
109 106 107 4f2b
0 3c :6 0 3d
:2 0 97 a :3 0
10b :7 0 10f 10c
10d 4f2b 0 3e
:6 0 3d :2 0 99
a :3 0 111 :7 0
115 112 113 4f2b
0 3f :6 0 3d
:2 0 9b a :3 0
117 :7 0 11b 118
119 4f2b 0 40
:6 0 3d :2 0 9d
a :3 0 11d :7 0
121 11e 11f 4f2b
0 41 :6 0 3d
:2 0 9f a :3 0
123 :7 0 127 124
125 4f2b 0 42
:6 0 3d :2 0 a1
a :3 0 129 :7 0
12d 12a 12b 4f2b
0 43 :6 0 3d
:2 0 a3 a :3 0
12f :7 0 133 130
131 4f2b 0 44
:6 0 3d :2 0 a5
a :3 0 135 :7 0
139 136 137 4f2b
0 45 :6 0 3d
:2 0 a7 a :3 0
13b :7 0 13f 13c
13d 4f2b 0 46
:6 0 3d :2 0 a9
a :3 0 141 :7 0
145 142 143 4f2b
0 47 :6 0 3d
:2 0 ab a :3 0
147 :7 0 14b 148
149 4f2b 0 48
:6 0 3d :2 0 ad
a :3 0 14d :7 0
151 14e 14f 4f2b
0 49 :6 0 3d
:2 0 af a :3 0
153 :7 0 157 154
155 4f2b 0 4a
:6 0 3d :2 0 b1
a :3 0 159 :7 0
15d 15a 15b 4f2b
0 4b :6 0 3d
:2 0 b3 a :3 0
15f :7 0 163 160
161 4f2b 0 4c
:6 0 3d :2 0 b5
a :3 0 165 :7 0
169 166 167 4f2b
0 4d :6 0 3d
:2 0 b7 a :3 0
16b :7 0 16f 16c
16d 4f2b 0 4e
:6 0 3d :2 0 b9
a :3 0 171 :7 0
175 172 173 4f2b
0 4f :6 0 3d
:2 0 bb a :3 0
177 :7 0 17b 178
179 4f2b 0 50
:6 0 3d :2 0 bd
a :3 0 17d :7 0
181 17e 17f 4f2b
0 51 :6 0 3d
:2 0 bf a :3 0
183 :7 0 187 184
185 4f2b 0 52
:6 0 3d :2 0 c1
a :3 0 189 :7 0
18d 18a 18b 4f2b
0 53 :6 0 3d
:2 0 c3 a :3 0
18f :7 0 193 190
191 4f2b 0 54
:6 0 3d :2 0 c5
a :3 0 195 :7 0
199 196 197 4f2b
0 55 :6 0 3d
:2 0 c7 a :3 0
19b :7 0 19f 19c
19d 4f2b 0 56
:6 0 5a :2 0 c9
a :3 0 1a1 :7 0
1a5 1a2 1a3 4f2b
0 57 :6 0 5a
:2 0 cb 59 :3 0
1a7 :7 0 1ab 1a8
1a9 4f2b 0 58
:6 0 5a :2 0 cd
59 :3 0 1ad :7 0
1b1 1ae 1af 4f2b
0 5b :6 0 5a
:2 0 cf 59 :3 0
1b3 :7 0 1b7 1b4
1b5 4f2b 0 5c
:6 0 5a :2 0 d1
59 :3 0 1b9 :7 0
1bd 1ba 1bb 4f2b
0 5d :6 0 3d
:2 0 d3 59 :3 0
1bf :7 0 1c3 1c0
1c1 4f2b 0 5e
:6 0 3d :2 0 d5
a :3 0 1c5 :7 0
1c9 1c6 1c7 4f2b
0 5f :6 0 3d
:2 0 d7 a :3 0
1cb :7 0 1cf 1cc
1cd 4f2b 0 60
:6 0 3d :2 0 d9
a :3 0 1d1 :7 0
1d5 1d2 1d3 4f2b
0 61 :6 0 3d
:2 0 db a :3 0
1d7 :7 0 1db 1d8
1d9 4f2b 0 62
:6 0 3d :2 0 dd
a :3 0 1dd :7 0
1e1 1de 1df 4f2b
0 63 :6 0 3d
:2 0 df a :3 0
1e3 :7 0 1e7 1e4
1e5 4f2b 0 64
:6 0 3d :2 0 e1
a :3 0 1e9 :7 0
1ed 1ea 1eb 4f2b
0 65 :6 0 68
:2 0 e3 a :3 0
1ef :7 0 1f3 1f0
1f1 4f2b 0 66
:6 0 68 :2 0 e7
4 :3 0 e5 1f5
1f7 :6 0 1fa 1f8
0 4f2b 0 67
:6 0 3d :2 0 eb
4 :3 0 e9 1fc
1fe :6 0 201 1ff
0 4f2b 0 69
:6 0 3d :2 0 ed
59 :3 0 203 :7 0
207 204 205 4f2b
0 6a :6 0 2a
:2 0 ef a :3 0
209 :7 0 20d 20a
20b 4f2b 0 6b
:6 0 3d :2 0 f3
4 :3 0 f1 20f
211 :6 0 214 212
0 4f2b 0 6c
:6 0 18 :2 0 f5
a :3 0 216 :7 0
21a 217 218 4f2b
0 6d :6 0 2a
:2 0 f9 4 :3 0
f7 21c 21e :6 0
221 21f 0 4f2b
0 6e :6 0 3d
:2 0 fd 4 :3 0
fb 223 225 :6 0
228 226 0 4f2b
0 6f :6 0 3d
:2 0 ff a :3 0
22a :7 0 22e 22b
22c 4f2b 0 70
:6 0 103 a19 0
101 a :3 0 230
:7 0 234 231 232
4f2b 0 71 :6 0
3d :2 0 105 2c
:3 0 236 :7 0 239
237 0 4f2b 0
72 :6 0 2c :3 0
23b :7 0 23e 23c
0 4f2b 0 73
:6 0 3d :2 0 107
59 :3 0 240 :7 0
244 241 242 4f2b
0 74 :6 0 3d
:2 0 109 59 :3 0
246 :7 0 24a 247
248 4f2b 0 75
:6 0 3d :2 0 10b
a :3 0 24c :7 0
250 24d 24e 4f2b
0 76 :6 0 3d
:2 0 10d a :3 0
252 :7 0 256 253
254 4f2b 0 77
:6 0 3d :2 0 10f
a :3 0 258 :7 0
25c 259 25a 4f2b
0 78 :6 0 3d
:2 0 111 a :3 0
25e :7 0 262 25f
260 4f2b 0 79
:6 0 3d :2 0 113
a :3 0 264 :7 0
268 265 266 4f2b
0 7a :6 0 3d
:2 0 115 a :3 0
26a :7 0 26e 26b
26c 4f2b 0 7b
:6 0 3d :2 0 117
a :3 0 270 :7 0
274 271 272 4f2b
0 7c :6 0 3d
:2 0 119 a :3 0
276 :7 0 27a 277
278 4f2b 0 7d
:6 0 18 :2 0 11b
a :3 0 27c :7 0
280 27d 27e 4f2b
0 7e :6 0 18
:2 0 11f 4 :3 0
11d 282 284 :6 0
287 285 0 4f2b
0 7f :6 0 18
:2 0 123 4 :3 0
121 289 28b :6 0
28e 28c 0 4f2b
0 80 :6 0 3d
:2 0 127 4 :3 0
125 290 292 :6 0
295 293 0 4f2b
0 81 :6 0 3d
:2 0 12e a :3 0
297 0 2 :5 0
83 :3 0 84 :3 0
42 :3 0 129 29a
29d 12c 299 29f
2a0 :2 0 2a3 298
2a1 4f2b 0 82
:6 0 3d :2 0 130
a :3 0 2a5 0
2 :5 0 2a9 2a6
2a7 4f2b 0 85
:9 0 132 a :3 0
2ab 0 2 :5 0
2af 2ac 2ad 4f2b
0 86 :6 0 87
:3 0 88 :a 0 2
2bb :4 0 2b1 2b4
0 2b2 :3 0 89
:3 0 8a :3 0 8b
:3 0 8c :3 0 21
:4 0 8d 1 :8 0
2bc 2b1 2b4 2bd
0 4f2b 134 2bd
2bf 2bc 2be :6 0
2bb :7 0 2bd 87
:3 0 8e :a 0 3
2cf :5 0 2c1 2c4
0 2c2 :3 0 8f
:3 0 90 :3 0 8b
:3 0 91 :3 0 22
:3 0 92 :3 0 21
:3 0 93 :3 0 94
:4 0 95 1 :8 0
2d0 2c1 2c4 2d1
0 4f2b 136 2d1
2d3 2d0 2d2 :6 0
2cf :7 0 2d1 87
:3 0 96 :a 0 4
347 :5 0 2d5 2d8
0 2d6 :3 0 97
:3 0 98 :3 0 99
:3 0 9a :3 0 99
:3 0 9b :3 0 97
:3 0 9c :3 0 99
:3 0 9d :3 0 99
:3 0 9e :3 0 9f
:3 0 a0 :3 0 99
:3 0 a1 :3 0 9f
:3 0 a0 :3 0 99
:3 0 a2 :3 0 99
:3 0 a1 :3 0 99
:3 0 a2 :3 0 99
:3 0 a3 :3 0 a4
:3 0 a3 :3 0 99
:3 0 a3 :3 0 99
:3 0 94 :3 0 99
:3 0 a5 :3 0 99
:3 0 a6 :3 0 99
:3 0 a7 :3 0 99
:3 0 a8 :3 0 99
:3 0 a9 :3 0 99
:3 0 aa :3 0 99
:3 0 ab :3 0 99
:3 0 ac :3 0 99
:3 0 ad :3 0 ae
:3 0 a4 :3 0 af
:3 0 97 :3 0 90
:3 0 99 :3 0 97
:3 0 91 :3 0 22
:3 0 97 :3 0 92
:3 0 23 :3 0 97
:3 0 93 :3 0 99
:3 0 98 :3 0 97
:3 0 98 :3 0 97
:3 0 8c :3 0 a4
:3 0 8c :3 0 97
:3 0 9c :3 0 b0
:3 0 b1 :3 0 b0
:3 0 b2 :3 0 b0
:3 0 b3 :3 0 b0
:3 0 b4 :3 0 b0
:3 0 b5 :3 0 b0
:3 0 b6 :3 0 b0
:3 0 b7 :3 0 b0
:3 0 b8 :3 0 b0
:3 0 b9 :3 0 b0
:3 0 ba :3 0 b0
:3 0 bb :3 0 b0
:3 0 bc :3 0 b0
:3 0 bd :3 0 b0
:3 0 be :3 0 b0
:3 0 bf :3 0 a4
:3 0 c0 :3 0 2f
:3 0 99 :3 0 a1
:4 0 c1 1 :8 0
348 2d5 2d8 349
0 4f2b 138 349
34b 348 34a :6 0
347 :7 0 349 353
354 0 13a 15
:3 0 16 :2 0 4
34d 34e 0 34f
:7 0 352 350 0
4f2b 0 c2 :6 0
c3 :3 0 c4 :3 0
c3 :3 0 c5 :3 0
356 357 0 2
:4 0 13c 358 35a
355 35c 35b :2 0
c6 :3 0 c7 :3 0
35e 35f 0 c8
:4 0 c3 :3 0 c9
:3 0 362 363 0
2 :4 0 13e 364
366 ca :4 0 cb
:2 0 cc :3 0 140
369 36b :3 0 cd
:4 0 cb :2 0 ce
:4 0 143 36e 370
:3 0 cb :2 0 3
:3 0 146 372 374
:3 0 cb :2 0 ce
:4 0 149 376 378
:3 0 cb :2 0 cf
:4 0 14c 37a 37c
:3 0 cb :2 0 ce
:4 0 14f 37e 380
:3 0 cb :2 0 5
:3 0 152 382 384
:3 0 cb :2 0 ce
:4 0 155 386 388
:3 0 cb :2 0 cf
:4 0 158 38a 38c
:3 0 cb :2 0 ce
:4 0 15b 38e 390
:3 0 cb :2 0 6
:3 0 15e 392 394
:3 0 cb :2 0 ce
:4 0 161 396 398
:3 0 cb :2 0 cf
:4 0 164 39a 39c
:3 0 cb :2 0 ce
:4 0 167 39e 3a0
:3 0 cb :2 0 7
:3 0 16a 3a2 3a4
:3 0 cb :2 0 ce
:4 0 16d 3a6 3a8
:3 0 cb :2 0 cf
:4 0 170 3aa 3ac
:3 0 cb :2 0 ce
:4 0 173 3ae 3b0
:3 0 cb :2 0 8
:3 0 176 3b2 3b4
:3 0 cb :2 0 ce
:4 0 179 3b6 3b8
:3 0 cb :2 0 cf
:4 0 17c 3ba 3bc
:3 0 cb :2 0 84
:3 0 d0 :3 0 9
:3 0 17f 3c0 3c2
d1 :4 0 181 3bf
3c5 184 3be 3c7
:3 0 cb :2 0 cf
:4 0 187 3c9 3cb
:3 0 cb :2 0 84
:3 0 d0 :3 0 b
:3 0 18a 3cf 3d1
d1 :4 0 18c 3ce
3d4 18f 3cd 3d6
:3 0 cb :2 0 cf
:4 0 192 3d8 3da
:3 0 cb :2 0 84
:3 0 d0 :3 0 c
:3 0 195 3de 3e0
d1 :4 0 197 3dd
3e3 19a 3dc 3e5
:3 0 cb :2 0 cf
:4 0 19d 3e7 3e9
:3 0 cb :2 0 84
:3 0 d0 :3 0 d
:3 0 1a0 3ed 3ef
d1 :4 0 1a2 3ec
3f2 1a5 3eb 3f4
:3 0 cb :2 0 cf
:4 0 1a8 3f6 3f8
:3 0 cb :2 0 84
:3 0 d0 :3 0 e
:3 0 1ab 3fc 3fe
d1 :4 0 1ad 3fb
401 1b0 3fa 403
:3 0 cb :2 0 cf
:4 0 1b3 405 407
:3 0 cb :2 0 84
:3 0 d0 :3 0 f
:3 0 1b6 40b 40d
d1 :4 0 1b8 40a
410 1bb 409 412
:3 0 cb :2 0 cf
:4 0 1be 414 416
:3 0 cb :2 0 84
:3 0 d0 :3 0 10
:3 0 1c1 41a 41c
d1 :4 0 1c3 419
41f 1c6 418 421
:3 0 cb :2 0 cf
:4 0 1c9 423 425
:3 0 cb :2 0 84
:3 0 d0 :3 0 11
:3 0 1cc 429 42b
d1 :4 0 1ce 428
42e 1d1 427 430
:3 0 cb :2 0 cf
:4 0 1d4 432 434
:3 0 cb :2 0 84
:3 0 d0 :3 0 12
:3 0 1d7 438 43a
d1 :4 0 1d9 437
43d 1dc 436 43f
:3 0 cb :2 0 cf
:4 0 1df 441 443
:3 0 cb :2 0 ce
:4 0 1e2 445 447
:3 0 cb :2 0 13
:3 0 1e5 449 44b
:3 0 cb :2 0 ce
:4 0 1e8 44d 44f
:3 0 cb :2 0 d2
:4 0 1eb 451 453
:3 0 1ee 360 455
:2 0 457 1f3 458
35d 457 0 459
1f5 0 4e8b c3
:3 0 c4 :3 0 45a
45b 0 c3 :3 0
d3 :3 0 45d 45e
0 2a :2 0 2
:4 0 d4 :2 0 d5
:2 0 2a :2 0 1f7
463 465 :3 0 1fa
45f 467 45c 469
468 :2 0 c6 :3 0
d6 :3 0 46b 46c
0 c8 :4 0 d7
:4 0 cb :2 0 cc
:3 0 1fe 470 472
:3 0 201 46d 474
:2 0 476 204 477
46a 476 0 478
206 0 4e8b 20
:3 0 3 :3 0 479
47a 0 4e8b 23
:3 0 5 :3 0 47c
47d 0 4e8b 21
:3 0 6 :3 0 47f
480 0 4e8b 1a
:3 0 13 :3 0 482
483 0 4e8b 31
:3 0 7 :3 0 485
486 0 4e8b 52
:3 0 11 :3 0 488
489 0 4e8b 52
:3 0 d8 :3 0 84
:3 0 52 :3 0 3d
:2 0 208 48d 490
b0 :3 0 d9 :3 0
492 493 0 20b
48c 495 48b 496
0 4e8b 84 :3 0
52 :3 0 3d :2 0
20e 498 49b b0
:3 0 da :2 0 db
:3 0 49d 49f 0
213 49e 4a1 :3 0
4a2 :2 0 84 :3 0
52 :3 0 3d :2 0
216 4a4 4a7 b0
:3 0 da :2 0 dc
:3 0 4a9 4ab 0
21b 4aa 4ad :3 0
4ae :2 0 4a3 4b0
4af :2 0 52 :3 0
dd :2 0 4b2 4b3
0 4b5 21e 4b6
4b1 4b5 0 4b7
220 0 4e8b 32
:3 0 31 :3 0 4b8
4b9 0 4e8b 3a
:3 0 de :4 0 4bb
4bc 0 4e8b 29
:3 0 de :4 0 4be
4bf 0 4e8b 45
:3 0 3d :2 0 4c1
4c2 0 4e8b 26
:3 0 de :4 0 4c4
4c5 0 4e8b 33
:4 0 4c7 4c8 0
4e8b 34 :4 0 4ca
4cb 0 4e8b df
:3 0 88 :4 0 4d0
:2 0 4e8b 4ce 4d1
:3 0 88 :3 0 56
:4 0 4d5 :2 0 4e8b
4d2 4d3 :3 0 88
:3 0 e0 :3 0 4d6
4d7 :3 0 15 :3 0
e1 :3 0 4d9 4da
0 3d :2 0 4db
4dc 0 4de 222
4e5 15 :3 0 e1
:3 0 4df 4e0 0
e2 :2 0 4e1 4e2
0 4e4 224 4e6
4d8 4de 0 4e7
0 4e4 0 4e7
226 0 4e8b 56
:3 0 15 :3 0 e1
:3 0 4e9 4ea 0
4e8 4eb 0 4e8b
e3 :3 0 88 :4 0
4f0 :2 0 4f6 4ee
0 15 :3 0 e1
:3 0 4f1 4f2 0
e4 :3 0 4f3 4f4
0 4f6 229 549
e5 :3 0 15 :3 0
e1 :3 0 4f8 4f9
0 e4 :3 0 4fa
4fb 0 4fd 22c
4ff 22e 4fe 4fd
:2 0 547 e6 :3 0
15 :3 0 e1 :3 0
502 503 0 e4
:3 0 504 505 0
53f c3 :3 0 c4
:3 0 507 508 0
c3 :3 0 d3 :3 0
50a 50b 0 e7
:2 0 2 :4 0 e8
:2 0 d5 :2 0 2a
:2 0 230 510 512
:3 0 233 50c 514
509 516 515 :2 0
c6 :3 0 d6 :3 0
518 519 0 c8
:4 0 e9 :4 0 cb
:2 0 cc :3 0 237
51d 51f :3 0 cb
:2 0 ea :4 0 23a
521 523 :3 0 cb
:2 0 eb :4 0 23d
525 527 :3 0 cb
:2 0 e4 :3 0 240
529 52b :3 0 cb
:2 0 ec :4 0 243
52d 52f :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 246 532 536
24a 531 538 :3 0
24d 51a 53a :2 0
53c 250 53d 517
53c 0 53e 252
0 53f 254 542
:3 0 542 0 542
541 53f 540 :6 0
544 5 :3 0 257
546 259 545 544
:2 0 547 25b :2 0
549 0 549 548
4f6 547 :6 0 4e8b
1 :3 0 84 :3 0
56 :3 0 3d :2 0
25e 54b 54e da
:2 0 e2 :2 0 263
550 552 :3 0 c3
:3 0 c4 :3 0 554
555 0 c3 :3 0
d3 :3 0 557 558
0 18 :2 0 2
:4 0 ef :2 0 d5
:2 0 2a :2 0 266
55d 55f :3 0 269
559 561 556 563
562 :2 0 c6 :3 0
d6 :3 0 565 566
0 c8 :4 0 f0
:4 0 cb :2 0 cc
:3 0 26d 56a 56c
:3 0 cb :2 0 ea
:4 0 270 56e 570
:3 0 cb :2 0 f1
:4 0 273 572 574
:3 0 cb :2 0 84
:3 0 d0 :3 0 b0
:3 0 f2 :3 0 579
57a 0 276 578
57c d1 :4 0 278
577 57f 27b 576
581 :3 0 cb :2 0
f3 :4 0 27e 583
585 :3 0 cb :2 0
ce :4 0 281 587
589 :3 0 cb :2 0
21 :3 0 284 58b
58d :3 0 cb :2 0
ce :4 0 287 58f
591 :3 0 28a 567
593 :2 0 595 28d
596 564 595 0
597 28f 0 5c1
c3 :3 0 c4 :3 0
598 599 0 c3
:3 0 d3 :3 0 59b
59c 0 2a :2 0
2 :4 0 f4 :2 0
d5 :2 0 2a :2 0
291 5a1 5a3 :3 0
294 59d 5a5 59a
5a7 5a6 :2 0 c6
:3 0 d6 :3 0 5a9
5aa 0 c8 :4 0
f5 :4 0 cb :2 0
cc :3 0 298 5ae
5b0 :3 0 29b 5ab
5b2 :2 0 5b4 29e
5b5 5a8 5b4 0
5b6 2a0 0 5c1
f6 :3 0 f7 :3 0
5b7 5b8 0 b0
:3 0 f2 :3 0 5ba
5bb 0 21 :3 0
f8 :3 0 2a2 5b9
5bf :2 0 5c1 2a6
5c2 553 5c1 0
5c3 2aa 0 4e8b
f9 :3 0 c0 :3 0
91 :3 0 fa :3 0
fb :3 0 fc :3 0
fd :3 0 42 :3 0
2f :3 0 22 :3 0
7f :3 0 80 :3 0
7d :3 0 49 :3 0
ae :3 0 8c :3 0
20 :4 0 fe 1
:8 0 5db 15 :3 0
e1 :3 0 5d6 5d7
0 e4 :3 0 5d8
5d9 0 5db 2ac
62e e5 :3 0 15
:3 0 e1 :3 0 5dd
5de 0 e4 :3 0
5df 5e0 0 5e2
2af 5e4 2b1 5e3
5e2 :2 0 62c e6
:3 0 15 :3 0 e1
:3 0 5e7 5e8 0
e4 :3 0 5e9 5ea
0 624 c3 :3 0
c4 :3 0 5ec 5ed
0 c3 :3 0 d3
:3 0 5ef 5f0 0
e7 :2 0 2 :4 0
ff :2 0 d5 :2 0
2a :2 0 2b3 5f5
5f7 :3 0 2b6 5f1
5f9 5ee 5fb 5fa
:2 0 c6 :3 0 d6
:3 0 5fd 5fe 0
c8 :4 0 100 :4 0
cb :2 0 cc :3 0
2ba 602 604 :3 0
cb :2 0 ea :4 0
2bd 606 608 :3 0
cb :2 0 eb :4 0
2c0 60a 60c :3 0
cb :2 0 e4 :3 0
2c3 60e 610 :3 0
cb :2 0 ec :4 0
2c6 612 614 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 2c9 617
61b 2cd 616 61d
:3 0 2d0 5ff 61f
:2 0 621 2d3 622
5fc 621 0 623
2d5 0 624 2d7
627 :3 0 627 0
627 626 624 625
:6 0 629 7 :3 0
2da 62b 2dc 62a
629 :2 0 62c 2de
:2 0 62e 0 62e
62d 5db 62c :6 0
4e8b 1 :3 0 15
:3 0 e1 :3 0 630
631 0 15 :3 0
da :2 0 101 :3 0
633 635 0 2e3
634 637 :3 0 638
:2 0 c3 :3 0 c4
:3 0 63a 63b 0
c3 :3 0 d3 :3 0
63d 63e 0 18
:2 0 2 :4 0 102
:2 0 d5 :2 0 2a
:2 0 2e6 643 645
:3 0 2e9 63f 647
63c 649 648 :2 0
c6 :3 0 d6 :3 0
64b 64c 0 c8
:4 0 103 :4 0 cb
:2 0 cc :3 0 2ed
650 652 :3 0 cb
:2 0 ea :4 0 2f0
654 656 :3 0 cb
:2 0 f1 :4 0 2f3
658 65a :3 0 cb
:2 0 84 :3 0 d0
:3 0 b0 :3 0 f2
:3 0 65f 660 0
2f6 65e 662 d1
:4 0 2f8 65d 665
2fb 65c 667 :3 0
cb :2 0 f3 :4 0
2fe 669 66b :3 0
cb :2 0 ce :4 0
301 66d 66f :3 0
cb :2 0 20 :3 0
304 671 673 :3 0
cb :2 0 ce :4 0
307 675 677 :3 0
30a 64d 679 :2 0
67b 30d 67c 64a
67b 0 67d 30f
0 6a7 c3 :3 0
c4 :3 0 67e 67f
0 c3 :3 0 d3
:3 0 681 682 0
2a :2 0 2 :4 0
104 :2 0 d5 :2 0
2a :2 0 311 687
689 :3 0 314 683
68b 680 68d 68c
:2 0 c6 :3 0 d6
:3 0 68f 690 0
c8 :4 0 105 :4 0
cb :2 0 cc :3 0
318 694 696 :3 0
31b 691 698 :2 0
69a 31e 69b 68e
69a 0 69c 320
0 6a7 f6 :3 0
f7 :3 0 69d 69e
0 b0 :3 0 f2
:3 0 6a0 6a1 0
20 :3 0 f8 :3 0
322 69f 6a5 :2 0
6a7 326 6a8 639
6a7 0 6a9 32a
0 4e8b 6b :3 0
3d :2 0 6aa 6ab
0 4e8b 106 :3 0
6b :3 0 107 :3 0
91 :3 0 22 :4 0
108 1 :8 0 6b8
15 :3 0 e1 :3 0
6b3 6b4 0 e4
:3 0 6b5 6b6 0
6b8 32c 70b e5
:3 0 15 :3 0 e1
:3 0 6ba 6bb 0
e4 :3 0 6bc 6bd
0 6bf 32f 6c1
331 6c0 6bf :2 0
709 e6 :3 0 15
:3 0 e1 :3 0 6c4
6c5 0 e4 :3 0
6c6 6c7 0 701
c3 :3 0 c4 :3 0
6c9 6ca 0 c3
:3 0 d3 :3 0 6cc
6cd 0 e7 :2 0
2 :4 0 109 :2 0
d5 :2 0 2a :2 0
333 6d2 6d4 :3 0
336 6ce 6d6 6cb
6d8 6d7 :2 0 c6
:3 0 d6 :3 0 6da
6db 0 c8 :4 0
10a :4 0 cb :2 0
cc :3 0 33a 6df
6e1 :3 0 cb :2 0
ea :4 0 33d 6e3
6e5 :3 0 cb :2 0
eb :4 0 340 6e7
6e9 :3 0 cb :2 0
e4 :3 0 343 6eb
6ed :3 0 cb :2 0
ec :4 0 346 6ef
6f1 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
349 6f4 6f8 34d
6f3 6fa :3 0 350
6dc 6fc :2 0 6fe
353 6ff 6d9 6fe
0 700 355 0
701 357 704 :3 0
704 0 704 703
701 702 :6 0 706
9 :3 0 35a 708
35c 707 706 :2 0
709 35e :2 0 70b
0 70b 70a 6b8
709 :6 0 4e8b 1
:3 0 84 :3 0 6b
:3 0 3d :2 0 361
70d 710 10b :2 0
3d :2 0 366 712
714 :3 0 c3 :3 0
c4 :3 0 716 717
0 c3 :3 0 d3
:3 0 719 71a 0
18 :2 0 2 :4 0
10c :2 0 d5 :2 0
2a :2 0 369 71f
721 :3 0 36c 71b
723 718 725 724
:2 0 c6 :3 0 d6
:3 0 727 728 0
c8 :4 0 10d :4 0
cb :2 0 cc :3 0
370 72c 72e :3 0
cb :2 0 ea :4 0
373 730 732 :3 0
cb :2 0 f1 :4 0
376 734 736 :3 0
cb :2 0 84 :3 0
d0 :3 0 b0 :3 0
10e :3 0 73b 73c
0 379 73a 73e
d1 :4 0 37b 739
741 37e 738 743
:3 0 cb :2 0 f3
:4 0 381 745 747
:3 0 cb :2 0 ce
:4 0 384 749 74b
:3 0 cb :2 0 22
:3 0 387 74d 74f
:3 0 cb :2 0 ce
:4 0 38a 751 753
:3 0 38d 729 755
:2 0 757 390 758
726 757 0 759
392 0 783 c3
:3 0 c4 :3 0 75a
75b 0 c3 :3 0
d3 :3 0 75d 75e
0 2a :2 0 2
:4 0 10f :2 0 d5
:2 0 2a :2 0 394
763 765 :3 0 397
75f 767 75c 769
768 :2 0 c6 :3 0
d6 :3 0 76b 76c
0 c8 :4 0 110
:4 0 cb :2 0 cc
:3 0 39b 770 772
:3 0 39e 76d 774
:2 0 776 3a1 777
76a 776 0 778
3a3 0 783 f6
:3 0 f7 :3 0 779
77a 0 b0 :3 0
10e :3 0 77c 77d
0 22 :3 0 f8
:3 0 3a5 77b 781
:2 0 783 3a9 784
715 783 0 785
3ad 0 4e8b 111
:3 0 f9 :3 0 111
:3 0 112 :3 0 84
:3 0 111 :3 0 113
:3 0 84 :3 0 111
:3 0 114 :3 0 111
:3 0 115 :3 0 111
:3 0 116 :3 0 117
:3 0 118 :3 0 119
:3 0 11a :3 0 41
:3 0 65 :3 0 4f
:3 0 50 :3 0 53
:3 0 5f :3 0 5e
:3 0 ae :3 0 111
:3 0 11b :3 0 11c
:3 0 119 :3 0 111
:3 0 116 :3 0 119
:3 0 8c :3 0 111
:3 0 8c :3 0 22
:4 0 11d 1 :8 0
7b1 15 :3 0 e1
:3 0 7ac 7ad 0
e4 :3 0 7ae 7af
0 7b1 3af 804
e5 :3 0 15 :3 0
e1 :3 0 7b3 7b4
0 e4 :3 0 7b5
7b6 0 7b8 3b2
7ba 3b4 7b9 7b8
:2 0 802 e6 :3 0
15 :3 0 e1 :3 0
7bd 7be 0 e4
:3 0 7bf 7c0 0
7fa c3 :3 0 c4
:3 0 7c2 7c3 0
c3 :3 0 d3 :3 0
7c5 7c6 0 e7
:2 0 2 :4 0 11e
:2 0 d5 :2 0 2a
:2 0 3b6 7cb 7cd
:3 0 3b9 7c7 7cf
7c4 7d1 7d0 :2 0
c6 :3 0 d6 :3 0
7d3 7d4 0 c8
:4 0 11f :4 0 cb
:2 0 cc :3 0 3bd
7d8 7da :3 0 cb
:2 0 ea :4 0 3c0
7dc 7de :3 0 cb
:2 0 eb :4 0 3c3
7e0 7e2 :3 0 cb
:2 0 e4 :3 0 3c6
7e4 7e6 :3 0 cb
:2 0 ec :4 0 3c9
7e8 7ea :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 3cc 7ed 7f1
3d0 7ec 7f3 :3 0
3d3 7d5 7f5 :2 0
7f7 3d6 7f8 7d2
7f7 0 7f9 3d8
0 7fa 3da 7fd
:3 0 7fd 0 7fd
7fc 7fa 7fb :6 0
7ff b :3 0 3dd
801 3df 800 7ff
:2 0 802 3e1 :2 0
804 0 804 803
7b1 802 :6 0 4e8b
1 :3 0 15 :3 0
e1 :3 0 806 807
0 15 :3 0 da
:2 0 101 :3 0 809
80b 0 3e6 80a
80d :3 0 80e :2 0
c3 :3 0 c4 :3 0
810 811 0 c3
:3 0 d3 :3 0 813
814 0 18 :2 0
2 :4 0 120 :2 0
d5 :2 0 2a :2 0
3e9 819 81b :3 0
3ec 815 81d 812
81f 81e :2 0 c6
:3 0 d6 :3 0 821
822 0 c8 :4 0
121 :4 0 cb :2 0
cc :3 0 3f0 826
828 :3 0 cb :2 0
ea :4 0 3f3 82a
82c :3 0 cb :2 0
f1 :4 0 3f6 82e
830 :3 0 cb :2 0
84 :3 0 d0 :3 0
b0 :3 0 f2 :3 0
835 836 0 3f9
834 838 d1 :4 0
3fb 833 83b 3fe
832 83d :3 0 cb
:2 0 f3 :4 0 401
83f 841 :3 0 cb
:2 0 ce :4 0 404
843 845 :3 0 cb
:2 0 20 :3 0 407
847 849 :3 0 cb
:2 0 ce :4 0 40a
84b 84d :3 0 40d
823 84f :2 0 851
410 852 820 851
0 853 412 0
87d c3 :3 0 c4
:3 0 854 855 0
c3 :3 0 d3 :3 0
857 858 0 2a
:2 0 2 :4 0 122
:2 0 d5 :2 0 2a
:2 0 414 85d 85f
:3 0 417 859 861
856 863 862 :2 0
c6 :3 0 d6 :3 0
865 866 0 c8
:4 0 123 :4 0 cb
:2 0 cc :3 0 41b
86a 86c :3 0 41e
867 86e :2 0 870
421 871 864 870
0 872 423 0
87d f6 :3 0 f7
:3 0 873 874 0
b0 :3 0 f2 :3 0
876 877 0 20
:3 0 f8 :3 0 425
875 87b :2 0 87d
429 87e 80f 87d
0 87f 42d 0
4e8b 124 :3 0 125
:3 0 36 :3 0 3a
:3 0 126 :3 0 92
:3 0 21 :4 0 127
1 :8 0 88d 15
:3 0 e1 :3 0 888
889 0 e4 :3 0
88a 88b 0 88d
42f 8e0 e5 :3 0
15 :3 0 e1 :3 0
88f 890 0 e4
:3 0 891 892 0
894 432 896 434
895 894 :2 0 8de
e6 :3 0 15 :3 0
e1 :3 0 899 89a
0 e4 :3 0 89b
89c 0 8d6 c3
:3 0 c4 :3 0 89e
89f 0 c3 :3 0
d3 :3 0 8a1 8a2
0 e7 :2 0 2
:4 0 128 :2 0 d5
:2 0 2a :2 0 436
8a7 8a9 :3 0 439
8a3 8ab 8a0 8ad
8ac :2 0 c6 :3 0
d6 :3 0 8af 8b0
0 c8 :4 0 129
:4 0 cb :2 0 cc
:3 0 43d 8b4 8b6
:3 0 cb :2 0 ea
:4 0 440 8b8 8ba
:3 0 cb :2 0 eb
:4 0 443 8bc 8be
:3 0 cb :2 0 e4
:3 0 446 8c0 8c2
:3 0 cb :2 0 ec
:4 0 449 8c4 8c6
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 44c
8c9 8cd 450 8c8
8cf :3 0 453 8b1
8d1 :2 0 8d3 456
8d4 8ae 8d3 0
8d5 458 0 8d6
45a 8d9 :3 0 8d9
0 8d9 8d8 8d6
8d7 :6 0 8db d
:3 0 45d 8dd 45f
8dc 8db :2 0 8de
461 :2 0 8e0 0
8e0 8df 88d 8de
:6 0 4e8b 1 :3 0
15 :3 0 e1 :3 0
8e2 8e3 0 15
:3 0 da :2 0 101
:3 0 8e5 8e7 0
466 8e6 8e9 :3 0
8ea :2 0 c3 :3 0
c4 :3 0 8ec 8ed
0 c3 :3 0 d3
:3 0 8ef 8f0 0
18 :2 0 2 :4 0
12a :2 0 d5 :2 0
2a :2 0 469 8f5
8f7 :3 0 46c 8f1
8f9 8ee 8fb 8fa
:2 0 c6 :3 0 d6
:3 0 8fd 8fe 0
c8 :4 0 12b :4 0
cb :2 0 cc :3 0
470 902 904 :3 0
cb :2 0 ea :4 0
473 906 908 :3 0
cb :2 0 f1 :4 0
476 90a 90c :3 0
cb :2 0 84 :3 0
d0 :3 0 b0 :3 0
f2 :3 0 911 912
0 479 910 914
d1 :4 0 47b 90f
917 47e 90e 919
:3 0 cb :2 0 f3
:4 0 481 91b 91d
:3 0 cb :2 0 ce
:4 0 484 91f 921
:3 0 cb :2 0 21
:3 0 487 923 925
:3 0 cb :2 0 ce
:4 0 48a 927 929
:3 0 48d 8ff 92b
:2 0 92d 490 92e
8fc 92d 0 92f
492 0 959 c3
:3 0 c4 :3 0 930
931 0 c3 :3 0
d3 :3 0 933 934
0 2a :2 0 2
:4 0 12c :2 0 d5
:2 0 2a :2 0 494
939 93b :3 0 497
935 93d 932 93f
93e :2 0 c6 :3 0
d6 :3 0 941 942
0 c8 :4 0 12d
:4 0 cb :2 0 cc
:3 0 49b 946 948
:3 0 49e 943 94a
:2 0 94c 4a1 94d
940 94c 0 94e
4a3 0 959 f6
:3 0 f7 :3 0 94f
950 0 b0 :3 0
f2 :3 0 952 953
0 21 :3 0 f8
:3 0 4a5 951 957
:2 0 959 4a9 95a
8eb 959 0 95b
4ad 0 4e8b 12e
:3 0 35 :3 0 12f
:3 0 130 :3 0 23
:4 0 131 1 :8 0
967 15 :3 0 e1
:3 0 962 963 0
e4 :3 0 964 965
0 967 4af 9ba
e5 :3 0 15 :3 0
e1 :3 0 969 96a
0 e4 :3 0 96b
96c 0 96e 4b2
970 4b4 96f 96e
:2 0 9b8 e6 :3 0
15 :3 0 e1 :3 0
973 974 0 e4
:3 0 975 976 0
9b0 c3 :3 0 c4
:3 0 978 979 0
c3 :3 0 d3 :3 0
97b 97c 0 e7
:2 0 2 :4 0 132
:2 0 d5 :2 0 2a
:2 0 4b6 981 983
:3 0 4b9 97d 985
97a 987 986 :2 0
c6 :3 0 d6 :3 0
989 98a 0 c8
:4 0 133 :4 0 cb
:2 0 cc :3 0 4bd
98e 990 :3 0 cb
:2 0 ea :4 0 4c0
992 994 :3 0 cb
:2 0 eb :4 0 4c3
996 998 :3 0 cb
:2 0 e4 :3 0 4c6
99a 99c :3 0 cb
:2 0 ec :4 0 4c9
99e 9a0 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 4cc 9a3 9a7
4d0 9a2 9a9 :3 0
4d3 98b 9ab :2 0
9ad 4d6 9ae 988
9ad 0 9af 4d8
0 9b0 4da 9b3
:3 0 9b3 0 9b3
9b2 9b0 9b1 :6 0
9b5 f :3 0 4dd
9b7 4df 9b6 9b5
:2 0 9b8 4e1 :2 0
9ba 0 9ba 9b9
967 9b8 :6 0 4e8b
1 :3 0 117 :3 0
134 :3 0 9bc 9bd
0 35 :3 0 135
:4 0 4e4 9be 9c1
da :2 0 3d :2 0
4e9 9c3 9c5 :3 0
35 :3 0 de :4 0
9c7 9c8 0 9ca
4ec 9cb 9c6 9ca
0 9cc 4ee 0
4e8b 15 :3 0 e1
:3 0 9cd 9ce 0
15 :3 0 da :2 0
101 :3 0 9d0 9d2
0 4f2 9d1 9d4
:3 0 9d5 :2 0 c3
:3 0 c4 :3 0 9d7
9d8 0 c3 :3 0
d3 :3 0 9da 9db
0 18 :2 0 2
:4 0 136 :2 0 d5
:2 0 2a :2 0 4f5
9e0 9e2 :3 0 4f8
9dc 9e4 9d9 9e6
9e5 :2 0 c6 :3 0
d6 :3 0 9e8 9e9
0 c8 :4 0 137
:4 0 cb :2 0 cc
:3 0 4fc 9ed 9ef
:3 0 cb :2 0 ea
:4 0 4ff 9f1 9f3
:3 0 cb :2 0 f1
:4 0 502 9f5 9f7
:3 0 cb :2 0 84
:3 0 d0 :3 0 b0
:3 0 f2 :3 0 9fc
9fd 0 505 9fb
9ff d1 :4 0 507
9fa a02 50a 9f9
a04 :3 0 cb :2 0
f3 :4 0 50d a06
a08 :3 0 cb :2 0
ce :4 0 510 a0a
a0c :3 0 cb :2 0
23 :3 0 513 a0e
a10 :3 0 cb :2 0
ce :4 0 516 a12
a14 :3 0 519 9ea
a16 :2 0 a18 51c
a19 9e7 a18 0
a1a 51e 0 a44
c3 :3 0 c4 :3 0
a1b a1c 0 c3
:3 0 d3 :3 0 a1e
a1f 0 2a :2 0
2 :4 0 138 :2 0
d5 :2 0 2a :2 0
520 a24 a26 :3 0
523 a20 a28 a1d
a2a a29 :2 0 c6
:3 0 d6 :3 0 a2c
a2d 0 c8 :4 0
139 :4 0 cb :2 0
cc :3 0 527 a31
a33 :3 0 52a a2e
a35 :2 0 a37 52d
a38 a2b a37 0
a39 52f 0 a44
f6 :3 0 f7 :3 0
a3a a3b 0 b0
:3 0 f2 :3 0 a3d
a3e 0 23 :3 0
f8 :3 0 531 a3c
a42 :2 0 a44 535
a45 9d6 a44 0
a46 539 0 4e8b
60 :3 0 12 :3 0
a47 a48 0 4e8b
62 :3 0 d8 :3 0
84 :3 0 60 :3 0
3d :2 0 53b a4c
a4f 13a :2 0 53e
a4b a52 a4a a53
0 4e8b 63 :3 0
d8 :3 0 84 :3 0
60 :3 0 3d :2 0
541 a57 a5a 13b
:2 0 544 a56 a5d
a55 a5e 0 4e8b
61 :3 0 d8 :3 0
84 :3 0 60 :3 0
3d :2 0 547 a62
a65 2a :2 0 54a
a61 a68 a60 a69
0 4e8b 82 :3 0
13c :2 0 3d :2 0
54f a6c a6e :3 0
84 :3 0 63 :3 0
3d :2 0 552 a70
a73 da :2 0 13b
:2 0 557 a75 a77
:3 0 13d :3 0 9a
:3 0 6b :3 0 90
:3 0 91 :3 0 22
:3 0 9b :3 0 22
:3 0 92 :3 0 21
:3 0 93 :4 0 13e
1 :8 0 a8a 15
:3 0 e1 :3 0 a85
a86 0 e4 :3 0
a87 a88 0 a8a
55a add e5 :3 0
15 :3 0 e1 :3 0
a8c a8d 0 e4
:3 0 a8e a8f 0
a91 55d a93 55f
a92 a91 :2 0 adb
e6 :3 0 15 :3 0
e1 :3 0 a96 a97
0 e4 :3 0 a98
a99 0 ad3 c3
:3 0 c4 :3 0 a9b
a9c 0 c3 :3 0
d3 :3 0 a9e a9f
0 e7 :2 0 2
:4 0 13f :2 0 d5
:2 0 2a :2 0 561
aa4 aa6 :3 0 564
aa0 aa8 a9d aaa
aa9 :2 0 c6 :3 0
d6 :3 0 aac aad
0 c8 :4 0 140
:4 0 cb :2 0 cc
:3 0 568 ab1 ab3
:3 0 cb :2 0 ea
:4 0 56b ab5 ab7
:3 0 cb :2 0 eb
:4 0 56e ab9 abb
:3 0 cb :2 0 e4
:3 0 571 abd abf
:3 0 cb :2 0 ec
:4 0 574 ac1 ac3
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 577
ac6 aca 57b ac5
acc :3 0 57e aae
ace :2 0 ad0 581
ad1 aab ad0 0
ad2 583 0 ad3
585 ad6 :3 0 ad6
0 ad6 ad5 ad3
ad4 :6 0 ad8 11
:3 0 588 ada 58a
ad9 ad8 :2 0 adb
58c :2 0 add 0
add adc a8a adb
:6 0 ae0 1 :3 0
141 :3 0 58f b52
84 :3 0 63 :3 0
3d :2 0 591 ae1
ae4 da :2 0 3d
:2 0 596 ae6 ae8
:3 0 13d :3 0 9a
:3 0 6b :3 0 90
:3 0 91 :3 0 22
:3 0 9b :3 0 91
:3 0 92 :3 0 21
:3 0 93 :4 0 142
1 :8 0 afb 15
:3 0 e1 :3 0 af6
af7 0 e4 :3 0
af8 af9 0 afb
599 b4e e5 :3 0
15 :3 0 e1 :3 0
afd afe 0 e4
:3 0 aff b00 0
b02 59c b04 59e
b03 b02 :2 0 b4c
e6 :3 0 15 :3 0
e1 :3 0 b07 b08
0 e4 :3 0 b09
b0a 0 b44 c3
:3 0 c4 :3 0 b0c
b0d 0 c3 :3 0
d3 :3 0 b0f b10
0 e7 :2 0 2
:4 0 143 :2 0 d5
:2 0 2a :2 0 5a0
b15 b17 :3 0 5a3
b11 b19 b0e b1b
b1a :2 0 c6 :3 0
d6 :3 0 b1d b1e
0 c8 :4 0 144
:4 0 cb :2 0 cc
:3 0 5a7 b22 b24
:3 0 cb :2 0 ea
:4 0 5aa b26 b28
:3 0 cb :2 0 eb
:4 0 5ad b2a b2c
:3 0 cb :2 0 e4
:3 0 5b0 b2e b30
:3 0 cb :2 0 ec
:4 0 5b3 b32 b34
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 5b6
b37 b3b 5ba b36
b3d :3 0 5bd b1f
b3f :2 0 b41 5c0
b42 b1c b41 0
b43 5c2 0 b44
5c4 b47 :3 0 b47
0 b47 b46 b44
b45 :6 0 b49 13
:3 0 5c7 b4b 5c9
b4a b49 :2 0 b4c
5cb :2 0 b4e 0
b4e b4d afb b4c
:6 0 b50 1 :3 0
5ce b51 ae9 b50
0 b53 a78 ae0
0 b53 5d0 0
bcd 84 :3 0 6b
:3 0 3d :2 0 5d3
b54 b57 10b :2 0
3d :2 0 5d8 b59
b5b :3 0 c3 :3 0
c4 :3 0 b5d b5e
0 c3 :3 0 d3
:3 0 b60 b61 0
18 :2 0 2 :4 0
145 :2 0 d5 :2 0
2a :2 0 5db b66
b68 :3 0 5de b62
b6a b5f b6c b6b
:2 0 c6 :3 0 d6
:3 0 b6e b6f 0
c8 :4 0 146 :4 0
cb :2 0 cc :3 0
5e2 b73 b75 :3 0
cb :2 0 ea :4 0
5e5 b77 b79 :3 0
cb :2 0 f1 :4 0
5e8 b7b b7d :3 0
cb :2 0 84 :3 0
d0 :3 0 b0 :3 0
147 :3 0 b82 b83
0 5eb b81 b85
d1 :4 0 5ed b80
b88 5f0 b7f b8a
:3 0 cb :2 0 f3
:4 0 5f3 b8c b8e
:3 0 cb :2 0 ce
:4 0 5f6 b90 b92
:3 0 cb :2 0 21
:3 0 5f9 b94 b96
:3 0 cb :2 0 ce
:4 0 5fc b98 b9a
:3 0 5ff b70 b9c
:2 0 b9e 602 b9f
b6d b9e 0 ba0
604 0 bca c3
:3 0 c4 :3 0 ba1
ba2 0 c3 :3 0
d3 :3 0 ba4 ba5
0 2a :2 0 2
:4 0 148 :2 0 d5
:2 0 2a :2 0 606
baa bac :3 0 609
ba6 bae ba3 bb0
baf :2 0 c6 :3 0
d6 :3 0 bb2 bb3
0 c8 :4 0 149
:4 0 cb :2 0 cc
:3 0 60d bb7 bb9
:3 0 610 bb4 bbb
:2 0 bbd 613 bbe
bb1 bbd 0 bbf
615 0 bca f6
:3 0 f7 :3 0 bc0
bc1 0 b0 :3 0
147 :3 0 bc3 bc4
0 21 :3 0 f8
:3 0 617 bc2 bc8
:2 0 bca 61b bcb
b5c bca 0 bcc
61f 0 bcd 621
da2 13d :3 0 9a
:3 0 77 :3 0 90
:3 0 91 :3 0 22
:3 0 91 :3 0 9b
:3 0 92 :3 0 23
:3 0 93 :4 0 14a
1 :8 0 bdf 15
:3 0 e1 :3 0 bda
bdb 0 e4 :3 0
bdc bdd 0 bdf
624 c32 e5 :3 0
15 :3 0 e1 :3 0
be1 be2 0 e4
:3 0 be3 be4 0
be6 627 be8 629
be7 be6 :2 0 c30
e6 :3 0 15 :3 0
e1 :3 0 beb bec
0 e4 :3 0 bed
bee 0 c28 c3
:3 0 c4 :3 0 bf0
bf1 0 c3 :3 0
d3 :3 0 bf3 bf4
0 e7 :2 0 2
:4 0 14b :2 0 d5
:2 0 2a :2 0 62b
bf9 bfb :3 0 62e
bf5 bfd bf2 bff
bfe :2 0 c6 :3 0
d6 :3 0 c01 c02
0 c8 :4 0 14c
:4 0 cb :2 0 cc
:3 0 632 c06 c08
:3 0 cb :2 0 ea
:4 0 635 c0a c0c
:3 0 cb :2 0 eb
:4 0 638 c0e c10
:3 0 cb :2 0 e4
:3 0 63b c12 c14
:3 0 cb :2 0 ec
:4 0 63e c16 c18
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 641
c1b c1f 645 c1a
c21 :3 0 648 c03
c23 :2 0 c25 64b
c26 c00 c25 0
c27 64d 0 c28
64f c2b :3 0 c2b
0 c2b c2a c28
c29 :6 0 c2d 15
:3 0 652 c2f 654
c2e c2d :2 0 c30
656 :2 0 c32 0
c32 c31 bdf c30
:6 0 da1 1 :3 0
13d :3 0 9a :3 0
76 :3 0 90 :3 0
91 :3 0 22 :3 0
91 :3 0 9b :3 0
92 :3 0 23 :3 0
93 :4 0 14d 1
:8 0 c45 15 :3 0
e1 :3 0 c40 c41
0 e4 :3 0 c42
c43 0 c45 659
c98 e5 :3 0 15
:3 0 e1 :3 0 c47
c48 0 e4 :3 0
c49 c4a 0 c4c
65c c4e 65e c4d
c4c :2 0 c96 e6
:3 0 15 :3 0 e1
:3 0 c51 c52 0
e4 :3 0 c53 c54
0 c8e c3 :3 0
c4 :3 0 c56 c57
0 c3 :3 0 d3
:3 0 c59 c5a 0
e7 :2 0 2 :4 0
14e :2 0 d5 :2 0
2a :2 0 660 c5f
c61 :3 0 663 c5b
c63 c58 c65 c64
:2 0 c6 :3 0 d6
:3 0 c67 c68 0
c8 :4 0 14f :4 0
cb :2 0 cc :3 0
667 c6c c6e :3 0
cb :2 0 ea :4 0
66a c70 c72 :3 0
cb :2 0 eb :4 0
66d c74 c76 :3 0
cb :2 0 e4 :3 0
670 c78 c7a :3 0
cb :2 0 ec :4 0
673 c7c c7e :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 676 c81
c85 67a c80 c87
:3 0 67d c69 c89
:2 0 c8b 680 c8c
c66 c8b 0 c8d
682 0 c8e 684
c91 :3 0 c91 0
c91 c90 c8e c8f
:6 0 c93 17 :3 0
687 c95 689 c94
c93 :2 0 c96 68b
:2 0 c98 0 c98
c97 c45 c96 :6 0
da1 1 :3 0 150
:3 0 84 :3 0 42
:3 0 3d :2 0 68e
c9b c9e 691 c9a
ca0 ca1 :2 0 13c
:2 0 3d :2 0 695
ca3 ca5 :3 0 84
:3 0 63 :3 0 3d
:2 0 698 ca7 caa
da :2 0 3d :2 0
69d cac cae :3 0
caf :2 0 84 :3 0
77 :3 0 3d :2 0
6a0 cb1 cb4 da
:2 0 3d :2 0 6a5
cb6 cb8 :3 0 cb9
:2 0 cb0 cbb cba
:2 0 cbc :2 0 ca6
cbe cbd :2 0 13d
:3 0 9a :3 0 6b
:3 0 90 :3 0 91
:3 0 22 :3 0 92
:3 0 21 :3 0 93
:3 0 94 :4 0 151
1 :8 0 cd0 15
:3 0 e1 :3 0 ccb
ccc 0 e4 :3 0
ccd cce 0 cd0
6a8 d23 e5 :3 0
15 :3 0 e1 :3 0
cd2 cd3 0 e4
:3 0 cd4 cd5 0
cd7 6ab cd9 6ad
cd8 cd7 :2 0 d21
e6 :3 0 15 :3 0
e1 :3 0 cdc cdd
0 e4 :3 0 cde
cdf 0 d19 c3
:3 0 c4 :3 0 ce1
ce2 0 c3 :3 0
d3 :3 0 ce4 ce5
0 e7 :2 0 2
:4 0 152 :2 0 d5
:2 0 2a :2 0 6af
cea cec :3 0 6b2
ce6 cee ce3 cf0
cef :2 0 c6 :3 0
d6 :3 0 cf2 cf3
0 c8 :4 0 153
:4 0 cb :2 0 cc
:3 0 6b6 cf7 cf9
:3 0 cb :2 0 ea
:4 0 6b9 cfb cfd
:3 0 cb :2 0 eb
:4 0 6bc cff d01
:3 0 cb :2 0 e4
:3 0 6bf d03 d05
:3 0 cb :2 0 ec
:4 0 6c2 d07 d09
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 6c5
d0c d10 6c9 d0b
d12 :3 0 6cc cf4
d14 :2 0 d16 6cf
d17 cf1 d16 0
d18 6d1 0 d19
6d3 d1c :3 0 d1c
0 d1c d1b d19
d1a :6 0 d1e 19
:3 0 6d6 d20 6d8
d1f d1e :2 0 d21
6da :2 0 d23 0
d23 d22 cd0 d21
:6 0 d9e 1 :3 0
84 :3 0 6b :3 0
3d :2 0 6dd d25
d28 10b :2 0 3d
:2 0 6e2 d2a d2c
:3 0 c3 :3 0 c4
:3 0 d2e d2f 0
c3 :3 0 d3 :3 0
d31 d32 0 18
:2 0 2 :4 0 154
:2 0 d5 :2 0 2a
:2 0 6e5 d37 d39
:3 0 6e8 d33 d3b
d30 d3d d3c :2 0
c6 :3 0 d6 :3 0
d3f d40 0 c8
:4 0 155 :4 0 cb
:2 0 cc :3 0 6ec
d44 d46 :3 0 cb
:2 0 ea :4 0 6ef
d48 d4a :3 0 cb
:2 0 f1 :4 0 6f2
d4c d4e :3 0 cb
:2 0 84 :3 0 d0
:3 0 b0 :3 0 147
:3 0 d53 d54 0
6f5 d52 d56 d1
:4 0 6f7 d51 d59
6fa d50 d5b :3 0
cb :2 0 f3 :4 0
6fd d5d d5f :3 0
cb :2 0 ce :4 0
700 d61 d63 :3 0
cb :2 0 21 :3 0
703 d65 d67 :3 0
cb :2 0 ce :4 0
706 d69 d6b :3 0
709 d41 d6d :2 0
d6f 70c d70 d3e
d6f 0 d71 70e
0 d9b c3 :3 0
c4 :3 0 d72 d73
0 c3 :3 0 d3
:3 0 d75 d76 0
2a :2 0 2 :4 0
156 :2 0 d5 :2 0
2a :2 0 710 d7b
d7d :3 0 713 d77
d7f d74 d81 d80
:2 0 c6 :3 0 d6
:3 0 d83 d84 0
c8 :4 0 157 :4 0
cb :2 0 cc :3 0
717 d88 d8a :3 0
71a d85 d8c :2 0
d8e 71d d8f d82
d8e 0 d90 71f
0 d9b f6 :3 0
f7 :3 0 d91 d92
0 b0 :3 0 147
:3 0 d94 d95 0
21 :3 0 f8 :3 0
721 d93 d99 :2 0
d9b 725 d9c d2d
d9b 0 d9d 729
0 d9e 72b d9f
cbf d9e 0 da0
72e 0 da1 730
da3 a6f bcd 0
da4 0 da1 0
da4 734 0 4e8b
2e :3 0 3d :4 0
da5 da6 0 4e8b
1b :3 0 de :4 0
da8 da9 0 4e8b
1c :3 0 de :4 0
dab dac 0 4e8b
1e :4 0 dae daf
0 4e8b 24 :4 0
db1 db2 0 4e8b
df :3 0 8e :4 0
db7 :2 0 4e8b db5
db8 :3 0 8e :3 0
56 :4 0 dbc :2 0
4e8b db9 dba :3 0
8e :3 0 e0 :3 0
dbd dbe :3 0 15
:3 0 e1 :3 0 dc0
dc1 0 3d :2 0
dc2 dc3 0 dc5
737 dcc 15 :3 0
e1 :3 0 dc6 dc7
0 e2 :2 0 dc8
dc9 0 dcb 739
dcd dbf dc5 0
dce 0 dcb 0
dce 73b 0 4e8b
15 :3 0 e1 :3 0
dcf dd0 0 15
:3 0 da :2 0 158
:3 0 dd2 dd4 0
740 dd3 dd6 :3 0
dd7 :2 0 4c :3 0
3d :2 0 dd9 dda
0 ddc 743 de1
4c :3 0 13b :2 0
ddd dde 0 de0
745 de2 dd8 ddc
0 de3 0 de0
0 de3 747 0
4e8b e3 :3 0 8e
:4 0 de7 :2 0 ded
de5 0 15 :3 0
e1 :3 0 de8 de9
0 e4 :3 0 dea
deb 0 ded 74a
e40 e5 :3 0 15
:3 0 e1 :3 0 def
df0 0 e4 :3 0
df1 df2 0 df4
74d df6 74f df5
df4 :2 0 e3e e6
:3 0 15 :3 0 e1
:3 0 df9 dfa 0
e4 :3 0 dfb dfc
0 e36 c3 :3 0
c4 :3 0 dfe dff
0 c3 :3 0 d3
:3 0 e01 e02 0
e7 :2 0 2 :4 0
159 :2 0 d5 :2 0
2a :2 0 751 e07
e09 :3 0 754 e03
e0b e00 e0d e0c
:2 0 c6 :3 0 d6
:3 0 e0f e10 0
c8 :4 0 15a :4 0
cb :2 0 cc :3 0
758 e14 e16 :3 0
cb :2 0 ea :4 0
75b e18 e1a :3 0
cb :2 0 eb :4 0
75e e1c e1e :3 0
cb :2 0 e4 :3 0
761 e20 e22 :3 0
cb :2 0 ec :4 0
764 e24 e26 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 767 e29
e2d 76b e28 e2f
:3 0 76e e11 e31
:2 0 e33 771 e34
e0e e33 0 e35
773 0 e36 775
e39 :3 0 e39 0
e39 e38 e36 e37
:6 0 e3b 1b :3 0
778 e3d 77a e3c
e3b :2 0 e3e 77c
:2 0 e40 0 e40
e3f ded e3e :6 0
4e8b 1 :3 0 84
:3 0 4c :3 0 3d
:2 0 77f e42 e45
da :2 0 13b :2 0
784 e47 e49 :3 0
13d :3 0 9a :3 0
56 :3 0 90 :3 0
91 :3 0 22 :3 0
92 :3 0 21 :3 0
93 :3 0 9b :3 0
22 :4 0 15b 1
:8 0 e5c 15 :3 0
e1 :3 0 e57 e58
0 e4 :3 0 e59
e5a 0 e5c 787
eaf e5 :3 0 15
:3 0 e1 :3 0 e5e
e5f 0 e4 :3 0
e60 e61 0 e63
78a e65 78c e64
e63 :2 0 ead e6
:3 0 15 :3 0 e1
:3 0 e68 e69 0
e4 :3 0 e6a e6b
0 ea5 c3 :3 0
c4 :3 0 e6d e6e
0 c3 :3 0 d3
:3 0 e70 e71 0
e7 :2 0 2 :4 0
15c :2 0 d5 :2 0
2a :2 0 78e e76
e78 :3 0 791 e72
e7a e6f e7c e7b
:2 0 c6 :3 0 d6
:3 0 e7e e7f 0
c8 :4 0 15d :4 0
cb :2 0 cc :3 0
795 e83 e85 :3 0
cb :2 0 ea :4 0
798 e87 e89 :3 0
cb :2 0 eb :4 0
79b e8b e8d :3 0
cb :2 0 e4 :3 0
79e e8f e91 :3 0
cb :2 0 ec :4 0
7a1 e93 e95 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 7a4 e98
e9c 7a8 e97 e9e
:3 0 7ab e80 ea0
:2 0 ea2 7ae ea3
e7d ea2 0 ea4
7b0 0 ea5 7b2
ea8 :3 0 ea8 0
ea8 ea7 ea5 ea6
:6 0 eaa 1d :3 0
7b5 eac 7b7 eab
eaa :2 0 ead 7b9
:2 0 eaf 0 eaf
eae e5c ead :6 0
15ed 1 :3 0 84
:3 0 56 :3 0 3d
:2 0 7bc eb1 eb4
10b :2 0 3d :2 0
7c1 eb6 eb8 :3 0
4c :3 0 2a :2 0
eba ebb 0 103c
a1 :3 0 a2 :3 0
33 :3 0 34 :3 0
90 :3 0 91 :3 0
22 :3 0 15e :3 0
92 :3 0 21 :3 0
93 :3 0 9b :3 0
22 :4 0 15f 1
:8 0 ed0 15 :3 0
e1 :3 0 ecb ecc
0 e4 :3 0 ecd
ece 0 ed0 7c4
f23 e5 :3 0 15
:3 0 e1 :3 0 ed2
ed3 0 e4 :3 0
ed4 ed5 0 ed7
7c7 ed9 7c9 ed8
ed7 :2 0 f21 e6
:3 0 15 :3 0 e1
:3 0 edc edd 0
e4 :3 0 ede edf
0 f19 c3 :3 0
c4 :3 0 ee1 ee2
0 c3 :3 0 d3
:3 0 ee4 ee5 0
e7 :2 0 2 :4 0
160 :2 0 d5 :2 0
2a :2 0 7cb eea
eec :3 0 7ce ee6
eee ee3 ef0 eef
:2 0 c6 :3 0 d6
:3 0 ef2 ef3 0
c8 :4 0 161 :4 0
cb :2 0 cc :3 0
7d2 ef7 ef9 :3 0
cb :2 0 ea :4 0
7d5 efb efd :3 0
cb :2 0 eb :4 0
7d8 eff f01 :3 0
cb :2 0 e4 :3 0
7db f03 f05 :3 0
cb :2 0 ec :4 0
7de f07 f09 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 7e1 f0c
f10 7e5 f0b f12
:3 0 7e8 ef4 f14
:2 0 f16 7eb f17
ef1 f16 0 f18
7ed 0 f19 7ef
f1c :3 0 f1c 0
f1c f1b f19 f1a
:6 0 f1e 1f :3 0
7f2 f20 7f4 f1f
f1e :2 0 f21 7f6
:2 0 f23 0 f23
f22 ed0 f21 :6 0
103c 1 :3 0 15
:3 0 e1 :3 0 f25
f26 0 15 :3 0
da :2 0 101 :3 0
f28 f2a 0 7fb
f29 f2c :3 0 f2d
:2 0 c3 :3 0 c4
:3 0 f2f f30 0
c3 :3 0 d3 :3 0
f32 f33 0 18
:2 0 2 :4 0 162
:2 0 d5 :2 0 2a
:2 0 7fe f38 f3a
:3 0 801 f34 f3c
f31 f3e f3d :2 0
c6 :3 0 d6 :3 0
f40 f41 0 c8
:4 0 163 :4 0 cb
:2 0 cc :3 0 805
f45 f47 :3 0 cb
:2 0 ea :4 0 808
f49 f4b :3 0 cb
:2 0 f1 :4 0 80b
f4d f4f :3 0 cb
:2 0 84 :3 0 d0
:3 0 b0 :3 0 147
:3 0 f54 f55 0
80e f53 f57 d1
:4 0 810 f52 f5a
813 f51 f5c :3 0
cb :2 0 f3 :4 0
816 f5e f60 :3 0
cb :2 0 ce :4 0
819 f62 f64 :3 0
cb :2 0 21 :3 0
81c f66 f68 :3 0
cb :2 0 ce :4 0
81f f6a f6c :3 0
822 f42 f6e :2 0
f70 825 f71 f3f
f70 0 f72 827
0 f9d c3 :3 0
c4 :3 0 f73 f74
0 c3 :3 0 d3
:3 0 f76 f77 0
2a :2 0 2 :4 0
164 :2 0 d5 :2 0
2a :2 0 829 f7c
f7e :3 0 82c f78
f80 f75 f82 f81
:2 0 c6 :3 0 d6
:3 0 f84 f85 0
c8 :4 0 165 :4 0
cb :2 0 cc :3 0
830 f89 f8b :3 0
833 f86 f8d :2 0
f8f 836 f90 f83
f8f 0 f91 838
0 f9d f6 :3 0
f7 :3 0 f92 f93
0 b0 :3 0 147
:3 0 f95 f96 0
21 :3 0 f8 :3 0
83a f94 f9a :2 0
f9d 141 :3 0 83e
103a 33 :3 0 166
:2 0 842 f9f fa0
:3 0 fa1 :2 0 fa2
:2 0 34 :3 0 166
:2 0 844 fa5 fa6
:3 0 fa7 :2 0 fa8
:2 0 fa3 faa fa9
:2 0 117 :3 0 167
:3 0 fac fad 0
33 :3 0 24 :3 0
168 :2 0 846 fae
fb2 10b :2 0 3d
:2 0 84c fb4 fb6
:3 0 fb7 :2 0 117
:3 0 167 :3 0 fb9
fba 0 34 :3 0
24 :3 0 168 :2 0
84f fbb fbf 169
:2 0 3d :2 0 855
fc1 fc3 :3 0 fc4
:2 0 fb8 fc6 fc5
:2 0 c3 :3 0 c4
:3 0 fc8 fc9 0
c3 :3 0 d3 :3 0
fcb fcc 0 18
:2 0 2 :4 0 16a
:2 0 d5 :2 0 2a
:2 0 858 fd1 fd3
:3 0 85b fcd fd5
fca fd7 fd6 :2 0
c6 :3 0 d6 :3 0
fd9 fda 0 c8
:4 0 16b :4 0 cb
:2 0 cc :3 0 85f
fde fe0 :3 0 cb
:2 0 ea :4 0 862
fe2 fe4 :3 0 cb
:2 0 f1 :4 0 865
fe6 fe8 :3 0 cb
:2 0 84 :3 0 d0
:3 0 b0 :3 0 147
:3 0 fed fee 0
868 fec ff0 d1
:4 0 86a feb ff3
86d fea ff5 :3 0
cb :2 0 f3 :4 0
870 ff7 ff9 :3 0
cb :2 0 ce :4 0
873 ffb ffd :3 0
cb :2 0 21 :3 0
876 fff 1001 :3 0
cb :2 0 ce :4 0
879 1003 1005 :3 0
87c fdb 1007 :2 0
1009 87f 100a fd8
1009 0 100b 881
0 1035 c3 :3 0
c4 :3 0 100c 100d
0 c3 :3 0 d3
:3 0 100f 1010 0
2a :2 0 2 :4 0
16c :2 0 d5 :2 0
2a :2 0 883 1015
1017 :3 0 886 1011
1019 100e 101b 101a
:2 0 c6 :3 0 d6
:3 0 101d 101e 0
c8 :4 0 16d :4 0
cb :2 0 cc :3 0
88a 1022 1024 :3 0
88d 101f 1026 :2 0
1028 890 1029 101c
1028 0 102a 892
0 1035 f6 :3 0
f7 :3 0 102b 102c
0 b0 :3 0 147
:3 0 102e 102f 0
21 :3 0 f8 :3 0
894 102d 1033 :2 0
1035 898 1036 fc7
1035 0 1037 89c
0 1038 89e 1039
fab 1038 0 103b
f2e f9d 0 103b
8a0 0 103c 8a3
103d eb9 103c 0
103e 8a7 0 15ed
84 :3 0 63 :3 0
3d :2 0 8a9 103f
1042 da :2 0 13b
:2 0 8ae 1044 1046
:3 0 4c :3 0 3d
:2 0 1048 1049 0
104e 1e :3 0 1e
:3 0 104b 104c 0
104e 8b1 15ea 98
:3 0 16e :3 0 1c
:3 0 55 :3 0 af
:3 0 8c :3 0 22
:3 0 93 :3 0 2e
:3 0 92 :3 0 21
:4 0 16f 1 :8 0
1060 15 :3 0 e1
:3 0 105b 105c 0
e4 :3 0 105d 105e
0 1060 8b4 10b3
e5 :3 0 15 :3 0
e1 :3 0 1062 1063
0 e4 :3 0 1064
1065 0 1067 8b7
1069 8b9 1068 1067
:2 0 10b1 e6 :3 0
15 :3 0 e1 :3 0
106c 106d 0 e4
:3 0 106e 106f 0
10a9 c3 :3 0 c4
:3 0 1071 1072 0
c3 :3 0 d3 :3 0
1074 1075 0 e7
:2 0 2 :4 0 170
:2 0 d5 :2 0 2a
:2 0 8bb 107a 107c
:3 0 8be 1076 107e
1073 1080 107f :2 0
c6 :3 0 d6 :3 0
1082 1083 0 c8
:4 0 171 :4 0 cb
:2 0 cc :3 0 8c2
1087 1089 :3 0 cb
:2 0 ea :4 0 8c5
108b 108d :3 0 cb
:2 0 eb :4 0 8c8
108f 1091 :3 0 cb
:2 0 e4 :3 0 8cb
1093 1095 :3 0 cb
:2 0 ec :4 0 8ce
1097 1099 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 8d1 109c 10a0
8d5 109b 10a2 :3 0
8d8 1084 10a4 :2 0
10a6 8db 10a7 1081
10a6 0 10a8 8dd
0 10a9 8df 10ac
:3 0 10ac 0 10ac
10ab 10a9 10aa :6 0
10ae 21 :3 0 8e2
10b0 8e4 10af 10ae
:2 0 10b1 8e6 :2 0
10b3 0 10b3 10b2
1060 10b1 :6 0 15e9
1 :3 0 15 :3 0
e1 :3 0 10b5 10b6
0 15 :3 0 da
:2 0 101 :3 0 10b8
10ba 0 8eb 10b9
10bc :3 0 10bd :2 0
1c :3 0 de :4 0
10bf 10c0 0 13d2
1c :3 0 172 :4 0
10c3 10c4 :3 0 10c2
10c5 0 13d2 71
:3 0 173 :3 0 3d
:2 0 84 :3 0 41
:3 0 3d :2 0 8ee
10ca 10cd b0 :3 0
174 :3 0 10cf 10d0
0 6e :3 0 8f1
10c8 10d3 10c7 10d4
0 13d2 6d :3 0
b0 :3 0 175 :3 0
10d7 10d8 0 10d6
10d9 0 13d2 6c
:3 0 3d :4 0 10db
10dc 0 13d2 70
:3 0 2a :2 0 10de
10df 0 13d2 71
:3 0 176 :3 0 22
:3 0 21 :3 0 41
:3 0 6d :3 0 6e
:3 0 6c :3 0 70
:3 0 8f6 10e2 10ea
10e1 10eb 0 13d2
7b :3 0 177 :3 0
84 :3 0 41 :3 0
3d :2 0 8fe 10ef
10f2 901 10ee 10f4
10ed 10f5 0 13d2
af :3 0 98 :3 0
91 :3 0 8c :3 0
9c :3 0 93 :3 0
92 :3 0 130 :3 0
16e :3 0 178 :3 0
179 :3 0 17a :3 0
17b :3 0 1c :3 0
22 :3 0 22 :3 0
41 :3 0 2e :3 0
21 :3 0 23 :3 0
b0 :3 0 175 :3 0
6e :3 0 7b :3 0
13 :3 0 cc :4 0
17c 1 :8 0 1127
e0 :4 0 1112 :3 0
f8 :3 0 da :2 0
905 1115 1116 :3 0
15 :3 0 e1 :3 0
1118 1119 0 3d
:2 0 111a 111b 0
111d 908 1124 15
:3 0 e1 :3 0 111e
111f 0 e2 :2 0
1120 1121 0 1123
90a 1125 1117 111d
0 1126 0 1123
0 1126 90c 0
1127 90f 117a e5
:3 0 15 :3 0 e1
:3 0 1129 112a 0
e4 :3 0 112b 112c
0 112e 912 1130
914 112f 112e :2 0
1178 e6 :3 0 15
:3 0 e1 :3 0 1133
1134 0 e4 :3 0
1135 1136 0 1170
c3 :3 0 c4 :3 0
1138 1139 0 c3
:3 0 d3 :3 0 113b
113c 0 e7 :2 0
2 :4 0 17d :2 0
d5 :2 0 2a :2 0
916 1141 1143 :3 0
919 113d 1145 113a
1147 1146 :2 0 c6
:3 0 d6 :3 0 1149
114a 0 c8 :4 0
17e :4 0 cb :2 0
cc :3 0 91d 114e
1150 :3 0 cb :2 0
ea :4 0 920 1152
1154 :3 0 cb :2 0
eb :4 0 923 1156
1158 :3 0 cb :2 0
e4 :3 0 926 115a
115c :3 0 cb :2 0
ec :4 0 929 115e
1160 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
92c 1163 1167 930
1162 1169 :3 0 933
114b 116b :2 0 116d
936 116e 1148 116d
0 116f 938 0
1170 93a 1173 :3 0
1173 0 1173 1172
1170 1171 :6 0 1175
23 :3 0 93d 1177
93f 1176 1175 :2 0
1178 941 :2 0 117a
0 117a 1179 1127
1178 :6 0 13d2 1
:3 0 17f :3 0 39
:3 0 12f :3 0 130
:3 0 23 :4 0 180
1 :8 0 1187 15
:3 0 e1 :3 0 1182
1183 0 e4 :3 0
1184 1185 0 1187
944 11da e5 :3 0
15 :3 0 e1 :3 0
1189 118a 0 e4
:3 0 118b 118c 0
118e 947 1190 949
118f 118e :2 0 11d8
e6 :3 0 15 :3 0
e1 :3 0 1193 1194
0 e4 :3 0 1195
1196 0 11d0 c3
:3 0 c4 :3 0 1198
1199 0 c3 :3 0
d3 :3 0 119b 119c
0 e7 :2 0 2
:4 0 181 :2 0 d5
:2 0 2a :2 0 94b
11a1 11a3 :3 0 94e
119d 11a5 119a 11a7
11a6 :2 0 c6 :3 0
d6 :3 0 11a9 11aa
0 c8 :4 0 182
:4 0 cb :2 0 cc
:3 0 952 11ae 11b0
:3 0 cb :2 0 ea
:4 0 955 11b2 11b4
:3 0 cb :2 0 eb
:4 0 958 11b6 11b8
:3 0 cb :2 0 e4
:3 0 95b 11ba 11bc
:3 0 cb :2 0 ec
:4 0 95e 11be 11c0
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 961
11c3 11c7 965 11c2
11c9 :3 0 968 11ab
11cb :2 0 11cd 96b
11ce 11a8 11cd 0
11cf 96d 0 11d0
96f 11d3 :3 0 11d3
0 11d3 11d2 11d0
11d1 :6 0 11d5 25
:3 0 972 11d7 974
11d6 11d5 :2 0 11d8
976 :2 0 11da 0
11da 11d9 1187 11d8
:6 0 13d2 1 :3 0
117 :3 0 134 :3 0
11dc 11dd 0 22
:3 0 20 :3 0 979
11de 11e1 da :2 0
3d :2 0 97e 11e3
11e5 :3 0 43 :3 0
3d :2 0 11e7 11e8
0 11ea 981 1253
13d :3 0 9a :3 0
43 :3 0 90 :3 0
91 :3 0 22 :3 0
9b :3 0 20 :3 0
93 :3 0 92 :3 0
23 :3 0 15e :4 0
183 1 :8 0 11fd
15 :3 0 e1 :3 0
11f8 11f9 0 e4
:3 0 11fa 11fb 0
11fd 983 1250 e5
:3 0 15 :3 0 e1
:3 0 11ff 1200 0
e4 :3 0 1201 1202
0 1204 986 1206
988 1205 1204 :2 0
124e e6 :3 0 15
:3 0 e1 :3 0 1209
120a 0 e4 :3 0
120b 120c 0 1246
c3 :3 0 c4 :3 0
120e 120f 0 c3
:3 0 d3 :3 0 1211
1212 0 e7 :2 0
2 :4 0 184 :2 0
d5 :2 0 2a :2 0
98a 1217 1219 :3 0
98d 1213 121b 1210
121d 121c :2 0 c6
:3 0 d6 :3 0 121f
1220 0 c8 :4 0
185 :4 0 cb :2 0
cc :3 0 991 1224
1226 :3 0 cb :2 0
ea :4 0 994 1228
122a :3 0 cb :2 0
eb :4 0 997 122c
122e :3 0 cb :2 0
e4 :3 0 99a 1230
1232 :3 0 cb :2 0
ec :4 0 99d 1234
1236 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
9a0 1239 123d 9a4
1238 123f :3 0 9a7
1221 1241 :2 0 1243
9aa 1244 121e 1243
0 1245 9ac 0
1246 9ae 1249 :3 0
1249 0 1249 1248
1246 1247 :6 0 124b
27 :3 0 9b1 124d
9b3 124c 124b :2 0
124e 9b5 :2 0 1250
0 1250 124f 11fd
124e :6 0 1252 1
:3 0 9b8 1254 11e6
11ea 0 1255 0
1252 0 1255 9ba
0 13d2 84 :3 0
43 :3 0 3d :2 0
9bd 1256 1259 da
:2 0 3d :2 0 9c2
125b 125d :3 0 ae
:3 0 186 :3 0 187
:3 0 186 :3 0 39
:3 0 36 :3 0 8c
:3 0 22 :3 0 91
:3 0 22 :4 0 188
1 :8 0 127f e0
:4 0 126a :3 0 f8
:3 0 da :2 0 9c7
126d 126e :3 0 15
:3 0 e1 :3 0 1270
1271 0 3d :2 0
1272 1273 0 1275
9ca 127c 15 :3 0
e1 :3 0 1276 1277
0 e2 :2 0 1278
1279 0 127b 9cc
127d 126f 1275 0
127e 0 127b 0
127e 9ce 0 127f
9d1 12d2 e5 :3 0
15 :3 0 e1 :3 0
1281 1282 0 e4
:3 0 1283 1284 0
1286 9d4 1288 9d6
1287 1286 :2 0 12d0
e6 :3 0 15 :3 0
e1 :3 0 128b 128c
0 e4 :3 0 128d
128e 0 12c8 c3
:3 0 c4 :3 0 1290
1291 0 c3 :3 0
d3 :3 0 1293 1294
0 e7 :2 0 2
:4 0 189 :2 0 d5
:2 0 2a :2 0 9d8
1299 129b :3 0 9db
1295 129d 1292 129f
129e :2 0 c6 :3 0
d6 :3 0 12a1 12a2
0 c8 :4 0 18a
:4 0 cb :2 0 cc
:3 0 9df 12a6 12a8
:3 0 cb :2 0 ea
:4 0 9e2 12aa 12ac
:3 0 cb :2 0 eb
:4 0 9e5 12ae 12b0
:3 0 cb :2 0 e4
:3 0 9e8 12b2 12b4
:3 0 cb :2 0 ec
:4 0 9eb 12b6 12b8
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 9ee
12bb 12bf 9f2 12ba
12c1 :3 0 9f5 12a3
12c3 :2 0 12c5 9f8
12c6 12a0 12c5 0
12c7 9fa 0 12c8
9fc 12cb :3 0 12cb
0 12cb 12ca 12c8
12c9 :6 0 12cd 29
:3 0 9ff 12cf a01
12ce 12cd :2 0 12d0
a03 :2 0 12d2 0
12d2 12d1 127f 12d0
:6 0 1350 1 :3 0
ae :3 0 186 :3 0
18b :3 0 186 :3 0
36 :3 0 186 :3 0
18c :3 0 36 :3 0
18c :3 0 186 :3 0
186 :3 0 186 :3 0
36 :3 0 91 :3 0
22 :3 0 8c :3 0
22 :4 0 18d 1
:8 0 12fb e0 :4 0
12e6 :3 0 f8 :3 0
da :2 0 a08 12e9
12ea :3 0 15 :3 0
e1 :3 0 12ec 12ed
0 3d :2 0 12ee
12ef 0 12f1 a0b
12f8 15 :3 0 e1
:3 0 12f2 12f3 0
e2 :2 0 12f4 12f5
0 12f7 a0d 12f9
12eb 12f1 0 12fa
0 12f7 0 12fa
a0f 0 12fb a12
134e e5 :3 0 15
:3 0 e1 :3 0 12fd
12fe 0 e4 :3 0
12ff 1300 0 1302
a15 1304 a17 1303
1302 :2 0 134c e6
:3 0 15 :3 0 e1
:3 0 1307 1308 0
e4 :3 0 1309 130a
0 1344 c3 :3 0
c4 :3 0 130c 130d
0 c3 :3 0 d3
:3 0 130f 1310 0
e7 :2 0 2 :4 0
18e :2 0 d5 :2 0
2a :2 0 a19 1315
1317 :3 0 a1c 1311
1319 130e 131b 131a
:2 0 c6 :3 0 d6
:3 0 131d 131e 0
c8 :4 0 18f :4 0
cb :2 0 cc :3 0
a20 1322 1324 :3 0
cb :2 0 ea :4 0
a23 1326 1328 :3 0
cb :2 0 eb :4 0
a26 132a 132c :3 0
cb :2 0 e4 :3 0
a29 132e 1330 :3 0
cb :2 0 ec :4 0
a2c 1332 1334 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 a2f 1337
133b a33 1336 133d
:3 0 a36 131f 133f
:2 0 1341 a39 1342
131c 1341 0 1343
a3b 0 1344 a3d
1347 :3 0 1347 0
1347 1346 1344 1345
:6 0 1349 2b :3 0
a40 134b a42 134a
1349 :2 0 134c a44
:2 0 134e 0 134e
134d 12fb 134c :6 0
1350 1 :3 0 a47
13ce ae :3 0 186
:3 0 18b :3 0 186
:3 0 36 :3 0 186
:3 0 18c :3 0 36
:3 0 18c :3 0 186
:3 0 186 :3 0 186
:3 0 36 :3 0 91
:3 0 22 :3 0 8c
:3 0 22 :4 0 18d
1 :8 0 1378 e0
:4 0 1363 :3 0 f8
:3 0 da :2 0 a4c
1366 1367 :3 0 15
:3 0 e1 :3 0 1369
136a 0 3d :2 0
136b 136c 0 136e
a4f 1375 15 :3 0
e1 :3 0 136f 1370
0 e2 :2 0 1371
1372 0 1374 a51
1376 1368 136e 0
1377 0 1374 0
1377 a53 0 1378
a56 13cb e5 :3 0
15 :3 0 e1 :3 0
137a 137b 0 e4
:3 0 137c 137d 0
137f a59 1381 a5b
1380 137f :2 0 13c9
e6 :3 0 15 :3 0
e1 :3 0 1384 1385
0 e4 :3 0 1386
1387 0 13c1 c3
:3 0 c4 :3 0 1389
138a 0 c3 :3 0
d3 :3 0 138c 138d
0 e7 :2 0 2
:4 0 190 :2 0 d5
:2 0 2a :2 0 a5d
1392 1394 :3 0 a60
138e 1396 138b 1398
1397 :2 0 c6 :3 0
d6 :3 0 139a 139b
0 c8 :4 0 191
:4 0 cb :2 0 cc
:3 0 a64 139f 13a1
:3 0 cb :2 0 ea
:4 0 a67 13a3 13a5
:3 0 cb :2 0 eb
:4 0 a6a 13a7 13a9
:3 0 cb :2 0 e4
:3 0 a6d 13ab 13ad
:3 0 cb :2 0 ec
:4 0 a70 13af 13b1
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 a73
13b4 13b8 a77 13b3
13ba :3 0 a7a 139c
13bc :2 0 13be a7d
13bf 1399 13be 0
13c0 a7f 0 13c1
a81 13c4 :3 0 13c4
0 13c4 13c3 13c1
13c2 :6 0 13c6 2d
:3 0 a84 13c8 a86
13c7 13c6 :2 0 13c9
a88 :2 0 13cb 0
13cb 13ca 1378 13c9
:6 0 13cd 1 :3 0
a8b 13cf 125e 1350
0 13d0 0 13cd
0 13d0 a8d 0
13d2 141 :3 0 a90
15e7 15 :3 0 e1
:3 0 13d3 13d4 0
15 :3 0 da :2 0
158 :3 0 13d6 13d8
0 a9f 13d7 13da
:3 0 13db :2 0 9a
:3 0 1e :3 0 90
:3 0 98 :3 0 1c
:3 0 15e :4 0 192
1 :8 0 13e9 15
:3 0 e1 :3 0 13e4
13e5 0 e4 :3 0
13e6 13e7 0 13e9
aa2 143c e5 :3 0
15 :3 0 e1 :3 0
13eb 13ec 0 e4
:3 0 13ed 13ee 0
13f0 aa5 13f2 aa7
13f1 13f0 :2 0 143a
e6 :3 0 15 :3 0
e1 :3 0 13f5 13f6
0 e4 :3 0 13f7
13f8 0 1432 c3
:3 0 c4 :3 0 13fa
13fb 0 c3 :3 0
d3 :3 0 13fd 13fe
0 e7 :2 0 2
:4 0 193 :2 0 d5
:2 0 2a :2 0 aa9
1403 1405 :3 0 aac
13ff 1407 13fc 1409
1408 :2 0 c6 :3 0
d6 :3 0 140b 140c
0 c8 :4 0 194
:4 0 cb :2 0 cc
:3 0 ab0 1410 1412
:3 0 cb :2 0 ea
:4 0 ab3 1414 1416
:3 0 cb :2 0 eb
:4 0 ab6 1418 141a
:3 0 cb :2 0 e4
:3 0 ab9 141c 141e
:3 0 cb :2 0 ec
:4 0 abc 1420 1422
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 abf
1425 1429 ac3 1424
142b :3 0 ac6 140d
142d :2 0 142f ac9
1430 140a 142f 0
1431 acb 0 1432
acd 1435 :3 0 1435
0 1435 1434 1432
1433 :6 0 1437 2f
:3 0 ad0 1439 ad2
1438 1437 :2 0 143a
ad4 :2 0 143c 0
143c 143b 13e9 143a
:6 0 15e5 1 :3 0
84 :3 0 55 :3 0
3d :2 0 ad7 143e
1441 b0 :3 0 da
:2 0 195 :3 0 1443
1445 0 adc 1444
1447 :3 0 71 :3 0
173 :3 0 3d :2 0
84 :3 0 41 :3 0
3d :2 0 adf 144c
144f b0 :3 0 174
:3 0 1451 1452 0
6e :3 0 ae2 144a
1455 1449 1456 0
14d4 af :3 0 130
:3 0 23 :3 0 16e
:3 0 b0 :3 0 175
:3 0 178 :3 0 6e
:3 0 98 :3 0 1c
:4 0 196 1 :8 0
1478 e0 :4 0 1463
:3 0 f8 :3 0 da
:2 0 ae9 1466 1467
:3 0 15 :3 0 e1
:3 0 1469 146a 0
3d :2 0 146b 146c
0 146e aec 1475
15 :3 0 e1 :3 0
146f 1470 0 e2
:2 0 1471 1472 0
1474 aee 1476 1468
146e 0 1477 0
1474 0 1477 af0
0 1478 af3 14cb
e5 :3 0 15 :3 0
e1 :3 0 147a 147b
0 e4 :3 0 147c
147d 0 147f af6
1481 af8 1480 147f
:2 0 14c9 e6 :3 0
15 :3 0 e1 :3 0
1484 1485 0 e4
:3 0 1486 1487 0
14c1 c3 :3 0 c4
:3 0 1489 148a 0
c3 :3 0 d3 :3 0
148c 148d 0 e7
:2 0 2 :4 0 197
:2 0 d5 :2 0 2a
:2 0 afa 1492 1494
:3 0 afd 148e 1496
148b 1498 1497 :2 0
c6 :3 0 d6 :3 0
149a 149b 0 c8
:4 0 198 :4 0 cb
:2 0 cc :3 0 b01
149f 14a1 :3 0 cb
:2 0 ea :4 0 b04
14a3 14a5 :3 0 cb
:2 0 eb :4 0 b07
14a7 14a9 :3 0 cb
:2 0 e4 :3 0 b0a
14ab 14ad :3 0 cb
:2 0 ec :4 0 b0d
14af 14b1 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 b10 14b4 14b8
b14 14b3 14ba :3 0
b17 149c 14bc :2 0
14be b1a 14bf 1499
14be 0 14c0 b1c
0 14c1 b1e 14c4
:3 0 14c4 0 14c4
14c3 14c1 14c2 :6 0
14c6 31 :3 0 b21
14c8 b23 14c7 14c6
:2 0 14c9 b25 :2 0
14cb 0 14cb 14ca
1478 14c9 :6 0 14d4
1 :3 0 71 :3 0
199 :3 0 1c :3 0
b28 14ce 14d0 14cd
14d1 0 14d4 141
:3 0 b2a 15e2 d8
:3 0 84 :3 0 55
:3 0 3d :2 0 b2e
14d6 14d9 b0 :3 0
175 :3 0 14db 14dc
0 b31 14d5 14de
14df :2 0 b0 :3 0
13c :2 0 175 :3 0
14e1 14e3 0 b36
14e2 14e5 :3 0 af
:3 0 130 :3 0 23
:3 0 16e :3 0 19a
:3 0 19b :3 0 16e
:3 0 b0 :3 0 175
:3 0 98 :3 0 1c
:4 0 19c 1 :8 0
1508 e0 :4 0 14f3
:3 0 f8 :3 0 da
:2 0 b3b 14f6 14f7
:3 0 15 :3 0 e1
:3 0 14f9 14fa 0
3d :2 0 14fb 14fc
0 14fe b3e 1505
15 :3 0 e1 :3 0
14ff 1500 0 e2
:2 0 1501 1502 0
1504 b40 1506 14f8
14fe 0 1507 0
1504 0 1507 b42
0 1508 b45 155b
e5 :3 0 15 :3 0
e1 :3 0 150a 150b
0 e4 :3 0 150c
150d 0 150f b48
1511 b4a 1510 150f
:2 0 1559 e6 :3 0
15 :3 0 e1 :3 0
1514 1515 0 e4
:3 0 1516 1517 0
1551 c3 :3 0 c4
:3 0 1519 151a 0
c3 :3 0 d3 :3 0
151c 151d 0 e7
:2 0 2 :4 0 19d
:2 0 d5 :2 0 2a
:2 0 b4c 1522 1524
:3 0 b4f 151e 1526
151b 1528 1527 :2 0
c6 :3 0 d6 :3 0
152a 152b 0 c8
:4 0 19e :4 0 cb
:2 0 cc :3 0 b53
152f 1531 :3 0 cb
:2 0 ea :4 0 b56
1533 1535 :3 0 cb
:2 0 eb :4 0 b59
1537 1539 :3 0 cb
:2 0 e4 :3 0 b5c
153b 153d :3 0 cb
:2 0 ec :4 0 b5f
153f 1541 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 b62 1544 1548
b66 1543 154a :3 0
b69 152c 154c :2 0
154e b6c 154f 1529
154e 0 1550 b6e
0 1551 b70 1554
:3 0 1554 0 1554
1553 1551 1552 :6 0
1556 33 :3 0 b73
1558 b75 1557 1556
:2 0 1559 b77 :2 0
155b 0 155b 155a
1508 1559 :6 0 1563
1 :3 0 71 :3 0
199 :3 0 1c :3 0
b7a 155e 1560 155d
1561 0 1563 b7c
1564 14e6 1563 0
15e4 af :3 0 130
:3 0 23 :3 0 16e
:3 0 19a :3 0 19b
:3 0 16e :3 0 b0
:3 0 19f :3 0 98
:3 0 1c :4 0 1a0
1 :8 0 1586 e0
:4 0 1571 :3 0 f8
:3 0 da :2 0 b81
1574 1575 :3 0 15
:3 0 e1 :3 0 1577
1578 0 3d :2 0
1579 157a 0 157c
b84 1583 15 :3 0
e1 :3 0 157d 157e
0 e2 :2 0 157f
1580 0 1582 b86
1584 1576 157c 0
1585 0 1582 0
1585 b88 0 1586
b8b 15d9 e5 :3 0
15 :3 0 e1 :3 0
1588 1589 0 e4
:3 0 158a 158b 0
158d b8e 158f b90
158e 158d :2 0 15d7
e6 :3 0 15 :3 0
e1 :3 0 1592 1593
0 e4 :3 0 1594
1595 0 15cf c3
:3 0 c4 :3 0 1597
1598 0 c3 :3 0
d3 :3 0 159a 159b
0 e7 :2 0 2
:4 0 1a1 :2 0 d5
:2 0 2a :2 0 b92
15a0 15a2 :3 0 b95
159c 15a4 1599 15a6
15a5 :2 0 c6 :3 0
d6 :3 0 15a8 15a9
0 c8 :4 0 1a2
:4 0 cb :2 0 cc
:3 0 b99 15ad 15af
:3 0 cb :2 0 ea
:4 0 b9c 15b1 15b3
:3 0 cb :2 0 eb
:4 0 b9f 15b5 15b7
:3 0 cb :2 0 e4
:3 0 ba2 15b9 15bb
:3 0 cb :2 0 ec
:4 0 ba5 15bd 15bf
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 ba8
15c2 15c6 bac 15c1
15c8 :3 0 baf 15aa
15ca :2 0 15cc bb2
15cd 15a7 15cc 0
15ce bb4 0 15cf
bb6 15d2 :3 0 15d2
0 15d2 15d1 15cf
15d0 :6 0 15d4 35
:3 0 bb9 15d6 bbb
15d5 15d4 :2 0 15d7
bbd :2 0 15d9 0
15d9 15d8 1586 15d7
:6 0 15e1 1 :3 0
71 :3 0 199 :3 0
1c :3 0 bc0 15dc
15de 15db 15df 0
15e1 bc2 15e3 1448
14d4 0 15e4 0
15e1 0 15e4 bc5
0 15e5 bc9 15e6
13dc 15e5 0 15e8
10be 13d2 0 15e8
bcc 0 15e9 bcf
15eb 1047 104e 0
15ec 0 15e9 0
15ec bd2 0 15ed
bd5 15f2 1e :3 0
1e :3 0 15ee 15ef
0 15f1 bd9 15f3
e4a 15ed 0 15f4
0 15f1 0 15f4
bdb 0 4e8b 67
:3 0 de :4 0 15f5
15f6 0 4e8b 69
:3 0 de :4 0 15f8
15f9 0 4e8b 26
:3 0 de :4 0 15fb
15fc 0 4e8b 71
:3 0 1a3 :4 0 15ff
1600 :3 0 15fe 1601
0 4e8b 5f :3 0
1a4 :3 0 116 :3 0
5e :3 0 117 :3 0
118 :3 0 119 :3 0
11a :3 0 5e :3 0
126 :3 0 1a4 :3 0
11b :3 0 11c :3 0
119 :3 0 1a4 :3 0
116 :3 0 119 :3 0
8c :3 0 1a4 :3 0
116 :3 0 92 :3 0
21 :4 0 1a5 1
:8 0 161f 15 :3 0
e1 :3 0 161a 161b
0 e4 :3 0 161c
161d 0 161f bde
1672 e5 :3 0 15
:3 0 e1 :3 0 1621
1622 0 e4 :3 0
1623 1624 0 1626
be1 1628 be3 1627
1626 :2 0 1670 e6
:3 0 15 :3 0 e1
:3 0 162b 162c 0
e4 :3 0 162d 162e
0 1668 c3 :3 0
c4 :3 0 1630 1631
0 c3 :3 0 d3
:3 0 1633 1634 0
e7 :2 0 2 :4 0
1a6 :2 0 d5 :2 0
2a :2 0 be5 1639
163b :3 0 be8 1635
163d 1632 163f 163e
:2 0 c6 :3 0 d6
:3 0 1641 1642 0
c8 :4 0 1a7 :4 0
cb :2 0 cc :3 0
bec 1646 1648 :3 0
cb :2 0 ea :4 0
bef 164a 164c :3 0
cb :2 0 eb :4 0
bf2 164e 1650 :3 0
cb :2 0 e4 :3 0
bf5 1652 1654 :3 0
cb :2 0 ec :4 0
bf8 1656 1658 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 bfb 165b
165f bff 165a 1661
:3 0 c02 1643 1663
:2 0 1665 c05 1666
1640 1665 0 1667
c07 0 1668 c09
166b :3 0 166b 0
166b 166a 1668 1669
:6 0 166d 37 :3 0
c0c 166f c0e 166e
166d :2 0 1670 c10
:2 0 1672 0 1672
1671 161f 1670 :6 0
4e8b 1 :3 0 15
:3 0 e1 :3 0 1674
1675 0 15 :3 0
13c :2 0 158 :3 0
1677 1679 0 c15
1678 167b :3 0 167c
:2 0 5e :3 0 1a8
:2 0 167e 167f 0
1681 c18 1682 167d
1681 0 1683 c1a
0 4e8b 84 :3 0
62 :3 0 3d :2 0
c1c 1684 1687 da
:2 0 3d :2 0 c21
1689 168b :3 0 168c
:2 0 84 :3 0 61
:3 0 3d :2 0 c24
168e 1691 da :2 0
3d :2 0 c29 1693
1695 :3 0 1696 :2 0
168d 1698 1697 :2 0
71 :3 0 1a9 :3 0
22 :3 0 21 :3 0
2e :3 0 67 :3 0
69 :3 0 85 :3 0
58 :3 0 5c :3 0
5b :3 0 5d :3 0
c2c 169b 16a6 169a
16a7 0 16fd 84
:3 0 62 :3 0 3d
:2 0 c37 16a9 16ac
da :2 0 3d :2 0
c3c 16ae 16b0 :3 0
58 :3 0 84 :3 0
58 :3 0 3d :2 0
c3f 16b3 16b6 1aa
:2 0 84 :3 0 5e
:3 0 3d :2 0 c42
16b9 16bc c45 16b8
16be :3 0 16b2 16bf
0 16d0 5c :3 0
84 :3 0 5c :3 0
3d :2 0 c48 16c2
16c5 1aa :2 0 84
:3 0 5e :3 0 3d
:2 0 c4b 16c8 16cb
c4e 16c7 16cd :3 0
16c1 16ce 0 16d0
c51 16d1 16b1 16d0
0 16d2 c54 0
16fd 84 :3 0 61
:3 0 3d :2 0 c56
16d3 16d6 da :2 0
3d :2 0 c5b 16d8
16da :3 0 5b :3 0
84 :3 0 5b :3 0
3d :2 0 c5e 16dd
16e0 1aa :2 0 84
:3 0 5e :3 0 3d
:2 0 c61 16e3 16e6
c64 16e2 16e8 :3 0
16dc 16e9 0 16fa
5d :3 0 84 :3 0
5d :3 0 3d :2 0
c67 16ec 16ef 1aa
:2 0 84 :3 0 5e
:3 0 3d :2 0 c6a
16f2 16f5 c6d 16f1
16f7 :3 0 16eb 16f8
0 16fa c70 16fb
16db 16fa 0 16fc
c73 0 16fd c75
16fe 1699 16fd 0
16ff c79 0 4e8b
4b :3 0 3d :2 0
1700 1701 0 4e8b
43 :3 0 3d :2 0
1703 1704 0 4e8b
81 :4 0 1706 1707
0 4e8b 1ab :3 0
1ac :3 0 78 :3 0
6f :3 0 107 :3 0
91 :3 0 22 :4 0
1ad 1 :8 0 1716
15 :3 0 e1 :3 0
1711 1712 0 e4
:3 0 1713 1714 0
1716 c7b 1769 e5
:3 0 15 :3 0 e1
:3 0 1718 1719 0
e4 :3 0 171a 171b
0 171d c7e 171f
c80 171e 171d :2 0
1767 e6 :3 0 15
:3 0 e1 :3 0 1722
1723 0 e4 :3 0
1724 1725 0 175f
c3 :3 0 c4 :3 0
1727 1728 0 c3
:3 0 d3 :3 0 172a
172b 0 e7 :2 0
2 :4 0 1ae :2 0
d5 :2 0 2a :2 0
c82 1730 1732 :3 0
c85 172c 1734 1729
1736 1735 :2 0 c6
:3 0 d6 :3 0 1738
1739 0 c8 :4 0
1af :4 0 cb :2 0
cc :3 0 c89 173d
173f :3 0 cb :2 0
ea :4 0 c8c 1741
1743 :3 0 cb :2 0
eb :4 0 c8f 1745
1747 :3 0 cb :2 0
e4 :3 0 c92 1749
174b :3 0 cb :2 0
ec :4 0 c95 174d
174f :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
c98 1752 1756 c9c
1751 1758 :3 0 c9f
173a 175a :2 0 175c
ca2 175d 1737 175c
0 175e ca4 0
175f ca6 1762 :3 0
1762 0 1762 1761
175f 1760 :6 0 1764
39 :3 0 ca9 1766
cab 1765 1764 :2 0
1767 cad :2 0 1769
0 1769 1768 1716
1767 :6 0 4e8b 1
:3 0 117 :3 0 134
:3 0 176b 176c 0
22 :3 0 20 :3 0
cb0 176d 1770 13c
:2 0 3d :2 0 cb5
1772 1774 :3 0 1b0
:3 0 1b1 :3 0 43
:3 0 1b2 :3 0 91
:3 0 22 :3 0 9b
:3 0 22 :3 0 92
:3 0 23 :4 0 1b3
1 :8 0 1786 15
:3 0 e1 :3 0 1781
1782 0 e4 :3 0
1783 1784 0 1786
cb8 17d9 e5 :3 0
15 :3 0 e1 :3 0
1788 1789 0 e4
:3 0 178a 178b 0
178d cbb 178f cbd
178e 178d :2 0 17d7
e6 :3 0 15 :3 0
e1 :3 0 1792 1793
0 e4 :3 0 1794
1795 0 17cf c3
:3 0 c4 :3 0 1797
1798 0 c3 :3 0
d3 :3 0 179a 179b
0 e7 :2 0 2
:4 0 1b4 :2 0 d5
:2 0 2a :2 0 cbf
17a0 17a2 :3 0 cc2
179c 17a4 1799 17a6
17a5 :2 0 c6 :3 0
d6 :3 0 17a8 17a9
0 c8 :4 0 1b5
:4 0 cb :2 0 cc
:3 0 cc6 17ad 17af
:3 0 cb :2 0 ea
:4 0 cc9 17b1 17b3
:3 0 cb :2 0 eb
:4 0 ccc 17b5 17b7
:3 0 cb :2 0 e4
:3 0 ccf 17b9 17bb
:3 0 cb :2 0 ec
:4 0 cd2 17bd 17bf
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 cd5
17c2 17c6 cd9 17c1
17c8 :3 0 cdc 17aa
17ca :2 0 17cc cdf
17cd 17a7 17cc 0
17ce ce1 0 17cf
ce3 17d2 :3 0 17d2
0 17d2 17d1 17cf
17d0 :6 0 17d4 3b
:3 0 ce6 17d6 ce8
17d5 17d4 :2 0 17d7
cea :2 0 17d9 0
17d9 17d8 1786 17d7
:6 0 17ef 1 :3 0
84 :3 0 43 :3 0
3d :2 0 ced 17db
17de da :2 0 3d
:2 0 cf2 17e0 17e2
:3 0 20 :3 0 22
:3 0 17e4 17e5 0
17e7 cf5 17ec 43
:3 0 3d :2 0 17e8
17e9 0 17eb cf7
17ed 17e3 17e7 0
17ee 0 17eb 0
17ee cf9 0 17ef
cfc 17f0 1775 17ef
0 17f1 cff 0
4e8b 84 :3 0 4c
:3 0 3d :2 0 d01
17f2 17f5 10b :2 0
3d :2 0 d06 17f7
17f9 :3 0 117 :3 0
134 :3 0 17fb 17fc
0 22 :3 0 20
:3 0 d09 17fd 1800
da :2 0 3d :2 0
d0e 1802 1804 :3 0
9a :3 0 98 :3 0
9d :3 0 9f :3 0
a0 :3 0 a1 :3 0
9f :3 0 a0 :3 0
a2 :3 0 a5 :3 0
a6 :3 0 a7 :3 0
a8 :3 0 9b :3 0
9e :3 0 a9 :3 0
aa :3 0 ab :3 0
ac :3 0 1e :3 0
17 :3 0 7c :3 0
2b :3 0 2d :3 0
78 :3 0 79 :3 0
74 :3 0 75 :3 0
20 :3 0 7e :3 0
44 :3 0 46 :3 0
47 :3 0 48 :3 0
90 :3 0 9b :3 0
22 :3 0 92 :3 0
23 :3 0 93 :4 0
1b6 1 :8 0 1834
15 :3 0 e1 :3 0
182f 1830 0 e4
:3 0 1831 1832 0
1834 d11 1887 e5
:3 0 15 :3 0 e1
:3 0 1836 1837 0
e4 :3 0 1838 1839
0 183b d14 183d
d16 183c 183b :2 0
1885 e6 :3 0 15
:3 0 e1 :3 0 1840
1841 0 e4 :3 0
1842 1843 0 187d
c3 :3 0 c4 :3 0
1845 1846 0 c3
:3 0 d3 :3 0 1848
1849 0 e7 :2 0
2 :4 0 1b7 :2 0
d5 :2 0 2a :2 0
d18 184e 1850 :3 0
d1b 184a 1852 1847
1854 1853 :2 0 c6
:3 0 d6 :3 0 1856
1857 0 c8 :4 0
1b8 :4 0 cb :2 0
cc :3 0 d1f 185b
185d :3 0 cb :2 0
ea :4 0 d22 185f
1861 :3 0 cb :2 0
eb :4 0 d25 1863
1865 :3 0 cb :2 0
e4 :3 0 d28 1867
1869 :3 0 cb :2 0
ec :4 0 d2b 186b
186d :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
d2e 1870 1874 d32
186f 1876 :3 0 d35
1858 1878 :2 0 187a
d38 187b 1855 187a
0 187c d3a 0
187d d3c 1880 :3 0
1880 0 1880 187f
187d 187e :6 0 1882
3d :3 0 d3f 1884
d41 1883 1882 :2 0
1885 d43 :2 0 1887
0 1887 1886 1834
1885 :6 0 2370 1
:3 0 81 :3 0 1e
:3 0 1889 188a 0
2370 57 :3 0 3d
:2 0 188c 188d 0
2370 84 :3 0 76
:3 0 3d :2 0 d46
188f 1892 da :2 0
2a :2 0 d4b 1894
1896 :3 0 84 :3 0
48 :3 0 3d :2 0
d4e 1898 189b 84
:3 0 13c :2 0 f
:3 0 3d :2 0 d51
189d 18a1 d56 189e
18a3 :3 0 18a4 :2 0
84 :3 0 44 :3 0
3d :2 0 d59 18a6
18a9 84 :3 0 13c
:2 0 b :3 0 3d
:2 0 d5c 18ab 18af
d61 18ac 18b1 :3 0
18b2 :2 0 18a5 18b4
18b3 :2 0 84 :3 0
4c :3 0 3d :2 0
d64 18b6 18b9 da
:2 0 2a :2 0 d69
18bb 18bd :3 0 18be
:2 0 d8 :3 0 84
:3 0 c :3 0 3d
:2 0 d6c 18c1 18c4
b0 :3 0 1b9 :3 0
18c6 18c7 0 d6f
18c0 18c9 18ca :2 0
13c :2 0 3d :2 0
d74 18cc 18ce :3 0
18cf :2 0 18bf 18d1
18d0 :2 0 c3 :3 0
c4 :3 0 18d3 18d4
0 c3 :3 0 d3
:3 0 18d6 18d7 0
18 :2 0 2 :4 0
1ba :2 0 d5 :2 0
2a :2 0 d77 18dc
18de :3 0 d7a 18d8
18e0 18d5 18e2 18e1
:2 0 c6 :3 0 d6
:3 0 18e4 18e5 0
c8 :4 0 1bb :4 0
cb :2 0 cc :3 0
d7e 18e9 18eb :3 0
cb :2 0 ea :4 0
d81 18ed 18ef :3 0
cb :2 0 f1 :4 0
d84 18f1 18f3 :3 0
cb :2 0 84 :3 0
d0 :3 0 b0 :3 0
1bc :3 0 18f8 18f9
0 d87 18f7 18fb
d1 :4 0 d89 18f6
18fe d8c 18f5 1900
:3 0 cb :2 0 f3
:4 0 d8f 1902 1904
:3 0 cb :2 0 ce
:4 0 d92 1906 1908
:3 0 cb :2 0 21
:3 0 d95 190a 190c
:3 0 cb :2 0 ce
:4 0 d98 190e 1910
:3 0 d9b 18e6 1912
:2 0 1914 d9e 1915
18e3 1914 0 1916
da0 0 1940 c3
:3 0 c4 :3 0 1917
1918 0 c3 :3 0
d3 :3 0 191a 191b
0 2a :2 0 2
:4 0 1bd :2 0 d5
:2 0 2a :2 0 da2
1920 1922 :3 0 da5
191c 1924 1919 1926
1925 :2 0 c6 :3 0
d6 :3 0 1928 1929
0 c8 :4 0 1be
:4 0 cb :2 0 cc
:3 0 da9 192d 192f
:3 0 dac 192a 1931
:2 0 1933 daf 1934
1927 1933 0 1935
db1 0 1940 f6
:3 0 f7 :3 0 1936
1937 0 b0 :3 0
1bc :3 0 1939 193a
0 21 :3 0 f8
:3 0 db3 1938 193e
:2 0 1940 db7 1954
57 :3 0 2a :2 0
1941 1942 0 1953
84 :3 0 f :3 0
3d :2 0 dbb 1944
1947 da :2 0 3d
:2 0 dc0 1949 194b
:3 0 7e :3 0 3d
:2 0 194d 194e 0
1950 dc3 1951 194c
1950 0 1952 dc5
0 1953 dc7 1955
18d2 1940 0 1956
0 1953 0 1956
dca 0 1957 dcd
1958 18b5 1957 0
1959 dcf 0 1975
3f :3 0 9 :3 0
195a 195b 0 1975
24 :3 0 7 :3 0
195d 195e 0 1975
26 :3 0 8 :3 0
1960 1961 0 1975
44 :3 0 b :3 0
1963 1964 0 1975
45 :3 0 10 :3 0
1966 1967 0 1975
46 :3 0 d :3 0
1969 196a 0 1975
47 :3 0 e :3 0
196c 196d 0 1975
4e :3 0 c :3 0
196f 1970 0 1975
48 :3 0 f :3 0
1972 1973 0 1975
dd1 1976 1897 1975
0 1977 ddc 0
2370 15 :3 0 e1
:3 0 1978 1979 0
15 :3 0 da :2 0
158 :3 0 197b 197d
0 de0 197c 197f
:3 0 1980 :2 0 1e
:3 0 166 :2 0 de3
1983 1984 :3 0 1985
:2 0 1986 :2 0 1981
1988 1987 :2 0 a0
:3 0 6f :3 0 2a
:2 0 2a :2 0 de5
198a 198e da :2 0
1bf :4 0 deb 1990
1992 :3 0 7a :3 0
1c0 :2 0 2a :2 0
dee 1995 1997 :3 0
1994 1998 0 19ab
15 :3 0 e1 :3 0
199a 199b 0 1c1
:3 0 22 :3 0 22
:3 0 1e :3 0 2b
:3 0 2d :3 0 74
:3 0 75 :3 0 7a
:3 0 79 :3 0 78
:3 0 df0 199d 19a8
199c 19a9 0 19ab
dfb 19ac 1993 19ab
0 19ad dfe 0
236c 84 :3 0 4c
:3 0 3d :2 0 e00
19ae 19b1 da :2 0
2a :2 0 e05 19b3
19b5 :3 0 90 :3 0
9a :3 0 1e :4 0
1c2 1 :8 0 19d0
e0 :4 0 19bb :3 0
f8 :3 0 da :2 0
e0a 19be 19bf :3 0
15 :3 0 e1 :3 0
19c1 19c2 0 3d
:2 0 19c3 19c4 0
19c6 e0d 19cd 15
:3 0 e1 :3 0 19c7
19c8 0 e2 :2 0
19c9 19ca 0 19cc
e0f 19ce 19c0 19c6
0 19cf 0 19cc
0 19cf e11 0
19d0 e14 1a23 e5
:3 0 15 :3 0 e1
:3 0 19d2 19d3 0
e4 :3 0 19d4 19d5
0 19d7 e17 19d9
e19 19d8 19d7 :2 0
1a21 e6 :3 0 15
:3 0 e1 :3 0 19dc
19dd 0 e4 :3 0
19de 19df 0 1a19
c3 :3 0 c4 :3 0
19e1 19e2 0 c3
:3 0 d3 :3 0 19e4
19e5 0 e7 :2 0
2 :4 0 1c3 :2 0
d5 :2 0 2a :2 0
e1b 19ea 19ec :3 0
e1e 19e6 19ee 19e3
19f0 19ef :2 0 c6
:3 0 d6 :3 0 19f2
19f3 0 c8 :4 0
1c4 :4 0 cb :2 0
cc :3 0 e22 19f7
19f9 :3 0 cb :2 0
ea :4 0 e25 19fb
19fd :3 0 cb :2 0
eb :4 0 e28 19ff
1a01 :3 0 cb :2 0
e4 :3 0 e2b 1a03
1a05 :3 0 cb :2 0
ec :4 0 e2e 1a07
1a09 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
e31 1a0c 1a10 e35
1a0b 1a12 :3 0 e38
19f4 1a14 :2 0 1a16
e3b 1a17 19f1 1a16
0 1a18 e3d 0
1a19 e3f 1a1c :3 0
1a1c 0 1a1c 1a1b
1a19 1a1a :6 0 1a1e
3f :3 0 e42 1a20
e44 1a1f 1a1e :2 0
1a21 e46 :2 0 1a23
0 1a23 1a22 19d0
1a21 :6 0 1b6f 1
:3 0 1b2 :3 0 91
:3 0 22 :3 0 9a
:3 0 1e :4 0 1c5
1 :8 0 1a40 e0
:4 0 1a2b :3 0 f8
:3 0 da :2 0 e4b
1a2e 1a2f :3 0 15
:3 0 e1 :3 0 1a31
1a32 0 3d :2 0
1a33 1a34 0 1a36
e4e 1a3d 15 :3 0
e1 :3 0 1a37 1a38
0 e2 :2 0 1a39
1a3a 0 1a3c e50
1a3e 1a30 1a36 0
1a3f 0 1a3c 0
1a3f e52 0 1a40
e55 1a93 e5 :3 0
15 :3 0 e1 :3 0
1a42 1a43 0 e4
:3 0 1a44 1a45 0
1a47 e58 1a49 e5a
1a48 1a47 :2 0 1a91
e6 :3 0 15 :3 0
e1 :3 0 1a4c 1a4d
0 e4 :3 0 1a4e
1a4f 0 1a89 c3
:3 0 c4 :3 0 1a51
1a52 0 c3 :3 0
d3 :3 0 1a54 1a55
0 e7 :2 0 2
:4 0 1c6 :2 0 d5
:2 0 2a :2 0 e5c
1a5a 1a5c :3 0 e5f
1a56 1a5e 1a53 1a60
1a5f :2 0 c6 :3 0
d6 :3 0 1a62 1a63
0 c8 :4 0 1c7
:4 0 cb :2 0 cc
:3 0 e63 1a67 1a69
:3 0 cb :2 0 ea
:4 0 e66 1a6b 1a6d
:3 0 cb :2 0 eb
:4 0 e69 1a6f 1a71
:3 0 cb :2 0 e4
:3 0 e6c 1a73 1a75
:3 0 cb :2 0 ec
:4 0 e6f 1a77 1a79
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 e72
1a7c 1a80 e76 1a7b
1a82 :3 0 e79 1a64
1a84 :2 0 1a86 e7c
1a87 1a61 1a86 0
1a88 e7e 0 1a89
e80 1a8c :3 0 1a8c
0 1a8c 1a8b 1a89
1a8a :6 0 1a8e 41
:3 0 e83 1a90 e85
1a8f 1a8e :2 0 1a91
e87 :2 0 1a93 0
1a93 1a92 1a40 1a91
:6 0 1b6f 1 :3 0
1e :3 0 81 :3 0
1a95 1a96 0 1b6f
1c8 :3 0 1c9 :3 0
2b :3 0 1b2 :3 0
91 :3 0 22 :3 0
9a :3 0 1e :4 0
1ca 1 :8 0 1aa6
15 :3 0 e1 :3 0
1aa1 1aa2 0 e4
:3 0 1aa3 1aa4 0
1aa6 e8a 1af9 e5
:3 0 15 :3 0 e1
:3 0 1aa8 1aa9 0
e4 :3 0 1aaa 1aab
0 1aad e8d 1aaf
e8f 1aae 1aad :2 0
1af7 e6 :3 0 15
:3 0 e1 :3 0 1ab2
1ab3 0 e4 :3 0
1ab4 1ab5 0 1aef
c3 :3 0 c4 :3 0
1ab7 1ab8 0 c3
:3 0 d3 :3 0 1aba
1abb 0 e7 :2 0
2 :4 0 1cb :2 0
d5 :2 0 2a :2 0
e91 1ac0 1ac2 :3 0
e94 1abc 1ac4 1ab9
1ac6 1ac5 :2 0 c6
:3 0 d6 :3 0 1ac8
1ac9 0 c8 :4 0
1cc :4 0 cb :2 0
cc :3 0 e98 1acd
1acf :3 0 cb :2 0
ea :4 0 e9b 1ad1
1ad3 :3 0 cb :2 0
eb :4 0 e9e 1ad5
1ad7 :3 0 cb :2 0
e4 :3 0 ea1 1ad9
1adb :3 0 cb :2 0
ec :4 0 ea4 1add
1adf :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
ea7 1ae2 1ae6 eab
1ae1 1ae8 :3 0 eae
1aca 1aea :2 0 1aec
eb1 1aed 1ac7 1aec
0 1aee eb3 0
1aef eb5 1af2 :3 0
1af2 0 1af2 1af1
1aef 1af0 :6 0 1af4
43 :3 0 eb8 1af6
eba 1af5 1af4 :2 0
1af7 ebc :2 0 1af9
0 1af9 1af8 1aa6
1af7 :6 0 1b6f 1
:3 0 117 :3 0 167
:3 0 1afb 1afc 0
24 :3 0 2b :3 0
168 :2 0 ebf 1afd
1b01 1cd :2 0 3d
:2 0 ec5 1b03 1b05
:3 0 117 :3 0 1ce
:3 0 9f :3 0 a0
:3 0 2b :3 0 117
:3 0 1cf :3 0 24
:3 0 24 :3 0 1d0
:4 0 1d1 1 :8 0
1b17 15 :3 0 e1
:3 0 1b12 1b13 0
e4 :3 0 1b14 1b15
0 1b17 ec8 1b6a
e5 :3 0 15 :3 0
e1 :3 0 1b19 1b1a
0 e4 :3 0 1b1b
1b1c 0 1b1e ecb
1b20 ecd 1b1f 1b1e
:2 0 1b68 e6 :3 0
15 :3 0 e1 :3 0
1b23 1b24 0 e4
:3 0 1b25 1b26 0
1b60 c3 :3 0 c4
:3 0 1b28 1b29 0
c3 :3 0 d3 :3 0
1b2b 1b2c 0 e7
:2 0 2 :4 0 1d2
:2 0 d5 :2 0 2a
:2 0 ecf 1b31 1b33
:3 0 ed2 1b2d 1b35
1b2a 1b37 1b36 :2 0
c6 :3 0 d6 :3 0
1b39 1b3a 0 c8
:4 0 1d3 :4 0 cb
:2 0 cc :3 0 ed6
1b3e 1b40 :3 0 cb
:2 0 ea :4 0 ed9
1b42 1b44 :3 0 cb
:2 0 eb :4 0 edc
1b46 1b48 :3 0 cb
:2 0 e4 :3 0 edf
1b4a 1b4c :3 0 cb
:2 0 ec :4 0 ee2
1b4e 1b50 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 ee5 1b53 1b57
ee9 1b52 1b59 :3 0
eec 1b3b 1b5b :2 0
1b5d eef 1b5e 1b38
1b5d 0 1b5f ef1
0 1b60 ef3 1b63
:3 0 1b63 0 1b63
1b62 1b60 1b61 :6 0
1b65 45 :3 0 ef6
1b67 ef8 1b66 1b65
:2 0 1b68 efa :2 0
1b6a 0 1b6a 1b69
1b17 1b68 :6 0 1b6c
1 :3 0 efd 1b6d
1b06 1b6c 0 1b6e
eff 0 1b6f f01
1fdc 84 :3 0 62
:3 0 3d :2 0 f07
1b70 1b73 da :2 0
3d :2 0 f0c 1b75
1b77 :3 0 1b78 :2 0
84 :3 0 61 :3 0
3d :2 0 f0f 1b7a
1b7d da :2 0 3d
:2 0 f14 1b7f 1b81
:3 0 1b82 :2 0 1b79
1b84 1b83 :2 0 84
:3 0 78 :3 0 3d
:2 0 f17 1b86 1b89
da :2 0 3d :2 0
f1c 1b8b 1b8d :3 0
90 :3 0 98 :3 0
1c :3 0 92 :3 0
21 :3 0 93 :3 0
a7 :3 0 58 :3 0
1d4 :3 0 5c :3 0
a8 :3 0 5b :3 0
1d5 :3 0 5d :3 0
9e :3 0 57 :3 0
9e :3 0 7e :3 0
1d6 :3 0 9d :3 0
1d7 :3 0 58 :3 0
1d8 :3 0 19a :3 0
19b :3 0 1d8 :3 0
b0 :3 0 1d9 :3 0
9a :3 0 1e :4 0
1da 1 :8 0 1bc3
e0 :4 0 1bae :3 0
f8 :3 0 da :2 0
f21 1bb1 1bb2 :3 0
15 :3 0 e1 :3 0
1bb4 1bb5 0 3d
:2 0 1bb6 1bb7 0
1bb9 f24 1bc0 15
:3 0 e1 :3 0 1bba
1bbb 0 e2 :2 0
1bbc 1bbd 0 1bbf
f26 1bc1 1bb3 1bb9
0 1bc2 0 1bbf
0 1bc2 f28 0
1bc3 f2b 1c16 e5
:3 0 15 :3 0 e1
:3 0 1bc5 1bc6 0
e4 :3 0 1bc7 1bc8
0 1bca f2e 1bcc
f30 1bcb 1bca :2 0
1c14 e6 :3 0 15
:3 0 e1 :3 0 1bcf
1bd0 0 e4 :3 0
1bd1 1bd2 0 1c0c
c3 :3 0 c4 :3 0
1bd4 1bd5 0 c3
:3 0 d3 :3 0 1bd7
1bd8 0 e7 :2 0
2 :4 0 1db :2 0
d5 :2 0 2a :2 0
f32 1bdd 1bdf :3 0
f35 1bd9 1be1 1bd6
1be3 1be2 :2 0 c6
:3 0 d6 :3 0 1be5
1be6 0 c8 :4 0
1dc :4 0 cb :2 0
cc :3 0 f39 1bea
1bec :3 0 cb :2 0
ea :4 0 f3c 1bee
1bf0 :3 0 cb :2 0
eb :4 0 f3f 1bf2
1bf4 :3 0 cb :2 0
e4 :3 0 f42 1bf6
1bf8 :3 0 cb :2 0
ec :4 0 f45 1bfa
1bfc :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
f48 1bff 1c03 f4c
1bfe 1c05 :3 0 f4f
1be7 1c07 :2 0 1c09
f52 1c0a 1be4 1c09
0 1c0b f54 0
1c0c f56 1c0f :3 0
1c0f 0 1c0f 1c0e
1c0c 1c0d :6 0 1c11
47 :3 0 f59 1c13
f5b 1c12 1c11 :2 0
1c14 f5d :2 0 1c16
0 1c16 1c15 1bc3
1c14 :6 0 1c18 1
:3 0 f60 1ca7 90
:3 0 98 :3 0 1c
:3 0 92 :3 0 21
:3 0 93 :3 0 a7
:3 0 58 :3 0 1d4
:3 0 5c :3 0 a8
:3 0 5b :3 0 1d5
:3 0 5d :3 0 9e
:3 0 57 :3 0 9e
:3 0 7e :3 0 1d6
:3 0 9d :3 0 1d7
:3 0 58 :3 0 1dd
:3 0 9d :3 0 9d
:3 0 5b :3 0 1d8
:3 0 19a :3 0 19b
:3 0 1d8 :3 0 b0
:3 0 1d9 :3 0 9a
:3 0 1e :4 0 1de
1 :8 0 1c51 e0
:4 0 1c3c :3 0 f8
:3 0 da :2 0 f64
1c3f 1c40 :3 0 15
:3 0 e1 :3 0 1c42
1c43 0 3d :2 0
1c44 1c45 0 1c47
f67 1c4e 15 :3 0
e1 :3 0 1c48 1c49
0 e2 :2 0 1c4a
1c4b 0 1c4d f69
1c4f 1c41 1c47 0
1c50 0 1c4d 0
1c50 f6b 0 1c51
f6e 1ca4 e5 :3 0
15 :3 0 e1 :3 0
1c53 1c54 0 e4
:3 0 1c55 1c56 0
1c58 f71 1c5a f73
1c59 1c58 :2 0 1ca2
e6 :3 0 15 :3 0
e1 :3 0 1c5d 1c5e
0 e4 :3 0 1c5f
1c60 0 1c9a c3
:3 0 c4 :3 0 1c62
1c63 0 c3 :3 0
d3 :3 0 1c65 1c66
0 e7 :2 0 2
:4 0 1df :2 0 d5
:2 0 2a :2 0 f75
1c6b 1c6d :3 0 f78
1c67 1c6f 1c64 1c71
1c70 :2 0 c6 :3 0
d6 :3 0 1c73 1c74
0 c8 :4 0 1e0
:4 0 cb :2 0 cc
:3 0 f7c 1c78 1c7a
:3 0 cb :2 0 ea
:4 0 f7f 1c7c 1c7e
:3 0 cb :2 0 eb
:4 0 f82 1c80 1c82
:3 0 cb :2 0 e4
:3 0 f85 1c84 1c86
:3 0 cb :2 0 ec
:4 0 f88 1c88 1c8a
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 f8b
1c8d 1c91 f8f 1c8c
1c93 :3 0 f92 1c75
1c95 :2 0 1c97 f95
1c98 1c72 1c97 0
1c99 f97 0 1c9a
f99 1c9d :3 0 1c9d
0 1c9d 1c9c 1c9a
1c9b :6 0 1c9f 49
:3 0 f9c 1ca1 f9e
1ca0 1c9f :2 0 1ca2
fa0 :2 0 1ca4 0
1ca4 1ca3 1c51 1ca2
:6 0 1ca6 1 :3 0
fa3 1ca8 1b8e 1c18
0 1ca9 0 1ca6
0 1ca9 fa5 0
1cab 141 :3 0 fa8
1ef2 84 :3 0 62
:3 0 3d :2 0 faa
1cac 1caf da :2 0
3d :2 0 faf 1cb1
1cb3 :3 0 90 :3 0
98 :3 0 1c :3 0
92 :3 0 21 :3 0
93 :3 0 a7 :3 0
58 :3 0 1d4 :3 0
5c :3 0 9e :3 0
57 :3 0 9e :3 0
7e :3 0 1d6 :3 0
9d :3 0 1d7 :3 0
58 :3 0 1d8 :3 0
19a :3 0 19b :3 0
1d8 :3 0 b0 :3 0
1d9 :3 0 9a :3 0
1e :4 0 1e1 1
:8 0 1ce5 e0 :4 0
1cd0 :3 0 f8 :3 0
da :2 0 fb4 1cd3
1cd4 :3 0 15 :3 0
e1 :3 0 1cd6 1cd7
0 3d :2 0 1cd8
1cd9 0 1cdb fb7
1ce2 15 :3 0 e1
:3 0 1cdc 1cdd 0
e2 :2 0 1cde 1cdf
0 1ce1 fb9 1ce3
1cd5 1cdb 0 1ce4
0 1ce1 0 1ce4
fbb 0 1ce5 fbe
1d38 e5 :3 0 15
:3 0 e1 :3 0 1ce7
1ce8 0 e4 :3 0
1ce9 1cea 0 1cec
fc1 1cee fc3 1ced
1cec :2 0 1d36 e6
:3 0 15 :3 0 e1
:3 0 1cf1 1cf2 0
e4 :3 0 1cf3 1cf4
0 1d2e c3 :3 0
c4 :3 0 1cf6 1cf7
0 c3 :3 0 d3
:3 0 1cf9 1cfa 0
e7 :2 0 2 :4 0
1e2 :2 0 d5 :2 0
2a :2 0 fc5 1cff
1d01 :3 0 fc8 1cfb
1d03 1cf8 1d05 1d04
:2 0 c6 :3 0 d6
:3 0 1d07 1d08 0
c8 :4 0 1e3 :4 0
cb :2 0 cc :3 0
fcc 1d0c 1d0e :3 0
cb :2 0 ea :4 0
fcf 1d10 1d12 :3 0
cb :2 0 eb :4 0
fd2 1d14 1d16 :3 0
cb :2 0 e4 :3 0
fd5 1d18 1d1a :3 0
cb :2 0 ec :4 0
fd8 1d1c 1d1e :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 fdb 1d21
1d25 fdf 1d20 1d27
:3 0 fe2 1d09 1d29
:2 0 1d2b fe5 1d2c
1d06 1d2b 0 1d2d
fe7 0 1d2e fe9
1d31 :3 0 1d31 0
1d31 1d30 1d2e 1d2f
:6 0 1d33 4b :3 0
fec 1d35 fee 1d34
1d33 :2 0 1d36 ff0
:2 0 1d38 0 1d38
1d37 1ce5 1d36 :6 0
1d3b 1 :3 0 141
:3 0 ff3 1d3c 1cb4
1d3b 0 1ef4 84
:3 0 61 :3 0 3d
:2 0 ff5 1d3d 1d40
da :2 0 3d :2 0
ffa 1d42 1d44 :3 0
5b :3 0 84 :3 0
5b :3 0 3d :2 0
ffd 1d47 1d4a 1aa
:2 0 84 :3 0 5e
:3 0 3d :2 0 1000
1d4d 1d50 1003 1d4c
1d52 :3 0 1d46 1d53
0 1e78 5d :3 0
84 :3 0 5d :3 0
3d :2 0 1006 1d56
1d59 1aa :2 0 84
:3 0 5e :3 0 3d
:2 0 1009 1d5c 1d5f
100c 1d5b 1d61 :3 0
1d55 1d62 0 1e78
84 :3 0 78 :3 0
3d :2 0 100f 1d64
1d67 da :2 0 3d
:2 0 1014 1d69 1d6b
:3 0 90 :3 0 98
:3 0 1c :3 0 92
:3 0 21 :3 0 93
:3 0 a8 :3 0 5b
:3 0 1d5 :3 0 5d
:3 0 9e :3 0 57
:3 0 9e :3 0 7e
:3 0 1d8 :3 0 19a
:3 0 19b :3 0 1d8
:3 0 b0 :3 0 1d9
:3 0 9a :3 0 1e
:4 0 1e4 1 :8 0
1d99 e0 :4 0 1d84
:3 0 f8 :3 0 da
:2 0 1019 1d87 1d88
:3 0 15 :3 0 e1
:3 0 1d8a 1d8b 0
3d :2 0 1d8c 1d8d
0 1d8f 101c 1d96
15 :3 0 e1 :3 0
1d90 1d91 0 e2
:2 0 1d92 1d93 0
1d95 101e 1d97 1d89
1d8f 0 1d98 0
1d95 0 1d98 1020
0 1d99 1023 1dec
e5 :3 0 15 :3 0
e1 :3 0 1d9b 1d9c
0 e4 :3 0 1d9d
1d9e 0 1da0 1026
1da2 1028 1da1 1da0
:2 0 1dea e6 :3 0
15 :3 0 e1 :3 0
1da5 1da6 0 e4
:3 0 1da7 1da8 0
1de2 c3 :3 0 c4
:3 0 1daa 1dab 0
c3 :3 0 d3 :3 0
1dad 1dae 0 e7
:2 0 2 :4 0 1e5
:2 0 d5 :2 0 2a
:2 0 102a 1db3 1db5
:3 0 102d 1daf 1db7
1dac 1db9 1db8 :2 0
c6 :3 0 d6 :3 0
1dbb 1dbc 0 c8
:4 0 1e6 :4 0 cb
:2 0 cc :3 0 1031
1dc0 1dc2 :3 0 cb
:2 0 ea :4 0 1034
1dc4 1dc6 :3 0 cb
:2 0 eb :4 0 1037
1dc8 1dca :3 0 cb
:2 0 e4 :3 0 103a
1dcc 1dce :3 0 cb
:2 0 ec :4 0 103d
1dd0 1dd2 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 1040 1dd5 1dd9
1044 1dd4 1ddb :3 0
1047 1dbd 1ddd :2 0
1ddf 104a 1de0 1dba
1ddf 0 1de1 104c
0 1de2 104e 1de5
:3 0 1de5 0 1de5
1de4 1de2 1de3 :6 0
1de7 4d :3 0 1051
1de9 1053 1de8 1de7
:2 0 1dea 1055 :2 0
1dec 0 1dec 1deb
1d99 1dea :6 0 1dee
1 :3 0 1058 1e75
90 :3 0 98 :3 0
1c :3 0 92 :3 0
21 :3 0 93 :3 0
a8 :3 0 5b :3 0
1d5 :3 0 5d :3 0
9e :3 0 57 :3 0
9e :3 0 7e :3 0
1dd :3 0 9d :3 0
9d :3 0 5b :3 0
1d8 :3 0 19a :3 0
19b :3 0 1d8 :3 0
b0 :3 0 1d9 :3 0
9a :3 0 1e :4 0
1e7 1 :8 0 1e1f
e0 :4 0 1e0a :3 0
f8 :3 0 da :2 0
105c 1e0d 1e0e :3 0
15 :3 0 e1 :3 0
1e10 1e11 0 3d
:2 0 1e12 1e13 0
1e15 105f 1e1c 15
:3 0 e1 :3 0 1e16
1e17 0 e2 :2 0
1e18 1e19 0 1e1b
1061 1e1d 1e0f 1e15
0 1e1e 0 1e1b
0 1e1e 1063 0
1e1f 1066 1e72 e5
:3 0 15 :3 0 e1
:3 0 1e21 1e22 0
e4 :3 0 1e23 1e24
0 1e26 1069 1e28
106b 1e27 1e26 :2 0
1e70 e6 :3 0 15
:3 0 e1 :3 0 1e2b
1e2c 0 e4 :3 0
1e2d 1e2e 0 1e68
c3 :3 0 c4 :3 0
1e30 1e31 0 c3
:3 0 d3 :3 0 1e33
1e34 0 e7 :2 0
2 :4 0 1e8 :2 0
d5 :2 0 2a :2 0
106d 1e39 1e3b :3 0
1070 1e35 1e3d 1e32
1e3f 1e3e :2 0 c6
:3 0 d6 :3 0 1e41
1e42 0 c8 :4 0
1e9 :4 0 cb :2 0
cc :3 0 1074 1e46
1e48 :3 0 cb :2 0
ea :4 0 1077 1e4a
1e4c :3 0 cb :2 0
eb :4 0 107a 1e4e
1e50 :3 0 cb :2 0
e4 :3 0 107d 1e52
1e54 :3 0 cb :2 0
ec :4 0 1080 1e56
1e58 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
1083 1e5b 1e5f 1087
1e5a 1e61 :3 0 108a
1e43 1e63 :2 0 1e65
108d 1e66 1e40 1e65
0 1e67 108f 0
1e68 1091 1e6b :3 0
1e6b 0 1e6b 1e6a
1e68 1e69 :6 0 1e6d
4f :3 0 1094 1e6f
1096 1e6e 1e6d :2 0
1e70 1098 :2 0 1e72
0 1e72 1e71 1e1f
1e70 :6 0 1e74 1
:3 0 109b 1e76 1d6c
1dee 0 1e77 0
1e74 0 1e77 109d
0 1e78 10a0 1e79
1d45 1e78 0 1ef4
90 :3 0 98 :3 0
1c :3 0 9e :3 0
57 :3 0 9e :3 0
7e :3 0 92 :3 0
21 :3 0 93 :3 0
9a :3 0 1e :4 0
1ea 1 :8 0 1e9c
e0 :4 0 1e87 :3 0
f8 :3 0 da :2 0
10a6 1e8a 1e8b :3 0
15 :3 0 e1 :3 0
1e8d 1e8e 0 3d
:2 0 1e8f 1e90 0
1e92 10a9 1e99 15
:3 0 e1 :3 0 1e93
1e94 0 e2 :2 0
1e95 1e96 0 1e98
10ab 1e9a 1e8c 1e92
0 1e9b 0 1e98
0 1e9b 10ad 0
1e9c 10b0 1eef e5
:3 0 15 :3 0 e1
:3 0 1e9e 1e9f 0
e4 :3 0 1ea0 1ea1
0 1ea3 10b3 1ea5
10b5 1ea4 1ea3 :2 0
1eed e6 :3 0 15
:3 0 e1 :3 0 1ea8
1ea9 0 e4 :3 0
1eaa 1eab 0 1ee5
c3 :3 0 c4 :3 0
1ead 1eae 0 c3
:3 0 d3 :3 0 1eb0
1eb1 0 e7 :2 0
2 :4 0 1eb :2 0
d5 :2 0 2a :2 0
10b7 1eb6 1eb8 :3 0
10ba 1eb2 1eba 1eaf
1ebc 1ebb :2 0 c6
:3 0 d6 :3 0 1ebe
1ebf 0 c8 :4 0
1ec :4 0 cb :2 0
cc :3 0 10be 1ec3
1ec5 :3 0 cb :2 0
ea :4 0 10c1 1ec7
1ec9 :3 0 cb :2 0
eb :4 0 10c4 1ecb
1ecd :3 0 cb :2 0
e4 :3 0 10c7 1ecf
1ed1 :3 0 cb :2 0
ec :4 0 10ca 1ed3
1ed5 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
10cd 1ed8 1edc 10d1
1ed7 1ede :3 0 10d4
1ec0 1ee0 :2 0 1ee2
10d7 1ee3 1ebd 1ee2
0 1ee4 10d9 0
1ee5 10db 1ee8 :3 0
1ee8 0 1ee8 1ee7
1ee5 1ee6 :6 0 1eea
51 :3 0 10de 1eec
10e0 1eeb 1eea :2 0
1eed 10e2 :2 0 1eef
0 1eef 1eee 1e9c
1eed :6 0 1ef1 1
:3 0 10e5 1ef3 1b85
1cab 0 1ef4 0
1ef1 0 1ef4 10e7
0 1fdb 1b2 :3 0
91 :3 0 22 :3 0
9a :3 0 1e :4 0
1c5 1 :8 0 1f10
e0 :4 0 1efb :3 0
f8 :3 0 da :2 0
10ee 1efe 1eff :3 0
15 :3 0 e1 :3 0
1f01 1f02 0 3d
:2 0 1f03 1f04 0
1f06 10f1 1f0d 15
:3 0 e1 :3 0 1f07
1f08 0 e2 :2 0
1f09 1f0a 0 1f0c
10f3 1f0e 1f00 1f06
0 1f0f 0 1f0c
0 1f0f 10f5 0
1f10 10f8 1f63 e5
:3 0 15 :3 0 e1
:3 0 1f12 1f13 0
e4 :3 0 1f14 1f15
0 1f17 10fb 1f19
10fd 1f18 1f17 :2 0
1f61 e6 :3 0 15
:3 0 e1 :3 0 1f1c
1f1d 0 e4 :3 0
1f1e 1f1f 0 1f59
c3 :3 0 c4 :3 0
1f21 1f22 0 c3
:3 0 d3 :3 0 1f24
1f25 0 e7 :2 0
2 :4 0 1ed :2 0
d5 :2 0 2a :2 0
10ff 1f2a 1f2c :3 0
1102 1f26 1f2e 1f23
1f30 1f2f :2 0 c6
:3 0 d6 :3 0 1f32
1f33 0 c8 :4 0
1ee :4 0 cb :2 0
cc :3 0 1106 1f37
1f39 :3 0 cb :2 0
ea :4 0 1109 1f3b
1f3d :3 0 cb :2 0
eb :4 0 110c 1f3f
1f41 :3 0 cb :2 0
e4 :3 0 110f 1f43
1f45 :3 0 cb :2 0
ec :4 0 1112 1f47
1f49 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
1115 1f4c 1f50 1119
1f4b 1f52 :3 0 111c
1f34 1f54 :2 0 1f56
111f 1f57 1f31 1f56
0 1f58 1121 0
1f59 1123 1f5c :3 0
1f5c 0 1f5c 1f5b
1f59 1f5a :6 0 1f5e
53 :3 0 1126 1f60
1128 1f5f 1f5e :2 0
1f61 112a :2 0 1f63
0 1f63 1f62 1f10
1f61 :6 0 1fdb 1
:3 0 1ef :3 0 1e
:3 0 1f0 :3 0 1f1
:3 0 1f2 :3 0 1f3
:3 0 1f4 :3 0 1f5
:3 0 1ef :3 0 98
:3 0 23 :4 0 1f6
1 :8 0 1f86 e0
:4 0 1f71 :3 0 f8
:3 0 da :2 0 112f
1f74 1f75 :3 0 15
:3 0 e1 :3 0 1f77
1f78 0 3d :2 0
1f79 1f7a 0 1f7c
1132 1f83 15 :3 0
e1 :3 0 1f7d 1f7e
0 e2 :2 0 1f7f
1f80 0 1f82 1134
1f84 1f76 1f7c 0
1f85 0 1f82 0
1f85 1136 0 1f86
1139 1fd9 e5 :3 0
15 :3 0 e1 :3 0
1f88 1f89 0 e4
:3 0 1f8a 1f8b 0
1f8d 113c 1f8f 113e
1f8e 1f8d :2 0 1fd7
e6 :3 0 15 :3 0
e1 :3 0 1f92 1f93
0 e4 :3 0 1f94
1f95 0 1fcf c3
:3 0 c4 :3 0 1f97
1f98 0 c3 :3 0
d3 :3 0 1f9a 1f9b
0 e7 :2 0 2
:4 0 1f7 :2 0 d5
:2 0 2a :2 0 1140
1fa0 1fa2 :3 0 1143
1f9c 1fa4 1f99 1fa6
1fa5 :2 0 c6 :3 0
d6 :3 0 1fa8 1fa9
0 c8 :4 0 1f8
:4 0 cb :2 0 cc
:3 0 1147 1fad 1faf
:3 0 cb :2 0 ea
:4 0 114a 1fb1 1fb3
:3 0 cb :2 0 eb
:4 0 114d 1fb5 1fb7
:3 0 cb :2 0 e4
:3 0 1150 1fb9 1fbb
:3 0 cb :2 0 ec
:4 0 1153 1fbd 1fbf
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 1156
1fc2 1fc6 115a 1fc1
1fc8 :3 0 115d 1faa
1fca :2 0 1fcc 1160
1fcd 1fa7 1fcc 0
1fce 1162 0 1fcf
1164 1fd2 :3 0 1fd2
0 1fd2 1fd1 1fcf
1fd0 :6 0 1fd4 55
:3 0 1167 1fd6 1169
1fd5 1fd4 :2 0 1fd7
116b :2 0 1fd9 0
1fd9 1fd8 1f86 1fd7
:6 0 1fdb 1 :3 0
116e 1fdd 19b6 1b6f
0 1fde 0 1fdb
0 1fde 1172 0
236c 56 :3 0 2a
:2 0 1fdf 1fe0 0
236c af :3 0 98
:3 0 17 :4 0 1f9
1 :8 0 1ffb e0
:4 0 1fe6 :3 0 f8
:3 0 da :2 0 1177
1fe9 1fea :3 0 15
:3 0 e1 :3 0 1fec
1fed 0 3d :2 0
1fee 1fef 0 1ff1
117a 1ff8 15 :3 0
e1 :3 0 1ff2 1ff3
0 e2 :2 0 1ff4
1ff5 0 1ff7 117c
1ff9 1feb 1ff1 0
1ffa 0 1ff7 0
1ffa 117e 0 1ffb
1181 204e e5 :3 0
15 :3 0 e1 :3 0
1ffd 1ffe 0 e4
:3 0 1fff 2000 0
2002 1184 2004 1186
2003 2002 :2 0 204c
e6 :3 0 15 :3 0
e1 :3 0 2007 2008
0 e4 :3 0 2009
200a 0 2044 c3
:3 0 c4 :3 0 200c
200d 0 c3 :3 0
d3 :3 0 200f 2010
0 e7 :2 0 2
:4 0 1fa :2 0 d5
:2 0 2a :2 0 1188
2015 2017 :3 0 118b
2011 2019 200e 201b
201a :2 0 c6 :3 0
d6 :3 0 201d 201e
0 c8 :4 0 1fb
:4 0 cb :2 0 cc
:3 0 118f 2022 2024
:3 0 cb :2 0 ea
:4 0 1192 2026 2028
:3 0 cb :2 0 eb
:4 0 1195 202a 202c
:3 0 cb :2 0 e4
:3 0 1198 202e 2030
:3 0 cb :2 0 ec
:4 0 119b 2032 2034
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 119e
2037 203b 11a2 2036
203d :3 0 11a5 201f
203f :2 0 2041 11a8
2042 201c 2041 0
2043 11aa 0 2044
11ac 2047 :3 0 2047
0 2047 2046 2044
2045 :6 0 2049 57
:3 0 11af 204b 11b1
204a 2049 :2 0 204c
11b3 :2 0 204e 0
204e 204d 1ffb 204c
:6 0 236c 1 :3 0
1fc :3 0 91 :3 0
22 :3 0 92 :3 0
23 :4 0 1fd 1
:8 0 206b e0 :4 0
2056 :3 0 f8 :3 0
da :2 0 11b8 2059
205a :3 0 15 :3 0
e1 :3 0 205c 205d
0 3d :2 0 205e
205f 0 2061 11bb
2068 15 :3 0 e1
:3 0 2062 2063 0
e2 :2 0 2064 2065
0 2067 11bd 2069
205b 2061 0 206a
0 2067 0 206a
11bf 0 206b 11c2
20be e5 :3 0 15
:3 0 e1 :3 0 206d
206e 0 e4 :3 0
206f 2070 0 2072
11c5 2074 11c7 2073
2072 :2 0 20bc e6
:3 0 15 :3 0 e1
:3 0 2077 2078 0
e4 :3 0 2079 207a
0 20b4 c3 :3 0
c4 :3 0 207c 207d
0 c3 :3 0 d3
:3 0 207f 2080 0
e7 :2 0 2 :4 0
1fe :2 0 d5 :2 0
2a :2 0 11c9 2085
2087 :3 0 11cc 2081
2089 207e 208b 208a
:2 0 c6 :3 0 d6
:3 0 208d 208e 0
c8 :4 0 1ff :4 0
cb :2 0 cc :3 0
11d0 2092 2094 :3 0
cb :2 0 ea :4 0
11d3 2096 2098 :3 0
cb :2 0 eb :4 0
11d6 209a 209c :3 0
cb :2 0 e4 :3 0
11d9 209e 20a0 :3 0
cb :2 0 ec :4 0
11dc 20a2 20a4 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 11df 20a7
20ab 11e3 20a6 20ad
:3 0 11e6 208f 20af
:2 0 20b1 11e9 20b2
208c 20b1 0 20b3
11eb 0 20b4 11ed
20b7 :3 0 20b7 0
20b7 20b6 20b4 20b5
:6 0 20b9 59 :3 0
11f0 20bb 11f2 20ba
20b9 :2 0 20bc 11f4
:2 0 20be 0 20be
20bd 206b 20bc :6 0
236c 1 :3 0 84
:3 0 76 :3 0 3d
:2 0 11f7 20c0 20c3
da :2 0 2a :2 0
11fc 20c5 20c7 :3 0
20c8 :2 0 84 :3 0
77 :3 0 3d :2 0
11ff 20ca 20cd da
:2 0 3d :2 0 1204
20cf 20d1 :3 0 20d2
:2 0 20c9 20d4 20d3
:2 0 4a :3 0 3d
:2 0 20d6 20d7 0
2369 40 :3 0 3d
:2 0 20d9 20da 0
2369 200 :3 0 201
:3 0 20dc 20dd 0
57 :3 0 2a :2 0
1207 20de 20e1 13c
:2 0 3d :2 0 120c
20e3 20e5 :3 0 51
:3 0 13b :2 0 20e7
20e8 0 20ea 120f
20ef 51 :3 0 2a
:2 0 20eb 20ec 0
20ee 1211 20f0 20e6
20ea 0 20f1 0
20ee 0 20f1 1213
0 2369 84 :3 0
f :3 0 3d :2 0
1216 20f2 20f5 da
:2 0 3d :2 0 121b
20f7 20f9 :3 0 44
:3 0 3d :2 0 20fb
20fc 0 20fe 121e
20ff 20fa 20fe 0
2100 1220 0 2369
6b :3 0 202 :3 0
31 :3 0 32 :3 0
84 :3 0 46 :3 0
3d :2 0 1222 2105
2108 84 :3 0 47
:3 0 3d :2 0 1225
210a 210d 84 :3 0
52 :3 0 3d :2 0
1228 210f 2112 84
:3 0 4e :3 0 3d
:2 0 122b 2114 2117
84 :3 0 48 :3 0
3d :2 0 122e 2119
211c 84 :3 0 45
:3 0 3d :2 0 1231
211e 2121 84 :3 0
4f :3 0 3d :2 0
1234 2123 2126 84
:3 0 50 :3 0 3d
:2 0 1237 2128 212b
44 :3 0 3f :3 0
49 :3 0 123a 2102
2130 2101 2131 0
2369 84 :3 0 6b
:3 0 3d :2 0 1248
2133 2136 b0 :3 0
da :2 0 203 :3 0
2138 213a 0 124d
2139 213c :3 0 c3
:3 0 c4 :3 0 213e
213f 0 c3 :3 0
d3 :3 0 2141 2142
0 204 :2 0 2
:4 0 205 :2 0 d5
:2 0 2a :2 0 1250
2147 2149 :3 0 1253
2143 214b 2140 214d
214c :2 0 c6 :3 0
d6 :3 0 214f 2150
0 c8 :4 0 206
:4 0 cb :2 0 cc
:3 0 1257 2154 2156
:3 0 cb :2 0 ea
:4 0 125a 2158 215a
:3 0 cb :2 0 f1
:4 0 125d 215c 215e
:3 0 cb :2 0 84
:3 0 d0 :3 0 b0
:3 0 203 :3 0 2163
2164 0 1260 2162
2166 d1 :4 0 1262
2161 2169 1265 2160
216b :3 0 1268 2151
216d :2 0 216f 126b
2170 214e 216f 0
2171 126d 0 219b
c3 :3 0 c4 :3 0
2172 2173 0 c3
:3 0 d3 :3 0 2175
2176 0 2a :2 0
2 :4 0 207 :2 0
d5 :2 0 2a :2 0
126f 217b 217d :3 0
1272 2177 217f 2174
2181 2180 :2 0 c6
:3 0 d6 :3 0 2183
2184 0 c8 :4 0
208 :4 0 cb :2 0
cc :3 0 1276 2188
218a :3 0 1279 2185
218c :2 0 218e 127c
218f 2182 218e 0
2190 127e 0 219b
f6 :3 0 209 :3 0
2191 2192 0 b0
:3 0 203 :3 0 2194
2195 0 f8 :3 0
1280 2193 2198 :2 0
219b 141 :3 0 1283
2214 84 :3 0 6b
:3 0 3d :2 0 1287
219c 219f 13c :2 0
3d :2 0 128c 21a1
21a3 :3 0 c3 :3 0
c4 :3 0 21a5 21a6
0 c3 :3 0 d3
:3 0 21a8 21a9 0
18 :2 0 2 :4 0
20a :2 0 d5 :2 0
2a :2 0 128f 21ae
21b0 :3 0 1292 21aa
21b2 21a7 21b4 21b3
:2 0 c6 :3 0 d6
:3 0 21b6 21b7 0
c8 :4 0 20b :4 0
cb :2 0 cc :3 0
1296 21bb 21bd :3 0
cb :2 0 ea :4 0
1299 21bf 21c1 :3 0
cb :2 0 f1 :4 0
129c 21c3 21c5 :3 0
cb :2 0 84 :3 0
d0 :3 0 6b :3 0
129f 21c9 21cb d1
:4 0 12a1 21c8 21ce
12a4 21c7 21d0 :3 0
cb :2 0 f3 :4 0
12a7 21d2 21d4 :3 0
cb :2 0 ce :4 0
12aa 21d6 21d8 :3 0
cb :2 0 22 :3 0
12ad 21da 21dc :3 0
cb :2 0 ce :4 0
12b0 21de 21e0 :3 0
12b3 21b8 21e2 :2 0
21e4 12b6 21e5 21b5
21e4 0 21e6 12b8
0 2212 c3 :3 0
c4 :3 0 21e7 21e8
0 c3 :3 0 d3
:3 0 21ea 21eb 0
2a :2 0 2 :4 0
20c :2 0 d5 :2 0
2a :2 0 12ba 21f0
21f2 :3 0 12bd 21ec
21f4 21e9 21f6 21f5
:2 0 c6 :3 0 d6
:3 0 21f8 21f9 0
c8 :4 0 20d :4 0
cb :2 0 cc :3 0
12c1 21fd 21ff :3 0
12c4 21fa 2201 :2 0
2203 12c7 2204 21f7
2203 0 2205 12c9
0 2212 f6 :3 0
f7 :3 0 2206 2207
0 84 :3 0 6b
:3 0 3d :2 0 12cb
2209 220c 22 :3 0
f8 :3 0 12ce 2208
2210 :2 0 2212 12d2
2213 21a4 2212 0
2215 213d 219b 0
2215 12d6 0 2369
84 :3 0 44 :3 0
3d :2 0 12d9 2216
2219 da :2 0 3d
:2 0 12de 221b 221d
:3 0 4e :3 0 b0
:3 0 20e :3 0 2220
2221 0 221f 2222
0 2224 12e1 2235
4e :3 0 200 :3 0
19b :3 0 2226 2227
0 84 :3 0 4e
:3 0 3d :2 0 12e3
2229 222c b0 :3 0
20e :3 0 222e 222f
0 12e6 2228 2231
2225 2232 0 2234
12e9 2236 221e 2224
0 2237 0 2234
0 2237 12eb 0
2369 6a :3 0 5a
:2 0 2238 2239 0
2369 66 :3 0 20f
:3 0 1e :3 0 22
:3 0 22 :3 0 21
:3 0 6a :3 0 41
:3 0 29 :3 0 29
:3 0 29 :3 0 29
:3 0 24 :3 0 26
:3 0 49 :3 0 3f
:3 0 44 :3 0 46
:3 0 47 :3 0 40
:3 0 52 :3 0 2e
:3 0 4a :3 0 4e
:3 0 48 :3 0 45
:3 0 51 :3 0 4f
:3 0 53 :3 0 1a
:3 0 3d :2 0 3d
:2 0 5a :2 0 5a
:2 0 12ee 223c 225d
223b 225e 0 2369
84 :3 0 66 :3 0
3d :2 0 130f 2260
2263 da :2 0 210
:2 0 1314 2265 2267
:3 0 c3 :3 0 c4
:3 0 2269 226a 0
c3 :3 0 d3 :3 0
226c 226d 0 18
:2 0 2 :4 0 211
:2 0 d5 :2 0 2a
:2 0 1317 2272 2274
:3 0 131a 226e 2276
226b 2278 2277 :2 0
c6 :3 0 d6 :3 0
227a 227b 0 c8
:4 0 212 :4 0 cb
:2 0 cc :3 0 131e
227f 2281 :3 0 cb
:2 0 ea :4 0 1321
2283 2285 :3 0 cb
:2 0 f1 :4 0 1324
2287 2289 :3 0 cb
:2 0 84 :3 0 d0
:3 0 b0 :3 0 213
:3 0 228e 228f 0
1327 228d 2291 d1
:4 0 1329 228c 2294
132c 228b 2296 :3 0
cb :2 0 f3 :4 0
132f 2298 229a :3 0
cb :2 0 ce :4 0
1332 229c 229e :3 0
cb :2 0 21 :3 0
1335 22a0 22a2 :3 0
cb :2 0 ce :4 0
1338 22a4 22a6 :3 0
133b 227c 22a8 :2 0
22aa 133e 22ab 2279
22aa 0 22ac 1340
0 22d7 c3 :3 0
c4 :3 0 22ad 22ae
0 c3 :3 0 d3
:3 0 22b0 22b1 0
2a :2 0 2 :4 0
214 :2 0 d5 :2 0
2a :2 0 1342 22b6
22b8 :3 0 1345 22b2
22ba 22af 22bc 22bb
:2 0 c6 :3 0 d6
:3 0 22be 22bf 0
c8 :4 0 215 :4 0
cb :2 0 cc :3 0
1349 22c3 22c5 :3 0
134c 22c0 22c7 :2 0
22c9 134f 22ca 22bd
22c9 0 22cb 1351
0 22d7 f6 :3 0
f7 :3 0 22cc 22cd
0 b0 :3 0 213
:3 0 22cf 22d0 0
21 :3 0 f8 :3 0
1353 22ce 22d4 :2 0
22d7 141 :3 0 1357
2366 84 :3 0 66
:3 0 3d :2 0 135b
22d8 22db 13c :2 0
3d :2 0 1360 22dd
22df :3 0 c3 :3 0
c4 :3 0 22e1 22e2
0 c3 :3 0 d3
:3 0 22e4 22e5 0
18 :2 0 2 :4 0
216 :2 0 d5 :2 0
2a :2 0 1363 22ea
22ec :3 0 1366 22e6
22ee 22e3 22f0 22ef
:2 0 c6 :3 0 d6
:3 0 22f2 22f3 0
c8 :4 0 217 :4 0
cb :2 0 cc :3 0
136a 22f7 22f9 :3 0
cb :2 0 ea :4 0
136d 22fb 22fd :3 0
cb :2 0 f1 :4 0
1370 22ff 2301 :3 0
cb :2 0 84 :3 0
d0 :3 0 b0 :3 0
1bc :3 0 2306 2307
0 1373 2305 2309
d1 :4 0 1375 2304
230c 1378 2303 230e
:3 0 cb :2 0 f3
:4 0 137b 2310 2312
:3 0 cb :2 0 ce
:4 0 137e 2314 2316
:3 0 cb :2 0 1c
:3 0 1381 2318 231a
:3 0 cb :2 0 ce
:4 0 1384 231c 231e
:3 0 1387 22f4 2320
:2 0 2322 138a 2323
22f1 2322 0 2324
138c 0 234e c3
:3 0 c4 :3 0 2325
2326 0 c3 :3 0
d3 :3 0 2328 2329
0 2a :2 0 2
:4 0 218 :2 0 d5
:2 0 2a :2 0 138e
232e 2330 :3 0 1391
232a 2332 2327 2334
2333 :2 0 c6 :3 0
d6 :3 0 2336 2337
0 c8 :4 0 219
:4 0 cb :2 0 cc
:3 0 1395 233b 233d
:3 0 1398 2338 233f
:2 0 2341 139b 2342
2335 2341 0 2343
139d 0 234e f6
:3 0 f7 :3 0 2344
2345 0 b0 :3 0
1bc :3 0 2347 2348
0 1c :3 0 f8
:3 0 139f 2346 234c
:2 0 234e 13a3 234f
22e0 234e 0 2368
6b :3 0 13a :2 0
2350 2351 0 2365
6b :3 0 21a :3 0
22 :3 0 1d :3 0
6b :3 0 84 :3 0
4f :3 0 3d :2 0
13a7 2358 235b 84
:3 0 3f :3 0 3d
:2 0 13aa 235d 2360
13ad 2354 2362 2353
2363 0 2365 13b3
2367 2268 22d7 0
2368 0 2365 0
2368 13b6 0 2369
13ba 236a 20d5 2369
0 236b 13c5 0
236c 13c7 236d 1989
236c 0 236e 13ce
0 2370 141 :3 0
13d0 2486 a0 :3 0
6f :3 0 2a :2 0
2a :2 0 13d6 2371
2375 da :2 0 1bf
:4 0 13dc 2377 2379
:3 0 9a :3 0 9f
:3 0 a0 :3 0 a1
:3 0 9f :3 0 a0
:3 0 a2 :3 0 a5
:3 0 a6 :3 0 a7
:3 0 a8 :3 0 81
:3 0 2b :3 0 2d
:3 0 78 :3 0 79
:3 0 74 :3 0 75
:3 0 90 :3 0 91
:3 0 22 :3 0 9b
:3 0 22 :3 0 93
:3 0 92 :3 0 23
:4 0 21b 1 :8 0
239b 15 :3 0 e1
:3 0 2396 2397 0
e4 :3 0 2398 2399
0 239b 13df 23ee
e5 :3 0 15 :3 0
e1 :3 0 239d 239e
0 e4 :3 0 239f
23a0 0 23a2 13e2
23a4 13e4 23a3 23a2
:2 0 23ec e6 :3 0
15 :3 0 e1 :3 0
23a7 23a8 0 e4
:3 0 23a9 23aa 0
23e4 c3 :3 0 c4
:3 0 23ac 23ad 0
c3 :3 0 d3 :3 0
23af 23b0 0 e7
:2 0 2 :4 0 21c
:2 0 d5 :2 0 2a
:2 0 13e6 23b5 23b7
:3 0 13e9 23b1 23b9
23ae 23bb 23ba :2 0
c6 :3 0 d6 :3 0
23bd 23be 0 c8
:4 0 21d :4 0 cb
:2 0 cc :3 0 13ed
23c2 23c4 :3 0 cb
:2 0 ea :4 0 13f0
23c6 23c8 :3 0 cb
:2 0 eb :4 0 13f3
23ca 23cc :3 0 cb
:2 0 e4 :3 0 13f6
23ce 23d0 :3 0 cb
:2 0 ec :4 0 13f9
23d2 23d4 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 13fc 23d7 23db
1400 23d6 23dd :3 0
1403 23bf 23df :2 0
23e1 1406 23e2 23bc
23e1 0 23e3 1408
0 23e4 140a 23e7
:3 0 23e7 0 23e7
23e6 23e4 23e5 :6 0
23e9 5b :3 0 140d
23eb 140f 23ea 23e9
:2 0 23ec 1411 :2 0
23ee 0 23ee 23ed
239b 23ec :6 0 241c
1 :3 0 15 :3 0
e1 :3 0 23f0 23f1
0 15 :3 0 da
:2 0 158 :3 0 23f3
23f5 0 1416 23f4
23f7 :3 0 23f8 :2 0
81 :3 0 166 :2 0
1419 23fb 23fc :3 0
23fd :2 0 23fe :2 0
23f9 2400 23ff :2 0
7a :3 0 1c0 :2 0
2a :2 0 141b 2403
2405 :3 0 2402 2406
0 2419 15 :3 0
e1 :3 0 2408 2409
0 1c1 :3 0 22
:3 0 22 :3 0 81
:3 0 2b :3 0 2d
:3 0 74 :3 0 75
:3 0 7a :3 0 79
:3 0 78 :3 0 141d
240b 2416 240a 2417
0 2419 1428 241a
2401 2419 0 241b
142b 0 241c 142d
241d 237a 241c 0
2488 9a :3 0 a5
:3 0 81 :3 0 78
:3 0 90 :3 0 91
:3 0 22 :3 0 9b
:3 0 22 :3 0 92
:3 0 23 :3 0 93
:4 0 21e 1 :8 0
2430 15 :3 0 e1
:3 0 242b 242c 0
e4 :3 0 242d 242e
0 2430 1430 2483
e5 :3 0 15 :3 0
e1 :3 0 2432 2433
0 e4 :3 0 2434
2435 0 2437 1433
2439 1435 2438 2437
:2 0 2481 e6 :3 0
15 :3 0 e1 :3 0
243c 243d 0 e4
:3 0 243e 243f 0
2479 c3 :3 0 c4
:3 0 2441 2442 0
c3 :3 0 d3 :3 0
2444 2445 0 e7
:2 0 2 :4 0 21f
:2 0 d5 :2 0 2a
:2 0 1437 244a 244c
:3 0 143a 2446 244e
2443 2450 244f :2 0
c6 :3 0 d6 :3 0
2452 2453 0 c8
:4 0 220 :4 0 cb
:2 0 cc :3 0 143e
2457 2459 :3 0 cb
:2 0 ea :4 0 1441
245b 245d :3 0 cb
:2 0 eb :4 0 1444
245f 2461 :3 0 cb
:2 0 e4 :3 0 1447
2463 2465 :3 0 cb
:2 0 ec :4 0 144a
2467 2469 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 144d 246c 2470
1451 246b 2472 :3 0
1454 2454 2474 :2 0
2476 1457 2477 2451
2476 0 2478 1459
0 2479 145b 247c
:3 0 247c 0 247c
247b 2479 247a :6 0
247e 5d :3 0 145e
2480 1460 247f 247e
:2 0 2481 1462 :2 0
2483 0 2483 2482
2430 2481 :6 0 2485
1 :3 0 1465 2487
1805 2370 0 2488
0 2485 0 2488
1467 0 248a 141
:3 0 146b 25a0 a0
:3 0 6f :3 0 2a
:2 0 2a :2 0 146d
248b 248f da :2 0
1bf :4 0 1473 2491
2493 :3 0 9a :3 0
9f :3 0 a0 :3 0
a1 :3 0 9f :3 0
a0 :3 0 a2 :3 0
a5 :3 0 a6 :3 0
a7 :3 0 a8 :3 0
81 :3 0 2b :3 0
2d :3 0 78 :3 0
79 :3 0 74 :3 0
75 :3 0 90 :3 0
91 :3 0 22 :3 0
9b :3 0 22 :3 0
92 :3 0 23 :3 0
93 :4 0 221 1
:8 0 24b5 15 :3 0
e1 :3 0 24b0 24b1
0 e4 :3 0 24b2
24b3 0 24b5 1476
2508 e5 :3 0 15
:3 0 e1 :3 0 24b7
24b8 0 e4 :3 0
24b9 24ba 0 24bc
1479 24be 147b 24bd
24bc :2 0 2506 e6
:3 0 15 :3 0 e1
:3 0 24c1 24c2 0
e4 :3 0 24c3 24c4
0 24fe c3 :3 0
c4 :3 0 24c6 24c7
0 c3 :3 0 d3
:3 0 24c9 24ca 0
e7 :2 0 2 :4 0
222 :2 0 d5 :2 0
2a :2 0 147d 24cf
24d1 :3 0 1480 24cb
24d3 24c8 24d5 24d4
:2 0 c6 :3 0 d6
:3 0 24d7 24d8 0
c8 :4 0 223 :4 0
cb :2 0 cc :3 0
1484 24dc 24de :3 0
cb :2 0 ea :4 0
1487 24e0 24e2 :3 0
cb :2 0 eb :4 0
148a 24e4 24e6 :3 0
cb :2 0 e4 :3 0
148d 24e8 24ea :3 0
cb :2 0 ec :4 0
1490 24ec 24ee :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 1493 24f1
24f5 1497 24f0 24f7
:3 0 149a 24d9 24f9
:2 0 24fb 149d 24fc
24d6 24fb 0 24fd
149f 0 24fe 14a1
2501 :3 0 2501 0
2501 2500 24fe 24ff
:6 0 2503 5f :3 0
14a4 2505 14a6 2504
2503 :2 0 2506 14a8
:2 0 2508 0 2508
2507 24b5 2506 :6 0
2536 1 :3 0 15
:3 0 e1 :3 0 250a
250b 0 15 :3 0
da :2 0 158 :3 0
250d 250f 0 14ad
250e 2511 :3 0 2512
:2 0 81 :3 0 166
:2 0 14b0 2515 2516
:3 0 2517 :2 0 2518
:2 0 2513 251a 2519
:2 0 7a :3 0 1c0
:2 0 2a :2 0 14b2
251d 251f :3 0 251c
2520 0 2533 15
:3 0 e1 :3 0 2522
2523 0 1c1 :3 0
22 :3 0 22 :3 0
81 :3 0 2b :3 0
2d :3 0 74 :3 0
75 :3 0 7a :3 0
79 :3 0 78 :3 0
14b4 2525 2530 2524
2531 0 2533 14bf
2534 251b 2533 0
2535 14c2 0 2536
14c4 2537 2494 2536
0 25a2 9a :3 0
a5 :3 0 81 :3 0
78 :3 0 90 :3 0
91 :3 0 22 :3 0
9b :3 0 22 :3 0
92 :3 0 23 :3 0
93 :4 0 224 1
:8 0 254a 15 :3 0
e1 :3 0 2545 2546
0 e4 :3 0 2547
2548 0 254a 14c7
259d e5 :3 0 15
:3 0 e1 :3 0 254c
254d 0 e4 :3 0
254e 254f 0 2551
14ca 2553 14cc 2552
2551 :2 0 259b e6
:3 0 15 :3 0 e1
:3 0 2556 2557 0
e4 :3 0 2558 2559
0 2593 c3 :3 0
c4 :3 0 255b 255c
0 c3 :3 0 d3
:3 0 255e 255f 0
e7 :2 0 2 :4 0
225 :2 0 d5 :2 0
2a :2 0 14ce 2564
2566 :3 0 14d1 2560
2568 255d 256a 2569
:2 0 c6 :3 0 d6
:3 0 256c 256d 0
c8 :4 0 226 :4 0
cb :2 0 cc :3 0
14d5 2571 2573 :3 0
cb :2 0 ea :4 0
14d8 2575 2577 :3 0
cb :2 0 eb :4 0
14db 2579 257b :3 0
cb :2 0 e4 :3 0
14de 257d 257f :3 0
cb :2 0 ec :4 0
14e1 2581 2583 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 14e4 2586
258a 14e8 2585 258c
:3 0 14eb 256e 258e
:2 0 2590 14ee 2591
256b 2590 0 2592
14f0 0 2593 14f2
2596 :3 0 2596 0
2596 2595 2593 2594
:6 0 2598 61 :3 0
14f5 259a 14f7 2599
2598 :2 0 259b 14f9
:2 0 259d 0 259d
259c 254a 259b :6 0
259f 1 :3 0 14fc
25a1 17fa 248a 0
25a2 0 259f 0
25a2 14fe 0 4e8b
1e :3 0 227 :2 0
1502 25a4 25a5 :3 0
25a6 :2 0 25a7 :2 0
84 :3 0 4c :3 0
3d :2 0 1504 25a9
25ac 10b :2 0 3d
:2 0 1509 25ae 25b0
:3 0 25b1 :2 0 25a8
25b3 25b2 :2 0 84
:3 0 63 :3 0 3d
:2 0 150c 25b5 25b8
da :2 0 3d :2 0
1511 25ba 25bc :3 0
25bd :2 0 25b4 25bf
25be :2 0 1e :3 0
de :4 0 25c1 25c2
0 289d 1e :3 0
172 :4 0 25c5 25c6
:3 0 25c4 25c7 0
289d 84 :3 0 e
:3 0 3d :2 0 1514
25c9 25cc da :2 0
e2 :2 0 1519 25ce
25d0 :3 0 7e :3 0
84 :3 0 44 :3 0
3d :2 0 151c 25d3
25d6 25d2 25d7 0
25d9 151f 25da 25d1
25d9 0 25db 1521
0 289d 84 :3 0
62 :3 0 3d :2 0
1523 25dc 25df da
:2 0 3d :2 0 1528
25e1 25e3 :3 0 25e4
:2 0 84 :3 0 61
:3 0 3d :2 0 152b
25e6 25e9 da :2 0
3d :2 0 1530 25eb
25ed :3 0 25ee :2 0
25e5 25f0 25ef :2 0
84 :3 0 62 :3 0
3d :2 0 1533 25f2
25f5 da :2 0 3d
:2 0 1538 25f7 25f9
:3 0 25fa :2 0 84
:3 0 61 :3 0 3d
:2 0 153b 25fc 25ff
da :2 0 3d :2 0
1540 2601 2603 :3 0
2604 :2 0 25fb 2606
2605 :2 0 90 :3 0
9a :3 0 98 :3 0
9b :3 0 91 :3 0
228 :3 0 229 :3 0
22a :3 0 22b :3 0
a1 :3 0 a2 :3 0
15e :3 0 22c :3 0
22d :3 0 92 :3 0
93 :3 0 94 :3 0
a5 :3 0 a6 :3 0
a7 :3 0 1d4 :3 0
a8 :3 0 1d5 :3 0
fa :3 0 fb :3 0
179 :3 0 1e :3 0
1c :3 0 22 :3 0
22 :3 0 b0 :3 0
22e :3 0 b0 :3 0
22f :3 0 31 :3 0
32 :3 0 31 :3 0
32 :3 0 31 :3 0
32 :3 0 21 :3 0
2e :3 0 1e :3 0
78 :3 0 79 :3 0
58 :3 0 5c :3 0
5b :3 0 5d :3 0
7f :3 0 80 :3 0
7b :4 0 230 1
:8 0 2652 e0 :4 0
263d :3 0 f8 :3 0
da :2 0 1545 2640
2641 :3 0 15 :3 0
e1 :3 0 2643 2644
0 3d :2 0 2645
2646 0 2648 1548
264f 15 :3 0 e1
:3 0 2649 264a 0
e2 :2 0 264b 264c
0 264e 154a 2650
2642 2648 0 2651
0 264e 0 2651
154c 0 2652 154f
26a5 e5 :3 0 15
:3 0 e1 :3 0 2654
2655 0 e4 :3 0
2656 2657 0 2659
1552 265b 1554 265a
2659 :2 0 26a3 e6
:3 0 15 :3 0 e1
:3 0 265e 265f 0
e4 :3 0 2660 2661
0 269b c3 :3 0
c4 :3 0 2663 2664
0 c3 :3 0 d3
:3 0 2666 2667 0
e7 :2 0 2 :4 0
231 :2 0 d5 :2 0
2a :2 0 1556 266c
266e :3 0 1559 2668
2670 2665 2672 2671
:2 0 c6 :3 0 d6
:3 0 2674 2675 0
c8 :4 0 232 :4 0
cb :2 0 cc :3 0
155d 2679 267b :3 0
cb :2 0 ea :4 0
1560 267d 267f :3 0
cb :2 0 eb :4 0
1563 2681 2683 :3 0
cb :2 0 e4 :3 0
1566 2685 2687 :3 0
cb :2 0 ec :4 0
1569 2689 268b :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 156c 268e
2692 1570 268d 2694
:3 0 1573 2676 2696
:2 0 2698 1576 2699
2673 2698 0 269a
1578 0 269b 157a
269e :3 0 269e 0
269e 269d 269b 269c
:6 0 26a0 63 :3 0
157d 26a2 157f 26a1
26a0 :2 0 26a3 1581
:2 0 26a5 0 26a5
26a4 2652 26a3 :6 0
26a8 1 :3 0 141
:3 0 1584 27f3 84
:3 0 61 :3 0 3d
:2 0 1586 26a9 26ac
da :2 0 3d :2 0
158b 26ae 26b0 :3 0
90 :3 0 9a :3 0
98 :3 0 9b :3 0
91 :3 0 228 :3 0
229 :3 0 22a :3 0
22b :3 0 a1 :3 0
a2 :3 0 15e :3 0
22c :3 0 22d :3 0
92 :3 0 93 :3 0
94 :3 0 a5 :3 0
a6 :3 0 a7 :3 0
1d4 :3 0 a8 :3 0
1d5 :3 0 fa :3 0
fb :3 0 179 :3 0
1e :3 0 1c :3 0
22 :3 0 22 :3 0
b0 :3 0 22e :3 0
b0 :3 0 22f :3 0
31 :3 0 32 :3 0
31 :3 0 32 :3 0
31 :3 0 32 :3 0
21 :3 0 2e :3 0
1e :3 0 78 :3 0
79 :3 0 58 :3 0
5c :3 0 5b :3 0
5d :3 0 7f :3 0
80 :3 0 7b :4 0
233 1 :8 0 26fc
e0 :4 0 26e7 :3 0
f8 :3 0 da :2 0
1590 26ea 26eb :3 0
15 :3 0 e1 :3 0
26ed 26ee 0 3d
:2 0 26ef 26f0 0
26f2 1593 26f9 15
:3 0 e1 :3 0 26f3
26f4 0 e2 :2 0
26f5 26f6 0 26f8
1595 26fa 26ec 26f2
0 26fb 0 26f8
0 26fb 1597 0
26fc 159a 274f e5
:3 0 15 :3 0 e1
:3 0 26fe 26ff 0
e4 :3 0 2700 2701
0 2703 159d 2705
159f 2704 2703 :2 0
274d e6 :3 0 15
:3 0 e1 :3 0 2708
2709 0 e4 :3 0
270a 270b 0 2745
c3 :3 0 c4 :3 0
270d 270e 0 c3
:3 0 d3 :3 0 2710
2711 0 e7 :2 0
2 :4 0 234 :2 0
d5 :2 0 2a :2 0
15a1 2716 2718 :3 0
15a4 2712 271a 270f
271c 271b :2 0 c6
:3 0 d6 :3 0 271e
271f 0 c8 :4 0
235 :4 0 cb :2 0
cc :3 0 15a8 2723
2725 :3 0 cb :2 0
ea :4 0 15ab 2727
2729 :3 0 cb :2 0
eb :4 0 15ae 272b
272d :3 0 cb :2 0
e4 :3 0 15b1 272f
2731 :3 0 cb :2 0
ec :4 0 15b4 2733
2735 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
15b7 2738 273c 15bb
2737 273e :3 0 15be
2720 2740 :2 0 2742
15c1 2743 271d 2742
0 2744 15c3 0
2745 15c5 2748 :3 0
2748 0 2748 2747
2745 2746 :6 0 274a
65 :3 0 15c8 274c
15ca 274b 274a :2 0
274d 15cc :2 0 274f
0 274f 274e 26fc
274d :6 0 2751 1
:3 0 15cf 2752 26b1
2751 0 27f5 90
:3 0 9a :3 0 98
:3 0 9b :3 0 91
:3 0 228 :3 0 229
:3 0 22a :3 0 22b
:3 0 a1 :3 0 a2
:3 0 15e :3 0 22c
:3 0 22d :3 0 92
:3 0 93 :3 0 94
:3 0 a5 :3 0 a6
:3 0 a7 :3 0 1d4
:3 0 a8 :3 0 1d5
:3 0 fa :3 0 fb
:3 0 179 :3 0 1e
:3 0 1c :3 0 22
:3 0 22 :3 0 b0
:3 0 22e :3 0 b0
:3 0 22f :3 0 31
:3 0 32 :3 0 31
:3 0 32 :3 0 31
:3 0 32 :3 0 21
:3 0 2e :3 0 1e
:3 0 78 :3 0 79
:3 0 58 :3 0 5c
:3 0 5b :3 0 5d
:3 0 7f :3 0 80
:3 0 7b :4 0 233
1 :8 0 279d e0
:4 0 2788 :3 0 f8
:3 0 da :2 0 15d3
278b 278c :3 0 15
:3 0 e1 :3 0 278e
278f 0 3d :2 0
2790 2791 0 2793
15d6 279a 15 :3 0
e1 :3 0 2794 2795
0 e2 :2 0 2796
2797 0 2799 15d8
279b 278d 2793 0
279c 0 2799 0
279c 15da 0 279d
15dd 27f0 e5 :3 0
15 :3 0 e1 :3 0
279f 27a0 0 e4
:3 0 27a1 27a2 0
27a4 15e0 27a6 15e2
27a5 27a4 :2 0 27ee
e6 :3 0 15 :3 0
e1 :3 0 27a9 27aa
0 e4 :3 0 27ab
27ac 0 27e6 c3
:3 0 c4 :3 0 27ae
27af 0 c3 :3 0
d3 :3 0 27b1 27b2
0 e7 :2 0 2
:4 0 236 :2 0 d5
:2 0 2a :2 0 15e4
27b7 27b9 :3 0 15e7
27b3 27bb 27b0 27bd
27bc :2 0 c6 :3 0
d6 :3 0 27bf 27c0
0 c8 :4 0 237
:4 0 cb :2 0 cc
:3 0 15eb 27c4 27c6
:3 0 cb :2 0 ea
:4 0 15ee 27c8 27ca
:3 0 cb :2 0 eb
:4 0 15f1 27cc 27ce
:3 0 cb :2 0 e4
:3 0 15f4 27d0 27d2
:3 0 cb :2 0 ec
:4 0 15f7 27d4 27d6
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 15fa
27d9 27dd 15fe 27d8
27df :3 0 1601 27c1
27e1 :2 0 27e3 1604
27e4 27be 27e3 0
27e5 1606 0 27e6
1608 27e9 :3 0 27e9
0 27e9 27e8 27e6
27e7 :6 0 27eb 67
:3 0 160b 27ed 160d
27ec 27eb :2 0 27ee
160f :2 0 27f0 0
27f0 27ef 279d 27ee
:6 0 27f2 1 :3 0
1612 27f4 2607 26a8
0 27f5 0 27f2
0 27f5 1614 0
27f6 1618 2897 90
:3 0 9a :3 0 98
:3 0 9b :3 0 91
:3 0 228 :3 0 229
:3 0 22a :3 0 22b
:3 0 a1 :3 0 a2
:3 0 15e :3 0 22c
:3 0 22d :3 0 92
:3 0 93 :3 0 94
:3 0 a5 :3 0 a6
:3 0 a7 :3 0 1d4
:3 0 a8 :3 0 1d5
:3 0 fa :3 0 fb
:3 0 179 :3 0 1e
:3 0 1c :3 0 22
:3 0 22 :3 0 b0
:3 0 22e :3 0 b0
:3 0 22f :3 0 31
:3 0 32 :3 0 31
:3 0 32 :3 0 31
:3 0 32 :3 0 21
:3 0 2e :3 0 1e
:3 0 78 :3 0 79
:3 0 58 :3 0 5c
:3 0 5b :3 0 5d
:3 0 7f :3 0 80
:3 0 7b :4 0 238
1 :8 0 2841 e0
:4 0 282c :3 0 f8
:3 0 da :2 0 161c
282f 2830 :3 0 15
:3 0 e1 :3 0 2832
2833 0 3d :2 0
2834 2835 0 2837
161f 283e 15 :3 0
e1 :3 0 2838 2839
0 e2 :2 0 283a
283b 0 283d 1621
283f 2831 2837 0
2840 0 283d 0
2840 1623 0 2841
1626 2894 e5 :3 0
15 :3 0 e1 :3 0
2843 2844 0 e4
:3 0 2845 2846 0
2848 1629 284a 162b
2849 2848 :2 0 2892
e6 :3 0 15 :3 0
e1 :3 0 284d 284e
0 e4 :3 0 284f
2850 0 288a c3
:3 0 c4 :3 0 2852
2853 0 c3 :3 0
d3 :3 0 2855 2856
0 e7 :2 0 2
:4 0 239 :2 0 d5
:2 0 2a :2 0 162d
285b 285d :3 0 1630
2857 285f 2854 2861
2860 :2 0 c6 :3 0
d6 :3 0 2863 2864
0 c8 :4 0 23a
:4 0 cb :2 0 cc
:3 0 1634 2868 286a
:3 0 cb :2 0 ea
:4 0 1637 286c 286e
:3 0 cb :2 0 eb
:4 0 163a 2870 2872
:3 0 cb :2 0 e4
:3 0 163d 2874 2876
:3 0 cb :2 0 ec
:4 0 1640 2878 287a
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 1643
287d 2881 1647 287c
2883 :3 0 164a 2865
2885 :2 0 2887 164d
2888 2862 2887 0
2889 164f 0 288a
1651 288d :3 0 288d
0 288d 288c 288a
288b :6 0 288f 69
:3 0 1654 2891 1656
2890 288f :2 0 2892
1658 :2 0 2894 0
2894 2893 2841 2892
:6 0 2896 1 :3 0
165b 2898 25f1 27f6
0 2899 0 2896
0 2899 165d 0
289d 56 :3 0 2a
:2 0 289a 289b 0
289d 1660 289e 25c0
289d 0 289f 1666
0 4e8b 2f :3 0
a0 :3 0 2f :3 0
cb :2 0 23b :4 0
1668 28a3 28a5 :3 0
2a :2 0 30 :2 0
166b 28a1 28a9 28a0
28aa 0 4e8b 1d
:3 0 de :4 0 28ac
28ad 0 4e8b 3f
:3 0 9 :3 0 28af
28b0 0 4e8b 17f
:3 0 39 :3 0 12f
:3 0 130 :3 0 23
:4 0 180 1 :8 0
28bd 15 :3 0 e1
:3 0 28b8 28b9 0
e4 :3 0 28ba 28bb
0 28bd 166f 2910
e5 :3 0 15 :3 0
e1 :3 0 28bf 28c0
0 e4 :3 0 28c1
28c2 0 28c4 1672
28c6 1674 28c5 28c4
:2 0 290e e6 :3 0
15 :3 0 e1 :3 0
28c9 28ca 0 e4
:3 0 28cb 28cc 0
2906 c3 :3 0 c4
:3 0 28ce 28cf 0
c3 :3 0 d3 :3 0
28d1 28d2 0 e7
:2 0 2 :4 0 23c
:2 0 d5 :2 0 2a
:2 0 1676 28d7 28d9
:3 0 1679 28d3 28db
28d0 28dd 28dc :2 0
c6 :3 0 d6 :3 0
28df 28e0 0 c8
:4 0 23d :4 0 cb
:2 0 cc :3 0 167d
28e4 28e6 :3 0 cb
:2 0 ea :4 0 1680
28e8 28ea :3 0 cb
:2 0 eb :4 0 1683
28ec 28ee :3 0 cb
:2 0 e4 :3 0 1686
28f0 28f2 :3 0 cb
:2 0 ec :4 0 1689
28f4 28f6 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 168c 28f9 28fd
1690 28f8 28ff :3 0
1693 28e1 2901 :2 0
2903 1696 2904 28de
2903 0 2905 1698
0 2906 169a 2909
:3 0 2909 0 2909
2908 2906 2907 :6 0
290b 6b :3 0 169d
290d 169f 290c 290b
:2 0 290e 16a1 :2 0
2910 0 2910 290f
28bd 290e :6 0 4e8b
1 :3 0 74 :3 0
5a :2 0 2912 2913
0 4e8b 75 :3 0
5a :2 0 2915 2916
0 4e8b 43 :3 0
3d :2 0 2918 2919
0 4e8b 82 :3 0
da :2 0 3d :2 0
16a6 291c 291e :3 0
291f :2 0 84 :3 0
77 :3 0 3d :2 0
16a9 2921 2924 da
:2 0 2a :2 0 16ae
2926 2928 :3 0 2929
:2 0 2920 292b 292a
:2 0 23e :3 0 99
:3 0 a1 :3 0 24
:3 0 ae :3 0 a4
:3 0 90 :3 0 99
:3 0 99 :3 0 91
:3 0 22 :3 0 99
:3 0 92 :3 0 23
:3 0 99 :3 0 9b
:3 0 a4 :3 0 8c
:3 0 a4 :3 0 f9
:3 0 b0 :3 0 b1
:3 0 b0 :3 0 b2
:3 0 b0 :3 0 b3
:3 0 b0 :3 0 b4
:3 0 b0 :3 0 b5
:3 0 b0 :3 0 b6
:3 0 b0 :3 0 b7
:3 0 b0 :3 0 b8
:3 0 b0 :3 0 b9
:3 0 b0 :3 0 ba
:3 0 b0 :3 0 bb
:3 0 b0 :3 0 bc
:3 0 b0 :3 0 bd
:3 0 b0 :3 0 be
:3 0 b0 :3 0 bf
:3 0 a4 :3 0 c0
:3 0 2f :4 0 23f
1 :8 0 2968 15
:3 0 e1 :3 0 2963
2964 0 e4 :3 0
2965 2966 0 2968
16b1 29bb e5 :3 0
15 :3 0 e1 :3 0
296a 296b 0 e4
:3 0 296c 296d 0
296f 16b4 2971 16b6
2970 296f :2 0 29b9
e6 :3 0 15 :3 0
e1 :3 0 2974 2975
0 e4 :3 0 2976
2977 0 29b1 c3
:3 0 c4 :3 0 2979
297a 0 c3 :3 0
d3 :3 0 297c 297d
0 e7 :2 0 2
:4 0 240 :2 0 d5
:2 0 2a :2 0 16b8
2982 2984 :3 0 16bb
297e 2986 297b 2988
2987 :2 0 c6 :3 0
d6 :3 0 298a 298b
0 c8 :4 0 241
:4 0 cb :2 0 cc
:3 0 16bf 298f 2991
:3 0 cb :2 0 ea
:4 0 16c2 2993 2995
:3 0 cb :2 0 eb
:4 0 16c5 2997 2999
:3 0 cb :2 0 e4
:3 0 16c8 299b 299d
:3 0 cb :2 0 ec
:4 0 16cb 299f 29a1
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 16ce
29a4 29a8 16d2 29a3
29aa :3 0 16d5 298c
29ac :2 0 29ae 16d8
29af 2989 29ae 0
29b0 16da 0 29b1
16dc 29b4 :3 0 29b4
0 29b4 29b3 29b1
29b2 :6 0 29b6 6d
:3 0 16df 29b8 16e1
29b7 29b6 :2 0 29b9
16e3 :2 0 29bb 0
29bb 29ba 2968 29b9
:6 0 29bd 1 :3 0
16e6 29be 292c 29bd
0 29bf 16e8 0
4e8b 24 :3 0 31
:3 0 29c0 29c1 0
4e8b df :3 0 96
:4 0 29c6 :2 0 4e8b
29c4 29c7 :3 0 242
:3 0 96 :3 0 1b
:3 0 1d :3 0 20
:3 0 41 :3 0 3f
:3 0 7e :3 0 72
:3 0 73 :3 0 27
:3 0 28 :3 0 49
:3 0 1f :3 0 78
:3 0 79 :3 0 74
:3 0 75 :3 0 44
:3 0 46 :3 0 47
:3 0 48 :3 0 4e
:4 0 29e0 :2 0 41b1
29c9 29e1 :3 0 16ea
:3 0 96 :3 0 e0
:3 0 29e2 29e3 :3 0
15 :3 0 e1 :3 0
29e5 29e6 0 3d
:2 0 29e7 29e8 0
29ea 1700 29f1 15
:3 0 e1 :3 0 29eb
29ec 0 e2 :2 0
29ed 29ee 0 29f0
1702 29f2 29e4 29ea
0 29f3 0 29f0
0 29f3 1704 0
41b1 15 :3 0 e1
:3 0 29f4 29f5 0
15 :3 0 13c :2 0
158 :3 0 29f7 29f9
0 1709 29f8 29fb
:3 0 29fc :2 0 243
:8 0 2a00 170c 2a01
29fd 2a00 0 2a02
170e 0 41b1 84
:3 0 3f :3 0 3d
:2 0 1710 2a03 2a06
da :2 0 3d :2 0
1715 2a08 2a0a :3 0
71 :3 0 244 :3 0
1b :3 0 3d :2 0
1718 2a0d 2a10 2a0c
2a11 0 2ca9 af
:3 0 98 :3 0 1b
:4 0 245 1 :8 0
2a2c e0 :4 0 2a17
:3 0 f8 :3 0 da
:2 0 171d 2a1a 2a1b
:3 0 15 :3 0 e1
:3 0 2a1d 2a1e 0
3d :2 0 2a1f 2a20
0 2a22 1720 2a29
15 :3 0 e1 :3 0
2a23 2a24 0 e2
:2 0 2a25 2a26 0
2a28 1722 2a2a 2a1c
2a22 0 2a2b 0
2a28 0 2a2b 1724
0 2a2c 1727 2a7f
e5 :3 0 15 :3 0
e1 :3 0 2a2e 2a2f
0 e4 :3 0 2a30
2a31 0 2a33 172a
2a35 172c 2a34 2a33
:2 0 2a7d e6 :3 0
15 :3 0 e1 :3 0
2a38 2a39 0 e4
:3 0 2a3a 2a3b 0
2a75 c3 :3 0 c4
:3 0 2a3d 2a3e 0
c3 :3 0 d3 :3 0
2a40 2a41 0 e7
:2 0 2 :4 0 246
:2 0 d5 :2 0 2a
:2 0 172e 2a46 2a48
:3 0 1731 2a42 2a4a
2a3f 2a4c 2a4b :2 0
c6 :3 0 d6 :3 0
2a4e 2a4f 0 c8
:4 0 247 :4 0 cb
:2 0 cc :3 0 1735
2a53 2a55 :3 0 cb
:2 0 ea :4 0 1738
2a57 2a59 :3 0 cb
:2 0 eb :4 0 173b
2a5b 2a5d :3 0 cb
:2 0 e4 :3 0 173e
2a5f 2a61 :3 0 cb
:2 0 ec :4 0 1741
2a63 2a65 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 1744 2a68 2a6c
1748 2a67 2a6e :3 0
174b 2a50 2a70 :2 0
2a72 174e 2a73 2a4d
2a72 0 2a74 1750
0 2a75 1752 2a78
:3 0 2a78 0 2a78
2a77 2a75 2a76 :6 0
2a7a 70 :3 0 1755
2a7c 1757 2a7b 2a7a
:2 0 2a7d 1759 :2 0
2a7f 0 2a7f 2a7e
2a2c 2a7d :6 0 2ca9
6f :3 0 90 :3 0
9a :3 0 1d :4 0
248 1 :8 0 2a9a
e0 :4 0 2a85 :3 0
f8 :3 0 da :2 0
175e 2a88 2a89 :3 0
15 :3 0 e1 :3 0
2a8b 2a8c 0 3d
:2 0 2a8d 2a8e 0
2a90 1761 2a97 15
:3 0 e1 :3 0 2a91
2a92 0 e2 :2 0
2a93 2a94 0 2a96
1763 2a98 2a8a 2a90
0 2a99 0 2a96
0 2a99 1765 0
2a9a 1768 2aed e5
:3 0 15 :3 0 e1
:3 0 2a9c 2a9d 0
e4 :3 0 2a9e 2a9f
0 2aa1 176b 2aa3
176d 2aa2 2aa1 :2 0
2aeb e6 :3 0 15
:3 0 e1 :3 0 2aa6
2aa7 0 e4 :3 0
2aa8 2aa9 0 2ae3
c3 :3 0 c4 :3 0
2aab 2aac 0 c3
:3 0 d3 :3 0 2aae
2aaf 0 e7 :2 0
2 :4 0 249 :2 0
d5 :2 0 2a :2 0
176f 2ab4 2ab6 :3 0
1772 2ab0 2ab8 2aad
2aba 2ab9 :2 0 c6
:3 0 d6 :3 0 2abc
2abd 0 c8 :4 0
24a :4 0 cb :2 0
cc :3 0 1776 2ac1
2ac3 :3 0 cb :2 0
ea :4 0 1779 2ac5
2ac7 :3 0 cb :2 0
eb :4 0 177c 2ac9
2acb :3 0 cb :2 0
e4 :3 0 177f 2acd
2acf :3 0 cb :2 0
ec :4 0 1782 2ad1
2ad3 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
1785 2ad6 2ada 1789
2ad5 2adc :3 0 178c
2abe 2ade :2 0 2ae0
178f 2ae1 2abb 2ae0
0 2ae2 1791 0
2ae3 1793 2ae6 :3 0
2ae6 0 2ae6 2ae5
2ae3 2ae4 :6 0 2ae8
72 :3 0 1796 2aea
1798 2ae9 2ae8 :2 0
2aeb 179a :2 0 2aed
0 2aed 2aec 2a9a
2aeb :6 0 2ca9 6f
:3 0 24b :3 0 9a
:3 0 1d :4 0 24c
1 :8 0 2b08 e0
:4 0 2af3 :3 0 f8
:3 0 da :2 0 179f
2af6 2af7 :3 0 15
:3 0 e1 :3 0 2af9
2afa 0 3d :2 0
2afb 2afc 0 2afe
17a2 2b05 15 :3 0
e1 :3 0 2aff 2b00
0 e2 :2 0 2b01
2b02 0 2b04 17a4
2b06 2af8 2afe 0
2b07 0 2b04 0
2b07 17a6 0 2b08
17a9 2b5b e5 :3 0
15 :3 0 e1 :3 0
2b0a 2b0b 0 e4
:3 0 2b0c 2b0d 0
2b0f 17ac 2b11 17ae
2b10 2b0f :2 0 2b59
e6 :3 0 15 :3 0
e1 :3 0 2b14 2b15
0 e4 :3 0 2b16
2b17 0 2b51 c3
:3 0 c4 :3 0 2b19
2b1a 0 c3 :3 0
d3 :3 0 2b1c 2b1d
0 e7 :2 0 2
:4 0 24d :2 0 d5
:2 0 2a :2 0 17b0
2b22 2b24 :3 0 17b3
2b1e 2b26 2b1b 2b28
2b27 :2 0 c6 :3 0
d6 :3 0 2b2a 2b2b
0 c8 :4 0 24e
:4 0 cb :2 0 cc
:3 0 17b7 2b2f 2b31
:3 0 cb :2 0 ea
:4 0 17ba 2b33 2b35
:3 0 cb :2 0 eb
:4 0 17bd 2b37 2b39
:3 0 cb :2 0 e4
:3 0 17c0 2b3b 2b3d
:3 0 cb :2 0 ec
:4 0 17c3 2b3f 2b41
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 17c6
2b44 2b48 17ca 2b43
2b4a :3 0 17cd 2b2c
2b4c :2 0 2b4e 17d0
2b4f 2b29 2b4e 0
2b50 17d2 0 2b51
17d4 2b54 :3 0 2b54
0 2b54 2b53 2b51
2b52 :6 0 2b56 74
:3 0 17d7 2b58 17d9
2b57 2b56 :2 0 2b59
17db :2 0 2b5b 0
2b5b 2b5a 2b08 2b59
:6 0 2ca9 6f :3 0
1b2 :3 0 91 :3 0
22 :3 0 9a :3 0
1d :4 0 24f 1
:8 0 2b78 e0 :4 0
2b63 :3 0 f8 :3 0
da :2 0 17e0 2b66
2b67 :3 0 15 :3 0
e1 :3 0 2b69 2b6a
0 3d :2 0 2b6b
2b6c 0 2b6e 17e3
2b75 15 :3 0 e1
:3 0 2b6f 2b70 0
e2 :2 0 2b71 2b72
0 2b74 17e5 2b76
2b68 2b6e 0 2b77
0 2b74 0 2b77
17e7 0 2b78 17ea
2bcb e5 :3 0 15
:3 0 e1 :3 0 2b7a
2b7b 0 e4 :3 0
2b7c 2b7d 0 2b7f
17ed 2b81 17ef 2b80
2b7f :2 0 2bc9 e6
:3 0 15 :3 0 e1
:3 0 2b84 2b85 0
e4 :3 0 2b86 2b87
0 2bc1 c3 :3 0
c4 :3 0 2b89 2b8a
0 c3 :3 0 d3
:3 0 2b8c 2b8d 0
e7 :2 0 2 :4 0
250 :2 0 d5 :2 0
2a :2 0 17f1 2b92
2b94 :3 0 17f4 2b8e
2b96 2b8b 2b98 2b97
:2 0 c6 :3 0 d6
:3 0 2b9a 2b9b 0
c8 :4 0 251 :4 0
cb :2 0 cc :3 0
17f8 2b9f 2ba1 :3 0
cb :2 0 ea :4 0
17fb 2ba3 2ba5 :3 0
cb :2 0 eb :4 0
17fe 2ba7 2ba9 :3 0
cb :2 0 e4 :3 0
1801 2bab 2bad :3 0
cb :2 0 ec :4 0
1804 2baf 2bb1 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 1807 2bb4
2bb8 180b 2bb3 2bba
:3 0 180e 2b9c 2bbc
:2 0 2bbe 1811 2bbf
2b99 2bbe 0 2bc0
1813 0 2bc1 1815
2bc4 :3 0 2bc4 0
2bc4 2bc3 2bc1 2bc2
:6 0 2bc6 76 :3 0
1818 2bc8 181a 2bc7
2bc6 :2 0 2bc9 181c
:2 0 2bcb 0 2bcb
2bca 2b78 2bc9 :6 0
2ca9 6f :3 0 252
:3 0 98 :3 0 1d
:4 0 253 1 :8 0
2be6 e0 :4 0 2bd1
:3 0 f8 :3 0 da
:2 0 1821 2bd4 2bd5
:3 0 15 :3 0 e1
:3 0 2bd7 2bd8 0
3d :2 0 2bd9 2bda
0 2bdc 1824 2be3
15 :3 0 e1 :3 0
2bdd 2bde 0 e2
:2 0 2bdf 2be0 0
2be2 1826 2be4 2bd6
2bdc 0 2be5 0
2be2 0 2be5 1828
0 2be6 182b 2c39
e5 :3 0 15 :3 0
e1 :3 0 2be8 2be9
0 e4 :3 0 2bea
2beb 0 2bed 182e
2bef 1830 2bee 2bed
:2 0 2c37 e6 :3 0
15 :3 0 e1 :3 0
2bf2 2bf3 0 e4
:3 0 2bf4 2bf5 0
2c2f c3 :3 0 c4
:3 0 2bf7 2bf8 0
c3 :3 0 d3 :3 0
2bfa 2bfb 0 e7
:2 0 2 :4 0 254
:2 0 d5 :2 0 2a
:2 0 1832 2c00 2c02
:3 0 1835 2bfc 2c04
2bf9 2c06 2c05 :2 0
c6 :3 0 d6 :3 0
2c08 2c09 0 c8
:4 0 255 :4 0 cb
:2 0 cc :3 0 1839
2c0d 2c0f :3 0 cb
:2 0 ea :4 0 183c
2c11 2c13 :3 0 cb
:2 0 eb :4 0 183f
2c15 2c17 :3 0 cb
:2 0 e4 :3 0 1842
2c19 2c1b :3 0 cb
:2 0 ec :4 0 1845
2c1d 2c1f :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 1848 2c22 2c26
184c 2c21 2c28 :3 0
184f 2c0a 2c2a :2 0
2c2c 1852 2c2d 2c07
2c2c 0 2c2e 1854
0 2c2f 1856 2c32
:3 0 2c32 0 2c32
2c31 2c2f 2c30 :6 0
2c34 78 :3 0 1859
2c36 185b 2c35 2c34
:2 0 2c37 185d :2 0
2c39 0 2c39 2c38
2be6 2c37 :6 0 2ca9
6f :3 0 1ef :3 0
98 :3 0 1d :4 0
256 1 :8 0 2c54
e0 :4 0 2c3f :3 0
f8 :3 0 da :2 0
1862 2c42 2c43 :3 0
15 :3 0 e1 :3 0
2c45 2c46 0 3d
:2 0 2c47 2c48 0
2c4a 1865 2c51 15
:3 0 e1 :3 0 2c4b
2c4c 0 e2 :2 0
2c4d 2c4e 0 2c50
1867 2c52 2c44 2c4a
0 2c53 0 2c50
0 2c53 1869 0
2c54 186c 2ca7 e5
:3 0 15 :3 0 e1
:3 0 2c56 2c57 0
e4 :3 0 2c58 2c59
0 2c5b 186f 2c5d
1871 2c5c 2c5b :2 0
2ca5 e6 :3 0 15
:3 0 e1 :3 0 2c60
2c61 0 e4 :3 0
2c62 2c63 0 2c9d
c3 :3 0 c4 :3 0
2c65 2c66 0 c3
:3 0 d3 :3 0 2c68
2c69 0 e7 :2 0
2 :4 0 257 :2 0
d5 :2 0 2a :2 0
1873 2c6e 2c70 :3 0
1876 2c6a 2c72 2c67
2c74 2c73 :2 0 c6
:3 0 d6 :3 0 2c76
2c77 0 c8 :4 0
258 :4 0 cb :2 0
cc :3 0 187a 2c7b
2c7d :3 0 cb :2 0
ea :4 0 187d 2c7f
2c81 :3 0 cb :2 0
eb :4 0 1880 2c83
2c85 :3 0 cb :2 0
e4 :3 0 1883 2c87
2c89 :3 0 cb :2 0
ec :4 0 1886 2c8b
2c8d :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
1889 2c90 2c94 188d
2c8f 2c96 :3 0 1890
2c78 2c98 :2 0 2c9a
1893 2c9b 2c75 2c9a
0 2c9c 1895 0
2c9d 1897 2ca0 :3 0
2ca0 0 2ca0 2c9f
2c9d 2c9e :6 0 2ca2
7a :3 0 189a 2ca4
189c 2ca3 2ca2 :2 0
2ca5 189e :2 0 2ca7
0 2ca7 2ca6 2c54
2ca5 :6 0 2ca9 6f
:3 0 18a1 41a4 259
:3 0 92 :3 0 21
:3 0 9a :3 0 1d
:3 0 91 :3 0 22
:3 0 92 :3 0 23
:4 0 25a 1 :8 0
2cc9 e0 :4 0 2cb4
:3 0 f8 :3 0 da
:2 0 18ab 2cb7 2cb8
:3 0 15 :3 0 e1
:3 0 2cba 2cbb 0
3d :2 0 2cbc 2cbd
0 2cbf 18ae 2cc6
15 :3 0 e1 :3 0
2cc0 2cc1 0 e2
:2 0 2cc2 2cc3 0
2cc5 18b0 2cc7 2cb9
2cbf 0 2cc8 0
2cc5 0 2cc8 18b2
0 2cc9 18b5 2d1c
e5 :3 0 15 :3 0
e1 :3 0 2ccb 2ccc
0 e4 :3 0 2ccd
2cce 0 2cd0 18b8
2cd2 18ba 2cd1 2cd0
:2 0 2d1a e6 :3 0
15 :3 0 e1 :3 0
2cd5 2cd6 0 e4
:3 0 2cd7 2cd8 0
2d12 c3 :3 0 c4
:3 0 2cda 2cdb 0
c3 :3 0 d3 :3 0
2cdd 2cde 0 e7
:2 0 2 :4 0 25b
:2 0 d5 :2 0 2a
:2 0 18bc 2ce3 2ce5
:3 0 18bf 2cdf 2ce7
2cdc 2ce9 2ce8 :2 0
c6 :3 0 d6 :3 0
2ceb 2cec 0 c8
:4 0 25c :4 0 cb
:2 0 cc :3 0 18c3
2cf0 2cf2 :3 0 cb
:2 0 ea :4 0 18c6
2cf4 2cf6 :3 0 cb
:2 0 eb :4 0 18c9
2cf8 2cfa :3 0 cb
:2 0 e4 :3 0 18cc
2cfc 2cfe :3 0 cb
:2 0 ec :4 0 18cf
2d00 2d02 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 18d2 2d05 2d09
18d6 2d04 2d0b :3 0
18d9 2ced 2d0d :2 0
2d0f 18dc 2d10 2cea
2d0f 0 2d11 18de
0 2d12 18e0 2d15
:3 0 2d15 0 2d15
2d14 2d12 2d13 :6 0
2d17 7c :3 0 18e3
2d19 18e5 2d18 2d17
:2 0 2d1a 18e7 :2 0
2d1c 0 2d1c 2d1b
2cc9 2d1a :6 0 41a3
6f :3 0 9a :3 0
17 :3 0 90 :3 0
91 :3 0 22 :3 0
9b :3 0 20 :3 0
92 :3 0 21 :3 0
93 :4 0 25d 1
:8 0 2d2e 15 :3 0
e1 :3 0 2d29 2d2a
0 e4 :3 0 2d2b
2d2c 0 2d2e 18ea
2d81 e5 :3 0 15
:3 0 e1 :3 0 2d30
2d31 0 e4 :3 0
2d32 2d33 0 2d35
18ed 2d37 18ef 2d36
2d35 :2 0 2d7f e6
:3 0 15 :3 0 e1
:3 0 2d3a 2d3b 0
e4 :3 0 2d3c 2d3d
0 2d77 c3 :3 0
c4 :3 0 2d3f 2d40
0 c3 :3 0 d3
:3 0 2d42 2d43 0
e7 :2 0 2 :4 0
25e :2 0 d5 :2 0
2a :2 0 18f1 2d48
2d4a :3 0 18f4 2d44
2d4c 2d41 2d4e 2d4d
:2 0 c6 :3 0 d6
:3 0 2d50 2d51 0
c8 :4 0 25f :4 0
cb :2 0 cc :3 0
18f8 2d55 2d57 :3 0
cb :2 0 ea :4 0
18fb 2d59 2d5b :3 0
cb :2 0 eb :4 0
18fe 2d5d 2d5f :3 0
cb :2 0 e4 :3 0
1901 2d61 2d63 :3 0
cb :2 0 ec :4 0
1904 2d65 2d67 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 1907 2d6a
2d6e 190b 2d69 2d70
:3 0 190e 2d52 2d72
:2 0 2d74 1911 2d75
2d4f 2d74 0 2d76
1913 0 2d77 1915
2d7a :3 0 2d7a 0
2d7a 2d79 2d77 2d78
:6 0 2d7c 7e :3 0
1918 2d7e 191a 2d7d
2d7c :2 0 2d7f 191c
:2 0 2d81 0 2d81
2d80 2d2e 2d7f :6 0
41a3 6f :3 0 15
:3 0 e1 :3 0 2d83
2d84 0 15 :3 0
da :2 0 101 :3 0
2d86 2d88 0 1921
2d87 2d8a :3 0 2d8b
:2 0 84 :3 0 4c
:3 0 3d :2 0 1924
2d8d 2d90 da :2 0
3d :2 0 1929 2d92
2d94 :3 0 1e :3 0
1d :3 0 2d96 2d97
0 2d99 192c 2d9a
2d95 2d99 0 2d9b
192e 0 41a0 98
:3 0 16e :3 0 178
:3 0 17 :3 0 3c
:3 0 19 :3 0 af
:3 0 8c :3 0 20
:3 0 92 :3 0 21
:3 0 93 :4 0 260
1 :8 0 2dae 15
:3 0 e1 :3 0 2da9
2daa 0 e4 :3 0
2dab 2dac 0 2dae
1930 2e01 e5 :3 0
15 :3 0 e1 :3 0
2db0 2db1 0 e4
:3 0 2db2 2db3 0
2db5 1933 2db7 1935
2db6 2db5 :2 0 2dff
e6 :3 0 15 :3 0
e1 :3 0 2dba 2dbb
0 e4 :3 0 2dbc
2dbd 0 2df7 c3
:3 0 c4 :3 0 2dbf
2dc0 0 c3 :3 0
d3 :3 0 2dc2 2dc3
0 e7 :2 0 2
:4 0 261 :2 0 d5
:2 0 2a :2 0 1937
2dc8 2dca :3 0 193a
2dc4 2dcc 2dc1 2dce
2dcd :2 0 c6 :3 0
d6 :3 0 2dd0 2dd1
0 c8 :4 0 262
:4 0 cb :2 0 cc
:3 0 193e 2dd5 2dd7
:3 0 cb :2 0 ea
:4 0 1941 2dd9 2ddb
:3 0 cb :2 0 eb
:4 0 1944 2ddd 2ddf
:3 0 cb :2 0 e4
:3 0 1947 2de1 2de3
:3 0 cb :2 0 ec
:4 0 194a 2de5 2de7
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 194d
2dea 2dee 1951 2de9
2df0 :3 0 1954 2dd2
2df2 :2 0 2df4 1957
2df5 2dcf 2df4 0
2df6 1959 0 2df7
195b 2dfa :3 0 2dfa
0 2dfa 2df9 2df7
2df8 :6 0 2dfc 80
:3 0 195e 2dfe 1960
2dfd 2dfc :2 0 2dff
1962 :2 0 2e01 0
2e01 2e00 2dae 2dff
:6 0 41a0 6f :3 0
15 :3 0 e1 :3 0
2e03 2e04 0 15
:3 0 da :2 0 158
:3 0 2e06 2e08 0
1967 2e07 2e0a :3 0
2e0b :2 0 d8 :3 0
84 :3 0 3c :3 0
3d :2 0 196a 2e0e
2e11 b0 :3 0 19f
:3 0 2e13 2e14 0
196d 2e0d 2e16 2e17
:2 0 b0 :3 0 da
:2 0 19f :3 0 2e19
2e1b 0 1972 2e1a
2e1d :3 0 3f :3 0
3d :2 0 2e1f 2e20
0 2e96 af :3 0
130 :3 0 23 :3 0
92 :3 0 21 :3 0
93 :3 0 98 :3 0
1b :4 0 263 1
:8 0 2e40 e0 :4 0
2e2b :3 0 f8 :3 0
da :2 0 1977 2e2e
2e2f :3 0 15 :3 0
e1 :3 0 2e31 2e32
0 3d :2 0 2e33
2e34 0 2e36 197a
2e3d 15 :3 0 e1
:3 0 2e37 2e38 0
e2 :2 0 2e39 2e3a
0 2e3c 197c 2e3e
2e30 2e36 0 2e3f
0 2e3c 0 2e3f
197e 0 2e40 1981
2e93 e5 :3 0 15
:3 0 e1 :3 0 2e42
2e43 0 e4 :3 0
2e44 2e45 0 2e47
1984 2e49 1986 2e48
2e47 :2 0 2e91 e6
:3 0 15 :3 0 e1
:3 0 2e4c 2e4d 0
e4 :3 0 2e4e 2e4f
0 2e89 c3 :3 0
c4 :3 0 2e51 2e52
0 c3 :3 0 d3
:3 0 2e54 2e55 0
e7 :2 0 2 :4 0
264 :2 0 d5 :2 0
2a :2 0 1988 2e5a
2e5c :3 0 198b 2e56
2e5e 2e53 2e60 2e5f
:2 0 c6 :3 0 d6
:3 0 2e62 2e63 0
c8 :4 0 265 :4 0
cb :2 0 cc :3 0
198f 2e67 2e69 :3 0
cb :2 0 ea :4 0
1992 2e6b 2e6d :3 0
cb :2 0 eb :4 0
1995 2e6f 2e71 :3 0
cb :2 0 e4 :3 0
1998 2e73 2e75 :3 0
cb :2 0 ec :4 0
199b 2e77 2e79 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 199e 2e7c
2e80 19a2 2e7b 2e82
:3 0 19a5 2e64 2e84
:2 0 2e86 19a8 2e87
2e61 2e86 0 2e88
19aa 0 2e89 19ac
2e8c :3 0 2e8c 0
2e8c 2e8b 2e89 2e8a
:6 0 2e8e 82 :3 0
19af 2e90 19b1 2e8f
2e8e :2 0 2e91 19b3
:2 0 2e93 0 2e93
2e92 2e40 2e91 :6 0
2e96 6f :3 0 141
:3 0 19b6 2fa7 84
:3 0 3c :3 0 3d
:2 0 19b9 2e97 2e9a
b0 :3 0 da :2 0
195 :3 0 2e9c 2e9e
0 19be 2e9d 2ea0
:3 0 71 :3 0 173
:3 0 3d :2 0 84
:3 0 41 :3 0 3d
:2 0 19c1 2ea5 2ea8
b0 :3 0 174 :3 0
2eaa 2eab 0 6e
:3 0 19c4 2ea3 2eae
2ea2 2eaf 0 2f29
af :3 0 16e :3 0
b0 :3 0 19f :3 0
178 :3 0 6e :3 0
130 :3 0 23 :3 0
92 :3 0 21 :3 0
93 :3 0 98 :3 0
1b :4 0 266 1
:8 0 2ed4 e0 :4 0
2ebf :3 0 f8 :3 0
da :2 0 19cb 2ec2
2ec3 :3 0 15 :3 0
e1 :3 0 2ec5 2ec6
0 3d :2 0 2ec7
2ec8 0 2eca 19ce
2ed1 15 :3 0 e1
:3 0 2ecb 2ecc 0
e2 :2 0 2ecd 2ece
0 2ed0 19d0 2ed2
2ec4 2eca 0 2ed3
0 2ed0 0 2ed3
19d2 0 2ed4 19d5
2f27 e5 :3 0 15
:3 0 e1 :3 0 2ed6
2ed7 0 e4 :3 0
2ed8 2ed9 0 2edb
19d8 2edd 19da 2edc
2edb :2 0 2f25 e6
:3 0 15 :3 0 e1
:3 0 2ee0 2ee1 0
e4 :3 0 2ee2 2ee3
0 2f1d c3 :3 0
c4 :3 0 2ee5 2ee6
0 c3 :3 0 d3
:3 0 2ee8 2ee9 0
e7 :2 0 2 :4 0
267 :2 0 d5 :2 0
2a :2 0 19dc 2eee
2ef0 :3 0 19df 2eea
2ef2 2ee7 2ef4 2ef3
:2 0 c6 :3 0 d6
:3 0 2ef6 2ef7 0
c8 :4 0 268 :4 0
cb :2 0 cc :3 0
19e3 2efb 2efd :3 0
cb :2 0 ea :4 0
19e6 2eff 2f01 :3 0
cb :2 0 eb :4 0
19e9 2f03 2f05 :3 0
cb :2 0 e4 :3 0
19ec 2f07 2f09 :3 0
cb :2 0 ec :4 0
19ef 2f0b 2f0d :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 19f2 2f10
2f14 19f6 2f0f 2f16
:3 0 19f9 2ef8 2f18
:2 0 2f1a 19fc 2f1b
2ef5 2f1a 0 2f1c
19fe 0 2f1d 1a00
2f20 :3 0 2f20 0
2f20 2f1f 2f1d 2f1e
:6 0 2f22 84 :3 0
1a03 2f24 1a05 2f23
2f22 :2 0 2f25 1a07
:2 0 2f27 0 2f27
2f26 2ed4 2f25 :6 0
2f29 6f :3 0 1a0a
2f2a 2ea1 2f29 0
2fa9 af :3 0 16e
:3 0 19a :3 0 19b
:3 0 3c :3 0 b0
:3 0 19f :3 0 178
:3 0 19 :3 0 130
:3 0 23 :3 0 92
:3 0 21 :3 0 93
:3 0 98 :3 0 1b
:4 0 269 1 :8 0
2f51 e0 :4 0 2f3c
:3 0 f8 :3 0 da
:2 0 1a0f 2f3f 2f40
:3 0 15 :3 0 e1
:3 0 2f42 2f43 0
3d :2 0 2f44 2f45
0 2f47 1a12 2f4e
15 :3 0 e1 :3 0
2f48 2f49 0 e2
:2 0 2f4a 2f4b 0
2f4d 1a14 2f4f 2f41
2f47 0 2f50 0
2f4d 0 2f50 1a16
0 2f51 1a19 2fa4
e5 :3 0 15 :3 0
e1 :3 0 2f53 2f54
0 e4 :3 0 2f55
2f56 0 2f58 1a1c
2f5a 1a1e 2f59 2f58
:2 0 2fa2 e6 :3 0
15 :3 0 e1 :3 0
2f5d 2f5e 0 e4
:3 0 2f5f 2f60 0
2f9a c3 :3 0 c4
:3 0 2f62 2f63 0
c3 :3 0 d3 :3 0
2f65 2f66 0 e7
:2 0 2 :4 0 26a
:2 0 d5 :2 0 2a
:2 0 1a20 2f6b 2f6d
:3 0 1a23 2f67 2f6f
2f64 2f71 2f70 :2 0
c6 :3 0 d6 :3 0
2f73 2f74 0 c8
:4 0 26b :4 0 cb
:2 0 cc :3 0 1a27
2f78 2f7a :3 0 cb
:2 0 ea :4 0 1a2a
2f7c 2f7e :3 0 cb
:2 0 eb :4 0 1a2d
2f80 2f82 :3 0 cb
:2 0 e4 :3 0 1a30
2f84 2f86 :3 0 cb
:2 0 ec :4 0 1a33
2f88 2f8a :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 1a36 2f8d 2f91
1a3a 2f8c 2f93 :3 0
1a3d 2f75 2f95 :2 0
2f97 1a40 2f98 2f72
2f97 0 2f99 1a42
0 2f9a 1a44 2f9d
:3 0 2f9d 0 2f9d
2f9c 2f9a 2f9b :6 0
2f9f 86 :3 0 1a47
2fa1 1a49 2fa0 2f9f
:2 0 2fa2 1a4b :2 0
2fa4 0 2fa4 2fa3
2f51 2fa2 :6 0 2fa6
6f :3 0 1a4e 2fa8
2e1e 2e96 0 2fa9
0 2fa6 0 2fa9
1a50 0 3031 15
:3 0 e1 :3 0 2faa
2fab 0 15 :3 0
da :2 0 158 :3 0
2fad 2faf 0 1a56
2fae 2fb1 :3 0 2fb2
:2 0 84 :3 0 3f
:3 0 3d :2 0 1a59
2fb4 2fb7 10b :2 0
3d :2 0 1a5e 2fb9
2fbb :3 0 2fbc :2 0
2fb3 2fbe 2fbd :2 0
af :3 0 98 :3 0
17 :4 0 1f9 1
:8 0 2fd9 e0 :4 0
2fc4 :3 0 f8 :3 0
da :2 0 1a63 2fc7
2fc8 :3 0 15 :3 0
e1 :3 0 2fca 2fcb
0 3d :2 0 2fcc
2fcd 0 2fcf 1a66
2fd6 15 :3 0 e1
:3 0 2fd0 2fd1 0
e2 :2 0 2fd2 2fd3
0 2fd5 1a68 2fd7
2fc9 2fcf 0 2fd8
0 2fd5 0 2fd8
1a6a 0 2fd9 1a6d
302c e5 :3 0 15
:3 0 e1 :3 0 2fdb
2fdc 0 e4 :3 0
2fdd 2fde 0 2fe0
1a70 2fe2 1a72 2fe1
2fe0 :2 0 302a e6
:3 0 15 :3 0 e1
:3 0 2fe5 2fe6 0
e4 :3 0 2fe7 2fe8
0 3022 c3 :3 0
c4 :3 0 2fea 2feb
0 c3 :3 0 d3
:3 0 2fed 2fee 0
e7 :2 0 2 :4 0
26c :2 0 d5 :2 0
2a :2 0 1a74 2ff3
2ff5 :3 0 1a77 2fef
2ff7 2fec 2ff9 2ff8
:2 0 c6 :3 0 d6
:3 0 2ffb 2ffc 0
c8 :4 0 26d :4 0
cb :2 0 cc :3 0
1a7b 3000 3002 :3 0
cb :2 0 ea :4 0
1a7e 3004 3006 :3 0
cb :2 0 eb :4 0
1a81 3008 300a :3 0
cb :2 0 e4 :3 0
1a84 300c 300e :3 0
cb :2 0 ec :4 0
1a87 3010 3012 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 1a8a 3015
3019 1a8e 3014 301b
:3 0 1a91 2ffd 301d
:2 0 301f 1a94 3020
2ffa 301f 0 3021
1a96 0 3022 1a98
3025 :3 0 3025 0
3025 3024 3022 3023
:6 0 3027 88 :3 0
1a9b 3029 1a9d 3028
3027 :2 0 302a 1a9f
:2 0 302c 0 302c
302b 2fd9 302a :6 0
302e 6f :3 0 1aa2
302f 2fbf 302e 0
3030 1aa4 0 3031
1aa6 3325 26e :3 0
98 :3 0 a0 :3 0
111 :3 0 186 :3 0
18b :3 0 111 :3 0
186 :3 0 17 :3 0
38 :3 0 ae :3 0
111 :3 0 af :3 0
26e :3 0 111 :3 0
8c :3 0 20 :3 0
111 :3 0 8c :3 0
26e :3 0 8c :3 0
26e :3 0 16e :3 0
26e :3 0 92 :3 0
111 :3 0 26f :3 0
9a :3 0 90 :3 0
270 :3 0 270 :3 0
98 :3 0 26e :3 0
98 :4 0 271 1
:8 0 305a 15 :3 0
e1 :3 0 3055 3056
0 e4 :3 0 3057
3058 0 305a 1aa9
30ad e5 :3 0 15
:3 0 e1 :3 0 305c
305d 0 e4 :3 0
305e 305f 0 3061
1aac 3063 1aae 3062
3061 :2 0 30ab e6
:3 0 15 :3 0 e1
:3 0 3066 3067 0
e4 :3 0 3068 3069
0 30a3 c3 :3 0
c4 :3 0 306b 306c
0 c3 :3 0 d3
:3 0 306e 306f 0
e7 :2 0 2 :4 0
272 :2 0 d5 :2 0
2a :2 0 1ab0 3074
3076 :3 0 1ab3 3070
3078 306d 307a 3079
:2 0 c6 :3 0 d6
:3 0 307c 307d 0
c8 :4 0 273 :4 0
cb :2 0 cc :3 0
1ab7 3081 3083 :3 0
cb :2 0 ea :4 0
1aba 3085 3087 :3 0
cb :2 0 eb :4 0
1abd 3089 308b :3 0
cb :2 0 e4 :3 0
1ac0 308d 308f :3 0
cb :2 0 ec :4 0
1ac3 3091 3093 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 1ac6 3096
309a 1aca 3095 309c
:3 0 1acd 307e 309e
:2 0 30a0 1ad0 30a1
307b 30a0 0 30a2
1ad2 0 30a3 1ad4
30a6 :3 0 30a6 0
30a6 30a5 30a3 30a4
:6 0 30a8 8a :3 0
1ad7 30aa 1ad9 30a9
30a8 :2 0 30ab 1adb
:2 0 30ad 0 30ad
30ac 305a 30ab :6 0
3324 6f :3 0 15
:3 0 e1 :3 0 30af
30b0 0 15 :3 0
da :2 0 158 :3 0
30b2 30b4 0 1ae0
30b3 30b6 :3 0 30b7
:2 0 17 :3 0 166
:2 0 1ae3 30ba 30bb
:3 0 30bc :2 0 30bd
:2 0 30b8 30bf 30be
:2 0 71 :3 0 173
:3 0 3d :2 0 84
:3 0 41 :3 0 3d
:2 0 1ae5 30c4 30c7
b0 :3 0 274 :3 0
30c9 30ca 0 6e
:3 0 1ae8 30c2 30cd
30c1 30ce 0 3227
af :3 0 98 :3 0
17 :4 0 1f9 1
:8 0 30e9 e0 :4 0
30d4 :3 0 f8 :3 0
da :2 0 1aef 30d7
30d8 :3 0 15 :3 0
e1 :3 0 30da 30db
0 3d :2 0 30dc
30dd 0 30df 1af2
30e6 15 :3 0 e1
:3 0 30e0 30e1 0
e2 :2 0 30e2 30e3
0 30e5 1af4 30e7
30d9 30df 0 30e8
0 30e5 0 30e8
1af6 0 30e9 1af9
313c e5 :3 0 15
:3 0 e1 :3 0 30eb
30ec 0 e4 :3 0
30ed 30ee 0 30f0
1afc 30f2 1afe 30f1
30f0 :2 0 313a e6
:3 0 15 :3 0 e1
:3 0 30f5 30f6 0
e4 :3 0 30f7 30f8
0 3132 c3 :3 0
c4 :3 0 30fa 30fb
0 c3 :3 0 d3
:3 0 30fd 30fe 0
e7 :2 0 2 :4 0
275 :2 0 d5 :2 0
2a :2 0 1b00 3103
3105 :3 0 1b03 30ff
3107 30fc 3109 3108
:2 0 c6 :3 0 d6
:3 0 310b 310c 0
c8 :4 0 276 :4 0
cb :2 0 cc :3 0
1b07 3110 3112 :3 0
cb :2 0 ea :4 0
1b0a 3114 3116 :3 0
cb :2 0 eb :4 0
1b0d 3118 311a :3 0
cb :2 0 e4 :3 0
1b10 311c 311e :3 0
cb :2 0 ec :4 0
1b13 3120 3122 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 1b16 3125
3129 1b1a 3124 312b
:3 0 1b1d 310d 312d
:2 0 312f 1b20 3130
310a 312f 0 3131
1b22 0 3132 1b24
3135 :3 0 3135 0
3135 3134 3132 3133
:6 0 3137 8c :3 0
1b27 3139 1b29 3138
3137 :2 0 313a 1b2b
:2 0 313c 0 313c
313b 30e9 313a :6 0
3227 6f :3 0 af
:3 0 92 :3 0 21
:3 0 93 :3 0 130
:3 0 23 :3 0 16e
:3 0 178 :3 0 6e
:3 0 98 :3 0 1b
:4 0 277 1 :8 0
315f e0 :4 0 314a
:3 0 f8 :3 0 da
:2 0 1b30 314d 314e
:3 0 15 :3 0 e1
:3 0 3150 3151 0
3d :2 0 3152 3153
0 3155 1b33 315c
15 :3 0 e1 :3 0
3156 3157 0 e2
:2 0 3158 3159 0
315b 1b35 315d 314f
3155 0 315e 0
315b 0 315e 1b37
0 315f 1b3a 31b2
e5 :3 0 15 :3 0
e1 :3 0 3161 3162
0 e4 :3 0 3163
3164 0 3166 1b3d
3168 1b3f 3167 3166
:2 0 31b0 e6 :3 0
15 :3 0 e1 :3 0
316b 316c 0 e4
:3 0 316d 316e 0
31a8 c3 :3 0 c4
:3 0 3170 3171 0
c3 :3 0 d3 :3 0
3173 3174 0 e7
:2 0 2 :4 0 278
:2 0 d5 :2 0 2a
:2 0 1b41 3179 317b
:3 0 1b44 3175 317d
3172 317f 317e :2 0
c6 :3 0 d6 :3 0
3181 3182 0 c8
:4 0 279 :4 0 cb
:2 0 cc :3 0 1b48
3186 3188 :3 0 cb
:2 0 ea :4 0 1b4b
318a 318c :3 0 cb
:2 0 eb :4 0 1b4e
318e 3190 :3 0 cb
:2 0 e4 :3 0 1b51
3192 3194 :3 0 cb
:2 0 ec :4 0 1b54
3196 3198 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 1b57 319b 319f
1b5b 319a 31a1 :3 0
1b5e 3183 31a3 :2 0
31a5 1b61 31a6 3180
31a5 0 31a7 1b63
0 31a8 1b65 31ab
:3 0 31ab 0 31ab
31aa 31a8 31a9 :6 0
31ad 8e :3 0 1b68
31af 1b6a 31ae 31ad
:2 0 31b0 1b6c :2 0
31b2 0 31b2 31b1
315f 31b0 :6 0 3227
6f :3 0 ae :3 0
186 :3 0 187 :3 0
186 :3 0 38 :3 0
36 :3 0 8c :3 0
20 :4 0 27a 1
:8 0 31d2 e0 :4 0
31bd :3 0 f8 :3 0
da :2 0 1b71 31c0
31c1 :3 0 15 :3 0
e1 :3 0 31c3 31c4
0 3d :2 0 31c5
31c6 0 31c8 1b74
31cf 15 :3 0 e1
:3 0 31c9 31ca 0
e2 :2 0 31cb 31cc
0 31ce 1b76 31d0
31c2 31c8 0 31d1
0 31ce 0 31d1
1b78 0 31d2 1b7b
3225 e5 :3 0 15
:3 0 e1 :3 0 31d4
31d5 0 e4 :3 0
31d6 31d7 0 31d9
1b7e 31db 1b80 31da
31d9 :2 0 3223 e6
:3 0 15 :3 0 e1
:3 0 31de 31df 0
e4 :3 0 31e0 31e1
0 321b c3 :3 0
c4 :3 0 31e3 31e4
0 c3 :3 0 d3
:3 0 31e6 31e7 0
e7 :2 0 2 :4 0
27b :2 0 d5 :2 0
2a :2 0 1b82 31ec
31ee :3 0 1b85 31e8
31f0 31e5 31f2 31f1
:2 0 c6 :3 0 d6
:3 0 31f4 31f5 0
c8 :4 0 27c :4 0
cb :2 0 cc :3 0
1b89 31f9 31fb :3 0
cb :2 0 ea :4 0
1b8c 31fd 31ff :3 0
cb :2 0 eb :4 0
1b8f 3201 3203 :3 0
cb :2 0 e4 :3 0
1b92 3205 3207 :3 0
cb :2 0 ec :4 0
1b95 3209 320b :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 1b98 320e
3212 1b9c 320d 3214
:3 0 1b9f 31f6 3216
:2 0 3218 1ba2 3219
31f3 3218 0 321a
1ba4 0 321b 1ba6
321e :3 0 321e 0
321e 321d 321b 321c
:6 0 3220 90 :3 0
1ba9 3222 1bab 3221
3220 :2 0 3223 1bad
:2 0 3225 0 3225
3224 31d2 3223 :6 0
3227 6f :3 0 1bb0
3321 71 :3 0 173
:3 0 3d :2 0 84
:3 0 41 :3 0 3d
:2 0 1bb5 322b 322e
b0 :3 0 174 :3 0
3230 3231 0 6e
:3 0 1bb8 3229 3234
3228 3235 0 3320
ae :3 0 186 :3 0
187 :3 0 186 :3 0
39 :3 0 36 :3 0
8c :3 0 20 :4 0
27d 1 :8 0 3255
e0 :4 0 3240 :3 0
f8 :3 0 da :2 0
1bbf 3243 3244 :3 0
15 :3 0 e1 :3 0
3246 3247 0 3d
:2 0 3248 3249 0
324b 1bc2 3252 15
:3 0 e1 :3 0 324c
324d 0 e2 :2 0
324e 324f 0 3251
1bc4 3253 3245 324b
0 3254 0 3251
0 3254 1bc6 0
3255 1bc9 32a8 e5
:3 0 15 :3 0 e1
:3 0 3257 3258 0
e4 :3 0 3259 325a
0 325c 1bcc 325e
1bce 325d 325c :2 0
32a6 e6 :3 0 15
:3 0 e1 :3 0 3261
3262 0 e4 :3 0
3263 3264 0 329e
c3 :3 0 c4 :3 0
3266 3267 0 c3
:3 0 d3 :3 0 3269
326a 0 e7 :2 0
2 :4 0 27e :2 0
d5 :2 0 2a :2 0
1bd0 326f 3271 :3 0
1bd3 326b 3273 3268
3275 3274 :2 0 c6
:3 0 d6 :3 0 3277
3278 0 c8 :4 0
27f :4 0 cb :2 0
cc :3 0 1bd7 327c
327e :3 0 cb :2 0
ea :4 0 1bda 3280
3282 :3 0 cb :2 0
eb :4 0 1bdd 3284
3286 :3 0 cb :2 0
e4 :3 0 1be0 3288
328a :3 0 cb :2 0
ec :4 0 1be3 328c
328e :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
1be6 3291 3295 1bea
3290 3297 :3 0 1bed
3279 3299 :2 0 329b
1bf0 329c 3276 329b
0 329d 1bf2 0
329e 1bf4 32a1 :3 0
32a1 0 32a1 32a0
329e 329f :6 0 32a3
92 :3 0 1bf7 32a5
1bf9 32a4 32a3 :2 0
32a6 1bfb :2 0 32a8
0 32a8 32a7 3255
32a6 :6 0 3320 6f
:3 0 af :3 0 92
:3 0 21 :3 0 93
:3 0 130 :3 0 23
:3 0 16e :3 0 178
:3 0 6e :3 0 98
:3 0 1b :4 0 280
1 :8 0 32cb e0
:4 0 32b6 :3 0 f8
:3 0 da :2 0 1c00
32b9 32ba :3 0 15
:3 0 e1 :3 0 32bc
32bd 0 3d :2 0
32be 32bf 0 32c1
1c03 32c8 15 :3 0
e1 :3 0 32c2 32c3
0 e2 :2 0 32c4
32c5 0 32c7 1c05
32c9 32bb 32c1 0
32ca 0 32c7 0
32ca 1c07 0 32cb
1c0a 331e e5 :3 0
15 :3 0 e1 :3 0
32cd 32ce 0 e4
:3 0 32cf 32d0 0
32d2 1c0d 32d4 1c0f
32d3 32d2 :2 0 331c
e6 :3 0 15 :3 0
e1 :3 0 32d7 32d8
0 e4 :3 0 32d9
32da 0 3314 c3
:3 0 c4 :3 0 32dc
32dd 0 c3 :3 0
d3 :3 0 32df 32e0
0 e7 :2 0 2
:4 0 281 :2 0 d5
:2 0 2a :2 0 1c11
32e5 32e7 :3 0 1c14
32e1 32e9 32de 32eb
32ea :2 0 c6 :3 0
d6 :3 0 32ed 32ee
0 c8 :4 0 282
:4 0 cb :2 0 cc
:3 0 1c18 32f2 32f4
:3 0 cb :2 0 ea
:4 0 1c1b 32f6 32f8
:3 0 cb :2 0 eb
:4 0 1c1e 32fa 32fc
:3 0 cb :2 0 e4
:3 0 1c21 32fe 3300
:3 0 cb :2 0 ec
:4 0 1c24 3302 3304
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 1c27
3307 330b 1c2b 3306
330d :3 0 1c2e 32ef
330f :2 0 3311 1c31
3312 32ec 3311 0
3313 1c33 0 3314
1c35 3317 :3 0 3317
0 3317 3316 3314
3315 :6 0 3319 94
:3 0 1c38 331b 1c3a
331a 3319 :2 0 331c
1c3c :2 0 331e 0
331e 331d 32cb 331c
:6 0 3320 6f :3 0
1c3f 3322 30c0 3227
0 3323 0 3320
0 3323 1c43 0
3324 1c46 3326 2e0c
3031 0 3327 0
3324 0 3327 1c49
0 41a0 84 :3 0
3f :3 0 3d :2 0
1c4c 3328 332b 10b
:2 0 3d :2 0 1c51
332d 332f :3 0 84
:3 0 65 :3 0 3d
:2 0 1c54 3331 3334
da :2 0 2a :2 0
1c59 3336 3338 :3 0
3339 :2 0 3a :3 0
166 :2 0 1c5c 333c
333d :3 0 333e :2 0
333f :2 0 333a 3341
3340 :2 0 125 :3 0
125 :3 0 3a :3 0
64 :3 0 ae :3 0
8c :3 0 20 :4 0
283 1 :8 0 3350
15 :3 0 e1 :3 0
334b 334c 0 e4
:3 0 334d 334e 0
3350 1c5e 33a3 e5
:3 0 15 :3 0 e1
:3 0 3352 3353 0
e4 :3 0 3354 3355
0 3357 1c61 3359
1c63 3358 3357 :2 0
33a1 e6 :3 0 15
:3 0 e1 :3 0 335c
335d 0 e4 :3 0
335e 335f 0 3399
c3 :3 0 c4 :3 0
3361 3362 0 c3
:3 0 d3 :3 0 3364
3365 0 e7 :2 0
2 :4 0 284 :2 0
d5 :2 0 2a :2 0
1c65 336a 336c :3 0
1c68 3366 336e 3363
3370 336f :2 0 c6
:3 0 d6 :3 0 3372
3373 0 c8 :4 0
285 :4 0 cb :2 0
cc :3 0 1c6c 3377
3379 :3 0 cb :2 0
ea :4 0 1c6f 337b
337d :3 0 cb :2 0
eb :4 0 1c72 337f
3381 :3 0 cb :2 0
e4 :3 0 1c75 3383
3385 :3 0 cb :2 0
ec :4 0 1c78 3387
3389 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
1c7b 338c 3390 1c7f
338b 3392 :3 0 1c82
3374 3394 :2 0 3396
1c85 3397 3371 3396
0 3398 1c87 0
3399 1c89 339c :3 0
339c 0 339c 339b
3399 339a :6 0 339e
96 :3 0 1c8c 33a0
1c8e 339f 339e :2 0
33a1 1c90 :2 0 33a3
0 33a3 33a2 3350
33a1 :6 0 3512 6f
:3 0 15 :3 0 e1
:3 0 33a5 33a6 0
15 :3 0 da :2 0
158 :3 0 33a8 33aa
0 1c95 33a9 33ac
:3 0 33ad :2 0 84
:3 0 64 :3 0 3d
:2 0 1c98 33af 33b2
da :2 0 3d :2 0
1c9d 33b4 33b6 :3 0
33b7 :2 0 33ae 33b9
33b8 :2 0 ae :3 0
125 :3 0 3a :3 0
8c :3 0 20 :4 0
286 1 :8 0 33d6
e0 :4 0 33c1 :3 0
f8 :3 0 da :2 0
1ca2 33c4 33c5 :3 0
15 :3 0 e1 :3 0
33c7 33c8 0 3d
:2 0 33c9 33ca 0
33cc 1ca5 33d3 15
:3 0 e1 :3 0 33cd
33ce 0 e2 :2 0
33cf 33d0 0 33d2
1ca7 33d4 33c6 33cc
0 33d5 0 33d2
0 33d5 1ca9 0
33d6 1cac 3429 e5
:3 0 15 :3 0 e1
:3 0 33d8 33d9 0
e4 :3 0 33da 33db
0 33dd 1caf 33df
1cb1 33de 33dd :2 0
3427 e6 :3 0 15
:3 0 e1 :3 0 33e2
33e3 0 e4 :3 0
33e4 33e5 0 341f
c3 :3 0 c4 :3 0
33e7 33e8 0 c3
:3 0 d3 :3 0 33ea
33eb 0 e7 :2 0
2 :4 0 287 :2 0
d5 :2 0 2a :2 0
1cb3 33f0 33f2 :3 0
1cb6 33ec 33f4 33e9
33f6 33f5 :2 0 c6
:3 0 d6 :3 0 33f8
33f9 0 c8 :4 0
288 :4 0 cb :2 0
cc :3 0 1cba 33fd
33ff :3 0 cb :2 0
ea :4 0 1cbd 3401
3403 :3 0 cb :2 0
eb :4 0 1cc0 3405
3407 :3 0 cb :2 0
e4 :3 0 1cc3 3409
340b :3 0 cb :2 0
ec :4 0 1cc6 340d
340f :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
1cc9 3412 3416 1ccd
3411 3418 :3 0 1cd0
33fa 341a :2 0 341c
1cd3 341d 33f7 341c
0 341e 1cd5 0
341f 1cd7 3422 :3 0
3422 0 3422 3421
341f 3420 :6 0 3424
98 :3 0 1cda 3426
1cdc 3425 3424 :2 0
3427 1cde :2 0 3429
0 3429 3428 33d6
3427 :6 0 342c 6f
:3 0 141 :3 0 1ce1
3510 15 :3 0 e1
:3 0 342d 342e 0
15 :3 0 da :2 0
158 :3 0 3430 3432
0 1ce5 3431 3434
:3 0 3435 :2 0 84
:3 0 64 :3 0 3d
:2 0 1ce8 3437 343a
da :2 0 13b :2 0
1ced 343c 343e :3 0
343f :2 0 3436 3441
3440 :2 0 e3 :3 0
96 :4 0 3446 :2 0
344c 3444 0 15
:3 0 e1 :3 0 3447
3448 0 e4 :3 0
3449 344a 0 344c
1cf0 349f e5 :3 0
15 :3 0 e1 :3 0
344e 344f 0 e4
:3 0 3450 3451 0
3453 1cf3 3455 1cf5
3454 3453 :2 0 349d
e6 :3 0 15 :3 0
e1 :3 0 3458 3459
0 e4 :3 0 345a
345b 0 3495 c3
:3 0 c4 :3 0 345d
345e 0 c3 :3 0
d3 :3 0 3460 3461
0 e7 :2 0 2
:4 0 289 :2 0 d5
:2 0 2a :2 0 1cf7
3466 3468 :3 0 1cfa
3462 346a 345f 346c
346b :2 0 c6 :3 0
d6 :3 0 346e 346f
0 c8 :4 0 28a
:4 0 cb :2 0 cc
:3 0 1cfe 3473 3475
:3 0 cb :2 0 ea
:4 0 1d01 3477 3479
:3 0 cb :2 0 eb
:4 0 1d04 347b 347d
:3 0 cb :2 0 e4
:3 0 1d07 347f 3481
:3 0 cb :2 0 ec
:4 0 1d0a 3483 3485
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 1d0d
3488 348c 1d11 3487
348e :3 0 1d14 3470
3490 :2 0 3492 1d17
3493 346d 3492 0
3494 1d19 0 3495
1d1b 3498 :3 0 3498
0 3498 3497 3495
3496 :6 0 349a 9a
:3 0 1d1e 349c 1d20
349b 349a :2 0 349d
1d22 :2 0 349f 0
349f 349e 344c 349d
:6 0 350e 6f :3 0
c3 :3 0 c4 :3 0
34a1 34a2 0 c3
:3 0 d3 :3 0 34a4
34a5 0 18 :2 0
2 :4 0 28b :2 0
d5 :2 0 2a :2 0
1d25 34aa 34ac :3 0
1d28 34a6 34ae 34a3
34b0 34af :2 0 c6
:3 0 d6 :3 0 34b2
34b3 0 c8 :4 0
28c :4 0 cb :2 0
cc :3 0 1d2c 34b7
34b9 :3 0 cb :2 0
ea :4 0 1d2f 34bb
34bd :3 0 cb :2 0
f1 :4 0 1d32 34bf
34c1 :3 0 cb :2 0
84 :3 0 d0 :3 0
b0 :3 0 28d :3 0
34c6 34c7 0 1d35
34c5 34c9 d1 :4 0
1d37 34c4 34cc 1d3a
34c3 34ce :3 0 cb
:2 0 f3 :4 0 1d3d
34d0 34d2 :3 0 cb
:2 0 ce :4 0 1d40
34d4 34d6 :3 0 cb
:2 0 20 :3 0 1d43
34d8 34da :3 0 cb
:2 0 ce :4 0 1d46
34dc 34de :3 0 1d49
34b4 34e0 :2 0 34e2
1d4c 34e3 34b1 34e2
0 34e4 1d4e 0
350e c3 :3 0 c4
:3 0 34e5 34e6 0
c3 :3 0 d3 :3 0
34e8 34e9 0 2a
:2 0 2 :4 0 28e
:2 0 d5 :2 0 2a
:2 0 1d50 34ee 34f0
:3 0 1d53 34ea 34f2
34e7 34f4 34f3 :2 0
c6 :3 0 d6 :3 0
34f6 34f7 0 c8
:4 0 28f :4 0 cb
:2 0 cc :3 0 1d57
34fb 34fd :3 0 1d5a
34f8 34ff :2 0 3501
1d5d 3502 34f5 3501
0 3503 1d5f 0
350e f6 :3 0 f7
:3 0 3504 3505 0
b0 :3 0 28d :3 0
3507 3508 0 20
:3 0 f8 :3 0 1d61
3506 350c :2 0 350e
1d65 350f 3442 350e
0 3511 33ba 342c
0 3511 1d6a 0
3512 1d6d 3513 3342
3512 0 3514 1d70
0 419d 7a :3 0
1c0 :2 0 2a :2 0
1d72 3516 3518 :3 0
3515 3519 0 419d
15 :3 0 e1 :3 0
351b 351c 0 1c1
:3 0 22 :3 0 20
:3 0 1d :3 0 72
:3 0 73 :3 0 74
:3 0 75 :3 0 7a
:3 0 79 :3 0 78
:3 0 1d74 351e 3529
351d 352a 0 419d
1b2 :3 0 91 :3 0
22 :3 0 9a :3 0
1d :4 0 24f 1
:8 0 3547 e0 :4 0
3532 :3 0 f8 :3 0
da :2 0 1d81 3535
3536 :3 0 15 :3 0
e1 :3 0 3538 3539
0 3d :2 0 353a
353b 0 353d 1d84
3544 15 :3 0 e1
:3 0 353e 353f 0
e2 :2 0 3540 3541
0 3543 1d86 3545
3537 353d 0 3546
0 3543 0 3546
1d88 0 3547 1d8b
359a e5 :3 0 15
:3 0 e1 :3 0 3549
354a 0 e4 :3 0
354b 354c 0 354e
1d8e 3550 1d90 354f
354e :2 0 3598 e6
:3 0 15 :3 0 e1
:3 0 3553 3554 0
e4 :3 0 3555 3556
0 3590 c3 :3 0
c4 :3 0 3558 3559
0 c3 :3 0 d3
:3 0 355b 355c 0
e7 :2 0 2 :4 0
290 :2 0 d5 :2 0
2a :2 0 1d92 3561
3563 :3 0 1d95 355d
3565 355a 3567 3566
:2 0 c6 :3 0 d6
:3 0 3569 356a 0
c8 :4 0 291 :4 0
cb :2 0 cc :3 0
1d99 356e 3570 :3 0
cb :2 0 ea :4 0
1d9c 3572 3574 :3 0
cb :2 0 eb :4 0
1d9f 3576 3578 :3 0
cb :2 0 e4 :3 0
1da2 357a 357c :3 0
cb :2 0 ec :4 0
1da5 357e 3580 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 1da8 3583
3587 1dac 3582 3589
:3 0 1daf 356b 358b
:2 0 358d 1db2 358e
3568 358d 0 358f
1db4 0 3590 1db6
3593 :3 0 3593 0
3593 3592 3590 3591
:6 0 3595 9c :3 0
1db9 3597 1dbb 3596
3595 :2 0 3598 1dbd
:2 0 359a 0 359a
3599 3547 3598 :6 0
419d 6f :3 0 26
:3 0 de :4 0 359c
359d 0 419d 82
:3 0 13c :2 0 3d
:2 0 1dc2 35a0 35a2
:3 0 3f :3 0 9
:3 0 35a4 35a5 0
39b3 24 :3 0 7
:3 0 35a7 35a8 0
39b3 26 :3 0 8
:3 0 35aa 35ab 0
39b3 44 :3 0 b
:3 0 35ad 35ae 0
39b3 45 :3 0 10
:3 0 35b0 35b1 0
39b3 46 :3 0 d
:3 0 35b3 35b4 0
39b3 47 :3 0 e
:3 0 35b6 35b7 0
39b3 4e :3 0 c
:3 0 35b9 35ba 0
39b3 48 :3 0 f
:3 0 35bc 35bd 0
39b3 d8 :3 0 84
:3 0 c :3 0 3d
:2 0 1dc5 35c0 35c3
b0 :3 0 1b9 :3 0
35c5 35c6 0 1dc8
35bf 35c8 35c9 :2 0
13c :2 0 3d :2 0
1dcd 35cb 35cd :3 0
35ce :2 0 84 :3 0
48 :3 0 3d :2 0
1dd0 35d0 35d3 84
:3 0 13c :2 0 f
:3 0 3d :2 0 1dd3
35d5 35d9 1dd8 35d6
35db :3 0 35dc :2 0
84 :3 0 44 :3 0
3d :2 0 1ddb 35de
35e1 84 :3 0 13c
:2 0 b :3 0 3d
:2 0 1dde 35e3 35e7
1de3 35e4 35e9 :3 0
35ea :2 0 35dd 35ec
35eb :2 0 35ed :2 0
35cf 35ef 35ee :2 0
35f0 :2 0 84 :3 0
e :3 0 3d :2 0
1de6 35f2 35f5 da
:2 0 e2 :2 0 1deb
35f7 35f9 :3 0 90
:3 0 92 :3 0 21
:3 0 93 :3 0 94
:3 0 4c :3 0 1e
:3 0 a7 :3 0 62
:3 0 58 :3 0 a7
:3 0 1d4 :3 0 62
:3 0 5c :3 0 1d4
:3 0 a8 :3 0 61
:3 0 5b :3 0 a8
:3 0 1d5 :3 0 61
:3 0 5d :3 0 1d5
:3 0 9e :3 0 b
:3 0 292 :3 0 3f
:3 0 ac :3 0 f
:3 0 1d8 :3 0 19a
:3 0 19b :3 0 1d8
:3 0 b0 :3 0 1d9
:3 0 9a :3 0 1d
:4 0 293 1 :8 0
3636 e0 :4 0 3621
:3 0 f8 :3 0 da
:2 0 1df0 3624 3625
:3 0 15 :3 0 e1
:3 0 3627 3628 0
3d :2 0 3629 362a
0 362c 1df3 3633
15 :3 0 e1 :3 0
362d 362e 0 e2
:2 0 362f 3630 0
3632 1df5 3634 3626
362c 0 3635 0
3632 0 3635 1df7
0 3636 1dfa 3689
e5 :3 0 15 :3 0
e1 :3 0 3638 3639
0 e4 :3 0 363a
363b 0 363d 1dfd
363f 1dff 363e 363d
:2 0 3687 e6 :3 0
15 :3 0 e1 :3 0
3642 3643 0 e4
:3 0 3644 3645 0
367f c3 :3 0 c4
:3 0 3647 3648 0
c3 :3 0 d3 :3 0
364a 364b 0 e7
:2 0 2 :4 0 294
:2 0 d5 :2 0 2a
:2 0 1e01 3650 3652
:3 0 1e04 364c 3654
3649 3656 3655 :2 0
c6 :3 0 d6 :3 0
3658 3659 0 c8
:4 0 295 :4 0 cb
:2 0 cc :3 0 1e08
365d 365f :3 0 cb
:2 0 ea :4 0 1e0b
3661 3663 :3 0 cb
:2 0 eb :4 0 1e0e
3665 3667 :3 0 cb
:2 0 e4 :3 0 1e11
3669 366b :3 0 cb
:2 0 ec :4 0 1e14
366d 366f :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 1e17 3672 3676
1e1b 3671 3678 :3 0
1e1e 365a 367a :2 0
367c 1e21 367d 3657
367c 0 367e 1e23
0 367f 1e25 3682
:3 0 3682 0 3682
3681 367f 3680 :6 0
3684 9e :3 0 1e28
3686 1e2a 3685 3684
:2 0 3687 1e2c :2 0
3689 0 3689 3688
3636 3687 :6 0 368b
6f :3 0 1e2f 371f
90 :3 0 92 :3 0
21 :3 0 93 :3 0
94 :3 0 4c :3 0
1e :3 0 a7 :3 0
62 :3 0 58 :3 0
a7 :3 0 1d4 :3 0
62 :3 0 5c :3 0
1d4 :3 0 a8 :3 0
61 :3 0 5b :3 0
a8 :3 0 1d5 :3 0
61 :3 0 5d :3 0
1d5 :3 0 a9 :3 0
b :3 0 292 :3 0
3f :3 0 ab :3 0
e :3 0 ac :3 0
f :3 0 1d8 :3 0
19a :3 0 19b :3 0
1d8 :3 0 b0 :3 0
1d9 :3 0 9a :3 0
1d :4 0 296 1
:8 0 36c9 e0 :4 0
36b4 :3 0 f8 :3 0
da :2 0 1e33 36b7
36b8 :3 0 15 :3 0
e1 :3 0 36ba 36bb
0 3d :2 0 36bc
36bd 0 36bf 1e36
36c6 15 :3 0 e1
:3 0 36c0 36c1 0
e2 :2 0 36c2 36c3
0 36c5 1e38 36c7
36b9 36bf 0 36c8
0 36c5 0 36c8
1e3a 0 36c9 1e3d
371c e5 :3 0 15
:3 0 e1 :3 0 36cb
36cc 0 e4 :3 0
36cd 36ce 0 36d0
1e40 36d2 1e42 36d1
36d0 :2 0 371a e6
:3 0 15 :3 0 e1
:3 0 36d5 36d6 0
e4 :3 0 36d7 36d8
0 3712 c3 :3 0
c4 :3 0 36da 36db
0 c3 :3 0 d3
:3 0 36dd 36de 0
e7 :2 0 2 :4 0
297 :2 0 d5 :2 0
2a :2 0 1e44 36e3
36e5 :3 0 1e47 36df
36e7 36dc 36e9 36e8
:2 0 c6 :3 0 d6
:3 0 36eb 36ec 0
c8 :4 0 298 :4 0
cb :2 0 cc :3 0
1e4b 36f0 36f2 :3 0
cb :2 0 ea :4 0
1e4e 36f4 36f6 :3 0
cb :2 0 eb :4 0
1e51 36f8 36fa :3 0
cb :2 0 e4 :3 0
1e54 36fc 36fe :3 0
cb :2 0 ec :4 0
1e57 3700 3702 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 1e5a 3705
3709 1e5e 3704 370b
:3 0 1e61 36ed 370d
:2 0 370f 1e64 3710
36ea 370f 0 3711
1e66 0 3712 1e68
3715 :3 0 3715 0
3715 3714 3712 3713
:6 0 3717 a0 :3 0
1e6b 3719 1e6d 3718
3717 :2 0 371a 1e6f
:2 0 371c 0 371c
371b 36c9 371a :6 0
371e 6f :3 0 1e72
3720 35fa 368b 0
3721 0 371e 0
3721 1e74 0 3722
1e77 37b0 90 :3 0
92 :3 0 21 :3 0
93 :3 0 94 :3 0
4c :3 0 1e :3 0
a7 :3 0 62 :3 0
58 :3 0 a7 :3 0
1d4 :3 0 62 :3 0
5c :3 0 1d4 :3 0
a8 :3 0 61 :3 0
5b :3 0 a8 :3 0
1d5 :3 0 61 :3 0
5d :3 0 1d5 :3 0
1d8 :3 0 19a :3 0
19b :3 0 1d8 :3 0
b0 :3 0 1d9 :3 0
292 :3 0 3f :3 0
9a :3 0 1d :4 0
299 1 :8 0 375a
e0 :4 0 3745 :3 0
f8 :3 0 da :2 0
1e7b 3748 3749 :3 0
15 :3 0 e1 :3 0
374b 374c 0 3d
:2 0 374d 374e 0
3750 1e7e 3757 15
:3 0 e1 :3 0 3751
3752 0 e2 :2 0
3753 3754 0 3756
1e80 3758 374a 3750
0 3759 0 3756
0 3759 1e82 0
375a 1e85 37ad e5
:3 0 15 :3 0 e1
:3 0 375c 375d 0
e4 :3 0 375e 375f
0 3761 1e88 3763
1e8a 3762 3761 :2 0
37ab e6 :3 0 15
:3 0 e1 :3 0 3766
3767 0 e4 :3 0
3768 3769 0 37a3
c3 :3 0 c4 :3 0
376b 376c 0 c3
:3 0 d3 :3 0 376e
376f 0 e7 :2 0
2 :4 0 29a :2 0
d5 :2 0 2a :2 0
1e8c 3774 3776 :3 0
1e8f 3770 3778 376d
377a 3779 :2 0 c6
:3 0 d6 :3 0 377c
377d 0 c8 :4 0
29b :4 0 cb :2 0
cc :3 0 1e93 3781
3783 :3 0 cb :2 0
ea :4 0 1e96 3785
3787 :3 0 cb :2 0
eb :4 0 1e99 3789
378b :3 0 cb :2 0
e4 :3 0 1e9c 378d
378f :3 0 cb :2 0
ec :4 0 1e9f 3791
3793 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
1ea2 3796 379a 1ea6
3795 379c :3 0 1ea9
377e 379e :2 0 37a0
1eac 37a1 377b 37a0
0 37a2 1eae 0
37a3 1eb0 37a6 :3 0
37a6 0 37a6 37a5
37a3 37a4 :6 0 37a8
a2 :3 0 1eb3 37aa
1eb5 37a9 37a8 :2 0
37ab 1eb7 :2 0 37ad
0 37ad 37ac 375a
37ab :6 0 37af 6f
:3 0 1eba 37b1 35f1
3722 0 37b2 0
37af 0 37b2 1ebc
0 39b3 84 :3 0
48 :3 0 3d :2 0
1ebf 37b3 37b6 da
:2 0 3d :2 0 1ec4
37b8 37ba :3 0 37bb
:2 0 84 :3 0 47
:3 0 3d :2 0 1ec7
37bd 37c0 13c :2 0
e2 :2 0 1ecc 37c2
37c4 :3 0 37c5 :2 0
37bc 37c7 37c6 :2 0
44 :3 0 3d :2 0
37c9 37ca 0 37cd
141 :3 0 1ecf 37e0
84 :3 0 47 :3 0
3d :2 0 1ed1 37ce
37d1 da :2 0 e2
:2 0 1ed6 37d3 37d5
:3 0 7e :3 0 84
:3 0 44 :3 0 3d
:2 0 1ed9 37d8 37db
37d7 37dc 0 37de
1edc 37df 37d6 37de
0 37e1 37c8 37cd
0 37e1 1ede 0
39b3 6b :3 0 202
:3 0 31 :3 0 32
:3 0 84 :3 0 46
:3 0 3d :2 0 1ee1
37e6 37e9 84 :3 0
47 :3 0 3d :2 0
1ee4 37eb 37ee 84
:3 0 52 :3 0 3d
:2 0 1ee7 37f0 37f3
84 :3 0 4e :3 0
3d :2 0 1eea 37f5
37f8 84 :3 0 48
:3 0 3d :2 0 1eed
37fa 37fd 84 :3 0
45 :3 0 3d :2 0
1ef0 37ff 3802 84
:3 0 4f :3 0 3d
:2 0 1ef3 3804 3807
84 :3 0 50 :3 0
3d :2 0 1ef6 3809
380c 44 :3 0 3f
:3 0 49 :3 0 1ef9
37e3 3811 37e2 3812
0 39b3 84 :3 0
6b :3 0 3d :2 0
1f07 3814 3817 b0
:3 0 da :2 0 203
:3 0 3819 381b 0
1f0c 381a 381d :3 0
e3 :3 0 96 :4 0
3822 :2 0 3828 3820
0 15 :3 0 e1
:3 0 3823 3824 0
e4 :3 0 3825 3826
0 3828 1f0f 387b
e5 :3 0 15 :3 0
e1 :3 0 382a 382b
0 e4 :3 0 382c
382d 0 382f 1f12
3831 1f14 3830 382f
:2 0 3879 e6 :3 0
15 :3 0 e1 :3 0
3834 3835 0 e4
:3 0 3836 3837 0
3871 c3 :3 0 c4
:3 0 3839 383a 0
c3 :3 0 d3 :3 0
383c 383d 0 e7
:2 0 2 :4 0 29c
:2 0 d5 :2 0 2a
:2 0 1f16 3842 3844
:3 0 1f19 383e 3846
383b 3848 3847 :2 0
c6 :3 0 d6 :3 0
384a 384b 0 c8
:4 0 29d :4 0 cb
:2 0 cc :3 0 1f1d
384f 3851 :3 0 cb
:2 0 ea :4 0 1f20
3853 3855 :3 0 cb
:2 0 eb :4 0 1f23
3857 3859 :3 0 cb
:2 0 e4 :3 0 1f26
385b 385d :3 0 cb
:2 0 ec :4 0 1f29
385f 3861 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 1f2c 3864 3868
1f30 3863 386a :3 0
1f33 384c 386c :2 0
386e 1f36 386f 3849
386e 0 3870 1f38
0 3871 1f3a 3874
:3 0 3874 0 3874
3873 3871 3872 :6 0
3876 a4 :3 0 1f3d
3878 1f3f 3877 3876
:2 0 3879 1f41 :2 0
387b 0 387b 387a
3828 3879 :6 0 38da
6f :3 0 c3 :3 0
c4 :3 0 387d 387e
0 c3 :3 0 d3
:3 0 3880 3881 0
204 :2 0 2 :4 0
29e :2 0 d5 :2 0
2a :2 0 1f44 3886
3888 :3 0 1f47 3882
388a 387f 388c 388b
:2 0 c6 :3 0 d6
:3 0 388e 388f 0
c8 :4 0 29f :4 0
cb :2 0 cc :3 0
1f4b 3893 3895 :3 0
cb :2 0 ea :4 0
1f4e 3897 3899 :3 0
cb :2 0 f1 :4 0
1f51 389b 389d :3 0
cb :2 0 84 :3 0
d0 :3 0 b0 :3 0
203 :3 0 38a2 38a3
0 1f54 38a1 38a5
d1 :4 0 1f56 38a0
38a8 1f59 389f 38aa
:3 0 1f5c 3890 38ac
:2 0 38ae 1f5f 38af
388d 38ae 0 38b0
1f61 0 38da c3
:3 0 c4 :3 0 38b1
38b2 0 c3 :3 0
d3 :3 0 38b4 38b5
0 2a :2 0 2
:4 0 2a0 :2 0 d5
:2 0 2a :2 0 1f63
38ba 38bc :3 0 1f66
38b6 38be 38b3 38c0
38bf :2 0 c6 :3 0
d6 :3 0 38c2 38c3
0 c8 :4 0 2a1
:4 0 cb :2 0 cc
:3 0 1f6a 38c7 38c9
:3 0 1f6d 38c4 38cb
:2 0 38cd 1f70 38ce
38c1 38cd 0 38cf
1f72 0 38da f6
:3 0 209 :3 0 38d0
38d1 0 b0 :3 0
203 :3 0 38d3 38d4
0 f8 :3 0 1f74
38d2 38d7 :2 0 38da
141 :3 0 1f77 39b1
84 :3 0 6b :3 0
3d :2 0 1f7c 38db
38de 13c :2 0 3d
:2 0 1f81 38e0 38e2
:3 0 e3 :3 0 96
:4 0 38e7 :2 0 38ed
38e5 0 15 :3 0
e1 :3 0 38e8 38e9
0 e4 :3 0 38ea
38eb 0 38ed 1f84
3940 e5 :3 0 15
:3 0 e1 :3 0 38ef
38f0 0 e4 :3 0
38f1 38f2 0 38f4
1f87 38f6 1f89 38f5
38f4 :2 0 393e e6
:3 0 15 :3 0 e1
:3 0 38f9 38fa 0
e4 :3 0 38fb 38fc
0 3936 c3 :3 0
c4 :3 0 38fe 38ff
0 c3 :3 0 d3
:3 0 3901 3902 0
e7 :2 0 2 :4 0
2a2 :2 0 d5 :2 0
2a :2 0 1f8b 3907
3909 :3 0 1f8e 3903
390b 3900 390d 390c
:2 0 c6 :3 0 d6
:3 0 390f 3910 0
c8 :4 0 2a3 :4 0
cb :2 0 cc :3 0
1f92 3914 3916 :3 0
cb :2 0 ea :4 0
1f95 3918 391a :3 0
cb :2 0 eb :4 0
1f98 391c 391e :3 0
cb :2 0 e4 :3 0
1f9b 3920 3922 :3 0
cb :2 0 ec :4 0
1f9e 3924 3926 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 1fa1 3929
392d 1fa5 3928 392f
:3 0 1fa8 3911 3931
:2 0 3933 1fab 3934
390e 3933 0 3935
1fad 0 3936 1faf
3939 :3 0 3939 0
3939 3938 3936 3937
:6 0 393b a6 :3 0
1fb2 393d 1fb4 393c
393b :2 0 393e 1fb6
:2 0 3940 0 3940
393f 38ed 393e :6 0
39af 6f :3 0 c3
:3 0 c4 :3 0 3942
3943 0 c3 :3 0
d3 :3 0 3945 3946
0 18 :2 0 2
:4 0 2a4 :2 0 d5
:2 0 2a :2 0 1fb9
394b 394d :3 0 1fbc
3947 394f 3944 3951
3950 :2 0 c6 :3 0
d6 :3 0 3953 3954
0 c8 :4 0 2a5
:4 0 cb :2 0 cc
:3 0 1fc0 3958 395a
:3 0 cb :2 0 ea
:4 0 1fc3 395c 395e
:3 0 cb :2 0 f1
:4 0 1fc6 3960 3962
:3 0 cb :2 0 84
:3 0 d0 :3 0 6b
:3 0 1fc9 3966 3968
d1 :4 0 1fcb 3965
396b 1fce 3964 396d
:3 0 cb :2 0 f3
:4 0 1fd1 396f 3971
:3 0 cb :2 0 ce
:4 0 1fd4 3973 3975
:3 0 cb :2 0 22
:3 0 1fd7 3977 3979
:3 0 cb :2 0 ce
:4 0 1fda 397b 397d
:3 0 1fdd 3955 397f
:2 0 3981 1fe0 3982
3952 3981 0 3983
1fe2 0 39af c3
:3 0 c4 :3 0 3984
3985 0 c3 :3 0
d3 :3 0 3987 3988
0 2a :2 0 2
:4 0 2a6 :2 0 d5
:2 0 2a :2 0 1fe4
398d 398f :3 0 1fe7
3989 3991 3986 3993
3992 :2 0 c6 :3 0
d6 :3 0 3995 3996
0 c8 :4 0 2a7
:4 0 cb :2 0 cc
:3 0 1feb 399a 399c
:3 0 1fee 3997 399e
:2 0 39a0 1ff1 39a1
3994 39a0 0 39a2
1ff3 0 39af f6
:3 0 f7 :3 0 39a3
39a4 0 84 :3 0
6b :3 0 3d :2 0
1ff5 39a6 39a9 22
:3 0 f8 :3 0 1ff8
39a5 39ad :2 0 39af
1ffc 39b0 38e3 39af
0 39b2 381e 38da
0 39b2 2001 0
39b3 2004 3acd 84
:3 0 44 :3 0 3d
:2 0 2012 39b4 39b7
da :2 0 3d :2 0
2017 39b9 39bb :3 0
84 :3 0 7e :3 0
3d :2 0 201a 39bd
39c0 13c :2 0 e2
:2 0 201f 39c2 39c4
:3 0 39bc 39c6 39c5
:2 0 84 :3 0 7e
:3 0 3d :2 0 2022
39c8 39cb 13c :2 0
3d :2 0 2027 39cd
39cf :3 0 39c7 39d1
39d0 :2 0 47 :3 0
e2 :2 0 39d3 39d4
0 39dd 44 :3 0
84 :3 0 7e :3 0
3d :2 0 202a 39d7
39da 39d6 39db 0
39dd 202d 39de 39d2
39dd 0 39df 2030
0 3acc 84 :3 0
49 :3 0 3d :2 0
2032 39e0 39e3 da
:2 0 3d :2 0 2037
39e5 39e7 :3 0 39e8
:2 0 84 :3 0 49
:3 0 3d :2 0 203a
39ea 39ed 84 :3 0
169 :2 0 4f :3 0
3d :2 0 203d 39ef
39f3 2042 39f0 39f5
:3 0 39f6 :2 0 39e9
39f8 39f7 :2 0 49
:3 0 84 :3 0 4f
:3 0 3d :2 0 2045
39fb 39fe 39fa 39ff
0 3a01 2048 3a15
49 :3 0 2a8 :3 0
84 :3 0 49 :3 0
3d :2 0 204a 3a04
3a07 2a9 :2 0 84
:3 0 4f :3 0 3d
:2 0 204d 3a0a 3a0d
2050 3a09 3a0f :3 0
2053 3a03 3a11 3a02
3a12 0 3a14 2055
3a16 39f9 3a01 0
3a17 0 3a14 0
3a17 2057 0 3acc
84 :3 0 49 :3 0
3d :2 0 205a 3a18
3a1b 169 :2 0 2aa
:2 0 205f 3a1d 3a1f
:3 0 49 :3 0 84
:3 0 49 :3 0 3d
:2 0 2062 3a22 3a25
1aa :2 0 3b :2 0
2065 3a27 3a29 :3 0
3a21 3a2a 0 3a2c
2068 3a3f 49 :3 0
2a8 :3 0 2ab :3 0
84 :3 0 49 :3 0
3d :2 0 206a 3a30
3a33 206d 3a2f 3a35
1aa :2 0 2ac :2 0
206f 3a37 3a39 :3 0
2072 3a2e 3a3b 3a2d
3a3c 0 3a3e 2074
3a40 3a20 3a2c 0
3a41 0 3a3e 0
3a41 2076 0 3acc
90 :3 0 92 :3 0
21 :3 0 93 :3 0
94 :3 0 4c :3 0
1e :3 0 a7 :3 0
62 :3 0 58 :3 0
a7 :3 0 1d4 :3 0
62 :3 0 5c :3 0
1d4 :3 0 a8 :3 0
61 :3 0 5b :3 0
a8 :3 0 1d5 :3 0
61 :3 0 5d :3 0
1d5 :3 0 1d8 :3 0
19a :3 0 19b :3 0
1d8 :3 0 b0 :3 0
1d9 :3 0 9a :3 0
1d :4 0 2ad 1
:8 0 3a77 e0 :4 0
3a62 :3 0 f8 :3 0
da :2 0 207b 3a65
3a66 :3 0 15 :3 0
e1 :3 0 3a68 3a69
0 3d :2 0 3a6a
3a6b 0 3a6d 207e
3a74 15 :3 0 e1
:3 0 3a6e 3a6f 0
e2 :2 0 3a70 3a71
0 3a73 2080 3a75
3a67 3a6d 0 3a76
0 3a73 0 3a76
2082 0 3a77 2085
3aca e5 :3 0 15
:3 0 e1 :3 0 3a79
3a7a 0 e4 :3 0
3a7b 3a7c 0 3a7e
2088 3a80 208a 3a7f
3a7e :2 0 3ac8 e6
:3 0 15 :3 0 e1
:3 0 3a83 3a84 0
e4 :3 0 3a85 3a86
0 3ac0 c3 :3 0
c4 :3 0 3a88 3a89
0 c3 :3 0 d3
:3 0 3a8b 3a8c 0
e7 :2 0 2 :4 0
2ae :2 0 d5 :2 0
2a :2 0 208c 3a91
3a93 :3 0 208f 3a8d
3a95 3a8a 3a97 3a96
:2 0 c6 :3 0 d6
:3 0 3a99 3a9a 0
c8 :4 0 2af :4 0
cb :2 0 cc :3 0
2093 3a9e 3aa0 :3 0
cb :2 0 ea :4 0
2096 3aa2 3aa4 :3 0
cb :2 0 eb :4 0
2099 3aa6 3aa8 :3 0
cb :2 0 e4 :3 0
209c 3aaa 3aac :3 0
cb :2 0 ec :4 0
209f 3aae 3ab0 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 20a2 3ab3
3ab7 20a6 3ab2 3ab9
:3 0 20a9 3a9b 3abb
:2 0 3abd 20ac 3abe
3a98 3abd 0 3abf
20ae 0 3ac0 20b0
3ac3 :3 0 3ac3 0
3ac3 3ac2 3ac0 3ac1
:6 0 3ac5 a8 :3 0
20b3 3ac7 20b5 3ac6
3ac5 :2 0 3ac8 20b7
:2 0 3aca 0 3aca
3ac9 3a77 3ac8 :6 0
3acc 6f :3 0 20ba
3ace 35a3 39b3 0
3acf 0 3acc 0
3acf 20bf 0 419d
4a :3 0 3d :2 0
3ad0 3ad1 0 419d
51 :3 0 2a :2 0
3ad3 3ad4 0 419d
40 :3 0 3d :2 0
3ad6 3ad7 0 419d
84 :3 0 44 :3 0
3d :2 0 20c2 3ad9
3adc da :2 0 3d
:2 0 20c7 3ade 3ae0
:3 0 84 :3 0 7e
:3 0 3d :2 0 20ca
3ae2 3ae5 da :2 0
e2 :2 0 20cf 3ae7
3ae9 :3 0 84 :3 0
7e :3 0 3d :2 0
20d2 3aeb 3aee da
:2 0 3d :2 0 20d7
3af0 3af2 :3 0 3aea
3af4 3af3 :2 0 3af5
:2 0 3ae1 3af7 3af6
:2 0 4e :3 0 b0
:3 0 20e :3 0 3afa
3afb 0 3af9 3afc
0 3afe 20da 3b0f
4e :3 0 200 :3 0
19b :3 0 3b00 3b01
0 84 :3 0 4e
:3 0 3d :2 0 20dc
3b03 3b06 b0 :3 0
20e :3 0 3b08 3b09
0 20df 3b02 3b0b
3aff 3b0c 0 3b0e
20e2 3b10 3af8 3afe
0 3b11 0 3b0e
0 3b11 20e4 0
419d 45 :3 0 3d
:2 0 3b12 3b13 0
419d 6a :3 0 5a
:2 0 3b15 3b16 0
419d 66 :3 0 20f
:3 0 1d :3 0 22
:3 0 20 :3 0 21
:3 0 6a :3 0 41
:3 0 29 :3 0 29
:3 0 29 :3 0 29
:3 0 24 :3 0 26
:3 0 49 :3 0 3f
:3 0 44 :3 0 46
:3 0 47 :3 0 40
:3 0 52 :3 0 2e
:3 0 4a :3 0 4e
:3 0 48 :3 0 45
:3 0 51 :3 0 4f
:3 0 53 :3 0 1a
:3 0 3d :2 0 3d
:2 0 5a :2 0 5a
:2 0 20e7 3b19 3b3a
3b18 3b3b 0 419d
84 :3 0 66 :3 0
3d :2 0 2108 3b3d
3b40 da :2 0 210
:2 0 210d 3b42 3b44
:3 0 e3 :3 0 96
:4 0 3b49 :2 0 3b4f
3b47 0 15 :3 0
e1 :3 0 3b4a 3b4b
0 e4 :3 0 3b4c
3b4d 0 3b4f 2110
3ba2 e5 :3 0 15
:3 0 e1 :3 0 3b51
3b52 0 e4 :3 0
3b53 3b54 0 3b56
2113 3b58 2115 3b57
3b56 :2 0 3ba0 e6
:3 0 15 :3 0 e1
:3 0 3b5b 3b5c 0
e4 :3 0 3b5d 3b5e
0 3b98 c3 :3 0
c4 :3 0 3b60 3b61
0 c3 :3 0 d3
:3 0 3b63 3b64 0
e7 :2 0 2 :4 0
2b0 :2 0 d5 :2 0
2a :2 0 2117 3b69
3b6b :3 0 211a 3b65
3b6d 3b62 3b6f 3b6e
:2 0 c6 :3 0 d6
:3 0 3b71 3b72 0
c8 :4 0 2b1 :4 0
cb :2 0 cc :3 0
211e 3b76 3b78 :3 0
cb :2 0 ea :4 0
2121 3b7a 3b7c :3 0
cb :2 0 eb :4 0
2124 3b7e 3b80 :3 0
cb :2 0 e4 :3 0
2127 3b82 3b84 :3 0
cb :2 0 ec :4 0
212a 3b86 3b88 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 212d 3b8b
3b8f 2131 3b8a 3b91
:3 0 2134 3b73 3b93
:2 0 3b95 2137 3b96
3b70 3b95 0 3b97
2139 0 3b98 213b
3b9b :3 0 3b9b 0
3b9b 3b9a 3b98 3b99
:6 0 3b9d aa :3 0
213e 3b9f 2140 3b9e
3b9d :2 0 3ba0 2142
:2 0 3ba2 0 3ba2
3ba1 3b4f 3ba0 :6 0
3c12 6f :3 0 c3
:3 0 c4 :3 0 3ba4
3ba5 0 c3 :3 0
d3 :3 0 3ba7 3ba8
0 18 :2 0 2
:4 0 2b2 :2 0 d5
:2 0 2a :2 0 2145
3bad 3baf :3 0 2148
3ba9 3bb1 3ba6 3bb3
3bb2 :2 0 c6 :3 0
d6 :3 0 3bb5 3bb6
0 c8 :4 0 2b3
:4 0 cb :2 0 cc
:3 0 214c 3bba 3bbc
:3 0 cb :2 0 ea
:4 0 214f 3bbe 3bc0
:3 0 cb :2 0 f1
:4 0 2152 3bc2 3bc4
:3 0 cb :2 0 84
:3 0 d0 :3 0 b0
:3 0 213 :3 0 3bc9
3bca 0 2155 3bc8
3bcc d1 :4 0 2157
3bc7 3bcf 215a 3bc6
3bd1 :3 0 cb :2 0
f3 :4 0 215d 3bd3
3bd5 :3 0 cb :2 0
ce :4 0 2160 3bd7
3bd9 :3 0 cb :2 0
21 :3 0 2163 3bdb
3bdd :3 0 cb :2 0
ce :4 0 2166 3bdf
3be1 :3 0 2169 3bb7
3be3 :2 0 3be5 216c
3be6 3bb4 3be5 0
3be7 216e 0 3c12
c3 :3 0 c4 :3 0
3be8 3be9 0 c3
:3 0 d3 :3 0 3beb
3bec 0 2a :2 0
2 :4 0 2b4 :2 0
d5 :2 0 2a :2 0
2170 3bf1 3bf3 :3 0
2173 3bed 3bf5 3bea
3bf7 3bf6 :2 0 c6
:3 0 d6 :3 0 3bf9
3bfa 0 c8 :4 0
2b5 :4 0 cb :2 0
cc :3 0 2177 3bfe
3c00 :3 0 217a 3bfb
3c02 :2 0 3c04 217d
3c05 3bf8 3c04 0
3c06 217f 0 3c12
f6 :3 0 f7 :3 0
3c07 3c08 0 b0
:3 0 213 :3 0 3c0a
3c0b 0 21 :3 0
f8 :3 0 2181 3c09
3c0f :2 0 3c12 141
:3 0 2185 3ce9 84
:3 0 66 :3 0 3d
:2 0 218a 3c13 3c16
13c :2 0 3d :2 0
218f 3c18 3c1a :3 0
e3 :3 0 96 :4 0
3c1f :2 0 3c25 3c1d
0 15 :3 0 e1
:3 0 3c20 3c21 0
e4 :3 0 3c22 3c23
0 3c25 2192 3c78
e5 :3 0 15 :3 0
e1 :3 0 3c27 3c28
0 e4 :3 0 3c29
3c2a 0 3c2c 2195
3c2e 2197 3c2d 3c2c
:2 0 3c76 e6 :3 0
15 :3 0 e1 :3 0
3c31 3c32 0 e4
:3 0 3c33 3c34 0
3c6e c3 :3 0 c4
:3 0 3c36 3c37 0
c3 :3 0 d3 :3 0
3c39 3c3a 0 e7
:2 0 2 :4 0 2b6
:2 0 d5 :2 0 2a
:2 0 2199 3c3f 3c41
:3 0 219c 3c3b 3c43
3c38 3c45 3c44 :2 0
c6 :3 0 d6 :3 0
3c47 3c48 0 c8
:4 0 2b7 :4 0 cb
:2 0 cc :3 0 21a0
3c4c 3c4e :3 0 cb
:2 0 ea :4 0 21a3
3c50 3c52 :3 0 cb
:2 0 eb :4 0 21a6
3c54 3c56 :3 0 cb
:2 0 e4 :3 0 21a9
3c58 3c5a :3 0 cb
:2 0 ec :4 0 21ac
3c5c 3c5e :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 21af 3c61 3c65
21b3 3c60 3c67 :3 0
21b6 3c49 3c69 :2 0
3c6b 21b9 3c6c 3c46
3c6b 0 3c6d 21bb
0 3c6e 21bd 3c71
:3 0 3c71 0 3c71
3c70 3c6e 3c6f :6 0
3c73 ac :3 0 21c0
3c75 21c2 3c74 3c73
:2 0 3c76 21c4 :2 0
3c78 0 3c78 3c77
3c25 3c76 :6 0 3ce7
6f :3 0 c3 :3 0
c4 :3 0 3c7a 3c7b
0 c3 :3 0 d3
:3 0 3c7d 3c7e 0
18 :2 0 2 :4 0
2b8 :2 0 d5 :2 0
2a :2 0 21c7 3c83
3c85 :3 0 21ca 3c7f
3c87 3c7c 3c89 3c88
:2 0 c6 :3 0 d6
:3 0 3c8b 3c8c 0
c8 :4 0 2b9 :4 0
cb :2 0 cc :3 0
21ce 3c90 3c92 :3 0
cb :2 0 ea :4 0
21d1 3c94 3c96 :3 0
cb :2 0 f1 :4 0
21d4 3c98 3c9a :3 0
cb :2 0 84 :3 0
d0 :3 0 b0 :3 0
1bc :3 0 3c9f 3ca0
0 21d7 3c9e 3ca2
d1 :4 0 21d9 3c9d
3ca5 21dc 3c9c 3ca7
:3 0 cb :2 0 f3
:4 0 21df 3ca9 3cab
:3 0 cb :2 0 ce
:4 0 21e2 3cad 3caf
:3 0 cb :2 0 1c
:3 0 21e5 3cb1 3cb3
:3 0 cb :2 0 ce
:4 0 21e8 3cb5 3cb7
:3 0 21eb 3c8d 3cb9
:2 0 3cbb 21ee 3cbc
3c8a 3cbb 0 3cbd
21f0 0 3ce7 c3
:3 0 c4 :3 0 3cbe
3cbf 0 c3 :3 0
d3 :3 0 3cc1 3cc2
0 2a :2 0 2
:4 0 2ba :2 0 d5
:2 0 2a :2 0 21f2
3cc7 3cc9 :3 0 21f5
3cc3 3ccb 3cc0 3ccd
3ccc :2 0 c6 :3 0
d6 :3 0 3ccf 3cd0
0 c8 :4 0 2bb
:4 0 cb :2 0 cc
:3 0 21f9 3cd4 3cd6
:3 0 21fc 3cd1 3cd8
:2 0 3cda 21ff 3cdb
3cce 3cda 0 3cdc
2201 0 3ce7 f6
:3 0 f7 :3 0 3cdd
3cde 0 b0 :3 0
1bc :3 0 3ce0 3ce1
0 1c :3 0 f8
:3 0 2203 3cdf 3ce5
:2 0 3ce7 2207 3ce8
3c1b 3ce7 0 3cea
3b45 3c12 0 3cea
220c 0 419d ae
:3 0 186 :3 0 187
:3 0 186 :3 0 39
:3 0 8c :3 0 20
:4 0 2bc 1 :8 0
3d08 e0 :4 0 3cf3
:3 0 f8 :3 0 da
:2 0 2211 3cf6 3cf7
:3 0 15 :3 0 e1
:3 0 3cf9 3cfa 0
3d :2 0 3cfb 3cfc
0 3cfe 2214 3d05
15 :3 0 e1 :3 0
3cff 3d00 0 e2
:2 0 3d01 3d02 0
3d04 2216 3d06 3cf8
3cfe 0 3d07 0
3d04 0 3d07 2218
0 3d08 221b 3d5b
e5 :3 0 15 :3 0
e1 :3 0 3d0a 3d0b
0 e4 :3 0 3d0c
3d0d 0 3d0f 221e
3d11 2220 3d10 3d0f
:2 0 3d59 e6 :3 0
15 :3 0 e1 :3 0
3d14 3d15 0 e4
:3 0 3d16 3d17 0
3d51 c3 :3 0 c4
:3 0 3d19 3d1a 0
c3 :3 0 d3 :3 0
3d1c 3d1d 0 e7
:2 0 2 :4 0 2bd
:2 0 d5 :2 0 2a
:2 0 2222 3d22 3d24
:3 0 2225 3d1e 3d26
3d1b 3d28 3d27 :2 0
c6 :3 0 d6 :3 0
3d2a 3d2b 0 c8
:4 0 2be :4 0 cb
:2 0 cc :3 0 2229
3d2f 3d31 :3 0 cb
:2 0 ea :4 0 222c
3d33 3d35 :3 0 cb
:2 0 eb :4 0 222f
3d37 3d39 :3 0 cb
:2 0 e4 :3 0 2232
3d3b 3d3d :3 0 cb
:2 0 ec :4 0 2235
3d3f 3d41 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 2238 3d44 3d48
223c 3d43 3d4a :3 0
223f 3d2c 3d4c :2 0
3d4e 2242 3d4f 3d29
3d4e 0 3d50 2244
0 3d51 2246 3d54
:3 0 3d54 0 3d54
3d53 3d51 3d52 :6 0
3d56 ae :3 0 2249
3d58 224b 3d57 3d56
:2 0 3d59 224d :2 0
3d5b 0 3d5b 3d5a
3d08 3d59 :6 0 419d
6f :3 0 252 :3 0
1d :3 0 2bf :3 0
8c :3 0 2bf :3 0
1f1 :3 0 2bf :3 0
1f2 :3 0 2bf :3 0
1f3 :3 0 2bf :3 0
1f4 :3 0 2c0 :3 0
2bf :3 0 11b :3 0
12f :3 0 2c1 :3 0
2bf :3 0 8c :3 0
2c1 :3 0 12e :3 0
2c1 :3 0 130 :3 0
23 :4 0 2c2 1
:8 0 3d8b e0 :4 0
3d76 :3 0 f8 :3 0
da :2 0 2252 3d79
3d7a :3 0 15 :3 0
e1 :3 0 3d7c 3d7d
0 3d :2 0 3d7e
3d7f 0 3d81 2255
3d88 15 :3 0 e1
:3 0 3d82 3d83 0
e2 :2 0 3d84 3d85
0 3d87 2257 3d89
3d7b 3d81 0 3d8a
0 3d87 0 3d8a
2259 0 3d8b 225c
3dde e5 :3 0 15
:3 0 e1 :3 0 3d8d
3d8e 0 e4 :3 0
3d8f 3d90 0 3d92
225f 3d94 2261 3d93
3d92 :2 0 3ddc e6
:3 0 15 :3 0 e1
:3 0 3d97 3d98 0
e4 :3 0 3d99 3d9a
0 3dd4 c3 :3 0
c4 :3 0 3d9c 3d9d
0 c3 :3 0 d3
:3 0 3d9f 3da0 0
e7 :2 0 2 :4 0
2c3 :2 0 d5 :2 0
2a :2 0 2263 3da5
3da7 :3 0 2266 3da1
3da9 3d9e 3dab 3daa
:2 0 c6 :3 0 d6
:3 0 3dad 3dae 0
c8 :4 0 2c4 :4 0
cb :2 0 cc :3 0
226a 3db2 3db4 :3 0
cb :2 0 ea :4 0
226d 3db6 3db8 :3 0
cb :2 0 eb :4 0
2270 3dba 3dbc :3 0
cb :2 0 e4 :3 0
2273 3dbe 3dc0 :3 0
cb :2 0 ec :4 0
2276 3dc2 3dc4 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 2279 3dc7
3dcb 227d 3dc6 3dcd
:3 0 2280 3daf 3dcf
:2 0 3dd1 2283 3dd2
3dac 3dd1 0 3dd3
2285 0 3dd4 2287
3dd7 :3 0 3dd7 0
3dd7 3dd6 3dd4 3dd5
:6 0 3dd9 b0 :3 0
228a 3ddb 228c 3dda
3dd9 :2 0 3ddc 228e
:2 0 3dde 0 3dde
3ddd 3d8b 3ddc :6 0
419d 6f :3 0 252
:3 0 1d :3 0 12e
:3 0 1f1 :3 0 1f2
:3 0 1f3 :3 0 1f4
:3 0 2c5 :3 0 252
:3 0 98 :3 0 23
:4 0 2c6 1 :8 0
3e01 e0 :4 0 3dec
:3 0 f8 :3 0 da
:2 0 2293 3def 3df0
:3 0 15 :3 0 e1
:3 0 3df2 3df3 0
3d :2 0 3df4 3df5
0 3df7 2296 3dfe
15 :3 0 e1 :3 0
3df8 3df9 0 e2
:2 0 3dfa 3dfb 0
3dfd 2298 3dff 3df1
3df7 0 3e00 0
3dfd 0 3e00 229a
0 3e01 229d 3e54
e5 :3 0 15 :3 0
e1 :3 0 3e03 3e04
0 e4 :3 0 3e05
3e06 0 3e08 22a0
3e0a 22a2 3e09 3e08
:2 0 3e52 e6 :3 0
15 :3 0 e1 :3 0
3e0d 3e0e 0 e4
:3 0 3e0f 3e10 0
3e4a c3 :3 0 c4
:3 0 3e12 3e13 0
c3 :3 0 d3 :3 0
3e15 3e16 0 e7
:2 0 2 :4 0 2c7
:2 0 d5 :2 0 2a
:2 0 22a4 3e1b 3e1d
:3 0 22a7 3e17 3e1f
3e14 3e21 3e20 :2 0
c6 :3 0 d6 :3 0
3e23 3e24 0 c8
:4 0 2c8 :4 0 cb
:2 0 cc :3 0 22ab
3e28 3e2a :3 0 cb
:2 0 ea :4 0 22ae
3e2c 3e2e :3 0 cb
:2 0 eb :4 0 22b1
3e30 3e32 :3 0 cb
:2 0 e4 :3 0 22b4
3e34 3e36 :3 0 cb
:2 0 ec :4 0 22b7
3e38 3e3a :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 22ba 3e3d 3e41
22be 3e3c 3e43 :3 0
22c1 3e25 3e45 :2 0
3e47 22c4 3e48 3e22
3e47 0 3e49 22c6
0 3e4a 22c8 3e4d
:3 0 3e4d 0 3e4d
3e4c 3e4a 3e4b :6 0
3e4f b2 :3 0 22cb
3e51 22cd 3e50 3e4f
:2 0 3e52 22cf :2 0
3e54 0 3e54 3e53
3e01 3e52 :6 0 419d
6f :3 0 1ef :3 0
1d :3 0 1f0 :3 0
1f1 :3 0 1f2 :3 0
1f3 :3 0 1f4 :3 0
1f5 :3 0 1ef :3 0
98 :3 0 23 :4 0
2c9 1 :8 0 3e77
e0 :4 0 3e62 :3 0
f8 :3 0 da :2 0
22d4 3e65 3e66 :3 0
15 :3 0 e1 :3 0
3e68 3e69 0 3d
:2 0 3e6a 3e6b 0
3e6d 22d7 3e74 15
:3 0 e1 :3 0 3e6e
3e6f 0 e2 :2 0
3e70 3e71 0 3e73
22d9 3e75 3e67 3e6d
0 3e76 0 3e73
0 3e76 22db 0
3e77 22de 3eca e5
:3 0 15 :3 0 e1
:3 0 3e79 3e7a 0
e4 :3 0 3e7b 3e7c
0 3e7e 22e1 3e80
22e3 3e7f 3e7e :2 0
3ec8 e6 :3 0 15
:3 0 e1 :3 0 3e83
3e84 0 e4 :3 0
3e85 3e86 0 3ec0
c3 :3 0 c4 :3 0
3e88 3e89 0 c3
:3 0 d3 :3 0 3e8b
3e8c 0 e7 :2 0
2 :4 0 2ca :2 0
d5 :2 0 2a :2 0
22e5 3e91 3e93 :3 0
22e8 3e8d 3e95 3e8a
3e97 3e96 :2 0 c6
:3 0 d6 :3 0 3e99
3e9a 0 c8 :4 0
2cb :4 0 cb :2 0
cc :3 0 22ec 3e9e
3ea0 :3 0 cb :2 0
ea :4 0 22ef 3ea2
3ea4 :3 0 cb :2 0
eb :4 0 22f2 3ea6
3ea8 :3 0 cb :2 0
e4 :3 0 22f5 3eaa
3eac :3 0 cb :2 0
ec :4 0 22f8 3eae
3eb0 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
22fb 3eb3 3eb7 22ff
3eb2 3eb9 :3 0 2302
3e9b 3ebb :2 0 3ebd
2305 3ebe 3e98 3ebd
0 3ebf 2307 0
3ec0 2309 3ec3 :3 0
3ec3 0 3ec3 3ec2
3ec0 3ec1 :6 0 3ec5
b4 :3 0 230c 3ec7
230e 3ec6 3ec5 :2 0
3ec8 2310 :2 0 3eca
0 3eca 3ec9 3e77
3ec8 :6 0 419d 6f
:3 0 4b :3 0 84
:3 0 4b :3 0 3d
:2 0 2313 3ecd 3ed0
d5 :2 0 2a :2 0
2316 3ed2 3ed4 :3 0
3ecc 3ed5 0 419d
43 :3 0 84 :3 0
43 :3 0 3d :2 0
2319 3ed8 3edb d5
:2 0 84 :3 0 3f
:3 0 3d :2 0 231c
3ede 3ee1 231f 3edd
3ee3 :3 0 3ed7 3ee4
0 419d 6c :3 0
2a :4 0 3ee6 3ee7
0 419d 71 :3 0
2cc :3 0 22 :3 0
23 :3 0 6c :3 0
2322 3eea 3eee 3ee9
3eef 0 419d 6c
:3 0 3d :4 0 3ef1
3ef2 0 419d 71
:3 0 2cc :3 0 22
:3 0 21 :3 0 6c
:3 0 2326 3ef5 3ef9
3ef4 3efa 0 419d
24b :3 0 92 :3 0
21 :3 0 91 :3 0
22 :3 0 9a :3 0
1d :4 0 2cd 1
:8 0 3f19 e0 :4 0
3f04 :3 0 f8 :3 0
da :2 0 232c 3f07
3f08 :3 0 15 :3 0
e1 :3 0 3f0a 3f0b
0 3d :2 0 3f0c
3f0d 0 3f0f 232f
3f16 15 :3 0 e1
:3 0 3f10 3f11 0
e2 :2 0 3f12 3f13
0 3f15 2331 3f17
3f09 3f0f 0 3f18
0 3f15 0 3f18
2333 0 3f19 2336
3f6c e5 :3 0 15
:3 0 e1 :3 0 3f1b
3f1c 0 e4 :3 0
3f1d 3f1e 0 3f20
2339 3f22 233b 3f21
3f20 :2 0 3f6a e6
:3 0 15 :3 0 e1
:3 0 3f25 3f26 0
e4 :3 0 3f27 3f28
0 3f62 c3 :3 0
c4 :3 0 3f2a 3f2b
0 c3 :3 0 d3
:3 0 3f2d 3f2e 0
e7 :2 0 2 :4 0
2ce :2 0 d5 :2 0
2a :2 0 233d 3f33
3f35 :3 0 2340 3f2f
3f37 3f2c 3f39 3f38
:2 0 c6 :3 0 d6
:3 0 3f3b 3f3c 0
c8 :4 0 2cf :4 0
cb :2 0 cc :3 0
2344 3f40 3f42 :3 0
cb :2 0 ea :4 0
2347 3f44 3f46 :3 0
cb :2 0 eb :4 0
234a 3f48 3f4a :3 0
cb :2 0 e4 :3 0
234d 3f4c 3f4e :3 0
cb :2 0 ec :4 0
2350 3f50 3f52 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 2353 3f55
3f59 2357 3f54 3f5b
:3 0 235a 3f3d 3f5d
:2 0 3f5f 235d 3f60
3f3a 3f5f 0 3f61
235f 0 3f62 2361
3f65 :3 0 3f65 0
3f65 3f64 3f62 3f63
:6 0 3f67 b6 :3 0
2364 3f69 2366 3f68
3f67 :2 0 3f6a 2368
:2 0 3f6c 0 3f6c
3f6b 3f19 3f6a :6 0
419d 6f :3 0 84
:3 0 62 :3 0 3d
:2 0 236b 3f6e 3f71
da :2 0 3d :2 0
2370 3f73 3f75 :3 0
3f76 :2 0 84 :3 0
61 :3 0 3d :2 0
2373 3f78 3f7b da
:2 0 3d :2 0 2378
3f7d 3f7f :3 0 3f80
:2 0 3f77 3f82 3f81
:2 0 259 :3 0 92
:3 0 21 :3 0 2d0
:3 0 58 :3 0 2d1
:3 0 5c :3 0 2d2
:3 0 5b :3 0 2d3
:3 0 5d :3 0 17a
:3 0 1a :3 0 17b
:3 0 cc :3 0 9a
:3 0 17 :4 0 2d4
1 :8 0 3fab e0
:4 0 3f96 :3 0 f8
:3 0 da :2 0 237d
3f99 3f9a :3 0 15
:3 0 e1 :3 0 3f9c
3f9d 0 3d :2 0
3f9e 3f9f 0 3fa1
2380 3fa8 15 :3 0
e1 :3 0 3fa2 3fa3
0 e2 :2 0 3fa4
3fa5 0 3fa7 2382
3fa9 3f9b 3fa1 0
3faa 0 3fa7 0
3faa 2384 0 3fab
2387 3ffe e5 :3 0
15 :3 0 e1 :3 0
3fad 3fae 0 e4
:3 0 3faf 3fb0 0
3fb2 238a 3fb4 238c
3fb3 3fb2 :2 0 3ffc
e6 :3 0 15 :3 0
e1 :3 0 3fb7 3fb8
0 e4 :3 0 3fb9
3fba 0 3ff4 c3
:3 0 c4 :3 0 3fbc
3fbd 0 c3 :3 0
d3 :3 0 3fbf 3fc0
0 e7 :2 0 2
:4 0 2d5 :2 0 d5
:2 0 2a :2 0 238e
3fc5 3fc7 :3 0 2391
3fc1 3fc9 3fbe 3fcb
3fca :2 0 c6 :3 0
d6 :3 0 3fcd 3fce
0 c8 :4 0 2d6
:4 0 cb :2 0 cc
:3 0 2395 3fd2 3fd4
:3 0 cb :2 0 ea
:4 0 2398 3fd6 3fd8
:3 0 cb :2 0 eb
:4 0 239b 3fda 3fdc
:3 0 cb :2 0 e4
:3 0 239e 3fde 3fe0
:3 0 cb :2 0 ec
:4 0 23a1 3fe2 3fe4
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 23a4
3fe7 3feb 23a8 3fe6
3fed :3 0 23ab 3fcf
3fef :2 0 3ff1 23ae
3ff2 3fcc 3ff1 0
3ff3 23b0 0 3ff4
23b2 3ff7 :3 0 3ff7
0 3ff7 3ff6 3ff4
3ff5 :6 0 3ff9 b8
:3 0 23b5 3ffb 23b7
3ffa 3ff9 :2 0 3ffc
23b9 :2 0 3ffe 0
3ffe 3ffd 3fab 3ffc
:6 0 4001 6f :3 0
141 :3 0 23bc 419a
84 :3 0 62 :3 0
3d :2 0 23be 4002
4005 da :2 0 3d
:2 0 23c3 4007 4009
:3 0 259 :3 0 92
:3 0 21 :3 0 2d0
:3 0 58 :3 0 2d1
:3 0 5c :3 0 17a
:3 0 1a :3 0 17b
:3 0 cc :3 0 9a
:3 0 17 :4 0 2d7
1 :8 0 402e e0
:4 0 4019 :3 0 f8
:3 0 da :2 0 23c8
401c 401d :3 0 15
:3 0 e1 :3 0 401f
4020 0 3d :2 0
4021 4022 0 4024
23cb 402b 15 :3 0
e1 :3 0 4025 4026
0 e2 :2 0 4027
4028 0 402a 23cd
402c 401e 4024 0
402d 0 402a 0
402d 23cf 0 402e
23d2 4081 e5 :3 0
15 :3 0 e1 :3 0
4030 4031 0 e4
:3 0 4032 4033 0
4035 23d5 4037 23d7
4036 4035 :2 0 407f
e6 :3 0 15 :3 0
e1 :3 0 403a 403b
0 e4 :3 0 403c
403d 0 4077 c3
:3 0 c4 :3 0 403f
4040 0 c3 :3 0
d3 :3 0 4042 4043
0 e7 :2 0 2
:4 0 2d8 :2 0 d5
:2 0 2a :2 0 23d9
4048 404a :3 0 23dc
4044 404c 4041 404e
404d :2 0 c6 :3 0
d6 :3 0 4050 4051
0 c8 :4 0 2d9
:4 0 cb :2 0 cc
:3 0 23e0 4055 4057
:3 0 cb :2 0 ea
:4 0 23e3 4059 405b
:3 0 cb :2 0 eb
:4 0 23e6 405d 405f
:3 0 cb :2 0 e4
:3 0 23e9 4061 4063
:3 0 cb :2 0 ec
:4 0 23ec 4065 4067
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 23ef
406a 406e 23f3 4069
4070 :3 0 23f6 4052
4072 :2 0 4074 23f9
4075 404f 4074 0
4076 23fb 0 4077
23fd 407a :3 0 407a
0 407a 4079 4077
4078 :6 0 407c ba
:3 0 2400 407e 2402
407d 407c :2 0 407f
2404 :2 0 4081 0
4081 4080 402e 407f
:6 0 408b 6f :3 0
5b :3 0 84 :3 0
75 :3 0 3d :2 0
2407 4084 4087 4083
4088 0 408b 141
:3 0 240a 408c 400a
408b 0 419c 84
:3 0 61 :3 0 3d
:2 0 240d 408d 4090
da :2 0 3d :2 0
2412 4092 4094 :3 0
259 :3 0 92 :3 0
21 :3 0 2d2 :3 0
5b :3 0 2d3 :3 0
5d :3 0 17a :3 0
1a :3 0 17b :3 0
cc :3 0 9a :3 0
17 :4 0 2da 1
:8 0 40b9 e0 :4 0
40a4 :3 0 f8 :3 0
da :2 0 2417 40a7
40a8 :3 0 15 :3 0
e1 :3 0 40aa 40ab
0 3d :2 0 40ac
40ad 0 40af 241a
40b6 15 :3 0 e1
:3 0 40b0 40b1 0
e2 :2 0 40b2 40b3
0 40b5 241c 40b7
40a9 40af 0 40b8
0 40b5 0 40b8
241e 0 40b9 2421
410c e5 :3 0 15
:3 0 e1 :3 0 40bb
40bc 0 e4 :3 0
40bd 40be 0 40c0
2424 40c2 2426 40c1
40c0 :2 0 410a e6
:3 0 15 :3 0 e1
:3 0 40c5 40c6 0
e4 :3 0 40c7 40c8
0 4102 c3 :3 0
c4 :3 0 40ca 40cb
0 c3 :3 0 d3
:3 0 40cd 40ce 0
e7 :2 0 2 :4 0
2db :2 0 d5 :2 0
2a :2 0 2428 40d3
40d5 :3 0 242b 40cf
40d7 40cc 40d9 40d8
:2 0 c6 :3 0 d6
:3 0 40db 40dc 0
c8 :4 0 2dc :4 0
cb :2 0 cc :3 0
242f 40e0 40e2 :3 0
cb :2 0 ea :4 0
2432 40e4 40e6 :3 0
cb :2 0 eb :4 0
2435 40e8 40ea :3 0
cb :2 0 e4 :3 0
2438 40ec 40ee :3 0
cb :2 0 ec :4 0
243b 40f0 40f2 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 243e 40f5
40f9 2442 40f4 40fb
:3 0 2445 40dd 40fd
:2 0 40ff 2448 4100
40da 40ff 0 4101
244a 0 4102 244c
4105 :3 0 4105 0
4105 4104 4102 4103
:6 0 4107 bc :3 0
244f 4109 2451 4108
4107 :2 0 410a 2453
:2 0 410c 0 410c
410b 40b9 410a :6 0
4115 6f :3 0 58
:3 0 84 :3 0 74
:3 0 3d :2 0 2456
410f 4112 410e 4113
0 4115 2459 4116
4095 4115 0 419c
259 :3 0 92 :3 0
21 :3 0 17a :3 0
1a :3 0 17b :3 0
cc :3 0 9a :3 0
17 :4 0 2dd 1
:8 0 4136 e0 :4 0
4121 :3 0 f8 :3 0
da :2 0 245e 4124
4125 :3 0 15 :3 0
e1 :3 0 4127 4128
0 3d :2 0 4129
412a 0 412c 2461
4133 15 :3 0 e1
:3 0 412d 412e 0
e2 :2 0 412f 4130
0 4132 2463 4134
4126 412c 0 4135
0 4132 0 4135
2465 0 4136 2468
4189 e5 :3 0 15
:3 0 e1 :3 0 4138
4139 0 e4 :3 0
413a 413b 0 413d
246b 413f 246d 413e
413d :2 0 4187 e6
:3 0 15 :3 0 e1
:3 0 4142 4143 0
e4 :3 0 4144 4145
0 417f c3 :3 0
c4 :3 0 4147 4148
0 c3 :3 0 d3
:3 0 414a 414b 0
e7 :2 0 2 :4 0
2de :2 0 d5 :2 0
2a :2 0 246f 4150
4152 :3 0 2472 414c
4154 4149 4156 4155
:2 0 c6 :3 0 d6
:3 0 4158 4159 0
c8 :4 0 2df :4 0
cb :2 0 cc :3 0
2476 415d 415f :3 0
cb :2 0 ea :4 0
2479 4161 4163 :3 0
cb :2 0 eb :4 0
247c 4165 4167 :3 0
cb :2 0 e4 :3 0
247f 4169 416b :3 0
cb :2 0 ec :4 0
2482 416d 416f :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 2485 4172
4176 2489 4171 4178
:3 0 248c 415a 417a
:2 0 417c 248f 417d
4157 417c 0 417e
2491 0 417f 2493
4182 :3 0 4182 0
4182 4181 417f 4180
:6 0 4184 be :3 0
2496 4186 2498 4185
4184 :2 0 4187 249a
:2 0 4189 0 4189
4188 4136 4187 :6 0
4199 6f :3 0 58
:3 0 84 :3 0 74
:3 0 3d :2 0 249d
418c 418f 418b 4190
0 4199 5b :3 0
84 :3 0 75 :3 0
3d :2 0 24a0 4193
4196 4192 4197 0
4199 24a3 419b 3f83
4001 0 419c 0
4199 0 419c 24a7
0 419d 24ac 419e
3330 419d 0 419f
24c7 0 41a0 24c9
41a1 2d8c 41a0 0
41a2 24ce 0 41a3
24d0 41a5 2a0b 2ca9
0 41a6 0 41a3
0 41a6 24d4 0
41b1 15 :3 0 e1
:3 0 41a7 41a8 0
2e0 :3 0 22 :3 0
23 :3 0 1a :3 0
24d7 41aa 41ae 41a9
41af 0 41b1 24db
41b3 242 :4 0 41b1
:4 0 4e8b e3 :3 0
96 :4 0 41b7 :2 0
41bd 41b5 0 15
:3 0 e1 :3 0 41b8
41b9 0 e4 :3 0
41ba 41bb 0 41bd
24e1 4210 e5 :3 0
15 :3 0 e1 :3 0
41bf 41c0 0 e4
:3 0 41c1 41c2 0
41c4 24e4 41c6 24e6
41c5 41c4 :2 0 420e
e6 :3 0 15 :3 0
e1 :3 0 41c9 41ca
0 e4 :3 0 41cb
41cc 0 4206 c3
:3 0 c4 :3 0 41ce
41cf 0 c3 :3 0
d3 :3 0 41d1 41d2
0 e7 :2 0 2
:4 0 2e1 :2 0 d5
:2 0 2a :2 0 24e8
41d7 41d9 :3 0 24eb
41d3 41db 41d0 41dd
41dc :2 0 c6 :3 0
d6 :3 0 41df 41e0
0 c8 :4 0 2e2
:4 0 cb :2 0 cc
:3 0 24ef 41e4 41e6
:3 0 cb :2 0 ea
:4 0 24f2 41e8 41ea
:3 0 cb :2 0 eb
:4 0 24f5 41ec 41ee
:3 0 cb :2 0 e4
:3 0 24f8 41f0 41f2
:3 0 cb :2 0 ec
:4 0 24fb 41f4 41f6
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 24fe
41f9 41fd 2502 41f8
41ff :3 0 2505 41e1
4201 :2 0 4203 2508
4204 41de 4203 0
4205 250a 0 4206
250c 4209 :3 0 4209
0 4209 4208 4206
4207 :6 0 420b c0
:3 0 250f 420d 2511
420c 420b :2 0 420e
2513 :2 0 4210 0
4210 420f 41bd 420e
:6 0 4e8b 1 :3 0
56 :3 0 3d :2 0
4212 4213 0 4e8b
84 :3 0 4b :3 0
3d :2 0 2516 4215
4218 10b :2 0 3d
:2 0 251b 421a 421c
:3 0 3f :3 0 9
:3 0 421e 421f 0
4735 84 :3 0 3f
:3 0 3d :2 0 251e
4221 4224 84 :3 0
2e3 :2 0 43 :3 0
3d :2 0 2521 4226
422a 2526 4227 422c
:3 0 3f :3 0 84
:3 0 3f :3 0 3d
:2 0 2529 422f 4232
1c0 :2 0 84 :3 0
43 :3 0 3d :2 0
252c 4235 4238 252f
4234 423a :3 0 422e
423b 0 464f 84
:3 0 3f :3 0 3d
:2 0 2532 423d 4240
10b :2 0 3d :2 0
2537 4242 4244 :3 0
4245 :2 0 84 :3 0
4c :3 0 3d :2 0
253a 4247 424a 10b
:2 0 3d :2 0 253f
424c 424e :3 0 424f
:2 0 4246 4251 4250
:2 0 1e :3 0 166
:2 0 2542 4254 4255
:3 0 4256 :2 0 4257
:2 0 4252 4259 4258
:2 0 24 :3 0 7
:3 0 425b 425c 0
464c 26 :3 0 8
:3 0 425e 425f 0
464c 44 :3 0 b
:3 0 4261 4262 0
464c 45 :3 0 10
:3 0 4264 4265 0
464c 46 :3 0 d
:3 0 4267 4268 0
464c 47 :3 0 e
:3 0 426a 426b 0
464c 4e :3 0 c
:3 0 426d 426e 0
464c 48 :3 0 f
:3 0 4270 4271 0
464c 84 :3 0 48
:3 0 3d :2 0 2544
4273 4276 da :2 0
3d :2 0 2549 4278
427a :3 0 84 :3 0
47 :3 0 3d :2 0
254c 427c 427f 13c
:2 0 e2 :2 0 2551
4281 4283 :3 0 427b
4285 4284 :2 0 44
:3 0 3d :2 0 4287
4288 0 428b 141
:3 0 2554 429e 84
:3 0 47 :3 0 3d
:2 0 2556 428c 428f
da :2 0 e2 :2 0
255b 4291 4293 :3 0
7e :3 0 84 :3 0
44 :3 0 3d :2 0
255e 4296 4299 4295
429a 0 429c 2561
429d 4294 429c 0
429f 4286 428b 0
429f 2563 0 464c
6b :3 0 202 :3 0
31 :3 0 32 :3 0
84 :3 0 46 :3 0
3d :2 0 2566 42a4
42a7 84 :3 0 47
:3 0 3d :2 0 2569
42a9 42ac 84 :3 0
52 :3 0 3d :2 0
256c 42ae 42b1 84
:3 0 4e :3 0 3d
:2 0 256f 42b3 42b6
84 :3 0 48 :3 0
3d :2 0 2572 42b8
42bb 84 :3 0 45
:3 0 3d :2 0 2575
42bd 42c0 84 :3 0
4f :3 0 3d :2 0
2578 42c2 42c5 84
:3 0 50 :3 0 3d
:2 0 257b 42c7 42ca
44 :3 0 3f :3 0
49 :3 0 257e 42a1
42cf 42a0 42d0 0
464c 84 :3 0 6b
:3 0 3d :2 0 258c
42d2 42d5 b0 :3 0
da :2 0 203 :3 0
42d7 42d9 0 2591
42d8 42db :3 0 c3
:3 0 c4 :3 0 42dd
42de 0 c3 :3 0
d3 :3 0 42e0 42e1
0 204 :2 0 2
:4 0 2e4 :2 0 d5
:2 0 2a :2 0 2594
42e6 42e8 :3 0 2597
42e2 42ea 42df 42ec
42eb :2 0 c6 :3 0
d6 :3 0 42ee 42ef
0 c8 :4 0 2e5
:4 0 cb :2 0 cc
:3 0 259b 42f3 42f5
:3 0 cb :2 0 ea
:4 0 259e 42f7 42f9
:3 0 cb :2 0 f1
:4 0 25a1 42fb 42fd
:3 0 cb :2 0 84
:3 0 d0 :3 0 b0
:3 0 203 :3 0 4302
4303 0 25a4 4301
4305 d1 :4 0 25a6
4300 4308 25a9 42ff
430a :3 0 25ac 42f0
430c :2 0 430e 25af
430f 42ed 430e 0
4310 25b1 0 433a
c3 :3 0 c4 :3 0
4311 4312 0 c3
:3 0 d3 :3 0 4314
4315 0 2a :2 0
2 :4 0 2e6 :2 0
d5 :2 0 2a :2 0
25b3 431a 431c :3 0
25b6 4316 431e 4313
4320 431f :2 0 c6
:3 0 d6 :3 0 4322
4323 0 c8 :4 0
2e7 :4 0 cb :2 0
cc :3 0 25ba 4327
4329 :3 0 25bd 4324
432b :2 0 432d 25c0
432e 4321 432d 0
432f 25c2 0 433a
f6 :3 0 209 :3 0
4330 4331 0 b0
:3 0 203 :3 0 4333
4334 0 f8 :3 0
25c4 4332 4337 :2 0
433a 141 :3 0 25c7
43b3 84 :3 0 6b
:3 0 3d :2 0 25cb
433b 433e 13c :2 0
3d :2 0 25d0 4340
4342 :3 0 c3 :3 0
c4 :3 0 4344 4345
0 c3 :3 0 d3
:3 0 4347 4348 0
18 :2 0 2 :4 0
2e8 :2 0 d5 :2 0
2a :2 0 25d3 434d
434f :3 0 25d6 4349
4351 4346 4353 4352
:2 0 c6 :3 0 d6
:3 0 4355 4356 0
c8 :4 0 2e9 :4 0
cb :2 0 cc :3 0
25da 435a 435c :3 0
cb :2 0 ea :4 0
25dd 435e 4360 :3 0
cb :2 0 f1 :4 0
25e0 4362 4364 :3 0
cb :2 0 84 :3 0
d0 :3 0 6b :3 0
25e3 4368 436a d1
:4 0 25e5 4367 436d
25e8 4366 436f :3 0
cb :2 0 f3 :4 0
25eb 4371 4373 :3 0
cb :2 0 ce :4 0
25ee 4375 4377 :3 0
cb :2 0 22 :3 0
25f1 4379 437b :3 0
cb :2 0 ce :4 0
25f4 437d 437f :3 0
25f7 4357 4381 :2 0
4383 25fa 4384 4354
4383 0 4385 25fc
0 43b1 c3 :3 0
c4 :3 0 4386 4387
0 c3 :3 0 d3
:3 0 4389 438a 0
2a :2 0 2 :4 0
2ea :2 0 d5 :2 0
2a :2 0 25fe 438f
4391 :3 0 2601 438b
4393 4388 4395 4394
:2 0 c6 :3 0 d6
:3 0 4397 4398 0
c8 :4 0 2eb :4 0
cb :2 0 cc :3 0
2605 439c 439e :3 0
2608 4399 43a0 :2 0
43a2 260b 43a3 4396
43a2 0 43a4 260d
0 43b1 f6 :3 0
f7 :3 0 43a5 43a6
0 84 :3 0 6b
:3 0 3d :2 0 260f
43a8 43ab 22 :3 0
f8 :3 0 2612 43a7
43af :2 0 43b1 2616
43b2 4343 43b1 0
43b4 42dc 433a 0
43b4 261a 0 464c
2ec :3 0 2ed :3 0
2ee :4 0 43b7 :4 0
43b8 :2 0 464c 15
:3 0 e1 :3 0 43ba
43bb 0 3d :2 0
43bc 43bd 0 464c
2ef :3 0 1c9 :3 0
1b0 :3 0 1b1 :3 0
1b2 :3 0 91 :3 0
22 :3 0 2f0 :3 0
1e :3 0 1c9 :4 0
2f1 1 :8 0 43df
e0 :4 0 43ca :3 0
f8 :3 0 da :2 0
261f 43cd 43ce :3 0
15 :3 0 e1 :3 0
43d0 43d1 0 3d
:2 0 43d2 43d3 0
43d5 2622 43dc 15
:3 0 e1 :3 0 43d6
43d7 0 e2 :2 0
43d8 43d9 0 43db
2624 43dd 43cf 43d5
0 43de 0 43db
0 43de 2626 0
43df 2629 4432 e5
:3 0 15 :3 0 e1
:3 0 43e1 43e2 0
e4 :3 0 43e3 43e4
0 43e6 262c 43e8
262e 43e7 43e6 :2 0
4430 e6 :3 0 15
:3 0 e1 :3 0 43eb
43ec 0 e4 :3 0
43ed 43ee 0 4428
c3 :3 0 c4 :3 0
43f0 43f1 0 c3
:3 0 d3 :3 0 43f3
43f4 0 e7 :2 0
2 :4 0 2f2 :2 0
d5 :2 0 2a :2 0
2630 43f9 43fb :3 0
2633 43f5 43fd 43f2
43ff 43fe :2 0 c6
:3 0 d6 :3 0 4401
4402 0 c8 :4 0
2f3 :4 0 cb :2 0
cc :3 0 2637 4406
4408 :3 0 cb :2 0
ea :4 0 263a 440a
440c :3 0 cb :2 0
eb :4 0 263d 440e
4410 :3 0 cb :2 0
e4 :3 0 2640 4412
4414 :3 0 cb :2 0
ec :4 0 2643 4416
4418 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
2646 441b 441f 264a
441a 4421 :3 0 264d
4403 4423 :2 0 4425
2650 4426 4400 4425
0 4427 2652 0
4428 2654 442b :3 0
442b 0 442b 442a
4428 4429 :6 0 442d
c2 :3 0 2657 442f
2659 442e 442d :2 0
4430 265b :2 0 4432
0 4432 4431 43df
4430 :6 0 464c 1
:3 0 26 :3 0 de
:4 0 4434 4435 0
464c 4a :3 0 3d
:2 0 4437 4438 0
464c 51 :3 0 3d
:2 0 443a 443b 0
464c 40 :3 0 3d
:2 0 443d 443e 0
464c 41 :3 0 2a
:2 0 4440 4441 0
464c 84 :3 0 44
:3 0 3d :2 0 265e
4443 4446 10b :2 0
3d :2 0 2663 4448
444a :3 0 84 :3 0
7e :3 0 3d :2 0
2666 444c 444f 13c
:2 0 e2 :2 0 266b
4451 4453 :3 0 84
:3 0 7e :3 0 3d
:2 0 266e 4455 4458
13c :2 0 3d :2 0
2673 445a 445c :3 0
4454 445e 445d :2 0
445f :2 0 444b 4461
4460 :2 0 84 :3 0
44 :3 0 3d :2 0
2676 4463 4466 da
:2 0 3d :2 0 267b
4468 446a :3 0 84
:3 0 7e :3 0 3d
:2 0 267e 446c 446f
13c :2 0 e2 :2 0
2683 4471 4473 :3 0
446b 4475 4474 :2 0
84 :3 0 7e :3 0
3d :2 0 2686 4477
447a 13c :2 0 3d
:2 0 268b 447c 447e
:3 0 4476 4480 447f
:2 0 47 :3 0 e2
:2 0 4482 4483 0
448c 44 :3 0 84
:3 0 7e :3 0 3d
:2 0 268e 4486 4489
4485 448a 0 448c
2691 448d 4481 448c
0 448e 2694 0
44a1 44 :3 0 200
:3 0 19b :3 0 4490
4491 0 84 :3 0
44 :3 0 3d :2 0
2696 4493 4496 b0
:3 0 2f4 :3 0 4498
4499 0 2699 4492
449b 448f 449c 0
44a1 4e :3 0 2a
:2 0 449e 449f 0
44a1 269c 44a6 4e
:3 0 3d :2 0 44a2
44a3 0 44a5 26a0
44a7 4462 44a1 0
44a8 0 44a5 0
44a8 26a2 0 464c
6a :3 0 5a :2 0
44a9 44aa 0 464c
66 :3 0 20f :3 0
1c :3 0 22 :3 0
22 :3 0 21 :3 0
6a :3 0 41 :3 0
29 :3 0 29 :3 0
29 :3 0 29 :3 0
24 :3 0 26 :3 0
49 :3 0 3f :3 0
44 :3 0 46 :3 0
47 :3 0 40 :3 0
52 :3 0 2e :3 0
4a :3 0 4e :3 0
48 :3 0 45 :3 0
51 :3 0 4f :3 0
53 :3 0 1a :3 0
3d :2 0 3d :2 0
5a :2 0 5a :2 0
26a5 44ad 44ce 44ac
44cf 0 464c 84
:3 0 66 :3 0 3d
:2 0 26c6 44d1 44d4
da :2 0 210 :2 0
26cb 44d6 44d8 :3 0
c3 :3 0 c4 :3 0
44da 44db 0 c3
:3 0 d3 :3 0 44dd
44de 0 18 :2 0
2 :4 0 2f5 :2 0
d5 :2 0 2a :2 0
26ce 44e3 44e5 :3 0
26d1 44df 44e7 44dc
44e9 44e8 :2 0 c6
:3 0 d6 :3 0 44eb
44ec 0 c8 :4 0
2f6 :4 0 cb :2 0
cc :3 0 26d5 44f0
44f2 :3 0 cb :2 0
ea :4 0 26d8 44f4
44f6 :3 0 cb :2 0
f1 :4 0 26db 44f8
44fa :3 0 cb :2 0
84 :3 0 d0 :3 0
b0 :3 0 213 :3 0
44ff 4500 0 26de
44fe 4502 d1 :4 0
26e0 44fd 4505 26e3
44fc 4507 :3 0 cb
:2 0 f3 :4 0 26e6
4509 450b :3 0 cb
:2 0 ce :4 0 26e9
450d 450f :3 0 cb
:2 0 21 :3 0 26ec
4511 4513 :3 0 cb
:2 0 ce :4 0 26ef
4515 4517 :3 0 26f2
44ed 4519 :2 0 451b
26f5 451c 44ea 451b
0 451d 26f7 0
4548 c3 :3 0 c4
:3 0 451e 451f 0
c3 :3 0 d3 :3 0
4521 4522 0 2a
:2 0 2 :4 0 2f7
:2 0 d5 :2 0 2a
:2 0 26f9 4527 4529
:3 0 26fc 4523 452b
4520 452d 452c :2 0
c6 :3 0 d6 :3 0
452f 4530 0 c8
:4 0 2f8 :4 0 cb
:2 0 cc :3 0 2700
4534 4536 :3 0 2703
4531 4538 :2 0 453a
2706 453b 452e 453a
0 453c 2708 0
4548 f6 :3 0 f7
:3 0 453d 453e 0
b0 :3 0 213 :3 0
4540 4541 0 21
:3 0 f8 :3 0 270a
453f 4545 :2 0 4548
141 :3 0 270e 4649
84 :3 0 66 :3 0
3d :2 0 2712 4549
454c 13c :2 0 3d
:2 0 2717 454e 4550
:3 0 c3 :3 0 c4
:3 0 4552 4553 0
c3 :3 0 d3 :3 0
4555 4556 0 18
:2 0 2 :4 0 2f9
:2 0 d5 :2 0 2a
:2 0 271a 455b 455d
:3 0 271d 4557 455f
4554 4561 4560 :2 0
c6 :3 0 d6 :3 0
4563 4564 0 c8
:4 0 2fa :4 0 cb
:2 0 cc :3 0 2721
4568 456a :3 0 cb
:2 0 ea :4 0 2724
456c 456e :3 0 cb
:2 0 f1 :4 0 2727
4570 4572 :3 0 cb
:2 0 84 :3 0 d0
:3 0 b0 :3 0 1bc
:3 0 4577 4578 0
272a 4576 457a d1
:4 0 272c 4575 457d
272f 4574 457f :3 0
cb :2 0 f3 :4 0
2732 4581 4583 :3 0
cb :2 0 ce :4 0
2735 4585 4587 :3 0
cb :2 0 1c :3 0
2738 4589 458b :3 0
cb :2 0 ce :4 0
273b 458d 458f :3 0
273e 4565 4591 :2 0
4593 2741 4594 4562
4593 0 4595 2743
0 45bf c3 :3 0
c4 :3 0 4596 4597
0 c3 :3 0 d3
:3 0 4599 459a 0
2a :2 0 2 :4 0
2fb :2 0 d5 :2 0
2a :2 0 2745 459f
45a1 :3 0 2748 459b
45a3 4598 45a5 45a4
:2 0 c6 :3 0 d6
:3 0 45a7 45a8 0
c8 :4 0 2fc :4 0
cb :2 0 cc :3 0
274c 45ac 45ae :3 0
274f 45a9 45b0 :2 0
45b2 2752 45b3 45a6
45b2 0 45b4 2754
0 45bf f6 :3 0
f7 :3 0 45b5 45b6
0 b0 :3 0 1bc
:3 0 45b8 45b9 0
1c :3 0 f8 :3 0
2756 45b7 45bd :2 0
45bf 275a 45c0 4551
45bf 0 464b 90
:3 0 98 :3 0 1c
:3 0 92 :3 0 21
:3 0 93 :3 0 a7
:3 0 61 :3 0 58
:3 0 5e :3 0 a7
:3 0 1d4 :3 0 61
:3 0 5c :3 0 5e
:3 0 1d4 :3 0 a8
:3 0 62 :3 0 5b
:3 0 5e :3 0 a8
:3 0 1d5 :3 0 62
:3 0 5d :3 0 5e
:3 0 1d5 :3 0 9a
:3 0 1e :4 0 2fd
1 :8 0 45f3 e0
:4 0 45de :3 0 f8
:3 0 da :2 0 2760
45e1 45e2 :3 0 15
:3 0 e1 :3 0 45e4
45e5 0 3d :2 0
45e6 45e7 0 45e9
2763 45f0 15 :3 0
e1 :3 0 45ea 45eb
0 e2 :2 0 45ec
45ed 0 45ef 2765
45f1 45e3 45e9 0
45f2 0 45ef 0
45f2 2767 0 45f3
276a 4646 e5 :3 0
15 :3 0 e1 :3 0
45f5 45f6 0 e4
:3 0 45f7 45f8 0
45fa 276d 45fc 276f
45fb 45fa :2 0 4644
e6 :3 0 15 :3 0
e1 :3 0 45ff 4600
0 e4 :3 0 4601
4602 0 463c c3
:3 0 c4 :3 0 4604
4605 0 c3 :3 0
d3 :3 0 4607 4608
0 e7 :2 0 2
:4 0 2fe :2 0 d5
:2 0 2a :2 0 2771
460d 460f :3 0 2774
4609 4611 4606 4613
4612 :2 0 c6 :3 0
d6 :3 0 4615 4616
0 c8 :4 0 2ff
:4 0 cb :2 0 cc
:3 0 2778 461a 461c
:3 0 cb :2 0 ea
:4 0 277b 461e 4620
:3 0 cb :2 0 eb
:4 0 277e 4622 4624
:3 0 cb :2 0 e4
:3 0 2781 4626 4628
:3 0 cb :2 0 ec
:4 0 2784 462a 462c
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 2787
462f 4633 278b 462e
4635 :3 0 278e 4617
4637 :2 0 4639 2791
463a 4614 4639 0
463b 2793 0 463c
2795 463f :3 0 463f
0 463f 463e 463c
463d :6 0 4641 c4
:3 0 2798 4643 279a
4642 4641 :2 0 4644
279c :2 0 4646 0
4646 4645 45f3 4644
:6 0 4648 1 :3 0
279f 464a 44d9 4548
0 464b 0 4648
0 464b 27a1 0
464c 27a5 464d 425a
464c 0 464e 27bd
0 464f 27bf 4650
422d 464f 0 4651
27c2 0 4735 13d
:3 0 56 :3 0 af
:3 0 8c :3 0 22
:3 0 92 :3 0 23
:3 0 93 :4 0 300
1 :8 0 4660 15
:3 0 e1 :3 0 465b
465c 0 e4 :3 0
465d 465e 0 4660
27c4 46b3 e5 :3 0
15 :3 0 e1 :3 0
4662 4663 0 e4
:3 0 4664 4665 0
4667 27c7 4669 27c9
4668 4667 :2 0 46b1
e6 :3 0 15 :3 0
e1 :3 0 466c 466d
0 e4 :3 0 466e
466f 0 46a9 c3
:3 0 c4 :3 0 4671
4672 0 c3 :3 0
d3 :3 0 4674 4675
0 e7 :2 0 2
:4 0 301 :2 0 d5
:2 0 2a :2 0 27cb
467a 467c :3 0 27ce
4676 467e 4673 4680
467f :2 0 c6 :3 0
d6 :3 0 4682 4683
0 c8 :4 0 302
:4 0 cb :2 0 cc
:3 0 27d2 4687 4689
:3 0 cb :2 0 ea
:4 0 27d5 468b 468d
:3 0 cb :2 0 eb
:4 0 27d8 468f 4691
:3 0 cb :2 0 e4
:3 0 27db 4693 4695
:3 0 cb :2 0 ec
:4 0 27de 4697 4699
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 27e1
469c 46a0 27e5 469b
46a2 :3 0 27e8 4684
46a4 :2 0 46a6 27eb
46a7 4681 46a6 0
46a8 27ed 0 46a9
27ef 46ac :3 0 46ac
0 46ac 46ab 46a9
46aa :6 0 46ae c6
:3 0 27f2 46b0 27f4
46af 46ae :2 0 46b1
27f6 :2 0 46b3 0
46b3 46b2 4660 46b1
:6 0 4735 1 :3 0
84 :3 0 56 :3 0
3d :2 0 27f9 46b5
46b8 da :2 0 3d
:2 0 27fe 46ba 46bc
:3 0 ae :3 0 186
:3 0 187 :3 0 186
:3 0 39 :3 0 91
:3 0 22 :3 0 8c
:3 0 22 :4 0 303
1 :8 0 46dd e0
:4 0 46c8 :3 0 f8
:3 0 da :2 0 2803
46cb 46cc :3 0 15
:3 0 e1 :3 0 46ce
46cf 0 3d :2 0
46d0 46d1 0 46d3
2806 46da 15 :3 0
e1 :3 0 46d4 46d5
0 e2 :2 0 46d6
46d7 0 46d9 2808
46db 46cd 46d3 0
46dc 0 46d9 0
46dc 280a 0 46dd
280d 4730 e5 :3 0
15 :3 0 e1 :3 0
46df 46e0 0 e4
:3 0 46e1 46e2 0
46e4 2810 46e6 2812
46e5 46e4 :2 0 472e
e6 :3 0 15 :3 0
e1 :3 0 46e9 46ea
0 e4 :3 0 46eb
46ec 0 4726 c3
:3 0 c4 :3 0 46ee
46ef 0 c3 :3 0
d3 :3 0 46f1 46f2
0 e7 :2 0 2
:4 0 304 :2 0 d5
:2 0 2a :2 0 2814
46f7 46f9 :3 0 2817
46f3 46fb 46f0 46fd
46fc :2 0 c6 :3 0
d6 :3 0 46ff 4700
0 c8 :4 0 305
:4 0 cb :2 0 cc
:3 0 281b 4704 4706
:3 0 cb :2 0 ea
:4 0 281e 4708 470a
:3 0 cb :2 0 eb
:4 0 2821 470c 470e
:3 0 cb :2 0 e4
:3 0 2824 4710 4712
:3 0 cb :2 0 ec
:4 0 2827 4714 4716
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 282a
4719 471d 282e 4718
471f :3 0 2831 4701
4721 :2 0 4723 2834
4724 46fe 4723 0
4725 2836 0 4726
2838 4729 :3 0 4729
0 4729 4728 4726
4727 :6 0 472b c8
:3 0 283b 472d 283d
472c 472b :2 0 472e
283f :2 0 4730 0
4730 472f 46dd 472e
:6 0 4732 1 :3 0
2842 4733 46bd 4732
0 4734 2844 0
4735 2846 4736 421d
4735 0 4737 284b
0 4e8b 63 :3 0
3d :2 0 4738 4739
0 4e8b 81 :3 0
166 :2 0 284d 473c
473d :3 0 473e :2 0
ae :3 0 1d8 :3 0
19a :3 0 19b :3 0
1d8 :3 0 b0 :3 0
306 :3 0 8c :3 0
22 :4 0 307 1
:8 0 475f e0 :4 0
474a :3 0 f8 :3 0
da :2 0 2851 474d
474e :3 0 15 :3 0
e1 :3 0 4750 4751
0 3d :2 0 4752
4753 0 4755 2854
475c 15 :3 0 e1
:3 0 4756 4757 0
e2 :2 0 4758 4759
0 475b 2856 475d
474f 4755 0 475e
0 475b 0 475e
2858 0 475f 285b
47b2 e5 :3 0 15
:3 0 e1 :3 0 4761
4762 0 e4 :3 0
4763 4764 0 4766
285e 4768 2860 4767
4766 :2 0 47b0 e6
:3 0 15 :3 0 e1
:3 0 476b 476c 0
e4 :3 0 476d 476e
0 47a8 c3 :3 0
c4 :3 0 4770 4771
0 c3 :3 0 d3
:3 0 4773 4774 0
e7 :2 0 2 :4 0
308 :2 0 d5 :2 0
2a :2 0 2862 4779
477b :3 0 2865 4775
477d 4772 477f 477e
:2 0 c6 :3 0 d6
:3 0 4781 4782 0
c8 :4 0 309 :4 0
cb :2 0 cc :3 0
2869 4786 4788 :3 0
cb :2 0 ea :4 0
286c 478a 478c :3 0
cb :2 0 eb :4 0
286f 478e 4790 :3 0
cb :2 0 e4 :3 0
2872 4792 4794 :3 0
cb :2 0 ec :4 0
2875 4796 4798 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 2878 479b
479f 287c 479a 47a1
:3 0 287f 4783 47a3
:2 0 47a5 2882 47a6
4780 47a5 0 47a7
2884 0 47a8 2886
47ab :3 0 47ab 0
47ab 47aa 47a8 47a9
:6 0 47ad ca :3 0
2889 47af 288b 47ae
47ad :2 0 47b0 288d
:2 0 47b2 0 47b2
47b1 475f 47b0 :6 0
4aeb 1 :3 0 84
:3 0 4c :3 0 3d
:2 0 2890 47b4 47b7
da :2 0 3d :2 0
2895 47b9 47bb :3 0
13d :3 0 56 :3 0
90 :3 0 91 :3 0
22 :3 0 94 :3 0
81 :4 0 30a 1
:8 0 47ca 15 :3 0
e1 :3 0 47c5 47c6
0 e4 :3 0 47c7
47c8 0 47ca 2898
481d e5 :3 0 15
:3 0 e1 :3 0 47cc
47cd 0 e4 :3 0
47ce 47cf 0 47d1
289b 47d3 289d 47d2
47d1 :2 0 481b e6
:3 0 15 :3 0 e1
:3 0 47d6 47d7 0
e4 :3 0 47d8 47d9
0 4813 c3 :3 0
c4 :3 0 47db 47dc
0 c3 :3 0 d3
:3 0 47de 47df 0
e7 :2 0 2 :4 0
30b :2 0 d5 :2 0
2a :2 0 289f 47e4
47e6 :3 0 28a2 47e0
47e8 47dd 47ea 47e9
:2 0 c6 :3 0 d6
:3 0 47ec 47ed 0
c8 :4 0 30c :4 0
cb :2 0 cc :3 0
28a6 47f1 47f3 :3 0
cb :2 0 ea :4 0
28a9 47f5 47f7 :3 0
cb :2 0 eb :4 0
28ac 47f9 47fb :3 0
cb :2 0 e4 :3 0
28af 47fd 47ff :3 0
cb :2 0 ec :4 0
28b2 4801 4803 :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 28b5 4806
480a 28b9 4805 480c
:3 0 28bc 47ee 480e
:2 0 4810 28bf 4811
47eb 4810 0 4812
28c1 0 4813 28c3
4816 :3 0 4816 0
4816 4815 4813 4814
:6 0 4818 cc :3 0
28c6 481a 28c8 4819
4818 :2 0 481b 28ca
:2 0 481d 0 481d
481c 47ca 481b :6 0
490e 1 :3 0 84
:3 0 56 :3 0 3d
:2 0 28cd 481f 4822
da :2 0 3d :2 0
28d2 4824 4826 :3 0
af :3 0 91 :3 0
22 :3 0 8c :3 0
22 :3 0 92 :3 0
23 :3 0 93 :4 0
30d 1 :8 0 4846
e0 :4 0 4831 :3 0
f8 :3 0 da :2 0
28d7 4834 4835 :3 0
15 :3 0 e1 :3 0
4837 4838 0 3d
:2 0 4839 483a 0
483c 28da 4843 15
:3 0 e1 :3 0 483d
483e 0 e2 :2 0
483f 4840 0 4842
28dc 4844 4836 483c
0 4845 0 4842
0 4845 28de 0
4846 28e1 4899 e5
:3 0 15 :3 0 e1
:3 0 4848 4849 0
e4 :3 0 484a 484b
0 484d 28e4 484f
28e6 484e 484d :2 0
4897 e6 :3 0 15
:3 0 e1 :3 0 4852
4853 0 e4 :3 0
4854 4855 0 488f
c3 :3 0 c4 :3 0
4857 4858 0 c3
:3 0 d3 :3 0 485a
485b 0 e7 :2 0
2 :4 0 30e :2 0
d5 :2 0 2a :2 0
28e8 4860 4862 :3 0
28eb 485c 4864 4859
4866 4865 :2 0 c6
:3 0 d6 :3 0 4868
4869 0 c8 :4 0
30f :4 0 cb :2 0
cc :3 0 28ef 486d
486f :3 0 cb :2 0
ea :4 0 28f2 4871
4873 :3 0 cb :2 0
eb :4 0 28f5 4875
4877 :3 0 cb :2 0
e4 :3 0 28f8 4879
487b :3 0 cb :2 0
ec :4 0 28fb 487d
487f :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
28fe 4882 4886 2902
4881 4888 :3 0 2905
486a 488a :2 0 488c
2908 488d 4867 488c
0 488e 290a 0
488f 290c 4892 :3 0
4892 0 4892 4891
488f 4890 :6 0 4894
ce :3 0 290f 4896
2911 4895 4894 :2 0
4897 2913 :2 0 4899
0 4899 4898 4846
4897 :6 0 490b 1
:3 0 1fc :3 0 91
:3 0 22 :3 0 92
:3 0 23 :4 0 1fd
1 :8 0 48b6 e0
:4 0 48a1 :3 0 f8
:3 0 da :2 0 2918
48a4 48a5 :3 0 15
:3 0 e1 :3 0 48a7
48a8 0 3d :2 0
48a9 48aa 0 48ac
291b 48b3 15 :3 0
e1 :3 0 48ad 48ae
0 e2 :2 0 48af
48b0 0 48b2 291d
48b4 48a6 48ac 0
48b5 0 48b2 0
48b5 291f 0 48b6
2922 4909 e5 :3 0
15 :3 0 e1 :3 0
48b8 48b9 0 e4
:3 0 48ba 48bb 0
48bd 2925 48bf 2927
48be 48bd :2 0 4907
e6 :3 0 15 :3 0
e1 :3 0 48c2 48c3
0 e4 :3 0 48c4
48c5 0 48ff c3
:3 0 c4 :3 0 48c7
48c8 0 c3 :3 0
d3 :3 0 48ca 48cb
0 e7 :2 0 2
:4 0 310 :2 0 d5
:2 0 2a :2 0 2929
48d0 48d2 :3 0 292c
48cc 48d4 48c9 48d6
48d5 :2 0 c6 :3 0
d6 :3 0 48d8 48d9
0 c8 :4 0 311
:4 0 cb :2 0 cc
:3 0 2930 48dd 48df
:3 0 cb :2 0 ea
:4 0 2933 48e1 48e3
:3 0 cb :2 0 eb
:4 0 2936 48e5 48e7
:3 0 cb :2 0 e4
:3 0 2939 48e9 48eb
:3 0 cb :2 0 ec
:4 0 293c 48ed 48ef
:3 0 cb :2 0 a0
:3 0 ed :3 0 2a
:2 0 ee :2 0 293f
48f2 48f6 2943 48f1
48f8 :3 0 2946 48da
48fa :2 0 48fc 2949
48fd 48d7 48fc 0
48fe 294b 0 48ff
294d 4902 :3 0 4902
0 4902 4901 48ff
4900 :6 0 4904 d0
:3 0 2950 4906 2952
4905 4904 :2 0 4907
2954 :2 0 4909 0
4909 4908 48b6 4907
:6 0 490b 1 :3 0
2957 490c 4827 490b
0 490d 295a 0
490e 295c 490f 47bc
490e 0 4910 295f
0 4aeb 84 :3 0
56 :3 0 3d :2 0
2961 4911 4914 10b
:2 0 3d :2 0 2966
4916 4918 :3 0 6b
:3 0 312 :2 0 491a
491b 0 4ae8 71
:3 0 21a :3 0 22
:3 0 81 :3 0 6b
:3 0 3d :2 0 3d
:2 0 2969 491e 4924
491d 4925 0 4ae8
a1 :3 0 a2 :3 0
313 :3 0 9f :3 0
a0 :3 0 22c :3 0
9f :3 0 a0 :3 0
22d :3 0 a5 :3 0
a6 :3 0 a7 :3 0
a8 :3 0 31 :3 0
32 :3 0 3f :3 0
2b :3 0 2d :3 0
78 :3 0 79 :3 0
74 :3 0 75 :3 0
90 :3 0 9a :3 0
81 :4 0 314 1
:8 0 4946 15 :3 0
e1 :3 0 4941 4942
0 e4 :3 0 4943
4944 0 4946 296f
4999 e5 :3 0 15
:3 0 e1 :3 0 4948
4949 0 e4 :3 0
494a 494b 0 494d
2972 494f 2974 494e
494d :2 0 4997 e6
:3 0 15 :3 0 e1
:3 0 4952 4953 0
e4 :3 0 4954 4955
0 498f c3 :3 0
c4 :3 0 4957 4958
0 c3 :3 0 d3
:3 0 495a 495b 0
e7 :2 0 2 :4 0
315 :2 0 d5 :2 0
2a :2 0 2976 4960
4962 :3 0 2979 495c
4964 4959 4966 4965
:2 0 c6 :3 0 d6
:3 0 4968 4969 0
c8 :4 0 316 :4 0
cb :2 0 cc :3 0
297d 496d 496f :3 0
cb :2 0 ea :4 0
2980 4971 4973 :3 0
cb :2 0 eb :4 0
2983 4975 4977 :3 0
cb :2 0 e4 :3 0
2986 4979 497b :3 0
cb :2 0 ec :4 0
2989 497d 497f :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 298c 4982
4986 2990 4981 4988
:3 0 2993 496a 498a
:2 0 498c 2996 498d
4967 498c 0 498e
2998 0 498f 299a
4992 :3 0 4992 0
4992 4991 498f 4990
:6 0 4994 d2 :3 0
299d 4996 299f 4995
4994 :2 0 4997 29a1
:2 0 4999 0 4999
4998 4946 4997 :6 0
4ae8 1 :3 0 15
:3 0 e1 :3 0 499b
499c 0 15 :3 0
da :2 0 158 :3 0
499e 49a0 0 29a6
499f 49a2 :3 0 49a3
:2 0 84 :3 0 78
:3 0 3d :2 0 29a9
49a5 49a8 da :2 0
3d :2 0 29ae 49aa
49ac :3 0 1fc :3 0
317 :3 0 31 :3 0
318 :3 0 32 :3 0
319 :3 0 3f :3 0
31a :3 0 3f :3 0
74 :3 0 31b :3 0
91 :3 0 22 :3 0
92 :3 0 23 :4 0
31c 1 :8 0 49d3
e0 :4 0 49be :3 0
f8 :3 0 da :2 0
29b3 49c1 49c2 :3 0
15 :3 0 e1 :3 0
49c4 49c5 0 3d
:2 0 49c6 49c7 0
49c9 29b6 49d0 15
:3 0 e1 :3 0 49ca
49cb 0 e2 :2 0
49cc 49cd 0 49cf
29b8 49d1 49c3 49c9
0 49d2 0 49cf
0 49d2 29ba 0
49d3 29bd 4a26 e5
:3 0 15 :3 0 e1
:3 0 49d5 49d6 0
e4 :3 0 49d7 49d8
0 49da 29c0 49dc
29c2 49db 49da :2 0
4a24 e6 :3 0 15
:3 0 e1 :3 0 49df
49e0 0 e4 :3 0
49e1 49e2 0 4a1c
c3 :3 0 c4 :3 0
49e4 49e5 0 c3
:3 0 d3 :3 0 49e7
49e8 0 e7 :2 0
2 :4 0 31d :2 0
d5 :2 0 2a :2 0
29c4 49ed 49ef :3 0
29c7 49e9 49f1 49e6
49f3 49f2 :2 0 c6
:3 0 d6 :3 0 49f5
49f6 0 c8 :4 0
31e :4 0 cb :2 0
cc :3 0 29cb 49fa
49fc :3 0 cb :2 0
ea :4 0 29ce 49fe
4a00 :3 0 cb :2 0
eb :4 0 29d1 4a02
4a04 :3 0 cb :2 0
e4 :3 0 29d4 4a06
4a08 :3 0 cb :2 0
ec :4 0 29d7 4a0a
4a0c :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
29da 4a0f 4a13 29de
4a0e 4a15 :3 0 29e1
49f7 4a17 :2 0 4a19
29e4 4a1a 49f4 4a19
0 4a1b 29e6 0
4a1c 29e8 4a1f :3 0
4a1f 0 4a1f 4a1e
4a1c 4a1d :6 0 4a21
d4 :3 0 29eb 4a23
29ed 4a22 4a21 :2 0
4a24 29ef :2 0 4a26
0 4a26 4a25 49d3
4a24 :6 0 4a28 1
:3 0 29f2 4aa6 1fc
:3 0 317 :3 0 31
:3 0 318 :3 0 32
:3 0 319 :3 0 3f
:3 0 31a :3 0 3f
:3 0 74 :3 0 31b
:3 0 3f :3 0 75
:3 0 91 :3 0 22
:3 0 92 :3 0 23
:4 0 31f 1 :8 0
4a50 e0 :4 0 4a3b
:3 0 f8 :3 0 da
:2 0 29f6 4a3e 4a3f
:3 0 15 :3 0 e1
:3 0 4a41 4a42 0
3d :2 0 4a43 4a44
0 4a46 29f9 4a4d
15 :3 0 e1 :3 0
4a47 4a48 0 e2
:2 0 4a49 4a4a 0
4a4c 29fb 4a4e 4a40
4a46 0 4a4f 0
4a4c 0 4a4f 29fd
0 4a50 2a00 4aa3
e5 :3 0 15 :3 0
e1 :3 0 4a52 4a53
0 e4 :3 0 4a54
4a55 0 4a57 2a03
4a59 2a05 4a58 4a57
:2 0 4aa1 e6 :3 0
15 :3 0 e1 :3 0
4a5c 4a5d 0 e4
:3 0 4a5e 4a5f 0
4a99 c3 :3 0 c4
:3 0 4a61 4a62 0
c3 :3 0 d3 :3 0
4a64 4a65 0 e7
:2 0 2 :4 0 320
:2 0 d5 :2 0 2a
:2 0 2a07 4a6a 4a6c
:3 0 2a0a 4a66 4a6e
4a63 4a70 4a6f :2 0
c6 :3 0 d6 :3 0
4a72 4a73 0 c8
:4 0 321 :4 0 cb
:2 0 cc :3 0 2a0e
4a77 4a79 :3 0 cb
:2 0 ea :4 0 2a11
4a7b 4a7d :3 0 cb
:2 0 eb :4 0 2a14
4a7f 4a81 :3 0 cb
:2 0 e4 :3 0 2a17
4a83 4a85 :3 0 cb
:2 0 ec :4 0 2a1a
4a87 4a89 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 2a1d 4a8c 4a90
2a21 4a8b 4a92 :3 0
2a24 4a74 4a94 :2 0
4a96 2a27 4a97 4a71
4a96 0 4a98 2a29
0 4a99 2a2b 4a9c
:3 0 4a9c 0 4a9c
4a9b 4a99 4a9a :6 0
4a9e d6 :3 0 2a2e
4aa0 2a30 4a9f 4a9e
:2 0 4aa1 2a32 :2 0
4aa3 0 4aa3 4aa2
4a50 4aa1 :6 0 4aa5
1 :3 0 2a35 4aa7
49ad 4a28 0 4aa8
0 4aa5 0 4aa8
2a37 0 4acb a0
:3 0 6f :3 0 2a
:2 0 2a :2 0 2a3a
4aa9 4aad da :2 0
1bf :4 0 2a40 4aaf
4ab1 :3 0 4ab2 :2 0
7a :3 0 3d :2 0
4ab4 4ab5 0 4ac8
15 :3 0 e1 :3 0
4ab7 4ab8 0 1c1
:3 0 22 :3 0 22
:3 0 81 :3 0 2b
:3 0 2d :3 0 74
:3 0 75 :3 0 7a
:3 0 79 :3 0 78
:3 0 2a43 4aba 4ac5
4ab9 4ac6 0 4ac8
2a4e 4ac9 4ab3 4ac8
0 4aca 2a51 0
4acb 2a53 4acc 49a4
4acb 0 4acd 2a56
0 4ae8 1e :3 0
166 :2 0 2a58 4acf
4ad0 :3 0 4ad1 :2 0
4ad2 :2 0 117 :3 0
134 :3 0 4ad4 4ad5
0 81 :3 0 1e
:3 0 2a5a 4ad6 4ad9
da :2 0 3d :2 0
2a5f 4adb 4add :3 0
4ade :2 0 4ad3 4ae0
4adf :2 0 63 :3 0
2a :2 0 4ae2 4ae3
0 4ae5 2a62 4ae6
4ae1 4ae5 0 4ae7
2a64 0 4ae8 2a66
4ae9 4919 4ae8 0
4aea 2a6c 0 4aeb
2a6e 4aec 473f 4aeb
0 4aed 2a72 0
4e8b 84 :3 0 4c
:3 0 3d :2 0 2a74
4aee 4af1 10b :2 0
3d :2 0 2a79 4af3
4af5 :3 0 4af6 :2 0
1e :3 0 166 :2 0
2a7c 4af9 4afa :3 0
4afb :2 0 4afc :2 0
4af7 4afe 4afd :2 0
84 :3 0 60 :3 0
3d :2 0 2a7e 4b00
4b03 da :2 0 2a
:2 0 2a83 4b05 4b07
:3 0 13d :3 0 56
:3 0 90 :3 0 91
:3 0 22 :3 0 94
:3 0 1e :3 0 92
:3 0 23 :3 0 93
:4 0 322 1 :8 0
4b19 15 :3 0 e1
:3 0 4b14 4b15 0
e4 :3 0 4b16 4b17
0 4b19 2a86 4b6c
e5 :3 0 15 :3 0
e1 :3 0 4b1b 4b1c
0 e4 :3 0 4b1d
4b1e 0 4b20 2a89
4b22 2a8b 4b21 4b20
:2 0 4b6a e6 :3 0
15 :3 0 e1 :3 0
4b25 4b26 0 e4
:3 0 4b27 4b28 0
4b62 c3 :3 0 c4
:3 0 4b2a 4b2b 0
c3 :3 0 d3 :3 0
4b2d 4b2e 0 e7
:2 0 2 :4 0 323
:2 0 d5 :2 0 2a
:2 0 2a8d 4b33 4b35
:3 0 2a90 4b2f 4b37
4b2c 4b39 4b38 :2 0
c6 :3 0 d6 :3 0
4b3b 4b3c 0 c8
:4 0 324 :4 0 cb
:2 0 cc :3 0 2a94
4b40 4b42 :3 0 cb
:2 0 ea :4 0 2a97
4b44 4b46 :3 0 cb
:2 0 eb :4 0 2a9a
4b48 4b4a :3 0 cb
:2 0 e4 :3 0 2a9d
4b4c 4b4e :3 0 cb
:2 0 ec :4 0 2aa0
4b50 4b52 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 2aa3 4b55 4b59
2aa7 4b54 4b5b :3 0
2aaa 4b3d 4b5d :2 0
4b5f 2aad 4b60 4b3a
4b5f 0 4b61 2aaf
0 4b62 2ab1 4b65
:3 0 4b65 0 4b65
4b64 4b62 4b63 :6 0
4b67 d8 :3 0 2ab4
4b69 2ab6 4b68 4b67
:2 0 4b6a 2ab8 :2 0
4b6c 0 4b6c 4b6b
4b19 4b6a :6 0 4c62
1 :3 0 84 :3 0
56 :3 0 3d :2 0
2abb 4b6e 4b71 10b
:2 0 3d :2 0 2ac0
4b73 4b75 :3 0 90
:3 0 94 :3 0 91
:3 0 22 :3 0 94
:3 0 1e :3 0 92
:3 0 23 :3 0 93
:4 0 325 1 :8 0
4b96 e0 :4 0 4b81
:3 0 f8 :3 0 da
:2 0 2ac5 4b84 4b85
:3 0 15 :3 0 e1
:3 0 4b87 4b88 0
3d :2 0 4b89 4b8a
0 4b8c 2ac8 4b93
15 :3 0 e1 :3 0
4b8d 4b8e 0 e2
:2 0 4b8f 4b90 0
4b92 2aca 4b94 4b86
4b8c 0 4b95 0
4b92 0 4b95 2acc
0 4b96 2acf 4be9
e5 :3 0 15 :3 0
e1 :3 0 4b98 4b99
0 e4 :3 0 4b9a
4b9b 0 4b9d 2ad2
4b9f 2ad4 4b9e 4b9d
:2 0 4be7 e6 :3 0
15 :3 0 e1 :3 0
4ba2 4ba3 0 e4
:3 0 4ba4 4ba5 0
4bdf c3 :3 0 c4
:3 0 4ba7 4ba8 0
c3 :3 0 d3 :3 0
4baa 4bab 0 e7
:2 0 2 :4 0 326
:2 0 d5 :2 0 2a
:2 0 2ad6 4bb0 4bb2
:3 0 2ad9 4bac 4bb4
4ba9 4bb6 4bb5 :2 0
c6 :3 0 d6 :3 0
4bb8 4bb9 0 c8
:4 0 327 :4 0 cb
:2 0 cc :3 0 2add
4bbd 4bbf :3 0 cb
:2 0 ea :4 0 2ae0
4bc1 4bc3 :3 0 cb
:2 0 eb :4 0 2ae3
4bc5 4bc7 :3 0 cb
:2 0 e4 :3 0 2ae6
4bc9 4bcb :3 0 cb
:2 0 ec :4 0 2ae9
4bcd 4bcf :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 2aec 4bd2 4bd6
2af0 4bd1 4bd8 :3 0
2af3 4bba 4bda :2 0
4bdc 2af6 4bdd 4bb7
4bdc 0 4bde 2af8
0 4bdf 2afa 4be2
:3 0 4be2 0 4be2
4be1 4bdf 4be0 :6 0
4be4 da :3 0 2afd
4be6 2aff 4be5 4be4
:2 0 4be7 2b01 :2 0
4be9 0 4be9 4be8
4b96 4be7 :6 0 4c5f
1 :3 0 1b2 :3 0
2f0 :3 0 9a :3 0
91 :3 0 22 :3 0
2f0 :3 0 1e :3 0
92 :3 0 23 :4 0
328 1 :8 0 4c0a
e0 :4 0 4bf5 :3 0
f8 :3 0 da :2 0
2b06 4bf8 4bf9 :3 0
15 :3 0 e1 :3 0
4bfb 4bfc 0 3d
:2 0 4bfd 4bfe 0
4c00 2b09 4c07 15
:3 0 e1 :3 0 4c01
4c02 0 e2 :2 0
4c03 4c04 0 4c06
2b0b 4c08 4bfa 4c00
0 4c09 0 4c06
0 4c09 2b0d 0
4c0a 2b10 4c5d e5
:3 0 15 :3 0 e1
:3 0 4c0c 4c0d 0
e4 :3 0 4c0e 4c0f
0 4c11 2b13 4c13
2b15 4c12 4c11 :2 0
4c5b e6 :3 0 15
:3 0 e1 :3 0 4c16
4c17 0 e4 :3 0
4c18 4c19 0 4c53
c3 :3 0 c4 :3 0
4c1b 4c1c 0 c3
:3 0 d3 :3 0 4c1e
4c1f 0 e7 :2 0
2 :4 0 329 :2 0
d5 :2 0 2a :2 0
2b17 4c24 4c26 :3 0
2b1a 4c20 4c28 4c1d
4c2a 4c29 :2 0 c6
:3 0 d6 :3 0 4c2c
4c2d 0 c8 :4 0
32a :4 0 cb :2 0
cc :3 0 2b1e 4c31
4c33 :3 0 cb :2 0
ea :4 0 2b21 4c35
4c37 :3 0 cb :2 0
eb :4 0 2b24 4c39
4c3b :3 0 cb :2 0
e4 :3 0 2b27 4c3d
4c3f :3 0 cb :2 0
ec :4 0 2b2a 4c41
4c43 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
2b2d 4c46 4c4a 2b31
4c45 4c4c :3 0 2b34
4c2e 4c4e :2 0 4c50
2b37 4c51 4c2b 4c50
0 4c52 2b39 0
4c53 2b3b 4c56 :3 0
4c56 0 4c56 4c55
4c53 4c54 :6 0 4c58
dc :3 0 2b3e 4c5a
2b40 4c59 4c58 :2 0
4c5b 2b42 :2 0 4c5d
0 4c5d 4c5c 4c0a
4c5b :6 0 4c5f 1
:3 0 2b45 4c60 4b76
4c5f 0 4c61 2b48
0 4c62 2b4a 4c63
4b08 4c62 0 4c64
2b4d 0 4e25 6b
:3 0 312 :2 0 4c65
4c66 0 4e25 71
:3 0 21a :3 0 22
:3 0 1e :3 0 6b
:3 0 3d :2 0 3d
:2 0 2b4f 4c69 4c6f
4c68 4c70 0 4e25
a1 :3 0 a2 :3 0
313 :3 0 9f :3 0
a0 :3 0 22c :3 0
9f :3 0 a0 :3 0
22d :3 0 a5 :3 0
a6 :3 0 a7 :3 0
a8 :3 0 31 :3 0
32 :3 0 3f :3 0
2b :3 0 2d :3 0
78 :3 0 79 :3 0
58 :3 0 5b :3 0
90 :3 0 9a :3 0
1e :4 0 32b 1
:8 0 4c91 15 :3 0
e1 :3 0 4c8c 4c8d
0 e4 :3 0 4c8e
4c8f 0 4c91 2b55
4ce4 e5 :3 0 15
:3 0 e1 :3 0 4c93
4c94 0 e4 :3 0
4c95 4c96 0 4c98
2b58 4c9a 2b5a 4c99
4c98 :2 0 4ce2 e6
:3 0 15 :3 0 e1
:3 0 4c9d 4c9e 0
e4 :3 0 4c9f 4ca0
0 4cda c3 :3 0
c4 :3 0 4ca2 4ca3
0 c3 :3 0 d3
:3 0 4ca5 4ca6 0
e7 :2 0 2 :4 0
32c :2 0 d5 :2 0
2a :2 0 2b5c 4cab
4cad :3 0 2b5f 4ca7
4caf 4ca4 4cb1 4cb0
:2 0 c6 :3 0 d6
:3 0 4cb3 4cb4 0
c8 :4 0 32d :4 0
cb :2 0 cc :3 0
2b63 4cb8 4cba :3 0
cb :2 0 ea :4 0
2b66 4cbc 4cbe :3 0
cb :2 0 eb :4 0
2b69 4cc0 4cc2 :3 0
cb :2 0 e4 :3 0
2b6c 4cc4 4cc6 :3 0
cb :2 0 ec :4 0
2b6f 4cc8 4cca :3 0
cb :2 0 a0 :3 0
ed :3 0 2a :2 0
ee :2 0 2b72 4ccd
4cd1 2b76 4ccc 4cd3
:3 0 2b79 4cb5 4cd5
:2 0 4cd7 2b7c 4cd8
4cb2 4cd7 0 4cd9
2b7e 0 4cda 2b80
4cdd :3 0 4cdd 0
4cdd 4cdc 4cda 4cdb
:6 0 4cdf de :3 0
2b83 4ce1 2b85 4ce0
4cdf :2 0 4ce2 2b87
:2 0 4ce4 0 4ce4
4ce3 4c91 4ce2 :6 0
4e25 1 :3 0 15
:3 0 e1 :3 0 4ce6
4ce7 0 15 :3 0
da :2 0 158 :3 0
4ce9 4ceb 0 2b8c
4cea 4ced :3 0 4cee
:2 0 84 :3 0 78
:3 0 3d :2 0 2b8f
4cf0 4cf3 da :2 0
3d :2 0 2b94 4cf5
4cf7 :3 0 1fc :3 0
317 :3 0 31 :3 0
318 :3 0 32 :3 0
319 :3 0 3f :3 0
31a :3 0 3f :3 0
58 :3 0 31b :3 0
91 :3 0 22 :3 0
92 :3 0 21 :4 0
32e 1 :8 0 4d1e
e0 :4 0 4d09 :3 0
f8 :3 0 da :2 0
2b99 4d0c 4d0d :3 0
15 :3 0 e1 :3 0
4d0f 4d10 0 3d
:2 0 4d11 4d12 0
4d14 2b9c 4d1b 15
:3 0 e1 :3 0 4d15
4d16 0 e2 :2 0
4d17 4d18 0 4d1a
2b9e 4d1c 4d0e 4d14
0 4d1d 0 4d1a
0 4d1d 2ba0 0
4d1e 2ba3 4d71 e5
:3 0 15 :3 0 e1
:3 0 4d20 4d21 0
e4 :3 0 4d22 4d23
0 4d25 2ba6 4d27
2ba8 4d26 4d25 :2 0
4d6f e6 :3 0 15
:3 0 e1 :3 0 4d2a
4d2b 0 e4 :3 0
4d2c 4d2d 0 4d67
c3 :3 0 c4 :3 0
4d2f 4d30 0 c3
:3 0 d3 :3 0 4d32
4d33 0 e7 :2 0
2 :4 0 32f :2 0
d5 :2 0 2a :2 0
2baa 4d38 4d3a :3 0
2bad 4d34 4d3c 4d31
4d3e 4d3d :2 0 c6
:3 0 d6 :3 0 4d40
4d41 0 c8 :4 0
330 :4 0 cb :2 0
cc :3 0 2bb1 4d45
4d47 :3 0 cb :2 0
ea :4 0 2bb4 4d49
4d4b :3 0 cb :2 0
eb :4 0 2bb7 4d4d
4d4f :3 0 cb :2 0
e4 :3 0 2bba 4d51
4d53 :3 0 cb :2 0
ec :4 0 2bbd 4d55
4d57 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
2bc0 4d5a 4d5e 2bc4
4d59 4d60 :3 0 2bc7
4d42 4d62 :2 0 4d64
2bca 4d65 4d3f 4d64
0 4d66 2bcc 0
4d67 2bce 4d6a :3 0
4d6a 0 4d6a 4d69
4d67 4d68 :6 0 4d6c
e0 :3 0 2bd1 4d6e
2bd3 4d6d 4d6c :2 0
4d6f 2bd5 :2 0 4d71
0 4d71 4d70 4d1e
4d6f :6 0 4d73 1
:3 0 2bd8 4df1 1fc
:3 0 317 :3 0 31
:3 0 318 :3 0 32
:3 0 319 :3 0 3f
:3 0 31a :3 0 3f
:3 0 58 :3 0 31b
:3 0 3f :3 0 5b
:3 0 91 :3 0 22
:3 0 92 :3 0 21
:4 0 331 1 :8 0
4d9b e0 :4 0 4d86
:3 0 f8 :3 0 da
:2 0 2bdc 4d89 4d8a
:3 0 15 :3 0 e1
:3 0 4d8c 4d8d 0
3d :2 0 4d8e 4d8f
0 4d91 2bdf 4d98
15 :3 0 e1 :3 0
4d92 4d93 0 e2
:2 0 4d94 4d95 0
4d97 2be1 4d99 4d8b
4d91 0 4d9a 0
4d97 0 4d9a 2be3
0 4d9b 2be6 4dee
e5 :3 0 15 :3 0
e1 :3 0 4d9d 4d9e
0 e4 :3 0 4d9f
4da0 0 4da2 2be9
4da4 2beb 4da3 4da2
:2 0 4dec e6 :3 0
15 :3 0 e1 :3 0
4da7 4da8 0 e4
:3 0 4da9 4daa 0
4de4 c3 :3 0 c4
:3 0 4dac 4dad 0
c3 :3 0 d3 :3 0
4daf 4db0 0 e7
:2 0 2 :4 0 332
:2 0 d5 :2 0 2a
:2 0 2bed 4db5 4db7
:3 0 2bf0 4db1 4db9
4dae 4dbb 4dba :2 0
c6 :3 0 d6 :3 0
4dbd 4dbe 0 c8
:4 0 333 :4 0 cb
:2 0 cc :3 0 2bf4
4dc2 4dc4 :3 0 cb
:2 0 ea :4 0 2bf7
4dc6 4dc8 :3 0 cb
:2 0 eb :4 0 2bfa
4dca 4dcc :3 0 cb
:2 0 e4 :3 0 2bfd
4dce 4dd0 :3 0 cb
:2 0 ec :4 0 2c00
4dd2 4dd4 :3 0 cb
:2 0 a0 :3 0 ed
:3 0 2a :2 0 ee
:2 0 2c03 4dd7 4ddb
2c07 4dd6 4ddd :3 0
2c0a 4dbf 4ddf :2 0
4de1 2c0d 4de2 4dbc
4de1 0 4de3 2c0f
0 4de4 2c11 4de7
:3 0 4de7 0 4de7
4de6 4de4 4de5 :6 0
4de9 e2 :3 0 2c14
4deb 2c16 4dea 4de9
:2 0 4dec 2c18 :2 0
4dee 0 4dee 4ded
4d9b 4dec :6 0 4df0
1 :3 0 2c1b 4df2
4cf8 4d73 0 4df3
0 4df0 0 4df3
2c1d 0 4e22 a0
:3 0 6f :3 0 2a
:2 0 2a :2 0 2c20
4df4 4df8 da :2 0
1bf :4 0 2c26 4dfa
4dfc :3 0 4dfd :2 0
84 :3 0 63 :3 0
3d :2 0 2c29 4dff
4e02 da :2 0 3d
:2 0 2c2e 4e04 4e06
:3 0 4e07 :2 0 4dfe
4e09 4e08 :2 0 7a
:3 0 3d :2 0 4e0b
4e0c 0 4e1f 15
:3 0 e1 :3 0 4e0e
4e0f 0 1c1 :3 0
22 :3 0 22 :3 0
1e :3 0 2b :3 0
2d :3 0 58 :3 0
5b :3 0 7a :3 0
79 :3 0 78 :3 0
2c31 4e11 4e1c 4e10
4e1d 0 4e1f 2c3c
4e20 4e0a 4e1f 0
4e21 2c3f 0 4e22
2c41 4e23 4cef 4e22
0 4e24 2c44 0
4e25 2c46 4e26 4aff
4e25 0 4e27 2c4c
0 4e8b 71 :3 0
334 :3 0 22 :3 0
2c4e 4e29 4e2b 4e28
4e2c 0 4e8b 15
:3 0 e1 :3 0 4e2e
4e2f 0 da :2 0
e2 :2 0 2c52 4e31
4e33 :3 0 15 :3 0
e1 :3 0 4e35 4e36
0 3d :2 0 4e37
4e38 0 4e3a 2c55
4e3b 4e34 4e3a 0
4e3c 2c57 0 4e8b
c3 :3 0 c4 :3 0
4e3d 4e3e 0 c3
:3 0 d3 :3 0 4e40
4e41 0 13a :2 0
2 :4 0 335 :2 0
d5 :2 0 2a :2 0
2c59 4e46 4e48 :3 0
2c5c 4e42 4e4a 4e3f
4e4c 4e4b :2 0 c6
:3 0 d6 :3 0 4e4e
4e4f 0 c8 :4 0
336 :4 0 cb :2 0
cc :3 0 2c60 4e53
4e55 :3 0 cb :2 0
ea :4 0 2c63 4e57
4e59 :3 0 cb :2 0
f1 :4 0 2c66 4e5b
4e5d :3 0 cb :2 0
84 :3 0 d0 :3 0
15 :3 0 e1 :3 0
4e62 4e63 0 2c69
4e61 4e65 d1 :4 0
2c6b 4e60 4e68 2c6e
4e5f 4e6a :3 0 cb
:2 0 f3 :4 0 2c71
4e6c 4e6e :3 0 cb
:2 0 ce :4 0 2c74
4e70 4e72 :3 0 cb
:2 0 1e :3 0 2c77
4e74 4e76 :3 0 cb
:2 0 ce :4 0 2c7a
4e78 4e7a :3 0 2c7d
4e50 4e7c :2 0 4e7e
2c80 4e7f 4e4d 4e7e
0 4e80 2c82 0
4e8b f6 :3 0 337
:3 0 4e81 4e82 0
15 :3 0 e1 :3 0
4e84 4e85 0 1e
:3 0 f8 :3 0 2c84
4e83 4e89 :2 0 4e8b
2c88 4f2c 15 :3 0
338 :2 0 4 4e8c
4e8d 0 c3 :3 0
c4 :3 0 4e8f 4e90
0 c3 :3 0 d3
:3 0 4e92 4e93 0
e7 :2 0 2 :4 0
339 :2 0 d5 :2 0
2a :2 0 2cdf 4e98
4e9a :3 0 2ce2 4e94
4e9c 4e91 4e9e 4e9d
:2 0 c6 :3 0 d6
:3 0 4ea0 4ea1 0
c8 :4 0 33a :4 0
cb :2 0 cc :3 0
2ce6 4ea5 4ea7 :3 0
2ce9 4ea2 4ea9 :2 0
4ec2 c6 :3 0 d6
:3 0 4eab 4eac 0
c8 :4 0 33b :4 0
cb :2 0 15 :3 0
33c :3 0 4eb1 4eb2
0 2cec 4eb0 4eb4
:3 0 cb :2 0 f3
:4 0 2cef 4eb6 4eb8
:3 0 cb :2 0 15
:3 0 33d :3 0 4ebb
4ebc 0 2cf2 4eba
4ebe :3 0 2cf5 4ead
4ec0 :2 0 4ec2 2cf8
4ec3 4e9f 4ec2 0
4ec4 2cfb 0 4ed4
df :3 0 c2 :3 0
15 :3 0 33c :3 0
33e :3 0 15 :3 0
33d :3 0 33f :3 0
1d0 :4 0 340 1
:a 0 4ed0 :2 0 4ed4
4ec6 4ece :2 0 14
:3 0 c2 :3 0 4ed2
:2 0 4ed4 2cfd 4ed6
2d01 4ed5 4ed4 :2 0
4f2a e6 :3 0 15
:3 0 e1 :3 0 4ed9
4eda 0 e4 :3 0
4edb 4edc 0 4f22
c3 :3 0 c4 :3 0
4ede 4edf 0 c3
:3 0 d3 :3 0 4ee1
4ee2 0 e7 :2 0
2 :4 0 341 :2 0
d5 :2 0 2a :2 0
2d03 4ee7 4ee9 :3 0
2d06 4ee3 4eeb 4ee0
4eed 4eec :2 0 c6
:3 0 d6 :3 0 4eef
4ef0 0 c8 :4 0
342 :4 0 cb :2 0
cc :3 0 2d0a 4ef4
4ef6 :3 0 2d0d 4ef1
4ef8 :2 0 4f12 c6
:3 0 d6 :3 0 4efa
4efb 0 c8 :4 0
eb :4 0 cb :2 0
e4 :3 0 2d10 4eff
4f01 :3 0 cb :2 0
ec :4 0 2d13 4f03
4f05 :3 0 cb :2 0
a0 :3 0 ed :3 0
2a :2 0 ee :2 0
2d16 4f08 4f0c 2d1a
4f07 4f0e :3 0 2d1d
4efc 4f10 :2 0 4f12
2d20 4f13 4eee 4f12
0 4f14 2d23 0
4f22 df :3 0 c2
:3 0 33e :3 0 15
:3 0 33d :3 0 33f
:3 0 1d0 :4 0 343
1 :a 0 4f1e :2 0
4f22 4f16 4f1c :2 0
14 :3 0 c2 :3 0
4f20 :2 0 4f22 2d25
4f25 :3 0 4f25 0
4f25 4f24 4f22 4f23
:6 0 4f27 1 :3 0
2d2a 4f29 2d2c 4f28
4f27 :2 0 4f2a 2d2e
:2 0 4f2c 2d31 4f2c
4f2b 4e8b 4f2a :6 0
4f2d :2 0 2 44
4f2c 4f30 :3 0 4f2f
4f2d 4f31 :8 0 
2d99
4
:3 0 1 3 1
7 1 b 1
f 1 13 1
17 1 1b 1
1f 1 23 1
27 1 2b 1
2f 1 33 1
37 1 3b f
6 a e 12
16 1a 1e 22
26 2a 2e 32
36 3a 3e 1
48 1 46 1
4f 1 4d 1
56 1 54 1
5d 1 5b 1
64 1 62 1
6b 1 69 1
72 1 70 1
79 1 77 1
80 1 7e 1
87 1 85 1
8e 1 8c 1
95 1 93 1
9a 1 9f 1
a4 1 a9 1
b0 1 ae 1
b5 1 ba 1
c1 1 bf 1
c8 1 c6 1
cd 1 d2 1
d7 1 dc 1
e3 1 e1 1
ea 1 e8 1
f1 1 ef 1
f8 1 f6 1
ff 1 fd 1
104 1 10a 1
110 1 116 1
11c 1 122 1
128 1 12e 1
134 1 13a 1
140 1 146 1
14c 1 152 1
158 1 15e 1
164 1 16a 1
170 1 176 1
17c 1 182 1
188 1 18e 1
194 1 19a 1
1a0 1 1a6 1
1ac 1 1b2 1
1b8 1 1be 1
1c4 1 1ca 1
1d0 1 1d6 1
1dc 1 1e2 1
1e8 1 1ee 1
1f6 1 1f4 1
1fd 1 1fb 1
202 1 208 1
210 1 20e 1
215 1 21d 1
21b 1 224 1
222 1 229 1
22f 1 235 1
23a 1 23f 1
245 1 24b 1
251 1 257 1
25d 1 263 1
269 1 26f 1
275 1 27b 1
283 1 281 1
28a 1 288 1
291 1 28f 2
29b 29c 1 29e
1 296 1 2a4
1 2aa 1 2ba
1 2ce 1 346
1 34c 1 359
1 365 2 368
36a 2 36d 36f
2 371 373 2
375 377 2 379
37b 2 37d 37f
2 381 383 2
385 387 2 389
38b 2 38d 38f
2 391 393 2
395 397 2 399
39b 2 39d 39f
2 3a1 3a3 2
3a5 3a7 2 3a9
3ab 2 3ad 3af
2 3b1 3b3 2
3b5 3b7 2 3b9
3bb 1 3c1 2
3c3 3c4 2 3bd
3c6 2 3c8 3ca
1 3d0 2 3d2
3d3 2 3cc 3d5
2 3d7 3d9 1
3df 2 3e1 3e2
2 3db 3e4 2
3e6 3e8 1 3ee
2 3f0 3f1 2
3ea 3f3 2 3f5
3f7 1 3fd 2
3ff 400 2 3f9
402 2 404 406
1 40c 2 40e
40f 2 408 411
2 413 415 1
41b 2 41d 41e
2 417 420 2
422 424 1 42a
2 42c 42d 2
426 42f 2 431
433 1 439 2
43b 43c 2 435
43e 2 440 442
2 444 446 2
448 44a 2 44c
44e 2 450 452
4 361 367 36c
454 1 456 1
458 2 462 464
3 460 461 466
2 46f 471 2
46e 473 1 475
1 477 2 48e
48f 2 491 494
2 499 49a 1
4a0 2 49c 4a0
2 4a5 4a6 1
4ac 2 4a8 4ac
1 4b4 1 4b6
1 4dd 1 4e3
2 4e5 4e6 2
4ef 4f5 1 4fc
1 4f7 2 50f
511 3 50d 50e
513 2 51c 51e
2 520 522 2
524 526 2 528
52a 2 52c 52e
3 533 534 535
2 530 537 2
51b 539 1 53b
1 53d 2 506
53e 1 542 1
501 2 4ff 546
2 54c 54d 1
551 2 54f 551
2 55c 55e 3
55a 55b 560 2
569 56b 2 56d
56f 2 571 573
1 57b 2 57d
57e 2 575 580
2 582 584 2
586 588 2 58a
58c 2 58e 590
2 568 592 1
594 1 596 2
5a0 5a2 3 59e
59f 5a4 2 5ad
5af 2 5ac 5b1
1 5b3 1 5b5
3 5bc 5bd 5be
3 597 5b6 5c0
1 5c2 2 5d5
5da 1 5e1 1
5dc 2 5f4 5f6
3 5f2 5f3 5f8
2 601 603 2
605 607 2 609
60b 2 60d 60f
2 611 613 3
618 619 61a 2
615 61c 2 600
61e 1 620 1
622 2 5eb 623
1 627 1 5e6
2 5e4 62b 1
636 2 632 636
2 642 644 3
640 641 646 2
64f 651 2 653
655 2 657 659
1 661 2 663
664 2 65b 666
2 668 66a 2
66c 66e 2 670
672 2 674 676
2 64e 678 1
67a 1 67c 2
686 688 3 684
685 68a 2 693
695 2 692 697
1 699 1 69b
3 6a2 6a3 6a4
3 67d 69c 6a6
1 6a8 2 6b2
6b7 1 6be 1
6b9 2 6d1 6d3
3 6cf 6d0 6d5
2 6de 6e0 2
6e2 6e4 2 6e6
6e8 2 6ea 6ec
2 6ee 6f0 3
6f5 6f6 6f7 2
6f2 6f9 2 6dd
6fb 1 6fd 1
6ff 2 6c8 700
1 704 1 6c3
2 6c1 708 2
70e 70f 1 713
2 711 713 2
71e 720 3 71c
71d 722 2 72b
72d 2 72f 731
2 733 735 1
73d 2 73f 740
2 737 742 2
744 746 2 748
74a 2 74c 74e
2 750 752 2
72a 754 1 756
1 758 2 762
764 3 760 761
766 2 76f 771
2 76e 773 1
775 1 777 3
77e 77f 780 3
759 778 782 1
784 2 7ab 7b0
1 7b7 1 7b2
2 7ca 7cc 3
7c8 7c9 7ce 2
7d7 7d9 2 7db
7dd 2 7df 7e1
2 7e3 7e5 2
7e7 7e9 3 7ee
7ef 7f0 2 7eb
7f2 2 7d6 7f4
1 7f6 1 7f8
2 7c1 7f9 1
7fd 1 7bc 2
7ba 801 1 80c
2 808 80c 2
818 81a 3 816
817 81c 2 825
827 2 829 82b
2 82d 82f 1
837 2 839 83a
2 831 83c 2
83e 840 2 842
844 2 846 848
2 84a 84c 2
824 84e 1 850
1 852 2 85c
85e 3 85a 85b
860 2 869 86b
2 868 86d 1
86f 1 871 3
878 879 87a 3
853 872 87c 1
87e 2 887 88c
1 893 1 88e
2 8a6 8a8 3
8a4 8a5 8aa 2
8b3 8b5 2 8b7
8b9 2 8bb 8bd
2 8bf 8c1 2
8c3 8c5 3 8ca
8cb 8cc 2 8c7
8ce 2 8b2 8d0
1 8d2 1 8d4
2 89d 8d5 1
8d9 1 898 2
896 8dd 1 8e8
2 8e4 8e8 2
8f4 8f6 3 8f2
8f3 8f8 2 901
903 2 905 907
2 909 90b 1
913 2 915 916
2 90d 918 2
91a 91c 2 91e
920 2 922 924
2 926 928 2
900 92a 1 92c
1 92e 2 938
93a 3 936 937
93c 2 945 947
2 944 949 1
94b 1 94d 3
954 955 956 3
92f 94e 958 1
95a 2 961 966
1 96d 1 968
2 980 982 3
97e 97f 984 2
98d 98f 2 991
993 2 995 997
2 999 99b 2
99d 99f 3 9a4
9a5 9a6 2 9a1
9a8 2 98c 9aa
1 9ac 1 9ae
2 977 9af 1
9b3 1 972 2
970 9b7 2 9bf
9c0 1 9c4 2
9c2 9c4 1 9c9
1 9cb 1 9d3
2 9cf 9d3 2
9df 9e1 3 9dd
9de 9e3 2 9ec
9ee 2 9f0 9f2
2 9f4 9f6 1
9fe 2 a00 a01
2 9f8 a03 2
a05 a07 2 a09
a0b 2 a0d a0f
2 a11 a13 2
9eb a15 1 a17
1 a19 2 a23
a25 3 a21 a22
a27 2 a30 a32
2 a2f a34 1
a36 1 a38 3
a3f a40 a41 3
a1a a39 a43 1
a45 2 a4d a4e
2 a50 a51 2
a58 a59 2 a5b
a5c 2 a63 a64
2 a66 a67 1
a6d 2 a6b a6d
2 a71 a72 1
a76 2 a74 a76
2 a84 a89 1
a90 1 a8b 2
aa3 aa5 3 aa1
aa2 aa7 2 ab0
ab2 2 ab4 ab6
2 ab8 aba 2
abc abe 2 ac0
ac2 3 ac7 ac8
ac9 2 ac4 acb
2 aaf acd 1
acf 1 ad1 2
a9a ad2 1 ad6
1 a95 2 a93
ada 1 add 2
ae2 ae3 1 ae7
2 ae5 ae7 2
af5 afa 1 b01
1 afc 2 b14
b16 3 b12 b13
b18 2 b21 b23
2 b25 b27 2
b29 b2b 2 b2d
b2f 2 b31 b33
3 b38 b39 b3a
2 b35 b3c 2
b20 b3e 1 b40
1 b42 2 b0b
b43 1 b47 1
b06 2 b04 b4b
1 b4e 2 b52
b51 2 b55 b56
1 b5a 2 b58
b5a 2 b65 b67
3 b63 b64 b69
2 b72 b74 2
b76 b78 2 b7a
b7c 1 b84 2
b86 b87 2 b7e
b89 2 b8b b8d
2 b8f b91 2
b93 b95 2 b97
b99 2 b71 b9b
1 b9d 1 b9f
2 ba9 bab 3
ba7 ba8 bad 2
bb6 bb8 2 bb5
bba 1 bbc 1
bbe 3 bc5 bc6
bc7 3 ba0 bbf
bc9 1 bcb 2
b53 bcc 2 bd9
bde 1 be5 1
be0 2 bf8 bfa
3 bf6 bf7 bfc
2 c05 c07 2
c09 c0b 2 c0d
c0f 2 c11 c13
2 c15 c17 3
c1c c1d c1e 2
c19 c20 2 c04
c22 1 c24 1
c26 2 bef c27
1 c2b 1 bea
2 be8 c2f 2
c3f c44 1 c4b
1 c46 2 c5e
c60 3 c5c c5d
c62 2 c6b c6d
2 c6f c71 2
c73 c75 2 c77
c79 2 c7b c7d
3 c82 c83 c84
2 c7f c86 2
c6a c88 1 c8a
1 c8c 2 c55
c8d 1 c91 1
c50 2 c4e c95
2 c9c c9d 1
c9f 1 ca4 2
ca2 ca4 2 ca8
ca9 1 cad 2
cab cad 2 cb2
cb3 1 cb7 2
cb5 cb7 2 cca
ccf 1 cd6 1
cd1 2 ce9 ceb
3 ce7 ce8 ced
2 cf6 cf8 2
cfa cfc 2 cfe
d00 2 d02 d04
2 d06 d08 3
d0d d0e d0f 2
d0a d11 2 cf5
d13 1 d15 1
d17 2 ce0 d18
1 d1c 1 cdb
2 cd9 d20 2
d26 d27 1 d2b
2 d29 d2b 2
d36 d38 3 d34
d35 d3a 2 d43
d45 2 d47 d49
2 d4b d4d 1
d55 2 d57 d58
2 d4f d5a 2
d5c d5e 2 d60
d62 2 d64 d66
2 d68 d6a 2
d42 d6c 1 d6e
1 d70 2 d7a
d7c 3 d78 d79
d7e 2 d87 d89
2 d86 d8b 1
d8d 1 d8f 3
d96 d97 d98 3
d71 d90 d9a 1
d9c 2 d23 d9d
1 d9f 3 c32
c98 da0 2 da2
da3 1 dc4 1
dca 2 dcc dcd
1 dd5 2 dd1
dd5 1 ddb 1
ddf 2 de1 de2
2 de6 dec 1
df3 1 dee 2
e06 e08 3 e04
e05 e0a 2 e13
e15 2 e17 e19
2 e1b e1d 2
e1f e21 2 e23
e25 3 e2a e2b
e2c 2 e27 e2e
2 e12 e30 1
e32 1 e34 2
dfd e35 1 e39
1 df8 2 df6
e3d 2 e43 e44
1 e48 2 e46
e48 2 e56 e5b
1 e62 1 e5d
2 e75 e77 3
e73 e74 e79 2
e82 e84 2 e86
e88 2 e8a e8c
2 e8e e90 2
e92 e94 3 e99
e9a e9b 2 e96
e9d 2 e81 e9f
1 ea1 1 ea3
2 e6c ea4 1
ea8 1 e67 2
e65 eac 2 eb2
eb3 1 eb7 2
eb5 eb7 2 eca
ecf 1 ed6 1
ed1 2 ee9 eeb
3 ee7 ee8 eed
2 ef6 ef8 2
efa efc 2 efe
f00 2 f02 f04
2 f06 f08 3
f0d f0e f0f 2
f0a f11 2 ef5
f13 1 f15 1
f17 2 ee0 f18
1 f1c 1 edb
2 ed9 f20 1
f2b 2 f27 f2b
2 f37 f39 3
f35 f36 f3b 2
f44 f46 2 f48
f4a 2 f4c f4e
1 f56 2 f58
f59 2 f50 f5b
2 f5d f5f 2
f61 f63 2 f65
f67 2 f69 f6b
2 f43 f6d 1
f6f 1 f71 2
f7b f7d 3 f79
f7a f7f 2 f88
f8a 2 f87 f8c
1 f8e 1 f90
3 f97 f98 f99
3 f72 f91 f9b
1 f9e 1 fa4
3 faf fb0 fb1
1 fb5 2 fb3
fb5 3 fbc fbd
fbe 1 fc2 2
fc0 fc2 2 fd0
fd2 3 fce fcf
fd4 2 fdd fdf
2 fe1 fe3 2
fe5 fe7 1 fef
2 ff1 ff2 2
fe9 ff4 2 ff6
ff8 2 ffa ffc
2 ffe 1000 2
1002 1004 2 fdc
1006 1 1008 1
100a 2 1014 1016
3 1012 1013 1018
2 1021 1023 2
1020 1025 1 1027
1 1029 3 1030
1031 1032 3 100b
102a 1034 1 1036
1 1037 2 103a
1039 3 ebc f23
103b 1 103d 2
1040 1041 1 1045
2 1043 1045 2
104a 104d 2 105a
105f 1 1066 1
1061 2 1079 107b
3 1077 1078 107d
2 1086 1088 2
108a 108c 2 108e
1090 2 1092 1094
2 1096 1098 3
109d 109e 109f 2
109a 10a1 2 1085
10a3 1 10a5 1
10a7 2 1070 10a8
1 10ac 1 106b
2 1069 10b0 1
10bb 2 10b7 10bb
2 10cb 10cc 4
10c9 10ce 10d1 10d2
7 10e3 10e4 10e5
10e6 10e7 10e8 10e9
2 10f0 10f1 1
10f3 1 1114 2
1113 1114 1 111c
1 1122 2 1124
1125 2 1111 1126
1 112d 1 1128
2 1140 1142 3
113e 113f 1144 2
114d 114f 2 1151
1153 2 1155 1157
2 1159 115b 2
115d 115f 3 1164
1165 1166 2 1161
1168 2 114c 116a
1 116c 1 116e
2 1137 116f 1
1173 1 1132 2
1130 1177 2 1181
1186 1 118d 1
1188 2 11a0 11a2
3 119e 119f 11a4
2 11ad 11af 2
11b1 11b3 2 11b5
11b7 2 11b9 11bb
2 11bd 11bf 3
11c4 11c5 11c6 2
11c1 11c8 2 11ac
11ca 1 11cc 1
11ce 2 1197 11cf
1 11d3 1 1192
2 1190 11d7 2
11df 11e0 1 11e4
2 11e2 11e4 1
11e9 2 11f7 11fc
1 1203 1 11fe
2 1216 1218 3
1214 1215 121a 2
1223 1225 2 1227
1229 2 122b 122d
2 122f 1231 2
1233 1235 3 123a
123b 123c 2 1237
123e 2 1222 1240
1 1242 1 1244
2 120d 1245 1
1249 1 1208 2
1206 124d 1 1250
2 1253 1254 2
1257 1258 1 125c
2 125a 125c 1
126c 2 126b 126c
1 1274 1 127a
2 127c 127d 2
1269 127e 1 1285
1 1280 2 1298
129a 3 1296 1297
129c 2 12a5 12a7
2 12a9 12ab 2
12ad 12af 2 12b1
12b3 2 12b5 12b7
3 12bc 12bd 12be
2 12b9 12c0 2
12a4 12c2 1 12c4
1 12c6 2 128f
12c7 1 12cb 1
128a 2 1288 12cf
1 12e8 2 12e7
12e8 1 12f0 1
12f6 2 12f8 12f9
2 12e5 12fa 1
1301 1 12fc 2
1314 1316 3 1312
1313 1318 2 1321
1323 2 1325 1327
2 1329 132b 2
132d 132f 2 1331
1333 3 1338 1339
133a 2 1335 133c
2 1320 133e 1
1340 1 1342 2
130b 1343 1 1347
1 1306 2 1304
134b 2 12d2 134e
1 1365 2 1364
1365 1 136d 1
1373 2 1375 1376
2 1362 1377 1
137e 1 1379 2
1391 1393 3 138f
1390 1395 2 139e
13a0 2 13a2 13a4
2 13a6 13a8 2
13aa 13ac 2 13ae
13b0 3 13b5 13b6
13b7 2 13b2 13b9
2 139d 13bb 1
13bd 1 13bf 2
1388 13c0 1 13c4
1 1383 2 1381
13c8 1 13cb 2
13ce 13cf c 10c1
10c6 10d5 10da 10dd
10e0 10ec 10f6 117a
11da 1255 13d0 1
13d9 2 13d5 13d9
2 13e3 13e8 1
13ef 1 13ea 2
1402 1404 3 1400
1401 1406 2 140f
1411 2 1413 1415
2 1417 1419 2
141b 141d 2 141f
1421 3 1426 1427
1428 2 1423 142a
2 140e 142c 1
142e 1 1430 2
13f9 1431 1 1435
1 13f4 2 13f2
1439 2 143f 1440
1 1446 2 1442
1446 2 144d 144e
4 144b 1450 1453
1454 1 1465 2
1464 1465 1 146d
1 1473 2 1475
1476 2 1462 1477
1 147e 1 1479
2 1491 1493 3
148f 1490 1495 2
149e 14a0 2 14a2
14a4 2 14a6 14a8
2 14aa 14ac 2
14ae 14b0 3 14b5
14b6 14b7 2 14b2
14b9 2 149d 14bb
1 14bd 1 14bf
2 1488 14c0 1
14c4 1 1483 2
1481 14c8 1 14cf
3 1457 14cb 14d2
2 14d7 14d8 2
14da 14dd 1 14e4
2 14e0 14e4 1
14f5 2 14f4 14f5
1 14fd 1 1503
2 1505 1506 2
14f2 1507 1 150e
1 1509 2 1521
1523 3 151f 1520
1525 2 152e 1530
2 1532 1534 2
1536 1538 2 153a
153c 2 153e 1540
3 1545 1546 1547
2 1542 1549 2
152d 154b 1 154d
1 154f 2 1518
1550 1 1554 1
1513 2 1511 1558
1 155f 2 155b
1562 1 1573 2
1572 1573 1 157b
1 1581 2 1583
1584 2 1570 1585
1 158c 1 1587
2 159f 15a1 3
159d 159e 15a3 2
15ac 15ae 2 15b0
15b2 2 15b4 15b6
2 15b8 15ba 2
15bc 15be 3 15c3
15c4 15c5 2 15c0
15c7 2 15ab 15c9
1 15cb 1 15cd
2 1596 15ce 1
15d2 1 1591 2
158f 15d6 1 15dd
2 15d9 15e0 3
15e2 1564 15e3 2
143c 15e4 2 15e7
15e6 2 10b3 15e8
2 15ea 15eb 3
eaf 103e 15ec 1
15f0 2 15f2 15f3
2 1619 161e 1
1625 1 1620 2
1638 163a 3 1636
1637 163c 2 1645
1647 2 1649 164b
2 164d 164f 2
1651 1653 2 1655
1657 3 165c 165d
165e 2 1659 1660
2 1644 1662 1
1664 1 1666 2
162f 1667 1 166b
1 162a 2 1628
166f 1 167a 2
1676 167a 1 1680
1 1682 2 1685
1686 1 168a 2
1688 168a 2 168f
1690 1 1694 2
1692 1694 a 169c
169d 169e 169f 16a0
16a1 16a2 16a3 16a4
16a5 2 16aa 16ab
1 16af 2 16ad
16af 2 16b4 16b5
2 16ba 16bb 2
16b7 16bd 2 16c3
16c4 2 16c9 16ca
2 16c6 16cc 2
16c0 16cf 1 16d1
2 16d4 16d5 1
16d9 2 16d7 16d9
2 16de 16df 2
16e4 16e5 2 16e1
16e7 2 16ed 16ee
2 16f3 16f4 2
16f0 16f6 2 16ea
16f9 1 16fb 3
16a8 16d2 16fc 1
16fe 2 1710 1715
1 171c 1 1717
2 172f 1731 3
172d 172e 1733 2
173c 173e 2 1740
1742 2 1744 1746
2 1748 174a 2
174c 174e 3 1753
1754 1755 2 1750
1757 2 173b 1759
1 175b 1 175d
2 1726 175e 1
1762 1 1721 2
171f 1766 2 176e
176f 1 1773 2
1771 1773 2 1780
1785 1 178c 1
1787 2 179f 17a1
3 179d 179e 17a3
2 17ac 17ae 2
17b0 17b2 2 17b4
17b6 2 17b8 17ba
2 17bc 17be 3
17c3 17c4 17c5 2
17c0 17c7 2 17ab
17c9 1 17cb 1
17cd 2 1796 17ce
1 17d2 1 1791
2 178f 17d6 2
17dc 17dd 1 17e1
2 17df 17e1 1
17e6 1 17ea 2
17ec 17ed 2 17d9
17ee 1 17f0 2
17f3 17f4 1 17f8
2 17f6 17f8 2
17fe 17ff 1 1803
2 1801 1803 2
182e 1833 1 183a
1 1835 2 184d
184f 3 184b 184c
1851 2 185a 185c
2 185e 1860 2
1862 1864 2 1866
1868 2 186a 186c
3 1871 1872 1873
2 186e 1875 2
1859 1877 1 1879
1 187b 2 1844
187c 1 1880 1
183f 2 183d 1884
2 1890 1891 1
1895 2 1893 1895
2 1899 189a 2
189f 18a0 1 18a2
2 189c 18a2 2
18a7 18a8 2 18ad
18ae 1 18b0 2
18aa 18b0 2 18b7
18b8 1 18bc 2
18ba 18bc 2 18c2
18c3 2 18c5 18c8
1 18cd 2 18cb
18cd 2 18db 18dd
3 18d9 18da 18df
2 18e8 18ea 2
18ec 18ee 2 18f0
18f2 1 18fa 2
18fc 18fd 2 18f4
18ff 2 1901 1903
2 1905 1907 2
1909 190b 2 190d
190f 2 18e7 1911
1 1913 1 1915
2 191f 1921 3
191d 191e 1923 2
192c 192e 2 192b
1930 1 1932 1
1934 3 193b 193c
193d 3 1916 1935
193f 2 1945 1946
1 194a 2 1948
194a 1 194f 1
1951 2 1943 1952
2 1954 1955 1
1956 1 1958 a
1959 195c 195f 1962
1965 1968 196b 196e
1971 1974 1 1976
1 197e 2 197a
197e 1 1982 3
198b 198c 198d 1
1991 2 198f 1991
1 1996 a 199e
199f 19a0 19a1 19a2
19a3 19a4 19a5 19a6
19a7 2 1999 19aa
1 19ac 2 19af
19b0 1 19b4 2
19b2 19b4 1 19bd
2 19bc 19bd 1
19c5 1 19cb 2
19cd 19ce 2 19ba
19cf 1 19d6 1
19d1 2 19e9 19eb
3 19e7 19e8 19ed
2 19f6 19f8 2
19fa 19fc 2 19fe
1a00 2 1a02 1a04
2 1a06 1a08 3
1a0d 1a0e 1a0f 2
1a0a 1a11 2 19f5
1a13 1 1a15 1
1a17 2 19e0 1a18
1 1a1c 1 19db
2 19d9 1a20 1
1a2d 2 1a2c 1a2d
1 1a35 1 1a3b
2 1a3d 1a3e 2
1a2a 1a3f 1 1a46
1 1a41 2 1a59
1a5b 3 1a57 1a58
1a5d 2 1a66 1a68
2 1a6a 1a6c 2
1a6e 1a70 2 1a72
1a74 2 1a76 1a78
3 1a7d 1a7e 1a7f
2 1a7a 1a81 2
1a65 1a83 1 1a85
1 1a87 2 1a50
1a88 1 1a8c 1
1a4b 2 1a49 1a90
2 1aa0 1aa5 1
1aac 1 1aa7 2
1abf 1ac1 3 1abd
1abe 1ac3 2 1acc
1ace 2 1ad0 1ad2
2 1ad4 1ad6 2
1ad8 1ada 2 1adc
1ade 3 1ae3 1ae4
1ae5 2 1ae0 1ae7
2 1acb 1ae9 1
1aeb 1 1aed 2
1ab6 1aee 1 1af2
1 1ab1 2 1aaf
1af6 3 1afe 1aff
1b00 1 1b04 2
1b02 1b04 2 1b11
1b16 1 1b1d 1
1b18 2 1b30 1b32
3 1b2e 1b2f 1b34
2 1b3d 1b3f 2
1b41 1b43 2 1b45
1b47 2 1b49 1b4b
2 1b4d 1b4f 3
1b54 1b55 1b56 2
1b51 1b58 2 1b3c
1b5a 1 1b5c 1
1b5e 2 1b27 1b5f
1 1b63 1 1b22
2 1b20 1b67 1
1b6a 1 1b6d 5
1a23 1a93 1a97 1af9
1b6e 2 1b71 1b72
1 1b76 2 1b74
1b76 2 1b7b 1b7c
1 1b80 2 1b7e
1b80 2 1b87 1b88
1 1b8c 2 1b8a
1b8c 1 1bb0 2
1baf 1bb0 1 1bb8
1 1bbe 2 1bc0
1bc1 2 1bad 1bc2
1 1bc9 1 1bc4
2 1bdc 1bde 3
1bda 1bdb 1be0 2
1be9 1beb 2 1bed
1bef 2 1bf1 1bf3
2 1bf5 1bf7 2
1bf9 1bfb 3 1c00
1c01 1c02 2 1bfd
1c04 2 1be8 1c06
1 1c08 1 1c0a
2 1bd3 1c0b 1
1c0f 1 1bce 2
1bcc 1c13 1 1c16
1 1c3e 2 1c3d
1c3e 1 1c46 1
1c4c 2 1c4e 1c4f
2 1c3b 1c50 1
1c57 1 1c52 2
1c6a 1c6c 3 1c68
1c69 1c6e 2 1c77
1c79 2 1c7b 1c7d
2 1c7f 1c81 2
1c83 1c85 2 1c87
1c89 3 1c8e 1c8f
1c90 2 1c8b 1c92
2 1c76 1c94 1
1c96 1 1c98 2
1c61 1c99 1 1c9d
1 1c5c 2 1c5a
1ca1 1 1ca4 2
1ca7 1ca8 1 1ca9
2 1cad 1cae 1
1cb2 2 1cb0 1cb2
1 1cd2 2 1cd1
1cd2 1 1cda 1
1ce0 2 1ce2 1ce3
2 1ccf 1ce4 1
1ceb 1 1ce6 2
1cfe 1d00 3 1cfc
1cfd 1d02 2 1d0b
1d0d 2 1d0f 1d11
2 1d13 1d15 2
1d17 1d19 2 1d1b
1d1d 3 1d22 1d23
1d24 2 1d1f 1d26
2 1d0a 1d28 1
1d2a 1 1d2c 2
1cf5 1d2d 1 1d31
1 1cf0 2 1cee
1d35 1 1d38 2
1d3e 1d3f 1 1d43
2 1d41 1d43 2
1d48 1d49 2 1d4e
1d4f 2 1d4b 1d51
2 1d57 1d58 2
1d5d 1d5e 2 1d5a
1d60 2 1d65 1d66
1 1d6a 2 1d68
1d6a 1 1d86 2
1d85 1d86 1 1d8e
1 1d94 2 1d96
1d97 2 1d83 1d98
1 1d9f 1 1d9a
2 1db2 1db4 3
1db0 1db1 1db6 2
1dbf 1dc1 2 1dc3
1dc5 2 1dc7 1dc9
2 1dcb 1dcd 2
1dcf 1dd1 3 1dd6
1dd7 1dd8 2 1dd3
1dda 2 1dbe 1ddc
1 1dde 1 1de0
2 1da9 1de1 1
1de5 1 1da4 2
1da2 1de9 1 1dec
1 1e0c 2 1e0b
1e0c 1 1e14 1
1e1a 2 1e1c 1e1d
2 1e09 1e1e 1
1e25 1 1e20 2
1e38 1e3a 3 1e36
1e37 1e3c 2 1e45
1e47 2 1e49 1e4b
2 1e4d 1e4f 2
1e51 1e53 2 1e55
1e57 3 1e5c 1e5d
1e5e 2 1e59 1e60
2 1e44 1e62 1
1e64 1 1e66 2
1e2f 1e67 1 1e6b
1 1e2a 2 1e28
1e6f 1 1e72 2
1e75 1e76 3 1d54
1d63 1e77 1 1e89
2 1e88 1e89 1
1e91 1 1e97 2
1e99 1e9a 2 1e86
1e9b 1 1ea2 1
1e9d 2 1eb5 1eb7
3 1eb3 1eb4 1eb9
2 1ec2 1ec4 2
1ec6 1ec8 2 1eca
1ecc 2 1ece 1ed0
2 1ed2 1ed4 3
1ed9 1eda 1edb 2
1ed6 1edd 2 1ec1
1edf 1 1ee1 1
1ee3 2 1eac 1ee4
1 1ee8 1 1ea7
2 1ea5 1eec 1
1eef 4 1ef2 1d3c
1e79 1ef3 1 1efd
2 1efc 1efd 1
1f05 1 1f0b 2
1f0d 1f0e 2 1efa
1f0f 1 1f16 1
1f11 2 1f29 1f2b
3 1f27 1f28 1f2d
2 1f36 1f38 2
1f3a 1f3c 2 1f3e
1f40 2 1f42 1f44
2 1f46 1f48 3
1f4d 1f4e 1f4f 2
1f4a 1f51 2 1f35
1f53 1 1f55 1
1f57 2 1f20 1f58
1 1f5c 1 1f1b
2 1f19 1f60 1
1f73 2 1f72 1f73
1 1f7b 1 1f81
2 1f83 1f84 2
1f70 1f85 1 1f8c
1 1f87 2 1f9f
1fa1 3 1f9d 1f9e
1fa3 2 1fac 1fae
2 1fb0 1fb2 2
1fb4 1fb6 2 1fb8
1fba 2 1fbc 1fbe
3 1fc3 1fc4 1fc5
2 1fc0 1fc7 2
1fab 1fc9 1 1fcb
1 1fcd 2 1f96
1fce 1 1fd2 1
1f91 2 1f8f 1fd6
3 1ef4 1f63 1fd9
2 1fdc 1fdd 1
1fe8 2 1fe7 1fe8
1 1ff0 1 1ff6
2 1ff8 1ff9 2
1fe5 1ffa 1 2001
1 1ffc 2 2014
2016 3 2012 2013
2018 2 2021 2023
2 2025 2027 2
2029 202b 2 202d
202f 2 2031 2033
3 2038 2039 203a
2 2035 203c 2
2020 203e 1 2040
1 2042 2 200b
2043 1 2047 1
2006 2 2004 204b
1 2058 2 2057
2058 1 2060 1
2066 2 2068 2069
2 2055 206a 1
2071 1 206c 2
2084 2086 3 2082
2083 2088 2 2091
2093 2 2095 2097
2 2099 209b 2
209d 209f 2 20a1
20a3 3 20a8 20a9
20aa 2 20a5 20ac
2 2090 20ae 1
20b0 1 20b2 2
207b 20b3 1 20b7
1 2076 2 2074
20bb 2 20c1 20c2
1 20c6 2 20c4
20c6 2 20cb 20cc
1 20d0 2 20ce
20d0 2 20df 20e0
1 20e4 2 20e2
20e4 1 20e9 1
20ed 2 20ef 20f0
2 20f3 20f4 1
20f8 2 20f6 20f8
1 20fd 1 20ff
2 2106 2107 2
210b 210c 2 2110
2111 2 2115 2116
2 211a 211b 2
211f 2120 2 2124
2125 2 2129 212a
d 2103 2104 2109
210e 2113 2118 211d
2122 2127 212c 212d
212e 212f 2 2134
2135 1 213b 2
2137 213b 2 2146
2148 3 2144 2145
214a 2 2153 2155
2 2157 2159 2
215b 215d 1 2165
2 2167 2168 2
215f 216a 2 2152
216c 1 216e 1
2170 2 217a 217c
3 2178 2179 217e
2 2187 2189 2
2186 218b 1 218d
1 218f 2 2196
2197 3 2171 2190
2199 2 219d 219e
1 21a2 2 21a0
21a2 2 21ad 21af
3 21ab 21ac 21b1
2 21ba 21bc 2
21be 21c0 2 21c2
21c4 1 21ca 2
21cc 21cd 2 21c6
21cf 2 21d1 21d3
2 21d5 21d7 2
21d9 21db 2 21dd
21df 2 21b9 21e1
1 21e3 1 21e5
2 21ef 21f1 3
21ed 21ee 21f3 2
21fc 21fe 2 21fb
2200 1 2202 1
2204 2 220a 220b
3 220d 220e 220f
3 21e6 2205 2211
2 2214 2213 2
2217 2218 1 221c
2 221a 221c 1
2223 2 222a 222b
2 222d 2230 1
2233 2 2235 2236
20 223d 223e 223f
2240 2241 2242 2243
2244 2245 2246 2247
2248 2249 224a 224b
224c 224d 224e 224f
2250 2251 2252 2253
2254 2255 2256 2257
2258 2259 225a 225b
225c 2 2261 2262
1 2266 2 2264
2266 2 2271 2273
3 226f 2270 2275
2 227e 2280 2
2282 2284 2 2286
2288 1 2290 2
2292 2293 2 228a
2295 2 2297 2299
2 229b 229d 2
229f 22a1 2 22a3
22a5 2 227d 22a7
1 22a9 1 22ab
2 22b5 22b7 3
22b3 22b4 22b9 2
22c2 22c4 2 22c1
22c6 1 22c8 1
22ca 3 22d1 22d2
22d3 3 22ac 22cb
22d5 2 22d9 22da
1 22de 2 22dc
22de 2 22e9 22eb
3 22e7 22e8 22ed
2 22f6 22f8 2
22fa 22fc 2 22fe
2300 1 2308 2
230a 230b 2 2302
230d 2 230f 2311
2 2313 2315 2
2317 2319 2 231b
231d 2 22f5 231f
1 2321 1 2323
2 232d 232f 3
232b 232c 2331 2
233a 233c 2 2339
233e 1 2340 1
2342 3 2349 234a
234b 3 2324 2343
234d 2 2359 235a
2 235e 235f 5
2355 2356 2357 235c
2361 2 2352 2364
3 2366 234f 2367
a 20d8 20db 20f1
2100 2132 2215 2237
223a 225f 2368 1
236a 6 19ad 1fde
1fe1 204e 20be 236b
1 236d 5 1887
188b 188e 1977 236e
3 2372 2373 2374
1 2378 2 2376
2378 2 2395 239a
1 23a1 1 239c
2 23b4 23b6 3
23b2 23b3 23b8 2
23c1 23c3 2 23c5
23c7 2 23c9 23cb
2 23cd 23cf 2
23d1 23d3 3 23d8
23d9 23da 2 23d5
23dc 2 23c0 23de
1 23e0 1 23e2
2 23ab 23e3 1
23e7 1 23a6 2
23a4 23eb 1 23f6
2 23f2 23f6 1
23fa 1 2404 a
240c 240d 240e 240f
2410 2411 2412 2413
2414 2415 2 2407
2418 1 241a 2
23ee 241b 2 242a
242f 1 2436 1
2431 2 2449 244b
3 2447 2448 244d
2 2456 2458 2
245a 245c 2 245e
2460 2 2462 2464
2 2466 2468 3
246d 246e 246f 2
246a 2471 2 2455
2473 1 2475 1
2477 2 2440 2478
1 247c 1 243b
2 2439 2480 1
2483 3 2486 241d
2487 1 2488 3
248c 248d 248e 1
2492 2 2490 2492
2 24af 24b4 1
24bb 1 24b6 2
24ce 24d0 3 24cc
24cd 24d2 2 24db
24dd 2 24df 24e1
2 24e3 24e5 2
24e7 24e9 2 24eb
24ed 3 24f2 24f3
24f4 2 24ef 24f6
2 24da 24f8 1
24fa 1 24fc 2
24c5 24fd 1 2501
1 24c0 2 24be
2505 1 2510 2
250c 2510 1 2514
1 251e a 2526
2527 2528 2529 252a
252b 252c 252d 252e
252f 2 2521 2532
1 2534 2 2508
2535 2 2544 2549
1 2550 1 254b
2 2563 2565 3
2561 2562 2567 2
2570 2572 2 2574
2576 2 2578 257a
2 257c 257e 2
2580 2582 3 2587
2588 2589 2 2584
258b 2 256f 258d
1 258f 1 2591
2 255a 2592 1
2596 1 2555 2
2553 259a 1 259d
3 25a0 2537 25a1
1 25a3 2 25aa
25ab 1 25af 2
25ad 25af 2 25b6
25b7 1 25bb 2
25b9 25bb 2 25ca
25cb 1 25cf 2
25cd 25cf 2 25d4
25d5 1 25d8 1
25da 2 25dd 25de
1 25e2 2 25e0
25e2 2 25e7 25e8
1 25ec 2 25ea
25ec 2 25f3 25f4
1 25f8 2 25f6
25f8 2 25fd 25fe
1 2602 2 2600
2602 1 263f 2
263e 263f 1 2647
1 264d 2 264f
2650 2 263c 2651
1 2658 1 2653
2 266b 266d 3
2669 266a 266f 2
2678 267a 2 267c
267e 2 2680 2682
2 2684 2686 2
2688 268a 3 268f
2690 2691 2 268c
2693 2 2677 2695
1 2697 1 2699
2 2662 269a 1
269e 1 265d 2
265b 26a2 1 26a5
2 26aa 26ab 1
26af 2 26ad 26af
1 26e9 2 26e8
26e9 1 26f1 1
26f7 2 26f9 26fa
2 26e6 26fb 1
2702 1 26fd 2
2715 2717 3 2713
2714 2719 2 2722
2724 2 2726 2728
2 272a 272c 2
272e 2730 2 2732
2734 3 2739 273a
273b 2 2736 273d
2 2721 273f 1
2741 1 2743 2
270c 2744 1 2748
1 2707 2 2705
274c 1 274f 1
278a 2 2789 278a
1 2792 1 2798
2 279a 279b 2
2787 279c 1 27a3
1 279e 2 27b6
27b8 3 27b4 27b5
27ba 2 27c3 27c5
2 27c7 27c9 2
27cb 27cd 2 27cf
27d1 2 27d3 27d5
3 27da 27db 27dc
2 27d7 27de 2
27c2 27e0 1 27e2
1 27e4 2 27ad
27e5 1 27e9 1
27a8 2 27a6 27ed
1 27f0 3 27f3
2752 27f4 1 27f5
1 282e 2 282d
282e 1 2836 1
283c 2 283e 283f
2 282b 2840 1
2847 1 2842 2
285a 285c 3 2858
2859 285e 2 2867
2869 2 286b 286d
2 286f 2871 2
2873 2875 2 2877
2879 3 287e 287f
2880 2 287b 2882
2 2866 2884 1
2886 1 2888 2
2851 2889 1 288d
1 284c 2 284a
2891 1 2894 2
2897 2898 5 25c3
25c8 25db 2899 289c
1 289e 2 28a2
28a4 3 28a6 28a7
28a8 2 28b7 28bc
1 28c3 1 28be
2 28d6 28d8 3
28d4 28d5 28da 2
28e3 28e5 2 28e7
28e9 2 28eb 28ed
2 28ef 28f1 2
28f3 28f5 3 28fa
28fb 28fc 2 28f7
28fe 2 28e2 2900
1 2902 1 2904
2 28cd 2905 1
2909 1 28c8 2
28c6 290d 1 291d
2 291b 291d 2
2922 2923 1 2927
2 2925 2927 2
2962 2967 1 296e
1 2969 2 2981
2983 3 297f 2980
2985 2 298e 2990
2 2992 2994 2
2996 2998 2 299a
299c 2 299e 29a0
3 29a5 29a6 29a7
2 29a2 29a9 2
298d 29ab 1 29ad
1 29af 2 2978
29b0 1 29b4 1
2973 2 2971 29b8
1 29bb 1 29be
15 29ca 29cb 29cc
29cd 29ce 29cf 29d0
29d1 29d2 29d3 29d4
29d5 29d6 29d7 29d8
29d9 29da 29db 29dc
29dd 29de 1 29e9
1 29ef 2 29f1
29f2 1 29fa 2
29f6 29fa 1 29ff
1 2a01 2 2a04
2a05 1 2a09 2
2a07 2a09 2 2a0e
2a0f 1 2a19 2
2a18 2a19 1 2a21
1 2a27 2 2a29
2a2a 2 2a16 2a2b
1 2a32 1 2a2d
2 2a45 2a47 3
2a43 2a44 2a49 2
2a52 2a54 2 2a56
2a58 2 2a5a 2a5c
2 2a5e 2a60 2
2a62 2a64 3 2a69
2a6a 2a6b 2 2a66
2a6d 2 2a51 2a6f
1 2a71 1 2a73
2 2a3c 2a74 1
2a78 1 2a37 2
2a35 2a7c 1 2a87
2 2a86 2a87 1
2a8f 1 2a95 2
2a97 2a98 2 2a84
2a99 1 2aa0 1
2a9b 2 2ab3 2ab5
3 2ab1 2ab2 2ab7
2 2ac0 2ac2 2
2ac4 2ac6 2 2ac8
2aca 2 2acc 2ace
2 2ad0 2ad2 3
2ad7 2ad8 2ad9 2
2ad4 2adb 2 2abf
2add 1 2adf 1
2ae1 2 2aaa 2ae2
1 2ae6 1 2aa5
2 2aa3 2aea 1
2af5 2 2af4 2af5
1 2afd 1 2b03
2 2b05 2b06 2
2af2 2b07 1 2b0e
1 2b09 2 2b21
2b23 3 2b1f 2b20
2b25 2 2b2e 2b30
2 2b32 2b34 2
2b36 2b38 2 2b3a
2b3c 2 2b3e 2b40
3 2b45 2b46 2b47
2 2b42 2b49 2
2b2d 2b4b 1 2b4d
1 2b4f 2 2b18
2b50 1 2b54 1
2b13 2 2b11 2b58
1 2b65 2 2b64
2b65 1 2b6d 1
2b73 2 2b75 2b76
2 2b62 2b77 1
2b7e 1 2b79 2
2b91 2b93 3 2b8f
2b90 2b95 2 2b9e
2ba0 2 2ba2 2ba4
2 2ba6 2ba8 2
2baa 2bac 2 2bae
2bb0 3 2bb5 2bb6
2bb7 2 2bb2 2bb9
2 2b9d 2bbb 1
2bbd 1 2bbf 2
2b88 2bc0 1 2bc4
1 2b83 2 2b81
2bc8 1 2bd3 2
2bd2 2bd3 1 2bdb
1 2be1 2 2be3
2be4 2 2bd0 2be5
1 2bec 1 2be7
2 2bff 2c01 3
2bfd 2bfe 2c03 2
2c0c 2c0e 2 2c10
2c12 2 2c14 2c16
2 2c18 2c1a 2
2c1c 2c1e 3 2c23
2c24 2c25 2 2c20
2c27 2 2c0b 2c29
1 2c2b 1 2c2d
2 2bf6 2c2e 1
2c32 1 2bf1 2
2bef 2c36 1 2c41
2 2c40 2c41 1
2c49 1 2c4f 2
2c51 2c52 2 2c3e
2c53 1 2c5a 1
2c55 2 2c6d 2c6f
3 2c6b 2c6c 2c71
2 2c7a 2c7c 2
2c7e 2c80 2 2c82
2c84 2 2c86 2c88
2 2c8a 2c8c 3
2c91 2c92 2c93 2
2c8e 2c95 2 2c79
2c97 1 2c99 1
2c9b 2 2c64 2c9c
1 2ca0 1 2c5f
2 2c5d 2ca4 7
2a12 2a7f 2aed 2b5b
2bcb 2c39 2ca7 1
2cb6 2 2cb5 2cb6
1 2cbe 1 2cc4
2 2cc6 2cc7 2
2cb3 2cc8 1 2ccf
1 2cca 2 2ce2
2ce4 3 2ce0 2ce1
2ce6 2 2cef 2cf1
2 2cf3 2cf5 2
2cf7 2cf9 2 2cfb
2cfd 2 2cff 2d01
3 2d06 2d07 2d08
2 2d03 2d0a 2
2cee 2d0c 1 2d0e
1 2d10 2 2cd9
2d11 1 2d15 1
2cd4 2 2cd2 2d19
2 2d28 2d2d 1
2d34 1 2d2f 2
2d47 2d49 3 2d45
2d46 2d4b 2 2d54
2d56 2 2d58 2d5a
2 2d5c 2d5e 2
2d60 2d62 2 2d64
2d66 3 2d6b 2d6c
2d6d 2 2d68 2d6f
2 2d53 2d71 1
2d73 1 2d75 2
2d3e 2d76 1 2d7a
1 2d39 2 2d37
2d7e 1 2d89 2
2d85 2d89 2 2d8e
2d8f 1 2d93 2
2d91 2d93 1 2d98
1 2d9a 2 2da8
2dad 1 2db4 1
2daf 2 2dc7 2dc9
3 2dc5 2dc6 2dcb
2 2dd4 2dd6 2
2dd8 2dda 2 2ddc
2dde 2 2de0 2de2
2 2de4 2de6 3
2deb 2dec 2ded 2
2de8 2def 2 2dd3
2df1 1 2df3 1
2df5 2 2dbe 2df6
1 2dfa 1 2db9
2 2db7 2dfe 1
2e09 2 2e05 2e09
2 2e0f 2e10 2
2e12 2e15 1 2e1c
2 2e18 2e1c 1
2e2d 2 2e2c 2e2d
1 2e35 1 2e3b
2 2e3d 2e3e 2
2e2a 2e3f 1 2e46
1 2e41 2 2e59
2e5b 3 2e57 2e58
2e5d 2 2e66 2e68
2 2e6a 2e6c 2
2e6e 2e70 2 2e72
2e74 2 2e76 2e78
3 2e7d 2e7e 2e7f
2 2e7a 2e81 2
2e65 2e83 1 2e85
1 2e87 2 2e50
2e88 1 2e8c 1
2e4b 2 2e49 2e90
2 2e21 2e93 2
2e98 2e99 1 2e9f
2 2e9b 2e9f 2
2ea6 2ea7 4 2ea4
2ea9 2eac 2ead 1
2ec1 2 2ec0 2ec1
1 2ec9 1 2ecf
2 2ed1 2ed2 2
2ebe 2ed3 1 2eda
1 2ed5 2 2eed
2eef 3 2eeb 2eec
2ef1 2 2efa 2efc
2 2efe 2f00 2
2f02 2f04 2 2f06
2f08 2 2f0a 2f0c
3 2f11 2f12 2f13
2 2f0e 2f15 2
2ef9 2f17 1 2f19
1 2f1b 2 2ee4
2f1c 1 2f20 1
2edf 2 2edd 2f24
2 2eb0 2f27 1
2f3e 2 2f3d 2f3e
1 2f46 1 2f4c
2 2f4e 2f4f 2
2f3b 2f50 1 2f57
1 2f52 2 2f6a
2f6c 3 2f68 2f69
2f6e 2 2f77 2f79
2 2f7b 2f7d 2
2f7f 2f81 2 2f83
2f85 2 2f87 2f89
3 2f8e 2f8f 2f90
2 2f8b 2f92 2
2f76 2f94 1 2f96
1 2f98 2 2f61
2f99 1 2f9d 1
2f5c 2 2f5a 2fa1
1 2fa4 3 2fa7
2f2a 2fa8 1 2fb0
2 2fac 2fb0 2
2fb5 2fb6 1 2fba
2 2fb8 2fba 1
2fc6 2 2fc5 2fc6
1 2fce 1 2fd4
2 2fd6 2fd7 2
2fc3 2fd8 1 2fdf
1 2fda 2 2ff2
2ff4 3 2ff0 2ff1
2ff6 2 2fff 3001
2 3003 3005 2
3007 3009 2 300b
300d 2 300f 3011
3 3016 3017 3018
2 3013 301a 2
2ffe 301c 1 301e
1 3020 2 2fe9
3021 1 3025 1
2fe4 2 2fe2 3029
1 302c 1 302f
2 2fa9 3030 2
3054 3059 1 3060
1 305b 2 3073
3075 3 3071 3072
3077 2 3080 3082
2 3084 3086 2
3088 308a 2 308c
308e 2 3090 3092
3 3097 3098 3099
2 3094 309b 2
307f 309d 1 309f
1 30a1 2 306a
30a2 1 30a6 1
3065 2 3063 30aa
1 30b5 2 30b1
30b5 1 30b9 2
30c5 30c6 4 30c3
30c8 30cb 30cc 1
30d6 2 30d5 30d6
1 30de 1 30e4
2 30e6 30e7 2
30d3 30e8 1 30ef
1 30ea 2 3102
3104 3 3100 3101
3106 2 310f 3111
2 3113 3115 2
3117 3119 2 311b
311d 2 311f 3121
3 3126 3127 3128
2 3123 312a 2
310e 312c 1 312e
1 3130 2 30f9
3131 1 3135 1
30f4 2 30f2 3139
1 314c 2 314b
314c 1 3154 1
315a 2 315c 315d
2 3149 315e 1
3165 1 3160 2
3178 317a 3 3176
3177 317c 2 3185
3187 2 3189 318b
2 318d 318f 2
3191 3193 2 3195
3197 3 319c 319d
319e 2 3199 31a0
2 3184 31a2 1
31a4 1 31a6 2
316f 31a7 1 31ab
1 316a 2 3168
31af 1 31bf 2
31be 31bf 1 31c7
1 31cd 2 31cf
31d0 2 31bc 31d1
1 31d8 1 31d3
2 31eb 31ed 3
31e9 31ea 31ef 2
31f8 31fa 2 31fc
31fe 2 3200 3202
2 3204 3206 2
3208 320a 3 320f
3210 3211 2 320c
3213 2 31f7 3215
1 3217 1 3219
2 31e2 321a 1
321e 1 31dd 2
31db 3222 4 30cf
313c 31b2 3225 2
322c 322d 4 322a
322f 3232 3233 1
3242 2 3241 3242
1 324a 1 3250
2 3252 3253 2
323f 3254 1 325b
1 3256 2 326e
3270 3 326c 326d
3272 2 327b 327d
2 327f 3281 2
3283 3285 2 3287
3289 2 328b 328d
3 3292 3293 3294
2 328f 3296 2
327a 3298 1 329a
1 329c 2 3265
329d 1 32a1 1
3260 2 325e 32a5
1 32b8 2 32b7
32b8 1 32c0 1
32c6 2 32c8 32c9
2 32b5 32ca 1
32d1 1 32cc 2
32e4 32e6 3 32e2
32e3 32e8 2 32f1
32f3 2 32f5 32f7
2 32f9 32fb 2
32fd 32ff 2 3301
3303 3 3308 3309
330a 2 3305 330c
2 32f0 330e 1
3310 1 3312 2
32db 3313 1 3317
1 32d6 2 32d4
331b 3 3236 32a8
331e 2 3321 3322
2 30ad 3323 2
3325 3326 2 3329
332a 1 332e 2
332c 332e 2 3332
3333 1 3337 2
3335 3337 1 333b
2 334a 334f 1
3356 1 3351 2
3369 336b 3 3367
3368 336d 2 3376
3378 2 337a 337c
2 337e 3380 2
3382 3384 2 3386
3388 3 338d 338e
338f 2 338a 3391
2 3375 3393 1
3395 1 3397 2
3360 3398 1 339c
1 335b 2 3359
33a0 1 33ab 2
33a7 33ab 2 33b0
33b1 1 33b5 2
33b3 33b5 1 33c3
2 33c2 33c3 1
33cb 1 33d1 2
33d3 33d4 2 33c0
33d5 1 33dc 1
33d7 2 33ef 33f1
3 33ed 33ee 33f3
2 33fc 33fe 2
3400 3402 2 3404
3406 2 3408 340a
2 340c 340e 3
3413 3414 3415 2
3410 3417 2 33fb
3419 1 341b 1
341d 2 33e6 341e
1 3422 1 33e1
2 33df 3426 1
3429 1 3433 2
342f 3433 2 3438
3439 1 343d 2
343b 343d 2 3445
344b 1 3452 1
344d 2 3465 3467
3 3463 3464 3469
2 3472 3474 2
3476 3478 2 347a
347c 2 347e 3480
2 3482 3484 3
3489 348a 348b 2
3486 348d 2 3471
348f 1 3491 1
3493 2 345c 3494
1 3498 1 3457
2 3455 349c 2
34a9 34ab 3 34a7
34a8 34ad 2 34b6
34b8 2 34ba 34bc
2 34be 34c0 1
34c8 2 34ca 34cb
2 34c2 34cd 2
34cf 34d1 2 34d3
34d5 2 34d7 34d9
2 34db 34dd 2
34b5 34df 1 34e1
1 34e3 2 34ed
34ef 3 34eb 34ec
34f1 2 34fa 34fc
2 34f9 34fe 1
3500 1 3502 3
3509 350a 350b 4
349f 34e4 3503 350d
2 3510 350f 2
33a3 3511 1 3513
1 3517 a 351f
3520 3521 3522 3523
3524 3525 3526 3527
3528 1 3534 2
3533 3534 1 353c
1 3542 2 3544
3545 2 3531 3546
1 354d 1 3548
2 3560 3562 3
355e 355f 3564 2
356d 356f 2 3571
3573 2 3575 3577
2 3579 357b 2
357d 357f 3 3584
3585 3586 2 3581
3588 2 356c 358a
1 358c 1 358e
2 3557 358f 1
3593 1 3552 2
3550 3597 1 35a1
2 359f 35a1 2
35c1 35c2 2 35c4
35c7 1 35cc 2
35ca 35cc 2 35d1
35d2 2 35d7 35d8
1 35da 2 35d4
35da 2 35df 35e0
2 35e5 35e6 1
35e8 2 35e2 35e8
2 35f3 35f4 1
35f8 2 35f6 35f8
1 3623 2 3622
3623 1 362b 1
3631 2 3633 3634
2 3620 3635 1
363c 1 3637 2
364f 3651 3 364d
364e 3653 2 365c
365e 2 3660 3662
2 3664 3666 2
3668 366a 2 366c
366e 3 3673 3674
3675 2 3670 3677
2 365b 3679 1
367b 1 367d 2
3646 367e 1 3682
1 3641 2 363f
3686 1 3689 1
36b6 2 36b5 36b6
1 36be 1 36c4
2 36c6 36c7 2
36b3 36c8 1 36cf
1 36ca 2 36e2
36e4 3 36e0 36e1
36e6 2 36ef 36f1
2 36f3 36f5 2
36f7 36f9 2 36fb
36fd 2 36ff 3701
3 3706 3707 3708
2 3703 370a 2
36ee 370c 1 370e
1 3710 2 36d9
3711 1 3715 1
36d4 2 36d2 3719
1 371c 2 371f
3720 1 3721 1
3747 2 3746 3747
1 374f 1 3755
2 3757 3758 2
3744 3759 1 3760
1 375b 2 3773
3775 3 3771 3772
3777 2 3780 3782
2 3784 3786 2
3788 378a 2 378c
378e 2 3790 3792
3 3797 3798 3799
2 3794 379b 2
377f 379d 1 379f
1 37a1 2 376a
37a2 1 37a6 1
3765 2 3763 37aa
1 37ad 2 37b0
37b1 2 37b4 37b5
1 37b9 2 37b7
37b9 2 37be 37bf
1 37c3 2 37c1
37c3 1 37cb 2
37cf 37d0 1 37d4
2 37d2 37d4 2
37d9 37da 1 37dd
2 37e0 37df 2
37e7 37e8 2 37ec
37ed 2 37f1 37f2
2 37f6 37f7 2
37fb 37fc 2 3800
3801 2 3805 3806
2 380a 380b d
37e4 37e5 37ea 37ef
37f4 37f9 37fe 3803
3808 380d 380e 380f
3810 2 3815 3816
1 381c 2 3818
381c 2 3821 3827
1 382e 1 3829
2 3841 3843 3
383f 3840 3845 2
384e 3850 2 3852
3854 2 3856 3858
2 385a 385c 2
385e 3860 3 3865
3866 3867 2 3862
3869 2 384d 386b
1 386d 1 386f
2 3838 3870 1
3874 1 3833 2
3831 3878 2 3885
3887 3 3883 3884
3889 2 3892 3894
2 3896 3898 2
389a 389c 1 38a4
2 38a6 38a7 2
389e 38a9 2 3891
38ab 1 38ad 1
38af 2 38b9 38bb
3 38b7 38b8 38bd
2 38c6 38c8 2
38c5 38ca 1 38cc
1 38ce 2 38d5
38d6 4 387b 38b0
38cf 38d8 2 38dc
38dd 1 38e1 2
38df 38e1 2 38e6
38ec 1 38f3 1
38ee 2 3906 3908
3 3904 3905 390a
2 3913 3915 2
3917 3919 2 391b
391d 2 391f 3921
2 3923 3925 3
392a 392b 392c 2
3927 392e 2 3912
3930 1 3932 1
3934 2 38fd 3935
1 3939 1 38f8
2 38f6 393d 2
394a 394c 3 3948
3949 394e 2 3957
3959 2 395b 395d
2 395f 3961 1
3967 2 3969 396a
2 3963 396c 2
396e 3970 2 3972
3974 2 3976 3978
2 397a 397c 2
3956 397e 1 3980
1 3982 2 398c
398e 3 398a 398b
3990 2 3999 399b
2 3998 399d 1
399f 1 39a1 2
39a7 39a8 3 39aa
39ab 39ac 4 3940
3983 39a2 39ae 2
39b1 39b0 d 35a6
35a9 35ac 35af 35b2
35b5 35b8 35bb 35be
37b2 37e1 3813 39b2
2 39b5 39b6 1
39ba 2 39b8 39ba
2 39be 39bf 1
39c3 2 39c1 39c3
2 39c9 39ca 1
39ce 2 39cc 39ce
2 39d8 39d9 2
39d5 39dc 1 39de
2 39e1 39e2 1
39e6 2 39e4 39e6
2 39eb 39ec 2
39f1 39f2 1 39f4
2 39ee 39f4 2
39fc 39fd 1 3a00
2 3a05 3a06 2
3a0b 3a0c 2 3a08
3a0e 1 3a10 1
3a13 2 3a15 3a16
2 3a19 3a1a 1
3a1e 2 3a1c 3a1e
2 3a23 3a24 2
3a26 3a28 1 3a2b
2 3a31 3a32 1
3a34 2 3a36 3a38
1 3a3a 1 3a3d
2 3a3f 3a40 1
3a64 2 3a63 3a64
1 3a6c 1 3a72
2 3a74 3a75 2
3a61 3a76 1 3a7d
1 3a78 2 3a90
3a92 3 3a8e 3a8f
3a94 2 3a9d 3a9f
2 3aa1 3aa3 2
3aa5 3aa7 2 3aa9
3aab 2 3aad 3aaf
3 3ab4 3ab5 3ab6
2 3ab1 3ab8 2
3a9c 3aba 1 3abc
1 3abe 2 3a87
3abf 1 3ac3 1
3a82 2 3a80 3ac7
4 39df 3a17 3a41
3aca 2 3acd 3ace
2 3ada 3adb 1
3adf 2 3add 3adf
2 3ae3 3ae4 1
3ae8 2 3ae6 3ae8
2 3aec 3aed 1
3af1 2 3aef 3af1
1 3afd 2 3b04
3b05 2 3b07 3b0a
1 3b0d 2 3b0f
3b10 20 3b1a 3b1b
3b1c 3b1d 3b1e 3b1f
3b20 3b21 3b22 3b23
3b24 3b25 3b26 3b27
3b28 3b29 3b2a 3b2b
3b2c 3b2d 3b2e 3b2f
3b30 3b31 3b32 3b33
3b34 3b35 3b36 3b37
3b38 3b39 2 3b3e
3b3f 1 3b43 2
3b41 3b43 2 3b48
3b4e 1 3b55 1
3b50 2 3b68 3b6a
3 3b66 3b67 3b6c
2 3b75 3b77 2
3b79 3b7b 2 3b7d
3b7f 2 3b81 3b83
2 3b85 3b87 3
3b8c 3b8d 3b8e 2
3b89 3b90 2 3b74
3b92 1 3b94 1
3b96 2 3b5f 3b97
1 3b9b 1 3b5a
2 3b58 3b9f 2
3bac 3bae 3 3baa
3bab 3bb0 2 3bb9
3bbb 2 3bbd 3bbf
2 3bc1 3bc3 1
3bcb 2 3bcd 3bce
2 3bc5 3bd0 2
3bd2 3bd4 2 3bd6
3bd8 2 3bda 3bdc
2 3bde 3be0 2
3bb8 3be2 1 3be4
1 3be6 2 3bf0
3bf2 3 3bee 3bef
3bf4 2 3bfd 3bff
2 3bfc 3c01 1
3c03 1 3c05 3
3c0c 3c0d 3c0e 4
3ba2 3be7 3c06 3c10
2 3c14 3c15 1
3c19 2 3c17 3c19
2 3c1e 3c24 1
3c2b 1 3c26 2
3c3e 3c40 3 3c3c
3c3d 3c42 2 3c4b
3c4d 2 3c4f 3c51
2 3c53 3c55 2
3c57 3c59 2 3c5b
3c5d 3 3c62 3c63
3c64 2 3c5f 3c66
2 3c4a 3c68 1
3c6a 1 3c6c 2
3c35 3c6d 1 3c71
1 3c30 2 3c2e
3c75 2 3c82 3c84
3 3c80 3c81 3c86
2 3c8f 3c91 2
3c93 3c95 2 3c97
3c99 1 3ca1 2
3ca3 3ca4 2 3c9b
3ca6 2 3ca8 3caa
2 3cac 3cae 2
3cb0 3cb2 2 3cb4
3cb6 2 3c8e 3cb8
1 3cba 1 3cbc
2 3cc6 3cc8 3
3cc4 3cc5 3cca 2
3cd3 3cd5 2 3cd2
3cd7 1 3cd9 1
3cdb 3 3ce2 3ce3
3ce4 4 3c78 3cbd
3cdc 3ce6 2 3ce9
3ce8 1 3cf5 2
3cf4 3cf5 1 3cfd
1 3d03 2 3d05
3d06 2 3cf2 3d07
1 3d0e 1 3d09
2 3d21 3d23 3
3d1f 3d20 3d25 2
3d2e 3d30 2 3d32
3d34 2 3d36 3d38
2 3d3a 3d3c 2
3d3e 3d40 3 3d45
3d46 3d47 2 3d42
3d49 2 3d2d 3d4b
1 3d4d 1 3d4f
2 3d18 3d50 1
3d54 1 3d13 2
3d11 3d58 1 3d78
2 3d77 3d78 1
3d80 1 3d86 2
3d88 3d89 2 3d75
3d8a 1 3d91 1
3d8c 2 3da4 3da6
3 3da2 3da3 3da8
2 3db1 3db3 2
3db5 3db7 2 3db9
3dbb 2 3dbd 3dbf
2 3dc1 3dc3 3
3dc8 3dc9 3dca 2
3dc5 3dcc 2 3db0
3dce 1 3dd0 1
3dd2 2 3d9b 3dd3
1 3dd7 1 3d96
2 3d94 3ddb 1
3dee 2 3ded 3dee
1 3df6 1 3dfc
2 3dfe 3dff 2
3deb 3e00 1 3e07
1 3e02 2 3e1a
3e1c 3 3e18 3e19
3e1e 2 3e27 3e29
2 3e2b 3e2d 2
3e2f 3e31 2 3e33
3e35 2 3e37 3e39
3 3e3e 3e3f 3e40
2 3e3b 3e42 2
3e26 3e44 1 3e46
1 3e48 2 3e11
3e49 1 3e4d 1
3e0c 2 3e0a 3e51
1 3e64 2 3e63
3e64 1 3e6c 1
3e72 2 3e74 3e75
2 3e61 3e76 1
3e7d 1 3e78 2
3e90 3e92 3 3e8e
3e8f 3e94 2 3e9d
3e9f 2 3ea1 3ea3
2 3ea5 3ea7 2
3ea9 3eab 2 3ead
3eaf 3 3eb4 3eb5
3eb6 2 3eb1 3eb8
2 3e9c 3eba 1
3ebc 1 3ebe 2
3e87 3ebf 1 3ec3
1 3e82 2 3e80
3ec7 2 3ece 3ecf
2 3ed1 3ed3 2
3ed9 3eda 2 3edf
3ee0 2 3edc 3ee2
3 3eeb 3eec 3eed
3 3ef6 3ef7 3ef8
1 3f06 2 3f05
3f06 1 3f0e 1
3f14 2 3f16 3f17
2 3f03 3f18 1
3f1f 1 3f1a 2
3f32 3f34 3 3f30
3f31 3f36 2 3f3f
3f41 2 3f43 3f45
2 3f47 3f49 2
3f4b 3f4d 2 3f4f
3f51 3 3f56 3f57
3f58 2 3f53 3f5a
2 3f3e 3f5c 1
3f5e 1 3f60 2
3f29 3f61 1 3f65
1 3f24 2 3f22
3f69 2 3f6f 3f70
1 3f74 2 3f72
3f74 2 3f79 3f7a
1 3f7e 2 3f7c
3f7e 1 3f98 2
3f97 3f98 1 3fa0
1 3fa6 2 3fa8
3fa9 2 3f95 3faa
1 3fb1 1 3fac
2 3fc4 3fc6 3
3fc2 3fc3 3fc8 2
3fd1 3fd3 2 3fd5
3fd7 2 3fd9 3fdb
2 3fdd 3fdf 2
3fe1 3fe3 3 3fe8
3fe9 3fea 2 3fe5
3fec 2 3fd0 3fee
1 3ff0 1 3ff2
2 3fbb 3ff3 1
3ff7 1 3fb6 2
3fb4 3ffb 1 3ffe
2 4003 4004 1
4008 2 4006 4008
1 401b 2 401a
401b 1 4023 1
4029 2 402b 402c
2 4018 402d 1
4034 1 402f 2
4047 4049 3 4045
4046 404b 2 4054
4056 2 4058 405a
2 405c 405e 2
4060 4062 2 4064
4066 3 406b 406c
406d 2 4068 406f
2 4053 4071 1
4073 1 4075 2
403e 4076 1 407a
1 4039 2 4037
407e 2 4085 4086
2 4081 4089 2
408e 408f 1 4093
2 4091 4093 1
40a6 2 40a5 40a6
1 40ae 1 40b4
2 40b6 40b7 2
40a3 40b8 1 40bf
1 40ba 2 40d2
40d4 3 40d0 40d1
40d6 2 40df 40e1
2 40e3 40e5 2
40e7 40e9 2 40eb
40ed 2 40ef 40f1
3 40f6 40f7 40f8
2 40f3 40fa 2
40de 40fc 1 40fe
1 4100 2 40c9
4101 1 4105 1
40c4 2 40c2 4109
2 4110 4111 2
410c 4114 1 4123
2 4122 4123 1
412b 1 4131 2
4133 4134 2 4120
4135 1 413c 1
4137 2 414f 4151
3 414d 414e 4153
2 415c 415e 2
4160 4162 2 4164
4166 2 4168 416a
2 416c 416e 3
4173 4174 4175 2
4170 4177 2 415b
4179 1 417b 1
417d 2 4146 417e
1 4182 1 4141
2 413f 4186 2
418d 418e 2 4194
4195 3 4189 4191
4198 4 419a 408c
4116 419b 1a 3514
351a 352b 359a 359e
3acf 3ad2 3ad5 3ad8
3b11 3b14 3b17 3b3c
3cea 3d5b 3dde 3e54
3eca 3ed6 3ee5 3ee8
3ef0 3ef3 3efb 3f6c
419c 1 419e 4
2d9b 2e01 3327 419f
1 41a1 3 2d1c
2d81 41a2 2 41a4
41a5 3 41ab 41ac
41ad 5 29df 29f3
2a02 41a6 41b0 2
41b6 41bc 1 41c3
1 41be 2 41d6
41d8 3 41d4 41d5
41da 2 41e3 41e5
2 41e7 41e9 2
41eb 41ed 2 41ef
41f1 2 41f3 41f5
3 41fa 41fb 41fc
2 41f7 41fe 2
41e2 4200 1 4202
1 4204 2 41cd
4205 1 4209 1
41c8 2 41c6 420d
2 4216 4217 1
421b 2 4219 421b
2 4222 4223 2
4228 4229 1 422b
2 4225 422b 2
4230 4231 2 4236
4237 2 4233 4239
2 423e 423f 1
4243 2 4241 4243
2 4248 4249 1
424d 2 424b 424d
1 4253 2 4274
4275 1 4279 2
4277 4279 2 427d
427e 1 4282 2
4280 4282 1 4289
2 428d 428e 1
4292 2 4290 4292
2 4297 4298 1
429b 2 429e 429d
2 42a5 42a6 2
42aa 42ab 2 42af
42b0 2 42b4 42b5
2 42b9 42ba 2
42be 42bf 2 42c3
42c4 2 42c8 42c9
d 42a2 42a3 42a8
42ad 42b2 42b7 42bc
42c1 42c6 42cb 42cc
42cd 42ce 2 42d3
42d4 1 42da 2
42d6 42da 2 42e5
42e7 3 42e3 42e4
42e9 2 42f2 42f4
2 42f6 42f8 2
42fa 42fc 1 4304
2 4306 4307 2
42fe 4309 2 42f1
430b 1 430d 1
430f 2 4319 431b
3 4317 4318 431d
2 4326 4328 2
4325 432a 1 432c
1 432e 2 4335
4336 3 4310 432f
4338 2 433c 433d
1 4341 2 433f
4341 2 434c 434e
3 434a 434b 4350
2 4359 435b 2
435d 435f 2 4361
4363 1 4369 2
436b 436c 2 4365
436e 2 4370 4372
2 4374 4376 2
4378 437a 2 437c
437e 2 4358 4380
1 4382 1 4384
2 438e 4390 3
438c 438d 4392 2
439b 439d 2 439a
439f 1 43a1 1
43a3 2 43a9 43aa
3 43ac 43ad 43ae
3 4385 43a4 43b0
2 43b3 43b2 1
43cc 2 43cb 43cc
1 43d4 1 43da
2 43dc 43dd 2
43c9 43de 1 43e5
1 43e0 2 43f8
43fa 3 43f6 43f7
43fc 2 4405 4407
2 4409 440b 2
440d 440f 2 4411
4413 2 4415 4417
3 441c 441d 441e
2 4419 4420 2
4404 4422 1 4424
1 4426 2 43ef
4427 1 442b 1
43ea 2 43e8 442f
2 4444 4445 1
4449 2 4447 4449
2 444d 444e 1
4452 2 4450 4452
2 4456 4457 1
445b 2 4459 445b
2 4464 4465 1
4469 2 4467 4469
2 446d 446e 1
4472 2 4470 4472
2 4478 4479 1
447d 2 447b 447d
2 4487 4488 2
4484 448b 1 448d
2 4494 4495 2
4497 449a 3 448e
449d 44a0 1 44a4
2 44a6 44a7 20
44ae 44af 44b0 44b1
44b2 44b3 44b4 44b5
44b6 44b7 44b8 44b9
44ba 44bb 44bc 44bd
44be 44bf 44c0 44c1
44c2 44c3 44c4 44c5
44c6 44c7 44c8 44c9
44ca 44cb 44cc 44cd
2 44d2 44d3 1
44d7 2 44d5 44d7
2 44e2 44e4 3
44e0 44e1 44e6 2
44ef 44f1 2 44f3
44f5 2 44f7 44f9
1 4501 2 4503
4504 2 44fb 4506
2 4508 450a 2
450c 450e 2 4510
4512 2 4514 4516
2 44ee 4518 1
451a 1 451c 2
4526 4528 3 4524
4525 452a 2 4533
4535 2 4532 4537
1 4539 1 453b
3 4542 4543 4544
3 451d 453c 4546
2 454a 454b 1
454f 2 454d 454f
2 455a 455c 3
4558 4559 455e 2
4567 4569 2 456b
456d 2 456f 4571
1 4579 2 457b
457c 2 4573 457e
2 4580 4582 2
4584 4586 2 4588
458a 2 458c 458e
2 4566 4590 1
4592 1 4594 2
459e 45a0 3 459c
459d 45a2 2 45ab
45ad 2 45aa 45af
1 45b1 1 45b3
3 45ba 45bb 45bc
3 4595 45b4 45be
1 45e0 2 45df
45e0 1 45e8 1
45ee 2 45f0 45f1
2 45dd 45f2 1
45f9 1 45f4 2
460c 460e 3 460a
460b 4610 2 4619
461b 2 461d 461f
2 4621 4623 2
4625 4627 2 4629
462b 3 4630 4631
4632 2 462d 4634
2 4618 4636 1
4638 1 463a 2
4603 463b 1 463f
1 45fe 2 45fc
4643 1 4646 3
4649 45c0 464a 17
425d 4260 4263 4266
4269 426c 426f 4272
429f 42d1 43b4 43b9
43be 4432 4436 4439
443c 443f 4442 44a8
44ab 44d0 464b 1
464d 2 423c 464e
1 4650 2 465a
465f 1 4666 1
4661 2 4679 467b
3 4677 4678 467d
2 4686 4688 2
468a 468c 2 468e
4690 2 4692 4694
2 4696 4698 3
469d 469e 469f 2
469a 46a1 2 4685
46a3 1 46a5 1
46a7 2 4670 46a8
1 46ac 1 466b
2 4669 46b0 2
46b6 46b7 1 46bb
2 46b9 46bb 1
46ca 2 46c9 46ca
1 46d2 1 46d8
2 46da 46db 2
46c7 46dc 1 46e3
1 46de 2 46f6
46f8 3 46f4 46f5
46fa 2 4703 4705
2 4707 4709 2
470b 470d 2 470f
4711 2 4713 4715
3 471a 471b 471c
2 4717 471e 2
4702 4720 1 4722
1 4724 2 46ed
4725 1 4729 1
46e8 2 46e6 472d
1 4730 1 4733
4 4220 4651 46b3
4734 1 4736 1
473b 1 474c 2
474b 474c 1 4754
1 475a 2 475c
475d 2 4749 475e
1 4765 1 4760
2 4778 477a 3
4776 4777 477c 2
4785 4787 2 4789
478b 2 478d 478f
2 4791 4793 2
4795 4797 3 479c
479d 479e 2 4799
47a0 2 4784 47a2
1 47a4 1 47a6
2 476f 47a7 1
47ab 1 476a 2
4768 47af 2 47b5
47b6 1 47ba 2
47b8 47ba 2 47c4
47c9 1 47d0 1
47cb 2 47e3 47e5
3 47e1 47e2 47e7
2 47f0 47f2 2
47f4 47f6 2 47f8
47fa 2 47fc 47fe
2 4800 4802 3
4807 4808 4809 2
4804 480b 2 47ef
480d 1 480f 1
4811 2 47da 4812
1 4816 1 47d5
2 47d3 481a 2
4820 4821 1 4825
2 4823 4825 1
4833 2 4832 4833
1 483b 1 4841
2 4843 4844 2
4830 4845 1 484c
1 4847 2 485f
4861 3 485d 485e
4863 2 486c 486e
2 4870 4872 2
4874 4876 2 4878
487a 2 487c 487e
3 4883 4884 4885
2 4880 4887 2
486b 4889 1 488b
1 488d 2 4856
488e 1 4892 1
4851 2 484f 4896
1 48a3 2 48a2
48a3 1 48ab 1
48b1 2 48b3 48b4
2 48a0 48b5 1
48bc 1 48b7 2
48cf 48d1 3 48cd
48ce 48d3 2 48dc
48de 2 48e0 48e2
2 48e4 48e6 2
48e8 48ea 2 48ec
48ee 3 48f3 48f4
48f5 2 48f0 48f7
2 48db 48f9 1
48fb 1 48fd 2
48c6 48fe 1 4902
1 48c1 2 48bf
4906 2 4899 4909
1 490c 2 481d
490d 1 490f 2
4912 4913 1 4917
2 4915 4917 5
491f 4920 4921 4922
4923 2 4940 4945
1 494c 1 4947
2 495f 4961 3
495d 495e 4963 2
496c 496e 2 4970
4972 2 4974 4976
2 4978 497a 2
497c 497e 3 4983
4984 4985 2 4980
4987 2 496b 4989
1 498b 1 498d
2 4956 498e 1
4992 1 4951 2
494f 4996 1 49a1
2 499d 49a1 2
49a6 49a7 1 49ab
2 49a9 49ab 1
49c0 2 49bf 49c0
1 49c8 1 49ce
2 49d0 49d1 2
49bd 49d2 1 49d9
1 49d4 2 49ec
49ee 3 49ea 49eb
49f0 2 49f9 49fb
2 49fd 49ff 2
4a01 4a03 2 4a05
4a07 2 4a09 4a0b
3 4a10 4a11 4a12
2 4a0d 4a14 2
49f8 4a16 1 4a18
1 4a1a 2 49e3
4a1b 1 4a1f 1
49de 2 49dc 4a23
1 4a26 1 4a3d
2 4a3c 4a3d 1
4a45 1 4a4b 2
4a4d 4a4e 2 4a3a
4a4f 1 4a56 1
4a51 2 4a69 4a6b
3 4a67 4a68 4a6d
2 4a76 4a78 2
4a7a 4a7c 2 4a7e
4a80 2 4a82 4a84
2 4a86 4a88 3
4a8d 4a8e 4a8f 2
4a8a 4a91 2 4a75
4a93 1 4a95 1
4a97 2 4a60 4a98
1 4a9c 1 4a5b
2 4a59 4aa0 1
4aa3 2 4aa6 4aa7
3 4aaa 4aab 4aac
1 4ab0 2 4aae
4ab0 a 4abb 4abc
4abd 4abe 4abf 4ac0
4ac1 4ac2 4ac3 4ac4
2 4ab6 4ac7 1
4ac9 2 4aa8 4aca
1 4acc 1 4ace
2 4ad7 4ad8 1
4adc 2 4ada 4adc
1 4ae4 1 4ae6
5 491c 4926 4999
4acd 4ae7 1 4ae9
3 47b2 4910 4aea
1 4aec 2 4aef
4af0 1 4af4 2
4af2 4af4 1 4af8
2 4b01 4b02 1
4b06 2 4b04 4b06
2 4b13 4b18 1
4b1f 1 4b1a 2
4b32 4b34 3 4b30
4b31 4b36 2 4b3f
4b41 2 4b43 4b45
2 4b47 4b49 2
4b4b 4b4d 2 4b4f
4b51 3 4b56 4b57
4b58 2 4b53 4b5a
2 4b3e 4b5c 1
4b5e 1 4b60 2
4b29 4b61 1 4b65
1 4b24 2 4b22
4b69 2 4b6f 4b70
1 4b74 2 4b72
4b74 1 4b83 2
4b82 4b83 1 4b8b
1 4b91 2 4b93
4b94 2 4b80 4b95
1 4b9c 1 4b97
2 4baf 4bb1 3
4bad 4bae 4bb3 2
4bbc 4bbe 2 4bc0
4bc2 2 4bc4 4bc6
2 4bc8 4bca 2
4bcc 4bce 3 4bd3
4bd4 4bd5 2 4bd0
4bd7 2 4bbb 4bd9
1 4bdb 1 4bdd
2 4ba6 4bde 1
4be2 1 4ba1 2
4b9f 4be6 1 4bf7
2 4bf6 4bf7 1
4bff 1 4c05 2
4c07 4c08 2 4bf4
4c09 1 4c10 1
4c0b 2 4c23 4c25
3 4c21 4c22 4c27
2 4c30 4c32 2
4c34 4c36 2 4c38
4c3a 2 4c3c 4c3e
2 4c40 4c42 3
4c47 4c48 4c49 2
4c44 4c4b 2 4c2f
4c4d 1 4c4f 1
4c51 2 4c1a 4c52
1 4c56 1 4c15
2 4c13 4c5a 2
4be9 4c5d 1 4c60
2 4b6c 4c61 1
4c63 5 4c6a 4c6b
4c6c 4c6d 4c6e 2
4c8b 4c90 1 4c97
1 4c92 2 4caa
4cac 3 4ca8 4ca9
4cae 2 4cb7 4cb9
2 4cbb 4cbd 2
4cbf 4cc1 2 4cc3
4cc5 2 4cc7 4cc9
3 4cce 4ccf 4cd0
2 4ccb 4cd2 2
4cb6 4cd4 1 4cd6
1 4cd8 2 4ca1
4cd9 1 4cdd 1
4c9c 2 4c9a 4ce1
1 4cec 2 4ce8
4cec 2 4cf1 4cf2
1 4cf6 2 4cf4
4cf6 1 4d0b 2
4d0a 4d0b 1 4d13
1 4d19 2 4d1b
4d1c 2 4d08 4d1d
1 4d24 1 4d1f
2 4d37 4d39 3
4d35 4d36 4d3b 2
4d44 4d46 2 4d48
4d4a 2 4d4c 4d4e
2 4d50 4d52 2
4d54 4d56 3 4d5b
4d5c 4d5d 2 4d58
4d5f 2 4d43 4d61
1 4d63 1 4d65
2 4d2e 4d66 1
4d6a 1 4d29 2
4d27 4d6e 1 4d71
1 4d88 2 4d87
4d88 1 4d90 1
4d96 2 4d98 4d99
2 4d85 4d9a 1
4da1 1 4d9c 2
4db4 4db6 3 4db2
4db3 4db8 2 4dc1
4dc3 2 4dc5 4dc7
2 4dc9 4dcb 2
4dcd 4dcf 2 4dd1
4dd3 3 4dd8 4dd9
4dda 2 4dd5 4ddc
2 4dc0 4dde 1
4de0 1 4de2 2
4dab 4de3 1 4de7
1 4da6 2 4da4
4deb 1 4dee 2
4df1 4df2 3 4df5
4df6 4df7 1 4dfb
2 4df9 4dfb 2
4e00 4e01 1 4e05
2 4e03 4e05 a
4e12 4e13 4e14 4e15
4e16 4e17 4e18 4e19
4e1a 4e1b 2 4e0d
4e1e 1 4e20 2
4df3 4e21 1 4e23
5 4c64 4c67 4c71
4ce4 4e24 1 4e26
1 4e2a 1 4e32
2 4e30 4e32 1
4e39 1 4e3b 2
4e45 4e47 3 4e43
4e44 4e49 2 4e52
4e54 2 4e56 4e58
2 4e5a 4e5c 1
4e64 2 4e66 4e67
2 4e5e 4e69 2
4e6b 4e6d 2 4e6f
4e71 2 4e73 4e75
2 4e77 4e79 2
4e51 4e7b 1 4e7d
1 4e7f 3 4e86
4e87 4e88 56 459
478 47b 47e 481
484 487 48a 497
4b7 4ba 4bd 4c0
4c3 4c6 4c9 4cc
4cf 4d4 4e7 4ec
549 5c3 62e 6a9
6ac 70b 785 804
87f 8e0 95b 9ba
9cc a46 a49 a54
a5f a6a da4 da7
daa dad db0 db3
db6 dbb dce de3
e40 15f4 15f7 15fa
15fd 1602 1672 1683
16ff 1702 1705 1708
1769 17f1 25a2 289f
28ab 28ae 28b1 2910
2914 2917 291a 29bf
29c2 29c5 41b3 4210
4214 4737 473a 4aed
4e27 4e2d 4e3c 4e80
4e8a 2 4e97 4e99
3 4e95 4e96 4e9b
2 4ea4 4ea6 2
4ea3 4ea8 2 4eaf
4eb3 2 4eb5 4eb7
2 4eb9 4ebd 2
4eae 4ebf 2 4eaa
4ec1 1 4ec3 3
4ec4 4ecf 4ed3 1
4e8e 2 4ee6 4ee8
3 4ee4 4ee5 4eea
2 4ef3 4ef5 2
4ef2 4ef7 2 4efe
4f00 2 4f02 4f04
3 4f09 4f0a 4f0b
2 4f06 4f0d 2
4efd 4f0f 2 4ef9
4f11 1 4f13 4
4edd 4f14 4f1d 4f21
1 4f25 1 4ed8
2 4ed6 4f29 67
4b 52 59 60
67 6e 75 7c
83 8a 91 98
9d a2 a7 ac
b3 b8 bd c4
cb d0 d5 da
df e6 ed f4
fb 102 108 10e
114 11a 120 126
12c 132 138 13e
144 14a 150 156
15c 162 168 16e
174 17a 180 186
18c 192 198 19e
1a4 1aa 1b0 1b6
1bc 1c2 1c8 1ce
1d4 1da 1e0 1e6
1ec 1f2 1f9 200
206 20c 213 219
220 227 22d 233
238 23d 243 249
24f 255 25b 261
267 26d 273 279
27f 286 28d 294
2a2 2a8 2ae 2bb
2cf 347 351 
1
4
0 
4f30
0
1
140
e4
77
0 1 1 1 1 5 1 7
1 9 1 b 1 d 1 f
1 11 1 13 1 15 1 17
1 19 1 1b 1 1d 1 1f
1 21 1 23 1 25 1 27
1 29 1 2b 1 2d 1 2f
1 31 1 33 1 35 1 37
1 39 1 3b 1 3d 1 3f
1 41 1 43 1 45 1 47
1 49 1 4b 1 4d 1 4f
1 51 1 53 1 55 1 57
1 59 1 5b 1 5d 1 5f
1 61 1 63 1 65 1 67
1 69 1 6b 1 6d 1 6f
70 6f 72 6f 74 6f 76 6f
78 6f 7a 6f 7c 6f 7e 6f
80 6f 82 6f 84 6f 86 6f
88 6f 8a 6f 8c 6f 8e 6f
90 6f 92 6f 94 6f 96 6f
98 6f 9a 6f 9c 6f 9e 6f
a0 6f a2 6f a4 6f a6 6f
a8 6f aa 6f ac 6f ae 6f
b0 6f b2 6f b4 6f b6 6f
b8 6f ba 6f bc 6f be 1
c0 1 c2 1 c4 1 c6 1
c8 1 ca 1 cc 1 ce 1
d0 1 d2 1 d4 1 d6 1
d8 1 da 1 dc 1 de 1
e0 1 e2 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0

245 1 0
3b 1 0
33 1 0
215 1 0
1ee 1 0
24b 1 0
37 1 0
dc 1 0
164 1 0
208 1 0
182 1 0
116 1 0
9f 1 0
69 1 0
222 1 0
2f 1 0
251 1 0
263 1 0
21b 1 0
70 1 0
27 1 0
1e2 1 0
b 1 0
15e 1 0
77 1 0
23 1 0
1be 1 0
bf 1 0
e1 1 0
19a 1 0
d7 1 0
2a4 1 0
1ac 1 0
5b 1 0
20e 1 0
17c 1 0
1b2 1 0
194 1 0
104 1 0
2d5 1 4
22f 1 0
1d0 1 0
122 1 0
27b 1 0
1b 1 0
140 1 0
2c1 1 3
1fb 1 0
110 1 0
cd 1 0
a4 1 0
2b1 1 2
1f4 1 0
9a 1 0
17 1 0
134 1 0
257 1 0
23f 1 0
4d 1 0
3 1 0
1ca 1 0
26f 1 0
2 0 1
296 1 0
16a 1 0
281 1 0
f6 1 0
23a 1 0
d2 1 0
a9 1 0
229 1 0
128 1 0
25d 1 0
176 1 0
10a 1 0
2b 1 0
28f 1 0
288 1 0
1dc 1 0
7e 1 0
46 1 0
c6 1 0
f 1 0
7 1 0
13a 1 0
12e 1 0
2aa 1 0
188 1 0
152 1 0
1e8 1 0
1c4 1 0
93 1 0
ef 1 0
275 1 0
269 1 0
1b8 1 0
11c 1 0
202 1 0
1a0 1 0
b5 1 0
8c 1 0
85 1 0
170 1 0
62 1 0
18e 1 0
1f 1 0
235 1 0
1d6 1 0
1a6 1 0
ba 1 0
e8 1 0
14c 1 0
54 1 0
fd 1 0
146 1 0
13 1 0
34c 1 0
158 1 0
ae 1 0
0

/
SHOW ERRORS FUNCTION SP_A_RESTOPROF;
ALTER FUNCTION SP_A_RESTOPROF COMPILE DEBUG;
/*
#Stored Procedure  :SP_U_RATES
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_U_RATES ==
CREATE OR REPLACE FUNCTION SP_U_RATES wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
5b
2 :e:
1FUNCTION:
1SP_U_RATES:
1ID:
1VARCHAR2:
1REG_COST:
1NUMBER:
1SPEC_COST:
1REG_RATE:
1SPEC_RATE:
1RATE_ADJUSTMENT001:
1PLS_INTEGER:
1RES_ID001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_ID:
132:
1H_RES_ID:
1H_REG_COST:
10:
1H_REG_RATE:
1H_SPEC_COST:
1H_SPEC_RATE:
1H_RATE_ADJUSTMENT:
1H_FLAG:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_U_RATES(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
11:
134:
1+:
1FPUT:
1               ENTER:: SP_U_RATES(35) :: cplan.sql(12536) :: :
1IS NULL:
139:
1        RETURN_ERROR:: SP_U_RATES(40) :: cplan.sql(12541) :: :
1 :
1STATE:: :
1PMO_CNST:
1INVALID_PARAMS:
1 ID:: :
142:
1                EXIT:: SP_U_RATES(43) :: cplan.sql(12544) :: :
1PMODEFINES:
1PMO_RETURN_ERROR_SP:
1TRUE:
1SC_STATE:
1FC_U_RATES:
1=:
1100:
14:
166:
1       RETURN_STATUS:: SP_U_RATES(67) :: cplan.sql(12568) :: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
116:
173:
1           EXCEPTION:: SP_U_RATES(74) :: cplan.sql(12575) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1OTHERS:
1SQLCODE:
185:
1           EXCEPTION:: SP_U_RATES(86) :: cplan.sql(12587) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
2a9
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 :2 a0
6b 1c 81 b0 :2 a0 6b :2 a0 6b
6e a5 b a 10 :2 a0 6b 6e
:2 a0 6b 6e a5 b 6e 7e a0
b4 2e 6e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :3 a0 a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c a0 7e b4 2e
5a :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 19 3c a0 51 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 6b :9 a0 a5
b d :2 a0 6b 7e 51 b4 2e
:2 a0 6b 51 d b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 :2 a0 6b :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 :2 a0 6b
:2 6e 7e :2 a0 6b b4 2e 7e 6e
b4 2e 7e :2 a0 6b b4 2e a5
57 b7 19 3c :9 a0 12a e9 dd
:2 a0 65 b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c :7 a0 12a e9 dd :2 a0 65
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 68 4f 1d 17 b5 
2a9
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 6f 6b 59 77 84
80 56 8c 95 91 7f 9d 7c
a2 a6 aa ae b1 b5 d2 bd
c1 c4 c5 cd bc ef dd b9
e1 e2 ea dc 10b fa fe d9
106 f9 127 116 11a f6 122 115
143 132 136 112 13e 131 15f 14e
152 12e 15a 14d 17b 16a 16e 14a
176 169 197 186 18a 166 192 185
1b7 1a2 1a6 182 1aa 1b2 1a1 1be
1c2 19e 1c6 1ca 1ce 1d1 1d6 1d7
1 1d9 1de 1e2 1e6 1e9 1ee 1f2
1f6 1f9 1fe 1ff 201 206 209 20d
20e 213 218 21b 220 221 226 229
22d 22e 233 236 23b 23c 241 244
249 24a 24f 252 256 25a 25e 25f
261 266 267 269 26a 26f 272 277
278 27d 280 284 288 28c 28d 28f
294 295 297 298 29d 2a0 2a5 2a6
2ab 2ae 2b2 2b6 2ba 2bb 2bd 2c2
2c3 2c5 2c6 2cb 2ce 2d3 2d4 2d9
2dc 2e0 2e4 2e8 2e9 2eb 2f0 2f1
2f3 2f4 2f9 2fc 301 302 307 30a
30e 312 316 317 319 31e 31f 321
322 327 32a 32f 330 335 338 33d
33e 343 346 34a 34b 350 353 358
359 35e 361 366 367 36c 36d 372
374 378 37b 37f 383 386 38a 38e
391 394 399 39c 39f 3a2 3a3 3a8
3a9 1 3ab 3b0 3b4 3b8 3bb 3c0
3c5 3c8 3cc 3cd 3d2 3d3 3d8 3da
3de 3e1 3e5 3e8 3e9 3ee 3f1 3f5
3f9 3fc 400 404 407 40a 40f 412
415 418 419 41e 41f 1 421 426
42a 42e 431 436 43b 43e 442 443
448 44b 450 451 456 459 45e 45f
464 467 46b 46f 473 477 47a 47b
47d 482 483 485 486 48b 48e 493
494 499 49c 4a1 4a2 4a7 4aa 4ae
4af 4b4 4b7 4bc 4bd 4c2 4c3 4c8
4ca 4ce 4d1 4d5 4d9 4dc 4e0 4e4
4e7 4ea 4ef 4f2 4f5 4f8 4f9 4fe
4ff 1 501 506 50a 50e 511 516
51b 51e 522 523 528 529 52e 530
534 537 53b 53f 542 546 54a 54d
551 555 556 55b 55d 561 564 568
56b 56f 573 577 57b 57f 583 587
58b 58f 593 597 59b 59f 5a3 5a7
5ab 5af 5b3 5b7 5bb 5bf 5c3 5c7
5cb 5ce 5d2 5d6 5da 5de 5e2 5e6
5ea 5ee 5f2 5f3 5f5 5f9 5fd 601
604 607 60a 60b 610 614 618 61b
61e 622 624 628 62b 62f 633 636
63a 63e 641 644 649 64c 64f 652
653 658 659 1 65b 660 664 668
66b 670 675 678 67c 67d 682 685
68a 68b 690 693 698 699 69e 6a1
6a5 6a9 6ad 6b1 6b4 6b5 6b7 6bc
6bd 6bf 6c0 6c5 6c8 6cd 6ce 6d3
6d6 6db 6dc 6e1 6e4 6e8 6e9 6ee
6f1 6f6 6f7 6fc 6fd 702 704 708
70b 70f 713 716 71a 71e 721 725
729 72a 72f 731 735 739 73c 740
744 747 74b 74f 752 755 75a 75d
760 763 764 769 76a 1 76c 771
775 779 77c 781 786 789 78d 78e
793 794 799 79d 7a1 7a4 7a9 7ae
7b1 7b5 7b9 7bc 7bd 7c2 7c5 7ca
7cb 7d0 7d3 7d7 7db 7de 7df 7e4
7e5 7ea 7ec 7f0 7f3 7f7 7fb 7ff
803 807 80b 80f 813 817 823 828
82c 830 834 838 83a 83b 840 1
844 848 84c 84f 853 857 85b 85f
862 866 86a 86d 870 875 878 87b
87e 87f 884 885 1 887 88c 890
894 897 89c 8a1 8a4 8a8 8a9 8ae
8af 8b4 8b8 8bc 8bf 8c4 8c9 8cc
8d0 8d1 8d6 8d9 8de 8df 8e4 8e7
8eb 8ef 8f2 8f5 8f6 8f8 8f9 8fe
8ff 904 906 90a 90d 911 915 919
91d 921 925 929 935 93a 93e 942
946 94a 94c 950 952 95e 960 962
963 968 96c 96e 97a 97e 980 981
98a 
2a9
2
0 1 a 6 9 :3 6 f :3 6
10 :3 6 f :3 6 10 :3 6 19 :3 6
10 :2 6 1 3 a 13 a :2 1
4 9 12 11 :2 9 :2 4 d 16
15 :2 d :2 4 :2 f 19 f :2 4 :2 f
19 f :2 4 :2 10 1a 10 :2 4 :2 10
1a 10 :2 4 :2 16 25 16 :2 4 :2 b
1a b :2 4 e 17 :3 e 4 7
:2 11 23 :2 2d 3a :2 23 :3 7 :2 d 19
25 :2 2f 43 :2 25 52 58 5b :2 52
69 7d 8 :2 69 d 10 :2 69 13
16 :2 69 b e :2 69 13 16 1f
27 :2 1f 32 :2 16 :2 69 b e :2 69
13 16 1f 27 :2 1f 33 :2 16 :2 69
b e :2 69 13 16 1f 27 :2 1f
32 :2 16 :2 69 b e :2 69 13 16
1f 27 :2 1f 33 :2 16 :2 69 b e
:2 69 13 16 1f 27 :2 1f 3c :2 16
:2 69 b e :2 69 13 16 :2 69 1b
1e :2 69 28 2b :2 69 30 33 :2 69
:2 7 48 :2 4 7 :2 11 23 :2 2d 39
3c 4a 4c 4d :2 4a :2 23 :3 7 :2 d
12 1e 5b 5e :2 1e :2 7 50 :2 4
:4 8 7 a :2 14 26 :2 30 3c 40
4e 50 51 :2 4e :2 26 :3 a :2 10 15
21 5e 61 :2 21 6e 71 :2 21 75
78 :2 21 82 85 8e 96 :2 9f :2 8e
b0 :2 85 :2 21 b8 bb :2 21 c3 c6
:2 21 cb ce :2 21 d1 d4 :2 21 :2 a
54 :2 7 a :2 14 26 :2 30 3c 3f
4d 4f 50 :2 4d :2 26 :3 a :2 10 15
21 5e 61 :2 21 :2 a 53 :3 7 :2 12
26 :2 2f 3f 43 :2 7 14 :3 4 e
:2 4 c :2 4 10 :2 4 12 :2 4 13
:2 4 12 :2 4 13 :2 4 19 :2 4 :2 d
19 24 2a 36 43 4f 23 36
40 :2 19 4 7 :2 10 19 1b :2 19
7 :2 10 1c 7 1f :2 4 7 :2 11
23 :2 2d 39 3c 4a 4c 4d :2 4a
:2 23 :3 7 :2 d 12 1e 5b 5e :2 1e
6b 6e :2 1e 72 75 :2 1e 7f 82
8b 93 :2 9c :2 8b a7 :2 82 :2 1e af
b2 :2 1e ba bd :2 1e c2 c5 :2 1e
c8 cb :2 1e :2 7 50 :3 4 :2 f 21
:2 2a 34 38 :2 4 1 9 12 9
a :2 14 26 :2 30 3c 40 4e 50
51 :2 4e :2 26 :3 a :2 10 15 21 5e
61 :2 21 :3 a :2 10 15 39 5c 5f
:2 68 :2 39 76 79 :2 39 81 84 :2 8d
:2 39 :2 a 54 :3 7 c 15 1e 2c
a 13 1e f e 7 e 7
e 7 29 :2 4 :2 9 7 :2 10 1c
:2 7 :2 11 23 :2 2d 39 3d 4b 4d
4e :2 4b :2 23 :3 7 :2 d 12 1e 5b
5e :2 1e :3 7 :2 d 12 36 42 45
:2 36 4d 50 :2 36 5c 5f 66 6f
72 :2 5f :2 36 :2 7 51 :2 4 7 c
1b a 13 1e f e 7 e
7 e 7 :2 4 :4 10 :2 4 :8 1 
2a9
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 :4 7 :4 8
:4 9 2 :4 a :2 1
:7 c :7 d :6 e :6 f
:6 10 :6 11 :6 12 :6 13
:7 15 :b 17 :11 18 19
:2 18 :2 19 :2 18 :2 19
:2 18 :2 1a :2 18 :9 1a
:2 18 :2 1b :2 18 :9 1b
:2 18 :2 1c :2 18 :9 1c
:2 18 :2 1d :2 18 :9 1d
:2 18 :2 1e :2 18 :9 1e
:2 18 :2 1f :2 18 :2 1f
:2 18 :2 1f :2 18 :2 1f
:2 18 :2 1f :4 18 :3 17
:11 22 :b 23 :3 22 :5 26
:11 27 :30 28 :3 27 :11 2a
:b 2b :3 2a :a 2d :3 26
:3 30 :3 33 :3 34 :3 35
:3 36 :3 37 :3 38 :3 39
:9 3b :3 3c :3 3b :7 3e
:5 3f :3 3e :11 42 :30 43
:3 42 :a 45 16 :3 48
:11 49 :b 4a :17 4b :3 49
:2 4d :3 4e :3 4f 50
4e 4d 4e :3 51
:3 48 :2 52 :5 54 :11 55
:b 56 :18 57 :3 55 :2 59
5a :3 5b 5c 5a
59 5a :3 5d :2 53
:6 52 47 :7 1 
98c
4
:3 0 1 :3 0 2
:a 0 2a4 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 6
:3 0 5 :7 0 9
8 :3 0 6 :3 0
7 :7 0 d c
:3 0 d 7c 0
b 6 :3 0 8
:7 0 11 10 :3 0
6 :3 0 9 :7 0
15 14 :3 0 11
:2 0 f b :3 0
a :7 0 19 18
:3 0 4 :3 0 c
:7 0 1d 1c :3 0
d :3 0 e :3 0
f :2 0 4 21
22 0 1f 23
0 2a4 2 24
:2 0 11 :2 0 1b
4 :3 0 11 :2 0
19 27 29 :6 0
2c 2a 0 2a2
0 10 :6 0 14
:2 0 1f 4 :3 0
1d 2e 30 :6 0
33 31 0 2a2
0 12 :6 0 14
:2 0 21 6 :3 0
35 :7 0 39 36
37 2a2 0 13
:6 0 14 :2 0 23
6 :3 0 3b :7 0
3f 3c 3d 2a2
0 15 :6 0 14
:2 0 25 6 :3 0
41 :7 0 45 42
43 2a2 0 16
:6 0 14 :2 0 27
6 :3 0 47 :7 0
4b 48 49 2a2
0 17 :6 0 14
:2 0 29 b :3 0
4d :7 0 51 4e
4f 2a2 0 18
:6 0 59 5a 0
2b b :3 0 53
:7 0 57 54 55
2a2 0 19 :6 0
5f 60 0 2d
e :3 0 f :2 0
4 5b :7 0 5e
5c 0 2a2 0
1a :6 0 1b :3 0
1c :3 0 1b :3 0
1d :3 0 62 63
0 2 :4 0 2f
64 66 61 68
67 :2 0 1e :3 0
1f :3 0 6a 6b
0 20 :4 0 1b
:3 0 21 :3 0 6e
6f 0 2 :4 0
31 70 72 22
:4 0 23 :2 0 24
:3 0 33 75 77
:3 0 25 :4 0 23
:2 0 26 :4 0 36
7a 7c :3 0 23
:2 0 3 :3 0 39
7e 80 :3 0 23
:2 0 26 :4 0 3c
82 84 :3 0 23
:2 0 27 :4 0 3f
86 88 :3 0 23
:2 0 28 :3 0 29
:3 0 5 :3 0 42
8c 8e 2a :4 0
44 8b 91 47
8a 93 :3 0 23
:2 0 27 :4 0 4a
95 97 :3 0 23
:2 0 28 :3 0 29
:3 0 7 :3 0 4d
9b 9d 2a :4 0
4f 9a a0 52
99 a2 :3 0 23
:2 0 27 :4 0 55
a4 a6 :3 0 23
:2 0 28 :3 0 29
:3 0 8 :3 0 58
aa ac 2a :4 0
5a a9 af 5d
a8 b1 :3 0 23
:2 0 27 :4 0 60
b3 b5 :3 0 23
:2 0 28 :3 0 29
:3 0 9 :3 0 63
b9 bb 2a :4 0
65 b8 be 68
b7 c0 :3 0 23
:2 0 27 :4 0 6b
c2 c4 :3 0 23
:2 0 28 :3 0 29
:3 0 a :3 0 6e
c8 ca 2a :4 0
70 c7 cd 73
c6 cf :3 0 23
:2 0 27 :4 0 76
d1 d3 :3 0 23
:2 0 26 :4 0 79
d5 d7 :3 0 23
:2 0 c :3 0 7c
d9 db :3 0 23
:2 0 26 :4 0 7f
dd df :3 0 23
:2 0 2b :4 0 82
e1 e3 :3 0 85
6c e5 :2 0 e7
8a e8 69 e7
0 e9 8c 0
202 1b :3 0 1c
:3 0 ea eb 0
1b :3 0 2c :3 0
ed ee 0 2d
:2 0 2 :4 0 2e
:2 0 2f :2 0 2d
:2 0 8e f3 f5
:3 0 91 ef f7
ec f9 f8 :2 0
1e :3 0 30 :3 0
fb fc 0 20
:4 0 31 :4 0 23
:2 0 24 :3 0 95
100 102 :3 0 98
fd 104 :2 0 106
9b 107 fa 106
0 108 9d 0
202 3 :3 0 32
:2 0 9f 10a 10b
:3 0 10c :2 0 1b
:3 0 1c :3 0 10e
10f 0 1b :3 0
2c :3 0 111 112
0 11 :2 0 2
:4 0 33 :2 0 2f
:2 0 2d :2 0 a1
117 119 :3 0 a4
113 11b 110 11d
11c :2 0 1e :3 0
30 :3 0 11f 120
0 20 :4 0 34
:4 0 23 :2 0 24
:3 0 a8 124 126
:3 0 23 :2 0 35
:4 0 ab 128 12a
:3 0 23 :2 0 36
:4 0 ae 12c 12e
:3 0 23 :2 0 28
:3 0 29 :3 0 37
:3 0 38 :3 0 133
134 0 b1 132
136 2a :4 0 b3
131 139 b6 130
13b :3 0 23 :2 0
39 :4 0 b9 13d
13f :3 0 23 :2 0
26 :4 0 bc 141
143 :3 0 23 :2 0
3 :3 0 bf 145
147 :3 0 23 :2 0
26 :4 0 c2 149
14b :3 0 c5 121
14d :2 0 14f c8
150 11e 14f 0
151 ca 0 17b
1b :3 0 1c :3 0
152 153 0 1b
:3 0 2c :3 0 155
156 0 2d :2 0
2 :4 0 3a :2 0
2f :2 0 2d :2 0
cc 15b 15d :3 0
cf 157 15f 154
161 160 :2 0 1e
:3 0 30 :3 0 163
164 0 20 :4 0
3b :4 0 23 :2 0
24 :3 0 d3 168
16a :3 0 d6 165
16c :2 0 16e d9
16f 162 16e 0
170 db 0 17b
3c :3 0 3d :3 0
171 172 0 37
:3 0 38 :3 0 174
175 0 3 :3 0
3e :3 0 dd 173
179 :2 0 17b e1
17c 10d 17b 0
17d e5 0 202
19 :3 0 2d :2 0
17e 17f 0 202
10 :3 0 3 :3 0
181 182 0 202
12 :3 0 c :3 0
184 185 0 202
13 :3 0 5 :3 0
187 188 0 202
16 :3 0 7 :3 0
18a 18b 0 202
15 :3 0 8 :3 0
18d 18e 0 202
17 :3 0 9 :3 0
190 191 0 202
18 :3 0 a :3 0
193 194 0 202
e :3 0 3f :3 0
196 197 0 40
:3 0 10 :3 0 13
:3 0 16 :3 0 15
:3 0 17 :3 0 18
:3 0 12 :3 0 19
:3 0 e7 199 1a2
198 1a3 0 202
e :3 0 3f :3 0
1a5 1a6 0 41
:2 0 42 :2 0 f2
1a8 1aa :3 0 e
:3 0 3f :3 0 1ac
1ad 0 14 :2 0
1ae 1af 0 1b1
f5 1b2 1ab 1b1
0 1b3 f7 0
202 1b :3 0 1c
:3 0 1b4 1b5 0
1b :3 0 2c :3 0
1b7 1b8 0 43
:2 0 2 :4 0 44
:2 0 2f :2 0 2d
:2 0 f9 1bd 1bf
:3 0 fc 1b9 1c1
1b6 1c3 1c2 :2 0
1e :3 0 30 :3 0
1c5 1c6 0 20
:4 0 45 :4 0 23
:2 0 24 :3 0 100
1ca 1cc :3 0 23
:2 0 35 :4 0 103
1ce 1d0 :3 0 23
:2 0 36 :4 0 106
1d2 1d4 :3 0 23
:2 0 28 :3 0 29
:3 0 e :3 0 3f
:3 0 1d9 1da 0
109 1d8 1dc 2a
:4 0 10b 1d7 1df
10e 1d6 1e1 :3 0
23 :2 0 39 :4 0
111 1e3 1e5 :3 0
23 :2 0 26 :4 0
114 1e7 1e9 :3 0
23 :2 0 3 :3 0
117 1eb 1ed :3 0
23 :2 0 26 :4 0
11a 1ef 1f1 :3 0
11d 1c7 1f3 :2 0
1f5 120 1f6 1c4
1f5 0 1f7 122
0 202 3c :3 0
46 :3 0 1f8 1f9
0 e :3 0 3f
:3 0 1fb 1fc 0
3 :3 0 3e :3 0
124 1fa 200 :2 0
202 128 2a3 e
:3 0 47 :2 0 4
203 204 0 1b
:3 0 1c :3 0 206
207 0 1b :3 0
2c :3 0 209 20a
0 48 :2 0 2
:4 0 49 :2 0 2f
:2 0 2d :2 0 138
20f 211 :3 0 13b
20b 213 208 215
214 :2 0 1e :3 0
30 :3 0 217 218
0 20 :4 0 4a
:4 0 23 :2 0 24
:3 0 13f 21c 21e
:3 0 142 219 220
:2 0 239 1e :3 0
30 :3 0 222 223
0 20 :4 0 4b
:4 0 23 :2 0 e
:3 0 4c :3 0 228
229 0 145 227
22b :3 0 23 :2 0
39 :4 0 148 22d
22f :3 0 23 :2 0
e :3 0 4d :3 0
232 233 0 14b
231 235 :3 0 14e
224 237 :2 0 239
151 23a 216 239
0 23b 154 0
24b 4e :3 0 1a
:3 0 e :3 0 4c
:3 0 4f :3 0 e
:3 0 4d :3 0 3
:3 0 50 :4 0 51
1 :a 0 247 :2 0
24b 23d 245 :2 0
d :3 0 1a :3 0
249 :2 0 24b 156
24d 15a 24c 24b
:2 0 2a1 52 :3 0
e :3 0 3f :3 0
250 251 0 53
:3 0 252 253 0
299 1b :3 0 1c
:3 0 255 256 0
1b :3 0 2c :3 0
258 259 0 48
:2 0 2 :4 0 54
:2 0 2f :2 0 2d
:2 0 15c 25e 260
:3 0 15f 25a 262
257 264 263 :2 0
1e :3 0 30 :3 0
266 267 0 20
:4 0 55 :4 0 23
:2 0 24 :3 0 163
26b 26d :3 0 166
268 26f :2 0 289
1e :3 0 30 :3 0
271 272 0 20
:4 0 56 :4 0 23
:2 0 53 :3 0 169
276 278 :3 0 23
:2 0 57 :4 0 16c
27a 27c :3 0 23
:2 0 58 :3 0 59
:3 0 2d :2 0 5a
:2 0 16f 27f 283
173 27e 285 :3 0
176 273 287 :2 0
289 179 28a 265
289 0 28b 17c
0 299 4e :3 0
1a :3 0 4f :3 0
e :3 0 4d :3 0
3 :3 0 50 :4 0
5b 1 :a 0 295
:2 0 299 28d 293
:2 0 d :3 0 1a
:3 0 297 :2 0 299
17e 29c :3 0 29c
0 29c 29b 299
29a :6 0 29e 1
:3 0 183 2a0 185
29f 29e :2 0 2a1
187 :2 0 2a3 18a
2a3 2a2 202 2a1
:6 0 2a4 :2 0 2
24 2a3 2a7 :3 0
2a6 2a4 2a8 :8 0

194
4
:3 0 1 3 1
7 1 b 1
f 1 13 1
17 1 1b 7
6 a e 12
16 1a 1e 1
28 1 26 1
2f 1 2d 1
34 1 3a 1
40 1 46 1
4c 1 52 1
58 1 65 1
71 2 74 76
2 79 7b 2
7d 7f 2 81
83 2 85 87
1 8d 2 8f
90 2 89 92
2 94 96 1
9c 2 9e 9f
2 98 a1 2
a3 a5 1 ab
2 ad ae 2
a7 b0 2 b2
b4 1 ba 2
bc bd 2 b6
bf 2 c1 c3
1 c9 2 cb
cc 2 c5 ce
2 d0 d2 2
d4 d6 2 d8
da 2 dc de
2 e0 e2 4
6d 73 78 e4
1 e6 1 e8
2 f2 f4 3
f0 f1 f6 2
ff 101 2 fe
103 1 105 1
107 1 109 2
116 118 3 114
115 11a 2 123
125 2 127 129
2 12b 12d 1
135 2 137 138
2 12f 13a 2
13c 13e 2 140
142 2 144 146
2 148 14a 2
122 14c 1 14e
1 150 2 15a
15c 3 158 159
15e 2 167 169
2 166 16b 1
16d 1 16f 3
176 177 178 3
151 170 17a 1
17c 8 19a 19b
19c 19d 19e 19f
1a0 1a1 1 1a9
2 1a7 1a9 1
1b0 1 1b2 2
1bc 1be 3 1ba
1bb 1c0 2 1c9
1cb 2 1cd 1cf
2 1d1 1d3 1
1db 2 1dd 1de
2 1d5 1e0 2
1e2 1e4 2 1e6
1e8 2 1ea 1ec
2 1ee 1f0 2
1c8 1f2 1 1f4
1 1f6 3 1fd
1fe 1ff f e9
108 17d 180 183
186 189 18c 18f
192 195 1a4 1b3
1f7 201 2 20e
210 3 20c 20d
212 2 21b 21d
2 21a 21f 2
226 22a 2 22c
22e 2 230 234
2 225 236 2
221 238 1 23a
3 23b 246 24a
1 205 2 25d
25f 3 25b 25c
261 2 26a 26c
2 269 26e 2
275 277 2 279
27b 3 280 281
282 2 27d 284
2 274 286 2
270 288 1 28a
4 254 28b 294
298 1 29c 1
24f 2 24d 2a0
9 2b 32 38
3e 44 4a 50
56 5d 
1
4
0 
2a7
0
1
14
2
11
0 1 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 
2 0 1
7 1 0
4c 1 0
3 1 0
13 1 0
1b 1 0
40 1 0
3a 1 0
f 1 0
26 1 0
b 1 0
2d 1 0
17 1 0
52 1 0
46 1 0
34 1 0
58 1 0
0

/
SHOW ERRORS FUNCTION SP_U_RATES;
ALTER FUNCTION SP_U_RATES COMPILE DEBUG;
/*
#Stored Procedure  :SP_U_RATES_EDITED
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_U_RATES_EDITED ==
CREATE OR REPLACE FUNCTION SP_U_RATES_EDITED wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
73
2 :e:
1FUNCTION:
1SP_U_RATES_EDITED:
1ID:
1VARCHAR2:
1RATES_EDITED001:
1PLS_INTEGER:
1RES_ID001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_ID:
132:
1H_RESOURCE_ID:
1H_REG_COST:
1NUMBER:
10:
1H_SPEC_COST:
1H_REG_RATE:
1H_SPEC_RATE:
1H_FLAG:
1H_OLD_REG_COST:
1H_OLD_REG_SALE:
1H_OLD_SPEC_COST:
1H_OLD_SPEC_RATE:
1H_RATE_ID:
1H_RATE_ADJUSTMENT:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_U_RATES_EDITED(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
11:
131:
1+:
1FPUT:
1               ENTER:: SP_U_RATES_EDITED(32) :: cplan.sql(12643) :: :
1REGULAR_COST:
1REGULAR_RATE:
1SPECIAL_COST:
1SPECIAL_RATE:
1RATE_ADJUSTMENT:
1TASK_ASSIGNMENTS:
1TSK_ASSIGNMENT_ID:
1SELECT REGULAR_COST, REGULAR_RATE,:n      SPECIAL_COST, SPECIAL_RATE, RATE_AD+
1JUSTMENT:n      INTO   h_old_reg_cost, h_old_reg_sale ,:n      h_old_spec_cos+
1t,h_old_spec_rate, h_rate_adjustment:n      FROM TASK_ASSIGNMENTS:n      WHER+
1E TSK_ASSIGNMENT_ID = id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
153:
1           EXCEPTION:: SP_U_RATES_EDITED(54) :: cplan.sql(12665) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1RATE_ID:
1RESOURCES:
1RESOURCE_ID:
1SELECT RATE_ID INTO h_rate_id:n      FROM RESOURCES:n      WHERE RESOURCE_ID +
1= res_id001:
170:
1           EXCEPTION:: SP_U_RATES_EDITED(71) :: cplan.sql(12682) :: :
1REGULAR_SALE:
1SPECIAL_SALE:
1RATES:
1SELECT REGULAR_COST, REGULAR_SALE,:n      SPECIAL_COST, SPECIAL_SALE:n      I+
1NTO   h_reg_cost, h_reg_rate ,:n      h_spec_cost,h_spec_rate:n      FROM RAT+
1ES:n      WHERE RATE_ID = h_rate_id:
190:
1           EXCEPTION:: SP_U_RATES_EDITED(91) :: cplan.sql(12702) :: :
1RATES_EDITED:
1BITAND:
1UPDATE TASK_ASSIGNMENTS:n      SET RATES_EDITED = BITAND(RATES_EDITED,rates_e+
1dited001):n      WHERE TSK_ASSIGNMENT_ID = id:
1FOUND:
1TRUE:
1=:
1100:
1111:
1           EXCEPTION:: SP_U_RATES_EDITED(112) :: cplan.sql(12723) :: :
1>:
12:
1FC_U_RATES:
14:
1129:
1       RETURN_STATUS:: SP_U_RATES_EDITED(130) :: cplan.sql(12741) :: :
1STATE:: :
1 ID:: :
1PMODEFINES:
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
1136:
1           EXCEPTION:: SP_U_RATES_EDITED(137) :: cplan.sql(12748) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1148:
1           EXCEPTION:: SP_U_RATES_EDITED(149) :: cplan.sql(12760) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
3b4
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 1c 51 81 b0 a3 :2 a0 6b
1c 81 b0 :2 a0 6b :2 a0 6b 6e
a5 b a 10 :2 a0 6b 6e :2 a0
6b 6e a5 b 6e 7e a0 b4
2e 6e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
d a0 51 d :d a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :5 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:b a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :7 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :3 a0 51 a5 b 51 a5
b 5a 7e 51 b4 2e :2 a0 d
:2 a0 d b7 19 3c :3 a0 51 a5
b 51 a5 b 5a 7e 51 b4
2e :2 a0 d :2 a0 d b7 19 3c
:2 a0 6b :9 a0 a5 b d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b :2 a0
a5 57 b7 :2 a0 6b :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 :2 a0 6b :2 6e
7e :2 a0 6b b4 2e 7e 6e b4
2e 7e :2 a0 6b b4 2e a5 57
b7 19 3c :9 a0 12a e9 dd :2 a0
65 b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c :7 a0 12a e9 dd :2 a0 65 b7
a4 b1 11 4f b7 a6 9 a4
b1 11 68 4f 1d 17 b5 
3b4
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 30
56 5a 5e 62 65 69 86 71
75 78 79 81 70 a3 91 6d
95 96 9e 90 bf ae b2 8d
ba ad db ca ce aa d6 c9
f7 e6 ea c6 f2 e5 113 102
106 e2 10e 101 12f 11e 122 fe
12a 11d 14b 13a 13e 11a 146 139
167 156 15a 136 162 155 183 172
176 152 17e 171 19f 18e 192 16e
19a 18d 1bc 1aa 18a 1ae 1af 1b7
1a9 1d8 1c7 1cb 1a6 1d3 1c6 1f8
1e3 1e7 1c3 1eb 1f3 1e2 1ff 203
1df 207 20b 20f 212 217 218 1
21a 21f 223 227 22a 22f 233 237
23a 23f 240 242 247 24a 24e 24f
254 259 25c 261 262 267 26a 26e
26f 274 277 27c 27d 282 285 28a
28b 290 293 297 29b 29f 2a0 2a2
2a7 2a8 2aa 2ab 2b0 2b3 2b8 2b9
2be 2c1 2c6 2c7 2cc 2cf 2d3 2d4
2d9 2dc 2e1 2e2 2e7 2ea 2ef 2f0
2f5 2f6 2fb 2fd 301 304 308 30c
30f 313 317 31a 31d 322 325 328
32b 32c 331 332 1 334 339 33d
341 344 349 34e 351 355 356 35b
35c 361 363 367 36a 36e 372 376
37a 37d 381 385 389 38d 391 395
399 39d 3a1 3a5 3a9 3ad 3b1 3b5
3c1 3c5 3c9 3cc 3d0 3d4 3d6 3da
3de 3e2 3e5 3e9 3ed 3ef 3f0 3f5
1 3f9 3fd 401 404 408 40c 410
414 417 41b 41f 422 425 42a 42d
430 433 434 439 43a 1 43c 441
445 449 44c 451 456 459 45d 45e
463 466 46b 46c 471 474 479 47a
47f 482 486 487 48c 48f 494 495
49a 49d 4a1 4a5 4a8 4ab 4ac 4ae
4af 4b4 4b5 4ba 4bc 4c0 4c3 4c5
4c9 4cb 4d7 4d9 4db 4dc 4e1 4e5
4e7 4f3 4f5 4f9 4fd 501 505 509
515 519 51d 520 524 528 52a 52e
532 536 539 53d 541 543 544 549
1 54d 551 555 558 55c 560 564
568 56b 56f 573 576 579 57e 581
584 587 588 58d 58e 1 590 595
599 59d 5a0 5a5 5aa 5ad 5b1 5b2
5b7 5ba 5bf 5c0 5c5 5c8 5cd 5ce
5d3 5d6 5da 5db 5e0 5e3 5e8 5e9
5ee 5f1 5f5 5f9 5fc 5ff 600 602
603 608 609 60e 610 614 617 619
61d 61f 62b 62d 62f 630 635 639
63b 647 649 64d 651 655 659 65d
661 665 669 66d 671 675 681 685
689 68c 690 694 696 69a 69e 6a2
6a5 6a9 6ad 6af 6b0 6b5 1 6b9
6bd 6c1 6c4 6c8 6cc 6d0 6d4 6d7
6db 6df 6e2 6e5 6ea 6ed 6f0 6f3
6f4 6f9 6fa 1 6fc 701 705 709
70c 711 716 719 71d 71e 723 726
72b 72c 731 734 739 73a 73f 742
746 747 74c 74f 754 755 75a 75d
761 765 768 76b 76c 76e 76f 774
775 77a 77c 780 783 785 789 78b
797 799 79b 79c 7a1 7a5 7a7 7b3
7b5 7b9 7bd 7c1 7c5 7c9 7cd 7d1
7dd 7e1 7e6 7ea 7ed 7ee 7f3 7f7
7fb 7fe 801 805 807 80b 80f 812
815 819 81b 81f 823 826 828 82c
830 834 837 83b 83f 841 842 847
1 84b 84f 853 856 85a 85e 862
866 869 86d 871 874 877 87c 87f
882 885 886 88b 88c 1 88e 893
897 89b 89e 8a3 8a8 8ab 8af 8b0
8b5 8b8 8bd 8be 8c3 8c6 8cb 8cc
8d1 8d4 8d8 8d9 8de 8e1 8e6 8e7
8ec 8ef 8f3 8f7 8fa 8fd 8fe 900
901 906 907 90c 90e 912 915 917
91b 91d 929 92b 92d 92e 933 937
939 945 947 94b 94f 953 956 957
959 95c 95d 95f 962 965 968 969
96e 972 976 97a 97e 982 986 988
98c 98f 993 997 99b 99e 99f 9a1
9a4 9a5 9a7 9aa 9ad 9b0 9b1 9b6
9ba 9be 9c2 9c6 9ca 9ce 9d0 9d4
9d7 9db 9df 9e2 9e6 9ea 9ee 9f2
9f6 9fa 9fe a02 a06 a07 a09 a0d
a11 a15 a18 a1c a20 a23 a26 a2b
a2e a31 a34 a35 a3a a3b 1 a3d
a42 a46 a4a a4d a52 a57 a5a a5e
a5f a64 a67 a6c a6d a72 a75 a7a
a7b a80 a83 a87 a8b a8f a93 a96
a97 a99 a9e a9f aa1 aa2 aa7 aaa
aaf ab0 ab5 ab8 abd abe ac3 ac6
aca acb ad0 ad3 ad8 ad9 ade adf
ae4 ae6 aea aed af1 af5 af8 afc
b00 b03 b07 b0b b0c b11 b13 b17
b1b b1e b22 b26 b29 b2d b31 b34
b37 b3c b3f b42 b45 b46 b4b b4c
1 b4e b53 b57 b5b b5e b63 b68
b6b b6f b70 b75 b76 b7b b7f b83
b86 b8b b90 b93 b97 b9b b9e b9f
ba4 ba7 bac bad bb2 bb5 bb9 bbd
bc0 bc1 bc6 bc7 bcc bce bd2 bd5
bd9 bdd be1 be5 be9 bed bf1 bf5
bf9 c05 c0a c0e c12 c16 c1a c1c
c1d c22 1 c26 c2a c2e c31 c35
c39 c3d c41 c44 c48 c4c c4f c52
c57 c5a c5d c60 c61 c66 c67 1
c69 c6e c72 c76 c79 c7e c83 c86
c8a c8b c90 c91 c96 c9a c9e ca1
ca6 cab cae cb2 cb3 cb8 cbb cc0
cc1 cc6 cc9 ccd cd1 cd4 cd7 cd8
cda cdb ce0 ce1 ce6 ce8 cec cef
cf3 cf7 cfb cff d03 d07 d0b d17
d1c d20 d24 d28 d2c d2e d32 d34
d40 d42 d44 d45 d4a d4e d50 d5c
d60 d62 d63 d6c 
3b4
2
0 1 a 6 9 :3 6 16 :3 6
10 :2 6 1 3 a 13 a :2 1
4 9 12 11 :2 9 :2 4 12 1b
1a :2 12 :2 4 :2 f 19 f :2 4 :2 10
1a 10 :2 4 :2 f 19 f :2 4 :2 10
1a 10 :2 4 :2 b 1a b :2 4 :2 13
1d 13 :2 4 :2 13 1d 13 :2 4 :2 14
1e 14 :2 4 :2 14 1e 14 :2 4 e
17 16 :2 e :2 4 :2 16 25 16 :2 4
e 17 :3 e 4 7 :2 11 23 :2 2d
3a :2 23 :3 7 :2 d 19 25 :2 2f 43
:2 25 59 5f 62 :2 59 70 8b 8
:2 70 d 10 :2 70 13 16 :2 70 b
e :2 70 13 16 1f 27 :2 1f 39
:2 16 :2 70 b e :2 70 13 16 :2 70
1b 1e :2 70 28 2b :2 70 30 33
:2 70 :2 7 4f :2 4 7 :2 11 23 :2 2d
39 3c 51 53 54 :2 51 :2 23 :3 7
:2 d 12 1e 62 65 :2 1e :2 7 57
:3 4 15 :2 4 e 4 e 1c 7
15 23 e 1e 7 17 28 c
d 21 :2 7 :2 10 1c 7 4 c
a :2 13 1f a 1a :2 7 :2 c a
:2 13 1f a d :2 17 29 :2 33 3f
43 58 5a 5b :2 58 :2 29 :3 d :2 13
18 24 68 6b :2 24 78 7b :2 24
7f 82 :2 24 8e 91 :2 24 99 9c
:2 24 a8 ab b2 bb be :2 ab :2 24
:2 d 5e :2 a :2 7 :4 13 :2 7 4 :3 1
e 1b c d 1b :2 7 :2 10 1c
7 4 c a :2 13 1f a 1a
:2 7 :2 c a :2 13 1f a d :2 17
29 :2 33 3f 43 58 5a 5b :2 58
:2 29 :3 d :2 13 18 24 68 6b :2 24
78 7b :2 24 7f 82 :2 24 8e 91
:2 24 99 9c :2 24 a8 ab b2 bb
be :2 ab :2 24 :2 d 5e :2 a :2 7 :4 13
:2 7 4 :3 1 e 1c 7 15 e
1a 7 13 c d 17 :2 7 :2 10
1c 7 4 c a :2 13 1f a
1a :2 7 :2 c a :2 13 1f a d
:2 17 29 :2 33 3f 43 58 5a 5b
:2 58 :2 29 :3 d :2 13 18 24 68 6b
:2 24 78 7b :2 24 7f 82 :2 24 8e
91 :2 24 99 9c :2 24 a8 ab b2
bb be :2 ab :2 24 :2 d 5e :2 a :2 7
:4 13 :2 7 4 :3 1 e b 1a 21
2e d 21 7 e a 16 :3 14
a :2 13 1f a 1b a :2 13 1f
a :4 7 4 c a :2 13 1f a
1a :2 7 :2 c a :2 13 1f a d
:2 17 29 :2 33 3f 43 58 5b 5c
:2 58 :2 29 :3 d :2 13 18 24 69 6c
:2 24 79 7c :2 24 80 83 :2 24 8f
92 :2 24 9a 9d :2 24 a9 ac b3
bc bf :2 ac :2 24 :2 d 5f :2 a :2 7
:4 13 :2 7 4 :3 1 8 f 18 29
:2 f 2d :2 8 7 31 33 :2 31 7
19 :2 7 1a 7 35 :2 4 8 f
18 29 :2 f 2d :2 8 7 31 33
:2 31 7 19 :2 7 1a 7 35 :3 4
:2 d 19 24 2a 3a 4b 23 34
47 56 :2 19 4 7 :2 11 23 :2 2d
39 3c 51 54 55 :2 51 :2 23 :3 7
:2 d 12 1e 63 66 :2 1e 73 76
:2 1e 7a 7d :2 1e 87 8a 93 9b
:2 a4 :2 93 af :2 8a :2 1e b7 ba :2 1e
c2 c5 :2 1e ca cd :2 1e d0 d3
:2 1e :2 7 58 :3 4 :2 f 21 :2 2a 34
38 :2 4 1 9 12 9 a :2 14
26 :2 30 3c 40 55 58 59 :2 55
:2 26 :3 a :2 10 15 21 66 69 :2 21
:3 a :2 10 15 39 5c 5f :2 68 :2 39
76 79 :2 39 81 84 :2 8d :2 39 :2 a
5c :3 7 c 15 1e 2c a 13
1e f e 7 e 7 e 7
29 :2 4 :2 9 7 :2 10 1c :2 7 :2 11
23 :2 2d 39 3d 52 55 56 :2 52
:2 23 :3 7 :2 d 12 1e 63 66 :2 1e
:3 7 :2 d 12 36 42 45 :2 36 4d
50 :2 36 5c 5f 66 6f 72 :2 5f
:2 36 :2 7 59 :2 4 7 c 1b a
13 1e f e 7 e 7 e
7 :2 4 :4 10 :2 4 :8 1 
3b4
4
0 :2 1 :4 3 :4 4
:4 5 2 :4 6 :2 1
:7 8 :7 9 :6 a :6 b
:6 c :6 d :6 e :6 f
:6 10 :6 11 :6 12 :7 13
:6 14 :7 16 :b 18 :11 19
1a :2 19 :2 1a :2 19
:2 1a :2 19 :2 1b :2 19
:9 1b :2 19 :2 1c :2 19
:2 1c :2 19 :2 1c :2 19
:2 1c :2 19 :2 1c :4 19
:3 18 :11 1f :b 20 :3 1f
:3 23 :3 25 :2 28 :3 29
:2 2a :3 2b 2c :2 2d
28 :5 2e 27 30
:5 31 :3 30 :2 32 :5 34
:11 35 :24 36 :3 35 :2 33
:6 32 2f :3 17 :2 3c
3d :2 3e 3c :5 3f
3b 41 :5 42 :3 41
:2 43 :5 45 :11 46 :24 47
:3 46 :2 44 :6 43 40
:3 17 :2 4d :2 4e :2 4f
:2 50 51 :2 52 4d
:5 53 4c 55 :5 56
:3 55 :2 57 :5 59 :11 5a
:24 5b :3 5a :2 58 :6 57
54 :3 17 61 :4 62
:2 63 61 :6 64 :5 65
64 :5 67 66 :3 64
60 6a :5 6b :3 6a
:2 6c :5 6e :11 6f :24 70
:3 6f :2 6d :6 6c 69
:3 17 :e 75 :3 76 :3 77
:3 75 :e 79 :3 7a :3 7b
:3 79 :8 7e :4 7f :3 7e
:11 81 :30 82 :3 81 :a 84
17 :3 87 :11 88 :b 89
:17 8a :3 88 :2 8c :3 8d
:3 8e 8f 8d 8c
8d :3 90 :3 87 :2 91
:5 93 :11 94 :b 95 :18 96
:3 94 :2 98 99 :3 9a
9b 99 98 99
:3 9c :2 92 :6 91 86
:7 1 
d6e
4
:3 0 1 :3 0 2
:a 0 3af 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
:2 0 7 6 :3 0
5 :7 0 9 8
:3 0 4 :3 0 7
:7 0 d c :3 0
8 :3 0 9 :3 0
a :2 0 4 11
12 0 f 13
0 3af 2 14
:2 0 c :2 0 f
4 :3 0 c :2 0
d 17 19 :6 0
1c 1a 0 3ad
0 b :6 0 10
:2 0 13 4 :3 0
11 1e 20 :6 0
23 21 0 3ad
0 d :6 0 10
:2 0 15 f :3 0
25 :7 0 29 26
27 3ad 0 e
:6 0 10 :2 0 17
f :3 0 2b :7 0
2f 2c 2d 3ad
0 11 :6 0 10
:2 0 19 f :3 0
31 :7 0 35 32
33 3ad 0 12
:6 0 10 :2 0 1b
f :3 0 37 :7 0
3b 38 39 3ad
0 13 :6 0 10
:2 0 1d 6 :3 0
3d :7 0 41 3e
3f 3ad 0 14
:6 0 10 :2 0 1f
f :3 0 43 :7 0
47 44 45 3ad
0 15 :6 0 10
:2 0 21 f :3 0
49 :7 0 4d 4a
4b 3ad 0 16
:6 0 10 :2 0 23
f :3 0 4f :7 0
53 50 51 3ad
0 17 :6 0 c
:2 0 25 f :3 0
55 :7 0 59 56
57 3ad 0 18
:6 0 10 :2 0 29
4 :3 0 27 5b
5d :6 0 60 5e
0 3ad 0 19
:6 0 68 69 0
2b 6 :3 0 62
:7 0 66 63 64
3ad 0 1a :6 0
6e 6f 0 2d
9 :3 0 a :2 0
4 6a :7 0 6d
6b 0 3ad 0
1b :6 0 1c :3 0
1d :3 0 1c :3 0
1e :3 0 71 72
0 2 :4 0 2f
73 75 70 77
76 :2 0 1f :3 0
20 :3 0 79 7a
0 21 :4 0 1c
:3 0 22 :3 0 7d
7e 0 2 :4 0
31 7f 81 23
:4 0 24 :2 0 25
:3 0 33 84 86
:3 0 26 :4 0 24
:2 0 27 :4 0 36
89 8b :3 0 24
:2 0 3 :3 0 39
8d 8f :3 0 24
:2 0 27 :4 0 3c
91 93 :3 0 24
:2 0 28 :4 0 3f
95 97 :3 0 24
:2 0 29 :3 0 2a
:3 0 5 :3 0 42
9b 9d 2b :4 0
44 9a a0 47
99 a2 :3 0 24
:2 0 28 :4 0 4a
a4 a6 :3 0 24
:2 0 27 :4 0 4d
a8 aa :3 0 24
:2 0 7 :3 0 50
ac ae :3 0 24
:2 0 27 :4 0 53
b0 b2 :3 0 24
:2 0 2c :4 0 56
b4 b6 :3 0 59
7b b8 :2 0 ba
5e bb 78 ba
0 bc 60 0
30d 1c :3 0 1d
:3 0 bd be 0
1c :3 0 2d :3 0
c0 c1 0 2e
:2 0 2 :4 0 2f
:2 0 30 :2 0 2e
:2 0 62 c6 c8
:3 0 65 c2 ca
bf cc cb :2 0
1f :3 0 31 :3 0
ce cf 0 21
:4 0 32 :4 0 24
:2 0 25 :3 0 69
d3 d5 :3 0 6c
d0 d7 :2 0 d9
6f da cd d9
0 db 71 0
30d d :3 0 7
:3 0 dc dd 0
30d 14 :3 0 10
:2 0 df e0 0
30d 33 :3 0 34
:3 0 35 :3 0 36
:3 0 37 :3 0 15
:3 0 16 :3 0 17
:3 0 18 :3 0 1a
:3 0 38 :3 0 39
:3 0 3 :4 0 3a
1 :8 0 f5 9
:3 0 3b :3 0 f0
f1 0 3c :3 0
f2 f3 0 f5
73 148 3d :3 0
9 :3 0 3b :3 0
f7 f8 0 3c
:3 0 f9 fa 0
fc 76 fe 78
fd fc :2 0 146
3e :3 0 9 :3 0
3b :3 0 101 102
0 3c :3 0 103
104 0 13e 1c
:3 0 1d :3 0 106
107 0 1c :3 0
2d :3 0 109 10a
0 3f :2 0 2
:4 0 40 :2 0 30
:2 0 2e :2 0 7a
10f 111 :3 0 7d
10b 113 108 115
114 :2 0 1f :3 0
31 :3 0 117 118
0 21 :4 0 41
:4 0 24 :2 0 25
:3 0 81 11c 11e
:3 0 24 :2 0 42
:4 0 84 120 122
:3 0 24 :2 0 43
:4 0 87 124 126
:3 0 24 :2 0 3c
:3 0 8a 128 12a
:3 0 24 :2 0 44
:4 0 8d 12c 12e
:3 0 24 :2 0 45
:3 0 46 :3 0 2e
:2 0 47 :2 0 90
131 135 94 130
137 :3 0 97 119
139 :2 0 13b 9a
13c 116 13b 0
13d 9c 0 13e
9e 141 :3 0 141
0 141 140 13e
13f :6 0 143 2
:3 0 a1 145 a3
144 143 :2 0 146
a5 :2 0 148 0
148 147 f5 146
:6 0 30d 1 :3 0
48 :3 0 19 :3 0
49 :3 0 4a :3 0
7 :4 0 4b 1
:8 0 155 9 :3 0
3b :3 0 150 151
0 3c :3 0 152
153 0 155 a8
1a8 3d :3 0 9
:3 0 3b :3 0 157
158 0 3c :3 0
159 15a 0 15c
ab 15e ad 15d
15c :2 0 1a6 3e
:3 0 9 :3 0 3b
:3 0 161 162 0
3c :3 0 163 164
0 19e 1c :3 0
1d :3 0 166 167
0 1c :3 0 2d
:3 0 169 16a 0
3f :2 0 2 :4 0
4c :2 0 30 :2 0
2e :2 0 af 16f
171 :3 0 b2 16b
173 168 175 174
:2 0 1f :3 0 31
:3 0 177 178 0
21 :4 0 4d :4 0
24 :2 0 25 :3 0
b6 17c 17e :3 0
24 :2 0 42 :4 0
b9 180 182 :3 0
24 :2 0 43 :4 0
bc 184 186 :3 0
24 :2 0 3c :3 0
bf 188 18a :3 0
24 :2 0 44 :4 0
c2 18c 18e :3 0
24 :2 0 45 :3 0
46 :3 0 2e :2 0
47 :2 0 c5 191
195 c9 190 197
:3 0 cc 179 199
:2 0 19b cf 19c
176 19b 0 19d
d1 0 19e d3
1a1 :3 0 1a1 0
1a1 1a0 19e 19f
:6 0 1a3 4 :3 0
d6 1a5 d8 1a4
1a3 :2 0 1a6 da
:2 0 1a8 0 1a8
1a7 155 1a6 :6 0
30d 1 :3 0 33
:3 0 4e :3 0 35
:3 0 4f :3 0 e
:3 0 12 :3 0 11
:3 0 13 :3 0 50
:3 0 48 :3 0 19
:4 0 51 1 :8 0
1bb 9 :3 0 3b
:3 0 1b6 1b7 0
3c :3 0 1b8 1b9
0 1bb dd 20e
3d :3 0 9 :3 0
3b :3 0 1bd 1be
0 3c :3 0 1bf
1c0 0 1c2 e0
1c4 e2 1c3 1c2
:2 0 20c 3e :3 0
9 :3 0 3b :3 0
1c7 1c8 0 3c
:3 0 1c9 1ca 0
204 1c :3 0 1d
:3 0 1cc 1cd 0
1c :3 0 2d :3 0
1cf 1d0 0 3f
:2 0 2 :4 0 52
:2 0 30 :2 0 2e
:2 0 e4 1d5 1d7
:3 0 e7 1d1 1d9
1ce 1db 1da :2 0
1f :3 0 31 :3 0
1dd 1de 0 21
:4 0 53 :4 0 24
:2 0 25 :3 0 eb
1e2 1e4 :3 0 24
:2 0 42 :4 0 ee
1e6 1e8 :3 0 24
:2 0 43 :4 0 f1
1ea 1ec :3 0 24
:2 0 3c :3 0 f4
1ee 1f0 :3 0 24
:2 0 44 :4 0 f7
1f2 1f4 :3 0 24
:2 0 45 :3 0 46
:3 0 2e :2 0 47
:2 0 fa 1f7 1fb
fe 1f6 1fd :3 0
101 1df 1ff :2 0
201 104 202 1dc
201 0 203 106
0 204 108 207
:3 0 207 0 207
206 204 205 :6 0
209 6 :3 0 10b
20b 10d 20a 209
:2 0 20c 10f :2 0
20e 0 20e 20d
1bb 20c :6 0 30d
1 :3 0 38 :3 0
54 :3 0 55 :3 0
54 :3 0 5 :3 0
39 :3 0 3 :4 0
56 1 :8 0 22d
57 :4 0 218 :3 0
58 :3 0 59 :2 0
114 21b 21c :3 0
9 :3 0 3b :3 0
21e 21f 0 10
:2 0 220 221 0
223 117 22a 9
:3 0 3b :3 0 224
225 0 5a :2 0
226 227 0 229
119 22b 21d 223
0 22c 0 229
0 22c 11b 0
22d 11e 280 3d
:3 0 9 :3 0 3b
:3 0 22f 230 0
3c :3 0 231 232
0 234 121 236
123 235 234 :2 0
27e 3e :3 0 9
:3 0 3b :3 0 239
23a 0 3c :3 0
23b 23c 0 276
1c :3 0 1d :3 0
23e 23f 0 1c
:3 0 2d :3 0 241
242 0 3f :2 0
2 :4 0 5b :2 0
30 :2 0 2e :2 0
125 247 249 :3 0
128 243 24b 240
24d 24c :2 0 1f
:3 0 31 :3 0 24f
250 0 21 :4 0
5c :4 0 24 :2 0
25 :3 0 12c 254
256 :3 0 24 :2 0
42 :4 0 12f 258
25a :3 0 24 :2 0
43 :4 0 132 25c
25e :3 0 24 :2 0
3c :3 0 135 260
262 :3 0 24 :2 0
44 :4 0 138 264
266 :3 0 24 :2 0
45 :3 0 46 :3 0
2e :2 0 47 :2 0
13b 269 26d 13f
268 26f :3 0 142
251 271 :2 0 273
145 274 24e 273
0 275 147 0
276 149 279 :3 0
279 0 279 278
276 277 :6 0 27b
8 :3 0 14c 27d
14e 27c 27b :2 0
27e 150 :2 0 280
0 280 27f 22d
27e :6 0 30d 1
:3 0 55 :3 0 29
:3 0 5 :3 0 10
:2 0 153 283 286
2e :2 0 156 282
289 28a :2 0 5d
:2 0 10 :2 0 15b
28c 28e :3 0 15
:3 0 e :3 0 290
291 0 296 17
:3 0 11 :3 0 293
294 0 296 15e
297 28f 296 0
298 161 0 30d
55 :3 0 29 :3 0
5 :3 0 10 :2 0
163 29a 29d 5e
:2 0 166 299 2a0
2a1 :2 0 5d :2 0
10 :2 0 16b 2a3
2a5 :3 0 16 :3 0
12 :3 0 2a7 2a8
0 2ad 18 :3 0
13 :3 0 2aa 2ab
0 2ad 16e 2ae
2a6 2ad 0 2af
171 0 30d 9
:3 0 3b :3 0 2b0
2b1 0 5f :3 0
b :3 0 15 :3 0
17 :3 0 16 :3 0
18 :3 0 1a :3 0
d :3 0 14 :3 0
173 2b3 2bc 2b2
2bd 0 30d 1c
:3 0 1d :3 0 2bf
2c0 0 1c :3 0
2d :3 0 2c2 2c3
0 60 :2 0 2
:4 0 61 :2 0 30
:2 0 2e :2 0 17c
2c8 2ca :3 0 17f
2c4 2cc 2c1 2ce
2cd :2 0 1f :3 0
31 :3 0 2d0 2d1
0 21 :4 0 62
:4 0 24 :2 0 25
:3 0 183 2d5 2d7
:3 0 24 :2 0 42
:4 0 186 2d9 2db
:3 0 24 :2 0 63
:4 0 189 2dd 2df
:3 0 24 :2 0 29
:3 0 2a :3 0 9
:3 0 3b :3 0 2e4
2e5 0 18c 2e3
2e7 2b :4 0 18e
2e2 2ea 191 2e1
2ec :3 0 24 :2 0
64 :4 0 194 2ee
2f0 :3 0 24 :2 0
27 :4 0 197 2f2
2f4 :3 0 24 :2 0
3 :3 0 19a 2f6
2f8 :3 0 24 :2 0
27 :4 0 19d 2fa
2fc :3 0 1a0 2d2
2fe :2 0 300 1a3
301 2cf 300 0
302 1a5 0 30d
65 :3 0 66 :3 0
303 304 0 9
:3 0 3b :3 0 306
307 0 3 :3 0
58 :3 0 1a7 305
30b :2 0 30d 1ab
3ae 9 :3 0 67
:2 0 4 30e 30f
0 1c :3 0 1d
:3 0 311 312 0
1c :3 0 2d :3 0
314 315 0 3f
:2 0 2 :4 0 68
:2 0 30 :2 0 2e
:2 0 1b9 31a 31c
:3 0 1bc 316 31e
313 320 31f :2 0
1f :3 0 31 :3 0
322 323 0 21
:4 0 69 :4 0 24
:2 0 25 :3 0 1c0
327 329 :3 0 1c3
324 32b :2 0 344
1f :3 0 31 :3 0
32d 32e 0 21
:4 0 6a :4 0 24
:2 0 9 :3 0 6b
:3 0 333 334 0
1c6 332 336 :3 0
24 :2 0 64 :4 0
1c9 338 33a :3 0
24 :2 0 9 :3 0
6c :3 0 33d 33e
0 1cc 33c 340
:3 0 1cf 32f 342
:2 0 344 1d2 345
321 344 0 346
1d5 0 356 6d
:3 0 1b :3 0 9
:3 0 6b :3 0 6e
:3 0 9 :3 0 6c
:3 0 3 :3 0 6f
:4 0 70 1 :a 0
352 :2 0 356 348
350 :2 0 8 :3 0
1b :3 0 354 :2 0
356 1d7 358 1db
357 356 :2 0 3ac
3e :3 0 9 :3 0
3b :3 0 35b 35c
0 3c :3 0 35d
35e 0 3a4 1c
:3 0 1d :3 0 360
361 0 1c :3 0
2d :3 0 363 364
0 3f :2 0 2
:4 0 71 :2 0 30
:2 0 2e :2 0 1dd
369 36b :3 0 1e0
365 36d 362 36f
36e :2 0 1f :3 0
31 :3 0 371 372
0 21 :4 0 72
:4 0 24 :2 0 25
:3 0 1e4 376 378
:3 0 1e7 373 37a
:2 0 394 1f :3 0
31 :3 0 37c 37d
0 21 :4 0 43
:4 0 24 :2 0 3c
:3 0 1ea 381 383
:3 0 24 :2 0 44
:4 0 1ed 385 387
:3 0 24 :2 0 45
:3 0 46 :3 0 2e
:2 0 47 :2 0 1f0
38a 38e 1f4 389
390 :3 0 1f7 37e
392 :2 0 394 1fa
395 370 394 0
396 1fd 0 3a4
6d :3 0 1b :3 0
6e :3 0 9 :3 0
6c :3 0 3 :3 0
6f :4 0 73 1
:a 0 3a0 :2 0 3a4
398 39e :2 0 8
:3 0 1b :3 0 3a2
:2 0 3a4 1ff 3a7
:3 0 3a7 0 3a7
3a6 3a4 3a5 :6 0
3a9 1 :3 0 204
3ab 206 3aa 3a9
:2 0 3ac 208 :2 0
3ae 20b 3ae 3ad
30d 3ac :6 0 3af
:2 0 2 14 3ae
3b2 :3 0 3b1 3af
3b3 :8 0 
21a
4
:3 0 1 3 1
7 1 b 3
6 a e 1
18 1 16 1
1f 1 1d 1
24 1 2a 1
30 1 36 1
3c 1 42 1
48 1 4e 1
54 1 5c 1
5a 1 61 1
67 1 74 1
80 2 83 85
2 88 8a 2
8c 8e 2 90
92 2 94 96
1 9c 2 9e
9f 2 98 a1
2 a3 a5 2
a7 a9 2 ab
ad 2 af b1
2 b3 b5 4
7c 82 87 b7
1 b9 1 bb
2 c5 c7 3
c3 c4 c9 2
d2 d4 2 d1
d6 1 d8 1
da 2 ef f4
1 fb 1 f6
2 10e 110 3
10c 10d 112 2
11b 11d 2 11f
121 2 123 125
2 127 129 2
12b 12d 3 132
133 134 2 12f
136 2 11a 138
1 13a 1 13c
2 105 13d 1
141 1 100 2
fe 145 2 14f
154 1 15b 1
156 2 16e 170
3 16c 16d 172
2 17b 17d 2
17f 181 2 183
185 2 187 189
2 18b 18d 3
192 193 194 2
18f 196 2 17a
198 1 19a 1
19c 2 165 19d
1 1a1 1 160
2 15e 1a5 2
1b5 1ba 1 1c1
1 1bc 2 1d4
1d6 3 1d2 1d3
1d8 2 1e1 1e3
2 1e5 1e7 2
1e9 1eb 2 1ed
1ef 2 1f1 1f3
3 1f8 1f9 1fa
2 1f5 1fc 2
1e0 1fe 1 200
1 202 2 1cb
203 1 207 1
1c6 2 1c4 20b
1 21a 2 219
21a 1 222 1
228 2 22a 22b
2 217 22c 1
233 1 22e 2
246 248 3 244
245 24a 2 253
255 2 257 259
2 25b 25d 2
25f 261 2 263
265 3 26a 26b
26c 2 267 26e
2 252 270 1
272 1 274 2
23d 275 1 279
1 238 2 236
27d 2 284 285
2 287 288 1
28d 2 28b 28d
2 292 295 1
297 2 29b 29c
2 29e 29f 1
2a4 2 2a2 2a4
2 2a9 2ac 1
2ae 8 2b4 2b5
2b6 2b7 2b8 2b9
2ba 2bb 2 2c7
2c9 3 2c5 2c6
2cb 2 2d4 2d6
2 2d8 2da 2
2dc 2de 1 2e6
2 2e8 2e9 2
2e0 2eb 2 2ed
2ef 2 2f1 2f3
2 2f5 2f7 2
2f9 2fb 2 2d3
2fd 1 2ff 1
301 3 308 309
30a d bc db
de e1 148 1a8
20e 280 298 2af
2be 302 30c 2
319 31b 3 317
318 31d 2 326
328 2 325 32a
2 331 335 2
337 339 2 33b
33f 2 330 341
2 32c 343 1
345 3 346 351
355 1 310 2
368 36a 3 366
367 36c 2 375
377 2 374 379
2 380 382 2
384 386 3 38b
38c 38d 2 388
38f 2 37f 391
2 37b 393 1
395 4 35f 396
39f 3a3 1 3a7
1 35a 2 358
3ab e 1b 22
28 2e 34 3a
40 46 4c 52
58 5f 65 6c

1
4
0 
3b2
0
1
14
a
12
0 1 2 1 4 1 6 1
8 1 0 0 0 0 0 0
0 0 0 0 
42 1 0
61 1 0
5a 1 0
1d 1 0
54 1 0
3 1 0
b 1 0
2a 1 0
30 1 0
2 0 1
48 1 0
4e 1 0
16 1 0
3c 1 0
36 1 0
7 1 0
24 1 0
67 1 0
0

/
SHOW ERRORS FUNCTION SP_U_RATES_EDITED;
ALTER FUNCTION SP_U_RATES_EDITED COMPILE DEBUG;
/*
#Stored Procedure  :SP_U_TIMECODE2
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_U_TIMECODE2 ==
CREATE OR REPLACE FUNCTION SP_U_TIMECODE2 wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
55
2 :e:
1FUNCTION:
1SP_U_TIMECODE2:
1ASSIGN_ID:
1VARCHAR2:
1ID:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_TEMP_SC_STATE:
1PLS_INTEGER:
10:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_U_TIMECODE2(:
1':
1, :
1);:
1CHECK_TRACE:
11:
117:
1+:
1FPUT:
1               ENTER:: SP_U_TIMECODE2(18) :: cplan.sql(12802) :: :
1IS NULL:
18:
122:
1      RETURN_INVALID:: SP_U_TIMECODE2(23) :: cplan.sql(12807) :: :
1 :
1STATE:: :
1COALESCE:
1TO_CHAR:
1PMO_CNST:
1INVALID_PARAMS:
1NULL:
125:
1                EXIT:: SP_U_TIMECODE2(26) :: cplan.sql(12810) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1TASK_ASSIGNMENTS:
1TIMECODE2_ID:
1TSK_ASSIGNMENT_ID:
1UPDATE TASK_ASSIGNMENTS SET TIMECODE2_ID = id WHERE TSK_ASSIGNMENT_ID = assig+
1n_id:
1FOUND:
1=:
1SC_STATE:
1100:
1NO_DATA_FOUND:
1SQLCODE:
1OTHERS:
116:
145:
1           EXCEPTION:: SP_U_TIMECODE2(46) :: cplan.sql(12830) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
14:
151:
1       RETURN_STATUS:: SP_U_TIMECODE2(52) :: cplan.sql(12836) :: :
1 ID:: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
158:
1           EXCEPTION:: SP_U_TIMECODE2(59) :: cplan.sql(12843) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
170:
1           EXCEPTION:: SP_U_TIMECODE2(71) :: cplan.sql(12855) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
250
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d b4 :3 a0 6b 2c 6a
a3 a0 1c 51 81 b0 a3 :2 a0
6b 1c 81 b0 :2 a0 6b :2 a0 6b
6e a5 b a 10 :2 a0 6b 6e
:2 a0 6b 6e a5 b 6e 7e a0
b4 2e 6e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c a0 7e b4 2e :2 5a
a0 7e b4 2e :2 5a 52 10 :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b a0 a5 57 b7 19
3c :5 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b :2 a0 a5 57 b7 :2 a0 6b
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
:2 a0 6b :2 6e 7e :2 a0 6b b4 2e
7e 6e b4 2e 7e :2 a0 6b b4
2e a5 57 b7 19 3c :9 a0 12a
e9 dd :2 a0 65 b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c :7 a0 12a e9 dd
:2 a0 65 b7 a4 b1 11 4f b7
a6 9 a4 b1 11 68 4f 1d
17 b5 
250
2
0 3 7 23 1f 1e 2b 38
34 1b 40 33 45 49 4d 30
51 55 71 5d 61 69 6c 5c
91 7c 80 59 84 8c 7b 98
9c 78 a0 a4 a8 ab b0 b1
1 b3 b8 bc c0 c3 c8 cc
d0 d3 d8 d9 db e0 e3 e7
e8 ed f2 f5 fa fb 100 103
107 108 10d 110 115 116 11b 11e
123 124 129 12c 131 132 137 13a
13e 13f 144 147 14c 14d 152 155
15a 15b 160 161 166 168 16c 16f
173 177 17a 17e 182 185 188 18d
190 193 196 197 19c 19d 1 19f
1a4 1a8 1ac 1af 1b4 1b9 1bc 1c0
1c1 1c6 1c7 1cc 1ce 1d2 1d5 1d9
1dc 1dd 1e2 1e5 1e8 1ec 1ef 1f0
1f5 1f8 1 1fb 200 204 208 20b
20f 213 216 219 21e 221 224 227
228 22d 22e 1 230 235 239 23d
240 245 24a 24d 251 252 257 25a
25f 260 265 268 26d 26e 273 276
27a 27e 282 286 289 28a 28c 291
292 294 295 29a 29b 2a0 2a2 2a6
2a9 2ad 2b1 2b4 2b8 2bc 2bf 2c2
2c7 2ca 2cd 2d0 2d1 2d6 2d7 1
2d9 2de 2e2 2e6 2e9 2ee 2f3 2f6
2fa 2fb 300 301 306 308 30c 30f
313 317 31a 31e 322 325 329 32a
32f 331 335 338 33c 340 344 348
34c 358 35c 361 365 368 369 36e
372 376 379 37c 380 382 386 38a
38d 390 394 396 39a 39e 3a1 3a3
3a7 3ab 3af 3b2 3b6 3ba 3bc 3bd
3c2 1 3c6 3ca 3ce 3d1 3d5 3d9
3dd 3e1 3e4 3e8 3ec 3ef 3f2 3f7
3fa 3fd 400 401 406 407 1 409
40e 412 416 419 41e 423 426 42a
42b 430 433 438 439 43e 441 446
447 44c 44f 453 454 459 45c 461
462 467 46a 46e 472 475 478 479
47b 47c 481 482 487 489 48d 490
492 496 498 4a4 4a6 4a8 4a9 4ae
4b2 4b4 4c0 4c2 4c6 4ca 4cd 4d1
4d5 4d8 4db 4e0 4e3 4e6 4e9 4ea
4ef 4f0 1 4f2 4f7 4fb 4ff 502
507 50c 50f 513 514 519 51c 521
522 527 52a 52f 530 535 538 53c
540 544 548 54b 54c 54e 553 554
556 557 55c 55f 564 565 56a 56d
572 573 578 57b 57f 580 585 588
58d 58e 593 594 599 59b 59f 5a2
5a6 5aa 5ad 5b1 5b5 5b8 5bc 5c0
5c1 5c6 5c8 5cc 5d0 5d3 5d7 5db
5de 5e2 5e6 5e9 5ec 5f1 5f4 5f7
5fa 5fb 600 601 1 603 608 60c
610 613 618 61d 620 624 625 62a
62b 630 634 638 63b 640 645 648
64c 650 653 654 659 65c 661 662
667 66a 66e 672 675 676 67b 67c
681 683 687 68a 68e 692 696 69a
69e 6a2 6a6 6aa 6ae 6ba 6bf 6c3
6c7 6cb 6cf 6d1 6d2 6d7 1 6db
6df 6e3 6e6 6ea 6ee 6f2 6f6 6f9
6fd 701 704 707 70c 70f 712 715
716 71b 71c 1 71e 723 727 72b
72e 733 738 73b 73f 740 745 746
74b 74f 753 756 75b 760 763 767
768 76d 770 775 776 77b 77e 782
786 789 78c 78d 78f 790 795 796
79b 79d 7a1 7a4 7a8 7ac 7b0 7b4
7b8 7bc 7c0 7cc 7d1 7d5 7d9 7dd
7e1 7e3 7e7 7e9 7f5 7f7 7f9 7fa
7ff 803 805 811 815 817 818 821 
250
2
0 1 a 6 10 :3 6 9 :2 6
1 3 a 13 a :2 1 4 :2 14
23 14 :2 4 e 17 :3 e 4 7
:2 11 23 :2 2d 3a :2 23 :3 7 :2 d 19
25 :2 2f 43 :2 25 56 5c 5f :2 56
6d 85 8 :2 6d d 10 :2 6d 1a
1d :2 6d b e :2 6d 13 16 :2 6d
1b 1e :2 6d 21 24 :2 6d 29 2c
:2 6d :2 7 4c :2 4 7 :2 11 23 :2 2d
39 3c 4e 50 51 :2 4e :2 23 :3 7
:2 d 12 1e 5f 62 :2 1e :2 7 54
:2 4 :4 9 8 7 :4 22 21 20 :2 7
a :2 14 26 :2 30 3c 3f 51 53
54 :2 51 :2 26 :3 a :2 10 15 21 62
65 :2 21 72 75 :2 21 79 7c :2 21
86 89 92 9a :2 a3 :2 92 b4 :2 89
:2 21 :2 a 57 :2 7 a :2 14 26 :2 30
3c 3f 51 53 54 :2 51 :2 26 :3 a
:2 10 15 21 62 65 :2 21 :2 a 57
:3 7 :2 12 28 :2 31 41 :2 7 2f :2 4
e 23 32 3b 4f 7 e a
16 :3 14 a :2 13 1f a 1b a
:2 13 1f a :4 7 4 c a :2 13
1f a 1a :2 7 :2 c a :2 13 1f
a d :2 17 29 :2 33 3f 43 55
57 58 :2 55 :2 29 :3 d :2 13 18 24
65 68 :2 24 75 78 :2 24 7c 7f
:2 24 8b 8e :2 24 96 99 :2 24 a5
a8 af b8 bb :2 a8 :2 24 :2 d 5b
:2 a :2 7 :4 13 :2 7 4 :3 1 7 :2 11
23 :2 2d 39 3c 4e 50 51 :2 4e
:2 23 :3 7 :2 d 12 1e 5f 62 :2 1e
6f 72 :2 1e 76 79 :2 1e 83 86
8f 97 :2 a0 :2 8f ab :2 86 :2 1e b3
b6 :2 1e be c1 :2 1e c6 c9 :2 1e
d3 d6 :2 1e :2 7 54 :3 4 :2 f 21
:2 2a 34 3f :2 4 1 9 12 9
a :2 14 26 :2 30 3c 40 52 54
55 :2 52 :2 26 :3 a :2 10 15 21 62
65 :2 21 :3 a :2 10 15 39 5c 5f
:2 68 :2 39 76 79 :2 39 81 84 :2 8d
:2 39 :2 a 58 :3 7 c 15 1e 2c
a 13 1e f e 7 e 7
e 7 29 :2 4 :2 9 7 :2 10 1c
:2 7 :2 11 23 :2 2d 39 3d 4f 51
52 :2 4f :2 23 :3 7 :2 d 12 1e 5f
62 :2 1e :3 7 :2 d 12 36 42 45
:2 36 4d 50 :2 36 5c 5f 66 6f
72 :2 5f :2 36 :2 7 55 :2 4 7 c
1b a 13 1e f e 7 e
7 e 7 :2 4 :4 10 :2 4 :8 1 
250
4
0 :2 1 :4 3 :4 4
2 :4 5 :2 1 :6 7
:7 9 :b b :11 c d
:2 c :2 d :2 c :2 d
:2 c :2 e :2 c :2 e
:2 c :2 e :2 c :2 e
:2 c :2 e :4 c :3 b
:11 11 :b 12 :3 11 :e 15
:11 16 :20 17 :3 16 :11 19
:b 1a :3 19 :9 1c :3 15
:6 21 :6 22 :5 23 22
:5 25 24 :3 22 20
28 :5 29 :3 28 :2 2a
:5 2c :11 2d :24 2e :3 2d
:2 2b :6 2a 27 :3 a
:11 33 :30 34 :3 33 :a 36
a :3 39 :11 3a :b 3b
:17 3c :3 3a :2 3e :3 3f
:3 40 41 3f 3e
3f :3 42 :3 39 :2 43
:5 45 :11 46 :b 47 :18 48
:3 46 :2 4a 4b :3 4c
4d 4b 4a 4b
:3 4e :2 44 :6 43 38
:7 1 
823
4
:3 0 1 :3 0 2
:a 0 24b 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 d
e 0 7 4
:3 0 5 :7 0 9
8 :3 0 6 :3 0
7 :3 0 8 :2 0
4 b f 0
24b 2 10 :2 0
19 1a 0 :2 a
:3 0 13 :7 0 b
:2 0 17 14 15
249 0 9 :6 0
1f 20 0 c
7 :3 0 8 :2 0
4 1b :7 0 1e
1c 0 249 0
c :6 0 d :3 0
e :3 0 d :3 0
f :3 0 22 23
0 2 :4 0 e
24 26 21 28
27 :2 0 10 :3 0
11 :3 0 2a 2b
0 12 :4 0 d
:3 0 13 :3 0 2e
2f 0 2 :4 0
10 30 32 14
:4 0 15 :2 0 16
:3 0 12 35 37
:3 0 17 :4 0 15
:2 0 18 :4 0 15
3a 3c :3 0 15
:2 0 3 :3 0 18
3e 40 :3 0 15
:2 0 18 :4 0 1b
42 44 :3 0 15
:2 0 19 :4 0 1e
46 48 :3 0 15
:2 0 18 :4 0 21
4a 4c :3 0 15
:2 0 5 :3 0 24
4e 50 :3 0 15
:2 0 18 :4 0 27
52 54 :3 0 15
:2 0 1a :4 0 2a
56 58 :3 0 2d
2c 5a :2 0 5c
32 5d 29 5c
0 5e 34 0
1a9 d :3 0 e
:3 0 5f 60 0
d :3 0 1b :3 0
62 63 0 1c
:2 0 2 :4 0 1d
:2 0 1e :2 0 1c
:2 0 36 68 6a
:3 0 39 64 6c
61 6e 6d :2 0
10 :3 0 1f :3 0
70 71 0 12
:4 0 20 :4 0 15
:2 0 16 :3 0 3d
75 77 :3 0 40
72 79 :2 0 7b
43 7c 6f 7b
0 7d 45 0
1a9 3 :3 0 21
:2 0 47 7f 80
:3 0 81 :2 0 82
:2 0 5 :3 0 21
:2 0 49 85 86
:3 0 87 :2 0 88
:2 0 83 8a 89
:2 0 d :3 0 e
:3 0 8c 8d 0
d :3 0 1b :3 0
8f 90 0 22
:2 0 2 :4 0 23
:2 0 1e :2 0 1c
:2 0 4b 95 97
:3 0 4e 91 99
8e 9b 9a :2 0
10 :3 0 1f :3 0
9d 9e 0 12
:4 0 24 :4 0 15
:2 0 16 :3 0 52
a2 a4 :3 0 15
:2 0 25 :4 0 55
a6 a8 :3 0 15
:2 0 26 :4 0 58
aa ac :3 0 15
:2 0 27 :3 0 28
:3 0 29 :3 0 2a
:3 0 b1 b2 0
5b b0 b4 2b
:4 0 5d af b7
60 ae b9 :3 0
63 9f bb :2 0
bd 66 be 9c
bd 0 bf 68
0 e8 d :3 0
e :3 0 c0 c1
0 d :3 0 1b
:3 0 c3 c4 0
1c :2 0 2 :4 0
2c :2 0 1e :2 0
1c :2 0 6a c9
cb :3 0 6d c5
cd c2 cf ce
:2 0 10 :3 0 1f
:3 0 d1 d2 0
12 :4 0 2d :4 0
15 :2 0 16 :3 0
71 d6 d8 :3 0
74 d3 da :2 0
dc 77 dd d0
dc 0 de 79
0 e8 2e :3 0
2f :3 0 df e0
0 29 :3 0 2a
:3 0 e2 e3 0
30 :3 0 7b e1
e6 :2 0 e8 7e
e9 8b e8 0
ea 82 0 1a9
31 :3 0 32 :3 0
5 :3 0 33 :3 0
3 :4 0 34 1
:8 0 106 35 :4 0
f1 :3 0 30 :3 0
36 :2 0 86 f4
f5 :3 0 7 :3 0
37 :3 0 f7 f8
0 b :2 0 f9
fa 0 fc 89
103 7 :3 0 37
:3 0 fd fe 0
38 :2 0 ff 100
0 102 8b 104
f6 fc 0 105
0 102 0 105
8d 0 106 90
159 39 :3 0 7
:3 0 37 :3 0 108
109 0 3a :3 0
10a 10b 0 10d
93 10f 95 10e
10d :2 0 157 3b
:3 0 7 :3 0 37
:3 0 112 113 0
3a :3 0 114 115
0 14f d :3 0
e :3 0 117 118
0 d :3 0 1b
:3 0 11a 11b 0
3c :2 0 2 :4 0
3d :2 0 1e :2 0
1c :2 0 97 120
122 :3 0 9a 11c
124 119 126 125
:2 0 10 :3 0 1f
:3 0 128 129 0
12 :4 0 3e :4 0
15 :2 0 16 :3 0
9e 12d 12f :3 0
15 :2 0 25 :4 0
a1 131 133 :3 0
15 :2 0 3f :4 0
a4 135 137 :3 0
15 :2 0 3a :3 0
a7 139 13b :3 0
15 :2 0 40 :4 0
aa 13d 13f :3 0
15 :2 0 41 :3 0
42 :3 0 1c :2 0
43 :2 0 ad 142
146 b1 141 148
:3 0 b4 12a 14a
:2 0 14c b7 14d
127 14c 0 14e
b9 0 14f bb
152 :3 0 152 0
152 151 14f 150
:6 0 154 2 :3 0
be 156 c0 155
154 :2 0 157 c2
:2 0 159 0 159
158 106 157 :6 0
1a9 1 :3 0 d
:3 0 e :3 0 15b
15c 0 d :3 0
1b :3 0 15e 15f
0 44 :2 0 2
:4 0 45 :2 0 1e
:2 0 1c :2 0 c5
164 166 :3 0 c8
160 168 15d 16a
169 :2 0 10 :3 0
1f :3 0 16c 16d
0 12 :4 0 46
:4 0 15 :2 0 16
:3 0 cc 171 173
:3 0 15 :2 0 25
:4 0 cf 175 177
:3 0 15 :2 0 26
:4 0 d2 179 17b
:3 0 15 :2 0 27
:3 0 28 :3 0 7
:3 0 37 :3 0 180
181 0 d5 17f
183 2b :4 0 d7
17e 186 da 17d
188 :3 0 15 :2 0
47 :4 0 dd 18a
18c :3 0 15 :2 0
18 :4 0 e0 18e
190 :3 0 15 :2 0
3 :3 0 e3 192
194 :3 0 15 :2 0
18 :4 0 e6 196
198 :3 0 e9 16e
19a :2 0 19c ec
19d 16b 19c 0
19e ee 0 1a9
2e :3 0 48 :3 0
19f 1a0 0 7
:3 0 37 :3 0 1a2
1a3 0 3 :3 0
30 :3 0 f0 1a1
1a7 :2 0 1a9 f4
24a 7 :3 0 49
:2 0 4 1aa 1ab
0 d :3 0 e
:3 0 1ad 1ae 0
d :3 0 1b :3 0
1b0 1b1 0 3c
:2 0 2 :4 0 4a
:2 0 1e :2 0 1c
:2 0 fb 1b6 1b8
:3 0 fe 1b2 1ba
1af 1bc 1bb :2 0
10 :3 0 1f :3 0
1be 1bf 0 12
:4 0 4b :4 0 15
:2 0 16 :3 0 102
1c3 1c5 :3 0 105
1c0 1c7 :2 0 1e0
10 :3 0 1f :3 0
1c9 1ca 0 12
:4 0 4c :4 0 15
:2 0 7 :3 0 4d
:3 0 1cf 1d0 0
108 1ce 1d2 :3 0
15 :2 0 47 :4 0
10b 1d4 1d6 :3 0
15 :2 0 7 :3 0
4e :3 0 1d9 1da
0 10e 1d8 1dc
:3 0 111 1cb 1de
:2 0 1e0 114 1e1
1bd 1e0 0 1e2
117 0 1f2 4f
:3 0 c :3 0 7
:3 0 4d :3 0 50
:3 0 7 :3 0 4e
:3 0 5 :3 0 51
:4 0 52 1 :a 0
1ee :2 0 1f2 1e4
1ec :2 0 6 :3 0
c :3 0 1f0 :2 0
1f2 119 1f4 11d
1f3 1f2 :2 0 248
3b :3 0 7 :3 0
37 :3 0 1f7 1f8
0 3a :3 0 1f9
1fa 0 240 d
:3 0 e :3 0 1fc
1fd 0 d :3 0
1b :3 0 1ff 200
0 3c :2 0 2
:4 0 53 :2 0 1e
:2 0 1c :2 0 11f
205 207 :3 0 122
201 209 1fe 20b
20a :2 0 10 :3 0
1f :3 0 20d 20e
0 12 :4 0 54
:4 0 15 :2 0 16
:3 0 126 212 214
:3 0 129 20f 216
:2 0 230 10 :3 0
1f :3 0 218 219
0 12 :4 0 3f
:4 0 15 :2 0 3a
:3 0 12c 21d 21f
:3 0 15 :2 0 40
:4 0 12f 221 223
:3 0 15 :2 0 41
:3 0 42 :3 0 1c
:2 0 43 :2 0 132
226 22a 136 225
22c :3 0 139 21a
22e :2 0 230 13c
231 20c 230 0
232 13f 0 240
4f :3 0 c :3 0
50 :3 0 7 :3 0
4e :3 0 5 :3 0
51 :4 0 55 1
:a 0 23c :2 0 240
234 23a :2 0 6
:3 0 c :3 0 23e
:2 0 240 141 243
:3 0 243 0 243
242 240 241 :6 0
245 1 :3 0 146
247 148 246 245
:2 0 248 14a :2 0
24a 14d 24a 249
1a9 248 :6 0 24b
:2 0 2 10 24a
24e :3 0 24d 24b
24f :8 0 
150
4
:3 0 1 3 1
7 2 6 a
1 12 1 18
1 25 1 31
2 34 36 2
39 3b 2 3d
3f 2 41 43
2 45 47 2
49 4b 2 4d
4f 2 51 53
2 55 57 4
2d 33 38 59
1 5b 1 5d
2 67 69 3
65 66 6b 2
74 76 2 73
78 1 7a 1
7c 1 7e 1
84 2 94 96
3 92 93 98
2 a1 a3 2
a5 a7 2 a9
ab 1 b3 2
b5 b6 2 ad
b8 2 a0 ba
1 bc 1 be
2 c8 ca 3
c6 c7 cc 2
d5 d7 2 d4
d9 1 db 1
dd 2 e4 e5
3 bf de e7
1 e9 1 f3
2 f2 f3 1
fb 1 101 2
103 104 2 f0
105 1 10c 1
107 2 11f 121
3 11d 11e 123
2 12c 12e 2
130 132 2 134
136 2 138 13a
2 13c 13e 3
143 144 145 2
140 147 2 12b
149 1 14b 1
14d 2 116 14e
1 152 1 111
2 10f 156 2
163 165 3 161
162 167 2 170
172 2 174 176
2 178 17a 1
182 2 184 185
2 17c 187 2
189 18b 2 18d
18f 2 191 193
2 195 197 2
16f 199 1 19b
1 19d 3 1a4
1a5 1a6 6 5e
7d ea 159 19e
1a8 2 1b5 1b7
3 1b3 1b4 1b9
2 1c2 1c4 2
1c1 1c6 2 1cd
1d1 2 1d3 1d5
2 1d7 1db 2
1cc 1dd 2 1c8
1df 1 1e1 3
1e2 1ed 1f1 1
1ac 2 204 206
3 202 203 208
2 211 213 2
210 215 2 21c
21e 2 220 222
3 227 228 229
2 224 22b 2
21b 22d 2 217
22f 1 231 4
1fb 232 23b 23f
1 243 1 1f6
2 1f4 247 2
16 1d 
1
4
0 
24e
0
1
14
4
5
0 1 2 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 
2 0 1
7 1 0
12 1 0
3 1 0
18 1 0
0

/
SHOW ERRORS FUNCTION SP_U_TIMECODE2;
ALTER FUNCTION SP_U_TIMECODE2 COMPILE DEBUG;
/*
#Stored Procedure  :SP_S_PROFILE
#Short Description :
#Description       :
#Table Name        :
#Parameters        :
 $$bitflag       Description
   0	-  WBS Project global view: Resources  Profile with Competencies  Skills
   1	-  WBS Project global view: Resources  Profiles
   6	-  CRI Project global view: Resources  Profiles
   2,3	-  WBS  CRI - Assignments View
   4	-  WBS  CRI - All assignments with effort (no participants, no project assignments) in the current project
   5	-  WBS  CRI - All participants (no effort) in the current project
   8	-  WBS  CRI - All assignments (effort, participants, project assignments) in the current project
   9	-  WBS  CRI - All assignments for a Resource / Profile under a element (project, deliverable, task)
   7	-  WBS Project Assignments
   11	-  DOC Assignments View (SC_TABLE_DOC, SC_TABLE_POL_DOC, SC_TABLE_RES_DOC, SC_TABLE_AST_DOC,SC_TABLE_CLN_DOC,SC_TABLE_VDR_DOC)
   12	-  WBS  CRI - display nickname for All assignments (effort, participants, project assignments) in the current project
   13	-  Returns nickname for All assignments (effort, participants, project assignments) in the current pool
#Returns           :
#Errors            :
*/
Prompt == FUNCTION SP_S_PROFILE ==
CREATE OR REPLACE FUNCTION SP_S_PROFILE wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
1fe
2 :e:
1FUNCTION:
1SP_S_PROFILE:
1ID:
1VARCHAR2:
1BITFLAG001:
1PLS_INTEGER:
1RES_ID001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_RANK:
1200:
1H_ID:
132:
1H_REC_USER:
1H_PROJECT_ID:
1H_IN:
1255:
1H_BIT_FLAG:
11:
1H_BITFLAG:
10:
1H_PER_DAY:
1H_TYPE_ID:
1H_COUNT:
1H_TEMP_SC_STATE:
1H_LEVEL_STATE:
1C7751A:
1C7751B:
1C7751C:
1C7751T:
1C7751D:
1C7751E:
1C7751F:
1C7751G:
1C7751H:
1C7751J:
1C7751L:
1C7751M:
1C7751N:
1C77513N:
1CLN_AST_RES:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_S_PROFILE(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
175:
1+:
1FPUT:
1               ENTER:: SP_S_PROFILE(76) :: cplan.sql(12970) :: :
1IS NULL:
182:
1        RETURN_ERROR:: SP_S_PROFILE(83) :: cplan.sql(12977) :: :
1 :
1STATE:: :
1PMO_CNST:
1INVALID_PARAMS:
1 ID:: :
185:
1                EXIT:: SP_S_PROFILE(86) :: cplan.sql(12980) :: :
1PMODEFINES:
1PMO_RETURN_ERROR_SP:
1TRUE:
1=:
12:
13:
1B:
1PROJECT_ID:
1TYPE_ID:
1A:
1MINUTE_PER_DAY:
1LEVEL_STATE:
1TMT_WBS:
1ELEMENT_ID:
1SELECT B.PROJECT_ID, B.TYPE_ID, COALESCE(A.MINUTE_PER_DAY, 480), B.LEVEL_STAT+
1E:n         INTO h_project_id, h_type_id, h_per_day, h_level_state:n         +
1FROM TMT_WBS A, TMT_WBS B WHERE B.ELEMENT_ID = h_id AND A.ELEMENT_ID = B.PROJ+
1ECT_ID:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
1110:
1           EXCEPTION:: SP_S_PROFILE(111) :: cplan.sql(13005) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SC_SQL_NO_DATA:
1TMT_CRI:
1SELECT B.PROJECT_ID, B.TYPE_ID, COALESCE(A.MINUTE_PER_DAY, 480):n            +
1INTO h_project_id, h_type_id, h_per_day:n            FROM TMT_WBS A, TMT_CRI +
1B WHERE B.ELEMENT_ID = h_id AND A.ELEMENT_ID = B.PROJECT_ID:
1129:
1           EXCEPTION:: SP_S_PROFILE(130) :: cplan.sql(13024) :: :
1TMT_POOLS:
1SELECT 480 INTO h_per_day FROM TMT_POOLS WHERE ELEMENT_ID = h_id:
1146:
1           EXCEPTION:: SP_S_PROFILE(147) :: cplan.sql(13041) :: :
1153:
1        RETURN_ERROR:: SP_S_PROFILE(154) :: cplan.sql(13048) :: :
1156:
1                EXIT:: SP_S_PROFILE(157) :: cplan.sql(13051) :: :
1ELSIF:
110:
1TASK_ASSIGNMENTS:
1TSK_ASSIGNMENT_ID:
1SELECT PROJECT_ID INTO h_project_id:n         FROM TASK_ASSIGNMENTS WHERE TSK+
1_ASSIGNMENT_ID = h_id:
1174:
1           EXCEPTION:: SP_S_PROFILE(175) :: cplan.sql(13069) :: :
1181:
1        RETURN_ERROR:: SP_S_PROFILE(182) :: cplan.sql(13076) :: :
1184:
1                EXIT:: SP_S_PROFILE(185) :: cplan.sql(13079) :: :
1SELECT COALESCE(MINUTE_PER_DAY, 480), LEVEL_STATE INTO h_per_day, h_level_sta+
1te:n         FROM TMT_WBS WHERE ELEMENT_ID = h_id:
1           EXCEPTION:: SP_S_PROFILE(201) :: cplan.sql(13095) :: :
113:
1RANK:
1SELECT RANK INTO h_rank FROM TMT_POOLS WHERE ELEMENT_ID = h_id:
1215:
1           EXCEPTION:: SP_S_PROFILE(216) :: cplan.sql(13110) :: :
1222:
1        RETURN_ERROR:: SP_S_PROFILE(223) :: cplan.sql(13117) :: :
1225:
1                EXIT:: SP_S_PROFILE(226) :: cplan.sql(13120) :: :
1480:
1!=:
1SC_SRCH_PARTICIPANT:
111:
1SC_ASSET_ASSIGNED_RESOURCES:
1SC_CLIENT_ASSIGNED_RESOURCES:
1SELECT COALESCE(MINUTE_PER_DAY, 480), LEVEL_STATE INTO h_per_day, h_level_sta+
1te:n         FROM TMT_WBS WHERE ELEMENT_ID = id:
1245:
1           EXCEPTION:: SP_S_PROFILE(246) :: cplan.sql(13140) :: :
1252:
1        RETURN_ERROR:: SP_S_PROFILE(253) :: cplan.sql(13147) :: :
1                EXIT:: SP_S_PROFILE(256) :: cplan.sql(13150) :: :
1EXECUTE:
1IMMEDIATE:
1DELETE FROM PMO_TMP_ASGNDRES_INFO:
1PMO_TMP_ASGNDRES_INFO:
1RESOURCE_ID:
1PARENT_ID:
1RATE_ID:
1PROFICIENCY_LEVEL_ID:
1PREFERENCE_LEVEL_ID:
1EXPERIENCE_ID:
1LAST_USED_DATETIME:
1R:
1RESOURCE_ASSIGNMENTS:
1JOIN:
1TMT_COMPETENCIES:
1RESOURCE_TYPE:
1INSERT INTO PMO_TMP_ASGNDRES_INFO:n         (:n         RESOURCE_ID,:n       +
1  PARENT_ID,:n         RATE_ID,:n         PROFICIENCY_LEVEL_ID,:n         PRE+
1FERENCE_LEVEL_ID,:n         EXPERIENCE_ID,:n         LAST_USED_DATETIME:n    +
1     ):n         SELECT DISTINCT:n         a.RESOURCE_ID,:n         r.PARENT_+
1ID,:n         r.RATE_ID,:n         r.PROFICIENCY_LEVEL_ID,:n         r.PREFER+
1ENCE_LEVEL_ID,:n         r.EXPERIENCE_ID,:n         r.LAST_USED_DATETIME:n   +
1      FROM RESOURCE_ASSIGNMENTS a JOIN TMT_COMPETENCIES r ON a.RESOURCE_ID=r.+
1RESOURCE_ID:n         WHERE PROJECT_ID = h_id AND RESOURCE_TYPE = '0':
1FOUND:
1100:
1301:
1           EXCEPTION:: SP_S_PROFILE(302) :: cplan.sql(13196) :: :
1DELETE FROM PMO_TMP_PROF_INFO:
1PMO_TMP_PROF_INFO:
1PROFILE_ID:
1INSERT INTO PMO_TMP_PROF_INFO (PROFILE_ID):n         SELECT DISTINCT RESOURCE+
1_ID:n         FROM RESOURCE_ASSIGNMENTS:n         WHERE PROJECT_ID = h_id AND+
1 RESOURCE_TYPE = '1':
1327:
1           EXCEPTION:: SP_S_PROFILE(328) :: cplan.sql(13222) :: :
1OPEN:
1TA:
1RES:
1RES_TYPE:
1P:
1FULL_NAME:
1ELEMENT_NAME:
1STATUS1_ID:
1E_MAIL_ADDRESS:
1D:
1ROLE_NAME:
1LAST_USED:
1RC:
1REGULAR_COST:
1REGULAR_SALE:
1SPECIAL_COST:
1SPECIAL_SALE:
1OBJECTIVES:
1RESOURCES:
1C:
1LEFT:
1OUTER:
1RATES:
1PRF:
1PROFILE_NAME:
1PROFILES:
1ROLE_ID:
1RESOURCE_ASSIGN_SKILLS:
1S:
1RES_ASSIGNMENT_ID:
1SKILL_ID:
1RESOURCE_ASSIGN_ROLES:
1SELECT CASE WHEN EXISTS(SELECT * FROM TASK_ASSIGNMENTS ta WHERE ta.PROJECT_ID+
1=h_id AND ta.RESOURCE_ID = RES.RESOURCE_ID) THEN 13 ELSE 185 END AS RES_TYPE,+
1:n      RES.RESOURCE_ID, p.FULL_NAME AS ELEMENT_NAME, p.STATUS1_ID, p.E_MAIL_+
1ADDRESS, d.TYPE_ID, d.ELEMENT_NAME AS ROLE_NAME,:n      RES.PROFICIENCY_LEVEL+
1_ID, RES.PREFERENCE_LEVEL_ID, RES.EXPERIENCE_ID, RES.LAST_USED_DATETIME AS LA+
1ST_USED,rc.REGULAR_COST,rc.REGULAR_SALE,rc.SPECIAL_COST,rc.SPECIAL_SALE, '' A+
1S OBJECTIVES,'' AS RANK:n      FROM PMO_TMP_ASGNDRES_INFO RES JOIN RESOURCES +
1p ON RES.RESOURCE_ID=p.RESOURCE_ID:n      JOIN TMT_COMPETENCIES c ON c.ELEMEN+
1T_ID=RES.PARENT_ID:n      JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID+
1:n      LEFT OUTER JOIN RATES rc ON rc.RATE_ID=RES.RATE_ID:n      :n      UNI+
1ON ALL:n      :n      SELECT 41 AS RES_TYPE,PRF.PROFILE_ID AS RESOURCE_ID, p.+
1PROFILE_NAME AS ELEMENT_NAME, 0 AS STATUS1_ID, '' AS E_MAIL_ADDRESS,:n      c+
1.TYPE_ID, c.ELEMENT_NAME AS ROLE_NAME,:n      r.PROFICIENCY_LEVEL_ID, r.PREFE+
1RENCE_LEVEL_ID,r.EXPERIENCE_ID,r.LAST_USED_DATETIME AS LAST_USED,:n      rc.R+
1EGULAR_COST,rc.REGULAR_SALE,rc.SPECIAL_COST,rc.SPECIAL_SALE,'' AS OBJECTIVES,+
1'' AS RANK:n      FROM PMO_TMP_PROF_INFO PRF  JOIN PROFILES p ON PRF.PROFILE_+
1ID=p.PROFILE_ID:n      LEFT OUTER JOIN TMT_COMPETENCIES r ON p.ROLE_ID=r.ELEM+
1ENT_ID:n      LEFT OUTER JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.PARENT_ID+
1:n      LEFT OUTER JOIN RATES rc ON rc.RATE_ID=r.RATE_ID:n      :n      UNION+
1 ALL:n      :n      SELECT 41 AS RES_TYPE,PRF.PROFILE_ID AS RESOURCE_ID,p.PRO+
1FILE_NAME AS ELEMENT_NAME,0 AS STATUS1_ID,'' AS E_MAIL_ADDRESS,:n      c.TYPE+
1_ID, c.ELEMENT_NAME AS ROLE_NAME,:n      r.PROFICIENCY_LEVEL_ID, r.PREFERENCE+
1_LEVEL_ID, r.EXPERIENCE_ID, r.LAST_USED_DATETIME AS LAST_USED,:n      rc.REGU+
1LAR_COST,rc.REGULAR_SALE,rc.SPECIAL_COST,rc.SPECIAL_SALE, '' AS OBJECTIVES,''+
1 AS RANK:n      FROM PMO_TMP_PROF_INFO PRF JOIN PROFILES p ON PRF.PROFILE_ID=+
1p.PROFILE_ID:n      JOIN RESOURCE_ASSIGN_SKILLS s ON p.PROFILE_ID=s.RES_ASSIG+
1NMENT_ID:n      LEFT OUTER JOIN TMT_COMPETENCIES r ON s.SKILL_ID=r.ELEMENT_ID+
1:n      LEFT OUTER JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.PARENT_ID:n     +
1 LEFT OUTER JOIN RATES rc ON rc.RATE_ID=r.RATE_ID:n      :n      UNION ALL:n +
1     :n      SELECT 41 AS RES_TYPE,PRF.PROFILE_ID AS RESOURCE_ID,p.PROFILE_NA+
1ME AS ELEMENT_NAME,0 AS STATUS1_ID,'' AS E_MAIL_ADDRESS,:n      c.TYPE_ID, c.+
1ELEMENT_NAME AS ROLE_NAME,:n      r.PROFICIENCY_LEVEL_ID, r.PREFERENCE_LEVEL_+
1ID, r.EXPERIENCE_ID, r.LAST_USED_DATETIME AS LAST_USED,:n      rc.REGULAR_COS+
1T,rc.REGULAR_SALE,rc.SPECIAL_COST,rc.SPECIAL_SALE, '' AS OBJECTIVES,'' AS RAN+
1K:n      FROM PMO_TMP_PROF_INFO PRF JOIN PROFILES p ON PRF.PROFILE_ID=p.PROFI+
1LE_ID:n      JOIN RESOURCE_ASSIGN_ROLES s ON p.PROFILE_ID=s.RES_ASSIGNMENT_ID+
1:n      LEFT OUTER JOIN TMT_COMPETENCIES r ON s.ROLE_ID=r.ELEMENT_ID:n      L+
1EFT OUTER JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.PARENT_ID:n      LEFT OUT+
1ER JOIN RATES rc ON rc.RATE_ID=r.RATE_ID:n      ORDER BY RES_TYPE, RESOURCE_I+
1D, TYPE_ID:
1376:
1                EXIT:: SP_S_PROFILE(377) :: cplan.sql(13271) :: :
16:
1DELETE FROM PMO_TMP_RES_DATA:
1PMO_TMP_RES_DATA:
1RA:
1PROJECT_ASSIGNMENT_ID:
1T:
1TASK_ID:
1INSERT INTO PMO_TMP_RES_DATA (RESOURCE_ID, TYPE_ID):n         SELECT DISTINCT+
1:n         ra.RESOURCE_ID,:n         CASE WHEN ta.TSK_ASSIGNMENT_ID IS NuLL T+
1HEN 185 ELSE 13 END AS TYPE_ID:n         FROM RESOURCE_ASSIGNMENTS ra LEFT OU+
1TER JOIN TASK_ASSIGNMENTS ta ON ta.PROJECT_ID=h_id AND ra.RESOURCE_ID = ta.RE+
1SOURCE_ID:n         WHERE ra.PROJECT_ID=h_id AND ra.RESOURCE_TYPE='0' AND ta.+
1PROJECT_ASSIGNMENT_ID IS NuLL AND:n         NOT EXISTS(SELECT * FROM TASK_ASS+
1IGNMENTS t WHERE ra.RESOURCE_ID = t.RESOURCE_ID AND t.TASK_ID=h_id):
1404:
1           EXCEPTION:: SP_S_PROFILE(405) :: cplan.sql(13299) :: :
1FC_U_CREATETMPTABLE:
1PMO_TMP_CPLN01_TEMP:
1ELEMENT_TYPE:
1INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME):n         SELECT DIS+
1TINCT r.PROFILE_ID ,r.PROFILE_NAME:n         FROM RESOURCE_ASSIGNMENTS a JOIN+
1 PROFILES r ON a.RESOURCE_TYPE='1' AND r.PROFILE_ID=a.RESOURCE_ID :n         +
1JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.ROLE_ID:n         JOIN TMT_COMPETEN+
1CIES d ON d.ELEMENT_ID=c.PARENT_ID :n         JOIN TASK_ASSIGNMENTS t ON t.RE+
1S_ASSIGNMENT_ID=a.RES_ASSIGNMENT_ID AND t.PROJECT_ASSIGNMENT_ID IS NuLL:n    +
1     WHERE a.PROJECT_ID=h_id AND a.ELEMENT_TYPE>1:
1431:
1           EXCEPTION:: SP_S_PROFILE(432) :: cplan.sql(13326) :: :
1TO_TIMESTAMP:
1PLANNED_START:
1PLANNED_FINISH:
1PLANNED_EFFORT:
1START_DATE:
1FINISH_DATE:
1WORK:
1REMAINING:
1T_UNIT:
1BEST_START:
1BEST_FINISH:
1CALENDAR:
1PLANNED_WORK:
1PERIOD_AVAILABILITY:
1ASSIGN_STRING:
1SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS EL+
1EMENT_NAME,a.TYPE_ID,p.E_MAIL_ADDRESS,:n      TO_TIMESTAMP('1899-12-31 00::00+
1::00') AS PLANNED_START, TO_TIMESTAMP('1899-12-31 00::00::00') AS PLANNED_FIN+
1ISH,0 AS PLANNED_EFFORT,:n      TO_TIMESTAMP('1899-12-31 00::00::00') AS STAR+
1T_DATE, TO_TIMESTAMP('1899-12-31 00::00::00') AS FINISH_DATE, 0 AS WORK, 0 AS+
1 REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,'' AS BEST_START,'' AS BES+
1T_FINISH,:n      0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, +
1'' AS ASSIGN_STRING, '' AS OBJECTIVES,'' AS RANK:n      FROM PMO_TMP_RES_DATA+
1 a JOIN RESOURCES p ON a.RESOURCE_ID=p.RESOURCE_ID:n      :n      UNION ALL:n+
1      :n      SELECT ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, ELEMENT_NAME,41 AS T+
1YPE_ID,'' AS E_MAIL_ADDRESS,:n      TO_TIMESTAMP('1899-12-31 00::00::00') AS +
1PLANNED_START, TO_TIMESTAMP('1899-12-31 00::00::00') AS PLANNED_FINISH,0 AS P+
1LANNED_EFFORT,:n      TO_TIMESTAMP('1899-12-31 00::00::00') AS START_DATE, TO+
1_TIMESTAMP('1899-12-31 00::00::00') AS FINISH_DATE, 0 AS WORK, 0 AS REMAINING+
1,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,:n      '' AS BEST_START,''  AS BEST+
1_FINISH,0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASS+
1IGN_STRING, '' AS OBJECTIVES,'' AS RANK:n      FROM PMO_TMP_CPLN01_TEMP:n    +
1  ORDER BY TYPE_ID:
1451:
1                EXIT:: SP_S_PROFILE(452) :: cplan.sql(13346) :: :
1PARTICIPANT:
1START_DT:
1FINISH_DT:
1WORK_LOCATION:
1SECURITY_ID:
1G:
1SECURITY_NAME:
1CRI_SECURITY_ID:
1G2:
1CRI_SECURITY_NAME:
1REGULAR_RATE:
1SPECIAL_RATE:
1FS_LABOR_CODE:
1REC_STATUS:
1TIMECODE2_ID:
1TC:
1TIMECODE2:
1C_EFFORT:
1C_UNIT:
1C_CONTOUR:
1CONTOUR_TYPE:
1C_OCCURRENCE:
1BILLABLE:
1ASSIGNMENT_DELAY:
1TMT_GLOBAL_SECURITY:
1TMT_TIMECODES:
1SELECT CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN t.TSK_ASSIGNMENT_ID ELS+
1E a.RES_ASSIGNMENT_ID END AS ELEMENT_ID,:n      a.PROFILE_ID, p.PROFILE_NAME +
1AS ELEMENT_NAME,41 AS TYPE_ID,a.RES_ASSIGNMENT_ID, a.PARTICIPANT, t.PLANNED_S+
1TART,t.PLANNED_FINISH,:n      t.PLANNED_WORK,t.START_DT,t.FINISH_DT,h_per_day+
1 AS MINUTE_PER_DAY, a.WORK_LOCATION, a.SECURITY_ID, g.ELEMENT_NAME AS SECURIT+
1Y_NAME,:n      COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_SECURITY_ID, +
1g2.ELEMENT_NAME AS CRI_SECURITY_NAME, t.REGULAR_COST,t.REGULAR_RATE,t.SPECIAL+
1_COST,t.SPECIAL_RATE,'' AS FS_LABOR_CODE,a.REC_STATUS,t.ASSIGN_STRING,:n     +
1 t.PROJECT_ASSIGNMENT_ID,t.TIMECODE2_ID,TC.ELEMENT_NAME AS TIMECODE2, t.T_UNI+
1T, t.C_EFFORT, t.C_UNIT, t.C_CONTOUR, t.CONTOUR_TYPE, t.C_OCCURRENCE, BILLABL+
1E,:n      t.ASSIGNMENT_DELAY:n      FROM RESOURCE_ASSIGNMENTS a LEFT OUTER JO+
1IN TASK_ASSIGNMENTS t ON t.TASK_ID = h_id AND a.RES_ASSIGNMENT_ID = t.RES_ASS+
1IGNMENT_ID:n      LEFT OUTER JOIN PROFILES p ON a.RESOURCE_ID = p.PROFILE_ID +
1LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_ID:n      LE+
1FT OUTER JOIN TMT_GLOBAL_SECURITY g2 ON COALESCE(a.CRI_SECURITY_ID,a.SECURITY+
1_ID)=g2.ELEMENT_ID:n      LEFT OUTER JOIN TMT_TIMECODES TC ON TC.ELEMENT_ID= +
1t.TIMECODE2_ID:n      WHERE a.ELEMENT_ID = h_id AND a.RESOURCE_TYPE='1' ORDER+
1 BY ELEMENT_NAME:
1467:
1                EXIT:: SP_S_PROFILE(468) :: cplan.sql(13362) :: :
1ISTASK:
1COUNT:
1SELECT COUNT(*) INTO h_count FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_proje+
1ct_id AND TASK_ID = h_id:
1483:
1           EXCEPTION:: SP_S_PROFILE(484) :: cplan.sql(13378) :: :
1DUAL:
1BIT_FLAG:
1SELECT COUNT('X') INTO h_count FROM DUAL WHERE EXISTS (:n               SELEC+
1T 'X' FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND TASK_ID = h_i+
1d AND BIT_FLAG = '0'):
1503:
1           EXCEPTION:: SP_S_PROFILE(504) :: cplan.sql(13398) :: :
1TIMESHEET_APPROVAL_STATUS:
1APV:
1TO_DATE:
1ACTUAL_START:
1WEEK_OF:
1SELECT COUNT('X') INTO h_count FROM DUAL WHERE EXISTS (:n                  SE+
1LECT 'X' FROM TASK_ASSIGNMENTS ta JOIN TIMESHEET_APPROVAL_STATUS apv:n       +
1           ON apv.ELEMENT_ID = ta.PROJECT_ID AND apv.RESOURCE_ID = ta.RESOURC+
1E_ID:n                  WHERE ta.PROJECT_ID = h_project_id AND ta.TASK_ID = h+
1_id AND:n                  (TO_DATE(SubStr(ta.ACTUAL_START, 1, 10)) - 7     )+
1 <= apv.WEEK_OF):
1523:
1           EXCEPTION:: SP_S_PROFILE(524) :: cplan.sql(13418) :: :
1SD:
1FD:
1SCHED_EFFORT:
1WORK_TODATE:
1UNITS_PERCENT:
1MILESTONE_PERCENT:
1TRUNC:
1TO_NUMBER:
1WORK_PERCENT:
1BAK_START_DT:
1BAK_FINISH_DT:
1BAK_WORK:
1FORECAST_START:
1FORECAST_FINISH:
1ETC:
1ACTUAL_FINISH:
1ACTUAL:
1PROPOSED_START:
1PROPOSED_FINISH:
1PROPOSED_EFFORT:
1PROPOSED_WORK:
1CLOB_DATA:
1EMPTYLOB:
1BASELINE1_START:
1BASELINE1_FINISH:
1BASELINE1_EFFORT:
1BASELINE1_WORK:
1TIMECODE2_NAME:
1CAPITAL_EXPENSE:
1RATE_ADJUSTMENT:
1ASSIGN_COUNT:
1DURATION:
1LEVEL_START:
1LEVEL_FINISH:
1LEVEL_DURATION:
1LEVEL_EFFORT:
1PERCENT_DISCRETE:
1TOP_ID:
1LEVEL_AFFECTED_FLAG:
1LABOR_COST_CC_ID:
1LABOR_COST_CC_NAME:
1LABOR_REVENUE_CC_ID:
1LABOR_REVENUE_CC_NAME:
1DEFAULT_LABOR_FLAG:
1CONTRACT_ID:
1CONTRACT_NAME:
1SCHED_FINISH:
1SC_UFS:
1SC_FS:
1SC_SS:
1SC_FF:
1SC_SF:
1SCHED_START:
1SCHED_WORK:
1ASSIGN_UNITS:
1UNITS_MSP:
1SCHED_DURATION:
1PLANNED_DURATION:
1UNRESOLVED_START:
1UNRESOLVED_FINISH:
1EC:
1CC:
1CTR:
1TMT_CHARGECODES:
1TMT_CONTRACTS:
1SELECT t.ELEMENT_ID, a.RESOURCE_ID, t.ELEMENT_NAME, t.TYPE_ID, '' AS RES_ASSI+
1GNMENT_ID, a.PARTICIPANT, t.START_DT,t.FINISH_DT,:n         t.SD, t.FD, t.STA+
1RT_DT AS PLANNED_START, t.FINISH_DT AS PLANNED_FINISH,:n         t.SCHED_EFFO+
1RT + t.PLANNED_EFFORT AS PLANNED_WORK, t.SCHED_EFFORT + t.PLANNED_EFFORT - t.+
1WORK_TODATE AS REMAINING,:n         0 UNITS_PERCENT, t.MILESTONE_PERCENT,:n  +
1       CASE WHEN t.WORK_TODATE = 0 THEN 0 ELSE Trunc((TO_NUMBER(t.WORK_TODATE+
1)/TO_NUMBER(t.SCHED_EFFORT + t.PLANNED_EFFORT))*10000) END AS WORK_PERCENT,:n+
1         TO_TIMESTAMP('1899-12-31 00::00::00') AS BAK_START_DT,TO_TIMESTAMP('+
11899-12-31 00::00::00') AS BAK_FINISH_DT,0 AS BAK_WORK,:n         t.FORECAST_+
1START,t.FORECAST_FINISH,t.ETC, t.ACTUAL_START,t.ACTUAL_FINISH,:n         t.WO+
1RK_TODATE AS ACTUAL,t.PROPOSED_START, t.PROPOSED_FINISH, t.PROPOSED_EFFORT AS+
1 PROPOSED_WORK,:n         h_per_day AS MINUTE_PER_DAY, 0 AS WORK_LOCATION, ''+
1 AS SECURITY_ID, '' AS SECURITY_NAME,:n         '' AS CRI_SECURITY_ID, '' AS +
1CRI_SECURITY_NAME, 0.00 AS REGULAR_COST,0.00 AS REGULAR_RATE,0.00 AS SPECIAL_+
1COST,0.00 AS SPECIAL_RATE,:n         '' AS FS_LABOR_CODE,t.REC_STATUS,(SELECT+
1 CLOB_DATA FROM EMPTYLOB) AS ASSIGN_STRING,t.BASELINE1_START,t.BASELINE1_FINI+
1SH,t.BASELINE1_EFFORT AS BASELINE1_WORK,:n         '' AS PROJECT_ASSIGNMENT_I+
1D,'' AS TIMECODE2_ID, '' AS TIMECODE2_NAME, 0 AS T_UNIT, 0 AS C_EFFORT, 0 AS +
1C_UNIT, 0 AS C_CONTOUR, 0 AS CONTOUR_TYPE, 0 AS C_OCCURRENCE, 0 AS BILLABLE, +
10 AS CAPITAL_EXPENSE,:n         0 AS ASSIGNMENT_DELAY, h_bit_flag AS BIT_FLAG+
1, 0 AS RATE_ADJUSTMENT, t.ASSIGN_COUNT, t.DURATION,:n         CASE WHEN h_lev+
1el_state = 1 THEN t.LEVEL_START ELSE TO_TIMESTAMP('1899-12-31 00::00::00') EN+
1D AS LEVEL_START,:n         CASE WHEN h_level_state = 1 THEN t.LEVEL_FINISH E+
1LSE TO_TIMESTAMP('1899-12-31 00::00::00') END AS LEVEL_FINISH,:n         CASE+
1 WHEN h_level_state = 1 THEN t.LEVEL_DURATION ELSE 0 END AS LEVEL_EFFORT, t.W+
1ORK_PERCENT AS PERCENT_DISCRETE, 0 as TOP_ID, '0' AS LEVEL_AFFECTED_FLAG,:n  +
1       '' AS LABOR_COST_CC_ID, '' AS LABOR_COST_CC_NAME, '' AS LABOR_REVENUE_+
1CC_ID, '' AS LABOR_REVENUE_CC_NAME, 'N' AS DEFAULT_LABOR_FLAG,:n         '' A+
1S CONTRACT_ID, '' AS CONTRACT_NAME:n         FROM TMT_WBS t LEFT JOIN RESOURC+
1E_ASSIGNMENTS a ON a.ELEMENT_ID = t.ELEMENT_ID AND a.RESOURCE_ID = h_rec_user+
1:n         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_I+
1D:n         WHERE t.PROJECT_ID = h_project_id AND t.ELEMENT_ID = h_id:n      +
1   UNION ALL:n         SELECT CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN +
1t.TSK_ASSIGNMENT_ID ELSE a.RES_ASSIGNMENT_ID END AS ELEMENT_ID, a.RESOURCE_ID+
1,:n         CASE WHEN a.RESOURCE_TYPE='1' THEN (SELECT p.PROFILE_NAME FROM PR+
1OFILES p WHERE a.RESOURCE_ID = p.PROFILE_ID):n         ELSE (SELECT r.FULL_NA+
1ME FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID) END AS ELEMENT_NAME,+
1:n         CASE WHEN t.TSK_ASSIGNMENT_ID IS NuLL THEN 185 WHEN a.RESOURCE_TYP+
1E='1' THEN 41 ELSE 13 END AS TYPE_ID,:n         a.RES_ASSIGNMENT_ID, a.PARTIC+
1IPANT, t.START_DT, COALESCE(t.PLANNED_FINISH,t.SCHED_FINISH) AS FINISH_DT,:n +
1        CASE WHEN t.SD = PMO_CNST.SC_UFS OR t.SD = PMO_CNST.SC_FS THEN PMO_CN+
1ST.SC_SS ELSE t.SD END AS SD, CASE WHEN t.FD = PMO_CNST.SC_FF THEN PMO_CNST.S+
1C_SF ELSE t.FD END AS FD, :n         COALESCE(PLANNED_START,SCHED_START) AS P+
1LANNED_START, COALESCE(t.PLANNED_FINISH,t.SCHED_FINISH) AS PLANNED_FINISH,:n +
1        CASE WHEN t.BIT_FLAG = '2' THEN t.SCHED_WORK ELSE t.PLANNED_WORK END +
1AS PLANNED_WORK,:n         CASE WHEN t.BIT_FLAG = '2' THEN t.SCHED_WORK - t.W+
1ORK_TODATE ELSE t.PLANNED_WORK END AS REMAINING,:n         CASE WHEN t.ASSIGN+
1_UNITS > t.UNITS_MSP THEN t.ASSIGN_UNITS ELSE t.UNITS_MSP END AS UNITS_PERCEN+
1T,:n         0 AS MILESTONE_PERCENT,:n         CASE WHEN t.WORK_TODATE = 0 TH+
1EN 0 ELSE Trunc((TO_NUMBER(t.WORK_TODATE)/TO_NUMBER(PLANNED_WORK+SCHED_WORK))+
1*10000) END AS WORK_PERCENT,:n         t.BAK_START_DT,t.BAK_FINISH_DT,t.BAK_W+
1ORK,:n         t.FORECAST_START,t.FORECAST_FINISH,t.ETC, t.ACTUAL_START,t.ACT+
1UAL_FINISH,:n         t.WORK_TODATE AS ACTUAL,PROPOSED_START, PROPOSED_FINISH+
1, PROPOSED_WORK,:n         h_per_day AS MINUTE_PER_DAY, a.WORK_LOCATION, a.SE+
1CURITY_ID, g.ELEMENT_NAME AS SECURITY_NAME,:n         COALESCE(a.CRI_SECURITY+
1_ID,a.SECURITY_ID) AS CRI_SECURITY_ID, g2.ELEMENT_NAME AS CRI_SECURITY_NAME, +
1t.REGULAR_COST,t.REGULAR_RATE,t.SPECIAL_COST,t.SPECIAL_RATE,:n         CASE W+
1HEN a.RESOURCE_TYPE='0' THEN (SELECT r.FS_LABOR_CODE FROM RESOURCES r WHERE a+
1.RESOURCE_ID = r.RESOURCE_ID):n         ELSE '' END AS FS_LABOR_CODE,a.REC_ST+
1ATUS,t.ASSIGN_STRING,t.BASELINE1_START,t.BASELINE1_FINISH,t.BASELINE1_WORK,:n+
1         t.PROJECT_ASSIGNMENT_ID,t.TIMECODE2_ID, TC.ELEMENT_NAME AS TIMECODE2+
1_NAME, t.T_UNIT, t.C_EFFORT, t.C_UNIT, t.C_CONTOUR, t.CONTOUR_TYPE, t.C_OCCUR+
1RENCE, t.BILLABLE, t.CAPITAL_EXPENSE,:n         t.ASSIGNMENT_DELAY, CASE WHEN+
1 t.BIT_FLAG IS NuLL THEN '' WHEN t.BIT_FLAG = '0' THEN '1' ELSE '2' END AS BI+
1T_FLAG, t.RATE_ADJUSTMENT, 0 AS ASSIGN_COUNT, :n         CASE WHEN t.BIT_FLAG+
1 = '2' THEN t.SCHED_DURATION ELSE t.PLANNED_DURATION END AS DURATION,:n      +
1   CASE WHEN h_level_state = 1 THEN COALESCE(t.LEVEL_START,t.UNRESOLVED_START+
1) ELSE TO_TIMESTAMP('1899-12-31 00::00::00') END AS LEVEL_START,:n         CA+
1SE WHEN h_level_state = 1 THEN COALESCE(t.LEVEL_FINISH,t.UNRESOLVED_FINISH) E+
1LSE TO_TIMESTAMP('1899-12-31 00::00::00') END AS LEVEL_FINISH,:n         CASE+
1 WHEN h_level_state = 1 THEN CASE WHEN t.LEVEL_EFFORT = 0 THEN t.ETC ELSE t.L+
1EVEL_EFFORT END ELSE 0 END AS LEVEL_EFFORT, t.MILESTONE_PERCENT AS PERCENT_DI+
1SCRETE, 1 as TOP_ID, '0' AS LEVEL_AFFECTED_FLAG,:n         T.LABOR_COST_CC_ID+
1,:n         EC.ELEMENT_NAME AS LABOR_COST_CC_NAME,:n         T.LABOR_REVENUE_+
1CC_ID,:n         CC.ELEMENT_NAME AS LABOR_REVENUE_CC_NAME, T.DEFAULT_LABOR_FL+
1AG,:n         T.CONTRACT_ID, CTR.ELEMENT_NAME AS CONTRACT_NAME:n         FROM+
1 RESOURCE_ASSIGNMENTS a LEFT OUTER JOIN TASK_ASSIGNMENTS t ON a.RES_ASSIGNMEN+
1T_ID = t.RES_ASSIGNMENT_ID:n         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON+
1 a.SECURITY_ID=g.ELEMENT_ID:n         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g2 +
1ON COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID:n         LEFT OUT+
1ER JOIN TMT_TIMECODES TC ON TC.ELEMENT_ID= t.TIMECODE2_ID:n         LEFT OUTE+
1R JOIN TMT_CHARGECODES EC ON EC.ELEMENT_ID = T.LABOR_COST_CC_ID:n         LEF+
1T OUTER JOIN TMT_CHARGECODES CC ON CC.ELEMENT_ID = T.LABOR_REVENUE_CC_ID:n   +
1      LEFT OUTER JOIN TMT_CONTRACTS CTR ON CTR.ELEMENT_ID = T.CONTRACT_ID:n  +
1       WHERE a.PROJECT_ID = h_project_id AND a.ELEMENT_ID = h_id AND a.REC_ST+
1ATUS NOT IN ('L','D'):n         ORDER BY TOP_ID, ELEMENT_NAME:
1604:
1                EXIT:: SP_S_PROFILE(605) :: cplan.sql(13499) :: :
1TMT_ORGANIZATIONS:
1SC_PROFILE:
1SC_POOLS:
1SC_ORGANIZATION:
1SC_RESOURCE:
1BEST:
1SELECT CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN t.TSK_ASSIGNMENT_ID ELS+
1E a.RES_ASSIGNMENT_ID END AS ELEMENT_ID,a.RESOURCE_ID,:n         CASE WHEN a.+
1RESOURCE_TYPE='0' THEN (SELECT r.FULL_NAME FROM RESOURCES r WHERE a.RESOURCE_+
1ID = r.RESOURCE_ID):n         WHEN a.RESOURCE_TYPE='1' THEN (SELECT p.PROFILE+
1_NAME FROM PROFILES p WHERE a.RESOURCE_ID = p.PROFILE_ID):n         WHEN a.RE+
1SOURCE_TYPE='2' THEN (SELECT p.ELEMENT_NAME FROM TMT_POOLS p WHERE a.RESOURCE+
1_ID = p.ELEMENT_ID):n         ELSE (SELECT p.ELEMENT_NAME FROM TMT_ORGANIZATI+
1ONS p WHERE a.RESOURCE_ID = p.ELEMENT_ID) END AS ELEMENT_NAME,:n         CASE+
1 WHEN a.RESOURCE_TYPE='1' THEN PMO_CNST.SC_PROFILE:n         WHEN a.RESOURCE_+
1TYPE='2' THEN PMO_CNST.SC_POOLS :n         WHEN a.RESOURCE_TYPE='3' THEN PMO_+
1CNST.SC_ORGANIZATION :n         WHEN t.TSK_ASSIGNMENT_ID IS NuLL THEN 185 ELS+
1E PMO_CNST.SC_RESOURCE END AS TYPE_ID,:n         a.RES_ASSIGNMENT_ID, a.PARTI+
1CIPANT, t.START_DT,t.FINISH_DT,:n         t.SD, t.FD, COALESCE(t.PLANNED_STAR+
1T,t.SCHED_START) AS PLANNED_START, COALESCE(t.PLANNED_FINISH,t.SCHED_FINISH) +
1AS PLANNED_FINISH,:n         t.BEST AS PLANNED_WORK, CASE WHEN t.BIT_FLAG = '+
12' THEN t.SCHED_WORK ELSE t.PLANNED_WORK END AS REMAINING,:n         t.UNITS_+
1MSP AS UNITS_PERCENT, t.MILESTONE_PERCENT, 0 AS WORK_PERCENT,:n         t.BAK+
1_START_DT,t.BAK_FINISH_DT,t.BAK_WORK,:n         t.FORECAST_START,t.FORECAST_F+
1INISH,t.ETC, t.ACTUAL_START,t.ACTUAL_FINISH,:n         t.WORK_TODATE AS ACTUA+
1L,PROPOSED_START, PROPOSED_FINISH, PROPOSED_WORK,:n         h_per_day AS MINU+
1TE_PER_DAY, a.WORK_LOCATION, a.SECURITY_ID, g.ELEMENT_NAME AS SECURITY_NAME,+
1:n         COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_SECURITY_ID, g2.E+
1LEMENT_NAME AS CRI_SECURITY_NAME, t.REGULAR_COST,t.REGULAR_RATE,t.SPECIAL_COS+
1T,t.SPECIAL_RATE,:n         CASE WHEN a.RESOURCE_TYPE='0' THEN (SELECT r.FS_L+
1ABOR_CODE FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID):n         ELS+
1E '' END AS FS_LABOR_CODE,a.REC_STATUS,t.ASSIGN_STRING,t.BASELINE1_START,t.BA+
1SELINE1_FINISH,t.BASELINE1_WORK,:n         t.PROJECT_ASSIGNMENT_ID,t.TIMECODE+
12_ID, TC.ELEMENT_NAME AS TIMECODE2_NAME, t.T_UNIT, t.C_EFFORT, t.C_UNIT, t.C_+
1CONTOUR, t.CONTOUR_TYPE, t.C_OCCURRENCE, t.BILLABLE, t.CAPITAL_EXPENSE,:n    +
1     t.ASSIGNMENT_DELAY, CASE WHEN t.BIT_FLAG IS NuLL THEN '' WHEN t.BIT_FLAG+
1 = '0' THEN '1' ELSE '2' END AS BIT_FLAG, t.RATE_ADJUSTMENT, 0 AS ASSIGN_COUN+
1T, 0 AS DURATION,:n         t.LEVEL_START, t.LEVEL_FINISH, t.LEVEL_EFFORT, t.+
1MILESTONE_PERCENT AS PERCENT_DISCRETE, 0 as TOP_ID,:n         CASE WHEN t.TSK+
1_ASSIGNMENT_ID IS NOT NuLL AND t.PROJECT_ASSIGNMENT_ID = t.TSK_ASSIGNMENT_ID +
1AND (t.LEVEL_START <> t.UNRESOLVED_START OR t.LEVEL_FINISH <> t.UNRESOLVED_FI+
1NISH) THEN '1' ELSE '0' END AS LEVEL_AFFECTED_FLAG,:n         T.LABOR_COST_CC+
1_ID,:n         EC.ELEMENT_NAME AS LABOR_COST_CC_NAME,:n         T.LABOR_REVEN+
1UE_CC_ID,:n         CC.ELEMENT_NAME AS LABOR_REVENUE_CC_NAME, T.DEFAULT_LABOR+
1_FLAG,:n         T.CONTRACT_ID, CTR.ELEMENT_NAME AS CONTRACT_NAME:n         F+
1ROM RESOURCE_ASSIGNMENTS a LEFT OUTER JOIN TASK_ASSIGNMENTS t ON a.RES_ASSIGN+
1MENT_ID = t.RES_ASSIGNMENT_ID:n         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g+
1 ON a.SECURITY_ID=g.ELEMENT_ID:n         LEFT OUTER JOIN TMT_GLOBAL_SECURITY +
1g2 ON COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID:n         LEFT +
1OUTER JOIN TMT_TIMECODES TC ON TC.ELEMENT_ID= t.TIMECODE2_ID:n         LEFT O+
1UTER JOIN TMT_CHARGECODES EC ON EC.ELEMENT_ID = T.LABOR_COST_CC_ID:n         +
1LEFT OUTER JOIN TMT_CHARGECODES CC ON CC.ELEMENT_ID = T.LABOR_REVENUE_CC_ID:n+
1         LEFT OUTER JOIN TMT_CONTRACTS CTR ON CTR.ELEMENT_ID = T.CONTRACT_ID+
1:n         WHERE a.REC_STATUS NOT IN ('L','D') AND a.ELEMENT_ID = h_id:n     +
1    ORDER BY TYPE_ID, ELEMENT_NAME:
1649:
1                EXIT:: SP_S_PROFILE(650) :: cplan.sql(13544) :: :
14:
1DELETE FROM PMO_TMP_RES_SET:
1PMO_TMP_RES_SET:
1MIN:
1MAX:
1SUM:
1INSERT INTO PMO_TMP_RES_SET:n         (:n         RESOURCE_ID,:n         RESO+
1URCE_TYPE ,:n         PLANNED_START ,:n         PLANNED_FINISH ,:n         PL+
1ANNED_EFFORT ,:n         START_DATE ,:n         FINISH_DATE ,:n         WORK +
1,:n         REMAINING:n         ):n         SELECT:n         RESOURCE_ID,:n  +
1       RESOURCE_TYPE,:n         MIN(PLANNED_START) AS PLANNED_START,:n       +
1  MAX(PLANNED_FINISH) AS PLANNED_FINISH,:n         SUM(PLANNED_WORK) AS PLANN+
1ED_EFFORT,:n         MIN(SCHED_START) AS START_DATE,:n         MAX(SCHED_FINI+
1SH) AS FINISH_DATE,:n         SUM(SCHED_WORK) AS WORK,:n         SUM(ETC) AS +
1REMAINING:n         FROM TASK_ASSIGNMENTS WHERE PROJECT_ID=h_id AND PROJECT_A+
1SSIGNMENT_ID IS NuLL GROUP BY RESOURCE_ID, RESOURCE_TYPE:
1694:
1           EXCEPTION:: SP_S_PROFILE(695) :: cplan.sql(13589) :: :
1INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME, PLANNED_START, PLANN+
1ED_FINISH, PLANNED_EFFORT,START_DATE, FINISH_DATE,:n         WORK, REMAINING)+
1:n         SELECT DISTINCT r.PROFILE_ID,r.PROFILE_NAME,:n         a.PLANNED_S+
1TART, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK,+
1 a.REMAINING:n         FROM PMO_TMP_RES_SET a JOIN PROFILES r ON a.RESOURCE_T+
1YPE='1' AND r.PROFILE_ID=a.RESOURCE_ID JOIN TMT_COMPETENCIES c ON c.ELEMENT_I+
1D=r.ROLE_ID:n         JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID:
1720:
1           EXCEPTION:: SP_S_PROFILE(721) :: cplan.sql(13615) :: :
1SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS EL+
1EMENT_NAME,13 AS TYPE_ID,p.E_MAIL_ADDRESS,:n      a.PLANNED_START, a.PLANNED_+
1FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 A+
1S T_UNIT,h_per_day AS MINUTE_PER_DAY,:n      '' AS BEST_START,'' AS BEST_FINI+
1SH, 0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_+
1STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK:n      FROM PMO_TMP_RES_S+
1ET a JOIN RESOURCES p ON a.RESOURCE_TYPE = '0' AND a.RESOURCE_ID=p.RESOURCE_I+
1D:n      :n      UNION ALL:n      :n      SELECT ELEMENT_ID,'' AS RES_ASSIGNM+
1ENT_ID, ELEMENT_NAME,41 AS TYPE_ID,'' AS E_MAIL_ADDRESS,:n      PLANNED_START+
1, PLANNED_FINISH,PLANNED_EFFORT,   START_DATE, FINISH_DATE, WORK, REMAINING,0+
1 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,:n      '' AS BEST_START,''  AS BEST_F+
1INISH,0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIG+
1N_STRING,'' AS FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK:n      FROM PMO_TMP+
1_CPLN01_TEMP ORDER BY TYPE_ID:
1737:
1                EXIT:: SP_S_PROFILE(738) :: cplan.sql(13632) :: :
15:
1781:
1           EXCEPTION:: SP_S_PROFILE(782) :: cplan.sql(13676) :: :
1INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME, RESOURCE_TYPE, TYPE_+
1ID, E_MAIL_ADDRESS, PLANNED_START, PLANNED_FINISH, PLANNED_EFFORT,START_DATE,+
1 FINISH_DATE,:n         WORK, REMAINING, FS_LABOR_CODE):n         SELECT DIST+
1INCT r.PROFILE_ID ,r.PROFILE_NAME ,'1',41 ,'',:n         a.PLANNED_START, a.P+
1LANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAIN+
1ING,:n         '':n         FROM PMO_TMP_RES_SET a JOIN PROFILES r ON a.RESOU+
1RCE_TYPE='1' AND r.PROFILE_ID=a.RESOURCE_ID JOIN TMT_COMPETENCIES c ON c.ELEM+
1ENT_ID=r.ROLE_ID:n         JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_I+
1D:n         UNION ALL:n         SELECT DISTINCT ra.RESOURCE_ID ,p.FULL_NAME ,+
1'0',185 ,p.E_MAIL_ADDRESS,:n         TO_TIMESTAMP('1899-12-31 00::00::00') , +
1TO_TIMESTAMP('1899-12-31 00::00::00') ,0 ,:n         TO_TIMESTAMP('1899-12-31+
1 00::00::00') , TO_TIMESTAMP('1899-12-31 00::00::00') , 0 , 0 ,p.FS_LABOR_COD+
1E:n         FROM RESOURCE_ASSIGNMENTS ra, RESOURCES p:n         WHERE ra.PROJ+
1ECT_ID=h_id AND ra.RESOURCE_ID = p.RESOURCE_ID:n         AND NOT EXISTS(SELEC+
1T ta.TSK_ASSIGNMENT_ID FROM TASK_ASSIGNMENTS ta:n         WHERE ta.RES_ASSIGN+
1MENT_ID IN (SELECT a.RES_ASSIGNMENT_ID:n         FROM RESOURCE_ASSIGNMENTS a +
1WHERE a.PROJECT_ID = ra.PROJECT_ID AND a.RESOURCE_ID = ra.RESOURCE_ID)):
1817:
1           EXCEPTION:: SP_S_PROFILE(818) :: cplan.sql(13712) :: :
1SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS EL+
1EMENT_NAME,13 AS TYPE_ID,p.E_MAIL_ADDRESS,:n      a.PLANNED_START, a.PLANNED_+
1FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 A+
1S T_UNIT,h_per_day AS MINUTE_PER_DAY,:n      '' AS BEST_START,'' AS BEST_FINI+
1SH, 0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_+
1STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK:n      FROM PMO_TMP_RES_S+
1ET a JOIN RESOURCES p ON a.RESOURCE_TYPE = '0' AND a.RESOURCE_ID=p.RESOURCE_I+
1D:n      UNION ALL:n      SELECT ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, ELEMENT_+
1NAME,TYPE_ID,E_MAIL_ADDRESS,:n      PLANNED_START, PLANNED_FINISH,PLANNED_EFF+
1ORT,   START_DATE, FINISH_DATE, WORK, REMAINING,0 AS T_UNIT,h_per_day AS MINU+
1TE_PER_DAY,:n      '' AS BEST_START,''  AS BEST_FINISH,0 AS CALENDAR, 0 AS PL+
1ANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING, FS_LABOR_CODE, '' +
1AS OBJECTIVES,'' AS RANK:n      FROM PMO_TMP_CPLN01_TEMP WHERE RESOURCE_TYPE=+
1'1' AND TYPE_ID = 41:n      UNION ALL:n      SELECT ELEMENT_ID,'' AS RES_ASSI+
1GNMENT_ID, ELEMENT_NAME,TYPE_ID,E_MAIL_ADDRESS,:n      PLANNED_START, PLANNED+
1_FINISH,PLANNED_EFFORT,:n      START_DATE, FINISH_DATE, WORK, REMAINING,0 AS +
1T_UNIT,h_per_day AS MINUTE_PER_DAY,'' AS BEST_START,'' AS BEST_FINISH,:n     +
1 0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STR+
1ING,FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK:n      FROM PMO_TMP_CPLN01_TEM+
1P:n      WHERE RESOURCE_TYPE = '0' AND TYPE_ID = 185:n      ORDER BY TYPE_ID +
1DESC, ELEMENT_NAME:
1840:
1                EXIT:: SP_S_PROFILE(841) :: cplan.sql(13735) :: :
17:
1PLANNED_REMAINING:
1PA:
1DESCRIPTION:
1LEVEL_PLANNED_START:
1LEVEL_PLANNED_FINISH:
1LEVEL_SCHED_START:
1LEVEL_SCHED_FINISH:
1ESTIMATED_BASELINE1_START:
1ESTIMATED_BASELINE1_FINISH:
1ESTIMATED_BASELINE1_EFFORT:
1ESTIMATED_BASELINE3_START:
1ESTIMATED_BASELINE3_FINISH:
1ESTIMATED_BASELINE3_EFFORT:
1PROJECT_ASSIGNMENTS:
1SELECT t.RESOURCE_ID AS ELEMENT_ID, t.RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEME+
1NT_NAME, 13 AS TYPE_ID,p.E_MAIL_ADDRESS,:n      b.PLANNED_START, b.PLANNED_FI+
1NISH, b.BEST AS PLANNED_EFFORT, b.PLANNED_WORK AS PLANNED_REMAINING, a.BAK_ST+
1ART_DT AS START_DATE, a.BAK_FINISH_DT AS FINISH_DATE,:n      a.BEST AS WORK, +
1a.SCHED_WORK AS REMAINING, h_per_day AS MINUTE_PER_DAY,'' AS BEST_START, '' A+
1S BEST_FINISH,:n      0 AS CALENDAR,0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILI+
1TY, (SELECT CLOB_DATA FROM EMPTYLOB) AS ASSIGN_STRING,p.FS_LABOR_CODE, '' AS +
1OBJECTIVES,'' AS RANK,:n      pa.REGULAR_COST, pa.REGULAR_RATE, pa.SPECIAL_CO+
1ST, pa.SPECIAL_RATE, pa.DESCRIPTION,:n      pa.LEVEL_PLANNED_START, pa.LEVEL_+
1PLANNED_FINISH, pa.LEVEL_SCHED_START, pa.LEVEL_SCHED_FINISH,:n      pa.ESTIMA+
1TED_BASELINE1_START, pa.ESTIMATED_BASELINE1_FINISH, pa.ESTIMATED_BASELINE1_EF+
1FORT,:n      pa.ESTIMATED_BASELINE3_START, pa.ESTIMATED_BASELINE3_FINISH, pa.+
1ESTIMATED_BASELINE3_EFFORT:n      FROM RESOURCE_ASSIGNMENTS t JOIN PROJECT_AS+
1SIGNMENTS pa ON pa.PROJECT_ID = h_id AND pa.RESOURCE_ID = t.RESOURCE_ID:n    +
1  LEFT JOIN TASK_ASSIGNMENTS a ON a.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND+
1 a.BIT_FLAG='2':n      LEFT JOIN TASK_ASSIGNMENTS b ON b.RES_ASSIGNMENT_ID=t.+
1RES_ASSIGNMENT_ID AND b.BIT_FLAG='0':n      JOIN RESOURCES p ON t.RESOURCE_ID+
1=p.RESOURCE_ID:n      WHERE t.ELEMENT_ID=h_id AND t.RESOURCE_TYPE='0' AND (a.+
1TSK_ASSIGNMENT_ID IS NOT NuLL OR b.TSK_ASSIGNMENT_ID IS NOT NuLL):n      UNIO+
1N ALL:n      SELECT t.RESOURCE_ID AS ELEMENT_ID, a.RES_ASSIGNMENT_ID, p.PROFI+
1LE_NAME AS ELEMENT_NAME, 41 AS TYPE_ID,'' AS E_MAIL_ADDRESS,:n      t.PLANNED+
1_START, t.PLANNED_FINISH,t.BEST AS PLANNED_EFFORT,t.PLANNED_WORK AS PLANNED_R+
1EMAINING,TO_TIMESTAMP('1899-12-31 00::00::00') AS START_DATE, TO_TIMESTAMP('1+
1899-12-31 00::00::00') AS FINISH_DATE,:n      0 AS WORK,0 AS REMAINING,h_per_+
1day AS MINUTE_PER_DAY,'' AS BEST_START, '' AS BEST_FINISH,:n      0 AS CALEND+
1AR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, t.ASSIGN_STRING,'' AS FS_LAB+
1OR_CODE, '' AS OBJECTIVES,'' AS RANK,:n      pa.REGULAR_COST, pa.REGULAR_RATE+
1, pa.SPECIAL_COST, pa.SPECIAL_RATE, pa.DESCRIPTION,:n      pa.LEVEL_PLANNED_S+
1TART, pa.LEVEL_PLANNED_FINISH, pa.LEVEL_SCHED_START, pa.LEVEL_SCHED_FINISH,:n+
1      pa.ESTIMATED_BASELINE1_START, pa.ESTIMATED_BASELINE1_FINISH, pa.ESTIMAT+
1ED_BASELINE1_EFFORT,:n      pa.ESTIMATED_BASELINE3_START, pa.ESTIMATED_BASELI+
1NE3_FINISH, pa.ESTIMATED_BASELINE3_EFFORT:n      FROM RESOURCE_ASSIGNMENTS a +
1JOIN PROFILES p ON a.RESOURCE_ID=p.PROFILE_ID:n      JOIN TASK_ASSIGNMENTS t +
1ON t.RES_ASSIGNMENT_ID=a.RES_ASSIGNMENT_ID:n      JOIN PROJECT_ASSIGNMENTS pa+
1 ON pa.PROJECT_ID = h_id AND pa.RESOURCE_ID = a.RESOURCE_ID:n      WHERE a.EL+
1EMENT_ID=h_id AND a.RESOURCE_TYPE='1':n      ORDER BY TYPE_ID DESC, ELEMENT_N+
1AME:
1938:
1                EXIT:: SP_S_PROFILE(939) :: cplan.sql(13833) :: :
18:
1DELETE FROM PMO_TMP_PRJ_INFO:
1PMO_TMP_PRJ_INFO:
1PRJ_PLANNED_START:
1PRJ_PLANNED_FINISH:
1PRJ_SCHED_START:
1PRJ_SCHED_FINISH:
1SCHED_REMAINING:
1INSERT INTO PMO_TMP_PRJ_INFO:n         (RESOURCE_ID,:n         RESOURCE_TYPE,+
1:n         PRJ_PLANNED_START,:n         PRJ_PLANNED_FINISH,:n         PLANNED+
1_REMAINING,:n         PLANNED_EFFORT,:n         PRJ_SCHED_START ,:n         P+
1RJ_SCHED_FINISH,:n         SCHED_REMAINING,:n         WORK,:n         TYPE_ID+
1:n         ):n         SELECT:n         RESOURCE_ID,:n         RESOURCE_TYPE,+
1:n         MIN(PLANNED_START)  AS PRJ_PLANNED_START,:n         MAX(PLANNED_FI+
1NISH) AS PRJ_PLANNED_FINISH,:n         SUM(PLANNED_WORK)   AS PLANNED_REMAINI+
1NG,:n         SUM(CASE WHEN BIT_FLAG='0' THEN BEST ELSE 0 END) AS PLANNED_EFF+
1ORT,:n         MIN(SCHED_START)    AS PRJ_SCHED_START,:n         MAX(SCHED_FI+
1NISH)   AS PRJ_SCHED_FINISH,:n         SUM(SCHED_WORK)     AS SCHED_REMAINING+
1,:n         SUM(CASE WHEN BIT_FLAG='2' THEN BEST ELSE 0 END) AS WORK, 13:n   +
1      FROM TASK_ASSIGNMENTS:n         WHERE PROJECT_ID=h_id AND TASK_ID=h_id+
1:n         GROUP BY RESOURCE_ID, RESOURCE_TYPE:
1986:
1           EXCEPTION:: SP_S_PROFILE(987) :: cplan.sql(13881) :: :
1DELETE FROM PMO_TMP_RES_INFO:
1PMO_TMP_RES_INFO:
1INSERT INTO PMO_TMP_RES_INFO:n         (:n         RESOURCE_ID,:n         RES+
1OURCE_TYPE,:n         PLANNED_START,:n         PLANNED_FINISH,:n         PLAN+
1NED_EFFORT,:n         START_DATE,:n         FINISH_DATE,:n         WORK,:n   +
1      REMAINING,:n         PRJ_PLANNED_START,:n         PRJ_PLANNED_FINISH,:n+
1         PLANNED_REMAINING,:n         PRJ_SCHED_START,:n         PRJ_SCHED_FI+
1NISH,:n         SCHED_REMAINING,:n         LEVEL_PLANNED_START,:n         LEV+
1EL_PLANNED_FINISH,:n         LEVEL_SCHED_START,:n         LEVEL_SCHED_FINISH,+
1:n         ESTIMATED_BASELINE1_START,:n         ESTIMATED_BASELINE1_FINISH,:n+
1         ESTIMATED_BASELINE1_EFFORT,:n         ESTIMATED_BASELINE3_START,:n  +
1       ESTIMATED_BASELINE3_FINISH,:n         ESTIMATED_BASELINE3_EFFORT,:n   +
1      REGULAR_COST,:n         REGULAR_RATE,:n         SPECIAL_COST,:n        +
1 SPECIAL_RATE,:n         TYPE_ID,:n         DESCRIPTION:n         ):n        +
1 SELECT:n         a.RESOURCE_ID,      a.RESOURCE_TYPE,        a.PLANNED_START+
1,        a.PLANNED_FINISH,:n         a.PLANNED_WORK,     a.SCHED_START,      +
1    a.SCHED_FINISH,         a.SCHED_WORK,:n         a.ETC,              b.PRJ+
1_PLANNED_START,    b.PRJ_PLANNED_FINISH,   b.PLANNED_REMAINING,:n         b.P+
1RJ_SCHED_START,  b.PRJ_SCHED_FINISH,     b.SCHED_REMAINING,:n         a.LEVEL+
1_PLANNED_START, a.LEVEL_PLANNED_FINISH, a.LEVEL_SCHED_START, a.LEVEL_SCHED_FI+
1NISH,:n         a.ESTIMATED_BASELINE1_START, a.ESTIMATED_BASELINE1_FINISH, a.+
1ESTIMATED_BASELINE1_EFFORT,:n         a.ESTIMATED_BASELINE3_START, a.ESTIMATE+
1D_BASELINE3_FINISH, a.ESTIMATED_BASELINE3_EFFORT,:n         a.REGULAR_COST,  +
1   a.REGULAR_RATE,         a.SPECIAL_COST,         a.SPECIAL_RATE,:n         +
1COALESCE(b.TYPE_ID,CASE WHEN (a.PLANNED_WORK+a.SCHED_WORK) = 0 THEN 185 ELSE +
113 END), a.DESCRIPTION:n         FROM PROJECT_ASSIGNMENTS a LEFT OUTER JOIN P+
1MO_TMP_PRJ_INFO b ON a.RESOURCE_ID = b.RESOURCE_ID:n         WHERE a.PROJECT_+
1ID=h_id:
11055:
1           EXCEPTION:: SP_S_PROFILE(1056) :: cplan.sql(13950) :: :
1RI:
1UPDATE PMO_TMP_RES_INFO RI SET TYPE_ID = 13 WHERE TYPE_ID = 185 AND EXISTS(:n+
1         SELECT TASK_ID FROM TASK_ASSIGNMENTS T WHERE T.PROJECT_ID=h_id AND T+
1.RESOURCE_ID = RI.RESOURCE_ID AND T.RESOURCE_TYPE = '0'):
11076:
1           EXCEPTION:: SP_S_PROFILE(1077) :: cplan.sql(13971) :: :
1SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS EL+
1EMENT_NAME, a.TYPE_ID,p.E_MAIL_ADDRESS,:n      a.PLANNED_START, a.PLANNED_FIN+
1ISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 AS T+
1_UNIT,h_per_day AS MINUTE_PER_DAY,:n      '' AS BEST_START,'' AS BEST_FINISH,+
1 0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STR+
1ING,p.FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK,:n      CASE WHEN a.PRJ_PLAN+
1NED_START IS NOT NuLL OR a.PRJ_SCHED_START IS NOT NuLL THEN a.RESOURCE_ID ELS+
1E '' END AS PROJECT_ASSIGNMENT_ID, a.PRJ_PLANNED_START, a.PRJ_PLANNED_FINISH,+
1 a.PLANNED_REMAINING, a.PRJ_SCHED_START, a.PRJ_SCHED_FINISH, a.SCHED_REMAININ+
1G,:n      h_id AS PROJECT_ID,:n      a.LEVEL_PLANNED_START, a.LEVEL_PLANNED_F+
1INISH, a.LEVEL_SCHED_START, a.LEVEL_SCHED_FINISH,:n      a.ESTIMATED_BASELINE+
11_START, a.ESTIMATED_BASELINE1_FINISH, a.ESTIMATED_BASELINE1_EFFORT,:n      a+
1.ESTIMATED_BASELINE3_START, a.ESTIMATED_BASELINE3_FINISH, a.ESTIMATED_BASELIN+
1E3_EFFORT,:n      a.REGULAR_COST, a.REGULAR_RATE, a.SPECIAL_COST, a.SPECIAL_R+
1ATE, a.DESCRIPTION:n      FROM PMO_TMP_RES_INFO a JOIN RESOURCES p ON a.RESOU+
1RCE_TYPE = '0' AND a.RESOURCE_ID=p.RESOURCE_ID:n      UNION ALL:n      SELECT+
1  r.PROFILE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, r.PROFILE_NAME AS ELEME+
1NT_NAME,41 AS TYPE_ID,'' AS E_MAIL_ADDRESS,:n      a.PLANNED_START, a.PLANNED+
1_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 +
1AS T_UNIT,h_per_day AS MINUTE_PER_DAY,:n      '' AS BEST_START,''  AS BEST_FI+
1NISH,0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN+
1_STRING,'' AS FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK,:n      CASE WHEN a.+
1PRJ_PLANNED_START IS NOT NuLL OR a.PRJ_SCHED_START IS NOT NuLL THEN a.RESOURC+
1E_ID ELSE '' END AS PROJECT_ASSIGNMENT_ID, a.PRJ_PLANNED_START, a.PRJ_PLANNED+
1_FINISH, a.PLANNED_REMAINING, a.PRJ_SCHED_START, a.PRJ_SCHED_FINISH, a.SCHED_+
1REMAINING,:n      h_id AS PROJECT_ID,:n      a.LEVEL_PLANNED_START, a.LEVEL_P+
1LANNED_FINISH, a.LEVEL_SCHED_START, a.LEVEL_SCHED_FINISH,:n      a.ESTIMATED_+
1BASELINE1_START, a.ESTIMATED_BASELINE1_FINISH, a.ESTIMATED_BASELINE1_EFFORT,+
1:n      a.ESTIMATED_BASELINE3_START, a.ESTIMATED_BASELINE3_FINISH, a.ESTIMATE+
1D_BASELINE3_EFFORT,:n      a.REGULAR_COST, a.REGULAR_RATE, a.SPECIAL_COST, a.+
1SPECIAL_RATE, a.DESCRIPTION:n      FROM PMO_TMP_RES_INFO a JOIN PROFILES r ON+
1 a.RESOURCE_TYPE='1' AND r.PROFILE_ID=a.RESOURCE_ID:n      JOIN TMT_COMPETENC+
1IES c ON c.ELEMENT_ID=r.ROLE_ID:n      JOIN TMT_COMPETENCIES d ON d.ELEMENT_I+
1D=c.PARENT_ID:
11105:
1                EXIT:: SP_S_PROFILE(1106) :: cplan.sql(14000) :: :
1SELECT PROFILE_NAME INTO h_in FROM PROFILES WHERE PROFILE_ID = h_id:
11119:
1           EXCEPTION:: SP_S_PROFILE(1120) :: cplan.sql(14014) :: :
1SC_SQL_SUCCESS:
1ROLE_TYPE:
1ORG_ID:
1ORG:
1ORG_NAME:
1GEO_ID:
1GEO:
1GEO_NAME:
1POOL_ID:
1POOL:
1POOL_NAME:
1TMT_GEOGRAPHIES:
1DS:
1CS:
1PMO_ORASTD:
1STRINGCONCAT:
1RTRIM:
1SELECT h_id AS ELEMENT_ID, h_in AS ELEMENT_NAME,41 AS TYPE_ID,30 AS ROLE_TYPE+
1, p.ROLE_ID, h_in AS ROLE_NAME,:n         ORG_ID, org.ELEMENT_NAME AS ORG_NAM+
1E, GEO_ID, geo.ELEMENT_NAME AS GEO_NAME, POOL_ID, pool.ELEMENT_NAME AS POOL_N+
1AME:n         FROM PROFILES p:n         LEFT JOIN TMT_ORGANIZATIONS org ON or+
1g.ELEMENT_ID  = ORG_ID:n         LEFT JOIN TMT_GEOGRAPHIES geo   ON geo.ELEME+
1NT_ID  = GEO_ID:n         LEFT JOIN TMT_POOLS pool        ON pool.ELEMENT_ID +
1= POOL_ID:n         WHERE p.PROFILE_ID = h_id:n         UNION ALL:n         S+
1ELECT h_id AS ELEMENT_ID, h_in AS ELEMENT_NAME,41 AS TYPE_ID,ds.TYPE_ID AS RO+
1LE_TYPE,:n         cs.ELEMENT_ID AS ROLE_ID, PMO_ORASTD.StringConcat(PMO_ORAS+
1TD.StringConcat(RTRIM(ds.ELEMENT_NAME), ', '), RTRIM(cs.ELEMENT_NAME)) AS ROL+
1E_NAME,:n         '' ORG_ID, '' AS ORG_NAME, '' GEO_ID, '' AS GEO_NAME, '' PO+
1OL_ID, '' AS POOL_NAME:n         FROM RESOURCE_ASSIGN_ROLES r LEFT OUTER JOIN+
1 TMT_COMPETENCIES cs ON r.ROLE_ID=cs.ELEMENT_ID:n         LEFT OUTER JOIN TMT+
1_COMPETENCIES ds ON ds.ELEMENT_ID=cs.PARENT_ID WHERE r.RES_ASSIGNMENT_ID=h_id+
1:n         UNION ALL:n         SELECT h_id AS ELEMENT_ID, h_in AS ELEMENT_NAM+
1E,41 AS TYPE_ID,ds.TYPE_ID AS ROLE_TYPE,:n         cs.ELEMENT_ID AS ROLE_ID, +
1PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(ds.ELEMENT_NAME), ', ')+
1, RTRIM(cs.ELEMENT_NAME)) AS ROLE_NAME,:n         '' ORG_ID, '' AS ORG_NAME, +
1'' GEO_ID, '' AS GEO_NAME, '' POOL_ID, '' AS POOL_NAME:n         FROM RESOURC+
1E_ASSIGN_SKILLS s LEFT OUTER JOIN TMT_COMPETENCIES cs ON s.SKILL_ID=cs.ELEMEN+
1T_ID:n         LEFT OUTER JOIN TMT_COMPETENCIES ds ON ds.ELEMENT_ID=cs.PARENT+
1_ID WHERE s.RES_ASSIGNMENT_ID=h_id:n         ORDER BY ELEMENT_ID,TYPE_ID:
11146:
1                EXIT:: SP_S_PROFILE(1147) :: cplan.sql(14041) :: :
1SELECT CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN t.TSK_ASSIGNMENT_ID ELS+
1E a.RES_ASSIGNMENT_ID END AS ELEMENT_ID, a.RESOURCE_ID,:n      CASE WHEN a.RE+
1SOURCE_TYPE='1' THEN (SELECT p.PROFILE_NAME FROM PROFILES p WHERE a.RESOURCE_+
1ID = p.PROFILE_ID):n      ELSE (SELECT r.FULL_NAME FROM RESOURCES r WHERE a.R+
1ESOURCE_ID = r.RESOURCE_ID) END AS ELEMENT_NAME,:n      CASE WHEN t.TSK_ASSIG+
1NMENT_ID IS NuLL THEN 185 WHEN a.RESOURCE_TYPE='1' THEN 41 ELSE 13 END AS TYP+
1E_ID,:n      a.RES_ASSIGNMENT_ID, a.PARTICIPANT, t.START_DT, COALESCE(PLANNED+
1_FINISH,SCHED_FINISH) AS FINISH_DT,:n      t.SD, t.FD, COALESCE(t.PLANNED_STA+
1RT,t.SCHED_START) AS PLANNED_START, COALESCE(PLANNED_FINISH,SCHED_FINISH) AS +
1PLANNED_FINISH,:n      CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK ELSE PLANNED_+
1WORK END AS PLANNED_WORK,:n      CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK - W+
1ORK_TODATE ELSE PLANNED_WORK END AS REMAINING,:n      CASE WHEN t.ASSIGN_UNIT+
1S > t.UNITS_MSP THEN t.ASSIGN_UNITS ELSE t.UNITS_MSP END AS UNITS_PERCENT,:n +
1     0 AS MILESTONE_PERCENT,:n      CASE WHEN t.WORK_TODATE = 0 THEN 0 ELSE T+
1runc((TO_NUMBER(t.WORK_TODATE)/TO_NUMBER(PLANNED_WORK+SCHED_WORK))*10000) END+
1 AS WORK_PERCENT,:n      t.BAK_START_DT,t.BAK_FINISH_DT,t.BAK_WORK,:n      t.+
1FORECAST_START,t.FORECAST_FINISH,t.ETC, t.ACTUAL_START,t.ACTUAL_FINISH,:n    +
1  t.WORK_TODATE AS ACTUAL,PROPOSED_START, PROPOSED_FINISH, PROPOSED_WORK,:n  +
1    h_per_day AS MINUTE_PER_DAY, a.WORK_LOCATION, a.SECURITY_ID, g.ELEMENT_NA+
1ME AS SECURITY_NAME,:n      COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_+
1SECURITY_ID, g2.ELEMENT_NAME AS CRI_SECURITY_NAME, t.REGULAR_COST,t.REGULAR_R+
1ATE,t.SPECIAL_COST,t.SPECIAL_RATE,:n      CASE WHEN a.RESOURCE_TYPE='0' THEN +
1(SELECT r.FS_LABOR_CODE FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID)+
1:n      ELSE '' END AS FS_LABOR_CODE,a.REC_STATUS,t.ASSIGN_STRING,t.BASELINE1+
1_START,t.BASELINE1_FINISH,t.BASELINE1_WORK,:n      t.PROJECT_ASSIGNMENT_ID,t.+
1TIMECODE2_ID, TC.ELEMENT_NAME AS TIMECODE2_NAME, t.T_UNIT, t.C_EFFORT, t.C_UN+
1IT, t.C_CONTOUR, t.CONTOUR_TYPE, t.C_OCCURRENCE, t.BILLABLE, t.CAPITAL_EXPENS+
1E,:n      t.ASSIGNMENT_DELAY, CASE WHEN t.BIT_FLAG IS NuLL THEN '' WHEN t.BIT+
1_FLAG = '0' THEN '1' ELSE '2' END AS BIT_FLAG, t.RATE_ADJUSTMENT, 0 AS ASSIGN+
1_COUNT, :n      CASE WHEN BIT_FLAG = '2' THEN SCHED_DURATION ELSE PLANNED_DUR+
1ATION END AS DURATION,:n      CASE WHEN h_level_state = 1 THEN COALESCE(t.LEV+
1EL_START,t.UNRESOLVED_START) ELSE TO_TIMESTAMP('1899-12-31 00::00::00') END A+
1S LEVEL_START,:n      CASE WHEN h_level_state = 1 THEN COALESCE(t.LEVEL_FINIS+
1H,t.UNRESOLVED_FINISH) ELSE TO_TIMESTAMP('1899-12-31 00::00::00') END AS LEVE+
1L_FINISH,:n      CASE WHEN h_level_state = 1 THEN CASE WHEN t.LEVEL_EFFORT = +
10 THEN t.ETC ELSE t.LEVEL_EFFORT END ELSE 0 END AS LEVEL_EFFORT, t.MILESTONE_+
1PERCENT AS PERCENT_DISCRETE, 0 as TOP_ID,:n      CASE WHEN t.TSK_ASSIGNMENT_I+
1D IS NOT NuLL AND t.PROJECT_ASSIGNMENT_ID = t.TSK_ASSIGNMENT_ID AND (t.LEVEL_+
1START <> t.UNRESOLVED_START OR t.LEVEL_FINISH <> t.UNRESOLVED_FINISH) THEN '1+
1' ELSE '0' END AS LEVEL_AFFECTED_FLAG,:n      T.LABOR_COST_CC_ID,:n      EC.E+
1LEMENT_NAME AS LABOR_COST_CC_NAME,:n      T.LABOR_REVENUE_CC_ID,:n      CC.EL+
1EMENT_NAME AS LABOR_REVENUE_CC_NAME, T.DEFAULT_LABOR_FLAG,:n      T.CONTRACT_+
1ID, CTR.ELEMENT_NAME AS CONTRACT_NAME:n      FROM RESOURCE_ASSIGNMENTS a JOIN+
1 TASK_ASSIGNMENTS t ON t.TSK_ASSIGNMENT_ID = h_id AND a.RES_ASSIGNMENT_ID = t+
1.RES_ASSIGNMENT_ID:n      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY+
1_ID=g.ELEMENT_ID:n      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g2 ON COALESCE(a.+
1CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID:n      LEFT OUTER JOIN TMT_TIMEC+
1ODES TC ON TC.ELEMENT_ID= t.TIMECODE2_ID:n      LEFT OUTER JOIN TMT_CHARGECOD+
1ES EC ON EC.ELEMENT_ID = T.LABOR_COST_CC_ID:n      LEFT OUTER JOIN TMT_CHARGE+
1CODES CC ON CC.ELEMENT_ID = T.LABOR_REVENUE_CC_ID:n      LEFT OUTER JOIN TMT_+
1CONTRACTS CTR ON CTR.ELEMENT_ID = T.CONTRACT_ID:n      ORDER BY TYPE_ID DESC,+
1 ELEMENT_NAME:
11190:
1                EXIT:: SP_S_PROFILE(1191) :: cplan.sql(14085) :: :
1ACCEPTED_DATE:
1DELAY:
1SELECT a.RES_ASSIGNMENT_ID AS ELEMENT_ID, a.RESOURCE_ID,:n      CASE WHEN a.R+
1ESOURCE_TYPE='1' THEN (SELECT p.PROFILE_NAME FROM PROFILES p WHERE a.RESOURCE+
1_ID = p.PROFILE_ID):n      ELSE (SELECT r.FULL_NAME FROM RESOURCES r WHERE a.+
1RESOURCE_ID = r.RESOURCE_ID) END AS ELEMENT_NAME,:n      CASE WHEN a.RESOURCE+
1_TYPE='1' THEN PMO_CNST.SC_PROFILE ELSE 185 END AS TYPE_ID,  a.RES_ASSIGNMENT+
1_ID, a.PARTICIPANT, TO_TIMESTAMP('1899-12-31 00::00::00') AS PLANNED_START, T+
1O_TIMESTAMP('1899-12-31 00::00::00') AS PLANNED_FINISH,:n      0 AS PLANNED_W+
1ORK,TO_TIMESTAMP('1899-12-31 00::00::00') AS START_DT,TO_TIMESTAMP('1899-12-3+
11 00::00::00') AS FINISH_DT,:n      TO_TIMESTAMP('1899-12-31 00::00::00') AS +
1SCHED_START,TO_TIMESTAMP('1899-12-31 00::00::00') AS SCHED_FINISH, 0 AS SCHED+
1_WORK,:n      TO_TIMESTAMP('1899-12-31 00::00::00') AS BAK_START_DT,TO_TIMEST+
1AMP('1899-12-31 00::00::00') AS BAK_FINISH_DT, 0 AS BAK_WORK,:n      TO_TIMES+
1TAMP('1899-12-31 00::00::00') AS FORECAST_START,TO_TIMESTAMP('1899-12-31 00::+
100::00') AS FORECAST_FINISH,0 AS ETC,:n      TO_TIMESTAMP('1899-12-31 00::00+
1::00') AS ACTUAL_START,TO_TIMESTAMP('1899-12-31 00::00::00') AS ACTUAL_FINISH+
1,:n      0 AS ACTUAL,h_per_day AS MINUTE_PER_DAY, a.WORK_LOCATION, TO_TIMESTA+
1MP('1899-12-31 00::00::00') AS ACCEPTED_DATE,:n      a.SECURITY_ID, g.ELEMENT+
1_NAME AS SECURITY_NAME, COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_SECU+
1RITY_ID, g2.ELEMENT_NAME AS CRI_SECURITY_NAME,:n      0 AS REGULAR_COST,0 AS +
1REGULAR_RATE,0 AS SPECIAL_COST, 0 AS SPECIAL_RATE,:n      CASE WHEN a.RESOURC+
1E_TYPE='0' THEN (SELECT r.FS_LABOR_CODE FROM RESOURCES r WHERE a.RESOURCE_ID +
1= r.RESOURCE_ID):n      ELSE '' END AS FS_LABOR_CODE,:n      a.REC_STATUS,'' +
1AS ASSIGN_STRING,:n      TO_TIMESTAMP('1899-12-31 00::00::00') AS BASELINE1_S+
1TART,TO_TIMESTAMP('1899-12-31 00::00::00') AS BASELINE1_FINISH,0 AS BASELINE1+
1_WORK,:n      '' AS PROJECT_ASSIGNMENT_ID, '' AS TIMECODE2_ID, '' AS TIMECODE+
12_NAME, 0 AS T_UNIT,:n      0 AS C_EFFORT, 0 AS C_UNIT, 0 AS C_CONTOUR, 0 AS +
1CONTOUR_TYPE, 0 AS C_OCCURRENCE, 0 AS BILLABLE,:n      0 AS DELAY:n      FROM+
1 RESOURCE_ASSIGNMENTS a:n      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SEC+
1URITY_ID=g.ELEMENT_ID:n      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g2 ON COALES+
1CE(a.CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID:n      WHERE a.REC_STATUS N+
1OT IN ('L','D') AND a.ELEMENT_ID = h_id:n      ORDER BY TYPE_ID DESC, ELEMENT+
1_NAME:
11219:
1                EXIT:: SP_S_PROFILE(1220) :: cplan.sql(14114) :: :
112:
1DELETE FROM PMO_TMP_RTA:
1PMO_TMP_RTA:
1PA_FLAG:
1TABLE_TYPE:
1SC_TABLE_WBS:
1INSERT INTO PMO_TMP_RTA (RESOURCE_ID, RESOURCE_TYPE, PA_FLAG):n         SELEC+
1T RESOURCE_ID, RESOURCE_TYPE, 'N' AS PA_FLAG:n         FROM TASK_ASSIGNMENTS+
1:n         WHERE PROJECT_ID = h_id AND:n         PROJECT_ASSIGNMENT_ID IS NuL+
1L AND:n         TABLE_TYPE = PMO_CNST.SC_TABLE_WBS:n         GROUP BY RESOURC+
1E_ID, RESOURCE_TYPE:
11249:
1           EXCEPTION:: SP_S_PROFILE(1250) :: cplan.sql(14144) :: :
1DELETE FROM PMO_TMP_CP_PRJ:
1PMO_TMP_CP_PRJ:
1INSERT INTO PMO_TMP_CP_PRJ (RESOURCE_ID, RESOURCE_TYPE, PA_FLAG):n         SE+
1LECT  RESOURCE_ID, RESOURCE_TYPE, 'Y' AS PA_FLAG:n         FROM TASK_ASSIGNME+
1NTS:n         WHERE PROJECT_ID = h_id AND:n         TASK_ID = h_id:n         +
1GROUP BY RESOURCE_ID, RESOURCE_TYPE:
11277:
1           EXCEPTION:: SP_S_PROFILE(1278) :: cplan.sql(14172) :: :
1DELETE FROM PMO_TMP_RESOURCE_DATA:
1PMO_TMP_RESOURCE_DATA:
1INSERT INTO PMO_TMP_RESOURCE_DATA (RESOURCE_ID, RESOURCE_TYPE, PA_FLAG):n    +
1     SELECT a.RESOURCE_ID, a.RESOURCE_TYPE, a.PA_FLAG:n         FROM PMO_TMP_+
1RTA a LEFT OUTER JOIN PMO_TMP_CP_PRJ b ON a.RESOURCE_ID=b.RESOURCE_ID:n      +
1   UNION:n         SELECT b.RESOURCE_ID, b.RESOURCE_TYPE, b.PA_FLAG:n        +
1 FROM PMO_TMP_CP_PRJ b LEFT OUTER JOIN PMO_TMP_RTA a ON a.RESOURCE_ID=b.RESOU+
1RCE_ID:
11305:
1           EXCEPTION:: SP_S_PROFILE(1306) :: cplan.sql(14200) :: :
1NICK_NAME:
1PRJ:
1RD:
1SELECT a.RESOURCE_ID AS ELEMENT_ID, p.FULL_NAME AS ELEMENT_NAME,13 AS TYPE_ID+
1, p.NICK_NAME, a.PA_FLAG:n      FROM PMO_TMP_RESOURCE_DATA a JOIN RESOURCES p+
1 ON a.RESOURCE_TYPE = '0' AND a.RESOURCE_ID=p.RESOURCE_ID:n      LEFT OUTER J+
1OIN PMO_TMP_CP_PRJ PRJ ON a.RESOURCE_ID=PRJ.RESOURCE_ID:n      :n      UNION +
1ALL:n      :n      SELECT DISTINCT r.PROFILE_ID AS ELEMENT_ID, r.PROFILE_NAME+
1 AS ELEMENT_NAME,41 AS TYPE_ID, ' ' AS NICK_NAME, a.PA_FLAG:n      FROM PMO_T+
1MP_RESOURCE_DATA a JOIN PROFILES r ON a.RESOURCE_TYPE = '1' AND r.PROFILE_ID +
1= a.RESOURCE_ID:n      JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.ROLE_ID:n   +
1   JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID:n      LEFT OUTER JOIN+
1 PMO_TMP_CP_PRJ PRJ ON a.RESOURCE_ID=PRJ.RESOURCE_ID:n      :n      UNION ALL+
1:n      :n      SELECT DISTINCT ra.RESOURCE_ID AS ELEMENT_ID, p.FULL_NAME AS +
1ELEMENT_NAME, 185 AS TYPE_ID, p.NICK_NAME, 'N' AS PA_FLAG:n      FROM RESOURC+
1E_ASSIGNMENTS ra, RESOURCES p:n      WHERE ra.PROJECT_ID=h_id AND:n      ra.R+
1ESOURCE_ID = p.RESOURCE_ID AND:n      NOT EXISTS(SELECT rd.RESOURCE_ID FROM P+
1MO_TMP_RESOURCE_DATA rd WHERE ra.RESOURCE_ID = rd.RESOURCE_ID):n      ORDER B+
1Y TYPE_ID DESC, ELEMENT_NAME:
11331:
1                EXIT:: SP_S_PROFILE(1332) :: cplan.sql(14226) :: :
1SELECT t.ELEMENT_ID, p.FULL_NAME AS ELEMENT_NAME,13 AS TYPE_ID, p.NICK_NAME, +
1'N' AS PA_FLAG:n      FROM TMT_POOLS t JOIN RESOURCES p:n      ON t.ELEMENT_I+
1D = p.RESOURCE_ID:n      WHERE t.TYPE_ID = 13:n      AND t.RANK LIKE h_rank |+
1| '%':n      ORDER BY ELEMENT_NAME:
11345:
1                EXIT:: SP_S_PROFILE(1346) :: cplan.sql(14240) :: :
1SELECT R.RESOURCE_ID AS ELEMENT_ID, R.FULL_NAME AS ELEMENT_NAME,PMO_CNST.SC_R+
1ESOURCE AS TYPE_ID, R.NICK_NAME, 'N' AS PA_FLAG:n      FROM PROJECT_ASSIGNMEN+
1TS RA, RESOURCES R:n      WHERE RA.PROJECT_ID = h_id:n      AND RA.RESOURCE_I+
1D = R.RESOURCE_ID:n      ORDER BY ELEMENT_NAME:
11359:
1                EXIT:: SP_S_PROFILE(1360) :: cplan.sql(14254) :: :
11364:
1       RETURN_STATUS:: SP_S_PROFILE(1365) :: cplan.sql(14259) :: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
11373:
1           EXCEPTION:: SP_S_PROFILE(1374) :: cplan.sql(14268) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1ERROR:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
11385:
1           EXCEPTION:: SP_S_PROFILE(1386) :: cplan.sql(14280) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
2055
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 :2 a0 6b 1c 81 b0
a3 :2 a0 6b 1c 81 b0 a3 :2 a0
6b 1c 81 b0 a3 :2 a0 6b 1c
81 b0 a3 :2 a0 6b 1c 81 b0
a3 :2 a0 6b 1c 81 b0 a3 :2 a0
6b 1c 81 b0 a3 :2 a0 6b 1c
81 b0 a3 :2 a0 6b 1c 81 b0
a3 :2 a0 6b 1c 81 b0 a3 :2 a0
6b 1c 81 b0 a3 :2 a0 6b 1c
81 b0 a3 :2 a0 6b 1c 81 b0
a3 :2 a0 6b 1c 81 b0 a3 :2 a0
6b 1c 81 b0 a3 :2 a0 6b 1c
81 b0 :2 a0 6b :2 a0 6b 6e a5
b a 10 :2 a0 6b 6e :2 a0 6b
6e a5 b 6e 7e a0 b4 2e
6e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c a0 7e
b4 2e 5a :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b :2 a0
a5 57 b7 19 3c :2 a0 d :2 a0
d :2 a0 d :2 a0 51 a5 b 7e
51 b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a 52 10 :18 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :15 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a :4 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
6b a0 7e a0 6b b4 2e 5a
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 b7
19 3c b7 19 3c b7 19 3c
a0 b7 :2 a0 51 a5 b 7e 51
b4 2e :5 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c :8 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f a0 b7 19 :2 a0 51 a5 b
7e 51 b4 2e :5 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 19
3c a0 51 d a0 b7 19 :2 a0
51 a5 b a0 7e a0 6b b4
2e 5a :2 a0 51 a5 b 7e 51
b4 2e 5a a 10 :2 a0 51 a5
b a0 7e a0 6b b4 2e 5a
a 10 :2 a0 51 a5 b a0 7e
a0 6b b4 2e 5a a 10 :8 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 19 3c b7 :2 19 3c :2 a0
51 a5 b 7e 51 b4 2e :2 a0
6e 11e 11d :2 a0 6b 51 d :22 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
6e 11e 11d :2 a0 6b 51 d :7 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :12d a0
12a e9 dd :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 65
a0 b7 :2 a0 51 a5 b 7e 51
b4 2e 5a :2 a0 51 a5 b 7e
51 b4 2e 5a 52 10 :2 a0 6e
11e 11d :2 a0 6b 51 d :26 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 b4 2e d :2e a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :49 a0 12a e9 dd
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 65 a0 b7 19
:2 a0 51 a5 b 7e 51 b4 2e
:8d a0 12a e9 dd :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
65 a0 b7 19 :2 a0 51 a5 b
7e 51 b4 2e :3 a0 51 a5 b
a5 b 7e 51 b4 2e a0 51
d :7 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 51 a5 b 7e
51 b4 2e a0 6e d b7 a0
51 d :9 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 51 a5 b
7e 51 b4 2e :1c a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 51
a5 b 7e 51 b4 2e a0 6e
d b7 a0 6e d b7 :2 19 3c
b7 a0 6e d b7 :2 19 3c b7
:2 19 3c :20a a0 12a e9 dd :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 65 b7 :147 a0 12a e9 dd
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 65 b7 :2 19 3c
a0 b7 19 :2 a0 51 a5 b 7e
51 b4 2e :2 a0 6e 11e 11d :2 a0
6b 51 d :27 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 b4 2e
d :35 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :4c a0 12a e9 dd :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 65 a0 b7 19 :2 a0 51 a5
b 7e 51 b4 2e :2 a0 6e 11e
11d :2 a0 6b 51 d :27 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b a0
b4 2e d :62 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :6a a0 12a e9 dd :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 65 a0 b7 19 :2 a0
51 a5 b 7e 51 b4 2e :e4 a0
12a e9 dd :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 65
a0 b7 19 :2 a0 51 a5 b 7e
51 b4 2e :2 a0 6e 11e 11d :2 a0
6b 51 d :2f a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6e 11e 11d :2 a0
6b 51 d :71 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :10 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :d5 a0 12a e9 dd :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 65 a0 b7 19 :2 a0
51 a5 b a0 7e a0 6b b4
2e :5 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a :9a a0 12a e9 dd
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 65 b7 19 3c
a0 b7 19 :2 a0 51 a5 b 7e
51 b4 2e :145 a0 12a e9 dd :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 65 a0 b7 19 :2 a0
51 a5 b 7e 51 b4 2e :94 a0
12a e9 dd :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 65
a0 b7 19 :2 a0 51 a5 b 7e
51 b4 2e :2 a0 6e 11e 11d :2 a0
6b 51 d :10 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6e 11e 11d :2 a0
6b 51 d :e a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6e 11e 11d :2 a0
6b 51 d :26 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :6c a0 12a e9 dd :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 65 a0 b7 19 :2 a0
51 a5 b 7e 51 b4 2e :1a a0
12a e9 dd :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 65
a0 b7 19 :2 a0 51 a5 b a0
7e a0 6b b4 2e 5a :2 a0 51
a5 b a0 7e a0 6b b4 2e
5a 52 10 :1a a0 12a e9 dd :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 65 b7 19 :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b :2 a0
a5 57 b7 :2 19 3c b7 :2 a0 6b
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
:2 a0 6b :2 6e 7e :2 a0 6b b4 2e
7e 6e b4 2e 7e :2 a0 6b b4
2e a5 57 b7 19 3c :9 a0 12a
e9 dd :2 a0 65 b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c :7 a0 12a e9 dd
:2 a0 65 b7 a4 b1 11 4f b7
a6 9 a4 b1 11 68 4f 1d
17 b5 
2055
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 30
56 5a 5e 62 65 69 86 71
75 78 79 81 70 a3 91 6d
95 96 9e 90 c0 ae 8d b2
b3 bb ad dd cb aa cf d0
d8 ca fa e8 c7 ec ed f5
e7 117 105 e4 109 10a 112 104
133 122 126 101 12e 121 14f 13e
142 11e 14a 13d 16b 15a 15e 13a
166 159 187 176 17a 156 182 175
1a3 192 196 172 19e 191 1bf 1ae
1b2 18e 1ba 1ad 1df 1ca 1ce 1aa
1d2 1da 1c9 1ff 1ea 1ee 1c6 1f2
1fa 1e9 21f 20a 20e 1e6 212 21a
209 23f 22a 22e 206 232 23a 229
25f 24a 24e 226 252 25a 249 27f
26a 26e 246 272 27a 269 29f 28a
28e 266 292 29a 289 2bf 2aa 2ae
286 2b2 2ba 2a9 2df 2ca 2ce 2a6
2d2 2da 2c9 2ff 2ea 2ee 2c6 2f2
2fa 2e9 31f 30a 30e 2e6 312 31a
309 33f 32a 32e 306 332 33a 329
35f 34a 34e 326 352 35a 349 37f
36a 36e 346 372 37a 369 39f 38a
38e 366 392 39a 389 3bf 3aa 3ae
386 3b2 3ba 3a9 3c6 3ca 3a6 3ce
3d2 3d6 3d9 3de 3df 1 3e1 3e6
3ea 3ee 3f1 3f6 3fa 3fe 401 406
407 409 40e 411 415 416 41b 420
423 428 429 42e 431 435 436 43b
43e 443 444 449 44c 451 452 457
45a 45e 462 466 467 469 46e 46f
471 472 477 47a 47f 480 485 488
48d 48e 493 496 49a 49b 4a0 4a3
4a8 4a9 4ae 4b1 4b6 4b7 4bc 4bd
4c2 4c4 4c8 4cb 4cf 4d3 4d6 4da
4de 4e1 4e4 4e9 4ec 4ef 4f2 4f3
4f8 4f9 1 4fb 500 504 508 50b
510 515 518 51c 51d 522 523 528
52a 52e 531 535 538 539 53e 541
545 549 54c 550 554 557 55a 55f
562 565 568 569 56e 56f 1 571
576 57a 57e 581 586 58b 58e 592
593 598 59b 5a0 5a1 5a6 5a9 5ae
5af 5b4 5b7 5bb 5bf 5c3 5c7 5ca
5cb 5cd 5d2 5d3 5d5 5d6 5db 5de
5e3 5e4 5e9 5ec 5f1 5f2 5f7 5fa
5fe 5ff 604 607 60c 60d 612 613
618 61a 61e 621 625 629 62c 630
634 637 63a 63f 642 645 648 649
64e 64f 1 651 656 65a 65e 661
666 66b 66e 672 673 678 679 67e
680 684 687 68b 68f 692 696 69a
69d 6a1 6a5 6a6 6ab 6ad 6b1 6b4
6b8 6bc 6c0 6c4 6c8 6cc 6d0 6d4
6d8 6dc 6e0 6e3 6e4 6e6 6e9 6ec
6ed 6f2 6f5 6f9 6fd 700 701 703
706 709 70a 70f 1 712 717 71b
71f 723 727 72b 72f 733 737 73b
73f 743 747 74b 74f 753 757 75b
75f 763 767 76b 76f 773 777 783
787 78b 78e 792 796 798 79c 7a0
7a4 7a7 7ab 7af 7b1 7b2 7b7 1
7bb 7bf 7c3 7c6 7ca 7ce 7d2 7d6
7d9 7dd 7e1 7e4 7e7 7ec 7ef 7f2
7f5 7f6 7fb 7fc 1 7fe 803 807
80b 80e 813 818 81b 81f 820 825
828 82d 82e 833 836 83b 83c 841
844 848 849 84e 851 856 857 85c
85f 863 867 86a 86d 86e 870 871
876 877 87c 87e 882 885 887 88b
88d 899 89b 89d 89e 8a3 8a7 8a9
8b5 8b7 8bb 8bf 8c2 8c6 8c9 8cd
8d0 8d1 8d6 8d9 8dd 8e1 8e5 8e9
8ed 8f1 8f5 8f9 8fd 901 905 909
90d 911 915 919 91d 921 925 929
92d 939 93d 941 944 948 94c 94e
952 956 95a 95d 961 965 967 968
96d 1 971 975 979 97c 980 984
988 98c 98f 993 997 99a 99d 9a2
9a5 9a8 9ab 9ac 9b1 9b2 1 9b4
9b9 9bd 9c1 9c4 9c9 9ce 9d1 9d5
9d6 9db 9de 9e3 9e4 9e9 9ec 9f1
9f2 9f7 9fa 9fe 9ff a04 a07 a0c
a0d a12 a15 a19 a1d a20 a23 a24
a26 a27 a2c a2d a32 a34 a38 a3b
a3d a41 a43 a4f a51 a53 a54 a59
a5d a5f a6b a6d a71 a75 a78 a7c
a7f a83 a86 a87 a8c a8f a93 a97
a9b a9f aab aaf ab3 ab6 aba abe
ac0 ac4 ac8 acc acf ad3 ad7 ad9
ada adf 1 ae3 ae7 aeb aee af2
af6 afa afe b01 b05 b09 b0c b0f
b14 b17 b1a b1d b1e b23 b24 1
b26 b2b b2f b33 b36 b3b b40 b43
b47 b48 b4d b50 b55 b56 b5b b5e
b63 b64 b69 b6c b70 b71 b76 b79
b7e b7f b84 b87 b8b b8f b92 b95
b96 b98 b99 b9e b9f ba4 ba6 baa
bad baf bb3 bb5 bc1 bc3 bc5 bc6
bcb bcf bd1 bdd bdf be3 be7 bea
bee bf1 bf5 bf8 bf9 bfe c01 c05
c09 c0c c10 c14 c17 c1a c1f c22
c25 c28 c29 c2e c2f 1 c31 c36
c3a c3e c41 c46 c4b c4e c52 c53
c58 c5b c60 c61 c66 c69 c6e c6f
c74 c77 c7b c7f c83 c87 c8a c8b
c8d c92 c93 c95 c96 c9b c9e ca3
ca4 ca9 cac cb1 cb2 cb7 cba cbe
cbf cc4 cc7 ccc ccd cd2 cd3 cd8
cda cde ce1 ce5 ce9 cec cf0 cf4
cf7 cfa cff d02 d05 d08 d09 d0e
d0f 1 d11 d16 d1a d1e d21 d26
d2b d2e d32 d33 d38 d39 d3e d40
d44 d47 d4b d4f d52 d56 d5a d5d
d61 d65 d66 d6b d6d d71 d74 d76
d7a d7d d7f d83 d86 d8a d8c d90
d94 d97 d98 d9a d9d da0 da1 da6
daa dae db2 db6 dba dc6 dca dce
dd1 dd5 dd9 ddb ddf de3 de7 dea
dee df2 df4 df5 dfa 1 dfe e02
e06 e09 e0d e11 e15 e19 e1c e20
e24 e27 e2a e2f e32 e35 e38 e39
e3e e3f 1 e41 e46 e4a e4e e51
e56 e5b e5e e62 e63 e68 e6b e70
e71 e76 e79 e7e e7f e84 e87 e8b
e8c e91 e94 e99 e9a e9f ea2 ea6
eaa ead eb0 eb1 eb3 eb4 eb9 eba
ebf ec1 ec5 ec8 eca ece ed0 edc
ede ee0 ee1 ee6 eea eec ef8 efa
efe f02 f05 f09 f0c f10 f13 f14
f19 f1c f20 f24 f27 f2b f2f f32
f35 f3a f3d f40 f43 f44 f49 f4a
1 f4c f51 f55 f59 f5c f61 f66
f69 f6d f6e f73 f76 f7b f7c f81
f84 f89 f8a f8f f92 f96 f9a f9e
fa2 fa5 fa6 fa8 fad fae fb0 fb1
fb6 fb9 fbe fbf fc4 fc7 fcc fcd
fd2 fd5 fd9 fda fdf fe2 fe7 fe8
fed fee ff3 ff5 ff9 ffc 1000 1004
1007 100b 100f 1012 1015 101a 101d 1020
1023 1024 1029 102a 1 102c 1031 1035
1039 103c 1041 1046 1049 104d 104e 1053
1054 1059 105b 105f 1062 1066 106a 106d
1071 1075 1078 107c 1080 1081 1086 1088
108c 108f 1093 1097 109b 109f 10a3 10a7
10ab 10af 10bb 10bf 10c3 10c6 10ca 10ce
10d0 10d4 10d8 10dc 10df 10e3 10e7 10e9
10ea 10ef 1 10f3 10f7 10fb 10fe 1102
1106 110a 110e 1111 1115 1119 111c 111f
1124 1127 112a 112d 112e 1133 1134 1
1136 113b 113f 1143 1146 114b 1150 1153
1157 1158 115d 1160 1165 1166 116b 116e
1173 1174 1179 117c 1180 1181 1186 1189
118e 118f 1194 1197 119b 119f 11a2 11a5
11a6 11a8 11a9 11ae 11af 11b4 11b6 11ba
11bd 11bf 11c3 11c5 11d1 11d3 11d5 11d6
11db 11df 11e1 11ed 11ef 11f3 11f5 11f9
11fd 1201 1204 1205 1207 120a 120d 120e
1213 1217 121b 121f 1223 1227 1233 1237
123b 123e 1242 1246 1248 124c 1250 1254
1257 125b 125f 1261 1262 1267 1 126b
126f 1273 1276 127a 127e 1282 1286 1289
128d 1291 1294 1297 129c 129f 12a2 12a5
12a6 12ab 12ac 1 12ae 12b3 12b7 12bb
12be 12c3 12c8 12cb 12cf 12d0 12d5 12d8
12dd 12de 12e3 12e6 12eb 12ec 12f1 12f4
12f8 12f9 12fe 1301 1306 1307 130c 130f
1313 1317 131a 131d 131e 1320 1321 1326
1327 132c 132e 1332 1335 1337 133b 133d
1349 134b 134d 134e 1353 1357 1359 1365
1367 136b 136f 1372 1376 1379 137d 1380
1381 1386 1389 138d 1391 1394 1398 139c
139f 13a2 13a7 13aa 13ad 13b0 13b1 13b6
13b7 1 13b9 13be 13c2 13c6 13c9 13ce
13d3 13d6 13da 13db 13e0 13e3 13e8 13e9
13ee 13f1 13f6 13f7 13fc 13ff 1403 1407
140b 140f 1412 1413 1415 141a 141b 141d
141e 1423 1426 142b 142c 1431 1434 1439
143a 143f 1442 1446 1447 144c 144f 1454
1455 145a 145b 1460 1462 1466 1469 146d
1471 1474 1478 147c 147f 1482 1487 148a
148d 1490 1491 1496 1497 1 1499 149e
14a2 14a6 14a9 14ae 14b3 14b6 14ba 14bb
14c0 14c1 14c6 14c8 14cc 14cf 14d3 14d7
14da 14de 14e2 14e5 14e9 14ed 14ee 14f3
14f5 14f9 14fc 1500 1503 1507 150b 150d
1511 1515 1519 151c 151d 151f 1523 1526
152a 152d 152e 1533 1536 153a 153e 1541
1542 1544 1547 154a 154b 1550 1 1553
1558 155c 1560 1563 1564 1566 156a 156d
1571 1574 1575 157a 1 157d 1582 1586
158a 158d 158e 1590 1594 1597 159b 159e
159f 15a4 1 15a7 15ac 15b0 15b4 15b8
15bc 15c0 15c4 15c8 15cc 15d8 15dc 15e0
15e3 15e7 15eb 15ed 15f1 15f5 15f9 15fc
1600 1604 1606 1607 160c 1 1610 1614
1618 161b 161f 1623 1627 162b 162e 1632
1636 1639 163c 1641 1644 1647 164a 164b
1650 1651 1 1653 1658 165c 1660 1663
1668 166d 1670 1674 1675 167a 167d 1682
1683 1688 168b 1690 1691 1696 1699 169d
169e 16a3 16a6 16ab 16ac 16b1 16b4 16b8
16bc 16bf 16c2 16c3 16c5 16c6 16cb 16cc
16d1 16d3 16d7 16da 16dc 16e0 16e2 16ee
16f0 16f2 16f3 16f8 16fc 16fe 170a 170c
1710 1714 1717 171b 171e 1722 1725 1726
172b 172e 1732 1736 1739 173d 1741 1744
1747 174c 174f 1752 1755 1756 175b 175c
1 175e 1763 1767 176b 176e 1773 1778
177b 177f 1780 1785 1788 178d 178e 1793
1796 179b 179c 17a1 17a4 17a8 17ac 17b0
17b4 17b7 17b8 17ba 17bf 17c0 17c2 17c3
17c8 17cb 17d0 17d1 17d6 17d9 17de 17df
17e4 17e7 17eb 17ec 17f1 17f4 17f9 17fa
17ff 1800 1805 1807 180b 180e 1812 1816
1819 181d 1821 1824 1827 182c 182f 1832
1835 1836 183b 183c 1 183e 1843 1847
184b 184e 1853 1858 185b 185f 1860 1865
1866 186b 186d 1871 1874 1878 187c 187f
1883 1887 188a 188e 1892 1893 1898 189a
189e 18a1 18a3 18a7 18ab 18ae 18b2 18b6
18b9 18ba 18bc 18bf 18c2 18c3 18c8 18cc
18d0 18d5 18da 18de 18e2 18e6 18e9 18ec
18f0 18f4 18f8 18fc 1900 1904 1908 190c
1910 1914 1918 191c 1920 1924 1928 192c
1930 1934 1938 193c 1940 1944 1948 194c
1950 1954 1958 195c 1960 1964 1968 196c
1970 1974 1978 1984 1988 198d 1991 1994
1995 199a 199e 19a2 19a5 19a8 19ac 19ae
19b2 19b6 19b9 19bc 19c0 19c2 19c6 19ca
19cd 19cf 19d3 19d7 19db 19de 19e2 19e6
19e8 19e9 19ee 1 19f2 19f6 19fa 19fd
1a01 1a05 1a09 1a0d 1a10 1a14 1a18 1a1b
1a1e 1a23 1a26 1a29 1a2c 1a2d 1a32 1a33
1 1a35 1a3a 1a3e 1a42 1a45 1a4a 1a4f
1a52 1a56 1a57 1a5c 1a5f 1a64 1a65 1a6a
1a6d 1a72 1a73 1a78 1a7b 1a7f 1a80 1a85
1a88 1a8d 1a8e 1a93 1a96 1a9a 1a9e 1aa1
1aa4 1aa5 1aa7 1aa8 1aad 1aae 1ab3 1ab5
1ab9 1abc 1abe 1ac2 1ac4 1ad0 1ad2 1ad4
1ad5 1ada 1ade 1ae0 1aec 1aee 1af2 1af6
1afb 1b00 1b04 1b08 1b0c 1b0f 1b12 1b16
1b1a 1b1e 1b22 1b26 1b2a 1b2e 1b32 1b3e
1b42 1b47 1b4b 1b4e 1b4f 1b54 1b58 1b5c
1b5f 1b62 1b66 1b68 1b6c 1b70 1b73 1b76
1b7a 1b7c 1b80 1b84 1b87 1b89 1b8d 1b91
1b95 1b98 1b9c 1ba0 1ba2 1ba3 1ba8 1
1bac 1bb0 1bb4 1bb7 1bbb 1bbf 1bc3 1bc7
1bca 1bce 1bd2 1bd5 1bd8 1bdd 1be0 1be3
1be6 1be7 1bec 1bed 1 1bef 1bf4 1bf8
1bfc 1bff 1c04 1c09 1c0c 1c10 1c11 1c16
1c19 1c1e 1c1f 1c24 1c27 1c2c 1c2d 1c32
1c35 1c39 1c3a 1c3f 1c42 1c47 1c48 1c4d
1c50 1c54 1c58 1c5b 1c5e 1c5f 1c61 1c62
1c67 1c68 1c6d 1c6f 1c73 1c76 1c78 1c7c
1c7e 1c8a 1c8c 1c8e 1c8f 1c94 1c98 1c9a
1ca6 1ca8 1cac 1cb0 1cb4 1cb8 1cbc 1cc0
1cc4 1cc8 1ccc 1cd0 1cd4 1cd8 1cdc 1ce0
1ce4 1ce8 1cec 1cf0 1cf4 1cf8 1cfc 1d00
1d04 1d08 1d0c 1d10 1d14 1d18 1d1c 1d20
1d24 1d28 1d2c 1d30 1d34 1d38 1d3c 1d40
1d44 1d48 1d4c 1d50 1d54 1d58 1d5c 1d60
1d64 1d68 1d6c 1d70 1d74 1d78 1d7c 1d80
1d84 1d88 1d8c 1d90 1d94 1d98 1d9c 1da0
1da4 1da8 1dac 1db0 1db4 1db8 1dbc 1dc0
1dc4 1dc8 1dcc 1dd0 1dd4 1dd8 1ddc 1de0
1de4 1de8 1dec 1df0 1df4 1df8 1dfc 1e00
1e04 1e08 1e0c 1e10 1e14 1e18 1e1c 1e20
1e24 1e28 1e2c 1e30 1e34 1e38 1e3c 1e40
1e44 1e48 1e4c 1e50 1e54 1e58 1e5c 1e60
1e64 1e68 1e6c 1e70 1e74 1e78 1e7c 1e80
1e84 1e88 1e8c 1e90 1e94 1e98 1e9c 1ea0
1ea4 1ea8 1eac 1eb0 1eb4 1eb8 1ebc 1ec0
1ec4 1ec8 1ecc 1ed0 1ed4 1ed8 1edc 1ee0
1ee4 1ee8 1eec 1ef0 1ef4 1ef8 1efc 1f00
1f04 1f08 1f0c 1f10 1f14 1f18 1f1c 1f20
1f24 1f28 1f2c 1f30 1f34 1f38 1f3c 1f40
1f44 1f48 1f4c 1f50 1f54 1f58 1f5c 1f60
1f64 1f68 1f6c 1f70 1f74 1f78 1f7c 1f80
1f84 1f88 1f8c 1f90 1f94 1f98 1f9c 1fa0
1fa4 1fa8 1fac 1fb0 1fb4 1fb8 1fbc 1fc0
1fc4 1fc8 1fcc 1fd0 1fd4 1fd8 1fdc 1fe0
1fe4 1fe8 1fec 1ff0 1ff4 1ff8 1ffc 2000
2004 2008 200c 2010 2014 2018 201c 2020
2024 2028 202c 2030 2034 2038 203c 2040
2044 2048 204c 2050 2054 2058 205c 2060
2064 2068 206c 2070 2074 2078 207c 2080
2084 2088 208c 2090 2094 2098 209c 20a0
20a4 20a8 20ac 20b0 20b4 20b8 20bc 20c0
20c4 20c8 20cc 20d0 20d4 20d8 20dc 20e0
20e4 20e8 20ec 20f0 20f4 20f8 20fc 2100
2104 2108 210c 2110 2114 2118 211c 2120
2124 2128 212c 2130 2134 2138 213c 2140
2144 2148 214c 2150 2154 2158 215c 2168
216d 2171 2175 2179 217c 2180 2184 2187
218a 218f 2192 2195 2198 2199 219e 219f
1 21a1 21a6 21aa 21ae 21b1 21b6 21bb
21be 21c2 21c3 21c8 21c9 21ce 21d0 21d4
21d7 21db 21df 21e3 21e7 21e9 21ed 21f1
21f4 21f5 21f7 21fa 21fd 21fe 2203 2206
220a 220e 2211 2212 2214 2217 221a 221b
2220 1 2223 2228 222c 2230 2235 223a
223e 2242 2246 2249 224c 2250 2254 2258
225c 2260 2264 2268 226c 2270 2274 2278
227c 2280 2284 2288 228c 2290 2294 2298
229c 22a0 22a4 22a8 22ac 22b0 22b4 22b8
22bc 22c0 22c4 22c8 22cc 22d0 22d4 22d8
22dc 22e0 22e4 22e8 22f4 22f8 22fd 2301
2304 2305 230a 230e 2312 2315 2318 231c
231e 2322 2326 2329 232c 2330 2332 2336
233a 233d 233f 2343 2347 234b 234e 2352
2356 2358 2359 235e 1 2362 2366 236a
236d 2371 2375 2379 237d 2380 2384 2388
238b 238e 2393 2396 2399 239c 239d 23a2
23a3 1 23a5 23aa 23ae 23b2 23b5 23ba
23bf 23c2 23c6 23c7 23cc 23cf 23d4 23d5
23da 23dd 23e2 23e3 23e8 23eb 23ef 23f0
23f5 23f8 23fd 23fe 2403 2406 240a 240e
2411 2414 2415 2417 2418 241d 241e 2423
2425 2429 242c 242e 2432 2434 2440 2442
2444 2445 244a 244e 2450 245c 245e 2462
2466 2469 246d 246e 2473 2477 247b 247f
2483 2487 248b 248f 2493 2497 249b 249f
24a3 24a7 24ab 24af 24b3 24b7 24bb 24bf
24c3 24c7 24cb 24cf 24d3 24d7 24db 24df
24e3 24e7 24eb 24ef 24f3 24f7 24fb 24ff
2503 2507 250b 250f 2513 2517 251b 251f
2523 2527 252b 252f 253b 253f 2544 2548
254b 254c 2551 2555 2559 255c 255f 2563
2565 2569 256d 2570 2573 2577 2579 257d
2581 2584 2586 258a 258e 2592 2595 2599
259d 259f 25a0 25a5 1 25a9 25ad 25b1
25b4 25b8 25bc 25c0 25c4 25c7 25cb 25cf
25d2 25d5 25da 25dd 25e0 25e3 25e4 25e9
25ea 1 25ec 25f1 25f5 25f9 25fc 2601
2606 2609 260d 260e 2613 2616 261b 261c
2621 2624 2629 262a 262f 2632 2636 2637
263c 263f 2644 2645 264a 264d 2651 2655
2658 265b 265c 265e 265f 2664 2665 266a
266c 2670 2673 2675 2679 267b 2687 2689
268b 268c 2691 2695 2697 26a3 26a5 26a9
26ad 26b1 26b5 26b9 26bd 26c1 26c5 26c9
26cd 26d1 26d5 26d9 26dd 26e1 26e5 26e9
26ed 26f1 26f5 26f9 26fd 2701 2705 2709
270d 2711 2715 2719 271d 2721 2725 2729
272d 2731 2735 2739 273d 2741 2745 2749
274d 2751 2755 2759 275d 2761 2765 2769
276d 2771 2775 2779 277d 2781 2785 2789
278d 2791 2795 2799 279d 27a1 27a5 27a9
27ad 27b1 27b5 27b9 27bd 27c1 27c5 27c9
27d5 27da 27de 27e2 27e6 27e9 27ed 27f1
27f4 27f7 27fc 27ff 2802 2805 2806 280b
280c 1 280e 2813 2817 281b 281e 2823
2828 282b 282f 2830 2835 2836 283b 283d
2841 2844 2848 284c 2850 2854 2856 285a
285e 2862 2865 2866 2868 286b 286e 286f
2874 2878 287c 2880 2884 2888 288c 2890
2894 2898 289c 28a0 28a4 28a8 28ac 28b0
28b4 28b8 28bc 28c0 28c4 28c8 28cc 28d0
28d4 28d8 28dc 28e0 28e4 28e8 28ec 28f0
28f4 28f8 28fc 2900 2904 2908 290c 2910
2914 2918 291c 2920 2924 2928 292c 2930
2934 2938 293c 2940 2944 2948 294c 2950
2954 2958 295c 2960 2964 2968 296c 2970
2974 2978 297c 2980 2984 2988 298c 2990
2994 2998 299c 29a0 29a4 29a8 29ac 29b0
29b4 29b8 29bc 29c0 29c4 29c8 29cc 29d0
29d4 29d8 29dc 29e0 29e4 29e8 29ec 29f0
29f4 29f8 29fc 2a00 2a04 2a08 2a0c 2a10
2a14 2a18 2a1c 2a20 2a24 2a28 2a2c 2a30
2a34 2a38 2a3c 2a40 2a44 2a48 2a4c 2a50
2a54 2a58 2a5c 2a60 2a64 2a68 2a6c 2a70
2a74 2a78 2a7c 2a80 2a84 2a88 2a8c 2a90
2a94 2a98 2a9c 2aa0 2aa4 2aa8 2ab4 2ab9
2abd 2ac1 2ac5 2ac8 2acc 2ad0 2ad3 2ad6
2adb 2ade 2ae1 2ae4 2ae5 2aea 2aeb 1
2aed 2af2 2af6 2afa 2afd 2b02 2b07 2b0a
2b0e 2b0f 2b14 2b15 2b1a 2b1c 2b20 2b23
2b27 2b2b 2b2f 2b33 2b35 2b39 2b3d 2b41
2b44 2b45 2b47 2b4a 2b4d 2b4e 2b53 2b57
2b5b 2b5f 2b62 2b63 2b65 2b66 2b68 2b6b
2b6e 2b6f 2b74 2b78 2b7b 2b7f 2b83 2b87
2b8b 2b8f 2b93 2b97 2b9b 2ba7 2bab 2baf
2bb2 2bb6 2bba 2bbc 2bc0 2bc4 2bc8 2bcb
2bcf 2bd3 2bd5 2bd6 2bdb 1 2bdf 2be3
2be7 2bea 2bee 2bf2 2bf6 2bfa 2bfd 2c01
2c05 2c08 2c0b 2c10 2c13 2c16 2c19 2c1a
2c1f 2c20 1 2c22 2c27 2c2b 2c2f 2c32
2c37 2c3c 2c3f 2c43 2c44 2c49 2c4c 2c51
2c52 2c57 2c5a 2c5f 2c60 2c65 2c68 2c6c
2c6d 2c72 2c75 2c7a 2c7b 2c80 2c83 2c87
2c8b 2c8e 2c91 2c92 2c94 2c95 2c9a 2c9b
2ca0 2ca2 2ca6 2ca9 2cab 2caf 2cb1 2cbd
2cbf 2cc1 2cc2 2cc7 2ccb 2ccd 2cd9 2cdb
2cdf 2ce3 2ce6 2ce7 2ce9 2cec 2cef 2cf0
2cf5 2cf9 2cfe 2d02 2d04 2d08 2d0b 2d0f
2d13 2d17 2d1b 2d1f 2d23 2d27 2d2b 2d2f
2d33 2d3f 2d43 2d47 2d4a 2d4e 2d52 2d54
2d58 2d5c 2d60 2d63 2d67 2d6b 2d6d 2d6e
2d73 1 2d77 2d7b 2d7f 2d82 2d86 2d8a
2d8e 2d92 2d95 2d99 2d9d 2da0 2da3 2da8
2dab 2dae 2db1 2db2 2db7 2db8 1 2dba
2dbf 2dc3 2dc7 2dca 2dcf 2dd4 2dd7 2ddb
2ddc 2de1 2de4 2de9 2dea 2def 2df2 2df7
2df8 2dfd 2e00 2e04 2e05 2e0a 2e0d 2e12
2e13 2e18 2e1b 2e1f 2e23 2e26 2e29 2e2a
2e2c 2e2d 2e32 2e33 2e38 2e3a 2e3e 2e41
2e43 2e47 2e49 2e55 2e57 2e59 2e5a 2e5f
2e63 2e65 2e71 2e73 2e77 2e7b 2e7e 2e7f
2e81 2e84 2e87 2e88 2e8d 2e91 2e95 2e99
2e9d 2ea1 2ea5 2ea9 2ead 2eb1 2eb5 2eb9
2ebd 2ec1 2ec5 2ec9 2ecd 2ed1 2ed5 2ed9
2edd 2ee1 2ee5 2ee9 2eed 2ef1 2ef5 2ef9
2efd 2f09 2f0d 2f11 2f14 2f18 2f1c 2f1e
2f22 2f26 2f2a 2f2d 2f31 2f35 2f37 2f38
2f3d 1 2f41 2f45 2f49 2f4c 2f50 2f54
2f58 2f5c 2f5f 2f63 2f67 2f6a 2f6d 2f72
2f75 2f78 2f7b 2f7c 2f81 2f82 1 2f84
2f89 2f8d 2f91 2f94 2f99 2f9e 2fa1 2fa5
2fa6 2fab 2fae 2fb3 2fb4 2fb9 2fbc 2fc1
2fc2 2fc7 2fca 2fce 2fcf 2fd4 2fd7 2fdc
2fdd 2fe2 2fe5 2fe9 2fed 2ff0 2ff3 2ff4
2ff6 2ff7 2ffc 2ffd 3002 3004 3008 300b
300d 3011 3013 301f 3021 3023 3024 3029
302d 302f 303b 303d 3041 3045 3048 3049
304b 304e 3051 3052 3057 305b 3060 3064
3066 306a 306f 3073 3075 3079 307d 3080
3082 3086 308b 308f 3091 3095 3099 309c
309e 30a2 30a6 30a9 30ad 30b1 30b5 30b9
30bd 30c1 30c5 30c9 30cd 30d1 30d5 30d9
30dd 30e1 30e5 30e9 30ed 30f1 30f5 30f9
30fd 3101 3105 3109 310d 3111 3115 3119
311d 3121 3125 3129 312d 3131 3135 3139
313d 3141 3145 3149 314d 3151 3155 3159
315d 3161 3165 3169 316d 3171 3175 3179
317d 3181 3185 3189 318d 3191 3195 3199
319d 31a1 31a5 31a9 31ad 31b1 31b5 31b9
31bd 31c1 31c5 31c9 31cd 31d1 31d5 31d9
31dd 31e1 31e5 31e9 31ed 31f1 31f5 31f9
31fd 3201 3205 3209 320d 3211 3215 3219
321d 3221 3225 3229 322d 3231 3235 3239
323d 3241 3245 3249 324d 3251 3255 3259
325d 3261 3265 3269 326d 3271 3275 3279
327d 3281 3285 3289 328d 3291 3295 3299
329d 32a1 32a5 32a9 32ad 32b1 32b5 32b9
32bd 32c1 32c5 32c9 32cd 32d1 32d5 32d9
32dd 32e1 32e5 32e9 32ed 32f1 32f5 32f9
32fd 3301 3305 3309 330d 3311 3315 3319
331d 3321 3325 3329 332d 3331 3335 3339
333d 3341 3345 3349 334d 3351 3355 3359
335d 3361 3365 3369 336d 3371 3375 3379
337d 3381 3385 3389 338d 3391 3395 3399
339d 33a1 33a5 33a9 33ad 33b1 33b5 33b9
33bd 33c1 33c5 33c9 33cd 33d1 33d5 33d9
33dd 33e1 33e5 33e9 33ed 33f1 33f5 33f9
33fd 3401 3405 3409 340d 3411 3415 3419
341d 3421 3425 3429 342d 3431 3435 3439
343d 3441 3445 3449 344d 3451 3455 3459
345d 3461 3465 3469 346d 3471 3475 3479
347d 3481 3485 3489 348d 3491 3495 3499
349d 34a1 34a5 34a9 34ad 34b1 34b5 34b9
34bd 34c1 34c5 34c9 34cd 34d1 34d5 34d9
34dd 34e1 34e5 34e9 34ed 34f1 34f5 34f9
34fd 3501 3505 3509 350d 3511 3515 3519
351d 3521 3525 3529 352d 3531 3535 3539
353d 3541 3545 3549 354d 3551 3555 3559
355d 3561 3565 3569 356d 3571 3575 3579
357d 3581 3585 3589 358d 3591 3595 3599
359d 35a1 35a5 35a9 35ad 35b1 35b5 35b9
35bd 35c1 35c5 35c9 35cd 35d1 35d5 35d9
35dd 35e1 35e5 35e9 35ed 35f1 35f5 35f9
35fd 3601 3605 3609 360d 3611 3615 3619
361d 3621 3625 3629 362d 3631 3635 3639
363d 3641 3645 3649 364d 3651 3655 3659
365d 3661 3665 3669 366d 3671 3675 3679
367d 3681 3685 3689 368d 3691 3695 3699
369d 36a1 36a5 36a9 36ad 36b1 36b5 36b9
36bd 36c1 36c5 36c9 36cd 36d1 36d5 36d9
36dd 36e1 36e5 36e9 36ed 36f1 36f5 36f9
36fd 3701 3705 3709 370d 3711 3715 3719
371d 3721 3725 3729 372d 3731 3735 3739
373d 3741 3745 3749 374d 3751 3755 3759
375d 3761 3765 3769 376d 3771 3775 3779
377d 3781 3785 3789 378d 3791 3795 3799
379d 37a1 37a5 37a9 37ad 37b1 37b5 37b9
37bd 37c1 37c5 37c9 37cd 37d1 37d5 37d9
37dd 37e1 37e5 37e9 37ed 37f1 37f5 37f9
37fd 3801 3805 3809 380d 3811 3815 3819
381d 3821 3825 3829 382d 3831 3835 3839
383d 3841 3845 3849 384d 3851 3855 3859
385d 3861 3865 3869 386d 3871 3875 3879
387d 3881 3885 3889 388d 3891 3895 3899
389d 38a1 38a5 38a9 38ad 38b1 38b5 38b9
38bd 38c1 38c5 38c9 38cd 38d1 38dd 38e2
38e6 38ea 38ee 38f1 38f5 38f9 38fc 38ff
3904 3907 390a 390d 390e 3913 3914 1
3916 391b 391f 3923 3926 392b 3930 3933
3937 3938 393d 393e 3943 3945 3949 394c
3950 3954 3958 395a 395e 3962 3966 396a
396e 3972 3976 397a 397e 3982 3986 398a
398e 3992 3996 399a 399e 39a2 39a6 39aa
39ae 39b2 39b6 39ba 39be 39c2 39c6 39ca
39ce 39d2 39d6 39da 39de 39e2 39e6 39ea
39ee 39f2 39f6 39fa 39fe 3a02 3a06 3a0a
3a0e 3a12 3a16 3a1a 3a1e 3a22 3a26 3a2a
3a2e 3a32 3a36 3a3a 3a3e 3a42 3a46 3a4a
3a4e 3a52 3a56 3a5a 3a5e 3a62 3a66 3a6a
3a6e 3a72 3a76 3a7a 3a7e 3a82 3a86 3a8a
3a8e 3a92 3a96 3a9a 3a9e 3aa2 3aa6 3aaa
3aae 3ab2 3ab6 3aba 3abe 3ac2 3ac6 3aca
3ace 3ad2 3ad6 3ada 3ade 3ae2 3ae6 3aea
3aee 3af2 3af6 3afa 3afe 3b02 3b06 3b0a
3b0e 3b12 3b16 3b1a 3b1e 3b22 3b26 3b2a
3b2e 3b32 3b36 3b3a 3b3e 3b42 3b46 3b4a
3b4e 3b52 3b56 3b5a 3b5e 3b62 3b66 3b6a
3b6e 3b72 3b76 3b7a 3b7e 3b82 3b86 3b8a
3b8e 3b92 3b96 3b9a 3b9e 3ba2 3ba6 3baa
3bae 3bb2 3bb6 3bba 3bbe 3bc2 3bc6 3bca
3bce 3bd2 3bd6 3bda 3bde 3be2 3be6 3bea
3bee 3bf2 3bf6 3bfa 3bfe 3c02 3c06 3c0a
3c0e 3c12 3c16 3c1a 3c1e 3c22 3c26 3c2a
3c2e 3c32 3c36 3c3a 3c3e 3c42 3c46 3c4a
3c4e 3c52 3c56 3c5a 3c5e 3c62 3c66 3c6a
3c6e 3c72 3c76 3c7a 3c7e 3c82 3c86 3c8a
3c8e 3c92 3c96 3c9a 3c9e 3ca2 3ca6 3caa
3cae 3cb2 3cb6 3cba 3cbe 3cc2 3cc6 3cca
3cce 3cd2 3cd6 3cda 3cde 3ce2 3ce6 3cea
3cee 3cf2 3cf6 3cfa 3cfe 3d02 3d06 3d0a
3d0e 3d12 3d16 3d1a 3d1e 3d22 3d26 3d2a
3d2e 3d32 3d36 3d3a 3d3e 3d42 3d46 3d4a
3d4e 3d52 3d56 3d5a 3d5e 3d62 3d66 3d6a
3d6e 3d72 3d76 3d7a 3d7e 3d82 3d86 3d8a
3d8e 3d92 3d96 3d9a 3d9e 3da2 3da6 3daa
3dae 3db2 3db6 3dba 3dbe 3dc2 3dc6 3dca
3dce 3dd2 3dd6 3dda 3dde 3de2 3de6 3dea
3dee 3df2 3df6 3dfa 3dfe 3e02 3e06 3e0a
3e0e 3e12 3e16 3e1a 3e1e 3e22 3e26 3e2a
3e2e 3e32 3e36 3e3a 3e3e 3e42 3e46 3e4a
3e4e 3e52 3e56 3e5a 3e5e 3e62 3e66 3e6a
3e6e 3e72 3e76 3e82 3e87 3e8b 3e8f 3e93
3e96 3e9a 3e9e 3ea1 3ea4 3ea9 3eac 3eaf
3eb2 3eb3 3eb8 3eb9 1 3ebb 3ec0 3ec4
3ec8 3ecb 3ed0 3ed5 3ed8 3edc 3edd 3ee2
3ee3 3ee8 3eea 3eee 3ef1 3ef5 3ef9 3efd
3eff 3f03 3f07 3f0a 3f0e 3f10 3f14 3f18
3f1c 3f1f 3f20 3f22 3f25 3f28 3f29 3f2e
3f32 3f36 3f3b 3f40 3f44 3f48 3f4c 3f4f
3f52 3f56 3f5a 3f5e 3f62 3f66 3f6a 3f6e
3f72 3f76 3f7a 3f7e 3f82 3f86 3f8a 3f8e
3f92 3f96 3f9a 3f9e 3fa2 3fa6 3faa 3fae
3fb2 3fb6 3fba 3fbe 3fc2 3fc6 3fca 3fce
3fd2 3fd6 3fda 3fde 3fe2 3fe6 3fea 3fee
3ff2 3ffe 4002 4007 400b 400e 400f 4014
4018 401c 401f 4022 4026 4028 402c 4030
4033 4036 403a 403c 4040 4044 4047 4049
404d 4051 4055 4058 405c 4060 4062 4063
4068 1 406c 4070 4074 4077 407b 407f
4083 4087 408a 408e 4092 4095 4098 409d
40a0 40a3 40a6 40a7 40ac 40ad 1 40af
40b4 40b8 40bc 40bf 40c4 40c9 40cc 40d0
40d1 40d6 40d9 40de 40df 40e4 40e7 40ec
40ed 40f2 40f5 40f9 40fa 40ff 4102 4107
4108 410d 4110 4114 4118 411b 411e 411f
4121 4122 4127 4128 412d 412f 4133 4136
4138 413c 413e 414a 414c 414e 414f 4154
4158 415a 4166 4168 416c 4170 4173 4177
4178 417d 4181 4185 4189 418d 4191 4195
4199 419d 41a1 41a5 41a9 41ad 41b1 41b5
41b9 41bd 41c1 41c5 41c9 41cd 41d1 41d5
41d9 41dd 41e1 41e5 41e9 41ed 41f1 41f5
41f9 41fd 4201 4205 4209 420d 4211 4215
4219 421d 4221 4225 4229 422d 4231 4235
4239 423d 4241 4245 4249 424d 4251 4255
4261 4265 426a 426e 4271 4272 4277 427b
427f 4282 4285 4289 428b 428f 4293 4296
4299 429d 429f 42a3 42a7 42aa 42ac 42b0
42b4 42b8 42bb 42bf 42c3 42c5 42c6 42cb
1 42cf 42d3 42d7 42da 42de 42e2 42e6
42ea 42ed 42f1 42f5 42f8 42fb 4300 4303
4306 4309 430a 430f 4310 1 4312 4317
431b 431f 4322 4327 432c 432f 4333 4334
4339 433c 4341 4342 4347 434a 434f 4350
4355 4358 435c 435d 4362 4365 436a 436b
4370 4373 4377 437b 437e 4381 4382 4384
4385 438a 438b 4390 4392 4396 4399 439b
439f 43a1 43ad 43af 43b1 43b2 43b7 43bb
43bd 43c9 43cb 43cf 43d3 43d7 43db 43df
43e3 43e7 43eb 43ef 43f3 43f7 43fb 43ff
4403 4407 440b 440f 4413 4417 441b 441f
4423 4427 442b 442f 4433 4437 443b 443f
4443 4447 444b 444f 4453 4457 445b 445f
4463 4467 446b 446f 4473 4477 447b 447f
4483 4487 448b 448f 4493 4497 449b 449f
44a3 44a7 44ab 44af 44b3 44b7 44bb 44bf
44c3 44c7 44cb 44cf 44d3 44d7 44db 44df
44e3 44e7 44eb 44ef 44f3 44f7 44fb 4507
450c 4510 4514 4518 451b 451f 4523 4526
4529 452e 4531 4534 4537 4538 453d 453e
1 4540 4545 4549 454d 4550 4555 455a
455d 4561 4562 4567 4568 456d 456f 4573
4576 457a 457e 4582 4586 4588 458c 4590
4594 4597 4598 459a 459d 45a0 45a1 45a6
45aa 45ae 45b3 45b8 45bc 45c0 45c4 45c7
45ca 45ce 45d2 45d6 45da 45de 45e2 45e6
45ea 45ee 45f2 45f6 45fa 45fe 4602 4606
460a 460e 4612 4616 461a 461e 4622 4626
462a 462e 4632 4636 463a 463e 4642 4646
464a 464e 4652 4656 465a 465e 4662 4666
466a 4676 467a 467f 4683 4686 4687 468c
4690 4694 4697 469a 469e 46a0 46a4 46a8
46ab 46ae 46b2 46b4 46b8 46bc 46bf 46c1
46c5 46c9 46cd 46d0 46d4 46d8 46da 46db
46e0 1 46e4 46e8 46ec 46ef 46f3 46f7
46fb 46ff 4702 4706 470a 470d 4710 4715
4718 471b 471e 471f 4724 4725 1 4727
472c 4730 4734 4737 473c 4741 4744 4748
4749 474e 4751 4756 4757 475c 475f 4764
4765 476a 476d 4771 4772 4777 477a 477f
4780 4785 4788 478c 4790 4793 4796 4797
4799 479a 479f 47a0 47a5 47a7 47ab 47ae
47b0 47b4 47b6 47c2 47c4 47c6 47c7 47cc
47d0 47d2 47de 47e0 47e4 47e8 47eb 47ef
47f0 47f5 47f9 47fd 4801 4805 4809 480d
4811 4815 4819 481d 4821 4825 4829 482d
4831 4835 4839 483d 4841 4845 4849 484d
4851 4855 4859 485d 4861 4865 4869 486d
4871 4875 4879 487d 4881 4885 4889 488d
4891 4895 4899 489d 48a1 48a5 48a9 48ad
48b1 48b5 48b9 48bd 48c1 48c5 48c9 48cd
48d1 48d5 48d9 48dd 48e1 48e5 48e9 48ed
48f1 48f5 48f9 48fd 4901 4905 4909 490d
4911 4915 4919 491d 4921 4925 4929 492d
4931 4935 4939 493d 4941 4945 4949 494d
4951 4955 4959 495d 4961 4965 4969 496d
4971 4975 4979 497d 4981 498d 4991 4996
499a 499d 499e 49a3 49a7 49ab 49ae 49b1
49b5 49b7 49bb 49bf 49c2 49c5 49c9 49cb
49cf 49d3 49d6 49d8 49dc 49e0 49e4 49e7
49eb 49ef 49f1 49f2 49f7 1 49fb 49ff
4a03 4a06 4a0a 4a0e 4a12 4a16 4a19 4a1d
4a21 4a24 4a27 4a2c 4a2f 4a32 4a35 4a36
4a3b 4a3c 1 4a3e 4a43 4a47 4a4b 4a4e
4a53 4a58 4a5b 4a5f 4a60 4a65 4a68 4a6d
4a6e 4a73 4a76 4a7b 4a7c 4a81 4a84 4a88
4a89 4a8e 4a91 4a96 4a97 4a9c 4a9f 4aa3
4aa7 4aaa 4aad 4aae 4ab0 4ab1 4ab6 4ab7
4abc 4abe 4ac2 4ac5 4ac7 4acb 4acd 4ad9
4adb 4add 4ade 4ae3 4ae7 4ae9 4af5 4af7
4afb 4aff 4b03 4b07 4b0b 4b0f 4b13 4b17
4b1b 4b1f 4b23 4b27 4b2b 4b2f 4b33 4b37
4b3b 4b3f 4b43 4b47 4b4b 4b4f 4b53 4b57
4b5b 4b5f 4b63 4b67 4b6b 4b6f 4b73 4b77
4b7b 4b7f 4b83 4b87 4b8b 4b8f 4b93 4b97
4b9b 4b9f 4ba3 4ba7 4bab 4baf 4bb3 4bb7
4bbb 4bbf 4bc3 4bc7 4bcb 4bcf 4bd3 4bd7
4bdb 4bdf 4be3 4be7 4beb 4bef 4bf3 4bf7
4bfb 4bff 4c03 4c07 4c0b 4c0f 4c13 4c17
4c1b 4c1f 4c23 4c27 4c2b 4c2f 4c33 4c37
4c3b 4c3f 4c43 4c47 4c4b 4c4f 4c53 4c57
4c5b 4c5f 4c63 4c67 4c6b 4c6f 4c73 4c77
4c7b 4c7f 4c83 4c87 4c8b 4c8f 4c93 4c97
4c9b 4c9f 4cab 4cb0 4cb4 4cb8 4cbc 4cbf
4cc3 4cc7 4cca 4ccd 4cd2 4cd5 4cd8 4cdb
4cdc 4ce1 4ce2 1 4ce4 4ce9 4ced 4cf1
4cf4 4cf9 4cfe 4d01 4d05 4d06 4d0b 4d0c
4d11 4d13 4d17 4d1a 4d1e 4d22 4d26 4d2a
4d2c 4d30 4d34 4d38 4d3b 4d3c 4d3e 4d41
4d44 4d45 4d4a 4d4e 4d52 4d56 4d5a 4d5e
4d62 4d66 4d6a 4d6e 4d72 4d76 4d7a 4d7e
4d82 4d86 4d8a 4d8e 4d92 4d96 4d9a 4d9e
4da2 4da6 4daa 4dae 4db2 4db6 4dba 4dbe
4dc2 4dc6 4dca 4dce 4dd2 4dd6 4dda 4dde
4de2 4de6 4dea 4dee 4df2 4df6 4dfa 4dfe
4e02 4e06 4e0a 4e0e 4e12 4e16 4e1a 4e1e
4e22 4e26 4e2a 4e2e 4e32 4e36 4e3a 4e3e
4e42 4e46 4e4a 4e4e 4e52 4e56 4e5a 4e5e
4e62 4e66 4e6a 4e6e 4e72 4e76 4e7a 4e7e
4e82 4e86 4e8a 4e8e 4e92 4e96 4e9a 4e9e
4ea2 4ea6 4eaa 4eae 4eb2 4eb6 4eba 4ebe
4ec2 4ec6 4eca 4ece 4ed2 4ed6 4eda 4ede
4ee2 4ee6 4eea 4eee 4ef2 4ef6 4efa 4efe
4f02 4f06 4f0a 4f0e 4f12 4f16 4f1a 4f1e
4f22 4f26 4f2a 4f2e 4f32 4f36 4f3a 4f3e
4f42 4f46 4f4a 4f4e 4f52 4f56 4f5a 4f5e
4f62 4f66 4f6a 4f6e 4f72 4f76 4f7a 4f7e
4f82 4f86 4f8a 4f8e 4f92 4f96 4f9a 4f9e
4fa2 4fa6 4faa 4fae 4fb2 4fb6 4fba 4fbe
4fc2 4fc6 4fca 4fce 4fd2 4fd6 4fda 4fde
4fe2 4fe6 4fea 4fee 4ff2 4ff6 4ffa 4ffe
5002 5006 500a 500e 5012 5016 501a 501e
5022 5026 502a 502e 5032 5036 503a 503e
5042 5046 504a 504e 5052 5056 505a 505e
5062 5066 506a 506e 5072 5076 507a 507e
5082 5086 508a 508e 5092 5096 509a 509e
50a2 50a6 50aa 50ae 50b2 50b6 50ba 50be
50c2 50c6 50ca 50ce 50d2 50d6 50da 50e6
50eb 50ef 50f3 50f7 50fa 50fe 5102 5105
5108 510d 5110 5113 5116 5117 511c 511d
1 511f 5124 5128 512c 512f 5134 5139
513c 5140 5141 5146 5147 514c 514e 5152
5155 5159 515d 5161 5165 5167 516b 516f
5173 5176 5177 5179 517c 517f 5180 5185
5189 518d 5192 5197 519b 519f 51a3 51a6
51a9 51ad 51b1 51b5 51b9 51bd 51c1 51c5
51c9 51cd 51d1 51d5 51d9 51dd 51e1 51e5
51e9 51ed 51f1 51f5 51f9 51fd 5201 5205
5209 520d 5211 5215 5219 521d 5221 5225
5229 522d 5231 5235 5239 523d 5241 5245
5249 524d 5251 5255 5259 525d 5261 5265
5269 5275 5279 527e 5282 5285 5286 528b
528f 5293 5296 5299 529d 529f 52a3 52a7
52aa 52ad 52b1 52b3 52b7 52bb 52be 52c0
52c4 52c8 52cc 52cf 52d3 52d7 52d9 52da
52df 1 52e3 52e7 52eb 52ee 52f2 52f6
52fa 52fe 5301 5305 5309 530c 530f 5314
5317 531a 531d 531e 5323 5324 1 5326
532b 532f 5333 5336 533b 5340 5343 5347
5348 534d 5350 5355 5356 535b 535e 5363
5364 5369 536c 5370 5371 5376 5379 537e
537f 5384 5387 538b 538f 5392 5395 5396
5398 5399 539e 539f 53a4 53a6 53aa 53ad
53af 53b3 53b5 53c1 53c3 53c5 53c6 53cb
53cf 53d1 53dd 53df 53e3 53e7 53ec 53f1
53f5 53f9 53fd 5400 5403 5407 540b 540f
5413 5417 541b 541f 5423 5427 542b 542f
5433 5437 543b 543f 5443 5447 544b 544f
5453 5457 545b 545f 5463 5467 546b 546f
5473 5477 547b 547f 5483 5487 548b 548f
5493 5497 549b 549f 54a3 54a7 54ab 54af
54b3 54b7 54bb 54bf 54c3 54c7 54cb 54cf
54d3 54d7 54db 54df 54e3 54e7 54eb 54ef
54f3 54f7 54fb 54ff 5503 5507 550b 550f
5513 5517 551b 551f 5523 5527 552b 552f
5533 5537 553b 553f 5543 5547 554b 554f
5553 5557 555b 555f 5563 5567 556b 556f
5573 5577 557b 557f 5583 5587 558b 558f
5593 5597 559b 559f 55a3 55a7 55ab 55af
55b3 55b7 55bb 55bf 55c3 55c7 55cb 55d7
55db 55e0 55e4 55e7 55e8 55ed 55f1 55f5
55f8 55fb 55ff 5601 5605 5609 560c 560f
5613 5615 5619 561d 5620 5622 5626 562a
562e 5631 5635 5639 563b 563c 5641 1
5645 5649 564d 5650 5654 5658 565c 5660
5663 5667 566b 566e 5671 5676 5679 567c
567f 5680 5685 5686 1 5688 568d 5691
5695 5698 569d 56a2 56a5 56a9 56aa 56af
56b2 56b7 56b8 56bd 56c0 56c5 56c6 56cb
56ce 56d2 56d3 56d8 56db 56e0 56e1 56e6
56e9 56ed 56f1 56f4 56f7 56f8 56fa 56fb
5700 5701 5706 5708 570c 570f 5711 5715
5717 5723 5725 5727 5728 572d 5731 5733
573f 5741 5745 5749 574d 5751 5755 5759
575d 5761 5765 5769 576d 5771 5775 5779
577d 5781 578d 5791 5796 579a 579d 579e
57a3 57a7 57ab 57ae 57b1 57b5 57b7 57bb
57bf 57c2 57c5 57c9 57cb 57cf 57d3 57d6
57d8 57dc 57e0 57e4 57e7 57eb 57ef 57f1
57f2 57f7 1 57fb 57ff 5803 5806 580a
580e 5812 5816 5819 581d 5821 5824 5827
582c 582f 5832 5835 5836 583b 583c 1
583e 5843 5847 584b 584e 5853 5858 585b
585f 5860 5865 5868 586d 586e 5873 5876
587b 587c 5881 5884 5888 5889 588e 5891
5896 5897 589c 589f 58a3 58a7 58aa 58ad
58ae 58b0 58b1 58b6 58b7 58bc 58be 58c2
58c5 58c7 58cb 58cd 58d9 58db 58dd 58de
58e3 58e7 58e9 58f5 58f7 58fb 58ff 5903
5907 590b 590f 5913 5917 591b 591f 5923
5927 592b 592f 5933 5937 593b 593f 5943
5947 594b 594f 5953 5957 595b 595f 5963
5967 596b 596f 5973 5977 597b 597f 5983
5987 598b 598f 5993 5997 599b 599f 59a3
59a7 59ab 59af 59b3 59b7 59bb 59bf 59c3
59c7 59cb 59cf 59d3 59d7 59db 59df 59e3
59e7 59eb 59ef 59f3 59f7 59fb 59ff 5a03
5a07 5a0b 5a0f 5a13 5a17 5a1b 5a1f 5a23
5a27 5a2b 5a2f 5a33 5a37 5a3b 5a3f 5a43
5a47 5a4b 5a4f 5a53 5a57 5a5b 5a5f 5a63
5a67 5a6b 5a6f 5a73 5a77 5a7b 5a7f 5a83
5a87 5a8b 5a8f 5a93 5a97 5a9b 5a9f 5aa3
5aa7 5aab 5aaf 5ab3 5ab7 5abb 5abf 5ac3
5ac7 5acb 5acf 5ad3 5ad7 5adb 5adf 5ae3
5ae7 5aeb 5aef 5af3 5af7 5afb 5aff 5b03
5b07 5b0b 5b0f 5b13 5b17 5b1b 5b1f 5b23
5b27 5b2b 5b2f 5b33 5b37 5b3b 5b3f 5b43
5b47 5b4b 5b4f 5b53 5b57 5b5b 5b5f 5b63
5b67 5b6b 5b6f 5b73 5b77 5b7b 5b7f 5b83
5b87 5b8b 5b8f 5b93 5b97 5b9b 5b9f 5ba3
5ba7 5bab 5baf 5bb3 5bb7 5bbb 5bbf 5bc3
5bc7 5bcb 5bcf 5bd3 5bd7 5bdb 5bdf 5be3
5be7 5beb 5bef 5bf3 5bf7 5bfb 5bff 5c03
5c07 5c0b 5c0f 5c13 5c17 5c1b 5c1f 5c23
5c27 5c2b 5c2f 5c33 5c37 5c3b 5c3f 5c43
5c47 5c4b 5c57 5c5c 5c60 5c64 5c68 5c6b
5c6f 5c73 5c76 5c79 5c7e 5c81 5c84 5c87
5c88 5c8d 5c8e 1 5c90 5c95 5c99 5c9d
5ca0 5ca5 5caa 5cad 5cb1 5cb2 5cb7 5cb8
5cbd 5cbf 5cc3 5cc6 5cca 5cce 5cd2 5cd6
5cd8 5cdc 5ce0 5ce4 5ce7 5ce8 5cea 5cee
5cf1 5cf5 5cf8 5cf9 5cfe 5d02 5d06 5d0a
5d0e 5d12 5d1e 5d22 5d26 5d29 5d2d 5d31
5d33 5d37 5d3b 5d3f 5d42 5d46 5d4a 5d4c
5d4d 5d52 1 5d56 5d5a 5d5e 5d61 5d65
5d69 5d6d 5d71 5d74 5d78 5d7c 5d7f 5d82
5d87 5d8a 5d8d 5d90 5d91 5d96 5d97 1
5d99 5d9e 5da2 5da6 5da9 5dae 5db3 5db6
5dba 5dbb 5dc0 5dc3 5dc8 5dc9 5dce 5dd1
5dd6 5dd7 5ddc 5ddf 5de3 5de4 5de9 5dec
5df1 5df2 5df7 5dfa 5dfe 5e02 5e05 5e08
5e09 5e0b 5e0c 5e11 5e12 5e17 5e19 5e1d
5e20 5e22 5e26 5e28 5e34 5e36 5e38 5e39
5e3e 5e42 5e44 5e50 5e52 5e56 5e5a 5e5d
5e61 5e64 5e68 5e6b 5e6c 5e71 5e74 5e78
5e7c 5e80 5e84 5e88 5e8c 5e90 5e94 5e98
5e9c 5ea0 5ea4 5ea8 5eac 5eb0 5eb4 5eb8
5ebc 5ec0 5ec4 5ec8 5ecc 5ed0 5ed4 5ed8
5edc 5ee0 5ee4 5ee8 5eec 5ef0 5ef4 5ef8
5efc 5f00 5f04 5f08 5f0c 5f10 5f14 5f18
5f1c 5f20 5f24 5f28 5f2c 5f30 5f34 5f38
5f3c 5f40 5f44 5f48 5f4c 5f50 5f54 5f58
5f5c 5f60 5f64 5f68 5f6c 5f70 5f74 5f78
5f7c 5f80 5f84 5f88 5f8c 5f90 5f94 5f98
5f9c 5fa0 5fa4 5fa8 5fac 5fb0 5fb4 5fb8
5fbc 5fc0 5fc4 5fc8 5fcc 5fd0 5fd4 5fd8
5fdc 5fe0 5fe4 5fe8 5fec 5ff0 5ff4 5ff8
5ffc 6000 6004 6008 600c 6010 6014 6018
601c 6020 6024 6028 602c 6030 6034 6038
603c 6040 6044 6048 604c 6050 6054 6058
605c 6060 6064 6068 606c 6070 6074 6078
607c 6080 6084 6088 608c 6090 6094 6098
609c 60a0 60a4 60a8 60ac 60b0 60b4 60b8
60bc 60c0 60c4 60c8 60cc 60d0 60d4 60d8
60dc 60e8 60ed 60f1 60f5 60f9 60fc 6100
6104 6107 610a 610f 6112 6115 6118 6119
611e 611f 1 6121 6126 612a 612e 6131
6136 613b 613e 6142 6143 6148 6149 614e
6150 6154 6157 615b 615f 6163 6165 6169
616c 6170 6172 6176 617a 617e 6181 6182
6184 6187 618a 618b 6190 6194 6198 619c
61a0 61a4 61a8 61ac 61b0 61b4 61b8 61bc
61c0 61c4 61c8 61cc 61d0 61d4 61d8 61dc
61e0 61e4 61e8 61ec 61f0 61f4 61f8 61fc
6200 6204 6208 620c 6210 6214 6218 621c
6220 6224 6228 622c 6230 6234 6238 623c
6240 6244 6248 624c 6250 6254 6258 625c
6260 6264 6268 626c 6270 6274 6278 627c
6280 6284 6288 628c 6290 6294 6298 629c
62a0 62a4 62a8 62ac 62b0 62b4 62b8 62bc
62c0 62c4 62c8 62cc 62d0 62d4 62d8 62dc
62e0 62e4 62e8 62ec 62f0 62f4 62f8 62fc
6300 6304 6308 630c 6310 6314 6318 631c
6320 6324 6328 632c 6330 6334 6338 633c
6340 6344 6348 634c 6350 6354 6358 635c
6360 6364 6368 636c 6370 6374 6378 637c
6380 6384 6388 638c 6390 6394 6398 639c
63a0 63a4 63a8 63ac 63b0 63b4 63b8 63bc
63c0 63c4 63c8 63cc 63d0 63d4 63d8 63dc
63e0 63e4 63e8 63ec 63f0 63f4 63f8 63fc
6400 6404 6408 640c 6410 6414 6418 641c
6420 6424 6428 642c 6430 6434 6438 643c
6440 6444 6448 644c 6450 6454 6458 645c
6460 6464 6468 646c 6470 6474 6478 647c
6480 6484 6488 648c 6490 6494 6498 649c
64a0 64a4 64a8 64ac 64b0 64b4 64b8 64bc
64c0 64c4 64c8 64cc 64d0 64d4 64d8 64dc
64e0 64e4 64e8 64ec 64f0 64f4 64f8 64fc
6500 6504 6508 650c 6510 6514 6518 651c
6520 6524 6528 652c 6530 6534 6538 653c
6540 6544 6548 654c 6550 6554 6558 655c
6560 6564 6568 656c 6570 6574 6578 657c
6580 6584 6588 658c 6590 6594 6598 659c
65a0 65a4 65a8 65ac 65b0 65b4 65b8 65bc
65c0 65c4 65c8 65cc 65d0 65d4 65d8 65dc
65e0 65e4 65e8 65ec 65f0 65f4 65f8 65fc
6600 6604 6608 660c 6610 6614 6618 661c
6620 6624 6628 662c 6630 6634 6638 663c
6640 6644 6648 664c 6650 6654 6658 665c
6660 6664 6668 666c 6670 6674 6678 667c
6680 6684 6688 668c 6690 6694 6698 669c
66a0 66a4 66b0 66b5 66b9 66bd 66c1 66c4
66c8 66cc 66cf 66d2 66d7 66da 66dd 66e0
66e1 66e6 66e7 1 66e9 66ee 66f2 66f6
66f9 66fe 6703 6706 670a 670b 6710 6711
6716 6718 671c 671f 6723 6727 672b 672f
6731 6735 6739 673d 6740 6741 6743 6746
6749 674a 674f 6753 6757 675b 675f 6763
6767 676b 676f 6773 6777 677b 677f 6783
6787 678b 678f 6793 6797 679b 679f 67a3
67a7 67ab 67af 67b3 67b7 67bb 67bf 67c3
67c7 67cb 67cf 67d3 67d7 67db 67df 67e3
67e7 67eb 67ef 67f3 67f7 67fb 67ff 6803
6807 680b 680f 6813 6817 681b 681f 6823
6827 682b 682f 6833 6837 683b 683f 6843
6847 684b 684f 6853 6857 685b 685f 6863
6867 686b 686f 6873 6877 687b 687f 6883
6887 688b 688f 6893 6897 689b 689f 68a3
68a7 68ab 68af 68b3 68b7 68bb 68bf 68c3
68c7 68cb 68cf 68d3 68d7 68db 68df 68e3
68e7 68eb 68ef 68f3 68f7 68fb 68ff 6903
6907 690b 690f 6913 6917 691b 691f 6923
6927 692b 692f 6933 6937 693b 693f 6943
6947 694b 694f 6953 6957 695b 695f 6963
6967 696b 696f 6973 6977 697b 697f 6983
6987 698b 698f 6993 6997 699b 699f 69ab
69b0 69b4 69b8 69bc 69bf 69c3 69c7 69ca
69cd 69d2 69d5 69d8 69db 69dc 69e1 69e2
1 69e4 69e9 69ed 69f1 69f4 69f9 69fe
6a01 6a05 6a06 6a0b 6a0c 6a11 6a13 6a17
6a1a 6a1e 6a22 6a26 6a2a 6a2c 6a30 6a34
6a38 6a3b 6a3c 6a3e 6a41 6a44 6a45 6a4a
6a4e 6a52 6a57 6a5c 6a60 6a64 6a68 6a6b
6a6e 6a72 6a76 6a7a 6a7e 6a82 6a86 6a8a
6a8e 6a92 6a96 6a9a 6a9e 6aa2 6aa6 6aaa
6aae 6ab2 6abe 6ac2 6ac7 6acb 6ace 6acf
6ad4 6ad8 6adc 6adf 6ae2 6ae6 6ae8 6aec
6af0 6af3 6af6 6afa 6afc 6b00 6b04 6b07
6b09 6b0d 6b11 6b15 6b18 6b1c 6b20 6b22
6b23 6b28 1 6b2c 6b30 6b34 6b37 6b3b
6b3f 6b43 6b47 6b4a 6b4e 6b52 6b55 6b58
6b5d 6b60 6b63 6b66 6b67 6b6c 6b6d 1
6b6f 6b74 6b78 6b7c 6b7f 6b84 6b89 6b8c
6b90 6b91 6b96 6b99 6b9e 6b9f 6ba4 6ba7
6bac 6bad 6bb2 6bb5 6bb9 6bba 6bbf 6bc2
6bc7 6bc8 6bcd 6bd0 6bd4 6bd8 6bdb 6bde
6bdf 6be1 6be2 6be7 6be8 6bed 6bef 6bf3
6bf6 6bf8 6bfc 6bfe 6c0a 6c0c 6c0e 6c0f
6c14 6c18 6c1a 6c26 6c28 6c2c 6c30 6c35
6c3a 6c3e 6c42 6c46 6c49 6c4c 6c50 6c54
6c58 6c5c 6c60 6c64 6c68 6c6c 6c70 6c74
6c78 6c7c 6c80 6c84 6c88 6c94 6c98 6c9d
6ca1 6ca4 6ca5 6caa 6cae 6cb2 6cb5 6cb8
6cbc 6cbe 6cc2 6cc6 6cc9 6ccc 6cd0 6cd2
6cd6 6cda 6cdd 6cdf 6ce3 6ce7 6ceb 6cee
6cf2 6cf6 6cf8 6cf9 6cfe 1 6d02 6d06
6d0a 6d0d 6d11 6d15 6d19 6d1d 6d20 6d24
6d28 6d2b 6d2e 6d33 6d36 6d39 6d3c 6d3d
6d42 6d43 1 6d45 6d4a 6d4e 6d52 6d55
6d5a 6d5f 6d62 6d66 6d67 6d6c 6d6f 6d74
6d75 6d7a 6d7d 6d82 6d83 6d88 6d8b 6d8f
6d90 6d95 6d98 6d9d 6d9e 6da3 6da6 6daa
6dae 6db1 6db4 6db5 6db7 6db8 6dbd 6dbe
6dc3 6dc5 6dc9 6dcc 6dce 6dd2 6dd4 6de0
6de2 6de4 6de5 6dea 6dee 6df0 6dfc 6dfe
6e02 6e06 6e0b 6e10 6e14 6e18 6e1c 6e1f
6e22 6e26 6e2a 6e2e 6e32 6e36 6e3a 6e3e
6e42 6e46 6e4a 6e4e 6e52 6e56 6e5a 6e5e
6e62 6e66 6e6a 6e6e 6e72 6e76 6e7a 6e7e
6e82 6e86 6e8a 6e8e 6e92 6e96 6e9a 6e9e
6ea2 6ea6 6eaa 6eae 6eb2 6eb6 6eba 6ebe
6eca 6ece 6ed3 6ed7 6eda 6edb 6ee0 6ee4
6ee8 6eeb 6eee 6ef2 6ef4 6ef8 6efc 6eff
6f02 6f06 6f08 6f0c 6f10 6f13 6f15 6f19
6f1d 6f21 6f24 6f28 6f2c 6f2e 6f2f 6f34
1 6f38 6f3c 6f40 6f43 6f47 6f4b 6f4f
6f53 6f56 6f5a 6f5e 6f61 6f64 6f69 6f6c
6f6f 6f72 6f73 6f78 6f79 1 6f7b 6f80
6f84 6f88 6f8b 6f90 6f95 6f98 6f9c 6f9d
6fa2 6fa5 6faa 6fab 6fb0 6fb3 6fb8 6fb9
6fbe 6fc1 6fc5 6fc6 6fcb 6fce 6fd3 6fd4
6fd9 6fdc 6fe0 6fe4 6fe7 6fea 6feb 6fed
6fee 6ff3 6ff4 6ff9 6ffb 6fff 7002 7004
7008 700a 7016 7018 701a 701b 7020 7024
7026 7032 7034 7038 703c 7040 7044 7048
704c 7050 7054 7058 705c 7060 7064 7068
706c 7070 7074 7078 707c 7080 7084 7088
708c 7090 7094 7098 709c 70a0 70a4 70a8
70ac 70b0 70b4 70b8 70bc 70c0 70c4 70c8
70cc 70d0 70d4 70d8 70dc 70e0 70e4 70e8
70ec 70f0 70f4 70f8 70fc 7100 7104 7108
710c 7110 7114 7118 711c 7120 7124 7128
712c 7130 7134 7138 713c 7140 7144 7148
714c 7150 7154 7158 715c 7160 7164 7168
716c 7170 7174 7178 717c 7180 7184 7188
718c 7190 7194 7198 719c 71a0 71a4 71a8
71ac 71b0 71b4 71b8 71bc 71c0 71c4 71c8
71cc 71d0 71d4 71d8 71dc 71e0 71e4 71f0
71f5 71f9 71fd 7201 7204 7208 720c 720f
7212 7217 721a 721d 7220 7221 7226 7227
1 7229 722e 7232 7236 7239 723e 7243
7246 724a 724b 7250 7251 7256 7258 725c
725f 7263 7267 726b 726f 7271 7275 7279
727d 7280 7281 7283 7286 7289 728a 728f
7293 7297 729b 729f 72a3 72a7 72ab 72af
72b3 72b7 72bb 72bf 72c3 72c7 72cb 72cf
72d3 72d7 72db 72df 72e3 72e7 72eb 72ef
72f3 72f7 7303 7308 730c 7310 7314 7317
731b 731f 7322 7325 732a 732d 7330 7333
7334 7339 733a 1 733c 7341 7345 7349
734c 7351 7356 7359 735d 735e 7363 7364
7369 736b 736f 7372 7376 737a 737e 7382
7384 7388 738c 7390 7393 7394 7396 739a
739d 73a1 73a4 73a5 73aa 73ad 73b1 73b5
73b8 73b9 73bb 73bf 73c2 73c6 73c9 73ca
73cf 1 73d2 73d7 73db 73df 73e3 73e7
73eb 73ef 73f3 73f7 73fb 73ff 7403 7407
740b 740f 7413 7417 741b 741f 7423 7427
742b 742f 7433 7437 743b 743f 744b 7450
7454 7458 745c 745f 7463 7467 746a 746d
7472 7475 7478 747b 747c 7481 7482 1
7484 7489 748d 7491 7494 7499 749e 74a1
74a5 74a6 74ab 74ac 74b1 74b3 74b7 74ba
74be 74c2 74c6 74c8 74cc 74d0 74d4 74d7
74db 74df 74e2 74e5 74ea 74ed 74f0 74f3
74f4 74f9 74fa 1 74fc 7501 7505 7509
750c 7511 7516 7519 751d 751e 7523 7526
752b 752c 7531 7534 7539 753a 753f 7542
7546 754a 754e 7552 7555 7556 7558 755d
755e 7560 7561 7566 7569 756e 756f 7574
7577 757c 757d 7582 7585 7589 758a 758f
7592 7597 7598 759d 759e 75a3 75a5 75a9
75ac 75b0 75b4 75b7 75bb 75bf 75c2 75c6
75ca 75cb 75d0 75d2 75d6 75da 75dd 75df
75e3 75e7 75ea 75ee 75f2 75f5 75f9 75fd
7600 7603 7608 760b 760e 7611 7612 7617
7618 1 761a 761f 7623 7627 762a 762f
7634 7637 763b 763c 7641 7642 7647 764b
764f 7652 7657 765c 765f 7663 7667 766a
766b 7670 7673 7678 7679 767e 7681 7685
7689 768c 768d 7692 7693 7698 769a 769e
76a1 76a5 76a9 76ad 76b1 76b5 76b9 76bd
76c1 76c5 76d1 76d6 76da 76de 76e2 76e6
76e8 76e9 76ee 1 76f2 76f6 76fa 76fd
7701 7705 7709 770d 7710 7714 7718 771b
771e 7723 7726 7729 772c 772d 7732 7733
1 7735 773a 773e 7742 7745 774a 774f
7752 7756 7757 775c 775d 7762 7766 776a
776d 7772 7777 777a 777e 777f 7784 7787
778c 778d 7792 7795 7799 779d 77a0 77a3
77a4 77a6 77a7 77ac 77ad 77b2 77b4 77b8
77bb 77bf 77c3 77c7 77cb 77cf 77d3 77d7
77e3 77e8 77ec 77f0 77f4 77f8 77fa 77fe
7800 780c 780e 7810 7811 7816 781a 781c
7828 782c 782e 782f 7838 
2055
2
0 1 a 6 9 :3 6 11 :3 6
10 :2 6 1 3 a 13 a :2 1
4 b 14 13 :2 b :2 4 9 12
11 :2 9 :2 4 f 18 17 :2 f :2 4
11 1a 19 :2 11 :2 4 9 12 11
:2 9 :2 4 f 18 17 :2 f :2 4 :2 e
1d e :2 4 :2 e 1d e :2 4 :2 e
1d e :2 4 :2 c 1b c :2 4 :2 14
23 14 :2 4 :2 12 21 12 4 7
e 17 :3 e :2 7 e 17 :3 e :2 7
e 17 :3 e :2 7 e 17 :3 e :2 7
e 17 :3 e :2 7 e 17 :3 e :2 7
e 17 :3 e :2 7 e 17 :3 e :2 7
e 17 :3 e :2 7 e 17 :3 e :2 7
e 17 :3 e :2 7 e 17 :3 e :2 7
e 17 :3 e :2 7 f 18 :3 f :2 7
13 1c :3 13 7 4 e 17 :3 e
4 7 :2 11 23 :2 2d 3a :2 23 :3 7
:2 d 19 25 :2 2f 43 :2 25 54 5a
5d :2 54 6b 81 8 :2 6b d 10
:2 6b 13 16 :2 6b b e :2 6b 13
16 1f 27 :2 1f 34 :2 16 :2 6b b
e :2 6b 13 16 :2 6b 1b 1e :2 6b
28 2b :2 6b 30 33 :2 6b :2 7 4a
:2 4 7 :2 11 23 :2 2d 39 3c 4c
4e 4f :2 4c :2 23 :3 7 :2 d 12 1e
5d 60 :2 1e :2 7 52 :2 4 :4 8 7
a :2 14 26 :2 30 3c 40 50 52
53 :2 50 :2 26 :3 a :2 10 15 21 60
63 :2 21 70 73 :2 21 77 7a :2 21
84 87 90 98 :2 a1 :2 90 b2 :2 87
:2 21 ba bd :2 21 c5 c8 :2 21 cd
d0 :2 21 d3 d6 :2 21 :2 a 56 :2 7
a :2 14 26 :2 30 3c 3f 4f 51
52 :2 4f :2 26 :3 a :2 10 15 21 60
63 :2 21 :2 a 55 :3 7 :2 12 26 :2 2f
3f 43 :2 7 14 :3 4 c :2 4 11
:2 4 12 4 8 11 1c :2 8 1f
21 :2 1f 7 28 31 3c :2 28 3f
41 :2 3f 27 :2 7 11 13 1f 21
2a 33 35 4b 4d f 1d 28
33 f 17 1a 22 2a 2c 39
42 44 51 53 :2 a :2 13 1f a
7 f d :2 16 22 d 1d :2 a
:2 f d :2 16 22 d 10 :2 1a 2c
:2 36 42 46 56 59 5a :2 56 :2 2c
:3 10 :2 16 1b 27 67 6a :2 27 77
7a :2 27 7e 81 :2 27 8d 90 :2 27
98 9b :2 27 a7 aa b1 ba bd
:2 aa :2 27 :2 10 5d :2 d :2 a :4 16 :2 a
7 :3 44 b :2 14 1f 1d :2 28 :2 1d
a 14 16 22 24 2d 36 38
12 20 2b 12 1a 1d 25 2d
2f 3c 45 47 54 56 :2 d :2 16
22 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 59 5c 5d
:2 59 :2 2f :3 13 :2 19 1e 2a 6a 6d
:2 2a 7a 7d :2 2a 81 84 :2 2a 90
93 :2 2a 9b 9e :2 2a aa ad b4
bd c0 :2 ad :2 2a :2 13 60 :2 10 :2 d
:4 19 :2 d a :3 38 e :2 17 22 20
:2 2b :2 20 d 20 2f 3f 4c :2 10
:2 19 25 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 5c 5f
60 :2 5c :2 32 :3 16 :2 1c 21 2d 6d
70 :2 2d 7d 80 :2 2d 84 87 :2 2d
93 96 :2 2d 9e a1 :2 2d ad b0
b7 c0 c3 :2 b0 :2 2d :2 16 63 :2 13
:2 10 :4 1c :2 10 d :3 3b 11 :2 1a 25
23 :2 2e :2 23 10 13 :2 1d 2f :2 39
45 49 59 5c 5d :2 59 :2 2f :3 13
:2 19 1e 2a 6a 6d :2 2a 7a 7d
:2 2a 81 84 :2 2a 8e 91 9a a2
:2 ab :2 9a bc :2 91 :2 2a c4 c7 :2 2a
cf d2 :2 2a d7 da :2 2a df e2
:2 2a :2 13 60 :2 10 13 :2 1d 2f :2 39
45 48 58 5b 5c :2 58 :2 2f :3 13
:2 19 1e 2a 6a 6d :2 2a :2 13 5f
:3 10 :2 1b 2f :2 38 48 4e :2 10 3e
:2 d 3b :2 a 38 :2 7 4 44 a
13 1e :2 a 21 23 :2 21 11 21
f 26 3a :2 a :2 13 1f a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 56 59 5a :2 56 :2 2c :3 10
:2 16 1b 27 67 6a :2 27 77 7a
:2 27 7e 81 :2 27 8d 90 :2 27 98
9b :2 27 a7 aa b1 ba bd :2 aa
:2 27 :2 10 5d :2 d :2 a :4 16 :2 a 7
:3 26 b :2 14 1f 1d :2 28 :2 1d a
d :2 17 29 :2 33 3f 43 53 56
57 :2 53 :2 29 :3 d :2 13 18 24 64
67 :2 24 74 77 :2 24 7b 7e :2 24
88 8b 94 9c :2 a5 :2 94 b6 :2 8b
:2 24 be c1 :2 24 c9 cc :2 24 d1
d4 :2 24 d7 da :2 24 :2 d 5a :2 a
d :2 17 29 :2 33 3f 42 52 55
56 :2 52 :2 29 :3 d :2 13 18 24 64
67 :2 24 :2 d 59 :3 a :2 15 29 :2 32
42 46 :2 a 38 :2 7 11 1a 30
41 4c f 1d 2a :2 a :2 13 1f
a 7 f d :2 16 22 d 1d
:2 a :2 f d :2 16 22 d 10 :2 1a
2c :2 36 42 46 56 59 5a :2 56
:2 2c :3 10 :2 16 1b 27 67 6a :2 27
77 7a :2 27 7e 81 :2 27 8d 90
:2 27 98 9b :2 27 a7 aa b1 ba
bd :2 aa :2 27 :2 10 5d :2 d :2 a :4 16
:2 a 7 :3 26 4 26 44 a 13
1e :2 a 21 23 :2 21 11 1b 27
37 44 :2 a :2 13 1f a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 56 59 5a :2 56 :2 2c :3 10 :2 16
1b 27 67 6a :2 27 77 7a :2 27
7e 81 :2 27 8d 90 :2 27 98 9b
:2 27 a7 aa b1 ba bd :2 aa :2 27
:2 10 5d :2 d :2 a :4 16 :2 a 7 :3 26
b :2 14 1f 1d :2 28 :2 1d a d
:2 17 29 :2 33 3f 43 53 56 57
:2 53 :2 29 :3 d :2 13 18 24 64 67
:2 24 74 77 :2 24 7b 7e :2 24 88
8b 94 9c :2 a5 :2 94 b6 :2 8b :2 24
be c1 :2 24 c9 cc :2 24 d1 d4
:2 24 d7 da :2 24 :2 d 5a :2 a d
:2 17 29 :2 33 3f 42 52 55 56
:2 52 :2 29 :3 d :2 13 18 24 64 67
:2 24 :2 d 59 :3 a :2 15 29 :2 32 42
46 :2 a 38 :3 7 14 7 4 26
44 b 14 1f :2 b 25 22 :2 2e
:2 22 a 48 51 5c :2 48 5f 62
:2 5f 47 :2 a 16 1f 2a :2 16 30
2d :2 39 :2 2d 15 :2 a 5b 11 1c
:2 5b 22 1f :2 2b :2 1f 5a :2 a 11
1a 30 41 4c f 1d 2a :2 a
:2 13 1f a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 56 59
5a :2 56 :2 2c :3 10 :2 16 1b 27 67
6a :2 27 77 7a :2 27 7e 81 :2 27
8d 90 :2 27 98 9b :2 27 a7 aa
b1 ba bd :2 aa :2 27 :2 10 5d :2 d
:2 a :4 16 :2 a 7 :3 49 b :2 14 1f
1d :2 28 :2 1d a d :2 17 29 :2 33
3f 43 53 56 57 :2 53 :2 29 :3 d
:2 13 18 24 64 67 :2 24 74 77
:2 24 7b 7e :2 24 88 8b 94 9c
:2 a5 :2 94 b6 :2 8b :2 24 be c1 :2 24
c9 cc :2 24 d1 d4 :2 24 d7 da
:2 24 :2 d 5a :2 a d :2 17 29 :2 33
3f 42 52 55 56 :2 52 :2 29 :3 d
:2 13 18 24 64 67 :2 24 :2 d 59
:3 a :2 15 29 :2 32 42 46 :2 a 38
:2 7 49 44 :2 4 7 10 1b :2 7
1e 20 :2 1e 7 f 19 :3 7 :2 10
1c 7 16 :8 a c a c a
c a c a c a c a
c f 24 26 2b 3c 41 43
4f 51 10 1d 26 a 11 d
19 :3 17 d :2 16 22 d 1e d
:2 16 22 d :4 a 7 f d :2 16
22 d 1d :2 a :2 f d :2 16 22
d 10 :2 1a 2c :2 36 42 46 56
59 5a :2 56 :2 2c :3 10 :2 16 1b 27
67 6a :2 27 77 7a :2 27 7e 81
:2 27 8d 90 :2 27 98 9b :2 27 a7
aa b1 ba bd :2 aa :2 27 :2 10 5d
:2 d :2 a :4 16 :2 a 7 :3 22 7 f
19 :3 7 :2 10 1c 7 16 29 1a
f 10 1d 26 a 11 d 19
:3 17 d :2 16 22 d 1e d :2 16
22 d :4 a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 56 59
5a :2 56 :2 2c :3 10 :2 16 1b 27 67
6a :2 27 77 7a :2 27 7e 81 :2 27
8d 90 :2 27 98 9b :2 27 a7 aa
b1 ba bd :2 aa :2 27 :2 10 5d :2 d
:2 a :4 16 :2 a 7 :3 22 7 c 3d
4e 57 5a 65 6e 71 7f 83
a8 7 b 18 1a 27 35 37
43 45 55 57 60 62 72 7
b 21 25 3a 3e 4d 51 67
71 74 81 84 91 94 a1 a4
b8 c9 c 22 26 2b 35 3a
3e 4a 4c 7 c 1d 22 24
2f 33 7 c 1d 22 24 2f
31 7 c 12 17 1d 23 26
2e 32 14 1d 21 2f 3c 3e
4e 61 73 7 9 12 14 24
7 9 1f 21 35 37 45 47
5d 7 a 17 1a 27 2a 37
3a 4d 5e c 1e 23 28 31
36 3a 45 47 7 c 12 17
28 2d 2f 37 39 7 c 12
17 28 2d 2f 3a 3c 7 c
12 17 1d 23 26 2e 30 14
1d 21 2f 3b 3d 4d 5f 70
7 9 12 14 24 7 9 1f
21 36 38 47 49 5f 7 a
17 1a 27 2a 37 3a 4e 5f
c 1e 22 27 30 35 39 44
46 7 c 23 28 2a 35 37
7 c 12 17 28 2d 2f 38
3a 7 c 12 17 28 2d 2f
3a 3c 7 c 12 17 1d 23
26 2e 30 14 1d 21 2f 3b
3d 4d 5f 70 7 9 12 14
24 7 9 1f 21 36 38 47
49 5f 7 a 17 1a 27 2a
37 3a 4e 5f c 1e 22 27
30 35 39 44 46 7 c 22
27 29 34 36 7 c 12 17
28 2d 2f 37 39 7 c 12
17 28 2d 2f 3a 3c 7 c
12 17 1d 23 26 2e 30 10
1a 27 17 7 17 a :2 14 26
:2 30 3c 3f 4f 52 53 :2 4f :2 26
:3 a :2 10 15 21 61 64 :2 21 :2 a
56 :3 7 e 7 4 22 b 14
1f :2 b 22 24 :2 22 a 2b 34
3f :2 2b 42 44 :2 42 2a :2 a 7
f 19 :3 7 :2 10 1c 7 16 28
35 a d 14 17 49 f 24
27 2c 32 37 48 4e 51 5c
65 68 76 79 10 13 1e 27
2a 40 43 23 34 3c 3f 4d
4f 5f 61 69 a 11 d 19
:3 17 d :2 16 22 d 1e d :2 16
22 d :4 a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 56 59
5a :2 56 :2 2c :3 10 :2 16 1b 27 67
6a :2 27 77 7a :2 27 7e 81 :2 27
8d 90 :2 27 98 9b :2 27 a7 aa
b1 ba bd :2 aa :2 27 :2 10 5d :2 d
:2 a :4 16 :2 a 7 :3 47 7 :2 10 :3 1c
7 16 2a 35 1a 1c 28 2a
f 24 26 2b 34 39 3b 51
53 5e 60 a f 20 25 27
32 34 a f 20 25 27 32
34 a f 20 25 27 39 3b
51 53 10 12 1d 26 28 a
11 d 19 :3 17 d :2 16 22 d
1e d :2 16 22 d :4 a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 56 59 5a :2 56 :2 2c :3 10 :2 16
1b 27 67 6a :2 27 77 7a :2 27
7e 81 :2 27 8d 90 :2 27 98 9b
:2 27 a7 aa b1 ba bd :2 aa :2 27
:2 10 5d :2 d :2 a :4 16 :2 a 7 :3 47
7 c 1e 20 2f 40 53 55
62 6f 71 79 7b 7 2e 3d
64 78 7 2e 3a 61 73 7e
8d 94 a1 b6 c7 c 1b 2e
49 5e 6f c 1d 1f 24 2e
33 35 41 43 e 1f 32 45
53 7 2e 3d 64 78 7 2e
3a 61 73 7e 8d 94 a1 d
1f 30 3f 52 6d 82 93 c
10 17 7 17 a :2 14 26 :2 30
3c 3f 4f 52 53 :2 4f :2 26 :3 a
:2 10 15 21 61 64 :2 21 :2 a 56
:3 7 e 7 4 47 22 a 13
1e :2 a 21 23 :2 21 7 c 28
2a 4d 4f 66 68 81 7 9
15 17 27 3a 42 44 57 59
66 68 76 78 7 9 16 18
21 23 2d 3a 4a 4c 5b 5d
6a 6c 7c 7 10 12 22 24
34 45 48 58 6b 6d 7a 7c
89 8b 98 9a ad bb bd c8
ca 7 9 1f 21 2e 31 41
4c 4e 56 58 62 64 6c 6e
79 7b 89 8b 99 7 9 c
21 23 28 2e 33 44 49 4b
55 5e 60 74 76 7 c 12
17 20 25 27 35 37 42 47
4d 52 66 6b 6d 79 7b 7
c 12 17 2b 31 3a 3c 4c
4e 5b 5e 7 c 12 17 25
2b 2e 3a 3c d f 1c 25
27 42 17 7 17 a :2 14 26
:2 30 3c 3f 4f 52 53 :2 4f :2 26
:3 a :2 10 15 21 61 64 :2 21 :2 a
56 :3 7 e 7 4 25 22 a
13 1e :2 a 21 23 :2 21 a 11
1a 25 :2 11 :2 a 29 2c :2 29 a
15 a 14 22 2f 46 53 64
6e :2 d :2 16 22 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
59 5c 5d :2 59 :2 2f :3 13 :2 19 1e
2a 6a 6d :2 2a 7a 7d :2 2a 81
84 :2 2a 90 93 :2 2a 9b 9e :2 2a
aa ad b4 bd c0 :2 ad :2 2a :2 13
60 :2 10 :2 d :4 19 :2 d a :3 2e d
16 1f :2 d 22 24 :2 22 d 1b
d 26 d 18 d 17 27 34
20 37 44 55 5f 68 :2 10 :2 19
25 10 d 15 13 :2 1c 28 13
23 :2 10 :2 15 13 :2 1c 28 13 16
:2 20 32 :2 3c 48 4c 5c 5f 60
:2 5c :2 32 :3 16 :2 1c 21 2d 6d 70
:2 2d 7d 80 :2 2d 84 87 :2 2d 93
96 :2 2d 9e a1 :2 2d ad b0 b7
c0 c3 :2 b0 :2 2d :2 16 63 :2 13 :2 10
:4 1c :2 10 d :3 a 10 19 22 :2 10
25 27 :2 25 1a 2a 37 23 34
37 3c 56 16 1a 27 2a 39
3d 4b 4e 19 1c 29 3a 3d
47 14 1c 23 26 49 4d :2 13
:2 1c 28 13 10 18 16 :2 1f 2b
16 26 :2 13 :2 18 16 :2 1f 2b 16
19 :2 23 35 :2 3f 4b 4f 5f 62
63 :2 5f :2 35 :3 19 :2 1f 24 30 70
73 :2 30 80 83 :2 30 87 8a :2 30
96 99 :2 30 a1 a4 :2 30 b0 b3
ba c3 c6 :2 b3 :2 30 :2 19 66 :2 16
:2 13 :4 1f :2 13 10 :3 29 13 1c 25
:2 13 28 2a :2 28 13 21 13 2c
13 21 13 :4 10 29 10 1e 10
:4 d :5 a f 21 23 2f 31 3e
40 4e 50 5f 72 74 81 83
8c 8e a c 10 12 16 18
24 33 35 42 a c 1b 1d
2f 3d 3f 4e 50 61 63 72
c 1b 1d 14 16 32 39 43
45 52 5c 5e 6d 6f 8e a
31 3e 65 78 a c 1b 1d
2d 2f 34 36 43 45 a c
1b 22 24 34 36 47 49 5c
a 17 2c 41 54 10 27 42
57 6c 81 10 1e 20 33 42
4f 5d 5f 6f 71 82 84 98
10 2c 40 55 62 71 7e 8e
a1 b4 c3 f 21 2f 3e 4f
51 5f 61 14 2b 2d 3e 69
14 2b 2d 3f 6a 14 2b 2d
4a 58 5a 6a 81 90 10 28
42 5d 7b 10 23 f 17 19
1e 23 38 3d 3f 4c 4e 5d
5f 6d a f 15 1a 2e 33
35 41 43 10 12 1f 30 32
3f 1b 1d 40 42 59 5b 74
80 82 14 16 35 37 49 52
5a 5c 6a 6c 17 19 28 32
3a 3c 4a 4c 60 14 16 3e
40 69 a c 1f 21 2e 30
3a 43 45 54 56 67 14 16
1b 24 2e 30 35 3e 49 52
5d 5f 69 77 79 7e 87 92
9b a6 a8 b2 a 13 21 31
40 49 4b 5a 5c 6d 14 16
2a 2c 3c 3e 52 14 16 2a
2c 39 3b 4c 4e 62 14 16
25 27 36 38 4a 4c 5d f
14 16 32 39 43 45 52 5c
69 84 a c 19 1b 29 2b
a c 1b 1d 2d 2f 34 36
43 45 a c 1b 22 32 43
a 17 27 29 38 3a 47 49
59 a 13 15 25 27 37 48
4b 5b 6e 70 7d 7f 8c 8e
9b 9d 14 16 35 37 4a 54
5c 5e 6c 6e 19 27 29 34
36 44 46 56 58 69 6b a
c 22 24 32 35 45 55 57
5f 61 6b 6d 75 77 82 84
92 94 a2 a4 ae b0 a c
28 2a 48 4a 72 7c 7e 94
14 16 2a 2c 40 42 5a 14
2b 34 36 42 44 5b 86 14
2b 34 36 43 45 5d 88 14
35 37 4d 4f 58 5a 79 87
89 9e b5 c4 a c a d
1d a c a d 1d 34 36
a c 19 1d 2d f 24 26
2b 31 36 47 4c 4e 62 64
a f 15 1a 2e 33 35 41
43 a f 15 1a 2e 34 3d
3f 4f 51 5e 61 a f 15
1a 28 2e 31 3d 3f a f
15 1a 2a 30 33 40 42 a
f 15 1a 2a 30 33 40 42
a f 15 1a 28 2f 33 40
42 10 12 1f 30 32 3f 48
4a 13 1b 1a a 1a d :2 17
29 :2 33 3f 42 52 55 56 :2 52
:2 29 :3 d :2 13 18 24 64 67 :2 24
:2 d 59 :3 a 11 a 2e a f
2b 2d 50 52 69 6b 84 8f
91 14 16 35 37 46 50 58
5a 68 6a f 11 30 32 44
4d 55 57 65 67 f 11 30
32 44 4e 56 58 66 68 17
19 2b 3d 45 47 55 57 6a
14 16 2d 36 f 11 28 31
f 11 28 31 f 11 39 42
55 a c 1f 21 2e 30 39
3b a c 10 12 16 1f 21
2f 31 41 50 59 5b 6a 6c
7d a c 14 2c 2e 42 44
54 56 6a a c 19 28 2a
42 a c 19 1b 29 2b a
c 1b 1d 2d 2f 34 36 43
45 a c 1b 22 32 43 a
17 27 29 38 3a 47 49 59
a 13 15 25 27 37 48 4b
5b 6e 70 7d 7f 8c 8e 9b
9d 14 16 35 37 4a 54 5c
5e 6c 6e 19 27 29 34 36
44 46 56 58 69 6b a c
22 24 32 35 45 55 57 5f
61 6b 6d 75 77 82 84 92
94 a2 a4 ae b0 a c 28
2a 48 4a 72 7c 7e 94 a7
a c 19 1b 29 2b 39 3b
50 67 14 16 38 3a 52 54
6b 6d 7c 7e 92 94 a4 a6
d2 a c a d 1d a c
a d 1d 34 36 a c 19
1d 2d f 24 26 2b 31 36
47 4c 4e 62 64 a f 15
1a 2e 33 35 41 43 a f
15 1a 2e 34 3d 3f 4f 51
5e 61 a f 15 1a 28 2e
31 3d 3f a f 15 1a 2a
30 33 40 42 a f 15 1a
2a 30 33 40 42 a f 15
1a 28 2f 33 40 42 10 12
32 34 41 13 1c 1a a 1a
d :2 17 29 :2 33 3f 42 52 55
56 :2 52 :2 29 :3 d :2 13 18 24 64
67 :2 24 :2 d 59 :3 a 11 a :4 7
4 25 22 a 13 1e :2 a 21
23 :2 21 7 f 19 :3 7 :2 10 1c
7 16 :c a e 20 a e 21
a e 1f a e 1e a e
1f a e 1d a e 16 f
26 31 3a 61 6e a 11 d
19 :3 17 d :2 16 22 d 1e d
:2 16 22 d :4 a 7 f d :2 16
22 d 1d :2 a :2 f d :2 16 22
d 10 :2 1a 2c :2 36 42 46 56
59 5a :2 56 :2 2c :3 10 :2 16 1b 27
67 6a :2 27 77 7a :2 27 7e 81
:2 27 8d 90 :2 27 98 9b :2 27 a7
aa b1 ba bd :2 aa :2 27 :2 10 5d
:2 d :2 a :4 16 :2 a 7 :3 25 7 :2 10
:3 1c 7 16 2a 35 43 52 62
71 7d a 10 1a 1c 27 29
a c 1b 1d 2c 2e 3e 40
4c 4e 5b 5d 63 65 f 1f
21 26 2f 34 36 4c 4e 59
5b 67 6c 7d 82 84 8f 91
a f 20 25 27 32 34 a
11 d 19 :3 17 d :2 16 22 d
1e d :2 16 22 d :4 a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 56 59 5a :2 56 :2 2c :3 10 :2 16
1b 27 67 6a :2 27 77 7a :2 27
7e 81 :2 27 8d 90 :2 27 98 9b
:2 27 a7 aa b1 ba bd :2 aa :2 27
:2 10 5d :2 d :2 a :4 16 :2 a 7 :3 25
7 c 1e 20 2f 40 53 55
62 75 7d 7f 7 9 18 1a
29 2b 3b 3d 49 4b 58 5a
60 62 71 78 85 d 1e 30
3f 52 6d 7b 7d 92 a3 c
1c 1e 23 2d 32 34 4c 4e
5a 5c e 1f 32 45 53 7
16 25 37 43 50 56 65 6c
79 d 1f 30 3f 52 6d 81
96 a7 c 29 17 7 17 a
:2 14 26 :2 30 3c 3f 4f 52 53
:2 4f :2 26 :3 a :2 10 15 21 61 64
:2 21 :2 a 56 :3 7 e 7 4 25
22 a 13 1e :2 a 21 23 :2 21
7 f 19 :3 7 :2 10 1c 7 16
:c a e 20 a e 21 a e
1f a e 1e a e 1f a
e 1d a e 16 f 26 31
3a 61 6e a 11 d 19 :3 17
d :2 16 22 d 1e d :2 16 22
d :4 a 7 f d :2 16 22 d
1d :2 a :2 f d :2 16 22 d 10
:2 1a 2c :2 36 42 46 56 59 5a
:2 56 :2 2c :3 10 :2 16 1b 27 67 6a
:2 27 77 7a :2 27 7e 81 :2 27 8d
90 :2 27 98 9b :2 27 a7 aa b1
ba bd :2 aa :2 27 :2 10 5d :2 d :2 a
:4 16 :2 a 7 :3 25 7 :2 10 :3 1c 7
16 2a 35 43 52 5b 6b 7a
8a 99 a5 a 10 1b 1a 1c
28 2a a c 1b 1d 2c 2e
3e 40 4c 4e 5b 5d 63 65
f 1f 21 26 2f 34 36 4c
4e 59 5b 67 6c 7d 82 84
8f 91 a f 20 25 27 32
34 1a 1d 2a 2c 40 42 a
30 a 30 5d 5f f 24 28
32 10 13 1e 27 2a 38 3a
20 23 3a 4b 10 13 30 32
f 24 2c 2e 3b 3e 4d 4f
5d 60 a 11 d 19 :3 17 d
:2 16 22 d 1e d :2 16 22 d
:4 a 7 f d :2 16 22 d 1d
:2 a :2 f d :2 16 22 d 10 :2 1a
2c :2 36 42 46 56 59 5a :2 56
:2 2c :3 10 :2 16 1b 27 67 6a :2 27
77 7a :2 27 7e 81 :2 27 8d 90
:2 27 98 9b :2 27 a7 aa b1 ba
bd :2 aa :2 27 :2 10 5d :2 d :2 a :4 16
:2 a 7 :3 25 7 c 1e 20 2f
40 53 55 62 75 7d 7f 7
9 18 1a 29 2b 3b 3d 49
4b 58 5a 60 62 71 78 85
d 1e 30 3f 52 6d 7b 7d
92 a3 c 1c 1e 23 2d 32
34 4c 4e 5a 5c e 1f 32
3f 47 7 16 25 37 43 50
56 65 6c 79 d 1f 30 3f
52 6d 7c 91 a2 c 26 3c
e 1f 32 3f 47 7 16 25
7 13 20 26 35 3c 49 5e
6f c 1b 2e 49 57 6c 7d
c d 25 10 1e 17 7 17
a :2 14 26 :2 30 3c 3f 4f 52
53 :2 4f :2 26 :3 a :2 10 15 21 61
64 :2 21 :2 a 56 :3 7 e 7 4
25 22 a 13 1e :2 a 21 23
:2 21 7 c 1e 20 2f 3b 3d
50 52 5f 73 7b 7d 7 9
18 1a 2a 2c 34 44 46 56
69 6b 7b 87 89 9a 7 9
11 17 19 27 32 3f 54 66
c 1a 2d 4a 59 66 74 76
8b 9c 7 a 18 1b 29 2c
3a 3d 4b 4e 7 a 1f 22
38 3b 4e 51 7 a 25 28
44 47 7 a 25 28 44 47
c 21 23 28 3c 42 45 52
5b 5e 6c 6e 7 c 11 22
27 29 3b 3d 53 55 7 c
11 22 27 29 3b 3d 53 55
7 c 16 1b 1d 29 2b d
f 1a 23 25 3c 3e 5f 61
e 10 1f 2b 2d 40 42 52
66 74 7 9 18 1a 29 2b
33 42 44 54 66 8d 99 c0
c 16 20 2d 42 54 c 1b
2e 43 45 59 6e 7f 7 a
18 1b 29 2c 3a 3d 4b 4e
7 a 1f 22 38 3b 4e 51
7 a 25 28 44 47 7 a
25 28 44 47 c 21 23 28
31 36 38 44 46 7 c 1d
22 24 36 38 7 c 20 26
29 36 3f 42 50 52 d f
1a 23 25 10 1e 17 7 17
a :2 14 26 :2 30 3c 3f 4f 52
53 :2 4f :2 26 :3 a :2 10 15 21 61
64 :2 21 :2 a 56 :3 7 e 7 4
25 22 a 13 1e :2 a 21 23
:2 21 7 f 19 :3 7 :2 10 1c 7
16 b :d a e 21 a e 21
a e 21 a 18 2a 3e a
e 21 a e 21 a e 21
a 18 2a 3e f 10 1b 24
2c 13 20 a 11 d 19 :3 17
d :2 16 22 d 1e d :2 16 22
d :4 a 7 f d :2 16 22 d
1d :2 a :2 f d :2 16 22 d 10
:2 1a 2c :2 36 42 46 56 59 5a
:2 56 :2 2c :3 10 :2 16 1b 27 67 6a
:2 27 77 7a :2 27 7e 81 :2 27 8d
90 :2 27 98 9b :2 27 a7 aa b1
ba bd :2 aa :2 27 :2 10 5d :2 d :2 a
:4 16 :2 a 7 :3 25 7 f 19 :3 7
:2 10 1c 7 16 :20 a c 1e 20
36 38 4e 50 a c 1e 20
36 38 4e 50 a c 1e 20
36 38 4e 50 a c 1e 20
36 38 a c 21 23 39 3b
4e 50 a c 27 29 45 47
a c 27 29 45 47 a c
1e 20 36 38 4e 50 a 13
15 28 2a 37 39 60 62 f
23 25 2a 30 35 46 4b 4d
5b 5d 10 12 1d a 11 d
19 :3 17 d :2 16 22 d 1e d
:2 16 22 d :4 a 7 f d :2 16
22 d 1d :2 a :2 f d :2 16 22
d 10 :2 1a 2c :2 36 42 46 56
5a 5b :2 56 :2 2c :3 10 :2 16 1b 27
68 6b :2 27 78 7b :2 27 7f 82
:2 27 8e 91 :2 27 99 9c :2 27 a8
ab b2 bb be :2 ab :2 27 :2 10 5e
:2 d :2 a :4 16 :2 a 7 :3 25 11 22
29 3c 11 1e 2f 37 39 44
4d 4f 5d 60 70 72 a 11
d 19 :3 17 d :2 16 22 d 1e
d :2 16 22 d :4 a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
56 5a 5b :2 56 :2 2c :3 10 :2 16 1b
27 68 6b :2 27 78 7b :2 27 7f
82 :2 27 8e 91 :2 27 99 9c :2 27
a8 ab b2 bb be :2 ab :2 27 :2 10
5e :2 d :2 a :4 16 :2 a 7 :3 25 7
c 1e 20 2f 40 53 55 62
70 72 7a 7c 7 9 18 1a
29 2b 3b 3d 49 4b 58 5a
60 62 71 78 85 d 1e 30
3f 52 6d 7b 7d 92 a3 11
13 34 36 57 59 74 8b 8d
a0 a2 b6 b8 cb cd de e0
f2 f4 7 f 7 9 1e 20
36 38 4b 4d 7 9 24 26
42 44 7 9 24 26 42 44
7 9 17 19 27 29 37 39
47 49 c 1d 1f 24 2e 33
35 4d 4f 5b 5d f 11 1f
30 43 45 55 68 76 7 9
18 1a 29 2b 3b 3d 49 4b
58 5a 60 62 71 78 85 d
1f 30 3f 52 6d 81 96 a7
11 13 34 36 57 59 74 8b
8d a0 a2 b6 b8 cb cd de
e0 f2 f4 7 f 7 9 1e
20 36 38 4b 4d 7 9 24
26 42 44 7 9 24 26 42
44 7 9 17 19 27 29 37
39 47 49 c 1d 1f 24 2d
32 34 4a 4c 57 59 7 c
1d 22 24 2f 31 7 c 1d
22 24 2f 31 17 7 17 a
:2 14 26 :2 30 3c 3f 4f 53 54
:2 4f :2 26 :3 a :2 10 15 21 62 65
:2 21 :2 a 57 :3 7 e 7 4 25
22 a 13 1e :2 a 23 21 :2 2c
:2 21 11 23 2d 3c 49 :2 a :2 13
1f a 7 f d :2 16 22 d
1d :2 a :2 f d :2 16 22 d 10
:2 1a 2c :2 36 42 46 56 5a 5b
:2 56 :2 2c :3 10 :2 16 1b 27 68 6b
:2 27 78 7b :2 27 7f 82 :2 27 8e
91 :2 27 99 9c :2 27 a8 ab b2
bb be :2 ab :2 27 :2 10 5e :2 d :2 a
:4 16 :2 a 7 :3 40 b :2 14 1f 1d
:2 28 :2 1d :2 a f 21 29 35 3d
50 5e 69 6b 74 7c a 12
16 26 30 38 3c 4c 56 5f
64 74 f 18 a f 14 26
2d 31 3f a f 14 24 2d
31 3f a f 14 1e 2d 32
3f 10 12 1f 11 19 25 2d
40 48 4b 56 a d 1b 24
2f 3c 47 54 5a 5d 73 79
7c 8e d 1b 28 36 43 52
f 25 27 2c 32 37 48 4e
50 58 5b a f 15 1a 2b
31 34 3f 42 52 54 66 11
19 25 2d 40 48 4b 56 a
d 1b 24 2f 3c 47 54 5a
5d 73 79 7c 8e d 1b 28
36 43 52 f 26 28 2d 33
38 49 4f 51 5a 5d a f
15 1a 2b 31 34 3f 42 52
54 66 13 1e 1a a 1a d
:2 17 29 :2 33 3f 42 52 56 57
:2 52 :2 29 :3 d :2 13 18 24 65 68
:2 24 :2 d 5a :3 a 11 a 38 :2 7
4 40 22 a 13 1e :2 a 21
23 :2 21 7 c 28 2a 4d 4f
66 68 81 8d 8f 11 13 32
34 46 4f 57 59 67 69 14
16 25 2f 37 39 47 49 5d
11 13 3b 3d 66 7 9 1c
1e 2b 2d 37 40 4f 60 7
9 d f 13 1c 1e 2c 2e
3e 4d 56 65 76 11 25 35
49 11 25 32 43 57 11 13
22 24 33 35 47 49 5a c
11 13 2f 36 40 42 4f 59
66 81 7 9 16 18 26 28
7 9 18 1a 2a 2c 31 33
40 42 7 9 18 1f 2f 40
7 14 24 26 35 37 44 46
56 7 10 12 22 24 34 45
48 58 6b 6d 7a 7c 89 8b
98 9a 11 13 32 34 47 51
59 5b 69 6b 16 24 26 31
33 41 43 53 55 66 68 7
9 1f 21 2f 32 42 52 54
5c 5e 68 6a 72 74 7f 81
8f 91 9f a1 ab ad 7 9
25 27 45 47 6f 79 7b 91
11 25 39 51 11 28 31 33
3f 41 58 83 11 28 31 33
40 42 5a 85 11 32 34 4a
4c 55 57 76 84 86 9b b2
11 13 35 37 4f 51 68 6a
79 7b 8f 91 a1 a3 cf 7
9 7 a 1a 7 9 7 a
1a 31 33 7 9 16 1a 2a
c 21 23 28 39 3e 40 54
5d 5f 73 75 7 c 12 17
2b 30 32 3e 40 7 c 12
17 2b 31 3a 3c 4c 4e 5b
5e 7 c 12 17 25 2b 2e
3a 3c 7 c 12 17 27 2d
30 3d 3f 7 c 12 17 27
2d 30 3d 3f 7 c 12 17
25 2c 30 3d 3f 10 1e 17
7 17 a :2 14 26 :2 30 3c 3f
4f 53 54 :2 4f :2 26 :3 a :2 10 15
21 62 65 :2 21 :2 a 57 :3 7 e
7 4 26 22 a 13 1e :2 a
21 23 :2 21 7 c 1e 20 35
41 43 11 13 32 34 46 4f
57 59 67 69 14 16 25 2f
37 39 47 49 5d 11 13 2a
33 4e 58 5a 6d 6f 7c a3
b2 d9 c 19 40 49 70 7
2e 3a 61 74 7 2e 3b 62
76 7 2e 3d 64 79 7 2e
3b 62 c 13 20 30 32 41
68 7 9 16 18 28 37 40
42 52 54 64 75 78 88 c
1e 30 43 11 13 32 34 47
51 59 5b 69 6b 16 7 9
1a 7 2e 3e 65 7b d 2a
3e 53 c 1b 28 38 4b 5e
:2 c 21 7 c 12 17 2b 30
32 3e 40 7 c 12 17 2b
31 3a 3c 4c 4e 5b 5e d
f 2f 31 3e 10 1e 17 7
17 a :2 14 26 :2 30 3c 3f 4f
53 54 :2 4f :2 26 :3 a :2 10 15 21
62 65 :2 21 :2 a 57 :3 7 e 7
4 26 22 a 13 1e :2 a 21
23 :2 21 7 f 19 :3 7 :2 10 1c
7 16 23 30 3f 11 1e 34
f 10 1d :2 a 17 20 13 20
a 11 d 19 :3 17 d :2 16 22
d 1e d :2 16 22 d :4 a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 56 5a 5b :2 56 :2 2c :3 10
:2 16 1b 27 68 6b :2 27 78 7b
:2 27 7f 82 :2 27 8e 91 :2 27 99
9c :2 27 a8 ab b2 bb be :2 ab
:2 27 :2 10 5e :2 d :2 a :4 16 :2 a 7
:3 26 7 f 19 :3 7 :2 10 1c 7
16 26 33 42 12 1f 35 f
10 1d a 14 13 20 a 11
d 19 :3 17 d :2 16 22 d 1e
d :2 16 22 d :4 a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
56 5a 5b :2 56 :2 2c :3 10 :2 16 1b
27 68 6b :2 27 78 7b :2 27 7f
82 :2 27 8e 91 :2 27 99 9c :2 27
a8 ab b2 bb be :2 ab :2 27 :2 10
5e :2 d :2 a :4 16 :2 a 7 :3 26 7
f 19 :3 7 :2 10 1c 7 16 2d
3a 49 11 13 20 22 31 33
f 1b 1d 22 28 2d 3c 41
43 4f 51 11 13 20 22 31
33 f 1e 20 25 2b 30 3c
41 43 4f 51 a 11 d 19
:3 17 d :2 16 22 d 1e d :2 16
22 d :4 a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 56 5a
5b :2 56 :2 2c :3 10 :2 16 1b 27 68
6b :2 27 78 7b :2 27 7f 82 :2 27
8e 91 :2 27 99 9c :2 27 a8 ab
b2 bb be :2 ab :2 27 :2 10 5e :2 d
:2 a :4 16 :2 a 7 :3 26 7 c 1e
20 2f 3b 3d 4a 5d 66 68
73 75 c 22 24 29 33 38
3a 52 54 60 62 7 c 12
17 26 2d 2f 3b 3f 17 19
27 33 35 45 58 68 73 75
c 22 24 29 32 37 39 51
53 60 62 7 c 1d 22 24
2f 31 7 c 1d 22 24 2f
31 7 c 12 17 26 2d 2f
3b 3f 17 1a 29 35 37 44
59 62 64 76 c 21 25 2f
d 10 1b 7 a 18 1a 19
1c 2d 43 4c 4f 5d 60 10
1e 17 7 17 a :2 14 26 :2 30
3c 3f 4f 53 54 :2 4f :2 26 :3 a
:2 10 15 21 62 65 :2 21 :2 a 57
:3 7 e 7 4 26 22 a 13
1e :2 a 21 23 :2 21 7 c 1f
21 2d 2f 3c 4f 58 5a 6c
c 16 18 1d 27 a c 19
1b d f b d 17 10 18
7 18 a :2 14 26 :2 30 3c 3f
4f 53 54 :2 4f :2 26 :3 a :2 10 15
21 62 65 :2 21 :2 a 57 :3 7 e
7 4 26 22 b 14 1f :2 b
24 22 :2 2d :2 22 a 4e 11 1c
:2 4e 21 1f :2 2a :2 1f 4d :2 a 7
c 23 25 34 40 42 4f 5c
65 74 7d 7f 91 c 20 24
2e d 10 1d b e 1c 1e
10 1c 7 1c a :2 14 26 :2 30
3c 3f 4f 53 54 :2 4f :2 26 :3 a
:2 10 15 21 62 65 :2 21 :2 a 57
:3 7 e 7 48 22 a :2 14 26
:2 30 3c 3f 4f 53 54 :2 4f :2 26
:3 a :2 10 15 21 62 65 :2 21 72
75 :2 21 79 7c :2 21 86 89 92
9a :2 a3 :2 92 b4 :2 89 :2 21 bc bf
:2 21 c7 ca :2 21 cf d2 :2 21 d5
d8 :2 21 :2 a 57 :3 7 :2 12 24 :2 2d
3d 41 :2 7 :4 4 1 9 12 9
a :2 14 26 :2 30 3c 40 50 54
55 :2 50 :2 26 :3 a :2 10 15 21 62
65 :2 21 :3 a :2 10 15 39 5c 5f
:2 68 :2 39 76 79 :2 39 81 84 :2 8d
:2 39 :2 a 58 :3 7 c 15 1e 2c
a 13 1e f e 7 e 7
e 7 29 :2 4 :2 9 7 :2 10 1c
:2 7 :2 11 23 :2 2d 39 3d 4d 51
52 :2 4d :2 23 :3 7 :2 d 12 1e 5f
62 :2 1e :3 7 :2 d 12 36 42 45
:2 36 4d 50 :2 36 5c 5f 66 6f
72 :2 5f :2 36 :2 7 55 :2 4 7 c
1b a 13 1e f e 7 e
7 e 7 :2 4 :4 10 :2 4 :8 1 
2055
4
0 :2 1 :4 3 :4 4
:4 5 2 :4 6 :2 1
:7 8 :7 9 :7 a :7 b
:7 c :7 d :6 e :6 f
:6 10 :6 11 :6 12 :6 13
:7 16 :7 19 :7 1c :7 1f
:7 22 :7 25 :7 28 :7 2b
:7 2e :7 31 :7 34 :7 37
:7 3a :7 3d :7 40 :7 42
:b 44 :11 45 46 :2 45
:2 46 :2 45 :2 46 :2 45
:2 47 :2 45 :9 47 :2 45
:2 48 :2 45 :2 48 :2 45
:2 48 :2 45 :2 48 :2 45
:2 48 :4 45 :3 44 :11 4b
:b 4c :3 4b :5 51 :11 52
:30 53 :3 52 :11 55 :b 56
:3 55 :a 58 :3 51 :3 5d
:3 5e :3 5f :16 62 :9 64
:4 65 :b 66 64 :5 67
63 69 :5 6a :3 69
:2 6b :5 6d :11 6e :24 6f
:3 6e :2 6c :6 6b 68
:3 62 :a 75 :7 77 :3 78
:b 79 77 :5 7a 76
7c :5 7d :3 7c :2 7e
:5 80 :11 81 :24 82 :3 81
:2 7f :6 7e 7b :3 75
:a 88 :5 8a :5 8b 89
8d :5 8e :3 8d :2 8f
:5 91 :11 92 :24 93 :3 92
:2 90 :6 8f 8c :3 88
:a 98 :11 99 :30 9a :3 99
:11 9c :b 9d :3 9c :a 9f
:3 98 :3 88 :3 75 a3
62 :9 a3 :2 a5 :3 a6
a5 :5 a7 a4 a9
:5 aa :3 a9 :2 ab :5 ad
:11 ae :24 af :3 ae :2 ac
:6 ab a8 :3 a3 :a b4
:11 b5 :30 b6 :3 b5 :11 b8
:b b9 :3 b8 :a bb :3 b4
:5 bf :3 c0 bf :5 c1
be c3 :5 c4 :3 c3
:2 c5 :5 c7 :11 c8 :24 c9
:3 c8 :2 c6 :6 c5 c2
:3 a3 cd a3 62
:9 cd :6 cf :5 d0 ce
d2 :5 d3 :3 d2 :2 d4
:5 d6 :11 d7 :24 d8 :3 d7
:2 d5 :6 d4 d1 :3 cd
:a dd :11 de :30 df :3 de
:11 e1 :b e2 :3 e1 :a e4
:3 dd :3 e7 e8 cd
62 :18 e8 :c e9 :2 e8
e9 :2 ea :2 e9 :6 ea
e9 :2 e8 :5 ec :3 ed
ec :5 ee eb f0
:5 f1 :3 f0 :2 f2 :5 f4
:11 f5 :24 f6 :3 f5 :2 f3
:6 f2 ef :3 ea :a fb
:11 fc :30 fd :3 fc :11 ff
:b 100 :3 ff :a 102 :3 fb
ea :3 62 :9 108 :5 10a
:5 10b 10e 110 111
112 113 114 115
116 :2 119 :2 11a :2 11b
:2 11c :2 11d :2 11e :2 11f
:9 120 :3 121 10e :6 122
:5 123 122 :5 125 124
:3 122 10d 128 :5 129
:3 128 :2 12a :5 12c :11 12d
:24 12e :3 12d :2 12b :6 12a
127 :3 108 :5 134 :5 135
:2 138 139 13a :3 13b
138 :6 13c :5 13d 13c
:5 13f 13e :3 13c 137
142 :5 143 :3 142 :2 144
:5 146 :11 147 :24 148 :3 147
:2 145 :6 144 141 :3 108
:c 14d :e 14e :13 14f :9 150
:7 151 :7 152 :9 153 :9 157
:5 158 :9 159 :a 15a :9 15b
:9 15c :9 15d :9 15e :9 162
:5 163 :9 164 :a 165 :9 166
:7 167 :9 168 :9 169 :9 16a
:9 16e :5 16f :9 170 :a 171
:9 172 :7 173 :9 174 :9 175
:9 176 :3 177 :3 14d :11 178
:b 179 :3 178 :3 17b 17c
108 :16 17c :5 17e :5 17f
:3 182 :2 184 :3 185 :e 186
:7 187 :9 188 182 :6 189
:5 18a 189 :5 18c 18b
:3 189 181 18f :5 190
:3 18f :2 191 :5 193 :11 194
:24 195 :3 194 :2 192 :6 191
18e :3 17c :7 19a :3 19d
:4 19e :b 19f :7 1a0 :7 1a1
:9 1a2 :5 1a3 19d :6 1a4
:5 1a5 1a4 :5 1a7 1a6
:3 1a4 19c 1aa :5 1ab
:3 1aa :2 1ac :5 1ae :11 1af
:24 1b0 :3 1af :2 1ad :6 1ac
1a9 :3 17c :d 1b5 :5 1b6
:b 1b7 :6 1b8 :9 1b9 :5 1bd
:5 1be :9 1bf :8 1c0 1c1
1c2 :3 1b5 :11 1c3 :b 1c4
:3 1c3 :3 1c6 1c7 17c
108 :9 1c7 :9 1c8 :e 1c9
:f 1ca :16 1cb :14 1cc :2 1cd
:e 1ce :12 1cf :c 1d0 :9 1d1
:6 1d2 :3 1c8 :11 1d3 :b 1d4
:3 1d3 :3 1d6 1d7 1c7
108 :9 1d7 :c 1d8 :3 1d9
:8 1db :5 1dc 1da 1de
:5 1df :3 1de :2 1e0 :5 1e2
:11 1e3 :24 1e4 :3 1e3 :2 1e1
:6 1e0 1dd :3 1d8 :9 1e9
:3 1ea 1e9 :3 1ec :3 1ee
:6 1ef 1ee :5 1f0 1ed
1f2 :5 1f3 :3 1f2 :2 1f4
:5 1f6 :11 1f7 :24 1f8 :3 1f7
:2 1f5 :6 1f4 1f1 :3 1eb
:9 1fd :3 1ff :5 200 :8 201
:6 202 :6 203 1ff :5 204
1fe 206 :5 207 :3 206
:2 208 :5 20a :11 20b :24 20c
:3 20b :2 209 :6 208 205
:3 1fd :9 211 :3 212 211
:3 214 213 :3 211 1fd
:3 217 216 :3 1fd 1eb
:3 1e9 :11 21f :a 220 :c 221
:3 222 :c 223 :5 224 :a 225
:a 226 :5 227 :6 228 :d 229
:b 22a :8 22b :5 22c :5 22d
:9 22e :5 22f :2 230 :d 231
:9 232 :6 233 :9 235 :a 236
:9 237 :5 238 :c 239 :16 23a
:a 23b :7 23c :9 23d :9 23e
23f :a 240 :6 241 :a 242
:6 243 :9 244 :11 245 :a 246
:b 247 :17 248 :a 249 :7 24a
:8 24b :8 24c :d 24d :2 24e
:3 24f :2 250 :5 251 :5 252
:b 253 :9 254 :c 255 :9 256
:9 257 :9 258 :9 259 :8 25a
:2 25b :3 21f :11 25c :b 25d
:3 25c :3 25f 1d8 :b 263
:a 264 :a 265 :a 266 :9 267
:4 268 :4 269 :4 26a :5 26b
:8 26c :10 26d :a 26e :6 26f
:6 270 :a 271 :6 272 :9 273
:11 274 :a 275 :b 276 :17 277
:b 278 :a 279 :f 27a :2 27b
:3 27c :2 27d :5 27e :5 27f
:b 280 :9 281 :c 282 :9 283
:9 284 :9 285 :9 286 :5 287
:2 288 :3 263 :11 289 :b 28a
:3 289 :3 28c 260 :3 1d8
28e 1d7 108 :9 28e
:5 290 :5 291 294 296
297 298 299 29a
29b 29c 29d 29e
2a1 2a2 :3 2a3 :3 2a4
:3 2a5 :3 2a6 :3 2a7 :3 2a8
:3 2a9 :6 2aa 294 :6 2ab
:5 2ac 2ab :5 2ae 2ad
:3 2ab 293 2b1 :5 2b2
:3 2b1 :2 2b3 :5 2b5 :11 2b6
:24 2b7 :3 2b6 :2 2b4 :6 2b3
2b0 :3 28e :7 2bc :8 2bf
:2 2c0 :4 2c1 :e 2c2 :12 2c3
:7 2c4 2bf :6 2c5 :5 2c6
2c5 :5 2c8 2c7 :3 2c5
2be 2cb :5 2cc :3 2cb
:2 2cd :5 2cf :11 2d0 :24 2d1
:3 2d0 :2 2ce :6 2cd 2ca
:3 28e :c 2d6 :11 2d7 :a 2d8
:b 2d9 :5 2dd :a 2de :9 2df
:2 2e0 :3 2d6 :11 2e1 :b 2e2
:3 2e1 :3 2e4 2e5 28e
108 :9 2e5 :5 2e7 :5 2e8
2eb 2ed 2ee 2ef
2f0 2f1 2f2 2f3
2f4 2f5 2f8 2f9
:3 2fa :3 2fb :3 2fc :3 2fd
:3 2fe :3 2ff :3 300 :6 301
2eb :6 302 :5 303 302
:5 305 304 :3 302 2ea
308 :5 309 :3 308 :2 30a
:5 30c :11 30d :24 30e :3 30d
:2 30b :6 30a 307 :3 2e5
:7 313 :b 316 :3 317 :4 318
:e 319 :12 31b :7 31c :6 31e
:2 31f :4 320 :4 321 :7 322
:4 323 :4 324 :a 325 316
:6 326 :5 327 326 :5 329
328 :3 326 315 32c
:5 32d :3 32c :2 32e :5 330
:11 331 :24 332 :3 331 :2 32f
:6 32e 32b :3 2e5 :c 337
:11 338 :a 339 :b 33a :5 33c
:a 33d :9 33e :3 33f :5 341
:3 342 :9 343 :7 344 345
:2 346 :2 347 :3 337 :11 348
:b 349 :3 348 :3 34b 38e
2e5 108 :9 38e :d 38f
:10 390 :a 391 :a 392 :a 393
:8 394 :6 395 :6 396 :c 397
:a 398 :a 399 :7 39a :9 39b
:a 39d :e 39e :6 39f :8 3a0
:a 3a1 :8 3a2 :6 3a3 :6 3a4
:9 3a5 :7 3a6 :a 3a7 :5 3a8
:2 3a9 :3 38f :11 3aa :b 3ab
:3 3aa :3 3ad 3ae 38e
108 :9 3ae :5 3b0 :5 3b1
3b4 3b5 3b6 3b7
3b8 3b9 3ba 3bb
3bc 3bd 3be 3bf
3c2 3c3 :3 3c4 :3 3c5
:3 3c6 :4 3c7 :3 3c8 :3 3c9
:3 3ca :4 3cb 3cc :4 3cd
:2 3ce 3b4 :6 3cf :5 3d0
3cf :5 3d2 3d1 :3 3cf
3b3 3d5 :5 3d6 :3 3d5
:2 3d7 :5 3d9 :11 3da :24 3db
:3 3da :2 3d8 :6 3d7 3d4
:3 3ae :5 3e2 :5 3e3 3e6
3e8 3e9 3ea 3eb
3ec 3ed 3ee 3ef
3f0 3f1 3f2 3f3
3f4 3f5 3f6 3f7
3f8 3f9 3fa 3fb
3fc 3fd 3fe 3ff
400 401 402 403
404 405 406 :8 409
:8 40a :8 40b :6 40c :8 40d
:6 40e :6 40f :8 410 :9 411
:b 412 :3 413 3e6 :6 414
:5 415 414 :5 417 416
:3 414 3e5 41a :5 41b
:3 41a :2 41c :5 41e :11 41f
:24 420 :3 41f :2 41d :6 41c
419 :3 3ae :4 427 :c 428
427 :6 429 :5 42a 429
:5 42c 42b :3 429 426
42f :5 430 :3 42f :2 431
:5 433 :11 434 :24 435 :3 434
:2 432 :6 431 42e :3 3ae
:d 43a :11 43b :a 43c :13 43d
:2 43e :8 43f :6 440 :6 441
:a 442 :b 443 :9 445 :11 446
:9 447 :13 448 :2 449 :8 44a
:6 44b :6 44c :a 44d :b 44e
:7 44f :7 450 :3 43a :11 451
:b 452 :3 451 :3 454 455
3ae 108 :b 455 :6 457
:5 458 456 45a :5 45b
:3 45a :2 45c :5 45e :11 45f
:24 460 :3 45f :2 45d :6 45c
459 :3 455 :a 465 :c 466
:c 467 :2 468 :7 469 :7 46a
:7 46b :3 46c :8 46e :e 46f
:6 470 :b 471 :c 472 :8 474
:e 475 :6 476 :b 477 :c 478
:2 479 :3 466 :11 47a :b 47b
:3 47a :3 47d :3 465 47f
455 108 :9 47f :b 480
:a 481 :9 482 :5 483 :a 484
:e 485 :4 486 :5 487 :9 488
489 :a 48a :6 48b :a 48c
:6 48d :9 48e :11 48f :a 490
:b 491 :17 492 :a 493 :4 494
:8 495 :8 496 :c 497 :f 498
:2 499 :3 49a :2 49b :5 49c
:5 49d :c 49e :9 49f :c 4a0
:9 4a1 :9 4a2 :9 4a3 :9 4a4
:2 4a5 :3 480 :11 4a6 :b 4a7
:3 4a6 :3 4a9 4aa 47f
108 :9 4aa :7 4ab :a 4ac
:9 4ad :d 4ae :5 4af :5 4b0
:5 4b1 :5 4b2 :4 4b3 :7 4b4
:e 4b5 :4 4b6 :a 4b7 4b8
:3 4b9 :5 4ba :4 4bb :6 4bc
4bd :2 4be :9 4bf :c 4c0
:5 4c1 :2 4c2 :3 4ab :11 4c3
:b 4c4 :3 4c3 :3 4c6 4c7
4aa 108 :9 4c7 :5 4cb
:5 4cc :4 4cf :3 4d0 4d1
:2 4d2 4d3 :3 4d4 :2 4d5
4cf :6 4d6 :5 4d7 4d6
:5 4d9 4d8 :3 4d6 4ce
4dc :5 4dd :3 4dc :2 4de
:5 4e0 :11 4e1 :24 4e2 :3 4e1
:2 4df :6 4de 4db :3 4c7
:5 4e8 :5 4e9 :4 4ec :3 4ed
4ee :2 4ef :2 4f0 :2 4f1
4ec :6 4f2 :5 4f3 4f2
:5 4f5 4f4 :3 4f2 4eb
4f8 :5 4f9 :3 4f8 :2 4fa
:5 4fc :11 4fd :24 4fe :3 4fd
:2 4fb :6 4fa 4f7 :3 4c7
:5 504 :5 505 :4 508 :6 509
:b 50a :6 50c :b 50d 508
:6 50e :5 50f 50e :5 511
510 :3 50e 507 514
:5 515 :3 514 :2 516 :5 518
:11 519 :24 51a :3 519 :2 517
:6 516 513 :3 4c7 :d 51f
:b 520 :9 521 :a 525 :b 526
:7 527 :7 528 :9 529 :a 52d
:4 52e :3 52f :4 530 :8 531
:2 532 :3 51f :11 533 :b 534
:3 533 :3 536 537 4c7
108 :9 537 :b 53b :5 53c
:4 53d :2 53e :3 53f 540
:3 53b :11 541 :b 542 :3 541
:3 544 545 537 108
:d 545 :2 546 :2 545 :6 546
:3 545 :e 54a :4 54b :3 54c
:4 54d 54e :3 54a :11 54f
:b 550 :3 54f :3 552 546
108 :11 554 :30 555 :3 554
:a 557 553 :3 108 43
:3 55c :11 55d :b 55e :17 55f
:3 55d :2 561 :3 562 :3 563
564 562 561 562
:3 565 :3 55c :2 566 :5 568
:11 569 :b 56a :18 56b :3 569
:2 56d 56e :3 56f 570
56e 56d 56e :3 571
:2 567 :6 566 55b :7 1

783a
4
:3 0 1 :3 0 2
:a 0 2050 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
:2 0 7 6 :3 0
5 :7 0 9 8
:3 0 4 :3 0 7
:7 0 d c :3 0
8 :3 0 9 :3 0
a :2 0 4 11
12 0 f 13
0 2050 2 14
:2 0 e :2 0 f
4 :3 0 c :2 0
d 17 19 :6 0
1c 1a 0 204e
0 b :6 0 e
:2 0 13 4 :3 0
11 1e 20 :6 0
23 21 0 204e
0 d :6 0 e
:2 0 17 4 :3 0
15 25 27 :6 0
2a 28 0 204e
0 f :6 0 12
:2 0 1b 4 :3 0
19 2c 2e :6 0
31 2f 0 204e
0 10 :6 0 14
:2 0 1f 4 :3 0
1d 33 35 :6 0
38 36 0 204e
0 11 :6 0 16
:2 0 23 4 :3 0
21 3a 3c :6 0
3f 3d 0 204e
0 13 :6 0 16
:2 0 25 6 :3 0
41 :7 0 45 42
43 204e 0 15
:6 0 16 :2 0 27
6 :3 0 47 :7 0
4b 48 49 204e
0 17 :6 0 16
:2 0 29 6 :3 0
4d :7 0 51 4e
4f 204e 0 18
:6 0 16 :2 0 2b
6 :3 0 53 :7 0
57 54 55 204e
0 19 :6 0 16
:2 0 2d 6 :3 0
59 :7 0 5d 5a
5b 204e 0 1a
:6 0 65 66 0
2f 6 :3 0 5f
:7 0 63 60 61
204e 0 1b :6 0
6c 6d 0 31
9 :3 0 a :2 0
4 67 :7 0 6a
68 0 204e 0
1c :6 0 73 74
0 33 9 :3 0
a :2 0 4 6e
:7 0 71 6f 0
204e 0 1d :6 0
7a 7b 0 35
9 :3 0 a :2 0
4 75 :7 0 78
76 0 204e 0
1e :6 0 81 82
0 37 9 :3 0
a :2 0 4 7c
:7 0 7f 7d 0
204e 0 1f :6 0
88 89 0 39
9 :3 0 a :2 0
4 83 :7 0 86
84 0 204e 0
20 :6 0 8f 90
0 3b 9 :3 0
a :2 0 4 8a
:7 0 8d 8b 0
204e 0 21 :6 0
96 97 0 3d
9 :3 0 a :2 0
4 91 :7 0 94
92 0 204e 0
22 :6 0 9d 9e
0 3f 9 :3 0
a :2 0 4 98
:7 0 9b 99 0
204e 0 23 :6 0
a4 a5 0 41
9 :3 0 a :2 0
4 9f :7 0 a2
a0 0 204e 0
24 :6 0 ab ac
0 43 9 :3 0
a :2 0 4 a6
:7 0 a9 a7 0
204e 0 25 :6 0
b2 b3 0 45
9 :3 0 a :2 0
4 ad :7 0 b0
ae 0 204e 0
26 :6 0 b9 ba
0 47 9 :3 0
a :2 0 4 b4
:7 0 b7 b5 0
204e 0 27 :6 0
c0 c1 0 49
9 :3 0 a :2 0
4 bb :7 0 be
bc 0 204e 0
28 :6 0 c7 c8
0 4b 9 :3 0
a :2 0 4 c2
:7 0 c5 c3 0
204e 0 29 :6 0
ce cf 0 4d
9 :3 0 a :2 0
4 c9 :7 0 cc
ca 0 204e 0
2a :6 0 d4 d5
0 4f 9 :3 0
a :2 0 4 d0
:7 0 d3 d1 0
204e 0 2b :6 0
2c :3 0 2d :3 0
2c :3 0 2e :3 0
d7 d8 0 2
:4 0 51 d9 db
d6 dd dc :2 0
2f :3 0 30 :3 0
df e0 0 31
:4 0 2c :3 0 32
:3 0 e3 e4 0
2 :4 0 53 e5
e7 33 :4 0 34
:2 0 35 :3 0 55
ea ec :3 0 36
:4 0 34 :2 0 37
:4 0 58 ef f1
:3 0 34 :2 0 3
:3 0 5b f3 f5
:3 0 34 :2 0 37
:4 0 5e f7 f9
:3 0 34 :2 0 38
:4 0 61 fb fd
:3 0 34 :2 0 39
:3 0 3a :3 0 5
:3 0 64 101 103
3b :4 0 66 100
106 69 ff 108
:3 0 34 :2 0 38
:4 0 6c 10a 10c
:3 0 34 :2 0 37
:4 0 6f 10e 110
:3 0 34 :2 0 7
:3 0 72 112 114
:3 0 34 :2 0 37
:4 0 75 116 118
:3 0 34 :2 0 3c
:4 0 78 11a 11c
:3 0 7b e1 11e
:2 0 120 80 121
de 120 0 122
82 0 1fae 2c
:3 0 2d :3 0 123
124 0 2c :3 0
3d :3 0 126 127
0 14 :2 0 2
:4 0 3e :2 0 3f
:2 0 14 :2 0 84
12c 12e :3 0 87
128 130 125 132
131 :2 0 2f :3 0
40 :3 0 134 135
0 31 :4 0 41
:4 0 34 :2 0 35
:3 0 8b 139 13b
:3 0 8e 136 13d
:2 0 13f 91 140
133 13f 0 141
93 0 1fae 3
:3 0 42 :2 0 95
143 144 :3 0 145
:2 0 2c :3 0 2d
:3 0 147 148 0
2c :3 0 3d :3 0
14a 14b 0 e
:2 0 2 :4 0 43
:2 0 3f :2 0 14
:2 0 97 150 152
:3 0 9a 14c 154
149 156 155 :2 0
2f :3 0 40 :3 0
158 159 0 31
:4 0 44 :4 0 34
:2 0 35 :3 0 9e
15d 15f :3 0 34
:2 0 45 :4 0 a1
161 163 :3 0 34
:2 0 46 :4 0 a4
165 167 :3 0 34
:2 0 39 :3 0 3a
:3 0 47 :3 0 48
:3 0 16c 16d 0
a7 16b 16f 3b
:4 0 a9 16a 172
ac 169 174 :3 0
34 :2 0 49 :4 0
af 176 178 :3 0
34 :2 0 37 :4 0
b2 17a 17c :3 0
34 :2 0 3 :3 0
b5 17e 180 :3 0
34 :2 0 37 :4 0
b8 182 184 :3 0
bb 15a 186 :2 0
188 be 189 157
188 0 18a c0
0 1b4 2c :3 0
2d :3 0 18b 18c
0 2c :3 0 3d
:3 0 18e 18f 0
14 :2 0 2 :4 0
4a :2 0 3f :2 0
14 :2 0 c2 194
196 :3 0 c5 190
198 18d 19a 199
:2 0 2f :3 0 40
:3 0 19c 19d 0
31 :4 0 4b :4 0
34 :2 0 35 :3 0
c9 1a1 1a3 :3 0
cc 19e 1a5 :2 0
1a7 cf 1a8 19b
1a7 0 1a9 d1
0 1b4 4c :3 0
4d :3 0 1aa 1ab
0 47 :3 0 48
:3 0 1ad 1ae 0
3 :3 0 4e :3 0
d3 1ac 1b2 :2 0
1b4 d7 1b5 146
1b4 0 1b6 db
0 1fae d :3 0
3 :3 0 1b7 1b8
0 1fae 15 :3 0
5 :3 0 1ba 1bb
0 1fae f :3 0
7 :3 0 1bd 1be
0 1fae 39 :3 0
15 :3 0 16 :2 0
dd 1c0 1c3 4f
:2 0 50 :2 0 e2
1c5 1c7 :3 0 1c8
:2 0 39 :3 0 15
:3 0 16 :2 0 e5
1ca 1cd 4f :2 0
51 :2 0 ea 1cf
1d1 :3 0 1d2 :2 0
1c9 1d4 1d3 :2 0
52 :3 0 53 :3 0
52 :3 0 54 :3 0
39 :3 0 55 :3 0
56 :3 0 52 :3 0
57 :3 0 10 :3 0
18 :3 0 17 :3 0
1b :3 0 58 :3 0
55 :3 0 58 :3 0
52 :3 0 52 :3 0
59 :3 0 d :3 0
55 :3 0 59 :3 0
52 :3 0 53 :4 0
5a 1 :8 0 1f4
9 :3 0 5b :3 0
1ef 1f0 0 5c
:3 0 1f1 1f2 0
1f4 ed 247 5d
:3 0 9 :3 0 5b
:3 0 1f6 1f7 0
5c :3 0 1f8 1f9
0 1fb f0 1fd
f2 1fc 1fb :2 0
245 5e :3 0 9
:3 0 5b :3 0 200
201 0 5c :3 0
202 203 0 23d
2c :3 0 2d :3 0
205 206 0 2c
:3 0 3d :3 0 208
209 0 5f :2 0
2 :4 0 60 :2 0
3f :2 0 14 :2 0
f4 20e 210 :3 0
f7 20a 212 207
214 213 :2 0 2f
:3 0 40 :3 0 216
217 0 31 :4 0
61 :4 0 34 :2 0
35 :3 0 fb 21b
21d :3 0 34 :2 0
45 :4 0 fe 21f
221 :3 0 34 :2 0
62 :4 0 101 223
225 :3 0 34 :2 0
5c :3 0 104 227
229 :3 0 34 :2 0
63 :4 0 107 22b
22d :3 0 34 :2 0
64 :3 0 65 :3 0
14 :2 0 66 :2 0
10a 230 234 10e
22f 236 :3 0 111
218 238 :2 0 23a
114 23b 215 23a
0 23c 116 0
23d 118 240 :3 0
240 0 240 23f
23d 23e :6 0 242
2 :3 0 11b 244
11d 243 242 :2 0
245 11f :2 0 247
0 247 246 1f4
245 :6 0 3ad 1
:3 0 9 :3 0 5b
:3 0 249 24a 0
9 :3 0 4f :2 0
67 :3 0 24c 24e
0 124 24d 250
:3 0 251 :2 0 52
:3 0 53 :3 0 52
:3 0 54 :3 0 39
:3 0 55 :3 0 56
:3 0 10 :3 0 18
:3 0 17 :3 0 58
:3 0 55 :3 0 68
:3 0 52 :3 0 52
:3 0 59 :3 0 d
:3 0 55 :3 0 59
:3 0 52 :3 0 53
:4 0 69 1 :8 0
26e 9 :3 0 5b
:3 0 269 26a 0
5c :3 0 26b 26c
0 26e 127 2c1
5d :3 0 9 :3 0
5b :3 0 270 271
0 5c :3 0 272
273 0 275 12a
277 12c 276 275
:2 0 2bf 5e :3 0
9 :3 0 5b :3 0
27a 27b 0 5c
:3 0 27c 27d 0
2b7 2c :3 0 2d
:3 0 27f 280 0
2c :3 0 3d :3 0
282 283 0 5f
:2 0 2 :4 0 6a
:2 0 3f :2 0 14
:2 0 12e 288 28a
:3 0 131 284 28c
281 28e 28d :2 0
2f :3 0 40 :3 0
290 291 0 31
:4 0 6b :4 0 34
:2 0 35 :3 0 135
295 297 :3 0 34
:2 0 45 :4 0 138
299 29b :3 0 34
:2 0 62 :4 0 13b
29d 29f :3 0 34
:2 0 5c :3 0 13e
2a1 2a3 :3 0 34
:2 0 63 :4 0 141
2a5 2a7 :3 0 34
:2 0 64 :3 0 65
:3 0 14 :2 0 66
:2 0 144 2aa 2ae
148 2a9 2b0 :3 0
14b 292 2b2 :2 0
2b4 14e 2b5 28f
2b4 0 2b6 150
0 2b7 152 2ba
:3 0 2ba 0 2ba
2b9 2b7 2b8 :6 0
2bc 4 :3 0 155
2be 157 2bd 2bc
:2 0 2bf 159 :2 0
2c1 0 2c1 2c0
26e 2bf :6 0 3a9
1 :3 0 9 :3 0
5b :3 0 2c3 2c4
0 9 :3 0 4f
:2 0 67 :3 0 2c6
2c8 0 15e 2c7
2ca :3 0 2cb :2 0
17 :3 0 6c :3 0
59 :3 0 d :4 0
6d 1 :8 0 2d7
9 :3 0 5b :3 0
2d2 2d3 0 5c
:3 0 2d4 2d5 0
2d7 161 32a 5d
:3 0 9 :3 0 5b
:3 0 2d9 2da 0
5c :3 0 2db 2dc
0 2de 164 2e0
166 2df 2de :2 0
328 5e :3 0 9
:3 0 5b :3 0 2e3
2e4 0 5c :3 0
2e5 2e6 0 320
2c :3 0 2d :3 0
2e8 2e9 0 2c
:3 0 3d :3 0 2eb
2ec 0 5f :2 0
2 :4 0 6e :2 0
3f :2 0 14 :2 0
168 2f1 2f3 :3 0
16b 2ed 2f5 2ea
2f7 2f6 :2 0 2f
:3 0 40 :3 0 2f9
2fa 0 31 :4 0
6f :4 0 34 :2 0
35 :3 0 16f 2fe
300 :3 0 34 :2 0
45 :4 0 172 302
304 :3 0 34 :2 0
62 :4 0 175 306
308 :3 0 34 :2 0
5c :3 0 178 30a
30c :3 0 34 :2 0
63 :4 0 17b 30e
310 :3 0 34 :2 0
64 :3 0 65 :3 0
14 :2 0 66 :2 0
17e 313 317 182
312 319 :3 0 185
2fb 31b :2 0 31d
188 31e 2f8 31d
0 31f 18a 0
320 18c 323 :3 0
323 0 323 322
320 321 :6 0 325
6 :3 0 18f 327
191 326 325 :2 0
328 193 :2 0 32a
0 32a 329 2d7
328 :6 0 3a6 1
:3 0 9 :3 0 5b
:3 0 32c 32d 0
9 :3 0 4f :2 0
67 :3 0 32f 331
0 198 330 333
:3 0 334 :2 0 2c
:3 0 2d :3 0 336
337 0 2c :3 0
3d :3 0 339 33a
0 e :2 0 2
:4 0 70 :2 0 3f
:2 0 14 :2 0 19b
33f 341 :3 0 19e
33b 343 338 345
344 :2 0 2f :3 0
40 :3 0 347 348
0 31 :4 0 71
:4 0 34 :2 0 35
:3 0 1a2 34c 34e
:3 0 34 :2 0 45
:4 0 1a5 350 352
:3 0 34 :2 0 46
:4 0 1a8 354 356
:3 0 34 :2 0 39
:3 0 3a :3 0 47
:3 0 48 :3 0 35b
35c 0 1ab 35a
35e 3b :4 0 1ad
359 361 1b0 358
363 :3 0 34 :2 0
49 :4 0 1b3 365
367 :3 0 34 :2 0
37 :4 0 1b6 369
36b :3 0 34 :2 0
d :3 0 1b9 36d
36f :3 0 34 :2 0
37 :4 0 1bc 371
373 :3 0 1bf 349
375 :2 0 377 1c2
378 346 377 0
379 1c4 0 3a3
2c :3 0 2d :3 0
37a 37b 0 2c
:3 0 3d :3 0 37d
37e 0 14 :2 0
2 :4 0 72 :2 0
3f :2 0 14 :2 0
1c6 383 385 :3 0
1c9 37f 387 37c
389 388 :2 0 2f
:3 0 40 :3 0 38b
38c 0 31 :4 0
73 :4 0 34 :2 0
35 :3 0 1cd 390
392 :3 0 1d0 38d
394 :2 0 396 1d3
397 38a 396 0
398 1d5 0 3a3
4c :3 0 4d :3 0
399 39a 0 47
:3 0 48 :3 0 39c
39d 0 d :3 0
4e :3 0 1d7 39b
3a1 :2 0 3a3 1db
3a4 335 3a3 0
3a5 1df 0 3a6
1e1 3a7 2cc 3a6
0 3a8 1e4 0
3a9 1e6 3aa 252
3a9 0 3ab 1e9
0 3ad 74 :3 0
1eb 6f3 39 :3 0
15 :3 0 16 :2 0
1ee 3ae 3b1 4f
:2 0 75 :2 0 1f3
3b3 3b5 :3 0 53
:3 0 10 :3 0 76
:3 0 77 :3 0 d
:4 0 78 1 :8 0
3c2 9 :3 0 5b
:3 0 3bd 3be 0
5c :3 0 3bf 3c0
0 3c2 1f6 415
5d :3 0 9 :3 0
5b :3 0 3c4 3c5
0 5c :3 0 3c6
3c7 0 3c9 1f9
3cb 1fb 3ca 3c9
:2 0 413 5e :3 0
9 :3 0 5b :3 0
3ce 3cf 0 5c
:3 0 3d0 3d1 0
40b 2c :3 0 2d
:3 0 3d3 3d4 0
2c :3 0 3d :3 0
3d6 3d7 0 5f
:2 0 2 :4 0 79
:2 0 3f :2 0 14
:2 0 1fd 3dc 3de
:3 0 200 3d8 3e0
3d5 3e2 3e1 :2 0
2f :3 0 40 :3 0
3e4 3e5 0 31
:4 0 7a :4 0 34
:2 0 35 :3 0 204
3e9 3eb :3 0 34
:2 0 45 :4 0 207
3ed 3ef :3 0 34
:2 0 62 :4 0 20a
3f1 3f3 :3 0 34
:2 0 5c :3 0 20d
3f5 3f7 :3 0 34
:2 0 63 :4 0 210
3f9 3fb :3 0 34
:2 0 64 :3 0 65
:3 0 14 :2 0 66
:2 0 213 3fe 402
217 3fd 404 :3 0
21a 3e6 406 :2 0
408 21d 409 3e3
408 0 40a 21f
0 40b 221 40e
:3 0 40e 0 40e
40d 40b 40c :6 0
410 8 :3 0 224
412 226 411 410
:2 0 413 228 :2 0
415 0 415 414
3c2 413 :6 0 4f5
1 :3 0 9 :3 0
5b :3 0 417 418
0 9 :3 0 4f
:2 0 67 :3 0 41a
41c 0 22d 41b
41e :3 0 41f :2 0
2c :3 0 2d :3 0
421 422 0 2c
:3 0 3d :3 0 424
425 0 e :2 0
2 :4 0 7b :2 0
3f :2 0 14 :2 0
230 42a 42c :3 0
233 426 42e 423
430 42f :2 0 2f
:3 0 40 :3 0 432
433 0 31 :4 0
7c :4 0 34 :2 0
35 :3 0 237 437
439 :3 0 34 :2 0
45 :4 0 23a 43b
43d :3 0 34 :2 0
46 :4 0 23d 43f
441 :3 0 34 :2 0
39 :3 0 3a :3 0
47 :3 0 48 :3 0
446 447 0 240
445 449 3b :4 0
242 444 44c 245
443 44e :3 0 34
:2 0 49 :4 0 248
450 452 :3 0 34
:2 0 37 :4 0 24b
454 456 :3 0 34
:2 0 3 :3 0 24e
458 45a :3 0 34
:2 0 37 :4 0 251
45c 45e :3 0 254
434 460 :2 0 462
257 463 431 462
0 464 259 0
48e 2c :3 0 2d
:3 0 465 466 0
2c :3 0 3d :3 0
468 469 0 14
:2 0 2 :4 0 7d
:2 0 3f :2 0 14
:2 0 25b 46e 470
:3 0 25e 46a 472
467 474 473 :2 0
2f :3 0 40 :3 0
476 477 0 31
:4 0 7e :4 0 34
:2 0 35 :3 0 262
47b 47d :3 0 265
478 47f :2 0 481
268 482 475 481
0 483 26a 0
48e 4c :3 0 4d
:3 0 484 485 0
47 :3 0 48 :3 0
487 488 0 3
:3 0 4e :3 0 26c
486 48c :2 0 48e
270 48f 420 48e
0 490 274 0
4f5 39 :3 0 56
:3 0 57 :3 0 17
:3 0 1b :3 0 58
:3 0 59 :3 0 d
:4 0 7f 1 :8 0
49f 9 :3 0 5b
:3 0 49a 49b 0
5c :3 0 49c 49d
0 49f 276 4f2
5d :3 0 9 :3 0
5b :3 0 4a1 4a2
0 5c :3 0 4a3
4a4 0 4a6 279
4a8 27b 4a7 4a6
:2 0 4f0 5e :3 0
9 :3 0 5b :3 0
4ab 4ac 0 5c
:3 0 4ad 4ae 0
4e8 2c :3 0 2d
:3 0 4b0 4b1 0
2c :3 0 3d :3 0
4b3 4b4 0 5f
:2 0 2 :4 0 c
:2 0 3f :2 0 14
:2 0 27d 4b9 4bb
:3 0 280 4b5 4bd
4b2 4bf 4be :2 0
2f :3 0 40 :3 0
4c1 4c2 0 31
:4 0 80 :4 0 34
:2 0 35 :3 0 284
4c6 4c8 :3 0 34
:2 0 45 :4 0 287
4ca 4cc :3 0 34
:2 0 62 :4 0 28a
4ce 4d0 :3 0 34
:2 0 5c :3 0 28d
4d2 4d4 :3 0 34
:2 0 63 :4 0 290
4d6 4d8 :3 0 34
:2 0 64 :3 0 65
:3 0 14 :2 0 66
:2 0 293 4db 4df
297 4da 4e1 :3 0
29a 4c3 4e3 :2 0
4e5 29d 4e6 4c0
4e5 0 4e7 29f
0 4e8 2a1 4eb
:3 0 4eb 0 4eb
4ea 4e8 4e9 :6 0
4ed a :3 0 2a4
4ef 2a6 4ee 4ed
:2 0 4f0 2a8 :2 0
4f2 0 4f2 4f1
49f 4f0 :6 0 4f5
1 :3 0 74 :3 0
2ab 4f6 3b6 4f5
0 6f4 39 :3 0
15 :3 0 16 :2 0
2af 4f7 4fa 4f
:2 0 81 :2 0 2b4
4fc 4fe :3 0 82
:3 0 b :3 0 6c
:3 0 59 :3 0 d
:4 0 83 1 :8 0
50b 9 :3 0 5b
:3 0 506 507 0
5c :3 0 508 509
0 50b 2b7 55e
5d :3 0 9 :3 0
5b :3 0 50d 50e
0 5c :3 0 50f
510 0 512 2ba
514 2bc 513 512
:2 0 55c 5e :3 0
9 :3 0 5b :3 0
517 518 0 5c
:3 0 519 51a 0
554 2c :3 0 2d
:3 0 51c 51d 0
2c :3 0 3d :3 0
51f 520 0 5f
:2 0 2 :4 0 84
:2 0 3f :2 0 14
:2 0 2be 525 527
:3 0 2c1 521 529
51e 52b 52a :2 0
2f :3 0 40 :3 0
52d 52e 0 31
:4 0 85 :4 0 34
:2 0 35 :3 0 2c5
532 534 :3 0 34
:2 0 45 :4 0 2c8
536 538 :3 0 34
:2 0 62 :4 0 2cb
53a 53c :3 0 34
:2 0 5c :3 0 2ce
53e 540 :3 0 34
:2 0 63 :4 0 2d1
542 544 :3 0 34
:2 0 64 :3 0 65
:3 0 14 :2 0 66
:2 0 2d4 547 54b
2d8 546 54d :3 0
2db 52f 54f :2 0
551 2de 552 52c
551 0 553 2e0
0 554 2e2 557
:3 0 557 0 557
556 554 555 :6 0
559 c :3 0 2e5
55b 2e7 55a 559
:2 0 55c 2e9 :2 0
55e 0 55e 55d
50b 55c :6 0 5de
1 :3 0 9 :3 0
5b :3 0 560 561
0 9 :3 0 4f
:2 0 67 :3 0 563
565 0 2ee 564
567 :3 0 568 :2 0
2c :3 0 2d :3 0
56a 56b 0 2c
:3 0 3d :3 0 56d
56e 0 e :2 0
2 :4 0 86 :2 0
3f :2 0 14 :2 0
2f1 573 575 :3 0
2f4 56f 577 56c
579 578 :2 0 2f
:3 0 40 :3 0 57b
57c 0 31 :4 0
87 :4 0 34 :2 0
35 :3 0 2f8 580
582 :3 0 34 :2 0
45 :4 0 2fb 584
586 :3 0 34 :2 0
46 :4 0 2fe 588
58a :3 0 34 :2 0
39 :3 0 3a :3 0
47 :3 0 48 :3 0
58f 590 0 301
58e 592 3b :4 0
303 58d 595 306
58c 597 :3 0 34
:2 0 49 :4 0 309
599 59b :3 0 34
:2 0 37 :4 0 30c
59d 59f :3 0 34
:2 0 3 :3 0 30f
5a1 5a3 :3 0 34
:2 0 37 :4 0 312
5a5 5a7 :3 0 315
57d 5a9 :2 0 5ab
318 5ac 57a 5ab
0 5ad 31a 0
5d7 2c :3 0 2d
:3 0 5ae 5af 0
2c :3 0 3d :3 0
5b1 5b2 0 14
:2 0 2 :4 0 88
:2 0 3f :2 0 14
:2 0 31c 5b7 5b9
:3 0 31f 5b3 5bb
5b0 5bd 5bc :2 0
2f :3 0 40 :3 0
5bf 5c0 0 31
:4 0 89 :4 0 34
:2 0 35 :3 0 323
5c4 5c6 :3 0 326
5c1 5c8 :2 0 5ca
329 5cb 5be 5ca
0 5cc 32b 0
5d7 4c :3 0 4d
:3 0 5cd 5ce 0
47 :3 0 48 :3 0
5d0 5d1 0 3
:3 0 4e :3 0 32d
5cf 5d5 :2 0 5d7
331 5d8 569 5d7
0 5d9 335 0
5de 17 :3 0 8a
:2 0 5da 5db 0
5de 74 :3 0 337
5df 4ff 5de 0
6f4 39 :3 0 15
:3 0 16 :2 0 33b
5e0 5e3 47 :3 0
8b :2 0 8c :3 0
5e5 5e7 0 340
5e6 5e9 :3 0 5ea
:2 0 39 :3 0 15
:3 0 16 :2 0 343
5ec 5ef 8b :2 0
8d :2 0 348 5f1
5f3 :3 0 5f4 :2 0
5eb 5f6 5f5 :2 0
39 :3 0 15 :3 0
16 :2 0 34b 5f8
5fb 47 :3 0 8b
:2 0 8e :3 0 5fd
5ff 0 350 5fe
601 :3 0 602 :2 0
5f7 604 603 :2 0
39 :3 0 15 :3 0
16 :2 0 353 606
609 47 :3 0 8b
:2 0 8f :3 0 60b
60d 0 358 60c
60f :3 0 610 :2 0
605 612 611 :2 0
39 :3 0 56 :3 0
57 :3 0 17 :3 0
1b :3 0 58 :3 0
59 :3 0 3 :4 0
90 1 :8 0 622
9 :3 0 5b :3 0
61d 61e 0 5c
:3 0 61f 620 0
622 35b 675 5d
:3 0 9 :3 0 5b
:3 0 624 625 0
5c :3 0 626 627
0 629 35e 62b
360 62a 629 :2 0
673 5e :3 0 9
:3 0 5b :3 0 62e
62f 0 5c :3 0
630 631 0 66b
2c :3 0 2d :3 0
633 634 0 2c
:3 0 3d :3 0 636
637 0 5f :2 0
2 :4 0 91 :2 0
3f :2 0 14 :2 0
362 63c 63e :3 0
365 638 640 635
642 641 :2 0 2f
:3 0 40 :3 0 644
645 0 31 :4 0
92 :4 0 34 :2 0
35 :3 0 369 649
64b :3 0 34 :2 0
45 :4 0 36c 64d
64f :3 0 34 :2 0
62 :4 0 36f 651
653 :3 0 34 :2 0
5c :3 0 372 655
657 :3 0 34 :2 0
63 :4 0 375 659
65b :3 0 34 :2 0
64 :3 0 65 :3 0
14 :2 0 66 :2 0
378 65e 662 37c
65d 664 :3 0 37f
646 666 :2 0 668
382 669 643 668
0 66a 384 0
66b 386 66e :3 0
66e 0 66e 66d
66b 66c :6 0 670
e :3 0 389 672
38b 671 670 :2 0
673 38d :2 0 675
0 675 674 622
673 :6 0 6f1 1
:3 0 9 :3 0 5b
:3 0 677 678 0
9 :3 0 4f :2 0
67 :3 0 67a 67c
0 392 67b 67e
:3 0 67f :2 0 2c
:3 0 2d :3 0 681
682 0 2c :3 0
3d :3 0 684 685
0 e :2 0 2
:4 0 93 :2 0 3f
:2 0 14 :2 0 395
68a 68c :3 0 398
686 68e 683 690
68f :2 0 2f :3 0
40 :3 0 692 693
0 31 :4 0 94
:4 0 34 :2 0 35
:3 0 39c 697 699
:3 0 34 :2 0 45
:4 0 39f 69b 69d
:3 0 34 :2 0 46
:4 0 3a2 69f 6a1
:3 0 34 :2 0 39
:3 0 3a :3 0 47
:3 0 48 :3 0 6a6
6a7 0 3a5 6a5
6a9 3b :4 0 3a7
6a4 6ac 3aa 6a3
6ae :3 0 34 :2 0
49 :4 0 3ad 6b0
6b2 :3 0 34 :2 0
37 :4 0 3b0 6b4
6b6 :3 0 34 :2 0
3 :3 0 3b3 6b8
6ba :3 0 34 :2 0
37 :4 0 3b6 6bc
6be :3 0 3b9 694
6c0 :2 0 6c2 3bc
6c3 691 6c2 0
6c4 3be 0 6ee
2c :3 0 2d :3 0
6c5 6c6 0 2c
:3 0 3d :3 0 6c8
6c9 0 14 :2 0
2 :4 0 12 :2 0
3f :2 0 14 :2 0
3c0 6ce 6d0 :3 0
3c3 6ca 6d2 6c7
6d4 6d3 :2 0 2f
:3 0 40 :3 0 6d6
6d7 0 31 :4 0
95 :4 0 34 :2 0
35 :3 0 3c7 6db
6dd :3 0 3ca 6d8
6df :2 0 6e1 3cd
6e2 6d5 6e1 0
6e3 3cf 0 6ee
4c :3 0 4d :3 0
6e4 6e5 0 47
:3 0 48 :3 0 6e7
6e8 0 3 :3 0
4e :3 0 3d1 6e6
6ec :2 0 6ee 3d5
6ef 680 6ee 0
6f0 3d9 0 6f1
3db 6f2 613 6f1
0 6f4 1d5 3ad
0 6f4 3de 0
1fae 39 :3 0 15
:3 0 16 :2 0 3e3
6f5 6f8 4f :2 0
16 :2 0 3e8 6fa
6fc :3 0 96 :3 0
97 :3 0 98 :4 0
700 :4 0 701 :2 0
964 9 :3 0 5b
:3 0 703 704 0
16 :2 0 705 706
0 964 99 :3 0
9a :3 0 9b :3 0
9c :3 0 9d :3 0
9e :3 0 9f :3 0
a0 :3 0 55 :3 0
9a :3 0 a1 :3 0
9b :3 0 a1 :3 0
9c :3 0 a1 :3 0
9d :3 0 a1 :3 0
9e :3 0 a1 :3 0
9f :3 0 a1 :3 0
a0 :3 0 a2 :3 0
55 :3 0 a3 :3 0
a4 :3 0 a1 :3 0
55 :3 0 9a :3 0
a1 :3 0 9a :3 0
53 :3 0 d :3 0
a5 :4 0 a6 1
:8 0 740 a7 :4 0
72b :3 0 4e :3 0
4f :2 0 3ed 72e
72f :3 0 9 :3 0
5b :3 0 731 732
0 16 :2 0 733
734 0 736 3f0
73d 9 :3 0 5b
:3 0 737 738 0
a8 :2 0 739 73a
0 73c 3f2 73e
730 736 0 73f
0 73c 0 73f
3f4 0 740 3f7
793 5d :3 0 9
:3 0 5b :3 0 742
743 0 5c :3 0
744 745 0 747
3fa 749 3fc 748
747 :2 0 791 5e
:3 0 9 :3 0 5b
:3 0 74c 74d 0
5c :3 0 74e 74f
0 789 2c :3 0
2d :3 0 751 752
0 2c :3 0 3d
:3 0 754 755 0
5f :2 0 2 :4 0
a9 :2 0 3f :2 0
14 :2 0 3fe 75a
75c :3 0 401 756
75e 753 760 75f
:2 0 2f :3 0 40
:3 0 762 763 0
31 :4 0 aa :4 0
34 :2 0 35 :3 0
405 767 769 :3 0
34 :2 0 45 :4 0
408 76b 76d :3 0
34 :2 0 62 :4 0
40b 76f 771 :3 0
34 :2 0 5c :3 0
40e 773 775 :3 0
34 :2 0 63 :4 0
411 777 779 :3 0
34 :2 0 64 :3 0
65 :3 0 14 :2 0
66 :2 0 414 77c
780 418 77b 782
:3 0 41b 764 784
:2 0 786 41e 787
761 786 0 788
420 0 789 422
78c :3 0 78c 0
78c 78b 789 78a
:6 0 78e 10 :3 0
425 790 427 78f
78e :2 0 791 429
:2 0 793 0 793
792 740 791 :6 0
964 1 :3 0 96
:3 0 97 :3 0 ab
:4 0 797 :4 0 798
:2 0 964 9 :3 0
5b :3 0 79a 79b
0 16 :2 0 79c
79d 0 964 ac
:3 0 ad :3 0 9a
:3 0 a2 :3 0 53
:3 0 d :3 0 a5
:4 0 ae 1 :8 0
7bc a7 :4 0 7a7
:3 0 4e :3 0 4f
:2 0 42e 7aa 7ab
:3 0 9 :3 0 5b
:3 0 7ad 7ae 0
16 :2 0 7af 7b0
0 7b2 431 7b9
9 :3 0 5b :3 0
7b3 7b4 0 a8
:2 0 7b5 7b6 0
7b8 433 7ba 7ac
7b2 0 7bb 0
7b8 0 7bb 435
0 7bc 438 80f
5d :3 0 9 :3 0
5b :3 0 7be 7bf
0 5c :3 0 7c0
7c1 0 7c3 43b
7c5 43d 7c4 7c3
:2 0 80d 5e :3 0
9 :3 0 5b :3 0
7c8 7c9 0 5c
:3 0 7ca 7cb 0
805 2c :3 0 2d
:3 0 7cd 7ce 0
2c :3 0 3d :3 0
7d0 7d1 0 5f
:2 0 2 :4 0 af
:2 0 3f :2 0 14
:2 0 43f 7d6 7d8
:3 0 442 7d2 7da
7cf 7dc 7db :2 0
2f :3 0 40 :3 0
7de 7df 0 31
:4 0 b0 :4 0 34
:2 0 35 :3 0 446
7e3 7e5 :3 0 34
:2 0 45 :4 0 449
7e7 7e9 :3 0 34
:2 0 62 :4 0 44c
7eb 7ed :3 0 34
:2 0 5c :3 0 44f
7ef 7f1 :3 0 34
:2 0 63 :4 0 452
7f3 7f5 :3 0 34
:2 0 64 :3 0 65
:3 0 14 :2 0 66
:2 0 455 7f8 7fc
459 7f7 7fe :3 0
45c 7e0 800 :2 0
802 45f 803 7dd
802 0 804 461
0 805 463 808
:3 0 808 0 808
807 805 806 :6 0
80a 12 :3 0 466
80c 468 80b 80a
:2 0 80d 46a :2 0
80f 0 80f 80e
7bc 80d :6 0 964
1 :3 0 b1 :3 0
1c :3 0 76 :3 0
b2 :3 0 b2 :3 0
53 :3 0 d :3 0
b2 :3 0 9a :3 0
b3 :3 0 9a :3 0
b4 :3 0 b3 :3 0
9a :3 0 b5 :3 0
b6 :3 0 b7 :3 0
b5 :3 0 b8 :3 0
b5 :3 0 b9 :3 0
ba :3 0 54 :3 0
ba :3 0 b7 :3 0
bb :3 0 b3 :3 0
9d :3 0 b3 :3 0
9e :3 0 b3 :3 0
9f :3 0 b3 :3 0
a0 :3 0 bc :3 0
bd :3 0 be :3 0
bd :3 0 bf :3 0
bd :3 0 c0 :3 0
bd :3 0 c1 :3 0
c2 :3 0 82 :3 0
99 :3 0 b3 :3 0
a3 :3 0 c3 :3 0
b5 :3 0 b3 :3 0
9a :3 0 b5 :3 0
9a :3 0 a3 :3 0
a4 :3 0 c4 :3 0
c4 :3 0 59 :3 0
b3 :3 0 9b :3 0
a3 :3 0 a4 :3 0
ba :3 0 ba :3 0
59 :3 0 c4 :3 0
9b :3 0 c5 :3 0
c6 :3 0 a3 :3 0
c7 :3 0 bd :3 0
bd :3 0 9c :3 0
b3 :3 0 9c :3 0
b4 :3 0 c8 :3 0
ad :3 0 9a :3 0
b5 :3 0 c9 :3 0
b7 :3 0 b8 :3 0
b9 :3 0 c4 :3 0
54 :3 0 c4 :3 0
b7 :3 0 bb :3 0
a1 :3 0 9d :3 0
a1 :3 0 9e :3 0
a1 :3 0 9f :3 0
a1 :3 0 a0 :3 0
bc :3 0 bd :3 0
be :3 0 bd :3 0
bf :3 0 bd :3 0
c0 :3 0 bd :3 0
c1 :3 0 c2 :3 0
82 :3 0 ac :3 0
c8 :3 0 a3 :3 0
ca :3 0 b5 :3 0
c8 :3 0 ad :3 0
b5 :3 0 ad :3 0
c5 :3 0 c6 :3 0
a3 :3 0 a4 :3 0
a1 :3 0 b5 :3 0
cb :3 0 a1 :3 0
59 :3 0 c5 :3 0
c6 :3 0 a3 :3 0
a4 :3 0 c4 :3 0
c4 :3 0 59 :3 0
a1 :3 0 9b :3 0
c5 :3 0 c6 :3 0
a3 :3 0 c7 :3 0
bd :3 0 bd :3 0
9c :3 0 a1 :3 0
9c :3 0 b4 :3 0
c8 :3 0 ad :3 0
9a :3 0 b5 :3 0
c9 :3 0 b7 :3 0
b8 :3 0 b9 :3 0
c4 :3 0 54 :3 0
c4 :3 0 b7 :3 0
bb :3 0 a1 :3 0
9d :3 0 a1 :3 0
9e :3 0 a1 :3 0
9f :3 0 a1 :3 0
a0 :3 0 bc :3 0
bd :3 0 be :3 0
bd :3 0 bf :3 0
bd :3 0 c0 :3 0
bd :3 0 c1 :3 0
c2 :3 0 82 :3 0
ac :3 0 c8 :3 0
a3 :3 0 ca :3 0
b5 :3 0 c8 :3 0
ad :3 0 b5 :3 0
ad :3 0 a3 :3 0
cc :3 0 cd :3 0
b5 :3 0 ad :3 0
cd :3 0 ce :3 0
c5 :3 0 c6 :3 0
a3 :3 0 a4 :3 0
a1 :3 0 cd :3 0
cf :3 0 a1 :3 0
59 :3 0 c5 :3 0
c6 :3 0 a3 :3 0
a4 :3 0 c4 :3 0
c4 :3 0 59 :3 0
a1 :3 0 9b :3 0
c5 :3 0 c6 :3 0
a3 :3 0 c7 :3 0
bd :3 0 bd :3 0
9c :3 0 a1 :3 0
9c :3 0 b4 :3 0
c8 :3 0 ad :3 0
9a :3 0 b5 :3 0
c9 :3 0 b7 :3 0
b8 :3 0 b9 :3 0
c4 :3 0 54 :3 0
c4 :3 0 b7 :3 0
bb :3 0 a1 :3 0
9d :3 0 a1 :3 0
9e :3 0 a1 :3 0
9f :3 0 a1 :3 0
a0 :3 0 bc :3 0
bd :3 0 be :3 0
bd :3 0 bf :3 0
bd :3 0 c0 :3 0
bd :3 0 c1 :3 0
c2 :3 0 82 :3 0
ac :3 0 c8 :3 0
a3 :3 0 ca :3 0
b5 :3 0 c8 :3 0
ad :3 0 b5 :3 0
ad :3 0 a3 :3 0
d0 :3 0 cd :3 0
b5 :3 0 ad :3 0
cd :3 0 ce :3 0
c5 :3 0 c6 :3 0
a3 :3 0 a4 :3 0
a1 :3 0 cd :3 0
cb :3 0 a1 :3 0
59 :3 0 c5 :3 0
c6 :3 0 a3 :3 0
a4 :3 0 c4 :3 0
c4 :3 0 59 :3 0
a1 :3 0 9b :3 0
c5 :3 0 c6 :3 0
a3 :3 0 c7 :3 0
bd :3 0 bd :3 0
9c :3 0 a1 :3 0
9c :3 0 b4 :3 0
9a :3 0 54 :4 0
d1 1 :a 0 940
:2 0 964 812 93e
:2 0 2c :3 0 2d
:3 0 941 942 0
2c :3 0 3d :3 0
944 945 0 14
:2 0 2 :4 0 d2
:2 0 3f :2 0 14
:2 0 46d 94a 94c
:3 0 470 946 94e
943 950 94f :2 0
2f :3 0 40 :3 0
952 953 0 31
:4 0 d3 :4 0 34
:2 0 35 :3 0 474
957 959 :3 0 477
954 95b :2 0 95d
47a 95e 951 95d
0 95f 47c 0
964 8 :3 0 1c
:3 0 961 :2 0 964
74 :3 0 47e 1fab
39 :3 0 15 :3 0
16 :2 0 488 965
968 4f :2 0 14
:2 0 48d 96a 96c
:3 0 96d :2 0 39
:3 0 15 :3 0 16
:2 0 490 96f 972
4f :2 0 d4 :2 0
495 974 976 :3 0
977 :2 0 96e 979
978 :2 0 96 :3 0
97 :3 0 d5 :4 0
97d :4 0 97e :2 0
b25 9 :3 0 5b
:3 0 980 981 0
16 :2 0 982 983
0 b25 d6 :3 0
9a :3 0 54 :3 0
d7 :3 0 9a :3 0
b2 :3 0 77 :3 0
54 :3 0 a2 :3 0
d7 :3 0 c5 :3 0
c6 :3 0 a3 :3 0
76 :3 0 b2 :3 0
b2 :3 0 53 :3 0
d :3 0 d7 :3 0
9a :3 0 b2 :3 0
9a :3 0 d7 :3 0
53 :3 0 d :3 0
d7 :3 0 a5 :3 0
b2 :3 0 d8 :3 0
76 :3 0 d9 :3 0
d7 :3 0 9a :3 0
d9 :3 0 9a :3 0
d9 :3 0 da :3 0
d :4 0 db 1
:8 0 9c1 a7 :4 0
9ac :3 0 4e :3 0
4f :2 0 49a 9af
9b0 :3 0 9 :3 0
5b :3 0 9b2 9b3
0 16 :2 0 9b4
9b5 0 9b7 49d
9be 9 :3 0 5b
:3 0 9b8 9b9 0
a8 :2 0 9ba 9bb
0 9bd 49f 9bf
9b1 9b7 0 9c0
0 9bd 0 9c0
4a1 0 9c1 4a4
a14 5d :3 0 9
:3 0 5b :3 0 9c3
9c4 0 5c :3 0
9c5 9c6 0 9c8
4a7 9ca 4a9 9c9
9c8 :2 0 a12 5e
:3 0 9 :3 0 5b
:3 0 9cd 9ce 0
5c :3 0 9cf 9d0
0 a0a 2c :3 0
2d :3 0 9d2 9d3
0 2c :3 0 3d
:3 0 9d5 9d6 0
5f :2 0 2 :4 0
dc :2 0 3f :2 0
14 :2 0 4ab 9db
9dd :3 0 4ae 9d7
9df 9d4 9e1 9e0
:2 0 2f :3 0 40
:3 0 9e3 9e4 0
31 :4 0 dd :4 0
34 :2 0 35 :3 0
4b2 9e8 9ea :3 0
34 :2 0 45 :4 0
4b5 9ec 9ee :3 0
34 :2 0 62 :4 0
4b8 9f0 9f2 :3 0
34 :2 0 5c :3 0
4bb 9f4 9f6 :3 0
34 :2 0 63 :4 0
4be 9f8 9fa :3 0
34 :2 0 64 :3 0
65 :3 0 14 :2 0
66 :2 0 4c1 9fd
a01 4c5 9fc a03
:3 0 4c8 9e5 a05
:2 0 a07 4cb a08
9e2 a07 0 a09
4cd 0 a0a 4cf
a0d :3 0 a0d 0
a0d a0c a0a a0b
:6 0 a0f 14 :3 0
4d2 a11 4d4 a10
a0f :2 0 a12 4d6
:2 0 a14 0 a14
a13 9c1 a12 :6 0
b25 1 :3 0 9
:3 0 5b :3 0 a16
a17 0 de :4 0
a19 a1a :3 0 a18
a1b 0 b25 df
:3 0 59 :3 0 b7
:3 0 a1 :3 0 ad
:3 0 a1 :3 0 c9
:3 0 a2 :3 0 55
:3 0 a3 :3 0 ca
:3 0 a1 :3 0 55
:3 0 a5 :3 0 a1
:3 0 ad :3 0 55
:3 0 9a :3 0 a3
:3 0 a4 :3 0 c4
:3 0 c4 :3 0 59
:3 0 a1 :3 0 cb
:3 0 a3 :3 0 a4
:3 0 ba :3 0 ba
:3 0 59 :3 0 c4
:3 0 9b :3 0 a3
:3 0 76 :3 0 d9
:3 0 d9 :3 0 ce
:3 0 55 :3 0 ce
:3 0 d9 :3 0 d8
:3 0 55 :3 0 53
:3 0 d :3 0 55
:3 0 e0 :4 0 e1
1 :8 0 a61 a7
:4 0 a4c :3 0 4e
:3 0 4f :2 0 4db
a4f a50 :3 0 9
:3 0 5b :3 0 a52
a53 0 16 :2 0
a54 a55 0 a57
4de a5e 9 :3 0
5b :3 0 a58 a59
0 a8 :2 0 a5a
a5b 0 a5d 4e0
a5f a51 a57 0
a60 0 a5d 0
a60 4e2 0 a61
4e5 ab4 5d :3 0
9 :3 0 5b :3 0
a63 a64 0 5c
:3 0 a65 a66 0
a68 4e8 a6a 4ea
a69 a68 :2 0 ab2
5e :3 0 9 :3 0
5b :3 0 a6d a6e
0 5c :3 0 a6f
a70 0 aaa 2c
:3 0 2d :3 0 a72
a73 0 2c :3 0
3d :3 0 a75 a76
0 5f :2 0 2
:4 0 e2 :2 0 3f
:2 0 14 :2 0 4ec
a7b a7d :3 0 4ef
a77 a7f a74 a81
a80 :2 0 2f :3 0
40 :3 0 a83 a84
0 31 :4 0 e3
:4 0 34 :2 0 35
:3 0 4f3 a88 a8a
:3 0 34 :2 0 45
:4 0 4f6 a8c a8e
:3 0 34 :2 0 62
:4 0 4f9 a90 a92
:3 0 34 :2 0 5c
:3 0 4fc a94 a96
:3 0 34 :2 0 63
:4 0 4ff a98 a9a
:3 0 34 :2 0 64
:3 0 65 :3 0 14
:2 0 66 :2 0 502
a9d aa1 506 a9c
aa3 :3 0 509 a85
aa5 :2 0 aa7 50c
aa8 a82 aa7 0
aa9 50e 0 aaa
510 aad :3 0 aad
0 aad aac aaa
aab :6 0 aaf 16
:3 0 513 ab1 515
ab0 aaf :2 0 ab2
517 :2 0 ab4 0
ab4 ab3 a61 ab2
:6 0 b25 1 :3 0
b1 :3 0 1d :3 0
55 :3 0 9a :3 0
59 :3 0 ce :3 0
b5 :3 0 b6 :3 0
b7 :3 0 55 :3 0
54 :3 0 b5 :3 0
b9 :3 0 e4 :3 0
e5 :3 0 e4 :3 0
e6 :3 0 e7 :3 0
e4 :3 0 e8 :3 0
e4 :3 0 e9 :3 0
ea :3 0 eb :3 0
ec :3 0 17 :3 0
56 :3 0 ed :3 0
ee :3 0 ef :3 0
f0 :3 0 f1 :3 0
f2 :3 0 c2 :3 0
82 :3 0 d6 :3 0
55 :3 0 a3 :3 0
c3 :3 0 b5 :3 0
55 :3 0 9a :3 0
b5 :3 0 9a :3 0
59 :3 0 ce :3 0
b7 :3 0 54 :3 0
b9 :3 0 e4 :3 0
e5 :3 0 e4 :3 0
e6 :3 0 e7 :3 0
e4 :3 0 e8 :3 0
e4 :3 0 e9 :3 0
ea :3 0 eb :3 0
ec :3 0 17 :3 0
56 :3 0 ed :3 0
ee :3 0 ef :3 0
f0 :3 0 f1 :3 0
f2 :3 0 c2 :3 0
82 :3 0 df :3 0
54 :4 0 f3 1
:a 0 b01 :2 0 b25
ab7 aff :2 0 2c
:3 0 2d :3 0 b02
b03 0 2c :3 0
3d :3 0 b05 b06
0 14 :2 0 2
:4 0 f4 :2 0 3f
:2 0 14 :2 0 51a
b0b b0d :3 0 51d
b07 b0f b04 b11
b10 :2 0 2f :3 0
40 :3 0 b13 b14
0 31 :4 0 f5
:4 0 34 :2 0 35
:3 0 521 b18 b1a
:3 0 524 b15 b1c
:2 0 b1e 527 b1f
b12 b1e 0 b20
529 0 b25 8
:3 0 1d :3 0 b22
:2 0 b25 74 :3 0
52b b26 97a b25
0 1fad 39 :3 0
15 :3 0 16 :2 0
534 b27 b2a 4f
:2 0 50 :2 0 539
b2c b2e :3 0 b1
:3 0 1e :3 0 d9
:3 0 77 :3 0 d9
:3 0 77 :3 0 55
:3 0 ce :3 0 59
:3 0 55 :3 0 ad
:3 0 b5 :3 0 c9
:3 0 b7 :3 0 54
:3 0 55 :3 0 ce
:3 0 55 :3 0 f6
:3 0 d9 :3 0 e5
:3 0 d9 :3 0 e6
:3 0 d9 :3 0 f0
:3 0 d9 :3 0 f7
:3 0 d9 :3 0 f8
:3 0 17 :3 0 56
:3 0 55 :3 0 f9
:3 0 55 :3 0 fa
:3 0 fb :3 0 b7
:3 0 fc :3 0 39
:3 0 55 :3 0 fd
:3 0 55 :3 0 fa
:3 0 fd :3 0 fe
:3 0 b7 :3 0 ff
:3 0 d9 :3 0 be
:3 0 d9 :3 0 100
:3 0 d9 :3 0 c0
:3 0 d9 :3 0 101
:3 0 102 :3 0 55
:3 0 103 :3 0 d9
:3 0 f2 :3 0 d9
:3 0 d8 :3 0 d9
:3 0 104 :3 0 105
:3 0 b7 :3 0 106
:3 0 d9 :3 0 ec
:3 0 d9 :3 0 107
:3 0 d9 :3 0 108
:3 0 d9 :3 0 109
:3 0 d9 :3 0 10a
:3 0 d9 :3 0 10b
:3 0 10c :3 0 d9
:3 0 10d :3 0 a2
:3 0 55 :3 0 c5
:3 0 c6 :3 0 a3
:3 0 76 :3 0 d9
:3 0 d9 :3 0 da
:3 0 d :3 0 55
:3 0 ce :3 0 d9
:3 0 ce :3 0 c5
:3 0 c6 :3 0 a3
:3 0 ca :3 0 b5
:3 0 55 :3 0 9a
:3 0 b5 :3 0 ad
:3 0 c5 :3 0 c6
:3 0 a3 :3 0 10e
:3 0 fb :3 0 55
:3 0 fa :3 0 fb
:3 0 59 :3 0 c5
:3 0 c6 :3 0 a3
:3 0 10e :3 0 fe
:3 0 39 :3 0 55
:3 0 fd :3 0 55
:3 0 fa :3 0 fe
:3 0 59 :3 0 c5
:3 0 c6 :3 0 a3
:3 0 10f :3 0 105
:3 0 105 :3 0 59
:3 0 d9 :3 0 104
:3 0 55 :3 0 59
:3 0 d :3 0 55
:3 0 a5 :3 0 b7
:4 0 110 1 :a 0
bbf :2 0 be3 b31
bbd :2 0 2c :3 0
2d :3 0 bc0 bc1
0 2c :3 0 3d
:3 0 bc3 bc4 0
14 :2 0 2 :4 0
111 :2 0 3f :2 0
14 :2 0 53c bc9
bcb :3 0 53f bc5
bcd bc2 bcf bce
:2 0 2f :3 0 40
:3 0 bd1 bd2 0
31 :4 0 112 :4 0
34 :2 0 35 :3 0
543 bd6 bd8 :3 0
546 bd3 bda :2 0
bdc 549 bdd bd0
bdc 0 bde 54b
0 be3 8 :3 0
1e :3 0 be0 :2 0
be3 74 :3 0 54d
be4 b2f be3 0
1fad 39 :3 0 15
:3 0 16 :2 0 551
be5 be8 4f :2 0
51 :2 0 556 bea
bec :3 0 113 :3 0
39 :3 0 18 :3 0
16 :2 0 559 bef
bf2 55c bee bf4
8b :2 0 16 :2 0
560 bf6 bf8 :3 0
19 :3 0 16 :2 0
bfa bfb 0 fa2
114 :3 0 19 :3 0
76 :3 0 53 :3 0
10 :3 0 da :3 0
d :4 0 115 1
:8 0 c0a 9 :3 0
5b :3 0 c05 c06
0 5c :3 0 c07
c08 0 c0a 563
c5d 5d :3 0 9
:3 0 5b :3 0 c0c
c0d 0 5c :3 0
c0e c0f 0 c11
566 c13 568 c12
c11 :2 0 c5b 5e
:3 0 9 :3 0 5b
:3 0 c16 c17 0
5c :3 0 c18 c19
0 c53 2c :3 0
2d :3 0 c1b c1c
0 2c :3 0 3d
:3 0 c1e c1f 0
5f :2 0 2 :4 0
116 :2 0 3f :2 0
14 :2 0 56a c24
c26 :3 0 56d c20
c28 c1d c2a c29
:2 0 2f :3 0 40
:3 0 c2c c2d 0
31 :4 0 117 :4 0
34 :2 0 35 :3 0
571 c31 c33 :3 0
34 :2 0 45 :4 0
574 c35 c37 :3 0
34 :2 0 62 :4 0
577 c39 c3b :3 0
34 :2 0 5c :3 0
57a c3d c3f :3 0
34 :2 0 63 :4 0
57d c41 c43 :3 0
34 :2 0 64 :3 0
65 :3 0 14 :2 0
66 :2 0 580 c46
c4a 584 c45 c4c
:3 0 587 c2e c4e
:2 0 c50 58a c51
c2b c50 0 c52
58c 0 c53 58e
c56 :3 0 c56 0
c56 c55 c53 c54
:6 0 c58 18 :3 0
591 c5a 593 c59
c58 :2 0 c5b 595
:2 0 c5d 0 c5d
c5c c0a c5b :6 0
fa2 1 :3 0 39
:3 0 19 :3 0 16
:2 0 598 c5f c62
4f :2 0 16 :2 0
59d c64 c66 :3 0
13 :3 0 14 :4 0
c68 c69 0 c6b
5a0 d70 19 :3 0
16 :2 0 c6c c6d
0 d6f 114 :3 0
19 :3 0 118 :3 0
76 :3 0 53 :3 0
10 :3 0 da :3 0
d :3 0 119 :4 0
11a 1 :8 0 c7e
9 :3 0 5b :3 0
c79 c7a 0 5c
:3 0 c7b c7c 0
c7e 5a2 cd1 5d
:3 0 9 :3 0 5b
:3 0 c80 c81 0
5c :3 0 c82 c83
0 c85 5a5 c87
5a7 c86 c85 :2 0
ccf 5e :3 0 9
:3 0 5b :3 0 c8a
c8b 0 5c :3 0
c8c c8d 0 cc7
2c :3 0 2d :3 0
c8f c90 0 2c
:3 0 3d :3 0 c92
c93 0 5f :2 0
2 :4 0 11b :2 0
3f :2 0 14 :2 0
5a9 c98 c9a :3 0
5ac c94 c9c c91
c9e c9d :2 0 2f
:3 0 40 :3 0 ca0
ca1 0 31 :4 0
11c :4 0 34 :2 0
35 :3 0 5b0 ca5
ca7 :3 0 34 :2 0
45 :4 0 5b3 ca9
cab :3 0 34 :2 0
62 :4 0 5b6 cad
caf :3 0 34 :2 0
5c :3 0 5b9 cb1
cb3 :3 0 34 :2 0
63 :4 0 5bc cb5
cb7 :3 0 34 :2 0
64 :3 0 65 :3 0
14 :2 0 66 :2 0
5bf cba cbe 5c3
cb9 cc0 :3 0 5c6
ca2 cc2 :2 0 cc4
5c9 cc5 c9f cc4
0 cc6 5cb 0
cc7 5cd cca :3 0
cca 0 cca cc9
cc7 cc8 :6 0 ccc
1a :3 0 5d0 cce
5d2 ccd ccc :2 0
ccf 5d4 :2 0 cd1
0 cd1 cd0 c7e
ccf :6 0 d6f 1
:3 0 39 :3 0 19
:3 0 16 :2 0 5d7
cd3 cd6 4f :2 0
16 :2 0 5dc cd8
cda :3 0 114 :3 0
19 :3 0 118 :3 0
76 :3 0 b2 :3 0
a3 :3 0 11d :3 0
11e :3 0 11e :3 0
59 :3 0 b2 :3 0
53 :3 0 11e :3 0
9a :3 0 b2 :3 0
9a :3 0 b2 :3 0
53 :3 0 10 :3 0
b2 :3 0 da :3 0
d :3 0 11f :3 0
64 :3 0 b2 :3 0
120 :3 0 11e :3 0
121 :4 0 122 1
:8 0 cfe 9 :3 0
5b :3 0 cf9 cfa
0 5c :3 0 cfb
cfc 0 cfe 5df
d51 5d :3 0 9
:3 0 5b :3 0 d00
d01 0 5c :3 0
d02 d03 0 d05
5e2 d07 5e4 d06
d05 :2 0 d4f 5e
:3 0 9 :3 0 5b
:3 0 d0a d0b 0
5c :3 0 d0c d0d
0 d47 2c :3 0
2d :3 0 d0f d10
0 2c :3 0 3d
:3 0 d12 d13 0
5f :2 0 2 :4 0
123 :2 0 3f :2 0
14 :2 0 5e6 d18
d1a :3 0 5e9 d14
d1c d11 d1e d1d
:2 0 2f :3 0 40
:3 0 d20 d21 0
31 :4 0 124 :4 0
34 :2 0 35 :3 0
5ed d25 d27 :3 0
34 :2 0 45 :4 0
5f0 d29 d2b :3 0
34 :2 0 62 :4 0
5f3 d2d d2f :3 0
34 :2 0 5c :3 0
5f6 d31 d33 :3 0
34 :2 0 63 :4 0
5f9 d35 d37 :3 0
34 :2 0 64 :3 0
65 :3 0 14 :2 0
66 :2 0 5fc d3a
d3e 600 d39 d40
:3 0 603 d22 d42
:2 0 d44 606 d45
d1f d44 0 d46
608 0 d47 60a
d4a :3 0 d4a 0
d4a d49 d47 d48
:6 0 d4c 1c :3 0
60d d4e 60f d4d
d4c :2 0 d4f 611
:2 0 d51 0 d51
d50 cfe d4f :6 0
d67 1 :3 0 39
:3 0 19 :3 0 16
:2 0 614 d53 d56
4f :2 0 16 :2 0
619 d58 d5a :3 0
13 :3 0 50 :4 0
d5c d5d 0 d5f
61c d64 13 :3 0
14 :4 0 d60 d61
0 d63 61e d65
d5b d5f 0 d66
0 d63 0 d66
620 0 d67 623
d6c 13 :3 0 14
:4 0 d68 d69 0
d6b 626 d6d cdb
d67 0 d6e 0
d6b 0 d6e 628
0 d6f 62b d71
c67 c6b 0 d72
0 d6f 0 d72
62f 0 fa2 b1
:3 0 1f :3 0 d9
:3 0 59 :3 0 55
:3 0 9a :3 0 d9
:3 0 b7 :3 0 d9
:3 0 54 :3 0 ce
:3 0 55 :3 0 f6
:3 0 d9 :3 0 f7
:3 0 d9 :3 0 f8
:3 0 d9 :3 0 125
:3 0 d9 :3 0 126
:3 0 d9 :3 0 f7
:3 0 e5 :3 0 d9
:3 0 f8 :3 0 e6
:3 0 d9 :3 0 127
:3 0 d9 :3 0 e7
:3 0 f0 :3 0 d9
:3 0 127 :3 0 d9
:3 0 e7 :3 0 d9
:3 0 128 :3 0 eb
:3 0 129 :3 0 d9
:3 0 12a :3 0 d9
:3 0 128 :3 0 12b
:3 0 12c :3 0 d9
:3 0 128 :3 0 12c
:3 0 d9 :3 0 127
:3 0 d9 :3 0 e7
:3 0 12d :3 0 e4
:3 0 12e :3 0 e4
:3 0 12f :3 0 130
:3 0 d9 :3 0 131
:3 0 d9 :3 0 132
:3 0 d9 :3 0 133
:3 0 d9 :3 0 120
:3 0 d9 :3 0 134
:3 0 d9 :3 0 128
:3 0 135 :3 0 d9
:3 0 136 :3 0 d9
:3 0 137 :3 0 d9
:3 0 138 :3 0 139
:3 0 17 :3 0 56
:3 0 f9 :3 0 fa
:3 0 fc :3 0 fd
:3 0 ff :3 0 be
:3 0 100 :3 0 c0
:3 0 101 :3 0 102
:3 0 d9 :3 0 103
:3 0 13a :3 0 13b
:3 0 f2 :3 0 d9
:3 0 13c :3 0 d9
:3 0 13d :3 0 d9
:3 0 13e :3 0 13f
:3 0 d8 :3 0 104
:3 0 140 :3 0 ec
:3 0 107 :3 0 108
:3 0 109 :3 0 10a
:3 0 10b :3 0 10c
:3 0 141 :3 0 10d
:3 0 13 :3 0 119
:3 0 142 :3 0 d9
:3 0 143 :3 0 d9
:3 0 144 :3 0 1b
:3 0 d9 :3 0 145
:3 0 e4 :3 0 145
:3 0 1b :3 0 d9
:3 0 146 :3 0 e4
:3 0 146 :3 0 1b
:3 0 d9 :3 0 147
:3 0 148 :3 0 d9
:3 0 12d :3 0 149
:3 0 14a :3 0 14b
:3 0 14c :3 0 14d
:3 0 14e :3 0 14f
:3 0 150 :3 0 151
:3 0 152 :3 0 58
:3 0 d9 :3 0 c5
:3 0 a3 :3 0 a2
:3 0 55 :3 0 55
:3 0 59 :3 0 d9
:3 0 59 :3 0 55
:3 0 9a :3 0 f
:3 0 c5 :3 0 c6
:3 0 a3 :3 0 10e
:3 0 fb :3 0 55
:3 0 fa :3 0 fb
:3 0 59 :3 0 d9
:3 0 53 :3 0 10
:3 0 d9 :3 0 59
:3 0 d :3 0 d9
:3 0 77 :3 0 d9
:3 0 77 :3 0 55
:3 0 ce :3 0 59
:3 0 55 :3 0 9a
:3 0 55 :3 0 a5
:3 0 b5 :3 0 c9
:3 0 ca :3 0 b5
:3 0 55 :3 0 9a
:3 0 b5 :3 0 ad
:3 0 a1 :3 0 b6
:3 0 c3 :3 0 a1
:3 0 55 :3 0 9a
:3 0 a1 :3 0 9a
:3 0 b7 :3 0 d9
:3 0 77 :3 0 55
:3 0 a5 :3 0 54
:3 0 55 :3 0 ce
:3 0 55 :3 0 f6
:3 0 d9 :3 0 f7
:3 0 39 :3 0 d9
:3 0 e6 :3 0 d9
:3 0 153 :3 0 f8
:3 0 d9 :3 0 125
:3 0 47 :3 0 154
:3 0 d9 :3 0 125
:3 0 47 :3 0 155
:3 0 47 :3 0 156
:3 0 d9 :3 0 125
:3 0 125 :3 0 d9
:3 0 126 :3 0 47
:3 0 157 :3 0 47
:3 0 158 :3 0 d9
:3 0 126 :3 0 126
:3 0 39 :3 0 e5
:3 0 159 :3 0 e5
:3 0 39 :3 0 d9
:3 0 e6 :3 0 d9
:3 0 153 :3 0 e6
:3 0 d9 :3 0 119
:3 0 d9 :3 0 15a
:3 0 d9 :3 0 f0
:3 0 f0 :3 0 d9
:3 0 119 :3 0 d9
:3 0 15a :3 0 d9
:3 0 128 :3 0 d9
:3 0 f0 :3 0 eb
:3 0 d9 :3 0 15b
:3 0 d9 :3 0 15c
:3 0 d9 :3 0 15b
:3 0 d9 :3 0 15c
:3 0 129 :3 0 12a
:3 0 d9 :3 0 128
:3 0 12b :3 0 12c
:3 0 d9 :3 0 128
:3 0 12c :3 0 f0
:3 0 15a :3 0 12d
:3 0 d9 :3 0 12e
:3 0 d9 :3 0 12f
:3 0 d9 :3 0 130
:3 0 d9 :3 0 131
:3 0 d9 :3 0 132
:3 0 d9 :3 0 133
:3 0 d9 :3 0 120
:3 0 d9 :3 0 134
:3 0 d9 :3 0 128
:3 0 135 :3 0 136
:3 0 137 :3 0 139
:3 0 17 :3 0 56
:3 0 55 :3 0 f9
:3 0 55 :3 0 fa
:3 0 fb :3 0 b7
:3 0 fc :3 0 39
:3 0 55 :3 0 fd
:3 0 55 :3 0 fa
:3 0 fd :3 0 fe
:3 0 b7 :3 0 ff
:3 0 d9 :3 0 be
:3 0 d9 :3 0 100
:3 0 d9 :3 0 c0
:3 0 d9 :3 0 101
:3 0 55 :3 0 a5
:3 0 a1 :3 0 102
:3 0 c3 :3 0 a1
:3 0 55 :3 0 9a
:3 0 a1 :3 0 9a
:3 0 102 :3 0 55
:3 0 103 :3 0 d9
:3 0 f2 :3 0 d9
:3 0 13c :3 0 d9
:3 0 13d :3 0 d9
:3 0 13f :3 0 d9
:3 0 d8 :3 0 d9
:3 0 104 :3 0 105
:3 0 b7 :3 0 140
:3 0 d9 :3 0 ec
:3 0 d9 :3 0 107
:3 0 d9 :3 0 108
:3 0 d9 :3 0 109
:3 0 d9 :3 0 10a
:3 0 d9 :3 0 10b
:3 0 d9 :3 0 10c
:3 0 d9 :3 0 141
:3 0 d9 :3 0 10d
:3 0 d9 :3 0 119
:3 0 d9 :3 0 119
:3 0 119 :3 0 d9
:3 0 142 :3 0 143
:3 0 d9 :3 0 119
:3 0 d9 :3 0 15d
:3 0 d9 :3 0 15e
:3 0 144 :3 0 1b
:3 0 39 :3 0 d9
:3 0 145 :3 0 d9
:3 0 15f :3 0 e4
:3 0 145 :3 0 1b
:3 0 39 :3 0 d9
:3 0 146 :3 0 d9
:3 0 160 :3 0 e4
:3 0 146 :3 0 1b
:3 0 d9 :3 0 148
:3 0 d9 :3 0 133
:3 0 d9 :3 0 148
:3 0 148 :3 0 d9
:3 0 12a :3 0 149
:3 0 14a :3 0 14b
:3 0 d9 :3 0 14c
:3 0 161 :3 0 b7
:3 0 14d :3 0 d9
:3 0 14e :3 0 162
:3 0 b7 :3 0 14f
:3 0 d9 :3 0 150
:3 0 d9 :3 0 151
:3 0 163 :3 0 b7
:3 0 152 :3 0 a2
:3 0 55 :3 0 c5
:3 0 c6 :3 0 a3
:3 0 76 :3 0 d9
:3 0 55 :3 0 ce
:3 0 d9 :3 0 ce
:3 0 c5 :3 0 c6
:3 0 a3 :3 0 10e
:3 0 fb :3 0 55
:3 0 fa :3 0 fb
:3 0 59 :3 0 c5
:3 0 c6 :3 0 a3
:3 0 10e :3 0 fe
:3 0 39 :3 0 55
:3 0 fd :3 0 55
:3 0 fa :3 0 fe
:3 0 59 :3 0 c5
:3 0 c6 :3 0 a3
:3 0 10f :3 0 105
:3 0 105 :3 0 59
:3 0 d9 :3 0 104
:3 0 c5 :3 0 c6
:3 0 a3 :3 0 164
:3 0 161 :3 0 161
:3 0 59 :3 0 d9
:3 0 14c :3 0 c5
:3 0 c6 :3 0 a3
:3 0 164 :3 0 162
:3 0 162 :3 0 59
:3 0 d9 :3 0 14e
:3 0 c5 :3 0 c6
:3 0 a3 :3 0 165
:3 0 163 :3 0 163
:3 0 59 :3 0 d9
:3 0 151 :3 0 55
:3 0 53 :3 0 10
:3 0 55 :3 0 59
:3 0 d :3 0 55
:3 0 103 :3 0 14a
:3 0 b7 :4 0 166
1 :a 0 f7f :2 0
fa2 d74 f7d :2 0
2c :3 0 2d :3 0
f80 f81 0 2c
:3 0 3d :3 0 f83
f84 0 14 :2 0
2 :4 0 167 :2 0
3f :2 0 14 :2 0
632 f89 f8b :3 0
635 f85 f8d f82
f8f f8e :2 0 2f
:3 0 40 :3 0 f91
f92 0 31 :4 0
168 :4 0 34 :2 0
35 :3 0 639 f96
f98 :3 0 63c f93
f9a :2 0 f9c 63f
f9d f90 f9c 0
f9e 641 0 fa2
8 :3 0 1f :3 0
fa0 :2 0 fa2 643
1110 b1 :3 0 20
:3 0 d9 :3 0 77
:3 0 d9 :3 0 77
:3 0 55 :3 0 ce
:3 0 59 :3 0 55
:3 0 9a :3 0 55
:3 0 a5 :3 0 a1
:3 0 b6 :3 0 c3
:3 0 a1 :3 0 55
:3 0 9a :3 0 a1
:3 0 9a :3 0 55
:3 0 a5 :3 0 b5
:3 0 c9 :3 0 ca
:3 0 b5 :3 0 55
:3 0 9a :3 0 b5
:3 0 ad :3 0 55
:3 0 a5 :3 0 b5
:3 0 b7 :3 0 6c
:3 0 b5 :3 0 55
:3 0 9a :3 0 b5
:3 0 59 :3 0 b5
:3 0 b7 :3 0 169
:3 0 b5 :3 0 55
:3 0 9a :3 0 b5
:3 0 59 :3 0 b7
:3 0 55 :3 0 a5
:3 0 47 :3 0 16a
:3 0 55 :3 0 a5
:3 0 47 :3 0 16b
:3 0 55 :3 0 a5
:3 0 47 :3 0 16c
:3 0 d9 :3 0 77
:3 0 47 :3 0 16d
:3 0 54 :3 0 55
:3 0 ce :3 0 55
:3 0 f6 :3 0 d9
:3 0 f7 :3 0 d9
:3 0 f8 :3 0 d9
:3 0 125 :3 0 d9
:3 0 126 :3 0 39
:3 0 d9 :3 0 e5
:3 0 d9 :3 0 159
:3 0 e5 :3 0 39
:3 0 d9 :3 0 e6
:3 0 d9 :3 0 153
:3 0 e6 :3 0 d9
:3 0 16e :3 0 f0
:3 0 d9 :3 0 119
:3 0 d9 :3 0 15a
:3 0 d9 :3 0 f0
:3 0 eb :3 0 d9
:3 0 15c :3 0 129
:3 0 d9 :3 0 12a
:3 0 12d :3 0 d9
:3 0 12e :3 0 d9
:3 0 12f :3 0 d9
:3 0 130 :3 0 d9
:3 0 131 :3 0 d9
:3 0 132 :3 0 d9
:3 0 133 :3 0 d9
:3 0 120 :3 0 d9
:3 0 134 :3 0 d9
:3 0 128 :3 0 135
:3 0 136 :3 0 137
:3 0 139 :3 0 17
:3 0 56 :3 0 55
:3 0 f9 :3 0 55
:3 0 fa :3 0 fb
:3 0 b7 :3 0 fc
:3 0 39 :3 0 55
:3 0 fd :3 0 55
:3 0 fa :3 0 fd
:3 0 fe :3 0 b7
:3 0 ff :3 0 d9
:3 0 be :3 0 d9
:3 0 100 :3 0 d9
:3 0 c0 :3 0 d9
:3 0 101 :3 0 55
:3 0 a5 :3 0 a1
:3 0 102 :3 0 c3
:3 0 a1 :3 0 55
:3 0 9a :3 0 a1
:3 0 9a :3 0 102
:3 0 55 :3 0 103
:3 0 d9 :3 0 f2
:3 0 d9 :3 0 13c
:3 0 d9 :3 0 13d
:3 0 d9 :3 0 13f
:3 0 d9 :3 0 d8
:3 0 d9 :3 0 104
:3 0 105 :3 0 b7
:3 0 140 :3 0 d9
:3 0 ec :3 0 d9
:3 0 107 :3 0 d9
:3 0 108 :3 0 d9
:3 0 109 :3 0 d9
:3 0 10a :3 0 d9
:3 0 10b :3 0 d9
:3 0 10c :3 0 d9
:3 0 141 :3 0 d9
:3 0 10d :3 0 d9
:3 0 119 :3 0 d9
:3 0 119 :3 0 119
:3 0 d9 :3 0 142
:3 0 143 :3 0 144
:3 0 d9 :3 0 145
:3 0 d9 :3 0 146
:3 0 d9 :3 0 148
:3 0 d9 :3 0 12a
:3 0 149 :3 0 14a
:3 0 d9 :3 0 77
:3 0 d9 :3 0 d8
:3 0 d9 :3 0 77
:3 0 d9 :3 0 145
:3 0 d9 :3 0 15f
:3 0 d9 :3 0 146
:3 0 d9 :3 0 160
:3 0 14b :3 0 d9
:3 0 14c :3 0 161
:3 0 b7 :3 0 14d
:3 0 d9 :3 0 14e
:3 0 162 :3 0 b7
:3 0 14f :3 0 d9
:3 0 150 :3 0 d9
:3 0 151 :3 0 163
:3 0 b7 :3 0 152
:3 0 a2 :3 0 55
:3 0 c5 :3 0 c6
:3 0 a3 :3 0 76
:3 0 d9 :3 0 55
:3 0 ce :3 0 d9
:3 0 ce :3 0 c5
:3 0 c6 :3 0 a3
:3 0 10e :3 0 fb
:3 0 55 :3 0 fa
:3 0 fb :3 0 59
:3 0 c5 :3 0 c6
:3 0 a3 :3 0 10e
:3 0 fe :3 0 39
:3 0 55 :3 0 fd
:3 0 55 :3 0 fa
:3 0 fe :3 0 59
:3 0 c5 :3 0 c6
:3 0 a3 :3 0 10f
:3 0 105 :3 0 105
:3 0 59 :3 0 d9
:3 0 104 :3 0 c5
:3 0 c6 :3 0 a3
:3 0 164 :3 0 161
:3 0 161 :3 0 59
:3 0 d9 :3 0 14c
:3 0 c5 :3 0 c6
:3 0 a3 :3 0 164
:3 0 162 :3 0 162
:3 0 59 :3 0 d9
:3 0 14e :3 0 c5
:3 0 c6 :3 0 a3
:3 0 165 :3 0 163
:3 0 163 :3 0 59
:3 0 d9 :3 0 151
:3 0 55 :3 0 103
:3 0 55 :3 0 59
:3 0 d :3 0 54
:3 0 b7 :4 0 16f
1 :a 0 10ec :2 0
110f fa4 10ea :2 0
2c :3 0 2d :3 0
10ed 10ee 0 2c
:3 0 3d :3 0 10f0
10f1 0 14 :2 0
2 :4 0 170 :2 0
3f :2 0 14 :2 0
64a 10f6 10f8 :3 0
64d 10f2 10fa 10ef
10fc 10fb :2 0 2f
:3 0 40 :3 0 10fe
10ff 0 31 :4 0
171 :4 0 34 :2 0
35 :3 0 651 1103
1105 :3 0 654 1100
1107 :2 0 1109 657
110a 10fd 1109 0
110b 659 0 110f
8 :3 0 20 :3 0
110d :2 0 110f 65b
1111 bf9 fa2 0
1112 0 110f 0
1112 65f 0 1114
74 :3 0 662 1115
bed 1114 0 1fad
39 :3 0 15 :3 0
16 :2 0 664 1116
1119 4f :2 0 172
:2 0 669 111b 111d
:3 0 96 :3 0 97
:3 0 173 :4 0 1121
:4 0 1122 :2 0 12d4
9 :3 0 5b :3 0
1124 1125 0 16
:2 0 1126 1127 0
12d4 174 :3 0 9a
:3 0 a5 :3 0 e5
:3 0 e6 :3 0 e7
:3 0 e8 :3 0 e9
:3 0 ea :3 0 eb
:3 0 9a :3 0 a5
:3 0 175 :3 0 e5
:3 0 e5 :3 0 176
:3 0 e6 :3 0 e6
:3 0 177 :3 0 f0
:3 0 e7 :3 0 175
:3 0 159 :3 0 e8
:3 0 176 :3 0 153
:3 0 e9 :3 0 177
:3 0 15a :3 0 ea
:3 0 177 :3 0 133
:3 0 eb :3 0 76
:3 0 53 :3 0 d
:3 0 d8 :3 0 9a
:3 0 a5 :4 0 178
1 :8 0 1166 a7
:4 0 1151 :3 0 4e
:3 0 4f :2 0 66e
1154 1155 :3 0 9
:3 0 5b :3 0 1157
1158 0 16 :2 0
1159 115a 0 115c
671 1163 9 :3 0
5b :3 0 115d 115e
0 a8 :2 0 115f
1160 0 1162 673
1164 1156 115c 0
1165 0 1162 0
1165 675 0 1166
678 11b9 5d :3 0
9 :3 0 5b :3 0
1168 1169 0 5c
:3 0 116a 116b 0
116d 67b 116f 67d
116e 116d :2 0 11b7
5e :3 0 9 :3 0
5b :3 0 1172 1173
0 5c :3 0 1174
1175 0 11af 2c
:3 0 2d :3 0 1177
1178 0 2c :3 0
3d :3 0 117a 117b
0 5f :2 0 2
:4 0 179 :2 0 3f
:2 0 14 :2 0 67f
1180 1182 :3 0 682
117c 1184 1179 1186
1185 :2 0 2f :3 0
40 :3 0 1188 1189
0 31 :4 0 17a
:4 0 34 :2 0 35
:3 0 686 118d 118f
:3 0 34 :2 0 45
:4 0 689 1191 1193
:3 0 34 :2 0 62
:4 0 68c 1195 1197
:3 0 34 :2 0 5c
:3 0 68f 1199 119b
:3 0 34 :2 0 63
:4 0 692 119d 119f
:3 0 34 :2 0 64
:3 0 65 :3 0 14
:2 0 66 :2 0 695
11a2 11a6 699 11a1
11a8 :3 0 69c 118a
11aa :2 0 11ac 69f
11ad 1187 11ac 0
11ae 6a1 0 11af
6a3 11b2 :3 0 11b2
0 11b2 11b1 11af
11b0 :6 0 11b4 1e
:3 0 6a6 11b6 6a8
11b5 11b4 :2 0 11b7
6aa :2 0 11b9 0
11b9 11b8 1166 11b7
:6 0 12d4 1 :3 0
9 :3 0 5b :3 0
11bb 11bc 0 de
:4 0 11be 11bf :3 0
11bd 11c0 0 12d4
df :3 0 59 :3 0
b7 :3 0 e5 :3 0
e6 :3 0 e7 :3 0
e8 :3 0 e9 :3 0
ea :3 0 eb :3 0
a1 :3 0 ad :3 0
a1 :3 0 c9 :3 0
55 :3 0 e5 :3 0
55 :3 0 e6 :3 0
55 :3 0 e7 :3 0
55 :3 0 e8 :3 0
55 :3 0 e9 :3 0
55 :3 0 ea :3 0
55 :3 0 eb :3 0
174 :3 0 55 :3 0
a3 :3 0 ca :3 0
a1 :3 0 55 :3 0
a5 :3 0 a1 :3 0
ad :3 0 55 :3 0
9a :3 0 a3 :3 0
a4 :3 0 c4 :3 0
c4 :3 0 59 :3 0
a1 :3 0 cb :3 0
a3 :3 0 a4 :3 0
ba :3 0 ba :3 0
59 :3 0 c4 :3 0
9b :4 0 17b 1
:8 0 120d a7 :4 0
11f8 :3 0 4e :3 0
4f :2 0 6af 11fb
11fc :3 0 9 :3 0
5b :3 0 11fe 11ff
0 16 :2 0 1200
1201 0 1203 6b2
120a 9 :3 0 5b
:3 0 1204 1205 0
a8 :2 0 1206 1207
0 1209 6b4 120b
11fd 1203 0 120c
0 1209 0 120c
6b6 0 120d 6b9
1260 5d :3 0 9
:3 0 5b :3 0 120f
1210 0 5c :3 0
1211 1212 0 1214
6bc 1216 6be 1215
1214 :2 0 125e 5e
:3 0 9 :3 0 5b
:3 0 1219 121a 0
5c :3 0 121b 121c
0 1256 2c :3 0
2d :3 0 121e 121f
0 2c :3 0 3d
:3 0 1221 1222 0
5f :2 0 2 :4 0
17c :2 0 3f :2 0
14 :2 0 6c0 1227
1229 :3 0 6c3 1223
122b 1220 122d 122c
:2 0 2f :3 0 40
:3 0 122f 1230 0
31 :4 0 17d :4 0
34 :2 0 35 :3 0
6c7 1234 1236 :3 0
34 :2 0 45 :4 0
6ca 1238 123a :3 0
34 :2 0 62 :4 0
6cd 123c 123e :3 0
34 :2 0 5c :3 0
6d0 1240 1242 :3 0
34 :2 0 63 :4 0
6d3 1244 1246 :3 0
34 :2 0 64 :3 0
65 :3 0 14 :2 0
66 :2 0 6d6 1249
124d 6da 1248 124f
:3 0 6dd 1231 1251
:2 0 1253 6e0 1254
122e 1253 0 1255
6e2 0 1256 6e4
1259 :3 0 1259 0
1259 1258 1256 1257
:6 0 125b 20 :3 0
6e7 125d 6e9 125c
125b :2 0 125e 6eb
:2 0 1260 0 1260
125f 120d 125e :6 0
12d4 1 :3 0 b1
:3 0 21 :3 0 55
:3 0 9a :3 0 59
:3 0 ce :3 0 b5
:3 0 b6 :3 0 b7
:3 0 54 :3 0 b5
:3 0 b9 :3 0 55
:3 0 e5 :3 0 55
:3 0 e6 :3 0 55
:3 0 e7 :3 0 55
:3 0 e8 :3 0 55
:3 0 e9 :3 0 55
:3 0 ea :3 0 55
:3 0 eb :3 0 ec
:3 0 17 :3 0 56
:3 0 ed :3 0 ee
:3 0 ef :3 0 f0
:3 0 f1 :3 0 f2
:3 0 b5 :3 0 102
:3 0 c2 :3 0 82
:3 0 174 :3 0 55
:3 0 a3 :3 0 c3
:3 0 b5 :3 0 55
:3 0 a5 :3 0 55
:3 0 9a :3 0 b5
:3 0 9a :3 0 59
:3 0 ce :3 0 b7
:3 0 54 :3 0 b9
:3 0 e5 :3 0 e6
:3 0 e7 :3 0 e8
:3 0 e9 :3 0 ea
:3 0 eb :3 0 ec
:3 0 17 :3 0 56
:3 0 ed :3 0 ee
:3 0 ef :3 0 f0
:3 0 f1 :3 0 f2
:3 0 102 :3 0 c2
:3 0 82 :3 0 df
:3 0 54 :4 0 17e
1 :a 0 12b0 :2 0
12d4 1263 12ae :2 0
2c :3 0 2d :3 0
12b1 12b2 0 2c
:3 0 3d :3 0 12b4
12b5 0 14 :2 0
2 :4 0 17f :2 0
3f :2 0 14 :2 0
6ee 12ba 12bc :3 0
6f1 12b6 12be 12b3
12c0 12bf :2 0 2f
:3 0 40 :3 0 12c2
12c3 0 31 :4 0
180 :4 0 34 :2 0
35 :3 0 6f5 12c7
12c9 :3 0 6f8 12c4
12cb :2 0 12cd 6fb
12ce 12c1 12cd 0
12cf 6fd 0 12d4
8 :3 0 21 :3 0
12d1 :2 0 12d4 74
:3 0 6ff 12d5 111e
12d4 0 1fad 39
:3 0 15 :3 0 16
:2 0 708 12d6 12d9
4f :2 0 181 :2 0
70d 12db 12dd :3 0
96 :3 0 97 :3 0
173 :4 0 12e1 :4 0
12e2 :2 0 14df 9
:3 0 5b :3 0 12e4
12e5 0 16 :2 0
12e6 12e7 0 14df
174 :3 0 9a :3 0
a5 :3 0 e5 :3 0
e6 :3 0 e7 :3 0
e8 :3 0 e9 :3 0
ea :3 0 eb :3 0
9a :3 0 a5 :3 0
175 :3 0 e5 :3 0
e5 :3 0 176 :3 0
e6 :3 0 e6 :3 0
177 :3 0 f0 :3 0
e7 :3 0 175 :3 0
159 :3 0 e8 :3 0
176 :3 0 153 :3 0
e9 :3 0 177 :3 0
15a :3 0 ea :3 0
177 :3 0 133 :3 0
eb :3 0 76 :3 0
53 :3 0 d :3 0
d8 :3 0 9a :3 0
a5 :4 0 178 1
:8 0 1326 a7 :4 0
1311 :3 0 4e :3 0
4f :2 0 712 1314
1315 :3 0 9 :3 0
5b :3 0 1317 1318
0 16 :2 0 1319
131a 0 131c 715
1323 9 :3 0 5b
:3 0 131d 131e 0
a8 :2 0 131f 1320
0 1322 717 1324
1316 131c 0 1325
0 1322 0 1325
719 0 1326 71c
1379 5d :3 0 9
:3 0 5b :3 0 1328
1329 0 5c :3 0
132a 132b 0 132d
71f 132f 721 132e
132d :2 0 1377 5e
:3 0 9 :3 0 5b
:3 0 1332 1333 0
5c :3 0 1334 1335
0 136f 2c :3 0
2d :3 0 1337 1338
0 2c :3 0 3d
:3 0 133a 133b 0
5f :2 0 2 :4 0
182 :2 0 3f :2 0
14 :2 0 723 1340
1342 :3 0 726 133c
1344 1339 1346 1345
:2 0 2f :3 0 40
:3 0 1348 1349 0
31 :4 0 183 :4 0
34 :2 0 35 :3 0
72a 134d 134f :3 0
34 :2 0 45 :4 0
72d 1351 1353 :3 0
34 :2 0 62 :4 0
730 1355 1357 :3 0
34 :2 0 5c :3 0
733 1359 135b :3 0
34 :2 0 63 :4 0
736 135d 135f :3 0
34 :2 0 64 :3 0
65 :3 0 14 :2 0
66 :2 0 739 1362
1366 73d 1361 1368
:3 0 740 134a 136a
:2 0 136c 743 136d
1347 136c 0 136e
745 0 136f 747
1372 :3 0 1372 0
1372 1371 136f 1370
:6 0 1374 22 :3 0
74a 1376 74c 1375
1374 :2 0 1377 74e
:2 0 1379 0 1379
1378 1326 1377 :6 0
14df 1 :3 0 9
:3 0 5b :3 0 137b
137c 0 de :4 0
137e 137f :3 0 137d
1380 0 14df df
:3 0 59 :3 0 b7
:3 0 a5 :3 0 54
:3 0 b9 :3 0 e5
:3 0 e6 :3 0 e7
:3 0 e8 :3 0 e9
:3 0 ea :3 0 eb
:3 0 102 :3 0 a1
:3 0 ad :3 0 a1
:3 0 c9 :3 0 55
:3 0 e5 :3 0 55
:3 0 e6 :3 0 55
:3 0 e7 :3 0 55
:3 0 e8 :3 0 55
:3 0 e9 :3 0 55
:3 0 ea :3 0 55
:3 0 eb :3 0 174
:3 0 55 :3 0 a3
:3 0 ca :3 0 a1
:3 0 55 :3 0 a5
:3 0 a1 :3 0 ad
:3 0 55 :3 0 9a
:3 0 a3 :3 0 a4
:3 0 c4 :3 0 c4
:3 0 59 :3 0 a1
:3 0 cb :3 0 a3
:3 0 a4 :3 0 ba
:3 0 ba :3 0 59
:3 0 c4 :3 0 9b
:3 0 d7 :3 0 9a
:3 0 b5 :3 0 b6
:3 0 b5 :3 0 b9
:3 0 e4 :3 0 e4
:3 0 e4 :3 0 e4
:3 0 b5 :3 0 102
:3 0 a2 :3 0 d7
:3 0 c3 :3 0 b5
:3 0 d7 :3 0 53
:3 0 d :3 0 d7
:3 0 9a :3 0 b5
:3 0 9a :3 0 b2
:3 0 77 :3 0 76
:3 0 b2 :3 0 b2
:3 0 ce :3 0 55
:3 0 ce :3 0 a2
:3 0 55 :3 0 55
:3 0 53 :3 0 d7
:3 0 53 :3 0 55
:3 0 9a :3 0 d7
:3 0 9a :4 0 184
1 :8 0 13fa a7
:4 0 13e5 :3 0 4e
:3 0 4f :2 0 753
13e8 13e9 :3 0 9
:3 0 5b :3 0 13eb
13ec 0 16 :2 0
13ed 13ee 0 13f0
756 13f7 9 :3 0
5b :3 0 13f1 13f2
0 a8 :2 0 13f3
13f4 0 13f6 758
13f8 13ea 13f0 0
13f9 0 13f6 0
13f9 75a 0 13fa
75d 144d 5d :3 0
9 :3 0 5b :3 0
13fc 13fd 0 5c
:3 0 13fe 13ff 0
1401 760 1403 762
1402 1401 :2 0 144b
5e :3 0 9 :3 0
5b :3 0 1406 1407
0 5c :3 0 1408
1409 0 1443 2c
:3 0 2d :3 0 140b
140c 0 2c :3 0
3d :3 0 140e 140f
0 5f :2 0 2
:4 0 185 :2 0 3f
:2 0 14 :2 0 764
1414 1416 :3 0 767
1410 1418 140d 141a
1419 :2 0 2f :3 0
40 :3 0 141c 141d
0 31 :4 0 186
:4 0 34 :2 0 35
:3 0 76b 1421 1423
:3 0 34 :2 0 45
:4 0 76e 1425 1427
:3 0 34 :2 0 62
:4 0 771 1429 142b
:3 0 34 :2 0 5c
:3 0 774 142d 142f
:3 0 34 :2 0 63
:4 0 777 1431 1433
:3 0 34 :2 0 64
:3 0 65 :3 0 14
:2 0 66 :2 0 77a
1436 143a 77e 1435
143c :3 0 781 141e
143e :2 0 1440 784
1441 141b 1440 0
1442 786 0 1443
788 1446 :3 0 1446
0 1446 1445 1443
1444 :6 0 1448 24
:3 0 78b 144a 78d
1449 1448 :2 0 144b
78f :2 0 144d 0
144d 144c 13fa 144b
:6 0 14df 1 :3 0
b1 :3 0 22 :3 0
55 :3 0 9a :3 0
59 :3 0 ce :3 0
b5 :3 0 b6 :3 0
b7 :3 0 54 :3 0
b5 :3 0 b9 :3 0
55 :3 0 e5 :3 0
55 :3 0 e6 :3 0
55 :3 0 e7 :3 0
55 :3 0 e8 :3 0
55 :3 0 e9 :3 0
55 :3 0 ea :3 0
55 :3 0 eb :3 0
ec :3 0 17 :3 0
56 :3 0 ed :3 0
ee :3 0 ef :3 0
f0 :3 0 f1 :3 0
f2 :3 0 b5 :3 0
102 :3 0 c2 :3 0
82 :3 0 174 :3 0
55 :3 0 a3 :3 0
c3 :3 0 b5 :3 0
55 :3 0 a5 :3 0
55 :3 0 9a :3 0
b5 :3 0 9a :3 0
59 :3 0 ce :3 0
b7 :3 0 54 :3 0
b9 :3 0 e5 :3 0
e6 :3 0 e7 :3 0
e8 :3 0 e9 :3 0
ea :3 0 eb :3 0
ec :3 0 17 :3 0
56 :3 0 ed :3 0
ee :3 0 ef :3 0
f0 :3 0 f1 :3 0
f2 :3 0 102 :3 0
c2 :3 0 82 :3 0
df :3 0 a5 :3 0
54 :3 0 59 :3 0
ce :3 0 b7 :3 0
54 :3 0 b9 :3 0
e5 :3 0 e6 :3 0
e7 :3 0 e8 :3 0
e9 :3 0 ea :3 0
eb :3 0 ec :3 0
17 :3 0 56 :3 0
ed :3 0 ee :3 0
ef :3 0 f0 :3 0
f1 :3 0 f2 :3 0
102 :3 0 c2 :3 0
82 :3 0 df :3 0
a5 :3 0 54 :3 0
54 :3 0 b7 :4 0
187 1 :a 0 14bb
:2 0 14df 1450 14b9
:2 0 2c :3 0 2d
:3 0 14bc 14bd 0
2c :3 0 3d :3 0
14bf 14c0 0 14
:2 0 2 :4 0 188
:2 0 3f :2 0 14
:2 0 792 14c5 14c7
:3 0 795 14c1 14c9
14be 14cb 14ca :2 0
2f :3 0 40 :3 0
14cd 14ce 0 31
:4 0 189 :4 0 34
:2 0 35 :3 0 799
14d2 14d4 :3 0 79c
14cf 14d6 :2 0 14d8
79f 14d9 14cc 14d8
0 14da 7a1 0
14df 8 :3 0 22
:3 0 14dc :2 0 14df
74 :3 0 7a3 14e0
12de 14df 0 1fad
39 :3 0 15 :3 0
16 :2 0 7ac 14e1
14e4 4f :2 0 18a
:2 0 7b1 14e6 14e8
:3 0 b1 :3 0 23
:3 0 d9 :3 0 9a
:3 0 59 :3 0 d9
:3 0 ce :3 0 b5
:3 0 b6 :3 0 b7
:3 0 54 :3 0 b5
:3 0 b9 :3 0 52
:3 0 e5 :3 0 52
:3 0 e6 :3 0 52
:3 0 16e :3 0 e7
:3 0 52 :3 0 f0
:3 0 18b :3 0 55
:3 0 12e :3 0 e8
:3 0 55 :3 0 12f
:3 0 e9 :3 0 55
:3 0 16e :3 0 ea
:3 0 55 :3 0 15a
:3 0 eb :3 0 17
:3 0 56 :3 0 ed
:3 0 ee :3 0 ef
:3 0 f0 :3 0 f1
:3 0 13a :3 0 13b
:3 0 f2 :3 0 b5
:3 0 102 :3 0 c2
:3 0 82 :3 0 18c
:3 0 be :3 0 18c
:3 0 100 :3 0 18c
:3 0 c0 :3 0 18c
:3 0 101 :3 0 18c
:3 0 18d :3 0 18c
:3 0 18e :3 0 18c
:3 0 18f :3 0 18c
:3 0 190 :3 0 18c
:3 0 191 :3 0 18c
:3 0 192 :3 0 18c
:3 0 193 :3 0 18c
:3 0 194 :3 0 18c
:3 0 195 :3 0 18c
:3 0 196 :3 0 18c
:3 0 197 :3 0 a2
:3 0 d9 :3 0 a3
:3 0 198 :3 0 18c
:3 0 18c :3 0 53
:3 0 d :3 0 18c
:3 0 9a :3 0 d9
:3 0 9a :3 0 c5
:3 0 a3 :3 0 76
:3 0 55 :3 0 55
:3 0 ce :3 0 d9
:3 0 ce :3 0 55
:3 0 119 :3 0 c5
:3 0 a3 :3 0 76
:3 0 52 :3 0 52
:3 0 ce :3 0 d9
:3 0 ce :3 0 52
:3 0 119 :3 0 a3
:3 0 c3 :3 0 b5
:3 0 d9 :3 0 9a
:3 0 b5 :3 0 9a
:3 0 d9 :3 0 59
:3 0 d :3 0 d9
:3 0 a5 :3 0 55
:3 0 77 :3 0 52
:3 0 77 :3 0 d9
:3 0 9a :3 0 59
:3 0 55 :3 0 ce
:3 0 b5 :3 0 c9
:3 0 b7 :3 0 54
:3 0 b9 :3 0 d9
:3 0 e5 :3 0 d9
:3 0 e6 :3 0 d9
:3 0 16e :3 0 e7
:3 0 d9 :3 0 f0
:3 0 18b :3 0 e4
:3 0 e8 :3 0 e4
:3 0 e9 :3 0 ea
:3 0 eb :3 0 17
:3 0 56 :3 0 ed
:3 0 ee :3 0 ef
:3 0 f0 :3 0 f1
:3 0 d9 :3 0 f2
:3 0 102 :3 0 c2
:3 0 82 :3 0 18c
:3 0 be :3 0 18c
:3 0 100 :3 0 18c
:3 0 c0 :3 0 18c
:3 0 101 :3 0 18c
:3 0 18d :3 0 18c
:3 0 18e :3 0 18c
:3 0 18f :3 0 18c
:3 0 190 :3 0 18c
:3 0 191 :3 0 18c
:3 0 192 :3 0 18c
:3 0 193 :3 0 18c
:3 0 194 :3 0 18c
:3 0 195 :3 0 18c
:3 0 196 :3 0 18c
:3 0 197 :3 0 a2
:3 0 55 :3 0 a3
:3 0 ca :3 0 b5
:3 0 55 :3 0 9a
:3 0 b5 :3 0 ad
:3 0 a3 :3 0 76
:3 0 d9 :3 0 d9
:3 0 ce :3 0 55
:3 0 ce :3 0 a3
:3 0 198 :3 0 18c
:3 0 18c :3 0 53
:3 0 d :3 0 18c
:3 0 9a :3 0 55
:3 0 9a :3 0 55
:3 0 59 :3 0 d
:3 0 55 :3 0 a5
:3 0 54 :3 0 b7
:4 0 199 1 :a 0
15d0 :2 0 15f4 14eb
15ce :2 0 2c :3 0
2d :3 0 15d1 15d2
0 2c :3 0 3d
:3 0 15d4 15d5 0
14 :2 0 2 :4 0
19a :2 0 3f :2 0
14 :2 0 7b4 15da
15dc :3 0 7b7 15d6
15de 15d3 15e0 15df
:2 0 2f :3 0 40
:3 0 15e2 15e3 0
31 :4 0 19b :4 0
34 :2 0 35 :3 0
7bb 15e7 15e9 :3 0
7be 15e4 15eb :2 0
15ed 7c1 15ee 15e1
15ed 0 15ef 7c3
0 15f4 8 :3 0
23 :3 0 15f1 :2 0
15f4 74 :3 0 7c5
15f5 14e9 15f4 0
1fad 39 :3 0 15
:3 0 16 :2 0 7c9
15f6 15f9 4f :2 0
19c :2 0 7ce 15fb
15fd :3 0 96 :3 0
97 :3 0 19d :4 0
1601 :4 0 1602 :2 0
18ff 9 :3 0 5b
:3 0 1604 1605 0
16 :2 0 1606 1607
0 18ff 19e :3 0
9a :3 0 a5 :3 0
19f :3 0 1a0 :3 0
18b :3 0 e7 :3 0
1a1 :3 0 1a2 :3 0
1a3 :3 0 ea :3 0
54 :3 0 9a :3 0
a5 :3 0 175 :3 0
e5 :3 0 19f :3 0
176 :3 0 e6 :3 0
1a0 :3 0 177 :3 0
f0 :3 0 18b :3 0
177 :3 0 119 :3 0
16e :3 0 e7 :3 0
175 :3 0 159 :3 0
1a1 :3 0 176 :3 0
153 :3 0 1a2 :3 0
177 :3 0 15a :3 0
1a3 :3 0 177 :3 0
119 :3 0 16e :3 0
ea :3 0 76 :3 0
53 :3 0 d :3 0
da :3 0 d :3 0
9a :3 0 a5 :4 0
1a4 1 :8 0 164e
a7 :4 0 1639 :3 0
4e :3 0 4f :2 0
7d3 163c 163d :3 0
9 :3 0 5b :3 0
163f 1640 0 16
:2 0 1641 1642 0
1644 7d6 164b 9
:3 0 5b :3 0 1645
1646 0 a8 :2 0
1647 1648 0 164a
7d8 164c 163e 1644
0 164d 0 164a
0 164d 7da 0
164e 7dd 16a1 5d
:3 0 9 :3 0 5b
:3 0 1650 1651 0
5c :3 0 1652 1653
0 1655 7e0 1657
7e2 1656 1655 :2 0
169f 5e :3 0 9
:3 0 5b :3 0 165a
165b 0 5c :3 0
165c 165d 0 1697
2c :3 0 2d :3 0
165f 1660 0 2c
:3 0 3d :3 0 1662
1663 0 5f :2 0
2 :4 0 1a5 :2 0
3f :2 0 14 :2 0
7e4 1668 166a :3 0
7e7 1664 166c 1661
166e 166d :2 0 2f
:3 0 40 :3 0 1670
1671 0 31 :4 0
1a6 :4 0 34 :2 0
35 :3 0 7eb 1675
1677 :3 0 34 :2 0
45 :4 0 7ee 1679
167b :3 0 34 :2 0
62 :4 0 7f1 167d
167f :3 0 34 :2 0
5c :3 0 7f4 1681
1683 :3 0 34 :2 0
63 :4 0 7f7 1685
1687 :3 0 34 :2 0
64 :3 0 65 :3 0
14 :2 0 66 :2 0
7fa 168a 168e 7fe
1689 1690 :3 0 801
1672 1692 :2 0 1694
804 1695 166f 1694
0 1696 806 0
1697 808 169a :3 0
169a 0 169a 1699
1697 1698 :6 0 169c
26 :3 0 80b 169e
80d 169d 169c :2 0
169f 80f :2 0 16a1
0 16a1 16a0 164e
169f :6 0 18ff 1
:3 0 96 :3 0 97
:3 0 1a7 :4 0 16a5
:4 0 16a6 :2 0 18ff
9 :3 0 5b :3 0
16a8 16a9 0 16
:2 0 16aa 16ab 0
18ff 1a8 :3 0 9a
:3 0 a5 :3 0 e5
:3 0 e6 :3 0 e7
:3 0 e8 :3 0 e9
:3 0 ea :3 0 eb
:3 0 19f :3 0 1a0
:3 0 18b :3 0 1a1
:3 0 1a2 :3 0 1a3
:3 0 18e :3 0 18f
:3 0 190 :3 0 191
:3 0 192 :3 0 193
:3 0 194 :3 0 195
:3 0 196 :3 0 197
:3 0 be :3 0 100
:3 0 c0 :3 0 101
:3 0 54 :3 0 18d
:3 0 55 :3 0 9a
:3 0 55 :3 0 a5
:3 0 55 :3 0 e5
:3 0 55 :3 0 e6
:3 0 55 :3 0 f0
:3 0 55 :3 0 159
:3 0 55 :3 0 153
:3 0 55 :3 0 15a
:3 0 55 :3 0 133
:3 0 52 :3 0 19f
:3 0 52 :3 0 1a0
:3 0 52 :3 0 18b
:3 0 52 :3 0 1a1
:3 0 52 :3 0 1a2
:3 0 52 :3 0 1a3
:3 0 55 :3 0 18e
:3 0 55 :3 0 18f
:3 0 55 :3 0 190
:3 0 55 :3 0 191
:3 0 55 :3 0 192
:3 0 55 :3 0 193
:3 0 55 :3 0 194
:3 0 55 :3 0 195
:3 0 55 :3 0 196
:3 0 55 :3 0 197
:3 0 55 :3 0 be
:3 0 55 :3 0 100
:3 0 55 :3 0 c0
:3 0 55 :3 0 101
:3 0 39 :3 0 52
:3 0 54 :3 0 55
:3 0 f0 :3 0 55
:3 0 15a :3 0 55
:3 0 18d :3 0 198
:3 0 55 :3 0 c5
:3 0 c6 :3 0 a3
:3 0 19e :3 0 52
:3 0 55 :3 0 9a
:3 0 52 :3 0 9a
:3 0 55 :3 0 53
:3 0 d :4 0 1a9
1 :8 0 1734 a7
:4 0 171f :3 0 4e
:3 0 4f :2 0 814
1722 1723 :3 0 9
:3 0 5b :3 0 1725
1726 0 16 :2 0
1727 1728 0 172a
817 1731 9 :3 0
5b :3 0 172b 172c
0 a8 :2 0 172d
172e 0 1730 819
1732 1724 172a 0
1733 0 1730 0
1733 81b 0 1734
81e 1787 5d :3 0
9 :3 0 5b :3 0
1736 1737 0 5c
:3 0 1738 1739 0
173b 821 173d 823
173c 173b :2 0 1785
5e :3 0 9 :3 0
5b :3 0 1740 1741
0 5c :3 0 1742
1743 0 177d 2c
:3 0 2d :3 0 1745
1746 0 2c :3 0
3d :3 0 1748 1749
0 5f :2 0 2
:4 0 1aa :2 0 3f
:2 0 14 :2 0 825
174e 1750 :3 0 828
174a 1752 1747 1754
1753 :2 0 2f :3 0
40 :3 0 1756 1757
0 31 :4 0 1ab
:4 0 34 :2 0 35
:3 0 82c 175b 175d
:3 0 34 :2 0 45
:4 0 82f 175f 1761
:3 0 34 :2 0 62
:4 0 832 1763 1765
:3 0 34 :2 0 5c
:3 0 835 1767 1769
:3 0 34 :2 0 63
:4 0 838 176b 176d
:3 0 34 :2 0 64
:3 0 65 :3 0 14
:2 0 66 :2 0 83b
1770 1774 83f 176f
1776 :3 0 842 1758
1778 :2 0 177a 845
177b 1755 177a 0
177c 847 0 177d
849 1780 :3 0 1780
0 1780 177f 177d
177e :6 0 1782 28
:3 0 84c 1784 84e
1783 1782 :2 0 1785
850 :2 0 1787 0
1787 1786 1734 1785
:6 0 18ff 1 :3 0
1a8 :3 0 1ac :3 0
54 :3 0 54 :3 0
da :3 0 76 :3 0
d9 :3 0 d9 :3 0
53 :3 0 d :3 0
d9 :3 0 9a :3 0
1ac :3 0 9a :3 0
d9 :3 0 a5 :4 0
1ad 1 :8 0 17af
a7 :4 0 179a :3 0
4e :3 0 4f :2 0
855 179d 179e :3 0
9 :3 0 5b :3 0
17a0 17a1 0 16
:2 0 17a2 17a3 0
17a5 858 17ac 9
:3 0 5b :3 0 17a6
17a7 0 a8 :2 0
17a8 17a9 0 17ab
85a 17ad 179f 17a5
0 17ae 0 17ab
0 17ae 85c 0
17af 85f 1802 5d
:3 0 9 :3 0 5b
:3 0 17b1 17b2 0
5c :3 0 17b3 17b4
0 17b6 862 17b8
864 17b7 17b6 :2 0
1800 5e :3 0 9
:3 0 5b :3 0 17bb
17bc 0 5c :3 0
17bd 17be 0 17f8
2c :3 0 2d :3 0
17c0 17c1 0 2c
:3 0 3d :3 0 17c3
17c4 0 5f :2 0
2 :4 0 1ae :2 0
3f :2 0 14 :2 0
866 17c9 17cb :3 0
869 17c5 17cd 17c2
17cf 17ce :2 0 2f
:3 0 40 :3 0 17d1
17d2 0 31 :4 0
1af :4 0 34 :2 0
35 :3 0 86d 17d6
17d8 :3 0 34 :2 0
45 :4 0 870 17da
17dc :3 0 34 :2 0
62 :4 0 873 17de
17e0 :3 0 34 :2 0
5c :3 0 876 17e2
17e4 :3 0 34 :2 0
63 :4 0 879 17e6
17e8 :3 0 34 :2 0
64 :3 0 65 :3 0
14 :2 0 66 :2 0
87c 17eb 17ef 880
17ea 17f1 :3 0 883
17d3 17f3 :2 0 17f5
886 17f6 17d0 17f5
0 17f7 888 0
17f8 88a 17fb :3 0
17fb 0 17fb 17fa
17f8 17f9 :6 0 17fd
2a :3 0 88d 17ff
88f 17fe 17fd :2 0
1800 891 :2 0 1802
0 1802 1801 17af
1800 :6 0 18ff 1
:3 0 b1 :3 0 24
:3 0 55 :3 0 9a
:3 0 59 :3 0 ce
:3 0 b5 :3 0 b6
:3 0 b7 :3 0 55
:3 0 54 :3 0 b5
:3 0 b9 :3 0 55
:3 0 e5 :3 0 55
:3 0 e6 :3 0 55
:3 0 e7 :3 0 55
:3 0 e8 :3 0 55
:3 0 e9 :3 0 55
:3 0 ea :3 0 55
:3 0 eb :3 0 ec
:3 0 17 :3 0 56
:3 0 ed :3 0 ee
:3 0 ef :3 0 f0
:3 0 f1 :3 0 f2
:3 0 b5 :3 0 102
:3 0 c2 :3 0 82
:3 0 55 :3 0 19f
:3 0 55 :3 0 1a1
:3 0 55 :3 0 9a
:3 0 d8 :3 0 55
:3 0 19f :3 0 55
:3 0 1a0 :3 0 55
:3 0 18b :3 0 55
:3 0 1a1 :3 0 55
:3 0 1a2 :3 0 55
:3 0 1a3 :3 0 d
:3 0 53 :3 0 55
:3 0 18e :3 0 55
:3 0 18f :3 0 55
:3 0 190 :3 0 55
:3 0 191 :3 0 55
:3 0 192 :3 0 55
:3 0 193 :3 0 55
:3 0 194 :3 0 55
:3 0 195 :3 0 55
:3 0 196 :3 0 55
:3 0 197 :3 0 55
:3 0 be :3 0 55
:3 0 100 :3 0 55
:3 0 c0 :3 0 55
:3 0 101 :3 0 55
:3 0 18d :3 0 1a8
:3 0 55 :3 0 a3
:3 0 c3 :3 0 b5
:3 0 55 :3 0 a5
:3 0 55 :3 0 9a
:3 0 b5 :3 0 9a
:3 0 a1 :3 0 ad
:3 0 59 :3 0 ce
:3 0 a1 :3 0 c9
:3 0 b7 :3 0 54
:3 0 b9 :3 0 55
:3 0 e5 :3 0 55
:3 0 e6 :3 0 55
:3 0 e7 :3 0 55
:3 0 e8 :3 0 55
:3 0 e9 :3 0 55
:3 0 ea :3 0 55
:3 0 eb :3 0 ec
:3 0 17 :3 0 56
:3 0 ed :3 0 ee
:3 0 ef :3 0 f0
:3 0 f1 :3 0 f2
:3 0 102 :3 0 c2
:3 0 82 :3 0 55
:3 0 19f :3 0 55
:3 0 1a1 :3 0 55
:3 0 9a :3 0 d8
:3 0 55 :3 0 19f
:3 0 55 :3 0 1a0
:3 0 55 :3 0 18b
:3 0 55 :3 0 1a1
:3 0 55 :3 0 1a2
:3 0 55 :3 0 1a3
:3 0 d :3 0 53
:3 0 55 :3 0 18e
:3 0 55 :3 0 18f
:3 0 55 :3 0 190
:3 0 55 :3 0 191
:3 0 55 :3 0 192
:3 0 55 :3 0 193
:3 0 55 :3 0 194
:3 0 55 :3 0 195
:3 0 55 :3 0 196
:3 0 55 :3 0 197
:3 0 55 :3 0 be
:3 0 55 :3 0 100
:3 0 55 :3 0 c0
:3 0 55 :3 0 101
:3 0 55 :3 0 18d
:3 0 1a8 :3 0 55
:3 0 a3 :3 0 ca
:3 0 a1 :3 0 55
:3 0 a5 :3 0 a1
:3 0 ad :3 0 55
:3 0 9a :3 0 a3
:3 0 a4 :3 0 c4
:3 0 c4 :3 0 59
:3 0 a1 :3 0 cb
:3 0 a3 :3 0 a4
:3 0 ba :3 0 ba
:3 0 59 :3 0 c4
:3 0 9b :4 0 1b0
1 :a 0 18db :2 0
18ff 1805 18d9 :2 0
2c :3 0 2d :3 0
18dc 18dd 0 2c
:3 0 3d :3 0 18df
18e0 0 14 :2 0
2 :4 0 1b1 :2 0
3f :2 0 14 :2 0
894 18e5 18e7 :3 0
897 18e1 18e9 18de
18eb 18ea :2 0 2f
:3 0 40 :3 0 18ed
18ee 0 31 :4 0
1b2 :4 0 34 :2 0
35 :3 0 89b 18f2
18f4 :3 0 89e 18ef
18f6 :2 0 18f8 8a1
18f9 18ec 18f8 0
18fa 8a3 0 18ff
8 :3 0 24 :3 0
18fc :2 0 18ff 74
:3 0 8a5 1900 15fe
18ff 0 1fad 39
:3 0 15 :3 0 16
:2 0 8b0 1901 1904
47 :3 0 4f :2 0
8c :3 0 1906 1908
0 8b5 1907 190a
:3 0 c9 :3 0 11
:3 0 ca :3 0 ad
:3 0 d :4 0 1b3
1 :8 0 1917 9
:3 0 5b :3 0 1912
1913 0 5c :3 0
1914 1915 0 1917
8b8 196a 5d :3 0
9 :3 0 5b :3 0
1919 191a 0 5c
:3 0 191b 191c 0
191e 8bb 1920 8bd
191f 191e :2 0 1968
5e :3 0 9 :3 0
5b :3 0 1923 1924
0 5c :3 0 1925
1926 0 1960 2c
:3 0 2d :3 0 1928
1929 0 2c :3 0
3d :3 0 192b 192c
0 5f :2 0 2
:4 0 1b4 :2 0 3f
:2 0 14 :2 0 8bf
1931 1933 :3 0 8c2
192d 1935 192a 1937
1936 :2 0 2f :3 0
40 :3 0 1939 193a
0 31 :4 0 1b5
:4 0 34 :2 0 35
:3 0 8c6 193e 1940
:3 0 34 :2 0 45
:4 0 8c9 1942 1944
:3 0 34 :2 0 62
:4 0 8cc 1946 1948
:3 0 34 :2 0 5c
:3 0 8cf 194a 194c
:3 0 34 :2 0 63
:4 0 8d2 194e 1950
:3 0 34 :2 0 64
:3 0 65 :3 0 14
:2 0 66 :2 0 8d5
1953 1957 8d9 1952
1959 :3 0 8dc 193b
195b :2 0 195d 8df
195e 1938 195d 0
195f 8e1 0 1960
8e3 1963 :3 0 1963
0 1963 1962 1960
1961 :6 0 1965 2c
:3 0 8e6 1967 8e8
1966 1965 :2 0 1968
8ea :2 0 196a 0
196a 1969 1917 1968
:6 0 1a39 1 :3 0
9 :3 0 5b :3 0
196c 196d 0 9
:3 0 4f :2 0 1b6
:3 0 196f 1971 0
8ef 1970 1973 :3 0
1974 :2 0 b1 :3 0
25 :3 0 d :3 0
59 :3 0 11 :3 0
b7 :3 0 54 :3 0
1b7 :3 0 b5 :3 0
cb :3 0 11 :3 0
bb :3 0 1b8 :3 0
1b9 :3 0 b7 :3 0
1ba :3 0 1bb :3 0
1bc :3 0 b7 :3 0
1bd :3 0 1be :3 0
1bf :3 0 b7 :3 0
1c0 :3 0 ca :3 0
b5 :3 0 c5 :3 0
a3 :3 0 169 :3 0
1b9 :3 0 1b9 :3 0
59 :3 0 1b8 :3 0
c5 :3 0 a3 :3 0
1c1 :3 0 1bc :3 0
1bc :3 0 59 :3 0
1bb :3 0 c5 :3 0
a3 :3 0 6c :3 0
1bf :3 0 1bf :3 0
59 :3 0 1be :3 0
b5 :3 0 ad :3 0
d :3 0 d :3 0
59 :3 0 11 :3 0
b7 :3 0 54 :3 0
1c2 :3 0 54 :3 0
1b7 :3 0 1c3 :3 0
59 :3 0 cb :3 0
1c4 :3 0 1c5 :3 0
1c4 :3 0 1c5 :3 0
1c6 :3 0 1c2 :3 0
b7 :3 0 1c6 :3 0
1c3 :3 0 b7 :3 0
bb :3 0 1b8 :3 0
1ba :3 0 1bb :3 0
1bd :3 0 1be :3 0
1c0 :3 0 d0 :3 0
a1 :3 0 c5 :3 0
c6 :3 0 a3 :3 0
a4 :3 0 1c3 :3 0
a1 :3 0 cb :3 0
1c3 :3 0 59 :3 0
c5 :3 0 c6 :3 0
a3 :3 0 a4 :3 0
1c2 :3 0 1c2 :3 0
59 :3 0 1c3 :3 0
9b :3 0 a1 :3 0
ce :3 0 d :3 0
d :3 0 59 :3 0
11 :3 0 b7 :3 0
54 :3 0 1c2 :3 0
54 :3 0 1b7 :3 0
1c3 :3 0 59 :3 0
cb :3 0 1c4 :3 0
1c5 :3 0 1c4 :3 0
1c5 :3 0 1c6 :3 0
1c2 :3 0 b7 :3 0
1c6 :3 0 1c3 :3 0
b7 :3 0 bb :3 0
1b8 :3 0 1ba :3 0
1bb :3 0 1bd :3 0
1be :3 0 1c0 :3 0
cc :3 0 cd :3 0
c5 :3 0 c6 :3 0
a3 :3 0 a4 :3 0
1c3 :3 0 cd :3 0
cf :3 0 1c3 :3 0
59 :3 0 c5 :3 0
c6 :3 0 a3 :3 0
a4 :3 0 1c2 :3 0
1c2 :3 0 59 :3 0
1c3 :3 0 9b :3 0
cd :3 0 ce :3 0
d :3 0 59 :3 0
54 :4 0 1c7 1
:a 0 1a12 :2 0 1a35
1977 1a10 :2 0 2c
:3 0 2d :3 0 1a13
1a14 0 2c :3 0
3d :3 0 1a16 1a17
0 14 :2 0 2
:4 0 1c8 :2 0 3f
:2 0 14 :2 0 8f2
1a1c 1a1e :3 0 8f5
1a18 1a20 1a15 1a22
1a21 :2 0 2f :3 0
40 :3 0 1a24 1a25
0 31 :4 0 1c9
:4 0 34 :2 0 35
:3 0 8f9 1a29 1a2b
:3 0 8fc 1a26 1a2d
:2 0 1a2f 8ff 1a30
1a23 1a2f 0 1a31
901 0 1a35 8
:3 0 25 :3 0 1a33
:2 0 1a35 903 1a36
1975 1a35 0 1a37
907 0 1a39 74
:3 0 909 1a3a 190b
1a39 0 1fad 39
:3 0 15 :3 0 16
:2 0 90c 1a3b 1a3e
4f :2 0 75 :2 0
911 1a40 1a42 :3 0
b1 :3 0 26 :3 0
d9 :3 0 77 :3 0
d9 :3 0 77 :3 0
55 :3 0 ce :3 0
59 :3 0 55 :3 0
9a :3 0 55 :3 0
a5 :3 0 b5 :3 0
c9 :3 0 ca :3 0
b5 :3 0 55 :3 0
9a :3 0 b5 :3 0
ad :3 0 a1 :3 0
b6 :3 0 c3 :3 0
a1 :3 0 55 :3 0
9a :3 0 a1 :3 0
9a :3 0 b7 :3 0
d9 :3 0 77 :3 0
55 :3 0 a5 :3 0
54 :3 0 55 :3 0
ce :3 0 55 :3 0
f6 :3 0 d9 :3 0
f7 :3 0 39 :3 0
e6 :3 0 153 :3 0
f8 :3 0 d9 :3 0
125 :3 0 d9 :3 0
126 :3 0 39 :3 0
d9 :3 0 e5 :3 0
d9 :3 0 159 :3 0
e5 :3 0 39 :3 0
e6 :3 0 153 :3 0
e6 :3 0 119 :3 0
15a :3 0 f0 :3 0
f0 :3 0 119 :3 0
15a :3 0 128 :3 0
f0 :3 0 eb :3 0
d9 :3 0 15b :3 0
d9 :3 0 15c :3 0
d9 :3 0 15b :3 0
d9 :3 0 15c :3 0
129 :3 0 12a :3 0
d9 :3 0 128 :3 0
12b :3 0 12c :3 0
d9 :3 0 128 :3 0
12c :3 0 f0 :3 0
15a :3 0 12d :3 0
d9 :3 0 12e :3 0
d9 :3 0 12f :3 0
d9 :3 0 130 :3 0
d9 :3 0 131 :3 0
d9 :3 0 132 :3 0
d9 :3 0 133 :3 0
d9 :3 0 120 :3 0
d9 :3 0 134 :3 0
d9 :3 0 128 :3 0
135 :3 0 136 :3 0
137 :3 0 139 :3 0
17 :3 0 56 :3 0
55 :3 0 f9 :3 0
55 :3 0 fa :3 0
fb :3 0 b7 :3 0
fc :3 0 39 :3 0
55 :3 0 fd :3 0
55 :3 0 fa :3 0
fd :3 0 fe :3 0
b7 :3 0 ff :3 0
d9 :3 0 be :3 0
d9 :3 0 100 :3 0
d9 :3 0 c0 :3 0
d9 :3 0 101 :3 0
55 :3 0 a5 :3 0
a1 :3 0 102 :3 0
c3 :3 0 a1 :3 0
55 :3 0 9a :3 0
a1 :3 0 9a :3 0
102 :3 0 55 :3 0
103 :3 0 d9 :3 0
f2 :3 0 d9 :3 0
13c :3 0 d9 :3 0
13d :3 0 d9 :3 0
13f :3 0 d9 :3 0
d8 :3 0 d9 :3 0
104 :3 0 105 :3 0
b7 :3 0 140 :3 0
d9 :3 0 ec :3 0
d9 :3 0 107 :3 0
d9 :3 0 108 :3 0
d9 :3 0 109 :3 0
d9 :3 0 10a :3 0
d9 :3 0 10b :3 0
d9 :3 0 10c :3 0
d9 :3 0 141 :3 0
d9 :3 0 10d :3 0
d9 :3 0 119 :3 0
d9 :3 0 119 :3 0
119 :3 0 d9 :3 0
142 :3 0 143 :3 0
119 :3 0 15d :3 0
15e :3 0 144 :3 0
1b :3 0 39 :3 0
d9 :3 0 145 :3 0
d9 :3 0 15f :3 0
e4 :3 0 145 :3 0
1b :3 0 39 :3 0
d9 :3 0 146 :3 0
d9 :3 0 160 :3 0
e4 :3 0 146 :3 0
1b :3 0 d9 :3 0
148 :3 0 d9 :3 0
133 :3 0 d9 :3 0
148 :3 0 148 :3 0
d9 :3 0 12a :3 0
149 :3 0 14a :3 0
d9 :3 0 77 :3 0
d9 :3 0 d8 :3 0
d9 :3 0 77 :3 0
d9 :3 0 145 :3 0
d9 :3 0 15f :3 0
d9 :3 0 146 :3 0
d9 :3 0 160 :3 0
14b :3 0 d9 :3 0
14c :3 0 161 :3 0
b7 :3 0 14d :3 0
d9 :3 0 14e :3 0
162 :3 0 b7 :3 0
14f :3 0 d9 :3 0
150 :3 0 d9 :3 0
151 :3 0 163 :3 0
b7 :3 0 152 :3 0
a2 :3 0 55 :3 0
a3 :3 0 76 :3 0
d9 :3 0 d9 :3 0
77 :3 0 d :3 0
55 :3 0 ce :3 0
d9 :3 0 ce :3 0
c5 :3 0 c6 :3 0
a3 :3 0 10e :3 0
fb :3 0 55 :3 0
fa :3 0 fb :3 0
59 :3 0 c5 :3 0
c6 :3 0 a3 :3 0
10e :3 0 fe :3 0
39 :3 0 55 :3 0
fd :3 0 55 :3 0
fa :3 0 fe :3 0
59 :3 0 c5 :3 0
c6 :3 0 a3 :3 0
10f :3 0 105 :3 0
105 :3 0 59 :3 0
d9 :3 0 104 :3 0
c5 :3 0 c6 :3 0
a3 :3 0 164 :3 0
161 :3 0 161 :3 0
59 :3 0 d9 :3 0
14c :3 0 c5 :3 0
c6 :3 0 a3 :3 0
164 :3 0 162 :3 0
162 :3 0 59 :3 0
d9 :3 0 14e :3 0
c5 :3 0 c6 :3 0
a3 :3 0 165 :3 0
163 :3 0 163 :3 0
59 :3 0 d9 :3 0
151 :3 0 54 :3 0
b7 :4 0 1ca 1
:a 0 1b8b :2 0 1baf
1a45 1b89 :2 0 2c
:3 0 2d :3 0 1b8c
1b8d 0 2c :3 0
3d :3 0 1b8f 1b90
0 14 :2 0 2
:4 0 1cb :2 0 3f
:2 0 14 :2 0 914
1b95 1b97 :3 0 917
1b91 1b99 1b8e 1b9b
1b9a :2 0 2f :3 0
40 :3 0 1b9d 1b9e
0 31 :4 0 1cc
:4 0 34 :2 0 35
:3 0 91b 1ba2 1ba4
:3 0 91e 1b9f 1ba6
:2 0 1ba8 921 1ba9
1b9c 1ba8 0 1baa
923 0 1baf 8
:3 0 26 :3 0 1bac
:2 0 1baf 74 :3 0
925 1bb0 1a43 1baf
0 1fad 39 :3 0
15 :3 0 16 :2 0
929 1bb1 1bb4 4f
:2 0 8d :2 0 92e
1bb6 1bb8 :3 0 b1
:3 0 27 :3 0 55
:3 0 ce :3 0 59
:3 0 55 :3 0 9a
:3 0 55 :3 0 a5
:3 0 b5 :3 0 c9
:3 0 ca :3 0 b5
:3 0 55 :3 0 9a
:3 0 b5 :3 0 ad
:3 0 a1 :3 0 b6
:3 0 c3 :3 0 a1
:3 0 55 :3 0 9a
:3 0 a1 :3 0 9a
:3 0 b7 :3 0 55
:3 0 a5 :3 0 47
:3 0 16a :3 0 54
:3 0 55 :3 0 ce
:3 0 55 :3 0 f6
:3 0 e4 :3 0 e5
:3 0 e4 :3 0 e6
:3 0 f0 :3 0 e4
:3 0 f7 :3 0 e4
:3 0 f8 :3 0 e4
:3 0 159 :3 0 e4
:3 0 153 :3 0 15a
:3 0 e4 :3 0 12e
:3 0 e4 :3 0 12f
:3 0 130 :3 0 e4
:3 0 131 :3 0 e4
:3 0 132 :3 0 133
:3 0 e4 :3 0 120
:3 0 e4 :3 0 134
:3 0 135 :3 0 17
:3 0 56 :3 0 55
:3 0 f9 :3 0 e4
:3 0 1cd :3 0 55
:3 0 fa :3 0 fb
:3 0 b7 :3 0 fc
:3 0 39 :3 0 55
:3 0 fd :3 0 55
:3 0 fa :3 0 fd
:3 0 fe :3 0 b7
:3 0 ff :3 0 be
:3 0 100 :3 0 c0
:3 0 101 :3 0 55
:3 0 a5 :3 0 a1
:3 0 102 :3 0 c3
:3 0 a1 :3 0 55
:3 0 9a :3 0 a1
:3 0 9a :3 0 102
:3 0 55 :3 0 103
:3 0 f2 :3 0 e4
:3 0 13c :3 0 e4
:3 0 13d :3 0 13f
:3 0 d8 :3 0 104
:3 0 140 :3 0 ec
:3 0 107 :3 0 108
:3 0 109 :3 0 10a
:3 0 10b :3 0 10c
:3 0 1ce :3 0 a2
:3 0 55 :3 0 c5
:3 0 c6 :3 0 a3
:3 0 10e :3 0 fb
:3 0 55 :3 0 fa
:3 0 fb :3 0 59
:3 0 c5 :3 0 c6
:3 0 a3 :3 0 10e
:3 0 fe :3 0 39
:3 0 55 :3 0 fd
:3 0 55 :3 0 fa
:3 0 fe :3 0 59
:3 0 55 :3 0 103
:3 0 55 :3 0 59
:3 0 d :3 0 54
:3 0 b7 :4 0 1cf
1 :a 0 1c50 :2 0
1c74 1bbb 1c4e :2 0
2c :3 0 2d :3 0
1c51 1c52 0 2c
:3 0 3d :3 0 1c54
1c55 0 14 :2 0
2 :4 0 1d0 :2 0
3f :2 0 14 :2 0
931 1c5a 1c5c :3 0
934 1c56 1c5e 1c53
1c60 1c5f :2 0 2f
:3 0 40 :3 0 1c62
1c63 0 31 :4 0
1d1 :4 0 34 :2 0
35 :3 0 938 1c67
1c69 :3 0 93b 1c64
1c6b :2 0 1c6d 93e
1c6e 1c61 1c6d 0
1c6f 940 0 1c74
8 :3 0 27 :3 0
1c71 :2 0 1c74 74
:3 0 942 1c75 1bb9
1c74 0 1fad 39
:3 0 15 :3 0 16
:2 0 946 1c76 1c79
4f :2 0 1d2 :2 0
94b 1c7b 1c7d :3 0
96 :3 0 97 :3 0
1d3 :4 0 1c81 :4 0
1c82 :2 0 1eb4 9
:3 0 5b :3 0 1c84
1c85 0 16 :2 0
1c86 1c87 0 1eb4
1d4 :3 0 9a :3 0
a5 :3 0 1d5 :3 0
9a :3 0 a5 :3 0
1d5 :3 0 76 :3 0
53 :3 0 d :3 0
d8 :3 0 1d6 :3 0
47 :3 0 1d7 :3 0
9a :3 0 a5 :4 0
1d8 1 :8 0 1caf
a7 :4 0 1c9a :3 0
4e :3 0 4f :2 0
950 1c9d 1c9e :3 0
9 :3 0 5b :3 0
1ca0 1ca1 0 16
:2 0 1ca2 1ca3 0
1ca5 953 1cac 9
:3 0 5b :3 0 1ca6
1ca7 0 a8 :2 0
1ca8 1ca9 0 1cab
955 1cad 1c9f 1ca5
0 1cae 0 1cab
0 1cae 957 0
1caf 95a 1d02 5d
:3 0 9 :3 0 5b
:3 0 1cb1 1cb2 0
5c :3 0 1cb3 1cb4
0 1cb6 95d 1cb8
95f 1cb7 1cb6 :2 0
1d00 5e :3 0 9
:3 0 5b :3 0 1cbb
1cbc 0 5c :3 0
1cbd 1cbe 0 1cf8
2c :3 0 2d :3 0
1cc0 1cc1 0 2c
:3 0 3d :3 0 1cc3
1cc4 0 5f :2 0
2 :4 0 1d9 :2 0
3f :2 0 14 :2 0
961 1cc9 1ccb :3 0
964 1cc5 1ccd 1cc2
1ccf 1cce :2 0 2f
:3 0 40 :3 0 1cd1
1cd2 0 31 :4 0
1da :4 0 34 :2 0
35 :3 0 968 1cd6
1cd8 :3 0 34 :2 0
45 :4 0 96b 1cda
1cdc :3 0 34 :2 0
62 :4 0 96e 1cde
1ce0 :3 0 34 :2 0
5c :3 0 971 1ce2
1ce4 :3 0 34 :2 0
63 :4 0 974 1ce6
1ce8 :3 0 34 :2 0
64 :3 0 65 :3 0
14 :2 0 66 :2 0
977 1ceb 1cef 97b
1cea 1cf1 :3 0 97e
1cd3 1cf3 :2 0 1cf5
981 1cf6 1cd0 1cf5
0 1cf7 983 0
1cf8 985 1cfb :3 0
1cfb 0 1cfb 1cfa
1cf8 1cf9 :6 0 1cfd
2e :3 0 988 1cff
98a 1cfe 1cfd :2 0
1d00 98c :2 0 1d02
0 1d02 1d01 1caf
1d00 :6 0 1eb4 1
:3 0 96 :3 0 97
:3 0 1db :4 0 1d06
:4 0 1d07 :2 0 1eb4
9 :3 0 5b :3 0
1d09 1d0a 0 16
:2 0 1d0b 1d0c 0
1eb4 1dc :3 0 9a
:3 0 a5 :3 0 1d5
:3 0 9a :3 0 a5
:3 0 1d5 :3 0 76
:3 0 53 :3 0 d
:3 0 da :3 0 d
:3 0 9a :3 0 a5
:4 0 1dd 1 :8 0
1d32 a7 :4 0 1d1d
:3 0 4e :3 0 4f
:2 0 991 1d20 1d21
:3 0 9 :3 0 5b
:3 0 1d23 1d24 0
16 :2 0 1d25 1d26
0 1d28 994 1d2f
9 :3 0 5b :3 0
1d29 1d2a 0 a8
:2 0 1d2b 1d2c 0
1d2e 996 1d30 1d22
1d28 0 1d31 0
1d2e 0 1d31 998
0 1d32 99b 1d85
5d :3 0 9 :3 0
5b :3 0 1d34 1d35
0 5c :3 0 1d36
1d37 0 1d39 99e
1d3b 9a0 1d3a 1d39
:2 0 1d83 5e :3 0
9 :3 0 5b :3 0
1d3e 1d3f 0 5c
:3 0 1d40 1d41 0
1d7b 2c :3 0 2d
:3 0 1d43 1d44 0
2c :3 0 3d :3 0
1d46 1d47 0 5f
:2 0 2 :4 0 1de
:2 0 3f :2 0 14
:2 0 9a2 1d4c 1d4e
:3 0 9a5 1d48 1d50
1d45 1d52 1d51 :2 0
2f :3 0 40 :3 0
1d54 1d55 0 31
:4 0 1df :4 0 34
:2 0 35 :3 0 9a9
1d59 1d5b :3 0 34
:2 0 45 :4 0 9ac
1d5d 1d5f :3 0 34
:2 0 62 :4 0 9af
1d61 1d63 :3 0 34
:2 0 5c :3 0 9b2
1d65 1d67 :3 0 34
:2 0 63 :4 0 9b5
1d69 1d6b :3 0 34
:2 0 64 :3 0 65
:3 0 14 :2 0 66
:2 0 9b8 1d6e 1d72
9bc 1d6d 1d74 :3 0
9bf 1d56 1d76 :2 0
1d78 9c2 1d79 1d53
1d78 0 1d7a 9c4
0 1d7b 9c6 1d7e
:3 0 1d7e 0 1d7e
1d7d 1d7b 1d7c :6 0
1d80 30 :3 0 9c9
1d82 9cb 1d81 1d80
:2 0 1d83 9cd :2 0
1d85 0 1d85 1d84
1d32 1d83 :6 0 1eb4
1 :3 0 96 :3 0
97 :3 0 1e0 :4 0
1d89 :4 0 1d8a :2 0
1eb4 9 :3 0 5b
:3 0 1d8c 1d8d 0
16 :2 0 1d8e 1d8f
0 1eb4 1e1 :3 0
9a :3 0 a5 :3 0
1d5 :3 0 55 :3 0
9a :3 0 55 :3 0
a5 :3 0 55 :3 0
1d5 :3 0 1d4 :3 0
55 :3 0 c5 :3 0
c6 :3 0 a3 :3 0
1dc :3 0 52 :3 0
55 :3 0 9a :3 0
52 :3 0 9a :3 0
52 :3 0 9a :3 0
52 :3 0 a5 :3 0
52 :3 0 1d5 :3 0
1dc :3 0 52 :3 0
c5 :3 0 c6 :3 0
a3 :3 0 1d4 :3 0
55 :3 0 55 :3 0
9a :3 0 52 :3 0
9a :4 0 1e2 1
:8 0 1dcd a7 :4 0
1db8 :3 0 4e :3 0
4f :2 0 9d2 1dbb
1dbc :3 0 9 :3 0
5b :3 0 1dbe 1dbf
0 16 :2 0 1dc0
1dc1 0 1dc3 9d5
1dca 9 :3 0 5b
:3 0 1dc4 1dc5 0
a8 :2 0 1dc6 1dc7
0 1dc9 9d7 1dcb
1dbd 1dc3 0 1dcc
0 1dc9 0 1dcc
9d9 0 1dcd 9dc
1e20 5d :3 0 9
:3 0 5b :3 0 1dcf
1dd0 0 5c :3 0
1dd1 1dd2 0 1dd4
9df 1dd6 9e1 1dd5
1dd4 :2 0 1e1e 5e
:3 0 9 :3 0 5b
:3 0 1dd9 1dda 0
5c :3 0 1ddb 1ddc
0 1e16 2c :3 0
2d :3 0 1dde 1ddf
0 2c :3 0 3d
:3 0 1de1 1de2 0
5f :2 0 2 :4 0
1e3 :2 0 3f :2 0
14 :2 0 9e3 1de7
1de9 :3 0 9e6 1de3
1deb 1de0 1ded 1dec
:2 0 2f :3 0 40
:3 0 1def 1df0 0
31 :4 0 1e4 :4 0
34 :2 0 35 :3 0
9ea 1df4 1df6 :3 0
34 :2 0 45 :4 0
9ed 1df8 1dfa :3 0
34 :2 0 62 :4 0
9f0 1dfc 1dfe :3 0
34 :2 0 5c :3 0
9f3 1e00 1e02 :3 0
34 :2 0 63 :4 0
9f6 1e04 1e06 :3 0
34 :2 0 64 :3 0
65 :3 0 14 :2 0
66 :2 0 9f9 1e09
1e0d 9fd 1e08 1e0f
:3 0 a00 1df1 1e11
:2 0 1e13 a03 1e14
1dee 1e13 0 1e15
a05 0 1e16 a07
1e19 :3 0 1e19 0
1e19 1e18 1e16 1e17
:6 0 1e1b 32 :3 0
a0a 1e1d a0c 1e1c
1e1b :2 0 1e1e a0e
:2 0 1e20 0 1e20
1e1f 1dcd 1e1e :6 0
1eb4 1 :3 0 b1
:3 0 28 :3 0 55
:3 0 9a :3 0 59
:3 0 b5 :3 0 b6
:3 0 b7 :3 0 54
:3 0 b5 :3 0 1e5
:3 0 55 :3 0 1d5
:3 0 1e1 :3 0 55
:3 0 a3 :3 0 c3
:3 0 b5 :3 0 55
:3 0 a5 :3 0 55
:3 0 9a :3 0 b5
:3 0 9a :3 0 c5
:3 0 c6 :3 0 a3
:3 0 1dc :3 0 1e6
:3 0 55 :3 0 9a
:3 0 1e6 :3 0 9a
:3 0 a1 :3 0 ad
:3 0 59 :3 0 a1
:3 0 c9 :3 0 b7
:3 0 54 :3 0 1e5
:3 0 55 :3 0 1d5
:3 0 1e1 :3 0 55
:3 0 a3 :3 0 ca
:3 0 a1 :3 0 55
:3 0 a5 :3 0 a1
:3 0 ad :3 0 55
:3 0 9a :3 0 a3
:3 0 a4 :3 0 c4
:3 0 c4 :3 0 59
:3 0 a1 :3 0 cb
:3 0 a3 :3 0 a4
:3 0 ba :3 0 ba
:3 0 59 :3 0 c4
:3 0 9b :3 0 c5
:3 0 c6 :3 0 a3
:3 0 1dc :3 0 1e6
:3 0 55 :3 0 9a
:3 0 1e6 :3 0 9a
:3 0 d7 :3 0 9a
:3 0 59 :3 0 b5
:3 0 b6 :3 0 b7
:3 0 54 :3 0 b5
:3 0 1e5 :3 0 1d5
:3 0 a2 :3 0 d7
:3 0 c3 :3 0 b5
:3 0 d7 :3 0 53
:3 0 d :3 0 d7
:3 0 9a :3 0 b5
:3 0 9a :3 0 1e7
:3 0 9a :3 0 1e1
:3 0 1e7 :3 0 d7
:3 0 9a :3 0 1e7
:3 0 9a :3 0 54
:3 0 b7 :4 0 1e8
1 :a 0 1e90 :2 0
1eb4 1e23 1e8e :2 0
2c :3 0 2d :3 0
1e91 1e92 0 2c
:3 0 3d :3 0 1e94
1e95 0 14 :2 0
2 :4 0 1e9 :2 0
3f :2 0 14 :2 0
a11 1e9a 1e9c :3 0
a14 1e96 1e9e 1e93
1ea0 1e9f :2 0 2f
:3 0 40 :3 0 1ea2
1ea3 0 31 :4 0
1ea :4 0 34 :2 0
35 :3 0 a18 1ea7
1ea9 :3 0 a1b 1ea4
1eab :2 0 1ead a1e
1eae 1ea1 1ead 0
1eaf a20 0 1eb4
8 :3 0 28 :3 0
1eb1 :2 0 1eb4 74
:3 0 a22 1eb5 1c7e
1eb4 0 1fad 39
:3 0 15 :3 0 16
:2 0 a2f 1eb6 1eb9
4f :2 0 81 :2 0
a34 1ebb 1ebd :3 0
b1 :3 0 29 :3 0
d9 :3 0 59 :3 0
b5 :3 0 b6 :3 0
b7 :3 0 54 :3 0
b5 :3 0 1e5 :3 0
1d5 :3 0 6c :3 0
d9 :3 0 a3 :3 0
c3 :3 0 b5 :3 0
d9 :3 0 59 :3 0
b5 :3 0 9a :3 0
d9 :3 0 54 :3 0
d9 :3 0 82 :3 0
b :3 0 b7 :4 0
1eb 1 :a 0 1edb
:2 0 1eff 1ec0 1ed9
:2 0 2c :3 0 2d
:3 0 1edc 1edd 0
2c :3 0 3d :3 0
1edf 1ee0 0 14
:2 0 2 :4 0 1ec
:2 0 3f :2 0 14
:2 0 a37 1ee5 1ee7
:3 0 a3a 1ee1 1ee9
1ede 1eeb 1eea :2 0
2f :3 0 40 :3 0
1eed 1eee 0 31
:4 0 1ed :4 0 34
:2 0 35 :3 0 a3e
1ef2 1ef4 :3 0 a41
1eef 1ef6 :2 0 1ef8
a44 1ef9 1eec 1ef8
0 1efa a46 0
1eff 8 :3 0 29
:3 0 1efc :2 0 1eff
74 :3 0 a48 1f00
1ebe 1eff 0 1fad
39 :3 0 15 :3 0
16 :2 0 a4c 1f01
1f04 47 :3 0 4f
:2 0 8e :3 0 1f06
1f08 0 a51 1f07
1f0a :3 0 1f0b :2 0
39 :3 0 15 :3 0
16 :2 0 a54 1f0d
1f10 47 :3 0 4f
:2 0 8f :3 0 1f12
1f14 0 a59 1f13
1f16 :3 0 1f17 :2 0
1f0c 1f19 1f18 :2 0
b1 :3 0 2a :3 0
a1 :3 0 9a :3 0
59 :3 0 a1 :3 0
b6 :3 0 b7 :3 0
47 :3 0 16d :3 0
54 :3 0 a1 :3 0
1e5 :3 0 1d5 :3 0
198 :3 0 d7 :3 0
c3 :3 0 a1 :3 0
d7 :3 0 53 :3 0
d :3 0 d7 :3 0
9a :3 0 a1 :3 0
9a :3 0 b7 :4 0
1ee 1 :a 0 1f37
:2 0 1f5a 1f1c 1f35
:2 0 2c :3 0 2d
:3 0 1f38 1f39 0
2c :3 0 3d :3 0
1f3b 1f3c 0 14
:2 0 2 :4 0 1ef
:2 0 3f :2 0 14
:2 0 a5c 1f41 1f43
:3 0 a5f 1f3d 1f45
1f3a 1f47 1f46 :2 0
2f :3 0 40 :3 0
1f49 1f4a 0 31
:4 0 1f0 :4 0 34
:2 0 35 :3 0 a63
1f4e 1f50 :3 0 a66
1f4b 1f52 :2 0 1f54
a69 1f55 1f48 1f54
0 1f56 a6b 0
1f5a 8 :3 0 2a
:3 0 1f58 :2 0 1f5a
a6d 1f5b 1f1a 1f5a
0 1fad 2c :3 0
2d :3 0 1f5c 1f5d
0 2c :3 0 3d
:3 0 1f5f 1f60 0
172 :2 0 2 :4 0
1f1 :2 0 3f :2 0
14 :2 0 a71 1f65
1f67 :3 0 a74 1f61
1f69 1f5e 1f6b 1f6a
:2 0 2f :3 0 40
:3 0 1f6d 1f6e 0
31 :4 0 1f2 :4 0
34 :2 0 35 :3 0
a78 1f72 1f74 :3 0
34 :2 0 45 :4 0
a7b 1f76 1f78 :3 0
34 :2 0 46 :4 0
a7e 1f7a 1f7c :3 0
34 :2 0 39 :3 0
3a :3 0 47 :3 0
48 :3 0 1f81 1f82
0 a81 1f80 1f84
3b :4 0 a83 1f7f
1f87 a86 1f7e 1f89
:3 0 34 :2 0 49
:4 0 a89 1f8b 1f8d
:3 0 34 :2 0 37
:4 0 a8c 1f8f 1f91
:3 0 34 :2 0 3
:3 0 a8f 1f93 1f95
:3 0 34 :2 0 37
:4 0 a92 1f97 1f99
:3 0 a95 1f6f 1f9b
:2 0 1f9d a98 1f9e
1f6c 1f9d 0 1f9f
a9a 0 1faa 4c
:3 0 1f3 :3 0 1fa0
1fa1 0 47 :3 0
48 :3 0 1fa3 1fa4
0 3 :3 0 4e
:3 0 a9c 1fa2 1fa8
:2 0 1faa aa0 1fac
6fd 964 0 1fad
0 1faa 0 1fad
aa3 0 1fae ab3
204f 9 :3 0 1f4
:2 0 4 1faf 1fb0
0 2c :3 0 2d
:3 0 1fb2 1fb3 0
2c :3 0 3d :3 0
1fb5 1fb6 0 5f
:2 0 2 :4 0 1f5
:2 0 3f :2 0 14
:2 0 abc 1fbb 1fbd
:3 0 abf 1fb7 1fbf
1fb4 1fc1 1fc0 :2 0
2f :3 0 40 :3 0
1fc3 1fc4 0 31
:4 0 1f6 :4 0 34
:2 0 35 :3 0 ac3
1fc8 1fca :3 0 ac6
1fc5 1fcc :2 0 1fe5
2f :3 0 40 :3 0
1fce 1fcf 0 31
:4 0 1f7 :4 0 34
:2 0 9 :3 0 1f8
:3 0 1fd4 1fd5 0
ac9 1fd3 1fd7 :3 0
34 :2 0 49 :4 0
acc 1fd9 1fdb :3 0
34 :2 0 9 :3 0
1f9 :3 0 1fde 1fdf
0 acf 1fdd 1fe1
:3 0 ad2 1fd0 1fe3
:2 0 1fe5 ad5 1fe6
1fc2 1fe5 0 1fe7
ad8 0 1ff7 b1
:3 0 2b :3 0 9
:3 0 1f8 :3 0 1fa
:3 0 9 :3 0 1f9
:3 0 3 :3 0 118
:4 0 1fb 1 :a 0
1ff3 :2 0 1ff7 1fe9
1ff1 :2 0 8 :3 0
2b :3 0 1ff5 :2 0
1ff7 ada 1ff9 ade
1ff8 1ff7 :2 0 204d
5e :3 0 9 :3 0
5b :3 0 1ffc 1ffd
0 5c :3 0 1ffe
1fff 0 2045 2c
:3 0 2d :3 0 2001
2002 0 2c :3 0
3d :3 0 2004 2005
0 5f :2 0 2
:4 0 1fc :2 0 3f
:2 0 14 :2 0 ae0
200a 200c :3 0 ae3
2006 200e 2003 2010
200f :2 0 2f :3 0
40 :3 0 2012 2013
0 31 :4 0 1fd
:4 0 34 :2 0 35
:3 0 ae7 2017 2019
:3 0 aea 2014 201b
:2 0 2035 2f :3 0
40 :3 0 201d 201e
0 31 :4 0 62
:4 0 34 :2 0 5c
:3 0 aed 2022 2024
:3 0 34 :2 0 63
:4 0 af0 2026 2028
:3 0 34 :2 0 64
:3 0 65 :3 0 14
:2 0 66 :2 0 af3
202b 202f af7 202a
2031 :3 0 afa 201f
2033 :2 0 2035 afd
2036 2011 2035 0
2037 b00 0 2045
b1 :3 0 2b :3 0
1fa :3 0 9 :3 0
1f9 :3 0 3 :3 0
118 :4 0 1fe 1
:a 0 2041 :2 0 2045
2039 203f :2 0 8
:3 0 2b :3 0 2043
:2 0 2045 b02 2048
:3 0 2048 0 2048
2047 2045 2046 :6 0
204a 1 :3 0 b07
204c b09 204b 204a
:2 0 204d b0b :2 0
204f b0e 204f 204e
1fae 204d :6 0 2050
:2 0 2 14 204f
2053 :3 0 2052 2050
2054 :8 0 
b2b
4
:3 0 1 3 1
7 1 b 3
6 a e 1
18 1 16 1
1f 1 1d 1
26 1 24 1
2d 1 2b 1
34 1 32 1
3b 1 39 1
40 1 46 1
4c 1 52 1
58 1 5e 1
64 1 6b 1
72 1 79 1
80 1 87 1
8e 1 95 1
9c 1 a3 1
aa 1 b1 1
b8 1 bf 1
c6 1 cd 1
da 1 e6 2
e9 eb 2 ee
f0 2 f2 f4
2 f6 f8 2
fa fc 1 102
2 104 105 2
fe 107 2 109
10b 2 10d 10f
2 111 113 2
115 117 2 119
11b 4 e2 e8
ed 11d 1 11f
1 121 2 12b
12d 3 129 12a
12f 2 138 13a
2 137 13c 1
13e 1 140 1
142 2 14f 151
3 14d 14e 153
2 15c 15e 2
160 162 2 164
166 1 16e 2
170 171 2 168
173 2 175 177
2 179 17b 2
17d 17f 2 181
183 2 15b 185
1 187 1 189
2 193 195 3
191 192 197 2
1a0 1a2 2 19f
1a4 1 1a6 1
1a8 3 1af 1b0
1b1 3 18a 1a9
1b3 1 1b5 2
1c1 1c2 1 1c6
2 1c4 1c6 2
1cb 1cc 1 1d0
2 1ce 1d0 2
1ee 1f3 1 1fa
1 1f5 2 20d
20f 3 20b 20c
211 2 21a 21c
2 21e 220 2
222 224 2 226
228 2 22a 22c
3 231 232 233
2 22e 235 2
219 237 1 239
1 23b 2 204
23c 1 240 1
1ff 2 1fd 244
1 24f 2 24b
24f 2 268 26d
1 274 1 26f
2 287 289 3
285 286 28b 2
294 296 2 298
29a 2 29c 29e
2 2a0 2a2 2
2a4 2a6 3 2ab
2ac 2ad 2 2a8
2af 2 293 2b1
1 2b3 1 2b5
2 27e 2b6 1
2ba 1 279 2
277 2be 1 2c9
2 2c5 2c9 2
2d1 2d6 1 2dd
1 2d8 2 2f0
2f2 3 2ee 2ef
2f4 2 2fd 2ff
2 301 303 2
305 307 2 309
30b 2 30d 30f
3 314 315 316
2 311 318 2
2fc 31a 1 31c
1 31e 2 2e7
31f 1 323 1
2e2 2 2e0 327
1 332 2 32e
332 2 33e 340
3 33c 33d 342
2 34b 34d 2
34f 351 2 353
355 1 35d 2
35f 360 2 357
362 2 364 366
2 368 36a 2
36c 36e 2 370
372 2 34a 374
1 376 1 378
2 382 384 3
380 381 386 2
38f 391 2 38e
393 1 395 1
397 3 39e 39f
3a0 3 379 398
3a2 1 3a4 2
32a 3a5 1 3a7
2 2c1 3a8 1
3aa 2 247 3ab
2 3af 3b0 1
3b4 2 3b2 3b4
2 3bc 3c1 1
3c8 1 3c3 2
3db 3dd 3 3d9
3da 3df 2 3e8
3ea 2 3ec 3ee
2 3f0 3f2 2
3f4 3f6 2 3f8
3fa 3 3ff 400
401 2 3fc 403
2 3e7 405 1
407 1 409 2
3d2 40a 1 40e
1 3cd 2 3cb
412 1 41d 2
419 41d 2 429
42b 3 427 428
42d 2 436 438
2 43a 43c 2
43e 440 1 448
2 44a 44b 2
442 44d 2 44f
451 2 453 455
2 457 459 2
45b 45d 2 435
45f 1 461 1
463 2 46d 46f
3 46b 46c 471
2 47a 47c 2
479 47e 1 480
1 482 3 489
48a 48b 3 464
483 48d 1 48f
2 499 49e 1
4a5 1 4a0 2
4b8 4ba 3 4b6
4b7 4bc 2 4c5
4c7 2 4c9 4cb
2 4cd 4cf 2
4d1 4d3 2 4d5
4d7 3 4dc 4dd
4de 2 4d9 4e0
2 4c4 4e2 1
4e4 1 4e6 2
4af 4e7 1 4eb
1 4aa 2 4a8
4ef 3 415 490
4f2 2 4f8 4f9
1 4fd 2 4fb
4fd 2 505 50a
1 511 1 50c
2 524 526 3
522 523 528 2
531 533 2 535
537 2 539 53b
2 53d 53f 2
541 543 3 548
549 54a 2 545
54c 2 530 54e
1 550 1 552
2 51b 553 1
557 1 516 2
514 55b 1 566
2 562 566 2
572 574 3 570
571 576 2 57f
581 2 583 585
2 587 589 1
591 2 593 594
2 58b 596 2
598 59a 2 59c
59e 2 5a0 5a2
2 5a4 5a6 2
57e 5a8 1 5aa
1 5ac 2 5b6
5b8 3 5b4 5b5
5ba 2 5c3 5c5
2 5c2 5c7 1
5c9 1 5cb 3
5d2 5d3 5d4 3
5ad 5cc 5d6 1
5d8 3 55e 5d9
5dc 2 5e1 5e2
1 5e8 2 5e4
5e8 2 5ed 5ee
1 5f2 2 5f0
5f2 2 5f9 5fa
1 600 2 5fc
600 2 607 608
1 60e 2 60a
60e 2 61c 621
1 628 1 623
2 63b 63d 3
639 63a 63f 2
648 64a 2 64c
64e 2 650 652
2 654 656 2
658 65a 3 65f
660 661 2 65c
663 2 647 665
1 667 1 669
2 632 66a 1
66e 1 62d 2
62b 672 1 67d
2 679 67d 2
689 68b 3 687
688 68d 2 696
698 2 69a 69c
2 69e 6a0 1
6a8 2 6aa 6ab
2 6a2 6ad 2
6af 6b1 2 6b3
6b5 2 6b7 6b9
2 6bb 6bd 2
695 6bf 1 6c1
1 6c3 2 6cd
6cf 3 6cb 6cc
6d1 2 6da 6dc
2 6d9 6de 1
6e0 1 6e2 3
6e9 6ea 6eb 3
6c4 6e3 6ed 1
6ef 2 675 6f0
4 6f3 4f6 5df
6f2 2 6f6 6f7
1 6fb 2 6f9
6fb 1 72d 2
72c 72d 1 735
1 73b 2 73d
73e 2 72a 73f
1 746 1 741
2 759 75b 3
757 758 75d 2
766 768 2 76a
76c 2 76e 770
2 772 774 2
776 778 3 77d
77e 77f 2 77a
781 2 765 783
1 785 1 787
2 750 788 1
78c 1 74b 2
749 790 1 7a9
2 7a8 7a9 1
7b1 1 7b7 2
7b9 7ba 2 7a6
7bb 1 7c2 1
7bd 2 7d5 7d7
3 7d3 7d4 7d9
2 7e2 7e4 2
7e6 7e8 2 7ea
7ec 2 7ee 7f0
2 7f2 7f4 3
7f9 7fa 7fb 2
7f6 7fd 2 7e1
7ff 1 801 1
803 2 7cc 804
1 808 1 7c7
2 7c5 80c 2
949 94b 3 947
948 94d 2 956
958 2 955 95a
1 95c 1 95e
9 702 707 793
799 79e 80f 93f
95f 962 2 966
967 1 96b 2
969 96b 2 970
971 1 975 2
973 975 1 9ae
2 9ad 9ae 1
9b6 1 9bc 2
9be 9bf 2 9ab
9c0 1 9c7 1
9c2 2 9da 9dc
3 9d8 9d9 9de
2 9e7 9e9 2
9eb 9ed 2 9ef
9f1 2 9f3 9f5
2 9f7 9f9 3
9fe 9ff a00 2
9fb a02 2 9e6
a04 1 a06 1
a08 2 9d1 a09
1 a0d 1 9cc
2 9ca a11 1
a4e 2 a4d a4e
1 a56 1 a5c
2 a5e a5f 2
a4b a60 1 a67
1 a62 2 a7a
a7c 3 a78 a79
a7e 2 a87 a89
2 a8b a8d 2
a8f a91 2 a93
a95 2 a97 a99
3 a9e a9f aa0
2 a9b aa2 2
a86 aa4 1 aa6
1 aa8 2 a71
aa9 1 aad 1
a6c 2 a6a ab1
2 b0a b0c 3
b08 b09 b0e 2
b17 b19 2 b16
b1b 1 b1d 1
b1f 8 97f 984
a14 a1c ab4 b00
b20 b23 2 b28
b29 1 b2d 2
b2b b2d 2 bc8
bca 3 bc6 bc7
bcc 2 bd5 bd7
2 bd4 bd9 1
bdb 1 bdd 3
bbe bde be1 2
be6 be7 1 beb
2 be9 beb 2
bf0 bf1 1 bf3
1 bf7 2 bf5
bf7 2 c04 c09
1 c10 1 c0b
2 c23 c25 3
c21 c22 c27 2
c30 c32 2 c34
c36 2 c38 c3a
2 c3c c3e 2
c40 c42 3 c47
c48 c49 2 c44
c4b 2 c2f c4d
1 c4f 1 c51
2 c1a c52 1
c56 1 c15 2
c13 c5a 2 c60
c61 1 c65 2
c63 c65 1 c6a
2 c78 c7d 1
c84 1 c7f 2
c97 c99 3 c95
c96 c9b 2 ca4
ca6 2 ca8 caa
2 cac cae 2
cb0 cb2 2 cb4
cb6 3 cbb cbc
cbd 2 cb8 cbf
2 ca3 cc1 1
cc3 1 cc5 2
c8e cc6 1 cca
1 c89 2 c87
cce 2 cd4 cd5
1 cd9 2 cd7
cd9 2 cf8 cfd
1 d04 1 cff
2 d17 d19 3
d15 d16 d1b 2
d24 d26 2 d28
d2a 2 d2c d2e
2 d30 d32 2
d34 d36 3 d3b
d3c d3d 2 d38
d3f 2 d23 d41
1 d43 1 d45
2 d0e d46 1
d4a 1 d09 2
d07 d4e 2 d54
d55 1 d59 2
d57 d59 1 d5e
1 d62 2 d64
d65 2 d51 d66
1 d6a 2 d6c
d6d 3 c6e cd1
d6e 2 d70 d71
2 f88 f8a 3
f86 f87 f8c 2
f95 f97 2 f94
f99 1 f9b 1
f9d 6 bfc c5d
d72 f7e f9e fa1
2 10f5 10f7 3
10f3 10f4 10f9 2
1102 1104 2 1101
1106 1 1108 1
110a 3 10eb 110b
110e 2 1110 1111
1 1112 2 1117
1118 1 111c 2
111a 111c 1 1153
2 1152 1153 1
115b 1 1161 2
1163 1164 2 1150
1165 1 116c 1
1167 2 117f 1181
3 117d 117e 1183
2 118c 118e 2
1190 1192 2 1194
1196 2 1198 119a
2 119c 119e 3
11a3 11a4 11a5 2
11a0 11a7 2 118b
11a9 1 11ab 1
11ad 2 1176 11ae
1 11b2 1 1171
2 116f 11b6 1
11fa 2 11f9 11fa
1 1202 1 1208
2 120a 120b 2
11f7 120c 1 1213
1 120e 2 1226
1228 3 1224 1225
122a 2 1233 1235
2 1237 1239 2
123b 123d 2 123f
1241 2 1243 1245
3 124a 124b 124c
2 1247 124e 2
1232 1250 1 1252
1 1254 2 121d
1255 1 1259 1
1218 2 1216 125d
2 12b9 12bb 3
12b7 12b8 12bd 2
12c6 12c8 2 12c5
12ca 1 12cc 1
12ce 8 1123 1128
11b9 11c1 1260 12af
12cf 12d2 2 12d7
12d8 1 12dc 2
12da 12dc 1 1313
2 1312 1313 1
131b 1 1321 2
1323 1324 2 1310
1325 1 132c 1
1327 2 133f 1341
3 133d 133e 1343
2 134c 134e 2
1350 1352 2 1354
1356 2 1358 135a
2 135c 135e 3
1363 1364 1365 2
1360 1367 2 134b
1369 1 136b 1
136d 2 1336 136e
1 1372 1 1331
2 132f 1376 1
13e7 2 13e6 13e7
1 13ef 1 13f5
2 13f7 13f8 2
13e4 13f9 1 1400
1 13fb 2 1413
1415 3 1411 1412
1417 2 1420 1422
2 1424 1426 2
1428 142a 2 142c
142e 2 1430 1432
3 1437 1438 1439
2 1434 143b 2
141f 143d 1 143f
1 1441 2 140a
1442 1 1446 1
1405 2 1403 144a
2 14c4 14c6 3
14c2 14c3 14c8 2
14d1 14d3 2 14d0
14d5 1 14d7 1
14d9 8 12e3 12e8
1379 1381 144d 14ba
14da 14dd 2 14e2
14e3 1 14e7 2
14e5 14e7 2 15d9
15db 3 15d7 15d8
15dd 2 15e6 15e8
2 15e5 15ea 1
15ec 1 15ee 3
15cf 15ef 15f2 2
15f7 15f8 1 15fc
2 15fa 15fc 1
163b 2 163a 163b
1 1643 1 1649
2 164b 164c 2
1638 164d 1 1654
1 164f 2 1667
1669 3 1665 1666
166b 2 1674 1676
2 1678 167a 2
167c 167e 2 1680
1682 2 1684 1686
3 168b 168c 168d
2 1688 168f 2
1673 1691 1 1693
1 1695 2 165e
1696 1 169a 1
1659 2 1657 169e
1 1721 2 1720
1721 1 1729 1
172f 2 1731 1732
2 171e 1733 1
173a 1 1735 2
174d 174f 3 174b
174c 1751 2 175a
175c 2 175e 1760
2 1762 1764 2
1766 1768 2 176a
176c 3 1771 1772
1773 2 176e 1775
2 1759 1777 1
1779 1 177b 2
1744 177c 1 1780
1 173f 2 173d
1784 1 179c 2
179b 179c 1 17a4
1 17aa 2 17ac
17ad 2 1799 17ae
1 17b5 1 17b0
2 17c8 17ca 3
17c6 17c7 17cc 2
17d5 17d7 2 17d9
17db 2 17dd 17df
2 17e1 17e3 2
17e5 17e7 3 17ec
17ed 17ee 2 17e9
17f0 2 17d4 17f2
1 17f4 1 17f6
2 17bf 17f7 1
17fb 1 17ba 2
17b8 17ff 2 18e4
18e6 3 18e2 18e3
18e8 2 18f1 18f3
2 18f0 18f5 1
18f7 1 18f9 a
1603 1608 16a1 16a7
16ac 1787 1802 18da
18fa 18fd 2 1902
1903 1 1909 2
1905 1909 2 1911
1916 1 191d 1
1918 2 1930 1932
3 192e 192f 1934
2 193d 193f 2
1941 1943 2 1945
1947 2 1949 194b
2 194d 194f 3
1954 1955 1956 2
1951 1958 2 193c
195a 1 195c 1
195e 2 1927 195f
1 1963 1 1922
2 1920 1967 1
1972 2 196e 1972
2 1a1b 1a1d 3
1a19 1a1a 1a1f 2
1a28 1a2a 2 1a27
1a2c 1 1a2e 1
1a30 3 1a11 1a31
1a34 1 1a36 2
196a 1a37 2 1a3c
1a3d 1 1a41 2
1a3f 1a41 2 1b94
1b96 3 1b92 1b93
1b98 2 1ba1 1ba3
2 1ba0 1ba5 1
1ba7 1 1ba9 3
1b8a 1baa 1bad 2
1bb2 1bb3 1 1bb7
2 1bb5 1bb7 2
1c59 1c5b 3 1c57
1c58 1c5d 2 1c66
1c68 2 1c65 1c6a
1 1c6c 1 1c6e
3 1c4f 1c6f 1c72
2 1c77 1c78 1
1c7c 2 1c7a 1c7c
1 1c9c 2 1c9b
1c9c 1 1ca4 1
1caa 2 1cac 1cad
2 1c99 1cae 1
1cb5 1 1cb0 2
1cc8 1cca 3 1cc6
1cc7 1ccc 2 1cd5
1cd7 2 1cd9 1cdb
2 1cdd 1cdf 2
1ce1 1ce3 2 1ce5
1ce7 3 1cec 1ced
1cee 2 1ce9 1cf0
2 1cd4 1cf2 1
1cf4 1 1cf6 2
1cbf 1cf7 1 1cfb
1 1cba 2 1cb8
1cff 1 1d1f 2
1d1e 1d1f 1 1d27
1 1d2d 2 1d2f
1d30 2 1d1c 1d31
1 1d38 1 1d33
2 1d4b 1d4d 3
1d49 1d4a 1d4f 2
1d58 1d5a 2 1d5c
1d5e 2 1d60 1d62
2 1d64 1d66 2
1d68 1d6a 3 1d6f
1d70 1d71 2 1d6c
1d73 2 1d57 1d75
1 1d77 1 1d79
2 1d42 1d7a 1
1d7e 1 1d3d 2
1d3b 1d82 1 1dba
2 1db9 1dba 1
1dc2 1 1dc8 2
1dca 1dcb 2 1db7
1dcc 1 1dd3 1
1dce 2 1de6 1de8
3 1de4 1de5 1dea
2 1df3 1df5 2
1df7 1df9 2 1dfb
1dfd 2 1dff 1e01
2 1e03 1e05 3
1e0a 1e0b 1e0c 2
1e07 1e0e 2 1df2
1e10 1 1e12 1
1e14 2 1ddd 1e15
1 1e19 1 1dd8
2 1dd6 1e1d 2
1e99 1e9b 3 1e97
1e98 1e9d 2 1ea6
1ea8 2 1ea5 1eaa
1 1eac 1 1eae
c 1c83 1c88 1d02
1d08 1d0d 1d85 1d8b
1d90 1e20 1e8f 1eaf
1eb2 2 1eb7 1eb8
1 1ebc 2 1eba
1ebc 2 1ee4 1ee6
3 1ee2 1ee3 1ee8
2 1ef1 1ef3 2
1ef0 1ef5 1 1ef7
1 1ef9 3 1eda
1efa 1efd 2 1f02
1f03 1 1f09 2
1f05 1f09 2 1f0e
1f0f 1 1f15 2
1f11 1f15 2 1f40
1f42 3 1f3e 1f3f
1f44 2 1f4d 1f4f
2 1f4c 1f51 1
1f53 1 1f55 3
1f36 1f56 1f59 2
1f64 1f66 3 1f62
1f63 1f68 2 1f71
1f73 2 1f75 1f77
2 1f79 1f7b 1
1f83 2 1f85 1f86
2 1f7d 1f88 2
1f8a 1f8c 2 1f8e
1f90 2 1f92 1f94
2 1f96 1f98 2
1f70 1f9a 1 1f9c
1 1f9e 3 1fa5
1fa6 1fa7 2 1f9f
1fa9 f 1fab b26
be4 1115 12d5 14e0
15f5 1900 1a3a 1bb0
1c75 1eb5 1f00 1f5b
1fac 8 122 141
1b6 1b9 1bc 1bf
6f4 1fad 2 1fba
1fbc 3 1fb8 1fb9
1fbe 2 1fc7 1fc9
2 1fc6 1fcb 2
1fd2 1fd6 2 1fd8
1fda 2 1fdc 1fe0
2 1fd1 1fe2 2
1fcd 1fe4 1 1fe6
3 1fe7 1ff2 1ff6
1 1fb1 2 2009
200b 3 2007 2008
200d 2 2016 2018
2 2015 201a 2
2021 2023 2 2025
2027 3 202c 202d
202e 2 2029 2030
2 2020 2032 2
201c 2034 1 2036
4 2000 2037 2040
2044 1 2048 1
1ffb 2 1ff9 204c
1c 1b 22 29
30 37 3e 44
4a 50 56 5c
62 69 70 77
7e 85 8c 93
9a a1 a8 af
b6 bd c4 cb
d2 
1
4
0 
2053
0
1
50
34
20
0 1 2 1 4 1 6 1
8 1 a 1 c 1 e 1
10 1 12 1 14 1 16 1
18 1 1a 1 1c 1 1e 1
20 1 22 1 24 1 26 1
28 1 2a 1 2c 1 2e 1
30 1 32 1 4552 4f53 5255 4543
495f 2944 5420 4548 204e 3331 4520 534c
2045 3831 2035 4e45 2044 5341 5220 5345
545f 5059 2c45 200a 2020 2020 5220 5345

3 1 0
c6 1 0
b 1 0
bf 1 0
52 1 0
2 0 1
64 1 0
6b 1 0
72 1 0
58 1 0
80 1 0
4c 1 0
87 1 0
8e 1 0
95 1 0
9c 1 0
a3 1 0
aa 1 0
b1 1 0
b8 1 0
79 1 0
40 1 0
1d 1 0
16 1 0
5e 1 0
39 1 0
32 1 0
2b 1 0
46 1 0
7 1 0
24 1 0
cd 1 0
0

/
SHOW ERRORS FUNCTION SP_S_PROFILE;
ALTER FUNCTION SP_S_PROFILE COMPILE DEBUG;
/*********************************************************************************************************************************************/
Prompt == FUNCTION FC_U_NAMES ==
CREATE OR REPLACE FUNCTION FC_U_NAMES wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
6f
2 :e:
1FUNCTION:
1FC_U_NAMES:
1A_ELEMENT_ID:
1OUT:
1NOCOPY:
1VARCHAR2:
1H_FULL_NAME:
1A_PARTICIPANT_ID:
1SH_H_TABLE_TYPE:
1PLS_INTEGER:
1RETURN:
1H_TABLE_TYPE:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= FC_U_NAMES(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
11:
119:
1+:
1FPUT:
1               ENTER:: FC_U_NAMES(20) :: cplan.sql(14316) :: :
1PMO_CNST:
1SC_TABLE_SCD:
1TMT_ASSIGNED_SCORECARDS:
1ASSIGN_NAMES:
1INSTR:
1REPLACE:
1LENGTH:
1ELEMENT_ID:
1UPDATE TMT_ASSIGNED_SCORECARDS SET ASSIGN_NAMES =:n            CASE:n        +
1    WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0:n            THEN h_ful+
1l_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', ''):n            +
1ELSE:n            CASE:n            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_+
1NAMES) > 2040 THEN ASSIGN_NAMES:n            ELSE h_full_name || ';' || ASSIG+
1N_NAMES:n            END:n            END:n            WHERE ELEMENT_ID = a_e+
1lement_id:
1FOUND:
1TRUE:
1=:
1PMO_DB2M:
1SC_STATE:
10:
1100:
1NO_DATA_FOUND:
1SQLCODE:
1OTHERS:
116:
148:
1           EXCEPTION:: FC_U_NAMES(49) :: cplan.sql(14345) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SC_TABLE_POL:
1TMT_POOLS:
1MANAGER_ID:
1UPDATE TMT_POOLS SET MANAGER_ID = a_participant_id, ASSIGN_NAMES =:n         +
1   CASE:n            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0:n     +
1       THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', +
1''):n            ELSE:n            CASE:n            WHEN LENGTH(h_full_name)+
1 + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES:n            ELSE h_full_nam+
1e || ';' || ASSIGN_NAMES:n            END:n            END:n            WHERE+
1 ELEMENT_ID = a_element_id:
180:
1           EXCEPTION:: FC_U_NAMES(81) :: cplan.sql(14377) :: :
1SC_TABLE_CRI:
1TMT_CRI:
1UPDATE TMT_CRI SET ASSIGN_NAMES =:n            CASE:n            WHEN InStr( +
1ASSIGN_NAMES, h_full_name || ';') > 0:n            THEN h_full_name || ';' ||+
1 REPLACE( ASSIGN_NAMES, h_full_name || ';', ''):n            ELSE:n          +
1  CASE:n            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 TH+
1EN ASSIGN_NAMES:n            ELSE h_full_name || ';' || ASSIGN_NAMES:n       +
1     END:n            END:n            WHERE ELEMENT_ID = a_element_id:
1111:
1           EXCEPTION:: FC_U_NAMES(112) :: cplan.sql(14408) :: :
1SC_TABLE_DOC:
1TMT_DOCUMENTS:
1UPDATE TMT_DOCUMENTS SET ASSIGN_NAMES =:n            CASE:n            WHEN I+
1nStr( ASSIGN_NAMES, h_full_name || ';') > 0:n            THEN h_full_name || +
1';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', ''):n            ELSE:n    +
1        CASE:n            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2+
1040 THEN ASSIGN_NAMES:n            ELSE h_full_name || ';' || ASSIGN_NAMES:n +
1           END:n            END:n            WHERE ELEMENT_ID = a_element_id:
1143:
1           EXCEPTION:: FC_U_NAMES(144) :: cplan.sql(14440) :: :
1SC_TABLE_POL_DOC:
1TMT_POOL_DOCUMENTS:
1UPDATE TMT_POOL_DOCUMENTS SET ASSIGN_NAMES =:n            CASE:n            W+
1HEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0:n            THEN h_full_nam+
1e || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', ''):n            ELSE+
1:n            CASE:n            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAME+
1S) > 2040 THEN ASSIGN_NAMES:n            ELSE h_full_name || ';' || ASSIGN_NA+
1MES:n            END:n            END:n            WHERE ELEMENT_ID = a_eleme+
1nt_id:
1175:
1           EXCEPTION:: FC_U_NAMES(176) :: cplan.sql(14472) :: :
1SC_TABLE_RES_DOC:
1TMT_RESOURCE_DOCUMENTS:
1UPDATE TMT_RESOURCE_DOCUMENTS SET ASSIGN_NAMES =:n            CASE:n         +
1   WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0:n            THEN h_full+
1_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', ''):n            E+
1LSE:n            CASE:n            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_N+
1AMES) > 2040 THEN ASSIGN_NAMES:n            ELSE h_full_name || ';' || ASSIGN+
1_NAMES:n            END:n            END:n            WHERE ELEMENT_ID = a_el+
1ement_id:
1207:
1           EXCEPTION:: FC_U_NAMES(208) :: cplan.sql(14504) :: :
1SC_TABLE_AST_DOC:
1TMT_ASSET_DOCUMENTS:
1UPDATE TMT_ASSET_DOCUMENTS SET ASSIGN_NAMES =:n            CASE:n            +
1WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0:n            THEN h_full_na+
1me || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', ''):n            ELSE+
1:n            CASE:n            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAME+
1S) > 2040 THEN ASSIGN_NAMES:n            ELSE h_full_name || ';' || ASSIGN_NA+
1MES:n            END:n            END:n            WHERE ELEMENT_ID = a_eleme+
1nt_id:
1239:
1           EXCEPTION:: FC_U_NAMES(240) :: cplan.sql(14536) :: :
1SC_TABLE_CLN_DOC:
1TMT_CLIENT_DOCUMENTS:
1UPDATE TMT_CLIENT_DOCUMENTS SET ASSIGN_NAMES =:n            CASE:n           +
1 WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0:n            THEN h_full_n+
1ame || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', ''):n            ELS+
1E:n            CASE:n            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAM+
1ES) > 2040 THEN ASSIGN_NAMES:n            ELSE h_full_name || ';' || ASSIGN_N+
1AMES:n            END:n            END:n            WHERE ELEMENT_ID = a_elem+
1ent_id:
1271:
1           EXCEPTION:: FC_U_NAMES(272) :: cplan.sql(14568) :: :
1SC_TABLE_VDR_DOC:
1TMT_VENDOR_DOCUMENTS:
1UPDATE TMT_VENDOR_DOCUMENTS SET ASSIGN_NAMES =:n            CASE:n           +
1 WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0:n            THEN h_full_n+
1ame || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', ''):n            ELS+
1E:n            CASE:n            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAM+
1ES) > 2040 THEN ASSIGN_NAMES:n            ELSE h_full_name || ';' || ASSIGN_N+
1AMES:n            END:n            END:n            WHERE ELEMENT_ID = a_elem+
1ent_id:
1303:
1           EXCEPTION:: FC_U_NAMES(304) :: cplan.sql(14600) :: :
1TMT_WBS:
1UPDATE TMT_WBS SET ASSIGN_NAMES =:n            CASE:n            WHEN InStr( +
1ASSIGN_NAMES, h_full_name || ';') > 0:n            THEN h_full_name || ';' ||+
1 REPLACE( ASSIGN_NAMES, h_full_name || ';', ''):n            ELSE:n          +
1  CASE:n            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 TH+
1EN ASSIGN_NAMES:n            ELSE h_full_name || ';' || ASSIGN_NAMES:n       +
1     END:n            END:n            WHERE ELEMENT_ID = a_element_id:
1335:
1           EXCEPTION:: FC_U_NAMES(336) :: cplan.sql(14632) :: :
1343:
1                EXIT:: FC_U_NAMES(344) :: cplan.sql(14640) :: :
1STATE:: :
0

0
0
5fc
2
0 a0 8d 90 :3 a0 b0 3f 90
:3 a0 b0 3f 90 :3 a0 b0 3f 8f
a0 b0 3d b4 :2 a0 2c 6a a3
a0 1c a0 81 b0 :2 a0 6b :2 a0
6b 6e a5 b a 10 :2 a0 6b
6e :2 a0 6b 6e a5 b 6e 7e
a0 b4 2e 6e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :3 a0 6b :12 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 a6 9 :2 a0 6b
:14 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 a6 9 :2 a0 6b :12 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 a6 9
:2 a0 6b :12 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 a6 9 :2 a0 6b :12 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
a6 9 :2 a0 6b :12 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 a6 9 :2 a0
6b :12 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 a6 9 :2 a0 6b :12 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 a6
9 :2 a0 6b :12 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 a6 9 :12 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 9
a4 14 :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :2 a0 6b b4 2e a5 57 b7
19 3c :3 a0 6b 65 b7 a4 b1
11 68 4f 1d 17 b5 
5fc
2
0 3 7 2b 1f 23 27 1e
32 47 3b 3f 43 1b 4e 5f
53 57 5b 3a 66 73 6f 37
7b 6e 80 84 88 8c a9 94
98 a0 a4 6b 90 b0 b4 b7
bb bf c2 c7 c8 1 ca cf
d3 d7 da df e3 e7 ea ef
f0 f2 f7 fa fe ff 104 109
10c 111 112 117 11a 11e 11f 124
127 12c 12d 132 135 13a 13b 140
143 148 149 14e 151 155 156 15b
15e 163 164 169 16c 171 172 177
17a 17f 180 185 188 18c 18d 192
195 19a 19b 1a0 1a3 1a8 1a9 1ae
1b1 1b5 1b9 1bd 1be 1c0 1c5 1c6
1c8 1c9 1ce 1d1 1d6 1d7 1dc 1dd
1e2 1e4 1e8 1eb 1ef 1f3 1f6 1fa
1fe 201 204 209 20c 20f 212 213
218 219 1 21b 220 224 228 22b
230 235 238 23c 23d 242 243 248
24a 24e 251 255 259 25d 260 264
268 26c 270 274 278 27c 280 284
288 28c 290 294 298 29c 2a0 2a4
2a8 2b4 2b8 2bd 2c1 2c4 2c5 2ca
2ce 2d2 2d5 2d8 2dc 2de 2e2 2e6
2e9 2ec 2f0 2f2 2f6 2fa 2fd 2ff
303 307 30b 30e 312 316 318 319
31e 1 322 326 32a 32d 331 335
339 33d 340 344 348 34b 34e 353
356 359 35c 35d 362 363 1 365
36a 36e 372 375 37a 37f 382 386
387 38c 38f 394 395 39a 39d 3a2
3a3 3a8 3ab 3af 3b0 3b5 3b8 3bd
3be 3c3 3c6 3ca 3ce 3d1 3d4 3d5
3d7 3d8 3dd 3de 3e3 3e5 3e9 3ec
3ee 3f2 3f4 400 402 404 405 40a
40e 410 41c 41e 420 421 426 42a
42e 431 435 439 43d 441 445 449
44d 451 455 459 45d 461 465 469
46d 471 475 479 47d 481 48d 491
496 49a 49d 49e 4a3 4a7 4ab 4ae
4b1 4b5 4b7 4bb 4bf 4c2 4c5 4c9
4cb 4cf 4d3 4d6 4d8 4dc 4e0 4e4
4e7 4eb 4ef 4f1 4f2 4f7 1 4fb
4ff 503 506 50a 50e 512 516 519
51d 521 524 527 52c 52f 532 535
536 53b 53c 1 53e 543 547 54b
54e 553 558 55b 55f 560 565 568
56d 56e 573 576 57b 57c 581 584
588 589 58e 591 596 597 59c 59f
5a3 5a7 5aa 5ad 5ae 5b0 5b1 5b6
5b7 5bc 5be 5c2 5c5 5c7 5cb 5cd
5d9 5db 5dd 5de 5e3 5e7 5e9 5f5
5f7 5f9 5fa 5ff 603 607 60a 60e
612 616 61a 61e 622 626 62a 62e
632 636 63a 63e 642 646 64a 64e
652 65e 662 667 66b 66e 66f 674
678 67c 67f 682 686 688 68c 690
693 696 69a 69c 6a0 6a4 6a7 6a9
6ad 6b1 6b5 6b8 6bc 6c0 6c2 6c3
6c8 1 6cc 6d0 6d4 6d7 6db 6df
6e3 6e7 6ea 6ee 6f2 6f5 6f8 6fd
700 703 706 707 70c 70d 1 70f
714 718 71c 71f 724 729 72c 730
731 736 739 73e 73f 744 747 74c
74d 752 755 759 75a 75f 762 767
768 76d 770 774 778 77b 77e 77f
781 782 787 788 78d 78f 793 796
798 79c 79e 7aa 7ac 7ae 7af 7b4
7b8 7ba 7c6 7c8 7ca 7cb 7d0 7d4
7d8 7db 7df 7e3 7e7 7eb 7ef 7f3
7f7 7fb 7ff 803 807 80b 80f 813
817 81b 81f 823 82f 833 838 83c
83f 840 845 849 84d 850 853 857
859 85d 861 864 867 86b 86d 871
875 878 87a 87e 882 886 889 88d
891 893 894 899 1 89d 8a1 8a5
8a8 8ac 8b0 8b4 8b8 8bb 8bf 8c3
8c6 8c9 8ce 8d1 8d4 8d7 8d8 8dd
8de 1 8e0 8e5 8e9 8ed 8f0 8f5
8fa 8fd 901 902 907 90a 90f 910
915 918 91d 91e 923 926 92a 92b
930 933 938 939 93e 941 945 949
94c 94f 950 952 953 958 959 95e
960 964 967 969 96d 96f 97b 97d
97f 980 985 989 98b 997 999 99b
99c 9a1 9a5 9a9 9ac 9b0 9b4 9b8
9bc 9c0 9c4 9c8 9cc 9d0 9d4 9d8
9dc 9e0 9e4 9e8 9ec 9f0 9f4 a00
a04 a09 a0d a10 a11 a16 a1a a1e
a21 a24 a28 a2a a2e a32 a35 a38
a3c a3e a42 a46 a49 a4b a4f a53
a57 a5a a5e a62 a64 a65 a6a 1
a6e a72 a76 a79 a7d a81 a85 a89
a8c a90 a94 a97 a9a a9f aa2 aa5
aa8 aa9 aae aaf 1 ab1 ab6 aba
abe ac1 ac6 acb ace ad2 ad3 ad8
adb ae0 ae1 ae6 ae9 aee aef af4
af7 afb afc b01 b04 b09 b0a b0f
b12 b16 b1a b1d b20 b21 b23 b24
b29 b2a b2f b31 b35 b38 b3a b3e
b40 b4c b4e b50 b51 b56 b5a b5c
b68 b6a b6c b6d b72 b76 b7a b7d
b81 b85 b89 b8d b91 b95 b99 b9d
ba1 ba5 ba9 bad bb1 bb5 bb9 bbd
bc1 bc5 bd1 bd5 bda bde be1 be2
be7 beb bef bf2 bf5 bf9 bfb bff
c03 c06 c09 c0d c0f c13 c17 c1a
c1c c20 c24 c28 c2b c2f c33 c35
c36 c3b 1 c3f c43 c47 c4a c4e
c52 c56 c5a c5d c61 c65 c68 c6b
c70 c73 c76 c79 c7a c7f c80 1
c82 c87 c8b c8f c92 c97 c9c c9f
ca3 ca4 ca9 cac cb1 cb2 cb7 cba
cbf cc0 cc5 cc8 ccc ccd cd2 cd5
cda cdb ce0 ce3 ce7 ceb cee cf1
cf2 cf4 cf5 cfa cfb d00 d02 d06
d09 d0b d0f d11 d1d d1f d21 d22
d27 d2b d2d d39 d3b d3d d3e d43
d47 d4b d4e d52 d56 d5a d5e d62
d66 d6a d6e d72 d76 d7a d7e d82
d86 d8a d8e d92 d96 da2 da6 dab
daf db2 db3 db8 dbc dc0 dc3 dc6
dca dcc dd0 dd4 dd7 dda dde de0
de4 de8 deb ded df1 df5 df9 dfc
e00 e04 e06 e07 e0c 1 e10 e14
e18 e1b e1f e23 e27 e2b e2e e32
e36 e39 e3c e41 e44 e47 e4a e4b
e50 e51 1 e53 e58 e5c e60 e63
e68 e6d e70 e74 e75 e7a e7d e82
e83 e88 e8b e90 e91 e96 e99 e9d
e9e ea3 ea6 eab eac eb1 eb4 eb8
ebc ebf ec2 ec3 ec5 ec6 ecb ecc
ed1 ed3 ed7 eda edc ee0 ee2 eee
ef0 ef2 ef3 ef8 efc efe f0a f0c
f0e f0f f14 f18 f1c f1f f23 f27
f2b f2f f33 f37 f3b f3f f43 f47
f4b f4f f53 f57 f5b f5f f63 f67
f73 f77 f7c f80 f83 f84 f89 f8d
f91 f94 f97 f9b f9d fa1 fa5 fa8
fab faf fb1 fb5 fb9 fbc fbe fc2
fc6 fca fcd fd1 fd5 fd7 fd8 fdd
1 fe1 fe5 fe9 fec ff0 ff4 ff8
ffc fff 1003 1007 100a 100d 1012 1015
1018 101b 101c 1021 1022 1 1024 1029
102d 1031 1034 1039 103e 1041 1045 1046
104b 104e 1053 1054 1059 105c 1061 1062
1067 106a 106e 106f 1074 1077 107c 107d
1082 1085 1089 108d 1090 1093 1094 1096
1097 109c 109d 10a2 10a4 10a8 10ab 10ad
10b1 10b3 10bf 10c1 10c3 10c4 10c9 10cd
10cf 10db 10dd 10df 10e0 10e5 10e9 10ed
10f0 10f4 10f8 10fc 1100 1104 1108 110c
1110 1114 1118 111c 1120 1124 1128 112c
1130 1134 1138 1144 1148 114d 1151 1154
1155 115a 115e 1162 1165 1168 116c 116e
1172 1176 1179 117c 1180 1182 1186 118a
118d 118f 1193 1197 119b 119e 11a2 11a6
11a8 11a9 11ae 1 11b2 11b6 11ba 11bd
11c1 11c5 11c9 11cd 11d0 11d4 11d8 11db
11de 11e3 11e6 11e9 11ec 11ed 11f2 11f3
1 11f5 11fa 11fe 1202 1205 120a 120f
1212 1216 1217 121c 121f 1224 1225 122a
122d 1232 1233 1238 123b 123f 1240 1245
1248 124d 124e 1253 1256 125a 125e 1261
1264 1265 1267 1268 126d 126e 1273 1275
1279 127c 127e 1282 1284 1290 1292 1294
1295 129a 129e 12a0 12ac 12ae 12b0 12b1
12b6 12ba 12be 12c2 12c6 12ca 12ce 12d2
12d6 12da 12de 12e2 12e6 12ea 12ee 12f2
12f6 12fa 12fe 130a 130e 1313 1317 131a
131b 1320 1324 1328 132b 132e 1332 1334
1338 133c 133f 1342 1346 1348 134c 1350
1353 1355 1359 135d 1361 1364 1368 136c
136e 136f 1374 1 1378 137c 1380 1383
1387 138b 138f 1393 1396 139a 139e 13a1
13a4 13a9 13ac 13af 13b2 13b3 13b8 13b9
1 13bb 13c0 13c4 13c8 13cb 13d0 13d5
13d8 13dc 13dd 13e2 13e5 13ea 13eb 13f0
13f3 13f8 13f9 13fe 1401 1405 1406 140b
140e 1413 1414 1419 141c 1420 1424 1427
142a 142b 142d 142e 1433 1434 1439 143b
143f 1442 1444 1448 144a 1456 1458 145a
145b 1460 1464 1466 1472 1474 1476 147b
147f 1484 1488 148c 148f 1493 1497 149a
149d 14a2 14a5 14a8 14ab 14ac 14b1 14b2
1 14b4 14b9 14bd 14c1 14c4 14c9 14ce
14d1 14d5 14d6 14db 14de 14e3 14e4 14e9
14ec 14f1 14f2 14f7 14fa 14fe 1502 1505
1506 150b 150c 1511 1513 1517 151a 151e
1522 1526 1529 152d 152f 1533 1535 1541
1545 1547 1548 1551 
5fc
2
0 1 a 6 16 1a 21 :3 6
15 19 20 :3 6 1a 1e 25 :3 6
16 :2 6 1 3 a :2 1 4 :2 11
20 11 4 7 :2 11 23 :2 2d 3a
:2 23 :3 7 :2 d 19 25 :2 2f 43 :2 25
52 58 5b :2 52 69 7d 8 :2 69
d 10 :2 69 1d 20 :2 69 b e
:2 69 13 16 :2 69 1b 1e :2 69 2a
2d :2 69 b e :2 69 13 16 :2 69
1b 1e :2 69 2f 32 :2 69 b e
:2 69 13 16 1f 27 :2 1f 39 :2 16
:2 69 41 44 :2 69 :2 7 48 :2 4 7
:2 11 23 :2 2d 39 3c 4a 4c 4d
:2 4a :2 23 :3 7 :2 d 12 1e 5b 5e
:2 1e :2 7 50 :2 4 9 c :2 15 14
30 12 19 27 12 28 31 3f
12 19 28 2f 49 12 28 13
20 d 14 10 1c :3 1a 10 :2 19
25 10 21 10 :2 19 25 10 :4 d
a 12 10 :2 19 25 10 20 :2 d
:2 12 10 :2 19 25 10 13 :2 1d 2f
:2 39 45 49 57 59 5a :2 57 :2 2f
:3 13 :2 19 1e 2a 67 6a :2 2a 77
7a :2 2a 7e 81 :2 2a 8d 90 :2 2a
98 9b :2 2a a7 aa b1 ba bd
:2 aa :2 2a :2 13 5d :2 10 :2 d :4 19 :2 d
a :4 22 :2 7 c :2 15 14 22 2f
41 12 19 27 12 28 31 3f
12 19 28 2f 49 12 28 13
20 d 14 10 1c :3 1a 10 :2 19
25 10 21 10 :2 19 25 10 :4 d
a 12 10 :2 19 25 10 20 :2 d
:2 12 10 :2 19 25 10 13 :2 1d 2f
:2 39 45 49 57 59 5a :2 57 :2 2f
:3 13 :2 19 1e 2a 67 6a :2 2a 77
7a :2 2a 7e 81 :2 2a 8d 90 :2 2a
98 9b :2 2a a7 aa b1 ba bd
:2 aa :2 2a :2 13 5d :2 10 :2 d :4 19 :2 d
a :4 22 :2 7 c :2 15 14 20 12
19 27 12 28 31 3f 12 19
28 2f 49 12 28 13 20 d
14 10 1c :3 1a 10 :2 19 25 10
21 10 :2 19 25 10 :4 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 57 5a 5b :2 57 :2 2f :3 13 :2 19
1e 2a 68 6b :2 2a 78 7b :2 2a
7f 82 :2 2a 8e 91 :2 2a 99 9c
:2 2a a8 ab b2 bb be :2 ab :2 2a
:2 13 5e :2 10 :2 d :4 19 :2 d a :4 22
:2 7 c :2 15 14 26 12 19 27
12 28 31 3f 12 19 28 2f
49 12 28 13 20 d 14 10
1c :3 1a 10 :2 19 25 10 21 10
:2 19 25 10 :4 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 57
5a 5b :2 57 :2 2f :3 13 :2 19 1e 2a
68 6b :2 2a 78 7b :2 2a 7f 82
:2 2a 8e 91 :2 2a 99 9c :2 2a a8
ab b2 bb be :2 ab :2 2a :2 13 5e
:2 10 :2 d :4 19 :2 d a :4 22 :2 7 c
:2 15 14 2b 12 19 27 12 28
31 3f 12 19 28 2f 49 12
28 13 20 d 14 10 1c :3 1a
10 :2 19 25 10 21 10 :2 19 25
10 :4 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 57 5a 5b
:2 57 :2 2f :3 13 :2 19 1e 2a 68 6b
:2 2a 78 7b :2 2a 7f 82 :2 2a 8e
91 :2 2a 99 9c :2 2a a8 ab b2
bb be :2 ab :2 2a :2 13 5e :2 10 :2 d
:4 19 :2 d a :4 26 :2 7 c :2 15 14
2f 12 19 27 12 28 31 3f
12 19 28 2f 49 12 28 13
20 d 14 10 1c :3 1a 10 :2 19
25 10 21 10 :2 19 25 10 :4 d
a 12 10 :2 19 25 10 20 :2 d
:2 12 10 :2 19 25 10 13 :2 1d 2f
:2 39 45 49 57 5a 5b :2 57 :2 2f
:3 13 :2 19 1e 2a 68 6b :2 2a 78
7b :2 2a 7f 82 :2 2a 8e 91 :2 2a
99 9c :2 2a a8 ab b2 bb be
:2 ab :2 2a :2 13 5e :2 10 :2 d :4 19 :2 d
a :4 26 :2 7 c :2 15 14 2c 12
19 27 12 28 31 3f 12 19
28 2f 49 12 28 13 20 d
14 10 1c :3 1a 10 :2 19 25 10
21 10 :2 19 25 10 :4 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 57 5a 5b :2 57 :2 2f :3 13 :2 19
1e 2a 68 6b :2 2a 78 7b :2 2a
7f 82 :2 2a 8e 91 :2 2a 99 9c
:2 2a a8 ab b2 bb be :2 ab :2 2a
:2 13 5e :2 10 :2 d :4 19 :2 d a :4 26
:2 7 c :2 15 14 2d 12 19 27
12 28 31 3f 12 19 28 2f
49 12 28 13 20 d 14 10
1c :3 1a 10 :2 19 25 10 21 10
:2 19 25 10 :4 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 57
5a 5b :2 57 :2 2f :3 13 :2 19 1e 2a
68 6b :2 2a 78 7b :2 2a 7f 82
:2 2a 8e 91 :2 2a 99 9c :2 2a a8
ab b2 bb be :2 ab :2 2a :2 13 5e
:2 10 :2 d :4 19 :2 d a :4 26 :2 7 c
:2 15 14 2d 12 19 27 12 28
31 3f 12 19 28 2f 49 12
28 13 20 d 14 10 1c :3 1a
10 :2 19 25 10 21 10 :2 19 25
10 :4 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 57 5a 5b
:2 57 :2 2f :3 13 :2 19 1e 2a 68 6b
:2 2a 78 7b :2 2a 7f 82 :2 2a 8e
91 :2 2a 99 9c :2 2a a8 ab b2
bb be :2 ab :2 2a :2 13 5e :2 10 :2 d
:4 19 :2 d a :4 26 :2 7 14 20 12
19 27 12 28 31 3f 12 19
28 2f 49 12 28 13 20 d
14 10 1c :3 1a 10 :2 19 25 10
21 10 :2 19 25 10 :4 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 57 5a 5b :2 57 :2 2f :3 13 :2 19
1e 2a 68 6b :2 2a 78 7b :2 2a
7f 82 :2 2a 8e 91 :2 2a 99 9c
:2 2a a8 ab b2 bb be :2 ab :2 2a
:2 13 5e :2 10 :2 d :4 19 :2 d a :4 7
:3 4 7 :2 11 23 :2 2d 39 3c 4a
4d 4e :2 4a :2 23 :3 7 :2 d 12 1e
5c 5f :2 1e 6c 6f :2 1e 73 76
:2 1e 80 83 :2 8c :2 1e :2 7 51 :3 4
b :2 14 4 :9 1 
5fc
4
0 :2 1 :6 3 :6 4
:6 5 :4 6 2 :2 7
:2 1 :6 9 :b b :11 c
d :2 c :2 d :2 c
:2 d :2 c :2 e :2 c
:2 e :2 c :2 e :2 c
:2 e :2 c :2 f :2 c
:2 f :2 c :2 f :2 c
:2 f :2 c :2 10 :2 c
:9 10 :2 c :2 10 :4 c
:3 b :11 13 :b 14 :3 13
17 :3 18 :2 1a :3 1c
:4 1d :5 20 :2 21 :2 24
1a :6 25 :5 26 25
:5 28 27 :3 25 19
2b :5 2c :3 2b :2 2d
:5 2f :11 30 :24 31 :3 30
:2 2e :6 2d 2a :6 18
:3 38 :4 3a :3 3c :4 3d
:5 40 :2 41 :2 44 3a
:6 45 :5 46 45 :5 48
47 :3 45 39 4b
:5 4c :3 4b :2 4d :5 4f
:11 50 :24 51 :3 50 :2 4e
:6 4d 4a :6 38 :3 57
:2 59 :3 5b :4 5c :5 5f
:2 60 :2 63 59 :6 64
:5 65 64 :5 67 66
:3 64 58 6a :5 6b
:3 6a :2 6c :5 6e :11 6f
:24 70 :3 6f :2 6d :6 6c
69 :6 57 :3 77 :2 79
:3 7b :4 7c :5 7f :2 80
:2 83 79 :6 84 :5 85
84 :5 87 86 :3 84
78 8a :5 8b :3 8a
:2 8c :5 8e :11 8f :24 90
:3 8f :2 8d :6 8c 89
:6 77 :3 97 :2 99 :3 9b
:4 9c :5 9f :2 a0 :2 a3
99 :6 a4 :5 a5 a4
:5 a7 a6 :3 a4 98
aa :5 ab :3 aa :2 ac
:5 ae :11 af :24 b0 :3 af
:2 ad :6 ac a9 :6 97
:3 b7 :2 b9 :3 bb :4 bc
:5 bf :2 c0 :2 c3 b9
:6 c4 :5 c5 c4 :5 c7
c6 :3 c4 b8 ca
:5 cb :3 ca :2 cc :5 ce
:11 cf :24 d0 :3 cf :2 cd
:6 cc c9 :6 b7 :3 d7
:2 d9 :3 db :4 dc :5 df
:2 e0 :2 e3 d9 :6 e4
:5 e5 e4 :5 e7 e6
:3 e4 d8 ea :5 eb
:3 ea :2 ec :5 ee :11 ef
:24 f0 :3 ef :2 ed :6 ec
e9 :6 d7 :3 f7 :2 f9
:3 fb :4 fc :5 ff :2 100
:2 103 f9 :6 104 :5 105
104 :5 107 106 :3 104
f8 10a :5 10b :3 10a
:2 10c :5 10e :11 10f :24 110
:3 10f :2 10d :6 10c 109
:6 f7 :3 117 :2 119 :3 11b
:4 11c :5 11f :2 120 :2 123
119 :6 124 :5 125 124
:5 127 126 :3 124 118
12a :5 12b :3 12a :2 12c
:5 12e :11 12f :24 130 :3 12f
:2 12d :6 12c 129 :6 117
:2 139 :3 13b :4 13c :5 13f
:2 140 :2 143 139 :6 144
:5 145 144 :5 147 146
:3 144 138 14a :5 14b
:3 14a :2 14c :5 14e :11 14f
:24 150 :3 14f :2 14d :6 14c
149 :4 137 :3 17 :11 157
:19 158 :3 157 :5 15a :2 a
:7 1 
1553
4
:3 0 1 :3 0 2
:a 0 5f7 1 :7 0
5 37 0 3
4 :3 0 5 :3 0
6 :3 0 3 :5 0
1 7 6 :3 0
9 6b 0 7
4 :3 0 5 :3 0
6 :3 0 7 :5 0
1 d c :3 0
4 :3 0 5 :3 0
6 :3 0 8 :5 0
1 13 12 :3 0
10 90 0 b
a :3 0 9 :7 0
17 16 :3 0 b
:3 0 a :3 0 19
1b 0 5f7 2
1c :2 0 d :3 0
a :3 0 1f :7 0
9 :3 0 23 20
21 5f5 0 c
:6 0 e :3 0 24
25 0 d :3 0
f :3 0 27 28
0 2 :4 0 12
29 2b 26 2d
2c :2 0 10 :3 0
11 :3 0 2f 30
0 12 :4 0 d
:3 0 13 :3 0 33
34 0 2 :4 0
14 35 37 14
:4 0 15 :2 0 16
:3 0 16 3a 3c
:3 0 17 :4 0 15
:2 0 18 :4 0 19
3f 41 :3 0 15
:2 0 3 :3 0 1c
43 45 :3 0 15
:2 0 18 :4 0 1f
47 49 :3 0 15
:2 0 19 :4 0 22
4b 4d :3 0 15
:2 0 18 :4 0 25
4f 51 :3 0 15
:2 0 7 :3 0 28
53 55 :3 0 15
:2 0 18 :4 0 2b
57 59 :3 0 15
:2 0 19 :4 0 2e
5b 5d :3 0 15
:2 0 18 :4 0 31
5f 61 :3 0 15
:2 0 8 :3 0 34
63 65 :3 0 15
:2 0 18 :4 0 37
67 69 :3 0 15
:2 0 19 :4 0 3a
6b 6d :3 0 15
:2 0 1a :3 0 1b
:3 0 9 :3 0 3d
71 73 1c :4 0
3f 70 76 42
6f 78 :3 0 15
:2 0 1d :4 0 45
7a 7c :3 0 48
31 7e :2 0 80
4d 81 2e 80
0 82 4f 0
5f3 d :3 0 e
:3 0 83 84 0
d :3 0 1e :3 0
86 87 0 1f
:2 0 2 :4 0 20
:2 0 21 :2 0 1f
:2 0 51 8c 8e
:3 0 54 88 90
85 92 91 :2 0
10 :3 0 22 :3 0
94 95 0 12
:4 0 23 :4 0 15
:2 0 16 :3 0 58
99 9b :3 0 5b
96 9d :2 0 9f
5e a0 93 9f
0 a1 60 0
5f3 c :3 0 24
:3 0 25 :3 0 a3
a4 0 26 :3 0
27 :3 0 28 :3 0
27 :3 0 7 :3 0
7 :3 0 29 :3 0
27 :3 0 7 :3 0
2a :3 0 7 :3 0
2a :3 0 27 :3 0
27 :3 0 7 :3 0
27 :3 0 2b :3 0
3 :4 0 2c 1
:8 0 ce 2d :4 0
b9 :3 0 2e :3 0
2f :2 0 64 bc
bd :3 0 30 :3 0
31 :3 0 bf c0
0 32 :2 0 c1
c2 0 c4 67
cb 30 :3 0 31
:3 0 c5 c6 0
33 :2 0 c7 c8
0 ca 69 cc
be c4 0 cd
0 ca 0 cd
6b 0 ce 6e
121 34 :3 0 30
:3 0 31 :3 0 d0
d1 0 35 :3 0
d2 d3 0 d5
71 d7 73 d6
d5 :2 0 11f 36
:3 0 30 :3 0 31
:3 0 da db 0
35 :3 0 dc dd
0 117 d :3 0
e :3 0 df e0
0 d :3 0 1e
:3 0 e2 e3 0
37 :2 0 2 :4 0
38 :2 0 21 :2 0
1f :2 0 75 e8
ea :3 0 78 e4
ec e1 ee ed
:2 0 10 :3 0 22
:3 0 f0 f1 0
12 :4 0 39 :4 0
15 :2 0 16 :3 0
7c f5 f7 :3 0
15 :2 0 3a :4 0
7f f9 fb :3 0
15 :2 0 3b :4 0
82 fd ff :3 0
15 :2 0 35 :3 0
85 101 103 :3 0
15 :2 0 3c :4 0
88 105 107 :3 0
15 :2 0 3d :3 0
3e :3 0 1f :2 0
3f :2 0 8b 10a
10e 8f 109 110
:3 0 92 f2 112
:2 0 114 95 115
ef 114 0 116
97 0 117 99
11a :3 0 11a 0
11a 119 117 118
:6 0 11c 2 :3 0
9c 11e 9e 11d
11c :2 0 11f a0
:2 0 121 0 121
120 ce 11f :6 0
123 1 :3 0 a3
125 a5 124 123
:2 0 5bf 24 :3 0
40 :3 0 126 127
0 41 :3 0 42
:3 0 8 :3 0 27
:3 0 28 :3 0 27
:3 0 7 :3 0 7
:3 0 29 :3 0 27
:3 0 7 :3 0 2a
:3 0 7 :3 0 2a
:3 0 27 :3 0 27
:3 0 7 :3 0 27
:3 0 2b :3 0 3
:4 0 43 1 :8 0
153 2d :4 0 13e
:3 0 2e :3 0 2f
:2 0 a9 141 142
:3 0 30 :3 0 31
:3 0 144 145 0
32 :2 0 146 147
0 149 ac 150
30 :3 0 31 :3 0
14a 14b 0 33
:2 0 14c 14d 0
14f ae 151 143
149 0 152 0
14f 0 152 b0
0 153 b3 1a6
34 :3 0 30 :3 0
31 :3 0 155 156
0 35 :3 0 157
158 0 15a b6
15c b8 15b 15a
:2 0 1a4 36 :3 0
30 :3 0 31 :3 0
15f 160 0 35
:3 0 161 162 0
19c d :3 0 e
:3 0 164 165 0
d :3 0 1e :3 0
167 168 0 37
:2 0 2 :4 0 44
:2 0 21 :2 0 1f
:2 0 ba 16d 16f
:3 0 bd 169 171
166 173 172 :2 0
10 :3 0 22 :3 0
175 176 0 12
:4 0 45 :4 0 15
:2 0 16 :3 0 c1
17a 17c :3 0 15
:2 0 3a :4 0 c4
17e 180 :3 0 15
:2 0 3b :4 0 c7
182 184 :3 0 15
:2 0 35 :3 0 ca
186 188 :3 0 15
:2 0 3c :4 0 cd
18a 18c :3 0 15
:2 0 3d :3 0 3e
:3 0 1f :2 0 3f
:2 0 d0 18f 193
d4 18e 195 :3 0
d7 177 197 :2 0
199 da 19a 174
199 0 19b dc
0 19c de 19f
:3 0 19f 0 19f
19e 19c 19d :6 0
1a1 4 :3 0 e1
1a3 e3 1a2 1a1
:2 0 1a4 e5 :2 0
1a6 0 1a6 1a5
153 1a4 :6 0 1a8
1 :3 0 e8 1aa
ea 1a9 1a8 :2 0
5bf 24 :3 0 46
:3 0 1ab 1ac 0
47 :3 0 27 :3 0
28 :3 0 27 :3 0
7 :3 0 7 :3 0
29 :3 0 27 :3 0
7 :3 0 2a :3 0
7 :3 0 2a :3 0
27 :3 0 27 :3 0
7 :3 0 27 :3 0
2b :3 0 3 :4 0
48 1 :8 0 1d6
2d :4 0 1c1 :3 0
2e :3 0 2f :2 0
ee 1c4 1c5 :3 0
30 :3 0 31 :3 0
1c7 1c8 0 32
:2 0 1c9 1ca 0
1cc f1 1d3 30
:3 0 31 :3 0 1cd
1ce 0 33 :2 0
1cf 1d0 0 1d2
f3 1d4 1c6 1cc
0 1d5 0 1d2
0 1d5 f5 0
1d6 f8 229 34
:3 0 30 :3 0 31
:3 0 1d8 1d9 0
35 :3 0 1da 1db
0 1dd fb 1df
fd 1de 1dd :2 0
227 36 :3 0 30
:3 0 31 :3 0 1e2
1e3 0 35 :3 0
1e4 1e5 0 21f
d :3 0 e :3 0
1e7 1e8 0 d
:3 0 1e :3 0 1ea
1eb 0 37 :2 0
2 :4 0 49 :2 0
21 :2 0 1f :2 0
ff 1f0 1f2 :3 0
102 1ec 1f4 1e9
1f6 1f5 :2 0 10
:3 0 22 :3 0 1f8
1f9 0 12 :4 0
4a :4 0 15 :2 0
16 :3 0 106 1fd
1ff :3 0 15 :2 0
3a :4 0 109 201
203 :3 0 15 :2 0
3b :4 0 10c 205
207 :3 0 15 :2 0
35 :3 0 10f 209
20b :3 0 15 :2 0
3c :4 0 112 20d
20f :3 0 15 :2 0
3d :3 0 3e :3 0
1f :2 0 3f :2 0
115 212 216 119
211 218 :3 0 11c
1fa 21a :2 0 21c
11f 21d 1f7 21c
0 21e 121 0
21f 123 222 :3 0
222 0 222 221
21f 220 :6 0 224
6 :3 0 126 226
128 225 224 :2 0
227 12a :2 0 229
0 229 228 1d6
227 :6 0 22b 1
:3 0 12d 22d 12f
22c 22b :2 0 5bf
24 :3 0 4b :3 0
22e 22f 0 4c
:3 0 27 :3 0 28
:3 0 27 :3 0 7
:3 0 7 :3 0 29
:3 0 27 :3 0 7
:3 0 2a :3 0 7
:3 0 2a :3 0 27
:3 0 27 :3 0 7
:3 0 27 :3 0 2b
:3 0 3 :4 0 4d
1 :8 0 259 2d
:4 0 244 :3 0 2e
:3 0 2f :2 0 133
247 248 :3 0 30
:3 0 31 :3 0 24a
24b 0 32 :2 0
24c 24d 0 24f
136 256 30 :3 0
31 :3 0 250 251
0 33 :2 0 252
253 0 255 138
257 249 24f 0
258 0 255 0
258 13a 0 259
13d 2ac 34 :3 0
30 :3 0 31 :3 0
25b 25c 0 35
:3 0 25d 25e 0
260 140 262 142
261 260 :2 0 2aa
36 :3 0 30 :3 0
31 :3 0 265 266
0 35 :3 0 267
268 0 2a2 d
:3 0 e :3 0 26a
26b 0 d :3 0
1e :3 0 26d 26e
0 37 :2 0 2
:4 0 4e :2 0 21
:2 0 1f :2 0 144
273 275 :3 0 147
26f 277 26c 279
278 :2 0 10 :3 0
22 :3 0 27b 27c
0 12 :4 0 4f
:4 0 15 :2 0 16
:3 0 14b 280 282
:3 0 15 :2 0 3a
:4 0 14e 284 286
:3 0 15 :2 0 3b
:4 0 151 288 28a
:3 0 15 :2 0 35
:3 0 154 28c 28e
:3 0 15 :2 0 3c
:4 0 157 290 292
:3 0 15 :2 0 3d
:3 0 3e :3 0 1f
:2 0 3f :2 0 15a
295 299 15e 294
29b :3 0 161 27d
29d :2 0 29f 164
2a0 27a 29f 0
2a1 166 0 2a2
168 2a5 :3 0 2a5
0 2a5 2a4 2a2
2a3 :6 0 2a7 8
:3 0 16b 2a9 16d
2a8 2a7 :2 0 2aa
16f :2 0 2ac 0
2ac 2ab 259 2aa
:6 0 2ae 1 :3 0
172 2b0 174 2af
2ae :2 0 5bf 24
:3 0 50 :3 0 2b1
2b2 0 51 :3 0
27 :3 0 28 :3 0
27 :3 0 7 :3 0
7 :3 0 29 :3 0
27 :3 0 7 :3 0
2a :3 0 7 :3 0
2a :3 0 27 :3 0
27 :3 0 7 :3 0
27 :3 0 2b :3 0
3 :4 0 52 1
:8 0 2dc 2d :4 0
2c7 :3 0 2e :3 0
2f :2 0 178 2ca
2cb :3 0 30 :3 0
31 :3 0 2cd 2ce
0 32 :2 0 2cf
2d0 0 2d2 17b
2d9 30 :3 0 31
:3 0 2d3 2d4 0
33 :2 0 2d5 2d6
0 2d8 17d 2da
2cc 2d2 0 2db
0 2d8 0 2db
17f 0 2dc 182
32f 34 :3 0 30
:3 0 31 :3 0 2de
2df 0 35 :3 0
2e0 2e1 0 2e3
185 2e5 187 2e4
2e3 :2 0 32d 36
:3 0 30 :3 0 31
:3 0 2e8 2e9 0
35 :3 0 2ea 2eb
0 325 d :3 0
e :3 0 2ed 2ee
0 d :3 0 1e
:3 0 2f0 2f1 0
37 :2 0 2 :4 0
53 :2 0 21 :2 0
1f :2 0 189 2f6
2f8 :3 0 18c 2f2
2fa 2ef 2fc 2fb
:2 0 10 :3 0 22
:3 0 2fe 2ff 0
12 :4 0 54 :4 0
15 :2 0 16 :3 0
190 303 305 :3 0
15 :2 0 3a :4 0
193 307 309 :3 0
15 :2 0 3b :4 0
196 30b 30d :3 0
15 :2 0 35 :3 0
199 30f 311 :3 0
15 :2 0 3c :4 0
19c 313 315 :3 0
15 :2 0 3d :3 0
3e :3 0 1f :2 0
3f :2 0 19f 318
31c 1a3 317 31e
:3 0 1a6 300 320
:2 0 322 1a9 323
2fd 322 0 324
1ab 0 325 1ad
328 :3 0 328 0
328 327 325 326
:6 0 32a a :3 0
1b0 32c 1b2 32b
32a :2 0 32d 1b4
:2 0 32f 0 32f
32e 2dc 32d :6 0
331 1 :3 0 1b7
333 1b9 332 331
:2 0 5bf 24 :3 0
55 :3 0 334 335
0 56 :3 0 27
:3 0 28 :3 0 27
:3 0 7 :3 0 7
:3 0 29 :3 0 27
:3 0 7 :3 0 2a
:3 0 7 :3 0 2a
:3 0 27 :3 0 27
:3 0 7 :3 0 27
:3 0 2b :3 0 3
:4 0 57 1 :8 0
35f 2d :4 0 34a
:3 0 2e :3 0 2f
:2 0 1bd 34d 34e
:3 0 30 :3 0 31
:3 0 350 351 0
32 :2 0 352 353
0 355 1c0 35c
30 :3 0 31 :3 0
356 357 0 33
:2 0 358 359 0
35b 1c2 35d 34f
355 0 35e 0
35b 0 35e 1c4
0 35f 1c7 3b2
34 :3 0 30 :3 0
31 :3 0 361 362
0 35 :3 0 363
364 0 366 1ca
368 1cc 367 366
:2 0 3b0 36 :3 0
30 :3 0 31 :3 0
36b 36c 0 35
:3 0 36d 36e 0
3a8 d :3 0 e
:3 0 370 371 0
d :3 0 1e :3 0
373 374 0 37
:2 0 2 :4 0 58
:2 0 21 :2 0 1f
:2 0 1ce 379 37b
:3 0 1d1 375 37d
372 37f 37e :2 0
10 :3 0 22 :3 0
381 382 0 12
:4 0 59 :4 0 15
:2 0 16 :3 0 1d5
386 388 :3 0 15
:2 0 3a :4 0 1d8
38a 38c :3 0 15
:2 0 3b :4 0 1db
38e 390 :3 0 15
:2 0 35 :3 0 1de
392 394 :3 0 15
:2 0 3c :4 0 1e1
396 398 :3 0 15
:2 0 3d :3 0 3e
:3 0 1f :2 0 3f
:2 0 1e4 39b 39f
1e8 39a 3a1 :3 0
1eb 383 3a3 :2 0
3a5 1ee 3a6 380
3a5 0 3a7 1f0
0 3a8 1f2 3ab
:3 0 3ab 0 3ab
3aa 3a8 3a9 :6 0
3ad c :3 0 1f5
3af 1f7 3ae 3ad
:2 0 3b0 1f9 :2 0
3b2 0 3b2 3b1
35f 3b0 :6 0 3b4
1 :3 0 1fc 3b6
1fe 3b5 3b4 :2 0
5bf 24 :3 0 5a
:3 0 3b7 3b8 0
5b :3 0 27 :3 0
28 :3 0 27 :3 0
7 :3 0 7 :3 0
29 :3 0 27 :3 0
7 :3 0 2a :3 0
7 :3 0 2a :3 0
27 :3 0 27 :3 0
7 :3 0 27 :3 0
2b :3 0 3 :4 0
5c 1 :8 0 3e2
2d :4 0 3cd :3 0
2e :3 0 2f :2 0
202 3d0 3d1 :3 0
30 :3 0 31 :3 0
3d3 3d4 0 32
:2 0 3d5 3d6 0
3d8 205 3df 30
:3 0 31 :3 0 3d9
3da 0 33 :2 0
3db 3dc 0 3de
207 3e0 3d2 3d8
0 3e1 0 3de
0 3e1 209 0
3e2 20c 435 34
:3 0 30 :3 0 31
:3 0 3e4 3e5 0
35 :3 0 3e6 3e7
0 3e9 20f 3eb
211 3ea 3e9 :2 0
433 36 :3 0 30
:3 0 31 :3 0 3ee
3ef 0 35 :3 0
3f0 3f1 0 42b
d :3 0 e :3 0
3f3 3f4 0 d
:3 0 1e :3 0 3f6
3f7 0 37 :2 0
2 :4 0 5d :2 0
21 :2 0 1f :2 0
213 3fc 3fe :3 0
216 3f8 400 3f5
402 401 :2 0 10
:3 0 22 :3 0 404
405 0 12 :4 0
5e :4 0 15 :2 0
16 :3 0 21a 409
40b :3 0 15 :2 0
3a :4 0 21d 40d
40f :3 0 15 :2 0
3b :4 0 220 411
413 :3 0 15 :2 0
35 :3 0 223 415
417 :3 0 15 :2 0
3c :4 0 226 419
41b :3 0 15 :2 0
3d :3 0 3e :3 0
1f :2 0 3f :2 0
229 41e 422 22d
41d 424 :3 0 230
406 426 :2 0 428
233 429 403 428
0 42a 235 0
42b 237 42e :3 0
42e 0 42e 42d
42b 42c :6 0 430
e :3 0 23a 432
23c 431 430 :2 0
433 23e :2 0 435
0 435 434 3e2
433 :6 0 437 1
:3 0 241 439 243
438 437 :2 0 5bf
24 :3 0 5f :3 0
43a 43b 0 60
:3 0 27 :3 0 28
:3 0 27 :3 0 7
:3 0 7 :3 0 29
:3 0 27 :3 0 7
:3 0 2a :3 0 7
:3 0 2a :3 0 27
:3 0 27 :3 0 7
:3 0 27 :3 0 2b
:3 0 3 :4 0 61
1 :8 0 465 2d
:4 0 450 :3 0 2e
:3 0 2f :2 0 247
453 454 :3 0 30
:3 0 31 :3 0 456
457 0 32 :2 0
458 459 0 45b
24a 462 30 :3 0
31 :3 0 45c 45d
0 33 :2 0 45e
45f 0 461 24c
463 455 45b 0
464 0 461 0
464 24e 0 465
251 4b8 34 :3 0
30 :3 0 31 :3 0
467 468 0 35
:3 0 469 46a 0
46c 254 46e 256
46d 46c :2 0 4b6
36 :3 0 30 :3 0
31 :3 0 471 472
0 35 :3 0 473
474 0 4ae d
:3 0 e :3 0 476
477 0 d :3 0
1e :3 0 479 47a
0 37 :2 0 2
:4 0 62 :2 0 21
:2 0 1f :2 0 258
47f 481 :3 0 25b
47b 483 478 485
484 :2 0 10 :3 0
22 :3 0 487 488
0 12 :4 0 63
:4 0 15 :2 0 16
:3 0 25f 48c 48e
:3 0 15 :2 0 3a
:4 0 262 490 492
:3 0 15 :2 0 3b
:4 0 265 494 496
:3 0 15 :2 0 35
:3 0 268 498 49a
:3 0 15 :2 0 3c
:4 0 26b 49c 49e
:3 0 15 :2 0 3d
:3 0 3e :3 0 1f
:2 0 3f :2 0 26e
4a1 4a5 272 4a0
4a7 :3 0 275 489
4a9 :2 0 4ab 278
4ac 486 4ab 0
4ad 27a 0 4ae
27c 4b1 :3 0 4b1
0 4b1 4b0 4ae
4af :6 0 4b3 10
:3 0 27f 4b5 281
4b4 4b3 :2 0 4b6
283 :2 0 4b8 0
4b8 4b7 465 4b6
:6 0 4ba 1 :3 0
286 4bc 288 4bb
4ba :2 0 5bf 24
:3 0 64 :3 0 4bd
4be 0 65 :3 0
27 :3 0 28 :3 0
27 :3 0 7 :3 0
7 :3 0 29 :3 0
27 :3 0 7 :3 0
2a :3 0 7 :3 0
2a :3 0 27 :3 0
27 :3 0 7 :3 0
27 :3 0 2b :3 0
3 :4 0 66 1
:8 0 4e8 2d :4 0
4d3 :3 0 2e :3 0
2f :2 0 28c 4d6
4d7 :3 0 30 :3 0
31 :3 0 4d9 4da
0 32 :2 0 4db
4dc 0 4de 28f
4e5 30 :3 0 31
:3 0 4df 4e0 0
33 :2 0 4e1 4e2
0 4e4 291 4e6
4d8 4de 0 4e7
0 4e4 0 4e7
293 0 4e8 296
53b 34 :3 0 30
:3 0 31 :3 0 4ea
4eb 0 35 :3 0
4ec 4ed 0 4ef
299 4f1 29b 4f0
4ef :2 0 539 36
:3 0 30 :3 0 31
:3 0 4f4 4f5 0
35 :3 0 4f6 4f7
0 531 d :3 0
e :3 0 4f9 4fa
0 d :3 0 1e
:3 0 4fc 4fd 0
37 :2 0 2 :4 0
67 :2 0 21 :2 0
1f :2 0 29d 502
504 :3 0 2a0 4fe
506 4fb 508 507
:2 0 10 :3 0 22
:3 0 50a 50b 0
12 :4 0 68 :4 0
15 :2 0 16 :3 0
2a4 50f 511 :3 0
15 :2 0 3a :4 0
2a7 513 515 :3 0
15 :2 0 3b :4 0
2aa 517 519 :3 0
15 :2 0 35 :3 0
2ad 51b 51d :3 0
15 :2 0 3c :4 0
2b0 51f 521 :3 0
15 :2 0 3d :3 0
3e :3 0 1f :2 0
3f :2 0 2b3 524
528 2b7 523 52a
:3 0 2ba 50c 52c
:2 0 52e 2bd 52f
509 52e 0 530
2bf 0 531 2c1
534 :3 0 534 0
534 533 531 532
:6 0 536 12 :3 0
2c4 538 2c6 537
536 :2 0 539 2c8
:2 0 53b 0 53b
53a 4e8 539 :6 0
53d 1 :3 0 2cb
53f 2cd 53e 53d
:2 0 5bf 69 :3 0
27 :3 0 28 :3 0
27 :3 0 7 :3 0
7 :3 0 29 :3 0
27 :3 0 7 :3 0
2a :3 0 7 :3 0
2a :3 0 27 :3 0
27 :3 0 7 :3 0
27 :3 0 2b :3 0
3 :4 0 6a 1
:8 0 568 2d :4 0
553 :3 0 2e :3 0
2f :2 0 2d1 556
557 :3 0 30 :3 0
31 :3 0 559 55a
0 32 :2 0 55b
55c 0 55e 2d4
565 30 :3 0 31
:3 0 55f 560 0
33 :2 0 561 562
0 564 2d6 566
558 55e 0 567
0 564 0 567
2d8 0 568 2db
5bb 34 :3 0 30
:3 0 31 :3 0 56a
56b 0 35 :3 0
56c 56d 0 56f
2de 571 2e0 570
56f :2 0 5b9 36
:3 0 30 :3 0 31
:3 0 574 575 0
35 :3 0 576 577
0 5b1 d :3 0
e :3 0 579 57a
0 d :3 0 1e
:3 0 57c 57d 0
37 :2 0 2 :4 0
6b :2 0 21 :2 0
1f :2 0 2e2 582
584 :3 0 2e5 57e
586 57b 588 587
:2 0 10 :3 0 22
:3 0 58a 58b 0
12 :4 0 6c :4 0
15 :2 0 16 :3 0
2e9 58f 591 :3 0
15 :2 0 3a :4 0
2ec 593 595 :3 0
15 :2 0 3b :4 0
2ef 597 599 :3 0
15 :2 0 35 :3 0
2f2 59b 59d :3 0
15 :2 0 3c :4 0
2f5 59f 5a1 :3 0
15 :2 0 3d :3 0
3e :3 0 1f :2 0
3f :2 0 2f8 5a4
5a8 2fc 5a3 5aa
:3 0 2ff 58c 5ac
:2 0 5ae 302 5af
589 5ae 0 5b0
304 0 5b1 306
5b4 :3 0 5b4 0
5b4 5b3 5b1 5b2
:6 0 5b6 14 :3 0
309 5b8 30b 5b7
5b6 :2 0 5b9 30d
:2 0 5bb 0 5bb
5ba 568 5b9 :6 0
5bd 1 :3 0 310
5be 0 5bd :2 0
5bf 312 :2 0 5c0
a2 5bf 0 5f3
0 d :3 0 e
:3 0 5c1 5c2 0
d :3 0 1e :3 0
5c4 5c5 0 1f
:2 0 2 :4 0 6d
:2 0 21 :2 0 1f
:2 0 31d 5ca 5cc
:3 0 320 5c6 5ce
5c3 5d0 5cf :2 0
10 :3 0 22 :3 0
5d2 5d3 0 12
:4 0 6e :4 0 15
:2 0 16 :3 0 324
5d7 5d9 :3 0 15
:2 0 3a :4 0 327
5db 5dd :3 0 15
:2 0 6f :4 0 32a
5df 5e1 :3 0 15
:2 0 30 :3 0 31
:3 0 5e4 5e5 0
32d 5e3 5e7 :3 0
330 5d4 5e9 :2 0
5eb 333 5ec 5d1
5eb 0 5ed 335
0 5f3 b :3 0
30 :3 0 31 :3 0
5ef 5f0 0 5f1
:2 0 5f3 337 5f6
:3 0 5f6 33d 5f6
5f5 5f3 5f4 :6 0
5f7 :2 0 2 1c
5f6 5fa :3 0 5f9
5f7 5fb :8 0 
33f
4
:3 0 1 3 1
9 1 f 1
15 4 8 e
14 18 1 1e
1 2a 1 36
2 39 3b 2
3e 40 2 42
44 2 46 48
2 4a 4c 2
4e 50 2 52
54 2 56 58
2 5a 5c 2
5e 60 2 62
64 2 66 68
2 6a 6c 1
72 2 74 75
2 6e 77 2
79 7b 4 32
38 3d 7d 1
7f 1 81 2
8b 8d 3 89
8a 8f 2 98
9a 2 97 9c
1 9e 1 a0
1 bb 2 ba
bb 1 c3 1
c9 2 cb cc
2 b8 cd 1
d4 1 cf 2
e7 e9 3 e5
e6 eb 2 f4
f6 2 f8 fa
2 fc fe 2
100 102 2 104
106 3 10b 10c
10d 2 108 10f
2 f3 111 1
113 1 115 2
de 116 1 11a
1 d9 2 d7
11e 1 121 1
a5 1 140 2
13f 140 1 148
1 14e 2 150
151 2 13d 152
1 159 1 154
2 16c 16e 3
16a 16b 170 2
179 17b 2 17d
17f 2 181 183
2 185 187 2
189 18b 3 190
191 192 2 18d
194 2 178 196
1 198 1 19a
2 163 19b 1
19f 1 15e 2
15c 1a3 1 1a6
1 128 1 1c3
2 1c2 1c3 1
1cb 1 1d1 2
1d3 1d4 2 1c0
1d5 1 1dc 1
1d7 2 1ef 1f1
3 1ed 1ee 1f3
2 1fc 1fe 2
200 202 2 204
206 2 208 20a
2 20c 20e 3
213 214 215 2
210 217 2 1fb
219 1 21b 1
21d 2 1e6 21e
1 222 1 1e1
2 1df 226 1
229 1 1ad 1
246 2 245 246
1 24e 1 254
2 256 257 2
243 258 1 25f
1 25a 2 272
274 3 270 271
276 2 27f 281
2 283 285 2
287 289 2 28b
28d 2 28f 291
3 296 297 298
2 293 29a 2
27e 29c 1 29e
1 2a0 2 269
2a1 1 2a5 1
264 2 262 2a9
1 2ac 1 230
1 2c9 2 2c8
2c9 1 2d1 1
2d7 2 2d9 2da
2 2c6 2db 1
2e2 1 2dd 2
2f5 2f7 3 2f3
2f4 2f9 2 302
304 2 306 308
2 30a 30c 2
30e 310 2 312
314 3 319 31a
31b 2 316 31d
2 301 31f 1
321 1 323 2
2ec 324 1 328
1 2e7 2 2e5
32c 1 32f 1
2b3 1 34c 2
34b 34c 1 354
1 35a 2 35c
35d 2 349 35e
1 365 1 360
2 378 37a 3
376 377 37c 2
385 387 2 389
38b 2 38d 38f
2 391 393 2
395 397 3 39c
39d 39e 2 399
3a0 2 384 3a2
1 3a4 1 3a6
2 36f 3a7 1
3ab 1 36a 2
368 3af 1 3b2
1 336 1 3cf
2 3ce 3cf 1
3d7 1 3dd 2
3df 3e0 2 3cc
3e1 1 3e8 1
3e3 2 3fb 3fd
3 3f9 3fa 3ff
2 408 40a 2
40c 40e 2 410
412 2 414 416
2 418 41a 3
41f 420 421 2
41c 423 2 407
425 1 427 1
429 2 3f2 42a
1 42e 1 3ed
2 3eb 432 1
435 1 3b9 1
452 2 451 452
1 45a 1 460
2 462 463 2
44f 464 1 46b
1 466 2 47e
480 3 47c 47d
482 2 48b 48d
2 48f 491 2
493 495 2 497
499 2 49b 49d
3 4a2 4a3 4a4
2 49f 4a6 2
48a 4a8 1 4aa
1 4ac 2 475
4ad 1 4b1 1
470 2 46e 4b5
1 4b8 1 43c
1 4d5 2 4d4
4d5 1 4dd 1
4e3 2 4e5 4e6
2 4d2 4e7 1
4ee 1 4e9 2
501 503 3 4ff
500 505 2 50e
510 2 512 514
2 516 518 2
51a 51c 2 51e
520 3 525 526
527 2 522 529
2 50d 52b 1
52d 1 52f 2
4f8 530 1 534
1 4f3 2 4f1
538 1 53b 1
4bf 1 555 2
554 555 1 55d
1 563 2 565
566 2 552 567
1 56e 1 569
2 581 583 3
57f 580 585 2
58e 590 2 592
594 2 596 598
2 59a 59c 2
59e 5a0 3 5a5
5a6 5a7 2 5a2
5a9 2 58d 5ab
1 5ad 1 5af
2 578 5b0 1
5b4 1 573 2
571 5b8 1 5bb
a 125 1aa 22d
2b0 333 3b6 439
4bc 53f 5be 2
5c9 5cb 3 5c7
5c8 5cd 2 5d6
5d8 2 5da 5dc
2 5de 5e0 2
5e2 5e6 2 5d5
5e8 1 5ea 1
5ec 5 82 a1
5c0 5ed 5f2 1
22 
1
4
0 
5fa
0
1
28
15
6
0 1 2 1 4 1 6 1
8 1 a 1 c 1 e 1
10 1 12 1 14 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0

2 0 1
f 1 0
3 1 0
1e 1 0
9 1 0
15 1 0
0

/
SHOW ERRORS FUNCTION FC_U_NAMES;
ALTER FUNCTION FC_U_NAMES COMPILE DEBUG;
/******************************************************************************************************************
update resource assignments SECURITY
assign_id:      RES_ASSIGNMENT_ID or TSK_ASSIGNMENT_ID
security_id:    SECURITY_ID in TMT_GLOBAL_SECURITY
*******************************************************************************************************************/
Prompt == FUNCTION SP_U_WP_DATES ==
CREATE OR REPLACE FUNCTION SP_U_WP_DATES wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
f0
2 :e:
1FUNCTION:
1SP_U_WP_DATES:
1ASSIGN_ID:
1VARCHAR2:
1P_PLANNED_START:
1TIMESTAMP:
1P_PLANNED_FINISH:
1RES_ID001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_TSK_ASSIGNMENT_ID:
132:
1H_PRJ_ASSIGN_ID:
1H_PROJECT_ID:
1H_ACTUAL_START:
1H_START_DATE:
1H_FINISH_DATE:
1H_SCHED_START:
1H_SCHED_FINISH:
1H_SD:
1DATE:
1H_FD:
1H_BIT_FLAG:
11:
1H_RES_TYPE:
1H_BILLABLE:
1PLS_INTEGER:
10:
1H_CAPITAL_EXPENSE:
1H_TPH_DELETE:
1H_DAYOFYEAR:
1H_LAST_SUBMITTED:
1H_PARTICIPANT:
1H_UPDATE:
1H_TABLE_TYPE:
1H_TRANSFED_CRI:
1H_TASK_ID:
1H_RES_ID:
1H_REC_USER:
1H_EFFORT:
1H_CONTOUR:
1H_DELAY:
1H_PERCENT_UNITS:
1H_WORK_PERCENT:
1H_WORK_TODATE:
1H_NEW_ACTUAL:
1H_WORK:
1H_SKIP:
1H_LAST_DAY:
1H_MINUTES:
1H_PER_DAY:
1H_WORK_TYPE:
1H_ASSIGN_COUNT:
1H_DURATION:
1H_BITFLAG:
1H_CONSIDER:
1H_EUNIT:
1H_UNIT:
1H_SCHED_EFFORT:
1H_TEMP_SC_STATE:
1H_TEMP_VAR:
1H_REG_COST:
1NUMBER:
1H_REG_SALE:
1H_SPEC_COST:
1H_SPEC_SALE:
1H_B_REG_COST:
1H_B_REG_SALE:
1H_B_SPEC_COST:
1H_B_SPEC_SALE:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_U_WP_DATES(:
1':
1, :
1COALESCE:
1NULLIF:
1'':
1NULL:
1);:
1CHECK_TRACE:
174:
1+:
1FPUT:
1               ENTER:: SP_U_WP_DATES(75) :: cplan.sql(14730) :: :
1PROJECT_ASSIGNMENT_ID:
1PROJECT_ID:
1RESOURCE_ID:
1RESOURCE_TYPE:
1BIT_FLAG:
1LAST_SUBMITTED:
1C_CONTOUR:
1C_EFFORT:
1T_UNIT:
1C_UNIT:
1UNITS_MSP:
1ACTUAL_START:
1WORK_TODATE:
1UNRESOLVED_START:
1UNRESOLVED_FINISH:
1PLANNED_WORK:
1SCHED_WORK:
1ASSIGNMENT_DELAY:
1LAST_SCHED_MIN:
1TO_DATE:
1SUBSTR:
1BILLABLE:
1CAPITAL_EXPENSE:
1REGULAR_COST:
1SPECIAL_COST:
1REGULAR_RATE:
1SPECIAL_RATE:
1B_REGULAR_COST:
1B_SPECIAL_COST:
1B_REGULAR_RATE:
1B_SPECIAL_RATE:
1MILESTONE_PERCENT:
1TASK_ID:
1TASK_ASSIGNMENTS:
1TSK_ASSIGNMENT_ID:
1SELECT PROJECT_ASSIGNMENT_ID, PROJECT_ID, RESOURCE_ID,:n      RESOURCE_TYPE, +
1BIT_FLAG, LAST_SUBMITTED, C_CONTOUR, C_EFFORT, T_UNIT, C_UNIT,:n      UNITS_M+
1SP, ACTUAL_START, WORK_TODATE, UNRESOLVED_START, UNRESOLVED_FINISH,:n      CA+
1SE WHEN BIT_FLAG = '0' THEN PLANNED_WORK ELSE SCHED_WORK END, ASSIGNMENT_DELA+
1Y, LAST_SCHED_MIN,:n      TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(S+
1ubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,:n      REGULAR_C+
1OST, SPECIAL_COST, REGULAR_RATE, SPECIAL_RATE,:n      B_REGULAR_COST, B_SPECI+
1AL_COST, B_REGULAR_RATE, B_SPECIAL_RATE,:n      MILESTONE_PERCENT, TASK_ID:n +
1     INTO h_prj_assign_id, h_project_id, h_res_id,:n      h_res_type, h_bit_f+
1lag, h_last_submitted, h_contour, h_work, h_eunit, h_unit,:n      h_percent_u+
1nits, h_actual_start, h_work_todate, h_start_date, h_finish_date,:n      h_ef+
1fort, h_delay, h_last_day,:n      h_SD, h_FD, h_billable, h_capital_expense,+
1:n      h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale,:n      h_b_reg_cost+
1, h_b_spec_cost, h_b_reg_sale, h_b_spec_sale,:n      h_work_percent, h_task_i+
1d:n      FROM TASK_ASSIGNMENTS:n      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignm+
1ent_id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
1110:
1           EXCEPTION:: SP_U_WP_DATES(111) :: cplan.sql(14766) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SQLERRM:
1500:
1=:
1SC_SQL_NO_DATA:
1118:
1        RETURN_ERROR:: SP_U_WP_DATES(119) :: cplan.sql(14774) :: :
1STATE:: :
1TO_CHAR:
1PMO_CNST:
1RECORD_NOT_FOUND:
1 ID:: :
1121:
1                EXIT:: SP_U_WP_DATES(122) :: cplan.sql(14777) :: :
1PMODEFINES:
1PMO_RETURN_ERROR_SP:
1TRUE:
1IS NOT NULL:
1PMO_ORASTD:
1STRNCMP:
110:
1>:
1!=:
1134:
1        RETURN_ERROR:: SP_U_WP_DATES(135) :: cplan.sql(14790) :: :
1F_DATE_EARLIER_THAN_S_DATE:
1137:
1                EXIT:: SP_U_WP_DATES(138) :: cplan.sql(14793) :: :
1MINUTE_PER_DAY:
1TMT_WBS:
1ELEMENT_ID:
1SELECT PROJECT_ID, MINUTE_PER_DAY:n         INTO h_project_id, h_per_day:n   +
1      FROM TMT_WBS WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id:
1155:
1           EXCEPTION:: SP_U_WP_DATES(156) :: cplan.sql(14811) :: :
18:
1167:
1      RETURN_INVALID:: SP_U_WP_DATES(168) :: cplan.sql(14823) :: :
1INVALID_PARAMS:
1170:
1                EXIT:: SP_U_WP_DATES(171) :: cplan.sql(14826) :: :
1PMO_RETURN_INVALID_SP:
1-:
1FC_TPHASE_SCHED_ASSIGN:
1FC_U_TDY_LAST_ACTUAL:
1FC_DEL_NOACTUAL:
12:
1DB2TIMESTAMP:
1TIME:
1DUAL:
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_+
1ORASTD.TIME(h_start_date)) + 1     :n         INTO h_start_date:n         FRO+
1M DUAL:
1211:
1           EXCEPTION:: SP_U_WP_DATES(212) :: cplan.sql(14867) :: :
1FC_RESCHEDULE:
1228:
1        RETURN_ERROR:: SP_U_WP_DATES(229) :: cplan.sql(14884) :: :
1TASK_ASSIGNMENT_ERROR:
1231:
1                EXIT:: SP_U_WP_DATES(232) :: cplan.sql(14887) :: :
1BAK_ASSIGN_STRING:
1UPDATE TASK_ASSIGNMENTS SET:n              BAK_ASSIGN_STRING = NULL:n        +
1 WHERE TSK_ASSIGNMENT_ID = H_TSK_ASSIGNMENT_ID:
1CONSIDER:
1ROLLUP_BITS:
1PMO_STPU:
1BIT_OR:
1SC_SCHED_BIT:
1SC_PLANNED_BIT:
1BAK_START_DT:
1SCHED_START:
1PLANNED_START:
1BAK_FINISH_DT:
1SCHED_FINISH:
1PLANNED_FINISH:
1ASSIGN_STRING:
1UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLU+
1P_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT)),:n         BAK_START_+
1DT = COALESCE(SCHED_START,PLANNED_START), BAK_FINISH_DT = COALESCE(SCHED_FINI+
1SH,PLANNED_FINISH),:n         BAK_ASSIGN_STRING = ASSIGN_STRING:n         WHE+
1RE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
1FOUND:
1100:
1257:
1           EXCEPTION:: SP_U_WP_DATES(258) :: cplan.sql(14913) :: :
1FC_ROLLUP:
1265:
1        RETURN_ERROR:: SP_U_WP_DATES(266) :: cplan.sql(14921) :: :
1268:
1                EXIT:: SP_U_WP_DATES(269) :: cplan.sql(14924) :: :
14:
1274:
1       RETURN_STATUS:: SP_U_WP_DATES(275) :: cplan.sql(14930) :: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
1281:
1           EXCEPTION:: SP_U_WP_DATES(282) :: cplan.sql(14937) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1ID:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1293:
1           EXCEPTION:: SP_U_WP_DATES(294) :: cplan.sql(14949) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
7f4
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d b4 :3 a0 6b 2c 6a
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 1c
81 b0 a3 a0 1c 81 b0 a3
a0 1c 81 b0 a3 a0 1c 81
b0 a3 a0 1c 81 b0 a3 a0
1c 81 b0 a3 a0 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 81 b0 a3 a0 1c
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 :2 a0 6b 1c 81
b0 :2 a0 6b :2 a0 6b 6e a5 b
a 10 :2 a0 6b 6e :2 a0 6b 6e
a5 b 6e 7e a0 b4 2e 6e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :2 a0 6e 7e a0 b4 2e 7e
6e b4 2e 6e a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
:2 a0 6e 7e a0 b4 2e 7e 6e
b4 2e 6e a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 d
:2 a0 d :49 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c a0
7e b4 2e 5a a0 7e b4 2e
5a a 10 :2 a0 d :2 a0 d :2 a0
6b :2 a0 51 a5 b 7e 51 b4
2e 5a :2 a0 6b :2 a0 51 a5 b
7e 51 b4 2e 5a a 10 :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 19
3c :9 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b :2 a0 51 a5
b 7e 51 b4 2e :2 a0 d a0
51 d b7 :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b a0
a5 57 b7 :2 19 3c b7 19 3c
a0 7e 51 b4 2e d :c a0 a5
b d :2 a0 51 a5 b 7e 51
b4 2e 5a a0 7e b4 2e :2 5a
a 10 :9 a0 51 a5 b d b7
19 3c a0 7e b4 2e 5a :4 a0
51 a5 b d b7 :2 a0 :2 51 a5
b 7e 6e b4 2e :4 a0 51 a5
b d b7 :4 a0 51 a5 b d
b7 :2 19 3c b7 :2 19 3c a0 7e
b4 2e 5a :a a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :3 a0 51 a5
b 7e :2 a0 51 a5 b b4 2e
d b7 19 3c a0 51 d :a a0
51 a5 b :2 a0 51 a5 b a5
b d :2 a0 51 a5 b 7e 51
b4 2e :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 :4 a0 12a :16 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c :3 a0
a5 b d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 51 a5 b
:2 a0 a5 57 :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 b7
:2 a0 6b :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 :2 a0 6b :2 6e 7e :2 a0 6b
b4 2e 7e 6e b4 2e 7e :2 a0
6b b4 2e a5 57 b7 19 3c
:9 a0 12a e9 dd :2 a0 65 b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c :7 a0 12a
e9 dd :2 a0 65 b7 a4 b1 11
4f b7 a6 9 a4 b1 11 68
4f 1d 17 b5 
7f4
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 59 6b 6f 73 56
77 7b 98 83 87 8a 8b 93
82 b5 a3 7f a7 a8 b0 a2
d2 c0 9f c4 c5 cd bf ee
dd e1 e9 bc 106 f5 f9 101
dc 122 111 115 11d d9 13a 129
12d 135 110 156 145 149 151 10d
16e 15d 161 169 144 18a 179 17d
185 141 1a6 191 195 198 199 1a1
178 1c3 1b1 175 1b5 1b6 1be 1b0
1df 1ce 1d2 1ad 1da 1cd 1fb 1ea
1ee 1ca 1f6 1e9 217 206 20a 1e6
212 205 233 222 226 22e 202 24b
23a 23e 246 221 267 256 25a 21e
262 255 283 272 276 252 27e 271
29f 28e 292 26e 29a 28d 2bb 2aa
2ae 28a 2b6 2a9 2d8 2c6 2a6 2ca
2cb 2d3 2c5 2f5 2e3 2c2 2e7 2e8
2f0 2e2 312 300 2df 304 305 30d
2ff 32e 31d 321 2fc 329 31c 34a
339 33d 319 345 338 366 355 359
335 361 354 382 371 375 351 37d
370 39e 38d 391 36d 399 38c 3ba
3a9 3ad 389 3b5 3a8 3d6 3c5 3c9
3a5 3d1 3c4 3f2 3e1 3e5 3c1 3ed
3e0 40e 3fd 401 3dd 409 3fc 42a
419 41d 3f9 425 418 446 435 439
415 441 434 462 451 455 431 45d
450 47e 46d 471 44d 479 46c 49a
489 48d 469 495 488 4b6 4a5 4a9
485 4b1 4a4 4d2 4c1 4c5 4a1 4cd
4c0 4ee 4dd 4e1 4bd 4e9 4dc 50a
4f9 4fd 4d9 505 4f8 526 515 519
4f5 521 514 542 531 535 511 53d
530 55e 54d 551 52d 559 54c 57a
569 56d 549 575 568 596 585 589
565 591 584 5b2 5a1 5a5 581 5ad
5a0 5ce 5bd 5c1 59d 5c9 5bc 5ea
5d9 5dd 5b9 5e5 5d8 606 5f5 5f9
5d5 601 5f4 622 611 615 5f1 61d
610 63e 62d 631 60d 639 62c 65a
649 64d 629 655 648 67a 665 669
645 66d 675 664 681 685 661 689
68d 691 694 699 69a 1 69c 6a1
6a5 6a9 6ac 6b1 6b5 6b9 6bc 6c1
6c2 6c4 6c9 6cc 6d0 6d1 6d6 6db
6de 6e3 6e4 6e9 6ec 6f0 6f1 6f6
6f9 6fe 6ff 704 707 70c 70d 712
715 719 71d 722 725 729 72a 72f
732 737 738 73d 742 743 745 74a
74b 74d 74e 753 756 75b 75c 761
764 768 76c 771 774 778 779 77e
781 786 787 78c 791 792 794 799
79a 79c 79d 7a2 7a5 7aa 7ab 7b0
7b3 7b8 7b9 7be 7c1 7c5 7c6 7cb
7ce 7d3 7d4 7d9 7dc 7e1 7e2 7e7
7e8 7ed 7ef 7f3 7f6 7fa 7fe 801
805 809 80c 80f 814 817 81a 81d
81e 823 824 1 826 82b 82f 833
836 83b 840 843 847 848 84d 84e
853 855 859 85c 860 864 868 86c
870 874 878 87c 880 884 888 88c
890 894 898 89c 8a0 8a4 8a8 8ac
8b0 8b4 8b8 8bc 8c0 8c4 8c8 8cc
8d0 8d4 8d8 8dc 8e0 8e4 8e8 8ec
8f0 8f4 8f8 8fc 900 904 908 90c
910 914 918 91c 920 924 928 92c
930 934 938 93c 940 944 948 94c
950 954 958 95c 960 964 968 96c
970 974 978 97c 980 984 988 98c
990 994 998 9a4 9a8 9ac 9af 9b3
9b7 9b9 9bd 9c1 9c5 9c8 9cc 9d0
9d2 9d3 9d8 1 9dc 9e0 9e4 9e7
9eb 9ef 9f3 9f7 9fa 9fe a02 a05
a08 a0d a10 a13 a16 a17 a1c a1d
1 a1f a24 a28 a2c a2f a34 a39
a3c a40 a41 a46 a49 a4e a4f a54
a57 a5c a5d a62 a65 a69 a6a a6f
a72 a77 a78 a7d a80 a84 a88 a8b
a8e a8f a91 a92 a97 a98 a9d a9f
aa3 aa6 aa8 aac aae aba abc abe
abf ac4 ac8 aca ad6 ad8 adc ae0
ae3 ae7 aea aee af1 af2 af7 afa
afe b02 b05 b09 b0d b10 b13 b18
b1b b1e b21 b22 b27 b28 1 b2a
b2f b33 b37 b3a b3f b44 b47 b4b
b4c b51 b54 b59 b5a b5f b62 b67
b68 b6d b70 b74 b78 b7c b80 b83
b84 b86 b8b b8c b8e b8f b94 b97
b9c b9d ba2 ba5 baa bab bb0 bb3
bb7 bb8 bbd bc0 bc5 bc6 bcb bcc
bd1 bd3 bd7 bda bde be2 be5 be9
bed bf0 bf3 bf8 bfb bfe c01 c02
c07 c08 1 c0a c0f c13 c17 c1a
c1f c24 c27 c2b c2c c31 c32 c37
c39 c3d c40 c44 c48 c4b c4f c53
c56 c5a c5e c5f c64 c66 c6a c6d
c71 c74 c75 c7a c7d c81 c84 c85
c8a 1 c8d c92 c96 c9a c9e ca2
ca6 caa cae cb2 cb5 cb9 cbd cc0
cc1 cc3 cc6 cc9 cca ccf cd2 cd6
cda cdd ce1 ce5 ce8 ce9 ceb cee
cf1 cf2 cf7 1 cfa cff d03 d07
d0a d0e d12 d15 d18 d1d d20 d23
d26 d27 d2c d2d 1 d2f d34 d38
d3c d3f d44 d49 d4c d50 d51 d56
d59 d5e d5f d64 d67 d6c d6d d72
d75 d79 d7d d81 d85 d88 d89 d8b
d90 d91 d93 d94 d99 d9c da1 da2
da7 daa daf db0 db5 db8 dbc dbd
dc2 dc5 dca dcb dd0 dd1 dd6 dd8
ddc ddf de3 de7 dea dee df2 df5
df8 dfd e00 e03 e06 e07 e0c e0d
1 e0f e14 e18 e1c e1f e24 e29
e2c e30 e31 e36 e37 e3c e3e e42
e45 e49 e4d e50 e54 e58 e5b e5f
e63 e64 e69 e6b e6f e72 e76 e7a
e7e e82 e86 e8a e8e e92 e96 ea2
ea6 eaa ead eb1 eb5 eb7 ebb ebf
ec3 ec6 eca ece ed0 ed1 ed6 1
eda ede ee2 ee5 ee9 eed ef1 ef5
ef8 efc f00 f03 f06 f0b f0e f11
f14 f15 f1a f1b 1 f1d f22 f26
f2a f2d f32 f37 f3a f3e f3f f44
f47 f4c f4d f52 f55 f5a f5b f60
f63 f67 f68 f6d f70 f75 f76 f7b
f7e f82 f86 f89 f8c f8d f8f f90
f95 f96 f9b f9d fa1 fa4 fa6 faa
fac fb8 fba fbc fbd fc2 fc6 fc8
fd4 fd6 fda fde fe1 fe5 fe9 fec
fed fef ff2 ff5 ff6 ffb fff 1003
1007 100b 100e 1012 1014 1018 101c 101f
1023 1027 102a 102d 1032 1035 1038 103b
103c 1041 1042 1 1044 1049 104d 1051
1054 1059 105e 1061 1065 1066 106b 106e
1073 1074 1079 107c 1081 1082 1087 108a
108e 1092 1096 109a 109d 109e 10a0 10a5
10a6 10a8 10a9 10ae 10af 10b4 10b6 10ba
10bd 10c1 10c5 10c8 10cc 10d0 10d3 10d6
10db 10de 10e1 10e4 10e5 10ea 10eb 1
10ed 10f2 10f6 10fa 10fd 1102 1107 110a
110e 110f 1114 1115 111a 111c 1120 1123
1127 112b 112e 1132 1136 1139 113d 113e
1143 1145 1149 114d 1150 1152 1156 1159
115d 1160 1163 1164 1169 116d 1171 1175
1179 117d 1181 1185 1189 118d 1191 1195
1199 119d 119e 11a0 11a4 11a8 11ac 11af
11b0 11b2 11b5 11b8 11b9 11be 11c1 11c5
11c8 11c9 11ce 11d1 1 11d4 11d9 11dd
11e1 11e5 11e9 11ed 11f1 11f5 11f9 11fd
1200 1201 1203 1207 1209 120d 1210 1214
1217 1218 121d 1220 1224 1228 122c 1230
1233 1234 1236 123a 123c 1240 1244 1247
124a 124b 124d 1250 1255 1256 125b 125f
1263 1267 126b 126e 126f 1271 1275 1277
127b 127f 1283 1287 128a 128b 128d 1291
1293 1297 129b 129e 12a0 12a4 12a8 12ab
12af 12b2 12b3 12b8 12bb 12bf 12c3 12c7
12cb 12cf 12d3 12d7 12db 12df 12e3 12ef
12f3 12f7 12fa 12fe 1302 1304 1308 130c
1310 1313 1317 131b 131d 131e 1323 1
1327 132b 132f 1332 1336 133a 133e 1342
1345 1349 134d 1350 1353 1358 135b 135e
1361 1362 1367 1368 1 136a 136f 1373
1377 137a 137f 1384 1387 138b 138c 1391
1394 1399 139a 139f 13a2 13a7 13a8 13ad
13b0 13b4 13b5 13ba 13bd 13c2 13c3 13c8
13cb 13cf 13d3 13d6 13d9 13da 13dc 13dd
13e2 13e3 13e8 13ea 13ee 13f1 13f3 13f7
13f9 1405 1407 1409 140a 140f 1413 1415
1421 1423 1427 142b 142f 1432 1433 1435
1438 143c 1440 1443 1444 1446 1447 144c
1450 1452 1456 1459 145d 1460 1464 1468
146c 1470 1474 1478 147c 1480 1484 1488
148c 148f 1490 1492 1496 149a 149d 149e
14a0 14a1 14a3 14a7 14ab 14af 14b2 14b3
14b5 14b8 14bb 14bc 14c1 14c5 14c9 14cc
14d0 14d4 14d7 14da 14df 14e2 14e5 14e8
14e9 14ee 14ef 1 14f1 14f6 14fa 14fe
1501 1506 150b 150e 1512 1513 1518 151b
1520 1521 1526 1529 152e 152f 1534 1537
153b 153f 1543 1547 154a 154b 154d 1552
1553 1555 1556 155b 155e 1563 1564 1569
156c 1571 1572 1577 157a 157e 157f 1584
1587 158c 158d 1592 1593 1598 159a 159e
15a1 15a5 15a9 15ac 15b0 15b4 15b7 15ba
15bf 15c2 15c5 15c8 15c9 15ce 15cf 1
15d1 15d6 15da 15de 15e1 15e6 15eb 15ee
15f2 15f3 15f8 15f9 15fe 1600 1604 1607
160b 160f 1612 1616 161a 161d 1621 1625
1626 162b 162d 1631 1635 1639 163d 1649
164d 1651 1655 1659 165d 1661 1665 1669
166d 1671 1675 1679 167d 1681 1685 1689
168d 1691 1695 1699 169d 16a1 16ad 16b1
16b6 16ba 16bd 16be 16c3 16c7 16cb 16ce
16d1 16d5 16d7 16db 16df 16e2 16e5 16e9
16eb 16ef 16f3 16f6 16f8 16fc 1700 1704
1707 170b 170f 1711 1712 1717 1 171b
171f 1723 1726 172a 172e 1732 1736 1739
173d 1741 1744 1747 174c 174f 1752 1755
1756 175b 175c 1 175e 1763 1767 176b
176e 1773 1778 177b 177f 1780 1785 1788
178d 178e 1793 1796 179b 179c 17a1 17a4
17a8 17a9 17ae 17b1 17b6 17b7 17bc 17bf
17c3 17c7 17ca 17cd 17ce 17d0 17d1 17d6
17d7 17dc 17de 17e2 17e5 17e7 17eb 17ed
17f9 17fb 17fd 17fe 1803 1807 1809 1815
1817 1819 181d 1821 1824 1828 182c 1830
1831 1833 1837 183b 183f 1842 1846 184a
184d 1850 1855 1858 185b 185e 185f 1864
1865 1 1867 186c 1870 1874 1877 187c
1881 1884 1888 1889 188e 1891 1896 1897
189c 189f 18a4 18a5 18aa 18ad 18b1 18b5
18b9 18ba 18bc 18c1 18c2 18c4 18c5 18ca
18cd 18d2 18d3 18d8 18db 18e0 18e1 18e6
18e9 18ed 18ee 18f3 18f6 18fb 18fc 1901
1902 1907 1909 190d 1910 1914 1918 191b
191f 1923 1926 1929 192e 1931 1934 1937
1938 193d 193e 1 1940 1945 1949 194d
1950 1955 195a 195d 1961 1962 1967 1968
196d 196f 1973 1976 197a 197e 1981 1985
1989 198c 198d 198f 1993 1997 1998 199d
19a1 19a5 19a8 19ac 19b0 19b3 19b6 19bb
19be 19c1 19c4 19c5 19ca 19cb 1 19cd
19d2 19d6 19da 19dd 19e2 19e7 19ea 19ee
19ef 19f4 19f7 19fc 19fd 1a02 1a05 1a0a
1a0b 1a10 1a13 1a17 1a1b 1a1f 1a23 1a26
1a27 1a29 1a2e 1a2f 1a31 1a32 1a37 1a3a
1a3f 1a40 1a45 1a48 1a4d 1a4e 1a53 1a56
1a5a 1a5b 1a60 1a63 1a68 1a69 1a6e 1a6f
1a74 1a76 1a7a 1a7d 1a81 1a85 1a88 1a8c
1a90 1a93 1a97 1a9b 1a9c 1aa1 1aa3 1aa7
1aab 1aae 1ab2 1ab6 1ab9 1abd 1ac1 1ac4
1ac7 1acc 1acf 1ad2 1ad5 1ad6 1adb 1adc
1 1ade 1ae3 1ae7 1aeb 1aee 1af3 1af8
1afb 1aff 1b00 1b05 1b06 1b0b 1b0f 1b13
1b16 1b1b 1b20 1b23 1b27 1b2b 1b2e 1b2f
1b34 1b37 1b3c 1b3d 1b42 1b45 1b49 1b4d
1b50 1b51 1b56 1b57 1b5c 1b5e 1b62 1b65
1b69 1b6d 1b71 1b75 1b79 1b7d 1b81 1b85
1b89 1b95 1b9a 1b9e 1ba2 1ba6 1baa 1bac
1bad 1bb2 1 1bb6 1bba 1bbe 1bc1 1bc5
1bc9 1bcd 1bd1 1bd4 1bd8 1bdc 1bdf 1be2
1be7 1bea 1bed 1bf0 1bf1 1bf6 1bf7 1
1bf9 1bfe 1c02 1c06 1c09 1c0e 1c13 1c16
1c1a 1c1b 1c20 1c21 1c26 1c2a 1c2e 1c31
1c36 1c3b 1c3e 1c42 1c43 1c48 1c4b 1c50
1c51 1c56 1c59 1c5d 1c61 1c64 1c67 1c68
1c6a 1c6b 1c70 1c71 1c76 1c78 1c7c 1c7f
1c83 1c87 1c8b 1c8f 1c93 1c97 1c9b 1ca7
1cac 1cb0 1cb4 1cb8 1cbc 1cbe 1cc2 1cc4
1cd0 1cd2 1cd4 1cd5 1cda 1cde 1ce0 1cec
1cf0 1cf2 1cf3 1cfc 
7f4
2
0 1 a 6 10 :3 6 16 :3 6
17 :3 6 10 :2 6 1 3 a 13
a :2 1 4 18 21 20 :2 18 :2 4
14 1d 1c :2 14 :2 4 11 1a 19
:2 11 :2 4 :3 13 :2 4 :3 11 :2 4 :3 12 :2 4
:3 12 :2 4 :3 13 :2 4 :3 9 :2 4 :3 9 :2 4
f 18 17 :2 f :2 4 f 18 17
:2 f :2 4 :2 f 1e f :2 4 :2 16 25
16 :2 4 :2 11 20 11 :2 4 :3 10 :2 4
:3 15 :2 4 :2 12 21 12 :2 4 :2 d 1c
d :2 4 :2 11 20 11 :2 4 :2 13 22
13 :2 4 e 17 16 :2 e :2 4 d
16 15 :2 d :2 4 f 18 17 :2 f
:2 4 :2 d 1c d :2 4 :2 e 1d e
:2 4 :2 c 1b c :2 4 :2 14 23 14
:2 4 :2 13 22 13 :2 4 :2 12 21 12
:2 4 :2 11 20 11 :2 4 :2 b 1a b
:2 4 :2 b 1a b :2 4 :2 f 1e f
:2 4 :2 e 1d e :2 4 :2 e 1d e
:2 4 :2 10 1f 10 :2 4 :2 13 22 13
:2 4 :2 f 1e f :2 4 :2 e 1d e
:2 4 :2 f 1e f :2 4 :2 c 1b c
:2 4 :2 b 1a b :2 4 :2 13 22 13
:2 4 :2 14 23 14 :2 4 :2 f 1e f
:2 4 :2 f 19 f :2 4 :2 f 19 f
:2 4 :2 10 1a 10 :2 4 :2 10 1a 10
:2 4 :2 11 1b 11 :2 4 :2 11 1b 11
:2 4 :2 12 1c 12 :2 4 :2 12 1c 12
:2 4 e 17 :3 e 4 7 :2 11 23
:2 2d 3a :2 23 :3 7 :2 d 19 25 :2 2f
43 :2 25 55 5b 5e :2 55 6c 83
8 :2 6c d 10 :2 6c 1a 1d :2 6c
b e :2 6c 13 16 1f 26 2b
2e :2 26 3e 41 :2 26 46 :2 1f 4f
:2 16 :2 6c b e :2 6c 13 16 1f
26 2b 2e :2 26 3f 42 :2 26 47
:2 1f 50 :2 16 :2 6c b e :2 6c 13
16 :2 6c 1b 1e :2 6c 28 2b :2 6c
30 33 :2 6c :2 7 4b :2 4 7 :2 11
23 :2 2d 39 3c 4d 4f 50 :2 4d
:2 23 :3 7 :2 d 12 1e 5e 61 :2 1e
:2 7 53 :3 4 1b :2 4 12 4 e
25 31 7 16 20 30 3b 45
4d 7 12 20 2d 3f 11 25
37 47 59 7 f 16 31 39
40 5c 66 7 15 23 31 7
17 27 37 7 1a c 1d 2b
7 13 1f 31 3c 44 4d 7
18 28 37 45 7 11 1a 7
d 13 1f 7 13 20 2c 7
15 24 32 7 17 c d 21
:2 7 :2 10 1c 7 4 c a :2 13
1f a 1a :2 7 :2 c a :2 13 1f
a d :2 17 29 :2 33 3f 43 54
57 58 :2 54 :2 29 :3 d :2 13 18 24
65 68 :2 24 75 78 :2 24 7c 7f
:2 24 8b 8e :2 24 96 99 :2 24 a5
a8 af b8 bb :2 a8 :2 24 :2 d 5b
:2 a :2 7 :4 13 :2 7 4 :3 1 8 :2 11
1c 1a :2 25 :2 1a 7 a :2 14 26
:2 30 3c 40 51 54 55 :2 51 :2 26
:3 a :2 10 15 21 62 65 :2 21 72
75 :2 21 79 7c :2 21 86 89 92
9a :2 a3 :2 92 b6 :2 89 :2 21 be c1
:2 21 c9 cc :2 21 d1 d4 :2 21 e8
eb :2 21 :2 a 58 :2 7 a :2 14 26
:2 30 3c 3f 50 53 54 :2 50 :2 26
:3 a :2 10 15 21 62 65 :2 21 :2 a
57 :3 7 :2 12 26 :2 2f 41 f :2 7
35 :2 4 :4 8 7 :4 2a 29 :3 7 18
:2 7 19 7 b :2 16 1e 2d 3d
:2 b 41 43 :2 41 a 4b :2 56 d
1d 2c :2 4b 30 33 :2 30 4a :2 a
d :2 17 29 :2 33 3f 43 54 57
58 :2 54 :2 29 :3 d :2 13 18 24 65
68 :2 24 75 78 :2 24 7c 7f :2 24
89 8c 95 9d :2 a6 :2 95 c3 :2 8c
:2 24 cb ce :2 24 d6 d9 :2 24 de
e1 :2 24 f5 f8 :2 24 :2 d 5b :2 a
d :2 17 29 :2 33 3f 42 53 56
57 :2 53 :2 29 :3 d :2 13 18 24 65
68 :2 24 :2 d 5a :3 a :2 15 29 :2 32
4e 12 :2 a 36 :2 7 11 1d f
1d f 1d 2a 3b 48 :2 a :2 13
1f a 7 f d :2 16 22 d
1d :2 a :2 f d :2 16 22 d 10
:2 1a 2c :2 36 42 46 57 5a 5b
:2 57 :2 2c :3 10 :2 16 1b 27 68 6b
:2 27 78 7b :2 27 7f 82 :2 27 8e
91 :2 27 99 9c :2 27 a8 ab b2
bb be :2 ab :2 27 :2 10 5e :2 d :2 a
:4 16 :2 a 7 :3 48 a :2 15 1d 2c
3a :2 a 3e 41 :2 3e a 1a :2 a
15 a 43 d :2 17 29 :2 33 3f
42 53 56 57 :2 53 :2 29 :3 d :2 13
18 24 65 68 :2 24 75 78 :2 24
7c 7f :2 24 89 8c 95 9d :2 a6
:2 95 b7 :2 8c :2 24 :2 d 5a :2 a d
:2 17 29 :2 33 3f 42 53 56 57
:2 53 :2 29 :3 d :2 13 18 24 65 68
:2 24 :2 d 5a :3 a :2 15 2b :2 34 44
:2 a :4 7 48 :3 4 14 15 :2 14 :2 4
17 2e 3c 47 5c 62 2d 39
45 53 2d :2 17 4 8 11 1d
:2 8 20 22 :2 20 7 :4 2b 2a 29
:3 7 15 2a 38 43 4d 26 3b
4d 59 :2 15 7 4a :2 4 :4 8 :2 7
1a 2a 38 4d :2 1a 7 25 a
11 1d 20 :2 a 23 25 :2 23 a
1d 2d 3b 50 :2 1d a 29 a
1d 2d 3b 50 :2 1d a :4 7 :4 4
:4 8 7 11 1c 29 31 38 53
5e 63 :2 f :2 a :2 13 1f a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 57 5a 5b :2 57 :2 2c :3 10
:2 16 1b 27 68 6b :2 27 78 7b
:2 27 7f 82 :2 27 8e 91 :2 27 99
9c :2 27 a8 ab b2 bb be :2 ab
:2 27 :2 10 5e :2 d :2 a :4 16 :2 a 7
:3 26 7 13 1c 26 :2 13 29 2b
34 43 :2 2b :2 13 7 26 :3 4 12
:2 4 17 25 3a 48 52 5e 24
30 39 42 :2 30 46 4f 60 :2 46
:2 17 4 7 10 21 :2 7 24 27
:2 24 a :2 14 26 :2 30 3c 40 51
54 55 :2 51 :2 26 :3 a :2 10 15 21
62 65 :2 21 72 75 :2 21 79 7c
:2 21 86 89 92 9a :2 a3 :2 92 bb
:2 89 :2 21 c3 c6 :2 21 ce d1 :2 21
d6 d9 :2 21 ed f0 :2 21 :2 a 58
:2 7 a :2 14 26 :2 30 3c 3f 50
53 54 :2 50 :2 26 :3 a :2 10 15 21
62 65 :2 21 :2 a 57 :3 7 :2 12 26
:2 2f 46 f :2 7 29 e f 10
24 7 11 26 34 42 4b 52
5f 68 75 7e a 19 22 2e
3e 4e 57 64 a 1e 10 24
a 11 d 19 :3 17 d :2 16 22
d 1e d :2 16 22 d :4 a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 57 5a 5b :2 57 :2 2c :3 10
:2 16 1b 27 68 6b :2 27 78 7b
:2 27 7f 82 :2 27 8e 91 :2 27 99
9c :2 27 a8 ab b2 bb be :2 ab
:2 27 :2 10 5e :2 d :2 a :4 16 :2 a 7
:8 4 12 1c :2 12 4 7 :2 11 23
:2 2d 39 3d 4e 51 52 :2 4e :2 23
:3 7 :2 d 12 1e 5f 62 :2 1e 6f
72 :2 1e 76 79 :2 1e 83 86 8f
97 :2 8f a4 :2 86 :2 1e ac af :2 1e
b7 ba :2 1e bf c2 :2 1e d6 d9
:2 1e :2 7 55 :2 4 7 :2 11 23 :2 2d
39 3c 4d 50 51 :2 4d :2 23 :3 7
:2 d 12 1e 5f 62 :2 1e :2 7 54
:3 4 :2 f 23 2c 38 :2 23 3c c
:2 4 7 :2 11 23 :2 2d 39 3c 4d
50 51 :2 4d :2 23 :3 7 :2 d 12 1e
5f 62 :2 1e 6f 72 :2 1e 76 79
:2 1e 83 86 8f 97 :2 a0 :2 8f ab
:2 86 :2 1e b3 b6 :2 1e be c1 :2 1e
c6 c9 :2 1e dd e0 :2 1e :2 7 54
:3 4 :2 f 21 :2 2a 34 49 :2 4 1
9 12 9 a :2 14 26 :2 30 3c
40 51 54 55 :2 51 :2 26 :3 a :2 10
15 21 62 65 :2 21 :3 a :2 10 15
39 5c 5f :2 68 :2 39 76 79 :2 39
81 84 :2 8d :2 39 :2 a 58 :3 7 c
15 1e 2c a 13 1e f e
7 e 7 e 7 29 :2 4 :2 9
7 :2 10 1c :2 7 :2 11 23 :2 2d 39
3d 4e 51 52 :2 4e :2 23 :3 7 :2 d
12 1e 5f 62 :2 1e :3 7 :2 d 12
36 42 45 :2 36 4d 50 :2 36 5c
5f 66 6f 72 :2 5f :2 36 :2 7 55
:2 4 7 c 1b a 13 1e f
e 7 e 7 e 7 :2 4 :4 10
:2 4 :8 1 
7f4
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 2 :4 7
:2 1 :7 9 :7 a :7 b
:5 c :5 d :5 e :5 f
:5 10 :5 11 :5 12 :7 13
:7 14 :6 15 :6 16 :6 17
:5 18 :5 19 :6 1a :6 1b
:6 1c :6 1d :7 1e :7 1f
:7 20 :6 21 :6 22 :6 23
:6 24 :6 25 :6 26 :6 27
:6 28 :6 29 :6 2a :6 2b
:6 2c :6 2d :6 2e :6 2f
:6 30 :6 31 :6 32 :6 33
:6 34 :6 35 :6 36 :6 37
:6 38 :6 39 :6 3a :6 3b
:6 3c :6 3d :6 3e :7 40
:b 42 :11 43 44 :2 43
:2 44 :2 43 :2 44 :2 43
:2 45 :2 43 :12 45 :2 43
:2 46 :2 43 :12 46 :2 43
:2 47 :2 43 :2 47 :2 43
:2 47 :2 43 :2 47 :2 43
:2 47 :4 43 :3 42 :11 4a
:b 4b :3 4a :3 51 :3 52
:3 55 :7 56 :5 57 :5 58
:8 59 :4 5a :4 5b :2 5c
:3 5d :7 5e :5 5f :3 60
:4 61 :4 62 :4 63 :2 64
65 :2 66 55 :5 67
54 69 :5 6a :3 69
:2 6b :5 6d :11 6e :24 6f
:3 6e :2 6c :6 6b 68
:3 41 :a 75 :11 76 :30 77
:3 76 :11 79 :b 7a :3 79
:7 7c 7d :2 7c :3 75
:c 80 :3 81 :3 82 :10 84
:3 85 :2 84 :4 85 :3 84
:11 86 :30 87 :3 86 :11 89
:b 8a :3 89 :7 8c 8d
:2 8c 85 :2 84 :2 91
:2 92 :5 93 91 :5 94
90 96 :5 97 :3 96
:2 98 :5 9a :11 9b :24 9c
:3 9b :2 99 :6 98 95
:3 80 :c a3 :3 a4 :3 a5
a3 :11 a7 :20 a8 :3 a7
:11 aa :b ab :3 aa :9 ad
a6 :3 a3 :3 80 :6 b2
:7 b3 :4 b4 b5 :3 b3
:12 b7 :6 b8 :4 b9 :3 b8
:3 b7 :5 bd :8 be bd
:a c0 :8 c1 c0 :8 c3
c2 :3 c0 bf :3 bd
:5 c7 :8 c9 ca cb
c9 :5 cc c8 ce
:5 cf :3 ce :2 d0 :5 d2
:11 d3 :24 d4 :3 d3 :2 d1
:6 d0 cd :3 c7 :f d9
:3 c7 :3 de :7 df :b e0
:3 df :9 e3 :11 e4 :30 e5
:3 e4 :11 e7 :b e8 :3 e7
:7 ea eb :2 ea e3
ed ee :2 ef ed
:a f2 :8 f3 :2 f4 :2 f5
f2 :6 f6 :5 f7 f6
:5 f9 f8 :3 f6 f1
fc :5 fd :3 fc :2 fe
:5 100 :11 101 :24 102 :3 101
:2 ff :6 fe fb :4 ec
:3 e3 :6 108 :11 109 :2e 10a
:3 109 :11 10c :b 10d :3 10c
:9 10f 110 :2 10f :11 112
:30 113 :3 112 :a 115 41
:3 118 :11 119 :b 11a :17 11b
:3 119 :2 11d :3 11e :3 11f
120 11e 11d 11e
:3 121 :3 118 :2 122 :5 124
:11 125 :b 126 :18 127 :3 125
:2 129 12a :3 12b 12c
12a 129 12a :3 12d
:2 123 :6 122 117 :7 1

1cfe
4
:3 0 1 :3 0 2
:a 0 7ef 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 6
:3 0 5 :7 0 9
8 :3 0 6 :3 0
7 :7 0 d c
:3 0 15 16 0
b 4 :3 0 8
:7 0 11 10 :3 0
9 :3 0 a :3 0
b :2 0 4 13
17 0 7ef 2
18 :2 0 d :2 0
12 4 :3 0 d
:2 0 10 1b 1d
:6 0 20 1e 0
7ed 0 c :6 0
d :2 0 16 4
:3 0 14 22 24
:6 0 27 25 0
7ed 0 e :6 0
1c d9 0 1a
4 :3 0 18 29
2b :6 0 2e 2c
0 7ed 0 f
:6 0 20 10d 0
1e 6 :3 0 30
:7 0 33 31 0
7ed 0 10 :6 0
6 :3 0 35 :7 0
38 36 0 7ed
0 11 :6 0 24
141 0 22 6
:3 0 3a :7 0 3d
3b 0 7ed 0
12 :6 0 6 :3 0
3f :7 0 42 40
0 7ed 0 13
:6 0 28 175 0
26 6 :3 0 44
:7 0 47 45 0
7ed 0 14 :6 0
16 :3 0 49 :7 0
4c 4a 0 7ed
0 15 :6 0 19
:2 0 2c 16 :3 0
4e :7 0 51 4f
0 7ed 0 17
:6 0 4 :3 0 19
:2 0 2a 53 55
:6 0 58 56 0
7ed 0 18 :6 0
1d :2 0 30 4
:3 0 2e 5a 5c
:6 0 5f 5d 0
7ed 0 1a :6 0
1d :2 0 32 1c
:3 0 61 :7 0 65
62 63 7ed 0
1b :6 0 1d :2 0
34 1c :3 0 67
:7 0 6b 68 69
7ed 0 1e :6 0
38 21e 0 36
1c :3 0 6d :7 0
71 6e 6f 7ed
0 1f :6 0 1d
:2 0 3a 16 :3 0
73 :7 0 76 74
0 7ed 0 20
:6 0 16 :3 0 78
:7 0 7b 79 0
7ed 0 21 :6 0
1d :2 0 3c 1c
:3 0 7d :7 0 81
7e 7f 7ed 0
22 :6 0 1d :2 0
3e 1c :3 0 83
:7 0 87 84 85
7ed 0 23 :6 0
1d :2 0 40 1c
:3 0 89 :7 0 8d
8a 8b 7ed 0
24 :6 0 d :2 0
42 1c :3 0 8f
:7 0 93 90 91
7ed 0 25 :6 0
d :2 0 46 4
:3 0 44 95 97
:6 0 9a 98 0
7ed 0 26 :6 0
d :2 0 4a 4
:3 0 48 9c 9e
:6 0 a1 9f 0
7ed 0 27 :6 0
1d :2 0 4e 4
:3 0 4c a3 a5
:6 0 a8 a6 0
7ed 0 28 :6 0
1d :2 0 50 1c
:3 0 aa :7 0 ae
ab ac 7ed 0
29 :6 0 1d :2 0
52 1c :3 0 b0
:7 0 b4 b1 b2
7ed 0 2a :6 0
1d :2 0 54 1c
:3 0 b6 :7 0 ba
b7 b8 7ed 0
2b :6 0 1d :2 0
56 1c :3 0 bc
:7 0 c0 bd be
7ed 0 2c :6 0
1d :2 0 58 1c
:3 0 c2 :7 0 c6
c3 c4 7ed 0
2d :6 0 1d :2 0
5a 1c :3 0 c8
:7 0 cc c9 ca
7ed 0 2e :6 0
1d :2 0 5c 1c
:3 0 ce :7 0 d2
cf d0 7ed 0
2f :6 0 1d :2 0
5e 1c :3 0 d4
:7 0 d8 d5 d6
7ed 0 30 :6 0
1d :2 0 60 1c
:3 0 da :7 0 de
db dc 7ed 0
31 :6 0 1d :2 0
62 1c :3 0 e0
:7 0 e4 e1 e2
7ed 0 32 :6 0
1d :2 0 64 1c
:3 0 e6 :7 0 ea
e7 e8 7ed 0
33 :6 0 1d :2 0
66 1c :3 0 ec
:7 0 f0 ed ee
7ed 0 34 :6 0
1d :2 0 68 1c
:3 0 f2 :7 0 f6
f3 f4 7ed 0
35 :6 0 1d :2 0
6a 1c :3 0 f8
:7 0 fc f9 fa
7ed 0 36 :6 0
1d :2 0 6c 1c
:3 0 fe :7 0 102
ff 100 7ed 0
37 :6 0 1d :2 0
6e 1c :3 0 104
:7 0 108 105 106
7ed 0 38 :6 0
1d :2 0 70 1c
:3 0 10a :7 0 10e
10b 10c 7ed 0
39 :6 0 1d :2 0
72 1c :3 0 110
:7 0 114 111 112
7ed 0 3a :6 0
1d :2 0 74 1c
:3 0 116 :7 0 11a
117 118 7ed 0
3b :6 0 1d :2 0
76 1c :3 0 11c
:7 0 120 11d 11e
7ed 0 3c :6 0
1d :2 0 78 1c
:3 0 122 :7 0 126
123 124 7ed 0
3d :6 0 1d :2 0
7a 1c :3 0 128
:7 0 12c 129 12a
7ed 0 3e :6 0
1d :2 0 7c 40
:3 0 12e :7 0 132
12f 130 7ed 0
3f :6 0 1d :2 0
7e 40 :3 0 134
:7 0 138 135 136
7ed 0 41 :6 0
1d :2 0 80 40
:3 0 13a :7 0 13e
13b 13c 7ed 0
42 :6 0 1d :2 0
82 40 :3 0 140
:7 0 144 141 142
7ed 0 43 :6 0
1d :2 0 84 40
:3 0 146 :7 0 14a
147 148 7ed 0
44 :6 0 1d :2 0
86 40 :3 0 14c
:7 0 150 14d 14e
7ed 0 45 :6 0
1d :2 0 88 40
:3 0 152 :7 0 156
153 154 7ed 0
46 :6 0 15e 15f
0 8a 40 :3 0
158 :7 0 15c 159
15a 7ed 0 47
:6 0 164 165 0
8c a :3 0 b
:2 0 4 160 :7 0
163 161 0 7ed
0 48 :6 0 49
:3 0 4a :3 0 49
:3 0 4b :3 0 167
168 0 2 :4 0
8e 169 16b 166
16d 16c :2 0 4c
:3 0 4d :3 0 16f
170 0 4e :4 0
49 :3 0 4f :3 0
173 174 0 2
:4 0 90 175 177
50 :4 0 51 :2 0
52 :3 0 92 17a
17c :3 0 53 :4 0
51 :2 0 54 :4 0
95 17f 181 :3 0
51 :2 0 3 :3 0
98 183 185 :3 0
51 :2 0 54 :4 0
9b 187 189 :3 0
51 :2 0 55 :4 0
9e 18b 18d :3 0
51 :2 0 56 :3 0
57 :3 0 54 :4 0
51 :2 0 5 :3 0
a1 193 195 :3 0
51 :2 0 54 :4 0
a4 197 199 :3 0
58 :4 0 a7 191
19c 59 :4 0 aa
190 19f ad 18f
1a1 :3 0 51 :2 0
55 :4 0 b0 1a3
1a5 :3 0 51 :2 0
56 :3 0 57 :3 0
54 :4 0 51 :2 0
7 :3 0 b3 1ab
1ad :3 0 51 :2 0
54 :4 0 b6 1af
1b1 :3 0 58 :4 0
b9 1a9 1b4 59
:4 0 bc 1a8 1b7
bf 1a7 1b9 :3 0
51 :2 0 55 :4 0
c2 1bb 1bd :3 0
51 :2 0 54 :4 0
c5 1bf 1c1 :3 0
51 :2 0 8 :3 0
c8 1c3 1c5 :3 0
51 :2 0 54 :4 0
cb 1c7 1c9 :3 0
51 :2 0 5a :4 0
ce 1cb 1cd :3 0
d1 171 1cf :2 0
1d1 d6 1d2 16e
1d1 0 1d3 d8
0 74d 49 :3 0
4a :3 0 1d4 1d5
0 49 :3 0 5b
:3 0 1d7 1d8 0
19 :2 0 2 :4 0
5c :2 0 5d :2 0
19 :2 0 da 1dd
1df :3 0 dd 1d9
1e1 1d6 1e3 1e2
:2 0 4c :3 0 5e
:3 0 1e5 1e6 0
4e :4 0 5f :4 0
51 :2 0 52 :3 0
e1 1ea 1ec :3 0
e4 1e7 1ee :2 0
1f0 e7 1f1 1e4
1f0 0 1f2 e9
0 74d c :3 0
3 :3 0 1f3 1f4
0 74d 28 :3 0
8 :3 0 1f6 1f7
0 74d 60 :3 0
61 :3 0 62 :3 0
63 :3 0 64 :3 0
65 :3 0 66 :3 0
67 :3 0 68 :3 0
69 :3 0 6a :3 0
6b :3 0 6c :3 0
6d :3 0 6e :3 0
64 :3 0 6f :3 0
70 :3 0 71 :3 0
72 :3 0 73 :3 0
74 :3 0 6d :3 0
73 :3 0 74 :3 0
6e :3 0 75 :3 0
76 :3 0 77 :3 0
78 :3 0 79 :3 0
7a :3 0 7b :3 0
7c :3 0 7d :3 0
7e :3 0 7f :3 0
80 :3 0 e :3 0
f :3 0 27 :3 0
1a :3 0 18 :3 0
21 :3 0 2a :3 0
30 :3 0 3a :3 0
3b :3 0 2c :3 0
10 :3 0 2e :3 0
11 :3 0 12 :3 0
29 :3 0 2b :3 0
32 :3 0 15 :3 0
17 :3 0 1b :3 0
1e :3 0 3f :3 0
42 :3 0 41 :3 0
43 :3 0 44 :3 0
46 :3 0 45 :3 0
47 :3 0 2d :3 0
26 :3 0 81 :3 0
82 :3 0 c :4 0
83 1 :8 0 248
a :3 0 84 :3 0
243 244 0 85
:3 0 245 246 0
248 eb 29b 86
:3 0 a :3 0 84
:3 0 24a 24b 0
85 :3 0 24c 24d
0 24f ee 251
f0 250 24f :2 0
299 87 :3 0 a
:3 0 84 :3 0 254
255 0 85 :3 0
256 257 0 291
49 :3 0 4a :3 0
259 25a 0 49
:3 0 5b :3 0 25c
25d 0 88 :2 0
2 :4 0 89 :2 0
5d :2 0 19 :2 0
f2 262 264 :3 0
f5 25e 266 25b
268 267 :2 0 4c
:3 0 5e :3 0 26a
26b 0 4e :4 0
8a :4 0 51 :2 0
52 :3 0 f9 26f
271 :3 0 51 :2 0
8b :4 0 fc 273
275 :3 0 51 :2 0
8c :4 0 ff 277
279 :3 0 51 :2 0
85 :3 0 102 27b
27d :3 0 51 :2 0
8d :4 0 105 27f
281 :3 0 51 :2 0
74 :3 0 8e :3 0
19 :2 0 8f :2 0
108 284 288 10c
283 28a :3 0 10f
26c 28c :2 0 28e
112 28f 269 28e
0 290 114 0
291 116 294 :3 0
294 0 294 293
291 292 :6 0 296
2 :3 0 119 298
11b 297 296 :2 0
299 11d :2 0 29b
0 29b 29a 248
299 :6 0 74d 1
:3 0 a :3 0 84
:3 0 29d 29e 0
a :3 0 90 :2 0
91 :3 0 2a0 2a2
0 122 2a1 2a4
:3 0 2a5 :2 0 49
:3 0 4a :3 0 2a7
2a8 0 49 :3 0
5b :3 0 2aa 2ab
0 d :2 0 2
:4 0 92 :2 0 5d
:2 0 19 :2 0 125
2b0 2b2 :3 0 128
2ac 2b4 2a9 2b6
2b5 :2 0 4c :3 0
5e :3 0 2b8 2b9
0 4e :4 0 93
:4 0 51 :2 0 52
:3 0 12c 2bd 2bf
:3 0 51 :2 0 8b
:4 0 12f 2c1 2c3
:3 0 51 :2 0 94
:4 0 132 2c5 2c7
:3 0 51 :2 0 56
:3 0 95 :3 0 96
:3 0 97 :3 0 2cc
2cd 0 135 2cb
2cf 59 :4 0 137
2ca 2d2 13a 2c9
2d4 :3 0 51 :2 0
98 :4 0 13d 2d6
2d8 :3 0 51 :2 0
54 :4 0 140 2da
2dc :3 0 51 :2 0
c :3 0 143 2de
2e0 :3 0 51 :2 0
54 :4 0 146 2e2
2e4 :3 0 149 2ba
2e6 :2 0 2e8 14c
2e9 2b7 2e8 0
2ea 14e 0 314
49 :3 0 4a :3 0
2eb 2ec 0 49
:3 0 5b :3 0 2ee
2ef 0 19 :2 0
2 :4 0 99 :2 0
5d :2 0 19 :2 0
150 2f4 2f6 :3 0
153 2f0 2f8 2ed
2fa 2f9 :2 0 4c
:3 0 5e :3 0 2fc
2fd 0 4e :4 0
9a :4 0 51 :2 0
52 :3 0 157 301
303 :3 0 15a 2fe
305 :2 0 307 15d
308 2fb 307 0
309 15f 0 314
9b :3 0 9c :3 0
30a 30b 0 96
:3 0 97 :3 0 30d
30e 0 c :3 0
9d :3 0 161 30c
312 :2 0 314 165
315 2a6 314 0
316 169 0 74d
5 :3 0 9e :2 0
16b 318 319 :3 0
31a :2 0 7 :3 0
9e :2 0 16d 31d
31e :3 0 31f :2 0
31b 321 320 :2 0
13 :3 0 5 :3 0
323 324 0 48c
14 :3 0 7 :3 0
326 327 0 48c
9f :3 0 a0 :3 0
329 32a 0 13
:3 0 14 :3 0 a1
:2 0 16f 32b 32f
a2 :2 0 1d :2 0
175 331 333 :3 0
334 :2 0 9f :3 0
a0 :3 0 336 337
0 14 :3 0 12
:3 0 a1 :2 0 178
338 33c a3 :2 0
1d :2 0 17e 33e
340 :3 0 341 :2 0
335 343 342 :2 0
49 :3 0 4a :3 0
345 346 0 49
:3 0 5b :3 0 348
349 0 d :2 0
2 :4 0 a4 :2 0
5d :2 0 19 :2 0
181 34e 350 :3 0
184 34a 352 347
354 353 :2 0 4c
:3 0 5e :3 0 356
357 0 4e :4 0
a5 :4 0 51 :2 0
52 :3 0 188 35b
35d :3 0 51 :2 0
8b :4 0 18b 35f
361 :3 0 51 :2 0
94 :4 0 18e 363
365 :3 0 51 :2 0
56 :3 0 95 :3 0
96 :3 0 a6 :3 0
36a 36b 0 191
369 36d 59 :4 0
193 368 370 196
367 372 :3 0 51
:2 0 98 :4 0 199
374 376 :3 0 51
:2 0 54 :4 0 19c
378 37a :3 0 51
:2 0 c :3 0 19f
37c 37e :3 0 51
:2 0 54 :4 0 1a2
380 382 :3 0 1a5
358 384 :2 0 386
1a8 387 355 386
0 388 1aa 0
3b2 49 :3 0 4a
:3 0 389 38a 0
49 :3 0 5b :3 0
38c 38d 0 19
:2 0 2 :4 0 a7
:2 0 5d :2 0 19
:2 0 1ac 392 394
:3 0 1af 38e 396
38b 398 397 :2 0
4c :3 0 5e :3 0
39a 39b 0 4e
:4 0 a8 :4 0 51
:2 0 52 :3 0 1b3
39f 3a1 :3 0 1b6
39c 3a3 :2 0 3a5
1b9 3a6 399 3a5
0 3a7 1bb 0
3b2 9b :3 0 9c
:3 0 3a8 3a9 0
96 :3 0 a6 :3 0
3ab 3ac 0 c
:3 0 9d :3 0 1bd
3aa 3b0 :2 0 3b2
1c1 3b3 344 3b2
0 3b4 1c5 0
48c 61 :3 0 a9
:3 0 f :3 0 34
:3 0 aa :3 0 61
:3 0 f :3 0 ab
:3 0 26 :4 0 ac
1 :8 0 3c4 a
:3 0 84 :3 0 3bf
3c0 0 85 :3 0
3c1 3c2 0 3c4
1c7 417 86 :3 0
a :3 0 84 :3 0
3c6 3c7 0 85
:3 0 3c8 3c9 0
3cb 1ca 3cd 1cc
3cc 3cb :2 0 415
87 :3 0 a :3 0
84 :3 0 3d0 3d1
0 85 :3 0 3d2
3d3 0 40d 49
:3 0 4a :3 0 3d5
3d6 0 49 :3 0
5b :3 0 3d8 3d9
0 88 :2 0 2
:4 0 ad :2 0 5d
:2 0 19 :2 0 1ce
3de 3e0 :3 0 1d1
3da 3e2 3d7 3e4
3e3 :2 0 4c :3 0
5e :3 0 3e6 3e7
0 4e :4 0 ae
:4 0 51 :2 0 52
:3 0 1d5 3eb 3ed
:3 0 51 :2 0 8b
:4 0 1d8 3ef 3f1
:3 0 51 :2 0 8c
:4 0 1db 3f3 3f5
:3 0 51 :2 0 85
:3 0 1de 3f7 3f9
:3 0 51 :2 0 8d
:4 0 1e1 3fb 3fd
:3 0 51 :2 0 74
:3 0 8e :3 0 19
:2 0 8f :2 0 1e4
400 404 1e8 3ff
406 :3 0 1eb 3e8
408 :2 0 40a 1ee
40b 3e5 40a 0
40c 1f0 0 40d
1f2 410 :3 0 410
0 410 40f 40d
40e :6 0 412 4
:3 0 1f5 414 1f7
413 412 :2 0 415
1f9 :2 0 417 0
417 416 3c4 415
:6 0 48c 1 :3 0
9f :3 0 a0 :3 0
419 41a 0 13
:3 0 11 :3 0 a1
:2 0 1fc 41b 41f
a3 :2 0 1d :2 0
202 421 423 :3 0
11 :3 0 13 :3 0
425 426 0 42b
2b :3 0 1d :2 0
428 429 0 42b
205 489 49 :3 0
4a :3 0 42c 42d
0 49 :3 0 5b
:3 0 42f 430 0
af :2 0 2 :4 0
b0 :2 0 5d :2 0
19 :2 0 208 435
437 :3 0 20b 431
439 42e 43b 43a
:2 0 4c :3 0 5e
:3 0 43d 43e 0
4e :4 0 b1 :4 0
51 :2 0 52 :3 0
20f 442 444 :3 0
51 :2 0 8b :4 0
212 446 448 :3 0
51 :2 0 94 :4 0
215 44a 44c :3 0
51 :2 0 56 :3 0
95 :3 0 96 :3 0
b2 :3 0 451 452
0 218 450 454
59 :4 0 21a 44f
457 21d 44e 459
:3 0 220 43f 45b
:2 0 45d 223 45e
43c 45d 0 45f
225 0 488 49
:3 0 4a :3 0 460
461 0 49 :3 0
5b :3 0 463 464
0 19 :2 0 2
:4 0 b3 :2 0 5d
:2 0 19 :2 0 227
469 46b :3 0 22a
465 46d 462 46f
46e :2 0 4c :3 0
5e :3 0 471 472
0 4e :4 0 b4
:4 0 51 :2 0 52
:3 0 22e 476 478
:3 0 231 473 47a
:2 0 47c 234 47d
470 47c 0 47e
236 0 488 9b
:3 0 b5 :3 0 47f
480 0 96 :3 0
b2 :3 0 482 483
0 9d :3 0 238
481 486 :2 0 488
23b 48a 424 42b
0 48b 0 488
0 48b 23f 0
48c 242 48d 322
48c 0 48e 248
0 74d 1f :3 0
b6 :2 0 19 :2 0
24a 490 492 :3 0
48f 493 0 74d
3d :3 0 b7 :3 0
f :3 0 26 :3 0
c :3 0 15 :3 0
17 :3 0 3f :3 0
41 :3 0 1f :3 0
1e :3 0 1b :3 0
24c 496 4a1 495
4a2 0 74d 56
:3 0 32 :3 0 1d
:2 0 257 4a4 4a7
a2 :2 0 1d :2 0
25c 4a9 4ab :3 0
4ac :2 0 21 :3 0
9e :2 0 25f 4af
4b0 :3 0 4b1 :2 0
4b2 :2 0 4ad 4b4
4b3 :2 0 3e :3 0
b8 :3 0 f :3 0
26 :3 0 27 :3 0
e :3 0 c :3 0
21 :3 0 32 :3 0
1d :2 0 261 4b7
4c0 4b6 4c1 0
4c3 26a 4c4 4b5
4c3 0 4c5 26c
0 74d e :3 0
9e :2 0 26e 4c7
4c8 :3 0 4c9 :2 0
3d :3 0 b9 :3 0
f :3 0 c :3 0
ba :2 0 270 4cc
4d0 4cb 4d1 0
4d3 274 4f4 74
:3 0 18 :3 0 19
:2 0 19 :2 0 276
4d4 4d8 90 :2 0
ba :4 0 27c 4da
4dc :3 0 3d :3 0
b9 :3 0 f :3 0
c :3 0 1d :2 0
27f 4df 4e3 4de
4e4 0 4e6 283
4f0 3d :3 0 b9
:3 0 f :3 0 c
:3 0 19 :2 0 285
4e8 4ec 4e7 4ed
0 4ef 289 4f1
4dd 4e6 0 4f2
0 4ef 0 4f2
28b 0 4f3 28e
4f5 4ca 4d3 0
4f6 0 4f3 0
4f6 290 0 74d
21 :3 0 9e :2 0
293 4f8 4f9 :3 0
4fa :2 0 9f :3 0
bb :3 0 73 :3 0
74 :3 0 21 :3 0
9f :3 0 bc :3 0
11 :3 0 11 :3 0
bd :4 0 be 1
:8 0 50c a :3 0
84 :3 0 507 508
0 85 :3 0 509
50a 0 50c 295
55f 86 :3 0 a
:3 0 84 :3 0 50e
50f 0 85 :3 0
510 511 0 513
298 515 29a 514
513 :2 0 55d 87
:3 0 a :3 0 84
:3 0 518 519 0
85 :3 0 51a 51b
0 555 49 :3 0
4a :3 0 51d 51e
0 49 :3 0 5b
:3 0 520 521 0
88 :2 0 2 :4 0
bf :2 0 5d :2 0
19 :2 0 29c 526
528 :3 0 29f 522
52a 51f 52c 52b
:2 0 4c :3 0 5e
:3 0 52e 52f 0
4e :4 0 c0 :4 0
51 :2 0 52 :3 0
2a3 533 535 :3 0
51 :2 0 8b :4 0
2a6 537 539 :3 0
51 :2 0 8c :4 0
2a9 53b 53d :3 0
51 :2 0 85 :3 0
2ac 53f 541 :3 0
51 :2 0 8d :4 0
2af 543 545 :3 0
51 :2 0 74 :3 0
8e :3 0 19 :2 0
8f :2 0 2b2 548
54c 2b6 547 54e
:3 0 2b9 530 550
:2 0 552 2bc 553
52d 552 0 554
2be 0 555 2c0
558 :3 0 558 0
558 557 555 556
:6 0 55a 6 :3 0
2c3 55c 2c5 55b
55a :2 0 55d 2c7
:2 0 55f 0 55f
55e 50c 55d :6 0
570 1 :3 0 29
:3 0 56 :3 0 29
:3 0 1d :2 0 2ca
562 565 b6 :2 0
56 :3 0 2e :3 0
1d :2 0 2cd 568
56b 2d0 567 56d
:3 0 561 56e 0
570 2d3 571 4fb
570 0 572 2d6
0 74d 3e :3 0
ba :2 0 573 574
0 74d 3d :3 0
c1 :3 0 c :3 0
11 :3 0 29 :3 0
3e :3 0 2a :3 0
28 :3 0 56 :3 0
2b :3 0 1d :2 0
2d8 57e 581 56
:3 0 2c :3 0 1d
:2 0 2db 583 586
2de 577 588 576
589 0 74d 56
:3 0 3d :3 0 1d
:2 0 2e7 58b 58e
a3 :2 0 1d :2 0
2ec 590 592 :3 0
49 :3 0 4a :3 0
594 595 0 49
:3 0 5b :3 0 597
598 0 d :2 0
2 :4 0 c2 :2 0
5d :2 0 19 :2 0
2ef 59d 59f :3 0
2f2 599 5a1 596
5a3 5a2 :2 0 4c
:3 0 5e :3 0 5a5
5a6 0 4e :4 0
c3 :4 0 51 :2 0
52 :3 0 2f6 5aa
5ac :3 0 51 :2 0
8b :4 0 2f9 5ae
5b0 :3 0 51 :2 0
94 :4 0 2fc 5b2
5b4 :3 0 51 :2 0
56 :3 0 95 :3 0
96 :3 0 c4 :3 0
5b9 5ba 0 2ff
5b8 5bc 59 :4 0
301 5b7 5bf 304
5b6 5c1 :3 0 51
:2 0 98 :4 0 307
5c3 5c5 :3 0 51
:2 0 54 :4 0 30a
5c7 5c9 :3 0 51
:2 0 c :3 0 30d
5cb 5cd :3 0 51
:2 0 54 :4 0 310
5cf 5d1 :3 0 313
5a7 5d3 :2 0 5d5
316 5d6 5a4 5d5
0 5d7 318 0
601 49 :3 0 4a
:3 0 5d8 5d9 0
49 :3 0 5b :3 0
5db 5dc 0 19
:2 0 2 :4 0 c5
:2 0 5d :2 0 19
:2 0 31a 5e1 5e3
:3 0 31d 5dd 5e5
5da 5e7 5e6 :2 0
4c :3 0 5e :3 0
5e9 5ea 0 4e
:4 0 c6 :4 0 51
:2 0 52 :3 0 321
5ee 5f0 :3 0 324
5eb 5f2 :2 0 5f4
327 5f5 5e8 5f4
0 5f6 329 0
601 9b :3 0 9c
:3 0 5f7 5f8 0
96 :3 0 c4 :3 0
5fa 5fb 0 c
:3 0 9d :3 0 32b
5f9 5ff :2 0 601
32f 689 81 :3 0
c7 :3 0 82 :3 0
c :4 0 c8 1
:8 0 688 81 :3 0
c9 :3 0 ca :3 0
cb :3 0 cc :3 0
ca :3 0 96 :3 0
cd :3 0 96 :3 0
ce :3 0 cf :3 0
56 :3 0 d0 :3 0
d1 :3 0 d2 :3 0
56 :3 0 d3 :3 0
d4 :3 0 c7 :3 0
d5 :3 0 82 :3 0
c :4 0 d6 1
:8 0 633 d7 :4 0
61e :3 0 9d :3 0
90 :2 0 335 621
622 :3 0 a :3 0
84 :3 0 624 625
0 1d :2 0 626
627 0 629 338
630 a :3 0 84
:3 0 62a 62b 0
d8 :2 0 62c 62d
0 62f 33a 631
623 629 0 632
0 62f 0 632
33c 0 633 33f
686 86 :3 0 a
:3 0 84 :3 0 635
636 0 85 :3 0
637 638 0 63a
342 63c 344 63b
63a :2 0 684 87
:3 0 a :3 0 84
:3 0 63f 640 0
85 :3 0 641 642
0 67c 49 :3 0
4a :3 0 644 645
0 49 :3 0 5b
:3 0 647 648 0
88 :2 0 2 :4 0
d9 :2 0 5d :2 0
19 :2 0 346 64d
64f :3 0 349 649
651 646 653 652
:2 0 4c :3 0 5e
:3 0 655 656 0
4e :4 0 da :4 0
51 :2 0 52 :3 0
34d 65a 65c :3 0
51 :2 0 8b :4 0
350 65e 660 :3 0
51 :2 0 8c :4 0
353 662 664 :3 0
51 :2 0 85 :3 0
356 666 668 :3 0
51 :2 0 8d :4 0
359 66a 66c :3 0
51 :2 0 74 :3 0
8e :3 0 19 :2 0
8f :2 0 35c 66f
673 360 66e 675
:3 0 363 657 677
:2 0 679 366 67a
654 679 0 67b
368 0 67c 36a
67f :3 0 67f 0
67f 67e 67c 67d
:6 0 681 8 :3 0
36d 683 36f 682
681 :2 0 684 371
:2 0 686 0 686
685 633 684 :6 0
688 1 :3 0 374
68a 593 601 0
68b 0 688 0
68b 377 0 74d
3e :3 0 db :3 0
f :3 0 37a 68d
68f 68c 690 0
74d 49 :3 0 4a
:3 0 692 693 0
49 :3 0 5b :3 0
695 696 0 d
:2 0 2 :4 0 dc
:2 0 5d :2 0 19
:2 0 37c 69b 69d
:3 0 37f 697 69f
694 6a1 6a0 :2 0
4c :3 0 5e :3 0
6a3 6a4 0 4e
:4 0 dd :4 0 51
:2 0 52 :3 0 383
6a8 6aa :3 0 51
:2 0 8b :4 0 386
6ac 6ae :3 0 51
:2 0 94 :4 0 389
6b0 6b2 :3 0 51
:2 0 56 :3 0 95
:3 0 3e :3 0 38c
6b6 6b8 59 :4 0
38e 6b5 6bb 391
6b4 6bd :3 0 51
:2 0 98 :4 0 394
6bf 6c1 :3 0 51
:2 0 54 :4 0 397
6c3 6c5 :3 0 51
:2 0 c :3 0 39a
6c7 6c9 :3 0 51
:2 0 54 :4 0 39d
6cb 6cd :3 0 3a0
6a5 6cf :2 0 6d1
3a3 6d2 6a2 6d1
0 6d3 3a5 0
74d 49 :3 0 4a
:3 0 6d4 6d5 0
49 :3 0 5b :3 0
6d7 6d8 0 19
:2 0 2 :4 0 de
:2 0 5d :2 0 19
:2 0 3a7 6dd 6df
:3 0 3aa 6d9 6e1
6d6 6e3 6e2 :2 0
4c :3 0 5e :3 0
6e5 6e6 0 4e
:4 0 df :4 0 51
:2 0 52 :3 0 3ae
6ea 6ec :3 0 3b1
6e7 6ee :2 0 6f0
3b4 6f1 6e4 6f0
0 6f2 3b6 0
74d 9b :3 0 9c
:3 0 6f3 6f4 0
56 :3 0 3e :3 0
1d :2 0 3b8 6f6
6f9 c :3 0 9d
:3 0 3bb 6f5 6fd
:2 0 74d 49 :3 0
4a :3 0 6ff 700
0 49 :3 0 5b
:3 0 702 703 0
e0 :2 0 2 :4 0
e1 :2 0 5d :2 0
19 :2 0 3bf 708
70a :3 0 3c2 704
70c 701 70e 70d
:2 0 4c :3 0 5e
:3 0 710 711 0
4e :4 0 e2 :4 0
51 :2 0 52 :3 0
3c6 715 717 :3 0
51 :2 0 8b :4 0
3c9 719 71b :3 0
51 :2 0 94 :4 0
3cc 71d 71f :3 0
51 :2 0 56 :3 0
95 :3 0 a :3 0
84 :3 0 724 725
0 3cf 723 727
59 :4 0 3d1 722
72a 3d4 721 72c
:3 0 51 :2 0 98
:4 0 3d7 72e 730
:3 0 51 :2 0 54
:4 0 3da 732 734
:3 0 51 :2 0 c
:3 0 3dd 736 738
:3 0 51 :2 0 54
:4 0 3e0 73a 73c
:3 0 3e3 712 73e
:2 0 740 3e6 741
70f 740 0 742
3e8 0 74d 9b
:3 0 e3 :3 0 743
744 0 a :3 0
84 :3 0 746 747
0 c :3 0 9d
:3 0 3ea 745 74b
:2 0 74d 3ee 7ee
a :3 0 e4 :2 0
4 74e 74f 0
49 :3 0 4a :3 0
751 752 0 49
:3 0 5b :3 0 754
755 0 88 :2 0
2 :4 0 e5 :2 0
5d :2 0 19 :2 0
404 75a 75c :3 0
407 756 75e 753
760 75f :2 0 4c
:3 0 5e :3 0 762
763 0 4e :4 0
e6 :4 0 51 :2 0
52 :3 0 40b 767
769 :3 0 40e 764
76b :2 0 784 4c
:3 0 5e :3 0 76d
76e 0 4e :4 0
e7 :4 0 51 :2 0
a :3 0 e8 :3 0
773 774 0 411
772 776 :3 0 51
:2 0 98 :4 0 414
778 77a :3 0 51
:2 0 a :3 0 e9
:3 0 77d 77e 0
417 77c 780 :3 0
41a 76f 782 :2 0
784 41d 785 761
784 0 786 420
0 796 ea :3 0
48 :3 0 a :3 0
e8 :3 0 eb :3 0
a :3 0 e9 :3 0
ec :3 0 bd :4 0
ed 1 :a 0 792
:2 0 796 788 790
:2 0 9 :3 0 48
:3 0 794 :2 0 796
422 798 426 797
796 :2 0 7ec 87
:3 0 a :3 0 84
:3 0 79b 79c 0
85 :3 0 79d 79e
0 7e4 49 :3 0
4a :3 0 7a0 7a1
0 49 :3 0 5b
:3 0 7a3 7a4 0
88 :2 0 2 :4 0
ee :2 0 5d :2 0
19 :2 0 428 7a9
7ab :3 0 42b 7a5
7ad 7a2 7af 7ae
:2 0 4c :3 0 5e
:3 0 7b1 7b2 0
4e :4 0 ef :4 0
51 :2 0 52 :3 0
42f 7b6 7b8 :3 0
432 7b3 7ba :2 0
7d4 4c :3 0 5e
:3 0 7bc 7bd 0
4e :4 0 8c :4 0
51 :2 0 85 :3 0
435 7c1 7c3 :3 0
51 :2 0 8d :4 0
438 7c5 7c7 :3 0
51 :2 0 74 :3 0
8e :3 0 19 :2 0
8f :2 0 43b 7ca
7ce 43f 7c9 7d0
:3 0 442 7be 7d2
:2 0 7d4 445 7d5
7b0 7d4 0 7d6
448 0 7e4 ea
:3 0 48 :3 0 eb
:3 0 a :3 0 e9
:3 0 ec :3 0 bd
:4 0 f0 1 :a 0
7e0 :2 0 7e4 7d8
7de :2 0 9 :3 0
48 :3 0 7e2 :2 0
7e4 44a 7e7 :3 0
7e7 0 7e7 7e6
7e4 7e5 :6 0 7e9
1 :3 0 44f 7eb
451 7ea 7e9 :2 0
7ec 453 :2 0 7ee
456 7ee 7ed 74d
7ec :6 0 7ef :2 0
2 18 7ee 7f2
:3 0 7f1 7ef 7f3
:8 0 
48e
4
:3 0 1 3 1
7 1 b 1
f 4 6 a
e 12 1 1c
1 1a 1 23
1 21 1 2a
1 28 1 2f
1 34 1 39
1 3e 1 43
1 48 1 4d
1 54 1 52
1 5b 1 59
1 60 1 66
1 6c 1 72
1 77 1 7c
1 82 1 88
1 8e 1 96
1 94 1 9d
1 9b 1 a4
1 a2 1 a9
1 af 1 b5
1 bb 1 c1
1 c7 1 cd
1 d3 1 d9
1 df 1 e5
1 eb 1 f1
1 f7 1 fd
1 103 1 109
1 10f 1 115
1 11b 1 121
1 127 1 12d
1 133 1 139
1 13f 1 145
1 14b 1 151
1 157 1 15d
1 16a 1 176
2 179 17b 2
17e 180 2 182
184 2 186 188
2 18a 18c 2
192 194 2 196
198 2 19a 19b
2 19d 19e 2
18e 1a0 2 1a2
1a4 2 1aa 1ac
2 1ae 1b0 2
1b2 1b3 2 1b5
1b6 2 1a6 1b8
2 1ba 1bc 2
1be 1c0 2 1c2
1c4 2 1c6 1c8
2 1ca 1cc 4
172 178 17d 1ce
1 1d0 1 1d2
2 1dc 1de 3
1da 1db 1e0 2
1e9 1eb 2 1e8
1ed 1 1ef 1
1f1 2 242 247
1 24e 1 249
2 261 263 3
25f 260 265 2
26e 270 2 272
274 2 276 278
2 27a 27c 2
27e 280 3 285
286 287 2 282
289 2 26d 28b
1 28d 1 28f
2 258 290 1
294 1 253 2
251 298 1 2a3
2 29f 2a3 2
2af 2b1 3 2ad
2ae 2b3 2 2bc
2be 2 2c0 2c2
2 2c4 2c6 1
2ce 2 2d0 2d1
2 2c8 2d3 2
2d5 2d7 2 2d9
2db 2 2dd 2df
2 2e1 2e3 2
2bb 2e5 1 2e7
1 2e9 2 2f3
2f5 3 2f1 2f2
2f7 2 300 302
2 2ff 304 1
306 1 308 3
30f 310 311 3
2ea 309 313 1
315 1 317 1
31c 3 32c 32d
32e 1 332 2
330 332 3 339
33a 33b 1 33f
2 33d 33f 2
34d 34f 3 34b
34c 351 2 35a
35c 2 35e 360
2 362 364 1
36c 2 36e 36f
2 366 371 2
373 375 2 377
379 2 37b 37d
2 37f 381 2
359 383 1 385
1 387 2 391
393 3 38f 390
395 2 39e 3a0
2 39d 3a2 1
3a4 1 3a6 3
3ad 3ae 3af 3
388 3a7 3b1 1
3b3 2 3be 3c3
1 3ca 1 3c5
2 3dd 3df 3
3db 3dc 3e1 2
3ea 3ec 2 3ee
3f0 2 3f2 3f4
2 3f6 3f8 2
3fa 3fc 3 401
402 403 2 3fe
405 2 3e9 407
1 409 1 40b
2 3d4 40c 1
410 1 3cf 2
3cd 414 3 41c
41d 41e 1 422
2 420 422 2
427 42a 2 434
436 3 432 433
438 2 441 443
2 445 447 2
449 44b 1 453
2 455 456 2
44d 458 2 440
45a 1 45c 1
45e 2 468 46a
3 466 467 46c
2 475 477 2
474 479 1 47b
1 47d 2 484
485 3 45f 47e
487 2 489 48a
5 325 328 3b4
417 48b 1 48d
1 491 a 497
498 499 49a 49b
49c 49d 49e 49f
4a0 2 4a5 4a6
1 4aa 2 4a8
4aa 1 4ae 8
4b8 4b9 4ba 4bb
4bc 4bd 4be 4bf
1 4c2 1 4c4
1 4c6 3 4cd
4ce 4cf 1 4d2
3 4d5 4d6 4d7
1 4db 2 4d9
4db 3 4e0 4e1
4e2 1 4e5 3
4e9 4ea 4eb 1
4ee 2 4f0 4f1
1 4f2 2 4f4
4f5 1 4f7 2
506 50b 1 512
1 50d 2 525
527 3 523 524
529 2 532 534
2 536 538 2
53a 53c 2 53e
540 2 542 544
3 549 54a 54b
2 546 54d 2
531 54f 1 551
1 553 2 51c
554 1 558 1
517 2 515 55c
2 563 564 2
569 56a 2 566
56c 2 55f 56f
1 571 2 57f
580 2 584 585
8 578 579 57a
57b 57c 57d 582
587 2 58c 58d
1 591 2 58f
591 2 59c 59e
3 59a 59b 5a0
2 5a9 5ab 2
5ad 5af 2 5b1
5b3 1 5bb 2
5bd 5be 2 5b5
5c0 2 5c2 5c4
2 5c6 5c8 2
5ca 5cc 2 5ce
5d0 2 5a8 5d2
1 5d4 1 5d6
2 5e0 5e2 3
5de 5df 5e4 2
5ed 5ef 2 5ec
5f1 1 5f3 1
5f5 3 5fc 5fd
5fe 3 5d7 5f6
600 1 620 2
61f 620 1 628
1 62e 2 630
631 2 61d 632
1 639 1 634
2 64c 64e 3
64a 64b 650 2
659 65b 2 65d
65f 2 661 663
2 665 667 2
669 66b 3 670
671 672 2 66d
674 2 658 676
1 678 1 67a
2 643 67b 1
67f 1 63e 2
63c 683 2 606
686 2 689 68a
1 68e 2 69a
69c 3 698 699
69e 2 6a7 6a9
2 6ab 6ad 2
6af 6b1 1 6b7
2 6b9 6ba 2
6b3 6bc 2 6be
6c0 2 6c2 6c4
2 6c6 6c8 2
6ca 6cc 2 6a6
6ce 1 6d0 1
6d2 2 6dc 6de
3 6da 6db 6e0
2 6e9 6eb 2
6e8 6ed 1 6ef
1 6f1 2 6f7
6f8 3 6fa 6fb
6fc 2 707 709
3 705 706 70b
2 714 716 2
718 71a 2 71c
71e 1 726 2
728 729 2 720
72b 2 72d 72f
2 731 733 2
735 737 2 739
73b 2 713 73d
1 73f 1 741
3 748 749 74a
15 1d3 1f2 1f5
1f8 29b 316 48e
494 4a3 4c5 4f6
572 575 58a 68b
691 6d3 6f2 6fe
742 74c 2 759
75b 3 757 758
75d 2 766 768
2 765 76a 2
771 775 2 777
779 2 77b 77f
2 770 781 2
76c 783 1 785
3 786 791 795
1 750 2 7a8
7aa 3 7a6 7a7
7ac 2 7b5 7b7
2 7b4 7b9 2
7c0 7c2 2 7c4
7c6 3 7cb 7cc
7cd 2 7c8 7cf
2 7bf 7d1 2
7bb 7d3 1 7d5
4 79f 7d6 7df
7e3 1 7e7 1
79a 2 798 7eb
37 1f 26 2d
32 37 3c 41
46 4b 50 57
5e 64 6a 70
75 7a 80 86
8c 92 99 a0
a7 ad b3 b9
bf c5 cb d1
d7 dd e3 e9
ef f5 fb 101
107 10d 113 119
11f 125 12b 131
137 13d 143 149
14f 155 15b 162

1
4
0 
7f2
0
1
14
a
3c
0 1 2 1 4 1 6 1
8 1 0 0 0 0 0 0
0 0 0 0 
11b 1 0
157 1 0
43 1 0
b5 1 0
127 1 0
d9 1 0
b 1 0
109 1 0
39 1 0
1a 1 0
6c 1 0
cd 1 0
df 1 0
f 1 0
59 1 0
3e 1 0
133 1 0
82 1 0
139 1 0
7c 1 0
72 1 0
121 1 0
e5 1 0
bb 1 0
f7 1 0
115 1 0
77 1 0
a9 1 0
c1 1 0
34 1 0
145 1 0
8e 1 0
60 1 0
103 1 0
4d 1 0
151 1 0
3 1 0
66 1 0
94 1 0
2f 1 0
c7 1 0
10f 1 0
d3 1 0
f1 1 0
52 1 0
13f 1 0
88 1 0
9b 1 0
28 1 0
eb 1 0
7 1 0
21 1 0
2 0 1
12d 1 0
48 1 0
fd 1 0
a2 1 0
af 1 0
14b 1 0
15d 1 0
0

/
SHOW ERRORS FUNCTION SP_U_WP_DATES;
ALTER FUNCTION SP_U_WP_DATES COMPILE DEBUG;
/**************************************************************************/
Prompt == FUNCTION FC_U_WP_ACTUAL ==
CREATE OR REPLACE FUNCTION FC_U_WP_ACTUAL wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
146
2 :e:
1FUNCTION:
1FC_U_WP_ACTUAL:
1H_ASSIGN_ID:
1OUT:
1NOCOPY:
1VARCHAR2:
1SH_H_NEW_WORK_PERCENT:
1PLS_INTEGER:
1SH_H_ACTUAL:
1SH_H_UNITS:
1H_REC_USER:
1H_ERROR_ID:
1RETURN:
1H_NEW_WORK_PERCENT:
1H_ACTUAL:
1H_UNITS:
1H_ID:
132:
1H_PRJ_ASSIGN_ID:
1H_PROJECT_ID:
1H_TASK_ID:
1H_RES_ID:
1H_SD:
1DATE:
1H_FD:
1H_START_DATE:
1TIMESTAMP:
1H_FINISH_DATE:
1H_LAST_SUBMITTED:
1H_WEEK_OF:
1H_BIT_FLAG:
11:
1H_RES_TYPE:
1H_REC_STATUS:
1H_BILLABLE:
10:
1H_CAPITAL_EXPENSE:
1H_EFFORT:
1H_WORK_PERCENT:
1H_PERCENT_UNITS:
1H_WORK_TODATE:
1H_NEW_ACTUAL:
1H_LAST_DAY:
1H_PER_DAY:
1H_TASK_TYPE:
1H_TPH_DELETE:
1H_TEMP_VAR:
1H_MINUTES:
1H_DURATION:
1H_CONTOUR:
1H_BITFLAG:
1H_CONSIDER:
1H_EUNIT:
1H_UNIT:
1H_WORK:
1H_SCHED_EFFORT:
1H_WORK_TYPE:
1H_ETC:
1H_CLOSE_TASK:
1H_REMAINING:
1H_ROLLUP_BITS:
1H_BITID:
1RAW:
136:
1PMO_CNST:
1SC_ZERO_BIT:
1H_BITID1:
1H_ASSIGN_STRING:
18192:
1H_REG_COST:
1NUMBER:
1H_REG_SALE:
1H_SPEC_COST:
1H_SPEC_SALE:
1H_B_REG_COST:
1H_B_REG_SALE:
1H_B_SPEC_COST:
1H_B_SPEC_SALE:
1DEBUG_PERFTIMING:
1T_TOTAL:
1S_TIME_1:
1S_TIME_2:
1S_TIME_3:
1S_TIME_4:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= FC_U_WP_ACTUAL(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
182:
1+:
1FPUT:
1               ENTER:: FC_U_WP_ACTUAL(83) :: cplan.sql(15048) :: :
1PROJECT_ID:
1RESOURCE_ID:
1RESOURCE_TYPE:
1BIT_FLAG:
1LAST_SUBMITTED:
1WORK_TODATE:
1SCHED_WORK:
1ETC:
1LAST_SCHED_MIN:
1PROJECT_ASSIGNMENT_ID:
1C_CONTOUR:
1C_EFFORT:
1T_UNIT:
1C_UNIT:
1ASSIGN_UNITS:
1UNITS_MSP:
1UNRESOLVED_START:
1UNRESOLVED_FINISH:
1TO_DATE:
1SUBSTR:
1BILLABLE:
1CAPITAL_EXPENSE:
1REGULAR_COST:
1SPECIAL_COST:
1REGULAR_RATE:
1SPECIAL_RATE:
1B_REGULAR_COST:
1B_SPECIAL_COST:
1B_REGULAR_RATE:
1B_SPECIAL_RATE:
1MILESTONE_PERCENT:
1TASK_ID:
1ROLLUP_BITS:
1TASK_ASSIGNMENTS:
1TSK_ASSIGNMENT_ID:
1SELECT PROJECT_ID, RESOURCE_ID, RESOURCE_TYPE, BIT_FLAG, LAST_SUBMITTED, WORK+
1_TODATE, SCHED_WORK-WORK_TODATE, ETC,:n      SCHED_WORK, LAST_SCHED_MIN, PROJ+
1ECT_ASSIGNMENT_ID, C_CONTOUR, C_EFFORT, T_UNIT, C_UNIT, CASE WHEN ASSIGN_UNIT+
1S > UNITS_MSP THEN ASSIGN_UNITS ELSE UNITS_MSP END,:n      UNRESOLVED_START, +
1UNRESOLVED_FINISH, TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(U+
1NRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,:n      REGULAR_COST, SP+
1ECIAL_COST, REGULAR_RATE, SPECIAL_RATE,:n      B_REGULAR_COST, B_SPECIAL_COST+
1, B_REGULAR_RATE, B_SPECIAL_RATE,:n      MILESTONE_PERCENT, TASK_ID, ROLLUP_B+
1ITS:n      INTO h_project_id, h_res_id, h_res_type, h_bit_flag, h_last_submit+
1ted, h_work_todate, h_remaining, h_etc,:n      h_effort, h_last_day, h_prj_as+
1sign_id, h_contour, h_work, h_eunit, h_unit, h_percent_units,:n      h_start_+
1date, h_finish_date, h_SD, h_FD, h_billable, h_capital_expense,:n      h_reg_+
1cost, h_spec_cost, h_reg_sale, h_spec_sale,:n      h_b_reg_cost, h_b_spec_cos+
1t, h_b_reg_sale, h_b_spec_sale,:n      h_work_percent, h_task_id, h_rollup_bi+
1ts:n      FROM TASK_ASSIGNMENTS:n      WHERE TSK_ASSIGNMENT_ID = h_id:
1PMO_DB2M:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
1110:
1           EXCEPTION:: FC_U_WP_ACTUAL(111) :: cplan.sql(15076) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SQLERRM:
1500:
1=:
1SC_SQL_NO_DATA:
1TASK_TYPE:
1SCHED_EFFORT:
1SCHED_DURATION:
1TMT_WBS:
1ELEMENT_ID:
1SELECT PROJECT_ID, TASK_TYPE, SCHED_EFFORT, WORK_TODATE, MILESTONE_PERCENT, S+
1CHED_DURATION:n         INTO h_project_id, h_task_type, h_effort, h_work_toda+
1te, h_work_percent, h_duration:n         FROM TMT_WBS:n         WHERE ELEMENT+
1_ID = h_id:
1129:
1           EXCEPTION:: FC_U_WP_ACTUAL(130) :: cplan.sql(15095) :: :
1137:
1                EXIT:: FC_U_WP_ACTUAL(138) :: cplan.sql(15103) :: :
1RECORD_NOT_FOUND:
1LEVEL_LOCK:
1PROJECT_DETAILS:
1SELECT LEVEL_LOCK:n         INTO h_temp_var:n         FROM PROJECT_DETAILS:n +
1        WHERE PROJECT_ID = h_project_id:
1156:
1           EXCEPTION:: FC_U_WP_ACTUAL(157) :: cplan.sql(15122) :: :
1>:
1REC_STATUS:
1SELECT REC_STATUS:n            INTO h_rec_status:n            FROM TMT_WBS:n +
1           WHERE ELEMENT_ID = h_project_id:
1175:
1           EXCEPTION:: FC_U_WP_ACTUAL(176) :: cplan.sql(15141) :: :
1PMO_ORASTD:
1STRCMP:
1L:
1183:
1                EXIT:: FC_U_WP_ACTUAL(184) :: cplan.sql(15149) :: :
1LEV_LOCKED_BY_COPY:
1COUNT:
1DUAL:
1A:
1TA:
1ACTIVITIES:
1TYPE_ID:
1SC_TIMESHEET_ACTIVITY:
1SELECT COUNT('X'):n         INTO h_temp_var:n         FROM DUAL:n         WHE+
1RE EXISTS(:n         SELECT A.ELEMENT_ID:n         FROM TASK_ASSIGNMENTS TA, +
1ACTIVITIES A:n         WHERE TA.PROJECT_ID = h_project_id:n         AND TA.TA+
1SK_ID = h_id:n         AND TA.TSK_ASSIGNMENT_ID = A.TSK_ASSIGNMENT_ID:n      +
1   AND A.TSK_ASSIGNMENT_ID <> A.ELEMENT_ID:n         AND A.TYPE_ID = PMO_CNST+
1.SC_TIMESHEET_ACTIVITY):
1210:
1           EXCEPTION:: FC_U_WP_ACTUAL(211) :: cplan.sql(15176) :: :
1218:
1                EXIT:: FC_U_WP_ACTUAL(219) :: cplan.sql(15184) :: :
1SC_TIMESHEET_STEP_EXISTS:
18:
1225:
1      RETURN_INVALID:: FC_U_WP_ACTUAL(226) :: cplan.sql(15191) :: :
1STATE:: :
1INVALID_PARAMS:
1228:
1                EXIT:: FC_U_WP_ACTUAL(229) :: cplan.sql(15194) :: :
1PMODEFINES:
1PMO_RETURN_INVALID:
1FALSE:
1ELSIF:
1!=:
1236:
1                EXIT:: FC_U_WP_ACTUAL(237) :: cplan.sql(15202) :: :
1SC_TASK_ACTUAL_GRATER_THAN_SCH:
1FC_U_TASK_PERCENT:
1248:
1      RETURN_INVALID:: FC_U_WP_ACTUAL(249) :: cplan.sql(15214) :: :
1251:
1                EXIT:: FC_U_WP_ACTUAL(252) :: cplan.sql(15217) :: :
1259:
1                EXIT:: FC_U_WP_ACTUAL(260) :: cplan.sql(15225) :: :
1278:
1           EXCEPTION:: FC_U_WP_ACTUAL(279) :: cplan.sql(15244) :: :
1297:
1           EXCEPTION:: FC_U_WP_ACTUAL(298) :: cplan.sql(15263) :: :
1305:
1                EXIT:: FC_U_WP_ACTUAL(306) :: cplan.sql(15271) :: :
1SELECT COUNT('X'):n         INTO h_temp_var:n         FROM DUAL:n         WHE+
1RE EXISTS(:n         SELECT ELEMENT_ID:n         FROM ACTIVITIES:n         WH+
1ERE TSK_ASSIGNMENT_ID = h_id:n         AND TSK_ASSIGNMENT_ID <> ELEMENT_ID:n +
1        AND TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY):
1330:
1           EXCEPTION:: FC_U_WP_ACTUAL(331) :: cplan.sql(15296) :: :
1338:
1                EXIT:: FC_U_WP_ACTUAL(339) :: cplan.sql(15304) :: :
1MINUTE_PER_DAY:
1SELECT PROJECT_ID, MINUTE_PER_DAY:n         INTO h_project_id, h_per_day:n   +
1      FROM TMT_WBS:n         WHERE PROJECT_ID = h_project_id:n         AND EL+
1EMENT_ID = h_task_id:
1357:
1           EXCEPTION:: FC_U_WP_ACTUAL(358) :: cplan.sql(15323) :: :
1-:
1FC_TPHASE_SCHED_ASSIGN:
110000:
1<:
1CONSIDER:
1PMO_STPU:
1BIT_OR:
1SC_SCHED_BIT:
1UPDATE TASK_ASSIGNMENTS:n                           SET CONSIDER = 7,:n      +
1                     ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SC+
1HED_BIT),:n                           SCHED_WORK = h_new_actual,:n           +
1                WORK_TODATE = h_new_actual,:n                           ETC =+
1 h_etc,:n                           MILESTONE_PERCENT = 10000:n              +
1             WHERE TSK_ASSIGNMENT_ID = h_id:
1FOUND:
1TRUE:
1100:
1400:
1           EXCEPTION:: FC_U_WP_ACTUAL(401) :: cplan.sql(15366) :: :
1PMO_FUNC:
1SC_BIT_30:
1UPDATE TASK_ASSIGNMENTS:n                           SET CONSIDER = 7,:n      +
1                     ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SC+
1HED_BIT),:n                           SCHED_WORK = h_new_actual,:n           +
1                WORK_TODATE = h_new_actual,:n                           ETC =+
1 0,:n                           MILESTONE_PERCENT = 10000:n                  +
1         WHERE TSK_ASSIGNMENT_ID = h_id:
1428:
1           EXCEPTION:: FC_U_WP_ACTUAL(429) :: cplan.sql(15394) :: :
1TRUNC:
1TO_NUMBER:
1/:
1*:
110000.00:
10.4:
1UPDATE TASK_ASSIGNMENTS:n                           SET CONSIDER = 7,:n      +
1                     ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SC+
1HED_BIT),:n                           SCHED_WORK = h_etc,:n                  +
1         WORK_TODATE = h_new_actual:n                           WHERE TSK_ASS+
1IGNMENT_ID = h_id:
1458:
1           EXCEPTION:: FC_U_WP_ACTUAL(459) :: cplan.sql(15424) :: :
1487:
1           EXCEPTION:: FC_U_WP_ACTUAL(488) :: cplan.sql(15453) :: :
1UPDATE TASK_ASSIGNMENTS:n                     SET CONSIDER = 7,:n            +
1         ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),:n +
1                    SCHED_WORK = h_new_actual,:n                     WORK_TOD+
1ATE = h_new_actual,:n                     ETC = 0,:n                     MILE+
1STONE_PERCENT = 10000:n                     WHERE TSK_ASSIGNMENT_ID = h_id:
1518:
1           EXCEPTION:: FC_U_WP_ACTUAL(519) :: cplan.sql(15484) :: :
1IS NOT NULL:
1MAX:
1WEEK_OF:
1TIMESHEET_APPROVAL_STATUS:
1SELECT MAX(WEEK_OF):n                     INTO h_week_of:n                   +
1  FROM TIMESHEET_APPROVAL_STATUS:n                     WHERE ELEMENT_ID = h_p+
1roject_id:n                     AND RESOURCE_ID = h_res_id:
1540:
1           EXCEPTION:: FC_U_WP_ACTUAL(541) :: cplan.sql(15506) :: :
1SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_week_of FROM DUAL:
1557:
1           EXCEPTION:: FC_U_WP_ACTUAL(558) :: cplan.sql(15523) :: :
1STRNCMP:
110:
1DB2TIMESTAMP:
1TIME:
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_+
1ORASTD.TIME(h_start_date)):n                        INTO h_start_date:n      +
1                  FROM DUAL:
1580:
1           EXCEPTION:: FC_U_WP_ACTUAL(581) :: cplan.sql(15546) :: :
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_+
1ORASTD.TIME(h_start_date)) + 1     :n                        INTO h_start_dat+
1e:n                        FROM DUAL:
1597:
1           EXCEPTION:: FC_U_WP_ACTUAL(598) :: cplan.sql(15563) :: :
1FC_DEL_NOACTUAL:
12:
1FC_RESCHEDULE:
1623:
1                EXIT:: FC_U_WP_ACTUAL(624) :: cplan.sql(15589) :: :
1FC_U_ASSIGN_PERCENT:
1649:
1                EXIT:: FC_U_WP_ACTUAL(650) :: cplan.sql(15615) :: :
1SC_PERCENT_COMPLETE_BIT:
1UPDATE TASK_ASSIGNMENTS:n            SET MILESTONE_PERCENT = h_new_work_perce+
1nt,:n            ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCEN+
1T_COMPLETE_BIT):n            WHERE TSK_ASSIGNMENT_ID = h_id:
1703:
1           EXCEPTION:: FC_U_WP_ACTUAL(704) :: cplan.sql(15669) :: :
1UPDATE TMT_WBS:n            SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO+
1_CNST.SC_PERCENT_COMPLETE_BIT):n            WHERE PROJECT_ID = h_project_id:n+
1            AND ELEMENT_ID = h_task_id:
1730:
1           EXCEPTION:: FC_U_WP_ACTUAL(731) :: cplan.sql(15696) :: :
1103079040:
1743:
1                EXIT:: FC_U_WP_ACTUAL(744) :: cplan.sql(15709) :: :
1SELECT UNITS_MSP:n               INTO h_temp_var:n               FROM TASK_AS+
1SIGNMENTS:n               WHERE TSK_ASSIGNMENT_ID = h_prj_assign_id:
1765:
1           EXCEPTION:: FC_U_WP_ACTUAL(766) :: cplan.sql(15731) :: :
1SC_SQL_SUCCESS:
1775:
1                EXIT:: FC_U_WP_ACTUAL(776) :: cplan.sql(15741) :: :
1TASK_ASSIGNMENT_ERROR:
1FC_U_TDY_LAST_ACTUAL:
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_+
1ORASTD.TIME(h_start_date)) + 1     :n               INTO h_start_date:n      +
1         FROM DUAL:
1871:
1           EXCEPTION:: FC_U_WP_ACTUAL(872) :: cplan.sql(15837) :: :
1890:
1                EXIT:: FC_U_WP_ACTUAL(891) :: cplan.sql(15856) :: :
1CONTOUR_TYPE:
1SC_PLANNED_BIT:
1UPDATE TASK_ASSIGNMENTS:n               SET CONSIDER = 7,:n               UNI+
1TS_MSP = h_percent_units,:n               C_EFFORT = 0,:n               C_UNI+
1T = 0,:n               C_CONTOUR = 0,:n               CONTOUR_TYPE = 0,:n    +
1           ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+P+
1MO_CNST.SC_PLANNED_BIT)):n               WHERE TSK_ASSIGNMENT_ID = h_id:
1916:
1           EXCEPTION:: FC_U_WP_ACTUAL(917) :: cplan.sql(15882) :: :
1FC_ROLLUP:
1934:
1                EXIT:: FC_U_WP_ACTUAL(935) :: cplan.sql(15900) :: :
0

0
0
12f9
2
0 a0 8d 90 :3 a0 b0 3f 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 90 :3 a0 b0 3f 90
:3 a0 b0 3f b4 :2 a0 2c 6a a3
a0 1c a0 81 b0 a3 a0 1c
a0 81 b0 a3 a0 1c a0 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 1c 81
b0 a3 a0 1c 81 b0 a3 a0
1c 81 b0 a3 a0 1c 81 b0
a3 a0 1c 81 b0 a3 a0 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 51 a5 1c
:2 a0 6b 81 b0 a3 a0 51 a5
1c :2 a0 6b 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
:2 a0 6b :2 a0 6b 6e a5 b a
10 :2 a0 6b 6e :2 a0 6b 6e a5
b 6e 7e a0 b4 2e 6e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 d
:4d a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b a0 7e a0 6b
b4 2e 5a :f a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b a0
7e a0 6b b4 2e 5a :2 a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :3 a0 6b 65 b7 19
3c a0 51 d :5 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 51
a5 b 7e 51 b4 2e :5 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 6b a0 6e a5 b 7e 51
b4 2e :2 a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :3 a0
6b 65 b7 19 3c b7 19 3c
:1b a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 51 a5 b 7e 51
b4 2e :2 a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :3 a0
6b 65 b7 19 3c :2 a0 51 a5
b 7e 51 b4 2e :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :3 a0 6b :2 a0
6b a0 a5 b 65 a0 b7 :2 a0
51 a5 b a0 7e a0 51 a5
b b4 2e a0 51 d :2 a0 51
a5 b a0 7e a0 51 a5 b
b4 2e :2 a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :3 a0
6b 65 a0 b7 :2 a0 51 a5 b
a0 7e a0 51 a5 b b4 2e
a0 51 d b7 :2 19 3c :2 a0 d
:8 a0 51 a5 b a5 b d b7
19 :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :3 a0 6b :2 a0 6b a0 a5 b
65 b7 :2 19 3c b7 :2 a0 51 a5
b 7e 51 b4 2e :2 a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :3 a0 6b 65 b7 19 3c
a0 51 d :5 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 51 a5
b 7e 51 b4 2e :5 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
6b a0 6e a5 b 7e 51 b4
2e :2 a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :3 a0 6b
65 b7 19 3c b7 19 3c :c a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 51 a5 b 7e 51 b4
2e :2 a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :3 a0 6b
65 b7 19 3c :9 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :3 a0 51
a5 b d :2 a0 51 a5 b a0
7e a0 51 a5 b b4 2e :2 a0
d :2 a0 51 a5 b a0 7e a0
51 a5 b b4 2e a0 7e 51
b4 2e d :c a0 a5 b d :2 a0
51 a5 b 7e 51 b4 2e :2 a0
51 a5 b a0 7e a0 51 a5
b b4 2e :2 a0 6b :2 a0 a5 b
7e 51 b4 2e :3 a0 51 a5 b
7e :2 a0 51 a5 b b4 2e d
a0 51 d :2 a0 51 a5 b 7e
:2 a0 51 a5 b 7e :2 a0 51 a5
b b4 2e 5a b4 2e :3 a0 51
a5 b 7e :2 a0 51 a5 b b4
2e 5a 7e :2 a0 51 a5 b b4
2e d :11 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :3 a0 6b :2 a0 51 a5 b
:2 a0 6b a5 b d b7 :10 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :2 19
3c b7 :2 a0 51 a5 b 7e :2 a0
51 a5 b 7e :2 a0 51 a5 b
b4 2e 5a b4 2e :3 a0 51 a5
b 7e :2 a0 51 a5 b b4 2e
d :5 a0 a5 b 51 a5 b 7e
:3 a0 a5 b 51 a5 b b4 2e
5a 7e 51 b4 2e 5a 7e 51
b4 2e a5 b d :e a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :3 a0 51
a5 b 7e :2 a0 51 a5 b b4
2e d a0 51 d :10 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :2 19 3c
b7 :2 19 3c b7 :3 a0 51 a5 b
7e :2 a0 51 a5 b b4 2e d
a0 51 d :10 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 :2 19 3c a0 7e
b4 2e 5a a0 4d d :8 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
a0 7e b4 2e 5a :5 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
6b :2 a0 51 a5 b 7e 51 b4
2e :2 a0 d b7 19 3c a0 b7
:2 a0 51 a5 b 7e 51 b4 2e
:3 a0 6b :2 a0 51 a5 b :2 a0 6b
a5 b 5a d :a a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 19
:a a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 :2 19 3c b7 19 3c
a0 7e b4 2e 5a :4 a0 51 a5
b d b7 :2 a0 :2 51 a5 b 7e
6e b4 2e :4 a0 51 a5 b d
b7 :4 a0 51 a5 b d b7 :2 19
3c b7 :2 19 3c a0 51 d :8 a0
51 :2 a0 51 a5 b a5 b d
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :3 a0 51 a5
b 65 b7 19 3c :2 a0 d b7
19 3c b7 :5 a0 a5 b 51 a5
b 7e :3 a0 a5 b 51 a5 b
b4 2e 5a 7e 51 b4 2e 5a
7e 51 b4 2e a5 b d b7
:2 19 3c :1b a0 51 a5 b a5 b
d :2 a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :3 a0 51
a5 b 65 a0 b7 :2 a0 51 a5
b a0 7e a0 51 a5 b b4
2e :b a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :b a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f a0 b7 19 :2 a0 51 a5 b
a0 7e a0 51 a5 b b4 2e
:2 a0 51 a5 b 7e 51 b4 2e
a0 51 d b7 :2 a0 51 a5 b
7e 51 b4 2e 5a :2 a0 51 a5
b 7e 51 b4 2e 5a 52 10
:2 a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :3 a0 6b 65
b7 19 3c :3 a0 51 a5 b d
b7 :2 19 3c a0 7e b4 2e 5a
:5 a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b a0 7e a0 6b
b4 2e 5a :2 a0 51 a5 b a0
7e a0 51 a5 b b4 2e 5a
a 10 :2 a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :3 a0
6b 65 b7 19 3c b7 19 3c
a0 7e 51 b4 2e d :c a0 a5
b d :2 a0 51 a5 b 7e 51
b4 2e 5a a0 7e b4 2e :2 5a
a 10 :9 a0 51 a5 b d b7
19 3c a0 7e b4 2e 5a :4 a0
51 a5 b d b7 :2 a0 :2 51 a5
b 7e 6e b4 2e :4 a0 51 a5
b d b7 :4 a0 51 a5 b d
b7 :2 19 3c b7 :2 19 3c a0 7e
b4 2e 5a :a a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :3 a0 51 a5
b 7e :2 a0 51 a5 b b4 2e
d b7 19 3c a0 51 d a0
51 d :8 a0 51 :2 a0 51 a5 b
a5 b d :2 a0 51 a5 b 7e
51 b4 2e :2 a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:3 a0 6b 65 b7 :12 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c b7
:2 19 3c b7 :2 19 3c :2 a0 6b :2 a0
a5 b d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :2 a0 6b b4 2e a5 57
b7 19 3c :3 a0 6b 65 b7 a4
b1 11 68 4f 1d 17 b5 
12f9
2
0 3 7 2b 1f 23 27 1e
32 3f 3b 1b 47 50 4c 3a
58 65 61 37 6d 7e 72 76
7a 60 85 9a 8e 92 96 5d
a1 8d a6 aa ae b2 cf ba
be c6 ca 8a eb d6 da e2
e6 b9 10b f6 fa 102 106 b6
127 112 116 119 11a 122 f5 144
132 f2 136 137 13f 131 161 14f
12e 153 154 15c 14e 17e 16c 14b
170 171 179 16b 19b 189 168 18d
18e 196 188 1b7 1a6 1aa 1b2 185
1cf 1be 1c2 1ca 1a5 1eb 1da 1de
1e6 1a2 203 1f2 1f6 1fe 1d9 21f
20e 212 21a 1d6 237 226 22a 232
20d 254 242 20a 246 247 24f 241
271 25f 23e 263 264 26c 25e 28e
27c 25b 280 281 289 27b 2aa 299
29d 278 2a5 298 2c6 2b5 2b9 295
2c1 2b4 2e2 2d1 2d5 2b1 2dd 2d0
2fe 2ed 2f1 2cd 2f9 2ec 31a 309
30d 2e9 315 308 336 325 329 305
331 324 352 341 345 321 34d 340
36e 35d 361 33d 369 35c 38a 379
37d 359 385 378 3a6 395 399 375
3a1 394 3c2 3b1 3b5 391 3bd 3b0
3de 3cd 3d1 3ad 3d9 3cc 3fa 3e9
3ed 3c9 3f5 3e8 416 405 409 3e5
411 404 432 421 425 401 42d 420
44e 43d 441 41d 449 43c 46a 459
45d 439 465 458 486 475 479 455
481 474 4a2 491 495 471 49d 490
4be 4ad 4b1 48d 4b9 4ac 4da 4c9
4cd 4a9 4d5 4c8 4f6 4e5 4e9 4c5
4f1 4e4 512 501 505 4e1 50d 500
52e 51d 521 4fd 529 51c 54a 539
53d 519 545 538 566 555 559 535
561 554 58e 571 551 575 576 57e
582 586 589 570 5b6 599 56d 59d
59e 5a6 5aa 5ae 5b1 598 5d3 5c1
595 5c5 5c6 5ce 5c0 5ef 5de 5e2
5bd 5ea 5dd 60b 5fa 5fe 5da 606
5f9 627 616 61a 5f6 622 615 643
632 636 612 63e 631 65f 64e 652
62e 65a 64d 67b 66a 66e 64a 676
669 697 686 68a 666 692 685 6b3
6a2 6a6 682 6ae 6a1 6cf 6be 6c2
69e 6ca 6bd 6eb 6da 6de 6ba 6e6
6d9 707 6f6 6fa 6d6 702 6f5 723
712 716 6f2 71e 711 73f 72e 732
70e 73a 72d 75b 74a 74e 72a 756
749 762 766 746 76a 76e 772 775
77a 77b 1 77d 782 786 78a 78d
792 796 79a 79d 7a2 7a3 7a5 7aa
7ad 7b1 7b2 7b7 7bc 7bf 7c4 7c5
7ca 7cd 7d1 7d2 7d7 7da 7df 7e0
7e5 7e8 7ed 7ee 7f3 7f6 7fa 7fe
802 803 805 80a 80b 80d 80e 813
816 81b 81c 821 824 828 82c 830
831 833 838 839 83b 83c 841 844
849 84a 84f 852 856 85a 85e 85f
861 866 867 869 86a 86f 872 877
878 87d 880 885 886 88b 88e 892
893 898 89b 8a0 8a1 8a6 8a9 8ae
8af 8b4 8b7 8bc 8bd 8c2 8c5 8c9
8ca 8cf 8d2 8d7 8d8 8dd 8e0 8e5
8e6 8eb 8ec 8f1 8f3 8f7 8fa 8fe
902 905 909 90d 910 913 918 91b
91e 921 922 927 928 1 92a 92f
933 937 93a 93f 944 947 94b 94c
951 952 957 959 95d 960 964 968
96c 970 974 978 97c 980 984 988
98c 990 994 998 99c 9a0 9a4 9a8
9ac 9b0 9b4 9b8 9bc 9c0 9c4 9c8
9cc 9d0 9d4 9d8 9dc 9e0 9e4 9e8
9ec 9f0 9f4 9f8 9fc a00 a04 a08
a0c a10 a14 a18 a1c a20 a24 a28
a2c a30 a34 a38 a3c a40 a44 a48
a4c a50 a54 a58 a5c a60 a64 a68
a6c a70 a74 a78 a7c a80 a84 a88
a8c a90 a94 a98 a9c aa0 aac ab0
ab4 ab7 abb abf ac1 ac5 ac9 acd
ad0 ad4 ad8 ada adb ae0 1 ae4
ae8 aec aef af3 af7 afb aff b02
b06 b0a b0d b10 b15 b18 b1b b1e
b1f b24 b25 1 b27 b2c b30 b34
b37 b3c b41 b44 b48 b49 b4e b51
b56 b57 b5c b5f b64 b65 b6a b6d
b71 b72 b77 b7a b7f b80 b85 b88
b8c b90 b93 b96 b97 b99 b9a b9f
ba0 ba5 ba7 bab bae bb0 bb4 bb6
bc2 bc4 bc6 bc7 bcc bd0 bd2 bde
be0 be4 be8 beb bef bf2 bf6 bf9
bfa bff c02 c06 c0a c0e c12 c16
c1a c1e c22 c26 c2a c2e c32 c36
c3a c3e c4a c4e c52 c55 c59 c5d
c5f c63 c67 c6b c6e c72 c76 c78
c79 c7e 1 c82 c86 c8a c8d c91
c95 c99 c9d ca0 ca4 ca8 cab cae
cb3 cb6 cb9 cbc cbd cc2 cc3 1
cc5 cca cce cd2 cd5 cda cdf ce2
ce6 ce7 cec cef cf4 cf5 cfa cfd
d02 d03 d08 d0b d0f d10 d15 d18
d1d d1e d23 d26 d2a d2e d31 d34
d35 d37 d38 d3d d3e d43 d45 d49
d4c d4e d52 d54 d60 d62 d64 d65
d6a d6e d70 d7c d7e d82 d86 d89
d8d d90 d94 d97 d98 d9d da0 da4
da8 dac db0 db4 db7 dbb dbf dc2
dc5 dca dcd dd0 dd3 dd4 dd9 dda
1 ddc de1 de5 de9 dec df1 df6
df9 dfd dfe e03 e04 e09 e0b e0f
e12 e16 e1a e1e e21 e25 e27 e2b
e2e e32 e35 e39 e3d e41 e45 e49
e4d e59 e5d e61 e64 e68 e6c e6e
e72 e76 e7a e7d e81 e85 e87 e88
e8d 1 e91 e95 e99 e9c ea0 ea4
ea8 eac eaf eb3 eb7 eba ebd ec2
ec5 ec8 ecb ecc ed1 ed2 1 ed4
ed9 edd ee1 ee4 ee9 eee ef1 ef5
ef6 efb efe f03 f04 f09 f0c f11
f12 f17 f1a f1e f1f f24 f27 f2c
f2d f32 f35 f39 f3d f40 f43 f44
f46 f47 f4c f4d f52 f54 f58 f5b
f5d f61 f63 f6f f71 f73 f74 f79
f7d f7f f8b f8d f91 f95 f98 f99
f9b f9e fa1 fa2 fa7 fab faf fb3
fb7 fbb fc7 fcb fcf fd2 fd6 fda
fdc fe0 fe4 fe8 feb fef ff3 ff5
ff6 ffb 1 fff 1003 1007 100a 100e
1012 1016 101a 101d 1021 1025 1028 102b
1030 1033 1036 1039 103a 103f 1040 1
1042 1047 104b 104f 1052 1057 105c 105f
1063 1064 1069 106c 1071 1072 1077 107a
107f 1080 1085 1088 108c 108d 1092 1095
109a 109b 10a0 10a3 10a7 10ab 10ae 10b1
10b2 10b4 10b5 10ba 10bb 10c0 10c2 10c6
10c9 10cb 10cf 10d1 10dd 10df 10e1 10e2
10e7 10eb 10ed 10f9 10fb 10ff 1103 1106
110a 110f 1110 1112 1115 1118 1119 111e
1122 1126 112a 112e 1132 1135 1139 113d
1140 1143 1148 114b 114e 1151 1152 1157
1158 1 115a 115f 1163 1167 116a 116f
1174 1177 117b 117c 1181 1182 1187 1189
118d 1190 1194 1198 119c 119f 11a3 11a5
11a9 11ac 11ae 11b2 11b5 11b9 11bd 11c1
11c5 11c9 11cd 11d1 11d5 11d9 11dd 11e1
11e5 11e9 11ed 11f1 11f5 11f9 11fd 1201
1205 1209 120d 1211 1215 1219 121d 1221
122d 1231 1235 1238 123c 1240 1242 1246
124a 124e 1251 1255 1259 125b 125c 1261
1 1265 1269 126d 1270 1274 1278 127c
1280 1283 1287 128b 128e 1291 1296 1299
129c 129f 12a0 12a5 12a6 1 12a8 12ad
12b1 12b5 12b8 12bd 12c2 12c5 12c9 12ca
12cf 12d2 12d7 12d8 12dd 12e0 12e5 12e6
12eb 12ee 12f2 12f3 12f8 12fb 1300 1301
1306 1309 130d 1311 1314 1317 1318 131a
131b 1320 1321 1326 1328 132c 132f 1331
1335 1337 1343 1345 1347 1348 134d 1351
1353 135f 1361 1365 1369 136c 136d 136f
1372 1375 1376 137b 137f 1383 1387 138b
138f 1392 1396 139a 139d 13a0 13a5 13a8
13ab 13ae 13af 13b4 13b5 1 13b7 13bc
13c0 13c4 13c7 13cc 13d1 13d4 13d8 13d9
13de 13df 13e4 13e6 13ea 13ed 13f1 13f5
13f9 13fc 1400 1402 1406 1409 140d 1411
1414 1415 1417 141a 141d 141e 1423 1427
142b 142e 1432 1436 1439 143c 1441 1444
1447 144a 144b 1450 1451 1 1453 1458
145c 1460 1463 1468 146d 1470 1474 1475
147a 147d 1482 1483 1488 148b 1490 1491
1496 1499 149d 14a1 14a5 14a9 14ac 14ad
14af 14b4 14b5 14b7 14b8 14bd 14be 14c3
14c5 14c9 14cc 14d0 14d4 14d7 14db 14df
14e2 14e5 14ea 14ed 14f0 14f3 14f4 14f9
14fa 1 14fc 1501 1505 1509 150c 1511
1516 1519 151d 151e 1523 1524 1529 152b
152f 1532 1536 153a 153e 1541 1545 1549
154c 1550 1551 1553 1557 155b 155d 1561
1565 1568 1569 156b 156f 1572 1576 1579
157a 157c 157d 1582 1586 1589 158d 1591
1595 1598 1599 159b 159f 15a2 15a6 15a9
15aa 15ac 15ad 15b2 15b6 15ba 15be 15c2
15c6 15c9 15cd 15d1 15d4 15d7 15dc 15df
15e2 15e5 15e6 15eb 15ec 1 15ee 15f3
15f7 15fb 15fe 1603 1608 160b 160f 1610
1615 1616 161b 161d 1621 1624 1628 162c
1630 1633 1637 163b 163d 1641 1645 1648
1649 164b 164f 1652 1656 1659 165a 165c
165d 1662 1666 1669 166d 166f 1673 1677
167a 167e 1682 1686 168a 168e 1692 1696
169a 169e 16a2 16a6 16a9 16aa 16ac 16ad
16af 16b3 16b5 16b9 16bd 16c1 16c4 16c8
16cc 16cf 16d2 16d7 16da 16dd 16e0 16e1
16e6 16e7 1 16e9 16ee 16f2 16f6 16f9
16fe 1703 1706 170a 170b 1710 1713 1718
1719 171e 1721 1726 1727 172c 172f 1733
1737 173b 173f 1742 1743 1745 174a 174b
174d 174e 1753 1754 1759 175b 175f 1762
1766 176a 176d 1771 1775 1778 177b 1780
1783 1786 1789 178a 178f 1790 1 1792
1797 179b 179f 17a2 17a7 17ac 17af 17b3
17b4 17b9 17ba 17bf 17c1 17c5 17c8 17cc
17d0 17d4 17d7 17db 17df 17e2 17e6 17e7
17e9 17ed 17ef 17f3 17f7 17fa 17fc 1800
1804 1807 1808 180a 180d 1810 1811 1816
181a 181e 1822 1826 182a 182d 1831 1835
1838 183b 1840 1843 1846 1849 184a 184f
1850 1 1852 1857 185b 185f 1862 1867
186c 186f 1873 1874 1879 187a 187f 1881
1885 1888 188c 1890 1894 1897 189b 189d
18a1 18a4 18a8 18ab 18af 18b3 18b7 18bb
18bf 18c3 18cf 18d3 18d7 18da 18de 18e2
18e4 18e8 18ec 18f0 18f3 18f7 18fb 18fd
18fe 1903 1 1907 190b 190f 1912 1916
191a 191e 1922 1925 1929 192d 1930 1933
1938 193b 193e 1941 1942 1947 1948 1
194a 194f 1953 1957 195a 195f 1964 1967
196b 196c 1971 1974 1979 197a 197f 1982
1987 1988 198d 1990 1994 1995 199a 199d
19a2 19a3 19a8 19ab 19af 19b3 19b6 19b9
19ba 19bc 19bd 19c2 19c3 19c8 19ca 19ce
19d1 19d3 19d7 19d9 19e5 19e7 19e9 19ea
19ef 19f3 19f5 1a01 1a03 1a07 1a0b 1a0e
1a0f 1a11 1a14 1a17 1a18 1a1d 1a21 1a25
1a29 1a2d 1a31 1a3d 1a41 1a45 1a48 1a4c
1a50 1a52 1a56 1a5a 1a5e 1a61 1a65 1a69
1a6b 1a6c 1a71 1 1a75 1a79 1a7d 1a80
1a84 1a88 1a8c 1a90 1a93 1a97 1a9b 1a9e
1aa1 1aa6 1aa9 1aac 1aaf 1ab0 1ab5 1ab6
1 1ab8 1abd 1ac1 1ac5 1ac8 1acd 1ad2
1ad5 1ad9 1ada 1adf 1ae2 1ae7 1ae8 1aed
1af0 1af5 1af6 1afb 1afe 1b02 1b03 1b08
1b0b 1b10 1b11 1b16 1b19 1b1d 1b21 1b24
1b27 1b28 1b2a 1b2b 1b30 1b31 1b36 1b38
1b3c 1b3f 1b41 1b45 1b47 1b53 1b55 1b57
1b58 1b5d 1b61 1b63 1b6f 1b71 1b75 1b79
1b7c 1b80 1b85 1b86 1b88 1b8b 1b8e 1b8f
1b94 1b98 1b9c 1ba0 1ba4 1ba8 1bab 1baf
1bb3 1bb6 1bb9 1bbe 1bc1 1bc4 1bc7 1bc8
1bcd 1bce 1 1bd0 1bd5 1bd9 1bdd 1be0
1be5 1bea 1bed 1bf1 1bf2 1bf7 1bf8 1bfd
1bff 1c03 1c06 1c0a 1c0e 1c12 1c15 1c19
1c1b 1c1f 1c22 1c24 1c28 1c2b 1c2f 1c33
1c37 1c3b 1c3f 1c43 1c47 1c4b 1c4f 1c53
1c57 1c5b 1c67 1c6b 1c6f 1c72 1c76 1c7a
1c7c 1c80 1c84 1c88 1c8b 1c8f 1c93 1c95
1c96 1c9b 1 1c9f 1ca3 1ca7 1caa 1cae
1cb2 1cb6 1cba 1cbd 1cc1 1cc5 1cc8 1ccb
1cd0 1cd3 1cd6 1cd9 1cda 1cdf 1ce0 1
1ce2 1ce7 1ceb 1cef 1cf2 1cf7 1cfc 1cff
1d03 1d04 1d09 1d0c 1d11 1d12 1d17 1d1a
1d1f 1d20 1d25 1d28 1d2c 1d2d 1d32 1d35
1d3a 1d3b 1d40 1d43 1d47 1d4b 1d4e 1d51
1d52 1d54 1d55 1d5a 1d5b 1d60 1d62 1d66
1d69 1d6b 1d6f 1d71 1d7d 1d7f 1d81 1d82
1d87 1d8b 1d8d 1d99 1d9b 1d9f 1da3 1da6
1da7 1da9 1dac 1daf 1db0 1db5 1db9 1dbd
1dc1 1dc5 1dc9 1dcc 1dd0 1dd4 1dd7 1dda
1ddf 1de2 1de5 1de8 1de9 1dee 1def 1
1df1 1df6 1dfa 1dfe 1e01 1e06 1e0b 1e0e
1e12 1e13 1e18 1e19 1e1e 1e20 1e24 1e27
1e2b 1e2f 1e33 1e36 1e3a 1e3c 1e40 1e43
1e47 1e4b 1e4f 1e53 1e57 1e5b 1e5f 1e63
1e67 1e73 1e77 1e7b 1e7e 1e82 1e86 1e88
1e8c 1e90 1e94 1e97 1e9b 1e9f 1ea1 1ea2
1ea7 1 1eab 1eaf 1eb3 1eb6 1eba 1ebe
1ec2 1ec6 1ec9 1ecd 1ed1 1ed4 1ed7 1edc
1edf 1ee2 1ee5 1ee6 1eeb 1eec 1 1eee
1ef3 1ef7 1efb 1efe 1f03 1f08 1f0b 1f0f
1f10 1f15 1f18 1f1d 1f1e 1f23 1f26 1f2b
1f2c 1f31 1f34 1f38 1f39 1f3e 1f41 1f46
1f47 1f4c 1f4f 1f53 1f57 1f5a 1f5d 1f5e
1f60 1f61 1f66 1f67 1f6c 1f6e 1f72 1f75
1f77 1f7b 1f7d 1f89 1f8b 1f8d 1f8e 1f93
1f97 1f99 1fa5 1fa7 1fab 1faf 1fb3 1fb6
1fb7 1fb9 1fbd 1fc1 1fc5 1fc8 1fc9 1fcb
1fcf 1fd2 1fd6 1fd9 1fda 1fdc 1fdd 1fe2
1fe6 1fea 1fee 1ff2 1ff6 1ff9 1ffa 1ffc
2000 2003 2007 200a 200b 200d 200e 2013
2017 201a 201d 201e 2023 2027 202b 202f
2033 2037 203b 203f 2043 2047 204b 204f
2053 2057 2058 205a 205e 2062 2066 2069
206a 206c 206f 2072 2073 2078 207c 2080
2083 2084 2086 208a 208d 2091 2094 2095
2097 2098 209d 20a1 20a5 20a8 20ac 20b0
20b1 20b3 20b6 20b9 20ba 20bf 20c3 20c7
20cb 20ce 20cf 20d1 20d4 20d8 20dc 20df
20e0 20e2 20e3 20e8 20ec 20f0 20f3 20f7
20fb 20ff 2102 2103 2105 2108 210c 2110
2113 2114 2116 2119 211d 2121 2124 2125
2127 2128 212d 2130 2131 2136 213a 213e
2142 2145 2146 2148 214b 214f 2153 2156
2157 2159 215a 215f 2162 2165 2169 216d
2170 2171 2173 2174 2179 217d 2181 2185
2189 218d 2191 2195 2199 219d 21a1 21a5
21a9 21ad 21b1 21b5 21b9 21bd 21c1 21cd
21d1 21d6 21da 21dd 21de 21e3 21e7 21eb
21ee 21f1 21f5 21f7 21fb 21ff 2202 2205
2209 220b 220f 2213 2216 2218 221c 2220
2224 2227 222b 222f 2231 2232 2237 1
223b 223f 2243 2246 224a 224e 2252 2256
2259 225d 2261 2264 2267 226c 226f 2272
2275 2276 227b 227c 1 227e 2283 2287
228b 228e 2293 2298 229b 229f 22a0 22a5
22a8 22ad 22ae 22b3 22b6 22bb 22bc 22c1
22c4 22c8 22c9 22ce 22d1 22d6 22d7 22dc
22df 22e3 22e7 22ea 22ed 22ee 22f0 22f1
22f6 22f7 22fc 22fe 2302 2305 2307 230b
230d 2319 231b 231d 231e 2323 2327 2329
2335 2337 233b 233f 2343 2346 234a 234e
2351 2352 2354 2358 235c 235f 2360 2362
2366 2368 236c 2370 2374 2378 237c 2380
2384 2388 238c 2390 2394 2398 239c 23a0
23a4 23a8 23b4 23b8 23bd 23c1 23c4 23c5
23ca 23ce 23d2 23d5 23d8 23dc 23de 23e2
23e6 23e9 23ec 23f0 23f2 23f6 23fa 23fd
23ff 2403 2407 240b 240e 2412 2416 2418
2419 241e 1 2422 2426 242a 242d 2431
2435 2439 243d 2440 2444 2448 244b 244e
2453 2456 2459 245c 245d 2462 2463 1
2465 246a 246e 2472 2475 247a 247f 2482
2486 2487 248c 248f 2494 2495 249a 249d
24a2 24a3 24a8 24ab 24af 24b0 24b5 24b8
24bd 24be 24c3 24c6 24ca 24ce 24d1 24d4
24d5 24d7 24d8 24dd 24de 24e3 24e5 24e9
24ec 24ee 24f2 24f4 2500 2502 2504 2505
250a 250e 2510 251c 251e 2520 2524 2528
252b 252d 2531 2535 2538 2539 253b 253e
2542 2546 2549 254a 254c 254f 2553 2557
255a 255b 255d 255e 2563 2566 2567 256c
2570 2574 2578 257b 257c 257e 2581 2585
2589 258c 258d 258f 2590 2595 2599 259d
25a1 25a5 25a9 25ad 25ae 25b0 25b3 25b4
25b6 25b9 25bd 25c1 25c5 25c6 25c8 25cb
25cc 25ce 25cf 25d4 25d7 25da 25dd 25de
25e3 25e6 25e9 25ec 25ed 25f2 25f3 25f5
25f9 25fd 2601 2605 2609 260d 2611 2615
2619 261d 2621 2625 2629 262d 2631 263d
2641 2646 264a 264d 264e 2653 2657 265b
265e 2661 2665 2667 266b 266f 2672 2675
2679 267b 267f 2683 2686 2688 268c 2690
2694 2697 269b 269f 26a1 26a2 26a7 1
26ab 26af 26b3 26b6 26ba 26be 26c2 26c6
26c9 26cd 26d1 26d4 26d7 26dc 26df 26e2
26e5 26e6 26eb 26ec 1 26ee 26f3 26f7
26fb 26fe 2703 2708 270b 270f 2710 2715
2718 271d 271e 2723 2726 272b 272c 2731
2734 2738 2739 273e 2741 2746 2747 274c
274f 2753 2757 275a 275d 275e 2760 2761
2766 2767 276c 276e 2772 2775 2777 277b
277d 2789 278b 278d 278e 2793 2797 2799
27a5 27a7 27a9 27ad 27b1 27b5 27b8 27b9
27bb 27be 27c2 27c6 27c9 27ca 27cc 27cd
27d2 27d6 27da 27dd 27e1 27e5 27e9 27ed
27f1 27f5 27f9 27fd 2801 2805 2809 280d
2811 2815 2819 281d 2821 282d 2831 2836
283a 283d 283e 2843 2847 284b 284e 2851
2855 2857 285b 285f 2862 2865 2869 286b
286f 2873 2876 2878 287c 2880 2884 2887
288b 288f 2891 2892 2897 1 289b 289f
28a3 28a6 28aa 28ae 28b2 28b6 28b9 28bd
28c1 28c4 28c7 28cc 28cf 28d2 28d5 28d6
28db 28dc 1 28de 28e3 28e7 28eb 28ee
28f3 28f8 28fb 28ff 2900 2905 2908 290d
290e 2913 2916 291b 291c 2921 2924 2928
2929 292e 2931 2936 2937 293c 293f 2943
2947 294a 294d 294e 2950 2951 2956 2957
295c 295e 2962 2965 2967 296b 296d 2979
297b 297d 297e 2983 2987 2989 2995 2997
2999 299d 29a1 29a4 29a6 29aa 29ae 29b1
29b3 29b7 29bb 29bf 29c2 29c3 29c5 29c8
29cc 29d0 29d3 29d4 29d6 29d7 29dc 29e0
29e4 29e7 29eb 29ef 29f3 29f7 29fb 29ff
2a03 2a07 2a0b 2a0f 2a13 2a17 2a1b 2a1f
2a23 2a27 2a2b 2a37 2a3b 2a40 2a44 2a47
2a48 2a4d 2a51 2a55 2a58 2a5b 2a5f 2a61
2a65 2a69 2a6c 2a6f 2a73 2a75 2a79 2a7d
2a80 2a82 2a86 2a8a 2a8e 2a91 2a95 2a99
2a9b 2a9c 2aa1 1 2aa5 2aa9 2aad 2ab0
2ab4 2ab8 2abc 2ac0 2ac3 2ac7 2acb 2ace
2ad1 2ad6 2ad9 2adc 2adf 2ae0 2ae5 2ae6
1 2ae8 2aed 2af1 2af5 2af8 2afd 2b02
2b05 2b09 2b0a 2b0f 2b12 2b17 2b18 2b1d
2b20 2b25 2b26 2b2b 2b2e 2b32 2b33 2b38
2b3b 2b40 2b41 2b46 2b49 2b4d 2b51 2b54
2b57 2b58 2b5a 2b5b 2b60 2b61 2b66 2b68
2b6c 2b6f 2b71 2b75 2b77 2b83 2b85 2b87
2b88 2b8d 2b91 2b93 2b9f 2ba1 2ba3 2ba7
2bab 2bae 2bb2 2bb5 2bb6 2bbb 2bbe 2bc2
2bc3 2bc7 2bcb 2bcf 2bd3 2bd7 2bdb 2bdf
2be3 2be7 2bf3 2bf7 2bfb 2bfe 2c02 2c06
2c08 2c0c 2c10 2c14 2c17 2c1b 2c1f 2c21
2c22 2c27 1 2c2b 2c2f 2c33 2c36 2c3a
2c3e 2c42 2c46 2c49 2c4d 2c51 2c54 2c57
2c5c 2c5f 2c62 2c65 2c66 2c6b 2c6c 1
2c6e 2c73 2c77 2c7b 2c7e 2c83 2c88 2c8b
2c8f 2c90 2c95 2c98 2c9d 2c9e 2ca3 2ca6
2cab 2cac 2cb1 2cb4 2cb8 2cb9 2cbe 2cc1
2cc6 2cc7 2ccc 2ccf 2cd3 2cd7 2cda 2cdd
2cde 2ce0 2ce1 2ce6 2ce7 2cec 2cee 2cf2
2cf5 2cf7 2cfb 2cfd 2d09 2d0b 2d0d 2d0e
2d13 2d17 2d19 2d25 2d27 2d2b 2d2e 2d2f
2d34 2d37 2d3b 2d3f 2d43 2d47 2d4b 2d57
2d5b 2d5f 2d62 2d66 2d6a 2d6c 2d70 2d74
2d78 2d7b 2d7f 2d83 2d85 2d86 2d8b 1
2d8f 2d93 2d97 2d9a 2d9e 2da2 2da6 2daa
2dad 2db1 2db5 2db8 2dbb 2dc0 2dc3 2dc6
2dc9 2dca 2dcf 2dd0 1 2dd2 2dd7 2ddb
2ddf 2de2 2de7 2dec 2def 2df3 2df4 2df9
2dfc 2e01 2e02 2e07 2e0a 2e0f 2e10 2e15
2e18 2e1c 2e1d 2e22 2e25 2e2a 2e2b 2e30
2e33 2e37 2e3b 2e3e 2e41 2e42 2e44 2e45
2e4a 2e4b 2e50 2e52 2e56 2e59 2e5b 2e5f
2e61 2e6d 2e6f 2e71 2e72 2e77 2e7b 2e7d
2e89 2e8b 2e8f 2e93 2e96 2e9a 2e9e 2ea1
2ea2 2ea4 2ea7 2eaa 2eab 2eb0 2eb4 2eb8
2ebc 2ebe 2ec2 2ec5 2ec9 2ecb 2ecf 2ed3
2ed6 2ed7 2ed9 2edc 2edf 2ee0 2ee5 2ee9
2eed 2ef1 2ef4 2ef8 2efc 2eff 2f00 2f02
2f06 2f0a 2f0d 2f0e 2f10 2f13 2f17 2f1b
2f1f 2f23 2f27 2f2b 2f2f 2f33 2f37 2f3b
2f3f 2f4b 2f4f 2f53 2f56 2f5a 2f5e 2f60
2f64 2f68 2f6c 2f6f 2f73 2f77 2f79 2f7a
2f7f 1 2f83 2f87 2f8b 2f8e 2f92 2f96
2f9a 2f9e 2fa1 2fa5 2fa9 2fac 2faf 2fb4
2fb7 2fba 2fbd 2fbe 2fc3 2fc4 1 2fc6
2fcb 2fcf 2fd3 2fd6 2fdb 2fe0 2fe3 2fe7
2fe8 2fed 2ff0 2ff5 2ff6 2ffb 2ffe 3003
3004 3009 300c 3010 3011 3016 3019 301e
301f 3024 3027 302b 302f 3032 3035 3036
3038 3039 303e 303f 3044 3046 304a 304d
304f 3053 3055 3061 3063 3065 3066 306b
306f 3071 307d 307f 3081 3085 3089 308d
3091 3095 3099 309d 30a1 30a5 30a9 30ad
30b9 30bd 30c1 30c4 30c8 30cc 30ce 30d2
30d6 30da 30dd 30e1 30e5 30e7 30e8 30ed
1 30f1 30f5 30f9 30fc 3100 3104 3108
310c 310f 3113 3117 311a 311d 3122 3125
3128 312b 312c 3131 3132 1 3134 3139
313d 3141 3144 3149 314e 3151 3155 3156
315b 315e 3163 3164 3169 316c 3171 3172
3177 317a 317e 317f 3184 3187 318c 318d
3192 3195 3199 319d 31a0 31a3 31a4 31a6
31a7 31ac 31ad 31b2 31b4 31b8 31bb 31bd
31c1 31c3 31cf 31d1 31d3 31d4 31d9 31dd
31df 31eb 31ed 31ef 31f3 31f7 31fa 31fc
3200 3203 3207 320a 320b 3210 3213 3217
321b 321f 3223 3226 3227 3229 322d 322f
3233 3237 323a 323d 323e 3240 3243 3248
3249 324e 3252 3256 325a 325e 3261 3262
3264 3268 326a 326e 3272 3276 327a 327d
327e 3280 3284 3286 328a 328e 3291 3293
3297 329b 329e 32a2 32a5 32a9 32ad 32b1
32b5 32b9 32bd 32c1 32c5 32c9 32cc 32d0
32d4 32d7 32d8 32da 32db 32dd 32e1 32e5
32e9 32ec 32ed 32ef 32f2 32f5 32f6 32fb
32ff 3303 3307 330b 330f 3312 3316 331a
331d 3320 3325 3328 332b 332e 332f 3334
3335 1 3337 333c 3340 3344 3347 334c
3351 3354 3358 3359 335e 335f 3364 3366
336a 336d 3371 3375 3379 337c 337d 337f
3383 3385 3389 338c 3390 3394 3398 339a
339e 33a1 33a3 33a7 33ab 33af 33b3 33b7
33b8 33ba 33bd 33be 33c0 33c3 33c7 33cb
33cf 33d0 33d2 33d5 33d6 33d8 33d9 33de
33e1 33e4 33e7 33e8 33ed 33f0 33f3 33f6
33f7 33fc 33fd 33ff 3403 3405 3409 340d
3410 3414 3418 341c 3420 3424 3428 342c
3430 3434 3438 343c 3440 3444 3448 344c
3450 3454 3458 345c 3460 3464 3468 346c
3470 3474 3478 347c 347f 3480 3482 3483
3485 3489 348d 3491 3495 3499 349d 34a0
34a4 34a8 34ab 34ae 34b3 34b6 34b9 34bc
34bd 34c2 34c3 1 34c5 34ca 34ce 34d2
34d5 34da 34df 34e2 34e6 34e7 34ec 34ed
34f2 34f4 34f8 34fb 34ff 3503 3507 350a
350b 350d 3511 3515 3517 351b 351f 3522
3523 3525 3529 352c 3530 3533 3534 3536
3537 353c 3540 3544 3548 354c 3550 3554
3558 355c 3560 3564 3568 3574 3578 357d
3581 3584 3585 358a 358e 3592 3595 3598
359c 359e 35a2 35a6 35a9 35ac 35b0 35b2
35b6 35ba 35bd 35bf 35c3 35c7 35cb 35ce
35d2 35d6 35d8 35d9 35de 1 35e2 35e6
35ea 35ed 35f1 35f5 35f9 35fd 3600 3604
3608 360b 360e 3613 3616 3619 361c 361d
3622 3623 1 3625 362a 362e 3632 3635
363a 363f 3642 3646 3647 364c 364f 3654
3655 365a 365d 3662 3663 3668 366b 366f
3670 3675 3678 367d 367e 3683 3686 368a
368e 3691 3694 3695 3697 3698 369d 369e
36a3 36a5 36a9 36ac 36ae 36b2 36b4 36c0
36c2 36c4 36c5 36ca 36ce 36d0 36dc 36de
36e2 36e6 36ea 36ee 36f2 36f6 36fa 36fe
3702 3706 370a 3716 371a 371f 3723 3726
3727 372c 3730 3734 3737 373a 373e 3740
3744 3748 374b 374e 3752 3754 3758 375c
375f 3761 3765 3769 376d 3770 3774 3778
377a 377b 3780 1 3784 3788 378c 378f
3793 3797 379b 379f 37a2 37a6 37aa 37ad
37b0 37b5 37b8 37bb 37be 37bf 37c4 37c5
1 37c7 37cc 37d0 37d4 37d7 37dc 37e1
37e4 37e8 37e9 37ee 37f1 37f6 37f7 37fc
37ff 3804 3805 380a 380d 3811 3812 3817
381a 381f 3820 3825 3828 382c 3830 3833
3836 3837 3839 383a 383f 3840 3845 3847
384b 384e 3850 3854 3856 3862 3864 3866
3867 386c 3870 3872 387e 3880 3884 3886
388a 388e 3892 3895 3896 3898 389c 389f
38a3 38a6 38a7 38a9 38aa 38af 38b3 38b7
38ba 38bb 38bd 38c0 38c3 38c4 38c9 38cd
38d0 38d4 38d6 38da 38de 38e1 38e2 38e4
38e7 38ea 38eb 38f0 38f3 38f7 38fb 38fe
38ff 3901 3904 3907 3908 390d 1 3910
3915 3919 391d 3921 3925 3929 392c 3930
3934 3937 393a 393f 3942 3945 3948 3949
394e 394f 1 3951 3956 395a 395e 3961
3966 396b 396e 3972 3973 3978 3979 397e
3980 3984 3987 398b 398f 3993 3996 399a
399c 39a0 39a3 39a7 39ab 39af 39b2 39b3
39b5 39b9 39bb 39bf 39c3 39c6 39ca 39cd
39ce 39d3 39d6 39da 39de 39e2 39e6 39ea
39f6 39fa 39fe 3a01 3a05 3a09 3a0b 3a0f
3a13 3a17 3a1a 3a1e 3a22 3a24 3a25 3a2a
1 3a2e 3a32 3a36 3a39 3a3d 3a41 3a45
3a49 3a4c 3a50 3a54 3a57 3a5a 3a5f 3a62
3a65 3a68 3a69 3a6e 3a6f 1 3a71 3a76
3a7a 3a7e 3a81 3a86 3a8b 3a8e 3a92 3a93
3a98 3a9b 3aa0 3aa1 3aa6 3aa9 3aae 3aaf
3ab4 3ab7 3abb 3abc 3ac1 3ac4 3ac9 3aca
3acf 3ad2 3ad6 3ada 3add 3ae0 3ae1 3ae3
3ae4 3ae9 3aea 3aef 3af1 3af5 3af8 3afa
3afe 3b00 3b0c 3b0e 3b10 3b11 3b16 3b1a
3b1c 3b28 3b2a 3b2e 3b32 3b35 3b39 3b3c
3b40 3b43 3b44 3b49 3b4c 3b50 3b54 3b57
3b58 3b5a 3b5e 3b61 3b65 3b68 3b69 3b6b
3b6c 3b71 1 3b74 3b79 3b7d 3b81 3b85
3b89 3b8d 3b90 3b94 3b98 3b9b 3b9e 3ba3
3ba6 3ba9 3bac 3bad 3bb2 3bb3 1 3bb5
3bba 3bbe 3bc2 3bc5 3bca 3bcf 3bd2 3bd6
3bd7 3bdc 3bdd 3be2 3be4 3be8 3beb 3bef
3bf3 3bf7 3bfa 3bfe 3c00 3c04 3c07 3c09
3c0d 3c10 3c14 3c17 3c1a 3c1b 3c20 3c24
3c28 3c2c 3c30 3c34 3c38 3c3c 3c40 3c44
3c48 3c4c 3c50 3c54 3c55 3c57 3c5b 3c5f
3c63 3c66 3c67 3c69 3c6c 3c6f 3c70 3c75
3c78 3c7c 3c7f 3c80 3c85 3c88 1 3c8b
3c90 3c94 3c98 3c9c 3ca0 3ca4 3ca8 3cac
3cb0 3cb4 3cb7 3cb8 3cba 3cbe 3cc0 3cc4
3cc7 3ccb 3cce 3ccf 3cd4 3cd7 3cdb 3cdf
3ce3 3ce7 3cea 3ceb 3ced 3cf1 3cf3 3cf7
3cfb 3cfe 3d01 3d02 3d04 3d07 3d0c 3d0d
3d12 3d16 3d1a 3d1e 3d22 3d25 3d26 3d28
3d2c 3d2e 3d32 3d36 3d3a 3d3e 3d41 3d42
3d44 3d48 3d4a 3d4e 3d52 3d55 3d57 3d5b
3d5f 3d62 3d66 3d69 3d6a 3d6f 3d72 3d76
3d7a 3d7e 3d82 3d86 3d8a 3d8e 3d92 3d96
3d9a 3da6 3daa 3dae 3db1 3db5 3db9 3dbb
3dbf 3dc3 3dc7 3dca 3dce 3dd2 3dd4 3dd5
3dda 1 3dde 3de2 3de6 3de9 3ded 3df1
3df5 3df9 3dfc 3e00 3e04 3e07 3e0a 3e0f
3e12 3e15 3e18 3e19 3e1e 3e1f 1 3e21
3e26 3e2a 3e2e 3e31 3e36 3e3b 3e3e 3e42
3e43 3e48 3e4b 3e50 3e51 3e56 3e59 3e5e
3e5f 3e64 3e67 3e6b 3e6c 3e71 3e74 3e79
3e7a 3e7f 3e82 3e86 3e8a 3e8d 3e90 3e91
3e93 3e94 3e99 3e9a 3e9f 3ea1 3ea5 3ea8
3eaa 3eae 3eb0 3ebc 3ebe 3ec0 3ec1 3ec6
3eca 3ecc 3ed8 3eda 3ede 3ee2 3ee6 3ee9
3eea 3eec 3eef 3ef3 3ef7 3efa 3efb 3efd
3efe 3f03 3f07 3f09 3f0d 3f10 3f14 3f17
3f1b 3f1f 3f22 3f26 3f2a 3f2e 3f32 3f36
3f3a 3f3e 3f42 3f46 3f49 3f4d 3f51 3f54
3f55 3f57 3f58 3f5a 3f5e 3f62 3f66 3f69
3f6a 3f6c 3f6f 3f72 3f73 3f78 3f7c 3f80
3f84 3f88 3f8c 3f8f 3f93 3f97 3f9a 3f9d
3fa2 3fa5 3fa8 3fab 3fac 3fb1 3fb2 1
3fb4 3fb9 3fbd 3fc1 3fc4 3fc9 3fce 3fd1
3fd5 3fd6 3fdb 3fdc 3fe1 3fe3 3fe7 3fea
3fee 3ff2 3ff6 3ff9 3ffd 3fff 4003 4007
400b 400f 4013 4017 401b 401f 4023 4027
402b 402f 4033 4037 403b 403f 4043 4047
4053 4057 405c 4060 4063 4064 4069 406d
4071 4074 4077 407b 407d 4081 4085 4088
408b 408f 4091 4095 4099 409c 409e 40a2
40a6 40aa 40ad 40b1 40b5 40b7 40b8 40bd
1 40c1 40c5 40c9 40cc 40d0 40d4 40d8
40dc 40df 40e3 40e7 40ea 40ed 40f2 40f5
40f8 40fb 40fc 4101 4102 1 4104 4109
410d 4111 4114 4119 411e 4121 4125 4126
412b 412e 4133 4134 4139 413c 4141 4142
4147 414a 414e 414f 4154 4157 415c 415d
4162 4165 4169 416d 4170 4173 4174 4176
4177 417c 417d 4182 4184 4188 418b 418d
4191 4193 419f 41a1 41a3 41a4 41a9 41ad
41af 41bb 41bd 41bf 41c3 41c7 41ca 41cc
41d0 41d4 41d7 41d9 41dd 41e1 41e4 41e8
41ec 41ef 41f3 41f7 41f8 41fa 41fe 4202
4206 4209 420d 4211 4214 4217 421c 421f
4222 4225 4226 422b 422c 1 422e 4233
4237 423b 423e 4243 4248 424b 424f 4250
4255 4258 425d 425e 4263 4266 426b 426c
4271 4274 4278 427c 427f 4280 4285 4286
428b 428d 4291 4294 4298 429c 42a0 42a3
42a7 42a9 42ad 42af 42bb 42bf 42c1 42c2
42cb 
12f9
2
0 1 a 6 15 19 20 :3 6
1c :3 6 12 :3 6 11 :3 6 14 18
1f :3 6 14 18 1f :2 6 1 3
a :2 1 4 :2 17 26 17 :2 4 :2 d
1c d :2 4 :2 c 1b c :2 4 9
12 11 :2 9 :2 4 14 1d 1c :2 14
:2 4 11 1a 19 :2 11 :2 4 e 17
16 :2 e :2 4 d 16 15 :2 d :2 4
:3 9 :2 4 :3 9 :2 4 :3 11 :2 4 :3 12 :2 4
:3 15 :2 4 :3 e :2 4 f 18 17 :2 f
:2 4 f 18 17 :2 f :2 4 11 1a
19 :2 11 :2 4 :2 f 1e f :2 4 :2 16
25 16 :2 4 :2 d 1c d :2 4 :2 13
22 13 :2 4 :2 14 23 14 :2 4 :2 12
21 12 :2 4 :2 11 20 11 :2 4 :2 f
1e f :2 4 :2 e 1d e :2 4 :2 10
1f 10 :2 4 :2 11 20 11 :2 4 :2 f
1e f :2 4 :2 e 1d e :2 4 :2 f
1e f :2 4 :2 e 1d e :2 4 :2 e
1d e :2 4 :2 f 1e f :2 4 :2 c
1b c :2 4 :2 b 1a b :2 4 :2 b
1a b :2 4 :2 13 22 13 :2 4 :2 10
1f 10 :2 4 :2 a 19 a :2 4 :2 11
20 11 :2 4 :2 10 1f 10 :2 4 :2 12
21 12 :2 4 c 10 f c 20
:2 29 c :2 4 d 11 10 d 21
:2 2a d :2 4 14 1d 1c :2 14 :2 4
:2 f 19 f :2 4 :2 f 19 f :2 4
:2 10 1a 10 :2 4 :2 10 1a 10 :2 4
:2 11 1b 11 :2 4 :2 11 1b 11 :2 4
:2 12 1c 12 :2 4 :2 12 1c 12 :2 4
:2 15 24 15 :2 4 :2 c 16 c :2 4
:2 d 1c d :2 4 :2 d 1c d :2 4
:2 d 1c d :2 4 :2 d 1c d 4
7 :2 11 23 :2 2d 3a :2 23 :3 7 :2 d
19 25 :2 2f 43 :2 25 56 5c 5f
:2 56 6d 85 8 :2 6d d 10 :2 6d
1c 1f :2 6d b e :2 6d 13 16
1f 27 :2 1f 3f :2 16 :2 6d b e
:2 6d 13 16 1f 27 :2 1f 35 :2 16
:2 6d b e :2 6d 13 16 1f 27
:2 1f 34 :2 16 :2 6d b e :2 6d 13
16 :2 6d 1b 1e :2 6d 29 2c :2 6d
b e :2 6d 13 16 :2 6d 1b 1e
:2 6d 29 2c :2 6d 31 34 :2 6d :2 7
4c :2 4 7 :2 11 23 :2 2d 39 3c
4e 50 51 :2 4e :2 23 :3 7 :2 d 12
1e 5f 62 :2 1e :2 7 54 :3 4 c
4 e 1a 27 36 40 50 5d
68 75 7 13 23 3a 45 4f
57 69 78 87 99 7 19 2c
34 3b 56 5e 65 81 8b 7
15 23 31 7 17 27 37 7
1a 23 c 1a 24 30 3c 4e
5d 6a 7 11 1d 2e 39 41
4a 52 7 15 24 2a 30 3c
7 13 20 2c 7 15 24 32
7 17 22 c d 21 :2 7 :2 10
1c 7 4 c a :2 13 1f a
1a :2 7 :2 c a :2 13 1f a d
:2 17 29 :2 33 3f 43 55 58 59
:2 55 :2 29 :3 d :2 13 18 24 66 69
:2 24 76 79 :2 24 7d 80 :2 24 8c
8f :2 24 97 9a :2 24 a6 a9 b0
b9 bc :2 a9 :2 24 :2 d 5c :2 a :2 7
:4 13 :2 7 4 :3 1 8 :2 11 1c 1a
:2 25 :2 1a 7 11 1d 28 36 43
56 f 1d 2a 34 43 53 f
10 1d :2 a :2 13 1f a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 58 5b 5c :2 58 :2 2c :3 10 :2 16
1b 27 69 6c :2 27 79 7c :2 27
80 83 :2 27 8f 92 :2 27 9a 9d
:2 27 a9 ac b3 bc bf :2 ac :2 27
:2 10 5f :2 d :2 a :4 16 :2 a 7 :3 35
b :2 14 1f 1d :2 28 :2 1d :2 a 18
a d :2 17 29 :2 33 3f 42 54
57 58 :2 54 :2 29 :3 d :2 13 18 24
66 69 :2 24 :2 d 5b :3 a 11 :2 1a
a 38 :3 7 15 7 11 :2 f 10
1d :2 a :2 13 1f a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
58 5b 5c :2 58 :2 2c :3 10 :2 16 1b
27 69 6c :2 27 79 7c :2 27 80
83 :2 27 8f 92 :2 27 9a 9d :2 27
a9 ac b3 bc bf :2 ac :2 27 :2 10
5f :2 d :2 a :4 16 :2 a 7 :3 35 a
13 1f :2 a 22 24 :2 22 14 :2 12
13 20 :2 d :2 16 22 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 5b 5e 5f :2 5b :2 2f :3 13 :2 19
1e 2a 6c 6f :2 2a 7c 7f :2 2a
83 86 :2 2a 92 95 :2 2a 9d a0
:2 2a ac af b6 bf c2 :2 af :2 2a
:2 13 62 :2 10 :2 d :4 19 :2 d a :3 26
d :2 18 1f 2d :2 d 32 34 :2 32
d 1b d 10 :2 1a 2c :2 36 42
45 57 5a 5b :2 57 :2 2c :3 10 :2 16
1b 27 69 6c :2 27 :2 10 5e :3 d
14 :2 1d d 36 :2 a 26 :2 7 11
:2 f 11 13 f 20 24 2f 10
13 20 e 11 1b e 11 25
27 e 10 25 27 e 10 1a
23 :2 a :2 13 1f a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
58 5b 5c :2 58 :2 2c :3 10 :2 16 1b
27 69 6c :2 27 79 7c :2 27 80
83 :2 27 8f 92 :2 27 9a 9d :2 27
a9 ac b3 bc bf :2 ac :2 27 :2 10
5f :2 d :2 a :4 16 :2 a 7 :3 35 a
13 1f :2 a 22 24 :2 22 a 18
a d :2 17 29 :2 33 3f 42 54
57 58 :2 54 :2 29 :3 d :2 13 18 24
66 69 :2 24 :2 d 5b :3 a 11 :2 1a
a 26 :2 7 a 13 1d :2 a 20
22 :2 20 d :2 17 29 :2 33 3f 42
54 57 58 :2 54 :2 29 :3 d :2 13 18
24 66 69 :2 24 76 79 :2 24 7d
80 :2 24 8a 8d 96 9e :2 a7 :2 96
b8 :2 8d :2 24 :2 d 5b :2 a d :2 17
29 :2 33 3f 42 54 57 58 :2 54
:2 29 :3 d :2 13 18 24 66 69 :2 24
:2 d 5b :3 a 11 :2 1c 2f :2 38 48
:2 11 a 7 24 d 16 20 :2 d
26 23 2f 3e :2 26 :2 23 a 1a
a d 16 20 :2 d 25 23 2e
38 :2 25 :2 23 d 1b d 10 :2 1a
2c :2 36 42 45 57 5a 5b :2 57
:2 2c :3 10 :2 16 1b 27 69 6c :2 27
:2 10 5e :3 d 14 :2 1d d a 3b
10 19 23 :2 10 28 26 31 3b
:2 28 :2 26 d 1d d 3e 3b :3 a
1a :2 a 18 2a 38 3e 4c 58
21 2f :2 58 :2 18 a 41 24 d
:2 17 29 :2 33 3f 42 54 57 58
:2 54 :2 29 :3 d :2 13 18 24 66 69
:2 24 76 79 :2 24 7d 80 :2 24 8a
8d 96 9e :2 a7 :2 96 b8 :2 8d :2 24
:2 d 5b :2 a d :2 17 29 :2 33 3f
42 54 57 58 :2 54 :2 29 :3 d :2 13
18 24 66 69 :2 24 :2 d 5b :3 a
11 :2 1c 2f :2 38 48 :2 11 a :4 7
35 a 13 1d :2 a 20 22 :2 20
a 18 a d :2 17 29 :2 33 3f
42 54 57 58 :2 54 :2 29 :3 d :2 13
18 24 66 69 :2 24 :2 d 5b :3 a
11 :2 1a a 24 :3 7 15 7 11
:2 f 10 1d :2 a :2 13 1f a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 58 5b 5c :2 58 :2 2c :3 10
:2 16 1b 27 69 6c :2 27 79 7c
:2 27 80 83 :2 27 8f 92 :2 27 9a
9d :2 27 a9 ac b3 bc bf :2 ac
:2 27 :2 10 5f :2 d :2 a :4 16 :2 a 7
:3 4 a 13 1f :2 a 22 24 :2 22
14 :2 12 13 20 :2 d :2 16 22 d
a 12 10 :2 19 25 10 20 :2 d
:2 12 10 :2 19 25 10 13 :2 1d 2f
:2 39 45 49 5b 5e 5f :2 5b :2 2f
:3 13 :2 19 1e 2a 6c 6f :2 2a 7c
7f :2 2a 83 86 :2 2a 92 95 :2 2a
9d a0 :2 2a ac af b6 bf c2
:2 af :2 2a :2 13 62 :2 10 :2 d :4 19 :2 d
a :3 26 d :2 18 1f 2d :2 d 32
34 :2 32 d 1b d 10 :2 1a 2c
:2 36 42 45 57 5a 5b :2 57 :2 2c
:3 10 :2 16 1b 27 69 6c :2 27 :2 10
5e :3 d 14 :2 1d d 36 :2 a 26
:2 7 11 :2 f 11 f 10 24 e
23 e 18 21 :2 a :2 13 1f a
7 f d :2 16 22 d 1d :2 a
:2 f d :2 16 22 d 10 :2 1a 2c
:2 36 42 46 58 5b 5c :2 58 :2 2c
:3 10 :2 16 1b 27 69 6c :2 27 79
7c :2 27 80 83 :2 27 8f 92 :2 27
9a 9d :2 27 a9 ac b3 bc bf
:2 ac :2 27 :2 10 5f :2 d :2 a :4 16 :2 a
7 :3 4 a 13 1f :2 a 22 24
:2 22 a 18 a d :2 17 29 :2 33
3f 42 54 57 58 :2 54 :2 29 :3 d
:2 13 18 24 66 69 :2 24 :2 d 5b
:3 a 11 :2 1a a 26 :2 7 11 1d
f 1d f 10 1d e 1b :2 a
:2 13 1f a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 58 5b
5c :2 58 :2 2c :3 10 :2 16 1b 27 69
6c :2 27 79 7c :2 27 80 83 :2 27
8f 92 :2 27 9a 9d :2 27 a9 ac
b3 bc bf :2 ac :2 27 :2 10 5f :2 d
:2 a :4 16 :2 a 7 :3 4 7 19 22
2c :2 19 7 a 13 1d :2 a 23
20 2c 3b :2 23 :2 20 a 1a a
d 16 24 :2 d 29 27 32 3c
:2 29 :2 27 d 1d 1e :2 1d :2 d 1b
32 40 4b 51 57 5d 28 34
42 55 :2 1b d 10 19 25 :2 10
28 2a :2 28 13 1c 29 :2 13 2f
2c 38 3f :2 2f :2 2c 16 :2 21 28
32 :2 16 3e 40 :2 3e 16 22 2b
39 :2 22 3c 3e 47 51 :2 3e :2 22
:2 16 28 16 19 22 30 :2 19 33
36 3f 4e :2 36 51 53 5c 63
:2 53 :2 36 35 :2 33 19 23 2c 3b
:2 23 3e 40 49 50 :2 40 :2 23 22
54 56 5f 6d :2 56 :2 22 19 23
20 1c 2a 33 3a 46 4f 1c
29 1c 2a 1c 22 1c 22 36
1c 23 1f 2b :3 29 1f :2 28 34
1f 30 1f :2 28 34 1f :4 1c 19
21 1f :2 28 34 1f 2f :2 1c :2 21
1f :2 28 34 1f 22 :2 2c 3e :2 48
54 58 6a 6d 6e :2 6a :2 3e :3 22
:2 28 2d 39 7b 7e :2 39 8b 8e
:2 39 92 95 :2 39 a1 a4 :2 39 ac
af :2 39 bb be c5 ce d1 :2 be
:2 39 :2 22 71 :2 1f :2 1c :4 28 :2 1c 19
:3 67 19 2b :2 34 3b 44 54 :2 3b
58 :2 61 :2 2b 19 67 23 20 1c
2a 33 3a 46 4f 1c 29 1c
2a :2 1c 22 36 1c 23 1f 2b
:3 29 1f :2 28 34 1f 30 1f :2 28
34 1f :4 1c 19 21 1f :2 28 34
1f 2f :2 1c :2 21 1f :2 28 34 1f
22 :2 2c 3e :2 48 54 58 6a 6d
6e :2 6a :2 3e :3 22 :2 28 2d 39 7b
7e :2 39 8b 8e :2 39 92 95 :2 39
a1 a4 :2 39 ac af :2 39 bb be
c5 ce d1 :2 be :2 39 :2 22 71 :2 1f
:2 1c :4 28 :2 1c 19 :7 16 42 19 22
30 :2 19 33 36 3f 4e :2 36 51
53 5c 63 :2 53 :2 36 35 :2 33 19
25 2e 35 :2 25 38 3a 43 4d
:2 3a :2 25 :2 19 2b 33 3c 46 :2 3c
55 :2 33 58 5a 63 6d :2 63 78
:2 5a :2 33 32 24 26 :2 32 31 30
32 :2 31 :2 2b 19 23 20 1c 2a
33 3a 46 4f 1c 29 1c 2a
22 36 1c 23 1f 2b :3 29 1f
:2 28 34 1f 30 1f :2 28 34 1f
:4 1c 19 21 1f :2 28 34 1f 2f
:2 1c :2 21 1f :2 28 34 1f 22 :2 2c
3e :2 48 54 58 6a 6d 6e :2 6a
:2 3e :3 22 :2 28 2d 39 7b 7e :2 39
8b 8e :2 39 92 95 :2 39 a1 a4
:2 39 ac af :2 39 bb be c5 ce
d1 :2 be :2 39 :2 22 71 :2 1f :2 1c :4 28
:2 1c 19 :4 67 19 25 2e 3c :2 25
3f 41 4a 54 :2 41 :2 25 :2 19 2b
19 23 20 1c 2a 33 3a 46
4f 1c 29 1c 2a :2 1c 22 36
1c 23 1f 2b :3 29 1f :2 28 34
1f 30 1f :2 28 34 1f :4 1c 19
21 1f :2 28 34 1f 2f :2 1c :2 21
1f :2 28 34 1f 22 :2 2c 3e :2 48
54 58 6a 6d 6e :2 6a :2 3e :3 22
:2 28 2d 39 7b 7e :2 39 8b 8e
:2 39 92 95 :2 39 a1 a4 :2 39 ac
af :2 39 bb be c5 ce d1 :2 be
:2 39 :2 22 71 :2 1f :2 1c :4 28 :2 1c 19
:7 16 :4 13 42 13 1f 28 36 :2 1f
39 3b 44 53 :2 3b :2 1f :2 13 25
13 1d 1a 16 24 2d 34 40
49 16 23 16 24 :2 16 1c 30
16 1d 19 25 :3 23 19 :2 22 2e
19 2a 19 :2 22 2e 19 :4 16 13
1b 19 :2 22 2e 19 29 :2 16 :2 1b
19 :2 22 2e 19 1c :2 26 38 :2 42
4e 52 64 67 68 :2 64 :2 38 :3 1c
:2 22 27 33 75 78 :2 33 85 88
:2 33 8c 8f :2 33 9b 9e :2 33 a6
a9 :2 33 b5 b8 bf c8 cb :2 b8
:2 33 :2 1c 6b :2 19 :2 16 :4 22 :2 16 13
:7 10 :4 14 :2 13 20 13 1d 21 :2 1b
1c 29 1a 28 :2 16 :2 1f 2b 16
13 1b 19 :2 22 2e 19 29 :2 16
:2 1b 19 :2 22 2e 19 1c :2 26 38
:2 42 4e 52 64 67 68 :2 64 :2 38
:3 1c :2 22 27 33 75 78 :2 33 85
88 :2 33 8c 8f :2 33 9b 9e :2 33
a6 a9 :2 33 b5 b8 bf c8 cb
:2 b8 :2 33 :2 1c 6b :2 19 :2 16 :4 22 :2 16
13 :3 32 :4 17 16 20 28 2f 50
5f :2 19 :2 22 2e 19 16 1e 1c
:2 25 31 1c 2c :2 19 :2 1e 1c :2 25
31 1c 1f :2 29 3b :2 45 51 55
67 6a 6b :2 67 :2 3b :3 1f :2 25 2a
36 78 7b :2 36 88 8b :2 36 8f
92 :2 36 9e a1 :2 36 a9 ac :2 36
b8 bb c2 cb ce :2 bb :2 36 :2 1f
6e :2 1c :2 19 :4 25 :2 19 16 :3 2e 19
:2 24 2c 3e 49 :2 19 4d 4f :2 4d
19 29 19 51 :2 16 13 2e 19
22 2d :2 19 30 32 :2 30 16 24
:2 2d 34 3d 48 :2 34 4c :2 55 :2 24
23 16 20 2b 38 40 47 62
6d 72 :2 1e :2 19 :2 22 2e 19 16
1e 1c :2 25 31 1c 2c :2 19 :2 1e
1c :2 25 31 1c 1f :2 29 3b :2 45
51 55 67 6a 6b :2 67 :2 3b :3 1f
:2 25 2a 36 78 7b :2 36 88 8b
:2 36 8f 92 :2 36 9e a1 :2 36 a9
ac :2 36 b8 bb c2 cb ce :2 bb
:2 36 :2 1f 6e :2 1c :2 19 :4 25 :2 19 16
:4 34 2e 20 2b 38 40 47 62
6d 72 :2 1e :2 19 :2 22 2e 19 16
1e 1c :2 25 31 1c 2c :2 19 :2 1e
1c :2 25 31 1c 1f :2 29 3b :2 45
51 55 67 6a 6b :2 67 :2 3b :3 1f
:2 25 2a 36 78 7b :2 36 88 8b
:2 36 8f 92 :2 36 9e a1 :2 36 a9
ac :2 36 b8 bb c2 cb ce :2 bb
:2 36 :2 1f 6e :2 1c :2 19 :4 25 :2 19 16
:7 13 32 :2 10 :4 14 :2 13 21 31 3f
45 :2 21 13 31 16 1d 29 2c
:2 16 2f 31 :2 2f 16 24 34 42
48 :2 24 16 35 16 24 34 42
48 :2 24 16 :4 13 :5 10 1e :2 10 1e
2c 32 40 4a 56 61 6d 1f
28 39 :2 1f :2 1e 10 13 1c 28
:2 13 2b 2e :2 2b 13 21 13 16
:2 20 32 :2 3c 48 4b 5d 60 61
:2 5d :2 32 :3 16 :2 1c 21 2d 6f 72
:2 2d :2 16 64 :3 13 1a 23 2f :2 1a
13 30 :3 10 1c 10 2c :2 d 3f
d 1f 27 30 3a :2 30 49 :2 27
4c 4e 57 61 :2 57 6c :2 4e :2 27
26 24 :3 26 25 30 32 :2 25 :2 1f
d :5 a 18 2c 3a 45 4f 55
25 31 3b 49 25 34 44 50
25 31 3e 4c 25 33 42 55
25 31 3d 46 51 :2 3d :2 18 :2 a
18 a d :2 17 29 :2 33 3f 42
54 57 58 :2 54 :2 29 :3 d :2 13 18
24 66 69 :2 24 :2 d 5b :3 a 11
1a 26 :2 11 a 7 3e d 16
2a :2 d 30 2d 39 49 :2 30 :2 2d
14 11 25 d 1b 24 2b 37
40 13 27 d 14 10 1c :3 1a
10 :2 19 25 10 21 10 :2 19 25
10 :4 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 5b 5e 5f
:2 5b :2 2f :3 13 :2 19 1e 2a 6c 6f
:2 2a 7c 7f :2 2a 83 86 :2 2a 92
95 :2 2a 9d a0 :2 2a ac af b6
bf c2 :2 af :2 2a :2 13 62 :2 10 :2 d
:4 19 :2 d a :3 4c 14 11 1f 28
2f 3b 44 13 20 11 1e d
14 10 1c :3 1a 10 :2 19 25 10
21 10 :2 19 25 10 :4 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 5b 5e 5f :2 5b :2 2f :3 13 :2 19
1e 2a 6c 6f :2 2a 7c 7f :2 2a
83 86 :2 2a 92 95 :2 2a 9d a0
:2 2a ac af b6 bf c2 :2 af :2 2a
:2 13 62 :2 10 :2 d :4 19 :2 d a :3 4c
7 4c 3e d 16 1f :2 d 25
22 2e 3f :2 25 :2 22 d 16 1f
:2 d 22 24 :2 22 d 20 d 26
11 1a 23 :2 11 26 28 :2 26 10
2f 38 41 :2 2f 44 46 :2 44 2e
:3 10 1e 10 13 :2 1d 2f :2 39 45
48 5a 5d 5e :2 5a :2 2f :3 13 :2 19
1e 2a 6c 6f :2 2a :2 13 61 :3 10
17 :2 20 10 51 :3 d 20 29 32
:2 20 d :4 a :4 e d 17 :2 15 16
2a :2 10 :2 19 25 10 d 15 13
:2 1c 28 13 23 :2 10 :2 15 13 :2 1c
28 13 16 :2 20 32 :2 3c 48 4c
5e 61 62 :2 5e :2 32 :3 16 :2 1c 21
2d 6f 72 :2 2d 7f 82 :2 2d 86
89 :2 2d 95 98 :2 2d a0 a3 :2 2d
af b2 b9 c2 c5 :2 b2 :2 2d :2 16
65 :2 13 :2 10 :4 1c :2 10 d :3 2b 11
:2 1a 25 23 :2 2e :2 23 10 43 4c
5d :2 43 62 60 13 1f :2 62 :2 60
42 :3 10 1e 10 13 :2 1d 2f :2 39
45 48 5a 5d 5e :2 5a :2 2f :3 13
:2 19 1e 2a 6c 6f :2 2a :2 13 61
:3 10 17 :2 20 10 23 :2 d 2b :3 a
1a 1b :2 1a :2 a 18 2f 3d 48
4e 54 5a 28 34 42 55 :2 18
a e 17 23 :2 e 26 28 :2 26
d :4 31 30 2f :3 d 1b 30 3e
49 53 64 26 38 44 :2 1b d
50 :2 a :4 e :2 d 1b 2b 39 3f
:2 1b d 2b 10 17 23 26 :2 10
29 2b :2 29 10 1e 2e 3c 42
:2 1e 10 2f 10 1e 2e 3c 42
:2 1e 10 :4 d :4 a :4 e d 17 22
2f 37 3e 59 64 69 :2 15 :2 10
:2 19 25 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 5e 61
62 :2 5e :2 32 :3 16 :2 1c 21 2d 6f
72 :2 2d 7f 82 :2 2d 86 89 :2 2d
95 98 :2 2d a0 a3 :2 2d af b2
b9 c2 c5 :2 b2 :2 2d :2 16 65 :2 13
:2 10 :4 1c :2 10 d :3 2c d 19 22
2c :2 19 2f 31 3a 49 :2 31 :2 19
d 2c :3 a 17 :2 a 18 :2 a 18
26 2c 3a 44 50 5b 67 1f
28 39 :2 1f :2 18 a d 16 22
:2 d 25 28 :2 25 d 1b d 10
:2 1a 2c :2 36 42 45 57 5a 5b
:2 57 :2 2c :3 10 :2 16 1b 27 69 6c
:2 27 :2 10 5e :3 d 14 :2 1d d 2a
17 14 10 1c :5 10 1e 27 2e
3b 44 51 5a 16 2a 10 17
13 1f :3 1d 13 :2 1c 28 13 24
13 :2 1c 28 13 :4 10 d 15 13
:2 1c 28 13 23 :2 10 :2 15 13 :2 1c
28 13 16 :2 20 32 :2 3c 48 4c
5e 61 62 :2 5e :2 32 :3 16 :2 1c 21
2d 6f 72 :2 2d 7f 82 :2 2d 86
89 :2 2d 95 98 :2 2d a0 a3 :2 2d
af b2 b9 c2 c5 :2 b2 :2 2d :2 16
65 :2 13 :2 10 :4 1c :2 10 d :7 a 42
3e :2 7 :5 4 :2 d 19 23 :2 19 4
7 :2 11 23 :2 2d 39 3c 4e 51
52 :2 4e :2 23 :3 7 :2 d 12 1e 60
63 :2 1e 70 73 :2 1e 77 7a :2 1e
84 87 :2 90 :2 1e :2 7 55 :3 4 b
:2 14 4 :9 1 
12f9
4
0 :2 1 :6 3 :4 4
:4 5 :4 6 :6 7 :6 8
2 :2 9 :2 1 :6 b
:6 c :6 d :7 e :7 f
:7 10 :7 11 :7 12 :5 13
:5 14 :5 15 :5 16 :5 17
:5 18 :7 19 :7 1a :7 1b
:6 1c :6 1d :6 1e :6 1f
:6 20 :6 21 :6 22 :6 23
:6 24 :6 25 :6 26 :6 27
:6 28 :6 29 :6 2a :6 2b
:6 2c :6 2d :6 2e :6 2f
:6 30 :6 31 :6 32 :6 33
:6 34 :6 35 :a 36 :a 37
:7 38 :6 39 :6 3a :6 3b
:6 3c :6 3d :6 3e :6 3f
:6 40 :6 41 :6 42 :6 43
:6 44 :6 45 :6 46 :b 48
:11 49 4a :2 49 :2 4a
:2 49 :2 4a :2 49 :2 4b
:2 49 :9 4b :2 49 :2 4c
:2 49 :9 4c :2 49 :2 4d
:2 49 :9 4d :2 49 :2 4e
:2 49 :2 4e :2 49 :2 4e
:2 49 :2 4e :2 49 :2 4f
:2 49 :2 4f :2 49 :2 4f
:2 49 :2 4f :2 49 :2 4f
:4 49 :3 48 :11 52 :b 53
:3 52 :3 56 :9 59 :b 5a
:a 5b :4 5c :4 5d :3 5e
:8 5f :8 60 :6 61 :4 62
:4 63 :3 64 65 :2 66
59 :5 67 58 69
:5 6a :3 69 :2 6b :5 6d
:11 6e :24 6f :3 6e :2 6c
:6 6b 68 :3 47 :a 74
:6 76 :6 77 78 :2 79
76 :5 7a 75 7c
:5 7d :3 7c :2 7e :5 80
:11 81 :24 82 :3 81 :2 7f
:6 7e 7b :3 74 :a 87
:3 88 :11 89 :b 8a :3 89
:5 8c :3 87 :3 8f 91
92 93 :2 94 91
:5 95 90 97 :5 98
:3 97 :2 99 :5 9b :11 9c
:24 9d :3 9c :2 9a :6 99
96 :3 74 :9 a2 a4
a5 a6 :2 a7 a4
:5 a8 a3 aa :5 ab
:3 aa :2 ac :5 ae :11 af
:24 b0 :3 af :2 ad :6 ac
a9 :3 a2 :b b5 :3 b6
:11 b7 :b b8 :3 b7 :5 ba
:3 b5 :3 a2 c0 c1
c2 :2 c4 :4 c5 :3 c6
:3 c7 :4 c8 :4 c9 :4 ca
c0 :5 cb bf cd
:5 ce :3 cd :2 cf :5 d1
:11 d2 :24 d3 :3 d2 :2 d0
:6 cf cc :3 74 :9 d8
:3 d9 :11 da :b db :3 da
:5 dd :3 d8 :9 e0 :11 e1
:20 e2 :3 e1 :11 e4 :b e5
:3 e4 :b e7 e8 e0
:d e8 :3 e9 :d ea :3 eb
:11 ec :b ed :3 ec :5 ef
f0 ea :d f0 :3 f1
f0 :3 ea :3 f4 :7 f5
:2 f6 :5 f5 e8 e0
:11 f8 :20 f9 :3 f8 :11 fb
:b fc :3 fb :b fe f7
:3 e0 74 :9 101 :3 102
:11 103 :b 104 :3 103 :5 106
:3 101 :3 109 10b 10c
10d :2 10e 10b :5 10f
10a 111 :5 112 :3 111
:2 113 :5 115 :11 116 :24 117
:3 116 :2 114 :6 113 110
:3 100 :9 11c 11e 11f
120 :2 121 11e :5 122
11d 124 :5 125 :3 124
:2 126 :5 128 :11 129 :24 12a
:3 129 :2 127 :6 126 123
:3 11c :b 12f :3 130 :11 131
:b 132 :3 131 :5 134 :3 12f
:3 11c 13a 13b 13c
13e 13f :2 140 :2 141
:3 142 13a :5 143 139
145 :5 146 :3 145 :2 147
:5 149 :11 14a :24 14b :3 14a
:2 148 :6 147 144 :3 100
:9 150 :3 151 :11 152 :b 153
:3 152 :5 155 :3 150 :2 159
:2 15a 15b :2 15c :2 15d
159 :5 15e 158 160
:5 161 :3 160 :2 162 :5 164
:11 165 :24 166 :3 165 :2 163
:6 162 15f :3 100 :7 16b
:d 16c :3 16d :d 16f :6 170
:8 171 :4 172 :3 171 :9 174
:d 175 :b 176 :f 177 :3 178
:16 17a :18 17b 17d 17e
:6 17f :2 180 :2 181 :2 182
183 :2 184 17d :6 185
:5 186 185 :5 188 187
:3 185 17c 18b :5 18c
:3 18b :2 18d :5 18f :11 190
:24 191 :3 190 :2 18e :6 18d
18a :3 17a :f 196 17a
199 19a :6 19b :2 19c
:2 19d 19e 19f :2 1a0
199 :6 1a1 :5 1a2 1a1
:5 1a4 1a3 :3 1a1 198
1a7 :5 1a8 :3 1a7 :2 1a9
:5 1ab :11 1ac :24 1ad :3 1ac
:2 1aa :6 1a9 1a6 :4 197
:3 17a 176 :16 1b3 :f 1b4
:16 1b5 :2 1b6 :3 1b5 :2 1b6
:5 1b5 1b9 1ba :6 1bb
:2 1bc :2 1bd :2 1be 1b9
:6 1bf :5 1c0 1bf :5 1c2
1c1 :3 1bf 1b8 1c5
:5 1c6 :3 1c5 :2 1c7 :5 1c9
:11 1ca :24 1cb :3 1ca :2 1c8
:6 1c7 1c4 :4 1b3 :f 1d0
:3 1d1 1d4 1d5 :6 1d6
:2 1d7 :2 1d8 1d9 1da
:2 1db 1d4 :6 1dc :5 1dd
1dc :5 1df 1de :3 1dc
1d3 1e2 :5 1e3 :3 1e2
:2 1e4 :5 1e6 :11 1e7 :24 1e8
:3 1e7 :2 1e5 :6 1e4 1e1
:4 1cf :3 1b3 1b2 :3 176
175 :f 1ef :3 1f0 1f3
1f4 :6 1f5 :2 1f6 :2 1f7
1f8 1f9 :2 1fa 1f3
:6 1fb :5 1fc 1fb :5 1fe
1fd :3 1fb 1f2 201
:5 202 :3 201 :2 203 :5 205
:11 206 :24 207 :3 206 :2 204
:6 203 200 :4 1ee :3 175
:5 20d :3 20e :2 210 211
212 :2 213 :2 214 210
:5 215 20f 217 :5 218
:3 217 :2 219 :5 21b :11 21c
:24 21d :3 21c :2 21a :6 219
216 :3 20d :5 223 :6 225
:5 226 224 228 :5 229
:3 228 :2 22a :5 22c :11 22d
:24 22e :3 22d :2 22b :6 22a
227 :3 223 :c 233 :3 234
:3 233 236 223 :9 236
:10 237 :8 23a 23b 23c
23a :5 23d 239 23f
:5 240 :3 23f :2 241 :5 243
:11 244 :24 245 :3 244 :2 242
:6 241 23e :4 236 223
:8 24b 24c 24d 24b
:5 24e 24a 250 :5 251
:3 250 :2 252 :5 254 :11 255
:24 256 :3 255 :2 253 :6 252
24f :4 249 :3 223 :3 20d
:5 25f :8 260 25f :a 262
:8 263 262 :8 265 264
:3 262 261 :3 25f :3 269
:9 26a :5 26b :3 26a :9 26d
:3 26e :11 26f :b 270 :3 26f
:7 272 :3 26d :3 275 :3 174
16f :16 279 :2 27a :3 279
:2 27a :5 279 277 :3 16f
:7 280 :4 281 :4 282 :4 283
:4 284 :7 285 :3 280 :3 288
:11 289 :b 28a :3 289 :7 28c
28d 16c :d 28d 2b0
:2 2b1 :6 2b2 :2 2b3 2b0
:6 2b4 :5 2b5 2b4 :5 2b7
2b6 :3 2b4 2af 2ba
:5 2bb :3 2ba :2 2bc :5 2be
:11 2bf :24 2c0 :3 2bf :2 2bd
:6 2bc 2b9 :3 28d 2cb
:6 2cc :2 2cd :2 2ce 2cb
:6 2cf :5 2d0 2cf :5 2d2
2d1 :3 2cf 2ca 2d5
:5 2d6 :3 2d5 :2 2d7 :5 2d9
:11 2da :24 2db :3 2da :2 2d8
:6 2d7 2d4 :3 28d 2e1
28d 16c :d 2e1 :9 2e2
:3 2e3 2e2 :16 2e5 :3 2e6
:11 2e7 :b 2e8 :3 2e7 :5 2ea
:3 2e5 :7 2ed 2e4 :3 2e2
:5 2f0 2f2 2f3 2f4
:2 2f5 2f2 :5 2f6 2f1
2f8 :5 2f9 :3 2f8 :2 2fa
:5 2fc :11 2fd :24 2fe :3 2fd
:2 2fb :6 2fa 2f7 :3 2f0
:11 304 :2 305 :7 304 :3 306
:11 307 :b 308 :3 307 :5 30a
305 :2 304 :3 2f0 :6 347
:8 348 :4 349 :3 348 :12 34b
:7 34c :3 34d :3 34c :3 34b
:5 351 :8 352 351 :a 354
:8 355 354 :8 357 356
:3 354 353 :3 351 :5 35b
:8 35d 35e 35f 35d
:5 360 35c 362 :5 363
:3 362 :2 364 :5 366 :11 367
:24 368 :3 367 :2 365 :6 364
361 :3 35b :f 36d :3 35b
:3 372 :3 373 :9 374 :5 375
:3 374 :9 378 :3 379 :11 37a
:b 37b :3 37a :5 37d 378
380 381 :2 382 383
384 385 386 :8 387
:2 388 380 :6 389 :5 38a
389 :5 38c 38b :3 389
37f 38f :5 390 :3 38f
:2 391 :5 393 :11 394 :24 395
:3 394 :2 392 :6 391 38e
:4 37e :3 378 2e1 :3 16c
100 :3 74 :8 39f :11 3a6
:19 3a7 :3 3a6 :5 3a9 :2 47
:7 1 
42cd
4
:3 0 1 :3 0 2
:a 0 12f4 1 :7 0
5 37 0 3
4 :3 0 5 :3 0
6 :3 0 3 :5 0
1 7 6 :3 0
9 5d 0 7
8 :3 0 7 :7 0
b a :3 0 8
:3 0 9 :7 0 f
e :3 0 d 8a
0 b 8 :3 0
a :7 0 13 12
:3 0 4 :3 0 5
:3 0 6 :3 0 b
:5 0 1 19 18
:3 0 16 b6 0
f 4 :3 0 5
:3 0 6 :3 0 c
:5 0 1 1f 1e
:3 0 d :3 0 8
:3 0 21 23 0
12f4 2 24 :2 0
1a f2 0 18
8 :3 0 27 :7 0
7 :3 0 2b 28
29 12f2 0 e
:6 0 8 :3 0 2d
:7 0 9 :3 0 31
2e 2f 12f2 0
f :6 0 12 :2 0
1e 8 :3 0 33
:7 0 a :3 0 37
34 35 12f2 0
10 :6 0 6 :3 0
12 :2 0 1c 39
3b :6 0 3e 3c
0 12f2 0 11
:6 0 12 :2 0 22
6 :3 0 20 40
42 :6 0 45 43
0 12f2 0 13
:6 0 12 :2 0 26
6 :3 0 24 47
49 :6 0 4c 4a
0 12f2 0 14
:6 0 12 :2 0 2a
6 :3 0 28 4e
50 :6 0 53 51
0 12f2 0 15
:6 0 30 1a2 0
2e 6 :3 0 2c
55 57 :6 0 5a
58 0 12f2 0
16 :6 0 34 1d6
0 32 18 :3 0
5c :7 0 5f 5d
0 12f2 0 17
:6 0 18 :3 0 61
:7 0 64 62 0
12f2 0 19 :6 0
38 20a 0 36
1b :3 0 66 :7 0
69 67 0 12f2
0 1a :6 0 1b
:3 0 6b :7 0 6e
6c 0 12f2 0
1c :6 0 20 :2 0
3a 18 :3 0 70
:7 0 73 71 0
12f2 0 1d :6 0
18 :3 0 75 :7 0
78 76 0 12f2
0 1e :6 0 20
:2 0 3e 6 :3 0
3c 7a 7c :6 0
7f 7d 0 12f2
0 1f :6 0 20
:2 0 42 6 :3 0
40 81 83 :6 0
86 84 0 12f2
0 21 :6 0 24
:2 0 46 6 :3 0
44 88 8a :6 0
8d 8b 0 12f2
0 22 :6 0 24
:2 0 48 8 :3 0
8f :7 0 93 90
91 12f2 0 23
:6 0 24 :2 0 4a
8 :3 0 95 :7 0
99 96 97 12f2
0 25 :6 0 24
:2 0 4c 8 :3 0
9b :7 0 9f 9c
9d 12f2 0 26
:6 0 24 :2 0 4e
8 :3 0 a1 :7 0
a5 a2 a3 12f2
0 27 :6 0 24
:2 0 50 8 :3 0
a7 :7 0 ab a8
a9 12f2 0 28
:6 0 24 :2 0 52
8 :3 0 ad :7 0
b1 ae af 12f2
0 29 :6 0 24
:2 0 54 8 :3 0
b3 :7 0 b7 b4
b5 12f2 0 2a
:6 0 24 :2 0 56
8 :3 0 b9 :7 0
bd ba bb 12f2
0 2b :6 0 24
:2 0 58 8 :3 0
bf :7 0 c3 c0
c1 12f2 0 2c
:6 0 24 :2 0 5a
8 :3 0 c5 :7 0
c9 c6 c7 12f2
0 2d :6 0 24
:2 0 5c 8 :3 0
cb :7 0 cf cc
cd 12f2 0 2e
:6 0 24 :2 0 5e
8 :3 0 d1 :7 0
d5 d2 d3 12f2
0 2f :6 0 24
:2 0 60 8 :3 0
d7 :7 0 db d8
d9 12f2 0 30
:6 0 24 :2 0 62
8 :3 0 dd :7 0
e1 de df 12f2
0 31 :6 0 24
:2 0 64 8 :3 0
e3 :7 0 e7 e4
e5 12f2 0 32
:6 0 24 :2 0 66
8 :3 0 e9 :7 0
ed ea eb 12f2
0 33 :6 0 24
:2 0 68 8 :3 0
ef :7 0 f3 f0
f1 12f2 0 34
:6 0 24 :2 0 6a
8 :3 0 f5 :7 0
f9 f6 f7 12f2
0 35 :6 0 24
:2 0 6c 8 :3 0
fb :7 0 ff fc
fd 12f2 0 36
:6 0 24 :2 0 6e
8 :3 0 101 :7 0
105 102 103 12f2
0 37 :6 0 24
:2 0 70 8 :3 0
107 :7 0 10b 108
109 12f2 0 38
:6 0 24 :2 0 72
8 :3 0 10d :7 0
111 10e 10f 12f2
0 39 :6 0 24
:2 0 74 8 :3 0
113 :7 0 117 114
115 12f2 0 3a
:6 0 24 :2 0 76
8 :3 0 119 :7 0
11d 11a 11b 12f2
0 3b :6 0 24
:2 0 78 8 :3 0
11f :7 0 123 120
121 12f2 0 3c
:6 0 40 :2 0 7a
8 :3 0 125 :7 0
129 126 127 12f2
0 3d :6 0 40
:2 0 7e 3f :3 0
7c 12b 12d 2
:5 0 41 :3 0 42
:3 0 12f 130 0
133 12e 131 12f2
0 3e :6 0 45
:2 0 82 3f :3 0
80 135 137 2
:5 0 41 :3 0 42
:3 0 139 13a 0
13d 138 13b 12f2
0 43 :6 0 24
:2 0 86 6 :3 0
84 13f 141 :6 0
144 142 0 12f2
0 44 :6 0 24
:2 0 88 47 :3 0
146 :7 0 14a 147
148 12f2 0 46
:6 0 24 :2 0 8a
47 :3 0 14c :7 0
150 14d 14e 12f2
0 48 :6 0 24
:2 0 8c 47 :3 0
152 :7 0 156 153
154 12f2 0 49
:6 0 24 :2 0 8e
47 :3 0 158 :7 0
15c 159 15a 12f2
0 4a :6 0 24
:2 0 90 47 :3 0
15e :7 0 162 15f
160 12f2 0 4b
:6 0 24 :2 0 92
47 :3 0 164 :7 0
168 165 166 12f2
0 4c :6 0 24
:2 0 94 47 :3 0
16a :7 0 16e 16b
16c 12f2 0 4d
:6 0 24 :2 0 96
47 :3 0 170 :7 0
174 171 172 12f2
0 4e :6 0 24
:2 0 98 8 :3 0
176 :7 0 17a 177
178 12f2 0 4f
:6 0 24 :2 0 9a
47 :3 0 17c :7 0
180 17d 17e 12f2
0 50 :6 0 24
:2 0 9c 8 :3 0
182 :7 0 186 183
184 12f2 0 51
:6 0 24 :2 0 9e
8 :3 0 188 :7 0
18c 189 18a 12f2
0 52 :6 0 24
:2 0 a0 8 :3 0
18e :7 0 192 18f
190 12f2 0 53
:6 0 199 19a 0
a2 8 :3 0 194
:7 0 198 195 196
12f2 0 54 :6 0
55 :3 0 56 :3 0
55 :3 0 57 :3 0
19c 19d 0 2
:4 0 a4 19e 1a0
19b 1a2 1a1 :2 0
58 :3 0 59 :3 0
1a4 1a5 0 5a
:4 0 55 :3 0 5b
:3 0 1a8 1a9 0
2 :4 0 a6 1aa
1ac 5c :4 0 5d
:2 0 5e :3 0 a8
1af 1b1 :3 0 5f
:4 0 5d :2 0 60
:4 0 ab 1b4 1b6
:3 0 5d :2 0 3
:3 0 ae 1b8 1ba
:3 0 5d :2 0 60
:4 0 b1 1bc 1be
:3 0 5d :2 0 61
:4 0 b4 1c0 1c2
:3 0 5d :2 0 62
:3 0 63 :3 0 7
:3 0 b7 1c6 1c8
64 :4 0 b9 1c5
1cb bc 1c4 1cd
:3 0 5d :2 0 61
:4 0 bf 1cf 1d1
:3 0 5d :2 0 62
:3 0 63 :3 0 9
:3 0 c2 1d5 1d7
64 :4 0 c4 1d4
1da c7 1d3 1dc
:3 0 5d :2 0 61
:4 0 ca 1de 1e0
:3 0 5d :2 0 62
:3 0 63 :3 0 a
:3 0 cd 1e4 1e6
64 :4 0 cf 1e3
1e9 d2 1e2 1eb
:3 0 5d :2 0 61
:4 0 d5 1ed 1ef
:3 0 5d :2 0 60
:4 0 d8 1f1 1f3
:3 0 5d :2 0 b
:3 0 db 1f5 1f7
:3 0 5d :2 0 60
:4 0 de 1f9 1fb
:3 0 5d :2 0 61
:4 0 e1 1fd 1ff
:3 0 5d :2 0 60
:4 0 e4 201 203
:3 0 5d :2 0 c
:3 0 e7 205 207
:3 0 5d :2 0 60
:4 0 ea 209 20b
:3 0 5d :2 0 65
:4 0 ed 20d 20f
:3 0 f0 1a6 211
:2 0 213 f5 214
1a3 213 0 215
f7 0 12f0 55
:3 0 56 :3 0 216
217 0 55 :3 0
66 :3 0 219 21a
0 20 :2 0 2
:4 0 67 :2 0 68
:2 0 20 :2 0 f9
21f 221 :3 0 fc
21b 223 218 225
224 :2 0 58 :3 0
69 :3 0 227 228
0 5a :4 0 6a
:4 0 5d :2 0 5e
:3 0 100 22c 22e
:3 0 103 229 230
:2 0 232 106 233
226 232 0 234
108 0 12f0 11
:3 0 3 :3 0 235
236 0 12f0 6b
:3 0 6c :3 0 6d
:3 0 6e :3 0 6f
:3 0 70 :3 0 71
:3 0 70 :3 0 72
:3 0 71 :3 0 73
:3 0 74 :3 0 75
:3 0 76 :3 0 77
:3 0 78 :3 0 79
:3 0 7a :3 0 79
:3 0 7a :3 0 7b
:3 0 7c :3 0 7d
:3 0 7e :3 0 7b
:3 0 7d :3 0 7e
:3 0 7c :3 0 7f
:3 0 80 :3 0 81
:3 0 82 :3 0 83
:3 0 84 :3 0 85
:3 0 86 :3 0 87
:3 0 88 :3 0 89
:3 0 8a :3 0 8b
:3 0 14 :3 0 16
:3 0 21 :3 0 1f
:3 0 1d :3 0 29
:3 0 3c :3 0 3a
:3 0 26 :3 0 2b
:3 0 13 :3 0 32
:3 0 37 :3 0 35
:3 0 36 :3 0 28
:3 0 1a :3 0 1c
:3 0 17 :3 0 19
:3 0 23 :3 0 25
:3 0 46 :3 0 49
:3 0 48 :3 0 4a
:3 0 4b :3 0 4d
:3 0 4c :3 0 4e
:3 0 27 :3 0 15
:3 0 3d :3 0 8c
:3 0 8d :3 0 11
:4 0 8e 1 :8 0
28b 8f :3 0 90
:3 0 286 287 0
91 :3 0 288 289
0 28b 10a 2de
92 :3 0 8f :3 0
90 :3 0 28d 28e
0 91 :3 0 28f
290 0 292 10d
294 10f 293 292
:2 0 2dc 93 :3 0
8f :3 0 90 :3 0
297 298 0 91
:3 0 299 29a 0
2d4 55 :3 0 56
:3 0 29c 29d 0
55 :3 0 66 :3 0
29f 2a0 0 94
:2 0 2 :4 0 95
:2 0 68 :2 0 20
:2 0 111 2a5 2a7
:3 0 114 2a1 2a9
29e 2ab 2aa :2 0
58 :3 0 69 :3 0
2ad 2ae 0 5a
:4 0 96 :4 0 5d
:2 0 5e :3 0 118
2b2 2b4 :3 0 5d
:2 0 97 :4 0 11b
2b6 2b8 :3 0 5d
:2 0 98 :4 0 11e
2ba 2bc :3 0 5d
:2 0 91 :3 0 121
2be 2c0 :3 0 5d
:2 0 99 :4 0 124
2c2 2c4 :3 0 5d
:2 0 7e :3 0 9a
:3 0 20 :2 0 9b
:2 0 127 2c7 2cb
12b 2c6 2cd :3 0
12e 2af 2cf :2 0
2d1 131 2d2 2ac
2d1 0 2d3 133
0 2d4 135 2d7
:3 0 2d7 0 2d7
2d6 2d4 2d5 :6 0
2d9 2 :3 0 138
2db 13a 2da 2d9
:2 0 2dc 13c :2 0
2de 0 2de 2dd
28b 2dc :6 0 12f0
1 :3 0 8f :3 0
90 :3 0 2e0 2e1
0 8f :3 0 9c
:2 0 9d :3 0 2e3
2e5 0 141 2e4
2e7 :3 0 2e8 :2 0
6b :3 0 9e :3 0
9f :3 0 70 :3 0
89 :3 0 a0 :3 0
14 :3 0 2d :3 0
26 :3 0 29 :3 0
27 :3 0 31 :3 0
a1 :3 0 a2 :3 0
11 :4 0 a3 1
:8 0 2ff 8f :3 0
90 :3 0 2fa 2fb
0 91 :3 0 2fc
2fd 0 2ff 144
352 92 :3 0 8f
:3 0 90 :3 0 301
302 0 91 :3 0
303 304 0 306
147 308 149 307
306 :2 0 350 93
:3 0 8f :3 0 90
:3 0 30b 30c 0
91 :3 0 30d 30e
0 348 55 :3 0
56 :3 0 310 311
0 55 :3 0 66
:3 0 313 314 0
94 :2 0 2 :4 0
a4 :2 0 68 :2 0
20 :2 0 14b 319
31b :3 0 14e 315
31d 312 31f 31e
:2 0 58 :3 0 69
:3 0 321 322 0
5a :4 0 a5 :4 0
5d :2 0 5e :3 0
152 326 328 :3 0
5d :2 0 97 :4 0
155 32a 32c :3 0
5d :2 0 98 :4 0
158 32e 330 :3 0
5d :2 0 91 :3 0
15b 332 334 :3 0
5d :2 0 99 :4 0
15e 336 338 :3 0
5d :2 0 7e :3 0
9a :3 0 20 :2 0
9b :2 0 161 33b
33f 165 33a 341
:3 0 168 323 343
:2 0 345 16b 346
320 345 0 347
16d 0 348 16f
34b :3 0 34b 0
34b 34a 348 349
:6 0 34d 4 :3 0
172 34f 174 34e
34d :2 0 350 176
:2 0 352 0 352
351 2ff 350 :6 0
66d 1 :3 0 8f
:3 0 90 :3 0 354
355 0 8f :3 0
9c :2 0 9d :3 0
357 359 0 17b
358 35b :3 0 35c
:2 0 c :3 0 11
:3 0 35e 35f 0
385 55 :3 0 56
:3 0 361 362 0
55 :3 0 66 :3 0
364 365 0 20
:2 0 2 :4 0 a6
:2 0 68 :2 0 20
:2 0 17e 36a 36c
:3 0 181 366 36e
363 370 36f :2 0
58 :3 0 69 :3 0
372 373 0 5a
:4 0 a7 :4 0 5d
:2 0 5e :3 0 185
377 379 :3 0 188
374 37b :2 0 37d
18b 37e 371 37d
0 37f 18d 0
385 d :3 0 41
:3 0 a8 :3 0 381
382 0 383 :2 0
385 18f 386 35d
385 0 387 193
0 66d 2f :3 0
24 :2 0 388 389
0 66d a9 :3 0
2f :3 0 aa :3 0
6b :3 0 14 :4 0
ab 1 :8 0 396
8f :3 0 90 :3 0
391 392 0 91
:3 0 393 394 0
396 195 3e9 92
:3 0 8f :3 0 90
:3 0 398 399 0
91 :3 0 39a 39b
0 39d 198 39f
19a 39e 39d :2 0
3e7 93 :3 0 8f
:3 0 90 :3 0 3a2
3a3 0 91 :3 0
3a4 3a5 0 3df
55 :3 0 56 :3 0
3a7 3a8 0 55
:3 0 66 :3 0 3aa
3ab 0 94 :2 0
2 :4 0 ac :2 0
68 :2 0 20 :2 0
19c 3b0 3b2 :3 0
19f 3ac 3b4 3a9
3b6 3b5 :2 0 58
:3 0 69 :3 0 3b8
3b9 0 5a :4 0
ad :4 0 5d :2 0
5e :3 0 1a3 3bd
3bf :3 0 5d :2 0
97 :4 0 1a6 3c1
3c3 :3 0 5d :2 0
98 :4 0 1a9 3c5
3c7 :3 0 5d :2 0
91 :3 0 1ac 3c9
3cb :3 0 5d :2 0
99 :4 0 1af 3cd
3cf :3 0 5d :2 0
7e :3 0 9a :3 0
20 :2 0 9b :2 0
1b2 3d2 3d6 1b6
3d1 3d8 :3 0 1b9
3ba 3da :2 0 3dc
1bc 3dd 3b7 3dc
0 3de 1be 0
3df 1c0 3e2 :3 0
3e2 0 3e2 3e1
3df 3e0 :6 0 3e4
6 :3 0 1c3 3e6
1c5 3e5 3e4 :2 0
3e7 1c7 :2 0 3e9
0 3e9 3e8 396
3e7 :6 0 66d 1
:3 0 62 :3 0 2f
:3 0 24 :2 0 1ca
3eb 3ee ae :2 0
24 :2 0 1cf 3f0
3f2 :3 0 af :3 0
22 :3 0 a1 :3 0
a2 :3 0 14 :4 0
b0 1 :8 0 3ff
8f :3 0 90 :3 0
3fa 3fb 0 91
:3 0 3fc 3fd 0
3ff 1d2 452 92
:3 0 8f :3 0 90
:3 0 401 402 0
91 :3 0 403 404
0 406 1d5 408
1d7 407 406 :2 0
450 93 :3 0 8f
:3 0 90 :3 0 40b
40c 0 91 :3 0
40d 40e 0 448
55 :3 0 56 :3 0
410 411 0 55
:3 0 66 :3 0 413
414 0 94 :2 0
2 :4 0 b1 :2 0
68 :2 0 20 :2 0
1d9 419 41b :3 0
1dc 415 41d 412
41f 41e :2 0 58
:3 0 69 :3 0 421
422 0 5a :4 0
b2 :4 0 5d :2 0
5e :3 0 1e0 426
428 :3 0 5d :2 0
97 :4 0 1e3 42a
42c :3 0 5d :2 0
98 :4 0 1e6 42e
430 :3 0 5d :2 0
91 :3 0 1e9 432
434 :3 0 5d :2 0
99 :4 0 1ec 436
438 :3 0 5d :2 0
7e :3 0 9a :3 0
20 :2 0 9b :2 0
1ef 43b 43f 1f3
43a 441 :3 0 1f6
423 443 :2 0 445
1f9 446 420 445
0 447 1fb 0
448 1fd 44b :3 0
44b 0 44b 44a
448 449 :6 0 44d
8 :3 0 200 44f
202 44e 44d :2 0
450 204 :2 0 452
0 452 451 3ff
450 :6 0 489 1
:3 0 b3 :3 0 b4
:3 0 454 455 0
22 :3 0 b5 :4 0
207 456 459 9c
:2 0 24 :2 0 20c
45b 45d :3 0 c
:3 0 14 :3 0 45f
460 0 486 55
:3 0 56 :3 0 462
463 0 55 :3 0
66 :3 0 465 466
0 20 :2 0 2
:4 0 b6 :2 0 68
:2 0 20 :2 0 20f
46b 46d :3 0 212
467 46f 464 471
470 :2 0 58 :3 0
69 :3 0 473 474
0 5a :4 0 b7
:4 0 5d :2 0 5e
:3 0 216 478 47a
:3 0 219 475 47c
:2 0 47e 21c 47f
472 47e 0 480
21e 0 486 d
:3 0 41 :3 0 b8
:3 0 482 483 0
484 :2 0 486 220
487 45e 486 0
488 224 0 489
226 48a 3f3 489
0 48b 229 0
66d b9 :3 0 2f
:3 0 ba :3 0 bb
:3 0 a2 :3 0 8c
:3 0 bc :3 0 bd
:3 0 bb :3 0 bc
:3 0 6b :3 0 14
:3 0 bc :3 0 8a
:3 0 11 :3 0 bc
:3 0 8d :3 0 bb
:3 0 8d :3 0 bb
:3 0 8d :3 0 bb
:3 0 a2 :3 0 bb
:3 0 be :3 0 41
:3 0 bf :4 0 c0
1 :8 0 4ad 8f
:3 0 90 :3 0 4a8
4a9 0 91 :3 0
4aa 4ab 0 4ad
22b 500 92 :3 0
8f :3 0 90 :3 0
4af 4b0 0 91
:3 0 4b1 4b2 0
4b4 22e 4b6 230
4b5 4b4 :2 0 4fe
93 :3 0 8f :3 0
90 :3 0 4b9 4ba
0 91 :3 0 4bb
4bc 0 4f6 55
:3 0 56 :3 0 4be
4bf 0 55 :3 0
66 :3 0 4c1 4c2
0 94 :2 0 2
:4 0 c1 :2 0 68
:2 0 20 :2 0 232
4c7 4c9 :3 0 235
4c3 4cb 4c0 4cd
4cc :2 0 58 :3 0
69 :3 0 4cf 4d0
0 5a :4 0 c2
:4 0 5d :2 0 5e
:3 0 239 4d4 4d6
:3 0 5d :2 0 97
:4 0 23c 4d8 4da
:3 0 5d :2 0 98
:4 0 23f 4dc 4de
:3 0 5d :2 0 91
:3 0 242 4e0 4e2
:3 0 5d :2 0 99
:4 0 245 4e4 4e6
:3 0 5d :2 0 7e
:3 0 9a :3 0 20
:2 0 9b :2 0 248
4e9 4ed 24c 4e8
4ef :3 0 24f 4d1
4f1 :2 0 4f3 252
4f4 4ce 4f3 0
4f5 254 0 4f6
256 4f9 :3 0 4f9
0 4f9 4f8 4f6
4f7 :6 0 4fb a
:3 0 259 4fd 25b
4fc 4fb :2 0 4fe
25d :2 0 500 0
500 4ff 4ad 4fe
:6 0 66d 1 :3 0
62 :3 0 2f :3 0
24 :2 0 260 502
505 ae :2 0 24
:2 0 265 507 509
:3 0 c :3 0 11
:3 0 50b 50c 0
532 55 :3 0 56
:3 0 50e 50f 0
55 :3 0 66 :3 0
511 512 0 20
:2 0 2 :4 0 c3
:2 0 68 :2 0 20
:2 0 268 517 519
:3 0 26b 513 51b
510 51d 51c :2 0
58 :3 0 69 :3 0
51f 520 0 5a
:4 0 c4 :4 0 5d
:2 0 5e :3 0 26f
524 526 :3 0 272
521 528 :2 0 52a
275 52b 51e 52a
0 52c 277 0
532 d :3 0 41
:3 0 c5 :3 0 52e
52f 0 530 :2 0
532 279 533 50a
532 0 534 27d
0 66d 62 :3 0
26 :3 0 24 :2 0
27f 535 538 9c
:2 0 24 :2 0 284
53a 53c :3 0 55
:3 0 56 :3 0 53e
53f 0 55 :3 0
66 :3 0 541 542
0 c6 :2 0 2
:4 0 c7 :2 0 68
:2 0 20 :2 0 287
547 549 :3 0 28a
543 54b 540 54d
54c :2 0 58 :3 0
69 :3 0 54f 550
0 5a :4 0 c8
:4 0 5d :2 0 5e
:3 0 28e 554 556
:3 0 5d :2 0 97
:4 0 291 558 55a
:3 0 5d :2 0 c9
:4 0 294 55c 55e
:3 0 5d :2 0 62
:3 0 63 :3 0 41
:3 0 ca :3 0 563
564 0 297 562
566 64 :4 0 299
561 569 29c 560
56b :3 0 29f 551
56d :2 0 56f 2a2
570 54e 56f 0
571 2a4 0 59d
55 :3 0 56 :3 0
572 573 0 55
:3 0 66 :3 0 575
576 0 20 :2 0
2 :4 0 cb :2 0
68 :2 0 20 :2 0
2a6 57b 57d :3 0
2a9 577 57f 574
581 580 :2 0 58
:3 0 69 :3 0 583
584 0 5a :4 0
cc :4 0 5d :2 0
5e :3 0 2ad 588
58a :3 0 2b0 585
58c :2 0 58e 2b3
58f 582 58e 0
590 2b5 0 59d
d :3 0 cd :3 0
ce :3 0 592 593
0 41 :3 0 ca
:3 0 595 596 0
cf :3 0 2b7 594
599 59a :2 0 59d
d0 :3 0 2ba 66a
62 :3 0 f :3 0
24 :2 0 2be 59e
5a1 62 :3 0 d1
:2 0 29 :3 0 24
:2 0 2c1 5a3 5a7
2c6 5a4 5a9 :3 0
3b :3 0 24 :2 0
5ab 5ac 0 609
62 :3 0 f :3 0
24 :2 0 2c9 5ae
5b1 62 :3 0 ae
:2 0 26 :3 0 24
:2 0 2cc 5b3 5b7
2d1 5b4 5b9 :3 0
c :3 0 11 :3 0
5bb 5bc 0 5e3
55 :3 0 56 :3 0
5be 5bf 0 55
:3 0 66 :3 0 5c1
5c2 0 20 :2 0
2 :4 0 d2 :2 0
68 :2 0 20 :2 0
2d4 5c7 5c9 :3 0
2d7 5c3 5cb 5c0
5cd 5cc :2 0 58
:3 0 69 :3 0 5cf
5d0 0 5a :4 0
d3 :4 0 5d :2 0
5e :3 0 2db 5d4
5d6 :3 0 2de 5d1
5d8 :2 0 5da 2e1
5db 5ce 5da 0
5dc 2e3 0 5e3
d :3 0 41 :3 0
d4 :3 0 5de 5df
0 5e0 :2 0 5e3
d0 :3 0 2e5 5f6
62 :3 0 f :3 0
24 :2 0 2e9 5e4
5e7 62 :3 0 9c
:2 0 26 :3 0 24
:2 0 2ec 5e9 5ed
2f1 5ea 5ef :3 0
3b :3 0 20 :2 0
5f1 5f2 0 5f4
2f4 5f5 5f0 5f4
0 5f7 5ba 5e3
0 5f7 2f6 0
609 2a :3 0 f
:3 0 5f8 5f9 0
609 2f :3 0 d5
:3 0 14 :3 0 11
:3 0 2a :3 0 b
:3 0 62 :3 0 3b
:3 0 24 :2 0 2f9
601 604 2fc 5fc
606 5fb 607 0
609 302 60a 5aa
609 0 66c 55
:3 0 56 :3 0 60b
60c 0 55 :3 0
66 :3 0 60e 60f
0 c6 :2 0 2
:4 0 d6 :2 0 68
:2 0 20 :2 0 307
614 616 :3 0 30a
610 618 60d 61a
619 :2 0 58 :3 0
69 :3 0 61c 61d
0 5a :4 0 d7
:4 0 5d :2 0 5e
:3 0 30e 621 623
:3 0 5d :2 0 97
:4 0 311 625 627
:3 0 5d :2 0 c9
:4 0 314 629 62b
:3 0 5d :2 0 62
:3 0 63 :3 0 41
:3 0 ca :3 0 630
631 0 317 62f
633 64 :4 0 319
62e 636 31c 62d
638 :3 0 31f 61e
63a :2 0 63c 322
63d 61b 63c 0
63e 324 0 669
55 :3 0 56 :3 0
63f 640 0 55
:3 0 66 :3 0 642
643 0 20 :2 0
2 :4 0 d8 :2 0
68 :2 0 20 :2 0
326 648 64a :3 0
329 644 64c 641
64e 64d :2 0 58
:3 0 69 :3 0 650
651 0 5a :4 0
d9 :4 0 5d :2 0
5e :3 0 32d 655
657 :3 0 330 652
659 :2 0 65b 333
65c 64f 65b 0
65d 335 0 669
d :3 0 cd :3 0
ce :3 0 65f 660
0 41 :3 0 ca
:3 0 662 663 0
cf :3 0 337 661
666 667 :2 0 669
33a 66b 53d 59d
0 66c 0 669
0 66c 33e 0
66d 342 12b3 62
:3 0 26 :3 0 24
:2 0 34b 66e 671
9c :2 0 24 :2 0
350 673 675 :3 0
c :3 0 11 :3 0
677 678 0 69e
55 :3 0 56 :3 0
67a 67b 0 55
:3 0 66 :3 0 67d
67e 0 20 :2 0
2 :4 0 da :2 0
68 :2 0 20 :2 0
353 683 685 :3 0
356 67f 687 67c
689 688 :2 0 58
:3 0 69 :3 0 68b
68c 0 5a :4 0
db :4 0 5d :2 0
5e :3 0 35a 690
692 :3 0 35d 68d
694 :2 0 696 360
697 68a 696 0
698 362 0 69e
d :3 0 41 :3 0
ca :3 0 69a 69b
0 69c :2 0 69e
364 69f 676 69e
0 6a0 368 0
12b2 2f :3 0 24
:2 0 6a1 6a2 0
12b2 a9 :3 0 2f
:3 0 aa :3 0 6b
:3 0 14 :4 0 ab
1 :8 0 6af 8f
:3 0 90 :3 0 6aa
6ab 0 91 :3 0
6ac 6ad 0 6af
36a 702 92 :3 0
8f :3 0 90 :3 0
6b1 6b2 0 91
:3 0 6b3 6b4 0
6b6 36d 6b8 36f
6b7 6b6 :2 0 700
93 :3 0 8f :3 0
90 :3 0 6bb 6bc
0 91 :3 0 6bd
6be 0 6f8 55
:3 0 56 :3 0 6c0
6c1 0 55 :3 0
66 :3 0 6c3 6c4
0 94 :2 0 2
:4 0 dc :2 0 68
:2 0 20 :2 0 371
6c9 6cb :3 0 374
6c5 6cd 6c2 6cf
6ce :2 0 58 :3 0
69 :3 0 6d1 6d2
0 5a :4 0 dd
:4 0 5d :2 0 5e
:3 0 378 6d6 6d8
:3 0 5d :2 0 97
:4 0 37b 6da 6dc
:3 0 5d :2 0 98
:4 0 37e 6de 6e0
:3 0 5d :2 0 91
:3 0 381 6e2 6e4
:3 0 5d :2 0 99
:4 0 384 6e6 6e8
:3 0 5d :2 0 7e
:3 0 9a :3 0 20
:2 0 9b :2 0 387
6eb 6ef 38b 6ea
6f1 :3 0 38e 6d3
6f3 :2 0 6f5 391
6f6 6d0 6f5 0
6f7 393 0 6f8
395 6fb :3 0 6fb
0 6fb 6fa 6f8
6f9 :6 0 6fd c
:3 0 398 6ff 39a
6fe 6fd :2 0 700
39c :2 0 702 0
702 701 6af 700
:6 0 12b2 1 :3 0
62 :3 0 2f :3 0
24 :2 0 39f 704
707 ae :2 0 24
:2 0 3a4 709 70b
:3 0 af :3 0 22
:3 0 a1 :3 0 a2
:3 0 14 :4 0 b0
1 :8 0 718 8f
:3 0 90 :3 0 713
714 0 91 :3 0
715 716 0 718
3a7 76b 92 :3 0
8f :3 0 90 :3 0
71a 71b 0 91
:3 0 71c 71d 0
71f 3aa 721 3ac
720 71f :2 0 769
93 :3 0 8f :3 0
90 :3 0 724 725
0 91 :3 0 726
727 0 761 55
:3 0 56 :3 0 729
72a 0 55 :3 0
66 :3 0 72c 72d
0 94 :2 0 2
:4 0 de :2 0 68
:2 0 20 :2 0 3ae
732 734 :3 0 3b1
72e 736 72b 738
737 :2 0 58 :3 0
69 :3 0 73a 73b
0 5a :4 0 df
:4 0 5d :2 0 5e
:3 0 3b5 73f 741
:3 0 5d :2 0 97
:4 0 3b8 743 745
:3 0 5d :2 0 98
:4 0 3bb 747 749
:3 0 5d :2 0 91
:3 0 3be 74b 74d
:3 0 5d :2 0 99
:4 0 3c1 74f 751
:3 0 5d :2 0 7e
:3 0 9a :3 0 20
:2 0 9b :2 0 3c4
754 758 3c8 753
75a :3 0 3cb 73c
75c :2 0 75e 3ce
75f 739 75e 0
760 3d0 0 761
3d2 764 :3 0 764
0 764 763 761
762 :6 0 766 e
:3 0 3d5 768 3d7
767 766 :2 0 769
3d9 :2 0 76b 0
76b 76a 718 769
:6 0 7a2 1 :3 0
b3 :3 0 b4 :3 0
76d 76e 0 22
:3 0 b5 :4 0 3dc
76f 772 9c :2 0
24 :2 0 3e1 774
776 :3 0 c :3 0
14 :3 0 778 779
0 79f 55 :3 0
56 :3 0 77b 77c
0 55 :3 0 66
:3 0 77e 77f 0
20 :2 0 2 :4 0
e0 :2 0 68 :2 0
20 :2 0 3e4 784
786 :3 0 3e7 780
788 77d 78a 789
:2 0 58 :3 0 69
:3 0 78c 78d 0
5a :4 0 e1 :4 0
5d :2 0 5e :3 0
3eb 791 793 :3 0
3ee 78e 795 :2 0
797 3f1 798 78b
797 0 799 3f3
0 79f d :3 0
41 :3 0 b8 :3 0
79b 79c 0 79d
:2 0 79f 3f5 7a0
777 79f 0 7a1
3f9 0 7a2 3fb
7a3 70c 7a2 0
7a4 3fe 0 12b2
b9 :3 0 2f :3 0
ba :3 0 a2 :3 0
bd :3 0 8d :3 0
11 :3 0 8d :3 0
a2 :3 0 be :3 0
41 :3 0 bf :4 0
e2 1 :8 0 7b7
8f :3 0 90 :3 0
7b2 7b3 0 91
:3 0 7b4 7b5 0
7b7 400 80a 92
:3 0 8f :3 0 90
:3 0 7b9 7ba 0
91 :3 0 7bb 7bc
0 7be 403 7c0
405 7bf 7be :2 0
808 93 :3 0 8f
:3 0 90 :3 0 7c3
7c4 0 91 :3 0
7c5 7c6 0 800
55 :3 0 56 :3 0
7c8 7c9 0 55
:3 0 66 :3 0 7cb
7cc 0 94 :2 0
2 :4 0 e3 :2 0
68 :2 0 20 :2 0
407 7d1 7d3 :3 0
40a 7cd 7d5 7ca
7d7 7d6 :2 0 58
:3 0 69 :3 0 7d9
7da 0 5a :4 0
e4 :4 0 5d :2 0
5e :3 0 40e 7de
7e0 :3 0 5d :2 0
97 :4 0 411 7e2
7e4 :3 0 5d :2 0
98 :4 0 414 7e6
7e8 :3 0 5d :2 0
91 :3 0 417 7ea
7ec :3 0 5d :2 0
99 :4 0 41a 7ee
7f0 :3 0 5d :2 0
7e :3 0 9a :3 0
20 :2 0 9b :2 0
41d 7f3 7f7 421
7f2 7f9 :3 0 424
7db 7fb :2 0 7fd
427 7fe 7d8 7fd
0 7ff 429 0
800 42b 803 :3 0
803 0 803 802
800 801 :6 0 805
10 :3 0 42e 807
430 806 805 :2 0
808 432 :2 0 80a
0 80a 809 7b7
808 :6 0 12b2 1
:3 0 62 :3 0 2f
:3 0 24 :2 0 435
80c 80f ae :2 0
24 :2 0 43a 811
813 :3 0 c :3 0
11 :3 0 815 816
0 83c 55 :3 0
56 :3 0 818 819
0 55 :3 0 66
:3 0 81b 81c 0
20 :2 0 2 :4 0
e5 :2 0 68 :2 0
20 :2 0 43d 821
823 :3 0 440 81d
825 81a 827 826
:2 0 58 :3 0 69
:3 0 829 82a 0
5a :4 0 e6 :4 0
5d :2 0 5e :3 0
444 82e 830 :3 0
447 82b 832 :2 0
834 44a 835 828
834 0 836 44c
0 83c d :3 0
41 :3 0 c5 :3 0
838 839 0 83a
:2 0 83c 44e 83d
814 83c 0 83e
452 0 12b2 6b
:3 0 e7 :3 0 14
:3 0 2c :3 0 a1
:3 0 6b :3 0 14
:3 0 a2 :3 0 15
:4 0 e8 1 :8 0
84e 8f :3 0 90
:3 0 849 84a 0
91 :3 0 84b 84c
0 84e 454 8a1
92 :3 0 8f :3 0
90 :3 0 850 851
0 91 :3 0 852
853 0 855 457
857 459 856 855
:2 0 89f 93 :3 0
8f :3 0 90 :3 0
85a 85b 0 91
:3 0 85c 85d 0
897 55 :3 0 56
:3 0 85f 860 0
55 :3 0 66 :3 0
862 863 0 94
:2 0 2 :4 0 e9
:2 0 68 :2 0 20
:2 0 45b 868 86a
:3 0 45e 864 86c
861 86e 86d :2 0
58 :3 0 69 :3 0
870 871 0 5a
:4 0 ea :4 0 5d
:2 0 5e :3 0 462
875 877 :3 0 5d
:2 0 97 :4 0 465
879 87b :3 0 5d
:2 0 98 :4 0 468
87d 87f :3 0 5d
:2 0 91 :3 0 46b
881 883 :3 0 5d
:2 0 99 :4 0 46e
885 887 :3 0 5d
:2 0 7e :3 0 9a
:3 0 20 :2 0 9b
:2 0 471 88a 88e
475 889 890 :3 0
478 872 892 :2 0
894 47b 895 86f
894 0 896 47d
0 897 47f 89a
:3 0 89a 0 89a
899 897 898 :6 0
89c 12 :3 0 482
89e 484 89d 89c
:2 0 89f 486 :2 0
8a1 0 8a1 8a0
84e 89f :6 0 12b2
1 :3 0 38 :3 0
62 :3 0 26 :3 0
24 :2 0 489 8a4
8a7 8a3 8a8 0
12b2 62 :3 0 f
:3 0 24 :2 0 48c
8aa 8ad 62 :3 0
d1 :2 0 29 :3 0
24 :2 0 48f 8af
8b3 494 8b0 8b5
:3 0 2a :3 0 f
:3 0 8b7 8b8 0
ef3 62 :3 0 2a
:3 0 24 :2 0 497
8ba 8bd 62 :3 0
ae :2 0 26 :3 0
24 :2 0 49a 8bf
8c3 49f 8c0 8c5
:3 0 2e :3 0 eb
:2 0 20 :2 0 4a2
8c8 8ca :3 0 8c7
8cb 0 e81 2f
:3 0 ec :3 0 14
:3 0 15 :3 0 11
:3 0 17 :3 0 19
:3 0 46 :3 0 48
:3 0 2e :3 0 25
:3 0 23 :3 0 4a4
8ce 8d9 8cd 8da
0 e81 62 :3 0
2f :3 0 24 :2 0
4af 8dc 8df 9c
:2 0 24 :2 0 4b4
8e1 8e3 :3 0 62
:3 0 3c :3 0 24
:2 0 4b7 8e5 8e8
62 :3 0 d1 :2 0
3a :3 0 24 :2 0
4ba 8ea 8ee 4bf
8eb 8f0 :3 0 b3
:3 0 b4 :3 0 8f2
8f3 0 16 :3 0
b :3 0 4c2 8f4
8f7 9c :2 0 24
:2 0 4c7 8f9 8fb
:3 0 26 :3 0 62
:3 0 2a :3 0 24
:2 0 4ca 8fe 901
eb :2 0 62 :3 0
26 :3 0 24 :2 0
4cd 904 907 4d0
903 909 :3 0 8fd
90a 0 a48 27
:3 0 ed :2 0 90c
90d 0 a48 62
:3 0 2a :3 0 24
:2 0 4d3 90f 912
ee :2 0 62 :3 0
29 :3 0 24 :2 0
4d6 915 918 68
:2 0 62 :3 0 3a
:3 0 24 :2 0 4d9
91b 91e 4dc 91a
920 :3 0 921 :2 0
4e1 914 923 :3 0
3a :3 0 62 :3 0
29 :3 0 24 :2 0
4e4 926 929 68
:2 0 62 :3 0 3a
:3 0 24 :2 0 4e7
92c 92f 4ea 92b
931 :3 0 932 :2 0
eb :2 0 62 :3 0
2a :3 0 24 :2 0
4ed 935 938 4f0
934 93a :3 0 925
93b 0 9c8 8c
:3 0 ef :3 0 8b
:3 0 f0 :3 0 f1
:3 0 8b :3 0 41
:3 0 f2 :3 0 71
:3 0 2a :3 0 70
:3 0 2a :3 0 72
:3 0 3a :3 0 89
:3 0 8d :3 0 11
:4 0 f3 1 :8 0
964 f4 :4 0 94f
:3 0 f5 :3 0 9c
:2 0 4f5 952 953
:3 0 8f :3 0 90
:3 0 955 956 0
24 :2 0 957 958
0 95a 4f8 961
8f :3 0 90 :3 0
95b 95c 0 f6
:2 0 95d 95e 0
960 4fa 962 954
95a 0 963 0
960 0 963 4fc
0 964 4ff 9b7
92 :3 0 8f :3 0
90 :3 0 966 967
0 91 :3 0 968
969 0 96b 502
96d 504 96c 96b
:2 0 9b5 93 :3 0
8f :3 0 90 :3 0
970 971 0 91
:3 0 972 973 0
9ad 55 :3 0 56
:3 0 975 976 0
55 :3 0 66 :3 0
978 979 0 94
:2 0 2 :4 0 f7
:2 0 68 :2 0 20
:2 0 506 97e 980
:3 0 509 97a 982
977 984 983 :2 0
58 :3 0 69 :3 0
986 987 0 5a
:4 0 f8 :4 0 5d
:2 0 5e :3 0 50d
98b 98d :3 0 5d
:2 0 97 :4 0 510
98f 991 :3 0 5d
:2 0 98 :4 0 513
993 995 :3 0 5d
:2 0 91 :3 0 516
997 999 :3 0 5d
:2 0 99 :4 0 519
99b 99d :3 0 5d
:2 0 7e :3 0 9a
:3 0 20 :2 0 9b
:2 0 51c 9a0 9a4
520 99f 9a6 :3 0
523 988 9a8 :2 0
9aa 526 9ab 985
9aa 0 9ac 528
0 9ad 52a 9b0
:3 0 9b0 0 9b0
9af 9ad 9ae :6 0
9b2 14 :3 0 52d
9b4 52f 9b3 9b2
:2 0 9b5 531 :2 0
9b7 0 9b7 9b6
964 9b5 :6 0 9c8
1 :3 0 27 :3 0
f9 :3 0 f1 :3 0
9ba 9bb 0 62
:3 0 27 :3 0 24
:2 0 534 9bd 9c0
41 :3 0 fa :3 0
9c2 9c3 0 537
9bc 9c5 9b9 9c6
0 9c8 53a a45
8c :3 0 ef :3 0
8b :3 0 f0 :3 0
f1 :3 0 8b :3 0
41 :3 0 f2 :3 0
71 :3 0 2a :3 0
70 :3 0 2a :3 0
72 :3 0 89 :3 0
8d :3 0 11 :4 0
fb 1 :8 0 9ef
f4 :4 0 9da :3 0
f5 :3 0 9c :2 0
540 9dd 9de :3 0
8f :3 0 90 :3 0
9e0 9e1 0 24
:2 0 9e2 9e3 0
9e5 543 9ec 8f
:3 0 90 :3 0 9e6
9e7 0 f6 :2 0
9e8 9e9 0 9eb
545 9ed 9df 9e5
0 9ee 0 9eb
0 9ee 547 0
9ef 54a a42 92
:3 0 8f :3 0 90
:3 0 9f1 9f2 0
91 :3 0 9f3 9f4
0 9f6 54d 9f8
54f 9f7 9f6 :2 0
a40 93 :3 0 8f
:3 0 90 :3 0 9fb
9fc 0 91 :3 0
9fd 9fe 0 a38
55 :3 0 56 :3 0
a00 a01 0 55
:3 0 66 :3 0 a03
a04 0 94 :2 0
2 :4 0 fc :2 0
68 :2 0 20 :2 0
551 a09 a0b :3 0
554 a05 a0d a02
a0f a0e :2 0 58
:3 0 69 :3 0 a11
a12 0 5a :4 0
fd :4 0 5d :2 0
5e :3 0 558 a16
a18 :3 0 5d :2 0
97 :4 0 55b a1a
a1c :3 0 5d :2 0
98 :4 0 55e a1e
a20 :3 0 5d :2 0
91 :3 0 561 a22
a24 :3 0 5d :2 0
99 :4 0 564 a26
a28 :3 0 5d :2 0
7e :3 0 9a :3 0
20 :2 0 9b :2 0
567 a2b a2f 56b
a2a a31 :3 0 56e
a13 a33 :2 0 a35
571 a36 a10 a35
0 a37 573 0
a38 575 a3b :3 0
a3b 0 a3b a3a
a38 a39 :6 0 a3d
16 :3 0 578 a3f
57a a3e a3d :2 0
a40 57c :2 0 a42
0 a42 a41 9ef
a40 :6 0 a44 1
:3 0 57f a46 924
9c8 0 a47 0
a44 0 a47 581
0 a48 584 b9c
62 :3 0 2a :3 0
24 :2 0 588 a49
a4c ee :2 0 62
:3 0 29 :3 0 24
:2 0 58b a4f a52
68 :2 0 62 :3 0
3a :3 0 24 :2 0
58e a55 a58 591
a54 a5a :3 0 a5b
:2 0 596 a4e a5d
:3 0 26 :3 0 62
:3 0 3a :3 0 24
:2 0 599 a60 a63
eb :2 0 62 :3 0
26 :3 0 24 :2 0
59c a66 a69 59f
a65 a6b :3 0 a5f
a6c 0 b09 27
:3 0 fe :3 0 62
:3 0 ff :3 0 2a
:3 0 5a2 a71 a73
24 :2 0 5a4 a70
a76 100 :2 0 62
:3 0 ff :3 0 26
:3 0 5a7 a7a a7c
24 :2 0 5a9 a79
a7f 5ac a78 a81
:3 0 a82 :2 0 101
:2 0 102 :2 0 5af
a84 a86 :3 0 a87
:2 0 68 :2 0 103
:2 0 5b2 a89 a8b
:3 0 5b5 a6f a8d
a6e a8e 0 b09
8c :3 0 ef :3 0
8b :3 0 f0 :3 0
f1 :3 0 8b :3 0
41 :3 0 f2 :3 0
71 :3 0 3a :3 0
70 :3 0 2a :3 0
8d :3 0 11 :4 0
104 1 :8 0 ab4
f4 :4 0 a9f :3 0
f5 :3 0 9c :2 0
5b9 aa2 aa3 :3 0
8f :3 0 90 :3 0
aa5 aa6 0 24
:2 0 aa7 aa8 0
aaa 5bc ab1 8f
:3 0 90 :3 0 aab
aac 0 f6 :2 0
aad aae 0 ab0
5be ab2 aa4 aaa
0 ab3 0 ab0
0 ab3 5c0 0
ab4 5c3 b07 92
:3 0 8f :3 0 90
:3 0 ab6 ab7 0
91 :3 0 ab8 ab9
0 abb 5c6 abd
5c8 abc abb :2 0
b05 93 :3 0 8f
:3 0 90 :3 0 ac0
ac1 0 91 :3 0
ac2 ac3 0 afd
55 :3 0 56 :3 0
ac5 ac6 0 55
:3 0 66 :3 0 ac8
ac9 0 94 :2 0
2 :4 0 105 :2 0
68 :2 0 20 :2 0
5ca ace ad0 :3 0
5cd aca ad2 ac7
ad4 ad3 :2 0 58
:3 0 69 :3 0 ad6
ad7 0 5a :4 0
106 :4 0 5d :2 0
5e :3 0 5d1 adb
add :3 0 5d :2 0
97 :4 0 5d4 adf
ae1 :3 0 5d :2 0
98 :4 0 5d7 ae3
ae5 :3 0 5d :2 0
91 :3 0 5da ae7
ae9 :3 0 5d :2 0
99 :4 0 5dd aeb
aed :3 0 5d :2 0
7e :3 0 9a :3 0
20 :2 0 9b :2 0
5e0 af0 af4 5e4
aef af6 :3 0 5e7
ad8 af8 :2 0 afa
5ea afb ad5 afa
0 afc 5ec 0
afd 5ee b00 :3 0
b00 0 b00 aff
afd afe :6 0 b02
18 :3 0 5f1 b04
5f3 b03 b02 :2 0
b05 5f5 :2 0 b07
0 b07 b06 ab4
b05 :6 0 b09 1
:3 0 5f8 b98 26
:3 0 62 :3 0 2a
:3 0 24 :2 0 5fc
b0b b0e eb :2 0
62 :3 0 26 :3 0
24 :2 0 5ff b11
b14 602 b10 b16
:3 0 b0a b17 0
b97 27 :3 0 ed
:2 0 b19 b1a 0
b97 8c :3 0 ef
:3 0 8b :3 0 f0
:3 0 f1 :3 0 8b
:3 0 41 :3 0 f2
:3 0 71 :3 0 2a
:3 0 70 :3 0 2a
:3 0 72 :3 0 89
:3 0 8d :3 0 11
:4 0 fb 1 :8 0
b42 f4 :4 0 b2d
:3 0 f5 :3 0 9c
:2 0 607 b30 b31
:3 0 8f :3 0 90
:3 0 b33 b34 0
24 :2 0 b35 b36
0 b38 60a b3f
8f :3 0 90 :3 0
b39 b3a 0 f6
:2 0 b3b b3c 0
b3e 60c b40 b32
b38 0 b41 0
b3e 0 b41 60e
0 b42 611 b95
92 :3 0 8f :3 0
90 :3 0 b44 b45
0 91 :3 0 b46
b47 0 b49 614
b4b 616 b4a b49
:2 0 b93 93 :3 0
8f :3 0 90 :3 0
b4e b4f 0 91
:3 0 b50 b51 0
b8b 55 :3 0 56
:3 0 b53 b54 0
55 :3 0 66 :3 0
b56 b57 0 94
:2 0 2 :4 0 107
:2 0 68 :2 0 20
:2 0 618 b5c b5e
:3 0 61b b58 b60
b55 b62 b61 :2 0
58 :3 0 69 :3 0
b64 b65 0 5a
:4 0 108 :4 0 5d
:2 0 5e :3 0 61f
b69 b6b :3 0 5d
:2 0 97 :4 0 622
b6d b6f :3 0 5d
:2 0 98 :4 0 625
b71 b73 :3 0 5d
:2 0 91 :3 0 628
b75 b77 :3 0 5d
:2 0 99 :4 0 62b
b79 b7b :3 0 5d
:2 0 7e :3 0 9a
:3 0 20 :2 0 9b
:2 0 62e b7e b82
632 b7d b84 :3 0
635 b66 b86 :2 0
b88 638 b89 b63
b88 0 b8a 63a
0 b8b 63c b8e
:3 0 b8e 0 b8e
b8d b8b b8c :6 0
b90 1a :3 0 63f
b92 641 b91 b90
:2 0 b93 643 :2 0
b95 0 b95 b94
b42 b93 :6 0 b97
1 :3 0 646 b99
a5e b09 0 b9a
0 b97 0 b9a
64a 0 b9b 64d
b9d 8fc a48 0
b9e 0 b9b 0
b9e 64f 0 b9f
652 c2e 26 :3 0
62 :3 0 2a :3 0
24 :2 0 654 ba1
ba4 eb :2 0 62
:3 0 29 :3 0 24
:2 0 657 ba7 baa
65a ba6 bac :3 0
ba0 bad 0 c2d
27 :3 0 ed :2 0
baf bb0 0 c2d
8c :3 0 ef :3 0
8b :3 0 f0 :3 0
f1 :3 0 8b :3 0
41 :3 0 f2 :3 0
71 :3 0 2a :3 0
70 :3 0 2a :3 0
72 :3 0 89 :3 0
8d :3 0 11 :4 0
109 1 :8 0 bd8
f4 :4 0 bc3 :3 0
f5 :3 0 9c :2 0
65f bc6 bc7 :3 0
8f :3 0 90 :3 0
bc9 bca 0 24
:2 0 bcb bcc 0
bce 662 bd5 8f
:3 0 90 :3 0 bcf
bd0 0 f6 :2 0
bd1 bd2 0 bd4
664 bd6 bc8 bce
0 bd7 0 bd4
0 bd7 666 0
bd8 669 c2b 92
:3 0 8f :3 0 90
:3 0 bda bdb 0
91 :3 0 bdc bdd
0 bdf 66c be1
66e be0 bdf :2 0
c29 93 :3 0 8f
:3 0 90 :3 0 be4
be5 0 91 :3 0
be6 be7 0 c21
55 :3 0 56 :3 0
be9 bea 0 55
:3 0 66 :3 0 bec
bed 0 94 :2 0
2 :4 0 10a :2 0
68 :2 0 20 :2 0
670 bf2 bf4 :3 0
673 bee bf6 beb
bf8 bf7 :2 0 58
:3 0 69 :3 0 bfa
bfb 0 5a :4 0
10b :4 0 5d :2 0
5e :3 0 677 bff
c01 :3 0 5d :2 0
97 :4 0 67a c03
c05 :3 0 5d :2 0
98 :4 0 67d c07
c09 :3 0 5d :2 0
91 :3 0 680 c0b
c0d :3 0 5d :2 0
99 :4 0 683 c0f
c11 :3 0 5d :2 0
7e :3 0 9a :3 0
20 :2 0 9b :2 0
686 c14 c18 68a
c13 c1a :3 0 68d
bfc c1c :2 0 c1e
690 c1f bf9 c1e
0 c20 692 0
c21 694 c24 :3 0
c24 0 c24 c23
c21 c22 :6 0 c26
1c :3 0 697 c28
699 c27 c26 :2 0
c29 69b :2 0 c2b
0 c2b c2a bd8
c29 :6 0 c2d 1
:3 0 69e c2f 8f1
b9f 0 c30 0
c2d 0 c30 6a2
0 e7e 1d :3 0
10c :2 0 6a5 c32
c33 :3 0 c34 :2 0
1e :4 0 c36 c37
0 dfe 10d :3 0
10e :3 0 1e :3 0
10f :3 0 a2 :3 0
14 :3 0 6c :3 0
16 :4 0 110 1
:8 0 c47 8f :3 0
90 :3 0 c42 c43
0 91 :3 0 c44
c45 0 c47 6a7
c9a 92 :3 0 8f
:3 0 90 :3 0 c49
c4a 0 91 :3 0
c4b c4c 0 c4e
6aa c50 6ac c4f
c4e :2 0 c98 93
:3 0 8f :3 0 90
:3 0 c53 c54 0
91 :3 0 c55 c56
0 c90 55 :3 0
56 :3 0 c58 c59
0 55 :3 0 66
:3 0 c5b c5c 0
94 :2 0 2 :4 0
111 :2 0 68 :2 0
20 :2 0 6ae c61
c63 :3 0 6b1 c5d
c65 c5a c67 c66
:2 0 58 :3 0 69
:3 0 c69 c6a 0
5a :4 0 112 :4 0
5d :2 0 5e :3 0
6b5 c6e c70 :3 0
5d :2 0 97 :4 0
6b8 c72 c74 :3 0
5d :2 0 98 :4 0
6bb c76 c78 :3 0
5d :2 0 91 :3 0
6be c7a c7c :3 0
5d :2 0 99 :4 0
6c1 c7e c80 :3 0
5d :2 0 7e :3 0
9a :3 0 20 :2 0
9b :2 0 6c4 c83
c87 6c8 c82 c89
:3 0 6cb c6b c8b
:2 0 c8d 6ce c8e
c68 c8d 0 c8f
6d0 0 c90 6d2
c93 :3 0 c93 0
c93 c92 c90 c91
:6 0 c95 1e :3 0
6d5 c97 6d7 c96
c95 :2 0 c98 6d9
:2 0 c9a 0 c9a
c99 c47 c98 :6 0
dfe 1 :3 0 1e
:3 0 10c :2 0 6dc
c9d c9e :3 0 c9f
:2 0 7d :3 0 7e
:3 0 1e :3 0 1e
:3 0 ba :4 0 113
1 :8 0 cac 8f
:3 0 90 :3 0 ca7
ca8 0 91 :3 0
ca9 caa 0 cac
6de cff 92 :3 0
8f :3 0 90 :3 0
cae caf 0 91
:3 0 cb0 cb1 0
cb3 6e1 cb5 6e3
cb4 cb3 :2 0 cfd
93 :3 0 8f :3 0
90 :3 0 cb8 cb9
0 91 :3 0 cba
cbb 0 cf5 55
:3 0 56 :3 0 cbd
cbe 0 55 :3 0
66 :3 0 cc0 cc1
0 94 :2 0 2
:4 0 114 :2 0 68
:2 0 20 :2 0 6e5
cc6 cc8 :3 0 6e8
cc2 cca cbf ccc
ccb :2 0 58 :3 0
69 :3 0 cce ccf
0 5a :4 0 115
:4 0 5d :2 0 5e
:3 0 6ec cd3 cd5
:3 0 5d :2 0 97
:4 0 6ef cd7 cd9
:3 0 5d :2 0 98
:4 0 6f2 cdb cdd
:3 0 5d :2 0 91
:3 0 6f5 cdf ce1
:3 0 5d :2 0 99
:4 0 6f8 ce3 ce5
:3 0 5d :2 0 7e
:3 0 9a :3 0 20
:2 0 9b :2 0 6fb
ce8 cec 6ff ce7
cee :3 0 702 cd0
cf0 :2 0 cf2 705
cf3 ccd cf2 0
cf4 707 0 cf5
709 cf8 :3 0 cf8
0 cf8 cf7 cf5
cf6 :6 0 cfa 20
:3 0 70c cfc 70e
cfb cfa :2 0 cfd
710 :2 0 cff 0
cff cfe cac cfd
:6 0 d14 1 :3 0
b3 :3 0 116 :3 0
d01 d02 0 1d
:3 0 1e :3 0 117
:2 0 713 d03 d07
ee :2 0 24 :2 0
719 d09 d0b :3 0
1a :3 0 1e :3 0
d0d d0e 0 d10
71c d11 d0c d10
0 d12 71e 0
d14 d0 :3 0 720
dfb 62 :3 0 32
:3 0 24 :2 0 723
d15 d18 ae :2 0
24 :2 0 728 d1a
d1c :3 0 32 :3 0
f9 :3 0 f1 :3 0
d1f d20 0 62
:3 0 32 :3 0 24
:2 0 72b d22 d25
41 :3 0 fa :3 0
d27 d28 0 72e
d21 d2a d2b :2 0
d1e d2c 0 d93
b3 :3 0 118 :3 0
7d :3 0 7e :3 0
1d :3 0 b3 :3 0
119 :3 0 1a :3 0
1a :3 0 ba :4 0
11a 1 :8 0 d3e
8f :3 0 90 :3 0
d39 d3a 0 91
:3 0 d3b d3c 0
d3e 731 d91 92
:3 0 8f :3 0 90
:3 0 d40 d41 0
91 :3 0 d42 d43
0 d45 734 d47
736 d46 d45 :2 0
d8f 93 :3 0 8f
:3 0 90 :3 0 d4a
d4b 0 91 :3 0
d4c d4d 0 d87
55 :3 0 56 :3 0
d4f d50 0 55
:3 0 66 :3 0 d52
d53 0 94 :2 0
2 :4 0 11b :2 0
68 :2 0 20 :2 0
738 d58 d5a :3 0
73b d54 d5c d51
d5e d5d :2 0 58
:3 0 69 :3 0 d60
d61 0 5a :4 0
11c :4 0 5d :2 0
5e :3 0 73f d65
d67 :3 0 5d :2 0
97 :4 0 742 d69
d6b :3 0 5d :2 0
98 :4 0 745 d6d
d6f :3 0 5d :2 0
91 :3 0 748 d71
d73 :3 0 5d :2 0
99 :4 0 74b d75
d77 :3 0 5d :2 0
7e :3 0 9a :3 0
20 :2 0 9b :2 0
74e d7a d7e 752
d79 d80 :3 0 755
d62 d82 :2 0 d84
758 d85 d5f d84
0 d86 75a 0
d87 75c d8a :3 0
d8a 0 d8a d89
d87 d88 :6 0 d8c
22 :3 0 75f d8e
761 d8d d8c :2 0
d8f 763 :2 0 d91
0 d91 d90 d3e
d8f :6 0 d93 1
:3 0 766 d94 d1d
d93 0 dfd b3
:3 0 118 :3 0 7d
:3 0 7e :3 0 1d
:3 0 b3 :3 0 119
:3 0 1a :3 0 1a
:3 0 ba :4 0 11d
1 :8 0 da5 8f
:3 0 90 :3 0 da0
da1 0 91 :3 0
da2 da3 0 da5
769 df8 92 :3 0
8f :3 0 90 :3 0
da7 da8 0 91
:3 0 da9 daa 0
dac 76c dae 76e
dad dac :2 0 df6
93 :3 0 8f :3 0
90 :3 0 db1 db2
0 91 :3 0 db3
db4 0 dee 55
:3 0 56 :3 0 db6
db7 0 55 :3 0
66 :3 0 db9 dba
0 94 :2 0 2
:4 0 11e :2 0 68
:2 0 20 :2 0 770
dbf dc1 :3 0 773
dbb dc3 db8 dc5
dc4 :2 0 58 :3 0
69 :3 0 dc7 dc8
0 5a :4 0 11f
:4 0 5d :2 0 5e
:3 0 777 dcc dce
:3 0 5d :2 0 97
:4 0 77a dd0 dd2
:3 0 5d :2 0 98
:4 0 77d dd4 dd6
:3 0 5d :2 0 91
:3 0 780 dd8 dda
:3 0 5d :2 0 99
:4 0 783 ddc dde
:3 0 5d :2 0 7e
:3 0 9a :3 0 20
:2 0 9b :2 0 786
de1 de5 78a de0
de7 :3 0 78d dc9
de9 :2 0 deb 790
dec dc6 deb 0
ded 792 0 dee
794 df1 :3 0 df1
0 df1 df0 dee
def :6 0 df3 24
:3 0 797 df5 799
df4 df3 :2 0 df6
79b :2 0 df8 0
df8 df7 da5 df6
:6 0 dfa 1 :3 0
79e dfc ca0 d14
0 dfd 0 dfa
0 dfd 7a0 0
dfe 7a4 dff c35
dfe 0 e00 7a8
0 e7e 13 :3 0
10c :2 0 7aa e02
e03 :3 0 e04 :2 0
2f :3 0 120 :3 0
14 :3 0 11 :3 0
121 :2 0 7ac e07
e0b e06 e0c 0
e0e 7b0 e2f 7e
:3 0 1f :3 0 20
:2 0 20 :2 0 7b2
e0f e13 9c :2 0
121 :4 0 7b8 e15
e17 :3 0 2f :3 0
120 :3 0 14 :3 0
11 :3 0 24 :2 0
7bb e1a e1e e19
e1f 0 e21 7bf
e2b 2f :3 0 120
:3 0 14 :3 0 11
:3 0 20 :2 0 7c1
e23 e27 e22 e28
0 e2a 7c5 e2c
e18 e21 0 e2d
0 e2a 0 e2d
7c7 0 e2e 7ca
e30 e05 e0e 0
e31 0 e2e 0
e31 7cc 0 e7e
2f :3 0 121 :2 0
e32 e33 0 e7e
2f :3 0 122 :3 0
11 :3 0 1a :3 0
26 :3 0 2f :3 0
32 :3 0 b :3 0
24 :2 0 62 :3 0
28 :3 0 24 :2 0
7cf e3e e41 7d2
e36 e43 e35 e44
0 e7e 62 :3 0
2f :3 0 24 :2 0
7db e46 e49 d1
:2 0 24 :2 0 7e0
e4b e4d :3 0 c
:3 0 11 :3 0 e4f
e50 0 e78 55
:3 0 56 :3 0 e52
e53 0 55 :3 0
66 :3 0 e55 e56
0 20 :2 0 2
:4 0 123 :2 0 68
:2 0 20 :2 0 7e3
e5b e5d :3 0 7e6
e57 e5f e54 e61
e60 :2 0 58 :3 0
69 :3 0 e63 e64
0 5a :4 0 124
:4 0 5d :2 0 5e
:3 0 7ea e68 e6a
:3 0 7ed e65 e6c
:2 0 e6e 7f0 e6f
e62 e6e 0 e70
7f2 0 e78 d
:3 0 62 :3 0 2f
:3 0 24 :2 0 7f4
e72 e75 e76 :2 0
e78 7f7 e79 e4e
e78 0 e7a 7fb
0 e7e 26 :3 0
2a :3 0 e7b e7c
0 e7e 7fd e7f
8e4 e7e 0 e80
805 0 e81 807
ea5 27 :3 0 fe
:3 0 62 :3 0 ff
:3 0 2a :3 0 80b
e85 e87 24 :2 0
80d e84 e8a 100
:2 0 62 :3 0 ff
:3 0 26 :3 0 810
e8e e90 24 :2 0
812 e8d e93 815
e8c e95 :3 0 e96
:2 0 101 :2 0 102
:2 0 818 e98 e9a
:3 0 e9b :2 0 68
:2 0 103 :2 0 81b
e9d e9f :3 0 81e
e83 ea1 e82 ea2
0 ea4 820 ea6
8c6 e81 0 ea7
0 ea4 0 ea7
822 0 ef3 2f
:3 0 125 :3 0 14
:3 0 15 :3 0 16
:3 0 11 :3 0 1d
:3 0 1f :3 0 26
:3 0 2a :3 0 38
:3 0 29 :3 0 27
:3 0 46 :3 0 49
:3 0 48 :3 0 4a
:3 0 4b :3 0 4d
:3 0 4c :3 0 4e
:3 0 25 :3 0 23
:3 0 2b :3 0 b
:3 0 62 :3 0 2c
:3 0 24 :2 0 825
ec1 ec4 828 ea9
ec6 ea8 ec7 0
ef3 c :3 0 11
:3 0 ec9 eca 0
ef3 55 :3 0 56
:3 0 ecc ecd 0
55 :3 0 66 :3 0
ecf ed0 0 20
:2 0 2 :4 0 126
:2 0 68 :2 0 20
:2 0 841 ed5 ed7
:3 0 844 ed1 ed9
ece edb eda :2 0
58 :3 0 69 :3 0
edd ede 0 5a
:4 0 127 :4 0 5d
:2 0 5e :3 0 848
ee2 ee4 :3 0 84b
edf ee6 :2 0 ee8
84e ee9 edc ee8
0 eea 850 0
ef3 d :3 0 62
:3 0 2f :3 0 24
:2 0 852 eec eef
ef0 :2 0 ef3 d0
:3 0 855 12b0 62
:3 0 e :3 0 24
:2 0 85c ef4 ef7
62 :3 0 d1 :2 0
27 :3 0 24 :2 0
85f ef9 efd 864
efa eff :3 0 8c
:3 0 89 :3 0 e
:3 0 8b :3 0 f0
:3 0 f1 :3 0 8b
:3 0 41 :3 0 128
:3 0 8d :3 0 11
:4 0 129 1 :8 0
f22 f4 :4 0 f0d
:3 0 f5 :3 0 9c
:2 0 869 f10 f11
:3 0 8f :3 0 90
:3 0 f13 f14 0
24 :2 0 f15 f16
0 f18 86c f1f
8f :3 0 90 :3 0
f19 f1a 0 f6
:2 0 f1b f1c 0
f1e 86e f20 f12
f18 0 f21 0
f1e 0 f21 870
0 f22 873 f75
92 :3 0 8f :3 0
90 :3 0 f24 f25
0 91 :3 0 f26
f27 0 f29 876
f2b 878 f2a f29
:2 0 f73 93 :3 0
8f :3 0 90 :3 0
f2e f2f 0 91
:3 0 f30 f31 0
f6b 55 :3 0 56
:3 0 f33 f34 0
55 :3 0 66 :3 0
f36 f37 0 94
:2 0 2 :4 0 12a
:2 0 68 :2 0 20
:2 0 87a f3c f3e
:3 0 87d f38 f40
f35 f42 f41 :2 0
58 :3 0 69 :3 0
f44 f45 0 5a
:4 0 12b :4 0 5d
:2 0 5e :3 0 881
f49 f4b :3 0 5d
:2 0 97 :4 0 884
f4d f4f :3 0 5d
:2 0 98 :4 0 887
f51 f53 :3 0 5d
:2 0 91 :3 0 88a
f55 f57 :3 0 5d
:2 0 99 :4 0 88d
f59 f5b :3 0 5d
:2 0 7e :3 0 9a
:3 0 20 :2 0 9b
:2 0 890 f5e f62
894 f5d f64 :3 0
897 f46 f66 :2 0
f68 89a f69 f43
f68 0 f6a 89c
0 f6b 89e f6e
:3 0 f6e 0 f6e
f6d f6b f6c :6 0
f70 26 :3 0 8a1
f72 8a3 f71 f70
:2 0 f73 8a5 :2 0
f75 0 f75 f74
f22 f73 :6 0 fee
1 :3 0 a1 :3 0
8b :3 0 f0 :3 0
f1 :3 0 8b :3 0
41 :3 0 128 :3 0
6b :3 0 14 :3 0
a2 :3 0 15 :4 0
12c 1 :8 0 f98
f4 :4 0 f83 :3 0
f5 :3 0 9c :2 0
8aa f86 f87 :3 0
8f :3 0 90 :3 0
f89 f8a 0 24
:2 0 f8b f8c 0
f8e 8ad f95 8f
:3 0 90 :3 0 f8f
f90 0 f6 :2 0
f91 f92 0 f94
8af f96 f88 f8e
0 f97 0 f94
0 f97 8b1 0
f98 8b4 feb 92
:3 0 8f :3 0 90
:3 0 f9a f9b 0
91 :3 0 f9c f9d
0 f9f 8b7 fa1
8b9 fa0 f9f :2 0
fe9 93 :3 0 8f
:3 0 90 :3 0 fa4
fa5 0 91 :3 0
fa6 fa7 0 fe1
55 :3 0 56 :3 0
fa9 faa 0 55
:3 0 66 :3 0 fac
fad 0 94 :2 0
2 :4 0 12d :2 0
68 :2 0 20 :2 0
8bb fb2 fb4 :3 0
8be fae fb6 fab
fb8 fb7 :2 0 58
:3 0 69 :3 0 fba
fbb 0 5a :4 0
12e :4 0 5d :2 0
5e :3 0 8c2 fbf
fc1 :3 0 5d :2 0
97 :4 0 8c5 fc3
fc5 :3 0 5d :2 0
98 :4 0 8c8 fc7
fc9 :3 0 5d :2 0
91 :3 0 8cb fcb
fcd :3 0 5d :2 0
99 :4 0 8ce fcf
fd1 :3 0 5d :2 0
7e :3 0 9a :3 0
20 :2 0 9b :2 0
8d1 fd4 fd8 8d5
fd3 fda :3 0 8d8
fbc fdc :2 0 fde
8db fdf fb9 fde
0 fe0 8dd 0
fe1 8df fe4 :3 0
fe4 0 fe4 fe3
fe1 fe2 :6 0 fe6
28 :3 0 8e2 fe8
8e4 fe7 fe6 :2 0
fe9 8e6 :2 0 feb
0 feb fea f98
fe9 :6 0 fee 1
:3 0 d0 :3 0 8e9
fef f00 fee 0
12b1 62 :3 0 10
:3 0 24 :2 0 8ec
ff0 ff3 62 :3 0
d1 :2 0 28 :3 0
24 :2 0 8ef ff5
ff9 8f4 ff6 ffb
:3 0 62 :3 0 10
:3 0 24 :2 0 8f7
ffd 1000 9c :2 0
24 :2 0 8fc 1002
1004 :3 0 28 :3 0
f6 :2 0 1006 1007
0 1009 8ff 1052
62 :3 0 10 :3 0
24 :2 0 901 100a
100d ee :2 0 20
:2 0 906 100f 1011
:3 0 1012 :2 0 62
:3 0 10 :3 0 24
:2 0 909 1014 1017
ae :2 0 12f :2 0
90e 1019 101b :3 0
101c :2 0 1013 101e
101d :2 0 c :3 0
11 :3 0 1020 1021
0 1047 55 :3 0
56 :3 0 1023 1024
0 55 :3 0 66
:3 0 1026 1027 0
20 :2 0 2 :4 0
130 :2 0 68 :2 0
20 :2 0 911 102c
102e :3 0 914 1028
1030 1025 1032 1031
:2 0 58 :3 0 69
:3 0 1034 1035 0
5a :4 0 131 :4 0
5d :2 0 5e :3 0
918 1039 103b :3 0
91b 1036 103d :2 0
103f 91e 1040 1033
103f 0 1041 920
0 1047 d :3 0
41 :3 0 ca :3 0
1043 1044 0 1045
:2 0 1047 922 1048
101f 1047 0 1049
926 0 1051 28
:3 0 62 :3 0 10
:3 0 24 :2 0 928
104b 104e 104a 104f
0 1051 92b 1053
1005 1009 0 1054
0 1051 0 1054
92e 0 12ae 13
:3 0 10c :2 0 931
1056 1057 :3 0 1058
:2 0 7a :3 0 2f
:3 0 8c :3 0 8d
:3 0 13 :4 0 132
1 :8 0 1065 8f
:3 0 90 :3 0 1060
1061 0 91 :3 0
1062 1063 0 1065
933 10b8 92 :3 0
8f :3 0 90 :3 0
1067 1068 0 91
:3 0 1069 106a 0
106c 936 106e 938
106d 106c :2 0 10b6
93 :3 0 8f :3 0
90 :3 0 1071 1072
0 91 :3 0 1073
1074 0 10ae 55
:3 0 56 :3 0 1076
1077 0 55 :3 0
66 :3 0 1079 107a
0 94 :2 0 2
:4 0 133 :2 0 68
:2 0 20 :2 0 93a
107f 1081 :3 0 93d
107b 1083 1078 1085
1084 :2 0 58 :3 0
69 :3 0 1087 1088
0 5a :4 0 134
:4 0 5d :2 0 5e
:3 0 941 108c 108e
:3 0 5d :2 0 97
:4 0 944 1090 1092
:3 0 5d :2 0 98
:4 0 947 1094 1096
:3 0 5d :2 0 91
:3 0 94a 1098 109a
:3 0 5d :2 0 99
:4 0 94d 109c 109e
:3 0 5d :2 0 7e
:3 0 9a :3 0 20
:2 0 9b :2 0 950
10a1 10a5 954 10a0
10a7 :3 0 957 1089
10a9 :2 0 10ab 95a
10ac 1086 10ab 0
10ad 95c 0 10ae
95e 10b1 :3 0 10b1
0 10b1 10b0 10ae
10af :6 0 10b3 2a
:3 0 961 10b5 963
10b4 10b3 :2 0 10b6
965 :2 0 10b8 0
10b8 10b7 1065 10b6
:6 0 10fe 1 :3 0
8f :3 0 90 :3 0
10ba 10bb 0 8f
:3 0 9c :2 0 135
:3 0 10bd 10bf 0
96a 10be 10c1 :3 0
10c2 :2 0 62 :3 0
28 :3 0 24 :2 0
96d 10c4 10c7 62
:3 0 ae :2 0 2f
:3 0 24 :2 0 970
10c9 10cd 975 10ca
10cf :3 0 10d0 :2 0
10c3 10d2 10d1 :2 0
c :3 0 13 :3 0
10d4 10d5 0 10fb
55 :3 0 56 :3 0
10d7 10d8 0 55
:3 0 66 :3 0 10da
10db 0 20 :2 0
2 :4 0 136 :2 0
68 :2 0 20 :2 0
978 10e0 10e2 :3 0
97b 10dc 10e4 10d9
10e6 10e5 :2 0 58
:3 0 69 :3 0 10e8
10e9 0 5a :4 0
137 :4 0 5d :2 0
5e :3 0 97f 10ed
10ef :3 0 982 10ea
10f1 :2 0 10f3 985
10f4 10e7 10f3 0
10f5 987 0 10fb
d :3 0 41 :3 0
138 :3 0 10f7 10f8
0 10f9 :2 0 10fb
989 10fc 10d3 10fb
0 10fd 98d 0
10fe 98f 10ff 1059
10fe 0 1100 992
0 12ae 2e :3 0
eb :2 0 20 :2 0
994 1102 1104 :3 0
1101 1105 0 12ae
2f :3 0 ec :3 0
14 :3 0 15 :3 0
11 :3 0 17 :3 0
19 :3 0 46 :3 0
48 :3 0 2e :3 0
25 :3 0 23 :3 0
996 1108 1113 1107
1114 0 12ae 62
:3 0 2b :3 0 24
:2 0 9a1 1116 1119
ae :2 0 24 :2 0
9a6 111b 111d :3 0
111e :2 0 1d :3 0
10c :2 0 9a9 1121
1122 :3 0 1123 :2 0
1124 :2 0 111f 1126
1125 :2 0 2f :3 0
139 :3 0 14 :3 0
15 :3 0 16 :3 0
13 :3 0 11 :3 0
1d :3 0 2b :3 0
24 :2 0 9ab 1129
1132 1128 1133 0
1135 9b4 1136 1127
1135 0 1137 9b6
0 12ae 13 :3 0
10c :2 0 9b8 1139
113a :3 0 113b :2 0
2f :3 0 120 :3 0
14 :3 0 11 :3 0
121 :2 0 9ba 113e
1142 113d 1143 0
1145 9be 1166 7e
:3 0 1f :3 0 20
:2 0 20 :2 0 9c0
1146 114a 9c :2 0
121 :4 0 9c6 114c
114e :3 0 2f :3 0
120 :3 0 14 :3 0
11 :3 0 24 :2 0
9c9 1151 1155 1150
1156 0 1158 9cd
1162 2f :3 0 120
:3 0 14 :3 0 11
:3 0 20 :2 0 9cf
115a 115e 1159 115f
0 1161 9d3 1163
114f 1158 0 1164
0 1161 0 1164
9d5 0 1165 9d8
1167 113c 1145 0
1168 0 1165 0
1168 9da 0 12ae
1d :3 0 10c :2 0
9dd 116a 116b :3 0
116c :2 0 b3 :3 0
118 :3 0 7d :3 0
7e :3 0 1d :3 0
b3 :3 0 119 :3 0
1a :3 0 1a :3 0
ba :4 0 13a 1
:8 0 117e 8f :3 0
90 :3 0 1179 117a
0 91 :3 0 117b
117c 0 117e 9df
11d1 92 :3 0 8f
:3 0 90 :3 0 1180
1181 0 91 :3 0
1182 1183 0 1185
9e2 1187 9e4 1186
1185 :2 0 11cf 93
:3 0 8f :3 0 90
:3 0 118a 118b 0
91 :3 0 118c 118d
0 11c7 55 :3 0
56 :3 0 118f 1190
0 55 :3 0 66
:3 0 1192 1193 0
94 :2 0 2 :4 0
13b :2 0 68 :2 0
20 :2 0 9e6 1198
119a :3 0 9e9 1194
119c 1191 119e 119d
:2 0 58 :3 0 69
:3 0 11a0 11a1 0
5a :4 0 13c :4 0
5d :2 0 5e :3 0
9ed 11a5 11a7 :3 0
5d :2 0 97 :4 0
9f0 11a9 11ab :3 0
5d :2 0 98 :4 0
9f3 11ad 11af :3 0
5d :2 0 91 :3 0
9f6 11b1 11b3 :3 0
5d :2 0 99 :4 0
9f9 11b5 11b7 :3 0
5d :2 0 7e :3 0
9a :3 0 20 :2 0
9b :2 0 9fc 11ba
11be a00 11b9 11c0
:3 0 a03 11a2 11c2
:2 0 11c4 a06 11c5
119f 11c4 0 11c6
a08 0 11c7 a0a
11ca :3 0 11ca 0
11ca 11c9 11c7 11c8
:6 0 11cc 2c :3 0
a0d 11ce a0f 11cd
11cc :2 0 11cf a11
:2 0 11d1 0 11d1
11d0 117e 11cf :6 0
11e2 1 :3 0 26
:3 0 62 :3 0 26
:3 0 24 :2 0 a14
11d4 11d7 eb :2 0
62 :3 0 29 :3 0
24 :2 0 a17 11da
11dd a1a 11d9 11df
:3 0 11d3 11e0 0
11e2 a1d 11e3 116d
11e2 0 11e4 a20
0 12ae 32 :3 0
24 :2 0 11e5 11e6
0 12ae 2f :3 0
121 :2 0 11e8 11e9
0 12ae 2f :3 0
122 :3 0 11 :3 0
1a :3 0 26 :3 0
2f :3 0 32 :3 0
b :3 0 24 :2 0
62 :3 0 28 :3 0
24 :2 0 a22 11f4
11f7 a25 11ec 11f9
11eb 11fa 0 12ae
62 :3 0 2f :3 0
24 :2 0 a2e 11fc
11ff d1 :2 0 24
:2 0 a33 1201 1203
:3 0 c :3 0 11
:3 0 1205 1206 0
122c 55 :3 0 56
:3 0 1208 1209 0
55 :3 0 66 :3 0
120b 120c 0 20
:2 0 2 :4 0 13d
:2 0 68 :2 0 20
:2 0 a36 1211 1213
:3 0 a39 120d 1215
120a 1217 1216 :2 0
58 :3 0 69 :3 0
1219 121a 0 5a
:4 0 13e :4 0 5d
:2 0 5e :3 0 a3d
121e 1220 :3 0 a40
121b 1222 :2 0 1224
a43 1225 1218 1224
0 1226 a45 0
122c d :3 0 41
:3 0 138 :3 0 1228
1229 0 122a :2 0
122c a47 12ab 8c
:3 0 ef :3 0 7a
:3 0 28 :3 0 76
:3 0 78 :3 0 75
:3 0 13f :3 0 8b
:3 0 f0 :3 0 f1
:3 0 8b :3 0 41
:3 0 f2 :3 0 41
:3 0 140 :3 0 8d
:3 0 11 :4 0 141
1 :8 0 1255 f4
:4 0 1240 :3 0 f5
:3 0 9c :2 0 a4d
1243 1244 :3 0 8f
:3 0 90 :3 0 1246
1247 0 24 :2 0
1248 1249 0 124b
a50 1252 8f :3 0
90 :3 0 124c 124d
0 f6 :2 0 124e
124f 0 1251 a52
1253 1245 124b 0
1254 0 1251 0
1254 a54 0 1255
a57 12a8 92 :3 0
8f :3 0 90 :3 0
1257 1258 0 91
:3 0 1259 125a 0
125c a5a 125e a5c
125d 125c :2 0 12a6
93 :3 0 8f :3 0
90 :3 0 1261 1262
0 91 :3 0 1263
1264 0 129e 55
:3 0 56 :3 0 1266
1267 0 55 :3 0
66 :3 0 1269 126a
0 94 :2 0 2
:4 0 142 :2 0 68
:2 0 20 :2 0 a5e
126f 1271 :3 0 a61
126b 1273 1268 1275
1274 :2 0 58 :3 0
69 :3 0 1277 1278
0 5a :4 0 143
:4 0 5d :2 0 5e
:3 0 a65 127c 127e
:3 0 5d :2 0 97
:4 0 a68 1280 1282
:3 0 5d :2 0 98
:4 0 a6b 1284 1286
:3 0 5d :2 0 91
:3 0 a6e 1288 128a
:3 0 5d :2 0 99
:4 0 a71 128c 128e
:3 0 5d :2 0 7e
:3 0 9a :3 0 20
:2 0 9b :2 0 a74
1291 1295 a78 1290
1297 :3 0 a7b 1279
1299 :2 0 129b a7e
129c 1276 129b 0
129d a80 0 129e
a82 12a1 :3 0 12a1
0 12a1 12a0 129e
129f :6 0 12a3 2e
:3 0 a85 12a5 a87
12a4 12a3 :2 0 12a6
a89 :2 0 12a8 0
12a8 12a7 1255 12a6
:6 0 12aa 1 :3 0
a8c 12ac 1204 122c
0 12ad 0 12aa
0 12ad a8e 0
12ae a91 12af ffc
12ae 0 12b1 8b6
ef3 0 12b1 a9d
0 12b2 aa1 12b4
2e9 66d 0 12b5
0 12b2 0 12b5
aab 0 12f0 8f
:3 0 90 :3 0 12b6
12b7 0 144 :3 0
14 :3 0 aae 12b9
12bb 12b8 12bc 0
12f0 55 :3 0 56
:3 0 12be 12bf 0
55 :3 0 66 :3 0
12c1 12c2 0 20
:2 0 2 :4 0 145
:2 0 68 :2 0 20
:2 0 ab0 12c7 12c9
:3 0 ab3 12c3 12cb
12c0 12cd 12cc :2 0
58 :3 0 69 :3 0
12cf 12d0 0 5a
:4 0 146 :4 0 5d
:2 0 5e :3 0 ab7
12d4 12d6 :3 0 5d
:2 0 97 :4 0 aba
12d8 12da :3 0 5d
:2 0 c9 :4 0 abd
12dc 12de :3 0 5d
:2 0 8f :3 0 90
:3 0 12e1 12e2 0
ac0 12e0 12e4 :3 0
ac3 12d1 12e6 :2 0
12e8 ac6 12e9 12ce
12e8 0 12ea ac8
0 12f0 d :3 0
8f :3 0 90 :3 0
12ec 12ed 0 12ee
:2 0 12f0 aca 12f3
:3 0 12f3 ad3 12f3
12f2 12f0 12f1 :6 0
12f4 :2 0 2 24
12f3 12f7 :3 0 12f6
12f4 12f8 :8 0 
b10
4
:3 0 1 3 1
9 1 d 1
11 1 15 1
1b 6 8 c
10 14 1a 20
1 26 1 2c
1 32 1 3a
1 38 1 41
1 3f 1 48
1 46 1 4f
1 4d 1 56
1 54 1 5b
1 60 1 65
1 6a 1 6f
1 74 1 7b
1 79 1 82
1 80 1 89
1 87 1 8e
1 94 1 9a
1 a0 1 a6
1 ac 1 b2
1 b8 1 be
1 c4 1 ca
1 d0 1 d6
1 dc 1 e2
1 e8 1 ee
1 f4 1 fa
1 100 1 106
1 10c 1 112
1 118 1 11e
1 124 1 12c
1 12a 1 136
1 134 1 140
1 13e 1 145
1 14b 1 151
1 157 1 15d
1 163 1 169
1 16f 1 175
1 17b 1 181
1 187 1 18d
1 193 1 19f
1 1ab 2 1ae
1b0 2 1b3 1b5
2 1b7 1b9 2
1bb 1bd 2 1bf
1c1 1 1c7 2
1c9 1ca 2 1c3
1cc 2 1ce 1d0
1 1d6 2 1d8
1d9 2 1d2 1db
2 1dd 1df 1
1e5 2 1e7 1e8
2 1e1 1ea 2
1ec 1ee 2 1f0
1f2 2 1f4 1f6
2 1f8 1fa 2
1fc 1fe 2 200
202 2 204 206
2 208 20a 2
20c 20e 4 1a7
1ad 1b2 210 1
212 1 214 2
21e 220 3 21c
21d 222 2 22b
22d 2 22a 22f
1 231 1 233
2 285 28a 1
291 1 28c 2
2a4 2a6 3 2a2
2a3 2a8 2 2b1
2b3 2 2b5 2b7
2 2b9 2bb 2
2bd 2bf 2 2c1
2c3 3 2c8 2c9
2ca 2 2c5 2cc
2 2b0 2ce 1
2d0 1 2d2 2
29b 2d3 1 2d7
1 296 2 294
2db 1 2e6 2
2e2 2e6 2 2f9
2fe 1 305 1
300 2 318 31a
3 316 317 31c
2 325 327 2
329 32b 2 32d
32f 2 331 333
2 335 337 3
33c 33d 33e 2
339 340 2 324
342 1 344 1
346 2 30f 347
1 34b 1 30a
2 308 34f 1
35a 2 356 35a
2 369 36b 3
367 368 36d 2
376 378 2 375
37a 1 37c 1
37e 3 360 37f
384 1 386 2
390 395 1 39c
1 397 2 3af
3b1 3 3ad 3ae
3b3 2 3bc 3be
2 3c0 3c2 2
3c4 3c6 2 3c8
3ca 2 3cc 3ce
3 3d3 3d4 3d5
2 3d0 3d7 2
3bb 3d9 1 3db
1 3dd 2 3a6
3de 1 3e2 1
3a1 2 39f 3e6
2 3ec 3ed 1
3f1 2 3ef 3f1
2 3f9 3fe 1
405 1 400 2
418 41a 3 416
417 41c 2 425
427 2 429 42b
2 42d 42f 2
431 433 2 435
437 3 43c 43d
43e 2 439 440
2 424 442 1
444 1 446 2
40f 447 1 44b
1 40a 2 408
44f 2 457 458
1 45c 2 45a
45c 2 46a 46c
3 468 469 46e
2 477 479 2
476 47b 1 47d
1 47f 3 461
480 485 1 487
2 452 488 1
48a 2 4a7 4ac
1 4b3 1 4ae
2 4c6 4c8 3
4c4 4c5 4ca 2
4d3 4d5 2 4d7
4d9 2 4db 4dd
2 4df 4e1 2
4e3 4e5 3 4ea
4eb 4ec 2 4e7
4ee 2 4d2 4f0
1 4f2 1 4f4
2 4bd 4f5 1
4f9 1 4b8 2
4b6 4fd 2 503
504 1 508 2
506 508 2 516
518 3 514 515
51a 2 523 525
2 522 527 1
529 1 52b 3
50d 52c 531 1
533 2 536 537
1 53b 2 539
53b 2 546 548
3 544 545 54a
2 553 555 2
557 559 2 55b
55d 1 565 2
567 568 2 55f
56a 2 552 56c
1 56e 1 570
2 57a 57c 3
578 579 57e 2
587 589 2 586
58b 1 58d 1
58f 2 597 598
3 571 590 59b
2 59f 5a0 2
5a5 5a6 1 5a8
2 5a2 5a8 2
5af 5b0 2 5b5
5b6 1 5b8 2
5b2 5b8 2 5c6
5c8 3 5c4 5c5
5ca 2 5d3 5d5
2 5d2 5d7 1
5d9 1 5db 3
5bd 5dc 5e1 2
5e5 5e6 2 5eb
5ec 1 5ee 2
5e8 5ee 1 5f3
2 5f6 5f5 2
602 603 5 5fd
5fe 5ff 600 605
4 5ad 5f7 5fa
608 2 613 615
3 611 612 617
2 620 622 2
624 626 2 628
62a 1 632 2
634 635 2 62c
637 2 61f 639
1 63b 1 63d
2 647 649 3
645 646 64b 2
654 656 2 653
658 1 65a 1
65c 2 664 665
3 63e 65d 668
3 66a 60a 66b
8 352 387 38a
3e9 48b 500 534
66c 2 66f 670
1 674 2 672
674 2 682 684
3 680 681 686
2 68f 691 2
68e 693 1 695
1 697 3 679
698 69d 1 69f
2 6a9 6ae 1
6b5 1 6b0 2
6c8 6ca 3 6c6
6c7 6cc 2 6d5
6d7 2 6d9 6db
2 6dd 6df 2
6e1 6e3 2 6e5
6e7 3 6ec 6ed
6ee 2 6e9 6f0
2 6d4 6f2 1
6f4 1 6f6 2
6bf 6f7 1 6fb
1 6ba 2 6b8
6ff 2 705 706
1 70a 2 708
70a 2 712 717
1 71e 1 719
2 731 733 3
72f 730 735 2
73e 740 2 742
744 2 746 748
2 74a 74c 2
74e 750 3 755
756 757 2 752
759 2 73d 75b
1 75d 1 75f
2 728 760 1
764 1 723 2
721 768 2 770
771 1 775 2
773 775 2 783
785 3 781 782
787 2 790 792
2 78f 794 1
796 1 798 3
77a 799 79e 1
7a0 2 76b 7a1
1 7a3 2 7b1
7b6 1 7bd 1
7b8 2 7d0 7d2
3 7ce 7cf 7d4
2 7dd 7df 2
7e1 7e3 2 7e5
7e7 2 7e9 7eb
2 7ed 7ef 3
7f4 7f5 7f6 2
7f1 7f8 2 7dc
7fa 1 7fc 1
7fe 2 7c7 7ff
1 803 1 7c2
2 7c0 807 2
80d 80e 1 812
2 810 812 2
820 822 3 81e
81f 824 2 82d
82f 2 82c 831
1 833 1 835
3 817 836 83b
1 83d 2 848
84d 1 854 1
84f 2 867 869
3 865 866 86b
2 874 876 2
878 87a 2 87c
87e 2 880 882
2 884 886 3
88b 88c 88d 2
888 88f 2 873
891 1 893 1
895 2 85e 896
1 89a 1 859
2 857 89e 2
8a5 8a6 2 8ab
8ac 2 8b1 8b2
1 8b4 2 8ae
8b4 2 8bb 8bc
2 8c1 8c2 1
8c4 2 8be 8c4
1 8c9 a 8cf
8d0 8d1 8d2 8d3
8d4 8d5 8d6 8d7
8d8 2 8dd 8de
1 8e2 2 8e0
8e2 2 8e6 8e7
2 8ec 8ed 1
8ef 2 8e9 8ef
2 8f5 8f6 1
8fa 2 8f8 8fa
2 8ff 900 2
905 906 2 902
908 2 910 911
2 916 917 2
91c 91d 2 919
91f 1 922 2
913 922 2 927
928 2 92d 92e
2 92a 930 2
936 937 2 933
939 1 951 2
950 951 1 959
1 95f 2 961
962 2 94e 963
1 96a 1 965
2 97d 97f 3
97b 97c 981 2
98a 98c 2 98e
990 2 992 994
2 996 998 2
99a 99c 3 9a1
9a2 9a3 2 99e
9a5 2 989 9a7
1 9a9 1 9ab
2 974 9ac 1
9b0 1 96f 2
96d 9b4 2 9be
9bf 2 9c1 9c4
3 93c 9b7 9c7
1 9dc 2 9db
9dc 1 9e4 1
9ea 2 9ec 9ed
2 9d9 9ee 1
9f5 1 9f0 2
a08 a0a 3 a06
a07 a0c 2 a15
a17 2 a19 a1b
2 a1d a1f 2
a21 a23 2 a25
a27 3 a2c a2d
a2e 2 a29 a30
2 a14 a32 1
a34 1 a36 2
9ff a37 1 a3b
1 9fa 2 9f8
a3f 1 a42 2
a45 a46 3 90b
90e a47 2 a4a
a4b 2 a50 a51
2 a56 a57 2
a53 a59 1 a5c
2 a4d a5c 2
a61 a62 2 a67
a68 2 a64 a6a
1 a72 2 a74
a75 1 a7b 2
a7d a7e 2 a77
a80 2 a83 a85
2 a88 a8a 1
a8c 1 aa1 2
aa0 aa1 1 aa9
1 aaf 2 ab1
ab2 2 a9e ab3
1 aba 1 ab5
2 acd acf 3
acb acc ad1 2
ada adc 2 ade
ae0 2 ae2 ae4
2 ae6 ae8 2
aea aec 3 af1
af2 af3 2 aee
af5 2 ad9 af7
1 af9 1 afb
2 ac4 afc 1
b00 1 abf 2
abd b04 3 a6d
a8f b07 2 b0c
b0d 2 b12 b13
2 b0f b15 1
b2f 2 b2e b2f
1 b37 1 b3d
2 b3f b40 2
b2c b41 1 b48
1 b43 2 b5b
b5d 3 b59 b5a
b5f 2 b68 b6a
2 b6c b6e 2
b70 b72 2 b74
b76 2 b78 b7a
3 b7f b80 b81
2 b7c b83 2
b67 b85 1 b87
1 b89 2 b52
b8a 1 b8e 1
b4d 2 b4b b92
3 b18 b1b b95
2 b98 b99 1
b9a 2 b9c b9d
1 b9e 2 ba2
ba3 2 ba8 ba9
2 ba5 bab 1
bc5 2 bc4 bc5
1 bcd 1 bd3
2 bd5 bd6 2
bc2 bd7 1 bde
1 bd9 2 bf1
bf3 3 bef bf0
bf5 2 bfe c00
2 c02 c04 2
c06 c08 2 c0a
c0c 2 c0e c10
3 c15 c16 c17
2 c12 c19 2
bfd c1b 1 c1d
1 c1f 2 be8
c20 1 c24 1
be3 2 be1 c28
3 bae bb1 c2b
2 c2e c2f 1
c31 2 c41 c46
1 c4d 1 c48
2 c60 c62 3
c5e c5f c64 2
c6d c6f 2 c71
c73 2 c75 c77
2 c79 c7b 2
c7d c7f 3 c84
c85 c86 2 c81
c88 2 c6c c8a
1 c8c 1 c8e
2 c57 c8f 1
c93 1 c52 2
c50 c97 1 c9c
2 ca6 cab 1
cb2 1 cad 2
cc5 cc7 3 cc3
cc4 cc9 2 cd2
cd4 2 cd6 cd8
2 cda cdc 2
cde ce0 2 ce2
ce4 3 ce9 cea
ceb 2 ce6 ced
2 cd1 cef 1
cf1 1 cf3 2
cbc cf4 1 cf8
1 cb7 2 cb5
cfc 3 d04 d05
d06 1 d0a 2
d08 d0a 1 d0f
1 d11 2 cff
d12 2 d16 d17
1 d1b 2 d19
d1b 2 d23 d24
2 d26 d29 2
d38 d3d 1 d44
1 d3f 2 d57
d59 3 d55 d56
d5b 2 d64 d66
2 d68 d6a 2
d6c d6e 2 d70
d72 2 d74 d76
3 d7b d7c d7d
2 d78 d7f 2
d63 d81 1 d83
1 d85 2 d4e
d86 1 d8a 1
d49 2 d47 d8e
2 d2d d91 2
d9f da4 1 dab
1 da6 2 dbe
dc0 3 dbc dbd
dc2 2 dcb dcd
2 dcf dd1 2
dd3 dd5 2 dd7
dd9 2 ddb ddd
3 de2 de3 de4
2 ddf de6 2
dca de8 1 dea
1 dec 2 db5
ded 1 df1 1
db0 2 dae df5
1 df8 3 dfb
d94 dfc 3 c38
c9a dfd 1 dff
1 e01 3 e08
e09 e0a 1 e0d
3 e10 e11 e12
1 e16 2 e14
e16 3 e1b e1c
e1d 1 e20 3
e24 e25 e26 1
e29 2 e2b e2c
1 e2d 2 e2f
e30 2 e3f e40
8 e37 e38 e39
e3a e3b e3c e3d
e42 2 e47 e48
1 e4c 2 e4a
e4c 2 e5a e5c
3 e58 e59 e5e
2 e67 e69 2
e66 e6b 1 e6d
1 e6f 2 e73
e74 3 e51 e70
e77 1 e79 7
c30 e00 e31 e34
e45 e7a e7d 1
e7f 3 8cc 8db
e80 1 e86 2
e88 e89 1 e8f
2 e91 e92 2
e8b e94 2 e97
e99 2 e9c e9e
1 ea0 1 ea3
2 ea5 ea6 2
ec2 ec3 18 eaa
eab eac ead eae
eaf eb0 eb1 eb2
eb3 eb4 eb5 eb6
eb7 eb8 eb9 eba
ebb ebc ebd ebe
ebf ec0 ec5 2
ed4 ed6 3 ed2
ed3 ed8 2 ee1
ee3 2 ee0 ee5
1 ee7 1 ee9
2 eed eee 6
8b9 ea7 ec8 ecb
eea ef1 2 ef5
ef6 2 efb efc
1 efe 2 ef8
efe 1 f0f 2
f0e f0f 1 f17
1 f1d 2 f1f
f20 2 f0c f21
1 f28 1 f23
2 f3b f3d 3
f39 f3a f3f 2
f48 f4a 2 f4c
f4e 2 f50 f52
2 f54 f56 2
f58 f5a 3 f5f
f60 f61 2 f5c
f63 2 f47 f65
1 f67 1 f69
2 f32 f6a 1
f6e 1 f2d 2
f2b f72 1 f85
2 f84 f85 1
f8d 1 f93 2
f95 f96 2 f82
f97 1 f9e 1
f99 2 fb1 fb3
3 faf fb0 fb5
2 fbe fc0 2
fc2 fc4 2 fc6
fc8 2 fca fcc
2 fce fd0 3
fd5 fd6 fd7 2
fd2 fd9 2 fbd
fdb 1 fdd 1
fdf 2 fa8 fe0
1 fe4 1 fa3
2 fa1 fe8 2
f75 feb 2 ff1
ff2 2 ff7 ff8
1 ffa 2 ff4
ffa 2 ffe fff
1 1003 2 1001
1003 1 1008 2
100b 100c 1 1010
2 100e 1010 2
1015 1016 1 101a
2 1018 101a 2
102b 102d 3 1029
102a 102f 2 1038
103a 2 1037 103c
1 103e 1 1040
3 1022 1041 1046
1 1048 2 104c
104d 2 1049 1050
2 1052 1053 1
1055 2 105f 1064
1 106b 1 1066
2 107e 1080 3
107c 107d 1082 2
108b 108d 2 108f
1091 2 1093 1095
2 1097 1099 2
109b 109d 3 10a2
10a3 10a4 2 109f
10a6 2 108a 10a8
1 10aa 1 10ac
2 1075 10ad 1
10b1 1 1070 2
106e 10b5 1 10c0
2 10bc 10c0 2
10c5 10c6 2 10cb
10cc 1 10ce 2
10c8 10ce 2 10df
10e1 3 10dd 10de
10e3 2 10ec 10ee
2 10eb 10f0 1
10f2 1 10f4 3
10d6 10f5 10fa 1
10fc 2 10b8 10fd
1 10ff 1 1103
a 1109 110a 110b
110c 110d 110e 110f
1110 1111 1112 2
1117 1118 1 111c
2 111a 111c 1
1120 8 112a 112b
112c 112d 112e 112f
1130 1131 1 1134
1 1136 1 1138
3 113f 1140 1141
1 1144 3 1147
1148 1149 1 114d
2 114b 114d 3
1152 1153 1154 1
1157 3 115b 115c
115d 1 1160 2
1162 1163 1 1164
2 1166 1167 1
1169 2 1178 117d
1 1184 1 117f
2 1197 1199 3
1195 1196 119b 2
11a4 11a6 2 11a8
11aa 2 11ac 11ae
2 11b0 11b2 2
11b4 11b6 3 11bb
11bc 11bd 2 11b8
11bf 2 11a3 11c1
1 11c3 1 11c5
2 118e 11c6 1
11ca 1 1189 2
1187 11ce 2 11d5
11d6 2 11db 11dc
2 11d8 11de 2
11d1 11e1 1 11e3
2 11f5 11f6 8
11ed 11ee 11ef 11f0
11f1 11f2 11f3 11f8
2 11fd 11fe 1
1202 2 1200 1202
2 1210 1212 3
120e 120f 1214 2
121d 121f 2 121c
1221 1 1223 1
1225 3 1207 1226
122b 1 1242 2
1241 1242 1 124a
1 1250 2 1252
1253 2 123f 1254
1 125b 1 1256
2 126e 1270 3
126c 126d 1272 2
127b 127d 2 127f
1281 2 1283 1285
2 1287 1289 2
128b 128d 3 1292
1293 1294 2 128f
1296 2 127a 1298
1 129a 1 129c
2 1265 129d 1
12a1 1 1260 2
125e 12a5 1 12a8
2 12ab 12ac b
1054 1100 1106 1115
1137 1168 11e4 11e7
11ea 11fb 12ad 3
12b0 fef 12af 9
6a0 6a3 702 7a4
80a 83e 8a1 8a9
12b1 2 12b3 12b4
1 12ba 2 12c6
12c8 3 12c4 12c5
12ca 2 12d3 12d5
2 12d7 12d9 2
12db 12dd 2 12df
12e3 2 12d2 12e5
1 12e7 1 12e9
8 215 234 237
2de 12b5 12bd 12ea
12ef 3c 2a 30
36 3d 44 4b
52 59 5e 63
68 6d 72 77
7e 85 8c 92
98 9e a4 aa
b0 b6 bc c2
c8 ce d4 da
e0 e6 ec f2
f8 fe 104 10a
110 116 11c 122
128 132 13c 143
149 14f 155 15b
161 167 16d 173
179 17f 185 18b
191 197 
1
4
0 
12f7
0
1
50
2f
43
0 1 2 1 4 1 6 1
8 1 a 1 c 1 e 1
10 1 12 1 14 1 16 1
18 1 1a 1 1c 1 1e 1
20 1 22 1 24 1 26 1
28 1 2a 1 2c 1 2e 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0

106 1 0
1b 1 0
16f 1 0
11e 1 0
d0 1 0
ee 1 0
6a 1 0
118 1 0
ca 1 0
b2 1 0
112 1 0
b8 1 0
134 1 0
80 1 0
175 1 0
14b 1 0
17b 1 0
3 1 0
26 1 0
151 1 0
12a 1 0
d6 1 0
d 1 0
a6 1 0
fa 1 0
6f 1 0
9a 1 0
87 1 0
a0 1 0
65 1 0
15d 1 0
8e 1 0
9 1 0
e8 1 0
124 1 0
11 1 0
60 1 0
169 1 0
2c 1 0
94 1 0
4d 1 0
38 1 0
ac 1 0
f4 1 0
2 0 1
100 1 0
32 1 0
10c 1 0
79 1 0
74 1 0
157 1 0
c4 1 0
54 1 0
46 1 0
be 1 0
3f 1 0
145 1 0
5b 1 0
181 1 0
187 1 0
dc 1 0
15 1 0
18d 1 0
193 1 0
13e 1 0
e2 1 0
163 1 0
0

/
SHOW ERRORS FUNCTION FC_U_WP_ACTUAL;
ALTER FUNCTION FC_U_WP_ACTUAL COMPILE DEBUG;
/******************************************************************************************************************
update Actual Work, %W, %C in Work Packages
assign_id:      TSK_ASSIGNMENT_ID or TASK_ID
*******************************************************************************************************************/
Prompt == FUNCTION SP_U_WP_ACTUAL ==
CREATE OR REPLACE FUNCTION SP_U_WP_ACTUAL wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
4d
2 :e:
1FUNCTION:
1SP_U_WP_ACTUAL:
1ASSIGN_ID:
1VARCHAR2:
1P_WORK_PERCENT:
1PLS_INTEGER:
1P_ACTUAL:
1P_UNITS:
1RES_ID001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_ASSIGN_ID:
132:
1H_NEW_WORK_PERCENT:
10:
1H_ACTUAL:
1H_UNITS:
1H_REC_USER:
1H_ERROR_ID:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_U_WP_ACTUAL(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
11:
128:
1+:
1FPUT:
1               ENTER:: SP_U_WP_ACTUAL(29) :: cplan.sql(15942) :: :
1SC_STATE:
1FC_U_WP_ACTUAL:
14:
143:
1       RETURN_STATUS:: SP_U_WP_ACTUAL(44) :: cplan.sql(15957) :: :
1 :
1STATE:: :
1 ID:: :
1PMODEFINES:
1PMO_RETURN_STATUS:
1TRUE:
1RETURN_ERROR_EXCEPTION:
116:
150:
1           EXCEPTION:: SP_U_WP_ACTUAL(51) :: cplan.sql(15964) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1ID:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1OTHERS:
1SQLCODE:
162:
1           EXCEPTION:: SP_U_WP_ACTUAL(63) :: cplan.sql(15976) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
1ec
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 :2 a0 6b 1c 81
b0 :2 a0 6b :2 a0 6b 6e a5 b
a 10 :2 a0 6b 6e :2 a0 6b 6e
a5 b 6e 7e a0 b4 2e 6e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c a0
4d d :2 a0 d :2 a0 d :2 a0 d
:2 a0 d :2 a0 d :2 a0 6b :7 a0 a5
b d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 :2 a0
6b :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 :2 a0 6b :2 6e 7e :2 a0 6b b4
2e 7e 6e b4 2e 7e :2 a0 6b
b4 2e a5 57 b7 19 3c :9 a0
12a e9 dd :2 a0 65 b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c :7 a0 12a e9
dd :2 a0 65 b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 68 4f
1d 17 b5 
1ec
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 6f 6b 59 77 56
7c 80 84 88 8b 8f ac 97
9b 9e 9f a7 96 c8 b7 bb
93 c3 b6 e4 d3 d7 b3 df
d2 100 ef f3 cf fb ee 11d
10b eb 10f 110 118 10a 13a 128
107 12c 12d 135 127 15a 145 149
124 14d 155 144 161 165 141 169
16d 171 174 179 17a 1 17c 181
185 189 18c 191 195 199 19c 1a1
1a2 1a4 1a9 1ac 1b0 1b1 1b6 1bb
1be 1c3 1c4 1c9 1cc 1d0 1d1 1d6
1d9 1de 1df 1e4 1e7 1ec 1ed 1f2
1f5 1f9 1fd 201 202 204 209 20a
20c 20d 212 215 21a 21b 220 223
227 22b 22f 230 232 237 238 23a
23b 240 243 248 249 24e 251 255
259 25d 25e 260 265 266 268 269
26e 271 276 277 27c 27f 284 285
28a 28d 291 292 297 29a 29f 2a0
2a5 2a8 2ad 2ae 2b3 2b4 2b9 2bb
2bf 2c2 2c6 2ca 2cd 2d1 2d5 2d8
2db 2e0 2e3 2e6 2e9 2ea 2ef 2f0
1 2f2 2f7 2fb 2ff 302 307 30c
30f 313 314 319 31a 31f 321 325
328 32c 32d 331 335 339 33d 341
345 349 34d 351 355 359 35d 361
365 369 36d 371 375 378 37c 380
384 388 38c 390 394 395 397 39b
39f 3a3 3a6 3aa 3ae 3b1 3b4 3b9
3bc 3bf 3c2 3c3 3c8 3c9 1 3cb
3d0 3d4 3d8 3db 3e0 3e5 3e8 3ec
3ed 3f2 3f5 3fa 3fb 400 403 408
409 40e 411 415 419 41d 421 424
425 427 42c 42d 42f 430 435 438
43d 43e 443 446 44b 44c 451 454
458 459 45e 461 466 467 46c 46d
472 474 478 47b 47f 483 486 48a
48e 491 495 499 49a 49f 4a1 4a5
4a9 4ac 4b0 4b4 4b7 4bb 4bf 4c2
4c5 4ca 4cd 4d0 4d3 4d4 4d9 4da
1 4dc 4e1 4e5 4e9 4ec 4f1 4f6
4f9 4fd 4fe 503 504 509 50d 511
514 519 51e 521 525 529 52c 52d
532 535 53a 53b 540 543 547 54b
54e 54f 554 555 55a 55c 560 563
567 56b 56f 573 577 57b 57f 583
587 593 598 59c 5a0 5a4 5a8 5aa
5ab 5b0 1 5b4 5b8 5bc 5bf 5c3
5c7 5cb 5cf 5d2 5d6 5da 5dd 5e0
5e5 5e8 5eb 5ee 5ef 5f4 5f5 1
5f7 5fc 600 604 607 60c 611 614
618 619 61e 61f 624 628 62c 62f
634 639 63c 640 641 646 649 64e
64f 654 657 65b 65f 662 665 666
668 669 66e 66f 674 676 67a 67d
681 685 689 68d 691 695 699 6a5
6aa 6ae 6b2 6b6 6ba 6bc 6c0 6c2
6ce 6d0 6d2 6d3 6d8 6dc 6de 6ea
6ee 6f0 6f1 6fa 
1ec
2
0 1 a 6 10 :3 6 15 :3 6
f :3 6 e :3 6 10 :2 6 1 3
a 13 a :2 1 4 10 19 18
:2 10 :2 4 :2 17 26 17 :2 4 :2 d 1c
d :2 4 :2 c 1b c :2 4 f 18
17 :2 f :2 4 f 18 17 :2 f :2 4
e 17 :3 e 4 7 :2 11 23 :2 2d
3a :2 23 :3 7 :2 d 19 25 :2 2f 43
:2 25 56 5c 5f :2 56 6d 85 8
:2 6d d 10 :2 6d 1a 1d :2 6d b
e :2 6d 13 16 1f 27 :2 1f 38
:2 16 :2 6d b e :2 6d 13 16 1f
27 :2 1f 32 :2 16 :2 6d b e :2 6d
13 16 1f 27 :2 1f 31 :2 16 :2 6d
b e :2 6d 13 16 :2 6d 1b 1e
:2 6d 28 2b :2 6d 30 33 :2 6d :2 7
4c :2 4 7 :2 11 23 :2 2d 39 3c
4e 50 51 :2 4e :2 23 :3 7 :2 d 12
1e 5f 62 :2 1e :2 7 54 :3 4 12
:2 4 13 :2 4 1a :2 4 10 :2 4 f
:2 4 12 :2 4 :2 d 19 28 35 49
53 5c 27 :2 19 4 7 :2 11 23
:2 2d 39 3c 4e 50 51 :2 4e :2 23
:3 7 :2 d 12 1e 5f 62 :2 1e 6f
72 :2 1e 76 79 :2 1e 83 86 8f
97 :2 a0 :2 8f ab :2 86 :2 1e b3 b6
:2 1e be c1 :2 1e c6 c9 :2 1e d4
d7 :2 1e :2 7 54 :3 4 :2 f 21 :2 2a
34 40 :2 4 1 9 12 9 a
:2 14 26 :2 30 3c 40 52 54 55
:2 52 :2 26 :3 a :2 10 15 21 62 65
:2 21 :3 a :2 10 15 39 5c 5f :2 68
:2 39 76 79 :2 39 81 84 :2 8d :2 39
:2 a 58 :3 7 c 15 1e 2c a
13 1e f e 7 e 7 e
7 29 :2 4 :2 9 7 :2 10 1c :2 7
:2 11 23 :2 2d 39 3d 4f 51 52
:2 4f :2 23 :3 7 :2 d 12 1e 5f 62
:2 1e :3 7 :2 d 12 36 42 45 :2 36
4d 50 :2 36 5c 5f 66 6f 72
:2 5f :2 36 :2 7 55 :2 4 7 c 1b
a 13 1e f e 7 e 7
e 7 :2 4 :4 10 :2 4 :8 1 
1ec
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 :4 7 2
:4 8 :2 1 :7 a :6 b
:6 c :6 d :7 e :7 f
:7 11 :b 13 :11 14 15
:2 14 :2 15 :2 14 :2 15
:2 14 :2 16 :2 14 :9 16
:2 14 :2 17 :2 14 :9 17
:2 14 :2 18 :2 14 :9 18
:2 14 :2 19 :2 14 :2 19
:2 14 :2 19 :2 14 :2 19
:2 14 :2 19 :4 14 :3 13
:11 1c :b 1d :3 1c :3 20
:3 22 :3 23 :3 24 :3 25
:3 26 :9 28 29 :3 28
:11 2b :30 2c :3 2b :a 2e
12 :3 31 :11 32 :b 33
:17 34 :3 32 :2 36 :3 37
:3 38 39 37 36
37 :3 3a :3 31 :2 3b
:5 3d :11 3e :b 3f :18 40
:3 3e :2 42 43 :3 44
45 43 42 43
:3 46 :2 3c :6 3b 30
:7 1 
6fc
4
:3 0 1 :3 0 2
:a 0 1e7 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 6
:3 0 5 :7 0 9
8 :3 0 6 :3 0
7 :7 0 d c
:3 0 d :2 0 b
6 :3 0 8 :7 0
11 10 :3 0 4
:3 0 9 :7 0 15
14 :3 0 a :3 0
b :3 0 c :2 0
4 19 1a 0
17 1b 0 1e7
2 1c :2 0 10
:2 0 15 4 :3 0
e :2 0 13 1f
21 :6 0 24 22
0 1e5 0 d
:6 0 10 :2 0 17
6 :3 0 26 :7 0
2a 27 28 1e5
0 f :6 0 10
:2 0 19 6 :3 0
2c :7 0 30 2d
2e 1e5 0 11
:6 0 e :2 0 1b
6 :3 0 32 :7 0
36 33 34 1e5
0 12 :6 0 e
:2 0 1f 4 :3 0
1d 38 3a :6 0
3d 3b 0 1e5
0 13 :6 0 46
47 0 23 4
:3 0 21 3f 41
:6 0 44 42 0
1e5 0 14 :6 0
4c 4d 0 25
b :3 0 c :2 0
4 48 :7 0 4b
49 0 1e5 0
15 :6 0 16 :3 0
17 :3 0 16 :3 0
18 :3 0 4f 50
0 2 :4 0 27
51 53 4e 55
54 :2 0 19 :3 0
1a :3 0 57 58
0 1b :4 0 16
:3 0 1c :3 0 5b
5c 0 2 :4 0
29 5d 5f 1d
:4 0 1e :2 0 1f
:3 0 2b 62 64
:3 0 20 :4 0 1e
:2 0 21 :4 0 2e
67 69 :3 0 1e
:2 0 3 :3 0 31
6b 6d :3 0 1e
:2 0 21 :4 0 34
6f 71 :3 0 1e
:2 0 22 :4 0 37
73 75 :3 0 1e
:2 0 23 :3 0 24
:3 0 5 :3 0 3a
79 7b 25 :4 0
3c 78 7e 3f
77 80 :3 0 1e
:2 0 22 :4 0 42
82 84 :3 0 1e
:2 0 23 :3 0 24
:3 0 7 :3 0 45
88 8a 25 :4 0
47 87 8d 4a
86 8f :3 0 1e
:2 0 22 :4 0 4d
91 93 :3 0 1e
:2 0 23 :3 0 24
:3 0 8 :3 0 50
97 99 25 :4 0
52 96 9c 55
95 9e :3 0 1e
:2 0 22 :4 0 58
a0 a2 :3 0 1e
:2 0 21 :4 0 5b
a4 a6 :3 0 1e
:2 0 9 :3 0 5e
a8 aa :3 0 1e
:2 0 21 :4 0 61
ac ae :3 0 1e
:2 0 26 :4 0 64
b0 b2 :3 0 67
59 b4 :2 0 b6
6c b7 56 b6
0 b8 6e 0
145 16 :3 0 17
:3 0 b9 ba 0
16 :3 0 27 :3 0
bc bd 0 28
:2 0 2 :4 0 29
:2 0 2a :2 0 28
:2 0 70 c2 c4
:3 0 73 be c6
bb c8 c7 :2 0
19 :3 0 2b :3 0
ca cb 0 1b
:4 0 2c :4 0 1e
:2 0 1f :3 0 77
cf d1 :3 0 7a
cc d3 :2 0 d5
7d d6 c9 d5
0 d7 7f 0
145 14 :4 0 d8
d9 0 145 d
:3 0 3 :3 0 db
dc 0 145 f
:3 0 5 :3 0 de
df 0 145 11
:3 0 7 :3 0 e1
e2 0 145 12
:3 0 8 :3 0 e4
e5 0 145 13
:3 0 9 :3 0 e7
e8 0 145 b
:3 0 2d :3 0 ea
eb 0 2e :3 0
d :3 0 f :3 0
11 :3 0 12 :3 0
13 :3 0 14 :3 0
81 ed f4 ec
f5 0 145 16
:3 0 17 :3 0 f7
f8 0 16 :3 0
27 :3 0 fa fb
0 2f :2 0 2
:4 0 30 :2 0 2a
:2 0 28 :2 0 88
100 102 :3 0 8b
fc 104 f9 106
105 :2 0 19 :3 0
2b :3 0 108 109
0 1b :4 0 31
:4 0 1e :2 0 1f
:3 0 8f 10d 10f
:3 0 1e :2 0 32
:4 0 92 111 113
:3 0 1e :2 0 33
:4 0 95 115 117
:3 0 1e :2 0 23
:3 0 24 :3 0 b
:3 0 2d :3 0 11c
11d 0 98 11b
11f 25 :4 0 9a
11a 122 9d 119
124 :3 0 1e :2 0
34 :4 0 a0 126
128 :3 0 1e :2 0
21 :4 0 a3 12a
12c :3 0 1e :2 0
14 :3 0 a6 12e
130 :3 0 1e :2 0
21 :4 0 a9 132
134 :3 0 ac 10a
136 :2 0 138 af
139 107 138 0
13a b1 0 145
35 :3 0 36 :3 0
13b 13c 0 b
:3 0 2d :3 0 13e
13f 0 14 :3 0
37 :3 0 b3 13d
143 :2 0 145 b7
1e6 b :3 0 38
:2 0 4 146 147
0 16 :3 0 17
:3 0 149 14a 0
16 :3 0 27 :3 0
14c 14d 0 39
:2 0 2 :4 0 3a
:2 0 2a :2 0 28
:2 0 c3 152 154
:3 0 c6 14e 156
14b 158 157 :2 0
19 :3 0 2b :3 0
15a 15b 0 1b
:4 0 3b :4 0 1e
:2 0 1f :3 0 ca
15f 161 :3 0 cd
15c 163 :2 0 17c
19 :3 0 2b :3 0
165 166 0 1b
:4 0 3c :4 0 1e
:2 0 b :3 0 3d
:3 0 16b 16c 0
d0 16a 16e :3 0
1e :2 0 34 :4 0
d3 170 172 :3 0
1e :2 0 b :3 0
3e :3 0 175 176
0 d6 174 178
:3 0 d9 167 17a
:2 0 17c dc 17d
159 17c 0 17e
df 0 18e 3f
:3 0 15 :3 0 b
:3 0 3d :3 0 40
:3 0 b :3 0 3e
:3 0 41 :3 0 42
:4 0 43 1 :a 0
18a :2 0 18e 180
188 :2 0 a :3 0
15 :3 0 18c :2 0
18e e1 190 e5
18f 18e :2 0 1e4
44 :3 0 b :3 0
2d :3 0 193 194
0 45 :3 0 195
196 0 1dc 16
:3 0 17 :3 0 198
199 0 16 :3 0
27 :3 0 19b 19c
0 39 :2 0 2
:4 0 46 :2 0 2a
:2 0 28 :2 0 e7
1a1 1a3 :3 0 ea
19d 1a5 19a 1a7
1a6 :2 0 19 :3 0
2b :3 0 1a9 1aa
0 1b :4 0 47
:4 0 1e :2 0 1f
:3 0 ee 1ae 1b0
:3 0 f1 1ab 1b2
:2 0 1cc 19 :3 0
2b :3 0 1b4 1b5
0 1b :4 0 48
:4 0 1e :2 0 45
:3 0 f4 1b9 1bb
:3 0 1e :2 0 49
:4 0 f7 1bd 1bf
:3 0 1e :2 0 4a
:3 0 4b :3 0 28
:2 0 4c :2 0 fa
1c2 1c6 fe 1c1
1c8 :3 0 101 1b6
1ca :2 0 1cc 104
1cd 1a8 1cc 0
1ce 107 0 1dc
3f :3 0 15 :3 0
40 :3 0 b :3 0
3e :3 0 41 :3 0
42 :4 0 4d 1
:a 0 1d8 :2 0 1dc
1d0 1d6 :2 0 a
:3 0 15 :3 0 1da
:2 0 1dc 109 1df
:3 0 1df 0 1df
1de 1dc 1dd :6 0
1e1 1 :3 0 10e
1e3 110 1e2 1e1
:2 0 1e4 112 :2 0
1e6 115 1e6 1e5
145 1e4 :6 0 1e7
:2 0 2 1c 1e6
1ea :3 0 1e9 1e7
1eb :8 0 
11d
4
:3 0 1 3 1
7 1 b 1
f 1 13 5
6 a e 12
16 1 20 1
1e 1 25 1
2b 1 31 1
39 1 37 1
40 1 3e 1
45 1 52 1
5e 2 61 63
2 66 68 2
6a 6c 2 6e
70 2 72 74
1 7a 2 7c
7d 2 76 7f
2 81 83 1
89 2 8b 8c
2 85 8e 2
90 92 1 98
2 9a 9b 2
94 9d 2 9f
a1 2 a3 a5
2 a7 a9 2
ab ad 2 af
b1 4 5a 60
65 b3 1 b5
1 b7 2 c1
c3 3 bf c0
c5 2 ce d0
2 cd d2 1
d4 1 d6 6
ee ef f0 f1
f2 f3 2 ff
101 3 fd fe
103 2 10c 10e
2 110 112 2
114 116 1 11e
2 120 121 2
118 123 2 125
127 2 129 12b
2 12d 12f 2
131 133 2 10b
135 1 137 1
139 3 140 141
142 b b8 d7
da dd e0 e3
e6 e9 f6 13a
144 2 151 153
3 14f 150 155
2 15e 160 2
15d 162 2 169
16d 2 16f 171
2 173 177 2
168 179 2 164
17b 1 17d 3
17e 189 18d 1
148 2 1a0 1a2
3 19e 19f 1a4
2 1ad 1af 2
1ac 1b1 2 1b8
1ba 2 1bc 1be
3 1c3 1c4 1c5
2 1c0 1c7 2
1b7 1c9 2 1b3
1cb 1 1cd 4
197 1ce 1d7 1db
1 1df 1 192
2 190 1e3 7
23 29 2f 35
3c 43 4a 
1
4
0 
1ea
0
1
14
2
d
0 1 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 
3e 1 0
2 0 1
13 1 0
1e 1 0
25 1 0
2b 1 0
3 1 0
31 1 0
f 1 0
7 1 0
b 1 0
37 1 0
45 1 0
0

/
SHOW ERRORS FUNCTION SP_U_WP_ACTUAL;
ALTER FUNCTION SP_U_WP_ACTUAL COMPILE DEBUG;
/******************************************************************************************************************
update resource assignments SECURITY
assign_id:      RES_ASSIGNMENT_ID or TSK_ASSIGNMENT_ID
security_id:    SECURITY_ID in TMT_GLOBAL_SECURITY
*******************************************************************************************************************/
Prompt == FUNCTION SP_U_PARTICIPANT ==
CREATE OR REPLACE FUNCTION SP_U_PARTICIPANT wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
218
2 :e:
1FUNCTION:
1SP_U_PARTICIPANT:
1ASSIGN_ID:
1VARCHAR2:
1SECURITY_ID001:
1P_PLANNED_START:
1TIMESTAMP:
1P_PLANNED_FINISH:
1P_DELAY:
1PLS_INTEGER:
1P_UNITS:
1P_WORK_PERCENT:
1P_ACTUAL:
1RES_ID001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_SECURITY_ID:
132:
1A_ELEMENT_ID:
1A_PARTICIPANT_ID:
1255:
1A_PARTICIPANT_NAME:
1A_SECURITY_NAME:
1H_FULL_NAME:
1A_RES_ASSIGNMENT_ID:
1A_SECURITY_ID:
1A_TEMP_ID:
1H_PRJ_ASSIGN_ID:
1H_CREATED_BY:
1H_PROJECT_ID:
1H_CRI_SECURITY_ID:
1H_ACTUAL_START:
1H_START_DATE:
1H_FINISH_DATE:
1H_SCHED_START:
1H_SCHED_FINISH:
1H_START_DT:
1H_SD:
1DATE:
1H_FD:
1H_WEEK_OF:
1H_BIT_FLAG:
11:
1H_RES_TYPE:
1H_BILLABLE:
10:
1H_CAPITAL_EXPENSE:
1H_TPH_DELETE:
1H_DAYOFYEAR:
1H_LAST_SUBMITTED:
1A_ASSIGNMENT_COUNT:
1A_ELEMENT_TYPE:
1H_RESPONSIBLE:
1A_PARTICIPANT:
1A_RESOURCE_TYPE:
1A_SECURITY_VALUE:
1H_PARTICIPANT:
1H_UPDATE:
1H_TABLE_TYPE:
1H_TRANSFED_CRI:
1H_TASK_ID:
1H_RES_ID:
1H_REC_USER:
1H_EFFORT:
1H_CONTOUR:
1H_DELAY:
1H_PERCENT_UNITS:
1H_WORK_PERCENT:
1H_WORK_TODATE:
1H_NEW_ACTUAL:
1H_WORK:
1H_LAST_DAY:
1H_MINUTES:
1H_TASK_TYPE:
1H_PER_DAY:
1H_WORK_TYPE:
1H_ASSIGN_COUNT:
1H_DURATION:
1H_BITFLAG:
1H_CONSIDER:
1H_EUNIT:
1H_UNIT:
1H_SCHED_EFFORT:
1H_ETC:
1H_ROLLUP_BITS:
1H_CLOSE_TASK:
1H_DAY_MIN:
1H_ASSIGN_STRING:
18192:
1H_TEMP_RES_TYPE:
1H_TEMP_PARTICIPANT:
1H_TEMP_SECURITY_ID:
1H_TEMP_BITFLAG:
1H_TEMP_SC_STATE:
1H_TEMP_VAR:
1H_REG_COST:
1NUMBER:
1H_REG_SALE:
1H_SPEC_COST:
1H_SPEC_SALE:
1H_B_REG_COST:
1H_B_REG_SALE:
1H_B_SPEC_COST:
1H_B_SPEC_SALE:
1STMT:
17999:
1I_RESCHED:
1I_BITFLAG:
1C_P_DELAY:
1C_R_DELAY:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_U_PARTICIPANT(:
1':
1, :
1COALESCE:
1NULLIF:
1'':
1NULL:
1TO_CHAR:
1);:
1CHECK_TRACE:
1120:
1+:
1FPUT:
1               ENTER:: SP_U_PARTICIPANT(121) :: cplan.sql(16118) :: :
1:
1RA:
1RES_ASSIGNMENT_ID:
1PARTICIPANT:
1ELEMENT_ID:
1ELEMENT_TYPE:
1RESOURCE_TYPE:
1CRI_SECURITY_ID:
1RESOURCE_ID:
1PROJECT_ID:
1TABLE_TYPE:
1RESOURCE_ASSIGNMENTS:
1SELECT ra.RES_ASSIGNMENT_ID, ra.PARTICIPANT, ra.ELEMENT_ID, ra.ELEMENT_TYPE,+
1:n      CASE WHEN ra.RESOURCE_TYPE = '0' THEN 0 ELSE 1 END,:n      CASE WHEN +
1ra.CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END,:n      ra.RESOURCE_ID,:n      r+
1a.PROJECT_ID, ra.TABLE_TYPE:n      INTO a_res_assignment_id, a_participant, a+
1_element_id, a_element_type,:n      a_resource_type,:n      h_transfed_cri,:n+
1      a_participant_id,:n      h_project_id, h_table_type:n      FROM RESOURC+
1E_ASSIGNMENTS ra:n      WHERE ra.RES_ASSIGNMENT_ID = a_temp_id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
1154:
1           EXCEPTION:: SP_U_PARTICIPANT(155) :: cplan.sql(16152) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1=:
1SC_SQL_NO_DATA:
1PROJECT_ASSIGNMENT_ID:
1BIT_FLAG:
1LAST_SUBMITTED:
1C_CONTOUR:
1C_EFFORT:
1T_UNIT:
1C_UNIT:
1CONTOUR_TYPE:
1ASSIGN_UNITS:
1UNITS_MSP:
1ACTUAL_START:
1WORK_TODATE:
1UNRESOLVED_START:
1UNRESOLVED_FINISH:
1PLANNED_WORK:
1SCHED_WORK:
1ASSIGNMENT_DELAY:
1LAST_SCHED_MIN:
1TO_DATE:
1BILLABLE:
1CAPITAL_EXPENSE:
1REGULAR_COST:
1SPECIAL_COST:
1REGULAR_RATE:
1SPECIAL_RATE:
1B_REGULAR_COST:
1B_SPECIAL_COST:
1B_REGULAR_RATE:
1B_SPECIAL_RATE:
1MILESTONE_PERCENT:
1TASK_ID:
1ROLLUP_BITS:
1TASK_ASSIGNMENTS:
1TSK_ASSIGNMENT_ID:
1SELECT RES_ASSIGNMENT_ID, PROJECT_ASSIGNMENT_ID, PROJECT_ID, RESOURCE_ID,:n  +
1       RESOURCE_TYPE, BIT_FLAG, LAST_SUBMITTED, C_CONTOUR, C_EFFORT, T_UNIT, +
1C_UNIT, CONTOUR_TYPE, :n         CASE WHEN ASSIGN_UNITS > UNITS_MSP THEN ASSI+
1GN_UNITS ELSE UNITS_MSP END, ACTUAL_START, WORK_TODATE, UNRESOLVED_START, UNR+
1ESOLVED_FINISH,:n         CASE WHEN BIT_FLAG = '0' THEN PLANNED_WORK ELSE SCH+
1ED_WORK END, ASSIGNMENT_DELAY,LAST_SCHED_MIN,:n         TO_DATE(SubStr(UNRESO+
1LVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAP+
1ITAL_EXPENSE,:n         REGULAR_COST, SPECIAL_COST, REGULAR_RATE, SPECIAL_RAT+
1E,:n         B_REGULAR_COST, B_SPECIAL_COST, B_REGULAR_RATE, B_SPECIAL_RATE,+
1:n         MILESTONE_PERCENT, TASK_ID, ROLLUP_BITS:n         INTO a_res_assig+
1nment_id, h_prj_assign_id, h_project_id, h_res_id,:n         h_res_type, h_bi+
1t_flag, h_last_submitted, h_contour, h_work, h_eunit, h_unit, h_bitflag,:n   +
1      h_percent_units, h_actual_start, h_work_todate, h_start_date, h_finish_+
1date,:n         h_effort, h_delay, h_last_day,:n         h_SD, h_FD, h_billab+
1le, h_capital_expense,:n         h_reg_cost, h_spec_cost, h_reg_sale, h_spec_+
1sale,:n         h_b_reg_cost, h_b_spec_cost, h_b_reg_sale, h_b_spec_sale,:n  +
1       h_work_percent, h_task_id, h_rollup_bits:n         FROM TASK_ASSIGNMEN+
1TS:n         WHERE TSK_ASSIGNMENT_ID = a_temp_id:
1188:
1           EXCEPTION:: SP_U_PARTICIPANT(189) :: cplan.sql(16186) :: :
1TASK_TYPE:
1MINUTE_PER_DAY:
1START_DT:
1FINISH_DT:
1UNRESOLVED_DURATION:
1ASSIGN_COUNT:
1SD:
1PLANNED_EFFORT:
1SCHED_EFFORT:
1WORK_PERCENT:
1ETC:
1FORECAST_START:
1FORECAST_FINISH:
1TMT_WBS:
1SELECT PROJECT_ID, TASK_TYPE, MINUTE_PER_DAY,/*UNRESOLVED_START, UNRESOLVED_F+
1INISH*/ START_DT, FINISH_DT, UNRESOLVED_DURATION,:n            ASSIGN_COUNT, +
1SD, WORK_TODATE, PLANNED_EFFORT + SCHED_EFFORT, WORK_PERCENT, ETC, COALESCE(F+
1ORECAST_START,FORECAST_FINISH):n            INTO h_project_id, h_task_type, h+
1_per_day, h_start_date, h_finish_date, h_duration,:n            h_assign_coun+
1t, h_work_type, h_work_todate, h_effort, h_work_percent, h_etc, h_start_DT:n +
1           FROM TMT_WBS WHERE ELEMENT_ID = a_temp_id:
1210:
1           EXCEPTION:: SP_U_PARTICIPANT(211) :: cplan.sql(16208) :: :
1218:
1        RETURN_ERROR:: SP_U_PARTICIPANT(219) :: cplan.sql(16216) :: :
1STATE:: :
1PMO_CNST:
1RECORD_NOT_FOUND:
1 ID:: :
1221:
1                EXIT:: SP_U_PARTICIPANT(222) :: cplan.sql(16219) :: :
1PMODEFINES:
1PMO_RETURN_ERROR_SP:
1TRUE:
1PMO_ORASTD:
1STRNCMP:
110:
1>:
1!=:
1231:
1        RETURN_ERROR:: SP_U_PARTICIPANT(232) :: cplan.sql(16229) :: :
1F_DATE_EARLIER_THAN_S_DATE:
1234:
1                EXIT:: SP_U_PARTICIPANT(235) :: cplan.sql(16232) :: :
1IS NOT NULL:
1-:
1246:
1        RETURN_ERROR:: SP_U_PARTICIPANT(247) :: cplan.sql(16244) :: :
1CANNOT_UPDATE_ASSIGN_ACCEPT_RE:
1249:
1                EXIT:: SP_U_PARTICIPANT(250) :: cplan.sql(16247) :: :
1DB2TIMESTAMP:
1DAY_YEAR:
1TIME:
1PMO_STPU:
1FIRST_AVAIL_TIME:
1BIT_SCHED:
1CALENDAR_DAY_OF_YEAR:
1SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL+
1_TIME(BIT_SCHED))) INTO h_sched_start:n                     FROM CALENDAR_DAY+
1_OF_YEAR:n                     WHERE ELEMENT_ID = h_project_id:n             +
1        AND DAY_YEAR = TO_DATE(SubStr(h_sched_start, 1, 10)):
1272:
1           EXCEPTION:: SP_U_PARTICIPANT(273) :: cplan.sql(16270) :: :
1DUAL:
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_sched_start, 1, 10)), PMO_ORA+
1STD.TIME('08::00')) INTO h_sched_start:n                        FROM DUAL:
1289:
1           EXCEPTION:: SP_U_PARTICIPANT(290) :: cplan.sql(16287) :: :
1LAST_AVAIL_TIME:
1SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_+
1TIME(BIT_SCHED))) INTO h_sched_finish:n                     FROM CALENDAR_DAY+
1_OF_YEAR:n                     WHERE ELEMENT_ID = h_project_id:n             +
1        AND DAY_YEAR = TO_DATE(SubStr(h_sched_finish, 1, 10)):
1312:
1           EXCEPTION:: SP_U_PARTICIPANT(313) :: cplan.sql(16310) :: :
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_sched_finish, 1, 10)), PMO_OR+
1ASTD.TIME('17::00')) INTO h_sched_finish:n                        FROM DUAL:
1329:
1           EXCEPTION:: SP_U_PARTICIPANT(330) :: cplan.sql(16327) :: :
1SC_SS:
1SCHED_START:
1SCHED_FINISH:
1SCHED_DURATION:
1UPDATE TMT_WBS SET SCHED_START = h_sched_start, SCHED_FINISH = h_sched_finish+
1, SCHED_DURATION = h_duration:n                     WHERE ELEMENT_ID = a_temp+
1_id:
1FOUND:
1100:
1354:
1           EXCEPTION:: SP_U_PARTICIPANT(355) :: cplan.sql(16352) :: :
1PLANNED_START:
1PLANNED_FINISH:
1PLANNED_DURATION:
1UPDATE TMT_WBS SET PLANNED_START = h_sched_start, PLANNED_FINISH = h_sched_fi+
1nish, PLANNED_DURATION = h_duration:n                     WHERE ELEMENT_ID = +
1a_temp_id:
1374:
1           EXCEPTION:: SP_U_PARTICIPANT(375) :: cplan.sql(16372) :: :
1SC_SQL_SUCCESS:
1FC_ROLLUP:
1COUNT:
1SELECT COUNT('X') INTO h_temp_var FROM DUAL WHERE EXISTS (:n                 +
1 SELECT 'X' FROM TASK_ASSIGNMENTS:n                  WHERE PROJECT_ID = h_pro+
1ject_id AND TASK_ID = a_temp_id:n                  AND ACTUAL_START IS NOT Nu+
1LL):
1397:
1           EXCEPTION:: SP_U_PARTICIPANT(398) :: cplan.sql(16395) :: :
1405:
1        RETURN_ERROR:: SP_U_PARTICIPANT(406) :: cplan.sql(16403) :: :
1CANNOT_UPDATE_HAS_ACTUAL:
1408:
1                EXIT:: SP_U_PARTICIPANT(409) :: cplan.sql(16406) :: :
1428:
1           EXCEPTION:: SP_U_PARTICIPANT(429) :: cplan.sql(16426) :: :
1445:
1           EXCEPTION:: SP_U_PARTICIPANT(446) :: cplan.sql(16443) :: :
1464:
1           EXCEPTION:: SP_U_PARTICIPANT(465) :: cplan.sql(16462) :: :
1481:
1           EXCEPTION:: SP_U_PARTICIPANT(482) :: cplan.sql(16479) :: :
1FC_CHANGE_TA_DATES:
1496:
1        RETURN_ERROR:: SP_U_PARTICIPANT(497) :: cplan.sql(16494) :: :
1499:
1                EXIT:: SP_U_PARTICIPANT(500) :: cplan.sql(16497) :: :
1ELSIF:
18:
1508:
1      RETURN_INVALID:: SP_U_PARTICIPANT(509) :: cplan.sql(16506) :: :
1SC_TASK_ACTUAL_GRATER_THAN_SCH:
1511:
1                EXIT:: SP_U_PARTICIPANT(512) :: cplan.sql(16509) :: :
1PMO_RETURN_INVALID_SP:
1A:
1TA:
1ACTIVITIES:
1TYPE_ID:
1SC_TIMESHEET_ACTIVITY:
1SELECT COUNT('X') INTO h_temp_var:n               FROM DUAL:n               W+
1HERE EXISTS(SELECT A.ELEMENT_ID FROM TASK_ASSIGNMENTS TA, ACTIVITIES A:n     +
1          WHERE TA.TASK_ID = a_temp_id:n               AND TA.TSK_ASSIGNMENT_+
1ID = A.TSK_ASSIGNMENT_ID:n               AND A.TSK_ASSIGNMENT_ID <> A.ELEMENT+
1_ID:n               AND A.TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY):
1536:
1           EXCEPTION:: SP_U_PARTICIPANT(537) :: cplan.sql(16534) :: :
1543:
1        RETURN_ERROR:: SP_U_PARTICIPANT(544) :: cplan.sql(16541) :: :
1SC_TIMESHEET_STEP_EXISTS:
1546:
1                EXIT:: SP_U_PARTICIPANT(547) :: cplan.sql(16544) :: :
1JOIN:
1TIMESHEET_APPROVAL_STATUS:
1APV:
1WEEK_OF:
1SELECT COUNT('X') INTO h_temp_var FROM DUAL WHERE EXISTS (:n               SE+
1LECT 'X' FROM TASK_ASSIGNMENTS ta JOIN TIMESHEET_APPROVAL_STATUS apv:n       +
1        ON apv.ELEMENT_ID = ta.PROJECT_ID AND apv.RESOURCE_ID = ta.RESOURCE_I+
1D:n               WHERE ta.PROJECT_ID = h_project_id AND ta.TASK_ID = a_temp_+
1id AND:n               (TO_DATE(SubStr(ta.ACTUAL_START, 1, 10)) - 7     ) <= +
1apv.WEEK_OF):
1567:
1           EXCEPTION:: SP_U_PARTICIPANT(568) :: cplan.sql(16565) :: :
1575:
1        RETURN_ERROR:: SP_U_PARTICIPANT(576) :: cplan.sql(16573) :: :
1TS_APPROVAL_ERROR:
1578:
1                EXIT:: SP_U_PARTICIPANT(579) :: cplan.sql(16576) :: :
1FC_U_TASK_PERCENT:
1599:
1        RETURN_ERROR:: SP_U_PARTICIPANT(600) :: cplan.sql(16597) :: :
1602:
1                EXIT:: SP_U_PARTICIPANT(603) :: cplan.sql(16600) :: :
1612:
1        RETURN_ERROR:: SP_U_PARTICIPANT(613) :: cplan.sql(16610) :: :
1615:
1                EXIT:: SP_U_PARTICIPANT(616) :: cplan.sql(16613) :: :
1SELECT PROJECT_ID, TASK_TYPE, MINUTE_PER_DAY:n            INTO h_project_id, +
1h_task_type, h_per_day:n            FROM TMT_WBS WHERE PROJECT_ID = h_project+
1_id AND ELEMENT_ID = h_task_id:
1633:
1           EXCEPTION:: SP_U_PARTICIPANT(634) :: cplan.sql(16631) :: :
1SELECT TASK_TYPE, MINUTE_PER_DAY INTO h_task_type, h_per_day:n               +
1   FROM TMT_WBS:n                  WHERE PROJECT_ID = h_project_id AND ELEMEN+
1T_ID = h_task_id:
1669:
1           EXCEPTION:: SP_U_PARTICIPANT(670) :: cplan.sql(16667) :: :
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_sched_finish, 1, 10)), PMO_OR+
1ASTD.TIME(h_finish_date)):n                  INTO h_sched_finish:n           +
1       FROM DUAL:
1770:
1           EXCEPTION:: SP_U_PARTICIPANT(771) :: cplan.sql(16768) :: :
1STRCMP:
1FC_TPHASE_SCHED_ASSIGN:
1TASK_DAY_OF_YEAR:
1SCHED_MIN:
1ACTUAL_MIN:
1UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN:n                     WHER+
1E PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = a_temp_id AND DAY_YEAR = +
1TO_DATE(SubStr(h_last_submitted, 1, 10)):
1797:
1           EXCEPTION:: SP_U_PARTICIPANT(798) :: cplan.sql(16795) :: :
12:
1FC_FD_CTD_TDY:
1SELECT TO_DATE(SubStr(h_start_date, 1, 10)), TO_DATE(SubStr(h_sched_finish, 1+
1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE:n              +
1    INTO h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale:n +
1                 FROM TASK_ASSIGNMENTS:n                  WHERE TSK_ASSIGNMEN+
1T_ID = a_temp_id:
1826:
1           EXCEPTION:: SP_U_PARTICIPANT(827) :: cplan.sql(16824) :: :
14:
1839:
1       RETURN_STATUS:: SP_U_PARTICIPANT(840) :: cplan.sql(16837) :: :
1PMO_RETURN_STATUS:
1<:
1SELECT START_DT INTO h_start_date:n                  FROM TMT_WBS:n          +
1        WHERE ELEMENT_ID = h_task_id:
1861:
1           EXCEPTION:: SP_U_PARTICIPANT(862) :: cplan.sql(16859) :: :
1SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR :
1WHERE ELEMENT_ID = ':
1' AND DAY_YEAR < TO_DATE(SubStr(':
1', 1, 10)) :
1ORDER BY DAY_YEAR DESC) WHERE ROWNUM<=:
1OPEN:
1LOOP:
1EXIT:
1CLOSE:
1919:
1           EXCEPTION:: SP_U_PARTICIPANT(920) :: cplan.sql(16917) :: :
1934:
1           EXCEPTION:: SP_U_PARTICIPANT(935) :: cplan.sql(16932) :: :
1IS NULL:
1944:
1        RETURN_ERROR:: SP_U_PARTICIPANT(945) :: cplan.sql(16942) :: :
1947:
1                EXIT:: SP_U_PARTICIPANT(948) :: cplan.sql(16945) :: :
1952:
1        RETURN_ERROR:: SP_U_PARTICIPANT(953) :: cplan.sql(16950) :: :
1955:
1                EXIT:: SP_U_PARTICIPANT(956) :: cplan.sql(16953) :: :
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORAST+
1D.TIME(h_start_date)):n                  INTO h_start_date:n                 +
1 FROM DUAL:
1973:
1           EXCEPTION:: SP_U_PARTICIPANT(974) :: cplan.sql(16971) :: :
1984:
1        RETURN_ERROR:: SP_U_PARTICIPANT(985) :: cplan.sql(16982) :: :
1987:
1                EXIT:: SP_U_PARTICIPANT(988) :: cplan.sql(16985) :: :
1103079040:
1997:
1      RETURN_INVALID:: SP_U_PARTICIPANT(998) :: cplan.sql(16995) :: :
1INVALID_PARAMS:
11000:
1                EXIT:: SP_U_PARTICIPANT(1001) :: cplan.sql(16998) :: :
1SELECT UNITS_MSP INTO h_temp_var:n                  FROM TASK_ASSIGNMENTS:n  +
1                WHERE TSK_ASSIGNMENT_ID = h_prj_assign_id:
11034:
1           EXCEPTION:: SP_U_PARTICIPANT(1035) :: cplan.sql(17032) :: :
11042:
1        RETURN_ERROR:: SP_U_PARTICIPANT(1043) :: cplan.sql(17040) :: :
1TASK_ASSIGNMENT_ERROR:
11045:
1                EXIT:: SP_U_PARTICIPANT(1046) :: cplan.sql(17043) :: :
190:
1CONSIDER:
1BIT_OR:
1SC_SCHED_BIT:
1SC_PLANNED_BIT:
1UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, UNITS_MSP = h_percent_units,:n     +
1          C_EFFORT = 0, C_UNIT = 0, C_CONTOUR = 0, CONTOUR_TYPE = 1,:n       +
1        ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_+
1CNST.SC_PLANNED_BIT)):n               WHERE TSK_ASSIGNMENT_ID = a_temp_id:
11135:
1           EXCEPTION:: SP_U_PARTICIPANT(1136) :: cplan.sql(17133) :: :
1FC_U_TDY_LAST_ACTUAL:
1FC_DEL_NOACTUAL:
1PMO_FUNC:
1SC_BIT_30:
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_+
1ORASTD.TIME(h_start_date)):n                     INTO h_start_date:n         +
1            FROM DUAL:
11185:
1           EXCEPTION:: SP_U_PARTICIPANT(1186) :: cplan.sql(17183) :: :
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_+
1ORASTD.TIME(h_start_date)) + 1     :n                     INTO h_start_date:n+
1                     FROM DUAL:
11202:
1           EXCEPTION:: SP_U_PARTICIPANT(1203) :: cplan.sql(17200) :: :
1FC_RESCHEDULE:
11220:
1        RETURN_ERROR:: SP_U_PARTICIPANT(1221) :: cplan.sql(17218) :: :
11223:
1                EXIT:: SP_U_PARTICIPANT(1224) :: cplan.sql(17221) :: :
1BAK_ASSIGN_STRING:
1UPDATE TASK_ASSIGNMENTS SET:n                       BAK_ASSIGN_STRING = NULL+
1:n                  WHERE TSK_ASSIGNMENT_ID = A_TEMP_ID:
1BAK_START_DT:
1BAK_FINISH_DT:
1ASSIGN_STRING:
1UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLU+
1P_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT)),:n                  B+
1AK_START_DT = COALESCE(SCHED_START,PLANNED_START), BAK_FINISH_DT = COALESCE(S+
1CHED_FINISH,PLANNED_FINISH),:n                  BAK_ASSIGN_STRING = ASSIGN_ST+
1RING:n                  WHERE TSK_ASSIGNMENT_ID = a_temp_id:
11248:
1           EXCEPTION:: SP_U_PARTICIPANT(1249) :: cplan.sql(17246) :: :
11256:
1        RETURN_ERROR:: SP_U_PARTICIPANT(1257) :: cplan.sql(17254) :: :
11259:
1                EXIT:: SP_U_PARTICIPANT(1260) :: cplan.sql(17257) :: :
1SELECT COUNT('X') INTO h_temp_var:n               FROM DUAL:n               W+
1HERE EXISTS(SELECT ELEMENT_ID FROM ACTIVITIES:n               WHERE TSK_ASSIG+
1NMENT_ID = a_temp_id:n               AND TSK_ASSIGNMENT_ID <> ELEMENT_ID:n   +
1            AND TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY):
11281:
1           EXCEPTION:: SP_U_PARTICIPANT(1282) :: cplan.sql(17279) :: :
11288:
1        RETURN_ERROR:: SP_U_PARTICIPANT(1289) :: cplan.sql(17286) :: :
11291:
1                EXIT:: SP_U_PARTICIPANT(1292) :: cplan.sql(17289) :: :
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_+
1ORASTD.TIME(h_start_date)):n                        INTO h_start_date:n      +
1                  FROM DUAL:
11331:
1           EXCEPTION:: SP_U_PARTICIPANT(1332) :: cplan.sql(17329) :: :
1SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_+
1ORASTD.TIME(h_start_date)) + 1     :n                        INTO h_start_dat+
1e:n                        FROM DUAL:
11348:
1           EXCEPTION:: SP_U_PARTICIPANT(1349) :: cplan.sql(17346) :: :
1SELECT SCHED_MIN - ACTUAL_MIN INTO h_minutes:n                        FROM TA+
1SK_DAY_OF_YEAR :n                        WHERE PROJECT_ID = h_project_id AND +
1TSK_ASSIGNMENT_ID = a_temp_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted,+
1 1, 10)):
11367:
1           EXCEPTION:: SP_U_PARTICIPANT(1368) :: cplan.sql(17365) :: :
110000:
1FORECAST_DURATION:
1UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLU+
1P_BITS,PMO_CNST.SC_SCHED_BIT),:n                  SCHED_WORK = h_new_actual, +
1WORK_TODATE = h_new_actual, ETC = 0, MILESTONE_PERCENT = 10000,:n            +
1      FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0:n +
1                 WHERE TSK_ASSIGNMENT_ID = a_temp_id:
11395:
1           EXCEPTION:: SP_U_PARTICIPANT(1396) :: cplan.sql(17393) :: :
11409:
1        RETURN_ERROR:: SP_U_PARTICIPANT(1410) :: cplan.sql(17407) :: :
11412:
1                EXIT:: SP_U_PARTICIPANT(1413) :: cplan.sql(17410) :: :
1TRUNC:
1TO_NUMBER:
1/:
1*:
110000.00:
10.4:
1FC_U_ASSIGN_PERCENT:
11435:
1        RETURN_ERROR:: SP_U_PARTICIPANT(1436) :: cplan.sql(17433) :: :
11438:
1                EXIT:: SP_U_PARTICIPANT(1439) :: cplan.sql(17436) :: :
11444:
1        RETURN_ERROR:: SP_U_PARTICIPANT(1445) :: cplan.sql(17442) :: :
11447:
1                EXIT:: SP_U_PARTICIPANT(1448) :: cplan.sql(17445) :: :
1SC_PERCENT_COMPLETE_BIT:
1UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = p_work_percent,:n            +
1   ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT+
1):n               WHERE TSK_ASSIGNMENT_ID = a_temp_id:
11501:
1           EXCEPTION:: SP_U_PARTICIPANT(1502) :: cplan.sql(17499) :: :
1UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERC+
1ENT_COMPLETE_BIT):n               WHERE PROJECT_ID = h_project_id AND ELEMENT+
1_ID = h_task_id:
11525:
1           EXCEPTION:: SP_U_PARTICIPANT(1526) :: cplan.sql(17523) :: :
11535:
1        RETURN_ERROR:: SP_U_PARTICIPANT(1536) :: cplan.sql(17533) :: :
11538:
1                EXIT:: SP_U_PARTICIPANT(1539) :: cplan.sql(17536) :: :
1SECURITY_ID:
1SELECT ra.SECURITY_ID, ra.PARTICIPANT, ra.ELEMENT_ID, ra.ELEMENT_TYPE,:n     +
1       CASE WHEN ra.RESOURCE_TYPE = '0' THEN 0 ELSE 1 END,:n            CASE +
1WHEN ra.CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END,:n            ra.RESOURCE_I+
1D,:n            ra.PROJECT_ID, ra.TABLE_TYPE:n            INTO h_security_id,+
1 a_participant, a_element_id, a_element_type,:n            a_resource_type,:n+
1            h_transfed_cri,:n            a_participant_id,:n            h_pro+
1ject_id, h_table_type:n            FROM RESOURCE_ASSIGNMENTS ra WHERE ra.RES_+
1ASSIGNMENT_ID = a_res_assignment_id:
11562:
1           EXCEPTION:: SP_U_PARTICIPANT(1563) :: cplan.sql(17560) :: :
1NOT:
11570:
1        RETURN_ERROR:: SP_U_PARTICIPANT(1571) :: cplan.sql(17568) :: :
11573:
1                EXIT:: SP_U_PARTICIPANT(1574) :: cplan.sql(17571) :: :
11582:
1        RETURN_ERROR:: SP_U_PARTICIPANT(1583) :: cplan.sql(17580) :: :
11585:
1                EXIT:: SP_U_PARTICIPANT(1586) :: cplan.sql(17583) :: :
1SC_TABLE_WBS:
1FC_U_ASSIGNED_SECURITY:
11600:
1       RETURN_STATUS:: SP_U_PARTICIPANT(1601) :: cplan.sql(17598) :: :
1RETURN_ERROR_EXCEPTION:
11607:
1           EXCEPTION:: SP_U_PARTICIPANT(1608) :: cplan.sql(17605) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1ERROR:
1ID:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
11619:
1           EXCEPTION:: SP_U_PARTICIPANT(1620) :: cplan.sql(17617) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
2628
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 1c 81 b0 a3 a0 1c 81
b0 a3 a0 1c 81 b0 a3 a0
1c 81 b0 a3 a0 1c 81 b0
a3 a0 1c 81 b0 a3 a0 1c
81 b0 a3 a0 1c 81 b0 a3
a0 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 81
b0 a3 a0 1c 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 1c
51 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 1c 81 b0
a3 a0 1c 81 b0 a3 :2 a0 6b
1c 81 b0 a3 :2 a0 6b 1c 81
b0 a3 :2 a0 6b 1c 81 b0 :2 a0
6b :2 a0 6b 6e a5 b a 10
:2 a0 6b 6e :2 a0 6b 6e a5 b
6e 7e a0 b4 2e 6e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :2 a0
6e 7e a0 b4 2e 7e 6e b4
2e 6e a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e :2 a0 6e
7e a0 b4 2e 7e 6e b4 2e
6e a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e :3 a0 a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
a0 51 d a0 6e d :2 a0 d
:2 a0 d :2 a0 d :20 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a :52 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :20 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c :2 a0
d :2 a0 d :2 a0 6b :2 a0 51 a5
b 7e 51 b4 2e 5a :2 a0 6b
:2 a0 51 a5 b 7e 51 b4 2e
5a a 10 :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b :2 a0
a5 57 b7 19 3c a0 7e b4
2e :2 5a :2 a0 51 a5 b 7e :2 a0
51 a5 b b4 2e 5a a0 7e
a0 51 a5 b b4 2e 5a 52
10 :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 19 3c :2 a0 6b :2 a0 51 a5
b 7e 51 b4 2e 5a :2 a0 6b
:2 a0 51 a5 b 7e 51 b4 2e
5a 52 10 :2 a0 51 a5 b 7e
51 b4 2e :2 a0 6b :2 a0 51 a5
b 7e 51 b4 2e :10 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
6b a0 7e a0 6b b4 2e 5a
:9 a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 19 3c :2 a0 6b :2 a0
51 a5 b 7e 51 b4 2e a0
4d d b7 19 3c b7 :10 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 6b a0 7e a0 6b b4 2e
5a :9 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 19 3c a0 51
d b7 :2 19 3c :2 a0 51 a5 b
a0 7e a0 6b b4 2e :9 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :9 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:2 19 3c :2 a0 6b a0 7e a0 6b
b4 2e 5a :3 a0 a5 b d b7
19 3c b7 :9 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 51 a5
b 7e 51 b4 2e :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c :2 a0
6b :2 a0 51 a5 b 7e 51 b4
2e :10 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a :9 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 19
3c b7 :10 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a :9 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
19 3c b7 :2 19 3c :c a0 51 a5
b 51 a5 b d :2 a0 51 a5
b 7e 51 b4 2e :3 a0 a5 b
d b7 :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 51 a5 b :2 a0
a5 57 b7 :2 19 3c b7 :2 19 3c
a0 b7 :2 a0 51 a5 b a0 7e
a0 51 a5 b b4 2e 5a :2 a0
51 a5 b 7e 51 b4 2e 5a
a 10 a0 51 d :2 a0 51 a5
b a0 7e a0 51 a5 b b4
2e :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b a0 a5 57
a0 b7 :2 a0 51 a5 b a0 7e
a0 51 a5 b b4 2e a0 51
d b7 :2 19 3c a0 51 d :18 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 51 a5 b 7e 51 b4
2e :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 19 3c :1c a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 51 a5
b 7e 51 b4 2e :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c :2 a0
d :8 a0 51 a5 b a5 b d
b7 :2 19 3c :2 a0 51 a5 b 7e
51 b4 2e :3 a0 a5 b d b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :3 a0 a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 51 a5 b :2 a0
a5 57 a0 b7 a0 7e b4 2e
5a a0 7e b4 2e 5a a 10
:2 a0 d :2 a0 d :2 a0 6b :2 a0 51
a5 b 7e 51 b4 2e 5a :2 a0
6b :2 a0 51 a5 b 7e 51 b4
2e 5a a 10 :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 19 3c :b a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 :2 51 a5 b 7e 6e b4 2e
a0 51 d b7 a0 51 d b7
:2 19 3c a0 51 d a0 51 d
:2 a0 6b :2 a0 51 a5 b 7e 51
b4 2e 5a :2 a0 6b :2 a0 51 a5
b 7e 51 b4 2e 5a 52 10
:2 a0 51 a5 b a0 7e a0 51
a5 b b4 2e 5a 52 10 :2 a0
6b :2 a0 51 a5 b 7e 51 b4
2e :9 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 d a0 51 d
a0 51 d a0 b7 :2 a0 6b :2 a0
51 a5 b 7e 51 b4 2e :a a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :3 a0 6b :2 a0 a5 b d a0
7e 51 b4 2e d :c a0 a5 b
d :2 a0 51 a5 b 7e 51 b4
2e 5a a0 7e b4 2e :2 5a a
10 :b a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 19 3c :2 a0 :2 51 a5 b
7e 6e b4 2e a0 51 d b7
a0 51 d b7 :2 19 3c :f a0 51
a0 51 a5 b d :13 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f a0
51 d :c a0 a5 b d :3 a0 a5
b d :2 a0 6b 51 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b :2 a0
a5 57 a0 b7 19 :2 a0 51 a5
b a0 7e a0 51 a5 b b4
2e :3 a0 51 a5 b 7e :2 a0 51
a5 b b4 2e d a0 51 d
a0 b7 19 :2 a0 51 a5 b 7e
51 b4 2e a0 51 d a0 51
d :5 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f a0 b7 19 :2 a0 51
a5 b a0 7e a0 51 a5 b
b4 2e :3 a0 51 a5 b 7e :2 a0
51 a5 b b4 2e d a0 4d
d a0 6e d :2 a0 51 a5 b
7e 51 b4 2e a0 6e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :3 a0
51 a5 b a5 b b4 2e d
:3 a0 11c 11d b7 a0 6e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :3 a0
51 a5 b a5 b b4 2e d
:3 a0 11c 11d b7 :2 19 3c :3 a0 51
a5 b 7e 51 b4 2e :2 a0 e9
d3 :2 a0 f :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:2 a0 e9 d3 :2 a0 f :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :2 19 3c :2 a0 6b a0 7e
a0 6b b4 2e 5a a0 2b b7
19 3c b7 a0 47 :2 a0 51 a5
b 7e 51 b4 2e :2 a0 e9 c1
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 a0 e9 c1 :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c a0
7e b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 :2 19 3c b7 19
3c :a a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f a0 51 d a0 51
d b7 :2 19 3c a0 b7 :2 a0 51
a5 b a0 7e a0 51 a5 b
b4 2e :2 a0 51 a5 b a0 7e
a0 51 a5 b b4 2e :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b :2 a0 a5 57 b7 19 3c
:2 a0 51 a5 b 7e 51 b4 2e
a0 51 d b7 :2 a0 51 a5 b
7e 51 b4 2e 5a :2 a0 51 a5
b 7e 51 b4 2e 5a 52 10
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b a0 a5 57 b7
19 3c :2 a0 d b7 :2 19 3c a0
7e b4 2e 5a :5 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a :2 a0
51 a5 b a0 7e a0 51 a5
b b4 2e 5a a 10 :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b :2 a0 a5 57 b7 19 3c
b7 19 3c a0 51 d a0 51
d :2 a0 51 a5 b 7e 51 b4
2e a0 51 d b7 19 3c :12 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
51 a5 b 7e 51 b4 2e a0
51 d b7 19 3c b7 :2 19 3c
a0 7e 51 b4 2e a0 7e 51
b4 2e d :c a0 a5 b d :2 a0
51 a5 b 7e 51 b4 2e 5a
a0 7e b4 2e :2 5a a 10 :9 a0
51 a5 b d b7 19 3c a0
7e b4 2e 5a :4 a0 51 a5 b
d b7 :2 a0 :2 51 a5 b 7e 6e
b4 2e :4 a0 51 a5 b d b7
:4 a0 51 a5 b d b7 :2 19 3c
b7 :2 19 3c a0 7e b4 2e 5a
:2 a0 51 a5 b 7e 51 b4 2e
:3 a0 6b :2 a0 51 a5 b :2 a0 6b
a5 b 5a d :a a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :a a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 :2 19 3c :3 a0 51 a5 b
7e :2 a0 51 a5 b b4 2e d
b7 19 3c a0 51 d :a a0 51
a5 b :2 a0 51 a5 b a5 b
d :2 a0 51 a5 b 7e 51 b4
2e :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
a0 b7 a0 7e 51 b4 2e :4 a0
12a :16 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 :2 19 3c :3 a0 a5 b d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 51 a5 b :2 a0 a5 57
b7 19 3c :2 a0 51 a5 b a0
7e a0 51 a5 b b4 2e :c a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 51 a5 b 7e 51 b4
2e :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 19 3c :2 a0 d :3 a0 51 a5
b d :2 a0 51 a5 b a0 7e
a0 51 a5 b b4 2e a0 7e
51 b4 2e d :c a0 a5 b d
a0 7e b4 2e 5a :4 a0 51 a5
b d b7 :2 a0 :2 51 a5 b 7e
6e b4 2e :4 a0 51 a5 b d
b7 :4 a0 51 a5 b d b7 :2 19
3c b7 :2 19 3c a0 51 d a0
7e b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e :3 a0 6b :2 a0 51
a5 b :2 a0 6b a5 b 5a d
:a a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 :a a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :2 19 3c
:2 a0 51 a5 b 7e 51 b4 2e
:c a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 19 3c b7 19 3c
:3 a0 51 a5 b 7e :2 a0 51 a5
b b4 2e 7e :2 a0 51 a5 b
b4 2e d a0 51 d :13 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a a0
51 d :8 a0 51 :2 a0 51 a5 b
a5 b d :2 a0 51 a5 b 7e
51 b4 2e :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 51 a5 b
:2 a0 a5 57 b7 19 3c b7 19
3c :3 a0 51 a5 b d b7 :5 a0
a5 b 51 a5 b 7e :3 a0 a5
b 51 a5 b b4 2e 5a 7e
51 b4 2e 5a 7e 51 b4 2e
a5 b d b7 :2 19 3c :1b a0 51
a5 b a5 b d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :3 a0 a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 51
a5 b :2 a0 a5 57 a0 b7 :2 a0
51 a5 b a0 7e a0 51 a5
b b4 2e :2 a0 51 a5 b 7e
51 b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a a 10 :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 19
3c :b a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :b a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 :2 19 3c :3 a0 a5 b d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :3 a0
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 51 a5 b :2 a0 a5 57
b7 19 :20 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a 7e b4 2e
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 b7
19 3c :2 a0 6b :2 a0 a5 b 7e
51 b4 2e :2 a0 6b 51 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 19
3c b7 :2 19 3c b7 19 3c :2 a0
d :2 a0 d :2 a0 d :3 a0 6b d
:2 a0 6b :5 a0 a5 b d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b :2 a0
a5 57 b7 :2 a0 6b :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 :2 a0 6b :2 6e
7e :2 a0 6b b4 2e 7e 6e b4
2e 7e :2 a0 6b b4 2e a5 57
b7 19 3c :9 a0 12a e9 dd :2 a0
65 b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c :7 a0 12a e9 dd :2 a0 65 b7
a4 b1 11 4f b7 a6 9 a4
b1 11 68 4f 1d 17 b5 
2628
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 6f 6b 59 77 84
80 56 8c 95 91 7f 9d aa
a6 7c b2 bb b7 a5 c3 a2
c8 cc d0 d4 d7 db f8 e3
e7 ea eb f3 e2 115 103 df
107 108 110 102 132 120 ff 124
125 12d 11f 14f 13d 11c 141 142
14a 13c 16c 15a 139 15e 15f 167
159 189 177 156 17b 17c 184 176
1a6 194 173 198 199 1a1 193 1c3
1b1 190 1b5 1b6 1be 1b0 1e0 1ce
1ad 1d2 1d3 1db 1cd 1fd 1eb 1ca
1ef 1f0 1f8 1ea 21a 208 1e7 20c
20d 215 207 237 225 204 229 22a
232 224 254 242 221 246 247 24f
241 270 25f 263 26b 23e 288 277
27b 283 25e 2a4 293 297 29f 25b
2bc 2ab 2af 2b7 292 2d8 2c7 2cb
2d3 28f 2f0 2df 2e3 2eb 2c6 30c
2fb 2ff 307 2c3 324 313 317 31f
2fa 340 32f 333 33b 2f7 35c 347
34b 34e 34f 357 32e 379 367 32b
36b 36c 374 366 395 384 388 363
390 383 3b1 3a0 3a4 380 3ac 39f
3cd 3bc 3c0 39c 3c8 3bb 3e9 3d8
3dc 3e4 3b8 401 3f0 3f4 3fc 3d7
41d 40c 410 3d4 418 40b 439 428
42c 408 434 427 455 444 448 424
450 443 471 460 464 440 46c 45f
48d 47c 480 45c 488 47b 4a9 498
49c 478 4a4 497 4c5 4b4 4b8 494
4c0 4b3 4e1 4d0 4d4 4b0 4dc 4cf
4fd 4ec 4f0 4cc 4f8 4eb 519 508
50c 4e8 514 507 536 524 504 528
529 531 523 553 541 520 545 546
54e 540 570 55e 53d 562 563 56b
55d 58c 57b 57f 55a 587 57a 5a8
597 59b 577 5a3 596 5c4 5b3 5b7
593 5bf 5b2 5e0 5cf 5d3 5af 5db
5ce 5fc 5eb 5ef 5cb 5f7 5ea 618
607 60b 5e7 613 606 634 623 627
603 62f 622 650 63f 643 61f 64b
63e 66c 65b 65f 63b 667 65a 688
677 67b 657 683 676 6a4 693 697
673 69f 692 6c0 6af 6b3 68f 6bb
6ae 6dc 6cb 6cf 6ab 6d7 6ca 6f8
6e7 6eb 6c7 6f3 6e6 714 703 707
6e3 70f 702 730 71f 723 6ff 72b
71e 74c 73b 73f 71b 747 73a 768
757 75b 737 763 756 784 773 777
753 77f 772 7a0 78f 793 76f 79b
78e 7bc 7ab 7af 78b 7b7 7aa 7d8
7c7 7cb 7a7 7d3 7c6 7f4 7e3 7e7
7c3 7ef 7e2 810 7ff 803 7df 80b
7fe 82d 81b 7fb 81f 820 828 81a
84a 838 817 83c 83d 845 837 866
855 859 834 861 854 883 871 851
875 876 87e 870 89f 88e 892 86d
89a 88d 8bb 8aa 8ae 88a 8b6 8a9
8d7 8c6 8ca 8a6 8d2 8c5 8f3 8e2
8e6 8c2 8ee 8e1 90f 8fe 902 8de
90a 8fd 92b 91a 91e 8fa 926 919
947 936 93a 916 942 935 963 952
956 932 95e 951 97f 96e 972 94e
97a 96d 99b 98a 98e 96a 996 989
9b7 9a6 9aa 986 9b2 9a5 9d4 9c2
9a2 9c6 9c7 9cf 9c1 9f0 9df 9e3
9eb 9be a08 9f7 9fb a03 9de a28
a13 a17 9db a1b a23 a12 a48 a33
a37 a0f a3b a43 a32 a68 a53 a57
a2f a5b a63 a52 a6f a73 a4f a77
a7b a7f a82 a87 a88 1 a8a a8f
a93 a97 a9a a9f aa3 aa7 aaa aaf
ab0 ab2 ab7 aba abe abf ac4 ac9
acc ad1 ad2 ad7 ada ade adf ae4
ae7 aec aed af2 af5 afa afb b00
b03 b08 b09 b0e b11 b15 b16 b1b
b1e b23 b24 b29 b2c b31 b32 b37
b3a b3e b42 b47 b4a b4e b4f b54
b57 b5c b5d b62 b67 b68 b6a b6f
b70 b72 b73 b78 b7b b80 b81 b86
b89 b8d b91 b96 b99 b9d b9e ba3
ba6 bab bac bb1 bb6 bb7 bb9 bbe
bbf bc1 bc2 bc7 bca bcf bd0 bd5
bd8 bdc be0 be4 be5 be7 bec bed
bef bf0 bf5 bf8 bfd bfe c03 c06
c0a c0e c12 c13 c15 c1a c1b c1d
c1e c23 c26 c2b c2c c31 c34 c38
c3c c40 c41 c43 c48 c49 c4b c4c
c51 c54 c59 c5a c5f c62 c66 c6a
c6e c6f c71 c76 c77 c79 c7a c7f
c82 c87 c88 c8d c90 c95 c96 c9b
c9e ca2 ca3 ca8 cab cb0 cb1 cb6
cb9 cbe cbf cc4 cc5 cca ccc cd0
cd3 cd7 cdb cde ce2 ce6 ce9 cec
cf1 cf4 cf7 cfa cfb d00 d01 1
d03 d08 d0c d10 d13 d18 d1d d20
d24 d25 d2a d2b d30 d32 d36 d39
d3d d40 d44 d48 d4d d51 d55 d59
d5d d61 d65 d69 d6d d71 d75 d79
d7d d81 d85 d89 d8d d91 d95 d99
d9d da1 da5 da9 dad db1 db5 db9
dbd dc1 dc5 dc9 dcd dd1 dd5 dd9
ddd de1 de5 de9 ded df1 df5 e01
e05 e09 e0c e10 e14 e16 e1a e1e
e22 e25 e29 e2d e2f e30 e35 1
e39 e3d e41 e44 e48 e4c e50 e54
e57 e5b e5f e62 e65 e6a e6d e70
e73 e74 e79 e7a 1 e7c e81 e85
e89 e8c e91 e96 e99 e9d e9e ea3
ea6 eab eac eb1 eb4 eb9 eba ebf
ec2 ec6 ec7 ecc ecf ed4 ed5 eda
edd ee1 ee5 ee8 eeb eec eee eef
ef4 ef5 efa efc f00 f03 f05 f09
f0b f17 f19 f1b f1c f21 f25 f27
f33 f35 f39 f3d f40 f44 f47 f4b
f4e f4f f54 f57 f5b f5f f63 f67
f6b f6f f73 f77 f7b f7f f83 f87
f8b f8f f93 f97 f9b f9f fa3 fa7
fab faf fb3 fb7 fbb fbf fc3 fc7
fcb fcf fd3 fd7 fdb fdf fe3 fe7
feb fef ff3 ff7 ffb fff 1003 1007
100b 100f 1013 1017 101b 101f 1023 1027
102b 102f 1033 1037 103b 103f 1043 1047
104b 104f 1053 1057 105b 105f 1063 1067
106b 106f 1073 1077 107b 107f 1083 1087
108b 108f 1093 1097 109b 109f 10ab 10af
10b3 10b6 10ba 10be 10c0 10c4 10c8 10cc
10cf 10d3 10d7 10d9 10da 10df 1 10e3
10e7 10eb 10ee 10f2 10f6 10fa 10fe 1101
1105 1109 110c 110f 1114 1117 111a 111d
111e 1123 1124 1 1126 112b 112f 1133
1136 113b 1140 1143 1147 1148 114d 1150
1155 1156 115b 115e 1163 1164 1169 116c
1170 1171 1176 1179 117e 117f 1184 1187
118b 118f 1192 1195 1196 1198 1199 119e
119f 11a4 11a6 11aa 11ad 11af 11b3 11b5
11c1 11c3 11c5 11c6 11cb 11cf 11d1 11dd
11df 11e3 11e7 11ea 11ee 11f1 11f5 11f8
11f9 11fe 1201 1205 1209 120d 1211 1215
1219 121d 1221 1225 1229 122d 1231 1235
1239 123d 1241 1245 1249 124d 1251 1255
1259 125d 1261 1265 1269 126d 1271 1275
1279 127d 1281 128d 1291 1295 1298 129c
12a0 12a2 12a6 12aa 12ae 12b1 12b5 12b9
12bb 12bc 12c1 1 12c5 12c9 12cd 12d0
12d4 12d8 12dc 12e0 12e3 12e7 12eb 12ee
12f1 12f6 12f9 12fc 12ff 1300 1305 1306
1 1308 130d 1311 1315 1318 131d 1322
1325 1329 132a 132f 1332 1337 1338 133d
1340 1345 1346 134b 134e 1352 1353 1358
135b 1360 1361 1366 1369 136d 1371 1374
1377 1378 137a 137b 1380 1381 1386 1388
138c 138f 1391 1395 1397 13a3 13a5 13a7
13a8 13ad 13b1 13b3 13bf 13c1 13c5 13c9
13cc 13d0 13d3 13d7 13da 13db 13e0 13e3
13e7 13eb 13ee 13f2 13f6 13f9 13fc 1401
1404 1407 140a 140b 1410 1411 1 1413
1418 141c 1420 1423 1428 142d 1430 1434
1435 143a 143d 1442 1443 1448 144b 1450
1451 1456 1459 145d 1461 1465 1469 146c
146d 146f 1474 1475 1477 1478 147d 1480
1485 1486 148b 148e 1493 1494 1499 149c
14a0 14a1 14a6 14a9 14ae 14af 14b4 14b5
14ba 14bc 14c0 14c3 14c7 14cb 14ce 14d2
14d6 14d9 14dc 14e1 14e4 14e7 14ea 14eb
14f0 14f1 1 14f3 14f8 14fc 1500 1503
1508 150d 1510 1514 1515 151a 151b 1520
1522 1526 1529 152d 1531 1534 1538 153c
153f 1543 1547 1548 154d 154f 1553 1556
155a 155e 1562 1566 156a 156e 1572 1576
1579 157d 1581 1584 1585 1587 158a 158d
158e 1593 1596 159a 159e 15a1 15a5 15a9
15ac 15ad 15af 15b2 15b5 15b6 15bb 1
15be 15c3 15c7 15cb 15ce 15d2 15d6 15d9
15dc 15e1 15e4 15e7 15ea 15eb 15f0 15f1
1 15f3 15f8 15fc 1600 1603 1608 160d
1610 1614 1615 161a 161d 1622 1623 1628
162b 1630 1631 1636 1639 163d 1641 1645
1649 164c 164d 164f 1654 1655 1657 1658
165d 1660 1665 1666 166b 166e 1673 1674
1679 167c 1680 1681 1686 1689 168e 168f
1694 1695 169a 169c 16a0 16a3 16a7 16ab
16ae 16b2 16b6 16b9 16bc 16c1 16c4 16c7
16ca 16cb 16d0 16d1 1 16d3 16d8 16dc
16e0 16e3 16e8 16ed 16f0 16f4 16f5 16fa
16fb 1700 1702 1706 1709 170d 1711 1714
1718 171c 171f 1723 1727 1728 172d 172f
1733 1736 173a 173d 173e 1743 1746 1749
174d 1751 1754 1755 1757 175a 175e 1762
1765 1766 1768 1769 176e 1771 1775 1778
177c 177f 1780 1782 1783 1788 1 178b
1790 1794 1798 179b 179f 17a3 17a6 17a9
17ae 17b1 17b4 17b7 17b8 17bd 17be 1
17c0 17c5 17c9 17cd 17d0 17d5 17da 17dd
17e1 17e2 17e7 17ea 17ef 17f0 17f5 17f8
17fd 17fe 1803 1806 180a 180e 1812 1816
1819 181a 181c 1821 1822 1824 1825 182a
182d 1832 1833 1838 183b 1840 1841 1846
1849 184d 184e 1853 1856 185b 185c 1861
1862 1867 1869 186d 1870 1874 1878 187b
187f 1883 1886 1889 188e 1891 1894 1897
1898 189d 189e 1 18a0 18a5 18a9 18ad
18b0 18b5 18ba 18bd 18c1 18c2 18c7 18c8
18cd 18cf 18d3 18d6 18da 18de 18e1 18e5
18e9 18ec 18f0 18f4 18f5 18fa 18fc 1900
1903 1907 190b 190e 1912 1916 1919 191a
191c 191f 1922 1923 1928 192b 192f 1933
1936 193a 193e 1941 1942 1944 1947 194a
194b 1950 1 1953 1958 195c 1960 1963
1964 1966 1969 196c 196d 1972 1976 197a
197d 1981 1985 1988 1989 198b 198e 1991
1992 1997 199b 199f 19a3 19a7 19ab 19af
19b3 19b7 19bb 19bf 19c3 19c7 19cb 19cf
19d3 19d7 19e3 19e7 19eb 19ee 19f2 19f6
19f8 19fc 1a00 1a04 1a07 1a0b 1a0f 1a11
1a12 1a17 1 1a1b 1a1f 1a23 1a26 1a2a
1a2e 1a32 1a36 1a39 1a3d 1a41 1a44 1a47
1a4c 1a4f 1a52 1a55 1a56 1a5b 1a5c 1
1a5e 1a63 1a67 1a6b 1a6e 1a73 1a78 1a7b
1a7f 1a80 1a85 1a88 1a8d 1a8e 1a93 1a96
1a9b 1a9c 1aa1 1aa4 1aa8 1aa9 1aae 1ab1
1ab6 1ab7 1abc 1abf 1ac3 1ac7 1aca 1acd
1ace 1ad0 1ad1 1ad6 1ad7 1adc 1ade 1ae2
1ae5 1ae7 1aeb 1aed 1af9 1afb 1afd 1afe
1b03 1b07 1b09 1b15 1b17 1b1b 1b1f 1b22
1b26 1b29 1b2d 1b30 1b31 1b36 1b39 1b3d
1b41 1b45 1b49 1b4d 1b51 1b55 1b59 1b5d
1b69 1b6d 1b71 1b74 1b78 1b7c 1b7e 1b82
1b86 1b8a 1b8d 1b91 1b95 1b97 1b98 1b9d
1 1ba1 1ba5 1ba9 1bac 1bb0 1bb4 1bb8
1bbc 1bbf 1bc3 1bc7 1bca 1bcd 1bd2 1bd5
1bd8 1bdb 1bdc 1be1 1be2 1 1be4 1be9
1bed 1bf1 1bf4 1bf9 1bfe 1c01 1c05 1c06
1c0b 1c0e 1c13 1c14 1c19 1c1c 1c21 1c22
1c27 1c2a 1c2e 1c2f 1c34 1c37 1c3c 1c3d
1c42 1c45 1c49 1c4d 1c50 1c53 1c54 1c56
1c57 1c5c 1c5d 1c62 1c64 1c68 1c6b 1c6d
1c71 1c73 1c7f 1c81 1c83 1c84 1c89 1c8d
1c8f 1c9b 1c9d 1c9f 1ca3 1ca6 1caa 1cae
1cb1 1cb5 1cb9 1cbc 1cbd 1cbf 1cc2 1cc5
1cc6 1ccb 1ccf 1cd0 1cd4 1cd6 1cda 1cdd
1cdf 1ce3 1ce7 1ceb 1cef 1cf3 1cf7 1cfb
1cff 1d03 1d07 1d0b 1d0f 1d13 1d17 1d1b
1d1f 1d2b 1d2f 1d33 1d36 1d3a 1d3e 1d40
1d44 1d48 1d4c 1d4f 1d53 1d57 1d59 1d5a
1d5f 1 1d63 1d67 1d6b 1d6e 1d72 1d76
1d7a 1d7e 1d81 1d85 1d89 1d8c 1d8f 1d94
1d97 1d9a 1d9d 1d9e 1da3 1da4 1 1da6
1dab 1daf 1db3 1db6 1dbb 1dc0 1dc3 1dc7
1dc8 1dcd 1dd0 1dd5 1dd6 1ddb 1dde 1de3
1de4 1de9 1dec 1df0 1df1 1df6 1df9 1dfe
1dff 1e04 1e07 1e0b 1e0f 1e12 1e15 1e16
1e18 1e19 1e1e 1e1f 1e24 1e26 1e2a 1e2d
1e2f 1e33 1e35 1e41 1e43 1e45 1e46 1e4b
1e4f 1e51 1e5d 1e5f 1e63 1e67 1e6a 1e6e
1e71 1e75 1e78 1e79 1e7e 1e81 1e85 1e89
1e8d 1e91 1e95 1e99 1e9d 1ea1 1ea5 1eb1
1eb5 1eb9 1ebc 1ec0 1ec4 1ec6 1eca 1ece
1ed2 1ed5 1ed9 1edd 1edf 1ee0 1ee5 1
1ee9 1eed 1ef1 1ef4 1ef8 1efc 1f00 1f04
1f07 1f0b 1f0f 1f12 1f15 1f1a 1f1d 1f20
1f23 1f24 1f29 1f2a 1 1f2c 1f31 1f35
1f39 1f3c 1f41 1f46 1f49 1f4d 1f4e 1f53
1f56 1f5b 1f5c 1f61 1f64 1f69 1f6a 1f6f
1f72 1f76 1f77 1f7c 1f7f 1f84 1f85 1f8a
1f8d 1f91 1f95 1f98 1f9b 1f9c 1f9e 1f9f
1fa4 1fa5 1faa 1fac 1fb0 1fb3 1fb5 1fb9
1fbb 1fc7 1fc9 1fcb 1fcc 1fd1 1fd5 1fd7
1fe3 1fe5 1fe7 1feb 1fee 1ff2 1ff5 1ff9
1ffb 1fff 2003 2006 200a 200e 2011 2012
2014 2018 201b 201f 2022 2023 2028 202c
2030 2034 2038 203c 2040 2044 2048 204c
2058 205c 2061 2065 2068 2069 206e 2072
2076 2079 207c 2080 2082 2086 208a 208d
2090 2094 2096 209a 209e 20a1 20a3 20a7
20ab 20af 20b2 20b6 20ba 20bc 20bd 20c2
1 20c6 20ca 20ce 20d1 20d5 20d9 20dd
20e1 20e4 20e8 20ec 20ef 20f2 20f7 20fa
20fd 2100 2101 2106 2107 1 2109 210e
2112 2116 2119 211e 2123 2126 212a 212b
2130 2133 2138 2139 213e 2141 2146 2147
214c 214f 2153 2154 2159 215c 2161 2162
2167 216a 216e 2172 2175 2178 2179 217b
217c 2181 2182 2187 2189 218d 2190 2192
2196 2198 21a4 21a6 21a8 21a9 21ae 21b2
21b4 21c0 21c2 21c4 21c8 21cc 21d0 21d4
21d8 21dc 21e0 21e4 21e8 21f4 21f8 21fd
2201 2204 2205 220a 220e 2212 2215 2218
221c 221e 2222 2226 2229 222c 2230 2232
2236 223a 223d 223f 2243 2247 224b 224e
2252 2256 2258 2259 225e 1 2262 2266
226a 226d 2271 2275 2279 227d 2280 2284
2288 228b 228e 2293 2296 2299 229c 229d
22a2 22a3 1 22a5 22aa 22ae 22b2 22b5
22ba 22bf 22c2 22c6 22c7 22cc 22cf 22d4
22d5 22da 22dd 22e2 22e3 22e8 22eb 22ef
22f0 22f5 22f8 22fd 22fe 2303 2306 230a
230e 2311 2314 2315 2317 2318 231d 231e
2323 2325 2329 232c 232e 2332 2334 2340
2342 2344 2345 234a 234e 2350 235c 235e
2360 2364 2368 236b 236f 2373 2376 237a
237d 2381 2384 2385 238a 238d 2391 2395
2399 239a 239c 23a0 23a2 23a6 23a9 23ab
23af 23b3 23b7 23bb 23bf 23c3 23c7 23cb
23cf 23db 23df 23e3 23e6 23ea 23ee 23f0
23f4 23f8 23fc 23ff 2403 2407 2409 240a
240f 1 2413 2417 241b 241e 2422 2426
242a 242e 2431 2435 2439 243c 243f 2444
2447 244a 244d 244e 2453 2454 1 2456
245b 245f 2463 2466 246b 2470 2473 2477
2478 247d 2480 2485 2486 248b 248e 2493
2494 2499 249c 24a0 24a1 24a6 24a9 24ae
24af 24b4 24b7 24bb 24bf 24c2 24c5 24c6
24c8 24c9 24ce 24cf 24d4 24d6 24da 24dd
24df 24e3 24e5 24f1 24f3 24f5 24f6 24fb
24ff 2501 250d 250f 2513 2517 251a 251b
251d 2520 2523 2524 2529 252d 2531 2534
2538 253c 253f 2542 2547 254a 254d 2550
2551 2556 2557 1 2559 255e 2562 2566
2569 256e 2573 2576 257a 257b 2580 2583
2588 2589 258e 2591 2596 2597 259c 259f
25a3 25a7 25ab 25af 25b2 25b3 25b5 25ba
25bb 25bd 25be 25c3 25c6 25cb 25cc 25d1
25d4 25d9 25da 25df 25e2 25e6 25e7 25ec
25ef 25f4 25f5 25fa 25fb 2600 2602 2606
2609 260d 2611 2614 2618 261c 261f 2622
2627 262a 262d 2630 2631 2636 2637 1
2639 263e 2642 2646 2649 264e 2653 2656
265a 265b 2660 2661 2666 2668 266c 266f
2673 2677 267a 267e 2682 2685 2689 268d
268e 2693 2695 2699 269c 26a0 26a4 26a7
26ab 26af 26b2 26b3 26b5 26b8 26bb 26bc
26c1 26c5 26c9 26cd 26d1 26d5 26d9 26dd
26e1 26e5 26e9 26ed 26f1 26f5 26f9 26fd
2701 270d 2711 2715 2718 271c 2720 2722
2726 272a 272e 2731 2735 2739 273b 273c
2741 1 2745 2749 274d 2750 2754 2758
275c 2760 2763 2767 276b 276e 2771 2776
2779 277c 277f 2780 2785 2786 1 2788
278d 2791 2795 2798 279d 27a2 27a5 27a9
27aa 27af 27b2 27b7 27b8 27bd 27c0 27c5
27c6 27cb 27ce 27d2 27d3 27d8 27db 27e0
27e1 27e6 27e9 27ed 27f1 27f4 27f7 27f8
27fa 27fb 2800 2801 2806 2808 280c 280f
2811 2815 2817 2823 2825 2827 2828 282d
2831 2833 283f 2841 2845 2849 284c 2850
2853 2857 285a 285b 2860 2863 2867 286b
286f 2873 2877 287b 287f 2883 2887 2893
2897 289b 289e 28a2 28a6 28a8 28ac 28b0
28b4 28b7 28bb 28bf 28c1 28c2 28c7 1
28cb 28cf 28d3 28d6 28da 28de 28e2 28e6
28e9 28ed 28f1 28f4 28f7 28fc 28ff 2902
2905 2906 290b 290c 1 290e 2913 2917
291b 291e 2923 2928 292b 292f 2930 2935
2938 293d 293e 2943 2946 294b 294c 2951
2954 2958 2959 295e 2961 2966 2967 296c
296f 2973 2977 297a 297d 297e 2980 2981
2986 2987 298c 298e 2992 2995 2997 299b
299d 29a9 29ab 29ad 29ae 29b3 29b7 29b9
29c5 29c7 29c9 29cd 29d0 29d2 29d6 29da
29de 29e2 29e6 29ea 29ee 29f2 29f6 29fa
29fe 2a02 2a06 2a0a 2a0e 2a12 2a1e 2a22
2a26 2a29 2a2d 2a31 2a33 2a37 2a3b 2a3f
2a42 2a46 2a4a 2a4c 2a4d 2a52 1 2a56
2a5a 2a5e 2a61 2a65 2a69 2a6d 2a71 2a74
2a78 2a7c 2a7f 2a82 2a87 2a8a 2a8d 2a90
2a91 2a96 2a97 1 2a99 2a9e 2aa2 2aa6
2aa9 2aae 2ab3 2ab6 2aba 2abb 2ac0 2ac3
2ac8 2ac9 2ace 2ad1 2ad6 2ad7 2adc 2adf
2ae3 2ae4 2ae9 2aec 2af1 2af2 2af7 2afa
2afe 2b02 2b05 2b08 2b09 2b0b 2b0c 2b11
2b12 2b17 2b19 2b1d 2b20 2b22 2b26 2b28
2b34 2b36 2b38 2b39 2b3e 2b42 2b44 2b50
2b52 2b56 2b5a 2b5d 2b61 2b64 2b68 2b6b
2b6c 2b71 2b74 2b78 2b7c 2b80 2b84 2b88
2b8c 2b90 2b94 2b98 2ba4 2ba8 2bac 2baf
2bb3 2bb7 2bb9 2bbd 2bc1 2bc5 2bc8 2bcc
2bd0 2bd2 2bd3 2bd8 1 2bdc 2be0 2be4
2be7 2beb 2bef 2bf3 2bf7 2bfa 2bfe 2c02
2c05 2c08 2c0d 2c10 2c13 2c16 2c17 2c1c
2c1d 1 2c1f 2c24 2c28 2c2c 2c2f 2c34
2c39 2c3c 2c40 2c41 2c46 2c49 2c4e 2c4f
2c54 2c57 2c5c 2c5d 2c62 2c65 2c69 2c6a
2c6f 2c72 2c77 2c78 2c7d 2c80 2c84 2c88
2c8b 2c8e 2c8f 2c91 2c92 2c97 2c98 2c9d
2c9f 2ca3 2ca6 2ca8 2cac 2cae 2cba 2cbc
2cbe 2cbf 2cc4 2cc8 2cca 2cd6 2cd8 2cda
2cde 2ce1 2ce3 2ce7 2ceb 2cee 2cf2 2cf6
2cfa 2cfe 2d02 2d06 2d0a 2d0e 2d12 2d16
2d1a 2d1e 2d21 2d22 2d24 2d27 2d28 2d2a
2d2e 2d32 2d36 2d39 2d3a 2d3c 2d3f 2d42
2d43 2d48 2d4c 2d50 2d54 2d55 2d57 2d5b
2d5d 2d61 2d65 2d68 2d6c 2d70 2d73 2d76
2d7b 2d7e 2d81 2d84 2d85 2d8a 2d8b 1
2d8d 2d92 2d96 2d9a 2d9d 2da2 2da7 2daa
2dae 2daf 2db4 2db7 2dbc 2dbd 2dc2 2dc5
2dca 2dcb 2dd0 2dd3 2dd7 2ddb 2ddf 2de0
2de2 2de7 2de8 2dea 2deb 2df0 2df3 2df8
2df9 2dfe 2e01 2e06 2e07 2e0c 2e0f 2e13
2e14 2e19 2e1c 2e21 2e22 2e27 2e28 2e2d
2e2f 2e33 2e36 2e3a 2e3e 2e41 2e45 2e49
2e4c 2e4f 2e54 2e57 2e5a 2e5d 2e5e 2e63
2e64 1 2e66 2e6b 2e6f 2e73 2e76 2e7b
2e80 2e83 2e87 2e88 2e8d 2e8e 2e93 2e95
2e99 2e9c 2ea0 2ea4 2ea7 2eab 2eaf 2eb2
2eb3 2eb5 2eb9 2ebd 2ebe 2ec3 2ec5 2ec9
2ecd 2ed0 2ed2 2ed6 2eda 2edd 2ee1 2ee3
2ee7 2eeb 2eee 2eef 2ef1 2ef5 2ef8 2efc
2eff 2f00 2f02 2f03 2f08 2f0b 2f0f 2f13
2f16 2f17 2f19 2f1c 2f1f 2f20 2f25 1
2f28 2f2d 2f31 2f34 2f38 2f3c 2f40 2f43
2f44 2f46 2f4a 2f4d 2f51 2f54 2f55 2f57
2f58 2f5d 2f61 2f65 2f68 2f6c 2f70 2f73
2f76 2f7b 2f7e 2f81 2f84 2f85 2f8a 2f8b
1 2f8d 2f92 2f96 2f9a 2f9d 2fa2 2fa7
2faa 2fae 2faf 2fb4 2fb7 2fbc 2fbd 2fc2
2fc5 2fca 2fcb 2fd0 2fd3 2fd7 2fdb 2fdf
2fe3 2fe6 2fe7 2fe9 2fee 2fef 2ff1 2ff2
2ff7 2ff8 2ffd 2fff 3003 3006 300a 300e
3011 3015 3019 301c 301f 3024 3027 302a
302d 302e 3033 3034 1 3036 303b 303f
3043 3046 304b 3050 3053 3057 3058 305d
305e 3063 3065 3069 306c 3070 3074 3077
307b 307f 3082 3086 3087 308c 3090 3092
3096 309a 309d 309e 30a0 30a4 30a7 30ab
30ae 30af 30b1 30b2 30b7 30bb 30be 30c2
30c4 30c8 30cc 30cf 30d3 30d6 30da 30de
30e2 30e6 30ea 30ee 30f2 30f6 30fa 30fe
3102 3106 310a 310e 3112 3116 311a 311e
3122 3126 312a 312e 3132 3136 313a 3146
314a 314e 3151 3155 3159 315b 315f 3163
3167 316a 316e 3172 3174 3175 317a 1
317e 3182 3186 3189 318d 3191 3195 3199
319c 31a0 31a4 31a7 31aa 31af 31b2 31b5
31b8 31b9 31be 31bf 1 31c1 31c6 31ca
31ce 31d1 31d6 31db 31de 31e2 31e3 31e8
31eb 31f0 31f1 31f6 31f9 31fe 31ff 3204
3207 320b 320c 3211 3214 3219 321a 321f
3222 3226 322a 322d 3230 3231 3233 3234
3239 323a 323f 3241 3245 3248 324a 324e
3250 325c 325e 3260 3261 3266 326a 326c
3278 327a 327e 3282 3285 3286 3288 328b
328e 328f 3294 3298 329c 329f 32a3 32a7
32aa 32ad 32b2 32b5 32b8 32bb 32bc 32c1
32c2 1 32c4 32c9 32cd 32d1 32d4 32d9
32de 32e1 32e5 32e6 32eb 32ee 32f3 32f4
32f9 32fc 3301 3302 3307 330a 330e 3312
3316 331a 331d 331e 3320 3325 3326 3328
3329 332e 3331 3336 3337 333c 333f 3344
3345 334a 334d 3351 3352 3357 335a 335f
3360 3365 3366 336b 336d 3371 3374 3378
337c 337f 3383 3387 338a 338d 3392 3395
3398 339b 339c 33a1 33a2 1 33a4 33a9
33ad 33b1 33b4 33b9 33be 33c1 33c5 33c6
33cb 33cc 33d1 33d3 33d7 33da 33de 33e2
33e5 33e9 33ed 33f0 33f4 33f8 33f9 33fe
3400 3404 3407 340b 340f 3413 3417 341b
341f 3423 3427 342b 342f 3433 3437 343b
343f 3443 3447 344b 344f 3453 3457 345b
345f 3463 3467 346b 346f 3473 3477 3483
3487 348b 348e 3492 3496 3498 349c 34a0
34a4 34a7 34ab 34af 34b1 34b2 34b7 1
34bb 34bf 34c3 34c6 34ca 34ce 34d2 34d6
34d9 34dd 34e1 34e4 34e7 34ec 34ef 34f2
34f5 34f6 34fb 34fc 1 34fe 3503 3507
350b 350e 3513 3518 351b 351f 3520 3525
3528 352d 352e 3533 3536 353b 353c 3541
3544 3548 3549 354e 3551 3556 3557 355c
355f 3563 3567 356a 356d 356e 3570 3571
3576 3577 357c 357e 3582 3585 3587 358b
358d 3599 359b 359d 359e 35a3 35a7 35a9
35b5 35b7 35bb 35bf 35c2 35c3 35c5 35c8
35cb 35cc 35d1 35d5 35d9 35dc 35e0 35e4
35e7 35ea 35ef 35f2 35f5 35f8 35f9 35fe
35ff 1 3601 3606 360a 360e 3611 3616
361b 361e 3622 3623 3628 362b 3630 3631
3636 3639 363e 363f 3644 3647 364b 364f
3653 3657 365a 365b 365d 3662 3663 3665
3666 366b 366e 3673 3674 3679 367c 3681
3682 3687 368a 368e 368f 3694 3697 369c
369d 36a2 36a3 36a8 36aa 36ae 36b1 36b5
36b9 36bc 36c0 36c4 36c7 36ca 36cf 36d2
36d5 36d8 36d9 36de 36df 1 36e1 36e6
36ea 36ee 36f1 36f6 36fb 36fe 3702 3703
3708 3709 370e 3710 3714 3717 371b 371f
3722 3726 372a 372d 3731 3735 3736 373b
373d 3741 3744 3748 374c 3750 3754 3758
375c 3760 3764 3768 376c 3770 3773 3774
3776 3777 3779 377d 377f 3783 3787 378a
378e 3792 3795 3796 3798 379b 379e 379f
37a4 37a8 37ac 37b0 37b1 37b3 37b7 37b9
37bd 37c0 37c4 37c8 37cb 37cf 37d3 37d6
37d9 37de 37e1 37e4 37e7 37e8 37ed 37ee
1 37f0 37f5 37f9 37fd 3800 3805 380a
380d 3811 3812 3817 381a 381f 3820 3825
3828 382d 382e 3833 3836 383a 383e 3842
3843 3845 384a 384b 384d 384e 3853 3856
385b 385c 3861 3864 3869 386a 386f 3872
3876 3877 387c 387f 3884 3885 388a 388b
3890 3892 3896 3899 389d 38a1 38a4 38a8
38ac 38af 38b2 38b7 38ba 38bd 38c0 38c1
38c6 38c7 1 38c9 38ce 38d2 38d6 38d9
38de 38e3 38e6 38ea 38eb 38f0 38f1 38f6
38f8 38fc 38ff 3903 3907 390a 390e 3912
3915 3916 3918 391c 3920 3921 3926 392a
392c 3930 3933 3934 3939 393c 3940 3943
3944 3949 1 394c 3951 3955 3959 395d
3961 3965 3969 396d 3971 3974 3978 397c
397f 3980 3982 3985 3988 3989 398e 3991
3995 3999 399c 39a0 39a4 39a7 39a8 39aa
39ad 39b0 39b1 39b6 1 39b9 39be 39c2
39c6 39c9 39cd 39d1 39d4 39d7 39dc 39df
39e2 39e5 39e6 39eb 39ec 1 39ee 39f3
39f7 39fb 39fe 3a03 3a08 3a0b 3a0f 3a10
3a15 3a18 3a1d 3a1e 3a23 3a26 3a2b 3a2c
3a31 3a34 3a38 3a3c 3a40 3a44 3a47 3a48
3a4a 3a4f 3a50 3a52 3a53 3a58 3a5b 3a60
3a61 3a66 3a69 3a6e 3a6f 3a74 3a77 3a7b
3a7c 3a81 3a84 3a89 3a8a 3a8f 3a90 3a95
3a97 3a9b 3a9e 3aa2 3aa6 3aa9 3aad 3ab1
3ab4 3ab7 3abc 3abf 3ac2 3ac5 3ac6 3acb
3acc 1 3ace 3ad3 3ad7 3adb 3ade 3ae3
3ae8 3aeb 3aef 3af0 3af5 3af6 3afb 3afd
3b01 3b04 3b08 3b0c 3b0f 3b13 3b17 3b1a
3b1e 3b22 3b23 3b28 3b2a 3b2e 3b31 3b35
3b39 3b3d 3b41 3b45 3b49 3b4d 3b51 3b55
3b59 3b5d 3b69 3b6d 3b71 3b74 3b78 3b7c
3b7e 3b82 3b86 3b8a 3b8d 3b91 3b95 3b97
3b98 3b9d 1 3ba1 3ba5 3ba9 3bac 3bb0
3bb4 3bb8 3bbc 3bbf 3bc3 3bc7 3bca 3bcd
3bd2 3bd5 3bd8 3bdb 3bdc 3be1 3be2 1
3be4 3be9 3bed 3bf1 3bf4 3bf9 3bfe 3c01
3c05 3c06 3c0b 3c0e 3c13 3c14 3c19 3c1c
3c21 3c22 3c27 3c2a 3c2e 3c2f 3c34 3c37
3c3c 3c3d 3c42 3c45 3c49 3c4d 3c50 3c53
3c54 3c56 3c57 3c5c 3c5d 3c62 3c64 3c68
3c6b 3c6d 3c71 3c73 3c7f 3c81 3c83 3c84
3c89 3c8d 3c8f 3c9b 3c9d 3ca1 3ca5 3ca8
3cab 3cac 3cae 3cb1 3cb6 3cb7 3cbc 3cc0
3cc3 3cc7 3cc9 3ccd 3cd0 3cd4 3cd6 3cda
3cde 3ce1 3ce5 3ce8 3cec 3cf0 3cf3 3cf7
3cfb 3cff 3d02 3d06 3d0a 3d0d 3d0e 3d10
3d13 3d16 3d17 3d1c 3d1f 3d23 3d27 3d2a
3d2e 3d32 3d35 3d36 3d38 3d3b 3d3e 3d3f
3d44 1 3d47 3d4c 3d50 3d54 3d57 3d58
3d5a 3d5e 3d61 3d65 3d68 3d69 3d6b 3d6c
3d71 1 3d74 3d79 3d7d 3d81 3d84 3d88
3d8c 3d8f 3d90 3d92 3d95 3d98 3d99 3d9e
3da2 3da6 3daa 3dae 3db2 3db6 3dba 3dbe
3dc2 3dce 3dd2 3dd6 3dd9 3ddd 3de1 3de3
3de7 3deb 3def 3df2 3df6 3dfa 3dfc 3dfd
3e02 1 3e06 3e0a 3e0e 3e11 3e15 3e19
3e1d 3e21 3e24 3e28 3e2c 3e2f 3e32 3e37
3e3a 3e3d 3e40 3e41 3e46 3e47 1 3e49
3e4e 3e52 3e56 3e59 3e5e 3e63 3e66 3e6a
3e6b 3e70 3e73 3e78 3e79 3e7e 3e81 3e86
3e87 3e8c 3e8f 3e93 3e94 3e99 3e9c 3ea1
3ea2 3ea7 3eaa 3eae 3eb2 3eb5 3eb8 3eb9
3ebb 3ebc 3ec1 3ec2 3ec7 3ec9 3ecd 3ed0
3ed2 3ed6 3ed8 3ee4 3ee6 3ee8 3ee9 3eee
3ef2 3ef4 3f00 3f02 3f06 3f0a 3f0e 3f12
3f15 3f19 3f1d 3f20 3f24 3f28 3f2a 3f2e
3f32 3f35 3f39 3f3d 3f40 3f41 3f43 3f46
3f49 3f4a 3f4f 3f53 3f57 3f5b 3f5f 3f63
3f67 3f6b 3f6f 3f73 3f77 3f83 3f87 3f8b
3f8e 3f92 3f96 3f98 3f9c 3fa0 3fa4 3fa7
3fab 3faf 3fb1 3fb2 3fb7 1 3fbb 3fbf
3fc3 3fc6 3fca 3fce 3fd2 3fd6 3fd9 3fdd
3fe1 3fe4 3fe7 3fec 3fef 3ff2 3ff5 3ff6
3ffb 3ffc 1 3ffe 4003 4007 400b 400e
4013 4018 401b 401f 4020 4025 4028 402d
402e 4033 4036 403b 403c 4041 4044 4048
4049 404e 4051 4056 4057 405c 405f 4063
4067 406a 406d 406e 4070 4071 4076 4077
407c 407e 4082 4085 4087 408b 408d 4099
409b 409d 409e 40a3 40a7 40a9 40b5 40b7
40bb 40bf 40c3 40c6 40ca 40ce 40cf 40d1
40d5 40d9 40dc 40df 40e0 40e5 40e9 40ed
40f1 40f5 40f9 40fd 4101 4105 4109 410d
4111 4115 4119 411a 411c 4120 4124 4128
412b 412c 412e 4131 4134 4135 413a 413d
4141 4144 4145 414a 414d 1 4150 4155
4159 415d 4161 4165 4169 416d 4171 4175
4179 417d 4181 418d 4191 4196 419a 419d
419e 41a3 41a7 41ab 41ae 41b1 41b5 41b7
41bb 41bf 41c2 41c5 41c9 41cb 41cf 41d3
41d6 41d8 41dc 41e0 41e4 41e7 41eb 41ef
41f1 41f2 41f7 1 41fb 41ff 4203 4206
420a 420e 4212 4216 4219 421d 4221 4224
4227 422c 422f 4232 4235 4236 423b 423c
1 423e 4243 4247 424b 424e 4253 4258
425b 425f 4260 4265 4268 426d 426e 4273
4276 427b 427c 4281 4284 4288 4289 428e
4291 4296 4297 429c 429f 42a3 42a7 42aa
42ad 42ae 42b0 42b1 42b6 42b7 42bc 42be
42c2 42c5 42c7 42cb 42cd 42d9 42db 42dd
42de 42e3 42e7 42e9 42f5 42f7 42f9 42fd
4300 4304 4308 430b 430e 430f 4311 4314
4319 431a 431f 4323 4326 432a 432c 4330
4333 4337 4339 433d 4341 4344 4348 434c
4350 4354 4358 435c 4360 4364 4368 436c
4370 4374 4378 437c 4380 4383 4387 438a
438b 438d 4391 4395 4399 439d 43a1 43a5
43a9 43ad 43b1 43b5 43b9 43bd 43c1 43c5
43c9 43cd 43d1 43d5 43d9 43dd 43e9 43ed
43f1 43f4 43f8 43fc 43fe 4402 4406 440a
440d 4411 4415 4417 4418 441d 1 4421
4425 4429 442c 4430 4434 4438 443c 443f
4443 4447 444a 444d 4452 4455 4458 445b
445c 4461 4462 1 4464 4469 446d 4471
4474 4479 447e 4481 4485 4486 448b 448e
4493 4494 4499 449c 44a1 44a2 44a7 44aa
44ae 44af 44b4 44b7 44bc 44bd 44c2 44c5
44c9 44cd 44d0 44d3 44d4 44d6 44d7 44dc
44dd 44e2 44e4 44e8 44eb 44ed 44f1 44f3
44ff 4501 4503 4504 4509 450d 450f 451b
451d 4521 4524 4528 452c 4530 4534 4538
453c 4540 4544 4548 454c 4550 4554 4558
4559 455b 455f 4563 4567 456b 456c 456e
4572 4576 457a 457d 4580 4584 4588 458c
458f 4593 4597 459a 459d 45a2 45a5 45a8
45ab 45ac 45b1 45b2 1 45b4 45b9 45bd
45c1 45c4 45c9 45ce 45d1 45d5 45d6 45db
45de 45e3 45e4 45e9 45ec 45f1 45f2 45f7
45fa 45fe 4602 4606 460a 460d 460e 4610
4615 4616 4618 4619 461e 4621 4626 4627
462c 462f 4634 4635 463a 463d 4641 4642
4647 464a 464f 4650 4655 4656 465b 465d
4661 4664 4668 466c 466f 4673 4677 467a
467e 4682 4683 4688 468c 468e 4692 4696
469a 469d 469e 46a0 46a4 46a7 46ab 46ae
46af 46b1 46b2 46b7 46bb 46bf 46c3 46c6
46c7 46c9 46cc 46d0 46d4 46d7 46d8 46da
46db 46e0 46e4 46e8 46eb 46ef 46f3 46f5
46f9 46fd 4701 4704 4705 4707 470a 470d
470e 4713 4717 471a 471e 4722 4725 4729
472d 4731 4735 4739 473d 4749 474d 4751
4754 4758 475c 475e 4762 4766 476a 476d
4771 4775 4777 4778 477d 1 4781 4785
4789 478c 4790 4794 4798 479c 479f 47a3
47a7 47aa 47ad 47b2 47b5 47b8 47bb 47bc
47c1 47c2 1 47c4 47c9 47cd 47d1 47d4
47d9 47de 47e1 47e5 47e6 47eb 47ee 47f3
47f4 47f9 47fc 4801 4802 4807 480a 480e
480f 4814 4817 481c 481d 4822 4825 4829
482d 4830 4833 4834 4836 4837 483c 483d
4842 4844 4848 484b 484d 4851 4853 485f
4861 4863 4864 4869 486d 486f 487b 487d
4881 4883 4887 488b 488f 4892 4893 4895
4899 489c 48a0 48a3 48a4 48a6 48a7 48ac
48b0 48b4 48b8 48bb 48bc 48be 48c1 48c5
48c9 48cc 48cd 48cf 48d0 48d5 48d9 48dd
48de 48e2 48e6 48eb 48ef 48f3 48f7 48fa
48fb 48fd 4900 4903 4904 4909 490d 4912
4915 491a 491b 4920 4923 4927 4928 492d
4930 4935 4936 493b 493e 4942 4943 4948
494b 4950 4951 4956 4959 495e 495f 4964
4967 496b 496f 4973 4976 4977 4979 497a
497c 497d 4982 4986 498a 498e 4992 4995
4999 499b 499f 49a4 49a7 49ac 49ad 49b2
49b5 49b9 49ba 49bf 49c2 49c7 49c8 49cd
49d0 49d4 49d5 49da 49dd 49e2 49e3 49e8
49eb 49f0 49f1 49f6 49f9 49fd 4a01 4a05
4a08 4a09 4a0b 4a0c 4a0e 4a0f 4a14 4a18
4a1c 4a20 4a24 4a27 4a2b 4a2d 4a31 4a35
4a38 4a3c 4a40 4a44 4a47 4a48 4a4a 4a4d
4a50 4a51 4a56 4a5a 4a5e 4a63 4a68 4a6c
4a70 4a75 4a79 4a7d 4a80 4a83 4a87 4a89
4a8d 4a91 4a94 4a97 4a9b 4a9d 4aa1 4aa5
4aa8 4aaa 4aae 4ab2 4ab7 4abc 4ac0 4ac4
4ac9 4acd 4ad1 4ad4 4ad7 4adb 4add 4ae1
4ae5 4ae8 4aeb 4aef 4af1 4af5 4af9 4afc
4afe 4b02 4b06 4b09 4b0d 4b11 4b14 4b18
4b1b 4b1f 4b22 4b23 4b28 4b2b 4b2f 4b35
4b37 4b3b 4b3e 4b40 4b44 4b4b 4b4f 4b53
4b56 4b57 4b59 4b5c 4b5f 4b60 4b65 4b69
4b6d 4b72 4b74 4b78 4b7c 4b7f 4b83 4b87
4b89 4b8d 4b91 4b95 4b98 4b9c 4ba0 4ba2
4ba3 4ba8 1 4bac 4bb0 4bb4 4bb7 4bbb
4bbf 4bc3 4bc7 4bca 4bce 4bd2 4bd5 4bd8
4bdd 4be0 4be3 4be6 4be7 4bec 4bed 1
4bef 4bf4 4bf8 4bfc 4bff 4c04 4c09 4c0c
4c10 4c11 4c16 4c19 4c1e 4c1f 4c24 4c27
4c2c 4c2d 4c32 4c35 4c39 4c3a 4c3f 4c42
4c47 4c48 4c4d 4c50 4c54 4c58 4c5b 4c5e
4c5f 4c61 4c62 4c67 4c68 4c6d 4c6f 4c73
4c76 4c78 4c7c 4c7e 4c8a 4c8c 4c8e 4c8f
4c94 4c98 4c9a 4ca6 4ca8 4caa 4cae 4cb2
4cb7 4cb9 4cbd 4cc1 4cc4 4cc8 4ccc 4cce
4cd2 4cd6 4cda 4cdd 4ce1 4ce5 4ce7 4ce8
4ced 1 4cf1 4cf5 4cf9 4cfc 4d00 4d04
4d08 4d0c 4d0f 4d13 4d17 4d1a 4d1d 4d22
4d25 4d28 4d2b 4d2c 4d31 4d32 1 4d34
4d39 4d3d 4d41 4d44 4d49 4d4e 4d51 4d55
4d56 4d5b 4d5e 4d63 4d64 4d69 4d6c 4d71
4d72 4d77 4d7a 4d7e 4d7f 4d84 4d87 4d8c
4d8d 4d92 4d95 4d99 4d9d 4da0 4da3 4da4
4da6 4da7 4dac 4dad 4db2 4db4 4db8 4dbb
4dbd 4dc1 4dc3 4dcf 4dd1 4dd3 4dd4 4dd9
4ddd 4ddf 4deb 4ded 4def 4df3 4df7 4dfa
4dfe 4e01 4e02 4e07 4e0a 4e0e 4e12 4e15
4e16 4e18 4e1b 4e1e 4e1f 4e24 4e28 4e2c
4e2f 4e33 4e37 4e3a 4e3d 4e42 4e45 4e48
4e4b 4e4c 4e51 4e52 1 4e54 4e59 4e5d
4e61 4e64 4e69 4e6e 4e71 4e75 4e76 4e7b
4e7e 4e83 4e84 4e89 4e8c 4e91 4e92 4e97
4e9a 4e9e 4ea2 4ea6 4eaa 4ead 4eae 4eb0
4eb5 4eb6 4eb8 4eb9 4ebe 4ec1 4ec6 4ec7
4ecc 4ecf 4ed4 4ed5 4eda 4edd 4ee1 4ee2
4ee7 4eea 4eef 4ef0 4ef5 4ef6 4efb 4efd
4f01 4f04 4f08 4f0c 4f0f 4f13 4f17 4f1a
4f1d 4f22 4f25 4f28 4f2b 4f2c 4f31 4f32
1 4f34 4f39 4f3d 4f41 4f44 4f49 4f4e
4f51 4f55 4f56 4f5b 4f5c 4f61 4f63 4f67
4f6a 4f6e 4f72 4f75 4f79 4f7d 4f80 4f84
4f88 4f89 4f8e 4f90 4f94 4f98 4f9b 4f9f
4fa3 4fa6 4fa9 4fae 4fb1 4fb4 4fb7 4fb8
4fbd 4fbe 1 4fc0 4fc5 4fc9 4fcd 4fd0
4fd5 4fda 4fdd 4fe1 4fe2 4fe7 4fea 4fef
4ff0 4ff5 4ff8 4ffd 4ffe 5003 5006 500a
500e 5012 5016 5019 501a 501c 5021 5022
5024 5025 502a 502d 5032 5033 5038 503b
5040 5041 5046 5049 504d 504e 5053 5056
505b 505c 5061 5062 5067 5069 506d 5070
5074 5078 507b 507f 5083 5086 5089 508e
5091 5094 5097 5098 509d 509e 1 50a0
50a5 50a9 50ad 50b0 50b5 50ba 50bd 50c1
50c2 50c7 50c8 50cd 50cf 50d3 50d6 50da
50de 50e1 50e5 50e9 50ec 50f0 50f4 50f5
50fa 50fc 5100 5104 5107 5109 510d 5110
5114 5118 511c 5120 5124 5128 512c 5130
5134 5138 5144 5148 514c 514f 5153 5157
5159 515d 5161 5165 5168 516c 5170 5172
5173 5178 1 517c 5180 5184 5187 518b
518f 5193 5197 519a 519e 51a2 51a5 51a8
51ad 51b0 51b3 51b6 51b7 51bc 51bd 1
51bf 51c4 51c8 51cc 51cf 51d4 51d9 51dc
51e0 51e1 51e6 51e9 51ee 51ef 51f4 51f7
51fc 51fd 5202 5205 5209 520a 520f 5212
5217 5218 521d 5220 5224 5228 522b 522e
522f 5231 5232 5237 5238 523d 523f 5243
5246 5248 524c 524e 525a 525c 525e 525f
5264 5268 526a 5276 5278 527c 527f 5283
5287 528a 528e 5290 5294 5298 529b 529f
52a1 52a5 52a9 52ac 52ad 52af 52b3 52b6
52ba 52bd 52be 52c0 52c1 52c6 52ca 52ce
52d1 52d2 52d4 52d8 52db 52df 52e2 52e3
52e5 52e6 52eb 52ef 52f3 52f6 52fa 52fe
5301 5304 5309 530c 530f 5312 5313 5318
5319 1 531b 5320 5324 5328 532b 5330
5335 5338 533c 533d 5342 5345 534a 534b
5350 5353 5358 5359 535e 5361 5365 5369
536d 5371 5374 5375 5377 537c 537d 537f
5380 5385 5388 538d 538e 5393 5396 539b
539c 53a1 53a4 53a8 53a9 53ae 53b1 53b6
53b7 53bc 53bd 53c2 53c4 53c8 53cb 53cf
53d3 53d6 53da 53de 53e1 53e4 53e9 53ec
53ef 53f2 53f3 53f8 53f9 1 53fb 5400
5404 5408 540b 5410 5415 5418 541c 541d
5422 5423 5428 542a 542e 5431 5435 5439
543c 5440 5444 5447 544b 544f 5450 5455
5457 545b 545e 5462 5466 5469 546a 546c
546f 5472 5473 5478 547c 547f 5483 5485
5489 548d 5490 5491 5493 5496 5499 549a
549f 54a2 54a6 54aa 54ad 54ae 54b0 54b3
54b6 54b7 54bc 1 54bf 54c4 54c8 54cc
54cf 54d3 54d7 54da 54dd 54e2 54e5 54e8
54eb 54ec 54f1 54f2 1 54f4 54f9 54fd
5501 5504 5509 550e 5511 5515 5516 551b
551e 5523 5524 5529 552c 5531 5532 5537
553a 553e 5542 5546 554a 554d 554e 5550
5555 5556 5558 5559 555e 555f 5564 5566
556a 556d 5571 5575 5578 557c 5580 5583
5586 558b 558e 5591 5594 5595 559a 559b
1 559d 55a2 55a6 55aa 55ad 55b2 55b7
55ba 55be 55bf 55c4 55c5 55ca 55cc 55d0
55d3 55d7 55db 55de 55e2 55e6 55e9 55ed
55ee 55f3 55f5 55f9 55fc 5600 5604 5608
560a 560e 5612 5615 5619 561c 561d 5622
5625 5629 562d 5631 5635 5639 5645 5649
564d 5650 5654 5658 565a 565e 5662 5666
5669 566d 5671 5673 5674 5679 1 567d
5681 5685 5688 568c 5690 5694 5698 569b
569f 56a3 56a6 56a9 56ae 56b1 56b4 56b7
56b8 56bd 56be 1 56c0 56c5 56c9 56cd
56d0 56d5 56da 56dd 56e1 56e2 56e7 56ea
56ef 56f0 56f5 56f8 56fd 56fe 5703 5706
570a 570b 5710 5713 5718 5719 571e 5721
5725 5729 572c 572f 5730 5732 5733 5738
5739 573e 5740 5744 5747 5749 574d 574f
575b 575d 575f 5760 5765 5769 576b 5777
5779 577d 5781 5784 5788 578b 578f 5792
5793 5798 579b 579f 57a3 57a6 57a7 57a9
57ad 57b0 57b4 57b7 57b8 57ba 57bb 57c0
1 57c3 57c8 57cc 57d0 57d3 57d7 57db
57de 57e1 57e6 57e9 57ec 57ef 57f0 57f5
57f6 1 57f8 57fd 5801 5805 5808 580d
5812 5815 5819 581a 581f 5822 5827 5828
582d 5830 5835 5836 583b 583e 5842 5846
584a 584e 5851 5852 5854 5859 585a 585c
585d 5862 5865 586a 586b 5870 5873 5878
5879 587e 5881 5885 5886 588b 588e 5893
5894 5899 589a 589f 58a1 58a5 58a8 58ac
58b0 58b3 58b7 58bb 58be 58c1 58c6 58c9
58cc 58cf 58d0 58d5 58d6 1 58d8 58dd
58e1 58e5 58e8 58ed 58f2 58f5 58f9 58fa
58ff 5900 5905 5907 590b 590e 5912 5916
5919 591d 5921 5924 5928 592c 592d 5932
5934 5938 593b 593d 5941 5944 5948 594b
594f 5953 5956 595a 595e 5962 5965 5966
5968 596b 596e 596f 5974 5978 597b 597f
5981 5985 5988 598c 5990 5994 5998 599c
59a0 59a4 59a8 59ac 59b0 59b4 59b8 59bc
59c0 59c4 59c8 59cc 59d0 59dc 59e0 59e5
59e9 59ec 59ed 59f2 59f6 59fa 59fd 5a00
5a04 5a06 5a0a 5a0e 5a11 5a14 5a18 5a1a
5a1e 5a22 5a25 5a27 5a2b 5a2f 5a33 5a36
5a3a 5a3e 5a40 5a41 5a46 1 5a4a 5a4e
5a52 5a55 5a59 5a5d 5a61 5a65 5a68 5a6c
5a70 5a73 5a76 5a7b 5a7e 5a81 5a84 5a85
5a8a 5a8b 1 5a8d 5a92 5a96 5a9a 5a9d
5aa2 5aa7 5aaa 5aae 5aaf 5ab4 5ab7 5abc
5abd 5ac2 5ac5 5aca 5acb 5ad0 5ad3 5ad7
5ad8 5add 5ae0 5ae5 5ae6 5aeb 5aee 5af2
5af6 5af9 5afc 5afd 5aff 5b00 5b05 5b06
5b0b 5b0d 5b11 5b14 5b16 5b1a 5b1c 5b28
5b2a 5b2c 5b2d 5b32 5b36 5b38 5b44 5b46
5b4a 5b4e 5b51 5b52 5b54 5b57 5b5a 5b5b
5b60 5b64 5b67 5b6b 5b6d 5b71 5b74 5b76
5b7a 5b7e 5b81 5b85 5b88 5b8b 5b8c 5b91
5b95 5b98 5b9b 5b9c 5ba1 5ba5 5ba9 5bad
5bb1 5bb5 5bb9 5bbd 5bc1 5bc5 5bc9 5bcd
5bd1 5bd5 5bd6 5bd8 5bdc 5be0 5be4 5be7
5be8 5bea 5bed 5bf0 5bf1 5bf6 5bf9 5bfd
5c00 5c01 5c06 5c09 1 5c0c 5c11 5c15
5c19 5c1d 5c21 5c25 5c29 5c2d 5c31 5c35
5c38 5c39 5c3b 5c3f 5c41 5c45 5c48 5c4c
5c4f 5c50 5c55 5c58 5c5c 5c60 5c64 5c68
5c6b 5c6c 5c6e 5c72 5c74 5c78 5c7c 5c7f
5c82 5c83 5c85 5c88 5c8d 5c8e 5c93 5c97
5c9b 5c9f 5ca3 5ca6 5ca7 5ca9 5cad 5caf
5cb3 5cb7 5cbb 5cbf 5cc2 5cc3 5cc5 5cc9
5ccb 5ccf 5cd3 5cd6 5cd8 5cdc 5ce0 5ce3
5ce7 5cea 5ceb 5cf0 5cf3 5cf7 5cfb 5cfe
5cff 5d01 5d04 5d07 5d08 5d0d 5d11 5d15
5d19 5d1c 5d20 5d24 5d27 5d28 5d2a 5d2e
5d32 5d35 5d36 5d38 5d3b 5d3f 5d43 5d47
5d4b 5d4f 5d53 5d57 5d5b 5d5f 5d63 5d67
5d73 5d77 5d7b 5d7e 5d82 5d86 5d88 5d8c
5d90 5d94 5d97 5d9b 5d9f 5da1 5da2 5da7
1 5dab 5daf 5db3 5db6 5dba 5dbe 5dc2
5dc6 5dc9 5dcd 5dd1 5dd4 5dd7 5ddc 5ddf
5de2 5de5 5de6 5deb 5dec 1 5dee 5df3
5df7 5dfb 5dfe 5e03 5e08 5e0b 5e0f 5e10
5e15 5e18 5e1d 5e1e 5e23 5e26 5e2b 5e2c
5e31 5e34 5e38 5e39 5e3e 5e41 5e46 5e47
5e4c 5e4f 5e53 5e57 5e5a 5e5d 5e5e 5e60
5e61 5e66 5e67 5e6c 5e6e 5e72 5e75 5e77
5e7b 5e7d 5e89 5e8b 5e8d 5e8e 5e93 5e97
5e99 5ea5 5ea7 5ea9 5ead 5eb1 5eb5 5eb9
5ebd 5ec1 5ec5 5ec9 5ecd 5ed1 5edd 5ee1
5ee5 5ee8 5eec 5ef0 5ef2 5ef6 5efa 5efe
5f01 5f05 5f09 5f0b 5f0c 5f11 1 5f15
5f19 5f1d 5f20 5f24 5f28 5f2c 5f30 5f33
5f37 5f3b 5f3e 5f41 5f46 5f49 5f4c 5f4f
5f50 5f55 5f56 1 5f58 5f5d 5f61 5f65
5f68 5f6d 5f72 5f75 5f79 5f7a 5f7f 5f82
5f87 5f88 5f8d 5f90 5f95 5f96 5f9b 5f9e
5fa2 5fa3 5fa8 5fab 5fb0 5fb1 5fb6 5fb9
5fbd 5fc1 5fc4 5fc7 5fc8 5fca 5fcb 5fd0
5fd1 5fd6 5fd8 5fdc 5fdf 5fe1 5fe5 5fe7
5ff3 5ff5 5ff7 5ff8 5ffd 6001 6003 600f
6011 6013 6017 601b 601e 6022 6026 602a
602d 602e 6030 6033 6037 603b 603e 603f
6041 6042 6047 604b 604d 6051 6054 6058
605b 605f 6063 6067 606b 606f 6073 6077
607b 607f 6083 6087 608a 608b 608d 6091
6095 6098 6099 609b 609c 609e 60a2 60a6
60aa 60ad 60ae 60b0 60b3 60b6 60b7 60bc
60c0 60c4 60c7 60cb 60cf 60d2 60d5 60da
60dd 60e0 60e3 60e4 60e9 60ea 1 60ec
60f1 60f5 60f9 60fc 6101 6106 6109 610d
610e 6113 6116 611b 611c 6121 6124 6129
612a 612f 6132 6136 613a 613e 6142 6145
6146 6148 614d 614e 6150 6151 6156 6159
615e 615f 6164 6167 616c 616d 6172 6175
6179 617a 617f 6182 6187 6188 618d 618e
6193 6195 6199 619c 61a0 61a4 61a7 61ab
61af 61b2 61b5 61ba 61bd 61c0 61c3 61c4
61c9 61ca 1 61cc 61d1 61d5 61d9 61dc
61e1 61e6 61e9 61ed 61ee 61f3 61f4 61f9
61fb 61ff 6202 6206 620a 620d 6211 6215
6218 621c 6220 6221 6226 622a 622c 6230
6233 6236 6237 623c 6240 6244 6248 624c
6258 625c 6260 6264 6268 626c 6270 6274
6278 627c 6280 6284 6288 628c 6290 6294
6298 629c 62a0 62a4 62a8 62ac 62b0 62bc
62c0 62c5 62c9 62cc 62cd 62d2 62d6 62da
62dd 62e0 62e4 62e6 62ea 62ee 62f1 62f4
62f8 62fa 62fe 6302 6305 6307 630b 630f
6313 6316 631a 631e 6320 6321 6326 1
632a 632e 6332 6335 6339 633d 6341 6345
6348 634c 6350 6353 6356 635b 635e 6361
6364 6365 636a 636b 1 636d 6372 6376
637a 637d 6382 6387 638a 638e 638f 6394
6397 639c 639d 63a2 63a5 63aa 63ab 63b0
63b3 63b7 63b8 63bd 63c0 63c5 63c6 63cb
63ce 63d2 63d6 63d9 63dc 63dd 63df 63e0
63e5 63e6 63eb 63ed 63f1 63f4 63f6 63fa
63fc 6408 640a 640c 640d 6412 6416 6418
6424 6426 6428 642c 6430 6433 6437 643b
643f 6440 6442 6446 644a 644e 6451 6455
6459 645c 645f 6464 6467 646a 646d 646e
6473 6474 1 6476 647b 647f 6483 6486
648b 6490 6493 6497 6498 649d 64a0 64a5
64a6 64ab 64ae 64b3 64b4 64b9 64bc 64c0
64c4 64c8 64c9 64cb 64d0 64d1 64d3 64d4
64d9 64dc 64e1 64e2 64e7 64ea 64ef 64f0
64f5 64f8 64fc 64fd 6502 6505 650a 650b
6510 6511 6516 6518 651c 651f 6523 6527
652a 652e 6532 6535 6538 653d 6540 6543
6546 6547 654c 654d 1 654f 6554 6558
655c 655f 6564 6569 656c 6570 6571 6576
6577 657c 657e 6582 6585 6589 658d 6590
6594 6598 659b 659c 659e 65a2 65a6 65a7
65ac 65ae 65b2 65b5 65b9 65bd 65c0 65c1
65c3 65c7 65ca 65ce 65d1 65d2 65d4 65d5
65da 65de 65e2 65e6 65ea 65ee 65f2 65f6
65fa 65fe 6602 6606 660a 6616 661a 661e
6621 6625 6629 662b 662f 6633 6637 663a
663e 6642 6644 6645 664a 1 664e 6652
6656 6659 665d 6661 6665 6669 666c 6670
6674 6677 667a 667f 6682 6685 6688 6689
668e 668f 1 6691 6696 669a 669e 66a1
66a6 66ab 66ae 66b2 66b3 66b8 66bb 66c0
66c1 66c6 66c9 66ce 66cf 66d4 66d7 66db
66dc 66e1 66e4 66e9 66ea 66ef 66f2 66f6
66fa 66fd 6700 6701 6703 6704 6709 670a
670f 6711 6715 6718 671a 671e 6720 672c
672e 6730 6731 6736 673a 673c 6748 674a
674e 6752 6755 6756 6758 675b 675e 675f
6764 6768 676c 676f 6773 6777 677a 677d
6782 6785 6788 678b 678c 6791 6792 1
6794 6799 679d 67a1 67a4 67a9 67ae 67b1
67b5 67b6 67bb 67be 67c3 67c4 67c9 67cc
67d1 67d2 67d7 67da 67de 67e2 67e6 67ea
67ed 67ee 67f0 67f5 67f6 67f8 67f9 67fe
6801 6806 6807 680c 680f 6814 6815 681a
681d 6821 6822 6827 682a 682f 6830 6835
6836 683b 683d 6841 6844 6848 684c 684f
6853 6857 685a 685d 6862 6865 6868 686b
686c 6871 6872 1 6874 6879 687d 6881
6884 6889 688e 6891 6895 6896 689b 689c
68a1 68a3 68a7 68aa 68ae 68b2 68b5 68b9
68bd 68c0 68c4 68c8 68c9 68ce 68d0 68d4
68d7 68db 68df 68e3 68e7 68eb 68ef 68f2
68f3 68f5 68f9 68fd 6901 6904 6905 6907
690b 690e 6912 6915 6916 6918 6919 691e
6922 6925 6928 6929 692e 6932 6936 693a
693e 6942 6946 694a 694e 6952 6956 695a
695e 6962 6963 6965 6969 696d 6970 6971
6976 6979 697d 6981 6985 6989 698c 698d
698f 6993 6995 6999 699d 69a0 69a3 69a4
69a6 69a9 69ae 69af 69b4 69b8 69bc 69c0
69c4 69c7 69c8 69ca 69ce 69d0 69d4 69d8
69dc 69e0 69e3 69e4 69e6 69ea 69ec 69f0
69f4 69f7 69f9 69fd 6a01 6a04 6a08 6a0b
6a0f 6a13 6a16 6a17 6a1c 6a1f 6a23 6a27
6a2a 6a2b 6a2d 6a30 6a33 6a34 6a39 6a3d
6a41 6a45 6a48 6a4c 6a50 6a53 6a54 6a56
6a5a 6a5e 6a61 6a62 6a64 6a67 6a6b 6a6f
6a73 6a77 6a7b 6a7f 6a83 6a87 6a8b 6a8f
6a93 6a9f 6aa3 6aa7 6aaa 6aae 6ab2 6ab4
6ab8 6abc 6ac0 6ac3 6ac7 6acb 6acd 6ace
6ad3 1 6ad7 6adb 6adf 6ae2 6ae6 6aea
6aee 6af2 6af5 6af9 6afd 6b00 6b03 6b08
6b0b 6b0e 6b11 6b12 6b17 6b18 1 6b1a
6b1f 6b23 6b27 6b2a 6b2f 6b34 6b37 6b3b
6b3c 6b41 6b44 6b49 6b4a 6b4f 6b52 6b57
6b58 6b5d 6b60 6b64 6b65 6b6a 6b6d 6b72
6b73 6b78 6b7b 6b7f 6b83 6b86 6b89 6b8a
6b8c 6b8d 6b92 6b93 6b98 6b9a 6b9e 6ba1
6ba3 6ba7 6ba9 6bb5 6bb7 6bb9 6bba 6bbf
6bc3 6bc5 6bd1 6bd3 6bd5 6bd9 6bdd 6be1
6be5 6be9 6bed 6bf1 6bf5 6bf9 6bfd 6c09
6c0d 6c11 6c14 6c18 6c1c 6c1e 6c22 6c26
6c2a 6c2d 6c31 6c35 6c37 6c38 6c3d 1
6c41 6c45 6c49 6c4c 6c50 6c54 6c58 6c5c
6c5f 6c63 6c67 6c6a 6c6d 6c72 6c75 6c78
6c7b 6c7c 6c81 6c82 1 6c84 6c89 6c8d
6c91 6c94 6c99 6c9e 6ca1 6ca5 6ca6 6cab
6cae 6cb3 6cb4 6cb9 6cbc 6cc1 6cc2 6cc7
6cca 6cce 6ccf 6cd4 6cd7 6cdc 6cdd 6ce2
6ce5 6ce9 6ced 6cf0 6cf3 6cf4 6cf6 6cf7
6cfc 6cfd 6d02 6d04 6d08 6d0b 6d0d 6d11
6d13 6d1f 6d21 6d23 6d24 6d29 6d2d 6d2f
6d3b 6d3d 6d3f 6d43 6d47 6d4a 6d4e 6d52
6d55 6d56 6d58 6d5b 6d5e 6d5f 6d64 6d68
6d6c 6d70 6d74 6d78 6d7c 6d80 6d84 6d88
6d8c 6d90 6d94 6da0 6da4 6da8 6dab 6daf
6db3 6db5 6db9 6dbd 6dc1 6dc4 6dc8 6dcc
6dce 6dcf 6dd4 1 6dd8 6ddc 6de0 6de3
6de7 6deb 6def 6df3 6df6 6dfa 6dfe 6e01
6e04 6e09 6e0c 6e0f 6e12 6e13 6e18 6e19
1 6e1b 6e20 6e24 6e28 6e2b 6e30 6e35
6e38 6e3c 6e3d 6e42 6e45 6e4a 6e4b 6e50
6e53 6e58 6e59 6e5e 6e61 6e65 6e66 6e6b
6e6e 6e73 6e74 6e79 6e7c 6e80 6e84 6e87
6e8a 6e8b 6e8d 6e8e 6e93 6e94 6e99 6e9b
6e9f 6ea2 6ea4 6ea8 6eaa 6eb6 6eb8 6eba
6ebb 6ec0 6ec4 6ec6 6ed2 6ed4 6ed6 6eda
6edd 6edf 6ee3 6ee6 6eea 6eee 6ef2 6ef5
6ef6 6ef8 6efb 6eff 6f03 6f06 6f07 6f09
6f0a 6f0f 6f12 6f16 6f1a 6f1d 6f1e 6f20
6f21 6f26 6f2a 6f2e 6f31 6f35 6f39 6f3d
6f41 6f45 6f49 6f4d 6f51 6f55 6f59 6f5d
6f61 6f65 6f69 6f6d 6f71 6f75 6f79 6f7d
6f81 6f8d 6f91 6f96 6f9a 6f9d 6f9e 6fa3
6fa7 6fab 6fae 6fb1 6fb5 6fb7 6fbb 6fbf
6fc2 6fc5 6fc9 6fcb 6fcf 6fd3 6fd6 6fd8
6fdc 6fe0 6fe4 6fe7 6feb 6fef 6ff1 6ff2
6ff7 1 6ffb 6fff 7003 7006 700a 700e
7012 7016 7019 701d 7021 7024 7027 702c
702f 7032 7035 7036 703b 703c 1 703e
7043 7047 704b 704e 7053 7058 705b 705f
7060 7065 7068 706d 706e 7073 7076 707b
707c 7081 7084 7088 7089 708e 7091 7096
7097 709c 709f 70a3 70a7 70aa 70ad 70ae
70b0 70b1 70b6 70b7 70bc 70be 70c2 70c5
70c7 70cb 70cd 70d9 70db 70dd 70de 70e3
70e7 70e9 70f5 70f7 70fb 70ff 7102 7106
7109 710d 7110 7111 7116 7119 711d 7120
7124 7128 712c 7130 7134 7138 713c 7140
7144 7147 714b 714f 7152 7153 7155 7156
7158 715c 7160 7164 7167 7168 716a 716d
7170 7171 7176 717a 717e 7181 7185 7189
718c 718f 7194 7197 719a 719d 719e 71a3
71a4 1 71a6 71ab 71af 71b3 71b6 71bb
71c0 71c3 71c7 71c8 71cd 71d0 71d5 71d6
71db 71de 71e3 71e4 71e9 71ec 71f0 71f4
71f8 71f9 71fb 7200 7201 7203 7204 7209
720c 7211 7212 7217 721a 721f 7220 7225
7228 722c 722d 7232 7235 723a 723b 7240
7241 7246 7248 724c 724f 7253 7257 725a
725e 7262 7265 7268 726d 7270 7273 7276
7277 727c 727d 1 727f 7284 7288 728c
728f 7294 7299 729c 72a0 72a1 72a6 72a7
72ac 72ae 72b2 72b5 72b9 72bd 72c0 72c4
72c8 72cb 72cc 72ce 72d2 72d6 72d7 72dc
72de 72e2 72e5 72e7 72eb 72ee 72f2 72f6
72fa 72fd 72fe 7300 7304 7306 730a 730e
7312 7316 731a 731b 731d 7320 7321 7323
7326 732a 732e 7332 7333 7335 7338 7339
733b 733c 7341 7344 7347 734a 734b 7350
7353 7356 7359 735a 735f 7360 7362 7366
7368 736c 7370 7373 7377 737b 737f 7383
7387 738b 738f 7393 7397 739b 739f 73a3
73a7 73ab 73af 73b3 73b7 73bb 73bf 73c3
73c7 73cb 73cf 73d3 73d7 73db 73df 73e2
73e3 73e5 73e6 73e8 73ec 73f0 73f4 73f7
73fb 73ff 7402 7405 740a 740d 7410 7413
7414 7419 741a 1 741c 7421 7425 7429
742c 7431 7436 7439 743d 743e 7443 7446
744b 744c 7451 7454 7459 745a 745f 7462
7466 746a 746e 746f 7471 7476 7477 7479
747a 747f 7482 7487 7488 748d 7490 7495
7496 749b 749e 74a2 74a3 74a8 74ab 74b0
74b1 74b6 74b7 74bc 74be 74c2 74c5 74c9
74cd 74d0 74d4 74d8 74db 74de 74e3 74e6
74e9 74ec 74ed 74f2 74f3 1 74f5 74fa
74fe 7502 7505 750a 750f 7512 7516 7517
751c 751d 7522 7524 7528 752b 752f 7533
7536 753a 753e 7541 7542 7544 7548 754c
754d 7552 7556 7558 755c 7560 7563 7564
7566 756a 756d 7571 7574 7575 7577 7578
757d 7581 7585 7588 7589 758b 758e 7591
7592 7597 759a 759e 75a2 75a5 75a6 75a8
75ab 75ae 75af 75b4 1 75b7 75bc 75c0
75c4 75c7 75cb 75cf 75d2 75d5 75da 75dd
75e0 75e3 75e4 75e9 75ea 1 75ec 75f1
75f5 75f9 75fc 7601 7606 7609 760d 760e
7613 7616 761b 761c 7621 7624 7629 762a
762f 7632 7636 763a 763e 7642 7645 7646
7648 764d 764e 7650 7651 7656 7659 765e
765f 7664 7667 766c 766d 7672 7675 7679
767a 767f 7682 7687 7688 768d 768e 7693
7695 7699 769c 76a0 76a4 76a7 76ab 76af
76b2 76b5 76ba 76bd 76c0 76c3 76c4 76c9
76ca 1 76cc 76d1 76d5 76d9 76dc 76e1
76e6 76e9 76ed 76ee 76f3 76f4 76f9 76fb
76ff 7702 7706 770a 770d 7711 7715 7718
771c 7720 7721 7726 7728 772c 772f 7733
7737 773b 773f 7743 7747 774b 774f 7753
7757 775b 7767 776b 7770 7774 7777 7778
777d 7781 7785 7788 778b 778f 7791 7795
7799 779c 779f 77a3 77a5 77a9 77ad 77b0
77b2 77b6 77ba 77be 77c1 77c5 77c9 77cb
77cc 77d1 1 77d5 77d9 77dd 77e0 77e4
77e8 77ec 77f0 77f3 77f7 77fb 77fe 7801
7806 7809 780c 780f 7810 7815 7816 1
7818 781d 7821 7825 7828 782d 7832 7835
7839 783a 783f 7842 7847 7848 784d 7850
7855 7856 785b 785e 7862 7863 7868 786b
7870 7871 7876 7879 787d 7881 7884 7887
7888 788a 788b 7890 7891 7896 7898 789c
789f 78a1 78a5 78a7 78b3 78b5 78b7 78b8
78bd 78c1 78c3 78cf 78d1 78d5 78d9 78dd
78e1 78e5 78e9 78ed 78f1 78f5 78f9 78fd
7909 790d 7912 7916 7919 791a 791f 7923
7927 792a 792d 7931 7933 7937 793b 793e
7941 7945 7947 794b 794f 7952 7954 7958
795c 7960 7963 7967 796b 796d 796e 7973
1 7977 797b 797f 7982 7986 798a 798e
7992 7995 7999 799d 79a0 79a3 79a8 79ab
79ae 79b1 79b2 79b7 79b8 1 79ba 79bf
79c3 79c7 79ca 79cf 79d4 79d7 79db 79dc
79e1 79e4 79e9 79ea 79ef 79f2 79f7 79f8
79fd 7a00 7a04 7a05 7a0a 7a0d 7a12 7a13
7a18 7a1b 7a1f 7a23 7a26 7a29 7a2a 7a2c
7a2d 7a32 7a33 7a38 7a3a 7a3e 7a41 7a43
7a47 7a49 7a55 7a57 7a59 7a5a 7a5f 7a63
7a65 7a71 7a73 7a75 7a79 7a7d 7a80 7a84
7a88 7a8c 7a8d 7a8f 7a93 7a97 7a9b 7a9e
7aa2 7aa6 7aa9 7aac 7ab1 7ab4 7ab7 7aba
7abb 7ac0 7ac1 1 7ac3 7ac8 7acc 7ad0
7ad3 7ad8 7add 7ae0 7ae4 7ae5 7aea 7aed
7af2 7af3 7af8 7afb 7b00 7b01 7b06 7b09
7b0d 7b11 7b15 7b16 7b18 7b1d 7b1e 7b20
7b21 7b26 7b29 7b2e 7b2f 7b34 7b37 7b3c
7b3d 7b42 7b45 7b49 7b4a 7b4f 7b52 7b57
7b58 7b5d 7b5e 7b63 7b65 7b69 7b6c 7b70
7b74 7b77 7b7b 7b7f 7b82 7b85 7b8a 7b8d
7b90 7b93 7b94 7b99 7b9a 1 7b9c 7ba1
7ba5 7ba9 7bac 7bb1 7bb6 7bb9 7bbd 7bbe
7bc3 7bc4 7bc9 7bcb 7bcf 7bd2 7bd6 7bda
7bdd 7be1 7be5 7be8 7be9 7beb 7bef 7bf3
7bf4 7bf9 7bfb 7bff 7c03 7c07 7c0b 7c0f
7c13 7c17 7c1b 7c1f 7c23 7c27 7c2b 7c2f
7c33 7c37 7c3b 7c3f 7c43 7c47 7c4b 7c4f
7c53 7c57 7c5b 7c5f 7c63 7c67 7c6b 7c6f
7c73 7c77 7c7b 7c7f 7c8b 7c8f 7c93 7c96
7c9a 7c9e 7ca0 7ca4 7ca8 7cac 7caf 7cb3
7cb7 7cb9 7cba 7cbf 1 7cc3 7cc7 7ccb
7cce 7cd2 7cd6 7cda 7cde 7ce1 7ce5 7ce9
7cec 7cef 7cf4 7cf7 7cfa 7cfd 7cfe 7d03
7d04 1 7d06 7d0b 7d0f 7d13 7d16 7d1b
7d20 7d23 7d27 7d28 7d2d 7d30 7d35 7d36
7d3b 7d3e 7d43 7d44 7d49 7d4c 7d50 7d51
7d56 7d59 7d5e 7d5f 7d64 7d67 7d6b 7d6f
7d72 7d75 7d76 7d78 7d79 7d7e 7d7f 7d84
7d86 7d8a 7d8d 7d8f 7d93 7d95 7da1 7da3
7da5 7da6 7dab 7daf 7db1 7dbd 7dbf 7dc3
7dc7 7dca 7dce 7dd1 7dd5 7dd8 7dd9 7dde
7de1 7de4 7de5 7dea 7dee 7df2 7df5 7df9
7dfd 7e00 7e03 7e08 7e0b 7e0e 7e11 7e12
7e17 7e18 1 7e1a 7e1f 7e23 7e27 7e2a
7e2f 7e34 7e37 7e3b 7e3c 7e41 7e44 7e49
7e4a 7e4f 7e52 7e57 7e58 7e5d 7e60 7e64
7e68 7e6c 7e70 7e73 7e74 7e76 7e7b 7e7c
7e7e 7e7f 7e84 7e87 7e8c 7e8d 7e92 7e95
7e9a 7e9b 7ea0 7ea3 7ea7 7ea8 7ead 7eb0
7eb5 7eb6 7ebb 7ebc 7ec1 7ec3 7ec7 7eca
7ece 7ed2 7ed5 7ed9 7edd 7ee0 7ee3 7ee8
7eeb 7eee 7ef1 7ef2 7ef7 7ef8 1 7efa
7eff 7f03 7f07 7f0a 7f0f 7f14 7f17 7f1b
7f1c 7f21 7f22 7f27 7f29 7f2d 7f30 7f34
7f38 7f3b 7f3f 7f43 7f46 7f4a 7f4e 7f4f
7f54 7f56 7f5a 7f5d 7f61 7f65 7f68 7f6c
7f70 7f71 7f73 7f76 7f79 7f7a 7f7f 7f83
7f87 7f8a 7f8d 7f91 7f95 7f99 7f9c 7fa0
7fa4 7fa7 7faa 7faf 7fb2 7fb5 7fb8 7fb9
7fbe 7fbf 1 7fc1 7fc6 7fca 7fce 7fd1
7fd6 7fdb 7fde 7fe2 7fe3 7fe8 7feb 7ff0
7ff1 7ff6 7ff9 7ffe 7fff 8004 8007 800b
800f 8013 8017 801a 801b 801d 8022 8023
8025 8026 802b 802e 8033 8034 8039 803c
8041 8042 8047 804a 804e 804f 8054 8057
805c 805d 8062 8063 8068 806a 806e 8071
8075 8079 807c 8080 8084 8087 808a 808f
8092 8095 8098 8099 809e 809f 1 80a1
80a6 80aa 80ae 80b1 80b6 80bb 80be 80c2
80c3 80c8 80c9 80ce 80d0 80d4 80d7 80db
80df 80e2 80e6 80ea 80ed 80f1 80f5 80f6
80fb 80fd 8101 8104 8106 810a 810e 8111
8113 8117 811a 811e 8122 8126 812a 812e
8132 8136 813a 813e 8142 8146 814a 814d
8151 8155 8159 815c 8160 8164 8168 816c
8170 8171 8173 8177 817b 817f 8182 8186
818a 818d 8190 8195 8198 819b 819e 819f
81a4 81a5 1 81a7 81ac 81b0 81b4 81b7
81bc 81c1 81c4 81c8 81c9 81ce 81d1 81d6
81d7 81dc 81df 81e4 81e5 81ea 81ed 81f1
81f5 81f9 81fd 8200 8201 8203 8208 8209
820b 820c 8211 8214 8219 821a 821f 8222
8227 8228 822d 8230 8234 8235 823a 823d
8242 8243 8248 8249 824e 8250 8254 8257
825b 825f 8262 8266 826a 826d 8271 8275
8276 827b 827d 8281 8285 8288 828c 8290
8293 8297 829b 829e 82a1 82a6 82a9 82ac
82af 82b0 82b5 82b6 1 82b8 82bd 82c1
82c5 82c8 82cd 82d2 82d5 82d9 82da 82df
82e0 82e5 82e9 82ed 82f0 82f5 82fa 82fd
8301 8305 8308 8309 830e 8311 8316 8317
831c 831f 8323 8327 832a 832b 8330 8331
8336 8338 833c 833f 8343 8347 834b 834f
8353 8357 835b 835f 8363 836f 8374 8378
837c 8380 8384 8386 8387 838c 1 8390
8394 8398 839b 839f 83a3 83a7 83ab 83ae
83b2 83b6 83b9 83bc 83c1 83c4 83c7 83ca
83cb 83d0 83d1 1 83d3 83d8 83dc 83e0
83e3 83e8 83ed 83f0 83f4 83f5 83fa 83fb
8400 8404 8408 840b 8410 8415 8418 841c
841d 8422 8425 842a 842b 8430 8433 8437
843b 843e 8441 8442 8444 8445 844a 844b
8450 8452 8456 8459 845d 8461 8465 8469
846d 8471 8475 8481 8486 848a 848e 8492
8496 8498 849c 849e 84aa 84ac 84ae 84af
84b4 84b8 84ba 84c6 84ca 84cc 84cd 84d6 
2628
2
0 1 a 6 10 :3 6 15 :3 6
16 :3 6 17 :3 6 e :3 6 e :3 6
15 :3 6 f :3 6 10 :2 6 1 3
a 13 a :2 1 4 12 1b 1a
:2 12 :2 4 11 1a 19 :2 11 :2 4 15
1e 1d :2 15 :2 4 17 20 1f :2 17
:2 4 14 1d 1c :2 14 :2 4 10 19
18 :2 10 :2 4 18 21 20 :2 18 :2 4
12 1b 1a :2 12 :2 4 e 17 16
:2 e :2 4 14 1d 1c :2 14 :2 4 11
1a 19 :2 11 :2 4 11 1a 19 :2 11
:2 4 16 1f 1e :2 16 :2 4 :3 13 :2 4
:3 11 :2 4 :3 12 :2 4 :3 12 :2 4 :3 13 :2 4
:3 f :2 4 :3 9 :2 4 :3 9 :2 4 :3 e :2 4
f 18 17 :2 f :2 4 f 18 17
:2 f :2 4 :2 f 1e f :2 4 :2 16 25
16 :2 4 :2 11 20 11 :2 4 :3 10 :2 4
:3 15 :2 4 :2 17 26 17 :2 4 :2 13 22
13 :2 4 :2 12 21 12 :2 4 :2 12 21
12 :2 4 :2 14 23 14 :2 4 :2 15 24
15 :2 4 :2 12 21 12 :2 4 :2 d 1c
d :2 4 :2 11 20 11 :2 4 :2 13 22
13 :2 4 e 17 16 :2 e :2 4 d
16 15 :2 d :2 4 f 18 17 :2 f
:2 4 :2 d 1c d :2 4 :2 e 1d e
:2 4 :2 c 1b c :2 4 :2 14 23 14
:2 4 :2 13 22 13 :2 4 :2 12 21 12
:2 4 :2 11 20 11 :2 4 :2 b 1a b
:2 4 :2 f 1e f :2 4 :2 e 1d e
:2 4 :2 10 1f 10 :2 4 :2 e 1d e
:2 4 :2 10 1f 10 :2 4 :2 13 22 13
:2 4 :2 f 1e f :2 4 :2 e 1d e
:2 4 :2 f 1e f :2 4 :2 c 1b c
:2 4 :2 b 1a b :2 4 :2 13 22 13
:2 4 :2 a 19 a :2 4 :2 12 21 12
:2 4 :2 11 20 11 :2 4 :2 e 1d e
:2 4 14 1d 1c :2 14 :2 4 14 1d
1c :2 14 :2 4 :2 17 26 17 :2 4 17
20 1f :2 17 :2 4 :2 13 22 13 :2 4
:2 14 23 14 :2 4 :2 f 1e f :2 4
:2 f 19 f :2 4 :2 f 19 f :2 4
:2 10 1a 10 :2 4 :2 10 1a 10 :2 4
:2 11 1b 11 :2 4 :2 11 1b 11 :2 4
:2 12 1c 12 :2 4 :2 12 1c 12 :2 4
9 12 11 :2 9 :2 4 :3 e :2 4 :3 e
4 7 11 1a :3 11 :2 7 11 1a
:3 11 7 4 e 17 :3 e 4 7
:2 11 23 :2 2d 3a :2 23 :3 7 :2 d 19
25 :2 2f 43 :2 25 58 5e 61 :2 58
6f 89 8 :2 6f d 10 :2 6f 1a
1d :2 6f b e :2 6f 13 16 :2 6f
1b 1e :2 6f 2d 30 :2 6f b e
:2 6f 13 16 1f 26 2b 2e :2 26
3e 41 :2 26 46 :2 1f 4f :2 16 :2 6f
b e :2 6f 13 16 1f 26 2b
2e :2 26 3f 42 :2 26 47 :2 1f 50
:2 16 :2 6f b e :2 6f 13 16 1f
27 :2 1f 31 :2 16 :2 6f b e :2 6f
13 16 1f 27 :2 1f 31 :2 16 :2 6f
b e :2 6f 13 16 1f 27 :2 1f
38 :2 16 :2 6f b e :2 6f 13 16
1f 27 :2 1f 32 :2 16 :2 6f b e
:2 6f 13 16 :2 6f 1b 1e :2 6f 28
2b :2 6f 30 33 :2 6f :2 7 4e :2 4
7 :2 11 23 :2 2d 39 3c 50 53
54 :2 50 :2 23 :3 7 :2 d 12 1e 62
65 :2 1e :2 7 57 :3 4 13 :2 4 14
:2 4 12 :2 4 15 :2 4 11 4 e
11 24 27 34 37 43 46 11
14 11 14 7 a 7 a 16
19 c 21 30 3e :4 7 15 c
21 d 10 24 :2 7 :2 10 1c 7
4 c a :2 13 1f a 1a :2 7
:2 c a :2 13 1f a d :2 17 29
:2 33 3f 43 57 5a 5b :2 57 :2 29
:3 d :2 13 18 24 68 6b :2 24 78
7b :2 24 7f 82 :2 24 8e 91 :2 24
99 9c :2 24 a8 ab b2 bb be
:2 ab :2 24 :2 d 5e :2 a :2 7 :4 13 :2 7
4 :3 1 8 :2 11 1c 1a :2 25 :2 1a
7 11 24 3b 47 a 19 23
33 3e 48 50 58 14 23 32
44 53 61 6e 80 14 28 3a
4a 5b a 12 19 34 3c 43
5f 69 a 18 26 34 a 1a
2a 3a a 1d 26 f 24 35
43 a 16 22 34 3f 47 50
58 a 1b 2b 3a 48 a 14
1d a 10 16 22 a 16 23
2f a 18 27 35 a 1a 25
f 10 24 :2 a :2 13 1f a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 5a 5d 5e :2 5a :2 2c :3 10
:2 16 1b 27 6b 6e :2 27 7b 7e
:2 27 82 85 :2 27 91 94 :2 27 9c
9f :2 27 ab ae b5 be c1 :2 ae
:2 27 :2 10 61 :2 d :2 a :4 16 :2 a 7
:3 35 b :2 14 1f 1d :2 28 :2 1d a
14 20 2b 62 6c 77 d 1b
1f 2c 3d 4b 59 5e 67 76
12 20 2d 38 46 55 d 1d
2a 39 43 53 5a 12 20 2d
:2 d :2 16 22 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 5d
60 61 :2 5d :2 2f :3 13 :2 19 1e 2a
6e 71 :2 2a 7e 81 :2 2a 85 88
:2 2a 94 97 :2 2a 9f a2 :2 2a ae
b1 b8 c1 c4 :2 b1 :2 2a :2 13 64
:2 10 :2 d :4 19 :2 d a :3 38 e :2 17
22 20 :2 2b :2 20 d 10 :2 1a 2c
:2 36 42 46 5a 5d 5e :2 5a :2 2c
:3 10 :2 16 1b 27 6b 6e :2 27 7b
7e :2 27 82 85 :2 27 8f 92 9b
a3 :2 ac :2 9b bf :2 92 :2 27 c7 ca
:2 27 d2 d5 :2 27 da dd :2 27 e7
ea :2 27 :2 10 61 :2 d 10 :2 1a 2c
:2 36 42 45 59 5c 5d :2 59 :2 2c
:3 10 :2 16 1b 27 6b 6e :2 27 :2 10
60 :3 d :2 18 2c :2 35 47 52 :2 d
3b :3 a 1b :2 a 1c a e :2 19
21 30 40 :2 e 44 46 :2 44 d
4e :2 59 10 20 2f :2 4e 33 36
:2 33 4d :2 d 10 :2 1a 2c :2 36 42
46 5a 5d 5e :2 5a :2 2c :3 10 :2 16
1b 27 6b 6e :2 27 7b 7e :2 27
82 85 :2 27 8f 92 9b a3 :2 ac
:2 9b c9 :2 92 :2 27 d1 d4 :2 27 dc
df :2 27 e4 e7 :2 27 f1 f4 :2 27
:2 10 61 :2 d 10 :2 1a 2c :2 36 42
45 59 5c 5d :2 59 :2 2c :3 10 :2 16
1b 27 6b 6e :2 27 :2 10 60 :3 d
:2 18 2c :2 35 51 15 :2 d 39 :2 a
:4 f e d 2d 36 40 :2 2d 43
45 4e 5d :2 45 :2 2d 2c 64 61
10 17 :2 64 :2 61 2b :2 d 10 :2 1a
2c :2 36 42 46 5a 5d 5e :2 5a
:2 2c :3 10 :2 16 1b 27 6b 6e :2 27
7b 7e :2 27 82 85 :2 27 8f 92
9b a3 :2 ac :2 9b cd :2 92 :2 27 d5
d8 :2 27 e0 e3 :2 27 e8 eb :2 27
f5 f8 :2 27 :2 10 61 :2 d 10 :2 1a
2c :2 36 42 45 59 5c 5d :2 59
:2 2c :3 10 :2 16 1b 27 6b 6e :2 27
:2 10 60 :3 d :2 18 2c :2 35 55 15
:2 d 1b :2 a e :2 19 21 30 3e
:2 e 42 45 :2 42 d 4c :2 57 5f
12 21 :2 4c 25 28 :2 25 4b :2 d
10 19 29 :2 10 2c 2e :2 2c 13
:2 1e 26 35 43 :2 13 47 4a :2 47
1d 28 35 3f 4a 4f 58 69
7b 1b 1c 29 1a 25 2d 34
:2 16 :2 1f 2b 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 66
69 6a :2 66 :2 38 :3 1c :2 22 27 33
77 7a :2 33 87 8a :2 33 8e 91
:2 33 9d a0 :2 33 a8 ab :2 33 b7
ba c1 ca cd :2 ba :2 33 :2 1c 6d
:2 19 :2 16 :4 22 :2 16 13 :3 4c 17 :2 20
2b 29 :2 34 :2 29 16 20 2b 38
40 47 5f 6a 7e 1e :2 19 :2 22
2e 19 16 1e 1c :2 25 31 1c
2c :2 19 :2 1e 1c :2 25 31 1c 1f
:2 29 3b :2 45 51 55 69 6c 6d
:2 69 :2 3b :3 1f :2 25 2a 36 7a 7d
:2 36 8a 8d :2 36 91 94 :2 36 a0
a3 :2 36 ab ae :2 36 ba bd c4
cd d0 :2 bd :2 36 :2 1f 70 :2 1c :2 19
:4 25 :2 19 16 :4 44 :2 13 16 :2 21 29
38 48 :2 16 4c 4e :2 4c 16 28
16 50 :2 13 4c 1d 28 35 3f
4a 4f 58 68 7a 1b 1c 29
1a 25 2d 34 :2 16 :2 1f 2b 16
13 1b 19 :2 22 2e 19 29 :2 16
:2 1b 19 :2 22 2e 19 1c :2 26 38
:2 42 4e 52 66 69 6a :2 66 :2 38
:3 1c :2 22 27 33 77 7a :2 33 87
8a :2 33 8e 91 :2 33 9d a0 :2 33
a8 ab :2 33 b7 ba c1 ca cd
:2 ba :2 33 :2 1c 6d :2 19 :2 16 :4 22 :2 16
13 :3 10 17 :2 20 2b 29 :2 34 :2 29
16 20 2b 38 40 47 60 6b
7f 1e :2 19 :2 22 2e 19 16 1e
1c :2 25 31 1c 2c :2 19 :2 1e 1c
:2 25 31 1c 1f :2 29 3b :2 45 51
55 69 6c 6d :2 69 :2 3b :3 1f :2 25
2a 36 7a 7d :2 36 8a 8d :2 36
91 94 :2 36 a0 a3 :2 36 ab ae
:2 36 ba bd c4 cd d0 :2 bd :2 36
:2 1f 70 :2 1c :2 19 :4 25 :2 19 16 :4 44
:3 13 21 13 :4 10 13 1c 29 :2 13
2e 2c :2 37 :2 2c 1d 29 37 46
55 65 76 1c 29 16 1d 19
25 :3 23 19 :2 22 2e 19 2a 19
:2 22 2e 19 :4 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 66
69 6a :2 66 :2 38 :3 1c :2 22 27 33
77 7a :2 33 87 8a :2 33 8e 91
:2 33 9d a0 :2 33 a8 ab :2 33 b7
ba c1 ca cd :2 ba :2 33 :2 1c 6d
:2 19 :2 16 :4 22 :2 16 13 :4 3d 1d 29
39 48 59 69 7c 1c 29 16
1d 19 25 :3 23 19 :2 22 2e 19
2a 19 :2 22 2e 19 :4 16 13 1b
19 :2 22 2e 19 29 :2 16 :2 1b 19
:2 22 2e 19 1c :2 26 38 :2 42 4e
52 66 69 6a :2 66 :2 38 :3 1c :2 22
27 33 77 7a :2 33 87 8a :2 33
8e 91 :2 33 9d a0 :2 33 a8 ab
:2 33 b7 ba c1 ca cd :2 ba :2 33
:2 1c 6d :2 19 :2 16 :4 22 :2 16 13 :7 10
14 :2 1d 28 26 :2 31 :2 26 :2 13 21
2b :2 21 13 41 :2 10 30 1a 2a
3a 23 19 26 37 41 17 :2 13
:2 1c 28 13 10 18 16 :2 1f 2b
16 26 :2 13 :2 18 16 :2 1f 2b 16
19 :2 23 35 :2 3f 4b 4f 63 66
67 :2 63 :2 35 :3 19 :2 1f 24 30 74
77 :2 30 84 87 :2 30 8b 8e :2 30
9a 9d :2 30 a5 a8 :2 30 b4 b7
be c7 ca :2 b7 :2 30 :2 19 6a :2 16
:2 13 :4 1f :2 13 10 :3 d 13 1c 28
:2 13 2b 2d :2 2b 16 :2 20 32 :2 3c
48 4c 60 63 64 :2 60 :2 32 :3 16
:2 1c 21 2d 71 74 :2 2d 81 84
:2 2d 88 8b :2 2d 95 98 a1 a9
:2 b2 :2 a1 cd :2 98 :2 2d d5 d8 :2 2d
e0 e3 :2 2d e8 eb :2 2d f5 f8
:2 2d :2 16 67 :2 13 16 :2 20 32 :2 3c
48 4b 5f 62 63 :2 5f :2 32 :3 16
:2 1c 21 2d 71 74 :2 2d :2 16 66
:3 13 :2 1e 32 :2 3b 55 1b :2 13 2f
:2 10 13 :2 1e 26 35 43 :2 13 47
4a :2 47 1d 28 35 3f 4a 4f
58 69 7b 1b 1c 29 1a 25
2d 34 :2 16 :2 1f 2b 16 13 1b
19 :2 22 2e 19 29 :2 16 :2 1b 19
:2 22 2e 19 1c :2 26 38 :2 42 4e
52 66 69 6a :2 66 :2 38 :3 1c :2 22
27 33 77 7a :2 33 87 8a :2 33
8e 91 :2 33 9d a0 :2 33 a8 ab
:2 33 b7 ba c1 ca cd :2 ba :2 33
:2 1c 6d :2 19 :2 16 :4 22 :2 16 13 :3 4c
17 :2 20 2b 29 :2 34 :2 29 16 20
2b 38 40 47 5f 6a 7e 1e
:2 19 :2 22 2e 19 16 1e 1c :2 25
31 1c 2c :2 19 :2 1e 1c :2 25 31
1c 1f :2 29 3b :2 45 51 55 69
6c 6d :2 69 :2 3b :3 1f :2 25 2a 36
7a 7d :2 36 8a 8d :2 36 91 94
:2 36 a0 a3 :2 36 ab ae :2 36 ba
bd c4 cd d0 :2 bd :2 36 :2 1f 70
:2 1c :2 19 :4 25 :2 19 16 :4 44 :2 13 4c
1d 28 35 3f 4a 4f 58 68
7a 1b 1c 29 1a 25 2d 34
:2 16 :2 1f 2b 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 66
69 6a :2 66 :2 38 :3 1c :2 22 27 33
77 7a :2 33 87 8a :2 33 8e 91
:2 33 9d a0 :2 33 a8 ab :2 33 b7
ba c1 ca cd :2 ba :2 33 :2 1c 6d
:2 19 :2 16 :4 22 :2 16 13 :3 10 17 :2 20
2b 29 :2 34 :2 29 16 20 2b 38
40 47 60 6b 7f 1e :2 19 :2 22
2e 19 16 1e 1c :2 25 31 1c
2c :2 19 :2 1e 1c :2 25 31 1c 1f
:2 29 3b :2 45 51 55 69 6c 6d
:2 69 :2 3b :3 1f :2 25 2a 36 7a 7d
:2 36 8a 8d :2 36 91 94 :2 36 a0
a3 :2 36 ab ae :2 36 ba bd c4
cd d0 :2 bd :2 36 :2 1f 70 :2 1c :2 19
:4 25 :2 19 16 :4 44 :2 13 :5 10 1e 31
3f 4a 58 67 24 34 3f 4b
54 61 :2 4b 24 :2 1e 10 13 1c
28 :2 13 2b 2d :2 2b 13 21 2b
:2 21 13 2f 16 :2 20 32 :2 3c 48
4c 60 63 64 :2 60 :2 32 :3 16 :2 1c
21 2d 71 74 :2 2d 81 84 :2 2d
88 8b :2 2d 95 98 a1 a9 :2 a1
b6 :2 98 :2 2d be c1 :2 2d c9 cc
:2 2d d1 d4 :2 2d de e1 :2 2d :2 16
67 :2 13 16 :2 20 32 :2 3c 48 4b
5f 62 63 :2 5f :2 32 :3 16 :2 1c 21
2d 71 74 :2 2d :2 16 66 :3 13 :2 1e
32 3b 47 :2 32 4b 56 :2 13 :4 10
:4 d a 2b 11 1a 24 :2 11 2a
27 33 42 :2 2a :2 27 10 4b 54
5e :2 4b 61 63 :2 61 4a :2 10 d
1d d 10 19 23 :2 10 28 26
31 3b :2 28 :2 26 13 :2 1d 2f :2 39
45 48 5c 5f 60 :2 5c :2 2f :3 13
:2 19 1e 2a 6e 71 :2 2a 7e 81
:2 2a 85 88 :2 2a 92 95 9e a6
:2 af :2 9e d0 :2 95 :2 2a :2 13 63 :2 10
13 :2 1d 2f :2 39 45 48 5c 5f
60 :2 5c :2 2f :3 13 :2 19 1e 2a 6e
71 :2 2a :2 13 63 :3 10 :2 1b 31 :2 3a
5a :2 10 d 3e 13 1c 26 :2 13
2b 29 34 3e :2 2b :2 29 10 20
10 41 3e :3 d 1b d 17 27
15 24 26 36 47 4b 56 16
19 23 14 17 2b 2d 14 16
2b 2d 14 16 20 29 :2 10 :2 19
25 10 d 15 13 :2 1c 28 13
23 :2 10 :2 15 13 :2 1c 28 13 16
:2 20 32 :2 3c 48 4c 60 63 64
:2 60 :2 32 :3 16 :2 1c 21 2d 71 74
:2 2d 81 84 :2 2d 88 8b :2 2d 97
9a :2 2d a2 a5 :2 2d b1 b4 bb
c4 c7 :2 b4 :2 2d :2 16 67 :2 13 :2 10
:4 1c :2 10 d :3 66 10 19 25 :2 10
28 2a :2 28 13 :2 1d 2f :2 39 45
49 5d 60 61 :2 5d :2 2f :3 13 :2 19
1e 2a 6e 71 :2 2a 7e 81 :2 2a
85 88 :2 2a 92 95 9e a6 :2 af
:2 9e ca :2 95 :2 2a d2 d5 :2 2a dd
e0 :2 2a e5 e8 :2 2a f2 f5 :2 2a
:2 13 64 :2 10 13 :2 1d 2f :2 39 45
48 5c 5f 60 :2 5c :2 2f :3 13 :2 19
1e 2a 6e 71 :2 2a :2 13 63 :3 10
:2 1b 2f :2 38 52 18 :2 10 2c :2 d
17 27 37 20 31 34 39 53
13 17 24 27 36 3a 48 4b
16 19 26 37 3a 44 11 19
20 23 46 4a :2 10 :2 19 25 10
d 15 13 :2 1c 28 13 23 :2 10
:2 15 13 :2 1c 28 13 16 :2 20 32
:2 3c 48 4c 60 63 64 :2 60 :2 32
:3 16 :2 1c 21 2d 71 74 :2 2d 81
84 :2 2d 88 8b :2 2d 97 9a :2 2d
a2 a5 :2 2d b1 b4 bb c4 c7
:2 b4 :2 2d :2 16 67 :2 13 :2 10 :4 1c :2 10
d :3 66 10 19 25 :2 10 28 2a
:2 28 13 :2 1d 2f :2 39 45 49 5d
60 61 :2 5d :2 2f :3 13 :2 19 1e 2a
6e 71 :2 2a 7e 81 :2 2a 85 88
:2 2a 92 95 9e a6 :2 af :2 9e c3
:2 95 :2 2a cb ce :2 2a d6 d9 :2 2a
de e1 :2 2a eb ee :2 2a :2 13 64
:2 10 13 :2 1d 2f :2 39 45 48 5c
5f 60 :2 5c :2 2f :3 13 :2 19 1e 2a
6e 71 :2 2a :2 13 63 :3 10 :2 1b 2f
:2 38 4b 56 :2 10 2c :3 d 1d :2 d
1b 2d 3b 46 54 60 21 2f
:2 60 :2 1b d 66 2b :2 a d 16
22 :2 d 25 27 :2 25 d 1b 25
:2 1b d 29 :2 a d :2 17 29 :2 33
3f 43 57 5a 5b :2 57 :2 29 :3 d
:2 13 18 24 68 6b :2 24 78 7b
:2 24 7f 82 :2 24 8c 8f 98 a0
:2 98 ad :2 8f :2 24 b5 b8 :2 24 c0
c3 :2 24 c8 cb :2 24 d5 d8 :2 24
:2 d 5e :2 a d :2 17 29 :2 33 3f
42 56 59 5a :2 56 :2 29 :3 d :2 13
18 24 68 6b :2 24 :2 d 5d :3 a
:2 15 29 32 3e :2 29 42 4d :2 a
7 38 :4 e d :4 30 2f :2 d a
1b :2 a 1c a e :2 19 21 30
40 :2 e 44 46 :2 44 d 4e :2 59
10 20 2f :2 4e 33 36 :2 33 4d
:2 d 10 :2 1a 2c :2 36 42 46 5a
5d 5e :2 5a :2 2c :3 10 :2 16 1b 27
6b 6e :2 27 7b 7e :2 27 82 85
:2 27 8f 92 9b a3 :2 ac :2 9b c9
:2 92 :2 27 d1 d4 :2 27 dc df :2 27
e4 e7 :2 27 f1 f4 :2 27 :2 10 61
:2 d 10 :2 1a 2c :2 36 42 45 59
5c 5d :2 59 :2 2c :3 10 :2 16 1b 27
6b 6e :2 27 :2 10 60 :3 d :2 18 2c
:2 35 51 15 :2 d 39 :2 a 14 20
2b 12 20 2d 12 20 2d 3e
4b :2 d :2 16 22 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
5d 60 61 :2 5d :2 2f :3 13 :2 19 1e
2a 6e 71 :2 2a 7e 81 :2 2a 85
88 :2 2a 94 97 :2 2a 9f a2 :2 2a
ae b1 b8 c1 c4 :2 b1 :2 2a :2 13
64 :2 10 :2 d :4 19 :2 d a :3 4e d
14 20 23 :2 d 26 28 :2 26 d
20 d 2c d 20 d :5 a 17
:2 a 17 a e :2 19 21 30 3e
:2 e 42 45 :2 42 d 4c :2 57 5f
12 21 :2 4c 25 28 :2 25 4b :2 d
2f 38 41 :2 2f 47 44 50 59
:2 47 :2 44 2e :2 d 10 :2 1b 23 32
40 :2 10 44 47 :2 44 1a 25 39
46 18 19 26 37 44 :2 13 :2 1c
28 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 63 66 67
:2 63 :2 35 :3 19 :2 1f 24 30 74 77
:2 30 84 87 :2 30 8b 8e :2 30 9a
9d :2 30 a5 a8 :2 30 b4 b7 be
c7 ca :2 b7 :2 30 :2 19 6a :2 16 :2 13
:4 1f :2 13 10 :3 49 10 20 :2 10 1b
:2 10 1d 10 d 49 13 :2 1e 26
36 45 :2 13 49 4c :2 49 1a 25
32 3a 41 5a 65 6a :2 18 :2 13
:2 1c 28 13 10 18 16 :2 1f 2b
16 26 :2 13 :2 18 16 :2 1f 2b 16
19 :2 23 35 :2 3f 4b 4f 63 66
67 :2 63 :2 35 :3 19 :2 1f 24 30 74
77 :2 30 84 87 :2 30 8b 8e :2 30
9a 9d :2 30 a5 a8 :2 30 b4 b7
be c7 ca :2 b7 :2 30 :2 19 6a :2 16
:2 13 :4 1f :2 13 10 :3 4e 10 1d :2 28
2f 3f :2 1d :2 10 20 21 :2 20 :2 10
1e 35 43 4e 59 5f 65 28
34 42 55 :2 1e 10 14 1d 29
:2 14 2c 2e :2 2c 13 :4 37 36 35
:2 13 1d 32 3e 1c 29 3a 4e
5c 67 6f 76 16 1d 19 25
:3 23 19 :2 22 2e 19 2a 19 :2 22
2e 19 :4 16 13 1b 19 :2 22 2e
19 29 :2 16 :2 1b 19 :2 22 2e 19
1c :2 26 38 :2 42 4e 52 66 69
6a :2 66 :2 38 :3 1c :2 22 27 33 77
7a :2 33 87 8a :2 33 8e 91 :2 33
9d a0 :2 33 a8 ab :2 33 b7 ba
c1 ca cd :2 ba :2 33 :2 1c 6d :2 19
:2 16 :4 22 :2 16 13 :4 56 :2 10 13 1a
26 29 :2 13 2c 2e :2 2c 13 22
13 32 13 22 13 :5 10 1e 2c
3a 45 50 5a 66 6c 1f 2d
3d 4a 55 1f 2b 2e 39 :2 1e
10 1a 22 29 40 48 4f 68
72 83 91 18 1e 24 30 43
4f 18 19 2d :2 13 :2 1c 28 13
10 18 16 :2 1f 2b 16 26 :2 13
:2 18 16 :2 1f 2b 16 19 :2 23 35
:2 3f 4b 4f 63 66 67 :2 63 :2 35
:3 19 :2 1f 24 30 74 77 :2 30 84
87 :2 30 8b 8e :2 30 9a 9d :2 30
a5 a8 :2 30 b4 b7 be c7 ca
:2 b7 :2 30 :2 19 6a :2 16 :2 13 :4 1f :2 13
10 :3 4e 10 20 :2 10 1e 35 43
4e 59 5f 65 28 34 42 55
:2 1e :2 10 1e 28 :2 1e :2 10 :2 19 25
10 13 :2 1d 2f :2 39 45 48 5c
5f 60 :2 5c :2 2f :3 13 :2 19 1e 2a
6e 71 :2 2a 7e 81 :2 2a 85 88
:2 2a 92 95 9e a6 :2 af :2 9e ba
:2 95 :2 2a c2 c5 :2 2a cd d0 :2 2a
d5 d8 :2 2a ec ef :2 2a :2 13 63
:3 10 :2 1b 2d :2 36 40 55 :2 10 d
4e 49 13 1c 25 :2 13 2a 28
33 3c :2 2a :2 28 10 1b 24 2d
:2 1b 30 32 3b 44 :2 32 :2 1b :2 10
1d 10 d 3f 49 13 1c 25
:2 13 28 2a :2 28 10 1b :2 10 1d
10 1a 28 18 19 26 :2 13 :2 1c
28 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 63 66 67
:2 63 :2 35 :3 19 :2 1f 24 30 74 77
:2 30 84 87 :2 30 8b 8e :2 30 9a
9d :2 30 a5 a8 :2 30 b4 b7 be
c7 ca :2 b7 :2 30 :2 19 6a :2 16 :2 13
:4 1f :2 13 10 :3 2c d 2c 49 13
1c 25 :2 13 2a 28 33 3c :2 2a
:2 28 10 1b 24 2d :2 1b 30 32
3b 44 :2 32 :2 1b :2 10 1f :2 10 18
10 13 1c 2d :2 13 30 32 :2 30
13 1b 1a 1d :2 1b 35 38 :2 1b
45 48 :2 1b 1f 22 :2 1b 2f 32
:2 1b 1a 1d :2 1b 46 49 51 5a
63 :2 51 :2 49 :2 1b :2 13 18 :2 26 13
34 13 1b 1a 1d :2 1b 35 38
:2 1b 41 44 :2 1b 6a 6d :2 1b 1f
22 :2 1b 1a 1d :2 1b 46 49 51
5a 63 :2 51 :2 49 :2 1b :2 13 18 :2 26
13 :5 10 16 1f 30 :2 16 33 35
:2 33 1c 2b :2 16 19 23 :2 19 :2 22
2e 19 29 19 :2 22 2e 19 :4 16
37 1c 2b :2 16 19 23 :2 19 :2 22
2e 19 29 19 :2 22 2e 19 :4 16
:4 13 17 :2 20 2c 29 :2 35 :2 29 :3 16
45 :2 13 10 14 3f 13 1c 2d
:2 13 30 32 :2 30 16 1c :3 16 :2 1f
2b 16 13 1b 19 :2 22 2e 19
29 :2 16 :2 1b 19 :2 22 2e 19 1c
:2 26 38 :2 42 4e 52 66 69 6a
:2 66 :2 38 :3 1c :2 22 27 33 77 7a
:2 33 87 8a :2 33 8e 91 :2 33 9d
a0 :2 33 a8 ab :2 33 b7 ba c1
ca cd :2 ba :2 33 :2 1c 6d :2 19 :2 16
:4 22 :2 16 13 :4 34 16 1c :3 16 :2 1f
2b 16 13 1b 19 :2 22 2e 19
29 :2 16 :2 1b 19 :2 22 2e 19 1c
:2 26 38 :2 42 4e 52 66 69 6a
:2 66 :2 38 :3 1c :2 22 27 33 77 7a
:2 33 87 8a :2 33 8e 91 :2 33 9d
a0 :2 33 a8 ab :2 33 b7 ba c1
ca cd :2 ba :2 33 :2 1c 6d :2 19 :2 16
:4 22 :2 16 13 :7 10 :4 14 13 16 1f
30 :2 16 33 35 :2 33 19 :2 23 35
:2 3f 4b 4f 63 66 67 :2 63 :2 35
:3 19 :2 1f 24 30 74 77 :2 30 84
87 :2 30 8b 8e :2 30 98 9b a4
ac :2 b5 :2 a4 c8 :2 9b :2 30 d0 d3
:2 30 db de :2 30 e3 e6 :2 30 f3
f6 :2 30 :2 19 6a :2 16 19 :2 23 35
:2 3f 4b 4e 62 65 66 :2 62 :2 35
:3 19 :2 1f 24 30 74 77 :2 30 :2 19
69 :3 16 :2 21 35 :2 3e 50 5e :2 16
37 19 :2 23 35 :2 3f 4b 4f 63
66 67 :2 63 :2 35 :3 19 :2 1f 24 30
74 77 :2 30 84 87 :2 30 8b 8e
:2 30 98 9b a4 ac :2 b5 :2 a4 c8
:2 9b :2 30 d0 d3 :2 30 db de :2 30
e3 e6 :2 30 ef f2 :2 30 :2 19 6a
:2 16 19 :2 23 35 :2 3f 4b 4e 62
65 66 :2 62 :2 35 :3 19 :2 1f 24 30
74 77 :2 30 :2 19 69 :3 16 :2 21 35
:2 3e 50 5a :2 16 :4 13 29 :2 10 1a
25 32 3a 41 57 62 67 :2 18
:2 13 :2 1c 28 13 10 18 16 :2 1f
2b 16 26 :2 13 :2 18 16 :2 1f 2b
16 19 :2 23 35 :2 3f 4b 4f 63
66 67 :2 63 :2 35 :3 19 :2 1f 24 30
74 77 :2 30 84 87 :2 30 8b 8e
:2 30 9a 9d :2 30 a5 a8 :2 30 b4
b7 be c7 ca :2 b7 :2 30 :2 19 6a
:2 16 :2 13 :4 1f :2 13 10 :3 3f 10 1b
:2 10 1d 10 3f 49 :2 d a 5d
10 19 22 :2 10 28 25 31 42
:2 28 :2 25 10 19 28 :2 10 2d 2b
36 40 :2 2d :2 2b 13 :2 1d 2f :2 39
45 49 5d 60 61 :2 5d :2 2f :3 13
:2 19 1e 2a 6e 71 :2 2a 7e 81
:2 2a 85 88 :2 2a 92 95 9e a6
:2 af :2 9e ba :2 95 :2 2a c2 c5 :2 2a
cd d0 :2 2a d5 d8 :2 2a e2 e5
:2 2a :2 13 64 :2 10 13 :2 1d 2f :2 39
45 48 5c 5f 60 :2 5c :2 2f :3 13
:2 19 1e 2a 6e 71 :2 2a :2 13 63
:3 10 :2 1b 2f :2 38 42 4d :2 10 43
:2 d 10 19 22 :2 10 25 27 :2 25
10 23 10 29 14 1d 26 :2 14
29 2b :2 29 13 32 3b 44 :2 32
47 49 :2 47 31 :2 13 16 :2 20 32
:2 3c 48 4b 5f 62 63 :2 5f :2 32
:3 16 :2 1c 21 2d 71 74 :2 2d 81
84 :2 2d 88 8b :2 2d 95 98 a1
a9 :2 b2 :2 a1 c3 :2 98 :2 2d :2 16 66
:2 13 16 :2 20 32 :2 3c 48 4b 5f
63 64 :2 5f :2 32 :3 16 :2 1c 21 2d
72 75 :2 2d :2 16 67 :3 13 :2 1e 34
:2 3d 4d :2 13 54 :3 10 23 10 :4 d
:4 11 10 1a 29 18 19 2d :2 13
:2 1c 28 13 10 18 16 :2 1f 2b
16 26 :2 13 :2 18 16 :2 1f 2b 16
19 :2 23 35 :2 3f 4b 4f 63 67
68 :2 63 :2 35 :3 19 :2 1f 24 30 75
78 :2 30 85 88 :2 30 8c 8f :2 30
9b 9e :2 30 a6 a9 :2 30 b5 b8
bf c8 cb :2 b8 :2 30 :2 19 6b :2 16
:2 13 :4 1f :2 13 10 :3 2e 14 :2 1d 28
26 :2 31 :2 26 13 46 4f 60 :2 46
65 63 16 22 :2 65 :2 63 45 :2 13
16 :2 20 32 :2 3c 48 4c 60 64
65 :2 60 :2 32 :3 16 :2 1c 21 2d 72
75 :2 2d 82 85 :2 2d 89 8c :2 2d
96 99 a2 aa :2 b3 :2 a2 cb :2 99
:2 2d d3 d6 :2 2d de e1 :2 2d e6
e9 :2 2d f9 fc :2 2d :2 16 68 :2 13
16 :2 20 32 :2 3c 48 4b 5f 63
64 :2 5f :2 32 :3 16 :2 1c 21 2d 72
75 :2 2d :2 16 67 :3 13 :2 1e 32 :2 3b
52 1b :2 13 26 :2 10 2e :3 d 18
:2 d 1a d 10 19 24 :2 10 27
29 :2 27 10 1d 10 2c :2 d 17
2c 3a 46 10 1e 2a 39 10
1e 27 2e 3b 44 51 5a 16
2a 10 17 13 1f :3 1d 13 :2 1c
28 13 24 13 :2 1c 28 13 :4 10
d 15 13 :2 1c 28 13 23 :2 10
:2 15 13 :2 1c 28 13 16 :2 20 32
:2 3c 48 4c 60 64 65 :2 60 :2 32
:3 16 :2 1c 21 2d 72 75 :2 2d 82
85 :2 2d 89 8c :2 2d 98 9b :2 2d
a3 a6 :2 2d b2 b5 bc c5 c8
:2 b5 :2 2d :2 16 68 :2 13 :2 10 :4 1c :2 10
d :3 45 10 19 24 :2 10 27 2a
:2 27 10 1d 10 2c :2 d 45 5d
:2 a d 17 19 :2 17 d 1d 1e
:2 1d :2 d 20 37 45 50 5b 61
67 2d 39 47 5a :2 20 d 11
1a 26 :2 11 29 2b :2 29 10 :4 34
33 32 :3 10 1e 33 41 4c 56
67 26 38 44 :2 1e 10 53 :2 d
:4 11 :2 10 23 33 41 4c :2 23 10
2e 13 1a 26 29 :2 13 2c 2e
:2 2c 13 26 36 44 4f :2 26 13
32 13 26 36 44 4f :2 26 13
:4 10 :4 d :4 11 10 13 1c 27 :2 13
2a 2c :2 2a 13 21 :2 2a 31 3a
45 :2 31 49 :2 52 :2 21 20 13 1d
28 35 3d 44 5f 6a 6f :2 1b
:2 16 :2 1f 2b 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 66
6a 6b :2 66 :2 38 :3 1c :2 22 27 33
78 7b :2 33 88 8b :2 33 8f 92
:2 33 9e a1 :2 33 a9 ac :2 33 b8
bb c2 cb ce :2 bb :2 33 :2 1c 6e
:2 19 :2 16 :4 22 :2 16 13 :4 2e 1d 28
35 3d 44 5f 6a 6f :2 1b :2 16
:2 1f 2b 16 13 1b 19 :2 22 2e
19 29 :2 16 :2 1b 19 :2 22 2e 19
1c :2 26 38 :2 42 4e 52 66 6a
6b :2 66 :2 38 :3 1c :2 22 27 33 78
7b :2 33 88 8b :2 33 8f 92 :2 33
9e a1 :2 33 a9 ac :2 33 b8 bb
c2 cb ce :2 bb :2 33 :2 1c 6e :2 19
:2 16 :4 22 :2 16 13 :8 10 1c 25 2f
:2 1c 32 34 3d 4c :2 34 :2 1c 10
2f :3 d 1f :2 d 20 2e 39 47
51 61 24 30 39 42 :2 30 46
4f 60 :2 46 :2 20 d 10 19 2a
:2 10 2d 30 :2 2d 13 :2 1d 2f :2 39
45 49 5d 61 62 :2 5d :2 2f :3 13
:2 19 1e 2a 6f 72 :2 2a 7f 82
:2 2a 86 89 :2 2a 93 96 9f a7
:2 b0 :2 9f c8 :2 96 :2 2a d0 d3 :2 2a
db de :2 2a e3 e6 :2 2a f0 f3
:2 2a :2 13 65 :2 10 13 :2 1d 2f :2 39
45 48 5c 60 61 :2 5c :2 2f :3 13
:2 19 1e 2a 6f 72 :2 2a :2 13 64
:3 10 :2 1b 2f :2 38 4f 5a :2 10 d
32 13 1d 1f :2 1d 17 18 19
2d 10 1a 2f 3d 4b 54 5b
68 71 7e 87 13 22 2b 37
47 57 60 6d 13 27 19 2d
13 1a 16 22 :3 20 16 :2 1f 2b
16 27 16 :2 1f 2b 16 :4 13 10
18 16 :2 1f 2b 16 26 :2 13 :2 18
16 :2 1f 2b 16 19 :2 23 35 :2 3f
4b 4f 63 67 68 :2 63 :2 35 :3 19
:2 1f 24 30 75 78 :2 30 85 88
:2 30 8c 8f :2 30 9b 9e :2 30 a6
a9 :2 30 b5 b8 bf c8 cb :2 b8
:2 30 :2 19 6b :2 16 :2 13 :4 1f :2 13 10
:4 21 32 :3 d 1b 25 :2 1b d 10
:2 1a 2c :2 36 42 46 5a 5e 5f
:2 5a :2 2c :3 10 :2 16 1b 27 6c 6f
:2 27 7c 7f :2 27 83 86 :2 27 90
93 9c a4 :2 9c b1 :2 93 :2 27 b9
bc :2 27 c4 c7 :2 27 cc cf :2 27
d9 dc :2 27 :2 10 62 :2 d 10 :2 1a
2c :2 36 42 45 59 5d 5e :2 59
:2 2c :3 10 :2 16 1b 27 6c 6f :2 27
:2 10 61 :3 d :2 18 2c 35 41 :2 2c
45 50 :2 d 1b :2 a d 16 20
:2 d 26 23 2f 3e :2 26 :2 23 17
27 15 24 34 16 2a 14 29
14 1e 27 :2 10 :2 19 25 10 d
15 13 :2 1c 28 13 23 :2 10 :2 15
13 :2 1c 28 13 16 :2 20 32 :2 3c
48 4c 60 64 65 :2 60 :2 32 :3 16
:2 1c 21 2d 72 75 :2 2d 82 85
:2 2d 89 8c :2 2d 98 9b :2 2d a3
a6 :2 2d b2 b5 bc c5 c8 :2 b5
:2 2d :2 16 68 :2 13 :2 10 :4 1c :2 10 d
:3 41 10 19 25 :2 10 28 2a :2 28
13 :2 1d 2f :2 39 45 49 5d 61
62 :2 5d :2 2f :3 13 :2 19 1e 2a 6f
72 :2 2a 7f 82 :2 2a 86 89 :2 2a
93 96 9f a7 :2 b0 :2 9f cb :2 96
:2 2a d3 d6 :2 2a de e1 :2 2a e6
e9 :2 2a f3 f6 :2 2a :2 13 65 :2 10
13 :2 1d 2f :2 39 45 48 5c 60
61 :2 5c :2 2f :3 13 :2 19 1e 2a 6f
72 :2 2a :2 13 64 :3 10 :2 1b 2f :2 38
52 18 :2 10 2c :3 d 1d :2 d 1f
28 32 :2 1f d 10 19 27 :2 10
2c 2a 35 3f :2 2c :2 2a 10 20
21 :2 20 :2 10 23 3a 48 53 5e
64 6a 2d 39 47 5a :2 23 10
:4 14 :2 13 26 36 44 4f :2 26 13
31 16 1d 29 2c :2 16 2f 31
:2 2f 16 29 39 47 52 :2 29 16
35 16 29 39 47 52 :2 29 16
:4 13 :5 10 1d 10 :4 14 13 16 1f
2a :2 16 2d 2f :2 2d 16 24 :2 2d
34 3d 48 :2 34 4c :2 55 :2 24 23
16 20 2b 38 40 47 62 6d
72 :2 1e :2 19 :2 22 2e 19 16 1e
1c :2 25 31 1c 2c :2 19 :2 1e 1c
:2 25 31 1c 1f :2 29 3b :2 45 51
55 69 6d 6e :2 69 :2 3b :3 1f :2 25
2a 36 7b 7e :2 36 8b 8e :2 36
92 95 :2 36 a1 a4 :2 36 ac af
:2 36 bb be c5 ce d1 :2 be :2 36
:2 1f 71 :2 1c :2 19 :4 25 :2 19 16 :4 31
20 2b 38 40 47 62 6d 72
:2 1e :2 19 :2 22 2e 19 16 1e 1c
:2 25 31 1c 2c :2 19 :2 1e 1c :2 25
31 1c 1f :2 29 3b :2 45 51 55
69 6d 6e :2 69 :2 3b :3 1f :2 25 2a
36 7b 7e :2 36 8b 8e :2 36 92
95 :2 36 a1 a4 :2 36 ac af :2 36
bb be c5 ce d1 :2 be :2 36 :2 1f
71 :2 1c :2 19 :4 25 :2 19 16 :7 13 16
1f 2b :2 16 2e 30 :2 2e 20 2c
3c 1e 1f 2c 3d 51 5f 6a
72 79 :2 19 :2 22 2e 19 16 1e
1c :2 25 31 1c 2c :2 19 :2 1e 1c
:2 25 31 1c 1f :2 29 3b :2 45 51
55 69 6d 6e :2 69 :2 3b :3 1f :2 25
2a 36 7b 7e :2 36 8b 8e :2 36
92 95 :2 36 a1 a4 :2 36 ac af
:2 36 bb be c5 ce d1 :2 be :2 36
:2 1f 71 :2 1c :2 19 :4 25 :2 19 16 :4 32
:2 13 32 :3 10 1c 25 33 :2 1c 36
38 41 50 :2 38 :2 1c 53 55 5e
69 :2 55 :2 1c :2 10 22 10 1a 2f
3d 4b 54 5b 67 70 13 20
2e 3c 4a 53 13 2a 42 19
2d 13 1a 16 22 :3 20 16 :2 1f
2b 16 27 16 :2 1f 2b 16 :4 13
10 18 16 :2 1f 2b 16 26 :2 13
:2 18 16 :2 1f 2b 16 19 :2 23 35
:2 3f 4b 4f 63 67 68 :2 63 :2 35
:3 19 :2 1f 24 30 75 78 :2 30 85
88 :2 30 8c 8f :2 30 9b 9e :2 30
a6 a9 :2 30 b5 b8 bf c8 cb
:2 b8 :2 30 :2 19 6b :2 16 :2 13 :4 1f :2 13
10 :3 42 14 :2 1d 28 26 :2 31 :2 26
:2 13 25 :2 13 21 2f 3a 48 52
62 6d 1f 22 2b 3c :2 22 :2 21
13 16 1f 2b :2 16 2e 31 :2 2e
19 :2 23 35 :2 3f 4b 4f 63 67
68 :2 63 :2 35 :3 19 :2 1f 24 30 75
78 :2 30 85 88 :2 30 8c 8f :2 30
99 9c a5 ad :2 a5 ba :2 9c :2 30
c2 c5 :2 30 cd d0 :2 30 d5 d8
:2 30 e2 e5 :2 30 :2 19 6b :2 16 19
:2 23 35 :2 3f 4b 4e 62 66 67
:2 62 :2 35 :3 19 :2 1f 24 30 75 78
:2 30 :2 19 6a :3 16 :2 21 35 3e 4a
:2 35 4e 59 :2 16 33 :2 13 41 :3 10
1c 25 33 :2 1c 10 42 10 22
2a 33 3d :2 33 4c :2 2a 4f 51
5a 64 :2 5a 6f :2 51 :2 2a 29 24
26 :2 29 28 30 32 :2 28 :2 22 10
:5 d 1b 2f 3d 48 52 5d 25
31 3b 49 25 34 44 50 25
31 3e 4c 25 33 42 55 25
31 3d 46 51 :2 3d :2 1b d 10
:2 1a 2c :2 36 42 46 5a 5e 5f
:2 5a :2 2c :3 10 :2 16 1b 27 6c 6f
:2 27 7c 7f :2 27 83 86 :2 27 90
93 9c a4 :2 9c b1 :2 93 :2 27 b9
bc :2 27 c4 c7 :2 27 cc cf :2 27
d9 dc :2 27 :2 10 62 :2 d 10 :2 1a
2c :2 36 42 45 59 5d 5e :2 59
:2 2c :3 10 :2 16 1b 27 6c 6f :2 27
:2 10 61 :3 d :2 18 2c 35 41 :2 2c
45 50 :2 d a 41 10 19 29
:2 10 2f 2c 38 48 :2 2f :2 2c 11
1a 29 :2 11 2c 2e :2 2c 10 36
3f 4f :2 36 52 54 :2 52 35 :2 10
13 :2 1d 2f :2 39 45 49 5d 61
62 :2 5d :2 2f :3 13 :2 19 1e 2a 6f
72 :2 2a 7f 82 :2 2a 86 89 :2 2a
93 96 9f a7 :2 b0 :2 9f bb :2 96
:2 2a c3 c6 :2 2a ce d1 :2 2a d6
d9 :2 2a e3 e6 :2 2a :2 13 65 :2 10
13 :2 1d 2f :2 39 45 48 5c 60
61 :2 5c :2 2f :3 13 :2 19 1e 2a 6f
72 :2 2a :2 13 64 :3 10 :2 1b 2f :2 38
42 4d :2 10 57 :2 d 17 2c 40
10 1e 27 2e 3a 43 16 2a
10 17 13 1f :3 1d 13 :2 1c 28
13 24 13 :2 1c 28 13 :4 10 d
15 13 :2 1c 28 13 23 :2 10 :2 15
13 :2 1c 28 13 16 :2 20 32 :2 3c
48 4c 60 64 65 :2 60 :2 32 :3 16
:2 1c 21 2d 72 75 :2 2d 82 85
:2 2d 89 8c :2 2d 98 9b :2 2d a3
a6 :2 2d b2 b5 bc c5 c8 :2 b5
:2 2d :2 16 68 :2 13 :2 10 :4 1c :2 10 d
:3 4b 17 23 31 3a 41 4d 56
16 23 34 41 10 17 13 1f
:3 1d 13 :2 1c 28 13 24 13 :2 1c
28 13 :4 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 60 64
65 :2 60 :2 32 :3 16 :2 1c 21 2d 72
75 :2 2d 82 85 :2 2d 89 8c :2 2d
98 9b :2 2d a3 a6 :2 2d b2 b5
bc c5 c8 :2 b5 :2 2d :2 16 68 :2 13
:2 10 :4 1c :2 10 d :4 4b 41 :3 a 18
22 :2 18 a d :2 17 29 :2 33 3f
43 57 5b 5c :2 57 :2 29 :3 d :2 13
18 24 69 6c :2 24 79 7c :2 24
80 83 :2 24 8d 90 99 a1 :2 99
ae :2 90 :2 24 b6 b9 :2 24 c1 c4
:2 24 c9 cc :2 24 d6 d9 :2 24 :2 d
5f :2 a d :2 17 29 :2 33 3f 42
56 5a 5b :2 56 :2 29 :3 d :2 13 18
24 69 6c :2 24 :2 d 5e :3 a :2 15
29 32 3e :2 29 42 4d :2 a 4e
38 14 17 24 27 34 37 43
46 17 1a 17 1a d 10 d
10 1c 1f 12 21 30 3e :4 d
1b 12 27 30 33 47 :2 d :2 16
22 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 5d 61 62
:2 5d :2 2f :3 13 :2 19 1e 2a 6f 72
:2 2a 7f 82 :2 2a 86 89 :2 2a 95
98 :2 2a a0 a3 :2 2a af b2 b9
c2 c5 :2 b2 :2 2a :2 13 65 :2 10 :2 d
:4 19 :2 d a :3 7 12 :2 1b 26 24
:2 2f :2 24 11 :3 d 10 :2 1a 2c :2 36
42 46 5a 5e 5f :2 5a :2 2c :3 10
:2 16 1b 27 6c 6f :2 27 7c 7f
:2 27 83 86 :2 27 90 93 9c a4
:2 ad :2 9c c0 :2 93 :2 27 c8 cb :2 27
d3 d6 :2 27 db de :2 27 f2 f5
:2 27 :2 10 62 :2 d 10 :2 1a 2c :2 36
42 45 59 5d 5e :2 59 :2 2c :3 10
:2 16 1b 27 6c 6f :2 27 :2 10 61
:3 d :2 18 2c :2 35 47 15 :2 d 3f
:2 a d :2 18 1f 2e :2 d 3d 3f
:2 3d d :2 16 22 d 10 :2 1a 2c
:2 36 42 46 5a 5e 5f :2 5a :2 2c
:3 10 :2 16 1b 27 6c 6f :2 27 7c
7f :2 27 83 86 :2 27 90 93 9c
a4 :2 ad :2 9c b8 :2 93 :2 27 c0 c3
:2 27 cb ce :2 27 d3 d6 :2 27 ea
ed :2 27 :2 10 62 :2 d 10 :2 1a 2c
:2 36 42 45 59 5d 5e :2 59 :2 2c
:3 10 :2 16 1b 27 6c 6f :2 27 :2 10
61 :3 d :2 18 2c :2 35 3f 54 :2 d
41 :2 a :4 7 35 :3 4 11 :2 4 15
:2 4 12 :2 4 14 :2 1d :2 4 :2 d 19
30 3e 49 58 :2 19 4 7 :2 11
23 :2 2d 39 3c 50 54 55 :2 50
:2 23 :3 7 :2 d 12 1e 63 66 :2 1e
73 76 :2 1e 7a 7d :2 1e 87 8a
93 9b :2 a4 :2 93 af :2 8a :2 1e b7
ba :2 1e c2 c5 :2 1e ca cd :2 1e
d7 da :2 1e :2 7 58 :3 4 :2 f 21
:2 2a 34 3f :2 4 1 9 12 9
a :2 14 26 :2 30 3c 40 54 58
59 :2 54 :2 26 :3 a :2 10 15 21 66
69 :2 21 :3 a :2 10 15 39 5c 5f
:2 68 :2 39 76 79 :2 39 81 84 :2 8d
:2 39 :2 a 5c :3 7 c 15 1e 2c
a 13 1e f e 7 e 7
e 7 29 :2 4 :2 9 7 :2 10 1c
:2 7 :2 11 23 :2 2d 39 3d 51 55
56 :2 51 :2 23 :3 7 :2 d 12 1e 63
66 :2 1e :3 7 :2 d 12 36 42 45
:2 36 4d 50 :2 36 5c 5f 66 6f
72 :2 5f :2 36 :2 7 59 :2 4 7 c
1b a 13 1e f e 7 e
7 e 7 :2 4 :4 10 :2 4 :8 1 
2628
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 :4 7 :4 8
:4 9 :4 a :4 b 2
:4 c :2 1 :7 e :7 f
:7 10 :7 11 :7 12 :7 13
:7 14 :7 15 :7 16 :7 17
:7 18 :7 19 :7 1a :5 1b
:5 1c :5 1d :5 1e :5 1f
:5 20 :5 21 :5 22 :5 23
:7 24 :7 25 :6 26 :6 27
:6 28 :5 29 :5 2a :6 2b
:6 2c :6 2d :6 2e :6 2f
:6 30 :6 31 :6 32 :6 33
:6 34 :7 35 :7 36 :7 37
:6 38 :6 39 :6 3a :6 3b
:6 3c :6 3d :6 3e :6 3f
:6 40 :6 41 :6 42 :6 43
:6 44 :6 45 :6 46 :6 47
:6 48 :6 49 :6 4a :6 4b
:6 4c :6 4d :6 4e :6 4f
:7 50 :7 51 :6 52 :7 53
:6 54 :6 55 :6 56 :6 57
:6 58 :6 59 :6 5a :6 5b
:6 5c :6 5d :6 5e :7 5f
:5 60 :5 61 :7 64 :7 67
:7 69 :b 6b :11 6c 6d
:2 6c :2 6d :2 6c :2 6d
:2 6c :2 6e :2 6c :2 6e
:2 6c :2 6e :2 6c :2 6e
:2 6c :2 6f :2 6c :12 6f
:2 6c :2 70 :2 6c :12 70
:2 6c :2 71 :2 6c :9 71
:2 6c :2 72 :2 6c :9 72
:2 6c :2 73 :2 6c :9 73
:2 6c :2 74 :2 6c :9 74
:2 6c :2 75 :2 6c :2 75
:2 6c :2 75 :2 6c :2 75
:2 6c :2 75 :4 6c :3 6b
:11 78 :b 79 :3 78 :3 7e
:3 7f :3 80 :3 81 :3 84
:8 87 :2 88 :2 89 :2 8a
:4 8b :4 8c 8d 8e
8f :2 90 :2 91 :3 92
87 :5 93 86 95
:5 96 :3 95 :2 97 :5 99
:11 9a :24 9b :3 9a :2 98
:6 97 94 :3 6a :a a1
:4 a3 :8 a4 :8 a5 :5 a6
:8 a7 :4 a8 :4 a9 :3 aa
:4 ab :8 ac :5 ad :3 ae
:4 af :4 b0 :4 b1 :3 b2
b3 :2 b4 a3 :5 b5
a2 b7 :5 b8 :3 b7
:2 b9 :5 bb :11 bc :24 bd
:3 bc :2 ba :6 b9 b6
:3 a1 :a c4 :6 c6 :a c7
:6 c8 :7 c9 :3 ca c6
:5 cb c5 cd :5 ce
:3 cd :2 cf :5 d1 :11 d2
:24 d3 :3 d2 :2 d0 :6 cf
cc :3 c4 :a d9 :11 da
:30 db :3 da :11 dd :b de
:3 dd :a e0 :3 d9 :3 e3
:3 e4 :10 e5 :3 e6 :2 e5
:4 e6 :3 e5 :11 e7 :30 e8
:3 e7 :11 ea :b eb :3 ea
:7 ed ee :2 ed e6
:2 e5 :16 f3 :2 f4 :7 f3
:11 f6 :30 f7 :3 f6 :11 f9
:b fa :3 f9 :7 fc fd
:2 fc f4 :2 f3 :11 100
:2 101 :2 100 :4 101 :3 100
:9 102 :c 103 :9 105 106
:2 107 :4 108 105 :5 109
104 10b :5 10c :3 10b
:2 10d :5 10f :11 110 :24 111
:3 110 :2 10e :6 10d 10a
:3 103 :a 116 :8 118 119
118 :5 11a 117 11c
:5 11d :3 11c :2 11e :5 120
:11 121 :24 122 :3 121 :2 11f
:6 11e 11b :6 116 :c 128
:3 129 :3 128 103 :9 12d
12e :2 12f :4 130 12d
:5 131 12c 133 :5 134
:3 133 :2 135 :5 137 :11 138
:24 139 :3 138 :2 136 :6 135
132 :3 12b :a 13e :8 140
141 140 :5 142 13f
144 :5 145 :3 144 :2 146
:5 148 :11 149 :24 14a :3 149
:2 147 :6 146 143 :6 13e
:3 150 12b :3 103 :b 153
:7 155 :2 156 155 :6 157
:5 158 157 :5 15a 159
:3 157 154 15d :5 15e
:3 15d :2 15f :5 161 :11 162
:24 163 :3 162 :2 160 :6 15f
15c :4 153 :7 169 :2 16a
169 :6 16b :5 16c 16b
:5 16e 16d :3 16b 168
171 :5 172 :3 171 :2 173
:5 175 :11 176 :24 177 :3 176
:2 174 :6 173 170 :4 167
:3 153 :a 17d :6 17e :3 17d
102 :3 182 183 :4 184
185 182 :5 186 181
188 :5 189 :3 188 :2 18a
:5 18c :11 18d :24 18e :3 18d
:2 18b :6 18a 187 :3 180
:9 194 :11 195 :30 196 :3 195
:11 198 :b 199 :3 198 :7 19b
19c :2 19b :3 194 :c 19f
:9 1a1 1a2 :2 1a3 :4 1a4
1a1 :5 1a5 1a0 1a7
:5 1a8 :3 1a7 :2 1a9 :5 1ab
:11 1ac :24 1ad :3 1ac :2 1aa
:6 1a9 1a6 :3 19f :a 1b2
:8 1b4 1b5 1b4 :5 1b6
1b3 1b8 :5 1b9 :3 1b8
:2 1ba :5 1bc :11 1bd :24 1be
:3 1bd :2 1bb :6 1ba 1b7
:6 1b2 19f :9 1c5 1c6
:2 1c7 :4 1c8 1c5 :5 1c9
1c4 1cb :5 1cc :3 1cb
:2 1cd :5 1cf :11 1d0 :24 1d1
:3 1d0 :2 1ce :6 1cd 1ca
:3 1c3 :a 1d6 :8 1d8 1d9
1d8 :5 1da 1d7 1dc
:5 1dd :3 1dc :2 1de :5 1e0
:11 1e1 :24 1e2 :3 1e1 :2 1df
:6 1de 1db :6 1d6 1c3
:3 19f :7 1e9 :8 1ea 1eb
:3 1e9 :9 1ed :6 1ee 1ed
:11 1f0 :2e 1f1 :3 1f0 :11 1f3
:b 1f4 :3 1f3 :c 1f6 1ef
:3 1ed 180 :3 102 1f9
101 :1a 1f9 :3 1fa :d 1fb
:11 1fc :20 1fd :3 1fc :11 1ff
:b 200 :3 1ff :9 202 203
1fb :d 203 :3 204 203
:3 1fb :3 208 :2 20a 20b
:6 20c :3 20d :4 20e :4 20f
:4 210 20a :5 211 209
213 :5 214 :3 213 :2 215
:5 217 :11 218 :24 219 :3 218
:2 216 :6 215 212 :3 1f9
:9 21e :11 21f :30 220 :3 21f
:11 222 :b 223 :3 222 :7 225
226 :2 225 :3 21e :3 22b
:5 22c :8 22d :6 22e :6 22f
22b :5 230 22a 232
:5 233 :3 232 :2 234 :5 236
:11 237 :24 238 :3 237 :2 235
:6 234 231 :3 1f9 :9 23e
:11 23f :30 240 :3 23f :11 242
:b 243 :3 242 :a 245 :3 23e
:3 248 :7 24e :2 24f :5 24e
1f9 101 :2 100 :9 253
:6 254 :3 253 :11 257 :2e 258
:3 257 :11 25a :b 25b :3 25a
:c 25d 25e c4 :c 25e
:3 25f :3 260 :10 262 :3 263
:2 262 :4 263 :3 262 :11 264
:30 265 :3 264 :11 267 :b 268
:3 267 :7 26a 26b :2 26a
263 :2 262 :3 26f :3 270
:5 271 26f :5 272 26e
274 :5 275 :3 274 :2 276
:5 278 :11 279 :24 27a :3 279
:2 277 :6 276 273 :3 25e
:a 280 :3 281 280 :3 283
282 :3 280 :3 286 :3 287
:11 28f :2 290 :2 28f :4 290
:3 28f :e 290 :2 28f :c 291
:4 293 294 :4 295 293
:5 296 292 298 :5 299
:3 298 :2 29a :5 29c :11 29d
:24 29e :3 29d :2 29b :6 29a
297 :3 291 :3 2f2 :3 2f3
:3 2f4 2f6 291 :c 2f6
:8 2f8 2f9 2fa 2f8
:5 2fb 2f7 2fd :5 2fe
:3 2fd :2 2ff :5 301 :11 302
:24 303 :3 302 :2 300 :6 2ff
2fc :3 2f6 :9 308 :6 30a
:8 30b :4 30c :3 30b :12 30e
:3 310 :8 311 310 :6 312
:5 313 312 :5 315 314
:3 312 30f 318 :5 319
:3 318 :2 31a :5 31c :11 31d
:24 31e :3 31d :2 31b :6 31a
317 :6 30e :a 324 :3 325
324 :3 327 326 :3 324
:9 32a :5 32b :4 32c :3 32a
:a 32f :6 330 331 :2 332
32f :5 333 32e 335
:5 336 :3 335 :2 337 :5 339
:11 33a :24 33b :3 33a :2 338
:6 337 334 :3 2f6 :3 340
:8 341 :4 342 :3 341 :6 344
:5 346 :11 347 :30 348 :3 347
:a 34a 34b 2f6 291
:d 34b :f 34c :3 34d 34e
34b 291 :9 34e :3 34f
:3 350 :2 353 354 :2 355
353 :5 356 352 358
:5 359 :3 358 :2 35a :5 35c
:11 35d :24 35e :3 35d :2 35b
:6 35a 357 :3 34e 362
34e 291 :d 362 :f 363
:3 364 :3 365 :9 367 :2 368
:2 369 :2 368 :2 369 :2 368
:2 369 :2 368 :2 36a :2 368
:2 36a :2 368 :2 36b :2 368
:9 36b :3 368 :5 36d 367
:2 36f :2 370 :2 36f :2 370
:2 36f :2 370 :2 36f :2 370
:2 36f :2 371 :2 36f :2 372
:2 36f :9 372 :3 36f :5 374
36e :3 367 377 :9 378
:4 379 :3 37a :5 37b 37a
:5 37d 37c :3 37a 378
:4 380 :3 381 :5 382 381
:5 384 383 :3 381 37f
:3 378 :a 388 :2 389 :3 388
377 38b 362 :9 38d
:4 38f :5 390 38e 392
:5 393 :3 392 :2 394 :5 396
:11 397 :24 398 :3 397 :2 395
:6 394 391 :4 38d :4 39e
:5 39f 39d 3a1 :5 3a2
:3 3a1 :2 3a3 :5 3a5 :11 3a6
:24 3a7 :3 3a6 :2 3a4 :6 3a3
3a0 :4 39c :3 38d :5 3ae
:9 3af :11 3b0 :30 3b1 :3 3b0
:11 3b3 :b 3b4 :3 3b3 :a 3b6
3af :11 3b8 :30 3b9 :3 3b8
:11 3bb :b 3bc :3 3bb :a 3be
3b7 :3 3af :3 3ae :8 3c3
3c4 3c5 3c3 :5 3c6
3c2 3c8 :5 3c9 :3 3c8
:2 3ca :5 3cc :11 3cd :24 3ce
:3 3cd :2 3cb :6 3ca 3c7
:3 362 :3 3d3 :3 3d4 362
:3 291 3d6 290 :d 3d6
:d 3d7 :11 3d8 :30 3d9 :3 3d8
:11 3db :b 3dc :3 3db :a 3de
:3 3d7 :9 3e1 :3 3e2 3e1
:16 3e4 :11 3e5 :20 3e6 :3 3e5
:11 3e8 :b 3e9 :3 3e8 :9 3eb
:3 3e4 :3 3ee 3e3 :3 3e1
:5 3fe :2 400 401 :2 402
400 :5 403 3ff 405
:5 406 :3 405 :2 407 :5 409
:11 40a :24 40b :3 40a :2 408
:6 407 404 :3 3fe :11 410
:2 411 :7 410 :11 412 :30 413
:3 412 :11 415 :b 416 :3 415
:7 418 419 :2 418 411
:2 410 :3 3fe :3 457 :3 458
:9 459 :3 45a :3 459 :4 460
:4 461 :8 462 :2 463 460
:6 464 :5 465 464 :5 467
466 :3 464 45f 46a
:5 46b :3 46a :2 46c :5 46e
:11 46f :24 470 :3 46f :2 46d
:6 46c 469 :3 3d6 :9 475
:3 476 :3 475 3d6 290
:2 28f :5 47d :6 47e :8 47f
:4 480 :3 47f :12 482 :7 483
:3 484 :3 483 :3 482 :5 488
:8 489 488 :a 48b :8 48c
48b :8 48e 48d :3 48b
48a :3 488 :5 492 :9 493
:10 494 :8 497 498 499
497 :5 49a 496 49c
:5 49d :3 49c :2 49e :5 4a0
:11 4a1 :24 4a2 :3 4a1 :2 49f
:6 49e 49b :4 493 :8 4a8
4a9 4aa 4a8 :5 4ab
4a7 4ad :5 4ae :3 4ad
:2 4af :5 4b1 :11 4b2 :24 4b3
:3 4b2 :2 4b0 :6 4af 4ac
:4 4a6 :3 493 :f 4b9 :3 492
:3 4be :7 4bf :b 4c0 :3 4bf
:9 4c3 :11 4c4 :30 4c5 :3 4c4
:11 4c7 :b 4c8 :3 4c7 :a 4ca
4cb 4c3 :5 4cb 4cc
4cd :2 4ce 4cc :a 4d1
:8 4d2 :2 4d3 :2 4d4 4d1
:6 4d5 :5 4d6 4d5 :5 4d8
4d7 :3 4d5 4d0 4db
:5 4dc :3 4db :2 4dd :5 4df
:11 4e0 :24 4e1 :3 4e0 :2 4de
:6 4dd 4da :4 4cb :3 4c3
:6 4e7 :11 4e8 :2e 4e9 :3 4e8
:11 4eb :b 4ec :3 4eb :c 4ee
:3 47d :d 4f1 :2 4f4 4f5
:2 4f6 :2 4f7 :2 4f8 :3 4f9
4f4 :5 4fa 4f3 4fc
:5 4fd :3 4fc :2 4fe :5 500
:11 501 :24 502 :3 501 :2 4ff
:6 4fe 4fb :3 4f1 :9 507
:11 508 :30 509 :3 508 :11 50b
:b 50c :3 50b :7 50e 50f
:2 50e :3 507 :3 512 :7 514
:d 515 :6 516 :8 517 :4 518
:3 517 :5 51a :8 51b 51a
:a 51d :8 51e 51d :8 520
51f :3 51d 51c :3 51a
:3 524 :5 525 :9 526 :10 527
:8 529 52a 52b 529
:5 52c 528 52e :5 52f
:3 52e :2 530 :5 532 :11 533
:24 534 :3 533 :2 531 :6 530
52d :4 526 :8 53a 53b
53c 53a :5 53d 539
53f :5 540 :3 53f :2 541
:5 543 :11 544 :24 545 :3 544
:2 542 :6 541 53e :4 538
:3 526 :9 54b :3 54d 54e
:8 54f 54d :5 550 54c
552 :5 553 :3 552 :2 554
:5 556 :11 557 :24 558 :3 557
:2 555 :6 554 551 :6 54b
:3 525 :17 55f :3 561 :8 564
:6 565 :3 566 :2 567 564
:6 568 :5 569 568 :5 56b
56a :3 568 563 56e
:5 56f :3 56e :2 570 :5 572
:11 573 :24 574 :3 573 :2 571
:6 570 56d :3 515 :a 579
:3 57b :8 57c :6 57d :3 57c
:9 580 :11 581 :2e 582 :3 581
:11 584 :b 585 :3 584 :c 587
:3 580 :3 579 :7 58b 515
:16 58d :2 58e :3 58d :2 58e
:5 58d 58c :3 515 :7 594
:4 595 :4 596 :4 597 :4 598
:7 599 :3 594 :11 59b :2e 59c
:3 59b :11 59e :b 59f :3 59e
:c 5a1 5a2 4f1 :d 5a2
:16 5a3 :11 5a4 :30 5a5 :3 5a4
:11 5a7 :b 5a8 :3 5a7 :a 5aa
:3 5a3 :3 5cf :6 5d0 :2 5d1
5cf :6 5d2 :5 5d3 5d2
:5 5d5 5d4 :3 5d2 5ce
5d8 :5 5d9 :3 5d8 :2 5da
:5 5dc :11 5dd :24 5de :3 5dd
:2 5db :6 5da 5d7 :3 5a2
:7 5e8 :4 5e9 5e8 :6 5ea
:5 5eb 5ea :5 5ed 5ec
:3 5ea 5e7 5f0 :5 5f1
:3 5f0 :2 5f2 :5 5f4 :11 5f5
:24 5f6 :3 5f5 :2 5f3 :6 5f2
5ef :4 5a2 :3 4f1 :6 5fe
:11 5ff :2e 600 :3 5ff :11 602
:b 603 :3 602 :c 605 25e
c4 :8 608 :2 609 :2 60a
:2 60b :4 60c :4 60d 60e
60f 610 :2 611 :5 612
608 :5 613 607 615
:5 616 :3 615 :2 617 :5 619
:11 61a :24 61b :3 61a :2 618
:6 617 614 :3 606 :d 621
:11 622 :30 623 :3 622 :11 625
:b 626 :3 625 :7 628 629
:2 628 :3 621 :b 62c :5 62d
:11 62e :30 62f :3 62e :11 631
:b 632 :3 631 :a 634 :3 62c
606 :3 c4 :3 a1 :3 639
:3 63a :3 63b :5 63d :b 63e
:11 640 :30 641 :3 640 :a 643
6a :3 646 :11 647 :b 648
:17 649 :3 647 :2 64b :3 64c
:3 64d 64e 64c 64b
64c :3 64f :3 646 :2 650
:5 652 :11 653 :b 654 :18 655
:3 653 :2 657 658 :3 659
65a 658 657 658
:3 65b :2 651 :6 650 645
:7 1 
84d8
4
:3 0 1 :3 0 2
:a 0 2623 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 4
:3 0 5 :7 0 9
8 :3 0 7 :3 0
6 :7 0 d c
:3 0 d 7c 0
b 7 :3 0 8
:7 0 11 10 :3 0
a :3 0 9 :7 0
15 14 :3 0 11
a2 0 f a
:3 0 b :7 0 19
18 :3 0 a :3 0
c :7 0 1d 1c
:3 0 15 :2 0 13
a :3 0 d :7 0
21 20 :3 0 4
:3 0 e :7 0 25
24 :3 0 f :3 0
10 :3 0 11 :2 0
4 29 2a 0
27 2b 0 2623
2 2c :2 0 13
:2 0 21 4 :3 0
13 :2 0 1f 2f
31 :6 0 34 32
0 2621 0 12
:6 0 16 :2 0 25
4 :3 0 23 36
38 :6 0 3b 39
0 2621 0 14
:6 0 16 :2 0 29
4 :3 0 27 3d
3f :6 0 42 40
0 2621 0 15
:6 0 13 :2 0 2d
4 :3 0 2b 44
46 :6 0 49 47
0 2621 0 17
:6 0 16 :2 0 31
4 :3 0 2f 4b
4d :6 0 50 4e
0 2621 0 18
:6 0 13 :2 0 35
4 :3 0 33 52
54 :6 0 57 55
0 2621 0 19
:6 0 13 :2 0 39
4 :3 0 37 59
5b :6 0 5e 5c
0 2621 0 1a
:6 0 13 :2 0 3d
4 :3 0 3b 60
62 :6 0 65 63
0 2621 0 1b
:6 0 13 :2 0 41
4 :3 0 3f 67
69 :6 0 6c 6a
0 2621 0 1c
:6 0 13 :2 0 45
4 :3 0 43 6e
70 :6 0 73 71
0 2621 0 1d
:6 0 13 :2 0 49
4 :3 0 47 75
77 :6 0 7a 78
0 2621 0 1e
:6 0 13 :2 0 4d
4 :3 0 4b 7c
7e :6 0 81 7f
0 2621 0 1f
:6 0 53 25b 0
51 4 :3 0 4f
83 85 :6 0 88
86 0 2621 0
20 :6 0 57 28f
0 55 7 :3 0
8a :7 0 8d 8b
0 2621 0 21
:6 0 7 :3 0 8f
:7 0 92 90 0
2621 0 22 :6 0
5b 2c3 0 59
7 :3 0 94 :7 0
97 95 0 2621
0 23 :6 0 7
:3 0 99 :7 0 9c
9a 0 2621 0
24 :6 0 5f 2f7
0 5d 7 :3 0
9e :7 0 a1 9f
0 2621 0 25
:6 0 7 :3 0 a3
:7 0 a6 a4 0
2621 0 26 :6 0
63 32b 0 61
28 :3 0 a8 :7 0
ab a9 0 2621
0 27 :6 0 28
:3 0 ad :7 0 b0
ae 0 2621 0
29 :6 0 2c :2 0
67 28 :3 0 b2
:7 0 b5 b3 0
2621 0 2a :6 0
4 :3 0 2c :2 0
65 b7 b9 :6 0
bc ba 0 2621
0 2b :6 0 2f
:2 0 6b 4 :3 0
69 be c0 :6 0
c3 c1 0 2621
0 2d :6 0 2f
:2 0 6d a :3 0
c5 :7 0 c9 c6
c7 2621 0 2e
:6 0 2f :2 0 6f
a :3 0 cb :7 0
cf cc cd 2621
0 30 :6 0 73
3d4 0 71 a
:3 0 d1 :7 0 d5
d2 d3 2621 0
31 :6 0 2f :2 0
75 28 :3 0 d7
:7 0 da d8 0
2621 0 32 :6 0
28 :3 0 dc :7 0
df dd 0 2621
0 33 :6 0 2f
:2 0 77 a :3 0
e1 :7 0 e5 e2
e3 2621 0 34
:6 0 2f :2 0 79
a :3 0 e7 :7 0
eb e8 e9 2621
0 35 :6 0 2f
:2 0 7b a :3 0
ed :7 0 f1 ee
ef 2621 0 36
:6 0 2f :2 0 7d
a :3 0 f3 :7 0
f7 f4 f5 2621
0 37 :6 0 2f
:2 0 7f a :3 0
f9 :7 0 fd fa
fb 2621 0 38
:6 0 2f :2 0 81
a :3 0 ff :7 0
103 100 101 2621
0 39 :6 0 2f
:2 0 83 a :3 0
105 :7 0 109 106
107 2621 0 3a
:6 0 2f :2 0 85
a :3 0 10b :7 0
10f 10c 10d 2621
0 3b :6 0 2f
:2 0 87 a :3 0
111 :7 0 115 112
113 2621 0 3c
:6 0 13 :2 0 89
a :3 0 117 :7 0
11b 118 119 2621
0 3d :6 0 13
:2 0 8d 4 :3 0
8b 11d 11f :6 0
122 120 0 2621
0 3e :6 0 13
:2 0 91 4 :3 0
8f 124 126 :6 0
129 127 0 2621
0 3f :6 0 2f
:2 0 95 4 :3 0
93 12b 12d :6 0
130 12e 0 2621
0 40 :6 0 2f
:2 0 97 a :3 0
132 :7 0 136 133
134 2621 0 41
:6 0 2f :2 0 99
a :3 0 138 :7 0
13c 139 13a 2621
0 42 :6 0 2f
:2 0 9b a :3 0
13e :7 0 142 13f
140 2621 0 43
:6 0 2f :2 0 9d
a :3 0 144 :7 0
148 145 146 2621
0 44 :6 0 2f
:2 0 9f a :3 0
14a :7 0 14e 14b
14c 2621 0 45
:6 0 2f :2 0 a1
a :3 0 150 :7 0
154 151 152 2621
0 46 :6 0 2f
:2 0 a3 a :3 0
156 :7 0 15a 157
158 2621 0 47
:6 0 2f :2 0 a5
a :3 0 15c :7 0
160 15d 15e 2621
0 48 :6 0 2f
:2 0 a7 a :3 0
162 :7 0 166 163
164 2621 0 49
:6 0 2f :2 0 a9
a :3 0 168 :7 0
16c 169 16a 2621
0 4a :6 0 2f
:2 0 ab a :3 0
16e :7 0 172 16f
170 2621 0 4b
:6 0 2f :2 0 ad
a :3 0 174 :7 0
178 175 176 2621
0 4c :6 0 2f
:2 0 af a :3 0
17a :7 0 17e 17b
17c 2621 0 4d
:6 0 2f :2 0 b1
a :3 0 180 :7 0
184 181 182 2621
0 4e :6 0 2f
:2 0 b3 a :3 0
186 :7 0 18a 187
188 2621 0 4f
:6 0 2f :2 0 b5
a :3 0 18c :7 0
190 18d 18e 2621
0 50 :6 0 2f
:2 0 b7 a :3 0
192 :7 0 196 193
194 2621 0 51
:6 0 2f :2 0 b9
a :3 0 198 :7 0
19c 199 19a 2621
0 52 :6 0 2f
:2 0 bb a :3 0
19e :7 0 1a2 19f
1a0 2621 0 53
:6 0 2f :2 0 bd
a :3 0 1a4 :7 0
1a8 1a5 1a6 2621
0 54 :6 0 2f
:2 0 bf a :3 0
1aa :7 0 1ae 1ab
1ac 2621 0 55
:6 0 2f :2 0 c1
a :3 0 1b0 :7 0
1b4 1b1 1b2 2621
0 56 :6 0 2f
:2 0 c3 a :3 0
1b6 :7 0 1ba 1b7
1b8 2621 0 57
:6 0 5a :2 0 c5
a :3 0 1bc :7 0
1c0 1bd 1be 2621
0 58 :6 0 2c
:2 0 c9 4 :3 0
c7 1c2 1c4 :6 0
1c7 1c5 0 2621
0 59 :6 0 2f
:2 0 cd 4 :3 0
cb 1c9 1cb :6 0
1ce 1cc 0 2621
0 5b :6 0 13
:2 0 cf a :3 0
1d0 :7 0 1d4 1d1
1d2 2621 0 5c
:6 0 2f :2 0 d3
4 :3 0 d1 1d6
1d8 :6 0 1db 1d9
0 2621 0 5d
:6 0 2f :2 0 d5
a :3 0 1dd :7 0
1e1 1de 1df 2621
0 5e :6 0 2f
:2 0 d7 a :3 0
1e3 :7 0 1e7 1e4
1e5 2621 0 5f
:6 0 2f :2 0 d9
a :3 0 1e9 :7 0
1ed 1ea 1eb 2621
0 60 :6 0 2f
:2 0 db 62 :3 0
1ef :7 0 1f3 1f0
1f1 2621 0 61
:6 0 2f :2 0 dd
62 :3 0 1f5 :7 0
1f9 1f6 1f7 2621
0 63 :6 0 2f
:2 0 df 62 :3 0
1fb :7 0 1ff 1fc
1fd 2621 0 64
:6 0 2f :2 0 e1
62 :3 0 201 :7 0
205 202 203 2621
0 65 :6 0 2f
:2 0 e3 62 :3 0
207 :7 0 20b 208
209 2621 0 66
:6 0 2f :2 0 e5
62 :3 0 20d :7 0
211 20e 20f 2621
0 67 :6 0 2f
:2 0 e7 62 :3 0
213 :7 0 217 214
215 2621 0 68
:6 0 6b :2 0 e9
62 :3 0 219 :7 0
21d 21a 21b 2621
0 69 :6 0 ef
9db 0 ed 4
:3 0 eb 21f 221
:6 0 224 222 0
2621 0 6a :6 0
230 231 0 f1
a :3 0 226 :7 0
229 227 0 2621
0 6c :6 0 a
:3 0 22b :7 0 22e
22c 0 2621 0
6d :6 0 237 238
0 f3 10 :3 0
11 :2 0 4 232
:7 0 235 233 0
2621 0 6e :6 0
23e 23f 0 f5
10 :3 0 11 :2 0
4 239 :7 0 23c
23a 0 2621 0
6f :6 0 244 245
0 f7 10 :3 0
11 :2 0 4 240
:7 0 243 241 0
2621 0 70 :6 0
71 :3 0 72 :3 0
71 :3 0 73 :3 0
247 248 0 2
:4 0 f9 249 24b
246 24d 24c :2 0
74 :3 0 75 :3 0
24f 250 0 76
:4 0 71 :3 0 77
:3 0 253 254 0
2 :4 0 fb 255
257 78 :4 0 79
:2 0 7a :3 0 fd
25a 25c :3 0 7b
:4 0 79 :2 0 7c
:4 0 100 25f 261
:3 0 79 :2 0 3
:3 0 103 263 265
:3 0 79 :2 0 7c
:4 0 106 267 269
:3 0 79 :2 0 7d
:4 0 109 26b 26d
:3 0 79 :2 0 7c
:4 0 10c 26f 271
:3 0 79 :2 0 5
:3 0 10f 273 275
:3 0 79 :2 0 7c
:4 0 112 277 279
:3 0 79 :2 0 7d
:4 0 115 27b 27d
:3 0 79 :2 0 7e
:3 0 7f :3 0 7c
:4 0 79 :2 0 6
:3 0 118 283 285
:3 0 79 :2 0 7c
:4 0 11b 287 289
:3 0 80 :4 0 11e
281 28c 81 :4 0
121 280 28f 124
27f 291 :3 0 79
:2 0 7d :4 0 127
293 295 :3 0 79
:2 0 7e :3 0 7f
:3 0 7c :4 0 79
:2 0 8 :3 0 12a
29b 29d :3 0 79
:2 0 7c :4 0 12d
29f 2a1 :3 0 80
:4 0 130 299 2a4
81 :4 0 133 298
2a7 136 297 2a9
:3 0 79 :2 0 7d
:4 0 139 2ab 2ad
:3 0 79 :2 0 7e
:3 0 82 :3 0 9
:3 0 13c 2b1 2b3
81 :4 0 13e 2b0
2b6 141 2af 2b8
:3 0 79 :2 0 7d
:4 0 144 2ba 2bc
:3 0 79 :2 0 7e
:3 0 82 :3 0 b
:3 0 147 2c0 2c2
81 :4 0 149 2bf
2c5 14c 2be 2c7
:3 0 79 :2 0 7d
:4 0 14f 2c9 2cb
:3 0 79 :2 0 7e
:3 0 82 :3 0 c
:3 0 152 2cf 2d1
81 :4 0 154 2ce
2d4 157 2cd 2d6
:3 0 79 :2 0 7d
:4 0 15a 2d8 2da
:3 0 79 :2 0 7e
:3 0 82 :3 0 d
:3 0 15d 2de 2e0
81 :4 0 15f 2dd
2e3 162 2dc 2e5
:3 0 79 :2 0 7d
:4 0 165 2e7 2e9
:3 0 79 :2 0 7c
:4 0 168 2eb 2ed
:3 0 79 :2 0 e
:3 0 16b 2ef 2f1
:3 0 79 :2 0 7c
:4 0 16e 2f3 2f5
:3 0 79 :2 0 83
:4 0 171 2f7 2f9
:3 0 174 251 2fb
:2 0 2fd 179 2fe
24e 2fd 0 2ff
17b 0 2581 71
:3 0 72 :3 0 300
301 0 71 :3 0
84 :3 0 303 304
0 2c :2 0 2
:4 0 85 :2 0 86
:2 0 2c :2 0 17d
309 30b :3 0 180
305 30d 302 30f
30e :2 0 74 :3 0
87 :3 0 311 312
0 76 :4 0 88
:4 0 79 :2 0 7a
:3 0 184 316 318
:3 0 187 313 31a
:2 0 31c 18a 31d
310 31c 0 31e
18c 0 2581 4b
:3 0 2f :2 0 31f
320 0 2581 1e
:3 0 89 :4 0 322
323 0 2581 40
:3 0 e :3 0 325
326 0 2581 1b
:3 0 5 :3 0 328
329 0 2581 1c
:3 0 3 :3 0 32b
32c 0 2581 8a
:3 0 8b :3 0 8a
:3 0 8c :3 0 8a
:3 0 8d :3 0 8a
:3 0 8e :3 0 8a
:3 0 8f :3 0 8a
:3 0 90 :3 0 8a
:3 0 91 :3 0 8a
:3 0 92 :3 0 8a
:3 0 93 :3 0 1a
:3 0 37 :3 0 14
:3 0 35 :3 0 38
:3 0 3d :3 0 15
:3 0 1f :3 0 3c
:3 0 94 :3 0 8a
:3 0 8a :3 0 8b
:3 0 1c :4 0 95
1 :8 0 354 10
:3 0 96 :3 0 34f
350 0 97 :3 0
351 352 0 354
18e 3a7 98 :3 0
10 :3 0 96 :3 0
356 357 0 97
:3 0 358 359 0
35b 191 35d 193
35c 35b :2 0 3a5
99 :3 0 10 :3 0
96 :3 0 360 361
0 97 :3 0 362
363 0 39d 71
:3 0 72 :3 0 365
366 0 71 :3 0
84 :3 0 368 369
0 9a :2 0 2
:4 0 9b :2 0 86
:2 0 2c :2 0 195
36e 370 :3 0 198
36a 372 367 374
373 :2 0 74 :3 0
87 :3 0 376 377
0 76 :4 0 9c
:4 0 79 :2 0 7a
:3 0 19c 37b 37d
:3 0 79 :2 0 9d
:4 0 19f 37f 381
:3 0 79 :2 0 9e
:4 0 1a2 383 385
:3 0 79 :2 0 97
:3 0 1a5 387 389
:3 0 79 :2 0 9f
:4 0 1a8 38b 38d
:3 0 79 :2 0 a0
:3 0 a1 :3 0 2c
:2 0 a2 :2 0 1ab
390 394 1af 38f
396 :3 0 1b2 378
398 :2 0 39a 1b5
39b 375 39a 0
39c 1b7 0 39d
1b9 3a0 :3 0 3a0
0 3a0 39f 39d
39e :6 0 3a2 2
:3 0 1bc 3a4 1be
3a3 3a2 :2 0 3a5
1c0 :2 0 3a7 0
3a7 3a6 354 3a5
:6 0 2581 1 :3 0
10 :3 0 96 :3 0
3a9 3aa 0 10
:3 0 a3 :2 0 a4
:3 0 3ac 3ae 0
1c5 3ad 3b0 :3 0
3b1 :2 0 8b :3 0
a5 :3 0 92 :3 0
91 :3 0 8f :3 0
a6 :3 0 a7 :3 0
a8 :3 0 a9 :3 0
aa :3 0 ab :3 0
ac :3 0 ad :3 0
ae :3 0 ad :3 0
ae :3 0 af :3 0
b0 :3 0 b1 :3 0
b2 :3 0 a6 :3 0
b3 :3 0 b4 :3 0
b5 :3 0 b6 :3 0
b7 :3 0 a0 :3 0
b1 :3 0 b7 :3 0
a0 :3 0 b2 :3 0
b8 :3 0 b9 :3 0
ba :3 0 bb :3 0
bc :3 0 bd :3 0
be :3 0 bf :3 0
c0 :3 0 c1 :3 0
c2 :3 0 c3 :3 0
c4 :3 0 1a :3 0
1d :3 0 1f :3 0
3f :3 0 2d :3 0
2b :3 0 33 :3 0
42 :3 0 48 :3 0
52 :3 0 53 :3 0
50 :3 0 44 :3 0
21 :3 0 46 :3 0
22 :3 0 23 :3 0
41 :3 0 43 :3 0
49 :3 0 27 :3 0
29 :3 0 2e :3 0
30 :3 0 61 :3 0
64 :3 0 63 :3 0
65 :3 0 66 :3 0
68 :3 0 67 :3 0
69 :3 0 45 :3 0
3e :3 0 56 :3 0
c5 :3 0 c6 :3 0
1c :4 0 c7 1
:8 0 40b 10 :3 0
96 :3 0 406 407
0 97 :3 0 408
409 0 40b 1c8
45e 98 :3 0 10
:3 0 96 :3 0 40d
40e 0 97 :3 0
40f 410 0 412
1cb 414 1cd 413
412 :2 0 45c 99
:3 0 10 :3 0 96
:3 0 417 418 0
97 :3 0 419 41a
0 454 71 :3 0
72 :3 0 41c 41d
0 71 :3 0 84
:3 0 41f 420 0
9a :2 0 2 :4 0
c8 :2 0 86 :2 0
2c :2 0 1cf 425
427 :3 0 1d2 421
429 41e 42b 42a
:2 0 74 :3 0 87
:3 0 42d 42e 0
76 :4 0 c9 :4 0
79 :2 0 7a :3 0
1d6 432 434 :3 0
79 :2 0 9d :4 0
1d9 436 438 :3 0
79 :2 0 9e :4 0
1dc 43a 43c :3 0
79 :2 0 97 :3 0
1df 43e 440 :3 0
79 :2 0 9f :4 0
1e2 442 444 :3 0
79 :2 0 a0 :3 0
a1 :3 0 2c :2 0
a2 :2 0 1e5 447
44b 1e9 446 44d
:3 0 1ec 42f 44f
:2 0 451 1ef 452
42c 451 0 453
1f1 0 454 1f3
457 :3 0 457 0
457 456 454 455
:6 0 459 4 :3 0
1f6 45b 1f8 45a
459 :2 0 45c 1fa
:2 0 45e 0 45e
45d 40b 45c :6 0
2517 1 :3 0 10
:3 0 96 :3 0 460
461 0 10 :3 0
a3 :2 0 a4 :3 0
463 465 0 1ff
464 467 :3 0 468
:2 0 92 :3 0 ca
:3 0 cb :3 0 cc
:3 0 cd :3 0 ce
:3 0 cf :3 0 d0
:3 0 b0 :3 0 d1
:3 0 d2 :3 0 d3
:3 0 d4 :3 0 7e
:3 0 d5 :3 0 d6
:3 0 1f :3 0 4b
:3 0 4c :3 0 22
:3 0 23 :3 0 4f
:3 0 4e :3 0 4d
:3 0 46 :3 0 41
:3 0 45 :3 0 55
:3 0 26 :3 0 d7
:3 0 8d :3 0 1c
:4 0 d8 1 :8 0
490 10 :3 0 96
:3 0 48b 48c 0
97 :3 0 48d 48e
0 490 202 4e3
98 :3 0 10 :3 0
96 :3 0 492 493
0 97 :3 0 494
495 0 497 205
499 207 498 497
:2 0 4e1 99 :3 0
10 :3 0 96 :3 0
49c 49d 0 97
:3 0 49e 49f 0
4d9 71 :3 0 72
:3 0 4a1 4a2 0
71 :3 0 84 :3 0
4a4 4a5 0 9a
:2 0 2 :4 0 d9
:2 0 86 :2 0 2c
:2 0 209 4aa 4ac
:3 0 20c 4a6 4ae
4a3 4b0 4af :2 0
74 :3 0 87 :3 0
4b2 4b3 0 76
:4 0 da :4 0 79
:2 0 7a :3 0 210
4b7 4b9 :3 0 79
:2 0 9d :4 0 213
4bb 4bd :3 0 79
:2 0 9e :4 0 216
4bf 4c1 :3 0 79
:2 0 97 :3 0 219
4c3 4c5 :3 0 79
:2 0 9f :4 0 21c
4c7 4c9 :3 0 79
:2 0 a0 :3 0 a1
:3 0 2c :2 0 a2
:2 0 21f 4cc 4d0
223 4cb 4d2 :3 0
226 4b4 4d4 :2 0
4d6 229 4d7 4b1
4d6 0 4d8 22b
0 4d9 22d 4dc
:3 0 4dc 0 4dc
4db 4d9 4da :6 0
4de 6 :3 0 230
4e0 232 4df 4de
:2 0 4e1 234 :2 0
4e3 0 4e3 4e2
490 4e1 :6 0 fdf
1 :3 0 10 :3 0
96 :3 0 4e5 4e6
0 10 :3 0 a3
:2 0 a4 :3 0 4e8
4ea 0 239 4e9
4ec :3 0 4ed :2 0
71 :3 0 72 :3 0
4ef 4f0 0 71
:3 0 84 :3 0 4f2
4f3 0 13 :2 0
2 :4 0 db :2 0
86 :2 0 2c :2 0
23c 4f8 4fa :3 0
23f 4f4 4fc 4f1
4fe 4fd :2 0 74
:3 0 87 :3 0 500
501 0 76 :4 0
dc :4 0 79 :2 0
7a :3 0 243 505
507 :3 0 79 :2 0
9d :4 0 246 509
50b :3 0 79 :2 0
dd :4 0 249 50d
50f :3 0 79 :2 0
7e :3 0 82 :3 0
de :3 0 df :3 0
514 515 0 24c
513 517 81 :4 0
24e 512 51a 251
511 51c :3 0 79
:2 0 e0 :4 0 254
51e 520 :3 0 79
:2 0 7c :4 0 257
522 524 :3 0 79
:2 0 1c :3 0 25a
526 528 :3 0 79
:2 0 7c :4 0 25d
52a 52c :3 0 260
502 52e :2 0 530
263 531 4ff 530
0 532 265 0
55c 71 :3 0 72
:3 0 533 534 0
71 :3 0 84 :3 0
536 537 0 2c
:2 0 2 :4 0 e1
:2 0 86 :2 0 2c
:2 0 267 53c 53e
:3 0 26a 538 540
535 542 541 :2 0
74 :3 0 87 :3 0
544 545 0 76
:4 0 e2 :4 0 79
:2 0 7a :3 0 26e
549 54b :3 0 271
546 54d :2 0 54f
274 550 543 54f
0 551 276 0
55c e3 :3 0 e4
:3 0 552 553 0
de :3 0 df :3 0
555 556 0 1c
:3 0 e5 :3 0 278
554 55a :2 0 55c
27c 55d 4ee 55c
0 55e 280 0
fdf 24 :3 0 6
:3 0 55f 560 0
fdf 25 :3 0 8
:3 0 562 563 0
fdf e6 :3 0 e7
:3 0 565 566 0
24 :3 0 25 :3 0
e8 :2 0 282 567
56b e9 :2 0 2f
:2 0 288 56d 56f
:3 0 570 :2 0 e6
:3 0 e7 :3 0 572
573 0 25 :3 0
23 :3 0 e8 :2 0
28b 574 578 ea
:2 0 2f :2 0 291
57a 57c :3 0 57d
:2 0 571 57f 57e
:2 0 71 :3 0 72
:3 0 581 582 0
71 :3 0 84 :3 0
584 585 0 13
:2 0 2 :4 0 eb
:2 0 86 :2 0 2c
:2 0 294 58a 58c
:3 0 297 586 58e
583 590 58f :2 0
74 :3 0 87 :3 0
592 593 0 76
:4 0 ec :4 0 79
:2 0 7a :3 0 29b
597 599 :3 0 79
:2 0 9d :4 0 29e
59b 59d :3 0 79
:2 0 dd :4 0 2a1
59f 5a1 :3 0 79
:2 0 7e :3 0 82
:3 0 de :3 0 ed
:3 0 5a6 5a7 0
2a4 5a5 5a9 81
:4 0 2a6 5a4 5ac
2a9 5a3 5ae :3 0
79 :2 0 e0 :4 0
2ac 5b0 5b2 :3 0
79 :2 0 7c :4 0
2af 5b4 5b6 :3 0
79 :2 0 1c :3 0
2b2 5b8 5ba :3 0
79 :2 0 7c :4 0
2b5 5bc 5be :3 0
2b8 594 5c0 :2 0
5c2 2bb 5c3 591
5c2 0 5c4 2bd
0 5ee 71 :3 0
72 :3 0 5c5 5c6
0 71 :3 0 84
:3 0 5c8 5c9 0
2c :2 0 2 :4 0
ee :2 0 86 :2 0
2c :2 0 2bf 5ce
5d0 :3 0 2c2 5ca
5d2 5c7 5d4 5d3
:2 0 74 :3 0 87
:3 0 5d6 5d7 0
76 :4 0 ef :4 0
79 :2 0 7a :3 0
2c6 5db 5dd :3 0
2c9 5d8 5df :2 0
5e1 2cc 5e2 5d5
5e1 0 5e3 2ce
0 5ee e3 :3 0
e4 :3 0 5e4 5e5
0 de :3 0 ed
:3 0 5e7 5e8 0
1c :3 0 e5 :3 0
2d0 5e6 5ec :2 0
5ee 2d4 5ef 580
5ee 0 5f0 2d8
0 fdf 26 :3 0
f0 :2 0 2da 5f2
5f3 :3 0 5f4 :2 0
5f5 :2 0 7e :3 0
41 :3 0 2f :2 0
2dc 5f7 5fa f1
:2 0 7e :3 0 46
:3 0 2f :2 0 2df
5fd 600 2e2 5fc
602 :3 0 603 :2 0
7e :3 0 ea :2 0
55 :3 0 2f :2 0
2e5 605 609 2ea
606 60b :3 0 60c
:2 0 5f6 60e 60d
:2 0 71 :3 0 72
:3 0 610 611 0
71 :3 0 84 :3 0
613 614 0 13
:2 0 2 :4 0 f2
:2 0 86 :2 0 2c
:2 0 2ed 619 61b
:3 0 2f0 615 61d
612 61f 61e :2 0
74 :3 0 87 :3 0
621 622 0 76
:4 0 f3 :4 0 79
:2 0 7a :3 0 2f4
626 628 :3 0 79
:2 0 9d :4 0 2f7
62a 62c :3 0 79
:2 0 dd :4 0 2fa
62e 630 :3 0 79
:2 0 7e :3 0 82
:3 0 de :3 0 f4
:3 0 635 636 0
2fd 634 638 81
:4 0 2ff 633 63b
302 632 63d :3 0
79 :2 0 e0 :4 0
305 63f 641 :3 0
79 :2 0 7c :4 0
308 643 645 :3 0
79 :2 0 1c :3 0
30b 647 649 :3 0
79 :2 0 7c :4 0
30e 64b 64d :3 0
311 623 64f :2 0
651 314 652 620
651 0 653 316
0 67d 71 :3 0
72 :3 0 654 655
0 71 :3 0 84
:3 0 657 658 0
2c :2 0 2 :4 0
f5 :2 0 86 :2 0
2c :2 0 318 65d
65f :3 0 31b 659
661 656 663 662
:2 0 74 :3 0 87
:3 0 665 666 0
76 :4 0 f6 :4 0
79 :2 0 7a :3 0
31f 66a 66c :3 0
322 667 66e :2 0
670 325 671 664
670 0 672 327
0 67d e3 :3 0
e4 :3 0 673 674
0 de :3 0 f4
:3 0 676 677 0
1c :3 0 e5 :3 0
329 675 67b :2 0
67d 32d 67e 60f
67d 0 67f 331
0 fdf e6 :3 0
e7 :3 0 680 681
0 24 :3 0 22
:3 0 e8 :2 0 333
682 686 ea :2 0
2f :2 0 339 688
68a :3 0 68b :2 0
e6 :3 0 e7 :3 0
68d 68e 0 25
:3 0 23 :3 0 e8
:2 0 33c 68f 693
ea :2 0 2f :2 0
342 695 697 :3 0
698 :2 0 68c 69a
699 :2 0 7e :3 0
4e :3 0 2f :2 0
345 69c 69f a3
:2 0 2f :2 0 34a
6a1 6a3 :3 0 e6
:3 0 e7 :3 0 6a5
6a6 0 24 :3 0
22 :3 0 e8 :2 0
34d 6a7 6ab ea
:2 0 2f :2 0 353
6ad 6af :3 0 e6
:3 0 f7 :3 0 f8
:3 0 e6 :3 0 f9
:3 0 fa :3 0 fb
:3 0 fc :3 0 24
:3 0 fd :3 0 8d
:3 0 1f :3 0 f8
:3 0 b7 :3 0 a0
:3 0 24 :4 0 fe
1 :8 0 6c7 10
:3 0 96 :3 0 6c2
6c3 0 97 :3 0
6c4 6c5 0 6c7
356 71a 98 :3 0
10 :3 0 96 :3 0
6c9 6ca 0 97
:3 0 6cb 6cc 0
6ce 359 6d0 35b
6cf 6ce :2 0 718
99 :3 0 10 :3 0
96 :3 0 6d3 6d4
0 97 :3 0 6d5
6d6 0 710 71
:3 0 72 :3 0 6d8
6d9 0 71 :3 0
84 :3 0 6db 6dc
0 9a :2 0 2
:4 0 ff :2 0 86
:2 0 2c :2 0 35d
6e1 6e3 :3 0 360
6dd 6e5 6da 6e7
6e6 :2 0 74 :3 0
87 :3 0 6e9 6ea
0 76 :4 0 100
:4 0 79 :2 0 7a
:3 0 364 6ee 6f0
:3 0 79 :2 0 9d
:4 0 367 6f2 6f4
:3 0 79 :2 0 9e
:4 0 36a 6f6 6f8
:3 0 79 :2 0 97
:3 0 36d 6fa 6fc
:3 0 79 :2 0 9f
:4 0 370 6fe 700
:3 0 79 :2 0 a0
:3 0 a1 :3 0 2c
:2 0 a2 :2 0 373
703 707 377 702
709 :3 0 37a 6eb
70b :2 0 70d 37d
70e 6e8 70d 0
70f 37f 0 710
381 713 :3 0 713
0 713 712 710
711 :6 0 715 8
:3 0 384 717 386
716 715 :2 0 718
388 :2 0 71a 0
71a 719 6c7 718
:6 0 79f 1 :3 0
10 :3 0 96 :3 0
71c 71d 0 10
:3 0 a3 :2 0 a4
:3 0 71f 721 0
38d 720 723 :3 0
724 :2 0 e6 :3 0
f7 :3 0 b7 :3 0
a0 :3 0 24 :3 0
e6 :3 0 f9 :3 0
24 :3 0 101 :4 0
102 1 :8 0 735
10 :3 0 96 :3 0
730 731 0 97
:3 0 732 733 0
735 390 788 98
:3 0 10 :3 0 96
:3 0 737 738 0
97 :3 0 739 73a
0 73c 393 73e
395 73d 73c :2 0
786 99 :3 0 10
:3 0 96 :3 0 741
742 0 97 :3 0
743 744 0 77e
71 :3 0 72 :3 0
746 747 0 71
:3 0 84 :3 0 749
74a 0 9a :2 0
2 :4 0 103 :2 0
86 :2 0 2c :2 0
397 74f 751 :3 0
39a 74b 753 748
755 754 :2 0 74
:3 0 87 :3 0 757
758 0 76 :4 0
104 :4 0 79 :2 0
7a :3 0 39e 75c
75e :3 0 79 :2 0
9d :4 0 3a1 760
762 :3 0 79 :2 0
9e :4 0 3a4 764
766 :3 0 79 :2 0
97 :3 0 3a7 768
76a :3 0 79 :2 0
9f :4 0 3aa 76c
76e :3 0 79 :2 0
a0 :3 0 a1 :3 0
2c :2 0 a2 :2 0
3ad 771 775 3b1
770 777 :3 0 3b4
759 779 :2 0 77b
3b7 77c 756 77b
0 77d 3b9 0
77e 3bb 781 :3 0
781 0 781 780
77e 77f :6 0 783
a :3 0 3be 785
3c0 784 783 :2 0
786 3c2 :2 0 788
0 788 787 735
786 :6 0 78a 1
:3 0 3c5 78b 725
78a 0 78c 3c7
0 79f e6 :3 0
e7 :3 0 78d 78e
0 24 :3 0 25
:3 0 e8 :2 0 3c9
78f 793 e9 :2 0
2f :2 0 3cf 795
797 :3 0 25 :4 0
799 79a 0 79c
3d2 79d 798 79c
0 79e 3d4 0
79f 3d6 880 e6
:3 0 f7 :3 0 f8
:3 0 e6 :3 0 f9
:3 0 fa :3 0 105
:3 0 fc :3 0 25
:3 0 fd :3 0 8d
:3 0 1f :3 0 f8
:3 0 b7 :3 0 a0
:3 0 25 :4 0 106
1 :8 0 7b6 10
:3 0 96 :3 0 7b1
7b2 0 97 :3 0
7b3 7b4 0 7b6
3da 809 98 :3 0
10 :3 0 96 :3 0
7b8 7b9 0 97
:3 0 7ba 7bb 0
7bd 3dd 7bf 3df
7be 7bd :2 0 807
99 :3 0 10 :3 0
96 :3 0 7c2 7c3
0 97 :3 0 7c4
7c5 0 7ff 71
:3 0 72 :3 0 7c7
7c8 0 71 :3 0
84 :3 0 7ca 7cb
0 9a :2 0 2
:4 0 107 :2 0 86
:2 0 2c :2 0 3e1
7d0 7d2 :3 0 3e4
7cc 7d4 7c9 7d6
7d5 :2 0 74 :3 0
87 :3 0 7d8 7d9
0 76 :4 0 108
:4 0 79 :2 0 7a
:3 0 3e8 7dd 7df
:3 0 79 :2 0 9d
:4 0 3eb 7e1 7e3
:3 0 79 :2 0 9e
:4 0 3ee 7e5 7e7
:3 0 79 :2 0 97
:3 0 3f1 7e9 7eb
:3 0 79 :2 0 9f
:4 0 3f4 7ed 7ef
:3 0 79 :2 0 a0
:3 0 a1 :3 0 2c
:2 0 a2 :2 0 3f7
7f2 7f6 3fb 7f1
7f8 :3 0 3fe 7da
7fa :2 0 7fc 401
7fd 7d7 7fc 0
7fe 403 0 7ff
405 802 :3 0 802
0 802 801 7ff
800 :6 0 804 c
:3 0 408 806 40a
805 804 :2 0 807
40c :2 0 809 0
809 808 7b6 807
:6 0 87f 1 :3 0
10 :3 0 96 :3 0
80b 80c 0 10
:3 0 a3 :2 0 a4
:3 0 80e 810 0
411 80f 812 :3 0
813 :2 0 e6 :3 0
f7 :3 0 b7 :3 0
a0 :3 0 25 :3 0
e6 :3 0 f9 :3 0
25 :3 0 101 :4 0
109 1 :8 0 824
10 :3 0 96 :3 0
81f 820 0 97
:3 0 821 822 0
824 414 877 98
:3 0 10 :3 0 96
:3 0 826 827 0
97 :3 0 828 829
0 82b 417 82d
419 82c 82b :2 0
875 99 :3 0 10
:3 0 96 :3 0 830
831 0 97 :3 0
832 833 0 86d
71 :3 0 72 :3 0
835 836 0 71
:3 0 84 :3 0 838
839 0 9a :2 0
2 :4 0 10a :2 0
86 :2 0 2c :2 0
41b 83e 840 :3 0
41e 83a 842 837
844 843 :2 0 74
:3 0 87 :3 0 846
847 0 76 :4 0
10b :4 0 79 :2 0
7a :3 0 422 84b
84d :3 0 79 :2 0
9d :4 0 425 84f
851 :3 0 79 :2 0
9e :4 0 428 853
855 :3 0 79 :2 0
97 :3 0 42b 857
859 :3 0 79 :2 0
9f :4 0 42e 85b
85d :3 0 79 :2 0
a0 :3 0 a1 :3 0
2c :2 0 a2 :2 0
431 860 864 435
85f 866 :3 0 438
848 868 :2 0 86a
43b 86b 845 86a
0 86c 43d 0
86d 43f 870 :3 0
870 0 870 86f
86d 86e :6 0 872
e :3 0 442 874
444 873 872 :2 0
875 446 :2 0 877
0 877 876 824
875 :6 0 879 1
:3 0 449 87a 814
879 0 87b 44b
0 87f 4f :3 0
2f :2 0 87c 87d
0 87f 44d 881
6b0 79f 0 882
0 87f 0 882
451 0 98e 7e
:3 0 4d :3 0 2f
:2 0 454 883 886
de :3 0 a3 :2 0
10c :3 0 888 88a
0 459 889 88c
:3 0 d7 :3 0 10d
:3 0 24 :3 0 10e
:3 0 25 :3 0 10f
:3 0 4f :3 0 8d
:3 0 1c :4 0 110
1 :8 0 8ad 111
:4 0 898 :3 0 e5
:3 0 a3 :2 0 45e
89b 89c :3 0 10
:3 0 96 :3 0 89e
89f 0 2f :2 0
8a0 8a1 0 8a3
461 8aa 10 :3 0
96 :3 0 8a4 8a5
0 112 :2 0 8a6
8a7 0 8a9 463
8ab 89d 8a3 0
8ac 0 8a9 0
8ac 465 0 8ad
468 900 98 :3 0
10 :3 0 96 :3 0
8af 8b0 0 97
:3 0 8b1 8b2 0
8b4 46b 8b6 46d
8b5 8b4 :2 0 8fe
99 :3 0 10 :3 0
96 :3 0 8b9 8ba
0 97 :3 0 8bb
8bc 0 8f6 71
:3 0 72 :3 0 8be
8bf 0 71 :3 0
84 :3 0 8c1 8c2
0 9a :2 0 2
:4 0 113 :2 0 86
:2 0 2c :2 0 46f
8c7 8c9 :3 0 472
8c3 8cb 8c0 8cd
8cc :2 0 74 :3 0
87 :3 0 8cf 8d0
0 76 :4 0 114
:4 0 79 :2 0 7a
:3 0 476 8d4 8d6
:3 0 79 :2 0 9d
:4 0 479 8d8 8da
:3 0 79 :2 0 9e
:4 0 47c 8dc 8de
:3 0 79 :2 0 97
:3 0 47f 8e0 8e2
:3 0 79 :2 0 9f
:4 0 482 8e4 8e6
:3 0 79 :2 0 a0
:3 0 a1 :3 0 2c
:2 0 a2 :2 0 485
8e9 8ed 489 8e8
8ef :3 0 48c 8d1
8f1 :2 0 8f3 48f
8f4 8ce 8f3 0
8f5 491 0 8f6
493 8f9 :3 0 8f9
0 8f9 8f8 8f6
8f7 :6 0 8fb 10
:3 0 496 8fd 498
8fc 8fb :2 0 8fe
49a :2 0 900 0
900 8ff 8ad 8fe
:6 0 902 1 :3 0
49d 978 d7 :3 0
115 :3 0 24 :3 0
116 :3 0 25 :3 0
117 :3 0 4f :3 0
8d :3 0 1c :4 0
118 1 :8 0 922
111 :4 0 90d :3 0
e5 :3 0 a3 :2 0
4a1 910 911 :3 0
10 :3 0 96 :3 0
913 914 0 2f
:2 0 915 916 0
918 4a4 91f 10
:3 0 96 :3 0 919
91a 0 112 :2 0
91b 91c 0 91e
4a6 920 912 918
0 921 0 91e
0 921 4a8 0
922 4ab 975 98
:3 0 10 :3 0 96
:3 0 924 925 0
97 :3 0 926 927
0 929 4ae 92b
4b0 92a 929 :2 0
973 99 :3 0 10
:3 0 96 :3 0 92e
92f 0 97 :3 0
930 931 0 96b
71 :3 0 72 :3 0
933 934 0 71
:3 0 84 :3 0 936
937 0 9a :2 0
2 :4 0 119 :2 0
86 :2 0 2c :2 0
4b2 93c 93e :3 0
4b5 938 940 935
942 941 :2 0 74
:3 0 87 :3 0 944
945 0 76 :4 0
11a :4 0 79 :2 0
7a :3 0 4b9 949
94b :3 0 79 :2 0
9d :4 0 4bc 94d
94f :3 0 79 :2 0
9e :4 0 4bf 951
953 :3 0 79 :2 0
97 :3 0 4c2 955
957 :3 0 79 :2 0
9f :4 0 4c5 959
95b :3 0 79 :2 0
a0 :3 0 a1 :3 0
2c :2 0 a2 :2 0
4c8 95e 962 4cc
95d 964 :3 0 4cf
946 966 :2 0 968
4d2 969 943 968
0 96a 4d4 0
96b 4d6 96e :3 0
96e 0 96e 96d
96b 96c :6 0 970
12 :3 0 4d9 972
4db 971 970 :2 0
973 4dd :2 0 975
0 975 974 922
973 :6 0 977 1
:3 0 4e0 979 88d
902 0 97a 0
977 0 97a 4e2
0 98e 10 :3 0
96 :3 0 97b 97c
0 10 :3 0 a3
:2 0 11b :3 0 97e
980 0 4e7 97f
982 :3 0 983 :2 0
60 :3 0 11c :3 0
1f :3 0 4ea 986
988 985 989 0
98b 4ec 98c 984
98b 0 98d 4ee
0 98e 4f0 cca
11d :3 0 60 :3 0
101 :3 0 c5 :3 0
92 :3 0 1f :3 0
c3 :3 0 1c :3 0
af :4 0 11e 1
:8 0 99e 10 :3 0
96 :3 0 999 99a
0 97 :3 0 99b
99c 0 99e 4f4
9f1 98 :3 0 10
:3 0 96 :3 0 9a0
9a1 0 97 :3 0
9a2 9a3 0 9a5
4f7 9a7 4f9 9a6
9a5 :2 0 9ef 99
:3 0 10 :3 0 96
:3 0 9aa 9ab 0
97 :3 0 9ac 9ad
0 9e7 71 :3 0
72 :3 0 9af 9b0
0 71 :3 0 84
:3 0 9b2 9b3 0
9a :2 0 2 :4 0
11f :2 0 86 :2 0
2c :2 0 4fb 9b8
9ba :3 0 4fe 9b4
9bc 9b1 9be 9bd
:2 0 74 :3 0 87
:3 0 9c0 9c1 0
76 :4 0 120 :4 0
79 :2 0 7a :3 0
502 9c5 9c7 :3 0
79 :2 0 9d :4 0
505 9c9 9cb :3 0
79 :2 0 9e :4 0
508 9cd 9cf :3 0
79 :2 0 97 :3 0
50b 9d1 9d3 :3 0
79 :2 0 9f :4 0
50e 9d5 9d7 :3 0
79 :2 0 a0 :3 0
a1 :3 0 2c :2 0
a2 :2 0 511 9da
9de 515 9d9 9e0
:3 0 518 9c2 9e2
:2 0 9e4 51b 9e5
9bf 9e4 0 9e6
51d 0 9e7 51f
9ea :3 0 9ea 0
9ea 9e9 9e7 9e8
:6 0 9ec 14 :3 0
522 9ee 524 9ed
9ec :2 0 9ef 526
:2 0 9f1 0 9f1
9f0 99e 9ef :6 0
cc9 1 :3 0 7e
:3 0 60 :3 0 2f
:2 0 529 9f3 9f6
e9 :2 0 2f :2 0
52e 9f8 9fa :3 0
71 :3 0 72 :3 0
9fc 9fd 0 71
:3 0 84 :3 0 9ff
a00 0 13 :2 0
2 :4 0 121 :2 0
86 :2 0 2c :2 0
531 a05 a07 :3 0
534 a01 a09 9fe
a0b a0a :2 0 74
:3 0 87 :3 0 a0d
a0e 0 76 :4 0
122 :4 0 79 :2 0
7a :3 0 538 a12
a14 :3 0 79 :2 0
9d :4 0 53b a16
a18 :3 0 79 :2 0
dd :4 0 53e a1a
a1c :3 0 79 :2 0
7e :3 0 82 :3 0
de :3 0 123 :3 0
a21 a22 0 541
a20 a24 81 :4 0
543 a1f a27 546
a1e a29 :3 0 79
:2 0 e0 :4 0 549
a2b a2d :3 0 79
:2 0 7c :4 0 54c
a2f a31 :3 0 79
:2 0 1c :3 0 54f
a33 a35 :3 0 79
:2 0 7c :4 0 552
a37 a39 :3 0 555
a0f a3b :2 0 a3d
558 a3e a0c a3d
0 a3f 55a 0
a69 71 :3 0 72
:3 0 a40 a41 0
71 :3 0 84 :3 0
a43 a44 0 2c
:2 0 2 :4 0 124
:2 0 86 :2 0 2c
:2 0 55c a49 a4b
:3 0 55f a45 a4d
a42 a4f a4e :2 0
74 :3 0 87 :3 0
a51 a52 0 76
:4 0 125 :4 0 79
:2 0 7a :3 0 563
a56 a58 :3 0 566
a53 a5a :2 0 a5c
569 a5d a50 a5c
0 a5e 56b 0
a69 e3 :3 0 e4
:3 0 a5f a60 0
de :3 0 123 :3 0
a62 a63 0 1c
:3 0 e5 :3 0 56d
a61 a67 :2 0 a69
571 a6a 9fb a69
0 a6b 575 0
cc9 e6 :3 0 e7
:3 0 a6c a6d 0
24 :3 0 22 :3 0
e8 :2 0 577 a6e
a72 ea :2 0 2f
:2 0 57d a74 a76
:3 0 e6 :3 0 f7
:3 0 f8 :3 0 e6
:3 0 f9 :3 0 fa
:3 0 fb :3 0 fc
:3 0 24 :3 0 fd
:3 0 8d :3 0 1f
:3 0 f8 :3 0 b7
:3 0 a0 :3 0 24
:4 0 fe 1 :8 0
a8e 10 :3 0 96
:3 0 a89 a8a 0
97 :3 0 a8b a8c
0 a8e 580 ae1
98 :3 0 10 :3 0
96 :3 0 a90 a91
0 97 :3 0 a92
a93 0 a95 583
a97 585 a96 a95
:2 0 adf 99 :3 0
10 :3 0 96 :3 0
a9a a9b 0 97
:3 0 a9c a9d 0
ad7 71 :3 0 72
:3 0 a9f aa0 0
71 :3 0 84 :3 0
aa2 aa3 0 9a
:2 0 2 :4 0 126
:2 0 86 :2 0 2c
:2 0 587 aa8 aaa
:3 0 58a aa4 aac
aa1 aae aad :2 0
74 :3 0 87 :3 0
ab0 ab1 0 76
:4 0 127 :4 0 79
:2 0 7a :3 0 58e
ab5 ab7 :3 0 79
:2 0 9d :4 0 591
ab9 abb :3 0 79
:2 0 9e :4 0 594
abd abf :3 0 79
:2 0 97 :3 0 597
ac1 ac3 :3 0 79
:2 0 9f :4 0 59a
ac5 ac7 :3 0 79
:2 0 a0 :3 0 a1
:3 0 2c :2 0 a2
:2 0 59d aca ace
5a1 ac9 ad0 :3 0
5a4 ab2 ad2 :2 0
ad4 5a7 ad5 aaf
ad4 0 ad6 5a9
0 ad7 5ab ada
:3 0 ada 0 ada
ad9 ad7 ad8 :6 0
adc 16 :3 0 5ae
ade 5b0 add adc
:2 0 adf 5b2 :2 0
ae1 0 ae1 ae0
a8e adf :6 0 b54
1 :3 0 10 :3 0
96 :3 0 ae3 ae4
0 10 :3 0 a3
:2 0 a4 :3 0 ae6
ae8 0 5b7 ae7
aea :3 0 aeb :2 0
e6 :3 0 f7 :3 0
b7 :3 0 a0 :3 0
24 :3 0 e6 :3 0
f9 :3 0 24 :3 0
101 :4 0 102 1
:8 0 afc 10 :3 0
96 :3 0 af7 af8
0 97 :3 0 af9
afa 0 afc 5ba
b4f 98 :3 0 10
:3 0 96 :3 0 afe
aff 0 97 :3 0
b00 b01 0 b03
5bd b05 5bf b04
b03 :2 0 b4d 99
:3 0 10 :3 0 96
:3 0 b08 b09 0
97 :3 0 b0a b0b
0 b45 71 :3 0
72 :3 0 b0d b0e
0 71 :3 0 84
:3 0 b10 b11 0
9a :2 0 2 :4 0
128 :2 0 86 :2 0
2c :2 0 5c1 b16
b18 :3 0 5c4 b12
b1a b0f b1c b1b
:2 0 74 :3 0 87
:3 0 b1e b1f 0
76 :4 0 129 :4 0
79 :2 0 7a :3 0
5c8 b23 b25 :3 0
79 :2 0 9d :4 0
5cb b27 b29 :3 0
79 :2 0 9e :4 0
5ce b2b b2d :3 0
79 :2 0 97 :3 0
5d1 b2f b31 :3 0
79 :2 0 9f :4 0
5d4 b33 b35 :3 0
79 :2 0 a0 :3 0
a1 :3 0 2c :2 0
a2 :2 0 5d7 b38
b3c 5db b37 b3e
:3 0 5de b20 b40
:2 0 b42 5e1 b43
b1d b42 0 b44
5e3 0 b45 5e5
b48 :3 0 b48 0
b48 b47 b45 b46
:6 0 b4a 18 :3 0
5e8 b4c 5ea b4b
b4a :2 0 b4d 5ec
:2 0 b4f 0 b4f
b4e afc b4d :6 0
b51 1 :3 0 5ef
b52 aec b51 0
b53 5f1 0 b54
5f3 c32 e6 :3 0
f7 :3 0 f8 :3 0
e6 :3 0 f9 :3 0
fa :3 0 105 :3 0
fc :3 0 25 :3 0
fd :3 0 8d :3 0
1f :3 0 f8 :3 0
b7 :3 0 a0 :3 0
25 :4 0 106 1
:8 0 b6b 10 :3 0
96 :3 0 b66 b67
0 97 :3 0 b68
b69 0 b6b 5f6
bbe 98 :3 0 10
:3 0 96 :3 0 b6d
b6e 0 97 :3 0
b6f b70 0 b72
5f9 b74 5fb b73
b72 :2 0 bbc 99
:3 0 10 :3 0 96
:3 0 b77 b78 0
97 :3 0 b79 b7a
0 bb4 71 :3 0
72 :3 0 b7c b7d
0 71 :3 0 84
:3 0 b7f b80 0
9a :2 0 2 :4 0
12a :2 0 86 :2 0
2c :2 0 5fd b85
b87 :3 0 600 b81
b89 b7e b8b b8a
:2 0 74 :3 0 87
:3 0 b8d b8e 0
76 :4 0 12b :4 0
79 :2 0 7a :3 0
604 b92 b94 :3 0
79 :2 0 9d :4 0
607 b96 b98 :3 0
79 :2 0 9e :4 0
60a b9a b9c :3 0
79 :2 0 97 :3 0
60d b9e ba0 :3 0
79 :2 0 9f :4 0
610 ba2 ba4 :3 0
79 :2 0 a0 :3 0
a1 :3 0 2c :2 0
a2 :2 0 613 ba7
bab 617 ba6 bad
:3 0 61a b8f baf
:2 0 bb1 61d bb2
b8c bb1 0 bb3
61f 0 bb4 621
bb7 :3 0 bb7 0
bb7 bb6 bb4 bb5
:6 0 bb9 1a :3 0
624 bbb 626 bba
bb9 :2 0 bbc 628
:2 0 bbe 0 bbe
bbd b6b bbc :6 0
c31 1 :3 0 10
:3 0 96 :3 0 bc0
bc1 0 10 :3 0
a3 :2 0 a4 :3 0
bc3 bc5 0 62d
bc4 bc7 :3 0 bc8
:2 0 e6 :3 0 f7
:3 0 b7 :3 0 a0
:3 0 25 :3 0 e6
:3 0 f9 :3 0 25
:3 0 101 :4 0 109
1 :8 0 bd9 10
:3 0 96 :3 0 bd4
bd5 0 97 :3 0
bd6 bd7 0 bd9
630 c2c 98 :3 0
10 :3 0 96 :3 0
bdb bdc 0 97
:3 0 bdd bde 0
be0 633 be2 635
be1 be0 :2 0 c2a
99 :3 0 10 :3 0
96 :3 0 be5 be6
0 97 :3 0 be7
be8 0 c22 71
:3 0 72 :3 0 bea
beb 0 71 :3 0
84 :3 0 bed bee
0 9a :2 0 2
:4 0 12c :2 0 86
:2 0 2c :2 0 637
bf3 bf5 :3 0 63a
bef bf7 bec bf9
bf8 :2 0 74 :3 0
87 :3 0 bfb bfc
0 76 :4 0 12d
:4 0 79 :2 0 7a
:3 0 63e c00 c02
:3 0 79 :2 0 9d
:4 0 641 c04 c06
:3 0 79 :2 0 9e
:4 0 644 c08 c0a
:3 0 79 :2 0 97
:3 0 647 c0c c0e
:3 0 79 :2 0 9f
:4 0 64a c10 c12
:3 0 79 :2 0 a0
:3 0 a1 :3 0 2c
:2 0 a2 :2 0 64d
c15 c19 651 c14
c1b :3 0 654 bfd
c1d :2 0 c1f 657
c20 bfa c1f 0
c21 659 0 c22
65b c25 :3 0 c25
0 c25 c24 c22
c23 :6 0 c27 1c
:3 0 65e c29 660
c28 c27 :2 0 c2a
662 :2 0 c2c 0
c2c c2b bd9 c2a
:6 0 c2e 1 :3 0
665 c2f bc9 c2e
0 c30 667 0
c31 669 c33 a77
b54 0 c34 0
c31 0 c34 66c
0 cc9 60 :3 0
12e :3 0 1f :3 0
1c :3 0 22 :3 0
23 :3 0 24 :3 0
25 :3 0 4c :3 0
40 :3 0 7e :3 0
4b :3 0 2f :2 0
66f c3f c42 2f
:2 0 672 c36 c45
c35 c46 0 cc9
7e :3 0 60 :3 0
2f :2 0 67d c48
c4b a3 :2 0 2f
:2 0 682 c4d c4f
:3 0 60 :3 0 11c
:3 0 1f :3 0 685
c52 c54 c51 c55
0 c57 687 cc6
71 :3 0 72 :3 0
c58 c59 0 71
:3 0 84 :3 0 c5b
c5c 0 13 :2 0
2 :4 0 12f :2 0
86 :2 0 2c :2 0
689 c61 c63 :3 0
68c c5d c65 c5a
c67 c66 :2 0 74
:3 0 87 :3 0 c69
c6a 0 76 :4 0
130 :4 0 79 :2 0
7a :3 0 690 c6e
c70 :3 0 79 :2 0
9d :4 0 693 c72
c74 :3 0 79 :2 0
dd :4 0 696 c76
c78 :3 0 79 :2 0
7e :3 0 82 :3 0
60 :3 0 699 c7c
c7e 81 :4 0 69b
c7b c81 69e c7a
c83 :3 0 79 :2 0
e0 :4 0 6a1 c85
c87 :3 0 79 :2 0
7c :4 0 6a4 c89
c8b :3 0 79 :2 0
1c :3 0 6a7 c8d
c8f :3 0 79 :2 0
7c :4 0 6aa c91
c93 :3 0 6ad c6b
c95 :2 0 c97 6b0
c98 c68 c97 0
c99 6b2 0 cc5
71 :3 0 72 :3 0
c9a c9b 0 71
:3 0 84 :3 0 c9d
c9e 0 2c :2 0
2 :4 0 131 :2 0
86 :2 0 2c :2 0
6b4 ca3 ca5 :3 0
6b7 c9f ca7 c9c
ca9 ca8 :2 0 74
:3 0 87 :3 0 cab
cac 0 76 :4 0
132 :4 0 79 :2 0
7a :3 0 6bb cb0
cb2 :3 0 6be cad
cb4 :2 0 cb6 6c1
cb7 caa cb6 0
cb8 6c3 0 cc5
e3 :3 0 e4 :3 0
cb9 cba 0 7e
:3 0 60 :3 0 2f
:2 0 6c5 cbc cbf
1c :3 0 e5 :3 0
6c8 cbb cc3 :2 0
cc5 6cc cc7 c50
c57 0 cc8 0
cc5 0 cc8 6d0
0 cc9 6d3 ccb
6a4 98e 0 ccc
0 cc9 0 ccc
6d9 0 cce 133
:3 0 6dc f5d 7e
:3 0 d :3 0 2f
:2 0 6de ccf cd2
7e :3 0 ea :2 0
46 :3 0 2f :2 0
6e1 cd4 cd8 6e6
cd5 cda :3 0 cdb
:2 0 7e :3 0 41
:3 0 2f :2 0 6e9
cdd ce0 e9 :2 0
2f :2 0 6ee ce2
ce4 :3 0 ce5 :2 0
cdc ce7 ce6 :2 0
57 :3 0 2f :2 0
ce9 cea 0 f5b
7e :3 0 d :3 0
2f :2 0 6f1 cec
cef 7e :3 0 e9
:2 0 41 :3 0 2f
:2 0 6f4 cf1 cf5
6f9 cf2 cf7 :3 0
71 :3 0 72 :3 0
cf9 cfa 0 71
:3 0 84 :3 0 cfc
cfd 0 134 :2 0
2 :4 0 135 :2 0
86 :2 0 2c :2 0
6fc d02 d04 :3 0
6ff cfe d06 cfb
d08 d07 :2 0 74
:3 0 87 :3 0 d0a
d0b 0 76 :4 0
136 :4 0 79 :2 0
7a :3 0 703 d0f
d11 :3 0 79 :2 0
9d :4 0 706 d13
d15 :3 0 79 :2 0
dd :4 0 709 d17
d19 :3 0 79 :2 0
7e :3 0 82 :3 0
de :3 0 137 :3 0
d1e d1f 0 70c
d1d d21 81 :4 0
70e d1c d24 711
d1b d26 :3 0 714
d0c d28 :2 0 d2a
717 d2b d09 d2a
0 d2c 719 0
d56 71 :3 0 72
:3 0 d2d d2e 0
71 :3 0 84 :3 0
d30 d31 0 2c
:2 0 2 :4 0 138
:2 0 86 :2 0 2c
:2 0 71b d36 d38
:3 0 71e d32 d3a
d2f d3c d3b :2 0
74 :3 0 87 :3 0
d3e d3f 0 76
:4 0 139 :4 0 79
:2 0 7a :3 0 722
d43 d45 :3 0 725
d40 d47 :2 0 d49
728 d4a d3d d49
0 d4b 72a 0
d56 e3 :3 0 13a
:3 0 d4c d4d 0
de :3 0 137 :3 0
d4f d50 0 e5
:3 0 72c d4e d53
:2 0 d56 133 :3 0
72f d69 7e :3 0
d :3 0 2f :2 0
733 d57 d5a 7e
:3 0 a3 :2 0 41
:3 0 2f :2 0 736
d5c d60 73b d5d
d62 :3 0 57 :3 0
2c :2 0 d64 d65
0 d67 73e d68
d63 d67 0 d6a
cf8 d56 0 d6a
740 0 f5b 60
:3 0 2f :2 0 d6b
d6c 0 f5b 11d
:3 0 60 :3 0 101
:3 0 13b :3 0 8d
:3 0 c5 :3 0 13c
:3 0 13d :3 0 13b
:3 0 13c :3 0 c3
:3 0 1c :3 0 13c
:3 0 c6 :3 0 13b
:3 0 c6 :3 0 13b
:3 0 c6 :3 0 13b
:3 0 8d :3 0 13b
:3 0 13e :3 0 de
:3 0 13f :4 0 140
1 :8 0 d8c 10
:3 0 96 :3 0 d87
d88 0 97 :3 0
d89 d8a 0 d8c
743 ddf 98 :3 0
10 :3 0 96 :3 0
d8e d8f 0 97
:3 0 d90 d91 0
d93 746 d95 748
d94 d93 :2 0 ddd
99 :3 0 10 :3 0
96 :3 0 d98 d99
0 97 :3 0 d9a
d9b 0 dd5 71
:3 0 72 :3 0 d9d
d9e 0 71 :3 0
84 :3 0 da0 da1
0 9a :2 0 2
:4 0 141 :2 0 86
:2 0 2c :2 0 74a
da6 da8 :3 0 74d
da2 daa d9f dac
dab :2 0 74 :3 0
87 :3 0 dae daf
0 76 :4 0 142
:4 0 79 :2 0 7a
:3 0 751 db3 db5
:3 0 79 :2 0 9d
:4 0 754 db7 db9
:3 0 79 :2 0 9e
:4 0 757 dbb dbd
:3 0 79 :2 0 97
:3 0 75a dbf dc1
:3 0 79 :2 0 9f
:4 0 75d dc3 dc5
:3 0 79 :2 0 a0
:3 0 a1 :3 0 2c
:2 0 a2 :2 0 760
dc8 dcc 764 dc7
dce :3 0 767 db0
dd0 :2 0 dd2 76a
dd3 dad dd2 0
dd4 76c 0 dd5
76e dd8 :3 0 dd8
0 dd8 dd7 dd5
dd6 :6 0 dda 1e
:3 0 771 ddc 773
ddb dda :2 0 ddd
775 :2 0 ddf 0
ddf dde d8c ddd
:6 0 f5b 1 :3 0
7e :3 0 60 :3 0
2f :2 0 778 de1
de4 e9 :2 0 2f
:2 0 77d de6 de8
:3 0 71 :3 0 72
:3 0 dea deb 0
71 :3 0 84 :3 0
ded dee 0 13
:2 0 2 :4 0 143
:2 0 86 :2 0 2c
:2 0 780 df3 df5
:3 0 783 def df7
dec df9 df8 :2 0
74 :3 0 87 :3 0
dfb dfc 0 76
:4 0 144 :4 0 79
:2 0 7a :3 0 787
e00 e02 :3 0 79
:2 0 9d :4 0 78a
e04 e06 :3 0 79
:2 0 dd :4 0 78d
e08 e0a :3 0 79
:2 0 7e :3 0 82
:3 0 de :3 0 145
:3 0 e0f e10 0
790 e0e e12 81
:4 0 792 e0d e15
795 e0c e17 :3 0
79 :2 0 e0 :4 0
798 e19 e1b :3 0
79 :2 0 7c :4 0
79b e1d e1f :3 0
79 :2 0 1c :3 0
79e e21 e23 :3 0
79 :2 0 7c :4 0
7a1 e25 e27 :3 0
7a4 dfd e29 :2 0
e2b 7a7 e2c dfa
e2b 0 e2d 7a9
0 e57 71 :3 0
72 :3 0 e2e e2f
0 71 :3 0 84
:3 0 e31 e32 0
2c :2 0 2 :4 0
146 :2 0 86 :2 0
2c :2 0 7ab e37
e39 :3 0 7ae e33
e3b e30 e3d e3c
:2 0 74 :3 0 87
:3 0 e3f e40 0
76 :4 0 147 :4 0
79 :2 0 7a :3 0
7b2 e44 e46 :3 0
7b5 e41 e48 :2 0
e4a 7b8 e4b e3e
e4a 0 e4c 7ba
0 e57 e3 :3 0
e4 :3 0 e4d e4e
0 de :3 0 145
:3 0 e50 e51 0
1c :3 0 e5 :3 0
7bc e4f e55 :2 0
e57 7c0 e58 de9
e57 0 e59 7c4
0 f5b 11d :3 0
60 :3 0 101 :3 0
c5 :3 0 13c :3 0
148 :3 0 149 :3 0
14a :3 0 14a :3 0
8d :3 0 13c :3 0
92 :3 0 14a :3 0
91 :3 0 13c :3 0
91 :3 0 13c :3 0
92 :3 0 1f :3 0
13c :3 0 c3 :3 0
1c :3 0 b7 :3 0
a0 :3 0 13c :3 0
af :3 0 14a :3 0
14b :4 0 14c 1
:8 0 e7c 10 :3 0
96 :3 0 e77 e78
0 97 :3 0 e79
e7a 0 e7c 7c6
ecf 98 :3 0 10
:3 0 96 :3 0 e7e
e7f 0 97 :3 0
e80 e81 0 e83
7c9 e85 7cb e84
e83 :2 0 ecd 99
:3 0 10 :3 0 96
:3 0 e88 e89 0
97 :3 0 e8a e8b
0 ec5 71 :3 0
72 :3 0 e8d e8e
0 71 :3 0 84
:3 0 e90 e91 0
9a :2 0 2 :4 0
14d :2 0 86 :2 0
2c :2 0 7cd e96
e98 :3 0 7d0 e92
e9a e8f e9c e9b
:2 0 74 :3 0 87
:3 0 e9e e9f 0
76 :4 0 14e :4 0
79 :2 0 7a :3 0
7d4 ea3 ea5 :3 0
79 :2 0 9d :4 0
7d7 ea7 ea9 :3 0
79 :2 0 9e :4 0
7da eab ead :3 0
79 :2 0 97 :3 0
7dd eaf eb1 :3 0
79 :2 0 9f :4 0
7e0 eb3 eb5 :3 0
79 :2 0 a0 :3 0
a1 :3 0 2c :2 0
a2 :2 0 7e3 eb8
ebc 7e7 eb7 ebe
:3 0 7ea ea0 ec0
:2 0 ec2 7ed ec3
e9d ec2 0 ec4
7ef 0 ec5 7f1
ec8 :3 0 ec8 0
ec8 ec7 ec5 ec6
:6 0 eca 20 :3 0
7f4 ecc 7f6 ecb
eca :2 0 ecd 7f8
:2 0 ecf 0 ecf
ece e7c ecd :6 0
f5b 1 :3 0 7e
:3 0 60 :3 0 2f
:2 0 7fb ed1 ed4
e9 :2 0 2f :2 0
800 ed6 ed8 :3 0
71 :3 0 72 :3 0
eda edb 0 71
:3 0 84 :3 0 edd
ede 0 13 :2 0
2 :4 0 14f :2 0
86 :2 0 2c :2 0
803 ee3 ee5 :3 0
806 edf ee7 edc
ee9 ee8 :2 0 74
:3 0 87 :3 0 eeb
eec 0 76 :4 0
150 :4 0 79 :2 0
7a :3 0 80a ef0
ef2 :3 0 79 :2 0
9d :4 0 80d ef4
ef6 :3 0 79 :2 0
dd :4 0 810 ef8
efa :3 0 79 :2 0
7e :3 0 82 :3 0
de :3 0 151 :3 0
eff f00 0 813
efe f02 81 :4 0
815 efd f05 818
efc f07 :3 0 79
:2 0 e0 :4 0 81b
f09 f0b :3 0 79
:2 0 7c :4 0 81e
f0d f0f :3 0 79
:2 0 1c :3 0 821
f11 f13 :3 0 79
:2 0 7c :4 0 824
f15 f17 :3 0 827
eed f19 :2 0 f1b
82a f1c eea f1b
0 f1d 82c 0
f47 71 :3 0 72
:3 0 f1e f1f 0
71 :3 0 84 :3 0
f21 f22 0 2c
:2 0 2 :4 0 152
:2 0 86 :2 0 2c
:2 0 82e f27 f29
:3 0 831 f23 f2b
f20 f2d f2c :2 0
74 :3 0 87 :3 0
f2f f30 0 76
:4 0 153 :4 0 79
:2 0 7a :3 0 835
f34 f36 :3 0 838
f31 f38 :2 0 f3a
83b f3b f2e f3a
0 f3c 83d 0
f47 e3 :3 0 e4
:3 0 f3d f3e 0
de :3 0 151 :3 0
f40 f41 0 1c
:3 0 e5 :3 0 83f
f3f f45 :2 0 f47
843 f48 ed9 f47
0 f49 847 0
f5b 47 :3 0 d
:3 0 f4a f4b 0
f5b 60 :3 0 154
:3 0 1f :3 0 1c
:3 0 47 :3 0 40
:3 0 7e :3 0 57
:3 0 2f :2 0 849
f53 f56 84c f4e
f58 f4d f59 0
f5b 852 f5c ce8
f5b 0 f5e 69b
cce 0 f5e 85c
0 fdf 7e :3 0
60 :3 0 2f :2 0
85f f5f f62 a3
:2 0 2f :2 0 864
f64 f66 :3 0 60
:3 0 11c :3 0 1f
:3 0 867 f69 f6b
f68 f6c 0 f6e
869 f6f f67 f6e
0 f70 86b 0
fdf 71 :3 0 72
:3 0 f71 f72 0
71 :3 0 84 :3 0
f74 f75 0 13
:2 0 2 :4 0 155
:2 0 86 :2 0 2c
:2 0 86d f7a f7c
:3 0 870 f76 f7e
f73 f80 f7f :2 0
74 :3 0 87 :3 0
f82 f83 0 76
:4 0 156 :4 0 79
:2 0 7a :3 0 874
f87 f89 :3 0 79
:2 0 9d :4 0 877
f8b f8d :3 0 79
:2 0 dd :4 0 87a
f8f f91 :3 0 79
:2 0 7e :3 0 82
:3 0 60 :3 0 87d
f95 f97 81 :4 0
87f f94 f9a 882
f93 f9c :3 0 79
:2 0 e0 :4 0 885
f9e fa0 :3 0 79
:2 0 7c :4 0 888
fa2 fa4 :3 0 79
:2 0 1c :3 0 88b
fa6 fa8 :3 0 79
:2 0 7c :4 0 88e
faa fac :3 0 891
f84 fae :2 0 fb0
894 fb1 f81 fb0
0 fb2 896 0
fdf 71 :3 0 72
:3 0 fb3 fb4 0
71 :3 0 84 :3 0
fb6 fb7 0 2c
:2 0 2 :4 0 157
:2 0 86 :2 0 2c
:2 0 898 fbc fbe
:3 0 89b fb8 fc0
fb5 fc2 fc1 :2 0
74 :3 0 87 :3 0
fc4 fc5 0 76
:4 0 158 :4 0 79
:2 0 7a :3 0 89f
fc9 fcb :3 0 8a2
fc6 fcd :2 0 fcf
8a5 fd0 fc3 fcf
0 fd1 8a7 0
fdf e3 :3 0 e4
:3 0 fd2 fd3 0
7e :3 0 60 :3 0
2f :2 0 8a9 fd5
fd8 1c :3 0 e5
:3 0 8ac fd4 fdc
:2 0 fdf 133 :3 0
8b0 2514 6 :3 0
f0 :2 0 8bc fe1
fe2 :3 0 fe3 :2 0
8 :3 0 f0 :2 0
8be fe6 fe7 :3 0
fe8 :2 0 fe4 fea
fe9 :2 0 24 :3 0
6 :3 0 fec fed
0 2399 25 :3 0
8 :3 0 fef ff0
0 2399 e6 :3 0
e7 :3 0 ff2 ff3
0 24 :3 0 25
:3 0 e8 :2 0 8c0
ff4 ff8 e9 :2 0
2f :2 0 8c6 ffa
ffc :3 0 ffd :2 0
e6 :3 0 e7 :3 0
fff 1000 0 25
:3 0 23 :3 0 e8
:2 0 8c9 1001 1005
ea :2 0 2f :2 0
8cf 1007 1009 :3 0
100a :2 0 ffe 100c
100b :2 0 71 :3 0
72 :3 0 100e 100f
0 71 :3 0 84
:3 0 1011 1012 0
13 :2 0 2 :4 0
159 :2 0 86 :2 0
2c :2 0 8d2 1017
1019 :3 0 8d5 1013
101b 1010 101d 101c
:2 0 74 :3 0 87
:3 0 101f 1020 0
76 :4 0 15a :4 0
79 :2 0 7a :3 0
8d9 1024 1026 :3 0
79 :2 0 9d :4 0
8dc 1028 102a :3 0
79 :2 0 dd :4 0
8df 102c 102e :3 0
79 :2 0 7e :3 0
82 :3 0 de :3 0
ed :3 0 1033 1034
0 8e2 1032 1036
81 :4 0 8e4 1031
1039 8e7 1030 103b
:3 0 79 :2 0 e0
:4 0 8ea 103d 103f
:3 0 79 :2 0 7c
:4 0 8ed 1041 1043
:3 0 79 :2 0 1c
:3 0 8f0 1045 1047
:3 0 79 :2 0 7c
:4 0 8f3 1049 104b
:3 0 8f6 1021 104d
:2 0 104f 8f9 1050
101e 104f 0 1051
8fb 0 107b 71
:3 0 72 :3 0 1052
1053 0 71 :3 0
84 :3 0 1055 1056
0 2c :2 0 2
:4 0 15b :2 0 86
:2 0 2c :2 0 8fd
105b 105d :3 0 900
1057 105f 1054 1061
1060 :2 0 74 :3 0
87 :3 0 1063 1064
0 76 :4 0 15c
:4 0 79 :2 0 7a
:3 0 904 1068 106a
:3 0 907 1065 106c
:2 0 106e 90a 106f
1062 106e 0 1070
90c 0 107b e3
:3 0 e4 :3 0 1071
1072 0 de :3 0
ed :3 0 1074 1075
0 1c :3 0 e5
:3 0 90e 1073 1079
:2 0 107b 912 107c
100d 107b 0 107d
916 0 2399 92
:3 0 ca :3 0 cb
:3 0 1f :3 0 4b
:3 0 4c :3 0 d7
:3 0 92 :3 0 1f
:3 0 8d :3 0 3e
:4 0 15d 1 :8 0
108f 10 :3 0 96
:3 0 108a 108b 0
97 :3 0 108c 108d
0 108f 918 10e2
98 :3 0 10 :3 0
96 :3 0 1091 1092
0 97 :3 0 1093
1094 0 1096 91b
1098 91d 1097 1096
:2 0 10e0 99 :3 0
10 :3 0 96 :3 0
109b 109c 0 97
:3 0 109d 109e 0
10d8 71 :3 0 72
:3 0 10a0 10a1 0
71 :3 0 84 :3 0
10a3 10a4 0 9a
:2 0 2 :4 0 15e
:2 0 86 :2 0 2c
:2 0 91f 10a9 10ab
:3 0 922 10a5 10ad
10a2 10af 10ae :2 0
74 :3 0 87 :3 0
10b1 10b2 0 76
:4 0 15f :4 0 79
:2 0 7a :3 0 926
10b6 10b8 :3 0 79
:2 0 9d :4 0 929
10ba 10bc :3 0 79
:2 0 9e :4 0 92c
10be 10c0 :3 0 79
:2 0 97 :3 0 92f
10c2 10c4 :3 0 79
:2 0 9f :4 0 932
10c6 10c8 :3 0 79
:2 0 a0 :3 0 a1
:3 0 2c :2 0 a2
:2 0 935 10cb 10cf
939 10ca 10d1 :3 0
93c 10b3 10d3 :2 0
10d5 93f 10d6 10b0
10d5 0 10d7 941
0 10d8 943 10db
:3 0 10db 0 10db
10da 10d8 10d9 :6 0
10dd 22 :3 0 946
10df 948 10de 10dd
:2 0 10e0 94a :2 0
10e2 0 10e2 10e1
108f 10e0 :6 0 2399
1 :3 0 a0 :3 0
2d :3 0 2c :2 0
2c :2 0 94d 10e4
10e8 a3 :2 0 2c
:4 0 953 10ea 10ec
:3 0 38 :3 0 2c
:2 0 10ee 10ef 0
10f1 956 10f6 38
:3 0 2f :2 0 10f2
10f3 0 10f5 958
10f7 10ed 10f1 0
10f8 0 10f5 0
10f8 95a 0 2399
6c :3 0 2f :2 0
10f9 10fa 0 2399
6d :3 0 2f :2 0
10fc 10fd 0 2399
e6 :3 0 e7 :3 0
10ff 1100 0 24
:3 0 22 :3 0 e8
:2 0 95d 1101 1105
ea :2 0 2f :2 0
963 1107 1109 :3 0
110a :2 0 e6 :3 0
e7 :3 0 110c 110d
0 25 :3 0 23
:3 0 e8 :2 0 966
110e 1112 ea :2 0
2f :2 0 96c 1114
1116 :3 0 1117 :2 0
110b 1119 1118 :2 0
7e :3 0 9 :3 0
2f :2 0 96f 111b
111e 7e :3 0 ea
:2 0 43 :3 0 2f
:2 0 972 1120 1124
977 1121 1126 :3 0
1127 :2 0 111a 1129
1128 :2 0 e6 :3 0
e7 :3 0 112b 112c
0 24 :3 0 22
:3 0 e8 :2 0 97a
112d 1131 ea :2 0
2f :2 0 980 1133
1135 :3 0 ca :3 0
cb :3 0 4b :3 0
4c :3 0 d7 :3 0
92 :3 0 1f :3 0
8d :3 0 3e :4 0
160 1 :8 0 1146
10 :3 0 96 :3 0
1141 1142 0 97
:3 0 1143 1144 0
1146 983 1199 98
:3 0 10 :3 0 96
:3 0 1148 1149 0
97 :3 0 114a 114b
0 114d 986 114f
988 114e 114d :2 0
1197 99 :3 0 10
:3 0 96 :3 0 1152
1153 0 97 :3 0
1154 1155 0 118f
71 :3 0 72 :3 0
1157 1158 0 71
:3 0 84 :3 0 115a
115b 0 9a :2 0
2 :4 0 161 :2 0
86 :2 0 2c :2 0
98a 1160 1162 :3 0
98d 115c 1164 1159
1166 1165 :2 0 74
:3 0 87 :3 0 1168
1169 0 76 :4 0
162 :4 0 79 :2 0
7a :3 0 991 116d
116f :3 0 79 :2 0
9d :4 0 994 1171
1173 :3 0 79 :2 0
9e :4 0 997 1175
1177 :3 0 79 :2 0
97 :3 0 99a 1179
117b :3 0 79 :2 0
9f :4 0 99d 117d
117f :3 0 79 :2 0
a0 :3 0 a1 :3 0
2c :2 0 a2 :2 0
9a0 1182 1186 9a4
1181 1188 :3 0 9a7
116a 118a :2 0 118c
9aa 118d 1167 118c
0 118e 9ac 0
118f 9ae 1192 :3 0
1192 0 1192 1191
118f 1190 :6 0 1194
24 :3 0 9b1 1196
9b3 1195 1194 :2 0
1197 9b5 :2 0 1199
0 1199 1198 1146
1197 :6 0 11a5 1
:3 0 22 :3 0 24
:3 0 119b 119c 0
11a5 43 :3 0 2f
:2 0 119e 119f 0
11a5 6c :3 0 2c
:2 0 11a1 11a2 0
11a5 133 :3 0 9b8
1754 e6 :3 0 e7
:3 0 11a6 11a7 0
25 :3 0 23 :3 0
e8 :2 0 9bd 11a8
11ac ea :2 0 2f
:2 0 9c3 11ae 11b0
:3 0 e6 :3 0 f7
:3 0 b7 :3 0 a0
:3 0 25 :3 0 e6
:3 0 f9 :3 0 23
:3 0 25 :3 0 101
:4 0 163 1 :8 0
11c2 10 :3 0 96
:3 0 11bd 11be 0
97 :3 0 11bf 11c0
0 11c2 9c6 1215
98 :3 0 10 :3 0
96 :3 0 11c4 11c5
0 97 :3 0 11c6
11c7 0 11c9 9c9
11cb 9cb 11ca 11c9
:2 0 1213 99 :3 0
10 :3 0 96 :3 0
11ce 11cf 0 97
:3 0 11d0 11d1 0
120b 71 :3 0 72
:3 0 11d3 11d4 0
71 :3 0 84 :3 0
11d6 11d7 0 9a
:2 0 2 :4 0 164
:2 0 86 :2 0 2c
:2 0 9cd 11dc 11de
:3 0 9d0 11d8 11e0
11d5 11e2 11e1 :2 0
74 :3 0 87 :3 0
11e4 11e5 0 76
:4 0 165 :4 0 79
:2 0 7a :3 0 9d4
11e9 11eb :3 0 79
:2 0 9d :4 0 9d7
11ed 11ef :3 0 79
:2 0 9e :4 0 9da
11f1 11f3 :3 0 79
:2 0 97 :3 0 9dd
11f5 11f7 :3 0 79
:2 0 9f :4 0 9e0
11f9 11fb :3 0 79
:2 0 a0 :3 0 a1
:3 0 2c :2 0 a2
:2 0 9e3 11fe 1202
9e7 11fd 1204 :3 0
9ea 11e6 1206 :2 0
1208 9ed 1209 11e3
1208 0 120a 9ef
0 120b 9f1 120e
:3 0 120e 0 120e
120d 120b 120c :6 0
1210 26 :3 0 9f4
1212 9f6 1211 1210
:2 0 1213 9f8 :2 0
1215 0 1215 1214
11c2 1213 :6 0 13c4
1 :3 0 6d :3 0
e6 :3 0 166 :3 0
1218 1219 0 25
:3 0 23 :3 0 9fb
121a 121d 1217 121e
0 13c4 31 :3 0
f1 :2 0 2c :2 0
9fe 1221 1223 :3 0
1220 1224 0 13c4
60 :3 0 167 :3 0
1f :3 0 3e :3 0
1c :3 0 27 :3 0
29 :3 0 61 :3 0
63 :3 0 31 :3 0
30 :3 0 2e :3 0
a00 1227 1232 1226
1233 0 13c4 7e
:3 0 49 :3 0 2f
:2 0 a0b 1235 1238
e9 :2 0 2f :2 0
a10 123a 123c :3 0
123d :2 0 33 :3 0
f0 :2 0 a13 1240
1241 :3 0 1242 :2 0
1243 :2 0 123e 1245
1244 :2 0 168 :3 0
169 :3 0 16a :3 0
92 :3 0 1f :3 0
c6 :3 0 1c :3 0
f8 :3 0 b7 :3 0
a0 :3 0 33 :4 0
16b 1 :8 0 1268
111 :4 0 1253 :3 0
e5 :3 0 a3 :2 0
a17 1256 1257 :3 0
10 :3 0 96 :3 0
1259 125a 0 2f
:2 0 125b 125c 0
125e a1a 1265 10
:3 0 96 :3 0 125f
1260 0 112 :2 0
1261 1262 0 1264
a1c 1266 1258 125e
0 1267 0 1264
0 1267 a1e 0
1268 a21 12bb 98
:3 0 10 :3 0 96
:3 0 126a 126b 0
97 :3 0 126c 126d
0 126f a24 1271
a26 1270 126f :2 0
12b9 99 :3 0 10
:3 0 96 :3 0 1274
1275 0 97 :3 0
1276 1277 0 12b1
71 :3 0 72 :3 0
1279 127a 0 71
:3 0 84 :3 0 127c
127d 0 9a :2 0
2 :4 0 16c :2 0
86 :2 0 2c :2 0
a28 1282 1284 :3 0
a2b 127e 1286 127b
1288 1287 :2 0 74
:3 0 87 :3 0 128a
128b 0 76 :4 0
16d :4 0 79 :2 0
7a :3 0 a2f 128f
1291 :3 0 79 :2 0
9d :4 0 a32 1293
1295 :3 0 79 :2 0
9e :4 0 a35 1297
1299 :3 0 79 :2 0
97 :3 0 a38 129b
129d :3 0 79 :2 0
9f :4 0 a3b 129f
12a1 :3 0 79 :2 0
a0 :3 0 a1 :3 0
2c :2 0 a2 :2 0
a3e 12a4 12a8 a42
12a3 12aa :3 0 a45
128c 12ac :2 0 12ae
a48 12af 1289 12ae
0 12b0 a4a 0
12b1 a4c 12b4 :3 0
12b4 0 12b4 12b3
12b1 12b2 :6 0 12b6
28 :3 0 a4f 12b8
a51 12b7 12b6 :2 0
12b9 a53 :2 0 12bb
0 12bb 12ba 1268
12b9 :6 0 12bd 1
:3 0 a56 12be 1246
12bd 0 12bf a58
0 13c4 a0 :3 0
2b :3 0 2c :2 0
2c :2 0 a5a 12c0
12c4 a3 :2 0 16e
:4 0 a60 12c6 12c8
:3 0 4d :3 0 16e
:2 0 12ca 12cb 0
12cd a63 12d2 4d
:3 0 2f :2 0 12ce
12cf 0 12d1 a65
12d3 12c9 12cd 0
12d4 0 12d1 0
12d4 a67 0 13c4
60 :3 0 16f :3 0
1f :3 0 3e :3 0
1c :3 0 3f :3 0
2d :3 0 27 :3 0
29 :3 0 22 :3 0
25 :3 0 4d :3 0
4c :3 0 44 :3 0
40 :3 0 16e :2 0
6d :3 0 2f :2 0
a6a 12d6 12e7 12d5
12e8 0 13c4 b7
:3 0 a0 :3 0 22
:3 0 b7 :3 0 a0
:3 0 25 :3 0 b8
:3 0 b9 :3 0 ba
:3 0 bc :3 0 27
:3 0 29 :3 0 2e
:3 0 30 :3 0 61
:3 0 63 :3 0 c5
:3 0 c6 :3 0 1c
:4 0 170 1 :8 0
1303 10 :3 0 96
:3 0 12fe 12ff 0
97 :3 0 1300 1301
0 1303 a7b 1356
98 :3 0 10 :3 0
96 :3 0 1305 1306
0 97 :3 0 1307
1308 0 130a a7e
130c a80 130b 130a
:2 0 1354 99 :3 0
10 :3 0 96 :3 0
130f 1310 0 97
:3 0 1311 1312 0
134c 71 :3 0 72
:3 0 1314 1315 0
71 :3 0 84 :3 0
1317 1318 0 9a
:2 0 2 :4 0 171
:2 0 86 :2 0 2c
:2 0 a82 131d 131f
:3 0 a85 1319 1321
1316 1323 1322 :2 0
74 :3 0 87 :3 0
1325 1326 0 76
:4 0 172 :4 0 79
:2 0 7a :3 0 a89
132a 132c :3 0 79
:2 0 9d :4 0 a8c
132e 1330 :3 0 79
:2 0 9e :4 0 a8f
1332 1334 :3 0 79
:2 0 97 :3 0 a92
1336 1338 :3 0 79
:2 0 9f :4 0 a95
133a 133c :3 0 79
:2 0 a0 :3 0 a1
:3 0 2c :2 0 a2
:2 0 a98 133f 1343
a9c 133e 1345 :3 0
a9f 1327 1347 :2 0
1349 aa2 134a 1324
1349 0 134b aa4
0 134c aa6 134f
:3 0 134f 0 134f
134e 134c 134d :6 0
1351 2a :3 0 aa9
1353 aab 1352 1351
:2 0 1354 aad :2 0
1356 0 1356 1355
1303 1354 :6 0 13c4
1 :3 0 31 :3 0
2f :2 0 1358 1359
0 13c4 60 :3 0
167 :3 0 1f :3 0
3e :3 0 1c :3 0
27 :3 0 29 :3 0
61 :3 0 63 :3 0
31 :3 0 30 :3 0
2e :3 0 ab0 135c
1367 135b 1368 0
13c4 60 :3 0 11c
:3 0 1f :3 0 abb
136b 136d 136a 136e
0 13c4 10 :3 0
96 :3 0 1370 1371
0 2f :2 0 1372
1373 0 13c4 71
:3 0 72 :3 0 1375
1376 0 71 :3 0
84 :3 0 1378 1379
0 173 :2 0 2
:4 0 174 :2 0 86
:2 0 2c :2 0 abd
137e 1380 :3 0 ac0
137a 1382 1377 1384
1383 :2 0 74 :3 0
87 :3 0 1386 1387
0 76 :4 0 175
:4 0 79 :2 0 7a
:3 0 ac4 138b 138d
:3 0 79 :2 0 9d
:4 0 ac7 138f 1391
:3 0 79 :2 0 dd
:4 0 aca 1393 1395
:3 0 79 :2 0 7e
:3 0 82 :3 0 10
:3 0 96 :3 0 139a
139b 0 acd 1399
139d 81 :4 0 acf
1398 13a0 ad2 1397
13a2 :3 0 79 :2 0
e0 :4 0 ad5 13a4
13a6 :3 0 79 :2 0
7c :4 0 ad8 13a8
13aa :3 0 79 :2 0
1a :3 0 adb 13ac
13ae :3 0 79 :2 0
7c :4 0 ade 13b0
13b2 :3 0 ae1 1388
13b4 :2 0 13b6 ae4
13b7 1385 13b6 0
13b8 ae6 0 13c4
e3 :3 0 176 :3 0
13b9 13ba 0 10
:3 0 96 :3 0 13bc
13bd 0 1a :3 0
e5 :3 0 ae8 13bb
13c1 :2 0 13c4 133
:3 0 aec 13c5 11b1
13c4 0 1755 7e
:3 0 43 :3 0 2f
:2 0 afb 13c6 13c9
7e :3 0 177 :2 0
9 :3 0 2f :2 0
afe 13cb 13cf b03
13cc 13d1 :3 0 43
:3 0 7e :3 0 9
:3 0 2f :2 0 b06
13d4 13d7 f1 :2 0
7e :3 0 43 :3 0
2f :2 0 b09 13da
13dd b0c 13d9 13df
:3 0 13d3 13e0 0
13e6 6c :3 0 2c
:2 0 13e2 13e3 0
13e6 133 :3 0 b0f
13e7 13d2 13e6 0
1755 7e :3 0 9
:3 0 2f :2 0 b12
13e8 13eb a3 :2 0
2f :2 0 b17 13ed
13ef :3 0 43 :3 0
2f :2 0 13f1 13f2
0 1458 6c :3 0
2c :2 0 13f4 13f5
0 1458 cc :3 0
22 :3 0 d7 :3 0
8d :3 0 3e :4 0
178 1 :8 0 1402
10 :3 0 96 :3 0
13fd 13fe 0 97
:3 0 13ff 1400 0
1402 b1a 1455 98
:3 0 10 :3 0 96
:3 0 1404 1405 0
97 :3 0 1406 1407
0 1409 b1d 140b
b1f 140a 1409 :2 0
1453 99 :3 0 10
:3 0 96 :3 0 140e
140f 0 97 :3 0
1410 1411 0 144b
71 :3 0 72 :3 0
1413 1414 0 71
:3 0 84 :3 0 1416
1417 0 9a :2 0
2 :4 0 179 :2 0
86 :2 0 2c :2 0
b21 141c 141e :3 0
b24 1418 1420 1415
1422 1421 :2 0 74
:3 0 87 :3 0 1424
1425 0 76 :4 0
17a :4 0 79 :2 0
7a :3 0 b28 1429
142b :3 0 79 :2 0
9d :4 0 b2b 142d
142f :3 0 79 :2 0
9e :4 0 b2e 1431
1433 :3 0 79 :2 0
97 :3 0 b31 1435
1437 :3 0 79 :2 0
9f :4 0 b34 1439
143b :3 0 79 :2 0
a0 :3 0 a1 :3 0
2c :2 0 a2 :2 0
b37 143e 1442 b3b
143d 1444 :3 0 b3e
1426 1446 :2 0 1448
b41 1449 1423 1448
0 144a b43 0
144b b45 144e :3 0
144e 0 144e 144d
144b 144c :6 0 1450
2c :3 0 b48 1452
b4a 1451 1450 :2 0
1453 b4c :2 0 1455
0 1455 1454 1402
1453 :6 0 1458 1
:3 0 133 :3 0 b4f
1459 13f0 1458 0
1755 7e :3 0 43
:3 0 2f :2 0 b53
145a 145d 7e :3 0
e9 :2 0 9 :3 0
2f :2 0 b56 145f
1463 b5b 1460 1465
:3 0 43 :3 0 7e
:3 0 43 :3 0 2f
:2 0 b5e 1468 146b
f1 :2 0 7e :3 0
9 :3 0 2f :2 0
b61 146e 1471 b64
146d 1473 :3 0 1467
1474 0 1752 32
:4 0 1476 1477 0
1752 6a :3 0 89
:4 0 1479 147a 0
1752 7e :3 0 38
:3 0 2f :2 0 b67
147c 147f a3 :2 0
2c :2 0 b6c 1481
1483 :3 0 6a :3 0
17b :4 0 79 :2 0
17c :4 0 b6f 1487
1489 :3 0 79 :2 0
1f :3 0 b72 148b
148d :3 0 79 :2 0
17d :4 0 b75 148f
1491 :3 0 79 :2 0
22 :3 0 b78 1493
1495 :3 0 79 :2 0
17e :4 0 b7b 1497
1499 :3 0 79 :2 0
17f :4 0 b7e 149b
149d :3 0 79 :2 0
82 :3 0 7e :3 0
43 :3 0 2f :2 0
b81 14a1 14a4 b84
14a0 14a6 b86 149f
14a8 :3 0 1485 14a9
0 14b0 180 :3 0
6e :3 0 6a :3 0
14ac 14ad 0 14ae
:2 0 14b0 b89 14dd
6a :3 0 17b :4 0
79 :2 0 17c :4 0
b8c 14b3 14b5 :3 0
79 :2 0 3f :3 0
b8f 14b7 14b9 :3 0
79 :2 0 17d :4 0
b92 14bb 14bd :3 0
79 :2 0 22 :3 0
b95 14bf 14c1 :3 0
79 :2 0 17e :4 0
b98 14c3 14c5 :3 0
79 :2 0 17f :4 0
b9b 14c7 14c9 :3 0
79 :2 0 82 :3 0
7e :3 0 43 :3 0
2f :2 0 b9e 14cd
14d0 ba1 14cc 14d2
ba3 14cb 14d4 :3 0
14b1 14d5 0 14dc
180 :3 0 6f :3 0
6a :3 0 14d8 14d9
0 14da :2 0 14dc
ba6 14de 1484 14b0
0 14df 0 14dc
0 14df ba9 0
1752 181 :3 0 7e
:3 0 38 :3 0 2f
:2 0 bac 14e1 14e4
a3 :2 0 2c :2 0
bb1 14e6 14e8 :3 0
6e :3 0 32 :4 0
14ed :2 0 1500 14ea
14eb :3 0 6e :3 0
111 :3 0 14ee 14ef
:3 0 10 :3 0 96
:3 0 14f1 14f2 0
2f :2 0 14f3 14f4
0 14f6 bb4 14fd
10 :3 0 96 :3 0
14f7 14f8 0 112
:2 0 14f9 14fa 0
14fc bb6 14fe 14f0
14f6 0 14ff 0
14fc 0 14ff bb8
0 1500 bbb 1518
6f :3 0 32 :4 0
1504 :2 0 1517 1501
1502 :3 0 6f :3 0
111 :3 0 1505 1506
:3 0 10 :3 0 96
:3 0 1508 1509 0
2f :2 0 150a 150b
0 150d bbe 1514
10 :3 0 96 :3 0
150e 150f 0 112
:2 0 1510 1511 0
1513 bc0 1515 1507
150d 0 1516 0
1513 0 1516 bc2
0 1517 bc5 1519
14e9 1500 0 151a
0 1517 0 151a
bc8 0 152a 10
:3 0 96 :3 0 151b
151c 0 10 :3 0
ea :2 0 11b :3 0
151e 1520 0 bcd
151f 1522 :3 0 1523
:2 0 182 :8 0 1527
bd0 1528 1524 1527
0 1529 bd2 0
152a bd4 152c 181
:4 0 152a :4 0 1752
7e :3 0 38 :3 0
2f :2 0 bd7 152d
1530 a3 :2 0 2c
:2 0 bdc 1532 1534
:3 0 183 :3 0 6e
:4 0 1539 :2 0 153f
1537 0 10 :3 0
96 :3 0 153a 153b
0 97 :3 0 153c
153d 0 153f bdf
1592 98 :3 0 10
:3 0 96 :3 0 1541
1542 0 97 :3 0
1543 1544 0 1546
be2 1548 be4 1547
1546 :2 0 1590 99
:3 0 10 :3 0 96
:3 0 154b 154c 0
97 :3 0 154d 154e
0 1588 71 :3 0
72 :3 0 1550 1551
0 71 :3 0 84
:3 0 1553 1554 0
9a :2 0 2 :4 0
184 :2 0 86 :2 0
2c :2 0 be6 1559
155b :3 0 be9 1555
155d 1552 155f 155e
:2 0 74 :3 0 87
:3 0 1561 1562 0
76 :4 0 185 :4 0
79 :2 0 7a :3 0
bed 1566 1568 :3 0
79 :2 0 9d :4 0
bf0 156a 156c :3 0
79 :2 0 9e :4 0
bf3 156e 1570 :3 0
79 :2 0 97 :3 0
bf6 1572 1574 :3 0
79 :2 0 9f :4 0
bf9 1576 1578 :3 0
79 :2 0 a0 :3 0
a1 :3 0 2c :2 0
a2 :2 0 bfc 157b
157f c00 157a 1581
:3 0 c03 1563 1583
:2 0 1585 c06 1586
1560 1585 0 1587
c08 0 1588 c0a
158b :3 0 158b 0
158b 158a 1588 1589
:6 0 158d 2f :3 0
c0d 158f c0f 158e
158d :2 0 1590 c11
:2 0 1592 0 1592
1591 153f 1590 :6 0
1594 1 :3 0 c14
15f4 183 :3 0 6f
:4 0 1598 :2 0 159e
1596 0 10 :3 0
96 :3 0 1599 159a
0 97 :3 0 159b
159c 0 159e c16
15f1 98 :3 0 10
:3 0 96 :3 0 15a0
15a1 0 97 :3 0
15a2 15a3 0 15a5
c19 15a7 c1b 15a6
15a5 :2 0 15ef 99
:3 0 10 :3 0 96
:3 0 15aa 15ab 0
97 :3 0 15ac 15ad
0 15e7 71 :3 0
72 :3 0 15af 15b0
0 71 :3 0 84
:3 0 15b2 15b3 0
9a :2 0 2 :4 0
186 :2 0 86 :2 0
2c :2 0 c1d 15b8
15ba :3 0 c20 15b4
15bc 15b1 15be 15bd
:2 0 74 :3 0 87
:3 0 15c0 15c1 0
76 :4 0 187 :4 0
79 :2 0 7a :3 0
c24 15c5 15c7 :3 0
79 :2 0 9d :4 0
c27 15c9 15cb :3 0
79 :2 0 9e :4 0
c2a 15cd 15cf :3 0
79 :2 0 97 :3 0
c2d 15d1 15d3 :3 0
79 :2 0 9f :4 0
c30 15d5 15d7 :3 0
79 :2 0 a0 :3 0
a1 :3 0 2c :2 0
a2 :2 0 c33 15da
15de c37 15d9 15e0
:3 0 c3a 15c2 15e2
:2 0 15e4 c3d 15e5
15bf 15e4 0 15e6
c3f 0 15e7 c41
15ea :3 0 15ea 0
15ea 15e9 15e7 15e8
:6 0 15ec 31 :3 0
c44 15ee c46 15ed
15ec :2 0 15ef c48
:2 0 15f1 0 15f1
15f0 159e 15ef :6 0
15f3 1 :3 0 c4b
15f5 1535 1594 0
15f6 0 15f3 0
15f6 c4d 0 1752
32 :3 0 188 :2 0
c50 15f8 15f9 :3 0
15fa :2 0 7e :3 0
38 :3 0 2f :2 0
c52 15fc 15ff a3
:2 0 2c :2 0 c57
1601 1603 :3 0 71
:3 0 72 :3 0 1605
1606 0 71 :3 0
84 :3 0 1608 1609
0 13 :2 0 2
:4 0 189 :2 0 86
:2 0 2c :2 0 c5a
160e 1610 :3 0 c5d
160a 1612 1607 1614
1613 :2 0 74 :3 0
87 :3 0 1616 1617
0 76 :4 0 18a
:4 0 79 :2 0 7a
:3 0 c61 161b 161d
:3 0 79 :2 0 9d
:4 0 c64 161f 1621
:3 0 79 :2 0 dd
:4 0 c67 1623 1625
:3 0 79 :2 0 7e
:3 0 82 :3 0 de
:3 0 df :3 0 162a
162b 0 c6a 1629
162d 81 :4 0 c6c
1628 1630 c6f 1627
1632 :3 0 79 :2 0
e0 :4 0 c72 1634
1636 :3 0 79 :2 0
7c :4 0 c75 1638
163a :3 0 79 :2 0
1f :3 0 c78 163c
163e :3 0 79 :2 0
7c :4 0 c7b 1640
1642 :3 0 c7e 1618
1644 :2 0 1646 c81
1647 1615 1646 0
1648 c83 0 1672
71 :3 0 72 :3 0
1649 164a 0 71
:3 0 84 :3 0 164c
164d 0 2c :2 0
2 :4 0 18b :2 0
86 :2 0 2c :2 0
c85 1652 1654 :3 0
c88 164e 1656 164b
1658 1657 :2 0 74
:3 0 87 :3 0 165a
165b 0 76 :4 0
18c :4 0 79 :2 0
7a :3 0 c8c 165f
1661 :3 0 c8f 165c
1663 :2 0 1665 c92
1666 1659 1665 0
1667 c94 0 1672
e3 :3 0 e4 :3 0
1668 1669 0 de
:3 0 df :3 0 166b
166c 0 1f :3 0
e5 :3 0 c96 166a
1670 :2 0 1672 c9a
16e1 71 :3 0 72
:3 0 1673 1674 0
71 :3 0 84 :3 0
1676 1677 0 13
:2 0 2 :4 0 18d
:2 0 86 :2 0 2c
:2 0 c9e 167c 167e
:3 0 ca1 1678 1680
1675 1682 1681 :2 0
74 :3 0 87 :3 0
1684 1685 0 76
:4 0 18e :4 0 79
:2 0 7a :3 0 ca5
1689 168b :3 0 79
:2 0 9d :4 0 ca8
168d 168f :3 0 79
:2 0 dd :4 0 cab
1691 1693 :3 0 79
:2 0 7e :3 0 82
:3 0 de :3 0 df
:3 0 1698 1699 0
cae 1697 169b 81
:4 0 cb0 1696 169e
cb3 1695 16a0 :3 0
79 :2 0 e0 :4 0
cb6 16a2 16a4 :3 0
79 :2 0 7c :4 0
cb9 16a6 16a8 :3 0
79 :2 0 3f :3 0
cbc 16aa 16ac :3 0
79 :2 0 7c :4 0
cbf 16ae 16b0 :3 0
cc2 1686 16b2 :2 0
16b4 cc5 16b5 1683
16b4 0 16b6 cc7
0 16e0 71 :3 0
72 :3 0 16b7 16b8
0 71 :3 0 84
:3 0 16ba 16bb 0
2c :2 0 2 :4 0
18f :2 0 86 :2 0
2c :2 0 cc9 16c0
16c2 :3 0 ccc 16bc
16c4 16b9 16c6 16c5
:2 0 74 :3 0 87
:3 0 16c8 16c9 0
76 :4 0 190 :4 0
79 :2 0 7a :3 0
cd0 16cd 16cf :3 0
cd3 16ca 16d1 :2 0
16d3 cd6 16d4 16c7
16d3 0 16d5 cd8
0 16e0 e3 :3 0
e4 :3 0 16d6 16d7
0 de :3 0 df
:3 0 16d9 16da 0
3f :3 0 e5 :3 0
cda 16d8 16de :2 0
16e0 cde 16e2 1604
1672 0 16e3 0
16e0 0 16e3 ce2
0 16e4 ce5 16e5
15fb 16e4 0 16e6
ce7 0 1752 e6
:3 0 f7 :3 0 b7
:3 0 a0 :3 0 32
:3 0 e6 :3 0 f9
:3 0 22 :3 0 22
:3 0 101 :4 0 191
1 :8 0 16f7 10
:3 0 96 :3 0 16f2
16f3 0 97 :3 0
16f4 16f5 0 16f7
ce9 174a 98 :3 0
10 :3 0 96 :3 0
16f9 16fa 0 97
:3 0 16fb 16fc 0
16fe cec 1700 cee
16ff 16fe :2 0 1748
99 :3 0 10 :3 0
96 :3 0 1703 1704
0 97 :3 0 1705
1706 0 1740 71
:3 0 72 :3 0 1708
1709 0 71 :3 0
84 :3 0 170b 170c
0 9a :2 0 2
:4 0 192 :2 0 86
:2 0 2c :2 0 cf0
1711 1713 :3 0 cf3
170d 1715 170a 1717
1716 :2 0 74 :3 0
87 :3 0 1719 171a
0 76 :4 0 193
:4 0 79 :2 0 7a
:3 0 cf7 171e 1720
:3 0 79 :2 0 9d
:4 0 cfa 1722 1724
:3 0 79 :2 0 9e
:4 0 cfd 1726 1728
:3 0 79 :2 0 97
:3 0 d00 172a 172c
:3 0 79 :2 0 9f
:4 0 d03 172e 1730
:3 0 79 :2 0 a0
:3 0 a1 :3 0 2c
:2 0 a2 :2 0 d06
1733 1737 d0a 1732
1739 :3 0 d0d 171b
173b :2 0 173d d10
173e 1718 173d 0
173f d12 0 1740
d14 1743 :3 0 1743
0 1743 1742 1740
1741 :6 0 1745 33
:3 0 d17 1747 d19
1746 1745 :2 0 1748
d1b :2 0 174a 0
174a 1749 16f7 1748
:6 0 1752 1 :3 0
43 :3 0 2f :2 0
174c 174d 0 1752
6c :3 0 2c :2 0
174f 1750 0 1752
d1e 1753 1466 1752
0 1755 1136 11a5
0 1755 d29 0
1757 133 :3 0 d2f
1a00 7e :3 0 b
:3 0 2f :2 0 d31
1758 175b 7e :3 0
ea :2 0 44 :3 0
2f :2 0 d34 175d
1761 d39 175e 1763
:3 0 7e :3 0 46
:3 0 2f :2 0 d3c
1765 1768 7e :3 0
a3 :2 0 41 :3 0
2f :2 0 d3f 176a
176e d44 176b 1770
:3 0 71 :3 0 72
:3 0 1772 1773 0
71 :3 0 84 :3 0
1775 1776 0 13
:2 0 2 :4 0 194
:2 0 86 :2 0 2c
:2 0 d47 177b 177d
:3 0 d4a 1777 177f
1774 1781 1780 :2 0
74 :3 0 87 :3 0
1783 1784 0 76
:4 0 195 :4 0 79
:2 0 7a :3 0 d4e
1788 178a :3 0 79
:2 0 9d :4 0 d51
178c 178e :3 0 79
:2 0 dd :4 0 d54
1790 1792 :3 0 79
:2 0 7e :3 0 82
:3 0 10 :3 0 96
:3 0 1797 1798 0
d57 1796 179a 81
:4 0 d59 1795 179d
d5c 1794 179f :3 0
79 :2 0 e0 :4 0
d5f 17a1 17a3 :3 0
79 :2 0 7c :4 0
d62 17a5 17a7 :3 0
79 :2 0 1c :3 0
d65 17a9 17ab :3 0
79 :2 0 7c :4 0
d68 17ad 17af :3 0
d6b 1785 17b1 :2 0
17b3 d6e 17b4 1782
17b3 0 17b5 d70
0 17df 71 :3 0
72 :3 0 17b6 17b7
0 71 :3 0 84
:3 0 17b9 17ba 0
2c :2 0 2 :4 0
196 :2 0 86 :2 0
2c :2 0 d72 17bf
17c1 :3 0 d75 17bb
17c3 17b8 17c5 17c4
:2 0 74 :3 0 87
:3 0 17c7 17c8 0
76 :4 0 197 :4 0
79 :2 0 7a :3 0
d79 17cc 17ce :3 0
d7c 17c9 17d0 :2 0
17d2 d7f 17d3 17c6
17d2 0 17d4 d81
0 17df e3 :3 0
e4 :3 0 17d5 17d6
0 10 :3 0 96
:3 0 17d8 17d9 0
1c :3 0 e5 :3 0
d83 17d7 17dd :2 0
17df d87 17e0 1771
17df 0 17e1 d8b
0 19fe 7e :3 0
b :3 0 2f :2 0
d8d 17e2 17e5 a3
:2 0 2f :2 0 d92
17e7 17e9 :3 0 44
:3 0 112 :2 0 17eb
17ec 0 17ee d95
1868 7e :3 0 b
:3 0 2f :2 0 d97
17ef 17f2 177 :2 0
2c :2 0 d9c 17f4
17f6 :3 0 17f7 :2 0
7e :3 0 b :3 0
2f :2 0 d9f 17f9
17fc e9 :2 0 198
:2 0 da4 17fe 1800
:3 0 1801 :2 0 17f8
1803 1802 :2 0 71
:3 0 72 :3 0 1805
1806 0 71 :3 0
84 :3 0 1808 1809
0 134 :2 0 2
:4 0 199 :2 0 86
:2 0 2c :2 0 da7
180e 1810 :3 0 daa
180a 1812 1807 1814
1813 :2 0 74 :3 0
87 :3 0 1816 1817
0 76 :4 0 19a
:4 0 79 :2 0 7a
:3 0 dae 181b 181d
:3 0 79 :2 0 9d
:4 0 db1 181f 1821
:3 0 79 :2 0 dd
:4 0 db4 1823 1825
:3 0 79 :2 0 7e
:3 0 82 :3 0 de
:3 0 19b :3 0 182a
182b 0 db7 1829
182d 81 :4 0 db9
1828 1830 dbc 1827
1832 :3 0 dbf 1818
1834 :2 0 1836 dc2
1837 1815 1836 0
1838 dc4 0 1861
71 :3 0 72 :3 0
1839 183a 0 71
:3 0 84 :3 0 183c
183d 0 2c :2 0
2 :4 0 19c :2 0
86 :2 0 2c :2 0
dc6 1842 1844 :3 0
dc9 183e 1846 183b
1848 1847 :2 0 74
:3 0 87 :3 0 184a
184b 0 76 :4 0
19d :4 0 79 :2 0
7a :3 0 dcd 184f
1851 :3 0 dd0 184c
1853 :2 0 1855 dd3
1856 1849 1855 0
1857 dd5 0 1861
e3 :3 0 13a :3 0
1858 1859 0 de
:3 0 19b :3 0 185b
185c 0 e5 :3 0
dd7 185a 185f :2 0
1861 dda 1862 1804
1861 0 1863 dde
0 1867 44 :3 0
b :3 0 1864 1865
0 1867 de0 1869
17ea 17ee 0 186a
0 1867 0 186a
de3 0 19fe 1d
:3 0 f0 :2 0 de6
186c 186d :3 0 186e
:2 0 ae :3 0 60
:3 0 c5 :3 0 c6
:3 0 1d :4 0 19e
1 :8 0 187b 10
:3 0 96 :3 0 1876
1877 0 97 :3 0
1878 1879 0 187b
de8 18ce 98 :3 0
10 :3 0 96 :3 0
187d 187e 0 97
:3 0 187f 1880 0
1882 deb 1884 ded
1883 1882 :2 0 18cc
99 :3 0 10 :3 0
96 :3 0 1887 1888
0 97 :3 0 1889
188a 0 18c4 71
:3 0 72 :3 0 188c
188d 0 71 :3 0
84 :3 0 188f 1890
0 9a :2 0 2
:4 0 19f :2 0 86
:2 0 2c :2 0 def
1895 1897 :3 0 df2
1891 1899 188e 189b
189a :2 0 74 :3 0
87 :3 0 189d 189e
0 76 :4 0 1a0
:4 0 79 :2 0 7a
:3 0 df6 18a2 18a4
:3 0 79 :2 0 9d
:4 0 df9 18a6 18a8
:3 0 79 :2 0 9e
:4 0 dfc 18aa 18ac
:3 0 79 :2 0 97
:3 0 dff 18ae 18b0
:3 0 79 :2 0 9f
:4 0 e02 18b2 18b4
:3 0 79 :2 0 a0
:3 0 a1 :3 0 2c
:2 0 a2 :2 0 e05
18b7 18bb e09 18b6
18bd :3 0 e0c 189f
18bf :2 0 18c1 e0f
18c2 189c 18c1 0
18c3 e11 0 18c4
e13 18c7 :3 0 18c7
0 18c7 18c6 18c4
18c5 :6 0 18c9 35
:3 0 e16 18cb e18
18ca 18c9 :2 0 18cc
e1a :2 0 18ce 0
18ce 18cd 187b 18cc
:6 0 195a 1 :3 0
10 :3 0 96 :3 0
18d0 18d1 0 10
:3 0 a3 :2 0 11b
:3 0 18d3 18d5 0
e1f 18d4 18d7 :3 0
18d8 :2 0 7e :3 0
44 :3 0 2f :2 0
e22 18da 18dd 7e
:3 0 e9 :2 0 60
:3 0 2f :2 0 e25
18df 18e3 e2a 18e0
18e5 :3 0 18e6 :2 0
18d9 18e8 18e7 :2 0
71 :3 0 72 :3 0
18ea 18eb 0 71
:3 0 84 :3 0 18ed
18ee 0 13 :2 0
2 :4 0 1a1 :2 0
86 :2 0 2c :2 0
e2d 18f3 18f5 :3 0
e30 18ef 18f7 18ec
18f9 18f8 :2 0 74
:3 0 87 :3 0 18fb
18fc 0 76 :4 0
1a2 :4 0 79 :2 0
7a :3 0 e34 1900
1902 :3 0 79 :2 0
9d :4 0 e37 1904
1906 :3 0 79 :2 0
dd :4 0 e3a 1908
190a :3 0 79 :2 0
7e :3 0 82 :3 0
de :3 0 1a3 :3 0
190f 1910 0 e3d
190e 1912 81 :4 0
e3f 190d 1915 e42
190c 1917 :3 0 79
:2 0 e0 :4 0 e45
1919 191b :3 0 79
:2 0 7c :4 0 e48
191d 191f :3 0 79
:2 0 1d :3 0 e4b
1921 1923 :3 0 79
:2 0 7c :4 0 e4e
1925 1927 :3 0 e51
18fd 1929 :2 0 192b
e54 192c 18fa 192b
0 192d e56 0
1957 71 :3 0 72
:3 0 192e 192f 0
71 :3 0 84 :3 0
1931 1932 0 2c
:2 0 2 :4 0 1a4
:2 0 86 :2 0 2c
:2 0 e58 1937 1939
:3 0 e5b 1933 193b
1930 193d 193c :2 0
74 :3 0 87 :3 0
193f 1940 0 76
:4 0 1a5 :4 0 79
:2 0 7a :3 0 e5f
1944 1946 :3 0 e62
1941 1948 :2 0 194a
e65 194b 193e 194a
0 194c e67 0
1957 e3 :3 0 e4
:3 0 194d 194e 0
de :3 0 1a3 :3 0
1950 1951 0 1d
:3 0 e5 :3 0 e69
194f 1955 :2 0 1957
e6d 1958 18e9 1957
0 1959 e71 0
195a e73 195b 186f
195a 0 195c e76
0 19fe 43 :3 0
2f :2 0 195d 195e
0 19fe 6c :3 0
16e :2 0 1960 1961
0 19fe 7e :3 0
50 :3 0 2f :2 0
e78 1963 1966 177
:2 0 1a6 :2 0 e7d
1968 196a :3 0 50
:3 0 2c :2 0 196c
196d 0 196f e80
1970 196b 196f 0
1971 e82 0 19fe
c5 :3 0 1a7 :3 0
ae :3 0 44 :3 0
a9 :3 0 ab :3 0
a8 :3 0 ac :3 0
c4 :3 0 fa :3 0
1a8 :3 0 c4 :3 0
de :3 0 1a9 :3 0
de :3 0 1aa :3 0
c6 :3 0 1c :4 0
1ab 1 :8 0 199a
111 :4 0 1985 :3 0
e5 :3 0 a3 :2 0
e86 1988 1989 :3 0
10 :3 0 96 :3 0
198b 198c 0 2f
:2 0 198d 198e 0
1990 e89 1997 10
:3 0 96 :3 0 1991
1992 0 112 :2 0
1993 1994 0 1996
e8b 1998 198a 1990
0 1999 0 1996
0 1999 e8d 0
199a e90 19ed 98
:3 0 10 :3 0 96
:3 0 199c 199d 0
97 :3 0 199e 199f
0 19a1 e93 19a3
e95 19a2 19a1 :2 0
19eb 99 :3 0 10
:3 0 96 :3 0 19a6
19a7 0 97 :3 0
19a8 19a9 0 19e3
71 :3 0 72 :3 0
19ab 19ac 0 71
:3 0 84 :3 0 19ae
19af 0 9a :2 0
2 :4 0 1ac :2 0
86 :2 0 2c :2 0
e97 19b4 19b6 :3 0
e9a 19b0 19b8 19ad
19ba 19b9 :2 0 74
:3 0 87 :3 0 19bc
19bd 0 76 :4 0
1ad :4 0 79 :2 0
7a :3 0 e9e 19c1
19c3 :3 0 79 :2 0
9d :4 0 ea1 19c5
19c7 :3 0 79 :2 0
9e :4 0 ea4 19c9
19cb :3 0 79 :2 0
97 :3 0 ea7 19cd
19cf :3 0 79 :2 0
9f :4 0 eaa 19d1
19d3 :3 0 79 :2 0
a0 :3 0 a1 :3 0
2c :2 0 a2 :2 0
ead 19d6 19da eb1
19d5 19dc :3 0 eb4
19be 19de :2 0 19e0
eb7 19e1 19bb 19e0
0 19e2 eb9 0
19e3 ebb 19e6 :3 0
19e6 0 19e6 19e5
19e3 19e4 :6 0 19e8
37 :3 0 ebe 19ea
ec0 19e9 19e8 :2 0
19eb ec2 :2 0 19ed
0 19ed 19ec 199a
19eb :6 0 19fe 1
:3 0 7e :3 0 42
:3 0 2f :2 0 ec5
19ef 19f2 ea :2 0
2f :2 0 eca 19f4
19f6 :3 0 42 :3 0
2f :2 0 19f8 19f9
0 19fb ecd 19fc
19f7 19fb 0 19fd
ecf 0 19fe ed1
19ff 1764 19fe 0
1a01 112a 1757 0
1a01 eda 0 2399
6c :3 0 e9 :2 0
2f :2 0 edf 1a03
1a05 :3 0 31 :3 0
f1 :2 0 2c :2 0
ee2 1a08 1a0a :3 0
1a07 1a0b 0 1d00
5f :3 0 167 :3 0
1f :3 0 3e :3 0
1c :3 0 27 :3 0
29 :3 0 61 :3 0
63 :3 0 31 :3 0
30 :3 0 2e :3 0
ee4 1a0e 1a19 1a0d
1a1a 0 1d00 7e
:3 0 49 :3 0 2f
:2 0 eef 1a1c 1a1f
e9 :2 0 2f :2 0
ef4 1a21 1a23 :3 0
1a24 :2 0 33 :3 0
f0 :2 0 ef7 1a27
1a28 :3 0 1a29 :2 0
1a2a :2 0 1a25 1a2c
1a2b :2 0 60 :3 0
1ae :3 0 1f :3 0
3e :3 0 3f :3 0
1d :3 0 1c :3 0
33 :3 0 49 :3 0
2f :2 0 ef9 1a2f
1a38 1a2e 1a39 0
1a3b f02 1a3c 1a2d
1a3b 0 1a3d f04
0 1d00 1d :3 0
f0 :2 0 f06 1a3f
1a40 :3 0 1a41 :2 0
5f :3 0 1af :3 0
1f :3 0 1c :3 0
16e :2 0 f08 1a44
1a48 1a43 1a49 0
1a4b f0c 1a6c a0
:3 0 2b :3 0 2c
:2 0 2c :2 0 f0e
1a4c 1a50 a3 :2 0
16e :4 0 f14 1a52
1a54 :3 0 5f :3 0
1af :3 0 1f :3 0
1c :3 0 2f :2 0
f17 1a57 1a5b 1a56
1a5c 0 1a5e f1b
1a68 5f :3 0 1af
:3 0 1f :3 0 1c
:3 0 2c :2 0 f1d
1a60 1a64 1a5f 1a65
0 1a67 f21 1a69
1a55 1a5e 0 1a6a
0 1a67 0 1a6a
f23 0 1a6b f26
1a6d 1a42 1a4b 0
1a6e 0 1a6b 0
1a6e f28 0 1d00
33 :3 0 f0 :2 0
f2b 1a70 1a71 :3 0
1a72 :2 0 7e :3 0
42 :3 0 2f :2 0
f2d 1a74 1a77 e9
:2 0 2f :2 0 f32
1a79 1a7b :3 0 42
:3 0 1b0 :3 0 1a8
:3 0 1a7e 1a7f 0
7e :3 0 42 :3 0
2f :2 0 f35 1a81
1a84 de :3 0 1b1
:3 0 1a86 1a87 0
f38 1a80 1a89 1a8a
:2 0 1a7d 1a8b 0
1af2 e6 :3 0 f7
:3 0 b7 :3 0 a0
:3 0 33 :3 0 e6
:3 0 f9 :3 0 22
:3 0 22 :3 0 101
:4 0 1b2 1 :8 0
1a9d 10 :3 0 96
:3 0 1a98 1a99 0
97 :3 0 1a9a 1a9b
0 1a9d f3b 1af0
98 :3 0 10 :3 0
96 :3 0 1a9f 1aa0
0 97 :3 0 1aa1
1aa2 0 1aa4 f3e
1aa6 f40 1aa5 1aa4
:2 0 1aee 99 :3 0
10 :3 0 96 :3 0
1aa9 1aaa 0 97
:3 0 1aab 1aac 0
1ae6 71 :3 0 72
:3 0 1aae 1aaf 0
71 :3 0 84 :3 0
1ab1 1ab2 0 9a
:2 0 2 :4 0 1b3
:2 0 86 :2 0 2c
:2 0 f42 1ab7 1ab9
:3 0 f45 1ab3 1abb
1ab0 1abd 1abc :2 0
74 :3 0 87 :3 0
1abf 1ac0 0 76
:4 0 1b4 :4 0 79
:2 0 7a :3 0 f49
1ac4 1ac6 :3 0 79
:2 0 9d :4 0 f4c
1ac8 1aca :3 0 79
:2 0 9e :4 0 f4f
1acc 1ace :3 0 79
:2 0 97 :3 0 f52
1ad0 1ad2 :3 0 79
:2 0 9f :4 0 f55
1ad4 1ad6 :3 0 79
:2 0 a0 :3 0 a1
:3 0 2c :2 0 a2
:2 0 f58 1ad9 1add
f5c 1ad8 1adf :3 0
f5f 1ac1 1ae1 :2 0
1ae3 f62 1ae4 1abe
1ae3 0 1ae5 f64
0 1ae6 f66 1ae9
:3 0 1ae9 0 1ae9
1ae8 1ae6 1ae7 :6 0
1aeb 39 :3 0 f69
1aed f6b 1aec 1aeb
:2 0 1aee f6d :2 0
1af0 0 1af0 1aef
1a9d 1aee :6 0 1af2
1 :3 0 f70 1b59
e6 :3 0 f7 :3 0
b7 :3 0 a0 :3 0
33 :3 0 e6 :3 0
f9 :3 0 22 :3 0
22 :3 0 101 :4 0
1b5 1 :8 0 1b03
10 :3 0 96 :3 0
1afe 1aff 0 97
:3 0 1b00 1b01 0
1b03 f73 1b56 98
:3 0 10 :3 0 96
:3 0 1b05 1b06 0
97 :3 0 1b07 1b08
0 1b0a f76 1b0c
f78 1b0b 1b0a :2 0
1b54 99 :3 0 10
:3 0 96 :3 0 1b0f
1b10 0 97 :3 0
1b11 1b12 0 1b4c
71 :3 0 72 :3 0
1b14 1b15 0 71
:3 0 84 :3 0 1b17
1b18 0 9a :2 0
2 :4 0 1b6 :2 0
86 :2 0 2c :2 0
f7a 1b1d 1b1f :3 0
f7d 1b19 1b21 1b16
1b23 1b22 :2 0 74
:3 0 87 :3 0 1b25
1b26 0 76 :4 0
1b7 :4 0 79 :2 0
7a :3 0 f81 1b2a
1b2c :3 0 79 :2 0
9d :4 0 f84 1b2e
1b30 :3 0 79 :2 0
9e :4 0 f87 1b32
1b34 :3 0 79 :2 0
97 :3 0 f8a 1b36
1b38 :3 0 79 :2 0
9f :4 0 f8d 1b3a
1b3c :3 0 79 :2 0
a0 :3 0 a1 :3 0
2c :2 0 a2 :2 0
f90 1b3f 1b43 f94
1b3e 1b45 :3 0 f97
1b27 1b47 :2 0 1b49
f9a 1b4a 1b24 1b49
0 1b4b f9c 0
1b4c f9e 1b4f :3 0
1b4f 0 1b4f 1b4e
1b4c 1b4d :6 0 1b51
3b :3 0 fa1 1b53
fa3 1b52 1b51 :2 0
1b54 fa5 :2 0 1b56
0 1b56 1b55 1b03
1b54 :6 0 1b58 1
:3 0 fa8 1b5a 1a7c
1af2 0 1b5b 0
1b58 0 1b5b faa
0 1b6b 41 :3 0
7e :3 0 41 :3 0
2f :2 0 fad 1b5d
1b60 f1 :2 0 7e
:3 0 46 :3 0 2f
:2 0 fb0 1b63 1b66
fb3 1b62 1b68 :3 0
1b5c 1b69 0 1b6b
fb6 1b6c 1a73 1b6b
0 1b6d fb9 0
1d00 5e :3 0 16e
:2 0 1b6e 1b6f 0
1d00 5f :3 0 1b8
:3 0 1c :3 0 22
:3 0 41 :3 0 5e
:3 0 42 :3 0 40
:3 0 7e :3 0 43
:3 0 2f :2 0 fbb
1b79 1b7c 7e :3 0
44 :3 0 2f :2 0
fbe 1b7e 1b81 fc1
1b72 1b83 1b71 1b84
0 1d00 7e :3 0
5f :3 0 2f :2 0
fca 1b86 1b89 ea
:2 0 2f :2 0 fcf
1b8b 1b8d :3 0 71
:3 0 72 :3 0 1b8f
1b90 0 71 :3 0
84 :3 0 1b92 1b93
0 13 :2 0 2
:4 0 1b9 :2 0 86
:2 0 2c :2 0 fd2
1b98 1b9a :3 0 fd5
1b94 1b9c 1b91 1b9e
1b9d :2 0 74 :3 0
87 :3 0 1ba0 1ba1
0 76 :4 0 1ba
:4 0 79 :2 0 7a
:3 0 fd9 1ba5 1ba7
:3 0 79 :2 0 9d
:4 0 fdc 1ba9 1bab
:3 0 79 :2 0 dd
:4 0 fdf 1bad 1baf
:3 0 79 :2 0 7e
:3 0 82 :3 0 de
:3 0 1a3 :3 0 1bb4
1bb5 0 fe2 1bb3
1bb7 81 :4 0 fe4
1bb2 1bba fe7 1bb1
1bbc :3 0 79 :2 0
e0 :4 0 fea 1bbe
1bc0 :3 0 79 :2 0
7c :4 0 fed 1bc2
1bc4 :3 0 79 :2 0
1c :3 0 ff0 1bc6
1bc8 :3 0 79 :2 0
7c :4 0 ff3 1bca
1bcc :3 0 ff6 1ba2
1bce :2 0 1bd0 ff9
1bd1 1b9f 1bd0 0
1bd2 ffb 0 1bfd
71 :3 0 72 :3 0
1bd3 1bd4 0 71
:3 0 84 :3 0 1bd6
1bd7 0 2c :2 0
2 :4 0 1bb :2 0
86 :2 0 2c :2 0
ffd 1bdc 1bde :3 0
1000 1bd8 1be0 1bd5
1be2 1be1 :2 0 74
:3 0 87 :3 0 1be4
1be5 0 76 :4 0
1bc :4 0 79 :2 0
7a :3 0 1004 1be9
1beb :3 0 1007 1be6
1bed :2 0 1bef 100a
1bf0 1be3 1bef 0
1bf1 100c 0 1bfd
e3 :3 0 e4 :3 0
1bf2 1bf3 0 de
:3 0 1a3 :3 0 1bf5
1bf6 0 1c :3 0
e5 :3 0 100e 1bf4
1bfa :2 0 1bfd 133
:3 0 1012 1c8b 6c
:3 0 a3 :2 0 2c
:2 0 1018 1bff 1c01
:3 0 c5 :3 0 1bd
:3 0 c6 :3 0 1c
:4 0 1be 1 :8 0
1c89 c5 :3 0 1a7
:3 0 c4 :3 0 fa
:3 0 1a8 :3 0 c4
:3 0 de :3 0 1a9
:3 0 de :3 0 1aa
:3 0 1bf :3 0 7e
:3 0 10d :3 0 115
:3 0 1c0 :3 0 7e
:3 0 10e :3 0 116
:3 0 1bd :3 0 1c1
:3 0 c6 :3 0 1c
:4 0 1c2 1 :8 0
1c34 111 :4 0 1c1f
:3 0 e5 :3 0 a3
:2 0 101d 1c22 1c23
:3 0 10 :3 0 96
:3 0 1c25 1c26 0
2f :2 0 1c27 1c28
0 1c2a 1020 1c31
10 :3 0 96 :3 0
1c2b 1c2c 0 112
:2 0 1c2d 1c2e 0
1c30 1022 1c32 1c24
1c2a 0 1c33 0
1c30 0 1c33 1024
0 1c34 1027 1c87
98 :3 0 10 :3 0
96 :3 0 1c36 1c37
0 97 :3 0 1c38
1c39 0 1c3b 102a
1c3d 102c 1c3c 1c3b
:2 0 1c85 99 :3 0
10 :3 0 96 :3 0
1c40 1c41 0 97
:3 0 1c42 1c43 0
1c7d 71 :3 0 72
:3 0 1c45 1c46 0
71 :3 0 84 :3 0
1c48 1c49 0 9a
:2 0 2 :4 0 1c3
:2 0 86 :2 0 2c
:2 0 102e 1c4e 1c50
:3 0 1031 1c4a 1c52
1c47 1c54 1c53 :2 0
74 :3 0 87 :3 0
1c56 1c57 0 76
:4 0 1c4 :4 0 79
:2 0 7a :3 0 1035
1c5b 1c5d :3 0 79
:2 0 9d :4 0 1038
1c5f 1c61 :3 0 79
:2 0 9e :4 0 103b
1c63 1c65 :3 0 79
:2 0 97 :3 0 103e
1c67 1c69 :3 0 79
:2 0 9f :4 0 1041
1c6b 1c6d :3 0 79
:2 0 a0 :3 0 a1
:3 0 2c :2 0 a2
:2 0 1044 1c70 1c74
1048 1c6f 1c76 :3 0
104b 1c58 1c78 :2 0
1c7a 104e 1c7b 1c55
1c7a 0 1c7c 1050
0 1c7d 1052 1c80
:3 0 1c80 0 1c80
1c7f 1c7d 1c7e :6 0
1c82 3d :3 0 1055
1c84 1057 1c83 1c82
:2 0 1c85 1059 :2 0
1c87 0 1c87 1c86
1c34 1c85 :6 0 1c89
1 :3 0 105c 1c8a
1c02 1c89 0 1c8c
1b8e 1bfd 0 1c8c
105f 0 1d00 60
:3 0 11c :3 0 1f
:3 0 1062 1c8e 1c90
1c8d 1c91 0 1d00
71 :3 0 72 :3 0
1c93 1c94 0 71
:3 0 84 :3 0 1c96
1c97 0 13 :2 0
2 :4 0 1c5 :2 0
86 :2 0 2c :2 0
1064 1c9c 1c9e :3 0
1067 1c98 1ca0 1c95
1ca2 1ca1 :2 0 74
:3 0 87 :3 0 1ca4
1ca5 0 76 :4 0
1c6 :4 0 79 :2 0
7a :3 0 106b 1ca9
1cab :3 0 79 :2 0
9d :4 0 106e 1cad
1caf :3 0 79 :2 0
dd :4 0 1071 1cb1
1cb3 :3 0 79 :2 0
7e :3 0 82 :3 0
60 :3 0 1074 1cb7
1cb9 81 :4 0 1076
1cb6 1cbc 1079 1cb5
1cbe :3 0 79 :2 0
e0 :4 0 107c 1cc0
1cc2 :3 0 79 :2 0
7c :4 0 107f 1cc4
1cc6 :3 0 79 :2 0
1c :3 0 1082 1cc8
1cca :3 0 79 :2 0
7c :4 0 1085 1ccc
1cce :3 0 1088 1ca6
1cd0 :2 0 1cd2 108b
1cd3 1ca3 1cd2 0
1cd4 108d 0 1d00
71 :3 0 72 :3 0
1cd5 1cd6 0 71
:3 0 84 :3 0 1cd8
1cd9 0 2c :2 0
2 :4 0 1c7 :2 0
86 :2 0 2c :2 0
108f 1cde 1ce0 :3 0
1092 1cda 1ce2 1cd7
1ce4 1ce3 :2 0 74
:3 0 87 :3 0 1ce6
1ce7 0 76 :4 0
1c8 :4 0 79 :2 0
7a :3 0 1096 1ceb
1ced :3 0 1099 1ce8
1cef :2 0 1cf1 109c
1cf2 1ce5 1cf1 0
1cf3 109e 0 1d00
e3 :3 0 e4 :3 0
1cf4 1cf5 0 7e
:3 0 60 :3 0 2f
:2 0 10a0 1cf7 1cfa
1c :3 0 e5 :3 0
10a3 1cf6 1cfe :2 0
1d00 10a7 1d01 1a06
1d00 0 1d02 10b4
0 2399 7e :3 0
d :3 0 2f :2 0
10b6 1d03 1d06 7e
:3 0 ea :2 0 46
:3 0 2f :2 0 10b9
1d08 1d0c 10be 1d09
1d0e :3 0 11d :3 0
60 :3 0 101 :3 0
8d :3 0 13d :3 0
c6 :3 0 1c :3 0
c6 :3 0 8d :3 0
13e :3 0 de :3 0
13f :4 0 1c9 1
:8 0 1d22 10 :3 0
96 :3 0 1d1d 1d1e
0 97 :3 0 1d1f
1d20 0 1d22 10c1
1d75 98 :3 0 10
:3 0 96 :3 0 1d24
1d25 0 97 :3 0
1d26 1d27 0 1d29
10c4 1d2b 10c6 1d2a
1d29 :2 0 1d73 99
:3 0 10 :3 0 96
:3 0 1d2e 1d2f 0
97 :3 0 1d30 1d31
0 1d6b 71 :3 0
72 :3 0 1d33 1d34
0 71 :3 0 84
:3 0 1d36 1d37 0
9a :2 0 2 :4 0
1ca :2 0 86 :2 0
2c :2 0 10c8 1d3c
1d3e :3 0 10cb 1d38
1d40 1d35 1d42 1d41
:2 0 74 :3 0 87
:3 0 1d44 1d45 0
76 :4 0 1cb :4 0
79 :2 0 7a :3 0
10cf 1d49 1d4b :3 0
79 :2 0 9d :4 0
10d2 1d4d 1d4f :3 0
79 :2 0 9e :4 0
10d5 1d51 1d53 :3 0
79 :2 0 97 :3 0
10d8 1d55 1d57 :3 0
79 :2 0 9f :4 0
10db 1d59 1d5b :3 0
79 :2 0 a0 :3 0
a1 :3 0 2c :2 0
a2 :2 0 10de 1d5e
1d62 10e2 1d5d 1d64
:3 0 10e5 1d46 1d66
:2 0 1d68 10e8 1d69
1d43 1d68 0 1d6a
10ea 0 1d6b 10ec
1d6e :3 0 1d6e 0
1d6e 1d6d 1d6b 1d6c
:6 0 1d70 3f :3 0
10ef 1d72 10f1 1d71
1d70 :2 0 1d73 10f3
:2 0 1d75 0 1d75
1d74 1d22 1d73 :6 0
21a2 1 :3 0 7e
:3 0 60 :3 0 2f
:2 0 10f6 1d77 1d7a
e9 :2 0 2f :2 0
10fb 1d7c 1d7e :3 0
71 :3 0 72 :3 0
1d80 1d81 0 71
:3 0 84 :3 0 1d83
1d84 0 13 :2 0
2 :4 0 1cc :2 0
86 :2 0 2c :2 0
10fe 1d89 1d8b :3 0
1101 1d85 1d8d 1d82
1d8f 1d8e :2 0 74
:3 0 87 :3 0 1d91
1d92 0 76 :4 0
1cd :4 0 79 :2 0
7a :3 0 1105 1d96
1d98 :3 0 79 :2 0
9d :4 0 1108 1d9a
1d9c :3 0 79 :2 0
dd :4 0 110b 1d9e
1da0 :3 0 79 :2 0
7e :3 0 82 :3 0
de :3 0 145 :3 0
1da5 1da6 0 110e
1da4 1da8 81 :4 0
1110 1da3 1dab 1113
1da2 1dad :3 0 79
:2 0 e0 :4 0 1116
1daf 1db1 :3 0 79
:2 0 7c :4 0 1119
1db3 1db5 :3 0 79
:2 0 1c :3 0 111c
1db7 1db9 :3 0 79
:2 0 7c :4 0 111f
1dbb 1dbd :3 0 1122
1d93 1dbf :2 0 1dc1
1125 1dc2 1d90 1dc1
0 1dc3 1127 0
1ded 71 :3 0 72
:3 0 1dc4 1dc5 0
71 :3 0 84 :3 0
1dc7 1dc8 0 2c
:2 0 2 :4 0 1ce
:2 0 86 :2 0 2c
:2 0 1129 1dcd 1dcf
:3 0 112c 1dc9 1dd1
1dc6 1dd3 1dd2 :2 0
74 :3 0 87 :3 0
1dd5 1dd6 0 76
:4 0 1cf :4 0 79
:2 0 7a :3 0 1130
1dda 1ddc :3 0 1133
1dd7 1dde :2 0 1de0
1136 1de1 1dd4 1de0
0 1de2 1138 0
1ded e3 :3 0 e4
:3 0 1de3 1de4 0
de :3 0 145 :3 0
1de6 1de7 0 1c
:3 0 e5 :3 0 113a
1de5 1deb :2 0 1ded
113e 1dee 1d7f 1ded
0 1def 1142 0
21a2 47 :3 0 d
:3 0 1df0 1df1 0
21a2 54 :3 0 7e
:3 0 41 :3 0 2f
:2 0 1144 1df4 1df7
1df3 1df8 0 21a2
7e :3 0 47 :3 0
2f :2 0 1147 1dfa
1dfd 7e :3 0 e9
:2 0 41 :3 0 2f
:2 0 114a 1dff 1e03
114f 1e00 1e05 :3 0
31 :3 0 f1 :2 0
2c :2 0 1152 1e08
1e0a :3 0 1e07 1e0b
0 20ec 5f :3 0
167 :3 0 1f :3 0
3e :3 0 1c :3 0
27 :3 0 29 :3 0
61 :3 0 63 :3 0
31 :3 0 30 :3 0
2e :3 0 1154 1e0e
1e19 1e0d 1e1a 0
20ec 1d :3 0 f0
:2 0 115f 1e1d 1e1e
:3 0 1e1f :2 0 5f
:3 0 1af :3 0 1f
:3 0 1c :3 0 16e
:2 0 1161 1e22 1e26
1e21 1e27 0 1e29
1165 1e4a a0 :3 0
2b :3 0 2c :2 0
2c :2 0 1167 1e2a
1e2e a3 :2 0 16e
:4 0 116d 1e30 1e32
:3 0 5f :3 0 1af
:3 0 1f :3 0 1c
:3 0 2f :2 0 1170
1e35 1e39 1e34 1e3a
0 1e3c 1174 1e46
5f :3 0 1af :3 0
1f :3 0 1c :3 0
2c :2 0 1176 1e3e
1e42 1e3d 1e43 0
1e45 117a 1e47 1e33
1e3c 0 1e48 0
1e45 0 1e48 117c
0 1e49 117f 1e4b
1e20 1e29 0 1e4c
0 1e49 0 1e4c
1181 0 20ec 4a
:3 0 2f :2 0 1e4d
1e4e 0 20ec 33
:3 0 f0 :2 0 1184
1e51 1e52 :3 0 1e53
:2 0 7e :3 0 42
:3 0 2f :2 0 1186
1e55 1e58 e9 :2 0
2f :2 0 118b 1e5a
1e5c :3 0 42 :3 0
1b0 :3 0 1a8 :3 0
1e5f 1e60 0 7e
:3 0 42 :3 0 2f
:2 0 118e 1e62 1e65
de :3 0 1b1 :3 0
1e67 1e68 0 1191
1e61 1e6a 1e6b :2 0
1e5e 1e6c 0 1ed3
e6 :3 0 f7 :3 0
b7 :3 0 a0 :3 0
33 :3 0 e6 :3 0
f9 :3 0 22 :3 0
22 :3 0 101 :4 0
1d0 1 :8 0 1e7e
10 :3 0 96 :3 0
1e79 1e7a 0 97
:3 0 1e7b 1e7c 0
1e7e 1194 1ed1 98
:3 0 10 :3 0 96
:3 0 1e80 1e81 0
97 :3 0 1e82 1e83
0 1e85 1197 1e87
1199 1e86 1e85 :2 0
1ecf 99 :3 0 10
:3 0 96 :3 0 1e8a
1e8b 0 97 :3 0
1e8c 1e8d 0 1ec7
71 :3 0 72 :3 0
1e8f 1e90 0 71
:3 0 84 :3 0 1e92
1e93 0 9a :2 0
2 :4 0 1d1 :2 0
86 :2 0 2c :2 0
119b 1e98 1e9a :3 0
119e 1e94 1e9c 1e91
1e9e 1e9d :2 0 74
:3 0 87 :3 0 1ea0
1ea1 0 76 :4 0
1d2 :4 0 79 :2 0
7a :3 0 11a2 1ea5
1ea7 :3 0 79 :2 0
9d :4 0 11a5 1ea9
1eab :3 0 79 :2 0
9e :4 0 11a8 1ead
1eaf :3 0 79 :2 0
97 :3 0 11ab 1eb1
1eb3 :3 0 79 :2 0
9f :4 0 11ae 1eb5
1eb7 :3 0 79 :2 0
a0 :3 0 a1 :3 0
2c :2 0 a2 :2 0
11b1 1eba 1ebe 11b5
1eb9 1ec0 :3 0 11b8
1ea2 1ec2 :2 0 1ec4
11bb 1ec5 1e9f 1ec4
0 1ec6 11bd 0
1ec7 11bf 1eca :3 0
1eca 0 1eca 1ec9
1ec7 1ec8 :6 0 1ecc
41 :3 0 11c2 1ece
11c4 1ecd 1ecc :2 0
1ecf 11c6 :2 0 1ed1
0 1ed1 1ed0 1e7e
1ecf :6 0 1ed3 1
:3 0 11c9 1f3a e6
:3 0 f7 :3 0 b7
:3 0 a0 :3 0 33
:3 0 e6 :3 0 f9
:3 0 22 :3 0 22
:3 0 101 :4 0 1d3
1 :8 0 1ee4 10
:3 0 96 :3 0 1edf
1ee0 0 97 :3 0
1ee1 1ee2 0 1ee4
11cc 1f37 98 :3 0
10 :3 0 96 :3 0
1ee6 1ee7 0 97
:3 0 1ee8 1ee9 0
1eeb 11cf 1eed 11d1
1eec 1eeb :2 0 1f35
99 :3 0 10 :3 0
96 :3 0 1ef0 1ef1
0 97 :3 0 1ef2
1ef3 0 1f2d 71
:3 0 72 :3 0 1ef5
1ef6 0 71 :3 0
84 :3 0 1ef8 1ef9
0 9a :2 0 2
:4 0 1d4 :2 0 86
:2 0 2c :2 0 11d3
1efe 1f00 :3 0 11d6
1efa 1f02 1ef7 1f04
1f03 :2 0 74 :3 0
87 :3 0 1f06 1f07
0 76 :4 0 1d5
:4 0 79 :2 0 7a
:3 0 11da 1f0b 1f0d
:3 0 79 :2 0 9d
:4 0 11dd 1f0f 1f11
:3 0 79 :2 0 9e
:4 0 11e0 1f13 1f15
:3 0 79 :2 0 97
:3 0 11e3 1f17 1f19
:3 0 79 :2 0 9f
:4 0 11e6 1f1b 1f1d
:3 0 79 :2 0 a0
:3 0 a1 :3 0 2c
:2 0 a2 :2 0 11e9
1f20 1f24 11ed 1f1f
1f26 :3 0 11f0 1f08
1f28 :2 0 1f2a 11f3
1f2b 1f05 1f2a 0
1f2c 11f5 0 1f2d
11f7 1f30 :3 0 1f30
0 1f30 1f2f 1f2d
1f2e :6 0 1f32 43
:3 0 11fa 1f34 11fc
1f33 1f32 :2 0 1f35
11fe :2 0 1f37 0
1f37 1f36 1ee4 1f35
:6 0 1f39 1 :3 0
1201 1f3b 1e5d 1ed3
0 1f3c 0 1f39
0 1f3c 1203 0
1fb0 7e :3 0 49
:3 0 2f :2 0 1206
1f3d 1f40 e9 :2 0
2f :2 0 120b 1f42
1f44 :3 0 169 :3 0
16a :3 0 4a :3 0
168 :3 0 92 :3 0
1f :3 0 c6 :3 0
1c :3 0 f8 :3 0
b7 :3 0 a0 :3 0
33 :4 0 1d6 1
:8 0 1f58 10 :3 0
96 :3 0 1f53 1f54
0 97 :3 0 1f55
1f56 0 1f58 120e
1fab 98 :3 0 10
:3 0 96 :3 0 1f5a
1f5b 0 97 :3 0
1f5c 1f5d 0 1f5f
1211 1f61 1213 1f60
1f5f :2 0 1fa9 99
:3 0 10 :3 0 96
:3 0 1f64 1f65 0
97 :3 0 1f66 1f67
0 1fa1 71 :3 0
72 :3 0 1f69 1f6a
0 71 :3 0 84
:3 0 1f6c 1f6d 0
9a :2 0 2 :4 0
1d7 :2 0 86 :2 0
2c :2 0 1215 1f72
1f74 :3 0 1218 1f6e
1f76 1f6b 1f78 1f77
:2 0 74 :3 0 87
:3 0 1f7a 1f7b 0
76 :4 0 1d8 :4 0
79 :2 0 7a :3 0
121c 1f7f 1f81 :3 0
79 :2 0 9d :4 0
121f 1f83 1f85 :3 0
79 :2 0 9e :4 0
1222 1f87 1f89 :3 0
79 :2 0 97 :3 0
1225 1f8b 1f8d :3 0
79 :2 0 9f :4 0
1228 1f8f 1f91 :3 0
79 :2 0 a0 :3 0
a1 :3 0 2c :2 0
a2 :2 0 122b 1f94
1f98 122f 1f93 1f9a
:3 0 1232 1f7c 1f9c
:2 0 1f9e 1235 1f9f
1f79 1f9e 0 1fa0
1237 0 1fa1 1239
1fa4 :3 0 1fa4 0
1fa4 1fa3 1fa1 1fa2
:6 0 1fa6 45 :3 0
123c 1fa8 123e 1fa7
1fa6 :2 0 1fa9 1240
:2 0 1fab 0 1fab
1faa 1f58 1fa9 :6 0
1fad 1 :3 0 1243
1fae 1f45 1fad 0
1faf 1245 0 1fb0
1247 1fb1 1e54 1fb0
0 1fb2 124a 0
20ec 41 :3 0 7e
:3 0 47 :3 0 2f
:2 0 124c 1fb4 1fb7
f1 :2 0 7e :3 0
46 :3 0 2f :2 0
124f 1fba 1fbd 1252
1fb9 1fbf :3 0 f1
:2 0 7e :3 0 4a
:3 0 2f :2 0 1255
1fc2 1fc5 1258 1fc1
1fc7 :3 0 1fb3 1fc8
0 20ec 45 :3 0
1d9 :2 0 1fca 1fcb
0 20ec c5 :3 0
1a7 :3 0 c4 :3 0
fa :3 0 1a8 :3 0
c4 :3 0 de :3 0
1a9 :3 0 b4 :3 0
47 :3 0 b0 :3 0
47 :3 0 d4 :3 0
c2 :3 0 d5 :3 0
d6 :3 0 1da :3 0
c6 :3 0 1c :4 0
1db 1 :8 0 1ff6
111 :4 0 1fe1 :3 0
e5 :3 0 a3 :2 0
125d 1fe4 1fe5 :3 0
10 :3 0 96 :3 0
1fe7 1fe8 0 2f
:2 0 1fe9 1fea 0
1fec 1260 1ff3 10
:3 0 96 :3 0 1fed
1fee 0 112 :2 0
1fef 1ff0 0 1ff2
1262 1ff4 1fe6 1fec
0 1ff5 0 1ff2
0 1ff5 1264 0
1ff6 1267 2049 98
:3 0 10 :3 0 96
:3 0 1ff8 1ff9 0
97 :3 0 1ffa 1ffb
0 1ffd 126a 1fff
126c 1ffe 1ffd :2 0
2047 99 :3 0 10
:3 0 96 :3 0 2002
2003 0 97 :3 0
2004 2005 0 203f
71 :3 0 72 :3 0
2007 2008 0 71
:3 0 84 :3 0 200a
200b 0 9a :2 0
2 :4 0 1dc :2 0
86 :2 0 2c :2 0
126e 2010 2012 :3 0
1271 200c 2014 2009
2016 2015 :2 0 74
:3 0 87 :3 0 2018
2019 0 76 :4 0
1dd :4 0 79 :2 0
7a :3 0 1275 201d
201f :3 0 79 :2 0
9d :4 0 1278 2021
2023 :3 0 79 :2 0
9e :4 0 127b 2025
2027 :3 0 79 :2 0
97 :3 0 127e 2029
202b :3 0 79 :2 0
9f :4 0 1281 202d
202f :3 0 79 :2 0
a0 :3 0 a1 :3 0
2c :2 0 a2 :2 0
1284 2032 2036 1288
2031 2038 :3 0 128b
201a 203a :2 0 203c
128e 203d 2017 203c
0 203e 1290 0
203f 1292 2042 :3 0
2042 0 2042 2041
203f 2040 :6 0 2044
47 :3 0 1295 2046
1297 2045 2044 :2 0
2047 1299 :2 0 2049
0 2049 2048 1ff6
2047 :6 0 20ec 1
:3 0 10 :3 0 96
:3 0 204b 204c 0
10 :3 0 a3 :2 0
11b :3 0 204e 2050
0 129e 204f 2052
:3 0 2053 :2 0 5e
:3 0 16e :2 0 2055
2056 0 20e2 60
:3 0 1b8 :3 0 1c
:3 0 22 :3 0 41
:3 0 5e :3 0 42
:3 0 40 :3 0 2f
:2 0 7e :3 0 44
:3 0 2f :2 0 12a1
2061 2064 12a4 2059
2066 2058 2067 0
20e2 7e :3 0 60
:3 0 2f :2 0 12ad
2069 206c ea :2 0
2f :2 0 12b2 206e
2070 :3 0 71 :3 0
72 :3 0 2072 2073
0 71 :3 0 84
:3 0 2075 2076 0
13 :2 0 2 :4 0
1de :2 0 86 :2 0
2c :2 0 12b5 207b
207d :3 0 12b8 2077
207f 2074 2081 2080
:2 0 74 :3 0 87
:3 0 2083 2084 0
76 :4 0 1df :4 0
79 :2 0 7a :3 0
12bc 2088 208a :3 0
79 :2 0 9d :4 0
12bf 208c 208e :3 0
79 :2 0 dd :4 0
12c2 2090 2092 :3 0
79 :2 0 7e :3 0
82 :3 0 60 :3 0
12c5 2096 2098 81
:4 0 12c7 2095 209b
12ca 2094 209d :3 0
79 :2 0 e0 :4 0
12cd 209f 20a1 :3 0
79 :2 0 7c :4 0
12d0 20a3 20a5 :3 0
79 :2 0 1c :3 0
12d3 20a7 20a9 :3 0
79 :2 0 7c :4 0
12d6 20ab 20ad :3 0
12d9 2085 20af :2 0
20b1 12dc 20b2 2082
20b1 0 20b3 12de
0 20df 71 :3 0
72 :3 0 20b4 20b5
0 71 :3 0 84
:3 0 20b7 20b8 0
2c :2 0 2 :4 0
1e0 :2 0 86 :2 0
2c :2 0 12e0 20bd
20bf :3 0 12e3 20b9
20c1 20b6 20c3 20c2
:2 0 74 :3 0 87
:3 0 20c5 20c6 0
76 :4 0 1e1 :4 0
79 :2 0 7a :3 0
12e7 20ca 20cc :3 0
12ea 20c7 20ce :2 0
20d0 12ed 20d1 20c4
20d0 0 20d2 12ef
0 20df e3 :3 0
e4 :3 0 20d3 20d4
0 7e :3 0 60
:3 0 2f :2 0 12f1
20d6 20d9 1c :3 0
e5 :3 0 12f4 20d5
20dd :2 0 20df 12f8
20e0 2071 20df 0
20e1 12fc 0 20e2
12fe 20e3 2054 20e2
0 20e4 1302 0
20ec 41 :3 0 7e
:3 0 47 :3 0 2f
:2 0 1304 20e6 20e9
20e5 20ea 0 20ec
1307 2110 45 :3 0
1e2 :3 0 7e :3 0
1e3 :3 0 47 :3 0
1312 20f0 20f2 2f
:2 0 1314 20ef 20f5
1e4 :2 0 7e :3 0
1e3 :3 0 41 :3 0
1317 20f9 20fb 2f
:2 0 1319 20f8 20fe
131c 20f7 2100 :3 0
2101 :2 0 1e5 :2 0
1e6 :2 0 131f 2103
2105 :3 0 2106 :2 0
86 :2 0 1e7 :2 0
1322 2108 210a :3 0
1325 20ee 210c 20ed
210d 0 210f 1327
2111 1e06 20ec 0
2112 0 210f 0
2112 1329 0 21a2
60 :3 0 1e8 :3 0
1f :3 0 3e :3 0
3f :3 0 1c :3 0
33 :3 0 2b :3 0
41 :3 0 47 :3 0
54 :3 0 46 :3 0
45 :3 0 61 :3 0
64 :3 0 63 :3 0
65 :3 0 66 :3 0
68 :3 0 67 :3 0
69 :3 0 30 :3 0
2e :3 0 49 :3 0
40 :3 0 7e :3 0
4c :3 0 2f :2 0
132c 212c 212f 132f
2114 2131 2113 2132
0 21a2 71 :3 0
72 :3 0 2134 2135
0 71 :3 0 84
:3 0 2137 2138 0
13 :2 0 2 :4 0
1e9 :2 0 86 :2 0
2c :2 0 1348 213d
213f :3 0 134b 2139
2141 2136 2143 2142
:2 0 74 :3 0 87
:3 0 2145 2146 0
76 :4 0 1ea :4 0
79 :2 0 7a :3 0
134f 214a 214c :3 0
79 :2 0 9d :4 0
1352 214e 2150 :3 0
79 :2 0 dd :4 0
1355 2152 2154 :3 0
79 :2 0 7e :3 0
82 :3 0 60 :3 0
1358 2158 215a 81
:4 0 135a 2157 215d
135d 2156 215f :3 0
79 :2 0 e0 :4 0
1360 2161 2163 :3 0
79 :2 0 7c :4 0
1363 2165 2167 :3 0
79 :2 0 1c :3 0
1366 2169 216b :3 0
79 :2 0 7c :4 0
1369 216d 216f :3 0
136c 2147 2171 :2 0
2173 136f 2174 2144
2173 0 2175 1371
0 21a2 71 :3 0
72 :3 0 2176 2177
0 71 :3 0 84
:3 0 2179 217a 0
2c :2 0 2 :4 0
1eb :2 0 86 :2 0
2c :2 0 1373 217f
2181 :3 0 1376 217b
2183 2178 2185 2184
:2 0 74 :3 0 87
:3 0 2187 2188 0
76 :4 0 1ec :4 0
79 :2 0 7a :3 0
137a 218c 218e :3 0
137d 2189 2190 :2 0
2192 1380 2193 2186
2192 0 2194 1382
0 21a2 e3 :3 0
e4 :3 0 2195 2196
0 7e :3 0 60
:3 0 2f :2 0 1384
2198 219b 1c :3 0
e5 :3 0 1387 2197
219f :2 0 21a2 133
:3 0 138b 2324 7e
:3 0 c :3 0 2f
:2 0 1395 21a3 21a6
7e :3 0 ea :2 0
45 :3 0 2f :2 0
1398 21a8 21ac 139d
21a9 21ae :3 0 7e
:3 0 46 :3 0 2f
:2 0 13a0 21b0 21b3
e9 :2 0 2f :2 0
13a5 21b5 21b7 :3 0
21b8 :2 0 7e :3 0
c :3 0 2f :2 0
13a8 21ba 21bd a3
:2 0 2f :2 0 13ad
21bf 21c1 :3 0 21c2
:2 0 21b9 21c4 21c3
:2 0 71 :3 0 72
:3 0 21c6 21c7 0
71 :3 0 84 :3 0
21c9 21ca 0 13
:2 0 2 :4 0 1ed
:2 0 86 :2 0 2c
:2 0 13b0 21cf 21d1
:3 0 13b3 21cb 21d3
21c8 21d5 21d4 :2 0
74 :3 0 87 :3 0
21d7 21d8 0 76
:4 0 1ee :4 0 79
:2 0 7a :3 0 13b7
21dc 21de :3 0 79
:2 0 9d :4 0 13ba
21e0 21e2 :3 0 79
:2 0 dd :4 0 13bd
21e4 21e6 :3 0 79
:2 0 7e :3 0 82
:3 0 10 :3 0 96
:3 0 21eb 21ec 0
13c0 21ea 21ee 81
:4 0 13c2 21e9 21f1
13c5 21e8 21f3 :3 0
79 :2 0 e0 :4 0
13c8 21f5 21f7 :3 0
79 :2 0 7c :4 0
13cb 21f9 21fb :3 0
79 :2 0 1c :3 0
13ce 21fd 21ff :3 0
79 :2 0 7c :4 0
13d1 2201 2203 :3 0
13d4 21d9 2205 :2 0
2207 13d7 2208 21d6
2207 0 2209 13d9
0 2233 71 :3 0
72 :3 0 220a 220b
0 71 :3 0 84
:3 0 220d 220e 0
2c :2 0 2 :4 0
1ef :2 0 86 :2 0
2c :2 0 13db 2213
2215 :3 0 13de 220f
2217 220c 2219 2218
:2 0 74 :3 0 87
:3 0 221b 221c 0
76 :4 0 1f0 :4 0
79 :2 0 7a :3 0
13e2 2220 2222 :3 0
13e5 221d 2224 :2 0
2226 13e8 2227 221a
2226 0 2228 13ea
0 2233 e3 :3 0
e4 :3 0 2229 222a
0 10 :3 0 96
:3 0 222c 222d 0
1c :3 0 e5 :3 0
13ec 222b 2231 :2 0
2233 13f0 2234 21c5
2233 0 2235 13f4
0 2322 c5 :3 0
c2 :3 0 c :3 0
c4 :3 0 fa :3 0
1a8 :3 0 c4 :3 0
de :3 0 1f1 :3 0
c6 :3 0 1c :4 0
1f2 1 :8 0 2257
111 :4 0 2242 :3 0
e5 :3 0 a3 :2 0
13f8 2245 2246 :3 0
10 :3 0 96 :3 0
2248 2249 0 2f
:2 0 224a 224b 0
224d 13fb 2254 10
:3 0 96 :3 0 224e
224f 0 112 :2 0
2250 2251 0 2253
13fd 2255 2247 224d
0 2256 0 2253
0 2256 13ff 0
2257 1402 22aa 98
:3 0 10 :3 0 96
:3 0 2259 225a 0
97 :3 0 225b 225c
0 225e 1405 2260
1407 225f 225e :2 0
22a8 99 :3 0 10
:3 0 96 :3 0 2263
2264 0 97 :3 0
2265 2266 0 22a0
71 :3 0 72 :3 0
2268 2269 0 71
:3 0 84 :3 0 226b
226c 0 9a :2 0
2 :4 0 1f3 :2 0
86 :2 0 2c :2 0
1409 2271 2273 :3 0
140c 226d 2275 226a
2277 2276 :2 0 74
:3 0 87 :3 0 2279
227a 0 76 :4 0
1f4 :4 0 79 :2 0
7a :3 0 1410 227e
2280 :3 0 79 :2 0
9d :4 0 1413 2282
2284 :3 0 79 :2 0
9e :4 0 1416 2286
2288 :3 0 79 :2 0
97 :3 0 1419 228a
228c :3 0 79 :2 0
9f :4 0 141c 228e
2290 :3 0 79 :2 0
a0 :3 0 a1 :3 0
2c :2 0 a2 :2 0
141f 2293 2297 1423
2292 2299 :3 0 1426
227b 229b :2 0 229d
1429 229e 2278 229d
0 229f 142b 0
22a0 142d 22a3 :3 0
22a3 0 22a3 22a2
22a0 22a1 :6 0 22a5
49 :3 0 1430 22a7
1432 22a6 22a5 :2 0
22a8 1434 :2 0 22aa
0 22aa 22a9 2257
22a8 :6 0 2322 1
:3 0 d7 :3 0 c4
:3 0 fa :3 0 1a8
:3 0 c4 :3 0 de
:3 0 1f1 :3 0 92
:3 0 1f :3 0 8d
:3 0 3e :4 0 1f5
1 :8 0 22cd 111
:4 0 22b8 :3 0 e5
:3 0 a3 :2 0 1439
22bb 22bc :3 0 10
:3 0 96 :3 0 22be
22bf 0 2f :2 0
22c0 22c1 0 22c3
143c 22ca 10 :3 0
96 :3 0 22c4 22c5
0 112 :2 0 22c6
22c7 0 22c9 143e
22cb 22bd 22c3 0
22cc 0 22c9 0
22cc 1440 0 22cd
1443 2320 98 :3 0
10 :3 0 96 :3 0
22cf 22d0 0 97
:3 0 22d1 22d2 0
22d4 1446 22d6 1448
22d5 22d4 :2 0 231e
99 :3 0 10 :3 0
96 :3 0 22d9 22da
0 97 :3 0 22db
22dc 0 2316 71
:3 0 72 :3 0 22de
22df 0 71 :3 0
84 :3 0 22e1 22e2
0 9a :2 0 2
:4 0 1f6 :2 0 86
:2 0 2c :2 0 144a
22e7 22e9 :3 0 144d
22e3 22eb 22e0 22ed
22ec :2 0 74 :3 0
87 :3 0 22ef 22f0
0 76 :4 0 1f7
:4 0 79 :2 0 7a
:3 0 1451 22f4 22f6
:3 0 79 :2 0 9d
:4 0 1454 22f8 22fa
:3 0 79 :2 0 9e
:4 0 1457 22fc 22fe
:3 0 79 :2 0 97
:3 0 145a 2300 2302
:3 0 79 :2 0 9f
:4 0 145d 2304 2306
:3 0 79 :2 0 a0
:3 0 a1 :3 0 2c
:2 0 a2 :2 0 1460
2309 230d 1464 2308
230f :3 0 1467 22f1
2311 :2 0 2313 146a
2314 22ee 2313 0
2315 146c 0 2316
146e 2319 :3 0 2319
0 2319 2318 2316
2317 :6 0 231b 4b
:3 0 1471 231d 1473
231c 231b :2 0 231e
1475 :2 0 2320 0
2320 231f 22cd 231e
:6 0 2322 1 :3 0
1478 2323 21af 2322
0 2325 1d0f 21a2
0 2325 147c 0
2399 60 :3 0 11c
:3 0 1f :3 0 147f
2327 2329 2326 232a
0 2399 71 :3 0
72 :3 0 232c 232d
0 71 :3 0 84
:3 0 232f 2330 0
13 :2 0 2 :4 0
1f8 :2 0 86 :2 0
2c :2 0 1481 2335
2337 :3 0 1484 2331
2339 232e 233b 233a
:2 0 74 :3 0 87
:3 0 233d 233e 0
76 :4 0 1f9 :4 0
79 :2 0 7a :3 0
1488 2342 2344 :3 0
79 :2 0 9d :4 0
148b 2346 2348 :3 0
79 :2 0 dd :4 0
148e 234a 234c :3 0
79 :2 0 7e :3 0
82 :3 0 60 :3 0
1491 2350 2352 81
:4 0 1493 234f 2355
1496 234e 2357 :3 0
79 :2 0 e0 :4 0
1499 2359 235b :3 0
79 :2 0 7c :4 0
149c 235d 235f :3 0
79 :2 0 1c :3 0
149f 2361 2363 :3 0
79 :2 0 7c :4 0
14a2 2365 2367 :3 0
14a5 233f 2369 :2 0
236b 14a8 236c 233c
236b 0 236d 14aa
0 2399 71 :3 0
72 :3 0 236e 236f
0 71 :3 0 84
:3 0 2371 2372 0
2c :2 0 2 :4 0
1fa :2 0 86 :2 0
2c :2 0 14ac 2377
2379 :3 0 14af 2373
237b 2370 237d 237c
:2 0 74 :3 0 87
:3 0 237f 2380 0
76 :4 0 1fb :4 0
79 :2 0 7a :3 0
14b3 2384 2386 :3 0
14b6 2381 2388 :2 0
238a 14b9 238b 237e
238a 0 238c 14bb
0 2399 e3 :3 0
e4 :3 0 238d 238e
0 7e :3 0 60
:3 0 2f :2 0 14bd
2390 2393 1c :3 0
e5 :3 0 14c0 238f
2397 :2 0 2399 14c4
239a feb 2399 0
2516 8a :3 0 1fc
:3 0 8a :3 0 8c
:3 0 8a :3 0 8d
:3 0 8a :3 0 8e
:3 0 8a :3 0 8f
:3 0 8a :3 0 90
:3 0 8a :3 0 91
:3 0 8a :3 0 92
:3 0 8a :3 0 93
:3 0 12 :3 0 37
:3 0 14 :3 0 35
:3 0 38 :3 0 3d
:3 0 15 :3 0 1f
:3 0 3c :3 0 94
:3 0 8a :3 0 8a
:3 0 8b :3 0 1a
:4 0 1fd 1 :8 0
23c1 10 :3 0 96
:3 0 23bc 23bd 0
97 :3 0 23be 23bf
0 23c1 14d3 2414
98 :3 0 10 :3 0
96 :3 0 23c3 23c4
0 97 :3 0 23c5
23c6 0 23c8 14d6
23ca 14d8 23c9 23c8
:2 0 2412 99 :3 0
10 :3 0 96 :3 0
23cd 23ce 0 97
:3 0 23cf 23d0 0
240a 71 :3 0 72
:3 0 23d2 23d3 0
71 :3 0 84 :3 0
23d5 23d6 0 9a
:2 0 2 :4 0 1fe
:2 0 86 :2 0 2c
:2 0 14da 23db 23dd
:3 0 14dd 23d7 23df
23d4 23e1 23e0 :2 0
74 :3 0 87 :3 0
23e3 23e4 0 76
:4 0 1ff :4 0 79
:2 0 7a :3 0 14e1
23e8 23ea :3 0 79
:2 0 9d :4 0 14e4
23ec 23ee :3 0 79
:2 0 9e :4 0 14e7
23f0 23f2 :3 0 79
:2 0 97 :3 0 14ea
23f4 23f6 :3 0 79
:2 0 9f :4 0 14ed
23f8 23fa :3 0 79
:2 0 a0 :3 0 a1
:3 0 2c :2 0 a2
:2 0 14f0 23fd 2401
14f4 23fc 2403 :3 0
14f7 23e5 2405 :2 0
2407 14fa 2408 23e2
2407 0 2409 14fc
0 240a 14fe 240d
:3 0 240d 0 240d
240c 240a 240b :6 0
240f 4d :3 0 1501
2411 1503 2410 240f
:2 0 2412 1505 :2 0
2414 0 2414 2413
23c1 2412 :6 0 2513
1 :3 0 10 :3 0
96 :3 0 2416 2417
0 10 :3 0 a3
:2 0 11b :3 0 2419
241b 0 150a 241a
241d :3 0 241e :2 0
200 :2 0 150d 2420
2421 :3 0 71 :3 0
72 :3 0 2423 2424
0 71 :3 0 84
:3 0 2426 2427 0
13 :2 0 2 :4 0
201 :2 0 86 :2 0
2c :2 0 150f 242c
242e :3 0 1512 2428
2430 2425 2432 2431
:2 0 74 :3 0 87
:3 0 2434 2435 0
76 :4 0 202 :4 0
79 :2 0 7a :3 0
1516 2439 243b :3 0
79 :2 0 9d :4 0
1519 243d 243f :3 0
79 :2 0 dd :4 0
151c 2441 2443 :3 0
79 :2 0 7e :3 0
82 :3 0 de :3 0
df :3 0 2448 2449
0 151f 2447 244b
81 :4 0 1521 2446
244e 1524 2445 2450
:3 0 79 :2 0 e0
:4 0 1527 2452 2454
:3 0 79 :2 0 7c
:4 0 152a 2456 2458
:3 0 79 :2 0 1a
:3 0 152d 245a 245c
:3 0 79 :2 0 7c
:4 0 1530 245e 2460
:3 0 1533 2436 2462
:2 0 2464 1536 2465
2433 2464 0 2466
1538 0 2490 71
:3 0 72 :3 0 2467
2468 0 71 :3 0
84 :3 0 246a 246b
0 2c :2 0 2
:4 0 203 :2 0 86
:2 0 2c :2 0 153a
2470 2472 :3 0 153d
246c 2474 2469 2476
2475 :2 0 74 :3 0
87 :3 0 2478 2479
0 76 :4 0 204
:4 0 79 :2 0 7a
:3 0 1541 247d 247f
:3 0 1544 247a 2481
:2 0 2483 1547 2484
2477 2483 0 2485
1549 0 2490 e3
:3 0 e4 :3 0 2486
2487 0 de :3 0
df :3 0 2489 248a
0 1a :3 0 e5
:3 0 154b 2488 248e
:2 0 2490 154f 2491
2422 2490 0 2492
1553 0 2513 e6
:3 0 166 :3 0 2493
2494 0 1b :3 0
12 :3 0 1555 2495
2498 a3 :2 0 2f
:2 0 155a 249a 249c
:3 0 10 :3 0 96
:3 0 249e 249f 0
2f :2 0 24a0 24a1
0 2510 71 :3 0
72 :3 0 24a3 24a4
0 71 :3 0 84
:3 0 24a6 24a7 0
13 :2 0 2 :4 0
205 :2 0 86 :2 0
2c :2 0 155d 24ac
24ae :3 0 1560 24a8
24b0 24a5 24b2 24b1
:2 0 74 :3 0 87
:3 0 24b4 24b5 0
76 :4 0 206 :4 0
79 :2 0 7a :3 0
1564 24b9 24bb :3 0
79 :2 0 9d :4 0
1567 24bd 24bf :3 0
79 :2 0 dd :4 0
156a 24c1 24c3 :3 0
79 :2 0 7e :3 0
82 :3 0 10 :3 0
96 :3 0 24c8 24c9
0 156d 24c7 24cb
81 :4 0 156f 24c6
24ce 1572 24c5 24d0
:3 0 79 :2 0 e0
:4 0 1575 24d2 24d4
:3 0 79 :2 0 7c
:4 0 1578 24d6 24d8
:3 0 79 :2 0 1a
:3 0 157b 24da 24dc
:3 0 79 :2 0 7c
:4 0 157e 24de 24e0
:3 0 1581 24b6 24e2
:2 0 24e4 1584 24e5
24b3 24e4 0 24e6
1586 0 2510 71
:3 0 72 :3 0 24e7
24e8 0 71 :3 0
84 :3 0 24ea 24eb
0 2c :2 0 2
:4 0 207 :2 0 86
:2 0 2c :2 0 1588
24f0 24f2 :3 0 158b
24ec 24f4 24e9 24f6
24f5 :2 0 74 :3 0
87 :3 0 24f8 24f9
0 76 :4 0 208
:4 0 79 :2 0 7a
:3 0 158f 24fd 24ff
:3 0 1592 24fa 2501
:2 0 2503 1595 2504
24f7 2503 0 2505
1597 0 2510 e3
:3 0 e4 :3 0 2506
2507 0 10 :3 0
96 :3 0 2509 250a
0 1a :3 0 e5
:3 0 1599 2508 250e
:2 0 2510 159d 2511
249d 2510 0 2512
15a2 0 2513 15a4
2515 469 fdf 0
2516 0 2513 0
2516 15a8 0 2517
15ac 2518 3b2 2517
0 2519 15af 0
2581 1c :3 0 3
:3 0 251a 251b 0
2581 1b :3 0 5
:3 0 251d 251e 0
2581 40 :3 0 e
:3 0 2520 2521 0
2581 3c :3 0 de
:3 0 209 :3 0 2524
2525 0 2523 2526
0 2581 10 :3 0
96 :3 0 2528 2529
0 20a :3 0 3c
:3 0 1c :3 0 1b
:3 0 40 :3 0 15b1
252b 2530 252a 2531
0 2581 71 :3 0
72 :3 0 2533 2534
0 71 :3 0 84
:3 0 2536 2537 0
173 :2 0 2 :4 0
20b :2 0 86 :2 0
2c :2 0 15b6 253c
253e :3 0 15b9 2538
2540 2535 2542 2541
:2 0 74 :3 0 87
:3 0 2544 2545 0
76 :4 0 20c :4 0
79 :2 0 7a :3 0
15bd 2549 254b :3 0
79 :2 0 9d :4 0
15c0 254d 254f :3 0
79 :2 0 dd :4 0
15c3 2551 2553 :3 0
79 :2 0 7e :3 0
82 :3 0 10 :3 0
96 :3 0 2558 2559
0 15c6 2557 255b
81 :4 0 15c8 2556
255e 15cb 2555 2560
:3 0 79 :2 0 e0
:4 0 15ce 2562 2564
:3 0 79 :2 0 7c
:4 0 15d1 2566 2568
:3 0 79 :2 0 1c
:3 0 15d4 256a 256c
:3 0 79 :2 0 7c
:4 0 15d7 256e 2570
:3 0 15da 2546 2572
:2 0 2574 15dd 2575
2543 2574 0 2576
15df 0 2581 e3
:3 0 176 :3 0 2577
2578 0 10 :3 0
96 :3 0 257a 257b
0 1c :3 0 e5
:3 0 15e1 2579 257f
:2 0 2581 15e5 2622
10 :3 0 20d :2 0
4 2582 2583 0
71 :3 0 72 :3 0
2585 2586 0 71
:3 0 84 :3 0 2588
2589 0 9a :2 0
2 :4 0 20e :2 0
86 :2 0 2c :2 0
15f6 258e 2590 :3 0
15f9 258a 2592 2587
2594 2593 :2 0 74
:3 0 87 :3 0 2596
2597 0 76 :4 0
20f :4 0 79 :2 0
7a :3 0 15fd 259b
259d :3 0 1600 2598
259f :2 0 25b8 74
:3 0 87 :3 0 25a1
25a2 0 76 :4 0
210 :4 0 79 :2 0
10 :3 0 211 :3 0
25a7 25a8 0 1603
25a6 25aa :3 0 79
:2 0 e0 :4 0 1606
25ac 25ae :3 0 79
:2 0 10 :3 0 212
:3 0 25b1 25b2 0
1609 25b0 25b4 :3 0
160c 25a3 25b6 :2 0
25b8 160f 25b9 2595
25b8 0 25ba 1612
0 25ca 180 :3 0
70 :3 0 10 :3 0
211 :3 0 213 :3 0
10 :3 0 212 :3 0
214 :3 0 101 :4 0
215 1 :a 0 25c6
:2 0 25ca 25bc 25c4
:2 0 f :3 0 70
:3 0 25c8 :2 0 25ca
1614 25cc 1618 25cb
25ca :2 0 2620 99
:3 0 10 :3 0 96
:3 0 25cf 25d0 0
97 :3 0 25d1 25d2
0 2618 71 :3 0
72 :3 0 25d4 25d5
0 71 :3 0 84
:3 0 25d7 25d8 0
9a :2 0 2 :4 0
216 :2 0 86 :2 0
2c :2 0 161a 25dd
25df :3 0 161d 25d9
25e1 25d6 25e3 25e2
:2 0 74 :3 0 87
:3 0 25e5 25e6 0
76 :4 0 217 :4 0
79 :2 0 7a :3 0
1621 25ea 25ec :3 0
1624 25e7 25ee :2 0
2608 74 :3 0 87
:3 0 25f0 25f1 0
76 :4 0 9e :4 0
79 :2 0 97 :3 0
1627 25f5 25f7 :3 0
79 :2 0 9f :4 0
162a 25f9 25fb :3 0
79 :2 0 a0 :3 0
a1 :3 0 2c :2 0
a2 :2 0 162d 25fe
2602 1631 25fd 2604
:3 0 1634 25f2 2606
:2 0 2608 1637 2609
25e4 2608 0 260a
163a 0 2618 180
:3 0 70 :3 0 213
:3 0 10 :3 0 212
:3 0 214 :3 0 101
:4 0 218 1 :a 0
2614 :2 0 2618 260c
2612 :2 0 f :3 0
70 :3 0 2616 :2 0
2618 163c 261b :3 0
261b 0 261b 261a
2618 2619 :6 0 261d
1 :3 0 1641 261f
1643 261e 261d :2 0
2620 1645 :2 0 2622
1648 2622 2621 2581
2620 :6 0 2623 :2 0
2 2c 2622 2626
:3 0 2625 2623 2627
:8 0 
16a0
4
:3 0 1 3 1
7 1 b 1
f 1 13 1
17 1 1b 1
1f 1 23 9
6 a e 12
16 1a 1e 22
26 1 30 1
2e 1 37 1
35 1 3e 1
3c 1 45 1
43 1 4c 1
4a 1 53 1
51 1 5a 1
58 1 61 1
5f 1 68 1
66 1 6f 1
6d 1 76 1
74 1 7d 1
7b 1 84 1
82 1 89 1
8e 1 93 1
98 1 9d 1
a2 1 a7 1
ac 1 b1 1
b8 1 b6 1
bf 1 bd 1
c4 1 ca 1
d0 1 d6 1
db 1 e0 1
e6 1 ec 1
f2 1 f8 1
fe 1 104 1
10a 1 110 1
116 1 11e 1
11c 1 125 1
123 1 12c 1
12a 1 131 1
137 1 13d 1
143 1 149 1
14f 1 155 1
15b 1 161 1
167 1 16d 1
173 1 179 1
17f 1 185 1
18b 1 191 1
197 1 19d 1
1a3 1 1a9 1
1af 1 1b5 1
1bb 1 1c3 1
1c1 1 1ca 1
1c8 1 1cf 1
1d7 1 1d5 1
1dc 1 1e2 1
1e8 1 1ee 1
1f4 1 1fa 1
200 1 206 1
20c 1 212 1
218 1 220 1
21e 1 225 1
22a 1 22f 1
236 1 23d 1
24a 1 256 2
259 25b 2 25e
260 2 262 264
2 266 268 2
26a 26c 2 26e
270 2 272 274
2 276 278 2
27a 27c 2 282
284 2 286 288
2 28a 28b 2
28d 28e 2 27e
290 2 292 294
2 29a 29c 2
29e 2a0 2 2a2
2a3 2 2a5 2a6
2 296 2a8 2
2aa 2ac 1 2b2
2 2b4 2b5 2
2ae 2b7 2 2b9
2bb 1 2c1 2
2c3 2c4 2 2bd
2c6 2 2c8 2ca
1 2d0 2 2d2
2d3 2 2cc 2d5
2 2d7 2d9 1
2df 2 2e1 2e2
2 2db 2e4 2
2e6 2e8 2 2ea
2ec 2 2ee 2f0
2 2f2 2f4 2
2f6 2f8 4 252
258 25d 2fa 1
2fc 1 2fe 2
308 30a 3 306
307 30c 2 315
317 2 314 319
1 31b 1 31d
2 34e 353 1
35a 1 355 2
36d 36f 3 36b
36c 371 2 37a
37c 2 37e 380
2 382 384 2
386 388 2 38a
38c 3 391 392
393 2 38e 395
2 379 397 1
399 1 39b 2
364 39c 1 3a0
1 35f 2 35d
3a4 1 3af 2
3ab 3af 2 405
40a 1 411 1
40c 2 424 426
3 422 423 428
2 431 433 2
435 437 2 439
43b 2 43d 43f
2 441 443 3
448 449 44a 2
445 44c 2 430
44e 1 450 1
452 2 41b 453
1 457 1 416
2 414 45b 1
466 2 462 466
2 48a 48f 1
496 1 491 2
4a9 4ab 3 4a7
4a8 4ad 2 4b6
4b8 2 4ba 4bc
2 4be 4c0 2
4c2 4c4 2 4c6
4c8 3 4cd 4ce
4cf 2 4ca 4d1
2 4b5 4d3 1
4d5 1 4d7 2
4a0 4d8 1 4dc
1 49b 2 499
4e0 1 4eb 2
4e7 4eb 2 4f7
4f9 3 4f5 4f6
4fb 2 504 506
2 508 50a 2
50c 50e 1 516
2 518 519 2
510 51b 2 51d
51f 2 521 523
2 525 527 2
529 52b 2 503
52d 1 52f 1
531 2 53b 53d
3 539 53a 53f
2 548 54a 2
547 54c 1 54e
1 550 3 557
558 559 3 532
551 55b 1 55d
3 568 569 56a
1 56e 2 56c
56e 3 575 576
577 1 57b 2
579 57b 2 589
58b 3 587 588
58d 2 596 598
2 59a 59c 2
59e 5a0 1 5a8
2 5aa 5ab 2
5a2 5ad 2 5af
5b1 2 5b3 5b5
2 5b7 5b9 2
5bb 5bd 2 595
5bf 1 5c1 1
5c3 2 5cd 5cf
3 5cb 5cc 5d1
2 5da 5dc 2
5d9 5de 1 5e0
1 5e2 3 5e9
5ea 5eb 3 5c4
5e3 5ed 1 5ef
1 5f1 2 5f8
5f9 2 5fe 5ff
2 5fb 601 2
607 608 1 60a
2 604 60a 2
618 61a 3 616
617 61c 2 625
627 2 629 62b
2 62d 62f 1
637 2 639 63a
2 631 63c 2
63e 640 2 642
644 2 646 648
2 64a 64c 2
624 64e 1 650
1 652 2 65c
65e 3 65a 65b
660 2 669 66b
2 668 66d 1
66f 1 671 3
678 679 67a 3
653 672 67c 1
67e 3 683 684
685 1 689 2
687 689 3 690
691 692 1 696
2 694 696 2
69d 69e 1 6a2
2 6a0 6a2 3
6a8 6a9 6aa 1
6ae 2 6ac 6ae
2 6c1 6c6 1
6cd 1 6c8 2
6e0 6e2 3 6de
6df 6e4 2 6ed
6ef 2 6f1 6f3
2 6f5 6f7 2
6f9 6fb 2 6fd
6ff 3 704 705
706 2 701 708
2 6ec 70a 1
70c 1 70e 2
6d7 70f 1 713
1 6d2 2 6d0
717 1 722 2
71e 722 2 72f
734 1 73b 1
736 2 74e 750
3 74c 74d 752
2 75b 75d 2
75f 761 2 763
765 2 767 769
2 76b 76d 3
772 773 774 2
76f 776 2 75a
778 1 77a 1
77c 2 745 77d
1 781 1 740
2 73e 785 1
788 1 78b 3
790 791 792 1
796 2 794 796
1 79b 1 79d
3 71a 78c 79e
2 7b0 7b5 1
7bc 1 7b7 2
7cf 7d1 3 7cd
7ce 7d3 2 7dc
7de 2 7e0 7e2
2 7e4 7e6 2
7e8 7ea 2 7ec
7ee 3 7f3 7f4
7f5 2 7f0 7f7
2 7db 7f9 1
7fb 1 7fd 2
7c6 7fe 1 802
1 7c1 2 7bf
806 1 811 2
80d 811 2 81e
823 1 82a 1
825 2 83d 83f
3 83b 83c 841
2 84a 84c 2
84e 850 2 852
854 2 856 858
2 85a 85c 3
861 862 863 2
85e 865 2 849
867 1 869 1
86b 2 834 86c
1 870 1 82f
2 82d 874 1
877 1 87a 3
809 87b 87e 2
880 881 2 884
885 1 88b 2
887 88b 1 89a
2 899 89a 1
8a2 1 8a8 2
8aa 8ab 2 897
8ac 1 8b3 1
8ae 2 8c6 8c8
3 8c4 8c5 8ca
2 8d3 8d5 2
8d7 8d9 2 8db
8dd 2 8df 8e1
2 8e3 8e5 3
8ea 8eb 8ec 2
8e7 8ee 2 8d2
8f0 1 8f2 1
8f4 2 8bd 8f5
1 8f9 1 8b8
2 8b6 8fd 1
900 1 90f 2
90e 90f 1 917
1 91d 2 91f
920 2 90c 921
1 928 1 923
2 93b 93d 3
939 93a 93f 2
948 94a 2 94c
94e 2 950 952
2 954 956 2
958 95a 3 95f
960 961 2 95c
963 2 947 965
1 967 1 969
2 932 96a 1
96e 1 92d 2
92b 972 1 975
2 978 979 1
981 2 97d 981
1 987 1 98a
1 98c 3 882
97a 98d 2 998
99d 1 9a4 1
99f 2 9b7 9b9
3 9b5 9b6 9bb
2 9c4 9c6 2
9c8 9ca 2 9cc
9ce 2 9d0 9d2
2 9d4 9d6 3
9db 9dc 9dd 2
9d8 9df 2 9c3
9e1 1 9e3 1
9e5 2 9ae 9e6
1 9ea 1 9a9
2 9a7 9ee 2
9f4 9f5 1 9f9
2 9f7 9f9 2
a04 a06 3 a02
a03 a08 2 a11
a13 2 a15 a17
2 a19 a1b 1
a23 2 a25 a26
2 a1d a28 2
a2a a2c 2 a2e
a30 2 a32 a34
2 a36 a38 2
a10 a3a 1 a3c
1 a3e 2 a48
a4a 3 a46 a47
a4c 2 a55 a57
2 a54 a59 1
a5b 1 a5d 3
a64 a65 a66 3
a3f a5e a68 1
a6a 3 a6f a70
a71 1 a75 2
a73 a75 2 a88
a8d 1 a94 1
a8f 2 aa7 aa9
3 aa5 aa6 aab
2 ab4 ab6 2
ab8 aba 2 abc
abe 2 ac0 ac2
2 ac4 ac6 3
acb acc acd 2
ac8 acf 2 ab3
ad1 1 ad3 1
ad5 2 a9e ad6
1 ada 1 a99
2 a97 ade 1
ae9 2 ae5 ae9
2 af6 afb 1
b02 1 afd 2
b15 b17 3 b13
b14 b19 2 b22
b24 2 b26 b28
2 b2a b2c 2
b2e b30 2 b32
b34 3 b39 b3a
b3b 2 b36 b3d
2 b21 b3f 1
b41 1 b43 2
b0c b44 1 b48
1 b07 2 b05
b4c 1 b4f 1
b52 2 ae1 b53
2 b65 b6a 1
b71 1 b6c 2
b84 b86 3 b82
b83 b88 2 b91
b93 2 b95 b97
2 b99 b9b 2
b9d b9f 2 ba1
ba3 3 ba8 ba9
baa 2 ba5 bac
2 b90 bae 1
bb0 1 bb2 2
b7b bb3 1 bb7
1 b76 2 b74
bbb 1 bc6 2
bc2 bc6 2 bd3
bd8 1 bdf 1
bda 2 bf2 bf4
3 bf0 bf1 bf6
2 bff c01 2
c03 c05 2 c07
c09 2 c0b c0d
2 c0f c11 3
c16 c17 c18 2
c13 c1a 2 bfe
c1c 1 c1e 1
c20 2 be9 c21
1 c25 1 be4
2 be2 c29 1
c2c 1 c2f 2
bbe c30 2 c32
c33 2 c40 c41
a c37 c38 c39
c3a c3b c3c c3d
c3e c43 c44 2
c49 c4a 1 c4e
2 c4c c4e 1
c53 1 c56 2
c60 c62 3 c5e
c5f c64 2 c6d
c6f 2 c71 c73
2 c75 c77 1
c7d 2 c7f c80
2 c79 c82 2
c84 c86 2 c88
c8a 2 c8c c8e
2 c90 c92 2
c6c c94 1 c96
1 c98 2 ca2
ca4 3 ca0 ca1
ca6 2 caf cb1
2 cae cb3 1
cb5 1 cb7 2
cbd cbe 3 cc0
cc1 cc2 3 c99
cb8 cc4 2 cc6
cc7 5 9f1 a6b
c34 c47 cc8 2
cca ccb 1 ccc
2 cd0 cd1 2
cd6 cd7 1 cd9
2 cd3 cd9 2
cde cdf 1 ce3
2 ce1 ce3 2
ced cee 2 cf3
cf4 1 cf6 2
cf0 cf6 2 d01
d03 3 cff d00
d05 2 d0e d10
2 d12 d14 2
d16 d18 1 d20
2 d22 d23 2
d1a d25 2 d0d
d27 1 d29 1
d2b 2 d35 d37
3 d33 d34 d39
2 d42 d44 2
d41 d46 1 d48
1 d4a 2 d51
d52 3 d2c d4b
d54 2 d58 d59
2 d5e d5f 1
d61 2 d5b d61
1 d66 2 d69
d68 2 d86 d8b
1 d92 1 d8d
2 da5 da7 3
da3 da4 da9 2
db2 db4 2 db6
db8 2 dba dbc
2 dbe dc0 2
dc2 dc4 3 dc9
dca dcb 2 dc6
dcd 2 db1 dcf
1 dd1 1 dd3
2 d9c dd4 1
dd8 1 d97 2
d95 ddc 2 de2
de3 1 de7 2
de5 de7 2 df2
df4 3 df0 df1
df6 2 dff e01
2 e03 e05 2
e07 e09 1 e11
2 e13 e14 2
e0b e16 2 e18
e1a 2 e1c e1e
2 e20 e22 2
e24 e26 2 dfe
e28 1 e2a 1
e2c 2 e36 e38
3 e34 e35 e3a
2 e43 e45 2
e42 e47 1 e49
1 e4b 3 e52
e53 e54 3 e2d
e4c e56 1 e58
2 e76 e7b 1
e82 1 e7d 2
e95 e97 3 e93
e94 e99 2 ea2
ea4 2 ea6 ea8
2 eaa eac 2
eae eb0 2 eb2
eb4 3 eb9 eba
ebb 2 eb6 ebd
2 ea1 ebf 1
ec1 1 ec3 2
e8c ec4 1 ec8
1 e87 2 e85
ecc 2 ed2 ed3
1 ed7 2 ed5
ed7 2 ee2 ee4
3 ee0 ee1 ee6
2 eef ef1 2
ef3 ef5 2 ef7
ef9 1 f01 2
f03 f04 2 efb
f06 2 f08 f0a
2 f0c f0e 2
f10 f12 2 f14
f16 2 eee f18
1 f1a 1 f1c
2 f26 f28 3
f24 f25 f2a 2
f33 f35 2 f32
f37 1 f39 1
f3b 3 f42 f43
f44 3 f1d f3c
f46 1 f48 2
f54 f55 5 f4f
f50 f51 f52 f57
9 ceb d6a d6d
ddf e59 ecf f49
f4c f5a 2 f5d
f5c 2 f60 f61
1 f65 2 f63
f65 1 f6a 1
f6d 1 f6f 2
f79 f7b 3 f77
f78 f7d 2 f86
f88 2 f8a f8c
2 f8e f90 1
f96 2 f98 f99
2 f92 f9b 2
f9d f9f 2 fa1
fa3 2 fa5 fa7
2 fa9 fab 2
f85 fad 1 faf
1 fb1 2 fbb
fbd 3 fb9 fba
fbf 2 fc8 fca
2 fc7 fcc 1
fce 1 fd0 2
fd6 fd7 3 fd9
fda fdb b 4e3
55e 561 564 5f0
67f f5e f70 fb2
fd1 fdd 1 fe0
1 fe5 3 ff5
ff6 ff7 1 ffb
2 ff9 ffb 3
1002 1003 1004 1
1008 2 1006 1008
2 1016 1018 3
1014 1015 101a 2
1023 1025 2 1027
1029 2 102b 102d
1 1035 2 1037
1038 2 102f 103a
2 103c 103e 2
1040 1042 2 1044
1046 2 1048 104a
2 1022 104c 1
104e 1 1050 2
105a 105c 3 1058
1059 105e 2 1067
1069 2 1066 106b
1 106d 1 106f
3 1076 1077 1078
3 1051 1070 107a
1 107c 2 1089
108e 1 1095 1
1090 2 10a8 10aa
3 10a6 10a7 10ac
2 10b5 10b7 2
10b9 10bb 2 10bd
10bf 2 10c1 10c3
2 10c5 10c7 3
10cc 10cd 10ce 2
10c9 10d0 2 10b4
10d2 1 10d4 1
10d6 2 109f 10d7
1 10db 1 109a
2 1098 10df 3
10e5 10e6 10e7 1
10eb 2 10e9 10eb
1 10f0 1 10f4
2 10f6 10f7 3
1102 1103 1104 1
1108 2 1106 1108
3 110f 1110 1111
1 1115 2 1113
1115 2 111c 111d
2 1122 1123 1
1125 2 111f 1125
3 112e 112f 1130
1 1134 2 1132
1134 2 1140 1145
1 114c 1 1147
2 115f 1161 3
115d 115e 1163 2
116c 116e 2 1170
1172 2 1174 1176
2 1178 117a 2
117c 117e 3 1183
1184 1185 2 1180
1187 2 116b 1189
1 118b 1 118d
2 1156 118e 1
1192 1 1151 2
114f 1196 4 1199
119d 11a0 11a3 3
11a9 11aa 11ab 1
11af 2 11ad 11af
2 11bc 11c1 1
11c8 1 11c3 2
11db 11dd 3 11d9
11da 11df 2 11e8
11ea 2 11ec 11ee
2 11f0 11f2 2
11f4 11f6 2 11f8
11fa 3 11ff 1200
1201 2 11fc 1203
2 11e7 1205 1
1207 1 1209 2
11d2 120a 1 120e
1 11cd 2 11cb
1212 2 121b 121c
1 1222 a 1228
1229 122a 122b 122c
122d 122e 122f 1230
1231 2 1236 1237
1 123b 2 1239
123b 1 123f 1
1255 2 1254 1255
1 125d 1 1263
2 1265 1266 2
1252 1267 1 126e
1 1269 2 1281
1283 3 127f 1280
1285 2 128e 1290
2 1292 1294 2
1296 1298 2 129a
129c 2 129e 12a0
3 12a5 12a6 12a7
2 12a2 12a9 2
128d 12ab 1 12ad
1 12af 2 1278
12b0 1 12b4 1
1273 2 1271 12b8
1 12bb 1 12be
3 12c1 12c2 12c3
1 12c7 2 12c5
12c7 1 12cc 1
12d0 2 12d2 12d3
10 12d7 12d8 12d9
12da 12db 12dc 12dd
12de 12df 12e0 12e1
12e2 12e3 12e4 12e5
12e6 2 12fd 1302
1 1309 1 1304
2 131c 131e 3
131a 131b 1320 2
1329 132b 2 132d
132f 2 1331 1333
2 1335 1337 2
1339 133b 3 1340
1341 1342 2 133d
1344 2 1328 1346
1 1348 1 134a
2 1313 134b 1
134f 1 130e 2
130c 1353 a 135d
135e 135f 1360 1361
1362 1363 1364 1365
1366 1 136c 2
137d 137f 3 137b
137c 1381 2 138a
138c 2 138e 1390
2 1392 1394 1
139c 2 139e 139f
2 1396 13a1 2
13a3 13a5 2 13a7
13a9 2 13ab 13ad
2 13af 13b1 2
1389 13b3 1 13b5
1 13b7 3 13be
13bf 13c0 e 1215
121f 1225 1234 12bf
12d4 12e9 1356 135a
1369 136f 1374 13b8
13c2 2 13c7 13c8
2 13cd 13ce 1
13d0 2 13ca 13d0
2 13d5 13d6 2
13db 13dc 2 13d8
13de 2 13e1 13e4
2 13e9 13ea 1
13ee 2 13ec 13ee
2 13fc 1401 1
1408 1 1403 2
141b 141d 3 1419
141a 141f 2 1428
142a 2 142c 142e
2 1430 1432 2
1434 1436 2 1438
143a 3 143f 1440
1441 2 143c 1443
2 1427 1445 1
1447 1 1449 2
1412 144a 1 144e
1 140d 2 140b
1452 3 13f3 13f6
1455 2 145b 145c
2 1461 1462 1
1464 2 145e 1464
2 1469 146a 2
146f 1470 2 146c
1472 2 147d 147e
1 1482 2 1480
1482 2 1486 1488
2 148a 148c 2
148e 1490 2 1492
1494 2 1496 1498
2 149a 149c 2
14a2 14a3 1 14a5
2 149e 14a7 2
14aa 14af 2 14b2
14b4 2 14b6 14b8
2 14ba 14bc 2
14be 14c0 2 14c2
14c4 2 14c6 14c8
2 14ce 14cf 1
14d1 2 14ca 14d3
2 14d6 14db 2
14dd 14de 2 14e2
14e3 1 14e7 2
14e5 14e7 1 14f5
1 14fb 2 14fd
14fe 2 14ec 14ff
1 150c 1 1512
2 1514 1515 2
1503 1516 2 1518
1519 1 1521 2
151d 1521 1 1526
1 1528 2 151a
1529 2 152e 152f
1 1533 2 1531
1533 2 1538 153e
1 1545 1 1540
2 1558 155a 3
1556 1557 155c 2
1565 1567 2 1569
156b 2 156d 156f
2 1571 1573 2
1575 1577 3 157c
157d 157e 2 1579
1580 2 1564 1582
1 1584 1 1586
2 154f 1587 1
158b 1 154a 2
1548 158f 1 1592
2 1597 159d 1
15a4 1 159f 2
15b7 15b9 3 15b5
15b6 15bb 2 15c4
15c6 2 15c8 15ca
2 15cc 15ce 2
15d0 15d2 2 15d4
15d6 3 15db 15dc
15dd 2 15d8 15df
2 15c3 15e1 1
15e3 1 15e5 2
15ae 15e6 1 15ea
1 15a9 2 15a7
15ee 1 15f1 2
15f4 15f5 1 15f7
2 15fd 15fe 1
1602 2 1600 1602
2 160d 160f 3
160b 160c 1611 2
161a 161c 2 161e
1620 2 1622 1624
1 162c 2 162e
162f 2 1626 1631
2 1633 1635 2
1637 1639 2 163b
163d 2 163f 1641
2 1619 1643 1
1645 1 1647 2
1651 1653 3 164f
1650 1655 2 165e
1660 2 165d 1662
1 1664 1 1666
3 166d 166e 166f
3 1648 1667 1671
2 167b 167d 3
1679 167a 167f 2
1688 168a 2 168c
168e 2 1690 1692
1 169a 2 169c
169d 2 1694 169f
2 16a1 16a3 2
16a5 16a7 2 16a9
16ab 2 16ad 16af
2 1687 16b1 1
16b3 1 16b5 2
16bf 16c1 3 16bd
16be 16c3 2 16cc
16ce 2 16cb 16d0
1 16d2 1 16d4
3 16db 16dc 16dd
3 16b6 16d5 16df
2 16e1 16e2 1
16e3 1 16e5 2
16f1 16f6 1 16fd
1 16f8 2 1710
1712 3 170e 170f
1714 2 171d 171f
2 1721 1723 2
1725 1727 2 1729
172b 2 172d 172f
3 1734 1735 1736
2 1731 1738 2
171c 173a 1 173c
1 173e 2 1707
173f 1 1743 1
1702 2 1700 1747
a 1475 1478 147b
14df 152c 15f6 16e6
174a 174e 1751 5
1754 13c5 13e7 1459
1753 1 1755 2
1759 175a 2 175f
1760 1 1762 2
175c 1762 2 1766
1767 2 176c 176d
1 176f 2 1769
176f 2 177a 177c
3 1778 1779 177e
2 1787 1789 2
178b 178d 2 178f
1791 1 1799 2
179b 179c 2 1793
179e 2 17a0 17a2
2 17a4 17a6 2
17a8 17aa 2 17ac
17ae 2 1786 17b0
1 17b2 1 17b4
2 17be 17c0 3
17bc 17bd 17c2 2
17cb 17cd 2 17ca
17cf 1 17d1 1
17d3 3 17da 17db
17dc 3 17b5 17d4
17de 1 17e0 2
17e3 17e4 1 17e8
2 17e6 17e8 1
17ed 2 17f0 17f1
1 17f5 2 17f3
17f5 2 17fa 17fb
1 17ff 2 17fd
17ff 2 180d 180f
3 180b 180c 1811
2 181a 181c 2
181e 1820 2 1822
1824 1 182c 2
182e 182f 2 1826
1831 2 1819 1833
1 1835 1 1837
2 1841 1843 3
183f 1840 1845 2
184e 1850 2 184d
1852 1 1854 1
1856 2 185d 185e
3 1838 1857 1860
1 1862 2 1863
1866 2 1868 1869
1 186b 2 1875
187a 1 1881 1
187c 2 1894 1896
3 1892 1893 1898
2 18a1 18a3 2
18a5 18a7 2 18a9
18ab 2 18ad 18af
2 18b1 18b3 3
18b8 18b9 18ba 2
18b5 18bc 2 18a0
18be 1 18c0 1
18c2 2 188b 18c3
1 18c7 1 1886
2 1884 18cb 1
18d6 2 18d2 18d6
2 18db 18dc 2
18e1 18e2 1 18e4
2 18de 18e4 2
18f2 18f4 3 18f0
18f1 18f6 2 18ff
1901 2 1903 1905
2 1907 1909 1
1911 2 1913 1914
2 190b 1916 2
1918 191a 2 191c
191e 2 1920 1922
2 1924 1926 2
18fe 1928 1 192a
1 192c 2 1936
1938 3 1934 1935
193a 2 1943 1945
2 1942 1947 1
1949 1 194b 3
1952 1953 1954 3
192d 194c 1956 1
1958 2 18ce 1959
1 195b 2 1964
1965 1 1969 2
1967 1969 1 196e
1 1970 1 1987
2 1986 1987 1
198f 1 1995 2
1997 1998 2 1984
1999 1 19a0 1
199b 2 19b3 19b5
3 19b1 19b2 19b7
2 19c0 19c2 2
19c4 19c6 2 19c8
19ca 2 19cc 19ce
2 19d0 19d2 3
19d7 19d8 19d9 2
19d4 19db 2 19bf
19dd 1 19df 1
19e1 2 19aa 19e2
1 19e6 1 19a5
2 19a3 19ea 2
19f0 19f1 1 19f5
2 19f3 19f5 1
19fa 1 19fc 8
17e1 186a 195c 195f
1962 1971 19ed 19fd
2 1a00 19ff 1
1a04 2 1a02 1a04
1 1a09 a 1a0f
1a10 1a11 1a12 1a13
1a14 1a15 1a16 1a17
1a18 2 1a1d 1a1e
1 1a22 2 1a20
1a22 1 1a26 8
1a30 1a31 1a32 1a33
1a34 1a35 1a36 1a37
1 1a3a 1 1a3c
1 1a3e 3 1a45
1a46 1a47 1 1a4a
3 1a4d 1a4e 1a4f
1 1a53 2 1a51
1a53 3 1a58 1a59
1a5a 1 1a5d 3
1a61 1a62 1a63 1
1a66 2 1a68 1a69
1 1a6a 2 1a6c
1a6d 1 1a6f 2
1a75 1a76 1 1a7a
2 1a78 1a7a 2
1a82 1a83 2 1a85
1a88 2 1a97 1a9c
1 1aa3 1 1a9e
2 1ab6 1ab8 3
1ab4 1ab5 1aba 2
1ac3 1ac5 2 1ac7
1ac9 2 1acb 1acd
2 1acf 1ad1 2
1ad3 1ad5 3 1ada
1adb 1adc 2 1ad7
1ade 2 1ac2 1ae0
1 1ae2 1 1ae4
2 1aad 1ae5 1
1ae9 1 1aa8 2
1aa6 1aed 2 1a8c
1af0 2 1afd 1b02
1 1b09 1 1b04
2 1b1c 1b1e 3
1b1a 1b1b 1b20 2
1b29 1b2b 2 1b2d
1b2f 2 1b31 1b33
2 1b35 1b37 2
1b39 1b3b 3 1b40
1b41 1b42 2 1b3d
1b44 2 1b28 1b46
1 1b48 1 1b4a
2 1b13 1b4b 1
1b4f 1 1b0e 2
1b0c 1b53 1 1b56
2 1b59 1b5a 2
1b5e 1b5f 2 1b64
1b65 2 1b61 1b67
2 1b5b 1b6a 1
1b6c 2 1b7a 1b7b
2 1b7f 1b80 8
1b73 1b74 1b75 1b76
1b77 1b78 1b7d 1b82
2 1b87 1b88 1
1b8c 2 1b8a 1b8c
2 1b97 1b99 3
1b95 1b96 1b9b 2
1ba4 1ba6 2 1ba8
1baa 2 1bac 1bae
1 1bb6 2 1bb8
1bb9 2 1bb0 1bbb
2 1bbd 1bbf 2
1bc1 1bc3 2 1bc5
1bc7 2 1bc9 1bcb
2 1ba3 1bcd 1
1bcf 1 1bd1 2
1bdb 1bdd 3 1bd9
1bda 1bdf 2 1be8
1bea 2 1be7 1bec
1 1bee 1 1bf0
3 1bf7 1bf8 1bf9
3 1bd2 1bf1 1bfb
1 1c00 2 1bfe
1c00 1 1c21 2
1c20 1c21 1 1c29
1 1c2f 2 1c31
1c32 2 1c1e 1c33
1 1c3a 1 1c35
2 1c4d 1c4f 3
1c4b 1c4c 1c51 2
1c5a 1c5c 2 1c5e
1c60 2 1c62 1c64
2 1c66 1c68 2
1c6a 1c6c 3 1c71
1c72 1c73 2 1c6e
1c75 2 1c59 1c77
1 1c79 1 1c7b
2 1c44 1c7c 1
1c80 1 1c3f 2
1c3d 1c84 2 1c07
1c87 2 1c8b 1c8a
1 1c8f 2 1c9b
1c9d 3 1c99 1c9a
1c9f 2 1ca8 1caa
2 1cac 1cae 2
1cb0 1cb2 1 1cb8
2 1cba 1cbb 2
1cb4 1cbd 2 1cbf
1cc1 2 1cc3 1cc5
2 1cc7 1cc9 2
1ccb 1ccd 2 1ca7
1ccf 1 1cd1 1
1cd3 2 1cdd 1cdf
3 1cdb 1cdc 1ce1
2 1cea 1cec 2
1ce9 1cee 1 1cf0
1 1cf2 2 1cf8
1cf9 3 1cfb 1cfc
1cfd c 1a0c 1a1b
1a3d 1a6e 1b6d 1b70
1b85 1c8c 1c92 1cd4
1cf3 1cff 1 1d01
2 1d04 1d05 2
1d0a 1d0b 1 1d0d
2 1d07 1d0d 2
1d1c 1d21 1 1d28
1 1d23 2 1d3b
1d3d 3 1d39 1d3a
1d3f 2 1d48 1d4a
2 1d4c 1d4e 2
1d50 1d52 2 1d54
1d56 2 1d58 1d5a
3 1d5f 1d60 1d61
2 1d5c 1d63 2
1d47 1d65 1 1d67
1 1d69 2 1d32
1d6a 1 1d6e 1
1d2d 2 1d2b 1d72
2 1d78 1d79 1
1d7d 2 1d7b 1d7d
2 1d88 1d8a 3
1d86 1d87 1d8c 2
1d95 1d97 2 1d99
1d9b 2 1d9d 1d9f
1 1da7 2 1da9
1daa 2 1da1 1dac
2 1dae 1db0 2
1db2 1db4 2 1db6
1db8 2 1dba 1dbc
2 1d94 1dbe 1
1dc0 1 1dc2 2
1dcc 1dce 3 1dca
1dcb 1dd0 2 1dd9
1ddb 2 1dd8 1ddd
1 1ddf 1 1de1
3 1de8 1de9 1dea
3 1dc3 1de2 1dec
1 1dee 2 1df5
1df6 2 1dfb 1dfc
2 1e01 1e02 1
1e04 2 1dfe 1e04
1 1e09 a 1e0f
1e10 1e11 1e12 1e13
1e14 1e15 1e16 1e17
1e18 1 1e1c 3
1e23 1e24 1e25 1
1e28 3 1e2b 1e2c
1e2d 1 1e31 2
1e2f 1e31 3 1e36
1e37 1e38 1 1e3b
3 1e3f 1e40 1e41
1 1e44 2 1e46
1e47 1 1e48 2
1e4a 1e4b 1 1e50
2 1e56 1e57 1
1e5b 2 1e59 1e5b
2 1e63 1e64 2
1e66 1e69 2 1e78
1e7d 1 1e84 1
1e7f 2 1e97 1e99
3 1e95 1e96 1e9b
2 1ea4 1ea6 2
1ea8 1eaa 2 1eac
1eae 2 1eb0 1eb2
2 1eb4 1eb6 3
1ebb 1ebc 1ebd 2
1eb8 1ebf 2 1ea3
1ec1 1 1ec3 1
1ec5 2 1e8e 1ec6
1 1eca 1 1e89
2 1e87 1ece 2
1e6d 1ed1 2 1ede
1ee3 1 1eea 1
1ee5 2 1efd 1eff
3 1efb 1efc 1f01
2 1f0a 1f0c 2
1f0e 1f10 2 1f12
1f14 2 1f16 1f18
2 1f1a 1f1c 3
1f21 1f22 1f23 2
1f1e 1f25 2 1f09
1f27 1 1f29 1
1f2b 2 1ef4 1f2c
1 1f30 1 1eef
2 1eed 1f34 1
1f37 2 1f3a 1f3b
2 1f3e 1f3f 1
1f43 2 1f41 1f43
2 1f52 1f57 1
1f5e 1 1f59 2
1f71 1f73 3 1f6f
1f70 1f75 2 1f7e
1f80 2 1f82 1f84
2 1f86 1f88 2
1f8a 1f8c 2 1f8e
1f90 3 1f95 1f96
1f97 2 1f92 1f99
2 1f7d 1f9b 1
1f9d 1 1f9f 2
1f68 1fa0 1 1fa4
1 1f63 2 1f61
1fa8 1 1fab 1
1fae 2 1f3c 1faf
1 1fb1 2 1fb5
1fb6 2 1fbb 1fbc
2 1fb8 1fbe 2
1fc3 1fc4 2 1fc0
1fc6 1 1fe3 2
1fe2 1fe3 1 1feb
1 1ff1 2 1ff3
1ff4 2 1fe0 1ff5
1 1ffc 1 1ff7
2 200f 2011 3
200d 200e 2013 2
201c 201e 2 2020
2022 2 2024 2026
2 2028 202a 2
202c 202e 3 2033
2034 2035 2 2030
2037 2 201b 2039
1 203b 1 203d
2 2006 203e 1
2042 1 2001 2
1fff 2046 1 2051
2 204d 2051 2
2062 2063 8 205a
205b 205c 205d 205e
205f 2060 2065 2
206a 206b 1 206f
2 206d 206f 2
207a 207c 3 2078
2079 207e 2 2087
2089 2 208b 208d
2 208f 2091 1
2097 2 2099 209a
2 2093 209c 2
209e 20a0 2 20a2
20a4 2 20a6 20a8
2 20aa 20ac 2
2086 20ae 1 20b0
1 20b2 2 20bc
20be 3 20ba 20bb
20c0 2 20c9 20cb
2 20c8 20cd 1
20cf 1 20d1 2
20d7 20d8 3 20da
20db 20dc 3 20b3
20d2 20de 1 20e0
3 2057 2068 20e1
1 20e3 2 20e7
20e8 a 1e0c 1e1b
1e4c 1e4f 1fb2 1fc9
1fcc 2049 20e4 20eb
1 20f1 2 20f3
20f4 1 20fa 2
20fc 20fd 2 20f6
20ff 2 2102 2104
2 2107 2109 1
210b 1 210e 2
2110 2111 2 212d
212e 18 2115 2116
2117 2118 2119 211a
211b 211c 211d 211e
211f 2120 2121 2122
2123 2124 2125 2126
2127 2128 2129 212a
212b 2130 2 213c
213e 3 213a 213b
2140 2 2149 214b
2 214d 214f 2
2151 2153 1 2159
2 215b 215c 2
2155 215e 2 2160
2162 2 2164 2166
2 2168 216a 2
216c 216e 2 2148
2170 1 2172 1
2174 2 217e 2180
3 217c 217d 2182
2 218b 218d 2
218a 218f 1 2191
1 2193 2 2199
219a 3 219c 219d
219e 9 1d75 1def
1df2 1df9 2112 2133
2175 2194 21a0 2
21a4 21a5 2 21aa
21ab 1 21ad 2
21a7 21ad 2 21b1
21b2 1 21b6 2
21b4 21b6 2 21bb
21bc 1 21c0 2
21be 21c0 2 21ce
21d0 3 21cc 21cd
21d2 2 21db 21dd
2 21df 21e1 2
21e3 21e5 1 21ed
2 21ef 21f0 2
21e7 21f2 2 21f4
21f6 2 21f8 21fa
2 21fc 21fe 2
2200 2202 2 21da
2204 1 2206 1
2208 2 2212 2214
3 2210 2211 2216
2 221f 2221 2
221e 2223 1 2225
1 2227 3 222e
222f 2230 3 2209
2228 2232 1 2234
1 2244 2 2243
2244 1 224c 1
2252 2 2254 2255
2 2241 2256 1
225d 1 2258 2
2270 2272 3 226e
226f 2274 2 227d
227f 2 2281 2283
2 2285 2287 2
2289 228b 2 228d
228f 3 2294 2295
2296 2 2291 2298
2 227c 229a 1
229c 1 229e 2
2267 229f 1 22a3
1 2262 2 2260
22a7 1 22ba 2
22b9 22ba 1 22c2
1 22c8 2 22ca
22cb 2 22b7 22cc
1 22d3 1 22ce
2 22e6 22e8 3
22e4 22e5 22ea 2
22f3 22f5 2 22f7
22f9 2 22fb 22fd
2 22ff 2301 2
2303 2305 3 230a
230b 230c 2 2307
230e 2 22f2 2310
1 2312 1 2314
2 22dd 2315 1
2319 1 22d8 2
22d6 231d 3 2235
22aa 2320 2 2324
2323 1 2328 2
2334 2336 3 2332
2333 2338 2 2341
2343 2 2345 2347
2 2349 234b 1
2351 2 2353 2354
2 234d 2356 2
2358 235a 2 235c
235e 2 2360 2362
2 2364 2366 2
2340 2368 1 236a
1 236c 2 2376
2378 3 2374 2375
237a 2 2383 2385
2 2382 2387 1
2389 1 238b 2
2391 2392 3 2394
2395 2396 e fee
ff1 107d 10e2 10f8
10fb 10fe 1a01 1d02
2325 232b 236d 238c
2398 2 23bb 23c0
1 23c7 1 23c2
2 23da 23dc 3
23d8 23d9 23de 2
23e7 23e9 2 23eb
23ed 2 23ef 23f1
2 23f3 23f5 2
23f7 23f9 3 23fe
23ff 2400 2 23fb
2402 2 23e6 2404
1 2406 1 2408
2 23d1 2409 1
240d 1 23cc 2
23ca 2411 1 241c
2 2418 241c 1
241f 2 242b 242d
3 2429 242a 242f
2 2438 243a 2
243c 243e 2 2440
2442 1 244a 2
244c 244d 2 2444
244f 2 2451 2453
2 2455 2457 2
2459 245b 2 245d
245f 2 2437 2461
1 2463 1 2465
2 246f 2471 3
246d 246e 2473 2
247c 247e 2 247b
2480 1 2482 1
2484 3 248b 248c
248d 3 2466 2485
248f 1 2491 2
2496 2497 1 249b
2 2499 249b 2
24ab 24ad 3 24a9
24aa 24af 2 24b8
24ba 2 24bc 24be
2 24c0 24c2 1
24ca 2 24cc 24cd
2 24c4 24cf 2
24d1 24d3 2 24d5
24d7 2 24d9 24db
2 24dd 24df 2
24b7 24e1 1 24e3
1 24e5 2 24ef
24f1 3 24ed 24ee
24f3 2 24fc 24fe
2 24fb 2500 1
2502 1 2504 3
250b 250c 250d 4
24a2 24e6 2505 250f
1 2511 3 2414
2492 2512 3 2514
239a 2515 2 45e
2516 1 2518 4
252c 252d 252e 252f
2 253b 253d 3
2539 253a 253f 2
2548 254a 2 254c
254e 2 2550 2552
1 255a 2 255c
255d 2 2554 255f
2 2561 2563 2
2565 2567 2 2569
256b 2 256d 256f
2 2547 2571 1
2573 1 2575 3
257c 257d 257e 10
2ff 31e 321 324
327 32a 32d 3a7
2519 251c 251f 2522
2527 2532 2576 2580
2 258d 258f 3
258b 258c 2591 2
259a 259c 2 2599
259e 2 25a5 25a9
2 25ab 25ad 2
25af 25b3 2 25a4
25b5 2 25a0 25b7
1 25b9 3 25ba
25c5 25c9 1 2584
2 25dc 25de 3
25da 25db 25e0 2
25e9 25eb 2 25e8
25ed 2 25f4 25f6
2 25f8 25fa 3
25ff 2600 2601 2
25fc 2603 2 25f3
2605 2 25ef 2607
1 2609 4 25d3
260a 2613 2617 1
261b 1 25ce 2
25cc 261f 57 33
3a 41 48 4f
56 5d 64 6b
72 79 80 87
8c 91 96 9b
a0 a5 aa af
b4 bb c2 c8
ce d4 d9 de
e4 ea f0 f6
fc 102 108 10e
114 11a 121 128
12f 135 13b 141
147 14d 153 159
15f 165 16b 171
177 17d 183 189
18f 195 19b 1a1
1a7 1ad 1b3 1b9
1bf 1c6 1cd 1d3
1da 1e0 1e6 1ec
1f2 1f8 1fe 204
20a 210 216 21c
223 228 22d 234
23b 242 
1
4
0 
2626
0
1
50
4f
61
0 1 2 1 4 1 6 1
8 1 a 1 c 1 e 1
10 1 12 1 14 1 16 1
18 1 1a 1 1c 1 1e 1
20 1 22 1 24 1 26 1
28 1 2a 1 2c 1 1 2f
1 31 1 33 1 35 1 37
1 39 1 3b 1 3d 1 3f
1 41 1 43 1 45 1 47
1 49 1 4b 1 4d 1 0

1a3 1 0
43 1 0
1cf 1 0
ec 1 0
218 1 0
9d 1 0
13d 1 0
1e8 1 0
f 1 0
fe 1 0
191 1 0
22a 1 0
93 1 0
1b5 1 0
d0 1 0
1d5 1 0
155 1 0
1a9 1 0
161 1 0
13 1 0
23 1 0
7 1 0
bd 1 0
98 1 0
1f4 1 0
22f 1 0
1c8 1 0
10a 1 0
21e 1 0
1fa 1 0
3c 1 0
1bb 1 0
104 1 0
d6 1 0
1e2 1 0
167 1 0
143 1 0
17f 1 0
19d 1 0
db 1 0
131 1 0
a2 1 0
66 1 0
149 1 0
f8 1 0
8e 1 0
206 1 0
116 1 0
c4 1 0
58 1 0
2e 1 0
18b 1 0
74 1 0
225 1 0
236 1 0
1af 1 0
ac 1 0
1dc 1 0
e6 1 0
212 1 0
35 1 0
3 1 0
ca 1 0
f2 1 0
11c 1 0
89 1 0
14f 1 0
197 1 0
15b 1 0
179 1 0
b6 1 0
82 1 0
b1 1 0
200 1 0
16d 1 0
110 1 0
123 1 0
7b 1 0
173 1 0
b 1 0
6d 1 0
5f 1 0
17 1 0
e0 1 0
1b 1 0
1ee 1 0
a7 1 0
1f 1 0
51 1 0
185 1 0
12a 1 0
2 0 1
1c1 1 0
4a 1 0
137 1 0
20c 1 0
23d 1 0
0

/
SHOW ERRORS FUNCTION SP_U_PARTICIPANT;
ALTER FUNCTION SP_U_PARTICIPANT COMPILE DEBUG;
/******************************************************************************************************************
update resource assignments SECURITY
assign_id:      RES_ASSIGNMENT_ID or TSK_ASSIGNMENT_ID
security_id:    SECURITY_ID in TMT_GLOBAL_SECURITY
*******************************************************************************************************************/
Prompt == FUNCTION SP_U_CRI_SECURITY ==
CREATE OR REPLACE FUNCTION SP_U_CRI_SECURITY wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
71
2 :e:
1FUNCTION:
1SP_U_CRI_SECURITY:
1ASSIGN_ID:
1VARCHAR2:
1SECURITY_ID001:
1P_PLANNED_START:
1TIMESTAMP:
1P_PLANNED_FINISH:
1P_DELAY:
1PLS_INTEGER:
1P_UNITS:
1P_WORK_PERCENT:
1P_ACTUAL:
1RES_ID001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1A_ELEMENT_ID:
132:
1A_PARTICIPANT_ID:
1255:
1A_PARTICIPANT_NAME:
1A_SECURITY_NAME:
1H_FULL_NAME:
1A_RES_ASSIGNMENT_ID:
1A_SECURITY_ID:
1A_TEMP_ID:
1H_CREATED_BY:
1H_PROJECT_ID:
1A_ASSIGNMENT_COUNT:
10:
1A_ELEMENT_TYPE:
1H_RESPONSIBLE:
1A_PARTICIPANT:
1A_RESOURCE_TYPE:
1A_SECURITY_VALUE:
1H_PARTICIPANT:
1H_UPDATE:
1H_TABLE_TYPE:
1H_ASSIGN_STRING:
18192:
1H_TEMP_RES_TYPE:
11:
1H_TEMP_PARTICIPANT:
1H_TEMP_SECURITY_ID:
1H_WBS_SECURITY_ID:
1H_TEMP_BITFLAG:
1H_TEMP_SC_STATE:
1H_TRANSFED_CRI:
1H_CRI_TABLE:
1H_CRI_ELEMENT_TYPE:
1H_REC_USER:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_U_CRI_SECURITY(:
1':
1, :
1COALESCE:
1NULLIF:
1'':
1NULL:
1TO_CHAR:
1);:
1CHECK_TRACE:
160:
1+:
1FPUT:
1               ENTER:: SP_U_CRI_SECURITY(61) :: cplan.sql(17699) :: :
1:
1PMO_CNST:
1SC_TABLE_CRI:
1SC_STATE:
1FC_U_ASSIGNED_SECURITY:
14:
176:
1       RETURN_STATUS:: SP_U_CRI_SECURITY(77) :: cplan.sql(17715) :: :
1 :
1STATE:: :
1 ID:: :
1PMODEFINES:
1PMO_RETURN_STATUS:
1TRUE:
1RETURN_ERROR_EXCEPTION:
116:
183:
1           EXCEPTION:: SP_U_CRI_SECURITY(84) :: cplan.sql(17722) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1ID:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1OTHERS:
1SQLCODE:
195:
1           EXCEPTION:: SP_U_CRI_SECURITY(96) :: cplan.sql(17734) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
2e4
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
1c 51 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 51 a5 1c 81 b0 a3
:2 a0 6b 1c 81 b0 :2 a0 6b :2 a0
6b 6e a5 b a 10 :2 a0 6b
6e :2 a0 6b 6e a5 b 6e 7e
a0 b4 2e 6e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :2 a0 6e 7e
a0 b4 2e 7e 6e b4 2e 6e
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e :2 a0 6e 7e a0
b4 2e 7e 6e b4 2e 6e a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
:3 a0 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e :3 a0 a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c a0 6e
d :2 a0 d :2 a0 d :2 a0 d :3 a0
6b d :2 a0 6b :5 a0 a5 b d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :4 a0
6b a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 :2 a0 6b :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 :2 a0
6b :2 6e 7e :2 a0 6b b4 2e 7e
6e b4 2e 7e :2 a0 6b b4 2e
a5 57 b7 19 3c :9 a0 12a e9
dd :2 a0 65 b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c :7 a0 12a e9 dd :2 a0
65 b7 a4 b1 11 4f b7 a6
9 a4 b1 11 68 4f 1d 17
b5 
2e4
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 6f 6b 59 77 84
80 56 8c 95 91 7f 9d aa
a6 7c b2 bb b7 a5 c3 a2
c8 cc d0 d4 d7 db f8 e3
e7 ea eb f3 e2 115 103 df
107 108 110 102 132 120 ff 124
125 12d 11f 14f 13d 11c 141 142
14a 13c 16c 15a 139 15e 15f 167
159 189 177 156 17b 17c 184 176
1a6 194 173 198 199 1a1 193 1c3
1b1 190 1b5 1b6 1be 1b0 1e0 1ce
1ad 1d2 1d3 1db 1cd 1fd 1eb 1ca
1ef 1f0 1f8 1ea 219 208 20c 1e7
214 207 235 224 228 204 230 223
251 240 244 220 24c 23f 26d 25c
260 23c 268 25b 289 278 27c 258
284 277 2a5 294 298 274 2a0 293
2c1 2b0 2b4 290 2bc 2af 2dd 2cc
2d0 2ac 2d8 2cb 2f9 2e8 2ec 2c8
2f4 2e7 316 304 2e4 308 309 311
303 333 321 300 325 326 32e 320
34f 33e 342 31d 34a 33d 36c 35a
33a 35e 35f 367 359 389 377 356
37b 37c 384 376 3a5 394 398 373
3a0 393 3c1 3b0 3b4 390 3bc 3af
3dd 3cc 3d0 3ac 3d8 3cb 3f9 3e8
3ec 3c8 3f4 3e7 415 404 408 3e4
410 403 432 420 400 424 425 42d
41f 452 43d 441 41c 445 44d 43c
459 45d 439 461 465 469 46c 471
472 1 474 479 47d 481 484 489
48d 491 494 499 49a 49c 4a1 4a4
4a8 4a9 4ae 4b3 4b6 4bb 4bc 4c1
4c4 4c8 4c9 4ce 4d1 4d6 4d7 4dc
4df 4e4 4e5 4ea 4ed 4f2 4f3 4f8
4fb 4ff 500 505 508 50d 50e 513
516 51b 51c 521 524 528 52c 531
534 538 539 53e 541 546 547 54c
551 552 554 559 55a 55c 55d 562
565 56a 56b 570 573 577 57b 580
583 587 588 58d 590 595 596 59b
5a0 5a1 5a3 5a8 5a9 5ab 5ac 5b1
5b4 5b9 5ba 5bf 5c2 5c6 5ca 5ce
5cf 5d1 5d6 5d7 5d9 5da 5df 5e2
5e7 5e8 5ed 5f0 5f4 5f8 5fc 5fd
5ff 604 605 607 608 60d 610 615
616 61b 61e 622 626 62a 62b 62d
632 633 635 636 63b 63e 643 644
649 64c 650 654 658 659 65b 660
661 663 664 669 66c 671 672 677
67a 67f 680 685 688 68c 68d 692
695 69a 69b 6a0 6a3 6a8 6a9 6ae
6af 6b4 6b6 6ba 6bd 6c1 6c5 6c8
6cc 6d0 6d3 6d6 6db 6de 6e1 6e4
6e5 6ea 6eb 1 6ed 6f2 6f6 6fa
6fd 702 707 70a 70e 70f 714 715
71a 71c 720 723 727 72c 730 734
738 73c 740 744 748 74c 750 754
758 75c 760 763 767 76b 76f 772
776 77a 77e 782 786 787 789 78d
791 795 798 79c 7a0 7a3 7a6 7ab
7ae 7b1 7b4 7b5 7ba 7bb 1 7bd
7c2 7c6 7ca 7cd 7d2 7d7 7da 7de
7df 7e4 7e7 7ec 7ed 7f2 7f5 7fa
7fb 800 803 807 80b 80f 813 816
817 819 81e 81f 821 822 827 82a
82f 830 835 838 83d 83e 843 846
84a 84b 850 853 858 859 85e 85f
864 866 86a 86d 871 875 878 87c
880 883 887 88b 88c 891 893 897
89b 89e 8a2 8a6 8a9 8ad 8b1 8b4
8b7 8bc 8bf 8c2 8c5 8c6 8cb 8cc
1 8ce 8d3 8d7 8db 8de 8e3 8e8
8eb 8ef 8f0 8f5 8f6 8fb 8ff 903
906 90b 910 913 917 91b 91e 91f
924 927 92c 92d 932 935 939 93d
940 941 946 947 94c 94e 952 955
959 95d 961 965 969 96d 971 975
979 985 98a 98e 992 996 99a 99c
99d 9a2 1 9a6 9aa 9ae 9b1 9b5
9b9 9bd 9c1 9c4 9c8 9cc 9cf 9d2
9d7 9da 9dd 9e0 9e1 9e6 9e7 1
9e9 9ee 9f2 9f6 9f9 9fe a03 a06
a0a a0b a10 a11 a16 a1a a1e a21
a26 a2b a2e a32 a33 a38 a3b a40
a41 a46 a49 a4d a51 a54 a57 a58
a5a a5b a60 a61 a66 a68 a6c a6f
a73 a77 a7b a7f a83 a87 a8b a97
a9c aa0 aa4 aa8 aac aae ab2 ab4
ac0 ac2 ac4 ac5 aca ace ad0 adc
ae0 ae2 ae3 aec 
2e4
2
0 1 a 6 10 :3 6 15 :3 6
16 :3 6 17 :3 6 e :3 6 e :3 6
15 :3 6 f :3 6 10 :2 6 1 3
a 13 a :2 1 4 11 1a 19
:2 11 :2 4 15 1e 1d :2 15 :2 4 17
20 1f :2 17 :2 4 14 1d 1c :2 14
:2 4 10 19 18 :2 10 :2 4 18 21
20 :2 18 :2 4 12 1b 1a :2 12 :2 4
e 17 16 :2 e :2 4 11 1a 19
:2 11 :2 4 11 1a 19 :2 11 :2 4 :2 17
26 17 :2 4 :2 13 22 13 :2 4 :2 12
21 12 :2 4 :2 12 21 12 :2 4 :2 14
23 14 :2 4 :2 15 24 15 :2 4 :2 12
21 12 :2 4 :2 d 1c d :2 4 :2 11
20 11 :2 4 14 1d 1c :2 14 :2 4
14 1d 1c :2 14 :2 4 :2 17 26 17
:2 4 17 20 1f :2 17 :2 4 16 1f
1e :2 16 :2 4 :2 13 22 13 :2 4 :2 14
23 14 :2 4 :2 13 22 13 :2 4 :2 10
1f 10 :2 4 :2 17 26 17 :2 4 f
18 17 :2 f :2 4 e 17 :3 e 4
7 :2 11 23 :2 2d 3a :2 23 :3 7 :2 d
19 25 :2 2f 43 :2 25 59 5f 62
:2 59 70 8b 8 :2 70 d 10 :2 70
1a 1d :2 70 b e :2 70 13 16
:2 70 1b 1e :2 70 2d 30 :2 70 b
e :2 70 13 16 1f 26 2b 2e
:2 26 3e 41 :2 26 46 :2 1f 4f :2 16
:2 70 b e :2 70 13 16 1f 26
2b 2e :2 26 3f 42 :2 26 47 :2 1f
50 :2 16 :2 70 b e :2 70 13 16
1f 27 :2 1f 31 :2 16 :2 70 b e
:2 70 13 16 1f 27 :2 1f 31 :2 16
:2 70 b e :2 70 13 16 1f 27
:2 1f 38 :2 16 :2 70 b e :2 70 13
16 1f 27 :2 1f 32 :2 16 :2 70 b
e :2 70 13 16 :2 70 1b 1e :2 70
28 2b :2 70 30 33 :2 70 :2 7 4f
:2 4 7 :2 11 23 :2 2d 39 3c 51
53 54 :2 51 :2 23 :3 7 :2 d 12 1e
62 65 :2 1e :2 7 57 :3 4 14 :2 4
11 :2 4 15 :2 4 12 :2 4 14 :2 1d
:2 4 :2 d 19 30 3e 49 58 :2 19
4 7 :2 11 23 :2 2d 39 3c 51
53 54 :2 51 :2 23 :3 7 :2 d 12 1e
62 65 :2 1e 72 75 :2 1e 79 7c
:2 1e 86 89 92 9a :2 a3 :2 92 ae
:2 89 :2 1e b6 b9 :2 1e c1 c4 :2 1e
c9 cc :2 1e d6 d9 :2 1e :2 7 57
:3 4 :2 f 21 :2 2a 34 3f :2 4 1
9 12 9 a :2 14 26 :2 30 3c
40 55 57 58 :2 55 :2 26 :3 a :2 10
15 21 65 68 :2 21 :3 a :2 10 15
39 5c 5f :2 68 :2 39 76 79 :2 39
81 84 :2 8d :2 39 :2 a 5b :3 7 c
15 1e 2c a 13 1e f e
7 e 7 e 7 29 :2 4 :2 9
7 :2 10 1c :2 7 :2 11 23 :2 2d 39
3d 52 54 55 :2 52 :2 23 :3 7 :2 d
12 1e 62 65 :2 1e :3 7 :2 d 12
36 42 45 :2 36 4d 50 :2 36 5c
5f 66 6f 72 :2 5f :2 36 :2 7 58
:2 4 7 c 1b a 13 1e f
e 7 e 7 e 7 :2 4 :4 10
:2 4 :8 1 
2e4
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 :4 7 :4 8
:4 9 :4 a :4 b 2
:4 c :2 1 :7 e :7 f
:7 10 :7 11 :7 12 :7 13
:7 14 :7 15 :7 16 :7 17
:6 18 :6 19 :6 1a :6 1b
:6 1c :6 1d :6 1e :6 1f
:6 20 :7 21 :7 22 :6 23
:7 24 :7 25 :6 26 :6 27
:6 28 :6 29 :6 2a :7 2b
:7 2d :b 2f :11 30 31
:2 30 :2 31 :2 30 :2 31
:2 30 :2 32 :2 30 :2 32
:2 30 :2 32 :2 30 :2 32
:2 30 :2 33 :2 30 :12 33
:2 30 :2 34 :2 30 :12 34
:2 30 :2 35 :2 30 :9 35
:2 30 :2 36 :2 30 :9 36
:2 30 :2 37 :2 30 :9 37
:2 30 :2 38 :2 30 :9 38
:2 30 :2 39 :2 30 :2 39
:2 30 :2 39 :2 30 :2 39
:2 30 :2 39 :4 30 :3 2f
:11 3c :b 3d :3 3c :3 42
:3 45 :3 46 :3 47 :5 49
:b 4a :11 4c :30 4d :3 4c
:a 4f 2e :3 52 :11 53
:b 54 :17 55 :3 53 :2 57
:3 58 :3 59 5a 58
57 58 :3 5b :3 52
:2 5c :5 5e :11 5f :b 60
:18 61 :3 5f :2 63 64
:3 65 66 64 63
64 :3 67 :2 5d :6 5c
51 :7 1 
aee
4
:3 0 1 :3 0 2
:a 0 2df 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 4
:3 0 5 :7 0 9
8 :3 0 7 :3 0
6 :7 0 d c
:3 0 d 7c 0
b 7 :3 0 8
:7 0 11 10 :3 0
a :3 0 9 :7 0
15 14 :3 0 11
a2 0 f a
:3 0 b :7 0 19
18 :3 0 a :3 0
c :7 0 1d 1c
:3 0 15 :2 0 13
a :3 0 d :7 0
21 20 :3 0 4
:3 0 e :7 0 25
24 :3 0 f :3 0
10 :3 0 11 :2 0
4 29 2a 0
27 2b 0 2df
2 2c :2 0 15
:2 0 21 4 :3 0
13 :2 0 1f 2f
31 :6 0 34 32
0 2dd 0 12
:6 0 15 :2 0 25
4 :3 0 23 36
38 :6 0 3b 39
0 2dd 0 14
:6 0 13 :2 0 29
4 :3 0 27 3d
3f :6 0 42 40
0 2dd 0 16
:6 0 15 :2 0 2d
4 :3 0 2b 44
46 :6 0 49 47
0 2dd 0 17
:6 0 13 :2 0 31
4 :3 0 2f 4b
4d :6 0 50 4e
0 2dd 0 18
:6 0 13 :2 0 35
4 :3 0 33 52
54 :6 0 57 55
0 2dd 0 19
:6 0 13 :2 0 39
4 :3 0 37 59
5b :6 0 5e 5c
0 2dd 0 1a
:6 0 13 :2 0 3d
4 :3 0 3b 60
62 :6 0 65 63
0 2dd 0 1b
:6 0 13 :2 0 41
4 :3 0 3f 67
69 :6 0 6c 6a
0 2dd 0 1c
:6 0 1f :2 0 45
4 :3 0 43 6e
70 :6 0 73 71
0 2dd 0 1d
:6 0 1f :2 0 47
a :3 0 75 :7 0
79 76 77 2dd
0 1e :6 0 1f
:2 0 49 a :3 0
7b :7 0 7f 7c
7d 2dd 0 20
:6 0 1f :2 0 4b
a :3 0 81 :7 0
85 82 83 2dd
0 21 :6 0 1f
:2 0 4d a :3 0
87 :7 0 8b 88
89 2dd 0 22
:6 0 1f :2 0 4f
a :3 0 8d :7 0
91 8e 8f 2dd
0 23 :6 0 1f
:2 0 51 a :3 0
93 :7 0 97 94
95 2dd 0 24
:6 0 1f :2 0 53
a :3 0 99 :7 0
9d 9a 9b 2dd
0 25 :6 0 1f
:2 0 55 a :3 0
9f :7 0 a3 a0
a1 2dd 0 26
:6 0 29 :2 0 57
a :3 0 a5 :7 0
a9 a6 a7 2dd
0 27 :6 0 2b
:2 0 5b 4 :3 0
59 ab ad :6 0
b0 ae 0 2dd
0 28 :6 0 1f
:2 0 5f 4 :3 0
5d b2 b4 :6 0
b7 b5 0 2dd
0 2a :6 0 13
:2 0 61 a :3 0
b9 :7 0 bd ba
bb 2dd 0 2c
:6 0 13 :2 0 65
4 :3 0 63 bf
c1 :6 0 c4 c2
0 2dd 0 2d
:6 0 1f :2 0 69
4 :3 0 67 c6
c8 :6 0 cb c9
0 2dd 0 2e
:6 0 1f :2 0 6b
a :3 0 cd :7 0
d1 ce cf 2dd
0 2f :6 0 1f
:2 0 6d a :3 0
d3 :7 0 d7 d4
d5 2dd 0 30
:6 0 1f :2 0 6f
a :3 0 d9 :7 0
dd da db 2dd
0 31 :6 0 1f
:2 0 71 a :3 0
df :7 0 e3 e0
e1 2dd 0 32
:6 0 13 :2 0 73
a :3 0 e5 :7 0
e9 e6 e7 2dd
0 33 :6 0 f2
f3 0 77 4
:3 0 75 eb ed
:6 0 f0 ee 0
2dd 0 34 :6 0
f8 f9 0 79
10 :3 0 11 :2 0
4 f4 :7 0 f7
f5 0 2dd 0
35 :6 0 36 :3 0
37 :3 0 36 :3 0
38 :3 0 fb fc
0 2 :4 0 7b
fd ff fa 101
100 :2 0 39 :3 0
3a :3 0 103 104
0 3b :4 0 36
:3 0 3c :3 0 107
108 0 2 :4 0
7d 109 10b 3d
:4 0 3e :2 0 3f
:3 0 7f 10e 110
:3 0 40 :4 0 3e
:2 0 41 :4 0 82
113 115 :3 0 3e
:2 0 3 :3 0 85
117 119 :3 0 3e
:2 0 41 :4 0 88
11b 11d :3 0 3e
:2 0 42 :4 0 8b
11f 121 :3 0 3e
:2 0 41 :4 0 8e
123 125 :3 0 3e
:2 0 5 :3 0 91
127 129 :3 0 3e
:2 0 41 :4 0 94
12b 12d :3 0 3e
:2 0 42 :4 0 97
12f 131 :3 0 3e
:2 0 43 :3 0 44
:3 0 41 :4 0 3e
:2 0 6 :3 0 9a
137 139 :3 0 3e
:2 0 41 :4 0 9d
13b 13d :3 0 45
:4 0 a0 135 140
46 :4 0 a3 134
143 a6 133 145
:3 0 3e :2 0 42
:4 0 a9 147 149
:3 0 3e :2 0 43
:3 0 44 :3 0 41
:4 0 3e :2 0 8
:3 0 ac 14f 151
:3 0 3e :2 0 41
:4 0 af 153 155
:3 0 45 :4 0 b2
14d 158 46 :4 0
b5 14c 15b b8
14b 15d :3 0 3e
:2 0 42 :4 0 bb
15f 161 :3 0 3e
:2 0 43 :3 0 47
:3 0 9 :3 0 be
165 167 46 :4 0
c0 164 16a c3
163 16c :3 0 3e
:2 0 42 :4 0 c6
16e 170 :3 0 3e
:2 0 43 :3 0 47
:3 0 b :3 0 c9
174 176 46 :4 0
cb 173 179 ce
172 17b :3 0 3e
:2 0 42 :4 0 d1
17d 17f :3 0 3e
:2 0 43 :3 0 47
:3 0 c :3 0 d4
183 185 46 :4 0
d6 182 188 d9
181 18a :3 0 3e
:2 0 42 :4 0 dc
18c 18e :3 0 3e
:2 0 43 :3 0 47
:3 0 d :3 0 df
192 194 46 :4 0
e1 191 197 e4
190 199 :3 0 3e
:2 0 42 :4 0 e7
19b 19d :3 0 3e
:2 0 41 :4 0 ea
19f 1a1 :3 0 3e
:2 0 e :3 0 ed
1a3 1a5 :3 0 3e
:2 0 41 :4 0 f0
1a7 1a9 :3 0 3e
:2 0 48 :4 0 f3
1ab 1ad :3 0 f6
105 1af :2 0 1b1
fb 1b2 102 1b1
0 1b3 fd 0
23d 36 :3 0 37
:3 0 1b4 1b5 0
36 :3 0 49 :3 0
1b7 1b8 0 2b
:2 0 2 :4 0 4a
:2 0 4b :2 0 2b
:2 0 ff 1bd 1bf
:3 0 102 1b9 1c1
1b6 1c3 1c2 :2 0
39 :3 0 4c :3 0
1c5 1c6 0 3b
:4 0 4d :4 0 3e
:2 0 3f :3 0 106
1ca 1cc :3 0 109
1c7 1ce :2 0 1d0
10c 1d1 1c4 1d0
0 1d2 10e 0
23d 1c :3 0 4e
:4 0 1d3 1d4 0
23d 1b :3 0 3
:3 0 1d6 1d7 0
23d 1a :3 0 5
:3 0 1d9 1da 0
23d 34 :3 0 e
:3 0 1dc 1dd 0
23d 27 :3 0 4f
:3 0 50 :3 0 1e0
1e1 0 1df 1e2
0 23d 10 :3 0
51 :3 0 1e4 1e5
0 52 :3 0 27
:3 0 1b :3 0 1a
:3 0 34 :3 0 110
1e7 1ec 1e6 1ed
0 23d 36 :3 0
37 :3 0 1ef 1f0
0 36 :3 0 49
:3 0 1f2 1f3 0
53 :2 0 2 :4 0
54 :2 0 4b :2 0
2b :2 0 115 1f8
1fa :3 0 118 1f4
1fc 1f1 1fe 1fd
:2 0 39 :3 0 4c
:3 0 200 201 0
3b :4 0 55 :4 0
3e :2 0 3f :3 0
11c 205 207 :3 0
3e :2 0 56 :4 0
11f 209 20b :3 0
3e :2 0 57 :4 0
122 20d 20f :3 0
3e :2 0 43 :3 0
47 :3 0 10 :3 0
51 :3 0 214 215
0 125 213 217
46 :4 0 127 212
21a 12a 211 21c
:3 0 3e :2 0 58
:4 0 12d 21e 220
:3 0 3e :2 0 41
:4 0 130 222 224
:3 0 3e :2 0 1b
:3 0 133 226 228
:3 0 3e :2 0 41
:4 0 136 22a 22c
:3 0 139 202 22e
:2 0 230 13c 231
1ff 230 0 232
13e 0 23d 59
:3 0 5a :3 0 233
234 0 10 :3 0
51 :3 0 236 237
0 1b :3 0 5b
:3 0 140 235 23b
:2 0 23d 144 2de
10 :3 0 5c :2 0
4 23e 23f 0
36 :3 0 37 :3 0
241 242 0 36
:3 0 49 :3 0 244
245 0 5d :2 0
2 :4 0 5e :2 0
4b :2 0 2b :2 0
14f 24a 24c :3 0
152 246 24e 243
250 24f :2 0 39
:3 0 4c :3 0 252
253 0 3b :4 0
5f :4 0 3e :2 0
3f :3 0 156 257
259 :3 0 159 254
25b :2 0 274 39
:3 0 4c :3 0 25d
25e 0 3b :4 0
60 :4 0 3e :2 0
10 :3 0 61 :3 0
263 264 0 15c
262 266 :3 0 3e
:2 0 58 :4 0 15f
268 26a :3 0 3e
:2 0 10 :3 0 62
:3 0 26d 26e 0
162 26c 270 :3 0
165 25f 272 :2 0
274 168 275 251
274 0 276 16b
0 286 63 :3 0
35 :3 0 10 :3 0
61 :3 0 64 :3 0
10 :3 0 62 :3 0
65 :3 0 66 :4 0
67 1 :a 0 282
:2 0 286 278 280
:2 0 f :3 0 35
:3 0 284 :2 0 286
16d 288 171 287
286 :2 0 2dc 68
:3 0 10 :3 0 51
:3 0 28b 28c 0
69 :3 0 28d 28e
0 2d4 36 :3 0
37 :3 0 290 291
0 36 :3 0 49
:3 0 293 294 0
5d :2 0 2 :4 0
6a :2 0 4b :2 0
2b :2 0 173 299
29b :3 0 176 295
29d 292 29f 29e
:2 0 39 :3 0 4c
:3 0 2a1 2a2 0
3b :4 0 6b :4 0
3e :2 0 3f :3 0
17a 2a6 2a8 :3 0
17d 2a3 2aa :2 0
2c4 39 :3 0 4c
:3 0 2ac 2ad 0
3b :4 0 6c :4 0
3e :2 0 69 :3 0
180 2b1 2b3 :3 0
3e :2 0 6d :4 0
183 2b5 2b7 :3 0
3e :2 0 6e :3 0
6f :3 0 2b :2 0
70 :2 0 186 2ba
2be 18a 2b9 2c0
:3 0 18d 2ae 2c2
:2 0 2c4 190 2c5
2a0 2c4 0 2c6
193 0 2d4 63
:3 0 35 :3 0 64
:3 0 10 :3 0 62
:3 0 65 :3 0 66
:4 0 71 1 :a 0
2d0 :2 0 2d4 2c8
2ce :2 0 f :3 0
35 :3 0 2d2 :2 0
2d4 195 2d7 :3 0
2d7 0 2d7 2d6
2d4 2d5 :6 0 2d9
1 :3 0 19a 2db
19c 2da 2d9 :2 0
2dc 19e :2 0 2de
1a1 2de 2dd 23d
2dc :6 0 2df :2 0
2 2c 2de 2e2
:3 0 2e1 2df 2e3
:8 0 
1c1
4
:3 0 1 3 1
7 1 b 1
f 1 13 1
17 1 1b 1
1f 1 23 9
6 a e 12
16 1a 1e 22
26 1 30 1
2e 1 37 1
35 1 3e 1
3c 1 45 1
43 1 4c 1
4a 1 53 1
51 1 5a 1
58 1 61 1
5f 1 68 1
66 1 6f 1
6d 1 74 1
7a 1 80 1
86 1 8c 1
92 1 98 1
9e 1 a4 1
ac 1 aa 1
b3 1 b1 1
b8 1 c0 1
be 1 c7 1
c5 1 cc 1
d2 1 d8 1
de 1 e4 1
ec 1 ea 1
f1 1 fe 1
10a 2 10d 10f
2 112 114 2
116 118 2 11a
11c 2 11e 120
2 122 124 2
126 128 2 12a
12c 2 12e 130
2 136 138 2
13a 13c 2 13e
13f 2 141 142
2 132 144 2
146 148 2 14e
150 2 152 154
2 156 157 2
159 15a 2 14a
15c 2 15e 160
1 166 2 168
169 2 162 16b
2 16d 16f 1
175 2 177 178
2 171 17a 2
17c 17e 1 184
2 186 187 2
180 189 2 18b
18d 1 193 2
195 196 2 18f
198 2 19a 19c
2 19e 1a0 2
1a2 1a4 2 1a6
1a8 2 1aa 1ac
4 106 10c 111
1ae 1 1b0 1
1b2 2 1bc 1be
3 1ba 1bb 1c0
2 1c9 1cb 2
1c8 1cd 1 1cf
1 1d1 4 1e8
1e9 1ea 1eb 2
1f7 1f9 3 1f5
1f6 1fb 2 204
206 2 208 20a
2 20c 20e 1
216 2 218 219
2 210 21b 2
21d 21f 2 221
223 2 225 227
2 229 22b 2
203 22d 1 22f
1 231 3 238
239 23a a 1b3
1d2 1d5 1d8 1db
1de 1e3 1ee 232
23c 2 249 24b
3 247 248 24d
2 256 258 2
255 25a 2 261
265 2 267 269
2 26b 26f 2
260 271 2 25c
273 1 275 3
276 281 285 1
240 2 298 29a
3 296 297 29c
2 2a5 2a7 2
2a4 2a9 2 2b0
2b2 2 2b4 2b6
3 2bb 2bc 2bd
2 2b8 2bf 2
2af 2c1 2 2ab
2c3 1 2c5 4
28f 2c6 2cf 2d3
1 2d7 1 28a
2 288 2db 1f
33 3a 41 48
4f 56 5d 64
6b 72 78 7e
84 8a 90 96
9c a2 a8 af
b6 bc c3 ca
d0 d6 dc e2
e8 ef f6 
1
4
0 
2e2
0
1
14
2
29
0 1 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 
3c 1 0
b8 1 0
80 1 0
f 1 0
92 1 0
be 1 0
13 1 0
23 1 0
7 1 0
b1 1 0
9e 1 0
35 1 0
98 1 0
d2 1 0
c5 1 0
e4 1 0
5f 1 0
8c 1 0
d8 1 0
51 1 0
66 1 0
cc 1 0
7a 1 0
2e 1 0
3 1 0
86 1 0
de 1 0
2 0 1
a4 1 0
6d 1 0
b 1 0
58 1 0
17 1 0
74 1 0
1b 1 0
1f 1 0
4a 1 0
ea 1 0
aa 1 0
43 1 0
f1 1 0
0

/
SHOW ERRORS FUNCTION SP_U_CRI_SECURITY;
ALTER FUNCTION SP_U_CRI_SECURITY COMPILE DEBUG;
/****************************************************************************************
 id = TSK_ASSIGN_ID
 bitflag:
        1 = Copy Schedule to Baseline (includes budgets)
        2 = Copy Planned to Schedule
        3 = Copy Proposed to Schedule
        4 = Copy Forecast to Schedule (includes budgets)
        5 = Copy Forecast  Scheduled to Planned
        6 = Move Planned to Schedule
        7 = Append Planned to Schedule
 bussiness rules:
    1. Do not copy date for completed tasks (MILESTONE_PERCENT = 100)
    2. If the task has ACTUAL do not change ETC, WORK_TODATE, SD, FD
    3. When copy Forecast Date to ..., ETC is Forecast Effort, and clear Forcast dates after copy
    4. If both Forcast dates exists ignor forcast start, consider only forcast finish and ETC
    5. Calculated Forecast Start should not be in the past (is >= NOW)
    6. When copy Forcast to and assignments exists forcast rollup wins, and clear Forcast dates after copy
    7. Actual Finish date does not rollup from Assignments to Task
    8. When copy Planned to Schedule clear Planned Dates, Duration  Effort
****************************************************************************************/
Prompt == FUNCTION SP_COMMIT_ASSIGN ==
CREATE OR REPLACE FUNCTION SP_COMMIT_ASSIGN wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
192
2 :e:
1FUNCTION:
1SP_COMMIT_ASSIGN:
1ID:
1VARCHAR2:
1BITFLAG001:
1PLS_INTEGER:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_BITID:
1RAW:
136:
1PMO_CNST:
1SC_ZERO_BIT:
1H_BITID1:
1H_PROJECT_ID:
132:
1H_DAYOFYEAR:
1DATE:
1H_RES_ID:
1H_TASK_ID:
1H_ID:
1H_TSK_ASSIGNMENT_ID:
1H_PRJ_ASSIGNMENT_ID:
1H_START_DATE:
1TIMESTAMP:
1H_FINISH_DATE:
1H_SCHED_WORK:
10:
1H_BILLABLE:
1H_TOTAL:
1H_DAY_MIN:
1H_BITFLAG:
1H_MIN_PER_DAY:
1H_CHANGE_FLAG:
1H_TYPE_ID:
1H_FROM_PRJ:
1H_TSK_PRJ:
1H_WORK:
1H_TEMP_VAR:
1H_TEMP_SC_STATE:
1H_REG_COST:
1NUMBER:
10.00:
1H_REG_SALE:
1STMT:
17999:
1CURSOR:
1C270:
1TA:
1TSK_ASSIGNMENT_ID:
1RESOURCE_ID:
1TASK_ASSIGNMENTS:
1JOIN:
1TMT_WBS:
1W:
1PROJECT_ID:
1ELEMENT_ID:
1TASK_ID:
1BIT_FLAG:
1RESOURCE_TYPE:
1PROJECT_ASSIGNMENT_ID:
1SELECT TA.TSK_ASSIGNMENT_ID, TA.RESOURCE_ID:n         FROM TASK_ASSIGNMENTS T+
1A JOIN TMT_WBS W:n         ON W.PROJECT_ID = TA.PROJECT_ID AND W.ELEMENT_ID =+
1 TA.TASK_ID:n         WHERE W.PROJECT_ID = h_project_id AND TA.BIT_FLAG = '0'+
1:n         AND TA.RESOURCE_TYPE = '0' AND TA.PROJECT_ASSIGNMENT_ID IS Null:
1CMM1:
1C27:
1DAY_YEAR:
1SCHED_MIN:
1BIT_SCHED:
1TASK_DAY_OF_YEAR:
1SELECT DAY_YEAR, SCHED_MIN, BIT_SCHED FROM TASK_DAY_OF_YEAR:n         WHERE P+
1ROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND BIT_FLAG = '0':
1C54:
1SELECT TASK_ID, TSK_ASSIGNMENT_ID:n         FROM TASK_ASSIGNMENTS:n         W+
1HERE BIT_FLAG = '0' AND PROJECT_ASSIGNMENT_ID = h_id:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_COMMIT_ASSIGN(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
11:
162:
1+:
1FPUT:
1               ENTER:: SP_COMMIT_ASSIGN(63) :: cplan.sql(17834) :: :
1MINUTE_PER_DAY:
1SELECT COALESCE(MINUTE_PER_DAY, 480) INTO h_min_per_day FROM TMT_WBS WHERE EL+
1EMENT_ID = h_id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
181:
1           EXCEPTION:: SP_COMMIT_ASSIGN(82) :: cplan.sql(17853) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1=:
1SC_SQL_NO_DATA:
1MILESTONE_PERCENT:
1SELECT PROJECT_ID, CASE WHEN PROJECT_ID = TASK_ID THEN 1 ELSE 0 END, CASE WHE+
1N PROJECT_ASSIGNMENT_ID IS NuLL THEN 0 ELSE 1 END:n         INTO h_project_id+
1,h_tsk_prj,h_from_prj:n         FROM TASK_ASSIGNMENTS WHERE MILESTONE_PERCENT+
1 < 10000 AND RESOURCE_TYPE='0' AND TSK_ASSIGNMENT_ID = h_id:
1100:
1           EXCEPTION:: SP_COMMIT_ASSIGN(101) :: cplan.sql(17872) :: :
18:
1107:
1      RETURN_INVALID:: SP_COMMIT_ASSIGN(108) :: cplan.sql(17879) :: :
1STATE:: :
1RECORD_NOT_FOUND:
1110:
1                EXIT:: SP_COMMIT_ASSIGN(111) :: cplan.sql(17882) :: :
1PMODEFINES:
1PMO_RETURN_INVALID_SP:
1TRUE:
1ELSIF:
1115:
1        RETURN_ERROR:: SP_COMMIT_ASSIGN(116) :: cplan.sql(17887) :: :
1PROJECT_ASSIGNMENT_EXISTS:
1 ID:: :
1118:
1                EXIT:: SP_COMMIT_ASSIGN(119) :: cplan.sql(17890) :: :
1PMO_RETURN_ERROR_SP:
1SELECT COALESCE(MINUTE_PER_DAY, 480) INTO h_min_per_day FROM TMT_WBS WHERE EL+
1EMENT_ID = h_project_id:
1133:
1           EXCEPTION:: SP_COMMIT_ASSIGN(134) :: cplan.sql(17905) :: :
1SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID, TASK_ID, C_EFFORT FROM TASK_ASSIGNMENT+
1S :
1WHERE TSK_ASSIGNMENT_ID = ':
1' AND MILESTONE_PERCENT < 10000 AND RESOURCE_TYPE='0' AND BIT_FLAG = '0' :
13:
1TYPE_ID:
1SC_PROPOSAL:
1SC_PROJECT:
1SCHED_START:
1PLANNED_START:
1SCHED_FINISH:
1PLANNED_FINISH:
1SCHED_DURATION:
1PLANNED_DURATION:
1SCHED_EFFORT:
1PLANNED_EFFORT:
1FORECAST_START:
1FORECAST_FINISH:
1FORECAST_EFFORT:
1ETC:
1FORECAST_DURATION:
1SC_TASK:
1SC_TASK_TYPE1:
1SC_TASK_TYPE2:
1SC_TASK_TYPE3:
1SC_MILESTONE:
1SC_PERSONAL_TASK:
1SC_ACTIVE_ADM_TASK:
1SC_INACTIVE_ADM_TASK:
1SC_WBS_CHANGE_REQUEST_TASK:
1SC_WBS_ACTION_ITEM_TASK:
1SC_WBS_RISK_TASK:
1SC_WBS_ISSUE_TASK:
1SC_WBS_REQUIREMENT_TASK:
1SC_WBS_DEFECT_TASK:
1SC_WBS_SERVICE_REQUEST_TASK:
1ASSIGN_COUNT:
1WORK_TODATE:
1UPDATE TMT_WBS SET:n            TYPE_ID         = CASE WHEN TYPE_ID = PMO_CNS+
1T.SC_PROPOSAL THEN PMO_CNST.SC_PROJECT ELSE TYPE_ID END,:n            SCHED_S+
1TART     = COALESCE(PLANNED_START, SCHED_START),:n            SCHED_FINISH   +
1 = COALESCE(PLANNED_FINISH, SCHED_FINISH),:n            SCHED_DURATION  = CAS+
1E WHEN PLANNED_DURATION = 0 THEN SCHED_DURATION ELSE PLANNED_DURATION END,:n +
1           SCHED_EFFORT    = PLANNED_EFFORT,:n            PLANNED_START   = N+
1ull,:n            PLANNED_FINISH  = Null,:n            PLANNED_DURATION = 0,+
1:n            FORECAST_START  = Null,:n            FORECAST_FINISH = Null,:n +
1           FORECAST_EFFORT = PLANNED_EFFORT,:n            ETC             = P+
1LANNED_EFFORT,:n            FORECAST_DURATION = 0:n            WHERE PROJECT_+
1ID = h_project_id AND TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO+
1_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_+
1PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_C+
1NST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_W+
1BS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_+
1CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) AND ASSIGN_COUN+
1T = 0 AND WORK_TODATE = 0 AND SCHED_EFFORT = 0:
1FOUND:
1175:
1           EXCEPTION:: SP_COMMIT_ASSIGN(176) :: cplan.sql(17947) :: :
1OPEN:
1LOOP:
1!=:
1SC_SQL_SUCCESS:
1EXIT:
1SCHED_WORK:
1PLANNED_WORK:
1BAK_START_DT:
1BAK_FINISH_DT:
1BAK_WORK:
1SCHED_LABOR_COST:
1PLANNED_LABOR_COST:
1SCHED_LABOR_REVENUE:
1PLANNED_LABOR_REVENUE:
1FORECAST_LABOR_COST:
1BILLABLE:
1REGULAR_COST:
1ACTUAL_LABOR_COST:
1FORECAST_LABOR_REVENUE:
1REGULAR_RATE:
1ACTUAL_LABOR_REVENUE:
1UPDATE TASK_ASSIGNMENTS SET:n               BIT_FLAG = '2',:n               S+
1CHED_START = COALESCE(PLANNED_START, SCHED_START),:n               SCHED_FINI+
1SH = COALESCE(PLANNED_FINISH, SCHED_FINISH),:n               SCHED_DURATION =+
1 CASE WHEN PLANNED_DURATION > 0 THEN PLANNED_DURATION ELSE SCHED_DURATION END+
1,:n               SCHED_WORK = PLANNED_WORK,:n               ETC = PLANNED_WO+
1RK,:n               BAK_START_DT = COALESCE(PLANNED_START, SCHED_START),:n   +
1            BAK_FINISH_DT = COALESCE(PLANNED_FINISH, SCHED_FINISH),:n        +
1       BAK_WORK = PLANNED_WORK,:n               SCHED_LABOR_COST = CASE WHEN +
1PLANNED_LABOR_COST > 0.00 THEN PLANNED_LABOR_COST ELSE SCHED_LABOR_COST END,+
1:n               SCHED_LABOR_REVENUE = CASE WHEN PLANNED_LABOR_REVENUE > 0.00+
1 THEN PLANNED_LABOR_REVENUE ELSE SCHED_LABOR_REVENUE END,:n               FOR+
1ECAST_LABOR_COST = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE ((ETC+PLANNED_WORK/6+
10.00) * REGULAR_COST) + ACTUAL_LABOR_COST END,:n               FORECAST_LABOR+
1_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE ((ETC+PLANNED_WORK/60.00) * +
1REGULAR_RATE) + ACTUAL_LABOR_REVENUE END,:n               PLANNED_START = Nul+
1l, PLANNED_FINISH = Null, PLANNED_DURATION = 0, PLANNED_WORK = 0,:n          +
1     FORECAST_START = Null, FORECAST_FINISH = Null, FORECAST_DURATION = 0:n  +
1             WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
1223:
1           EXCEPTION:: SP_COMMIT_ASSIGN(224) :: cplan.sql(17995) :: :
1UPDATE TASK_DAY_OF_YEAR SET BIT_FLAG = '2':n               WHERE PROJECT_ID =+
1 h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
1243:
1           EXCEPTION:: SP_COMMIT_ASSIGN(244) :: cplan.sql(18015) :: :
1CLOSE:
1260:
1           EXCEPTION:: SP_COMMIT_ASSIGN(261) :: cplan.sql(18032) :: :
1ROLLUP_BITS:
1SC_ASSIGNMENT_BIT:
1UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_ASSIGNMENT_BIT:n         WHERE P+
1ROJECT_ID = h_project_id AND ASSIGN_COUNT > 0 AND WORK_TODATE = 0 AND SCHED_E+
1FFORT = 0:
1281:
1           EXCEPTION:: SP_COMMIT_ASSIGN(282) :: cplan.sql(18053) :: :
12:
1SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID, TASK_ID, C_EFFORT FROM TASK_ASSIGNMENT+
1S WHERE PROJECT_ID = TASK_ID AND :
1PROJECT_ID = ':
1BUDGETS:
1B:
1ESTIMATED_BASELINE3:
1ESTIMATED_BASELINE2:
1ESTIMATED_BASELINE1:
1ESTIMATED_BUDGET:
1BUDGET_BASELINE3:
1BUDGET_BASELINE2:
1BUDGET_BASELINE1:
1BUDGET:
1UPDATE BUDGETS B SET:n               B.ESTIMATED_BASELINE3 = B.ESTIMATED_BASE+
1LINE2,:n               B.ESTIMATED_BASELINE2 = B.ESTIMATED_BASELINE1,:n      +
1         B.ESTIMATED_BASELINE1 = B.ESTIMATED_BUDGET,:n               B.BUDGET+
1_BASELINE3 = B.BUDGET_BASELINE2,:n               B.BUDGET_BASELINE2 = B.BUDGE+
1T_BASELINE1,:n               B.BUDGET_BASELINE1 = B.BUDGET:n               WH+
1ERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id:
1341:
1           EXCEPTION:: SP_COMMIT_ASSIGN(342) :: cplan.sql(18113) :: :
1BASELINE3_START:
1BASELINE2_START:
1BASELINE3_FINISH:
1BASELINE2_FINISH:
1BASELINE3_DURATION:
1BASELINE2_DURATION:
1BASELINE3_WORK:
1BASELINE2_WORK:
1BASELINE3_LABOR_COST:
1BASELINE2_LABOR_COST:
1BASELINE3_LABOR_REVENUE:
1BASELINE2_LABOR_REVENUE:
1BASELINE1_START:
1BASELINE1_FINISH:
1BASELINE1_DURATION:
1BASELINE1_WORK:
1BASELINE1_LABOR_COST:
1BASELINE1_LABOR_REVENUE:
1UNRESOLVED_START:
1UNRESOLVED_FINISH:
1UNRESOLVED_DURATION:
1B_REGULAR_COST:
1B_REGULAR_RATE:
1B_SPECIAL_COST:
1SPECIAL_COST:
1B_SPECIAL_RATE:
1SPECIAL_RATE:
1UPDATE TASK_ASSIGNMENTS TA SET:n               TA.BASELINE3_START = TA.BASELI+
1NE2_START, TA.BASELINE3_FINISH = TA.BASELINE2_FINISH, TA.BASELINE3_DURATION =+
1 TA.BASELINE2_DURATION, TA.BASELINE3_WORK = TA.BASELINE2_WORK,:n             +
1  TA.BASELINE3_LABOR_COST = TA.BASELINE2_LABOR_COST, TA.BASELINE3_LABOR_REVEN+
1UE = TA.BASELINE2_LABOR_REVENUE,:n               TA.BASELINE2_START = TA.BASE+
1LINE1_START, TA.BASELINE2_FINISH = TA.BASELINE1_FINISH, TA.BASELINE2_DURATION+
1 = TA.BASELINE1_DURATION, TA.BASELINE2_WORK = TA.BASELINE1_WORK,:n           +
1    TA.BASELINE2_LABOR_COST = TA.BASELINE1_LABOR_COST, TA.BASELINE2_LABOR_REV+
1ENUE = TA.BASELINE1_LABOR_REVENUE,:n               TA.BASELINE1_START = TA.UN+
1RESOLVED_START, TA.BASELINE1_FINISH = TA.UNRESOLVED_FINISH, TA.BASELINE1_DURA+
1TION = TA.UNRESOLVED_DURATION, TA.BASELINE1_WORK = TA.SCHED_WORK + TA.PLANNED+
1_WORK,:n               TA.BASELINE1_LABOR_COST = TA.SCHED_LABOR_COST + TA.PLA+
1NNED_LABOR_COST, TA.BASELINE1_LABOR_REVENUE = TA.SCHED_LABOR_REVENUE + TA.PLA+
1NNED_LABOR_REVENUE,:n               TA.B_REGULAR_COST = TA.REGULAR_COST, TA.B+
1_REGULAR_RATE = TA.REGULAR_RATE, TA.B_SPECIAL_COST = TA.SPECIAL_COST, TA.B_SP+
1ECIAL_RATE = TA.SPECIAL_RATE:n               WHERE TSK_ASSIGNMENT_ID = h_id:
1369:
1           EXCEPTION:: SP_COMMIT_ASSIGN(370) :: cplan.sql(18141) :: :
1PROJECT_ASSIGNMENTS:
1ESTIMATED_BASELINE3_START:
1ESTIMATED_BASELINE2_START:
1ESTIMATED_BASELINE1_START:
1ESTIMATED_SCHED_START:
1ESTIMATED_PLANNED_START:
1ESTIMATED_BASELINE3_FINISH:
1ESTIMATED_BASELINE2_FINISH:
1ESTIMATED_BASELINE1_FINISH:
1ESTIMATED_SCHED_FINISH:
1ESTIMATED_PLANNED_FINISH:
1ESTIMATED_BASELINE3_EFFORT:
1ESTIMATED_BASELINE2_EFFORT:
1ESTIMATED_BASELINE1_EFFORT:
1ESTIMATED_SCHED_EFFORT:
1ESTIMATED_PLANNED_EFFORT:
1ESTIMATED_BASELINE3_COST:
1ESTIMATED_BASELINE2_COST:
1ESTIMATED_BASELINE1_COST:
1ESTIMATED_SCHED_COST:
1ESTIMATED_PLANNED_COST:
1ESTIMATED_BASELINE3_REVENUE:
1ESTIMATED_BASELINE2_REVENUE:
1ESTIMATED_BASELINE1_REVENUE:
1ESTIMATED_SCHED_REVENUE:
1ESTIMATED_PLANNED_REVENUE:
1BASELINE_START:
1BASELINE_FINISH:
1BASELINE_DURATION:
1BASELINE_WORK:
1BASELINE_LABOR_COST:
1BASELINE_LABOR_REVENUE:
1UPDATE PROJECT_ASSIGNMENTS SET:n               ESTIMATED_BASELINE3_START = ES+
1TIMATED_BASELINE2_START,:n               ESTIMATED_BASELINE2_START = ESTIMATE+
1D_BASELINE1_START,:n               ESTIMATED_BASELINE1_START = CASE WHEN ESTI+
1MATED_SCHED_START IS NOT NULL THEN ESTIMATED_SCHED_START ELSE ESTIMATED_PLANN+
1ED_START END,:n               ESTIMATED_BASELINE3_FINISH = ESTIMATED_BASELINE+
12_FINISH,:n               ESTIMATED_BASELINE2_FINISH = ESTIMATED_BASELINE1_FI+
1NISH,:n               ESTIMATED_BASELINE1_FINISH = CASE WHEN ESTIMATED_SCHED_+
1FINISH IS NOT NULL THEN ESTIMATED_SCHED_FINISH ELSE ESTIMATED_PLANNED_FINISH +
1END,:n               ESTIMATED_BASELINE3_EFFORT = ESTIMATED_BASELINE2_EFFORT,+
1:n               ESTIMATED_BASELINE2_EFFORT = ESTIMATED_BASELINE1_EFFORT,:n  +
1             ESTIMATED_BASELINE1_EFFORT = CASE WHEN ESTIMATED_SCHED_EFFORT > +
10 THEN ESTIMATED_SCHED_EFFORT ELSE ESTIMATED_PLANNED_EFFORT END,:n           +
1    ESTIMATED_BASELINE3_COST = ESTIMATED_BASELINE2_COST,:n               ESTI+
1MATED_BASELINE2_COST = ESTIMATED_BASELINE1_COST,:n               ESTIMATED_BA+
1SELINE1_COST = CASE WHEN ESTIMATED_SCHED_COST > 0.00 THEN ESTIMATED_SCHED_COS+
1T ELSE ESTIMATED_PLANNED_COST END,:n               ESTIMATED_BASELINE3_REVENU+
1E = ESTIMATED_BASELINE2_REVENUE,:n               ESTIMATED_BASELINE2_REVENUE +
1= ESTIMATED_BASELINE1_REVENUE,:n               ESTIMATED_BASELINE1_REVENUE = +
1CASE WHEN ESTIMATED_SCHED_REVENUE > 0.00 THEN ESTIMATED_SCHED_REVENUE ELSE ES+
1TIMATED_PLANNED_REVENUE END,:n               BASELINE_START = UNRESOLVED_STAR+
1T,:n               BASELINE_FINISH = UNRESOLVED_FINISH,:n               BASEL+
1INE_DURATION = UNRESOLVED_DURATION,:n               BASELINE_WORK = SCHED_WOR+
1K + PLANNED_WORK,:n               BASELINE_LABOR_COST = SCHED_LABOR_COST + PL+
1ANNED_LABOR_COST,:n               BASELINE_LABOR_REVENUE = SCHED_LABOR_REVENU+
1E + PLANNED_LABOR_REVENUE:n               WHERE PROJECT_ID = h_project_id:
1410:
1           EXCEPTION:: SP_COMMIT_ASSIGN(411) :: cplan.sql(18182) :: :
1SELECT PLANNED_WORK INTO h_day_min FROM TASK_ASSIGNMENTS WHERE BIT_FLAG = '0'+
1 AND TSK_ASSIGNMENT_ID = h_id:
1431:
1           EXCEPTION:: SP_COMMIT_ASSIGN(432) :: cplan.sql(18203) :: :
1IS NULL:
1CASEEND0000:
1C_EFFORT:
1SELECT TSK_ASSIGNMENT_ID, CASE WHEN C_EFFORT = 0 THEN 0 ELSE C_EFFORT + h_wor+
1k END:n               INTO h_prj_assignment_id, h_work:n               FROM T+
1ASK_ASSIGNMENTS WHERE BIT_FLAG = '2' AND TASK_ID = h_project_id AND RESOURCE_+
1ID = h_res_id:
1452:
1           EXCEPTION:: SP_COMMIT_ASSIGN(453) :: cplan.sql(18224) :: :
1IS NOT NULL:
1DELETE FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_id:
1477:
1           EXCEPTION:: SP_COMMIT_ASSIGN(478) :: cplan.sql(18249) :: :
1EFFORT_WEEK:
1DELETE FROM EFFORT_WEEK WHERE TSK_ASSIGNMENT_ID = h_id:
1496:
1           EXCEPTION:: SP_COMMIT_ASSIGN(497) :: cplan.sql(18268) :: :
1RESOURCE_ASSIGN_ROLES:
1RES_ASSIGNMENT_ID:
1DELETE FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID = h_id:
1515:
1           EXCEPTION:: SP_COMMIT_ASSIGN(516) :: cplan.sql(18287) :: :
1RESOURCE_ASSIGN_SKILLS:
1DELETE FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID = h_id:
1534:
1           EXCEPTION:: SP_COMMIT_ASSIGN(535) :: cplan.sql(18306) :: :
1UTL_RAW:
1BIT_OR:
1UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_day_min,:n             +
1        BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID):n                     +
1WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id A+
1ND DAY_YEAR = h_dayofyear:
1583:
1           EXCEPTION:: SP_COMMIT_ASSIGN(584) :: cplan.sql(18355) :: :
1DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNME+
1NT_ID = h_id AND DAY_YEAR = h_dayofyear:
1604:
1           EXCEPTION:: SP_COMMIT_ASSIGN(605) :: cplan.sql(18376) :: :
1UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_prj_assignment_id WHERE PRO+
1JECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_dayofyea+
1r:
1623:
1           EXCEPTION:: SP_COMMIT_ASSIGN(624) :: cplan.sql(18395) :: :
1643:
1           EXCEPTION:: SP_COMMIT_ASSIGN(644) :: cplan.sql(18415) :: :
1659:
1           EXCEPTION:: SP_COMMIT_ASSIGN(660) :: cplan.sql(18431) :: :
1PARENT_ID:
1UPDATE TASK_DAY_OF_YEAR SET BIT_FLAG = '2', PARENT_ID = h_prj_assignment_id:n+
1               WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_id:
1679:
1           EXCEPTION:: SP_COMMIT_ASSIGN(680) :: cplan.sql(18451) :: :
1SD:
1SC_SS:
1FD:
1SC_SF:
1UPDATE TASK_ASSIGNMENTS SET:n                  BIT_FLAG = '2',:n             +
1     SD = PMO_CNST.SC_SS, FD = PMO_CNST.SC_SF,:n                  SCHED_START+
1 = PLANNED_START,:n                  SCHED_FINISH = PLANNED_FINISH,:n        +
1          SCHED_DURATION = PLANNED_DURATION,:n                  SCHED_WORK = +
1PLANNED_WORK,:n                  BAK_START_DT = PLANNED_START,:n             +
1     BAK_FINISH_DT = PLANNED_FINISH,:n                  BAK_WORK = PLANNED_WO+
1RK,:n                  SCHED_LABOR_COST = CASE WHEN PLANNED_LABOR_COST > 0.00+
1 THEN PLANNED_LABOR_COST ELSE SCHED_LABOR_COST END,:n                  SCHED_+
1LABOR_REVENUE = CASE WHEN PLANNED_LABOR_REVENUE > 0.00 THEN PLANNED_LABOR_REV+
1ENUE ELSE SCHED_LABOR_REVENUE END,:n                  ETC = PLANNED_WORK,:n  +
1                FORECAST_LABOR_COST = ((ETC+PLANNED_WORK/60.00) * REGULAR_COS+
1T) + ACTUAL_LABOR_COST,:n                  FORECAST_LABOR_REVENUE = CASE WHEN+
1 BILLABLE = 0 THEN 0.00 ELSE ((ETC+PLANNED_WORK/60.00) * REGULAR_RATE) + ACTU+
1AL_LABOR_REVENUE END,:n                  PLANNED_START = NuLL, PLANNED_FINISH+
1 = NuLL, PLANNED_DURATION = 0, PLANNED_WORK = 0,:n                  FORECAST_+
1START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0,:n               +
1   PROJECT_ASSIGNMENT_ID = NuLL:n                  WHERE TSK_ASSIGNMENT_ID = +
1h_id AND SCHED_WORK = 0 AND PLANNED_WORK > 0:
1717:
1           EXCEPTION:: SP_COMMIT_ASSIGN(718) :: cplan.sql(18489) :: :
1U_ASSIGN_STRING:
1UPDATE TASK_ASSIGNMENTS SET:n                     BIT_FLAG = '2',:n          +
1           SD = PMO_CNST.SC_SS,:n                     FD = PMO_CNST.SC_SF,:n +
1                    SCHED_START = PLANNED_START,:n                     SCHED_+
1FINISH = PLANNED_FINISH,:n                     SCHED_DURATION = PLANNED_DURAT+
1ION,:n                     SCHED_WORK = PLANNED_WORK,:n                     B+
1AK_START_DT = PLANNED_START,:n                     BAK_FINISH_DT = PLANNED_FI+
1NISH,:n                     BAK_WORK = PLANNED_WORK,:n                     SC+
1HED_LABOR_COST = CASE WHEN PLANNED_LABOR_COST > 0.00 THEN PLANNED_LABOR_COST +
1ELSE SCHED_LABOR_COST END,:n                     SCHED_LABOR_REVENUE = CASE W+
1HEN PLANNED_LABOR_REVENUE > 0.00 THEN PLANNED_LABOR_REVENUE ELSE SCHED_LABOR_+
1REVENUE END,:n                     ETC = PLANNED_WORK,:n                     +
1FORECAST_LABOR_COST = ((ETC+PLANNED_WORK/60.00) * REGULAR_COST) + ACTUAL_LABO+
1R_COST,:n                     FORECAST_LABOR_REVENUE = CASE WHEN BILLABLE = 0+
1 THEN 0.00 ELSE ((ETC+PLANNED_WORK/60.00) * REGULAR_RATE) + ACTUAL_LABOR_REVE+
1NUE END,:n                     PLANNED_START = NuLL, PLANNED_FINISH = NuLL, P+
1LANNED_DURATION = 0, PLANNED_WORK = 0,:n                     FORECAST_START =+
1 NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0,:n                     P+
1ROJECT_ASSIGNMENT_ID = h_prj_assignment_id:n                     WHERE TSK_AS+
1SIGNMENT_ID = h_tsk_assignment_id:
1776:
1           EXCEPTION:: SP_COMMIT_ASSIGN(777) :: cplan.sql(18548) :: :
1UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_ASSIGNMENT_BIT WHERE ELEMENT_ID +
1= h_task_id:
1797:
1           EXCEPTION:: SP_COMMIT_ASSIGN(798) :: cplan.sql(18569) :: :
1813:
1           EXCEPTION:: SP_COMMIT_ASSIGN(814) :: cplan.sql(18585) :: :
1>:
1UPDATE TASK_ASSIGNMENTS SET C_EFFORT = h_work WHERE TSK_ASSIGNMENT_ID = h_prj+
1_assignment_id:
1833:
1           EXCEPTION:: SP_COMMIT_ASSIGN(834) :: cplan.sql(18605) :: :
1BEST:
1SELECT PROJECT_ASSIGNMENT_ID, BEST, SCHED_START, SCHED_FINISH, BILLABLE, REGU+
1LAR_COST, REGULAR_RATE:n                  INTO h_prj_assignment_id, h_sched_w+
1ork, h_start_date, h_finish_date, h_billable, h_reg_cost, h_reg_sale:n       +
1           FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID   = h_prj_assignment+
1_id:
1855:
1           EXCEPTION:: SP_COMMIT_ASSIGN(856) :: cplan.sql(18627) :: :
1UPDATE PROJECT_ASSIGNMENTS SET:n                     ESTIMATED_PLANNED_START +
1= NuLL,:n                     ESTIMATED_PLANNED_FINISH = NuLL,:n             +
1        ESTIMATED_PLANNED_EFFORT = 0,:n                     ESTIMATED_PLANNED+
1_COST = 0.00,:n                     ESTIMATED_PLANNED_REVENUE = 0.00,:n      +
1               ESTIMATED_SCHED_START = h_start_date,:n                     ES+
1TIMATED_SCHED_FINISH = h_finish_date,:n                     ESTIMATED_SCHED_E+
1FFORT = h_sched_work,:n                     ESTIMATED_SCHED_COST = (h_sched_w+
1ork / 60.0) * h_reg_cost,:n                     ESTIMATED_SCHED_REVENUE = 0.0+
10:n                     WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_r+
1es_id:
1887:
1           EXCEPTION:: SP_COMMIT_ASSIGN(888) :: cplan.sql(18659) :: :
1UPDATE PROJECT_ASSIGNMENTS SET:n                     ESTIMATED_PLANNED_START +
1= NuLL,:n                     ESTIMATED_PLANNED_FINISH = NuLL,:n             +
1        ESTIMATED_PLANNED_EFFORT = 0,:n                     ESTIMATED_PLANNED+
1_COST = 0.00,:n                     ESTIMATED_PLANNED_REVENUE = 0.00,:n      +
1               ESTIMATED_SCHED_START = h_start_date,:n                     ES+
1TIMATED_SCHED_FINISH = h_finish_date,:n                     ESTIMATED_SCHED_E+
1FFORT = h_sched_work,:n                     ESTIMATED_SCHED_COST = (h_sched_w+
1ork / 60.0) * h_reg_cost,:n                     ESTIMATED_SCHED_REVENUE = (h_+
1sched_work / 60.0) * h_reg_sale:n                     WHERE PROJECT_ID = h_pr+
1oject_id AND RESOURCE_ID = h_res_id:
1917:
1           EXCEPTION:: SP_COMMIT_ASSIGN(918) :: cplan.sql(18689) :: :
1946:
1           EXCEPTION:: SP_COMMIT_ASSIGN(947) :: cplan.sql(18718) :: :
1964:
1           EXCEPTION:: SP_COMMIT_ASSIGN(965) :: cplan.sql(18736) :: :
1FC_ROLLUP:
1SC_NS:
1SC_NF:
1SELECT CASE WHEN (SD <> PMO_CNST.SC_NS) AND (FD <> PMO_CNST.SC_NF) AND (TYPE_+
1ID = PMO_CNST.SC_PROPOSAL) THEN 1 ELSE 0 END:n      INTO h_change_flag:n     +
1 FROM TMT_WBS WHERE ELEMENT_ID = h_project_id:
1984:
1           EXCEPTION:: SP_COMMIT_ASSIGN(985) :: cplan.sql(18756) :: :
1FC_CHANGE_PROJ_TYPE:
1RESOURCE_ASSIGNMENTS:
1ELEMENT_TYPE:
1UPDATE RESOURCE_ASSIGNMENTS SET:n      ELEMENT_TYPE  = (SELECT TYPE_ID FROM T+
1MT_WBS WHERE ELEMENT_ID = h_project_id):n      WHERE ELEMENT_ID = h_project_i+
1d:
11010:
1           EXCEPTION:: SP_COMMIT_ASSIGN(1011) :: cplan.sql(18782) :: :
14:
11020:
1       RETURN_STATUS:: SP_COMMIT_ASSIGN(1021) :: cplan.sql(18792) :: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
11027:
1           EXCEPTION:: SP_COMMIT_ASSIGN(1028) :: cplan.sql(18799) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1ERROR:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
11039:
1           EXCEPTION:: SP_COMMIT_ASSIGN(1040) :: cplan.sql(18811) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
15eb
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d b4 :3 a0 6b 2c 6a
a3 a0 51 a5 1c :2 a0 6b 81
b0 a3 a0 51 a5 1c :2 a0 6b
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 1c 81 b0 a3 a0
1c 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 51
a5 1c 81 b0 a0 f4 b4 bf
c8 :1a a0 12a bd b7 11 a4 b1
a3 :2 a0 6b 1c 81 b0 a0 f4
b4 bf c8 :9 a0 12a bd b7 11
a4 b1 a0 f4 b4 bf c8 :6 a0
12a bd b7 11 a4 b1 a3 :2 a0
6b 1c 81 b0 :2 a0 6b :2 a0 6b
6e a5 b a 10 :2 a0 6b 6e
:2 a0 6b 6e a5 b 6e 7e a0
b4 2e 6e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :3 a0 a5 b 6e
a5 b b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 d :2 a0 d :6 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a :c a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b a0 a5
57 a0 b7 :2 a0 51 a5 b 7e
51 b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a a 10 :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 :2 19
3c :6 a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f a0 6e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e d b7 :2 a0 d :2 a0 51 a5
b 7e 51 b4 2e :44 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 e9 dd
b3 :4 a0 e9 d3 5 :2 a0 f :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c :2 a0 6b a0 7e a0
6b b4 2e 5a a0 2b b7 19
3c :39 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :6 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 a0 47 :2 a0 e9 c1 :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
19 3c :9 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f a0 51 d a0 51 d
a0 6e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e d b7
:2 19 3c :3 a0 11c 11d :6 a0 e9 d3
5 :2 a0 f :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c :2 a0
6b a0 7e a0 6b b4 2e 5a
a0 2b b7 19 3c a0 51 :1e a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :62 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :3a a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
a6 9 51 :6 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b a0
7e a0 6b b4 2e 5a a0 7e
b4 2e :2 5a 52 10 a0 3b b7
19 3c :c a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 6b a0 7e
a0 6b b4 2e 5a a0 7e b4
2e :2 5a a 10 :2 a0 51 a5 b
7e 51 b4 2e a0 51 d b7
:3 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:3 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:3 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:3 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
a0 51 d b7 :2 19 3c b7 :2 a0
d a0 51 d b7 :2 19 3c a0
51 d :2 a0 e9 dd b3 :5 a0 e9
d3 5 :2 a0 f :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
:2 a0 6b a0 7e a0 6b b4 2e
5a a0 2b b7 19 3c :2 a0 51
a5 b 7e 51 b4 2e :f a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
a0 7e a0 6b b4 2e 5a :7 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:9 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c a0 b7 :2 a0 51 a5
b 7e 51 b4 2e :9 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :2 19 3c
b7 a0 47 :2 a0 e9 c1 :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :8 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 51
a5 b 7e 51 b4 2e :37 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 51
a5 b 7e 51 b4 2e a0 51
d :5 a0 :2 51 a5 b d b7 19
3c b7 :2 a0 e9 dd b3 :4 a0 e9
d3 5 :2 a0 f :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
:2 a0 6b a0 7e a0 6b b4 2e
5a a0 2b b7 19 3c :36 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :6 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 a0
47 :2 a0 e9 c1 :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 51 a5 b
7e 51 b4 2e 5a :2 a0 51 a5
b 7e 51 b4 2e 5a a 10
:5 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 19 3c a0 51 d :5 a0 :2 51
a5 b d :11 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 51 a5
b 7e 51 b4 2e :14 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :16 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :2 19
3c b7 :2 19 3c 93 4f b0 46
b7 a6 9 4f b7 9 a4 14
:6 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 a0 47 :2 a0 e9 c1 :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :3 a0 a5
b d :d a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 51 a5 b
7e 51 b4 2e :3 a0 6b d :2 a0
6b :4 a0 a5 b d b7 19 3c
:8 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 6b 7e 51 b4 2e :2 a0 6b
51 d b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 :2 a0 6b :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 :2 a0 6b :2 6e 7e
:2 a0 6b b4 2e 7e 6e b4 2e
7e :2 a0 6b b4 2e a5 57 b7
19 3c :9 a0 12a e9 dd :2 a0 65
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
:7 a0 12a e9 dd :2 a0 65 b7 a4
b1 11 4f b7 a6 9 a4 b1
11 68 4f 1d 17 b5 
15eb
2
0 3 7 23 1f 1e 2b 38
34 1b 40 33 45 49 4d 30
51 55 7d 5d 61 64 65 6d
71 75 78 5c a5 88 59 8c
8d 95 99 9d a0 87 c2 b0
84 b4 b5 bd af de cd d1
d9 ac fa e5 e9 ec ed f5
cc 117 105 c9 109 10a 112 104
134 122 101 126 127 12f 121 151
13f 11e 143 144 14c 13e 16e 15c
13b 160 161 169 15b 18a 179 17d
185 158 1a2 191 195 19d 178 1be
1ad 1b1 175 1b9 1ac 1da 1c9 1cd
1a9 1d5 1c8 1f6 1e5 1e9 1c5 1f1
1e4 212 201 205 1e1 20d 200 22e
21d 221 1fd 229 21c 24a 239 23d
219 245 238 266 255 259 235 261
254 282 271 275 251 27d 270 29e
28d 291 26d 299 28c 2ba 2a9 2ad
289 2b5 2a8 2d6 2c5 2c9 2a5 2d1
2c4 2f2 2e1 2e5 2c1 2ed 2e0 30e
2fd 301 2dd 309 2fc 32a 319 31d
2f9 325 318 346 335 339 315 341
334 363 351 331 355 356 35e 350
36a 36e 34d 37f 382 386 38a 38e
392 396 39a 39e 3a2 3a6 3aa 3ae
3b2 3b6 3ba 3be 3c2 3c6 3ca 3ce
3d2 3d6 3da 3de 3e2 3e6 3ea 3ee
3fa 3ff 401 40d 411 42f 417 41b
41f 422 42a 416 436 43a 413 44b
44e 452 456 45a 45e 462 466 46a
46e 472 476 482 487 489 495 499
49b 49f 4b0 4b1 4b4 4b8 4bc 4c0
4c4 4c8 4cc 4d0 4dc 4e1 4e3 4ef
4f3 511 4f9 4fd 501 504 50c 4f8
518 51c 4f5 520 524 528 52b 530
531 1 533 538 53c 540 543 548
54c 550 553 558 559 55b 560 563
567 568 56d 572 575 57a 57b 580
583 587 588 58d 590 595 596 59b
59e 5a3 5a4 5a9 5ac 5b0 5b4 5b8
5b9 5bb 5c0 5c1 5c3 5c4 5c9 5cc
5d1 5d2 5d7 5d8 5dd 5df 5e3 5e6
5ea 5ee 5f1 5f5 5f9 5fc 5ff 604
607 60a 60d 60e 613 614 1 616
61b 61f 623 626 62b 630 633 637
638 63d 63e 643 645 649 64c 650
654 658 65c 660 664 668 66c 670
674 678 67c 688 68c 690 693 697
69b 69d 6a1 6a5 6a9 6ac 6b0 6b4
6b6 6b7 6bc 1 6c0 6c4 6c8 6cb
6cf 6d3 6d7 6db 6de 6e2 6e6 6e9
6ec 6f1 6f4 6f7 6fa 6fb 700 701
1 703 708 70c 710 713 718 71d
720 724 725 72a 72d 732 733 738
73b 740 741 746 749 74d 74e 753
756 75b 75c 761 764 768 76c 76f
772 773 775 776 77b 77c 781 783
787 78a 78c 790 792 79e 7a0 7a2
7a3 7a8 7ac 7ae 7ba 7bc 7c0 7c4
7c7 7cb 7ce 7d2 7d5 7d6 7db 7de
7e2 7e6 7ea 7ee 7f2 7f6 7fa 7fe
802 806 80a 80e 81a 81e 822 825
829 82d 82f 833 837 83b 83e 842
846 848 849 84e 1 852 856 85a
85d 861 865 869 86d 870 874 878
87b 87e 883 886 889 88c 88d 892
893 1 895 89a 89e 8a2 8a5 8aa
8af 8b2 8b6 8b7 8bc 8bf 8c4 8c5
8ca 8cd 8d2 8d3 8d8 8db 8df 8e0
8e5 8e8 8ed 8ee 8f3 8f6 8fa 8fe
901 904 905 907 908 90d 90e 913
915 919 91c 91e 922 924 930 932
934 935 93a 93e 940 94c 94e 952
956 959 95d 960 964 967 968 96d
970 974 978 97b 97f 983 986 989
98e 991 994 997 998 99d 99e 1
9a0 9a5 9a9 9ad 9b0 9b5 9ba 9bd
9c1 9c2 9c7 9ca 9cf 9d0 9d5 9d8
9dd 9de 9e3 9e6 9ea 9ee 9f2 9f6
9f9 9fa 9fc a01 a02 a04 a05 a0a
a0b a10 a12 a16 a19 a1d a21 a24
a28 a2c a2f a32 a37 a3a a3d a40
a41 a46 a47 1 a49 a4e a52 a56
a59 a5e a63 a66 a6a a6b a70 a71
a76 a78 a7c a7f a83 a87 a8a a8e
a92 a95 a99 a9a a9f aa3 aa5 aa9
aad ab0 ab1 ab3 ab6 ab9 aba abf
ac2 ac6 aca acd ace ad0 ad3 ad6
ad7 adc 1 adf ae4 ae8 aec aef
af3 af7 afa afd b02 b05 b08 b0b
b0c b11 b12 1 b14 b19 b1d b21
b24 b29 b2e b31 b35 b36 b3b b3e
b43 b44 b49 b4c b51 b52 b57 b5a
b5e b62 b66 b6a b6d b6e b70 b75
b76 b78 b79 b7e b81 b86 b87 b8c
b8f b94 b95 b9a b9d ba1 ba2 ba7
baa baf bb0 bb5 bb6 bbb bbd bc1
bc4 bc8 bcc bcf bd3 bd7 bda bdd
be2 be5 be8 beb bec bf1 bf2 1
bf4 bf9 bfd c01 c04 c09 c0e c11
c15 c16 c1b c1c c21 c23 c27 c2a
c2e c32 c35 c39 c3d c40 c44 c48
c49 c4e c50 c54 c58 c5b c5f c63
c67 c6b c6f c73 c7f c83 c87 c8a
c8e c92 c94 c98 c9c ca0 ca3 ca7
cab cad cae cb3 1 cb7 cbb cbf
cc2 cc6 cca cce cd2 cd5 cd9 cdd
ce0 ce3 ce8 ceb cee cf1 cf2 cf7
cf8 1 cfa cff d03 d07 d0a d0f
d14 d17 d1b d1c d21 d24 d29 d2a
d2f d32 d37 d38 d3d d40 d44 d45
d4a d4d d52 d53 d58 d5b d5f d63
d66 d69 d6a d6c d6d d72 d73 d78
d7a d7e d81 d83 d87 d89 d95 d97
d99 d9a d9f da3 da5 db1 db3 db7
dbc dbf dc4 dc5 dca dcd dd1 dd2
dd7 dda ddf de0 de5 de9 deb def
df3 df7 dfb dff e02 e03 e05 e08
e0b e0c e11 e15 e19 e1d e21 e25
e29 e2d e31 e35 e39 e3d e41 e45
e49 e4d e51 e55 e59 e5d e61 e65
e69 e6d e71 e75 e79 e7d e81 e85
e89 e8d e91 e95 e99 e9d ea1 ea5
ea9 ead eb1 eb5 eb9 ebd ec1 ec5
ec9 ecd ed1 ed5 ed9 edd ee1 ee5
ee9 eed ef1 ef5 ef9 efd f01 f05
f09 f0d f11 f15 f19 f1d f21 f2d
f31 f36 f3a f3d f3e f43 f47 f4b
f4e f51 f55 f57 f5b f5f f62 f65
f69 f6b f6f f73 f76 f78 f7c f80
f84 f87 f8b f8f f91 f92 f97 1
f9b f9f fa3 fa6 faa fae fb2 fb6
fb9 fbd fc1 fc4 fc7 fcc fcf fd2
fd5 fd6 fdb fdc 1 fde fe3 fe7
feb fee ff3 ff8 ffb fff 1000 1005
1008 100d 100e 1013 1016 101b 101c 1021
1024 1028 1029 102e 1031 1036 1037 103c
103f 1043 1047 104a 104d 104e 1050 1051
1056 1057 105c 105e 1062 1065 1067 106b
106d 1079 107b 107d 107e 1083 1087 1089
1095 1097 109b 109f 10a4 10a8 10a9 10ad
10b1 10b5 10b9 10be 10c3 10c7 10cb 10cf
10d4 10d8 10dc 10df 10e2 10e6 10e8 10ec
10f0 10f3 10f6 10fa 10fc 1100 1104 1107
110b 110f 1112 1116 1119 111d 1120 1121
1126 1129 112d 1133 1135 1139 113c 1140
1144 1148 114c 1150 1154 1158 115c 1160
1164 1168 116c 1170 1174 1178 117c 1180
1184 1188 118c 1190 1194 1198 119c 11a0
11a4 11a8 11ac 11b0 11b4 11b8 11bc 11c0
11c4 11c8 11cc 11d0 11d4 11d8 11dc 11e0
11e4 11e8 11ec 11f0 11f4 11f8 11fc 1200
1204 1208 120c 1210 1214 1218 121c 1220
122c 1230 1235 1239 123c 123d 1242 1246
124a 124d 1250 1254 1256 125a 125e 1261
1264 1268 126a 126e 1272 1275 1277 127b
127f 1283 1286 128a 128e 1290 1291 1296
1 129a 129e 12a2 12a5 12a9 12ad 12b1
12b5 12b8 12bc 12c0 12c3 12c6 12cb 12ce
12d1 12d4 12d5 12da 12db 1 12dd 12e2
12e6 12ea 12ed 12f2 12f7 12fa 12fe 12ff
1304 1307 130c 130d 1312 1315 131a 131b
1320 1323 1327 1328 132d 1330 1335 1336
133b 133e 1342 1346 1349 134c 134d 134f
1350 1355 1356 135b 135d 1361 1364 1366
136a 136c 1378 137a 137c 137d 1382 1386
1388 1394 1396 139a 139e 13a2 13a6 13aa
13ae 13ba 13be 13c3 13c7 13ca 13cb 13d0
13d4 13d8 13db 13de 13e2 13e4 13e8 13ec
13ef 13f2 13f6 13f8 13fc 1400 1403 1405
1409 140d 1411 1414 1418 141c 141e 141f
1424 1 1428 142c 1430 1433 1437 143b
143f 1443 1446 144a 144e 1451 1454 1459
145c 145f 1462 1463 1468 1469 1 146b
1470 1474 1478 147b 1480 1485 1488 148c
148d 1492 1495 149a 149b 14a0 14a3 14a8
14a9 14ae 14b1 14b5 14b6 14bb 14be 14c3
14c4 14c9 14cc 14d0 14d4 14d7 14da 14db
14dd 14de 14e3 14e4 14e9 14eb 14ef 14f2
14f4 14f8 14fa 1506 1508 150a 150b 1510
1514 1516 1522 1524 1526 152a 1531 1535
1539 153e 1540 1544 1548 154b 154f 1553
1555 1559 155d 1561 1564 1568 156c 156e
156f 1574 1 1578 157c 1580 1583 1587
158b 158f 1593 1596 159a 159e 15a1 15a4
15a9 15ac 15af 15b2 15b3 15b8 15b9 1
15bb 15c0 15c4 15c8 15cb 15d0 15d5 15d8
15dc 15dd 15e2 15e5 15ea 15eb 15f0 15f3
15f8 15f9 15fe 1601 1605 1606 160b 160e
1613 1614 1619 161c 1620 1624 1627 162a
162b 162d 162e 1633 1634 1639 163b 163f
1642 1644 1648 164a 1656 1658 165a 165b
1660 1664 1666 1672 1674 1676 167a 167d
1681 1685 1689 168d 1691 1695 1699 169d
16a1 16ad 16b1 16b6 16ba 16bd 16be 16c3
16c7 16cb 16ce 16d1 16d5 16d7 16db 16df
16e2 16e5 16e9 16eb 16ef 16f3 16f6 16f8
16fc 1700 1704 1707 170b 170f 1711 1712
1717 1 171b 171f 1723 1726 172a 172e
1732 1736 1739 173d 1741 1744 1747 174c
174f 1752 1755 1756 175b 175c 1 175e
1763 1767 176b 176e 1773 1778 177b 177f
1780 1785 1788 178d 178e 1793 1796 179b
179c 17a1 17a4 17a8 17a9 17ae 17b1 17b6
17b7 17bc 17bf 17c3 17c7 17ca 17cd 17ce
17d0 17d1 17d6 17d7 17dc 17de 17e2 17e5
17e7 17eb 17ed 17f9 17fb 17fd 17fe 1803
1807 1809 1815 1817 181b 181e 1822 1826
1829 182d 1831 1836 1839 183e 183f 1844
1847 184b 184c 1851 1854 1859 185a 185f
1863 1865 1869 186d 1870 1874 1878 187c
187f 1883 1887 188b 188f 1893 1897 189b
18a0 18a5 18a9 18ad 18b1 18b6 18ba 18be
18c1 18c4 18c8 18ca 18ce 18d2 18d5 18d8
18dc 18de 18e2 18e6 18e9 18ed 18f1 18f4
18f8 18fb 18ff 1902 1903 1908 190b 190f
1915 1917 191b 191e 1922 1925 1929 192d
1931 1935 1939 193d 1941 1945 1949 194d
1951 1955 1959 195d 1961 1965 1969 196d
1971 1975 1979 197d 1981 1985 1989 198d
1991 1995 1999 199d 19a9 19ad 19b2 19b6
19b9 19ba 19bf 19c3 19c7 19ca 19cd 19d1
19d3 19d7 19db 19de 19e1 19e5 19e7 19eb
19ef 19f2 19f4 19f8 19fc 1a00 1a03 1a07
1a0b 1a0d 1a0e 1a13 1 1a17 1a1b 1a1f
1a22 1a26 1a2a 1a2e 1a32 1a35 1a39 1a3d
1a40 1a43 1a48 1a4b 1a4e 1a51 1a52 1a57
1a58 1 1a5a 1a5f 1a63 1a67 1a6a 1a6f
1a74 1a77 1a7b 1a7c 1a81 1a84 1a89 1a8a
1a8f 1a92 1a97 1a98 1a9d 1aa0 1aa4 1aa5
1aaa 1aad 1ab2 1ab3 1ab8 1abb 1abf 1ac3
1ac6 1ac9 1aca 1acc 1acd 1ad2 1ad3 1ad8
1ada 1ade 1ae1 1ae3 1ae7 1ae9 1af5 1af7
1af9 1afa 1aff 1b03 1b05 1b11 1b13 1b17
1b1b 1b1f 1b23 1b27 1b2b 1b2f 1b33 1b37
1b3b 1b3f 1b43 1b47 1b4b 1b4f 1b53 1b57
1b5b 1b5f 1b63 1b67 1b6b 1b6f 1b73 1b77
1b7b 1b7f 1b83 1b87 1b8b 1b8f 1b93 1b97
1b9b 1b9f 1ba3 1ba7 1bab 1baf 1bb3 1bb7
1bbb 1bbf 1bc3 1bc7 1bcb 1bcf 1bd3 1bd7
1bdb 1bdf 1be3 1be7 1beb 1bef 1bf3 1bf7
1bfb 1bff 1c03 1c07 1c0b 1c0f 1c13 1c17
1c1b 1c1f 1c23 1c27 1c2b 1c2f 1c33 1c37
1c3b 1c3f 1c43 1c47 1c4b 1c4f 1c53 1c57
1c5b 1c5f 1c63 1c67 1c6b 1c6f 1c73 1c77
1c7b 1c7f 1c83 1c87 1c8b 1c8f 1c93 1c97
1c9b 1ca7 1cab 1cb0 1cb4 1cb7 1cb8 1cbd
1cc1 1cc5 1cc8 1ccb 1ccf 1cd1 1cd5 1cd9
1cdc 1cdf 1ce3 1ce5 1ce9 1ced 1cf0 1cf2
1cf6 1cfa 1cfe 1d01 1d05 1d09 1d0b 1d0c
1d11 1 1d15 1d19 1d1d 1d20 1d24 1d28
1d2c 1d30 1d33 1d37 1d3b 1d3e 1d41 1d46
1d49 1d4c 1d4f 1d50 1d55 1d56 1 1d58
1d5d 1d61 1d65 1d68 1d6d 1d72 1d75 1d79
1d7a 1d7f 1d82 1d87 1d88 1d8d 1d90 1d95
1d96 1d9b 1d9e 1da2 1da3 1da8 1dab 1db0
1db1 1db6 1db9 1dbd 1dc1 1dc4 1dc7 1dc8
1dca 1dcb 1dd0 1dd1 1dd6 1dd8 1ddc 1ddf
1de1 1de5 1de7 1df3 1df5 1df7 1df8 1dfd
1e01 1e03 1e0f 1e11 1e15 1e19 1e1d 1e21
1e25 1e29 1e2d 1e31 1e35 1e39 1e3d 1e41
1e45 1e49 1e4d 1e51 1e55 1e59 1e5d 1e61
1e65 1e69 1e6d 1e71 1e75 1e79 1e7d 1e81
1e85 1e89 1e8d 1e91 1e95 1e99 1e9d 1ea1
1ea5 1ea9 1ead 1eb1 1eb5 1eb9 1ebd 1ec1
1ec5 1ec9 1ecd 1ed1 1ed5 1ed9 1edd 1ee1
1ee5 1ee9 1eed 1ef1 1ef5 1ef9 1f05 1f09
1f0e 1f12 1f15 1f16 1f1b 1f1f 1f23 1f26
1f29 1f2d 1f2f 1f33 1f37 1f3a 1f3d 1f41
1f43 1f47 1f4b 1f4e 1f50 1f54 1f58 1f5c
1f5f 1f63 1f67 1f69 1f6a 1f6f 1 1f73
1f77 1f7b 1f7e 1f82 1f86 1f8a 1f8e 1f91
1f95 1f99 1f9c 1f9f 1fa4 1fa7 1faa 1fad
1fae 1fb3 1fb4 1 1fb6 1fbb 1fbf 1fc3
1fc6 1fcb 1fd0 1fd3 1fd7 1fd8 1fdd 1fe0
1fe5 1fe6 1feb 1fee 1ff3 1ff4 1ff9 1ffc
2000 2001 2006 2009 200e 200f 2014 2017
201b 201f 2022 2025 2026 2028 2029 202e
202f 2034 2036 203a 203d 203f 2043 2045
2051 2053 2055 2056 205b 205f 2061 206d
206f 2071 2072 2077 207a 207e 2082 2086
208a 208e 2092 209e 20a2 20a6 20a9 20ad
20b1 20b3 20b7 20bb 20bf 20c2 20c6 20ca
20cc 20cd 20d2 1 20d6 20da 20de 20e1
20e5 20e9 20ed 20f1 20f4 20f8 20fc 20ff
2102 2107 210a 210d 2110 2111 2116 2117
1 2119 211e 2122 2126 2129 212e 2133
2136 213a 213b 2140 2143 2148 2149 214e
2151 2156 2157 215c 215f 2163 2164 2169
216c 2171 2172 2177 217a 217e 2182 2185
2188 2189 218b 218c 2191 2192 2197 2199
219d 21a0 21a2 21a6 21a8 21b4 21b6 21b8
21b9 21be 21c2 21c4 21d0 21d2 21d6 21da
21dd 21e1 21e4 21e8 21eb 21ec 21f1 21f4
21f8 21fb 21fc 2201 2204 1 2207 220c
2210 2216 2218 221c 221f 2223 2227 222b
222f 2233 2237 223b 223f 2243 2247 224b
224f 225b 225f 2263 2266 226a 226e 2270
2274 2278 227c 227f 2283 2287 2289 228a
228f 1 2293 2297 229b 229e 22a2 22a6
22aa 22ae 22b1 22b5 22b9 22bc 22bf 22c4
22c7 22ca 22cd 22ce 22d3 22d4 1 22d6
22db 22df 22e3 22e6 22eb 22f0 22f3 22f7
22f8 22fd 2300 2305 2306 230b 230e 2313
2314 2319 231c 2320 2321 2326 2329 232e
232f 2334 2337 233b 233f 2342 2345 2346
2348 2349 234e 234f 2354 2356 235a 235d
235f 2363 2365 2371 2373 2375 2376 237b
237f 2381 238d 238f 2393 2397 239a 239e
23a1 23a5 23a8 23a9 23ae 23b1 23b5 23b8
23b9 23be 23c1 1 23c4 23c9 23cd 23d1
23d4 23d5 23d7 23da 23dd 23de 23e3 23e7
23ea 23ee 23f0 23f4 23f8 23fc 2408 240c
2411 2415 2418 2419 241e 2422 2426 2429
242c 2430 2432 2436 243a 243d 2440 2444
2446 244a 244e 2451 2453 2457 245b 245f
2462 2466 246a 246c 246d 2472 1 2476
247a 247e 2481 2485 2489 248d 2491 2494
2498 249c 249f 24a2 24a7 24aa 24ad 24b0
24b1 24b6 24b7 1 24b9 24be 24c2 24c6
24c9 24ce 24d3 24d6 24da 24db 24e0 24e3
24e8 24e9 24ee 24f1 24f6 24f7 24fc 24ff
2503 2504 2509 250c 2511 2512 2517 251a
251e 2522 2525 2528 2529 252b 252c 2531
2532 2537 2539 253d 2540 2542 2546 2548
2554 2556 2558 2559 255e 2562 2564 2570
2572 2576 257a 257e 258a 258e 2593 2597
259a 259b 25a0 25a4 25a8 25ab 25ae 25b2
25b4 25b8 25bc 25bf 25c2 25c6 25c8 25cc
25d0 25d3 25d5 25d9 25dd 25e1 25e4 25e8
25ec 25ee 25ef 25f4 1 25f8 25fc 2600
2603 2607 260b 260f 2613 2616 261a 261e
2621 2624 2629 262c 262f 2632 2633 2638
2639 1 263b 2640 2644 2648 264b 2650
2655 2658 265c 265d 2662 2665 266a 266b
2670 2673 2678 2679 267e 2681 2685 2686
268b 268e 2693 2694 2699 269c 26a0 26a4
26a7 26aa 26ab 26ad 26ae 26b3 26b4 26b9
26bb 26bf 26c2 26c4 26c8 26ca 26d6 26d8
26da 26db 26e0 26e4 26e6 26f2 26f4 26f8
26fc 2700 270c 2710 2715 2719 271c 271d
2722 2726 272a 272d 2730 2734 2736 273a
273e 2741 2744 2748 274a 274e 2752 2755
2757 275b 275f 2763 2766 276a 276e 2770
2771 2776 1 277a 277e 2782 2785 2789
278d 2791 2795 2798 279c 27a0 27a3 27a6
27ab 27ae 27b1 27b4 27b5 27ba 27bb 1
27bd 27c2 27c6 27ca 27cd 27d2 27d7 27da
27de 27df 27e4 27e7 27ec 27ed 27f2 27f5
27fa 27fb 2800 2803 2807 2808 280d 2810
2815 2816 281b 281e 2822 2826 2829 282c
282d 282f 2830 2835 2836 283b 283d 2841
2844 2846 284a 284c 2858 285a 285c 285d
2862 2866 2868 2874 2876 287a 287e 2882
288e 2892 2897 289b 289e 289f 28a4 28a8
28ac 28af 28b2 28b6 28b8 28bc 28c0 28c3
28c6 28ca 28cc 28d0 28d4 28d7 28d9 28dd
28e1 28e5 28e8 28ec 28f0 28f2 28f3 28f8
1 28fc 2900 2904 2907 290b 290f 2913
2917 291a 291e 2922 2925 2928 292d 2930
2933 2936 2937 293c 293d 1 293f 2944
2948 294c 294f 2954 2959 295c 2960 2961
2966 2969 296e 296f 2974 2977 297c 297d
2982 2985 2989 298a 298f 2992 2997 2998
299d 29a0 29a4 29a8 29ab 29ae 29af 29b1
29b2 29b7 29b8 29bd 29bf 29c3 29c6 29c8
29cc 29ce 29da 29dc 29de 29df 29e4 29e8
29ea 29f6 29f8 29fc 29ff 2a03 2a05 2a09
2a0d 2a10 2a12 2a16 2a1a 2a1e 2a22 2a25
2a29 2a2b 2a2f 2a33 2a36 2a3a 2a3d 2a41
2a45 2a49 2a4e 2a52 2a53 2a57 2a5b 2a5f
2a63 2a67 2a6c 2a71 2a75 2a79 2a7d 2a82
2a86 2a8a 2a8d 2a90 2a94 2a96 2a9a 2a9e
2aa1 2aa4 2aa8 2aaa 2aae 2ab2 2ab5 2ab9
2abd 2ac0 2ac4 2ac7 2acb 2ace 2acf 2ad4
2ad7 2adb 2ae1 2ae3 2ae7 2aea 2aee 2af2
2af5 2af6 2af8 2afb 2afe 2aff 2b04 2b08
2b0c 2b10 2b14 2b18 2b1c 2b20 2b24 2b28
2b2c 2b30 2b34 2b38 2b3c 2b40 2b4c 2b50
2b55 2b59 2b5c 2b5d 2b62 2b66 2b6a 2b6d
2b70 2b74 2b76 2b7a 2b7e 2b81 2b84 2b88
2b8a 2b8e 2b92 2b95 2b97 2b9b 2b9f 2ba3
2ba6 2baa 2bae 2bb0 2bb1 2bb6 1 2bba
2bbe 2bc2 2bc5 2bc9 2bcd 2bd1 2bd5 2bd8
2bdc 2be0 2be3 2be6 2beb 2bee 2bf1 2bf4
2bf5 2bfa 2bfb 1 2bfd 2c02 2c06 2c0a
2c0d 2c12 2c17 2c1a 2c1e 2c1f 2c24 2c27
2c2c 2c2d 2c32 2c35 2c3a 2c3b 2c40 2c43
2c47 2c48 2c4d 2c50 2c55 2c56 2c5b 2c5e
2c62 2c66 2c69 2c6c 2c6d 2c6f 2c70 2c75
2c76 2c7b 2c7d 2c81 2c84 2c86 2c8a 2c8c
2c98 2c9a 2c9c 2c9d 2ca2 2ca6 2ca8 2cb4
2cb6 2cba 2cbe 2cc1 2cc5 2cc8 2ccc 2ccf
2cd0 2cd5 2cd8 2cdc 2ce0 2ce4 2ce8 2cec
2cf0 2cf4 2d00 2d04 2d09 2d0d 2d10 2d11
2d16 2d1a 2d1e 2d21 2d24 2d28 2d2a 2d2e
2d32 2d35 2d38 2d3c 2d3e 2d42 2d46 2d49
2d4b 2d4f 2d53 2d57 2d5a 2d5e 2d62 2d64
2d65 2d6a 1 2d6e 2d72 2d76 2d79 2d7d
2d81 2d85 2d89 2d8c 2d90 2d94 2d97 2d9a
2d9f 2da2 2da5 2da8 2da9 2dae 2daf 1
2db1 2db6 2dba 2dbe 2dc1 2dc6 2dcb 2dce
2dd2 2dd3 2dd8 2ddb 2de0 2de1 2de6 2de9
2dee 2def 2df4 2df7 2dfb 2dfc 2e01 2e04
2e09 2e0a 2e0f 2e12 2e16 2e1a 2e1d 2e20
2e21 2e23 2e24 2e29 2e2a 2e2f 2e31 2e35
2e38 2e3a 2e3e 2e40 2e4c 2e4e 2e50 2e51
2e56 2e5a 2e5c 2e68 2e6a 2e6c 2e70 2e74
2e78 2e7c 2e80 2e84 2e88 2e8c 2e90 2e9c
2ea0 2ea5 2ea9 2eac 2ead 2eb2 2eb6 2eba
2ebd 2ec0 2ec4 2ec6 2eca 2ece 2ed1 2ed4
2ed8 2eda 2ede 2ee2 2ee5 2ee7 2eeb 2eef
2ef3 2ef6 2efa 2efe 2f00 2f01 2f06 1
2f0a 2f0e 2f12 2f15 2f19 2f1d 2f21 2f25
2f28 2f2c 2f30 2f33 2f36 2f3b 2f3e 2f41
2f44 2f45 2f4a 2f4b 1 2f4d 2f52 2f56
2f5a 2f5d 2f62 2f67 2f6a 2f6e 2f6f 2f74
2f77 2f7c 2f7d 2f82 2f85 2f8a 2f8b 2f90
2f93 2f97 2f98 2f9d 2fa0 2fa5 2fa6 2fab
2fae 2fb2 2fb6 2fb9 2fbc 2fbd 2fbf 2fc0
2fc5 2fc6 2fcb 2fcd 2fd1 2fd4 2fd6 2fda
2fdc 2fe8 2fea 2fec 2fed 2ff2 2ff6 2ff8
3004 3006 3008 300c 3010 3013 3017 3019
301d 3021 3024 3025 3027 302a 302d 302e
3033 3037 303b 303f 3043 3047 304b 304f
3053 3057 3063 3067 306c 3070 3073 3074
3079 307d 3081 3084 3087 308b 308d 3091
3095 3098 309b 309f 30a1 30a5 30a9 30ac
30ae 30b2 30b6 30ba 30bd 30c1 30c5 30c7
30c8 30cd 1 30d1 30d5 30d9 30dc 30e0
30e4 30e8 30ec 30ef 30f3 30f7 30fa 30fd
3102 3105 3108 310b 310c 3111 3112 1
3114 3119 311d 3121 3124 3129 312e 3131
3135 3136 313b 313e 3143 3144 3149 314c
3151 3152 3157 315a 315e 315f 3164 3167
316c 316d 3172 3175 3179 317d 3180 3183
3184 3186 3187 318c 318d 3192 3194 3198
319b 319d 31a1 31a3 31af 31b1 31b3 31b4
31b9 31bd 31bf 31cb 31cd 31cf 31d3 31d7
31da 31dc 31e0 31e7 31eb 31ef 31f4 31f6
31fa 31fe 3201 3205 3209 320b 320f 3213
3217 321a 321e 3222 3224 3225 322a 1
322e 3232 3236 3239 323d 3241 3245 3249
324c 3250 3254 3257 325a 325f 3262 3265
3268 3269 326e 326f 1 3271 3276 327a
327e 3281 3286 328b 328e 3292 3293 3298
329b 32a0 32a1 32a6 32a9 32ae 32af 32b4
32b7 32bb 32bc 32c1 32c4 32c9 32ca 32cf
32d2 32d6 32da 32dd 32e0 32e1 32e3 32e4
32e9 32ea 32ef 32f1 32f5 32f8 32fa 32fe
3300 330c 330e 3310 3311 3316 331a 331c
3328 332a 332e 3332 3336 333a 333e 3342
3346 334a 3356 335a 335f 3363 3366 3367
336c 3370 3374 3377 337a 337e 3380 3384
3388 338b 338e 3392 3394 3398 339c 339f
33a1 33a5 33a9 33ad 33b0 33b4 33b8 33ba
33bb 33c0 1 33c4 33c8 33cc 33cf 33d3
33d7 33db 33df 33e2 33e6 33ea 33ed 33f0
33f5 33f8 33fb 33fe 33ff 3404 3405 1
3407 340c 3410 3414 3417 341c 3421 3424
3428 3429 342e 3431 3436 3437 343c 343f
3444 3445 344a 344d 3451 3452 3457 345a
345f 3460 3465 3468 346c 3470 3473 3476
3477 3479 347a 347f 3480 3485 3487 348b
348e 3490 3494 3496 34a2 34a4 34a6 34a7
34ac 34b0 34b2 34be 34c0 34c4 34c8 34cb
34cc 34ce 34d1 34d4 34d5 34da 34de 34e2
34e6 34ea 34ee 34f2 34f6 34fa 34fe 3502
3506 350a 350e 3512 3516 351a 351e 3522
3526 352a 352e 3532 3536 353a 353e 3542
3546 354a 354e 3552 3556 355a 355e 3562
3566 356a 356e 3572 3576 357a 357e 3582
3586 358a 358e 3592 3596 359a 359e 35a2
35a6 35aa 35ae 35b2 35b6 35c2 35c6 35cb
35cf 35d2 35d3 35d8 35dc 35e0 35e3 35e6
35ea 35ec 35f0 35f4 35f7 35fa 35fe 3600
3604 3608 360b 360d 3611 3615 3619 361c
3620 3624 3626 3627 362c 1 3630 3634
3638 363b 363f 3643 3647 364b 364e 3652
3656 3659 365c 3661 3664 3667 366a 366b
3670 3671 1 3673 3678 367c 3680 3683
3688 368d 3690 3694 3695 369a 369d 36a2
36a3 36a8 36ab 36b0 36b1 36b6 36b9 36bd
36be 36c3 36c6 36cb 36cc 36d1 36d4 36d8
36dc 36df 36e2 36e3 36e5 36e6 36eb 36ec
36f1 36f3 36f7 36fa 36fc 3700 3702 370e
3710 3712 3713 3718 371c 371e 372a 372c
3730 3734 3737 3738 373a 373d 3740 3741
3746 374a 374d 3751 3755 3759 375d 3761
3765 3768 376b 376c 376e 3772 3774 3778
377b 377d 3781 3785 378a 378e 378f 3793
3797 379b 379f 37a4 37a9 37ad 37b1 37b5
37ba 37be 37c2 37c5 37c8 37cc 37ce 37d2
37d6 37d9 37dc 37e0 37e2 37e6 37ea 37ed
37f1 37f5 37f8 37fc 37ff 3803 3806 3807
380c 380f 3813 3819 381b 381f 3822 3826
382a 382e 3832 3836 383a 383e 3842 3846
384a 384e 3852 3856 385a 385e 3862 3866
386a 386e 3872 3876 387a 387e 3882 3886
388a 388e 3892 3896 389a 389e 38a2 38a6
38aa 38ae 38b2 38b6 38ba 38be 38c2 38c6
38ca 38ce 38d2 38d6 38da 38de 38e2 38e6
38ea 38ee 38f2 38f6 38fa 3906 390a 390f
3913 3916 3917 391c 3920 3924 3927 392a
392e 3930 3934 3938 393b 393e 3942 3944
3948 394c 394f 3951 3955 3959 395d 3960
3964 3968 396a 396b 3970 1 3974 3978
397c 397f 3983 3987 398b 398f 3992 3996
399a 399d 39a0 39a5 39a8 39ab 39ae 39af
39b4 39b5 1 39b7 39bc 39c0 39c4 39c7
39cc 39d1 39d4 39d8 39d9 39de 39e1 39e6
39e7 39ec 39ef 39f4 39f5 39fa 39fd 3a01
3a02 3a07 3a0a 3a0f 3a10 3a15 3a18 3a1c
3a20 3a23 3a26 3a27 3a29 3a2a 3a2f 3a30
3a35 3a37 3a3b 3a3e 3a40 3a44 3a46 3a52
3a54 3a56 3a57 3a5c 3a60 3a62 3a6e 3a70
3a74 3a78 3a7c 3a80 3a84 3a88 3a94 3a98
3a9d 3aa1 3aa4 3aa5 3aaa 3aae 3ab2 3ab5
3ab8 3abc 3abe 3ac2 3ac6 3ac9 3acc 3ad0
3ad2 3ad6 3ada 3add 3adf 3ae3 3ae7 3aeb
3aee 3af2 3af6 3af8 3af9 3afe 1 3b02
3b06 3b0a 3b0d 3b11 3b15 3b19 3b1d 3b20
3b24 3b28 3b2b 3b2e 3b33 3b36 3b39 3b3c
3b3d 3b42 3b43 1 3b45 3b4a 3b4e 3b52
3b55 3b5a 3b5f 3b62 3b66 3b67 3b6c 3b6f
3b74 3b75 3b7a 3b7d 3b82 3b83 3b88 3b8b
3b8f 3b90 3b95 3b98 3b9d 3b9e 3ba3 3ba6
3baa 3bae 3bb1 3bb4 3bb5 3bb7 3bb8 3bbd
3bbe 3bc3 3bc5 3bc9 3bcc 3bce 3bd2 3bd4
3be0 3be2 3be4 3be5 3bea 3bee 3bf0 3bfc
3bfe 3c00 3c04 3c0b 3c0f 3c13 3c18 3c1a
3c1e 3c22 3c25 3c29 3c2d 3c2f 3c33 3c37
3c3b 3c3e 3c42 3c46 3c48 3c49 3c4e 1
3c52 3c56 3c5a 3c5d 3c61 3c65 3c69 3c6d
3c70 3c74 3c78 3c7b 3c7e 3c83 3c86 3c89
3c8c 3c8d 3c92 3c93 1 3c95 3c9a 3c9e
3ca2 3ca5 3caa 3caf 3cb2 3cb6 3cb7 3cbc
3cbf 3cc4 3cc5 3cca 3ccd 3cd2 3cd3 3cd8
3cdb 3cdf 3ce0 3ce5 3ce8 3ced 3cee 3cf3
3cf6 3cfa 3cfe 3d01 3d04 3d05 3d07 3d08
3d0d 3d0e 3d13 3d15 3d19 3d1c 3d1e 3d22
3d24 3d30 3d32 3d34 3d35 3d3a 3d3e 3d40
3d4c 3d4e 3d52 3d56 3d59 3d5a 3d5c 3d5f
3d62 3d63 3d68 3d6b 3d6f 3d73 3d76 3d77
3d79 3d7c 3d7f 3d80 3d85 1 3d88 3d8d
3d91 3d95 3d99 3d9d 3da1 3dad 3db1 3db6
3dba 3dbd 3dbe 3dc3 3dc7 3dcb 3dce 3dd1
3dd5 3dd7 3ddb 3ddf 3de2 3de5 3de9 3deb
3def 3df3 3df6 3df8 3dfc 3e00 3e04 3e07
3e0b 3e0f 3e11 3e12 3e17 1 3e1b 3e1f
3e23 3e26 3e2a 3e2e 3e32 3e36 3e39 3e3d
3e41 3e44 3e47 3e4c 3e4f 3e52 3e55 3e56
3e5b 3e5c 1 3e5e 3e63 3e67 3e6b 3e6e
3e73 3e78 3e7b 3e7f 3e80 3e85 3e88 3e8d
3e8e 3e93 3e96 3e9b 3e9c 3ea1 3ea4 3ea8
3ea9 3eae 3eb1 3eb6 3eb7 3ebc 3ebf 3ec3
3ec7 3eca 3ecd 3ece 3ed0 3ed1 3ed6 3ed7
3edc 3ede 3ee2 3ee5 3ee7 3eeb 3eed 3ef9
3efb 3efd 3efe 3f03 3f07 3f09 3f15 3f17
3f19 3f1d 3f20 3f24 3f27 3f2b 3f2f 3f33
3f37 3f3b 3f3f 3f42 3f45 3f46 3f48 3f4c
3f50 3f54 3f58 3f5c 3f60 3f64 3f68 3f6c
3f70 3f74 3f78 3f7c 3f80 3f84 3f88 3f8c
3f90 3f9c 3fa0 3fa4 3fa7 3fab 3faf 3fb1
3fb5 3fb9 3fbd 3fc0 3fc4 3fc8 3fca 3fcb
3fd0 1 3fd4 3fd8 3fdc 3fdf 3fe3 3fe7
3feb 3fef 3ff2 3ff6 3ffa 3ffd 4000 4005
4008 400b 400e 400f 4014 4015 1 4017
401c 4020 4024 4027 402c 4031 4034 4038
4039 403e 4041 4046 4047 404c 404f 4054
4055 405a 405d 4061 4062 4067 406a 406f
4070 4075 4078 407c 4080 4083 4086 4087
4089 408a 408f 4090 4095 4097 409b 409e
40a0 40a4 40a6 40b2 40b4 40b6 40b7 40bc
40c0 40c2 40ce 40d0 40d4 40d8 40db 40dc
40de 40e1 40e4 40e5 40ea 40ee 40f2 40f6
40fa 40fe 4102 4106 410a 410e 4112 4116
411a 411e 4122 4126 412a 412e 4132 4136
413a 4146 414a 414f 4153 4156 4157 415c
4160 4164 4167 416a 416e 4170 4174 4178
417b 417e 4182 4184 4188 418c 418f 4191
4195 4199 419d 41a0 41a4 41a8 41aa 41ab
41b0 1 41b4 41b8 41bc 41bf 41c3 41c7
41cb 41cf 41d2 41d6 41da 41dd 41e0 41e5
41e8 41eb 41ee 41ef 41f4 41f5 1 41f7
41fc 4200 4204 4207 420c 4211 4214 4218
4219 421e 4221 4226 4227 422c 422f 4234
4235 423a 423d 4241 4242 4247 424a 424f
4250 4255 4258 425c 4260 4263 4266 4267
4269 426a 426f 4270 4275 4277 427b 427e
4280 4284 4286 4292 4294 4296 4297 429c
42a0 42a2 42ae 42b0 42b2 42b6 42ba 42be
42c2 42c6 42ca 42ce 42d2 42d6 42da 42de
42e2 42e6 42ea 42ee 42f2 42f6 42fa 42fe
4302 4306 430a 4316 431a 431f 4323 4326
4327 432c 4330 4334 4337 433a 433e 4340
4344 4348 434b 434e 4352 4354 4358 435c
435f 4361 4365 4369 436d 4370 4374 4378
437a 437b 4380 1 4384 4388 438c 438f
4393 4397 439b 439f 43a2 43a6 43aa 43ad
43b0 43b5 43b8 43bb 43be 43bf 43c4 43c5
1 43c7 43cc 43d0 43d4 43d7 43dc 43e1
43e4 43e8 43e9 43ee 43f1 43f6 43f7 43fc
43ff 4404 4405 440a 440d 4411 4412 4417
441a 441f 4420 4425 4428 442c 4430 4433
4436 4437 4439 443a 443f 4440 4445 4447
444b 444e 4450 4454 4456 4462 4464 4466
4467 446c 4470 4472 447e 4480 4482 4486
448a 448d 448f 4493 4497 449a 44a2 44a4
44a5 44a8 44aa 44ab 44b0 44b2 44b4 44b9
44bd 44c2 44c6 44ca 44ce 44d2 44d6 44da
44e6 44ea 44ef 44f3 44f6 44f7 44fc 4500
4504 4507 450a 450e 4510 4514 4518 451b
451e 4522 4524 4528 452c 452f 4531 4535
4539 453d 4540 4544 4548 454a 454b 4550
1 4554 4558 455c 455f 4563 4567 456b
456f 4572 4576 457a 457d 4580 4585 4588
458b 458e 458f 4594 4595 1 4597 459c
45a0 45a4 45a7 45ac 45b1 45b4 45b8 45b9
45be 45c1 45c6 45c7 45cc 45cf 45d4 45d5
45da 45dd 45e1 45e2 45e7 45ea 45ef 45f0
45f5 45f8 45fc 4600 4603 4606 4607 4609
460a 460f 4610 4615 4617 461b 461e 4620
4624 4626 4632 4634 4636 4637 463c 4640
4642 464e 4650 4652 4656 465d 4661 4665
466a 466c 4670 4674 4677 467b 467f 4681
4685 4689 468d 4690 4694 4698 469a 469b
46a0 1 46a4 46a8 46ac 46af 46b3 46b7
46bb 46bf 46c2 46c6 46ca 46cd 46d0 46d5
46d8 46db 46de 46df 46e4 46e5 1 46e7
46ec 46f0 46f4 46f7 46fc 4701 4704 4708
4709 470e 4711 4716 4717 471c 471f 4724
4725 472a 472d 4731 4732 4737 473a 473f
4740 4745 4748 474c 4750 4753 4756 4757
4759 475a 475f 4760 4765 4767 476b 476e
4770 4774 4776 4782 4784 4786 4787 478c
4790 4792 479e 47a0 47a4 47a8 47ac 47ad
47af 47b3 47b7 47bb 47bf 47c3 47c7 47cb
47cf 47d3 47d7 47db 47df 47e3 47e7 47f3
47f7 47fb 47fe 4802 4806 4808 480c 4810
4814 4817 481b 481f 4821 4822 4827 1
482b 482f 4833 4836 483a 483e 4842 4846
4849 484d 4851 4854 4857 485c 485f 4862
4865 4866 486b 486c 1 486e 4873 4877
487b 487e 4883 4888 488b 488f 4890 4895
4898 489d 489e 48a3 48a6 48ab 48ac 48b1
48b4 48b8 48b9 48be 48c1 48c6 48c7 48cc
48cf 48d3 48d7 48da 48dd 48de 48e0 48e1
48e6 48e7 48ec 48ee 48f2 48f5 48f7 48fb
48fd 4909 490b 490d 490e 4913 4917 4919
4925 4927 492b 492f 4932 4933 4935 4938
493b 493c 4941 4945 4949 494d 4950 4954
4958 495c 495f 4963 4967 496b 496f 4970
4972 4976 4978 497c 497f 4983 4987 498b
498f 4993 4997 499b 499f 49ab 49af 49b4
49b8 49bb 49bc 49c1 49c5 49c9 49cc 49cf
49d3 49d5 49d9 49dd 49e0 49e3 49e7 49e9
49ed 49f1 49f4 49f6 49fa 49fe 4a02 4a05
4a09 4a0d 4a0f 4a10 4a15 1 4a19 4a1d
4a21 4a24 4a28 4a2c 4a30 4a34 4a37 4a3b
4a3f 4a42 4a45 4a4a 4a4d 4a50 4a53 4a54
4a59 4a5a 1 4a5c 4a61 4a65 4a69 4a6c
4a71 4a76 4a79 4a7d 4a7e 4a83 4a86 4a8b
4a8c 4a91 4a94 4a99 4a9a 4a9f 4aa2 4aa6
4aa7 4aac 4aaf 4ab4 4ab5 4aba 4abd 4ac1
4ac5 4ac8 4acb 4acc 4ace 4acf 4ad4 4ad5
4ada 4adc 4ae0 4ae3 4ae5 4ae9 4aeb 4af7
4af9 4afb 4afc 4b01 4b05 4b07 4b13 4b15
4b19 4b1d 4b20 4b23 4b26 4b27 4b2c 4b30
4b34 4b37 4b3a 4b3e 4b40 4b44 4b47 4b4b
4b4f 4b52 4b56 4b5a 4b5d 4b60 4b65 4b68
4b6b 4b6e 4b6f 4b74 4b75 1 4b77 4b7c
4b80 4b84 4b87 4b8c 4b91 4b94 4b98 4b99
4b9e 4ba1 4ba6 4ba7 4bac 4baf 4bb4 4bb5
4bba 4bbd 4bc1 4bc5 4bc9 4bcd 4bd0 4bd1
4bd3 4bd8 4bd9 4bdb 4bdc 4be1 4be4 4be9
4bea 4bef 4bf2 4bf7 4bf8 4bfd 4c00 4c04
4c05 4c0a 4c0d 4c12 4c13 4c18 4c19 4c1e
4c20 4c24 4c27 4c2b 4c2f 4c32 4c36 4c3a
4c3d 4c41 4c45 4c46 4c4b 4c4d 4c51 4c55
4c58 4c5c 4c60 4c63 4c67 4c6b 4c6e 4c71
4c76 4c79 4c7c 4c7f 4c80 4c85 4c86 1
4c88 4c8d 4c91 4c95 4c98 4c9d 4ca2 4ca5
4ca9 4caa 4caf 4cb0 4cb5 4cb9 4cbd 4cc0
4cc5 4cca 4ccd 4cd1 4cd5 4cd8 4cd9 4cde
4ce1 4ce6 4ce7 4cec 4cef 4cf3 4cf7 4cfa
4cfb 4d00 4d01 4d06 4d08 4d0c 4d0f 4d13
4d17 4d1b 4d1f 4d23 4d27 4d2b 4d2f 4d33
4d3f 4d44 4d48 4d4c 4d50 4d54 4d56 4d57
4d5c 1 4d60 4d64 4d68 4d6b 4d6f 4d73
4d77 4d7b 4d7e 4d82 4d86 4d89 4d8c 4d91
4d94 4d97 4d9a 4d9b 4da0 4da1 1 4da3
4da8 4dac 4db0 4db3 4db8 4dbd 4dc0 4dc4
4dc5 4dca 4dcb 4dd0 4dd4 4dd8 4ddb 4de0
4de5 4de8 4dec 4ded 4df2 4df5 4dfa 4dfb
4e00 4e03 4e07 4e0b 4e0e 4e11 4e12 4e14
4e15 4e1a 4e1b 4e20 4e22 4e26 4e29 4e2d
4e31 4e35 4e39 4e3d 4e41 4e45 4e51 4e56
4e5a 4e5e 4e62 4e66 4e68 4e6c 4e6e 4e7a
4e7c 4e7e 4e7f 4e84 4e88 4e8a 4e96 4e9a
4e9c 4e9d 4ea6 
15eb
2
0 1 a 6 9 :3 6 11 :2 6
1 3 a 13 a :2 1 4 c
10 f c 20 :2 29 c :2 4 d
11 10 d 21 :2 2a d :2 4 11
1a 19 :2 11 :2 4 :3 10 :2 4 d 16
15 :2 d :2 4 e 17 16 :2 e :2 4
9 12 11 :2 9 :2 4 18 21 20
:2 18 :2 4 18 21 20 :2 18 :2 4 :3 11
:2 4 :3 12 :2 4 :2 11 20 11 :2 4 :2 f
1e f :2 4 :2 c 1b c :2 4 :2 e
1d e :2 4 :2 e 1d e :2 4 :2 12
21 12 :2 4 :2 12 21 12 :2 4 :2 e
1d e :2 4 :2 f 1e f :2 4 :2 e
1d e :2 4 :2 b 1a b :2 4 :2 f
1e f :2 4 :2 14 23 14 :2 4 :2 f
19 f :2 4 :2 f 19 f :2 4 9
12 11 :2 9 4 7 e 0 :2 7
11 14 27 2a f 20 23 28
30 d f 1c 1f 2e 30 3d
40 10 12 1f 30 33 e 11
29 2c a :6 7 c 15 :3 c :2 7
e 0 :2 7 11 1b 26 35 10
1d 2e 42 4b a :6 7 e 0
:2 7 11 1a f 10 23 3b a
:5 7 4 e 17 :3 e 4 7 :2 11
23 :2 2d 3a :2 23 :3 7 :2 d 19 25
:2 2f 43 :2 25 58 5e 61 :2 58 6f
89 8 :2 6f d 10 :2 6f 13 16
:2 6f b e :2 6f 13 16 1f 27
:2 1f 34 :2 16 :2 6f 3c 3f :2 6f :2 7
4e :2 4 7 :2 11 23 :2 2d 39 3c
50 52 53 :2 50 :2 23 :3 7 :2 d 12
1e 61 64 :2 1e :2 7 56 :3 4 11
:2 4 c 4 e 17 31 44 52
5f :2 7 :2 10 1c 7 4 c a
:2 13 1f a 1a :2 7 :2 c a :2 13
1f a d :2 17 29 :2 33 3f 43
57 59 5a :2 57 :2 29 :3 d :2 13 18
24 67 6a :2 24 77 7a :2 24 7e
81 :2 24 8d 90 :2 24 98 9b :2 24
a7 aa b1 ba bd :2 aa :2 24 :2 d
5d :2 a :2 7 :4 13 :2 7 4 :3 1 8
:2 11 1c 1a :2 25 :2 1a 7 11 27
34 59 f 1c 26 f 26 44
5a 6e :2 a :2 13 1f a 7 f
d :2 16 22 d 1d :2 a :2 f d
:2 16 22 d 10 :2 1a 2c :2 36 42
46 5a 5d 5e :2 5a :2 2c :3 10 :2 16
1b 27 6b 6e :2 27 7b 7e :2 27
82 85 :2 27 91 94 :2 27 9c 9f
:2 27 ab ae b5 be c1 :2 ae :2 27
:2 10 61 :2 d :2 a :4 16 :2 a 7 :3 35
b :2 14 1f 1d :2 28 :2 1d a d
:2 17 29 :2 33 3f 42 56 59 5a
:2 56 :2 29 :3 d :2 13 18 24 68 6b
:2 24 78 7b :2 24 7f 82 :2 24 8c
8f 98 a0 :2 a9 :2 98 bc :2 8f :2 24
:2 d 5d :2 a d :2 17 29 :2 33 3f
42 56 59 5a :2 56 :2 29 :3 d :2 13
18 24 68 6b :2 24 :2 d 5d :3 a
:2 15 2b :2 34 46 :2 a 7 38 e
17 22 :2 e 25 27 :2 25 d 2f
38 44 :2 2f 47 49 :2 47 2e :3 d
:2 17 29 :2 33 3f 43 57 5a 5b
:2 57 :2 29 :3 d :2 13 18 24 68 6b
:2 24 78 7b :2 24 7f 82 :2 24 8c
8f 98 a0 :2 a9 :2 98 c5 :2 8f :2 24
cd d0 :2 24 d8 db :2 24 e0 e3
:2 24 e8 eb :2 24 :2 d 5e :2 a d
:2 17 29 :2 33 3f 42 56 59 5a
:2 56 :2 29 :3 d :2 13 18 24 68 6b
:2 24 :2 d 5d :3 a :2 15 29 :2 32 4d
53 :2 a 4c 38 :2 7 11 1a 34
47 55 62 :2 a :2 13 1f a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 5a 5d 5e :2 5a :2 2c :3 10
:2 16 1b 27 6b 6e :2 27 7b 7e
:2 27 82 85 :2 27 91 94 :2 27 9c
9f :2 27 ab ae b5 be c1 :2 ae
:2 27 :2 10 61 :2 d :2 a :4 16 :2 a 7
:3 35 7 f e 11 :2 f 30 33
:2 f 13 16 :2 f 7 35 7 17
7 a 13 1e :2 a 21 23 :2 21
14 d 29 33 3c 4d 56 66
d 1f 28 37 d 1f 28 38
d 29 43 57 d 1f :6 d 1f
d 1f d 13 20 31 3d 46
4e 57 65 6e 7c 85 93 9c
a9 b2 c3 cc df e8 fd 106
121 12a 142 14b 15c 165 177 180
198 1a1 1b4 1bd 1de 1f3 207 d
14 10 1c :3 1a 10 :2 19 25 10
21 10 :2 19 25 10 :4 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 5d 60 61 :2 5d :2 2f :3 13 :2 19
1e 2a 6e 71 :2 2a 7e 81 :2 2a
85 88 :2 2a 94 97 :2 2a 9f a2
:2 2a ae b1 b8 c1 c4 :2 b1 :2 2a
:2 13 64 :2 10 :2 d :4 19 :2 d a :3 25
a f :4 a 13 1d 32 :3 d 10
15 :2 10 :2 19 25 10 1b 10 :2 19
25 10 :4 d 11 :2 1a 26 23 :2 2f
:2 23 :3 10 3f :2 d 17 :2 10 1e 27
36 10 1f 28 38 10 2b 45
5b 10 1d 10 16 10 1f 28
37 10 20 29 39 10 1b 10
2d 4c 64 10 30 52 6d 10
30 4e 52 68 78 10 33 51
55 6b 7b 10 26 3d 53 10
27 3f 16 2a 10 17 13 1f
:3 1d 13 :2 1c 28 13 24 13 :2 1c
28 13 :4 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 60 63
64 :2 60 :2 32 :3 16 :2 1c 21 2d 71
74 :2 2d 81 84 :2 2d 88 8b :2 2d
97 9a :2 2d a2 a5 :2 2d b1 b4
bb c4 c7 :2 b4 :2 2d :2 16 67 :2 13
:2 10 :4 1c :2 10 d :3 a 17 2c 16
23 34 48 10 17 13 1f :3 1d
13 :2 1c 28 13 24 13 :2 1c 28
13 :4 10 d 15 13 :2 1c 28 13
23 :2 10 :2 15 13 :2 1c 28 13 16
:2 20 32 :2 3c 48 4c 60 63 64
:2 60 :2 32 :3 16 :2 1c 21 2d 71 74
:2 2d 81 84 :2 2d 88 8b :2 2d 97
9a :2 2d a2 a5 :2 2d b1 b4 bb
c4 c7 :2 b4 :2 2d :2 16 67 :2 13 :2 10
:4 1c :2 10 d :4 a e 25 d 13
:3 d :2 16 22 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 5d
60 61 :2 5d :2 2f :3 13 :2 19 1e 2a
6e 71 :2 2a 7e 81 :2 2a 85 88
:2 2a 94 97 :2 2a 9f a2 :2 2a ae
b1 b8 c1 c4 :2 b1 :2 2a :2 13 64
:2 10 :2 d :4 19 :2 d a :4 25 :2 7 11
1d 2b 34 10 1d 2e 43 57
a 11 d 19 :3 17 d :2 16 22
d 1e d :2 16 22 d :4 a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 5a 5d 5e :2 5a :2 2c :3 10
:2 16 1b 27 6b 6e :2 27 7b 7e
:2 27 82 85 :2 27 91 94 :2 27 9c
9f :2 27 ab ae b5 be c1 :2 ae
:2 27 :2 10 61 :2 d :2 a :4 16 :2 a 7
:3 4 7 14 :2 7 14 :2 7 d e
11 :2 d 23 26 :2 d 13 16 :2 d
7 :5 4 9 :2 12 :2 4 d 17 1d
27 32 :3 7 a f :2 a :2 13 1f
a 15 a :2 13 1f a :4 7 b
:2 14 20 1d :2 29 :2 1d :3 a 39 :2 7
c f 17 1f 10 12 28 2a
10 12 28 2a 10 12 28 2a
10 12 25 27 10 12 25 27
10 12 25 27 16 23 34 41
10 17 13 1f :3 1d 13 :2 1c 28
13 24 13 :2 1c 28 13 :4 10 d
15 13 :2 1c 28 13 23 :2 10 :2 15
13 :2 1c 28 13 16 :2 20 32 :2 3c
48 4c 60 63 64 :2 60 :2 32 :3 16
:2 1c 21 2d 71 74 :2 2d 81 84
:2 2d 88 8b :2 2d 97 9a :2 2d a2
a5 :2 2d b1 b4 bb c4 c7 :2 b4
:2 2d :2 16 67 :2 13 :2 10 :4 1c :2 10 d
:3 11 17 28 10 13 25 28 39
3c 4f 52 64 67 7c 7f 93
96 a7 aa 10 13 2a 2d 43
46 60 63 10 13 25 28 39
3c 4f 52 64 67 7c 7f 93
96 a7 aa 10 13 2a 2d 43
46 60 63 10 13 25 28 3a
3d 50 53 66 69 7e 81 96
99 aa ad ba bd 10 13 2a
2d 40 43 57 5a 74 77 8d
90 10 13 24 27 35 38 49
4c 5a 5d 6e 71 7f 82 93
96 16 2a 10 17 13 1f :3 1d
13 :2 1c 28 13 24 13 :2 1c 28
13 :4 10 d 15 13 :2 1c 28 13
23 :2 10 :2 15 13 :2 1c 28 13 16
:2 20 32 :2 3c 48 4c 60 63 64
:2 60 :2 32 :3 16 :2 1c 21 2d 71 74
:2 2d 81 84 :2 2d 88 8b :2 2d 97
9a :2 2d a2 a5 :2 2d b1 b4 bb
c4 c7 :2 b4 :2 2d :2 16 67 :2 13 :2 10
:4 1c :2 10 d :3 11 17 10 2c 10
2c 10 36 5d 78 10 2d 10
2d 10 37 5f 7b 10 2d 10
2d 10 37 57 73 10 2b 10
2b 10 35 56 70 10 2e 10
2e 10 38 5c 79 10 21 10
22 10 24 10 20 2d 10 26
39 10 29 3f 16 23 10 17
13 1f :3 1d 13 :2 1c 28 13 24
13 :2 1c 28 13 :4 10 d 15 13
:2 1c 28 13 23 :2 10 :2 15 13 :2 1c
28 13 16 :2 20 32 :2 3c 48 4c
60 63 64 :2 60 :2 32 :3 16 :2 1c 21
2d 71 74 :2 2d 81 84 :2 2d 88
8b :2 2d 97 9a :2 2d a2 a5 :2 2d
b1 b4 bb c4 c7 :2 b4 :2 2d :2 16
67 :2 13 :2 10 :4 1c :2 10 d :4 11 :2 a
f 17 29 38 4f 62 76 :2 10
:2 19 25 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 60 63
64 :2 60 :2 32 :3 16 :2 1c 21 2d 71
74 :2 2d 81 84 :2 2d 88 8b :2 2d
97 9a :2 2d a2 a5 :2 2d b1 b4
bb c4 c7 :2 b4 :2 2d :2 16 67 :2 13
:2 10 :4 1c :2 10 d :4 11 :2 1a 26 23
:2 2f :2 23 10 :4 44 43 42 :2 10 15
10 58 :2 d 17 34 4d 58 15
2a 15 2c 3f 49 5a 68 :2 10
:2 19 25 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 60 63
64 :2 60 :2 32 :3 16 :2 1c 21 2d 71
74 :2 2d 81 84 :2 2d 88 8b :2 2d
97 9a :2 2d a2 a5 :2 2d b1 b4
bb c4 c7 :2 b4 :2 2d :2 16 67 :2 13
:2 10 :4 1c :2 10 d :4 11 :2 1a 25 23
:2 2e :2 23 10 :4 44 43 42 :2 10 13
1c 27 :2 13 2a 2c :2 2a 13 1e
13 2e 22 39 4d 16 1d 19
25 :3 23 19 :2 22 2e 19 2a 19
:2 22 2e 19 :4 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 66
69 6a :2 66 :2 38 :3 1c :2 22 27 33
77 7a :2 33 87 8a :2 33 8e 91
:2 33 9d a0 :2 33 a8 ab :2 33 b7
ba c1 ca cd :2 ba :2 33 :2 1c 6d
:2 19 :2 16 :4 22 :2 16 13 :3 10 22 34
48 16 1d 19 25 :3 23 19 :2 22
2e 19 2a 19 :2 22 2e 19 :4 16
13 1b 19 :2 22 2e 19 29 :2 16
:2 1b 19 :2 22 2e 19 1c :2 26 38
:2 42 4e 52 66 69 6a :2 66 :2 38
:3 1c :2 22 27 33 77 7a :2 33 87
8a :2 33 8e 91 :2 33 9d a0 :2 33
a8 ab :2 33 b7 ba c1 ca cd
:2 ba :2 33 :2 1c 6d :2 19 :2 16 :4 22 :2 16
13 :3 10 22 3e 52 16 1d 19
25 :3 23 19 :2 22 2e 19 2a 19
:2 22 2e 19 :4 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 66
69 6a :2 66 :2 38 :3 1c :2 22 27 33
77 7a :2 33 87 8a :2 33 8e 91
:2 33 9d a0 :2 33 a8 ab :2 33 b7
ba c1 ca cd :2 ba :2 33 :2 1c 6d
:2 19 :2 16 :4 22 :2 16 13 :3 10 22 3f
53 16 1d 19 25 :3 23 19 :2 22
2e 19 2a 19 :2 22 2e 19 :4 16
13 1b 19 :2 22 2e 19 29 :2 16
:2 1b 19 :2 22 2e 19 1c :2 26 38
:2 42 4e 52 66 69 6a :2 66 :2 38
:3 1c :2 22 27 33 77 7a :2 33 87
8a :2 33 8e 91 :2 33 9d a0 :2 33
a8 ab :2 33 b7 ba c1 ca cd
:2 ba :2 33 :2 1c 6d :2 19 :2 16 :4 22 :2 16
13 :3 10 13 1e 13 :4 10 66 10
27 :2 10 1b 10 :5 d 1a :2 d 12
:4 d 16 1f 2c 37 :3 10 13 17
:2 13 :2 1c 28 13 1d 13 :2 1c 28
13 :4 10 14 :2 1d 29 26 :2 32 :2 26
:3 13 42 :2 10 13 1c 25 :2 13 28
2a :2 28 1d 32 3e 4a 16 22
2a 31 3c 1c 29 3a 4e 66
71 16 1d 19 25 :3 23 19 :2 22
2e 19 2a 19 :2 22 2e 19 :4 16
13 1b 19 :2 22 2e 19 29 :2 16
:2 1b 19 :2 22 2e 19 1c :2 26 38
:2 42 4e 52 66 69 6a :2 66 :2 38
:3 1c :2 22 27 33 77 7a :2 33 87
8a :2 33 8e 91 :2 33 9d a0 :2 33
a8 ab :2 33 b7 ba c1 ca cd
:2 ba :2 33 :2 1c 6d :2 19 :2 16 :4 22 :2 16
13 :3 2c 17 :2 20 2b 29 :2 34 :2 29
16 25 3c 49 5a 6e 77 82
19 20 1c 28 :3 26 1c :2 25 31
1c 2d 1c :2 25 31 1c :4 19 16
1e 1c :2 25 31 1c 2c :2 19 :2 1e
1c :2 25 31 1c 1f :2 29 3b :2 45
51 55 69 6c 6d :2 69 :2 3b :3 1f
:2 25 2a 36 7a 7d :2 36 8a 8d
:2 36 91 94 :2 36 a0 a3 :2 36 ab
ae :2 36 ba bd c4 cd d0 :2 bd
:2 36 :2 1f 70 :2 1c :2 19 :4 25 :2 19 16
:4 44 20 35 49 63 70 81 95
9e a9 19 20 1c 28 :3 26 1c
:2 25 31 1c 2d 1c :2 25 31 1c
:4 19 16 1e 1c :2 25 31 1c 2c
:2 19 :2 1e 1c :2 25 31 1c 1f :2 29
3b :2 45 51 55 69 6c 6d :2 69
:2 3b :3 1f :2 25 2a 36 7a 7d :2 36
8a 8d :2 36 91 94 :2 36 a0 a3
:2 36 ab ae :2 36 ba bd c4 cd
d0 :2 bd :2 36 :2 1f 70 :2 1c :2 19 :4 25
:2 19 16 :7 13 10 2c 16 1f 28
:2 16 2b 2d :2 2b 1d 32 46 60
6d 7e 92 9b a6 16 1d 19
25 :3 23 19 :2 22 2e 19 2a 19
:2 22 2e 19 :4 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 66
69 6a :2 66 :2 38 :3 1c :2 22 27 33
77 7a :2 33 87 8a :2 33 8e 91
:2 33 9d a0 :2 33 a8 ab :2 33 b7
ba c1 ca cd :2 ba :2 33 :2 1c 6d
:2 19 :2 16 :4 22 :2 16 13 :4 2f 2c :2 10
d :2 11 10 16 :3 10 :2 19 25 10
d 15 13 :2 1c 28 13 23 :2 10
:2 15 13 :2 1c 28 13 16 :2 20 32
:2 3c 48 4c 60 63 64 :2 60 :2 32
:3 16 :2 1c 21 2d 71 74 :2 2d 81
84 :2 2d 88 8b :2 2d 97 9a :2 2d
a2 a5 :2 2d b1 b4 bb c4 c7
:2 b4 :2 2d :2 16 67 :2 13 :2 10 :4 1c :2 10
d :3 11 17 2c 3c 48 16 23
34 40 10 17 13 1f :3 1d 13
:2 1c 28 13 24 13 :2 1c 28 13
:4 10 d 15 13 :2 1c 28 13 23
:2 10 :2 15 13 :2 1c 28 13 16 :2 20
32 :2 3c 48 4c 60 63 64 :2 60
:2 32 :3 16 :2 1c 21 2d 71 74 :2 2d
81 84 :2 2d 88 8b :2 2d 97 9a
:2 2d a2 a5 :2 2d b1 b4 bb c4
c7 :2 b4 :2 2d :2 16 67 :2 13 :2 10 :4 1c
:2 10 d :3 11 10 19 24 :2 10 27
29 :2 27 1a :2 13 18 21 28 2d
36 13 21 13 22 13 24 13
20 13 22 13 23 13 1e 13
30 4f 67 13 33 55 70 13
19 13 2b 2f 45 55 13 36
54 58 6e 7e 13 29 40 56
13 2a 42 13 19 2d 36 49
13 1a 16 22 :3 20 16 :2 1f 2b
16 27 16 :2 1f 2b 16 :4 13 10
18 16 :2 1f 2b 16 26 :2 13 :2 18
16 :2 1f 2b 16 19 :2 23 35 :2 3f
4b 4f 63 66 67 :2 63 :2 35 :3 19
:2 1f 24 30 74 77 :2 30 84 87
:2 30 8b 8e :2 30 9a 9d :2 30 a5
a8 :2 30 b4 b7 be c7 ca :2 b7
:2 30 :2 19 6a :2 16 :2 13 :4 1f :2 13 10
:3 2b 13 1c 25 :2 13 28 2a :2 28
13 21 :2 13 26 36 44 59 65
68 :2 26 13 2c :2 10 2b 10 15
:4 10 19 22 2d :3 13 16 1a :2 16
:2 1f 2b 16 20 16 :2 1f 2b 16
:4 13 17 :2 20 2c 29 :2 35 :2 29 :3 16
45 :2 13 1d :2 16 1b 24 16 1b
24 16 24 16 25 16 27 16
23 16 25 16 26 16 21 16
33 52 6a 16 36 58 73 16
1c 16 2e 32 48 58 16 39
57 5b 71 81 16 2c 43 59
16 2d 45 16 2e 1c 30 16
1d 19 25 :3 23 19 :2 22 2e 19
2a 19 :2 22 2e 19 :4 16 13 1b
19 :2 22 2e 19 29 :2 16 :2 1b 19
:2 22 2e 19 1c :2 26 38 :2 42 4e
52 66 69 6a :2 66 :2 38 :3 1c :2 22
27 33 77 7a :2 33 87 8a :2 33
8e 91 :2 33 9d a0 :2 33 a8 ab
:2 33 b7 ba c1 ca cd :2 ba :2 33
:2 1c 6d :2 19 :2 16 :4 22 :2 16 13 :3 10
1d 29 37 40 58 65 16 1d
19 25 :3 23 19 :2 22 2e 19 2a
19 :2 22 2e 19 :4 16 13 1b 19
:2 22 2e 19 29 :2 16 :2 1b 19 :2 22
2e 19 1c :2 26 38 :2 42 4e 52
66 69 6a :2 66 :2 38 :3 1c :2 22 27
33 77 7a :2 33 87 8a :2 33 8e
91 :2 33 9d a0 :2 33 a8 ab :2 33
b7 ba c1 ca cd :2 ba :2 33 :2 1c
6d :2 19 :2 16 :4 22 :2 16 13 :4 10 14
d 13 19 :3 13 :2 1c 28 13 10
18 16 :2 1f 2b 16 26 :2 13 :2 18
16 :2 1f 2b 16 19 :2 23 35 :2 3f
4b 4f 63 66 67 :2 63 :2 35 :3 19
:2 1f 24 30 74 77 :2 30 84 87
:2 30 8b 8e :2 30 9a 9d :2 30 a5
a8 :2 30 b4 b7 be c7 ca :2 b7
:2 30 :2 19 6a :2 16 :2 13 :4 1f :2 13 10
:3 d 14 1d 28 :2 14 2b 2d :2 2b
13 35 3e 46 :2 35 49 4b :2 49
34 :2 13 1d 32 3d 4a 5e 16
1d 19 25 :3 23 19 :2 22 2e 19
2a 19 :2 22 2e 19 :4 16 13 1b
19 :2 22 2e 19 29 :2 16 :2 1b 19
:2 22 2e 19 1c :2 26 38 :2 42 4e
52 66 69 6a :2 66 :2 38 :3 1c :2 22
27 33 77 7a :2 33 87 8a :2 33
8e 91 :2 33 9d a0 :2 33 a8 ab
:2 33 b7 ba c1 ca cd :2 ba :2 33
:2 1c 6d :2 19 :2 16 :4 22 :2 16 13 :4 4e
:3 10 1e :2 10 23 33 41 56 62
65 :2 23 10 1a 31 37 44 52
5c 6a 18 2d 3b 49 58 64
70 18 2f 45 :2 13 :2 1c 28 13
10 18 16 :2 1f 2b 16 26 :2 13
:2 18 16 :2 1f 2b 16 19 :2 23 35
:2 3f 4b 4f 63 66 67 :2 63 :2 35
:3 19 :2 1f 24 30 74 77 :2 30 84
87 :2 30 8b 8e :2 30 9a 9d :2 30
a5 a8 :2 30 b4 b7 be c7 ca
:2 b7 :2 30 :2 19 6a :2 16 :2 13 :4 1f :2 13
10 :3 d 13 1c 28 :2 13 2b 2d
:2 2b 1d :6 16 2e 16 2f 16 2f
16 2e 45 16 1c 29 3a 48
16 1d 19 25 :3 23 19 :2 22 2e
19 2a 19 :2 22 2e 19 :4 16 13
1b 19 :2 22 2e 19 29 :2 16 :2 1b
19 :2 22 2e 19 1c :2 26 38 :2 42
4e 52 66 69 6a :2 66 :2 38 :3 1c
:2 22 27 33 77 7a :2 33 87 8a
:2 33 8e 91 :2 33 9d a0 :2 33 a8
ab :2 33 b7 ba c1 ca cd :2 ba
:2 33 :2 1c 6d :2 19 :2 16 :4 22 :2 16 13
:4 2f 1d :6 16 2e 16 2f 16 2f
16 2e 45 16 31 48 1c 29
3a 48 16 1d 19 25 :3 23 19
:2 22 2e 19 2a 19 :2 22 2e 19
:4 16 13 1b 19 :2 22 2e 19 29
:2 16 :2 1b 19 :2 22 2e 19 1c :2 26
38 :2 42 4e 52 66 69 6a :2 66
:2 38 :3 1c :2 22 27 33 77 7a :2 33
87 8a :2 33 8e 91 :2 33 9d a0
:2 33 a8 ab :2 33 b7 ba c1 ca
cd :2 ba :2 33 :2 1c 6d :2 19 :2 16 :4 22
:2 16 13 :7 10 :4 d f d :3 11 :2 a
d a :3 7 11 1d 2b 34 4c
59 a 11 d 19 :3 17 d :2 16
22 d 1e d :2 16 22 d :4 a
7 f d :2 16 22 d 1d :2 a
:2 f d :2 16 22 d 10 :2 1a 2c
:2 36 42 46 5a 5d 5e :2 5a :2 2c
:3 10 :2 16 1b 27 6b 6e :2 27 7b
7e :2 27 82 85 :2 27 91 94 :2 27
9c 9f :2 27 ab ae b5 be c1
:2 ae :2 27 :2 10 61 :2 d :2 a :4 16 :2 a
7 :4 4 8 1 7 d :3 7 :2 10
1c 7 4 c a :2 13 1f a
1a :2 7 :2 c a :2 13 1f a d
:2 17 29 :2 33 3f 43 57 5a 5b
:2 57 :2 29 :3 d :2 13 18 24 68 6b
:2 24 78 7b :2 24 7f 82 :2 24 8e
91 :2 24 99 9c :2 24 a8 ab b2
bb be :2 ab :2 24 :2 d 5e :2 a :2 7
:4 13 :2 7 4 :3 1 4 17 21 :2 17
4 19 1f 28 34 3a 43 4f
59 62 :2 c 1a 27 :2 7 :2 10 1c
7 4 c a :2 13 1f a 1a
:2 7 :2 c a :2 13 1f a d :2 17
29 :2 33 3f 43 57 5a 5b :2 57
:2 29 :3 d :2 13 18 24 68 6b :2 24
78 7b :2 24 7f 82 :2 24 8e 91
:2 24 99 9c :2 24 a8 ab b2 bb
be :2 ab :2 24 :2 d 5e :2 a :2 7 :4 13
:2 7 4 :3 1 7 10 1f :2 7 22
24 :2 22 7 14 :2 1d :2 7 :2 10 1c
30 3e 4d :2 1c 7 26 :2 4 e
7 1f 2c 3a 47 d 1a 7
e a 16 :3 14 a :2 13 1f a
1b a :2 13 1f a :4 7 4 c
a :2 13 1f a 1a :2 7 :2 c a
:2 13 1f a d :2 17 29 :2 33 3f
43 57 5b 5c :2 57 :2 29 :3 d :2 13
18 24 69 6c :2 24 79 7c :2 24
80 83 :2 24 8f 92 :2 24 9a 9d
:2 24 a9 ac b3 bc bf :2 ac :2 24
:2 d 5f :2 a :2 7 :4 13 :2 7 4 :3 1
7 :2 10 19 1b :2 19 7 :2 10 1c
7 1f :2 4 7 :2 11 23 :2 2d 39
3c 50 54 55 :2 50 :2 23 :3 7 :2 d
12 1e 63 66 :2 1e 73 76 :2 1e
7a 7d :2 1e 87 8a 93 9b :2 a4
:2 93 af :2 8a :2 1e b7 ba :2 1e c2
c5 :2 1e ca cd :2 1e d0 d3 :2 1e
:2 7 58 :3 4 :2 f 21 :2 2a 34 38
:2 4 1 9 12 9 a :2 14 26
:2 30 3c 40 54 58 59 :2 54 :2 26
:3 a :2 10 15 21 66 69 :2 21 :3 a
:2 10 15 39 5c 5f :2 68 :2 39 76
79 :2 39 81 84 :2 8d :2 39 :2 a 5c
:3 7 c 15 1e 2c a 13 1e
f e 7 e 7 e 7 29
:2 4 :2 9 7 :2 10 1c :2 7 :2 11 23
:2 2d 39 3d 51 55 56 :2 51 :2 23
:3 7 :2 d 12 1e 63 66 :2 1e :3 7
:2 d 12 36 42 45 :2 36 4d 50
:2 36 5c 5f 66 6f 72 :2 5f :2 36
:2 7 59 :2 4 7 c 1b a 13
1e f e 7 e 7 e 7
:2 4 :4 10 :2 4 :8 1 
15eb
4
0 :2 1 :4 3 :4 4
2 :4 5 :2 1 :a 7
:a 8 :7 9 :5 a :7 b
:7 c :7 d :7 e :7 f
:5 10 :5 11 :6 12 :6 13
:6 14 :6 15 :6 16 :6 17
:6 18 :6 19 :6 1a :6 1b
:6 1c :6 1d :6 1e :6 1f
:6 20 :7 21 :2 23 0
:2 23 :4 24 :5 25 :8 26
:5 27 :4 28 24 :5 23
:7 2b :2 2d 0 :2 2d
:4 2e :5 2f 2e :5 2d
:2 31 0 :2 31 :2 32
33 :3 34 32 :5 31
:7 36 :b 38 :11 39 3a
:2 39 :2 3a :2 39 :2 3a
:2 39 :2 3b :2 39 :9 3b
:2 39 :2 3b :4 39 :3 38
:11 3e :b 3f :3 3e :3 45
:3 46 :7 49 :5 4a 48
4c :5 4d :3 4c :2 4e
:5 50 :11 51 :24 52 :3 51
:2 4f :6 4e 4b :3 37
:a 58 :4 5a :3 5b :5 5c
5a :5 5d 59 5f
:5 60 :3 5f :2 61 :5 63
:11 64 :24 65 :3 64 :2 62
:6 61 5e :3 58 :a 6a
:11 6b :20 6c :3 6b :11 6e
:b 6f :3 6e :9 71 72
6a :16 72 :11 73 :30 74
:3 73 :11 76 :b 77 :3 76
:a 79 72 :3 6a :7 7d
:5 7e 7c 80 :5 81
:3 80 :2 82 :5 84 :11 85
:24 86 :3 85 :2 83 :6 82
7f :3 58 :2 8b :2 8c
:2 8b :2 8c :2 8b :2 8d
:3 8b 58 :3 8f :9 92
95 :7 96 :4 97 :4 98
:4 99 :2 9a 9b 9c
9d 9e 9f :2 a0
:2 a1 a2 :24 a3 95
:6 a4 :5 a5 a4 :5 a7
a6 :3 a4 94 aa
:5 ab :3 aa :2 ac :5 ae
:11 af :24 b0 :3 af :2 ad
:6 ac a9 :3 92 :5 b5
b7 :6 b8 :3 b9 :5 ba
b9 :5 bc bb :3 b9
:a be :2 bf :3 be c3
c4 :4 c5 :4 c6 :4 c7
:2 c8 :2 c9 :4 ca :4 cb
:2 cc :4 cd :4 ce :6 cf
:6 d0 :4 d1 :3 d2 :2 d3
c3 :6 d4 :5 d5 d4
:5 d7 d6 :3 d4 c2
da :5 db :3 da :2 dc
:5 de :11 df :24 e0 :3 df
:2 dd :6 dc d9 :3 b7
:2 e6 :4 e7 e6 :6 e8
:5 e9 e8 :5 eb ea
:3 e8 e5 ee :5 ef
:3 ee :2 f0 :5 f2 :11 f3
:24 f4 :3 f3 :2 f1 :6 f0
ed :4 b7 f8 92
:4 fc :5 fd fb ff
:5 100 :3 ff :2 101 :5 103
:11 104 :24 105 :3 104 :2 102
:6 101 fe :6 92 :4 10c
:5 10d 10c :6 10e :5 10f
10e :5 111 110 :3 10e
10b 114 :5 115 :3 114
:2 116 :5 118 :11 119 :24 11a
:3 119 :2 117 :6 116 113
:3 8e :3 11f :3 120 122
123 :2 124 :2 123 :2 124
:2 123 :2 125 :2 123 122
8e :3 58 :5 12c 12e
:8 12f :3 130 :5 131 130
:5 133 132 :3 130 :a 136
:2 137 :3 136 13b 13d
:2 142 :4 143 :4 144 :4 145
:4 146 :4 147 :4 148 :4 149
142 :6 14a :5 14b 14a
:5 14d 14c :3 14a 141
150 :5 151 :3 150 :2 152
:5 154 :11 155 :24 156 :3 155
:2 153 :6 152 14f :3 13d
:2 15d :10 15e :8 15f :10 160
:8 161 :12 162 :c 163 :10 164
:2 165 15d :6 166 :5 167
166 :5 169 168 :3 166
15c 16c :5 16d :3 16c
:2 16e :5 170 :11 171 :24 172
:3 171 :2 16f :6 16e 16b
:3 13d 178 :2 179 :2 17a
:4 17b :2 17c :2 17d :4 17e
:2 17f :2 180 :4 181 :2 182
:2 183 :4 184 :2 185 :2 186
:4 187 :2 188 :2 189 :2 18a
:3 18b :3 18c :3 18d :2 18e
178 :6 18f :5 190 18f
:5 192 191 :3 18f 177
195 :5 196 :3 195 :2 197
:5 199 :11 19a :24 19b :3 19a
:2 198 :6 197 194 :6 13d
1a2 :7 1a7 :5 1a8 1a6
1aa :5 1ab :3 1aa :2 1ac
:5 1ae :11 1af :24 1b0 :3 1af
:2 1ad :6 1ac 1a9 :3 1a2
:12 1b5 :2 1b6 :3 1b5 :4 1ba
:2 1bb :6 1bc 1ba :5 1bd
1b9 1bf :5 1c0 :3 1bf
:2 1c1 :5 1c3 :11 1c4 :24 1c5
:3 1c4 :2 1c2 :6 1c1 1be
:3 1a2 :12 1cb :9 1cc :3 1cd
1cc :4 1d1 :6 1d2 :5 1d3
1d2 :5 1d5 1d4 :3 1d2
1d0 1d8 :5 1d9 :3 1d8
:2 1da :5 1dc :11 1dd :24 1de
:3 1dd :2 1db :6 1da 1d7
:3 1ce :4 1e4 :6 1e5 :5 1e6
1e5 :5 1e8 1e7 :3 1e5
1e3 1eb :5 1ec :3 1eb
:2 1ed :5 1ef :11 1f0 :24 1f1
:3 1f0 :2 1ee :6 1ed 1ea
:3 1ce :4 1f7 :6 1f8 :5 1f9
1f8 :5 1fb 1fa :3 1f8
1f6 1fe :5 1ff :3 1fe
:2 200 :5 202 :11 203 :24 204
:3 203 :2 201 :6 200 1fd
:3 1ce :4 20a :6 20b :5 20c
20b :5 20e 20d :3 20b
209 211 :5 212 :3 211
:2 213 :5 215 :11 216 :24 217
:3 216 :2 214 :6 213 210
:3 1ce :3 21c 1ce :3 1cc
1cb :3 21f :3 220 21e
:3 1cb :3 223 :5 228 22a
:7 22b :3 22c :5 22d 22c
:5 22f 22e :3 22c :a 232
:2 233 :3 232 :9 237 :4 239
:5 23a :6 23b 239 :6 23c
:5 23d 23c :5 23f 23e
:3 23c 238 242 :5 243
:3 242 :2 244 :5 246 :11 247
:24 248 :3 247 :2 245 :6 244
241 :3 237 :a 24e :8 250
:6 251 :5 252 251 :5 254
253 :3 251 24f 257
:5 258 :3 257 :2 259 :5 25b
:11 25c :24 25d :3 25c :2 25a
:6 259 256 :4 24e :a 263
:6 264 :5 265 264 :5 267
266 :3 264 262 26a
:5 26b :3 26a :2 26c :5 26e
:11 26f :24 270 :3 26f :2 26d
:6 26c 269 :4 261 :3 24e
275 237 :9 275 :a 277
:6 278 :5 279 278 :5 27b
27a :3 278 276 27e
:5 27f :3 27e :2 280 :5 282
:11 283 :24 284 :3 283 :2 281
:6 280 27d :4 275 :3 237
22a 289 1a2 :4 28b
:5 28c 28a 28e :5 28f
:3 28e :2 290 :5 292 :11 293
:24 294 :3 293 :2 291 :6 290
28d :3 1a2 :4 29a :4 29b
29a :6 29c :5 29d 29c
:5 29f 29e :3 29c 299
2a2 :5 2a3 :3 2a2 :2 2a4
:5 2a6 :11 2a7 :24 2a8 :3 2a7
:2 2a5 :6 2a4 2a1 :3 1a2
:9 2ad 2af 2b0 :6 2b1
:2 2b2 :2 2b3 :2 2b4 :2 2b5
:2 2b6 :2 2b7 :2 2b8 :4 2b9
:4 2ba :2 2bb :5 2bc :6 2bd
:4 2be :3 2bf 2c0 :4 2c1
2af :6 2c2 :5 2c3 2c2
:5 2c5 2c4 :3 2c2 2ae
2c8 :5 2c9 :3 2c8 :2 2ca
:5 2cc :11 2cd :24 2ce :3 2cd
:2 2cb :6 2ca 2c7 :3 2ad
:9 2d3 :3 2d5 :a 2d6 :3 2d3
2ad :5 2db 2dd :6 2de
:3 2df :5 2e0 2df :5 2e2
2e1 :3 2df :a 2e4 :2 2e5
:3 2e4 2e9 2ea :3 2eb
:3 2ec :2 2ed :2 2ee :2 2ef
:2 2f0 :2 2f1 :2 2f2 :2 2f3
:4 2f4 :4 2f5 :2 2f6 :5 2f7
:6 2f8 :4 2f9 :3 2fa :2 2fb
:2 2fc 2e9 :6 2fd :5 2fe
2fd :5 300 2ff :3 2fd
2e8 303 :5 304 :3 303
:2 305 :5 307 :11 308 :24 309
:3 308 :2 306 :6 305 302
:3 2dd :7 311 :6 312 :5 313
312 :5 315 314 :3 312
310 318 :5 319 :3 318
:2 31a :5 31c :11 31d :24 31e
:3 31d :2 31b :6 31a 317
:4 2dd 322 2d8 :4 325
:5 326 324 328 :5 329
:3 328 :2 32a :5 32c :11 32d
:24 32e :3 32d :2 32b :6 32a
327 :3 2d8 :16 333 :6 335
:6 336 :5 337 336 :5 339
338 :3 336 334 33c
:5 33d :3 33c :2 33e :5 340
:11 341 :24 342 :3 341 :2 33f
:6 33e 33b :6 333 :3 349
:a 34a :7 34d :7 34e :3 34f
34d :5 350 34c 352
:5 353 :3 352 :2 354 :5 356
:11 357 :24 358 :3 357 :2 355
:6 354 351 :3 2d8 :9 35e
360 361 362 363
364 365 :2 366 :2 367
:2 368 :3 369 36a :4 36b
360 :6 36c :5 36d 36c
:5 36f 36e :3 36c 35f
372 :5 373 :3 372 :2 374
:5 376 :11 377 :24 378 :3 377
:2 375 :6 374 371 :4 35e
37e 37f 380 381
382 383 :2 384 :2 385
:2 386 :3 387 :3 388 :4 389
37e :6 38a :5 38b 38a
:5 38d 38c :3 38a 37d
390 :5 391 :3 390 :2 392
:5 394 :11 395 :24 396 :3 395
:2 393 :6 392 38f :4 37c
:3 35e 2d8 :3 2ad 39e
39f :5 1a2 3a1 3a0
:3 13b :7 3a6 :6 3a7 :5 3a8
3a7 :5 3aa 3a9 :3 3a7
3a5 3ad :5 3ae :3 3ad
:2 3af :5 3b1 :11 3b2 :24 3b3
:3 3b2 :2 3b0 :6 3af 3ac
:4 12e 3b8 37 :4 3bc
:5 3bd 3bb 3bf :5 3c0
:3 3bf :2 3c1 :5 3c3 :11 3c4
:24 3c5 :3 3c4 :2 3c2 :6 3c1
3be :3 37 :6 3ca :9 3ce
3cf :3 3d0 3ce :5 3d1
3cd 3d3 :5 3d4 :3 3d3
:2 3d5 :5 3d7 :11 3d8 :24 3d9
:3 3d8 :2 3d6 :6 3d5 3d2
:3 37 :9 3de :5 3df :a 3e0
:3 3de 3e4 :5 3e5 :2 3e6
3e4 :6 3e7 :5 3e8 3e7
:5 3ea 3e9 :3 3e7 3e3
3ed :5 3ee :3 3ed :2 3ef
:5 3f1 :11 3f2 :24 3f3 :3 3f2
:2 3f0 :6 3ef 3ec :3 37
:7 3f8 :5 3f9 :3 3f8 :11 3fc
:30 3fd :3 3fc :a 3ff 37
:3 402 :11 403 :b 404 :17 405
:3 403 :2 407 :3 408 :3 409
40a 408 407 408
:3 40b :3 402 :2 40c :5 40e
:11 40f :b 410 :18 411 :3 40f
:2 413 414 :3 415 416
414 413 414 :3 417
:2 40d :6 40c 401 :7 1

4ea8
4
:3 0 1 :3 0 2
:a 0 15e6 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 d
e 0 7 6
:3 0 5 :7 0 9
8 :3 0 7 :3 0
8 :3 0 9 :2 0
4 b f 0
15e6 2 10 :2 0
c :2 0 c b
:3 0 c :2 0 a
13 15 2 :5 0
d :3 0 e :3 0
17 18 0 1b
16 19 15e4 0
a :6 0 11 :2 0
10 b :3 0 e
1d 1f 2 :5 0
d :3 0 e :3 0
21 22 0 25
20 23 15e4 0
f :6 0 16 c9
0 14 4 :3 0
12 27 29 :6 0
2c 2a 0 15e4
0 10 :6 0 11
:2 0 1a 13 :3 0
2e :7 0 31 2f
0 15e4 0 12
:6 0 4 :3 0 11
:2 0 18 33 35
:6 0 38 36 0
15e4 0 14 :6 0
11 :2 0 1e 4
:3 0 1c 3a 3c
:6 0 3f 3d 0
15e4 0 15 :6 0
11 :2 0 22 4
:3 0 20 41 43
:6 0 46 44 0
15e4 0 16 :6 0
11 :2 0 26 4
:3 0 24 48 4a
:6 0 4d 4b 0
15e4 0 17 :6 0
2c 175 0 2a
4 :3 0 28 4f
51 :6 0 54 52
0 15e4 0 18
:6 0 1d :2 0 2e
1a :3 0 56 :7 0
59 57 0 15e4
0 19 :6 0 1a
:3 0 5b :7 0 5e
5c 0 15e4 0
1b :6 0 1d :2 0
30 6 :3 0 60
:7 0 64 61 62
15e4 0 1c :6 0
1d :2 0 32 6
:3 0 66 :7 0 6a
67 68 15e4 0
1e :6 0 1d :2 0
34 6 :3 0 6c
:7 0 70 6d 6e
15e4 0 1f :6 0
1d :2 0 36 6
:3 0 72 :7 0 76
73 74 15e4 0
20 :6 0 1d :2 0
38 6 :3 0 78
:7 0 7c 79 7a
15e4 0 21 :6 0
1d :2 0 3a 6
:3 0 7e :7 0 82
7f 80 15e4 0
22 :6 0 1d :2 0
3c 6 :3 0 84
:7 0 88 85 86
15e4 0 23 :6 0
1d :2 0 3e 6
:3 0 8a :7 0 8e
8b 8c 15e4 0
24 :6 0 1d :2 0
40 6 :3 0 90
:7 0 94 91 92
15e4 0 25 :6 0
1d :2 0 42 6
:3 0 96 :7 0 9a
97 98 15e4 0
26 :6 0 1d :2 0
44 6 :3 0 9c
:7 0 a0 9d 9e
15e4 0 27 :6 0
1d :2 0 46 6
:3 0 a2 :7 0 a6
a3 a4 15e4 0
28 :6 0 2c :2 0
48 6 :3 0 a8
:7 0 ac a9 aa
15e4 0 29 :6 0
2c :2 0 4a 2b
:3 0 ae :7 0 b2
af b0 15e4 0
2a :6 0 2f :2 0
4c 2b :3 0 b4
:7 0 b8 b5 b6
15e4 0 2d :9 0
50 4 :3 0 4e
ba bc :6 0 bf
bd 0 15e4 0
2e :6 0 30 :3 0
31 :a 0 2 e0
:4 0 c1 c4 0
c2 :3 0 32 :3 0
33 :3 0 32 :3 0
34 :3 0 35 :3 0
32 :3 0 36 :3 0
37 :3 0 38 :3 0
38 :3 0 39 :3 0
32 :3 0 39 :3 0
38 :3 0 3a :3 0
32 :3 0 3b :3 0
38 :3 0 39 :3 0
10 :3 0 32 :3 0
3c :3 0 32 :3 0
3d :3 0 32 :3 0
3e :4 0 3f 1
:8 0 e1 c1 c4
e2 0 15e4 52
e2 e4 e1 e3
:6 0 e0 :7 0 e2
:3 0 54 8 :3 0
9 :2 0 4 e6
e7 0 e8 :7 0
eb e9 0 15e4
0 40 :6 0 30
:3 0 41 :a 0 3
fb :4 0 ed f0
0 ee :3 0 42
:3 0 43 :3 0 44
:3 0 45 :3 0 39
:3 0 10 :3 0 33
:3 0 16 :3 0 3c
:4 0 46 1 :8 0
fc ed f0 fd
0 15e4 56 fd
ff fc fe :6 0
fb :7 0 fd 30
:3 0 47 :a 0 4
10c :5 0 101 104
0 102 :3 0 3b
:3 0 33 :3 0 35
:3 0 3c :3 0 3e
:3 0 16 :4 0 48
1 :8 0 10d 101
104 10e 0 15e4
58 10e 110 10d
10f :6 0 10c :7 0
10e 118 119 0
5a 8 :3 0 9
:2 0 4 112 113
0 114 :7 0 117
115 0 15e4 0
49 :6 0 4a :3 0
4b :3 0 4a :3 0
4c :3 0 11b 11c
0 2 :4 0 5c
11d 11f 11a 121
120 :2 0 4d :3 0
4e :3 0 123 124
0 4f :4 0 4a
:3 0 50 :3 0 127
128 0 2 :4 0
5e 129 12b 51
:4 0 52 :2 0 53
:3 0 60 12e 130
:3 0 54 :4 0 52
:2 0 55 :4 0 63
133 135 :3 0 52
:2 0 3 :3 0 66
137 139 :3 0 52
:2 0 55 :4 0 69
13b 13d :3 0 52
:2 0 56 :4 0 6c
13f 141 :3 0 52
:2 0 57 :3 0 58
:3 0 5 :3 0 6f
145 147 59 :4 0
71 144 14a 74
143 14c :3 0 52
:2 0 5a :4 0 77
14e 150 :3 0 7a
125 152 :2 0 154
7f 155 122 154
0 156 81 0
1544 4a :3 0 4b
:3 0 157 158 0
4a :3 0 5b :3 0
15a 15b 0 5c
:2 0 2 :4 0 5d
:2 0 5e :2 0 5c
:2 0 83 160 162
:3 0 86 15c 164
159 166 165 :2 0
4d :3 0 5f :3 0
168 169 0 4f
:4 0 60 :4 0 52
:2 0 53 :3 0 8a
16d 16f :3 0 8d
16a 171 :2 0 173
90 174 167 173
0 175 92 0
1544 21 :3 0 5
:3 0 176 177 0
1544 16 :3 0 3
:3 0 179 17a 0
1544 57 :3 0 61
:3 0 22 :3 0 37
:3 0 3a :3 0 16
:4 0 62 1 :8 0
188 8 :3 0 63
:3 0 183 184 0
64 :3 0 185 186
0 188 94 1db
65 :3 0 8 :3 0
63 :3 0 18a 18b
0 64 :3 0 18c
18d 0 18f 97
191 99 190 18f
:2 0 1d9 66 :3 0
8 :3 0 63 :3 0
194 195 0 64
:3 0 196 197 0
1d1 4a :3 0 4b
:3 0 199 19a 0
4a :3 0 5b :3 0
19c 19d 0 67
:2 0 2 :4 0 68
:2 0 5e :2 0 5c
:2 0 9b 1a2 1a4
:3 0 9e 19e 1a6
19b 1a8 1a7 :2 0
4d :3 0 5f :3 0
1aa 1ab 0 4f
:4 0 69 :4 0 52
:2 0 53 :3 0 a2
1af 1b1 :3 0 52
:2 0 6a :4 0 a5
1b3 1b5 :3 0 52
:2 0 6b :4 0 a8
1b7 1b9 :3 0 52
:2 0 64 :3 0 ab
1bb 1bd :3 0 52
:2 0 6c :4 0 ae
1bf 1c1 :3 0 52
:2 0 6d :3 0 6e
:3 0 5c :2 0 6f
:2 0 b1 1c4 1c8
b5 1c3 1ca :3 0
b8 1ac 1cc :2 0
1ce bb 1cf 1a9
1ce 0 1d0 bd
0 1d1 bf 1d4
:3 0 1d4 0 1d4
1d3 1d1 1d2 :6 0
1d6 5 :3 0 c2
1d8 c4 1d7 1d6
:2 0 1d9 c6 :2 0
1db 0 1db 1da
188 1d9 :6 0 1544
1 :3 0 8 :3 0
63 :3 0 1dd 1de
0 8 :3 0 70
:2 0 71 :3 0 1e0
1e2 0 cb 1e1
1e4 :3 0 1e5 :2 0
39 :3 0 39 :3 0
3b :3 0 3e :3 0
10 :3 0 26 :3 0
25 :3 0 35 :3 0
72 :3 0 3d :3 0
33 :3 0 16 :4 0
73 1 :8 0 1f9
8 :3 0 63 :3 0
1f4 1f5 0 64
:3 0 1f6 1f7 0
1f9 ce 24c 65
:3 0 8 :3 0 63
:3 0 1fb 1fc 0
64 :3 0 1fd 1fe
0 200 d1 202
d3 201 200 :2 0
24a 66 :3 0 8
:3 0 63 :3 0 205
206 0 64 :3 0
207 208 0 242
4a :3 0 4b :3 0
20a 20b 0 4a
:3 0 5b :3 0 20d
20e 0 67 :2 0
2 :4 0 74 :2 0
5e :2 0 5c :2 0
d5 213 215 :3 0
d8 20f 217 20c
219 218 :2 0 4d
:3 0 5f :3 0 21b
21c 0 4f :4 0
75 :4 0 52 :2 0
53 :3 0 dc 220
222 :3 0 52 :2 0
6a :4 0 df 224
226 :3 0 52 :2 0
6b :4 0 e2 228
22a :3 0 52 :2 0
64 :3 0 e5 22c
22e :3 0 52 :2 0
6c :4 0 e8 230
232 :3 0 52 :2 0
6d :3 0 6e :3 0
5c :2 0 6f :2 0
eb 235 239 ef
234 23b :3 0 f2
21d 23d :2 0 23f
f5 240 21a 23f
0 241 f7 0
242 f9 245 :3 0
245 0 245 244
242 243 :6 0 247
7 :3 0 fc 249
fe 248 247 :2 0
24a 100 :2 0 24c
0 24c 24b 1f9
24a :6 0 3ad 1
:3 0 8 :3 0 63
:3 0 24e 24f 0
8 :3 0 70 :2 0
71 :3 0 251 253
0 105 252 255
:3 0 256 :2 0 4a
:3 0 4b :3 0 258
259 0 4a :3 0
5b :3 0 25b 25c
0 76 :2 0 2
:4 0 77 :2 0 5e
:2 0 5c :2 0 108
261 263 :3 0 10b
25d 265 25a 267
266 :2 0 4d :3 0
5f :3 0 269 26a
0 4f :4 0 78
:4 0 52 :2 0 53
:3 0 10f 26e 270
:3 0 52 :2 0 6a
:4 0 112 272 274
:3 0 52 :2 0 79
:4 0 115 276 278
:3 0 52 :2 0 57
:3 0 58 :3 0 d
:3 0 7a :3 0 27d
27e 0 118 27c
280 59 :4 0 11a
27b 283 11d 27a
285 :3 0 120 26b
287 :2 0 289 123
28a 268 289 0
28b 125 0 2b5
4a :3 0 4b :3 0
28c 28d 0 4a
:3 0 5b :3 0 28f
290 0 5c :2 0
2 :4 0 7b :2 0
5e :2 0 5c :2 0
127 295 297 :3 0
12a 291 299 28e
29b 29a :2 0 4d
:3 0 5f :3 0 29d
29e 0 4f :4 0
7c :4 0 52 :2 0
53 :3 0 12e 2a2
2a4 :3 0 131 29f
2a6 :2 0 2a8 134
2a9 29c 2a8 0
2aa 136 0 2b5
7d :3 0 7e :3 0
2ab 2ac 0 d
:3 0 7a :3 0 2ae
2af 0 7f :3 0
138 2ad 2b2 :2 0
2b5 80 :3 0 13b
33b 57 :3 0 26
:3 0 1d :2 0 13f
2b6 2b9 70 :2 0
1d :2 0 144 2bb
2bd :3 0 2be :2 0
57 :3 0 25 :3 0
1d :2 0 147 2c0
2c3 70 :2 0 5c
:2 0 14c 2c5 2c7
:3 0 2c8 :2 0 2bf
2ca 2c9 :2 0 4a
:3 0 4b :3 0 2cc
2cd 0 4a :3 0
5b :3 0 2cf 2d0
0 11 :2 0 2
:4 0 81 :2 0 5e
:2 0 5c :2 0 14f
2d5 2d7 :3 0 152
2d1 2d9 2ce 2db
2da :2 0 4d :3 0
5f :3 0 2dd 2de
0 4f :4 0 82
:4 0 52 :2 0 53
:3 0 156 2e2 2e4
:3 0 52 :2 0 6a
:4 0 159 2e6 2e8
:3 0 52 :2 0 79
:4 0 15c 2ea 2ec
:3 0 52 :2 0 57
:3 0 58 :3 0 d
:3 0 83 :3 0 2f1
2f2 0 15f 2f0
2f4 59 :4 0 161
2ef 2f7 164 2ee
2f9 :3 0 52 :2 0
84 :4 0 167 2fb
2fd :3 0 52 :2 0
55 :4 0 16a 2ff
301 :3 0 52 :2 0
16 :3 0 16d 303
305 :3 0 52 :2 0
55 :4 0 170 307
309 :3 0 173 2df
30b :2 0 30d 176
30e 2dc 30d 0
30f 178 0 339
4a :3 0 4b :3 0
310 311 0 4a
:3 0 5b :3 0 313
314 0 5c :2 0
2 :4 0 85 :2 0
5e :2 0 5c :2 0
17a 319 31b :3 0
17d 315 31d 312
31f 31e :2 0 4d
:3 0 5f :3 0 321
322 0 4f :4 0
86 :4 0 52 :2 0
53 :3 0 181 326
328 :3 0 184 323
32a :2 0 32c 187
32d 320 32c 0
32e 189 0 339
7d :3 0 87 :3 0
32f 330 0 d
:3 0 83 :3 0 332
333 0 16 :3 0
7f :3 0 18b 331
337 :2 0 339 18f
33a 2cb 339 0
33c 257 2b5 0
33c 193 0 3ad
57 :3 0 61 :3 0
22 :3 0 37 :3 0
3a :3 0 10 :4 0
88 1 :8 0 349
8 :3 0 63 :3 0
344 345 0 64
:3 0 346 347 0
349 196 39c 65
:3 0 8 :3 0 63
:3 0 34b 34c 0
64 :3 0 34d 34e
0 350 199 352
19b 351 350 :2 0
39a 66 :3 0 8
:3 0 63 :3 0 355
356 0 64 :3 0
357 358 0 392
4a :3 0 4b :3 0
35a 35b 0 4a
:3 0 5b :3 0 35d
35e 0 67 :2 0
2 :4 0 89 :2 0
5e :2 0 5c :2 0
19d 363 365 :3 0
1a0 35f 367 35c
369 368 :2 0 4d
:3 0 5f :3 0 36b
36c 0 4f :4 0
8a :4 0 52 :2 0
53 :3 0 1a4 370
372 :3 0 52 :2 0
6a :4 0 1a7 374
376 :3 0 52 :2 0
6b :4 0 1aa 378
37a :3 0 52 :2 0
64 :3 0 1ad 37c
37e :3 0 52 :2 0
6c :4 0 1b0 380
382 :3 0 52 :2 0
6d :3 0 6e :3 0
5c :2 0 6f :2 0
1b3 385 389 1b7
384 38b :3 0 1ba
36d 38d :2 0 38f
1bd 390 36a 38f
0 391 1bf 0
392 1c1 395 :3 0
395 0 395 394
392 393 :6 0 397
9 :3 0 1c4 399
1c6 398 397 :2 0
39a 1c8 :2 0 39c
0 39c 39b 349
39a :6 0 3ad 1
:3 0 2e :3 0 8b
:4 0 52 :2 0 8c
:4 0 1cb 3a0 3a2
:3 0 52 :2 0 16
:3 0 1ce 3a4 3a6
:3 0 52 :2 0 8d
:4 0 1d1 3a8 3aa
:3 0 39e 3ab 0
3ad 1d4 699 10
:3 0 16 :3 0 3ae
3af 0 698 57
:3 0 21 :3 0 1d
:2 0 1d9 3b1 3b4
70 :2 0 8e :2 0
1de 3b6 3b8 :3 0
37 :3 0 8f :3 0
8f :3 0 d :3 0
90 :3 0 d :3 0
91 :3 0 8f :3 0
92 :3 0 57 :3 0
93 :3 0 92 :3 0
94 :3 0 57 :3 0
95 :3 0 94 :3 0
96 :3 0 97 :3 0
96 :3 0 97 :3 0
98 :3 0 99 :3 0
93 :3 0 95 :3 0
97 :3 0 9a :3 0
9b :3 0 9c :3 0
99 :3 0 9d :3 0
99 :3 0 9e :3 0
39 :3 0 10 :3 0
8f :3 0 d :3 0
9f :3 0 d :3 0
a0 :3 0 d :3 0
a1 :3 0 d :3 0
a2 :3 0 d :3 0
a3 :3 0 d :3 0
a4 :3 0 d :3 0
a5 :3 0 d :3 0
a6 :3 0 d :3 0
a7 :3 0 d :3 0
a8 :3 0 d :3 0
a9 :3 0 d :3 0
aa :3 0 d :3 0
ab :3 0 d :3 0
ac :3 0 d :3 0
ad :3 0 ae :3 0
af :3 0 98 :4 0
b0 1 :8 0 414
b1 :4 0 3ff :3 0
7f :3 0 70 :2 0
1e3 402 403 :3 0
8 :3 0 63 :3 0
405 406 0 1d
:2 0 407 408 0
40a 1e6 411 8
:3 0 63 :3 0 40b
40c 0 74 :2 0
40d 40e 0 410
1e8 412 404 40a
0 413 0 410
0 413 1ea 0
414 1ed 467 65
:3 0 8 :3 0 63
:3 0 416 417 0
64 :3 0 418 419
0 41b 1f0 41d
1f2 41c 41b :2 0
465 66 :3 0 8
:3 0 63 :3 0 420
421 0 64 :3 0
422 423 0 45d
4a :3 0 4b :3 0
425 426 0 4a
:3 0 5b :3 0 428
429 0 67 :2 0
2 :4 0 b2 :2 0
5e :2 0 5c :2 0
1f4 42e 430 :3 0
1f7 42a 432 427
434 433 :2 0 4d
:3 0 5f :3 0 436
437 0 4f :4 0
b3 :4 0 52 :2 0
53 :3 0 1fb 43b
43d :3 0 52 :2 0
6a :4 0 1fe 43f
441 :3 0 52 :2 0
6b :4 0 201 443
445 :3 0 52 :2 0
64 :3 0 204 447
449 :3 0 52 :2 0
6c :4 0 207 44b
44d :3 0 52 :2 0
6d :3 0 6e :3 0
5c :2 0 6f :2 0
20a 450 454 20e
44f 456 :3 0 211
438 458 :2 0 45a
214 45b 435 45a
0 45c 216 0
45d 218 460 :3 0
460 0 460 45f
45d 45e :6 0 462
b :3 0 21b 464
21d 463 462 :2 0
465 21f :2 0 467
0 467 466 414
465 :6 0 60c 1
:3 0 b4 :3 0 31
:4 0 46c :2 0 60c
46a 46d :3 0 b5
:3 0 31 :3 0 17
:3 0 14 :4 0 473
:2 0 5ab 46f 474
:3 0 222 :3 0 31
:3 0 b1 :3 0 475
476 :3 0 8 :3 0
63 :3 0 478 479
0 1d :2 0 47a
47b 0 47d 225
484 8 :3 0 63
:3 0 47e 47f 0
74 :2 0 480 481
0 483 227 485
477 47d 0 486
0 483 0 486
229 0 5ab 8
:3 0 63 :3 0 487
488 0 8 :3 0
b6 :2 0 b7 :3 0
48a 48c 0 22e
48b 48e :3 0 48f
:2 0 b8 :8 0 493
231 494 490 493
0 495 233 0
5ab 35 :3 0 3c
:3 0 92 :3 0 57
:3 0 93 :3 0 92
:3 0 94 :3 0 57
:3 0 95 :3 0 94
:3 0 96 :3 0 97
:3 0 97 :3 0 96
:3 0 b9 :3 0 ba
:3 0 9d :3 0 ba
:3 0 bb :3 0 57
:3 0 93 :3 0 92
:3 0 bc :3 0 57
:3 0 95 :3 0 94
:3 0 bd :3 0 ba
:3 0 be :3 0 bf
:3 0 bf :3 0 be
:3 0 c0 :3 0 c1
:3 0 c1 :3 0 c0
:3 0 c2 :3 0 c3
:3 0 9d :3 0 ba
:3 0 c4 :3 0 c5
:3 0 c6 :3 0 c3
:3 0 9d :3 0 ba
:3 0 c7 :3 0 c8
:3 0 93 :3 0 95
:3 0 97 :3 0 ba
:3 0 9a :3 0 9b
:3 0 9e :3 0 33
:3 0 17 :4 0 c9
1 :8 0 4e5 b1
:4 0 4d0 :3 0 7f
:3 0 70 :2 0 237
4d3 4d4 :3 0 8
:3 0 63 :3 0 4d6
4d7 0 1d :2 0
4d8 4d9 0 4db
23a 4e2 8 :3 0
63 :3 0 4dc 4dd
0 74 :2 0 4de
4df 0 4e1 23c
4e3 4d5 4db 0
4e4 0 4e1 0
4e4 23e 0 4e5
241 538 65 :3 0
8 :3 0 63 :3 0
4e7 4e8 0 64
:3 0 4e9 4ea 0
4ec 244 4ee 246
4ed 4ec :2 0 536
66 :3 0 8 :3 0
63 :3 0 4f1 4f2
0 64 :3 0 4f3
4f4 0 52e 4a
:3 0 4b :3 0 4f6
4f7 0 4a :3 0
5b :3 0 4f9 4fa
0 67 :2 0 2
:4 0 ca :2 0 5e
:2 0 5c :2 0 248
4ff 501 :3 0 24b
4fb 503 4f8 505
504 :2 0 4d :3 0
5f :3 0 507 508
0 4f :4 0 cb
:4 0 52 :2 0 53
:3 0 24f 50c 50e
:3 0 52 :2 0 6a
:4 0 252 510 512
:3 0 52 :2 0 6b
:4 0 255 514 516
:3 0 52 :2 0 64
:3 0 258 518 51a
:3 0 52 :2 0 6c
:4 0 25b 51c 51e
:3 0 52 :2 0 6d
:3 0 6e :3 0 5c
:2 0 6f :2 0 25e
521 525 262 520
527 :3 0 265 509
529 :2 0 52b 268
52c 506 52b 0
52d 26a 0 52e
26c 531 :3 0 531
0 531 530 52e
52f :6 0 533 e
:3 0 26f 535 271
534 533 :2 0 536
273 :2 0 538 0
538 537 4e5 536
:6 0 5ab d :3 0
45 :3 0 3c :3 0
39 :3 0 10 :3 0
33 :3 0 17 :4 0
cc 1 :8 0 556
b1 :4 0 541 :3 0
7f :3 0 70 :2 0
278 544 545 :3 0
8 :3 0 63 :3 0
547 548 0 1d
:2 0 549 54a 0
54c 27b 553 8
:3 0 63 :3 0 54d
54e 0 74 :2 0
54f 550 0 552
27d 554 546 54c
0 555 0 552
0 555 27f 0
556 282 5a9 65
:3 0 8 :3 0 63
:3 0 558 559 0
64 :3 0 55a 55b
0 55d 285 55f
287 55e 55d :2 0
5a7 66 :3 0 8
:3 0 63 :3 0 562
563 0 64 :3 0
564 565 0 59f
4a :3 0 4b :3 0
567 568 0 4a
:3 0 5b :3 0 56a
56b 0 67 :2 0
2 :4 0 cd :2 0
5e :2 0 5c :2 0
289 570 572 :3 0
28c 56c 574 569
576 575 :2 0 4d
:3 0 5f :3 0 578
579 0 4f :4 0
ce :4 0 52 :2 0
53 :3 0 290 57d
57f :3 0 52 :2 0
6a :4 0 293 581
583 :3 0 52 :2 0
6b :4 0 296 585
587 :3 0 52 :2 0
64 :3 0 299 589
58b :3 0 52 :2 0
6c :4 0 29c 58d
58f :3 0 52 :2 0
6d :3 0 6e :3 0
5c :2 0 6f :2 0
29f 592 596 2a3
591 598 :3 0 2a6
57a 59a :2 0 59c
2a9 59d 577 59c
0 59e 2ab 0
59f 2ad 5a2 :3 0
5a2 0 5a2 5a1
59f 5a0 :6 0 5a4
10 :3 0 2b0 5a6
2b2 5a5 5a4 :2 0
5a7 2b4 :2 0 5a9
0 5a9 5a8 556
5a7 :6 0 5ab d
:3 0 2b7 5ad b5
:4 0 5ab :4 0 60c
cf :3 0 31 :4 0
5b1 :2 0 5b7 5af
0 8 :3 0 63
:3 0 5b2 5b3 0
64 :3 0 5b4 5b5
0 5b7 2bd 60a
65 :3 0 8 :3 0
63 :3 0 5b9 5ba
0 64 :3 0 5bb
5bc 0 5be 2c0
5c0 2c2 5bf 5be
:2 0 608 66 :3 0
8 :3 0 63 :3 0
5c3 5c4 0 64
:3 0 5c5 5c6 0
600 4a :3 0 4b
:3 0 5c8 5c9 0
4a :3 0 5b :3 0
5cb 5cc 0 67
:2 0 2 :4 0 d0
:2 0 5e :2 0 5c
:2 0 2c4 5d1 5d3
:3 0 2c7 5cd 5d5
5ca 5d7 5d6 :2 0
4d :3 0 5f :3 0
5d9 5da 0 4f
:4 0 d1 :4 0 52
:2 0 53 :3 0 2cb
5de 5e0 :3 0 52
:2 0 6a :4 0 2ce
5e2 5e4 :3 0 52
:2 0 6b :4 0 2d1
5e6 5e8 :3 0 52
:2 0 64 :3 0 2d4
5ea 5ec :3 0 52
:2 0 6c :4 0 2d7
5ee 5f0 :3 0 52
:2 0 6d :3 0 6e
:3 0 5c :2 0 6f
:2 0 2da 5f3 5f7
2de 5f2 5f9 :3 0
2e1 5db 5fb :2 0
5fd 2e4 5fe 5d8
5fd 0 5ff 2e6
0 600 2e8 603
:3 0 603 0 603
602 600 601 :6 0
605 12 :3 0 2eb
607 2ed 606 605
:2 0 608 2ef :2 0
60a 0 60a 609
5b7 608 :6 0 60c
1 :3 0 2f2 60d
3b9 60c 0 60e
2f7 0 698 37
:3 0 d2 :3 0 d
:3 0 d3 :3 0 39
:3 0 10 :3 0 ae
:3 0 af :3 0 98
:4 0 d4 1 :8 0
62e b1 :4 0 619
:3 0 7f :3 0 70
:2 0 2fb 61c 61d
:3 0 8 :3 0 63
:3 0 61f 620 0
1d :2 0 621 622
0 624 2fe 62b
8 :3 0 63 :3 0
625 626 0 74
:2 0 627 628 0
62a 300 62c 61e
624 0 62d 0
62a 0 62d 302
0 62e 305 681
65 :3 0 8 :3 0
63 :3 0 630 631
0 64 :3 0 632
633 0 635 308
637 30a 636 635
:2 0 67f 66 :3 0
8 :3 0 63 :3 0
63a 63b 0 64
:3 0 63c 63d 0
677 4a :3 0 4b
:3 0 63f 640 0
4a :3 0 5b :3 0
642 643 0 67
:2 0 2 :4 0 d5
:2 0 5e :2 0 5c
:2 0 30c 648 64a
:3 0 30f 644 64c
641 64e 64d :2 0
4d :3 0 5f :3 0
650 651 0 4f
:4 0 d6 :4 0 52
:2 0 53 :3 0 313
655 657 :3 0 52
:2 0 6a :4 0 316
659 65b :3 0 52
:2 0 6b :4 0 319
65d 65f :3 0 52
:2 0 64 :3 0 31c
661 663 :3 0 52
:2 0 6c :4 0 31f
665 667 :3 0 52
:2 0 6d :3 0 6e
:3 0 5c :2 0 6f
:2 0 322 66a 66e
326 669 670 :3 0
329 652 672 :2 0
674 32c 675 64f
674 0 676 32e
0 677 330 67a
:3 0 67a 0 67a
679 677 678 :6 0
67c 14 :3 0 333
67e 335 67d 67c
:2 0 67f 337 :2 0
681 0 681 680
62e 67f :6 0 698
1 :3 0 26 :3 0
5c :2 0 683 684
0 698 21 :3 0
d7 :2 0 686 687
0 698 2e :3 0
d8 :4 0 52 :2 0
d9 :4 0 33a 68b
68d :3 0 52 :2 0
16 :3 0 33d 68f
691 :3 0 52 :2 0
8d :4 0 340 693
695 :3 0 689 696
0 698 343 69a
1e6 3ad 0 69b
0 698 0 69b
34a 0 1544 b4
:3 0 40 :3 0 2e
:3 0 69d 69e 0
69f :2 0 1544 b5
:3 0 40 :3 0 16
:3 0 14 :3 0 15
:3 0 27 :4 0 6a8
:2 0 138a 6a2 6a9
:3 0 34d :3 0 40
:3 0 b1 :3 0 6aa
6ab :3 0 8 :3 0
63 :3 0 6ad 6ae
0 1d :2 0 6af
6b0 0 6b2 352
6b9 8 :3 0 63
:3 0 6b3 6b4 0
74 :2 0 6b5 6b6
0 6b8 354 6ba
6ac 6b2 0 6bb
0 6b8 0 6bb
356 0 138a 8
:3 0 63 :3 0 6bc
6bd 0 8 :3 0
b6 :2 0 b7 :3 0
6bf 6c1 0 35b
6c0 6c3 :3 0 6c4
:2 0 b8 :8 0 6c8
35e 6c9 6c5 6c8
0 6ca 360 0
138a 21 :3 0 5c
:2 0 da :3 0 db
:3 0 db :3 0 dc
:3 0 db :3 0 dd
:3 0 db :3 0 dd
:3 0 db :3 0 de
:3 0 db :3 0 de
:3 0 db :3 0 df
:3 0 db :3 0 e0
:3 0 db :3 0 e1
:3 0 db :3 0 e1
:3 0 db :3 0 e2
:3 0 db :3 0 e2
:3 0 db :3 0 e3
:3 0 39 :3 0 10
:3 0 3a :3 0 15
:4 0 e4 1 :8 0
701 b1 :4 0 6ec
:3 0 7f :3 0 70
:2 0 364 6ef 6f0
:3 0 8 :3 0 63
:3 0 6f2 6f3 0
1d :2 0 6f4 6f5
0 6f7 367 6fe
8 :3 0 63 :3 0
6f8 6f9 0 74
:2 0 6fa 6fb 0
6fd 369 6ff 6f1
6f7 0 700 0
6fd 0 700 36b
0 701 36e 754
65 :3 0 8 :3 0
63 :3 0 703 704
0 64 :3 0 705
706 0 708 371
70a 373 709 708
:2 0 752 66 :3 0
8 :3 0 63 :3 0
70d 70e 0 64
:3 0 70f 710 0
74a 4a :3 0 4b
:3 0 712 713 0
4a :3 0 5b :3 0
715 716 0 67
:2 0 2 :4 0 e5
:2 0 5e :2 0 5c
:2 0 375 71b 71d
:3 0 378 717 71f
714 721 720 :2 0
4d :3 0 5f :3 0
723 724 0 4f
:4 0 e6 :4 0 52
:2 0 53 :3 0 37c
728 72a :3 0 52
:2 0 6a :4 0 37f
72c 72e :3 0 52
:2 0 6b :4 0 382
730 732 :3 0 52
:2 0 64 :3 0 385
734 736 :3 0 52
:2 0 6c :4 0 388
738 73a :3 0 52
:2 0 6d :3 0 6e
:3 0 5c :2 0 6f
:2 0 38b 73d 741
38f 73c 743 :3 0
392 725 745 :2 0
747 395 748 722
747 0 749 397
0 74a 399 74d
:3 0 74d 0 74d
74c 74a 74b :6 0
74f 17 :3 0 39c
751 39e 750 74f
:2 0 752 3a0 :2 0
754 0 754 753
701 752 :6 0 8c8
16 :3 0 35 :3 0
32 :3 0 32 :3 0
e7 :3 0 32 :3 0
e8 :3 0 32 :3 0
e9 :3 0 32 :3 0
ea :3 0 32 :3 0
eb :3 0 32 :3 0
ec :3 0 32 :3 0
ed :3 0 32 :3 0
ee :3 0 32 :3 0
ef :3 0 32 :3 0
f0 :3 0 32 :3 0
f1 :3 0 32 :3 0
f2 :3 0 32 :3 0
e8 :3 0 32 :3 0
f3 :3 0 32 :3 0
ea :3 0 32 :3 0
f4 :3 0 32 :3 0
ec :3 0 32 :3 0
f5 :3 0 32 :3 0
ee :3 0 32 :3 0
f6 :3 0 32 :3 0
f0 :3 0 32 :3 0
f7 :3 0 32 :3 0
f2 :3 0 32 :3 0
f8 :3 0 32 :3 0
f3 :3 0 32 :3 0
f9 :3 0 32 :3 0
f4 :3 0 32 :3 0
fa :3 0 32 :3 0
f5 :3 0 32 :3 0
fb :3 0 32 :3 0
f6 :3 0 32 :3 0
b9 :3 0 32 :3 0
ba :3 0 32 :3 0
f7 :3 0 32 :3 0
be :3 0 32 :3 0
bf :3 0 32 :3 0
f8 :3 0 32 :3 0
c0 :3 0 32 :3 0
c1 :3 0 32 :3 0
fc :3 0 32 :3 0
c4 :3 0 32 :3 0
fd :3 0 32 :3 0
c7 :3 0 32 :3 0
fe :3 0 32 :3 0
ff :3 0 32 :3 0
100 :3 0 32 :3 0
101 :3 0 33 :3 0
16 :4 0 102 1
:8 0 7ce b1 :4 0
7b9 :3 0 7f :3 0
70 :2 0 3a5 7bc
7bd :3 0 8 :3 0
63 :3 0 7bf 7c0
0 1d :2 0 7c1
7c2 0 7c4 3a8
7cb 8 :3 0 63
:3 0 7c5 7c6 0
74 :2 0 7c7 7c8
0 7ca 3aa 7cc
7be 7c4 0 7cd
0 7ca 0 7cd
3ac 0 7ce 3af
821 65 :3 0 8
:3 0 63 :3 0 7d0
7d1 0 64 :3 0
7d2 7d3 0 7d5
3b2 7d7 3b4 7d6
7d5 :2 0 81f 66
:3 0 8 :3 0 63
:3 0 7da 7db 0
64 :3 0 7dc 7dd
0 817 4a :3 0
4b :3 0 7df 7e0
0 4a :3 0 5b
:3 0 7e2 7e3 0
67 :2 0 2 :4 0
103 :2 0 5e :2 0
5c :2 0 3b6 7e8
7ea :3 0 3b9 7e4
7ec 7e1 7ee 7ed
:2 0 4d :3 0 5f
:3 0 7f0 7f1 0
4f :4 0 104 :4 0
52 :2 0 53 :3 0
3bd 7f5 7f7 :3 0
52 :2 0 6a :4 0
3c0 7f9 7fb :3 0
52 :2 0 6b :4 0
3c3 7fd 7ff :3 0
52 :2 0 64 :3 0
3c6 801 803 :3 0
52 :2 0 6c :4 0
3c9 805 807 :3 0
52 :2 0 6d :3 0
6e :3 0 5c :2 0
6f :2 0 3cc 80a
80e 3d0 809 810
:3 0 3d3 7f2 812
:2 0 814 3d6 815
7ef 814 0 816
3d8 0 817 3da
81a :3 0 81a 0
81a 819 817 818
:6 0 81c 19 :3 0
3dd 81e 3df 81d
81c :2 0 81f 3e1
:2 0 821 0 821
820 7ce 81f :6 0
8c8 16 :3 0 105
:3 0 106 :3 0 107
:3 0 107 :3 0 108
:3 0 108 :3 0 109
:3 0 109 :3 0 10a
:3 0 10b :3 0 10c
:3 0 10c :3 0 10d
:3 0 10d :3 0 10e
:3 0 10e :3 0 10f
:3 0 110 :3 0 111
:3 0 111 :3 0 112
:3 0 112 :3 0 113
:3 0 113 :3 0 114
:3 0 115 :3 0 116
:3 0 116 :3 0 117
:3 0 117 :3 0 118
:3 0 118 :3 0 119
:3 0 11a :3 0 11b
:3 0 11b :3 0 11c
:3 0 11c :3 0 11d
:3 0 11d :3 0 11e
:3 0 11f :3 0 f9
:3 0 120 :3 0 fa
:3 0 121 :3 0 fb
:3 0 122 :3 0 b9
:3 0 ba :3 0 123
:3 0 be :3 0 bf
:3 0 124 :3 0 c0
:3 0 c1 :3 0 39
:3 0 10 :4 0 125
1 :8 0 873 b1
:4 0 85e :3 0 7f
:3 0 70 :2 0 3e6
861 862 :3 0 8
:3 0 63 :3 0 864
865 0 1d :2 0
866 867 0 869
3e9 870 8 :3 0
63 :3 0 86a 86b
0 74 :2 0 86c
86d 0 86f 3eb
871 863 869 0
872 0 86f 0
872 3ed 0 873
3f0 8c6 65 :3 0
8 :3 0 63 :3 0
875 876 0 64
:3 0 877 878 0
87a 3f3 87c 3f5
87b 87a :2 0 8c4
66 :3 0 8 :3 0
63 :3 0 87f 880
0 64 :3 0 881
882 0 8bc 4a
:3 0 4b :3 0 884
885 0 4a :3 0
5b :3 0 887 888
0 67 :2 0 2
:4 0 126 :2 0 5e
:2 0 5c :2 0 3f7
88d 88f :3 0 3fa
889 891 886 893
892 :2 0 4d :3 0
5f :3 0 895 896
0 4f :4 0 127
:4 0 52 :2 0 53
:3 0 3fe 89a 89c
:3 0 52 :2 0 6a
:4 0 401 89e 8a0
:3 0 52 :2 0 6b
:4 0 404 8a2 8a4
:3 0 52 :2 0 64
:3 0 407 8a6 8a8
:3 0 52 :2 0 6c
:4 0 40a 8aa 8ac
:3 0 52 :2 0 6d
:3 0 6e :3 0 5c
:2 0 6f :2 0 40d
8af 8b3 411 8ae
8b5 :3 0 414 897
8b7 :2 0 8b9 417
8ba 894 8b9 0
8bb 419 0 8bc
41b 8bf :3 0 8bf
0 8bf 8be 8bc
8bd :6 0 8c1 1b
:3 0 41e 8c3 420
8c2 8c1 :2 0 8c4
422 :2 0 8c6 0
8c6 8c5 873 8c4
:6 0 8c8 16 :3 0
425 8ca 429 8c9
8c8 :2 0 1317 d7
:2 0 ba :3 0 20
:3 0 35 :3 0 3c
:3 0 33 :3 0 16
:4 0 128 1 :8 0
8d8 8 :3 0 63
:3 0 8d3 8d4 0
64 :3 0 8d5 8d6
0 8d8 42b 92b
65 :3 0 8 :3 0
63 :3 0 8da 8db
0 64 :3 0 8dc
8dd 0 8df 42e
8e1 430 8e0 8df
:2 0 929 66 :3 0
8 :3 0 63 :3 0
8e4 8e5 0 64
:3 0 8e6 8e7 0
921 4a :3 0 4b
:3 0 8e9 8ea 0
4a :3 0 5b :3 0
8ec 8ed 0 67
:2 0 2 :4 0 129
:2 0 5e :2 0 5c
:2 0 432 8f2 8f4
:3 0 435 8ee 8f6
8eb 8f8 8f7 :2 0
4d :3 0 5f :3 0
8fa 8fb 0 4f
:4 0 12a :4 0 52
:2 0 53 :3 0 439
8ff 901 :3 0 52
:2 0 6a :4 0 43c
903 905 :3 0 52
:2 0 6b :4 0 43f
907 909 :3 0 52
:2 0 64 :3 0 442
90b 90d :3 0 52
:2 0 6c :4 0 445
90f 911 :3 0 52
:2 0 6d :3 0 6e
:3 0 5c :2 0 6f
:2 0 448 914 918
44c 913 91a :3 0
44f 8fc 91c :2 0
91e 452 91f 8f9
91e 0 920 454
0 921 456 924
:3 0 924 0 924
923 921 922 :6 0
926 1d :3 0 459
928 45b 927 926
:2 0 929 45d :2 0
92b 0 92b 92a
8d8 929 :6 0 1311
16 :3 0 8 :3 0
63 :3 0 92d 92e
0 8 :3 0 b6
:2 0 b7 :3 0 930
932 0 462 931
934 :3 0 935 :2 0
20 :3 0 12b :2 0
465 938 939 :3 0
93a :2 0 93b :2 0
936 93d 93c :2 0
12c :3 0 93f :4 0
941 467 942 93e
941 0 943 469
0 1311 33 :3 0
12d :3 0 12d :3 0
27 :3 0 18 :3 0
27 :3 0 35 :3 0
3c :3 0 3b :3 0
10 :3 0 34 :3 0
14 :4 0 12e 1
:8 0 956 8 :3 0
63 :3 0 951 952
0 64 :3 0 953
954 0 956 46b
9a9 65 :3 0 8
:3 0 63 :3 0 958
959 0 64 :3 0
95a 95b 0 95d
46e 95f 470 95e
95d :2 0 9a7 66
:3 0 8 :3 0 63
:3 0 962 963 0
64 :3 0 964 965
0 99f 4a :3 0
4b :3 0 967 968
0 4a :3 0 5b
:3 0 96a 96b 0
67 :2 0 2 :4 0
12f :2 0 5e :2 0
5c :2 0 472 970
972 :3 0 475 96c
974 969 976 975
:2 0 4d :3 0 5f
:3 0 978 979 0
4f :4 0 130 :4 0
52 :2 0 53 :3 0
479 97d 97f :3 0
52 :2 0 6a :4 0
47c 981 983 :3 0
52 :2 0 6b :4 0
47f 985 987 :3 0
52 :2 0 64 :3 0
482 989 98b :3 0
52 :2 0 6c :4 0
485 98d 98f :3 0
52 :2 0 6d :3 0
6e :3 0 5c :2 0
6f :2 0 488 992
996 48c 991 998
:3 0 48f 97a 99a
:2 0 99c 492 99d
977 99c 0 99e
494 0 99f 496
9a2 :3 0 9a2 0
9a2 9a1 99f 9a0
:6 0 9a4 1f :3 0
499 9a6 49b 9a5
9a4 :2 0 9a7 49d
:2 0 9a9 0 9a9
9a8 956 9a7 :6 0
1311 16 :3 0 8
:3 0 63 :3 0 9ab
9ac 0 8 :3 0
70 :2 0 b7 :3 0
9ae 9b0 0 4a2
9af 9b2 :3 0 9b3
:2 0 18 :3 0 131
:2 0 4a5 9b6 9b7
:3 0 9b8 :2 0 9b9
:2 0 9b4 9bb 9ba
:2 0 57 :3 0 26
:3 0 1d :2 0 4a7
9bd 9c0 70 :2 0
1d :2 0 4ac 9c2
9c4 :3 0 1f :3 0
d7 :2 0 9c6 9c7
0 9c9 4af b86
35 :3 0 33 :3 0
16 :4 0 132 1
:8 0 9e3 b1 :4 0
9ce :3 0 7f :3 0
70 :2 0 4b3 9d1
9d2 :3 0 8 :3 0
63 :3 0 9d4 9d5
0 1d :2 0 9d6
9d7 0 9d9 4b6
9e0 8 :3 0 63
:3 0 9da 9db 0
74 :2 0 9dc 9dd
0 9df 4b8 9e1
9d3 9d9 0 9e2
0 9df 0 9e2
4ba 0 9e3 4bd
a36 65 :3 0 8
:3 0 63 :3 0 9e5
9e6 0 64 :3 0
9e7 9e8 0 9ea
4c0 9ec 4c2 9eb
9ea :2 0 a34 66
:3 0 8 :3 0 63
:3 0 9ef 9f0 0
64 :3 0 9f1 9f2
0 a2c 4a :3 0
4b :3 0 9f4 9f5
0 4a :3 0 5b
:3 0 9f7 9f8 0
67 :2 0 2 :4 0
133 :2 0 5e :2 0
5c :2 0 4c4 9fd
9ff :3 0 4c7 9f9
a01 9f6 a03 a02
:2 0 4d :3 0 5f
:3 0 a05 a06 0
4f :4 0 134 :4 0
52 :2 0 53 :3 0
4cb a0a a0c :3 0
52 :2 0 6a :4 0
4ce a0e a10 :3 0
52 :2 0 6b :4 0
4d1 a12 a14 :3 0
52 :2 0 64 :3 0
4d4 a16 a18 :3 0
52 :2 0 6c :4 0
4d7 a1a a1c :3 0
52 :2 0 6d :3 0
6e :3 0 5c :2 0
6f :2 0 4da a1f
a23 4de a1e a25
:3 0 4e1 a07 a27
:2 0 a29 4e4 a2a
a04 a29 0 a2b
4e6 0 a2c 4e8
a2f :3 0 a2f 0
a2f a2e a2c a2d
:6 0 a31 21 :3 0
4eb a33 4ed a32
a31 :2 0 a34 4ef
:2 0 a36 0 a36
a35 9e3 a34 :6 0
b85 16 :3 0 135
:3 0 33 :3 0 16
:4 0 136 1 :8 0
a51 b1 :4 0 a3c
:3 0 7f :3 0 70
:2 0 4f4 a3f a40
:3 0 8 :3 0 63
:3 0 a42 a43 0
1d :2 0 a44 a45
0 a47 4f7 a4e
8 :3 0 63 :3 0
a48 a49 0 74
:2 0 a4a a4b 0
a4d 4f9 a4f a41
a47 0 a50 0
a4d 0 a50 4fb
0 a51 4fe aa4
65 :3 0 8 :3 0
63 :3 0 a53 a54
0 64 :3 0 a55
a56 0 a58 501
a5a 503 a59 a58
:2 0 aa2 66 :3 0
8 :3 0 63 :3 0
a5d a5e 0 64
:3 0 a5f a60 0
a9a 4a :3 0 4b
:3 0 a62 a63 0
4a :3 0 5b :3 0
a65 a66 0 67
:2 0 2 :4 0 137
:2 0 5e :2 0 5c
:2 0 505 a6b a6d
:3 0 508 a67 a6f
a64 a71 a70 :2 0
4d :3 0 5f :3 0
a73 a74 0 4f
:4 0 138 :4 0 52
:2 0 53 :3 0 50c
a78 a7a :3 0 52
:2 0 6a :4 0 50f
a7c a7e :3 0 52
:2 0 6b :4 0 512
a80 a82 :3 0 52
:2 0 64 :3 0 515
a84 a86 :3 0 52
:2 0 6c :4 0 518
a88 a8a :3 0 52
:2 0 6d :3 0 6e
:3 0 5c :2 0 6f
:2 0 51b a8d a91
51f a8c a93 :3 0
522 a75 a95 :2 0
a97 525 a98 a72
a97 0 a99 527
0 a9a 529 a9d
:3 0 a9d 0 a9d
a9c a9a a9b :6 0
a9f 23 :3 0 52c
aa1 52e aa0 a9f
:2 0 aa2 530 :2 0
aa4 0 aa4 aa3
a51 aa2 :6 0 b85
16 :3 0 139 :3 0
13a :3 0 16 :4 0
13b 1 :8 0 abf
b1 :4 0 aaa :3 0
7f :3 0 70 :2 0
535 aad aae :3 0
8 :3 0 63 :3 0
ab0 ab1 0 1d
:2 0 ab2 ab3 0
ab5 538 abc 8
:3 0 63 :3 0 ab6
ab7 0 74 :2 0
ab8 ab9 0 abb
53a abd aaf ab5
0 abe 0 abb
0 abe 53c 0
abf 53f b12 65
:3 0 8 :3 0 63
:3 0 ac1 ac2 0
64 :3 0 ac3 ac4
0 ac6 542 ac8
544 ac7 ac6 :2 0
b10 66 :3 0 8
:3 0 63 :3 0 acb
acc 0 64 :3 0
acd ace 0 b08
4a :3 0 4b :3 0
ad0 ad1 0 4a
:3 0 5b :3 0 ad3
ad4 0 67 :2 0
2 :4 0 13c :2 0
5e :2 0 5c :2 0
546 ad9 adb :3 0
549 ad5 add ad2
adf ade :2 0 4d
:3 0 5f :3 0 ae1
ae2 0 4f :4 0
13d :4 0 52 :2 0
53 :3 0 54d ae6
ae8 :3 0 52 :2 0
6a :4 0 550 aea
aec :3 0 52 :2 0
6b :4 0 553 aee
af0 :3 0 52 :2 0
64 :3 0 556 af2
af4 :3 0 52 :2 0
6c :4 0 559 af6
af8 :3 0 52 :2 0
6d :3 0 6e :3 0
5c :2 0 6f :2 0
55c afb aff 560
afa b01 :3 0 563
ae3 b03 :2 0 b05
566 b06 ae0 b05
0 b07 568 0
b08 56a b0b :3 0
b0b 0 b0b b0a
b08 b09 :6 0 b0d
25 :3 0 56d b0f
56f b0e b0d :2 0
b10 571 :2 0 b12
0 b12 b11 abf
b10 :6 0 b85 16
:3 0 13e :3 0 13a
:3 0 16 :4 0 13f
1 :8 0 b2d b1
:4 0 b18 :3 0 7f
:3 0 70 :2 0 576
b1b b1c :3 0 8
:3 0 63 :3 0 b1e
b1f 0 1d :2 0
b20 b21 0 b23
579 b2a 8 :3 0
63 :3 0 b24 b25
0 74 :2 0 b26
b27 0 b29 57b
b2b b1d b23 0
b2c 0 b29 0
b2c 57d 0 b2d
580 b80 65 :3 0
8 :3 0 63 :3 0
b2f b30 0 64
:3 0 b31 b32 0
b34 583 b36 585
b35 b34 :2 0 b7e
66 :3 0 8 :3 0
63 :3 0 b39 b3a
0 64 :3 0 b3b
b3c 0 b76 4a
:3 0 4b :3 0 b3e
b3f 0 4a :3 0
5b :3 0 b41 b42
0 67 :2 0 2
:4 0 140 :2 0 5e
:2 0 5c :2 0 587
b47 b49 :3 0 58a
b43 b4b b40 b4d
b4c :2 0 4d :3 0
5f :3 0 b4f b50
0 4f :4 0 141
:4 0 52 :2 0 53
:3 0 58e b54 b56
:3 0 52 :2 0 6a
:4 0 591 b58 b5a
:3 0 52 :2 0 6b
:4 0 594 b5c b5e
:3 0 52 :2 0 64
:3 0 597 b60 b62
:3 0 52 :2 0 6c
:4 0 59a b64 b66
:3 0 52 :2 0 6d
:3 0 6e :3 0 5c
:2 0 6f :2 0 59d
b69 b6d 5a1 b68
b6f :3 0 5a4 b51
b71 :2 0 b73 5a7
b74 b4e b73 0
b75 5a9 0 b76
5ab b79 :3 0 b79
0 b79 b78 b76
b77 :6 0 b7b 27
:3 0 5ae b7d 5b0
b7c b7b :2 0 b7e
5b2 :2 0 b80 0
b80 b7f b2d b7e
:6 0 b85 16 :3 0
1f :3 0 5c :2 0
b82 b83 0 b85
5b5 b87 9c5 9c9
0 b88 0 b85
0 b88 5bb 0
b89 5be b91 18
:3 0 16 :3 0 b8a
b8b 0 b90 1f
:3 0 1d :2 0 b8d
b8e 0 b90 5c0
b92 9bc b89 0
b93 0 b90 0
b93 5c3 0 1311
20 :3 0 1d :2 0
b94 b95 0 1311
b4 :3 0 41 :4 0
b9a :2 0 1311 b98
b9b :3 0 b5 :3 0
41 :3 0 12 :3 0
20 :3 0 a :4 0
ba2 :2 0 dc0 b9d
ba3 :3 0 5c6 :3 0
41 :3 0 b1 :3 0
ba4 ba5 :3 0 8
:3 0 63 :3 0 ba7
ba8 0 1d :2 0
ba9 baa 0 bac
5ca bb3 8 :3 0
63 :3 0 bad bae
0 74 :2 0 baf
bb0 0 bb2 5cc
bb4 ba6 bac 0
bb5 0 bb2 0
bb5 5ce 0 dc0
8 :3 0 63 :3 0
bb6 bb7 0 8
:3 0 b6 :2 0 b7
:3 0 bb9 bbb 0
5d3 bba bbd :3 0
bbe :2 0 b8 :8 0
bc2 5d6 bc3 bbf
bc2 0 bc4 5d8
0 dc0 57 :3 0
1f :3 0 1d :2 0
5da bc5 bc8 70
:2 0 5c :2 0 5df
bca bcc :3 0 45
:3 0 43 :3 0 43
:3 0 20 :3 0 44
:3 0 142 :3 0 143
:3 0 44 :3 0 a
:3 0 39 :3 0 10
:3 0 33 :3 0 18
:3 0 42 :3 0 12
:4 0 144 1 :8 0
bf3 b1 :4 0 bde
:3 0 7f :3 0 70
:2 0 5e4 be1 be2
:3 0 8 :3 0 63
:3 0 be4 be5 0
1d :2 0 be6 be7
0 be9 5e7 bf0
8 :3 0 63 :3 0
bea beb 0 74
:2 0 bec bed 0
bef 5e9 bf1 be3
be9 0 bf2 0
bef 0 bf2 5eb
0 bf3 5ee c46
65 :3 0 8 :3 0
63 :3 0 bf5 bf6
0 64 :3 0 bf7
bf8 0 bfa 5f1
bfc 5f3 bfb bfa
:2 0 c44 66 :3 0
8 :3 0 63 :3 0
bff c00 0 64
:3 0 c01 c02 0
c3c 4a :3 0 4b
:3 0 c04 c05 0
4a :3 0 5b :3 0
c07 c08 0 67
:2 0 2 :4 0 145
:2 0 5e :2 0 5c
:2 0 5f5 c0d c0f
:3 0 5f8 c09 c11
c06 c13 c12 :2 0
4d :3 0 5f :3 0
c15 c16 0 4f
:4 0 146 :4 0 52
:2 0 53 :3 0 5fc
c1a c1c :3 0 52
:2 0 6a :4 0 5ff
c1e c20 :3 0 52
:2 0 6b :4 0 602
c22 c24 :3 0 52
:2 0 64 :3 0 605
c26 c28 :3 0 52
:2 0 6c :4 0 608
c2a c2c :3 0 52
:2 0 6d :3 0 6e
:3 0 5c :2 0 6f
:2 0 60b c2f c33
60f c2e c35 :3 0
612 c17 c37 :2 0
c39 615 c3a c14
c39 0 c3b 617
0 c3c 619 c3f
:3 0 c3f 0 c3f
c3e c3c c3d :6 0
c41 2a :3 0 61c
c43 61e c42 c41
:2 0 c44 620 :2 0
c46 0 c46 c45
bf3 c44 :6 0 d3e
29 :3 0 8 :3 0
63 :3 0 c48 c49
0 8 :3 0 70
:2 0 b7 :3 0 c4b
c4d 0 625 c4c
c4f :3 0 c50 :2 0
45 :3 0 39 :3 0
10 :3 0 33 :3 0
16 :3 0 42 :3 0
12 :4 0 147 1
:8 0 c6f b1 :4 0
c5a :3 0 7f :3 0
70 :2 0 62a c5d
c5e :3 0 8 :3 0
63 :3 0 c60 c61
0 1d :2 0 c62
c63 0 c65 62d
c6c 8 :3 0 63
:3 0 c66 c67 0
74 :2 0 c68 c69
0 c6b 62f c6d
c5f c65 0 c6e
0 c6b 0 c6e
631 0 c6f 634
cc2 65 :3 0 8
:3 0 63 :3 0 c71
c72 0 64 :3 0
c73 c74 0 c76
637 c78 639 c77
c76 :2 0 cc0 66
:3 0 8 :3 0 63
:3 0 c7b c7c 0
64 :3 0 c7d c7e
0 cb8 4a :3 0
4b :3 0 c80 c81
0 4a :3 0 5b
:3 0 c83 c84 0
67 :2 0 2 :4 0
148 :2 0 5e :2 0
5c :2 0 63b c89
c8b :3 0 63e c85
c8d c82 c8f c8e
:2 0 4d :3 0 5f
:3 0 c91 c92 0
4f :4 0 149 :4 0
52 :2 0 53 :3 0
642 c96 c98 :3 0
52 :2 0 6a :4 0
645 c9a c9c :3 0
52 :2 0 6b :4 0
648 c9e ca0 :3 0
52 :2 0 64 :3 0
64b ca2 ca4 :3 0
52 :2 0 6c :4 0
64e ca6 ca8 :3 0
52 :2 0 6d :3 0
6e :3 0 5c :2 0
6f :2 0 651 cab
caf 655 caa cb1
:3 0 658 c93 cb3
:2 0 cb5 65b cb6
c90 cb5 0 cb7
65d 0 cb8 65f
cbb :3 0 cbb 0
cbb cba cb8 cb9
:6 0 cbd 2c :3 0
662 cbf 664 cbe
cbd :2 0 cc0 666
:2 0 cc2 0 cc2
cc1 c6f cc0 :6 0
cc4 29 :3 0 669
d3a 45 :3 0 33
:3 0 18 :3 0 39
:3 0 10 :3 0 33
:3 0 16 :3 0 42
:3 0 12 :4 0 14a
1 :8 0 ce4 b1
:4 0 ccf :3 0 7f
:3 0 70 :2 0 66d
cd2 cd3 :3 0 8
:3 0 63 :3 0 cd5
cd6 0 1d :2 0
cd7 cd8 0 cda
670 ce1 8 :3 0
63 :3 0 cdb cdc
0 74 :2 0 cdd
cde 0 ce0 672
ce2 cd4 cda 0
ce3 0 ce0 0
ce3 674 0 ce4
677 d37 65 :3 0
8 :3 0 63 :3 0
ce6 ce7 0 64
:3 0 ce8 ce9 0
ceb 67a ced 67c
cec ceb :2 0 d35
66 :3 0 8 :3 0
63 :3 0 cf0 cf1
0 64 :3 0 cf2
cf3 0 d2d 4a
:3 0 4b :3 0 cf5
cf6 0 4a :3 0
5b :3 0 cf8 cf9
0 67 :2 0 2
:4 0 14b :2 0 5e
:2 0 5c :2 0 67e
cfe d00 :3 0 681
cfa d02 cf7 d04
d03 :2 0 4d :3 0
5f :3 0 d06 d07
0 4f :4 0 14c
:4 0 52 :2 0 53
:3 0 685 d0b d0d
:3 0 52 :2 0 6a
:4 0 688 d0f d11
:3 0 52 :2 0 6b
:4 0 68b d13 d15
:3 0 52 :2 0 64
:3 0 68e d17 d19
:3 0 52 :2 0 6c
:4 0 691 d1b d1d
:3 0 52 :2 0 6d
:3 0 6e :3 0 5c
:2 0 6f :2 0 694
d20 d24 698 d1f
d26 :3 0 69b d08
d28 :2 0 d2a 69e
d2b d05 d2a 0
d2c 6a0 0 d2d
6a2 d30 :3 0 d30
0 d30 d2f d2d
d2e :6 0 d32 2e
:3 0 6a5 d34 6a7
d33 d32 :2 0 d35
6a9 :2 0 d37 0
d37 d36 ce4 d35
:6 0 d39 29 :3 0
6ac d3b c51 cc4
0 d3c 0 d39
0 d3c 6ae 0
d3e 80 :3 0 6b1
dbe 57 :3 0 1f
:3 0 1d :2 0 6b4
d3f d42 70 :2 0
d7 :2 0 6b9 d44
d46 :3 0 45 :3 0
33 :3 0 18 :3 0
39 :3 0 10 :3 0
33 :3 0 16 :3 0
42 :3 0 12 :4 0
14a 1 :8 0 d67
b1 :4 0 d52 :3 0
7f :3 0 70 :2 0
6be d55 d56 :3 0
8 :3 0 63 :3 0
d58 d59 0 1d
:2 0 d5a d5b 0
d5d 6c1 d64 8
:3 0 63 :3 0 d5e
d5f 0 74 :2 0
d60 d61 0 d63
6c3 d65 d57 d5d
0 d66 0 d63
0 d66 6c5 0
d67 6c8 dba 65
:3 0 8 :3 0 63
:3 0 d69 d6a 0
64 :3 0 d6b d6c
0 d6e 6cb d70
6cd d6f d6e :2 0
db8 66 :3 0 8
:3 0 63 :3 0 d73
d74 0 64 :3 0
d75 d76 0 db0
4a :3 0 4b :3 0
d78 d79 0 4a
:3 0 5b :3 0 d7b
d7c 0 67 :2 0
2 :4 0 14d :2 0
5e :2 0 5c :2 0
6cf d81 d83 :3 0
6d2 d7d d85 d7a
d87 d86 :2 0 4d
:3 0 5f :3 0 d89
d8a 0 4f :4 0
14e :4 0 52 :2 0
53 :3 0 6d6 d8e
d90 :3 0 52 :2 0
6a :4 0 6d9 d92
d94 :3 0 52 :2 0
6b :4 0 6dc d96
d98 :3 0 52 :2 0
64 :3 0 6df d9a
d9c :3 0 52 :2 0
6c :4 0 6e2 d9e
da0 :3 0 52 :2 0
6d :3 0 6e :3 0
5c :2 0 6f :2 0
6e5 da3 da7 6e9
da2 da9 :3 0 6ec
d8b dab :2 0 dad
6ef dae d88 dad
0 daf 6f1 0
db0 6f3 db3 :3 0
db3 0 db3 db2
db0 db1 :6 0 db5
30 :3 0 6f6 db7
6f8 db6 db5 :2 0
db8 6fa :2 0 dba
0 dba db9 d67
db8 :6 0 dbc 29
:3 0 6fd dbd d47
dbc 0 dbf bcd
d3e 0 dbf 6ff
0 dc0 702 dc2
b5 :4 0 dc0 :4 0
1311 cf :3 0 41
:4 0 dc6 :2 0 dcc
dc4 0 8 :3 0
63 :3 0 dc7 dc8
0 64 :3 0 dc9
dca 0 dcc 707
e1f 65 :3 0 8
:3 0 63 :3 0 dce
dcf 0 64 :3 0
dd0 dd1 0 dd3
70a dd5 70c dd4
dd3 :2 0 e1d 66
:3 0 8 :3 0 63
:3 0 dd8 dd9 0
64 :3 0 dda ddb
0 e15 4a :3 0
4b :3 0 ddd dde
0 4a :3 0 5b
:3 0 de0 de1 0
67 :2 0 2 :4 0
14f :2 0 5e :2 0
5c :2 0 70e de6
de8 :3 0 711 de2
dea ddf dec deb
:2 0 4d :3 0 5f
:3 0 dee def 0
4f :4 0 150 :4 0
52 :2 0 53 :3 0
715 df3 df5 :3 0
52 :2 0 6a :4 0
718 df7 df9 :3 0
52 :2 0 6b :4 0
71b dfb dfd :3 0
52 :2 0 64 :3 0
71e dff e01 :3 0
52 :2 0 6c :4 0
721 e03 e05 :3 0
52 :2 0 6d :3 0
6e :3 0 5c :2 0
6f :2 0 724 e08
e0c 728 e07 e0e
:3 0 72b df0 e10
:2 0 e12 72e e13
ded e12 0 e14
730 0 e15 732
e18 :3 0 e18 0
e18 e17 e15 e16
:6 0 e1a 32 :3 0
735 e1c 737 e1b
e1a :2 0 e1d 739
:2 0 e1f 0 e1f
e1e dcc e1d :6 0
1311 16 :3 0 45
:3 0 3c :3 0 151
:3 0 18 :3 0 39
:3 0 10 :3 0 151
:3 0 16 :4 0 152
1 :8 0 e3f b1
:4 0 e2a :3 0 7f
:3 0 70 :2 0 73e
e2d e2e :3 0 8
:3 0 63 :3 0 e30
e31 0 1d :2 0
e32 e33 0 e35
741 e3c 8 :3 0
63 :3 0 e36 e37
0 74 :2 0 e38
e39 0 e3b 743
e3d e2f e35 0
e3e 0 e3b 0
e3e 745 0 e3f
748 e92 65 :3 0
8 :3 0 63 :3 0
e41 e42 0 64
:3 0 e43 e44 0
e46 74b e48 74d
e47 e46 :2 0 e90
66 :3 0 8 :3 0
63 :3 0 e4b e4c
0 64 :3 0 e4d
e4e 0 e88 4a
:3 0 4b :3 0 e50
e51 0 4a :3 0
5b :3 0 e53 e54
0 67 :2 0 2
:4 0 153 :2 0 5e
:2 0 5c :2 0 74f
e59 e5b :3 0 752
e55 e5d e52 e5f
e5e :2 0 4d :3 0
5f :3 0 e61 e62
0 4f :4 0 154
:4 0 52 :2 0 53
:3 0 756 e66 e68
:3 0 52 :2 0 6a
:4 0 759 e6a e6c
:3 0 52 :2 0 6b
:4 0 75c e6e e70
:3 0 52 :2 0 64
:3 0 75f e72 e74
:3 0 52 :2 0 6c
:4 0 762 e76 e78
:3 0 52 :2 0 6d
:3 0 6e :3 0 5c
:2 0 6f :2 0 765
e7b e7f 769 e7a
e81 :3 0 76c e63
e83 :2 0 e85 76f
e86 e60 e85 0
e87 771 0 e88
773 e8b :3 0 e8b
0 e8b e8a e88
e89 :6 0 e8d 34
:3 0 776 e8f 778
e8e e8d :2 0 e90
77a :2 0 e92 0
e92 e91 e3f e90
:6 0 1311 16 :3 0
57 :3 0 26 :3 0
1d :2 0 77d e94
e97 70 :2 0 1d
:2 0 782 e99 e9b
:3 0 35 :3 0 3c
:3 0 155 :3 0 d
:3 0 156 :3 0 157
:3 0 d :3 0 158
:3 0 92 :3 0 93
:3 0 94 :3 0 95
:3 0 96 :3 0 97
:3 0 b9 :3 0 ba
:3 0 bb :3 0 93
:3 0 bc :3 0 95
:3 0 bd :3 0 ba
:3 0 be :3 0 bf
:3 0 bf :3 0 be
:3 0 c0 :3 0 c1
:3 0 c1 :3 0 c0
:3 0 9d :3 0 ba
:3 0 c2 :3 0 9d
:3 0 ba :3 0 c4
:3 0 c5 :3 0 c6
:3 0 c3 :3 0 9d
:3 0 ba :3 0 c7
:3 0 c8 :3 0 93
:3 0 95 :3 0 97
:3 0 ba :3 0 9a
:3 0 9b :3 0 9e
:3 0 3e :3 0 33
:3 0 16 :3 0 b9
:3 0 ba :4 0 159
1 :8 0 eea b1
:4 0 ed5 :3 0 7f
:3 0 70 :2 0 787
ed8 ed9 :3 0 8
:3 0 63 :3 0 edb
edc 0 1d :2 0
edd ede 0 ee0
78a ee7 8 :3 0
63 :3 0 ee1 ee2
0 74 :2 0 ee3
ee4 0 ee6 78c
ee8 eda ee0 0
ee9 0 ee6 0
ee9 78e 0 eea
791 f3d 65 :3 0
8 :3 0 63 :3 0
eec eed 0 64
:3 0 eee eef 0
ef1 794 ef3 796
ef2 ef1 :2 0 f3b
66 :3 0 8 :3 0
63 :3 0 ef6 ef7
0 64 :3 0 ef8
ef9 0 f33 4a
:3 0 4b :3 0 efb
efc 0 4a :3 0
5b :3 0 efe eff
0 67 :2 0 2
:4 0 15a :2 0 5e
:2 0 5c :2 0 798
f04 f06 :3 0 79b
f00 f08 efd f0a
f09 :2 0 4d :3 0
5f :3 0 f0c f0d
0 4f :4 0 15b
:4 0 52 :2 0 53
:3 0 79f f11 f13
:3 0 52 :2 0 6a
:4 0 7a2 f15 f17
:3 0 52 :2 0 6b
:4 0 7a5 f19 f1b
:3 0 52 :2 0 64
:3 0 7a8 f1d f1f
:3 0 52 :2 0 6c
:4 0 7ab f21 f23
:3 0 52 :2 0 6d
:3 0 6e :3 0 5c
:2 0 6f :2 0 7ae
f26 f2a 7b2 f25
f2c :3 0 7b5 f0e
f2e :2 0 f30 7b8
f31 f0b f30 0
f32 7ba 0 f33
7bc f36 :3 0 f36
0 f36 f35 f33
f34 :6 0 f38 36
:3 0 7bf f3a 7c1
f39 f38 :2 0 f3b
7c3 :2 0 f3d 0
f3d f3c eea f3b
:6 0 f58 16 :3 0
57 :3 0 1f :3 0
1d :2 0 7c6 f3f
f42 70 :2 0 d7
:2 0 7cb f44 f46
:3 0 28 :3 0 8e
:2 0 f48 f49 0
f55 29 :3 0 15c
:3 0 10 :3 0 18
:3 0 28 :3 0 1d
:2 0 1d :2 0 7ce
f4c f52 f4b f53
0 f55 7d4 f56
f47 f55 0 f57
7d7 0 f58 7d9
130a b4 :3 0 47
:4 0 f5c :2 0 1309
f5a f5d :3 0 b5
:3 0 47 :3 0 15
:3 0 17 :4 0 f63
:2 0 1098 f5f f64
:3 0 7dc :3 0 47
:3 0 b1 :3 0 f65
f66 :3 0 8 :3 0
63 :3 0 f68 f69
0 1d :2 0 f6a
f6b 0 f6d 7df
f74 8 :3 0 63
:3 0 f6e f6f 0
74 :2 0 f70 f71
0 f73 7e1 f75
f67 f6d 0 f76
0 f73 0 f76
7e3 0 1098 8
:3 0 63 :3 0 f77
f78 0 8 :3 0
b6 :2 0 b7 :3 0
f7a f7c 0 7e8
f7b f7e :3 0 f7f
:2 0 b8 :8 0 f83
7eb f84 f80 f83
0 f85 7ed 0
1098 35 :3 0 3c
:3 0 155 :3 0 d
:3 0 156 :3 0 157
:3 0 d :3 0 158
:3 0 92 :3 0 93
:3 0 94 :3 0 95
:3 0 96 :3 0 97
:3 0 b9 :3 0 ba
:3 0 bb :3 0 93
:3 0 bc :3 0 95
:3 0 bd :3 0 ba
:3 0 be :3 0 bf
:3 0 bf :3 0 be
:3 0 c0 :3 0 c1
:3 0 c1 :3 0 c0
:3 0 9d :3 0 ba
:3 0 c2 :3 0 9d
:3 0 ba :3 0 c4
:3 0 c5 :3 0 c6
:3 0 c3 :3 0 9d
:3 0 ba :3 0 c7
:3 0 c8 :3 0 93
:3 0 95 :3 0 97
:3 0 ba :3 0 9a
:3 0 9b :3 0 9e
:3 0 3e :3 0 18
:3 0 33 :3 0 17
:4 0 15d 1 :8 0
fd2 b1 :4 0 fbd
:3 0 7f :3 0 70
:2 0 7f1 fc0 fc1
:3 0 8 :3 0 63
:3 0 fc3 fc4 0
1d :2 0 fc5 fc6
0 fc8 7f4 fcf
8 :3 0 63 :3 0
fc9 fca 0 74
:2 0 fcb fcc 0
fce 7f6 fd0 fc2
fc8 0 fd1 0
fce 0 fd1 7f8
0 fd2 7fb 1025
65 :3 0 8 :3 0
63 :3 0 fd4 fd5
0 64 :3 0 fd6
fd7 0 fd9 7fe
fdb 800 fda fd9
:2 0 1023 66 :3 0
8 :3 0 63 :3 0
fde fdf 0 64
:3 0 fe0 fe1 0
101b 4a :3 0 4b
:3 0 fe3 fe4 0
4a :3 0 5b :3 0
fe6 fe7 0 67
:2 0 2 :4 0 15e
:2 0 5e :2 0 5c
:2 0 802 fec fee
:3 0 805 fe8 ff0
fe5 ff2 ff1 :2 0
4d :3 0 5f :3 0
ff4 ff5 0 4f
:4 0 15f :4 0 52
:2 0 53 :3 0 809
ff9 ffb :3 0 52
:2 0 6a :4 0 80c
ffd fff :3 0 52
:2 0 6b :4 0 80f
1001 1003 :3 0 52
:2 0 64 :3 0 812
1005 1007 :3 0 52
:2 0 6c :4 0 815
1009 100b :3 0 52
:2 0 6d :3 0 6e
:3 0 5c :2 0 6f
:2 0 818 100e 1012
81c 100d 1014 :3 0
81f ff6 1016 :2 0
1018 822 1019 ff3
1018 0 101a 824
0 101b 826 101e
:3 0 101e 0 101e
101d 101b 101c :6 0
1020 39 :3 0 829
1022 82b 1021 1020
:2 0 1023 82d :2 0
1025 0 1025 1024
fd2 1023 :6 0 1098
38 :3 0 37 :3 0
d2 :3 0 d :3 0
d3 :3 0 3a :3 0
15 :4 0 160 1
:8 0 1043 b1 :4 0
102e :3 0 7f :3 0
70 :2 0 832 1031
1032 :3 0 8 :3 0
63 :3 0 1034 1035
0 1d :2 0 1036
1037 0 1039 835
1040 8 :3 0 63
:3 0 103a 103b 0
74 :2 0 103c 103d
0 103f 837 1041
1033 1039 0 1042
0 103f 0 1042
839 0 1043 83c
1096 65 :3 0 8
:3 0 63 :3 0 1045
1046 0 64 :3 0
1047 1048 0 104a
83f 104c 841 104b
104a :2 0 1094 66
:3 0 8 :3 0 63
:3 0 104f 1050 0
64 :3 0 1051 1052
0 108c 4a :3 0
4b :3 0 1054 1055
0 4a :3 0 5b
:3 0 1057 1058 0
67 :2 0 2 :4 0
161 :2 0 5e :2 0
5c :2 0 843 105d
105f :3 0 846 1059
1061 1056 1063 1062
:2 0 4d :3 0 5f
:3 0 1065 1066 0
4f :4 0 162 :4 0
52 :2 0 53 :3 0
84a 106a 106c :3 0
52 :2 0 6a :4 0
84d 106e 1070 :3 0
52 :2 0 6b :4 0
850 1072 1074 :3 0
52 :2 0 64 :3 0
853 1076 1078 :3 0
52 :2 0 6c :4 0
856 107a 107c :3 0
52 :2 0 6d :3 0
6e :3 0 5c :2 0
6f :2 0 859 107f
1083 85d 107e 1085
:3 0 860 1067 1087
:2 0 1089 863 108a
1064 1089 0 108b
865 0 108c 867
108f :3 0 108f 0
108f 108e 108c 108d
:6 0 1091 3b :3 0
86a 1093 86c 1092
1091 :2 0 1094 86e
:2 0 1096 0 1096
1095 1043 1094 :6 0
1098 38 :3 0 871
109a b5 :4 0 1098
:4 0 1309 cf :3 0
47 :4 0 109e :2 0
10a4 109c 0 8
:3 0 63 :3 0 109f
10a0 0 64 :3 0
10a1 10a2 0 10a4
877 10f7 65 :3 0
8 :3 0 63 :3 0
10a6 10a7 0 64
:3 0 10a8 10a9 0
10ab 87a 10ad 87c
10ac 10ab :2 0 10f5
66 :3 0 8 :3 0
63 :3 0 10b0 10b1
0 64 :3 0 10b2
10b3 0 10ed 4a
:3 0 4b :3 0 10b5
10b6 0 4a :3 0
5b :3 0 10b8 10b9
0 67 :2 0 2
:4 0 163 :2 0 5e
:2 0 5c :2 0 87e
10be 10c0 :3 0 881
10ba 10c2 10b7 10c4
10c3 :2 0 4d :3 0
5f :3 0 10c6 10c7
0 4f :4 0 164
:4 0 52 :2 0 53
:3 0 885 10cb 10cd
:3 0 52 :2 0 6a
:4 0 888 10cf 10d1
:3 0 52 :2 0 6b
:4 0 88b 10d3 10d5
:3 0 52 :2 0 64
:3 0 88e 10d7 10d9
:3 0 52 :2 0 6c
:4 0 891 10db 10dd
:3 0 52 :2 0 6d
:3 0 6e :3 0 5c
:2 0 6f :2 0 894
10e0 10e4 898 10df
10e6 :3 0 89b 10c8
10e8 :2 0 10ea 89e
10eb 10c5 10ea 0
10ec 8a0 0 10ed
8a2 10f0 :3 0 10f0
0 10f0 10ef 10ed
10ee :6 0 10f2 3d
:3 0 8a5 10f4 8a7
10f3 10f2 :2 0 10f5
8a9 :2 0 10f7 0
10f7 10f6 10a4 10f5
:6 0 1309 16 :3 0
57 :3 0 26 :3 0
1d :2 0 8ac 10f9
10fc 70 :2 0 5c
:2 0 8b1 10fe 1100
:3 0 1101 :2 0 57
:3 0 27 :3 0 1d
:2 0 8b4 1103 1106
165 :2 0 1d :2 0
8b9 1108 110a :3 0
110b :2 0 1102 110d
110c :2 0 35 :3 0
12d :3 0 27 :3 0
33 :3 0 18 :4 0
166 1 :8 0 112a
b1 :4 0 1115 :3 0
7f :3 0 70 :2 0
8be 1118 1119 :3 0
8 :3 0 63 :3 0
111b 111c 0 1d
:2 0 111d 111e 0
1120 8c1 1127 8
:3 0 63 :3 0 1121
1122 0 74 :2 0
1123 1124 0 1126
8c3 1128 111a 1120
0 1129 0 1126
0 1129 8c5 0
112a 8c8 117d 65
:3 0 8 :3 0 63
:3 0 112c 112d 0
64 :3 0 112e 112f
0 1131 8cb 1133
8cd 1132 1131 :2 0
117b 66 :3 0 8
:3 0 63 :3 0 1136
1137 0 64 :3 0
1138 1139 0 1173
4a :3 0 4b :3 0
113b 113c 0 4a
:3 0 5b :3 0 113e
113f 0 67 :2 0
2 :4 0 167 :2 0
5e :2 0 5c :2 0
8cf 1144 1146 :3 0
8d2 1140 1148 113d
114a 1149 :2 0 4d
:3 0 5f :3 0 114c
114d 0 4f :4 0
168 :4 0 52 :2 0
53 :3 0 8d6 1151
1153 :3 0 52 :2 0
6a :4 0 8d9 1155
1157 :3 0 52 :2 0
6b :4 0 8dc 1159
115b :3 0 52 :2 0
64 :3 0 8df 115d
115f :3 0 52 :2 0
6c :4 0 8e2 1161
1163 :3 0 52 :2 0
6d :3 0 6e :3 0
5c :2 0 6f :2 0
8e5 1166 116a 8e9
1165 116c :3 0 8ec
114e 116e :2 0 1170
8ef 1171 114b 1170
0 1172 8f1 0
1173 8f3 1176 :3 0
1176 0 1176 1175
1173 1174 :6 0 1178
3f :3 0 8f6 117a
8f8 1179 1178 :2 0
117b 8fa :2 0 117d
0 117d 117c 112a
117b :6 0 117f 16
:3 0 8fd 1180 110e
117f 0 1181 8ff
0 1309 28 :3 0
8e :2 0 1182 1183
0 1309 29 :3 0
15c :3 0 10 :3 0
18 :3 0 28 :3 0
1d :2 0 1d :2 0
901 1186 118c 1185
118d 0 1309 3e
:3 0 169 :3 0 92
:3 0 94 :3 0 c3
:3 0 c4 :3 0 c7
:3 0 18 :3 0 1c
:3 0 19 :3 0 1b
:3 0 1e :3 0 2a
:3 0 2d :3 0 35
:3 0 33 :3 0 18
:4 0 16a 1 :8 0
11a6 8 :3 0 63
:3 0 11a1 11a2 0
64 :3 0 11a3 11a4
0 11a6 907 11f9
65 :3 0 8 :3 0
63 :3 0 11a8 11a9
0 64 :3 0 11aa
11ab 0 11ad 90a
11af 90c 11ae 11ad
:2 0 11f7 66 :3 0
8 :3 0 63 :3 0
11b2 11b3 0 64
:3 0 11b4 11b5 0
11ef 4a :3 0 4b
:3 0 11b7 11b8 0
4a :3 0 5b :3 0
11ba 11bb 0 67
:2 0 2 :4 0 16b
:2 0 5e :2 0 5c
:2 0 90e 11c0 11c2
:3 0 911 11bc 11c4
11b9 11c6 11c5 :2 0
4d :3 0 5f :3 0
11c8 11c9 0 4f
:4 0 16c :4 0 52
:2 0 53 :3 0 915
11cd 11cf :3 0 52
:2 0 6a :4 0 918
11d1 11d3 :3 0 52
:2 0 6b :4 0 91b
11d5 11d7 :3 0 52
:2 0 64 :3 0 91e
11d9 11db :3 0 52
:2 0 6c :4 0 921
11dd 11df :3 0 52
:2 0 6d :3 0 6e
:3 0 5c :2 0 6f
:2 0 924 11e2 11e6
928 11e1 11e8 :3 0
92b 11ca 11ea :2 0
11ec 92e 11ed 11c7
11ec 0 11ee 930
0 11ef 932 11f2
:3 0 11f2 0 11f2
11f1 11ef 11f0 :6 0
11f4 41 :3 0 935
11f6 937 11f5 11f4
:2 0 11f7 939 :2 0
11f9 0 11f9 11f8
11a6 11f7 :6 0 1309
16 :3 0 57 :3 0
1e :3 0 1d :2 0
93c 11fb 11fe 70
:2 0 1d :2 0 941
1200 1202 :3 0 105
:3 0 10a :3 0 10f
:3 0 114 :3 0 119
:3 0 11e :3 0 109
:3 0 19 :3 0 10e
:3 0 1b :3 0 113
:3 0 1c :3 0 118
:3 0 1c :3 0 2a
:3 0 11d :3 0 39
:3 0 10 :3 0 34
:3 0 14 :4 0 16d
1 :8 0 122e b1
:4 0 1219 :3 0 7f
:3 0 70 :2 0 946
121c 121d :3 0 8
:3 0 63 :3 0 121f
1220 0 1d :2 0
1221 1222 0 1224
949 122b 8 :3 0
63 :3 0 1225 1226
0 74 :2 0 1227
1228 0 122a 94b
122c 121e 1224 0
122d 0 122a 0
122d 94d 0 122e
950 1281 65 :3 0
8 :3 0 63 :3 0
1230 1231 0 64
:3 0 1232 1233 0
1235 953 1237 955
1236 1235 :2 0 127f
66 :3 0 8 :3 0
63 :3 0 123a 123b
0 64 :3 0 123c
123d 0 1277 4a
:3 0 4b :3 0 123f
1240 0 4a :3 0
5b :3 0 1242 1243
0 67 :2 0 2
:4 0 16e :2 0 5e
:2 0 5c :2 0 957
1248 124a :3 0 95a
1244 124c 1241 124e
124d :2 0 4d :3 0
5f :3 0 1250 1251
0 4f :4 0 16f
:4 0 52 :2 0 53
:3 0 95e 1255 1257
:3 0 52 :2 0 6a
:4 0 961 1259 125b
:3 0 52 :2 0 6b
:4 0 964 125d 125f
:3 0 52 :2 0 64
:3 0 967 1261 1263
:3 0 52 :2 0 6c
:4 0 96a 1265 1267
:3 0 52 :2 0 6d
:3 0 6e :3 0 5c
:2 0 6f :2 0 96d
126a 126e 971 1269
1270 :3 0 974 1252
1272 :2 0 1274 977
1275 124f 1274 0
1276 979 0 1277
97b 127a :3 0 127a
0 127a 1279 1277
1278 :6 0 127c 43
:3 0 97e 127e 980
127d 127c :2 0 127f
982 :2 0 1281 0
1281 1280 122e 127f
:6 0 1283 16 :3 0
985 1306 105 :3 0
10a :3 0 10f :3 0
114 :3 0 119 :3 0
11e :3 0 109 :3 0
19 :3 0 10e :3 0
1b :3 0 113 :3 0
1c :3 0 118 :3 0
1c :3 0 2a :3 0
11d :3 0 1c :3 0
2d :3 0 39 :3 0
10 :3 0 34 :3 0
14 :4 0 170 1
:8 0 12b0 b1 :4 0
129b :3 0 7f :3 0
70 :2 0 989 129e
129f :3 0 8 :3 0
63 :3 0 12a1 12a2
0 1d :2 0 12a3
12a4 0 12a6 98c
12ad 8 :3 0 63
:3 0 12a7 12a8 0
74 :2 0 12a9 12aa
0 12ac 98e 12ae
12a0 12a6 0 12af
0 12ac 0 12af
990 0 12b0 993
1303 65 :3 0 8
:3 0 63 :3 0 12b2
12b3 0 64 :3 0
12b4 12b5 0 12b7
996 12b9 998 12b8
12b7 :2 0 1301 66
:3 0 8 :3 0 63
:3 0 12bc 12bd 0
64 :3 0 12be 12bf
0 12f9 4a :3 0
4b :3 0 12c1 12c2
0 4a :3 0 5b
:3 0 12c4 12c5 0
67 :2 0 2 :4 0
171 :2 0 5e :2 0
5c :2 0 99a 12ca
12cc :3 0 99d 12c6
12ce 12c3 12d0 12cf
:2 0 4d :3 0 5f
:3 0 12d2 12d3 0
4f :4 0 172 :4 0
52 :2 0 53 :3 0
9a1 12d7 12d9 :3 0
52 :2 0 6a :4 0
9a4 12db 12dd :3 0
52 :2 0 6b :4 0
9a7 12df 12e1 :3 0
52 :2 0 64 :3 0
9aa 12e3 12e5 :3 0
52 :2 0 6c :4 0
9ad 12e7 12e9 :3 0
52 :2 0 6d :3 0
6e :3 0 5c :2 0
6f :2 0 9b0 12ec
12f0 9b4 12eb 12f2
:3 0 9b7 12d4 12f4
:2 0 12f6 9ba 12f7
12d1 12f6 0 12f8
9bc 0 12f9 9be
12fc :3 0 12fc 0
12fc 12fb 12f9 12fa
:6 0 12fe 45 :3 0
9c1 1300 9c3 12ff
12fe :2 0 1301 9c5
:2 0 1303 0 1303
1302 12b0 1301 :6 0
1305 16 :3 0 9c8
1307 1203 1283 0
1308 0 1305 0
1308 9ca 0 1309
9cd 130b e9c f58
0 130c 0 1309
0 130c 9d6 0
1311 12c :5 0 130f
47 0 1310 9d9
130f 130e 1311 9db
1313 9e7 1312 1311
:2 0 1317 0 1315
9e9 1316 0 1315
:2 0 1317 9eb :2 0
1318 6cb 1317 0
138a 0 37 :3 0
d2 :3 0 d :3 0
d3 :3 0 3a :3 0
15 :4 0 160 1
:8 0 1335 b1 :4 0
1320 :3 0 7f :3 0
70 :2 0 9f1 1323
1324 :3 0 8 :3 0
63 :3 0 1326 1327
0 1d :2 0 1328
1329 0 132b 9f4
1332 8 :3 0 63
:3 0 132c 132d 0
74 :2 0 132e 132f
0 1331 9f6 1333
1325 132b 0 1334
0 1331 0 1334
9f8 0 1335 9fb
1388 65 :3 0 8
:3 0 63 :3 0 1337
1338 0 64 :3 0
1339 133a 0 133c
9fe 133e a00 133d
133c :2 0 1386 66
:3 0 8 :3 0 63
:3 0 1341 1342 0
64 :3 0 1343 1344
0 137e 4a :3 0
4b :3 0 1346 1347
0 4a :3 0 5b
:3 0 1349 134a 0
67 :2 0 2 :4 0
173 :2 0 5e :2 0
5c :2 0 a02 134f
1351 :3 0 a05 134b
1353 1348 1355 1354
:2 0 4d :3 0 5f
:3 0 1357 1358 0
4f :4 0 174 :4 0
52 :2 0 53 :3 0
a09 135c 135e :3 0
52 :2 0 6a :4 0
a0c 1360 1362 :3 0
52 :2 0 6b :4 0
a0f 1364 1366 :3 0
52 :2 0 64 :3 0
a12 1368 136a :3 0
52 :2 0 6c :4 0
a15 136c 136e :3 0
52 :2 0 6d :3 0
6e :3 0 5c :2 0
6f :2 0 a18 1371
1375 a1c 1370 1377
:3 0 a1f 1359 1379
:2 0 137b a22 137c
1356 137b 0 137d
a24 0 137e a26
1381 :3 0 1381 0
1381 1380 137e 137f
:6 0 1383 47 :3 0
a29 1385 a2b 1384
1383 :2 0 1386 a2d
:2 0 1388 0 1388
1387 1335 1386 :6 0
138a 16 :3 0 a30
138c b5 :4 0 138a
:4 0 1544 cf :3 0
40 :4 0 1390 :2 0
1396 138e 0 8
:3 0 63 :3 0 1391
1392 0 64 :3 0
1393 1394 0 1396
a36 13e9 65 :3 0
8 :3 0 63 :3 0
1398 1399 0 64
:3 0 139a 139b 0
139d a39 139f a3b
139e 139d :2 0 13e7
66 :3 0 8 :3 0
63 :3 0 13a2 13a3
0 64 :3 0 13a4
13a5 0 13df 4a
:3 0 4b :3 0 13a7
13a8 0 4a :3 0
5b :3 0 13aa 13ab
0 67 :2 0 2
:4 0 175 :2 0 5e
:2 0 5c :2 0 a3d
13b0 13b2 :3 0 a40
13ac 13b4 13a9 13b6
13b5 :2 0 4d :3 0
5f :3 0 13b8 13b9
0 4f :4 0 176
:4 0 52 :2 0 53
:3 0 a44 13bd 13bf
:3 0 52 :2 0 6a
:4 0 a47 13c1 13c3
:3 0 52 :2 0 6b
:4 0 a4a 13c5 13c7
:3 0 52 :2 0 64
:3 0 a4d 13c9 13cb
:3 0 52 :2 0 6c
:4 0 a50 13cd 13cf
:3 0 52 :2 0 6d
:3 0 6e :3 0 5c
:2 0 6f :2 0 a53
13d2 13d6 a57 13d1
13d8 :3 0 a5a 13ba
13da :2 0 13dc a5d
13dd 13b7 13dc 0
13de a5f 0 13df
a61 13e2 :3 0 13e2
0 13e2 13e1 13df
13e0 :6 0 13e4 49
:3 0 a64 13e6 a66
13e5 13e4 :2 0 13e7
a68 :2 0 13e9 0
13e9 13e8 1396 13e7
:6 0 1544 1 :3 0
29 :3 0 177 :3 0
10 :3 0 a6b 13ec
13ee 13eb 13ef 0
1544 155 :3 0 d
:3 0 178 :3 0 157
:3 0 d :3 0 179
:3 0 8f :3 0 d
:3 0 90 :3 0 23
:3 0 37 :3 0 3a
:3 0 10 :4 0 17a
1 :8 0 1404 8
:3 0 63 :3 0 13ff
1400 0 64 :3 0
1401 1402 0 1404
a6d 1457 65 :3 0
8 :3 0 63 :3 0
1406 1407 0 64
:3 0 1408 1409 0
140b a70 140d a72
140c 140b :2 0 1455
66 :3 0 8 :3 0
63 :3 0 1410 1411
0 64 :3 0 1412
1413 0 144d 4a
:3 0 4b :3 0 1415
1416 0 4a :3 0
5b :3 0 1418 1419
0 67 :2 0 2
:4 0 17b :2 0 5e
:2 0 5c :2 0 a74
141e 1420 :3 0 a77
141a 1422 1417 1424
1423 :2 0 4d :3 0
5f :3 0 1426 1427
0 4f :4 0 17c
:4 0 52 :2 0 53
:3 0 a7b 142b 142d
:3 0 52 :2 0 6a
:4 0 a7e 142f 1431
:3 0 52 :2 0 6b
:4 0 a81 1433 1435
:3 0 52 :2 0 64
:3 0 a84 1437 1439
:3 0 52 :2 0 6c
:4 0 a87 143b 143d
:3 0 52 :2 0 6d
:3 0 6e :3 0 5c
:2 0 6f :2 0 a8a
1440 1444 a8e 143f
1446 :3 0 a91 1428
1448 :2 0 144a a94
144b 1425 144a 0
144c a96 0 144d
a98 1450 :3 0 1450
0 1450 144f 144d
144e :6 0 1452 4b
:3 0 a9b 1454 a9d
1453 1452 :2 0 1455
a9f :2 0 1457 0
1457 1456 1404 1455
:6 0 1544 1 :3 0
57 :3 0 23 :3 0
1d :2 0 aa2 1459
145c 70 :2 0 5c
:2 0 aa7 145e 1460
:3 0 24 :3 0 d
:3 0 90 :3 0 1463
1464 0 1462 1465
0 1471 8 :3 0
63 :3 0 1467 1468
0 17d :3 0 10
:3 0 23 :3 0 24
:3 0 aaa 146a 146e
1469 146f 0 1471
aae 1472 1461 1471
0 1473 ab1 0
1544 17e :3 0 17f
:3 0 8f :3 0 37
:3 0 3a :3 0 10
:3 0 3a :3 0 10
:4 0 180 1 :8 0
1492 b1 :4 0 147d
:3 0 7f :3 0 70
:2 0 ab5 1480 1481
:3 0 8 :3 0 63
:3 0 1483 1484 0
1d :2 0 1485 1486
0 1488 ab8 148f
8 :3 0 63 :3 0
1489 148a 0 74
:2 0 148b 148c 0
148e aba 1490 1482
1488 0 1491 0
148e 0 1491 abc
0 1492 abf 14e5
65 :3 0 8 :3 0
63 :3 0 1494 1495
0 64 :3 0 1496
1497 0 1499 ac2
149b ac4 149a 1499
:2 0 14e3 66 :3 0
8 :3 0 63 :3 0
149e 149f 0 64
:3 0 14a0 14a1 0
14db 4a :3 0 4b
:3 0 14a3 14a4 0
4a :3 0 5b :3 0
14a6 14a7 0 67
:2 0 2 :4 0 181
:2 0 5e :2 0 5c
:2 0 ac6 14ac 14ae
:3 0 ac9 14a8 14b0
14a5 14b2 14b1 :2 0
4d :3 0 5f :3 0
14b4 14b5 0 4f
:4 0 182 :4 0 52
:2 0 53 :3 0 acd
14b9 14bb :3 0 52
:2 0 6a :4 0 ad0
14bd 14bf :3 0 52
:2 0 6b :4 0 ad3
14c1 14c3 :3 0 52
:2 0 64 :3 0 ad6
14c5 14c7 :3 0 52
:2 0 6c :4 0 ad9
14c9 14cb :3 0 52
:2 0 6d :3 0 6e
:3 0 5c :2 0 6f
:2 0 adc 14ce 14d2
ae0 14cd 14d4 :3 0
ae3 14b6 14d6 :2 0
14d8 ae6 14d9 14b3
14d8 0 14da ae8
0 14db aea 14de
:3 0 14de 0 14de
14dd 14db 14dc :6 0
14e0 4d :3 0 aed
14e2 aef 14e1 14e0
:2 0 14e3 af1 :2 0
14e5 0 14e5 14e4
1492 14e3 :6 0 1544
1 :3 0 8 :3 0
63 :3 0 14e7 14e8
0 70 :2 0 74
:2 0 af6 14ea 14ec
:3 0 8 :3 0 63
:3 0 14ee 14ef 0
1d :2 0 14f0 14f1
0 14f3 af9 14f4
14ed 14f3 0 14f5
afb 0 1544 4a
:3 0 4b :3 0 14f6
14f7 0 4a :3 0
5b :3 0 14f9 14fa
0 183 :2 0 2
:4 0 184 :2 0 5e
:2 0 5c :2 0 afd
14ff 1501 :3 0 b00
14fb 1503 14f8 1505
1504 :2 0 4d :3 0
5f :3 0 1507 1508
0 4f :4 0 185
:4 0 52 :2 0 53
:3 0 b04 150c 150e
:3 0 52 :2 0 6a
:4 0 b07 1510 1512
:3 0 52 :2 0 79
:4 0 b0a 1514 1516
:3 0 52 :2 0 57
:3 0 58 :3 0 8
:3 0 63 :3 0 151b
151c 0 b0d 151a
151e 59 :4 0 b0f
1519 1521 b12 1518
1523 :3 0 52 :2 0
84 :4 0 b15 1525
1527 :3 0 52 :2 0
55 :4 0 b18 1529
152b :3 0 52 :2 0
3 :3 0 b1b 152d
152f :3 0 52 :2 0
55 :4 0 b1e 1531
1533 :3 0 b21 1509
1535 :2 0 1537 b24
1538 1506 1537 0
1539 b26 0 1544
7d :3 0 186 :3 0
153a 153b 0 8
:3 0 63 :3 0 153d
153e 0 3 :3 0
7f :3 0 b28 153c
1542 :2 0 1544 b2c
15e5 8 :3 0 187
:2 0 4 1545 1546
0 4a :3 0 4b
:3 0 1548 1549 0
4a :3 0 5b :3 0
154b 154c 0 67
:2 0 2 :4 0 188
:2 0 5e :2 0 5c
:2 0 b3d 1551 1553
:3 0 b40 154d 1555
154a 1557 1556 :2 0
4d :3 0 5f :3 0
1559 155a 0 4f
:4 0 189 :4 0 52
:2 0 53 :3 0 b44
155e 1560 :3 0 b47
155b 1562 :2 0 157b
4d :3 0 5f :3 0
1564 1565 0 4f
:4 0 18a :4 0 52
:2 0 8 :3 0 18b
:3 0 156a 156b 0
b4a 1569 156d :3 0
52 :2 0 84 :4 0
b4d 156f 1571 :3 0
52 :2 0 8 :3 0
18c :3 0 1574 1575
0 b50 1573 1577
:3 0 b53 1566 1579
:2 0 157b b56 157c
1558 157b 0 157d
b59 0 158d b4
:3 0 49 :3 0 8
:3 0 18b :3 0 18d
:3 0 8 :3 0 18c
:3 0 3 :3 0 18e
:4 0 18f 1 :a 0
1589 :2 0 158d 157f
1587 :2 0 7 :3 0
49 :3 0 158b :2 0
158d b5b 158f b5f
158e 158d :2 0 15e3
66 :3 0 8 :3 0
63 :3 0 1592 1593
0 64 :3 0 1594
1595 0 15db 4a
:3 0 4b :3 0 1597
1598 0 4a :3 0
5b :3 0 159a 159b
0 67 :2 0 2
:4 0 190 :2 0 5e
:2 0 5c :2 0 b61
15a0 15a2 :3 0 b64
159c 15a4 1599 15a6
15a5 :2 0 4d :3 0
5f :3 0 15a8 15a9
0 4f :4 0 191
:4 0 52 :2 0 53
:3 0 b68 15ad 15af
:3 0 b6b 15aa 15b1
:2 0 15cb 4d :3 0
5f :3 0 15b3 15b4
0 4f :4 0 6b
:4 0 52 :2 0 64
:3 0 b6e 15b8 15ba
:3 0 52 :2 0 6c
:4 0 b71 15bc 15be
:3 0 52 :2 0 6d
:3 0 6e :3 0 5c
:2 0 6f :2 0 b74
15c1 15c5 b78 15c0
15c7 :3 0 b7b 15b5
15c9 :2 0 15cb b7e
15cc 15a7 15cb 0
15cd b81 0 15db
b4 :3 0 49 :3 0
18d :3 0 8 :3 0
18c :3 0 3 :3 0
18e :4 0 192 1
:a 0 15d7 :2 0 15db
15cf 15d5 :2 0 7
:3 0 49 :3 0 15d9
:2 0 15db b83 15de
:3 0 15de 0 15de
15dd 15db 15dc :6 0
15e0 1 :3 0 b88
15e2 b8a 15e1 15e0
:2 0 15e3 b8c :2 0
15e5 b8f 15e5 15e4
1544 15e3 :6 0 15e6
:2 0 2 10 15e5
15e9 :3 0 15e8 15e6
15ea :8 0 
bb0
4
:3 0 1 3 1
7 2 6 a
1 14 1 12
1 1e 1 1c
1 28 1 26
1 2d 1 34
1 32 1 3b
1 39 1 42
1 40 1 49
1 47 1 50
1 4e 1 55
1 5a 1 5f
1 65 1 6b
1 71 1 77
1 7d 1 83
1 89 1 8f
1 95 1 9b
1 a1 1 a7
1 ad 1 b3
1 bb 1 b9
1 df 1 e5
1 fa 1 10b
1 111 1 11e
1 12a 2 12d
12f 2 132 134
2 136 138 2
13a 13c 2 13e
140 1 146 2
148 149 2 142
14b 2 14d 14f
4 126 12c 131
151 1 153 1
155 2 15f 161
3 15d 15e 163
2 16c 16e 2
16b 170 1 172
1 174 2 182
187 1 18e 1
189 2 1a1 1a3
3 19f 1a0 1a5
2 1ae 1b0 2
1b2 1b4 2 1b6
1b8 2 1ba 1bc
2 1be 1c0 3
1c5 1c6 1c7 2
1c2 1c9 2 1ad
1cb 1 1cd 1
1cf 2 198 1d0
1 1d4 1 193
2 191 1d8 1
1e3 2 1df 1e3
2 1f3 1f8 1
1ff 1 1fa 2
212 214 3 210
211 216 2 21f
221 2 223 225
2 227 229 2
22b 22d 2 22f
231 3 236 237
238 2 233 23a
2 21e 23c 1
23e 1 240 2
209 241 1 245
1 204 2 202
249 1 254 2
250 254 2 260
262 3 25e 25f
264 2 26d 26f
2 271 273 2
275 277 1 27f
2 281 282 2
279 284 2 26c
286 1 288 1
28a 2 294 296
3 292 293 298
2 2a1 2a3 2
2a0 2a5 1 2a7
1 2a9 2 2b0
2b1 3 28b 2aa
2b3 2 2b7 2b8
1 2bc 2 2ba
2bc 2 2c1 2c2
1 2c6 2 2c4
2c6 2 2d4 2d6
3 2d2 2d3 2d8
2 2e1 2e3 2
2e5 2e7 2 2e9
2eb 1 2f3 2
2f5 2f6 2 2ed
2f8 2 2fa 2fc
2 2fe 300 2
302 304 2 306
308 2 2e0 30a
1 30c 1 30e
2 318 31a 3
316 317 31c 2
325 327 2 324
329 1 32b 1
32d 3 334 335
336 3 30f 32e
338 2 33b 33a
2 343 348 1
34f 1 34a 2
362 364 3 360
361 366 2 36f
371 2 373 375
2 377 379 2
37b 37d 2 37f
381 3 386 387
388 2 383 38a
2 36e 38c 1
38e 1 390 2
359 391 1 395
1 354 2 352
399 2 39f 3a1
2 3a3 3a5 2
3a7 3a9 4 24c
33c 39c 3ac 2
3b2 3b3 1 3b7
2 3b5 3b7 1
401 2 400 401
1 409 1 40f
2 411 412 2
3fe 413 1 41a
1 415 2 42d
42f 3 42b 42c
431 2 43a 43c
2 43e 440 2
442 444 2 446
448 2 44a 44c
3 451 452 453
2 44e 455 2
439 457 1 459
1 45b 2 424
45c 1 460 1
41f 2 41d 464
2 470 471 1
47c 1 482 2
484 485 1 48d
2 489 48d 1
492 1 494 1
4d2 2 4d1 4d2
1 4da 1 4e0
2 4e2 4e3 2
4cf 4e4 1 4eb
1 4e6 2 4fe
500 3 4fc 4fd
502 2 50b 50d
2 50f 511 2
513 515 2 517
519 2 51b 51d
3 522 523 524
2 51f 526 2
50a 528 1 52a
1 52c 2 4f5
52d 1 531 1
4f0 2 4ee 535
1 543 2 542
543 1 54b 1
551 2 553 554
2 540 555 1
55c 1 557 2
56f 571 3 56d
56e 573 2 57c
57e 2 580 582
2 584 586 2
588 58a 2 58c
58e 3 593 594
595 2 590 597
2 57b 599 1
59b 1 59d 2
566 59e 1 5a2
1 561 2 55f
5a6 5 472 486
495 538 5a9 2
5b0 5b6 1 5bd
1 5b8 2 5d0
5d2 3 5ce 5cf
5d4 2 5dd 5df
2 5e1 5e3 2
5e5 5e7 2 5e9
5eb 2 5ed 5ef
3 5f4 5f5 5f6
2 5f1 5f8 2
5dc 5fa 1 5fc
1 5fe 2 5c7
5ff 1 603 1
5c2 2 5c0 607
4 467 46b 5ad
60a 1 60d 1
61b 2 61a 61b
1 623 1 629
2 62b 62c 2
618 62d 1 634
1 62f 2 647
649 3 645 646
64b 2 654 656
2 658 65a 2
65c 65e 2 660
662 2 664 666
3 66b 66c 66d
2 668 66f 2
653 671 1 673
1 675 2 63e
676 1 67a 1
639 2 637 67e
2 68a 68c 2
68e 690 2 692
694 6 3b0 60e
681 685 688 697
2 699 69a 4
6a3 6a4 6a5 6a6
1 6b1 1 6b7
2 6b9 6ba 1
6c2 2 6be 6c2
1 6c7 1 6c9
1 6ee 2 6ed
6ee 1 6f6 1
6fc 2 6fe 6ff
2 6eb 700 1
707 1 702 2
71a 71c 3 718
719 71e 2 727
729 2 72b 72d
2 72f 731 2
733 735 2 737
739 3 73e 73f
740 2 73b 742
2 726 744 1
746 1 748 2
711 749 1 74d
1 70c 2 70a
751 1 7bb 2
7ba 7bb 1 7c3
1 7c9 2 7cb
7cc 2 7b8 7cd
1 7d4 1 7cf
2 7e7 7e9 3
7e5 7e6 7eb 2
7f4 7f6 2 7f8
7fa 2 7fc 7fe
2 800 802 2
804 806 3 80b
80c 80d 2 808
80f 2 7f3 811
1 813 1 815
2 7de 816 1
81a 1 7d9 2
7d7 81e 1 860
2 85f 860 1
868 1 86e 2
870 871 2 85d
872 1 879 1
874 2 88c 88e
3 88a 88b 890
2 899 89b 2
89d 89f 2 8a1
8a3 2 8a5 8a7
2 8a9 8ab 3
8b0 8b1 8b2 2
8ad 8b4 2 898
8b6 1 8b8 1
8ba 2 883 8bb
1 8bf 1 87e
2 87c 8c3 3
754 821 8c6 1
6cc 2 8d2 8d7
1 8de 1 8d9
2 8f1 8f3 3
8ef 8f0 8f5 2
8fe 900 2 902
904 2 906 908
2 90a 90c 2
90e 910 3 915
916 917 2 912
919 2 8fd 91b
1 91d 1 91f
2 8e8 920 1
924 1 8e3 2
8e1 928 1 933
2 92f 933 1
937 1 940 1
942 2 950 955
1 95c 1 957
2 96f 971 3
96d 96e 973 2
97c 97e 2 980
982 2 984 986
2 988 98a 2
98c 98e 3 993
994 995 2 990
997 2 97b 999
1 99b 1 99d
2 966 99e 1
9a2 1 961 2
95f 9a6 1 9b1
2 9ad 9b1 1
9b5 2 9be 9bf
1 9c3 2 9c1
9c3 1 9c8 1
9d0 2 9cf 9d0
1 9d8 1 9de
2 9e0 9e1 2
9cd 9e2 1 9e9
1 9e4 2 9fc
9fe 3 9fa 9fb
a00 2 a09 a0b
2 a0d a0f 2
a11 a13 2 a15
a17 2 a19 a1b
3 a20 a21 a22
2 a1d a24 2
a08 a26 1 a28
1 a2a 2 9f3
a2b 1 a2f 1
9ee 2 9ec a33
1 a3e 2 a3d
a3e 1 a46 1
a4c 2 a4e a4f
2 a3b a50 1
a57 1 a52 2
a6a a6c 3 a68
a69 a6e 2 a77
a79 2 a7b a7d
2 a7f a81 2
a83 a85 2 a87
a89 3 a8e a8f
a90 2 a8b a92
2 a76 a94 1
a96 1 a98 2
a61 a99 1 a9d
1 a5c 2 a5a
aa1 1 aac 2
aab aac 1 ab4
1 aba 2 abc
abd 2 aa9 abe
1 ac5 1 ac0
2 ad8 ada 3
ad6 ad7 adc 2
ae5 ae7 2 ae9
aeb 2 aed aef
2 af1 af3 2
af5 af7 3 afc
afd afe 2 af9
b00 2 ae4 b02
1 b04 1 b06
2 acf b07 1
b0b 1 aca 2
ac8 b0f 1 b1a
2 b19 b1a 1
b22 1 b28 2
b2a b2b 2 b17
b2c 1 b33 1
b2e 2 b46 b48
3 b44 b45 b4a
2 b53 b55 2
b57 b59 2 b5b
b5d 2 b5f b61
2 b63 b65 3
b6a b6b b6c 2
b67 b6e 2 b52
b70 1 b72 1
b74 2 b3d b75
1 b79 1 b38
2 b36 b7d 5
a36 aa4 b12 b80
b84 2 b86 b87
1 b88 2 b8c
b8f 2 b91 b92
3 b9e b9f ba0
1 bab 1 bb1
2 bb3 bb4 1
bbc 2 bb8 bbc
1 bc1 1 bc3
2 bc6 bc7 1
bcb 2 bc9 bcb
1 be0 2 bdf
be0 1 be8 1
bee 2 bf0 bf1
2 bdd bf2 1
bf9 1 bf4 2
c0c c0e 3 c0a
c0b c10 2 c19
c1b 2 c1d c1f
2 c21 c23 2
c25 c27 2 c29
c2b 3 c30 c31
c32 2 c2d c34
2 c18 c36 1
c38 1 c3a 2
c03 c3b 1 c3f
1 bfe 2 bfc
c43 1 c4e 2
c4a c4e 1 c5c
2 c5b c5c 1
c64 1 c6a 2
c6c c6d 2 c59
c6e 1 c75 1
c70 2 c88 c8a
3 c86 c87 c8c
2 c95 c97 2
c99 c9b 2 c9d
c9f 2 ca1 ca3
2 ca5 ca7 3
cac cad cae 2
ca9 cb0 2 c94
cb2 1 cb4 1
cb6 2 c7f cb7
1 cbb 1 c7a
2 c78 cbf 1
cc2 1 cd1 2
cd0 cd1 1 cd9
1 cdf 2 ce1
ce2 2 cce ce3
1 cea 1 ce5
2 cfd cff 3
cfb cfc d01 2
d0a d0c 2 d0e
d10 2 d12 d14
2 d16 d18 2
d1a d1c 3 d21
d22 d23 2 d1e
d25 2 d09 d27
1 d29 1 d2b
2 cf4 d2c 1
d30 1 cef 2
ced d34 1 d37
2 d3a d3b 2
c46 d3c 2 d40
d41 1 d45 2
d43 d45 1 d54
2 d53 d54 1
d5c 1 d62 2
d64 d65 2 d51
d66 1 d6d 1
d68 2 d80 d82
3 d7e d7f d84
2 d8d d8f 2
d91 d93 2 d95
d97 2 d99 d9b
2 d9d d9f 3
da4 da5 da6 2
da1 da8 2 d8c
daa 1 dac 1
dae 2 d77 daf
1 db3 1 d72
2 d70 db7 1
dba 2 dbe dbd
4 ba1 bb5 bc4
dbf 2 dc5 dcb
1 dd2 1 dcd
2 de5 de7 3
de3 de4 de9 2
df2 df4 2 df6
df8 2 dfa dfc
2 dfe e00 2
e02 e04 3 e09
e0a e0b 2 e06
e0d 2 df1 e0f
1 e11 1 e13
2 ddc e14 1
e18 1 dd7 2
dd5 e1c 1 e2c
2 e2b e2c 1
e34 1 e3a 2
e3c e3d 2 e29
e3e 1 e45 1
e40 2 e58 e5a
3 e56 e57 e5c
2 e65 e67 2
e69 e6b 2 e6d
e6f 2 e71 e73
2 e75 e77 3
e7c e7d e7e 2
e79 e80 2 e64
e82 1 e84 1
e86 2 e4f e87
1 e8b 1 e4a
2 e48 e8f 2
e95 e96 1 e9a
2 e98 e9a 1
ed7 2 ed6 ed7
1 edf 1 ee5
2 ee7 ee8 2
ed4 ee9 1 ef0
1 eeb 2 f03
f05 3 f01 f02
f07 2 f10 f12
2 f14 f16 2
f18 f1a 2 f1c
f1e 2 f20 f22
3 f27 f28 f29
2 f24 f2b 2
f0f f2d 1 f2f
1 f31 2 efa
f32 1 f36 1
ef5 2 ef3 f3a
2 f40 f41 1
f45 2 f43 f45
5 f4d f4e f4f
f50 f51 2 f4a
f54 1 f56 2
f3d f57 2 f60
f61 1 f6c 1
f72 2 f74 f75
1 f7d 2 f79
f7d 1 f82 1
f84 1 fbf 2
fbe fbf 1 fc7
1 fcd 2 fcf
fd0 2 fbc fd1
1 fd8 1 fd3
2 feb fed 3
fe9 fea fef 2
ff8 ffa 2 ffc
ffe 2 1000 1002
2 1004 1006 2
1008 100a 3 100f
1010 1011 2 100c
1013 2 ff7 1015
1 1017 1 1019
2 fe2 101a 1
101e 1 fdd 2
fdb 1022 1 1030
2 102f 1030 1
1038 1 103e 2
1040 1041 2 102d
1042 1 1049 1
1044 2 105c 105e
3 105a 105b 1060
2 1069 106b 2
106d 106f 2 1071
1073 2 1075 1077
2 1079 107b 3
1080 1081 1082 2
107d 1084 2 1068
1086 1 1088 1
108a 2 1053 108b
1 108f 1 104e
2 104c 1093 5
f62 f76 f85 1025
1096 2 109d 10a3
1 10aa 1 10a5
2 10bd 10bf 3
10bb 10bc 10c1 2
10ca 10cc 2 10ce
10d0 2 10d2 10d4
2 10d6 10d8 2
10da 10dc 3 10e1
10e2 10e3 2 10de
10e5 2 10c9 10e7
1 10e9 1 10eb
2 10b4 10ec 1
10f0 1 10af 2
10ad 10f4 2 10fa
10fb 1 10ff 2
10fd 10ff 2 1104
1105 1 1109 2
1107 1109 1 1117
2 1116 1117 1
111f 1 1125 2
1127 1128 2 1114
1129 1 1130 1
112b 2 1143 1145
3 1141 1142 1147
2 1150 1152 2
1154 1156 2 1158
115a 2 115c 115e
2 1160 1162 3
1167 1168 1169 2
1164 116b 2 114f
116d 1 116f 1
1171 2 113a 1172
1 1176 1 1135
2 1133 117a 1
117d 1 1180 5
1187 1188 1189 118a
118b 2 11a0 11a5
1 11ac 1 11a7
2 11bf 11c1 3
11bd 11be 11c3 2
11cc 11ce 2 11d0
11d2 2 11d4 11d6
2 11d8 11da 2
11dc 11de 3 11e3
11e4 11e5 2 11e0
11e7 2 11cb 11e9
1 11eb 1 11ed
2 11b6 11ee 1
11f2 1 11b1 2
11af 11f6 2 11fc
11fd 1 1201 2
11ff 1201 1 121b
2 121a 121b 1
1223 1 1229 2
122b 122c 2 1218
122d 1 1234 1
122f 2 1247 1249
3 1245 1246 124b
2 1254 1256 2
1258 125a 2 125c
125e 2 1260 1262
2 1264 1266 3
126b 126c 126d 2
1268 126f 2 1253
1271 1 1273 1
1275 2 123e 1276
1 127a 1 1239
2 1237 127e 1
1281 1 129d 2
129c 129d 1 12a5
1 12ab 2 12ad
12ae 2 129a 12af
1 12b6 1 12b1
2 12c9 12cb 3
12c7 12c8 12cd 2
12d6 12d8 2 12da
12dc 2 12de 12e0
2 12e2 12e4 2
12e6 12e8 3 12ed
12ee 12ef 2 12ea
12f1 2 12d5 12f3
1 12f5 1 12f7
2 12c0 12f8 1
12fc 1 12bb 2
12b9 1300 1 1303
2 1306 1307 8
f5b 109a 10f7 1181
1184 118e 11f9 1308
2 130a 130b 1
130d b 92b 943
9a9 b93 b96 b99
dc2 e1f e92 130c
1310 1 8cb 1
1314 3 8ca 1313
1316 1 1322 2
1321 1322 1 132a
1 1330 2 1332
1333 2 131f 1334
1 133b 1 1336
2 134e 1350 3
134c 134d 1352 2
135b 135d 2 135f
1361 2 1363 1365
2 1367 1369 2
136b 136d 3 1372
1373 1374 2 136f
1376 2 135a 1378
1 137a 1 137c
2 1345 137d 1
1381 1 1340 2
133e 1385 5 6a7
6bb 6ca 1318 1388
2 138f 1395 1
139c 1 1397 2
13af 13b1 3 13ad
13ae 13b3 2 13bc
13be 2 13c0 13c2
2 13c4 13c6 2
13c8 13ca 2 13cc
13ce 3 13d3 13d4
13d5 2 13d0 13d7
2 13bb 13d9 1
13db 1 13dd 2
13a6 13de 1 13e2
1 13a1 2 139f
13e6 1 13ed 2
13fe 1403 1 140a
1 1405 2 141d
141f 3 141b 141c
1421 2 142a 142c
2 142e 1430 2
1432 1434 2 1436
1438 2 143a 143c
3 1441 1442 1443
2 143e 1445 2
1429 1447 1 1449
1 144b 2 1414
144c 1 1450 1
140f 2 140d 1454
2 145a 145b 1
145f 2 145d 145f
3 146b 146c 146d
2 1466 1470 1
1472 1 147f 2
147e 147f 1 1487
1 148d 2 148f
1490 2 147c 1491
1 1498 1 1493
2 14ab 14ad 3
14a9 14aa 14af 2
14b8 14ba 2 14bc
14be 2 14c0 14c2
2 14c4 14c6 2
14c8 14ca 3 14cf
14d0 14d1 2 14cc
14d3 2 14b7 14d5
1 14d7 1 14d9
2 14a2 14da 1
14de 1 149d 2
149b 14e2 1 14eb
2 14e9 14eb 1
14f2 1 14f4 2
14fe 1500 3 14fc
14fd 1502 2 150b
150d 2 150f 1511
2 1513 1515 1
151d 2 151f 1520
2 1517 1522 2
1524 1526 2 1528
152a 2 152c 152e
2 1530 1532 2
150a 1534 1 1536
1 1538 3 153f
1540 1541 10 156
175 178 17b 1db
69b 6a0 138c 13e9
13f0 1457 1473 14e5
14f5 1539 1543 2
1550 1552 3 154e
154f 1554 2 155d
155f 2 155c 1561
2 1568 156c 2
156e 1570 2 1572
1576 2 1567 1578
2 1563 157a 1
157c 3 157d 1588
158c 1 1547 2
159f 15a1 3 159d
159e 15a3 2 15ac
15ae 2 15ab 15b0
2 15b7 15b9 2
15bb 15bd 3 15c2
15c3 15c4 2 15bf
15c6 2 15b6 15c8
2 15b2 15ca 1
15cc 4 1596 15cd
15d6 15da 1 15de
1 1591 2 158f
15e2 20 1a 24
2b 30 37 3e
45 4c 53 58
5d 63 69 6f
75 7b 81 87
8d 93 99 9f
a5 ab b1 b7
be e0 ea fb
10c 116 
1
4
0 
15e9
0
1
50
4f
24
0 1 1 1 1 5 1 7
1 9 1 b 1 d e d
10 1 12 1 14 1 16 17
16 19 16 1b 16 1d 16 1f
16 21 16 23 16 25 16 27
16 29 2a 29 2c 29 2e 29
30 16 32 16 34 16 36 16
38 39 38 3b 16 3d 16 3f
16 41 16 43 16 45 16 47
1 49 1 4b 1 4d 1 0

2 0 1
83 1 0
a1 1 0
3 1 0
7d 1 0
5a 1 0
47 1 0
4e 1 0
ed 1 3
1c 1 0
101 1 4
b3 1 0
b9 1 0
12 1 0
71 1 0
2d 1 0
a7 1 0
95 1 0
89 1 0
e5 1 0
55 1 0
65 1 0
77 1 0
5f 1 0
6b 1 0
8f 1 0
40 1 0
39 1 0
9b 1 0
32 1 0
26 1 0
130d 16 47
7 1 0
ad 1 0
c1 1 2
111 1 0
0

/
SHOW ERRORS FUNCTION SP_COMMIT_ASSIGN;
ALTER FUNCTION SP_COMMIT_ASSIGN COMPILE DEBUG;
/****************************************************************************************
Accept ETC and/or FORECAST_START, Assign level
  Bitflag =
    1 - ETC
    2 - FORECAST_START
    3 - ETC + FORECAST_START
****************************************************************************************/
Prompt == FUNCTION SP_ACPT_ETC_ASSIGN ==
CREATE OR REPLACE FUNCTION SP_ACPT_ETC_ASSIGN wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
ab
2 :e:
1FUNCTION:
1SP_ACPT_ETC_ASSIGN:
1ID:
1VARCHAR2:
1BITFLAG001:
1PLS_INTEGER:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_TSK_ASSIGNMENT_ID:
132:
1H_PRJ_ASSIGNMENT_ID:
1H_REC_USER:
1H_PROJECT_ID:
1H_TASK_ID:
1H_RES_ID:
1H_RES_TYPE:
11:
1H_TA_START_DT:
1TIMESTAMP:
1H_TA_FINISH_DT:
1H_START_DATE:
1H_FINISH_DATE:
1H_FINISH_D:
1DATE:
1H_LAST_SUBMITTED:
1H_WEEK_OF:
1H_BIT_FLAG:
1H_EFFORT:
10:
1H_CONTOUR:
1H_BITFLAG:
1H_REMAINING:
1H_TOTAL:
1H_CONSTR_DT:
1H_DAYOFYEAR:
1H_FINISH_TIME:
18:
1H_MINUTE:
1H_CONSTR_TYPE:
1H_FF_FLAG:
1H_ETC:
1H_SCHED_WORK:
1H_TEMP_VAR:
1H_TEMP_SC_STATE:
1H_REG_COST:
1NUMBER:
10.00:
1H_REG_SALE:
1H_SD:
1H_FD:
1H_BILLABLE:
1H_CAPITAL_EXPENSE:
1H_TPH_DELETE:
1H_MLST_PERCENT:
1H_PROJAFFECTED_START:
1H_WORK_TODATE:
1H_PASSED:
1H_NOT_EFFORT_PA:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_ACPT_ETC_ASSIGN(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
160:
1+:
1FPUT:
1               ENTER:: SP_ACPT_ETC_ASSIGN(61) :: cplan.sql(18894) :: :
1PROJECT_ID:
1TASK_ID:
1PROJECT_ASSIGNMENT_ID:
1SCHED_WORK:
1PLANNED_WORK:
1FORECAST_START:
1FORECAST_FINISH:
1ETC:
1WORK_TODATE:
1C_CONTOUR:
1LAST_SUBMITTED:
1PLANNED_START:
1SCHED_START:
1PLANNED_FINISH:
1SCHED_FINISH:
1UNRESOLVED_START:
1UNRESOLVED_FINISH:
1BILLABLE:
1CAPITAL_EXPENSE:
1REGULAR_COST:
1REGULAR_RATE:
1BIT_FLAG:
1MILESTONE_PERCENT:
1RESOURCE_ID:
1RESOURCE_TYPE:
1TASK_ASSIGNMENTS:
1TSK_ASSIGNMENT_ID:
1SELECT:n      PROJECT_ID,:n      TASK_ID,:n      PROJECT_ASSIGNMENT_ID,:n    +
1  SCHED_WORK+PLANNED_WORK,:n      FORECAST_START,:n      FORECAST_FINISH,:n  +
1    CASE WHEN h_bitflag = 2 AND FORECAST_START IS NuLL THEN 0 ELSE ETC END,:n+
1      SCHED_WORK+PLANNED_WORK - WORK_TODATE,:n      ETC,:n      C_CONTOUR,:n +
1     LAST_SUBMITTED,:n      CASE WHEN FORECAST_FINISH IS NOT NuLL AND FORECAS+
1T_START IS NuLL AND FORECAST_FINISH BETWEEN COALESCE(PLANNED_START, SCHED_STA+
1RT) AND COALESCE(PLANNED_FINISH, SCHED_FINISH) THEN 1 ELSE 0 END,:n      UNRE+
1SOLVED_START,:n      UNRESOLVED_FINISH,:n      BILLABLE,:n      CAPITAL_EXPEN+
1SE,:n      REGULAR_COST,:n      REGULAR_RATE,:n      BIT_FLAG,:n      MILESTO+
1NE_PERCENT,:n      RESOURCE_ID,:n      RESOURCE_TYPE,:n      WORK_TODATE:n   +
1   INTO:n      h_project_id,:n      h_task_id,:n      h_prj_assignment_id,:n +
1     h_sched_work,:n      h_start_date,:n      h_finish_date,:n      h_effort+
1,:n      h_remaining,:n      h_etc,:n      h_contour,:n      h_last_submitted+
1,:n      h_FF_flag,:n      h_ta_start_DT,:n      h_ta_finish_DT,:n      h_bil+
1lable,:n      h_capital_expense,:n      h_reg_cost,:n      h_reg_sale,:n     +
1 h_bit_flag,:n      h_mlst_percent,:n      h_res_id,:n      h_res_type,:n    +
1  h_work_todate:n      FROM TASK_ASSIGNMENTS:n      WHERE TSK_ASSIGNMENT_ID =+
1 h_tsk_assignment_id:n      AND (((SCHED_WORK+PLANNED_WORK) > 0:n      AND (S+
1CHED_WORK+PLANNED_WORK - WORK_TODATE) <> ETC):n      OR FORECAST_START IS NOT+
1 NULL OR FORECAST_FINISH IS NOT NULL):
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
1140:
1           EXCEPTION:: SP_ACPT_ETC_ASSIGN(141) :: cplan.sql(18974) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1=:
1SC_SQL_NO_DATA:
1147:
1        RETURN_ERROR:: SP_ACPT_ETC_ASSIGN(148) :: cplan.sql(18981) :: :
1STATE:: :
1PMO_CNST:
1RECORD_NOT_FOUND:
1 ID:: :
1150:
1                EXIT:: SP_ACPT_ETC_ASSIGN(151) :: cplan.sql(18984) :: :
1PMODEFINES:
1PMO_RETURN_ERROR_SP:
1TRUE:
1FC_ACPT_EFFORT:
1>:
1200000:
1169:
1        RETURN_ERROR:: SP_ACPT_ETC_ASSIGN(170) :: cplan.sql(19003) :: :
1172:
1                EXIT:: SP_ACPT_ETC_ASSIGN(173) :: cplan.sql(19006) :: :
1!=:
1SELECT ETC INTO h_total:n            FROM TASK_ASSIGNMENTS:n            WHERE+
1 TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
1193:
1           EXCEPTION:: SP_ACPT_ETC_ASSIGN(194) :: cplan.sql(19027) :: :
1UPDATE TASK_ASSIGNMENTS:n               SET ETC  = h_etc:n               WHER+
1E TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
1FOUND:
1100:
1215:
1           EXCEPTION:: SP_ACPT_ETC_ASSIGN(216) :: cplan.sql(19049) :: :
1FC_ROLLUP:
14:
1226:
1       RETURN_STATUS:: SP_ACPT_ETC_ASSIGN(227) :: cplan.sql(19060) :: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
1233:
1           EXCEPTION:: SP_ACPT_ETC_ASSIGN(234) :: cplan.sql(19067) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1245:
1           EXCEPTION:: SP_ACPT_ETC_ASSIGN(246) :: cplan.sql(19079) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
532
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 1c
81 b0 a3 a0 1c 81 b0 a3
a0 1c 81 b0 a3 a0 1c 81
b0 a3 a0 1c 81 b0 a3 a0
1c 81 b0 a3 a0 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
81 b0 a3 a0 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 81
b0 a3 a0 1c 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 :2 a0 6b
1c 81 b0 :2 a0 6b :2 a0 6b 6e
a5 b a 10 :2 a0 6b 6e :2 a0
6b 6e a5 b 6e 7e a0 b4
2e 6e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
d :2 a0 d :2 a0 d a0 4d d
a0 4d d :46 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b a0
7e a0 6b b4 2e 5a :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b :2 a0 a5 57 b7 19 3c
:1c a0 a5 b d :2 a0 51 a5 b
7e 51 b4 2e :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 51 a5
b :2 a0 a5 57 b7 a0 51 d
:2 a0 51 a5 b 7e 51 b4 2e
:5 a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 51 a5 b a0 7e
a0 51 a5 b b4 2e :5 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 19
3c b7 19 3c :3 a0 a5 b d
b7 :2 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 b7
:2 a0 6b :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 :2 a0 6b :2 6e 7e :2 a0 6b
b4 2e 7e 6e b4 2e 7e :2 a0
6b b4 2e a5 57 b7 19 3c
:9 a0 12a e9 dd :2 a0 65 b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c :7 a0 12a
e9 dd :2 a0 65 b7 a4 b1 11
4f b7 a6 9 a4 b1 11 68
4f 1d 17 b5 
532
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 30
56 5a 5e 62 65 69 86 71
75 78 79 81 70 a3 91 6d
95 96 9e 90 c0 ae 8d b2
b3 bb ad dd cb aa cf d0
d8 ca fa e8 c7 ec ed f5
e7 117 105 e4 109 10a 112 104
134 122 101 126 127 12f 121 150
13f 143 14b 11e 168 157 15b 163
13e 184 173 177 17f 13b 19c 18b
18f 197 172 1b8 1a7 1ab 1b3 16f
1d0 1bf 1c3 1cb 1a6 1ec 1db 1df
1e7 1a3 208 1f3 1f7 1fa 1fb 203
1da 224 213 217 1d7 21f 212 240
22f 233 20f 23b 22e 25c 24b 24f
22b 257 24a 278 267 26b 247 273
266 294 283 287 263 28f 282 2b0
29f 2a3 2ab 27f 2c8 2b7 2bb 2c3
29e 2e5 2d3 29b 2d7 2d8 2e0 2d2
301 2f0 2f4 2cf 2fc 2ef 31d 30c
310 2ec 318 30b 339 328 32c 308
334 327 355 344 348 324 350 343
371 360 364 340 36c 35f 38d 37c
380 35c 388 37b 3a9 398 39c 378
3a4 397 3c5 3b4 3b8 394 3c0 3b3
3e1 3d0 3d4 3b0 3dc 3cf 3fd 3ec
3f0 3f8 3cc 415 404 408 410 3eb
431 420 424 3e8 42c 41f 44d 43c
440 41c 448 43b 469 458 45c 438
464 457 485 474 478 454 480 473
4a1 490 494 49c 470 4bc 4a8 4ac
4b4 4b7 48f 4d8 4c7 4cb 48c 4d3
4c6 4f4 4e3 4e7 4c3 4ef 4e2 514
4ff 503 4df 507 50f 4fe 51b 51f
4fb 523 527 52b 52e 533 534 1
536 53b 53f 543 546 54b 54f 553
556 55b 55c 55e 563 566 56a 56b
570 575 578 57d 57e 583 586 58a
58b 590 593 598 599 59e 5a1 5a6
5a7 5ac 5af 5b3 5b7 5bb 5bc 5be
5c3 5c4 5c6 5c7 5cc 5cf 5d4 5d5
5da 5dd 5e2 5e3 5e8 5eb 5ef 5f0
5f5 5f8 5fd 5fe 603 606 60b 60c
611 612 617 619 61d 620 624 628
62b 62f 633 636 639 63e 641 644
647 648 64d 64e 1 650 655 659
65d 660 665 66a 66d 671 672 677
678 67d 67f 683 686 68a 68e 692
696 69a 69e 6a2 6a6 6aa 6ae 6af
6b3 6b7 6b8 6bc 6c0 6c4 6c8 6cc
6d0 6d4 6d8 6dc 6e0 6e4 6e8 6ec
6f0 6f4 6f8 6fc 700 704 708 70c
710 714 718 71c 720 724 728 72c
730 734 738 73c 740 744 748 74c
750 754 758 75c 760 764 768 76c
770 774 778 77c 780 784 788 78c
790 794 798 79c 7a0 7a4 7a8 7ac
7b0 7b4 7b8 7bc 7c0 7c4 7c8 7cc
7d0 7d4 7e0 7e4 7e8 7eb 7ef 7f3
7f5 7f9 7fd 801 804 808 80c 80e
80f 814 1 818 81c 820 823 827
82b 82f 833 836 83a 83e 841 844
849 84c 84f 852 853 858 859 1
85b 860 864 868 86b 870 875 878
87c 87d 882 885 88a 88b 890 893
898 899 89e 8a1 8a5 8a6 8ab 8ae
8b3 8b4 8b9 8bc 8c0 8c4 8c7 8ca
8cb 8cd 8ce 8d3 8d4 8d9 8db 8df
8e2 8e4 8e8 8ea 8f6 8f8 8fa 8fb
900 904 906 912 914 918 91c 91f
923 926 92a 92d 92e 933 936 93a
93e 941 945 949 94c 94f 954 957
95a 95d 95e 963 964 1 966 96b
96f 973 976 97b 980 983 987 988
98d 990 995 996 99b 99e 9a3 9a4
9a9 9ac 9b0 9b4 9b8 9bc 9bf 9c0
9c2 9c7 9c8 9ca 9cb 9d0 9d3 9d8
9d9 9de 9e1 9e6 9e7 9ec 9ef 9f3
9f4 9f9 9fc a01 a02 a07 a08 a0d
a0f a13 a16 a1a a1e a21 a25 a29
a2c a2f a34 a37 a3a a3d a3e a43
a44 1 a46 a4b a4f a53 a56 a5b
a60 a63 a67 a68 a6d a6e a73 a75
a79 a7c a80 a84 a87 a8b a8f a92
a96 a9a a9b aa0 aa2 aa6 aa9 aad
ab1 ab5 ab9 abd ac1 ac5 ac9 acd
ad1 ad5 ad9 add ae1 ae5 ae9 aed
af1 af5 af9 afd b01 b05 b09 b0d
b11 b15 b19 b1a b1c b20 b24 b28
b2b b2c b2e b31 b34 b35 b3a b3e
b42 b45 b49 b4d b50 b53 b58 b5b
b5e b61 b62 b67 b68 1 b6a b6f
b73 b77 b7a b7f b84 b87 b8b b8c
b91 b94 b99 b9a b9f ba2 ba7 ba8
bad bb0 bb4 bb8 bbc bbd bbf bc4
bc5 bc7 bc8 bcd bd0 bd5 bd6 bdb
bde be3 be4 be9 bec bf0 bf1 bf6
bf9 bfe bff c04 c05 c0a c0c c10
c13 c17 c1b c1e c22 c26 c29 c2c
c31 c34 c37 c3a c3b c40 c41 1
c43 c48 c4c c50 c53 c58 c5d c60
c64 c65 c6a c6b c70 c72 c76 c79
c7d c81 c84 c88 c8c c8f c90 c92
c96 c9a c9b ca0 ca2 ca6 ca9 cad
cb1 cb5 cb8 cb9 cbb cbe cc1 cc2
cc7 ccb ccf cd3 cd7 cdb ce7 ceb
cef cf2 cf6 cfa cfc d00 d04 d08
d0b d0f d13 d15 d16 d1b 1 d1f
d23 d27 d2a d2e d32 d36 d3a d3d
d41 d45 d48 d4b d50 d53 d56 d59
d5a d5f d60 1 d62 d67 d6b d6f
d72 d77 d7c d7f d83 d84 d89 d8c
d91 d92 d97 d9a d9f da0 da5 da8
dac dad db2 db5 dba dbb dc0 dc3
dc7 dcb dce dd1 dd2 dd4 dd5 dda
ddb de0 de2 de6 de9 deb def df1
dfd dff e01 e02 e07 e0b e0d e19
e1b e1f e23 e26 e27 e29 e2d e30
e34 e37 e38 e3a e3b e40 e44 e48
e4c e50 e54 e60 e64 e69 e6d e70
e71 e76 e7a e7e e81 e84 e88 e8a
e8e e92 e95 e98 e9c e9e ea2 ea6
ea9 eab eaf eb3 eb7 eba ebe ec2
ec4 ec5 eca 1 ece ed2 ed6 ed9
edd ee1 ee5 ee9 eec ef0 ef4 ef7
efa eff f02 f05 f08 f09 f0e f0f
1 f11 f16 f1a f1e f21 f26 f2b
f2e f32 f33 f38 f3b f40 f41 f46
f49 f4e f4f f54 f57 f5b f5c f61
f64 f69 f6a f6f f72 f76 f7a f7d
f80 f81 f83 f84 f89 f8a f8f f91
f95 f98 f9a f9e fa0 fac fae fb0
fb1 fb6 fba fbc fc8 fca fcc fd0
fd3 fd5 fd9 fdc fe0 fe4 fe8 fe9
feb fef ff1 ff5 ff9 ffc 1000 1004
1007 100b 100f 1012 1015 101a 101d 1020
1023 1024 1029 102a 1 102c 1031 1035
1039 103c 1041 1046 1049 104d 104e 1053
1056 105b 105c 1061 1064 1069 106a 106f
1072 1076 107a 107e 1082 1085 1086 1088
108d 108e 1090 1091 1096 1099 109e 109f
10a4 10a7 10ac 10ad 10b2 10b5 10b9 10ba
10bf 10c2 10c7 10c8 10cd 10ce 10d3 10d5
10d9 10dc 10e0 10e4 10e7 10eb 10ef 10f2
10f6 10fa 10fb 1100 1102 1106 110a 110d
1111 1115 1118 111c 1120 1123 1126 112b
112e 1131 1134 1135 113a 113b 1 113d
1142 1146 114a 114d 1152 1157 115a 115e
115f 1164 1165 116a 116e 1172 1175 117a
117f 1182 1186 118a 118d 118e 1193 1196
119b 119c 11a1 11a4 11a8 11ac 11af 11b0
11b5 11b6 11bb 11bd 11c1 11c4 11c8 11cc
11d0 11d4 11d8 11dc 11e0 11e4 11e8 11f4
11f9 11fd 1201 1205 1209 120b 120c 1211
1 1215 1219 121d 1220 1224 1228 122c
1230 1233 1237 123b 123e 1241 1246 1249
124c 124f 1250 1255 1256 1 1258 125d
1261 1265 1268 126d 1272 1275 1279 127a
127f 1280 1285 1289 128d 1290 1295 129a
129d 12a1 12a2 12a7 12aa 12af 12b0 12b5
12b8 12bc 12c0 12c3 12c6 12c7 12c9 12ca
12cf 12d0 12d5 12d7 12db 12de 12e2 12e6
12ea 12ee 12f2 12f6 12fa 1306 130b 130f
1313 1317 131b 131d 1321 1323 132f 1331
1333 1334 1339 133d 133f 134b 134f 1351
1352 135b 
532
2
0 1 a 6 9 :3 6 11 :3 6
12 :2 6 1 3 a 13 a :2 1
4 18 21 20 :2 18 :2 4 18 21
20 :2 18 :2 4 f 18 17 :2 f :2 4
11 1a 19 :2 11 :2 4 e 17 16
:2 e :2 4 d 16 15 :2 d :2 4 f
18 17 :2 f :2 4 :3 12 :2 4 :3 13 :2 4
:3 11 :2 4 :3 12 :2 4 :3 f :2 4 :3 15 :2 4
:3 e :2 4 f 18 17 :2 f :2 4 :2 d
1c d :2 4 :2 e 1d e :2 4 :2 e
1d e :2 4 :2 10 1f 10 :2 4 :2 c
1b c :2 4 :3 10 :2 4 :3 10 :2 4 12
1b 1a :2 12 :2 4 :2 d 1c d :2 4
:2 12 21 12 :2 4 :2 e 1d e :2 4
:2 a 19 a :2 4 :2 11 20 11 :2 4
:2 f 1e f :2 4 :2 14 23 14 :2 4
:2 f 19 f :2 4 :2 f 19 f :2 4
:3 9 :2 4 :3 9 :2 4 :2 f 1e f :2 4
:2 16 25 16 :2 4 :2 11 20 11 :2 4
:2 13 22 13 :2 4 :3 19 :2 4 :2 12 21
12 :2 4 :2 d 1c d :2 4 :2 14 23
14 :2 4 e 17 :3 e 4 7 :2 11
23 :2 2d 3a :2 23 :3 7 :2 d 19 25
:2 2f 43 :2 25 5a 60 63 :2 5a 71
8d 8 :2 71 d 10 :2 71 13 16
:2 71 b e :2 71 13 16 1f 27
:2 1f 34 :2 16 :2 71 b e :2 71 13
16 :2 71 1b 1e :2 71 2a 2d :2 71
32 35 :2 71 :2 7 50 :2 4 7 :2 11
23 :2 2d 39 3c 52 54 55 :2 52
:2 23 :3 7 :2 d 12 1e 63 66 :2 1e
:2 7 58 :3 4 11 :2 4 1b :2 4 12
:2 4 14 :2 4 15 4 :4 7 12 :2 7
11 23 46 7 12 21 :3 7 11
31 4c 64 6d 7c 8d 96 a6
:22 7 c d 21 e 19 c 17
26 36 a 28 fffa 7 :2 10 1c
7 4 c a :2 13 1f a 1a
:2 7 :2 c a :2 13 1f a d :2 17
29 :2 33 3f 43 59 5c 5d :2 59
:2 29 :3 d :2 13 18 24 6a 6d :2 24
7a 7d :2 24 81 84 :2 24 90 93
:2 24 9b 9e :2 24 aa ad b4 bd
c0 :2 ad :2 24 :2 d 60 :2 a :2 7 :4 13
:2 7 4 :3 1 8 :2 11 1c 1a :2 25
:2 1a 7 a :2 14 26 :2 30 3c 40
56 59 5a :2 56 :2 26 :3 a :2 10 15
21 67 6a :2 21 77 7a :2 21 7e
81 :2 21 8b 8e 97 9f :2 a8 :2 97
bb :2 8e :2 21 c3 c6 :2 21 ce d1
:2 21 d6 d9 :2 21 ed f0 :2 21 :2 a
5d :2 7 a :2 14 26 :2 30 3c 3f
55 58 59 :2 55 :2 26 :3 a :2 10 15
21 67 6a :2 21 :2 a 5c :3 7 :2 12
26 :2 2f 41 f :2 7 35 :3 4 17
26 34 3f 54 25 33 41 50
5a 25 2c 37 49 54 25 35
41 54 25 31 3d 4d 57 25
34 3f :2 17 4 7 10 21 :2 7
24 26 :2 24 a :2 14 26 :2 30 3c
40 56 59 5a :2 56 :2 26 :3 a :2 10
15 21 67 6a :2 21 77 7a :2 21
7e 81 :2 21 8b 8e 97 9f :2 97
b1 :2 8e :2 21 b9 bc :2 21 c4 c7
:2 21 cc cf :2 21 e3 e6 :2 21 :2 a
5d :2 7 a :2 14 26 :2 30 3c 3f
55 58 59 :2 55 :2 26 :3 a :2 10 15
21 67 6a :2 21 :2 a 5c :3 7 :2 12
26 2f 40 :2 26 44 f :2 7 2d
7 12 7 a 13 1e :2 a 21
24 :2 21 14 1d 12 13 27 :2 d
:2 16 22 d a 12 10 :2 19 25
10 20 :2 d :2 12 10 :2 19 25 10
13 :2 1d 2f :2 39 45 49 5f 62
63 :2 5f :2 2f :3 13 :2 19 1e 2a 70
73 :2 2a 80 83 :2 2a 87 8a :2 2a
96 99 :2 2a a1 a4 :2 2a b0 b3
ba c3 c6 :2 b3 :2 2a :2 13 66 :2 10
:2 d :4 19 :2 d a :3 26 d 16 1d
:2 d 22 20 2b 34 :2 22 :2 20 17
14 1b 16 2a 10 17 13 1f
:3 1d 13 :2 1c 28 13 24 13 :2 1c
28 13 :4 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 62 65
66 :2 62 :2 32 :3 16 :2 1c 21 2d 73
76 :2 2d 83 86 :2 2d 8a 8d :2 2d
99 9c :2 2d a4 a7 :2 2d b3 b6
bd c6 c9 :2 b6 :2 2d :2 16 69 :2 13
:2 10 :4 1c :2 10 d :4 37 :2 a 26 :3 7
1a 24 :2 1a 7 :4 4 7 :2 11 23
:2 2d 39 3c 52 55 56 :2 52 :2 23
:3 7 :2 d 12 1e 64 67 :2 1e 74
77 :2 1e 7b 7e :2 1e 88 8b 94
9c :2 a5 :2 94 b0 :2 8b :2 1e b8 bb
:2 1e c3 c6 :2 1e cb ce :2 1e d1
d4 :2 1e :2 7 59 :3 4 :2 f 21 :2 2a
34 38 :2 4 1 9 12 9 a
:2 14 26 :2 30 3c 40 56 59 5a
:2 56 :2 26 :3 a :2 10 15 21 67 6a
:2 21 :3 a :2 10 15 39 5c 5f :2 68
:2 39 76 79 :2 39 81 84 :2 8d :2 39
:2 a 5d :3 7 c 15 1e 2c a
13 1e f e 7 e 7 e
7 29 :2 4 :2 9 7 :2 10 1c :2 7
:2 11 23 :2 2d 39 3d 53 56 57
:2 53 :2 23 :3 7 :2 d 12 1e 64 67
:2 1e :3 7 :2 d 12 36 42 45 :2 36
4d 50 :2 36 5c 5f 66 6f 72
:2 5f :2 36 :2 7 5a :2 4 7 c 1b
a 13 1e f e 7 e 7
e 7 :2 4 :4 10 :2 4 :8 1 
532
4
0 :2 1 :4 3 :4 4
:4 5 2 :4 6 :2 1
:7 8 :7 9 :7 a :7 b
:7 c :7 d :7 e :5 f
:5 10 :5 11 :5 12 :5 13
:5 14 :5 15 :7 16 :6 17
:6 18 :6 19 :6 1a :6 1b
:5 1c :5 1d :7 1e :6 1f
:6 20 :6 21 :6 22 :6 23
:6 24 :6 25 :6 26 :6 27
:5 28 :5 29 :6 2a :6 2b
:6 2c :6 2d :5 2e :6 2f
:6 30 :6 31 :7 33 :b 35
:11 36 37 :2 36 :2 37
:2 36 :2 37 :2 36 :2 38
:2 36 :9 38 :2 36 :2 39
:2 36 :2 39 :2 36 :2 39
:2 36 :2 39 :2 36 :2 39
:4 36 :3 35 :11 3c :b 3d
:3 3c :3 44 :3 45 :3 46
:3 4a :3 4c 51 52
53 :2 54 55 56
:3 57 :3 58 59 5a
5b :9 5c 5d 5e
5f 60 61 62
63 64 65 66
67 69 6a 6b
6c 6d 6e 6f
70 71 72 73
74 75 76 77
78 79 7a 7b
7c 7d 7e 7f
80 :2 81 :2 82 :4 83
:2 84 51 :5 85 4f
87 :5 88 :3 87 :2 89
:5 8b :11 8c :24 8d :3 8c
:2 8a :6 89 86 :3 34
:a 92 :11 93 :30 94 :3 93
:11 96 :b 97 :3 96 :7 99
9a :2 99 :3 92 :6 9d
:5 9e :5 9f :4 a0 :5 a1
:3 a2 :3 9d :9 a6 :11 a9
:2e aa :3 a9 :11 ac :b ad
:3 ac :9 af b0 :2 af
a6 :3 b2 :9 b5 :2 b7
b8 :2 b9 b7 :5 ba
b6 bc :5 bd :3 bc
:2 be :5 c0 :11 c1 :24 c2
:3 c1 :2 bf :6 be bb
:3 b5 :d c7 c9 :2 ca
:2 cb c9 :6 cc :5 cd
cc :5 cf ce :3 cc
c8 d2 :5 d3 :3 d2
:2 d4 :5 d6 :11 d7 :24 d8
:3 d7 :2 d5 :6 d4 d1
:6 c7 :3 b5 :6 df b1
:3 a6 :11 e2 :30 e3 :3 e2
:a e5 34 :3 e8 :11 e9
:b ea :17 eb :3 e9 :2 ed
:3 ee :3 ef f0 ee
ed ee :3 f1 :3 e8
:2 f2 :5 f4 :11 f5 :b f6
:18 f7 :3 f5 :2 f9 fa
:3 fb fc fa f9
fa :3 fd :2 f3 :6 f2
e7 :7 1 
135d
4
:3 0 1 :3 0 2
:a 0 52d 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
:2 0 7 6 :3 0
5 :7 0 9 8
:3 0 4 :3 0 7
:7 0 d c :3 0
8 :3 0 9 :3 0
a :2 0 4 11
12 0 f 13
0 52d 2 14
:2 0 c :2 0 f
4 :3 0 c :2 0
d 17 19 :6 0
1c 1a 0 52b
0 b :6 0 c
:2 0 13 4 :3 0
11 1e 20 :6 0
23 21 0 52b
0 d :6 0 c
:2 0 17 4 :3 0
15 25 27 :6 0
2a 28 0 52b
0 e :6 0 c
:2 0 1b 4 :3 0
19 2c 2e :6 0
31 2f 0 52b
0 f :6 0 c
:2 0 1f 4 :3 0
1d 33 35 :6 0
38 36 0 52b
0 10 :6 0 13
:2 0 23 4 :3 0
21 3a 3c :6 0
3f 3d 0 52b
0 11 :6 0 29
13b 0 27 4
:3 0 25 41 43
:6 0 46 44 0
52b 0 12 :6 0
2d 16f 0 2b
15 :3 0 48 :7 0
4b 49 0 52b
0 14 :6 0 15
:3 0 4d :7 0 50
4e 0 52b 0
16 :6 0 31 1a3
0 2f 15 :3 0
52 :7 0 55 53
0 52b 0 17
:6 0 15 :3 0 57
:7 0 5a 58 0
52b 0 18 :6 0
35 1d7 0 33
1a :3 0 5c :7 0
5f 5d 0 52b
0 19 :6 0 1a
:3 0 61 :7 0 64
62 0 52b 0
1b :6 0 1f :2 0
39 1a :3 0 66
:7 0 69 67 0
52b 0 1c :6 0
4 :3 0 13 :2 0
37 6b 6d :6 0
70 6e 0 52b
0 1d :6 0 1f
:2 0 3b 6 :3 0
72 :7 0 76 73
74 52b 0 1e
:6 0 1f :2 0 3d
6 :3 0 78 :7 0
7c 79 7a 52b
0 20 :6 0 1f
:2 0 3f 6 :3 0
7e :7 0 82 7f
80 52b 0 21
:6 0 1f :2 0 41
6 :3 0 84 :7 0
88 85 86 52b
0 22 :6 0 45
29b 0 43 6
:3 0 8a :7 0 8e
8b 8c 52b 0
23 :6 0 27 :2 0
47 15 :3 0 90
:7 0 93 91 0
52b 0 24 :6 0
1a :3 0 95 :7 0
98 96 0 52b
0 25 :6 0 1f
:2 0 4b 4 :3 0
49 9a 9c :6 0
9f 9d 0 52b
0 26 :6 0 1f
:2 0 4d 6 :3 0
a1 :7 0 a5 a2
a3 52b 0 28
:6 0 1f :2 0 4f
6 :3 0 a7 :7 0
ab a8 a9 52b
0 29 :6 0 1f
:2 0 51 6 :3 0
ad :7 0 b1 ae
af 52b 0 2a
:6 0 1f :2 0 53
6 :3 0 b3 :7 0
b7 b4 b5 52b
0 2b :6 0 1f
:2 0 55 6 :3 0
b9 :7 0 bd ba
bb 52b 0 2c
:6 0 1f :2 0 57
6 :3 0 bf :7 0
c3 c0 c1 52b
0 2d :6 0 31
:2 0 59 6 :3 0
c5 :7 0 c9 c6
c7 52b 0 2e
:6 0 31 :2 0 5b
30 :3 0 cb :7 0
cf cc cd 52b
0 2f :6 0 5f
3e8 0 5d 30
:3 0 d1 :7 0 d5
d2 d3 52b 0
32 :6 0 1f :2 0
61 1a :3 0 d7
:7 0 da d8 0
52b 0 33 :6 0
1a :3 0 dc :7 0
df dd 0 52b
0 34 :6 0 1f
:2 0 63 6 :3 0
e1 :7 0 e5 e2
e3 52b 0 35
:6 0 1f :2 0 65
6 :3 0 e7 :7 0
eb e8 e9 52b
0 36 :6 0 1f
:2 0 67 6 :3 0
ed :7 0 f1 ee
ef 52b 0 37
:6 0 6b 48c 0
69 6 :3 0 f3
:7 0 f7 f4 f5
52b 0 38 :6 0
1f :2 0 6d 1a
:3 0 f9 :7 0 fc
fa 0 52b 0
39 :6 0 6 :3 0
fe :7 0 1f :2 0
102 ff 100 52b
0 3a :6 0 1f
:2 0 6f 6 :3 0
104 :7 0 108 105
106 52b 0 3b
:6 0 110 111 0
71 6 :3 0 10a
:7 0 10e 10b 10c
52b 0 3c :6 0
116 117 0 73
9 :3 0 a :2 0
4 112 :7 0 115
113 0 52b 0
3d :6 0 3e :3 0
3f :3 0 3e :3 0
40 :3 0 119 11a
0 2 :4 0 75
11b 11d 118 11f
11e :2 0 41 :3 0
42 :3 0 121 122
0 43 :4 0 3e
:3 0 44 :3 0 125
126 0 2 :4 0
77 127 129 45
:4 0 46 :2 0 47
:3 0 79 12c 12e
:3 0 48 :4 0 46
:2 0 49 :4 0 7c
131 133 :3 0 46
:2 0 3 :3 0 7f
135 137 :3 0 46
:2 0 49 :4 0 82
139 13b :3 0 46
:2 0 4a :4 0 85
13d 13f :3 0 46
:2 0 4b :3 0 4c
:3 0 5 :3 0 88
143 145 4d :4 0
8a 142 148 8d
141 14a :3 0 46
:2 0 4a :4 0 90
14c 14e :3 0 46
:2 0 49 :4 0 93
150 152 :3 0 46
:2 0 7 :3 0 96
154 156 :3 0 46
:2 0 49 :4 0 99
158 15a :3 0 46
:2 0 4e :4 0 9c
15c 15e :3 0 9f
123 160 :2 0 162
a4 163 120 162
0 164 a6 0
48b 3e :3 0 3f
:3 0 165 166 0
3e :3 0 4f :3 0
168 169 0 13
:2 0 2 :4 0 50
:2 0 51 :2 0 13
:2 0 a8 16e 170
:3 0 ab 16a 172
167 174 173 :2 0
41 :3 0 52 :3 0
176 177 0 43
:4 0 53 :4 0 46
:2 0 47 :3 0 af
17b 17d :3 0 b2
178 17f :2 0 181
b5 182 175 181
0 183 b7 0
48b 21 :3 0 5
:3 0 184 185 0
48b b :3 0 3
:3 0 187 188 0
48b e :3 0 7
:3 0 18a 18b 0
48b 17 :4 0 18d
18e 0 48b 18
:4 0 190 191 0
48b 54 :3 0 55
:3 0 56 :3 0 57
:3 0 58 :3 0 59
:3 0 5a :3 0 21
:3 0 59 :3 0 5b
:3 0 57 :3 0 58
:3 0 5c :3 0 5b
:3 0 5d :3 0 5e
:3 0 5a :3 0 59
:3 0 5a :3 0 4b
:3 0 5f :3 0 60
:3 0 4b :3 0 61
:3 0 62 :3 0 63
:3 0 64 :3 0 65
:3 0 66 :3 0 67
:3 0 68 :3 0 69
:3 0 6a :3 0 6b
:3 0 6c :3 0 5c
:3 0 f :3 0 10
:3 0 d :3 0 2c
:3 0 17 :3 0 18
:3 0 1e :3 0 22
:3 0 2b :3 0 20
:3 0 1b :3 0 2a
:3 0 14 :3 0 16
:3 0 35 :3 0 36
:3 0 2f :3 0 32
:3 0 1d :3 0 38
:3 0 11 :3 0 12
:3 0 3a :3 0 6d
:3 0 6e :3 0 b
:3 0 57 :3 0 58
:3 0 57 :3 0 58
:3 0 5c :3 0 5b
:3 0 59 :3 0 5a
:4 0 6f 1 :8 0
1df 9 :3 0 70
:3 0 1da 1db 0
71 :3 0 1dc 1dd
0 1df b9 232
72 :3 0 9 :3 0
70 :3 0 1e1 1e2
0 71 :3 0 1e3
1e4 0 1e6 bc
1e8 be 1e7 1e6
:2 0 230 73 :3 0
9 :3 0 70 :3 0
1eb 1ec 0 71
:3 0 1ed 1ee 0
228 3e :3 0 3f
:3 0 1f0 1f1 0
3e :3 0 4f :3 0
1f3 1f4 0 74
:2 0 2 :4 0 75
:2 0 51 :2 0 13
:2 0 c0 1f9 1fb
:3 0 c3 1f5 1fd
1f2 1ff 1fe :2 0
41 :3 0 52 :3 0
201 202 0 43
:4 0 76 :4 0 46
:2 0 47 :3 0 c7
206 208 :3 0 46
:2 0 77 :4 0 ca
20a 20c :3 0 46
:2 0 78 :4 0 cd
20e 210 :3 0 46
:2 0 71 :3 0 d0
212 214 :3 0 46
:2 0 79 :4 0 d3
216 218 :3 0 46
:2 0 7a :3 0 7b
:3 0 13 :2 0 7c
:2 0 d6 21b 21f
da 21a 221 :3 0
dd 203 223 :2 0
225 e0 226 200
225 0 227 e2
0 228 e4 22b
:3 0 22b 0 22b
22a 228 229 :6 0
22d 2 :3 0 e7
22f e9 22e 22d
:2 0 230 eb :2 0
232 0 232 231
1df 230 :6 0 48b
1 :3 0 9 :3 0
70 :3 0 234 235
0 9 :3 0 7d
:2 0 7e :3 0 237
239 0 f0 238
23b :3 0 23c :2 0
3e :3 0 3f :3 0
23e 23f 0 3e
:3 0 4f :3 0 241
242 0 c :2 0
2 :4 0 7f :2 0
51 :2 0 13 :2 0
f3 247 249 :3 0
f6 243 24b 240
24d 24c :2 0 41
:3 0 52 :3 0 24f
250 0 43 :4 0
80 :4 0 46 :2 0
47 :3 0 fa 254
256 :3 0 46 :2 0
77 :4 0 fd 258
25a :3 0 46 :2 0
81 :4 0 100 25c
25e :3 0 46 :2 0
4b :3 0 4c :3 0
82 :3 0 83 :3 0
263 264 0 103
262 266 4d :4 0
105 261 269 108
260 26b :3 0 46
:2 0 84 :4 0 10b
26d 26f :3 0 46
:2 0 49 :4 0 10e
271 273 :3 0 46
:2 0 b :3 0 111
275 277 :3 0 46
:2 0 49 :4 0 114
279 27b :3 0 117
251 27d :2 0 27f
11a 280 24e 27f
0 281 11c 0
2ab 3e :3 0 3f
:3 0 282 283 0
3e :3 0 4f :3 0
285 286 0 13
:2 0 2 :4 0 85
:2 0 51 :2 0 13
:2 0 11e 28b 28d
:3 0 121 287 28f
284 291 290 :2 0
41 :3 0 52 :3 0
293 294 0 43
:4 0 86 :4 0 46
:2 0 47 :3 0 125
298 29a :3 0 128
295 29c :2 0 29e
12b 29f 292 29e
0 2a0 12d 0
2ab 87 :3 0 88
:3 0 2a1 2a2 0
82 :3 0 83 :3 0
2a4 2a5 0 b
:3 0 89 :3 0 12f
2a3 2a9 :2 0 2ab
133 2ac 23d 2ab
0 2ad 137 0
48b 2e :3 0 8a
:3 0 f :3 0 10
:3 0 b :3 0 d
:3 0 2c :3 0 17
:3 0 18 :3 0 1e
:3 0 22 :3 0 2b
:3 0 20 :3 0 1b
:3 0 2a :3 0 14
:3 0 16 :3 0 35
:3 0 36 :3 0 2f
:3 0 32 :3 0 1d
:3 0 38 :3 0 11
:3 0 12 :3 0 3a
:3 0 21 :3 0 e
:3 0 139 2af 2ca
2ae 2cb 0 48b
4b :3 0 2e :3 0
1f :2 0 154 2cd
2d0 8b :2 0 8c
:2 0 159 2d2 2d4
:3 0 3e :3 0 3f
:3 0 2d6 2d7 0
3e :3 0 4f :3 0
2d9 2da 0 c
:2 0 2 :4 0 8d
:2 0 51 :2 0 13
:2 0 15c 2df 2e1
:3 0 15f 2db 2e3
2d8 2e5 2e4 :2 0
41 :3 0 52 :3 0
2e7 2e8 0 43
:4 0 8e :4 0 46
:2 0 47 :3 0 163
2ec 2ee :3 0 46
:2 0 77 :4 0 166
2f0 2f2 :3 0 46
:2 0 81 :4 0 169
2f4 2f6 :3 0 46
:2 0 4b :3 0 4c
:3 0 2e :3 0 16c
2fa 2fc 4d :4 0
16e 2f9 2ff 171
2f8 301 :3 0 46
:2 0 84 :4 0 174
303 305 :3 0 46
:2 0 49 :4 0 177
307 309 :3 0 46
:2 0 b :3 0 17a
30b 30d :3 0 46
:2 0 49 :4 0 17d
30f 311 :3 0 180
2e9 313 :2 0 315
183 316 2e6 315
0 317 185 0
343 3e :3 0 3f
:3 0 318 319 0
3e :3 0 4f :3 0
31b 31c 0 13
:2 0 2 :4 0 8f
:2 0 51 :2 0 13
:2 0 187 321 323
:3 0 18a 31d 325
31a 327 326 :2 0
41 :3 0 52 :3 0
329 32a 0 43
:4 0 90 :4 0 46
:2 0 47 :3 0 18e
32e 330 :3 0 191
32b 332 :2 0 334
194 335 328 334
0 336 196 0
343 87 :3 0 88
:3 0 337 338 0
4b :3 0 2e :3 0
1f :2 0 198 33a
33d b :3 0 89
:3 0 19b 339 341
:2 0 343 19f 43a
23 :3 0 1f :2 0
344 345 0 439
4b :3 0 2a :3 0
1f :2 0 1a3 347
34a 91 :2 0 13
:2 0 1a8 34c 34e
:3 0 5b :3 0 23
:3 0 6d :3 0 6e
:3 0 b :4 0 92
1 :8 0 35b 9
:3 0 70 :3 0 356
357 0 71 :3 0
358 359 0 35b
1ab 3ae 72 :3 0
9 :3 0 70 :3 0
35d 35e 0 71
:3 0 35f 360 0
362 1ae 364 1b0
363 362 :2 0 3ac
73 :3 0 9 :3 0
70 :3 0 367 368
0 71 :3 0 369
36a 0 3a4 3e
:3 0 3f :3 0 36c
36d 0 3e :3 0
4f :3 0 36f 370
0 74 :2 0 2
:4 0 93 :2 0 51
:2 0 13 :2 0 1b2
375 377 :3 0 1b5
371 379 36e 37b
37a :2 0 41 :3 0
52 :3 0 37d 37e
0 43 :4 0 94
:4 0 46 :2 0 47
:3 0 1b9 382 384
:3 0 46 :2 0 77
:4 0 1bc 386 388
:3 0 46 :2 0 78
:4 0 1bf 38a 38c
:3 0 46 :2 0 71
:3 0 1c2 38e 390
:3 0 46 :2 0 79
:4 0 1c5 392 394
:3 0 46 :2 0 7a
:3 0 7b :3 0 13
:2 0 7c :2 0 1c8
397 39b 1cc 396
39d :3 0 1cf 37f
39f :2 0 3a1 1d2
3a2 37c 3a1 0
3a3 1d4 0 3a4
1d6 3a7 :3 0 3a7
0 3a7 3a6 3a4
3a5 :6 0 3a9 4
:3 0 1d9 3ab 1db
3aa 3a9 :2 0 3ac
1dd :2 0 3ae 0
3ae 3ad 35b 3ac
:6 0 430 1 :3 0
4b :3 0 2b :3 0
1f :2 0 1e0 3b0
3b3 4b :3 0 8b
:2 0 23 :3 0 1f
:2 0 1e3 3b5 3b9
1e8 3b6 3bb :3 0
6d :3 0 5b :3 0
2b :3 0 6e :3 0
b :4 0 95 1
:8 0 3d8 96 :4 0
3c3 :3 0 89 :3 0
7d :2 0 1ed 3c6
3c7 :3 0 9 :3 0
70 :3 0 3c9 3ca
0 1f :2 0 3cb
3cc 0 3ce 1f0
3d5 9 :3 0 70
:3 0 3cf 3d0 0
97 :2 0 3d1 3d2
0 3d4 1f2 3d6
3c8 3ce 0 3d7
0 3d4 0 3d7
1f4 0 3d8 1f7
42b 72 :3 0 9
:3 0 70 :3 0 3da
3db 0 71 :3 0
3dc 3dd 0 3df
1fa 3e1 1fc 3e0
3df :2 0 429 73
:3 0 9 :3 0 70
:3 0 3e4 3e5 0
71 :3 0 3e6 3e7
0 421 3e :3 0
3f :3 0 3e9 3ea
0 3e :3 0 4f
:3 0 3ec 3ed 0
74 :2 0 2 :4 0
98 :2 0 51 :2 0
13 :2 0 1fe 3f2
3f4 :3 0 201 3ee
3f6 3eb 3f8 3f7
:2 0 41 :3 0 52
:3 0 3fa 3fb 0
43 :4 0 99 :4 0
46 :2 0 47 :3 0
205 3ff 401 :3 0
46 :2 0 77 :4 0
208 403 405 :3 0
46 :2 0 78 :4 0
20b 407 409 :3 0
46 :2 0 71 :3 0
20e 40b 40d :3 0
46 :2 0 79 :4 0
211 40f 411 :3 0
46 :2 0 7a :3 0
7b :3 0 13 :2 0
7c :2 0 214 414
418 218 413 41a
:3 0 21b 3fc 41c
:2 0 41e 21e 41f
3f9 41e 0 420
220 0 421 222
424 :3 0 424 0
424 423 421 422
:6 0 426 6 :3 0
225 428 227 427
426 :2 0 429 229
:2 0 42b 0 42b
42a 3d8 429 :6 0
42d 1 :3 0 22c
42e 3bc 42d 0
42f 22e 0 430
230 431 34f 430
0 432 233 0
439 2e :3 0 9a
:3 0 f :3 0 235
434 436 433 437
0 439 237 43b
2d5 343 0 43c
0 439 0 43c
23b 0 48b 3e
:3 0 3f :3 0 43d
43e 0 3e :3 0
4f :3 0 440 441
0 9b :2 0 2
:4 0 9c :2 0 51
:2 0 13 :2 0 23e
446 448 :3 0 241
442 44a 43f 44c
44b :2 0 41 :3 0
52 :3 0 44e 44f
0 43 :4 0 9d
:4 0 46 :2 0 47
:3 0 245 453 455
:3 0 46 :2 0 77
:4 0 248 457 459
:3 0 46 :2 0 81
:4 0 24b 45b 45d
:3 0 46 :2 0 4b
:3 0 4c :3 0 9
:3 0 70 :3 0 462
463 0 24e 461
465 4d :4 0 250
460 468 253 45f
46a :3 0 46 :2 0
84 :4 0 256 46c
46e :3 0 46 :2 0
49 :4 0 259 470
472 :3 0 46 :2 0
3 :3 0 25c 474
476 :3 0 46 :2 0
49 :4 0 25f 478
47a :3 0 262 450
47c :2 0 47e 265
47f 44d 47e 0
480 267 0 48b
87 :3 0 9e :3 0
481 482 0 9
:3 0 70 :3 0 484
485 0 3 :3 0
89 :3 0 269 483
489 :2 0 48b 26d
52c 9 :3 0 9f
:2 0 4 48c 48d
0 3e :3 0 3f
:3 0 48f 490 0
3e :3 0 4f :3 0
492 493 0 74
:2 0 2 :4 0 a0
:2 0 51 :2 0 13
:2 0 27b 498 49a
:3 0 27e 494 49c
491 49e 49d :2 0
41 :3 0 52 :3 0
4a0 4a1 0 43
:4 0 a1 :4 0 46
:2 0 47 :3 0 282
4a5 4a7 :3 0 285
4a2 4a9 :2 0 4c2
41 :3 0 52 :3 0
4ab 4ac 0 43
:4 0 a2 :4 0 46
:2 0 9 :3 0 a3
:3 0 4b1 4b2 0
288 4b0 4b4 :3 0
46 :2 0 84 :4 0
28b 4b6 4b8 :3 0
46 :2 0 9 :3 0
a4 :3 0 4bb 4bc
0 28e 4ba 4be
:3 0 291 4ad 4c0
:2 0 4c2 294 4c3
49f 4c2 0 4c4
297 0 4d4 a5
:3 0 3d :3 0 9
:3 0 a3 :3 0 a6
:3 0 9 :3 0 a4
:3 0 3 :3 0 a7
:4 0 a8 1 :a 0
4d0 :2 0 4d4 4c6
4ce :2 0 8 :3 0
3d :3 0 4d2 :2 0
4d4 299 4d6 29d
4d5 4d4 :2 0 52a
73 :3 0 9 :3 0
70 :3 0 4d9 4da
0 71 :3 0 4db
4dc 0 522 3e
:3 0 3f :3 0 4de
4df 0 3e :3 0
4f :3 0 4e1 4e2
0 74 :2 0 2
:4 0 a9 :2 0 51
:2 0 13 :2 0 29f
4e7 4e9 :3 0 2a2
4e3 4eb 4e0 4ed
4ec :2 0 41 :3 0
52 :3 0 4ef 4f0
0 43 :4 0 aa
:4 0 46 :2 0 47
:3 0 2a6 4f4 4f6
:3 0 2a9 4f1 4f8
:2 0 512 41 :3 0
52 :3 0 4fa 4fb
0 43 :4 0 78
:4 0 46 :2 0 71
:3 0 2ac 4ff 501
:3 0 46 :2 0 79
:4 0 2af 503 505
:3 0 46 :2 0 7a
:3 0 7b :3 0 13
:2 0 7c :2 0 2b2
508 50c 2b6 507
50e :3 0 2b9 4fc
510 :2 0 512 2bc
513 4ee 512 0
514 2bf 0 522
a5 :3 0 3d :3 0
a6 :3 0 9 :3 0
a4 :3 0 3 :3 0
a7 :4 0 ab 1
:a 0 51e :2 0 522
516 51c :2 0 8
:3 0 3d :3 0 520
:2 0 522 2c1 525
:3 0 525 0 525
524 522 523 :6 0
527 1 :3 0 2c6
529 2c8 528 527
:2 0 52a 2ca :2 0
52c 2cd 52c 52b
48b 52a :6 0 52d
:2 0 2 14 52c
530 :3 0 52f 52d
531 :8 0 
2f9
4
:3 0 1 3 1
7 1 b 3
6 a e 1
18 1 16 1
1f 1 1d 1
26 1 24 1
2d 1 2b 1
34 1 32 1
3b 1 39 1
42 1 40 1
47 1 4c 1
51 1 56 1
5b 1 60 1
65 1 6c 1
6a 1 71 1
77 1 7d 1
83 1 89 1
8f 1 94 1
9b 1 99 1
a0 1 a6 1
ac 1 b2 1
b8 1 be 1
c4 1 ca 1
d0 1 d6 1
db 1 e0 1
e6 1 ec 1
f2 1 f8 1
fd 1 103 1
109 1 10f 1
11c 1 128 2
12b 12d 2 130
132 2 134 136
2 138 13a 2
13c 13e 1 144
2 146 147 2
140 149 2 14b
14d 2 14f 151
2 153 155 2
157 159 2 15b
15d 4 124 12a
12f 15f 1 161
1 163 2 16d
16f 3 16b 16c
171 2 17a 17c
2 179 17e 1
180 1 182 2
1d9 1de 1 1e5
1 1e0 2 1f8
1fa 3 1f6 1f7
1fc 2 205 207
2 209 20b 2
20d 20f 2 211
213 2 215 217
3 21c 21d 21e
2 219 220 2
204 222 1 224
1 226 2 1ef
227 1 22b 1
1ea 2 1e8 22f
1 23a 2 236
23a 2 246 248
3 244 245 24a
2 253 255 2
257 259 2 25b
25d 1 265 2
267 268 2 25f
26a 2 26c 26e
2 270 272 2
274 276 2 278
27a 2 252 27c
1 27e 1 280
2 28a 28c 3
288 289 28e 2
297 299 2 296
29b 1 29d 1
29f 3 2a6 2a7
2a8 3 281 2a0
2aa 1 2ac 1a
2b0 2b1 2b2 2b3
2b4 2b5 2b6 2b7
2b8 2b9 2ba 2bb
2bc 2bd 2be 2bf
2c0 2c1 2c2 2c3
2c4 2c5 2c6 2c7
2c8 2c9 2 2ce
2cf 1 2d3 2
2d1 2d3 2 2de
2e0 3 2dc 2dd
2e2 2 2eb 2ed
2 2ef 2f1 2
2f3 2f5 1 2fb
2 2fd 2fe 2
2f7 300 2 302
304 2 306 308
2 30a 30c 2
30e 310 2 2ea
312 1 314 1
316 2 320 322
3 31e 31f 324
2 32d 32f 2
32c 331 1 333
1 335 2 33b
33c 3 33e 33f
340 3 317 336
342 2 348 349
1 34d 2 34b
34d 2 355 35a
1 361 1 35c
2 374 376 3
372 373 378 2
381 383 2 385
387 2 389 38b
2 38d 38f 2
391 393 3 398
399 39a 2 395
39c 2 380 39e
1 3a0 1 3a2
2 36b 3a3 1
3a7 1 366 2
364 3ab 2 3b1
3b2 2 3b7 3b8
1 3ba 2 3b4
3ba 1 3c5 2
3c4 3c5 1 3cd
1 3d3 2 3d5
3d6 2 3c2 3d7
1 3de 1 3d9
2 3f1 3f3 3
3ef 3f0 3f5 2
3fe 400 2 402
404 2 406 408
2 40a 40c 2
40e 410 3 415
416 417 2 412
419 2 3fd 41b
1 41d 1 41f
2 3e8 420 1
424 1 3e3 2
3e1 428 1 42b
1 42e 2 3ae
42f 1 431 1
435 3 346 432
438 2 43a 43b
2 445 447 3
443 444 449 2
452 454 2 456
458 2 45a 45c
1 464 2 466
467 2 45e 469
2 46b 46d 2
46f 471 2 473
475 2 477 479
2 451 47b 1
47d 1 47f 3
486 487 488 d
164 183 186 189
18c 18f 192 232
2ad 2cc 43c 480
48a 2 497 499
3 495 496 49b
2 4a4 4a6 2
4a3 4a8 2 4af
4b3 2 4b5 4b7
2 4b9 4bd 2
4ae 4bf 2 4aa
4c1 1 4c3 3
4c4 4cf 4d3 1
48e 2 4e6 4e8
3 4e4 4e5 4ea
2 4f3 4f5 2
4f2 4f7 2 4fe
500 2 502 504
3 509 50a 50b
2 506 50d 2
4fd 50f 2 4f9
511 1 513 4
4dd 514 51d 521
1 525 1 4d8
2 4d6 529 2b
1b 22 29 30
37 3e 45 4a
4f 54 59 5e
63 68 6f 75
7b 81 87 8d
92 97 9e a4
aa b0 b6 bc
c2 c8 ce d4
d9 de e4 ea
f0 f6 fb 101
107 10d 114 
1
4
0 
530
0
1
14
8
2f
0 1 2 1 4 1 6 1
0 0 0 0 0 0 0 0
0 0 0 0 
b 1 0
2 0 1
83 1 0
be 1 0
3 1 0
f2 1 0
56 1 0
16 1 0
ec 1 0
1d 1 0
b2 1 0
a0 1 0
40 1 0
d0 1 0
94 1 0
c4 1 0
109 1 0
60 1 0
71 1 0
47 1 0
51 1 0
e0 1 0
7d 1 0
b8 1 0
99 1 0
db 1 0
89 1 0
e6 1 0
32 1 0
f8 1 0
4c 1 0
fd 1 0
6a 1 0
8f 1 0
65 1 0
39 1 0
2b 1 0
7 1 0
d6 1 0
ca 1 0
5b 1 0
24 1 0
77 1 0
ac 1 0
a6 1 0
10f 1 0
103 1 0
0

/
SHOW ERRORS FUNCTION SP_ACPT_ETC_ASSIGN;
ALTER FUNCTION SP_ACPT_ETC_ASSIGN COMPILE DEBUG;
/****************************************************************************************
Accept ETC and/or FORECAST_START, WBS level
  Bitflag =
    1 - ETC
    2 - FORECAST DATES
    3 - ETC + FORECAST DATES
****************************************************************************************/
Prompt == FUNCTION SP_ACPT_ETC_WBS ==
CREATE OR REPLACE FUNCTION SP_ACPT_ETC_WBS wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
ec
2 :e:
1FUNCTION:
1SP_ACPT_ETC_WBS:
1ID:
1VARCHAR2:
1BITFLAG001:
1PLS_INTEGER:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_PRJ_ASSIGNMENT_ID:
132:
1H_PROJECT_ID:
1H_RANK:
1200:
1H_REC_USER:
1H_RES_ID:
1H_RES_TYPE:
11:
1H_TA_START_DT:
1TIMESTAMP:
1H_TA_FINISH_DT:
1H_START_DATE:
1H_FINISH_DATE:
1H_START_DT:
1H_FINISH_DT:
1H_FINISH_D:
1DATE:
1H_WEEK_OF:
1H_LAST_SUBMITTED:
1H_TASK_ID:
1H_TSK_ASSIGNMENT_ID:
1H_BIT_FLAG:
1H_BITFLAG:
10:
1H_CONTOUR:
1H_EFFORT:
1H_REMAINING:
1H_TOTAL:
1H_TYPE:
1H_PASSED:
1H_CONSTR_DT:
1H_DAYOFYEAR:
1H_FINISH_TIME:
18:
1H_MINUTE:
1H_CONSTR_TYPE:
1H_ETC:
1H_TEMP_VAR:
1H_TEMP_SC_STATE:
1H_SCHED_START:
1H_SCHED_FINISH:
1H_SCHED_WORK:
1H_WORK_TODATE:
1H_MLST_PERCENT:
1H_REG_COST:
1NUMBER:
10.00:
1H_REG_SALE:
1H_SD:
1H_FD:
1H_BILLABLE:
1H_CAPITAL_EXPENSE:
1H_TPH_DELETE:
1H_NOT_EFFORT_PA:
1H_FF_FLAG:
1NULL_RETURN:
1LONG:
1ORA_TEMP_SC_STATE:
1CURSOR:
1CAWTSK:
1PROJECT_ID:
1ELEMENT_ID:
1SCHED_EFFORT:
1PLANNED_EFFORT:
1FORECAST_START:
1COALESCE:
1PLANNED_START:
1SCHED_START:
1FORECAST_FINISH:
1PLANNED_FINISH:
1SCHED_FINISH:
1TMT_WBS:
1RANK:
1TYPE_ID:
1PMO_CNST:
1SC_TASK:
1SC_TASK_TYPE1:
1SC_TASK_TYPE2:
1SC_TASK_TYPE3:
1SC_MILESTONE:
1SC_PERSONAL_TASK:
1SC_ACTIVE_ADM_TASK:
1SC_INACTIVE_ADM_TASK:
1SC_WBS_CHANGE_REQUEST_TASK:
1SC_WBS_ACTION_ITEM_TASK:
1SC_WBS_RISK_TASK:
1SC_WBS_ISSUE_TASK:
1SC_WBS_REQUIREMENT_TASK:
1SC_WBS_DEFECT_TASK:
1SC_WBS_SERVICE_REQUEST_TASK:
1WORK_TODATE:
1ETC:
1SELECT PROJECT_ID, ELEMENT_ID, SCHED_EFFORT+PLANNED_EFFORT,:n         CASE WH+
1EN FORECAST_START IS NULL THEN COALESCE(PLANNED_START, SCHED_START) ELSE FORE+
1CAST_START END,:n         CASE WHEN FORECAST_FINISH IS NULL THEN COALESCE(PLA+
1NNED_FINISH, SCHED_FINISH) ELSE FORECAST_FINISH END:n         FROM TMT_WBS WH+
1ERE RANK LIKE h_rank AND TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,+
1PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.+
1SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PM+
1O_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.S+
1C_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,P+
1MO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK):n         AN+
1D (((SCHED_EFFORT+PLANNED_EFFORT) > 0 AND ((SCHED_EFFORT+PLANNED_EFFORT-WORK_+
1TODATE) <> ETC)) OR:n         (FORECAST_START IS NOT NULL OR FORECAST_FINISH +
1IS NOT NULL)) ORDER BY COALESCE(PLANNED_START, SCHED_START):
1CAWASS:
1TSK_ASSIGNMENT_ID:
1PROJECT_ASSIGNMENT_ID:
1SCHED_WORK:
1PLANNED_WORK:
1C_CONTOUR:
1LAST_SUBMITTED:
1UNRESOLVED_START:
1UNRESOLVED_FINISH:
1BILLABLE:
1CAPITAL_EXPENSE:
1REGULAR_COST:
1REGULAR_RATE:
1BIT_FLAG:
1MILESTONE_PERCENT:
1RESOURCE_ID:
1RESOURCE_TYPE:
1TASK_ASSIGNMENTS:
1TASK_ID:
1SELECT:n         TSK_ASSIGNMENT_ID,:n         PROJECT_ASSIGNMENT_ID,:n       +
1  SCHED_WORK+PLANNED_WORK,:n         FORECAST_START,:n         FORECAST_FINIS+
1H,:n         CASE WHEN h_bitflag = 2 AND FORECAST_START IS NuLL THEN 0 ELSE E+
1TC END,:n         SCHED_WORK+PLANNED_WORK - WORK_TODATE,:n         ETC,:n    +
1     C_CONTOUR,:n         LAST_SUBMITTED,:n         CASE WHEN FORECAST_FINISH+
1 IS NOT NuLL AND FORECAST_START IS NuLL AND FORECAST_FINISH BETWEEN COALESCE(+
1PLANNED_START, SCHED_START) AND COALESCE(PLANNED_FINISH, SCHED_FINISH) THEN 1+
1 ELSE 0 END,:n         UNRESOLVED_START,:n         UNRESOLVED_FINISH,:n      +
1   BILLABLE,:n         CAPITAL_EXPENSE,:n         REGULAR_COST,:n         REG+
1ULAR_RATE,:n         BIT_FLAG,:n         MILESTONE_PERCENT,:n         RESOURC+
1E_ID,:n         RESOURCE_TYPE,:n         WORK_TODATE:n         FROM TASK_ASSI+
1GNMENTS:n         WHERE PROJECT_ID = h_project_id:n         AND TASK_ID = h_t+
1ask_id:n         AND (((SCHED_WORK+PLANNED_WORK) > 0:n         AND ((SCHED_WO+
1RK + PLANNED_WORK - WORK_TODATE) <> ETC)):n         OR (FORECAST_START IS NOT+
1 NULL OR FORECAST_FINISH IS NOT NULL)):
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_ACPT_ETC_WBS(:
1':
1, :
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
1106:
1+:
1FPUT:
1               ENTER:: SP_ACPT_ETC_WBS(107) :: cplan.sql(19208) :: :
1RTRIM:
1SELECT RTRIM(RANK), TYPE_ID INTO h_rank, h_type:n      FROM TMT_WBS WHERE ELE+
1MENT_ID = id/*HOST(h_id)*/:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
1125:
1           EXCEPTION:: SP_ACPT_ETC_WBS(126) :: cplan.sql(19227) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1=:
1SC_SQL_SUCCESS:
1IS NOT NULL:
1%:
1-:
1OPEN:
1LOOP:
1FOUND:
1100:
1!=:
1EXIT:
12:
1FC_ACPT_DURATION:
1FC_ACPT_EFFORT:
1ELSIF:
1CANNOT_ACCEPT_FROM_PROJECT_AFF:
1CLOSE:
1232:
1           EXCEPTION:: SP_ACPT_ETC_WBS(233) :: cplan.sql(19334) :: :
1>:
1CONSTRAINT_DT:
1CONSTRAINT_TYPE:
1SELECT CONSTRAINT_DT, CONSTRAINT_TYPE, FORECAST_START, FORECAST_FINISH:n     +
1             INTO h_constr_DT, h_constr_type, h_start_DT, h_finish_DT:n      +
1            FROM TMT_WBS WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_t+
1ask_id:
1256:
1           EXCEPTION:: SP_ACPT_ETC_WBS(257) :: cplan.sql(19358) :: :
1TO_DATE:
1UPDATE TMT_WBS SET:n                        CONSTRAINT_DT =:n                +
1        CASE:n                        WHEN h_constr_type IN (1,2,3,4) AND TO_+
1DATE(SubStr(h_constr_DT, 1, 10)) < TO_DATE(SubStr(h_finish_DT, 1, 10)) THEN h+
1_finish_DT:n                        WHEN h_constr_type IN (0,5,6,7) THEN h_fi+
1nish_DT:n                        ELSE h_constr_DT:n                        EN+
1D,:n                        CONSTRAINT_TYPE =:n                        CASE:n+
1                        WHEN h_constr_type IN (1,2,3,4) AND TO_DATE(SubStr(h_+
1constr_DT, 1, 10)) < TO_DATE(SubStr(h_finish_DT, 1, 10)) THEN 2:n            +
1            WHEN h_constr_type IN (0,5,6,7) THEN 2:n                        E+
1LSE h_constr_type:n                        END:n                        WHERE+
1 PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id:
1TRUE:
1292:
1           EXCEPTION:: SP_ACPT_ETC_WBS(293) :: cplan.sql(19394) :: :
1UPDATE TMT_WBS SET CONSTRAINT_DT = h_finish_DT, CONSTRAINT_TYPE = 2:n        +
1                WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id:
1312:
1           EXCEPTION:: SP_ACPT_ETC_WBS(313) :: cplan.sql(19414) :: :
1IS NULL:
1UPDATE TMT_WBS SET:n                        CONSTRAINT_DT =:n                +
1        CASE:n                        WHEN h_constr_type IN (0,5,6,7) AND TO_+
1DATE(SubStr(h_constr_DT, 1, 10)) > TO_DATE(SubStr(h_start_DT, 1, 10)) THEN h_+
1start_DT:n                        WHEN h_constr_type IN (1,2,3,4) THEN h_star+
1t_DT:n                        ELSE h_constr_DT:n                        END,+
1:n                        CONSTRAINT_TYPE =:n                        CASE:n  +
1                      WHEN h_constr_type IN (0,5,6,7) AND TO_DATE(SubStr(h_co+
1nstr_DT, 1, 10)) > TO_DATE(SubStr(h_start_DT, 1, 10)) THEN 6:n               +
1         WHEN h_constr_type IN (1,2,3,4) THEN 6:n                        ELSE+
1 h_constr_type:n                        END:n                        WHERE PR+
1OJECT_ID = h_project_id AND ELEMENT_ID = h_task_id:
1347:
1           EXCEPTION:: SP_ACPT_ETC_WBS(348) :: cplan.sql(19449) :: :
1UPDATE TMT_WBS SET CONSTRAINT_DT = h_start_DT, CONSTRAINT_TYPE = 6:n         +
1               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id:
1367:
1           EXCEPTION:: SP_ACPT_ETC_WBS(368) :: cplan.sql(19469) :: :
1388:
1           EXCEPTION:: SP_ACPT_ETC_WBS(389) :: cplan.sql(19490) :: :
1NLEN:
1LENGTH:
1NOT:
1MOD:
14:
1PMODEFINES:
1PMO_RETURN_ERROR_SP:
1INVALID_RANK_LIKE:
1ROLLUP_BITS:
1PMO_STPU:
1BIT_OR:
1SC_PROPOSED_BIT:
1UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PROP+
1OSED_BIT):n            WHERE PROJECT_ID = h_project_id AND RANK LIKE h_rank:
1425:
1           EXCEPTION:: SP_ACPT_ETC_WBS(426) :: cplan.sql(19527) :: :
1FC_ROLLUP:
1442:
1       RETURN_STATUS:: SP_ACPT_ETC_WBS(443) :: cplan.sql(19544) :: :
1STATE:: :
1 ID:: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
1449:
1           EXCEPTION:: SP_ACPT_ETC_WBS(450) :: cplan.sql(19551) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1ERROR:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1461:
1           EXCEPTION:: SP_ACPT_ETC_WBS(462) :: cplan.sql(19563) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
930
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
1c 81 b0 a3 a0 1c 81 b0
a3 a0 1c 81 b0 a3 a0 1c
81 b0 a3 a0 1c 81 b0 a3
a0 1c 81 b0 a3 a0 1c 81
b0 a3 a0 1c 81 b0 a3 a0
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 81 b0 a3
a0 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 81 b0 a3 a0
1c 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 81 b0 a3 a0
1c 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 81 b0 a3 a0
1c 51 81 b0 a0 f4 b4 bf
c8 :3b a0 12a bd b7 11 a4 b1
a0 f4 b4 bf c8 :30 a0 12a bd
b7 11 a4 b1 a3 :2 a0 6b 1c
81 b0 :2 a0 6b :2 a0 6b 6e a5
b a 10 :2 a0 6b 6e :2 a0 6b
6e a5 b 6e 7e a0 b4 2e
6e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :3 a0 a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 b7 19 3c :2 a0 d
:2 a0 d a0 51 d :8 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
6b a0 7e a0 6b b4 2e 5a
a0 7e b4 2e :2 5a a 10 :3 a0
7e 6e b4 2e :2 51 a5 b d
a0 7e 51 b4 2e d :2 a0 e9
dd b3 :7 a0 e9 d3 5 :2 a0 f
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c :2 a0 6b a0 7e
a0 6b b4 2e 5a a0 2b b7
19 3c :2 a0 51 a5 b 7e 51
b4 2e :2 a0 51 a5 b 7e 51
b4 2e :3 a0 6b d :7 a0 a5 b
d :2 a0 6b a0 d :3 a0 51 a5
b 7e 51 b4 2e d b7 19
3c b7 a0 4d d a0 4d d
:2 a0 e9 dd b3 :18 a0 e9 d3 5
:2 a0 f :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c :2 a0 6b
a0 7e a0 6b b4 2e 5a a0
2b b7 19 3c :1c a0 a5 b d
:2 a0 51 a5 b 7e 51 b4 2e
:3 a0 51 a5 b 7e 51 b4 2e
d a0 b7 :2 a0 51 a5 b a0
7e a0 6b b4 2e :3 a0 51 a5
b 7e 51 b4 2e d b7 :2 19
3c b7 a0 47 :2 a0 e9 c1 :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
51 a5 b 7e 51 b4 2e 5a
:2 a0 51 a5 b 7e 51 b4 2e
5a a 10 a0 4d d a0 4d
d :d a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f a0 7e b4 2e 5a
a0 7e b4 2e 5a :1b a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :8 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :2 19
3c a0 b7 a0 7e b4 2e :2 5a
a0 7e b4 2e :2 5a a 10 a0
7e b4 2e 5a :1b a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :8 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :2 19 3c
b7 :2 19 3c b7 19 3c b7 :2 19
3c b7 a0 47 :2 a0 e9 c1 :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
51 a5 b 7e 51 b4 2e 5a
:2 a0 51 a5 b :2 7e 51 b4 2e
b4 2e 5a a 10 a3 a0 1c
51 81 b0 a0 7e b4 2e 5a
:4 a0 a5 b 51 a5 b d 51
7e 51 b4 2e a0 7e 51 b4
2e a 10 :2 a0 7e 51 b4 2e
d b7 19 3c b7 19 3c a0
7e 51 b4 2e 7e b4 2e :2 a0
7e 51 b4 2e 5a 7e 51 b4
2e 52 10 :2 a0 6b :2 a0 6b :2 a0
a5 57 b7 19 3c b7 a4 b1
11 4f :b a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :3 a0 a5 b d b7 19
3c b7 19 3c :2 a0 51 a5 b
7e 51 b4 2e :2 a0 6b :2 a0 6b
d a0 b7 :2 a0 51 a5 b 7e
51 b4 2e :2 a0 6b :2 a0 51 a5
b d b7 :2 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 :2 a0 6b :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 :2 a0 6b :2 6e 7e
:2 a0 6b b4 2e 7e 6e b4 2e
7e :2 a0 6b b4 2e a5 57 b7
19 3c :9 a0 12a e9 dd :2 a0 65
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
:7 a0 12a e9 dd :2 a0 65 b7 a4
b1 11 4f b7 a6 9 a4 b1
11 68 4f 1d 17 b5 
930
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 30
56 5a 5e 62 65 69 86 71
75 78 79 81 70 a3 91 6d
95 96 9e 90 c0 ae 8d b2
b3 bb ad dd cb aa cf d0
d8 ca fa e8 c7 ec ed f5
e7 117 105 e4 109 10a 112 104
133 122 126 12e 101 14b 13a 13e
146 121 167 156 15a 162 11e 17f
16e 172 17a 155 19b 18a 18e 196
152 1b3 1a2 1a6 1ae 189 1cf 1be
1c2 1ca 186 1e7 1d6 1da 1e2 1bd
203 1f2 1f6 1fe 1ba 21f 20a 20e
211 212 21a 1f1 23c 22a 1ee 22e
22f 237 229 259 247 226 24b 24c
254 246 275 264 268 243 270 263
291 280 284 260 28c 27f 2ad 29c
2a0 27c 2a8 29b 2c9 2b8 2bc 298
2c4 2b7 2e5 2d4 2d8 2b4 2e0 2d3
301 2f0 2f4 2d0 2fc 2ef 31d 30c
310 2ec 318 30b 339 328 32c 334
308 351 340 344 34c 327 36e 35c
324 360 361 369 35b 38a 379 37d
358 385 378 3a6 395 399 375 3a1
394 3c2 3b1 3b5 391 3bd 3b0 3de
3cd 3d1 3ad 3d9 3cc 3fa 3e9 3ed
3c9 3f5 3e8 416 405 409 411 3e5
42e 41d 421 429 404 44a 439 43d
401 445 438 466 455 459 435 461
454 482 471 475 451 47d 470 49e
48d 491 46d 499 48c 4ba 4a9 4ad
489 4b5 4a8 4d6 4c5 4c9 4d1 4a5
4ee 4dd 4e1 4e9 4c4 50a 4f9 4fd
4c1 505 4f8 526 515 519 4f5 521
514 542 531 535 511 53d 530 55e
54d 551 52d 559 54c 57a 569 56d
549 575 568 596 585 589 591 565
5b1 59d 5a1 5a9 5ac 584 5b8 5bc
581 5cd 5d0 5d4 5d8 5dc 5e0 5e4
5e8 5ec 5f0 5f4 5f8 5fc 600 604
608 60c 610 614 618 61c 620 624
628 62c 630 634 638 63c 640 644
648 64c 650 654 658 65c 660 664
668 66c 670 674 678 67c 680 684
688 68c 690 694 698 69c 6a0 6a4
6a8 6ac 6b0 6b4 6b8 6bc 6c0 6cc
6d1 6d3 6df 6e3 6e5 6e9 6fa 6fb
6fe 702 706 70a 70e 712 716 71a
71e 722 726 72a 72e 732 736 73a
73e 742 746 74a 74e 752 756 75a
75e 762 766 76a 76e 772 776 77a
77e 782 786 78a 78e 792 796 79a
79e 7a2 7a6 7aa 7ae 7b2 7b6 7ba
7be 7c2 7ce 7d3 7d5 7e1 7e5 803
7eb 7ef 7f3 7f6 7fe 7ea 80a 80e
7e7 812 816 81a 81d 822 823 1
825 82a 82e 832 835 83a 83e 842
845 84a 84b 84d 852 855 859 85a
85f 864 867 86c 86d 872 875 879
87a 87f 882 887 888 88d 890 895
896 89b 89e 8a2 8a6 8aa 8ab 8ad
8b2 8b3 8b5 8b6 8bb 8be 8c3 8c4
8c9 8cc 8d1 8d2 8d7 8da 8de 8df
8e4 8e7 8ec 8ed 8f2 8f5 8fa 8fb
900 901 906 908 90c 90f 913 917
91a 91e 922 925 928 92d 930 933
936 937 93c 93d 1 93f 944 948
94c 94f 954 959 95c 960 961 966
967 96c 96e 972 975 979 97d 981
985 989 98d 991 994 998 99c 9a0
9a4 9a8 9ac 9b0 9b4 9b8 9c4 9c8
9cc 9cf 9d3 9d7 9d9 9dd 9e1 9e5
9e8 9ec 9f0 9f2 9f3 9f8 1 9fc
a00 a04 a07 a0b a0f a13 a17 a1a
a1e a22 a25 a28 a2d a30 a33 a36
a37 a3c a3d 1 a3f a44 a48 a4c
a4f a54 a59 a5c a60 a61 a66 a69
a6e a6f a74 a77 a7c a7d a82 a85
a89 a8a a8f a92 a97 a98 a9d aa0
aa4 aa8 aab aae aaf ab1 ab2 ab7
ab8 abd abf ac3 ac6 ac8 acc ace
ada adc ade adf ae4 ae8 aea af6
af8 afc b00 b03 b07 b0a b0e b11
b12 b17 b1a b1e b21 b22 b27 b2a
1 b2d b32 b36 b3a b3e b41 b46
b47 b4c b4f b52 b53 b55 b59 b5d
b60 b63 b64 b69 b6d b71 b75 b7a
b7e b7f b83 b87 b8b b8f b93 b97
b9b ba0 ba5 ba9 bad bb1 bb6 bba
bbe bc1 bc4 bc8 bca bce bd2 bd5
bd8 bdc bde be2 be6 be9 bed bf1
bf4 bf8 bfb bff c02 c03 c08 c0b
c0f c15 c17 c1b c1e c22 c26 c29
c2a c2c c2f c32 c33 c38 c3c c40
c43 c44 c46 c49 c4c c4d c52 c56
c5a c5e c61 c65 c69 c6d c71 c75
c79 c7d c81 c82 c84 c88 c8c c90
c93 c97 c9b c9f ca3 ca7 caa cab
cad cb0 cb3 cb4 cb9 cbd cbf cc3
cc6 cc8 ccc ccd cd1 cd5 cd6 cda
cde ce2 ce7 ceb cec cf0 cf4 cf8
cfc d00 d04 d08 d0c d10 d14 d18
d1c d20 d24 d28 d2c d30 d34 d38
d3c d40 d44 d48 d4c d51 d56 d5a
d5e d62 d67 d6b d6f d72 d75 d79
d7b d7f d83 d86 d89 d8d d8f d93
d97 d9a d9e da2 da5 da9 dac db0
db3 db4 db9 dbc dc0 dc6 dc8 dcc
dcf dd3 dd7 ddb ddf de3 de7 deb
def df3 df7 dfb dff e03 e07 e0b
e0f e13 e17 e1b e1f e23 e27 e2b
e2f e33 e37 e3b e3f e40 e42 e46
e4a e4e e51 e52 e54 e57 e5a e5b
e60 e64 e68 e6c e6f e70 e72 e75
e78 e79 e7e e82 e86 e88 e8c e90
e93 e94 e96 e9a e9d ea1 ea4 ea5
eaa eae eb2 eb6 eb9 eba ebc ebf
ec2 ec3 ec8 ecc ece ed2 ed6 ed9
edb edf ee6 eea eee ef3 ef5 ef9
efd f00 f04 f08 f0a f0e f12 f16
f19 f1d f21 f23 f24 f29 1 f2d
f31 f35 f38 f3c f40 f44 f48 f4b
f4f f53 f56 f59 f5e f61 f64 f67
f68 f6d f6e 1 f70 f75 f79 f7d
f80 f85 f8a f8d f91 f92 f97 f9a
f9f fa0 fa5 fa8 fad fae fb3 fb6
fba fbb fc0 fc3 fc8 fc9 fce fd1
fd5 fd9 fdc fdf fe0 fe2 fe3 fe8
fe9 fee ff0 ff4 ff7 ff9 ffd fff
100b 100d 100f 1010 1015 1019 101b 1027
1029 102d 1031 1034 1035 1037 103a 103d
103e 1043 1046 104a 104e 1051 1052 1054
1057 105a 105b 1060 1 1063 1068 106c
106d 1071 1075 1076 107a 107e 1082 1086
108a 108e 1092 1096 109a 109e 10a2 10a6
10aa 10ae 10ba 10be 10c2 10c5 10c9 10cd
10cf 10d3 10d7 10db 10de 10e2 10e6 10e8
10e9 10ee 1 10f2 10f6 10fa 10fd 1101
1105 1109 110d 1110 1114 1118 111b 111e
1123 1126 1129 112c 112d 1132 1133 1
1135 113a 113e 1142 1145 114a 114f 1152
1156 1157 115c 115f 1164 1165 116a 116d
1172 1173 1178 117b 117f 1180 1185 1188
118d 118e 1193 1196 119a 119e 11a1 11a4
11a5 11a7 11a8 11ad 11ae 11b3 11b5 11b9
11bc 11be 11c2 11c4 11d0 11d2 11d4 11d5
11da 11de 11e0 11ec 11ee 11f2 11f5 11f6
11fb 11fe 1202 1205 1206 120b 120e 1212
1216 121a 121e 1222 1226 122a 122e 1232
1236 123a 123e 1242 1246 124a 124e 1252
1256 125a 125e 1262 1266 126a 126e 1272
1276 127a 1286 128a 128f 1293 1296 1297
129c 12a0 12a4 12a7 12aa 12ae 12b0 12b4
12b8 12bb 12be 12c2 12c4 12c8 12cc 12cf
12d1 12d5 12d9 12dd 12e0 12e4 12e8 12ea
12eb 12f0 1 12f4 12f8 12fc 12ff 1303
1307 130b 130f 1312 1316 131a 131d 1320
1325 1328 132b 132e 132f 1334 1335 1
1337 133c 1340 1344 1347 134c 1351 1354
1358 1359 135e 1361 1366 1367 136c 136f
1374 1375 137a 137d 1381 1382 1387 138a
138f 1390 1395 1398 139c 13a0 13a3 13a6
13a7 13a9 13aa 13af 13b0 13b5 13b7 13bb
13be 13c0 13c4 13c6 13d2 13d4 13d6 13d7
13dc 13e0 13e2 13ee 13f0 13f2 13f6 13fa
13fe 1402 1406 140a 140e 1412 141e 1422
1427 142b 142e 142f 1434 1438 143c 143f
1442 1446 1448 144c 1450 1453 1456 145a
145c 1460 1464 1467 1469 146d 1471 1475
1478 147c 1480 1482 1483 1488 1 148c
1490 1494 1497 149b 149f 14a3 14a7 14aa
14ae 14b2 14b5 14b8 14bd 14c0 14c3 14c6
14c7 14cc 14cd 1 14cf 14d4 14d8 14dc
14df 14e4 14e9 14ec 14f0 14f1 14f6 14f9
14fe 14ff 1504 1507 150c 150d 1512 1515
1519 151a 151f 1522 1527 1528 152d 1530
1534 1538 153b 153e 153f 1541 1542 1547
1548 154d 154f 1553 1556 1558 155c 155e
156a 156c 156e 156f 1574 1578 157a 1586
1588 158a 158e 1592 1595 1599 159b 159f
15a2 15a3 15a8 15ab 15ae 15b2 15b5 15b6
15bb 15be 1 15c1 15c6 15ca 15cd 15ce
15d3 15d6 15da 15de 15e2 15e6 15ea 15ee
15f2 15f6 15fa 15fe 1602 1606 160a 160e
1612 1616 161a 161e 1622 1626 162a 162e
1632 1636 163a 163e 1642 164e 1652 1657
165b 165e 165f 1664 1668 166c 166f 1672
1676 1678 167c 1680 1683 1686 168a 168c
1690 1694 1697 1699 169d 16a1 16a5 16a8
16ac 16b0 16b2 16b3 16b8 1 16bc 16c0
16c4 16c7 16cb 16cf 16d3 16d7 16da 16de
16e2 16e5 16e8 16ed 16f0 16f3 16f6 16f7
16fc 16fd 1 16ff 1704 1708 170c 170f
1714 1719 171c 1720 1721 1726 1729 172e
172f 1734 1737 173c 173d 1742 1745 1749
174a 174f 1752 1757 1758 175d 1760 1764
1768 176b 176e 176f 1771 1772 1777 1778
177d 177f 1783 1786 1788 178c 178e 179a
179c 179e 179f 17a4 17a8 17aa 17b6 17b8
17ba 17be 17c2 17c6 17ca 17ce 17d2 17d6
17da 17e6 17ea 17ef 17f3 17f6 17f7 17fc
1800 1804 1807 180a 180e 1810 1814 1818
181b 181e 1822 1824 1828 182c 182f 1831
1835 1839 183d 1840 1844 1848 184a 184b
1850 1 1854 1858 185c 185f 1863 1867
186b 186f 1872 1876 187a 187d 1880 1885
1888 188b 188e 188f 1894 1895 1 1897
189c 18a0 18a4 18a7 18ac 18b1 18b4 18b8
18b9 18be 18c1 18c6 18c7 18cc 18cf 18d4
18d5 18da 18dd 18e1 18e2 18e7 18ea 18ef
18f0 18f5 18f8 18fc 1900 1903 1906 1907
1909 190a 190f 1910 1915 1917 191b 191e
1920 1924 1926 1932 1934 1936 1937 193c
1940 1942 194e 1950 1952 1956 195a 195d
195f 1963 1967 196a 196c 1970 1973 1975
1979 197d 1980 1982 1986 198d 1991 1995
199a 199c 19a0 19a4 19a7 19ab 19af 19b1
19b5 19b9 19bd 19c0 19c4 19c8 19ca 19cb
19d0 1 19d4 19d8 19dc 19df 19e3 19e7
19eb 19ef 19f2 19f6 19fa 19fd 1a00 1a05
1a08 1a0b 1a0e 1a0f 1a14 1a15 1 1a17
1a1c 1a20 1a24 1a27 1a2c 1a31 1a34 1a38
1a39 1a3e 1a41 1a46 1a47 1a4c 1a4f 1a54
1a55 1a5a 1a5d 1a61 1a62 1a67 1a6a 1a6f
1a70 1a75 1a78 1a7c 1a80 1a83 1a86 1a87
1a89 1a8a 1a8f 1a90 1a95 1a97 1a9b 1a9e
1aa0 1aa4 1aa6 1ab2 1ab4 1ab6 1ab7 1abc
1ac0 1ac2 1ace 1ad0 1ad4 1ad8 1adb 1adc
1ade 1ae1 1ae4 1ae5 1aea 1aed 1af1 1af5
1af8 1af9 1afb 1afe 1b01 1b04 1b05 1b0a
1b0b 1b10 1 1b13 1b30 1b1c 1b20 1b28
1b2b 1b1b 1b37 1b18 1b3b 1b3c 1b41 1b44
1b48 1b4c 1b50 1b54 1b55 1b57 1b5a 1b5b
1b5d 1b61 1b64 1b67 1b6a 1b6b 1b70 1b74
1b77 1b7a 1b7b 1 1b80 1b85 1b89 1b8d
1b90 1b93 1b94 1b99 1b9d 1b9f 1ba3 1ba6
1ba8 1bac 1baf 1bb3 1bb6 1bb9 1bba 1bbf
1bc2 1bc3 1bc8 1bcc 1bd0 1bd3 1bd6 1bd7
1bdc 1bdf 1be2 1be5 1be6 1 1beb 1bf0
1bf4 1bf8 1bfb 1bff 1c03 1c06 1c0a 1c0e
1c0f 1c14 1c16 1c1a 1c1d 1c1f 1c23 1c25
1c31 1c33 1c37 1c3b 1c3f 1c43 1c47 1c4b
1c4f 1c53 1c57 1c5b 1c5f 1c6b 1c6f 1c74
1c78 1c7b 1c7c 1c81 1c85 1c89 1c8c 1c8f
1c93 1c95 1c99 1c9d 1ca0 1ca3 1ca7 1ca9
1cad 1cb1 1cb4 1cb6 1cba 1cbe 1cc2 1cc5
1cc9 1ccd 1ccf 1cd0 1cd5 1 1cd9 1cdd
1ce1 1ce4 1ce8 1cec 1cf0 1cf4 1cf7 1cfb
1cff 1d02 1d05 1d0a 1d0d 1d10 1d13 1d14
1d19 1d1a 1 1d1c 1d21 1d25 1d29 1d2c
1d31 1d36 1d39 1d3d 1d3e 1d43 1d46 1d4b
1d4c 1d51 1d54 1d59 1d5a 1d5f 1d62 1d66
1d67 1d6c 1d6f 1d74 1d75 1d7a 1d7d 1d81
1d85 1d88 1d8b 1d8c 1d8e 1d8f 1d94 1d95
1d9a 1d9c 1da0 1da3 1da5 1da9 1dab 1db7
1db9 1dbb 1dbc 1dc1 1dc5 1dc7 1dd3 1dd5
1dd9 1ddd 1de1 1de2 1de4 1de8 1dea 1dee
1df1 1df3 1df7 1dfa 1dfe 1e02 1e05 1e06
1e08 1e0b 1e0e 1e0f 1e14 1e18 1e1c 1e1f
1e23 1e27 1e2a 1e2e 1e32 1e34 1e38 1e3c
1e3f 1e40 1e42 1e45 1e48 1e49 1e4e 1e52
1e56 1e59 1e5d 1e61 1e64 1e65 1e67 1e6b
1e6d 1e71 1e75 1e78 1e7c 1e80 1e83 1e87
1e8b 1e8e 1e91 1e96 1e99 1e9c 1e9f 1ea0
1ea5 1ea6 1 1ea8 1ead 1eb1 1eb5 1eb8
1ebd 1ec2 1ec5 1ec9 1eca 1ecf 1ed2 1ed7
1ed8 1edd 1ee0 1ee5 1ee6 1eeb 1eee 1ef2
1ef6 1efa 1efe 1f01 1f02 1f04 1f09 1f0a
1f0c 1f0d 1f12 1f15 1f1a 1f1b 1f20 1f23
1f28 1f29 1f2e 1f31 1f35 1f36 1f3b 1f3e
1f43 1f44 1f49 1f4a 1f4f 1f51 1f55 1f58
1f5c 1f60 1f63 1f67 1f6b 1f6e 1f72 1f76
1f77 1f7c 1f7e 1f82 1f86 1f89 1f8d 1f91
1f94 1f98 1f9c 1f9f 1fa2 1fa7 1faa 1fad
1fb0 1fb1 1fb6 1fb7 1 1fb9 1fbe 1fc2
1fc6 1fc9 1fce 1fd3 1fd6 1fda 1fdb 1fe0
1fe1 1fe6 1fea 1fee 1ff1 1ff6 1ffb 1ffe
2002 2006 2009 200a 200f 2012 2017 2018
201d 2020 2024 2028 202b 202c 2031 2032
2037 2039 203d 2040 2044 2048 204c 2050
2054 2058 205c 2060 2064 2070 2075 2079
207d 2081 2085 2087 2088 208d 1 2091
2095 2099 209c 20a0 20a4 20a8 20ac 20af
20b3 20b7 20ba 20bd 20c2 20c5 20c8 20cb
20cc 20d1 20d2 1 20d4 20d9 20dd 20e1
20e4 20e9 20ee 20f1 20f5 20f6 20fb 20fc
2101 2105 2109 210c 2111 2116 2119 211d
211e 2123 2126 212b 212c 2131 2134 2138
213c 213f 2142 2143 2145 2146 214b 214c
2151 2153 2157 215a 215e 2162 2166 216a
216e 2172 2176 2182 2187 218b 218f 2193
2197 2199 219d 219f 21ab 21ad 21af 21b0
21b5 21b9 21bb 21c7 21cb 21cd 21ce 21d7 
930
2
0 1 a 6 9 :3 6 11 :3 6
12 :2 6 1 3 a 13 a :2 1
4 18 21 20 :2 18 :2 4 11 1a
19 :2 11 :2 4 b 14 13 :2 b :2 4
f 18 17 :2 f :2 4 d 16 15
:2 d :2 4 f 18 17 :2 f :2 4 :3 12
:2 4 :3 13 :2 4 :3 11 :2 4 :3 12 :2 4 :3 f
:2 4 :3 10 :2 4 :3 f :2 4 :3 e :2 4 :3 15
:2 4 e 17 16 :2 e :2 4 18 21
20 :2 18 :2 4 f 18 17 :2 f :2 4
:2 e 1d e :2 4 :2 e 1d e :2 4
:2 d 1c d :2 4 :2 10 1f 10 :2 4
:2 c 1b c :2 4 :2 b 1a b :2 4
:2 d 1c d :2 4 :3 10 :2 4 :3 10 :2 4
12 1b 1a :2 12 :2 4 :2 d 1c d
:2 4 :2 12 21 12 :2 4 :2 a 19 a
:2 4 :2 f 1e f :2 4 :2 14 23 14
:2 4 :3 12 :2 4 :3 13 :2 4 :2 11 20 11
:2 4 :2 12 21 12 :2 4 :2 13 22 13
:2 4 :2 f 19 f :2 4 :2 f 19 f
:2 4 :3 9 :2 4 :3 9 :2 4 :2 f 1e f
:2 4 :2 16 25 16 :2 4 :2 11 20 11
:2 4 :2 14 23 14 :2 4 :2 e 1d e
:2 4 :3 10 :2 4 :2 16 2e 16 4 7
e 0 :2 7 11 1d 29 36 14
30 39 48 5a 14 31 3a 4a
5d f 1d 27 32 3e 47 4f
58 66 6f 7d 86 94 9d aa
b3 c4 cd e0 e9 fe 107 122
12b 143 14c 15d 166 178 181 199
1a2 1b5 1be 11 1e 38 45 54
64 b 29 50 59 68 a :6 7
e 0 :2 7 :3 a 15 :2 a 14 26
49 a 15 24 :3 a 14 34 4f
67 70 7f 90 99 a9 :b a f
10 1d e 18 11 1c 10 1d
2c 3c e 2c fffa :5 7 4 e
17 :3 e 4 7 :2 11 23 :2 2d 3a
:2 23 :3 7 :2 d 19 25 :2 2f 43 :2 25
57 5d 60 :2 57 6e 87 8 :2 6e
d 10 :2 6e 13 16 :2 6e b e
:2 6e 13 16 1f 27 :2 1f 34 :2 16
:2 6e b e :2 6e 13 16 :2 6e 1b
1e :2 6e 2a 2d :2 6e 32 35 :2 6e
:2 7 4d :2 4 7 :2 11 23 :2 2d 39
3c 4f 52 53 :2 4f :2 23 :3 7 :2 d
12 1e 61 64 :2 1e :2 7 56 :3 4
11 :2 4 12 :2 4 17 4 e 14
1b 28 30 c 1a 27 :2 7 :2 10
1c 7 4 c a :2 13 1f a
1a :2 7 :2 c a :2 13 1f a d
:2 17 29 :2 33 3f 43 56 59 5a
:2 56 :2 29 :3 d :2 13 18 24 67 6a
:2 24 77 7a :2 24 7e 81 :2 24 8d
90 :2 24 98 9b :2 24 a7 aa b1
ba bd :2 aa :2 24 :2 d 5d :2 a :2 7
:4 13 :2 7 4 :3 1 8 :2 11 1c 1a
:2 25 :2 1a 7 :4 3b 3a 39 :3 7 11
18 1f 22 :2 18 27 2a :2 11 :2 7
13 14 :2 13 :2 7 c :4 7 10 1c
2a 35 3f 4d :3 a d 14 :2 d
:2 16 22 d 1a d :2 16 22 d
:4 a e :2 17 23 20 :2 2c :2 20 :3 d
3c :2 a d 16 20 :2 d 23 25
:2 23 10 19 24 :2 10 27 29 :2 27
10 25 :2 2e :2 10 1f 30 3e 49
57 66 :2 1f :2 10 :2 19 25 :2 10 1c
25 2f :2 1c 32 34 :2 1c 10 2b
:2 d 27 d 1d :2 d 1e :2 d 12
:4 d 16 :19 10 13 1a :2 13 :2 1c 28
13 20 13 :2 1c 28 13 :4 10 14
:2 1d 29 26 :2 32 :2 26 :3 13 42 :3 10
23 32 40 4b 60 25 33 41
50 5a 25 2c 37 49 54 25
35 41 54 25 31 3d 4d 57
25 34 3f :2 23 10 13 1c 2d
:2 13 30 32 :2 30 13 1f 28 32
:2 1f 35 37 :2 1f 13 10 34 16
1f 30 :2 16 35 33 :2 3e :2 33 13
26 2f 40 :2 26 43 45 :2 26 13
5d 34 :2 10 d 11 a 10 16
:3 10 :2 19 25 10 d 15 13 :2 1c
28 13 23 :2 10 :2 15 13 :2 1c 28
13 16 :2 20 32 :2 3c 48 4c 5f
62 63 :2 5f :2 32 :3 16 :2 1c 21 2d
70 73 :2 2d 80 83 :2 2d 87 8a
:2 2d 96 99 :2 2d a1 a4 :2 2d b0
b3 ba c3 c6 :2 b3 :2 2d :2 16 66
:2 13 :2 10 :4 1c :2 10 d :3 a 11 1a
25 :2 11 28 2b :2 28 10 33 3c
46 :2 33 49 4b :2 49 32 :3 10 1e
:2 10 1f 10 1a 29 3a 4a 18
25 34 40 18 26 33 44 51
:2 13 :2 1c 28 13 10 18 16 :2 1f
2b 16 26 :2 13 :2 18 16 :2 1f 2b
16 19 :2 23 35 :2 3f 4b 4f 62
65 66 :2 62 :2 35 :3 19 :2 1f 24 30
73 76 :2 30 83 86 :2 30 8a 8d
:2 30 99 9c :2 30 a4 a7 :2 30 b3
b6 bd c6 c9 :2 b6 :2 30 :2 19 69
:2 16 :2 13 :4 1f :2 13 10 :3 4e :4 14 13
:4 17 16 20 19 1e 3d 45 4c
63 6b 72 8c 1e 3e 1e 19
1e 3d 45 4c 63 6b 72 :2 1e
1f 2c 3d 4a 19 20 1c 28
:3 26 1c :2 25 31 1c 2d 1c :2 25
31 1c :4 19 16 1e 1c :2 25 31
1c 2c :2 19 :2 1e 1c :2 25 31 1c
1f :2 29 3b :2 45 51 55 68 6b
6c :2 68 :2 3b :3 1f :2 25 2a 36 79
7c :2 36 89 8c :2 36 90 93 :2 36
9f a2 :2 36 aa ad :2 36 b9 bc
c3 cc cf :2 bc :2 36 :2 1f 6f :2 1c
:2 19 :4 25 :2 19 16 :4 30 20 2c 3c
49 1f 2c 3d 4a 19 20 1c
28 :3 26 1c :2 25 31 1c 2d 1c
:2 25 31 1c :4 19 16 1e 1c :2 25
31 1c 2c :2 19 :2 1e 1c :2 25 31
1c 1f :2 29 3b :2 45 51 55 68
6b 6c :2 68 :2 3b :3 1f :2 25 2a 36
79 7c :2 36 89 8c :2 36 90 93
:2 36 9f a2 :2 36 aa ad :2 36 b9
bc c3 cc cf :2 bc :2 36 :2 1f 6f
:2 1c :2 19 :4 25 :2 19 16 :7 13 10 2d
:4 18 17 16 :4 37 36 35 :2 16 :4 17
16 20 19 1e 3d 45 4c 63
6b 72 8b 1e 3e 1e 19 1e
3d 45 4c 63 6b 72 :2 1e 1f
2c 3d 4a 19 20 1c 28 :3 26
1c :2 25 31 1c 2d 1c :2 25 31
1c :4 19 16 1e 1c :2 25 31 1c
2c :2 19 :2 1e 1c :2 25 31 1c 1f
:2 29 3b :2 45 51 55 68 6b 6c
:2 68 :2 3b :3 1f :2 25 2a 36 79 7c
:2 36 89 8c :2 36 90 93 :2 36 9f
a2 :2 36 aa ad :2 36 b9 bc c3
cc cf :2 bc :2 36 :2 1f 6f :2 1c :2 19
:4 25 :2 19 16 :4 30 20 2c 3c 48
1f 2c 3d 4a 19 20 1c 28
:3 26 1c :2 25 31 1c 2d 1c :2 25
31 1c :4 19 16 1e 1c :2 25 31
1c 2c :2 19 :2 1e 1c :2 25 31 1c
1f :2 29 3b :2 45 51 55 68 6b
6c :2 68 :2 3b :3 1f :2 25 2a 36 79
7c :2 36 89 8c :2 36 90 93 :2 36
9f a2 :2 36 aa ad :2 36 b9 bc
c3 cc cf :2 bc :2 36 :2 1f 6f :2 1c
:2 19 :4 25 :2 19 16 :7 13 4d 2d :2 10
4e :2 d :4 a 7 b 50 a 10
:3 a :2 13 1f a 7 f d :2 16
22 d 1d :2 a :2 f d :2 16 22
d 10 :2 1a 2c :2 36 42 46 59
5c 5d :2 59 :2 2c :3 10 :2 16 1b 27
6a 6d :2 27 7a 7d :2 27 81 84
:2 27 90 93 :2 27 9b 9e :2 27 aa
ad b4 bd c0 :2 ad :2 27 :2 10 60
:2 d :2 a :4 16 :2 a 7 :3 50 b 14
1e :2 b 21 23 :2 21 a 2b 34
3e :2 2b 41 43 44 :2 43 :2 41 2a
:2 a b :2 10 1e 10 b :4 f e
c 14 1d 24 :2 1d 2d :2 14 c
f 11 14 :2 11 1a 1f 22 :2 1f
:2 f c 14 19 1b :2 14 c 24
:2 c 23 :2 b 12 17 1a :2 17 :3 e
20 :2 25 29 :2 20 1f 2c 2f :2 2c
:2 e c :2 17 2b :2 34 47 4f :2 c
31 :2 b :2 a :3 47 14 20 2e 37
3e 4a 53 13 20 31 3b d
14 10 1c :3 1a 10 :2 19 25 10
21 10 :2 19 25 10 :4 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 5c 5f 60 :2 5c :2 2f :3 13 :2 19
1e 2a 6d 70 :2 2a 7d 80 :2 2a
84 87 :2 2a 93 96 :2 2a 9e a1
:2 2a ad b0 b7 c0 c3 :2 b0 :2 2a
:2 13 63 :2 10 :2 d :4 19 :2 d a :3 47
a 1d 27 :2 1d a 47 :2 7 50
:2 4 7 10 21 :2 7 24 26 :2 24
7 :2 10 1c :2 25 7 4 28 a
13 24 :2 a 27 29 :2 27 7 :2 10
1c 25 36 :2 1c 7 2d 28 :2 4
7 :2 11 23 :2 2d 39 3c 4f 52
53 :2 4f :2 23 :3 7 :2 d 12 1e 61
64 :2 1e 71 74 :2 1e 78 7b :2 1e
85 88 91 99 :2 a2 :2 91 ad :2 88
:2 1e b5 b8 :2 1e c0 c3 :2 1e c8
cb :2 1e ce d1 :2 1e :2 7 56 :3 4
:2 f 21 :2 2a 34 38 :2 4 1 9
12 9 a :2 14 26 :2 30 3c 40
53 56 57 :2 53 :2 26 :3 a :2 10 15
21 64 67 :2 21 :3 a :2 10 15 39
5c 5f :2 68 :2 39 76 79 :2 39 81
84 :2 8d :2 39 :2 a 5a :3 7 c 15
1e 2c a 13 1e f e 7
e 7 e 7 29 :2 4 :2 9 7
:2 10 1c :2 7 :2 11 23 :2 2d 39 3d
50 53 54 :2 50 :2 23 :3 7 :2 d 12
1e 61 64 :2 1e :3 7 :2 d 12 36
42 45 :2 36 4d 50 :2 36 5c 5f
66 6f 72 :2 5f :2 36 :2 7 57 :2 4
7 c 1b a 13 1e f e
7 e 7 e 7 :2 4 :4 10 :2 4
:8 1 
930
4
0 :2 1 :4 3 :4 4
:4 5 2 :4 6 :2 1
:7 8 :7 9 :7 a :7 b
:7 c :7 d :5 e :5 f
:5 10 :5 11 :5 12 :5 13
:5 14 :5 15 :5 16 :7 17
:7 18 :7 19 :6 1a :6 1b
:6 1c :6 1d :6 1e :6 1f
:6 20 :5 21 :5 22 :7 23
:6 24 :6 25 :6 26 :6 27
:6 28 :5 29 :5 2a :6 2b
:6 2c :6 2d :6 2e :6 2f
:5 30 :5 31 :6 32 :6 33
:6 34 :6 35 :6 36 :5 37
:6 38 :2 3a 0 :2 3a
:4 3b :5 3c :5 3d :22 3e
:6 3f :5 40 3b :5 3a
:2 42 0 :2 42 44
45 :2 46 47 48
:3 49 :3 4a 4b 4c
4d :9 4e 4f 50
51 52 53 54
55 56 57 58
59 5a :2 5b :2 5c
:2 5d :4 5e :2 5f 44
:5 42 :7 61 :b 63 :11 64
65 :2 64 :2 65 :2 64
:2 65 :2 64 :2 66 :2 64
:9 66 :2 64 :2 67 :2 64
:2 67 :2 64 :2 67 :2 64
:2 67 :2 64 :2 67 :4 64
:3 63 :11 6a :b 6b :3 6a
:3 6f :3 70 :3 71 :5 74
:3 75 74 :5 76 73
78 :5 79 :3 78 :2 7a
:5 7c :11 7d :24 7e :3 7d
:2 7b :6 7a 77 :3 62
:12 83 :c 84 :6 85 :5 89
8b :9 8c :3 8d :5 8e
8d :5 90 8f :3 8d
:a 93 :2 94 :3 93 :9 99
:9 9b :5 9d :a 9e :5 9f
:b a1 :3 9b 99 :3 a5
:3 a7 :5 aa ac ad
ae af b0 b1
b2 b3 b4 b5
b6 b7 b8 b9
ba bb bc bd
be bf c0 c1
c2 c3 :3 ad :3 c4
:5 c5 c4 :5 c7 c6
:3 c4 :a ca :2 cb :3 ca
:6 d0 :5 d1 :5 d2 :4 d3
:5 d4 :3 d5 :3 d0 :9 d7
:b d8 d9 d7 :b d9
:b da d9 :3 d7 ac
dc a3 :4 e0 :5 e1
df e3 :5 e4 :3 e3
:2 e5 :5 e7 :11 e8 :24 e9
:3 e8 :2 e6 :6 e5 e2
:3 a3 :16 f1 :3 f2 :3 f3
:4 f6 :4 f7 :5 f8 f6
:5 f9 f5 fb :5 fc
:3 fb :2 fd :5 ff :11 100
:24 101 :3 100 :2 fe :6 fd
fa :3 f1 :5 107 :5 109
10b 10c :8 10e :2 10f
110 112 :7 114 115
116 :4 118 10b :6 119
:5 11a 119 :5 11c 11b
:3 119 10a 11f :5 120
:3 11f :2 121 :5 123 :11 124
:24 125 :3 124 :2 122 :6 121
11e :4 109 :4 12b :4 12c
12b :6 12d :5 12e 12d
:5 130 12f :3 12d 12a
133 :5 134 :3 133 :2 135
:5 137 :11 138 :24 139 :3 138
:2 136 :6 135 132 :4 129
:3 109 13e 107 :e 13e
:5 140 142 143 :8 145
:2 146 147 149 :7 14b
14c 14d :4 14f 142
:6 150 :5 151 150 :5 153
152 :3 150 141 156
:5 157 :3 156 :2 158 :5 15a
:11 15b :24 15c :3 15b :2 159
:6 158 155 :4 140 :4 162
:4 163 162 :6 164 :5 165
164 :5 167 166 :3 164
161 16a :5 16b :3 16a
:2 16c :5 16e :11 16f :24 170
:3 16f :2 16d :6 16c 169
:4 160 :3 140 13e :3 107
:3 f1 a3 :3 99 8b
178 83 :4 17c :5 17d
17b 17f :5 180 :3 17f
:2 181 :5 183 :11 184 :24 185
:3 184 :2 182 :6 181 17e
:3 83 :19 18c :6 18e :5 190
:a 191 :c 192 :7 193 :3 192
:3 190 :15 196 :a 197 :3 196
:2 18f :3 18c :7 19c :4 19d
19c :6 19e :5 19f 19e
:5 1a1 1a0 :3 19e 19b
1a4 :5 1a5 :3 1a4 :2 1a6
:5 1a8 :11 1a9 :24 1aa :3 1a9
:2 1a7 :6 1a6 1a3 :3 18c
:6 1af :3 18c :3 83 :9 1b4
:7 1b5 1b6 1b4 :9 1b6
:9 1b7 1b6 :3 1b4 :11 1ba
:30 1bb :3 1ba :a 1bd 62
:3 1c0 :11 1c1 :b 1c2 :17 1c3
:3 1c1 :2 1c5 :3 1c6 :3 1c7
1c8 1c6 1c5 1c6
:3 1c9 :3 1c0 :2 1ca :5 1cc
:11 1cd :b 1ce :18 1cf :3 1cd
:2 1d1 1d2 :3 1d3 1d4
1d2 1d1 1d2 :3 1d5
:2 1cb :6 1ca 1bf :7 1

21d9
4
:3 0 1 :3 0 2
:a 0 92b 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
:2 0 7 6 :3 0
5 :7 0 9 8
:3 0 4 :3 0 7
:7 0 d c :3 0
8 :3 0 9 :3 0
a :2 0 4 11
12 0 f 13
0 92b 2 14
:2 0 c :2 0 f
4 :3 0 c :2 0
d 17 19 :6 0
1c 1a 0 929
0 b :6 0 f
:2 0 13 4 :3 0
11 1e 20 :6 0
23 21 0 929
0 d :6 0 c
:2 0 17 4 :3 0
15 25 27 :6 0
2a 28 0 929
0 e :6 0 c
:2 0 1b 4 :3 0
19 2c 2e :6 0
31 2f 0 929
0 10 :6 0 13
:2 0 1f 4 :3 0
1d 33 35 :6 0
38 36 0 929
0 11 :6 0 25
11e 0 23 4
:3 0 21 3a 3c
:6 0 3f 3d 0
929 0 12 :6 0
29 152 0 27
15 :3 0 41 :7 0
44 42 0 929
0 14 :6 0 15
:3 0 46 :7 0 49
47 0 929 0
16 :6 0 2d 186
0 2b 15 :3 0
4b :7 0 4e 4c
0 929 0 17
:6 0 15 :3 0 50
:7 0 53 51 0
929 0 18 :6 0
31 1ba 0 2f
15 :3 0 55 :7 0
58 56 0 929
0 19 :6 0 15
:3 0 5a :7 0 5d
5b 0 929 0
1a :6 0 35 1ee
0 33 1c :3 0
5f :7 0 62 60
0 929 0 1b
:6 0 1c :3 0 64
:7 0 67 65 0
929 0 1d :6 0
c :2 0 39 1c
:3 0 69 :7 0 6c
6a 0 929 0
1e :6 0 4 :3 0
c :2 0 37 6e
70 :6 0 73 71
0 929 0 1f
:6 0 13 :2 0 3d
4 :3 0 3b 75
77 :6 0 7a 78
0 929 0 20
:6 0 23 :2 0 41
4 :3 0 3f 7c
7e :6 0 81 7f
0 929 0 21
:6 0 23 :2 0 43
6 :3 0 83 :7 0
87 84 85 929
0 22 :6 0 23
:2 0 45 6 :3 0
89 :7 0 8d 8a
8b 929 0 24
:6 0 23 :2 0 47
6 :3 0 8f :7 0
93 90 91 929
0 25 :6 0 23
:2 0 49 6 :3 0
95 :7 0 99 96
97 929 0 26
:6 0 23 :2 0 4b
6 :3 0 9b :7 0
9f 9c 9d 929
0 27 :6 0 23
:2 0 4d 6 :3 0
a1 :7 0 a5 a2
a3 929 0 28
:6 0 51 324 0
4f 6 :3 0 a7
:7 0 ab a8 a9
929 0 29 :6 0
2d :2 0 53 15
:3 0 ad :7 0 b0
ae 0 929 0
2a :6 0 1c :3 0
b2 :7 0 b5 b3
0 929 0 2b
:6 0 23 :2 0 57
4 :3 0 55 b7
b9 :6 0 bc ba
0 929 0 2c
:6 0 23 :2 0 59
6 :3 0 be :7 0
c2 bf c0 929
0 2e :6 0 23
:2 0 5b 6 :3 0
c4 :7 0 c8 c5
c6 929 0 2f
:6 0 23 :2 0 5d
6 :3 0 ca :7 0
ce cb cc 929
0 30 :6 0 23
:2 0 5f 6 :3 0
d0 :7 0 d4 d1
d2 929 0 31
:6 0 63 401 0
61 6 :3 0 d6
:7 0 da d7 d8
929 0 32 :6 0
23 :2 0 65 15
:3 0 dc :7 0 df
dd 0 929 0
33 :6 0 15 :3 0
e1 :7 0 e4 e2
0 929 0 34
:6 0 23 :2 0 67
6 :3 0 e6 :7 0
ea e7 e8 929
0 35 :6 0 23
:2 0 69 6 :3 0
ec :7 0 f0 ed
ee 929 0 36
:6 0 3a :2 0 6b
6 :3 0 f2 :7 0
f6 f3 f4 929
0 37 :6 0 3a
:2 0 6d 39 :3 0
f8 :7 0 fc f9
fa 929 0 38
:6 0 71 4c1 0
6f 39 :3 0 fe
:7 0 102 ff 100
929 0 3b :6 0
23 :2 0 73 1c
:3 0 104 :7 0 107
105 0 929 0
3c :6 0 1c :3 0
109 :7 0 10c 10a
0 929 0 3d
:6 0 23 :2 0 75
6 :3 0 10e :7 0
112 10f 110 929
0 3e :6 0 23
:2 0 77 6 :3 0
114 :7 0 118 115
116 929 0 3f
:6 0 23 :2 0 79
6 :3 0 11a :7 0
11e 11b 11c 929
0 40 :6 0 23
:2 0 7b 6 :3 0
120 :7 0 124 121
122 929 0 41
:6 0 7f 581 0
7d 6 :3 0 126
:7 0 12a 127 128
929 0 42 :9 0
81 44 :3 0 12c
:7 0 12f 12d 0
929 0 43 :6 0
6 :3 0 131 0
2 :5 0 23 :2 0
135 132 133 929
0 45 :6 0 46
:3 0 47 :a 0 2
177 :4 0 137 13a
0 138 :3 0 48
:3 0 49 :3 0 4a
:3 0 4b :3 0 4c
:3 0 4d :3 0 4e
:3 0 4f :3 0 4c
:3 0 50 :3 0 4d
:3 0 51 :3 0 52
:3 0 50 :3 0 53
:3 0 54 :3 0 e
:3 0 55 :3 0 56
:3 0 57 :3 0 56
:3 0 58 :3 0 56
:3 0 59 :3 0 56
:3 0 5a :3 0 56
:3 0 5b :3 0 56
:3 0 5c :3 0 56
:3 0 5d :3 0 56
:3 0 5e :3 0 56
:3 0 5f :3 0 56
:3 0 60 :3 0 56
:3 0 61 :3 0 56
:3 0 62 :3 0 56
:3 0 63 :3 0 56
:3 0 64 :3 0 56
:3 0 65 :3 0 4a
:3 0 4b :3 0 4a
:3 0 4b :3 0 66
:3 0 67 :3 0 4c
:3 0 50 :3 0 4d
:3 0 4e :3 0 4f
:4 0 68 1 :8 0
178 137 13a 179
0 929 83 179
17b 178 17a :6 0
177 :7 0 179 46
:3 0 69 :a 0 3
1b2 :5 0 17d 180
0 17e :3 0 6a
:3 0 6b :3 0 6c
:3 0 6d :3 0 4c
:3 0 50 :3 0 22
:3 0 4c :3 0 67
:3 0 6c :3 0 6d
:3 0 66 :3 0 67
:3 0 6e :3 0 6f
:3 0 50 :3 0 4c
:3 0 50 :3 0 4d
:3 0 4e :3 0 4f
:3 0 4d :3 0 51
:3 0 52 :3 0 70
:3 0 71 :3 0 72
:3 0 73 :3 0 74
:3 0 75 :3 0 76
:3 0 77 :3 0 78
:3 0 79 :3 0 66
:3 0 7a :3 0 48
:3 0 d :3 0 7b
:3 0 1f :3 0 6c
:3 0 6d :3 0 6c
:3 0 6d :3 0 66
:3 0 67 :3 0 4c
:3 0 50 :4 0 7c
1 :8 0 1b3 17d
180 1b4 0 929
85 1b4 1b6 1b3
1b5 :6 0 1b2 :7 0
1b4 1be 1bf 0
87 9 :3 0 a
:2 0 4 1b8 1b9
0 1ba :7 0 1bd
1bb 0 929 0
7d :6 0 7e :3 0
7f :3 0 7e :3 0
80 :3 0 1c1 1c2
0 2 :4 0 89
1c3 1c5 1c0 1c7
1c6 :2 0 81 :3 0
82 :3 0 1c9 1ca
0 83 :4 0 7e
:3 0 84 :3 0 1cd
1ce 0 2 :4 0
8b 1cf 1d1 85
:4 0 86 :2 0 87
:3 0 8d 1d4 1d6
:3 0 88 :4 0 86
:2 0 89 :4 0 90
1d9 1db :3 0 86
:2 0 3 :3 0 93
1dd 1df :3 0 86
:2 0 89 :4 0 96
1e1 1e3 :3 0 86
:2 0 8a :4 0 99
1e5 1e7 :3 0 86
:2 0 4d :3 0 8b
:3 0 5 :3 0 9c
1eb 1ed 8c :4 0
9e 1ea 1f0 a1
1e9 1f2 :3 0 86
:2 0 8a :4 0 a4
1f4 1f6 :3 0 86
:2 0 89 :4 0 a7
1f8 1fa :3 0 86
:2 0 7 :3 0 aa
1fc 1fe :3 0 86
:2 0 89 :4 0 ad
200 202 :3 0 86
:2 0 8d :4 0 b0
204 206 :3 0 b3
1cb 208 :2 0 20a
b8 20b 1c8 20a
0 20c ba 0
889 7e :3 0 7f
:3 0 20d 20e 0
7e :3 0 8e :3 0
210 211 0 13
:2 0 2 :4 0 8f
:2 0 90 :2 0 13
:2 0 bc 216 218
:3 0 bf 212 21a
20f 21c 21b :2 0
81 :3 0 91 :3 0
21e 21f 0 83
:4 0 92 :4 0 86
:2 0 87 :3 0 c3
223 225 :3 0 c6
220 227 :2 0 229
c9 22a 21d 229
0 22b cb 0
889 22 :3 0 5
:3 0 22c 22d 0
889 10 :3 0 7
:3 0 22f 230 0
889 41 :3 0 23
:2 0 232 233 0
889 93 :3 0 54
:3 0 55 :3 0 e
:3 0 28 :3 0 53
:3 0 49 :3 0 3
:4 0 94 1 :8 0
243 9 :3 0 95
:3 0 23e 23f 0
96 :3 0 240 241
0 243 cd 296
97 :3 0 9 :3 0
95 :3 0 245 246
0 96 :3 0 247
248 0 24a d0
24c d2 24b 24a
:2 0 294 98 :3 0
9 :3 0 95 :3 0
24f 250 0 96
:3 0 251 252 0
28c 7e :3 0 7f
:3 0 254 255 0
7e :3 0 8e :3 0
257 258 0 99
:2 0 2 :4 0 9a
:2 0 90 :2 0 13
:2 0 d4 25d 25f
:3 0 d7 259 261
256 263 262 :2 0
81 :3 0 91 :3 0
265 266 0 83
:4 0 9b :4 0 86
:2 0 87 :3 0 db
26a 26c :3 0 86
:2 0 9c :4 0 de
26e 270 :3 0 86
:2 0 9d :4 0 e1
272 274 :3 0 86
:2 0 96 :3 0 e4
276 278 :3 0 86
:2 0 9e :4 0 e7
27a 27c :3 0 86
:2 0 9f :3 0 a0
:3 0 13 :2 0 a1
:2 0 ea 27f 283
ee 27e 285 :3 0
f1 267 287 :2 0
289 f4 28a 264
289 0 28b f6
0 28c f8 28f
:3 0 28f 0 28f
28e 28c 28d :6 0
291 4 :3 0 fb
293 fd 292 291
:2 0 294 ff :2 0
296 0 296 295
243 294 :6 0 889
1 :3 0 9 :3 0
95 :3 0 298 299
0 9 :3 0 a2
:2 0 a3 :3 0 29b
29d 0 104 29c
29f :3 0 2a0 :2 0
28 :3 0 a4 :2 0
107 2a3 2a4 :3 0
2a5 :2 0 2a6 :2 0
2a1 2a8 2a7 :2 0
e :3 0 9f :3 0
e :3 0 86 :2 0
a5 :4 0 109 2ad
2af :3 0 13 :2 0
f :2 0 10c 2ab
2b3 2aa 2b4 0
810 25 :3 0 a6
:2 0 13 :2 0 110
2b7 2b9 :3 0 2b6
2ba 0 810 a7
:3 0 47 :4 0 2bf
:2 0 810 2bd 2c0
:3 0 a8 :3 0 47
:3 0 d :3 0 1f
:3 0 25 :3 0 17
:3 0 18 :4 0 2c9
:2 0 6c2 2c2 2ca
:3 0 112 :3 0 47
:3 0 a9 :3 0 2cb
2cc :3 0 9 :3 0
95 :3 0 2ce 2cf
0 23 :2 0 2d0
2d1 0 2d3 118
2da 9 :3 0 95
:3 0 2d4 2d5 0
aa :2 0 2d6 2d7
0 2d9 11a 2db
2cd 2d3 0 2dc
0 2d9 0 2dc
11c 0 6c2 9
:3 0 95 :3 0 2dd
2de 0 9 :3 0
ab :2 0 a3 :3 0
2e0 2e2 0 121
2e1 2e4 :3 0 2e5
:2 0 ac :8 0 2e9
124 2ea 2e6 2e9
0 2eb 126 0
6c2 4d :3 0 25
:3 0 23 :2 0 128
2ec 2ef a2 :2 0
23 :2 0 12d 2f1
2f3 :3 0 4d :3 0
22 :3 0 23 :2 0
130 2f5 2f8 a2
:2 0 ad :2 0 135
2fa 2fc :3 0 45
:3 0 9 :3 0 95
:3 0 2ff 300 0
2fe 301 0 31d
43 :3 0 ae :3 0
d :3 0 1f :3 0
17 :3 0 18 :3 0
10 :3 0 138 304
30a 303 30b 0
31d 9 :3 0 95
:3 0 30d 30e 0
45 :3 0 30f 310
0 31d 29 :3 0
4d :3 0 29 :3 0
23 :2 0 13e 313
316 90 :2 0 13
:2 0 141 318 31a
:3 0 312 31b 0
31d 144 31e 2fd
31d 0 31f 149
0 320 14b 6bf
17 :4 0 321 322
0 6be 18 :4 0
324 325 0 6be
a7 :3 0 69 :4 0
32a :2 0 6be 328
32b :3 0 a8 :3 0
69 :3 0 20 :3 0
b :3 0 35 :3 0
17 :3 0 18 :3 0
25 :3 0 26 :3 0
30 :3 0 24 :3 0
1e :3 0 42 :3 0
14 :3 0 16 :3 0
3e :3 0 3f :3 0
38 :3 0 3b :3 0
21 :3 0 37 :3 0
11 :3 0 12 :3 0
36 :4 0 345 :2 0
3b7 32d 346 :3 0
14d :3 0 69 :3 0
a9 :3 0 347 348
:3 0 9 :3 0 95
:3 0 34a 34b 0
23 :2 0 34c 34d
0 34f 164 356
9 :3 0 95 :3 0
350 351 0 aa
:2 0 352 353 0
355 166 357 349
34f 0 358 0
355 0 358 168
0 3b7 9 :3 0
95 :3 0 359 35a
0 9 :3 0 ab
:2 0 a3 :3 0 35c
35e 0 16d 35d
360 :3 0 361 :2 0
ac :8 0 365 170
366 362 365 0
367 172 0 3b7
32 :3 0 af :3 0
d :3 0 1f :3 0
20 :3 0 b :3 0
35 :3 0 17 :3 0
18 :3 0 25 :3 0
26 :3 0 30 :3 0
24 :3 0 1e :3 0
42 :3 0 14 :3 0
16 :3 0 3e :3 0
3f :3 0 38 :3 0
3b :3 0 21 :3 0
37 :3 0 11 :3 0
12 :3 0 36 :3 0
22 :3 0 10 :3 0
174 369 384 368
385 0 3b7 4d
:3 0 32 :3 0 23
:2 0 18f 387 38a
a2 :2 0 23 :2 0
194 38c 38e :3 0
29 :3 0 4d :3 0
29 :3 0 23 :2 0
197 391 394 90
:2 0 13 :2 0 19a
396 398 :3 0 390
399 0 39c b0
:3 0 19d 3b5 4d
:3 0 32 :3 0 23
:2 0 19f 39d 3a0
56 :3 0 a2 :2 0
b1 :3 0 3a2 3a4
0 1a4 3a3 3a6
:3 0 41 :3 0 4d
:3 0 41 :3 0 23
:2 0 1a7 3a9 3ac
90 :2 0 13 :2 0
1aa 3ae 3b0 :3 0
3a8 3b1 0 3b3
1ad 3b4 3a7 3b3
0 3b6 38f 39c
0 3b6 1af 0
3b7 1b2 3b9 a8
:4 0 3b7 :4 0 6be
b2 :3 0 69 :4 0
3bd :2 0 3c3 3bb
0 9 :3 0 95
:3 0 3be 3bf 0
96 :3 0 3c0 3c1
0 3c3 1b8 416
97 :3 0 9 :3 0
95 :3 0 3c5 3c6
0 96 :3 0 3c7
3c8 0 3ca 1bb
3cc 1bd 3cb 3ca
:2 0 414 98 :3 0
9 :3 0 95 :3 0
3cf 3d0 0 96
:3 0 3d1 3d2 0
40c 7e :3 0 7f
:3 0 3d4 3d5 0
7e :3 0 8e :3 0
3d7 3d8 0 99
:2 0 2 :4 0 b3
:2 0 90 :2 0 13
:2 0 1bf 3dd 3df
:3 0 1c2 3d9 3e1
3d6 3e3 3e2 :2 0
81 :3 0 91 :3 0
3e5 3e6 0 83
:4 0 b4 :4 0 86
:2 0 87 :3 0 1c6
3ea 3ec :3 0 86
:2 0 9c :4 0 1c9
3ee 3f0 :3 0 86
:2 0 9d :4 0 1cc
3f2 3f4 :3 0 86
:2 0 96 :3 0 1cf
3f6 3f8 :3 0 86
:2 0 9e :4 0 1d2
3fa 3fc :3 0 86
:2 0 9f :3 0 a0
:3 0 13 :2 0 a1
:2 0 1d5 3ff 403
1d9 3fe 405 :3 0
1dc 3e7 407 :2 0
409 1df 40a 3e4
409 0 40b 1e1
0 40c 1e3 40f
:3 0 40f 0 40f
40e 40c 40d :6 0
411 8 :3 0 1e6
413 1e8 412 411
:2 0 414 1ea :2 0
416 0 416 415
3c3 414 :6 0 6be
6 :3 0 4d :3 0
22 :3 0 23 :2 0
1ed 418 41b ab
:2 0 13 :2 0 1f2
41d 41f :3 0 420
:2 0 4d :3 0 29
:3 0 23 :2 0 1f5
422 425 b5 :2 0
23 :2 0 1fa 427
429 :3 0 42a :2 0
421 42c 42b :2 0
19 :4 0 42e 42f
0 6bb 1a :4 0
431 432 0 6bb
b6 :3 0 b7 :3 0
4c :3 0 50 :3 0
2a :3 0 2f :3 0
19 :3 0 1a :3 0
53 :3 0 48 :3 0
d :3 0 49 :3 0
1f :4 0 b8 1
:8 0 447 9 :3 0
95 :3 0 442 443
0 96 :3 0 444
445 0 447 1fd
49a 97 :3 0 9
:3 0 95 :3 0 449
44a 0 96 :3 0
44b 44c 0 44e
200 450 202 44f
44e :2 0 498 98
:3 0 9 :3 0 95
:3 0 453 454 0
96 :3 0 455 456
0 490 7e :3 0
7f :3 0 458 459
0 7e :3 0 8e
:3 0 45b 45c 0
99 :2 0 2 :4 0
b9 :2 0 90 :2 0
13 :2 0 204 461
463 :3 0 207 45d
465 45a 467 466
:2 0 81 :3 0 91
:3 0 469 46a 0
83 :4 0 ba :4 0
86 :2 0 87 :3 0
20b 46e 470 :3 0
86 :2 0 9c :4 0
20e 472 474 :3 0
86 :2 0 9d :4 0
211 476 478 :3 0
86 :2 0 96 :3 0
214 47a 47c :3 0
86 :2 0 9e :4 0
217 47e 480 :3 0
86 :2 0 9f :3 0
a0 :3 0 13 :2 0
a1 :2 0 21a 483
487 21e 482 489
:3 0 221 46b 48b
:2 0 48d 224 48e
468 48d 0 48f
226 0 490 228
493 :3 0 493 0
493 492 490 491
:6 0 495 a :3 0
22b 497 22d 496
495 :2 0 498 22f
:2 0 49a 0 49a
499 447 498 :6 0
6bb 6 :3 0 1a
:3 0 a4 :2 0 232
49d 49e :3 0 49f
:2 0 2a :3 0 a4
:2 0 234 4a2 4a3
:3 0 4a4 :2 0 53
:3 0 b6 :3 0 2f
:3 0 bb :3 0 9f
:3 0 2a :3 0 bb
:3 0 9f :3 0 1a
:3 0 1a :3 0 2f
:3 0 1a :3 0 2a
:3 0 b7 :3 0 2f
:3 0 bb :3 0 9f
:3 0 2a :3 0 bb
:3 0 9f :3 0 1a
:3 0 2f :3 0 2f
:3 0 48 :3 0 d
:3 0 49 :3 0 1f
:4 0 bc 1 :8 0
4d7 a9 :4 0 4c2
:3 0 bd :3 0 a2
:2 0 238 4c5 4c6
:3 0 9 :3 0 95
:3 0 4c8 4c9 0
23 :2 0 4ca 4cb
0 4cd 23b 4d4
9 :3 0 95 :3 0
4ce 4cf 0 aa
:2 0 4d0 4d1 0
4d3 23d 4d5 4c7
4cd 0 4d6 0
4d3 0 4d6 23f
0 4d7 242 52a
97 :3 0 9 :3 0
95 :3 0 4d9 4da
0 96 :3 0 4db
4dc 0 4de 245
4e0 247 4df 4de
:2 0 528 98 :3 0
9 :3 0 95 :3 0
4e3 4e4 0 96
:3 0 4e5 4e6 0
520 7e :3 0 7f
:3 0 4e8 4e9 0
7e :3 0 8e :3 0
4eb 4ec 0 99
:2 0 2 :4 0 be
:2 0 90 :2 0 13
:2 0 249 4f1 4f3
:3 0 24c 4ed 4f5
4ea 4f7 4f6 :2 0
81 :3 0 91 :3 0
4f9 4fa 0 83
:4 0 bf :4 0 86
:2 0 87 :3 0 250
4fe 500 :3 0 86
:2 0 9c :4 0 253
502 504 :3 0 86
:2 0 9d :4 0 256
506 508 :3 0 86
:2 0 96 :3 0 259
50a 50c :3 0 86
:2 0 9e :4 0 25c
50e 510 :3 0 86
:2 0 9f :3 0 a0
:3 0 13 :2 0 a1
:2 0 25f 513 517
263 512 519 :3 0
266 4fb 51b :2 0
51d 269 51e 4f8
51d 0 51f 26b
0 520 26d 523
:3 0 523 0 523
522 520 521 :6 0
525 c :3 0 270
527 272 526 525
:2 0 528 274 :2 0
52a 0 52a 529
4d7 528 :6 0 52c
6 :3 0 277 5a1
53 :3 0 b6 :3 0
1a :3 0 b7 :3 0
48 :3 0 d :3 0
49 :3 0 1f :4 0
c0 1 :8 0 54b
a9 :4 0 536 :3 0
bd :3 0 a2 :2 0
27b 539 53a :3 0
9 :3 0 95 :3 0
53c 53d 0 23
:2 0 53e 53f 0
541 27e 548 9
:3 0 95 :3 0 542
543 0 aa :2 0
544 545 0 547
280 549 53b 541
0 54a 0 547
0 54a 282 0
54b 285 59e 97
:3 0 9 :3 0 95
:3 0 54d 54e 0
96 :3 0 54f 550
0 552 288 554
28a 553 552 :2 0
59c 98 :3 0 9
:3 0 95 :3 0 557
558 0 96 :3 0
559 55a 0 594
7e :3 0 7f :3 0
55c 55d 0 7e
:3 0 8e :3 0 55f
560 0 99 :2 0
2 :4 0 c1 :2 0
90 :2 0 13 :2 0
28c 565 567 :3 0
28f 561 569 55e
56b 56a :2 0 81
:3 0 91 :3 0 56d
56e 0 83 :4 0
c2 :4 0 86 :2 0
87 :3 0 293 572
574 :3 0 86 :2 0
9c :4 0 296 576
578 :3 0 86 :2 0
9d :4 0 299 57a
57c :3 0 86 :2 0
96 :3 0 29c 57e
580 :3 0 86 :2 0
9e :4 0 29f 582
584 :3 0 86 :2 0
9f :3 0 a0 :3 0
13 :2 0 a1 :2 0
2a2 587 58b 2a6
586 58d :3 0 2a9
56f 58f :2 0 591
2ac 592 56c 591
0 593 2ae 0
594 2b0 597 :3 0
597 0 597 596
594 595 :6 0 599
e :3 0 2b3 59b
2b5 59a 599 :2 0
59c 2b7 :2 0 59e
0 59e 59d 54b
59c :6 0 5a0 6
:3 0 2ba 5a2 4a5
52c 0 5a3 0
5a0 0 5a3 2bc
0 5a5 b0 :3 0
2bf 6b9 19 :3 0
a4 :2 0 2c1 5a7
5a8 :3 0 5a9 :2 0
5aa :2 0 1a :3 0
c3 :2 0 2c3 5ad
5ae :3 0 5af :2 0
5b0 :2 0 5ab 5b2
5b1 :2 0 2a :3 0
a4 :2 0 2c5 5b5
5b6 :3 0 5b7 :2 0
53 :3 0 b6 :3 0
2f :3 0 bb :3 0
9f :3 0 2a :3 0
bb :3 0 9f :3 0
19 :3 0 19 :3 0
2f :3 0 19 :3 0
2a :3 0 b7 :3 0
2f :3 0 bb :3 0
9f :3 0 2a :3 0
bb :3 0 9f :3 0
19 :3 0 2f :3 0
2f :3 0 48 :3 0
d :3 0 49 :3 0
1f :4 0 c4 1
:8 0 5ea a9 :4 0
5d5 :3 0 bd :3 0
a2 :2 0 2c9 5d8
5d9 :3 0 9 :3 0
95 :3 0 5db 5dc
0 23 :2 0 5dd
5de 0 5e0 2cc
5e7 9 :3 0 95
:3 0 5e1 5e2 0
aa :2 0 5e3 5e4
0 5e6 2ce 5e8
5da 5e0 0 5e9
0 5e6 0 5e9
2d0 0 5ea 2d3
63d 97 :3 0 9
:3 0 95 :3 0 5ec
5ed 0 96 :3 0
5ee 5ef 0 5f1
2d6 5f3 2d8 5f2
5f1 :2 0 63b 98
:3 0 9 :3 0 95
:3 0 5f6 5f7 0
96 :3 0 5f8 5f9
0 633 7e :3 0
7f :3 0 5fb 5fc
0 7e :3 0 8e
:3 0 5fe 5ff 0
99 :2 0 2 :4 0
c5 :2 0 90 :2 0
13 :2 0 2da 604
606 :3 0 2dd 600
608 5fd 60a 609
:2 0 81 :3 0 91
:3 0 60c 60d 0
83 :4 0 c6 :4 0
86 :2 0 87 :3 0
2e1 611 613 :3 0
86 :2 0 9c :4 0
2e4 615 617 :3 0
86 :2 0 9d :4 0
2e7 619 61b :3 0
86 :2 0 96 :3 0
2ea 61d 61f :3 0
86 :2 0 9e :4 0
2ed 621 623 :3 0
86 :2 0 9f :3 0
a0 :3 0 13 :2 0
a1 :2 0 2f0 626
62a 2f4 625 62c
:3 0 2f7 60e 62e
:2 0 630 2fa 631
60b 630 0 632
2fc 0 633 2fe
636 :3 0 636 0
636 635 633 634
:6 0 638 10 :3 0
301 63a 303 639
638 :2 0 63b 305
:2 0 63d 0 63d
63c 5ea 63b :6 0
63f 6 :3 0 308
6b4 53 :3 0 b6
:3 0 19 :3 0 b7
:3 0 48 :3 0 d
:3 0 49 :3 0 1f
:4 0 c7 1 :8 0
65e a9 :4 0 649
:3 0 bd :3 0 a2
:2 0 30c 64c 64d
:3 0 9 :3 0 95
:3 0 64f 650 0
23 :2 0 651 652
0 654 30f 65b
9 :3 0 95 :3 0
655 656 0 aa
:2 0 657 658 0
65a 311 65c 64e
654 0 65d 0
65a 0 65d 313
0 65e 316 6b1
97 :3 0 9 :3 0
95 :3 0 660 661
0 96 :3 0 662
663 0 665 319
667 31b 666 665
:2 0 6af 98 :3 0
9 :3 0 95 :3 0
66a 66b 0 96
:3 0 66c 66d 0
6a7 7e :3 0 7f
:3 0 66f 670 0
7e :3 0 8e :3 0
672 673 0 99
:2 0 2 :4 0 c8
:2 0 90 :2 0 13
:2 0 31d 678 67a
:3 0 320 674 67c
671 67e 67d :2 0
81 :3 0 91 :3 0
680 681 0 83
:4 0 c9 :4 0 86
:2 0 87 :3 0 324
685 687 :3 0 86
:2 0 9c :4 0 327
689 68b :3 0 86
:2 0 9d :4 0 32a
68d 68f :3 0 86
:2 0 96 :3 0 32d
691 693 :3 0 86
:2 0 9e :4 0 330
695 697 :3 0 86
:2 0 9f :3 0 a0
:3 0 13 :2 0 a1
:2 0 333 69a 69e
337 699 6a0 :3 0
33a 682 6a2 :2 0
6a4 33d 6a5 67f
6a4 0 6a6 33f
0 6a7 341 6aa
:3 0 6aa 0 6aa
6a9 6a7 6a8 :6 0
6ac 12 :3 0 344
6ae 346 6ad 6ac
:2 0 6af 348 :2 0
6b1 0 6b1 6b0
65e 6af :6 0 6b3
6 :3 0 34b 6b5
5b8 63f 0 6b6
0 6b3 0 6b6
34d 0 6b7 350
6b8 5b3 6b7 0
6ba 4a0 5a5 0
6ba 352 0 6bb
355 6bc 42d 6bb
0 6bd 35a 0
6be 35c 6c0 2f4
320 0 6c1 0
6be 0 6c1 363
0 6c2 366 6c4
a8 :4 0 6c2 :4 0
810 b2 :3 0 47
:4 0 6c8 :2 0 6ce
6c6 0 9 :3 0
95 :3 0 6c9 6ca
0 96 :3 0 6cb
6cc 0 6ce 36b
721 97 :3 0 9
:3 0 95 :3 0 6d0
6d1 0 96 :3 0
6d2 6d3 0 6d5
36e 6d7 370 6d6
6d5 :2 0 71f 98
:3 0 9 :3 0 95
:3 0 6da 6db 0
96 :3 0 6dc 6dd
0 717 7e :3 0
7f :3 0 6df 6e0
0 7e :3 0 8e
:3 0 6e2 6e3 0
99 :2 0 2 :4 0
ca :2 0 90 :2 0
13 :2 0 372 6e8
6ea :3 0 375 6e4
6ec 6e1 6ee 6ed
:2 0 81 :3 0 91
:3 0 6f0 6f1 0
83 :4 0 cb :4 0
86 :2 0 87 :3 0
379 6f5 6f7 :3 0
86 :2 0 9c :4 0
37c 6f9 6fb :3 0
86 :2 0 9d :4 0
37f 6fd 6ff :3 0
86 :2 0 96 :3 0
382 701 703 :3 0
86 :2 0 9e :4 0
385 705 707 :3 0
86 :2 0 9f :3 0
a0 :3 0 13 :2 0
a1 :2 0 388 70a
70e 38c 709 710
:3 0 38f 6f2 712
:2 0 714 392 715
6ef 714 0 716
394 0 717 396
71a :3 0 71a 0
71a 719 717 718
:6 0 71c 14 :3 0
399 71e 39b 71d
71c :2 0 71f 39d
:2 0 721 0 721
720 6ce 71f :6 0
810 1 :3 0 4d
:3 0 29 :3 0 23
:2 0 3a0 723 726
b5 :2 0 23 :2 0
3a5 728 72a :3 0
72b :2 0 4d :3 0
25 :3 0 23 :2 0
3a8 72d 730 b5
:2 0 a6 :2 0 13
:2 0 3ab 733 735
:3 0 3af 732 737
:3 0 738 :2 0 72c
73a 739 :2 0 a4
:2 0 3b2 6 :3 0
73d :7 0 23 :2 0
741 73e 73f 78e
0 cc :6 0 e
:3 0 3b4 743 744
:3 0 745 :2 0 cc
:3 0 4d :3 0 cd
:3 0 e :3 0 3b6
749 74b 23 :2 0
3b8 748 74e 747
74f 0 767 13
:2 0 ab :2 0 23
:2 0 3bd 752 754
:3 0 cc :3 0 ab
:2 0 23 :2 0 3c2
757 759 :3 0 755
75b 75a :2 0 cc
:3 0 cc :3 0 a6
:2 0 13 :2 0 3c5
75f 761 :3 0 75d
762 0 764 3c8
765 75c 764 0
766 3ca 0 767
3cc 768 746 767
0 769 3cf 0
78c cc :3 0 ab
:2 0 23 :2 0 3d3
76b 76d :3 0 ce
:2 0 3d6 76f 770
:3 0 cc :3 0 cf
:3 0 cf :2 0 d0
:2 0 3d8 774 776
:3 0 777 :2 0 ab
:2 0 23 :2 0 3dd
779 77b :3 0 771
77d 77c :2 0 d1
:3 0 d2 :3 0 77f
780 0 56 :3 0
d3 :3 0 782 783
0 e :3 0 bd
:3 0 3e0 781 787
:2 0 789 3e4 78a
77e 789 0 78b
3e6 0 78c 3e8
78f :3 0 78f 3eb
78f 78e 78c 78d
:6 0 80d 1 :3 0
53 :3 0 d4 :3 0
d5 :3 0 d6 :3 0
d4 :3 0 56 :3 0
d7 :3 0 48 :3 0
d :3 0 54 :3 0
e :4 0 d8 1
:8 0 7b2 a9 :4 0
79d :3 0 bd :3 0
a2 :2 0 3ef 7a0
7a1 :3 0 9 :3 0
95 :3 0 7a3 7a4
0 23 :2 0 7a5
7a6 0 7a8 3f2
7af 9 :3 0 95
:3 0 7a9 7aa 0
aa :2 0 7ab 7ac
0 7ae 3f4 7b0
7a2 7a8 0 7b1
0 7ae 0 7b1
3f6 0 7b2 3f9
805 97 :3 0 9
:3 0 95 :3 0 7b4
7b5 0 96 :3 0
7b6 7b7 0 7b9
3fc 7bb 3fe 7ba
7b9 :2 0 803 98
:3 0 9 :3 0 95
:3 0 7be 7bf 0
96 :3 0 7c0 7c1
0 7fb 7e :3 0
7f :3 0 7c3 7c4
0 7e :3 0 8e
:3 0 7c6 7c7 0
99 :2 0 2 :4 0
d9 :2 0 90 :2 0
13 :2 0 400 7cc
7ce :3 0 403 7c8
7d0 7c5 7d2 7d1
:2 0 81 :3 0 91
:3 0 7d4 7d5 0
83 :4 0 da :4 0
86 :2 0 87 :3 0
407 7d9 7db :3 0
86 :2 0 9c :4 0
40a 7dd 7df :3 0
86 :2 0 9d :4 0
40d 7e1 7e3 :3 0
86 :2 0 96 :3 0
410 7e5 7e7 :3 0
86 :2 0 9e :4 0
413 7e9 7eb :3 0
86 :2 0 9f :3 0
a0 :3 0 13 :2 0
a1 :2 0 416 7ee
7f2 41a 7ed 7f4
:3 0 41d 7d6 7f6
:2 0 7f8 420 7f9
7d3 7f8 0 7fa
422 0 7fb 424
7fe :3 0 7fe 0
7fe 7fd 7fb 7fc
:6 0 800 17 :3 0
427 802 429 801
800 :2 0 803 42b
:2 0 805 0 805
804 7b2 803 :6 0
80d 1 :3 0 32
:3 0 db :3 0 d
:3 0 42e 808 80a
807 80b 0 80d
430 80e 73b 80d
0 80f 434 0
810 436 811 2a9
810 0 812 43d
0 889 4d :3 0
41 :3 0 23 :2 0
43f 813 816 b5
:2 0 23 :2 0 444
818 81a :3 0 9
:3 0 95 :3 0 81c
81d 0 56 :3 0
b1 :3 0 81f 820
0 81e 821 0
824 b0 :3 0 447
839 4d :3 0 32
:3 0 23 :2 0 449
825 828 b5 :2 0
aa :2 0 44e 82a
82c :3 0 9 :3 0
95 :3 0 82e 82f
0 4d :3 0 32
:3 0 23 :2 0 451
831 834 830 835
0 837 454 838
82d 837 0 83a
81b 824 0 83a
456 0 889 7e
:3 0 7f :3 0 83b
83c 0 7e :3 0
8e :3 0 83e 83f
0 d0 :2 0 2
:4 0 dc :2 0 90
:2 0 13 :2 0 459
844 846 :3 0 45c
840 848 83d 84a
849 :2 0 81 :3 0
91 :3 0 84c 84d
0 83 :4 0 dd
:4 0 86 :2 0 87
:3 0 460 851 853
:3 0 86 :2 0 9c
:4 0 463 855 857
:3 0 86 :2 0 de
:4 0 466 859 85b
:3 0 86 :2 0 4d
:3 0 8b :3 0 9
:3 0 95 :3 0 860
861 0 469 85f
863 8c :4 0 46b
85e 866 46e 85d
868 :3 0 86 :2 0
df :4 0 471 86a
86c :3 0 86 :2 0
89 :4 0 474 86e
870 :3 0 86 :2 0
3 :3 0 477 872
874 :3 0 86 :2 0
89 :4 0 47a 876
878 :3 0 47d 84e
87a :2 0 87c 480
87d 84b 87c 0
87e 482 0 889
d1 :3 0 e0 :3 0
87f 880 0 9
:3 0 95 :3 0 882
883 0 3 :3 0
bd :3 0 484 881
887 :2 0 889 488
92a 9 :3 0 e1
:2 0 4 88a 88b
0 7e :3 0 7f
:3 0 88d 88e 0
7e :3 0 8e :3 0
890 891 0 99
:2 0 2 :4 0 e2
:2 0 90 :2 0 13
:2 0 493 896 898
:3 0 496 892 89a
88f 89c 89b :2 0
81 :3 0 91 :3 0
89e 89f 0 83
:4 0 e3 :4 0 86
:2 0 87 :3 0 49a
8a3 8a5 :3 0 49d
8a0 8a7 :2 0 8c0
81 :3 0 91 :3 0
8a9 8aa 0 83
:4 0 e4 :4 0 86
:2 0 9 :3 0 e5
:3 0 8af 8b0 0
4a0 8ae 8b2 :3 0
86 :2 0 df :4 0
4a3 8b4 8b6 :3 0
86 :2 0 9 :3 0
e6 :3 0 8b9 8ba
0 4a6 8b8 8bc
:3 0 4a9 8ab 8be
:2 0 8c0 4ac 8c1
89d 8c0 0 8c2
4af 0 8d2 a7
:3 0 7d :3 0 9
:3 0 e5 :3 0 e7
:3 0 9 :3 0 e6
:3 0 3 :3 0 e8
:4 0 e9 1 :a 0
8ce :2 0 8d2 8c4
8cc :2 0 8 :3 0
7d :3 0 8d0 :2 0
8d2 4b1 8d4 4b5
8d3 8d2 :2 0 928
98 :3 0 9 :3 0
95 :3 0 8d7 8d8
0 96 :3 0 8d9
8da 0 920 7e
:3 0 7f :3 0 8dc
8dd 0 7e :3 0
8e :3 0 8df 8e0
0 99 :2 0 2
:4 0 ea :2 0 90
:2 0 13 :2 0 4b7
8e5 8e7 :3 0 4ba
8e1 8e9 8de 8eb
8ea :2 0 81 :3 0
91 :3 0 8ed 8ee
0 83 :4 0 eb
:4 0 86 :2 0 87
:3 0 4be 8f2 8f4
:3 0 4c1 8ef 8f6
:2 0 910 81 :3 0
91 :3 0 8f8 8f9
0 83 :4 0 9d
:4 0 86 :2 0 96
:3 0 4c4 8fd 8ff
:3 0 86 :2 0 9e
:4 0 4c7 901 903
:3 0 86 :2 0 9f
:3 0 a0 :3 0 13
:2 0 a1 :2 0 4ca
906 90a 4ce 905
90c :3 0 4d1 8fa
90e :2 0 910 4d4
911 8ec 910 0
912 4d7 0 920
a7 :3 0 7d :3 0
e7 :3 0 9 :3 0
e6 :3 0 3 :3 0
e8 :4 0 ec 1
:a 0 91c :2 0 920
914 91a :2 0 8
:3 0 7d :3 0 91e
:2 0 920 4d9 923
:3 0 923 0 923
922 920 921 :6 0
925 1 :3 0 4de
927 4e0 926 925
:2 0 928 4e2 :2 0
92a 4e5 92a 929
889 928 :6 0 92b
:2 0 2 14 92a
92e :3 0 92d 92b
92f :8 0 
51a
4
:3 0 1 3 1
7 1 b 3
6 a e 1
18 1 16 1
1f 1 1d 1
26 1 24 1
2d 1 2b 1
34 1 32 1
3b 1 39 1
40 1 45 1
4a 1 4f 1
54 1 59 1
5e 1 63 1
68 1 6f 1
6d 1 76 1
74 1 7d 1
7b 1 82 1
88 1 8e 1
94 1 9a 1
a0 1 a6 1
ac 1 b1 1
b8 1 b6 1
bd 1 c3 1
c9 1 cf 1
d5 1 db 1
e0 1 e5 1
eb 1 f1 1
f7 1 fd 1
103 1 108 1
10d 1 113 1
119 1 11f 1
125 1 12b 1
130 1 176 1
1b1 1 1b7 1
1c4 1 1d0 2
1d3 1d5 2 1d8
1da 2 1dc 1de
2 1e0 1e2 2
1e4 1e6 1 1ec
2 1ee 1ef 2
1e8 1f1 2 1f3
1f5 2 1f7 1f9
2 1fb 1fd 2
1ff 201 2 203
205 4 1cc 1d2
1d7 207 1 209
1 20b 2 215
217 3 213 214
219 2 222 224
2 221 226 1
228 1 22a 2
23d 242 1 249
1 244 2 25c
25e 3 25a 25b
260 2 269 26b
2 26d 26f 2
271 273 2 275
277 2 279 27b
3 280 281 282
2 27d 284 2
268 286 1 288
1 28a 2 253
28b 1 28f 1
24e 2 24c 293
1 29e 2 29a
29e 1 2a2 2
2ac 2ae 3 2b0
2b1 2b2 1 2b8
5 2c3 2c4 2c5
2c6 2c7 1 2d2
1 2d8 2 2da
2db 1 2e3 2
2df 2e3 1 2e8
1 2ea 2 2ed
2ee 1 2f2 2
2f0 2f2 2 2f6
2f7 1 2fb 2
2f9 2fb 5 305
306 307 308 309
2 314 315 2
317 319 4 302
30c 311 31c 1
31e 1 31f 16
32e 32f 330 331
332 333 334 335
336 337 338 339
33a 33b 33c 33d
33e 33f 340 341
342 343 1 34e
1 354 2 356
357 1 35f 2
35b 35f 1 364
1 366 1a 36a
36b 36c 36d 36e
36f 370 371 372
373 374 375 376
377 378 379 37a
37b 37c 37d 37e
37f 380 381 382
383 2 388 389
1 38d 2 38b
38d 2 392 393
2 395 397 1
39a 2 39e 39f
1 3a5 2 3a1
3a5 2 3aa 3ab
2 3ad 3af 1
3b2 2 3b5 3b4
5 344 358 367
386 3b6 2 3bc
3c2 1 3c9 1
3c4 2 3dc 3de
3 3da 3db 3e0
2 3e9 3eb 2
3ed 3ef 2 3f1
3f3 2 3f5 3f7
2 3f9 3fb 3
400 401 402 2
3fd 404 2 3e8
406 1 408 1
40a 2 3d3 40b
1 40f 1 3ce
2 3cc 413 2
419 41a 1 41e
2 41c 41e 2
423 424 1 428
2 426 428 2
441 446 1 44d
1 448 2 460
462 3 45e 45f
464 2 46d 46f
2 471 473 2
475 477 2 479
47b 2 47d 47f
3 484 485 486
2 481 488 2
46c 48a 1 48c
1 48e 2 457
48f 1 493 1
452 2 450 497
1 49c 1 4a1
1 4c4 2 4c3
4c4 1 4cc 1
4d2 2 4d4 4d5
2 4c1 4d6 1
4dd 1 4d8 2
4f0 4f2 3 4ee
4ef 4f4 2 4fd
4ff 2 501 503
2 505 507 2
509 50b 2 50d
50f 3 514 515
516 2 511 518
2 4fc 51a 1
51c 1 51e 2
4e7 51f 1 523
1 4e2 2 4e0
527 1 52a 1
538 2 537 538
1 540 1 546
2 548 549 2
535 54a 1 551
1 54c 2 564
566 3 562 563
568 2 571 573
2 575 577 2
579 57b 2 57d
57f 2 581 583
3 588 589 58a
2 585 58c 2
570 58e 1 590
1 592 2 55b
593 1 597 1
556 2 554 59b
1 59e 2 5a1
5a2 1 5a3 1
5a6 1 5ac 1
5b4 1 5d7 2
5d6 5d7 1 5df
1 5e5 2 5e7
5e8 2 5d4 5e9
1 5f0 1 5eb
2 603 605 3
601 602 607 2
610 612 2 614
616 2 618 61a
2 61c 61e 2
620 622 3 627
628 629 2 624
62b 2 60f 62d
1 62f 1 631
2 5fa 632 1
636 1 5f5 2
5f3 63a 1 63d
1 64b 2 64a
64b 1 653 1
659 2 65b 65c
2 648 65d 1
664 1 65f 2
677 679 3 675
676 67b 2 684
686 2 688 68a
2 68c 68e 2
690 692 2 694
696 3 69b 69c
69d 2 698 69f
2 683 6a1 1
6a3 1 6a5 2
66e 6a6 1 6aa
1 669 2 667
6ae 1 6b1 2
6b4 6b5 1 6b6
2 6b9 6b8 4
430 433 49a 6ba
1 6bc 6 323
326 329 3b9 416
6bd 2 6bf 6c0
4 2c8 2dc 2eb
6c1 2 6c7 6cd
1 6d4 1 6cf
2 6e7 6e9 3
6e5 6e6 6eb 2
6f4 6f6 2 6f8
6fa 2 6fc 6fe
2 700 702 2
704 706 3 70b
70c 70d 2 708
70f 2 6f3 711
1 713 1 715
2 6de 716 1
71a 1 6d9 2
6d7 71e 2 724
725 1 729 2
727 729 2 72e
72f 1 734 1
736 2 731 736
1 73c 1 742
1 74a 2 74c
74d 1 753 2
751 753 1 758
2 756 758 2
75e 760 1 763
1 765 2 750
766 1 768 1
76c 2 76a 76c
1 76e 2 772
775 1 77a 2
778 77a 3 784
785 786 1 788
1 78a 2 769
78b 1 740 1
79f 2 79e 79f
1 7a7 1 7ad
2 7af 7b0 2
79c 7b1 1 7b8
1 7b3 2 7cb
7cd 3 7c9 7ca
7cf 2 7d8 7da
2 7dc 7de 2
7e0 7e2 2 7e4
7e6 2 7e8 7ea
3 7ef 7f0 7f1
2 7ec 7f3 2
7d7 7f5 1 7f7
1 7f9 2 7c2
7fa 1 7fe 1
7bd 2 7bb 802
1 809 3 78f
805 80c 1 80e
6 2b5 2bb 2be
6c4 721 80f 1
811 2 814 815
1 819 2 817
819 1 822 2
826 827 1 82b
2 829 82b 2
832 833 1 836
2 839 838 2
843 845 3 841
842 847 2 850
852 2 854 856
2 858 85a 1
862 2 864 865
2 85c 867 2
869 86b 2 86d
86f 2 871 873
2 875 877 2
84f 879 1 87b
1 87d 3 884
885 886 a 20c
22b 22e 231 234
296 812 83a 87e
888 2 895 897
3 893 894 899
2 8a2 8a4 2
8a1 8a6 2 8ad
8b1 2 8b3 8b5
2 8b7 8bb 2
8ac 8bd 2 8a8
8bf 1 8c1 3
8c2 8cd 8d1 1
88c 2 8e4 8e6
3 8e2 8e3 8e8
2 8f1 8f3 2
8f0 8f5 2 8fc
8fe 2 900 902
3 907 908 909
2 904 90b 2
8fb 90d 2 8f7
90f 1 911 4
8db 912 91b 91f
1 923 1 8d6
2 8d4 927 34
1b 22 29 30
37 3e 43 48
4d 52 57 5c
61 66 6b 72
79 80 86 8c
92 98 9e a4
aa af b4 bb
c1 c7 cd d3
d9 de e3 e9
ef f5 fb 101
106 10b 111 117
11d 123 129 12e
134 177 1b2 1bc

1
4
0 
92e
0
1
28
19
39
0 1 1 1 4 1 6 6
8 6 a 6 c 6 e 6
10 6 12 1 14 1 1 17
1 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0

b 1 0
2 0 1
e0 1 0
94 1 0
cf 1 0
3 1 0
f1 1 0
4f 1 0
74 1 0
119 1 0
16 1 0
c9 1 0
bd 1 0
39 1 0
db 1 0
fd 1 0
12b 1 0
b1 1 0
d5 1 0
73c 16 0
11f 1 0
68 1 0
8e 1 0
54 1 0
40 1 0
4a 1 0
137 1 2
10d 1 0
82 1 0
e5 1 0
b6 1 0
108 1 0
59 1 0
9a 1 0
113 1 0
6d 1 0
24 1 0
45 1 0
130 1 0
eb 1 0
7b 1 0
ac 1 0
63 1 0
a0 1 0
32 1 0
1d 1 0
17d 1 3
7 1 0
103 1 0
f7 1 0
5e 1 0
2b 1 0
88 1 0
125 1 0
c3 1 0
1b7 1 0
a6 1 0
0

/
SHOW ERRORS FUNCTION SP_ACPT_ETC_WBS;
ALTER FUNCTION SP_ACPT_ETC_WBS COMPILE DEBUG;
/****************************************************************************************
Reject ETC and/or FORECAST_START, WBS level
  Bitflag =
    1 - ETC
    2 - FORECAST_START
    3 - ETC + FORECAST_START
****************************************************************************************/
Prompt == FUNCTION SP_RJCT_ETC_ASSIGN ==
CREATE OR REPLACE FUNCTION SP_RJCT_ETC_ASSIGN wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
a2
2 :e:
1FUNCTION:
1SP_RJCT_ETC_ASSIGN:
1ID:
1VARCHAR2:
1BITFLAG001:
1PLS_INTEGER:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_TSK_ASSIGNMENT_ID:
132:
1H_PROJECT_ID:
1H_TASK_ID:
1H_START_DATE:
1TIMESTAMP:
1H_BIT_FLAG:
11:
1H_BITFLAG:
10:
1H_TEMP_SC_STATE:
1H_STEPS_EXIST:
1H_SCHED_WORK:
1H_WORK_TODATE:
1H_MLST_PERCENT:
1I_PERCENT_COMPLETE:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_RJCT_ETC_ASSIGN(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
130:
1+:
1FPUT:
1               ENTER:: SP_RJCT_ETC_ASSIGN(31) :: cplan.sql(19616) :: :
1PROJECT_ID:
1TASK_ID:
1BIT_FLAG:
1SCHED_WORK:
1PLANNED_WORK:
1WORK_TODATE:
1MILESTONE_PERCENT:
1TASK_ASSIGNMENTS:
1TSK_ASSIGNMENT_ID:
1SELECT PROJECT_ID, TASK_ID, BIT_FLAG, SCHED_WORK+PLANNED_WORK, WORK_TODATE, M+
1ILESTONE_PERCENT:n      INTO h_project_id, h_task_id, h_bit_flag, h_sched_wor+
1k, h_work_todate, h_mlst_percent:n      FROM TASK_ASSIGNMENTS WHERE TSK_ASSIG+
1NMENT_ID = h_tsk_assignment_id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
149:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(50) :: cplan.sql(19635) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1=:
12:
1ETC:
1UPDATE TASK_ASSIGNMENTS SET ETC = SCHED_WORK - WORK_TODATE--, MILESTONE_PERCE+
1NT = (WORK_TODATE * 10000) / SCHED_WORK:n            WHERE TSK_ASSIGNMENT_ID +
1= h_tsk_assignment_id:
1FOUND:
1TRUE:
1100:
171:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(72) :: cplan.sql(19657) :: :
1ACTIVITIES:
1ETC_MODIFIED:
1ELEMENT_ID:
1TYPE_ID:
1PMO_CNST:
1SC_TIMESHEET_ACTIVITY:
1UPDATE ACTIVITIES:n            SET ETC_MODIFIED = 'N':n            WHERE TSK_+
1ASSIGNMENT_ID = h_tsk_assignment_id:n            AND TSK_ASSIGNMENT_ID <> ELE+
1MENT_ID:n            AND TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY:
194:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(95) :: cplan.sql(19680) :: :
1UPDATE TASK_ASSIGNMENTS SET ETC = PLANNED_WORK, WORK_TODATE = 0, MILESTONE_PE+
1RCENT = 0:n            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
1114:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(115) :: cplan.sql(19700) :: :
1TMT_WBS:
1ROLLUP_BITS:
1PMO_STPU:
1BIT_OR:
1SC_SCHED_BIT:
1UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_SCH+
1ED_BIT) WHERE ELEMENT_ID = h_task_id:
1134:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(135) :: cplan.sql(19720) :: :
1FORECAST_START:
1FORECAST_FINISH:
1FORECAST_DURATION:
1UPDATE TASK_ASSIGNMENTS SET FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FO+
1RECAST_DURATION = 0:n            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_i+
1d:
1155:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(156) :: cplan.sql(19741) :: :
1174:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(175) :: cplan.sql(19760) :: :
1UPDATE TASK_ASSIGNMENTS SET FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FO+
1RECAST_DURATION = 0,:n               ETC = SCHED_WORK - WORK_TODATE--, MILEST+
1ONE_PERCENT = (WORK_TODATE * 10000) / SCHED_WORK:n               WHERE TSK_AS+
1SIGNMENT_ID = h_tsk_assignment_id:
1196:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(197) :: cplan.sql(19782) :: :
1UPDATE ACTIVITIES:n               SET ETC_MODIFIED = 'N':n               WHER+
1E TSK_ASSIGNMENT_ID = h_tsk_assignment_id:n               AND TSK_ASSIGNMENT_+
1ID <> ELEMENT_ID:n               AND TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY:
1219:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(220) :: cplan.sql(19805) :: :
1UPDATE TASK_ASSIGNMENTS SET FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FO+
1RECAST_DURATION = 0,:n               ETC = PLANNED_WORK, WORK_TODATE = 0, MIL+
1ESTONE_PERCENT = 0:n               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment+
1_id:
1240:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(241) :: cplan.sql(19826) :: :
1260:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(261) :: cplan.sql(19846) :: :
1BITAND:
1>:
1TRUNC:
1TO_NUMBER:
1/:
1*:
110000.00:
1-:
1<:
1SC_PERCENT_COMPLETE_BIT:
1ACTUAL_FINISH:
1SCHED_FINISH:
1UPDATE TASK_ASSIGNMENTS SET:n            ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP+
1_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT),:n            ACTUAL_FINISH = CASE:n +
1           WHEN ACTUAL_FINISH IS NULL AND ETC = 0 THEN SCHED_FINISH:n        +
1    ELSE ACTUAL_FINISH:n            END:n            WHERE TSK_ASSIGNMENT_ID +
1= h_tsk_assignment_id:
1292:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(293) :: cplan.sql(19878) :: :
1UPDATE TASK_ASSIGNMENTS SET:n         ACTUAL_FINISH = CASE:n         WHEN ACT+
1UAL_FINISH IS NULL AND ETC = 0 THEN SCHED_FINISH:n         ELSE ACTUAL_FINISH+
1:n         END:n         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
1317:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(318) :: cplan.sql(19903) :: :
1SC_SQL_SUCCESS:
1!=:
1FC_FIX_SUMMARY_TS:
1FC_ROLLUP:
14:
1332:
1       RETURN_STATUS:: SP_RJCT_ETC_ASSIGN(333) :: cplan.sql(19918) :: :
1STATE:: :
1 ID:: :
1PMODEFINES:
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
1339:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(340) :: cplan.sql(19925) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1351:
1           EXCEPTION:: SP_RJCT_ETC_ASSIGN(352) :: cplan.sql(19937) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
861
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 81 b0 a3 :2 a0
6b 1c 81 b0 :2 a0 6b :2 a0 6b
6e a5 b a 10 :2 a0 6b 6e
:2 a0 6b 6e a5 b 6e 7e a0
b4 2e 6e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :3 a0 a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 d :2 a0 d :10 a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 51
a5 b 7e 51 b4 2e :2 a0 :2 51
a5 b 7e 6e b4 2e :6 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :9 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :7 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:2 19 3c :9 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 :2 a0 51 a5 b 7e
51 b4 2e :6 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :9 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 :2 a0 :2 51 a5 b
7e 6e b4 2e :9 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :9 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :a a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :2 19 3c
:9 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c b7 :2 19 3c :3 a0 51
a5 b 51 a5 b 5a 7e 51
b4 2e 5a :2 a0 51 a5 b 7e
51 b4 2e 5a a 10 :5 a0 a5
b 51 a5 b 7e :3 a0 a5 b
51 a5 b b4 2e 5a 7e 51
b4 2e 5a 7e :2 a0 51 a5 b
b4 2e a5 b d a0 7e 51
b4 2e 5a a0 :2 7e 51 b4 2e
b4 2e 52 10 :e a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 19 3c b7
:8 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c :2 a0 6b a0 7e a0
6b b4 2e 5a :2 a0 51 a5 b
7e 51 b4 2e 5a a 10 :4 a0
a5 b d b7 19 3c :3 a0 a5
b d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 :2 a0
6b :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 :2 a0 6b :2 6e 7e :2 a0 6b b4
2e 7e 6e b4 2e 7e :2 a0 6b
b4 2e a5 57 b7 19 3c :9 a0
12a e9 dd :2 a0 65 b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c :7 a0 12a e9
dd :2 a0 65 b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 68 4f
1d 17 b5 
861
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 30
56 5a 5e 62 65 69 86 71
75 78 79 81 70 a3 91 6d
95 96 9e 90 c0 ae 8d b2
b3 bb ad dc cb cf d7 aa
f8 e3 e7 ea eb f3 ca 114
103 107 c7 10f 102 130 11f 123
ff 12b 11e 14c 13b 13f 11b 147
13a 168 157 15b 137 163 156 184
173 177 153 17f 172 1a0 18f 193
16f 19b 18e 1bc 1ab 1af 1b7 18b
1db 1c3 1c7 1cb 1ce 1d6 1aa 1e2
1e6 1a7 1ea 1ee 1f2 1f5 1fa 1fb
1 1fd 202 206 20a 20d 212 216
21a 21d 222 223 225 22a 22d 231
232 237 23c 23f 244 245 24a 24d
251 252 257 25a 25f 260 265 268
26d 26e 273 276 27a 27e 282 283
285 28a 28b 28d 28e 293 296 29b
29c 2a1 2a4 2a9 2aa 2af 2b2 2b6
2b7 2bc 2bf 2c4 2c5 2ca 2cd 2d2
2d3 2d8 2d9 2de 2e0 2e4 2e7 2eb
2ef 2f2 2f6 2fa 2fd 300 305 308
30b 30e 30f 314 315 1 317 31c
320 324 327 32c 331 334 338 339
33e 33f 344 346 34a 34d 351 355
359 35d 361 365 369 36d 371 375
379 37d 381 385 389 38d 391 395
399 39d 3a1 3a5 3b1 3b5 3b9 3bc
3c0 3c4 3c6 3ca 3ce 3d2 3d5 3d9
3dd 3df 3e0 3e5 1 3e9 3ed 3f1
3f4 3f8 3fc 400 404 407 40b 40f
412 415 41a 41d 420 423 424 429
42a 1 42c 431 435 439 43c 441
446 449 44d 44e 453 456 45b 45c
461 464 469 46a 46f 472 476 477
47c 47f 484 485 48a 48d 491 495
498 49b 49c 49e 49f 4a4 4a5 4aa
4ac 4b0 4b3 4b5 4b9 4bb 4c7 4c9
4cb 4cc 4d1 4d5 4d7 4e3 4e5 4e9
4ed 4f0 4f1 4f3 4f6 4f9 4fa 4ff
503 507 50a 50d 50e 510 513 518
519 51e 522 526 52a 52e 532 536
542 546 54b 54f 552 553 558 55c
560 563 566 56a 56c 570 574 577
57a 57e 580 584 588 58b 58d 591
595 599 59c 5a0 5a4 5a6 5a7 5ac
1 5b0 5b4 5b8 5bb 5bf 5c3 5c7
5cb 5ce 5d2 5d6 5d9 5dc 5e1 5e4
5e7 5ea 5eb 5f0 5f1 1 5f3 5f8
5fc 600 603 608 60d 610 614 615
61a 61d 622 623 628 62b 630 631
636 639 63d 63e 643 646 64b 64c
651 654 658 65c 65f 662 663 665
666 66b 66c 671 673 677 67a 67c
680 682 68e 690 692 693 698 69c
69e 6aa 6ac 6b0 6b4 6b8 6bc 6c0
6c4 6c8 6cc 6d0 6dc 6e0 6e5 6e9
6ec 6ed 6f2 6f6 6fa 6fd 700 704
706 70a 70e 711 714 718 71a 71e
722 725 727 72b 72f 733 736 73a
73e 740 741 746 1 74a 74e 752
755 759 75d 761 765 768 76c 770
773 776 77b 77e 781 784 785 78a
78b 1 78d 792 796 79a 79d 7a2
7a7 7aa 7ae 7af 7b4 7b7 7bc 7bd
7c2 7c5 7ca 7cb 7d0 7d3 7d7 7d8
7dd 7e0 7e5 7e6 7eb 7ee 7f2 7f6
7f9 7fc 7fd 7ff 800 805 806 80b
80d 811 814 816 81a 81c 828 82a
82c 82d 832 836 838 844 846 848
84c 850 854 858 85c 860 864 870
874 879 87d 880 881 886 88a 88e
891 894 898 89a 89e 8a2 8a5 8a8
8ac 8ae 8b2 8b6 8b9 8bb 8bf 8c3
8c7 8ca 8ce 8d2 8d4 8d5 8da 1
8de 8e2 8e6 8e9 8ed 8f1 8f5 8f9
8fc 900 904 907 90a 90f 912 915
918 919 91e 91f 1 921 926 92a
92e 931 936 93b 93e 942 943 948
94b 950 951 956 959 95e 95f 964
967 96b 96c 971 974 979 97a 97f
982 986 98a 98d 990 991 993 994
999 99a 99f 9a1 9a5 9a8 9aa 9ae
9b0 9bc 9be 9c0 9c1 9c6 9ca 9cc
9d8 9da 9dc 9e0 9e4 9e7 9eb 9ef
9f3 9f7 9fb 9ff a03 a07 a0b a17
a1b a20 a24 a27 a28 a2d a31 a35
a38 a3b a3f a41 a45 a49 a4c a4f
a53 a55 a59 a5d a60 a62 a66 a6a
a6e a71 a75 a79 a7b a7c a81 1
a85 a89 a8d a90 a94 a98 a9c aa0
aa3 aa7 aab aae ab1 ab6 ab9 abc
abf ac0 ac5 ac6 1 ac8 acd ad1
ad5 ad8 add ae2 ae5 ae9 aea aef
af2 af7 af8 afd b00 b05 b06 b0b
b0e b12 b13 b18 b1b b20 b21 b26
b29 b2d b31 b34 b37 b38 b3a b3b
b40 b41 b46 b48 b4c b4f b51 b55
b57 b63 b65 b67 b68 b6d b71 b73
b7f b81 b83 b87 b8b b8e b8f b91
b94 b97 b98 b9d ba1 ba5 ba9 bad
bb1 bb5 bc1 bc5 bca bce bd1 bd2
bd7 bdb bdf be2 be5 be9 beb bef
bf3 bf6 bf9 bfd bff c03 c07 c0a
c0c c10 c14 c18 c1b c1f c23 c25
c26 c2b 1 c2f c33 c37 c3a c3e
c42 c46 c4a c4d c51 c55 c58 c5b
c60 c63 c66 c69 c6a c6f c70 1
c72 c77 c7b c7f c82 c87 c8c c8f
c93 c94 c99 c9c ca1 ca2 ca7 caa
caf cb0 cb5 cb8 cbc cbd cc2 cc5
cca ccb cd0 cd3 cd7 cdb cde ce1
ce2 ce4 ce5 cea ceb cf0 cf2 cf6
cf9 cfb cff d01 d0d d0f d11 d12
d17 d1b d1d d29 d2b d2f d33 d37
d3b d3f d43 d47 d4b d4f d5b d5f
d64 d68 d6b d6c d71 d75 d79 d7c
d7f d83 d85 d89 d8d d90 d93 d97
d99 d9d da1 da4 da6 daa dae db2
db5 db9 dbd dbf dc0 dc5 1 dc9
dcd dd1 dd4 dd8 ddc de0 de4 de7
deb def df2 df5 dfa dfd e00 e03
e04 e09 e0a 1 e0c e11 e15 e19
e1c e21 e26 e29 e2d e2e e33 e36
e3b e3c e41 e44 e49 e4a e4f e52
e56 e57 e5c e5f e64 e65 e6a e6d
e71 e75 e78 e7b e7c e7e e7f e84
e85 e8a e8c e90 e93 e95 e99 e9b
ea7 ea9 eab eac eb1 eb5 eb7 ec3
ec5 ec7 ecb ecf ed2 ed5 ed6 ed8
edb ee0 ee1 ee6 eea eee ef2 ef6
efa efe f02 f06 f0a f16 f1a f1f
f23 f26 f27 f2c f30 f34 f37 f3a
f3e f40 f44 f48 f4b f4e f52 f54
f58 f5c f5f f61 f65 f69 f6d f70
f74 f78 f7a f7b f80 1 f84 f88
f8c f8f f93 f97 f9b f9f fa2 fa6
faa fad fb0 fb5 fb8 fbb fbe fbf
fc4 fc5 1 fc7 fcc fd0 fd4 fd7
fdc fe1 fe4 fe8 fe9 fee ff1 ff6
ff7 ffc fff 1004 1005 100a 100d 1011
1012 1017 101a 101f 1020 1025 1028 102c
1030 1033 1036 1037 1039 103a 103f 1040
1045 1047 104b 104e 1050 1054 1056 1062
1064 1066 1067 106c 1070 1072 107e 1080
1084 1088 108c 1090 1094 1098 109c 10a0
10a4 10b0 10b4 10b9 10bd 10c0 10c1 10c6
10ca 10ce 10d1 10d4 10d8 10da 10de 10e2
10e5 10e8 10ec 10ee 10f2 10f6 10f9 10fb
10ff 1103 1107 110a 110e 1112 1114 1115
111a 1 111e 1122 1126 1129 112d 1131
1135 1139 113c 1140 1144 1147 114a 114f
1152 1155 1158 1159 115e 115f 1 1161
1166 116a 116e 1171 1176 117b 117e 1182
1183 1188 118b 1190 1191 1196 1199 119e
119f 11a4 11a7 11ab 11ac 11b1 11b4 11b9
11ba 11bf 11c2 11c6 11ca 11cd 11d0 11d1
11d3 11d4 11d9 11da 11df 11e1 11e5 11e8
11ea 11ee 11f0 11fc 11fe 1200 1201 1206
120a 120c 1218 121a 121c 1220 1224 1228
122c 1230 1234 1238 123c 1240 1244 1250
1254 1259 125d 1260 1261 1266 126a 126e
1271 1274 1278 127a 127e 1282 1285 1288
128c 128e 1292 1296 1299 129b 129f 12a3
12a7 12aa 12ae 12b2 12b4 12b5 12ba 1
12be 12c2 12c6 12c9 12cd 12d1 12d5 12d9
12dc 12e0 12e4 12e7 12ea 12ef 12f2 12f5
12f8 12f9 12fe 12ff 1 1301 1306 130a
130e 1311 1316 131b 131e 1322 1323 1328
132b 1330 1331 1336 1339 133e 133f 1344
1347 134b 134c 1351 1354 1359 135a 135f
1362 1366 136a 136d 1370 1371 1373 1374
1379 137a 137f 1381 1385 1388 138a 138e
1390 139c 139e 13a0 13a1 13a6 13aa 13ac
13b8 13ba 13bc 13c0 13c4 13c7 13cb 13cf
13d3 13d7 13db 13df 13e3 13e7 13eb 13f7
13fb 1400 1404 1407 1408 140d 1411 1415
1418 141b 141f 1421 1425 1429 142c 142f
1433 1435 1439 143d 1440 1442 1446 144a
144e 1451 1455 1459 145b 145c 1461 1
1465 1469 146d 1470 1474 1478 147c 1480
1483 1487 148b 148e 1491 1496 1499 149c
149f 14a0 14a5 14a6 1 14a8 14ad 14b1
14b5 14b8 14bd 14c2 14c5 14c9 14ca 14cf
14d2 14d7 14d8 14dd 14e0 14e5 14e6 14eb
14ee 14f2 14f3 14f8 14fb 1500 1501 1506
1509 150d 1511 1514 1517 1518 151a 151b
1520 1521 1526 1528 152c 152f 1531 1535
1537 1543 1545 1547 1548 154d 1551 1553
155f 1561 1563 1567 156b 156e 1570 1574
1578 157b 157f 1583 1587 158a 158b 158d
1590 1591 1593 1596 1599 159c 159d 15a2
15a5 15a9 15ad 15b0 15b1 15b3 15b6 15b9
15ba 15bf 1 15c2 15c7 15cb 15cf 15d3
15d7 15db 15dc 15de 15e1 15e2 15e4 15e7
15eb 15ef 15f3 15f4 15f6 15f9 15fa 15fc
15fd 1602 1605 1608 160b 160c 1611 1614
1617 161b 161f 1622 1623 1625 1626 162b
162c 162e 1632 1636 1639 163c 163d 1642
1645 1649 164c 164f 1652 1653 1658 1659
1 165e 1663 1667 166b 166f 1673 1677
167b 167f 1683 1687 168b 168f 1693 1697
169b 16a7 16ab 16b0 16b4 16b7 16b8 16bd
16c1 16c5 16c8 16cb 16cf 16d1 16d5 16d9
16dc 16df 16e3 16e5 16e9 16ed 16f0 16f2
16f6 16fa 16fe 1701 1705 1709 170b 170c
1711 1 1715 1719 171d 1720 1724 1728
172c 1730 1733 1737 173b 173e 1741 1746
1749 174c 174f 1750 1755 1756 1 1758
175d 1761 1765 1768 176d 1772 1775 1779
177a 177f 1782 1787 1788 178d 1790 1795
1796 179b 179e 17a2 17a3 17a8 17ab 17b0
17b1 17b6 17b9 17bd 17c1 17c4 17c7 17c8
17ca 17cb 17d0 17d1 17d6 17d8 17dc 17df
17e1 17e5 17e7 17f3 17f5 17f7 17f8 17fd
1801 1803 180f 1811 1813 1817 181a 181c
1820 1824 1828 182c 1830 1834 1838 183c
1848 184c 1851 1855 1858 1859 185e 1862
1866 1869 186c 1870 1872 1876 187a 187d
1880 1884 1886 188a 188e 1891 1893 1897
189b 189f 18a2 18a6 18aa 18ac 18ad 18b2
1 18b6 18ba 18be 18c1 18c5 18c9 18cd
18d1 18d4 18d8 18dc 18df 18e2 18e7 18ea
18ed 18f0 18f1 18f6 18f7 1 18f9 18fe
1902 1906 1909 190e 1913 1916 191a 191b
1920 1923 1928 1929 192e 1931 1936 1937
193c 193f 1943 1944 1949 194c 1951 1952
1957 195a 195e 1962 1965 1968 1969 196b
196c 1971 1972 1977 1979 197d 1980 1982
1986 1988 1994 1996 1998 1999 199e 19a2
19a4 19b0 19b2 19b4 19b8 19bc 19bf 19c3
19c7 19ca 19ce 19d1 19d5 19d8 19d9 19de
19e1 19e5 19e9 19ec 19ed 19ef 19f2 19f5
19f6 19fb 1 19fe 1a03 1a07 1a0b 1a0f
1a13 1a14 1a16 1a1a 1a1c 1a20 1a23 1a27
1a2b 1a2f 1a30 1a32 1a36 1a3a 1a3e 1a41
1a45 1a49 1a4c 1a4f 1a54 1a57 1a5a 1a5d
1a5e 1a63 1a64 1 1a66 1a6b 1a6f 1a73
1a76 1a7b 1a80 1a83 1a87 1a88 1a8d 1a90
1a95 1a96 1a9b 1a9e 1aa3 1aa4 1aa9 1aac
1ab0 1ab4 1ab8 1abc 1abf 1ac0 1ac2 1ac7
1ac8 1aca 1acb 1ad0 1ad3 1ad8 1ad9 1ade
1ae1 1ae6 1ae7 1aec 1aef 1af3 1af4 1af9
1afc 1b01 1b02 1b07 1b08 1b0d 1b0f 1b13
1b16 1b1a 1b1e 1b21 1b25 1b29 1b2c 1b30
1b34 1b35 1b3a 1b3c 1b40 1b44 1b47 1b4b
1b4f 1b52 1b56 1b5a 1b5d 1b60 1b65 1b68
1b6b 1b6e 1b6f 1b74 1b75 1 1b77 1b7c
1b80 1b84 1b87 1b8c 1b91 1b94 1b98 1b99
1b9e 1b9f 1ba4 1ba8 1bac 1baf 1bb4 1bb9
1bbc 1bc0 1bc4 1bc7 1bc8 1bcd 1bd0 1bd5
1bd6 1bdb 1bde 1be2 1be6 1be9 1bea 1bef
1bf0 1bf5 1bf7 1bfb 1bfe 1c02 1c06 1c0a
1c0e 1c12 1c16 1c1a 1c1e 1c22 1c2e 1c33
1c37 1c3b 1c3f 1c43 1c45 1c46 1c4b 1
1c4f 1c53 1c57 1c5a 1c5e 1c62 1c66 1c6a
1c6d 1c71 1c75 1c78 1c7b 1c80 1c83 1c86
1c89 1c8a 1c8f 1c90 1 1c92 1c97 1c9b
1c9f 1ca2 1ca7 1cac 1caf 1cb3 1cb4 1cb9
1cba 1cbf 1cc3 1cc7 1cca 1ccf 1cd4 1cd7
1cdb 1cdc 1ce1 1ce4 1ce9 1cea 1cef 1cf2
1cf6 1cfa 1cfd 1d00 1d01 1d03 1d04 1d09
1d0a 1d0f 1d11 1d15 1d18 1d1c 1d20 1d24
1d28 1d2c 1d30 1d34 1d40 1d45 1d49 1d4d
1d51 1d55 1d57 1d5b 1d5d 1d69 1d6b 1d6d
1d6e 1d73 1d77 1d79 1d85 1d89 1d8b 1d8c
1d95 
861
2
0 1 a 6 9 :3 6 11 :3 6
12 :2 6 1 3 a 13 a :2 1
4 18 21 20 :2 18 :2 4 11 1a
19 :2 11 :2 4 e 17 16 :2 e :2 4
:3 11 :2 4 f 18 17 :2 f :2 4 :2 e
1d e :2 4 :2 14 23 14 :2 4 :2 12
21 12 :2 4 :2 11 20 11 :2 4 :2 12
21 12 :2 4 :2 13 22 13 :2 4 :3 17
:2 4 e 17 :3 e 4 7 :2 11 23
:2 2d 3a :2 23 :3 7 :2 d 19 25 :2 2f
43 :2 25 5a 60 63 :2 5a 71 8d
8 :2 71 d 10 :2 71 13 16 :2 71
b e :2 71 13 16 1f 27 :2 1f
34 :2 16 :2 71 b e :2 71 13 16
:2 71 1b 1e :2 71 2a 2d :2 71 32
35 :2 71 :2 7 50 :2 4 7 :2 11 23
:2 2d 39 3c 52 54 55 :2 52 :2 23
:3 7 :2 d 12 1e 63 66 :2 1e :2 7
58 :3 4 11 :2 4 1b 4 e 1a
23 2d 38 46 53 c 1a 25
31 3f 4e c 23 37 :2 7 :2 10
1c 7 4 c a :2 13 1f a
1a :2 7 :2 c a :2 13 1f a d
:2 17 29 :2 33 3f 43 59 5b 5c
:2 59 :2 29 :3 d :2 13 18 24 69 6c
:2 24 79 7c :2 24 80 83 :2 24 8f
92 :2 24 9a 9d :2 24 a9 ac b3
bc bf :2 ac :2 24 :2 d 5f :2 a :2 7
:4 13 :2 7 4 :3 1 7 10 1b :2 7
1e 20 :2 1e a 11 1d 20 :2 a
23 25 :2 23 14 29 2f 3c 13
27 d 14 10 1c :3 1a 10 :2 19
25 10 21 10 :2 19 25 10 :4 d
a 12 10 :2 19 25 10 20 :2 d
:2 12 10 :2 19 25 10 13 :2 1d 2f
:2 39 45 49 5f 61 62 :2 5f :2 2f
:3 13 :2 19 1e 2a 6f 72 :2 2a 7f
82 :2 2a 86 89 :2 2a 95 98 :2 2a
a0 a3 :2 2a af b2 b9 c2 c5
:2 b2 :2 2a :2 13 65 :2 10 :2 d :4 19 :2 d
a :3 29 14 11 13 27 11 26
11 1b 24 d 14 10 1c :3 1a
10 :2 19 25 10 21 10 :2 19 25
10 :4 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 5f 61 62
:2 5f :2 2f :3 13 :2 19 1e 2a 6f 72
:2 2a 7f 82 :2 2a 86 89 :2 2a 95
98 :2 2a a0 a3 :2 2a af b2 b9
c2 c5 :2 b2 :2 2a :2 13 65 :2 10 :2 d
:4 19 :2 d a :4 29 14 29 2f 3d
4e 13 27 d 14 10 1c :3 1a
10 :2 19 25 10 21 10 :2 19 25
10 :4 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 5f 62 63
:2 5f :2 2f :3 13 :2 19 1e 2a 70 73
:2 2a 80 83 :2 2a 87 8a :2 2a 96
99 :2 2a a1 a4 :2 2a b0 b3 ba
c3 c6 :2 b3 :2 2a :2 13 66 :2 10 :2 d
:4 19 :2 d a :7 7 11 1d 2b 34
3b 48 51 65 72 a 11 d
19 :3 17 d :2 16 22 d 1e d
:2 16 22 d :4 a 7 f d :2 16
22 d 1d :2 a :2 f d :2 16 22
d 10 :2 1a 2c :2 36 42 46 5c
5f 60 :2 5c :2 2c :3 10 :2 16 1b 27
6d 70 :2 27 7d 80 :2 27 84 87
:2 27 93 96 :2 27 9e a1 :2 27 ad
b0 b7 c0 c3 :2 b0 :2 27 :2 10 63
:2 d :2 a :4 16 :2 a 7 :4 22 a 13
1e :2 a 21 23 :2 21 14 29 40
58 13 27 d 14 10 1c :3 1a
10 :2 19 25 10 21 10 :2 19 25
10 :4 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 5f 62 63
:2 5f :2 2f :3 13 :2 19 1e 2a 70 73
:2 2a 80 83 :2 2a 87 8a :2 2a 96
99 :2 2a a1 a4 :2 2a b0 b3 ba
c3 c6 :2 b3 :2 2a :2 13 66 :2 10 :2 d
:4 19 :2 d a :3 25 14 20 2e 37
3e 4b 54 68 75 d 14 10
1c :3 1a 10 :2 19 25 10 21 10
:2 19 25 10 :4 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 5f
62 63 :2 5f :2 2f :3 13 :2 19 1e 2a
70 73 :2 2a 80 83 :2 2a 87 8a
:2 2a 96 99 :2 2a a1 a4 :2 2a b0
b3 ba c3 c6 :2 b3 :2 2a :2 13 66
:2 10 :2 d :4 19 :2 d a :4 25 d 14
20 23 :2 d 26 28 :2 26 17 2c
43 5b 10 16 23 16 2a 10
17 13 1f :3 1d 13 :2 1c 28 13
24 13 :2 1c 28 13 :4 10 d 15
13 :2 1c 28 13 23 :2 10 :2 15 13
:2 1c 28 13 16 :2 20 32 :2 3c 48
4c 62 65 66 :2 62 :2 32 :3 16 :2 1c
21 2d 73 76 :2 2d 83 86 :2 2d
8a 8d :2 2d 99 9c :2 2d a4 a7
:2 2d b3 b6 bd c6 c9 :2 b6 :2 2d
:2 16 69 :2 13 :2 10 :4 1c :2 10 d :3 2c
17 14 16 2a 14 29 14 1e
27 10 17 13 1f :3 1d 13 :2 1c
28 13 24 13 :2 1c 28 13 :4 10
d 15 13 :2 1c 28 13 23 :2 10
:2 15 13 :2 1c 28 13 16 :2 20 32
:2 3c 48 4c 62 65 66 :2 62 :2 32
:3 16 :2 1c 21 2d 73 76 :2 2d 83
86 :2 2d 8a 8d :2 2d 99 9c :2 2d
a4 a7 :2 2d b3 b6 bd c6 c9
:2 b6 :2 2d :2 16 69 :2 13 :2 10 :4 1c :2 10
d :4 2c 17 2c 43 5b 10 16
24 35 16 2a 10 17 13 1f
:3 1d 13 :2 1c 28 13 24 13 :2 1c
28 13 :4 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 62 65
66 :2 62 :2 32 :3 16 :2 1c 21 2d 73
76 :2 2d 83 86 :2 2d 8a 8d :2 2d
99 9c :2 2d a4 a7 :2 2d b3 b6
bd c6 c9 :2 b6 :2 2d :2 16 69 :2 13
:2 10 :4 1c :2 10 d :7 a 14 20 2e
37 3e 4b 54 68 75 d 14
10 1c :3 1a 10 :2 19 25 10 21
10 :2 19 25 10 :4 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
5f 62 63 :2 5f :2 2f :3 13 :2 19 1e
2a 70 73 :2 2a 80 83 :2 2a 87
8a :2 2a 96 99 :2 2a a1 a4 :2 2a
b0 b3 ba c3 c6 :2 b3 :2 2a :2 13
66 :2 10 :2 d :4 19 :2 d a :7 7 :4 4
9 10 19 24 :2 10 28 :2 9 8
2c 2e :2 2c 7 36 3f 4f :2 36
52 54 :2 52 35 :3 7 1d 25 2e
38 :2 2e 48 :2 25 4b 4d 28 32
:2 28 41 :2 4d :2 25 24 45 47 :2 24
23 51 53 28 38 :2 53 :2 23 :2 1d
7 b 1e 20 :2 1e a 26 39
3b 3c :2 3b :2 39 :2 a 14 d 1b
24 2b 37 40 d 12 2c 39
12 13 27 d 14 10 1c :3 1a
10 :2 19 25 10 21 10 :2 19 25
10 :4 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 5f 62 63
:2 5f :2 2f :3 13 :2 19 1e 2a 70 73
:2 2a 80 83 :2 2a 87 8a :2 2a 96
99 :2 2a a1 a4 :2 2a b0 b3 ba
c3 c6 :2 b3 :2 2a :2 13 66 :2 10 :2 d
:4 19 :2 d a :4 3e :2 7 57 11 a
f 29 36 f 10 24 a 11
d 19 :3 17 d :2 16 22 d 1e
d :2 16 22 d :4 a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
5c 5f 60 :2 5c :2 2c :3 10 :2 16 1b
27 6d 70 :2 27 7d 80 :2 27 84
87 :2 27 93 96 :2 27 9e a1 :2 27
ad b0 b7 c0 c3 :2 b0 :2 27 :2 10
63 :2 d :2 a :4 16 :2 a 7 :7 4 8
:2 11 1c 1a :2 25 :2 1a 7 3a 43
4e :2 3a 51 54 :2 51 39 :3 7 1a
2c 3a :2 1a 7 57 :3 4 17 21
:2 17 4 7 :2 11 23 :2 2d 39 3c
52 55 56 :2 52 :2 23 :3 7 :2 d 12
1e 64 67 :2 1e 74 77 :2 1e 7b
7e :2 1e 88 8b 94 9c :2 a5 :2 94
b0 :2 8b :2 1e b8 bb :2 1e c3 c6
:2 1e cb ce :2 1e d1 d4 :2 1e :2 7
59 :3 4 :2 f 21 :2 2a 34 38 :2 4
1 9 12 9 a :2 14 26 :2 30
3c 40 56 59 5a :2 56 :2 26 :3 a
:2 10 15 21 67 6a :2 21 :3 a :2 10
15 39 5c 5f :2 68 :2 39 76 79
:2 39 81 84 :2 8d :2 39 :2 a 5d :3 7
c 15 1e 2c a 13 1e f
e 7 e 7 e 7 29 :2 4
:2 9 7 :2 10 1c :2 7 :2 11 23 :2 2d
39 3d 53 56 57 :2 53 :2 23 :3 7
:2 d 12 1e 64 67 :2 1e :3 7 :2 d
12 36 42 45 :2 36 4d 50 :2 36
5c 5f 66 6f 72 :2 5f :2 36 :2 7
5a :2 4 7 c 1b a 13 1e
f e 7 e 7 e 7 :2 4
:4 10 :2 4 :8 1 
861
4
0 :2 1 :4 3 :4 4
:4 5 2 :4 6 :2 1
:7 8 :7 9 :7 a :5 b
:7 c :6 d :6 e :6 f
:6 10 :6 11 :6 12 :5 13
:7 15 :b 17 :11 18 19
:2 18 :2 19 :2 18 :2 19
:2 18 :2 1a :2 18 :9 1a
:2 18 :2 1b :2 18 :2 1b
:2 18 :2 1b :2 18 :2 1b
:2 18 :2 1b :4 18 :3 17
:11 1e :b 1f :3 1e :3 23
:3 24 :7 27 :6 28 :3 29
27 :5 2a 26 2c
:5 2d :3 2c :2 2e :5 30
:11 31 :24 32 :3 31 :2 2f
:6 2e 2b :3 16 :9 37
:a 38 :4 3a :2 3b 3a
:6 3c :5 3d 3c :5 3f
3e :3 3c 39 42
:5 43 :3 42 :2 44 :5 46
:11 47 :24 48 :3 47 :2 45
:6 44 41 :3 38 4e
4f :2 50 :2 51 :3 52
4e :6 53 :5 54 53
:5 56 55 :3 53 4d
59 :5 5a :3 59 :2 5b
:5 5d :11 5e :24 5f :3 5e
:2 5c :6 5b 58 :4 38
:5 65 :2 66 65 :6 67
:5 68 67 :5 6a 69
:3 67 64 6d :5 6e
:3 6d :2 6f :5 71 :11 72
:24 73 :3 72 :2 70 :6 6f
6c :4 63 :3 38 :a 7a
:6 7b :5 7c 7b :5 7e
7d :3 7b 79 81
:5 82 :3 81 :2 83 :5 85
:11 86 :24 87 :3 86 :2 84
:6 83 80 :4 37 :9 8c
:4 8e :2 8f 8e :6 90
:5 91 90 :5 93 92
:3 90 8d 96 :5 97
:3 96 :2 98 :5 9a :11 9b
:24 9c :3 9b :2 99 :6 98
95 :3 8c :a a2 :6 a3
:5 a4 a3 :5 a6 a5
:3 a3 a1 a9 :5 aa
:3 a9 :2 ab :5 ad :11 ae
:24 af :3 ae :2 ac :6 ab
a8 :4 8c :a b4 :4 b6
:3 b7 :2 b8 b6 :6 b9
:5 ba b9 :5 bc bb
:3 b9 b5 bf :5 c0
:3 bf :2 c1 :5 c3 :11 c4
:24 c5 :3 c4 :2 c2 :6 c1
be :3 b4 cb cc
:2 cd :2 ce :3 cf cb
:6 d0 :5 d1 d0 :5 d3
d2 :3 d0 ca d6
:5 d7 :3 d6 :2 d8 :5 da
:11 db :24 dc :3 db :2 d9
:6 d8 d5 :4 b4 :4 e2
:4 e3 :2 e4 e2 :6 e5
:5 e6 e5 :5 e8 e7
:3 e5 e1 eb :5 ec
:3 eb :2 ed :5 ef :11 f0
:24 f1 :3 f0 :2 ee :6 ed
ea :4 e0 :3 b4 :a f8
:6 f9 :5 fa f9 :5 fc
fb :3 f9 f7 ff
:5 100 :3 ff :2 101 :5 103
:11 104 :24 105 :3 104 :2 102
:6 101 fe :4 b3 :3 8c
8b :3 37 :1b 10c :c 10d
:5 10e :5 10d :2 10e :3 10d
:2 10e :2 10f :2 10e :5 10d
:10 110 112 :6 113 114
:3 115 116 :2 118 112
:6 119 :5 11a 119 :5 11c
11b :3 119 111 11f
:5 120 :3 11f :2 121 :5 123
:11 124 :24 125 :3 124 :2 122
:6 121 11e :6 110 10c
12c 12d :3 12e 12f
:2 131 12c :6 132 :5 133
132 :5 135 134 :3 132
12b 138 :5 139 :3 138
:2 13a :5 13c :11 13d :24 13e
:3 13d :2 13b :6 13a 137
:4 12a :3 10c :16 144 :7 146
:3 144 :6 149 :11 14c :30 14d
:3 14c :a 14f 16 :3 152
:11 153 :b 154 :17 155 :3 153
:2 157 :3 158 :3 159 15a
158 157 158 :3 15b
:3 152 :2 15c :5 15e :11 15f
:b 160 :18 161 :3 15f :2 163
164 :3 165 166 164
163 164 :3 167 :2 15d
:6 15c 151 :7 1 
1d97
4
:3 0 1 :3 0 2
:a 0 85c 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
:2 0 7 6 :3 0
5 :7 0 9 8
:3 0 4 :3 0 7
:7 0 d c :3 0
8 :3 0 9 :3 0
a :2 0 4 11
12 0 f 13
0 85c 2 14
:2 0 c :2 0 f
4 :3 0 c :2 0
d 17 19 :6 0
1c 1a 0 85a
0 b :6 0 c
:2 0 13 4 :3 0
11 1e 20 :6 0
23 21 0 85a
0 d :6 0 19
c7 0 17 4
:3 0 15 25 27
:6 0 2a 28 0
85a 0 e :6 0
14 :2 0 1d 10
:3 0 2c :7 0 2f
2d 0 85a 0
f :6 0 4 :3 0
12 :2 0 1b 31
33 :6 0 36 34
0 85a 0 11
:6 0 14 :2 0 1f
6 :3 0 38 :7 0
3c 39 3a 85a
0 13 :6 0 14
:2 0 21 6 :3 0
3e :7 0 42 3f
40 85a 0 15
:6 0 14 :2 0 23
6 :3 0 44 :7 0
48 45 46 85a
0 16 :6 0 14
:2 0 25 6 :3 0
4a :7 0 4e 4b
4c 85a 0 17
:6 0 14 :2 0 27
6 :3 0 50 :7 0
54 51 52 85a
0 18 :6 0 2b
1a7 0 29 6
:3 0 56 :7 0 5a
57 58 85a 0
19 :6 0 67 68
0 2d 6 :3 0
5c :7 0 5f 5d
0 85a 0 1a
:6 0 9 :3 0 a
:2 0 4 61 62
0 63 :7 0 66
64 0 85a 0
1b :6 0 1c :3 0
1d :3 0 1c :3 0
1e :3 0 6a 6b
0 2 :4 0 2f
6c 6e 69 70
6f :2 0 1f :3 0
20 :3 0 72 73
0 21 :4 0 1c
:3 0 22 :3 0 76
77 0 2 :4 0
31 78 7a 23
:4 0 24 :2 0 25
:3 0 33 7d 7f
:3 0 26 :4 0 24
:2 0 27 :4 0 36
82 84 :3 0 24
:2 0 3 :3 0 39
86 88 :3 0 24
:2 0 27 :4 0 3c
8a 8c :3 0 24
:2 0 28 :4 0 3f
8e 90 :3 0 24
:2 0 29 :3 0 2a
:3 0 5 :3 0 42
94 96 2b :4 0
44 93 99 47
92 9b :3 0 24
:2 0 28 :4 0 4a
9d 9f :3 0 24
:2 0 27 :4 0 4d
a1 a3 :3 0 24
:2 0 7 :3 0 50
a5 a7 :3 0 24
:2 0 27 :4 0 53
a9 ab :3 0 24
:2 0 2c :4 0 56
ad af :3 0 59
74 b1 :2 0 b3
5e b4 71 b3
0 b5 60 0
7ba 1c :3 0 1d
:3 0 b6 b7 0
1c :3 0 2d :3 0
b9 ba 0 12
:2 0 2 :4 0 2e
:2 0 2f :2 0 12
:2 0 62 bf c1
:3 0 65 bb c3
b8 c5 c4 :2 0
1f :3 0 30 :3 0
c7 c8 0 21
:4 0 31 :4 0 24
:2 0 25 :3 0 69
cc ce :3 0 6c
c9 d0 :2 0 d2
6f d3 c6 d2
0 d4 71 0
7ba 13 :3 0 5
:3 0 d5 d6 0
7ba b :3 0 3
:3 0 d8 d9 0
7ba 32 :3 0 33
:3 0 34 :3 0 35
:3 0 36 :3 0 37
:3 0 38 :3 0 d
:3 0 e :3 0 11
:3 0 17 :3 0 18
:3 0 19 :3 0 39
:3 0 3a :3 0 b
:4 0 3b 1 :8 0
f1 9 :3 0 3c
:3 0 ec ed 0
3d :3 0 ee ef
0 f1 73 144
3e :3 0 9 :3 0
3c :3 0 f3 f4
0 3d :3 0 f5
f6 0 f8 76
fa 78 f9 f8
:2 0 142 3f :3 0
9 :3 0 3c :3 0
fd fe 0 3d
:3 0 ff 100 0
13a 1c :3 0 1d
:3 0 102 103 0
1c :3 0 2d :3 0
105 106 0 40
:2 0 2 :4 0 41
:2 0 2f :2 0 12
:2 0 7a 10b 10d
:3 0 7d 107 10f
104 111 110 :2 0
1f :3 0 30 :3 0
113 114 0 21
:4 0 42 :4 0 24
:2 0 25 :3 0 81
118 11a :3 0 24
:2 0 43 :4 0 84
11c 11e :3 0 24
:2 0 44 :4 0 87
120 122 :3 0 24
:2 0 3d :3 0 8a
124 126 :3 0 24
:2 0 45 :4 0 8d
128 12a :3 0 24
:2 0 46 :3 0 47
:3 0 12 :2 0 48
:2 0 90 12d 131
94 12c 133 :3 0
97 115 135 :2 0
137 9a 138 112
137 0 139 9c
0 13a 9e 13d
:3 0 13d 0 13d
13c 13a 13b :6 0
13f 2 :3 0 a1
141 a3 140 13f
:2 0 142 a5 :2 0
144 0 144 143
f1 142 :6 0 7ba
1 :3 0 29 :3 0
13 :3 0 14 :2 0
a8 146 149 49
:2 0 12 :2 0 ad
14b 14d :3 0 46
:3 0 11 :3 0 12
:2 0 12 :2 0 b0
14f 153 49 :2 0
4a :4 0 b6 155
157 :3 0 39 :3 0
4b :3 0 35 :3 0
37 :3 0 3a :3 0
b :4 0 4c 1
:8 0 175 4d :4 0
160 :3 0 4e :3 0
49 :2 0 bb 163
164 :3 0 9 :3 0
3c :3 0 166 167
0 14 :2 0 168
169 0 16b be
172 9 :3 0 3c
:3 0 16c 16d 0
4f :2 0 16e 16f
0 171 c0 173
165 16b 0 174
0 171 0 174
c2 0 175 c5
1c8 3e :3 0 9
:3 0 3c :3 0 177
178 0 3d :3 0
179 17a 0 17c
c8 17e ca 17d
17c :2 0 1c6 3f
:3 0 9 :3 0 3c
:3 0 181 182 0
3d :3 0 183 184
0 1be 1c :3 0
1d :3 0 186 187
0 1c :3 0 2d
:3 0 189 18a 0
40 :2 0 2 :4 0
50 :2 0 2f :2 0
12 :2 0 cc 18f
191 :3 0 cf 18b
193 188 195 194
:2 0 1f :3 0 30
:3 0 197 198 0
21 :4 0 51 :4 0
24 :2 0 25 :3 0
d3 19c 19e :3 0
24 :2 0 43 :4 0
d6 1a0 1a2 :3 0
24 :2 0 44 :4 0
d9 1a4 1a6 :3 0
24 :2 0 3d :3 0
dc 1a8 1aa :3 0
24 :2 0 45 :4 0
df 1ac 1ae :3 0
24 :2 0 46 :3 0
47 :3 0 12 :2 0
48 :2 0 e2 1b1
1b5 e6 1b0 1b7
:3 0 e9 199 1b9
:2 0 1bb ec 1bc
196 1bb 0 1bd
ee 0 1be f0
1c1 :3 0 1c1 0
1c1 1c0 1be 1bf
:6 0 1c3 4 :3 0
f3 1c5 f5 1c4
1c3 :2 0 1c6 f7
:2 0 1c8 0 1c8
1c7 175 1c6 :6 0
23e 1 :3 0 52
:3 0 53 :3 0 3a
:3 0 b :3 0 3a
:3 0 54 :3 0 55
:3 0 56 :3 0 57
:4 0 58 1 :8 0
1e9 4d :4 0 1d4
:3 0 4e :3 0 49
:2 0 fc 1d7 1d8
:3 0 9 :3 0 3c
:3 0 1da 1db 0
14 :2 0 1dc 1dd
0 1df ff 1e6
9 :3 0 3c :3 0
1e0 1e1 0 4f
:2 0 1e2 1e3 0
1e5 101 1e7 1d9
1df 0 1e8 0
1e5 0 1e8 103
0 1e9 106 23c
3e :3 0 9 :3 0
3c :3 0 1eb 1ec
0 3d :3 0 1ed
1ee 0 1f0 109
1f2 10b 1f1 1f0
:2 0 23a 3f :3 0
9 :3 0 3c :3 0
1f5 1f6 0 3d
:3 0 1f7 1f8 0
232 1c :3 0 1d
:3 0 1fa 1fb 0
1c :3 0 2d :3 0
1fd 1fe 0 40
:2 0 2 :4 0 59
:2 0 2f :2 0 12
:2 0 10d 203 205
:3 0 110 1ff 207
1fc 209 208 :2 0
1f :3 0 30 :3 0
20b 20c 0 21
:4 0 5a :4 0 24
:2 0 25 :3 0 114
210 212 :3 0 24
:2 0 43 :4 0 117
214 216 :3 0 24
:2 0 44 :4 0 11a
218 21a :3 0 24
:2 0 3d :3 0 11d
21c 21e :3 0 24
:2 0 45 :4 0 120
220 222 :3 0 24
:2 0 46 :3 0 47
:3 0 12 :2 0 48
:2 0 123 225 229
127 224 22b :3 0
12a 20d 22d :2 0
22f 12d 230 20a
22f 0 231 12f
0 232 131 235
:3 0 235 0 235
234 232 233 :6 0
237 6 :3 0 134
239 136 238 237
:2 0 23a 138 :2 0
23c 0 23c 23b
1e9 23a :6 0 23e
1 :3 0 13b 2b2
39 :3 0 4b :3 0
36 :3 0 37 :3 0
38 :3 0 3a :3 0
b :4 0 5b 1
:8 0 25c 4d :4 0
247 :3 0 4e :3 0
49 :2 0 140 24a
24b :3 0 9 :3 0
3c :3 0 24d 24e
0 14 :2 0 24f
250 0 252 143
259 9 :3 0 3c
:3 0 253 254 0
4f :2 0 255 256
0 258 145 25a
24c 252 0 25b
0 258 0 25b
147 0 25c 14a
2af 3e :3 0 9
:3 0 3c :3 0 25e
25f 0 3d :3 0
260 261 0 263
14d 265 14f 264
263 :2 0 2ad 3f
:3 0 9 :3 0 3c
:3 0 268 269 0
3d :3 0 26a 26b
0 2a5 1c :3 0
1d :3 0 26d 26e
0 1c :3 0 2d
:3 0 270 271 0
40 :2 0 2 :4 0
5c :2 0 2f :2 0
12 :2 0 151 276
278 :3 0 154 272
27a 26f 27c 27b
:2 0 1f :3 0 30
:3 0 27e 27f 0
21 :4 0 5d :4 0
24 :2 0 25 :3 0
158 283 285 :3 0
24 :2 0 43 :4 0
15b 287 289 :3 0
24 :2 0 44 :4 0
15e 28b 28d :3 0
24 :2 0 3d :3 0
161 28f 291 :3 0
24 :2 0 45 :4 0
164 293 295 :3 0
24 :2 0 46 :3 0
47 :3 0 12 :2 0
48 :2 0 167 298
29c 16b 297 29e
:3 0 16e 280 2a0
:2 0 2a2 171 2a3
27d 2a2 0 2a4
173 0 2a5 175
2a8 :3 0 2a8 0
2a8 2a7 2a5 2a6
:6 0 2aa 8 :3 0
178 2ac 17a 2ab
2aa :2 0 2ad 17c
:2 0 2af 0 2af
2ae 25c 2ad :6 0
2b1 1 :3 0 17f
2b3 158 23e 0
2b4 0 2b1 0
2b4 181 0 329
5e :3 0 5f :3 0
60 :3 0 61 :3 0
5f :3 0 56 :3 0
62 :3 0 54 :3 0
e :4 0 63 1
:8 0 2d4 4d :4 0
2bf :3 0 4e :3 0
49 :2 0 186 2c2
2c3 :3 0 9 :3 0
3c :3 0 2c5 2c6
0 14 :2 0 2c7
2c8 0 2ca 189
2d1 9 :3 0 3c
:3 0 2cb 2cc 0
4f :2 0 2cd 2ce
0 2d0 18b 2d2
2c4 2ca 0 2d3
0 2d0 0 2d3
18d 0 2d4 190
327 3e :3 0 9
:3 0 3c :3 0 2d6
2d7 0 3d :3 0
2d8 2d9 0 2db
193 2dd 195 2dc
2db :2 0 325 3f
:3 0 9 :3 0 3c
:3 0 2e0 2e1 0
3d :3 0 2e2 2e3
0 31d 1c :3 0
1d :3 0 2e5 2e6
0 1c :3 0 2d
:3 0 2e8 2e9 0
40 :2 0 2 :4 0
64 :2 0 2f :2 0
12 :2 0 197 2ee
2f0 :3 0 19a 2ea
2f2 2e7 2f4 2f3
:2 0 1f :3 0 30
:3 0 2f6 2f7 0
21 :4 0 65 :4 0
24 :2 0 25 :3 0
19e 2fb 2fd :3 0
24 :2 0 43 :4 0
1a1 2ff 301 :3 0
24 :2 0 44 :4 0
1a4 303 305 :3 0
24 :2 0 3d :3 0
1a7 307 309 :3 0
24 :2 0 45 :4 0
1aa 30b 30d :3 0
24 :2 0 46 :3 0
47 :3 0 12 :2 0
48 :2 0 1ad 310
314 1b1 30f 316
:3 0 1b4 2f8 318
:2 0 31a 1b7 31b
2f5 31a 0 31c
1b9 0 31d 1bb
320 :3 0 320 0
320 31f 31d 31e
:6 0 322 a :3 0
1be 324 1c0 323
322 :2 0 325 1c2
:2 0 327 0 327
326 2d4 325 :6 0
329 1 :3 0 1c5
5fe 29 :3 0 13
:3 0 14 :2 0 1c8
32a 32d 49 :2 0
4a :2 0 1cd 32f
331 :3 0 39 :3 0
66 :3 0 67 :3 0
68 :3 0 3a :3 0
b :4 0 69 1
:8 0 34f 4d :4 0
33a :3 0 4e :3 0
49 :2 0 1d2 33d
33e :3 0 9 :3 0
3c :3 0 340 341
0 14 :2 0 342
343 0 345 1d5
34c 9 :3 0 3c
:3 0 346 347 0
4f :2 0 348 349
0 34b 1d7 34d
33f 345 0 34e
0 34b 0 34e
1d9 0 34f 1dc
3a2 3e :3 0 9
:3 0 3c :3 0 351
352 0 3d :3 0
353 354 0 356
1df 358 1e1 357
356 :2 0 3a0 3f
:3 0 9 :3 0 3c
:3 0 35b 35c 0
3d :3 0 35d 35e
0 398 1c :3 0
1d :3 0 360 361
0 1c :3 0 2d
:3 0 363 364 0
40 :2 0 2 :4 0
6a :2 0 2f :2 0
12 :2 0 1e3 369
36b :3 0 1e6 365
36d 362 36f 36e
:2 0 1f :3 0 30
:3 0 371 372 0
21 :4 0 6b :4 0
24 :2 0 25 :3 0
1ea 376 378 :3 0
24 :2 0 43 :4 0
1ed 37a 37c :3 0
24 :2 0 44 :4 0
1f0 37e 380 :3 0
24 :2 0 3d :3 0
1f3 382 384 :3 0
24 :2 0 45 :4 0
1f6 386 388 :3 0
24 :2 0 46 :3 0
47 :3 0 12 :2 0
48 :2 0 1f9 38b
38f 1fd 38a 391
:3 0 200 373 393
:2 0 395 203 396
370 395 0 397
205 0 398 207
39b :3 0 39b 0
39b 39a 398 399
:6 0 39d c :3 0
20a 39f 20c 39e
39d :2 0 3a0 20e
:2 0 3a2 0 3a2
3a1 34f 3a0 :6 0
418 1 :3 0 5e
:3 0 5f :3 0 60
:3 0 61 :3 0 5f
:3 0 56 :3 0 62
:3 0 54 :3 0 e
:4 0 63 1 :8 0
3c3 4d :4 0 3ae
:3 0 4e :3 0 49
:2 0 213 3b1 3b2
:3 0 9 :3 0 3c
:3 0 3b4 3b5 0
14 :2 0 3b6 3b7
0 3b9 216 3c0
9 :3 0 3c :3 0
3ba 3bb 0 4f
:2 0 3bc 3bd 0
3bf 218 3c1 3b3
3b9 0 3c2 0
3bf 0 3c2 21a
0 3c3 21d 416
3e :3 0 9 :3 0
3c :3 0 3c5 3c6
0 3d :3 0 3c7
3c8 0 3ca 220
3cc 222 3cb 3ca
:2 0 414 3f :3 0
9 :3 0 3c :3 0
3cf 3d0 0 3d
:3 0 3d1 3d2 0
40c 1c :3 0 1d
:3 0 3d4 3d5 0
1c :3 0 2d :3 0
3d7 3d8 0 40
:2 0 2 :4 0 6c
:2 0 2f :2 0 12
:2 0 224 3dd 3df
:3 0 227 3d9 3e1
3d6 3e3 3e2 :2 0
1f :3 0 30 :3 0
3e5 3e6 0 21
:4 0 6d :4 0 24
:2 0 25 :3 0 22b
3ea 3ec :3 0 24
:2 0 43 :4 0 22e
3ee 3f0 :3 0 24
:2 0 44 :4 0 231
3f2 3f4 :3 0 24
:2 0 3d :3 0 234
3f6 3f8 :3 0 24
:2 0 45 :4 0 237
3fa 3fc :3 0 24
:2 0 46 :3 0 47
:3 0 12 :2 0 48
:2 0 23a 3ff 403
23e 3fe 405 :3 0
241 3e7 407 :2 0
409 244 40a 3e4
409 0 40b 246
0 40c 248 40f
:3 0 40f 0 40f
40e 40c 40d :6 0
411 e :3 0 24b
413 24d 412 411
:2 0 414 24f :2 0
416 0 416 415
3c3 414 :6 0 418
1 :3 0 252 5fa
46 :3 0 11 :3 0
12 :2 0 12 :2 0
255 419 41d 49
:2 0 4a :4 0 25b
41f 421 :3 0 39
:3 0 66 :3 0 67
:3 0 68 :3 0 4b
:3 0 35 :3 0 37
:3 0 3a :3 0 b
:4 0 6e 1 :8 0
442 4d :4 0 42d
:3 0 4e :3 0 49
:2 0 260 430 431
:3 0 9 :3 0 3c
:3 0 433 434 0
14 :2 0 435 436
0 438 263 43f
9 :3 0 3c :3 0
439 43a 0 4f
:2 0 43b 43c 0
43e 265 440 432
438 0 441 0
43e 0 441 267
0 442 26a 495
3e :3 0 9 :3 0
3c :3 0 444 445
0 3d :3 0 446
447 0 449 26d
44b 26f 44a 449
:2 0 493 3f :3 0
9 :3 0 3c :3 0
44e 44f 0 3d
:3 0 450 451 0
48b 1c :3 0 1d
:3 0 453 454 0
1c :3 0 2d :3 0
456 457 0 40
:2 0 2 :4 0 6f
:2 0 2f :2 0 12
:2 0 271 45c 45e
:3 0 274 458 460
455 462 461 :2 0
1f :3 0 30 :3 0
464 465 0 21
:4 0 70 :4 0 24
:2 0 25 :3 0 278
469 46b :3 0 24
:2 0 43 :4 0 27b
46d 46f :3 0 24
:2 0 44 :4 0 27e
471 473 :3 0 24
:2 0 3d :3 0 281
475 477 :3 0 24
:2 0 45 :4 0 284
479 47b :3 0 24
:2 0 46 :3 0 47
:3 0 12 :2 0 48
:2 0 287 47e 482
28b 47d 484 :3 0
28e 466 486 :2 0
488 291 489 463
488 0 48a 293
0 48b 295 48e
:3 0 48e 0 48e
48d 48b 48c :6 0
490 10 :3 0 298
492 29a 491 490
:2 0 493 29c :2 0
495 0 495 494
442 493 :6 0 50b
1 :3 0 52 :3 0
53 :3 0 3a :3 0
b :3 0 3a :3 0
54 :3 0 55 :3 0
56 :3 0 57 :4 0
71 1 :8 0 4b6
4d :4 0 4a1 :3 0
4e :3 0 49 :2 0
2a1 4a4 4a5 :3 0
9 :3 0 3c :3 0
4a7 4a8 0 14
:2 0 4a9 4aa 0
4ac 2a4 4b3 9
:3 0 3c :3 0 4ad
4ae 0 4f :2 0
4af 4b0 0 4b2
2a6 4b4 4a6 4ac
0 4b5 0 4b2
0 4b5 2a8 0
4b6 2ab 509 3e
:3 0 9 :3 0 3c
:3 0 4b8 4b9 0
3d :3 0 4ba 4bb
0 4bd 2ae 4bf
2b0 4be 4bd :2 0
507 3f :3 0 9
:3 0 3c :3 0 4c2
4c3 0 3d :3 0
4c4 4c5 0 4ff
1c :3 0 1d :3 0
4c7 4c8 0 1c
:3 0 2d :3 0 4ca
4cb 0 40 :2 0
2 :4 0 72 :2 0
2f :2 0 12 :2 0
2b2 4d0 4d2 :3 0
2b5 4cc 4d4 4c9
4d6 4d5 :2 0 1f
:3 0 30 :3 0 4d8
4d9 0 21 :4 0
73 :4 0 24 :2 0
25 :3 0 2b9 4dd
4df :3 0 24 :2 0
43 :4 0 2bc 4e1
4e3 :3 0 24 :2 0
44 :4 0 2bf 4e5
4e7 :3 0 24 :2 0
3d :3 0 2c2 4e9
4eb :3 0 24 :2 0
45 :4 0 2c5 4ed
4ef :3 0 24 :2 0
46 :3 0 47 :3 0
12 :2 0 48 :2 0
2c8 4f2 4f6 2cc
4f1 4f8 :3 0 2cf
4da 4fa :2 0 4fc
2d2 4fd 4d7 4fc
0 4fe 2d4 0
4ff 2d6 502 :3 0
502 0 502 501
4ff 500 :6 0 504
12 :3 0 2d9 506
2db 505 504 :2 0
507 2dd :2 0 509
0 509 508 4b6
507 :6 0 50b 1
:3 0 2e0 582 39
:3 0 66 :3 0 67
:3 0 68 :3 0 4b
:3 0 36 :3 0 37
:3 0 38 :3 0 3a
:3 0 b :4 0 74
1 :8 0 52c 4d
:4 0 517 :3 0 4e
:3 0 49 :2 0 2e5
51a 51b :3 0 9
:3 0 3c :3 0 51d
51e 0 14 :2 0
51f 520 0 522
2e8 529 9 :3 0
3c :3 0 523 524
0 4f :2 0 525
526 0 528 2ea
52a 51c 522 0
52b 0 528 0
52b 2ec 0 52c
2ef 57f 3e :3 0
9 :3 0 3c :3 0
52e 52f 0 3d
:3 0 530 531 0
533 2f2 535 2f4
534 533 :2 0 57d
3f :3 0 9 :3 0
3c :3 0 538 539
0 3d :3 0 53a
53b 0 575 1c
:3 0 1d :3 0 53d
53e 0 1c :3 0
2d :3 0 540 541
0 40 :2 0 2
:4 0 75 :2 0 2f
:2 0 12 :2 0 2f6
546 548 :3 0 2f9
542 54a 53f 54c
54b :2 0 1f :3 0
30 :3 0 54e 54f
0 21 :4 0 76
:4 0 24 :2 0 25
:3 0 2fd 553 555
:3 0 24 :2 0 43
:4 0 300 557 559
:3 0 24 :2 0 44
:4 0 303 55b 55d
:3 0 24 :2 0 3d
:3 0 306 55f 561
:3 0 24 :2 0 45
:4 0 309 563 565
:3 0 24 :2 0 46
:3 0 47 :3 0 12
:2 0 48 :2 0 30c
568 56c 310 567
56e :3 0 313 550
570 :2 0 572 316
573 54d 572 0
574 318 0 575
31a 578 :3 0 578
0 578 577 575
576 :6 0 57a 14
:3 0 31d 57c 31f
57b 57a :2 0 57d
321 :2 0 57f 0
57f 57e 52c 57d
:6 0 581 1 :3 0
324 583 422 50b
0 584 0 581
0 584 326 0
5f9 5e :3 0 5f
:3 0 60 :3 0 61
:3 0 5f :3 0 56
:3 0 62 :3 0 54
:3 0 e :4 0 63
1 :8 0 5a4 4d
:4 0 58f :3 0 4e
:3 0 49 :2 0 32b
592 593 :3 0 9
:3 0 3c :3 0 595
596 0 14 :2 0
597 598 0 59a
32e 5a1 9 :3 0
3c :3 0 59b 59c
0 4f :2 0 59d
59e 0 5a0 330
5a2 594 59a 0
5a3 0 5a0 0
5a3 332 0 5a4
335 5f7 3e :3 0
9 :3 0 3c :3 0
5a6 5a7 0 3d
:3 0 5a8 5a9 0
5ab 338 5ad 33a
5ac 5ab :2 0 5f5
3f :3 0 9 :3 0
3c :3 0 5b0 5b1
0 3d :3 0 5b2
5b3 0 5ed 1c
:3 0 1d :3 0 5b5
5b6 0 1c :3 0
2d :3 0 5b8 5b9
0 40 :2 0 2
:4 0 77 :2 0 2f
:2 0 12 :2 0 33c
5be 5c0 :3 0 33f
5ba 5c2 5b7 5c4
5c3 :2 0 1f :3 0
30 :3 0 5c6 5c7
0 21 :4 0 78
:4 0 24 :2 0 25
:3 0 343 5cb 5cd
:3 0 24 :2 0 43
:4 0 346 5cf 5d1
:3 0 24 :2 0 44
:4 0 349 5d3 5d5
:3 0 24 :2 0 3d
:3 0 34c 5d7 5d9
:3 0 24 :2 0 45
:4 0 34f 5db 5dd
:3 0 24 :2 0 46
:3 0 47 :3 0 12
:2 0 48 :2 0 352
5e0 5e4 356 5df
5e6 :3 0 359 5c8
5e8 :2 0 5ea 35c
5eb 5c5 5ea 0
5ec 35e 0 5ed
360 5f0 :3 0 5f0
0 5f0 5ef 5ed
5ee :6 0 5f2 16
:3 0 363 5f4 365
5f3 5f2 :2 0 5f5
367 :2 0 5f7 0
5f7 5f6 5a4 5f5
:6 0 5f9 1 :3 0
36a 5fb 332 418
0 5fc 0 5f9
0 5fc 36d 0
5fd 370 5ff 14e
329 0 600 0
5fd 0 600 372
0 7ba 79 :3 0
29 :3 0 13 :3 0
14 :2 0 375 602
605 12 :2 0 378
601 608 609 :2 0
49 :2 0 12 :2 0
37d 60b 60d :3 0
60e :2 0 29 :3 0
19 :3 0 14 :2 0
380 610 613 7a
:2 0 14 :2 0 385
615 617 :3 0 618
:2 0 60f 61a 619
:2 0 1a :3 0 7b
:3 0 29 :3 0 7c
:3 0 18 :3 0 388
61f 621 14 :2 0
38a 61e 624 7d
:2 0 29 :3 0 7c
:3 0 17 :3 0 38d
628 62a 14 :2 0
38f 627 62d 392
626 62f :3 0 630
:2 0 7e :2 0 7f
:2 0 395 632 634
:3 0 635 :2 0 80
:2 0 29 :3 0 19
:3 0 14 :2 0 398
638 63b 39b 637
63d :3 0 39e 61d
63f 61c 640 0
6ce 1a :3 0 7a
:2 0 12 :2 0 3a2
643 645 :3 0 646
:2 0 1a :3 0 81
:2 0 80 :2 0 12
:2 0 3a5 64a 64c
:3 0 3a9 649 64e
:3 0 647 650 64f
:2 0 39 :3 0 5f
:3 0 60 :3 0 61
:3 0 5f :3 0 56
:3 0 82 :3 0 83
:3 0 83 :3 0 4b
:3 0 84 :3 0 83
:3 0 3a :3 0 b
:4 0 85 1 :8 0
676 4d :4 0 661
:3 0 4e :3 0 49
:2 0 3ae 664 665
:3 0 9 :3 0 3c
:3 0 667 668 0
14 :2 0 669 66a
0 66c 3b1 673
9 :3 0 3c :3 0
66d 66e 0 4f
:2 0 66f 670 0
672 3b3 674 666
66c 0 675 0
672 0 675 3b5
0 676 3b8 6c9
3e :3 0 9 :3 0
3c :3 0 678 679
0 3d :3 0 67a
67b 0 67d 3bb
67f 3bd 67e 67d
:2 0 6c7 3f :3 0
9 :3 0 3c :3 0
682 683 0 3d
:3 0 684 685 0
6bf 1c :3 0 1d
:3 0 687 688 0
1c :3 0 2d :3 0
68a 68b 0 40
:2 0 2 :4 0 86
:2 0 2f :2 0 12
:2 0 3bf 690 692
:3 0 3c2 68c 694
689 696 695 :2 0
1f :3 0 30 :3 0
698 699 0 21
:4 0 87 :4 0 24
:2 0 25 :3 0 3c6
69d 69f :3 0 24
:2 0 43 :4 0 3c9
6a1 6a3 :3 0 24
:2 0 44 :4 0 3cc
6a5 6a7 :3 0 24
:2 0 3d :3 0 3cf
6a9 6ab :3 0 24
:2 0 45 :4 0 3d2
6ad 6af :3 0 24
:2 0 46 :3 0 47
:3 0 12 :2 0 48
:2 0 3d5 6b2 6b6
3d9 6b1 6b8 :3 0
3dc 69a 6ba :2 0
6bc 3df 6bd 697
6bc 0 6be 3e1
0 6bf 3e3 6c2
:3 0 6c2 0 6c2
6c1 6bf 6c0 :6 0
6c4 18 :3 0 3e6
6c6 3e8 6c5 6c4
:2 0 6c7 3ea :2 0
6c9 0 6c9 6c8
676 6c7 :6 0 6cb
1 :3 0 3ed 6cc
651 6cb 0 6cd
3ef 0 6ce 3f1
743 39 :3 0 83
:3 0 83 :3 0 4b
:3 0 84 :3 0 83
:3 0 3a :3 0 b
:4 0 88 1 :8 0
6ed 4d :4 0 6d8
:3 0 4e :3 0 49
:2 0 3f6 6db 6dc
:3 0 9 :3 0 3c
:3 0 6de 6df 0
14 :2 0 6e0 6e1
0 6e3 3f9 6ea
9 :3 0 3c :3 0
6e4 6e5 0 4f
:2 0 6e6 6e7 0
6e9 3fb 6eb 6dd
6e3 0 6ec 0
6e9 0 6ec 3fd
0 6ed 400 740
3e :3 0 9 :3 0
3c :3 0 6ef 6f0
0 3d :3 0 6f1
6f2 0 6f4 403
6f6 405 6f5 6f4
:2 0 73e 3f :3 0
9 :3 0 3c :3 0
6f9 6fa 0 3d
:3 0 6fb 6fc 0
736 1c :3 0 1d
:3 0 6fe 6ff 0
1c :3 0 2d :3 0
701 702 0 40
:2 0 2 :4 0 89
:2 0 2f :2 0 12
:2 0 407 707 709
:3 0 40a 703 70b
700 70d 70c :2 0
1f :3 0 30 :3 0
70f 710 0 21
:4 0 8a :4 0 24
:2 0 25 :3 0 40e
714 716 :3 0 24
:2 0 43 :4 0 411
718 71a :3 0 24
:2 0 44 :4 0 414
71c 71e :3 0 24
:2 0 3d :3 0 417
720 722 :3 0 24
:2 0 45 :4 0 41a
724 726 :3 0 24
:2 0 46 :3 0 47
:3 0 12 :2 0 48
:2 0 41d 729 72d
421 728 72f :3 0
424 711 731 :2 0
733 427 734 70e
733 0 735 429
0 736 42b 739
:3 0 739 0 739
738 736 737 :6 0
73b 1a :3 0 42e
73d 430 73c 73b
:2 0 73e 432 :2 0
740 0 740 73f
6ed 73e :6 0 742
1 :3 0 435 744
61b 6ce 0 745
0 742 0 745
437 0 7ba 9
:3 0 3c :3 0 746
747 0 9 :3 0
49 :2 0 8b :3 0
749 74b 0 43c
74a 74d :3 0 74e
:2 0 29 :3 0 13
:3 0 14 :2 0 43f
750 753 8c :2 0
4a :2 0 444 755
757 :3 0 758 :2 0
74f 75a 759 :2 0
15 :3 0 8d :3 0
d :3 0 b :3 0
447 75d 760 75c
761 0 763 44a
764 75b 763 0
765 44c 0 7ba
15 :3 0 8e :3 0
d :3 0 44e 767
769 766 76a 0
7ba 1c :3 0 1d
:3 0 76c 76d 0
1c :3 0 2d :3 0
76f 770 0 8f
:2 0 2 :4 0 90
:2 0 2f :2 0 12
:2 0 450 775 777
:3 0 453 771 779
76e 77b 77a :2 0
1f :3 0 30 :3 0
77d 77e 0 21
:4 0 91 :4 0 24
:2 0 25 :3 0 457
782 784 :3 0 24
:2 0 43 :4 0 45a
786 788 :3 0 24
:2 0 92 :4 0 45d
78a 78c :3 0 24
:2 0 29 :3 0 2a
:3 0 9 :3 0 3c
:3 0 791 792 0
460 790 794 2b
:4 0 462 78f 797
465 78e 799 :3 0
24 :2 0 93 :4 0
468 79b 79d :3 0
24 :2 0 27 :4 0
46b 79f 7a1 :3 0
24 :2 0 3 :3 0
46e 7a3 7a5 :3 0
24 :2 0 27 :4 0
471 7a7 7a9 :3 0
474 77f 7ab :2 0
7ad 477 7ae 77c
7ad 0 7af 479
0 7ba 94 :3 0
95 :3 0 7b0 7b1
0 9 :3 0 3c
:3 0 7b3 7b4 0
3 :3 0 4e :3 0
47b 7b2 7b8 :2 0
7ba 47f 85b 9
:3 0 96 :2 0 4
7bb 7bc 0 1c
:3 0 1d :3 0 7be
7bf 0 1c :3 0
2d :3 0 7c1 7c2
0 40 :2 0 2
:4 0 97 :2 0 2f
:2 0 12 :2 0 48b
7c7 7c9 :3 0 48e
7c3 7cb 7c0 7cd
7cc :2 0 1f :3 0
30 :3 0 7cf 7d0
0 21 :4 0 98
:4 0 24 :2 0 25
:3 0 492 7d4 7d6
:3 0 495 7d1 7d8
:2 0 7f1 1f :3 0
30 :3 0 7da 7db
0 21 :4 0 99
:4 0 24 :2 0 9
:3 0 9a :3 0 7e0
7e1 0 498 7df
7e3 :3 0 24 :2 0
93 :4 0 49b 7e5
7e7 :3 0 24 :2 0
9 :3 0 9b :3 0
7ea 7eb 0 49e
7e9 7ed :3 0 4a1
7dc 7ef :2 0 7f1
4a4 7f2 7ce 7f1
0 7f3 4a7 0
803 9c :3 0 1b
:3 0 9 :3 0 9a
:3 0 9d :3 0 9
:3 0 9b :3 0 3
:3 0 9e :4 0 9f
1 :a 0 7ff :2 0
803 7f5 7fd :2 0
8 :3 0 1b :3 0
801 :2 0 803 4a9
805 4ad 804 803
:2 0 859 3f :3 0
9 :3 0 3c :3 0
808 809 0 3d
:3 0 80a 80b 0
851 1c :3 0 1d
:3 0 80d 80e 0
1c :3 0 2d :3 0
810 811 0 40
:2 0 2 :4 0 a0
:2 0 2f :2 0 12
:2 0 4af 816 818
:3 0 4b2 812 81a
80f 81c 81b :2 0
1f :3 0 30 :3 0
81e 81f 0 21
:4 0 a1 :4 0 24
:2 0 25 :3 0 4b6
823 825 :3 0 4b9
820 827 :2 0 841
1f :3 0 30 :3 0
829 82a 0 21
:4 0 44 :4 0 24
:2 0 3d :3 0 4bc
82e 830 :3 0 24
:2 0 45 :4 0 4bf
832 834 :3 0 24
:2 0 46 :3 0 47
:3 0 12 :2 0 48
:2 0 4c2 837 83b
4c6 836 83d :3 0
4c9 82b 83f :2 0
841 4cc 842 81d
841 0 843 4cf
0 851 9c :3 0
1b :3 0 9d :3 0
9 :3 0 9b :3 0
3 :3 0 9e :4 0
a2 1 :a 0 84d
:2 0 851 845 84b
:2 0 8 :3 0 1b
:3 0 84f :2 0 851
4d1 854 :3 0 854
0 854 853 851
852 :6 0 856 1
:3 0 4d6 858 4d8
857 856 :2 0 859
4da :2 0 85b 4dd
85b 85a 7ba 859
:6 0 85c :2 0 2
14 85b 85f :3 0
85e 85c 860 :8 0

4eb
4
:3 0 1 3 1
7 1 b 3
6 a e 1
18 1 16 1
1f 1 1d 1
26 1 24 1
2b 1 32 1
30 1 37 1
3d 1 43 1
49 1 4f 1
55 1 5b 1
60 1 6d 1
79 2 7c 7e
2 81 83 2
85 87 2 89
8b 2 8d 8f
1 95 2 97
98 2 91 9a
2 9c 9e 2
a0 a2 2 a4
a6 2 a8 aa
2 ac ae 4
75 7b 80 b0
1 b2 1 b4
2 be c0 3
bc bd c2 2
cb cd 2 ca
cf 1 d1 1
d3 2 eb f0
1 f7 1 f2
2 10a 10c 3
108 109 10e 2
117 119 2 11b
11d 2 11f 121
2 123 125 2
127 129 3 12e
12f 130 2 12b
132 2 116 134
1 136 1 138
2 101 139 1
13d 1 fc 2
fa 141 2 147
148 1 14c 2
14a 14c 3 150
151 152 1 156
2 154 156 1
162 2 161 162
1 16a 1 170
2 172 173 2
15f 174 1 17b
1 176 2 18e
190 3 18c 18d
192 2 19b 19d
2 19f 1a1 2
1a3 1a5 2 1a7
1a9 2 1ab 1ad
3 1b2 1b3 1b4
2 1af 1b6 2
19a 1b8 1 1ba
1 1bc 2 185
1bd 1 1c1 1
180 2 17e 1c5
1 1d6 2 1d5
1d6 1 1de 1
1e4 2 1e6 1e7
2 1d3 1e8 1
1ef 1 1ea 2
202 204 3 200
201 206 2 20f
211 2 213 215
2 217 219 2
21b 21d 2 21f
221 3 226 227
228 2 223 22a
2 20e 22c 1
22e 1 230 2
1f9 231 1 235
1 1f4 2 1f2
239 2 1c8 23c
1 249 2 248
249 1 251 1
257 2 259 25a
2 246 25b 1
262 1 25d 2
275 277 3 273
274 279 2 282
284 2 286 288
2 28a 28c 2
28e 290 2 292
294 3 299 29a
29b 2 296 29d
2 281 29f 1
2a1 1 2a3 2
26c 2a4 1 2a8
1 267 2 265
2ac 1 2af 2
2b2 2b3 1 2c1
2 2c0 2c1 1
2c9 1 2cf 2
2d1 2d2 2 2be
2d3 1 2da 1
2d5 2 2ed 2ef
3 2eb 2ec 2f1
2 2fa 2fc 2
2fe 300 2 302
304 2 306 308
2 30a 30c 3
311 312 313 2
30e 315 2 2f9
317 1 319 1
31b 2 2e4 31c
1 320 1 2df
2 2dd 324 2
2b4 327 2 32b
32c 1 330 2
32e 330 1 33c
2 33b 33c 1
344 1 34a 2
34c 34d 2 339
34e 1 355 1
350 2 368 36a
3 366 367 36c
2 375 377 2
379 37b 2 37d
37f 2 381 383
2 385 387 3
38c 38d 38e 2
389 390 2 374
392 1 394 1
396 2 35f 397
1 39b 1 35a
2 358 39f 1
3b0 2 3af 3b0
1 3b8 1 3be
2 3c0 3c1 2
3ad 3c2 1 3c9
1 3c4 2 3dc
3de 3 3da 3db
3e0 2 3e9 3eb
2 3ed 3ef 2
3f1 3f3 2 3f5
3f7 2 3f9 3fb
3 400 401 402
2 3fd 404 2
3e8 406 1 408
1 40a 2 3d3
40b 1 40f 1
3ce 2 3cc 413
2 3a2 416 3
41a 41b 41c 1
420 2 41e 420
1 42f 2 42e
42f 1 437 1
43d 2 43f 440
2 42c 441 1
448 1 443 2
45b 45d 3 459
45a 45f 2 468
46a 2 46c 46e
2 470 472 2
474 476 2 478
47a 3 47f 480
481 2 47c 483
2 467 485 1
487 1 489 2
452 48a 1 48e
1 44d 2 44b
492 1 4a3 2
4a2 4a3 1 4ab
1 4b1 2 4b3
4b4 2 4a0 4b5
1 4bc 1 4b7
2 4cf 4d1 3
4cd 4ce 4d3 2
4dc 4de 2 4e0
4e2 2 4e4 4e6
2 4e8 4ea 2
4ec 4ee 3 4f3
4f4 4f5 2 4f0
4f7 2 4db 4f9
1 4fb 1 4fd
2 4c6 4fe 1
502 1 4c1 2
4bf 506 2 495
509 1 519 2
518 519 1 521
1 527 2 529
52a 2 516 52b
1 532 1 52d
2 545 547 3
543 544 549 2
552 554 2 556
558 2 55a 55c
2 55e 560 2
562 564 3 569
56a 56b 2 566
56d 2 551 56f
1 571 1 573
2 53c 574 1
578 1 537 2
535 57c 1 57f
2 582 583 1
591 2 590 591
1 599 1 59f
2 5a1 5a2 2
58e 5a3 1 5aa
1 5a5 2 5bd
5bf 3 5bb 5bc
5c1 2 5ca 5cc
2 5ce 5d0 2
5d2 5d4 2 5d6
5d8 2 5da 5dc
3 5e1 5e2 5e3
2 5de 5e5 2
5c9 5e7 1 5e9
1 5eb 2 5b4
5ec 1 5f0 1
5af 2 5ad 5f4
2 584 5f7 2
5fa 5fb 1 5fc
2 5fe 5ff 2
603 604 2 606
607 1 60c 2
60a 60c 2 611
612 1 616 2
614 616 1 620
2 622 623 1
629 2 62b 62c
2 625 62e 2
631 633 2 639
63a 2 636 63c
1 63e 1 644
2 642 644 1
64b 1 64d 2
648 64d 1 663
2 662 663 1
66b 1 671 2
673 674 2 660
675 1 67c 1
677 2 68f 691
3 68d 68e 693
2 69c 69e 2
6a0 6a2 2 6a4
6a6 2 6a8 6aa
2 6ac 6ae 3
6b3 6b4 6b5 2
6b0 6b7 2 69b
6b9 1 6bb 1
6bd 2 686 6be
1 6c2 1 681
2 67f 6c6 1
6c9 1 6cc 2
641 6cd 1 6da
2 6d9 6da 1
6e2 1 6e8 2
6ea 6eb 2 6d7
6ec 1 6f3 1
6ee 2 706 708
3 704 705 70a
2 713 715 2
717 719 2 71b
71d 2 71f 721
2 723 725 3
72a 72b 72c 2
727 72e 2 712
730 1 732 1
734 2 6fd 735
1 739 1 6f8
2 6f6 73d 1
740 2 743 744
1 74c 2 748
74c 2 751 752
1 756 2 754
756 2 75e 75f
1 762 1 764
1 768 2 774
776 3 772 773
778 2 781 783
2 785 787 2
789 78b 1 793
2 795 796 2
78d 798 2 79a
79c 2 79e 7a0
2 7a2 7a4 2
7a6 7a8 2 780
7aa 1 7ac 1
7ae 3 7b5 7b6
7b7 b b5 d4
d7 da 144 600
745 765 76b 7af
7b9 2 7c6 7c8
3 7c4 7c5 7ca
2 7d3 7d5 2
7d2 7d7 2 7de
7e2 2 7e4 7e6
2 7e8 7ec 2
7dd 7ee 2 7d9
7f0 1 7f2 3
7f3 7fe 802 1
7bd 2 815 817
3 813 814 819
2 822 824 2
821 826 2 82d
82f 2 831 833
3 838 839 83a
2 835 83c 2
82c 83e 2 828
840 1 842 4
80c 843 84c 850
1 854 1 807
2 805 858 d
1b 22 29 2e
35 3b 41 47
4d 53 59 5e
65 
1
4
0 
85f
0
1
28
1c
11
0 1 2 1 4 1 6 1
8 1 a 1 c 1 e 1
10 1 12 1 14 1 16 1
18 1 1a 1 0 0 0 0
0 0 0 0 0 0 0 0

b 1 0
3 1 0
55 1 0
16 1 0
2 0 1
3d 1 0
2b 1 0
37 1 0
49 1 0
24 1 0
4f 1 0
30 1 0
43 1 0
1d 1 0
7 1 0
5b 1 0
60 1 0
0

/
SHOW ERRORS FUNCTION SP_RJCT_ETC_ASSIGN;
ALTER FUNCTION SP_RJCT_ETC_ASSIGN COMPILE DEBUG;
/****************************************************************************************
Reject ETC and/or FORECAST_START, WBS level
  Bitflag =
    1 - ETC
    2 - FORECAST_START
    3 - ETC + FORECAST_START
****************************************************************************************/
Prompt == FUNCTION SP_RJCT_ETC_WBS ==
CREATE OR REPLACE FUNCTION SP_RJCT_ETC_WBS wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
eb
2 :e:
1FUNCTION:
1SP_RJCT_ETC_WBS:
1ID:
1VARCHAR2:
1BITFLAG001:
1PLS_INTEGER:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_FINISH_DATE:
1TIMESTAMP:
1H_PROJECT_ID:
132:
1H_RANK:
1200:
1H_START_DATE:
1H_TASK_ID:
1H_TSK_ASSIGNMENT_ID:
1H_BIT_FLAG:
11:
1H_BITFLAG:
10:
1H_EFFORT:
1H_TYPE:
1H_START_D:
1DATE:
1H_TEMP_SC_STATE:
1H_TEMP_VAR:
1H_TEMP_SC_NODATA:
1H_STEPS_EXIST:
1H_SCHED_WORK:
1H_WORK_TODATE:
1H_MLST_PERCENT:
1H_TEMP_SC_STATE_ALERT:
1H_ALERT_ID:
1H_REC_USER:
1H_CALLTYPE:
1H_TABLE_TYPE:
1I_STEPS_EXIST:
1I_PERCENT_COMPLETE:
1CURSOR:
1CAWTSKR:
1ELEMENT_ID:
1SCHED_EFFORT:
1PLANNED_EFFORT:
1FORECAST_START:
1SCHED_START:
1FORECAST_FINISH:
1SCHED_FINISH:
1TMT_WBS:
1PROJECT_ID:
1RANK:
1TYPE_ID:
1PMO_CNST:
1SC_TASK:
1SC_TASK_TYPE1:
1SC_TASK_TYPE2:
1SC_TASK_TYPE3:
1SC_MILESTONE:
1SC_PERSONAL_TASK:
1SC_ACTIVE_ADM_TASK:
1SC_INACTIVE_ADM_TASK:
1SC_WBS_CHANGE_REQUEST_TASK:
1SC_WBS_ACTION_ITEM_TASK:
1SC_WBS_RISK_TASK:
1SC_WBS_ISSUE_TASK:
1SC_WBS_REQUIREMENT_TASK:
1SC_WBS_DEFECT_TASK:
1SC_WBS_SERVICE_REQUEST_TASK:
1WORK_TODATE:
1ETC:
1SELECT ELEMENT_ID, SCHED_EFFORT+PLANNED_EFFORT,:n         CASE WHEN FORECAST_+
1START IS NULL THEN SCHED_START ELSE FORECAST_START END,:n         CASE WHEN F+
1ORECAST_FINISH IS NULL THEN SCHED_FINISH ELSE FORECAST_FINISH END:n         F+
1ROM TMT_WBS:n         WHERE PROJECT_ID = h_project_id AND RANK LIKE h_rank AN+
1D TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,+
1PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CN+
1ST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_RE+
1QUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNS+
1T.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_T+
1ASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK):n         AND (((SCHED_EFFORT+PLANN+
1ED_EFFORT) > 0 AND ((SCHED_EFFORT+PLANNED_EFFORT-WORK_TODATE) <> ETC)):n     +
1    OR (FORECAST_START IS NOT NULL OR FORECAST_FINISH IS NOT NULL)):
1CAWASSR:
1TSK_ASSIGNMENT_ID:
1BIT_FLAG:
1BEST:
1SCHED_WORK:
1PLANNED_WORK:
1MILESTONE_PERCENT:
1TASK_ASSIGNMENTS:
1TASK_ID:
1SELECT TSK_ASSIGNMENT_ID, BIT_FLAG, BEST, SCHED_WORK+PLANNED_WORK, WORK_TODAT+
1E, MILESTONE_PERCENT:n         FROM TASK_ASSIGNMENTS:n         WHERE PROJECT_+
1ID = h_project_id:n         AND TASK_ID = h_task_id:n         AND (((SCHED_WO+
1RK+PLANNED_WORK) > 0 AND ((SCHED_WORK+PLANNED_WORK-WORK_TODATE) <> ETC)):n   +
1      OR (FORECAST_START IS NOT NULL OR FORECAST_FINISH IS NOT NULL)):
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_RJCT_ETC_WBS(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
160:
1+:
1FPUT:
1               ENTER:: SP_RJCT_ETC_WBS(61) :: cplan.sql(20020) :: :
13:
1SCETCFORCASTREJECTED:
1RTRIM:
1SELECT PROJECT_ID, RTRIM(RANK), TYPE_ID:n      INTO h_project_id, h_rank, h_t+
1ype:n      FROM TMT_WBS WHERE ELEMENT_ID = id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
185:
1           EXCEPTION:: SP_RJCT_ETC_WBS(86) :: cplan.sql(20045) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1=:
1SC_SQL_SUCCESS:
1IS NOT NULL:
1%:
1-:
1START_FINISH_DATE:
1SELECT START_FINISH_DATE INTO h_start_D FROM TMT_WBS WHERE ELEMENT_ID = h_pro+
1ject_id:
1105:
1           EXCEPTION:: SP_RJCT_ETC_WBS(106) :: cplan.sql(20065) :: :
1OPEN:
1LOOP:
1FOUND:
1100:
1!=:
1EXIT:
12:
1FORECAST_DURATION:
1UPDATE TMT_WBS SET:n                  FORECAST_START = NULL,:n               +
1   FORECAST_FINISH = NULL,:n                  FORECAST_DURATION = 0:n        +
1          WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id:
1TRUE:
1147:
1           EXCEPTION:: SP_RJCT_ETC_WBS(148) :: cplan.sql(20107) :: :
1FC_ISSUE_ALERT:
1CONTINUE0000:
1ACTIVITIES:
1ETC_MODIFIED:
1SC_TIMESHEET_ACTIVITY:
1UPDATE ACTIVITIES:n                        SET ETC_MODIFIED = 'N':n          +
1              WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:n                +
1        AND TSK_ASSIGNMENT_ID <> ELEMENT_ID:n                        AND TYPE+
1_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY:
1198:
1           EXCEPTION:: SP_RJCT_ETC_WBS(199) :: cplan.sql(20158) :: :
1BAK_ASSIGN_STRING:
1UPDATE TASK_ASSIGNMENTS SET:n                                BAK_ASSIGN_STRIN+
1G = NULL:n                           WHERE TSK_ASSIGNMENT_ID = H_TSK_ASSIGNME+
1NT_ID:
1PROPOSED_START:
1PROPOSED_FINISH:
1PROPOSED_WORK:
1BAK_START_DT:
1BAK_FINISH_DT:
1BAK_WORK:
1ASSIGN_STRING:
1ACTUAL_FINISH:
1UPDATE TASK_ASSIGNMENTS SET:n                           PROPOSED_START      =+
1 SCHED_START,:n                           PROPOSED_FINISH     = SCHED_FINISH,+
1:n                           PROPOSED_WORK       = SCHED_WORK,:n             +
1              BAK_START_DT        = SCHED_START,:n                           +
1BAK_FINISH_DT       = SCHED_FINISH,:n                           BAK_WORK     +
1       = SCHED_WORK,:n                           BAK_ASSIGN_STRING   = ASSIGN+
1_STRING,:n                           BEST                = SCHED_WORK,:n     +
1                      ETC                 = SCHED_WORK - WORK_TODATE,:n      +
1                     ACTUAL_FINISH       = NULL:n                           W+
1HERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
1233:
1           EXCEPTION:: SP_RJCT_ETC_WBS(234) :: cplan.sql(20193) :: :
1UPDATE TASK_ASSIGNMENTS SET:n                           SCHED_WORK	= SCHED_WO+
1RK,:n                           ETC			= SCHED_WORK - WORK_TODATE:n           +
1                WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
1263:
1           EXCEPTION:: SP_RJCT_ETC_WBS(264) :: cplan.sql(20223) :: :
1ACTUAL_DURATION:
1UPDATE TASK_ASSIGNMENTS SET:n                        ETC					= PLANNED_WORK, +
1:n                        MILESTONE_PERCENT	= 0 ,:n                        AC+
1TUAL_FINISH		= NULL, :n                        ACTUAL_DURATION		= 0:n        +
1                WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
1288:
1           EXCEPTION:: SP_RJCT_ETC_WBS(289) :: cplan.sql(20248) :: :
1UPDATE TASK_ASSIGNMENTS SET:n                        FORECAST_START = NULL,:n+
1                        FORECAST_FINISH = NULL,:n                        FORE+
1CAST_DURATION = 0:n                        WHERE TSK_ASSIGNMENT_ID = h_tsk_as+
1signment_id:
1315:
1           EXCEPTION:: SP_RJCT_ETC_WBS(316) :: cplan.sql(20275) :: :
1UPDATE ACTIVITIES:n                           SET ETC_MODIFIED = 'N':n       +
1                    WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:n          +
1                 AND TSK_ASSIGNMENT_ID <> ELEMENT_ID:n                       +
1    AND TYPE_ID = PMO_CNST.SC_TIMESHEET_ACTIVITY:
1340:
1           EXCEPTION:: SP_RJCT_ETC_WBS(341) :: cplan.sql(20300) :: :
1UPDATE TASK_ASSIGNMENTS SET:n                           FORECAST_START = NULL+
1,:n                           FORECAST_FINISH = NULL,:n                      +
1     FORECAST_DURATION = 0,:n                           SCHED_WORK = CASE WHE+
1N PROPOSED_WORK > WORK_TODATE THEN PROPOSED_WORK ELSE SCHED_WORK END,:n      +
1                     ETC = CASE WHEN PROPOSED_WORK > WORK_TODATE THEN PROPOSE+
1D_WORK - WORK_TODATE ELSE 0 END/*,:n                           MILESTONE_PERC+
1ENT = (WORK_TODATE * 10000) / SCHED_WORK*/:n                           WHERE +
1TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
1366:
1           EXCEPTION:: SP_RJCT_ETC_WBS(367) :: cplan.sql(20326) :: :
1UPDATE TASK_ASSIGNMENTS SET:n                           FORECAST_START = NULL+
1,:n                           FORECAST_FINISH = NULL,:n                      +
1     FORECAST_DURATION = 0,:n                           ETC = PLANNED_WORK, M+
1ILESTONE_PERCENT = 0:n                           WHERE TSK_ASSIGNMENT_ID = h_+
1tsk_assignment_id:
1390:
1           EXCEPTION:: SP_RJCT_ETC_WBS(391) :: cplan.sql(20350) :: :
1SC_SQL_NO_DATA:
1BITAND:
1>:
1TRUNC:
1TO_NUMBER:
1/:
1*:
110000.00:
1<:
1ROLLUP_BITS:
1PMO_STPU:
1BIT_OR:
1SC_PERCENT_COMPLETE_BIT:
1UPDATE TASK_ASSIGNMENTS SET:n                           ROLLUP_BITS = PMO_STP+
1U.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT),:n                    +
1       ACTUAL_FINISH = CASE:n                           WHEN ACTUAL_FINISH IS+
1 NULL AND ETC = 0 THEN SCHED_FINISH:n                           ELSE ACTUAL_F+
1INISH:n                           END:n                           WHERE TSK_A+
1SSIGNMENT_ID = h_tsk_assignment_id:
1434:
1           EXCEPTION:: SP_RJCT_ETC_WBS(435) :: cplan.sql(20394) :: :
1UPDATE TASK_ASSIGNMENTS SET:n                           ACTUAL_FINISH = CASE +
1WHEN (ACTUAL_FINISH IS NULL) AND ETC = 0 THEN SCHED_FINISH ELSE ACTUAL_FINISH+
1 END:n                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_i+
1d:
1455:
1           EXCEPTION:: SP_RJCT_ETC_WBS(456) :: cplan.sql(20415) :: :
1FC_FIX_SUMMARY_TS:
1CLOSE:
1480:
1           EXCEPTION:: SP_RJCT_ETC_WBS(481) :: cplan.sql(20440) :: :
1CONSTRAINT_TYPE:
1TO_DATE:
1CONSTRAINT_DT:
1UPDATE TMT_WBS SET:n            CONSTRAINT_TYPE =:n            CASE:n        +
1    WHEN TO_DATE(SubStr(SCHED_START, 1, 10)) <> TO_DATE(SubStr(h_start_D, 1, +
110)) THEN 6:n            ELSE 0:n            END,:n            CONSTRAINT_DT +
1=:n            CASE:n            WHEN TO_DATE(SubStr(SCHED_START, 1, 10)) <> +
1TO_DATE(SubStr(h_start_D, 1, 10)) THEN SCHED_START:n            ELSE NULL:n  +
1          END:n            WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h+
1_task_id:
1512:
1           EXCEPTION:: SP_RJCT_ETC_WBS(513) :: cplan.sql(20472) :: :
1531:
1           EXCEPTION:: SP_RJCT_ETC_WBS(532) :: cplan.sql(20491) :: :
1FC_ROLLUP:
14:
1543:
1       RETURN_STATUS:: SP_RJCT_ETC_WBS(544) :: cplan.sql(20503) :: :
1STATE:: :
1 ID:: :
1PMODEFINES:
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
1550:
1           EXCEPTION:: SP_RJCT_ETC_WBS(551) :: cplan.sql(20510) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1ERROR:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1562:
1           EXCEPTION:: SP_RJCT_ETC_WBS(563) :: cplan.sql(20522) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
b48
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 81 b0 a0 f4 b4
bf c8 :35 a0 12a bd b7 11 a4
b1 a0 f4 b4 bf c8 :14 a0 12a
bd b7 11 a4 b1 a3 :2 a0 6b
1c 81 b0 :2 a0 6b :2 a0 6b 6e
a5 b a 10 :2 a0 6b 6e :2 a0
6b 6e a5 b 6e 7e a0 b4
2e 6e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
d :2 a0 d a0 51 d a0 6e
d :a a0 12a :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b a0 7e a0
6b b4 2e 5a a0 7e b4 2e
:2 5a a 10 :3 a0 7e 6e b4 2e
:2 51 a5 b d a0 7e 51 b4
2e d :5 a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 e9 dd b3
:6 a0 e9 d3 5 :2 a0 f :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c :2 a0 6b a0 7e a0 6b
b4 2e 5a a0 2b b7 19 3c
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 51 a5 b 7e 51 b4 2e
:8 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
a0 51 d :8 a0 51 a5 b a5
b d b7 a0 3b b7 :2 19 3c
b7 :2 a0 e9 dd b3 :8 a0 e9 d3
5 :2 a0 f :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c :2 a0
6b a0 7e a0 6b b4 2e 5a
a0 2b b7 19 3c a0 51 d
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 :2 51 a5 b 7e 6e b4 2e
:9 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 51 a5 b :2 7e 51 b4 2e
b4 2e :4 a0 12a :17 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :8 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :2 19 3c
b7 :8 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 :2 19 3c b7 :2 a0 51 a5
b 7e 51 b4 2e :6 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :2 a0 :2 51
a5 b 7e 6e b4 2e :9 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :10 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f b7 :9 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:2 19 3c b7 :2 19 3c b7 :2 19 3c
:2 a0 6b a0 7e a0 6b b4 2e
5a a0 51 d b7 19 3c a0
51 d :8 a0 51 a5 b a5 b
d :2 a0 51 a5 b 7e 51 b4
2e :3 a0 51 a5 b 51 a5 b
5a 7e 51 b4 2e 5a :2 a0 51
a5 b 7e 51 b4 2e 5a a
10 :5 a0 a5 b 51 a5 b 7e
:3 a0 a5 b 51 a5 b b4 2e
5a 7e 51 b4 2e 5a 7e :2 a0
51 a5 b b4 2e a5 b d
a0 7e 51 b4 2e 5a a0 :2 7e
51 b4 2e b4 2e 52 10 :e a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:8 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c :4 a0 a5 b d b7
19 3c b7 19 3c b7 a0 47
:2 a0 e9 c1 :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 :2 19 3c :14 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f 93 4f
b0 46 b7 a0 47 :2 a0 e9 c1
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 51 a5 b :2 7e 51 b4 2e
b4 2e :3 a0 a5 b d b7 19
3c b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 :2 a0 6b :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 :2 a0 6b :2 6e 7e :2 a0
6b b4 2e 7e 6e b4 2e 7e
:2 a0 6b b4 2e a5 57 b7 19
3c :9 a0 12a e9 dd :2 a0 65 b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c :7 a0
12a e9 dd :2 a0 65 b7 a4 b1
11 4f b7 a6 9 a4 b1 11
68 4f 1d 17 b5 
b48
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 30
56 5a 5e 62 65 69 82 71
75 7d 70 9f 8d 6d 91 92
9a 8c bc aa 89 ae af b7
a9 d8 c7 cb d3 a6 f4 df
e3 e6 e7 ef c6 111 ff c3
103 104 10c fe 12e 11c fb 120
121 129 11b 14a 139 13d 118 145
138 166 155 159 135 161 154 182
171 175 151 17d 170 19e 18d 191
199 16d 1b9 1a5 1a9 1b1 1b4 18c
1d5 1c4 1c8 189 1d0 1c3 1f1 1e0
1e4 1c0 1ec 1df 20d 1fc 200 1dc
208 1fb 229 218 21c 1f8 224 217
245 234 238 214 240 233 261 250
254 230 25c 24f 27d 26c 270 24c
278 26b 29a 288 268 28c 28d 295
287 2b7 2a5 284 2a9 2aa 2b2 2a4
2d3 2c2 2c6 2a1 2ce 2c1 2ef 2de
2e2 2be 2ea 2dd 30b 2fa 2fe 2da
306 2f9 327 316 31a 322 2f6 312
32e 33f 340 343 347 34b 34f 353
357 35b 35f 363 367 36b 36f 373
377 37b 37f 383 387 38b 38f 393
397 39b 39f 3a3 3a7 3ab 3af 3b3
3b7 3bb 3bf 3c3 3c7 3cb 3cf 3d3
3d7 3db 3df 3e3 3e7 3eb 3ef 3f3
3f7 3fb 3ff 403 407 40b 40f 413
417 41b 427 42c 42e 43a 43e 440
444 455 456 459 45d 461 465 469
46d 471 475 479 47d 481 485 489
48d 491 495 499 49d 4a1 4a5 4a9
4ad 4b9 4be 4c0 4cc 4d0 4ee 4d6
4da 4de 4e1 4e9 4d5 4f5 4f9 4d2
4fd 501 505 508 50d 50e 1 510
515 519 51d 520 525 529 52d 530
535 536 538 53d 540 544 545 54a
54f 552 557 558 55d 560 564 565
56a 56d 572 573 578 57b 580 581
586 589 58d 591 595 596 598 59d
59e 5a0 5a1 5a6 5a9 5ae 5af 5b4
5b7 5bc 5bd 5c2 5c5 5c9 5ca 5cf
5d2 5d7 5d8 5dd 5e0 5e5 5e6 5eb
5ec 5f1 5f3 5f7 5fa 5fe 602 605
609 60d 610 613 618 61b 61e 621
622 627 628 1 62a 62f 633 637
63a 63f 644 647 64b 64c 651 652
657 659 65d 660 664 668 66c 670
674 678 67c 67f 683 687 68c 690
694 698 69c 6a0 6a4 6a8 6ac 6b0
6b4 6b8 6c4 6c8 6cc 6cf 6d3 6d7
6d9 6dd 6e1 6e5 6e8 6ec 6f0 6f2
6f3 6f8 1 6fc 700 704 707 70b
70f 713 717 71a 71e 722 725 728
72d 730 733 736 737 73c 73d 1
73f 744 748 74c 74f 754 759 75c
760 761 766 769 76e 76f 774 777
77c 77d 782 785 789 78a 78f 792
797 798 79d 7a0 7a4 7a8 7ab 7ae
7af 7b1 7b2 7b7 7b8 7bd 7bf 7c3
7c6 7c8 7cc 7ce 7da 7dc 7de 7df
7e4 7e8 7ea 7f6 7f8 7fc 800 803
807 80a 80e 811 812 817 81a 81e
821 822 827 82a 1 82d 832 836
83a 83e 841 846 847 84c 84f 852
853 855 859 85d 860 863 864 869
86d 871 875 879 87d 881 88d 891
895 898 89c 8a0 8a2 8a6 8aa 8ae
8b1 8b5 8b9 8bb 8bc 8c1 1 8c5
8c9 8cd 8d0 8d4 8d8 8dc 8e0 8e3
8e7 8eb 8ee 8f1 8f6 8f9 8fc 8ff
900 905 906 1 908 90d 911 915
918 91d 922 925 929 92a 92f 932
937 938 93d 940 945 946 94b 94e
952 953 958 95b 960 961 966 969
96d 971 974 977 978 97a 97b 980
981 986 988 98c 98f 991 995 997
9a3 9a5 9a7 9a8 9ad 9b1 9b3 9bf
9c1 9c5 9c9 9ce 9d2 9d3 9d7 9db
9df 9e3 9e7 9eb 9f0 9f5 9f9 9fd
a01 a06 a0a a0e a11 a14 a18 a1a
a1e a22 a25 a28 a2c a2e a32 a36
a39 a3d a41 a44 a48 a4b a4f a52
a53 a58 a5b a5f a65 a67 a6b a6e
a72 a76 a79 a7a a7c a7f a82 a83
a88 a8c a90 a93 a94 a96 a99 a9c
a9d aa2 aa6 aaa aae ab2 ab6 aba
abe ac2 ace ad2 ad7 adb ade adf
ae4 ae8 aec aef af2 af6 af8 afc
b00 b03 b06 b0a b0c b10 b14 b17
b19 b1d b21 b25 b28 b2c b30 b32
b33 b38 1 b3c b40 b44 b47 b4b
b4f b53 b57 b5a b5e b62 b65 b68
b6d b70 b73 b76 b77 b7c b7d 1
b7f b84 b88 b8c b8f b94 b99 b9c
ba0 ba1 ba6 ba9 bae baf bb4 bb7
bbc bbd bc2 bc5 bc9 bca bcf bd2
bd7 bd8 bdd be0 be4 be8 beb bee
bef bf1 bf2 bf7 bf8 bfd bff c03
c06 c08 c0c c0e c1a c1c c1e c1f
c24 c28 c2a c36 c38 c3c c3f c43
c47 c4b c4f c53 c57 c5b c5f c63
c66 c67 c69 c6a c6c c70 c72 c76
c7c c7e c82 c86 c89 c8b c8f c93
c98 c9c c9d ca1 ca5 ca9 cad cb1
cb5 cb9 cbd cc2 cc7 ccb ccf cd3
cd8 cdc ce0 ce3 ce6 cea cec cf0
cf4 cf7 cfa cfe d00 d04 d08 d0b
d0f d13 d16 d1a d1d d21 d24 d25
d2a d2d d31 d37 d39 d3d d40 d44
d47 d4b d4f d53 d56 d57 d59 d5c
d5f d60 d65 d69 d6d d70 d73 d74
d76 d79 d7e d7f d84 d88 d8c d90
d94 d98 d9c da0 da4 da8 db4 db8
dbd dc1 dc4 dc5 dca dce dd2 dd5
dd8 ddc dde de2 de6 de9 dec df0
df2 df6 dfa dfd dff e03 e07 e0b
e0e e12 e16 e18 e19 e1e 1 e22
e26 e2a e2d e31 e35 e39 e3d e40
e44 e48 e4b e4e e53 e56 e59 e5c
e5d e62 e63 1 e65 e6a e6e e72
e75 e7a e7f e82 e86 e87 e8c e8f
e94 e95 e9a e9d ea2 ea3 ea8 eab
eaf eb0 eb5 eb8 ebd ebe ec3 ec6
eca ece ed1 ed4 ed5 ed7 ed8 edd
ede ee3 ee5 ee9 eec eee ef2 ef4
f00 f02 f04 f05 f0a f0e f10 f1c
f1e f22 f26 f29 f2a f2c f2f f32
f35 f36 f3b f3c f41 f45 f49 f4d
f51 f5d f61 f65 f69 f6d f71 f75
f79 f7d f81 f85 f89 f8d f91 f95
f99 f9d fa1 fa5 fa9 fad fb1 fb5
fb9 fc5 fc9 fce fd2 fd5 fd6 fdb
fdf fe3 fe6 fe9 fed fef ff3 ff7
ffa ffd 1001 1003 1007 100b 100e 1010
1014 1018 101c 101f 1023 1027 1029 102a
102f 1 1033 1037 103b 103e 1042 1046
104a 104e 1051 1055 1059 105c 105f 1064
1067 106a 106d 106e 1073 1074 1 1076
107b 107f 1083 1086 108b 1090 1093 1097
1098 109d 10a0 10a5 10a6 10ab 10ae 10b3
10b4 10b9 10bc 10c0 10c1 10c6 10c9 10ce
10cf 10d4 10d7 10db 10df 10e2 10e5 10e6
10e8 10e9 10ee 10ef 10f4 10f6 10fa 10fd
10ff 1103 1105 1111 1113 1115 1116 111b
111f 1121 112d 112f 1131 1135 1139 113d
1141 1145 1149 114d 1151 115d 1161 1166
116a 116d 116e 1173 1177 117b 117e 1181
1185 1187 118b 118f 1192 1195 1199 119b
119f 11a3 11a6 11a8 11ac 11b0 11b4 11b7
11bb 11bf 11c1 11c2 11c7 1 11cb 11cf
11d3 11d6 11da 11de 11e2 11e6 11e9 11ed
11f1 11f4 11f7 11fc 11ff 1202 1205 1206
120b 120c 1 120e 1213 1217 121b 121e
1223 1228 122b 122f 1230 1235 1238 123d
123e 1243 1246 124b 124c 1251 1254 1258
1259 125e 1261 1266 1267 126c 126f 1273
1277 127a 127d 127e 1280 1281 1286 1287
128c 128e 1292 1295 1297 129b 129d 12a9
12ab 12ad 12ae 12b3 12b7 12b9 12c5 12c7
12c9 12cd 12d1 12d4 12d6 12da 12de 12e2
12e6 12ea 12ee 12f2 12f6 1302 1306 130b
130f 1312 1313 1318 131c 1320 1323 1326
132a 132c 1330 1334 1337 133a 133e 1340
1344 1348 134b 134d 1351 1355 1359 135c
1360 1364 1366 1367 136c 1 1370 1374
1378 137b 137f 1383 1387 138b 138e 1392
1396 1399 139c 13a1 13a4 13a7 13aa 13ab
13b0 13b1 1 13b3 13b8 13bc 13c0 13c3
13c8 13cd 13d0 13d4 13d5 13da 13dd 13e2
13e3 13e8 13eb 13f0 13f1 13f6 13f9 13fd
13fe 1403 1406 140b 140c 1411 1414 1418
141c 141f 1422 1423 1425 1426 142b 142c
1431 1433 1437 143a 143c 1440 1442 144e
1450 1452 1453 1458 145c 145e 146a 146c
146e 1472 1476 1479 147b 147f 1483 1486
1487 1489 148c 148f 1490 1495 1499 149d
14a1 14a5 14a9 14ad 14b9 14bd 14c2 14c6
14c9 14ca 14cf 14d3 14d7 14da 14dd 14e1
14e3 14e7 14eb 14ee 14f1 14f5 14f7 14fb
14ff 1502 1504 1508 150c 1510 1513 1517
151b 151d 151e 1523 1 1527 152b 152f
1532 1536 153a 153e 1542 1545 1549 154d
1550 1553 1558 155b 155e 1561 1562 1567
1568 1 156a 156f 1573 1577 157a 157f
1584 1587 158b 158c 1591 1594 1599 159a
159f 15a2 15a7 15a8 15ad 15b0 15b4 15b5
15ba 15bd 15c2 15c3 15c8 15cb 15cf 15d3
15d6 15d9 15da 15dc 15dd 15e2 15e3 15e8
15ea 15ee 15f1 15f3 15f7 15f9 1605 1607
1609 160a 160f 1613 1615 1621 1623 1625
1629 162d 1630 1633 1634 1636 1639 163e
163f 1644 1648 164c 1650 1654 1658 165c
1660 1664 1668 1674 1678 167d 1681 1684
1685 168a 168e 1692 1695 1698 169c 169e
16a2 16a6 16a9 16ac 16b0 16b2 16b6 16ba
16bd 16bf 16c3 16c7 16cb 16ce 16d2 16d6
16d8 16d9 16de 1 16e2 16e6 16ea 16ed
16f1 16f5 16f9 16fd 1700 1704 1708 170b
170e 1713 1716 1719 171c 171d 1722 1723
1 1725 172a 172e 1732 1735 173a 173f
1742 1746 1747 174c 174f 1754 1755 175a
175d 1762 1763 1768 176b 176f 1770 1775
1778 177d 177e 1783 1786 178a 178e 1791
1794 1795 1797 1798 179d 179e 17a3 17a5
17a9 17ac 17ae 17b2 17b4 17c0 17c2 17c4
17c5 17ca 17ce 17d0 17dc 17de 17e2 17e6
17ea 17ee 17f2 17f6 17fa 17fe 1802 1806
180a 180e 1812 1816 181a 181e 182a 182e
1833 1837 183a 183b 1840 1844 1848 184b
184e 1852 1854 1858 185c 185f 1862 1866
1868 186c 1870 1873 1875 1879 187d 1881
1884 1888 188c 188e 188f 1894 1 1898
189c 18a0 18a3 18a7 18ab 18af 18b3 18b6
18ba 18be 18c1 18c4 18c9 18cc 18cf 18d2
18d3 18d8 18d9 1 18db 18e0 18e4 18e8
18eb 18f0 18f5 18f8 18fc 18fd 1902 1905
190a 190b 1910 1913 1918 1919 191e 1921
1925 1926 192b 192e 1933 1934 1939 193c
1940 1944 1947 194a 194b 194d 194e 1953
1954 1959 195b 195f 1962 1964 1968 196a
1976 1978 197a 197b 1980 1984 1986 1992
1994 1996 199a 199e 19a2 19a6 19aa 19ae
19b2 19b6 19ba 19c6 19ca 19cf 19d3 19d6
19d7 19dc 19e0 19e4 19e7 19ea 19ee 19f0
19f4 19f8 19fb 19fe 1a02 1a04 1a08 1a0c
1a0f 1a11 1a15 1a19 1a1d 1a20 1a24 1a28
1a2a 1a2b 1a30 1 1a34 1a38 1a3c 1a3f
1a43 1a47 1a4b 1a4f 1a52 1a56 1a5a 1a5d
1a60 1a65 1a68 1a6b 1a6e 1a6f 1a74 1a75
1 1a77 1a7c 1a80 1a84 1a87 1a8c 1a91
1a94 1a98 1a99 1a9e 1aa1 1aa6 1aa7 1aac
1aaf 1ab4 1ab5 1aba 1abd 1ac1 1ac2 1ac7
1aca 1acf 1ad0 1ad5 1ad8 1adc 1ae0 1ae3
1ae6 1ae7 1ae9 1aea 1aef 1af0 1af5 1af7
1afb 1afe 1b00 1b04 1b06 1b12 1b14 1b16
1b17 1b1c 1b20 1b22 1b2e 1b30 1b32 1b36
1b3a 1b3d 1b3f 1b43 1b47 1b4a 1b4c 1b50
1b54 1b57 1b5b 1b5f 1b62 1b66 1b69 1b6d
1b70 1b71 1b76 1b79 1b7d 1b80 1b84 1b86
1b8a 1b8d 1b91 1b94 1b98 1b9c 1ba0 1ba4
1ba8 1bac 1bb0 1bb4 1bb8 1bbb 1bbc 1bbe
1bbf 1bc1 1bc5 1bc9 1bcd 1bd0 1bd1 1bd3
1bd6 1bd9 1bda 1bdf 1be3 1be7 1beb 1bee
1bef 1bf1 1bf4 1bf5 1bf7 1bfa 1bfd 1c00
1c01 1c06 1c09 1c0d 1c11 1c14 1c15 1c17
1c1a 1c1d 1c1e 1c23 1 1c26 1c2b 1c2f
1c33 1c37 1c3b 1c3f 1c40 1c42 1c45 1c46
1c48 1c4b 1c4f 1c53 1c57 1c58 1c5a 1c5d
1c5e 1c60 1c61 1c66 1c69 1c6c 1c6f 1c70
1c75 1c78 1c7b 1c7f 1c83 1c86 1c87 1c89
1c8a 1c8f 1c90 1c92 1c96 1c9a 1c9d 1ca0
1ca1 1ca6 1ca9 1cad 1cb0 1cb3 1cb6 1cb7
1cbc 1cbd 1 1cc2 1cc7 1ccb 1ccf 1cd3
1cd7 1cdb 1cdf 1ce3 1ce7 1ceb 1cef 1cf3
1cf7 1cfb 1cff 1d0b 1d0f 1d14 1d18 1d1b
1d1c 1d21 1d25 1d29 1d2c 1d2f 1d33 1d35
1d39 1d3d 1d40 1d43 1d47 1d49 1d4d 1d51
1d54 1d56 1d5a 1d5e 1d62 1d65 1d69 1d6d
1d6f 1d70 1d75 1 1d79 1d7d 1d81 1d84
1d88 1d8c 1d90 1d94 1d97 1d9b 1d9f 1da2
1da5 1daa 1dad 1db0 1db3 1db4 1db9 1dba
1 1dbc 1dc1 1dc5 1dc9 1dcc 1dd1 1dd6
1dd9 1ddd 1dde 1de3 1de6 1deb 1dec 1df1
1df4 1df9 1dfa 1dff 1e02 1e06 1e07 1e0c
1e0f 1e14 1e15 1e1a 1e1d 1e21 1e25 1e28
1e2b 1e2c 1e2e 1e2f 1e34 1e35 1e3a 1e3c
1e40 1e43 1e45 1e49 1e4b 1e57 1e59 1e5b
1e5c 1e61 1e65 1e67 1e73 1e75 1e77 1e7b
1e7f 1e83 1e87 1e8b 1e8f 1e93 1e97 1ea3
1ea7 1eac 1eb0 1eb3 1eb4 1eb9 1ebd 1ec1
1ec4 1ec7 1ecb 1ecd 1ed1 1ed5 1ed8 1edb
1edf 1ee1 1ee5 1ee9 1eec 1eee 1ef2 1ef6
1efa 1efd 1f01 1f05 1f07 1f08 1f0d 1
1f11 1f15 1f19 1f1c 1f20 1f24 1f28 1f2c
1f2f 1f33 1f37 1f3a 1f3d 1f42 1f45 1f48
1f4b 1f4c 1f51 1f52 1 1f54 1f59 1f5d
1f61 1f64 1f69 1f6e 1f71 1f75 1f76 1f7b
1f7e 1f83 1f84 1f89 1f8c 1f91 1f92 1f97
1f9a 1f9e 1f9f 1fa4 1fa7 1fac 1fad 1fb2
1fb5 1fb9 1fbd 1fc0 1fc3 1fc4 1fc6 1fc7
1fcc 1fcd 1fd2 1fd4 1fd8 1fdb 1fdd 1fe1
1fe3 1fef 1ff1 1ff3 1ff4 1ff9 1ffd 1fff
200b 200d 200f 2013 2017 201a 201e 2022
2026 202a 202b 202d 2031 2033 2037 203a
203c 2040 2043 2045 2049 2050 2054 2058
205d 205f 2063 2067 206a 206e 2072 2074
2078 207c 2080 2083 2087 208b 208d 208e
2093 1 2097 209b 209f 20a2 20a6 20aa
20ae 20b2 20b5 20b9 20bd 20c0 20c3 20c8
20cb 20ce 20d1 20d2 20d7 20d8 1 20da
20df 20e3 20e7 20ea 20ef 20f4 20f7 20fb
20fc 2101 2104 2109 210a 210f 2112 2117
2118 211d 2120 2124 2125 212a 212d 2132
2133 2138 213b 213f 2143 2146 2149 214a
214c 214d 2152 2153 2158 215a 215e 2161
2163 2167 2169 2175 2177 2179 217a 217f
2183 2185 2191 2193 2195 2199 219d 21a0
21a4 21a8 21ac 21b0 21b4 21b8 21bc 21c0
21c4 21c8 21cc 21d0 21d4 21d8 21dc 21e0
21e4 21e8 21ec 21f0 21fc 2200 2205 2209
220c 220d 2212 2216 221a 221d 2220 2224
2226 222a 222e 2231 2234 2238 223a 223e
2242 2245 2247 224b 224f 2253 2256 225a
225e 2260 2261 2266 1 226a 226e 2272
2275 2279 227d 2281 2285 2288 228c 2290
2293 2296 229b 229e 22a1 22a4 22a5 22aa
22ab 1 22ad 22b2 22b6 22ba 22bd 22c2
22c7 22ca 22ce 22cf 22d4 22d7 22dc 22dd
22e2 22e5 22ea 22eb 22f0 22f3 22f7 22f8
22fd 2300 2305 2306 230b 230e 2312 2316
2319 231c 231d 231f 2320 2325 2326 232b
232d 2331 2334 2336 233a 233c 2348 234a
234c 234d 2352 2356 2358 2364 2366 236e
2370 2371 2374 2376 237a 2381 2385 2389
238e 2390 2394 2398 239b 239f 23a3 23a5
23a9 23ad 23b1 23b4 23b8 23bc 23be 23bf
23c4 1 23c8 23cc 23d0 23d3 23d7 23db
23df 23e3 23e6 23ea 23ee 23f1 23f4 23f9
23fc 23ff 2402 2403 2408 2409 1 240b
2410 2414 2418 241b 2420 2425 2428 242c
242d 2432 2435 243a 243b 2440 2443 2448
2449 244e 2451 2455 2456 245b 245e 2463
2464 2469 246c 2470 2474 2477 247a 247b
247d 247e 2483 2484 2489 248b 248f 2492
2494 2498 249a 24a6 24a8 24aa 24ab 24b0
24b4 24b6 24c2 24c4 24c8 24cc 24cf 24d0
24d2 24d5 24d8 24db 24dc 24e1 24e2 24e7
24eb 24ef 24f3 24f4 24f6 24fa 24fc 2500
2503 2505 2509 250c 2510 2514 2517 251b
251f 2522 2525 252a 252d 2530 2533 2534
2539 253a 1 253c 2541 2545 2549 254c
2551 2556 2559 255d 255e 2563 2566 256b
256c 2571 2574 2579 257a 257f 2582 2586
258a 258e 2592 2595 2596 2598 259d 259e
25a0 25a1 25a6 25a9 25ae 25af 25b4 25b7
25bc 25bd 25c2 25c5 25c9 25ca 25cf 25d2
25d7 25d8 25dd 25de 25e3 25e5 25e9 25ec
25f0 25f4 25f7 25fb 25ff 2602 2606 260a
260b 2610 2612 2616 261a 261d 2621 2625
2628 262c 2630 2633 2636 263b 263e 2641
2644 2645 264a 264b 1 264d 2652 2656
265a 265d 2662 2667 266a 266e 266f 2674
2675 267a 267e 2682 2685 268a 268f 2692
2696 269a 269d 269e 26a3 26a6 26ab 26ac
26b1 26b4 26b8 26bc 26bf 26c0 26c5 26c6
26cb 26cd 26d1 26d4 26d8 26dc 26e0 26e4
26e8 26ec 26f0 26f4 26f8 2704 2709 270d
2711 2715 2719 271b 271c 2721 1 2725
2729 272d 2730 2734 2738 273c 2740 2743
2747 274b 274e 2751 2756 2759 275c 275f
2760 2765 2766 1 2768 276d 2771 2775
2778 277d 2782 2785 2789 278a 278f 2790
2795 2799 279d 27a0 27a5 27aa 27ad 27b1
27b2 27b7 27ba 27bf 27c0 27c5 27c8 27cc
27d0 27d3 27d6 27d7 27d9 27da 27df 27e0
27e5 27e7 27eb 27ee 27f2 27f6 27fa 27fe
2802 2806 280a 2816 281b 281f 2823 2827
282b 282d 2831 2833 283f 2841 2843 2844
2849 284d 284f 285b 285f 2861 2862 286b 
b48
2
0 1 a 6 9 :3 6 11 :3 6
12 :2 6 1 3 a 13 a :2 1
4 :3 12 :2 4 11 1a 19 :2 11 :2 4
b 14 13 :2 b :2 4 :3 11 :2 4 e
17 16 :2 e :2 4 18 21 20 :2 18
:2 4 f 18 17 :2 f :2 4 :2 e 1d
e :2 4 :2 d 1c d :2 4 :2 b 1a
b :2 4 :3 e :2 4 :2 14 23 14 :2 4
:2 f 1e f :2 4 :2 15 24 15 :2 4
:2 12 21 12 :2 4 :2 11 20 11 :2 4
:2 12 21 12 :2 4 :2 13 22 13 :2 4
:2 1a 29 1a :2 4 f 18 17 :2 f
:2 4 f 18 17 :2 f :2 4 :2 f 1e
f :2 4 :2 11 20 11 :2 4 :2 12 2a
12 :2 4 :3 17 4 7 e 0 :2 7
11 1d 2a 14 30 41 14 31
43 f 10 1d 2e 38 43 4f
58 60 69 77 80 8e 97 a5
ae bb c4 d5 de f1 fa 10f
118 133 13c 154 15d 16e 177 189
192 1aa 1b3 1c6 1cf 11 1e 38
45 54 64 e 2c a :6 7 e
0 :2 7 11 24 2e 34 3f 4d
5a f 10 1d e 18 11 1c
34 3f 4c 5c e 2c a :5 7
4 e 17 :3 e 4 7 :2 11 23
:2 2d 3a :2 23 :3 7 :2 d 19 25 :2 2f
43 :2 25 57 5d 60 :2 57 6e 87
8 :2 6e d 10 :2 6e 13 16 :2 6e
b e :2 6e 13 16 1f 27 :2 1f
34 :2 16 :2 6e b e :2 6e 13 16
:2 6e 1b 1e :2 6e 2a 2d :2 6e 32
35 :2 6e :2 7 4d :2 4 7 :2 11 23
:2 2d 39 3c 4f 51 52 :2 4f :2 23
:3 7 :2 d 12 1e 60 63 :2 1e :2 7
55 :3 4 11 :2 4 12 :2 4 12 :2 4
12 4 e 1a 20 27 c 1a
22 c 1a 27 :2 7 :2 10 1c 7
4 c a :2 13 1f a 1a :2 7
:2 c a :2 13 1f a d :2 17 29
:2 33 3f 43 56 58 59 :2 56 :2 29
:3 d :2 13 18 24 66 69 :2 24 76
79 :2 24 7d 80 :2 24 8c 8f :2 24
97 9a :2 24 a6 a9 b0 b9 bc
:2 a9 :2 24 :2 d 5c :2 a :2 7 :4 13 :2 7
4 :3 1 8 :2 11 1c 1a :2 25 :2 1a
7 :4 3b 3a 39 :3 7 11 18 1f
22 :2 18 27 2a :2 11 :2 7 13 14
:2 13 7 11 28 37 45 52 :2 a
:2 13 1f a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 59 5c
5d :2 59 :2 2c :3 10 :2 16 1b 27 6a
6d :2 27 7a 7d :2 27 81 84 :2 27
90 93 :2 27 9b 9e :2 27 aa ad
b4 bd c0 :2 ad :2 27 :2 10 60 :2 d
:2 a :4 16 :2 a 7 :3 50 7 c :4 7
10 1d 28 32 40 :3 a d 15
:2 d :2 16 22 d 1b d :2 16 22
d :4 a e :2 17 23 20 :2 2c :2 20
:3 d 3c :2 a d 16 20 :2 d 23
25 :2 23 10 19 24 :2 10 27 29
:2 27 1a :3 13 19 26 37 44 13
1a 16 22 :3 20 16 :2 1f 2b 16
27 16 :2 1f 2b 16 :4 13 10 18
16 :2 1f 2b 16 26 :2 13 :2 18 16
:2 1f 2b 16 19 :2 23 35 :2 3f 4b
4f 62 65 66 :2 62 :2 35 :3 19 :2 1f
24 30 73 76 :2 30 83 86 :2 30
8a 8d :2 30 99 9c :2 30 a4 a7
:2 30 b3 b6 bd c6 c9 :2 b6 :2 30
:2 19 69 :2 16 :2 13 :4 1f :2 13 10 :3 2b
10 20 :2 10 29 38 44 4f 5b
67 29 37 :2 67 :2 29 10 2b 15
10 :4 d 27 d 12 :4 d 16 23
38 44 50 5e 6d :3 10 13 1b
:2 13 :2 1c 28 13 21 13 :2 1c 28
13 :4 10 14 :2 1d 29 26 :2 32 :2 26
:3 13 42 :3 10 24 10 13 1c 27
:2 13 2a 2c :2 2a 16 1d 29 2c
:2 16 2f 31 :2 2f 20 1d 1f 33
1d 32 1d 27 30 19 20 1c
28 :3 26 1c :2 25 31 1c 2d 1c
:2 25 31 1c :4 19 16 1e 1c :2 25
31 1c 2c :2 19 :2 1e 1c :2 25 31
1c 1f :2 29 3b :2 45 51 55 68
6b 6c :2 68 :2 3b :3 1f :2 25 2a 36
79 7c :2 36 89 8c :2 36 90 93
:2 36 9f a2 :2 36 aa ad :2 36 b9
bc c3 cc cf :2 bc :2 36 :2 1f 6f
:2 1c :2 19 :4 25 :2 19 16 :3 35 19 22
2e :2 19 31 33 34 :2 33 :2 31 20
21 22 36 19 23 1c 32 1c
32 1c 32 1c 32 1c 32 1c
32 1c 32 1c 32 1c 32 3f
1c 22 36 1c 23 1f 2b :3 29
1f :2 28 34 1f 30 1f :2 28 34
1f :4 1c 19 21 1f :2 28 34 1f
2f :2 1c :2 21 1f :2 28 34 1f 22
:2 2c 3e :2 48 54 58 6b 6e 6f
:2 6b :2 3e :3 22 :2 28 2d 39 7c 7f
:2 39 8c 8f :2 39 93 96 :2 39 a2
a5 :2 39 ad b0 :2 39 bc bf c6
cf d2 :2 bf :2 39 :2 22 72 :2 1f :2 1c
:4 28 :2 1c 19 :4 36 23 1c 29 1c
24 31 22 36 1c 23 1f 2b
:3 29 1f :2 28 34 1f 30 1f :2 28
34 1f :4 1c 19 21 1f :2 28 34
1f 2f :2 1c :2 21 1f :2 28 34 1f
22 :2 2c 3e :2 48 54 58 6b 6e
6f :2 6b :2 3e :3 22 :2 28 2d 39 7c
7f :2 39 8c 8f :2 39 93 96 :2 39
a2 a5 :2 39 ad b0 :2 39 bc bf
c6 cf d2 :2 bf :2 39 :2 22 72 :2 1f
:2 1c :4 28 :2 1c 19 :7 16 35 20 19
23 :3 19 1f 33 19 20 1c 28
:3 26 1c :2 25 31 1c 2d 1c :2 25
31 1c :4 19 16 1e 1c :2 25 31
1c 2c :2 19 :2 1e 1c :2 25 31 1c
1f :2 29 3b :2 45 51 55 68 6b
6c :2 68 :2 3b :3 1f :2 25 2a 36 79
7c :2 36 89 8c :2 36 90 93 :2 36
9f a2 :2 36 aa ad :2 36 b9 bc
c3 cc cf :2 bc :2 36 :2 1f 6f :2 1c
:2 19 :4 25 :2 19 16 :7 13 2e 16 1f
2a :2 16 2d 2f :2 2d 20 :3 19 1f
33 19 20 1c 28 :3 26 1c :2 25
31 1c 2d 1c :2 25 31 1c :4 19
16 1e 1c :2 25 31 1c 2c :2 19
:2 1e 1c :2 25 31 1c 1f :2 29 3b
:2 45 51 55 68 6b 6c :2 68 :2 3b
:3 1f :2 25 2a 36 79 7c :2 36 89
8c :2 36 90 93 :2 36 9f a2 :2 36
aa ad :2 36 b9 bc c3 cc cf
:2 bc :2 36 :2 1f 6f :2 1c :2 19 :4 25 :2 19
16 :4 31 19 20 2c 2f :2 19 32
34 :2 32 23 20 22 36 20 35
20 2a 33 1c 23 1f 2b :3 29
1f :2 28 34 1f 30 1f :2 28 34
1f :4 1c 19 21 1f :2 28 34 1f
2f :2 1c :2 21 1f :2 28 34 1f 22
:2 2c 3e :2 48 54 58 6b 6e 6f
:2 6b :2 3e :3 22 :2 28 2d 39 7c 7f
:2 39 8c 8f :2 39 93 96 :2 39 a2
a5 :2 39 ad b0 :2 39 bc bf c6
cf d2 :2 bf :2 39 :2 22 72 :2 1f :2 1c
:4 28 :2 1c 19 :3 38 23 :4 1c 33 43
54 67 1c 2c 3c 4d 5d 22
36 1c 23 1f 2b :3 29 1f :2 28
34 1f 30 1f :2 28 34 1f :4 1c
19 21 1f :2 28 34 1f 2f :2 1c
:2 21 1f :2 28 34 1f 22 :2 2c 3e
:2 48 54 58 6b 6e 6f :2 6b :2 3e
:3 22 :2 28 2d 39 7c 7f :2 39 8c
8f :2 39 93 96 :2 39 a2 a5 :2 39
ad b0 :2 39 bc bf c6 cf d2
:2 bf :2 39 :2 22 72 :2 1f :2 1c :4 28 :2 1c
19 :4 38 23 :4 1c 22 30 22 36
1c 23 1f 2b :3 29 1f :2 28 34
1f 30 1f :2 28 34 1f :4 1c 19
21 1f :2 28 34 1f 2f :2 1c :2 21
1f :2 28 34 1f 22 :2 2c 3e :2 48
54 58 6b 6e 6f :2 6b :2 3e :3 22
:2 28 2d 39 7c 7f :2 39 8c 8f
:2 39 93 96 :2 39 a2 a5 :2 39 ad
b0 :2 39 bc bf c6 cf d2 :2 bf
:2 39 :2 22 72 :2 1f :2 1c :4 28 :2 1c 19
:7 16 :4 13 :4 10 14 :2 1d 28 26 :2 31
:2 26 :2 13 27 13 41 :3 10 20 :2 10
29 38 44 59 65 2b 34 42
:2 2b :2 29 10 13 1c 2e :2 13 31
33 :2 31 18 1f 28 33 :2 1f 37
:2 18 17 3b 3d :2 3b 16 45 4e
5e :2 45 61 63 :2 61 44 :3 16 2c
34 3d 47 :2 3d 57 :2 34 5a 5c
28 32 :2 28 41 :2 5c :2 34 33 45
47 :2 33 32 51 53 28 38 :2 53
:2 32 :2 2c 16 1a 2d 2f :2 2d 19
35 48 4a 4b :2 4a :2 48 :2 19 23
1c 2a 33 3a 46 4f 1c 21
3b 48 21 22 36 1c 23 1f
2b :3 29 1f :2 28 34 1f 30 1f
:2 28 34 1f :4 1c 19 21 1f :2 28
34 1f 2f :2 1c :2 21 1f :2 28 34
1f 22 :2 2c 3e :2 48 54 58 6b
6e 6f :2 6b :2 3e :3 22 :2 28 2d 39
7c 7f :2 39 8c 8f :2 39 93 96
:2 39 a2 a5 :2 39 ad b0 :2 39 bc
bf c6 cf d2 :2 bf :2 39 :2 22 72
:2 1f :2 1c :4 28 :2 1c 19 :4 4d 23 1c
37 52 5f 71 22 36 1c 23
1f 2b :3 29 1f :2 28 34 1f 30
1f :2 28 34 1f :4 1c 19 21 1f
:2 28 34 1f 2f :2 1c :2 21 1f :2 28
34 1f 22 :2 2c 3e :2 48 54 58
6b 6e 6f :2 6b :2 3e :3 22 :2 28 2d
39 7c 7f :2 39 8c 8f :2 39 93
96 :2 39 a2 a5 :2 39 ad b0 :2 39
bc bf c6 cf d2 :2 bf :2 39 :2 22
72 :2 1f :2 1c :4 28 :2 1c 19 :8 16 29
3b 49 :2 29 16 66 :2 13 35 :2 10
d 11 a 10 16 :3 10 :2 19 25
10 d 15 13 :2 1c 28 13 23
:2 10 :2 15 13 :2 1c 28 13 16 :2 20
32 :2 3c 48 4c 5f 62 63 :2 5f
:2 32 :3 16 :2 1c 21 2d 70 73 :2 2d
80 83 :2 2d 87 8a :2 2d 96 99
:2 2d a1 a4 :2 2d b0 b3 ba c3
c6 :2 b3 :2 2d :2 16 66 :2 13 :2 10 :4 1c
:2 10 d :7 a 14 d 12 1a 21
39 41 48 d 12 1a 21 39
41 48 60 13 20 31 3e d
14 10 1c :3 1a 10 :2 19 25 10
21 10 :2 19 25 10 :4 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 5c 5f 60 :2 5c :2 2f :3 13 :2 19
1e 2a 6d 70 :2 2a 7d 80 :2 2a
84 87 :2 2a 93 96 :2 2a 9e a1
:2 2a ad b0 b7 c0 c3 :2 b0 :2 2a
:2 13 63 :2 10 :2 d :4 19 :2 d a :3 7
c a :3 7 b 50 a 10 :3 a
:2 13 1f a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 59 5c
5d :2 59 :2 2c :3 10 :2 16 1b 27 6a
6d :2 27 7a 7d :2 27 81 84 :2 27
90 93 :2 27 9b 9e :2 27 aa ad
b4 bd c0 :2 ad :2 27 :2 10 60 :2 d
:2 a :4 16 :2 a 7 :3 50 a 13 1d
:2 a 20 22 23 :2 22 :2 20 a 1d
27 :2 1d a 25 :2 7 50 :2 4 7
:2 11 23 :2 2d 39 3c 4f 52 53
:2 4f :2 23 :3 7 :2 d 12 1e 61 64
:2 1e 71 74 :2 1e 78 7b :2 1e 85
88 91 99 :2 a2 :2 91 ad :2 88 :2 1e
b5 b8 :2 1e c0 c3 :2 1e c8 cb
:2 1e ce d1 :2 1e :2 7 56 :3 4 :2 f
21 :2 2a 34 38 :2 4 1 9 12
9 a :2 14 26 :2 30 3c 40 53
56 57 :2 53 :2 26 :3 a :2 10 15 21
64 67 :2 21 :3 a :2 10 15 39 5c
5f :2 68 :2 39 76 79 :2 39 81 84
:2 8d :2 39 :2 a 5a :3 7 c 15 1e
2c a 13 1e f e 7 e
7 e 7 29 :2 4 :2 9 7 :2 10
1c :2 7 :2 11 23 :2 2d 39 3d 50
53 54 :2 50 :2 23 :3 7 :2 d 12 1e
61 64 :2 1e :3 7 :2 d 12 36 42
45 :2 36 4d 50 :2 36 5c 5f 66
6f 72 :2 5f :2 36 :2 7 57 :2 4 7
c 1b a 13 1e f e 7
e 7 e 7 :2 4 :4 10 :2 4 :8 1

b48
4
0 :2 1 :4 3 :4 4
:4 5 2 :4 6 :2 1
:5 8 :7 9 :7 a :5 b
:7 c :7 d :7 e :6 f
:6 10 :6 11 :5 12 :6 13
:6 14 :6 15 :6 16 :6 17
:6 18 :6 19 :6 1a :7 1b
:7 1c :6 1d :6 1e :6 1f
:5 20 :2 22 0 :2 22
:3 23 :3 24 :3 25 26
:23 27 :6 28 :2 29 23
:5 22 :2 2b 0 :2 2b
:7 2c 2d :2 2e :2 2f
:6 30 :2 31 2c :5 2b
:7 33 :b 35 :11 36 37
:2 36 :2 37 :2 36 :2 37
:2 36 :2 38 :2 36 :9 38
:2 36 :2 39 :2 36 :2 39
:2 36 :2 39 :2 36 :2 39
:2 36 :2 39 :4 36 :3 35
:11 3c :b 3d :3 3c :3 43
:3 46 :3 47 :3 48 :4 4b
:3 4c :3 4d 4b :5 4e
4a 50 :5 51 :3 50
:2 52 :5 54 :11 55 :24 56
:3 55 :2 53 :6 52 4f
:3 34 :12 5c :c 5d :6 5e
:6 61 :5 62 60 64
:5 65 :3 64 :2 66 :5 68
:11 69 :24 6a :3 69 :2 67
:6 66 63 :3 5c :5 6f
71 :8 72 :3 73 :5 74
73 :5 76 75 :3 73
:a 79 :2 7a :3 79 :9 7e
:9 80 83 84 85
86 :4 87 83 :6 88
:5 89 88 :5 8b 8a
:3 88 82 8e :5 8f
:3 8e :2 90 :5 92 :11 93
:24 94 :3 93 :2 91 :6 90
8d :3 80 :3 9a :7 9b
:2 9c :5 9b 80 :2 9e
9d :3 80 7e :5 a3
a5 :a a6 :3 a7 :5 a8
a7 :5 aa a9 :3 a7
:a ad :2 ae :3 ad :3 b2
:9 b3 :a b4 b6 b7
:2 b8 :2 b9 :3 ba b6
:6 bb :5 bc bb :5 be
bd :3 bb b5 c1
:5 c2 :3 c1 :2 c3 :5 c5
:11 c6 :24 c7 :3 c6 :2 c4
:6 c3 c0 :3 b4 :c cc
cd ce :2 cf cd
d2 :2 d3 :2 d4 :2 d5
:2 d6 :2 d7 :2 d8 :2 d9
:2 da :3 db dc :2 dd
d2 :6 de :5 df de
:5 e1 e0 :3 de d1
e4 :5 e5 :3 e4 :2 e6
:5 e8 :11 e9 :24 ea :3 e9
:2 e7 :6 e6 e3 :4 cc
f8 :2 f9 :3 fa :2 fb
f8 :6 fc :5 fd fc
:5 ff fe :3 fc f7
102 :5 103 :3 102 :2 104
:5 106 :11 107 :24 108 :3 107
:2 105 :6 104 101 :4 ee
:3 cc b4 10f :2 110
111 112 113 :2 114
10f :6 115 :5 116 115
:5 118 117 :3 115 10e
11b :5 11c :3 11b :2 11d
:5 11f :11 120 :24 121 :3 120
:2 11e :6 11d 11a :4 10d
:3 b4 b3 :9 128 12b
12c 12d 12e :2 12f
12b :6 130 :5 131 130
:5 133 132 :3 130 12a
136 :5 137 :3 136 :2 138
:5 13a :11 13b :24 13c :3 13b
:2 139 :6 138 135 :4 128
:a 142 144 145 :2 146
:2 147 :3 148 144 :6 149
:5 14a 149 :5 14c 14b
:3 149 143 14f :5 150
:3 14f :2 151 :5 153 :11 154
:24 155 :3 154 :2 152 :6 151
14e :3 142 15b 15c
15d 15e :5 15f :5 160
:2 162 15b :6 163 :5 164
163 :5 166 165 :3 163
15a 169 :5 16a :3 169
:2 16b :5 16d :11 16e :24 16f
:3 16e :2 16c :6 16b 168
:4 142 175 176 177
178 :3 179 :2 17a 175
:6 17b :5 17c 17b :5 17e
17d :3 17b 174 181
:5 182 :3 181 :2 183 :5 185
:11 186 :24 187 :3 186 :2 184
:6 183 180 :4 173 :3 142
140 :3 128 127 :3 b3
:a 18f :3 190 :3 18f :3 194
:6 195 :5 196 :3 195 :9 199
:1b 19a :c 19b :5 19c :5 19b
:2 19c :3 19b :2 19c :2 19d
:2 19c :5 19b :10 19e 1a0
:6 1a1 1a2 :3 1a3 1a4
:2 1a6 1a0 :6 1a7 :5 1a8
1a7 :5 1aa 1a9 :3 1a7
19f 1ad :5 1ae :3 1ad
:2 1af :5 1b1 :11 1b2 :24 1b3
:3 1b2 :2 1b0 :6 1af 1ac
:4 19e 1b9 :5 1ba :2 1bb
1b9 :6 1bc :5 1bd 1bc
:5 1bf 1be :3 1bc 1b8
1c2 :5 1c3 :3 1c2 :2 1c4
:5 1c6 :11 1c7 :24 1c8 :3 1c7
:2 1c5 :6 1c4 1c1 :4 1b7
:3 19e :7 1d1 :3 19a :3 199
a5 1d4 a0 :4 1d8
:5 1d9 1d7 1db :5 1dc
:3 1db :2 1dd :5 1df :11 1e0
:24 1e1 :3 1e0 :2 1de :6 1dd
1da :4 a0 :3 7e 1e9
1ea :6 1ec 1ef :7 1f1
:4 1f4 1e9 :6 1f5 :5 1f6
1f5 :5 1f8 1f7 :3 1f5
1e8 1fb :5 1fc :3 1fb
:2 1fd :5 1ff :11 200 :24 201
:3 200 :2 1fe :6 1fd 1fa
:3 71 205 206 :3 71
207 5c :4 20b :5 20c
20a 20e :5 20f :3 20e
:2 210 :5 212 :11 213 :24 214
:3 213 :2 211 :6 210 20d
:3 5c :c 219 :6 21a :3 219
:3 5c :11 21f :30 220 :3 21f
:a 222 34 :3 225 :11 226
:b 227 :17 228 :3 226 :2 22a
:3 22b :3 22c 22d 22b
22a 22b :3 22e :3 225
:2 22f :5 231 :11 232 :b 233
:18 234 :3 232 :2 236 237
:3 238 239 237 236
237 :3 23a :2 230 :6 22f
224 :7 1 
286d
4
:3 0 1 :3 0 2
:a 0 b43 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
:2 0 7 6 :3 0
5 :7 0 9 8
:3 0 4 :3 0 7
:7 0 d c :3 0
8 :3 0 9 :3 0
a :2 0 4 11
12 0 f 13
0 b43 2 14
:2 0 e :2 0 d
c :3 0 17 :7 0
1a 18 0 b41
0 b :6 0 10
:2 0 11 4 :3 0
f 1c 1e :6 0
21 1f 0 b41
0 d :6 0 17
c3 0 15 4
:3 0 13 23 25
:6 0 28 26 0
b41 0 f :6 0
e :2 0 1b c
:3 0 2a :7 0 2d
2b 0 b41 0
11 :6 0 4 :3 0
e :2 0 19 2f
31 :6 0 34 32
0 b41 0 12
:6 0 15 :2 0 1f
4 :3 0 1d 36
38 :6 0 3b 39
0 b41 0 13
:6 0 17 :2 0 23
4 :3 0 21 3d
3f :6 0 42 40
0 b41 0 14
:6 0 17 :2 0 25
6 :3 0 44 :7 0
48 45 46 b41
0 16 :6 0 17
:2 0 27 6 :3 0
4a :7 0 4e 4b
4c b41 0 18
:6 0 2b 189 0
29 6 :3 0 50
:7 0 54 51 52
b41 0 19 :6 0
17 :2 0 2d 1b
:3 0 56 :7 0 59
57 0 b41 0
1a :6 0 6 :3 0
5b :7 0 17 :2 0
5f 5c 5d b41
0 1c :6 0 17
:2 0 2f 6 :3 0
61 :7 0 65 62
63 b41 0 1d
:6 0 17 :2 0 31
6 :3 0 67 :7 0
6b 68 69 b41
0 1e :6 0 17
:2 0 33 6 :3 0
6d :7 0 71 6e
6f b41 0 1f
:6 0 17 :2 0 35
6 :3 0 73 :7 0
77 74 75 b41
0 20 :6 0 17
:2 0 37 6 :3 0
79 :7 0 7d 7a
7b b41 0 21
:6 0 17 :2 0 39
6 :3 0 7f :7 0
83 80 81 b41
0 22 :6 0 e
:2 0 3b 6 :3 0
85 :7 0 89 86
87 b41 0 23
:6 0 e :2 0 3f
4 :3 0 3d 8b
8d :6 0 90 8e
0 b41 0 24
:6 0 17 :2 0 43
4 :3 0 41 92
94 :6 0 97 95
0 b41 0 25
:6 0 17 :2 0 45
6 :3 0 99 :7 0
9d 9a 9b b41
0 26 :6 0 17
:2 0 47 6 :3 0
9f :7 0 a3 a0
a1 b41 0 27
:6 0 4b 312 0
49 6 :3 0 a5
0 2 :5 0 a9
a6 a7 b41 0
28 :6 0 2a :3 0
6 :3 0 ab :7 0
ae ac 0 b41
0 29 :6 0 2b
:a 0 2 ea :5 0
b0 b3 0 b1
:3 0 2c :3 0 2d
:3 0 2e :3 0 2f
:3 0 30 :3 0 2f
:3 0 31 :3 0 32
:3 0 31 :3 0 33
:3 0 34 :3 0 d
:3 0 35 :3 0 f
:3 0 36 :3 0 37
:3 0 38 :3 0 37
:3 0 39 :3 0 37
:3 0 3a :3 0 37
:3 0 3b :3 0 37
:3 0 3c :3 0 37
:3 0 3d :3 0 37
:3 0 3e :3 0 37
:3 0 3f :3 0 37
:3 0 40 :3 0 37
:3 0 41 :3 0 37
:3 0 42 :3 0 37
:3 0 43 :3 0 37
:3 0 44 :3 0 37
:3 0 45 :3 0 37
:3 0 46 :3 0 2d
:3 0 2e :3 0 2d
:3 0 2e :3 0 47
:3 0 48 :3 0 2f
:3 0 31 :4 0 49
1 :8 0 eb b0
b3 ec 0 b41
4d ec ee eb
ed :6 0 ea :7 0
ec 2a :3 0 4a
:a 0 3 109 :5 0
f0 f3 0 f1
:3 0 4b :3 0 4c
:3 0 4d :3 0 4e
:3 0 4f :3 0 47
:3 0 50 :3 0 51
:3 0 34 :3 0 d
:3 0 52 :3 0 12
:3 0 4e :3 0 4f
:3 0 4e :3 0 4f
:3 0 47 :3 0 48
:3 0 2f :3 0 31
:4 0 53 1 :8 0
10a f0 f3 10b
0 b41 4f 10b
10d 10a 10c :6 0
109 :7 0 10b 115
116 0 51 9
:3 0 a :2 0 4
10f 110 0 111
:7 0 114 112 0
b41 0 54 :6 0
55 :3 0 56 :3 0
55 :3 0 57 :3 0
118 119 0 2
:4 0 53 11a 11c
117 11e 11d :2 0
58 :3 0 59 :3 0
120 121 0 5a
:4 0 55 :3 0 5b
:3 0 124 125 0
2 :4 0 55 126
128 5c :4 0 5d
:2 0 5e :3 0 57
12b 12d :3 0 5f
:4 0 5d :2 0 60
:4 0 5a 130 132
:3 0 5d :2 0 3
:3 0 5d 134 136
:3 0 5d :2 0 60
:4 0 60 138 13a
:3 0 5d :2 0 61
:4 0 63 13c 13e
:3 0 5d :2 0 62
:3 0 63 :3 0 5
:3 0 66 142 144
64 :4 0 68 141
147 6b 140 149
:3 0 5d :2 0 61
:4 0 6e 14b 14d
:3 0 5d :2 0 60
:4 0 71 14f 151
:3 0 5d :2 0 7
:3 0 74 153 155
:3 0 5d :2 0 60
:4 0 77 157 159
:3 0 5d :2 0 65
:4 0 7a 15b 15d
:3 0 7d 122 15f
:2 0 161 82 162
11f 161 0 163
84 0 aa1 55
:3 0 56 :3 0 164
165 0 55 :3 0
66 :3 0 167 168
0 15 :2 0 2
:4 0 67 :2 0 68
:2 0 15 :2 0 86
16d 16f :3 0 89
169 171 166 173
172 :2 0 58 :3 0
69 :3 0 175 176
0 5a :4 0 6a
:4 0 5d :2 0 5e
:3 0 8d 17a 17c
:3 0 90 177 17e
:2 0 180 93 181
174 180 0 182
95 0 aa1 16
:3 0 5 :3 0 183
184 0 aa1 25
:3 0 7 :3 0 186
187 0 aa1 26
:3 0 6b :2 0 189
18a 0 aa1 24
:3 0 6c :4 0 18c
18d 0 aa1 34
:3 0 6d :3 0 35
:3 0 36 :3 0 d
:3 0 f :3 0 19
:3 0 33 :3 0 2c
:3 0 3 :4 0 6e
1 :8 0 19f 9
:3 0 6f :3 0 19a
19b 0 70 :3 0
19c 19d 0 19f
97 1f2 71 :3 0
9 :3 0 6f :3 0
1a1 1a2 0 70
:3 0 1a3 1a4 0
1a6 9a 1a8 9c
1a7 1a6 :2 0 1f0
72 :3 0 9 :3 0
6f :3 0 1ab 1ac
0 70 :3 0 1ad
1ae 0 1e8 55
:3 0 56 :3 0 1b0
1b1 0 55 :3 0
66 :3 0 1b3 1b4
0 73 :2 0 2
:4 0 74 :2 0 68
:2 0 15 :2 0 9e
1b9 1bb :3 0 a1
1b5 1bd 1b2 1bf
1be :2 0 58 :3 0
69 :3 0 1c1 1c2
0 5a :4 0 75
:4 0 5d :2 0 5e
:3 0 a5 1c6 1c8
:3 0 5d :2 0 76
:4 0 a8 1ca 1cc
:3 0 5d :2 0 77
:4 0 ab 1ce 1d0
:3 0 5d :2 0 70
:3 0 ae 1d2 1d4
:3 0 5d :2 0 78
:4 0 b1 1d6 1d8
:3 0 5d :2 0 79
:3 0 7a :3 0 15
:2 0 7b :2 0 b4
1db 1df b8 1da
1e1 :3 0 bb 1c3
1e3 :2 0 1e5 be
1e6 1c0 1e5 0
1e7 c0 0 1e8
c2 1eb :3 0 1eb
0 1eb 1ea 1e8
1e9 :6 0 1ed 4
:3 0 c5 1ef c7
1ee 1ed :2 0 1f0
c9 :2 0 1f2 0
1f2 1f1 19f 1f0
:6 0 aa1 1 :3 0
9 :3 0 6f :3 0
1f4 1f5 0 9
:3 0 7c :2 0 7d
:3 0 1f7 1f9 0
ce 1f8 1fb :3 0
1fc :2 0 19 :3 0
7e :2 0 d1 1ff
200 :3 0 201 :2 0
202 :2 0 1fd 204
203 :2 0 f :3 0
79 :3 0 f :3 0
5d :2 0 7f :4 0
d3 209 20b :3 0
15 :2 0 10 :2 0
d6 207 20f 206
210 0 a50 18
:3 0 80 :2 0 15
:2 0 da 213 215
:3 0 212 216 0
a50 81 :3 0 1a
:3 0 33 :3 0 2c
:3 0 d :4 0 82
1 :8 0 223 9
:3 0 6f :3 0 21e
21f 0 70 :3 0
220 221 0 223
dc 276 71 :3 0
9 :3 0 6f :3 0
225 226 0 70
:3 0 227 228 0
22a df 22c e1
22b 22a :2 0 274
72 :3 0 9 :3 0
6f :3 0 22f 230
0 70 :3 0 231
232 0 26c 55
:3 0 56 :3 0 234
235 0 55 :3 0
66 :3 0 237 238
0 73 :2 0 2
:4 0 83 :2 0 68
:2 0 15 :2 0 e3
23d 23f :3 0 e6
239 241 236 243
242 :2 0 58 :3 0
69 :3 0 245 246
0 5a :4 0 84
:4 0 5d :2 0 5e
:3 0 ea 24a 24c
:3 0 5d :2 0 76
:4 0 ed 24e 250
:3 0 5d :2 0 77
:4 0 f0 252 254
:3 0 5d :2 0 70
:3 0 f3 256 258
:3 0 5d :2 0 78
:4 0 f6 25a 25c
:3 0 5d :2 0 79
:3 0 7a :3 0 15
:2 0 7b :2 0 f9
25f 263 fd 25e
265 :3 0 100 247
267 :2 0 269 103
26a 244 269 0
26b 105 0 26c
107 26f :3 0 26f
0 26f 26e 26c
26d :6 0 271 6
:3 0 10a 273 10c
272 271 :2 0 274
10e :2 0 276 0
276 275 223 274
:6 0 a50 1 :3 0
85 :3 0 2b :4 0
27b :2 0 a50 279
27c :3 0 86 :3 0
2b :3 0 12 :3 0
18 :3 0 11 :3 0
b :4 0 284 :2 0
9da 27e 285 :3 0
111 :3 0 2b :3 0
87 :3 0 286 287
:3 0 9 :3 0 6f
:3 0 289 28a 0
17 :2 0 28b 28c
0 28e 116 295
9 :3 0 6f :3 0
28f 290 0 88
:2 0 291 292 0
294 118 296 288
28e 0 297 0
294 0 297 11a
0 9da 9 :3 0
6f :3 0 298 299
0 9 :3 0 89
:2 0 7d :3 0 29b
29d 0 11f 29c
29f :3 0 2a0 :2 0
8a :8 0 2a4 122
2a5 2a1 2a4 0
2a6 124 0 9da
62 :3 0 18 :3 0
17 :2 0 126 2a7
2aa 7c :2 0 17
:2 0 12b 2ac 2ae
:3 0 62 :3 0 16
:3 0 17 :2 0 12e
2b0 2b3 7c :2 0
8b :2 0 133 2b5
2b7 :3 0 33 :3 0
2f :3 0 31 :3 0
8c :3 0 34 :3 0
d :3 0 2c :3 0
12 :4 0 8d 1
:8 0 2d7 87 :4 0
2c2 :3 0 8e :3 0
7c :2 0 138 2c5
2c6 :3 0 9 :3 0
6f :3 0 2c8 2c9
0 17 :2 0 2ca
2cb 0 2cd 13b
2d4 9 :3 0 6f
:3 0 2ce 2cf 0
88 :2 0 2d0 2d1
0 2d3 13d 2d5
2c7 2cd 0 2d6
0 2d3 0 2d6
13f 0 2d7 142
32a 71 :3 0 9
:3 0 6f :3 0 2d9
2da 0 70 :3 0
2db 2dc 0 2de
145 2e0 147 2df
2de :2 0 328 72
:3 0 9 :3 0 6f
:3 0 2e3 2e4 0
70 :3 0 2e5 2e6
0 320 55 :3 0
56 :3 0 2e8 2e9
0 55 :3 0 66
:3 0 2eb 2ec 0
73 :2 0 2 :4 0
8f :2 0 68 :2 0
15 :2 0 149 2f1
2f3 :3 0 14c 2ed
2f5 2ea 2f7 2f6
:2 0 58 :3 0 69
:3 0 2f9 2fa 0
5a :4 0 90 :4 0
5d :2 0 5e :3 0
150 2fe 300 :3 0
5d :2 0 76 :4 0
153 302 304 :3 0
5d :2 0 77 :4 0
156 306 308 :3 0
5d :2 0 70 :3 0
159 30a 30c :3 0
5d :2 0 78 :4 0
15c 30e 310 :3 0
5d :2 0 79 :3 0
7a :3 0 15 :2 0
7b :2 0 15f 313
317 163 312 319
:3 0 166 2fb 31b
:2 0 31d 169 31e
2f8 31d 0 31f
16b 0 320 16d
323 :3 0 323 0
323 322 320 321
:6 0 325 9 :3 0
170 327 172 326
325 :2 0 328 174
:2 0 32a 0 32a
329 2d7 328 :6 0
33d 8 :3 0 27
:3 0 15 :2 0 32c
32d 0 33d 23
:3 0 91 :3 0 24
:3 0 12 :3 0 25
:3 0 26 :3 0 62
:3 0 27 :3 0 17
:2 0 177 335 338
17a 330 33a 32f
33b 0 33d 180
341 92 :3 0 33e
:4 0 340 184 342
2b8 33d 0 343
0 340 0 343
186 0 344 189
954 85 :3 0 4a
:4 0 348 :2 0 953
346 349 :3 0 86
:3 0 4a :3 0 13
:3 0 14 :3 0 1d
:3 0 20 :3 0 21
:3 0 22 :4 0 353
:2 0 8f2 34b 354
:3 0 18b :3 0 4a
:3 0 87 :3 0 355
356 :3 0 9 :3 0
6f :3 0 358 359
0 17 :2 0 35a
35b 0 35d 192
364 9 :3 0 6f
:3 0 35e 35f 0
88 :2 0 360 361
0 363 194 365
357 35d 0 366
0 363 0 366
196 0 8f2 9
:3 0 6f :3 0 367
368 0 9 :3 0
89 :2 0 7d :3 0
36a 36c 0 19b
36b 36e :3 0 36f
:2 0 8a :8 0 373
19e 374 370 373
0 375 1a0 0
8f2 1e :3 0 17
:2 0 376 377 0
8f2 62 :3 0 16
:3 0 17 :2 0 1a2
379 37c 7c :2 0
15 :2 0 1a7 37e
380 :3 0 79 :3 0
14 :3 0 15 :2 0
15 :2 0 1aa 382
386 7c :2 0 8b
:4 0 1b0 388 38a
:3 0 93 :3 0 94
:3 0 4b :3 0 13
:3 0 4b :3 0 2c
:3 0 36 :3 0 37
:3 0 95 :4 0 96
1 :8 0 3ab 87
:4 0 396 :3 0 8e
:3 0 7c :2 0 1b5
399 39a :3 0 9
:3 0 6f :3 0 39c
39d 0 17 :2 0
39e 39f 0 3a1
1b8 3a8 9 :3 0
6f :3 0 3a2 3a3
0 88 :2 0 3a4
3a5 0 3a7 1ba
3a9 39b 3a1 0
3aa 0 3a7 0
3aa 1bc 0 3ab
1bf 3fe 71 :3 0
9 :3 0 6f :3 0
3ad 3ae 0 70
:3 0 3af 3b0 0
3b2 1c2 3b4 1c4
3b3 3b2 :2 0 3fc
72 :3 0 9 :3 0
6f :3 0 3b7 3b8
0 70 :3 0 3b9
3ba 0 3f4 55
:3 0 56 :3 0 3bc
3bd 0 55 :3 0
66 :3 0 3bf 3c0
0 73 :2 0 2
:4 0 97 :2 0 68
:2 0 15 :2 0 1c6
3c5 3c7 :3 0 1c9
3c1 3c9 3be 3cb
3ca :2 0 58 :3 0
69 :3 0 3cd 3ce
0 5a :4 0 98
:4 0 5d :2 0 5e
:3 0 1cd 3d2 3d4
:3 0 5d :2 0 76
:4 0 1d0 3d6 3d8
:3 0 5d :2 0 77
:4 0 1d3 3da 3dc
:3 0 5d :2 0 70
:3 0 1d6 3de 3e0
:3 0 5d :2 0 78
:4 0 1d9 3e2 3e4
:3 0 5d :2 0 79
:3 0 7a :3 0 15
:2 0 7b :2 0 1dc
3e7 3eb 1e0 3e6
3ed :3 0 1e3 3cf
3ef :2 0 3f1 1e6
3f2 3cc 3f1 0
3f3 1e8 0 3f4
1ea 3f7 :3 0 3f7
0 3f7 3f6 3f4
3f5 :6 0 3f9 c
:3 0 1ed 3fb 1ef
3fa 3f9 :2 0 3fc
1f1 :2 0 3fe 0
3fe 3fd 3ab 3fc
:6 0 50b b :3 0
62 :3 0 1d :3 0
17 :2 0 1f4 400
403 7c :2 0 80
:2 0 15 :2 0 1f7
406 408 :3 0 1fb
405 40a :3 0 51
:3 0 99 :3 0 4b
:3 0 13 :4 0 9a
1 :8 0 493 51
:3 0 9b :3 0 30
:3 0 9c :3 0 32
:3 0 9d :3 0 4e
:3 0 9e :3 0 30
:3 0 9f :3 0 32
:3 0 a0 :3 0 4e
:3 0 99 :3 0 a1
:3 0 4d :3 0 4e
:3 0 48 :3 0 4e
:3 0 47 :3 0 a2
:3 0 4b :3 0 13
:4 0 a3 1 :8 0
43e 87 :4 0 429
:3 0 8e :3 0 7c
:2 0 200 42c 42d
:3 0 9 :3 0 6f
:3 0 42f 430 0
17 :2 0 431 432
0 434 203 43b
9 :3 0 6f :3 0
435 436 0 88
:2 0 437 438 0
43a 205 43c 42e
434 0 43d 0
43a 0 43d 207
0 43e 20a 491
71 :3 0 9 :3 0
6f :3 0 440 441
0 70 :3 0 442
443 0 445 20d
447 20f 446 445
:2 0 48f 72 :3 0
9 :3 0 6f :3 0
44a 44b 0 70
:3 0 44c 44d 0
487 55 :3 0 56
:3 0 44f 450 0
55 :3 0 66 :3 0
452 453 0 73
:2 0 2 :4 0 a4
:2 0 68 :2 0 15
:2 0 211 458 45a
:3 0 214 454 45c
451 45e 45d :2 0
58 :3 0 69 :3 0
460 461 0 5a
:4 0 a5 :4 0 5d
:2 0 5e :3 0 218
465 467 :3 0 5d
:2 0 76 :4 0 21b
469 46b :3 0 5d
:2 0 77 :4 0 21e
46d 46f :3 0 5d
:2 0 70 :3 0 221
471 473 :3 0 5d
:2 0 78 :4 0 224
475 477 :3 0 5d
:2 0 79 :3 0 7a
:3 0 15 :2 0 7b
:2 0 227 47a 47e
22b 479 480 :3 0
22e 462 482 :2 0
484 231 485 45f
484 0 486 233
0 487 235 48a
:3 0 48a 0 48a
489 487 488 :6 0
48c e :3 0 238
48e 23a 48d 48c
:2 0 48f 23c :2 0
491 0 491 490
43e 48f :6 0 493
b :3 0 23f 508
51 :3 0 4e :3 0
4e :3 0 48 :3 0
4e :3 0 47 :3 0
4b :3 0 13 :4 0
a6 1 :8 0 4b2
87 :4 0 49d :3 0
8e :3 0 7c :2 0
244 4a0 4a1 :3 0
9 :3 0 6f :3 0
4a3 4a4 0 17
:2 0 4a5 4a6 0
4a8 247 4af 9
:3 0 6f :3 0 4a9
4aa 0 88 :2 0
4ab 4ac 0 4ae
249 4b0 4a2 4a8
0 4b1 0 4ae
0 4b1 24b 0
4b2 24e 505 71
:3 0 9 :3 0 6f
:3 0 4b4 4b5 0
70 :3 0 4b6 4b7
0 4b9 251 4bb
253 4ba 4b9 :2 0
503 72 :3 0 9
:3 0 6f :3 0 4be
4bf 0 70 :3 0
4c0 4c1 0 4fb
55 :3 0 56 :3 0
4c3 4c4 0 55
:3 0 66 :3 0 4c6
4c7 0 73 :2 0
2 :4 0 a7 :2 0
68 :2 0 15 :2 0
255 4cc 4ce :3 0
258 4c8 4d0 4c5
4d2 4d1 :2 0 58
:3 0 69 :3 0 4d4
4d5 0 5a :4 0
a8 :4 0 5d :2 0
5e :3 0 25c 4d9
4db :3 0 5d :2 0
76 :4 0 25f 4dd
4df :3 0 5d :2 0
77 :4 0 262 4e1
4e3 :3 0 5d :2 0
70 :3 0 265 4e5
4e7 :3 0 5d :2 0
78 :4 0 268 4e9
4eb :3 0 5d :2 0
79 :3 0 7a :3 0
15 :2 0 7b :2 0
26b 4ee 4f2 26f
4ed 4f4 :3 0 272
4d6 4f6 :2 0 4f8
275 4f9 4d3 4f8
0 4fa 277 0
4fb 279 4fe :3 0
4fe 0 4fe 4fd
4fb 4fc :6 0 500
10 :3 0 27c 502
27e 501 500 :2 0
503 280 :2 0 505
0 505 504 4b2
503 :6 0 507 b
:3 0 283 509 40b
493 0 50a 0
507 0 50a 285
0 50b 288 580
51 :3 0 48 :3 0
4f :3 0 50 :3 0
a2 :3 0 a9 :3 0
4b :3 0 13 :4 0
aa 1 :8 0 52a
87 :4 0 515 :3 0
8e :3 0 7c :2 0
28d 518 519 :3 0
9 :3 0 6f :3 0
51b 51c 0 17
:2 0 51d 51e 0
520 290 527 9
:3 0 6f :3 0 521
522 0 88 :2 0
523 524 0 526
292 528 51a 520
0 529 0 526
0 529 294 0
52a 297 57d 71
:3 0 9 :3 0 6f
:3 0 52c 52d 0
70 :3 0 52e 52f
0 531 29a 533
29c 532 531 :2 0
57b 72 :3 0 9
:3 0 6f :3 0 536
537 0 70 :3 0
538 539 0 573
55 :3 0 56 :3 0
53b 53c 0 55
:3 0 66 :3 0 53e
53f 0 73 :2 0
2 :4 0 ab :2 0
68 :2 0 15 :2 0
29e 544 546 :3 0
2a1 540 548 53d
54a 549 :2 0 58
:3 0 69 :3 0 54c
54d 0 5a :4 0
ac :4 0 5d :2 0
5e :3 0 2a5 551
553 :3 0 5d :2 0
76 :4 0 2a8 555
557 :3 0 5d :2 0
77 :4 0 2ab 559
55b :3 0 5d :2 0
70 :3 0 2ae 55d
55f :3 0 5d :2 0
78 :4 0 2b1 561
563 :3 0 5d :2 0
79 :3 0 7a :3 0
15 :2 0 7b :2 0
2b4 566 56a 2b8
565 56c :3 0 2bb
54e 56e :2 0 570
2be 571 54b 570
0 572 2c0 0
573 2c2 576 :3 0
576 0 576 575
573 574 :6 0 578
12 :3 0 2c5 57a
2c7 579 578 :2 0
57b 2c9 :2 0 57d
0 57d 57c 52a
57b :6 0 57f b
:3 0 2cc 581 38b
50b 0 582 0
57f 0 582 2ce
0 583 2d1 776
62 :3 0 16 :3 0
17 :2 0 2d3 584
587 7c :2 0 8b
:2 0 2d8 589 58b
:3 0 51 :3 0 2f
:3 0 31 :3 0 8c
:3 0 4b :3 0 13
:4 0 ad 1 :8 0
5a9 87 :4 0 594
:3 0 8e :3 0 7c
:2 0 2dd 597 598
:3 0 9 :3 0 6f
:3 0 59a 59b 0
17 :2 0 59c 59d
0 59f 2e0 5a6
9 :3 0 6f :3 0
5a0 5a1 0 88
:2 0 5a2 5a3 0
5a5 2e2 5a7 599
59f 0 5a8 0
5a5 0 5a8 2e4
0 5a9 2e7 5fc
71 :3 0 9 :3 0
6f :3 0 5ab 5ac
0 70 :3 0 5ad
5ae 0 5b0 2ea
5b2 2ec 5b1 5b0
:2 0 5fa 72 :3 0
9 :3 0 6f :3 0
5b5 5b6 0 70
:3 0 5b7 5b8 0
5f2 55 :3 0 56
:3 0 5ba 5bb 0
55 :3 0 66 :3 0
5bd 5be 0 73
:2 0 2 :4 0 ae
:2 0 68 :2 0 15
:2 0 2ee 5c3 5c5
:3 0 2f1 5bf 5c7
5bc 5c9 5c8 :2 0
58 :3 0 69 :3 0
5cb 5cc 0 5a
:4 0 af :4 0 5d
:2 0 5e :3 0 2f5
5d0 5d2 :3 0 5d
:2 0 76 :4 0 2f8
5d4 5d6 :3 0 5d
:2 0 77 :4 0 2fb
5d8 5da :3 0 5d
:2 0 70 :3 0 2fe
5dc 5de :3 0 5d
:2 0 78 :4 0 301
5e0 5e2 :3 0 5d
:2 0 79 :3 0 7a
:3 0 15 :2 0 7b
:2 0 304 5e5 5e9
308 5e4 5eb :3 0
30b 5cd 5ed :2 0
5ef 30e 5f0 5ca
5ef 0 5f1 310
0 5f2 312 5f5
:3 0 5f5 0 5f5
5f4 5f2 5f3 :6 0
5f7 14 :3 0 315
5f9 317 5f8 5f7
:2 0 5fa 319 :2 0
5fc 0 5fc 5fb
5a9 5fa :6 0 5fe
b :3 0 31c 772
79 :3 0 14 :3 0
15 :2 0 15 :2 0
31e 5ff 603 7c
:2 0 8b :4 0 324
605 607 :3 0 93
:3 0 94 :3 0 4b
:3 0 13 :3 0 4b
:3 0 2c :3 0 36
:3 0 37 :3 0 95
:4 0 b0 1 :8 0
628 87 :4 0 613
:3 0 8e :3 0 7c
:2 0 329 616 617
:3 0 9 :3 0 6f
:3 0 619 61a 0
17 :2 0 61b 61c
0 61e 32c 625
9 :3 0 6f :3 0
61f 620 0 88
:2 0 621 622 0
624 32e 626 618
61e 0 627 0
624 0 627 330
0 628 333 67b
71 :3 0 9 :3 0
6f :3 0 62a 62b
0 70 :3 0 62c
62d 0 62f 336
631 338 630 62f
:2 0 679 72 :3 0
9 :3 0 6f :3 0
634 635 0 70
:3 0 636 637 0
671 55 :3 0 56
:3 0 639 63a 0
55 :3 0 66 :3 0
63c 63d 0 73
:2 0 2 :4 0 b1
:2 0 68 :2 0 15
:2 0 33a 642 644
:3 0 33d 63e 646
63b 648 647 :2 0
58 :3 0 69 :3 0
64a 64b 0 5a
:4 0 b2 :4 0 5d
:2 0 5e :3 0 341
64f 651 :3 0 5d
:2 0 76 :4 0 344
653 655 :3 0 5d
:2 0 77 :4 0 347
657 659 :3 0 5d
:2 0 70 :3 0 34a
65b 65d :3 0 5d
:2 0 78 :4 0 34d
65f 661 :3 0 5d
:2 0 79 :3 0 7a
:3 0 15 :2 0 7b
:2 0 350 664 668
354 663 66a :3 0
357 64c 66c :2 0
66e 35a 66f 649
66e 0 670 35c
0 671 35e 674
:3 0 674 0 674
673 671 672 :6 0
676 16 :3 0 361
678 363 677 676
:2 0 679 365 :2 0
67b 0 67b 67a
628 679 :6 0 6f8
b :3 0 51 :3 0
2f :3 0 31 :3 0
8c :3 0 4e :3 0
9d :3 0 47 :3 0
9d :3 0 4e :3 0
48 :3 0 9d :3 0
47 :3 0 9d :3 0
47 :3 0 4b :3 0
13 :4 0 b3 1
:8 0 6a3 87 :4 0
68e :3 0 8e :3 0
7c :2 0 36a 691
692 :3 0 9 :3 0
6f :3 0 694 695
0 17 :2 0 696
697 0 699 36d
6a0 9 :3 0 6f
:3 0 69a 69b 0
88 :2 0 69c 69d
0 69f 36f 6a1
693 699 0 6a2
0 69f 0 6a2
371 0 6a3 374
6f6 71 :3 0 9
:3 0 6f :3 0 6a5
6a6 0 70 :3 0
6a7 6a8 0 6aa
377 6ac 379 6ab
6aa :2 0 6f4 72
:3 0 9 :3 0 6f
:3 0 6af 6b0 0
70 :3 0 6b1 6b2
0 6ec 55 :3 0
56 :3 0 6b4 6b5
0 55 :3 0 66
:3 0 6b7 6b8 0
73 :2 0 2 :4 0
b4 :2 0 68 :2 0
15 :2 0 37b 6bd
6bf :3 0 37e 6b9
6c1 6b6 6c3 6c2
:2 0 58 :3 0 69
:3 0 6c5 6c6 0
5a :4 0 b5 :4 0
5d :2 0 5e :3 0
382 6ca 6cc :3 0
5d :2 0 76 :4 0
385 6ce 6d0 :3 0
5d :2 0 77 :4 0
388 6d2 6d4 :3 0
5d :2 0 70 :3 0
38b 6d6 6d8 :3 0
5d :2 0 78 :4 0
38e 6da 6dc :3 0
5d :2 0 79 :3 0
7a :3 0 15 :2 0
7b :2 0 391 6df
6e3 395 6de 6e5
:3 0 398 6c7 6e7
:2 0 6e9 39b 6ea
6c4 6e9 0 6eb
39d 0 6ec 39f
6ef :3 0 6ef 0
6ef 6ee 6ec 6ed
:6 0 6f1 18 :3 0
3a2 6f3 3a4 6f2
6f1 :2 0 6f4 3a6
:2 0 6f6 0 6f6
6f5 6a3 6f4 :6 0
6f8 b :3 0 3a9
76e 51 :3 0 2f
:3 0 31 :3 0 8c
:3 0 48 :3 0 4f
:3 0 50 :3 0 4b
:3 0 13 :4 0 b6
1 :8 0 718 87
:4 0 703 :3 0 8e
:3 0 7c :2 0 3ae
706 707 :3 0 9
:3 0 6f :3 0 709
70a 0 17 :2 0
70b 70c 0 70e
3b1 715 9 :3 0
6f :3 0 70f 710
0 88 :2 0 711
712 0 714 3b3
716 708 70e 0
717 0 714 0
717 3b5 0 718
3b8 76b 71 :3 0
9 :3 0 6f :3 0
71a 71b 0 70
:3 0 71c 71d 0
71f 3bb 721 3bd
720 71f :2 0 769
72 :3 0 9 :3 0
6f :3 0 724 725
0 70 :3 0 726
727 0 761 55
:3 0 56 :3 0 729
72a 0 55 :3 0
66 :3 0 72c 72d
0 73 :2 0 2
:4 0 b7 :2 0 68
:2 0 15 :2 0 3bf
732 734 :3 0 3c2
72e 736 72b 738
737 :2 0 58 :3 0
69 :3 0 73a 73b
0 5a :4 0 b8
:4 0 5d :2 0 5e
:3 0 3c6 73f 741
:3 0 5d :2 0 76
:4 0 3c9 743 745
:3 0 5d :2 0 77
:4 0 3cc 747 749
:3 0 5d :2 0 70
:3 0 3cf 74b 74d
:3 0 5d :2 0 78
:4 0 3d2 74f 751
:3 0 5d :2 0 79
:3 0 7a :3 0 15
:2 0 7b :2 0 3d5
754 758 3d9 753
75a :3 0 3dc 73c
75c :2 0 75e 3df
75f 739 75e 0
760 3e1 0 761
3e3 764 :3 0 764
0 764 763 761
762 :6 0 766 1a
:3 0 3e6 768 3e8
767 766 :2 0 769
3ea :2 0 76b 0
76b 76a 718 769
:6 0 76d b :3 0
3ed 76f 608 6f8
0 770 0 76d
0 770 3ef 0
771 3f2 773 58c
5fe 0 774 0
771 0 774 3f4
0 775 3f7 777
381 583 0 778
0 775 0 778
3f9 0 8f2 9
:3 0 6f :3 0 779
77a 0 9 :3 0
7c :2 0 b9 :3 0
77c 77e 0 3fe
77d 780 :3 0 781
:2 0 1e :3 0 15
:2 0 783 784 0
786 401 787 782
786 0 788 403
0 8f2 27 :3 0
17 :2 0 789 78a
0 8f2 23 :3 0
91 :3 0 24 :3 0
13 :3 0 25 :3 0
26 :3 0 62 :3 0
27 :3 0 17 :2 0
405 792 795 408
78d 797 78c 798
0 8f2 62 :3 0
1e :3 0 17 :2 0
40e 79a 79d 7c
:2 0 17 :2 0 413
79f 7a1 :3 0 ba
:3 0 62 :3 0 16
:3 0 17 :2 0 416
7a4 7a7 15 :2 0
419 7a3 7aa 7ab
:2 0 7c :2 0 15
:2 0 41e 7ad 7af
:3 0 7b0 :2 0 62
:3 0 22 :3 0 17
:2 0 421 7b2 7b5
bb :2 0 17 :2 0
426 7b7 7b9 :3 0
7ba :2 0 7b1 7bc
7bb :2 0 29 :3 0
bc :3 0 62 :3 0
bd :3 0 21 :3 0
429 7c1 7c3 17
:2 0 42b 7c0 7c6
be :2 0 62 :3 0
bd :3 0 20 :3 0
42e 7ca 7cc 17
:2 0 430 7c9 7cf
433 7c8 7d1 :3 0
7d2 :2 0 bf :2 0
c0 :2 0 436 7d4
7d6 :3 0 7d7 :2 0
80 :2 0 62 :3 0
22 :3 0 17 :2 0
439 7da 7dd 43c
7d9 7df :3 0 43f
7bf 7e1 7be 7e2
0 8ec 29 :3 0
bb :2 0 15 :2 0
443 7e5 7e7 :3 0
7e8 :2 0 29 :3 0
c1 :2 0 80 :2 0
15 :2 0 446 7ec
7ee :3 0 44a 7eb
7f0 :3 0 7e9 7f2
7f1 :2 0 51 :3 0
c2 :3 0 c3 :3 0
c4 :3 0 c2 :3 0
37 :3 0 c5 :3 0
a2 :3 0 a2 :3 0
48 :3 0 32 :3 0
a2 :3 0 4b :3 0
13 :4 0 c6 1
:8 0 818 87 :4 0
803 :3 0 8e :3 0
7c :2 0 44f 806
807 :3 0 9 :3 0
6f :3 0 809 80a
0 17 :2 0 80b
80c 0 80e 452
815 9 :3 0 6f
:3 0 80f 810 0
88 :2 0 811 812
0 814 454 816
808 80e 0 817
0 814 0 817
456 0 818 459
86b 71 :3 0 9
:3 0 6f :3 0 81a
81b 0 70 :3 0
81c 81d 0 81f
45c 821 45e 820
81f :2 0 869 72
:3 0 9 :3 0 6f
:3 0 824 825 0
70 :3 0 826 827
0 861 55 :3 0
56 :3 0 829 82a
0 55 :3 0 66
:3 0 82c 82d 0
73 :2 0 2 :4 0
c7 :2 0 68 :2 0
15 :2 0 460 832
834 :3 0 463 82e
836 82b 838 837
:2 0 58 :3 0 69
:3 0 83a 83b 0
5a :4 0 c8 :4 0
5d :2 0 5e :3 0
467 83f 841 :3 0
5d :2 0 76 :4 0
46a 843 845 :3 0
5d :2 0 77 :4 0
46d 847 849 :3 0
5d :2 0 70 :3 0
470 84b 84d :3 0
5d :2 0 78 :4 0
473 84f 851 :3 0
5d :2 0 79 :3 0
7a :3 0 15 :2 0
7b :2 0 476 854
858 47a 853 85a
:3 0 47d 83c 85c
:2 0 85e 480 85f
839 85e 0 860
482 0 861 484
864 :3 0 864 0
864 863 861 862
:6 0 866 1c :3 0
487 868 489 867
866 :2 0 869 48b
:2 0 86b 0 86b
86a 818 869 :6 0
86d b :3 0 48e
8e2 51 :3 0 a2
:3 0 a2 :3 0 48
:3 0 32 :3 0 a2
:3 0 4b :3 0 13
:4 0 c9 1 :8 0
88c 87 :4 0 877
:3 0 8e :3 0 7c
:2 0 492 87a 87b
:3 0 9 :3 0 6f
:3 0 87d 87e 0
17 :2 0 87f 880
0 882 495 889
9 :3 0 6f :3 0
883 884 0 88
:2 0 885 886 0
888 497 88a 87c
882 0 88b 0
888 0 88b 499
0 88c 49c 8df
71 :3 0 9 :3 0
6f :3 0 88e 88f
0 70 :3 0 890
891 0 893 49f
895 4a1 894 893
:2 0 8dd 72 :3 0
9 :3 0 6f :3 0
898 899 0 70
:3 0 89a 89b 0
8d5 55 :3 0 56
:3 0 89d 89e 0
55 :3 0 66 :3 0
8a0 8a1 0 73
:2 0 2 :4 0 ca
:2 0 68 :2 0 15
:2 0 4a3 8a6 8a8
:3 0 4a6 8a2 8aa
89f 8ac 8ab :2 0
58 :3 0 69 :3 0
8ae 8af 0 5a
:4 0 cb :4 0 5d
:2 0 5e :3 0 4aa
8b3 8b5 :3 0 5d
:2 0 76 :4 0 4ad
8b7 8b9 :3 0 5d
:2 0 77 :4 0 4b0
8bb 8bd :3 0 5d
:2 0 70 :3 0 4b3
8bf 8c1 :3 0 5d
:2 0 78 :4 0 4b6
8c3 8c5 :3 0 5d
:2 0 79 :3 0 7a
:3 0 15 :2 0 7b
:2 0 4b9 8c8 8cc
4bd 8c7 8ce :3 0
4c0 8b0 8d0 :2 0
8d2 4c3 8d3 8ad
8d2 0 8d4 4c5
0 8d5 4c7 8d8
:3 0 8d8 0 8d8
8d7 8d5 8d6 :6 0
8da 1e :3 0 4ca
8dc 4cc 8db 8da
:2 0 8dd 4ce :2 0
8df 0 8df 8de
88c 8dd :6 0 8e1
b :3 0 4d1 8e3
7f3 86d 0 8e4
0 8e1 0 8e4
4d3 0 8ec 1c
:3 0 cc :3 0 d
:3 0 13 :3 0 4d6
8e6 8e9 8e5 8ea
0 8ec 4d9 8ed
7bd 8ec 0 8ee
4dd 0 8ef 4df
8f0 7a2 8ef 0
8f1 4e1 0 8f2
4e3 8f4 86 :4 0
8f2 :4 0 953 cd
:3 0 4a :4 0 8f8
:2 0 8fe 8f6 0
9 :3 0 6f :3 0
8f9 8fa 0 70
:3 0 8fb 8fc 0
8fe 4ed 951 71
:3 0 9 :3 0 6f
:3 0 900 901 0
70 :3 0 902 903
0 905 4f0 907
4f2 906 905 :2 0
94f 72 :3 0 9
:3 0 6f :3 0 90a
90b 0 70 :3 0
90c 90d 0 947
55 :3 0 56 :3 0
90f 910 0 55
:3 0 66 :3 0 912
913 0 73 :2 0
2 :4 0 ce :2 0
68 :2 0 15 :2 0
4f4 918 91a :3 0
4f7 914 91c 911
91e 91d :2 0 58
:3 0 69 :3 0 920
921 0 5a :4 0
cf :4 0 5d :2 0
5e :3 0 4fb 925
927 :3 0 5d :2 0
76 :4 0 4fe 929
92b :3 0 5d :2 0
77 :4 0 501 92d
92f :3 0 5d :2 0
70 :3 0 504 931
933 :3 0 5d :2 0
78 :4 0 507 935
937 :3 0 5d :2 0
79 :3 0 7a :3 0
15 :2 0 7b :2 0
50a 93a 93e 50e
939 940 :3 0 511
922 942 :2 0 944
514 945 91f 944
0 946 516 0
947 518 94a :3 0
94a 0 94a 949
947 948 :6 0 94c
20 :3 0 51b 94e
51d 94d 94c :2 0
94f 51f :2 0 951
0 951 950 8fe
94f :6 0 953 8
:3 0 522 955 2af
344 0 956 0
953 0 956 526
0 9da 33 :3 0
d0 :3 0 d1 :3 0
79 :3 0 30 :3 0
d1 :3 0 79 :3 0
1a :3 0 d2 :3 0
d1 :3 0 79 :3 0
30 :3 0 d1 :3 0
79 :3 0 1a :3 0
30 :3 0 34 :3 0
d :3 0 2c :3 0
12 :4 0 d3 1
:8 0 981 87 :4 0
96c :3 0 8e :3 0
7c :2 0 52b 96f
970 :3 0 9 :3 0
6f :3 0 972 973
0 17 :2 0 974
975 0 977 52e
97e 9 :3 0 6f
:3 0 978 979 0
88 :2 0 97a 97b
0 97d 530 97f
971 977 0 980
0 97d 0 980
532 0 981 535
9d4 71 :3 0 9
:3 0 6f :3 0 983
984 0 70 :3 0
985 986 0 988
538 98a 53a 989
988 :2 0 9d2 72
:3 0 9 :3 0 6f
:3 0 98d 98e 0
70 :3 0 98f 990
0 9ca 55 :3 0
56 :3 0 992 993
0 55 :3 0 66
:3 0 995 996 0
73 :2 0 2 :4 0
d4 :2 0 68 :2 0
15 :2 0 53c 99b
99d :3 0 53f 997
99f 994 9a1 9a0
:2 0 58 :3 0 69
:3 0 9a3 9a4 0
5a :4 0 d5 :4 0
5d :2 0 5e :3 0
543 9a8 9aa :3 0
5d :2 0 76 :4 0
546 9ac 9ae :3 0
5d :2 0 77 :4 0
549 9b0 9b2 :3 0
5d :2 0 70 :3 0
54c 9b4 9b6 :3 0
5d :2 0 78 :4 0
54f 9b8 9ba :3 0
5d :2 0 79 :3 0
7a :3 0 15 :2 0
7b :2 0 552 9bd
9c1 556 9bc 9c3
:3 0 559 9a5 9c5
:2 0 9c7 55c 9c8
9a2 9c7 0 9c9
55e 0 9ca 560
9cd :3 0 9cd 0
9cd 9cc 9ca 9cb
:6 0 9cf 22 :3 0
563 9d1 565 9d0
9cf :2 0 9d2 567
:2 0 9d4 0 9d4
9d3 981 9d2 :6 0
9da 8 :3 0 92
:5 0 9d8 24 0
9d9 56a 9d8 9d7
9da 56c 9dc 86
:4 0 9da :4 0 a50
cd :3 0 2b :4 0
9e0 :2 0 9e6 9de
0 9 :3 0 6f
:3 0 9e1 9e2 0
70 :3 0 9e3 9e4
0 9e6 573 a39
71 :3 0 9 :3 0
6f :3 0 9e8 9e9
0 70 :3 0 9ea
9eb 0 9ed 576
9ef 578 9ee 9ed
:2 0 a37 72 :3 0
9 :3 0 6f :3 0
9f2 9f3 0 70
:3 0 9f4 9f5 0
a2f 55 :3 0 56
:3 0 9f7 9f8 0
55 :3 0 66 :3 0
9fa 9fb 0 73
:2 0 2 :4 0 d6
:2 0 68 :2 0 15
:2 0 57a a00 a02
:3 0 57d 9fc a04
9f9 a06 a05 :2 0
58 :3 0 69 :3 0
a08 a09 0 5a
:4 0 d7 :4 0 5d
:2 0 5e :3 0 581
a0d a0f :3 0 5d
:2 0 76 :4 0 584
a11 a13 :3 0 5d
:2 0 77 :4 0 587
a15 a17 :3 0 5d
:2 0 70 :3 0 58a
a19 a1b :3 0 5d
:2 0 78 :4 0 58d
a1d a1f :3 0 5d
:2 0 79 :3 0 7a
:3 0 15 :2 0 7b
:2 0 590 a22 a26
594 a21 a28 :3 0
597 a0a a2a :2 0
a2c 59a a2d a07
a2c 0 a2e 59c
0 a2f 59e a32
:3 0 a32 0 a32
a31 a2f a30 :6 0
a34 24 :3 0 5a1
a36 5a3 a35 a34
:2 0 a37 5a5 :2 0
a39 0 a39 a38
9e6 a37 :6 0 a50
1 :3 0 62 :3 0
18 :3 0 17 :2 0
5a8 a3b a3e bb
:2 0 80 :2 0 15
:2 0 5ab a41 a43
:3 0 5af a40 a45
:3 0 1c :3 0 d8
:3 0 d :3 0 5b2
a48 a4a a47 a4b
0 a4d 5b4 a4e
a46 a4d 0 a4f
5b6 0 a50 5b8
a51 205 a50 0
a52 5c0 0 aa1
55 :3 0 56 :3 0
a53 a54 0 55
:3 0 66 :3 0 a56
a57 0 d9 :2 0
2 :4 0 da :2 0
68 :2 0 15 :2 0
5c2 a5c a5e :3 0
5c5 a58 a60 a55
a62 a61 :2 0 58
:3 0 69 :3 0 a64
a65 0 5a :4 0
db :4 0 5d :2 0
5e :3 0 5c9 a69
a6b :3 0 5d :2 0
76 :4 0 5cc a6d
a6f :3 0 5d :2 0
dc :4 0 5cf a71
a73 :3 0 5d :2 0
62 :3 0 63 :3 0
9 :3 0 6f :3 0
a78 a79 0 5d2
a77 a7b 64 :4 0
5d4 a76 a7e 5d7
a75 a80 :3 0 5d
:2 0 dd :4 0 5da
a82 a84 :3 0 5d
:2 0 60 :4 0 5dd
a86 a88 :3 0 5d
:2 0 3 :3 0 5e0
a8a a8c :3 0 5d
:2 0 60 :4 0 5e3
a8e a90 :3 0 5e6
a66 a92 :2 0 a94
5e9 a95 a63 a94
0 a96 5eb 0
aa1 de :3 0 df
:3 0 a97 a98 0
9 :3 0 6f :3 0
a9a a9b 0 3
:3 0 8e :3 0 5ed
a99 a9f :2 0 aa1
5f1 b42 9 :3 0
e0 :2 0 4 aa2
aa3 0 55 :3 0
56 :3 0 aa5 aa6
0 55 :3 0 66
:3 0 aa8 aa9 0
73 :2 0 2 :4 0
e1 :2 0 68 :2 0
15 :2 0 5fc aae
ab0 :3 0 5ff aaa
ab2 aa7 ab4 ab3
:2 0 58 :3 0 69
:3 0 ab6 ab7 0
5a :4 0 e2 :4 0
5d :2 0 5e :3 0
603 abb abd :3 0
606 ab8 abf :2 0
ad8 58 :3 0 69
:3 0 ac1 ac2 0
5a :4 0 e3 :4 0
5d :2 0 9 :3 0
e4 :3 0 ac7 ac8
0 609 ac6 aca
:3 0 5d :2 0 dd
:4 0 60c acc ace
:3 0 5d :2 0 9
:3 0 e5 :3 0 ad1
ad2 0 60f ad0
ad4 :3 0 612 ac3
ad6 :2 0 ad8 615
ad9 ab5 ad8 0
ada 618 0 aea
85 :3 0 54 :3 0
9 :3 0 e4 :3 0
e6 :3 0 9 :3 0
e5 :3 0 3 :3 0
e7 :4 0 e8 1
:a 0 ae6 :2 0 aea
adc ae4 :2 0 8
:3 0 54 :3 0 ae8
:2 0 aea 61a aec
61e aeb aea :2 0
b40 72 :3 0 9
:3 0 6f :3 0 aef
af0 0 70 :3 0
af1 af2 0 b38
55 :3 0 56 :3 0
af4 af5 0 55
:3 0 66 :3 0 af7
af8 0 73 :2 0
2 :4 0 e9 :2 0
68 :2 0 15 :2 0
620 afd aff :3 0
623 af9 b01 af6
b03 b02 :2 0 58
:3 0 69 :3 0 b05
b06 0 5a :4 0
ea :4 0 5d :2 0
5e :3 0 627 b0a
b0c :3 0 62a b07
b0e :2 0 b28 58
:3 0 69 :3 0 b10
b11 0 5a :4 0
77 :4 0 5d :2 0
70 :3 0 62d b15
b17 :3 0 5d :2 0
78 :4 0 630 b19
b1b :3 0 5d :2 0
79 :3 0 7a :3 0
15 :2 0 7b :2 0
633 b1e b22 637
b1d b24 :3 0 63a
b12 b26 :2 0 b28
63d b29 b04 b28
0 b2a 640 0
b38 85 :3 0 54
:3 0 e6 :3 0 9
:3 0 e5 :3 0 3
:3 0 e7 :4 0 eb
1 :a 0 b34 :2 0
b38 b2c b32 :2 0
8 :3 0 54 :3 0
b36 :2 0 b38 642
b3b :3 0 b3b 0
b3b b3a b38 b39
:6 0 b3d 1 :3 0
647 b3f 649 b3e
b3d :2 0 b40 64b
:2 0 b42 64e b42
b41 aa1 b40 :6 0
b43 :2 0 2 14
b42 b46 :3 0 b45
b43 b47 :8 0 
66b
4
:3 0 1 3 1
7 1 b 3
6 a e 1
16 1 1d 1
1b 1 24 1
22 1 29 1
30 1 2e 1
37 1 35 1
3e 1 3c 1
43 1 49 1
4f 1 55 1
5a 1 60 1
66 1 6c 1
72 1 78 1
7e 1 84 1
8c 1 8a 1
93 1 91 1
98 1 9e 1
a4 1 aa 1
e9 1 108 1
10e 1 11b 1
127 2 12a 12c
2 12f 131 2
133 135 2 137
139 2 13b 13d
1 143 2 145
146 2 13f 148
2 14a 14c 2
14e 150 2 152
154 2 156 158
2 15a 15c 4
123 129 12e 15e
1 160 1 162
2 16c 16e 3
16a 16b 170 2
179 17b 2 178
17d 1 17f 1
181 2 199 19e
1 1a5 1 1a0
2 1b8 1ba 3
1b6 1b7 1bc 2
1c5 1c7 2 1c9
1cb 2 1cd 1cf
2 1d1 1d3 2
1d5 1d7 3 1dc
1dd 1de 2 1d9
1e0 2 1c4 1e2
1 1e4 1 1e6
2 1af 1e7 1
1eb 1 1aa 2
1a8 1ef 1 1fa
2 1f6 1fa 1
1fe 2 208 20a
3 20c 20d 20e
1 214 2 21d
222 1 229 1
224 2 23c 23e
3 23a 23b 240
2 249 24b 2
24d 24f 2 251
253 2 255 257
2 259 25b 3
260 261 262 2
25d 264 2 248
266 1 268 1
26a 2 233 26b
1 26f 1 22e
2 22c 273 4
27f 280 281 282
1 28d 1 293
2 295 296 1
29e 2 29a 29e
1 2a3 1 2a5
2 2a8 2a9 1
2ad 2 2ab 2ad
2 2b1 2b2 1
2b6 2 2b4 2b6
1 2c4 2 2c3
2c4 1 2cc 1
2d2 2 2d4 2d5
2 2c1 2d6 1
2dd 1 2d8 2
2f0 2f2 3 2ee
2ef 2f4 2 2fd
2ff 2 301 303
2 305 307 2
309 30b 2 30d
30f 3 314 315
316 2 311 318
2 2fc 31a 1
31c 1 31e 2
2e7 31f 1 323
1 2e2 2 2e0
327 2 336 337
5 331 332 333
334 339 3 32a
32e 33c 1 33f
2 341 342 1
343 6 34c 34d
34e 34f 350 351
1 35c 1 362
2 364 365 1
36d 2 369 36d
1 372 1 374
2 37a 37b 1
37f 2 37d 37f
3 383 384 385
1 389 2 387
389 1 398 2
397 398 1 3a0
1 3a6 2 3a8
3a9 2 395 3aa
1 3b1 1 3ac
2 3c4 3c6 3
3c2 3c3 3c8 2
3d1 3d3 2 3d5
3d7 2 3d9 3db
2 3dd 3df 2
3e1 3e3 3 3e8
3e9 3ea 2 3e5
3ec 2 3d0 3ee
1 3f0 1 3f2
2 3bb 3f3 1
3f7 1 3b6 2
3b4 3fb 2 401
402 1 407 1
409 2 404 409
1 42b 2 42a
42b 1 433 1
439 2 43b 43c
2 428 43d 1
444 1 43f 2
457 459 3 455
456 45b 2 464
466 2 468 46a
2 46c 46e 2
470 472 2 474
476 3 47b 47c
47d 2 478 47f
2 463 481 1
483 1 485 2
44e 486 1 48a
1 449 2 447
48e 2 410 491
1 49f 2 49e
49f 1 4a7 1
4ad 2 4af 4b0
2 49c 4b1 1
4b8 1 4b3 2
4cb 4cd 3 4c9
4ca 4cf 2 4d8
4da 2 4dc 4de
2 4e0 4e2 2
4e4 4e6 2 4e8
4ea 3 4ef 4f0
4f1 2 4ec 4f3
2 4d7 4f5 1
4f7 1 4f9 2
4c2 4fa 1 4fe
1 4bd 2 4bb
502 1 505 2
508 509 2 3fe
50a 1 517 2
516 517 1 51f
1 525 2 527
528 2 514 529
1 530 1 52b
2 543 545 3
541 542 547 2
550 552 2 554
556 2 558 55a
2 55c 55e 2
560 562 3 567
568 569 2 564
56b 2 54f 56d
1 56f 1 571
2 53a 572 1
576 1 535 2
533 57a 1 57d
2 580 581 1
582 2 585 586
1 58a 2 588
58a 1 596 2
595 596 1 59e
1 5a4 2 5a6
5a7 2 593 5a8
1 5af 1 5aa
2 5c2 5c4 3
5c0 5c1 5c6 2
5cf 5d1 2 5d3
5d5 2 5d7 5d9
2 5db 5dd 2
5df 5e1 3 5e6
5e7 5e8 2 5e3
5ea 2 5ce 5ec
1 5ee 1 5f0
2 5b9 5f1 1
5f5 1 5b4 2
5b2 5f9 1 5fc
3 600 601 602
1 606 2 604
606 1 615 2
614 615 1 61d
1 623 2 625
626 2 612 627
1 62e 1 629
2 641 643 3
63f 640 645 2
64e 650 2 652
654 2 656 658
2 65a 65c 2
65e 660 3 665
666 667 2 662
669 2 64d 66b
1 66d 1 66f
2 638 670 1
674 1 633 2
631 678 1 690
2 68f 690 1
698 1 69e 2
6a0 6a1 2 68d
6a2 1 6a9 1
6a4 2 6bc 6be
3 6ba 6bb 6c0
2 6c9 6cb 2
6cd 6cf 2 6d1
6d3 2 6d5 6d7
2 6d9 6db 3
6e0 6e1 6e2 2
6dd 6e4 2 6c8
6e6 1 6e8 1
6ea 2 6b3 6eb
1 6ef 1 6ae
2 6ac 6f3 2
67b 6f6 1 705
2 704 705 1
70d 1 713 2
715 716 2 702
717 1 71e 1
719 2 731 733
3 72f 730 735
2 73e 740 2
742 744 2 746
748 2 74a 74c
2 74e 750 3
755 756 757 2
752 759 2 73d
75b 1 75d 1
75f 2 728 760
1 764 1 723
2 721 768 1
76b 2 76e 76f
1 770 2 772
773 1 774 2
776 777 1 77f
2 77b 77f 1
785 1 787 2
793 794 5 78e
78f 790 791 796
2 79b 79c 1
7a0 2 79e 7a0
2 7a5 7a6 2
7a8 7a9 1 7ae
2 7ac 7ae 2
7b3 7b4 1 7b8
2 7b6 7b8 1
7c2 2 7c4 7c5
1 7cb 2 7cd
7ce 2 7c7 7d0
2 7d3 7d5 2
7db 7dc 2 7d8
7de 1 7e0 1
7e6 2 7e4 7e6
1 7ed 1 7ef
2 7ea 7ef 1
805 2 804 805
1 80d 1 813
2 815 816 2
802 817 1 81e
1 819 2 831
833 3 82f 830
835 2 83e 840
2 842 844 2
846 848 2 84a
84c 2 84e 850
3 855 856 857
2 852 859 2
83d 85b 1 85d
1 85f 2 828
860 1 864 1
823 2 821 868
1 86b 1 879
2 878 879 1
881 1 887 2
889 88a 2 876
88b 1 892 1
88d 2 8a5 8a7
3 8a3 8a4 8a9
2 8b2 8b4 2
8b6 8b8 2 8ba
8bc 2 8be 8c0
2 8c2 8c4 3
8c9 8ca 8cb 2
8c6 8cd 2 8b1
8cf 1 8d1 1
8d3 2 89c 8d4
1 8d8 1 897
2 895 8dc 1
8df 2 8e2 8e3
2 8e7 8e8 3
7e3 8e4 8eb 1
8ed 1 8ee 1
8f0 9 352 366
375 378 778 788
78b 799 8f1 2
8f7 8fd 1 904
1 8ff 2 917
919 3 915 916
91b 2 924 926
2 928 92a 2
92c 92e 2 930
932 2 934 936
3 93b 93c 93d
2 938 93f 2
923 941 1 943
1 945 2 90e
946 1 94a 1
909 2 907 94e
3 347 8f4 951
2 954 955 1
96e 2 96d 96e
1 976 1 97c
2 97e 97f 2
96b 980 1 987
1 982 2 99a
99c 3 998 999
99e 2 9a7 9a9
2 9ab 9ad 2
9af 9b1 2 9b3
9b5 2 9b7 9b9
3 9be 9bf 9c0
2 9bb 9c2 2
9a6 9c4 1 9c6
1 9c8 2 991
9c9 1 9cd 1
98c 2 98a 9d1
1 9d6 6 283
297 2a6 956 9d4
9d9 2 9df 9e5
1 9ec 1 9e7
2 9ff a01 3
9fd 9fe a03 2
a0c a0e 2 a10
a12 2 a14 a16
2 a18 a1a 2
a1c a1e 3 a23
a24 a25 2 a20
a27 2 a0b a29
1 a2b 1 a2d
2 9f6 a2e 1
a32 1 9f1 2
9ef a36 2 a3c
a3d 1 a42 1
a44 2 a3f a44
1 a49 1 a4c
1 a4e 7 211
217 276 27a 9dc
a39 a4f 1 a51
2 a5b a5d 3
a59 a5a a5f 2
a68 a6a 2 a6c
a6e 2 a70 a72
1 a7a 2 a7c
a7d 2 a74 a7f
2 a81 a83 2
a85 a87 2 a89
a8b 2 a8d a8f
2 a67 a91 1
a93 1 a95 3
a9c a9d a9e a
163 182 185 188
18b 18e 1f2 a52
a96 aa0 2 aad
aaf 3 aab aac
ab1 2 aba abc
2 ab9 abe 2
ac5 ac9 2 acb
acd 2 acf ad3
2 ac4 ad5 2
ac0 ad7 1 ad9
3 ada ae5 ae9
1 aa4 2 afc
afe 3 afa afb
b00 2 b09 b0b
2 b08 b0d 2
b14 b16 2 b18
b1a 3 b1f b20
b21 2 b1c b23
2 b13 b25 2
b0f b27 1 b29
4 af3 b2a b33
b37 1 b3b 1
aee 2 aec b3f
1c 19 20 27
2c 33 3a 41
47 4d 53 58
5e 64 6a 70
76 7c 82 88
8f 96 9c a2
a8 ad ea 109
113 
1
4
0 
b46
0
1
28
26
21
0 1 1 1 4 1 6 1
8 9 8 b c b e b
10 b 12 b 14 b 16 b
18 b 1a b 1c b 1e 8
20 8 22 1 24 1 0 0

b 1 0
60 1 0
3 1 0
7e 1 0
16 1 0
35 1 0
98 1 0
2 0 1
9d6 8 24
84 1 0
5a 1 0
49 1 0
29 1 0
8a 1 0
f0 1 3
43 1 0
72 1 0
2e 1 0
22 1 0
78 1 0
a4 1 0
3c 1 0
6c 1 0
9e 1 0
4f 1 0
55 1 0
1b 1 0
66 1 0
7 1 0
91 1 0
b0 1 2
aa 1 0
10e 1 0
0

/
SHOW ERRORS FUNCTION SP_RJCT_ETC_WBS;
ALTER FUNCTION SP_RJCT_ETC_WBS COMPILE DEBUG;
Prompt == FUNCTION SP_U_WP_FORECAST ==
CREATE OR REPLACE FUNCTION SP_U_WP_FORECAST wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
12b
2 :e:
1FUNCTION:
1SP_U_WP_FORECAST:
1ID:
1VARCHAR2:
1START_DATE001:
1TIMESTAMP:
1FINISH_DATE001:
1ACC_REJ_BITFLAG:
1PLS_INTEGER:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_PROJECT_ID:
132:
1H_TASK_ID:
1H_PRJ_ASSIGNMENT_ID:
1H_TSK_ASSIGNMENT_ID:
1H_RES_ASSIGNMENT_ID:
1H_RES_ID:
1H_REC_USER:
1H_SECURITY_ID:
1H_START_DATE:
1H_FINISH_DATE:
1H_START_DT:
1H_FINISH_DT:
1H_FORECAST_START:
1H_FORECAST_FINISH:
1H_ACTUAL_START:
1H_RANK:
1200:
1H_MLST_PERCENT:
10:
1H_TYPE:
1H_REG_COST:
1NUMBER:
10.00:
1H_REG_SALE:
1H_TA_START_DT:
1H_TA_FINISH_DT:
1H_LAST_SUBMITTED:
1DATE:
1H_PROJAFFECTED_START:
1H_DAYOFYEAR:
1H_WEEK_OF:
1H_BIT_FLAG:
11:
1H_BILLABLE:
1H_CAPITAL_EXPENSE:
1H_ETC:
1H_TOTAL:
1H_SCHED_WORK:
1H_CONTINUE:
1H_EFFORT:
1H_CONTOUR:
1H_BITFLAG:
1H_ACC_REJ_BITFLAG:
1H_REMAINING:
1H_FF:
1H_TEMP_VAR:
1H_TEMP_SC_STATE:
1H_ALERT_FLAG:
1H_ALERT_ID:
1NULL_RETURN:
1LONG:
1ORA_TEMP_SC_STATE:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_U_WP_FORECAST(:
1':
1, :
1COALESCE:
1NULLIF:
1'':
1NULL:
1TO_CHAR:
1);:
1CHECK_TRACE:
167:
1+:
1FPUT:
1               ENTER:: SP_U_WP_FORECAST(68) :: cplan.sql(20606) :: :
1:
1=:
1PMO_CNST:
1SC_ACCEPT_REJECT_FORECAST:
1PROJECT_ID:
1RES_ASSIGNMENT_ID:
1PROJECT_ASSIGNMENT_ID:
1ACTUAL_START:
1MILESTONE_PERCENT:
1BIT_FLAG:
1FORECAST_START:
1SCHED_START:
1FORECAST_FINISH:
1SCHED_FINISH:
1TASK_ID:
1RESOURCE_ID:
1SCHED_WORK:
1PLANNED_WORK:
1WORK_TODATE:
1ETC:
1C_CONTOUR:
1LAST_SUBMITTED:
1UNRESOLVED_START:
1UNRESOLVED_FINISH:
1BILLABLE:
1CAPITAL_EXPENSE:
1REGULAR_COST:
1REGULAR_RATE:
1TASK_ASSIGNMENTS:
1TSK_ASSIGNMENT_ID:
1SELECT PROJECT_ID, RES_ASSIGNMENT_ID, PROJECT_ASSIGNMENT_ID, ACTUAL_START, MI+
1LESTONE_PERCENT, BIT_FLAG, :n      COALESCE(FORECAST_START,SCHED_START), COAL+
1ESCE(FORECAST_FINISH,SCHED_FINISH), FORECAST_START, FORECAST_FINISH, TASK_ID,+
1 RESOURCE_ID, SCHED_WORK, (PLANNED_WORK + SCHED_WORK - WORK_TODATE), ETC,:n  +
1    C_CONTOUR, LAST_SUBMITTED, UNRESOLVED_START, UNRESOLVED_FINISH, BILLABLE,+
1 CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE:n      INTO h_project_id, h_res_+
1assignment_id, h_prj_assignment_id, h_actual_start, h_mlst_percent, h_bit_fla+
1g,:n      h_start_DT, h_finish_DT, h_forecast_start, h_forecast_finish, h_tas+
1k_id, h_res_id, h_sched_work,h_remaining, h_etc,:n      h_contour, h_last_sub+
1mitted, h_ta_start_DT, h_ta_finish_DT, h_billable, h_capital_expense, h_reg_c+
1ost, h_reg_sale:n      FROM TASK_ASSIGNMENTS:n      WHERE TSK_ASSIGNMENT_ID =+
1 id:
1SC_STATE:
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
1103:
1           EXCEPTION:: SP_U_WP_FORECAST(104) :: cplan.sql(20642) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1SC_SQL_NO_DATA:
1111:
1        RETURN_ERROR:: SP_U_WP_FORECAST(112) :: cplan.sql(20650) :: :
1STATE:: :
1RECORD_NOT_FOUND:
1 ID:: :
1114:
1                EXIT:: SP_U_WP_FORECAST(115) :: cplan.sql(20653) :: :
1PMODEFINES:
1PMO_RETURN_ERROR_SP:
1TRUE:
110000:
1121:
1        RETURN_ERROR:: SP_U_WP_FORECAST(122) :: cplan.sql(20660) :: :
1ASSIGNMENT_ALREADY_CLOSED:
1124:
1                EXIT:: SP_U_WP_FORECAST(125) :: cplan.sql(20663) :: :
1IS NOT NULL:
1PMO_ORASTD:
1STRNCMP:
110:
1<:
1132:
1        RETURN_ERROR:: SP_U_WP_FORECAST(133) :: cplan.sql(20671) :: :
1F_DATE_EARLIER_THAN_S_DATE:
1135:
1                EXIT:: SP_U_WP_FORECAST(136) :: cplan.sql(20674) :: :
1MAX:
1WEEK_OF:
1TIMESHEET_APPROVAL_STATUS:
1ELEMENT_ID:
1CURRENT_STATE:
1SC_APV_STATE_RM_APPROVED:
1SC_APV_STATE_PM_APPROVED:
1SC_APV_STATE_SUBMITTED:
1SELECT MAX(WEEK_OF) + 7      INTO h_week_of FROM TIMESHEET_APPROVAL_STATUS:n +
1     WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_res_id:n      AND CU+
1RRENT_STATE IN (PMO_CNST.SC_APV_STATE_RM_APPROVED,PMO_CNST.SC_APV_STATE_PM_AP+
1PROVED,PMO_CNST.SC_APV_STATE_SUBMITTED):
1154:
1           EXCEPTION:: SP_U_WP_FORECAST(155) :: cplan.sql(20693) :: :
1STRCMP:
1!=:
1SCETCFORCASTCHANGED:
1FC_ISSUE_ALERT:
1SCREDUCEDETC:
1IS NULL:
1FORECAST_DURATION:
1UPDATE TASK_ASSIGNMENTS SET:n         FORECAST_START  = start_date001,:n     +
1    FORECAST_FINISH = finish_date001,:n         FORECAST_DURATION = 0:n      +
1   WHERE TSK_ASSIGNMENT_ID = id:
1FOUND:
1100:
1           EXCEPTION:: SP_U_WP_FORECAST(201) :: cplan.sql(20739) :: :
1UPDATE TASK_ASSIGNMENTS SET:n            FORECAST_START  = start_date001,:n  +
1          FORECAST_DURATION = 0:n            WHERE TSK_ASSIGNMENT_ID = id:
1227:
1           EXCEPTION:: SP_U_WP_FORECAST(228) :: cplan.sql(20766) :: :
1UPDATE TASK_ASSIGNMENTS SET:n            FORECAST_FINISH = finish_date001,:n +
1           FORECAST_DURATION = 0:n            WHERE TSK_ASSIGNMENT_ID = id:
1259:
1           EXCEPTION:: SP_U_WP_FORECAST(260) :: cplan.sql(20798) :: :
1>:
1283:
1        RETURN_ERROR:: SP_U_WP_FORECAST(284) :: cplan.sql(20822) :: :
1FORECAST_APPROVAL_EXISTS:
1286:
1                EXIT:: SP_U_WP_FORECAST(287) :: cplan.sql(20825) :: :
1294:
1        RETURN_ERROR:: SP_U_WP_FORECAST(295) :: cplan.sql(20833) :: :
1CANNOT_UPDATE_HAS_ACTUAL:
1297:
1                EXIT:: SP_U_WP_FORECAST(298) :: cplan.sql(20836) :: :
1ELSIF:
1DB2TIMESTAMP:
1DAY_YEAR:
1TIME:
1PMO_STPU:
1FIRST_AVAIL_TIME:
1BIT_SCHED:
1CALENDAR_DAY_OF_YEAR:
1TO_DATE:
1SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL+
1_TIME(BIT_SCHED))) INTO h_start_date:n                  FROM CALENDAR_DAY_OF_+
1YEAR:n                  WHERE ELEMENT_ID = h_project_id:n                  AN+
1D DAY_YEAR = TO_DATE(SubStr(start_date001, 1, 10)):
1320:
1           EXCEPTION:: SP_U_WP_FORECAST(321) :: cplan.sql(20859) :: :
1DUAL:
1SELECT  (PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(start_date001, 1, 10)), PMO_O+
1RASTD.TIME('08::00')))  INTO  h_start_date FROM DUAL:
1337:
1           EXCEPTION:: SP_U_WP_FORECAST(338) :: cplan.sql(20876) :: :
1344:
1        RETURN_ERROR:: SP_U_WP_FORECAST(345) :: cplan.sql(20883) :: :
1347:
1                EXIT:: SP_U_WP_FORECAST(348) :: cplan.sql(20886) :: :
1360:
1        RETURN_ERROR:: SP_U_WP_FORECAST(361) :: cplan.sql(20899) :: :
1INVALID_PARAMS:
1363:
1                EXIT:: SP_U_WP_FORECAST(364) :: cplan.sql(20902) :: :
1<=:
18:
1369:
1      RETURN_INVALID:: SP_U_WP_FORECAST(370) :: cplan.sql(20908) :: :
1372:
1                EXIT:: SP_U_WP_FORECAST(373) :: cplan.sql(20911) :: :
1PMO_RETURN_INVALID_SP:
1LAST_AVAIL_TIME:
1SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_+
1TIME(BIT_SCHED))) INTO h_finish_date:n                  FROM CALENDAR_DAY_OF_+
1YEAR:n                  WHERE ELEMENT_ID = h_project_id:n                  AN+
1D DAY_YEAR = TO_DATE(SubStr(finish_date001, 1, 10)):
1401:
1           EXCEPTION:: SP_U_WP_FORECAST(402) :: cplan.sql(20940) :: :
1SELECT  (PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(finish_date001, 1, 10)), PMO_+
1ORASTD.TIME('17::00')))  INTO  h_finish_date FROM DUAL:
1419:
1           EXCEPTION:: SP_U_WP_FORECAST(420) :: cplan.sql(20958) :: :
1427:
1        RETURN_ERROR:: SP_U_WP_FORECAST(428) :: cplan.sql(20966) :: :
1430:
1                EXIT:: SP_U_WP_FORECAST(431) :: cplan.sql(20969) :: :
1UPDATE TASK_ASSIGNMENTS SET:n               FORECAST_START  = h_start_date,:n+
1               FORECAST_FINISH = h_finish_date,:n               FORECAST_DURA+
1TION = 0:n               WHERE TSK_ASSIGNMENT_ID = id:
1461:
1           EXCEPTION:: SP_U_WP_FORECAST(462) :: cplan.sql(21000) :: :
1COUNT:
1TASK_DAY_OF_YEAR:
1SELECT COUNT('X') INTO h_temp_var:n                     FROM DUAL:n          +
1           WHERE EXISTS (SELECT DAY_YEAR FROM TASK_DAY_OF_YEAR:n             +
1        WHERE PROJECT_ID = h_project_id AND ((TSK_ASSIGNMENT_ID = h_prj_assig+
1nment_id) OR (TSK_ASSIGNMENT_ID = h_tsk_assignment_id)):n                    +
1 AND DAY_YEAR = TO_DATE(SubStr(finish_date001, 1, 10))):
1485:
1           EXCEPTION:: SP_U_WP_FORECAST(486) :: cplan.sql(21024) :: :
1493:
1        RETURN_ERROR:: SP_U_WP_FORECAST(494) :: cplan.sql(21032) :: :
1PROJECT_ASSIGNMENT_REMAINING:
1496:
1                EXIT:: SP_U_WP_FORECAST(497) :: cplan.sql(21035) :: :
1SELECT COUNT('X') INTO h_temp_var:n                     FROM DUAL:n          +
1           WHERE EXISTS (SELECT DAY_YEAR FROM TASK_DAY_OF_YEAR:n             +
1        WHERE PROJECT_ID = h_project_id AND ((TSK_ASSIGNMENT_ID = h_prj_assig+
1nment_id) OR (TSK_ASSIGNMENT_ID = h_tsk_assignment_id)):n                    +
1 AND DAY_YEAR = TO_DATE(SubStr(start_date001, 1, 10))):
1517:
1           EXCEPTION:: SP_U_WP_FORECAST(518) :: cplan.sql(21056) :: :
1525:
1        RETURN_ERROR:: SP_U_WP_FORECAST(526) :: cplan.sql(21064) :: :
1528:
1                EXIT:: SP_U_WP_FORECAST(529) :: cplan.sql(21067) :: :
1SUM:
1SCHED_MIN:
1PARENT_ID:
1SELECT SUM(SCHED_MIN) INTO h_total FROM TASK_DAY_OF_YEAR:n                   +
1     WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id AND +
1DAY_YEAR >= TO_DATE(SubStr(start_date001, 1, 10)):
1548:
1           EXCEPTION:: SP_U_WP_FORECAST(549) :: cplan.sql(21087) :: :
1554:
1        RETURN_ERROR:: SP_U_WP_FORECAST(555) :: cplan.sql(21093) :: :
1557:
1                EXIT:: SP_U_WP_FORECAST(558) :: cplan.sql(21096) :: :
1565:
1        RETURN_ERROR:: SP_U_WP_FORECAST(566) :: cplan.sql(21104) :: :
1TASK_ASSIGNMENT_ERROR:
1568:
1                EXIT:: SP_U_WP_FORECAST(569) :: cplan.sql(21107) :: :
12:
1FC_ACPT_ETC_ASSIGN:
1U_ASSIGN_STRING:
13:
1FC_ROLLUP:
14:
1619:
1       RETURN_STATUS:: SP_U_WP_FORECAST(620) :: cplan.sql(21158) :: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
1627:
1           EXCEPTION:: SP_U_WP_FORECAST(628) :: cplan.sql(21166) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_ID:
1OPEN:
1ERROR:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1639:
1           EXCEPTION:: SP_U_WP_FORECAST(640) :: cplan.sql(21178) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
112f
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d b4
:3 a0 6b 2c 6a a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 1c 81
b0 a3 a0 1c 81 b0 a3 a0
1c 81 b0 a3 a0 1c 81 b0
a3 a0 1c 81 b0 a3 a0 1c
81 b0 a3 a0 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
1c 51 81 b0 a3 a0 1c 51
81 b0 a3 a0 1c 51 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 81 b0 a3 a0 1c 81 b0
a3 a0 1c 81 b0 a3 a0 1c
81 b0 a3 a0 1c 81 b0 a3
a0 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 1c
51 81 b0 a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 1c 81
b0 a3 a0 1c 51 81 b0 a3
:2 a0 6b 1c 81 b0 :2 a0 6b :2 a0
6b 6e a5 b a 10 :2 a0 6b
6e :2 a0 6b 6e a5 b 6e 7e
a0 b4 2e 6e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :2 a0 6e 7e
a0 b4 2e 7e 6e b4 2e 6e
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e :2 a0 6e 7e a0
b4 2e 7e 6e b4 2e 6e a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e :3 a0 a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c a0
6e d a0 6e d a0 51 d
:2 a0 d :2 a0 d :2 a0 d :2 a0 d
:2 a0 51 a5 b 7e 51 b4 2e
5a :3 a0 6b d b7 19 3c :37 a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 6b a0 7e a0 6b b4
2e 5a :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 b7 19 3c :2 a0 51 a5 b
7e 51 b4 2e :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 19 3c a0 7e
b4 2e :2 5a a0 7e b4 2e :2 5a
a 10 :2 a0 6b :2 a0 51 a5 b
7e 51 b4 2e :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 19 3c b7 19
3c a0 4d d :f a0 12a :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 d
:2 a0 6b :2 a0 a5 b 7e 51 b4
2e 5a :2 a0 51 a5 b a0 7e
a0 6b b4 2e 5a a 10 a0
6e d a0 51 d :3 a0 6b d
:6 a0 51 a5 b d :2 a0 6b a0
d a0 6e d a0 51 d :3 a0
6b d :6 a0 51 a5 b d :2 a0
6b a0 d b7 19 3c a0 7e
b4 2e :2 5a :2 a0 6b :2 a0 51 a5
b 7e 51 b4 2e 5a a 10
:2 a0 d b7 19 3c a0 7e b4
2e :2 5a a0 7e b4 2e :2 5a a
10 :8 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 a0 7e b4 2e 5a a0
7e b4 2e 5a a 10 :6 a0 12a
a0 f a0 7e b4 2e :2 a0 6b
51 d b7 :2 a0 6b 51 d b7
:2 19 3c b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
:2 a0 51 a5 b 7e 51 b4 2e
a0 51 d b7 19 3c b7 19
3c a0 7e b4 2e 5a a0 7e
b4 2e 5a a 10 :6 a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b :2 a0
51 a5 b 7e 51 b4 2e a0
51 d b7 19 3c b7 19 3c
:2 a0 51 a5 b 7e 51 b4 2e
a0 7e b4 2e :2 5a :2 a0 6b :2 a0
51 a5 b 7e 51 b4 2e 5a
a 10 :2 a0 d b7 19 3c a0
7e b4 2e :2 5a a0 7e b4 2e
:2 5a a 10 :2 a0 6b :2 a0 a5 b
7e 51 b4 2e 5a a 10 :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 19
3c a0 7e b4 2e 5a a0 7e
b4 2e 5a :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b :2 a0
a5 57 b7 19 3c :2 a0 d :2 a0
6b :2 a0 51 a5 b 7e 51 b4
2e a0 4d d a0 b7 :2 a0 6b
:2 a0 51 a5 b 7e 51 b4 2e
:10 a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b a0 7e a0 6b
b4 2e 5a :9 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 19 3c
a0 b7 19 a0 7e b4 2e 5a
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :2 a0
51 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b 51 :2 a0 a5 57 b7 :2 19
3c b7 a0 51 d a0 4d d
b7 :2 19 3c a0 7e b4 2e 5a
a0 7e b4 2e :2 5a :2 a0 6b :2 a0
51 a5 b 7e 51 b4 2e 5a
a 10 :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e :4 a0 6b a5 b 6e a5 b
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b :2 a0 a5
57 a0 b7 a0 7e b4 2e :2 5a
:2 a0 6b :2 a0 51 a5 b 7e 51
b4 2e 5a a 10 :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b a0 a5 57 b7 :2 19 3c a0
7e b4 2e :2 5a :2 a0 6b :2 a0 51
a5 b 7e 51 b4 2e 5a a
10 a0 51 d b7 a0 51 d
b7 :2 19 3c :2 a0 6b :2 a0 51 a5
b 7e 51 b4 2e a0 4d d
a0 b7 :2 a0 6b :2 a0 a5 b 7e
51 b4 2e :10 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :2 a0 6b a0
7e a0 6b b4 2e 5a :9 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 19 3c a0 b7 19 :2 a0 6b
:2 a0 51 a5 b 7e 51 b4 2e
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e :2 a0
51 a5 b 6e a5 b b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 b7 19 3c
:2 a0 6b 51 :2 a0 a5 57 b7 :2 19
3c b7 a0 4d d b7 :2 19 3c
:2 a0 d a0 51 d :2 a0 51 a5
b a0 7e a0 6b b4 2e :8 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
a0 7e b4 2e 5a a0 7e b4
2e 5a :f a0 12a :2 a0 6b a0 d
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 51 a5 b
7e 51 b4 2e :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 19 3c a0 b7
a0 7e b4 2e 5a :f a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
51 a5 b 7e 51 b4 2e :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b :2 a0 a5 57 b7 :c a0
12a :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 51 a5 b a0 7e a0
51 a5 b b4 2e :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e :4 a0 6b a5 b
6e a5 b b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c :2 a0 6b :2 a0
6b :2 a0 a5 57 b7 19 3c b7
:2 19 3c b7 19 :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :4 a0 6b a5 b 6e
a5 b b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 :2 19 3c b7 19
3c a0 51 d :2 a0 d a0 7e
b4 2e 5a a0 7e b4 2e :2 5a
a 10 :2 a0 d b7 19 3c :18 a0
51 a5 b d a0 51 d :5 a0
:2 51 a5 b d a0 7e b4 2e
5a a0 51 d :5 a0 :2 51 a5 b
d b7 19 3c b7 :2 19 3c b7
19 3c b7 :2 19 3c :2 a0 6b :2 a0
a5 b 7e 51 b4 2e 5a :2 a0
51 a5 b a0 7e a0 6b b4
2e 5a a 10 a0 6e d a0
51 d :3 a0 6b d :6 a0 51 a5
b d :2 a0 6b a0 d a0 6e
d a0 51 d :3 a0 6b d :6 a0
51 a5 b d :2 a0 6b a0 d
b7 19 3c :3 a0 a5 b d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :4 a0 6b
a5 b 6e a5 b b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e a5
57 b7 19 3c :2 a0 6b :2 a0 6b
:2 a0 a5 57 b7 :2 a0 6b :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 :2 a0 6b
:2 6e 7e :2 a0 6b b4 2e 7e 6e
b4 2e 7e :2 a0 6b b4 2e a5
57 b7 19 3c :9 a0 12a e9 dd
:2 a0 65 b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e a5 57 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c :7 a0 12a e9 dd :2 a0 65
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 68 4f 1d 17 b5 
112f
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 6f 6b 59 77 56
7c 80 84 88 8b 8f ac 97
9b 9e 9f a7 96 c9 b7 93
bb bc c4 b6 e6 d4 b3 d8
d9 e1 d3 103 f1 d0 f5 f6
fe f0 120 10e ed 112 113 11b
10d 13d 12b 10a 12f 130 138 12a
15a 148 127 14c 14d 155 147 177
165 144 169 16a 172 164 193 182
186 18e 161 1ab 19a 19e 1a6 181
1c7 1b6 1ba 1c2 17e 1df 1ce 1d2
1da 1b5 1fb 1ea 1ee 1f6 1b2 213
202 206 20e 1e9 22f 21e 222 22a
1e6 24b 236 23a 23d 23e 246 21d
267 256 25a 21a 262 255 283 272
276 252 27e 271 29f 28e 292 26e
29a 28d 2bb 2aa 2ae 28a 2b6 2a9
2d7 2c6 2ca 2d2 2a6 2ef 2de 2e2
2ea 2c5 30b 2fa 2fe 306 2c2 323
312 316 31e 2f9 33f 32e 332 33a
2f6 357 346 34a 352 32d 374 362
32a 366 367 36f 361 390 37f 383
35e 38b 37e 3ac 39b 39f 37b 3a7
39a 3c8 3b7 3bb 397 3c3 3b6 3e4
3d3 3d7 3b3 3df 3d2 400 3ef 3f3
3cf 3fb 3ee 41c 40b 40f 3eb 417
40a 438 427 42b 407 433 426 454
443 447 423 44f 442 470 45f 463
43f 46b 45e 48c 47b 47f 45b 487
47a 4a8 497 49b 477 4a3 496 4c4
4b3 4b7 493 4bf 4b2 4e0 4cf 4d3
4af 4db 4ce 4fc 4eb 4ef 4cb 4f7
4ea 518 507 50b 4e7 513 506 535
523 503 527 528 530 522 551 540
544 54c 51f 56c 558 55c 564 567
53f 58c 577 57b 53c 57f 587 576
593 597 573 59b 59f 5a3 5a6 5ab
5ac 1 5ae 5b3 5b7 5bb 5be 5c3
5c7 5cb 5ce 5d3 5d4 5d6 5db 5de
5e2 5e3 5e8 5ed 5f0 5f5 5f6 5fb
5fe 602 603 608 60b 610 611 616
619 61e 61f 624 627 62b 62f 634
637 63b 63c 641 644 649 64a 64f
654 655 657 65c 65d 65f 660 665
668 66d 66e 673 676 67a 67e 683
686 68a 68b 690 693 698 699 69e
6a3 6a4 6a6 6ab 6ac 6ae 6af 6b4
6b7 6bc 6bd 6c2 6c5 6c9 6cd 6d1
6d2 6d4 6d9 6da 6dc 6dd 6e2 6e5
6ea 6eb 6f0 6f3 6f8 6f9 6fe 701
705 706 70b 70e 713 714 719 71c
721 722 727 728 72d 72f 733 736
73a 73e 741 745 749 74c 74f 754
757 75a 75d 75e 763 764 1 766
76b 76f 773 776 77b 780 783 787
788 78d 78e 793 795 799 79c 7a0
7a5 7a9 7ad 7b2 7b6 7ba 7bd 7c1
7c5 7c9 7cd 7d1 7d5 7d9 7dd 7e1
7e5 7e9 7ed 7f1 7f5 7f9 7fc 7fd
7ff 802 805 806 80b 80e 812 816
81a 81d 821 823 827 82a 82e 832
836 83a 83e 842 846 84a 84e 852
856 85a 85e 862 866 86a 86e 872
876 87a 87e 882 886 88a 88e 892
896 89a 89e 8a2 8a6 8aa 8ae 8b2
8b6 8ba 8be 8c2 8c6 8ca 8ce 8d2
8d6 8da 8de 8e2 8e6 8ea 8ee 8f2
8f6 8fa 8fe 902 906 912 916 91a
91d 921 925 927 92b 92f 933 936
93a 93e 940 941 946 1 94a 94e
952 955 959 95d 961 965 968 96c
970 973 976 97b 97e 981 984 985
98a 98b 1 98d 992 996 99a 99d
9a2 9a7 9aa 9ae 9af 9b4 9b7 9bc
9bd 9c2 9c5 9ca 9cb 9d0 9d3 9d7
9d8 9dd 9e0 9e5 9e6 9eb 9ee 9f2
9f6 9f9 9fc 9fd 9ff a00 a05 a06
a0b a0d a11 a14 a16 a1a a1c a28
a2a a2c a2d a32 a36 a38 a44 a46
a4a a4e a51 a55 a58 a5c a5f a60
a65 a68 a6c a70 a73 a77 a7b a7e
a81 a86 a89 a8c a8f a90 a95 a96
1 a98 a9d aa1 aa5 aa8 aad ab2
ab5 ab9 aba abf ac2 ac7 ac8 acd
ad0 ad5 ad6 adb ade ae2 ae6 aea
aee af1 af2 af4 af9 afa afc afd
b02 b05 b0a b0b b10 b13 b18 b19
b1e b21 b25 b26 b2b b2e b33 b34
b39 b3a b3f b41 b45 b48 b4c b50
b53 b57 b5b b5e b61 b66 b69 b6c
b6f b70 b75 b76 1 b78 b7d b81
b85 b88 b8d b92 b95 b99 b9a b9f
ba0 ba5 ba7 bab bae bb2 bb6 bb9
bbd bc1 bc4 bc8 bcc bcd bd2 bd4
bd8 bdb bdf be3 be6 be7 be9 bec
bef bf0 bf5 bf9 bfd c00 c04 c08
c0b c0e c13 c16 c19 c1c c1d c22
c23 1 c25 c2a c2e c32 c35 c3a
c3f c42 c46 c47 c4c c4f c54 c55
c5a c5d c62 c63 c68 c6b c6f c73
c77 c7b c7e c7f c81 c86 c87 c89
c8a c8f c92 c97 c98 c9d ca0 ca5
ca6 cab cae cb2 cb3 cb8 cbb cc0
cc1 cc6 cc7 ccc cce cd2 cd5 cd9
cdd ce0 ce4 ce8 ceb cee cf3 cf6
cf9 cfc cfd d02 d03 1 d05 d0a
d0e d12 d15 d1a d1f d22 d26 d27
d2c d2d d32 d34 d38 d3b d3f d43
d46 d4a d4e d51 d55 d59 d5a d5f
d61 d65 d68 d6c d6f d70 d75 d78
d7b d7f d82 d83 d88 d8b 1 d8e
d93 d97 d9b d9e da2 da6 da9 daa
dac daf db2 db3 db8 dbc dc0 dc3
dc7 dcb dce dd1 dd6 dd9 ddc ddf
de0 de5 de6 1 de8 ded df1 df5
df8 dfd e02 e05 e09 e0a e0f e12
e17 e18 e1d e20 e25 e26 e2b e2e
e32 e36 e3a e3e e41 e42 e44 e49
e4a e4c e4d e52 e55 e5a e5b e60
e63 e68 e69 e6e e71 e75 e76 e7b
e7e e83 e84 e89 e8a e8f e91 e95
e98 e9c ea0 ea3 ea7 eab eae eb1
eb6 eb9 ebc ebf ec0 ec5 ec6 1
ec8 ecd ed1 ed5 ed8 edd ee2 ee5
ee9 eea eef ef0 ef5 ef7 efb efe
f02 f06 f09 f0d f11 f14 f18 f1c
f1d f22 f24 f28 f2b f2d f31 f34
f38 f39 f3d f41 f45 f49 f4d f51
f55 f59 f5d f61 f65 f69 f6d f71
f75 f79 f85 f89 f8d f90 f94 f98
f9a f9e fa2 fa6 fa9 fad fb1 fb3
fb4 fb9 1 fbd fc1 fc5 fc8 fcc
fd0 fd4 fd8 fdb fdf fe3 fe6 fe9
fee ff1 ff4 ff7 ff8 ffd ffe 1
1000 1005 1009 100d 1010 1015 101a 101d
1021 1022 1027 102a 102f 1030 1035 1038
103d 103e 1043 1046 104a 104b 1050 1053
1058 1059 105e 1061 1065 1069 106c 106f
1070 1072 1073 1078 1079 107e 1080 1084
1087 1089 108d 108f 109b 109d 109f 10a0
10a5 10a9 10ab 10b7 10b9 10bd 10c1 10c5
10c9 10cd 10d0 10d4 10d8 10d9 10db 10de
10e1 10e2 10e7 10ea 10ee 10f2 10f5 10f6
10f8 10fc 10ff 1103 1106 1107 110c 1
110f 1114 1118 111d 1121 1125 1128 112c
1130 1134 1138 113b 113f 1143 1147 114b
114f 1153 1157 115a 115b 115d 1161 1165
1169 116c 1170 1174 1178 117d 1181 1185
1188 118c 1190 1194 1198 119b 119f 11a3
11a7 11ab 11af 11b3 11b7 11ba 11bb 11bd
11c1 11c5 11c9 11cc 11d0 11d4 11d6 11da
11dd 11e1 11e4 11e5 11ea 11ed 11f0 11f4
11f8 11fb 11ff 1203 1206 1207 1209 120c
120f 1210 1215 1 1218 121d 1221 1225
1229 122b 122f 1232 1236 1239 123a 123f
1242 1245 1249 124c 124d 1252 1255 1
1258 125d 1261 1265 1269 126d 1271 1275
1279 127d 1289 128d 1292 1296 1299 129a
129f 12a3 12a7 12aa 12ad 12b1 12b3 12b7
12bb 12be 12c1 12c5 12c7 12cb 12cf 12d2
12d4 12d8 12dc 12e0 12e3 12e7 12eb 12ed
12ee 12f3 1 12f7 12fb 12ff 1302 1306
130a 130e 1312 1315 1319 131d 1320 1323
1328 132b 132e 1331 1332 1337 1338 1
133a 133f 1343 1347 134a 134f 1354 1357
135b 135c 1361 1364 1369 136a 136f 1372
1377 1378 137d 1380 1384 1385 138a 138d
1392 1393 1398 139b 139f 13a3 13a6 13a9
13aa 13ac 13ad 13b2 13b3 13b8 13ba 13be
13c1 13c3 13c7 13c9 13d5 13d7 13d9 13da
13df 13e3 13e5 13f1 13f3 13f5 13f9 13fc
13fd 1402 1405 1409 140c 140d 1412 1
1415 141a 141e 1422 1426 142a 142e 1432
143e 1442 1447 144b 144e 144f 1454 1458
145c 145f 1462 1466 1468 146c 1470 1473
1476 147a 147c 1480 1484 1487 1489 148d
1491 1495 1498 149c 14a0 14a2 14a3 14a8
1 14ac 14b0 14b4 14b7 14bb 14bf 14c3
14c7 14ca 14ce 14d2 14d5 14d8 14dd 14e0
14e3 14e6 14e7 14ec 14ed 1 14ef 14f4
14f8 14fc 14ff 1504 1509 150c 1510 1511
1516 1519 151e 151f 1524 1527 152c 152d
1532 1535 1539 153a 153f 1542 1547 1548
154d 1550 1554 1558 155b 155e 155f 1561
1562 1567 1568 156d 156f 1573 1576 1578
157c 157e 158a 158c 158e 158f 1594 1598
159a 15a6 15a8 15ac 15b0 15b3 15b7 15bb
15be 15bf 15c1 15c4 15c7 15c8 15cd 15d1
15d4 15d8 15da 15de 15e1 15e3 15e7 15ea
15ee 15f1 15f2 15f7 15fa 15fe 1601 1602
1607 1 160a 160f 1613 1617 161b 161f
1623 1627 1633 1637 163c 1640 1643 1644
1649 164d 1651 1654 1657 165b 165d 1661
1665 1668 166b 166f 1671 1675 1679 167c
167e 1682 1686 168a 168d 1691 1695 1697
1698 169d 1 16a1 16a5 16a9 16ac 16b0
16b4 16b8 16bc 16bf 16c3 16c7 16ca 16cd
16d2 16d5 16d8 16db 16dc 16e1 16e2 1
16e4 16e9 16ed 16f1 16f4 16f9 16fe 1701
1705 1706 170b 170e 1713 1714 1719 171c
1721 1722 1727 172a 172e 172f 1734 1737
173c 173d 1742 1745 1749 174d 1750 1753
1754 1756 1757 175c 175d 1762 1764 1768
176b 176d 1771 1773 177f 1781 1783 1784
1789 178d 178f 179b 179d 17a1 17a5 17a8
17ac 17b0 17b3 17b4 17b6 17b9 17bc 17bd
17c2 17c6 17c9 17cd 17cf 17d3 17d6 17d8
17dc 17df 17e3 17e7 17ea 17eb 17ed 17f0
17f3 17f4 17f9 17fd 1800 1801 1806 1809
180c 1810 1814 1817 181b 181f 1822 1823
1825 1828 182b 182c 1831 1 1834 1839
183d 1841 1845 1847 184b 184e 1852 1855
1856 185b 185e 1861 1865 1868 1869 186e
1871 1 1874 1879 187d 1881 1884 1888
188c 188d 188f 1892 1895 1896 189b 1
189e 18a3 18a7 18ab 18ae 18b2 18b6 18b9
18bc 18c1 18c4 18c7 18ca 18cb 18d0 18d1
1 18d3 18d8 18dc 18e0 18e3 18e8 18ed
18f0 18f4 18f5 18fa 18fd 1902 1903 1908
190b 1910 1911 1916 1919 191d 1921 1925
1929 192c 192d 192f 1934 1935 1937 1938
193d 1940 1945 1946 194b 194e 1953 1954
1959 195c 1960 1961 1966 1969 196e 196f
1974 1975 197a 197c 1980 1983 1987 198b
198e 1992 1996 1999 199c 19a1 19a4 19a7
19aa 19ab 19b0 19b1 1 19b3 19b8 19bc
19c0 19c3 19c8 19cd 19d0 19d4 19d5 19da
19db 19e0 19e2 19e6 19e9 19ed 19f1 19f4
19f8 19fc 19ff 1a03 1a07 1a08 1a0d 1a0f
1a13 1a16 1a1a 1a1d 1a1e 1a23 1a26 1a2a
1a2d 1a2e 1a33 1a36 1a3a 1a3e 1a41 1a45
1a49 1a4c 1a4f 1a54 1a57 1a5a 1a5d 1a5e
1a63 1a64 1 1a66 1a6b 1a6f 1a73 1a76
1a7b 1a80 1a83 1a87 1a88 1a8d 1a90 1a95
1a96 1a9b 1a9e 1aa3 1aa4 1aa9 1aac 1ab0
1ab4 1ab8 1abc 1abf 1ac0 1ac2 1ac7 1ac8
1aca 1acb 1ad0 1ad3 1ad8 1ad9 1ade 1ae1
1ae6 1ae7 1aec 1aef 1af3 1af4 1af9 1afc
1b01 1b02 1b07 1b08 1b0d 1b0f 1b13 1b16
1b1a 1b1e 1b21 1b25 1b29 1b2c 1b2f 1b34
1b37 1b3a 1b3d 1b3e 1b43 1b44 1 1b46
1b4b 1b4f 1b53 1b56 1b5b 1b60 1b63 1b67
1b68 1b6d 1b6e 1b73 1b75 1b79 1b7c 1b80
1b84 1b87 1b8b 1b8f 1b92 1b96 1b9a 1b9b
1ba0 1ba2 1ba6 1ba9 1bad 1bb1 1bb5 1bb9
1bbd 1bc0 1bc4 1bc8 1bcb 1bcc 1bce 1bd1
1bd4 1bd5 1bda 1bde 1bdf 1be3 1be7 1be9
1bed 1bf1 1bf4 1bf8 1bfc 1bff 1c00 1c02
1c05 1c08 1c09 1c0e 1c12 1c16 1c1a 1c1e
1c22 1c26 1c2a 1c2e 1c32 1c36 1c3a 1c3e
1c42 1c46 1c4a 1c4e 1c5a 1c5e 1c62 1c65
1c69 1c6d 1c6f 1c73 1c77 1c7b 1c7e 1c82
1c86 1c88 1c89 1c8e 1 1c92 1c96 1c9a
1c9d 1ca1 1ca5 1ca9 1cad 1cb0 1cb4 1cb8
1cbb 1cbe 1cc3 1cc6 1cc9 1ccc 1ccd 1cd2
1cd3 1 1cd5 1cda 1cde 1ce2 1ce5 1cea
1cef 1cf2 1cf6 1cf7 1cfc 1cff 1d04 1d05
1d0a 1d0d 1d12 1d13 1d18 1d1b 1d1f 1d20
1d25 1d28 1d2d 1d2e 1d33 1d36 1d3a 1d3e
1d41 1d44 1d45 1d47 1d48 1d4d 1d4e 1d53
1d55 1d59 1d5c 1d5e 1d62 1d64 1d70 1d72
1d74 1d75 1d7a 1d7e 1d80 1d8c 1d8e 1d92
1d96 1d99 1d9d 1da0 1da4 1da7 1da8 1dad
1db0 1db4 1db8 1dbc 1dc0 1dc4 1dc8 1dcc
1dd0 1dd4 1de0 1de4 1de8 1deb 1def 1df3
1df5 1df9 1dfd 1e01 1e04 1e08 1e0c 1e0e
1e0f 1e14 1 1e18 1e1c 1e20 1e23 1e27
1e2b 1e2f 1e33 1e36 1e3a 1e3e 1e41 1e44
1e49 1e4c 1e4f 1e52 1e53 1e58 1e59 1
1e5b 1e60 1e64 1e68 1e6b 1e70 1e75 1e78
1e7c 1e7d 1e82 1e85 1e8a 1e8b 1e90 1e93
1e98 1e99 1e9e 1ea1 1ea5 1ea6 1eab 1eae
1eb3 1eb4 1eb9 1ebc 1ec0 1ec4 1ec7 1eca
1ecb 1ecd 1ece 1ed3 1ed4 1ed9 1edb 1edf
1ee2 1ee4 1ee8 1eea 1ef6 1ef8 1efa 1efb
1f00 1f04 1f06 1f12 1f14 1f16 1f1a 1f1d
1f21 1f23 1f27 1f2b 1f2e 1f2f 1f34 1f37
1f3b 1f3f 1f42 1f46 1f4a 1f4d 1f50 1f55
1f58 1f5b 1f5e 1f5f 1f64 1f65 1 1f67
1f6c 1f70 1f74 1f77 1f7c 1f81 1f84 1f88
1f89 1f8e 1f91 1f96 1f97 1f9c 1f9f 1fa4
1fa5 1faa 1fad 1fb1 1fb5 1fb8 1fb9 1fbb
1fc0 1fc1 1fc3 1fc4 1fc9 1fcc 1fd1 1fd2
1fd7 1fda 1fdf 1fe0 1fe5 1fe8 1fec 1fed
1ff2 1ff5 1ffa 1ffb 2000 2001 2006 2008
200c 200f 2013 2017 201a 201e 2022 2025
2028 202d 2030 2033 2036 2037 203c 203d
1 203f 2044 2048 204c 204f 2054 2059
205c 2060 2061 2066 2067 206c 206e 2072
2075 2079 207d 2080 2083 2087 208b 208c
2091 2093 2097 209b 209e 20a0 20a4 20a7
20ab 20af 20b0 20b4 20b6 20ba 20be 20c1
20c5 20c8 20c9 20ce 20d1 20d5 20d8 20d9
20de 20e1 20e4 20e8 20ec 20ef 20f3 20f7
20fa 20fb 20fd 2100 2103 2104 2109 1
210c 2111 2115 2119 211c 2120 2124 2127
212a 212f 2132 2135 2138 2139 213e 213f
1 2141 2146 214a 214e 2151 2156 215b
215e 2162 2163 2168 216b 2170 2171 2176
2179 217e 217f 2184 2187 218b 218f 2193
2197 219a 219b 219d 21a2 21a3 21a5 21a6
21ab 21ae 21b3 21b4 21b9 21bc 21c1 21c2
21c7 21ca 21ce 21cf 21d4 21d7 21dc 21dd
21e2 21e3 21e8 21ea 21ee 21f1 21f5 21f9
21fc 2200 2204 2207 220a 220f 2212 2215
2218 2219 221e 221f 1 2221 2226 222a
222e 2231 2236 223b 223e 2242 2243 2248
2249 224e 2250 2254 2257 225b 225f 2262
2266 226a 226d 2271 2275 2276 227b 227f
2281 2285 2288 2289 228e 2291 2294 2298
229c 229f 22a3 22a7 22aa 22ab 22ad 22b0
22b3 22b4 22b9 1 22bc 22c1 22c5 22c9
22cc 22d0 22d4 22d7 22da 22df 22e2 22e5
22e8 22e9 22ee 22ef 1 22f1 22f6 22fa
22fe 2301 2306 230b 230e 2312 2313 2318
231b 2320 2321 2326 2329 232e 232f 2334
2337 233b 233f 2343 2347 234a 234b 234d
2352 2353 2355 2356 235b 235c 2361 2363
2367 236a 236e 2372 2375 2379 237d 2380
2383 2388 238b 238e 2391 2392 2397 2398
1 239a 239f 23a3 23a7 23aa 23af 23b4
23b7 23bb 23bc 23c1 23c2 23c7 23c9 23cd
23d0 23d4 23d8 23db 23df 23e3 23e6 23ea
23eb 23f0 23f2 23f6 23fa 23fd 2401 2404
2405 240a 240d 2410 2414 2418 241b 241f
2423 2426 2427 2429 242c 242f 2430 2435
1 2438 243d 2441 2444 2448 244a 244e
2451 2455 2457 245b 245f 2462 2466 246a
246d 2471 2475 2478 2479 247b 247e 2481
2482 2487 248b 248c 2490 2494 2496 249a
249e 24a1 24a5 24a9 24aa 24ac 24af 24b2
24b3 24b8 24bc 24c0 24c4 24c8 24cc 24d0
24d4 24d8 24dc 24e0 24e4 24e8 24ec 24f0
24f4 24f8 2504 2508 250c 250f 2513 2517
2519 251d 2521 2525 2528 252c 2530 2532
2533 2538 1 253c 2540 2544 2547 254b
254f 2553 2557 255a 255e 2562 2565 2568
256d 2570 2573 2576 2577 257c 257d 1
257f 2584 2588 258c 258f 2594 2599 259c
25a0 25a1 25a6 25a9 25ae 25af 25b4 25b7
25bc 25bd 25c2 25c5 25c9 25ca 25cf 25d2
25d7 25d8 25dd 25e0 25e4 25e8 25eb 25ee
25ef 25f1 25f2 25f7 25f8 25fd 25ff 2603
2606 2608 260c 260e 261a 261c 261e 261f
2624 2628 262a 2636 2638 263c 2640 2643
2647 264a 264e 2651 2652 2657 265a 265e
2662 2666 266a 266e 2672 2676 267a 267e
268a 268e 2692 2695 2699 269d 269f 26a3
26a7 26ab 26ae 26b2 26b6 26b8 26b9 26be
1 26c2 26c6 26ca 26cd 26d1 26d5 26d9
26dd 26e0 26e4 26e8 26eb 26ee 26f3 26f6
26f9 26fc 26fd 2702 2703 1 2705 270a
270e 2712 2715 271a 271f 2722 2726 2727
272c 272f 2734 2735 273a 273d 2742 2743
2748 274b 274f 2750 2755 2758 275d 275e
2763 2766 276a 276e 2771 2774 2775 2777
2778 277d 277e 2783 2785 2789 278c 278e
2792 2794 27a0 27a2 27a4 27a5 27aa 27ae
27b0 27bc 27be 27c0 27c4 27c7 27cb 27cd
27d1 27d5 27d9 27dc 27e0 27e4 27e7 27e8
27ea 27ed 27f0 27f1 27f6 27fa 27fe 2801
2805 2809 280c 280f 2814 2817 281a 281d
281e 2823 2824 1 2826 282b 282f 2833
2836 283b 2840 2843 2847 2848 284d 2850
2855 2856 285b 285e 2863 2864 2869 286c
2870 2874 2877 2878 287a 287f 2880 2882
2883 2888 288b 2890 2891 2896 2899 289e
289f 28a4 28a7 28ab 28ac 28b1 28b4 28b9
28ba 28bf 28c0 28c5 28c7 28cb 28ce 28d2
28d6 28d9 28dd 28e1 28e4 28e7 28ec 28ef
28f2 28f5 28f6 28fb 28fc 1 28fe 2903
2907 290b 290e 2913 2918 291b 291f 2920
2925 2926 292b 292d 2931 2934 2938 293c
293f 2942 2946 294a 294b 2950 2952 2956
295a 295d 295f 2963 2964 2968 296a 296e
2972 2975 2979 297d 2981 2985 2988 298c
2990 2994 2997 2998 299a 299e 29a1 29a5
29a8 29a9 29ae 29b2 29b6 29ba 29be 29c2
29c6 29ca 29ce 29da 29de 29e3 29e7 29ea
29eb 29f0 29f4 29f8 29fb 29fe 2a02 2a04
2a08 2a0c 2a0f 2a12 2a16 2a18 2a1c 2a20
2a23 2a25 2a29 2a2d 2a31 2a34 2a38 2a3c
2a3e 2a3f 2a44 1 2a48 2a4c 2a50 2a53
2a57 2a5b 2a5f 2a63 2a66 2a6a 2a6e 2a71
2a74 2a79 2a7c 2a7f 2a82 2a83 2a88 2a89
1 2a8b 2a90 2a94 2a98 2a9b 2aa0 2aa5
2aa8 2aac 2aad 2ab2 2ab5 2aba 2abb 2ac0
2ac3 2ac8 2ac9 2ace 2ad1 2ad5 2ad6 2adb
2ade 2ae3 2ae4 2ae9 2aec 2af0 2af4 2af7
2afa 2afb 2afd 2afe 2b03 2b04 2b09 2b0b
2b0f 2b12 2b14 2b18 2b1a 2b26 2b28 2b2a
2b2b 2b30 2b34 2b36 2b42 2b44 2b46 2b4a
2b4d 2b4e 2b53 2b56 2b5a 2b5d 2b5e 2b63
2b66 2b6a 2b6e 2b72 2b76 2b7a 2b7e 2b82
2b86 2b8a 2b8e 2b92 2b96 2b9a 2b9e 2ba2
2bae 2bb2 2bb6 2bb9 2bbd 2bc1 2bc3 2bc7
2bcb 2bcf 2bd2 2bd6 2bda 2bdc 2bdd 2be2
1 2be6 2bea 2bee 2bf1 2bf5 2bf9 2bfd
2c01 2c04 2c08 2c0c 2c0f 2c12 2c17 2c1a
2c1d 2c20 2c21 2c26 2c27 1 2c29 2c2e
2c32 2c36 2c39 2c3e 2c43 2c46 2c4a 2c4b
2c50 2c53 2c58 2c59 2c5e 2c61 2c66 2c67
2c6c 2c6f 2c73 2c74 2c79 2c7c 2c81 2c82
2c87 2c8a 2c8e 2c92 2c95 2c98 2c99 2c9b
2c9c 2ca1 2ca2 2ca7 2ca9 2cad 2cb0 2cb2
2cb6 2cb8 2cc4 2cc6 2cc8 2cc9 2cce 2cd2
2cd4 2ce0 2ce2 2ce6 2cea 2ced 2cee 2cf0
2cf3 2cf6 2cf7 2cfc 2d00 2d04 2d07 2d0b
2d0f 2d12 2d15 2d1a 2d1d 2d20 2d23 2d24
2d29 2d2a 1 2d2c 2d31 2d35 2d39 2d3c
2d41 2d46 2d49 2d4d 2d4e 2d53 2d56 2d5b
2d5c 2d61 2d64 2d69 2d6a 2d6f 2d72 2d76
2d7a 2d7e 2d82 2d85 2d86 2d88 2d8d 2d8e
2d90 2d91 2d96 2d99 2d9e 2d9f 2da4 2da7
2dac 2dad 2db2 2db5 2db9 2dba 2dbf 2dc2
2dc7 2dc8 2dcd 2dce 2dd3 2dd5 2dd9 2ddc
2de0 2de4 2de7 2deb 2def 2df2 2df5 2dfa
2dfd 2e00 2e03 2e04 2e09 2e0a 1 2e0c
2e11 2e15 2e19 2e1c 2e21 2e26 2e29 2e2d
2e2e 2e33 2e34 2e39 2e3b 2e3f 2e42 2e46
2e4a 2e4d 2e51 2e55 2e58 2e5c 2e60 2e61
2e66 2e68 2e6c 2e6f 2e73 2e75 2e79 2e7c
2e7d 2e82 2e85 2e89 2e8d 2e91 2e95 2e99
2e9d 2ea1 2ea5 2ea9 2ead 2eb1 2eb5 2eb9
2ebd 2ec1 2ecd 2ed1 2ed5 2ed8 2edc 2ee0
2ee2 2ee6 2eea 2eee 2ef1 2ef5 2ef9 2efb
2efc 2f01 1 2f05 2f09 2f0d 2f10 2f14
2f18 2f1c 2f20 2f23 2f27 2f2b 2f2e 2f31
2f36 2f39 2f3c 2f3f 2f40 2f45 2f46 1
2f48 2f4d 2f51 2f55 2f58 2f5d 2f62 2f65
2f69 2f6a 2f6f 2f72 2f77 2f78 2f7d 2f80
2f85 2f86 2f8b 2f8e 2f92 2f93 2f98 2f9b
2fa0 2fa1 2fa6 2fa9 2fad 2fb1 2fb4 2fb7
2fb8 2fba 2fbb 2fc0 2fc1 2fc6 2fc8 2fcc
2fcf 2fd1 2fd5 2fd7 2fe3 2fe5 2fe7 2fe8
2fed 2ff1 2ff3 2fff 3001 3005 3009 300c
300d 300f 3012 3015 3016 301b 301f 3023
3026 302a 302e 3031 3034 3039 303c 303f
3042 3043 3048 3049 1 304b 3050 3054
3058 305b 3060 3065 3068 306c 306d 3072
3075 307a 307b 3080 3083 3088 3089 308e
3091 3095 3099 309d 30a1 30a4 30a5 30a7
30ac 30ad 30af 30b0 30b5 30b8 30bd 30be
30c3 30c6 30cb 30cc 30d1 30d4 30d8 30d9
30de 30e1 30e6 30e7 30ec 30ed 30f2 30f4
30f8 30fb 30ff 3103 3106 310a 310e 3111
3114 3119 311c 311f 3122 3123 3128 3129
1 312b 3130 3134 3138 313b 3140 3145
3148 314c 314d 3152 3153 3158 315a 315e
3161 3165 3169 316c 3170 3174 3177 317b
317f 3180 3185 3187 318b 318f 3193 3197
319b 319f 31a3 31a7 31ab 31af 31b3 31b7
31c3 31c7 31cb 31ce 31d2 31d6 31d8 31dc
31e0 31e4 31e7 31eb 31ef 31f1 31f2 31f7
1 31fb 31ff 3203 3206 320a 320e 3212
3216 3219 321d 3221 3224 3227 322c 322f
3232 3235 3236 323b 323c 1 323e 3243
3247 324b 324e 3253 3258 325b 325f 3260
3265 3268 326d 326e 3273 3276 327b 327c
3281 3284 3288 3289 328e 3291 3296 3297
329c 329f 32a3 32a7 32aa 32ad 32ae 32b0
32b1 32b6 32b7 32bc 32be 32c2 32c5 32c7
32cb 32cd 32d9 32db 32dd 32de 32e3 32e7
32e9 32f5 32f7 32fb 32ff 3302 3303 3305
3309 330c 3310 3313 3314 3316 3317 331c
3320 3324 3327 332b 332f 3332 3335 333a
333d 3340 3343 3344 3349 334a 1 334c
3351 3355 3359 335c 3361 3366 3369 336d
336e 3373 3376 337b 337c 3381 3384 3389
338a 338f 3392 3396 339a 339e 33a2 33a5
33a6 33a8 33ad 33ae 33b0 33b1 33b6 33b9
33be 33bf 33c4 33c7 33cc 33cd 33d2 33d5
33d9 33da 33df 33e2 33e7 33e8 33ed 33ee
33f3 33f5 33f9 33fc 3400 3404 3407 340b
340f 3412 3415 341a 341d 3420 3423 3424
3429 342a 1 342c 3431 3435 3439 343c
3441 3446 3449 344d 344e 3453 3454 3459
345b 345f 3462 3466 346a 346d 3471 3475
3478 347c 3480 3481 3486 3488 348c 348f
3491 3495 3499 349c 349e 34a2 34a6 34aa
34ad 34b1 34b5 34b8 34bb 34c0 34c3 34c6
34c9 34ca 34cf 34d0 1 34d2 34d7 34db
34df 34e2 34e7 34ec 34ef 34f3 34f4 34f9
34fc 3501 3502 3507 350a 350f 3510 3515
3518 351c 3520 3524 3528 352b 352c 352e
3533 3534 3536 3537 353c 353f 3544 3545
354a 354d 3552 3553 3558 355b 355f 3560
3565 3568 356d 356e 3573 3574 3579 357b
357f 3582 3586 358a 358d 3591 3595 3598
359b 35a0 35a3 35a6 35a9 35aa 35af 35b0
1 35b2 35b7 35bb 35bf 35c2 35c7 35cc
35cf 35d3 35d4 35d9 35da 35df 35e1 35e5
35e8 35ec 35f0 35f3 35f7 35fb 35fe 3602
3606 3607 360c 360e 3612 3616 3619 361b
361f 3622 3626 3629 362d 3631 3635 3639
363d 3640 3641 3646 3649 364d 3650 3651
3656 3659 1 365c 3661 3665 3669 366d
366f 3673 3676 367a 367e 3682 3686 368a
368e 3692 3696 369a 369e 36a2 36a6 36aa
36ae 36b2 36b6 36ba 36be 36c2 36c6 36ca
36ce 36d2 36d6 36d9 36da 36dc 36e0 36e4
36e7 36eb 36ef 36f3 36f7 36fb 36ff 3702
3705 3706 3708 370c 3710 3713 3714 3719
371c 3720 3723 3727 372b 372f 3733 3737
373b 373e 3741 3742 3744 3748 374a 374e
3751 3753 3757 375b 375e 3760 3764 3767
3769 376d 3771 3774 3778 377c 377f 3783
3787 3788 378a 378d 3790 3791 3796 3799
379d 37a1 37a4 37a5 37a7 37ab 37ae 37b2
37b5 37b6 37bb 1 37be 37c3 37c7 37cc
37d0 37d4 37d7 37db 37df 37e3 37e7 37ea
37ee 37f2 37f6 37fa 37fe 3802 3806 3809
380a 380c 3810 3814 3818 381b 381f 3823
3827 382c 3830 3834 3837 383b 383f 3843
3847 384a 384e 3852 3856 385a 385e 3862
3866 3869 386a 386c 3870 3874 3878 387b
387f 3883 3885 3889 388c 3890 3894 3898
3899 389b 389f 38a3 38a7 38aa 38ae 38b2
38b5 38b8 38bd 38c0 38c3 38c6 38c7 38cc
38cd 1 38cf 38d4 38d8 38dc 38df 38e4
38e9 38ec 38f0 38f1 38f6 38f9 38fe 38ff
3904 3907 390c 390d 3912 3915 3919 391d
3921 3925 3928 3929 392b 3930 3931 3933
3934 3939 393c 3941 3942 3947 394a 394f
3950 3955 3958 395c 395d 3962 3965 396a
396b 3970 3971 3976 3978 397c 397f 3983
3987 398a 398e 3992 3995 3999 399d 399e
39a3 39a5 39a9 39ad 39b0 39b4 39b8 39bb
39bf 39c3 39c6 39c9 39ce 39d1 39d4 39d7
39d8 39dd 39de 1 39e0 39e5 39e9 39ed
39f0 39f5 39fa 39fd 3a01 3a02 3a07 3a08
3a0d 3a11 3a15 3a18 3a1d 3a22 3a25 3a29
3a2d 3a30 3a31 3a36 3a39 3a3e 3a3f 3a44
3a47 3a4b 3a4f 3a52 3a53 3a58 3a59 3a5e
3a60 3a64 3a67 3a6b 3a6f 3a73 3a77 3a7b
3a7f 3a83 3a87 3a8b 3a97 3a9c 3aa0 3aa4
3aa8 3aac 3aae 3aaf 3ab4 1 3ab8 3abc
3ac0 3ac3 3ac7 3acb 3acf 3ad3 3ad6 3ada
3ade 3ae1 3ae4 3ae9 3aec 3aef 3af2 3af3
3af8 3af9 1 3afb 3b00 3b04 3b08 3b0b
3b10 3b15 3b18 3b1c 3b1d 3b22 3b23 3b28
3b2c 3b30 3b33 3b38 3b3d 3b40 3b44 3b45
3b4a 3b4d 3b52 3b53 3b58 3b5b 3b5f 3b63
3b66 3b69 3b6a 3b6c 3b6d 3b72 3b73 3b78
3b7a 3b7e 3b81 3b85 3b89 3b8d 3b91 3b95
3b99 3b9d 3ba9 3bae 3bb2 3bb6 3bba 3bbe
3bc0 3bc4 3bc6 3bd2 3bd4 3bd6 3bd7 3bdc
3be0 3be2 3bee 3bf2 3bf4 3bf5 3bfe 
112f
2
0 1 a 6 9 :3 6 14 :3 6
15 :3 6 16 :3 6 12 :2 6 1 3
a 13 a :2 1 4 11 1a 19
:2 11 :2 4 e 17 16 :2 e :2 4 18
21 20 :2 18 :2 4 18 21 20 :2 18
:2 4 18 21 20 :2 18 :2 4 d 16
15 :2 d :2 4 f 18 17 :2 f :2 4
12 1b 1a :2 12 :2 4 :3 11 :2 4 :3 12
:2 4 :3 f :2 4 :3 10 :2 4 :3 15 :2 4 :3 16
:2 4 :3 13 :2 4 b 14 13 :2 b :2 4
:2 13 22 13 :2 4 :2 b 1a b :2 4
:2 f 19 f :2 4 :2 f 19 f :2 4
:3 12 :2 4 :3 13 :2 4 :3 15 :2 4 :3 19 :2 4
:3 10 :2 4 :3 e :2 4 f 18 17 :2 f
:2 4 :2 f 1e f :2 4 :2 16 25 16
:2 4 :2 a 19 a :2 4 :2 c 1b c
:2 4 :2 11 20 11 :2 4 :2 f 1e f
:2 4 :2 d 1c d :2 4 :2 e 1d e
:2 4 :2 e 1d e :2 4 :2 16 25 16
:2 4 :2 10 1f 10 :2 4 :2 9 18 9
:2 4 :2 f 1e f :2 4 :2 14 23 14
:2 4 :2 11 20 11 :2 4 f 18 17
:2 f :2 4 :3 10 :2 4 :2 16 2e 16 :2 4
e 17 :3 e 4 7 :2 11 23 :2 2d
3a :2 23 :3 7 :2 d 19 25 :2 2f 43
:2 25 58 5e 61 :2 58 6f 89 8
:2 6f d 10 :2 6f 13 16 :2 6f b
e :2 6f 13 16 1f 26 2b 2e
:2 26 3c 3f :2 26 44 :2 1f 4d :2 16
:2 6f b e :2 6f 13 16 1f 26
2b 2e :2 26 3d 40 :2 26 45 :2 1f
4e :2 16 :2 6f b e :2 6f 13 16
1f 27 :2 1f 39 :2 16 :2 6f b e
:2 6f 13 16 :2 6f 1b 1e :2 6f 2a
2d :2 6f 32 35 :2 6f :2 7 4e :2 4
7 :2 11 23 :2 2d 39 3c 50 52
53 :2 50 :2 23 :3 7 :2 d 12 1e 61
64 :2 1e :2 7 56 :3 4 14 :2 4 15
:2 4 12 :2 4 14 :2 4 15 :2 4 1b
:2 4 19 4 8 11 24 :2 8 27
29 :2 27 :2 7 1c :2 25 7 2c :2 4
e 1a 2d 44 52 65 7 10
1f 2d 36 46 55 65 76 7f
8c 99 a8 b5 c3 7 12 22
34 47 51 62 70 c 1a 2f
44 54 64 7 13 20 32 45
50 5a 67 74 7 12 24 33
43 4f 62 6e c d 21 :2 7
:2 10 1c 7 4 c a :2 13 1f
a 1a :2 7 :2 c a :2 13 1f a
d :2 17 29 :2 33 3f 43 57 5a
5b :2 57 :2 29 :3 d :2 13 18 24 68
6b :2 24 78 7b :2 24 7f 82 :2 24
8e 91 :2 24 99 9c :2 24 a8 ab
b2 bb be :2 ab :2 24 :2 d 5e :2 a
:2 7 :4 13 :2 7 4 :3 1 8 :2 11 1c
1a :2 25 :2 1a 7 a :2 14 26 :2 30
3c 40 54 57 58 :2 54 :2 26 :3 a
:2 10 15 21 65 68 :2 21 75 78
:2 21 7c 7f :2 21 89 8c 95 9d
:2 a6 :2 95 b9 :2 8c :2 21 c1 c4 :2 21
cc cf :2 21 d4 d7 :2 21 da dd
:2 21 :2 a 5b :2 7 a :2 14 26 :2 30
3c 3f 53 56 57 :2 53 :2 26 :3 a
:2 10 15 21 65 68 :2 21 :2 a 5a
:3 7 :2 12 26 :2 2f 41 45 :2 7 35
:2 4 7 10 20 :2 7 23 25 :2 23
a :2 14 26 :2 30 3c 40 54 57
58 :2 54 :2 26 :3 a :2 10 15 21 65
68 :2 21 75 78 :2 21 7c 7f :2 21
89 8c 95 9d :2 a6 :2 95 c2 :2 8c
:2 21 ca cd :2 21 d5 d8 :2 21 dd
e0 :2 21 e3 e6 :2 21 :2 a 5b :2 7
a :2 14 26 :2 30 3c 3f 53 56
57 :2 53 :2 26 :3 a :2 10 15 21 65
68 :2 21 :2 a 5a :3 7 :2 12 26 :2 2f
4a 4e :2 7 2b :2 4 :4 9 8 7
:4 2b 2a 29 :2 7 a :2 15 1d 2c
3a :2 a 3e 40 :2 3e d :2 17 29
:2 33 3f 43 57 5a 5b :2 57 :2 29
:3 d :2 13 18 24 68 6b :2 24 78
7b :2 24 7f 82 :2 24 8c 8f 98
a0 :2 a9 :2 98 c6 :2 8f :2 24 ce d1
:2 24 d9 dc :2 24 e1 e4 :2 24 e7
ea :2 24 :2 d 5e :2 a d :2 17 29
:2 33 3f 42 56 59 5a :2 56 :2 29
:3 d :2 13 18 24 68 6b :2 24 :2 d
5d :3 a :2 15 29 :2 32 4e 52 :2 a
42 :2 7 48 :3 4 11 4 e 12
29 38 d 1a 2b 39 b 1d
26 3f 48 61 6a :2 7 :2 10 1c
7 4 c a :2 13 1f a 1a
:2 7 :2 c a :2 13 1f a d :2 17
29 :2 33 3f 43 57 5a 5b :2 57
:2 29 :3 d :2 13 18 24 68 6b :2 24
78 7b :2 24 7f 82 :2 24 8e 91
:2 24 99 9c :2 24 a8 ab b2 bb
be :2 ab :2 24 :2 d 5e :2 a :2 7 :4 13
:2 7 4 :3 1 4 12 4 8 :2 13
1a 24 :2 8 30 32 :2 30 7 3a
a 1d :2 3a 23 20 :2 2c :2 20 39
:3 7 15 :2 7 17 :2 7 1c :2 25 :2 7
16 25 31 46 52 60 :2 16 :2 7
:2 10 1c :2 7 15 :2 7 17 :2 7 1c
:2 25 :2 7 16 25 31 46 52 60
:2 16 :2 7 :2 10 1c 7 47 :2 4 :4 9
8 7 29 :2 34 3c 4a 56 :2 29
5a 5c :2 5a 28 :3 7 17 7 5f
:2 4 :4 9 8 7 :4 27 26 25 :2 7
11 a 1c a 1c a 10 24
a 11 d 19 :3 17 d :2 16 22
d 1e d :2 16 22 d :4 a 7
f d :2 16 22 d 1d :2 a :2 f
d :2 16 22 d 10 :2 1a 2c :2 36
42 46 5a 5d 5e :2 5a :2 2c :3 10
:2 16 1b 27 6b 6e :2 27 7b 7e
:2 27 82 85 :2 27 91 94 :2 27 9c
9f :2 27 ab ae b5 be c1 :2 ae
:2 27 :2 10 61 :2 d :2 a :4 16 :2 a 7
:4 40 :4 b a :4 27 26 :2 a 14 d
1f d 13 27 d 14 10 1c
:3 1a 10 :2 19 25 10 21 10 :2 19
25 10 :4 d a 12 10 :2 19 25
10 20 :2 d :2 12 10 :2 19 25 10
13 :2 1d 2f :2 39 45 49 5d 60
61 :2 5d :2 2f :3 13 :2 19 1e 2a 6e
71 :2 2a 7e 81 :2 2a 85 88 :2 2a
94 97 :2 2a 9f a2 :2 2a ae b1
b8 c1 c4 :2 b1 :2 2a :2 13 64 :2 10
:2 d :4 19 :2 d a :3 45 d :2 18 20
2f 42 :2 d 46 48 :2 46 d 1b
d 4a :2 a 45 :2 7 :4 b a :4 28
27 :2 a 14 d 1f d 13 27
d 14 10 1c :3 1a 10 :2 19 25
10 21 10 :2 19 25 10 :4 d a
12 10 :2 19 25 10 20 :2 d :2 12
10 :2 19 25 10 13 :2 1d 2f :2 39
45 49 5d 60 61 :2 5d :2 2f :3 13
:2 19 1e 2a 6e 71 :2 2a 7e 81
:2 2a 85 88 :2 2a 94 97 :2 2a 9f
a2 :2 2a ae b1 b8 c1 c4 :2 b1
:2 2a :2 13 64 :2 10 :2 d :4 19 :2 d a
:3 47 d :2 18 20 2e 40 :2 d 44
46 :2 44 d 1b d 48 :2 a 47
:2 7 a 13 1f :2 a 22 24 :2 22
:4 f e d 30 :2 3b 43 52 5f
:2 30 63 65 :2 63 2f :3 d 1e d
68 :2 a :4 f e d :4 2d 2c 2b
:2 d 4e :2 59 60 12 :2 4e 21 23
:2 21 4d :2 d 10 :2 1a 2c :2 36 42
46 5a 5d 5e :2 5a :2 2c :3 10 :2 16
1b 27 6b 6e :2 27 7b 7e :2 27
82 85 :2 27 8f 92 9b a3 :2 ac
:2 9b c7 :2 92 :2 27 cf d2 :2 27 da
dd :2 27 e2 e5 :2 27 ee f1 :2 27
:2 10 61 :2 d 10 :2 1a 2c :2 36 42
45 59 5c 5d :2 59 :2 2c :3 10 :2 16
1b 27 6b 6e :2 27 :2 10 60 :3 d
:2 18 2c :2 35 4f 59 :2 d 26 :2 a
:4 e d :4 11 10 13 :2 1d 2f :2 39
45 49 5d 60 61 :2 5d :2 2f :3 13
:2 19 1e 2a 6e 71 :2 2a 7e 81
:2 2a 85 88 :2 2a 92 95 9e a6
:2 af :2 9e ca :2 95 :2 2a d2 d5 :2 2a
dd e0 :2 2a e5 e8 :2 2a eb ee
:2 2a :2 13 64 :2 10 13 :2 1d 2f :2 39
45 48 5c 5f 60 :2 5c :2 2f :3 13
:2 19 1e 2a 6e 71 :2 2a :2 13 63
:3 10 :2 1b 2f :2 38 52 56 :2 10 2d
:3 d 19 d 10 :2 1b 23 31 40
:2 10 44 46 :2 44 10 20 10 d
48 13 :2 1e 26 34 40 :2 13 44
47 :2 44 1a 25 32 3c 47 4c
55 66 78 18 19 26 17 22
2a 31 :2 13 :2 1c 28 13 10 18
16 :2 1f 2b 16 26 :2 13 :2 18 16
:2 1f 2b 16 19 :2 23 35 :2 3f 4b
4f 63 66 67 :2 63 :2 35 :3 19 :2 1f
24 30 74 77 :2 30 84 87 :2 30
8b 8e :2 30 9a 9d :2 30 a5 a8
:2 30 b4 b7 be c7 ca :2 b7 :2 30
:2 19 6a :2 16 :2 13 :4 1f :2 13 10 :3 49
14 :2 1d 28 26 :2 31 :2 26 13 1f
2a 37 3f 46 5e 69 80 92
:2 16 :2 1f 2b 16 13 1b 19 :2 22
2e 19 29 :2 16 :2 1b 19 :2 22 2e
19 1c :2 26 38 :2 42 4e 52 66
69 6a :2 66 :2 38 :3 1c :2 22 27 33
77 7a :2 33 87 8a :2 33 8e 91
:2 33 9d a0 :2 33 a8 ab :2 33 b7
ba c1 ca cd :2 ba :2 33 :2 1c 6d
:2 19 :2 16 :4 22 :2 16 13 :4 41 :2 10 d
49 48 :4 14 :2 13 :2 1d 2f :2 39 45
49 5d 60 61 :2 5d :2 2f :3 13 :2 19
1e 2a 6e 71 :2 2a 7e 81 :2 2a
85 88 :2 2a 92 95 9e a6 :2 9e
aa :2 95 :2 2a b2 b5 :2 2a bd c0
:2 2a c5 c8 :2 2a cb ce :2 2a :2 13
64 :2 10 13 :2 1d 2f :2 39 45 48
5c 5f 60 :2 5c :2 2f :3 13 :2 19 1e
2a 6e 71 :2 2a :2 13 63 :3 10 :2 1b
2f 32 36 :2 10 2c 48 :2 d 29
d 19 :2 d 1d d :4 a :4 e d
:4 12 11 10 2f :2 3a 42 51 60
:2 2f 64 66 :2 64 2e :2 10 13 :2 1d
2f :2 39 45 49 5d 60 61 :2 5d
:2 2f :3 13 :2 19 1e 2a 6e 71 :2 2a
7e 81 :2 2a 85 88 :2 2a 92 95
9e a6 :2 af :2 9e c0 :2 95 :2 2a c8
cb :2 2a d3 d6 :2 2a db de :2 2a
e1 e4 :2 2a :2 13 64 :2 10 13 :2 1d
2f :2 39 45 48 5c 5f 60 :2 5c
:2 2f :3 13 :2 19 1e 2a 6e 71 :2 2a
:2 13 63 :3 10 :2 1b 2f :2 38 48 4c
:2 10 d 69 :4 15 14 13 39 :2 44
4c 5b 15 :2 39 19 1c :2 19 38
:3 13 :2 1d 2f :2 39 45 48 5c 5f
60 :2 5c :2 2f :3 13 :2 19 1e 2a 6e
71 :2 2a 7e 81 :2 2a 85 88 :2 2a
92 95 9e a6 :2 af :2 9e ca :2 95
:2 2a :2 13 63 :2 10 13 :2 1d 2f :2 39
45 48 5c 5f 60 :2 5c :2 2f :3 13
:2 19 1e 2a 6e 71 :2 2a :2 13 63
:3 10 :2 1b 31 :2 3a 54 :2 10 1f 69
:2 d :4 12 11 10 2f :2 3a 42 51
5e :2 2f 62 64 :2 62 2e :3 10 18
10 67 10 18 10 :4 d 10 :2 1b
23 32 42 :2 10 46 48 :2 46 10
21 10 d 4a 13 :2 1e 25 34
:2 13 41 44 :2 41 1a 25 32 3c
47 4c 55 65 77 18 19 26
17 22 2a 31 :2 13 :2 1c 28 13
10 18 16 :2 1f 2b 16 26 :2 13
:2 18 16 :2 1f 2b 16 19 :2 23 35
:2 3f 4b 4f 63 66 67 :2 63 :2 35
:3 19 :2 1f 24 30 74 77 :2 30 84
87 :2 30 8b 8e :2 30 9a 9d :2 30
a5 a8 :2 30 b4 b7 be c7 ca
:2 b7 :2 30 :2 19 6a :2 16 :2 13 :4 1f :2 13
10 :3 46 14 :2 1d 28 26 :2 31 :2 26
13 1f 2a 37 3f 46 5f 6a
81 94 :2 16 :2 1f 2b 16 13 1b
19 :2 22 2e 19 29 :2 16 :2 1b 19
:2 22 2e 19 1c :2 26 38 :2 42 4e
52 66 69 6a :2 66 :2 38 :3 1c :2 22
27 33 77 7a :2 33 87 8a :2 33
8e 91 :2 33 9d a0 :2 33 a8 ab
:2 33 b7 ba c1 ca cd :2 ba :2 33
:2 1c 6d :2 19 :2 16 :4 22 :2 16 13 :4 41
:2 10 d 46 4a 13 :2 1e 26 35
44 :2 13 48 4a :2 48 13 :2 1d 2f
:2 39 45 49 5d 60 61 :2 5d :2 2f
:3 13 :2 19 1e 2a 6e 71 :2 2a 7e
81 :2 2a 85 88 :2 2a 92 95 9e
a6 :2 9e aa :2 95 :2 2a b2 b5 :2 2a
bd c0 :2 2a c5 c8 :2 2a cb ce
:2 2a :2 13 64 :2 10 13 :2 1d 2f :2 39
45 48 5c 5f 60 :2 5c :2 2f :3 13
:2 19 1e 2a 6e 71 :2 2a :2 13 63
:3 10 :2 1b 2f 32 36 :2 10 4c 4a
:2 d 2a d 1e d :5 a 18 :2 a
18 a d 16 29 :2 d 2f 2c
:2 38 :2 2c 17 10 22 10 22 10
16 2a 10 17 13 1f :3 1d 13
:2 1c 28 13 24 13 :2 1c 28 13
:4 10 d 15 13 :2 1c 28 13 23
:2 10 :2 15 13 :2 1c 28 13 16 :2 20
32 :2 3c 48 4c 60 63 64 :2 60
:2 32 :3 16 :2 1c 21 2d 71 74 :2 2d
81 84 :2 2d 88 8b :2 2d 97 9a
:2 2d a2 a5 :2 2d b1 b4 bb c4
c7 :2 b4 :2 2d :2 16 67 :2 13 :2 10 :4 1c
:2 10 d :4 52 :4 11 10 :4 14 13 1d
2d 1b 2b 39 1c 29 3c 50
69 7d 1a 25 2d 34 :2 16 :2 1f
2b 16 13 1b 19 :2 22 2e 19
29 :2 16 :2 1b 19 :2 22 2e 19 1c
:2 26 38 :2 42 4e 52 66 69 6a
:2 66 :2 38 :3 1c :2 22 27 33 77 7a
:2 33 87 8a :2 33 8e 91 :2 33 9d
a0 :2 33 a8 ab :2 33 b7 ba c1
ca cd :2 ba :2 33 :2 1c 6d :2 19 :2 16
:4 22 :2 16 13 :3 30 16 1f 2b :2 16
2e 30 :2 2e 19 :2 23 35 :2 3f 4b
4f 63 66 67 :2 63 :2 35 :3 19 :2 1f
24 30 74 77 :2 30 84 87 :2 30
8b 8e :2 30 98 9b a4 ac :2 b5
:2 a4 d4 :2 9b :2 30 dc df :2 30 e7
ea :2 30 ef f2 :2 30 106 109 :2 30
:2 19 6a :2 16 19 :2 23 35 :2 3f 4b
4e 62 65 66 :2 62 :2 35 :3 19 :2 1f
24 30 74 77 :2 30 :2 19 69 :3 16
:2 21 35 :2 3e 5c 1e :2 16 32 :2 13
10 30 :4 17 16 1d 2d 1b 2b
39 1c 29 3c 50 69 7d 1a
25 2d 34 :2 16 :2 1f 2b 16 13
1b 19 :2 22 2e 19 29 :2 16 :2 1b
19 :2 22 2e 19 1c :2 26 38 :2 42
4e 52 66 69 6a :2 66 :2 38 :3 1c
:2 22 27 33 77 7a :2 33 87 8a
:2 33 8e 91 :2 33 9d a0 :2 33 a8
ab :2 33 b7 ba c1 ca cd :2 ba
:2 33 :2 1c 6d :2 19 :2 16 :4 22 :2 16 13
:3 32 16 1f 2b :2 16 2e 30 :2 2e
19 :2 23 35 :2 3f 4b 4f 63 66
67 :2 63 :2 35 :3 19 :2 1f 24 30 74
77 :2 30 84 87 :2 30 8b 8e :2 30
98 9b a4 ac :2 b5 :2 a4 d4 :2 9b
:2 30 dc df :2 30 e7 ea :2 30 ef
f2 :2 30 106 109 :2 30 :2 19 6a :2 16
19 :2 23 35 :2 3f 4b 4e 62 65
66 :2 62 :2 35 :3 19 :2 1f 24 30 74
77 :2 30 :2 19 69 :3 16 :2 21 35 :2 3e
5c 1e :2 16 32 20 24 34 41
1f 2c 3d 49 61 6d 75 7c
:2 19 :2 22 2e 19 16 1e 1c :2 25
31 1c 2c :2 19 :2 1e 1c :2 25 31
1c 1f :2 29 3b :2 45 51 55 69
6c 6d :2 69 :2 3b :3 1f :2 25 2a 36
7a 7d :2 36 8a 8d :2 36 91 94
:2 36 a0 a3 :2 36 ab ae :2 36 ba
bd c4 cd d0 :2 bd :2 36 :2 1f 70
:2 1c :2 19 :4 25 :2 19 16 :3 13 19 22
2b :2 19 30 2e 39 40 :2 30 :2 2e
1c :2 26 38 :2 42 4e 52 66 69
6a :2 66 :2 38 :3 1c :2 22 27 33 77
7a :2 33 87 8a :2 33 8e 91 :2 33
9b 9e a7 af :2 b8 :2 a7 d7 :2 9e
:2 33 df e2 :2 33 ea ed :2 33 f2
f5 :2 33 109 10c :2 33 :2 1c 6d :2 19
1c :2 26 38 :2 42 4e 51 65 68
69 :2 65 :2 38 :3 1c :2 22 27 33 77
7a :2 33 :2 1c 6c :3 19 :2 24 38 :2 41
5f 21 :2 19 43 :2 16 :4 13 32 30
16 :2 20 32 :2 3c 48 4c 60 63
64 :2 60 :2 32 :3 16 :2 1c 21 2d 71
74 :2 2d 81 84 :2 2d 88 8b :2 2d
95 98 a1 a9 :2 b2 :2 a1 ca :2 98
:2 2d d2 d5 :2 2d dd e0 :2 2d e5
e8 :2 2d fc ff :2 2d :2 16 67 :2 13
16 :2 20 32 :2 3c 48 4b 5f 62
63 :2 5f :2 32 :3 16 :2 1c 21 2d 71
74 :2 2d :2 16 66 :3 13 :2 1e 32 :2 3b
52 1b :2 13 :4 10 32 :3 d 1a :2 d
24 d :4 11 10 :4 2d 2c 2b :3 10
20 10 4c :3 d 20 33 41 4c
29 3e 4c 29 3b 47 51 29
37 3e 49 4f 29 38 48 54
29 35 48 54 :2 20 :2 d 1b :2 d
20 30 3e 53 5f 62 :2 20 d
:4 11 :2 10 1e :2 10 23 33 41 56
62 65 :2 23 10 32 :2 d :4 a 26
:2 7 :4 4 8 :2 13 1a 24 :2 8 30
32 :2 30 7 3a a 1d :2 3a 23
20 :2 2c :2 20 39 :3 7 15 :2 7 17
:2 7 1c :2 25 :2 7 16 25 31 46
52 60 :2 16 :2 7 :2 10 1c :2 7 15
:2 7 17 :2 7 1c :2 25 :2 7 16 25
31 46 52 60 :2 16 :2 7 :2 10 1c
7 47 :3 4 17 21 :2 17 4 7
:2 11 23 :2 2d 39 3c 50 53 54
:2 50 :2 23 :3 7 :2 d 12 1e 62 65
:2 1e 72 75 :2 1e 79 7c :2 1e 86
89 92 9a :2 a3 :2 92 ae :2 89 :2 1e
b6 b9 :2 1e c1 c4 :2 1e c9 cc
:2 1e cf d2 :2 1e :2 7 57 :3 4 :2 f
21 :2 2a 34 38 :2 4 1 9 12
9 a :2 14 26 :2 30 3c 40 54
57 58 :2 54 :2 26 :3 a :2 10 15 21
65 68 :2 21 :3 a :2 10 15 39 5c
5f :2 68 :2 39 76 79 :2 39 81 84
:2 8d :2 39 :2 a 5b :3 7 c 15 1e
2c a 13 1e f e 7 e
7 e 7 29 :2 4 :2 9 7 :2 10
1c :2 7 :2 11 23 :2 2d 39 3d 51
54 55 :2 51 :2 23 :3 7 :2 d 12 1e
62 65 :2 1e :3 7 :2 d 12 36 42
45 :2 36 4d 50 :2 36 5c 5f 66
6f 72 :2 5f :2 36 :2 7 58 :2 4 7
c 1b a 13 1e f e 7
e 7 e 7 :2 4 :4 10 :2 4 :8 1

112f
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 :4 7 2
:4 8 :2 1 :7 a :7 b
:7 c :7 d :7 e :7 f
:7 10 :7 11 :5 12 :5 13
:5 14 :5 15 :5 16 :5 17
:5 18 :7 19 :6 1a :6 1b
:6 1c :6 1d :5 1e :5 1f
:5 20 :5 21 :5 22 :5 23
:7 24 :6 25 :6 26 :6 27
:6 28 :6 29 :6 2a :6 2b
:6 2c :6 2d :6 2e :6 2f
:6 30 :6 31 :6 32 :6 33
:7 34 :5 35 :6 36 :7 38
:b 3a :11 3b 3c :2 3b
:2 3c :2 3b :2 3c :2 3b
:2 3d :2 3b :12 3d :2 3b
:2 3e :2 3b :12 3e :2 3b
:2 3f :2 3b :9 3f :2 3b
:2 40 :2 3b :2 40 :2 3b
:2 40 :2 3b :2 40 :2 3b
:2 40 :4 3b :3 3a :11 43
:b 44 :3 43 :3 4a :3 4b
:3 4d :3 4f :3 50 :3 51
:3 52 :a 53 :5 54 :3 53
:6 58 :f 59 :8 5a :6 5b
:9 5c :8 5d 5e :2 5f
58 :5 60 57 62
:5 63 :3 62 :2 64 :5 66
:11 67 :24 68 :3 67 :2 65
:6 64 61 :3 39 :a 6e
:11 6f :30 70 :3 6f :11 72
:b 73 :3 72 :a 75 :3 6e
:9 78 :11 79 :30 7a :3 79
:11 7c :b 7d :3 7c :a 7f
:3 78 :e 82 :c 83 :11 84
:30 85 :3 84 :11 87 :b 88
:3 87 :a 8a :3 83 :3 82
:3 8e :4 90 :4 91 :7 92
90 :5 93 8f 95
:5 96 :3 95 :2 97 :5 99
:11 9a :24 9b :3 9a :2 98
:6 97 94 :3 39 :3 a1
:d a2 :2 a3 :2 a2 :6 a3
:3 a2 :3 a4 :3 a5 :5 a6
:a a7 :5 a8 :3 aa :3 ab
:5 ac :a ad :5 ae a3
:2 a2 :15 b2 :3 b3 :3 b2
:e b6 b8 :2 b9 :2 ba
bb :2 bc b8 :6 bd
:5 be bd :5 c0 bf
:3 bd b7 c3 :5 c4
:3 c3 :2 c5 :5 c7 :11 c8
:24 c9 :3 c8 :2 c6 :6 c5
c2 :4 b6 :c d0 d4
:2 d5 d6 :2 d7 d4
:6 d8 :5 d9 d8 :5 db
da :3 d8 d3 de
:5 df :3 de :2 e0 :5 e2
:11 e3 :24 e4 :3 e3 :2 e1
:6 e0 dd :3 d0 :c eb
:3 ed :3 eb :3 d0 :c f1
f4 :2 f5 f6 :2 f7
f4 :6 f8 :5 f9 f8
:5 fb fa :3 f8 f3
fe :5 ff :3 fe :2 100
:5 102 :11 103 :24 104 :3 103
:2 101 :6 100 fd :3 f1
:c 10c :3 10e :3 10c :3 f1
:9 112 :15 115 :3 116 :3 115
:12 119 11a :2 119 :4 11a
:3 119 :11 11b :30 11c :3 11b
:11 11e :b 11f :3 11e :a 121
11a :2 119 :5 124 :5 125
:11 126 :30 127 :3 126 :11 129
:b 12a :3 129 :a 12c :3 125
:3 130 :c 131 :3 132 133
131 :c 133 :9 135 136
:2 137 :4 138 135 :5 139
134 13b :5 13c :3 13b
:2 13d :5 13f :11 140 :24 141
:3 140 :2 13e :6 13d 13a
:3 133 :a 146 :a 149 :5 14a
148 14c :5 14d :3 14c
:2 14e :5 150 :11 151 :24 152
:3 151 :2 14f :6 14e 14b
:6 146 157 133 131
:5 157 :11 158 :2e 159 :3 158
:11 15b :b 15c :3 15b :8 15e
157 :3 131 124 :3 161
:3 162 160 :3 124 :5 165
:15 167 :11 168 :30 169 :3 168
:11 16b :b 16c :3 16b :a 16e
16f 167 :b 16f 170
:2 16f :4 170 :3 16f :11 171
:20 172 :3 171 :11 174 :b 175
:3 174 :9 177 170 :3 167
:15 17a :3 17b 17a :3 17d
17c :3 17a :c 181 :3 182
183 181 :b 183 :9 186
187 :2 188 :4 189 186
:5 18a 185 18c :5 18d
:3 18c :2 18e :5 190 :11 191
:24 192 :3 191 :2 18f :6 18e
18b :3 183 :a 198 :a 19b
:5 19c 19a 19e :5 19f
:3 19e :2 1a0 :5 1a2 :11 1a3
:24 1a4 :3 1a3 :2 1a1 :6 1a0
19d :6 198 1a9 183
181 :c 1a9 :11 1ab :2e 1ac
:3 1ab :11 1ae :b 1af :3 1ae
:8 1b1 1a9 :3 181 165
:3 1b4 1b3 :3 165 :3 1b7
:3 1b8 :b 1bb 1bd :2 1be
:2 1bf 1c0 :2 1c1 1bd
:6 1c2 :5 1c3 1c2 :5 1c5
1c4 :3 1c2 1bc 1c8
:5 1c9 :3 1c8 :2 1ca :5 1cc
:11 1cd :24 1ce :3 1cd :2 1cb
:6 1ca 1c7 :4 1bb :5 1d5
:5 1d7 :2 1d9 1da :2 1db
:6 1dc :4 1dd 1d9 :5 1de
1d8 1e0 :5 1e1 :3 1e0
:2 1e2 :5 1e4 :11 1e5 :24 1e6
:3 1e5 :2 1e3 :6 1e2 1df
:3 1d7 :9 1ea :11 1ed :30 1ee
:3 1ed :11 1f0 :b 1f1 :3 1f0
:7 1f3 1f4 :2 1f3 :3 1ea
1f6 1d7 :5 1f6 :2 1f9
1fa :2 1fb :6 1fc :4 1fd
1f9 :5 1fe 1f8 200
:5 201 :3 200 :2 202 :5 204
:11 205 :24 206 :3 205 :2 203
:6 202 1ff :3 1f6 :9 20b
:11 20d :30 20e :3 20d :11 210
:b 211 :3 210 :7 213 214
:2 213 20b :4 21b :8 21c
21b :5 21d 21a 21f
:5 220 :3 21f :2 221 :5 223
:11 224 :24 225 :3 224 :2 222
:6 221 21e :3 215 :d 229
:11 22a :30 22b :3 22a :11 22d
:b 22e :3 22d :7 230 231
:2 230 :3 229 215 :3 20b
1f6 1d7 :11 235 :30 236
:3 235 :11 238 :b 239 :3 238
:7 23b 23c :2 23b 234
:3 1d7 :3 1d5 :3 241 :3 242
:d 244 :3 245 :3 244 :5 248
:3 249 :4 24a :5 24b :4 24c
:4 24d :3 248 :3 24f :a 250
:5 252 :3 253 :a 254 :3 252
1d2 :3 1bb :3 112 ce
:3 b6 :d 25a :2 25b :2 25a
:6 25b :3 25a :3 25c :3 25d
:5 25e :a 25f :5 260 :3 262
:3 263 :5 264 :a 265 :5 266
25b :2 25a :6 269 :11 26b
:30 26c :3 26b :a 26e 39
:3 272 :11 273 :b 274 :17 275
:3 273 :2 277 :3 278 :3 279
27a 278 277 278
:3 27b :3 272 :2 27c :5 27e
:11 27f :b 280 :18 281 :3 27f
:2 283 284 :3 285 286
284 283 284 :3 287
:2 27d :6 27c 271 :7 1

3c00
4
:3 0 1 :3 0 2
:a 0 112a 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 6
:3 0 5 :7 0 9
8 :3 0 6 :3 0
7 :7 0 d c
:3 0 d :2 0 b
9 :3 0 8 :7 0
11 10 :3 0 4
:3 0 a :7 0 15
14 :3 0 b :3 0
c :3 0 d :2 0
4 19 1a 0
17 1b 0 112a
2 1c :2 0 f
:2 0 15 4 :3 0
f :2 0 13 1f
21 :6 0 24 22
0 1128 0 e
:6 0 f :2 0 19
4 :3 0 17 26
28 :6 0 2b 29
0 1128 0 10
:6 0 f :2 0 1d
4 :3 0 1b 2d
2f :6 0 32 30
0 1128 0 11
:6 0 f :2 0 21
4 :3 0 1f 34
36 :6 0 39 37
0 1128 0 12
:6 0 f :2 0 25
4 :3 0 23 3b
3d :6 0 40 3e
0 1128 0 13
:6 0 f :2 0 29
4 :3 0 27 42
44 :6 0 47 45
0 1128 0 14
:6 0 f :2 0 2d
4 :3 0 2b 49
4b :6 0 4e 4c
0 1128 0 15
:6 0 33 17e 0
31 4 :3 0 2f
50 52 :6 0 55
53 0 1128 0
16 :6 0 37 1b2
0 35 6 :3 0
57 :7 0 5a 58
0 1128 0 17
:6 0 6 :3 0 5c
:7 0 5f 5d 0
1128 0 18 :6 0
3b 1e6 0 39
6 :3 0 61 :7 0
64 62 0 1128
0 19 :6 0 6
:3 0 66 :7 0 69
67 0 1128 0
1a :6 0 3f 21a
0 3d 6 :3 0
6b :7 0 6e 6c
0 1128 0 1b
:6 0 6 :3 0 70
:7 0 73 71 0
1128 0 1c :6 0
21 :2 0 43 6
:3 0 75 :7 0 78
76 0 1128 0
1d :6 0 4 :3 0
1f :2 0 41 7a
7c :6 0 7f 7d
0 1128 0 1e
:6 0 21 :2 0 45
9 :3 0 81 :7 0
85 82 83 1128
0 20 :6 0 25
:2 0 47 9 :3 0
87 :7 0 8b 88
89 1128 0 22
:6 0 25 :2 0 49
24 :3 0 8d :7 0
91 8e 8f 1128
0 23 :6 0 4d
2c2 0 4b 24
:3 0 93 :7 0 97
94 95 1128 0
26 :6 0 51 2f6
0 4f 6 :3 0
99 :7 0 9c 9a
0 1128 0 27
:6 0 6 :3 0 9e
:7 0 a1 9f 0
1128 0 28 :6 0
55 32a 0 53
2a :3 0 a3 :7 0
a6 a4 0 1128
0 29 :6 0 2a
:3 0 a8 :7 0 ab
a9 0 1128 0
2b :6 0 2f :2 0
57 2a :3 0 ad
:7 0 b0 ae 0
1128 0 2c :6 0
2a :3 0 b2 :7 0
b5 b3 0 1128
0 2d :6 0 21
:2 0 5b 4 :3 0
59 b7 b9 :6 0
bc ba 0 1128
0 2e :6 0 21
:2 0 5d 9 :3 0
be :7 0 c2 bf
c0 1128 0 30
:6 0 21 :2 0 5f
9 :3 0 c4 :7 0
c8 c5 c6 1128
0 31 :6 0 21
:2 0 61 9 :3 0
ca :7 0 ce cb
cc 1128 0 32
:6 0 21 :2 0 63
9 :3 0 d0 :7 0
d4 d1 d2 1128
0 33 :6 0 21
:2 0 65 9 :3 0
d6 :7 0 da d7
d8 1128 0 34
:6 0 21 :2 0 67
9 :3 0 dc :7 0
e0 dd de 1128
0 35 :6 0 21
:2 0 69 9 :3 0
e2 :7 0 e6 e3
e4 1128 0 36
:6 0 21 :2 0 6b
9 :3 0 e8 :7 0
ec e9 ea 1128
0 37 :6 0 21
:2 0 6d 9 :3 0
ee :7 0 f2 ef
f0 1128 0 38
:6 0 21 :2 0 6f
9 :3 0 f4 :7 0
f8 f5 f6 1128
0 39 :6 0 21
:2 0 71 9 :3 0
fa :7 0 fe fb
fc 1128 0 3a
:6 0 21 :2 0 73
9 :3 0 100 :7 0
104 101 102 1128
0 3b :6 0 21
:2 0 75 9 :3 0
106 :7 0 10a 107
108 1128 0 3c
:6 0 21 :2 0 77
9 :3 0 10c :7 0
110 10d 10e 1128
0 3d :6 0 f
:2 0 79 9 :3 0
112 :7 0 116 113
114 1128 0 3e
:6 0 7f 53c 0
7d 4 :3 0 7b
118 11a :6 0 11d
11b 0 1128 0
3f :6 0 12a 12b
0 81 41 :3 0
11f :7 0 122 120
0 1128 0 40
:6 0 9 :3 0 124
0 2 :5 0 21
:2 0 128 125 126
1128 0 42 :6 0
130 131 0 83
c :3 0 d :2 0
4 12c :7 0 12f
12d 0 1128 0
43 :6 0 44 :3 0
45 :3 0 44 :3 0
46 :3 0 133 134
0 2 :4 0 85
135 137 132 139
138 :2 0 47 :3 0
48 :3 0 13b 13c
0 49 :4 0 44
:3 0 4a :3 0 13f
140 0 2 :4 0
87 141 143 4b
:4 0 4c :2 0 4d
:3 0 89 146 148
:3 0 4e :4 0 4c
:2 0 4f :4 0 8c
14b 14d :3 0 4c
:2 0 3 :3 0 8f
14f 151 :3 0 4c
:2 0 4f :4 0 92
153 155 :3 0 4c
:2 0 50 :4 0 95
157 159 :3 0 4c
:2 0 51 :3 0 52
:3 0 4f :4 0 4c
:2 0 5 :3 0 98
15f 161 :3 0 4c
:2 0 4f :4 0 9b
163 165 :3 0 53
:4 0 9e 15d 168
54 :4 0 a1 15c
16b a4 15b 16d
:3 0 4c :2 0 50
:4 0 a7 16f 171
:3 0 4c :2 0 51
:3 0 52 :3 0 4f
:4 0 4c :2 0 7
:3 0 aa 177 179
:3 0 4c :2 0 4f
:4 0 ad 17b 17d
:3 0 53 :4 0 b0
175 180 54 :4 0
b3 174 183 b6
173 185 :3 0 4c
:2 0 50 :4 0 b9
187 189 :3 0 4c
:2 0 51 :3 0 55
:3 0 8 :3 0 bc
18d 18f 54 :4 0
be 18c 192 c1
18b 194 :3 0 4c
:2 0 50 :4 0 c4
196 198 :3 0 4c
:2 0 4f :4 0 c7
19a 19c :3 0 4c
:2 0 a :3 0 ca
19e 1a0 :3 0 4c
:2 0 4f :4 0 cd
1a2 1a4 :3 0 4c
:2 0 56 :4 0 d0
1a6 1a8 :3 0 d3
13d 1aa :2 0 1ac
d8 1ad 13a 1ac
0 1ae da 0
1088 44 :3 0 45
:3 0 1af 1b0 0
44 :3 0 57 :3 0
1b2 1b3 0 2f
:2 0 2 :4 0 58
:2 0 59 :2 0 2f
:2 0 dc 1b8 1ba
:3 0 df 1b4 1bc
1b1 1be 1bd :2 0
47 :3 0 5a :3 0
1c0 1c1 0 49
:4 0 5b :4 0 4c
:2 0 4d :3 0 e3
1c5 1c7 :3 0 e6
1c2 1c9 :2 0 1cb
e9 1cc 1bf 1cb
0 1cd eb 0
1088 17 :3 0 5c
:4 0 1ce 1cf 0
1088 18 :3 0 5c
:4 0 1d1 1d2 0
1088 35 :3 0 2f
:2 0 1d4 1d5 0
1088 17 :3 0 5
:3 0 1d7 1d8 0
1088 18 :3 0 7
:3 0 1da 1db 0
1088 12 :3 0 3
:3 0 1dd 1de 0
1088 39 :3 0 8
:3 0 1e0 1e1 0
1088 51 :3 0 39
:3 0 21 :2 0 ed
1e3 1e6 5d :2 0
2f :2 0 f2 1e8
1ea :3 0 1eb :2 0
39 :3 0 5e :3 0
5f :3 0 1ee 1ef
0 1ed 1f0 0
1f2 f5 1f3 1ec
1f2 0 1f4 f7
0 1088 60 :3 0
61 :3 0 62 :3 0
63 :3 0 64 :3 0
65 :3 0 51 :3 0
66 :3 0 67 :3 0
51 :3 0 68 :3 0
69 :3 0 66 :3 0
68 :3 0 6a :3 0
6b :3 0 6c :3 0
6d :3 0 6c :3 0
6e :3 0 6f :3 0
70 :3 0 71 :3 0
72 :3 0 73 :3 0
74 :3 0 75 :3 0
76 :3 0 77 :3 0
e :3 0 13 :3 0
11 :3 0 1d :3 0
20 :3 0 2e :3 0
19 :3 0 1a :3 0
1b :3 0 1c :3 0
10 :3 0 14 :3 0
34 :3 0 3a :3 0
32 :3 0 37 :3 0
29 :3 0 27 :3 0
28 :3 0 30 :3 0
31 :3 0 23 :3 0
26 :3 0 78 :3 0
79 :3 0 3 :4 0
7a 1 :8 0 232
c :3 0 7b :3 0
22d 22e 0 7c
:3 0 22f 230 0
232 f9 285 7d
:3 0 c :3 0 7b
:3 0 234 235 0
7c :3 0 236 237
0 239 fc 23b
fe 23a 239 :2 0
283 7e :3 0 c
:3 0 7b :3 0 23e
23f 0 7c :3 0
240 241 0 27b
44 :3 0 45 :3 0
243 244 0 44
:3 0 57 :3 0 246
247 0 7f :2 0
2 :4 0 80 :2 0
59 :2 0 2f :2 0
100 24c 24e :3 0
103 248 250 245
252 251 :2 0 47
:3 0 5a :3 0 254
255 0 49 :4 0
81 :4 0 4c :2 0
4d :3 0 107 259
25b :3 0 4c :2 0
82 :4 0 10a 25d
25f :3 0 4c :2 0
83 :4 0 10d 261
263 :3 0 4c :2 0
7c :3 0 110 265
267 :3 0 4c :2 0
84 :4 0 113 269
26b :3 0 4c :2 0
85 :3 0 86 :3 0
2f :2 0 87 :2 0
116 26e 272 11a
26d 274 :3 0 11d
256 276 :2 0 278
120 279 253 278
0 27a 122 0
27b 124 27e :3 0
27e 0 27e 27d
27b 27c :6 0 280
2 :3 0 127 282
129 281 280 :2 0
283 12b :2 0 285
0 285 284 232
283 :6 0 1088 1
:3 0 c :3 0 7b
:3 0 287 288 0
c :3 0 5d :2 0
88 :3 0 28a 28c
0 130 28b 28e
:3 0 28f :2 0 44
:3 0 45 :3 0 291
292 0 44 :3 0
57 :3 0 294 295
0 f :2 0 2
:4 0 89 :2 0 59
:2 0 2f :2 0 133
29a 29c :3 0 136
296 29e 293 2a0
29f :2 0 47 :3 0
5a :3 0 2a2 2a3
0 49 :4 0 8a
:4 0 4c :2 0 4d
:3 0 13a 2a7 2a9
:3 0 4c :2 0 82
:4 0 13d 2ab 2ad
:3 0 4c :2 0 8b
:4 0 140 2af 2b1
:3 0 4c :2 0 51
:3 0 55 :3 0 5e
:3 0 8c :3 0 2b6
2b7 0 143 2b5
2b9 54 :4 0 145
2b4 2bc 148 2b3
2be :3 0 4c :2 0
8d :4 0 14b 2c0
2c2 :3 0 4c :2 0
4f :4 0 14e 2c4
2c6 :3 0 4c :2 0
3 :3 0 151 2c8
2ca :3 0 4c :2 0
4f :4 0 154 2cc
2ce :3 0 157 2a4
2d0 :2 0 2d2 15a
2d3 2a1 2d2 0
2d4 15c 0 2fe
44 :3 0 45 :3 0
2d5 2d6 0 44
:3 0 57 :3 0 2d8
2d9 0 2f :2 0
2 :4 0 8e :2 0
59 :2 0 2f :2 0
15e 2de 2e0 :3 0
161 2da 2e2 2d7
2e4 2e3 :2 0 47
:3 0 5a :3 0 2e6
2e7 0 49 :4 0
8f :4 0 4c :2 0
4d :3 0 165 2eb
2ed :3 0 168 2e8
2ef :2 0 2f1 16b
2f2 2e5 2f1 0
2f3 16d 0 2fe
90 :3 0 91 :3 0
2f4 2f5 0 5e
:3 0 8c :3 0 2f7
2f8 0 3 :3 0
92 :3 0 16f 2f6
2fc :2 0 2fe 173
2ff 290 2fe 0
300 177 0 1088
51 :3 0 20 :3 0
21 :2 0 179 301
304 5d :2 0 93
:2 0 17e 306 308
:3 0 44 :3 0 45
:3 0 30a 30b 0
44 :3 0 57 :3 0
30d 30e 0 f
:2 0 2 :4 0 94
:2 0 59 :2 0 2f
:2 0 181 313 315
:3 0 184 30f 317
30c 319 318 :2 0
47 :3 0 5a :3 0
31b 31c 0 49
:4 0 95 :4 0 4c
:2 0 4d :3 0 188
320 322 :3 0 4c
:2 0 82 :4 0 18b
324 326 :3 0 4c
:2 0 8b :4 0 18e
328 32a :3 0 4c
:2 0 51 :3 0 55
:3 0 5e :3 0 96
:3 0 32f 330 0
191 32e 332 54
:4 0 193 32d 335
196 32c 337 :3 0
4c :2 0 8d :4 0
199 339 33b :3 0
4c :2 0 4f :4 0
19c 33d 33f :3 0
4c :2 0 3 :3 0
19f 341 343 :3 0
4c :2 0 4f :4 0
1a2 345 347 :3 0
1a5 31d 349 :2 0
34b 1a8 34c 31a
34b 0 34d 1aa
0 377 44 :3 0
45 :3 0 34e 34f
0 44 :3 0 57
:3 0 351 352 0
2f :2 0 2 :4 0
97 :2 0 59 :2 0
2f :2 0 1ac 357
359 :3 0 1af 353
35b 350 35d 35c
:2 0 47 :3 0 5a
:3 0 35f 360 0
49 :4 0 98 :4 0
4c :2 0 4d :3 0
1b3 364 366 :3 0
1b6 361 368 :2 0
36a 1b9 36b 35e
36a 0 36c 1bb
0 377 90 :3 0
91 :3 0 36d 36e
0 5e :3 0 96
:3 0 370 371 0
3 :3 0 92 :3 0
1bd 36f 375 :2 0
377 1c1 378 309
377 0 379 1c5
0 1088 5 :3 0
99 :2 0 1c7 37b
37c :3 0 37d :2 0
37e :2 0 7 :3 0
99 :2 0 1c9 381
382 :3 0 383 :2 0
384 :2 0 37f 386
385 :2 0 9a :3 0
9b :3 0 388 389
0 18 :3 0 17
:3 0 9c :2 0 1cb
38a 38e 9d :2 0
21 :2 0 1d1 390
392 :3 0 44 :3 0
45 :3 0 394 395
0 44 :3 0 57
:3 0 397 398 0
f :2 0 2 :4 0
9e :2 0 59 :2 0
2f :2 0 1d4 39d
39f :3 0 1d7 399
3a1 396 3a3 3a2
:2 0 47 :3 0 5a
:3 0 3a5 3a6 0
49 :4 0 9f :4 0
4c :2 0 4d :3 0
1db 3aa 3ac :3 0
4c :2 0 82 :4 0
1de 3ae 3b0 :3 0
4c :2 0 8b :4 0
1e1 3b2 3b4 :3 0
4c :2 0 51 :3 0
55 :3 0 5e :3 0
a0 :3 0 3b9 3ba
0 1e4 3b8 3bc
54 :4 0 1e6 3b7
3bf 1e9 3b6 3c1
:3 0 4c :2 0 8d
:4 0 1ec 3c3 3c5
:3 0 4c :2 0 4f
:4 0 1ef 3c7 3c9
:3 0 4c :2 0 3
:3 0 1f2 3cb 3cd
:3 0 4c :2 0 4f
:4 0 1f5 3cf 3d1
:3 0 1f8 3a7 3d3
:2 0 3d5 1fb 3d6
3a4 3d5 0 3d7
1fd 0 401 44
:3 0 45 :3 0 3d8
3d9 0 44 :3 0
57 :3 0 3db 3dc
0 2f :2 0 2
:4 0 a1 :2 0 59
:2 0 2f :2 0 1ff
3e1 3e3 :3 0 202
3dd 3e5 3da 3e7
3e6 :2 0 47 :3 0
5a :3 0 3e9 3ea
0 49 :4 0 a2
:4 0 4c :2 0 4d
:3 0 206 3ee 3f0
:3 0 209 3eb 3f2
:2 0 3f4 20c 3f5
3e8 3f4 0 3f6
20e 0 401 90
:3 0 91 :3 0 3f7
3f8 0 5e :3 0
a0 :3 0 3fa 3fb
0 3 :3 0 92
:3 0 210 3f9 3ff
:2 0 401 214 402
393 401 0 403
218 0 404 21a
405 387 404 0
406 21c 0 1088
2d :4 0 407 408
0 1088 a3 :3 0
a4 :3 0 2d :3 0
a5 :3 0 a6 :3 0
e :3 0 6b :3 0
14 :3 0 a7 :3 0
5e :3 0 a8 :3 0
5e :3 0 a9 :3 0
5e :3 0 aa :4 0
ab 1 :8 0 41f
c :3 0 7b :3 0
41a 41b 0 7c
:3 0 41c 41d 0
41f 21e 472 7d
:3 0 c :3 0 7b
:3 0 421 422 0
7c :3 0 423 424
0 426 221 428
223 427 426 :2 0
470 7e :3 0 c
:3 0 7b :3 0 42b
42c 0 7c :3 0
42d 42e 0 468
44 :3 0 45 :3 0
430 431 0 44
:3 0 57 :3 0 433
434 0 7f :2 0
2 :4 0 ac :2 0
59 :2 0 2f :2 0
225 439 43b :3 0
228 435 43d 432
43f 43e :2 0 47
:3 0 5a :3 0 441
442 0 49 :4 0
ad :4 0 4c :2 0
4d :3 0 22c 446
448 :3 0 4c :2 0
82 :4 0 22f 44a
44c :3 0 4c :2 0
83 :4 0 232 44e
450 :3 0 4c :2 0
7c :3 0 235 452
454 :3 0 4c :2 0
84 :4 0 238 456
458 :3 0 4c :2 0
85 :3 0 86 :3 0
2f :2 0 87 :2 0
23b 45b 45f 23f
45a 461 :3 0 242
443 463 :2 0 465
245 466 440 465
0 467 247 0
468 249 46b :3 0
46b 0 46b 46a
468 469 :6 0 46d
4 :3 0 24c 46f
24e 46e 46d :2 0
470 250 :2 0 472
0 472 471 41f
470 :6 0 1088 1
:3 0 15 :3 0 a
:3 0 474 475 0
1088 9a :3 0 ae
:3 0 477 478 0
14 :3 0 15 :3 0
253 479 47c 5d
:2 0 21 :2 0 258
47e 480 :3 0 481
:2 0 51 :3 0 39
:3 0 21 :2 0 25b
483 486 5e :3 0
af :2 0 5f :3 0
488 48a 0 260
489 48c :3 0 48d
:2 0 482 48f 48e
:2 0 3f :3 0 b0
:4 0 491 492 0
4c5 3e :3 0 2f
:2 0 494 495 0
4c5 42 :3 0 c
:3 0 7b :3 0 498
499 0 497 49a
0 4c5 40 :3 0
b1 :3 0 3f :3 0
12 :3 0 15 :3 0
3e :3 0 21 :2 0
263 49d 4a3 49c
4a4 0 4c5 c
:3 0 7b :3 0 4a6
4a7 0 42 :3 0
4a8 4a9 0 4c5
3f :3 0 b2 :4 0
4ab 4ac 0 4c5
3e :3 0 21 :2 0
4ae 4af 0 4c5
42 :3 0 c :3 0
7b :3 0 4b2 4b3
0 4b1 4b4 0
4c5 40 :3 0 b1
:3 0 3f :3 0 12
:3 0 15 :3 0 3e
:3 0 21 :2 0 269
4b7 4bd 4b6 4be
0 4c5 c :3 0
7b :3 0 4c0 4c1
0 42 :3 0 4c2
4c3 0 4c5 26f
4c6 490 4c5 0
4c7 27a 0 1088
17 :3 0 99 :2 0
27c 4c9 4ca :3 0
4cb :2 0 4cc :2 0
9a :3 0 9b :3 0
4ce 4cf 0 17
:3 0 19 :3 0 9c
:2 0 27e 4d0 4d4
5d :2 0 21 :2 0
284 4d6 4d8 :3 0
4d9 :2 0 4cd 4db
4da :2 0 17 :3 0
19 :3 0 4dd 4de
0 4e0 287 4e1
4dc 4e0 0 4e2
289 0 1088 5
:3 0 b3 :2 0 28b
4e4 4e5 :3 0 4e6
:2 0 4e7 :2 0 7
:3 0 b3 :2 0 28d
4ea 4eb :3 0 4ec
:2 0 4ed :2 0 4e8
4ef 4ee :2 0 78
:3 0 66 :3 0 5
:3 0 68 :3 0 7
:3 0 b4 :3 0 79
:3 0 3 :4 0 b5
1 :8 0 50f b6
:4 0 4fa :3 0 92
:3 0 5d :2 0 291
4fd 4fe :3 0 c
:3 0 7b :3 0 500
501 0 21 :2 0
502 503 0 505
294 50c c :3 0
7b :3 0 506 507
0 b7 :2 0 508
509 0 50b 296
50d 4ff 505 0
50e 0 50b 0
50e 298 0 50f
29b 562 7d :3 0
c :3 0 7b :3 0
511 512 0 7c
:3 0 513 514 0
516 29e 518 2a0
517 516 :2 0 560
7e :3 0 c :3 0
7b :3 0 51b 51c
0 7c :3 0 51d
51e 0 558 44
:3 0 45 :3 0 520
521 0 44 :3 0
57 :3 0 523 524
0 7f :2 0 2
:4 0 1f :2 0 59
:2 0 2f :2 0 2a2
529 52b :3 0 2a5
525 52d 522 52f
52e :2 0 47 :3 0
5a :3 0 531 532
0 49 :4 0 b8
:4 0 4c :2 0 4d
:3 0 2a9 536 538
:3 0 4c :2 0 82
:4 0 2ac 53a 53c
:3 0 4c :2 0 83
:4 0 2af 53e 540
:3 0 4c :2 0 7c
:3 0 2b2 542 544
:3 0 4c :2 0 84
:4 0 2b5 546 548
:3 0 4c :2 0 85
:3 0 86 :3 0 2f
:2 0 87 :2 0 2b8
54b 54f 2bc 54a
551 :3 0 2bf 533
553 :2 0 555 2c2
556 530 555 0
557 2c4 0 558
2c6 55b :3 0 55b
0 55b 55a 558
559 :6 0 55d 6
:3 0 2c9 55f 2cb
55e 55d :2 0 560
2cd :2 0 562 0
562 561 50f 560
:6 0 564 1 :3 0
2d0 fe0 5 :3 0
b3 :2 0 2d2 566
567 :3 0 568 :2 0
1b :3 0 99 :2 0
2d4 56b 56c :3 0
56d :2 0 569 56f
56e :2 0 78 :3 0
66 :3 0 5 :3 0
b4 :3 0 79 :3 0
3 :4 0 b9 1
:8 0 58d b6 :4 0
578 :3 0 92 :3 0
5d :2 0 2d8 57b
57c :3 0 c :3 0
7b :3 0 57e 57f
0 21 :2 0 580
581 0 583 2db
58a c :3 0 7b
:3 0 584 585 0
b7 :2 0 586 587
0 589 2dd 58b
57d 583 0 58c
0 589 0 58c
2df 0 58d 2e2
5e0 7d :3 0 c
:3 0 7b :3 0 58f
590 0 7c :3 0
591 592 0 594
2e5 596 2e7 595
594 :2 0 5de 7e
:3 0 c :3 0 7b
:3 0 599 59a 0
7c :3 0 59b 59c
0 5d6 44 :3 0
45 :3 0 59e 59f
0 44 :3 0 57
:3 0 5a1 5a2 0
7f :2 0 2 :4 0
ba :2 0 59 :2 0
2f :2 0 2e9 5a7
5a9 :3 0 2ec 5a3
5ab 5a0 5ad 5ac
:2 0 47 :3 0 5a
:3 0 5af 5b0 0
49 :4 0 bb :4 0
4c :2 0 4d :3 0
2f0 5b4 5b6 :3 0
4c :2 0 82 :4 0
2f3 5b8 5ba :3 0
4c :2 0 83 :4 0
2f6 5bc 5be :3 0
4c :2 0 7c :3 0
2f9 5c0 5c2 :3 0
4c :2 0 84 :4 0
2fc 5c4 5c6 :3 0
4c :2 0 85 :3 0
86 :3 0 2f :2 0
87 :2 0 2ff 5c9
5cd 303 5c8 5cf
:3 0 306 5b1 5d1
:2 0 5d3 309 5d4
5ae 5d3 0 5d5
30b 0 5d6 30d
5d9 :3 0 5d9 0
5d9 5d8 5d6 5d7
:6 0 5db 8 :3 0
310 5dd 312 5dc
5db :2 0 5de 314
:2 0 5e0 0 5e0
5df 58d 5de :6 0
5f4 1 :3 0 9a
:3 0 9b :3 0 5e2
5e3 0 18 :3 0
1c :3 0 9c :2 0
317 5e4 5e8 5d
:2 0 21 :2 0 31d
5ea 5ec :3 0 35
:3 0 21 :2 0 5ee
5ef 0 5f1 320
5f2 5ed 5f1 0
5f3 322 0 5f4
324 5f5 570 5f4
0 5f6 327 0
fdf 7 :3 0 b3
:2 0 329 5f8 5f9
:3 0 5fa :2 0 1c
:3 0 99 :2 0 32b
5fd 5fe :3 0 5ff
:2 0 5fb 601 600
:2 0 78 :3 0 68
:3 0 7 :3 0 b4
:3 0 79 :3 0 3
:4 0 bc 1 :8 0
61f b6 :4 0 60a
:3 0 92 :3 0 5d
:2 0 32f 60d 60e
:3 0 c :3 0 7b
:3 0 610 611 0
21 :2 0 612 613
0 615 332 61c
c :3 0 7b :3 0
616 617 0 b7
:2 0 618 619 0
61b 334 61d 60f
615 0 61e 0
61b 0 61e 336
0 61f 339 672
7d :3 0 c :3 0
7b :3 0 621 622
0 7c :3 0 623
624 0 626 33c
628 33e 627 626
:2 0 670 7e :3 0
c :3 0 7b :3 0
62b 62c 0 7c
:3 0 62d 62e 0
668 44 :3 0 45
:3 0 630 631 0
44 :3 0 57 :3 0
633 634 0 7f
:2 0 2 :4 0 bd
:2 0 59 :2 0 2f
:2 0 340 639 63b
:3 0 343 635 63d
632 63f 63e :2 0
47 :3 0 5a :3 0
641 642 0 49
:4 0 be :4 0 4c
:2 0 4d :3 0 347
646 648 :3 0 4c
:2 0 82 :4 0 34a
64a 64c :3 0 4c
:2 0 83 :4 0 34d
64e 650 :3 0 4c
:2 0 7c :3 0 350
652 654 :3 0 4c
:2 0 84 :4 0 353
656 658 :3 0 4c
:2 0 85 :3 0 86
:3 0 2f :2 0 87
:2 0 356 65b 65f
35a 65a 661 :3 0
35d 643 663 :2 0
665 360 666 640
665 0 667 362
0 668 364 66b
:3 0 66b 0 66b
66a 668 669 :6 0
66d a :3 0 367
66f 369 66e 66d
:2 0 670 36b :2 0
672 0 672 671
61f 670 :6 0 686
1 :3 0 9a :3 0
9b :3 0 674 675
0 17 :3 0 1b
:3 0 9c :2 0 36e
676 67a 5d :2 0
21 :2 0 374 67c
67e :3 0 35 :3 0
21 :2 0 680 681
0 683 377 684
67f 683 0 685
379 0 686 37b
687 602 686 0
688 37e 0 fdf
51 :3 0 35 :3 0
21 :2 0 380 689
68c 5d :2 0 2f
:2 0 385 68e 690
:3 0 18 :3 0 99
:2 0 388 693 694
:3 0 695 :2 0 696
:2 0 9a :3 0 9b
:3 0 698 699 0
18 :3 0 1a :3 0
9c :2 0 38a 69a
69e 5d :2 0 21
:2 0 390 6a0 6a2
:3 0 6a3 :2 0 697
6a5 6a4 :2 0 18
:3 0 1a :3 0 6a7
6a8 0 6aa 393
6ab 6a6 6aa 0
6ac 395 0 fdc
2d :3 0 99 :2 0
397 6ae 6af :3 0
6b0 :2 0 6b1 :2 0
18 :3 0 99 :2 0
399 6b4 6b5 :3 0
6b6 :2 0 6b7 :2 0
6b2 6b9 6b8 :2 0
9a :3 0 ae :3 0
6bb 6bc 0 2d
:3 0 18 :3 0 39b
6bd 6c0 bf :2 0
21 :2 0 3a0 6c2
6c4 :3 0 6c5 :2 0
6ba 6c7 6c6 :2 0
44 :3 0 45 :3 0
6c9 6ca 0 44
:3 0 57 :3 0 6cc
6cd 0 f :2 0
2 :4 0 c0 :2 0
59 :2 0 2f :2 0
3a3 6d2 6d4 :3 0
3a6 6ce 6d6 6cb
6d8 6d7 :2 0 47
:3 0 5a :3 0 6da
6db 0 49 :4 0
c1 :4 0 4c :2 0
4d :3 0 3aa 6df
6e1 :3 0 4c :2 0
82 :4 0 3ad 6e3
6e5 :3 0 4c :2 0
8b :4 0 3b0 6e7
6e9 :3 0 4c :2 0
51 :3 0 55 :3 0
5e :3 0 c2 :3 0
6ee 6ef 0 3b3
6ed 6f1 54 :4 0
3b5 6ec 6f4 3b8
6eb 6f6 :3 0 4c
:2 0 8d :4 0 3bb
6f8 6fa :3 0 4c
:2 0 4f :4 0 3be
6fc 6fe :3 0 4c
:2 0 14 :3 0 3c1
700 702 :3 0 4c
:2 0 4f :4 0 3c4
704 706 :3 0 3c7
6dc 708 :2 0 70a
3ca 70b 6d9 70a
0 70c 3cc 0
736 44 :3 0 45
:3 0 70d 70e 0
44 :3 0 57 :3 0
710 711 0 2f
:2 0 2 :4 0 c3
:2 0 59 :2 0 2f
:2 0 3ce 716 718
:3 0 3d1 712 71a
70f 71c 71b :2 0
47 :3 0 5a :3 0
71e 71f 0 49
:4 0 c4 :4 0 4c
:2 0 4d :3 0 3d5
723 725 :3 0 3d8
720 727 :2 0 729
3db 72a 71d 729
0 72b 3dd 0
736 90 :3 0 91
:3 0 72c 72d 0
5e :3 0 c2 :3 0
72f 730 0 14
:3 0 92 :3 0 3df
72e 734 :2 0 736
3e3 737 6c8 736
0 738 3e7 0
fdc 5 :3 0 99
:2 0 3e9 73a 73b
:3 0 73c :2 0 1d
:3 0 99 :2 0 3eb
73f 740 :3 0 741
:2 0 44 :3 0 45
:3 0 743 744 0
44 :3 0 57 :3 0
746 747 0 f
:2 0 2 :4 0 c5
:2 0 59 :2 0 2f
:2 0 3ed 74c 74e
:3 0 3f0 748 750
745 752 751 :2 0
47 :3 0 5a :3 0
754 755 0 49
:4 0 c6 :4 0 4c
:2 0 4d :3 0 3f4
759 75b :3 0 4c
:2 0 82 :4 0 3f7
75d 75f :3 0 4c
:2 0 8b :4 0 3fa
761 763 :3 0 4c
:2 0 51 :3 0 55
:3 0 5e :3 0 c7
:3 0 768 769 0
3fd 767 76b 54
:4 0 3ff 766 76e
402 765 770 :3 0
4c :2 0 8d :4 0
405 772 774 :3 0
4c :2 0 4f :4 0
408 776 778 :3 0
4c :2 0 3 :3 0
40b 77a 77c :3 0
4c :2 0 4f :4 0
40e 77e 780 :3 0
411 756 782 :2 0
784 414 785 753
784 0 786 416
0 7b0 44 :3 0
45 :3 0 787 788
0 44 :3 0 57
:3 0 78a 78b 0
2f :2 0 2 :4 0
c8 :2 0 59 :2 0
2f :2 0 418 790
792 :3 0 41b 78c
794 789 796 795
:2 0 47 :3 0 5a
:3 0 798 799 0
49 :4 0 c9 :4 0
4c :2 0 4d :3 0
41f 79d 79f :3 0
422 79a 7a1 :2 0
7a3 425 7a4 797
7a3 0 7a5 427
0 7b0 90 :3 0
91 :3 0 7a6 7a7
0 5e :3 0 c7
:3 0 7a9 7aa 0
3 :3 0 92 :3 0
429 7a8 7ae :2 0
7b0 42d 7b1 742
7b0 0 7b2 431
0 924 36 :3 0
32 :3 0 7b3 7b4
0 924 9a :3 0
9b :3 0 7b6 7b7
0 17 :3 0 27
:3 0 9c :2 0 433
7b8 7bc 5d :2 0
21 :2 0 439 7be
7c0 :3 0 17 :4 0
7c2 7c3 0 7c6
ca :3 0 43c 922
9a :3 0 9b :3 0
7c7 7c8 0 17
:3 0 19 :3 0 9c
:2 0 43e 7c9 7cd
af :2 0 21 :2 0
444 7cf 7d1 :3 0
9a :3 0 cb :3 0
cc :3 0 9a :3 0
cd :3 0 ce :3 0
cf :3 0 d0 :3 0
17 :3 0 d1 :3 0
a6 :3 0 e :3 0
cc :3 0 d2 :3 0
85 :3 0 5 :4 0
d3 1 :8 0 7e9
c :3 0 7b :3 0
7e4 7e5 0 7c
:3 0 7e6 7e7 0
7e9 447 83c 7d
:3 0 c :3 0 7b
:3 0 7eb 7ec 0
7c :3 0 7ed 7ee
0 7f0 44a 7f2
44c 7f1 7f0 :2 0
83a 7e :3 0 c
:3 0 7b :3 0 7f5
7f6 0 7c :3 0
7f7 7f8 0 832
44 :3 0 45 :3 0
7fa 7fb 0 44
:3 0 57 :3 0 7fd
7fe 0 7f :2 0
2 :4 0 d4 :2 0
59 :2 0 2f :2 0
44e 803 805 :3 0
451 7ff 807 7fc
809 808 :2 0 47
:3 0 5a :3 0 80b
80c 0 49 :4 0
d5 :4 0 4c :2 0
4d :3 0 455 810
812 :3 0 4c :2 0
82 :4 0 458 814
816 :3 0 4c :2 0
83 :4 0 45b 818
81a :3 0 4c :2 0
7c :3 0 45e 81c
81e :3 0 4c :2 0
84 :4 0 461 820
822 :3 0 4c :2 0
85 :3 0 86 :3 0
2f :2 0 87 :2 0
464 825 829 468
824 82b :3 0 46b
80d 82d :2 0 82f
46e 830 80a 82f
0 831 470 0
832 472 835 :3 0
835 0 835 834
832 833 :6 0 837
c :3 0 475 839
477 838 837 :2 0
83a 479 :2 0 83c
0 83c 83b 7e9
83a :6 0 8b0 1
:3 0 c :3 0 7b
:3 0 83e 83f 0
c :3 0 5d :2 0
88 :3 0 841 843
0 47e 842 845
:3 0 846 :2 0 9a
:3 0 cb :3 0 d2
:3 0 85 :3 0 5
:3 0 9a :3 0 cd
:3 0 17 :3 0 d6
:4 0 d7 1 :8 0
857 c :3 0 7b
:3 0 852 853 0
7c :3 0 854 855
0 857 481 8aa
7d :3 0 c :3 0
7b :3 0 859 85a
0 7c :3 0 85b
85c 0 85e 484
860 486 85f 85e
:2 0 8a8 7e :3 0
c :3 0 7b :3 0
863 864 0 7c
:3 0 865 866 0
8a0 44 :3 0 45
:3 0 868 869 0
44 :3 0 57 :3 0
86b 86c 0 7f
:2 0 2 :4 0 d8
:2 0 59 :2 0 2f
:2 0 488 871 873
:3 0 48b 86d 875
86a 877 876 :2 0
47 :3 0 5a :3 0
879 87a 0 49
:4 0 d9 :4 0 4c
:2 0 4d :3 0 48f
87e 880 :3 0 4c
:2 0 82 :4 0 492
882 884 :3 0 4c
:2 0 83 :4 0 495
886 888 :3 0 4c
:2 0 7c :3 0 498
88a 88c :3 0 4c
:2 0 84 :4 0 49b
88e 890 :3 0 4c
:2 0 85 :3 0 86
:3 0 2f :2 0 87
:2 0 49e 893 897
4a2 892 899 :3 0
4a5 87b 89b :2 0
89d 4a8 89e 878
89d 0 89f 4aa
0 8a0 4ac 8a3
:3 0 8a3 0 8a3
8a2 8a0 8a1 :6 0
8a5 e :3 0 4af
8a7 4b1 8a6 8a5
:2 0 8a8 4b3 :2 0
8aa 0 8aa 8a9
857 8a8 :6 0 8ac
1 :3 0 4b6 8ad
847 8ac 0 8ae
4b8 0 8b0 ca
:3 0 4ba 8b1 7d2
8b0 0 923 7
:3 0 b3 :2 0 4bd
8b3 8b4 :3 0 8b5
:2 0 44 :3 0 45
:3 0 8b7 8b8 0
44 :3 0 57 :3 0
8ba 8bb 0 f
:2 0 2 :4 0 da
:2 0 59 :2 0 2f
:2 0 4bf 8c0 8c2
:3 0 4c2 8bc 8c4
8b9 8c6 8c5 :2 0
47 :3 0 5a :3 0
8c8 8c9 0 49
:4 0 db :4 0 4c
:2 0 4d :3 0 4c6
8cd 8cf :3 0 4c
:2 0 82 :4 0 4c9
8d1 8d3 :3 0 4c
:2 0 8b :4 0 4cc
8d5 8d7 :3 0 4c
:2 0 51 :3 0 55
:3 0 21 :2 0 4cf
8db 8dd 54 :4 0
4d1 8da 8e0 4d4
8d9 8e2 :3 0 4c
:2 0 8d :4 0 4d7
8e4 8e6 :3 0 4c
:2 0 4f :4 0 4da
8e8 8ea :3 0 4c
:2 0 3 :3 0 4dd
8ec 8ee :3 0 4c
:2 0 4f :4 0 4e0
8f0 8f2 :3 0 4e3
8ca 8f4 :2 0 8f6
4e6 8f7 8c7 8f6
0 8f8 4e8 0
920 44 :3 0 45
:3 0 8f9 8fa 0
44 :3 0 57 :3 0
8fc 8fd 0 2f
:2 0 2 :4 0 dc
:2 0 59 :2 0 2f
:2 0 4ea 902 904
:3 0 4ed 8fe 906
8fb 908 907 :2 0
47 :3 0 5a :3 0
90a 90b 0 49
:4 0 dd :4 0 4c
:2 0 4d :3 0 4f1
90f 911 :3 0 4f4
90c 913 :2 0 915
4f7 916 909 915
0 917 4f9 0
920 90 :3 0 91
:3 0 918 919 0
21 :2 0 3 :3 0
92 :3 0 4fb 91a
91e :2 0 920 4ff
921 8b6 920 0
923 7c1 7c6 0
923 503 0 924
507 92c 36 :3 0
21 :2 0 925 926
0 92b 17 :4 0
928 929 0 92b
50b 92d 73d 924
0 92e 0 92b
0 92e 50e 0
fdc 7 :3 0 99
:2 0 511 930 931
:3 0 932 :2 0 5
:3 0 b3 :2 0 513
935 936 :3 0 937
:2 0 938 :2 0 9a
:3 0 9b :3 0 93a
93b 0 18 :3 0
27 :3 0 9c :2 0
515 93c 940 9d
:2 0 21 :2 0 51b
942 944 :3 0 945
:2 0 939 947 946
:2 0 44 :3 0 45
:3 0 949 94a 0
44 :3 0 57 :3 0
94c 94d 0 f
:2 0 2 :4 0 de
:2 0 59 :2 0 2f
:2 0 51e 952 954
:3 0 521 94e 956
94b 958 957 :2 0
47 :3 0 5a :3 0
95a 95b 0 49
:4 0 df :4 0 4c
:2 0 4d :3 0 525
95f 961 :3 0 4c
:2 0 82 :4 0 528
963 965 :3 0 4c
:2 0 8b :4 0 52b
967 969 :3 0 4c
:2 0 51 :3 0 55
:3 0 5e :3 0 e0
:3 0 96e 96f 0
52e 96d 971 54
:4 0 530 96c 974
533 96b 976 :3 0
4c :2 0 8d :4 0
536 978 97a :3 0
4c :2 0 4f :4 0
539 97c 97e :3 0
4c :2 0 3 :3 0
53c 980 982 :3 0
4c :2 0 4f :4 0
53f 984 986 :3 0
542 95c 988 :2 0
98a 545 98b 959
98a 0 98c 547
0 9b7 44 :3 0
45 :3 0 98d 98e
0 44 :3 0 57
:3 0 990 991 0
2f :2 0 2 :4 0
e1 :2 0 59 :2 0
2f :2 0 549 996
998 :3 0 54c 992
99a 98f 99c 99b
:2 0 47 :3 0 5a
:3 0 99e 99f 0
49 :4 0 e2 :4 0
4c :2 0 4d :3 0
550 9a3 9a5 :3 0
553 9a0 9a7 :2 0
9a9 556 9aa 99d
9a9 0 9ab 558
0 9b7 90 :3 0
91 :3 0 9ac 9ad
0 5e :3 0 e0
:3 0 9af 9b0 0
3 :3 0 92 :3 0
55a 9ae 9b4 :2 0
9b7 ca :3 0 55e
a2b 29 :3 0 99
:2 0 562 9b9 9ba
:3 0 9bb :2 0 9bc
:2 0 9a :3 0 9b
:3 0 9be 9bf 0
18 :3 0 29 :3 0
9c :2 0 564 9c0
9c4 e3 :2 0 21
:2 0 56a 9c6 9c8
:3 0 9c9 :2 0 9bd
9cb 9ca :2 0 44
:3 0 45 :3 0 9cd
9ce 0 44 :3 0
57 :3 0 9d0 9d1
0 e4 :2 0 2
:4 0 e5 :2 0 59
:2 0 2f :2 0 56d
9d6 9d8 :3 0 570
9d2 9da 9cf 9dc
9db :2 0 47 :3 0
5a :3 0 9de 9df
0 49 :4 0 e6
:4 0 4c :2 0 4d
:3 0 574 9e3 9e5
:3 0 4c :2 0 82
:4 0 577 9e7 9e9
:3 0 4c :2 0 8b
:4 0 57a 9eb 9ed
:3 0 4c :2 0 51
:3 0 55 :3 0 5e
:3 0 c7 :3 0 9f2
9f3 0 57d 9f1
9f5 54 :4 0 57f
9f0 9f8 582 9ef
9fa :3 0 585 9e0
9fc :2 0 9fe 588
9ff 9dd 9fe 0
a00 58a 0 a29
44 :3 0 45 :3 0
a01 a02 0 44
:3 0 57 :3 0 a04
a05 0 2f :2 0
2 :4 0 e7 :2 0
59 :2 0 2f :2 0
58c a0a a0c :3 0
58f a06 a0e a03
a10 a0f :2 0 47
:3 0 5a :3 0 a12
a13 0 49 :4 0
e8 :4 0 4c :2 0
4d :3 0 593 a17
a19 :3 0 596 a14
a1b :2 0 a1d 599
a1e a11 a1d 0
a1f 59b 0 a29
90 :3 0 e9 :3 0
a20 a21 0 5e
:3 0 c7 :3 0 a23
a24 0 92 :3 0
59d a22 a27 :2 0
a29 5a0 a2a 9cc
a29 0 a2c 948
9b7 0 a2c 5a4
0 bc1 5 :3 0
b3 :2 0 5a7 a2e
a2f :3 0 a30 :2 0
a31 :2 0 9a :3 0
9b :3 0 a33 a34
0 18 :3 0 1a
:3 0 9c :2 0 5a9
a35 a39 9d :2 0
21 :2 0 5af a3b
a3d :3 0 a3e :2 0
a32 a40 a3f :2 0
3b :3 0 2f :2 0
a42 a43 0 a45
5b2 a4a 3b :3 0
21 :2 0 a46 a47
0 a49 5b4 a4b
a41 a45 0 a4c
0 a49 0 a4c
5b6 0 bc1 9a
:3 0 9b :3 0 a4d
a4e 0 18 :3 0
28 :3 0 9c :2 0
5b9 a4f a53 5d
:2 0 21 :2 0 5bf
a55 a57 :3 0 18
:4 0 a59 a5a 0
a5d ca :3 0 5c2
bbf 9a :3 0 ae
:3 0 a5e a5f 0
18 :3 0 1a :3 0
5c4 a60 a63 af
:2 0 21 :2 0 5c9
a65 a67 :3 0 9a
:3 0 cb :3 0 cc
:3 0 9a :3 0 cd
:3 0 ce :3 0 ea
:3 0 d0 :3 0 18
:3 0 d1 :3 0 a6
:3 0 e :3 0 cc
:3 0 d2 :3 0 85
:3 0 7 :4 0 eb
1 :8 0 a7f c
:3 0 7b :3 0 a7a
a7b 0 7c :3 0
a7c a7d 0 a7f
5cc ad2 7d :3 0
c :3 0 7b :3 0
a81 a82 0 7c
:3 0 a83 a84 0
a86 5cf a88 5d1
a87 a86 :2 0 ad0
7e :3 0 c :3 0
7b :3 0 a8b a8c
0 7c :3 0 a8d
a8e 0 ac8 44
:3 0 45 :3 0 a90
a91 0 44 :3 0
57 :3 0 a93 a94
0 7f :2 0 2
:4 0 ec :2 0 59
:2 0 2f :2 0 5d3
a99 a9b :3 0 5d6
a95 a9d a92 a9f
a9e :2 0 47 :3 0
5a :3 0 aa1 aa2
0 49 :4 0 ed
:4 0 4c :2 0 4d
:3 0 5da aa6 aa8
:3 0 4c :2 0 82
:4 0 5dd aaa aac
:3 0 4c :2 0 83
:4 0 5e0 aae ab0
:3 0 4c :2 0 7c
:3 0 5e3 ab2 ab4
:3 0 4c :2 0 84
:4 0 5e6 ab6 ab8
:3 0 4c :2 0 85
:3 0 86 :3 0 2f
:2 0 87 :2 0 5e9
abb abf 5ed aba
ac1 :3 0 5f0 aa3
ac3 :2 0 ac5 5f3
ac6 aa0 ac5 0
ac7 5f5 0 ac8
5f7 acb :3 0 acb
0 acb aca ac8
ac9 :6 0 acd 10
:3 0 5fa acf 5fc
ace acd :2 0 ad0
5fe :2 0 ad2 0
ad2 ad1 a7f ad0
:6 0 b46 1 :3 0
c :3 0 7b :3 0
ad4 ad5 0 c
:3 0 5d :2 0 88
:3 0 ad7 ad9 0
603 ad8 adb :3 0
adc :2 0 9a :3 0
cb :3 0 d2 :3 0
85 :3 0 7 :3 0
9a :3 0 cd :3 0
18 :3 0 d6 :4 0
ee 1 :8 0 aed
c :3 0 7b :3 0
ae8 ae9 0 7c
:3 0 aea aeb 0
aed 606 b40 7d
:3 0 c :3 0 7b
:3 0 aef af0 0
7c :3 0 af1 af2
0 af4 609 af6
60b af5 af4 :2 0
b3e 7e :3 0 c
:3 0 7b :3 0 af9
afa 0 7c :3 0
afb afc 0 b36
44 :3 0 45 :3 0
afe aff 0 44
:3 0 57 :3 0 b01
b02 0 7f :2 0
2 :4 0 ef :2 0
59 :2 0 2f :2 0
60d b07 b09 :3 0
610 b03 b0b b00
b0d b0c :2 0 47
:3 0 5a :3 0 b0f
b10 0 49 :4 0
f0 :4 0 4c :2 0
4d :3 0 614 b14
b16 :3 0 4c :2 0
82 :4 0 617 b18
b1a :3 0 4c :2 0
83 :4 0 61a b1c
b1e :3 0 4c :2 0
7c :3 0 61d b20
b22 :3 0 4c :2 0
84 :4 0 620 b24
b26 :3 0 4c :2 0
85 :3 0 86 :3 0
2f :2 0 87 :2 0
623 b29 b2d 627
b28 b2f :3 0 62a
b11 b31 :2 0 b33
62d b34 b0e b33
0 b35 62f 0
b36 631 b39 :3 0
b39 0 b39 b38
b36 b37 :6 0 b3b
12 :3 0 634 b3d
636 b3c b3b :2 0
b3e 638 :2 0 b40
0 b40 b3f aed
b3e :6 0 b42 1
:3 0 63b b43 add
b42 0 b44 63d
0 b46 ca :3 0
63f b47 a68 b46
0 bc0 9a :3 0
9b :3 0 b48 b49
0 18 :3 0 27
:3 0 9c :2 0 642
b4a b4e 5d :2 0
21 :2 0 648 b50
b52 :3 0 44 :3 0
45 :3 0 b54 b55
0 44 :3 0 57
:3 0 b57 b58 0
f :2 0 2 :4 0
f1 :2 0 59 :2 0
2f :2 0 64b b5d
b5f :3 0 64e b59
b61 b56 b63 b62
:2 0 47 :3 0 5a
:3 0 b65 b66 0
49 :4 0 f2 :4 0
4c :2 0 4d :3 0
652 b6a b6c :3 0
4c :2 0 82 :4 0
655 b6e b70 :3 0
4c :2 0 8b :4 0
658 b72 b74 :3 0
4c :2 0 51 :3 0
55 :3 0 21 :2 0
65b b78 b7a 54
:4 0 65d b77 b7d
660 b76 b7f :3 0
4c :2 0 8d :4 0
663 b81 b83 :3 0
4c :2 0 4f :4 0
666 b85 b87 :3 0
4c :2 0 3 :3 0
669 b89 b8b :3 0
4c :2 0 4f :4 0
66c b8d b8f :3 0
66f b67 b91 :2 0
b93 672 b94 b64
b93 0 b95 674
0 bbd 44 :3 0
45 :3 0 b96 b97
0 44 :3 0 57
:3 0 b99 b9a 0
2f :2 0 2 :4 0
f3 :2 0 59 :2 0
2f :2 0 676 b9f
ba1 :3 0 679 b9b
ba3 b98 ba5 ba4
:2 0 47 :3 0 5a
:3 0 ba7 ba8 0
49 :4 0 f4 :4 0
4c :2 0 4d :3 0
67d bac bae :3 0
680 ba9 bb0 :2 0
bb2 683 bb3 ba6
bb2 0 bb4 685
0 bbd 90 :3 0
91 :3 0 bb5 bb6
0 21 :2 0 3
:3 0 92 :3 0 687
bb7 bbb :2 0 bbd
68b bbe b53 bbd
0 bc0 a58 a5d
0 bc0 68f 0
bc1 693 bc6 18
:4 0 bc2 bc3 0
bc5 697 bc7 933
bc1 0 bc8 0
bc5 0 bc8 699
0 fdc 15 :3 0
a :3 0 bc9 bca
0 fdc 3c :3 0
21 :2 0 bcc bcd
0 fdc 51 :3 0
39 :3 0 21 :2 0
69c bcf bd2 5e
:3 0 af :2 0 5f
:3 0 bd4 bd6 0
6a1 bd5 bd8 :3 0
78 :3 0 66 :3 0
17 :3 0 68 :3 0
18 :3 0 b4 :3 0
79 :3 0 3 :4 0
f5 1 :8 0 bf8
b6 :4 0 be3 :3 0
92 :3 0 5d :2 0
6a6 be6 be7 :3 0
c :3 0 7b :3 0
be9 bea 0 21
:2 0 beb bec 0
bee 6a9 bf5 c
:3 0 7b :3 0 bef
bf0 0 b7 :2 0
bf1 bf2 0 bf4
6ab bf6 be8 bee
0 bf7 0 bf4
0 bf7 6ad 0
bf8 6b0 c4b 7d
:3 0 c :3 0 7b
:3 0 bfa bfb 0
7c :3 0 bfc bfd
0 bff 6b3 c01
6b5 c00 bff :2 0
c49 7e :3 0 c
:3 0 7b :3 0 c04
c05 0 7c :3 0
c06 c07 0 c41
44 :3 0 45 :3 0
c09 c0a 0 44
:3 0 57 :3 0 c0c
c0d 0 7f :2 0
2 :4 0 f6 :2 0
59 :2 0 2f :2 0
6b7 c12 c14 :3 0
6ba c0e c16 c0b
c18 c17 :2 0 47
:3 0 5a :3 0 c1a
c1b 0 49 :4 0
f7 :4 0 4c :2 0
4d :3 0 6be c1f
c21 :3 0 4c :2 0
82 :4 0 6c1 c23
c25 :3 0 4c :2 0
83 :4 0 6c4 c27
c29 :3 0 4c :2 0
7c :3 0 6c7 c2b
c2d :3 0 4c :2 0
84 :4 0 6ca c2f
c31 :3 0 4c :2 0
85 :3 0 86 :3 0
2f :2 0 87 :2 0
6cd c34 c38 6d1
c33 c3a :3 0 6d4
c1c c3c :2 0 c3e
6d7 c3f c19 c3e
0 c40 6d9 0
c41 6db c44 :3 0
c44 0 c44 c43
c41 c42 :6 0 c46
14 :3 0 6de c48
6e0 c47 c46 :2 0
c49 6e2 :2 0 c4b
0 c4b c4a bf8
c49 :6 0 c4d 1
:3 0 6e5 fd9 11
:3 0 99 :2 0 6e7
c4f c50 :3 0 c51
:2 0 7 :3 0 99
:2 0 6e9 c54 c55
:3 0 c56 :2 0 f8
:3 0 3c :3 0 d6
:3 0 cc :3 0 f9
:3 0 60 :3 0 e
:3 0 79 :3 0 11
:3 0 79 :3 0 12
:3 0 cc :3 0 d2
:3 0 85 :3 0 7
:4 0 fa 1 :8 0
c6d c :3 0 7b
:3 0 c68 c69 0
7c :3 0 c6a c6b
0 c6d 6eb cc0
7d :3 0 c :3 0
7b :3 0 c6f c70
0 7c :3 0 c71
c72 0 c74 6ee
c76 6f0 c75 c74
:2 0 cbe 7e :3 0
c :3 0 7b :3 0
c79 c7a 0 7c
:3 0 c7b c7c 0
cb6 44 :3 0 45
:3 0 c7e c7f 0
44 :3 0 57 :3 0
c81 c82 0 7f
:2 0 2 :4 0 fb
:2 0 59 :2 0 2f
:2 0 6f2 c87 c89
:3 0 6f5 c83 c8b
c80 c8d c8c :2 0
47 :3 0 5a :3 0
c8f c90 0 49
:4 0 fc :4 0 4c
:2 0 4d :3 0 6f9
c94 c96 :3 0 4c
:2 0 82 :4 0 6fc
c98 c9a :3 0 4c
:2 0 83 :4 0 6ff
c9c c9e :3 0 4c
:2 0 7c :3 0 702
ca0 ca2 :3 0 4c
:2 0 84 :4 0 705
ca4 ca6 :3 0 4c
:2 0 85 :3 0 86
:3 0 2f :2 0 87
:2 0 708 ca9 cad
70c ca8 caf :3 0
70f c91 cb1 :2 0
cb3 712 cb4 c8e
cb3 0 cb5 714
0 cb6 716 cb9
:3 0 cb9 0 cb9
cb8 cb6 cb7 :6 0
cbb 16 :3 0 719
cbd 71b cbc cbb
:2 0 cbe 71d :2 0
cc0 0 cc0 cbf
c6d cbe :6 0 d3c
1 :3 0 51 :3 0
3c :3 0 21 :2 0
720 cc2 cc5 5d
:2 0 21 :2 0 725
cc7 cc9 :3 0 44
:3 0 45 :3 0 ccb
ccc 0 44 :3 0
57 :3 0 cce ccf
0 f :2 0 2
:4 0 fd :2 0 59
:2 0 2f :2 0 728
cd4 cd6 :3 0 72b
cd0 cd8 ccd cda
cd9 :2 0 47 :3 0
5a :3 0 cdc cdd
0 49 :4 0 fe
:4 0 4c :2 0 4d
:3 0 72f ce1 ce3
:3 0 4c :2 0 82
:4 0 732 ce5 ce7
:3 0 4c :2 0 8b
:4 0 735 ce9 ceb
:3 0 4c :2 0 51
:3 0 55 :3 0 5e
:3 0 ff :3 0 cf0
cf1 0 738 cef
cf3 54 :4 0 73a
cee cf6 73d ced
cf8 :3 0 4c :2 0
8d :4 0 740 cfa
cfc :3 0 4c :2 0
4f :4 0 743 cfe
d00 :3 0 4c :2 0
12 :3 0 746 d02
d04 :3 0 4c :2 0
4f :4 0 749 d06
d08 :3 0 74c cde
d0a :2 0 d0c 74f
d0d cdb d0c 0
d0e 751 0 d38
44 :3 0 45 :3 0
d0f d10 0 44
:3 0 57 :3 0 d12
d13 0 2f :2 0
2 :4 0 100 :2 0
59 :2 0 2f :2 0
753 d18 d1a :3 0
756 d14 d1c d11
d1e d1d :2 0 47
:3 0 5a :3 0 d20
d21 0 49 :4 0
101 :4 0 4c :2 0
4d :3 0 75a d25
d27 :3 0 75d d22
d29 :2 0 d2b 760
d2c d1f d2b 0
d2d 762 0 d38
90 :3 0 91 :3 0
d2e d2f 0 5e
:3 0 ff :3 0 d31
d32 0 12 :3 0
92 :3 0 764 d30
d36 :2 0 d38 768
d39 cca d38 0
d3a 76c 0 d3c
ca :3 0 76e f7b
5 :3 0 99 :2 0
771 d3e d3f :3 0
d40 :2 0 f8 :3 0
3c :3 0 d6 :3 0
cc :3 0 f9 :3 0
60 :3 0 e :3 0
79 :3 0 11 :3 0
79 :3 0 12 :3 0
cc :3 0 d2 :3 0
85 :3 0 5 :4 0
102 1 :8 0 d57
c :3 0 7b :3 0
d52 d53 0 7c
:3 0 d54 d55 0
d57 773 daa 7d
:3 0 c :3 0 7b
:3 0 d59 d5a 0
7c :3 0 d5b d5c
0 d5e 776 d60
778 d5f d5e :2 0
da8 7e :3 0 c
:3 0 7b :3 0 d63
d64 0 7c :3 0
d65 d66 0 da0
44 :3 0 45 :3 0
d68 d69 0 44
:3 0 57 :3 0 d6b
d6c 0 7f :2 0
2 :4 0 103 :2 0
59 :2 0 2f :2 0
77a d71 d73 :3 0
77d d6d d75 d6a
d77 d76 :2 0 47
:3 0 5a :3 0 d79
d7a 0 49 :4 0
104 :4 0 4c :2 0
4d :3 0 781 d7e
d80 :3 0 4c :2 0
82 :4 0 784 d82
d84 :3 0 4c :2 0
83 :4 0 787 d86
d88 :3 0 4c :2 0
7c :3 0 78a d8a
d8c :3 0 4c :2 0
84 :4 0 78d d8e
d90 :3 0 4c :2 0
85 :3 0 86 :3 0
2f :2 0 87 :2 0
790 d93 d97 794
d92 d99 :3 0 797
d7b d9b :2 0 d9d
79a d9e d78 d9d
0 d9f 79c 0
da0 79e da3 :3 0
da3 0 da3 da2
da0 da1 :6 0 da5
18 :3 0 7a1 da7
7a3 da6 da5 :2 0
da8 7a5 :2 0 daa
0 daa da9 d57
da8 :6 0 f0b 1
:3 0 51 :3 0 3c
:3 0 21 :2 0 7a8
dac daf 5d :2 0
21 :2 0 7ad db1
db3 :3 0 44 :3 0
45 :3 0 db5 db6
0 44 :3 0 57
:3 0 db8 db9 0
f :2 0 2 :4 0
105 :2 0 59 :2 0
2f :2 0 7b0 dbe
dc0 :3 0 7b3 dba
dc2 db7 dc4 dc3
:2 0 47 :3 0 5a
:3 0 dc6 dc7 0
49 :4 0 106 :4 0
4c :2 0 4d :3 0
7b7 dcb dcd :3 0
4c :2 0 82 :4 0
7ba dcf dd1 :3 0
4c :2 0 8b :4 0
7bd dd3 dd5 :3 0
4c :2 0 51 :3 0
55 :3 0 5e :3 0
ff :3 0 dda ddb
0 7c0 dd9 ddd
54 :4 0 7c2 dd8
de0 7c5 dd7 de2
:3 0 4c :2 0 8d
:4 0 7c8 de4 de6
:3 0 4c :2 0 4f
:4 0 7cb de8 dea
:3 0 4c :2 0 12
:3 0 7ce dec dee
:3 0 4c :2 0 4f
:4 0 7d1 df0 df2
:3 0 7d4 dc8 df4
:2 0 df6 7d7 df7
dc5 df6 0 df8
7d9 0 e22 44
:3 0 45 :3 0 df9
dfa 0 44 :3 0
57 :3 0 dfc dfd
0 2f :2 0 2
:4 0 107 :2 0 59
:2 0 2f :2 0 7db
e02 e04 :3 0 7de
dfe e06 dfb e08
e07 :2 0 47 :3 0
5a :3 0 e0a e0b
0 49 :4 0 108
:4 0 4c :2 0 4d
:3 0 7e2 e0f e11
:3 0 7e5 e0c e13
:2 0 e15 7e8 e16
e09 e15 0 e17
7ea 0 e22 90
:3 0 91 :3 0 e18
e19 0 5e :3 0
ff :3 0 e1b e1c
0 12 :3 0 92
:3 0 7ec e1a e20
:2 0 e22 7f0 f08
109 :3 0 10a :3 0
33 :3 0 f9 :3 0
60 :3 0 e :3 0
10b :3 0 11 :3 0
cc :3 0 d2 :3 0
85 :3 0 5 :4 0
10c 1 :8 0 e35
c :3 0 7b :3 0
e30 e31 0 7c
:3 0 e32 e33 0
e35 7f4 e88 7d
:3 0 c :3 0 7b
:3 0 e37 e38 0
7c :3 0 e39 e3a
0 e3c 7f7 e3e
7f9 e3d e3c :2 0
e86 7e :3 0 c
:3 0 7b :3 0 e41
e42 0 7c :3 0
e43 e44 0 e7e
44 :3 0 45 :3 0
e46 e47 0 44
:3 0 57 :3 0 e49
e4a 0 7f :2 0
2 :4 0 10d :2 0
59 :2 0 2f :2 0
7fb e4f e51 :3 0
7fe e4b e53 e48
e55 e54 :2 0 47
:3 0 5a :3 0 e57
e58 0 49 :4 0
10e :4 0 4c :2 0
4d :3 0 802 e5c
e5e :3 0 4c :2 0
82 :4 0 805 e60
e62 :3 0 4c :2 0
83 :4 0 808 e64
e66 :3 0 4c :2 0
7c :3 0 80b e68
e6a :3 0 4c :2 0
84 :4 0 80e e6c
e6e :3 0 4c :2 0
85 :3 0 86 :3 0
2f :2 0 87 :2 0
811 e71 e75 815
e70 e77 :3 0 818
e59 e79 :2 0 e7b
81b e7c e56 e7b
0 e7d 81d 0
e7e 81f e81 :3 0
e81 0 e81 e80
e7e e7f :6 0 e83
1a :3 0 822 e85
824 e84 e83 :2 0
e86 826 :2 0 e88
0 e88 e87 e35
e86 :6 0 f07 1
:3 0 51 :3 0 33
:3 0 21 :2 0 829
e8a e8d 51 :3 0
9d :2 0 32 :3 0
21 :2 0 82c e8f
e93 831 e90 e95
:3 0 44 :3 0 45
:3 0 e97 e98 0
44 :3 0 57 :3 0
e9a e9b 0 f
:2 0 2 :4 0 10f
:2 0 59 :2 0 2f
:2 0 834 ea0 ea2
:3 0 837 e9c ea4
e99 ea6 ea5 :2 0
47 :3 0 5a :3 0
ea8 ea9 0 49
:4 0 110 :4 0 4c
:2 0 4d :3 0 83b
ead eaf :3 0 4c
:2 0 82 :4 0 83e
eb1 eb3 :3 0 4c
:2 0 8b :4 0 841
eb5 eb7 :3 0 4c
:2 0 51 :3 0 55
:3 0 5e :3 0 ff
:3 0 ebc ebd 0
844 ebb ebf 54
:4 0 846 eba ec2
849 eb9 ec4 :3 0
4c :2 0 8d :4 0
84c ec6 ec8 :3 0
4c :2 0 4f :4 0
84f eca ecc :3 0
4c :2 0 12 :3 0
852 ece ed0 :3 0
4c :2 0 4f :4 0
855 ed2 ed4 :3 0
858 eaa ed6 :2 0
ed8 85b ed9 ea7
ed8 0 eda 85d
0 f04 44 :3 0
45 :3 0 edb edc
0 44 :3 0 57
:3 0 ede edf 0
2f :2 0 2 :4 0
111 :2 0 59 :2 0
2f :2 0 85f ee4
ee6 :3 0 862 ee0
ee8 edd eea ee9
:2 0 47 :3 0 5a
:3 0 eec eed 0
49 :4 0 112 :4 0
4c :2 0 4d :3 0
866 ef1 ef3 :3 0
869 eee ef5 :2 0
ef7 86c ef8 eeb
ef7 0 ef9 86e
0 f04 90 :3 0
91 :3 0 efa efb
0 5e :3 0 ff
:3 0 efd efe 0
12 :3 0 92 :3 0
870 efc f02 :2 0
f04 874 f05 e96
f04 0 f06 878
0 f07 87a f09
db4 e22 0 f0a
0 f07 0 f0a
87d 0 f0b 880
f0c d41 f0b 0
f7d 44 :3 0 45
:3 0 f0d f0e 0
44 :3 0 57 :3 0
f10 f11 0 f
:2 0 2 :4 0 113
:2 0 59 :2 0 2f
:2 0 883 f16 f18
:3 0 886 f12 f1a
f0f f1c f1b :2 0
47 :3 0 5a :3 0
f1e f1f 0 49
:4 0 114 :4 0 4c
:2 0 4d :3 0 88a
f23 f25 :3 0 4c
:2 0 82 :4 0 88d
f27 f29 :3 0 4c
:2 0 8b :4 0 890
f2b f2d :3 0 4c
:2 0 51 :3 0 55
:3 0 5e :3 0 115
:3 0 f32 f33 0
893 f31 f35 54
:4 0 895 f30 f38
898 f2f f3a :3 0
4c :2 0 8d :4 0
89b f3c f3e :3 0
4c :2 0 4f :4 0
89e f40 f42 :3 0
4c :2 0 12 :3 0
8a1 f44 f46 :3 0
4c :2 0 4f :4 0
8a4 f48 f4a :3 0
8a7 f20 f4c :2 0
f4e 8aa f4f f1d
f4e 0 f50 8ac
0 f7a 44 :3 0
45 :3 0 f51 f52
0 44 :3 0 57
:3 0 f54 f55 0
2f :2 0 2 :4 0
116 :2 0 59 :2 0
2f :2 0 8ae f5a
f5c :3 0 8b1 f56
f5e f53 f60 f5f
:2 0 47 :3 0 5a
:3 0 f62 f63 0
49 :4 0 117 :4 0
4c :2 0 4d :3 0
8b5 f67 f69 :3 0
8b8 f64 f6b :2 0
f6d 8bb f6e f61
f6d 0 f6f 8bd
0 f7a 90 :3 0
91 :3 0 f70 f71
0 5e :3 0 115
:3 0 f73 f74 0
12 :3 0 92 :3 0
8bf f72 f78 :2 0
f7a 8c3 f7c c57
d3c 0 f7d 0
f7a 0 f7d 8c7
0 f7e 8cb f7f
c52 f7e 0 f80
8cd 0 fd8 38
:3 0 118 :2 0 f81
f82 0 fd8 12
:3 0 3 :3 0 f84
f85 0 fd8 17
:3 0 b3 :2 0 8cf
f88 f89 :3 0 f8a
:2 0 29 :3 0 99
:2 0 8d1 f8d f8e
:3 0 f8f :2 0 f90
:2 0 f8b f92 f91
:2 0 17 :3 0 19
:3 0 f94 f95 0
f97 8d3 f98 f93
f97 0 f99 8d5
0 fd8 3d :3 0
119 :3 0 e :3 0
10 :3 0 12 :3 0
11 :3 0 17 :3 0
18 :3 0 29 :3 0
2e :3 0 36 :3 0
3a :3 0 34 :3 0
32 :3 0 37 :3 0
3b :3 0 38 :3 0
27 :3 0 28 :3 0
23 :3 0 26 :3 0
30 :3 0 31 :3 0
15 :3 0 21 :2 0
8d7 f9b fb3 f9a
fb4 0 fd8 3c
:3 0 118 :2 0 fb6
fb7 0 fd8 3d
:3 0 11a :3 0 e
:3 0 12 :3 0 3c
:3 0 21 :2 0 21
:2 0 8ef fba fc0
fb9 fc1 0 fd8
11 :3 0 99 :2 0
8f5 fc4 fc5 :3 0
fc6 :2 0 3c :3 0
11b :2 0 fc8 fc9
0 fd5 3d :3 0
11a :3 0 e :3 0
11 :3 0 3c :3 0
21 :2 0 21 :2 0
8f7 fcc fd2 fcb
fd3 0 fd5 8fd
fd6 fc7 fd5 0
fd7 900 0 fd8
902 fda bd9 c4d
0 fdb 0 fd8
0 fdb 90b 0
fdc 90e fdd 691
fdc 0 fde 916
0 fdf 918 fe1
4f0 564 0 fe2
0 fdf 0 fe2
91c 0 1088 9a
:3 0 ae :3 0 fe3
fe4 0 14 :3 0
15 :3 0 91f fe5
fe8 5d :2 0 21
:2 0 924 fea fec
:3 0 fed :2 0 51
:3 0 39 :3 0 21
:2 0 927 fef ff2
5e :3 0 af :2 0
5f :3 0 ff4 ff6
0 92c ff5 ff8
:3 0 ff9 :2 0 fee
ffb ffa :2 0 3f
:3 0 b0 :4 0 ffd
ffe 0 1031 3e
:3 0 118 :2 0 1000
1001 0 1031 42
:3 0 c :3 0 7b
:3 0 1004 1005 0
1003 1006 0 1031
40 :3 0 b1 :3 0
3f :3 0 12 :3 0
15 :3 0 3e :3 0
21 :2 0 92f 1009
100f 1008 1010 0
1031 c :3 0 7b
:3 0 1012 1013 0
42 :3 0 1014 1015
0 1031 3f :3 0
b2 :4 0 1017 1018
0 1031 3e :3 0
118 :2 0 101a 101b
0 1031 42 :3 0
c :3 0 7b :3 0
101e 101f 0 101d
1020 0 1031 40
:3 0 b1 :3 0 3f
:3 0 12 :3 0 15
:3 0 3e :3 0 21
:2 0 935 1023 1029
1022 102a 0 1031
c :3 0 7b :3 0
102c 102d 0 42
:3 0 102e 102f 0
1031 93b 1032 ffc
1031 0 1033 946
0 1088 3d :3 0
11c :3 0 e :3 0
948 1035 1037 1034
1038 0 1088 44
:3 0 45 :3 0 103a
103b 0 44 :3 0
57 :3 0 103d 103e
0 11d :2 0 2
:4 0 11e :2 0 59
:2 0 2f :2 0 94a
1043 1045 :3 0 94d
103f 1047 103c 1049
1048 :2 0 47 :3 0
5a :3 0 104b 104c
0 49 :4 0 11f
:4 0 4c :2 0 4d
:3 0 951 1050 1052
:3 0 4c :2 0 82
:4 0 954 1054 1056
:3 0 4c :2 0 8b
:4 0 957 1058 105a
:3 0 4c :2 0 51
:3 0 55 :3 0 c
:3 0 7b :3 0 105f
1060 0 95a 105e
1062 54 :4 0 95c
105d 1065 95f 105c
1067 :3 0 4c :2 0
8d :4 0 962 1069
106b :3 0 4c :2 0
4f :4 0 965 106d
106f :3 0 4c :2 0
3 :3 0 968 1071
1073 :3 0 4c :2 0
4f :4 0 96b 1075
1077 :3 0 96e 104d
1079 :2 0 107b 971
107c 104a 107b 0
107d 973 0 1088
90 :3 0 120 :3 0
107e 107f 0 c
:3 0 7b :3 0 1081
1082 0 3 :3 0
92 :3 0 975 1080
1086 :2 0 1088 979
1129 c :3 0 121
:2 0 4 1089 108a
0 44 :3 0 45
:3 0 108c 108d 0
44 :3 0 57 :3 0
108f 1090 0 7f
:2 0 2 :4 0 122
:2 0 59 :2 0 2f
:2 0 992 1095 1097
:3 0 995 1091 1099
108e 109b 109a :2 0
47 :3 0 5a :3 0
109d 109e 0 49
:4 0 123 :4 0 4c
:2 0 4d :3 0 999
10a2 10a4 :3 0 99c
109f 10a6 :2 0 10bf
47 :3 0 5a :3 0
10a8 10a9 0 49
:4 0 124 :4 0 4c
:2 0 c :3 0 a7
:3 0 10ae 10af 0
99f 10ad 10b1 :3 0
4c :2 0 8d :4 0
9a2 10b3 10b5 :3 0
4c :2 0 c :3 0
125 :3 0 10b8 10b9
0 9a5 10b7 10bb
:3 0 9a8 10aa 10bd
:2 0 10bf 9ab 10c0
109c 10bf 0 10c1
9ae 0 10d1 126
:3 0 43 :3 0 c
:3 0 a7 :3 0 127
:3 0 c :3 0 125
:3 0 3 :3 0 d6
:4 0 128 1 :a 0
10cd :2 0 10d1 10c3
10cb :2 0 b :3 0
43 :3 0 10cf :2 0
10d1 9b0 10d3 9b4
10d2 10d1 :2 0 1127
7e :3 0 c :3 0
7b :3 0 10d6 10d7
0 7c :3 0 10d8
10d9 0 111f 44
:3 0 45 :3 0 10db
10dc 0 44 :3 0
57 :3 0 10de 10df
0 7f :2 0 2
:4 0 129 :2 0 59
:2 0 2f :2 0 9b6
10e4 10e6 :3 0 9b9
10e0 10e8 10dd 10ea
10e9 :2 0 47 :3 0
5a :3 0 10ec 10ed
0 49 :4 0 12a
:4 0 4c :2 0 4d
:3 0 9bd 10f1 10f3
:3 0 9c0 10ee 10f5
:2 0 110f 47 :3 0
5a :3 0 10f7 10f8
0 49 :4 0 83
:4 0 4c :2 0 7c
:3 0 9c3 10fc 10fe
:3 0 4c :2 0 84
:4 0 9c6 1100 1102
:3 0 4c :2 0 85
:3 0 86 :3 0 2f
:2 0 87 :2 0 9c9
1105 1109 9cd 1104
110b :3 0 9d0 10f9
110d :2 0 110f 9d3
1110 10eb 110f 0
1111 9d6 0 111f
126 :3 0 43 :3 0
127 :3 0 c :3 0
125 :3 0 3 :3 0
d6 :4 0 12b 1
:a 0 111b :2 0 111f
1113 1119 :2 0 b
:3 0 43 :3 0 111d
:2 0 111f 9d8 1122
:3 0 1122 0 1122
1121 111f 1120 :6 0
1124 1 :3 0 9dd
1126 9df 1125 1124
:2 0 1127 9e1 :2 0
1129 9e4 1129 1128
1088 1127 :6 0 112a
:2 0 2 1c 1129
112d :3 0 112c 112a
112e :8 0 
a13
4
:3 0 1 3 1
7 1 b 1
f 1 13 5
6 a e 12
16 1 20 1
1e 1 27 1
25 1 2e 1
2c 1 35 1
33 1 3c 1
3a 1 43 1
41 1 4a 1
48 1 51 1
4f 1 56 1
5b 1 60 1
65 1 6a 1
6f 1 74 1
7b 1 79 1
80 1 86 1
8c 1 92 1
98 1 9d 1
a2 1 a7 1
ac 1 b1 1
b8 1 b6 1
bd 1 c3 1
c9 1 cf 1
d5 1 db 1
e1 1 e7 1
ed 1 f3 1
f9 1 ff 1
105 1 10b 1
111 1 119 1
117 1 11e 1
123 1 129 1
136 1 142 2
145 147 2 14a
14c 2 14e 150
2 152 154 2
156 158 2 15e
160 2 162 164
2 166 167 2
169 16a 2 15a
16c 2 16e 170
2 176 178 2
17a 17c 2 17e
17f 2 181 182
2 172 184 2
186 188 1 18e
2 190 191 2
18a 193 2 195
197 2 199 19b
2 19d 19f 2
1a1 1a3 2 1a5
1a7 4 13e 144
149 1a9 1 1ab
1 1ad 2 1b7
1b9 3 1b5 1b6
1bb 2 1c4 1c6
2 1c3 1c8 1
1ca 1 1cc 2
1e4 1e5 1 1e9
2 1e7 1e9 1
1f1 1 1f3 2
22c 231 1 238
1 233 2 24b
24d 3 249 24a
24f 2 258 25a
2 25c 25e 2
260 262 2 264
266 2 268 26a
3 26f 270 271
2 26c 273 2
257 275 1 277
1 279 2 242
27a 1 27e 1
23d 2 23b 282
1 28d 2 289
28d 2 299 29b
3 297 298 29d
2 2a6 2a8 2
2aa 2ac 2 2ae
2b0 1 2b8 2
2ba 2bb 2 2b2
2bd 2 2bf 2c1
2 2c3 2c5 2
2c7 2c9 2 2cb
2cd 2 2a5 2cf
1 2d1 1 2d3
2 2dd 2df 3
2db 2dc 2e1 2
2ea 2ec 2 2e9
2ee 1 2f0 1
2f2 3 2f9 2fa
2fb 3 2d4 2f3
2fd 1 2ff 2
302 303 1 307
2 305 307 2
312 314 3 310
311 316 2 31f
321 2 323 325
2 327 329 1
331 2 333 334
2 32b 336 2
338 33a 2 33c
33e 2 340 342
2 344 346 2
31e 348 1 34a
1 34c 2 356
358 3 354 355
35a 2 363 365
2 362 367 1
369 1 36b 3
372 373 374 3
34d 36c 376 1
378 1 37a 1
380 3 38b 38c
38d 1 391 2
38f 391 2 39c
39e 3 39a 39b
3a0 2 3a9 3ab
2 3ad 3af 2
3b1 3b3 1 3bb
2 3bd 3be 2
3b5 3c0 2 3c2
3c4 2 3c6 3c8
2 3ca 3cc 2
3ce 3d0 2 3a8
3d2 1 3d4 1
3d6 2 3e0 3e2
3 3de 3df 3e4
2 3ed 3ef 2
3ec 3f1 1 3f3
1 3f5 3 3fc
3fd 3fe 3 3d7
3f6 400 1 402
1 403 1 405
2 419 41e 1
425 1 420 2
438 43a 3 436
437 43c 2 445
447 2 449 44b
2 44d 44f 2
451 453 2 455
457 3 45c 45d
45e 2 459 460
2 444 462 1
464 1 466 2
42f 467 1 46b
1 42a 2 428
46f 2 47a 47b
1 47f 2 47d
47f 2 484 485
1 48b 2 487
48b 5 49e 49f
4a0 4a1 4a2 5
4b8 4b9 4ba 4bb
4bc a 493 496
49b 4a5 4aa 4ad
4b0 4b5 4bf 4c4
1 4c6 1 4c8
3 4d1 4d2 4d3
1 4d7 2 4d5
4d7 1 4df 1
4e1 1 4e3 1
4e9 1 4fc 2
4fb 4fc 1 504
1 50a 2 50c
50d 2 4f9 50e
1 515 1 510
2 528 52a 3
526 527 52c 2
535 537 2 539
53b 2 53d 53f
2 541 543 2
545 547 3 54c
54d 54e 2 549
550 2 534 552
1 554 1 556
2 51f 557 1
55b 1 51a 2
518 55f 1 562
1 565 1 56a
1 57a 2 579
57a 1 582 1
588 2 58a 58b
2 577 58c 1
593 1 58e 2
5a6 5a8 3 5a4
5a5 5aa 2 5b3
5b5 2 5b7 5b9
2 5bb 5bd 2
5bf 5c1 2 5c3
5c5 3 5ca 5cb
5cc 2 5c7 5ce
2 5b2 5d0 1
5d2 1 5d4 2
59d 5d5 1 5d9
1 598 2 596
5dd 3 5e5 5e6
5e7 1 5eb 2
5e9 5eb 1 5f0
1 5f2 2 5e0
5f3 1 5f5 1
5f7 1 5fc 1
60c 2 60b 60c
1 614 1 61a
2 61c 61d 2
609 61e 1 625
1 620 2 638
63a 3 636 637
63c 2 645 647
2 649 64b 2
64d 64f 2 651
653 2 655 657
3 65c 65d 65e
2 659 660 2
644 662 1 664
1 666 2 62f
667 1 66b 1
62a 2 628 66f
3 677 678 679
1 67d 2 67b
67d 1 682 1
684 2 672 685
1 687 2 68a
68b 1 68f 2
68d 68f 1 692
3 69b 69c 69d
1 6a1 2 69f
6a1 1 6a9 1
6ab 1 6ad 1
6b3 2 6be 6bf
1 6c3 2 6c1
6c3 2 6d1 6d3
3 6cf 6d0 6d5
2 6de 6e0 2
6e2 6e4 2 6e6
6e8 1 6f0 2
6f2 6f3 2 6ea
6f5 2 6f7 6f9
2 6fb 6fd 2
6ff 701 2 703
705 2 6dd 707
1 709 1 70b
2 715 717 3
713 714 719 2
722 724 2 721
726 1 728 1
72a 3 731 732
733 3 70c 72b
735 1 737 1
739 1 73e 2
74b 74d 3 749
74a 74f 2 758
75a 2 75c 75e
2 760 762 1
76a 2 76c 76d
2 764 76f 2
771 773 2 775
777 2 779 77b
2 77d 77f 2
757 781 1 783
1 785 2 78f
791 3 78d 78e
793 2 79c 79e
2 79b 7a0 1
7a2 1 7a4 3
7ab 7ac 7ad 3
786 7a5 7af 1
7b1 3 7b9 7ba
7bb 1 7bf 2
7bd 7bf 1 7c4
3 7ca 7cb 7cc
1 7d0 2 7ce
7d0 2 7e3 7e8
1 7ef 1 7ea
2 802 804 3
800 801 806 2
80f 811 2 813
815 2 817 819
2 81b 81d 2
81f 821 3 826
827 828 2 823
82a 2 80e 82c
1 82e 1 830
2 7f9 831 1
835 1 7f4 2
7f2 839 1 844
2 840 844 2
851 856 1 85d
1 858 2 870
872 3 86e 86f
874 2 87d 87f
2 881 883 2
885 887 2 889
88b 2 88d 88f
3 894 895 896
2 891 898 2
87c 89a 1 89c
1 89e 2 867
89f 1 8a3 1
862 2 860 8a7
1 8aa 1 8ad
2 83c 8ae 1
8b2 2 8bf 8c1
3 8bd 8be 8c3
2 8cc 8ce 2
8d0 8d2 2 8d4
8d6 1 8dc 2
8de 8df 2 8d8
8e1 2 8e3 8e5
2 8e7 8e9 2
8eb 8ed 2 8ef
8f1 2 8cb 8f3
1 8f5 1 8f7
2 901 903 3
8ff 900 905 2
90e 910 2 90d
912 1 914 1
916 3 91b 91c
91d 3 8f8 917
91f 3 922 8b1
921 3 7b2 7b5
923 2 927 92a
2 92c 92d 1
92f 1 934 3
93d 93e 93f 1
943 2 941 943
2 951 953 3
94f 950 955 2
95e 960 2 962
964 2 966 968
1 970 2 972
973 2 96a 975
2 977 979 2
97b 97d 2 97f
981 2 983 985
2 95d 987 1
989 1 98b 2
995 997 3 993
994 999 2 9a2
9a4 2 9a1 9a6
1 9a8 1 9aa
3 9b1 9b2 9b3
3 98c 9ab 9b5
1 9b8 3 9c1
9c2 9c3 1 9c7
2 9c5 9c7 2
9d5 9d7 3 9d3
9d4 9d9 2 9e2
9e4 2 9e6 9e8
2 9ea 9ec 1
9f4 2 9f6 9f7
2 9ee 9f9 2
9e1 9fb 1 9fd
1 9ff 2 a09
a0b 3 a07 a08
a0d 2 a16 a18
2 a15 a1a 1
a1c 1 a1e 2
a25 a26 3 a00
a1f a28 2 a2b
a2a 1 a2d 3
a36 a37 a38 1
a3c 2 a3a a3c
1 a44 1 a48
2 a4a a4b 3
a50 a51 a52 1
a56 2 a54 a56
1 a5b 2 a61
a62 1 a66 2
a64 a66 2 a79
a7e 1 a85 1
a80 2 a98 a9a
3 a96 a97 a9c
2 aa5 aa7 2
aa9 aab 2 aad
aaf 2 ab1 ab3
2 ab5 ab7 3
abc abd abe 2
ab9 ac0 2 aa4
ac2 1 ac4 1
ac6 2 a8f ac7
1 acb 1 a8a
2 a88 acf 1
ada 2 ad6 ada
2 ae7 aec 1
af3 1 aee 2
b06 b08 3 b04
b05 b0a 2 b13
b15 2 b17 b19
2 b1b b1d 2
b1f b21 2 b23
b25 3 b2a b2b
b2c 2 b27 b2e
2 b12 b30 1
b32 1 b34 2
afd b35 1 b39
1 af8 2 af6
b3d 1 b40 1
b43 2 ad2 b44
3 b4b b4c b4d
1 b51 2 b4f
b51 2 b5c b5e
3 b5a b5b b60
2 b69 b6b 2
b6d b6f 2 b71
b73 1 b79 2
b7b b7c 2 b75
b7e 2 b80 b82
2 b84 b86 2
b88 b8a 2 b8c
b8e 2 b68 b90
1 b92 1 b94
2 b9e ba0 3
b9c b9d ba2 2
bab bad 2 baa
baf 1 bb1 1
bb3 3 bb8 bb9
bba 3 b95 bb4
bbc 3 bbf b47
bbe 3 a2c a4c
bc0 1 bc4 2
bc6 bc7 2 bd0
bd1 1 bd7 2
bd3 bd7 1 be5
2 be4 be5 1
bed 1 bf3 2
bf5 bf6 2 be2
bf7 1 bfe 1
bf9 2 c11 c13
3 c0f c10 c15
2 c1e c20 2
c22 c24 2 c26
c28 2 c2a c2c
2 c2e c30 3
c35 c36 c37 2
c32 c39 2 c1d
c3b 1 c3d 1
c3f 2 c08 c40
1 c44 1 c03
2 c01 c48 1
c4b 1 c4e 1
c53 2 c67 c6c
1 c73 1 c6e
2 c86 c88 3
c84 c85 c8a 2
c93 c95 2 c97
c99 2 c9b c9d
2 c9f ca1 2
ca3 ca5 3 caa
cab cac 2 ca7
cae 2 c92 cb0
1 cb2 1 cb4
2 c7d cb5 1
cb9 1 c78 2
c76 cbd 2 cc3
cc4 1 cc8 2
cc6 cc8 2 cd3
cd5 3 cd1 cd2
cd7 2 ce0 ce2
2 ce4 ce6 2
ce8 cea 1 cf2
2 cf4 cf5 2
cec cf7 2 cf9
cfb 2 cfd cff
2 d01 d03 2
d05 d07 2 cdf
d09 1 d0b 1
d0d 2 d17 d19
3 d15 d16 d1b
2 d24 d26 2
d23 d28 1 d2a
1 d2c 3 d33
d34 d35 3 d0e
d2d d37 1 d39
2 cc0 d3a 1
d3d 2 d51 d56
1 d5d 1 d58
2 d70 d72 3
d6e d6f d74 2
d7d d7f 2 d81
d83 2 d85 d87
2 d89 d8b 2
d8d d8f 3 d94
d95 d96 2 d91
d98 2 d7c d9a
1 d9c 1 d9e
2 d67 d9f 1
da3 1 d62 2
d60 da7 2 dad
dae 1 db2 2
db0 db2 2 dbd
dbf 3 dbb dbc
dc1 2 dca dcc
2 dce dd0 2
dd2 dd4 1 ddc
2 dde ddf 2
dd6 de1 2 de3
de5 2 de7 de9
2 deb ded 2
def df1 2 dc9
df3 1 df5 1
df7 2 e01 e03
3 dff e00 e05
2 e0e e10 2
e0d e12 1 e14
1 e16 3 e1d
e1e e1f 3 df8
e17 e21 2 e2f
e34 1 e3b 1
e36 2 e4e e50
3 e4c e4d e52
2 e5b e5d 2
e5f e61 2 e63
e65 2 e67 e69
2 e6b e6d 3
e72 e73 e74 2
e6f e76 2 e5a
e78 1 e7a 1
e7c 2 e45 e7d
1 e81 1 e40
2 e3e e85 2
e8b e8c 2 e91
e92 1 e94 2
e8e e94 2 e9f
ea1 3 e9d e9e
ea3 2 eac eae
2 eb0 eb2 2
eb4 eb6 1 ebe
2 ec0 ec1 2
eb8 ec3 2 ec5
ec7 2 ec9 ecb
2 ecd ecf 2
ed1 ed3 2 eab
ed5 1 ed7 1
ed9 2 ee3 ee5
3 ee1 ee2 ee7
2 ef0 ef2 2
eef ef4 1 ef6
1 ef8 3 eff
f00 f01 3 eda
ef9 f03 1 f05
2 e88 f06 2
f08 f09 2 daa
f0a 2 f15 f17
3 f13 f14 f19
2 f22 f24 2
f26 f28 2 f2a
f2c 1 f34 2
f36 f37 2 f2e
f39 2 f3b f3d
2 f3f f41 2
f43 f45 2 f47
f49 2 f21 f4b
1 f4d 1 f4f
2 f59 f5b 3
f57 f58 f5d 2
f66 f68 2 f65
f6a 1 f6c 1
f6e 3 f75 f76
f77 3 f50 f6f
f79 3 f7b f0c
f7c 1 f7d 1
f7f 1 f87 1
f8c 1 f96 1
f98 17 f9c f9d
f9e f9f fa0 fa1
fa2 fa3 fa4 fa5
fa6 fa7 fa8 fa9
faa fab fac fad
fae faf fb0 fb1
fb2 5 fbb fbc
fbd fbe fbf 1
fc3 5 fcd fce
fcf fd0 fd1 2
fca fd4 1 fd6
8 f80 f83 f86
f99 fb5 fb8 fc2
fd7 2 fd9 fda
7 6ac 738 92e
bc8 bcb bce fdb
1 fdd 3 5f6
688 fde 2 fe0
fe1 2 fe6 fe7
1 feb 2 fe9
feb 2 ff0 ff1
1 ff7 2 ff3
ff7 5 100a 100b
100c 100d 100e 5
1024 1025 1026 1027
1028 a fff 1002
1007 1011 1016 1019
101c 1021 102b 1030
1 1032 1 1036
2 1042 1044 3
1040 1041 1046 2
104f 1051 2 1053
1055 2 1057 1059
1 1061 2 1063
1064 2 105b 1066
2 1068 106a 2
106c 106e 2 1070
1072 2 1074 1076
2 104e 1078 1
107a 1 107c 3
1083 1084 1085 18
1ae 1cd 1d0 1d3
1d6 1d9 1dc 1df
1e2 1f4 285 300
379 406 409 472
476 4c7 4e2 fe2
1033 1039 107d 1087
2 1094 1096 3
1092 1093 1098 2
10a1 10a3 2 10a0
10a5 2 10ac 10b0
2 10b2 10b4 2
10b6 10ba 2 10ab
10bc 2 10a7 10be
1 10c0 3 10c1
10cc 10d0 1 108b
2 10e3 10e5 3
10e1 10e2 10e7 2
10f0 10f2 2 10ef
10f4 2 10fb 10fd
2 10ff 1101 3
1106 1107 1108 2
1103 110a 2 10fa
110c 2 10f6 110e
1 1110 4 10da
1111 111a 111e 1
1122 1 10d5 2
10d3 1126 2e 23
2a 31 38 3f
46 4d 54 59
5e 63 68 6d
72 77 7e 84
8a 90 96 9b
a0 a5 aa af
b4 bb c1 c7
cd d3 d9 df
e5 eb f1 f7
fd 103 109 10f
115 11c 121 127
12e 
1
4
0 
112d
0
1
28
1c
34
0 1 2 1 4 1 6 1
8 1 a 1 c 1 e 1
10 1 12 1 14 1 16 1
18 1 1a 1 0 0 0 0
0 0 0 0 0 0 0 0

13 1 0
f9 1 0
105 1 0
6f 1 0
3 1 0
80 1 0
5b 1 0
33 1 0
2c 1 0
c9 1 0
111 1 0
92 1 0
f3 1 0
11e 1 0
ac 1 0
10b 1 0
a2 1 0
e1 1 0
60 1 0
98 1 0
56 1 0
117 1 0
bd 1 0
4f 1 0
ed 1 0
d5 1 0
65 1 0
ff 1 0
cf 1 0
3a 1 0
c3 1 0
25 1 0
79 1 0
a7 1 0
9d 1 0
74 1 0
123 1 0
7 1 0
b6 1 0
b1 1 0
86 1 0
41 1 0
1e 1 0
2 0 1
6a 1 0
f 1 0
8c 1 0
db 1 0
48 1 0
e7 1 0
b 1 0
129 1 0
0

/
SHOW ERRORS FUNCTION SP_U_WP_FORECAST;
ALTER FUNCTION SP_U_WP_FORECAST COMPILE DEBUG;
/*****************************************************************************/
Prompt == FUNCTION SP_U_ASSIGN_DESCR ==
CREATE OR REPLACE FUNCTION SP_U_ASSIGN_DESCR wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
57
2 :e:
1FUNCTION:
1SP_U_ASSIGN_DESCR:
1PRJ_ID:
1VARCHAR2:
1RES_ID001:
1DESCRIPTION001:
1REC_USER001:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_U_ASSIGN_DESCR(:
1':
1, :
1);:
1CHECK_TRACE:
11:
120:
1+:
1FPUT:
1               ENTER:: SP_U_ASSIGN_DESCR(21) :: cplan.sql(21215) :: :
1IS NULL:
132:
125:
1        RETURN_ERROR:: SP_U_ASSIGN_DESCR(26) :: cplan.sql(21220) :: :
1 :
1STATE:: :
1COALESCE:
1TO_CHAR:
1PMO_CNST:
1INVALID_PARAMS:
1NULL:
1 ID:: :
128:
1                EXIT:: SP_U_ASSIGN_DESCR(29) :: cplan.sql(21223) :: :
1PMODEFINES:
1PMO_RETURN_ERROR_SP:
1TRUE:
1PROJECT_ASSIGNMENTS:
1DESCRIPTION:
1RESOURCE_ID:
1PROJECT_ID:
1UPDATE PROJECT_ASSIGNMENTS:n      SET DESCRIPTION = description001:n      WHE+
1RE RESOURCE_ID = res_id001:n      AND PROJECT_ID = prj_id:
1FOUND:
1=:
1SC_STATE:
10:
1100:
1NO_DATA_FOUND:
1SQLCODE:
1OTHERS:
116:
151:
1           EXCEPTION:: SP_U_ASSIGN_DESCR(52) :: cplan.sql(21246) :: :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
14:
157:
1       RETURN_STATUS:: SP_U_ASSIGN_DESCR(58) :: cplan.sql(21252) :: :
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
164:
1           EXCEPTION:: SP_U_ASSIGN_DESCR(65) :: cplan.sql(21259) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1OPEN:
1ERROR:
1ID:
1DUAL:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
176:
1           EXCEPTION:: SP_U_ASSIGN_DESCR(77) :: cplan.sql(21271) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
283
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d 8f a0 b0 3d 8f
a0 b0 3d b4 :3 a0 6b 2c 6a
a3 :2 a0 6b 1c 81 b0 :2 a0 6b
:2 a0 6b 6e a5 b a 10 :2 a0
6b 6e :2 a0 6b 6e a5 b 6e
7e a0 b4 2e 6e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e a5 57 b7
19 3c :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 b7 19 3c a0 7e b4
2e 5a a0 7e b4 2e 5a 52
10 :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
:4 a0 6b a5 b 6e a5 b b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e a5 57 b7 19 3c :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b :2 a0 a5 57
b7 19 3c :7 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e :4 a0 6b a5 b 6e a5
b b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e a5 57 b7 19 3c
:2 a0 6b :2 a0 6b :2 a0 a5 57 b7
:2 a0 6b :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 :2 a0 6b :2 6e 7e :2 a0 6b
b4 2e 7e 6e b4 2e 7e :2 a0
6b b4 2e a5 57 b7 19 3c
:9 a0 12a e9 dd :2 a0 65 b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c :7 a0 12a
e9 dd :2 a0 65 b7 a4 b1 11
4f b7 a6 9 a4 b1 11 68
4f 1d 17 b5 
283
2
0 3 7 23 1f 1e 2b 38
34 1b 40 49 45 33 51 5e
5a 30 66 59 6b 6f 73 56
77 7b 9b 83 87 8b 8e 96
82 a2 a6 7f aa ae b2 b5
ba bb 1 bd c2 c6 ca cd
d2 d6 da dd e2 e3 e5 ea
ed f1 f2 f7 fc ff 104 105
10a 10d 111 112 117 11a 11f 120
125 128 12d 12e 133 136 13b 13c
141 144 148 149 14e 151 156 157
15c 15f 164 165 16a 16d 172 173
178 17b 17f 180 185 188 18d 18e
193 196 19b 19c 1a1 1a4 1a9 1aa
1af 1b2 1b6 1b7 1bc 1bf 1c4 1c5
1ca 1cd 1d2 1d3 1d8 1d9 1de 1e0
1e4 1e7 1eb 1ef 1f2 1f6 1fa 1fd
200 205 208 20b 20e 20f 214 215
1 217 21c 220 224 227 22c 231
234 238 239 23e 23f 244 246 24a
24d 251 254 255 25a 25d 261 264
265 26a 1 26d 272 276 27a 27d
281 285 288 28b 290 293 296 299
29a 29f 2a0 1 2a2 2a7 2ab 2af
2b2 2b7 2bc 2bf 2c3 2c4 2c9 2cc
2d1 2d2 2d7 2da 2df 2e0 2e5 2e8
2ec 2f0 2f4 2f8 2fb 2fc 2fe 303
304 306 307 30c 30f 314 315 31a
31d 322 323 328 32b 32f 330 335
338 33d 33e 343 344 349 34b 34f
352 356 35a 35d 361 365 368 36b
370 373 376 379 37a 37f 380 1
382 387 38b 38f 392 397 39c 39f
3a3 3a4 3a9 3aa 3af 3b1 3b5 3b8
3bc 3c0 3c3 3c7 3cb 3ce 3d2 3d6
3d7 3dc 3de 3e2 3e5 3e9 3ed 3f1
3f5 3f9 3fd 401 40d 411 416 41a
41d 41e 423 427 42b 42e 431 435
437 43b 43f 442 445 449 44b 44f
453 456 458 45c 460 464 467 46b
46f 471 472 477 1 47b 47f 483
486 48a 48e 492 496 499 49d 4a1
4a4 4a7 4ac 4af 4b2 4b5 4b6 4bb
4bc 1 4be 4c3 4c7 4cb 4ce 4d3
4d8 4db 4df 4e0 4e5 4e8 4ed 4ee
4f3 4f6 4fb 4fc 501 504 508 509
50e 511 516 517 51c 51f 523 527
52a 52d 52e 530 531 536 537 53c
53e 542 545 547 54b 54d 559 55b
55d 55e 563 567 569 575 577 57b
57f 582 586 58a 58d 590 595 598
59b 59e 59f 5a4 5a5 1 5a7 5ac
5b0 5b4 5b7 5bc 5c1 5c4 5c8 5c9
5ce 5d1 5d6 5d7 5dc 5df 5e4 5e5
5ea 5ed 5f1 5f5 5f9 5fd 600 601
603 608 609 60b 60c 611 614 619
61a 61f 622 627 628 62d 630 634
635 63a 63d 642 643 648 649 64e
650 654 657 65b 65f 662 666 66a
66d 671 675 676 67b 67d 681 685
688 68c 690 693 697 69b 69e 6a1
6a6 6a9 6ac 6af 6b0 6b5 6b6 1
6b8 6bd 6c1 6c5 6c8 6cd 6d2 6d5
6d9 6da 6df 6e0 6e5 6e9 6ed 6f0
6f5 6fa 6fd 701 705 708 709 70e
711 716 717 71c 71f 723 727 72a
72b 730 731 736 738 73c 73f 743
747 74b 74f 753 757 75b 75f 763
76f 774 778 77c 780 784 786 787
78c 1 790 794 798 79b 79f 7a3
7a7 7ab 7ae 7b2 7b6 7b9 7bc 7c1
7c4 7c7 7ca 7cb 7d0 7d1 1 7d3
7d8 7dc 7e0 7e3 7e8 7ed 7f0 7f4
7f5 7fa 7fb 800 804 808 80b 810
815 818 81c 81d 822 825 82a 82b
830 833 837 83b 83e 841 842 844
845 84a 84b 850 852 856 859 85d
861 865 869 86d 871 875 881 886
88a 88e 892 896 898 89c 89e 8aa
8ac 8ae 8af 8b4 8b8 8ba 8c6 8ca
8cc 8cd 8d6 
283
2
0 1 a 6 d :3 6 10 :3 6
15 :3 6 12 :2 6 1 3 a 13
a :2 1 4 e 17 :3 e 4 7
:2 11 23 :2 2d 3a :2 23 :3 7 :2 d 19
25 :2 2f 43 :2 25 59 5f 62 :2 59
70 8b 8 :2 70 d 10 :2 70 17
1a :2 70 b e :2 70 13 16 :2 70
1b 1e :2 70 28 2b :2 70 b e
:2 70 13 16 :2 70 1b 1e :2 70 2d
30 :2 70 b e :2 70 13 16 :2 70
1b 1e :2 70 2a 2d :2 70 32 35
:2 70 :2 7 4f :2 4 7 :2 11 23 :2 2d
39 3c 51 53 54 :2 51 :2 23 :3 7
:2 d 12 1e 62 65 :2 1e :2 7 57
:2 4 :4 8 7 :4 1c 1b :2 7 a :2 14
26 :2 30 3c 40 55 57 58 :2 55
:2 26 :3 a :2 10 15 21 65 68 :2 21
75 78 :2 21 7c 7f :2 21 89 8c
95 9d :2 a6 :2 95 b7 :2 8c :2 21 bf
c2 :2 21 ca cd :2 21 d2 d5 :2 21
dc df :2 21 :2 a 5b :2 7 a :2 14
26 :2 30 3c 3f 54 56 57 :2 54
:2 26 :3 a :2 10 15 21 65 68 :2 21
:2 a 5a :3 7 :2 12 26 :2 2f 3f 47
:2 7 2f :2 4 e b 19 d 1b
b 18 7 e a 16 :3 14 a
:2 13 1f a 1b a :2 13 1f a
:4 7 4 c a :2 13 1f a 1a
:2 7 :2 c a :2 13 1f a d :2 17
29 :2 33 3f 43 58 5a 5b :2 58
:2 29 :3 d :2 13 18 24 68 6b :2 24
78 7b :2 24 7f 82 :2 24 8e 91
:2 24 99 9c :2 24 a8 ab b2 bb
be :2 ab :2 24 :2 d 5e :2 a :2 7 :4 13
:2 7 4 :3 1 7 :2 11 23 :2 2d 39
3c 51 53 54 :2 51 :2 23 :3 7 :2 d
12 1e 62 65 :2 1e 72 75 :2 1e
79 7c :2 1e 86 89 92 9a :2 a3
:2 92 ae :2 89 :2 1e b6 b9 :2 1e c1
c4 :2 1e c9 cc :2 1e d3 d6 :2 1e
:2 7 57 :3 4 :2 f 21 :2 2a 34 3c
:2 4 1 9 12 9 a :2 14 26
:2 30 3c 40 55 57 58 :2 55 :2 26
:3 a :2 10 15 21 65 68 :2 21 :3 a
:2 10 15 39 5c 5f :2 68 :2 39 76
79 :2 39 81 84 :2 8d :2 39 :2 a 5b
:3 7 c 15 1e 2c a 13 1e
f e 7 e 7 e 7 29
:2 4 :2 9 7 :2 10 1c :2 7 :2 11 23
:2 2d 39 3d 52 54 55 :2 52 :2 23
:3 7 :2 d 12 1e 62 65 :2 1e :3 7
:2 d 12 36 42 45 :2 36 4d 50
:2 36 5c 5f 66 6f 72 :2 5f :2 36
:2 7 58 :2 4 7 c 1b a 13
1e f e 7 e 7 e 7
:2 4 :4 10 :2 4 :8 1 
283
4
0 :2 1 :4 3 :4 4
:4 5 :4 6 2 :4 7
:2 1 :7 a :b c :11 d
e :2 d :2 e :2 d
:2 e :2 d :2 f :2 d
:2 f :2 d :2 f :2 d
:2 f :2 d :2 10 :2 d
:2 10 :2 d :2 10 :2 d
:2 10 :2 d :2 11 :2 d
:2 11 :2 d :2 11 :2 d
:2 11 :2 d :2 11 :4 d
:3 c :11 14 :b 15 :3 14
:c 18 :11 19 :30 1a :3 19
:11 1c :b 1d :3 1c :a 1f
:3 18 24 :2 25 :2 26
:2 27 24 :6 28 :5 29
28 :5 2b 2a :3 28
23 2e :5 2f :3 2e
:2 30 :5 32 :11 33 :24 34
:3 33 :2 31 :6 30 2d
:3 b :11 39 :30 3a :3 39
:a 3c b :3 3f :11 40
:b 41 :17 42 :3 40 :2 44
:3 45 :3 46 47 45
44 45 :3 48 :3 3f
:2 49 :5 4b :11 4c :b 4d
:18 4e :3 4c :2 50 51
:3 52 53 51 50
51 :3 54 :2 4a :6 49
3e :7 1 
8d8
4
:3 0 1 :3 0 2
:a 0 27e 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 9
56 0 7 4
:3 0 5 :7 0 9
8 :3 0 4 :3 0
6 :7 0 d c
:3 0 15 16 0
b 4 :3 0 7
:7 0 11 10 :3 0
8 :3 0 9 :3 0
a :2 0 4 13
17 0 27e 2
18 :2 0 21 22
0 10 9 :3 0
a :2 0 4 1b
1c 0 1d :7 0
20 1e 0 27c
0 b :6 0 c
:3 0 d :3 0 c
:3 0 e :3 0 24
25 0 2 :4 0
12 26 28 23
2a 29 :2 0 f
:3 0 10 :3 0 2c
2d 0 11 :4 0
c :3 0 12 :3 0
30 31 0 2
:4 0 14 32 34
13 :4 0 14 :2 0
15 :3 0 16 37
39 :3 0 16 :4 0
14 :2 0 17 :4 0
19 3c 3e :3 0
14 :2 0 3 :3 0
1c 40 42 :3 0
14 :2 0 17 :4 0
1f 44 46 :3 0
14 :2 0 18 :4 0
22 48 4a :3 0
14 :2 0 17 :4 0
25 4c 4e :3 0
14 :2 0 5 :3 0
28 50 52 :3 0
14 :2 0 17 :4 0
2b 54 56 :3 0
14 :2 0 18 :4 0
2e 58 5a :3 0
14 :2 0 17 :4 0
31 5c 5e :3 0
14 :2 0 6 :3 0
34 60 62 :3 0
14 :2 0 17 :4 0
37 64 66 :3 0
14 :2 0 18 :4 0
3a 68 6a :3 0
14 :2 0 17 :4 0
3d 6c 6e :3 0
14 :2 0 7 :3 0
40 70 72 :3 0
14 :2 0 17 :4 0
43 74 76 :3 0
14 :2 0 19 :4 0
46 78 7a :3 0
49 2e 7c :2 0
7e 4e 7f 2b
7e 0 80 50
0 1dc c :3 0
d :3 0 81 82
0 c :3 0 1a
:3 0 84 85 0
1b :2 0 2 :4 0
1c :2 0 1d :2 0
1b :2 0 52 8a
8c :3 0 55 86
8e 83 90 8f
:2 0 f :3 0 1e
:3 0 92 93 0
11 :4 0 1f :4 0
14 :2 0 15 :3 0
59 97 99 :3 0
5c 94 9b :2 0
9d 5f 9e 91
9d 0 9f 61
0 1dc 3 :3 0
20 :2 0 63 a1
a2 :3 0 a3 :2 0
5 :3 0 20 :2 0
65 a6 a7 :3 0
a8 :2 0 a4 aa
a9 :2 0 c :3 0
d :3 0 ac ad
0 c :3 0 1a
:3 0 af b0 0
21 :2 0 2 :4 0
22 :2 0 1d :2 0
1b :2 0 67 b5
b7 :3 0 6a b1
b9 ae bb ba
:2 0 f :3 0 1e
:3 0 bd be 0
11 :4 0 23 :4 0
14 :2 0 15 :3 0
6e c2 c4 :3 0
14 :2 0 24 :4 0
71 c6 c8 :3 0
14 :2 0 25 :4 0
74 ca cc :3 0
14 :2 0 26 :3 0
27 :3 0 28 :3 0
29 :3 0 d1 d2
0 77 d0 d4
2a :4 0 79 cf
d7 7c ce d9
:3 0 14 :2 0 2b
:4 0 7f db dd
:3 0 14 :2 0 17
:4 0 82 df e1
:3 0 14 :2 0 3
:3 0 85 e3 e5
:3 0 14 :2 0 17
:4 0 88 e7 e9
:3 0 8b bf eb
:2 0 ed 8e ee
bc ed 0 ef
90 0 119 c
:3 0 d :3 0 f0
f1 0 c :3 0
1a :3 0 f3 f4
0 1b :2 0 2
:4 0 2c :2 0 1d
:2 0 1b :2 0 92
f9 fb :3 0 95
f5 fd f2 ff
fe :2 0 f :3 0
1e :3 0 101 102
0 11 :4 0 2d
:4 0 14 :2 0 15
:3 0 99 106 108
:3 0 9c 103 10a
:2 0 10c 9f 10d
100 10c 0 10e
a1 0 119 2e
:3 0 2f :3 0 10f
110 0 28 :3 0
29 :3 0 112 113
0 3 :3 0 30
:3 0 a3 111 117
:2 0 119 a7 11a
ab 119 0 11b
ab 0 1dc 31
:3 0 32 :3 0 6
:3 0 33 :3 0 5
:3 0 34 :3 0 3
:4 0 35 1 :8 0
139 36 :4 0 124
:3 0 30 :3 0 37
:2 0 af 127 128
:3 0 9 :3 0 38
:3 0 12a 12b 0
39 :2 0 12c 12d
0 12f b2 136
9 :3 0 38 :3 0
130 131 0 3a
:2 0 132 133 0
135 b4 137 129
12f 0 138 0
135 0 138 b6
0 139 b9 18c
3b :3 0 9 :3 0
38 :3 0 13b 13c
0 3c :3 0 13d
13e 0 140 bc
142 be 141 140
:2 0 18a 3d :3 0
9 :3 0 38 :3 0
145 146 0 3c
:3 0 147 148 0
182 c :3 0 d
:3 0 14a 14b 0
c :3 0 1a :3 0
14d 14e 0 3e
:2 0 2 :4 0 3f
:2 0 1d :2 0 1b
:2 0 c0 153 155
:3 0 c3 14f 157
14c 159 158 :2 0
f :3 0 1e :3 0
15b 15c 0 11
:4 0 40 :4 0 14
:2 0 15 :3 0 c7
160 162 :3 0 14
:2 0 24 :4 0 ca
164 166 :3 0 14
:2 0 41 :4 0 cd
168 16a :3 0 14
:2 0 3c :3 0 d0
16c 16e :3 0 14
:2 0 42 :4 0 d3
170 172 :3 0 14
:2 0 43 :3 0 44
:3 0 1b :2 0 45
:2 0 d6 175 179
da 174 17b :3 0
dd 15d 17d :2 0
17f e0 180 15a
17f 0 181 e2
0 182 e4 185
:3 0 185 0 185
184 182 183 :6 0
187 2 :3 0 e7
189 e9 188 187
:2 0 18a eb :2 0
18c 0 18c 18b
139 18a :6 0 1dc
1 :3 0 c :3 0
d :3 0 18e 18f
0 c :3 0 1a
:3 0 191 192 0
46 :2 0 2 :4 0
47 :2 0 1d :2 0
1b :2 0 ee 197
199 :3 0 f1 193
19b 190 19d 19c
:2 0 f :3 0 1e
:3 0 19f 1a0 0
11 :4 0 48 :4 0
14 :2 0 15 :3 0
f5 1a4 1a6 :3 0
14 :2 0 24 :4 0
f8 1a8 1aa :3 0
14 :2 0 25 :4 0
fb 1ac 1ae :3 0
14 :2 0 26 :3 0
27 :3 0 9 :3 0
38 :3 0 1b3 1b4
0 fe 1b2 1b6
2a :4 0 100 1b1
1b9 103 1b0 1bb
:3 0 14 :2 0 2b
:4 0 106 1bd 1bf
:3 0 14 :2 0 17
:4 0 109 1c1 1c3
:3 0 14 :2 0 3
:3 0 10c 1c5 1c7
:3 0 14 :2 0 17
:4 0 10f 1c9 1cb
:3 0 112 1a1 1cd
:2 0 1cf 115 1d0
19e 1cf 0 1d1
117 0 1dc 2e
:3 0 49 :3 0 1d2
1d3 0 9 :3 0
38 :3 0 1d5 1d6
0 3 :3 0 30
:3 0 119 1d4 1da
:2 0 1dc 11d 27d
9 :3 0 4a :2 0
4 1dd 1de 0
c :3 0 d :3 0
1e0 1e1 0 c
:3 0 1a :3 0 1e3
1e4 0 3e :2 0
2 :4 0 4b :2 0
1d :2 0 1b :2 0
124 1e9 1eb :3 0
127 1e5 1ed 1e2
1ef 1ee :2 0 f
:3 0 1e :3 0 1f1
1f2 0 11 :4 0
4c :4 0 14 :2 0
15 :3 0 12b 1f6
1f8 :3 0 12e 1f3
1fa :2 0 213 f
:3 0 1e :3 0 1fc
1fd 0 11 :4 0
4d :4 0 14 :2 0
9 :3 0 4e :3 0
202 203 0 131
201 205 :3 0 14
:2 0 2b :4 0 134
207 209 :3 0 14
:2 0 9 :3 0 4f
:3 0 20c 20d 0
137 20b 20f :3 0
13a 1fe 211 :2 0
213 13d 214 1f0
213 0 215 140
0 225 50 :3 0
b :3 0 9 :3 0
4e :3 0 51 :3 0
9 :3 0 4f :3 0
52 :3 0 53 :4 0
54 1 :a 0 221
:2 0 225 217 21f
:2 0 8 :3 0 b
:3 0 223 :2 0 225
142 227 146 226
225 :2 0 27b 3d
:3 0 9 :3 0 38
:3 0 22a 22b 0
3c :3 0 22c 22d
0 273 c :3 0
d :3 0 22f 230
0 c :3 0 1a
:3 0 232 233 0
3e :2 0 2 :4 0
55 :2 0 1d :2 0
1b :2 0 148 238
23a :3 0 14b 234
23c 231 23e 23d
:2 0 f :3 0 1e
:3 0 240 241 0
11 :4 0 56 :4 0
14 :2 0 15 :3 0
14f 245 247 :3 0
152 242 249 :2 0
263 f :3 0 1e
:3 0 24b 24c 0
11 :4 0 41 :4 0
14 :2 0 3c :3 0
155 250 252 :3 0
14 :2 0 42 :4 0
158 254 256 :3 0
14 :2 0 43 :3 0
44 :3 0 1b :2 0
45 :2 0 15b 259
25d 15f 258 25f
:3 0 162 24d 261
:2 0 263 165 264
23f 263 0 265
168 0 273 50
:3 0 b :3 0 51
:3 0 9 :3 0 4f
:3 0 52 :3 0 53
:4 0 57 1 :a 0
26f :2 0 273 267
26d :2 0 8 :3 0
b :3 0 271 :2 0
273 16a 276 :3 0
276 0 276 275
273 274 :6 0 278
1 :3 0 16f 27a
171 279 278 :2 0
27b 173 :2 0 27d
176 27d 27c 1dc
27b :6 0 27e :2 0
2 18 27d 281
:3 0 280 27e 282
:8 0 
178
4
:3 0 1 3 1
7 1 b 1
f 4 6 a
e 12 1 1a
1 27 1 33
2 36 38 2
3b 3d 2 3f
41 2 43 45
2 47 49 2
4b 4d 2 4f
51 2 53 55
2 57 59 2
5b 5d 2 5f
61 2 63 65
2 67 69 2
6b 6d 2 6f
71 2 73 75
2 77 79 4
2f 35 3a 7b
1 7d 1 7f
2 89 8b 3
87 88 8d 2
96 98 2 95
9a 1 9c 1
9e 1 a0 1
a5 2 b4 b6
3 b2 b3 b8
2 c1 c3 2
c5 c7 2 c9
cb 1 d3 2
d5 d6 2 cd
d8 2 da dc
2 de e0 2
e2 e4 2 e6
e8 2 c0 ea
1 ec 1 ee
2 f8 fa 3
f6 f7 fc 2
105 107 2 104
109 1 10b 1
10d 3 114 115
116 3 ef 10e
118 1 11a 1
126 2 125 126
1 12e 1 134
2 136 137 2
123 138 1 13f
1 13a 2 152
154 3 150 151
156 2 15f 161
2 163 165 2
167 169 2 16b
16d 2 16f 171
3 176 177 178
2 173 17a 2
15e 17c 1 17e
1 180 2 149
181 1 185 1
144 2 142 189
2 196 198 3
194 195 19a 2
1a3 1a5 2 1a7
1a9 2 1ab 1ad
1 1b5 2 1b7
1b8 2 1af 1ba
2 1bc 1be 2
1c0 1c2 2 1c4
1c6 2 1c8 1ca
2 1a2 1cc 1
1ce 1 1d0 3
1d7 1d8 1d9 6
80 9f 11b 18c
1d1 1db 2 1e8
1ea 3 1e6 1e7
1ec 2 1f5 1f7
2 1f4 1f9 2
200 204 2 206
208 2 20a 20e
2 1ff 210 2
1fb 212 1 214
3 215 220 224
1 1df 2 237
239 3 235 236
23b 2 244 246
2 243 248 2
24f 251 2 253
255 3 25a 25b
25c 2 257 25e
2 24e 260 2
24a 262 1 264
4 22e 265 26e
272 1 276 1
229 2 227 27a
1 1f 
1
4
0 
281
0
1
14
4
6
0 1 2 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 
f 1 0
b 1 0
7 1 0
2 0 1
3 1 0
1a 1 0
0

/
SHOW ERRORS FUNCTION SP_U_ASSIGN_DESCR;
ALTER FUNCTION SP_U_ASSIGN_DESCR COMPILE DEBUG;
/****************************************************************************************/
Prompt == FUNCTION SP_TEST_ASSIGN ==
CREATE OR REPLACE FUNCTION SP_TEST_ASSIGN wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
bc
2 :e:
1FUNCTION:
1SP_TEST_ASSIGN:
1RANK001:
1VARCHAR2:
1BITFLAG001:
1PLS_INTEGER:
1RETURN:
1PMO_DB2M:
1REF_CURSOR:
1H_RANK:
1200:
1H_FULL_NAME:
1255:
1H_PROJECT_ID:
132:
1H_RES_ASSIGNMENT_ID:
1H_TSK_ASSIGNMENT_ID:
1H_SECURITY_ID:
1H_RES_ID:
1H_RES_TYPE:
11:
1H_PARTICIPANT:
10:
1H_BITFLAG:
1CURSOR:
1C_RPA0:
1ELEMENT_ID:
1TMT_WBS:
1TYPE_ID:
1PMO_CNST:
1SC_PROJECT:
1SC_PROPOSAL:
1SC_PROJ_TYPE1:
1SC_PROJ_TYPE2:
1SC_PROJ_TYPE3:
1RANK:
1SELECT ELEMENT_ID FROM TMT_WBS:n         WHERE TYPE_ID IN (PMO_CNST.SC_PROJEC+
1T,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST+
1.SC_PROJ_TYPE3) AND RANK LIKE h_rank || '%':
1C_RPA1:
1RES_ASSIGNMENT_ID:
1TSK_ASSIGNMENT_ID:
1RESOURCE_ID:
1RESOURCE_TYPE:
1TASK_ASSIGNMENTS:
1PROJECT_ID:
1TASK_ID:
1SELECT RES_ASSIGNMENT_ID, TSK_ASSIGNMENT_ID, RESOURCE_ID, RESOURCE_TYPE:n    +
1     FROM TASK_ASSIGNMENTS:n         WHERE PROJECT_ID = h_project_id AND TASK+
1_ID = h_project_id:
1C_FPA1:
1RA:
1PARTICIPANT:
1SECURITY_ID:
1RESOURCE_ASSIGNMENTS:
1PROJECT_ASSIGNMENTS:
1PA:
1SELECT ra.RESOURCE_ID, ra.RESOURCE_TYPE, ra.PARTICIPANT, ra.SECURITY_ID:n    +
1     FROM RESOURCE_ASSIGNMENTS ra:n         WHERE ELEMENT_ID = h_project_id A+
1ND NOT EXISTS (:n         SELECT 'X' FROM PROJECT_ASSIGNMENTS pa WHERE pa.PRO+
1JECT_ID = h_project_id AND pa.RESOURCE_ID = ra.RESOURCE_ID):
1RESULTSET:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= SP_TEST_ASSIGN(:
1':
1, :
1COALESCE:
1TO_CHAR:
1NULL:
1);:
1CHECK_TRACE:
141:
1+:
1FPUT:
1               ENTER:: SP_TEST_ASSIGN(42) :: cplan.sql(21329) :: :
1OPEN:
1LOOP:
1FOUND:
1SC_STATE:
1100:
1!=:
1SC_SQL_SUCCESS:
1EXIT:
1=:
1COUNT:
1DUAL:
1PROJECT_ASSIGNMENT_ID:
1SELECT COUNT('X') INTO h_bitflag FROM DUAL WHERE EXISTS (SELECT 'X' FROM TASK+
1_ASSIGNMENTS:n               WHERE PROJECT_ASSIGNMENT_ID = h_tsk_assignment_i+
1d AND RESOURCE_ID = h_res_id):
1SQLCODE:
1NO_DATA_FOUND:
1OTHERS:
116:
186:
1           EXCEPTION:: SP_TEST_ASSIGN(87) :: cplan.sql(21374) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1UPDATE TASK_ASSIGNMENTS SET PROJECT_ASSIGNMENT_ID = NuLL:n                  W+
1HERE PROJECT_ASSIGNMENT_ID = h_tsk_assignment_id:
1TRUE:
1107:
1           EXCEPTION:: SP_TEST_ASSIGN(108) :: cplan.sql(21395) :: :
1TASK_DAY_OF_YEAR:
1PARENT_ID:
1UPDATE TASK_DAY_OF_YEAR SET PARENT_ID = TSK_ASSIGNMENT_ID:n                  +
1WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_tsk_assignment_id:
1127:
1           EXCEPTION:: SP_TEST_ASSIGN(128) :: cplan.sql(21415) :: :
1DELETE FROM RESOURCE_ASSIGNMENTS:n               WHERE RES_ASSIGNMENT_ID = h_+
1res_assignment_id:n               AND SECURITY_ID = 'SCPROJECTASSIGNMENT':
1149:
1           EXCEPTION:: SP_TEST_ASSIGN(150) :: cplan.sql(21437) :: :
1DELETE FROM PROJECT_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND RESOURCE_+
1ID = h_res_id:
1169:
1           EXCEPTION:: SP_TEST_ASSIGN(170) :: cplan.sql(21457) :: :
1NICK_NAME:
1RESOURCES:
1SELECT NICK_NAME || ';' INTO h_full_name:n                     FROM RESOURCES+
1:n                     WHERE RESOURCE_ID = h_res_id:
1187:
1           EXCEPTION:: SP_TEST_ASSIGN(188) :: cplan.sql(21475) :: :
1PROFILE_NAME:
1PROFILES:
1PROFILE_ID:
1SELECT PROFILE_NAME || ';' INTO h_full_name:n                     FROM PROFIL+
1ES:n                     WHERE PROFILE_ID = h_res_id:
1204:
1           EXCEPTION:: SP_TEST_ASSIGN(205) :: cplan.sql(21492) :: :
1ASSIGN_NAMES:
1REPLACE:
1UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE (ASSIGN_NAMES, h_full_name, ''):n  +
1                WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id:
1225:
1           EXCEPTION:: SP_TEST_ASSIGN(226) :: cplan.sql(21513) :: :
1PMO_STPU:
1BIT_ANDNOT:
1UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = PMO_STPU.BIT_ANDNOT(PARTICIPANT+
1,4096):n                  WHERE RES_ASSIGNMENT_ID = h_res_assignment_id:
1245:
1           EXCEPTION:: SP_TEST_ASSIGN(246) :: cplan.sql(21533) :: :
1DELETE FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id:
1265:
1           EXCEPTION:: SP_TEST_ASSIGN(266) :: cplan.sql(21553) :: :
1DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNME+
1NT_ID = h_tsk_assignment_id:
1284:
1           EXCEPTION:: SP_TEST_ASSIGN(285) :: cplan.sql(21572) :: :
1CLOSE:
1300:
1           EXCEPTION:: SP_TEST_ASSIGN(301) :: cplan.sql(21588) :: :
1COMMIT:
1ELSIF:
1SELECT NICK_NAME || ';' INTO h_full_name:n                  FROM RESOURCES:n +
1                 WHERE RESOURCE_ID = h_res_id:
1333:
1           EXCEPTION:: SP_TEST_ASSIGN(334) :: cplan.sql(21621) :: :
1SELECT PROFILE_NAME || ';' INTO h_full_name:n                  FROM PROFILES+
1:n                  WHERE PROFILE_ID = h_res_id:
1350:
1           EXCEPTION:: SP_TEST_ASSIGN(351) :: cplan.sql(21638) :: :
1INSTR:
1UPDATE TMT_WBS SET ASSIGN_NAMES = CASE:n               WHEN InStr(ASSIGN_NAME+
1S, h_full_name) > 0 THEN ASSIGN_NAMES:n               ELSE ASSIGN_NAMES || h_+
1full_name END:n               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID +
1= h_project_id:
1373:
1           EXCEPTION:: SP_TEST_ASSIGN(374) :: cplan.sql(21661) :: :
1ELEMENT_TYPE_BITS:
1ROLLUP_BITS:
1SC_ASSIGNMENT_BIT:
1INSERT INTO PROJECT_ASSIGNMENTS (PROJECT_ID, RESOURCE_ID, PARTICIPANT, SECURI+
1TY_ID, RESOURCE_TYPE, ELEMENT_TYPE_BITS,ROLLUP_BITS):n               VALUES(h+
1_project_id, h_res_id, h_participant, h_security_id, h_res_type, 1, PMO_CNST.+
1SC_ASSIGNMENT_BIT):
1393:
1           EXCEPTION:: SP_TEST_ASSIGN(394) :: cplan.sql(21681) :: :
1410:
1           EXCEPTION:: SP_TEST_ASSIGN(411) :: cplan.sql(21698) :: :
1429:
1           EXCEPTION:: SP_TEST_ASSIGN(430) :: cplan.sql(21717) :: :
14:
1435:
1       RETURN_STATUS:: SP_TEST_ASSIGN(436) :: cplan.sql(21723) :: :
1STATE:: :
1 ID:: :
1PMODEFINES:
1PMO_RETURN_STATUS:
1RETURN_ERROR_EXCEPTION:
1442:
1           EXCEPTION:: SP_TEST_ASSIGN(443) :: cplan.sql(21730) :: :
1RETURN_ERROR_EXCEPTION :: STATE:: :
1CURRENT_STATE:
1CURRENT_ID:
1ERROR:
1ID:
1SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n +
1        FROM DUAL:
1454:
1           EXCEPTION:: SP_TEST_ASSIGN(455) :: cplan.sql(21742) :: :
1SELECT -9999 "ERROR" ,:n         PMO_DB2M.CURRENT_ID "ID":n         FROM DUAL:
0

0
0
a5d
2
0 a0 8d 8f a0 b0 3d 8f
a0 b0 3d b4 :3 a0 6b 2c 6a
a3 a0 51 a5 1c 81 b0 a3
a0 51 a5 1c 81 b0 a3 a0
51 a5 1c 81 b0 a3 a0 51
a5 1c 81 b0 a3 a0 51 a5
1c 81 b0 a3 a0 51 a5 1c
81 b0 a3 a0 51 a5 1c 81
b0 a3 a0 51 a5 1c 81 b0
a3 a0 1c 51 81 b0 a3 a0
1c 51 81 b0 a0 f4 b4 bf
c8 :f a0 12a bd b7 11 a4 b1
a0 f4 b4 bf c8 :9 a0 12a bd
b7 11 a4 b1 a0 f4 b4 bf
c8 :15 a0 12a bd b7 11 a4 b1
a3 :2 a0 6b 1c 81 b0 :2 a0 6b
:2 a0 6b 6e a5 b a 10 :2 a0
6b 6e :2 a0 6b 6e a5 b 6e
7e a0 b4 2e 6e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e :3 a0 a5
b 6e a5 b b4 2e 7e 6e
b4 2e a5 57 b7 19 3c :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e a5 57 b7
19 3c :2 a0 d :2 a0 d :2 a0 e9
dd b3 :3 a0 e9 d3 :2 a0 f :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c :2 a0 6b a0 7e a0
6b b4 2e 5a a0 2b b7 19
3c :2 a0 51 a5 b 7e 51 b4
2e :2 a0 e9 dd b3 :6 a0 e9 d3
5 :2 a0 f :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c :2 a0
6b a0 7e a0 6b b4 2e 5a
a0 2b b7 19 3c :8 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
51 a5 b 7e 51 b4 2e :4 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :7 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
19 3c :4 a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 6b a0 7e a0 6b
b4 2e 5a :5 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f :2 a0 :2 51 a5 b 7e
6e b4 2e :5 a0 12a :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 :5 a0 12a
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
b7 :2 19 3c :9 a0 12a a0 f a0
7e b4 2e :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 :7 a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f b7 :2 19 3c :3 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :5 a0
12a a0 f a0 7e b4 2e :2 a0
6b 51 d b7 :2 a0 6b 51 d
b7 :2 19 3c b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
a0 47 :2 a0 e9 c1 :2 a0 6b a0
d b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f a0 12a a0
b7 :2 a0 51 a5 b 7e 51 b4
2e :2 a0 e9 dd b3 :6 a0 e9 d3
5 :2 a0 f :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c :2 a0
6b a0 7e a0 6b b4 2e 5a
a0 2b b7 19 3c :2 a0 :2 51 a5
b 7e 6e b4 2e :5 a0 12a :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f b7
:5 a0 12a :2 a0 6b a0 d b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f b7 :2 19 3c :c a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f :f a0 12a a0
f a0 7e b4 2e :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c b7 :3 a0 6b a0 d b7 a6
9 a0 53 :2 a0 6b a0 d :2 a0
6b :2 a0 6b 51 6e 51 7e 51
b4 2e a5 b a 10 :2 a0 6b
:2 6e 7e a0 b4 2e 7e 6e b4
2e 7e 6e b4 2e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c b7 a4 b1 11 4f b7 a6
9 a4 b1 11 4f b7 a0 47
:2 a0 e9 c1 :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f a0 12a b7 :2 19 3c
b7 a0 47 :2 a0 e9 c1 :2 a0 6b
a0 d b7 :3 a0 6b a0 d b7
a6 9 a0 53 :2 a0 6b a0 d
:2 a0 6b :2 a0 6b 51 6e 51 7e
51 b4 2e a5 b a 10 :2 a0
6b :2 6e 7e a0 b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e 7e :2 a0
:2 51 a5 b b4 2e a5 57 b7
19 3c b7 a4 b1 11 4f b7
a6 9 a4 b1 11 4f :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e :4 a0 6b a5
b 6e a5 b b4 2e 7e 6e
b4 2e 7e 6e b4 2e 7e a0
b4 2e 7e 6e b4 2e a5 57
b7 19 3c :2 a0 6b :2 a0 6b :2 a0
a5 57 b7 :2 a0 6b :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e a5 57 :2 a0 6b :2 6e
7e :2 a0 6b b4 2e 7e 6e b4
2e 7e :2 a0 6b b4 2e a5 57
b7 19 3c :9 a0 12a e9 dd :2 a0
65 b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
a5 57 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e :2 a0 :2 51
a5 b b4 2e a5 57 b7 19
3c :7 a0 12a e9 dd :2 a0 65 b7
a4 b1 11 4f b7 a6 9 a4
b1 11 68 4f 1d 17 b5 
a5d
2
0 3 7 23 1f 1e 2b 38
34 1b 40 33 45 49 4d 30
51 55 72 5d 61 64 65 6d
5c 8f 7d 59 81 82 8a 7c
ac 9a 79 9e 9f a7 99 c9
b7 96 bb bc c4 b6 e6 d4
b3 d8 d9 e1 d3 103 f1 d0
f5 f6 fe f0 120 10e ed 112
113 11b 10d 13d 12b 10a 12f 130
138 12a 159 148 14c 127 154 147
175 164 168 144 170 163 17c 180
160 191 194 198 19c 1a0 1a4 1a8
1ac 1b0 1b4 1b8 1bc 1c0 1c4 1c8
1cc 1d0 1d4 1e0 1e5 1e7 1f3 1f7
1f9 1fd 20e 20f 212 216 21a 21e
222 226 22a 22e 232 236 23a 246
24b 24d 259 25d 25f 263 274 275
278 27c 280 284 288 28c 290 294
298 29c 2a0 2a4 2a8 2ac 2b0 2b4
2b8 2bc 2c0 2c4 2c8 2cc 2d0 2dc
2e1 2e3 2ef 2f3 311 2f9 2fd 301
304 30c 2f8 318 31c 2f5 320 324
328 32b 330 331 1 333 338 33c
340 343 348 34c 350 353 358 359
35b 360 363 367 368 36d 372 375
37a 37b 380 383 387 388 38d 390
395 396 39b 39e 3a3 3a4 3a9 3ac
3b0 3b4 3b8 3b9 3bb 3c0 3c1 3c3
3c4 3c9 3cc 3d1 3d2 3d7 3d8 3dd
3df 3e3 3e6 3ea 3ee 3f1 3f5 3f9
3fc 3ff 404 407 40a 40d 40e 413
414 1 416 41b 41f 423 426 42b
430 433 437 438 43d 43e 443 445
449 44c 450 454 458 45c 460 464
468 46c 471 475 476 47a 47e 482
487 48c 490 494 499 49d 4a1 4a4
4a7 4ab 4ad 4b1 4b5 4b8 4bb 4bf
4c1 4c5 4c9 4cc 4d0 4d4 4d7 4db
4de 4e2 4e5 4e6 4eb 4ee 4f2 4f8
4fa 4fe 501 505 509 50c 50d 50f
512 515 516 51b 51f 523 528 52c
52d 531 535 539 53d 541 545 54a
54f 553 557 55b 560 564 568 56b
56e 572 574 578 57c 57f 582 586
588 58c 590 593 597 59b 59e 5a2
5a5 5a9 5ac 5ad 5b2 5b5 5b9 5bf
5c1 5c5 5c8 5cc 5d0 5d4 5d8 5dc
5e0 5e4 5e8 5f4 5f8 5fc 5ff 603
607 609 60d 611 615 618 61c 620
622 623 628 1 62c 630 634 637
63b 63f 643 647 64a 64e 652 655
658 65d 660 663 666 667 66c 66d
1 66f 674 678 67c 67f 684 689
68c 690 691 696 699 69e 69f 6a4
6a7 6ac 6ad 6b2 6b5 6b9 6ba 6bf
6c2 6c7 6c8 6cd 6d0 6d4 6d8 6db
6de 6df 6e1 6e2 6e7 6e8 6ed 6ef
6f3 6f6 6f8 6fc 6fe 70a 70c 70e
70f 714 718 71a 726 728 72c 730
733 734 736 739 73c 73d 742 746
74a 74e 752 75e 762 767 76b 76e
76f 774 778 77c 77f 782 786 788
78c 790 793 796 79a 79c 7a0 7a4
7a7 7a9 7ad 7b1 7b5 7b8 7bc 7c0
7c2 7c3 7c8 1 7cc 7d0 7d4 7d7
7db 7df 7e3 7e7 7ea 7ee 7f2 7f5
7f8 7fd 800 803 806 807 80c 80d
1 80f 814 818 81c 81f 824 829
82c 830 831 836 839 83e 83f 844
847 84c 84d 852 855 859 85a 85f
862 867 868 86d 870 874 878 87b
87e 87f 881 882 887 888 88d 88f
893 896 898 89c 89e 8aa 8ac 8ae
8af 8b4 8b8 8ba 8c6 8c8 8cc 8d0
8d4 8d8 8dc 8e0 8e4 8f0 8f4 8f9
8fd 900 901 906 90a 90e 911 914
918 91a 91e 922 925 928 92c 92e
932 936 939 93b 93f 943 947 94a
94e 952 954 955 95a 1 95e 962
966 969 96d 971 975 979 97c 980
984 987 98a 98f 992 995 998 999
99e 99f 1 9a1 9a6 9aa 9ae 9b1
9b6 9bb 9be 9c2 9c3 9c8 9cb 9d0
9d1 9d6 9d9 9de 9df 9e4 9e7 9eb
9ec 9f1 9f4 9f9 9fa 9ff a02 a06
a0a a0d a10 a11 a13 a14 a19 a1a
a1f a21 a25 a28 a2a a2e a30 a3c
a3e a40 a41 a46 a4a a4c a58 a5a
a5c a60 a63 a67 a6b a6f a73 a7f
a83 a88 a8c a8f a90 a95 a99 a9d
aa0 aa3 aa7 aa9 aad ab1 ab4 ab7
abb abd ac1 ac5 ac8 aca ace ad2
ad6 ad9 add ae1 ae3 ae4 ae9 1
aed af1 af5 af8 afc b00 b04 b08
b0b b0f b13 b16 b19 b1e b21 b24
b27 b28 b2d b2e 1 b30 b35 b39
b3d b40 b45 b4a b4d b51 b52 b57
b5a b5f b60 b65 b68 b6d b6e b73
b76 b7a b7b b80 b83 b88 b89 b8e
b91 b95 b99 b9c b9f ba0 ba2 ba3
ba8 ba9 bae bb0 bb4 bb7 bb9 bbd
bbf bcb bcd bcf bd0 bd5 bd9 bdb
be7 be9 bed bf1 bf4 bf8 bfb bff
c02 c03 c08 c0b c0f c13 c17 c1b
c1f c2b c2f c34 c38 c3b c3c c41
c45 c49 c4c c4f c53 c55 c59 c5d
c60 c63 c67 c69 c6d c71 c74 c76
c7a c7e c82 c85 c89 c8d c8f c90
c95 1 c99 c9d ca1 ca4 ca8 cac
cb0 cb4 cb7 cbb cbf cc2 cc5 cca
ccd cd0 cd3 cd4 cd9 cda 1 cdc
ce1 ce5 ce9 cec cf1 cf6 cf9 cfd
cfe d03 d06 d0b d0c d11 d14 d19
d1a d1f d22 d26 d27 d2c d2f d34
d35 d3a d3d d41 d45 d48 d4b d4c
d4e d4f d54 d55 d5a d5c d60 d63
d65 d69 d6b d77 d79 d7b d7c d81
d85 d87 d93 d95 d99 d9d da0 da3
da4 da6 da9 dae daf db4 db8 dbc
dc0 dc4 dc8 dd4 dd8 ddc ddf de3
de7 de9 ded df1 df5 df8 dfc e00
e02 e03 e08 1 e0c e10 e14 e17
e1b e1f e23 e27 e2a e2e e32 e35
e38 e3d e40 e43 e46 e47 e4c e4d
1 e4f e54 e58 e5c e5f e64 e69
e6c e70 e71 e76 e79 e7e e7f e84
e87 e8c e8d e92 e95 e99 e9a e9f
ea2 ea7 ea8 ead eb0 eb4 eb8 ebb
ebe ebf ec1 ec2 ec7 ec8 ecd ecf
ed3 ed6 ed8 edc ede eea eec eee
eef ef4 ef8 efa f06 f08 f0a f0e
f12 f16 f1a f1e f2a f2e f32 f35
f39 f3d f3f f43 f47 f4b f4e f52
f56 f58 f59 f5e 1 f62 f66 f6a
f6d f71 f75 f79 f7d f80 f84 f88
f8b f8e f93 f96 f99 f9c f9d fa2
fa3 1 fa5 faa fae fb2 fb5 fba
fbf fc2 fc6 fc7 fcc fcf fd4 fd5
fda fdd fe2 fe3 fe8 feb fef ff0
ff5 ff8 ffd ffe 1003 1006 100a 100e
1011 1014 1015 1017 1018 101d 101e 1023
1025 1029 102c 102e 1032 1034 1040 1042
1044 1045 104a 104e 1050 105c 105e 1060
1064 1068 106b 106f 1073 1077 107b 107f
1083 1087 108b 108f 109b 109f 10a4 10a8
10ab 10ac 10b1 10b5 10b9 10bc 10bf 10c3
10c5 10c9 10cd 10d0 10d3 10d7 10d9 10dd
10e1 10e4 10e6 10ea 10ee 10f2 10f5 10f9
10fd 10ff 1100 1105 1 1109 110d 1111
1114 1118 111c 1120 1124 1127 112b 112f
1132 1135 113a 113d 1140 1143 1144 1149
114a 1 114c 1151 1155 1159 115c 1161
1166 1169 116d 116e 1173 1176 117b 117c
1181 1184 1189 118a 118f 1192 1196 1197
119c 119f 11a4 11a5 11aa 11ad 11b1 11b5
11b8 11bb 11bc 11be 11bf 11c4 11c5 11ca
11cc 11d0 11d3 11d5 11d9 11db 11e7 11e9
11eb 11ec 11f1 11f5 11f7 1203 1205 1207
120b 120f 1213 1217 121b 121f 1223 122f
1233 1238 123c 123f 1240 1245 1249 124d
1250 1253 1257 1259 125d 1261 1264 1267
126b 126d 1271 1275 1278 127a 127e 1282
1286 1289 128d 1291 1293 1294 1299 1
129d 12a1 12a5 12a8 12ac 12b0 12b4 12b8
12bb 12bf 12c3 12c6 12c9 12ce 12d1 12d4
12d7 12d8 12dd 12de 1 12e0 12e5 12e9
12ed 12f0 12f5 12fa 12fd 1301 1302 1307
130a 130f 1310 1315 1318 131d 131e 1323
1326 132a 132b 1330 1333 1338 1339 133e
1341 1345 1349 134c 134f 1350 1352 1353
1358 1359 135e 1360 1364 1367 1369 136d
136f 137b 137d 137f 1380 1385 1389 138b
1397 1399 139b 139f 13a3 13a6 13aa 13ae
13b2 13be 13c2 13c7 13cb 13ce 13cf 13d4
13d8 13dc 13df 13e2 13e6 13e8 13ec 13f0
13f3 13f6 13fa 13fc 1400 1404 1407 1409
140d 1411 1415 1418 141c 1420 1422 1423
1428 1 142c 1430 1434 1437 143b 143f
1443 1447 144a 144e 1452 1455 1458 145d
1460 1463 1466 1467 146c 146d 1 146f
1474 1478 147c 147f 1484 1489 148c 1490
1491 1496 1499 149e 149f 14a4 14a7 14ac
14ad 14b2 14b5 14b9 14ba 14bf 14c2 14c7
14c8 14cd 14d0 14d4 14d8 14db 14de 14df
14e1 14e2 14e7 14e8 14ed 14ef 14f3 14f6
14f8 14fc 14fe 150a 150c 150e 150f 1514
1518 151a 1526 1528 152c 1530 1534 1538
153c 1548 154c 1551 1555 1558 1559 155e
1562 1566 1569 156c 1570 1572 1576 157a
157d 1580 1584 1586 158a 158e 1591 1593
1597 159b 159f 15a2 15a6 15aa 15ac 15ad
15b2 1 15b6 15ba 15be 15c1 15c5 15c9
15cd 15d1 15d4 15d8 15dc 15df 15e2 15e7
15ea 15ed 15f0 15f1 15f6 15f7 1 15f9
15fe 1602 1606 1609 160e 1613 1616 161a
161b 1620 1623 1628 1629 162e 1631 1636
1637 163c 163f 1643 1644 1649 164c 1651
1652 1657 165a 165e 1662 1665 1668 1669
166b 166c 1671 1672 1677 1679 167d 1680
1682 1686 1688 1694 1696 1698 1699 169e
16a2 16a4 16b0 16b2 16b4 16b8 16bf 16c3
16c7 16cc 16ce 16d2 16d6 16d9 16dd 16e1
16e3 16e7 16eb 16ef 16f2 16f6 16fa 16fc
16fd 1702 1 1706 170a 170e 1711 1715
1719 171d 1721 1724 1728 172c 172f 1732
1737 173a 173d 1740 1741 1746 1747 1
1749 174e 1752 1756 1759 175e 1763 1766
176a 176b 1770 1773 1778 1779 177e 1781
1786 1787 178c 178f 1793 1794 1799 179c
17a1 17a2 17a7 17aa 17ae 17b2 17b5 17b8
17b9 17bb 17bc 17c1 17c2 17c7 17c9 17cd
17d0 17d2 17d6 17d8 17e4 17e6 17e8 17e9
17ee 17f2 17f4 1800 1802 1806 1812 1816
1818 181c 1820 1823 1824 1826 1829 182c
182d 1832 1836 183a 183f 1843 1844 1848
184c 1850 1854 1858 185c 1861 1866 186a
186e 1872 1877 187b 187f 1882 1885 1889
188b 188f 1893 1896 1899 189d 189f 18a3
18a7 18aa 18ae 18b2 18b5 18b9 18bc 18c0
18c3 18c4 18c9 18cc 18d0 18d6 18d8 18dc
18df 18e3 18e7 18ea 18ed 18ee 18f0 18f3
18f8 18f9 18fe 1902 1906 190a 190e 1912
191e 1922 1926 1929 192d 1931 1933 1937
193b 193f 1942 1946 194a 194c 194d 1952
1 1956 195a 195e 1961 1965 1969 196d
1971 1974 1978 197c 197f 1982 1987 198a
198d 1990 1991 1996 1997 1 1999 199e
19a2 19a6 19a9 19ae 19b3 19b6 19ba 19bb
19c0 19c3 19c8 19c9 19ce 19d1 19d6 19d7
19dc 19df 19e3 19e4 19e9 19ec 19f1 19f2
19f7 19fa 19fe 1a02 1a05 1a08 1a09 1a0b
1a0c 1a11 1a12 1a17 1a19 1a1d 1a20 1a22
1a26 1a28 1a34 1a36 1a38 1a39 1a3e 1a42
1a44 1a50 1a52 1a54 1a58 1a5c 1a60 1a64
1a68 1a74 1a78 1a7c 1a7f 1a83 1a87 1a89
1a8d 1a91 1a95 1a98 1a9c 1aa0 1aa2 1aa3
1aa8 1 1aac 1ab0 1ab4 1ab7 1abb 1abf
1ac3 1ac7 1aca 1ace 1ad2 1ad5 1ad8 1add
1ae0 1ae3 1ae6 1ae7 1aec 1aed 1 1aef
1af4 1af8 1afc 1aff 1b04 1b09 1b0c 1b10
1b11 1b16 1b19 1b1e 1b1f 1b24 1b27 1b2c
1b2d 1b32 1b35 1b39 1b3a 1b3f 1b42 1b47
1b48 1b4d 1b50 1b54 1b58 1b5b 1b5e 1b5f
1b61 1b62 1b67 1b68 1b6d 1b6f 1b73 1b76
1b78 1b7c 1b7e 1b8a 1b8c 1b8e 1b8f 1b94
1b98 1b9a 1ba6 1ba8 1baa 1bae 1bb2 1bb5
1bb9 1bbd 1bc1 1bc5 1bc9 1bcd 1bd1 1bd5
1bd9 1bdd 1be1 1be5 1bf1 1bf5 1bfa 1bfe
1c01 1c02 1c07 1c0b 1c0f 1c12 1c15 1c19
1c1b 1c1f 1c23 1c26 1c29 1c2d 1c2f 1c33
1c37 1c3a 1c3c 1c40 1c44 1c48 1c4b 1c4f
1c53 1c55 1c56 1c5b 1 1c5f 1c63 1c67
1c6a 1c6e 1c72 1c76 1c7a 1c7d 1c81 1c85
1c88 1c8b 1c90 1c93 1c96 1c99 1c9a 1c9f
1ca0 1 1ca2 1ca7 1cab 1caf 1cb2 1cb7
1cbc 1cbf 1cc3 1cc4 1cc9 1ccc 1cd1 1cd2
1cd7 1cda 1cdf 1ce0 1ce5 1ce8 1cec 1ced
1cf2 1cf5 1cfa 1cfb 1d00 1d03 1d07 1d0b
1d0e 1d11 1d12 1d14 1d15 1d1a 1d1b 1d20
1d22 1d26 1d29 1d2b 1d2f 1d31 1d3d 1d3f
1d41 1d42 1d47 1d4b 1d4d 1d59 1d5b 1d5f
1d63 1d67 1d6b 1d6f 1d73 1d77 1d7b 1d7f
1d83 1d87 1d8b 1d8f 1d93 1d97 1da3 1da7
1dac 1db0 1db3 1db4 1db9 1dbd 1dc1 1dc4
1dc7 1dcb 1dcd 1dd1 1dd5 1dd8 1ddb 1ddf
1de1 1de5 1de9 1dec 1dee 1df2 1df6 1dfa
1dfd 1e01 1e05 1e07 1e08 1e0d 1 1e11
1e15 1e19 1e1c 1e20 1e24 1e28 1e2c 1e2f
1e33 1e37 1e3a 1e3d 1e42 1e45 1e48 1e4b
1e4c 1e51 1e52 1 1e54 1e59 1e5d 1e61
1e64 1e69 1e6e 1e71 1e75 1e76 1e7b 1e7e
1e83 1e84 1e89 1e8c 1e91 1e92 1e97 1e9a
1e9e 1e9f 1ea4 1ea7 1eac 1ead 1eb2 1eb5
1eb9 1ebd 1ec0 1ec3 1ec4 1ec6 1ec7 1ecc
1ecd 1ed2 1ed4 1ed8 1edb 1edd 1ee1 1ee3
1eef 1ef1 1ef3 1ef4 1ef9 1efd 1eff 1f0b
1f0d 1f0f 1f13 1f1a 1f1e 1f22 1f27 1f29
1f2d 1f31 1f34 1f38 1f3c 1f3e 1f42 1f46
1f4a 1f4d 1f51 1f55 1f57 1f58 1f5d 1
1f61 1f65 1f69 1f6c 1f70 1f74 1f78 1f7c
1f7f 1f83 1f87 1f8a 1f8d 1f92 1f95 1f98
1f9b 1f9c 1fa1 1fa2 1 1fa4 1fa9 1fad
1fb1 1fb4 1fb9 1fbe 1fc1 1fc5 1fc6 1fcb
1fce 1fd3 1fd4 1fd9 1fdc 1fe1 1fe2 1fe7
1fea 1fee 1fef 1ff4 1ff7 1ffc 1ffd 2002
2005 2009 200d 2010 2013 2014 2016 2017
201c 201d 2022 2024 2028 202b 202d 2031
2033 203f 2041 2043 2044 2049 204d 204f
205b 205d 2061 206d 206f 2073 2077 207a
207c 2080 2087 208b 208f 2094 2096 209a
209e 20a1 20a5 20a9 20ab 20af 20b3 20b7
20ba 20be 20c2 20c4 20c5 20ca 1 20ce
20d2 20d6 20d9 20dd 20e1 20e5 20e9 20ec
20f0 20f4 20f7 20fa 20ff 2102 2105 2108
2109 210e 210f 1 2111 2116 211a 211e
2121 2126 212b 212e 2132 2133 2138 213b
2140 2141 2146 2149 214e 214f 2154 2157
215b 215c 2161 2164 2169 216a 216f 2172
2176 217a 217d 2180 2181 2183 2184 2189
218a 218f 2191 2195 2198 219a 219e 21a0
21ac 21ae 21b0 21b1 21b6 21ba 21bc 21c8
21ca 21ce 21d2 21d5 21d9 21dd 21e0 21e3
21e8 21eb 21ee 21f1 21f2 21f7 21f8 1
21fa 21ff 2203 2207 220a 220f 2214 2217
221b 221c 2221 2224 2229 222a 222f 2232
2237 2238 223d 2240 2244 2248 224c 2250
2253 2254 2256 225b 225c 225e 225f 2264
2267 226c 226d 2272 2275 227a 227b 2280
2283 2287 2288 228d 2290 2295 2296 229b
229c 22a1 22a3 22a7 22aa 22ae 22b2 22b5
22b9 22bd 22c0 22c4 22c8 22c9 22ce 22d0
22d4 22d8 22db 22df 22e3 22e6 22ea 22ee
22f1 22f4 22f9 22fc 22ff 2302 2303 2308
2309 1 230b 2310 2314 2318 231b 2320
2325 2328 232c 232d 2332 2333 2338 233c
2340 2343 2348 234d 2350 2354 2358 235b
235c 2361 2364 2369 236a 236f 2372 2376
237a 237d 237e 2383 2384 2389 238b 238f
2392 2396 239a 239e 23a2 23a6 23aa 23ae
23b2 23b6 23c2 23c7 23cb 23cf 23d3 23d7
23d9 23da 23df 1 23e3 23e7 23eb 23ee
23f2 23f6 23fa 23fe 2401 2405 2409 240c
240f 2414 2417 241a 241d 241e 2423 2424
1 2426 242b 242f 2433 2436 243b 2440
2443 2447 2448 244d 244e 2453 2457 245b
245e 2463 2468 246b 246f 2470 2475 2478
247d 247e 2483 2486 248a 248e 2491 2494
2495 2497 2498 249d 249e 24a3 24a5 24a9
24ac 24b0 24b4 24b8 24bc 24c0 24c4 24c8
24d4 24d9 24dd 24e1 24e5 24e9 24eb 24ef
24f1 24fd 24ff 2501 2502 2507 250b 250d
2519 251d 251f 2520 2529 
a5d
2
0 1 a 6 e :3 6 11 :2 6
1 3 a 13 a :2 1 4 b
14 13 :2 b :2 4 10 19 18 :2 10
:2 4 11 1a 19 :2 11 :2 4 18 21
20 :2 18 :2 4 18 21 20 :2 18 :2 4
12 1b 1a :2 12 :2 4 d 16 15
:2 d :2 4 f 18 17 :2 f :2 4 :2 12
21 12 :2 4 :2 e 1d e 4 7
e 0 :2 7 11 21 10 1c 25
30 39 45 4e 5c 65 73 7c
8f 99 a :6 7 e 0 :2 7 11
24 37 44 f 10 1d 2e 38
a :6 7 e 0 :2 7 11 14 21
24 33 36 43 46 f 24 10
1d 1a 2e 37 3a 47 58 5b
69 6c a :5 7 4 e 17 :3 e
4 7 :2 11 23 :2 2d 3a :2 23 :3 7
:2 d 19 25 :2 2f 43 :2 25 56 5c
5f :2 56 6d 85 8 :2 6d d 10
:2 6d 18 1b :2 6d b e :2 6d 13
16 1f 27 :2 1f 34 :2 16 :2 6d 3c
3f :2 6d :2 7 4c :2 4 7 :2 11 23
:2 2d 39 3c 4e 50 51 :2 4e :2 23
:3 7 :2 d 12 1e 5f 62 :2 1e :2 7
54 :3 4 e :2 4 11 :2 4 9 :4 4
d 19 :2 7 a 11 :2 a :2 13 1f
a 17 a :2 13 1f a :4 7 b
:2 14 20 1d :2 29 :2 1d :3 a 39 :2 7
a 13 1e :2 a 21 23 :2 21 a
f :4 a 13 1f 34 49 53 :3 d
10 17 :2 10 :2 19 25 10 1d 10
:2 19 25 10 :4 d 11 :2 1a 26 23
:2 2f :2 23 :3 10 3f :2 d 17 27 36
59 16 2e 46 54 :2 10 :2 19 25
10 d 15 13 :2 1c 28 13 23
:2 10 :2 15 13 :2 1c 28 13 16 :2 20
32 :2 3c 48 4c 5e 60 61 :2 5e
:2 32 :3 16 :2 1c 21 2d 6e 71 :2 2d
7e 81 :2 2d 85 88 :2 2d 94 97
:2 2d 9f a2 :2 2d ae b1 b8 c1
c4 :2 b1 :2 2d :2 16 64 :2 13 :2 10 :4 1c
:2 10 d :3 a 10 19 24 :2 10 27
2a :2 27 1a 2f 19 31 13 1a
16 22 :3 20 16 :2 1f 2b 16 27
16 :2 1f 2b 16 :4 13 10 18 16
:2 1f 2b 16 26 :2 13 :2 18 16 :2 1f
2b 16 19 :2 23 35 :2 3f 4b 4f
61 64 65 :2 61 :2 35 :3 19 :2 1f 24
30 72 75 :2 30 82 85 :2 30 89
8c :2 30 98 9b :2 30 a3 a6 :2 30
b2 b5 bc c5 c8 :2 b5 :2 30 :2 19
68 :2 16 :2 13 :4 1f :2 13 10 :3 2c 1a
2f 3b 19 26 37 43 13 1a
16 22 :3 20 16 :2 1f 2b 16 27
16 :2 1f 2b 16 :4 13 10 18 16
:2 1f 2b 16 26 :2 13 :2 18 16 :2 1f
2b 16 19 :2 23 35 :2 3f 4b 4f
61 64 65 :2 61 :2 35 :3 19 :2 1f 24
30 72 75 :2 30 82 85 :2 30 89
8c :2 30 98 9b :2 30 a3 a6 :2 30
b2 b5 bc c5 c8 :2 b5 :2 30 :2 19
68 :2 16 :2 13 :4 1f :2 13 10 :4 2c :2 d
1c 16 2a 14 10 17 13 1f
:3 1d 13 :2 1c 28 13 24 13 :2 1c
28 13 :4 10 d 15 13 :2 1c 28
13 23 :2 10 :2 15 13 :2 1c 28 13
16 :2 20 32 :2 3c 48 4c 5e 61
62 :2 5e :2 32 :3 16 :2 1c 21 2d 6f
72 :2 2d 7f 82 :2 2d 86 89 :2 2d
95 98 :2 2d a0 a3 :2 2d af b2
b9 c2 c5 :2 b2 :2 2d :2 16 65 :2 13
:2 10 :4 1c :2 10 d :3 a 11 :2 1a 25
23 :2 2e :2 23 10 1f 39 46 57
65 13 1a 16 22 :3 20 16 :2 1f
2b 16 27 16 :2 1f 2b 16 :4 13
10 18 16 :2 1f 2b 16 26 :2 13
:2 18 16 :2 1f 2b 16 19 :2 23 35
:2 3f 4b 4f 61 64 65 :2 61 :2 35
:3 19 :2 1f 24 30 72 75 :2 30 82
85 :2 30 89 8c :2 30 98 9b :2 30
a3 a6 :2 30 b2 b5 bc c5 c8
:2 b5 :2 30 :2 19 68 :2 16 :2 13 :4 1f :2 13
10 :3 3e 13 1a 26 29 :2 13 2c
2e :2 2c 1d 33 1b 1c 2a :2 16
:2 1f 2b 16 13 1b 19 :2 22 2e
19 29 :2 16 :2 1b 19 :2 22 2e 19
1c :2 26 38 :2 42 4e 52 64 67
68 :2 64 :2 38 :3 1c :2 22 27 33 75
78 :2 33 85 88 :2 33 8c 8f :2 33
9b 9e :2 33 a6 a9 :2 33 b5 b8
bf c8 cb :2 b8 :2 33 :2 1c 6b :2 19
:2 16 :4 22 :2 16 13 :4 32 1d 36 1b
1c 29 :2 16 :2 1f 2b 16 13 1b
19 :2 22 2e 19 29 :2 16 :2 1b 19
:2 22 2e 19 1c :2 26 38 :2 42 4e
52 64 67 68 :2 64 :2 38 :3 1c :2 22
27 33 75 78 :2 33 85 88 :2 33
8c 8f :2 33 9b 9e :2 33 a6 a9
:2 33 b5 b8 bf c8 cb :2 b8 :2 33
:2 1c 6b :2 19 :2 16 :4 22 :2 16 13 :7 10
1a 26 35 3e 4c 19 26 37
44 13 1a 16 22 :3 20 16 :2 1f
2b 16 27 16 :2 1f 2b 16 :4 13
10 18 16 :2 1f 2b 16 26 :2 13
:2 18 16 :2 1f 2b 16 19 :2 23 35
:2 3f 4b 4f 61 64 65 :2 61 :2 35
:3 19 :2 1f 24 30 72 75 :2 30 82
85 :2 30 89 8c :2 30 98 9b :2 30
a3 a6 :2 30 b2 b5 bc c5 c8
:2 b5 :2 30 :2 19 68 :2 16 :2 13 :4 1f :2 13
10 :4 3e 1a 33 41 4a 55 19
2d 13 1a 16 22 :3 20 16 :2 1f
2b 16 27 16 :2 1f 2b 16 :4 13
10 18 16 :2 1f 2b 16 26 :2 13
:2 18 16 :2 1f 2b 16 19 :2 23 35
:2 3f 4b 4f 61 64 65 :2 61 :2 35
:3 19 :2 1f 24 30 72 75 :2 30 82
85 :2 30 89 8c :2 30 98 9b :2 30
a3 a6 :2 30 b2 b5 bc c5 c8
:2 b5 :2 30 :2 19 68 :2 16 :2 13 :4 1f :2 13
10 :7 d 1c 33 47 10 17 13
1f :3 1d 13 :2 1c 28 13 24 13
:2 1c 28 13 :4 10 d 15 13 :2 1c
28 13 23 :2 10 :2 15 13 :2 1c 28
13 16 :2 20 32 :2 3c 48 4c 5e
61 62 :2 5e :2 32 :3 16 :2 1c 21 2d
6f 72 :2 2d 7f 82 :2 2d 86 89
:2 2d 95 98 :2 2d a0 a3 :2 2d af
b2 b9 c2 c5 :2 b2 :2 2d :2 16 65
:2 13 :2 10 :4 1c :2 10 d :3 a 1c 33
40 51 65 10 17 13 1f :3 1d
13 :2 1c 28 13 24 13 :2 1c 28
13 :4 10 d 15 13 :2 1c 28 13
23 :2 10 :2 15 13 :2 1c 28 13 16
:2 20 32 :2 3c 48 4c 5e 61 62
:2 5e :2 32 :3 16 :2 1c 21 2d 6f 72
:2 2d 7f 82 :2 2d 86 89 :2 2d 95
98 :2 2d a0 a3 :2 2d af b2 b9
c2 c5 :2 b2 :2 2d :2 16 65 :2 13 :2 10
:4 1c :2 10 d :4 a e 25 d 13
:3 d :2 16 22 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 5b
5e 5f :2 5b :2 2f :3 13 :2 19 1e 2a
6c 6f :2 2a 7c 7f :2 2a 83 86
:2 2a 92 95 :2 2a 9d a0 :2 2a ac
af b6 bf c2 :2 af :2 2a :2 13 62
:2 10 :2 d :4 19 :2 d a :3 25 :2 a 7
25 d 16 21 :2 d 24 26 :2 24
a f :4 a 13 1f 29 35 44
:3 d 10 17 :2 10 :2 19 25 10 1d
10 :2 19 25 10 :4 d 11 :2 1a 26
23 :2 2f :2 23 :3 10 3f :2 d 10 17
23 26 :2 10 29 2b :2 29 1a 30
18 19 27 :2 13 :2 1c 28 13 10
18 16 :2 1f 2b 16 26 :2 13 :2 18
16 :2 1f 2b 16 19 :2 23 35 :2 3f
4b 4f 61 64 65 :2 61 :2 35 :3 19
:2 1f 24 30 72 75 :2 30 82 85
:2 30 89 8c :2 30 98 9b :2 30 a3
a6 :2 30 b2 b5 bc c5 c8 :2 b5
:2 30 :2 19 68 :2 16 :2 13 :4 1f :2 13 10
:4 2f 1a 33 18 19 26 :2 13 :2 1c
28 13 10 18 16 :2 1f 2b 16
26 :2 13 :2 18 16 :2 1f 2b 16 19
:2 23 35 :2 3f 4b 4f 61 64 65
:2 61 :2 35 :3 19 :2 1f 24 30 72 75
:2 30 82 85 :2 30 89 8c :2 30 98
9b :2 30 a3 a6 :2 30 b2 b5 bc
c5 c8 :2 b5 :2 30 :2 19 68 :2 16 :2 13
:4 1f :2 13 10 :7 d 17 23 15 1b
29 3f 15 25 16 23 34 41
10 17 13 1f :3 1d 13 :2 1c 28
13 24 13 :2 1c 28 13 :4 10 d
15 13 :2 1c 28 13 23 :2 10 :2 15
13 :2 1c 28 13 16 :2 20 32 :2 3c
48 4c 5e 61 62 :2 5e :2 32 :3 16
:2 1c 21 2d 6f 72 :2 2d 7f 82
:2 2d 86 89 :2 2d 95 98 :2 2d a0
a3 :2 2d af b2 b9 c2 c5 :2 b2
:2 2d :2 16 65 :2 13 :2 10 :4 1c :2 10 d
:3 a 1c 31 3d 4a 57 64 73
85 17 25 2f 3e 4d 5c 65
10 17 13 1f :3 1d 13 :2 1c 28
13 24 13 :2 1c 28 13 :4 10 d
15 13 :2 1c 28 13 23 :2 10 :2 15
13 :2 1c 28 13 16 :2 20 32 :2 3c
48 4c 5e 61 62 :2 5e :2 32 :3 16
:2 1c 21 2d 6f 72 :2 2d 7f 82
:2 2d 86 89 :2 2d 95 98 :2 2d a0
a3 :2 2d af b2 b9 c2 c5 :2 b2
:2 2d :2 16 65 :2 13 :2 10 :4 1c :2 10 d
:4 a e 28 d 13 :3 d :2 16 22
d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 5b 5e 5f :2 5b
:2 2f :3 13 :2 19 1e 2a 6c 6f :2 2a
7c 7f :2 2a 83 86 :2 2a 92 95
:2 2a 9d a0 :2 2a ac af b6 bf
c2 :2 af :2 2a :2 13 62 :2 10 :2 d :4 19
:2 d a :3 28 :2 a 28 25 :2 7 4
8 1 7 d :3 7 :2 10 1c 7
4 c a :2 13 1f a 1a :2 7
:2 c a :2 13 1f a d :2 17 29
:2 33 3f 43 55 58 59 :2 55 :2 29
:3 d :2 13 18 24 66 69 :2 24 76
79 :2 24 7d 80 :2 24 8c 8f :2 24
97 9a :2 24 a6 a9 b0 b9 bc
:2 a9 :2 24 :2 d 5c :2 a :2 7 :4 13 :2 7
4 :3 1 7 :2 11 23 :2 2d 39 3c
4e 51 52 :2 4e :2 23 :3 7 :2 d 12
1e 60 63 :2 1e 70 73 :2 1e 77
7a :2 1e 84 87 90 98 :2 a1 :2 90
ac :2 87 :2 1e b4 b7 :2 1e bf c2
:2 1e c7 ca :2 1e d1 d4 :2 1e :2 7
55 :3 4 :2 f 21 :2 2a 34 3c :2 4
1 9 12 9 a :2 14 26 :2 30
3c 40 52 55 56 :2 52 :2 26 :3 a
:2 10 15 21 63 66 :2 21 :3 a :2 10
15 39 5c 5f :2 68 :2 39 76 79
:2 39 81 84 :2 8d :2 39 :2 a 59 :3 7
c 15 1e 2c a 13 1e f
e 7 e 7 e 7 29 :2 4
:2 9 7 :2 10 1c :2 7 :2 11 23 :2 2d
39 3d 4f 52 53 :2 4f :2 23 :3 7
:2 d 12 1e 60 63 :2 1e :3 7 :2 d
12 36 42 45 :2 36 4d 50 :2 36
5c 5f 66 6f 72 :2 5f :2 36 :2 7
56 :2 4 7 c 1b a 13 1e
f e 7 e 7 e 7 :2 4
:4 10 :2 4 :8 1 
a5d
4
0 :2 1 :4 3 :4 4
2 :4 5 :2 1 :7 7
:7 8 :7 9 :7 a :7 b
:7 c :7 d :7 e :6 f
:6 10 :2 12 0 :2 12
:2 13 :d 14 13 :5 12
:2 16 0 :2 16 :4 17
18 :4 19 17 :5 16
:2 1b 0 :2 1b :8 1c
:2 1d :2 1e :9 1f 1c
:5 1b :7 21 :b 23 :11 24
25 :2 24 :2 25 :2 24
:2 25 :2 24 :2 26 :2 24
:9 26 :2 24 :2 26 :4 24
:3 23 :11 29 :b 2a :3 29
:3 2d :3 2e :5 30 32
:4 33 :3 34 :5 35 34
:5 37 36 :3 34 :a 39
:2 3a :3 39 :9 3e :5 3f
41 :8 42 :3 43 :5 44
43 :5 46 45 :3 43
:a 48 :2 49 :3 48 :4 4d
:4 4e 4d :5 4f 4c
51 :5 52 :3 51 :2 53
:5 55 :11 56 :24 57 :3 56
:2 54 :6 53 50 :3 41
:9 5c :2 5e :2 5f 5e
:6 60 :5 61 60 :5 63
62 :3 60 5d 66
:5 67 :3 66 :2 68 :5 6a
:11 6b :24 6c :3 6b :2 69
:6 68 65 :3 5c :3 72
:4 73 72 :6 74 :5 75
74 :5 77 76 :3 74
71 7a :5 7b :3 7a
:2 7c :5 7e :11 7f :24 80
:3 7f :2 7d :6 7c 79
:6 5c 87 :2 88 89
87 :6 8a :5 8b 8a
:5 8d 8c :3 8a 86
90 :5 91 :3 90 :2 92
:5 94 :11 95 :24 96 :3 95
:2 93 :6 92 8f :3 41
:a 9b :6 9d :6 9e :5 9f
9e :5 a1 a0 :3 9e
9c a4 :5 a5 :3 a4
:2 a6 :5 a8 :11 a9 :24 aa
:3 a9 :2 a7 :6 a6 a3
:3 9b :a af :2 b1 b2
:2 b3 b1 :5 b4 b0
b6 :5 b7 :3 b6 :2 b8
:5 ba :11 bb :24 bc :3 bb
:2 b9 :6 b8 b5 :4 af
:2 c2 c3 :2 c4 c2
:5 c5 c1 c7 :5 c8
:3 c7 :2 c9 :5 cb :11 cc
:24 cd :3 cc :2 ca :6 c9
c6 :4 c0 :3 af :5 d4
:4 d5 d4 :6 d6 :5 d7
d6 :5 d9 d8 :3 d6
d3 dc :5 dd :3 dc
:2 de :5 e0 :11 e1 :24 e2
:3 e1 :2 df :6 de db
:4 9b :5 e8 :2 e9 e8
:6 ea :5 eb ea :5 ed
ec :3 ea e7 f0
:5 f1 :3 f0 :2 f2 :5 f4
:11 f5 :24 f6 :3 f5 :2 f3
:6 f2 ef :4 e6 :3 9b
:4 fd :6 fe :5 ff fe
:5 101 100 :3 fe fc
104 :5 105 :3 104 :2 106
:5 108 :11 109 :24 10a :3 109
:2 107 :6 106 103 :3 41
:6 110 :6 111 :5 112 111
:5 114 113 :3 111 10f
117 :5 118 :3 117 :2 119
:5 11b :11 11c :24 11d :3 11c
:2 11a :6 119 116 :4 41
121 3e :4 124 :5 125
123 127 :5 128 :3 127
:2 129 :5 12b :11 12c :24 12d
:3 12c :2 12a :6 129 126
:3 3e :2 132 133 3e
:9 133 :5 134 136 :8 137
:3 138 :5 139 138 :5 13b
13a :3 138 :a 13d :2 13e
:3 13d :a 141 :2 143 144
:2 145 143 :5 146 142
148 :5 149 :3 148 :2 14a
:5 14c :11 14d :24 14e :3 14d
:2 14b :6 14a 147 :4 141
:2 154 155 :2 156 154
:5 157 153 159 :5 15a
:3 159 :2 15b :5 15d :11 15e
:24 15f :3 15e :2 15c :6 15b
158 :4 152 :3 141 :2 166
:4 167 :2 168 :4 169 166
:6 16a :5 16b 16a :5 16d
16c :3 16a 165 170
:5 171 :3 170 :2 172 :5 174
:11 175 :24 176 :3 175 :2 173
:6 172 16f :3 136 :8 17c
:7 17d 17c :6 17e :5 17f
17e :5 181 180 :3 17e
17b 184 :5 185 :3 184
:2 186 :5 188 :11 189 :24 18a
:3 189 :2 187 :6 186 183
:4 136 18f 133 :4 192
:5 193 191 195 :5 196
:3 195 :2 197 :5 199 :11 19a
:24 19b :3 19a :2 198 :6 197
194 :3 133 :2 1a0 133
:3 3e 32 1a2 22
:4 1a5 :5 1a6 1a4 1a8
:5 1a9 :3 1a8 :2 1aa :5 1ac
:11 1ad :24 1ae :3 1ad :2 1ab
:6 1aa 1a7 :3 22 :11 1b3
:30 1b4 :3 1b3 :a 1b6 22
:3 1b9 :11 1ba :b 1bb :17 1bc
:3 1ba :2 1be :3 1bf :3 1c0
1c1 1bf 1be 1bf
:3 1c2 :3 1b9 :2 1c3 :5 1c5
:11 1c6 :b 1c7 :18 1c8 :3 1c6
:2 1ca 1cb :3 1cc 1cd
1cb 1ca 1cb :3 1ce
:2 1c4 :6 1c3 1b8 :7 1

252b
4
:3 0 1 :3 0 2
:a 0 a58 1 :7 0
5 30 0 3
4 :3 0 3 :7 0
5 4 :3 0 d
e 0 7 6
:3 0 5 :7 0 9
8 :3 0 7 :3 0
8 :3 0 9 :2 0
4 b f 0
a58 2 10 :2 0
d :2 0 c 4
:3 0 b :2 0 a
13 15 :6 0 18
16 0 a56 0
a :6 0 f :2 0
10 4 :3 0 e
1a 1c :6 0 1f
1d 0 a56 0
c :6 0 f :2 0
14 4 :3 0 12
21 23 :6 0 26
24 0 a56 0
e :6 0 f :2 0
18 4 :3 0 16
28 2a :6 0 2d
2b 0 a56 0
10 :6 0 f :2 0
1c 4 :3 0 1a
2f 31 :6 0 34
32 0 a56 0
11 :6 0 f :2 0
20 4 :3 0 1e
36 38 :6 0 3b
39 0 a56 0
12 :6 0 15 :2 0
24 4 :3 0 22
3d 3f :6 0 42
40 0 a56 0
13 :6 0 17 :2 0
28 4 :3 0 26
44 46 :6 0 49
47 0 a56 0
14 :6 0 17 :2 0
2a 6 :3 0 4b
:7 0 4f 4c 4d
a56 0 16 :9 0
2c 6 :3 0 51
:7 0 55 52 53
a56 0 18 :6 0
19 :3 0 1a :a 0
2 6b :4 0 57
5a 0 58 :3 0
1b :3 0 1c :3 0
1d :3 0 1e :3 0
1f :3 0 1e :3 0
20 :3 0 1e :3 0
21 :3 0 1e :3 0
22 :3 0 1e :3 0
23 :3 0 24 :3 0
a :4 0 25 1
:8 0 6c 57 5a
6d 0 a56 2e
6d 6f 6c 6e
:6 0 6b :7 0 6d
19 :3 0 26 :a 0
3 7f :5 0 71
74 0 72 :3 0
27 :3 0 28 :3 0
29 :3 0 2a :3 0
2b :3 0 2c :3 0
e :3 0 2d :3 0
e :4 0 2e 1
:8 0 80 71 74
81 0 a56 30
81 83 80 82
:6 0 7f :7 0 81
19 :3 0 2f :a 0
4 9f :5 0 85
88 0 86 :3 0
30 :3 0 29 :3 0
30 :3 0 2a :3 0
30 :3 0 31 :3 0
30 :3 0 32 :3 0
33 :3 0 30 :3 0
1b :3 0 e :3 0
34 :3 0 35 :3 0
35 :3 0 2c :3 0
e :3 0 35 :3 0
29 :3 0 30 :3 0
29 :4 0 36 1
:8 0 a0 85 88
a1 0 a56 32
a1 a3 a0 a2
:6 0 9f :7 0 a1
ab ac 0 34
8 :3 0 9 :2 0
4 a5 a6 0
a7 :7 0 aa a8
0 a56 0 37
:6 0 38 :3 0 39
:3 0 38 :3 0 3a
:3 0 ae af 0
2 :4 0 36 b0
b2 ad b4 b3
:2 0 3b :3 0 3c
:3 0 b6 b7 0
3d :4 0 38 :3 0
3e :3 0 ba bb
0 2 :4 0 38
bc be 3f :4 0
40 :2 0 41 :3 0
3a c1 c3 :3 0
42 :4 0 40 :2 0
43 :4 0 3d c6
c8 :3 0 40 :2 0
3 :3 0 40 ca
cc :3 0 40 :2 0
43 :4 0 43 ce
d0 :3 0 40 :2 0
44 :4 0 46 d2
d4 :3 0 40 :2 0
45 :3 0 46 :3 0
5 :3 0 49 d8
da 47 :4 0 4b
d7 dd 4e d6
df :3 0 40 :2 0
48 :4 0 51 e1
e3 :3 0 54 b8
e5 :2 0 e7 59
e8 b5 e7 0
e9 5b 0 9b6
38 :3 0 39 :3 0
ea eb 0 38
:3 0 49 :3 0 ed
ee 0 15 :2 0
2 :4 0 4a :2 0
4b :2 0 15 :2 0
5d f3 f5 :3 0
60 ef f7 ec
f9 f8 :2 0 3b
:3 0 4c :3 0 fb
fc 0 3d :4 0
4d :4 0 40 :2 0
41 :3 0 64 100
102 :3 0 67 fd
104 :2 0 106 6a
107 fa 106 0
108 6c 0 9b6
a :3 0 3 :3 0
109 10a 0 9b6
18 :3 0 5 :3 0
10c 10d 0 9b6
4e :3 0 1a :4 0
112 :2 0 9b6 110
113 :3 0 4f :3 0
1a :3 0 e :4 0
118 :2 0 907 115
116 :3 0 1a :3 0
50 :3 0 119 11a
:3 0 8 :3 0 51
:3 0 11c 11d 0
17 :2 0 11e 11f
0 121 6e 128
8 :3 0 51 :3 0
122 123 0 52
:2 0 124 125 0
127 70 129 11b
121 0 12a 0
127 0 12a 72
0 907 8 :3 0
51 :3 0 12b 12c
0 8 :3 0 53
:2 0 54 :3 0 12e
130 0 77 12f
132 :3 0 133 :2 0
55 :8 0 137 7a
138 134 137 0
139 7c 0 907
45 :3 0 18 :3 0
17 :2 0 7e 13a
13d 56 :2 0 17
:2 0 83 13f 141
:3 0 4e :3 0 26
:4 0 146 :2 0 6a7
144 147 :3 0 4f
:3 0 26 :3 0 10
:3 0 11 :3 0 13
:3 0 14 :4 0 14f
:2 0 643 149 150
:3 0 86 :3 0 26
:3 0 50 :3 0 151
152 :3 0 8 :3 0
51 :3 0 154 155
0 17 :2 0 156
157 0 159 8b
160 8 :3 0 51
:3 0 15a 15b 0
52 :2 0 15c 15d
0 15f 8d 161
153 159 0 162
0 15f 0 162
8f 0 643 8
:3 0 51 :3 0 163
164 0 8 :3 0
53 :2 0 54 :3 0
166 168 0 94
167 16a :3 0 16b
:2 0 55 :8 0 16f
97 170 16c 16f
0 171 99 0
643 57 :3 0 18
:3 0 58 :3 0 2b
:3 0 59 :3 0 11
:3 0 29 :3 0 13
:4 0 5a 1 :8 0
180 8 :3 0 51
:3 0 17b 17c 0
5b :3 0 17d 17e
0 180 9b 1d3
5c :3 0 8 :3 0
51 :3 0 182 183
0 5b :3 0 184
185 0 187 9e
189 a0 188 187
:2 0 1d1 5d :3 0
8 :3 0 51 :3 0
18c 18d 0 5b
:3 0 18e 18f 0
1c9 38 :3 0 39
:3 0 191 192 0
38 :3 0 49 :3 0
194 195 0 5e
:2 0 2 :4 0 5f
:2 0 4b :2 0 15
:2 0 a2 19a 19c
:3 0 a5 196 19e
193 1a0 19f :2 0
3b :3 0 4c :3 0
1a2 1a3 0 3d
:4 0 60 :4 0 40
:2 0 41 :3 0 a9
1a7 1a9 :3 0 40
:2 0 61 :4 0 ac
1ab 1ad :3 0 40
:2 0 62 :4 0 af
1af 1b1 :3 0 40
:2 0 5b :3 0 b2
1b3 1b5 :3 0 40
:2 0 63 :4 0 b5
1b7 1b9 :3 0 40
:2 0 64 :3 0 65
:3 0 15 :2 0 66
:2 0 b8 1bc 1c0
bc 1bb 1c2 :3 0
bf 1a4 1c4 :2 0
1c6 c2 1c7 1a1
1c6 0 1c8 c4
0 1c9 c6 1cc
:3 0 1cc 0 1cc
1cb 1c9 1ca :6 0
1ce 7 :3 0 c9
1d0 cb 1cf 1ce
:2 0 1d1 cd :2 0
1d3 0 1d3 1d2
180 1d1 :6 0 643
6 :3 0 45 :3 0
18 :3 0 17 :2 0
d0 1d5 1d8 53
:2 0 17 :2 0 d5
1da 1dc :3 0 2b
:3 0 59 :3 0 59
:3 0 11 :4 0 67
1 :8 0 1f8 50
:4 0 1e3 :3 0 68
:3 0 56 :2 0 da
1e6 1e7 :3 0 8
:3 0 51 :3 0 1e9
1ea 0 17 :2 0
1eb 1ec 0 1ee
dd 1f5 8 :3 0
51 :3 0 1ef 1f0
0 52 :2 0 1f1
1f2 0 1f4 df
1f6 1e8 1ee 0
1f7 0 1f4 0
1f7 e1 0 1f8
e4 24b 5c :3 0
8 :3 0 51 :3 0
1fa 1fb 0 5b
:3 0 1fc 1fd 0
1ff e7 201 e9
200 1ff :2 0 249
5d :3 0 8 :3 0
51 :3 0 204 205
0 5b :3 0 206
207 0 241 38
:3 0 39 :3 0 209
20a 0 38 :3 0
49 :3 0 20c 20d
0 5e :2 0 2
:4 0 69 :2 0 4b
:2 0 15 :2 0 eb
212 214 :3 0 ee
20e 216 20b 218
217 :2 0 3b :3 0
4c :3 0 21a 21b
0 3d :4 0 6a
:4 0 40 :2 0 41
:3 0 f2 21f 221
:3 0 40 :2 0 61
:4 0 f5 223 225
:3 0 40 :2 0 62
:4 0 f8 227 229
:3 0 40 :2 0 5b
:3 0 fb 22b 22d
:3 0 40 :2 0 63
:4 0 fe 22f 231
:3 0 40 :2 0 64
:3 0 65 :3 0 15
:2 0 66 :2 0 101
234 238 105 233
23a :3 0 108 21c
23c :2 0 23e 10b
23f 219 23e 0
240 10d 0 241
10f 244 :3 0 244
0 244 243 241
242 :6 0 246 9
:3 0 112 248 114
247 246 :2 0 249
116 :2 0 24b 0
24b 24a 1f8 249
:6 0 2bf 6 :3 0
6b :3 0 6c :3 0
28 :3 0 2c :3 0
e :3 0 6c :3 0
11 :4 0 6d 1
:8 0 26a 50 :4 0
255 :3 0 68 :3 0
56 :2 0 11b 258
259 :3 0 8 :3 0
51 :3 0 25b 25c
0 17 :2 0 25d
25e 0 260 11e
267 8 :3 0 51
:3 0 261 262 0
52 :2 0 263 264
0 266 120 268
25a 260 0 269
0 266 0 269
122 0 26a 125
2bd 5c :3 0 8
:3 0 51 :3 0 26c
26d 0 5b :3 0
26e 26f 0 271
128 273 12a 272
271 :2 0 2bb 5d
:3 0 8 :3 0 51
:3 0 276 277 0
5b :3 0 278 279
0 2b3 38 :3 0
39 :3 0 27b 27c
0 38 :3 0 49
:3 0 27e 27f 0
5e :2 0 2 :4 0
6e :2 0 4b :2 0
15 :2 0 12c 284
286 :3 0 12f 280
288 27d 28a 289
:2 0 3b :3 0 4c
:3 0 28c 28d 0
3d :4 0 6f :4 0
40 :2 0 41 :3 0
133 291 293 :3 0
40 :2 0 61 :4 0
136 295 297 :3 0
40 :2 0 62 :4 0
139 299 29b :3 0
40 :2 0 5b :3 0
13c 29d 29f :3 0
40 :2 0 63 :4 0
13f 2a1 2a3 :3 0
40 :2 0 64 :3 0
65 :3 0 15 :2 0
66 :2 0 142 2a6
2aa 146 2a5 2ac
:3 0 149 28e 2ae
:2 0 2b0 14c 2b1
28b 2b0 0 2b2
14e 0 2b3 150
2b6 :3 0 2b6 0
2b6 2b5 2b3 2b4
:6 0 2b8 b :3 0
153 2ba 155 2b9
2b8 :2 0 2bb 157
:2 0 2bd 0 2bd
2bc 26a 2bb :6 0
2bf 6 :3 0 15a
2c0 1dd 2bf 0
2c1 15d 0 643
33 :3 0 27 :3 0
10 :3 0 32 :4 0
70 1 :8 0 2dc
50 :4 0 2c7 :3 0
68 :3 0 56 :2 0
161 2ca 2cb :3 0
8 :3 0 51 :3 0
2cd 2ce 0 17
:2 0 2cf 2d0 0
2d2 164 2d9 8
:3 0 51 :3 0 2d3
2d4 0 52 :2 0
2d5 2d6 0 2d8
166 2da 2cc 2d2
0 2db 0 2d8
0 2db 168 0
2dc 16b 32f 5c
:3 0 8 :3 0 51
:3 0 2de 2df 0
5b :3 0 2e0 2e1
0 2e3 16e 2e5
170 2e4 2e3 :2 0
32d 5d :3 0 8
:3 0 51 :3 0 2e8
2e9 0 5b :3 0
2ea 2eb 0 325
38 :3 0 39 :3 0
2ed 2ee 0 38
:3 0 49 :3 0 2f0
2f1 0 5e :2 0
2 :4 0 71 :2 0
4b :2 0 15 :2 0
172 2f6 2f8 :3 0
175 2f2 2fa 2ef
2fc 2fb :2 0 3b
:3 0 4c :3 0 2fe
2ff 0 3d :4 0
72 :4 0 40 :2 0
41 :3 0 179 303
305 :3 0 40 :2 0
61 :4 0 17c 307
309 :3 0 40 :2 0
62 :4 0 17f 30b
30d :3 0 40 :2 0
5b :3 0 182 30f
311 :3 0 40 :2 0
63 :4 0 185 313
315 :3 0 40 :2 0
64 :3 0 65 :3 0
15 :2 0 66 :2 0
188 318 31c 18c
317 31e :3 0 18f
300 320 :2 0 322
192 323 2fd 322
0 324 194 0
325 196 328 :3 0
328 0 328 327
325 326 :6 0 32a
d :3 0 199 32c
19b 32b 32a :2 0
32d 19d :2 0 32f
0 32f 32e 2dc
32d :6 0 643 6
:3 0 8 :3 0 51
:3 0 331 332 0
8 :3 0 56 :2 0
54 :3 0 334 336
0 1a2 335 338
:3 0 339 :2 0 34
:3 0 2c :3 0 e
:3 0 29 :3 0 13
:4 0 73 1 :8 0
356 50 :4 0 341
:3 0 68 :3 0 56
:2 0 1a7 344 345
:3 0 8 :3 0 51
:3 0 347 348 0
17 :2 0 349 34a
0 34c 1aa 353
8 :3 0 51 :3 0
34d 34e 0 52
:2 0 34f 350 0
352 1ac 354 346
34c 0 355 0
352 0 355 1ae
0 356 1b1 3a9
5c :3 0 8 :3 0
51 :3 0 358 359
0 5b :3 0 35a
35b 0 35d 1b4
35f 1b6 35e 35d
:2 0 3a7 5d :3 0
8 :3 0 51 :3 0
362 363 0 5b
:3 0 364 365 0
39f 38 :3 0 39
:3 0 367 368 0
38 :3 0 49 :3 0
36a 36b 0 5e
:2 0 2 :4 0 74
:2 0 4b :2 0 15
:2 0 1b8 370 372
:3 0 1bb 36c 374
369 376 375 :2 0
3b :3 0 4c :3 0
378 379 0 3d
:4 0 75 :4 0 40
:2 0 41 :3 0 1bf
37d 37f :3 0 40
:2 0 61 :4 0 1c2
381 383 :3 0 40
:2 0 62 :4 0 1c5
385 387 :3 0 40
:2 0 5b :3 0 1c8
389 38b :3 0 40
:2 0 63 :4 0 1cb
38d 38f :3 0 40
:2 0 64 :3 0 65
:3 0 15 :2 0 66
:2 0 1ce 392 396
1d2 391 398 :3 0
1d5 37a 39a :2 0
39c 1d8 39d 377
39c 0 39e 1da
0 39f 1dc 3a2
:3 0 3a2 0 3a2
3a1 39f 3a0 :6 0
3a4 f :3 0 1df
3a6 1e1 3a5 3a4
:2 0 3a7 1e3 :2 0
3a9 0 3a9 3a8
356 3a7 :6 0 4ee
6 :3 0 64 :3 0
14 :3 0 15 :2 0
15 :2 0 1e6 3ab
3af 56 :2 0 17
:4 0 1ec 3b1 3b3
:3 0 76 :3 0 c
:3 0 77 :3 0 29
:3 0 13 :4 0 78
1 :8 0 3c0 8
:3 0 51 :3 0 3bb
3bc 0 5b :3 0
3bd 3be 0 3c0
1ef 413 5c :3 0
8 :3 0 51 :3 0
3c2 3c3 0 5b
:3 0 3c4 3c5 0
3c7 1f2 3c9 1f4
3c8 3c7 :2 0 411
5d :3 0 8 :3 0
51 :3 0 3cc 3cd
0 5b :3 0 3ce
3cf 0 409 38
:3 0 39 :3 0 3d1
3d2 0 38 :3 0
49 :3 0 3d4 3d5
0 5e :2 0 2
:4 0 79 :2 0 4b
:2 0 15 :2 0 1f6
3da 3dc :3 0 1f9
3d6 3de 3d3 3e0
3df :2 0 3b :3 0
4c :3 0 3e2 3e3
0 3d :4 0 7a
:4 0 40 :2 0 41
:3 0 1fd 3e7 3e9
:3 0 40 :2 0 61
:4 0 200 3eb 3ed
:3 0 40 :2 0 62
:4 0 203 3ef 3f1
:3 0 40 :2 0 5b
:3 0 206 3f3 3f5
:3 0 40 :2 0 63
:4 0 209 3f7 3f9
:3 0 40 :2 0 64
:3 0 65 :3 0 15
:2 0 66 :2 0 20c
3fc 400 210 3fb
402 :3 0 213 3e4
404 :2 0 406 216
407 3e1 406 0
408 218 0 409
21a 40c :3 0 40c
0 40c 40b 409
40a :6 0 40e 11
:3 0 21d 410 21f
40f 40e :2 0 411
221 :2 0 413 0
413 412 3c0 411
:6 0 415 6 :3 0
224 477 7b :3 0
c :3 0 7c :3 0
7d :3 0 13 :4 0
7e 1 :8 0 421
8 :3 0 51 :3 0
41c 41d 0 5b
:3 0 41e 41f 0
421 226 474 5c
:3 0 8 :3 0 51
:3 0 423 424 0
5b :3 0 425 426
0 428 229 42a
22b 429 428 :2 0
472 5d :3 0 8
:3 0 51 :3 0 42d
42e 0 5b :3 0
42f 430 0 46a
38 :3 0 39 :3 0
432 433 0 38
:3 0 49 :3 0 435
436 0 5e :2 0
2 :4 0 7f :2 0
4b :2 0 15 :2 0
22d 43b 43d :3 0
230 437 43f 434
441 440 :2 0 3b
:3 0 4c :3 0 443
444 0 3d :4 0
80 :4 0 40 :2 0
41 :3 0 234 448
44a :3 0 40 :2 0
61 :4 0 237 44c
44e :3 0 40 :2 0
62 :4 0 23a 450
452 :3 0 40 :2 0
5b :3 0 23d 454
456 :3 0 40 :2 0
63 :4 0 240 458
45a :3 0 40 :2 0
64 :3 0 65 :3 0
15 :2 0 66 :2 0
243 45d 461 247
45c 463 :3 0 24a
445 465 :2 0 467
24d 468 442 467
0 469 24f 0
46a 251 46d :3 0
46d 0 46d 46c
46a 46b :6 0 46f
13 :3 0 254 471
256 470 46f :2 0
472 258 :2 0 474
0 474 473 421
472 :6 0 476 6
:3 0 25b 478 3b4
415 0 479 0
476 0 479 25d
0 4ee 1c :3 0
81 :3 0 82 :3 0
81 :3 0 c :3 0
2c :3 0 e :3 0
1b :3 0 e :4 0
83 1 :8 0 499
50 :4 0 484 :3 0
68 :3 0 56 :2 0
262 487 488 :3 0
8 :3 0 51 :3 0
48a 48b 0 17
:2 0 48c 48d 0
48f 265 496 8
:3 0 51 :3 0 490
491 0 52 :2 0
492 493 0 495
267 497 489 48f
0 498 0 495
0 498 269 0
499 26c 4ec 5c
:3 0 8 :3 0 51
:3 0 49b 49c 0
5b :3 0 49d 49e
0 4a0 26f 4a2
271 4a1 4a0 :2 0
4ea 5d :3 0 8
:3 0 51 :3 0 4a5
4a6 0 5b :3 0
4a7 4a8 0 4e2
38 :3 0 39 :3 0
4aa 4ab 0 38
:3 0 49 :3 0 4ad
4ae 0 5e :2 0
2 :4 0 84 :2 0
4b :2 0 15 :2 0
273 4b3 4b5 :3 0
276 4af 4b7 4ac
4b9 4b8 :2 0 3b
:3 0 4c :3 0 4bb
4bc 0 3d :4 0
85 :4 0 40 :2 0
41 :3 0 27a 4c0
4c2 :3 0 40 :2 0
61 :4 0 27d 4c4
4c6 :3 0 40 :2 0
62 :4 0 280 4c8
4ca :3 0 40 :2 0
5b :3 0 283 4cc
4ce :3 0 40 :2 0
63 :4 0 286 4d0
4d2 :3 0 40 :2 0
64 :3 0 65 :3 0
15 :2 0 66 :2 0
289 4d5 4d9 28d
4d4 4db :3 0 290
4bd 4dd :2 0 4df
293 4e0 4ba 4df
0 4e1 295 0
4e2 297 4e5 :3 0
4e5 0 4e5 4e4
4e2 4e3 :6 0 4e7
15 :3 0 29a 4e9
29c 4e8 4e7 :2 0
4ea 29e :2 0 4ec
0 4ec 4eb 499
4ea :6 0 4ee 6
:3 0 2a1 562 33
:3 0 31 :3 0 86
:3 0 87 :3 0 31
:3 0 27 :3 0 10
:4 0 88 1 :8 0
50c 50 :4 0 4f7
:3 0 68 :3 0 56
:2 0 2a7 4fa 4fb
:3 0 8 :3 0 51
:3 0 4fd 4fe 0
17 :2 0 4ff 500
0 502 2aa 509
8 :3 0 51 :3 0
503 504 0 52
:2 0 505 506 0
508 2ac 50a 4fc
502 0 50b 0
508 0 50b 2ae
0 50c 2b1 55f
5c :3 0 8 :3 0
51 :3 0 50e 50f
0 5b :3 0 510
511 0 513 2b4
515 2b6 514 513
:2 0 55d 5d :3 0
8 :3 0 51 :3 0
518 519 0 5b
:3 0 51a 51b 0
555 38 :3 0 39
:3 0 51d 51e 0
38 :3 0 49 :3 0
520 521 0 5e
:2 0 2 :4 0 89
:2 0 4b :2 0 15
:2 0 2b8 526 528
:3 0 2bb 522 52a
51f 52c 52b :2 0
3b :3 0 4c :3 0
52e 52f 0 3d
:4 0 8a :4 0 40
:2 0 41 :3 0 2bf
533 535 :3 0 40
:2 0 61 :4 0 2c2
537 539 :3 0 40
:2 0 62 :4 0 2c5
53b 53d :3 0 40
:2 0 5b :3 0 2c8
53f 541 :3 0 40
:2 0 63 :4 0 2cb
543 545 :3 0 40
:2 0 64 :3 0 65
:3 0 15 :2 0 66
:2 0 2ce 548 54c
2d2 547 54e :3 0
2d5 530 550 :2 0
552 2d8 553 52d
552 0 554 2da
0 555 2dc 558
:3 0 558 0 558
557 555 556 :6 0
55a 17 :3 0 2df
55c 2e1 55b 55a
:2 0 55d 2e3 :2 0
55f 0 55f 55e
50c 55d :6 0 561
6 :3 0 2e6 563
33a 4ee 0 564
0 561 0 564
2e8 0 643 2b
:3 0 28 :3 0 11
:4 0 8b 1 :8 0
57e 50 :4 0 569
:3 0 68 :3 0 56
:2 0 2ed 56c 56d
:3 0 8 :3 0 51
:3 0 56f 570 0
17 :2 0 571 572
0 574 2f0 57b
8 :3 0 51 :3 0
575 576 0 52
:2 0 577 578 0
57a 2f2 57c 56e
574 0 57d 0
57a 0 57d 2f4
0 57e 2f7 5d1
5c :3 0 8 :3 0
51 :3 0 580 581
0 5b :3 0 582
583 0 585 2fa
587 2fc 586 585
:2 0 5cf 5d :3 0
8 :3 0 51 :3 0
58a 58b 0 5b
:3 0 58c 58d 0
5c7 38 :3 0 39
:3 0 58f 590 0
38 :3 0 49 :3 0
592 593 0 5e
:2 0 2 :4 0 8c
:2 0 4b :2 0 15
:2 0 2fe 598 59a
:3 0 301 594 59c
591 59e 59d :2 0
3b :3 0 4c :3 0
5a0 5a1 0 3d
:4 0 8d :4 0 40
:2 0 41 :3 0 305
5a5 5a7 :3 0 40
:2 0 61 :4 0 308
5a9 5ab :3 0 40
:2 0 62 :4 0 30b
5ad 5af :3 0 40
:2 0 5b :3 0 30e
5b1 5b3 :3 0 40
:2 0 63 :4 0 311
5b5 5b7 :3 0 40
:2 0 64 :3 0 65
:3 0 15 :2 0 66
:2 0 314 5ba 5be
318 5b9 5c0 :3 0
31b 5a2 5c2 :2 0
5c4 31e 5c5 59f
5c4 0 5c6 320
0 5c7 322 5ca
:3 0 5ca 0 5ca
5c9 5c7 5c8 :6 0
5cc 19 :3 0 325
5ce 327 5cd 5cc
:2 0 5cf 329 :2 0
5d1 0 5d1 5d0
57e 5cf :6 0 643
6 :3 0 6b :3 0
2c :3 0 e :3 0
28 :3 0 11 :4 0
8e 1 :8 0 5ee
50 :4 0 5d9 :3 0
68 :3 0 56 :2 0
32e 5dc 5dd :3 0
8 :3 0 51 :3 0
5df 5e0 0 17
:2 0 5e1 5e2 0
5e4 331 5eb 8
:3 0 51 :3 0 5e5
5e6 0 52 :2 0
5e7 5e8 0 5ea
333 5ec 5de 5e4
0 5ed 0 5ea
0 5ed 335 0
5ee 338 641 5c
:3 0 8 :3 0 51
:3 0 5f0 5f1 0
5b :3 0 5f2 5f3
0 5f5 33b 5f7
33d 5f6 5f5 :2 0
63f 5d :3 0 8
:3 0 51 :3 0 5fa
5fb 0 5b :3 0
5fc 5fd 0 637
38 :3 0 39 :3 0
5ff 600 0 38
:3 0 49 :3 0 602
603 0 5e :2 0
2 :4 0 8f :2 0
4b :2 0 15 :2 0
33f 608 60a :3 0
342 604 60c 601
60e 60d :2 0 3b
:3 0 4c :3 0 610
611 0 3d :4 0
90 :4 0 40 :2 0
41 :3 0 346 615
617 :3 0 40 :2 0
61 :4 0 349 619
61b :3 0 40 :2 0
62 :4 0 34c 61d
61f :3 0 40 :2 0
5b :3 0 34f 621
623 :3 0 40 :2 0
63 :4 0 352 625
627 :3 0 40 :2 0
64 :3 0 65 :3 0
15 :2 0 66 :2 0
355 62a 62e 359
629 630 :3 0 35c
612 632 :2 0 634
35f 635 60f 634
0 636 361 0
637 363 63a :3 0
63a 0 63a 639
637 638 :6 0 63c
1b :3 0 366 63e
368 63d 63c :2 0
63f 36a :2 0 641
0 641 640 5ee
63f :6 0 643 6
:3 0 36d 645 4f
:4 0 643 :4 0 6a7
91 :3 0 26 :4 0
649 :2 0 64f 647
0 8 :3 0 51
:3 0 64a 64b 0
5b :3 0 64c 64d
0 64f 377 6a2
5c :3 0 8 :3 0
51 :3 0 651 652
0 5b :3 0 653
654 0 656 37a
658 37c 657 656
:2 0 6a0 5d :3 0
8 :3 0 51 :3 0
65b 65c 0 5b
:3 0 65d 65e 0
698 38 :3 0 39
:3 0 660 661 0
38 :3 0 49 :3 0
663 664 0 5e
:2 0 2 :4 0 92
:2 0 4b :2 0 15
:2 0 37e 669 66b
:3 0 381 665 66d
662 66f 66e :2 0
3b :3 0 4c :3 0
671 672 0 3d
:4 0 93 :4 0 40
:2 0 41 :3 0 385
676 678 :3 0 40
:2 0 61 :4 0 388
67a 67c :3 0 40
:2 0 62 :4 0 38b
67e 680 :3 0 40
:2 0 5b :3 0 38e
682 684 :3 0 40
:2 0 63 :4 0 391
686 688 :3 0 40
:2 0 64 :3 0 65
:3 0 15 :2 0 66
:2 0 394 68b 68f
398 68a 691 :3 0
39b 673 693 :2 0
695 39e 696 670
695 0 697 3a0
0 698 3a2 69b
:3 0 69b 0 69b
69a 698 699 :6 0
69d 1d :3 0 3a5
69f 3a7 69e 69d
:2 0 6a0 3a9 :2 0
6a2 0 6a2 6a1
64f 6a0 :6 0 6a7
5 :3 0 94 :4 0
94 1 :8 0 6a7
95 :3 0 3ac 905
45 :3 0 18 :3 0
17 :2 0 3b1 6a8
6ab 56 :2 0 15
:2 0 3b6 6ad 6af
:3 0 4e :3 0 2f
:4 0 6b4 :2 0 903
6b2 6b5 :3 0 4f
:3 0 2f :3 0 13
:3 0 14 :3 0 16
:3 0 12 :4 0 6bd
:2 0 8a0 6b7 6be
:3 0 3b9 :3 0 2f
:3 0 50 :3 0 6bf
6c0 :3 0 8 :3 0
51 :3 0 6c2 6c3
0 17 :2 0 6c4
6c5 0 6c7 3be
6ce 8 :3 0 51
:3 0 6c8 6c9 0
52 :2 0 6ca 6cb
0 6cd 3c0 6cf
6c1 6c7 0 6d0
0 6cd 0 6d0
3c2 0 8a0 8
:3 0 51 :3 0 6d1
6d2 0 8 :3 0
53 :2 0 54 :3 0
6d4 6d6 0 3c7
6d5 6d8 :3 0 6d9
:2 0 55 :8 0 6dd
3ca 6de 6da 6dd
0 6df 3cc 0
8a0 64 :3 0 14
:3 0 15 :2 0 15
:2 0 3ce 6e0 6e4
56 :2 0 17 :4 0
3d4 6e6 6e8 :3 0
76 :3 0 c :3 0
77 :3 0 29 :3 0
13 :4 0 96 1
:8 0 6f5 8 :3 0
51 :3 0 6f0 6f1
0 5b :3 0 6f2
6f3 0 6f5 3d7
748 5c :3 0 8
:3 0 51 :3 0 6f7
6f8 0 5b :3 0
6f9 6fa 0 6fc
3da 6fe 3dc 6fd
6fc :2 0 746 5d
:3 0 8 :3 0 51
:3 0 701 702 0
5b :3 0 703 704
0 73e 38 :3 0
39 :3 0 706 707
0 38 :3 0 49
:3 0 709 70a 0
5e :2 0 2 :4 0
97 :2 0 4b :2 0
15 :2 0 3de 70f
711 :3 0 3e1 70b
713 708 715 714
:2 0 3b :3 0 4c
:3 0 717 718 0
3d :4 0 98 :4 0
40 :2 0 41 :3 0
3e5 71c 71e :3 0
40 :2 0 61 :4 0
3e8 720 722 :3 0
40 :2 0 62 :4 0
3eb 724 726 :3 0
40 :2 0 5b :3 0
3ee 728 72a :3 0
40 :2 0 63 :4 0
3f1 72c 72e :3 0
40 :2 0 64 :3 0
65 :3 0 15 :2 0
66 :2 0 3f4 731
735 3f8 730 737
:3 0 3fb 719 739
:2 0 73b 3fe 73c
716 73b 0 73d
400 0 73e 402
741 :3 0 741 0
741 740 73e 73f
:6 0 743 20 :3 0
405 745 407 744
743 :2 0 746 409
:2 0 748 0 748
747 6f5 746 :6 0
74a 1f :3 0 40c
7ac 7b :3 0 c
:3 0 7c :3 0 7d
:3 0 13 :4 0 99
1 :8 0 756 8
:3 0 51 :3 0 751
752 0 5b :3 0
753 754 0 756
40e 7a9 5c :3 0
8 :3 0 51 :3 0
758 759 0 5b
:3 0 75a 75b 0
75d 411 75f 413
75e 75d :2 0 7a7
5d :3 0 8 :3 0
51 :3 0 762 763
0 5b :3 0 764
765 0 79f 38
:3 0 39 :3 0 767
768 0 38 :3 0
49 :3 0 76a 76b
0 5e :2 0 2
:4 0 9a :2 0 4b
:2 0 15 :2 0 415
770 772 :3 0 418
76c 774 769 776
775 :2 0 3b :3 0
4c :3 0 778 779
0 3d :4 0 9b
:4 0 40 :2 0 41
:3 0 41c 77d 77f
:3 0 40 :2 0 61
:4 0 41f 781 783
:3 0 40 :2 0 62
:4 0 422 785 787
:3 0 40 :2 0 5b
:3 0 425 789 78b
:3 0 40 :2 0 63
:4 0 428 78d 78f
:3 0 40 :2 0 64
:3 0 65 :3 0 15
:2 0 66 :2 0 42b
792 796 42f 791
798 :3 0 432 77a
79a :2 0 79c 435
79d 777 79c 0
79e 437 0 79f
439 7a2 :3 0 7a2
0 7a2 7a1 79f
7a0 :6 0 7a4 22
:3 0 43c 7a6 43e
7a5 7a4 :2 0 7a7
440 :2 0 7a9 0
7a9 7a8 756 7a7
:6 0 7ab 1f :3 0
443 7ad 6e9 74a
0 7ae 0 7ab
0 7ae 445 0
8a0 1c :3 0 81
:3 0 9c :3 0 81
:3 0 c :3 0 81
:3 0 81 :3 0 c
:3 0 2c :3 0 e
:3 0 1b :3 0 e
:4 0 9d 1 :8 0
7d1 50 :4 0 7bc
:3 0 68 :3 0 56
:2 0 44a 7bf 7c0
:3 0 8 :3 0 51
:3 0 7c2 7c3 0
17 :2 0 7c4 7c5
0 7c7 44d 7ce
8 :3 0 51 :3 0
7c8 7c9 0 52
:2 0 7ca 7cb 0
7cd 44f 7cf 7c1
7c7 0 7d0 0
7cd 0 7d0 451
0 7d1 454 824
5c :3 0 8 :3 0
51 :3 0 7d3 7d4
0 5b :3 0 7d5
7d6 0 7d8 457
7da 459 7d9 7d8
:2 0 822 5d :3 0
8 :3 0 51 :3 0
7dd 7de 0 5b
:3 0 7df 7e0 0
81a 38 :3 0 39
:3 0 7e2 7e3 0
38 :3 0 49 :3 0
7e5 7e6 0 5e
:2 0 2 :4 0 9e
:2 0 4b :2 0 15
:2 0 45b 7eb 7ed
:3 0 45e 7e7 7ef
7e4 7f1 7f0 :2 0
3b :3 0 4c :3 0
7f3 7f4 0 3d
:4 0 9f :4 0 40
:2 0 41 :3 0 462
7f8 7fa :3 0 40
:2 0 61 :4 0 465
7fc 7fe :3 0 40
:2 0 62 :4 0 468
800 802 :3 0 40
:2 0 5b :3 0 46b
804 806 :3 0 40
:2 0 63 :4 0 46e
808 80a :3 0 40
:2 0 64 :3 0 65
:3 0 15 :2 0 66
:2 0 471 80d 811
475 80c 813 :3 0
478 7f5 815 :2 0
817 47b 818 7f2
817 0 819 47d
0 81a 47f 81d
:3 0 81d 0 81d
81c 81a 81b :6 0
81f 24 :3 0 482
821 484 820 81f
:2 0 822 486 :2 0
824 0 824 823
7d1 822 :6 0 8a0
1f :3 0 34 :3 0
2c :3 0 29 :3 0
31 :3 0 32 :3 0
2a :3 0 a0 :3 0
a1 :3 0 e :3 0
13 :3 0 16 :3 0
12 :3 0 14 :3 0
1e :3 0 a2 :4 0
a3 1 :8 0 84b
50 :4 0 836 :3 0
68 :3 0 56 :2 0
48b 839 83a :3 0
8 :3 0 51 :3 0
83c 83d 0 17
:2 0 83e 83f 0
841 48e 848 8
:3 0 51 :3 0 842
843 0 52 :2 0
844 845 0 847
490 849 83b 841
0 84a 0 847
0 84a 492 0
84b 495 89e 5c
:3 0 8 :3 0 51
:3 0 84d 84e 0
5b :3 0 84f 850
0 852 498 854
49a 853 852 :2 0
89c 5d :3 0 8
:3 0 51 :3 0 857
858 0 5b :3 0
859 85a 0 894
38 :3 0 39 :3 0
85c 85d 0 38
:3 0 49 :3 0 85f
860 0 5e :2 0
2 :4 0 a4 :2 0
4b :2 0 15 :2 0
49c 865 867 :3 0
49f 861 869 85e
86b 86a :2 0 3b
:3 0 4c :3 0 86d
86e 0 3d :4 0
a5 :4 0 40 :2 0
41 :3 0 4a3 872
874 :3 0 40 :2 0
61 :4 0 4a6 876
878 :3 0 40 :2 0
62 :4 0 4a9 87a
87c :3 0 40 :2 0
5b :3 0 4ac 87e
880 :3 0 40 :2 0
63 :4 0 4af 882
884 :3 0 40 :2 0
64 :3 0 65 :3 0
15 :2 0 66 :2 0
4b2 887 88b 4b6
886 88d :3 0 4b9
86f 88f :2 0 891
4bc 892 86c 891
0 893 4be 0
894 4c0 897 :3 0
897 0 897 896
894 895 :6 0 899
26 :3 0 4c3 89b
4c5 89a 899 :2 0
89c 4c7 :2 0 89e
0 89e 89d 84b
89c :6 0 8a0 1f
:3 0 4ca 8a2 4f
:4 0 8a0 :4 0 903
91 :3 0 2f :4 0
8a6 :2 0 8ac 8a4
0 8 :3 0 51
:3 0 8a7 8a8 0
5b :3 0 8a9 8aa
0 8ac 4d1 8ff
5c :3 0 8 :3 0
51 :3 0 8ae 8af
0 5b :3 0 8b0
8b1 0 8b3 4d4
8b5 4d6 8b4 8b3
:2 0 8fd 5d :3 0
8 :3 0 51 :3 0
8b8 8b9 0 5b
:3 0 8ba 8bb 0
8f5 38 :3 0 39
:3 0 8bd 8be 0
38 :3 0 49 :3 0
8c0 8c1 0 5e
:2 0 2 :4 0 a6
:2 0 4b :2 0 15
:2 0 4d8 8c6 8c8
:3 0 4db 8c2 8ca
8bf 8cc 8cb :2 0
3b :3 0 4c :3 0
8ce 8cf 0 3d
:4 0 a7 :4 0 40
:2 0 41 :3 0 4df
8d3 8d5 :3 0 40
:2 0 61 :4 0 4e2
8d7 8d9 :3 0 40
:2 0 62 :4 0 4e5
8db 8dd :3 0 40
:2 0 5b :3 0 4e8
8df 8e1 :3 0 40
:2 0 63 :4 0 4eb
8e3 8e5 :3 0 40
:2 0 64 :3 0 65
:3 0 15 :2 0 66
:2 0 4ee 8e8 8ec
4f2 8e7 8ee :3 0
4f5 8d0 8f0 :2 0
8f2 4f8 8f3 8cd
8f2 0 8f4 4fa
0 8f5 4fc 8f8
:3 0 8f8 0 8f8
8f7 8f5 8f6 :6 0
8fa 28 :3 0 4ff
8fc 501 8fb 8fa
:2 0 8fd 503 :2 0
8ff 0 8ff 8fe
8ac 8fd :6 0 903
5 :3 0 94 :4 0
94 1 :8 0 903
506 904 6b0 903
0 906 142 6a7
0 906 50b 0
907 50e 909 4f
:4 0 907 :4 0 9b6
91 :3 0 1a :4 0
90d :2 0 913 90b
0 8 :3 0 51
:3 0 90e 90f 0
5b :3 0 910 911
0 913 513 966
5c :3 0 8 :3 0
51 :3 0 915 916
0 5b :3 0 917
918 0 91a 516
91c 518 91b 91a
:2 0 964 5d :3 0
8 :3 0 51 :3 0
91f 920 0 5b
:3 0 921 922 0
95c 38 :3 0 39
:3 0 924 925 0
38 :3 0 49 :3 0
927 928 0 5e
:2 0 2 :4 0 a8
:2 0 4b :2 0 15
:2 0 51a 92d 92f
:3 0 51d 929 931
926 933 932 :2 0
3b :3 0 4c :3 0
935 936 0 3d
:4 0 a9 :4 0 40
:2 0 41 :3 0 521
93a 93c :3 0 40
:2 0 61 :4 0 524
93e 940 :3 0 40
:2 0 62 :4 0 527
942 944 :3 0 40
:2 0 5b :3 0 52a
946 948 :3 0 40
:2 0 63 :4 0 52d
94a 94c :3 0 40
:2 0 64 :3 0 65
:3 0 15 :2 0 66
:2 0 530 94f 953
534 94e 955 :3 0
537 937 957 :2 0
959 53a 95a 934
959 0 95b 53c
0 95c 53e 95f
:3 0 95f 0 95f
95e 95c 95d :6 0
961 2a :3 0 541
963 543 962 961
:2 0 964 545 :2 0
966 0 966 965
913 964 :6 0 9b6
1 :3 0 38 :3 0
39 :3 0 968 969
0 38 :3 0 49
:3 0 96b 96c 0
aa :2 0 2 :4 0
ab :2 0 4b :2 0
15 :2 0 548 971
973 :3 0 54b 96d
975 96a 977 976
:2 0 3b :3 0 4c
:3 0 979 97a 0
3d :4 0 ac :4 0
40 :2 0 41 :3 0
54f 97e 980 :3 0
40 :2 0 61 :4 0
552 982 984 :3 0
40 :2 0 ad :4 0
555 986 988 :3 0
40 :2 0 45 :3 0
46 :3 0 8 :3 0
51 :3 0 98d 98e
0 558 98c 990
47 :4 0 55a 98b
993 55d 98a 995
:3 0 40 :2 0 ae
:4 0 560 997 999
:3 0 40 :2 0 43
:4 0 563 99b 99d
:3 0 40 :2 0 a
:3 0 566 99f 9a1
:3 0 40 :2 0 43
:4 0 569 9a3 9a5
:3 0 56c 97b 9a7
:2 0 9a9 56f 9aa
978 9a9 0 9ab
571 0 9b6 af
:3 0 b0 :3 0 9ac
9ad 0 8 :3 0
51 :3 0 9af 9b0
0 a :3 0 68
:3 0 573 9ae 9b4
:2 0 9b6 577 a57
8 :3 0 b1 :2 0
4 9b7 9b8 0
38 :3 0 39 :3 0
9ba 9bb 0 38
:3 0 49 :3 0 9bd
9be 0 5e :2 0
2 :4 0 b2 :2 0
4b :2 0 15 :2 0
581 9c3 9c5 :3 0
584 9bf 9c7 9bc
9c9 9c8 :2 0 3b
:3 0 4c :3 0 9cb
9cc 0 3d :4 0
b3 :4 0 40 :2 0
41 :3 0 588 9d0
9d2 :3 0 58b 9cd
9d4 :2 0 9ed 3b
:3 0 4c :3 0 9d6
9d7 0 3d :4 0
b4 :4 0 40 :2 0
8 :3 0 b5 :3 0
9dc 9dd 0 58e
9db 9df :3 0 40
:2 0 ae :4 0 591
9e1 9e3 :3 0 40
:2 0 8 :3 0 b6
:3 0 9e6 9e7 0
594 9e5 9e9 :3 0
597 9d8 9eb :2 0
9ed 59a 9ee 9ca
9ed 0 9ef 59d
0 9ff 4e :3 0
37 :3 0 8 :3 0
b5 :3 0 b7 :3 0
8 :3 0 b6 :3 0
b8 :3 0 58 :4 0
b9 1 :a 0 9fb
:2 0 9ff 9f1 9f9
:2 0 7 :3 0 37
:3 0 9fd :2 0 9ff
59f a01 5a3 a00
9ff :2 0 a55 5d
:3 0 8 :3 0 51
:3 0 a04 a05 0
5b :3 0 a06 a07
0 a4d 38 :3 0
39 :3 0 a09 a0a
0 38 :3 0 49
:3 0 a0c a0d 0
5e :2 0 2 :4 0
ba :2 0 4b :2 0
15 :2 0 5a5 a12
a14 :3 0 5a8 a0e
a16 a0b a18 a17
:2 0 3b :3 0 4c
:3 0 a1a a1b 0
3d :4 0 bb :4 0
40 :2 0 41 :3 0
5ac a1f a21 :3 0
5af a1c a23 :2 0
a3d 3b :3 0 4c
:3 0 a25 a26 0
3d :4 0 62 :4 0
40 :2 0 5b :3 0
5b2 a2a a2c :3 0
40 :2 0 63 :4 0
5b5 a2e a30 :3 0
40 :2 0 64 :3 0
65 :3 0 15 :2 0
66 :2 0 5b8 a33
a37 5bc a32 a39
:3 0 5bf a27 a3b
:2 0 a3d 5c2 a3e
a19 a3d 0 a3f
5c5 0 a4d 4e
:3 0 37 :3 0 b7
:3 0 8 :3 0 b6
:3 0 b8 :3 0 58
:4 0 bc 1 :a 0
a49 :2 0 a4d a41
a47 :2 0 7 :3 0
37 :3 0 a4b :2 0
a4d 5c7 a50 :3 0
a50 0 a50 a4f
a4d a4e :6 0 a52
1 :3 0 5cc a54
5ce a53 a52 :2 0
a55 5d0 :2 0 a57
5d3 a57 a56 9b6
a55 :6 0 a58 :2 0
2 10 a57 a5b
:3 0 a5a a58 a5c
:8 0 
5e2
4
:3 0 1 3 1
7 2 6 a
1 14 1 12
1 1b 1 19
1 22 1 20
1 29 1 27
1 30 1 2e
1 37 1 35
1 3e 1 3c
1 45 1 43
1 4a 1 50
1 6a 1 7e
1 9e 1 a4
1 b1 1 bd
2 c0 c2 2
c5 c7 2 c9
cb 2 cd cf
2 d1 d3 1
d9 2 db dc
2 d5 de 2
e0 e2 4 b9
bf c4 e4 1
e6 1 e8 2
f2 f4 3 f0
f1 f6 2 ff
101 2 fe 103
1 105 1 107
1 120 1 126
2 128 129 1
131 2 12d 131
1 136 1 138
2 13b 13c 1
140 2 13e 140
4 14a 14b 14c
14d 1 158 1
15e 2 160 161
1 169 2 165
169 1 16e 1
170 2 17a 17f
1 186 1 181
2 199 19b 3
197 198 19d 2
1a6 1a8 2 1aa
1ac 2 1ae 1b0
2 1b2 1b4 2
1b6 1b8 3 1bd
1be 1bf 2 1ba
1c1 2 1a5 1c3
1 1c5 1 1c7
2 190 1c8 1
1cc 1 18b 2
189 1d0 2 1d6
1d7 1 1db 2
1d9 1db 1 1e5
2 1e4 1e5 1
1ed 1 1f3 2
1f5 1f6 2 1e2
1f7 1 1fe 1
1f9 2 211 213
3 20f 210 215
2 21e 220 2
222 224 2 226
228 2 22a 22c
2 22e 230 3
235 236 237 2
232 239 2 21d
23b 1 23d 1
23f 2 208 240
1 244 1 203
2 201 248 1
257 2 256 257
1 25f 1 265
2 267 268 2
254 269 1 270
1 26b 2 283
285 3 281 282
287 2 290 292
2 294 296 2
298 29a 2 29c
29e 2 2a0 2a2
3 2a7 2a8 2a9
2 2a4 2ab 2
28f 2ad 1 2af
1 2b1 2 27a
2b2 1 2b6 1
275 2 273 2ba
2 24b 2bd 1
2c0 1 2c9 2
2c8 2c9 1 2d1
1 2d7 2 2d9
2da 2 2c6 2db
1 2e2 1 2dd
2 2f5 2f7 3
2f3 2f4 2f9 2
302 304 2 306
308 2 30a 30c
2 30e 310 2
312 314 3 319
31a 31b 2 316
31d 2 301 31f
1 321 1 323
2 2ec 324 1
328 1 2e7 2
2e5 32c 1 337
2 333 337 1
343 2 342 343
1 34b 1 351
2 353 354 2
340 355 1 35c
1 357 2 36f
371 3 36d 36e
373 2 37c 37e
2 380 382 2
384 386 2 388
38a 2 38c 38e
3 393 394 395
2 390 397 2
37b 399 1 39b
1 39d 2 366
39e 1 3a2 1
361 2 35f 3a6
3 3ac 3ad 3ae
1 3b2 2 3b0
3b2 2 3ba 3bf
1 3c6 1 3c1
2 3d9 3db 3
3d7 3d8 3dd 2
3e6 3e8 2 3ea
3ec 2 3ee 3f0
2 3f2 3f4 2
3f6 3f8 3 3fd
3fe 3ff 2 3fa
401 2 3e5 403
1 405 1 407
2 3d0 408 1
40c 1 3cb 2
3c9 410 1 413
2 41b 420 1
427 1 422 2
43a 43c 3 438
439 43e 2 447
449 2 44b 44d
2 44f 451 2
453 455 2 457
459 3 45e 45f
460 2 45b 462
2 446 464 1
466 1 468 2
431 469 1 46d
1 42c 2 42a
471 1 474 2
477 478 1 486
2 485 486 1
48e 1 494 2
496 497 2 483
498 1 49f 1
49a 2 4b2 4b4
3 4b0 4b1 4b6
2 4bf 4c1 2
4c3 4c5 2 4c7
4c9 2 4cb 4cd
2 4cf 4d1 3
4d6 4d7 4d8 2
4d3 4da 2 4be
4dc 1 4de 1
4e0 2 4a9 4e1
1 4e5 1 4a4
2 4a2 4e9 3
3a9 479 4ec 1
4f9 2 4f8 4f9
1 501 1 507
2 509 50a 2
4f6 50b 1 512
1 50d 2 525
527 3 523 524
529 2 532 534
2 536 538 2
53a 53c 2 53e
540 2 542 544
3 549 54a 54b
2 546 54d 2
531 54f 1 551
1 553 2 51c
554 1 558 1
517 2 515 55c
1 55f 2 562
563 1 56b 2
56a 56b 1 573
1 579 2 57b
57c 2 568 57d
1 584 1 57f
2 597 599 3
595 596 59b 2
5a4 5a6 2 5a8
5aa 2 5ac 5ae
2 5b0 5b2 2
5b4 5b6 3 5bb
5bc 5bd 2 5b8
5bf 2 5a3 5c1
1 5c3 1 5c5
2 58e 5c6 1
5ca 1 589 2
587 5ce 1 5db
2 5da 5db 1
5e3 1 5e9 2
5eb 5ec 2 5d8
5ed 1 5f4 1
5ef 2 607 609
3 605 606 60b
2 614 616 2
618 61a 2 61c
61e 2 620 622
2 624 626 3
62b 62c 62d 2
628 62f 2 613
631 1 633 1
635 2 5fe 636
1 63a 1 5f9
2 5f7 63e 9
14e 162 171 1d3
2c1 32f 564 5d1
641 2 648 64e
1 655 1 650
2 668 66a 3
666 667 66c 2
675 677 2 679
67b 2 67d 67f
2 681 683 2
685 687 3 68c
68d 68e 2 689
690 2 674 692
1 694 1 696
2 65f 697 1
69b 1 65a 2
658 69f 4 145
645 6a2 6a5 2
6a9 6aa 1 6ae
2 6ac 6ae 4
6b8 6b9 6ba 6bb
1 6c6 1 6cc
2 6ce 6cf 1
6d7 2 6d3 6d7
1 6dc 1 6de
3 6e1 6e2 6e3
1 6e7 2 6e5
6e7 2 6ef 6f4
1 6fb 1 6f6
2 70e 710 3
70c 70d 712 2
71b 71d 2 71f
721 2 723 725
2 727 729 2
72b 72d 3 732
733 734 2 72f
736 2 71a 738
1 73a 1 73c
2 705 73d 1
741 1 700 2
6fe 745 1 748
2 750 755 1
75c 1 757 2
76f 771 3 76d
76e 773 2 77c
77e 2 780 782
2 784 786 2
788 78a 2 78c
78e 3 793 794
795 2 790 797
2 77b 799 1
79b 1 79d 2
766 79e 1 7a2
1 761 2 75f
7a6 1 7a9 2
7ac 7ad 1 7be
2 7bd 7be 1
7c6 1 7cc 2
7ce 7cf 2 7bb
7d0 1 7d7 1
7d2 2 7ea 7ec
3 7e8 7e9 7ee
2 7f7 7f9 2
7fb 7fd 2 7ff
801 2 803 805
2 807 809 3
80e 80f 810 2
80b 812 2 7f6
814 1 816 1
818 2 7e1 819
1 81d 1 7dc
2 7da 821 1
838 2 837 838
1 840 1 846
2 848 849 2
835 84a 1 851
1 84c 2 864
866 3 862 863
868 2 871 873
2 875 877 2
879 87b 2 87d
87f 2 881 883
3 888 889 88a
2 885 88c 2
870 88e 1 890
1 892 2 85b
893 1 897 1
856 2 854 89b
6 6bc 6d0 6df
7ae 824 89e 2
8a5 8ab 1 8b2
1 8ad 2 8c5
8c7 3 8c3 8c4
8c9 2 8d2 8d4
2 8d6 8d8 2
8da 8dc 2 8de
8e0 2 8e2 8e4
3 8e9 8ea 8eb
2 8e6 8ed 2
8d1 8ef 1 8f1
1 8f3 2 8bc
8f4 1 8f8 1
8b7 2 8b5 8fc
4 6b3 8a2 8ff
902 2 905 904
4 117 12a 139
906 2 90c 912
1 919 1 914
2 92c 92e 3
92a 92b 930 2
939 93b 2 93d
93f 2 941 943
2 945 947 2
949 94b 3 950
951 952 2 94d
954 2 938 956
1 958 1 95a
2 923 95b 1
95f 1 91e 2
91c 963 2 970
972 3 96e 96f
974 2 97d 97f
2 981 983 2
985 987 1 98f
2 991 992 2
989 994 2 996
998 2 99a 99c
2 99e 9a0 2
9a2 9a4 2 97c
9a6 1 9a8 1
9aa 3 9b1 9b2
9b3 9 e9 108
10b 10e 111 909
966 9ab 9b5 2
9c2 9c4 3 9c0
9c1 9c6 2 9cf
9d1 2 9ce 9d3
2 9da 9de 2
9e0 9e2 2 9e4
9e8 2 9d9 9ea
2 9d5 9ec 1
9ee 3 9ef 9fa
9fe 1 9b9 2
a11 a13 3 a0f
a10 a15 2 a1e
a20 2 a1d a22
2 a29 a2b 2
a2d a2f 3 a34
a35 a36 2 a31
a38 2 a28 a3a
2 a24 a3c 1
a3e 4 a08 a3f
a48 a4c 1 a50
1 a03 2 a01
a54 e 17 1e
25 2c 33 3a
41 48 4e 54
6b 7f 9f a9

1
4
0 
a5b
0
1
50
2c
11
0 1 1 1 1 5 6 7
6 9 6 b 6 d 6 f
6 11 6 13 6 15 6 17
6 19 6 1b 5 1d 5 1f
20 1f 22 1f 24 1f 26 5
28 1 2a 1 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0
0 0 0 0 0 0 0 0

3 1 0
57 1 2
2e 1 0
71 1 3
85 1 4
43 1 0
4a 1 0
35 1 0
50 1 0
27 1 0
12 1 0
3c 1 0
20 1 0
2 0 1
7 1 0
19 1 0
a4 1 0
0

/
SHOW ERRORS FUNCTION SP_TEST_ASSIGN;
ALTER FUNCTION SP_TEST_ASSIGN COMPILE DEBUG;
/*********************************************************************************************************************************************/
Prompt == FUNCTION FC_MIGRATION ==
CREATE OR REPLACE FUNCTION FC_MIGRATION wrapped 
0
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
abcd
3
8
9200000
1
4
0 
83
2 :e:
1FUNCTION:
1FC_MIGRATION:
1H_PROJECT_ID:
1OUT:
1NOCOPY:
1VARCHAR2:
1RETURN:
1PLS_INTEGER:
1H_LEVEL:
10:
1H_ROLLUP_BITS:
1H_TEMP_SC_STATE:
1CURSOR:
1C01:
1PROJECT_ID:
1LEVEL_ID:
1TMT_WBS:
1TYPE_ID:
1PMO_CNST:
1SC_PROJECT:
1SC_PROPOSAL:
1SC_PROJ_TYPE1:
1SC_PROJ_TYPE2:
1SC_PROJ_TYPE3:
1RANK:
1SELECT PROJECT_ID, LEVEL_ID FROM TMT_WBS:n         WHERE TYPE_ID IN (PMO_CNST+
1.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE+
12,PMO_CNST.SC_PROJ_TYPE3):n         ORDER BY RANK DESC:
1C02:
1C03:
1ELEMENT_ID:
1SELECT LEVEL_ID FROM TMT_WBS:n         WHERE TYPE_ID IN (PMO_CNST.SC_PROJECT,+
1PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.S+
1C_PROJ_TYPE3) AND ELEMENT_ID = h_project_id:n         ORDER BY RANK DESC:
1C04:
1SELECT /*PROJECT_ID,*/ LEVEL_ID:n         FROM TMT_WBS:n         WHERE TYPE_I+
1D IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNS+
1T.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND ELEMENT_ID = h_project_id:n      +
1   ORDER BY RANK DESC:
1ALLTRACES:
1TRACE_ENABLED:
1CHECK_PARAMS:
1LFILE:
1FPUT_PARAMS:
1AllTrace:
1LINES_BEFORE_PARAMS:
1-- :
1||:
1SYSTIMESTAMP:
1::r ::= FC_MIGRATION(:
1':
1);:
1CHECK_TRACE:
11:
137:
1+:
1FPUT:
1               ENTER:: FC_MIGRATION(38) :: cplan.sql(21796) :: :
1IS NULL:
1OPEN:
1LOOP:
1FOUND:
1PMO_DB2M:
1SC_STATE:
1100:
1!=:
1SC_SQL_SUCCESS:
1EXIT:
13269638:
1TASK_ASSIGNMENTS:
1ROLLUP_BITS:
1ACTUAL_START:
1SC_SET_ALL_BITS:
1UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = CASE WHEN ACTUAL_START IS NuLL THEN+
1 PMO_CNST.SC_SET_ALL_BITS ELSE h_rollup_bits END:n            WHERE PROJECT_I+
1D = h_project_id:
1TRUE:
1=:
1NO_DATA_FOUND:
1SQLCODE:
1OTHERS:
116:
174:
1           EXCEPTION:: FC_MIGRATION(75) :: cplan.sql(21833) :: :
1 :
1SQLCODE:: :
1 SQLERR:: :
1SUBSTR:
1SQLERRM:
1500:
1PMO_FUNC:
1BIT_OR:
1SC_PLANNED_BIT:
1SC_SCHED_BIT:
1PLANNED_FINISH:
1SCHED_FINISH:
1ACTUAL_FINISH:
1FORECAST_FINISH:
1FORECAST_START:
1PLANNED_EFFORT:
1SCHED_EFFORT:
1UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, PLANNED_FINISH = N+
1uLL, SCHED_FINISH = NuLL,:n            ACTUAL_FINISH = NuLL,:n            FOR+
1ECAST_FINISH = CASE WHEN FORECAST_START IS NuLL THEN FORECAST_FINISH ELSE NuL+
1L END:n            WHERE PROJECT_ID = h_project_id AND (PLANNED_EFFORT + SCHE+
1D_EFFORT > 0):
198:
1           EXCEPTION:: FC_MIGRATION(99) :: cplan.sql(21857) :: :
1UPDATE TMT_WBS SET ROLLUP_BITS = h_rollup_bits, PLANNED_FINISH = NuLL, SCHED_+
1FINISH = NuLL:n            WHERE PROJECT_ID = h_project_id AND ((PLANNED_EFFO+
1RT + SCHED_EFFORT) = 0):
1118:
1           EXCEPTION:: FC_MIGRATION(119) :: cplan.sql(21877) :: :
1TMT_CRI:
1UPDATE TMT_CRI SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, PLANNED_FINISH = N+
1uLL, SCHED_FINISH = NuLL:n            WHERE PROJECT_ID = h_project_id AND TYP+
1E_ID <> 14:
1138:
1           EXCEPTION:: FC_MIGRATION(139) :: cplan.sql(21897) :: :
1COALESCE:
1FC_ROLLUP:
1COMMIT:
1CLOSE:
1160:
1           EXCEPTION:: FC_MIGRATION(161) :: cplan.sql(21919) :: :
1UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, ACTUAL_FINISH = Nu+
1LL,:n            FORECAST_FINISH = CASE WHEN FORECAST_START IS NuLL THEN FORE+
1CAST_FINISH ELSE NuLL END:n            WHERE PROJECT_ID = h_project_id AND ((+
1PLANNED_EFFORT + SCHED_EFFORT) = 0):
1197:
1           EXCEPTION:: FC_MIGRATION(198) :: cplan.sql(21956) :: :
1219:
1           EXCEPTION:: FC_MIGRATION(220) :: cplan.sql(21978) :: :
1UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS WHERE PROJ+
1ECT_ID = h_project_id:
1256:
1           EXCEPTION:: FC_MIGRATION(257) :: cplan.sql(22015) :: :
1278:
1           EXCEPTION:: FC_MIGRATION(279) :: cplan.sql(22037) :: :
1298:
1           EXCEPTION:: FC_MIGRATION(299) :: cplan.sql(22057) :: :
1318:
1           EXCEPTION:: FC_MIGRATION(319) :: cplan.sql(22077) :: :
1340:
1           EXCEPTION:: FC_MIGRATION(341) :: cplan.sql(22099) :: :
1377:
1           EXCEPTION:: FC_MIGRATION(378) :: cplan.sql(22136) :: :
1399:
1           EXCEPTION:: FC_MIGRATION(400) :: cplan.sql(22158) :: :
1406:
1                EXIT:: FC_MIGRATION(407) :: cplan.sql(22165) :: :
1STATE:: :
0

0
0
853
2
0 a0 8d 90 :3 a0 b0 3f b4
:2 a0 2c 6a a3 a0 1c 51 81
b0 a3 a0 1c 51 81 b0 a3
a0 1c 51 81 b0 a0 f4 b4
bf c8 :f a0 12a bd b7 11 a4
b1 a0 f4 b4 bf c8 :f a0 12a
bd b7 11 a4 b1 a0 f4 b4
bf c8 :10 a0 12a bd b7 11 a4
b1 a0 f4 b4 bf c8 :10 a0 12a
bd b7 11 a4 b1 :2 a0 6b :2 a0
6b 6e a5 b a 10 :2 a0 6b
6e :2 a0 6b 6e a5 b 6e 7e
a0 b4 2e 6e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e a5 57 b7 19
3c :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e a5
57 b7 19 3c a0 7e b4 2e
5a :2 a0 e9 dd b3 :4 a0 e9 d3
5 :2 a0 f :2 a0 6b 51 d b7
:2 a0 6b 51 d b7 :2 19 3c :2 a0
6b a0 7e a0 6b b4 2e 5a
a0 2b b7 19 3c a0 51 d
:8 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:3 a0 6b :2 a0 6b :2 a0 6b a5 b
d :e a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :9 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :9 a0 12a a0 f a0 7e b4
2e :2 a0 6b 51 d b7 :2 a0 6b
51 d b7 :2 19 3c b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f :2 a0 51 a5 b 7e 51 b4
2e :3 a0 a5 b d a0 12a b7
19 3c b7 a0 47 :2 a0 e9 c1
:2 a0 6b a0 d b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 e9 dd b3 :4 a0 e9 d3 5
:2 a0 f :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c :2 a0 6b
a0 7e a0 6b b4 2e 5a a0
2b b7 19 3c :c a0 12a a0 f
a0 7e b4 2e :2 a0 6b 51 d
b7 :2 a0 6b 51 d b7 :2 19 3c
b7 :3 a0 6b a0 d b7 a6 9
a0 53 :2 a0 6b a0 d :2 a0 6b
:2 a0 6b 51 6e 51 7e 51 b4
2e a5 b a 10 :2 a0 6b :2 6e
7e a0 b4 2e 7e 6e b4 2e
7e 6e b4 2e 7e a0 b4 2e
7e 6e b4 2e 7e :2 a0 :2 51 a5
b b4 2e a5 57 b7 19 3c
b7 a4 b1 11 4f b7 a6 9
a4 b1 11 4f :2 a0 51 a5 b
7e 51 b4 2e :3 a0 a5 b d
a0 12a b7 19 3c b7 a0 47
:2 a0 e9 c1 :2 a0 6b a0 d b7
:3 a0 6b a0 d b7 a6 9 a0
53 :2 a0 6b a0 d :2 a0 6b :2 a0
6b 51 6e 51 7e 51 b4 2e
a5 b a 10 :2 a0 6b :2 6e 7e
a0 b4 2e 7e 6e b4 2e 7e
6e b4 2e 7e a0 b4 2e 7e
6e b4 2e 7e :2 a0 :2 51 a5 b
b4 2e a5 57 b7 19 3c b7
a4 b1 11 4f b7 a6 9 a4
b1 11 4f b7 :2 a0 e9 dd b3
:3 a0 e9 d3 :2 a0 f :2 a0 6b 51
d b7 :2 a0 6b 51 d b7 :2 19
3c :2 a0 6b a0 7e a0 6b b4
2e 5a a0 2b b7 19 3c :3 a0
6b :2 a0 6b :2 a0 6b a5 b d
:6 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:e a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:9 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:9 a0 12a a0 f a0 7e b4 2e
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c b7 :3 a0 6b a0
d b7 a6 9 a0 53 :2 a0 6b
a0 d :2 a0 6b :2 a0 6b 51 6e
51 7e 51 b4 2e a5 b a
10 :2 a0 6b :2 6e 7e a0 b4 2e
7e 6e b4 2e 7e 6e b4 2e
7e a0 b4 2e 7e 6e b4 2e
7e :2 a0 :2 51 a5 b b4 2e a5
57 b7 19 3c b7 a4 b1 11
4f b7 a6 9 a4 b1 11 4f
:2 a0 51 a5 b 7e 51 b4 2e
:3 a0 a5 b d a0 12a b7 19
3c b7 a0 47 :2 a0 e9 c1 :2 a0
6b a0 d b7 :3 a0 6b a0 d
b7 a6 9 a0 53 :2 a0 6b a0
d :2 a0 6b :2 a0 6b 51 6e 51
7e 51 b4 2e a5 b a 10
:2 a0 6b :2 6e 7e a0 b4 2e 7e
6e b4 2e 7e 6e b4 2e 7e
a0 b4 2e 7e 6e b4 2e 7e
:2 a0 :2 51 a5 b b4 2e a5 57
b7 19 3c b7 a4 b1 11 4f
b7 a6 9 a4 b1 11 4f :2 a0
e9 dd b3 :3 a0 e9 d3 :2 a0 f
:2 a0 6b 51 d b7 :2 a0 6b 51
d b7 :2 19 3c :2 a0 6b a0 7e
a0 6b b4 2e 5a a0 2b b7
19 3c :c a0 12a a0 f a0 7e
b4 2e :2 a0 6b 51 d b7 :2 a0
6b 51 d b7 :2 19 3c b7 :3 a0
6b a0 d b7 a6 9 a0 53
:2 a0 6b a0 d :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e a0 b4 2e 7e 6e
b4 2e 7e :2 a0 :2 51 a5 b b4
2e a5 57 b7 19 3c b7 a4
b1 11 4f b7 a6 9 a4 b1
11 4f :2 a0 51 a5 b 7e 51
b4 2e :3 a0 a5 b d a0 12a
b7 19 3c b7 a0 47 :2 a0 e9
c1 :2 a0 6b a0 d b7 :3 a0 6b
a0 d b7 a6 9 a0 53 :2 a0
6b a0 d :2 a0 6b :2 a0 6b 51
6e 51 7e 51 b4 2e a5 b
a 10 :2 a0 6b :2 6e 7e a0 b4
2e 7e 6e b4 2e 7e 6e b4
2e 7e a0 b4 2e 7e 6e b4
2e 7e :2 a0 :2 51 a5 b b4 2e
a5 57 b7 19 3c b7 a4 b1
11 4f b7 a6 9 a4 b1 11
4f b7 :2 19 3c :2 a0 6b :2 a0 6b
51 6e 51 7e 51 b4 2e a5
b a 10 :2 a0 6b :2 6e 7e a0
b4 2e 7e 6e b4 2e 7e 6e
b4 2e 7e :2 a0 6b b4 2e a5
57 b7 19 3c :3 a0 6b 65 b7
a4 b1 11 68 4f 1d 17 b5 
853
2
0 3 7 2b 1f 23 27 1e
32 1b 37 3b 3f 43 5f 4b
4f 57 5a 4a 7b 6a 6e 47
76 69 97 86 8a 66 92 85
9e a2 82 b3 b6 ba be c2
c6 ca ce d2 d6 da de e2
e6 ea ee f2 f6 102 107 109
115 119 11b 11f 130 131 134 138
13c 140 144 148 14c 150 154 158
15c 160 164 168 16c 170 174 180
185 187 193 197 199 19d 1ae 1af
1b2 1b6 1ba 1be 1c2 1c6 1ca 1ce
1d2 1d6 1da 1de 1e2 1e6 1ea 1ee
1f2 1f6 202 207 209 215 219 21b
21f 230 231 234 238 23c 240 244
248 24c 250 254 258 25c 260 264
268 26c 270 274 278 284 289 28b
297 29b 29d 2a1 2a5 2a8 2ac 2b0
2b3 2b8 2b9 1 2bb 2c0 2c4 2c8
2cb 2d0 2d4 2d8 2db 2e0 2e1 2e3
2e8 2eb 2ef 2f0 2f5 2fa 2fd 302
303 308 30b 30f 310 315 318 31d
31e 323 326 32b 32c 331 332 337
339 33d 340 344 348 34b 34f 353
356 359 35e 361 364 367 368 36d
36e 1 370 375 379 37d 380 385
38a 38d 391 392 397 398 39d 39f
3a3 3a6 3aa 3ad 3ae 3b3 3b6 3ba
3be 3c3 3c7 3c8 3cc 3d0 3d4 3d8
3dd 3e2 3e6 3ea 3ee 3f3 3f7 3fb
3fe 401 405 407 40b 40f 412 415
419 41b 41f 423 426 42a 42e 431
435 438 43c 43f 440 445 448 44c
452 454 458 45b 45f 462 466 46a
46e 472 476 47a 47e 482 486 492
496 49b 49f 4a2 4a3 4a8 4ac 4b0
4b3 4b6 4ba 4bc 4c0 4c4 4c7 4ca
4ce 4d0 4d4 4d8 4db 4dd 4e1 4e5
4e9 4ec 4f0 4f4 4f6 4f7 4fc 1
500 504 508 50b 50f 513 517 51b
51e 522 526 529 52c 531 534 537
53a 53b 540 541 1 543 548 54c
550 553 558 55d 560 564 565 56a
56d 572 573 578 57b 580 581 586
589 58d 58e 593 596 59b 59c 5a1
5a4 5a8 5ac 5af 5b2 5b3 5b5 5b6
5bb 5bc 5c1 5c3 5c7 5ca 5cc 5d0
5d2 5de 5e0 5e2 5e3 5e8 5ec 5ee
5fa 5fc 600 604 608 60b 60f 613
616 61a 61e 621 622 624 628 62c
630 634 638 63c 640 644 648 64c
650 654 658 65c 660 66c 670 675
679 67c 67d 682 686 68a 68d 690
694 696 69a 69e 6a1 6a4 6a8 6aa
6ae 6b2 6b5 6b7 6bb 6bf 6c3 6c6
6ca 6ce 6d0 6d1 6d6 1 6da 6de
6e2 6e5 6e9 6ed 6f1 6f5 6f8 6fc
700 703 706 70b 70e 711 714 715
71a 71b 1 71d 722 726 72a 72d
732 737 73a 73e 73f 744 747 74c
74d 752 755 75a 75b 760 763 767
768 76d 770 775 776 77b 77e 782
786 789 78c 78d 78f 790 795 796
79b 79d 7a1 7a4 7a6 7aa 7ac 7b8
7ba 7bc 7bd 7c2 7c6 7c8 7d4 7d6
7da 7de 7e2 7e6 7ea 7ee 7f2 7f6
7fa 806 80a 80f 813 816 817 81c
820 824 827 82a 82e 830 834 838
83b 83e 842 844 848 84c 84f 851
855 859 85d 860 864 868 86a 86b
870 1 874 878 87c 87f 883 887
88b 88f 892 896 89a 89d 8a0 8a5
8a8 8ab 8ae 8af 8b4 8b5 1 8b7
8bc 8c0 8c4 8c7 8cc 8d1 8d4 8d8
8d9 8de 8e1 8e6 8e7 8ec 8ef 8f4
8f5 8fa 8fd 901 902 907 90a 90f
910 915 918 91c 920 923 926 927
929 92a 92f 930 935 937 93b 93e
940 944 946 952 954 956 957 95c
960 962 96e 970 974 978 97c 980
984 988 98c 990 994 9a0 9a4 9a9
9ad 9b0 9b1 9b6 9ba 9be 9c1 9c4
9c8 9ca 9ce 9d2 9d5 9d8 9dc 9de
9e2 9e6 9e9 9eb 9ef 9f3 9f7 9fa
9fe a02 a04 a05 a0a 1 a0e a12
a16 a19 a1d a21 a25 a29 a2c a30
a34 a37 a3a a3f a42 a45 a48 a49
a4e a4f 1 a51 a56 a5a a5e a61
a66 a6b a6e a72 a73 a78 a7b a80
a81 a86 a89 a8e a8f a94 a97 a9b
a9c aa1 aa4 aa9 aaa aaf ab2 ab6
aba abd ac0 ac1 ac3 ac4 ac9 aca
acf ad1 ad5 ad8 ada ade ae0 aec
aee af0 af1 af6 afa afc b08 b0a
b0e b12 b15 b16 b18 b1b b1e b1f
b24 b28 b2c b30 b31 b33 b37 b3b
b47 b49 b4d b50 b52 b56 b5d b61
b65 b6a b6c b70 b74 b77 b7b b7f
b81 b85 b89 b8d b90 b94 b98 b9a
b9b ba0 1 ba4 ba8 bac baf bb3
bb7 bbb bbf bc2 bc6 bca bcd bd0
bd5 bd8 bdb bde bdf be4 be5 1
be7 bec bf0 bf4 bf7 bfc c01 c04
c08 c09 c0e c11 c16 c17 c1c c1f
c24 c25 c2a c2d c31 c32 c37 c3a
c3f c40 c45 c48 c4c c50 c53 c56
c57 c59 c5a c5f c60 c65 c67 c6b
c6e c70 c74 c76 c82 c84 c86 c87
c8c c90 c92 c9e ca0 ca4 ca8 cad
cb1 cb2 cb6 cba cbe cc2 cc7 ccc
cd0 cd4 cd8 cdd ce1 ce5 ce8 ceb
cef cf1 cf5 cf9 cfc cff d03 d05
d09 d0d d10 d14 d18 d1b d1f d22
d26 d29 d2a d2f d32 d36 d3c d3e
d42 d45 d49 d4d d51 d55 d59 d5d
d61 d65 d69 d6d d71 d75 d81 d85
d8a d8e d91 d92 d97 d9b d9f da2
da5 da9 dab daf db3 db6 db9 dbd
dbf dc3 dc7 dca dcc dd0 dd4 dd8
ddb ddf de3 de5 de6 deb 1 def
df3 df7 dfa dfe e02 e06 e0a e0d
e11 e15 e18 e1b e20 e23 e26 e29
e2a e2f e30 1 e32 e37 e3b e3f
e42 e47 e4c e4f e53 e54 e59 e5c
e61 e62 e67 e6a e6f e70 e75 e78
e7c e7d e82 e85 e8a e8b e90 e93
e97 e9b e9e ea1 ea2 ea4 ea5 eaa
eab eb0 eb2 eb6 eb9 ebb ebf ec1
ecd ecf ed1 ed2 ed7 edb edd ee9
eeb eef ef3 ef6 ef7 ef9 efc eff
f00 f05 f09 f0d f11 f12 f14 f18
f1c f28 f2a f2e f31 f33 f37 f3e
f42 f46 f4b f4d f51 f55 f58 f5c
f60 f62 f66 f6a f6e f71 f75 f79
f7b f7c f81 1 f85 f89 f8d f90
f94 f98 f9c fa0 fa3 fa7 fab fae
fb1 fb6 fb9 fbc fbf fc0 fc5 fc6
1 fc8 fcd fd1 fd5 fd8 fdd fe2
fe5 fe9 fea fef ff2 ff7 ff8 ffd
1000 1005 1006 100b 100e 1012 1013 1018
101b 1020 1021 1026 1029 102d 1031 1034
1037 1038 103a 103b 1040 1041 1046 1048
104c 104f 1051 1055 1057 1063 1065 1067
1068 106d 1071 1073 107f 1081 1083 1087
108b 1090 1094 1095 1099 109d 10a1 10a6
10ab 10af 10b3 10b8 10bc 10c0 10c3 10c6
10ca 10cc 10d0 10d4 10d7 10da 10de 10e0
10e4 10e8 10eb 10ef 10f3 10f6 10fa 10fd
1101 1104 1105 110a 110d 1111 1117 1119
111d 1120 1124 1128 112c 112f 1133 1137
113a 113e 1142 1145 1146 1148 114c 1150
1154 1158 115c 1160 1164 1170 1174 1179
117d 1180 1181 1186 118a 118e 1191 1194
1198 119a 119e 11a2 11a5 11a8 11ac 11ae
11b2 11b6 11b9 11bb 11bf 11c3 11c7 11ca
11ce 11d2 11d4 11d5 11da 1 11de 11e2
11e6 11e9 11ed 11f1 11f5 11f9 11fc 1200
1204 1207 120a 120f 1212 1215 1218 1219
121e 121f 1 1221 1226 122a 122e 1231
1236 123b 123e 1242 1243 1248 124b 1250
1251 1256 1259 125e 125f 1264 1267 126b
126c 1271 1274 1279 127a 127f 1282 1286
128a 128d 1290 1291 1293 1294 1299 129a
129f 12a1 12a5 12a8 12aa 12ae 12b0 12bc
12be 12c0 12c1 12c6 12ca 12cc 12d8 12da
12de 12e2 12e6 12ea 12ee 12f2 12f6 12fa
12fe 1302 1306 130a 130e 1312 131e 1322
1327 132b 132e 132f 1334 1338 133c 133f
1342 1346 1348 134c 1350 1353 1356 135a
135c 1360 1364 1367 1369 136d 1371 1375
1378 137c 1380 1382 1383 1388 1 138c
1390 1394 1397 139b 139f 13a3 13a7 13aa
13ae 13b2 13b5 13b8 13bd 13c0 13c3 13c6
13c7 13cc 13cd 1 13cf 13d4 13d8 13dc
13df 13e4 13e9 13ec 13f0 13f1 13f6 13f9
13fe 13ff 1404 1407 140c 140d 1412 1415
1419 141a 141f 1422 1427 1428 142d 1430
1434 1438 143b 143e 143f 1441 1442 1447
1448 144d 144f 1453 1456 1458 145c 145e
146a 146c 146e 146f 1474 1478 147a 1486
1488 148c 1490 1494 1498 149c 14a0 14a4
14a8 14ac 14b8 14bc 14c1 14c5 14c8 14c9
14ce 14d2 14d6 14d9 14dc 14e0 14e2 14e6
14ea 14ed 14f0 14f4 14f6 14fa 14fe 1501
1503 1507 150b 150f 1512 1516 151a 151c
151d 1522 1 1526 152a 152e 1531 1535
1539 153d 1541 1544 1548 154c 154f 1552
1557 155a 155d 1560 1561 1566 1567 1
1569 156e 1572 1576 1579 157e 1583 1586
158a 158b 1590 1593 1598 1599 159e 15a1
15a6 15a7 15ac 15af 15b3 15b4 15b9 15bc
15c1 15c2 15c7 15ca 15ce 15d2 15d5 15d8
15d9 15db 15dc 15e1 15e2 15e7 15e9 15ed
15f0 15f2 15f6 15f8 1604 1606 1608 1609
160e 1612 1614 1620 1622 1626 162a 162e
1632 1636 163a 163e 1642 1646 1652 1656
165b 165f 1662 1663 1668 166c 1670 1673
1676 167a 167c 1680 1684 1687 168a 168e
1690 1694 1698 169b 169d 16a1 16a5 16a9
16ac 16b0 16b4 16b6 16b7 16bc 1 16c0
16c4 16c8 16cb 16cf 16d3 16d7 16db 16de
16e2 16e6 16e9 16ec 16f1 16f4 16f7 16fa
16fb 1700 1701 1 1703 1708 170c 1710
1713 1718 171d 1720 1724 1725 172a 172d
1732 1733 1738 173b 1740 1741 1746 1749
174d 174e 1753 1756 175b 175c 1761 1764
1768 176c 176f 1772 1773 1775 1776 177b
177c 1781 1783 1787 178a 178c 1790 1792
179e 17a0 17a2 17a3 17a8 17ac 17ae 17ba
17bc 17c0 17c4 17c7 17c8 17ca 17cd 17d0
17d1 17d6 17da 17de 17e2 17e3 17e5 17e9
17ed 17f9 17fb 17ff 1802 1804 1808 180f
1813 1817 181c 181e 1822 1826 1829 182d
1831 1833 1837 183b 183f 1842 1846 184a
184c 184d 1852 1 1856 185a 185e 1861
1865 1869 186d 1871 1874 1878 187c 187f
1882 1887 188a 188d 1890 1891 1896 1897
1 1899 189e 18a2 18a6 18a9 18ae 18b3
18b6 18ba 18bb 18c0 18c3 18c8 18c9 18ce
18d1 18d6 18d7 18dc 18df 18e3 18e4 18e9
18ec 18f1 18f2 18f7 18fa 18fe 1902 1905
1908 1909 190b 190c 1911 1912 1917 1919
191d 1920 1922 1926 1928 1934 1936 1938
1939 193e 1942 1944 1950 1952 1956 195a
195f 1963 1964 1968 196c 1970 1975 197a
197e 1982 1987 198b 198f 1992 1995 1999
199b 199f 19a3 19a6 19a9 19ad 19af 19b3
19b7 19ba 19be 19c2 19c5 19c9 19cc 19d0
19d3 19d4 19d9 19dc 19e0 19e6 19e8 19ec
19ef 19f3 19f7 19fb 19ff 1a03 1a07 1a0b
1a0f 1a13 1a17 1a1b 1a1f 1a2b 1a2f 1a34
1a38 1a3b 1a3c 1a41 1a45 1a49 1a4c 1a4f
1a53 1a55 1a59 1a5d 1a60 1a63 1a67 1a69
1a6d 1a71 1a74 1a76 1a7a 1a7e 1a82 1a85
1a89 1a8d 1a8f 1a90 1a95 1 1a99 1a9d
1aa1 1aa4 1aa8 1aac 1ab0 1ab4 1ab7 1abb
1abf 1ac2 1ac5 1aca 1acd 1ad0 1ad3 1ad4
1ad9 1ada 1 1adc 1ae1 1ae5 1ae9 1aec
1af1 1af6 1af9 1afd 1afe 1b03 1b06 1b0b
1b0c 1b11 1b14 1b19 1b1a 1b1f 1b22 1b26
1b27 1b2c 1b2f 1b34 1b35 1b3a 1b3d 1b41
1b45 1b48 1b4b 1b4c 1b4e 1b4f 1b54 1b55
1b5a 1b5c 1b60 1b63 1b65 1b69 1b6b 1b77
1b79 1b7b 1b7c 1b81 1b85 1b87 1b93 1b95
1b99 1b9d 1ba0 1ba1 1ba3 1ba6 1ba9 1baa
1baf 1bb3 1bb7 1bbb 1bbc 1bbe 1bc2 1bc6
1bd2 1bd4 1bd8 1bdb 1bdd 1be1 1be8 1bec
1bf0 1bf5 1bf7 1bfb 1bff 1c02 1c06 1c0a
1c0c 1c10 1c14 1c18 1c1b 1c1f 1c23 1c25
1c26 1c2b 1 1c2f 1c33 1c37 1c3a 1c3e
1c42 1c46 1c4a 1c4d 1c51 1c55 1c58 1c5b
1c60 1c63 1c66 1c69 1c6a 1c6f 1c70 1
1c72 1c77 1c7b 1c7f 1c82 1c87 1c8c 1c8f
1c93 1c94 1c99 1c9c 1ca1 1ca2 1ca7 1caa
1caf 1cb0 1cb5 1cb8 1cbc 1cbd 1cc2 1cc5
1cca 1ccb 1cd0 1cd3 1cd7 1cdb 1cde 1ce1
1ce2 1ce4 1ce5 1cea 1ceb 1cf0 1cf2 1cf6
1cf9 1cfb 1cff 1d01 1d0d 1d0f 1d11 1d12
1d17 1d1b 1d1d 1d29 1d2b 1d2d 1d31 1d35
1d38 1d3c 1d40 1d43 1d47 1d4b 1d4e 1d51
1d56 1d59 1d5c 1d5f 1d60 1d65 1d66 1
1d68 1d6d 1d71 1d75 1d78 1d7d 1d82 1d85
1d89 1d8a 1d8f 1d92 1d97 1d98 1d9d 1da0
1da5 1da6 1dab 1dae 1db2 1db6 1db9 1dba
1dbf 1dc0 1dc5 1dc7 1dcb 1dce 1dd2 1dd6
1dda 1ddd 1de1 1de3 1de7 1de9 1df5 1df9
1dfb 1dfc 1e05 
853
2
0 1 a 6 16 1a 21 :2 6
1 3 a :2 1 4 :2 c 1b c
:2 4 :2 12 21 12 :2 4 :2 14 23 14
4 7 e 0 :2 7 11 1d 2b
10 1c 25 30 39 45 4e 5c
65 73 7c 13 a :6 7 e 0
:2 7 11 1d 2b 10 1c 25 30
39 45 4e 5c 65 73 7c 13
a :6 7 e 0 :2 7 11 1f 10
1c 25 30 39 45 4e 5c 65
73 7c 8f 9c 13 a :6 7 e
0 :2 7 21 f 10 1c 25 30
39 45 4e 5c 65 73 7c 8f
9c 13 a :6 7 :2 11 23 :2 2d 3a
:2 23 :3 7 :2 d 19 25 :2 2f 43 :2 25
54 5a 5d :2 54 6b 81 8 :2 6b
d 10 :2 6b 1d 20 :2 6b 25 28
:2 6b :2 7 4a :2 4 7 :2 11 23 :2 2d
39 3c 4c 4e 4f :2 4c :2 23 :3 7
:2 d 12 1e 5d 60 :2 1e :2 7 52
:2 4 :4 8 :2 7 c :4 7 10 19 27
:3 a d 11 :2 d :2 16 22 d 17
d :2 16 22 d :4 a e :2 17 23
20 :2 2c :2 20 :3 d 3c :3 a 1b a
14 29 41 5b 64 79 13 20
d 14 10 1c :3 1a 10 :2 19 25
10 21 10 :2 19 25 10 :4 d a
12 10 :2 19 25 10 20 :2 d :2 12
10 :2 19 25 10 13 :2 1d 2f :2 39
45 49 59 5b 5c :2 59 :2 2f :3 13
:2 19 1e 2a 69 6c :2 2a 79 7c
:2 2a 80 83 :2 2a 8f 92 :2 2a 9a
9d :2 2a a9 ac b3 bc bf :2 ac
:2 2a :2 13 5f :2 10 :2 d :4 19 :2 d a
:3 7 a 1b :2 24 2b :2 34 44 :2 4d
:2 1b a 14 20 2e 37 48 5f
:2 d 29 45 13 20 32 43 d
14 10 1c :3 1a 10 :2 19 25 10
21 10 :2 19 25 10 :4 d a 12
10 :2 19 25 10 20 :2 d :2 12 10
:2 19 25 10 13 :2 1d 2f :2 39 45
49 59 5b 5c :2 59 :2 2f :3 13 :2 19
1e 2a 69 6c :2 2a 79 7c :2 2a
80 83 :2 2a 8f 92 :2 2a 9a 9d
:2 2a a9 ac b3 bc bf :2 ac :2 2a
:2 13 5f :2 10 :2 d :4 19 :2 d a :3 7
14 20 2e 3d 54 13 20 33
44 d 14 10 1c :3 1a 10 :2 19
25 10 21 10 :2 19 25 10 :4 d
a 12 10 :2 19 25 10 20 :2 d
:2 12 10 :2 19 25 10 13 :2 1d 2f
:2 39 45 49 59 5c 5d :2 59 :2 2f
:3 13 :2 19 1e 2a 6a 6d :2 2a 7a
7d :2 2a 81 84 :2 2a 90 93 :2 2a
9b 9e :2 2a aa ad b4 bd c0
:2 ad :2 2a :2 13 60 :2 10 :2 d :4 19 :2 d
a :3 7 14 20 2e 37 48 5f
13 20 31 d 14 10 1c :3 1a
10 :2 19 25 10 21 10 :2 19 25
10 :4 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 59 5c 5d
:2 59 :2 2f :3 13 :2 19 1e 2a 6a 6d
:2 2a 7a 7d :2 2a 81 84 :2 2a 90
93 :2 2a 9b 9e :2 2a aa ad b4
bd c0 :2 ad :2 2a :2 13 60 :2 10 :2 d
:4 19 :2 d a :3 7 d 16 1f :2 d
22 24 :2 22 d 20 2a :2 20 :3 d
26 :2 a 7 b 1e a 10 :3 a
:2 13 1f a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 56 59
5a :2 56 :2 2c :3 10 :2 16 1b 27 67
6a :2 27 77 7a :2 27 7e 81 :2 27
8d 90 :2 27 98 9b :2 27 a7 aa
b1 ba bd :2 aa :2 27 :2 10 5d :2 d
:2 a :4 16 :2 a 7 :3 1e 7 c :4 7
10 19 27 :3 a d 11 :2 d :2 16
22 d 17 d :2 16 22 d :4 a
e :2 17 23 20 :2 2c :2 20 :3 d 3c
:2 a 14 20 2e 37 48 d 29
45 13 20 33 44 d 14 10
1c :3 1a 10 :2 19 25 10 21 10
:2 19 25 10 :4 d a 12 10 :2 19
25 10 20 :2 d :2 12 10 :2 19 25
10 13 :2 1d 2f :2 39 45 49 59
5c 5d :2 59 :2 2f :3 13 :2 19 1e 2a
6a 6d :2 2a 7a 7d :2 2a 81 84
:2 2a 90 93 :2 2a 9b 9e :2 2a aa
ad b4 bd c0 :2 ad :2 2a :2 13 60
:2 10 :2 d :4 19 :2 d a :3 7 d 16
1f :2 d 22 24 :2 22 d 20 2a
:2 20 :3 d 26 :2 a 7 b 1e a
10 :3 a :2 13 1f a 7 f d
:2 16 22 d 1d :2 a :2 f d :2 16
22 d 10 :2 1a 2c :2 36 42 46
56 59 5a :2 56 :2 2c :3 10 :2 16 1b
27 67 6a :2 27 77 7a :2 27 7e
81 :2 27 8d 90 :2 27 98 9b :2 27
a7 aa b1 ba bd :2 aa :2 27 :2 10
5d :2 d :2 a :4 16 :2 a 7 :4 1e 7
c :4 7 10 19 :2 a d 11 :2 d
:2 16 22 d 17 d :2 16 22 d
:4 a e :2 17 23 20 :2 2c :2 20 :3 d
3c :3 a 1b :2 24 2b :2 34 44 :2 4d
:2 1b a 14 29 37 40 56 63
d 14 10 1c :3 1a 10 :2 19 25
10 21 10 :2 19 25 10 :4 d a
12 10 :2 19 25 10 20 :2 d :2 12
10 :2 19 25 10 13 :2 1d 2f :2 39
45 49 59 5c 5d :2 59 :2 2f :3 13
:2 19 1e 2a 6a 6d :2 2a 7a 7d
:2 2a 81 84 :2 2a 90 93 :2 2a 9b
9e :2 2a aa ad b4 bd c0 :2 ad
:2 2a :2 13 60 :2 10 :2 d :4 19 :2 d a
:3 7 14 20 2e 37 48 5f :2 d
29 45 13 20 32 43 d 14
10 1c :3 1a 10 :2 19 25 10 21
10 :2 19 25 10 :4 d a 12 10
:2 19 25 10 20 :2 d :2 12 10 :2 19
25 10 13 :2 1d 2f :2 39 45 49
59 5c 5d :2 59 :2 2f :3 13 :2 19 1e
2a 6a 6d :2 2a 7a 7d :2 2a 81
84 :2 2a 90 93 :2 2a 9b 9e :2 2a
aa ad b4 bd c0 :2 ad :2 2a :2 13
60 :2 10 :2 d :4 19 :2 d a :3 7 14
20 2e 3d 54 13 20 33 44
d 14 10 1c :3 1a 10 :2 19 25
10 21 10 :2 19 25 10 :4 d a
12 10 :2 19 25 10 20 :2 d :2 12
10 :2 19 25 10 13 :2 1d 2f :2 39
45 49 59 5c 5d :2 59 :2 2f :3 13
:2 19 1e 2a 6a 6d :2 2a 7a 7d
:2 2a 81 84 :2 2a 90 93 :2 2a 9b
9e :2 2a aa ad b4 bd c0 :2 ad
:2 2a :2 13 60 :2 10 :2 d :4 19 :2 d a
:3 7 14 20 2e 37 48 5f 13
20 31 d 14 10 1c :3 1a 10
:2 19 25 10 21 10 :2 19 25 10
:4 d a 12 10 :2 19 25 10 20
:2 d :2 12 10 :2 19 25 10 13 :2 1d
2f :2 39 45 49 59 5c 5d :2 59
:2 2f :3 13 :2 19 1e 2a 6a 6d :2 2a
7a 7d :2 2a 81 84 :2 2a 90 93
:2 2a 9b 9e :2 2a aa ad b4 bd
c0 :2 ad :2 2a :2 13 60 :2 10 :2 d :4 19
:2 d a :3 7 d 16 1f :2 d 22
24 :2 22 d 20 2a :2 20 :3 d 26
:2 a 7 b 4 a 10 :3 a :2 13
1f a 7 f d :2 16 22 d
1d :2 a :2 f d :2 16 22 d 10
:2 1a 2c :2 36 42 46 56 59 5a
:2 56 :2 2c :3 10 :2 16 1b 27 67 6a
:2 27 77 7a :2 27 7e 81 :2 27 8d
90 :2 27 98 9b :2 27 a7 aa b1
ba bd :2 aa :2 27 :2 10 5d :2 d :2 a
:4 16 :2 a 7 :3 4 7 c :4 7 10
19 :2 a d 11 :2 d :2 16 22 d
17 d :2 16 22 d :4 a e :2 17
23 20 :2 2c :2 20 :3 d 3c :2 a 14
20 2e 37 48 d 29 45 13
20 33 44 d 14 10 1c :3 1a
10 :2 19 25 10 21 10 :2 19 25
10 :4 d a 12 10 :2 19 25 10
20 :2 d :2 12 10 :2 19 25 10 13
:2 1d 2f :2 39 45 49 59 5c 5d
:2 59 :2 2f :3 13 :2 19 1e 2a 6a 6d
:2 2a 7a 7d :2 2a 81 84 :2 2a 90
93 :2 2a 9b 9e :2 2a aa ad b4
bd c0 :2 ad :2 2a :2 13 60 :2 10 :2 d
:4 19 :2 d a :3 7 d 16 1f :2 d
22 24 :2 22 d 20 2a :2 20 :3 d
26 :2 a 7 b 4 a 10 :3 a
:2 13 1f a 7 f d :2 16 22
d 1d :2 a :2 f d :2 16 22 d
10 :2 1a 2c :2 36 42 46 56 59
5a :2 56 :2 2c :3 10 :2 16 1b 27 67
6a :2 27 77 7a :2 27 7e 81 :2 27
8d 90 :2 27 98 9b :2 27 a7 aa
b1 ba bd :2 aa :2 27 :2 10 5d :2 d
:2 a :4 16 :2 a 7 :7 4 7 :2 11 23
:2 2d 39 3c 4c 4f 50 :2 4c :2 23
:3 7 :2 d 12 1e 5e 61 :2 1e 6e
71 :2 1e 75 78 :2 1e 82 85 :2 8e
:2 1e :2 7 53 :3 4 b :2 14 4 :9 1

853
4
0 :2 1 :6 3 2
:2 4 :2 1 :6 6 :6 7
:6 8 :2 a 0 :2 a
:3 b :b c d b
:5 a :2 f 0 :2 f
:3 10 :b 11 12 10
:5 f :2 14 0 :2 14
:2 15 :d 16 17 15
:5 14 :2 19 0 :2 19
1a 1b :d 1c 1d
1a :5 19 :b 20 :11 21
22 :2 21 :2 22 :2 21
:2 22 :2 21 :2 22 :4 21
:3 20 :11 25 :b 26 :3 25
:5 29 :5 2c 2e :6 2f
:3 30 :5 31 30 :5 33
32 :3 30 :a 35 :2 36
:3 35 :3 39 :6 3d :2 3e
3d :6 3f :5 40 3f
:5 42 41 :3 3f 3c
45 :5 46 :3 45 :2 47
:5 49 :11 4a :24 4b :3 4a
:2 48 :6 47 44 :3 2e
:d 50 :6 53 54 :3 55
:4 56 53 :6 57 :5 58
57 :5 5a 59 :3 57
52 5d :5 5e :3 5d
:2 5f :5 61 :11 62 :24 63
:3 62 :2 60 :6 5f 5c
:3 2e :5 69 :4 6a 69
:6 6b :5 6c 6b :5 6e
6d :3 6b 68 71
:5 72 :3 71 :2 73 :5 75
:11 76 :24 77 :3 76 :2 74
:6 73 70 :3 2e :6 7d
:3 7e 7d :6 7f :5 80
7f :5 82 81 :3 7f
7c 85 :5 86 :3 85
:2 87 :5 89 :11 8a :24 8b
:3 8a :2 88 :6 87 84
:3 2e :9 90 :6 91 :2 93
:3 90 2e 95 29
:4 98 :5 99 97 9b
:5 9c :3 9b :2 9d :5 9f
:11 a0 :24 a1 :3 a0 :2 9e
:6 9d 9a :3 29 :5 a8
aa :6 ab :3 ac :5 ad
ac :5 af ae :3 ac
:a b2 :2 b3 :3 b2 :5 b7
:3 b8 :4 b9 b7 :6 ba
:5 bb ba :5 bd bc
:3 ba b6 c0 :5 c1
:3 c0 :2 c2 :5 c4 :11 c5
:24 c6 :3 c5 :2 c3 :6 c2
bf :3 aa :9 cb :6 cc
:2 ce :3 cb aa d0
29 :4 d3 :5 d4 d2
d6 :5 d7 :3 d6 :2 d8
:5 da :11 db :24 dc :3 db
:2 d9 :6 d8 d5 :4 29
:5 e3 e5 :4 e6 :3 e7
:5 e8 e7 :5 ea e9
:3 e7 :a ed :2 ee :3 ed
:d f1 :7 f4 :6 f5 :5 f6
f5 :5 f8 f7 :3 f5
f3 fb :5 fc :3 fb
:2 fd :5 ff :11 100 :24 101
:3 100 :2 fe :6 fd fa
:3 e5 :6 107 108 :3 109
:4 10a 107 :6 10b :5 10c
10b :5 10e 10d :3 10b
106 111 :5 112 :3 111
:2 113 :5 115 :11 116 :24 117
:3 116 :2 114 :6 113 110
:3 e5 :5 11d :4 11e 11d
:6 11f :5 120 11f :5 122
121 :3 11f 11c 125
:5 126 :3 125 :2 127 :5 129
:11 12a :24 12b :3 12a :2 128
:6 127 124 :3 e5 :6 131
:3 132 131 :6 133 :5 134
133 :5 136 135 :3 133
130 139 :5 13a :3 139
:2 13b :5 13d :11 13e :24 13f
:3 13e :2 13c :6 13b 138
:3 e5 :9 144 :6 145 :2 147
:3 144 e5 149 e0
:4 14c :5 14d 14b 14f
:5 150 :3 14f :2 151 :5 153
:11 154 :24 155 :3 154 :2 152
:6 151 14e :3 e0 :5 15c
15e :4 15f :3 160 :5 161
160 :5 163 162 :3 160
:a 166 :2 167 :3 166 :5 16b
:3 16c :4 16d 16b :6 16e
:5 16f 16e :5 171 170
:3 16e 16a 174 :5 175
:3 174 :2 176 :5 178 :11 179
:24 17a :3 179 :2 177 :6 176
173 :3 15e :9 17f :6 180
:2 182 :3 17f 15e 184
e0 :4 187 :5 188 186
18a :5 18b :3 18a :2 18c
:5 18e :11 18f :24 190 :3 18f
:2 18d :6 18c 189 :4 e0
:3 29 :11 196 :19 197 :3 196
:5 199 :2 1f :7 1 
1e07
4
:3 0 1 :3 0 2
:a 0 84e 1 :7 0
5 :2 0 3 4
:3 0 5 :3 0 6
:3 0 3 :5 0 1
7 6 :3 0 7
:3 0 8 :3 0 9
b 0 84e 2
c :2 0 a :2 0
7 8 :3 0 f
:7 0 a :2 0 13
10 11 84c 0
9 :6 0 a :2 0
9 8 :3 0 15
:7 0 19 16 17
84c 0 b :9 0
b 8 :3 0 1b
:7 0 1f 1c 1d
84c 0 c :6 0
d :3 0 e :a 0
2 35 :4 0 21
24 0 22 :3 0
f :3 0 10 :3 0
11 :3 0 12 :3 0
13 :3 0 14 :3 0
13 :3 0 15 :3 0
13 :3 0 16 :3 0
13 :3 0 17 :3 0
13 :3 0 18 :3 0
19 :4 0 1a 1
:8 0 36 21 24
37 0 84c d
37 39 36 38
:6 0 35 :7 0 37
d :3 0 1b :a 0
3 4f :5 0 3b
3e 0 3c :3 0
f :3 0 10 :3 0
11 :3 0 12 :3 0
13 :3 0 14 :3 0
13 :3 0 15 :3 0
13 :3 0 16 :3 0
13 :3 0 17 :3 0
13 :3 0 18 :3 0
19 :4 0 1a 1
:8 0 50 3b 3e
51 0 84c f
51 53 50 52
:6 0 4f :7 0 51
d :3 0 1c :a 0
4 6a :5 0 55
58 0 56 :3 0
10 :3 0 11 :3 0
12 :3 0 13 :3 0
14 :3 0 13 :3 0
15 :3 0 13 :3 0
16 :3 0 13 :3 0
17 :3 0 13 :3 0
18 :3 0 1d :3 0
3 :3 0 19 :4 0
1e 1 :8 0 6b
55 58 6c 0
84c 11 6c 6e
6b 6d :6 0 6a
:7 0 6c d :3 0
1f :a 0 5 85
:5 0 70 73 0
71 :3 0 10 :3 0
11 :3 0 12 :3 0
13 :3 0 14 :3 0
13 :3 0 15 :3 0
13 :3 0 16 :3 0
13 :3 0 17 :3 0
13 :3 0 18 :3 0
1d :3 0 3 :3 0
19 :4 0 20 1
:8 0 86 70 73
87 0 84c 13
87 89 86 88
:6 0 85 :7 0 87
21 :3 0 22 :3 0
8a 8b 0 21
:3 0 23 :3 0 8d
8e 0 2 :4 0
15 8f 91 8c
93 92 :2 0 24
:3 0 25 :3 0 95
96 0 26 :4 0
21 :3 0 27 :3 0
99 9a 0 2
:4 0 17 9b 9d
28 :4 0 29 :2 0
2a :3 0 19 a0
a2 :3 0 2b :4 0
29 :2 0 2c :4 0
1c a5 a7 :3 0
29 :2 0 3 :3 0
1f a9 ab :3 0
29 :2 0 2c :4 0
22 ad af :3 0
29 :2 0 2d :4 0
25 b1 b3 :3 0
28 97 b5 :2 0
b7 2d b8 94
b7 0 b9 2f
0 84a 21 :3 0
22 :3 0 ba bb
0 21 :3 0 2e
:3 0 bd be 0
2f :2 0 2 :4 0
30 :2 0 31 :2 0
2f :2 0 31 c3
c5 :3 0 34 bf
c7 bc c9 c8
:2 0 24 :3 0 32
:3 0 cb cc 0
26 :4 0 33 :4 0
29 :2 0 2a :3 0
38 d0 d2 :3 0
3b cd d4 :2 0
d6 3e d7 ca
d6 0 d8 40
0 84a 3 :3 0
34 :2 0 42 da
db :3 0 dc :2 0
35 :3 0 e :4 0
e1 :2 0 47d df
e2 :3 0 36 :3 0
e :3 0 3 :3 0
9 :4 0 e8 :2 0
303 e4 e9 :3 0
44 :3 0 e :3 0
37 :3 0 ea eb
:3 0 38 :3 0 39
:3 0 ed ee 0
a :2 0 ef f0
0 f2 47 f9
38 :3 0 39 :3 0
f3 f4 0 3a
:2 0 f5 f6 0
f8 49 fa ec
f2 0 fb 0
f8 0 fb 4b
0 303 38 :3 0
39 :3 0 fc fd
0 38 :3 0 3b
:2 0 3c :3 0 ff
101 0 50 100
103 :3 0 104 :2 0
3d :8 0 108 53
109 105 108 0
10a 55 0 303
b :3 0 3e :2 0
10b 10c 0 303
3f :3 0 40 :3 0
41 :3 0 13 :3 0
42 :3 0 b :3 0
f :3 0 3 :4 0
43 1 :8 0 12c
37 :4 0 117 :3 0
44 :3 0 45 :2 0
59 11a 11b :3 0
38 :3 0 39 :3 0
11d 11e 0 a
:2 0 11f 120 0
122 5c 129 38
:3 0 39 :3 0 123
124 0 3a :2 0
125 126 0 128
5e 12a 11c 122
0 12b 0 128
0 12b 60 0
12c 63 17f 46
:3 0 38 :3 0 39
:3 0 12e 12f 0
47 :3 0 130 131
0 133 66 135
68 134 133 :2 0
17d 48 :3 0 38
:3 0 39 :3 0 138
139 0 47 :3 0
13a 13b 0 175
21 :3 0 22 :3 0
13d 13e 0 21
:3 0 2e :3 0 140
141 0 49 :2 0
2 :4 0 4a :2 0
31 :2 0 2f :2 0
6a 146 148 :3 0
6d 142 14a 13f
14c 14b :2 0 24
:3 0 32 :3 0 14e
14f 0 26 :4 0
4b :4 0 29 :2 0
2a :3 0 71 153
155 :3 0 29 :2 0
4c :4 0 74 157
159 :3 0 29 :2 0
4d :4 0 77 15b
15d :3 0 29 :2 0
47 :3 0 7a 15f
161 :3 0 29 :2 0
4e :4 0 7d 163
165 :3 0 29 :2 0
4f :3 0 50 :3 0
2f :2 0 51 :2 0
80 168 16c 84
167 16e :3 0 87
150 170 :2 0 172
8a 173 14d 172
0 174 8c 0
175 8e 178 :3 0
178 0 178 177
175 176 :6 0 17a
7 :3 0 91 17c
93 17b 17a :2 0
17d 95 :2 0 17f
0 17f 17e 12c
17d :6 0 303 6
:3 0 b :3 0 52
:3 0 53 :3 0 182
183 0 13 :3 0
54 :3 0 185 186
0 13 :3 0 55
:3 0 188 189 0
98 184 18b 181
18c 0 303 11
:3 0 40 :3 0 13
:3 0 42 :3 0 56
:3 0 57 :3 0 58
:3 0 59 :3 0 5a
:3 0 59 :3 0 f
:3 0 3 :3 0 5b
:3 0 5c :4 0 5d
1 :8 0 1b2 37
:4 0 19d :3 0 44
:3 0 45 :2 0 9d
1a0 1a1 :3 0 38
:3 0 39 :3 0 1a3
1a4 0 a :2 0
1a5 1a6 0 1a8
a0 1af 38 :3 0
39 :3 0 1a9 1aa
0 3a :2 0 1ab
1ac 0 1ae a2
1b0 1a2 1a8 0
1b1 0 1ae 0
1b1 a4 0 1b2
a7 205 46 :3 0
38 :3 0 39 :3 0
1b4 1b5 0 47
:3 0 1b6 1b7 0
1b9 aa 1bb ac
1ba 1b9 :2 0 203
48 :3 0 38 :3 0
39 :3 0 1be 1bf
0 47 :3 0 1c0
1c1 0 1fb 21
:3 0 22 :3 0 1c3
1c4 0 21 :3 0
2e :3 0 1c6 1c7
0 49 :2 0 2
:4 0 5e :2 0 31
:2 0 2f :2 0 ae
1cc 1ce :3 0 b1
1c8 1d0 1c5 1d2
1d1 :2 0 24 :3 0
32 :3 0 1d4 1d5
0 26 :4 0 5f
:4 0 29 :2 0 2a
:3 0 b5 1d9 1db
:3 0 29 :2 0 4c
:4 0 b8 1dd 1df
:3 0 29 :2 0 4d
:4 0 bb 1e1 1e3
:3 0 29 :2 0 47
:3 0 be 1e5 1e7
:3 0 29 :2 0 4e
:4 0 c1 1e9 1eb
:3 0 29 :2 0 4f
:3 0 50 :3 0 2f
:2 0 51 :2 0 c4
1ee 1f2 c8 1ed
1f4 :3 0 cb 1d6
1f6 :2 0 1f8 ce
1f9 1d3 1f8 0
1fa d0 0 1fb
d2 1fe :3 0 1fe
0 1fe 1fd 1fb
1fc :6 0 200 9
:3 0 d5 202 d7
201 200 :2 0 203
d9 :2 0 205 0
205 204 1b2 203
:6 0 303 6 :3 0
11 :3 0 40 :3 0
b :3 0 56 :3 0
57 :3 0 f :3 0
3 :3 0 5b :3 0
5c :4 0 60 1
:8 0 226 37 :4 0
211 :3 0 44 :3 0
45 :2 0 de 214
215 :3 0 38 :3 0
39 :3 0 217 218
0 a :2 0 219
21a 0 21c e1
223 38 :3 0 39
:3 0 21d 21e 0
3a :2 0 21f 220
0 222 e3 224
216 21c 0 225
0 222 0 225
e5 0 226 e8
279 46 :3 0 38
:3 0 39 :3 0 228
229 0 47 :3 0
22a 22b 0 22d
eb 22f ed 22e
22d :2 0 277 48
:3 0 38 :3 0 39
:3 0 232 233 0
47 :3 0 234 235
0 26f 21 :3 0
22 :3 0 237 238
0 21 :3 0 2e
:3 0 23a 23b 0
49 :2 0 2 :4 0
61 :2 0 31 :2 0
2f :2 0 ef 240
242 :3 0 f2 23c
244 239 246 245
:2 0 24 :3 0 32
:3 0 248 249 0
26 :4 0 62 :4 0
29 :2 0 2a :3 0
f6 24d 24f :3 0
29 :2 0 4c :4 0
f9 251 253 :3 0
29 :2 0 4d :4 0
fc 255 257 :3 0
29 :2 0 47 :3 0
ff 259 25b :3 0
29 :2 0 4e :4 0
102 25d 25f :3 0
29 :2 0 4f :3 0
50 :3 0 2f :2 0
51 :2 0 105 262
266 109 261 268
:3 0 10c 24a 26a
:2 0 26c 10f 26d
247 26c 0 26e
111 0 26f 113
272 :3 0 272 0
272 271 26f 270
:6 0 274 b :3 0
116 276 118 275
274 :2 0 277 11a
:2 0 279 0 279
278 226 277 :6 0
303 6 :3 0 63
:3 0 40 :3 0 13
:3 0 42 :3 0 56
:3 0 57 :3 0 f
:3 0 3 :3 0 12
:4 0 64 1 :8 0
29a 37 :4 0 285
:3 0 44 :3 0 45
:2 0 11f 288 289
:3 0 38 :3 0 39
:3 0 28b 28c 0
a :2 0 28d 28e
0 290 122 297
38 :3 0 39 :3 0
291 292 0 3a
:2 0 293 294 0
296 124 298 28a
290 0 299 0
296 0 299 126
0 29a 129 2ed
46 :3 0 38 :3 0
39 :3 0 29c 29d
0 47 :3 0 29e
29f 0 2a1 12c
2a3 12e 2a2 2a1
:2 0 2eb 48 :3 0
38 :3 0 39 :3 0
2a6 2a7 0 47
:3 0 2a8 2a9 0
2e3 21 :3 0 22
:3 0 2ab 2ac 0
21 :3 0 2e :3 0
2ae 2af 0 49
:2 0 2 :4 0 65
:2 0 31 :2 0 2f
:2 0 130 2b4 2b6
:3 0 133 2b0 2b8
2ad 2ba 2b9 :2 0
24 :3 0 32 :3 0
2bc 2bd 0 26
:4 0 66 :4 0 29
:2 0 2a :3 0 137
2c1 2c3 :3 0 29
:2 0 4c :4 0 13a
2c5 2c7 :3 0 29
:2 0 4d :4 0 13d
2c9 2cb :3 0 29
:2 0 47 :3 0 140
2cd 2cf :3 0 29
:2 0 4e :4 0 143
2d1 2d3 :3 0 29
:2 0 4f :3 0 50
:3 0 2f :2 0 51
:2 0 146 2d6 2da
14a 2d5 2dc :3 0
14d 2be 2de :2 0
2e0 150 2e1 2bb
2e0 0 2e2 152
0 2e3 154 2e6
:3 0 2e6 0 2e6
2e5 2e3 2e4 :6 0
2e8 d :3 0 157
2ea 159 2e9 2e8
:2 0 2eb 15b :2 0
2ed 0 2ed 2ec
29a 2eb :6 0 303
6 :3 0 67 :3 0
9 :3 0 a :2 0
15e 2ef 2f2 45
:2 0 a :2 0 163
2f4 2f6 :3 0 c
:3 0 68 :3 0 3
:3 0 166 2f9 2fb
2f8 2fc 0 300
69 :4 0 69 1
:8 0 300 168 301
2f7 300 0 302
16b 0 303 16d
305 36 :4 0 303
:4 0 47d 6a :3 0
e :4 0 309 :2 0
30f 307 0 38
:3 0 39 :3 0 30a
30b 0 47 :3 0
30c 30d 0 30f
178 362 46 :3 0
38 :3 0 39 :3 0
311 312 0 47
:3 0 313 314 0
316 17b 318 17d
317 316 :2 0 360
48 :3 0 38 :3 0
39 :3 0 31b 31c
0 47 :3 0 31d
31e 0 358 21
:3 0 22 :3 0 320
321 0 21 :3 0
2e :3 0 323 324
0 49 :2 0 2
:4 0 6b :2 0 31
:2 0 2f :2 0 17f
329 32b :3 0 182
325 32d 322 32f
32e :2 0 24 :3 0
32 :3 0 331 332
0 26 :4 0 6c
:4 0 29 :2 0 2a
:3 0 186 336 338
:3 0 29 :2 0 4c
:4 0 189 33a 33c
:3 0 29 :2 0 4d
:4 0 18c 33e 340
:3 0 29 :2 0 47
:3 0 18f 342 344
:3 0 29 :2 0 4e
:4 0 192 346 348
:3 0 29 :2 0 4f
:3 0 50 :3 0 2f
:2 0 51 :2 0 195
34b 34f 199 34a
351 :3 0 19c 333
353 :2 0 355 19f
356 330 355 0
357 1a1 0 358
1a3 35b :3 0 35b
0 35b 35a 358
359 :6 0 35d f
:3 0 1a6 35f 1a8
35e 35d :2 0 360
1aa :2 0 362 0
362 361 30f 360
:6 0 47d 1 :3 0
35 :3 0 1b :4 0
367 :2 0 47d 365
368 :3 0 36 :3 0
1b :3 0 3 :3 0
9 :4 0 36e :2 0
41c 36a 36f :3 0
1ad :3 0 1b :3 0
37 :3 0 370 371
:3 0 38 :3 0 39
:3 0 373 374 0
a :2 0 375 376
0 378 1b0 37f
38 :3 0 39 :3 0
379 37a 0 3a
:2 0 37b 37c 0
37e 1b2 380 372
378 0 381 0
37e 0 381 1b4
0 41c 38 :3 0
39 :3 0 382 383
0 38 :3 0 3b
:2 0 3c :3 0 385
387 0 1b9 386
389 :3 0 38a :2 0
3d :8 0 38e 1bc
38f 38b 38e 0
390 1be 0 41c
11 :3 0 40 :3 0
13 :3 0 42 :3 0
58 :3 0 59 :3 0
5a :3 0 59 :3 0
f :3 0 3 :3 0
5b :3 0 5c :4 0
6d 1 :8 0 3b3
37 :4 0 39e :3 0
44 :3 0 45 :2 0
1c2 3a1 3a2 :3 0
38 :3 0 39 :3 0
3a4 3a5 0 a
:2 0 3a6 3a7 0
3a9 1c5 3b0 38
:3 0 39 :3 0 3aa
3ab 0 3a :2 0
3ac 3ad 0 3af
1c7 3b1 3a3 3a9
0 3b2 0 3af
0 3b2 1c9 0
3b3 1cc 406 46
:3 0 38 :3 0 39
:3 0 3b5 3b6 0
47 :3 0 3b7 3b8
0 3ba 1cf 3bc
1d1 3bb 3ba :2 0
404 48 :3 0 38
:3 0 39 :3 0 3bf
3c0 0 47 :3 0
3c1 3c2 0 3fc
21 :3 0 22 :3 0
3c4 3c5 0 21
:3 0 2e :3 0 3c7
3c8 0 49 :2 0
2 :4 0 6e :2 0
31 :2 0 2f :2 0
1d3 3cd 3cf :3 0
1d6 3c9 3d1 3c6
3d3 3d2 :2 0 24
:3 0 32 :3 0 3d5
3d6 0 26 :4 0
6f :4 0 29 :2 0
2a :3 0 1da 3da
3dc :3 0 29 :2 0
4c :4 0 1dd 3de
3e0 :3 0 29 :2 0
4d :4 0 1e0 3e2
3e4 :3 0 29 :2 0
47 :3 0 1e3 3e6
3e8 :3 0 29 :2 0
4e :4 0 1e6 3ea
3ec :3 0 29 :2 0
4f :3 0 50 :3 0
2f :2 0 51 :2 0
1e9 3ef 3f3 1ed
3ee 3f5 :3 0 1f0
3d7 3f7 :2 0 3f9
1f3 3fa 3d4 3f9
0 3fb 1f5 0
3fc 1f7 3ff :3 0
3ff 0 3ff 3fe
3fc 3fd :6 0 401
12 :3 0 1fa 403
1fc 402 401 :2 0
404 1fe :2 0 406
0 406 405 3b3
404 :6 0 41c 11
:3 0 67 :3 0 9
:3 0 a :2 0 201
408 40b 45 :2 0
a :2 0 206 40d
40f :3 0 c :3 0
68 :3 0 3 :3 0
209 412 414 411
415 0 419 69
:4 0 69 1 :8 0
419 20b 41a 410
419 0 41b 20e
0 41c 210 41e
36 :4 0 41c :4 0
47d 6a :3 0 1b
:4 0 422 :2 0 428
420 0 38 :3 0
39 :3 0 423 424
0 47 :3 0 425
426 0 428 216
47b 46 :3 0 38
:3 0 39 :3 0 42a
42b 0 47 :3 0
42c 42d 0 42f
219 431 21b 430
42f :2 0 479 48
:3 0 38 :3 0 39
:3 0 434 435 0
47 :3 0 436 437
0 471 21 :3 0
22 :3 0 439 43a
0 21 :3 0 2e
:3 0 43c 43d 0
49 :2 0 2 :4 0
70 :2 0 31 :2 0
2f :2 0 21d 442
444 :3 0 220 43e
446 43b 448 447
:2 0 24 :3 0 32
:3 0 44a 44b 0
26 :4 0 71 :4 0
29 :2 0 2a :3 0
224 44f 451 :3 0
29 :2 0 4c :4 0
227 453 455 :3 0
29 :2 0 4d :4 0
22a 457 459 :3 0
29 :2 0 47 :3 0
22d 45b 45d :3 0
29 :2 0 4e :4 0
230 45f 461 :3 0
29 :2 0 4f :3 0
50 :3 0 2f :2 0
51 :2 0 233 464
468 237 463 46a
:3 0 23a 44c 46c
:2 0 46e 23d 46f
449 46e 0 470
23f 0 471 241
474 :3 0 474 0
474 473 471 472
:6 0 476 14 :3 0
244 478 246 477
476 :2 0 479 248
:2 0 47b 0 47b
47a 428 479 :6 0
47d 1 :3 0 24b
815 35 :3 0 1c
:4 0 481 :2 0 814
47f 482 :3 0 36
:3 0 1c :3 0 9
:4 0 487 :2 0 69c
484 485 :3 0 1c
:3 0 37 :3 0 488
489 :3 0 38 :3 0
39 :3 0 48b 48c
0 a :2 0 48d
48e 0 490 252
497 38 :3 0 39
:3 0 491 492 0
3a :2 0 493 494
0 496 254 498
48a 490 0 499
0 496 0 499
256 0 69c 38
:3 0 39 :3 0 49a
49b 0 38 :3 0
3b :2 0 3c :3 0
49d 49f 0 25b
49e 4a1 :3 0 4a2
:2 0 3d :8 0 4a6
25e 4a7 4a3 4a6
0 4a8 260 0
69c b :3 0 52
:3 0 53 :3 0 4aa
4ab 0 13 :3 0
54 :3 0 4ad 4ae
0 13 :3 0 55
:3 0 4b0 4b1 0
262 4ac 4b3 4a9
4b4 0 69c 3f
:3 0 40 :3 0 13
:3 0 42 :3 0 f
:3 0 3 :4 0 72
1 :8 0 4d2 37
:4 0 4bd :3 0 44
:3 0 45 :2 0 267
4c0 4c1 :3 0 38
:3 0 39 :3 0 4c3
4c4 0 a :2 0
4c5 4c6 0 4c8
26a 4cf 38 :3 0
39 :3 0 4c9 4ca
0 3a :2 0 4cb
4cc 0 4ce 26c
4d0 4c2 4c8 0
4d1 0 4ce 0
4d1 26e 0 4d2
271 525 46 :3 0
38 :3 0 39 :3 0
4d4 4d5 0 47
:3 0 4d6 4d7 0
4d9 274 4db 276
4da 4d9 :2 0 523
48 :3 0 38 :3 0
39 :3 0 4de 4df
0 47 :3 0 4e0
4e1 0 51b 21
:3 0 22 :3 0 4e3
4e4 0 21 :3 0
2e :3 0 4e6 4e7
0 49 :2 0 2
:4 0 73 :2 0 31
:2 0 2f :2 0 278
4ec 4ee :3 0 27b
4e8 4f0 4e5 4f2
4f1 :2 0 24 :3 0
32 :3 0 4f4 4f5
0 26 :4 0 74
:4 0 29 :2 0 2a
:3 0 27f 4f9 4fb
:3 0 29 :2 0 4c
:4 0 282 4fd 4ff
:3 0 29 :2 0 4d
:4 0 285 501 503
:3 0 29 :2 0 47
:3 0 288 505 507
:3 0 29 :2 0 4e
:4 0 28b 509 50b
:3 0 29 :2 0 4f
:3 0 50 :3 0 2f
:2 0 51 :2 0 28e
50e 512 292 50d
514 :3 0 295 4f6
516 :2 0 518 298
519 4f3 518 0
51a 29a 0 51b
29c 51e :3 0 51e
0 51e 51d 51b
51c :6 0 520 17
:3 0 29f 522 2a1
521 520 :2 0 523
2a3 :2 0 525 0
525 524 4d2 523
:6 0 69c 16 :3 0
11 :3 0 40 :3 0
13 :3 0 42 :3 0
56 :3 0 57 :3 0
58 :3 0 59 :3 0
5a :3 0 59 :3 0
f :3 0 3 :3 0
5b :3 0 5c :4 0
5d 1 :8 0 54b
37 :4 0 536 :3 0
44 :3 0 45 :2 0
2a8 539 53a :3 0
38 :3 0 39 :3 0
53c 53d 0 a
:2 0 53e 53f 0
541 2ab 548 38
:3 0 39 :3 0 542
543 0 3a :2 0
544 545 0 547
2ad 549 53b 541
0 54a 0 547
0 54a 2af 0
54b 2b2 59e 46
:3 0 38 :3 0 39
:3 0 54d 54e 0
47 :3 0 54f 550
0 552 2b5 554
2b7 553 552 :2 0
59c 48 :3 0 38
:3 0 39 :3 0 557
558 0 47 :3 0
559 55a 0 594
21 :3 0 22 :3 0
55c 55d 0 21
:3 0 2e :3 0 55f
560 0 49 :2 0
2 :4 0 75 :2 0
31 :2 0 2f :2 0
2b9 565 567 :3 0
2bc 561 569 55e
56b 56a :2 0 24
:3 0 32 :3 0 56d
56e 0 26 :4 0
76 :4 0 29 :2 0
2a :3 0 2c0 572
574 :3 0 29 :2 0
4c :4 0 2c3 576
578 :3 0 29 :2 0
4d :4 0 2c6 57a
57c :3 0 29 :2 0
47 :3 0 2c9 57e
580 :3 0 29 :2 0
4e :4 0 2cc 582
584 :3 0 29 :2 0
4f :3 0 50 :3 0
2f :2 0 51 :2 0
2cf 587 58b 2d3
586 58d :3 0 2d6
56f 58f :2 0 591
2d9 592 56c 591
0 593 2db 0
594 2dd 597 :3 0
597 0 597 596
594 595 :6 0 599
19 :3 0 2e0 59b
2e2 59a 599 :2 0
59c 2e4 :2 0 59e
0 59e 59d 54b
59c :6 0 69c 16
:3 0 11 :3 0 40
:3 0 b :3 0 56
:3 0 57 :3 0 f
:3 0 3 :3 0 5b
:3 0 5c :4 0 60
1 :8 0 5bf 37
:4 0 5aa :3 0 44
:3 0 45 :2 0 2e9
5ad 5ae :3 0 38
:3 0 39 :3 0 5b0
5b1 0 a :2 0
5b2 5b3 0 5b5
2ec 5bc 38 :3 0
39 :3 0 5b6 5b7
0 3a :2 0 5b8
5b9 0 5bb 2ee
5bd 5af 5b5 0
5be 0 5bb 0
5be 2f0 0 5bf
2f3 612 46 :3 0
38 :3 0 39 :3 0
5c1 5c2 0 47
:3 0 5c3 5c4 0
5c6 2f6 5c8 2f8
5c7 5c6 :2 0 610
48 :3 0 38 :3 0
39 :3 0 5cb 5cc
0 47 :3 0 5cd
5ce 0 608 21
:3 0 22 :3 0 5d0
5d1 0 21 :3 0
2e :3 0 5d3 5d4
0 49 :2 0 2
:4 0 77 :2 0 31
:2 0 2f :2 0 2fa
5d9 5db :3 0 2fd
5d5 5dd 5d2 5df
5de :2 0 24 :3 0
32 :3 0 5e1 5e2
0 26 :4 0 78
:4 0 29 :2 0 2a
:3 0 301 5e6 5e8
:3 0 29 :2 0 4c
:4 0 304 5ea 5ec
:3 0 29 :2 0 4d
:4 0 307 5ee 5f0
:3 0 29 :2 0 47
:3 0 30a 5f2 5f4
:3 0 29 :2 0 4e
:4 0 30d 5f6 5f8
:3 0 29 :2 0 4f
:3 0 50 :3 0 2f
:2 0 51 :2 0 310
5fb 5ff 314 5fa
601 :3 0 317 5e3
603 :2 0 605 31a
606 5e0 605 0
607 31c 0 608
31e 60b :3 0 60b
0 60b 60a 608
609 :6 0 60d 1b
:3 0 321 60f 323
60e 60d :2 0 610
325 :2 0 612 0
612 611 5bf 610
:6 0 69c 16 :3 0
63 :3 0 40 :3 0
13 :3 0 42 :3 0
56 :3 0 57 :3 0
f :3 0 3 :3 0
12 :4 0 64 1
:8 0 633 37 :4 0
61e :3 0 44 :3 0
45 :2 0 32a 621
622 :3 0 38 :3 0
39 :3 0 624 625
0 a :2 0 626
627 0 629 32d
630 38 :3 0 39
:3 0 62a 62b 0
3a :2 0 62c 62d
0 62f 32f 631
623 629 0 632
0 62f 0 632
331 0 633 334
686 46 :3 0 38
:3 0 39 :3 0 635
636 0 47 :3 0
637 638 0 63a
337 63c 339 63b
63a :2 0 684 48
:3 0 38 :3 0 39
:3 0 63f 640 0
47 :3 0 641 642
0 67c 21 :3 0
22 :3 0 644 645
0 21 :3 0 2e
:3 0 647 648 0
49 :2 0 2 :4 0
79 :2 0 31 :2 0
2f :2 0 33b 64d
64f :3 0 33e 649
651 646 653 652
:2 0 24 :3 0 32
:3 0 655 656 0
26 :4 0 7a :4 0
29 :2 0 2a :3 0
342 65a 65c :3 0
29 :2 0 4c :4 0
345 65e 660 :3 0
29 :2 0 4d :4 0
348 662 664 :3 0
29 :2 0 47 :3 0
34b 666 668 :3 0
29 :2 0 4e :4 0
34e 66a 66c :3 0
29 :2 0 4f :3 0
50 :3 0 2f :2 0
51 :2 0 351 66f
673 355 66e 675
:3 0 358 657 677
:2 0 679 35b 67a
654 679 0 67b
35d 0 67c 35f
67f :3 0 67f 0
67f 67e 67c 67d
:6 0 681 1d :3 0
362 683 364 682
681 :2 0 684 366
:2 0 686 0 686
685 633 684 :6 0
69c 16 :3 0 67
:3 0 9 :3 0 a
:2 0 369 688 68b
45 :2 0 a :2 0
36e 68d 68f :3 0
c :3 0 68 :3 0
3 :3 0 371 692
694 691 695 0
699 69 :4 0 69
1 :8 0 699 373
69a 690 699 0
69b 376 0 69c
378 69e 36 :4 0
69c :4 0 814 6a
:3 0 1c :4 0 6a2
:2 0 6a8 6a0 0
38 :3 0 39 :3 0
6a3 6a4 0 47
:3 0 6a5 6a6 0
6a8 382 6fb 46
:3 0 38 :3 0 39
:3 0 6aa 6ab 0
47 :3 0 6ac 6ad
0 6af 385 6b1
387 6b0 6af :2 0
6f9 48 :3 0 38
:3 0 39 :3 0 6b4
6b5 0 47 :3 0
6b6 6b7 0 6f1
21 :3 0 22 :3 0
6b9 6ba 0 21
:3 0 2e :3 0 6bc
6bd 0 49 :2 0
2 :4 0 7b :2 0
31 :2 0 2f :2 0
389 6c2 6c4 :3 0
38c 6be 6c6 6bb
6c8 6c7 :2 0 24
:3 0 32 :3 0 6ca
6cb 0 26 :4 0
7c :4 0 29 :2 0
2a :3 0 390 6cf
6d1 :3 0 29 :2 0
4c :4 0 393 6d3
6d5 :3 0 29 :2 0
4d :4 0 396 6d7
6d9 :3 0 29 :2 0
47 :3 0 399 6db
6dd :3 0 29 :2 0
4e :4 0 39c 6df
6e1 :3 0 29 :2 0
4f :3 0 50 :3 0
2f :2 0 51 :2 0
39f 6e4 6e8 3a3
6e3 6ea :3 0 3a6
6cc 6ec :2 0 6ee
3a9 6ef 6c9 6ee
0 6f0 3ab 0
6f1 3ad 6f4 :3 0
6f4 0 6f4 6f3
6f1 6f2 :6 0 6f6
1f :3 0 3b0 6f8
3b2 6f7 6f6 :2 0
6f9 3b4 :2 0 6fb
0 6fb 6fa 6a8
6f9 :6 0 814 1
:3 0 35 :3 0 1f
:4 0 700 :2 0 814
6fe 701 :3 0 36
:3 0 1f :3 0 9
:4 0 706 :2 0 7b3
703 704 :3 0 1f
:3 0 37 :3 0 707
708 :3 0 38 :3 0
39 :3 0 70a 70b
0 a :2 0 70c
70d 0 70f 3b7
716 38 :3 0 39
:3 0 710 711 0
3a :2 0 712 713
0 715 3b9 717
709 70f 0 718
0 715 0 718
3bb 0 7b3 38
:3 0 39 :3 0 719
71a 0 38 :3 0
3b :2 0 3c :3 0
71c 71e 0 3c0
71d 720 :3 0 721
:2 0 3d :8 0 725
3c3 726 722 725
0 727 3c5 0
7b3 11 :3 0 40
:3 0 13 :3 0 42
:3 0 58 :3 0 59
:3 0 5a :3 0 59
:3 0 f :3 0 3
:3 0 5b :3 0 5c
:4 0 6d 1 :8 0
74a 37 :4 0 735
:3 0 44 :3 0 45
:2 0 3c9 738 739
:3 0 38 :3 0 39
:3 0 73b 73c 0
a :2 0 73d 73e
0 740 3cc 747
38 :3 0 39 :3 0
741 742 0 3a
:2 0 743 744 0
746 3ce 748 73a
740 0 749 0
746 0 749 3d0
0 74a 3d3 79d
46 :3 0 38 :3 0
39 :3 0 74c 74d
0 47 :3 0 74e
74f 0 751 3d6
753 3d8 752 751
:2 0 79b 48 :3 0
38 :3 0 39 :3 0
756 757 0 47
:3 0 758 759 0
793 21 :3 0 22
:3 0 75b 75c 0
21 :3 0 2e :3 0
75e 75f 0 49
:2 0 2 :4 0 7d
:2 0 31 :2 0 2f
:2 0 3da 764 766
:3 0 3dd 760 768
75d 76a 769 :2 0
24 :3 0 32 :3 0
76c 76d 0 26
:4 0 7e :4 0 29
:2 0 2a :3 0 3e1
771 773 :3 0 29
:2 0 4c :4 0 3e4
775 777 :3 0 29
:2 0 4d :4 0 3e7
779 77b :3 0 29
:2 0 47 :3 0 3ea
77d 77f :3 0 29
:2 0 4e :4 0 3ed
781 783 :3 0 29
:2 0 4f :3 0 50
:3 0 2f :2 0 51
:2 0 3f0 786 78a
3f4 785 78c :3 0
3f7 76e 78e :2 0
790 3fa 791 76b
790 0 792 3fc
0 793 3fe 796
:3 0 796 0 796
795 793 794 :6 0
798 22 :3 0 401
79a 403 799 798
:2 0 79b 405 :2 0
79d 0 79d 79c
74a 79b :6 0 7b3
21 :3 0 67 :3 0
9 :3 0 a :2 0
408 79f 7a2 45
:2 0 a :2 0 40d
7a4 7a6 :3 0 c
:3 0 68 :3 0 3
:3 0 410 7a9 7ab
7a8 7ac 0 7b0
69 :4 0 69 1
:8 0 7b0 412 7b1
7a7 7b0 0 7b2
415 0 7b3 417
7b5 36 :4 0 7b3
:4 0 814 6a :3 0
1f :4 0 7b9 :2 0
7bf 7b7 0 38
:3 0 39 :3 0 7ba
7bb 0 47 :3 0
7bc 7bd 0 7bf
41d 812 46 :3 0
38 :3 0 39 :3 0
7c1 7c2 0 47
:3 0 7c3 7c4 0
7c6 420 7c8 422
7c7 7c6 :2 0 810
48 :3 0 38 :3 0
39 :3 0 7cb 7cc
0 47 :3 0 7cd
7ce 0 808 21
:3 0 22 :3 0 7d0
7d1 0 21 :3 0
2e :3 0 7d3 7d4
0 49 :2 0 2
:4 0 7f :2 0 31
:2 0 2f :2 0 424
7d9 7db :3 0 427
7d5 7dd 7d2 7df
7de :2 0 24 :3 0
32 :3 0 7e1 7e2
0 26 :4 0 80
:4 0 29 :2 0 2a
:3 0 42b 7e6 7e8
:3 0 29 :2 0 4c
:4 0 42e 7ea 7ec
:3 0 29 :2 0 4d
:4 0 431 7ee 7f0
:3 0 29 :2 0 47
:3 0 434 7f2 7f4
:3 0 29 :2 0 4e
:4 0 437 7f6 7f8
:3 0 29 :2 0 4f
:3 0 50 :3 0 2f
:2 0 51 :2 0 43a
7fb 7ff 43e 7fa
801 :3 0 441 7e3
803 :2 0 805 444
806 7e0 805 0
807 446 0 808
448 80b :3 0 80b
0 80b 80a 808
809 :6 0 80d 24
:3 0 44b 80f 44d
80e 80d :2 0 810
44f :2 0 812 0
812 811 7bf 810
:6 0 814 1 :3 0
452 816 dd 47d
0 817 0 814
0 817 459 0
84a 21 :3 0 22
:3 0 818 819 0
21 :3 0 2e :3 0
81b 81c 0 2f
:2 0 2 :4 0 81
:2 0 31 :2 0 2f
:2 0 45c 821 823
:3 0 45f 81d 825
81a 827 826 :2 0
24 :3 0 32 :3 0
829 82a 0 26
:4 0 82 :4 0 29
:2 0 2a :3 0 463
82e 830 :3 0 29
:2 0 4c :4 0 466
832 834 :3 0 29
:2 0 83 :4 0 469
836 838 :3 0 29
:2 0 38 :3 0 39
:3 0 83b 83c 0
46c 83a 83e :3 0
46f 82b 840 :2 0
842 472 843 828
842 0 844 474
0 84a 7 :3 0
38 :3 0 39 :3 0
846 847 0 848
:2 0 84a 476 84d
:3 0 84d 47c 84d
84c 84a 84b :6 0
84e :2 0 2 c
84d 851 :3 0 850
84e 852 :8 0 
484
4
:3 0 1 3 1
8 1 e 1
14 1 1a 1
34 1 4e 1
69 1 84 1
90 1 9c 2
9f a1 2 a4
a6 2 a8 aa
2 ac ae 2
b0 b2 4 98
9e a3 b4 1
b6 1 b8 2
c2 c4 3 c0
c1 c6 2 cf
d1 2 ce d3
1 d5 1 d7
1 d9 2 e5
e6 1 f1 1
f7 2 f9 fa
1 102 2 fe
102 1 107 1
109 1 119 2
118 119 1 121
1 127 2 129
12a 2 116 12b
1 132 1 12d
2 145 147 3
143 144 149 2
152 154 2 156
158 2 15a 15c
2 15e 160 2
162 164 3 169
16a 16b 2 166
16d 2 151 16f
1 171 1 173
2 13c 174 1
178 1 137 2
135 17c 2 187
18a 1 19f 2
19e 19f 1 1a7
1 1ad 2 1af
1b0 2 19c 1b1
1 1b8 1 1b3
2 1cb 1cd 3
1c9 1ca 1cf 2
1d8 1da 2 1dc
1de 2 1e0 1e2
2 1e4 1e6 2
1e8 1ea 3 1ef
1f0 1f1 2 1ec
1f3 2 1d7 1f5
1 1f7 1 1f9
2 1c2 1fa 1
1fe 1 1bd 2
1bb 202 1 213
2 212 213 1
21b 1 221 2
223 224 2 210
225 1 22c 1
227 2 23f 241
3 23d 23e 243
2 24c 24e 2
250 252 2 254
256 2 258 25a
2 25c 25e 3
263 264 265 2
260 267 2 24b
269 1 26b 1
26d 2 236 26e
1 272 1 231
2 22f 276 1
287 2 286 287
1 28f 1 295
2 297 298 2
284 299 1 2a0
1 29b 2 2b3
2b5 3 2b1 2b2
2b7 2 2c0 2c2
2 2c4 2c6 2
2c8 2ca 2 2cc
2ce 2 2d0 2d2
3 2d7 2d8 2d9
2 2d4 2db 2
2bf 2dd 1 2df
1 2e1 2 2aa
2e2 1 2e6 1
2a5 2 2a3 2ea
2 2f0 2f1 1
2f5 2 2f3 2f5
1 2fa 2 2fd
2ff 1 301 a
e7 fb 10a 10d
17f 18d 205 279
2ed 302 2 308
30e 1 315 1
310 2 328 32a
3 326 327 32c
2 335 337 2
339 33b 2 33d
33f 2 341 343
2 345 347 3
34c 34d 34e 2
349 350 2 334
352 1 354 1
356 2 31f 357
1 35b 1 31a
2 318 35f 2
36b 36c 1 377
1 37d 2 37f
380 1 388 2
384 388 1 38d
1 38f 1 3a0
2 39f 3a0 1
3a8 1 3ae 2
3b0 3b1 2 39d
3b2 1 3b9 1
3b4 2 3cc 3ce
3 3ca 3cb 3d0
2 3d9 3db 2
3dd 3df 2 3e1
3e3 2 3e5 3e7
2 3e9 3eb 3
3f0 3f1 3f2 2
3ed 3f4 2 3d8
3f6 1 3f8 1
3fa 2 3c3 3fb
1 3ff 1 3be
2 3bc 403 2
409 40a 1 40e
2 40c 40e 1
413 2 416 418
1 41a 5 36d
381 390 406 41b
2 421 427 1
42e 1 429 2
441 443 3 43f
440 445 2 44e
450 2 452 454
2 456 458 2
45a 45c 2 45e
460 3 465 466
467 2 462 469
2 44d 46b 1
46d 1 46f 2
438 470 1 474
1 433 2 431
478 6 e0 305
362 366 41e 47b
1 48f 1 495
2 497 498 1
4a0 2 49c 4a0
1 4a5 1 4a7
2 4af 4b2 1
4bf 2 4be 4bf
1 4c7 1 4cd
2 4cf 4d0 2
4bc 4d1 1 4d8
1 4d3 2 4eb
4ed 3 4e9 4ea
4ef 2 4f8 4fa
2 4fc 4fe 2
500 502 2 504
506 2 508 50a
3 50f 510 511
2 50c 513 2
4f7 515 1 517
1 519 2 4e2
51a 1 51e 1
4dd 2 4db 522
1 538 2 537
538 1 540 1
546 2 548 549
2 535 54a 1
551 1 54c 2
564 566 3 562
563 568 2 571
573 2 575 577
2 579 57b 2
57d 57f 2 581
583 3 588 589
58a 2 585 58c
2 570 58e 1
590 1 592 2
55b 593 1 597
1 556 2 554
59b 1 5ac 2
5ab 5ac 1 5b4
1 5ba 2 5bc
5bd 2 5a9 5be
1 5c5 1 5c0
2 5d8 5da 3
5d6 5d7 5dc 2
5e5 5e7 2 5e9
5eb 2 5ed 5ef
2 5f1 5f3 2
5f5 5f7 3 5fc
5fd 5fe 2 5f9
600 2 5e4 602
1 604 1 606
2 5cf 607 1
60b 1 5ca 2
5c8 60f 1 620
2 61f 620 1
628 1 62e 2
630 631 2 61d
632 1 639 1
634 2 64c 64e
3 64a 64b 650
2 659 65b 2
65d 65f 2 661
663 2 665 667
2 669 66b 3
670 671 672 2
66d 674 2 658
676 1 678 1
67a 2 643 67b
1 67f 1 63e
2 63c 683 2
689 68a 1 68e
2 68c 68e 1
693 2 696 698
1 69a 9 486
499 4a8 4b5 525
59e 612 686 69b
2 6a1 6a7 1
6ae 1 6a9 2
6c1 6c3 3 6bf
6c0 6c5 2 6ce
6d0 2 6d2 6d4
2 6d6 6d8 2
6da 6dc 2 6de
6e0 3 6e5 6e6
6e7 2 6e2 6e9
2 6cd 6eb 1
6ed 1 6ef 2
6b8 6f0 1 6f4
1 6b3 2 6b1
6f8 1 70e 1
714 2 716 717
1 71f 2 71b
71f 1 724 1
726 1 737 2
736 737 1 73f
1 745 2 747
748 2 734 749
1 750 1 74b
2 763 765 3
761 762 767 2
770 772 2 774
776 2 778 77a
2 77c 77e 2
780 782 3 787
788 789 2 784
78b 2 76f 78d
1 78f 1 791
2 75a 792 1
796 1 755 2
753 79a 2 7a0
7a1 1 7a5 2
7a3 7a5 1 7aa
2 7ad 7af 1
7b1 5 705 718
727 79d 7b2 2
7b8 7be 1 7c5
1 7c0 2 7d8
7da 3 7d6 7d7
7dc 2 7e5 7e7
2 7e9 7eb 2
7ed 7ef 2 7f1
7f3 2 7f5 7f7
3 7fc 7fd 7fe
2 7f9 800 2
7e4 802 1 804
1 806 2 7cf
807 1 80b 1
7ca 2 7c8 80f
6 480 69e 6fb
6ff 7b5 812 2
815 816 2 820
822 3 81e 81f
824 2 82d 82f
2 831 833 2
835 837 2 839
83d 2 82c 83f
1 841 1 843
5 b9 d8 817
844 849 7 12
18 1e 35 4f
6a 85 
1
4
0 
851
0
1
28
25
9
0 1 1 1 1 1 6 7
6 9 6 b 6 d 1 f
1 11 12 1 14 1 16 17
16 19 16 1b 16 1d 1 1f
1 21 22 1 24 0 0 0

21 1 2
3b 1 3
55 1 4
70 1 5
1a 1 0
14 1 0
2 0 1
e 1 0
3 1 0
0

/
SHOW ERRORS FUNCTION FC_MIGRATION;
ALTER FUNCTION FC_MIGRATION COMPILE DEBUG;
